/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;

using TestFramework.ApplicationAPI;
using TestFramework.AuthenticateAPI;

namespace TestFramework
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class APISetup
	{

		//User, password, DataSource Name are use to authenticate with the service
		public static String username = "administrator";
		public static String password = "rpm";
		public static String dsn = "jdbc/RPMDATASOURCE";

		//The url will describe the location of all the services
		//It will need to have the service attach to the url like
		//http://localhost:8080/rpm/services/Application
		public static String url = "http://localhost:9999/rpm-web-services/services/";

		//Object AuthenticateService which provide de 
		//login/logout/change password interface
		//It is a wsdl imported from project->addWebReference
		public static AuthenticateService authenticate;

		//Object ApplicationService which provide de
		//Save/load/checkin/checkout interface
		//It is a wsdl imported from project->addWebReference
		public static ApplicationService application;

		public static String SetUp() 
		{
			String sessionid;
			
			//Get an instance of authenticateService
			authenticate = new AuthenticateService();
			
			//Specify the url to connect to which is represented
			//by the location of all the services and the specified
			//service Authenticate
			authenticate.Url = url + "Authenticate";

			//Login from the authenticateService api with your
			//user/password/dsn. This will return your session ID
			//To use for the application acces.
			sessionid = authenticate.login(username, password, dsn);

			//Get an instance of ApplicationService
			application = new ApplicationService();

			//Specify the url to connect to which is represented
			//by the location of all the services and the specified
			//service Application
			application.Url = url+ "Application";

			return sessionid;
		}

		public static void CleanUp(String sessionid) 
		{
			//log off the authenticateservice api
			authenticate.logout(sessionid);
		}

		public static void checkForErrors( SaveResult save)
		{
			//goes through the save exceptions wich represent errors
			// and writes them in the console
            if (save != null && save.errors != null)
			{
                foreach (RPMException err in save.errors)
				{
					System.Console.Out.WriteLine(err.rpmMessage);
				}
			}
		}

		public APISetup()
		{
			//
			// TODO: Add constructor logic here
			//
		}
	}
}
