/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;
using System.Collections;
using NUnit.Framework;
using TestFramework.ApplicationAPI;


namespace TestFramework
{
	[TestFixture]
	public class TestHelper
	{

		//User, password, DataSource Name are use to authenticate with the service
		protected static String username = "administrator";
		protected static String password = "rpm";
		protected static String dsn = "jdbc/RPMDATASOURCE";

		//Session ID is use for all transaction and is get from the 
		//Authenticate.login function. The session ID will replace the
		//User password while transacting
		protected static String sessionid;

		//The url will describe the location of all the services
		//It will need to have the service attach to the url like
		//http://localhost:8080/rpm/services/Application
		protected static String url = "http://localhost:8080/rpm/services/";

		//Object AuthenticateService which provide de 
		//login/logout/change password interface
		//It is a wsdl imported from project->addWebReference
		protected static AuthenticateAPI.AuthenticateService auth;

		//Object ApplicationService which provide de
		//Save/load/checkin/checkout interface
		//It is a wsdl imported from project->addWebReference
		protected static ApplicationAPI.ApplicationService service;

		public static ArrayList skipSetters, skipGetters;
		public TestHelper()
		{
			setSkipper();
		}

		public static ArrayList setSkipper()
		{
			skipSetters = new ArrayList();
			skipSetters.Add("ID");
			skipSetters.Add("typeID");
			skipSetters.Add("typeIDModified");
			skipSetters.Add("rankID");
			skipSetters.Add("externalID");
			skipSetters.Add("externalIDModified");
			skipSetters.Add("deleted");
			skipSetters.Add("deletedModified");
			skipSetters.Add("modified");
			skipSetters.Add("nameModified");
			return skipSetters;
		}

		public static ArrayList setGetter()
		{
			//Getters
			skipGetters = new ArrayList();
			skipGetters.Add("ID");
			skipGetters.Add("TypeID");
			skipGetters.Add("TypeIDModified");
			skipGetters.Add("RankID");
			skipGetters.Add("ExternalID");
			skipGetters.Add("ExternalIDModified");
			skipGetters.Add("Deleted");
			skipGetters.Add("DeletedModified");
			skipGetters.Add("Modified");
			skipGetters.Add("NameModified");

			return skipGetters;
		}
		[SetUp]
		public void SetUp() 
		{
			//Get an instance of authenticateService
			auth = new AuthenticateAPI.AuthenticateService();
			
			//Specify the url to connect to which is represented
			//by the location of all the services and the specified
			//service Authenticate
			auth.Url = url + "Authenticate";

			//Login from the authenticateService api with your
			//user/password/dsn. This will return your session ID
			//To use for the application acces.
			try
			{
				sessionid = auth.login(username, password, dsn);
			}
			catch(System.Web.Services.Protocols.SoapException x)
			{
				// exception generated by axis web services
				// message is readable
				System.Console.WriteLine(x.ToString());
			}
			catch(System.Xml.XmlException b)
			{
				// exception generated by websphere web services
				// unable to read the error message easily
				System.Console.WriteLine(b.ToString());
			}

			//Get an instance of ApplicationService
			service = new TestFramework.ApplicationAPI.ApplicationService();

			//Specify the url to connect to which is represented
			//by the location of all the services and the specified
			//service Application
			service.Url = url+ "Application";
		}

		[TearDown]
		public void CleanUp() 
		{
			auth.logout(sessionid);
		}

		public static ApplicationAPI.RPMObject loadFromID(ApplicationAPI.RPMObject rpmObject, ApplicationAPI.RPMObjectScope rpmObjectScope)
		{
			ApplicationAPI.RPMObject result = service.loadFromID(sessionid, rpmObject, rpmObjectScope).rpmObject;
	    	
			if (result == null)
			{
				Assert.Fail("Loaded object is null");
			}

			return result;
		}

		public static ApplicationAPI.RPMObject[] loadFromXPath(String type, String name, ApplicationAPI.RPMObjectScope scope)
		{
			TestFramework.ApplicationAPI.RPMObject[] result = 
				loadFromXPath("/" + type + "[name='" +name + "']", scope);
			return result;
		}

		public static ApplicationAPI.RPMObject[] loadFromXPath(String query, ApplicationAPI.RPMObjectScope scope)
		{
			TestFramework.ApplicationAPI.RPMObject[] result = 
				service.loadFromXpath(sessionid, query, scope).rpmObjectList;

			if (result == null)
			{
				Assert.Fail("Loaded object array is null");
			}

			return result;
		}

		public static RPMObject save(ApplicationAPI.RPMObject rpmObject, ApplicationAPI.RPMObjectScope rpmObjectScope)
		{
			ApplicationAPI.SaveResult saveResult = service.save(sessionid, rpmObject, rpmObjectScope,
				ApplicationAPI.ReloadType.ReloadResult);
	    	
			traceErrorsAndFail(saveResult);

			return saveResult.rpmObject;
		}

		//public static ApplicationAPI.SaveResult save(ApplicationAPI.RPMObject rpmObject, ApplicationAPI.RPMObjectScope rpmObjectScope)
		//{
		//	ApplicationAPI.SaveResult saveResult = service.save(sessionid, rpmObject, rpmObjectScope,
		//		ApplicationAPI.ReloadType.ReloadResult);
		//	
		//	traceErrorsAndFail(saveResult);
		//
		//	return saveResult;
		//}

		public static void delete(ApplicationAPI.RPMObject rpmObject)
		{
			service.delete(sessionid, rpmObject);
		}

		public static void traceErrorsAndFail(SaveResult result)
		{
			if (result == null)
			{
				Assert.Fail("SaveResult is null");
			}
            if (result.errors != null && result.errors.Length > 0)
			{
                foreach (RPMException err in result.errors)
				{
					System.Console.Out.WriteLine(err.rpmMessage + "\n\n");
				}
                Assert.Fail(result.errors[0].rpmMessage);
			}
			if (result.rpmObject == null)
			{
				Assert.Fail("SaveResult.rpmObject is null");
			}
		}
	}
}
