/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;

using NUnit.Framework;

using TestFramework.ApplicationAPI;
using TestFramework.AuthenticateAPI;
using TestFramework;

namespace Tests.pool
{
   /// <sample-summary>
   /// Summary description for PoolCCC.
   /// 
   /// This example tries to assign a Cost
   /// Center to a pool or creates one if there isn't any Cost 
   /// Center and then assigns it to a pool.
   /// </sample-summary>
   [TestFixture]
   public class PoolCCC
   {
      public PoolCCC()
      {

      }

      //All these variables are used in this sample
      public static String poolName = "IBMPOOL_123";
      public static String cccName = "IBMCLIENTCOSTCENTER";
      public static String folderName = "IBMCATEGORY";
      public static String folderDesc = "ABCDEF_123";
      public static String cccDesc = "ABCDEF_123";

      //Session ID is use for all transaction and is get from the 
      //Authenticate.login function. The session ID will replace 
      // the User password while transacting
      public static String sessionid = "";

      [Test]
      public void sampleTest()
      {


         // start the communication with the api
         sessionid = APISetup.SetUp();


         // Declare the pool wich we are going to use
         Pool pool = null;

         // create a poolscope to keep track of the relation 
         // between the pool and its attribute
         PoolScope pScope = new PoolScope();

         // adjust the pool scope so that it keeps the cost center
         // this scope is not currently implemented 
         //pScope.clientCostCenter = true;

         // declare a save result object
         SaveResult pSave = null;

         // Find the pool in RPM using XPath. 
         pool = (Pool)APISetup.application.loadFromXpath(sessionid
            , "/Pool[name='" + poolName + "']",
            pScope).rpmObjectList[0];

         // create a client cost center
         ClientCostCenter ccc = new ClientCostCenter();

         // create a client cost center scope 
         // and adjust it so it saves the client cost center
         ClientCostCenterScope cScope = new ClientCostCenterScope();
         cScope.clientCostCenters = new ClientCostCenterScope();

         // load the client cost center to assign it
         LoadResult loader = APISetup.application.loadFromXpath(
            sessionid, 
            "/ClientCostCenter[name='" + cccName + "']", 
            cScope );

         // if logic verifies the result of the xpath query
         // rpmObjectList contains cost centers returned
         if( loader.rpmObjectList == null || 
            loader.rpmObjectList.Length == 0 )
         {
         
            // create a client folder and division (if needed)
            // those are the parents of the client cost center
            ClientFolder folder = new ClientFolder();
            folder.name = folderName;
            folder.description = folderDesc;

            // initialise the client cost 
            // center with default values
            ccc.description = cccDesc;
            ccc.name = cccName;

            // assign the folder as a parent 
            // of the client cost center
            ccc.parent = folder;

            // assign the client cost center to the folder
            folder.clientCostCenters = 
               new ClientCostCenter[] {ccc};
         
            // save the folder and client cost center in the 
            // RPM repository use a reload type that enables 
            // you to use the saved data
            SaveResult save = APISetup.application.save(
               sessionid, folder, cScope, 
               ReloadType.SavedResult );
            APISetup.checkForErrors( save );

            // load the client cost center to use it
            ccc = (ClientCostCenter)APISetup.application.
               loadFromXpath(sessionid, 
               "/ClientCostCenter[name='" + cccName + "']", 
               cScope ).rpmObjectList[0];
         }
         else
         {
            // in this case the cost center already exists
            ccc = (ClientCostCenter)loader.rpmObjectList[0];
         }

         // assign the pool as the parent of the Cost Center
         ccc.parent = pool;

         // assign the Cost Center to the pool
         pool.clientCostCenter = ccc;

         // save the pool in the RPM repository
         // use the sessionid acquired in the setUp
         pSave = new SaveResult();
         pSave = APISetup.application.save(sessionid, pool, 
            pScope, ReloadType.ReloadResult);

         // method showing the detail of any error in the save
         APISetup.checkForErrors( pSave );

         pool = null;
         pool = (Pool)pSave.rpmObject;

         //TODO : uncomment Assert when 
         // pScope.clientCostCenter exists
         //Assert.NotNull(pool.clientCostCenter);

         // end the communication with the api.
         APISetup.CleanUp(sessionid);

      }

   }
}
