/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;

using NUnit.Framework;

using TestFramework.ApplicationAPI;
using TestFramework.AuthenticateAPI;
using TestFramework;

namespace Tests.resource 
{
   /// <sample-summary>
   /// Summary description for ResourceCurrency.
   /// 
   /// this sample creates a currency and exchange range
   /// and then assigns the currency to a resource
   /// </sample-summary>
   [TestFixture]
   public class ResourceCurrency
   {
      // Constructor
      public ResourceCurrency(){}

      //All these variables are used in this sample
      public static String resourceName = "IBM_testABCDEF";
      public static String currencyName = "Canadian dollars";
      public static double rate = 50.00;

      //Session ID is use for all transaction and is get from the 
      //Authenticate.login function. The session ID will replace 
      // the User password while transacting
      public static String sessionid = "";

      [Test]
      public void sampleTest()
      {
         // start the communication with the api
         sessionid = APISetup.SetUp();

         // Declare the resource wich we are going to use
         Resource res = null;

         // create a save result objet
         SaveResult save = new SaveResult();

         // create a resourceScope to keep track of the relation 
         // between the resource and the currency and rates
         ResourceScope rScope = new ResourceScope();

         // adjust the resource scope to keep the 
         // relations with currency
         rScope.currency = true;

         // create a currency
         Currency  currency = new Currency();

         // load the currency
         currency = (Currency)APISetup.application.
            loadFromXpath(sessionid, 
            "/Currency[name='" + currencyName + "']", 
            null).rpmObjectList[0];

         // Find the resource in RPM using XPath.
         // Use the fullName to find the resource
         res = (Resource)APISetup.application.
            loadFromXpath(sessionid, 
            "/Resource[fullname='" + resourceName + "']", 
            rScope ).rpmObjectList[0];

         // assign the currency and exchange rate to the resource
         res.currency = currency;
         //res.currency.parent = null;

         // save the resource in the RPM repository
         // use the sessionid acquired in the setUp
         // use the resource scope
         save = APISetup.application.save(sessionid, 
             res, rScope, ReloadType.ReloadResult  );
         APISetup.checkForErrors( save );

         res = null;
         res = (Resource)save.rpmObject;

         Assert.IsNotNull(res.currency);

         // end the communication with the api.
         APISetup.CleanUp(sessionid);

      }

   }
}
