/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

using System;

using NUnit.Framework;

using TestFramework.ApplicationAPI;
using TestFramework.AuthenticateAPI;
using TestFramework;

namespace Tests.wbs
{
   /// <sample-summary>
   /// Summary description for WBSTaskResource.
   /// 
   /// this sample loads a task from RPM repository
   /// it then assigns a resource profile to it
   /// </sample-summary>
   [TestFixture]
   public class WBSTaskProfile
   {
      public WBSTaskProfile()
      {

      }

      // these are all variables that are used in this sample
      public static String taskName = "IBM_CHILD_TASK";
      public static String poolName = "IBM_POOL_ABC";
      public static String roleName = "Task Responsible";
      public static String profileName = "IBM_PROFILE_NAME";
      public static String resName = "IBM_TASK_RESOURCE";
      public static String aplName = "ATTIBUTE_PROFICIENCY_LEVEL";
      
      //Session ID is use for all transaction and is get from the 
      //Authenticate.login function. The session ID will replace 
      // the User password while transacting
      public static String sessionid;

      [Test]
      public void sampleTest()
      {

         // initialize a session with the API
         sessionid = APISetup.SetUp();

         // create a save result object 
         SaveResult save = null;

         // create a task scope
         // this scope enables us to save 
         // information around the task
         WorkElementScope tScope = new WorkElementScope();

         // initialize task scope
         tScope.resourceTaskAssignments = 
            new ResourceTaskAssignmentScope();
         tScope.parent = new WorkElementScope();
         tScope.resourceTaskAssignments.profile = 
            new ProfileScope();
         tScope.resourceTaskAssignments.profile.
                profileCompetencyAssignments = true;
      
         // create a task object
         Task task = new Task();

         // query the task
         task = (Task)APISetup.application.loadFromXpath(
            sessionid, "/Task[name='" + taskName + "']", 
            tScope ).rpmObjectList[0];

         // create a profile competency assignement object
         // this is a mandatory object to save a profile
         ProfileCompetencyAssignment pCompA = 
            new ProfileCompetencyAssignment();

         // create a resource attribute scope
         ResourceAttributeScope raScope = 
            new ResourceAttributeScope();
         raScope.classifications = new ResourceAttributeScope();
         raScope.classifications.attributes = 
            new ResourceAttributeScope();
         raScope.classifications.attributes.categories = 
            new ResourceAttributeScope();
            raScope.classifications.attributes.proficiencyLevels =
                true;
         
         // load a resource attribute competency categoy
         // from RPM repository
         ResourceAttributeCompetencyCategory category = 
            (ResourceAttributeCompetencyCategory)
            APISetup.application.loadFromXpath(sessionid, 
            "/ResourceAttributeCompetencyCategory"
            + "[name='STATIC_COMPETENCY']"
            ,raScope).rpmObjectList[0];

         // assign the profiency level from resource attribute 
         // competency categoy to profile proficiency level
         pCompA.proficiencyLevel = category.classifications[0].
            attributes[0].proficiencyLevels[0];

         // create a profile object
         Profile profile = new Profile();

         // assign profile as a parent of profile competency
         pCompA.parent = profile;

         // assign profile competency assignement to profile
         profile.profileCompetencyAssignments = 
            new ProfileCompetencyAssignment[] {pCompA};

         // create a resource task assignement object
         ResourceTaskAssignment rtAssign = 
            new ResourceTaskAssignment();

         // initialize resource task assignement
         // with objects created or loaded before
         rtAssign.parent = task;
         rtAssign.profile = profile;

         // create a new security role
         SecurityRole sRole = new SecurityRole();
         
         sRole.name = roleName;

         save = APISetup.application.save(sessionid, sRole
            , null, ReloadType.ReloadResult);

         // assign a security role from RPM repository
         sRole = (SecurityRole)save.rpmObject;

         rtAssign.securityRole = sRole;

         // assign resource task assignement as parent of profile
         profile.parent = rtAssign;

         // assign to the task
         task.resourceTaskAssignments = 
            new ResourceTaskAssignment[] {rtAssign};

         // save task into RPM repository
         // use a scope that includes any information added
         // a reload type is not necessary at this point since
         // we don't use this object again
         save = new SaveResult();
         save = APISetup.application.save(sessionid, task,
            tScope, ReloadType.ReloadResult );
         APISetup.checkForErrors( save );

         task = null;
         task = (Task)save.rpmObject;

         Assert.IsNotNull(task.resourceTaskAssignments[0].profile);

         // close connection with API
         APISetup.CleanUp(sessionid);
      }

   }
}
