/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

/**
 * This sample modifies one of the RTF-formatted data fields
 * in the Cost Centers view. 
 * 
 */
package com.ibm.rpm.clientcostcenters.sample;

import com.ibm.rpm.applicationadministration.containers.ClientRTF;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.framework.*;

/**
 * This sample fills in the Client Description field in the Client Description
 * field using RTF
 * 
 */
public class ClientCostCenterSample extends SampleTestController
{

    // All these variables are used in this sample
    public static String cccName = "COST_CENTER_ABC";

    // client rtf value must exist in RPM repository
    public static String rtfType = "Client Description";

    public static String rtfValue = "{\\rtf1\\ansi\\deff0{\\"
            + "fonttbl{\\f0\\fnil\\fcharset0 Arial;}{\\f1\\fnil\\"
            + "fcharset0 Tahoma;}}\n{\\colortbl ;\\red0\\green255\\"
            + "blue0;}\n\\viewkind4\\uc1\\pard\\cf1\\lang1033\\ul\\"
            + "fs16 This is a test\\cf0\\ulnone\\f1\\fs20 \n\\par }";

    /*
     * @preconditions : "Client Description" must already exist as a field in
     * the Cost Centers view
     * 
     * @postconditions : The Client Description field of _COST_CENTER_ABC will
     * be updated to an rtf-formatted string.
     */
    protected void runSample() throws Exception
    {
        try
        {
            // Contains the output of the web service and its exceptions (if
            // any)
            SaveResult result = null;

            // Create a new costCenter and re-read it back into [result]
            ClientCostCenter costCenter = new ClientCostCenter();
            costCenter.setName(cccName);
            result = save(costCenter, null, ReloadType.ReloadResult);
            costCenter = (ClientCostCenter) result.getRpmObject();

            ClientCostCenterScope costCenterScope = new ClientCostCenterScope();
            costCenterScope.setRtfAssignments(true);

            ClientRTF field = (ClientRTF) loadFromXpath(
                "/ClientRTF[value='" + rtfType + "']", null).getRpmObjectList()[0];

            RtfAssignment fieldAssignment = new RtfAssignment();
            // associate the custom field to its contents
            fieldAssignment.setRtf(field);
            fieldAssignment.setValue(rtfValue);

            // Link the assignment with the WBS object
            fieldAssignment.setParent(costCenter);
            costCenter.setRtfAssignments(new RtfAssignment[]
            {
                fieldAssignment
            });

            result = save(costCenter, costCenterScope, ReloadType.ReloadResult);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    public static void main(String[] args)
    {
        new ClientCostCenterSample();
    }
}
