/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.scope.SecurityRoleScope;

/**
 * This sample assigns a security role to a pool.
 * 
 * @desc Assign a security role to a pool.
 * 
 */
public class PoolResSample extends SampleTestController
{
    public static String poolName = "POOL_123";

    public static String resName = "RESOURCE";

    public static String roleName = "Project Manager";

    protected void runSample() throws Exception
    {
        PoolScope pScope = new PoolScope();
        Resource res = new Resource();
        res.setFullName(resName);

        /**
         * Associate the resource to its pool
         */
        pScope.setResourceAssignments(true);
        Pool pool = (Pool) loadFromXpath("/Pool[@name='" + poolName + "']",
            pScope).getRpmObjectList()[0];
        res.setParent(pool);

        /**
         * Save the new resource
         */
        SaveResult result = save(res, null, ReloadType.ReloadResult);

        /**
         * Load the new resource to use it
         */
        res = (Resource) result.getRpmObject();

        /**
         * Load the security role we want to assign from the repository
         */
        SecurityRole sRole = (SecurityRole) loadFromXpath(
            "/SecurityRole[name='" + roleName + "']", null).getRpmObjectList()[0];
        result = save(sRole, new SecurityRoleScope(), ReloadType.ReloadResult);

        sRole = (SecurityRole) result.getRpmObject();

        /**
         * Associate the newly-created security role to our resource
         */
        ResourceRoleAssignment rrAssign = new ResourceRoleAssignment();
        rrAssign.setResource(res);
        rrAssign.setSecurityRole(sRole);
        rrAssign.setParent(pool);
        pool.setResourceAssignments(new ResourceRoleAssignment[]
        {
            rrAssign
        });

        save(pool, pScope, ReloadType.ReloadResult);
    }

    public static void main(String[] args)
    {
        new PoolResSample();
    }

}
