/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.scope.PoolScope;

/**
 * This sample creates a new pool in the RPM database and then writes data to
 * it.
 * 
 * @desc Create a new pool.
 */
public class PoolSample extends SampleTestController
{
    // All these variables are used in this sample
    static final String poolName = "POOL_123";

    static final String poolDesc = "This is a description";

    static final String poolAdr = "1  Way";

    static final String poolCont = "Sam Samuels";

    static final String poolFax = "555-555-5555";

    protected void runSample() throws Exception
    {

        Pool pool = null;
        PoolScope poolScope = new PoolScope();
        SaveResult saveResult = null;

        LoadResult loadResult = application.loadFromXpath(sessionID,
            "/Pool[name='" + poolName + "']", poolScope);

        /*
         * Create a new Pool and store it in the database if the load command
         * yielded no results.
         */
        if (loadResult.getRpmObjectList() == null
                || loadResult.getRpmObjectList().length == 0)
        {
            pool = new Pool();
            pool.setName(poolName);
            pool.setDescription(poolDesc);

            saveResult = save(pool, poolScope, ReloadType.SavedResult);

            pool = (Pool) loadFromXpath("/Pool[name='" + poolName + "']",
                poolScope).getRpmObjectList()[0];
        }
        else
        {
            /*
             * In case the pool does indeed exist in the database, simply
             * retrieve it
             */
            pool = (Pool) loadResult.getRpmObjectList()[0];
        }

        /*
         * Update the pool with new data locally, then propagate the changes to
         * the RPM repository
         */
        pool.setAddress(poolAdr);
        pool.setContactName(poolCont);
        pool.setFax(poolFax);
        saveResult = application.save(sessionID, pool, poolScope,
            ReloadType.ReloadResult);

        printRPMExceptions(saveResult);

        /*
         * Verify again that the update did indeed work as planned
         */
        pool = (Pool) saveResult.getRpmObject();

        if (pool.getAddress() == null)
        {
            System.err.println("*** Fatal : The update failed to complete");
        }
    }

    public static void main(String[] args)
    {
        new PoolSample();
    }
}
