/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample associates a Calendar to a resource
 * 
 * @desc Associate a Calendar to a resource
 * 
 */
public class ResourceCalendarSample extends SampleTestController
{
    /*
     * Both variables are used in this sample
     */
    static String resourceName = "RESOURCE";

    /*
     * The resource must exist under the following pool for the sameple to run
     * successfully.
     */
    static String poolName = "POOL_123";

    public static String calendarName = "Work";

    protected void runSample() throws Exception
    {
        /*
         * Load the calendar from the database by setting the correct scope on
         * the resourceScope object and store the result in calendar.
         */
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        RPMCalendar calendar = new RPMCalendar();

        Resource resource = (Resource) loadFromXpath(
            "/Resource[@fullname='" + resourceName + "']", resourceScope)
            .getRpmObjectList()[0];
        calendar.setName(calendarName);

        calendar.setName(calendarName);
        calendar.setStartDate(Calendar.getInstance());

        /*
         * Save the calendar in the RPM repository and re-read it from the
         * database
         */
        SaveResult result = save(calendar, null, ReloadType.ReloadResult);
        calendar = (RPMCalendar) result.getRpmObject();

        /*
         * Assign the calendar to our resource and store the data into the
         * repository
         */
        resource.setCalendar(calendar);
        result = save(resource, resourceScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new ResourceCalendarSample();
    }
}
