/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.applicationadministration.containers.ContactGroupAssignment;
import com.ibm.rpm.applicationadministration.scope.ContactGroupScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample adds an existing resource to a group
 * 
 * @desc Associate a resource to a group
 * 
 */
public class ResourceContactGroupSample extends SampleTestController
{

    public static String resourceName = "RESOURCE";

    public static String cGroupName = "GROUP_ABCDEF";

    protected void runSample() throws Exception
    {
        ResourceScope rScope = new ResourceScope();
        rScope.setContactGroupAssignments(true);

        Resource res = (Resource) loadFromXpath(
            "/Resource[@fullname='" + resourceName + "']", rScope)
            .getRpmObjectList()[0];

        /**
         * First save the contact group, then associate it to the resource using
         * ContactGroupAssignment
         */
        ContactGroup cGroup = new ContactGroup();
        ContactGroupScope cgScope = new ContactGroupScope();
        cGroup.setName(cGroupName);

        SaveResult result = save(cGroup, cgScope, ReloadType.ReloadResult);
        cGroup = (ContactGroup) result.getRpmObject();

        /**
         * This object binds together the resource and the group
         */
        ContactGroupAssignment cgAssignment = new ContactGroupAssignment();
        cgAssignment.setGroups(cGroup);
        res.setContactGroupAssignments(new ContactGroupAssignment[]
        {
            cgAssignment
        });

        save(res, rScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new ResourceContactGroupSample();
    }
}
