/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import java.util.Calendar;

import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceCostCenterAssignment;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample associates a resource to a cost center
 * 
 * @desc Associate a resource to a cost center
 * 
 */
public class ResourceCostCenterSample extends SampleTestController
{
    public static String resourceName = "RESOURCE";

    public static String cccName = "CLIENTCOSTCENTER";

    public static String cccDesc = "ABCDEF123";

    public static String folderName = "FOLDER";

    public static String folderDesc = "ABCDEF123";

    /**
     * @see com.ibm.rpm.framework.SampleTestController#runSample()
     */
    protected void runSample() throws Exception
    {
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setResourceCostCenterAssignments(true);

        Resource resource = (Resource) loadFromXpath(
            "/Resource[fullname='" + resourceName + "']", resourceScope)
            .getRpmObjectList()[0];

        /*
         * Initialize the client cost center and the folder - used to aggregate
         * resources
         */
        ClientFolder folder = new ClientFolder();
        folder.setName(folderName);
        folder.setDescription(folderDesc);

        ClientCostCenter costCenter = new ClientCostCenter();
        costCenter.setDescription(cccDesc);
        costCenter.setName(cccName);
        costCenter.setParent(folder);

        /**
         * Indicate that we want to save two levels of the object graph
         */
        folder.setClientCostCenters(new ClientCostCenter[]
        {
            costCenter
        });
        ClientCostCenterScope cScope = new ClientCostCenterScope();
        cScope.setClientCostCenters(new ClientCostCenterScope());

        /**
         * Save the folder and the client cost center in the repository
         */
        save(folder, cScope, ReloadType.None);

        costCenter = (ClientCostCenter) loadFromXpath(
            "/ClientCostCenter[@name='" + cccName + "']", cScope)
            .getRpmObjectList()[0];

        Calendar start = Calendar.getInstance();
        Calendar finish = Calendar.getInstance();
        finish.roll(Calendar.YEAR, true);
        ResourceCostCenterAssignment assignment = new ResourceCostCenterAssignment();
        assignment.setStartDate(start);
        assignment.setFinishDate(finish);

        assignment.setClientCostCenter(costCenter);
        assignment.setParent(resource);

        /**
         * Assign the client cost center resource assignment to the resource
         */
        resource
            .setResourceCostCenterAssignments(new ResourceCostCenterAssignment[]
            {
                assignment
            });

        save(resource, resourceScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new ResourceCostCenterSample();
    }
}
