/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample sets the currency for an existing resource
 * 
 * @preconditions resourceName, currencyName must exist in the repository
 * @postconditions My Currency in the Rates portlet will be updated to
 *                 currencyName
 * @desc Set the currency for an existing resourcee
 * 
 */
public class ResourceCurrency extends SampleTestController
{
    static String resourceName = "RESOURCE";

    static String currencyName = "Canadian dollars";

    static double rate = 50.00;

    protected void runSample() throws Exception
    {
        /**
         * Indicate that we wish to operate on currency as well
         */
        ResourceScope rScope = new ResourceScope();
        rScope.setCurrency(true);

        Currency currency = (Currency) loadFromXpath(
            "/Currency[@name='" + currencyName + "']", null).getRpmObjectList()[0];

        /**
         * Load the resource from the repository and associate to it a currency
         */
        Resource res = (Resource) loadFromXpath(
            "/Resource[@fullname='" + resourceName + "']", rScope)
            .getRpmObjectList()[0];

        res.setCurrency(currency);

        /**
         * Save the resource in the repository and keep its assigned currency
         */
        save(res, rScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new ResourceCurrency();
    }
}
