/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import com.ibm.rpm.applicationadministration.containers.City;
import com.ibm.rpm.applicationadministration.containers.Country;
import com.ibm.rpm.applicationadministration.containers.CountryState;
import com.ibm.rpm.applicationadministration.containers.GeographicalAssignment;
import com.ibm.rpm.applicationadministration.containers.Location;
import com.ibm.rpm.applicationadministration.containers.Region;
import com.ibm.rpm.applicationadministration.scope.GeographicalScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample stores an object tree representing a physical location, then
 * associates it to a resource
 * 
 * @desc Associate a new location to a resource.
 * 
 */
public class ResourceGeoSample extends SampleTestController
{
    public static String resourceName = "RESOURCE";

    public static String regionName = "REGION";

    public static String regionDesc = "ABCDEF123";

    public static String countryName = "COUNTRY";

    public static String countryDesc = "ABCDEF123";

    public static String stateName = "STATE";

    public static String stateDesc = "STATEDESCRIPTION";

    public static String cityName = "CITY";

    public static String cityDesc = "ABCDEF123";

    public static String locationName = "LOCATION";

    public static String locationDesc = "ABCDEF123";

    /*
     * The resource must exist under the following pool for the sameple to run
     * successfully.
     */
    static String poolName = "POOL_123";

    /**
     * We are already logged in to the Web services framework
     * 
     * @see SampleTestController
     */
    protected void runSample() throws Exception
    {

        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setGeographicalAssignment(true);

        Resource resource = (Resource) loadFromXpath(
            "/Resource[@fullname='" + resourceName + "']", resourceScope)
            .getRpmObjectList()[0];

        Region region = new Region();
        CountryState state = new CountryState();
        Country country = new Country();
        City city = new City();
        Location location = new Location();

        /*
         * Initialize each element in the following hierarchy
         * 
         * region country state city location
         */
        region.setName(regionName);
        region.setDescription(regionDesc);
        GeographicalScope geoScope = new GeographicalScope();
        region.setChildren(new Country[]
        {
            country
        });

        /*
         * Set up the country object and the scope level required to save it.
         */
        country.setName(countryName);
        country.setDescription(countryDesc);

        geoScope.setChildren(new GeographicalScope());

        state.setName(stateName);
        state.setDescription(stateDesc);
        country.setChildren(new CountryState[]
        {
            state
        });
        geoScope.getChildren().setChildren(new GeographicalScope());

        city.setName(cityName);
        city.setDescription(cityDesc);
        state.setChildren(new City[]
        {
            city
        });
        geoScope.getChildren().getChildren().setChildren(
            new GeographicalScope());

        location.setName(locationName);
        location.setDescription(locationDesc);
        city.setChildren(new Location[]
        {
            location
        });
        geoScope.getChildren().getChildren().getChildren().setChildren(
            new GeographicalScope());

        /*
         * Save the object graph we have created into the RPM repository. Then
         * load the resource in question from the RPM repository
         */
        SaveResult result = save(region, geoScope, ReloadType.ReloadResult);
        region = (Region) result.getRpmObject();
        GeographicalAssignment geoAssign = new GeographicalAssignment();
        geoAssign.setParent(resource);
        geoAssign.setGeographical(region);
        SaveResult geoAssignSaveResult = save(geoAssign, null,
            ReloadType.ReloadResult);
        geoAssign = (GeographicalAssignment) geoAssignSaveResult.getRpmObject();

        resource.setGeographicalAssignment(geoAssign);

        /*
         * Save the modified resource to the databse
         */
        save(resource, resourceScope, ReloadType.ReloadResult);
    }

    public static void main(String[] args)
    {
        new ResourceGeoSample();
    }
}
