/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import com.ibm.rpm.applicationadministration.containers.Classification;
import com.ibm.rpm.applicationadministration.containers.IndustryType;
import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.resource.containers.IndustryExperience;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample modifies the experience fields of an existing resource
 * 
 * @preconditions resourceName must exist in the repository
 * 
 * @desc Update the experience fields of a resource.
 */
public class ResourceIndusExpSample extends SampleTestController
{
    public static String resourceName = "RESOURCE";

    protected void runSample() throws Exception
    {

        IndustryExperience exp = new IndustryExperience();

        /**
         * Load the resource and indicate we wish to modify her experience
         * fields
         */
        ResourceScope rScope = new ResourceScope();
        rScope.setIndustryExperiences(true);
        Resource res = (Resource) loadFromXpath(
            "/Resource[@fullname='" + resourceName + "']", rScope)
            .getRpmObjectList()[0];

        exp.setParent(res);

        /**
         * Set a pseudo-random classification and an industry type for the
         * acquired experience
         */
        exp.setClassification((Classification) loadFromXpath("/Classification",
            null).getRpmObjectList()[0]);
        exp.setIndustry((IndustryType) loadFromXpath("/Industrytype", null)
            .getRpmObjectList()[0]);
        exp.setProficiency((ProficiencyLevel) loadFromXpath(
            "/Proficiencylevel", null).getRpmObjectList()[0]);

        /**
         * Associate the experience acquired to the resource
         */
        res.setIndustryExperiences(new IndustryExperience[]
        {
            exp
        });

        save(res, rScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new ResourceIndusExpSample();
    }
}
