/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import com.ibm.rpm.applicationadministration.containers.Language;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.resource.containers.LanguageProficiency;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample associates a new language proficiency level to an existing
 * resource and sets it as the default
 * 
 * @desc Associate a language proficiency level to a resource.
 * 
 */
public class ResourceLangSample extends SampleTestController
{
    public static String resourceName = "RESOURCE";

    protected void runSample() throws Exception
    {
        ResourceScope rScope = new ResourceScope();
        rScope.setLanguageProficiencies(true);
        LanguageProficiency lProf = new LanguageProficiency();
        Resource res = (Resource) loadFromXpath(
            "/Resource[@fullname='" + resourceName + "']", rScope)
            .getRpmObjectList()[0];

        /**
         * Assign a pseudorandom language to the resource
         */
        Language lang = (Language) loadFromXpath("/Language", null)
            .getRpmObjectList()[0];

        lProf.setDefaultLanguage(Boolean.TRUE);
        lProf.setLanguage(lang);

        /**
         * Set the proficiency levels attained by the resource
         */
        lProf.setReadLevel(new Integer(2));
        lProf.setSpeakLevel(new Integer(2));
        lProf.setWriteLevel(new Integer(2));

        /**
         * Associate the proficiency level to the resource
         */
        lProf.setParent(res);
        res.setLanguageProficiencies(new LanguageProficiency[]
        {
            lProf
        });

        save(res, rScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new ResourceLangSample();
    }
}
