/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import com.ibm.rpm.applicationadministration.containers.Department;
import com.ibm.rpm.applicationadministration.containers.Group;
import com.ibm.rpm.applicationadministration.containers.Office;
import com.ibm.rpm.applicationadministration.containers.Organization;
import com.ibm.rpm.applicationadministration.containers.OrganizationalAssignment;
import com.ibm.rpm.applicationadministration.scope.OrganizationalScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample creates a new organization, and associates to it an existing
 * resource.
 * 
 * @desc Associate a new organization to an existing resource
 * 
 */
public class ResourceOrganizationSample extends SampleTestController
{
    static String resourceName = "RESOURCE";

    static String orgName = "ORGANIZATION";

    static String orgDesc = "ABCDEF123";

    static String groupName = "GROUP";

    static String groupDesc = "ABCDEF123";

    static String depName = "DEPARTMENT";

    static String depDesc = "ABCDEF123";

    static String officeName = "OFFICE";

    static String officeDesc = "ABCDEF123";

    protected void runSample() throws Exception
    {
        /**
         * Indicate that we wish to also update the organizational field of the
         * resource record
         */
        ResourceScope rScope = new ResourceScope();
        rScope.setOrganizationalAssignment(true);

        /**
         * Create the object graph to fully descibe the user's organizational
         * structure
         */
        Organization org = new Organization();
        Group group = new Group();
        Department dep = new Department();
        Office office = new Office();

        OrganizationalScope oScope = new OrganizationalScope();
        org.setName(orgName);
        org.setDescription(orgDesc);
        oScope.setChildren(new OrganizationalScope());

        group.setName(groupName);
        group.setDescription(groupDesc);
        org.setChildren(new Group[]
        {
            group
        });
        oScope.setChildren(new OrganizationalScope());

        dep.setName(depName);
        dep.setDescription(depDesc);
        group.setChildren(new Department[]
        {
            dep
        });
        oScope.getChildren().setChildren(new OrganizationalScope());

        office.setName(officeName);
        office.setDescription(officeDesc);
        dep.setChildren(new Office[]
        {
            office
        });
        oScope.getChildren().getChildren().setChildren(
            new OrganizationalScope());

        /**
         * Associate the newly-created organization to the resource
         */
        Resource res = (Resource) loadFromXpath(
            "/Resource[@fullname='" + resourceName + "']", rScope)
            .getRpmObjectList()[0];

        SaveResult result = save(org, oScope, ReloadType.ReloadResult);

        org = (Organization) result.getRpmObject();
        res.setOrganizationalAssignment(new OrganizationalAssignment());
        res.getOrganizationalAssignment().setOrganizational(org);

        save(res, rScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new ResourceOrganizationSample();
    }
}
