/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.CitizenshipCountryCode;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Passport;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.Visa;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample adds a new passport and a visa to an existing resource
 * 
 * @desc Add a new passport and visa to a resource
 * 
 */
public class ResourcePassportSample extends SampleTestController
{
    /*
     * The resource must exist under the following pool for the sameple to run
     * successfully.
     */
    static String poolName = "POOL_123";

    static String resourceName = "testABCDEF";

    protected void runSample() throws Exception
    {
        /*
         * Load a specific resource using XPath alongwith all the passports she
         * may have.
         */
        ResourceScope resScope = new ResourceScope();
        resScope.setPassport(true);

        Resource resource = (Resource) application.loadFromXpath(sessionID,
            "/Resource[@FullName='" + resourceName + "']", resScope)
            .getRpmObjectList()[0];

        /*
         * Assign a new passport to this resource
         */
        Passport passport = new Passport();
        passport.setHasValidPassport(Boolean.TRUE);
        Calendar expiryDate = Calendar.getInstance();
        expiryDate.set(Calendar.YEAR, 2020);
        passport.setExpiryDate(expiryDate);
        passport.setParent(resource);

        /*
         * Add additional visas to international readiness.
         */
        Visa visa = new Visa();
        CitizenshipCountryCode usCountryCode = (CitizenshipCountryCode) application
            .loadFromXpath(sessionID, "/CitizenshipCountryCode[@value='USA']",
                null).getRpmObjectList()[0];
        visa.setCountry(usCountryCode);
        visa.setExpiryDate(expiryDate);
        visa.setParent(passport);
        passport.setVisas(new Visa[]
        {
            visa
        });
        resource.setPassport(passport);

        /*
         * Retrieve the resource's enclosing pool
         */
        Pool group = (Pool) loadFromXpath("/Pool[@name='" + poolName + "']",
            null).getRpmObjectList()[0];
        resource.setParent(group);

        SaveResult result = save(resource, resScope, ReloadType.ReloadResult);

        resource = (Resource) result.getRpmObject();
        if (resource.getPassport() == null)
        {
            System.err.println("*** Fatal error : Failed to save the passport");
        }

    }

    public static void main(String[] args)
    {
        new ResourcePassportSample();
    }
}
