/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.Experience;
import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * This sample manipulates the resource object from the database
 * 
 * @desc Create a new resource in the repository.
 */
public class ResourceSample extends SampleTestController
{
    public static String resourceExternalID = "testABCDEF";

    public static String poolName = "POOL_123";

    public static String resourceName = "testABCDEF";

    public static String userid = "USER";

    public static String passid = "PASS";

    protected void runSample() throws Exception
    {
        LoadResult loadResult = loadFromXpath("/Resource[@externalid='"
                + resourceExternalID + "']", null);
        Resource resource = null;

        /*
         * If the load operation did not return any objects, create a new
         * resource with the external ID specified
         */
        if (loadResult.getRpmObjectList() == null
                || loadResult.getRpmObjectList().length == 0)
        {
            /*
             * Query for the pool we would like to add the resource to, then
             * associate the resource to the pool in question
             */
            Pool pool = (Pool) loadFromXpath("/Pool[@name='" + poolName + "']",
                null).getRpmObjectList()[0];
            resource = new Resource();
            resource.setExternalID(resourceExternalID);
            resource.setFullName(resourceName);
            resource.setParent(pool);
            resource.setUserName(userid);
            resource.setPassword(passid);

            /*
             * Store the resource in the RPM repository
             */
            SaveResult result = save(resource, null, ReloadType.SavedResult);
            resource = (Resource) result.getRpmObject();

        }
        else
        {
            /*
             * If the resource exists already
             */
            resource = (Resource) loadResult.getRpmObjectList()[0];
        }

        /*
         * Initialize the resource's various parameters
         */
        resource.setActive(Boolean.TRUE);
        resource.setNickName(resourceName);
        resource.setWeeklyMaxHours(new Integer(40));
        resource.setWeeklyMinHours(new Integer(0));

        ResourceScope resourceScope = new ResourceScope();

        /*
         * This type of experience attribute must exist in the RPM repository
         * for the sample to complete successfully.
         */
        Experience experience = (Experience) loadFromXpath(
            "/Experience[@value='5  years +']", null).getRpmObjectList()[0];
        resource.setExperience(experience);
        resourceScope.setExperience(true);

        resource.setEffectiveStartDate(Calendar.getInstance());
        Calendar discontinueDate = Calendar.getInstance();
        discontinueDate.roll(Calendar.YEAR, true);
        resource.setDiscontinueDate(discontinueDate);

        /*
         * Save the resource into the RPM repository
         */
        save(resource, resourceScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new ResourceSample();
    }

}
