/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.timesheet.sample;

import java.text.SimpleDateFormat;

import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.wbs.containers.TaskAssignment;

public class CreateAndUpdateStepAndStepTimesheet extends SampleTestController
{
    private static TaskAssignment taskAssignment = null;

    private static SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

    private static void createSummaryTimesheet() throws Exception
    {

        System.out.println("\nCreate a SummaryTimesheet");

        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();

        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(taskAssignment.getAssignmentStartDate());

        SingleResult result1 = application.save(sessionID, summaryTimesheet,
            null, ReloadType.ReloadResult);

        printRPMExceptions(result1);

        if (result1.isSuccessful())
        {
            summaryTimesheet = (SummaryTimesheet) result1.getRpmObject();

            System.out.println("SummaryTimesheet created for week of "
                    + format.format(summaryTimesheet.getWeekOf().getTime()));
        }

    }

    private static void createStepWithStepTimesheet() throws Exception
    {

        System.out.println("\nCreate a Step with a StepTimesheet");

        // Create step
        Step step = new Step();
        step.setName("Sample step");

        // Create stepTimesheet (step must have at least one stepTimesheet)
        StepTimesheet timesheet = new StepTimesheet();
        timesheet.setWeekOf(taskAssignment.getAssignmentStartDate());
        timesheet.setRegularTuesday(new Double(6.5));

        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(new StepTimesheet[]
        {
            timesheet
        });

        // Add stepTimesheet (mandatory to save the stepTimesheet)
        StepScope stScope = new StepScope();
        stScope.setTimesheets(true);

        SingleResult result1 = application.save(sessionID, step, stScope,
            ReloadType.ReloadResult);

        printRPMExceptions(result1);

        if (result1.isSuccessful())
        {
            step = (Step) result1.getRpmObject();
            timesheet = step.getStepTimesheets()[0];

            System.out
                .println("Step was created with a StepTimesheet for week of "
                        + format.format(timesheet.getWeekOf().getTime()));
        }

    }

    private static void loadFromXPathAndUpdate() throws Exception
    {

        System.out
            .println("\nLoad a StepTimesheet from xpath and update the StepTimesheet");

        StepTimesheet timesheet = null;

        String xpath = "/Step[@resource[fullname = '"
                + TimesheetSampleSetUp.resourceName
                + "'] and @workElement[name = '"
                + TimesheetSampleSetUp.taskName + "']]/StepTimesheet";

        System.out.println("xpath: " + xpath);

        ArrayResult result1 = application.loadFromXpath(sessionID, xpath, null);

        printRPMExceptions(result1);

        if (result1.getRpmObjectList() != null)
        {
            timesheet = (StepTimesheet) result1.getRpmObjectList()[0];
            System.out.println("StepTimesheet was loaded for week of "
                    + format.format(timesheet.getWeekOf().getTime())
                    + ", Special tuesday hours = "
                    + timesheet.getSpecialTuesday());

            timesheet.setSpecialTuesday(new Double(2));

            SingleResult result2 = application.save(sessionID, timesheet, null,
                ReloadType.ReloadResult);

            printRPMExceptions(result2);

            if (result2.isSuccessful())
            {
                timesheet = (StepTimesheet) result2.getRpmObject();

                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

                System.out.println("StepTimesheet was updated for week of "
                        + format.format(timesheet.getWeekOf().getTime())
                        + ", Special tuesday hours = "
                        + timesheet.getSpecialTuesday());

            }
        }
        else
        {
            throw new Exception("Unable to load Step from xpath");
        }

    }

    protected void runSample() throws Exception
    {
        System.out.println("\n***CreateAndUpdateStep***");

        login();

        // Create objects needed to run this code sample
        TimesheetSampleSetUp.setUp();

        System.out.println("\nLoading an existing TaskAssignment from xpath");

        String xpath = "Task[name = '" + TimesheetSampleSetUp.taskName
                + "']/ResourceTaskAssignment[@resource[fullname = '"
                + TimesheetSampleSetUp.resourceName + "']]/TaskAssignment";

        System.out.println("xpath: " + xpath);

        LoadResult result1 = application.loadFromXpath(sessionID, xpath, null);

        if (result1.getRpmObjectList() != null)
        {
            taskAssignment = (TaskAssignment) result1.getRpmObjectList()[0];
        }
        else
        {
            throw new Exception(
                "Task assignment not found. Use TimesheetSampleSetup to create the TaskAssignment");
        }

        // A summaryTimesheet must exist for the week of the StepTimesheet
        createSummaryTimesheet();

        createStepWithStepTimesheet();

        loadFromXPathAndUpdate();

        System.out.println("Done");
    }

    public static void main(String[] args) throws Exception
    {
        new CreateAndUpdateStepAndStepTimesheet();
    }
}
