/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.wbs.sample;

import java.rmi.RemoteException;

import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

public class CreateObjectFromTemplateSample extends SampleTestController
{

    public static void main(String[] args) throws Exception
    {
        new CreateObjectFromTemplateSample();
    }

    protected void runSample() throws Exception
    {
        createGenericProjectFromTemplate();
        createWorkElementFromTemplate();
        createScopeElementFromTemplate();
        System.out.println("\nDone");
    }

    /**
     * ScopeElement: Input: Action, Issue, Requirement, Risk, ChangeRequest,
     * Defect, ServiceRequest Template: Action, Issue, Requirement, Risk,
     * ChangeRequest, Defect, ServiceRequest
     * 
     * @throws Exception
     */
    private static void createScopeElementFromTemplate() throws Exception
    {
        System.out.println("\n Creating ScopeElement From Template - Begin");

        // --Defines required scope
        ScopeElementScope mainScope = new ScopeElementScope();
        mainScope.setParent(new ScopeElementScope());
        mainScope.setChildren(mainScope);

        // --Create or Load the parent scope folder
        LoadResult result = createScopeFolder("WBS-createScopeElementFromTemplate");

        if (result != null && result.isSuccessful())
        {
            ScopeFolder scopeFolder = (ScopeFolder) result.getRpmObjectList()[0];

            // --Create or Load the template (this version does not support
            // creation of templates)
            AbstractScope template = null;

            LoadResult loadResult = application.loadFromXpath(sessionID,
                "/Action[published='" + PublishedType._ToTemplate + "']",
                mainScope);

            printRPMExceptions(loadResult);

            if (loadResult.getRpmObjectList()[0] instanceof Action)
            {
                template = (Action) loadResult.getRpmObjectList()[0];

                // --Create input object
                Action newAction = new Action();
                newAction.setName("CAMERA_ACTION");
                newAction.setPublished(PublishedType.None);
                newAction.setParent(scopeFolder);

                // --Call createObjectFromTemplate method
                SingleResult newResult = application.createObjectFromTemplate(
                    sessionID, newAction, template, mainScope);

                printRPMExceptions(newResult);

                if (newResult.isSuccessful()
                        && newResult.getRpmObject() instanceof Action
                        && newResult.getRpmObject().getID().equals(
                            newAction.getID()))
                {
                    System.out
                        .println("Creation of object from template is successfull...");
                }
                else
                    System.out
                        .println("Creation of object from template failed...");

            }

        }

        System.out.println("\n Creating ScopeElement From Template - End");
    }

    /**
     * WorkElement: Input: Milestone, Task, SummaryTask, WorkProduct,
     * Deliverable Template: Milestone, Task, SummaryTask, WorkProduct,
     * Deliverable
     * 
     * @throws Exception
     */
    private static void createWorkElementFromTemplate() throws Exception
    {
        System.out.println("\n Creating WorkElement From Template - Begin");

        // --Defines required scope
        WorkElementScope scope = new WorkElementScope();

        // --Create the parent
        Proposal proposalParent = new Proposal();
        proposalParent.setName("WBS-WE-Proposal");

        // saving parent
        SingleResult saveResult = application.save(sessionID, proposalParent,
            null, ReloadType.ReloadResult);

        printRPMExceptions(saveResult);

        if (saveResult.isSuccessful())
        {
            proposalParent = (Proposal) saveResult.getRpmObject();

            // --Create or Load the template (this version does not support
            // creation of templates)
            Deliverable template = null;

            LoadResult loadResult = application.loadFromXpath(sessionID,
                "/Deliverable[published='" + ProjectPublishedType._ToTemplate
                        + "']", scope);

            printRPMExceptions(loadResult);

            if (loadResult.isSuccessful()
                    && loadResult.getRpmObjectList()[0] instanceof Deliverable)
            {
                template = (Deliverable) loadResult.getRpmObjectList()[0];

                // --creating new deliverable input
                Deliverable inputDev = new Deliverable();
                inputDev.setName("WBS-WE-InputDeliverable");
                inputDev.setPublished(ProjectPublishedType.None);
                inputDev.setParent(proposalParent);

                // --creating new deliverable from template
                SingleResult newResult = application.createObjectFromTemplate(
                    sessionID, inputDev, template, null);

                printRPMExceptions(newResult);

                if (newResult.isSuccessful()
                        && newResult.getRpmObject() instanceof Deliverable
                        && newResult.getRpmObject().getID().equals(
                            inputDev.getID()))
                {
                    System.out
                        .println("Creation of object from template is successfull...");
                }
                else
                    System.out
                        .println("Creation of object from template failed...");
            }
        }

        System.out.println("\n Creating WorkElement From Template - End");

    }

    /**
     * GenericProject: Input: Proposal, Project, WorkOrganization Template:
     * Proposal, Project, WorkOrganization
     * 
     * @throws RemoteException
     */
    private static void createGenericProjectFromTemplate()
            throws RemoteException
    {
        System.out.println("\n Creating GenericProject From Template - Begin");

        // --Defines required scope
        WorkElementScope scope = new WorkElementScope();

        // --Create the parent
        Proposal proposalParent = new Proposal();
        proposalParent.setName("WBS-GP-Proposal");

        // saving parent
        SingleResult saveResult = application.save(sessionID, proposalParent,
            null, ReloadType.ReloadResult);

        printRPMExceptions(saveResult);

        if (saveResult.isSuccessful())
        {
            proposalParent = (Proposal) saveResult.getRpmObject();

            // --Create or Load the template (this version does not support
            // creation of templates)
            GenericProject template = null;

            LoadResult loadResult = application.loadFromXpath(sessionID,
                "/GenericProject[published='"
                        + ProjectPublishedType._ToTemplate + "']", scope);

            printRPMExceptions(loadResult);

            if (loadResult.isSuccessful()
                    && loadResult.getRpmObjectList()[0] instanceof GenericProject)
            {
                template = (GenericProject) loadResult.getRpmObjectList()[0];

                // --creating new Project input
                Project inputProj = new Project();
                inputProj.setName("WBS-GP-InputProject");
                inputProj.setPublished(ProjectPublishedType.None);
                inputProj.setParent(proposalParent);

                // --creating new Project from template
                SingleResult newResult = application.createObjectFromTemplate(
                    sessionID, inputProj, template, null);

                printRPMExceptions(newResult);

                if (newResult.isSuccessful()
                        && newResult.getRpmObject() instanceof Project
                        && newResult.getRpmObject().getID().equals(
                            inputProj.getID()))
                {
                    System.out
                        .println("Creation of object from template is successfull...");
                }
                else
                    System.out
                        .println("Creation of object from template failed...");
            }
        }

        System.out.println("\n Creating GenericProject From Template - End");

    }

    // --Utility Method
    private static LoadResult createScopeFolder(String folderName)
            throws Exception
    {
        LoadResult loadResult = null;

        loadResult = application.loadFromXpath(sessionID, "/ScopeFolder[name='"
                + folderName + "']", null);

        if (loadResult == null || loadResult.getRpmObjectList() == null)
        {
            // create a Proposal and save it
            System.out.println("Creating a Proposal");

            Proposal proposal = new Proposal();
            proposal.setName(folderName);
            SingleResult singleResult = application.save(sessionID, proposal,
                null, ReloadType.ReloadResult);

            printRPMExceptions(singleResult);

            if (singleResult.isSuccessful())
            {
                // load the ScopeFolder associated with the proposal
                loadResult = application.loadFromXpath(sessionID,
                    "/ScopeFolder[name='" + folderName + "']", null);

                printRPMExceptions(loadResult);
            }
        }
        else
        {
            System.out.println("ERROR: ScopeFolder with name " + folderName
                    + " already exists.");
            loadResult.setSuccessful(false);
        }

        return loadResult;
    }
}
