/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.wbs.sample;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.CalendarWeekDay;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.applicationadministration.types.CalendarWeekDayType;
import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;
import com.ibm.rpm.wbs.types.ResourceType;

public class EarnedValuesSample extends SampleTestController
{

    private static final String EV_RESOURCE = "ev_resource";
    private static final String EV_POOL = "ev_pool";
    private static final String EV_TASK = "ev_task";
    private static final String EV_PROJECT = "ev_project";
    private static final String EV_CALENDAR = "ev_calendar";
    private static WorkElementScope mainScope = null;
    private static WorkElementScope taskScope = null;
    private static WorkElementScope allScope = null;
    private static CalendarScope calScope = null;
    private static ResourceScope evResourceScope = null;
    private static ResourceTaskAssignmentScope rtaScope = null;
    private static final int NB_WEEKS  = 0;

    protected void runSample() throws Exception
    {
        doEarnedValuesCreationAndLoading();
        
        System.out.println("\nDone");
    }

    /**
     * @param args
     */
    public static void main(String[] args)
    {
        new EarnedValuesSample();
    }
    
    private static void doEarnedValuesCreationAndLoading()
            throws AssertionError, Exception
    {
        System.out.println("\n Creating EarnedValues Data - Begin");
        
        createScopes();

        Calendar startDate = Calendar.getInstance();
        startDate.set(2007, 0, 1);

        Calendar finishDate = Calendar.getInstance();
        finishDate.set(2007, 0, 5);

        // 1. create Project
        Project evProject = new Project();        
        evProject.setName(EV_PROJECT);
        evProject.setPublished(ProjectPublishedType.None);
        evProject.setAssignmentType(AssignmentType.Work);
        evProject.setEstimatedStartDate(startDate);
        evProject.setEstimatedFinishDate(finishDate);

        SingleResult result = null;
        result = application.save(sessionID, evProject,
            mainScope, ReloadType.ReloadResult);
        
        printRPMExceptions(result);        
        evProject = (Project) result.getRpmObject();

        // 2. set Project Setting to activate EV Calculation
        application.enableProjectEVCalculation(sessionID,
            evProject, mainScope);

        // 3. set appAdmin.EV_Delays(week)=0
        application.activateEVWeekDelays(sessionID, NB_WEEKS);

        // 4. Add Task to Project as child
        Task evTask = new Task();
        evTask.setName(EV_TASK);
        evTask.setParent(evProject);

        result = null;
        result = application.save(sessionID, evTask, taskScope,
            ReloadType.ReloadResult);
        printRPMExceptions(result);        
        evTask = (Task) result.getRpmObject();

        // 5. create and save pool
        Pool evPoolParent = new Pool();
        evPoolParent.setName(EV_POOL);

        result = null;
        result = application.save(sessionID, evPoolParent,
            new PoolScope(), ReloadType.ReloadResult);
        printRPMExceptions(result);        

        // 6. create resource
        Resource evResource = new Resource();
        evResource.setFullName(EV_RESOURCE);
        evResource.setActive(new Boolean(true));
        evResource.setParent(evPoolParent);

        evResource.setEffectiveStartDate(startDate);

        ResourceSecurityRights resourceSecurityRights = new ResourceSecurityRights();
        evResource.setResourceSecurityRights(resourceSecurityRights);

        // 6.1 create, bind and save calendar, rates to resource (horaire)
        RPMCalendar rpmCal = createCalendar(calScope);
        evResource.setCalendar(rpmCal);

        // 6.2 save,bind resource to calendar, rates
        ResourceRates rates = new ResourceRates();
        rates.setRegularCostPerHour(new Double(25));
        rates.setSpecialCostPerHour(new Double(50));
        rates.setSellingRatePerHour(new Double(35));
        rates.setSpecialSellingRatePerHour(new Double(65));

        evResource.setRates(rates);
        evResource.getRates().setParent(evResource);

        result = null;
        result = application.save(sessionID, evResource,
            evResourceScope, ReloadType.ReloadResult);
        printRPMExceptions(result);        
        evResource = (Resource) result.getRpmObject();

        // 6.3 create RTA and TA, loading SecurityRole and binding all together
        ResourceTaskAssignment rta = new ResourceTaskAssignment();
        ArrayResult objs = null;
        objs = application.loadFromXpath(sessionID,"/SecurityRole[name='Task Responsible']",
            new SecurityRoleScope());
        printRPMExceptions(objs);        

        rta.setResource(evResource);
        rta.setSecurityRole((SecurityRole) objs.getRpmObjectList()[0]);
        rta.setResourceType(ResourceType.Resource);
        rta.setTaskAssignment(new TaskAssignment());
        rta.getTaskAssignment().setEETCHours(new Double(40.00));

        rta.getTaskAssignment().setAssignmentStartDate(startDate);

        evTask.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            rta
        });

        // 6.4 saving rta via task
        result = null;
        result = application.save(sessionID, evTask, allScope,
            ReloadType.ReloadResult);
        printRPMExceptions(result);        
        evTask = (Task) result.getRpmObject();

        // 7. Copy proposed to plan (project)
        result = null;
        result = application.copyProposedToPlan(sessionID,
            evProject, allScope);
        printRPMExceptions(result);        
        evProject = (Project) result.getRpmObject();

        // 8. Baseline save baseline (project)
        result = null;
        result = application.saveProjectBaseline(sessionID,
            evProject, allScope);
        printRPMExceptions(result);        
        evProject = (Project) result.getRpmObject();

        // 9. rollUp (project)
        result = null;
        result = application.rollupProjectEV(sessionID,
            evProject, allScope);
        printRPMExceptions(result);        

        //9.1 reloading containers
        evProject = (Project) result.getRpmObject();
        evTask = (Task) evProject.getChildren()[0];
        rta = evTask.getResourceTaskAssignments()[0];

        //10. update Task Actuals        
        rta.getTaskAssignment().setActualWork(new Double(50));

        result = null;
        result = application.save(sessionID, evTask, allScope,
            ReloadType.ReloadResult);
        printRPMExceptions(result);        
        evTask = (Task) result.getRpmObject();

        //11. rollup (project)
        result = null;
        result = application.rollupProjectEV(sessionID,
            evProject, allScope);
        printRPMExceptions(result);        
        evProject = (Project) result.getRpmObject();
        
        System.out.println("\n Creating EarnedValues Data - End");

    }

    private static void createScopes()
    {
        // Basic project scoping
        mainScope = new WorkElementScope();
        mainScope.setEarnedValues(true);
        mainScope.setParent(new WorkElementScope());
        mainScope.setContainingProject(new WorkElementScope());

        // Task scoping with rta and ta relations
        taskScope = new WorkElementScope();
        rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setTaskAssignment(new TaskAssignmentScope());
        taskScope.setResourceTaskAssignments(rtaScope);
        taskScope.setContainingProject(new WorkElementScope());

        // Complete scope of the tree: Project,Task,RTA,TA,POOL,RESOURCE
        allScope = new WorkElementScope();
        allScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        allScope.getResourceTaskAssignments().setResource(new ResourceScope());
        allScope.getResourceTaskAssignments().setTaskAssignment(
            new TaskAssignmentScope());
        allScope.getResourceTaskAssignments().getResource().setCalendar(
            new CalendarScope());
        allScope.setContainingProject(new WorkElementScope());

        allScope.setChildren(taskScope);
        allScope.setEarnedValues(true);

        // Resource scoping
        calScope = new CalendarScope();
        evResourceScope = new ResourceScope();
        evResourceScope.setParent(new PoolScope());
        evResourceScope.setRates(true);
        evResourceScope.setCalendar(calScope);
    }
    
    public static RPMCalendar createCalendar(RPMObjectScope scope) throws Exception
    {
        RPMCalendar cal = createDefaultCalendar();
         
        SaveResult sr = application.save(sessionID, cal, scope,ReloadType.ReloadResult);
        printRPMExceptions(sr); 
        cal = (RPMCalendar) sr.getRpmObject();
        return cal;
    }
    
    private static RPMCalendar createDefaultCalendar() throws Exception
    {
        RPMCalendar calendar = new RPMCalendar();

        calendar.setName(EV_CALENDAR
                + new Double(Math.random() * 100).intValue());
        calendar.setHoursPerDay(new Double(8));
        calendar.setHoursPerWeek(new Double(40));

        CalendarWeekDay defaultWD = new CalendarWeekDay();
        defaultWD.setParent(calendar);
        defaultWD.setType(CalendarWeekDayType.UseDefault);

        CalendarWeekDay non_workingWD = new CalendarWeekDay();
        non_workingWD.setParent(calendar);
        non_workingWD.setType(CalendarWeekDayType.NonWorking);

        // Creates standardMonday
        CalendarWeekDay standardMonday = defaultWD;
        calendar.setStandardMonday(standardMonday);

        // Creates standardTuesday
        CalendarWeekDay standardTuesday = defaultWD;
        calendar.setStandardTuesday(standardTuesday);

        // Creates standardWednesday
        CalendarWeekDay standardWednesday = defaultWD;
        calendar.setStandardWednesday(standardWednesday);

        // Creates StandardThursday
        CalendarWeekDay standardThursday = defaultWD;
        calendar.setStandardThursday(standardThursday);

        // Creates standardFriday
        CalendarWeekDay standardFriday = defaultWD;
        calendar.setStandardFriday(standardFriday);

        // Creates standardSaturday
        CalendarWeekDay standardSaturday = non_workingWD;
        calendar.setStandardSaturday(standardSaturday);

        // Creates standardSunday
        CalendarWeekDay standardSunday = non_workingWD;
        calendar.setStandardSunday(standardSunday);

        return calendar;
    }



}
