/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.wbs.sample;

import java.util.Calendar;

import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * This sample creates a project and associates a few measures to it, such as
 * Tasks, milestones and deliverables.
 * 
 * @desc Associate tasks, milestones to a project.
 * 
 */
public class WBSProjectChildrenSample extends SampleTestController
{
    static String projectName = "GENERIC_PROJECT_ABC";

    static String deliverableName = "DELIVERABLE";

    static String sumTaskName = "SUMMARY_TASK";

    static String taskName = "CHILD_TASK";

    static String milestoneName = "MILESTONE";

    static String wProductName = "WORK_PRODUCT";

    protected void runSample() throws Exception
    {
        WorkElementScope weScope = new WorkElementScope();
        weScope.setParent(new WorkElementScope());

        Project project = (Project) loadFromXpath(
            "/Project[@name='" + projectName + "']", weScope)
            .getRpmObjectList()[0];

        Deliverable deliverable = new Deliverable();
        SummaryTask sumTask = new SummaryTask();
        Task task = new Task();
        Milestone milestone = new Milestone();
        WorkProduct wProduct = new WorkProduct();

        /**
         * Initialize the objects and associate them to their parents.
         */
        deliverable.setParent(project);
        deliverable.setName(deliverableName);
        SaveResult result = save(deliverable, null, ReloadType.ReloadResult);
        deliverable = (Deliverable) result.getRpmObject();

        wProduct.setParent(project);
        wProduct.setName(wProductName);
        wProduct = (WorkProduct) save(wProduct, null, ReloadType.ReloadResult)
            .getRpmObject();

        sumTask.setParent(deliverable);
        sumTask.setName(sumTaskName);
        sumTask = (SummaryTask) save(sumTask, null, ReloadType.ReloadResult)
            .getRpmObject();

        milestone.setParent(sumTask);
        milestone.setName(milestoneName);
        milestone = (Milestone) save(milestone, null, ReloadType.ReloadResult)
            .getRpmObject();

        task.setParent(sumTask);
        task.setName(taskName);
        task.setPlanDate(new WbsScheduleDate());
        task.getPlanDate().setStartDate(Calendar.getInstance());
        task = (Task) save(task, null, ReloadType.ReloadResult).getRpmObject();

        /**
         * Save the entire object graph using the scope we have specified
         */
        save(project, weScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new WBSProjectChildrenSample();
    }

}
