/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.wbs.sample;

import com.ibm.rpm.applicationadministration.containers.ProjectRTF;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * This sample modifies a specific field of a project
 * 
 * @desc Modify an RTF field of a project
 * 
 */
public class WBSProjectRTFSample extends SampleTestController
{
    static String projectName = "GENERIC_PROJECT_ABC";

    static String rtfType = "Key Accomplishments";

    static String rtfValue = "{\\rtf1\\ansi\\deff0{\\fonttbl"
            + "{\\f0\\fnil\\fcharset0 Arial;}{\\f1\\fnil\\fcharset0 Tahoma;}}"
            + "\n{\\colortbl ;\\red255\\green0\\blue0;}\n\\viewkind4\\uc1\\"
            + "pard\\cf1\\lang1033\\b\\fs28 This is a test.\\cf0\\b0\\f1\\fs20"
            + "\n\\par }";

    protected void runSample() throws Exception
    {
        WorkElementScope pScope = new WorkElementScope();
        pScope.setRtfAssignments(true);

        /**
         * Load the project and the field we wish to modify
         */
        Project project = (Project) loadFromXpath(
            "/Project[@name='" + projectName + "']", pScope).getRpmObjectList()[0];

        ProjectRTF pRtf = (ProjectRTF) loadFromXpath(
            "/ProjectRTF[@value='" + rtfType + "']", null).getRpmObjectList()[0];

        /**
         * We associate our custom RTF text to the field we wish to modify
         */
        RtfAssignment rtfAssign = new RtfAssignment();

        rtfAssign.setRtf(pRtf);
        rtfAssign.setValue(rtfValue);

        /**
         * We then associate the project and the assignment, in both directions
         */
        rtfAssign.setParent(project);
        project.setRtfAssignments(new RtfAssignment[]
        {
            rtfAssign
        });

        project.setPublished(ProjectPublishedType.None);

        save(project, pScope, ReloadType.None);
    }

    public static void main(String[] args)
    {
        new WBSProjectRTFSample();
    }
}
