/*
 * Created on March 31, 2006
 */
package com.ibm.rpm.applicationadministration.checkpoints;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.CalendarException;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.util.WorkIntervalUtil;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

public class CalendarExceptionCheckpoint extends AbstractCheckpoint
{
    private static Class CALENDAREXCEPTION_PARENTS = RPMCalendar.class;

    protected static CalendarExceptionCheckpoint instance = new CalendarExceptionCheckpoint();

    public static CalendarExceptionCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, true, CALENDAREXCEPTION_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, CalendarException.class, msgCtx))
            return;

        // Initialize
        CalendarException mainCalendarException = (CalendarException) rpmObject;

        // Field validation: intervals
        if (mainCalendarException.getIntervals() != null)
        {
            GenericValidator.validateMaxSize(mainCalendarException,
                "intervals", mainCalendarException.getIntervals().length,
                ValidationConstants.CALENDARWEEKDAY_INTERVALS_MAX, msgCtx);

            try
            {
                WorkIntervalUtil.validate(mainCalendarException.getIntervals());
            }
            catch (RPMException e)
            {
                addException(e, msgCtx);
            }
        }        
    }    
    
}
