/*
 * Created on Sep 16, 2005
 */
package com.ibm.rpm.applicationadministration.checkpoints;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.CalendarWeekDay;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.util.WorkIntervalUtil;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

public class CalendarWeekDayCheckpoint extends AbstractCheckpoint
{
    private static Class CALENDARWEEKDAY_PARENTS = RPMCalendar.class;

    protected static CalendarWeekDayCheckpoint instance = new CalendarWeekDayCheckpoint();

    public static CalendarWeekDayCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, false, false, CALENDARWEEKDAY_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, CalendarWeekDay.class, msgCtx))
            return;

        // Initialize
        CalendarWeekDay mainCalendarWeekDay = (CalendarWeekDay) rpmObject;

        // Field validation: intervals
        if (mainCalendarWeekDay.getIntervals() != null)
        {
            GenericValidator.validateMaxSize(mainCalendarWeekDay,
                "intervals", mainCalendarWeekDay.getIntervals().length,
                ValidationConstants.CALENDARWEEKDAY_INTERVALS_MAX, msgCtx);

            try
            {
                WorkIntervalUtil.validate(mainCalendarWeekDay.getIntervals());
            }
            catch (RPMException e)
            {
                addException(e, msgCtx);
            }
        }        
    }    
   
}
