/*
 * Created on Sep 16, 2005
 */
package com.ibm.rpm.applicationadministration.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.AdministrationModule;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

/**
 * @author dominiqu
 */
public class CurrencyCheckpoint extends AbstractCheckpoint
{
    private static final List CURRENCY_PARENTS = new ArrayList();

    protected static CurrencyCheckpoint instance = new CurrencyCheckpoint();
    
    static
    {
        CURRENCY_PARENTS.add(AdministrationModule.class);
    }

    public static CurrencyCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, false, CURRENCY_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, Currency.class, msgCtx))
            return;

        // Initialize
        Currency mainCurrency = (Currency) rpmObject;

        // Field validation: name

        // name cannot be null or empty if an exchange rate has been specified
        if ((mainCurrency.getExchangeRate() != null && mainCurrency.getExchangeRate()
            .doubleValue() != 0)
            || (mainCurrency.getExchangeRate() == null))
        {
            GenericValidator.validateMandatory(mainCurrency, "name",
                mainCurrency.getName(), msgCtx);

            GenericValidator.validateMinLength(mainCurrency, "name",
                mainCurrency.getName(), ValidationConstants.CURRENCY_NAME_MIN, msgCtx);
        }

        GenericValidator.validateMaxLength(mainCurrency, "name",
            mainCurrency.getName(), ValidationConstants.CURRENCY_NAME_MAX, msgCtx);

        // Field validation: exchangeRate

        // exchangeRate cannot be null or empty if a name has been specified
        if (mainCurrency.getName() != null && mainCurrency.getName().length() != 0)
        {
            GenericValidator.validateMandatory(mainCurrency, "exchangeRate",
                mainCurrency.getExchangeRate(), msgCtx);
        }

        GenericValidator.validateRange(mainCurrency, "exchangeRate",
            mainCurrency.getExchangeRate(),
            ValidationConstants.CURRENCY_EXCHANGERATE_MIN,
            ValidationConstants.CURRENCY_EXCHANGERATE_MAX, msgCtx);        
    }   
    
}