/*
 * Created on Sep 19, 2005
 */
package com.ibm.rpm.applicationadministration.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.AdministrationModule;
import com.ibm.rpm.applicationadministration.containers.Department;
import com.ibm.rpm.applicationadministration.containers.Group;
import com.ibm.rpm.applicationadministration.containers.Office;
import com.ibm.rpm.applicationadministration.containers.Organization;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

/**
 * @author gosselin
 */
public class OrganizationalCheckpoint extends AbstractCheckpoint
{
    private static final List OFFICE_PARENTS = new ArrayList();

    private static final List DEPARTMENTGROUP_PARENTS = new ArrayList();

    private static final List ORGANIZATION_PARENTS = new ArrayList();

    static
    {
        ORGANIZATION_PARENTS.add(AdministrationModule.class);
        ORGANIZATION_PARENTS.add(Organization.class);
        
        OFFICE_PARENTS.add(Organization.class);
        OFFICE_PARENTS.add(Department.class);
        OFFICE_PARENTS.add(Group.class);
        OFFICE_PARENTS.add(Office.class);

        DEPARTMENTGROUP_PARENTS.add(Organization.class);
        DEPARTMENTGROUP_PARENTS.add(Department.class);
        DEPARTMENTGROUP_PARENTS.add(Group.class);
    }

    protected static OrganizationalCheckpoint instance = new OrganizationalCheckpoint();

    public static OrganizationalCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        if (!GenericValidator.validateInstance(rpmObject, Organizational.class, msgCtx))
            return;
        
        // Check parent type
        if (rpmObject instanceof Office)
        {
            super.isValidParent(rpmObject, msgCtx, true, OFFICE_PARENTS);
        }
        else if (rpmObject instanceof Department || rpmObject instanceof Group)
        {
            super.isValidParent(rpmObject, msgCtx, true, DEPARTMENTGROUP_PARENTS);
        }
        else if (rpmObject instanceof Organization)
        {
            super.isValidParent(rpmObject, msgCtx, false, ORGANIZATION_PARENTS);
        }
        else
        {
            List expectedTypes = new ArrayList();
            expectedTypes.add( Office.class );
            expectedTypes.add( Department.class );
            expectedTypes.add( Group.class );
            expectedTypes.add( Organization.class );

            GenericValidator.validateCannotCreate(rpmObject, expectedTypes, msgCtx);
        }

        // Initialize
        Organizational mainOrganizational = (Organizational) rpmObject;

        // Field validation: name
        if (mainOrganizational.testNameModified())
        {
            GenericValidator.validateMaxLength(mainOrganizational,
                "name", mainOrganizational.getName(),
                ValidationConstants.ORGANIZATIONAL_NAME_MAX, msgCtx);
        }

        // Field validation: description
        if (mainOrganizational.testDescriptionModified())
        {
            GenericValidator.validateMaxLength(mainOrganizational,
                "description", mainOrganizational.getDescription(),
                ValidationConstants.ORGANIZATIONAL_DESCRIPTION_MAX, msgCtx);
        }        
        
    }    
    
}
