/*
 * Created on Sep 16, 2005
 */
package com.ibm.rpm.applicationadministration.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.AdministrationModule;
import com.ibm.rpm.applicationadministration.containers.CalendarException;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ContainerToCheckpointMap;
import com.ibm.rpm.framework.util.GenericValidator;

public class RPMCalendarCheckpoint extends AbstractCheckpoint
{
    private static final List RPMCALENDAR_PARENTS_LIST = new ArrayList();

    static
    {
        RPMCALENDAR_PARENTS_LIST.add(RPMCalendar.class);
        RPMCALENDAR_PARENTS_LIST.add(AdministrationModule.class);
    }

    protected static RPMCalendarCheckpoint instance = new RPMCalendarCheckpoint();

    public static RPMCalendarCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, false, RPMCALENDAR_PARENTS_LIST);

        if (!GenericValidator.validateInstance(rpmObject, RPMCalendar.class,
            msgCtx))
            return;

        // Initialize
        RPMCalendar mainCalendar = (RPMCalendar) rpmObject;
        CalendarScope scope = (CalendarScope) rpmScope;

        // Field validation: name
        if (mainCalendar.testNameModified())
        {
            GenericValidator.validateMaxLength(mainCalendar, "name",
                mainCalendar.getName(), ValidationConstants.CALENDAR_NAME_MAX,
                msgCtx);
        }

        // Field validation: hoursPerDay
        if (mainCalendar.testHoursPerDayModified())
        {
            GenericValidator.validateRange(mainCalendar, "hoursPerDay",
                mainCalendar.getHoursPerDay(),
                ValidationConstants.CALENDAR_HOURSPERDAY_MIN,
                ValidationConstants.CALENDAR_HOURSPERDAY_MAX, msgCtx);
        }

        // Field validation: hoursPerWeek
        if (mainCalendar.testHoursPerWeekModified())
        {
            GenericValidator.validateRange(mainCalendar, "hoursPerWeek",
                mainCalendar.getHoursPerWeek(),
                ValidationConstants.CALENDAR_HOURSPERWEEK_MIN,
                ValidationConstants.CALENDAR_HOURSPERWEEK_MAX, msgCtx);
        }

        AbstractCheckpoint ce = ContainerToCheckpointMap
        .getCheckpoint(CalendarException.class);

        
        if (scope != null && scope.isExceptions() && 
                mainCalendar.getExceptions() != null)
        {
            for (int i = 0; i < mainCalendar.getExceptions().length; ++i)
                ce.validateSave(mainCalendar.getExceptions()[i], rpmScope,
                    msgCtx);
        }        
    }    
    
}
