package com.ibm.rpm.applicationadministration.managers;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.AdministrationModule;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.AdministrationModuleScope;
import com.ibm.rpm.framework.AbstractModuleManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;

/**
 * @author Priti
 *
 */
public class AdministrationModuleManager extends AbstractModuleManager {

    private static Log logger = LogFactory.getLog(AdministrationModuleManager.class);

    protected Log getLogger()
    {
        return logger;
    }

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(AdministrationModule.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                AdministrationModuleManager.class);
        }
    }

    {
        localContextName = ContextUtil.NONE;
    }



    protected RPMObject createContainer(int typeId) throws RPMException
    {
        return new AdministrationModule();
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        
        if (manager == null)
        {
            joinCondition = new JoinCondition();
        }
        else if(manager instanceof DatafieldManager)
        {
        	joinCondition = new JoinCondition();      	
        }
        else if(manager instanceof RPMCalendarManager)
        {
        	joinCondition = new JoinCondition();      	
        }
        else if(manager instanceof OrganizationalManager)
        {
        	joinCondition = new JoinCondition();      	
        }
        else if(manager instanceof GeographicalManager)
        {
        	joinCondition = new JoinCondition();      	
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }
    
    protected void loadChildren(AbstractModule abstractModule,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        AdministrationModule module = (AdministrationModule) abstractModule;
        AdministrationModuleScope scope = (AdministrationModuleScope) rpmScope;

        //loadCurrencies
        if(scope.isCurrencies())
        {
        	String extraWhere = "LEVEL_ID=2 AND TYPE_ID=37 AND DESCRIPTION IS NOT NULL ";
        	Currency[]  currency = (Currency[] )loadElements(
        	        module, module.getCurrencies(),
        			null, Currency.class, null,extraWhere, messageContext, isDelta );
        	module.setCurrencies(currency);
        }
        
        //loadRpmCalendars
        if(scope.getRpmCalendars() != null)
        { 
            String extraWhere = RPMCalendarManager.NAME_TYPE_ID 
	            + "=" + RPMCalendar.TYPE_ID + " AND " 
	            + RPMCalendarManager.NAME_LEVEL_ID + "=0";
            RPMCalendar[]  rpmCalender = (RPMCalendar[] )loadElements(module, module.getRpmCalendars(),
        			scope.getRpmCalendars(),RPMCalendar.class, 
					null, extraWhere, messageContext, isDelta );
        	module.setRpmCalendars(rpmCalender);
        }

        //loadOrgnizationals
        if(scope.getOrganizationals() != null)
        {
        	String extraWhere = " LEVEL_ID = 0";
        	Organizational[]  orgnizational = (Organizational[]) loadElements(module, module.getOrganizationals(),
        			scope.getOrganizationals(),Organizational.class, 
					null,extraWhere, messageContext, isDelta );
        	module.setOrganizationals(orgnizational);
        }
        
        //loadGeographicals
        if(scope.getGeographicals() != null)
        {
        	String extraWhere = " LEVEL_ID = 0";
        	Geographical[]  geographical = (Geographical[] )loadElements(module, module.getGeographicals(),
        			scope.getGeographicals(),Geographical.class, 
					null,extraWhere, messageContext, isDelta );
        	module.setGeographicals(geographical);
        }

        //loadDatafieldCategories
        if(scope.isDatafieldCategories())
        {
        	String extraWhere = " LEVEL_ID = 1";
        	DatafieldCategory[]  datafieldCategories = 
        	    	(DatafieldCategory[] )loadElements(module, 
        	    	        module.getDatafieldCategories(),
        			null,DatafieldCategory.class, 
					null,extraWhere, messageContext, isDelta);
        	module.setDatafieldCategories(datafieldCategories);
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        AdministrationModuleScope scope = (AdministrationModuleScope) rpmScope;
        AdministrationModule module = (AdministrationModule) rpmObject;

        if( scope != null )
        {
	        //saveRpmCalendars
	        if(scope.getRpmCalendars() !=null)
	        {
	 	       	RPMCalendar[] rpmCalendar = (RPMCalendar[]) saveElements(
	        			module, module.getRpmCalendars(),
						scope.getRpmCalendars(), messageContext,
						RPMCalendar.class );
	        	module.setRpmCalendars(rpmCalendar);
	        }
	        
	        //saveOrganizationals
	        if(scope.getOrganizationals() !=null)
	        {
	 	       	Organizational[] organizational = (Organizational[]) saveElements(
	        			module, module.getOrganizationals(),
						scope.getOrganizationals(), messageContext,
						Organizational.class );
	        	module.setOrganizationals(organizational);
	        }
	        
	        //saveGeographicals
	        if(scope.getGeographicals() !=null)
	        {
	 	       	Geographical[] geographical = (Geographical[]) saveElements(
	        			module, module.getGeographicals(),
						scope.getGeographicals(), messageContext,
						Geographical.class );
	        	module.setGeographicals(geographical);
	        }
	
	        if(scope.isDatafieldCategories())
	        {
	            DatafieldCategory[] categories = (DatafieldCategory[]) saveElements(
	        			module, module.getDatafieldCategories(),
						scope.isDatafieldCategories(), messageContext,
						DatafieldCategory.class );
	        	module.setDatafieldCategories(categories);
	        }
        }

        return module;
    }
}
