/*
 * Created on Sep 12, 2005
 *
 */
package com.ibm.rpm.applicationadministration.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.GenericAttribute;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.applicationadministration.util.AttributeUtil;
import com.ibm.rpm.applicationadministration.util.InputTypeUtil;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.managers.AssetManager;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.managers.ClientManager;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.PoolManager;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsManager;

/**
 * @author karim
 *  
 */
public class AttributeAssignmentManager extends AbstractRPMObjectManager
{
    private static final HashSet CONTAINERS = new HashSet();

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    static
    {
        CONTAINERS.add(AttributeAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                AttributeAssignmentManager.class);
        }
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TYPE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_TYPE_ID = "TYPE_ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID, NAME_TYPE_ID);
    }

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "MAX_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     *  
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "CHILD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     *  
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "DELETED_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "ELEMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "ELEMENT_NAME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_PARENT_ID = "PARENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID, NAME_PARENT_ID);
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     *  
     */
    public static final int ID_REC_USER = 9;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     *  
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     *  
     */
    public static final int ID_REC_DATETIME = 11;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column MULTI_SELECT of type Types.CHAR mapped to String.
     *  
     */
    public static final int ID_MULTI_SELECT = 12;

    public static final int TYPE_MULTI_SELECT = Types.CHAR;

    public static final String NAME_MULTI_SELECT = "MULTI_SELECT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MULTI_SELECT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MULTI_SELECT,NAME_MULTI_SELECT);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     *  
     */
    public static final int ID_RANK = 13;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     *  
     */
    public static final int ID_DESCRIPTION = 14;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "DESCRIPTION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_DESCRIPTION = "VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column WEIGHT of type Types.SMALLINT mapped to Integer.
     *  
     */
    public static final int ID_WEIGHT = 15;

    public static final int TYPE_WEIGHT = Types.SMALLINT;

    public static final String NAME_WEIGHT = "WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WEIGHT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WEIGHT,NAME_WEIGHT);}

    /**
     * Column LOCAL of type Types.CHAR mapped to String.
     *  
     */
    public static final int ID_LOCAL = 16;

    public static final int TYPE_LOCAL = Types.CHAR;

    public static final String NAME_LOCAL = "LOCAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LOCAL = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LOCAL,NAME_LOCAL);}

    /**
     * Column PARENT_NAME of type Types.VARCHAR mapped to String.
     *  
     */
    public static final int ID_PARENT_NAME = 17;

    public static final int TYPE_PARENT_NAME = Types.VARCHAR;

    public static final String NAME_PARENT_NAME = "PARENT_NAME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARENT_NAME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_NAME,NAME_PARENT_NAME);}

    /**
     * Column INPUT_TYPE of type Types.SMALLINT mapped to Integer.
     *  
     */
    public static final int ID_INPUT_TYPE = 18;

    public static final int TYPE_INPUT_TYPE = Types.SMALLINT;

    public static final String NAME_INPUT_TYPE = "INPUT_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_INPUT_TYPE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_INPUT_TYPE,NAME_INPUT_TYPE);}

    /**
     * Create an array of type string containing all the fields of the
     * appropriate attributes table.
     */
    private static final String[] FIELD_NAMES =
    {
        NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT, NAME_DELETED_COUNT,
        NAME_ELEMENT_ID, NAME_ELEMENT_NAME, NAME_PARENT_ID, NAME_REC_USER,
        NAME_REC_STATUS, NAME_REC_DATETIME, NAME_MULTI_SELECT, NAME_RANK,
        NAME_DESCRIPTION, NAME_WEIGHT, NAME_LOCAL, NAME_PARENT_NAME, NAME_INPUT_TYPE
    };

    /**
     * Field that contains the comma separated fields of the appropriate
     * attributes table.
     */
    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new AttributeAssignmentManager instance.
     * 
     * @param Resource
     * 
     * @return AttributeAssignmentManager
     */
    public AttributeAssignmentManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {

        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return null;
    }

    /**
     * Creates a new AttributeAssignmentManager instance.
     * 
     * @param Resource
     * @return AttributeAssignmentManager
     */
    public String[] getPrimaryKeyArray(RPMObject rpmObject)
    {

        /*
         * ELEMENT_ID is mapped to the parent (ClientCostCenter, Asset,
         * Document... )and PARENT_ID is mapped to the Attribute
         */
        return new String[]
        {
            NAME_ELEMENT_ID, NAME_PARENT_ID, NAME_TYPE_ID
        };
    }

    public Object[] getPrimaryKeyValues(RPMObject rpmObject)
    {
        AttributeAssignment object = (AttributeAssignment) rpmObject;

        String parentId = object.getID();
        
        String attributeId = null;
        
        if (object.getAttribute() != null)
        {
            attributeId = object.getAttribute().getID();
        }
        
    	Integer typeId = object.retrieveTypeID();
    	
        if(typeId == null)
        {
        	typeId = retrieveTypeId(object);
        }
        	
        return new Object[]
        {
            parentId, attributeId, typeId
        };
    }

    protected RPMObject setPrimaryKeyFields(Object[] primaryKey, RPMObject rpmObject)
    {

        AttributeAssignment attributeAssignment = (AttributeAssignment) rpmObject;

        attributeAssignment.setID((String) primaryKey[0]);

        GenericAttribute genericAttribute = new GenericAttribute();
        genericAttribute.setID((String) primaryKey[1]);
        genericAttribute.setContextName(attributeAssignment.getContextName());

        attributeAssignment.setAttribute(genericAttribute);

        attributeAssignment.assignTypeID(new Integer((String) primaryKey[2]));

        return attributeAssignment;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws RPMException,
            SQLException
    {

        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    public String getTableName()
    {
        ExceptionUtil.handleGetTableName(this);
        return null;
    }

    public String getTableName(RPMObject rpmObject)
    {
        String contextName = getContextName(rpmObject);
        return getTableName(contextName);
    }

    public String getTableName(String contextName)
    {
        return getTableNameFromContext(contextName);
    }

    public static String getTableNameFromContext(String contextName)
    {
        return AttributeUtil.getInstance().getTableName(contextName);
    }

    protected Integer getTableType(RPMObject rpmObject)
    {
        String contextName = getContextName(rpmObject);
        return getTableType(contextName);
    }

    protected Integer getTableType(String contextName)
    {
        return AttributeUtil.getInstance().getTableType(contextName);
    }

    private String getContextName(RPMObject rpmObject){
        AttributeAssignment assignment = (AttributeAssignment)rpmObject;
        //try to get the context from the assignment object itself
        String contextName = assignment.getContextName();
        if(contextName == null){
            //try to get the context from the attribute attached to it
            contextName = assignment.getAttribute().getContextName();
            
            if(contextName == null){
                //try to get the context from the parent
                contextName = assignment.getParent().getContextName();
            }
        }
        return contextName;
    }
    
    public String getFilter()
    {
        StringBuffer filter = new StringBuffer();

        filter.append(NAME_REC_STATUS);
        filter.append(" != ");
        filter.append("'D'");
        
        return filter.toString();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String, int)
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager == null)
        {
            ExceptionUtil.handleNoParentManager(this, containerName);
        }
        else if (isValidParentManager(manager))
        {

            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            if (condition != null)
            {
                where.append(condition);
            }
            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof AttributeManager)
        {

            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();

            where.append("(LEVEL_ID >= ");
            where.append(level);
            where.append(") ");
            where.append(" AND ");
            where.append(NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_PARENT_ID);
            where.append(" FROM ");
            where.append(getTableName(contextName));
            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName, String contextName)
            throws RPMException
    {
        JoinCondition joinCondition = null;
        if (isValidParentManager(manager))
        {
            joinCondition = new JoinCondition();
            StringBuffer sb = new StringBuffer();
            sb.append(manager.getPrimaryKey() + " IN ");
            sb.append("( SELECT " + NAME_ELEMENT_ID + " FROM "
                + getTableName(contextName));
            if (condition != null)
            {
                sb.append(" WHERE " + condition);
            }
            
            joinCondition.setCondition(sb.toString());
        }
        else
        {

            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }
        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (isValidParentManager(manager))
        {

            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(" ELEMENT_ID ");
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1, null,
                contextName);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            
            String tableName = getTableName(contextName);
            
            //exclude attribute assignments related to deleted attributes
            StringBuffer condition2 = new StringBuffer(joinCondition.getCondition());
            condition2.append(" AND ");
            condition2.append(NAME_PARENT_ID);
            condition2.append(" IN (SELECT ELEMENT_ID FROM ");
            condition2.append(tableName);

            /* This change is done to fix defect RATLC00257017
               This requires that the AttributeAssignment should be loaded with the Attributes
               for the Attributes that are soft deleted */
            //condition2.append(" WHERE REC_STATUS != 'D')");
            condition2.append(")");
            joinCondition.setCondition(condition2.toString());
            
            joinCondition.setTableName(tableName);

        }
        else if (manager instanceof AttributeManager)
        {
            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(" PARENT_ID ");
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1, null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    private boolean isValidParentManager(RPMObjectManager manager)
    {
        return manager instanceof ClientManager || manager instanceof PoolManager
            || manager instanceof ResourceManager || manager instanceof DocumentManager
            || manager instanceof AssetManager //Asset manager
            || manager instanceof ScopeManager //Scope manager
            || manager instanceof WbsManager //WBS manager
        ;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId)
    {

        RPMObject object = null;
        object = new AttributeAssignment();
        object.assignTypeID(new Integer(typeId));

        return object;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields, int index,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        //Everything is loaded without scope Except parent of Attribute Assignment
        AttributeAssignment attributeAssignment = (AttributeAssignment) rpmObject;

        //Load the parent of the Attribute Assignment based on the Context
        if(rpmScope != null && rpmScope.getParent() != null)
        {
        	String contextName = attributeAssignment.getContextName();
        	if(contextName != null)
        	{
        		RPMObject parent = loadParent(contextName, attributeAssignment.getID(), rpmScope, messageContext, isDelta);
        		if(isDelta)
            	{
            		attributeAssignment.deltaParent(parent);
            	}
            	else
            	{
            		attributeAssignment.setParent(parent);
            	}
        	}
        }
        
        //Related object Attribute
        GenericAttribute genericAttribute = null;
        if (attributeAssignment.getAttribute() != null)
        {
            genericAttribute = attributeAssignment.getAttribute();
        }
        else
        {
            //TODO make sure it's ok not to set typeId
            genericAttribute = new GenericAttribute();
            String attrId = (String) relatedFields.get(index, ID_PARENT_ID);
            genericAttribute.setID(attrId);
            genericAttribute.setContextName(attributeAssignment.getContextName());
        }
        AttributeManager manager = (AttributeManager) getManagerInstance(Attribute.class
            .getName());
        
            genericAttribute = (GenericAttribute) manager.loadByPrimaryKey(
                genericAttribute, null, messageContext, isDelta);
        
        if (isDelta)
        {
            attributeAssignment.deltaAttribute(genericAttribute);
        }
        else
        {
            attributeAssignment.setAttribute(genericAttribute);
        }
        return rpmObject;
    }

    private RPMObject loadParent(String contextName, String ID, RPMObjectScope rpmScope, 
    		MessageContext messageContext, boolean isDelta) throws RPMException, SQLException, ParseException
    {
    	RPMObject parent = null;
    	RPMObjectManager manager = null;
    	if(ContextUtil.WBS_CONTEXT.equalsIgnoreCase(contextName))
		{
    		manager= getManagerInstance(WorkElement.class);
			parent = new WorkElement();
		}
		else if(ContextUtil.SCOPE_CONTEXT.equalsIgnoreCase(contextName))
		{
			manager = getManagerInstance(ScopeElement.class);
			parent = new ScopeElement();
		}
		else if(ContextUtil.POOL_CONTEXT.equalsIgnoreCase(contextName))
		{
			manager = getManagerInstance(Pool.class);
			parent = new Pool();
		}
		else if(ContextUtil.ASSET_CONTEXT.equalsIgnoreCase(contextName))
		{
			manager = getManagerInstance(Asset.class);
			parent = new Asset();
		}
		else if(ContextUtil.CLIENT_CONTEXT.equalsIgnoreCase(contextName))
		{
			manager = getManagerInstance(ClientCostCenter.class);
			parent = new ClientCostCenter();
		}
		else if(ContextUtil.getInstance().isDocumentContext(contextName))
		{
			manager = getManagerInstance(GenericDocument.class);
			parent = new DocumentElement();
		}
    	
    	if(parent != null && manager != null)
    	{
    		parent.setID(ID);
    		parent = manager.loadByPrimaryKey(parent, rpmScope.getParent(), messageContext,isDelta);
    	}
    	return parent;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.containers.RPMObject, int)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject object, boolean isDelta)
            throws SQLException, ParseException
    {

        AttributeAssignment assignment = (AttributeAssignment) object;

        assignment.setID(rs.getString(ID_ELEMENT_ID));
        
        //typeId is mandatory for load and save operations and is not
        //predefined in the container
        assignment.assignTypeID(Manager.getInteger(rs, ID_TYPE_ID));
        
		Object value = decodeValue(assignment, rs.getString(ID_DESCRIPTION));
		
        if (isDelta)
        {
            assignment.deltaValue(value);
        }
        else
        {
            assignment.setValue(value);
        }

        return assignment;
    }

    private Object decodeValue(AttributeAssignment assignment,
            String stringValue) throws ParseException
    {
        InputType inputType = null;

        GenericAttribute genericAttribute = assignment.getAttribute();
        if (genericAttribute != null && genericAttribute instanceof Attribute)
        {
            inputType = ((Attribute) genericAttribute).getInputType();
        }

        if (inputType != null && inputType == InputType.Date)
        {
            return DateUtil.convertToCalendarNoTime(stringValue);
        }
        return stringValue;
    }
    
    private Object encodeValue(AttributeAssignment assignment)
    {
        InputType inputType = null;

        GenericAttribute genericAttribute = assignment.getAttribute();
        if (genericAttribute != null && genericAttribute instanceof Attribute)
        {
            inputType = ((Attribute) genericAttribute).getInputType();
        }

        Object value = assignment.getValue();
        if (inputType == InputType.Date && value instanceof Calendar)
        {
            return DateUtil.convertToString((Calendar) value);
        }

        if( value == null)
        {
            return null;
        }
        
        return value.toString();
    }
    
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException, SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        return relatedFields;
    }

        
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        AttributeAssignment assignment = (AttributeAssignment) rpmObject;

        assignment.setID(assignment.getParent().getID());

        assignment.setContextName(assignment.getParent().getContextName());
        SP_A_ATR(assignment, messageContext);

        //afterInsert(assignment); // listener callback
    }

    private boolean isResourceWBSAttributeAssignment(AttributeAssignment assignment)
    {
        return assignment.getParent() instanceof Resource;
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        AttributeAssignment assignment = (AttributeAssignment) rpmObject;

        if (!isResourceWBSAttributeAssignment(assignment))
        {
            Attribute attribute = (Attribute) assignment.getAttribute();
            if (attribute != null && attribute.getInputType() != null && assignment.testValueModified())
            {
                SP_U_ATR(assignment, messageContext);
            }
        }

        return assignment;
    }

    protected void doDelete(RPMObject object, MessageContext messageContext)
            throws RPMException, SQLException
    {
        if (object != null)
        {
            SP_UA_ATR((AttributeAssignment) object, messageContext);
        }
    }

    /**
     * In case of a relatedAttributeAssignment, the type id needs to be set by the WbsManager.       
     * Else, the type id is deduced from the parent type
     * @param assignment
     * @return
     */
    private Integer retrieveTypeId(AttributeAssignment assignment)
    {
        Integer typeId = null;
        RPMObject parent = assignment.getParent();

        if (parent instanceof ScopeElement)
        {
        	if(parent instanceof AbstractScopeTask)
        	{
        		typeId = AttributeUtil.TYPE_ID_TASK_ATTRIBUTE;
        	}
        	else if(parent instanceof AbstractAggregateScope)
        	{
        		if(parent instanceof AbstractScope)
        		{
        			typeId = parent.retrieveTypeID();
        		}
        		else
        		{
        			typeId = AttributeUtil.TYPE_ID_DELIVERABLE_ATTRIBUTE;
        		}
        	}
        }
        else if (parent instanceof WorkElement)
        {
        	if(parent instanceof AggregateNode)
        	{
        		if(parent instanceof GenericProject)
        		{
        			typeId = getTableType(assignment);
        		}
        		else
        		{
        			typeId = AttributeUtil.TYPE_ID_DELIVERABLE_ATTRIBUTE;
        		}
        	}
        	else if(parent instanceof SimpleNode)
        	{
        		typeId =  AttributeUtil.TYPE_ID_TASK_ATTRIBUTE;
        	}
        }

        if(typeId == null){
        	typeId = getTableType(assignment);
        }
        
        return typeId;
    }
    
    private Integer retrieveAndUpdateTypeId(AttributeAssignment assignment)
    {
    	Integer typeId = retrieveTypeId(assignment);
    	assignment.assignTypeID(typeId);
        return typeId;
    }
    
    private String SP_A_ATR(AttributeAssignment assignment, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_A_ATR";
        Object args[] = new Object[4];
        args[0] = retrieveAndUpdateTypeId(assignment);
        if (assignment.getParent() != null)
        {
            args[1] = assignment.getParent().getID();
        }
        if (assignment.getAttribute() != null)
        {
            args[2] = assignment.getAttribute().getID();
        }
        args[3] = getUser(messageContext).getID();
        return executeProcedure(messageContext, name, args);
    }

    private void SP_U_ATR(AttributeAssignment assignment, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Attribute attribute = (Attribute) assignment.getAttribute();
        
        String name = "SP_U_ATR";
        Object args[] = new Object[8];
        args[0] = retrieveAndUpdateTypeId(assignment);
        if (assignment.getParent() != null)
        {
            args[1] = assignment.getParent().getID();
        }
        if (attribute != null)
        {
            args[2] = attribute.getID();
            args[3] = attribute.getName();
        }
        args[4] = encodeValue(assignment);
        args[5] = new Integer(0);
        InputType type = null;
        if (attribute != null)
        {
            type = attribute.getInputType();
        }
        args[6] = InputTypeUtil.getIndexByInputType(type);
        args[7] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_UA_ATR(AttributeAssignment assignment, MessageContext messageContext)
            throws RPMException, SQLException
    {
        String name = "SP_UA_ATR";
        Object args[] = new Object[4];
        args[0] = retrieveAndUpdateTypeId(assignment);
        if (assignment.getParent() != null)
        {
            args[1] = assignment.getParent().getID();
        }
        if (assignment.getAttribute() != null)
        {
            args[2] = assignment.getAttribute().getID();
        }
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    public void isValidPrimaryKey(RPMObject rpmObject, MessageContext messageContext) throws RPMException
    {
        boolean valid = true;
        StringBuffer invalidPrimaryKeys = new StringBuffer();

        if (rpmObject.getID() == null)
        {
            invalidPrimaryKeys.append("ID");
            valid = false;
        }

        if (((AttributeAssignment) rpmObject).getAttribute() == null
            || ((AttributeAssignment) rpmObject).getAttribute().getID() == null)
        {
            if (invalidPrimaryKeys.length() != 0)
                invalidPrimaryKeys.append(", ");
            invalidPrimaryKeys.append("Attribute.ID");
            valid = false;
        }

        if (!valid)
        {
            ExceptionUtil.handleMissingPrimaryKey(this, rpmObject, invalidPrimaryKeys
                .toString());
        }
    }
}