package com.ibm.rpm.applicationadministration.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.CurrencyExchangeRange;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;

public class CurrencyExchangeManager extends AbstractRPMObjectManager
{
    private static final String START_DATE_FIELD = "startDate";

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(CurrencyExchangeRange.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    CurrencyExchangeManager.class);
        }
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "CURRENCY_EXCHANGES.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column CURRENCY_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CURRENCY_ID = 2;

    public static final int TYPE_CURRENCY_ID = Types.INTEGER;

    public static final String NAME_CURRENCY_ID = "CURRENCY_EXCHANGES.CURRENCY_ID";

    public static final String PROPERTY_CURRENCY_ID = "CURRENCYID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CURRENCY_ID, NAME_CURRENCY_ID);
    }

    /**
     * Column START_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_START_DATE = 3;

    public static final int TYPE_START_DATE = Types.DATE;

    public static final String NAME_START_DATE = "CURRENCY_EXCHANGES.START_DATE";

    public static final String PROPERTY_START_DATE = "STARTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_START_DATE, NAME_START_DATE);
    }

    /**
     * Column FINISH_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_FINISH_DATE = 4;

    public static final int TYPE_FINISH_DATE = Types.DATE;

    public static final String NAME_FINISH_DATE = "CURRENCY_EXCHANGES.FINISH_DATE";

    public static final String PROPERTY_FINISH_DATE = "FINISHDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FINISH_DATE, NAME_FINISH_DATE);
    }

    /**
     * Column EXCHANGE_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_EXCHANGE_RATE = 5;

    public static final int TYPE_EXCHANGE_RATE = Types.DECIMAL;

    public static final String NAME_EXCHANGE_RATE = "CURRENCY_EXCHANGES.EXCHANGE_RATE";

    public static final String PROPERTY_EXCHANGE_RATE = "EXCHANGERATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EXCHANGE_RATE, NAME_EXCHANGE_RATE);
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 6;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "CURRENCY_EXCHANGES.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 7;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "CURRENCY_EXCHANGES.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 8;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "CURRENCY_EXCHANGES.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    public static final String TABLE_NAME = "CURRENCY_EXCHANGES";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_COMPETENCIES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_CURRENCY_ID, NAME_START_DATE,
            NAME_FINISH_DATE, NAME_EXCHANGE_RATE, NAME_REC_USER,
            NAME_REC_STATUS, NAME_REC_DATETIME
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new CurrencyExchangeManager instance.
     * 
     * @return CurrencyExchangeManager
     */
    public CurrencyExchangeManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
    {

        return new Integer(CurrencyExchangeRange.TYPE_ID);

    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId)
    {

        RPMObject object = new CurrencyExchangeRange();

        return object;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject pObject,
            boolean isDelta) throws SQLException
    {

        CurrencyExchangeRange currencyExchangeRange = (CurrencyExchangeRange) pObject;
        currencyExchangeRange.setID(rs.getString(ID_ELEMENT_ID));

        if (isDelta)
        {
            Calendar newStartDate = DateUtil.convertToCalendar(rs.getDate(ID_START_DATE));
            if (isDeltaDateNeeded(currencyExchangeRange.getStartDate(),
                    newStartDate))
            {
                currencyExchangeRange.deltaStartDate(newStartDate);
            }
            Calendar newFinishDate = DateUtil.convertToCalendar(rs
                    .getDate(ID_FINISH_DATE));
            if (isDeltaDateNeeded(currencyExchangeRange.getFinishDate(),
                    newFinishDate))
            {
                currencyExchangeRange.deltaFinishDate(newFinishDate);
            }

            currencyExchangeRange.deltaExchangeRate(new Double(rs
                    .getDouble(ID_EXCHANGE_RATE)));
        }
        else
        {
            currencyExchangeRange.setStartDate(DateUtil.convertToCalendar(rs
                    .getDate(ID_START_DATE)));
            currencyExchangeRange.setFinishDate(DateUtil.convertToCalendar(rs
                    .getDate(ID_FINISH_DATE)));
            currencyExchangeRange.setExchangeRate(new Double(rs
                    .getDouble(ID_EXCHANGE_RATE)));
        }

        return currencyExchangeRange;

    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws SQLException
    {

        String parentId = resultSet.getString(ID_CURRENCY_ID);
        relatedFields.put(index, ID_CURRENCY_ID, parentId);

        return relatedFields;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        CurrencyExchangeRange currencyExchangeRange = null;
        if (rpmObject != null)
        {
            currencyExchangeRange = (CurrencyExchangeRange) rpmObject;
            SP_D_EXCHANGE(currencyExchangeRange, messageContext);
        }
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition(TABLE_NAME, condition);
        }
        else if (manager instanceof CurrencyExchangeManager)
        {
            joinCondition = new JoinCondition(condition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(
            RPMObjectManager callingManager, String condition, int level,
            String containerName, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (callingManager instanceof DatafieldManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME, condition);
            //joinCondition.appendOrderBy(NAME_START_DATE);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, callingManager,
            ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof DatafieldManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendEqualQuestionMark(NAME_CURRENCY_ID);
            joinCondition.appendCondition(extraWhere);
            joinCondition.appendOrderBy(NAME_START_DATE);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
    	 if (rpmScope != null && rpmScope.getParent() != null){
             

        //parent currency is always loaded

        String oldParentId = (String) relatedFields.get(index, ID_CURRENCY_ID);
        Currency newParent = null;
        RPMObjectManager currencyManager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(Currency.class);

        if (isDelta)
        {
            newParent = (Currency) rpmObject.getParent();
            Currency oldParent = null;
            if (oldParentId != null)
            {
                oldParent = new Currency();
                oldParent.setID(oldParentId);
                if (newParent != null && newParent.getID() != null
                        && newParent.getID().equals(oldParentId))
                {
                    oldParent = (Currency) currencyManager.loadByPrimaryKey(
                            newParent, rpmScope.getParent(), messageContext, true);
                }
                else
                {
                    //In this case, old parent is loaded without the scope to
                    // avoid problem with the cache
                    oldParent = (Currency) currencyManager.loadByPrimaryKey(
                            oldParent, null, messageContext, false);
                }
            }
            rpmObject.deltaParent(oldParent);
        }
        else
        {
            if (oldParentId != null)
            {
            	
            	 if (newParent == null)
                	
                {//parent not in cache
                	newParent = new Currency();
                	newParent.setID(oldParentId);
                    newParent = (Currency) currencyManager.loadByPrimaryKey(
                            newParent, rpmScope
                            .getParent(), messageContext, false);
                }
                rpmObject.setParent(newParent);
            }
        }
    	 }
        return rpmObject;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        CurrencyExchangeRange currencyExchangeRange = (CurrencyExchangeRange) rpmObject;
        currencyExchangeRange.setID(SP_I_EXCHANGE(currencyExchangeRange,
                messageContext));
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException
    {
        CurrencyExchangeRange exchangeRange = (CurrencyExchangeRange) rpmObject;

        if (exchangeRange.testExchangeRateModified())
        {
            SP_U_EXCHANGE_RATE(exchangeRange, messageContext);
        }
        if (exchangeRange.testStartDateModified())
        {
            SP_U_EXCHANGE_START(exchangeRange, messageContext);
        }

        return exchangeRange;
    }

    private String SP_I_EXCHANGE(CurrencyExchangeRange currencyExchangeRange,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String sp_name = "SP_I_EXCHANGE";
        Object args[] = new Object[2];
        args[0] = currencyExchangeRange.getParent().getID();
        args[1] = getUser(messageContext).getID();

        String elementId = executeProcedure(messageContext, sp_name, args);
        return elementId;
    }

    private void SP_U_EXCHANGE_RATE(
            CurrencyExchangeRange currencyExchangeRange,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String sp_name = "SP_U_EXCHANGE_RATE";

        Object args[] = new Object[3];
        args[0] = currencyExchangeRange.getID();
        args[1] = currencyExchangeRange.getExchangeRate();
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, sp_name, args);
    }

    /**
     * Important business note: SP_U_EXCHANGE_START cannot be called on the
     * first CurrencyExchangeRange.
     */
    private void SP_U_EXCHANGE_START(
            CurrencyExchangeRange currencyExchangeRange,
            MessageContext messageContext) throws RPMException, SQLException
    {
        // SLegault: Patch to avoid calling SP_U_EXCHANGE_START on first range.
        if (!isFirstRange(currencyExchangeRange, messageContext))
        {
            String sp_name = "SP_U_EXCHANGE_START";

            Object args[] = new Object[3];
            args[0] = currencyExchangeRange.getID();
            args[1] = DateUtil.convertToString(currencyExchangeRange
                    .getStartDate());
            args[2] = getUser(messageContext).getID();

            executeProcedure(messageContext, sp_name, args);
        }
    }

    private boolean isFirstRange(CurrencyExchangeRange range,
            MessageContext messageContext) throws RPMException
    {
        SqlBuffer statement = new SqlBuffer();
        statement.appendSelect(NAME_CURRENCY_ID);
        statement.appendComma();
        statement.append(NAME_START_DATE);
        statement.appendFrom(TABLE_NAME);
        statement.appendWhere();
        statement.appendEqual(NAME_ELEMENT_ID, range.getID());
        List records = ManagerUtil.select(new Class[]
        {
                Integer.class, Date.class
        }, range, START_DATE_FIELD, statement, null, messageContext);
        List currencyAndDate = (List) records.get(0);
        Integer currencyId = (Integer) currencyAndDate.get(0);
        Date startDate = (Date) currencyAndDate.get(1);

        SqlBuffer condition = new SqlBuffer();
        condition.appendEqual(NAME_CURRENCY_ID, currencyId);
        condition.appendCondition(NAME_START_DATE + " < ");
        condition.appendParam(startDate);
        Object previousDate = ManagerUtil.selectColumnValue(Calendar.class,
                range, START_DATE_FIELD, NAME_START_DATE, TABLE_NAME,
                condition, messageContext);

        return previousDate == null;
    }

    private void SP_D_EXCHANGE(CurrencyExchangeRange currencyExchangeRange,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String sp_name = "SP_D_EXCHANGE";

        Object args[] = new Object[2];
        args[0] = currencyExchangeRange.getID();
        args[1] = getUser(messageContext).getID();

        executeProcedure(messageContext, sp_name, args);
    }

    // This method is needed because we only need to compare years, months and
    // days to determine if the two dates are different.
    private boolean isDeltaDateNeeded(Calendar oldDate, Calendar newDate)
    {
        String oldDateString = DateUtil.convertToString(oldDate);
        String newDateString = DateUtil.convertToString(newDate);
        return oldDateString == null || !oldDateString.equals(newDateString);
    }

}
