package com.ibm.rpm.applicationadministration.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.GeographicalAssignment;
import com.ibm.rpm.applicationadministration.containers.Region;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.managers.AssetManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;

public class GeographicalAssignmentManager extends AbstractRPMObjectManager {
    
	private static Log logger = LogFactory.getLog(GeographicalAssignmentManager.class);
	
	private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static{
        CONTAINERS.add(GeographicalAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                		GeographicalAssignmentManager.class);
        }
    }
    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_LEVEL_ID = 1;
    public static final int TYPE_LEVEL_ID = Types.INTEGER;
    public static final String NAME_LEVEL_ID = "TMT_GEOGRAPHIES.LEVEL_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;
    public static final int TYPE_TYPE_ID = Types.INTEGER;
    public static final String NAME_TYPE_ID = "TMT_GEOGRAPHIES.TYPE_ID";
    public static final String PROPERTY_TYPE_ID = "TYPE_ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MAX_RANK = 3;
    public static final int TYPE_MAX_RANK = Types.INTEGER;
    public static final String NAME_MAX_RANK = "TMT_GEOGRAPHIES.MAX_RANK";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CHILD_COUNT = 4;
    public static final int TYPE_CHILD_COUNT = Types.INTEGER;
    public static final String NAME_CHILD_COUNT = "TMT_GEOGRAPHIES.CHILD_COUNT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;
    public static final int TYPE_DELETED_COUNT = Types.INTEGER;
    public static final String NAME_DELETED_COUNT = "TMT_GEOGRAPHIES.DELETED_COUNT";
    public static final String PROPERTY_DELETED_COUNT = "DELETED";
    static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "TMT_GEOGRAPHIES.ELEMENT_ID";
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;
    public static final int TYPE_ELEMENT_NAME = Types.CHAR;
    public static final String NAME_ELEMENT_NAME = "TMT_GEOGRAPHIES.ELEMENT_NAME";
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME,NAME_ELEMENT_NAME);}

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;
    public static final int TYPE_PARENT_ID = Types.CHAR;
    public static final String NAME_PARENT_ID = "TMT_GEOGRAPHIES.PARENT_ID";
    public static final String PROPERTY_PARENT_ID = "PARENT";
    static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_USER = 9;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "TMT_GEOGRAPHIES.REC_USER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_STATUS = 10;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "TMT_GEOGRAPHIES.REC_STATUS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_DATETIME = 11;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "TMT_GEOGRAPHIES.REC_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column DEFAULT_FLAG of type Types.CHAR mapped to String.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_DEFAULT_FLAG = 12;
    public static final int TYPE_DEFAULT_FLAG = Types.CHAR;
    public static final String NAME_DEFAULT_FLAG = "TMT_GEOGRAPHIES.DEFAULT_FLAG";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DEFAULT_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_FLAG,NAME_DEFAULT_FLAG);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 13;
    public static final int TYPE_RANK = Types.VARCHAR;
    public static final String NAME_RANK = "TMT_GEOGRAPHIES.RANK";
    public static final String PROPERTY_RANK = "RANKID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 14;
    public static final int TYPE_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_DESCRIPTION = "TMT_GEOGRAPHIES.DESCRIPTION";
    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static{ FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION,NAME_DESCRIPTION);}

    private static final String TABLE_NAME = "TMT_GEOGRAPHIES";

    /**
     * Create an array of type string containing all the fields of the TMT_GEOGRAPHIES table.
     */
    private static final String[] FIELD_NAMES = 
    {
         NAME_LEVEL_ID
        ,NAME_TYPE_ID
        ,NAME_MAX_RANK
        ,NAME_CHILD_COUNT
        ,NAME_DELETED_COUNT
        ,NAME_ELEMENT_ID
        ,NAME_ELEMENT_NAME
        ,NAME_PARENT_ID
        ,NAME_REC_USER
        ,NAME_REC_STATUS
        ,NAME_REC_DATETIME
        ,NAME_DEFAULT_FLAG
        ,NAME_RANK
        ,NAME_DESCRIPTION
    };

    private static final String ALL_FIELDS;
    static{
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    };

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName() {
        return TABLE_NAME;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames(){
        return FIELD_NAMES;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
     protected String getAllFields() {
        return ALL_FIELDS;
    }  

	/**
	 * 
	 */
	public GeographicalAssignmentManager() {
		super();
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
	 */
	public String getPrimaryKey() {
		return null;  ////no primary key
	}

    public String[] getPrimaryKeyArray(RPMObject rpmObject) {
        
        /* ELEMENT_ID is mapped to the Resource and 
         * PARENT_ID is mapped to the Geographical*/
        return new String[]{NAME_ELEMENT_ID, NAME_PARENT_ID};
    }
    
    public Object[] getPrimaryKeyValues(RPMObject rpmObject){
        

        GeographicalAssignment object = (GeographicalAssignment)rpmObject;        
        
        String resourceId = object.getID();
        String geographicalId = null;
              
        if(object.getGeographical() != null){
            geographicalId = object.getGeographical().getID();
        }
                
    	return new Object[]{resourceId, geographicalId};
    }
    
    protected RPMObject setPrimaryKeyFields(Object[] primaryKey, RPMObject rpmObject){
    	
        GeographicalAssignment object = (GeographicalAssignment)rpmObject;
        
        object.setID((String)primaryKey[0]);
        
        Resource resource = new Resource();
        resource.setID((String)primaryKey[0]);
        object.setParent(resource);
        
        Geographical geographical = new Geographical();
        geographical.setID((String)primaryKey[1]);
    	object.setGeographical(geographical);
    	
    	return rpmObject;

    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws RPMException,
            SQLException {
        
        int typeId = resultSet.getInt(ID_TYPE_ID);        
        return new Integer(typeId);
    }
    
	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager, java.lang.String)
	 */
	protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
			String extraWhere, String contextName) throws RPMException {
    	JoinCondition joinCondition = null;
        
        if(manager instanceof ResourceManager){
    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(" ELEMENT_ID ");
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager,condition.toString(),1, null, null);
            if(extraWhere != null && extraWhere.length() > 0){
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }else if(manager instanceof AssetManager){
	    
        //where clause of the join condition
	    StringBuffer condition = new StringBuffer();
	    condition.append(" ELEMENT_ID ");
	    condition.append(" = ?");
                    
        joinCondition = getJoinCondition(manager,condition.toString(),1, null, null);
        if(extraWhere != null && extraWhere.length() > 0){
            StringBuffer sb = new StringBuffer();
            sb.append(joinCondition.getCondition());
            sb.append(" AND ");
            sb.append(extraWhere);
            joinCondition.setCondition(sb.toString());

        }
        joinCondition.setTableName(getTableName());
    }else if(manager instanceof GeographicalManager){

    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(" PARENT_ID ");
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager,condition.toString(),1, null ,null);
            if(extraWhere != null && extraWhere.length() > 0){
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
        else if(manager instanceof GeographicalAssignmentManager){
    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(" ELEMENT_ID ");
    	    condition.append(" = ?");
    	    condition.append(" AND ");
    	    condition.append(" PARENT_ID ");
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager,condition.toString(),1, null, null);
            if(extraWhere != null && extraWhere.length() > 0){
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}        
        return joinCondition;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager, java.lang.String, int)
	 */
	protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
			int level, String containerName, String contextName) throws RPMException {
        JoinCondition joinCondition = null;
        if(manager == null){
            ExceptionUtil.handleNoParentManager(this, containerName);        
        }       
        else if(manager instanceof ResourceManager){
            
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            where.append(NAME_TYPE_ID);
    	    where.append(" = ");
    	    where.append(" 13 ");
    	    if(condition != null){
    	        where.append(" AND ");
    	        where.append(condition);
    	    }
    	    
    	    joinCondition.setCondition(where.toString());
            
        }else if(manager instanceof GeographicalAssignmentManager){
            
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            where.append(NAME_TYPE_ID);
    	    where.append(" = ");
    	    where.append(" 13 ");
    	    if(condition != null){
    	        where.append(" AND ");
    	        where.append(condition);
    	    }
    	    
    	    joinCondition.setCondition(where.toString());
            
        }else if(manager instanceof AssetManager){
            
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            where.append(NAME_TYPE_ID);
    	    where.append(" = ");
    	    where.append(Asset.TYPE_ID);
    	    if(condition != null){
    	        where.append(" AND ");
    	        where.append(condition);
    	    }
    	    joinCondition.setCondition(where.toString());
        }else {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }               	
        
        return joinCondition;
	}
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);

        return joinCondition;
    }
    
	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
	 */
	protected HashMap getPropertiesMap() 
	{
        return FIELDPROPERTYMAP;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
	 */
	protected RPMObject createContainer(int typeId) throws RPMException {
        RPMObject object = null;

        object = new GeographicalAssignment();
        return object;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#loadFromResultSet(java.sql.ResultSet, com.ibm.rpm.scope.RPMObjectScope)
	 */
	protected RPMObject loadRelatedObjects(RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext, FieldValueMap relatedFields, int index, boolean isDelta)
			throws RPMException, SQLException, ParseException {

	    
	  	//Everything is loaded without scope
	    
	    GeographicalAssignment object = (GeographicalAssignment)rpmObject;
	    
        GeographicalManager geoManager = (GeographicalManager) RPMManagerFactory
        .getInstance().getRPMObjectManager(Region.class.getName());
        
        Geographical geo;
        if (object.getGeographical() != null)
        {
            geo = object.getGeographical();
        }
        else
        {
            geo = new Geographical();
            geo.setID(object.getGeographical().getID());
        }
        
        geo = (Geographical) geoManager.loadByPrimaryKey(geo, null, messageContext, isDelta);
        
        if (isDelta) {
            object.deltaGeographical(geo);
        } else {
            object.setGeographical(geo);
        }
	        
        //Related resource
        String resourceId = (String)relatedFields.get(index,ID_ELEMENT_ID);
        Integer resourceTypeId = (Integer)relatedFields.get(index,ID_TYPE_ID);
        if ((resourceId != null)&&(resourceTypeId!=null)&&(resourceTypeId.intValue()==Asset.TYPE_ID)) {
            AssetManager assetManager = (AssetManager)RPMManagerFactory.getInstance().getRPMObjectManager(Asset.class.getName());
            Asset asset = new Asset();
            asset.setID(resourceId);
            asset = (Asset)assetManager.loadByPrimaryKey(asset, null, messageContext, isDelta);
            if(isDelta){
            	object.deltaParent(asset);
            } else {
            	object.setParent(asset);	
            }
        } else if (resourceId != null) {
            ResourceManager resourceManager = (ResourceManager)RPMManagerFactory.getInstance().getRPMObjectManager(Resource.class.getName());
            Resource resource = new Resource();
            resource.setID(resourceId);
            resource = (Resource)resourceManager.loadByPrimaryKey(resource, null, messageContext, isDelta);
            if(isDelta){
            	object.deltaParent(resource);
            } else {
            	object.setParent(resource);	
            }
        } else {
            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.RESOURCE_ID);
        }            
	        
	    return rpmObject;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet, com.ibm.rpm.containers.RPMObject)
	 */
	protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject, boolean isDelta) throws RPMException,
			SQLException, ParseException {
    	RPMObject pObject = rpmObject;
        pObject.setID(rs.getString(ID_ELEMENT_ID));

        return pObject;
	}
	
    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet, com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException, SQLException{
    
        String resourceId = resultSet.getString(ID_ELEMENT_ID);
        relatedFields.put(index, ID_ELEMENT_ID, resourceId);
        Integer resourceTypeId = new Integer(resultSet.getInt(ID_TYPE_ID));
        relatedFields.put(index, ID_TYPE_ID, resourceTypeId);
       
        return relatedFields;
    }

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
	 */
	protected void doDelete(RPMObject rpmObject, MessageContext messageContext) throws RPMException, SQLException,
			ParseException {
        if(rpmObject != null){
        	GeographicalAssignment object = (GeographicalAssignment) rpmObject;
        	if (object.getParent() instanceof Resource){
            	SP_UA_RES_GEO(object, messageContext);
        	}else if (object.getParent() instanceof Resource){
            	SP_UA_ASSET_GEO(object, messageContext);
        	}

        }	         
	}

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject, boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext) throws SQLException, RPMException, ParseException
    {
        //beforeUpdate(pObject); // listener callback
        // nothing to update
        //     afterUpdate(pObject); 
        // listener callback
       
        return rpmObject;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext) throws SQLException, RPMException, ParseException
    {
    	GeographicalAssignment pObject = (GeographicalAssignment)rpmObject;
        //  beforeInsert(pObject); // listener callback
    	
        pObject.setID(pObject.getParent().getID());
        if (pObject.getParent() instanceof Resource){
            SP_A_RES_GEO(pObject, messageContext);
        } else if (pObject.getParent() instanceof Asset){
            SP_A_ASSET_GEO(pObject, messageContext);
        }
        
        //afterInsert(pObject); // listener callback
    }

    private String SP_A_RES_GEO(GeographicalAssignment assignment, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_RES_GEO";
        Object args[] = new Object[3];
        if(assignment.getParent() != null){
            args[0] = assignment.getParent().getID();
        }
        if(assignment.getGeographical() != null){
            args[1] = assignment.getGeographical().getID();
        }
        args[2] = getUser(messageContext).getID();
        
        return executeProcedure(messageContext, name, args);
    }
    
   
    private void SP_UA_RES_GEO(GeographicalAssignment assignment, MessageContext messageContext)throws RPMException, SQLException
    {
        String name = "SP_UA_RES_GEO";
        Object args[] = new Object[3];
        if(assignment.getParent() != null){
            args[0] = assignment.getParent().getID();
        }
        if(assignment.getGeographical() != null){
            args[1] = assignment.getGeographical().getID();
        }
        args[2] = getUser(messageContext).getID();
        
        executeProcedure(messageContext, name, args);
    }        
    
    private String SP_A_ASSET_GEO(GeographicalAssignment assignment, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_ASSET_GEO";
        Object args[] = new Object[3];
        if(assignment.getParent() != null){
            args[0] = assignment.getParent().getID();
        }
        if(assignment.getGeographical() != null){
            args[1] = assignment.getGeographical().getID();
        }
        args[2] = getUser(messageContext).getID();
        
        return executeProcedure(messageContext, name, args);
    }
    
   
    private void SP_UA_ASSET_GEO(GeographicalAssignment assignment, MessageContext messageContext)throws RPMException, SQLException
    {
        String name = "SP_UA_ASSET_GEO";
        Object args[] = new Object[3];
        if(assignment.getParent() != null){
            args[0] = assignment.getParent().getID();
        }
        if(assignment.getGeographical() != null){
            args[1] = assignment.getGeographical().getID();
        }
        args[2] = getUser(messageContext).getID();
        
        executeProcedure(messageContext, name, args);
    }        

    public void isValidPrimaryKey(RPMObject rpmObject, MessageContext messageContext) throws RPMException
    {
        boolean valid = true;
        StringBuffer invalidPrimaryKeys = new StringBuffer();
        
        if (rpmObject.getID() == null)
        {
            invalidPrimaryKeys.append("ID");
            valid = false;
        }
        
        if(((GeographicalAssignment) rpmObject).getGeographical() == null
                || ((GeographicalAssignment) rpmObject).getGeographical().getID() == null)
        {
            if (invalidPrimaryKeys.length() != 0)
                invalidPrimaryKeys.append(" and ");
            invalidPrimaryKeys.append("geographical.ID");
            valid = false;
        }
        
        if (!valid)
        {
            ExceptionUtil.handleMissingPrimaryKey(this, rpmObject, invalidPrimaryKeys.toString());
        }
    }
}
