package com.ibm.rpm.applicationadministration.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.Organization;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.containers.OrganizationalAssignment;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;


public class OrganizationalAssignmentManager extends AbstractRPMObjectManager {

	private static Log logger = LogFactory.getLog(OrganizationalAssignmentManager.class);
	
	private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static{
        CONTAINERS.add(OrganizationalAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                		OrganizationalAssignmentManager.class);
        }
    }
    
    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     * REPLACE PROPERTYNAME with container property in upper case.
     * 
     */
    public static final int ID_LEVEL_ID = 1;
    public static final int TYPE_LEVEL_ID = Types.INTEGER;
    public static final String NAME_LEVEL_ID = "TMT_ORGANIZATIONS.LEVEL_ID";

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;
    public static final int TYPE_TYPE_ID = Types.INTEGER;
    public static final String NAME_TYPE_ID = "TMT_ORGANIZATIONS.TYPE_ID";
    public static final String PROPERTY_TYPE_ID = "TYPE_ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}
    
    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_MAX_RANK = 3;
    public static final int TYPE_MAX_RANK = Types.INTEGER;
    public static final String NAME_MAX_RANK = "TMT_ORGANIZATIONS.MAX_RANK";

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CHILD_COUNT = 4;
    public static final int TYPE_CHILD_COUNT = Types.INTEGER;
    public static final String NAME_CHILD_COUNT = "TMT_ORGANIZATIONS.CHILD_COUNT";

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;
    public static final int TYPE_DELETED_COUNT = Types.INTEGER;
    public static final String NAME_DELETED_COUNT = "TMT_ORGANIZATIONS.DELETED_COUNT";
    public static final String PROPERTY_DELETED_COUNT = "DELETED";
    static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}
    
    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "TMT_ORGANIZATIONS.ELEMENT_ID";
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}    

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;
    public static final int TYPE_ELEMENT_NAME = Types.CHAR;
    public static final String NAME_ELEMENT_NAME = "TMT_ORGANIZATIONS.ELEMENT_NAME";
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME,NAME_ELEMENT_NAME);}    

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;
    public static final int TYPE_PARENT_ID = Types.CHAR;
    public static final String NAME_PARENT_ID = "TMT_ORGANIZATIONS.PARENT_ID";
    public static final String PROPERTY_PARENT_ID = "PARENT";
    static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}
    
    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_USER = 9;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "TMT_ORGANIZATIONS.REC_USER";

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_STATUS = 10;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "TMT_ORGANIZATIONS.REC_STATUS";

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_DATETIME = 11;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "TMT_ORGANIZATIONS.REC_DATETIME";

    /**
     * Column DEFAULT_FLAG of type Types.CHAR mapped to String.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_DEFAULT_FLAG = 12;
    public static final int TYPE_DEFAULT_FLAG = Types.CHAR;
    public static final String NAME_DEFAULT_FLAG = "TMT_ORGANIZATIONS.DEFAULT_FLAG";

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 13;
    public static final int TYPE_RANK = Types.VARCHAR;
    public static final String NAME_RANK = "TMT_ORGANIZATIONS.RANK";
    public static final String PROPERTY_RANK = "RANKID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}
    
    /**
     * Column FS_CODE of type Types.VARCHAR mapped to String.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_FS_CODE = 14;
    public static final int TYPE_FS_CODE = Types.VARCHAR;
    public static final String NAME_FS_CODE = "TMT_ORGANIZATIONS.FS_CODE";

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 15;
    public static final int TYPE_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_DESCRIPTION = "TMT_ORGANIZATIONS.DESCRIPTION";
    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static{ FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION,NAME_DESCRIPTION);}    


    private static final String TABLE_NAME = "TMT_ORGANIZATIONS";
    
   /**
     * Create an array of type string containing all the fields of the TMT_ORGANIZATIONS table.
     */
    private static final String[] FIELD_NAMES = 
    {
            NAME_LEVEL_ID,
            NAME_TYPE_ID,
            NAME_MAX_RANK,
            NAME_CHILD_COUNT,
            NAME_DELETED_COUNT,
            NAME_ELEMENT_ID,
            NAME_ELEMENT_NAME,
            NAME_PARENT_ID,
            NAME_REC_USER,
            NAME_REC_STATUS,
            NAME_REC_DATETIME,
            NAME_DEFAULT_FLAG,
            NAME_RANK,
            NAME_FS_CODE,
            NAME_DESCRIPTION       
    };

    /**
     * Field that contains the comma separated fields of the TMT_ORGANIZATIONS table.
     */
    private static final String ALL_FIELDS;
	static
	{
		StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
    	for(int i = 1 ; i < FIELD_NAMES.length; i++)
    	{
    		fieldcat.append(",");
    		fieldcat.append(FIELD_NAMES[i]);
    	}
    	ALL_FIELDS= fieldcat.toString();
    };   

    /**
	 * 
	 */
	public OrganizationalAssignmentManager() {
		super();
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
	 */
	public String getTableName() {
		return TABLE_NAME;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getFieldsNames()
	 */
	protected String[] getFieldsNames() {
		return FIELD_NAMES;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
	 */
	protected String getAllFields() {
		return ALL_FIELDS;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
	 */
	public String getPrimaryKey() {

		return null;  ////no primary key
	}

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws RPMException,
            SQLException {
        
        int typeId = resultSet.getInt(ID_TYPE_ID);        
        return new Integer(typeId);
    }

    public String[] getPrimaryKeyArray(RPMObject rpmObject) {
        /* ELEMENT_ID is mapped to the Resource and 
         * PARENT_ID is mapped to the Organizational*/
        return new String[]{NAME_ELEMENT_ID, NAME_PARENT_ID};
    }
    
    public Object[] getPrimaryKeyValues(RPMObject rpmObject){
        
        OrganizationalAssignment object = (OrganizationalAssignment)rpmObject;
        
        String resourceId = object.getID();
        String organizationalId = null;
        
        if(object.getOrganizational() != null){
            organizationalId = object.getOrganizational().getID();
        }
                
    	return new Object[]{resourceId, organizationalId};
    	
    }
    
    protected RPMObject setPrimaryKeyFields(Object[] primaryKey, RPMObject rpmObject){
        
        OrganizationalAssignment object = (OrganizationalAssignment)rpmObject;
        
        object.setID((String)primaryKey[0]);
        
        Resource resource = new Resource();
        resource.setID((String)primaryKey[0]);
        object.setParent(resource);
        
        Organizational organizational = new Organizational();
        organizational.setID((String)primaryKey[1]);
    	object.setOrganizational(organizational);
    	
    	return rpmObject;
    }
    
	
	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager, java.lang.String)
	 */
	protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
			String extraWhere, String contextName) throws RPMException {
    	JoinCondition joinCondition = null;
        
        if(manager instanceof ResourceManager){
    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(" ELEMENT_ID ");
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager,condition.toString(),1, null, null);
            if(extraWhere != null && extraWhere.length() > 0){
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
        else if(manager instanceof OrganizationalManager){
    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(" PARENT_ID ");
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager,condition.toString(),1, null, null);
            if(extraWhere != null && extraWhere.length() > 0){
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
        else if(manager instanceof OrganizationalAssignmentManager){
    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(" ELEMENT_ID ");
    	    condition.append(" = ?");
    	    condition.append(" AND ");
    	    condition.append(" PARENT_ID ");
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager,condition.toString(),1, null, null);
            if(extraWhere != null && extraWhere.length() > 0){
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}        
        return joinCondition;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager, java.lang.String, int)
	 */
	protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
			int level, String containerName, String contextName) throws RPMException {
        JoinCondition joinCondition = null;
        
        if(manager == null){
            ExceptionUtil.handleNoParentManager(this, containerName);         
        }        
        else if(manager instanceof ResourceManager){
            
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            where.append(NAME_TYPE_ID);
    	    where.append(" = ");
    	    where.append(" 13 ");
    	    if(condition != null){
    	        where.append(" AND ");
    	        where.append(condition);
    	    }
    	    
    	    joinCondition.setCondition(where.toString());
            
        }else if(manager instanceof OrganizationalAssignmentManager){
            
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            where.append(NAME_TYPE_ID);
    	    where.append(" = ");
    	    where.append(" 13 ");
    	    if(condition != null){
    	        where.append(" AND ");
    	        where.append(condition);
    	    }
    	    
    	    joinCondition.setCondition(where.toString());
            
        }else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        
        return joinCondition;
	}
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);


        return joinCondition;
    }
    
	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
	 */
	protected HashMap getPropertiesMap() {
		return FIELDPROPERTYMAP;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
	 */
	protected RPMObject createContainer(int typeId) throws RPMException {
        RPMObject object = null;

        object = new OrganizationalAssignment();
        return object;
	}

	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#loadFromResultSet(java.sql.ResultSet, com.ibm.rpm.scope.RPMObjectScope)
	 */
	protected RPMObject loadRelatedObjects(RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext, FieldValueMap relatedFields, int index, boolean isDelta)
			throws RPMException, SQLException, ParseException {

	    //Everything is loaded without scope

	    OrganizationalAssignment object = (OrganizationalAssignment) rpmObject;

        OrganizationalManager orgManager = (OrganizationalManager) RPMManagerFactory
                .getInstance()
                .getRPMObjectManager(Organization.class.getName());
        
        Organizational org;
        if (object.getOrganizational() != null)
        {
            org = object.getOrganizational();
        } 
        else
        {
            org = new Organizational();
            org.setID(object.getOrganizational().getID());  
        }
        
        org = (Organizational) orgManager.loadByPrimaryKey(org, null,
                messageContext, isDelta);
        
        if (isDelta) 
        {
            object.deltaOrganizational(org);
        } 
        else 
        {
            object.setOrganizational(org);
        }

        //Related resource
        String resourceId = (String) relatedFields.get(index, ID_ELEMENT_ID);
        if (resourceId != null) {
            ResourceManager resourceManager = (ResourceManager) RPMManagerFactory
                    .getInstance()
                    .getRPMObjectManager(Resource.class.getName());
            Resource resource = new Resource();
            resource.setID(resourceId);
            resource = (Resource) resourceManager.loadByPrimaryKey(resource,
                    null, messageContext, isDelta);
            if (isDelta) {
                object.deltaParent(resource);
            } else {
                object.setParent(resource);
            }
        } else {
            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.RESOURCE_ID);
        }

        return rpmObject;
	}

	/*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.containers.RPMObject)
     */
	protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject, boolean isDelta) throws RPMException,
			SQLException, ParseException {
    	RPMObject pObject = rpmObject;
		
        pObject.setID(rs.getString(ID_ELEMENT_ID));
        
        return pObject;
	}

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet, com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException, SQLException{
    
        String resourceId = resultSet.getString(ID_ELEMENT_ID);
        relatedFields.put(index, ID_ELEMENT_ID, resourceId);
        
        return relatedFields;
    }
	
	/* (non-Javadoc)
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
	 */
	protected void doDelete(RPMObject rpmObject, MessageContext messageContext) throws RPMException, SQLException,
			ParseException {
        if(rpmObject != null){
        	OrganizationalAssignment object = (OrganizationalAssignment)rpmObject;
            RPMObject parent = object.getParent();

            if(parent instanceof Resource){
                SP_UA_RES_ORG(object, messageContext);    
            }else{
                //TODO call method related to the parent type
            }            
   	
        }	         
	}

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject, boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext) throws SQLException, RPMException, ParseException
    {

        // nothing to update      
        return rpmObject;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext) throws SQLException, RPMException, ParseException
    {
    	OrganizationalAssignment pObject = (OrganizationalAssignment)rpmObject;
        RPMObject parent = pObject.getParent();

        pObject.setID(parent.getID());
        if(parent instanceof Resource){
            SP_A_RES_ORG(pObject, messageContext);    
        }else{
            //TODO call method related to the parent type
        }
        

    }

    private String SP_A_RES_ORG(OrganizationalAssignment assignment, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_RES_ORG";
        Object args[] = new Object[3];
        if(assignment.getParent() != null){
            args[0] = assignment.getParent().getID();
        }
        if(assignment.getOrganizational() != null){
            args[1] = assignment.getOrganizational().getID();
        }
        args[2] = getUser(messageContext).getID();
        
        return executeProcedure(messageContext, name, args);
    }
    
   
    private void SP_UA_RES_ORG(OrganizationalAssignment assignment, MessageContext messageContext)throws RPMException, SQLException
    {
        String name = "SP_UA_RES_ORG";
        Object args[] = new Object[3];
        if(assignment.getParent() != null){
            args[0] = assignment.getParent().getID();
        }
        if(assignment.getOrganizational() != null){
            args[1] = assignment.getOrganizational().getID();
        }
        args[2] = getUser(messageContext).getID();
        
        executeProcedure(messageContext, name, args);
    }        
    
    public void isValidPrimaryKey(RPMObject rpmObject, MessageContext messageContext) throws RPMException
    {
        boolean valid = true;
        StringBuffer invalidPrimaryKeys = new StringBuffer();
        
        if (rpmObject.getID() == null)
        {
            invalidPrimaryKeys.append("ID");
            valid = false;
        }
        
        if(((OrganizationalAssignment) rpmObject).getOrganizational() == null
                || ((OrganizationalAssignment) rpmObject).getOrganizational().getID() == null)
        {
            if (invalidPrimaryKeys.length() != 0)
                invalidPrimaryKeys.append(" and ");
            invalidPrimaryKeys.append("organizational.ID");
            valid = false;
        }
        
        if (!valid)
        {
            ExceptionUtil.handleMissingPrimaryKey(this, rpmObject, invalidPrimaryKeys.toString());
        }
    }
}
