/*
 * Created on Sep 16, 2005
 *
 */
package com.ibm.rpm.applicationadministration.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.GenericAttribute;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.clientcostcenters.containers.ClientModule;
import com.ibm.rpm.document.containers.DocumentModule;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.resource.containers.PoolModule;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.wbs.containers.WbsModule;

/**
 * @author karim
 * 
 */
public class AttributeUtil 
{
    private Map tableNameByContext = new HashMap();
    private Map tableTypeByContext = new HashMap();
    
    private static final List VALID_ATTRIBUTE_ROOT_LIST = new ArrayList();
	public static final Integer TYPE_ID_TASK_ATTRIBUTE = new Integer(3);
	public static final Integer TYPE_ID_DELIVERABLE_ATTRIBUTE = new Integer(2);
	public static final Integer TYPE_ID_SUMMARYTASK_ATTRIBUTE = new Integer(2);
    
    private static AttributeUtil singleton = new AttributeUtil();

    private AttributeUtil()
    {
         init();     
    }
    
    synchronized public static AttributeUtil getInstance()
    {
        return singleton;
    } 
    
    private void init(){
    	tableNameByContext.put(ContextUtil.ASSET_CONTEXT, "TMT_ASSET_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.ASSET_CONTEXT, new Integer(228));
    	tableNameByContext.put(ContextUtil.CLIENT_CONTEXT, "TMT_CLIENT_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.CLIENT_CONTEXT, new Integer(229));
    	tableNameByContext.put(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT, "TMT_DOCUMENT_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT, new Integer(14));
    	tableNameByContext.put(ContextUtil.POOL_CONTEXT, "TMT_POOL_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.POOL_CONTEXT, new Integer(11));
    	tableNameByContext.put(ContextUtil.SCOPE_CONTEXT, "TMT_CRI_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.SCOPE_CONTEXT, new Integer(4));
    	tableNameByContext.put(ContextUtil.WBS_CONTEXT, "TMT_WBS_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.WBS_CONTEXT, new Integer(1));
    	tableNameByContext.put(ContextUtil.RESOURCE_CONTEXT, "TMT_WBS_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.RESOURCE_CONTEXT, new Integer(13));
    	
    	//composed contexts
    	tableNameByContext.put(ContextUtil.ASSET_DOCUMENT_CONTEXT, "TMT_DOCUMENT_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.ASSET_DOCUMENT_CONTEXT, new Integer(14));
    	tableNameByContext.put(ContextUtil.CLIENT_DOCUMENT_CONTEXT, "TMT_DOCUMENT_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.CLIENT_DOCUMENT_CONTEXT, new Integer(14));
    	tableNameByContext.put(ContextUtil.RESOURCE_DOCUMENT_CONTEXT, "TMT_DOCUMENT_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.RESOURCE_DOCUMENT_CONTEXT, new Integer(14));
    	tableNameByContext.put(ContextUtil.SCOPE_DOCUMENT_CONTEXT, "TMT_DOCUMENT_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.SCOPE_DOCUMENT_CONTEXT, new Integer(14));
    	tableNameByContext.put(ContextUtil.WBS_DOCUMENT_CONTEXT, "TMT_DOCUMENT_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.WBS_DOCUMENT_CONTEXT, new Integer(14));
    	tableNameByContext.put(ContextUtil.POOL_DOCUMENT_CONTEXT, "TMT_DOCUMENT_ATTRIBUTES");
    	tableTypeByContext.put(ContextUtil.POOL_DOCUMENT_CONTEXT, new Integer(14));
    	
    	// valid modules
    	VALID_ATTRIBUTE_ROOT_LIST.add(AssetModule.class);
    	VALID_ATTRIBUTE_ROOT_LIST.add(ClientModule.class);
    	VALID_ATTRIBUTE_ROOT_LIST.add(DocumentModule.class);
    	VALID_ATTRIBUTE_ROOT_LIST.add(PoolModule.class);
    	VALID_ATTRIBUTE_ROOT_LIST.add(ScopeManagementModule.class);
    	VALID_ATTRIBUTE_ROOT_LIST.add(WbsModule.class);
    }
    
    public String getTableName(String contextName)
    {
        return (String) tableNameByContext.get(contextName);
    }

    public Integer getTableType(String contextName)
    {
        return (Integer) tableTypeByContext.get(contextName);
    }
    
    /**
     * Returns the root of the structure.
     * 
     * @param attribute
     * @return
     */
    public RPMObject getAttributeRoot(GenericAttribute attribute)
    {
        RPMObject parent = attribute.getParent();
        while (parent != null && parent.getParent() != null)
            parent = parent.getParent();
        
        return parent;
    }
    
    public String getContextNameFromModule(GenericAttribute attribute)
    {
        RPMObject root = getAttributeRoot(attribute);
        String contextName = root.getContextName();
        if (contextName == null)
        {
            contextName = getContextNameFromModule(root);
        }
        return contextName;
    }
    
    public String getContextNameFromModule(RPMObject root)
    {
        if (root == null)
            return null;
        if (root instanceof WbsModule)
            return ContextUtil.WBS_CONTEXT;
        if (root instanceof AssetModule)
            return ContextUtil.ASSET_CONTEXT;
        if (root instanceof ClientModule)
            return ContextUtil.CLIENT_CONTEXT;
        if (root instanceof DocumentModule)
            return ContextUtil.UMBRELLA_DOCUMENT_CONTEXT;
        if (root instanceof PoolModule)
            return ContextUtil.POOL_CONTEXT;
        if (root instanceof ScopeManagementModule)
            return ContextUtil.SCOPE_CONTEXT;
        return null;
    }
    
    public AbstractModule getModuleFromContextName(String contextName)
    {
        AbstractModule module = null;
        if (contextName.equals(ContextUtil.WBS_CONTEXT))
            module = new WbsModule();
        if (contextName.equals(ContextUtil.ASSET_CONTEXT))
            module = new AssetModule();
        if (contextName.equals(ContextUtil.CLIENT_CONTEXT))
            module = new ClientModule();
        if (contextName.equals(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT))
            module = new DocumentModule();
        if (contextName.equals(ContextUtil.POOL_CONTEXT))
            module = new PoolModule();
        if (contextName.equals(ContextUtil.SCOPE_CONTEXT))
            module = new ScopeManagementModule();

        return module;
    }
    
    public List getValidAttributeRoot()
    {
        return VALID_ATTRIBUTE_ROOT_LIST;
    }
}
