/*
 * Created on Aug 10, 2005
 *
 */

package com.ibm.rpm.applicationadministration.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.WorkInterval;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.StringUtil;

/**
 * @author Genevieve
 *  
 */
public class WorkIntervalUtil
{   
    private static SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm");

    public WorkIntervalUtil()
    {
        super();
    }

    public static WorkInterval createWorkInterval(String from, String to)
            throws RPMException
    {

        WorkInterval workInterval = null;

        if (from == null)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(WorkInterval.class) + " from"
            };
            throw new RPMException(ErrorCodes.MANDATORY, errorParams, WorkInterval.class.getName(), "from");
        }
        
        if (to == null)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(WorkInterval.class) + " to"
            };
            throw new RPMException(ErrorCodes.MANDATORY, errorParams, WorkInterval.class.getName(), "to");
        }

        workInterval = new WorkInterval();

        workInterval.setFrom(from);
        workInterval.setTo(to);

        return workInterval;

    }

    public static void validate(WorkInterval[] intervals) throws RPMException
    {

        if (intervals != null)
        {
            WorkInterval interval = null;
            StringBuffer msg = new StringBuffer();

            for (int i = 0; i < intervals.length; i++)
            {
                interval = intervals[i];
                if (interval != null)
                {
                    if (!isValid(interval))
                    {
                        msg.append("\"");
                        msg.append(interval.getFrom());
                        msg.append(",");
                        msg.append(interval.getTo());
                        msg.append("\" ");
                    }
                }
            }
            if (msg.length() > 0)
            {
                String[] errorParams =
                {
                    StringUtil.getShortClassName(WorkInterval.class),
                    msg.toString()
                };
                throw new RPMException(
                    com.ibm.rpm.applicationadministration.constants.ErrorCodes.
                    INVALID_WORKINTERVAL, errorParams, WorkInterval.class.getName());
            }

        }
    }

    private static boolean isValid(WorkInterval interval)
    {

        boolean valid = false;

        int[] from = getHoursAndMinutes(interval.getFrom());
        int[] to = getHoursAndMinutes(interval.getTo());

        try
        {

            if (from != null && to != null)
            {

                if (validateFrom(from) && validateTo(to))
                {
                    Calendar fromCal = DateUtil.getCalendarInstance();
                    fromCal.setTime(dateFormat.parse(interval.getFrom()));

                    Calendar toCal = DateUtil.getCalendarInstance();
                    toCal.setTime(dateFormat.parse(interval.getTo()));

                    //From < To

                    if (fromCal.getTimeInMillis() < toCal.getTimeInMillis())
                    {
                        valid = true;
                    }
                }

            }

        }
        catch (ParseException e)
        {
            //do nothing
        }

        return valid;
    }



    private static int[] getHoursAndMinutes(String time){
        
        int[] hoursAndMinutes = null;
        
        if (time != null)
        {

            int index = 0;
            index = time.indexOf(":");

            if (index > 0)
            {

                int hours = Integer.parseInt(time.substring(0, index));
                int minutes = Integer.parseInt(time.substring(index + 1, time
                    .length()));
                
                if (hours >= 0 && hours <= 24 && minutes >= 0 && minutes < 60 && (minutes%15 == 0))
                {
                    hoursAndMinutes = new int[]
                    {
                            hours, minutes
                    };
                }

            }
        }
        
        return hoursAndMinutes;
        
    }
    
    private static boolean validateFrom(int[] time)
    {

        boolean isValid = false;

        int hours = time[0];
        int minutes = time[1];

        //From between 0:15 and 23:45

        if ((hours == 0 && minutes > 0) || (hours > 0 && hours < 24))
        {
            isValid = true;
        }

        return isValid;
    }

    private static boolean validateTo(int[] time)
    {

        boolean isValid = false;

        int hours = time[0];
        int minutes = time[1];

        //To between 0:30 and 24:00

        if ((hours == 0 && minutes > 15) || (hours == 24 && minutes == 0)
                || (hours > 0 && hours < 24))
        {
            isValid = true;
        }

        return isValid;
    } 
    
    public static String formatTime(String time)
    {

        String formatedTime = null;

        if (time != null)
        {

            int[] hhmm = getHoursAndMinutes(time);

            if (hhmm != null)
            {

                int hours = hhmm[0];
                int minutes = hhmm[1];

                formatedTime = (hours >= 10 ? "" + hours : "0" + hours) + ":"
                        + (minutes < 10 ? "0" + minutes : "" + minutes);
            }
        }

        return formatedTime;

    }

    
    

}
