/*
 * Created on 26/09/2005
 *
 */
package com.ibm.rpm.asset.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.asset.constants.ValidationConstants;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.document.checkpoints.DocumentElementCheckpoint;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

/**
 * @author rosman, mmatwiy
 */
public class AssetCheckpoint extends GenericAssetCheckpoint
{
    private static final List ASSET_PARENTS = new ArrayList();

    private static final List ASSET_CHILDREN_DOCUMENT_LIST = new ArrayList();

    static
    {
        ASSET_PARENTS.add(AssetFolder.class);
        ASSET_PARENTS.add(Asset.class);

        ASSET_CHILDREN_DOCUMENT_LIST.add(DocumentFolder.class);
        ASSET_CHILDREN_DOCUMENT_LIST.add(DelDoc.class);
        ASSET_CHILDREN_DOCUMENT_LIST.add(Document.class);
        ASSET_CHILDREN_DOCUMENT_LIST.add(Software.class);
        ASSET_CHILDREN_DOCUMENT_LIST.add(UrlDocument.class);
        ASSET_CHILDREN_DOCUMENT_LIST.add(WPDoc.class);
    }

    protected static AssetCheckpoint instance = new AssetCheckpoint();

    public static AssetCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, msgCtx, true, ASSET_PARENTS);

        Asset asset = (Asset) rpmObject;
        AssetScope scope = (AssetScope) rpmScope;

        if (asset.testAssetLifeModified())
        {
            GenericValidator.validateRange(asset,
                ValidationConstants.ASSET_LIFE_FIELD, asset.getAssetLife(),
                ValidationConstants.ASSET_LIFE_MIN,
                ValidationConstants.ASSET_LIFE_MAX, msgCtx);
        }

        if (asset.testBarcodeNumberModified())
        {
            GenericValidator.validateMaxLength(asset,
                ValidationConstants.BARCODE_NUMBER_FIELD, asset
                    .getBarcodeNumber(),
                ValidationConstants.BARCODE_NUMBER_MAX, msgCtx);
        }

        if (asset.testContactModified())
        {
            GenericValidator.validateMaxLength(asset,
                ValidationConstants.CONTACT_FIELD, asset.getContact(),
                ValidationConstants.CONTACT_MAX, msgCtx);
        }

        if (asset.testContactEmailModified())
        {
            GenericValidator.validateMaxLength(asset,
                ValidationConstants.CONTACT_EMAIL_FIELD, asset
                    .getContactEmail(), ValidationConstants.CONTACT_EMAIL_MAX,
                msgCtx);
        }

        if (asset.testContactPhoneModified())
        {
            GenericValidator.validateMaxLength(asset,
                ValidationConstants.CONTACT_PHONE_FIELD, asset
                    .getContactPhone(), ValidationConstants.CONTACT_PHONE_MAX,
                msgCtx);
        }

        if (asset.testDepreciationMethodModified())
        {
            GenericValidator.validateRange(asset,
                ValidationConstants.DEPRECIATION_METHOD_FIELD, asset
                    .getDepreciationMethod(),
                ValidationConstants.DEPRECIATION_METHOD_MIN,
                ValidationConstants.DEPRECIATION_METHOD_MAX, msgCtx);
        }

        if (asset.testInitialCostModified())
        {
            GenericValidator.validateRange(asset,
                ValidationConstants.INITIAL_COST_FIELD, asset.getInitialCost(),
                ValidationConstants.INITIAL_COST_MIN,
                ValidationConstants.INITIAL_COST_MAX, msgCtx);
        }

        if (asset.testModelModified())
        {
            GenericValidator.validateMaxLength(asset,
                ValidationConstants.MODEL_FIELD, asset.getModel(),
                ValidationConstants.MODEL_MAX, msgCtx);
        }

        if (asset.testModelNumberModified())
        {
            GenericValidator.validateMaxLength(asset,
                ValidationConstants.MODEL_NUMBER_FIELD, asset.getModelNumber(),
                ValidationConstants.MODEL_NUMBER_MAX, msgCtx);
        }

        if (asset.testQuantityModified())
        {
            GenericValidator.validateRange(asset,
                ValidationConstants.ASSET_QUANTITY_FIELD, asset.getQuantity(),
                ValidationConstants.ASSET_QUANTITY_MIN,
                ValidationConstants.ASSET_QUANTITY_MAX, msgCtx);
        }

        if (asset.testSerialNumberModified())
        {
            GenericValidator.validateMaxLength(asset,
                ValidationConstants.SERIAL_NUMBER_FIELD, asset
                    .getSerialNumber(), ValidationConstants.SERIAL_NUMBER_MAX,
                msgCtx);
        }

        if (scope != null && scope.getDocumentFolder() != null)
        {
            // DocumentElements validation
            DocumentElementCheckpoint.validateDocumentElements(asset,
                new DocumentElement[]
                {
                    asset.getDocumentFolder()
                }, ASSET_CHILDREN_DOCUMENT_LIST, msgCtx);
        }
        
        if(asset.getID() != null)
        {
        	validateUpdate(asset,scope,msgCtx);
        }
    }
    
    private void validateUpdate(Asset asset, AssetScope assetScope, MessageContext msgCtx)
    {
    	if(asset.testTrackedModified())
        {
        	GenericValidator.validateMandatory(asset, 
        			ValidationConstants.TRACKED_FIELD, asset.getTracked(), msgCtx);
        }
    }
}