/*
 * Created on Oct 25, 2005
 *
 */
package com.ibm.rpm.asset.managers;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.AssetCategory;
import com.ibm.rpm.applicationadministration.containers.AssetRTF;
import com.ibm.rpm.applicationadministration.containers.AssetState;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.ChargeType;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.managers.AttributeManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.scope.AssetModuleScope;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.managers.CustomFieldManager;
import com.ibm.rpm.framework.AbstractModuleManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.managers.ScorecardManager;

/**
 * @author karim
 *  
 */
public class AssetModuleManager extends AbstractModuleManager
{

    private static Log logger = LogFactory.getLog(AssetModuleManager.class);

    protected Log getLogger()
    {
        return logger;
    }

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(AssetModule.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    AssetModuleManager.class);
        }
    }

    {
        localContextName = ContextUtil.ASSET_CONTEXT;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = new AssetModule();
        return object;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof AttributeManager)
        {
            if (localContextName.equalsIgnoreCase(contextName))
            {
                joinCondition = new JoinCondition();
            }
            else
            {//context not linked to this manager
                logger
                        .error("AssetModuleManager.getJoinCondition: contextName "
                                + contextName
                                + " is not linked with this manager.");
            }
        }
        else if (manager instanceof CustomFieldManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setCondition(CustomFieldManager
                    .createJoinCondition(localContextName));
        }
        else if (manager instanceof AssetManager)
        {
                joinCondition = new JoinCondition();
        }
        else if (manager instanceof ScorecardManager)
        {
                joinCondition = new JoinCondition();
        }

        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected void loadChildren(AbstractModule abstractModule,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        AssetModule module = (AssetModule) abstractModule;
        AssetModuleScope scope = (AssetModuleScope) rpmScope;
        
    	DatafieldManager datafieldManager = (DatafieldManager)getManagerInstance(DatafieldCategory.class);

        //loadAssetFolders
        if (scope.getAssetFolders() != null)
        {
        	String extraWhere = " TYPE_ID = "+ AssetFolder.TYPE_ID;
        	AssetFolder[] assetFolders = (AssetFolder[]) loadElements(module, module.getAssetFolders(),
        			scope.getAssetFolders(),AssetFolder.class, 
					null,extraWhere, messageContext, isDelta );        
	        module.setAssetFolders(assetFolders);
        }

        //loadAttributes
        if (scope.getAttributeCategories() != null)
        {
            AttributeManager attributeManager = (AttributeManager) getManagerInstance(AttributeCategory.class);

            AttributeCategory[] attributeCategory = attributeManager
                .loadAttributeCategories(module, module.getAttributeCategories(), scope
                    .getAttributeCategories(), this, localContextName,
                    messageContext, isDelta);
            module.setAttributeCategories(attributeCategory);
        }
        
        //loadChargeTypes
        if(scope.isChargeTypes())
        {        	
        	ChargeType[] chargeTypes = (ChargeType[]) datafieldManager
                .loadDatafields(module, module.getChargeTypes(), ChargeType.class,
                    ChargeType.TYPE_ID, this, localContextName,
                    messageContext, isDelta);
            module.setChargeTypes(chargeTypes);
        }
        
        //loadAssetCategories
        if (scope.isAssetCategories())
        {
            AssetCategory[] assetCategories = (AssetCategory[]) datafieldManager
                .loadDatafields(module, module.getAssetCategories(),
                    AssetCategory.class, AssetCategory.TYPE_ID,
                    this, localContextName, messageContext, isDelta);
            module.setAssetCategories(assetCategories);
        }

        //loadAssetStates
        if(scope.isAssetStates())
        {        	
        	AssetState[] assetStates = (AssetState[]) datafieldManager
                .loadDatafields(module, module.getAssetStates(), AssetState.class,
                    AssetState.TYPE_ID, this, localContextName,
                    messageContext, isDelta);
            module.setAssetStates(assetStates);
        }
        
        //loadAssetRtfs
        if(scope.isAssetRtfs())
        {        	
        	AssetRTF[] assetRTFs = (AssetRTF[]) datafieldManager.loadDatafields(module, 
                module.getAssetRtfs(), AssetRTF.class, AssetRTF.TYPE_ID,
                this, localContextName, messageContext, isDelta);
            module.setAssetRtfs(assetRTFs);
        }
        
        //loadCustomFields
        if (scope.getCustomFieldCategories() != null)
        {
            CustomFieldManager manager = (CustomFieldManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            CustomFieldCategory.class);
            CustomFieldCategory[] categories = manager
                    .loadCustomFieldCategories(localContextName,
                            scope.getCustomFieldCategories(), messageContext);
            module.setCustomFieldCategories(categories);
        }
        
        //loadScorecardFolders
        if (scope.getScorecardFolders() != null)
        {
        	String extraWhere = " TYPE_ID = "+ ScorecardFolder.TYPE_ID;
	        ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) loadElements(
                    module, module.getScorecardFolders(), scope
                            .getScorecardFolders(), ScorecardFolder.class,
                    null, extraWhere, messageContext, isDelta);        
            module.setScorecardFolders(scorecardFolders);
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        AssetModuleScope scope = (AssetModuleScope) rpmScope;
        AssetModule module = (AssetModule) rpmObject;
        module.setContextName(localContextName);

        if(scope!=null){
            
            //AssetFolders
            if(scope.getAssetFolders() != null)
            {
            	AssetFolder[] assetFolders = (AssetFolder[]) saveElements(
            			module, module.getAssetFolders(),
    					scope.getAssetFolders(), messageContext,
    					AssetFolder.class );
            	module.setAssetFolders(assetFolders);
            }

            //AttributeCategories
            if (scope.getAttributeCategories() != null)
            {
                AttributeCategory[] attributeCategory = (AttributeCategory[]) saveElements(
                        module, module.getAttributeCategories(), scope
                                .getAttributeCategories(), messageContext,
                        AttributeCategory.class); 
                module.setAttributeCategories(attributeCategory);
            }

            //CustomFields
            if (scope.getCustomFieldCategories() != null)
            {
                CustomFieldCategory[] customFieldCategories = (CustomFieldCategory[]) saveElements(
                        module, module.getCustomFieldCategories(), scope
                                .getCustomFieldCategories(), messageContext,
                        CustomFieldCategory.class);
                module.setCustomFieldCategories(customFieldCategories);
            }
            
            //ScorecardFolders
            if(scope.getScorecardFolders() != null)
            {
            	ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) saveElements(
            			module, module.getScorecardFolders(),
    					scope.getScorecardFolders(), messageContext,
    					ScorecardFolder.class );
            	module.setScorecardFolders(scorecardFolders);
            }            
        }

        return module;
    }
}
