package com.ibm.rpm.asset.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.asset.containers.AssetResourceAssignment;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;

public class AssetResourcesManager extends AbstractRPMObjectManager{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static{
        // Add the complete class name of all the containers that could be loaded by this managers.
        CONTAINERS.add(AssetResourceAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                AssetResourcesManager.class);
        }
    }
    /**
     * Column ASSET_RES_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ASSET_RES_ID = 1;
    public static final int TYPE_ASSET_RES_ID = Types.CHAR;
    public static final String NAME_ASSET_RES_ID = "ASSET_RESOURCES.ASSET_RES_ID";
    public static final String PROPERTY_ASSET_RES_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ASSET_RES_ID,NAME_ASSET_RES_ID);}


    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 2;
    public static final int TYPE_RESOURCE_ID = Types.CHAR;
    public static final String NAME_RESOURCE_ID = "ASSET_RESOURCES.RESOURCE_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_RESOURCE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID,NAME_RESOURCE_ID);}

    /**
     * Column ASSET_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ASSET_ID = 3;
    public static final int TYPE_ASSET_ID = Types.CHAR;
    public static final String NAME_ASSET_ID = "ASSET_RESOURCES.ASSET_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ASSET_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSET_ID,NAME_ASSET_ID);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 4;
    public static final int TYPE_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_DESCRIPTION = "ASSET_RESOURCES.DESCRIPTION";
    //public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION,NAME_DESCRIPTION);}


    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 5;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "ASSET_RESOURCES.REC_STATUS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 6;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "ASSET_RESOURCES.REC_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 7;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "ASSET_RESOURCES.REC_USER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    private static final String TABLE_NAME = "ASSET_RESOURCES";

    /**
     * Create an array of type string containing all the fields of the ASSET_RESOURCES table.
     */
    private static final String[] FIELD_NAMES = 
    {
         NAME_ASSET_RES_ID
        ,NAME_RESOURCE_ID
        ,NAME_ASSET_ID
        ,NAME_DESCRIPTION
        ,NAME_REC_STATUS
        ,NAME_REC_DATETIME
        ,NAME_REC_USER
    };

    private static final String ALL_FIELDS;
    static{
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    }

    /**
     * Creates a new AssetResourcesManager instance.
     * @return AssetResourcesManager
     */   
    public AssetResourcesManager(){
        super();
    }

    public String getTableName() {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames(){
        return FIELD_NAMES;
    }

    protected String getAllFields() {
        return ALL_FIELDS;
    }  

    public String getPrimaryKey() {
        return NAME_ASSET_RES_ID;
    }  

    protected Integer getContainerType(ResultSet resultSet) throws SQLException {

    	//If container type doesn't apply to this manager
    	return new Integer(0);
    }  

    protected HashMap getPropertiesMap() {
        return FIELDPROPERTYMAP;
    }  

    protected HashSet getValidContainerNames() {
        return CONTAINERS;
    }  

    protected RPMObject createContainer(int typeId) throws RPMException {

        RPMObject object = new AssetResourceAssignment();

         return object;
     }
    
    /**
     * Create a record for Asset Resource in DB and set its ID.
     */


    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException {

    	AssetResourceAssignment pObject = (AssetResourceAssignment)rpmObject;

        // Insert the record
        pObject.setID(SP_A_ASSET_RES(pObject, messageContext));
    } // createObject

    /**
     * decode a row of Asset Resource Assignment. this includes only the id
     */


    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException {
    	AssetResourceAssignment pObject = (AssetResourceAssignment)rpmObject;
		
        pObject.setID(rs.getString(ID_ASSET_RES_ID));
        
        return pObject;
    }// decodeRow

    /**
     * delete an Asset Resources Assignment record
     */

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException {

        	if(rpmObject != null)
            {
        		SP_UA_ASSET_RES((AssetResourceAssignment) rpmObject, 
                                    messageContext);
            }
        } // delete

    /**
     * Returns the foreign key sql conditions of the asset that exists in the
     * Asset Resources Assignment table
     */

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException {
        if (!(manager instanceof AssetManager))
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);

        JoinCondition joinCondition = new JoinCondition();
        joinCondition.setTableName(getTableName());

        //where clause of the join condition
        StringBuffer condition = new StringBuffer();
        condition.append(NAME_ASSET_ID);
        condition.append(" = ?");
        if (extraWhere != null && extraWhere.length() > 0)
        {
            condition.append(" AND ");
            condition.append(extraWhere);
        }
        joinCondition.setCondition(condition.toString());

        return joinCondition;
    } // getForeignKeyCondition    

    /**
     * Return JoinCondition object with condition input parameter
     */

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName, String contextName) throws RPMException 
            {

                JoinCondition joinCondition = new JoinCondition();
                if (manager instanceof AssetManager)
                {
                    // for XPath queries like AssetResourceAssignment[<condition>]/Asset
                    StringBuffer where = new StringBuffer();
                    where.append(AssetManager.NAME_ELEMENT_ID).append(" IN (");
                    where.append(" SELECT ").append(NAME_ASSET_ID);
                    where.append(" FROM ").append(getTableName());
                    if (condition != null)
                    {
                        where.append(" WHERE ").append(condition);
                    }
                    joinCondition.setCondition(where.toString());
                }
                else if (manager == null)

                {
                    // joinCondition = new JoinCondition();
                    joinCondition.setTableName(getTableName());
                    
                    if (condition != null)
                    {
                        joinCondition.setCondition(condition);
                    }
            
                }

                else if (manager instanceof ResourceManager)
                {
                    // for XPath queries like /AssetResourceAssignment[<condition>]/Resource
                    StringBuffer where = new StringBuffer();
                    where.append(ResourceManager.NAME_RESOURCE_ID).append(" IN (");
                    where.append(" SELECT ").append(NAME_RESOURCE_ID);
                    where.append(" FROM ").append(getTableName());
                    if (condition != null)
                    {
                        where.append(" WHERE ").append(condition);
                    }
                    joinCondition.setCondition(where.toString());
                }
                
                else
        	    {
                    ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                            ExceptionUtil.JOIN);
                }
            
                return joinCondition;
            } // getJoinCondition

    
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
			{

	    		JoinCondition joinCondition = new JoinCondition();
	    		if (manager instanceof AssetManager)
	    		{
	    			// for XPath queries like /Asset[AssetResourceAssignment[<condition>]]
	    			StringBuffer where = new StringBuffer();
	    			where.append(AssetManager.NAME_ELEMENT_ID).append(" IN (");
	    			where.append(" SELECT ").append(NAME_ASSET_ID);
	    			where.append(" FROM ").append(getTableName());
	    			if (condition != null)
	    			{
	    				where.append(" WHERE ").append(condition);
	    			}
	    			joinCondition.setCondition(where.toString());
	    		}
	    		else if (manager instanceof ResourceManager)
	    		{
	    			// for XPath queries like /Resource[AssetResourceAssignment[<condition>]]
	    			StringBuffer where = new StringBuffer();
	    			where.append(ResourceManager.NAME_RESOURCE_ID).append(" IN (");
	    			where.append(" SELECT ").append(NAME_RESOURCE_ID);
	    			where.append(" FROM ").append(getTableName());
	    			if (condition != null)
	    			{
	    				where.append(" WHERE ").append(condition);
	    			}
	    			joinCondition.setCondition(where.toString());
	    		}
	    		else
	    		{
	    			ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                        ExceptionUtil.SUB_TABLE_JOIN);
	    		}

	    		return joinCondition;
			} // getSubTableJoinCondition

	/**
	 * The decoded related objects is the parent asset and Resource
	 * 
	 */
	protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
        FieldValueMap relatedFields, int index) throws RPMException,
        SQLException
	{
		String parentId = resultSet.getString(ID_ASSET_ID);
		relatedFields.put(index, ID_ASSET_ID, parentId);
		String resourceId = resultSet.getString(ID_RESOURCE_ID);
		relatedFields.put(index, ID_RESOURCE_ID, resourceId);

		return relatedFields;
	} // decodeRelatedFields

	/**
	 * There are two related objects: parent asset & Assigned Resource
	 */
	protected RPMObject loadRelatedObjects(RPMObject rpmObject,
        RPMObjectScope rpmScope, MessageContext messageContext,
        FieldValueMap relatedFields, int index, boolean isDelta)
        throws RPMException, SQLException, ParseException
		{

			//Load Assigned Resource. Parent will be setted by the AssetManager
			AssetResourceAssignment object = (AssetResourceAssignment) rpmObject;

			//ID of Related organizaion
			String resourceId = (String) relatedFields.get(index, ID_RESOURCE_ID);

			if ( (resourceId != null))
			{
				// Load the Resource Manager
				ResourceManager resourceManager = (ResourceManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(Resource.class.getName());
				Resource resource = new Resource();
				// Set the Organizational ID
				resource.setID(resourceId);

				// Load the Organizational record
				resource = (Resource) resourceManager.loadByPrimaryKey(resource,
                null, messageContext, isDelta);
				if (isDelta)
				{
					object.deltaResource(resource);
				}
				else
				{
					object.setResource(resource);
				}
			}// if ( (resourceId != null))
			if ((resourceId == null) )
			{
	            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.RESOURCE_ID);
			} // if ((resourceId == null) )

			return rpmObject;
		} // loadRelatedObjects
       
    /**
     * updateObject - Nothing to update
     */

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException {
        // nothing to update      
        return rpmObject;

    }

    /**
     * Inserts an Asset Resource Assignment record
     * 
     * @param assignment -
     *            AssetResourceAssignment record to save
     * @param messageContext -
     *            context
     * @return a String that represents the ID of the new created AssetResourceAssignment
     * @throws SQLException
     * @throws RPMException
     */

    private String SP_A_ASSET_RES(AssetResourceAssignment assignment, MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_A_ASSET_RES";
        Object args[] = new Object[3];

        args[0] = assignment.getParent().getID();
        args[1] = assignment.getResource().getID();
        args[2] = getUser(messageContext).getID();
        
        return executeProcedure(messageContext, name, args);
    } // SP_A_ASSET_RES  
    
    /**
     * Delete a Asset Resource Assignment record
     * 
     * @param assignment -
     *            AssetResourceAssignment record to delete
     * @param messageContext -
     *            context
     * @throws SQLException
     * @throws RPMException
     */    
    private void SP_UA_ASSET_RES(AssetResourceAssignment assignment, MessageContext messageContext)throws RPMException, SQLException
    {
        String name = "SP_UA_ASSET_RES";
        Object args[] = new Object[3];
        if(assignment.getParent() != null){
            args[0] = assignment.getParent().getID();
        }
        args[1] = assignment.getID();
        args[2] = getUser(messageContext).getID();
        
        executeProcedure(messageContext, name, args);
    } // SP_UA_ASSET_RES   
}
