// SLegault: I replaced org.apache.log4j.Logger for org.apache.commons.logging.Log.
// SLegault: I added the second argument in most call to logger.error to log the cause. 
// SLegault: I replaced one loadProperties call by a clone. 
// It allows us to use setProperties. See below.

// Design note: This class depends on the following files.
// com.ibm.rpm.AuthController.java
// com.ibm.rpm.auth.AuthControllerException.java
// com.ibm.rpm.comm.ERRORS.java
// com.ibm.rpm.auth.jndi.JNDIController.properties

// SLegault: Please, enventually clean up this code 
// or replace with IBM's standard cwa.jar. 
// See http://tst2bluepages.mkm.can.ibm.com

package com.ibm.rpm.authentication;

import java.util.Properties;

import javax.naming.AuthenticationException;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

// import com.ibm.rpm.auth.AuthControllerException;
// import com.ibm.rpm.comm.ERRORS;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * 
 * This class is responsible for the actual authentication with the LDAP server
 * 
 */
public class LdapAuthenticationController
{

    private static final String LDAPSEARCHCONTEXT = "ldapsearchcontext";

    private static final String LDAPUSERIDATTR = "ldapuseridattr";

    private static final String LDAPUSERAUTHDN = "ldapuserauthdn";

    /**
     * Properties object used to store properies relavent to this object.
     */
    private Properties properties = new Properties();

    private RpmOptions currentOptions;

    private static Log logger = LogFactory
        .getLog(LdapAuthenticationController.class);

    /**
     * 
     * 
     */
    public LdapAuthenticationController(RpmOptions options)
    {
        currentOptions = options;
        String ldapConfiguration = currentOptions.getLdapConfiguration();
        parseSingleStringConfiguration(ldapConfiguration, properties);
    }

    /**
     * This method expects the parameters to be contained in a single string and
     * each parameter is in the format : parametername="parametervalue"
     * 
     * @param configurationString
     */
    public static void parseSingleStringConfiguration(
            String configurationString, Properties propertiesToFill)
    {
        String parameterName;
        String parameterValue;

        int equalSignPosition = 0;
        int valueDelimiterStartPosition = 0;
        int valueDelimiterEndPosition = 0;

        equalSignPosition = configurationString.indexOf('=');
        if (equalSignPosition != -1)
        {
            parameterName = configurationString.substring(0, equalSignPosition);
            parameterName = cleanParameterName(parameterName);
        }
        else
        {
            return;
        }

        while (equalSignPosition != -1)
        {
            valueDelimiterStartPosition = configurationString.indexOf('\"',
                equalSignPosition);
            valueDelimiterEndPosition = configurationString.indexOf('\"',
                valueDelimiterStartPosition + 1);

            if (valueDelimiterStartPosition == -1)
            {
                throw new IllegalArgumentException(
                    "Missing opening quote for argument : " + parameterName);
            }

            if (valueDelimiterEndPosition == -1
                    || valueDelimiterStartPosition == valueDelimiterEndPosition)
            {
                throw new IllegalArgumentException(
                    "Missing closing quote for argument : " + parameterName);
            }

            parameterValue = configurationString.substring(
                valueDelimiterStartPosition + 1, valueDelimiterEndPosition);

            propertiesToFill.put(parameterName, parameterValue);

            // set the next parameter name
            equalSignPosition = configurationString.indexOf('=',
                valueDelimiterEndPosition);

            if (equalSignPosition == -1)
            {
                // return;
            }
            else
            {
                parameterName = configurationString.substring(
                    valueDelimiterEndPosition + 1, equalSignPosition);
                parameterName = StringUtils.deleteWhitespace(parameterName);
            }
        }
        
        // legacy configuration variables used a different prefix, map those old 
        // variable names to the new configuration list
        convertLegacyPropertyName(propertiesToFill, LDAPSEARCHCONTEXT);
        convertLegacyPropertyName(propertiesToFill, LDAPUSERAUTHDN);
        convertLegacyPropertyName(propertiesToFill, LDAPUSERIDATTR);
    }

    private static void convertLegacyPropertyName(Properties propertiesToFill, String propertyName)
    {
        String legacyPropertyName = "com.ibm.rpm.auth.jndi." + propertyName;
        if( ! isPropertyValueValid(propertiesToFill, propertyName))
        {
            if( isPropertyValueValid(propertiesToFill, legacyPropertyName))
            {
                Object propertyValue = propertiesToFill.get(legacyPropertyName);
                propertiesToFill.put(propertyName, propertyValue);
            }
        }
    }

    private static String cleanParameterName(String parameterName)
    {
        parameterName = parameterName.replace('\n', ' ');
        parameterName = parameterName.replace('\t', ' ');
        parameterName = parameterName.replace('\r', ' ');
        parameterName = StringUtils.deleteWhitespace(parameterName);
        return parameterName;
    }

    /**
     * 
     * The authenticate method for JNDI makes a connection to the server in
     * order to
     * 
     * authenticate the said user. The result of this operation will give a
     * unique
     * 
     * user identifier to by used by rpmffice.
     * 
     * @param userid -
     *            The userid of the user to authenticate
     * 
     * @param passwd -
     *            The password of the user to authenticate
     * 
     * @return java.lang.String
     * 
     * @throws com.ibm.rpm.auth.AuthControllerException
     * 
     * @roseuid 4043EDE20365
     * 
     */
    public String authenticate(String userid, String passwd)
            throws RPMException
    {

        // this variable contains a set of properties used to perform
        // LDAP authentication with the proper user name and password
        Properties authenticationProperties = (Properties) properties.clone();

        validateAuthenticationProperties(authenticationProperties);

        DirContext ldapContext = null;
        try
        {
            try
            {
                ldapContext = new InitialDirContext(properties);
            }
            catch (NamingException ex)
            {
                // internal configuration error
                throw new RPMException(
                    ErrorCodes.EXTERNAL_AUTHENTICATION_CONFIGURATION_ERROR,
                    new String[]
                    {
                        ex.getMessage()
                    }, ex);
            }

            logger.debug("Trying LDAP lookup using context: "
                    + getLdapProperty(LDAPSEARCHCONTEXT) + " and "
                    + getLdapProperty(LDAPUSERIDATTR) + "=" + userid);
            NamingEnumeration results = null;

            results = findLdapSearchContext(userid, ldapContext, results);

            SearchResult sr = null;
            Attribute attr = null;
            try
            {
                if (results != null && results.hasMore())
                {
                    sr = (SearchResult) results.next();
                    Attributes attributes = null;
                    attributes = sr.getAttributes();
                    String attprop = getLdapProperty(LDAPUSERAUTHDN);
                    if (attprop != null)
                    {
                        attr = attributes.get(getLdapProperty(LDAPUSERAUTHDN));
                    }

                    if (attr == null)
                    {
                        logger.error("Cannot find the: " + LDAPUSERAUTHDN
                                + " attibute. Trying getName()");
                    }

                    fillDnInAuthenticationProperties(userid, authenticationProperties,
                        sr, attr);
                }
                else
                {
                    // User doesn't exist
                    throw new RPMException(ErrorCodes.FAILED_LOGIN);
                }
            }
            catch (NamingException ex3)
            {
                // Failed login
                throw new RPMException(ErrorCodes.FAILED_LOGIN, new String[]
                {}, ex3);
            }

        }
        finally
        {
            closeLdapContext(ldapContext);
        }

        authenticateUserWithPassword(userid, passwd, authenticationProperties);

        return userid;
    }

    private void validateAuthenticationProperties(
            Properties authenticationProperties) throws RPMException
    {
        validateValidPropertyValue(authenticationProperties, LDAPSEARCHCONTEXT);

        // We need either the "USER ID ATTR" or the "USER AUTH DN" to be specified
        if (!isPropertyValueValid(authenticationProperties, LDAPUSERIDATTR)
                && !isPropertyValueValid(authenticationProperties,
                    LDAPUSERAUTHDN))
        {
            failWithMissingProperty(LDAPUSERIDATTR + " or " + LDAPUSERAUTHDN);
        }
        validateValidPropertyValue(authenticationProperties,
            Context.INITIAL_CONTEXT_FACTORY);
    }

    private void validateValidPropertyValue(
            Properties authenticationProperties, String propertyName) throws RPMException
    {
        if( ! isPropertyValueValid(authenticationProperties, propertyName) )
        {
            failWithMissingProperty(propertyName);
        }
    }

    private static boolean isPropertyValueValid(
            Properties authenticationProperties, String propertyName)
    {
        Object currentValue = authenticationProperties.get(propertyName);
        if( currentValue instanceof String )
        {
            String currentString = (String) currentValue;
            if( currentString.length() == 0 )
            {
                return false;
            }
        }
        else
        {
            return false;
        }
        return true;
    }
    
    private void failWithMissingProperty(String propertyName) throws RPMException
    {
        throw new RPMException(
            ErrorCodes.EXTERNAL_AUTHENTICATION_CONFIGURATION_ERROR,
            new String[]
            {
                "Missing Property " + propertyName
            });
    }

    private void fillDnInAuthenticationProperties(String userid,
            Properties authprops, SearchResult sr, Attribute attr)
            throws RPMException
    {
        String dn = null;
        try
        {
            String ctx = getLdapProperty(LDAPSEARCHCONTEXT);
            if (attr != null)
            {
                dn = (String) attr.get();
            }
            else
            {
                dn = sr.getName();
                if (!StringUtils.isEmpty(ctx))
                {
                    dn += "," + ctx;
                }
            }
            logger.debug("DN: " + dn);
            authprops.setProperty(Context.SECURITY_PRINCIPAL, dn);
        }
        catch (NamingException ex4)
        {
            // Failed login
            throw new RPMException(ErrorCodes.FAILED_LOGIN, new String[]
            {}, ex4);
        }
    }

    private void closeLdapContext(DirContext ldapContext)
    {
        try
        {
            if (ldapContext != null)
            {
                ldapContext.close();
            }
        }
        catch (Exception e)
        {
        }
    }

    private void authenticateUserWithPassword(String userid, String password,
            Properties authprops) throws RPMException
    {
        DirContext authLdapContext = null;
        try
        {
            if (password == null)
            {
                // Failed login, invalid password
                throw new RPMException(ErrorCodes.FAILED_LOGIN);
            }

            authprops.setProperty(Context.SECURITY_CREDENTIALS, password);
            authprops.setProperty(Context.SECURITY_AUTHENTICATION, "simple");

            try
            {
                authLdapContext = new InitialDirContext(authprops);
            }
            catch (NamingException ex)
            {
                if (ex instanceof AuthenticationException)
                {
                    // LDAP identified a invalid password
                    throw new RPMException(ErrorCodes.FAILED_LOGIN,
                        new String[]
                        {}, ex);
                }

                // internal configuration error
                throw new RPMException(
                    ErrorCodes.EXTERNAL_AUTHENTICATION_CONFIGURATION_ERROR,
                    new String[]
                    {
                        ex.getMessage()
                    }, ex);
            }

            // Search for objects with those matching attributes
            logger.debug("Trying LDAP lookup using context: "
                    + authprops.getProperty(LDAPSEARCHCONTEXT) + " and "
                    + authprops.getProperty(LDAPUSERIDATTR) + "=" + userid);

            NamingEnumeration results = null;
            try
            {
                results = callLdapWithUserNameAndPassword(userid,
                    authLdapContext);

                if (results == null)
                {
                    // Failed login
                    throw new RPMException(ErrorCodes.FAILED_LOGIN);
                }
            }
            catch (NamingException ex1)
            {
                // Failed login
                throw new RPMException(ErrorCodes.FAILED_LOGIN, new String[]
                {}, ex1);
            }

        }
        finally
        {
            closeLdapContext(authLdapContext);
        }
    }

    private NamingEnumeration callLdapWithUserNameAndPassword(String userid,
            DirContext authLdapContext) throws NamingException
    {
        SearchControls ctls = null;
        ctls = new SearchControls();
        ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
        ctls.setCountLimit(0);
        ctls.setTimeLimit(0);
        ctls.setReturningAttributes(new String[0]);

        NamingEnumeration results;
        results = authLdapContext.search(getLdapProperty(LDAPSEARCHCONTEXT),
            getLdapProperty(LDAPUSERIDATTR) + "=" + userid, ctls);
        return results;
    }

    private NamingEnumeration findLdapSearchContext(String userid,
            DirContext ldapContext, NamingEnumeration results)
            throws RPMException
    {
        SearchControls ctls = null;
        ctls = new SearchControls();
        ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
        ctls.setCountLimit(0);
        ctls.setTimeLimit(0);
        ctls.setReturningAttributes(new String[]
        {
            getLdapProperty(LDAPUSERAUTHDN)
        });

        try
        {
            results = ldapContext.search(getLdapProperty(LDAPSEARCHCONTEXT),
                getLdapProperty(LDAPUSERIDATTR) + "=" + userid, ctls);
        }
        catch (NamingException ex2)
        {
            // user doesn't exist, fail the login
            throw new RPMException(ErrorCodes.FAILED_LOGIN);
        }
        catch (Throwable ex2)
        {
            // internal configuration error
            throw new RPMException(
                ErrorCodes.EXTERNAL_AUTHENTICATION_CONFIGURATION_ERROR,
                new String[]
                {
                    ex2.getMessage()
                }, ex2);
        }
        return results;
    }

    /**
     * This method goes to JNDI and find tries to search for the user in the
     * LDAP directory. If the user is found it will return true.
     * 
     * @param userid
     * @return boolean true if the user exists
     */
    public boolean lookup(String userid) throws RPMException
    {
        DirContext ldapContext = null;
        SearchControls ctls = null;
        try
        {
            try
            {
                ldapContext = new InitialDirContext(properties);
            }
            catch (NamingException ex)
            {
                logger
                    .error("Unable to authenticate for lookup using current configuration");
                throw new RPMException(
                    ErrorCodes.EXTERNAL_AUTHENTICATION_CONFIGURATION_ERROR,
                    new String[]
                    {
                        ex.getMessage()
                    }, ex);
            }
            
            ctls = new SearchControls();
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            ctls.setCountLimit(0);
            ctls.setTimeLimit(0);
            ctls.setReturningAttributes(new String[0]);
            logger.debug("Trying LDAP lookup using context: "
                    + properties.getProperty(LDAPSEARCHCONTEXT) + " and "
                    + properties.getProperty(LDAPUSERIDATTR) + "=" + userid);
            NamingEnumeration results = null;
            try
            {
                results = ldapContext.search(properties
                    .getProperty(LDAPSEARCHCONTEXT), properties
                    .getProperty(LDAPUSERIDATTR)
                        + "=" + userid, ctls);
            }
            catch (NamingException ex2)
            {
                logger.error("User:" + userid + " not found due to exception.", ex2);
                return false;
            }
            try
            {
                if (results == null || !results.hasMore())
                {
                    logger.error("User:" + userid + " not found.");
                    return false;
                }
            }
            catch (NamingException ex3)
            {
                logger.error("User:" + userid
                        + " not found due to LDAP Exception while getting "
                        + properties.getProperty(LDAPUSERAUTHDN) + ".");
                throw new RPMException(ErrorCodes.USER_DOES_NOT_EXIST, null, ex3);
            }
            logger.debug("User:" + userid + " found!");
        }
        finally
        {
            ctls = null;
            try
            {
                ldapContext.close();
            }
            catch (NamingException ex5)
            {
                logger.debug("Problem closing ldap context.", ex5);
            }
            ldapContext = null;
        }
        return true;
    }    
    
    private String getLdapProperty(String propertyToFetch)
    {
        return properties.getProperty(propertyToFetch);
    }

    public void setProperties(Properties ldapProperties)
    {
        properties = (Properties) ldapProperties.clone();
    }
    

}
