package com.ibm.rpm.client.junit;

import java.lang.reflect.Method;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.Calendar;

import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.scope.WorkflowResponseScope;
import com.ibm.rpm.framework.AlreadyCheckedOutResult;
import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.PropertyArray;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectArray;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.interfaces.Application;
import com.ibm.rpm.interfaces.ApplicationLiteral;
import com.ibm.rpm.interfaces.CyclicReferenceRemover;
import com.ibm.rpm.interfaces.wrapper.ResourceArray;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.document.containers.GenericDocument;

/**
 * @author eturcot
 */
public class ApplicationController implements Application
{ 
    private Application appInterface = null;

    private ApplicationLiteral appLiteralInterface = null;

    private boolean useLiteral = false;

    public ApplicationController(String url, boolean useLiteral)
            throws Exception
    {
        super();
        this.useLiteral = useLiteral;
        if (useLiteral)
        {
            URL service = new URL(url + "ApplicationLiteral");
            Class locatorClass = Class
                .forName("com.ibm.rpm.interfaces.ApplicationLiteralServiceLocator");
            Object locator = locatorClass.newInstance();
            Method getApplication = locatorClass.getMethod(
                "getApplicationLiteral", new Class[]
                {
                    service.getClass()
                });
            appLiteralInterface = (ApplicationLiteral) getApplication.invoke(
                locator, new Object[]
                {
                    service
                });
        }
        else
        {
            URL service = new URL(url + "Application");
            Class locatorClass = Class
                .forName("com.ibm.rpm.interfaces.ApplicationServiceLocator");
            Object locator = locatorClass.newInstance();
            Method getApplication = locatorClass.getMethod("getApplication",
                new Class[]
                {
                    service.getClass()
                });
            appInterface = (Application) getApplication.invoke(locator,
                new Object[]
                {
                    service
                });
        }
    }
    
    public Result checkOut(String sessionID, RPMObject rpmObject,
            boolean isTimeless) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(rpmObject);
            return appLiteralInterface.checkOut(sessionID, rpmObject, isTimeless);
        }
        return appInterface.checkOut(sessionID, rpmObject, isTimeless);
    }

    public Result checkIn(String sessionID, RPMObject rpmObject)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(rpmObject);
            return appLiteralInterface.checkIn(sessionID, rpmObject);
        }
        return appInterface.checkIn(sessionID, rpmObject);
    }
    
    public AlreadyCheckedOutResult isCheckedOut(String sessionID,
            RPMObject rpmObject) throws RemoteException
    {
       if(useLiteral)
       {
           CyclicReferenceRemover.remove(rpmObject);
           return appLiteralInterface.isCheckedOut(sessionID, rpmObject);
       }
       return appInterface.isCheckedOut(sessionID, rpmObject);
    }

    public Result forceCheckinOnSessionsCheckOut(String sessionID,
            String targetSessionID) throws RemoteException
    {
        return appInterface.forceCheckinOnSessionsCheckOut(sessionID,
            targetSessionID);
    }
    
    public Result forceCheckinOnUsersCheckOut(String sessionID,
            Resource user) throws RemoteException
    {
        return appInterface.forceCheckinOnUsersCheckOut(sessionID, user);
    }
    
    public Result forceCheckinOnRPMObjectCheckOut(String sessionID,
            RPMObject rpmObject)throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(rpmObject);
            return appInterface
                .forceCheckinOnRPMObjectCheckOut(sessionID, rpmObject);
        }
        return appInterface.forceCheckinOnRPMObjectCheckOut(sessionID, rpmObject);
    }
    
    public SingleResult save(String sessionID, RPMObject rpmObject,
            RPMObjectScope rpmObjectScope, ReloadType reloadType)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(rpmObject);
            CyclicReferenceRemover.remove(rpmObjectScope);
            return appLiteralInterface.save(sessionID, rpmObject,
                rpmObjectScope, reloadType);
        }
        return appInterface.save(sessionID, rpmObject, rpmObjectScope,
            reloadType);
    }

    public Result delete(String sessionID, RPMObject rpmObject)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(rpmObject);
            return appLiteralInterface.delete(sessionID, rpmObject);
        }
        return appInterface.delete(sessionID, rpmObject);
    }

    public ArrayResult loadFromXpath(String sessionID, String xpath,
            RPMObjectScope scope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(scope);
            return appLiteralInterface.loadFromXpath(sessionID, xpath, scope);
        }
        return appInterface.loadFromXpath(sessionID, xpath, scope);
    }

    public SingleResult loadFromID(String sessionID, RPMObject rpmObject,
            RPMObjectScope scope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(rpmObject);
            CyclicReferenceRemover.remove(scope);
            return appLiteralInterface.loadFromID(sessionID, rpmObject, scope);
        }
        return appInterface.loadFromID(sessionID, rpmObject, scope);
    }

    public ArrayResult loadArrayFromID(String sessionID,
            RPMObjectArray rpmObjectArray, RPMObjectScope scope)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(rpmObjectArray);
            CyclicReferenceRemover.remove(scope);
            return appLiteralInterface.loadArrayFromID(sessionID,
                rpmObjectArray, scope);
        }
        return appInterface.loadArrayFromID(sessionID, rpmObjectArray, scope);
    }

    public PropertyArray getSystemInfo(String sessionID)
            throws RemoteException, RPMException
    {
        if (useLiteral)
        {
            return appLiteralInterface.getSystemInfo(sessionID);
        }
        return appInterface.getSystemInfo(sessionID);
    }
    
    public Result enableAutomaticCheckoutMode(String sessionID)
            throws RemoteException
    {
        if (useLiteral)
        {
            return appLiteralInterface.enableAutomaticCheckoutMode(sessionID);
        }
        return appInterface.enableAutomaticCheckoutMode(sessionID);
    }
    
    public Result disableAutomaticCheckoutMode(String sessionID)
            throws RemoteException
    {
        if (useLiteral)
        {
            return appLiteralInterface.disableAutomaticCheckoutMode(sessionID);
        }
        return appInterface.disableAutomaticCheckoutMode(sessionID);
    }
 
    public SingleResult assignScoreCard(String sessionID,
            RPMObject objectToAssign, RPMObjectScope objectToAssignScope,
            Scorecard scoreCardToAssign, ReloadType reloadType)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(objectToAssign);
            CyclicReferenceRemover.remove(objectToAssignScope);
            CyclicReferenceRemover.remove(scoreCardToAssign);
            return appLiteralInterface.assignScoreCard(sessionID,
                objectToAssign, objectToAssignScope, scoreCardToAssign,
                reloadType);
        }
        return appInterface.assignScoreCard(sessionID, objectToAssign,
            objectToAssignScope, scoreCardToAssign, reloadType);
    }

    public SingleResult publishScorecard(String sessionID,
            AssignedScorecard assignedScorecard,
            AbstractScorecardScope scorecardScope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(assignedScorecard);
            CyclicReferenceRemover.remove(scorecardScope);
            return appLiteralInterface.publishScorecard(sessionID,
                assignedScorecard, scorecardScope);
        }
        return appInterface.publishScorecard(sessionID, assignedScorecard,
            scorecardScope);
    }

    public SingleResult acceptTaskAssignmentChanges(String sessionID,
            TaskAssignment taskAssignment, TaskAssignmentScope scope,
            boolean acceptEETC, boolean acceptForecastDates)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(taskAssignment);
            CyclicReferenceRemover.remove(scope);
            return appLiteralInterface.acceptTaskAssignmentChanges(sessionID,
                taskAssignment, scope, acceptEETC, acceptForecastDates);
        }
        return appInterface.acceptTaskAssignmentChanges(sessionID,
            taskAssignment, scope, acceptEETC, acceptForecastDates);
    }

    public SingleResult rejectTaskAssignmentChanges(String sessionID,
            TaskAssignment taskAssignment, TaskAssignmentScope scope,
            boolean rejectEETC, boolean rejectForecastDates)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(taskAssignment);
            CyclicReferenceRemover.remove(scope);
            return appLiteralInterface.rejectTaskAssignmentChanges(sessionID,
                taskAssignment, scope, rejectEETC, rejectForecastDates);
        }
        return appInterface.rejectTaskAssignmentChanges(sessionID,
            taskAssignment, scope, rejectEETC, rejectForecastDates);
    }

    public SingleResult copyProposedToPlan(String sessionID,
            WorkElement workElement, WorkElementScope workElementScope)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(workElement);
            CyclicReferenceRemover.remove(workElementScope);
            return appLiteralInterface.copyProposedToPlan(sessionID,
                workElement, workElementScope);
        }
        return appInterface.copyProposedToPlan(sessionID, workElement,
            workElementScope);
    }

    public SingleResult copyAssignmentProposedToPlan(String sessionID,
            ResourceTaskAssignment resourceTaskAssignment,
            ResourceTaskAssignmentScope scope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(resourceTaskAssignment);
            CyclicReferenceRemover.remove(scope);
            return appLiteralInterface.copyAssignmentProposedToPlan(sessionID,
                resourceTaskAssignment, scope);
        }
        return appInterface.copyAssignmentProposedToPlan(sessionID,
            resourceTaskAssignment, scope);
    }

    public SingleResult commitAffectedResources(String sessionID,
            GenericProject genericProject, WorkElementScope workElementScope)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(genericProject);
            CyclicReferenceRemover.remove(workElementScope);
            return appLiteralInterface.commitAffectedResources(sessionID,
                genericProject, workElementScope);
        }
        return appInterface.commitAffectedResources(sessionID, genericProject,
            workElementScope);
    }

    public SingleResult replaceProfileWithResource(String sessionID,
            ResourceTaskAssignment rta, ResourceTaskAssignmentScope rtaScope,
            Resource resource) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(rta);
            CyclicReferenceRemover.remove(rtaScope);
            CyclicReferenceRemover.remove(resource);
            return appLiteralInterface.replaceProfileWithResource(sessionID,
                rta, rtaScope, resource);
        }
        return appInterface.replaceProfileWithResource(sessionID, rta,
            rtaScope, resource);
    }

    public SingleResult replaceResourceWithProfile(String sessionID,
            ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(assignment);
            CyclicReferenceRemover.remove(rtaScope);
            return appLiteralInterface.replaceResourceWithProfile(sessionID,
                assignment, rtaScope);
        }
        return appInterface.replaceResourceWithProfile(sessionID, assignment,
            rtaScope);
    }

    public Result transferUnderScope(String sessionID,
            AbstractAggregateScope scopeElement, WorkElement workElement)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(workElement);
            CyclicReferenceRemover.remove(scopeElement);
            return appLiteralInterface.transferUnderScope(sessionID,
                scopeElement, workElement);
        }
        return appInterface.transferUnderScope(sessionID, scopeElement,
            workElement);
    }

    public Result transferUnderWBS(String sessionID, WorkElement workElement,
            AbstractScope abstractScope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(workElement);
            CyclicReferenceRemover.remove(abstractScope);
            return appLiteralInterface.transferUnderWBS(sessionID, workElement,
                abstractScope);
        }
        return appInterface.transferUnderWBS(sessionID, workElement,
            abstractScope);
    }

    public Result moveUnderWBS(String sessionID, WorkElement workElement,
            AbstractScope abstractScope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(workElement);
            CyclicReferenceRemover.remove(abstractScope);
            return appLiteralInterface.moveUnderWBS(sessionID, workElement,
                abstractScope);
        }
        return appInterface.moveUnderWBS(sessionID, workElement, abstractScope);
    }

    public Result moveAndTransferUnderWBS(String sessionID,
            WorkElement workElement, AbstractScope abstractScope)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(workElement);
            CyclicReferenceRemover.remove(abstractScope);
            return appLiteralInterface.moveAndTransferUnderWBS(sessionID,
                workElement, abstractScope);
        }
        return appInterface.moveAndTransferUnderWBS(sessionID, workElement,
            abstractScope);
    }

    public SingleResult convertContainer(String sessionID,
            RPMObject inputContainer, RPMObject outputContainer,
            RPMObjectScope rpmScope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(inputContainer);
            CyclicReferenceRemover.remove(outputContainer);
            CyclicReferenceRemover.remove(rpmScope);
            return appLiteralInterface.convertContainer(sessionID,
                inputContainer, outputContainer, rpmScope);
        }
        return appInterface.convertContainer(sessionID, inputContainer,
            outputContainer, rpmScope);
    }

    public SingleResult createObjectFromTemplate(String sessionID,
        RPMObject inputProject, RPMObject template, RPMObjectScope scope)
        throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(inputProject);
            CyclicReferenceRemover.remove(template);
            CyclicReferenceRemover.remove(scope);
            return appLiteralInterface.createObjectFromTemplate(sessionID,
                inputProject, template, scope);
        }
        return appInterface.createObjectFromTemplate(sessionID, inputProject,
            template, scope);
    }
    
    public SingleResult saveProjectBaseline(String sessionID,
            RPMObject projectObject, RPMObjectScope scope)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(projectObject);
            CyclicReferenceRemover.remove(scope);
            return appLiteralInterface.saveProjectBaseline(sessionID,
                projectObject, scope);
        }
        return appInterface
            .saveProjectBaseline(sessionID, projectObject, scope);
    }
    
    public Result activateEVWeekDelays(String sessionID, int evDelays)
            throws RemoteException
    {
        if (useLiteral)
        {
             return appLiteralInterface.activateEVWeekDelays(sessionID, evDelays);
        }
        return appInterface
            .activateEVWeekDelays(sessionID, evDelays);
    }


    public SingleResult rollupProjectEV(String sessionID,
            RPMObject projectObject, RPMObjectScope scope)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(projectObject);
            CyclicReferenceRemover.remove(scope);
            return appLiteralInterface.rollupProjectEV(sessionID,
                projectObject, scope);
        }
        return appInterface.rollupProjectEV(sessionID, projectObject, scope);
    }
    
    public SingleResult enableProjectEVCalculation(String sessionID,
            RPMObject projectObject, RPMObjectScope scope)
            throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(projectObject);
            CyclicReferenceRemover.remove(scope);
            return appLiteralInterface.enableProjectEVCalculation(sessionID,
                projectObject, scope);
        }
        return appInterface.enableProjectEVCalculation(sessionID, projectObject, scope);
    }



    public Result associateWorkflow(String sessionID,
            WorkflowProcess process, GenericProject genericProject,
            boolean associate, boolean isDefault) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(process);
            CyclicReferenceRemover.remove(genericProject);
            return appLiteralInterface.associateWorkflow(sessionID, process,
                genericProject, associate, isDefault);
        }
        return appInterface.associateWorkflow(sessionID, process,
            genericProject, associate, isDefault);
    }

    public Result cancelWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(runningProcess);
            return appLiteralInterface.cancelWorkflow(sessionID, runningProcess);
        }
        return appInterface.cancelWorkflow(sessionID, runningProcess);
    }

    public SingleResult resumeWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(runningProcess);
            return appLiteralInterface.resumeWorkflow(sessionID,
                runningProcess, scope);
        }
        return appInterface.resumeWorkflow(sessionID, runningProcess, scope);
    }

    public SingleResult startWorkflow(String sessionID,
            WorkflowProcess process, RPMObject object,
            RunningWorkflowProcessScope scope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(process);
            CyclicReferenceRemover.remove(object);
            return appLiteralInterface.startWorkflow(sessionID, process,
                object, scope);
        }
        return appInterface.startWorkflow(sessionID, process, object, scope);
    }

    public SingleResult suspendWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(runningProcess);
            return appLiteralInterface.suspendWorkflow(sessionID,
                runningProcess, scope);
        }
        return appInterface.suspendWorkflow(sessionID, runningProcess, scope);
    }
    
    public SingleResult sendNotification(String sessionID, 
            GenericCommunicationScope scope, RPMObject attachment, ResourceArray recipients, 
            String subject, String comments) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(scope);
            CyclicReferenceRemover.remove(attachment);
            CyclicReferenceRemover.remove(recipients);
            CyclicReferenceRemover.remove(subject);
            CyclicReferenceRemover.remove(comments);
            return appLiteralInterface.sendNotification(sessionID, scope, 
                attachment, recipients, subject, comments);
        }
        return appInterface.sendNotification(sessionID, scope, 
            attachment, recipients, subject, comments);
    }

    public SingleResult sendStaffingRequest(String sessionID, 
            GenericCommunicationScope scope, RPMObject attachment, ResourceArray recipients, 
            String subject, String comments, Calendar dueDate) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(scope);
            CyclicReferenceRemover.remove(attachment);
            CyclicReferenceRemover.remove(recipients);
            CyclicReferenceRemover.remove(subject);
            CyclicReferenceRemover.remove(comments);
            return appLiteralInterface.sendStaffingRequest(sessionID, scope, 
                attachment, recipients, subject, comments, dueDate);
        }
        return appInterface.sendStaffingRequest(sessionID, scope, 
            attachment, recipients, subject, comments, dueDate);
    }
    
    public SingleResult saveWorkflowResponse(java.lang.String sessionID, 
            WorkflowResponse response, WorkflowResponseScope scope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(response);
            CyclicReferenceRemover.remove(scope);

            return appLiteralInterface.saveWorkflowResponse(sessionID, response, scope);
        }
        return appInterface.sendWorkflowResponse(sessionID, response, scope);
    }
    
    public SingleResult sendWorkflowResponse(java.lang.String sessionID, 
            WorkflowResponse response, WorkflowResponseScope scope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(response);
            CyclicReferenceRemover.remove(scope);

            return appLiteralInterface.sendWorkflowResponse(sessionID, response, scope);
        }
        return appInterface.sendWorkflowResponse(sessionID, response, scope);
    }
    
    public SingleResult updateReadAndFolder(String sessionID,
            WorkflowResponse response, WorkflowResponseScope scope) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(response);
            CyclicReferenceRemover.remove(scope);

            return appLiteralInterface.updateReadAndFolder(sessionID, response, scope);
        }
        
        return appInterface.updateReadAndFolder(sessionID, response, scope);
    }
    
    public Result moveBeforeSibling(String sessionID, ScopeElement objectToMove,
            ScopeElement nextSibling) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(objectToMove);
            CyclicReferenceRemover.remove(nextSibling);

            return appLiteralInterface.moveBeforeSibling(sessionID,
                objectToMove, nextSibling);
        }
        return appInterface.moveBeforeSibling(sessionID, objectToMove,
            nextSibling);
    }

    public SingleResult lockDocument(String sessionID, GenericDocument documentToLock) throws RemoteException
    {
    	if(useLiteral)
    	{
    		CyclicReferenceRemover.remove(documentToLock);
    		
    		return appLiteralInterface.lockDocument(sessionID,documentToLock);
    	}
    	
    	return appInterface.lockDocument(sessionID,documentToLock);
    	
    }
    
    public SingleResult unLockDocument(String sessionID, GenericDocument documentToUnlock) throws RemoteException
    {
    	if(useLiteral)
    	{
    		CyclicReferenceRemover.remove(documentToUnlock);
    		
    		return appLiteralInterface.unLockDocument(sessionID, documentToUnlock);
    	}
    	
    	return appInterface.unLockDocument(sessionID, documentToUnlock);
    }
    
    public Calendar getDatabaseTimestamp(String sessionID) throws RemoteException
    {
        if(useLiteral)
        {
            return appLiteralInterface.getDatabaseTimestamp(sessionID);
        }
        
        return appInterface.getDatabaseTimestamp(sessionID); 
    }

    public SingleResult assignScoreCardToResource(String sessionID, GenericProject project, 
            Resource objectToAssign, ResourceScope objectToAssignScope, 
            Scorecard scoreCardToAssign, ReloadType reloadType) throws RemoteException
    {
        if (useLiteral)
        {
            CyclicReferenceRemover.remove(objectToAssign);
            CyclicReferenceRemover.remove(objectToAssignScope);
            CyclicReferenceRemover.remove(scoreCardToAssign);
            CyclicReferenceRemover.remove(project);
            return appLiteralInterface.assignScoreCardToResource(sessionID, project, objectToAssign,
                objectToAssignScope, scoreCardToAssign, reloadType);
        }
        return appInterface.assignScoreCardToResource(sessionID, project, objectToAssign,
            objectToAssignScope, scoreCardToAssign, reloadType);
    }
}
