/*
 * Created on Dec 23, 2005
 *
 */
package com.ibm.rpm.client.junit;

import junit.framework.TestCase;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.interfaces.impl.ServerFactory;

/**
 * @author mcduffs
 *  
 */
public class TestSessionTimeout extends TestCase
{

    private TimeoutSessionUtility timeoutSessionUtility;

    private String resourceID;

    private String loginDSN;

    private String sessionID;

    private int timeoutDelaySeconds = 3;

    private int timeoutDelayMilliSeconds = timeoutDelaySeconds * 1000;

    private int idleLoopTime = timeoutDelaySeconds * 1000 / 2;

    private int forceTimeoutDelay = timeoutDelayMilliSeconds * 2;

    private int smallDelay = idleLoopTime / 2;

    public void testTimeout() throws Exception
    {
        timeoutSessionUtility = TimeoutSessionUtility.INSTANCE;
        
        timeoutSessionUtility.setIdleLoopDelay(idleLoopTime);
        timeoutSessionUtility
            .setSessionTimeoutDelayInSeconds(timeoutDelaySeconds);
        ServerFactory.getInstance().register(timeoutSessionUtility);

        resourceID = "PMO_SUPERVISOR                  ";
        loginDSN = "jdbc/RPMDATASOURCE";

        checkNoTimeoutWhenNotStarted();

        timeoutSessionUtility.superStart();

        Thread.sleep(smallDelay);

        checkSessionTimeout();

        checkSessionKeepAlive();

        checkSessionFlushOnStop();

        checkTimeoutWhenDisabled();
    }

    /**
     * @throws RPMException
     * @throws InterruptedException
     */
    private void checkNoTimeoutWhenNotStarted() throws RPMException,
            InterruptedException
    {
        registerNewSessionID();

        fillMessageContext();

        Thread.sleep(forceTimeoutDelay);

        fillMessageContext();
    }
    
    public ServerFactory getServerFactory()
    {
        return ServerFactory.getInstance();
    }

    /**
     * @throws RPMException
     */
    private void registerNewSessionID() throws RPMException
    {
        sessionID = timeoutSessionUtility.addResourceToContext(new MessageContext(getServerFactory()), resourceID,
            loginDSN);
    }

    /**
     * @throws RPMException
     */
    private void fillMessageContext() throws RPMException
    {
        // test the newly added session is still available
        timeoutSessionUtility.fillContext(new MessageContext(getServerFactory()), sessionID, "fake");
    }

    /**
     * @throws RPMException
     */
    private void checkSessionFlushOnStop() throws RPMException
    {
        registerNewSessionID();

        timeoutSessionUtility.superStop();

        // check that the session is destroyed as soon as we stop the session
        // utility
        validateSessionTimedout();
    }

    /**
     * @throws RPMException
     * @throws InterruptedException
     */
    private void checkSessionKeepAlive() throws RPMException,
            InterruptedException
    {
        registerNewSessionID();

        long startTime = System.currentTimeMillis();

        for (int i = 0; i < 8; i++)
        {
            // make sure the sessoin stays active if we probe it enough
            Thread.sleep(smallDelay);
            long currentTime = System.currentTimeMillis();
            System.out.println(i + " time : " + (currentTime - startTime));
            fillMessageContext();
        }

        Thread.sleep(forceTimeoutDelay);

        validateSessionTimedout();
    }

    /**
     * @throws RPMException
     * @throws InterruptedException
     */
    private void checkSessionTimeout() throws RPMException,
            InterruptedException
    {
        fillMessageContext();

        Thread.sleep(forceTimeoutDelay);

        validateSessionTimedout();
    }

    /**
     * @throws RPMException
     * @throws InterruptedException
     */
    private void checkTimeoutWhenDisabled() throws RPMException,
            InterruptedException
    {
        timeoutSessionUtility.setSessionTimeoutUsed(false);

        timeoutSessionUtility.superStart();

        registerNewSessionID();
        
        fillMessageContext();

        Thread.sleep(forceTimeoutDelay);

        fillMessageContext();

        timeoutSessionUtility.superStop();
    }

    /**
     * @param timeoutSessionUtility
     * @param sessionID
     */
    private void validateSessionTimedout()
    {
        try
        {
            fillMessageContext();

            // session should have timed out
            fail();

        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.INVALID_SESSION_ID, e.getErrorID());
        }
    }

}
