/*
 * Created on Jul 4, 2005
 */
package com.ibm.rpm.client.junit;

import java.sql.Driver;
import java.sql.DriverManager;

import javax.sql.DataSource;

import org.apache.commons.dbcp.ConnectionFactory;
import org.apache.commons.dbcp.DriverManagerConnectionFactory;
import org.apache.commons.dbcp.PoolableConnectionFactory;
import org.apache.commons.dbcp.PoolingDataSource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.pool.impl.GenericObjectPool;
import org.apache.commons.pool.impl.StackKeyedObjectPoolFactory;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.interfaces.impl.DefaultSessionUtility;
import com.ibm.rpm.interfaces.impl.ServerFactory;

/**
 * @author gosselin
 */
public class UnitTestSessionUtility extends DefaultSessionUtility
{

    public static final UnitTestSessionUtility INSTANCE = new UnitTestSessionUtility(ServerFactory.getInstance());

    protected static Log logger = LogFactory
        .getLog(UnitTestSessionUtility.class);

    private static DataSource _rpmDataSource = null;

    public UnitTestSessionUtility(ServerFactory usefactory)
    {
        super(usefactory);
    }

    public DataSource getDataSource(String dsn) throws RPMException
    {
        DataSource ds = null;

        createRpmDataSource(dsn);
        ds = _rpmDataSource;

        return ds;
    }

    public void createRpmDataSource(String dsn) throws RPMException
    {
        if (_rpmDataSource != null)
        {
            return;
        }

        try
        {
            /*
             * DB2SimpleDataSource ds = null; ds = new DB2SimpleDataSource();
             * ds.setDriverType(4); ds.setDatabaseName("ibmrpm");
             * ds.setUser("rpmbuild"); ds.setPassword("Rati0nal");
             * ds.setServerName("rpmdev07.mtllab.ibm.com");
             * ds.setPortNumber(50000); _rpmDataSource = ds;
             */
            GenericObjectPool connectionPool = new GenericObjectPool(null);
            Driver clazz = (Driver) Class.forName(
                RpmTestProperties.getProperty("rpm.test.db.driver.name"))
                .newInstance();

            DriverManager.registerDriver(clazz);
            ConnectionFactory connectionFactory = new DriverManagerConnectionFactory(
                RpmTestProperties.getProperty("rpm.test.db.connect.url"),
                RpmTestProperties.getProperty("rpm.test.db.user.name"),
                RpmTestProperties.getProperty("rpm.test.db.user.password"));

            PoolableConnectionFactory poolableConnectionFactory = new PoolableConnectionFactory(
                connectionFactory, connectionPool,
                new StackKeyedObjectPoolFactory(100), RpmTestProperties
                    .getProperty("rpm.test.db.connection.test.query"), false,
                true);
            poolableConnectionFactory.getClass();

            PoolingDataSource dataSource = new PoolingDataSource(connectionPool);
            _rpmDataSource = dataSource;

            logger.debug("CREATE RPM DATASOURCE");
            return;

        }
        catch (Exception e)
        {
            throw new RPMException(
                "invalid data source specified for test framework : " + dsn);
        }

    }

    public void start()
    {

    }

    public void stop()
    {

    }

    public void superStart()
    {
        super.start();
    }

    public void superStop()
    {
        super.stop();
    }
}
