/*
 * Created on Sep 15, 2005
 *
 */
package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Comparator;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.containers.GenericAttribute;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author karim
 *
 */
public class AttributeAssignmentFixture extends RPMObjectFixture
{

    protected static ArrayList attrSkipSetters;

    static
    {
        attrSkipSetters = new ArrayList();
        attrSkipSetters.addAll(skipSetters);
        attrSkipSetters.add("setValue");
    }

    //this method should only be used for WBS children attributes assignments
    public static AttributeAssignment createAssignmentFixtureWithSpecificContext(
            String contextName, RPMObject parent, ClientTestController ctc)
            throws Exception
    {
        return createAssignment(contextName, parent, ctc);
    }

    public static AttributeAssignment createAssignmentFixture(
            RPMObject parent, ClientTestController ctc)
            throws Exception
    {
        String contextName = determineContextName(parent);
        return createAssignment(contextName, parent, ctc);
    }
    
    private static String determineContextName(RPMObject parent){
        if(parent instanceof Asset){
            return AttributeFixture.ASSET_CONTEXT;
        } else if (parent instanceof ClientCostCenter){
            return AttributeFixture.CLIENT_CONTEXT;
        } else if (parent instanceof DocumentElement){
            return AttributeFixture.DOCUMENT_CONTEXT;
        } else if (parent instanceof Pool){
            return AttributeFixture.POOL_CONTEXT;
        } else if (parent instanceof Resource){
            return AttributeFixture.RESOURCE_CONTEXT;
        } else if (parent instanceof WorkElement || parent instanceof ScopeTask 
                || parent instanceof ScopeMilestone || parent instanceof ScopeDeliverable){
            return AttributeFixture.WBS_CONTEXT;
        } else if (parent instanceof ScopeElement){
            return AttributeFixture.SCOPE_CONTEXT;
        }
        return null;
    }
    
    private static AttributeAssignment createAssignment(String contextName, 
            RPMObject parent, ClientTestController ctc) throws Exception
    {
        AttributeAssignment assignment = new AttributeAssignment();

        assignment.setParent(parent);
        
        AttributeCategory category = AttributeFixture.createSaveFixture(contextName, ctc);
        Attribute attribute = category.getClassifications()[0].getAttributes()[0];

        assignment.setAttribute(attribute);

        HelperTestUtilities.setByIntrospection(assignment, random(),
            attrSkipSetters);
        
        if (attribute.getInputType() == InputType.None)
            assignment.setValue(null);
        else if (attribute.getInputType() == InputType.Text)
            assignment.setValue("Text");
        // Numeric, RiskAppetite and RiskTolerance all take String, Integer or Double - this way we test them all.
        else if (attribute.getInputType() == InputType.Numeric)
            assignment.setValue("3");
        else if (attribute.getInputType() == InputType.RiskAppetite)
            assignment.setValue(new Integer(3));
        else if (attribute.getInputType() == InputType.RiskTolerance)
            assignment.setValue(new Double(3.3));
        else if (attribute.getInputType() == InputType.Date)
        {
            Calendar calendar = Calendar.getInstance();
            calendar.set(1996, 3, 16);
            assignment.setValue(calendar);
        }

        return assignment;
    }

    public static AttributeAssignment createWBSClassificationAssignmentFixture(
            RPMObject parent, ClientTestController ctc)
            throws Exception
    {
        return createWBSClassificationAssignment(parent, ctc);
    }

    private static AttributeAssignment createWBSClassificationAssignment(
            RPMObject parent, ClientTestController ctc)
            throws Exception
    {
        AttributeAssignment assignment = new AttributeAssignment();

        assignment.setParent(parent);
        
        String contextName = determineContextName(parent);

        AttributeCategory category = AttributeFixture.createSaveFixture(
            contextName, ctc);
        AttributeClassification classification = category.getClassifications()[0];

        assignment.setAttribute(classification);

        return assignment;

    }

    public static void updateAssignmentFixture(AttributeAssignment assignment)
            throws Exception
    {
        updateAssignment(random(), assignment);
    }

    private static void updateAssignment(int index,
            AttributeAssignment assignment) throws Exception
    {
        HelperTestUtilities.setByIntrospection(assignment, index,
            attrSkipSetters);
    }

    public static RPMObjectScope createParentScope(RPMObject parent) throws Exception
    {
        RPMObjectScope parentScope = null;

        if (parent instanceof Asset)
        {
            parentScope = new AssetScope();
            ((AssetScope)parentScope).setAttributeAssignments(true);
        }
        else if (parent instanceof Resource)
        {
            parentScope = new ResourceScope();
            ((ResourceScope)parentScope).setExecutiveAttributes(true);
        }
        else if (parent instanceof WorkElement)
        {
            parentScope = new WorkElementScope();
            ((WorkElementScope)parentScope).setAttributeAssignments(true);
            ((WorkElementScope)parentScope).setRelatedAttributeAssignments(true);
        }
        else if (parent instanceof ClientCostCenter)
        {
            parentScope = new ClientCostCenterScope();
            ((ClientCostCenterScope)parentScope).setAttributeAssignments(true);
        }
        else if(parent instanceof Pool)
        {
            parentScope = new PoolScope();
            ((PoolScope)parentScope).setAttributeAssignments(true);
        }
        else if (parent instanceof AbstractAggregateScope)
        {
            parentScope = new ScopeElementScope();
            ((ScopeElementScope)parentScope).setAttributeAssignments(true);
        }
        else if (parent instanceof GenericDocument)
        {
            parentScope = new DocumentScope();
            ((DocumentScope)parentScope).setAttributeAssignments(true);
        }
        else
        {
            logger
                    .debug("AttributeAssignmentFixture.createParentScope : parent of type "
                            + parent.getClass().getName()
                            + " is not supported.");
        }
        return parentScope;
    }
    
    public static AttributeAssignment[] getAttributeAssignmentsFromParent(RPMObject parent) throws Exception
    {
        if (parent instanceof Asset)
        {
            return ((Asset)parent).getAttributeAssignments();
        }
        else if (parent instanceof Resource)
        {
            return ((Resource)parent).getExecutiveAttributes();
        }
        else if (parent instanceof WorkElement)
        {
            return ((WorkElement)parent).getAttributeAssignments();
        }
        else if (parent instanceof ClientCostCenter)
        {
            return ((ClientCostCenter)parent).getAttributeAssignments();
        }
        else if(parent instanceof Pool)
        {
            return ((Pool)parent).getAttributeAssignments();
        }
        else if (parent instanceof AbstractAggregateScope)
        {
            return ((AbstractAggregateScope)parent).getAttributeAssignments();
        }
        else if (parent instanceof GenericDocument)
        {
            return ((GenericDocument)parent).getAttributeAssignments();
        }
        else
        {
            logger
                    .debug("AttributeAssignmentFixture.getAttributeAssignments : parent of type "
                            + parent.getClass().getName()
                            + " is not supported.");
        }
        return null;
    }
    
    private static void setAttributeAssignmentsFromParent(RPMObject parent, AttributeAssignment[] assignments) throws Exception
    {
        if (parent instanceof Asset)
        {
            ((Asset)parent).setAttributeAssignments(assignments);
        }
        else if (parent instanceof Resource)
        {
            ((Resource)parent).setExecutiveAttributes(assignments);
        }
        else if (parent instanceof WorkElement)
        {
            ((WorkElement)parent).setAttributeAssignments(assignments);
        }
        else if (parent instanceof ClientCostCenter)
        {
            ((ClientCostCenter)parent).setAttributeAssignments(assignments);
        }
        else if(parent instanceof Pool)
        {
            ((Pool)parent).setAttributeAssignments(assignments);
        }
        else if (parent instanceof AbstractAggregateScope)
        {
            ((AbstractAggregateScope)parent).setAttributeAssignments(assignments);
        }
        else if (parent instanceof GenericDocument)
        {
            ((GenericDocument)parent).setAttributeAssignments(assignments);
        }
        else
        {
            logger
                    .debug("AttributeAssignmentFixture.getAttributeAssignments : parent of type "
                            + parent.getClass().getName()
                            + " is not supported.");
        }
    }
    
    public static RPMObject createParent(RPMObject parent, RPMObjectScope parentScope,
            ClientTestController ctc) throws Exception
    {
        RPMObject createdParent = null;

        if (parent instanceof Asset)
        {
            createdParent = AssetFixture.createFixture((AssetScope)parentScope, ctc);
        }
        else if (parent instanceof Resource)
        {
            Pool pool = PoolFixture.createSaveFixture(ctc);
            createdParent = ResourceFixture.createFixture(pool, (ResourceScope)parentScope, ctc);
        }
        else if (parent instanceof WorkElement)
        {
            createdParent = WorkElementFixture.createFixture(
                    (WorkElement)parent, (WorkElementScope)parentScope, ctc);
        }
        else if (parent instanceof ClientCostCenter)
        {
            createdParent = ClientCostCenterFixture.createFixture((ClientCostCenterScope)parentScope,
                    ctc);
        } else if(parent instanceof Pool){
            createdParent = PoolFixture.createFixture(false, 1, (PoolScope)parentScope, ctc);
        }
        else if (parent instanceof AbstractAggregateScope)
        {
            ScopeFolder mainScopeFolder = ScopeElementFixture.createMainParent(ctc);
            createdParent = ScopeElementFixture.createScopeElementFixture((ScopeElementScope)parentScope, 1, (ScopeElement)parent, mainScopeFolder, ctc);
        } else if(parent instanceof GenericDocument){
            //create a parent under which we will create documents
            ClientCostCenterScope cccScope = new ClientCostCenterScope();
            cccScope.setDocumentFolder(new DocumentScope());
            ClientCostCenter ccc = ClientCostCenterFixture.createSaveFixture(cccScope, ctc);
            createdParent = DocumentElementFixture.createFixture(ccc.getDocumentFolder(), parent, 0, (DocumentScope)parentScope, ctc);
        }
        else
        {
            logger
                    .debug("AttributeAssignmentFixture.createParent : parent of type "
                            + parent.getClass().getName()
                            + " is not supported.");
        }
        return createdParent;
    }
    
    public static void updateAssignmentsFieldsFromParent(RPMObject parent)
    throws Exception
	{
        AttributeAssignment[] assignments = getAttributeAssignmentsFromParent(parent);
        if(assignments!=null){
            for(int i=0;i<assignments.length;i++){
                updateAssignmentFixture(assignments[i]);
            }
        }
	}
    
    public static void updateAssignmentsStructureFromParent(RPMObject parent, ClientTestController ctc)
    throws Exception
	{
        AttributeAssignment[] assignments = getAttributeAssignmentsFromParent(parent);
        int newLength = (assignments!=null)?assignments.length+1:1;
        
        AttributeAssignment[] newAssignments = new AttributeAssignment[newLength];
        if(assignments!=null){
            for(int i=0;i<newLength-1;i++){
                newAssignments[i] = assignments[i];
            }
        }
        
        newAssignments[newLength-1] = createAssignmentFixture(parent,ctc);
        
        setAttributeAssignmentsFromParent(parent, newAssignments);        
	}
    
    public static void compareParents(RPMObject parent1, RPMObject parent2, RPMObjectScope parentScope)
            throws AssertionFailedError, Exception
    {
        if (parent1 instanceof ClientCostCenter)
        {
            ClientCostCenterFixture.compare((ClientCostCenter) parent1,
                (ClientCostCenter) parent2, (ClientCostCenterScope)parentScope);
        }
        else if (parent1 instanceof Asset)
        {
            AssetFixture.compare((Asset) parent1, (Asset) parent2, (AssetScope)parentScope);
        }
        else if (parent1 instanceof GenericDocument)
        {
            DocumentElementFixture.compare((GenericDocument) parent1,
                (GenericDocument) parent2, (DocumentScope)parentScope);
        }
        else if (parent1 instanceof Pool)
        {
            PoolFixture.compare((Pool) parent1, (Pool) parent2, (PoolScope)parentScope);
        } 
        else if (parent1 instanceof WorkElement)
        {
            WorkElementFixture.compare((WorkElement)parent1, (WorkElement)parent2, (WorkElementScope)parentScope);
        }
        else if (parent1 instanceof Resource)
        {
            ResourceFixture.compare((Resource) parent1, (Resource) parent2, (ResourceScope)parentScope);
        }
        else if(parent1 instanceof AbstractAggregateScope)
        {
            ScopeElementFixture.compare((AbstractAggregateScope)parent1, (AbstractAggregateScope)parent2, (ScopeElementScope)parentScope);
        }
        else
        {
            Assert
                .fail("AttributeAssignmentFixture.compareParents : parent.class "
                        + parent1.getClass().getName() + " not supported by this method.");
        }
    }

    public static void compare(AttributeAssignment obj1,
            AttributeAssignment obj2) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(obj1, obj2);

        compareParents(obj1.getParent(), obj2.getParent(), null);

        AttributeFixture
            .compare(obj1.getAttribute(), obj2.getAttribute(), null);
    }

    public static void compare(AttributeAssignment[] pArray1,
            AttributeAssignment[] pArray2) throws AssertionFailedError,
            Exception
    {

        sortWithComparator(pArray1, new RPMObjectAttributeComparator());
        sortWithComparator(pArray2, new RPMObjectAttributeComparator());

        RPMObjectFixture.compare(pArray1, pArray2);

        if (pArray1 == null)
            return;

        for (int i = 0; i < pArray1.length; i++)
        {
            AttributeAssignmentFixture.compare(pArray1[i], pArray2[i]);
        }
    }

    protected static class RPMObjectAttributeComparator implements Comparator
    {
        /**
         *
         */
        public RPMObjectAttributeComparator()
        {
            super();
        }

        public int compare(Object o1, Object o2)
        {

            AttributeAssignment obj1 = (AttributeAssignment) o1;
            AttributeAssignment obj2 = (AttributeAssignment) o2;

            GenericAttribute attribute1 = null;
            GenericAttribute attribute2 = null;

            if (obj1 != null)
            {
                attribute1 = obj1.getAttribute();
            }
            if (obj2 != null)
            {
                attribute2 = obj2.getAttribute();
            }

            if (attribute1 == null && attribute2 == null)
            {
                return 0;
            }
            if (attribute1 == null && attribute2 != null)
            {
                return -1;
            }
            if (attribute1 != null && attribute2 == null)
            {
                return 1;
            }
            return RPMObjectNameAndIDComparator.INSTANCE.compare(attribute1, attribute2);
        }

    }
}
