package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.applicationadministration.types.SelectType;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.scope.AssetModuleScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetModuleFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientModuleFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentModuleFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolModuleFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceModuleFixture;
import com.ibm.rpm.client.junit.scope.fixtures.ScopeManagementModuleFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WbsModuleFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientModule;
import com.ibm.rpm.clientcostcenters.scope.ClientModuleScope;
import com.ibm.rpm.document.containers.DocumentModule;
import com.ibm.rpm.document.scope.DocumentModuleScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.resource.containers.PoolModule;
import com.ibm.rpm.resource.scope.PoolModuleScope;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.scope.ScopeManagementModuleScope;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.scope.WbsModuleScope;

/**
 * @author karim
 *  
 */
public class AttributeFixture extends RPMObjectFixture
{
    //TODO add subcategories support
    protected static final int DEFAULT_NB_LEVELS = 1;

    protected static final int DEFAULT_NB_CATEGORIES = 1;

    protected static final int DEFAULT_NB_CLASSIFICATIONS = 1;

    protected static final int DEFAULT_NB_ATTRIBUTES = 1;

    private static InputType[] inputTypes = new InputType[]
    {
            InputType.None, InputType.Text, InputType.Date, InputType.Numeric,
            InputType.RiskTolerance, InputType.RiskAppetite
    };

    private static int inputTypeIndex = 0;
    
    private static SelectType[] selectTypes = new SelectType[]
    {
            SelectType.Multi, SelectType.Single
    };

    //creates a new structure
    public static AttributeCategory createSaveFixture(String contextName,
            ClientTestController ctc) throws Exception
    {
        AttributeScope scope = createScope(1, DEFAULT_NB_LEVELS,
            DEFAULT_NB_CATEGORIES, DEFAULT_NB_CLASSIFICATIONS,
            DEFAULT_NB_ATTRIBUTES);
        AttributeCategory category = createFixture(DEFAULT_NB_CATEGORIES,
            DEFAULT_NB_CLASSIFICATIONS, DEFAULT_NB_ATTRIBUTES, scope);
                
        category = saveParentModuleWithScope(ctc, category, scope, contextName);

        return category;
    }

    public static AttributeCategory createFixture(int nbCategories,
            int nbClassifications, int nbAttributes, AttributeScope scope)
            throws Exception
    {
        return createCategory(nbCategories, nbClassifications, nbAttributes,
            scope);
    }

    private static AttributeCategory createCategory(int nbCategories,
            int nbClassifications, int nbAttributes, AttributeScope scope)
            throws Exception
    {
        AttributeCategory category = new AttributeCategory();
        HelperTestUtilities.setByIntrospection(category, random(), skipSetters);

        if (scope != null && scope.getClassifications() != null)
        {
            if (nbClassifications > 0)
            {
                AttributeClassification classification = null;
                AttributeClassification[] classifications = new AttributeClassification[nbClassifications];
                for (int i = 0; i < nbClassifications; i++)
                {
                    classification = createClassification(nbAttributes, scope
                        .getClassifications());
                    classification.setParent(category);
                    classifications[i] = classification;
                }
                category.setClassifications(classifications);
            }
        }
        return category;
    }

    private static AttributeClassification createClassification(
            int nbAttributes, AttributeScope scope) throws Exception
    {
        int index = random();
        AttributeClassification classification = new AttributeClassification();
        HelperTestUtilities.setByIntrospection(classification, index,
            skipSetters);

        if (scope != null && scope.getAttributes() != null)
        {
            if (nbAttributes > 0)
            {
                Attribute attribute = null;
                Attribute[] attributes = new Attribute[nbAttributes];
                for (int i = 0; i < nbAttributes; i++)
                {
                    attribute = createAttribute();
                    attribute.setParent(classification);
                    attributes[i] = attribute;
                }
                classification.setAttributes(attributes);
            }
        }
        return classification;
    }

    private static Attribute createAttribute() throws Exception
    {
        int index = random();
        Attribute attribute = new Attribute();
        HelperTestUtilities.setByIntrospection(attribute, index, skipSetters);
        attribute.setInputType(inputTypes[inputTypeIndex]);
        inputTypeIndex = (++inputTypeIndex)%6;
        return attribute;
    }

    //updates the names
    public static void updateCategoryFixture(AttributeCategory category,
            AttributeScope scope) throws Exception
    {
        updateCategory(category, scope, random());
    }

    public static void updateClassificationFixture(
            AttributeClassification classification, AttributeScope scope)
            throws Exception
    {
        updateClassification(classification, scope, random());
    }

    public static void updateAttributeFixture(Attribute attribute)
            throws Exception
    {
        updateAttribute(attribute, random());
    }

    private static void updateCategory(AttributeCategory category,
            AttributeScope scope, int index) throws Exception
    {
        HelperTestUtilities.setByIntrospection(category, index, skipSetters);
        if (scope != null && scope.getClassifications() != null)
        {
            AttributeClassification[] classifications = category
                .getClassifications();
            for (int i = 0; i < classifications.length; i++)
            {
                updateClassificationFixture(classifications[i], scope
                    .getClassifications());
            }
        }
    }

    private static void updateClassification(
            AttributeClassification classification, AttributeScope scope,
            int index) throws Exception
    {
        HelperTestUtilities.setByIntrospection(classification, index,
            skipSetters);

        //update selectType
        updateSelectType(classification);

        if (scope != null && scope.getAttributes() != null)
        {
            Attribute[] attributes = classification.getAttributes();
            for (int i = 0; i < attributes.length; i++)
            {
                updateAttributeFixture(attributes[i]);
            }
        }
    }

    private static void updateAttribute(Attribute attribute,
            int index) throws Exception
    {
        HelperTestUtilities.setByIntrospection(attribute, index, skipSetters);

        //update inputType
        updateInputType(attribute);
    }

    private static void updateSelectType(AttributeClassification classification)
    {
        SelectType newSelectType = getNextSelectType(classification
            .getSelectType());
        classification.setSelectType(newSelectType);
    }

    private static void updateInputType(Attribute attribute)
    {
        InputType newInputType = getNextInputType(attribute.getInputType());
        attribute.setInputType(newInputType);
    }

    //updates the structure add/remove/element
    public static void updateCategoryFixture(AttributeCategory category,
            int deltaNbClassifications, int deltaNbAttributesByClass,
            AttributeScope scope) throws Exception
    {
        updateCategory(category, deltaNbClassifications,
            deltaNbAttributesByClass, scope);
    }

    public static void updateClassificationFixture(
            AttributeClassification classification,
            int deltaNbAttributesByClass, AttributeScope scope)
            throws Exception
    {
        updateClassification(classification, deltaNbAttributesByClass, scope);
    }

    private static void updateCategory(AttributeCategory category,
            int deltaNbClassifications, int deltaNbAttributesByClass,
            AttributeScope scope) throws Exception
    {
        if (scope != null && scope.getClassifications() != null)
        {
            if (deltaNbClassifications > 0)
            {//add new elements
                AttributeClassification newClassification = null;
                AttributeClassification[] newClassifications = new AttributeClassification[deltaNbClassifications];
                int originalNbAttributesByClass = getOriginalNbAttributesByClass(category);
                for (int i = 0; i < deltaNbClassifications; i++)
                {
                    newClassification = createClassification(
                        originalNbAttributesByClass, scope.getClassifications());
                    newClassification.setParent(category);
                    newClassifications[i] = newClassification;
                }

                ArrayList temp = mergeArrays(category.getClassifications(),
                    newClassifications);
                AttributeClassification[] totalClassifications = new AttributeClassification[temp
                    .size()];
                temp.toArray(totalClassifications);

                category.setClassifications(totalClassifications);
            }
            AttributeClassification[] classifications = category
                .getClassifications();
            for (int i = 0; i < classifications.length; i++)
            {
                updateClassificationFixture(classifications[i],
                    deltaNbAttributesByClass, scope.getClassifications());
            }
        }
    }

    private static void updateClassification(
            AttributeClassification classification,
            int deltaNbAttributesByClass, AttributeScope scope)
            throws Exception
    {
        if (scope != null && scope.getAttributes() != null)
        {
            if (deltaNbAttributesByClass > 0)
            {//add new elements
                Attribute newAttribute = null;
                Attribute[] newAttributes = new Attribute[deltaNbAttributesByClass];
                for (int i = 0; i < deltaNbAttributesByClass; i++)
                {
                    newAttribute = createAttribute();
                    newAttribute.setParent(classification);
                    newAttributes[i] = newAttribute;
                }

                ArrayList temp = mergeArrays(classification.getAttributes(),
                    newAttributes);
                Attribute[] totalAttributes = new Attribute[temp.size()];
                temp.toArray(totalAttributes);

                classification.setAttributes(totalAttributes);
            }
            Attribute[] attributes = classification.getAttributes();
            for (int i = 0; i < attributes.length; i++)
            {
                updateAttributeFixture(attributes[i]);
            }
        }
    }

    private static int getOriginalNbAttributesByClass(AttributeCategory category)
    {
        AttributeClassification[] classifications = category
            .getClassifications();
        if (classifications != null && classifications.length != 0)
        {
            Attribute[] attributes = classifications[0].getAttributes();
            if (attributes != null)
            {
                return attributes.length;
            }
        }
        return 0;
    }

    //Compare section
    public static void compare(RPMObject[] array1, RPMObject[] array2,
            AttributeScope scope) throws AssertionFailedError, Exception
    {

        sortWithComparator(array1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(array2, RPMObjectNameAndIDComparator.INSTANCE);

        RPMObjectFixture.compare(array1, array2);

        if (array1 != null && array2 != null)
        {
            for (int i = 0; i < array1.length; i++)
            {
                compare(array1[i], array2[i], scope);
            }
        }
    }

    public static void compare(RPMObject obj1, RPMObject obj2,
            AttributeScope scope) throws AssertionFailedError, Exception
    {

        RPMObjectFixture.compare(obj1, obj2);

        if (obj1 instanceof AttributeCategory)
        {
            compareCategory((AttributeCategory) obj1, (AttributeCategory) obj2,
                scope);
        }
        else if (obj1 instanceof AttributeClassification)
        {
            compareClassification((AttributeClassification) obj1,
                (AttributeClassification) obj2, scope);
        }
        else if (obj1 instanceof Attribute)
        {
            // no extra comparison other than the standard one called before this
            // method

        }
        else
        {
            Assert.fail("AttributeFixture.compare : class "
                    + obj1.getClass().getName() + " not supported.");
        }
    }

    private static void compareCategory(AttributeCategory obj1,
            AttributeCategory obj2, AttributeScope scope)
            throws AssertionFailedError, Exception
    {

        AttributeCategory category1 = obj1;
        AttributeCategory category2 = obj2;

        //compare children
        if (scope != null && scope.getClassifications() != null)
        {
            compare(category1.getClassifications(), category2
                .getClassifications(), scope.getClassifications());
        }

    }

    private static void compareClassification(AttributeClassification obj1,
            AttributeClassification obj2, AttributeScope scope)
            throws AssertionFailedError, Exception
    {

        AttributeClassification classification1 = obj1;
        AttributeClassification classification2 = obj2;

        //compare children
        if (scope != null && scope.getAttributes() != null)
        {
            compare(classification1.getAttributes(), classification2
                .getAttributes(), scope.getAttributes());
        }

    }

    public static AttributeScope createScope(int currentLevel, int maxLevels,
            int nbCategories, int nbClassifications, int nbAttributes)
    {
        AttributeScope scope = new AttributeScope();

        if (currentLevel <= maxLevels)
        {
            if (nbCategories > 0)
            {
                //create subcategories scope
                AttributeScope subCategoryScope = createScope(currentLevel + 1,
                    maxLevels, nbCategories, nbClassifications, nbAttributes);
                subCategoryScope.setParent(scope);
                scope.setCategories(subCategoryScope);
            }
            if (nbClassifications > 0)
            {
                //create classifications scope
                AttributeScope classificationScope = new AttributeScope();
                if (nbAttributes > 0)
                {
                    //create attributes scope
                    AttributeScope attributeScope = new AttributeScope();
                    attributeScope.setParent(classificationScope);
                    classificationScope.setAttributes(attributeScope);
                }
                classificationScope.setParent(scope);
                scope.setClassifications(classificationScope);
            }
        }
        return scope;
    }

    private static ArrayList mergeArrays(Object[] array1, Object[] array2)
    {
        ArrayList temp = new ArrayList();
        addArray(temp, array1);
        addArray(temp, array2);
        return temp;
    }

    private static void addArray(ArrayList result, Object[] array)
    {
        if (array != null && array.length != 0)
        {
            for (int i = 0; i < array.length; i++)
            {
                result.add(array[i]);
            }
        }
    }


    private static InputType getNextInputType(InputType currentInputType)
    {
        int currentIndex = getInputTypeIndex(currentInputType);
        int nextIndex = (currentIndex + 1) % inputTypes.length;
        return inputTypes[nextIndex];
    }

    private static int getInputTypeIndex(InputType currentInputType)
    {
        if (currentInputType != null)
        {
            for (int i = 0; i < inputTypes.length; i++)
            {
                if (currentInputType.equals(inputTypes[i]))
                {
                    return i;
                }
            }
        }
        return 0;
    }

    private static SelectType getNextSelectType(SelectType currentSelectType)
    {
        int currentIndex = getSelectTypeIndex(currentSelectType);
        int nextIndex = (currentIndex + 1) % selectTypes.length;
        return selectTypes[nextIndex];
    }

    private static int getSelectTypeIndex(SelectType currentSelectType)
    {
        if (currentSelectType != null)
        {
            for (int i = 0; i < selectTypes.length; i++)
            {
                if (currentSelectType.equals(selectTypes[i]))
                {
                    return i;
                }
            }
        }
        return 0;
    }

    //TODO remove these once the Module classes are implemented
    public static final String ASSET_CONTEXT = "Asset";

    public static final String CLIENT_CONTEXT = "Client";

    public static final String DOCUMENT_CONTEXT = "Document";

    public static final String POOL_CONTEXT = "Pool";

    public static final String SCOPE_CONTEXT = "Scope";

    public static final String WBS_CONTEXT = "WBS";

    public static final String RESOURCE_CONTEXT = "Resource";

    //composed document contexts
    public static String ASSET_DOCUMENT_CONTEXT = "Asset_Document";

    public static String CLIENT_DOCUMENT_CONTEXT = "Client_Document";

    public static String RESOURCE_DOCUMENT_CONTEXT = "Resource_Document";

    public static String SCOPE_DOCUMENT_CONTEXT = "Scope_Document";

    public static String WBS_DOCUMENT_CONTEXT = "Wbs_Document";

    //TODO END TODO REMOVE

    public static boolean isClientContext(String contextName)
    {
        return isSameContext(contextName, CLIENT_CONTEXT);
    }

    public static boolean isAssetContext(String contextName)
    {
        return isSameContext(contextName, ASSET_CONTEXT);
    }

    public static boolean isDocumentContext(String contextName)
    {
        return isSameContext(contextName, DOCUMENT_CONTEXT)
                || isSameContext(contextName, ASSET_DOCUMENT_CONTEXT)
                || isSameContext(contextName, CLIENT_DOCUMENT_CONTEXT)
                || isSameContext(contextName, RESOURCE_DOCUMENT_CONTEXT)
                || isSameContext(contextName, SCOPE_DOCUMENT_CONTEXT)
                || isSameContext(contextName, WBS_DOCUMENT_CONTEXT);
    }

    public static boolean isPoolContext(String contextName)
    {
        return isSameContext(contextName, POOL_CONTEXT);
    }

    public static boolean isScopeContext(String contextName)
    {
        return isSameContext(contextName, SCOPE_CONTEXT);
    }

    public static boolean isWBSContext(String contextName)
    {
        return isSameContext(contextName, WBS_CONTEXT);
    }

    public static boolean isResourceContext(String contextName)
    {
        return isSameContext(contextName, RESOURCE_CONTEXT);
    }

    private static boolean isSameContext(String contextName,
            String refContextName)
    {
        if (contextName != null)
        {
            return refContextName.equalsIgnoreCase(contextName);
        }
        return false;
    }
    
    public static AbstractModule getParentFromContext(String context, 
            ClientTestController ctc) throws Exception
    {
        AbstractModule module = null;
        
        if (context.equals(WBS_CONTEXT))
            module = WbsModuleFixture.createFixture(null, ctc);
        else if (context.equals(SCOPE_CONTEXT))
            module = ScopeManagementModuleFixture.createFixture(null, ctc);
        else if (context.equals(DOCUMENT_CONTEXT))
            module = DocumentModuleFixture.createFixture(null, ctc);
        else if (context.equals(ASSET_CONTEXT))
            module = AssetModuleFixture.createFixture(null, ctc);
        else if (context.equals(CLIENT_CONTEXT))
            module = ClientModuleFixture.createFixture(null, ctc);
        else if (context.equals(POOL_CONTEXT))
            module = PoolModuleFixture.createFixture(null, ctc);
        else if (context.equals(RESOURCE_CONTEXT))
            module = ResourceModuleFixture.createFixture(null, ctc);
        
        return module;
    }

    private static AttributeCategory saveParentModuleWithScope(ClientTestController ctc, 
            AttributeCategory category, AttributeScope scope, 
            String context) throws Exception
    {                
        if (context.equals(WBS_CONTEXT) || context.equals(RESOURCE_CONTEXT))
        {
            // Resource Executive Atributes are the WBS attributes, therefore use WbsModule.
            WbsModule parent = WbsModuleFixture.createFixture(null, ctc);
            parent.setAttributeCategories(new AttributeCategory[] { category });
            WbsModuleScope parentScope = new WbsModuleScope();
            parentScope.setAttributeCategories(scope);
            parent = (WbsModule) save(ctc, parent, parentScope);
            return parent.getAttributeCategories()[0];
        }
        else if (context.equals(SCOPE_CONTEXT))
        {
            ScopeManagementModule parent = ScopeManagementModuleFixture.createFixture(null, ctc);
            parent.setAttributeCategories(new AttributeCategory[] { category });
            ScopeManagementModuleScope parentScope = new ScopeManagementModuleScope();
            parentScope.setAttributeCategories(scope);
            parent = (ScopeManagementModule) save(ctc, parent, parentScope);
            return parent.getAttributeCategories()[0];
        }
        else if (context.equals(DOCUMENT_CONTEXT))
        {
            DocumentModule parent = DocumentModuleFixture.createFixture(null, ctc);
            parent.setAttributeCategories(new AttributeCategory[] { category });
            DocumentModuleScope parentScope = new DocumentModuleScope();
            parentScope.setAttributeCategories(scope);
            parent = (DocumentModule) save(ctc, parent, parentScope);
            return parent.getAttributeCategories()[0];
        }
        else if (context.equals(ASSET_CONTEXT))
        {
            AssetModule parent = AssetModuleFixture.createFixture(null, ctc);
            parent.setAttributeCategories(new AttributeCategory[] { category });
            AssetModuleScope parentScope = new AssetModuleScope();
            parentScope.setAttributeCategories(scope);
            parent = (AssetModule) save(ctc, parent, parentScope);
            return parent.getAttributeCategories()[0];
        }
        else if (context.equals(CLIENT_CONTEXT))
        {
            ClientModule parent = ClientModuleFixture.createFixture(null, ctc);
            parent.setAttributeCategories(new AttributeCategory[] { category });
            ClientModuleScope parentScope = new ClientModuleScope();
            parentScope.setAttributeCategories(scope);
            parent = (ClientModule) save(ctc, parent, parentScope);
            return parent.getAttributeCategories()[0];
        }
        else if (context.equals(POOL_CONTEXT))
        {
            PoolModule parent = PoolModuleFixture.createFixture(null, ctc);
            parent.setAttributeCategories(new AttributeCategory[] { category });
            PoolModuleScope parentScope = new PoolModuleScope();
            parentScope.setAttributeCategories(scope);
            parent = (PoolModule) save(ctc, parent, parentScope);
            return parent.getAttributeCategories()[0];
        }
        
        return category;
    }


}
