package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import java.util.Calendar;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.CurrencyExchangeRange;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;

/**
 * @author karim
 *  
 */
public class CurrencyFixture extends RPMObjectFixture
{

    /**
     * @param nbExchangeRanges
     * @param scope
     * @throws Exception
     */
    public static Currency createFixture(boolean saveCurrency,
            int nbExchangeRanges, ClientTestController ctc) throws Exception
    {
        Currency currency = createCurrency(42, nbExchangeRanges);
        if (saveCurrency)
        {
            currency = (Currency) save(ctc, currency, null);
        }

        return currency;
    }

    /*
     * Creates a Currency with exchange ranges if in scope
     */
    private static Currency createCurrency(int index, int nbExchangeRanges)
            throws Exception
    {
        Currency currency = new Currency();
        HelperTestUtilities.setByIntrospection(currency, index, skipSetters);
        updateCurrency(currency, index, nbExchangeRanges);
        return currency;
    }

    private static void updateCurrency(Currency currency, int index,
            int nbExchangeRanges) throws Exception
    {
        CurrencyExchangeRange currencyExchangeRange = null;
        CurrencyExchangeRange[] exchangeRanges = new CurrencyExchangeRange[nbExchangeRanges];
        Calendar[] exchangeRangeSD = new Calendar[nbExchangeRanges];
        for (int i = 0; i < nbExchangeRanges; i++)
        {
            currencyExchangeRange = CurrencyExchangeRangeFixture
                    .createFixture(currency);

            // Currency exchange ranges need increasing start dates.
            exchangeRangeSD[i] = Calendar.getInstance();
            exchangeRangeSD[i].set(2012 + ((i / 12) % 214), 1 + (i % 12),
                    1 + (i % 28));
            currencyExchangeRange.setStartDate(exchangeRangeSD[i]);

            exchangeRanges[i] = currencyExchangeRange;
        }
        currency.setExchangeRanges(exchangeRanges);
    }

    //update fields
    public static void updateFixtureFields(Currency currency) throws Exception
    {
        updateFields(currency, random());
    }

    private static void updateFields(Currency currency, int index)
            throws Exception
    {
        HelperTestUtilities.setByIntrospection(currency, index, skipSetters);
        for (int i = 0; i < currency.getExchangeRanges().length; i++)
        {
            currency.getExchangeRanges()[i].setExchangeRate(currency
                    .getExchangeRate());
        }
    }

    public static void compare(Currency obj1, Currency obj2)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(obj1, obj2);
        CurrencyExchangeRangeFixture.compare(obj1.getExchangeRanges(), obj2
                .getExchangeRanges());
    }

    public static void compare(Currency[] array1, Currency[] array2) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(array1, array2);
        if (array1 == null)
            return;

        for (int i = 0; i < array1.length; i++)
        {
            CurrencyFixture.compare(array1[i], array2[i]);
        }
    }

}
