/*
 * Created on Jul 11, 2005
 */
package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.applicationadministration.containers.AssetCategory;
import com.ibm.rpm.applicationadministration.containers.AssetFinancialCategory;
import com.ibm.rpm.applicationadministration.containers.AssetRTF;
import com.ibm.rpm.applicationadministration.containers.AssetState;
import com.ibm.rpm.applicationadministration.containers.AssignmentStatus;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestState;
import com.ibm.rpm.applicationadministration.containers.ChargeType;
import com.ibm.rpm.applicationadministration.containers.CitizenshipCountryCode;
import com.ibm.rpm.applicationadministration.containers.Classification;
import com.ibm.rpm.applicationadministration.containers.ClientRTF;
import com.ibm.rpm.applicationadministration.containers.CompanySize;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.CustomDatafield;
import com.ibm.rpm.applicationadministration.containers.CustomDatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.DatafieldRTF;
import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.applicationadministration.containers.DefectState;
import com.ibm.rpm.applicationadministration.containers.DeliverableState;
import com.ibm.rpm.applicationadministration.containers.DocumentState;
import com.ibm.rpm.applicationadministration.containers.EmployeeStatus;
import com.ibm.rpm.applicationadministration.containers.Experience;
import com.ibm.rpm.applicationadministration.containers.GenericDatafield;
import com.ibm.rpm.applicationadministration.containers.IndustryType;
import com.ibm.rpm.applicationadministration.containers.IssueState;
import com.ibm.rpm.applicationadministration.containers.Language;
import com.ibm.rpm.applicationadministration.containers.NoteState;
import com.ibm.rpm.applicationadministration.containers.OrganizationType;
import com.ibm.rpm.applicationadministration.containers.PreferenceRank;
import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.applicationadministration.containers.RequirementState;
import com.ibm.rpm.applicationadministration.containers.RiskState;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestState;
import com.ibm.rpm.applicationadministration.containers.State;
import com.ibm.rpm.applicationadministration.containers.TaskState;
import com.ibm.rpm.applicationadministration.containers.ValueDatafield;
import com.ibm.rpm.applicationadministration.containers.WorkLocation;
import com.ibm.rpm.applicationadministration.containers.WorkingStatus;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;

/**
 * @author gosselin
 */
public class DatafieldFixture extends RPMObjectFixture
{
    public static final String COMPANY_SIZE_FORTUNE_100 = "132";

    public static final String COMPANY_SIZE_FORTUNE_500 = "141";

    public static final String COMPANY_SIZE_GREATER_THEN_500 = "179";

    public static final String COMPANY_SIZE_FORTUNE_1000 = "150";

    public static final String COMPANY_SIZE_GREATER_THEN_1000 = "167";

    public static final String COMPANY_SIZE_GLOBAL_3000 = "159";

    public static final String[] COMPANY_SIZES =
    {
            COMPANY_SIZE_FORTUNE_100, COMPANY_SIZE_FORTUNE_500,
            COMPANY_SIZE_GREATER_THEN_500, COMPANY_SIZE_FORTUNE_1000,
            COMPANY_SIZE_GREATER_THEN_1000, COMPANY_SIZE_GLOBAL_3000
    };

    public static final String CLASSIFICATION_1 = "2723";

    public static final String CLASSIFICATION_2 = "2724";

    public static final String[] CLASSIFICATIONS =
    {
            CLASSIFICATION_1, CLASSIFICATION_2
    };

    public static final String INDUSTRYTYPE_1 = "470";

    public static final String INDUSTRYTYPE_2 = "472";

    public static final String[] INDUSTRYTYPES =
    {
            INDUSTRYTYPE_1, INDUSTRYTYPE_2
    };

    public static final String PROFICIENCYLEVEL_1 = "256";

    public static final String PROFICIENCYLEVEL_2 = "257";

    public static final String PROFICIENCYLEVEL_3 = "258";

    public static final String[] PROFICIENCYLEVELS =
    {
            PROFICIENCYLEVEL_1, PROFICIENCYLEVEL_2, PROFICIENCYLEVEL_3
    };

    // Language category.
    public static final String LANGUAGE_CATEGORY_ID = "371";

    // Some language values.
    public static final String LANGUAGE_FRENCH = "413";

    public static final String LANGUAGE_ENGLISH = "404";

    public static final String LANGUAGE_CAMBODIAN = "452";

    public static final String LANGUAGE_SPANISH = "406";

    public static final String LANGUAGE_PORTUGUESE = "488";

    public static final String LANGUAGE_AFAR = "380";

    public static final String[] LANGUAGES =
    {
            LANGUAGE_FRENCH, LANGUAGE_ENGLISH, LANGUAGE_CAMBODIAN,
            LANGUAGE_SPANISH, LANGUAGE_PORTUGUESE, LANGUAGE_AFAR
    };

    // OrganizationType category.
    public static final String ORGANIZATION_TYPE_CATEGORY = "7000";
    
    // OrganizationType values.
    public static final String ORGANIZATION_TYPE_OWNING = "7001";
    
    public static final String ORGANIZATION_TYPE_EXECUTING = "7002";
    
    
    public static final String CITIZENSHIPCOUNTRYCODE_1 = "3788";

    public static final String CITIZENSHIPCOUNTRYCODE_2 = "3789";

    public static final String CITIZENSHIPCOUNTRYCODE_3 = "3790";

    public static final String[] CITIZENSHIPCOUNTRYCODES =
    {
            CITIZENSHIPCOUNTRYCODE_1, CITIZENSHIPCOUNTRYCODE_2,
            CITIZENSHIPCOUNTRYCODE_3
    };

    public static final String EXPERIENCECODE_1 = "2765";

    public static final String EXPERIENCECODE_2 = "2766";

    public static final String EXPERIENCECODE_3 = "2767";

    public static final String[] EXPERIENCECODES =
    {
            EXPERIENCECODE_1, EXPERIENCECODE_2, EXPERIENCECODE_3
    };

    public static final String WORKLOCATION_1 = "265";

    public static final String WORKLOCATION_2 = "266";

    public static final String WORKLOCATION_3 = "267";

    public static final String[] WORKLOCATIONS =
    {
            WORKLOCATION_1, WORKLOCATION_2, WORKLOCATION_3
    };

    public static final String EMPLOYEESTATUS_1 = "316";

    public static final String EMPLOYEESTATUS_2 = "317";

    public static final String EMPLOYEESTATUS_3 = "318";

    public static final String EMPLOYEESTATUS_4 = "321";

    public static final String EMPLOYEESTATUS_5 = "322";

    public static final String EMPLOYEESTATUS_6 = "323";

    public static final String EMPLOYEESTATUS_7 = "324";

    public static final String[] EMPLOYEESTATUS =
    {
            EMPLOYEESTATUS_1, EMPLOYEESTATUS_2, EMPLOYEESTATUS_3
    };

    public static final String EXPERIENCE_10years = "2765";

    public static final String EXPERIENCE_6years = "2766";

    public static final String EXPERIENCE_5years = "2767";

    public static final String EXPERIENCE_4years = "2768";

    public static final String EXPERIENCE_3years = "2769";

    public static final String EXPERIENCE_2years = "2770";

    public static final String EXPERIENCE_1years = "2771";

    public static final String[] EXPERIENCES =
    {
            EXPERIENCE_10years, EXPERIENCE_6years, EXPERIENCE_5years,
            EXPERIENCE_4years, EXPERIENCE_3years, EXPERIENCE_2years,
            EXPERIENCE_1years
    };

    public static final String WORKINGSTATUS_1 = "309";

    public static final String WORKINGSTATUS_2 = "310";

    public static final String WORKINGSTATUS_3 = "311";

    public static final String WORKINGSTATUS_8 = "301";

    public static final String WORKINGSTATUS_9 = "302";

    public static final String WORKINGSTATUS_10 = "303";

    public static final String WORKINGSTATUS_11 = "304";

    public static final String WORKINGSTATUS_12 = "305";

    public static final String WORKINGSTATUS_13 = "306";

    public static final String WORKINGSTATUS_14 = "307";

    public static final String WORKINGSTATUS_15 = "308";

    public static final String[] WORKINGSTATUS =
    {
            WORKINGSTATUS_1, WORKINGSTATUS_2, WORKINGSTATUS_3, WORKINGSTATUS_8,
            WORKINGSTATUS_9, WORKINGSTATUS_10, WORKINGSTATUS_11,
            WORKINGSTATUS_12, WORKINGSTATUS_13, WORKINGSTATUS_14,
            WORKINGSTATUS_15
    };

    public static final String PREFERENCE_RANK_LOW = "2755";

    public static final String PREFERENCE_RANK_MED = "2759";

    public static final String PREFERENCE_RANK_HIGH = "2762";

    public static final String PREFERENCE_RANK_VERY_H = "2763";

    public static final String PREFERENCE_RANK_EXTREMELY_H = "2764";

    public static final String[] PREFERENCE_RANKS =
    {
            PREFERENCE_RANK_LOW, PREFERENCE_RANK_MED, PREFERENCE_RANK_HIGH,
            PREFERENCE_RANK_VERY_H, PREFERENCE_RANK_EXTREMELY_H
    };

    public static final String CURRENCY_USD = "219";

    public static final String CURRENCY_CAD = "220";

    public static final String CURRENCY_FF = "221";

    public static final String CURRENCY_RR = "222";

    public static final String CURRENCY_YEN = "223";

    public static final String CURRENCY_BPS = "224";

    public static final String CURRENCY_HKD = "225";

    public static final String CURRENCY_EUR = "226";

    public static final String CURRENCY_PES = "227";

    public static final String CURRENCY_DM = "228";

    public static final String[] CURRENCIES =
    {
            CURRENCY_USD, CURRENCY_CAD, CURRENCY_FF, CURRENCY_RR, CURRENCY_YEN,
            CURRENCY_BPS, CURRENCY_HKD, CURRENCY_EUR, CURRENCY_PES, CURRENCY_DM
    };

    public static final String CLIENT_RTF_CLIENT_DESCRIPTION = "6051";

    public static final String CLIENT_RTF_PRIMARY_CLIENT_RELATIONSHIP = "6052";

    public static final String ASSET_STATE_OPEN = "4971";

    public static final String ASSET_STATE_TECH_OVERVIEW = "4972";

    public static final String ASSET_STATE_FIN_OVERVIEW = "4973";

    public static final String ASSET_STATE_APPROVED = "4974";

    public static final String ASSET_CATEGORY_1 = "5152";

    public static final String ASSET_CHARGE_TYPE_NONE = "6601";

    public static final String ASSET_RTF_DESCRIPTION = "6041";

    public static final String ASSET_RTF_CLOSURE_CRITERIA = "6042";

    public static final String ASSET_RTF_SOLUTIONS_CONSIDERED = "6043";

    public static final String ASSET_RTF_SOLUTIONS_RETAINED = "6044";

    public static final String ASSET_RTF_SOLUTIONS_IMPLEMENTED = "6045";

    public static final String ASSET_FINANCIAL_CATEGORY_NONE = "6501";

    public static final String ASSET_FINANCIAL_CATEGORY_APP = "6502";

    public static final String ASSET_FINANCIAL_CATEGORY_PROD_SUPPORT = "6503";

    public static final String ASSET_FINANCIAL_CATEGORY_PREVENT_MAINTENANCE = "6504";

    public static final String ASSET_FINANCIAL_CATEGORY_SPECIAL_INITIATIVE = "6505";

    public static final String PROJECT_STATE_NONE = "740";

    public static final String PROJECT_STATE_RFQ = "741";

    public static final String DELIVERABLE_STATE_NONE = "5470";

    public static final String DELIVERABLE_STATE_RFQ = "5471";

    public static final String TASK_STATE_NONE = "5500";
    
    public static final String TASK_STATE_CLOSED = "5521";

    public static final String TASK_STATE_RFQ = "5501";
    
    // ACTION STATE
    public static final String ACTION_STATE_OPEN = "6063";
    public static final String ACTION_STATE_CLOSED = "6117"; 
    
    // CHANGEREQUEST STATE
    public static final String CHANGEREQUEST_STATE_OPEN = "786";
    public static final String CHANGEREQUEST_STATE_CLOSED = "4910";
    
    // DEFECT STATE
    public static final String DEFECT_STATE_OPEN = "5365";
    public static final String DEFECT_STATE_CLOSED = "5386";
    
    // ISSUE STATE
    public static final String ISSUE_STATE_OPEN = "4809";
    public static final String ISSUE_STATE_CLOSED = "4939";
    
    // RISK STATE
    public static final String RISK_STATE_OPEN = "4830";
    public static final String RISK_STATE_CLOSED = "4920";
    
    // SERVICE REQUEST STATE
    public static final String SERVICEREQUEST_STATE_OPEN = "5396";
    public static final String SERVICEREQUEST_STATE_CLOSED = "5417";
    
    // REQUIREMENT STATE
    public static final String REQUIREMENT_STATE_OPEN = "4852";
    public static final String REQUIREMENT_STATE_CLOSED = "4930";
    
    // Abstract Document States
    public static final String DOCUMENT_STATE_OPEN = "4953";
    public static final String DOCUMENT_STATE_CLIENT_TECHNICAL_REVIEW = "4954";
    public static final String DOCUMENT_STATE_CLIENT_FINANCIAL_REVIEW = "4955";
    public static final String DOCUMENT_STATE_CLIENT_APPROVED = "4966";
    
    // NoteMinutes States
    public static final String NOTEMINUTES_STATE_OPEN = "5438";
    public static final String NOTEMINUTES_STATE_CLIENT_TECHNICAL_REVIEW = "5439";
    public static final String NOTEMINUTES_STATE_CLIENT_FINANCIAL_REVIEW = "5440";
    public static final String NOTEMINUTES_STATE_CLIENT_APPROVED = "5441";
    
    //Assignment Status
    public static final String ASSIGNMENT_STATUS_IN_INTERVIEW = "789";
    public static final String ASSIGNMENT_STATUS_APPROVED = "790";
    public static final String ASSIGNMENT_STATUS_COMMITED = "791";
    public static final String ASSIGNMENT_STATUS_PLANNED = "792";
    public static final String ASSIGNMENT_STATUS_PROPOSED = "793";
    public static final String ASSIGNMENT_STATUS_ACCEPTED = "794";
    public static final String ASSIGNMENT_STATUS_CONFIRMED = "795";
    public static final String ASSIGNMENT_STATUS_SCHEDULED = "796";
    
    // Project RTF
    public static final String PROJECT_RTF_CATEGORY_ID = "5562";
    public static final String PROJECT_RTF_1 = "5581";
    public static final String PROJECT_RTF_2 = "5582";
    
    // Project States
    public static final String PROJECT_STATE_IN_PROGRESS = "4899";
    public static final String PROJECT_STATE_TBV = "4948";
    public static final String PROJECT_STATE_CLOSED = "4949";
    
    //Deliverable States
    public static final String DELIVERABLE_STATE_CLOSED = "5491";

    protected static ArrayList datafieldSkipSetters;

    static
    {
        datafieldSkipSetters = new ArrayList();
        datafieldSkipSetters.addAll(skipSetters);
        datafieldSkipSetters.add("setName");
    }

    public static AssignmentStatus createAssignmentStatus(String assignmentStatusId)
    {
        AssignmentStatus assignmentStatus = new AssignmentStatus();
        assignmentStatus.setValue(assignmentStatusId);
        assignmentStatus.setID(assignmentStatusId);
        return assignmentStatus;
    }
    
    public static CompanySize createRandomCompanySize()
    {
        int random = random(  COMPANY_SIZES.length);
        return createCompanySize(COMPANY_SIZES[random]);
    }

    public static CompanySize createCompanySize(String companySize)
    {
        CompanySize cs = new CompanySize();
        cs.setValue(companySize);
        cs.setID(companySize);
        return cs;
    }

    public static Language createRandomLanguage()
    {
        int random = random(  LANGUAGES.length);
        return createLanguage(LANGUAGES[random]);
    }

    public static Language createLanguage(String lang)
    {
        Language language = new Language();
        language.setValue(lang);
        language.setID(lang);
        return language;
    }

    public static OrganizationType createOrganizationType(String typeId)
    {
        OrganizationType orgType = new OrganizationType();
        orgType.setValue(typeId);
        orgType.setID(typeId);
        return orgType;
    }

    public static Classification createRandomClassification()
    {
        int random = random(  CLASSIFICATIONS.length);
        return createClassification(CLASSIFICATIONS[random]);
    }

    public static Classification createClassification(String classification)
    {
        Classification cs = new Classification();
        cs.setValue(classification);
        cs.setID(classification);
        return cs;
    }

    public static IndustryType createRandomIndustryType()
    {
        int random = random(  INDUSTRYTYPES.length);
        return createIndustryType(INDUSTRYTYPES[random]);
    }

    public static IndustryType createIndustryType(String industrytype)
    {
        IndustryType cs = new IndustryType();
        cs.setValue(industrytype);
        cs.setID(industrytype);
        return cs;
    }

    public static ProficiencyLevel createRandomProficiencyLevel()
    {
        int random = random(  PROFICIENCYLEVELS.length);
        return createProficiencyLevel(PROFICIENCYLEVELS[random]);
    }

    public static ProficiencyLevel createProficiencyLevel(
            String proficiencylevel)
    {
        ProficiencyLevel cs = new ProficiencyLevel();
        cs.setValue(proficiencylevel);
        cs.setID(proficiencylevel);
        return cs;
    }

    public static CitizenshipCountryCode createRandomCitizenshipCountryCode()
    {
        int random = random( CITIZENSHIPCOUNTRYCODES.length);
        return createCitizenshipCountryCode(CITIZENSHIPCOUNTRYCODES[random]);
    }

    public static CitizenshipCountryCode createCitizenshipCountryCode(
            String citizenshipCountryCode)
    {
        CitizenshipCountryCode citizen = new CitizenshipCountryCode();
        citizen.setValue(citizenshipCountryCode);
        citizen.setID(citizenshipCountryCode);
        return citizen;
    }

    public static EmployeeStatus createRandomEmployeeStatus()
    {
        int random = random(  EMPLOYEESTATUS.length);
        return createEmployeeStatus(EMPLOYEESTATUS[random]);
    }

    public static EmployeeStatus createEmployeeStatus(String status)
    {
        EmployeeStatus es = new EmployeeStatus();
        es.setValue(status);
        es.setID(status);
        return es;
    }

    public static Experience createRandomExperience()
    {
        int random = random(  EXPERIENCES.length);
        return createExperience(EXPERIENCES[random]);
    }

    public static Experience createExperience(String ex)
    {
        Experience exp = new Experience();
        exp.setValue(ex);
        exp.setID(ex);
        return exp;
    }

    public static WorkingStatus createRandomWorkingStatus()
    {
        int random = random(  WORKINGSTATUS.length);
        return createWorkingStatus(WORKINGSTATUS[random]);
    }

    public static WorkingStatus createWorkingStatus(String ws)
    {
        WorkingStatus status = new WorkingStatus();
        status.setID(ws);
        status.setValue(ws);
        return status;
    }

    public static WorkLocation createRandomWorkLocation()
    {
        int random = random(  WORKLOCATIONS.length);
        return createWorkLocation(WORKLOCATIONS[random]);
    }

    public static WorkLocation createWorkLocation(String wl)
    {
        WorkLocation location = new WorkLocation();
        location.setID(wl);
        location.setValue(wl);
        return location;
    }

    public static PreferenceRank createRandomPreferenceRank()
    {
        int random = random(  PREFERENCE_RANKS.length);
        return createPreferenceRank(PREFERENCE_RANKS[random]);
    }

    public static PreferenceRank createPreferenceRank(String pref)
    {
        PreferenceRank p = new PreferenceRank();
        p.setID(pref);
        p.setValue(pref);
        return p;
    }

    public static Currency createRandomCurrency()
    {
        int random = random(  CURRENCIES.length);
        return createCurrency(CURRENCIES[random]);
    }

    public static Currency createCurrency(String cur)
    {
        Currency c = new Currency();
        c.setID(cur);
        return c;
    }

    public static DatafieldRTF createDatafieldRTF(String id)
    {
        DatafieldRTF datafieldRTF = null;
        if (isClientRTF(id))
        {
            datafieldRTF = new ClientRTF();
        }
        else if (isAssetRTF(id))
        {
            datafieldRTF = new AssetRTF();
        }
        else
        //add other supported types

        if (datafieldRTF != null)
        {
            datafieldRTF.setID(id);
        }
        return datafieldRTF;
    }

    public static boolean isClientRTF(String id)
    {
        return CLIENT_RTF_CLIENT_DESCRIPTION.equals(id)
                || CLIENT_RTF_PRIMARY_CLIENT_RELATIONSHIP.equals(id);
    }

    public static boolean isAssetRTF(String id)
    {
        return ASSET_RTF_CLOSURE_CRITERIA.equals(id)
                || ASSET_RTF_DESCRIPTION.equals(id)
                || ASSET_RTF_SOLUTIONS_IMPLEMENTED.equals(id)
                || ASSET_RTF_SOLUTIONS_CONSIDERED.equals(id)
                || ASSET_RTF_SOLUTIONS_RETAINED.equals(id);
    }

    public static AssetState createAssetState(String s)
    {
        AssetState state = new AssetState();
        state.setID(s);
        state.setValue(s);
        return state;
    }

    public static AssetFinancialCategory createAssetFinancialCategory(String s)
    {
        AssetFinancialCategory cat = new AssetFinancialCategory();
        cat.setID(s);
        cat.setValue(s);
        return cat;
    }

    public static DatafieldState createState(RPMObject rpmObject)
    {
        DatafieldState state = null;
        if (rpmObject instanceof GenericProject)
        {
            state = new State();
            state.setID(DatafieldFixture.PROJECT_STATE_NONE);
        }
        else if (rpmObject instanceof AggregateNode)
        {
            state = new DeliverableState();
            state.setID(DatafieldFixture.DELIVERABLE_STATE_NONE);
        }
        else if(rpmObject instanceof Task || rpmObject instanceof Milestone)
        {
            state = new TaskState();
            state.setID(DatafieldFixture.TASK_STATE_NONE);
        }
        else if(rpmObject instanceof Action)
        {
        	state = new ActionState();
        	state.setID(DatafieldFixture.ACTION_STATE_OPEN);
        }
        else if(rpmObject instanceof ChangeRequest)
        {
        	state = new ChangeRequestState();
        	state.setID(DatafieldFixture.CHANGEREQUEST_STATE_OPEN);
        }
        else if(rpmObject instanceof Defect)
        {
        	state = new DefectState();
        	state.setID(DatafieldFixture.DEFECT_STATE_OPEN);
        }
        else if(rpmObject instanceof Issue)
        {
        	state = new IssueState();
        	state.setID(DatafieldFixture.ISSUE_STATE_OPEN);
        }
        else if(rpmObject instanceof Requirement)
        {
            state = new RequirementState();
            state.setID(DatafieldFixture.REQUIREMENT_STATE_OPEN);;
        }
        else if(rpmObject instanceof Risk)
        {
        	state = new RiskState();
        	state.setID(DatafieldFixture.RISK_STATE_OPEN);
        }
        else if(rpmObject instanceof ServiceRequest)
        {
        	state = new ServiceRequestState();
        	state.setID(DatafieldFixture.SERVICEREQUEST_STATE_OPEN);
        }
        else if(rpmObject instanceof AbstractDocument)
        {
        	state = new DocumentState();
        	state.setID(DOCUMENT_STATE_OPEN);
        }
        else if(rpmObject instanceof NoteMinutes)
        {
        	state = new NoteState();
        	state.setID(NOTEMINUTES_STATE_OPEN);
        }
        if(state != null)
        {
        	state.setValue(state.getID());
        }
        return state;
    }

    public static DatafieldState updateState(RPMObject rpmObject)
    {
        DatafieldState state = null;
        if (rpmObject instanceof GenericProject)
        {
            state = ((GenericProject) rpmObject).getState();
            if(state==null)
            {
                state = new DatafieldState();
            }
            state.setID(DatafieldFixture.PROJECT_STATE_RFQ);
            state.setValue(DatafieldFixture.PROJECT_STATE_RFQ);
            ((GenericProject) rpmObject).setState(state);
        }
        else if (rpmObject instanceof AggregateNode)
        {
            state = ((AggregateNode) rpmObject).getState();
            if(state==null)
            {
                state = new DatafieldState();
            }
            state.setID(DatafieldFixture.DELIVERABLE_STATE_RFQ);
            state.setValue(DatafieldFixture.DELIVERABLE_STATE_RFQ);
            ((AggregateNode) rpmObject).setState(state);
        }
        else
        {
            state = ((WorkElement) rpmObject).getState();
            if(state==null)
            {
                state = new DatafieldState();
            }
            state.setID(DatafieldFixture.TASK_STATE_RFQ);
            state.setValue(DatafieldFixture.TASK_STATE_RFQ);
            ((WorkElement) rpmObject).setState(state);
        }
        return state;
    }

    public static AssetCategory createAssetsAssetCategory(String s)
    {
        AssetCategory cat = new AssetCategory();
        cat.setID(s);
        cat.setValue(s);
        return cat;
    }

    public static ChargeType createChargeType(String s)
    {
        ChargeType chargeType = new ChargeType();
        chargeType.setID(s);
        chargeType.setValue(s);
        return chargeType;
    }

    public static void updateFixture(Datafield rpmObject) throws Exception
    {
        HelperTestUtilities.setByIntrospection(rpmObject, random(),
                datafieldSkipSetters);
    }



    // ===============
    // OBJECT BUILDERS
    // ===============

    public static final String PREFIX = "DatafieldFixture ";
    
    /**
     * Creates the appropriate instance of <code>DatafieldCategory</code>
     * derived class from the specified <code>id</code>. Use for load.
     * 
     * @param id
     *            Identifier for the desired category.
     * @return
     */
    public static DatafieldCategory createDatafieldCategory(String id)
    {
        DatafieldCategory object = new DatafieldCategory();
        object.setID(id);
        return object;
    }

    public static CustomDatafieldCategory createCustomDatafieldCategory(
            int childrenCount)
    {
        CustomDatafieldCategory category = new CustomDatafieldCategory();
        category.setName(PREFIX + "category" + getSequenceNo());

        if (childrenCount > 0)
        {
            Datafield[] children = new Datafield[childrenCount];
            for (int i = 0; i < childrenCount; i++)
            {
                children[i] = createCustomDatafield(category);
            }
            category.setChildren(children);
        }

        return category;
    }

    public static CustomDatafield createCustomDatafield() throws Exception
    {
        CustomDatafieldCategory category = createCustomDatafieldCategory(1);
        CustomDatafield datafield = (CustomDatafield) category.getChildren()[0];

        return datafield;
    }

    public static CustomDatafield createCustomDatafield(
            CustomDatafieldCategory parent)
    {
        CustomDatafield datafield = new CustomDatafield();
        datafield.setParent(parent);
        int no = getSequenceNo();
        datafield.setName(PREFIX + "datafield " + no);

        return datafield;
    }

    // =============
    // MODIFICATIONS
    // =============

    public static void modify(GenericDatafield object) throws Exception
    {
        if (object == null)
            return;
        if (isCustom(object))
        	object.setName(trimAndAppend(object.getName(), UPDATE_SUFFIX));
        if (object instanceof Datafield)
        {
            modifyDatafield((Datafield) object);
        }
        else if (object instanceof DatafieldCategory)
        {
            modifyDatafieldCategory((DatafieldCategory) object);
        }
    }
    
    private static void modifyDatafieldCategory(DatafieldCategory object)
            throws Exception
    {
        Datafield[] children = object.getChildren();
        int childrenCount = 0;
        if (children != null)
        {
            childrenCount = children.length;
            for (int i = 0; i < childrenCount; i++)
            {
                Datafield child = children[i];
                switch (i % 3)
                {
                case 0: // Child modification.
                    modify(child);
                    break;
                case 1: // Child ignored.
                    children[i] = null;
                    break;
                case 2: // Child deletion.
                    if (isCustom(child))
                        child.setDeleted(Boolean.TRUE);
                    else
                        modify(child);
                    break;
                }
            }
        }
        // Add new children.
        int creationCount = isCustom(object) ? CREATION_COUNT : 0;
        Datafield[] newChildren = new Datafield[childrenCount + creationCount];
        if (childrenCount > 0)
        {
            System.arraycopy(children, 0, newChildren, 0, childrenCount);
        }
        if (creationCount > 0)
        {
            CustomDatafieldCategory parent = (CustomDatafieldCategory) object;
	        for (int i = 0; i < creationCount; i++)
	        {
	            // Create new child.
	            CustomDatafield newChild = createCustomDatafield(parent);
	            newChild.setName(trimAndAppend(newChild.getName(), ADD_SUFFIX));
	            // Associate parent and child.
	            newChildren[childrenCount + i] = newChild;
	            newChild.setParent(object);
	        }
        }
        object.setChildren(newChildren);
    }

    private static void modifyDatafield(Datafield object)
    {
        if (object instanceof ValueDatafield)
        {
            ValueDatafield valueDatafield = (ValueDatafield) object;
            valueDatafield.setValue(trimAndAppend(valueDatafield.getValue(), UPDATE_SUFFIX));
        }
        
        // TODO SLegault: add cases for other types.
    }

    private static boolean isCustom(RPMObject rpmObject)
    {
        return (rpmObject instanceof CustomDatafield || rpmObject instanceof CustomDatafieldCategory);
    }
    
    private static String trimAndAppend(String str, String suffix)
    {
        return str.trim() + suffix;
    }

    private static final int CREATION_COUNT = 2;
    
    private static final String UPDATE_SUFFIX = " upd";

    private static final String ADD_SUFFIX = " add";

    // ===========
    // SEQUENCE NO
    // ===========

    private static int getSequenceNo()
    {
        sequenceNo = (sequenceNo + 1) % 1000;
        return sequenceNo;
    }

    private static int sequenceNo = (int) (System.currentTimeMillis() % 1000L);

    // ===================
    // EQUALITY ASSERTIONS
    // ===================

    public static void assertEquals(RPMObject expected, RPMObject actual)
    {
        Assert.assertNotSame(expected, actual);
        doAssertTypeIdNameEquals(expected, actual);
        if (expected != null)
        {
            if (expected instanceof DatafieldCategory)
                doAssertEquals((DatafieldCategory) expected, actual);
            else if (expected instanceof ValueDatafield)
                doAssertEquals((ValueDatafield) expected, actual);
            
            // TODO SLegault: add other cases.
        }
    }

    private static void doAssertEquals(ValueDatafield expected,
            RPMObject actualObject)
    {
        ValueDatafield actual = (ValueDatafield) actualObject;
        Assert.assertEquals("Value", expected.getValue(), actual.getValue());
        // TODO: add special cases for DatafieldRTF and DatafieldState.
    }

    private static void doAssertEquals(DatafieldCategory expected,
            RPMObject actualObject)
    {
        DatafieldCategory actual = (DatafieldCategory) actualObject;
        doAssertChildrenEqual(expected.getChildren(), actual.getChildren());
    }

    private static void doAssertChildrenEqual(Datafield[] expected,
            Datafield[] actual)
    {
        if (expected == null)
        {
            Assert
                    .assertNull("Expected a null but actual is not null.",
                            actual);
            return;
        }
        Assert.assertNotNull(actual);
        Assert.assertTrue(
                "Actual must have at least the as many children as expected.",
                count(expected) <= count(actual));
        for (int i = 0; i < expected.length; i++)
        {
            Datafield expectedChild = expected[i];
            Datafield actualChild = (Datafield) findInArray(expectedChild, actual);
            assertEquals(expectedChild, actualChild);
        }
    }

    private static void doAssertTypeIdNameEquals(RPMObject expectedObject,
            RPMObject actualObject)
    {
        if (expectedObject == null)
        {
            Assert.assertNull("Expected a null but actual is not null.",
                    actualObject);
            return;
        }
        Assert.assertNotNull(actualObject);
        Assert.assertTrue(expectedObject instanceof GenericDatafield);
        Assert.assertTrue(actualObject instanceof GenericDatafield);
        GenericDatafield expected = (GenericDatafield) expectedObject;
        GenericDatafield actual = (GenericDatafield) actualObject;
        // Design note: TypeID is not exposed in WSDL.
        //Assert.assertEquals("TypeID", expectedObject.retrieveTypeID(), actual
        //        .retrieveTypeID());
        Assert.assertEquals("ID", expectedObject.getID(), actual.getID());
        if (expectedObject instanceof CustomDatafieldCategory
                || expectedObject instanceof CustomDatafield)
        {
            if (actual.getName() == null)
            {
                Assert.assertNull("Expected is non-null but actual is null.",
                        expected);
            }
            Assert.assertEquals("Name", expected.getName().trim(), actual.getName()
                    .trim());
        }
    }

    private static int count(Datafield[] children)
    {
        int count = 0;
        if (children != null)
        {
            for (int i = 0; i < children.length; i++)
            {
                Datafield child = children[i];
                if (child != null
                        && (child.getDeleted() == null || !child.getDeleted()
                                .booleanValue()))
                    count++;
            }
        }
        return count;
    }

    public static RPMObject findInArray(RPMObject object, RPMObject[] objects)
    {
        if (object != null)
        {
            String expectedID = object.getID();
            if( expectedID == null )
            {
                TestCase.fail("Attempting to find a loaded object with a null ID");
            }
            
            for (int i = 0; i < objects.length; i++)
            {
                RPMObject candidate = objects[i];
                if (candidate != null && expectedID.equals(candidate.getID()))
                    return candidate;
            }
        }
        return null;
    }

    public static void compare(Datafield d1, Datafield d2)
            throws AssertionFailedError, Exception
    {
        if (d1 == null && d2 == null) return;
        
        if (d1 != null) {
            Assert.assertNotNull("d1 is not null, while d2 is null, class: " + d1.getClass().getName(), d2);
        }
        if (d2 != null) {
            Assert.assertNotNull("d1 is null, while d2 is not null, class: " +  d2.getClass().getName(), d1);
        }

        Assert.assertEquals("Datafields are not equals, class: " + d1.getClass().getName(), 
                d1.getID().trim(), d2.getID().trim());
        //Assert.assertEquals(d1.getValue(), d2.getValue());
    }

    public static void compare(Datafield[] d1, Datafield[] d2)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(d1, d2);
        if (d1 == null)
            return;

        for (int i = 0; i < d1.length; i++)
        {
            compare(d1[i], d2[i]);
        }
    }

    // ===================
    // DELETION ASSERTIONS
    // ===================

    /**
     * Asserts that all datafields created for tests have been deleted from
     * the database.
     * 
     * @param controller
     */
    public static void assertAllDeleted(ClientTestController controller)
            throws Exception
    {
        RPMObject[] loaded = loadNonDeleted(controller);
        Assert.assertTrue("All datafields should be deleted", loaded == null
                || loaded.length == 0);
    }

    public static void assertDeleted(ClientTestController controller,
            GenericDatafield original) throws Exception
    {
        doAssertDeleted(controller, original);
        if (original instanceof DatafieldCategory)
            doAssertChildrenDeleted(controller, ((DatafieldCategory) original)
                    .getChildren());
    }

    public static void doAssertChildrenDeleted(ClientTestController controller,
            Datafield[] children) throws Exception
    {
        if (children != null)
        {
            for (int i = 0; i < children.length; i++)
            {
                doAssertDeleted(controller, children[i]);
            }
        }
    }

    private static void doAssertDeleted(ClientTestController controller,
            GenericDatafield original) throws Exception
    {
        if (original instanceof CustomDatafieldCategory
                || original instanceof CustomDatafield)
        {
            try
            {
                controller.loadFromID(original, null);
                Assert.fail("Object '" + original.getID() + "'not deleted.");
            }
            catch (Exception x)
            {
                // Normal case: object should be deleted.
            }
        }
    }

    // ======
    // DELETE
    // ======

    /**
     * Deletes all saved datafields that have a name that starts with
     * <code>CustomFieldFixture.NAME_PREFIX</code>.
     * 
     * @param controller
     */
    public static void deleteAll(ClientTestController controller)
            throws Exception
    {
        RPMObject[] nonDeleted = loadNonDeleted(controller);
        if (nonDeleted == null)
            return;
        // Delete all charge codes.
        for (int i = 0; i < nonDeleted.length; i++)
        {
            try
            {
                controller.delete(nonDeleted[i]);
            }
            catch (Exception x)
            {
                // Datatfield might have been deleted when ancestor
                // was deleted.
            }
        }
    }

    private static RPMObject[] loadNonDeleted(ClientTestController controller)
            throws Exception
    {
        // Build query.
        final String query = "/GenericDatafield[starts-with(Name, '"
                + PREFIX + "')]";
        logger.debug("XPath query: " + query);

        // Load the set of objects that match the specified property.
        RPMObject[] loaded = controller.loadFromXpath(query, null);
        return loaded;
    }
    
    public static TaskState createClosedTaskState() 
    {
        TaskState state = new TaskState();
        state.setID(TASK_STATE_CLOSED);
        state.setValue(TASK_STATE_CLOSED);
        
        return state;
    }
    
    public static DatafieldState createClosedState(RPMObject rpmObject) 
    {
    	DatafieldState closedState = null;
    	if(rpmObject instanceof Requirement)
    	{
    		closedState = new RequirementState();
    		closedState.setID(REQUIREMENT_STATE_CLOSED);
    		closedState.setValue(REQUIREMENT_STATE_CLOSED);
    	}
    	return closedState;
    }
}
