/*
 * Created on Jul 19, 2005
 *
 */
package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.City;
import com.ibm.rpm.applicationadministration.containers.Country;
import com.ibm.rpm.applicationadministration.containers.CountryState;
import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.Location;
import com.ibm.rpm.applicationadministration.containers.Region;
import com.ibm.rpm.applicationadministration.scope.GeographicalScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;

/**
 * @author karim
 *  
 */
public class GeographicalFixture extends RPMObjectFixture
{

    private static final int DEFAULT_NB_LEVELS = 6;

    public static Geographical createSaveFixture(int geographicalsByLevel,
            ClientTestController ctc) throws Exception
    {

        GeographicalScope scope = createScope(DEFAULT_NB_LEVELS);
        Geographical geographical = createFixture(geographicalsByLevel, scope);

        //Save Geographical
        geographical = (Geographical) save(ctc, geographical, scope);

        return geographical;

    }

    public static Geographical createFixture(int geographicalsByLevel,
            GeographicalScope scope) throws Exception
    {
        Geographical geographical = createGeographical(random(),
            geographicalsByLevel, null, scope);
        return geographical;
    }

    private static Geographical createGeographical(int index,
            int geographicalsByLevel, Geographical parent,
            GeographicalScope scope) throws Exception
    {
        Geographical geographical = createChild(parent);
        HelperTestUtilities
            .setByIntrospection(geographical, index, skipSetters);
        geographical.setParent(parent);
        if (canHaveChildren(geographical, scope))
        {
            GeographicalScope childrenScope = getChildrenScope(geographical,
                scope);
            Geographical childGeographical = null;
            Geographical[] childGeographicals = createChildrenArray(
                geographical, geographicalsByLevel);
            for (int i = 1; i <= geographicalsByLevel; i++)
            {
                childGeographical = createGeographical(random(),
                    geographicalsByLevel, geographical, childrenScope);
                childGeographicals[i - 1] = childGeographical;
            }
            setChildren(geographical, childGeographicals);
        }
        return geographical;
    }

    private static boolean canHaveChildren(Geographical parent,
            GeographicalScope scope)
    {
        if (scope != null)
        {
            if (isLocation(parent))
            {
                return canHaveLocationChildren(scope);
            }
            else if (isCity(parent))
            {
                return canHaveCityChildren(scope);
            }
            else if (isCountryState(parent))
            {
                return canHaveCountryStateChildren(scope);
            }
            else if (isRegion(parent))
            {
                return canHaveRegionChildren(scope);
            }
            else if (isCountry(parent))
            {
                return canHaveCountryChildren(scope);
            }
        }
        return false;
    }

    public static Geographical createChild(Geographical parent)
    {
        Geographical child = null;
        if (isLocation(parent))
        {
            child = new Location();
        }
        else if (isCity(parent))
        {
            child = new Location();
        }
        else if (isCountryState(parent))
        {
            child = new City();
        }
        else if (isRegion(parent))
        {
            child = new Country();
        }
        else if (isCountry(parent))
        {
            child = new CountryState();
        }
        else if (parent == null)
        {
            child = new Region();
        }
        return child;
    }

    public static Geographical[] createChildrenArray(Geographical parent,
            int length)
    {
        Geographical[] child = null;
        if (isLocation(parent))
        {
            child = new Location[length];
        }
        else if (isCity(parent))
        {
            child = new Location[length];
        }
        else if (isCountryState(parent))
        {
            child = new City[length];
        }
        else if (isRegion(parent))
        {
            child = new Country[length];
        }
        else if (isCountry(parent))
        {
            child = new CountryState[length];
        }
        else if (parent == null)
        {
            child = new Region[length];
        }
        return child;
    }

    private static Geographical[] getChildren(Geographical parent)
    {
        if (isRegion(parent))
        {
            return ((Region) parent).getChildren();
        }
        else if (isCountry(parent))
        {
            return ((Country) parent).getChildren();
        }
        else if (isCountryState(parent))
        {
            return ((CountryState) parent).getChildren();
        }
        else if (isCity(parent))
        {
            return ((City) parent).getChildren();
        }
        else if (isLocation(parent))
        {
            return ((Location) parent).getChildren();
        }
        return null;
    }

    private static void setChildren(Geographical parent, Geographical[] children)
    {
        if (isLocation(parent) || isCity(parent) || isCountryState(parent)
                || isRegion(parent) || isCountry(parent))
        {
            parent.setChildren(children);
        }
    }

    private static GeographicalScope getChildrenScope(Geographical parent,
            GeographicalScope scope)
    {
        if (isLocation(parent))
        {
            return scope.getChildren();
        }
        else if (isCity(parent))
        {
            return scope.getChildren();
        }
        else if (isCountryState(parent))
        {
            return scope.getChildren();
        }
        else if (isRegion(parent))
        {
            return scope.getChildren();
        }
        else if (isCountry(parent))
        {
            return scope.getChildren();
        }
        return null;
    }

    private static void setChildrenScope(Geographical parent,
            GeographicalScope scope, GeographicalScope childrenScope)
    {
        if (isLocation(parent))
        {
            scope.setChildren(childrenScope);
        }
        else if (isCity(parent))
        {
            scope.setChildren(childrenScope);
        }
        else if (isCountryState(parent))
        {
            scope.setChildren(childrenScope);
        }
        else if (isRegion(parent))
        {
            scope.setChildren(childrenScope);
        }
        else if (isCountry(parent))
        {
            scope.setChildren(childrenScope);
        }
    }

    private static boolean canHaveRegionChildren(GeographicalScope scope)
    {
        return scope.getChildren() != null;
    }

    private static boolean canHaveCountryChildren(GeographicalScope scope)
    {
        return scope.getChildren() != null;
    }

    private static boolean canHaveCountryStateChildren(GeographicalScope scope)
    {
        return scope.getChildren() != null;
    }

    private static boolean canHaveCityChildren(GeographicalScope scope)
    {
        return scope.getChildren() != null;
    }

    private static boolean canHaveLocationChildren(GeographicalScope scope)
    {
        return scope.getChildren() != null;
    }

    private static boolean isRegion(Geographical geo)
    {
        return geo instanceof Region;
    }

    private static boolean isCountry(Geographical geo)
    {
        return geo instanceof Country;
    }

    private static boolean isCountryState(Geographical geo)
    {
        return geo instanceof CountryState;
    }

    private static boolean isCity(Geographical geo)
    {
        return geo instanceof City;
    }

    private static boolean isLocation(Geographical geo)
    {
        return geo instanceof Location;
    }

    //update fields
    public static void updateFixtureFields(Geographical geographical,
            GeographicalScope scope) throws Exception
    {
        updateFields(geographical, random(), scope);
    }

    private static void updateFields(Geographical geographical, int index,
            GeographicalScope scope) throws Exception
    {
        HelperTestUtilities
            .setByIntrospection(geographical, index, skipSetters);
        if (canHaveChildren(geographical, scope))
        {
            GeographicalScope childrenScope = getChildrenScope(geographical,
                scope);
            Geographical[] children = getChildren(geographical);
            for (int i = 0; i < children.length; i++)
            {
                updateFixtureFields(children[i], childrenScope);
            }
        }
    }

    //update structure
    public static void updateFixtureStructure(Geographical geographical,
            int deltaNbGeographicalByLevel, int nbGeographicalByLevel,
            GeographicalScope scope) throws Exception
    {
        updateStructure(geographical, deltaNbGeographicalByLevel,
            nbGeographicalByLevel, scope);
    }

    private static void updateStructure(Geographical geographical,
            int deltaNbGeographicalByLevel, int nbGeographicalByLevel,
            GeographicalScope scope) throws Exception
    {
        if (canHaveChildren(geographical, scope))
        {
            GeographicalScope childrenScope = getChildrenScope(geographical,
                scope);
            if (deltaNbGeographicalByLevel > 0)
            {//add new elements
                Geographical newGeographical = null;
                Geographical[] newGeographicals = createChildrenArray(
                    geographical, deltaNbGeographicalByLevel);
                for (int i = 0; i < deltaNbGeographicalByLevel; i++)
                {
                    newGeographical = createGeographical(random(),
                        nbGeographicalByLevel, geographical, childrenScope);
                    newGeographical.setParent(geographical);
                    newGeographicals[i] = newGeographical;
                }

                Geographical[] children1 = getChildren(geographical);
                ArrayList temp = mergeArrays(children1, newGeographicals);
                Geographical[] totalGeographical = createChildrenArray(
                    geographical, temp.size());
                temp.toArray(totalGeographical);

                setChildren(geographical, totalGeographical);
            }
            Geographical[] children2 = getChildren(geographical);
            for (int i = 0; i < children2.length; i++)
            {
                updateFixtureStructure(children2[i],
                    deltaNbGeographicalByLevel, nbGeographicalByLevel,
                    childrenScope);
            }
        }
    }

    public static GeographicalScope createScope(int maxLevels)
    {
        GeographicalScope scope = new GeographicalScope();
        if (maxLevels > 1)
        {
            GeographicalScope countryScope = new GeographicalScope();
            countryScope.setParent(scope);
            scope.setChildren(countryScope);
            if (maxLevels > 2)
            {
                GeographicalScope countryStateScope = new GeographicalScope();
                countryStateScope.setParent(countryScope);
                countryScope.setChildren(countryStateScope);
                if (maxLevels > 3)
                {
                    GeographicalScope cityScope = new GeographicalScope();
                    cityScope.setParent(countryStateScope);
                    countryStateScope.setChildren(cityScope);
                    if (maxLevels > 4)
                    {
                        GeographicalScope locationScope = new GeographicalScope();
                        locationScope.setParent(cityScope);
                        cityScope.setChildren(locationScope);
                        if (maxLevels > 5)
                        {
                            GeographicalScope childLocationScope = new GeographicalScope();
                            childLocationScope.setParent(locationScope);
                            locationScope.setChildren(childLocationScope);
                        }
                    }
                }
            }
        }
        return scope;
    }

    public static void compare(Geographical geo1, Geographical geo2,
            GeographicalScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(geo1, geo2);

        if (scope == null)
            return;

        if (canHaveChildren(geo1, scope))
        {
            GeographicalScope childrenScope = getChildrenScope(geo1, scope);
            if (childrenScope != null)
            {
                childrenScope.setParent(null);
            }

            GeographicalFixture.compare(getChildren(geo1), getChildren(geo2),
                childrenScope);
        }

        if (scope.getParent() != null)
        {
            if (geo1.getParent() == null)
                Assert.fail("RPMObject in scope is null");

            GeographicalScope s = (GeographicalScope)scope.getParent();
            if (s != null)
            {
                setChildrenScope((Geographical) geo1.getParent(), scope, null);
            }

            GeographicalFixture.compare((Geographical) geo1.getParent(),
                (Geographical) geo2.getParent(), s);
        }
    }

    public static void compare(Geographical[] pArray1, Geographical[] pArray2,
            GeographicalScope scope) throws AssertionFailedError, Exception
    {
        sortWithComparator(pArray1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(pArray2, RPMObjectNameAndIDComparator.INSTANCE);

        RPMObjectFixture.compare(pArray1, pArray2);
        if (pArray1 == null)
            return;

        for (int i = 0; i < pArray1.length; i++)
        {
            GeographicalFixture.compare(pArray1[i], pArray2[i], scope);
        }
    }

    //TODO move elsewhere
    private static ArrayList mergeArrays(Object[] array1, Object[] array2)
    {
        ArrayList temp = new ArrayList();
        addArray(temp, array1);
        addArray(temp, array2);
        return temp;
    }

    private static void addArray(ArrayList result, Object[] array)
    {
        if (array != null && array.length != 0)
        {
            for (int i = 0; i < array.length; i++)
            {
                result.add(array[i]);
            }
        }
    }


}