/*
 * Created on Jul 19, 2005
 *
 */
package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Department;
import com.ibm.rpm.applicationadministration.containers.Group;
import com.ibm.rpm.applicationadministration.containers.Office;
import com.ibm.rpm.applicationadministration.containers.Organization;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.scope.OrganizationalScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;

/**
 * @author karim
 *  
 */
public class OrganizationalFixture extends RPMObjectFixture
{

    private static final int DEFAULT_NB_LEVELS = 4;

    public static Organizational createSaveFixture(int contactGroupsByLevel,
            ClientTestController ctc) throws Exception
    {

        OrganizationalScope scope = createScope(DEFAULT_NB_LEVELS);
        Organizational organizational = createFixture(contactGroupsByLevel,
            scope);

        //Save Organizational
        organizational = (Organizational) save(ctc, organizational, scope);

        return organizational;

    }

    public static Organizational createFixture(int organizationalsByLevel,
            OrganizationalScope scope) throws Exception
    {
        Organizational organizational = createOrganizational(random(),
            organizationalsByLevel, null, scope);
        return organizational;
    }

    private static Organizational createOrganizational(int index,
            int organizationalsByLevel, Organizational parent,
            OrganizationalScope scope) throws Exception
    {
        Organizational organizational = createChild(parent);
        HelperTestUtilities.setByIntrospection(organizational, index,
            skipSetters);
        organizational.setParent(parent);
        if (canHaveChildren(organizational, scope))
        {
            OrganizationalScope childrenScope = getChildrenScope(
                organizational, scope);
            Organizational childOrganizational = null;
            Organizational[] childOrganizationals = createChildrenArray(
                organizational, organizationalsByLevel);
            for (int i = 1; i <= organizationalsByLevel; i++)
            {
                childOrganizational = createOrganizational(random(),
                    organizationalsByLevel, organizational, childrenScope);
                childOrganizationals[i - 1] = childOrganizational;
            }
            setChildren(organizational, childOrganizationals);
        }
        return organizational;
    }

    private static boolean canHaveChildren(Organizational parent,
            OrganizationalScope scope)
    {
        if (scope != null)
        {
            if (isGroup(parent))
            {
                return canHaveGroupChildren(scope);
            }
            else if (isDepartment(parent))
            {
                return canHaveDepartmentChildren(scope);
            }
            else if (isOrganization(parent))
            {
                return canHaveOrganizationChildren(scope);
            }
        }
        return false;
    }

    public static Organizational createChild(Organizational parent)
    {
        Organizational child = null;
        if (isGroup(parent))
        {
            child = new Office();
        }
        else if (isDepartment(parent))
        {
            child = new Group();
        }
        else if (isOrganization(parent))
        {
            child = new Department();
        }
        else if (parent == null)
        {
            child = new Organization();
        }
        return child;
    }

    public static Organizational[] createChildrenArray(Organizational parent,
            int length)
    {
        Organizational[] child = null;
        if (isGroup(parent))
        {
            child = new Office[length];
        }
        else if (isDepartment(parent))
        {
            child = new Group[length];
        }
        else if (isOrganization(parent))
        {
            child = new Department[length];
        }
        else if (parent == null)
        {
            child = new Organization[length];
        }
        return child;
    }

    private static Organizational[] getChildren(Organizational parent)
    {
        return parent.getChildren();
    }

    private static void setChildren(Organizational parent,
            Organizational[] children)
    {
        	parent.setChildren(children);
    }

    private static OrganizationalScope getChildrenScope(Organizational parent,
            OrganizationalScope scope)
    {
        return scope.getChildren();
    }

    private static void setChildrenScope(Organizational parent,
            OrganizationalScope scope, OrganizationalScope childrenScope)
    {
        scope.setChildren(childrenScope);
    }

    private static boolean canHaveOrganizationChildren(OrganizationalScope scope)
    {
        return scope.getChildren() != null;
    }

    private static boolean canHaveDepartmentChildren(OrganizationalScope scope)
    {
        return scope.getChildren() != null;
    }

    private static boolean canHaveGroupChildren(OrganizationalScope scope)
    {
        return scope.getChildren() != null;
    }

    private static boolean isOrganization(Organizational org)
    {
        return org instanceof Organization;
    }

    private static boolean isDepartment(Organizational org)
    {
        return org instanceof Department;
    }

    private static boolean isGroup(Organizational org)
    {
        return org instanceof Group;
    }


    //update fields
    public static void updateFixtureFields(Organizational organizational,
            OrganizationalScope scope) throws Exception
    {
        updateFields(organizational, random(), scope);
    }

    private static void updateFields(Organizational organizational, int index,
            OrganizationalScope scope) throws Exception
    {
        HelperTestUtilities.setByIntrospection(organizational, index,
            skipSetters);
        if (canHaveChildren(organizational, scope))
        {
            OrganizationalScope childrenScope = getChildrenScope(
                organizational, scope);
            Organizational[] children = getChildren(organizational);
            for (int i = 0; i < children.length; i++)
            {
                updateFixtureFields(children[i], childrenScope);
            }
        }
    }

    //update structure
    public static void updateFixtureStructure(Organizational organizational,
            int deltaNbOrganizationalByLevel, int nbOrganizationalByLevel,
            OrganizationalScope scope) throws Exception
    {
        updateStructure(organizational, deltaNbOrganizationalByLevel,
            nbOrganizationalByLevel, scope);
    }

    private static void updateStructure(Organizational organizational,
            int deltaNbOrganizationalByLevel, int nbOrganizationalByLevel,
            OrganizationalScope scope) throws Exception
    {
        if (canHaveChildren(organizational, scope))
        {
            OrganizationalScope childrenScope = getChildrenScope(
                organizational, scope);
            if (deltaNbOrganizationalByLevel > 0)
            {//add new elements
                Organizational newOrganizational = null;
                Organizational[] newOrganizationals = createChildrenArray(
                    organizational, deltaNbOrganizationalByLevel);
                for (int i = 0; i < deltaNbOrganizationalByLevel; i++)
                {
                    newOrganizational = createOrganizational(random(),
                        nbOrganizationalByLevel, organizational, childrenScope);
                    newOrganizational.setParent(organizational);
                    newOrganizationals[i] = newOrganizational;
                }

                Organizational[] children1 = getChildren(organizational);
                ArrayList temp = mergeArrays(children1, newOrganizationals);
                Organizational[] totalOrganizational = createChildrenArray(
                    organizational, temp.size());
                temp.toArray(totalOrganizational);

                setChildren(organizational, totalOrganizational);
            }
            Organizational[] children2 = getChildren(organizational);
            for (int i = 0; i < children2.length; i++)
            {
                updateFixtureStructure(children2[i],
                    deltaNbOrganizationalByLevel, nbOrganizationalByLevel,
                    childrenScope);
            }
        }
    }

    public static OrganizationalScope createScope(int maxLevels)
    {
        OrganizationalScope scope = new OrganizationalScope();
        if (maxLevels > 1)
        {
            OrganizationalScope departementScope = new OrganizationalScope();
            departementScope.setParent(scope);
            scope.setChildren(departementScope);
            if (maxLevels > 2)
            {
                OrganizationalScope groupScope = new OrganizationalScope();
                groupScope.setParent(departementScope);
                departementScope.setChildren(groupScope);
                if (maxLevels > 3)
                {
                    OrganizationalScope officeScope = new OrganizationalScope();
                    officeScope.setParent(groupScope);
                    groupScope.setChildren(officeScope);
                }
            }
        }
        return scope;
    }

    public static void compare(Organizational org1, Organizational org2,
            OrganizationalScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(org1, org2);

        if (scope == null)
            return;

        if (canHaveChildren(org1, scope))
        {
            OrganizationalScope childrenScope = getChildrenScope(org1, scope);
            if (childrenScope != null)
            {
                childrenScope.setParent(null);
            }

            OrganizationalFixture.compare(getChildren(org1), getChildren(org2),
                childrenScope);
        }

        if (scope.getParent() != null)
        {
            if (org1.getParent() == null)
                Assert.fail("RPMObject in scope is null");
            
            OrganizationalScope s = (OrganizationalScope)scope.getParent();
            if (s != null)
            {
                setChildrenScope((Organizational) org1.getParent(), scope, null);
            }

            OrganizationalFixture.compare((Organizational) org1.getParent(),
                (Organizational) org2.getParent(), s);
        }
    }

    public static void compare(Organizational[] pArray1,
            Organizational[] pArray2, OrganizationalScope scope)
            throws AssertionFailedError, Exception
    {
        sortWithComparator(pArray1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(pArray2, RPMObjectNameAndIDComparator.INSTANCE);

        RPMObjectFixture.compare(pArray1, pArray2);
        if (pArray1 == null)
            return;

        for (int i = 0; i < pArray1.length; i++)
        {
            OrganizationalFixture.compare(pArray1[i], pArray2[i], scope);
        }
    }

    private static ArrayList mergeArrays(Object[] array1, Object[] array2)
    {
        ArrayList temp = new ArrayList();
        addArray(temp, array1);
        addArray(temp, array2);
        return temp;
    }

    private static void addArray(ArrayList result, Object[] array)
    {
        if (array != null && array.length != 0)
        {
            for (int i = 0; i < array.length; i++)
            {
                result.add(array[i]);
            }
        }
    }


}