/*
 * Created on Jul 12, 2005
 */
package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.applicationadministration.scope.PoolDomainScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;

/**
 * @author gosselin
 */
public class PoolDomainFixture extends RPMObjectFixture
{
	private static final int DEFAULT_NB_LEVELS = 4;
	
    public static PoolDomain createSaveFixture(int nbPoolDomainsByLevel, ClientTestController ctc) throws Exception {

    	PoolDomainScope scope = createScope(1, DEFAULT_NB_LEVELS);
    	PoolDomain poolDomain = createFixture(nbPoolDomainsByLevel, scope);

        //Save PoolDomain
    	poolDomain = (PoolDomain) save(ctc, poolDomain, scope);

        return poolDomain;

    }
	
	public static PoolDomain createFixture(int nbPoolDomainsByLevel, PoolDomainScope scope) throws Exception
    {
    	return createPoolDomain(random(), nbPoolDomainsByLevel, null, scope);
    }
	
    private static PoolDomain createPoolDomain(int index, int nbPoolDomainsByLevel, PoolDomain parentPoolDomain, PoolDomainScope scope)throws Exception
	{
    	PoolDomain poolDomain = new PoolDomain();
    	HelperTestUtilities.setByIntrospection(poolDomain, index, skipSetters);
    	poolDomain.setParent(parentPoolDomain);
    	if(scope!=null && scope.getPoolDomains()!=null){
    		PoolDomain childPoolDomain = null;
    		PoolDomain[] childpoolDomains = new PoolDomain[nbPoolDomainsByLevel]; 
    		for(int i=1;i<=nbPoolDomainsByLevel;i++){
    			childPoolDomain = createPoolDomain(random(), nbPoolDomainsByLevel, poolDomain, scope.getPoolDomains());
    			childpoolDomains[i-1] = childPoolDomain;
    		}
    		poolDomain.setPoolDomains(childpoolDomains);
    	}
    	return poolDomain;
    }
    
    //update fields
    public static void updateFixtureFields(PoolDomain poolDomain, PoolDomainScope scope) throws Exception
    {
    	updateFields(poolDomain, random(), scope);
    }
    
    private static void updateFields(PoolDomain poolDomain, int index, PoolDomainScope scope) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(poolDomain, index, skipSetters);
		if (scope != null && scope.getPoolDomains() != null) {
			for(int i=0;i<poolDomain.getPoolDomains().length;i++){
				updateFixtureFields(poolDomain.getPoolDomains()[i], scope.getPoolDomains());
			}
		}
    }
    
    //update structure
    public static void updateFixtureStructure(PoolDomain poolDomain, int deltaNbPoolDomainsByLevel, int nbPoolDomainsByLevel, PoolDomainScope scope)throws Exception{
    	updateStructure(poolDomain, deltaNbPoolDomainsByLevel, nbPoolDomainsByLevel, scope);
    }
    
    private static void updateStructure(PoolDomain poolDomain, int deltaNbPoolDomainsByLevel, int nbPoolDomainsByLevel, PoolDomainScope scope)throws Exception{
		if (scope != null && scope.getPoolDomains() != null) {
			if(deltaNbPoolDomainsByLevel>0){//add new elements
				PoolDomain newPoolDomain = null;
				PoolDomain[] newPoolDomains = new PoolDomain[deltaNbPoolDomainsByLevel];
		    	for(int i=0;i<deltaNbPoolDomainsByLevel;i++){
		    		newPoolDomain = createFixture(nbPoolDomainsByLevel, scope.getPoolDomains());
		    		newPoolDomain.setParent(poolDomain);
		    		newPoolDomains[i] = newPoolDomain;
		    	}
		    	
		    	ArrayList temp = mergeArrays(poolDomain.getPoolDomains(), newPoolDomains);
		    	PoolDomain[] totalPoolDomains = new PoolDomain[temp.size()];
		    	temp.toArray(totalPoolDomains);
		    	
		    	poolDomain.setPoolDomains(totalPoolDomains);		    	
			}
			for(int i=0;i<poolDomain.getPoolDomains().length;i++){
				updateFixtureStructure(poolDomain.getPoolDomains()[i], deltaNbPoolDomainsByLevel, nbPoolDomainsByLevel, scope.getPoolDomains());
			}
		}
    }
	
	
    public static void compare(PoolDomain p1, PoolDomain p2,
            PoolDomainScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(p1, p2);

        if (scope == null)
            return;

        if (scope.getPoolDomains() != null)
        {
            PoolDomainScope s = scope.getPoolDomains();
            if (s != null)
            {
                s.setParent(null);
            }

            PoolDomainFixture.compare(p1.getPoolDomains(), p2.getPoolDomains(), s);
        }

        if (scope.getParent() != null)
        {
            PoolDomainScope s = (PoolDomainScope)scope.getParent();
            if (s != null)
            {
                s.setPoolDomains(null);
            }

            PoolDomainFixture.compare((PoolDomain) p1.getParent(),
                    (PoolDomain) p2.getParent(), s);
        }
    }

    public static void compare(PoolDomain[] pArray1, PoolDomain[] pArray2,
            PoolDomainScope scope) throws AssertionFailedError, Exception
    {
    	sortWithComparator(pArray1, RPMObjectNameAndIDComparator.INSTANCE);
    	sortWithComparator(pArray2, RPMObjectNameAndIDComparator.INSTANCE); 
    	
        RPMObjectFixture.compare(pArray1, pArray2);
        if(pArray1 == null) return;

        for (int i = 0; i < pArray1.length; i++)
        {
            PoolDomainFixture.compare(pArray1[i], pArray2[i], scope);
        }
    }
    
    public static PoolDomainScope createScope(int level, int maxLevels){
    	PoolDomainScope scope = new PoolDomainScope();
    	if(level<maxLevels){
    		level++;
    		PoolDomainScope childScope = createScope(level,maxLevels);
    		childScope.setParent(scope);
    		scope.setPoolDomains(childScope);
    	}
    	return scope;
    }
    
    private static ArrayList mergeArrays(Object[] array1, Object[] array2){
    	ArrayList temp = new ArrayList();
    	addArray(temp, array1);
    	addArray(temp, array2);
    	return temp;
    }
    
   private static void addArray(ArrayList result, Object[] array){
    	if(array!=null && array.length!=0){
    		for(int i=0;i<array.length;i++){
    			result.add(array[i]);
    		}
    	}    	
    }
    
}
