/*
 * Created on Jul 5, 2005
 */
package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Comparator;
import java.util.GregorianCalendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.CalendarException;
import com.ibm.rpm.applicationadministration.containers.CalendarWeekDay;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.containers.WorkInterval;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.applicationadministration.types.CalendarExceptionType;
import com.ibm.rpm.applicationadministration.types.CalendarWeekDayType;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMContainer;
import com.ibm.rpm.framework.RPMObject;

/**
 * @author gosselin
 */
public class RPMCalendarFixture extends RPMObjectFixture
{

    private static RPMCalendar STATIC_CALENDAR_FIXTURE_1 = null;

    private static RPMCalendar STATIC_CALENDAR_FIXTURE_2 = null;

    private static String XPATH_CALENDAR_NAME_1 = "STATIC_CALENDAR_FIXTURE_1";

    private static String XPATH_CALENDAR_NAME_2 = "STATIC_CALENDAR_FIXTURE_2";

    private static SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm");
    
    public static class CalendarExceptionComparator implements Comparator
    {

        public CalendarExceptionComparator()
        {
            super();
        }

        public int compare(Object o1, Object o2)
        {

            CalendarException obj1 = (CalendarException) o1;
            CalendarException obj2 = (CalendarException) o2;

            Calendar key1 = null;
            Calendar key2 = null;

            if (obj1 != null)
            {
                key1 = obj1.getDay();
            }
            if (obj2 != null)
            {
                key2 = obj2.getDay();
            }

            if (key1 == null && key2 == null)
            {
                return 0;
            }
            if (key1 == null && key2 != null)
            {
                return -1;
            }
            if (key1 != null && key2 == null)
            {
                return 1;
            }

            if (key1.before(key2))
            {
                return -1;
            }
            else if (key1.after(key2))
            {
                return 1;
            }
            else
            {
                return 0;
            }

        }
    }

    /**
     * List of methods that are not called through introspection for comparison
     */
    protected static ArrayList calendarWeekDaySkipSetters,
            calendarWeekDaySkipGetters;

    protected static ArrayList calendarSkipSetters, calendarSkipGetters;

    static
    {

        //Setters
        calendarSkipSetters = new ArrayList();
        calendarSkipSetters.addAll(skipSetters);
        calendarSkipSetters.add("setStartDate");

        //Getters
        calendarSkipGetters = new ArrayList();
        calendarSkipGetters.addAll(skipGetters);
        calendarSkipGetters.add("getStartDate");

        //Setters
        calendarWeekDaySkipSetters = new ArrayList();
        calendarWeekDaySkipSetters.add("setDeleted");
        calendarWeekDaySkipSetters.add("setMinutes");
        calendarWeekDaySkipSetters.add("setName");

        //Getters
        calendarWeekDaySkipGetters = new ArrayList();
        calendarWeekDaySkipGetters.add("getMinutes");
        calendarWeekDaySkipGetters.add("getDeleted");
        calendarWeekDaySkipGetters.add("getName");

    }

    /**
     * Create an RPMCalendar with no CalendarWeekDayTypes but including a parent
     * if it is in the scope
     */
    public static RPMCalendar createFixture(CalendarScope scope)
            throws Exception
    {
        RPMCalendar mainCalendar = createCalendar(new Double(
            Math.random() * 100).intValue(), scope);

        return mainCalendar;
    }

    public static RPMCalendar createFixture(CalendarScope scope,
            int nbChildrenByLevel, int nbExceptionByLevel) throws Exception
    {
        RPMCalendar mainCalendar = createCalendar(new Double(
            Math.random() * 100).intValue(), scope, nbChildrenByLevel,
            nbExceptionByLevel);

        return mainCalendar;
    }

    public static RPMCalendar createSaveFixture(CalendarScope scope,
            ClientTestController ctc) throws Exception
    {

        RPMCalendar mainCalendar = createFixture(scope);

        ctc.addObjectToDelete(mainCalendar);

        return (RPMCalendar) save(ctc, mainCalendar, scope);
    }

    /**
     * Use this fixture if you want to test the update of the reference to a
     * calendar.
     * 
     * @param ctc
     * @return
     * @throws Exception
     */
    public static RPMCalendar retrieveStaticFixtureForInserts(
            ClientTestController ctc) throws Exception
    {

        RPMCalendar cal = STATIC_CALENDAR_FIXTURE_1;

        if (cal == null)
        {
            RPMObject[] rpmObjects = ctc.loadFromXpath("/RPMCalendar[name = '"
                    + XPATH_CALENDAR_NAME_1 + "']", null);

            if (rpmObjects != null && rpmObjects[0] != null)
            {
                cal = (RPMCalendar) rpmObjects[0];
                STATIC_CALENDAR_FIXTURE_1 = cal;
            }
        }

        if (cal == null)
        {

            cal = new RPMCalendar();

            cal.setName(XPATH_CALENDAR_NAME_1);
            cal.setHoursPerDay(new Double(8));
            cal.setHoursPerWeek(new Double(40));

            cal = (RPMCalendar) saveWithoutDelete(ctc, cal, null);
        }

        return cal;

    }

    /**
     * Use this fixture if you want to test the update of the reference to a
     * calendar.
     * 
     * @param ctc
     * @return
     * @throws Exception
     */
    public static RPMCalendar retrieveStaticFixtureForUpdates(
            ClientTestController ctc) throws Exception
    {

        RPMCalendar cal = STATIC_CALENDAR_FIXTURE_2;

        if (cal == null)
        {
            RPMObject[] rpmObjects = ctc.loadFromXpath("/RPMCalendar[name = '"
                    + XPATH_CALENDAR_NAME_2 + "']", null);

            if (rpmObjects != null && rpmObjects[0] != null)
            {
                cal = (RPMCalendar) rpmObjects[0];
                STATIC_CALENDAR_FIXTURE_2 = cal;
            }
        }

        if (cal == null)
        {

            cal = new RPMCalendar();

            cal.setName(XPATH_CALENDAR_NAME_2);
            cal.setHoursPerDay(new Double(7));
            cal.setHoursPerWeek(new Double(35));

            cal = (RPMCalendar) saveWithoutDelete(ctc, cal, null);
        }

        return cal;

    }

    public static RPMCalendar createSaveDefaultCalendar(CalendarScope scope,
            ClientTestController ctc) throws Exception
    {
        RPMCalendar mainCalendar = createDefaultCalendar();
        ctc.addObjectToDelete(mainCalendar);
        return (RPMCalendar) save(ctc, mainCalendar, scope);
    }

    /**
     * Create a RPMCalendar with no CalendarWeekDayTypes but including a parent
     * if it is in the scope
     */
    private static RPMCalendar createCalendar(int index, CalendarScope scope)
            throws Exception
    {
        RPMCalendar calendar = new RPMCalendar();

        HelperTestUtilities.setByIntrospection(calendar, index,
            calendarSkipSetters);

        calendar.setHoursPerDay(new Double(
            (Math.floor(Math.random() * 24.0) * 4.0) / 4.0));
        calendar.setHoursPerWeek(new Double(
            (Math.floor(Math.random() * 168.0) * 4.0) / 4.0));

        if (scope != null && scope.getParent() != null)
        {
            RPMCalendar parent = createCalendar(index + 5, null);
            parent.setCalendars(new RPMCalendar[]
            {
                calendar
            });
            calendar.setParent(parent);
        }

        return calendar;
    }

    /**
     * Create a RPMCalendar with all related containers excluding parent and
     * children RPMCalendar
     */
    public static RPMCalendar createCalendar(int index, CalendarScope scope,
            int nbChildrenByLevel, int nbExceptionByLevel) throws Exception
    {
        RPMCalendar calendar = new RPMCalendar();
        HelperTestUtilities.setByIntrospection(calendar, index,
            calendarSkipSetters);

        calendar.setHoursPerDay(new Double(
            (Math.floor(Math.random() * 24.0) * 4.0) / 4.0));
        calendar.setHoursPerWeek(new Double(
            (Math.floor(Math.random() * 168.0) * 4.0) / 4.0));

        // Creates standardMonday
        CalendarWeekDay standardMonday = createCalendarDay(calendar, true,
            CalendarWeekDay.class);
        calendar.setStandardMonday(standardMonday);

        // Creates standardTuesday
        CalendarWeekDay standardTuesday = createCalendarDay(calendar, true,
            CalendarWeekDay.class);
        calendar.setStandardTuesday(standardTuesday);

        // Creates standardWednesday
        CalendarWeekDay standardWednesday = createCalendarDay(calendar, true,
            CalendarWeekDay.class);
        calendar.setStandardWednesday(standardWednesday);

        // Creates StandardThursday
        CalendarWeekDay standardThursday = createCalendarDay(calendar, true,
            CalendarWeekDay.class);
        calendar.setStandardThursday(standardThursday);

        // Creates standardFriday
        CalendarWeekDay standardFriday = createCalendarDay(calendar, true,
            CalendarWeekDay.class);
        calendar.setStandardFriday(standardFriday);

        // Creates standardSaturday
        CalendarWeekDay standardSaturday = createCalendarDay(calendar, true,
            CalendarWeekDay.class);
        calendar.setStandardSaturday(standardSaturday);

        // Creates standardSunday
        CalendarWeekDay standardSunday = createCalendarDay(calendar, true,
            CalendarWeekDay.class);
        calendar.setStandardSunday(standardSunday);

        if (scope == null)
            return calendar;

        if (scope.getParent() != null)
        {

            RPMCalendar parent = createCalendar(index + 5, null, 0, 0);
            parent.setCalendars(new RPMCalendar[]
            {
                calendar
            });
            calendar.setParent(parent);

        }

        if (scope.getCalendars() != null)
        {

            RPMCalendar[] childCals = new RPMCalendar[nbChildrenByLevel];
            for (int i = 0; i < nbChildrenByLevel; i++)
            {
                childCals[i] = createCalendar(new Double(Math.random() * 100)
                    .intValue(), scope.getCalendars(), nbChildrenByLevel,
                    nbExceptionByLevel);

            }
            calendar.setCalendars(childCals);

        }

        if (scope.isExceptions())
        {
            CalendarException[] exceptions = new CalendarException[nbExceptionByLevel];
            Calendar day = new GregorianCalendar();
            int year = day.get(Calendar.YEAR);
            int month = day.get(Calendar.MONTH);
            int dayOfMonth = day.get(Calendar.DAY_OF_MONTH);

            for (int i = 0; i < nbExceptionByLevel; i++)
            {
                CalendarException calExcep = new CalendarException();
                calExcep = createCalendarException(calendar, CalendarException.class);
                calExcep.setParent(calendar);
                Calendar excDay = new GregorianCalendar();
                excDay.set(year, month, dayOfMonth++);
                calExcep.setDay(excDay);
                exceptions[i] = calExcep;
            }
            calendar.setExceptions(exceptions);
        }

        return calendar;
    }

    private static RPMCalendar createDefaultCalendar() throws Exception
    {
        RPMCalendar calendar = new RPMCalendar();

        calendar.setName(HelperTestUtilities.createName()
                + new Double(Math.random() * 100).intValue());
        calendar.setHoursPerDay(new Double(8));
        calendar.setHoursPerWeek(new Double(40));

        CalendarWeekDay defaultWD = new CalendarWeekDay();
        defaultWD.setParent(calendar);
        defaultWD.setType(CalendarWeekDayType.UseDefault);

        CalendarWeekDay non_workingWD = new CalendarWeekDay();
        non_workingWD.setParent(calendar);
        non_workingWD.setType(CalendarWeekDayType.NonWorking);

        // Creates standardMonday
        CalendarWeekDay standardMonday = defaultWD;
        calendar.setStandardMonday(standardMonday);

        // Creates standardTuesday
        CalendarWeekDay standardTuesday = defaultWD;
        calendar.setStandardTuesday(standardTuesday);

        // Creates standardWednesday
        CalendarWeekDay standardWednesday = defaultWD;
        calendar.setStandardWednesday(standardWednesday);

        // Creates StandardThursday
        CalendarWeekDay standardThursday = defaultWD;
        calendar.setStandardThursday(standardThursday);

        // Creates standardFriday
        CalendarWeekDay standardFriday = defaultWD;
        calendar.setStandardFriday(standardFriday);

        // Creates standardSaturday
        CalendarWeekDay standardSaturday = non_workingWD;
        calendar.setStandardSaturday(standardSaturday);

        // Creates standardSunday
        CalendarWeekDay standardSunday = non_workingWD;
        calendar.setStandardSunday(standardSunday);

        return calendar;
    }

    /**
     * Update an existing calendar fixture
     */
    public static RPMCalendar updateFixture(RPMCalendar calendar,
            CalendarScope scope) throws Exception
    {

        int nbChildrenByLevel = 0;
        int nbExceptionByLevel = 0;

        HelperTestUtilities.setByIntrospection(calendar, new Double(Math
            .random() * 100).intValue(), calendarSkipSetters);

        calendar.setHoursPerDay(new Double(
            (Math.floor(Math.random() * 24.0) * 4.0) / 4.0));
        calendar.setHoursPerWeek(new Double(
            (Math.floor(Math.random() * 168.0) * 4.0) / 4.0));

        // Creates standardMonday
        CalendarWeekDay standardMonday = createCalendarDay(calendar, false,
            CalendarWeekDay.class);
        calendar.setStandardMonday(standardMonday);

        // Creates standardTuesday
        CalendarWeekDay standardTuesday = createCalendarDay(calendar, false,
            CalendarWeekDay.class);
        calendar.setStandardTuesday(standardTuesday);

        // Creates standardWednesday
        CalendarWeekDay standardWednesday = createCalendarDay(calendar, false,
            CalendarWeekDay.class);
        calendar.setStandardWednesday(standardWednesday);

        // Creates StandardThursday
        CalendarWeekDay standardThursday = createCalendarDay(calendar, false,
            CalendarWeekDay.class);
        calendar.setStandardThursday(standardThursday);

        // Creates standardFriday
        CalendarWeekDay standardFriday = createCalendarDay(calendar, false,
            CalendarWeekDay.class);
        calendar.setStandardFriday(standardFriday);

        // Creates standardSaturday
        CalendarWeekDay standardSaturday = createCalendarDay(calendar, false,
            CalendarWeekDay.class);
        calendar.setStandardSaturday(standardSaturday);

        // Creates standardSunday
        CalendarWeekDay standardSunday = createCalendarDay(calendar, false,
            CalendarWeekDay.class);
        calendar.setStandardSunday(standardSunday);

        if (scope == null)
            return calendar;

        if (scope.getParent() != null)
        {
            updateFixture((RPMCalendar) calendar.getParent(), null);
        }

        if (calendar.getCalendars() != null)
        {
            nbChildrenByLevel = calendar.getCalendars().length;
        }

        if (calendar.getCalendars() != null)
        {
            nbExceptionByLevel = calendar.getExceptions().length;
        }

        if (scope.getCalendars() != null)
        {

            RPMCalendar[] childCals = calendar.getCalendars();

            RPMCalendar[] updatedChildCals = new RPMCalendar[childCals.length * 2];

            //Delete first calendar
            childCals[0].setDeleted(Boolean.TRUE);
            updatedChildCals[0] = childCals[0];

            //Update other calendars
            for (int i = 1; i < childCals.length; i++)
            {
                updatedChildCals[i] = updateFixture(childCals[i], scope
                    .getCalendars());
            }

            //Add a null object to the array
            updatedChildCals[childCals.length] = null;

            //Add new calendars
            for (int i = 1; i < childCals.length; i++)
            {
                updatedChildCals[childCals.length + i] = createFixture(scope
                    .getCalendars(), nbChildrenByLevel, nbExceptionByLevel);
            }

            calendar.setCalendars(updatedChildCals);

        }

        if (scope.isExceptions())
        {

            CalendarException[] exceptions = calendar.getExceptions();
            CalendarException[] updatedExceptions = new CalendarException[exceptions.length * 2];

            Calendar day = new GregorianCalendar();
            int year = day.get(Calendar.YEAR);
            int month = day.get(Calendar.MONTH);
            int dayOfMonth = day.get(Calendar.DAY_OF_MONTH);

            //Delete first exception
            exceptions[0].setDeleted(Boolean.TRUE);
            updatedExceptions[0] = exceptions[0];

            //Update other exceptions
            for (int i = 1; i < exceptions.length; i++)
            {

                CalendarException updateCalExcep = new CalendarException();
                updateCalExcep.setDay(exceptions[i].getDay());
                updateCalExcep.setID(exceptions[i].getID());
                updateCalExcep.setParent(calendar);

                updatedExceptions[i] = updateCalExcep;
            }

            //Add a null object to the array
            updatedExceptions[exceptions.length] = null;

            //Remove exceptions and replace with new exceptions
            for (int i = 1; i < exceptions.length; i++)
            {

                CalendarException newCalExcep = new CalendarException();
                newCalExcep = createCalendarException(calendar, CalendarException.class);
                newCalExcep.setParent(calendar);
                Calendar excDay = new GregorianCalendar();
                excDay.set(year, month + 1, dayOfMonth++);
                newCalExcep.setDay(excDay);
                updatedExceptions[exceptions.length + i] = newCalExcep;

            }

            calendar.setExceptions(updatedExceptions);

        }

        return calendar;
    }

    private static CalendarWeekDayType getRandomWeekDayType(
            boolean canBeNonWorking)
    {

        CalendarWeekDayType calendarWeekDayType = null;
        double nb = Math.random() * 5;

        if (nb <= 1)
        {
            calendarWeekDayType = CalendarWeekDayType.UseDefault;
        }
        else if (nb <= 2 && canBeNonWorking)
        {
            calendarWeekDayType = CalendarWeekDayType.NonWorking;
        }
        else
        {
            calendarWeekDayType = CalendarWeekDayType.NonDefaultWorking;
        }

        return calendarWeekDayType;

    }

    private static CalendarExceptionType getRandomExceptionType()
    {

        CalendarExceptionType calendarExceptionType = null;
        double nb = Math.random() * 5;

        if (nb <= 2)
        {
            calendarExceptionType = CalendarExceptionType.NonWorking;
        }
        else
        {
            calendarExceptionType = CalendarExceptionType.NonDefaultWorking;
        }

        return calendarExceptionType;

    }

    private static CalendarWeekDay createCalendarDay(RPMCalendar parent,
            boolean canBeNonWorking, Class dayType)
            throws Exception
    {

        CalendarWeekDayType type = getRandomWeekDayType(canBeNonWorking);

        CalendarWeekDay calWD = (CalendarWeekDay) dayType.getConstructor(null)
            .newInstance(null);
        calWD.setParent(parent);

        if (type == CalendarWeekDayType.UseDefault)
        {

            calWD.setType(CalendarWeekDayType.UseDefault);
            //Default system intervals...

        }
        else if (type == CalendarWeekDayType.NonWorking)
        {

            calWD.setMinutes(new Integer(0));
            calWD.setType(CalendarWeekDayType.NonWorking);

        }
        else
        {

            calWD.setType(CalendarWeekDayType.NonDefaultWorking);

            // Must be created in GMT time
            WorkInterval[] intervals = new WorkInterval[5];
            intervals[0] = createWorkInterval("8:00", "10:00");
            intervals[1] = createWorkInterval("10:15", "12:00");
            intervals[2] = createWorkInterval("12:15", "14:30");
            intervals[3] = createWorkInterval("15:00", "16:00");
            intervals[4] = createWorkInterval("16:30", "17:00");

            calWD.setIntervals(intervals);
            calWD.setMinutes(new Integer(420));
        }

        return calWD;
    }

    public static CalendarException createCalendarException(
            RPMCalendar parent, Class dayType) throws Exception
    {

        CalendarExceptionType type = getRandomExceptionType();

        CalendarException calWD = (CalendarException) dayType.getConstructor(
            null).newInstance(null);
        calWD.setParent(parent);

        if (type == CalendarExceptionType.NonWorking)
        {
            calWD.setMinutes(new Integer(0));
            calWD.setType(CalendarExceptionType.NonWorking);
        }
        else
        {
            calWD.setType(CalendarExceptionType.NonDefaultWorking);

            // Must be created in GMT time
            WorkInterval[] intervals = new WorkInterval[5];
            intervals[0] = createWorkInterval("8:00", "10:00");
            intervals[1] = createWorkInterval("10:15", "12:15");
            intervals[2] = createWorkInterval("12:00", "14:30");
            intervals[3] = createWorkInterval("15:00", "16:00");
            intervals[4] = createWorkInterval("16:30", "17:00");

            calWD.setIntervals(intervals);
            calWD.setMinutes(new Integer(420));
        }

        return calWD;
    }

    public static WorkInterval createWorkInterval(String from, String to)
    {

        WorkInterval workInterval = null;

        if ((from == null) || (to == null))
            Assert.fail("Work Interval values are invalid!");

        workInterval = new WorkInterval();

        workInterval.setFrom(from);
        workInterval.setTo(to);

        return workInterval;

    }

    public static void compare(RPMCalendar c1, RPMCalendar c2,
            CalendarScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(c1, c2, calendarSkipGetters);

        if (scope == null)
            return;

        if (scope.isExceptions())
        {
            compareCalendarExceptions(c1.getExceptions(), c2.getExceptions());
        }

        if (scope.getParent() != null)
        {
            CalendarScope parentScope = (CalendarScope)scope.getParent();
            parentScope.setParent(null);
            compare((RPMCalendar) c1.getParent(), (RPMCalendar) c2.getParent(),
                parentScope);
        }

        if (scope.getCalendars() != null)
        {
            CalendarScope childrenScope = scope.getCalendars();
            childrenScope.setParent(null);
            compare(c1.getCalendars(), c2.getCalendars(), childrenScope);
        }

        // Compares standardMonday
        compareCalendarWeekDay(c1.getStandardMonday(), c2.getStandardMonday());

        // Compares standardTuesday
        compareCalendarWeekDay(c1.getStandardTuesday(), c2.getStandardTuesday());

        // Compares standardWednesday
        compareCalendarWeekDay(c1.getStandardWednesday(), c2
            .getStandardWednesday());

        // Compares StandardThursday
        compareCalendarWeekDay(c1.getStandardThursday(), c2
            .getStandardThursday());

        // Compares standardFriday
        compareCalendarWeekDay(c1.getStandardFriday(), c2.getStandardFriday());

        // Compares standardSaturday
        compareCalendarWeekDay(c1.getStandardSaturday(), c2
            .getStandardSaturday());

        // Compares standardSunday
        compareCalendarWeekDay(c1.getStandardSunday(), c2.getStandardSunday());

    }

    public static void compare(RPMCalendar[] cArray1, RPMCalendar[] cArray2,
            CalendarScope scope) throws AssertionFailedError, Exception
    {

        cArray1 = removeDeletedRPMCalendar(cArray1);

        sortWithComparator(cArray1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(cArray2, RPMObjectNameAndIDComparator.INSTANCE);

        RPMObjectFixture.compare(cArray1, cArray2);

        for (int i = 0; i < cArray1.length; i++)
        {
            compare(cArray1[i], cArray2[i], scope);
        }
    }

    public static void compareCalendarWeekDay(CalendarWeekDay c1,
            CalendarWeekDay c2) throws AssertionFailedError, Exception
    {
        if (c1.getType() == CalendarWeekDayType.UseDefault)
        {
            Assert.assertEquals(c1.getType(), c2.getType());
        }
        else
        {
            Assert.assertEquals(c1.getType(), c2.getType());
            RPMObjectFixture
                .compareWithSkip(c1, c2, calendarWeekDaySkipGetters);
            compareWorkInterval(c1.getIntervals(), c2.getIntervals());
        }
    }

    public static void compareCalendarWeekDayMinutesToIntervalMinutes(RPMCalendar c) throws AssertionFailedError, Exception
    {
        compareCalendarWeekDayMinutesToIntervalMinutes(c.getStandardMonday());
        compareCalendarWeekDayMinutesToIntervalMinutes(c.getStandardTuesday());
        compareCalendarWeekDayMinutesToIntervalMinutes(c.getStandardWednesday());
        compareCalendarWeekDayMinutesToIntervalMinutes(c.getStandardThursday());
        compareCalendarWeekDayMinutesToIntervalMinutes(c.getStandardFriday());
        compareCalendarWeekDayMinutesToIntervalMinutes(c.getStandardSaturday());
        compareCalendarWeekDayMinutesToIntervalMinutes(c.getStandardSunday());
    }
    
    public static void compareCalendarWeekDayMinutesToIntervalMinutes(CalendarWeekDay c) throws AssertionFailedError, Exception
    {
        // Ensure minutes = to the addition of all work intervals
        if (c.getIntervals() != null)
	    {
	        int expectedminutes = (c.getMinutes() != null)? c.getMinutes().intValue(): 0;
	        int actualminutes = 0;
	        for (int i = c.getIntervals().length - 1; i >= 0; --i)
	        {
	            actualminutes += getToFromDifference(c.getIntervals()[i]);
	        }
	        Assert.assertTrue("The number of minutes (" + expectedminutes + ") isn't equivalent to the actual number of minutes in the WorkInterval array: " + actualminutes, actualminutes == expectedminutes);
	    }
    }
    
    private static int getToFromDifference(WorkInterval interval)
    {
        int difference = 0;

        int[] from = getHoursAndMinutes(interval.getFrom());
        int[] to = getHoursAndMinutes(interval.getTo());

        try
        {
            if (from != null && to != null)
            {
            	Calendar fromCal = Calendar.getInstance();
            	fromCal.setTime(dateFormat.parse(interval.getFrom()));

                Calendar toCal = Calendar.getInstance();
                toCal.setTime(dateFormat.parse(interval.getTo()));

                difference = (int) ((toCal.getTimeInMillis() - fromCal.getTimeInMillis())/1000)/60;
            }
        }
        catch (ParseException e)
        {
            //do nothing
        }

        return difference;
    }
    
    private static int[] getHoursAndMinutes(String time){
        
        int[] hoursAndMinutes = null;
        
        if (time != null)
        {

            int index = 0;
            index = time.indexOf(":");

            if (index > 0)
            {

                int hours = Integer.parseInt(time.substring(0, index));
                int minutes = Integer.parseInt(time.substring(index + 1, time
                    .length()));
                
                if (hours >= 0 && hours <= 24 && minutes >= 0 && minutes < 60 && (minutes%15 == 0))
                {
                    hoursAndMinutes = new int[]
                    {
                            hours, minutes
                    };
                }

            }
        }
        
        return hoursAndMinutes;
        
    }
    
    public static void compareCalendarExceptions(CalendarException c1,
            CalendarException c2) throws AssertionFailedError, Exception
    {
        Assert.assertEquals(c1.getType(), c2.getType());
        RPMObjectFixture.compareWithSkip(c1, c2, calendarWeekDaySkipGetters);
        compareWorkInterval(c1.getIntervals(), c2.getIntervals());
    }

    public static void compareCalendarExceptions(CalendarException[] cArray1,
            CalendarException[] cArray2) throws AssertionFailedError, Exception
    {

        sortWithComparator(cArray1, new CalendarExceptionComparator());
        sortWithComparator(cArray2, new CalendarExceptionComparator());

        cArray1 = removeDeletedCalendarExceptions(cArray1);

        RPMObjectFixture.compare(cArray1, cArray2);

        for (int i = 0; i < cArray1.length; i++)
        {
            compareCalendarExceptions(cArray1[i], cArray2[i]);
        }
    }

    public static void compareWorkInterval(RPMContainer[] pArray1,
            RPMContainer[] pArray2) throws AssertionFailedError, Exception
    {
        Assert.assertTrue((pArray1 == null && pArray2 == null)
                || (pArray1 != null && pArray2 != null));
        if (pArray1 == null)
            return;

        Assert.assertEquals(pArray1.length, pArray2.length);
        for (int i = 0; i < pArray1.length; i++)
        {
            compareWorkInterval(pArray1[i], pArray2[i]);
        }
    }

    public static void compareWorkInterval(RPMContainer p1, RPMContainer p2)
            throws AssertionFailedError, Exception
    {
        Assert.assertTrue((p1 == null && p2 == null)
                || (p1 != null && p2 != null));
        if (p1 == null)
            return;

        if (((WorkInterval) p1).getTo() != null && ((WorkInterval) p1).getTo().length() == 4)
            ((WorkInterval) p1).setTo("0" + ((WorkInterval) p1).getTo());
        if (((WorkInterval) p1).getFrom() != null && ((WorkInterval) p1).getFrom().length() == 4)
            ((WorkInterval) p1).setFrom("0" + ((WorkInterval) p1).getFrom());
        if (((WorkInterval) p2).getTo() != null && ((WorkInterval) p2).getTo().length() == 4)
            ((WorkInterval) p2).setTo("0" + ((WorkInterval) p2).getTo());
        if (((WorkInterval) p2).getFrom() != null && ((WorkInterval) p2).getFrom().length() == 4)
            ((WorkInterval) p2).setFrom("0" + ((WorkInterval) p2).getFrom());
        
        HelperTestUtilities.assertEqualsByIntrospection(p1, p2,
            calendarWeekDaySkipGetters);
    }

    public static RPMCalendar[] removeDeletedRPMCalendar(RPMCalendar[] pArray)
    {

        RPMCalendar[] newArray = null;

        ArrayList temp = new ArrayList(Arrays.asList(pArray));

        temp = removeNullAndDeletedRPMObjects(temp);

        if (temp != null && temp.size() > 0)
        {
            newArray = new RPMCalendar[temp.size()];
            temp.toArray(newArray);
        }

        return newArray;

    }

    public static CalendarWeekDay[] removeDeletedCalendarWeekDays(
            CalendarWeekDay[] pArray)
    {

        CalendarWeekDay[] newArray = null;

        ArrayList temp = new ArrayList(Arrays.asList(pArray));

        temp = removeNullAndDeletedRPMObjects(temp);

        if (temp != null && temp.size() > 0)
        {
            newArray = new CalendarWeekDay[temp.size()];
            temp.toArray(newArray);
        }

        return newArray;

    }

    public static CalendarException[] removeDeletedCalendarExceptions(
            CalendarException[] pArray)
    {

        CalendarException[] newArray = null;

        ArrayList temp = new ArrayList(Arrays.asList(pArray));

        temp = removeNullAndDeletedRPMObjects(temp);

        if (temp != null && temp.size() > 0)
        {
            newArray = new CalendarException[temp.size()];
            temp.toArray(newArray);
        }

        return newArray;

    }


}