/*
 * Created on Aug 9, 2005
 *
 */
package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import java.util.ArrayList;
import java.util.Comparator;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.ActionRTF;
import com.ibm.rpm.applicationadministration.containers.AssetRTF;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestRTF;
import com.ibm.rpm.applicationadministration.containers.ClientRTF;
import com.ibm.rpm.applicationadministration.containers.DatafieldRTF;
import com.ibm.rpm.applicationadministration.containers.DefectRTF;
import com.ibm.rpm.applicationadministration.containers.DeliverableRTF;
import com.ibm.rpm.applicationadministration.containers.IssueRTF;
import com.ibm.rpm.applicationadministration.containers.MilestoneRTF;
import com.ibm.rpm.applicationadministration.containers.NoteMinutesRTF;
import com.ibm.rpm.applicationadministration.containers.ProjectRTF;
import com.ibm.rpm.applicationadministration.containers.RequirementRTF;
import com.ibm.rpm.applicationadministration.containers.RiskRTF;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestRTF;
import com.ibm.rpm.applicationadministration.containers.SummaryTaskRTF;
import com.ibm.rpm.applicationadministration.containers.TaskRTF;
import com.ibm.rpm.applicationadministration.containers.WorkProductRTF;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author karim
 *
 */
public class RtfAssignmentFixture extends RPMObjectFixture{
    
	protected static ArrayList localSkipGetters = new ArrayList();
	protected static ArrayList localSkipSetters = new ArrayList();
	static
    {
    	//set skipGetters
    	localSkipGetters.addAll(skipGetters);
    	localSkipGetters.add("getName");
    	
    	//set skipSetters
    	localSkipSetters.addAll(skipSetters);
    	localSkipSetters.add("setName");
    }
	
    public static RtfAssignment[] createAllAssignmentFixtures(RPMObject parent, ClientTestController ctc) throws Exception
    {
    	DatafieldRTF rtf;
    	String datafieldRTFId = null; 
    	RPMObject[] rtfs = createDatafieldRTFs(parent, ctc);
    	ArrayList temp = new ArrayList();
    	for(int i=0;(i < 8) && (i<rtfs.length);i++){
    		rtf = (DatafieldRTF)rtfs[i];
    		if(rtf.getValue()!=null && rtf.getName()!=null){
        		datafieldRTFId = rtfs[i].getID();
        		temp.add(createAssignmentFixture(parent, datafieldRTFId, rtf, ctc));    			
    		}
    	}
    	RtfAssignment[] rtfAssignments = new RtfAssignment[temp.size()];
    	temp.toArray(rtfAssignments);
    	return rtfAssignments;
    }
	
	
    public static RtfAssignment createAssignmentFixture(RPMObject parent, String datafieldRTFId, DatafieldRTF datafieldRTF, ClientTestController ctc) throws Exception
    {
        return createAssignment(parent, datafieldRTFId, datafieldRTF, ctc);
    }

    private static RtfAssignment createAssignment(RPMObject parent, String datafieldRTFId, DatafieldRTF datafieldRTF, ClientTestController ctc) throws Exception
    {
    	RtfAssignment assignment = new RtfAssignment();
    	
   		assignment.setParent(parent);
   		
    	//set the datafieldRTF
   		if(datafieldRTF==null){
   	    	datafieldRTF = createDatafieldRTF(datafieldRTFId, ctc);   			
   		}
    	assignment.setRtf(datafieldRTF);
    	
    	HelperTestUtilities.setByIntrospection(assignment, random(), localSkipSetters);
    	
    	return assignment;
    }
	
    private static DatafieldRTF initRtf(RPMObject parent, ClientTestController ctc)
    {
        DatafieldRTF rtf = null;
        if (parent instanceof ClientCostCenter)
        {
            rtf = new ClientRTF();
        }
        else if (parent instanceof Asset)
        {
            rtf = new AssetRTF();
        }
        else if (parent instanceof DocumentElement)
        {
            rtf = new NoteMinutesRTF();
        }
        else if (parent instanceof ScopeElement)
        {
            rtf = initScopeRtf(parent);
        }
        else if (parent instanceof WorkElement)
        {
            rtf = initWBSRtf(parent, ctc);
        }
        return rtf;
    }
    
    private static DatafieldRTF initWBSRtf(RPMObject parent, ClientTestController ctc)
    {
        if (parent instanceof GenericProject)
        {
            return new ProjectRTF();
        }
        else if (parent instanceof Deliverable)
        {
            return new DeliverableRTF();
        }
        else if (parent instanceof WorkProduct)
        {
            return new WorkProductRTF();
        }
        else if (parent instanceof SummaryTask)
        {
            // Try to load a corresponding ScopeElement, in case it has been transferred from Scope Management
            try
            {
                ScopeElement scopeElement = null;
                if(parent!=null && parent.getID()!=null)
                {
                    scopeElement = new ScopeElement();
                    scopeElement.setID(parent.getID());
                    scopeElement = (ScopeElement) ctc.loadFromID(scopeElement, null);
                }
                
                if (scopeElement == null)
                {
                    return new SummaryTaskRTF();
                }
                if (scopeElement instanceof Action)
                {
                    return new ActionRTF();
                }
                else if (scopeElement instanceof ChangeRequest)
                {
                    return new ChangeRequestRTF();  
                }
                else if (scopeElement instanceof Defect)
                {
                    return new DefectRTF();  
                }
                else if (scopeElement instanceof Issue)
                {
                    return new IssueRTF();  
                }
                else if (scopeElement instanceof Requirement)
                {
                    return new RequirementRTF();  
                }
                else if (scopeElement instanceof Risk)
                {
                    return new RiskRTF();  
                }
                else if (scopeElement instanceof ServiceRequest)
                {
                    return new ServiceRequestRTF();  
                }
            }
            catch(Exception e)
            {
            }
            return new SummaryTaskRTF();
        }
        else if (parent instanceof Milestone)
        {
            return new MilestoneRTF();
        }
        else
        {
            // Try to load a corresponding ScopeElement, in case it has been transferred from Scope Management
            try
            {
                ScopeElement scopeElement = null;
                if(parent!=null && parent.getID()!=null)
                {
                    scopeElement = new ScopeElement();
                    scopeElement.setID(parent.getID());
                    scopeElement = (ScopeElement) ctc.loadFromID(scopeElement, null);
                }
                
                if (scopeElement == null)
                {
                    return new TaskRTF();
                }
                if (scopeElement instanceof Action)
                {
                    return new ActionRTF();
                }
                else if (scopeElement instanceof ChangeRequest)
                {
                    return new ChangeRequestRTF();  
                }
                else if (scopeElement instanceof Defect)
                {
                    return new DefectRTF();  
                }
                else if (scopeElement instanceof Issue)
                {
                    return new IssueRTF();  
                }
                else if (scopeElement instanceof Requirement)
                {
                    return new RequirementRTF();  
                }
                else if (scopeElement instanceof Risk)
                {
                    return new RiskRTF();  
                }
                else if (scopeElement instanceof ServiceRequest)
                {
                    return new ServiceRequestRTF();  
                }
            }
            catch(Exception e)
            {
            }
            return new TaskRTF();
        }
    }
    
    private static DatafieldRTF initScopeRtf(RPMObject parent)
    {
        if(parent instanceof ScopeTask)
        {
            return new TaskRTF();
        }
        else if(parent instanceof ScopeMilestone)
        {
            return new MilestoneRTF();
        }
        else if(parent instanceof ScopeSummaryTask)
        {
            return new SummaryTaskRTF();
        }
        else if(parent instanceof ScopeDeliverable)
        {
            return new DeliverableRTF();
        }
        else if(parent instanceof Defect)
        {
            return new DefectRTF();
        }
        else if(parent instanceof ChangeRequest)
        {
            return new ChangeRequestRTF();
        }
        else if(parent instanceof ServiceRequest)
        {
            return new ServiceRequestRTF();
        }
        else if(parent instanceof Issue)
        {
            return new IssueRTF();
        }
        else if(parent instanceof Action)
        {
            return new ActionRTF();
        }
        else if(parent instanceof Requirement)
        {
            return new RequirementRTF();
        }
        else if(parent instanceof Risk)
        {
            return new RiskRTF();
        }
        else
        {
            return null;
        }
    }
    
    private static RPMObject[] createDatafieldRTFs(RPMObject parent,
            ClientTestController ctc) throws Exception
    {
    	DatafieldRTF rtf = initRtf(parent, ctc);
    	String className = rtf.getClass().getName();
    	String containerName = className.substring(className.lastIndexOf(".")+1); 
    	String xpath = "/"+containerName;//+"[TYPE_ID='"+rtf.retrieveTypeID()+"']";
        return ctc.loadFromXpath(xpath, null);
    }
    
    private static DatafieldRTF createDatafieldRTF(String datafieldRTFId,
            ClientTestController ctc) throws Exception
    {
        DatafieldRTF datafieldRTF = DatafieldFixture
                .createDatafieldRTF(datafieldRTFId);
        datafieldRTF = (DatafieldRTF) ctc.loadFromID(datafieldRTF, null);
        return datafieldRTF;
    }
     
    public static void updateAssignmentFixture(RtfAssignment assignment)throws Exception{
    	updateAssignment(random(), assignment);
    }
    
    private static void updateAssignment(int index, RtfAssignment assignment)throws Exception{
    	HelperTestUtilities.setByIntrospection(assignment, index, localSkipSetters);
    }
	
    private static void compareParents(RPMObject parent1, RPMObject parent2)throws AssertionFailedError,
    Exception{
    	if(parent1 instanceof ClientCostCenter){
        	ClientCostCenterFixture.compare((ClientCostCenter)parent1, (ClientCostCenter)parent2, null);
    	} else if (parent1 instanceof Asset){
    		AssetFixture.compare((Asset)parent1, (Asset)parent2, null);
    	} else if (parent1 instanceof DocumentElement){
    	    DocumentElementFixture.compare((DocumentElement)parent1, (DocumentElement)parent2, null);
    	} else if (parent1 instanceof ScopeElement){
    		ScopeElementFixture.compare((ScopeElement)parent1, (ScopeElement)parent2, null);
    	} else if (parent1 instanceof WorkElement){
    		WorkElementFixture.compare((WorkElement)parent1, (WorkElement)parent2, null);
    	} else {
    		Assert.fail("RtfAssignmentFixture.compareParents : parent1.class "+parent1.getClass()+" not supported by this method.");
    	}
    }
    
    // compareParents required as it might go into infinite loop
    public static void compare(RtfAssignment obj1,
    		RtfAssignment obj2, boolean compareParents) throws AssertionFailedError,
            Exception
    {
    	RPMObjectFixture.compare(obj1, obj2);

    	if (compareParents == true) {
    	    compareParents(obj1.getParent(), obj2.getParent());    
    	}
    	
    	DatafieldFixture.compare(obj1.getRtf(), obj2.getRtf());    	
    }

    public static void compare(RtfAssignment obj1,
    		RtfAssignment obj2) throws AssertionFailedError,
            Exception
    {
        compare(obj1, obj2, true);
    }

    public static void compare(RtfAssignment[] pArray1,
    		RtfAssignment[] pArray2)
            throws AssertionFailedError, Exception
    {
    	
    	sortWithComparator(pArray1, new RPMObjectAttributeComparator());
    	sortWithComparator(pArray2, new RPMObjectAttributeComparator()); 
    	
        RPMObjectFixture.compare(pArray1, pArray2);

        if (pArray1 == null)
            return;

        for (int i = 0; i < pArray1.length; i++)
        {
            compare(pArray1[i],
                    pArray2[i]);
        }
    }
    
    public static RPMObject createParent(RPMObject parent,
            RPMObjectScope parentScope, ClientTestController ctc)
            throws Exception
    {
        RPMObject createdParent = null;

        if (parent instanceof Asset)
        {
            createdParent = AssetFixture.createFixture(
                (AssetScope) parentScope, ctc);
        }
        else if (parent instanceof WorkElement)
        {
            createdParent = WorkElementFixture.createFixture(
                (WorkElement) parent, (WorkElementScope) parentScope, ctc);
        }
        else if (parent instanceof ClientCostCenter)
        {
            createdParent = ClientCostCenterFixture.createFixture(
                (ClientCostCenterScope) parentScope, ctc);
        }
        else if (parent instanceof AbstractAggregateScope)
        {
            ScopeFolder mainScopeFolder = ScopeElementFixture.createMainParent(ctc);
            createdParent = ScopeElementFixture.createActionFixture(
                (ScopeElementScope) parentScope, 1, mainScopeFolder, ctc);
        }
        else if (parent instanceof NoteMinutes)
        {
            //create a parent under which we will create documents
            ScopeElementScope sesScope = new ScopeElementScope();
            sesScope.setDocumentFolder(new DocumentScope());
            ChangeRequest cr = ScopeElementFixture.createSaveChangeRequest(ctc);
            createdParent = DocumentElementFixture.createFixture(
                cr.getDocumentFolder(), parent, 0, (DocumentScope) parentScope,
                ctc);
        }
        else
        {
            logger.debug("RtfAssignmentFixture.createParent : parent of type "
                    + parent.getClass().getName() + " is not supported.");
        }
        return createdParent;
    }

    public static RPMObjectScope createParentScope(RPMObject parent)
            throws Exception
    {
        RPMObjectScope parentScope = null;

        if (parent instanceof Asset)
        {
            parentScope = new AssetScope();
            ((AssetScope) parentScope).setRtfAssignments(true);
        }
        else if (parent instanceof WorkElement)
        {
            parentScope = new WorkElementScope();
            ((WorkElementScope) parentScope).setRtfAssignments(true);
        }
        else if (parent instanceof ClientCostCenter)
        {
            parentScope = new ClientCostCenterScope();
            ((ClientCostCenterScope) parentScope).setRtfAssignments(true);
        }
        else if (parent instanceof AbstractAggregateScope)
        {
            parentScope = new ScopeElementScope();
            ((ScopeElementScope) parentScope).setRtfAssignments(true);
        }
        else if (parent instanceof NoteMinutes)
        {
            parentScope = new DocumentScope();
            ((DocumentScope) parentScope).setRtfAssignments(true);
        }
        else
        {
            logger.debug("RtfAssignmentFixture.createParentScope : parent of type "
                    + parent.getClass().getName() + " is not supported.");
        }
        return parentScope;
    }

    public static void compareParents(RPMObject parent1, RPMObject parent2,
            RPMObjectScope parentScope) throws AssertionFailedError, Exception
    {
        if (parent1 instanceof ClientCostCenter)
        {
            ClientCostCenterFixture.compare((ClientCostCenter) parent1,
                (ClientCostCenter) parent2, (ClientCostCenterScope) parentScope);
        }
        else if (parent1 instanceof Asset)
        {
            AssetFixture.compare((Asset) parent1, (Asset) parent2,
                (AssetScope) parentScope);
        }
        else if (parent1 instanceof WorkElement)
        {
            WorkElementFixture.compare((WorkElement) parent1,
                (WorkElement) parent2, (WorkElementScope) parentScope);
        }
        else if (parent1 instanceof AbstractAggregateScope)
        {
            ScopeElementFixture.compare((AbstractAggregateScope) parent1,
                (AbstractAggregateScope) parent2,
                (ScopeElementScope) parentScope);
        }
        else if (parent1 instanceof NoteMinutes)
        {
            DocumentElementFixture.compare((NoteMinutes) parent1,
                (NoteMinutes) parent2, (DocumentScope) parentScope);
        }
        else
        {
            Assert.fail("RtfAssignmentFixture.compareParents : parent.class "
                    + parent1.getClass().getName()
                    + " not supported by this method.");
        }
    }

    public static RtfAssignment[] getRtfAssignmentsFromParent(RPMObject parent)
            throws Exception
    {
        if (parent instanceof Asset)
        {
            return ((Asset) parent).getRtfAssignments();
        }
        else if (parent instanceof WorkElement)
        {
            return ((WorkElement) parent).getRtfAssignments();
        }
        else if (parent instanceof ClientCostCenter)
        {
            return ((ClientCostCenter) parent).getRtfAssignments();
        }
        else if (parent instanceof AbstractAggregateScope)
        {
            return ((AbstractAggregateScope) parent).getRtfAssignments();
        }
        else if (parent instanceof NoteMinutes)
        {
            return ((NoteMinutes) parent).getRtfAssignments();
        }
        else
        {
            logger.debug("RtfAssignmentFixture.getRtfAssignments : parent of type "
                    + parent.getClass().getName() + " is not supported.");
        }
        return null;
    }

    private static void setRtfAssignmentsFromParent(RPMObject parent,
            RtfAssignment[] assignments) throws Exception
    {
        if (parent instanceof Asset)
        {
            ((Asset) parent).setRtfAssignments(assignments);
        }
        else if (parent instanceof WorkElement)
        {
            ((WorkElement) parent).setRtfAssignments(assignments);
        }
        else if (parent instanceof ClientCostCenter)
        {
            ((ClientCostCenter) parent).setRtfAssignments(assignments);
        }
        else if (parent instanceof AbstractAggregateScope)
        {
            ((AbstractAggregateScope) parent).setRtfAssignments(assignments);
        }
        else
        {
            logger.debug("RtfAssignmentFixture.getRtfAssignments : parent of type "
                    + parent.getClass().getName() + " is not supported.");
        }
    }

    public static void updateRtfAssignmentsFieldsFromParent(RPMObject parent)
            throws Exception
    {
        RtfAssignment[] assignments = getRtfAssignmentsFromParent(parent);
        if (assignments != null)
        {
            for (int i = 0; i < assignments.length; i++)
            {
                RtfAssignmentFixture.updateAssignmentFixture(assignments[i]);
            }
        }
    }


    protected static class RPMObjectAttributeComparator implements Comparator {
        /**
         * 
         */
        public RPMObjectAttributeComparator() {
            super();
        }
        
        public int compare(Object o1, Object o2){ 
            
            RtfAssignment obj1 = (RtfAssignment)o1;
            RtfAssignment obj2 = (RtfAssignment)o2;
            
            String key1 = null;
            String key2 = null;
            
            if(obj1 != null){
                key1 = obj1.getID();
            }
            if(obj2 != null){
                key2 = obj2.getID();
            }          
            
            int compare = compareStrings(key1, key2);

            if(compare == 0){
                if(obj1 != null && obj1.getValue() != null){
                    key1 = obj1.getValue();
                }
                if(obj2 != null && obj2.getValue() != null){
                    key2 = obj2.getValue();
                }               
                
                compare = compareStrings(key1, key2);
            }
            
            if(compare == 0){
                if(obj1 != null && obj1.getRtf() != null){
                    key1 = obj1.getRtf().getID();
                }
                if(obj2 != null && obj2.getRtf() != null){
                    key2 = obj2.getRtf().getID();
                }               
                
                compare = compareStrings(key1, key2);
            }
            
            if(compare == 0){
                if(obj1 != null){
                    key1 = obj1.getID();
                }
                if(obj2 != null){
                    key2 = obj2.getID();
                }                
                
                compare = compareStrings(key1, key2);
            }
            
            return compare;
        }
        
        private int compareStrings(String key1, String key2){
            if (key1 == null && key2 == null) {
                return 0;
            }
            if (key1 == null && key2 != null) {
                return -1;
            }
            if (key1 != null && key2 == null) {
                return 1;
            }
            return key1.compareTo(key2);
        }
    
    }
	
}
