/*
 * Created on Apr 3, 2006
 *
 */
package com.ibm.rpm.client.junit.applicationadministration.fixtures;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;

/**
 * @author karim
 *
 */
public class SecurityRoleFixture extends RPMObjectFixture
{

    public static SecurityRole createFixture(SecurityRole parentSecurityRole) throws Exception
    {
        SecurityRole securityRole =  createSecurityRole(random(), parentSecurityRole);
    	return securityRole;
    }
	
    
    private static SecurityRole createSecurityRole(int index, SecurityRole parentSecurityRole)throws Exception
	{
        SecurityRole securityRole = new SecurityRole();
    	HelperTestUtilities.setByIntrospection(securityRole, index, skipSetters);
    	securityRole.setParent(parentSecurityRole);
    	return securityRole;
    }
    
    public static SecurityRole createParticipantSecurityRole(RPMObject parent) throws Exception
    {
    	SecurityRole securityRole = new SecurityRole();
        
        if(parent == null)
        {
            TestCase.fail("Can't determine the security role for a null parent object");
        } 
        else if (parent instanceof Action)
        {
            securityRole.setID("SCACTIONITEMPARTICIPANT");
        }
        else if (parent instanceof ChangeRequest)
        {
            securityRole.setID("SCCHANGEREQUESTPARTICIPANT");
        }
        else if (parent instanceof Defect)
        {
            securityRole.setID("SCDEFECTPARTICIPANT");
        }
        else if (parent instanceof ScopeDeliverable || 
                parent instanceof Deliverable || parent instanceof WorkProduct 
                || parent instanceof ScopeSummaryTask || parent instanceof SummaryTask)
        {
            securityRole.setID("SCDELIVERABLEPARTICIPANT");
        }
        else if (parent instanceof Document)
        {
            securityRole.setID("SCDOCUMENTPARTICIPANT");
        }
        else if (parent instanceof ScopeFolder)
        {
            securityRole.setID("SCFOLDERPARTICIPANT");   
        }
        else if (parent instanceof Issue)
        {
            securityRole.setID("SCISSUEPARTICIPANT");
        }
        else if (parent instanceof NoteMinutes)
        {
            securityRole.setID("SCNOTEMINUTESPARTICIPANT");
        }
        else if (parent instanceof Pool)
        {
            securityRole.setID("SCPOOLPARTICIPANT");
        }
        else if (parent instanceof Project || parent instanceof Proposal 
                || parent instanceof WorkOrganization)
        {
            securityRole.setID("SCPROJECTPARTICIPANT");
        }    
        else if (parent instanceof Requirement)
        {
            securityRole.setID("SCREQUIREMENTPARTICIPANT");
        }
        else if (parent instanceof Risk)
        {
            securityRole.setID("SCRISKPARTICIPANT");
        }
        else if (parent instanceof ServiceRequest)
        {
            securityRole.setID("SCSERVICEREQUESTPARTICIPANT");
        }
        else if (parent instanceof ScopeTask
                || parent instanceof ScopeMilestone || parent instanceof Task
                || parent instanceof Milestone )
        {
            securityRole.setID("SCTASKPARTICIPANT");
        }
        else if(AbstractDocument.class.isAssignableFrom(parent.getClass()))
        {
        	securityRole.setID("SCDOCUMENTPARTICIPANT");
        }
        else if(parent instanceof NoteMinutes)
        {
        	securityRole.setID("SCNOTEMINITESPARTICIPANT");
        }
        else
        {
            TestCase.fail("Could not create the proper security type for " + parent.getClass());
        }
        
        return securityRole;	
    }
    
    public static SecurityRole createSecurityRole(RPMObject parent) throws Exception
    {
        SecurityRole securityRole = new SecurityRole();
         
        if(parent == null)
        {
            TestCase.fail("Can't determine the security role for a null parent object");
        } 
        else if (parent instanceof Action)
        {
            securityRole.setID("SCACTIONITEMRESPONSIBLE");
        }
        else if (parent instanceof ChangeRequest)
        {
            securityRole.setID("SCCHANGEREQUESTRESPONSIBLE");
        }
        else if (parent instanceof Defect)
        {
            securityRole.setID("SCDEFECTRESPONSIBLE");
        }
        else if (parent instanceof ScopeDeliverable || 
                parent instanceof Deliverable || parent instanceof WorkProduct 
                || parent instanceof ScopeSummaryTask || parent instanceof SummaryTask)
        {
            securityRole.setID("SCDELIVERABLERESPONSIBLE");
        }
        else if (parent instanceof Document)
        {
            securityRole.setID("SCDOCUMENTRESPONSIBLE");
        }
        else if (parent instanceof ScopeFolder)
        {
            securityRole.setID("SCFOLDERMANAGER");   
        }
        else if (parent instanceof Issue)
        {
            securityRole.setID("SCISSUERESPONSIBLE");
        }
        else if (parent instanceof NoteMinutes)
        {
            securityRole.setID("SCNOTEMINUTESRESPONSIBLE");
        }
        else if (parent instanceof Pool)
        {
            securityRole.setID("SCPOOLPARTICIPANT");
        }
        else if (parent instanceof Project || parent instanceof Proposal 
                || parent instanceof WorkOrganization)
        {
            securityRole.setID("SCPROJECTMANAGER");
        }    
        else if (parent instanceof Requirement)
        {
            securityRole.setID("SCREQUIREMENTRESPONSIBLE");
        }
        else if (parent instanceof Risk)
        {
            securityRole.setID("SCRISKRESPONSIBLE");
        }
        else if (parent instanceof ServiceRequest)
        {
            securityRole.setID("SCSERVICEREQUESTRESPONSIBLE");
        }
        else if (parent instanceof ScopeTask
                || parent instanceof ScopeMilestone || parent instanceof Task
                || parent instanceof Milestone )
        {
            securityRole.setID("SCTASKRESPONSIBLE");
        }
        else if(AbstractDocument.class.isAssignableFrom(parent.getClass()))
        {
        	securityRole.setID("SCDOCUMENTRESPONSIBLE");
        }
        else if(parent instanceof NoteMinutes)
        {
        	securityRole.setID("SCNOTEMINITESRESPONSIBLE");
        }
        else
        {
            TestCase.fail("Could not create the proper security type for " + parent.getClass());
        }
        
        return securityRole;
    }

    
    public static SecurityRole updateFixture(SecurityRole securityRole) throws Exception
    {
        return updateSecurityRole(random(), securityRole);
    }
	
    
    private static SecurityRole updateSecurityRole(int index, SecurityRole securityRole)throws Exception
	{
    	HelperTestUtilities.setByIntrospection(securityRole, index, skipSetters);
    	
    	updateSecurityRoleRights(securityRole);
    	
    	return securityRole;
    }
    
    private static void updateSecurityRoleRights(SecurityRole securityRole)
    {
        if(securityRole instanceof ProjectSecurityRole)
        {
            ProjectSecurityRole psr = (ProjectSecurityRole)securityRole;
            psr.setCanDeleteWBSItems(new Boolean(!psr.getCanDeleteWBSItems().booleanValue()));
        }
    }
    
    public static SecurityRoleScope createScope(){
        SecurityRoleScope scope = new SecurityRoleScope();
    	return scope;
    } 
    
    public static void compare(SecurityRole sr1, SecurityRole sr2,
            SecurityRoleScope scope) throws AssertionFailedError, Exception
    {
        if(sr1.getClass().equals(SecurityRole.class))
        {
            //initial creation
            Assert.assertEquals(sr1.getName(), sr2.getName());
        } 
        else
        {
            RPMObjectFixture.compare(sr1, sr2);
        }

        if (scope == null) return;
   
        if(scope.getParent() != null)
        {
            SecurityRoleScope s = (SecurityRoleScope)scope.getParent();
            if (s != null)
            {
                s.setSecurityRoles(null);
            }

            SecurityRoleFixture.compare((SecurityRole) sr1.getParent(),
                    (SecurityRole) sr2.getParent(), s);
        }
    }
}
