/*
 * Created on Feb 6, 2006
 *
 */
package com.ibm.rpm.client.junit.applicationadministration.testcases;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.scope.AssetModuleScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author karim
 *
 */
public class TestAttributeAssignment extends ClientTestController
{
    
    // Global Variables
    RPMObject parent = null; 
    RPMObjectScope parentScope = null;
    
    public TestAttributeAssignment()
    {
        //Default values
        parent = new Proposal();
    }

    public TestAttributeAssignment(String testName, RPMObject parent)
    {
        setName(testName);
        this.parent = parent;
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        
        parentScope = AttributeAssignmentFixture.createParentScope(parent);
        parent = AttributeAssignmentFixture.createParent(parent, parentScope, this);
    }


    
    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        insert(false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(true);
    }


    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(true);
    }
    
    private void insert(boolean loadFromId)
            throws AssertionError, Exception
    {
        //insert an element
        parent = insertAttributeAssignment(loadFromId);

        //delete the element
        deleteAttributeAssignment(loadFromId);
    }

    private RPMObject insertAttributeAssignment(boolean loadFromId)
            throws Exception
    {
        //Insert the element into the DB
        SingleResult result = save(parent, parentScope);

        //compare the returned element
        RPMObject returnedParent = result.getRpmObject();
        AttributeAssignmentFixture.compareParents(parent, returnedParent, parentScope);

        //reload the element and compare it
        returnedParent = load(returnedParent, parentScope, loadFromId);
        AttributeAssignmentFixture.compareParents(parent, returnedParent, parentScope);

        //return the DocumentElement
        return returnedParent;
    }

    private void update(boolean loadFromId) throws AssertionError, Exception
    {
        //insert an element
        parent = insertAttributeAssignment(loadFromId);

        //update the element
        parent = updateAttributeAssignmentFields(loadFromId);
        parent = updateAttributeAssignmentStructure(loadFromId);

        //delete the DocumentElement
        deleteAttributeAssignment(loadFromId);
    }

    private RPMObject updateAttributeAssignmentFields(boolean loadFromId) throws Exception
    {
        return updateAttributeAssignment(loadFromId, false);
    }

    private RPMObject updateAttributeAssignmentStructure(boolean loadFromId) throws Exception
    {
        return updateAttributeAssignment(loadFromId, true);
    }

    private RPMObject updateAttributeAssignment(boolean loadFromId, boolean updateStructure)
            throws Exception
    {
        if (updateStructure)
        {
            //updates the structure by adding elements
            AttributeAssignmentFixture.updateAssignmentsStructureFromParent(parent, this);
        }
        else
        {
            //only updates the fields
            AttributeAssignmentFixture.updateAssignmentsFieldsFromParent(parent);
        }

        //Insert the updated element into the DB
        SingleResult result = save(parent, parentScope);

        //compare the returned element
        RPMObject returnedParent = result.getRpmObject();
        AttributeAssignmentFixture.compareParents(parent, returnedParent, parentScope);

        //reload the element and compare it
        returnedParent = load(returnedParent, parentScope, loadFromId);
        AttributeAssignmentFixture.compareParents(parent, returnedParent, parentScope);

        //return the DocumentElement
        return returnedParent;
    }

    private void deleteAttributeAssignment(boolean loadFromId)
            throws Exception
    {
    	AttributeAssignment[] assignments = AttributeAssignmentFixture.getAttributeAssignmentsFromParent(parent);
    	
    	if(assignments!=null)
    	{
    	    for(int i=0;i<assignments.length;i++){
    	        assignments[i].setDeleted(new Boolean(true));
    	    }
    	}
    	
        //delete the elements
    	SingleResult result = save(parent, parentScope);

        //reload the deleted elements
        RPMObject returnedParent = load(result.getRpmObject(), parentScope, loadFromId);
        AttributeAssignment[] deletedAssignments = AttributeAssignmentFixture.getAttributeAssignmentsFromParent(returnedParent);
        
        //Validate the deleted elements
        assertNull(deletedAssignments);
    }
   
    private RPMObject load(RPMObject parent, RPMObjectScope parentScope,
            boolean loadFromId) throws Exception
    {
        RPMObject obj = null;

        if (loadFromId)
        {
            
            obj = (RPMObject)parent.getClass().newInstance();
            obj.setID(parent.getID());
            //useful only for Documents to determine the context
            obj.setContextName(parent.getContextName());
            return loadFromID(obj, parentScope);
        }
        else
        {
            //load from an xpath
            String containerName = StringUtil.getShortClassName(parent.getClass());
            RPMObject[] objs = loadFromXpath("/"+containerName+"[id='"
                    + parent.getID() + "']", parentScope);
            if (objs == null || objs.length == 0)
            {
                return null;
            }
            else
            {
                return objs[0];
            }
        }
    }
    
    // null dates used to cause NullPointerException
    public void test67484() throws Exception
    {
        AttributeScope scope = new AttributeScope();
        scope.setAttributes(scope);
        scope.setCategories(scope);
        scope.setClassifications(scope);
        AssetModuleScope moduleScope = new AssetModuleScope();
        moduleScope.setAttributeCategories(scope);
        
        AssetModule module = new AssetModule();
        
        AttributeCategory createFixture = AttributeFixture.createFixture(1, 1, 1, scope);
        createFixture.setParent(module);
        Attribute attribute = createFixture.getClassifications()[0].getAttributes()[0];
        attribute.setInputType(InputType.Date);
        module.setAttributeCategories(new AttributeCategory[]{createFixture});
        
        SingleResult save = save(createFixture, scope);
        createFixture = (AttributeCategory) save.getRpmObject();
        attribute = createFixture.getClassifications()[0].getAttributes()[0];
        
        RPMObjectScope parentScope;
        RPMObject parent = new Asset();
        parentScope = AttributeAssignmentFixture.createParentScope(parent);
        parent = AttributeAssignmentFixture.createParent(parent, parentScope, this);
        
        AttributeAssignment[] attributeAssignmentsFromParent = AttributeAssignmentFixture.getAttributeAssignmentsFromParent(parent);
        AttributeAssignment assignmentToTest = attributeAssignmentsFromParent[0];
        assignmentToTest.setAttribute(attribute);
        assignmentToTest.setValue("2000-11-22");
        
        Asset savedAsset = (Asset) save(parent, parentScope).getRpmObject();
        assignmentToTest = savedAsset.getAttributeAssignments()[0];
        assertEquals("2000-11-22", assignmentToTest.getValue());
        
        SingleResult result = null;
        
        assignmentToTest.setValue("2000-11-23");
        result = save(savedAsset, parentScope);
        savedAsset = (Asset) result.getRpmObject();
        assignmentToTest = savedAsset.getAttributeAssignments()[0];
        assertEquals("2000-11-23", assignmentToTest.getValue());
        
        // NOTE : this is a bad behavior, the API specified that a calendar will
        // be returned, but it doesn't return a calendar
        assertFalse(result.isWarningFree());
        
        assignmentToTest.setValue(null);
        result = save(savedAsset, parentScope);
        savedAsset = (Asset) result.getRpmObject();
        assignmentToTest = savedAsset.getAttributeAssignments()[0];
        assertNull(assignmentToTest.getValue());
        assertTrue(result.isWarningFree());

        Calendar currentCalendar = Calendar.getInstance();
        currentCalendar.set(2003, 9, 13);
        assignmentToTest.setValue(currentCalendar);
        result = save(savedAsset, parentScope);
        savedAsset = (Asset) result.getRpmObject();
        assignmentToTest = savedAsset.getAttributeAssignments()[0];
        assertEquals("2003-10-13", assignmentToTest.getValue());
        // NOTE : this is a bad behavior, the API specified that a calendar will
        // be returned, but it doesn't return a calendar
        assertTrue(result.isWarningFree());
        
        assignmentToTest.setValue("bad-date");
        assertNotNull(saveMustFail(savedAsset, parentScope).getErrors()[0]);

    }
    
    public void testDelete_RATLC00308046() throws AssertionError, Exception
    {
    	parent = save(parent, parentScope).getRpmObject();
    	
    	Object object[] = (Object [])parent.getClass().getMethod("getAttributeAssignments", null).invoke(parent, null);
    	
    	for(int i=0; i<object.length; i++)
    	{
    		AttributeAssignment attributeAssignment = (AttributeAssignment) object[0];
    		// Uncomment the below line while executing
//    		attributeAssignment.assignTypeID(null);
    		delete(attributeAssignment);
    	}
    }
}
