package com.ibm.rpm.client.junit.applicationadministration.testcases;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestAttributeAssignment1 extends ClientTestController {
	
	private Attribute attribute= null;
	
	public void testLoadAttributeAssignmentWithParentForWbs() throws Exception
	{
		loadAttributeAssignmentWithParent(AttributeFixture.WBS_CONTEXT, new Task());
	}
	
	public void testLoadAttributeAssignmentWithParentForScope() throws Exception
	{
		loadAttributeAssignmentWithParent(AttributeFixture.SCOPE_CONTEXT, new Requirement());
	}
	
	public void testLoadAttributeAssignmentWithParentForPool() throws Exception
	{
		loadAttributeAssignmentWithParent(AttributeFixture.POOL_CONTEXT, new Pool());
	}
	
	public void testLoadAttributeAssignmentWithParentForDocument() throws Exception
	{
		loadAttributeAssignmentWithParent(AttributeFixture.DOCUMENT_CONTEXT, new Document());
	}
	
	public void testLoadAttributeAssignmentWithParentForAsset() throws Exception
	{
		loadAttributeAssignmentWithParent(AttributeFixture.ASSET_CONTEXT, new Asset());
	}
	
	public void testLoadAttributeAssignmentWithParentForCCC() throws Exception
	{
		loadAttributeAssignmentWithParent(AttributeFixture.CLIENT_CONTEXT, new ClientCostCenter());
	}
	
	private void loadAttributeAssignmentWithParent(String contextName, RPMObject parent)throws Exception
	{
		createSaveAttribute(contextName);
		parent = AttributeAssignmentFixture.createParent(parent, null, this);
		parent = save(parent,null).getRpmObject();
		AttributeAssignment attributeAssignment = assignSaveAttributeAssignment(parent);
		loadAndValidateAttributeAssignment(attributeAssignment,parent);
	}
	
	private void loadAndValidateAttributeAssignment(AttributeAssignment attributeAssignment, RPMObject parent) throws Exception
	{
		RPMObjectScope rpmObjectScope = new RPMObjectScope();
		rpmObjectScope.setParent(new RPMObjectScope());
		attributeAssignment = (AttributeAssignment)loadFromID(attributeAssignment, rpmObjectScope);

		assertNotNull(attributeAssignment);
		assertNotNull(attributeAssignment.getParent());
		assertTrue(attributeAssignment.getParent().getID().equals(parent.getID()));
	}
	
	private AttributeAssignment assignSaveAttributeAssignment(RPMObject rpmObject) throws Exception
	{
		AttributeAssignment attributeAssignment = new AttributeAssignment();
		attributeAssignment.setAttribute(attribute);
		attributeAssignment.setValue("Test");
		
		rpmObject = saveAttributeAssignments(rpmObject, new AttributeAssignment[]{attributeAssignment});
		assertNotNull(rpmObject);
		
		AttributeAssignment attributeAssignments[] = null;
		if(rpmObject instanceof WorkElement)
		{
			attributeAssignments = ((WorkElement)rpmObject).getAttributeAssignments();
		}
		else if(rpmObject instanceof AbstractAggregateScope)
		{
			attributeAssignments = ((AbstractAggregateScope)rpmObject).getAttributeAssignments();
		}
		else if(rpmObject instanceof Pool)
		{
			attributeAssignments = ((Pool)rpmObject).getAttributeAssignments();
		}
		else if(rpmObject instanceof GenericDocument)
		{
			attributeAssignments = ((GenericDocument)rpmObject).getAttributeAssignments();
		}
		else if(rpmObject instanceof Asset)
		{
			attributeAssignments = ((Asset)rpmObject).getAttributeAssignments();
		}
		else if(rpmObject instanceof ClientCostCenter)
		{
			attributeAssignments = ((ClientCostCenter)rpmObject).getAttributeAssignments();
		}
		
		assertNotNull(attributeAssignments);
		assertEquals(1, attributeAssignments.length);
		assertNotNull(attributeAssignments[0]);
		
		return attributeAssignments[0];
	}
	
	private void createSaveAttribute(String contextName) throws Exception
	{
		AttributeCategory attributeCategory = AttributeFixture.createSaveFixture(contextName, this);
		
		assertNotNull(attributeCategory.getClassifications());
		assertEquals(1, attributeCategory.getClassifications().length);
		assertNotNull(attributeCategory.getClassifications()[0]);
		
		AttributeClassification attributeClassification = attributeCategory.getClassifications()[0];
		assertNotNull(attributeClassification.getAttributes());
		assertEquals(1, attributeClassification.getAttributes().length);
		assertNotNull(attributeClassification.getAttributes()[0]);
		
		attribute = attributeCategory.getClassifications()[0].getAttributes()[0];
		attribute.setInputType(InputType.Text);
		attribute = (Attribute)save(attribute, null).getRpmObject();
		assertNotNull(attribute);
		assertEquals(InputType.Text, attribute.getInputType());
	}
	
	private RPMObject saveAttributeAssignments(RPMObject rpmObject, AttributeAssignment[] attributeAssignments) throws Exception
	{
		RPMObjectScope rpmObjectScope = null;
		if(rpmObject instanceof WorkElement)
		{
			rpmObjectScope = new WorkElementScope();
			((WorkElementScope)rpmObjectScope).setAttributeAssignments(true);
			((WorkElement)rpmObject).setAttributeAssignments(attributeAssignments);
		}
		else if(rpmObject instanceof AbstractAggregateScope)
		{
			rpmObjectScope = new ScopeElementScope();
			((ScopeElementScope)rpmObjectScope).setAttributeAssignments(true);
			((AbstractAggregateScope)rpmObject).setAttributeAssignments(attributeAssignments);
		}
		else if(rpmObject instanceof Pool)
		{
			rpmObjectScope = new PoolScope();
			((PoolScope)rpmObjectScope).setAttributeAssignments(true);
			((Pool)rpmObject).setAttributeAssignments(attributeAssignments);
		}
		else if(rpmObject instanceof GenericDocument)
		{
			rpmObjectScope = new DocumentScope();
			((DocumentScope)rpmObjectScope).setAttributeAssignments(true);
			((GenericDocument)rpmObject).setAttributeAssignments(attributeAssignments);
		}
		else if(rpmObject instanceof Asset)
		{
			rpmObjectScope = new AssetScope();
			((AssetScope)rpmObjectScope).setAttributeAssignments(true);
			((Asset)rpmObject).setAttributeAssignments(attributeAssignments);
		}
		else if(rpmObject instanceof ClientCostCenter)
		{
			rpmObjectScope = new ClientCostCenterScope();
			((ClientCostCenterScope)rpmObjectScope).setAttributeAssignments(true);
			((ClientCostCenter)rpmObject).setAttributeAssignments(attributeAssignments);
		}
		
		rpmObject = save(rpmObject, rpmObjectScope).getRpmObject();
		
		return rpmObject;
	}
}
