package com.ibm.rpm.client.junit.applicationadministration.testcases;

import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.applicationadministration.scope.ContactGroupScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.ContactGroupFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author karim
 *
 */
public class TestContactGroups extends ClientTestController
{

    private ContactGroupScope mainScope;

    private static final int NB_LEVELS = 4;//should always be > 1
    private static final int NB_CONTACT_GROUPS_BY_LEVEL = 2;
    private static final int DELTA_NB_CONTACT_GROUPS_BY_LEVEL = 1;
    
    public TestContactGroups(String testName){
        setName(testName);
    }
    
    public void setUp() throws Exception
    {
        logger.info("setUp");
		super.setUp();

		mainScope = ContactGroupFixture.createScope(1, NB_LEVELS);

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError, Exception
	{
    	insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(mainScope, true);
    }
    
    public void testUpdateWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError, Exception
    {
    	update(mainScope, true);
    }

    private void insert(ContactGroupScope contactGroupScope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert a contact group
        ContactGroup contactGroups = insertContactGroup(contactGroupScope, loadFromId);
    	
    	//delete the contact group
    	deleteContactGroup(contactGroups, loadFromId);    	
    }
    
    private void update(ContactGroupScope contactGroupScope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert a contact group
    	ContactGroup contactGroups = insertContactGroup(contactGroupScope, loadFromId);
    	
        //update the contact group
    	contactGroups = updateContactGroup(contactGroups, contactGroupScope, loadFromId, false);
    	contactGroups = updateContactGroup(contactGroups, contactGroupScope, loadFromId, true);
    	
    	//delete the contact group
    	deleteContactGroup(contactGroups, loadFromId);
    }
    
    private ContactGroup insertContactGroup(ContactGroupScope scope, boolean loadFromId) throws Exception
    {
    	ContactGroup contactGroups = ContactGroupFixture.createFixture(NB_CONTACT_GROUPS_BY_LEVEL, scope);
    	       
        //Insert the contact group into the DB
    	SingleResult result =  save(contactGroups,scope);
        
        //compare the returned contact group
        ContactGroup returnedContactGroup = (ContactGroup)result.getRpmObject();
        ContactGroupFixture.compare(contactGroups, returnedContactGroup, scope);
        
        //reload the contact group and compare it
        returnedContactGroup = (ContactGroup)load(returnedContactGroup, scope, loadFromId);
        ContactGroupFixture.compare(contactGroups, returnedContactGroup, scope);
        
        //return the contact group
        return returnedContactGroup;
    }
    

    private ContactGroup updateContactGroup(ContactGroup contactGroups, ContactGroupScope scope, boolean loadFromId, boolean updateStructure) throws Exception
	{
    	if(updateStructure){
            //updates the structure by adding elements 
    		ContactGroupFixture.updateFixtureStructure(contactGroups, DELTA_NB_CONTACT_GROUPS_BY_LEVEL, NB_CONTACT_GROUPS_BY_LEVEL, scope);    		
    	} else {
    		//only updates the fields
    		ContactGroupFixture.updateFixtureFields(contactGroups, scope);
    	}

        //Insert the contact group into the DB
    	SingleResult result =  save(contactGroups,scope);
        
        //compare the returned contact group
        ContactGroup returnedContactGroup = (ContactGroup)result.getRpmObject();
        ContactGroupFixture.compare(contactGroups, returnedContactGroup, scope);
        
        //reload the contact group and compare it
        returnedContactGroup = (ContactGroup)load(returnedContactGroup, scope, loadFromId);
        ContactGroupFixture.compare(contactGroups, returnedContactGroup, scope);
        
        //return the contact group
        return returnedContactGroup;
	}
    
    private void deleteContactGroup(ContactGroup contactGroups, boolean loadFromId)throws Exception
	{    	   	
        //delete the contact group
       	delete(contactGroups);
    	
        //reload the deleted contact group
       	ContactGroup deletedContactGroup = loadDeletedContactGroup(contactGroups, loadFromId);
        
        //validate deleted contact group
        assertNull(deletedContactGroup);
    }
    
    private RPMObject load(ContactGroup contactGroups, ContactGroupScope contactGroupScope, boolean loadFromId) throws Exception
	{
    	if(loadFromId){
            //load from an id
            RPMObject obj = new ContactGroup();
            obj.setID(contactGroups.getID());
            return loadFromID(obj, contactGroupScope);
    	}
    		//load from an xpath
        RPMObject[] objs = loadFromXpath("/CONTACTGROUP[id='"
                + contactGroups.getID() + "']", contactGroupScope);
        if (objs == null || objs.length == 0)
        {
            return null;
        }
        return objs[0];
    	
    }
    
    private ContactGroup loadDeletedContactGroup(ContactGroup contactGroups, boolean loadFromId)throws Exception{
        try{
        	contactGroups = (ContactGroup)load(contactGroups, null, loadFromId);
           	return contactGroups;
        }catch (Exception e){
            //contact group does not exist
        }
        return null;
    }
    
}
