package com.ibm.rpm.client.junit.applicationadministration.testcases;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.CurrencyExchangeRange;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.CurrencyFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author karim
 */
public class TestCurrency extends ClientTestController
{
    private static final int NB_EXCHANGE_RANGES = 1;
    
    public TestCurrency(String name)
    {
        setName(name);
    }
  
    public void setUp() throws Exception
    {
		super.setUp();
    }

    public void tearDown() throws Exception
    {    	
    	super.tearDown();
    }
    
    public void testDeleteAllTestCurrenciesBeforeAllTests() throws Exception
	{
        deleteAllTestCurrencies();
    }

    public void testXPathUsDollar() throws Exception
    {
        RPMObject[] result = loadFromXpath(
                "/Currency[starts-with(name, 'US')]", null);
        assertNotNull(result);
        assertEquals(1, result.length);
        Currency currency = (Currency) result[0];
        CurrencyExchangeRange[] exchanges = currency.getExchangeRanges();
        assertNotNull(exchanges);
        assertTrue(exchanges.length > 0);
    }

    public void testXPathUsDollarExchangeRange() throws AssertionError,
            Exception
    {
        RPMObject[] result = loadFromXpath(
                "/Currency[starts-with(name, 'US')]/CurrencyExchangeRange",
                null);
        assertNotNull(result);
        assertTrue(result.length > 0);
    }

    public void testSaveWithoutChange() throws Exception
    {
        RPMObject[] result = loadFromXpath(
                "/Currency[starts-with(name, 'US')]", null);
        assertNotNull(result);
        assertEquals(1, result.length);
        Currency currency = (Currency) result[0];

        Currency reloaded = (Currency) saveWithoutDelete(currency, null)
                .getRpmObject();
        assertNotNull(reloaded);
    }

    public void testCreateCurrencyOnly() throws Exception
    {
        Currency expected = CurrencyFixture.createFixture(false, 0, this);
        Currency actual = (Currency) save(expected, null).getRpmObject();
        CurrencyFixture.compare(expected, actual);
    }

    public void testCreateCurrencyWithOneExchangeRange() throws Exception
    {
        Currency expected = CurrencyFixture.createFixture(false, 1, this);
        Currency actual = (Currency) save(expected, null).getRpmObject();
        CurrencyFixture.compare(expected, actual);
    }

    public void testCreateCurrencyWithTwoExchangeRanges() throws Exception
    {
        Currency expected = CurrencyFixture.createFixture(false, 2, this);
        Currency actual = (Currency) save(expected, null).getRpmObject();
        CurrencyFixture.compare(expected, actual);
    }

    public void testCreateLoadFromXPath() throws Exception
    {
        insert(false);
    }

    public void testCreateLoadFromID() throws Exception
    {
        insert(true);
    }

    public void testUpdateLoadFromID() throws Exception
    {
        //insert an element
        Currency currency = insert(true);
        //update the element
        currency = updateCurrency(currency, true);
    }

    public void testDeleteAllTestCurrenciesAfterAllTests() throws Exception
    {
        deleteAllTestCurrencies();
    }

    private void deleteAllTestCurrencies() throws Exception
    {
        RPMObject[] nonDeleted = loadFromXpath(
                "/Currency[starts-with(name, 'TC')]", null);
        if (nonDeleted != null)
        {
            for (int i = 0; i < nonDeleted.length; i++)
            {
                delete(nonDeleted[i]);
            }
        }
    }

    private Currency insert(boolean loadFromId) throws Exception
    {
        Currency currency = CurrencyFixture.createFixture(false,
                NB_EXCHANGE_RANGES, this);

        //Insert the element into the DB
        SingleResult result = save(currency, null);

        //compare the returned element
        Currency returnedCurrency = (Currency) result.getRpmObject();
        CurrencyFixture.compare(currency, returnedCurrency);

        //reload the element and compare it
        returnedCurrency = (Currency) load(returnedCurrency, loadFromId);
        CurrencyFixture.compare(currency, returnedCurrency);

        //return the element
        return returnedCurrency;
    }

    private Currency updateCurrency(Currency currency, boolean loadFromId)
            throws Exception
	{
   		//only updates the fields
        CurrencyFixture.updateFixtureFields(currency);

        //Insert the element into the DB
        SingleResult result = save(currency, null);
        
        //compare the returned element
        Currency returnedCurrency = (Currency)result.getRpmObject();
        CurrencyFixture.compare(currency, returnedCurrency);
        
        //reload the element and compare it
        returnedCurrency = (Currency) load(returnedCurrency, loadFromId);
        CurrencyFixture.compare(currency, returnedCurrency);
        
        //return the element
        return returnedCurrency;
	}
    
    private RPMObject load(Currency currency, boolean loadFromId)
            throws Exception
    {
        if (loadFromId)
        {
            //load from an id
            RPMObject obj = new Currency();
            obj.setID(currency.getID());
            return loadFromID(obj, null);
        }
    		//load from an xpath
        RPMObject[] objs = loadFromXpath("/CURRENCY[id='" + currency.getID()
                + "']", null);
        if (objs == null || objs.length == 0)
        {
            return null;
        }
    			return objs[0];
    }
}
