package com.ibm.rpm.client.junit.applicationadministration.testcases;

import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.scope.OrganizationalScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author karim
 *
 */
public class TestOrganizational extends ClientTestController
{
    private OrganizationalScope mainScope;

    private static final int NB_LEVELS = 3; //max value = 6 
    private static final int NB_ORGANIZATIONALS_BY_LEVEL = 2;
    private static final int DELTA_NB_ORGANIZATIONALS_BY_LEVEL = 1;
    
    public TestOrganizational(String testName){
        setName(testName);
    }
	
    public void setUp() throws Exception
    {
        logger.info("setUp");
		super.setUp();

		mainScope = OrganizationalFixture.createScope(NB_LEVELS);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError, Exception
	{
    	insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(mainScope, true);
    }
    
    public void testUpdateWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError, Exception
    {
    	update(mainScope, true);
    }

    private void insert(OrganizationalScope scope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert an element
        Organizational organizational = insertOrganizational(scope, loadFromId);
    	
    	//delete the element
    	deleteOrganizational(organizational, loadFromId);    	
    }
    
    private void update(OrganizationalScope scope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert a element
    	Organizational organizational = insertOrganizational(scope, loadFromId);
    	
        //update the element
    	organizational = updateOrganizationalFields(organizational, scope, loadFromId);
    	organizational = updateOrganizationalStructure(organizational, scope, loadFromId);
    	
    	//delete the element
    	deleteOrganizational(organizational, loadFromId);
    }
    
    private Organizational insertOrganizational(OrganizationalScope scope, boolean loadFromId) throws Exception
    {
    	Organizational organizational = OrganizationalFixture.createFixture(NB_ORGANIZATIONALS_BY_LEVEL, scope);
    	       
        //Insert the element into the DB
    	SingleResult result =  save(organizational,scope);
        
        //compare the returned element
        Organizational returnedOrganizational = (Organizational)result.getRpmObject();
        OrganizationalFixture.compare(organizational, returnedOrganizational,scope);
        
        //reload the element and compare it
        returnedOrganizational = (Organizational)load(returnedOrganizational, scope, loadFromId);
        OrganizationalFixture.compare(organizational, returnedOrganizational, scope);
        
        //return the element
        return returnedOrganizational;
    }
    
    private Organizational updateOrganizationalFields(Organizational organizational, OrganizationalScope scope, boolean loadFromId) throws Exception
	{
    	return updateOrganizational(organizational, scope, loadFromId, false);
	}
    
    private Organizational updateOrganizationalStructure(Organizational organizational, OrganizationalScope scope, boolean loadFromId) throws Exception
	{
    	return updateOrganizational(organizational, scope, loadFromId, true);
	}

    private Organizational updateOrganizational(Organizational organizational, OrganizationalScope scope, boolean loadFromId, boolean updateStructure) throws Exception
	{
    	if(updateStructure){
            //updates the structure by adding elements 
    		OrganizationalFixture.updateFixtureStructure(organizational, DELTA_NB_ORGANIZATIONALS_BY_LEVEL, NB_ORGANIZATIONALS_BY_LEVEL, scope);    		
    	} else {
    		//only updates the fields
    		OrganizationalFixture.updateFixtureFields(organizational, scope);
    	}

        //Insert the element into the DB
    	SingleResult result =  save(organizational,scope);
        
        //compare the returned element
        Organizational returnedOrganizational = (Organizational)result.getRpmObject();
        OrganizationalFixture.compare(organizational, returnedOrganizational, scope);
        
        //reload the element and compare it
        returnedOrganizational = (Organizational)load(returnedOrganizational, scope, loadFromId);
        OrganizationalFixture.compare(organizational, returnedOrganizational, scope);
        
        //return the element
        return returnedOrganizational;
	}
    
    private void deleteOrganizational(Organizational organizational, boolean loadFromId)throws Exception
	{    	   	
        //delete the element
       	delete(organizational);
    	
        //reload the deleted element
       	Organizational deletedOrganizational = loadDeletedOrganizational(organizational, loadFromId);
        
        //validate deleted element
        assertNull(deletedOrganizational);
    }
    
    private RPMObject load(Organizational organizational, OrganizationalScope scope, boolean loadFromId) throws Exception
	{
    	if(loadFromId){
            //load from an id
            RPMObject obj = OrganizationalFixture.createChild((Organizational)organizational.getParent());
            obj.setID(organizational.getID());
            return loadFromID(obj, scope);
    	} else {
    		//load from an xpath
    		RPMObject[] objs = loadFromXpath("/ORGANIZATION[id='"+ organizational.getID() + "']", scope);
    		if(objs==null || objs.length==0){
    			return null;
    		} else {
    			return objs[0];
    		}
    	}
    }
    
    private Organizational loadDeletedOrganizational(Organizational organizational, boolean loadFromId)throws Exception{
        try{
        	organizational = (Organizational)load(organizational, null, loadFromId);
           	return organizational;
        }catch (Exception e){
            //element does not exist
        }
        return null;
    }
	
}
