package com.ibm.rpm.client.junit.applicationadministration.testcases;

import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.applicationadministration.scope.PoolDomainScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.PoolDomainFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author karim
 *
 */
public class TestPoolDomains extends ClientTestController
{
    private PoolDomainScope msScope;

    private static final int NB_LEVELS = 4;//should always be > 1
    private static final int NB_POOL_DOMAINS_BY_LEVEL = 2;
    private static final int DELTA_NB_POOL_DOMAINS_BY_LEVEL = 1;
    
    public TestPoolDomains(String testName){
        setName(testName);
    }
	
    public void setUp() throws Exception
    {
        logger.info("setUp");
		super.setUp();

		msScope = PoolDomainFixture.createScope(1, NB_LEVELS);

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError, Exception
	{
    	insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(msScope, true);
    }
    
    public void testUpdateWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError, Exception
    {
    	update(msScope, true);
    }
    
    public void testDeleteChild() throws AssertionError, Exception
    {
        PoolDomainScope mScope = PoolDomainFixture.createScope(3, NB_LEVELS);
        
    	PoolDomain poolDomain = PoolDomainFixture.createFixture(
                NB_POOL_DOMAINS_BY_LEVEL, mScope);

        //Insert the pool domain into the DB
    	SingleResult result = save(poolDomain, mScope);

        //compare the returned pool domain
        PoolDomain returnedPoolDomain = (PoolDomain) result.getRpmObject();
        PoolDomainFixture.compare(poolDomain, returnedPoolDomain, mScope);

        //because the compare have modified the scope
        mScope = PoolDomainFixture.createScope(3, NB_LEVELS);
        
        RPMObject[] rpmObjects = loadFromXpath("/PoolDomain[name='" + poolDomain.getName() +"']", mScope);
        PoolDomainFixture.compare(poolDomain, (PoolDomain)rpmObjects[0], mScope);
        
        PoolDomain mainPoolDomain = (PoolDomain) rpmObjects[0];
		assertNotNull(mainPoolDomain);
		PoolDomain myChild11 = mainPoolDomain.getPoolDomains()[0];
		assertNotNull(myChild11);
		PoolDomain myChild12 = mainPoolDomain.getPoolDomains()[1];
		assertNotNull(myChild12);
		
    	delete(myChild12);
        rpmObjects = loadFromXpath("/PoolDomain[name='"+ myChild12.getName() +"']", mScope);
        assertNull(rpmObjects);
    }


    private void insert(PoolDomainScope poolDomainScope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert a pool domain
        PoolDomain poolDomain = insertPoolDomain(poolDomainScope, loadFromId);
    	
    	//delete the pool domain
    	deletePoolDomain(poolDomain, loadFromId);    	
    }
    
    private void update(PoolDomainScope poolDomainScope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert a pool domain
    	PoolDomain poolDomain = insertPoolDomain(poolDomainScope, loadFromId);
    	
        //update the pool domain
    	poolDomain = updatePoolDomainFields(poolDomain, poolDomainScope, loadFromId);
    	poolDomain = updatePoolDomainStructure(poolDomain, poolDomainScope, loadFromId);
    	
    	//delete the pool domain
    	deletePoolDomain(poolDomain, loadFromId);
    }
    
    private PoolDomain insertPoolDomain(PoolDomainScope scope, boolean loadFromId) throws Exception
    {
    	PoolDomain poolDomain = PoolDomainFixture.createFixture(NB_POOL_DOMAINS_BY_LEVEL, scope);
    	       
        //Insert the pool domain into the DB
    	SingleResult result =  save(poolDomain,scope);
        
        //compare the returned pool domain
        PoolDomain returnedPoolDomain = (PoolDomain)result.getRpmObject();
        PoolDomainFixture.compare(poolDomain, returnedPoolDomain, scope);
        
        //reload the pool domain and compare it
        returnedPoolDomain = (PoolDomain)load(returnedPoolDomain, scope, loadFromId);
        PoolDomainFixture.compare(poolDomain, returnedPoolDomain, scope);
        
        //return the pool domain
        return returnedPoolDomain;
    }
    
    private PoolDomain updatePoolDomainFields(PoolDomain poolDomain, PoolDomainScope scope, boolean loadFromId) throws Exception
	{
    	return updatePoolDomain(poolDomain, scope, loadFromId, false);
	}
    
    private PoolDomain updatePoolDomainStructure(PoolDomain poolDomain, PoolDomainScope scope, boolean loadFromId) throws Exception
	{
    	return updatePoolDomain(poolDomain, scope, loadFromId, true);
	}

    private PoolDomain updatePoolDomain(PoolDomain poolDomain, PoolDomainScope scope, boolean loadFromId, boolean updateStructure) throws Exception
	{
    	if(updateStructure){
            //updates the structure by adding elements 
    		PoolDomainFixture.updateFixtureStructure(poolDomain, DELTA_NB_POOL_DOMAINS_BY_LEVEL, NB_POOL_DOMAINS_BY_LEVEL, scope);    		
    	} else {
    		//only updates the fields
    		PoolDomainFixture.updateFixtureFields(poolDomain, scope);
    	}

        //Insert the pool domain into the DB
    	SingleResult result =  save(poolDomain,scope);
        
        //compare the returned pool domain
        PoolDomain returnedPoolDomain = (PoolDomain)result.getRpmObject();
        PoolDomainFixture.compare(poolDomain, returnedPoolDomain, scope);
        
        //reload the pool domain and compare it
        returnedPoolDomain = (PoolDomain)load(returnedPoolDomain, scope, loadFromId);
        PoolDomainFixture.compare(poolDomain, returnedPoolDomain, scope);
        
        //return the pool domain
        return returnedPoolDomain;
	}
    
    private void deletePoolDomain(PoolDomain poolDomain, boolean loadFromId)throws Exception
	{    	   	
        //delete the pool domain
       	delete(poolDomain);
    	
        //reload the deleted pool domain
       	PoolDomain deletedPoolDomain = loadDeletedPoolDomain(poolDomain, loadFromId);
        
        //validate deleted pool domain
        assertNull(deletedPoolDomain);
    }
    
    private RPMObject load(PoolDomain poolDomain, PoolDomainScope poolDomainScope, boolean loadFromId) throws Exception
	{
    	if(loadFromId){
            //load from an id
            RPMObject obj = new PoolDomain();
            obj.setID(poolDomain.getID());
            return loadFromID(obj, poolDomainScope);
    	}
    	
        //load from an xpath
        RPMObject[] objs = loadFromXpath("/POOLDOMAIN[id='"
                + poolDomain.getID() + "']", poolDomainScope);
        if (objs == null || objs.length == 0)
        {
            return null;
        }
        return objs[0];
    }
    
    private PoolDomain loadDeletedPoolDomain(PoolDomain poolDomain, boolean loadFromId)throws Exception{
        try{
        	poolDomain = (PoolDomain)load(poolDomain, null, loadFromId);
           	return poolDomain;
        }catch (Exception e){
            //pool domain does not exist
        }
        return null;
    }
	
}
