package com.ibm.rpm.client.junit.applicationadministration.testcases;


import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.SecurityRoleFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.security.containers.GenericSecurityRole;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.containers.SecurityRoleCategory;
import com.ibm.rpm.security.scope.SecurityRoleScope;

/**
 * @author karim
 */
public class TestSecurityRoles extends ClientTestController
{
    private SecurityRoleScope mainScope;

    private SecurityRoleScope fullScope;

    private SecurityRole parentSecurityRole;
    
    private static final String aCategoryId = "SCPROJECTGROUP";
        
    private static final String aRoleId = "SCPROJECTMANAGER";
            
    public TestSecurityRoles(String testName){
        setName(testName);
    }
    
    public void setUp() throws Exception
    {
        logger.info("setUp");
		super.setUp();

		mainScope = SecurityRoleFixture.createScope();
		fullScope = new SecurityRoleScope();
		fullScope.setSecurityRoles(fullScope);
		fullScope.setParent(fullScope);
		
		parentSecurityRole = new SecurityRole();
		parentSecurityRole.setID(aRoleId);
    }
   
    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testGetProjectManagerSecurityRole() throws Exception
    {

        RPMObject[] loadResult = loadFromXpath(
            "/SecurityRole[name='Project Manager']", null);
        assertNotNull(loadResult);
        ProjectSecurityRole psr = (ProjectSecurityRole) loadResult[0];
        assertNotNull(psr);

        SecurityRoleScope mainScope = new SecurityRoleScope();

        ProjectSecurityRole childsr = new ProjectSecurityRole();
        childsr.setName("test");
        childsr.setParent(psr);
        childsr.setCanImportExport(new Boolean(false));
        mainScope.setSecurityRoles(new SecurityRoleScope());

        SaveResult result = save(childsr, mainScope);
        ProjectSecurityRole object = (ProjectSecurityRole) result
            .getRpmObject();
        object.setCanImportExport(new Boolean(true));
        //must set parent for user-defined roles before calling save on them
        object.setParent(psr);
        SaveResult result1 = save(object, mainScope);
    }
   
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError, Exception
	{
    	insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(mainScope, true);
    }
    
    /* SecurityRole is not fully implemented and these tests will not currently pass.
    public void testUpdateWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError, Exception
    {
    	update(mainScope, true);
    }
    */
    
    public void testXPathChildrenFromParent() throws AssertionError, Exception
    {
        doTestXPathCategory("/SecurityRoleCategory[id='" + aCategoryId
                + "']/securityRoles", fullScope);
    }

    public void testXPathParentFromChild() throws AssertionError, Exception
    {
        doTestXPathCategory("/SecurityRoleCategory[securityRoles[id='"
                + aRoleId + "']]", fullScope);
    }

    public void testXPathAllGenericCategories() throws AssertionError,
            Exception
    {
        doTestXPathCategory("/GenericSecurityRole", null);
    }

    public void testXPathAllCategories() throws AssertionError, Exception
    {
        doTestXPathCategory("/SecurityRoleCategory", fullScope);
    }

    public void testXPathAllRoles() throws AssertionError, Exception
    {
        doTestXPathCategory("/SecurityRole", null);
    }

    public void testXPathSpecificRoles() throws AssertionError, Exception
    {
        doTestXPathCategory("/ProjectSecurityRole", null);
    }

    private void doTestXPathCategory(String xPath, SecurityRoleScope scope)
            throws AssertionError, Exception
    {
        RPMObject[] objects = loadFromXpath(xPath, scope);
        assertNotNull(objects);
        if (scope == fullScope)
        {
            GenericSecurityRole firstObject = (GenericSecurityRole) objects[0];
            if (firstObject instanceof SecurityRole)
            {
                assertNotNull(firstObject.getParent());
            }
            if (firstObject instanceof SecurityRoleCategory)
            {
                assertNotNull(firstObject.getSecurityRoles());
            }
        }
    }
    
    private void insert(SecurityRoleScope securityRoleScope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert an element
        SecurityRole securityRole = insertSecurityRole(securityRoleScope, loadFromId);
    	
    	//delete the element
    	deleteSecurityRole(securityRole, loadFromId);    	
    }
    
    private void update(SecurityRoleScope securityRoleScope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert an element
        SecurityRole securityRole = insertSecurityRole(securityRoleScope, loadFromId);
    	
        //update the element
    	securityRole = updateSecurityRole(securityRole, securityRoleScope, loadFromId);
    	
    	//delete the element
    	deleteSecurityRole(securityRole, loadFromId);
    }
    
    private SecurityRole insertSecurityRole(SecurityRoleScope scope, boolean loadFromId) throws Exception
    {
        //load parent
		parentSecurityRole = load(parentSecurityRole, null, loadFromId);
        
		//create element
        SecurityRole securityRole = SecurityRoleFixture.createFixture(parentSecurityRole);
    	       
        //Insert the element into the DB
    	SingleResult result =  save(securityRole,scope);
        
        //compare the returned element
    	SecurityRole returnedSecurityRole = (SecurityRole)result.getRpmObject();
    	SecurityRoleFixture.compare(securityRole, returnedSecurityRole, scope);
        
        //reload the element and compare it
        returnedSecurityRole = load(returnedSecurityRole, scope, loadFromId);
        SecurityRoleFixture.compare(securityRole, returnedSecurityRole, scope);
        
        //return the element
        return returnedSecurityRole;
    }
    

    private SecurityRole updateSecurityRole(SecurityRole securityRole, SecurityRoleScope scope, boolean loadFromId) throws Exception
	{
   		//only updates the fields
        SecurityRoleFixture.updateFixture(securityRole);

        //Insert the element into the DB
    	SingleResult result =  save(securityRole,scope);
        
        //compare the returned element
    	SecurityRole returnedSecurityRole = (SecurityRole)result.getRpmObject();
    	SecurityRoleFixture.compare(securityRole, returnedSecurityRole, scope);
        
        //reload the element and compare it
        returnedSecurityRole = load(returnedSecurityRole, scope, loadFromId);
        SecurityRoleFixture.compare(securityRole, returnedSecurityRole, scope);
        
        //return the element
        return returnedSecurityRole;
	}
    
    private void deleteSecurityRole(SecurityRole securityRole, boolean loadFromId)throws Exception
	{    	   	
        //delete the element
       	delete(securityRole);
    	
        //reload the deleted element
       	SecurityRole deletedSecurityRole = loadDeletedSecurityRole(securityRole, loadFromId);
        
        //validate deleted element
        assertNull(deletedSecurityRole);
    }
    
    private SecurityRole load(SecurityRole securityRole, SecurityRoleScope securityRoleScope, boolean loadFromId) throws Exception
	{
    	if(loadFromId){
            //load from an id
            RPMObject obj = new SecurityRole();
            obj.setID(securityRole.getID());
            return (SecurityRole)loadFromID(obj, securityRoleScope);
    	}
    		//load from an xpath
        RPMObject[] objs = loadFromXpath("/SecurityRole[id='"
                + securityRole.getID() + "']", securityRoleScope);
        if (objs == null || objs.length == 0)
        {
            return null;
        }
        return (SecurityRole)objs[0];
    	
    }
    
    private SecurityRole loadDeletedSecurityRole(SecurityRole securityRole, boolean loadFromId)throws Exception{
        try{
            securityRole = load(securityRole, null, loadFromId);
           	return securityRole;
        }catch (Exception e){
            //element does not exist
        }
        return null;
    }
    
}
