package com.ibm.rpm.client.junit.applicationadministration.testcases;

import java.util.ArrayList;

import com.ibm.rpm.applicationadministration.containers.ValueDatafield;
import com.ibm.rpm.applicationadministration.containers.DatafieldRTF;
import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.applicationadministration.containers.GenericDatafield;
import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.framework.testcases.util.PackageUtil;
import com.ibm.rpm.client.junit.framework.testcases.xpath.XpathTesterUtils;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;

/**
 * @author Genevieve
 */
public class TestValueDatafield extends ClientTestController
{

    private ArrayList datafieldsClasses;

    private ArrayList originalDatafields;

    private int MAX_DATAFIELD_INSTANCES_TO_TEST = 5;

    protected void setUp() throws Exception
    {
        super.setUp();

        datafieldsClasses = getDatafieldClasses();
        originalDatafields = new ArrayList();
    }

    protected void tearDown() throws Exception
    {
        // Update datafields to their original values
        for (int i = 0; i < originalDatafields.size(); ++i)
        {
            save((ValueDatafield) originalDatafields.get(i), null);
        }

        super.tearDown();
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        for (int i = 0; i < datafieldsClasses.size(); ++i)
        {
            RPMObject[] allInstances = loadAllInstances((Class) datafieldsClasses
                    .get(i));
            if (allInstances != null)
            {
                // We only test the update on up to
                // MAX_DATAFIELD_INSTANCES_TO_TEST
                // because some datafields have up to 1000 instances
                for (int j = 0; j < allInstances.length
                        && j < MAX_DATAFIELD_INSTANCES_TO_TEST; ++j)
                {
                    originalDatafields.add(allInstances[j]);
                    update((ValueDatafield) allInstances[j], true);
                }
            }
        }
    }

    private void update(ValueDatafield obj, boolean loadFromId)
            throws AssertionError, Exception
    {
        // Load the datafield
        obj = (ValueDatafield) load(obj, loadFromId);

        // Update the datafield
        obj = updateDatafield(obj, loadFromId);
    }

    private ValueDatafield updateDatafield(ValueDatafield obj, boolean loadFromId)
            throws Exception
    {
        // Only updates the fields
        DatafieldFixture.updateFixture(obj);

        // Update the ValueDatafield
        SingleResult result = save(obj, null);

        // Compare the returned ValueDatafield
        ValueDatafield returnedDatafield = (ValueDatafield) result.getRpmObject();
        DatafieldFixture.compare(obj, returnedDatafield);

        // Reload the ValueDatafield and compare it
        returnedDatafield = (ValueDatafield) load(returnedDatafield, loadFromId);
        DatafieldFixture.compare(obj, returnedDatafield);

        // Return the ValueDatafield
        return obj;
    }

    private RPMObject load(ValueDatafield obj, boolean loadFromId) throws Exception
    {
        // Load from the ID
        if (loadFromId)
        {
            RPMObject rpmObject = new PoolDomain();
            rpmObject.setID(obj.getID());
            return loadFromID(obj, null);
        }

        // Load from an XPath
        RPMObject[] objs = loadFromXpath("/"
                + StringUtil.getShortClassName(obj.getClass()) + "[id="
                + obj.getID() + "]", null);
        if (objs == null || objs.length == 0)
        {
            return null;
        }
        return objs[0];
    }

    private RPMObject[] loadAllInstances(Class datafieldClass) throws Exception
    {
        StringBuffer xpath = new StringBuffer();
        String shortClassName = StringUtil.getShortClassName(datafieldClass
                .getName());
        xpath.append(XpathTesterUtils.FORWARD_SLASH).append(shortClassName);

        RPMObject loadedObjects[] = loadFromXpath(xpath.toString(), null);
        return loadedObjects;
    }

    private static ArrayList getDatafieldClasses()
    {
        ArrayList allDatafieldClasses = PackageUtil.getClasses(
                GenericDatafield.class.getPackage(), ValueDatafield.class);
        allDatafieldClasses.remove(ValueDatafield.class);
        allDatafieldClasses.remove(DatafieldRTF.class);
        allDatafieldClasses.remove(DatafieldState.class);
        return allDatafieldClasses;
    }
}
