/*
 * Created on Sep 26, 2006
 */
package com.ibm.rpm.client.junit.applicationadministration.testcases.fixes;

import com.ibm.rpm.applicationadministration.containers.ActionRTF;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestRTF;
import com.ibm.rpm.applicationadministration.containers.DefectRTF;
import com.ibm.rpm.applicationadministration.containers.IssueRTF;
import com.ibm.rpm.applicationadministration.containers.RequirementRTF;
import com.ibm.rpm.applicationadministration.containers.RiskRTF;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestRTF;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RtfAssignmentFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author dominiqu
 */
public class Test49667 extends ClientTestController
{
    ScopeElement scopeElement = null;

    ScopeElement scopeParent = null;

    ScopeElementScope scopeElementScope = null;

    WorkElement wbsElement = null;

    WorkElementScope wbsElementScope = null;

    int NB_CHILDREN = 2;

    public void setUp() throws Exception
    {
        super.setUp();

        scopeElementScope = new ScopeElementScope();
        scopeElementScope.setRtfAssignments(true);

        wbsElementScope = new WorkElementScope();
        wbsElementScope.setRtfAssignments(true);

        // create a Proposal and save it; this will create a ScopeFolder
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);
        scopeParent = (ScopeFolder) rpmObjects[0];
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testActionTaskRtfAssignmentValidation() throws Exception
    {
        scopeElement = ScopeElementFixture.createActionFixture(
            scopeElementScope, NB_CHILDREN, scopeParent, this);
        ((Action) scopeElement).setTask(new Boolean(true));

        insertAndUpdate();

        assertTrue(wbsElement.getRtfAssignments()[0].getRtf() instanceof ActionRTF);
    }

    public void testChangeRequestTaskRtfAssignmentValidation() throws Exception
    {
        scopeElement = ScopeElementFixture.createChangeRequestFixture(
            scopeElementScope, NB_CHILDREN, scopeParent, this);
        ((ChangeRequest) scopeElement).setTask(new Boolean(true));

        insertAndUpdate();

        assertTrue(wbsElement.getRtfAssignments()[0].getRtf() instanceof ChangeRequestRTF);
    }

    public void testDefectTaskRtfAssignmentValidation() throws Exception
    {
        scopeElement = ScopeElementFixture.createDefectFixture(
            scopeElementScope, NB_CHILDREN, scopeParent, this);
        ((Defect) scopeElement).setTask(new Boolean(true));

        insertAndUpdate();

        assertTrue(wbsElement.getRtfAssignments()[0].getRtf() instanceof DefectRTF);
    }

    public void testIssueTaskRtfAssignmentValidation() throws Exception
    {
        scopeElement = ScopeElementFixture.createIssueFixture(
            scopeElementScope, NB_CHILDREN, scopeParent, this);
        ((Issue) scopeElement).setTask(new Boolean(true));

        insertAndUpdate();

        assertTrue(wbsElement.getRtfAssignments()[0].getRtf() instanceof IssueRTF);
    }
    
    public void testRequirementTaskRtfAssignmentValidation() throws Exception
    {
        scopeElement = ScopeElementFixture.createRequirementFixture(
            scopeElementScope, NB_CHILDREN, scopeParent, this);
        ((Requirement) scopeElement).setTask(new Boolean(true));

        insertAndUpdate();

        assertTrue(wbsElement.getRtfAssignments()[0].getRtf() instanceof RequirementRTF);
    }

    public void testRiskTaskRtfAssignmentValidation() throws Exception
    {
        scopeElement = ScopeElementFixture.createRiskFixture(scopeElementScope,
            NB_CHILDREN, scopeParent, this);
        ((Risk) scopeElement).setTask(new Boolean(true));

        insertAndUpdate();

        assertTrue(wbsElement.getRtfAssignments()[0].getRtf() instanceof RiskRTF);
    }

    public void testServiceRequestTaskRtfAssignmentValidation()
            throws Exception
    {
        scopeElement = ScopeElementFixture.createServiceRequestFixture(
            scopeElementScope, NB_CHILDREN, scopeParent, this);
        ((ServiceRequest) scopeElement).setTask(new Boolean(true));

        insertAndUpdate();

        assertTrue(wbsElement.getRtfAssignments()[0].getRtf() instanceof ServiceRequestRTF);
    }

    public void insertAndUpdate() throws Exception
    {
        SingleResult result = save(scopeElement, scopeElementScope);

        assertNull(result.getErrors());

        scopeParent = (ScopeElement) result.getRpmObject();
        scopeParent.setParent(scopeParent);

        Result result2 = getApplicationInterface().transferUnderWBS(
            getSessionID(), null, (AbstractScope) scopeParent);

        assertNull(result2.getErrors());

        wbsElement = new WorkElement();
        wbsElement.setID(scopeElement.getID());
        wbsElement = (WorkElement) loadFromID(wbsElement, wbsElementScope);

        assertNotNull(
            StringUtil.getShortClassName(scopeElement.getClass())
                    + " Task RtfAssignments were not loaded after transfer under WBS",
            wbsElement.getRtfAssignments());
        assertNotNull(
            StringUtil.getShortClassName(scopeElement.getClass())
                    + " Task RtfAssignments were not loaded after transfer under WBS",
            wbsElement.getRtfAssignments()[0]);

        // Update the fixture, including new rtf assignments, and save
        RtfAssignment[] rtfAssignments = RtfAssignmentFixture.createAllAssignmentFixtures(
            wbsElement, this);
        wbsElement.setRtfAssignments(rtfAssignments);

        SingleResult result3 = save(wbsElement, wbsElementScope);

        assertNull(result3.getErrors());

        wbsElement = (WorkElement) result3.getRpmObject();

        assertNotNull(wbsElement);
        assertNotNull(wbsElement.getRtfAssignments());
        assertNotNull(wbsElement.getRtfAssignments()[0]);
    }
}
