package com.ibm.rpm.client.junit.applicationadministration.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.RpmTestProperties;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.scope.SecurityGroupScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestRATLC00252745 extends ClientTestController {
	
	private Proposal proposal;
	
	protected void setUp() throws Exception
	{
		super.setUp();
		proposal = (Proposal)createSaveWorkElement(new Proposal(), null);
		createSaveWorkElement(new Task(), proposal);
	}

	public void testLoadResource() throws Exception
	{
		loadAndValidateAdminUser();
		loadAndValidateNormalUser();
	}
	
	private void loadAndValidateAdminUser() throws Exception
	{
		String admin_username = RpmTestProperties.getProperty("rpm.test.user.name");
		loadAndValidateResource(admin_username);
	}
	
	private void loadAndValidateNormalUser() throws Exception
	{
		String sec_username = RpmTestProperties.getProperty("rpm.test.security.user.name");
		loadAndValidateResource(sec_username);
	}
	
	public void testLoadObjectWithDifferentScopesInSingleCall() throws Exception
	{
		WorkElementScope workElementScope = new WorkElementScope();
		WorkElementScope childrenScope = new WorkElementScope();
		WorkElementScope parentScope = new WorkElementScope();
		parentScope.setDocumentFolder(new DocumentScope());
		WorkElementScope childrenScope1 = new WorkElementScope();
		childrenScope1.setParent(parentScope);
		childrenScope1.setChildren(new WorkElementScope());
		parentScope.setChildren(childrenScope1);
		childrenScope.setParent(parentScope);
		workElementScope.setChildren(childrenScope);
		
		proposal = (Proposal)loadFromID(proposal, workElementScope);
		
		assertNotNull(proposal);
		assertNotNull(proposal.getChildren());
		assertTrue(proposal.getChildren().length == 1);
		assertNotNull(proposal.getChildren()[0]);
		assertNotNull(proposal.getChildren()[0].getParent());
		assertNotNull(((Proposal)proposal.getChildren()[0].getParent()).getDocumentFolder());
	}
	
	private WorkElement createSaveWorkElement(WorkElement baseElement, WorkElement parent) throws Exception
	{
		baseElement = WorkElementFixture.createSaveFixture(parent, baseElement, null, this);
		assertNotNull(baseElement);
		return baseElement;
	}
	
	private void loadAndValidateResource(String username) throws Exception
	{
		String xPath = "/Resource [username = '" + username + "']";
		ResourceScope resourceScope = new ResourceScope();
		resourceScope.setSecurityGroup(new SecurityGroupScope());
		Resource resource = (Resource)loadOneFromXpath(xPath, resourceScope);
		assertNotNull(resource);
		assertNotNull(resource.getSecurityGroup());
	}
}
