package com.ibm.rpm.client.junit.applicationadministration.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.containers.Issue;

/**
 * @author dominiqu
 */
public class TestAttributeAssignmentValidation extends GenericValidation
{
    private ClientCostCenterScope mainScope;

    private ClientCostCenter mainCCC;

    private AttributeAssignment[] cccAttributes;

    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ClientCostCenterScope();
        mainScope.setAttributeAssignments(true);

        // creates 2 attributes
        mainCCC = ClientCostCenterFixture.createFixture(mainScope, this);

        cccAttributes = new AttributeAssignment[3];
        cccAttributes[0] = AttributeAssignmentFixture.createAssignmentFixture(
            mainCCC, this);
        ((Attribute) cccAttributes[0].getAttribute())
            .setInputType(InputType.None);
        cccAttributes[0].setValue(null);
        cccAttributes[1] = AttributeAssignmentFixture.createAssignmentFixture(
            mainCCC, this);
        ((Attribute) cccAttributes[1].getAttribute())
            .setInputType(InputType.Numeric);
        cccAttributes[1].setValue("2");
        cccAttributes[2] = AttributeAssignmentFixture.createAssignmentFixture(
            mainCCC, this);
        ((Attribute) cccAttributes[2].getAttribute())
            .setInputType(InputType.Text);
        cccAttributes[2].setValue("abc");
        

        mainCCC.setAttributeAssignments(cccAttributes);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        cccAttributes[0].setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, cccAttributes[0]
                .getClass().getName(), "parent", cccAttributes[0].getID())
        };

        verifySaveMustFail(cccAttributes[0], null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        cccAttributes[0].setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, cccAttributes[0]
                .getClass().getName(), "parent", cccAttributes[0].getID())
        };

        verifySaveMustFail(cccAttributes[0], null, expectedExceptions);
    }

    public void testFailureWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        mainCCC.getAttributeAssignments()[0].setAttribute(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, mainCCC
                .getAttributeAssignments()[0].getClass().getName(),
                "attribute", mainCCC.getAttributeAssignments()[0].getID())
        };

        verifySaveMustFail(mainCCC, mainScope, expectedExceptions);
    }

    public void testFailureWithInvalidAttributeType() throws AssertionError,
            Exception
    {
        AttributeAssignment[] wrongAttributes = new AttributeAssignment[2];

        AttributeClassification attrClassification = new AttributeClassification();
        attrClassification.setID("Not null"); // Set a random id because it must
        // exist
        wrongAttributes[0] = AttributeAssignmentFixture
            .createAssignmentFixture(mainCCC, this);
        wrongAttributes[0].setValue(null);
        wrongAttributes[0].setAttribute(attrClassification);
        
        AttributeCategory attrCategory = new AttributeCategory();
        attrCategory.setID("Not null"); // Set a random id because it must exist
        wrongAttributes[1] = AttributeAssignmentFixture
            .createAssignmentFixture(mainCCC, this);
        wrongAttributes[1].setAttribute(attrCategory);
        mainCCC.setAttributeAssignments(wrongAttributes);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainCCC
                    .getAttributeAssignments()[0].getClass().getName(),
                    "parent", mainCCC.getAttributeAssignments()[0].getID()),
                createRPMException(ErrorCodes.INVALID_TYPE, mainCCC
                    .getAttributeAssignments()[1].getClass().getName(),
                    "attribute", mainCCC.getAttributeAssignments()[1].getID())
        };

        verifySaveMustFail(mainCCC, mainScope, expectedExceptions);
    }
    
    public void testFailureWithInvalidValue() throws AssertionError,
    Exception
	{
		// InputType.Numeric must have a valid number specified as a string.
		cccAttributes[1].setValue("abc");
		
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    com.ibm.rpm.applicationadministration.constants.ErrorCodes.ATTRIBUTE_ASSIGNMENT_STRING_MUST_BE_VALID_NUMBER,
                    mainCCC.getAttributeAssignments()[0].getClass().getName(),
                    "value", mainCCC.getAttributeAssignments()[0].getID())
        };

        verifySaveMustFail(mainCCC, mainScope, expectedExceptions);
	}
    
    public void testFailureWithInvalidValueType() throws AssertionError,
            Exception
    {
        // InputType.Numeric must have a valid number specified as a string.
        ((Attribute) cccAttributes[0].getAttribute())
            .setInputType(InputType.Numeric);
        cccAttributes[0].setValue(new Issue());
        // InputType.Numeric cannot have a calendar object for value.
        // (Valid types: String, Integer, Double)
        cccAttributes[1].setValue(Calendar.getInstance());
        // InputType.Text cannot have an Integer object for value.
        // (Valid type: String)
        cccAttributes[2].setValue(new Integer(2));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.INVALID_TYPE, mainCCC
                    .getAttributeAssignments()[0].getClass().getName(),
                    "value", mainCCC.getAttributeAssignments()[0].getID()),
                createRPMException(ErrorCodes.INVALID_TYPE, mainCCC
                    .getAttributeAssignments()[1].getClass().getName(),
                    "value", mainCCC.getAttributeAssignments()[1].getID()),
                createRPMException(ErrorCodes.INVALID_TYPE, mainCCC
                    .getAttributeAssignments()[2].getClass().getName(),
                    "value", mainCCC.getAttributeAssignments()[2].getID())
        };

        verifySaveMustFail(mainCCC, mainScope, expectedExceptions);
    }

    public void testSuccessValidationWithWarningValueTypes()
            throws AssertionError, Exception
    {
        // Set value with valid field types according to Date
        // InputType.Numeric valid types with warning: String
        ((Attribute) cccAttributes[0].getAttribute())
            .setInputType(InputType.Date);
        cccAttributes[0].setValue("1983-09-09");
        // Set value with valid field types according to InputType.Numeric
        // InputType.Numeric valid types with warning: Integer, Double
        cccAttributes[1].setValue(new Double(
            ValidationConstants.ATTRIBUTEASSIGNMENT_NUMERIC_MAX));
        ((Attribute) cccAttributes[2].getAttribute())
            .setInputType(InputType.Numeric);
        cccAttributes[2].setValue(new Integer(2));

        // Attempt to save the ClientCostCenter object with the attributes
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(com.ibm.rpm.applicationadministration.constants.ErrorCodes.ATTRIBUTE_ASSIGNMENT_WARNING_INPUT_NOT_THE_SAME_AS_OUTPUT, null),
                createRPMException(com.ibm.rpm.applicationadministration.constants.ErrorCodes.ATTRIBUTE_ASSIGNMENT_WARNING_INPUT_NOT_THE_SAME_AS_OUTPUT, null),
                createRPMException(com.ibm.rpm.applicationadministration.constants.ErrorCodes.ATTRIBUTE_ASSIGNMENT_WARNING_INPUT_NOT_THE_SAME_AS_OUTPUT, null)
        };
        
        verifySaveMustFail(mainCCC, mainScope, null, expectedWarnings);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with valid max and verify that they are accepted
        cccAttributes[1].setValue(String
            .valueOf(ValidationConstants.ATTRIBUTEASSIGNMENT_NUMERIC_MAX));
        cccAttributes[2]
            .setValue(getTestString(ValidationConstants.ATTRIBUTEASSIGNMENT_TEXT_MAX));

        // Attempt to save the ClientCostCenter object with the attributes
        mainCCC = (ClientCostCenter) saveMustSucceed(mainCCC,
            ClientCostCenter.class, mainScope);

        // Load the inserted ClientCostCenter from id
        RPMObject obj = new ClientCostCenter();
        obj.setID(mainCCC.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ClientCostCenter);

        // Check that the loaded attribute assignments are identical to the ones
        // we requested to store
        AttributeAssignmentFixture.compare(mainCCC.getAttributeAssignments(),
            ((ClientCostCenter) rpmObject).getAttributeAssignments());
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with valid max and verify that they are accepted
        cccAttributes[1].setValue(String
            .valueOf(ValidationConstants.ATTRIBUTEASSIGNMENT_NUMERIC_MIN));

        // Attempt to save the ClientCostCenter object with the attributes
        mainCCC = (ClientCostCenter) saveMustSucceed(mainCCC,
            ClientCostCenter.class, mainScope);

        // Load the inserted ClientCostCenter from id
        RPMObject obj = new ClientCostCenter();
        obj.setID(mainCCC.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ClientCostCenter);

        // Check that the loaded attribute assignments are identical to the ones
        // we requested to store
        AttributeAssignmentFixture.compare(mainCCC.getAttributeAssignments(),
            ((ClientCostCenter) rpmObject).getAttributeAssignments());
    }

    public void testSuccessValidationNull() throws AssertionError, Exception
    {
        // Set field values with null and verify that they are accepted
        cccAttributes[0].setValue(null);
        cccAttributes[1].setValue(null);
        cccAttributes[2].setValue(null);

        // Attempt to save the Resource object with the compentency and skill
        mainCCC = (ClientCostCenter) saveMustSucceed(mainCCC,
            ClientCostCenter.class, mainScope);

        // Load the inserted ClientCostCenter from id
        RPMObject obj = new ClientCostCenter();
        obj.setID(mainCCC.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ClientCostCenter);

        // Check that the loaded attribute assignments are identical to the ones
        // we requested to store
        AttributeAssignmentFixture.compare(mainCCC.getAttributeAssignments(),
            ((ClientCostCenter) rpmObject).getAttributeAssignments());
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set field values with max+1 or max*10 and verify that they are not accepted
        // For exponents, only so many bits of precision in Java so cannot test with max+1
        cccAttributes[0].setValue("1");
        cccAttributes[1]
            .setValue(String
                .valueOf(ValidationConstants.ATTRIBUTEASSIGNMENT_NUMERIC_MAX * 10));
        cccAttributes[2]
            .setValue(getTestString(ValidationConstants.ATTRIBUTEASSIGNMENT_TEXT_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MUST_BE_NULL, cccAttributes[0]
                    .getClass().getName(), "value", cccAttributes[0].getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, cccAttributes[1]
                    .getClass().getName(), "value", cccAttributes[1].getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, cccAttributes[2]
                    .getClass().getName(), "value", cccAttributes[2].getID())
        };

        verifySaveMustFail(mainCCC, mainScope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set field values with min*10 and verify that they are not accepted
        // Only so many bits of precision in Java so cannot test with min-1
        cccAttributes[1]
            .setValue(new Double(ValidationConstants.ATTRIBUTEASSIGNMENT_NUMERIC_MIN*10));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, cccAttributes[1]
                .getClass().getName(), "value", cccAttributes[1].getID())
        };

        verifySaveMustFail(mainCCC, mainScope, expectedExceptions);
    }
    
    public void testUpdateFailureValidationMandatory() throws AssertionError, Exception
    {
    	mainCCC = (ClientCostCenter)save(mainCCC,mainScope).getRpmObject();
    	
    	mainCCC.getAttributeAssignments()[0].setParent(null);
    	
    	RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, mainCCC.getAttributeAssignments()[0]
                .getClass().getName(), ValidationConstants.PARENT_FIELD, mainCCC.getAttributeAssignments()[0].getID())
        };
    	
    	verifySaveMustFail(mainCCC.getAttributeAssignments()[0], null, expectedExceptions);
    }
    
    public void testDeleteFailureValidationMandatory() throws AssertionError, Exception
    {
    	mainCCC = (ClientCostCenter)save(mainCCC,mainScope).getRpmObject();
    	
    	mainCCC.getAttributeAssignments()[0].setParent(null);
    	
    	RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, mainCCC.getAttributeAssignments()[0]
                .getClass().getName(), ValidationConstants.PARENT_FIELD, mainCCC.getAttributeAssignments()[0].getID())
        };
    	
    	verifyDeleteMustFail(mainCCC.getAttributeAssignments()[0], expectedExceptions, null);
    }
}
