/*
 * Created on 12-Oct-2005
 */

package com.ibm.rpm.client.junit.applicationadministration.testcases.validation;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.Department;
import com.ibm.rpm.applicationadministration.containers.Group;
import com.ibm.rpm.applicationadministration.containers.Office;
import com.ibm.rpm.applicationadministration.containers.Organization;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author dominiqu
 */
public class TestOrganizationalValidation extends GenericValidation
{
    private Organization testOrganization;

    private Department[] testDepartment;

    private Group[] testGroup;

    private Office[] testOffice;

    protected void setUp() throws Exception
    {
        super.setUp();

        // Initialize
        testOrganization = new Organization();
        testDepartment = new Department[1];
        testDepartment[0] = new Department();
        testGroup = new Group[1];
        testGroup[0] = new Group();
        testOffice = new Office[1];
        testOffice[0] = new Office();

        // Set up a valid hierarchy
        testOffice[0].setParent(testGroup[0]);
        testGroup[0].setChildren(testOffice);
        testGroup[0].setParent(testDepartment[0]);
        testDepartment[0].setChildren(testGroup);
        testDepartment[0].setParent(testOrganization);
        testOrganization.setChildren(testDepartment);
        testOrganization.setParent(null);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureOrganizationWithInvalidParent() throws AssertionError,
            Exception
    {
        // An organization with a department as parent shoudn't be allowed
        testOrganization.setParent(testDepartment[0]);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, testOrganization.getClass()
                .getName(), "parent", testOrganization.getID())
        };

        verifySaveMustFail(testOrganization, null, expectedExceptions);
    }

    public void testFailureDepartmentWithoutParent() throws AssertionError, Exception
    {
        testDepartment[0].setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, testDepartment[0].getClass()
                .getName(), "parent", testDepartment[0].getID())
        };

        verifySaveMustFail(testDepartment[0], null, expectedExceptions);
    }

    public void testFailureDepartmentWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        testDepartment[0].setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, testDepartment[0].getClass()
                .getName(), "parent", testDepartment[0].getID())
        };

        verifySaveMustFail(testDepartment[0], null, expectedExceptions);
    }

    public void testFailureGroupWithoutParent() throws AssertionError, Exception
    {
        testGroup[0].setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, testGroup[0].getClass().getName(),
                "parent", testGroup[0].getID())
        };

        verifySaveMustFail(testGroup[0], null, expectedExceptions);
    }

    public void testFailureGroupWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        testGroup[0].setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, testGroup[0].getClass()
                .getName(), "parent", testGroup[0].getID())
        };

        verifySaveMustFail(testGroup[0], null, expectedExceptions);
    }

    public void testFailureOfficeWithoutParent() throws AssertionError, Exception
    {
        testOffice[0].setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, testOffice[0].getClass().getName(),
                "parent", testOffice[0].getID())
        };

        verifySaveMustFail(testOffice[0], null, expectedExceptions);
    }

    public void testFailureOfficeWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        testOffice[0].setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, testOffice[0].getClass()
                .getName(), "parent", testOffice[0].getID())
        };

        verifySaveMustFail(testOffice[0], null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        testOrganization
            .setName(getTestString(ValidationConstants.ORGANIZATIONAL_NAME_MAX));
        testOrganization
            .setDescription(getTestString(ValidationConstants.ORGANIZATIONAL_DESCRIPTION_MAX));
        testDepartment[0]
            .setName(getTestString(ValidationConstants.ORGANIZATIONAL_NAME_MAX));
        testDepartment[0]
            .setDescription(getTestString(ValidationConstants.ORGANIZATIONAL_DESCRIPTION_MAX));
        testGroup[0].setName(getTestString(ValidationConstants.ORGANIZATIONAL_NAME_MAX));
        testGroup[0]
            .setDescription(getTestString(ValidationConstants.ORGANIZATIONAL_DESCRIPTION_MAX));
        testOffice[0].setName(getTestString(ValidationConstants.ORGANIZATIONAL_NAME_MAX));
        testOffice[0]
            .setDescription(getTestString(ValidationConstants.ORGANIZATIONAL_DESCRIPTION_MAX));

        // Attempt to save the organizationals
        RPMObject savedOrganization = (Organization) saveMustSucceed(testOrganization,
            Organization.class, null);
        testOrganization.setID(savedOrganization.getID());
        RPMObject savedDepartment = (Department) saveMustSucceed(testDepartment[0],
            Department.class, null);
        testDepartment[0].setID(savedDepartment.getID());
        RPMObject savedGroup = (Group) saveMustSucceed(testGroup[0], Group.class, null);
        testGroup[0].setID(savedGroup.getID());
        testOffice[0] = (Office) saveMustSucceed(testOffice[0], Office.class, null);

        // Load the inserted organizationals from xpath
        RPMObject[] rpmOrganizations = loadFromXpath("/Organization[id='"
            + testOrganization.getID() + "']", null);
        RPMObject[] rpmDepartments = loadFromXpath("/Department[id='"
            + testDepartment[0].getID() + "']", null);
        RPMObject[] rpmGroups = loadFromXpath(
            "/Group[id='" + testGroup[0].getID() + "']", null);
        RPMObject[] rpmOffices = loadFromXpath("/Office[id='" + testOffice[0].getID()
            + "']", null);

        // Check that the loaded organizationals are identical to the ones we
        // requested to store
        OrganizationalFixture.compare(testOrganization,
            (Organization) rpmOrganizations[0], null);
        OrganizationalFixture.compare(testDepartment[0], (Department) rpmDepartments[0],
            null);
        OrganizationalFixture.compare(testGroup[0], (Group) rpmGroups[0], null);
        OrganizationalFixture.compare(testOffice[0], (Office) rpmOffices[0], null);
    }

    public void testFailureOrganizationValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are accepted
        testOrganization
            .setName(getTestString(ValidationConstants.ORGANIZATIONAL_NAME_MAX + 1));
        testOrganization
            .setDescription(getTestString(ValidationConstants.ORGANIZATIONAL_DESCRIPTION_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, testOrganization.getClass()
                .getName(), "name", testOrganization.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, testOrganization.getClass()
                .getName(), "description", testOrganization.getID())
        };

        verifySaveMustFail(testOrganization, null, expectedExceptions);
    }

    public void testFailureDepartmentValidation() throws AssertionError, Exception
    {
        saveMustSucceed(testOrganization, Organization.class, null);

        // Set field values with maxLength+1 and verify that they are accepted
        testDepartment[0]
            .setName(getTestString(ValidationConstants.ORGANIZATIONAL_NAME_MAX + 1));
        testDepartment[0]
            .setDescription(getTestString(ValidationConstants.ORGANIZATIONAL_DESCRIPTION_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, testDepartment[0].getClass()
                .getName(), "name", testDepartment[0].getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, testDepartment[0].getClass()
                .getName(), "description", testDepartment[0].getID())
        };

        verifySaveMustFail(testDepartment[0], null, expectedExceptions);
    }

    public void testFailureGroupValidation() throws AssertionError, Exception
    {
        RPMObject savedOrganization = (Organization) saveMustSucceed(testOrganization,
            Organization.class, null);
        testOrganization.setID(savedOrganization.getID());
        RPMObject savedDepartment = (Department) saveMustSucceed(testDepartment[0],
            Department.class, null);
        testDepartment[0].setID(savedDepartment.getID());

        // Set field values with maxLength+1 and verify that they are accepted
        testGroup[0]
            .setName(getTestString(ValidationConstants.ORGANIZATIONAL_NAME_MAX + 1));
        testGroup[0]
            .setDescription(getTestString(ValidationConstants.ORGANIZATIONAL_DESCRIPTION_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, testGroup[0].getClass().getName(),
                "name", testGroup[0].getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, testGroup[0].getClass().getName(),
                "description", testGroup[0].getID())
        };

        verifySaveMustFail(testGroup[0], null, expectedExceptions);
    }

    public void testFailureOfficeValidation() throws AssertionError, Exception
    {
        RPMObject savedOrganization = (Organization) saveMustSucceed(testOrganization,
            Organization.class, null);
        testOrganization.setID(savedOrganization.getID());
        RPMObject savedDepartment = (Department) saveMustSucceed(testDepartment[0],
            Department.class, null);
        testDepartment[0].setID(savedDepartment.getID());
        RPMObject savedGroup = (Group) saveMustSucceed(testGroup[0], Group.class, null);
        testGroup[0].setID(savedGroup.getID());

        // Set field values with maxLength+1 and verify that they are accepted
        testOffice[0]
            .setName(getTestString(ValidationConstants.ORGANIZATIONAL_NAME_MAX + 1));
        testOffice[0]
            .setDescription(getTestString(ValidationConstants.ORGANIZATIONAL_DESCRIPTION_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, testOffice[0].getClass().getName(),
                "name", testOffice[0].getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, testOffice[0].getClass().getName(),
                "description", testOffice[0].getID())
        };

        verifySaveMustFail(testOffice[0], null, expectedExceptions);
    }
}
