/*
 * Created on 11-Oct-2005
 */

package com.ibm.rpm.client.junit.applicationadministration.testcases.validation;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.applicationadministration.scope.PoolDomainScope;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.PoolDomainFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

/*
 * @author dominiqu
 */
public class TestPoolDomainValidation extends GenericValidation
{
    private PoolDomainScope mainScope;

    private PoolDomain mainPoolDomain;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainScope = new PoolDomainScope();
        mainPoolDomain = PoolDomainFixture.createFixture(1, mainScope);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        mainPoolDomain.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainPoolDomain.getClass()
                .getName(), "parent", mainPoolDomain.getID())
        };

        verifySaveMustFail(mainPoolDomain, mainScope, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        mainPoolDomain.setName(getTestString(ValidationConstants.POOLDOMAIN_NAME_MAX));
        mainPoolDomain
            .setDescription(getTestString(ValidationConstants.POOLDOMAIN_DESCRIPTION_MAX));

        // Attempt to save the PoolDomain object
        mainPoolDomain = (PoolDomain) saveMustSucceed(mainPoolDomain, PoolDomain.class,
            mainScope);

        // Load the inserted pool domain from xpath
        RPMObject[] rpmObjects = loadFromXpath("/PoolDomain[id='"
            + mainPoolDomain.getID() + "']", mainScope);

        // Check that the loaded pool domain is identical to the one we
        // requested to store
        PoolDomainFixture.compare(mainPoolDomain, (PoolDomain) rpmObjects[0], mainScope);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        mainPoolDomain
            .setName(getTestString(ValidationConstants.POOLDOMAIN_NAME_MAX + 1));
        mainPoolDomain
            .setDescription(getTestString(ValidationConstants.POOLDOMAIN_DESCRIPTION_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH,
                mainPoolDomain.getClass().getName(), "name", mainPoolDomain.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH,
                mainPoolDomain.getClass().getName(), "description", mainPoolDomain
                    .getID())
        };

        verifySaveMustFail(mainPoolDomain, mainScope, expectedExceptions);
    }
}
