/*
 * Created on 11-Oct-2005
 */

package com.ibm.rpm.client.junit.applicationadministration.testcases.validation;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.CalendarWeekDay;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.containers.WorkInterval;
import com.ibm.rpm.applicationadministration.types.CalendarWeekDayType;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author dominiqu
 */
public class TestRPMCalendarValidation extends GenericValidation
{
    private RPMCalendar mainCalendar;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainCalendar = RPMCalendarFixture.createFixture(null);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        mainCalendar.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainCalendar
                .getClass().getName(), "parent", mainCalendar.getID())
        };

        verifySaveMustFail(mainCalendar, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainCalendar
            .setName(getTestString(ValidationConstants.CALENDAR_NAME_MAX));
        mainCalendar.setHoursPerDay(new Double(
            ValidationConstants.CALENDAR_HOURSPERDAY_MAX));
        mainCalendar.setHoursPerWeek(new Double(
            ValidationConstants.CALENDAR_HOURSPERWEEK_MAX));

        // Create a day with the maximum number of work intervals
        WorkInterval[] wis = new WorkInterval[ValidationConstants.CALENDARWEEKDAY_INTERVALS_MAX];
        for (int i = 0; i < ValidationConstants.CALENDARWEEKDAY_INTERVALS_MAX; ++i)
            wis[i] = RPMCalendarFixture.createWorkInterval(+i % 24 + ":15", +i
                    % 24 + ":45");

        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        day.setType(CalendarWeekDayType.NonDefaultWorking);
        day.setIntervals(wis);

        mainCalendar.setStandardMonday(day);
        mainCalendar.setStandardTuesday(day);
        mainCalendar.setStandardWednesday(day);
        mainCalendar.setStandardThursday(day);
        mainCalendar.setStandardFriday(day);
        mainCalendar.setStandardSaturday(day);
        mainCalendar.setStandardSunday(day);

        // Attempt to save the RPMCalendar object
        mainCalendar = (RPMCalendar) saveMustSucceed(mainCalendar,
            RPMCalendar.class, null);

        // Load the inserted calendar from xpath
        RPMObject[] rpmObjects = loadFromXpath("/RPMCalendar[id='"
                + mainCalendar.getID() + "']", null);

        // Check that the loaded calendar is identical to the one we requested
        // to store
        RPMCalendarFixture.compare(mainCalendar, (RPMCalendar) rpmObjects[0],
            null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        mainCalendar = RPMCalendarFixture.createFixture(null, 0, 0);

        // Set field values with min and verify that they are accepted
        mainCalendar.setHoursPerDay(new Double(
            ValidationConstants.CALENDAR_HOURSPERDAY_MIN));
        mainCalendar.setHoursPerWeek(new Double(
            ValidationConstants.CALENDAR_HOURSPERWEEK_MIN));

        // Set default calendar week days
        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        mainCalendar.setStandardMonday(day);
        mainCalendar.setStandardTuesday(day);
        mainCalendar.setStandardWednesday(day);
        mainCalendar.setStandardThursday(day);
        mainCalendar.setStandardFriday(day);
        mainCalendar.setStandardSaturday(day);
        mainCalendar.setStandardSunday(day);

        // Attempt to save the RPMCalendar object
        mainCalendar = (RPMCalendar) saveMustSucceed(mainCalendar,
            RPMCalendar.class, null);

        // Load the inserted calendar from xpath
        RPMObject[] rpmObjects = loadFromXpath("/RPMCalendar[id='"
                + mainCalendar.getID() + "']", null);

        // Check that the loaded calendar is identical to the one we requested
        // to store
        RPMCalendarFixture.compare(mainCalendar, (RPMCalendar) rpmObjects[0],
            null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set field values with max+1 and verify that they are not accepted
        mainCalendar
            .setName(getTestString(ValidationConstants.CALENDAR_NAME_MAX + 1));
        mainCalendar.setHoursPerDay(new Double(
            ValidationConstants.CALENDAR_HOURSPERDAY_MAX + 1));
        mainCalendar.setHoursPerWeek(new Double(
            ValidationConstants.CALENDAR_HOURSPERWEEK_MAX + 1));

        // Create a day with the maximum number of work intervals
        WorkInterval[] wis = new WorkInterval[ValidationConstants.CALENDARWEEKDAY_INTERVALS_MAX + 1];
        for (int i = 0; i < ValidationConstants.CALENDARWEEKDAY_INTERVALS_MAX + 1; ++i)
            wis[i] = RPMCalendarFixture.createWorkInterval(+i % 24 + ":15", +i
                    % 24 + ":30");

        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        day.setType(CalendarWeekDayType.NonDefaultWorking);
        day.setIntervals(wis);

        mainCalendar.setStandardMonday(day);
        mainCalendar.setStandardTuesday(day);
        mainCalendar.setStandardWednesday(day);
        mainCalendar.setStandardThursday(day);
        mainCalendar.setStandardFriday(day);
        mainCalendar.setStandardSaturday(day);
        mainCalendar.setStandardSunday(day);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, mainCalendar
                    .getClass().getName(), "name", mainCalendar.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainCalendar
                    .getClass().getName(), "hoursPerDay", mainCalendar.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainCalendar
                    .getClass().getName(), "hoursPerWeek", mainCalendar.getID()),
                // Should receive the following error 7 times, one for each day
                createRPMException(ErrorCodes.MAX_SIZE, day.getClass()
                    .getName(), "intervals", day.getID()),
                createRPMException(ErrorCodes.MAX_SIZE, day.getClass()
                    .getName(), "intervals", day.getID()),
                createRPMException(ErrorCodes.MAX_SIZE, day.getClass()
                    .getName(), "intervals", day.getID()),
                createRPMException(ErrorCodes.MAX_SIZE, day.getClass()
                    .getName(), "intervals", day.getID()),
                createRPMException(ErrorCodes.MAX_SIZE, day.getClass()
                    .getName(), "intervals", day.getID()),
                createRPMException(ErrorCodes.MAX_SIZE, day.getClass()
                    .getName(), "intervals", day.getID()),
                createRPMException(ErrorCodes.MAX_SIZE, day.getClass()
                    .getName(), "intervals", day.getID())
        };

        verifySaveMustFail(mainCalendar, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        mainCalendar = RPMCalendarFixture.createFixture(null, 0, 0);

        // Set field values with min-1 and verify that they are not accepted
        mainCalendar.setHoursPerDay(new Double(
            ValidationConstants.CALENDAR_HOURSPERDAY_MIN - 1));
        mainCalendar.setHoursPerWeek(new Double(
            ValidationConstants.CALENDAR_HOURSPERWEEK_MIN - 1));

        // Set default calendar week days
        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        mainCalendar.setStandardMonday(day);
        mainCalendar.setStandardTuesday(day);
        mainCalendar.setStandardWednesday(day);
        mainCalendar.setStandardThursday(day);
        mainCalendar.setStandardFriday(day);
        mainCalendar.setStandardSaturday(day);
        mainCalendar.setStandardSunday(day);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainCalendar
                    .getClass().getName(), "hoursPerDay", mainCalendar.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainCalendar
                    .getClass().getName(), "hoursPerWeek", mainCalendar.getID())
        };

        verifySaveMustFail(mainCalendar, null, expectedExceptions);
    }

    public void testSuccessWorkIntervalMinLimits() throws AssertionError,
            Exception
    {
        WorkInterval mainWorkInterval = RPMCalendarFixture.createWorkInterval(
            "0:15", "0:30");

        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        day.setType(CalendarWeekDayType.NonDefaultWorking);
        day.setIntervals(new WorkInterval[]
        {
            mainWorkInterval
        });

        mainCalendar.setStandardMonday(day);

        // Attempt to save the RPMCalendar object
        mainCalendar = (RPMCalendar) saveMustSucceed(mainCalendar,
            RPMCalendar.class, null);

        // Load the inserted calendar from xpath
        RPMObject[] rpmObjects = loadFromXpath("/RPMCalendar[id='"
                + mainCalendar.getID() + "']", null);

        // Check that the loaded calendar is identical to the one we requested
        // to store
        RPMCalendarFixture.compare(mainCalendar, (RPMCalendar) rpmObjects[0],
            null);

    }

    public void testSuccessWorkIntervalMaxLimits() throws AssertionError,
            Exception
    {
        WorkInterval mainWorkInterval = RPMCalendarFixture.createWorkInterval(
            "23:45", "24:00");

        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        day.setType(CalendarWeekDayType.NonDefaultWorking);
        day.setIntervals(new WorkInterval[]
        {
            mainWorkInterval
        });

        mainCalendar.setStandardMonday(day);

        // Attempt to save the RPMCalendar object
        mainCalendar = (RPMCalendar) saveMustSucceed(mainCalendar,
            RPMCalendar.class, null);

        // Load the inserted calendar from xpath
        RPMObject[] rpmObjects = loadFromXpath("/RPMCalendar[id='"
                + mainCalendar.getID() + "']", null);

        // Check that the loaded calendar is identical to the one we requested
        // to store
        RPMCalendarFixture.compare(mainCalendar, (RPMCalendar) rpmObjects[0],
            null);

    }

    public void testFailureWorkIntervalMaxFromTime() throws AssertionError,
            Exception
    {
        WorkInterval mainWorkInterval = RPMCalendarFixture.createWorkInterval(
            "24:00", "24:00");

        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        day.setType(CalendarWeekDayType.NonDefaultWorking);
        day.setIntervals(new WorkInterval[]
        {
            mainWorkInterval
        });

        mainCalendar.setStandardMonday(day);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.applicationadministration.constants.ErrorCodes.INVALID_WORKINTERVAL,
                mainWorkInterval.getClass().getName())
        };

        verifySaveMustFail(mainCalendar, null, expectedExceptions);
    }

    public void testFailureWorkIntervalMinFromTime() throws AssertionError,
            Exception
    {
        WorkInterval mainWorkInterval = RPMCalendarFixture.createWorkInterval(
            "0:00", "00:30");

        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        day.setType(CalendarWeekDayType.NonDefaultWorking);
        day.setIntervals(new WorkInterval[]
        {
            mainWorkInterval
        });

        mainCalendar.setStandardMonday(day);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.applicationadministration.constants.ErrorCodes.INVALID_WORKINTERVAL,
                mainWorkInterval.getClass().getName())
        };

        verifySaveMustFail(mainCalendar, null, expectedExceptions);
    }

    public void testFailureWorkIntervalMaxToTime() throws AssertionError,
            Exception
    {
        WorkInterval mainWorkInterval = RPMCalendarFixture.createWorkInterval(
            "23:00", "24:10");

        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        day.setType(CalendarWeekDayType.NonDefaultWorking);
        day.setIntervals(new WorkInterval[]
        {
            mainWorkInterval
        });

        mainCalendar.setStandardMonday(day);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.applicationadministration.constants.ErrorCodes.INVALID_WORKINTERVAL,
                mainWorkInterval.getClass().getName())
        };

        verifySaveMustFail(mainCalendar, null, expectedExceptions);
    }

    public void testFailureWorkIntervalMinToTime() throws AssertionError,
            Exception
    {
        WorkInterval mainWorkInterval = RPMCalendarFixture.createWorkInterval(
            "0:15", "00:15");

        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        day.setType(CalendarWeekDayType.NonDefaultWorking);
        day.setIntervals(new WorkInterval[]
        {
            mainWorkInterval
        });

        mainCalendar.setStandardMonday(day);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.applicationadministration.constants.ErrorCodes.INVALID_WORKINTERVAL,
                mainWorkInterval.getClass().getName())
        };

        verifySaveMustFail(mainCalendar, null, expectedExceptions);
    }

    public void testFailureInvalidWorkIntervalMinutes() throws AssertionError,
            Exception
    {
        WorkInterval mainWorkInterval = RPMCalendarFixture.createWorkInterval(
            "8:12", "8:45");

        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        day.setType(CalendarWeekDayType.NonDefaultWorking);
        day.setIntervals(new WorkInterval[]
        {
            mainWorkInterval
        });

        mainCalendar.setStandardMonday(day);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.applicationadministration.constants.ErrorCodes.INVALID_WORKINTERVAL,
                mainWorkInterval.getClass().getName())
        };

        verifySaveMustFail(mainCalendar, null, expectedExceptions);
    }

    public void testFailureWithInvalidWorkInterval() throws AssertionError,
            Exception
    {
        WorkInterval mainWorkInterval = RPMCalendarFixture.createWorkInterval(
            "8:00", "7:45");

        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        day.setType(CalendarWeekDayType.NonDefaultWorking);
        day.setIntervals(new WorkInterval[]
        {
            mainWorkInterval
        });

        mainCalendar.setStandardMonday(day);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.applicationadministration.constants.ErrorCodes.INVALID_WORKINTERVAL,
                mainWorkInterval.getClass().getName())
        };

        verifySaveMustFail(mainCalendar, null, expectedExceptions);
    }

    public void testSuccessWithUnsortedWorkIntervals() throws AssertionError,
            Exception
    {
        WorkInterval workInterval1 = RPMCalendarFixture.createWorkInterval(
            "10:00", "12:00");
        WorkInterval workInterval2 = RPMCalendarFixture.createWorkInterval(
            "7:00", "8:00");

        CalendarWeekDay day = new CalendarWeekDay();
        day.setParent(mainCalendar);
        day.setType(CalendarWeekDayType.NonDefaultWorking);
        day.setIntervals(new WorkInterval[]
        {
                workInterval1, workInterval2
        });

        mainCalendar.setStandardMonday(day);

        // Attempt to save the RPMCalendar object
        mainCalendar = (RPMCalendar) saveMustSucceed(mainCalendar,
            RPMCalendar.class, null);

        // Load the inserted calendar from xpath
        RPMObject[] rpmObjects = loadFromXpath("/RPMCalendar[id='"
                + mainCalendar.getID() + "']", null);

        // Check that the loaded calendar is identical to the one we requested
        // to store
        RPMCalendarFixture.compare(mainCalendar, (RPMCalendar) rpmObjects[0],
            null);
    }
}
