package com.ibm.rpm.client.junit.applicationadministration.testcases.validation;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RtfAssignmentFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;

/**
 * @author dominiqu
 */
public class TestRtfAssignmentValidation extends GenericValidation
{
    private ClientCostCenterScope mainScope;

    private ClientCostCenter mainCCC;

    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ClientCostCenterScope();
        mainScope.setRtfAssignments(true);
        mainCCC = ClientCostCenterFixture.createFixture(mainScope, this);
        assertNotNull(mainCCC.getRtfAssignments());
        assertNotNull(mainCCC.getRtfAssignments()[0]);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainCCC.getRtfAssignments()[0].setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainCCC.getRtfAssignments()[0]
                .getClass().getName(), "parent", mainCCC.getRtfAssignments()[0].getID())
        };

        verifySaveMustFail(mainCCC.getRtfAssignments()[0], null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainCCC.getRtfAssignments()[0].setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE,
                mainCCC.getRtfAssignments()[0].getClass().getName(), "parent", mainCCC
                    .getRtfAssignments()[0].getID())
        };

        verifySaveMustFail(mainCCC.getRtfAssignments()[0], null, expectedExceptions);
    }

//COMMENTED OUT BY KARIM    
//TODO see how to test failure otherwise because context usage is now removed from
//all the tests and fixtures
//    public void testFailureWithInvalidType() throws AssertionError, Exception
//    {
//        // Create RtfAssignment with wrong context
//        RtfAssignment[] wrongAssignments = new RtfAssignment[2];
//        wrongAssignments = RtfAssignmentFixture.createAllAssignmentFixtures(
//            mainCCC, this);
//        mainCCC.setRtfAssignments(wrongAssignments);
//
//        RPMExceptionMock[] expectedExceptions =
//        {
//            createRPMException(ErrorCodes.INVALID_TYPE, mainCCC.getRtfAssignments()[0]
//                .getClass().getName(), "rtf", mainCCC.getRtfAssignments()[0].getID())
//        };
//
//        verifySaveMustFail(mainCCC, mainScope, expectedExceptions);
//    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with valid max and verify that they are accepted
        mainCCC.getRtfAssignments()[0]
            .setValue(getTestString(ValidationConstants.RTFASSIGNMENT_VALUE_MAX));

        // Attempt to save the ClientCostCenter object with the RtfAssignment
        mainCCC = (ClientCostCenter) saveMustSucceed(mainCCC, ClientCostCenter.class,
            mainScope);

        // Load the inserted rtf assignment and ClientCostCenter from id
        RPMObject obj = new ClientCostCenter();
        obj.setID(mainCCC.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ClientCostCenter);

        // Check that the loaded rtf assignments are identical to the ones we
        // requested to store
        RtfAssignmentFixture.compare(mainCCC.getRtfAssignments(),
            ((ClientCostCenter) rpmObject).getRtfAssignments());
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Set field values with max+1 and verify that they are not accepted
        mainCCC.getRtfAssignments()[0]
            .setValue(getTestString(ValidationConstants.RTFASSIGNMENT_VALUE_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainCCC.getRtfAssignments()[0]
                .getClass().getName(), "value", mainCCC.getRtfAssignments()[0].getID())
        };

        verifySaveMustFail(mainCCC, mainScope, expectedExceptions);
    }
}
