package com.ibm.rpm.client.junit.asset.fixtures;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Random;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AssetCategory;
import com.ibm.rpm.applicationadministration.containers.AssetState;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.GeographicalAssignment;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.AssetOrganizationalAssignment;
import com.ibm.rpm.asset.containers.DepreciationSchedule;
import com.ibm.rpm.asset.containers.GenericAsset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RtfAssignmentFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.document.containers.DocumentFolder;

/**
 * @author rosman
 *
 */
public class AssetFixture extends RPMObjectFixture
{
    protected static ArrayList assetSkipSetters, assetSkipGetters;

    static
    {
        //Setters
        assetSkipSetters = new ArrayList();
        assetSkipSetters.addAll(skipSetters);
        //        assetSkipSetters.add("setDescription");
        assetSkipSetters.add("setCrossChargeToDate");
        assetSkipSetters.add("setSalvageValue");
        assetSkipSetters.add("setNextScheduleMaintenance");
        assetSkipSetters.add("setWBSActivityFinish");
        assetSkipSetters.add("setDepreciationToDate");
        assetSkipSetters.add("setCapitalValue");
        assetSkipSetters.add("setCurrentValue");
        assetSkipSetters.add("setEstimateMaintenance");
        assetSkipSetters.add("setMaintenanceToDate");
        assetSkipSetters.add("setEstimateCrossCharge");
        assetSkipSetters.add("setWBSActivityStart");
        assetSkipSetters.add("setAssetLife");
        assetSkipSetters.add("setDepreciationMethod");

        //Getters
        assetSkipGetters = new ArrayList();
        assetSkipGetters.addAll(skipGetters);
        //        assetSkipGetters.add("getDescription");
        assetSkipGetters.add("getCrossChargeToDate");
        assetSkipGetters.add("getSalvageValue");
        assetSkipGetters.add("getNextScheduleMaintenance");
        assetSkipGetters.add("getWBSActivityFinish");
        assetSkipGetters.add("getDepreciationToDate");
        assetSkipGetters.add("getCapitalValue");
        assetSkipGetters.add("getCurrentValue");
        assetSkipGetters.add("getEstimateMaintenance");
        assetSkipGetters.add("getMaintenanceToDate");
        assetSkipGetters.add("getEstimateCrossCharge");
        assetSkipGetters.add("getWBSActivityStart");
        assetSkipGetters.add("getAssetLife");
        assetSkipSetters.add("getDepreciationMethod");
    }

    public static Asset createSaveFixture(ClientTestController ctc)
            throws Exception
    {
        return createSaveFixture(null, ctc);
    }

    public static Asset createSaveFixture(AssetScope scope,
            ClientTestController ctc) throws Exception
    {
        AssetFolder folder = AssetFolderFixture.createSaveFixture(ctc, null);

        Asset asset = AssetFixture.createFixture(folder, scope, ctc);
        if (scope == null)
        {
            scope = new AssetScope();
        }
        scope.setParent(new AssetScope());

        return (Asset) save(ctc, asset, scope);
    }

    public static Asset createSaveFixture(AssetScope scope, int nbChildren,
            ClientTestController ctc) throws Exception
    {
        AssetFolder folder = AssetFolderFixture.createSaveFixture(ctc, null);

        Asset asset = AssetFixture.createFixture(folder, nbChildren, scope, ctc);
        if (scope == null)
        {
            scope = new AssetScope();
        }
        scope.setParent(new AssetScope());

        return (Asset) save(ctc, asset, scope);
    }

    public static Asset createFixture(AssetScope scope, ClientTestController ctc)
            throws Exception
    {
        return createFixture(AssetFolderFixture.createSaveFixture(ctc, null),
            2, scope, ctc);
    }

    public static Asset createFixture(AssetFolder parent, AssetScope scope,
            ClientTestController ctc) throws Exception
    {
        return createFixture(parent, 2, scope, ctc);
    }

    public static Asset createFixture(AssetFolder parent, int nbChildren,
            AssetScope scope, ClientTestController ctc) throws Exception
    {
        Asset asset = new Asset();
        HelperTestUtilities.setByIntrospection(asset, new Double(
            getNextPseudoRandomDouble()).intValue(), assetSkipSetters);
        asset.setCrossChargeToDate(new Double(0));// Default value, unless cross
        // charges are created
        asset.setSalvageValue(new Double(0));// Default value
        asset.setNextScheduleMaintenance(Calendar.getInstance()); // Default value
        asset.setWBSActivityStart(Calendar.getInstance());// Default value
        asset.setWBSActivityFinish(Calendar.getInstance());// Default value
        asset.setDepreciationToDate(new Double(0));// Default value
        asset.setCapitalValue(new Double(0));// Default value
        asset.setCurrentValue(new Double(0));// Default value
        asset.setEstimateMaintenance(new Double(0));// Default value
        asset.setEstimateCrossCharge(new Double(0));// Default value
        asset.setMaintenanceToDate(new Double(0));// Default value
        asset.setAssetLife(new Integer(12));// Default value - a year
        asset.setDepreciationMethod(new Integer(0));// Default value - a year
        asset.setTracked(Boolean.TRUE);

        // Todo: change it to load dynamically
        AssetCategory assetCategory = DatafieldFixture
            .createAssetsAssetCategory(DatafieldFixture.ASSET_CATEGORY_1);
        asset.setCategory(assetCategory);
        if (scope != null && scope.isAttributeAssignments())
        {
            AttributeAssignment asgn = AttributeAssignmentFixture
                .createAssignmentFixture(asset, ctc);
            asset.setAttributeAssignments(new AttributeAssignment[]
            {
                asgn
            });
        }
        if (scope != null && scope.isRtfAssignments())
        {
            RtfAssignment[] rtfAssignments = RtfAssignmentFixture
                .createAllAssignmentFixtures(asset, ctc);
            asset.setRtfAssignments(rtfAssignments);
        }
        if (scope != null && scope.isGeographicalAssignment())
        {
            GeographicalAssignment geoAssignments = AssetGeographicalAssignmentFixture
                .createFixture(asset, ctc);
            asset.setGeographicalAssignment(geoAssignments);
        }
        if (scope != null && scope.getDocumentFolder() != null)
        {
            DocumentFolder fld = (DocumentFolder) DocumentElementFixture
                .createFixture(asset, new DocumentFolder(), nbChildren, scope
                    .getDocumentFolder(), ctc);
            asset.setDocumentFolder(fld);
        }
        if (parent != null)
        {
            asset.setParent(parent);
        }
        else
        {
            Assert.fail("Asset must have a parent folder");
        }
        if (scope != null)
        {
            if (scope.isAttributeAssignments())
            {
                AttributeAssignment asgn = AttributeAssignmentFixture
                    .createAssignmentFixture(asset, ctc);
                asset.setAttributeAssignments(new AttributeAssignment[]
                {
                    asgn
                });
            }
            if (scope.isRtfAssignments())
            {
                RtfAssignment[] rtfAssignments = RtfAssignmentFixture
                    .createAllAssignmentFixtures(asset, ctc);
                asset.setRtfAssignments(rtfAssignments);
            }
            if (scope != null && scope.getDocumentFolder() != null)
            {
                DocumentFolder fld = (DocumentFolder) DocumentElementFixture
                    .createFixture(asset, new DocumentFolder(), nbChildren,
                        scope.getDocumentFolder(), ctc);
                asset.setDocumentFolder(fld);
            }
            if ((scope.getAssets() != null))
            {
                AssetScope aScope = scope.getAssets();
                aScope.setParent(null);

                Asset[] aArray = new Asset[nbChildren];
                for (int i = 0; i < nbChildren; i++)
                {
                    Asset a = AssetFixture.createFixture(aScope,ctc);
                    aArray[i] = a;
                }

                asset.setAssets(aArray);
            }
            if (scope.getDepreciationSchedules() != null)
            {
                int numDeps = getNextInt() % 10 + 1;
                DepreciationSchedule[] deps = new DepreciationSchedule[numDeps];
                double depreciationAmount = asset.getCapitalValue()
                    .doubleValue()
                        / numDeps;
                for (int i = 0; i < numDeps; ++i)
                {
                    deps[i] = new DepreciationSchedule();
                    (deps[i]).setDepreciationDate(new GregorianCalendar(2006,
                        i, i + 5));
                    (deps[i]).setAmount(new Double(depreciationAmount));
                    (deps[i]).setParent(asset);

                }
                asset.setDepreciationSchedules(deps);

            }

            if (scope.isAssetOrganizationalAssignment())
            {
                AssetOrganizationalAssignment assetOrganizationalAssignment = new AssetOrganizationalAssignment();
                //create and save new organization
                Organizational organizational = OrganizationalFixture
                    .createSaveFixture(1, ctc);

                //create new Organizational Assignment
                assetOrganizationalAssignment = AssetOrganizationalAssignmentFixture
                    .createFixture(asset, organizational);

                asset
                    .setAssetOrganizationalAssignment(assetOrganizationalAssignment);

            }
        }

        AssetState s = DatafieldFixture
            .createAssetState(DatafieldFixture.ASSET_STATE_OPEN);
        asset.setState(s);
        return asset;
    }

    public static Asset updateFixture(Asset asset, int nbChildren,
            ClientTestController ctc) throws Exception
    {
        Asset updAsset = createFixture(null, ctc); // TODO: check it
        updAsset.setID(asset.getID());
        if (asset.getParent() != null)
        {
            updAsset.setParent(asset.getParent());
        }
        if (asset.getAssets() != null)
        {

            GenericAsset[] assets = asset.getAssets();
            GenericAsset[] rArray = new Asset[nbChildren * 2];
            for (int i = 0; i < nbChildren; i++)
            {
                //delete existing resource
                assets[i].setDeleted(Boolean.TRUE);
                rArray[i] = assets[i];

                //Add new resources
                AssetFolder parentFolder = AssetFolderFixture
                    .createSaveFixture(ctc, null);
                Asset newAsset = AssetFixture.createFixture(parentFolder, null,
                    ctc);
                rArray[nbChildren + i] = newAsset;
            }

            updAsset.setAssets(rArray);
        }

        return updAsset;
    }

    public static void compare(Asset a1, Asset a2, AssetScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(a1, a2, assetSkipGetters);

        compareWithScope(a1, a2, scope);
    }

    public static void compare(Asset[] as1, Asset[] as2, AssetScope scope)
            throws AssertionFailedError, Exception
    {

        if (as1.length != as2.length)
        {
            throw new AssertionFailedError();
        }
        sortWithComparator(as1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(as2, RPMObjectNameAndIDComparator.INSTANCE);

        for (int i = 0; i < as1.length; ++i)
        {
            RPMObjectFixture.compareWithSkip(as1[i], as2[i], assetSkipGetters);
            compareWithScope(as1[i], as2[i], scope);
        }
    }

    private static void compareWithScope(Asset a1, Asset a2, AssetScope scope)
            throws AssertionFailedError, Exception
    {

        if (scope == null)
            return;

        //        Parent
        if (scope.getParent() != null)
        {
            if (a2.getParent() == null)
                Assert.fail("RPMObject in scope is null");

            if (a2.getParent() instanceof Asset)
            {
                AssetFixture.compare((Asset) a1.getParent(), (Asset) a2
                    .getParent(), (AssetScope)scope.getParent());
            }
            else if (a2.getParent() instanceof AssetFolder)
            {
                AssetFolderFixture.compare((AssetFolder) a1.getParent(),
                    (AssetFolder) a2.getParent(), (AssetScope)scope.getParent());
            }
        }
        if (scope.getAssets() != null)
        {
            AssetScope chldScope = scope.getAssets();
            GenericAsset[] a1Assets = a1.getAssets();
            GenericAsset[] a2Assets = a2.getAssets();
            if (a1Assets != null && a2Assets != null)
            {
                Assert.assertTrue(a1Assets.length == a2Assets.length);
                for (int i = 0; i < a1Assets.length; ++i)
                {
                    Asset a1a = (Asset) a1Assets[i];
                    Asset a2a = (Asset) a2Assets[i];
                    compare(a1a, a2a, chldScope);
                }
            }
        }
        if (scope.isGeographicalAssignment())
            AssetGeographicalAssignmentFixture.compare(a1
                .getGeographicalAssignment(), a2.getGeographicalAssignment());
        if (scope.isAssetOrganizationalAssignment())
            AssetOrganizationalAssignmentFixture.compare(a1
                .getAssetOrganizationalAssignment(), a2
                .getAssetOrganizationalAssignment());
        if (scope.getDocumentFolder() != null)
            DocumentElementFixture.compare(a1.getDocumentFolder(), a2
                .getDocumentFolder(), scope.getDocumentFolder());
        if (scope.isAssetResourceAssignments())
            AssetResourcelAssignmentFixture.compare(a1
                .getAssetResourceAssignments(), a2
                .getAssetResourceAssignments());
        if (scope.isAttributeAssignments())
            AttributeAssignmentFixture.compare(a1.getAttributeAssignments(), a2
                .getAttributeAssignments());
        // Cross-Charge-Financial missing
        if (scope.getDepreciationSchedules() != null)
            DepreciationScheduleFixture.compare(a1.getDepreciationSchedules(),
                a2.getDepreciationSchedules());
        if (scope.getMaintenanceEstimates() != null)
            MaintenanceEstimatesFixture.compare(a1.getMaintenanceEstimates(),
                a2.getMaintenanceEstimates());
        if (scope.isRtfAssignments())
            RtfAssignmentFixture.compare(a1.getRtfAssignments(), a2
                .getRtfAssignments());
        if (scope.getAssignedScorecard() != null)
        {
            PublishedScorecardFixture.compare(a1.getAssignedScorecard(), a2
                .getAssignedScorecard(), scope.getAssignedScorecard());
        }

        //compare documentElements
        if (scope.getDocumentFolder() != null)
        {
            DocumentFolder documentFolder1 = a1.getDocumentFolder();
            DocumentFolder documentFolder2 = a2.getDocumentFolder();
            DocumentElementFixture.compare(documentFolder1, documentFolder2,
                scope.getDocumentFolder());
        }
    }

    public static AssetScope createScope(int currentLevel, int maxLevels,
            int nbAssetsByLevel, AssetScope parentScope)
    {
        AssetScope scope = new AssetScope();
        scope.setParent(parentScope);
        if (currentLevel <= maxLevels)
        {
            if ((nbAssetsByLevel > 0) && (currentLevel < maxLevels))
            {
                //create client cost center scope
                AssetScope childScope = AssetFixture.createScope(
                    currentLevel + 1, maxLevels, nbAssetsByLevel, scope);
                scope.setAssets(childScope);
            }
        }
        return scope;
    }

    private static int INDEX = 1;

    private static int getNextInt()
    {
        return INDEX++;
    }

    private static double getNextPseudoRandomDouble()
    {
        int MAX_VALUE = 10000;
        int newInt = getNextInt();
        Random rnd = (new Random());
        rnd.setSeed(newInt);
        Integer hashVal = new Integer(rnd.nextInt());
        int modded = hashVal.intValue() % MAX_VALUE;
        BigDecimal moddedAbs = new BigDecimal(modded);
        moddedAbs = moddedAbs.abs();
        moddedAbs = moddedAbs.setScale(2);
        BigDecimal divided = (moddedAbs).divide(
            (new BigDecimal(MAX_VALUE / 100)), BigDecimal.ROUND_HALF_UP);
        return divided.doubleValue();

    }
}