/*
 * Created on Oct 26, 2005
 *
 */
package com.ibm.rpm.client.junit.asset.fixtures;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.scope.AssetModuleScope;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.fixtures.AbstractModuleFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.scope.ScorecardFolderScope;

/**
 * @author karim
 *  
 */
public class AssetModuleFixture extends AbstractModuleFixture
{

    public static AssetModule createFixture(AssetModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        return createModule(scope, ctc);
    }

    public static AssetModule updateFixture(AssetModule module,
            AssetModuleScope scope, ClientTestController ctc) throws Exception
    {
        return updateModule(module, scope, ctc);
    }

    private static AssetModule createModule(AssetModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        AssetModule module = new AssetModule();

        if (scope != null)
        {
            if (scope.getAssetFolders() != null)
            {
	    	    AssetFolder[] assetFolders = (AssetFolder[]) createElements(
                    scope.getAssetFolders(), AssetFolder.class, ctc);
	    	    module.setAssetFolders(assetFolders);
            }

            if (scope.getAttributeCategories() != null)
            {
                AttributeCategory[] attributeCategories = (AttributeCategory[]) createElements(
                    scope.getAttributeCategories(), AttributeCategory.class,
                    ctc);
                module.setAttributeCategories(attributeCategories);
            }
            
            if (scope.getCustomFieldCategories() != null)
            {
                CustomFieldCategory[] customFieldCategories = (CustomFieldCategory[]) createElements(
                        scope.getCustomFieldCategories(),
                        CustomFieldCategory.class, ctc);
                module.setCustomFieldCategories(customFieldCategories);
            }
            
            if (scope.getScorecardFolders() != null)
            {
		    	ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) createElements(
                    scope.getScorecardFolders(), ScorecardFolder.class, ctc);
		    	module.setScorecardFolders(scorecardFolders);
            }
        }

        return module;
    }

    private static AssetModule updateModule(AssetModule module,
            AssetModuleScope scope, ClientTestController ctc) throws Exception
    {
        if (scope != null)
        {
            if (scope.getAssetFolders() != null)
            {
                //AssetFolders
    		    AssetFolder[] assetFolders = (AssetFolder[]) updateElements(
    		            module.getAssetFolders(), scope.getAssetFolders(), ctc);
    		    module.setAssetFolders(assetFolders);
            }

            if (scope.getAttributeCategories() != null)
            {
    		    //AttributeCategories
                AttributeCategory[] attributeCategories = (AttributeCategory[]) updateElements(
                        module.getAttributeCategories(), scope
                                .getAttributeCategories(), ctc);
                module.setAttributeCategories(attributeCategories);
            }
            
            if (scope.getCustomFieldCategories() != null)
            {
    	        //CustomFields
                module.setCustomFieldCategories(CustomFieldFixture
                        .modifyCustomFieldCategories(module
                                .getCustomFieldCategories(), scope
                                .getCustomFieldCategories()));
            }
            
            if (scope.getScorecardFolders() != null)
            {
                //ScorecardFolders
    		    ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) updateElements(
    		            module.getScorecardFolders(), scope.getScorecardFolders(), ctc);
    		    module.setScorecardFolders(scorecardFolders);
            }

        }
        return module;
    }

    public static AssetModuleScope createFullScope()
    {
        return createScope(true,true,true,true,true,true,true,true);
    }
    
    public static AssetModuleScope createScope(boolean isAssetFolders, boolean isAttributeCategories,
    		boolean isChargeTypes, boolean isAssetCategories,
			boolean isAssetStates, boolean isAssetRtfs,
			boolean isCustomFields, boolean isScorecardFolders
    		)
    {
        AssetModuleScope scope = new AssetModuleScope();

        if(isAssetFolders){
            scope.setAssetFolders(new AssetScope());
        }
        
        if (isAttributeCategories)
        {
            scope.setAttributeCategories(new AttributeScope());
        }

	    if(isChargeTypes){
	        scope.setChargeTypes(true);
	    }
	    
	    if(isAssetCategories){
	        scope.setAssetCategories(true);
	    }
	    
	    if(isAssetStates){
	        scope.setAssetStates(true);
	    }
	    
	    if(isAssetRtfs){
	        scope.setAssetRtfs(true);
	    }
	    
        if (isCustomFields)
        {
            scope.setCustomFieldCategories(new CustomFieldScope());
        }
	    
		if(isScorecardFolders){
		    ScorecardFolderScope sfScope = ScorecardFixture.createScope();		    
		    scope.setScorecardFolders(sfScope);
		}

        return scope;
    }

    public static void compare(AssetModule obj1, AssetModule obj2,
            AssetModuleScope scope) throws AssertionFailedError, Exception
    {

    	if(scope.getAssetFolders()!=null){
            AssetFolderFixture.compare(obj1.getAssetFolders(), obj2
                    .getAssetFolders(), scope.getAssetFolders());    	    
    	}

    	if(scope.getAttributeCategories()!=null){
            AttributeFixture.compare(obj1.getAttributeCategories(), obj2
                    .getAttributeCategories(), scope.getAttributeCategories());    	    
    	}

    	if(scope.isChargeTypes()){
            DatafieldFixture.compare(obj1.getChargeTypes(), obj2.getChargeTypes());    	    
    	}

    	if(scope.isAssetCategories()){
    		DatafieldFixture.compare(obj1.getAssetCategories(), obj2.getAssetCategories());    	    
    	}

    	if(scope.isAssetStates()){
    		DatafieldFixture.compare(obj1.getAssetStates(), obj2.getAssetStates());    	    
    	}

    	if(scope.isAssetRtfs()){
    		DatafieldFixture.compare(obj1.getAssetRtfs(), obj2.getAssetRtfs());    	    
    	}

    	if(scope.getCustomFieldCategories()!=null){
            CustomFieldFixture.assertIdEqual(obj1.getCustomFieldCategories(), obj2
                    .getCustomFieldCategories(), scope.getCustomFieldCategories());    	    
    	}

    	if(scope.getScorecardFolders()!=null){
    		ScorecardFixture.compare(obj1.getScorecardFolders(), obj2
    		        .getScorecardFolders(), scope.getScorecardFolders());    	    
    	}
    }
}
