package com.ibm.rpm.client.junit.asset.testcases;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.CrossChargeFinancials;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.asset.scope.CrossChargeFinancialsScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetCrossChargeFinancialsFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.clientcostcenters.containers.AbstractCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author adler
 *
 * A test case for Asset Cross Charge Financials. It tests the loading, updating and
 * deleting with and without a scope
 */
public class TestAssetCrossChargeFinancials extends ClientTestController
{
	private Asset mainAsset;

    private CrossChargeFinancialsScope mainCrossChargeFinancialsScope;

    private CrossChargeFinancials[] crossChargeFinancials;

    private AbstractCostCenter[] projectOrClient;

    // TODO add more charged objects
    public static final int NUMBER_RECORDS = 1;

    public static final int CLIENT_I = 0;
    // Example index for Proposal (not checked)
    public static final int PROPOSAL_I = 1;

    public void setUp() throws Exception
    {
        super.setUp();

        //create a new asset in the database
        mainAsset = AssetFixture.createSaveFixture(null, this);

        crossChargeFinancials = new CrossChargeFinancials[NUMBER_RECORDS];
        projectOrClient = new AbstractCostCenter[NUMBER_RECORDS];

        save(mainAsset, null);

        //create and save a new ClientCostCenter
        projectOrClient[CLIENT_I] = ClientCostCenterFixture.createSaveFixture(this);

        //TODO save ProjectCostCenter

        //create new CrossChargeFinancials
        crossChargeFinancials = AssetCrossChargeFinancialsFixture.createFixture(mainAsset,
        		projectOrClient, NUMBER_RECORDS);

        //assign the CrossChargeFinancials for the asset
        mainAsset.setCrossChargeFinancials(crossChargeFinancials);

        mainCrossChargeFinancialsScope = new CrossChargeFinancialsScope();
        // Set the scope for parent Asset
        mainCrossChargeFinancialsScope.setParent(new AssetScope());

        // Set the scope for Clients
        mainCrossChargeFinancialsScope.setCostCenter(new ClientCostCenterScope());
        // TODO Set the scope for ProjectCostCenterScope

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    /**
     * Test loading from XPath Loading and comparing a known ID
     *  /CrossChargeFinancials[<condition>]
     *  No Scope
     */
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        // Save Cross Charge Financials objects
    	saveCrossChargeFinancials(crossChargeFinancials);

        RPMObject[] loadedCrossChargeFinancials;

        //load CrossChargeFinancials object via xpath (id)
        loadedCrossChargeFinancials = loadFromXpath("/CrossChargeFinancials[id='"
                + crossChargeFinancials[0].getID() + "']", null);
        assertNotNull(loadedCrossChargeFinancials);
        assertTrue(loadedCrossChargeFinancials[0] instanceof CrossChargeFinancials);
        assertEquals(1, loadedCrossChargeFinancials.length);
        AssetCrossChargeFinancialsFixture.compare(crossChargeFinancials[0],
                (CrossChargeFinancials) loadedCrossChargeFinancials[0], null);

    }// testInsertWithNoScopeLoadFromXPath}

    /**
     * Test loading from XPath Using the asset with Scope
     * /Asset[<condition>]/CrossChargeFinancials[<condition>]
     */
    public void testLoadFromXPathIncludeAsset() throws AssertionError,
            Exception
    {
        // Save Cross Charge Financials objects
    	saveCrossChargeFinancials(crossChargeFinancials);

        RPMObject[] loadedCrossChargeFinancials;

        // test loading CrossChargeFinancials using asset and CrossChargeFinancials ids
        loadedCrossChargeFinancials = loadFromXpath("/Asset[Id='" + mainAsset.getID()
                + "']/CrossChargeFinancials[Id='" + crossChargeFinancials[0].getID()
                + "']", mainCrossChargeFinancialsScope);
        assertNotNull(loadedCrossChargeFinancials);
        assertEquals(1, loadedCrossChargeFinancials.length);
        assertTrue(loadedCrossChargeFinancials[0] instanceof CrossChargeFinancials);
        AssetCrossChargeFinancialsFixture.compare(this.crossChargeFinancials[0],
                (CrossChargeFinancials) loadedCrossChargeFinancials[0], mainCrossChargeFinancialsScope);

        // test loading all CrossChargeFinancials related to an Asset id
        loadedCrossChargeFinancials = loadFromXpath("/Asset[Id='" + mainAsset.getID()
                + "']/CrossChargeFinancials", mainCrossChargeFinancialsScope);
        assertNotNull(loadedCrossChargeFinancials);
        assertEquals(NUMBER_RECORDS, loadedCrossChargeFinancials.length);

        CrossChargeFinancials[] newCrossChargeFinancials = (CrossChargeFinancials[])(Arrays.asList(loadedCrossChargeFinancials).toArray(new CrossChargeFinancials[0]));

        AssetCrossChargeFinancialsFixture.compare(crossChargeFinancials,
        		newCrossChargeFinancials);

    } // testLoadFromXPathIncludeAsset

    /**
     * Test loading ChargedElement from XPath Using the crossChargeFinancials, Scope
     * /CrossChargeFinancials[<condition>]/ClientCostCenter[<condition>]
     */
    public void testLoadChargedElementFromXPathIncludeCrossChargeFinancials() throws AssertionError,
            Exception
    {
        // Save Cross Charge Financials objects
    	saveCrossChargeFinancials(crossChargeFinancials);

    	// TODO /CrossChargeFinancials[<condition>]/ProjectCostCenter[<condition>]

        RPMObject[] loadedClientCostCenter;

        // test loading ClientCostCenter using crossChargeFinancials and ClientCostCenter ID
        loadedClientCostCenter = loadFromXpath("/CrossChargeFinancials[Id='" + crossChargeFinancials[CLIENT_I].getID()
                + "']/ClientCostCenter[Id='" + ((ClientCostCenter)projectOrClient[CLIENT_I]).getID()
                + "']", new ClientCostCenterScope());
        assertNotNull(loadedClientCostCenter);
        assertEquals(1, loadedClientCostCenter.length);
        assertTrue(loadedClientCostCenter[0] instanceof ClientCostCenter);
        ClientCostCenterFixture.compare( (ClientCostCenter)this.projectOrClient[CLIENT_I],
                (ClientCostCenter) loadedClientCostCenter[0], new ClientCostCenterScope());

    } // testLoadChargedElementFromXPathIncludeCrossChargeFinancials

    /**
     * Test loading CrossChargeFinancials from XPath Using CrossChargeFinancials, nested Charged Object
     * and with Scope
     * /CrossChargeFinancials[ClientCostCenter[<condition>]]
     * /CrossChargeFinancials[Proposal[<condition>]]
     */
    public void testLoadCrossChargeFinancialsFromXPathNestedChargedObjects() throws AssertionError,
    Exception
	{
        // Save Cross Charge Financials objects
    	saveCrossChargeFinancials(crossChargeFinancials);

        RPMObject[] loadedCrossChargeFinancials;

		// test loading crossChargeFinancials using nested ClientCostCenter using id
        loadedCrossChargeFinancials = loadFromXpath("/CrossChargeFinancials[ClientCostCenter[Id='" + projectOrClient[CLIENT_I].getID() +"']]",mainCrossChargeFinancialsScope);
		assertNotNull(loadedCrossChargeFinancials);
        assertEquals(1, loadedCrossChargeFinancials.length);

		// TODO /CrossChargeFinancials[ProjectCostCenter[<condition>]]

	} // testLoadCrossChargeFinancialsFromXPathNestedChargedObjects

    /**
     * Inserts CrossChargeFinancials, then deletes them and tests that they can be deleted.
     *  Uses no scope
     */
    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        // Save Cross Charge Financials objects
    	saveCrossChargeFinancials(crossChargeFinancials);

        //load from DB via id
        List loadedCrossChargeFinancials = new LinkedList();
        for (int i = 0; i < this.crossChargeFinancials.length; i++)
        {
        	CrossChargeFinancials loadedCrossChargeFinancial = loadCrossChargeFinancial(
        			crossChargeFinancials[i], null);
        	AssetCrossChargeFinancialsFixture.compare(this.crossChargeFinancials[i],
            		loadedCrossChargeFinancial, null);
            loadedCrossChargeFinancials.add(loadedCrossChargeFinancial);
        }

        // Check the delete
        //can't delete because no Asset and charged object loaded

        //for (int i = 0; i < loadedCrossChargeFinancials.size(); i++)
        //{
            //deleteCrossChargeFinancial((CrossChargeFinancials)loadedCrossChargeFinancials.get(i),
                    //null);
        //}

    } // testInsertWithNoScopeLoadFromID

    /**
     * Inserts CrossChargeFinancials, then deletes them and tests that they can be deleted.
     *  Uses scope
     */
    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        // Save Cross Charge Financials objects
    	saveCrossChargeFinancials(crossChargeFinancials);

        //load from DB via id
        List loadedCrossChargeFinancials = new LinkedList();
        for (int i = 0; i < this.crossChargeFinancials.length; i++)
        {
        	CrossChargeFinancials loadedCrossChargeFinancial = loadCrossChargeFinancial(
        			crossChargeFinancials[i], mainCrossChargeFinancialsScope);
        	AssetCrossChargeFinancialsFixture.compare(this.crossChargeFinancials[i],
            		loadedCrossChargeFinancial, mainCrossChargeFinancialsScope);
            loadedCrossChargeFinancials.add(loadedCrossChargeFinancial);
        }

        // Check the delete
        for (int i = 0; i < loadedCrossChargeFinancials.size(); i++)
        {
        	deleteCrossChargeFinancial((CrossChargeFinancials)loadedCrossChargeFinancials.get(i),
            		mainCrossChargeFinancialsScope);
        }

    } // testInsertWithScopeLoadFromID

    /**
     * Inserts a few CrossChargeFinancials, then update them and test the update
     * Uses no scope
     */
    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        // Save Cross Charge Financials objects
    	saveCrossChargeFinancials(crossChargeFinancials);

        //create new CrossChargeFinancials
    	CrossChargeFinancials[] updatedCrossChargeFinancials = AssetCrossChargeFinancialsFixture.updateFixture(mainAsset,
        		projectOrClient, NUMBER_RECORDS);


        for (int i = 0; i < this.crossChargeFinancials.length; i++)
        {

            //By setting the ID to the crossChargeFinancials ID, the updatedCrossChargeFinancials represent
            //the maincrossChargeFinancials with all its attribute updated.
        	updatedCrossChargeFinancials[i].setID(crossChargeFinancials[i].getID());

        	updatedCrossChargeFinancials[i].setCost(new Double(1000.50));
        	updatedCrossChargeFinancials[i] = saveCrossChargeFinancial(updatedCrossChargeFinancials[i]);
        	if(updatedCrossChargeFinancials[i]!=null)
        	{
        	    CrossChargeFinancials loadedCrossChargeFinancials = loadCrossChargeFinancial(updatedCrossChargeFinancials[i],null);


        	    AssetCrossChargeFinancialsFixture.compare(updatedCrossChargeFinancials[i], loadedCrossChargeFinancials,null);
        	}

        }

    } // testUpdateWithNoScopeLoadFromID

    /**
     * Inserts a few CrossChargeFinancials, then update them and test the update
     * Uses scope
     */
    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        // Save Cross Charge Financials objects
    	saveCrossChargeFinancials(crossChargeFinancials);

        //create new CrossChargeFinancials
    	CrossChargeFinancials[] updatedCrossChargeFinancials = AssetCrossChargeFinancialsFixture.updateFixture(mainAsset,
        		projectOrClient, NUMBER_RECORDS);
        for (int i = 0; i < this.crossChargeFinancials.length; i++)
        {
        	//By setting the ID to the crossChargeFinancials ID, the updatedCrossChargeFinancials represent
            //the maincrossChargeFinancials with all its attribute updated.
        	updatedCrossChargeFinancials[i].setID(crossChargeFinancials[i].getID());

        	updatedCrossChargeFinancials[i].setCost(new Double(1000.50));
        	updatedCrossChargeFinancials[i] = saveCrossChargeFinancial(updatedCrossChargeFinancials[i]);

        	if(updatedCrossChargeFinancials[i]!=null)
        	{
	        	CrossChargeFinancials loadedCrossChargeFinancials = loadCrossChargeFinancial(updatedCrossChargeFinancials[i],mainCrossChargeFinancialsScope);
	        	AssetCrossChargeFinancialsFixture.compare(updatedCrossChargeFinancials[i], loadedCrossChargeFinancials,mainCrossChargeFinancialsScope);
        	}
        }

    } // testUpdateWithScopeLoadFromID

// TODO: enable when ProjectCostCenter is ready
//    public void testDummyReminderForProjectCostCenter() throws AssertionError,
//            Exception
//    {
//    	fail("Reminder: ProjectCostCenter is not ready yet, please follow the TODO comments both in TestAssetCrossChargeFinancials and in AssetCrossChargeFinancialsFixture for implementation");
//    } 
    /**
     * saveCrossChargeFinancials - Save CrossChargeFinancials objects (without scope)
     *
     * @param crossChargeFinancials []
     * @return None
     * @throws Exception
     * @throws AssertionFailedError
     *
     */
    private void saveCrossChargeFinancials(CrossChargeFinancials[] crossChargeFinancials)
    throws Exception
	{
    	for (int i = 0; i < crossChargeFinancials.length; i++)
    	{
    	    crossChargeFinancials[i] = saveCrossChargeFinancial(crossChargeFinancials[i]);
    	}
	} // saveCrossChargeFinancials

    /**
     * saveCrossChargeFinancial - Save one CrossChargeFinancial object (without scope)
     *
     * @param CrossChargeFinancial
     *            that we are working on now
     * @return None
     * @throws Exception
     * @throws AssertionFailedError
     *
     */
    private CrossChargeFinancials saveCrossChargeFinancial(CrossChargeFinancials p) throws AssertionError,
            Exception
    {
        boolean deleted = p.getDeleted().booleanValue();

        SingleResult result;
        // Save the Object
        result = save(p, mainCrossChargeFinancialsScope);

        if(deleted)
        {
            assertNotNull(result);
            assertNull(result.getRpmObject());
            return null;
        }
        else
        {
            assertNotNull(result);
            assertNotNull(result.getRpmObject());
            assertTrue(result.getRpmObject() instanceof CrossChargeFinancials);

            return (CrossChargeFinancials)result.getRpmObject();
        }
    } // saveCrossChargeFinancial

    /**
     * loadCrossChargeFinancial - Load an CrossChargeFinancials object with scope
     *
     * @param CrossChargeFinancials
     *            the object to load
     * @param CrossChargeFinancialsScope
     *            the scope
     * @return Loaded CrossChargeFinancials
     * @throws Exception
     * @throws AssertionFailedError
     */
    private CrossChargeFinancials loadCrossChargeFinancial(CrossChargeFinancials crossChargeFinancials,
    		CrossChargeFinancialsScope scope) throws Exception
    {
        //load from an id
        RPMObject obj = new CrossChargeFinancials();
        obj.setID(crossChargeFinancials.getID());
        RPMObject rpmObject = loadFromID(obj, scope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof CrossChargeFinancials);

        return (CrossChargeFinancials) rpmObject;
    } // loadCrossChargeFinancial

    /**
     * deleteCrossChargeFinancial - Delete an CrossChargeFinancials object with scope
     * @param CrossChargeFinancials the object to delete
     * @param CrossChargeFinancialsScope the scope
     * @return None
     * @throws Exception
     *
     */
    private void deleteCrossChargeFinancial(CrossChargeFinancials crossChargeFinancials,
    		CrossChargeFinancialsScope scope) throws Exception
    {
        delete(crossChargeFinancials);

        RPMObject rpmObject = null;
        try
        {
            rpmObject = loadFromID(crossChargeFinancials, scope);
        }
        catch (Exception e)
        {
            logger
                    .debug("This is a \"Good\" exception (verifies that the object were deleted)");
        }
        assertNull(rpmObject);
    } // deleteCrossChargeFinancial
}


