package com.ibm.rpm.client.junit.asset.testcases;

import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author rosman
 *
 */
public class TestAssetFolder extends ClientTestController
{
    protected AssetFolder mainAssetFolder;

    protected AssetFolder parentAssetFolder;

    protected AssetScope mainScope;

    protected AssetScope parentScope;

    private static final int NB_MAX_LEVELS = 3;

    private static final int NB_FOLDERS_BY_LEVEL = 1;

    private static final int NB_ASSETS_BY_LEVEL = 1;

    private AssetFolder parent = null;

    protected void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        parentScope = AssetFolderFixture.createScope(1, NB_MAX_LEVELS,
                NB_FOLDERS_BY_LEVEL, 0, null);

        mainScope = AssetFolderFixture.createScope(1, NB_MAX_LEVELS,
                NB_FOLDERS_BY_LEVEL, NB_ASSETS_BY_LEVEL, parentScope);
        parentAssetFolder = AssetFolderFixture.createSaveFixture(this,
                parentScope);

    }

    public void tearDown() throws Exception
    {

        if (parent != null)
        {
            try
            {
                delete(parent);
            }
            catch (Exception e)
            {
                //do nothing
            }
        }

        super.tearDown();

    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.client.junit.interfaces.RPMUnitTestI#testInsertWithNoScopeLoadFromXPath()
     */
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        mainAssetFolder = AssetFolderFixture
                .createFixture(false, 0, null, this);
        AssetFolder saved = saveAssetFolder(mainAssetFolder, mainScope);
        mainAssetFolder.setID(saved.getID());// In order to get the ID, which is
        // set only after we've saved the
        // folder
        RPMObject[] rpmObjects = loadFromXpath("/AssetFolder[id='"
                + mainAssetFolder.getID() + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssetFolder);
        AssetFolderFixture.compare(mainAssetFolder,
                (AssetFolder) rpmObjects[0], null);
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.client.junit.interfaces.RPMUnitTestI#testInsertWithNoScopeLoadFromXPath()
     */
    public void testInsertWithNoScopeLoadFromXPathLevel2()
            throws AssertionError, Exception
    {
        initWithFullScope();

        AssetFolder savedParent = saveAssetFolder(parentAssetFolder,
                parentScope);
        AssetFolder[] savedFolders = savedParent.getFolders();
        AssetFolder saved = savedFolders[0];
        mainAssetFolder.setID(saved.getID());
        AssetFolder[] children = saved.getFolders();
        RPMObject[] loaded = loadArrayFromID(new RPMObject[]
        {
            mainAssetFolder
        }, mainScope);
        assertNotNull(loaded);
        RPMObject[] rpmObjects = loadFromXpath("/AssetFolder[name='" + saved.getName()
            + "']/@folders[name='" + children[0].getName() + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssetFolder);
        AssetFolderFixture.compare(children[0], (AssetFolder) rpmObjects[0],
                null);
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.client.junit.interfaces.RPMUnitTestI#testInsertWithNoScopeLoadFromXPath()
     */
    public void testInsertWithNoScopeLoadFromXPathLevel3()
            throws AssertionError, Exception
    {
        initWithFullScope();

        AssetFolder savedParent = saveAssetFolder(parentAssetFolder,
                parentScope);
        AssetFolder[] savedFolders = savedParent.getFolders();
        AssetFolder saved = savedFolders[0];
        mainAssetFolder.setID(saved.getID());
        AssetFolder[] children = saved.getFolders();
        RPMObject[] loaded = loadArrayFromID(new RPMObject[]
        {
            mainAssetFolder
        }, mainScope);
        assertNotNull(loaded);
        assertTrue(children != null);
        assertTrue(children.length > 0);
        AssetFolder[] grandChildren = children[0].getFolders();
        assertTrue(grandChildren != null);
        assertTrue(grandChildren.length > 0);

        RPMObject[] rpmObjects = loadFromXpath("/AssetFolder[name='" + saved.getName()
            + "']/@folders[name='" + children[0].getName() 
            + "']/@folders[name='" + grandChildren[0].getName() + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssetFolder);
        assertFalse(grandChildren[0].getID().equals(children[0].getID()));
        assertFalse(mainAssetFolder.getID().equals(children[0].getID()));
        assertFalse(grandChildren[0].getID().equals(mainAssetFolder.getID()));
        AssetFolderFixture.compare(grandChildren[0],
                (AssetFolder) rpmObjects[0], null);
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.client.junit.interfaces.RPMUnitTestI#testInsertWithNoScopeLoadFromXPath()
     */
    public void testInsertWithNoScopeLoadFromXPathSubTable()
            throws AssertionError, Exception
    {
        parentScope = new AssetScope();
        parentScope.setFolders(new AssetScope());
        parentAssetFolder = AssetFolderFixture.createFixture(false, 1,
                parentScope, this);
        AssetFolder[] folders = parentAssetFolder.getFolders();
        mainAssetFolder = folders[0];
        AssetFolder savedParent = saveAssetFolder(parentAssetFolder,
                parentScope);
        AssetFolder[] savedFolders = savedParent.getFolders();
        AssetFolder saved = savedFolders[0];
        mainAssetFolder.setID(saved.getID());
        RPMObject[] loaded = loadArrayFromID(new RPMObject[]
        {
            mainAssetFolder
        }, mainScope);
        assertNotNull(loaded);
        RPMObject[] rpmObjects = loadFromXpath("/AssetFolder[id='"
                + savedParent.getID() + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssetFolder);
        AssetFolderFixture.compare(parentAssetFolder,
            (AssetFolder) rpmObjects[0], null);
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.client.junit.interfaces.RPMUnitTestI#testInsertWithNoScopeLoadFromID()
     */
    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainAssetFolder = AssetFolderFixture.createFixture(false,
                NB_ASSETS_BY_LEVEL, null, this);
        AssetFolder saved = saveAssetFolder(mainAssetFolder, mainScope);
        mainAssetFolder.setID(saved.getID());
        loadFolderFromId(mainAssetFolder, null);
    }
    
    public void testDefect56838_ManualCheckoutOfAssetFolder()
            throws AssertionError, Exception
    {
        mainAssetFolder = AssetFolderFixture.createFixture(false,
            NB_ASSETS_BY_LEVEL, null, this);
        AssetFolder saved = saveAssetFolder(mainAssetFolder, mainScope);
        this.disableAutomaticCheckout();
        checkOut(saved);
        assertFalse(isCheckedOut(saved));
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.client.junit.interfaces.RPMUnitTestI#testInsertWithScopeLoadFromID()
     */
    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.client.junit.interfaces.RPMUnitTestI#testUpdateWithNoScopeLoadFromID()
     */
    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainAssetFolder = AssetFolderFixture.createFixture(false,
                NB_ASSETS_BY_LEVEL, null, this);
        AssetFolder saved = saveAssetFolder(mainAssetFolder, mainScope);
        mainAssetFolder.setID(saved.getID());
        loadFolderFromId(mainAssetFolder, null);
        AssetFolder updatedFolder = AssetFolderFixture.createFixture(false,
                NB_ASSETS_BY_LEVEL, null, this);
        updatedFolder.setID(mainAssetFolder.getID());
        AssetFolder saved2 = saveAssetFolder(updatedFolder, null);
        AssetFolderFixture.compare(updatedFolder, saved2, null);
        AssetFolder loaded = (AssetFolder) loadFromID(updatedFolder, null);
        AssetFolderFixture.compare(loaded, saved2, null);
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.client.junit.interfaces.RPMUnitTestI#testUpdateWithScopeLoadFromID()
     */
    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        initWithFullScope();

        AssetFolder savedParent = saveAssetFolder(parentAssetFolder,
                parentScope);
        AssetFolder[] savedFolders = savedParent.getFolders();
        AssetFolder saved = savedFolders[0];
        mainAssetFolder.setID(saved.getID());
        AssetFolder updatedFolder = AssetFolderFixture.updateFixture(
                saved, NB_ASSETS_BY_LEVEL, this);
        updatedFolder.setID(mainAssetFolder.getID());
        AssetFolder saved2 = saveAssetFolder(updatedFolder, mainScope);
        AssetFolderFixture.compare(updatedFolder, saved2, mainScope);
        AssetFolder loaded = (AssetFolder) loadFromID(updatedFolder, mainScope);
        AssetFolderFixture.compare(loaded, saved2, mainScope);
    }

    protected AssetFolder saveAssetFolder(AssetFolder ast, AssetScope scope)
            throws AssertionError, Exception
    {

        //Parent needs to be in scope to be saved...
        if (scope == null)
        {
            scope = new AssetScope();
        }

        SingleResult result = save(ast, scope);

        ast = (AssetFolder) result.getRpmObject();
        return ast;
    }

    private void loadFolderFromId(AssetFolder objectToCompare, AssetScope scope)
            throws AssertionError, Exception
    {
        logger.info("loadFolderFromId");

        RPMObject obj = new AssetFolder();
        obj.setID(objectToCompare.getID());
        RPMObject rpmObject = loadFromID(obj, scope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof AssetFolder);

        if (scope != null)
        {
            logger.info("Scoping" + scope.toString());
        }
        else
        {
            logger.info("Scoping is null");
        }
        logger.info("======================");
        logger.info("ObjectToCompare: " + objectToCompare);
        logger.info("======================");
        logger.info("LoadedObject: " + rpmObject);

        //Needed by tearDown method
        if (rpmObject != null)
        {
            parent = (AssetFolder) rpmObject.getParent();
        }

        AssetFolderFixture.compare(objectToCompare, (AssetFolder) rpmObject,
                scope);
    }

    public void testDelete() throws AssertionError, Exception
    {

        mainAssetFolder = AssetFolderFixture.createFixture(false,
                NB_ASSETS_BY_LEVEL, mainScope, this);
        mainAssetFolder = saveAssetFolder(mainAssetFolder, mainScope);

        RPMObject rpmObject = loadFromID(mainAssetFolder, mainScope);

        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof AssetFolder);
        AssetFolderFixture.compare(mainAssetFolder, (AssetFolder) rpmObject,
                null);

        delete(mainAssetFolder);

        RPMObject deleted = null;
        try
        {
            deleted = loadFromID(mainAssetFolder, null);
        }
        catch (Exception e)
        {
            //deleted...
        }

        assertNull(deleted);

    }

    private void initWithFullScope() throws Exception
    {

        mainScope = new AssetScope();
        parentScope = new AssetScope();
        mainScope.setParent(parentScope);
        parentScope.setFolders(mainScope);
        AssetScope childrenScope = new AssetScope();
        mainScope.setFolders(childrenScope);
        AssetScope grandChildrenScope = new AssetScope();
        childrenScope.setFolders(grandChildrenScope);
        parentAssetFolder = AssetFolderFixture.createFixture(true,
                NB_FOLDERS_BY_LEVEL, parentScope, this);
        AssetFolder[] folders = parentAssetFolder.getFolders();
        mainAssetFolder = folders[0];
    }

}