package com.ibm.rpm.client.junit.asset.testcases;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AssetCategory;
import com.ibm.rpm.applicationadministration.containers.AssetRTF;
import com.ibm.rpm.applicationadministration.containers.AssetState;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.ChargeType;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.scope.AssetModuleScope;
import com.ibm.rpm.client.junit.asset.fixtures.AssetModuleFixture;
import com.ibm.rpm.client.junit.framework.testcases.ModuleTestController;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;

/**
 * @author karim
 *  
 */
public class TestAssetModule extends ModuleTestController
{
    public TestAssetModule(String testName)
    {
        setName(testName);
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = AssetModuleFixture.createFullScope();
        moduleContaineName = "ASSETMODULE";
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
        
    }
    
    public RPMObject load(AbstractModule aModule, RPMObjectScope aScope)
            throws Exception
    {
        AssetModule module = (AssetModule) aModule;
        AssetModuleScope scope = (AssetModuleScope) aScope;

        AssetModule loadedModule = new AssetModule();

        if(scope!=null)
        {
        	if(scope.getAssetFolders()!=null){
        	    //AssetFolders
        	    AssetFolder[] assetFolders = (AssetFolder[]) loadElements( 
        	    		module.getAssetFolders(),
        				scope.getAssetFolders(),
        				AssetFolder.class);
        	    loadedModule.setAssetFolders(assetFolders);
        	}

        	if(scope.getAttributeCategories()!=null){
        	    //AttributeCategories
                AttributeCategory[] attributeCategories = loadAttributeCategories(
                        module.getAttributeCategories(), scope.getAttributeCategories());
                loadedModule.setAttributeCategories(attributeCategories);
        	}

        	if(scope.isChargeTypes()){
        	    //ChargeTypes
        	    ChargeType[] chargeTypes = (ChargeType[]) loadDatafields(
        	    		module.getChargeTypes(),
        				scope.isChargeTypes(),
        				ChargeType.class);
        	    loadedModule.setChargeTypes(chargeTypes);    	    
        	}

        	if(scope.isAssetCategories()){
        	    //AssetCategories
        	    AssetCategory[] assetCategories = (AssetCategory[]) loadDatafields(
        	    		module.getAssetCategories(),
        				scope.isAssetCategories(),
        				AssetCategory.class);
        	    loadedModule.setAssetCategories(assetCategories);   	    
        	}

        	if(scope.isAssetStates()){
        	    //AssetStates
        	    AssetState[] assetStates = (AssetState[]) loadDatafields(
        	    		module.getAssetStates(),
        				scope.isAssetStates(),
        				AssetState.class);
        	    loadedModule.setAssetStates(assetStates);    	    
        	}

        	if(scope.isAssetRtfs()){
        	    //AssetRTFs
        	    AssetRTF[] assetRTFs = (AssetRTF[]) loadDatafields(
        	    		module.getAssetRtfs(),
        				scope.isAssetRtfs(),
        				AssetRTF.class);
        	    loadedModule.setAssetRtfs(assetRTFs);    	    
        	}

        	if(scope.getCustomFieldCategories()!=null){
        	    //CustomFields
                CustomFieldCategory[] customFieldCategories = (CustomFieldCategory[]) loadElements(
                        module.getCustomFieldCategories(), scope
                                .getCustomFieldCategories(), CustomFieldCategory.class);
                loadedModule.setCustomFieldCategories(customFieldCategories);    	    
        	}

        	if(scope.getScorecardFolders()!=null){
        	    //ScorecardFolders
        	    ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) loadElements( 
        	    		module.getScorecardFolders(),
        				scope.getScorecardFolders(),
        				ScorecardFolder.class);
        	    loadedModule.setScorecardFolders(scorecardFolders);    	    
        	}
        }

        return loadedModule;
    }

    protected void deleteModule(AbstractModule aModule, RPMObjectScope aScope)
            throws AssertionError, Exception
    {
        AssetModule module = (AssetModule) aModule;
        AssetModuleScope scope = (AssetModuleScope) aScope;
        
        //delete the element
        if (module != null && scope != null)
        {
            if (module.getAssetFolders() != null && scope.getAssetFolders()!=null)
	        {
	            delete(module.getAssetFolders());
	        }
            if (module.getAttributeCategories() != null && scope.getAttributeCategories()!=null)
            {
                delete(module.getAttributeCategories());
            }
            if (module.getCustomFieldCategories() != null && scope.getCustomFieldCategories()!=null)
            {
                delete(module.getCustomFieldCategories());
            }
	        if (module.getScorecardFolders() != null && scope.getScorecardFolders()!=null)
	        {
	            delete(module.getScorecardFolders());
	        }
	        
	        //reload the deleted element
	        AssetModule deletedModule = (AssetModule) loadDeletedModule(module,
	                scope);

	        //Validate the deleted element
            if (module.getAssetFolders() != null && scope.getAssetFolders()!=null)
	        {
    	        assertNull(deletedModule.getAssetFolders());
	        }
            if (module.getAttributeCategories() != null && scope.getAttributeCategories()!=null)
            {
    	        assertNull(deletedModule.getAttributeCategories());
            }
            if (module.getCustomFieldCategories() != null && scope.getCustomFieldCategories()!=null)
            {
    	        assertNull(deletedModule.getCustomFieldCategories());
            }
	        if (module.getScorecardFolders() != null && scope.getScorecardFolders()!=null)
	        {
		        assertNull(deletedModule.getScorecardFolders());
	        }
        }
    }

    protected AbstractModule createFixture(RPMObjectScope scope)
            throws Exception
    {
        return AssetModuleFixture.createFixture((AssetModuleScope) scope, this);
    }

    protected AbstractModule updateFixture(AbstractModule module,
            RPMObjectScope scope) throws Exception
    {
        return AssetModuleFixture.updateFixture((AssetModule) module,
                (AssetModuleScope) scope, this);
    }

    protected void compare(AbstractModule module,
            AbstractModule returnedModule, RPMObjectScope scope)
            throws AssertionFailedError, Exception
    {
        AssetModuleFixture.compare((AssetModule) module,
                (AssetModule) returnedModule, (AssetModuleScope) scope);
    }
}
