/*
 * Created on January 9, 2006
 * @author mmatwiy 
 * Test cases for creation of an AssetFolder.
 */
package com.ibm.rpm.client.junit.asset.testcases.validation;

import com.ibm.rpm.asset.constants.ValidationConstants;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

public class TestAssetFolderValidation extends GenericValidation
{

    private AssetFolder assetFolder;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithParentAssetFolder() throws AssertionError,
            Exception
    {
        assetFolder = new AssetFolder();
        assetFolder
            .setName(getTestString(ValidationConstants.ABSTRACTASSET_NAME_MAX));

        AssetFolder parentFolder = AssetFolderFixture.createSaveFixture(this,
            null);
        assetFolder.setParent(parentFolder);

        // Attempt to save the object
        assetFolder = (AssetFolder) saveMustSucceed(assetFolder, AssetFolder.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/AssetFolder[id='" + assetFolder.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssetFolder);

        // Check that the loaded AssetFolder is identical to the one we
        // requested to store
        AssetFolderFixture.compare(assetFolder, (AssetFolder) rpmObjects[0], null);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        assetFolder = new AssetFolder();

        // set mandatory field
        assetFolder.setName("test");

        // create and set invalid parent
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        assetFolder.setParent(parentAsset);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, assetFolder.getClass()
                .getName(), ValidationConstants.PARENT, assetFolder.getID())
        };

        verifySaveMustFail(assetFolder, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        assetFolder = new AssetFolder();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY,
                assetFolder.getClass().getName(), ValidationConstants.NAME_FIELD,
                assetFolder.getID())
        };

        verifySaveMustFail(assetFolder, null, expectedExceptions);
    }
    
    public void testFailureValidationMin() throws AssertionError, Exception
    {
        assetFolder = new AssetFolder();

        assetFolder.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MIN_LENGTH, assetFolder.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, assetFolder
                    .getID())
        };

        verifySaveMustFail(assetFolder, null, expectedExceptions);
    }
    
    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        assetFolder = new AssetFolder();
        assetFolder
            .setName(getTestString(ValidationConstants.ABSTRACTASSET_NAME_MAX));

        // Attempt to save the AssetFolder object
        assetFolder = (AssetFolder) saveMustSucceed(assetFolder, AssetFolder.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/AssetFolder[id='"
                + assetFolder.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssetFolder);

        // Check that the loaded AssetFolder is identical to the one we
        // requested to store
        AssetFolderFixture.compare(assetFolder, (AssetFolder) rpmObjects[0], null);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        assetFolder = new AssetFolder();

        // Set field values with maxLength and verify that they are accepted
        assetFolder
            .setName(getTestString(ValidationConstants.ABSTRACTASSET_NAME_MAX));

        // Attempt to save the AssetFolder object
        assetFolder = (AssetFolder) saveMustSucceed(assetFolder, AssetFolder.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/AssetFolder[id='" + assetFolder.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssetFolder);

        // Check that the loaded AssetFolder is identical to the one we
        // requested to store
        AssetFolderFixture.compare(assetFolder, (AssetFolder) rpmObjects[0], null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        assetFolder = new AssetFolder();

        // Set field values with maxLength+1 and verify that they are not accepted
        assetFolder
            .setName(getTestString(ValidationConstants.ABSTRACTASSET_NAME_MAX + 1));
         RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, assetFolder.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, assetFolder.getID())        };

        verifySaveMustFail(assetFolder, null, expectedExceptions);
    }

}
