/*
 * Created on January 10, 2006
 * @author mmatwiy 
 * Test cases for creation of an AssetResourceAssignment.
 */
package com.ibm.rpm.client.junit.asset.testcases.validation;

import com.ibm.rpm.asset.constants.ValidationConstants;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.AssetResourceAssignment;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Resource;

public class TestAssetResourceAssignmentValidation extends GenericValidation
{

    private AssetResourceAssignment assetAssignment;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithParentAsset() throws AssertionError, Exception
    {
        assetAssignment = new AssetResourceAssignment();

        Asset parentAsset = AssetFixture.createSaveFixture(this);
        assetAssignment.setParent(parentAsset);
        
        Resource resource = ResourceFixture.createSaveFixture(this);
        assetAssignment.setResource(resource);

        // Attempt to save the object
        assetAssignment = (AssetResourceAssignment) saveMustSucceed(
            assetAssignment, AssetResourceAssignment.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath(
            "/Asset[AssetResourceAssignment[id='" + assetAssignment.getID()
                    + "']]", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Asset);

        // Check that the loaded AssetResourceAssignment is identical to the one we
        // requested to store
        AssetFixture.compare(parentAsset, (Asset) rpmObjects[0], null);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        assetAssignment = new AssetResourceAssignment();

        // create and set invalid parent
        AssetFolder parentFolder = AssetFolderFixture.createSaveFixture(this,
            null);
        assetAssignment.setParent(parentFolder);
        
        // set mandatory field
        Resource resource = ResourceFixture.createSaveFixture(this);
        assetAssignment.setResource(resource);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, assetAssignment
                .getClass().getName(), ValidationConstants.PARENT,
                assetAssignment.getID())
        };

        verifySaveMustFail(assetAssignment, null, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        assetAssignment = new AssetResourceAssignment();
        
        // set mandatory field
        Resource resource = ResourceFixture.createSaveFixture(this);
        assetAssignment.setResource(resource);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, assetAssignment.getClass()
                .getName(), ValidationConstants.PARENT, assetAssignment.getID())
        };

        verifySaveMustFail(assetAssignment, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        assetAssignment = new AssetResourceAssignment();

        // create and set parent
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        assetAssignment.setParent(parentAsset);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, assetAssignment.getClass()
                .getName(), ValidationConstants.RESOURCE_FIELD, assetAssignment
                .getID())
        };

        verifySaveMustFail(assetAssignment, null, expectedExceptions);
    }

}
