/*
 * Created on January 9, 2006
 * @author mmatwiy 
 * Test cases for creation of an Asset.
 */
package com.ibm.rpm.client.junit.asset.testcases.validation;

import com.ibm.rpm.asset.constants.ValidationConstants;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.CrossChargeFinancials;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

public class TestAssetValidation extends GenericValidation
{

    private Asset asset;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithParentAssetFolder() throws AssertionError,
            Exception
    {
        asset = new Asset();
        asset
            .setName(getTestString(ValidationConstants.ABSTRACTASSET_NAME_MAX));

        AssetFolder parentFolder = AssetFolderFixture.createSaveFixture(this,
            null);
        asset.setParent(parentFolder);

        // Attempt to save the object
        asset = (Asset) saveMustSucceed(asset, Asset.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Asset[id='" + asset.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Asset);

        // Check that the loaded Asset is identical to the one we
        // requested to store
        AssetFixture.compare(asset, (Asset) rpmObjects[0], null);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        asset = new Asset();

        // set mandatory field
        asset.setName("test");

        // create and set invalid parent
        asset.setParent(new CrossChargeFinancials());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, asset.getClass()
                .getName(), ValidationConstants.PARENT, asset.getID())
        };

        verifySaveMustFail(asset, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        asset = new Asset();
        
        AssetFolder parentFolder = AssetFolderFixture.createSaveFixture(this,
            null);
        asset.setParent(parentFolder);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY,
                asset.getClass().getName(), ValidationConstants.NAME_FIELD,
                asset.getID())
        };

        verifySaveMustFail(asset, null, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        asset = new Asset();
        asset
            .setName(getTestString(ValidationConstants.ABSTRACTASSET_NAME_MAX));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT,
                asset.getClass().getName(), ValidationConstants.PARENT, asset
                    .getID())
        };

        verifySaveMustFail(asset, null, expectedExceptions);
    }

    public void testUpdateWithoutMandatoryField() throws AssertionError, Exception
    {
    	asset = AssetFixture.createSaveFixture(this);
    	assertNotNull(asset);
    	
    	asset.setTracked(null);
    	
    	RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY,
                asset.getClass().getName(), ValidationConstants.TRACKED_FIELD, asset
                    .getID())
        };

        verifySaveMustFail(asset, null, expectedExceptions);
    }
    
    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        asset = new Asset();
        
        AssetFolder parentFolder = AssetFolderFixture.createSaveFixture(this,
            null);
        asset.setParent(parentFolder);

        // Set field values with maxLength and verify that they are accepted
        asset
            .setName(getTestString(ValidationConstants.ABSTRACTASSET_NAME_MAX));
        asset
            .setBarcodeNumber(getTestString(ValidationConstants.BARCODE_NUMBER_MAX));
        asset.setContact(getTestString(ValidationConstants.CONTACT_MAX));
        asset
            .setContactEmail(getTestString(ValidationConstants.CONTACT_EMAIL_MAX));
        asset
            .setContactPhone(getTestString(ValidationConstants.CONTACT_PHONE_MAX));
        asset.setModel(getTestString(ValidationConstants.MODEL_MAX));
        asset
            .setModelNumber(getTestString(ValidationConstants.MODEL_NUMBER_MAX));
        asset
            .setSerialNumber(getTestString(ValidationConstants.SERIAL_NUMBER_MAX));
        asset.setAssetLife(new Integer(ValidationConstants.ASSET_LIFE_MAX));
        asset.setDepreciationMethod(new Integer(
            ValidationConstants.DEPRECIATION_METHOD_MAX));
        asset.setInitialCost(new Double(ValidationConstants.INITIAL_COST_MAX));
        asset.setQuantity(new Integer(ValidationConstants.ASSET_QUANTITY_MAX));

        // Attempt to save the Asset object
        asset = (Asset) saveMustSucceed(asset, Asset.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Asset[id='" + asset.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Asset);

        // Check that the loaded Asset is identical to the one we
        // requested to store
        AssetFixture.compare(asset, (Asset) rpmObjects[0], null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        asset = new Asset();
        
        AssetFolder parentFolder = AssetFolderFixture.createSaveFixture(this,
            null);
        asset.setParent(parentFolder);
        
        // Set field values with maxLength+1 and verify that they are not accepted
        asset
            .setName(getTestString(ValidationConstants.ABSTRACTASSET_NAME_MAX + 1));
        asset
            .setBarcodeNumber(getTestString(ValidationConstants.BARCODE_NUMBER_MAX + 1));
        asset.setContact(getTestString(ValidationConstants.CONTACT_MAX + 1));
        asset
            .setContactEmail(getTestString(ValidationConstants.CONTACT_EMAIL_MAX + 1));
        asset
            .setContactPhone(getTestString(ValidationConstants.CONTACT_PHONE_MAX + 1));
        asset.setModel(getTestString(ValidationConstants.MODEL_MAX + 1));
        asset
            .setModelNumber(getTestString(ValidationConstants.MODEL_NUMBER_MAX + 1));
        asset
            .setSerialNumber(getTestString(ValidationConstants.SERIAL_NUMBER_MAX + 1));
        asset.setAssetLife(new Integer(ValidationConstants.ASSET_LIFE_MAX + 1));
        asset.setDepreciationMethod(new Integer(
            ValidationConstants.DEPRECIATION_METHOD_MAX + 1));
        asset.setInitialCost(new Double(
            ValidationConstants.INITIAL_COST_MAX + 1));
        asset.setQuantity(new Integer(
            ValidationConstants.ASSET_QUANTITY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, asset.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, asset.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, asset.getClass()
                    .getName(), ValidationConstants.BARCODE_NUMBER_FIELD, asset
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, asset.getClass()
                    .getName(), ValidationConstants.CONTACT_FIELD, asset
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, asset.getClass()
                    .getName(), ValidationConstants.CONTACT_EMAIL_FIELD, asset
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, asset.getClass()
                    .getName(), ValidationConstants.CONTACT_PHONE_FIELD, asset
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, asset.getClass()
                    .getName(), ValidationConstants.MODEL_FIELD, asset.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, asset.getClass()
                    .getName(), ValidationConstants.MODEL_NUMBER_FIELD, asset
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, asset.getClass()
                    .getName(), ValidationConstants.SERIAL_NUMBER_FIELD, asset
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, asset.getClass()
                    .getName(), ValidationConstants.ASSET_LIFE_FIELD, asset
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, asset.getClass()
                    .getName(), ValidationConstants.DEPRECIATION_METHOD_FIELD,
                    asset.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, asset.getClass()
                    .getName(), ValidationConstants.INITIAL_COST_FIELD, asset
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, asset.getClass()
                    .getName(), ValidationConstants.ASSET_QUANTITY_FIELD, asset
                    .getID())

        };

        verifySaveMustFail(asset, null, expectedExceptions);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        asset = new Asset();
        
        AssetFolder parentFolder = AssetFolderFixture.createSaveFixture(this,
            null);
        asset.setParent(parentFolder);
        
        asset
            .setName(getTestString(ValidationConstants.ABSTRACTASSET_NAME_MAX));

        // Set field values with minLength and verify that they are accepted
        asset.setAssetLife(new Integer(ValidationConstants.ASSET_LIFE_MIN));
        asset.setDepreciationMethod(new Integer(
            ValidationConstants.DEPRECIATION_METHOD_MIN));
        asset.setInitialCost(new Double(ValidationConstants.INITIAL_COST_MIN));
        asset.setQuantity(new Integer(ValidationConstants.ASSET_QUANTITY_MIN));

        // Attempt to save the Asset object
        asset = (Asset) saveMustSucceed(asset, Asset.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Asset[id='" + asset.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Asset);

        // Check that the loaded Asset is identical to the one we
        // requested to store
        AssetFixture.compare(asset, (Asset) rpmObjects[0], null);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        asset = new Asset();
        
        AssetFolder parentFolder = AssetFolderFixture.createSaveFixture(this,
            null);
        asset.setParent(parentFolder);
        
        // Set field values with minLength -1 and verify that they are not
        // accepted        
        asset.setName("");
        asset.setAssetLife(new Integer(ValidationConstants.ASSET_LIFE_MIN - 1));
        asset.setDepreciationMethod(new Integer(
            ValidationConstants.DEPRECIATION_METHOD_MIN - 1));
        asset.setInitialCost(new Double(
            ValidationConstants.INITIAL_COST_MIN - 1));
        asset.setQuantity(new Integer(
            ValidationConstants.ASSET_QUANTITY_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MIN_LENGTH, asset.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, asset
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, asset.getClass()
                    .getName(), ValidationConstants.ASSET_LIFE_FIELD, asset
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, asset.getClass()
                    .getName(), ValidationConstants.DEPRECIATION_METHOD_FIELD,
                    asset.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, asset.getClass()
                    .getName(), ValidationConstants.INITIAL_COST_FIELD, asset
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, asset.getClass()
                    .getName(), ValidationConstants.ASSET_QUANTITY_FIELD, asset
                    .getID())
        };

        verifySaveMustFail(asset, null, expectedExceptions);
    }
}
