/*
 * Created on January 10, 2006
 * @author mmatwiy 
 * Test cases for creation of a DepreciationSchedule container.
 */
package com.ibm.rpm.client.junit.asset.testcases.validation;

import com.ibm.rpm.asset.constants.ValidationConstants;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.DepreciationSchedule;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.client.junit.asset.fixtures.DepreciationScheduleFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

public class TestDepreciationScheduleValidation extends GenericValidation
{

    private DepreciationSchedule depreciationSchedule;

    public void setUp() throws Exception
    {
        super.setUp();
        
        depreciationSchedule = new DepreciationSchedule();

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithParentAsset() throws AssertionError, Exception
    {
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        depreciationSchedule.setParent(parentAsset);

        // Attempt to save the object
        depreciationSchedule = (DepreciationSchedule) saveMustSucceed(
            depreciationSchedule, DepreciationSchedule.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/DepreciationSchedule[id='"
                + depreciationSchedule.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof DepreciationSchedule);

        // Check that the loaded DepreciationSchedule is identical to the one we
        // requested to store
        DepreciationScheduleFixture.compare(depreciationSchedule, (DepreciationSchedule) rpmObjects[0], null);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        AssetFolder parentFolder = AssetFolderFixture.createSaveFixture(this,
            null);
        depreciationSchedule.setParent(parentFolder);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE,
                depreciationSchedule.getClass().getName(),
                ValidationConstants.PARENT, depreciationSchedule.getID())
        };

        verifySaveMustFail(depreciationSchedule, null, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, depreciationSchedule
                .getClass().getName(), ValidationConstants.PARENT,
                depreciationSchedule.getID())
        };

        verifySaveMustFail(depreciationSchedule, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        depreciationSchedule.setParent(parentAsset);

        depreciationSchedule.setAmount(new Double(ValidationConstants.AMOUNT_MAX));

        // Attempt to save the DepreciationSchedule object
        depreciationSchedule = (DepreciationSchedule) saveMustSucceed(
            depreciationSchedule, DepreciationSchedule.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/DepreciationSchedule[id='"
                + depreciationSchedule.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof DepreciationSchedule);

        // Check that the loaded DepreciationSchedule is identical to the one we
        // requested to store
        DepreciationScheduleFixture.compare(depreciationSchedule,
             (DepreciationSchedule) rpmObjects[0], null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        depreciationSchedule.setParent(parentAsset);

        // Set field values with maxLength + 1 and verify that they are not accepted
        depreciationSchedule.setAmount(new Double(
            ValidationConstants.AMOUNT_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, depreciationSchedule
                    .getClass().getName(), ValidationConstants.AMOUNT_FIELD,
                    depreciationSchedule.getID())
        };

        verifySaveMustFail(depreciationSchedule, null, expectedExceptions);
    }
    

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        depreciationSchedule.setParent(parentAsset);

        // Set minimum values and verify that they are accepted
        depreciationSchedule.setAmount(new Double(ValidationConstants.AMOUNT_MIN));

        // Attempt to save the DepreciationSchedule object
        depreciationSchedule = (DepreciationSchedule) saveMustSucceed(
            depreciationSchedule, DepreciationSchedule.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/DepreciationSchedule[id='"
                + depreciationSchedule.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof DepreciationSchedule);

        // Check that the loaded DepreciationSchedule is identical to the one we
        // requested to store
        DepreciationScheduleFixture.compare(depreciationSchedule,
             (DepreciationSchedule) rpmObjects[0], null);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        Asset parentAsset = AssetFixture.createSaveFixture(this);
        depreciationSchedule.setParent(parentAsset);

        // Set field values with maxLength + 1 and verify that they are not accepted
        depreciationSchedule.setAmount(new Double(
            ValidationConstants.AMOUNT_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, depreciationSchedule
                    .getClass().getName(), ValidationConstants.AMOUNT_FIELD,
                    depreciationSchedule.getID())
        };

        verifySaveMustFail(depreciationSchedule, null, expectedExceptions);
    }

}
