/*
 * Created on Aug 4, 2005
 *
 */
package com.ibm.rpm.client.junit.clientcostcenters.fixtures;

import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RtfAssignmentFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.containers.Division;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.framework.RPMObject;


/**
 * @author sailajap
 *
 */
public class ClientCostCenterFixture extends RPMObjectFixture 
{

    private static final String DEFAULT_TEST_CLIENTCOSTCENTER_NAME = "DEFAULT_TEST_CLIENTCOSTCENTER";
    
	private static final int NB_ATTRIBUTE_ASSIGNMENTS = 2;
	
	protected static ArrayList localSkipGetters = new ArrayList();
	protected static ArrayList localSkipSetters = new ArrayList();
	static
    {
    	//set skipGetters
    	localSkipGetters.addAll(skipGetters);
    	
    	//set skipSetters
    	localSkipSetters.addAll(skipSetters);
    	localSkipSetters.add("setDefaultFlag");
    }
	
	public static ClientCostCenter loadOrCreateDefaultTestClientCostCenter(ClientTestController ctc)throws Exception{
	    RPMObject[] objs = ctc.loadFromXpath("/CLIENTCOSTCENTER[NAME='" +DEFAULT_TEST_CLIENTCOSTCENTER_NAME+ "']", null);
        if (objs == null || objs.length == 0)
        {
            ClientCostCenter defaultTestClientCostCenter = createFixture(null, ctc);
            defaultTestClientCostCenter.setName(DEFAULT_TEST_CLIENTCOSTCENTER_NAME);
            defaultTestClientCostCenter = (ClientCostCenter)save(ctc, defaultTestClientCostCenter, null);
            //we don't want the root portfolio to be removed at the end of the test
            ctc.removeObjectToDelete(defaultTestClientCostCenter);
            return defaultTestClientCostCenter;
        } else {
            return (ClientCostCenter)objs[0];
        }
	}
	
    public static ClientCostCenter createSaveFixture(ClientTestController ctc) throws Exception {

    	ClientCostCenterScope scope = createScope(true, true);
    	
    	return createSaveFixture(scope, ctc);
    }

    public static ClientCostCenter createSaveFixture(ClientCostCenterScope scope, ClientTestController ctc) throws Exception {

    	ClientCostCenter costCenter = createFixture(scope, ctc);

    	//save the cost center
    	costCenter = (ClientCostCenter) save(ctc, costCenter, scope);  

    	return costCenter;
    }
    
    public static ClientCostCenter createFixture(ClientCostCenterScope scope, ClientTestController ctc) throws Exception 
    {
    	return createClientCostCenter(random(), scope, ctc);
    }
    
    private static ClientCostCenter createClientCostCenter(int index, ClientCostCenterScope scope, ClientTestController ctc)throws Exception
	{
    	ClientCostCenter costCenter = new ClientCostCenter();
    	
    	//set the fields
    	HelperTestUtilities.setByIntrospection(costCenter, index, localSkipSetters);
    	
   	
    	//rtf assignments
    	if(scope!=null && scope.isRtfAssignments()){
    		RtfAssignment[] rtfAssignments = RtfAssignmentFixture.createAllAssignmentFixtures(costCenter,ctc);
    		costCenter.setRtfAssignments(rtfAssignments);
    	}
    	   	  	
    	//attribute assignments
    	if(scope!=null && scope.isAttributeAssignments()){
            AttributeAssignment[] attributeAssignments = new AttributeAssignment[NB_ATTRIBUTE_ASSIGNMENTS];
            for (int i = 0; i < NB_ATTRIBUTE_ASSIGNMENTS; i++)
            {
                AttributeAssignment paa = AttributeAssignmentFixture.createAssignmentFixture(costCenter, ctc);
                attributeAssignments[i] = paa;
            }        
            costCenter.setAttributeAssignments(attributeAssignments);
    	}
    	
    	//Document Element
    	if(scope!=null && scope.getDocumentFolder()!=null){
    		DocumentFolder documentFolder = (DocumentFolder)DocumentElementFixture.createFixture(costCenter, new DocumentFolder(), 0, scope.getDocumentFolder(), ctc);
    		costCenter.setDocumentFolder(documentFolder);
    	}
    	
    	return costCenter;
    }
    
    public static void setDefault(ClientCostCenter costCenter){
        costCenter.setDefaultFlag(new Boolean(true));
    }
    
    public static void updateFixture(ClientCostCenter costCenter, ClientCostCenterScope scope)throws Exception
	{
    	updateClientCostCenter(costCenter, random(), scope);
    }
    
    private static void updateClientCostCenter(ClientCostCenter costCenter, int index, ClientCostCenterScope scope)throws Exception
	{
    	//update the fields
    	HelperTestUtilities.setByIntrospection(costCenter, index, localSkipSetters);
    	
    	//update rtf assignments
    	if(scope!=null && scope.isRtfAssignments()){
    		RtfAssignment[] rtfAssignments = costCenter.getRtfAssignments();
    		if(rtfAssignments!=null){
            	for(int i=0;i<rtfAssignments.length;i++){
            		RtfAssignmentFixture.updateAssignmentFixture(rtfAssignments[i]);
            	}    			
    		}
    	}    	

    	//update attribute assignment
    	if(scope!=null && scope.isAttributeAssignments()){
    		AttributeAssignment[] attributeAssignments =  costCenter.getAttributeAssignments();
    		if(attributeAssignments!=null){
    			for(int i=0;i<attributeAssignments.length;i++){
    				AttributeAssignmentFixture.updateAssignmentFixture(attributeAssignments[i]);
    			}
    		}
    	}
    }
    
    public static void compare(ClientCostCenter ccc1, ClientCostCenter ccc2,
    		ClientCostCenterScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(ccc1, ccc2, localSkipGetters);

        if(ccc1.getDefaultFlag()!=null && ccc1.getDefaultFlag().booleanValue()){
        	//only compare default flag if set to yes, because the default flag SP is only called 
        	//when the default flag is set to yes 
        	Assert.assertEquals(ccc1.getDefaultFlag(), ccc2.getDefaultFlag());
        }
            
        if (scope == null)
            return;
 
        //validate the parents
        if (scope.getParent() != null)
        {
        	ClientCostCenterScope parentScope = (ClientCostCenterScope)scope.getParent();
            if (parentScope != null)
            {
            	parentScope.setClientCostCenters(null);
            }

            if(isDivisionParent(ccc1)){
                DivisionFixture.compare((Division) ccc1.getParent(),
                        (Division) ccc2.getParent(), parentScope);
            } else if (isFolderParent(ccc2)){
            	FolderFixture.compare((ClientFolder) ccc1.getParent(),
                        (ClientFolder) ccc2.getParent(), parentScope);
            }
        }
        
        //compare the rtfAssignments
        if(scope.isRtfAssignments()){
    		AttributeAssignment[] attributeAssignments1 =  ccc1.getAttributeAssignments();
    		AttributeAssignment[] attributeAssignments2 =  ccc2.getAttributeAssignments();
    		AttributeAssignmentFixture.compare(attributeAssignments1, attributeAssignments2);
        }
        
        //compare the attribute assignments
        if(scope.isAttributeAssignments()){
    		RtfAssignment[] rtfAssignments1 =  ccc1.getRtfAssignments();
    		RtfAssignment[] rtfAssignments2 =  ccc2.getRtfAssignments();
    		RtfAssignmentFixture.compare(rtfAssignments1, rtfAssignments2);
        }
        
        // DocumentFolder
        if(scope.getDocumentFolder()!=null){
    		DocumentFolder documentFolder1 =  ccc1.getDocumentFolder();
    		DocumentFolder documentFolder2 =  ccc2.getDocumentFolder();
    		DocumentElementFixture.compare(documentFolder1, documentFolder2, scope.getDocumentFolder());
        }
    }

    public static void compare(ClientCostCenter[] cArray1, ClientCostCenter[] cArray2,
    		ClientCostCenterScope scope) throws AssertionFailedError, Exception
    {
    	sortWithComparator(cArray1, RPMObjectNameAndIDComparator.INSTANCE);
    	sortWithComparator(cArray2, RPMObjectNameAndIDComparator.INSTANCE); 
    	
        RPMObjectFixture.compare(cArray1, cArray2);
        if(cArray1 == null) return;

        for (int i = 0; i < cArray1.length; i++)
        {
            ClientCostCenterFixture.compare(cArray1[i], cArray2[i], scope);
        }
    }
    
    //indicated whether the parent of the current ccc is a AssetFolder
    private static boolean isFolderParent(ClientCostCenter ccc){
    	if(ccc!=null && ccc.getParent()!=null && ccc.getParent() instanceof ClientFolder){
    		return true;
    	}
    	return false;
    }
    
    //indicated whether the parent of the current ccc is a Division
    private static boolean isDivisionParent(ClientCostCenter ccc){
    	if(ccc!=null && ccc.getParent()!=null && ccc.getParent() instanceof Division){
    		return true;
    	}
    	return false;
    }
    
    public static ClientCostCenterScope createScope(boolean includeRtfAssignments, boolean includeAttributeAssignments){
    	ClientCostCenterScope scope = new ClientCostCenterScope();
    	
       	scope.setRtfAssignments(includeRtfAssignments);    		
    	scope.setAttributeAssignments(includeAttributeAssignments);
    	
    	return scope;
    }

}