/*
 * Created on Sep 23, 2005
 *
 */
package com.ibm.rpm.client.junit.clientcostcenters.fixtures;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.fixtures.AbstractModuleFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.containers.ClientModule;
import com.ibm.rpm.clientcostcenters.containers.Division;
import com.ibm.rpm.clientcostcenters.containers.GenericClient;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.clientcostcenters.scope.ClientModuleScope;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;

/**
 * @author karim
 *  
 */
public class ClientModuleFixture extends AbstractModuleFixture
{

    public static ClientModule createFixture(ClientModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        return createModule(scope, ctc);
    }

    public static ClientModule updateFixture(ClientModule module,
            ClientModuleScope scope, ClientTestController ctc) throws Exception
    {
        return updateModule(module, scope, ctc);
    }

    private static ClientModule createModule(ClientModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        ClientModule module = new ClientModule();

        if(scope!=null){
            //Clients
            if(scope.getClients()!=null){
    	        GenericClient[] clients = (GenericClient[]) createElements(
    	                scope.getClients(), ClientFolder.class, ctc);
    	        module.setClients(clients);
            }
            
            //AttributeCategories
            if(scope.getAttributeCategories()!=null){
                AttributeCategory[] attributeCategories = (AttributeCategory[]) createElements(
                        scope.getAttributeCategories(),
                        AttributeCategory.class, ctc);
                module.setAttributeCategories(attributeCategories);                
            }

            //CustomFields
            if(scope.getCustomFieldCategories()!=null){
                CustomFieldCategory[] customFieldCategories = (CustomFieldCategory[]) createElements(
                        scope.getCustomFieldCategories(),
                        CustomFieldCategory.class, ctc);
                module.setCustomFieldCategories(customFieldCategories);
            }
        }

        return module;
    }

    private static ClientModule updateModule(ClientModule module,
            ClientModuleScope scope, ClientTestController ctc) throws Exception
    {
        if(scope!=null){

            //Clients
            if(scope.getClients()!=null){
                GenericClient[] clients = (GenericClient[]) updateElements(module
                        .getClients(), scope.getClients(), ctc);
                module.setClients(clients);                
            }
            
            //AttributeCategories
            if(scope.getAttributeCategories()!=null){
                AttributeCategory[] attributeCategories = (AttributeCategory[]) updateElements(
                        module.getAttributeCategories(), scope.getAttributeCategories(), ctc);
                module.setAttributeCategories(attributeCategories);                
            }

            //CustomFields
            if(scope.getCustomFieldCategories()!=null){
                module.setCustomFieldCategories(CustomFieldFixture
                        .modifyCustomFieldCategories(module.getCustomFieldCategories(),
                                scope.getCustomFieldCategories()));                
            }
        }
        
        return module;
    }

    public static ClientModuleScope createFullScope()
    {
        return createScope(true, true, true, true, true);
    }
    
    private static ClientModuleScope createScope(boolean isAttributeCategories,
            boolean isClients, boolean isClientRtfs,
            boolean isCostCenterStages, boolean isCustomFields)
    {
        ClientModuleScope scope = new ClientModuleScope();

        if (isClients)
        {
            scope.setClients(new ClientCostCenterScope());
        }
        if (isAttributeCategories)
        {
            scope.setAttributeCategories(new AttributeScope());
        }
        if (isClientRtfs)
        {
            scope.setClientRtfs(true);
        }
        if (isCostCenterStages)
        {
            scope.setCostCenterStages(true);
        }
        if (isCustomFields)
        {
            scope.setCustomFieldCategories(CustomFieldFixture
                    .createCustomFieldScope(2));
        }
        return scope;
    }

    public static void compare(ClientModule obj1, ClientModule obj2,
            ClientModuleScope scope) throws AssertionFailedError, Exception
    {
        if(scope.getClients()!=null)
        {
            compareClients(obj1.getClients(), obj2.getClients(), scope.getClients());
        }

        if(scope.getAttributeCategories()!=null){
            AttributeFixture.compare(obj1.getAttributeCategories(), obj2
                    .getAttributeCategories(), scope.getAttributeCategories());            
        }

        if(scope.isClientRtfs()){
            DatafieldFixture.compare(obj1.getClientRtfs(), obj2.getClientRtfs());            
        }

        if(scope.isCostCenterStages()){
            DatafieldFixture.compare(obj1.getClientRtfs(), obj2.getClientRtfs());           
        }

        if(scope.getCustomFieldCategories()!=null){
            CustomFieldFixture.assertIdEqual(obj1.getCustomFieldCategories(), obj2
                    .getCustomFieldCategories(), scope.getCustomFieldCategories());            
        }
    }
    
    private static void compareClients(GenericClient[] clients1, GenericClient[] clients2, ClientCostCenterScope clientScope)throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(clients1, clients2);

        if (clients1 != null && clients2 != null)
        {
            for (int i = 0; i < clients1.length; i++)
            {
                compareClient(clients1[i], clients2[i], clientScope);
            }
        }
        
    }
    
    private static void compareClient(GenericClient client1, GenericClient client2, ClientCostCenterScope clientScope)throws AssertionFailedError, Exception
    {
        if(client1!=null)
        {
            if(client1 instanceof ClientFolder)
            {
                FolderFixture.compare((ClientFolder)client1, (ClientFolder)client2, clientScope);
            }
            else if(client1 instanceof Division)
            {
                DivisionFixture.compare((Division)client1, (Division)client2, clientScope);
            }
            else if(client1 instanceof ClientCostCenter)
            {
                ClientCostCenterFixture.compare((ClientCostCenter)client1, (ClientCostCenter)client2, clientScope);
            }
        }
    }
}
