/*
 * Created on Jul 12, 2005
 */
package com.ibm.rpm.client.junit.clientcostcenters.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.containers.Division;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;

/**
 * @author Karim
 */
public class FolderFixture extends RPMObjectFixture
{
	private static final int DEFAULT_NB_LEVELS = 1;
	private static final int DEFAULT_NB_FOLDERS_BY_LEVEL = 1;
	private static final int DEFAULT_NB_DIVISIONS_BY_LEVEL = 1;
	
    public static ClientFolder createSaveFixture(int nbClientCostCenters, ClientTestController ctc) throws Exception {

    	ClientCostCenterScope scope = createScope(1, DEFAULT_NB_LEVELS, DEFAULT_NB_FOLDERS_BY_LEVEL, DEFAULT_NB_DIVISIONS_BY_LEVEL, nbClientCostCenters);
    	ClientFolder folder = createFixture(DEFAULT_NB_FOLDERS_BY_LEVEL, DEFAULT_NB_DIVISIONS_BY_LEVEL, nbClientCostCenters, scope, ctc);

        //Save AssetFolder
    	folder = (ClientFolder) save(ctc, folder, scope);

        return folder;

    }
	
	public static ClientFolder createFixture(int nbFoldersByLevel, int nbDivisionsByLevel, int nbClientCostCentersByLevel, ClientCostCenterScope scope, ClientTestController ctc) throws Exception
    {
    	return createFolder(random(), nbFoldersByLevel, nbDivisionsByLevel, nbClientCostCentersByLevel, scope, ctc);
    }
	
    private static ClientFolder createFolder(int index, int nbFoldersByLevel, int nbDivisionsByLevel, int nbClientCostCentersByLevel, ClientCostCenterScope scope, ClientTestController ctc)throws Exception
	{
    	ClientFolder folder = new ClientFolder();

    	HelperTestUtilities.setByIntrospection(folder, index, skipSetters);

    	if(scope!=null){
    		//create subfolders
        	if(scope.getFolders()!=null){
        		ClientFolder subFolder = null;
        		ClientFolder[] subFolders = new ClientFolder[nbFoldersByLevel]; 
        		for(int i=1;i<=nbFoldersByLevel;i++){
        			subFolder = FolderFixture.createFixture(nbFoldersByLevel, nbDivisionsByLevel, nbClientCostCentersByLevel, scope.getFolders(), ctc);
        			subFolders[i-1] = subFolder;
        		}
        		folder.setFolders(subFolders);
        	}
        	//create divisions
        	if(scope.getDivisions()!=null){
        		Division division = null;
        		Division[] divisions = new Division[nbDivisionsByLevel]; 
        		for(int i=1;i<=nbDivisionsByLevel;i++){
        			division = DivisionFixture.createFixture(nbClientCostCentersByLevel, scope.getDivisions(), ctc);
        			divisions[i-1] = division;
        		}
        		folder.setDivisions(divisions);
        	}
        	//create client cost centers
        	if(scope.getClientCostCenters()!=null){
        		ClientCostCenter clientCostCenter = null;
        		ClientCostCenter[] clientCostCenters = new ClientCostCenter[nbClientCostCentersByLevel]; 
        		for(int i=1;i<=nbClientCostCentersByLevel;i++){
        			clientCostCenter = ClientCostCenterFixture.createFixture(scope.getClientCostCenters(), ctc);
        			clientCostCenters[i-1] = clientCostCenter;
        		}
        		folder.setClientCostCenters(clientCostCenters);
        	}
    	}
    	return folder;
    }
    
    //update fields
    public static void updateFixtureFields(ClientFolder folder, ClientCostCenterScope scope) throws Exception
    {
    	updateFields(folder, random(), scope);
    }
    
    private static void updateFields(ClientFolder folder, int index, ClientCostCenterScope scope) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(folder, index, skipSetters);

    	if(scope!=null){
    		//update subfolders fields
    		if (scope.getFolders() != null) {
    			for(int i=0;i<folder.getFolders().length;i++){
    				FolderFixture.updateFixtureFields(folder.getFolders()[i], scope.getFolders());
    			}
    		}	
    		//update divisions fields
    		if (scope.getDivisions() != null) {
    			for(int i=0;i<folder.getDivisions().length;i++){
    				DivisionFixture.updateFixtureFields(folder.getDivisions()[i], scope.getDivisions());
    			}
    		}
    		//update client cost centers fields
    		if (scope.getClientCostCenters() != null) {
    			for(int i=0;i<folder.getClientCostCenters().length;i++){
    				ClientCostCenterFixture.updateFixture(folder.getClientCostCenters()[i], scope.getClientCostCenters());
    			}
    		}	
    	}

    }
    
    //update structure
    public static void updateFixtureStructure(ClientFolder folder, int deltaNbFoldersByLevel, int deltaNbDivisionsByLevel, int deltaNbClientCostCentersByLevel, ClientCostCenterScope scope, ClientTestController ctc)throws Exception{
    	updateStructure(folder, deltaNbFoldersByLevel, deltaNbDivisionsByLevel, deltaNbClientCostCentersByLevel, scope, ctc);
    }
    
    private static void updateStructure(ClientFolder folder, int deltaNbFoldersByLevel, int deltaNbDivisionsByLevel, int deltaNbClientCostCentersByLevel, ClientCostCenterScope scope, ClientTestController ctc)throws Exception{
    	
    	if(scope!=null){

    		int newNbClientCostCentersByLevel = 0;
    		int newNbDivisionsByLevel = 0;
    		
    		//add new client cost centers
    		if (scope.getClientCostCenters() != null) {
    			if(deltaNbClientCostCentersByLevel>0){//add new elements
    	    		ClientCostCenter newClientCostCenter = null;
    	    		ClientCostCenter[] newClientCostCenters = new ClientCostCenter[deltaNbClientCostCentersByLevel]; 
    		    	for(int i=0;i<deltaNbClientCostCentersByLevel;i++){
    		    		newClientCostCenter = ClientCostCenterFixture.createFixture(scope.getClientCostCenters(), ctc);
    		    		newClientCostCenter.setParent(folder);
    		    		newClientCostCenters[i] = newClientCostCenter;
    		    	}
    		    	
    		    	ArrayList temp = mergeArrays(folder.getClientCostCenters(), newClientCostCenters);
    		    	ClientCostCenter[] totalClientCostCenters = new ClientCostCenter[temp.size()];
    		    	temp.toArray(totalClientCostCenters);
    		    	
    		    	folder.setClientCostCenters(totalClientCostCenters);
    		    	
    		    	newNbClientCostCentersByLevel = totalClientCostCenters.length;
    			}
    		}
    		
    		//add new divisions
    		if (scope.getDivisions() != null) {
    			if(deltaNbDivisionsByLevel>0){//add new elements
    				Division newDivision = null;
    				Division[] newDivisions = new Division[deltaNbDivisionsByLevel]; 
    		    	for(int i=0;i<deltaNbDivisionsByLevel;i++){
    		    		newDivision = DivisionFixture.createFixture(newNbClientCostCentersByLevel, scope.getDivisions(), ctc);
    		    		newDivision.setParent(folder);
    		    		newDivisions[i] = newDivision;
    		    	}
    		    	
    		    	ArrayList temp = mergeArrays(folder.getDivisions(), newDivisions);
    		    	Division[] totalDivisions = new Division[temp.size()];
    		    	temp.toArray(totalDivisions);
    		    	
    		    	folder.setDivisions(totalDivisions);
    		    	
    		    	newNbDivisionsByLevel = totalDivisions.length;
    			}
    		}
    		
    		//add new subfolders
    		if (scope.getFolders() != null) {
    			if(deltaNbFoldersByLevel>0){//add new elements
    	    		ClientFolder newFolder = null;
    	    		ClientFolder[] newFolders = new ClientFolder[deltaNbFoldersByLevel]; 
    		    	for(int i=0;i<deltaNbFoldersByLevel;i++){
    		    		newFolder = FolderFixture.createFixture(deltaNbFoldersByLevel, newNbDivisionsByLevel, newNbClientCostCentersByLevel, scope.getFolders(), ctc);
    		    		newFolder.setParent(folder);
    		    		newFolders[i] = newFolder;
    		    	}
    		    	
    		    	ArrayList temp = mergeArrays(folder.getFolders(), newFolders);
    		    	ClientFolder[] totalFolders = new ClientFolder[temp.size()];
    		    	temp.toArray(totalFolders);
    		    	
    		    	folder.setFolders(totalFolders);
    			}
    		}
    	}
    }
	
	
    public static void compare(ClientFolder f1, ClientFolder f2,
    		ClientCostCenterScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(f1, f2);

        if (scope == null)
            return;

        //validate subfolder
        if (scope.getFolders() != null)
        {
        	ClientCostCenterScope s = scope.getFolders();
            if (s != null)
            {
                s.setParent(null);
            }

            FolderFixture.compare(f1.getFolders(), f2.getFolders(), s);
        }

        //validate divisions
        if (scope.getDivisions() != null)
        {
        	ClientCostCenterScope s = scope.getDivisions();
            if (s != null)
            {
                s.setParent(null);
            }

            DivisionFixture.compare(f1.getDivisions(), f2.getDivisions(), s);
        }
        
        //validate client cost centers
        if (scope.getClientCostCenters() != null)
        {
        	ClientCostCenterScope s = scope.getClientCostCenters();
            if (s != null)
            {
                s.setParent(null);
            }

            ClientCostCenterFixture.compare(f1.getClientCostCenters(), f2.getClientCostCenters(), s);
        }
        
        //validate parents
        if (scope.getParent() != null)
        {
        	ClientCostCenterScope s = (ClientCostCenterScope)scope.getParent();
            if (s != null)
            {
                s.setDivisions(null);
            }
            FolderFixture.compare((ClientFolder) f1.getParent(),
                    (ClientFolder) f2.getParent(), s);
        }
    }

    public static void compare(ClientFolder[] fArray1, ClientFolder[] fArray2,
    		ClientCostCenterScope scope) throws AssertionFailedError, Exception
    {
    	sortWithComparator(fArray1, RPMObjectNameAndIDComparator.INSTANCE);
    	sortWithComparator(fArray2, RPMObjectNameAndIDComparator.INSTANCE); 
    	
        RPMObjectFixture.compare(fArray1, fArray2);
        if(fArray1 == null) return;

        for (int i = 0; i < fArray1.length; i++)
        {
            FolderFixture.compare(fArray1[i], fArray2[i], scope);
        }
    }
    
    public static ClientCostCenterScope createScope(int currentLevel, int maxLevels, int nbFoldersByLevel, int nbDivisionsByLevel, int nbClientCostCentersByLevel){
    	ClientCostCenterScope scope = new ClientCostCenterScope();
    	if(currentLevel<=maxLevels){
    		if(nbFoldersByLevel>0 && currentLevel<maxLevels){
        		//create subfolders scope
        		ClientCostCenterScope subFolderScope = FolderFixture.createScope(currentLevel+1, maxLevels, nbFoldersByLevel, nbDivisionsByLevel, nbClientCostCentersByLevel);
        		subFolderScope.setParent(scope);
        		scope.setFolders(subFolderScope);    			
    		}
    		if(nbDivisionsByLevel>0){
        		//create divisions scope
        		ClientCostCenterScope divisionScope = DivisionFixture.createScope(nbClientCostCentersByLevel);
        		divisionScope.setParent(scope);
        		scope.setDivisions(divisionScope);    			
    		}
    		if(nbClientCostCentersByLevel>0){
        		//create client cost center scope
        		ClientCostCenterScope cccScope = ClientCostCenterFixture.createScope(true, true);
        		cccScope.setParent(scope);
        		scope.setClientCostCenters(cccScope);    			
    		}
    		
    	}
    	return scope;
    }
    
    private static ArrayList mergeArrays(Object[] array1, Object[] array2){
    	ArrayList temp = new ArrayList();
    	addArray(temp, array1);
    	addArray(temp, array2);
    	return temp;
    }
    
   private static void addArray(ArrayList result, Object[] array){
    	if(array!=null && array.length!=0){
    		for(int i=0;i<array.length;i++){
    			result.add(array[i]);
    		}
    	}    	
    }
    
}
