package com.ibm.rpm.client.junit.clientcostcenters.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;


/**
 * @author sailajap
 *
 */
public class TestClientCostCenter extends ClientTestController
{
    private ClientCostCenterScope mainScope;
    
    public TestClientCostCenter(String testName){
        setName(testName);
    }
	
    public void setUp() throws Exception
    {
        logger.info("setUp");
		super.setUp();

		mainScope = ClientCostCenterFixture.createScope(true, true);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError, Exception
	{
    	insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(mainScope, true);
    }
    
    public void testUpdateWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError, Exception
    {
    	update(mainScope, true);
    }
  
//    public void testUpdateDefaultFlag()throws AssertionError, Exception
//    {
//        updateDefaultFlag(null, true);    	
//    }
//    
//    private void updateDefaultFlag(ClientCostCenterScope scope, boolean loadFromId) throws AssertionError, Exception
//	{
//        //insert a client cost center
//        ClientCostCenter  costCenter = insertClientCostCenter(scope, loadFromId);
//    	
//        //update the default flag (to true)
//    	costCenter = setClientCostCenterAsDefault(costCenter, scope, loadFromId);   
//    	
//        //load default test CCC and set its default flag
//    	ClientCostCenter defaultTestClientCostCenter = ClientCostCenterFixture.loadOrCreateDefaultTestClientCostCenter(this);
//    	defaultTestClientCostCenter = setClientCostCenterAsDefault(defaultTestClientCostCenter, scope, loadFromId);
//    	//we don't want the default test CCC to be removed after the end of the tests
//        removeObjectToDelete(defaultTestClientCostCenter);
//    	
//    	//delete the client cost center
//    	deleteClientCostCenter(costCenter, loadFromId);    	
//    }
    
    private void insert(ClientCostCenterScope scope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert a client cost center
        ClientCostCenter  costCenter = insertClientCostCenter(scope, loadFromId);
    	
    	//delete the client cost center
    	deleteClientCostCenter(costCenter, loadFromId);    	
    }
    
    private ClientCostCenter insertClientCostCenter(ClientCostCenterScope scope, boolean loadFromId) throws Exception
    {
    	ClientCostCenter clientCostCenter = ClientCostCenterFixture.createFixture(scope, this);
    	
        //Insert the client cost center into the DB
        SingleResult result =  save(clientCostCenter,scope);
        
        //compare the returned client cost center
        ClientCostCenter returnedClientCostCenter = (ClientCostCenter)result.getRpmObject(); 
        ClientCostCenterFixture.compare(clientCostCenter, returnedClientCostCenter, scope);
        
        //reload the client cost center and compare it
        returnedClientCostCenter = (ClientCostCenter)load(returnedClientCostCenter, scope, loadFromId);
        ClientCostCenterFixture.compare(clientCostCenter, returnedClientCostCenter, scope);
        
        //return the client cost center
        return returnedClientCostCenter;
    }
    
    private void update(ClientCostCenterScope costCenterScope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert a Client Cost Center
    	ClientCostCenter costCenter = insertClientCostCenter(costCenterScope, loadFromId);
    	
        //update the Client Cost Center
    	costCenter = updateClientCostCenter(costCenter, costCenterScope, loadFromId);    	
    	
    	//delete the Client Cost Center
    	deleteClientCostCenter(costCenter, loadFromId); 
    }    
   
    private ClientCostCenter updateClientCostCenter(ClientCostCenter costCenter, ClientCostCenterScope scope, boolean loadFromId) throws Exception
	{    	
    	ClientCostCenterFixture.updateFixture(costCenter,scope);
        //Insert the Client Cost Center into the DB
        SingleResult result =  save(costCenter,scope);
        
        //compare the returned Cost Center
        ClientCostCenter  returnedCostCenter = (ClientCostCenter)result.getRpmObject();
        ClientCostCenterFixture.compare(costCenter, returnedCostCenter, scope);
                
        //reload the Client Cost Center and compare it
        returnedCostCenter = (ClientCostCenter)load(returnedCostCenter, scope, loadFromId);
        ClientCostCenterFixture.compare(costCenter, returnedCostCenter, scope);
        
        //return the Client Cost Center
        return returnedCostCenter;
	}
    
//    private ClientCostCenter setClientCostCenterAsDefault(ClientCostCenter costCenter, ClientCostCenterScope scope, boolean loadFromId) throws Exception
//	{    	
//    	ClientCostCenterFixture.setDefault(costCenter);
//        //Insert the Client Cost Center into the DB
//        SingleResult result =  save(costCenter,scope);
//        
//        //compare the returned Cost Center
//        ClientCostCenter  returnedCostCenter = (ClientCostCenter)result.getRpmObject();
//        ClientCostCenterFixture.compare(costCenter, returnedCostCenter, scope);
//                
//        //reload the Client Cost Center and compare it
//        returnedCostCenter = (ClientCostCenter)load(returnedCostCenter, scope, loadFromId);
//        ClientCostCenterFixture.compare(costCenter, returnedCostCenter, scope);
//        
//        //return the Client Cost Center
//        return returnedCostCenter;
//	}
    
    private void deleteClientCostCenter(ClientCostCenter costCenter, boolean loadFromId)throws Exception
	{    	   	
        //delete the cost center
       	delete(costCenter);
    	
        //reload the deleted cost center
       	ClientCostCenter deletedCostCenter = loadDeletedCostCenter(costCenter, loadFromId);
        
        //Validate the deleted cost center
        assertNull(deletedCostCenter);
    }
    
    private RPMObject load(ClientCostCenter costCenter, ClientCostCenterScope scope, boolean loadFromId) throws Exception
	{
    	if(loadFromId){
            //load from an id
            RPMObject obj = new ClientCostCenter();
            obj.setID(costCenter.getID());

        	
            return loadFromID(obj, scope);
    	} else {
    		//load from an xpath
    		RPMObject[] objs = loadFromXpath("/CLIENTCOSTCENTER[id='"+ costCenter.getID() + "']", scope);
    		if(objs==null || objs.length==0){
    			return null;
    		} else {
    			return objs[0];
    		}
    	}
    }
    
    private ClientCostCenter loadDeletedCostCenter(ClientCostCenter costCenter, boolean loadFromId)throws Exception{
        try{
        	costCenter = (ClientCostCenter)load(costCenter, null, loadFromId);
           	return costCenter;
        }catch (Exception e){
            //Client cost center does not exist
        }
        return null;
    }
    
	
}
