/*
 * Created on Sep 19, 2006
 */
package com.ibm.rpm.client.junit.communications.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.communications.containers.ChecklistItem;
import com.ibm.rpm.communications.containers.GenericResponseType;
import com.ibm.rpm.communications.containers.VoteResponse;
import com.ibm.rpm.framework.RPMObject;

/**
 * @author dominiqu
 */
public class GenericResponseTypeFixture extends RPMObjectFixture
{
    protected static ArrayList ciSkipSetters, ciSkipGetters, vrSkipSetters,
            vrSkipGetters;

    static
    {
        // *** ChecklistItem

        // Setters
        ciSkipSetters = new ArrayList();
        ciSkipSetters.addAll(skipSetters);
        // Read only fields
        ciSkipSetters.add("setDescription");

        // Getters
        ciSkipGetters = new ArrayList();
        ciSkipGetters.addAll(skipGetters);

        // *** VoteResponse

        // Setters
        vrSkipSetters = new ArrayList();
        vrSkipSetters.addAll(skipSetters);
        // Read only fields
        vrSkipSetters.add("setResponse");

        // Getters
        vrSkipGetters = new ArrayList();
        vrSkipGetters.addAll(skipGetters);
    }

    public static void updateFixture(GenericResponseType grt)
            throws AssertionFailedError, Exception
    {
        updateFixture(grt, false);
    }

    public static void updateFixture(GenericResponseType grt, boolean saveOnly)
            throws AssertionFailedError, Exception
    {
        int index = random();

        if (grt instanceof ChecklistItem)
        {
            if (saveOnly)
            {
                // The boolean value can be set however
                HelperTestUtilities.setByIntrospection(grt, index,
                    ciSkipSetters);
            }
            else
            {
                // The boolean value must be true
                ((ChecklistItem) grt).setComplete(Boolean.TRUE);
            }
        }
        else
        {
            for (int i = 0; i < ((VoteResponse) grt).getReasons().length; ++i)
            {
                ((VoteResponse) grt).getReasons()[i].setSelected(Boolean.FALSE);
            }
            // Always select the second reasonm if there is one.
            ((VoteResponse) grt).getReasons()[1 % ((VoteResponse) grt).getReasons().length].setSelected(Boolean.TRUE);
        }
    }

    public static void updateFixture(GenericResponseType[] grt)
            throws AssertionFailedError, Exception
    {
        updateFixture(grt, false);
    }

    public static void updateFixture(GenericResponseType[] grt, boolean saveOnly)
            throws AssertionFailedError, Exception
    {
        if (grt == null)
            return;

        if (grt[0] instanceof ChecklistItem)
        {
            for (int i = 0; i < grt.length; ++i)
            {
                updateFixture(grt[i], saveOnly);
            }
        }
        else
        {
            // Only update one vote response - preferably one with multiple
            // possible responses
            if (((VoteResponse) grt[0]).getReasons().length > 1)
            {
                updateFixture(grt[0], false);
            }
            else
            {
                updateFixture(grt[1], false);
            }
        }
    }

    public static void compare(GenericResponseType grt1,
            GenericResponseType grt2) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(grt1, grt2, skipGetters);
    }

    public static void compare(RPMObject[] eArray1, RPMObject[] eArray2)
            throws AssertionFailedError, Exception
    {
        sortWithComparator(eArray1, new RPMObjectIDComparator());
        sortWithComparator(eArray2, new RPMObjectIDComparator());

        RPMObjectFixture.compare(eArray1, eArray2);
        if (eArray1 == null)
            return;

        for (int i = 0; i < eArray1.length; i++)
        {
            compare(eArray1[i], eArray2[i]);
        }
    }
}
