/*
 * Created on Oct 17, 2006
 */
package com.ibm.rpm.client.junit.communications.testcases;

import com.ibm.rpm.client.junit.communications.fixtures.AbstractTestGenericCommunication;
import com.ibm.rpm.client.junit.communications.fixtures.GenericCommunicationFixture;
import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.interfaces.wrapper.ResourceArray;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author dominiqu
 */
public class TestNotification extends AbstractTestGenericCommunication
{
    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = GenericCommunicationFixture.createFullScope(true);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testSendNotificationWithNoScopeLoadFromID()
            throws AssertionError, Exception
    {
        sendNotification(null, true);
    }

    public void testSendNotificationWithNoScopeLoadFromXPath()
            throws AssertionError, Exception
    {
        sendNotification(null, false);
    }

    public void testSendNotificationWithScopeLoadFromID()
            throws AssertionError, Exception
    {
        sendNotification(mainScope, true);
    }

    public void testSendNotificationWithScopeLoadFromXPath()
            throws AssertionError, Exception
    {
        sendNotification(mainScope, false);
    }

    public void testUpdateReadAndFolderWithScopeLoadFromID()
            throws AssertionError, Exception
    {
        updateReadAndFolder(new Notification(), true);
    }

    public void testUpdateReadAndFolderWithScopeLoadFromXpath()
            throws AssertionError, Exception
    {
        updateReadAndFolder(new Notification(), false);
    }

    private Notification sendNotification(GenericCommunicationScope scope,
            boolean loadFromId) throws Exception
    {
        // Always use mainScope on cration because responses are mandatory for
        // sending
        // Use the provided scope for loading and comparing
        Notification notification = (Notification) GenericCommunicationFixture.createFixture(
            null, new Notification(), mainScope, this);

        // Insert the notification
        Resource[] resource = new Resource[notification.getResponses().length];
        for (int i = 0; i < resource.length; ++i)
        {
            if (notification.getResponses()[i] != null)
            {
                resource[i] = notification.getResponses()[i].getResource();
            }
        }
        SingleResult result = sendNotification(scope,
            notification.getWorkflowAttachment().getAttachment(), resource,
            notification.getSubject(), notification.getComments());

        // Compare the returned container
        Notification returnedNotification = (Notification) result.getRpmObject();
        GenericCommunicationFixture.compare(notification, returnedNotification,
            scope);
        GenericCommunicationFixture.validateReadOnlyValues(returnedNotification);
        GenericCommunicationFixture.validateFolderAndRead(returnedNotification,
            new Resource[]
            {
                notification.getAuthor()
            }, CommunicationsFolder.Sent, Boolean.TRUE);
        GenericCommunicationFixture.validateFolderAndRead(returnedNotification,
            resource, CommunicationsFolder.Inbox, Boolean.FALSE);
        
        // Reload the container and compare it
        returnedNotification = (Notification) load(returnedNotification, scope,
            loadFromId);
        GenericCommunicationFixture.compare(notification, returnedNotification,
            scope);
        GenericCommunicationFixture.validateReadOnlyValues(returnedNotification);
        GenericCommunicationFixture.validateFolderAndRead(returnedNotification,
            new Resource[]
            {
                notification.getAuthor()
            }, CommunicationsFolder.Sent, Boolean.TRUE);
        GenericCommunicationFixture.validateFolderAndRead(returnedNotification,
            resource, CommunicationsFolder.Inbox, Boolean.FALSE);
        
        return returnedNotification;
    }

    private SingleResult sendNotification(GenericCommunicationScope scope,
            RPMObject attachment, Resource[] recipients, String subject,
            String comments) throws Exception
    {
        ResourceArray resourceArray = new ResourceArray();
        resourceArray.setResourceArray(recipients);
        return getApplicationInterface().sendNotification(sessionid, scope,
            attachment, resourceArray, subject, comments);
    }

}
