/*
 * Created on Aug 22, 2006
 */
package com.ibm.rpm.client.junit.communications.testcases;

import java.util.Calendar;

import com.ibm.rpm.client.junit.communications.fixtures.AbstractTestGenericCommunication;
import com.ibm.rpm.client.junit.communications.fixtures.GenericCommunicationFixture;
import com.ibm.rpm.client.junit.communications.fixtures.WorkflowResponseFixture;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.interfaces.wrapper.ResourceArray;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author dominiqu
 */
public class TestStaffingRequest extends AbstractTestGenericCommunication
{
    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = GenericCommunicationFixture.createFullScope(true);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testSendStaffingRequestWithNoScopeLoadFromID()
            throws AssertionError, Exception
    {
        sendStaffingRequest(null, true);
    }

    public void testSendStaffingRequestWithNoScopeLoadFromXPath()
            throws AssertionError, Exception
    {
        sendStaffingRequest(null, false);
    }

    public void testSendStaffingRequestWithScopeLoadFromID()
            throws AssertionError, Exception
    {
        sendStaffingRequest(mainScope, true);
    }

    public void testSendStaffingRequestWithScopeLoadFromXPath()
            throws AssertionError, Exception
    {
        sendStaffingRequest(mainScope, false);
    }

    public void testSendResponseWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        sendResponse(mainScope, true);
    }

    public void testSendResponseWithScopeLoadFromXpath() throws AssertionError,
            Exception
    {
        sendResponse(mainScope, false);
    }

    public void testUpdateReadAndFolderWithScopeLoadFromID()
            throws AssertionError, Exception
    {
        updateReadAndFolder(new StaffingRequest(), true);
    }

    public void testUpdateReadAndFolderWithScopeLoadFromXpath()
            throws AssertionError, Exception
    {
        updateReadAndFolder(new StaffingRequest(), false);
    }

    public void testDelete() throws AssertionError, Exception
    {
        StaffingRequest sr = sendStaffingRequest(mainScope, false);

        StaffingRequest srToDelete = new StaffingRequest();
        srToDelete.setID(sr.getID());

        Result result = delete(srToDelete);
        assertTrue(result == null || result.getErrors() == null);

        // Find the response belonging to the Administrator
        WorkflowResponse adminResponse = null;
        Resource resource = null;
        for (int i = 0; i < sr.getResponses().length; ++i)
        {
            if (sr.getResponses()[i] != null
                    && sr.getResponses()[i].getVisible() == Boolean.FALSE)
            {
                // This is the WorkflowResponse of the person that sent the
                // StaffingRequest
                adminResponse = sr.getResponses()[i];
                resource = sr.getResponses()[(i + 1)
                        % (sr.getResponses().length)].getResource();
            }
        }
        assertNotNull("The sent workflow response was not loaded.",
            adminResponse);

        // Pick another resource to log onto web services

        RPMObject obj = null;
        try
        {
            obj = load(adminResponse, null, true);
        }
        catch (RPMException e)
        {
            assertTrue(e.getErrorID() == ErrorCodes.AM_INVALID_RPMOBJECTID);
        }
        assertNull(obj);

        // Give one of the other resources the right to log into web services
        resource.getResourceSecurityRights().setCanUseWebServicesAPI(
            Boolean.TRUE);
        resource = (Resource) save(resource, null).getRpmObject();
        changePassword(resource.getUserName(), "ibmrpm", "rpm");

        // Now, test that the Staffing Request can still be loaded by other
        // resources
        getAuthenticateInterface().logout(sessionid);

        performLogin(resource.getUserName(), "rpm");

        StaffingRequest obj2 = (StaffingRequest) load(sr, mainScope, true);
        assertNotNull(obj2);
        for (int i = 0; i < obj2.getResponses().length; ++i)
        {
            if (obj2.getResponses()[i].getVisible() == Boolean.FALSE)
            {
                // This is the WorkflowResponse of the person that sent the
                // StaffingRequest
                assertTrue(obj2.getResponses()[i].getFolder() == CommunicationsFolder.NotVisible);
                break;
            }
        }

        // Log back in as administrator, for teardown
        performLogin();
    }

    private StaffingRequest sendStaffingRequest(
            GenericCommunicationScope scope, boolean loadFromId)
            throws Exception
    {
        // Always use mainScope on cration because responses are mandatory for
        // sending
        // Use the provided scope for loading and comparing
        StaffingRequest staffingRequest = (StaffingRequest) GenericCommunicationFixture.createFixture(
            null, new StaffingRequest(), mainScope, this);

        // Insert the staffing request
        Resource[] resource = new Resource[staffingRequest.getResponses().length];
        for (int i = 0; i < resource.length; ++i)
        {
            if (staffingRequest.getResponses()[i] != null)
            {
                resource[i] = staffingRequest.getResponses()[i].getResource();
            }
        }
        SingleResult result = sendStaffingRequest(scope,
            staffingRequest.getWorkflowAttachment().getAttachment(), resource,
            staffingRequest.getSubject(), staffingRequest.getComments(),
            staffingRequest.getDueDate());

        // Compare the returned container
        StaffingRequest returnedStaffingRequest = (StaffingRequest) result.getRpmObject();
        GenericCommunicationFixture.compare(staffingRequest,
            returnedStaffingRequest, scope);
        GenericCommunicationFixture.validateReadOnlyValues(returnedStaffingRequest);
        GenericCommunicationFixture.validateFolderAndRead(returnedStaffingRequest,
            new Resource[]
            {
                staffingRequest.getAuthor()
            }, CommunicationsFolder.Sent, Boolean.TRUE);
        GenericCommunicationFixture.validateFolderAndRead(returnedStaffingRequest,
            resource, CommunicationsFolder.Inbox, Boolean.FALSE);

        // Reload the container and compare it
        returnedStaffingRequest = (StaffingRequest) load(
            returnedStaffingRequest, scope, loadFromId);
        GenericCommunicationFixture.compare(staffingRequest,
            returnedStaffingRequest, scope);
        GenericCommunicationFixture.validateReadOnlyValues(returnedStaffingRequest);
        GenericCommunicationFixture.validateFolderAndRead(returnedStaffingRequest,
            new Resource[]
            {
                staffingRequest.getAuthor()
            }, CommunicationsFolder.Sent, Boolean.TRUE);
        GenericCommunicationFixture.validateFolderAndRead(returnedStaffingRequest,
            resource, CommunicationsFolder.Inbox, Boolean.FALSE);
        
        return returnedStaffingRequest;
    }

    private SingleResult sendStaffingRequest(GenericCommunicationScope scope,
            RPMObject attachment, Resource[] recipients, String subject,
            String comments, Calendar dueDate) throws Exception
    {
        ResourceArray resourceArray = new ResourceArray();
        resourceArray.setResourceArray(recipients);
        return getApplicationInterface().sendStaffingRequest(sessionid, scope,
            attachment, resourceArray, subject, comments, dueDate);
    }

    private StaffingRequest sendResponse(GenericCommunicationScope scope,
            boolean loadFromId) throws Exception
    {
        if (scope.getResponses() == null)
            return null;

        StaffingRequest staffingRequest = (StaffingRequest) GenericCommunicationFixture.createSaveFixture(
            null, new StaffingRequest(), mainScope, this);

        for (int i = 0; i < staffingRequest.getResponses().length; ++i)
        {
            if (staffingRequest.getResponses()[i].getVisible() == Boolean.TRUE)
            {
                staffingRequest.getResponses()[i] = WorkflowResponseFixture.updateFixture(staffingRequest.getResponses()[i]);

                // Send the response to the staffing request
                SingleResult result = sendWorkflowResponse(
                    staffingRequest.getResponses()[i], scope.getResponses());

                // Compare the returned container
                WorkflowResponse returnedWorkflowResponse = (WorkflowResponse) result.getRpmObject();
                WorkflowResponseFixture.compare(
                    staffingRequest.getResponses()[i],
                    returnedWorkflowResponse, scope.getResponses());
                WorkflowResponseFixture.validateReadOnlyValues(returnedWorkflowResponse);
                
                // Reload the container and compare it
                returnedWorkflowResponse = (WorkflowResponse) load(
                    returnedWorkflowResponse, scope.getResponses(), loadFromId);
                WorkflowResponseFixture.compare(
                    staffingRequest.getResponses()[i],
                    returnedWorkflowResponse, scope.getResponses());
                WorkflowResponseFixture.validateReadOnlyValues(returnedWorkflowResponse);

                staffingRequest.getResponses()[i] = returnedWorkflowResponse;
            }
        }

        return staffingRequest;
    }
}
