/*
 * Created on Feb 21, 2006
 *
 */
package com.ibm.rpm.client.junit.contract.fixtures;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.AbstractModuleFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.containers.ContractFolder;
import com.ibm.rpm.contract.containers.ContractModule;
import com.ibm.rpm.contract.containers.GenericContract;
import com.ibm.rpm.contract.scope.ContractModuleScope;
import com.ibm.rpm.contract.scope.ContractScope;

/**
 * @author karim
 *
 */
public class ContractModuleFixture extends AbstractModuleFixture
{
    public static ContractModule createFixture(
            ContractModuleScope scope, ClientTestController ctc) throws Exception
    {
        return createModule( scope, ctc);
    }

    public static ContractModule updateFixture(ContractModule module,
            ContractModuleScope scope, ClientTestController ctc) throws Exception
    {
        return updateModule(module, scope, ctc);
    }

    private static ContractModule createModule(
            ContractModuleScope scope, ClientTestController ctc) throws Exception
    {
        ContractModule module = new ContractModule();
        
        if(scope!=null){
            if (scope.getContracts() != null)
            {
                GenericContract[] genericContracts = (GenericContract[])createElements(
                        scope.getContracts(), GenericContract.class, ctc);
                module.setContracts(genericContracts);
            }
        }
    	                                           	
        return module;
    }

    private static ContractModule updateModule(ContractModule module,
            ContractModuleScope scope, ClientTestController ctc) throws Exception
    {
        if (scope != null)
        {
            if (scope.getContracts() != null)
            {
                GenericContract[] genericContracts = (GenericContract[])updateElements(module.getContracts(),
                        scope.getContracts(), ctc);
                module.setContracts(genericContracts);
            }
        }
        return module;
    }

    public static ContractModuleScope createFullScope()
    {
        return createScope(true,true);
    }
    
    public static ContractModuleScope createScope(boolean isContracts, boolean isContractTypes)
    {
        ContractModuleScope scope = new ContractModuleScope();

        if(isContracts){
            scope.setContracts(new ContractScope());
        }
        if(isContractTypes){
            scope.setContractTypes(isContractTypes);
        }
        return scope;
    }

    public static void compare(ContractModule obj1, ContractModule obj2,
            ContractModuleScope scope) throws AssertionFailedError, Exception
    {
        if (scope != null)
        {
            if (scope.getContracts() != null)
            {
                compareContracts(obj1.getContracts(), obj2
                        .getContracts(), scope.getContracts());
            }
            if (scope.isContractTypes())
            {
                DatafieldFixture.compare(obj1.getContractTypes(), obj2
                        .getContractTypes());
            }
        }
    }
    
    private static void compareContracts(GenericContract[] contracts1, GenericContract[] contracts2, ContractScope contractScope)throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(contracts1, contracts2);

        if (contracts1 != null && contracts2 != null)
        {
            for (int i = 0; i < contracts1.length; i++)
            {
                compareContract(contracts1[i], contracts2[i], contractScope);
            }
        }
        
    }
    
    private static void compareContract(GenericContract contract1, GenericContract contract2, ContractScope contractScope)throws AssertionFailedError, Exception
    {
        if(contract1!=null)
        {
            if(contract1 instanceof ContractFolder)
            {
                ContractFolderFixture.compare((ContractFolder)contract1, (ContractFolder)contract2, contractScope);
            }
            else if(contract1 instanceof Contract)
            {
                ContractFixture.compare((Contract)contract1, (Contract)contract2, contractScope);
            }
        }
    }
}
