package com.ibm.rpm.client.junit.contract.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.contract.fixtures.ContractFixture;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.scope.ContractScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author shiv
 *  
 */

public class TestContract extends ClientTestController 
{
	private ContractScope mainScope;
	
	public TestContract(String testName) {
		setName(testName);
	}
	
	public void setUp() throws Exception {
		logger.info("setUp");
		super.setUp();
		mainScope = ContractFixture.createScope();
	}
	
	public void tearDown() throws Exception {
		super.tearDown();
	}
	
	public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
	Exception {
		insert(null, false);
	}
	
	public void testInsertWithNoScopeLoadFromID() throws AssertionError,
	Exception {
		
		insert(null, true);
	}
	
	public void testInsertWithScopeLoadFromID() throws AssertionError,
	Exception {
		insert(mainScope, true);
	}
	
	public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
	Exception {
		update(null, true);
	}
	
	public void testUpdateWithScopeLoadFromID() throws AssertionError,
	Exception {
		update(mainScope, true);
	}
	
	private void insert(ContractScope scope, boolean loadFromId)
	throws AssertionError, Exception {
		//insert a contract
		Contract contract = insertContract(scope, loadFromId);
		//delete the contract
		deleteContract(contract, loadFromId);
	}
	
	private Contract insertContract(ContractScope scope, boolean loadFromId)
	throws Exception {
		Contract contract = ContractFixture.createFixture(scope, this);
		SingleResult result = save(contract, scope);
		Contract returnedContract = (Contract) result.getRpmObject();
		ContractFixture.compare(contract, returnedContract, scope);
		
		//reload the contract and compare it
		returnedContract = (Contract) load(returnedContract, scope, loadFromId);
		ContractFixture.compare(contract, returnedContract, scope);
		
		//return the contract
		return returnedContract;
	}
	
	private void update(ContractScope contractScope, boolean loadFromId)
	throws AssertionError, Exception {
		//insert a contract
		Contract contract = insertContract(contractScope, loadFromId);
		
		//update the contract
		contract = updateContract(contract, contractScope, loadFromId);
		
		//delete the contract
		deleteContract(contract, loadFromId);
	}
	
	private Contract updateContract(Contract contract, ContractScope scope,
			boolean loadFromId) throws Exception {
		ContractFixture.updateFixture(contract, scope);
		//Insert the contract into the DB
		SingleResult result = save(contract, scope);
		
		//compare the returned contract
		Contract returnedContract = (Contract) result.getRpmObject();
		ContractFixture.compare(contract, returnedContract, scope);
		
		//reload the contract and compare it
		returnedContract = (Contract) load(returnedContract, scope, loadFromId);
		ContractFixture.compare(contract, returnedContract, scope);
		
		//return the contract
		return returnedContract;
	}
	
	private void deleteContract(Contract contract, boolean loadFromId)
	throws Exception {
		//delete the contract
		delete(contract);
		
		//reload the deleted contract
		Contract deletedContract = loadDeletedContract(contract, loadFromId);
		
		//Validate the deleted contract
		assertNull(deletedContract);
	}
	
	private RPMObject load(Contract contract, ContractScope scope,
			boolean loadFromId) throws Exception {
		if (loadFromId) {
			//load from an id
			RPMObject obj = new Contract();
			obj.setID(contract.getID());
			
			return loadFromID(obj, scope);
		} else {
			//load from an xpath
			RPMObject[] objs = loadFromXpath("/CONTRACT[id='"
					+ contract.getID() + "']", scope);
			if (objs == null || objs.length == 0) {
				return null;
			} else {
				return objs[0];
			}
		}
	}
	
	private Contract loadDeletedContract(Contract contract, boolean loadFromId)
	throws Exception {
		try {
			contract = (Contract) load(contract, null, loadFromId);
			return contract;
		} catch (Exception e) {
			//contract does not exist
		}
		return null;
	}
	
}

