package com.ibm.rpm.client.junit.contract.testcases;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.contract.fixtures.ContractChargeScheduleFixture;
import com.ibm.rpm.client.junit.contract.fixtures.ContractFixture;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.containers.ContractChargeSchedule;
import com.ibm.rpm.contract.scope.ContractScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author Shiv
 */
public class TestContractChargeSchedule extends ClientTestController
{
	private ContractScope mainScope;
	
	public TestContractChargeSchedule(String testName) {
		setName(testName);
	}
	
	public void setUp() throws Exception {
		logger.info("setUp");
		super.setUp();
		mainScope = ContractChargeScheduleFixture.createScope();
	}
	
	public void tearDown() throws Exception {
		super.tearDown();
	}
	
	public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
	Exception {
		insert(null, false);
	}
	
	public void testInsertWithNoScopeLoadFromID() throws AssertionError,
	Exception {
		
		insert(null, true);
	}
	
	public void testInsertWithScopeLoadFromID() throws AssertionError,
	Exception {
		insert(mainScope, true);
	}
	
	public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
	Exception {
		update(null, true);
	}
	
	public void testUpdateWithScopeLoadFromID() throws AssertionError,
	Exception {
		update(mainScope, true);
	}
	
	private void insert(ContractScope scope, boolean loadFromId)
	throws AssertionError, Exception {
		//insert a contract
		ContractChargeSchedule contractchargeschedule = insertContractChargeSchedule(scope, loadFromId);
		//delete the contract
		deleteContractChargeSchedule(contractchargeschedule, loadFromId);
	}
	
	private ContractChargeSchedule insertContractChargeSchedule(ContractScope scope, boolean loadFromId)
	throws Exception {
		ContractChargeSchedule contractChargeSchedule = ContractChargeScheduleFixture.createFixture(scope, this);
		
		Contract contract = ContractFixture.createSaveFixture(null,this);
		//ContractFolder contractfolder = ContractFolderFixture.createSaveFixture(1,ctc);
		contractChargeSchedule.setParent(contract);
		Calendar c = contractChargeSchedule.getScheduleDate();
		c.set(2005,3,1);
		contractChargeSchedule.setScheduleDate(c);
		
		SingleResult result = save(contractChargeSchedule, scope);
		ContractChargeSchedule returnedContractChargeSchedule = (ContractChargeSchedule) result.getRpmObject();
		ContractChargeScheduleFixture.compare(contractChargeSchedule, returnedContractChargeSchedule, scope);
		
		//reload the contract and compare it
		returnedContractChargeSchedule = (ContractChargeSchedule) load(returnedContractChargeSchedule, scope, loadFromId);
		ContractChargeScheduleFixture.compare(contractChargeSchedule, returnedContractChargeSchedule, scope);
		
		//return the contract
		return returnedContractChargeSchedule;
	}
	
	private void update(ContractScope contractScope, boolean loadFromId)
	throws AssertionError, Exception {
		//insert a contract
		ContractChargeSchedule contractChargeSchedule = insertContractChargeSchedule(contractScope, loadFromId);
		
		//update the contract
		contractChargeSchedule = updateContractChargeSchedule(contractChargeSchedule, contractScope, loadFromId);
		
		//delete the contract
		deleteContractChargeSchedule(contractChargeSchedule, loadFromId);
	}
	
	private ContractChargeSchedule updateContractChargeSchedule(ContractChargeSchedule contractChargeSchedue, ContractScope scope,
			boolean loadFromId) throws Exception {
		ContractChargeScheduleFixture.updateFixture(contractChargeSchedue, scope);
		//Insert the contract into the DB
		SingleResult result = save(contractChargeSchedue, scope);
		
		//compare the returned contract
		ContractChargeSchedule returnedContractChargeSchedule = (ContractChargeSchedule) result.getRpmObject();
		ContractChargeScheduleFixture.compare(contractChargeSchedue, returnedContractChargeSchedule, scope);
		
		//reload the contract and compare it
		returnedContractChargeSchedule = (ContractChargeSchedule) load(returnedContractChargeSchedule, scope, loadFromId);
		ContractChargeScheduleFixture.compare(contractChargeSchedue, returnedContractChargeSchedule, scope);
		
		//return the contract
		return returnedContractChargeSchedule;
	}
	
	private void deleteContractChargeSchedule(ContractChargeSchedule contractChargeSchedule, boolean loadFromId)
	throws Exception {
		//delete the contract
		delete(contractChargeSchedule);
		//reload the deleted contract
		ContractChargeSchedule deletedContractChargeSchedule = loadDeletedContractChargeSchedule(contractChargeSchedule, loadFromId);
		//Validate the deleted contract
		assertNull(deletedContractChargeSchedule);
	}
	
	private RPMObject load(ContractChargeSchedule contractChargeSchedule, ContractScope scope,
			boolean loadFromId) throws Exception {
		if (loadFromId) {
			//load from an id
			RPMObject obj = new ContractChargeSchedule();
			obj.setID(contractChargeSchedule.getID());
			return loadFromID(obj, scope);
		} else {
			//load from an xpath
			RPMObject[] objs = loadFromXpath("/CONTRACTCHARGESCHEDULE[id='"
					+ contractChargeSchedule.getID() + "']", scope);
			if (objs == null || objs.length == 0) {
				return null;
			} else {
				return objs[0];
			}
		}
	}
	
	private ContractChargeSchedule loadDeletedContractChargeSchedule(ContractChargeSchedule contractChargeSchedule, boolean loadFromId)
	throws Exception {
		try {
			contractChargeSchedule = (ContractChargeSchedule) load(contractChargeSchedule, null, loadFromId);
			return contractChargeSchedule;
		} catch (Exception e) {
			//ContractChargeSchedule does not exist
		}
		return null;
	}
	
}



