package com.ibm.rpm.client.junit.contract.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.contract.fixtures.ContractFolderFixture;
import com.ibm.rpm.contract.containers.ContractFolder;
import com.ibm.rpm.contract.scope.ContractScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author shiv
 */
public class TestContractFolder extends ClientTestController
{
	
	private ContractScope mainScope;
	
	//TODO: Confirm these values by couple of runs
	private static final int NB_MAX_LEVELS = 2;
	
	private static final int NB_FOLDERS_BY_LEVEL = 1;
	
	private static final int NB_CONTRACTS_BY_LEVEL = 1;
	
	private static final int DELTA_FOLDERS_BY_LEVEL = 1;
	
	private static final int DELTA_CONTRACTS_BY_LEVEL = 1;
	
	public TestContractFolder(String testName) {
		
		setName(testName);
		
	}
	
	public void setUp() throws Exception {
		logger.info("setUp");
		super.setUp();
		mainScope = ContractFolderFixture.createScope(1, NB_MAX_LEVELS,
				NB_FOLDERS_BY_LEVEL, NB_CONTRACTS_BY_LEVEL);
	}
	
	public void tearDown() throws Exception {
		super.tearDown();
	}
	
	public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
	Exception {
		insert(null, false);
	}
	
	public void testInsertWithNoScopeLoadFromID() throws AssertionError,
	Exception {
		insert(null, true);
	}
	
	public void testInsertWithScopeLoadFromID() throws AssertionError,
	Exception {
		insert(mainScope, true);
	}
	
	public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
	Exception {
		update(null, true);
	}
	
	public void testUpdateWithScopeLoadFromID() throws AssertionError,
	Exception {
		update(mainScope, true);
	}
	
	private void insert(ContractScope scope, boolean loadFromId)
	throws AssertionError, Exception {
		//insert a AssetFolder
		ContractFolder folder = insertFolder(scope, loadFromId);
		//delete the Division
		deleteFolder(folder, loadFromId);
	}
	
	private ContractFolder insertFolder(ContractScope scope, boolean loadFromId)
	throws Exception {
		ContractFolder folder = ContractFolderFixture.createFixture(
				NB_FOLDERS_BY_LEVEL, NB_CONTRACTS_BY_LEVEL, scope, this);
		//Insert the Folder into the DB
		SingleResult result = save(folder, scope);
		
		//compare the returned folder
		ContractFolder returnedFolder = (ContractFolder) result.getRpmObject();
		ContractFolderFixture.compare(folder, returnedFolder, scope);
		
		//reload the Folder and compare it
		returnedFolder = (ContractFolder) load(returnedFolder, scope,
				loadFromId);
		ContractFolderFixture.compare(folder, returnedFolder, scope);
		
		//return the Folder
		return returnedFolder;
	}
	
	private void update(ContractScope contractScope, boolean loadFromId)
	throws AssertionError, Exception {
		//insert a Folder
		ContractFolder folder = insertFolder(contractScope, loadFromId);
		
		//update the folder
		folder = updateFolderFields(folder, contractScope, loadFromId);
		folder = updateFolderStructure(folder, contractScope, loadFromId);
		
		//delete the folder
		deleteFolder(folder, loadFromId);
	}
	
	private ContractFolder updateFolderFields(ContractFolder folder,
			ContractScope scope, boolean loadFromId) throws Exception {
		return updateFolder(folder, scope, loadFromId, false);
	}
	
	private ContractFolder updateFolderStructure(ContractFolder folder,
			ContractScope scope, boolean loadFromId) throws Exception {
		return updateFolder(folder, scope, loadFromId, true);
	}
	
	private ContractFolder updateFolder(ContractFolder folder,
			ContractScope scope, boolean loadFromId, boolean updateStructure)
	throws Exception {
		if (updateStructure) {
			//updates the structure by adding elements
			ContractFolderFixture.updateFixtureStructure(folder,
					DELTA_FOLDERS_BY_LEVEL, DELTA_CONTRACTS_BY_LEVEL, scope,
					this);
		} else {
			//only updates the fields
			ContractFolderFixture.updateFixtureFields(folder, scope);
		}
		
		//Insert the updated folder into the DB
		SingleResult result = save(folder, scope);
		
		//compare the returned folder
		ContractFolder returnedFolder = (ContractFolder) result.getRpmObject();
		ContractFolderFixture.compare(folder, returnedFolder, scope);
		
		//reload the folder and compare it
		returnedFolder = (ContractFolder) load(returnedFolder, scope,
				loadFromId);
		ContractFolderFixture.compare(folder, returnedFolder, scope);
		
		//return the folder
		return returnedFolder;
	}
	
	private void deleteFolder(ContractFolder folder, boolean loadFromId)
	throws Exception {
		//delete the Folder
		delete(folder);
		
		//reload the deleted Division
		ContractFolder deletedFolder = loadDeletedFolder(folder, loadFromId);
		
		//Validate the deleted Division
		assertNull(deletedFolder);
	}
	
	private ContractFolder loadDeletedFolder(ContractFolder folder,
			boolean loadFromId) throws Exception {
		try {
			folder = (ContractFolder) load(folder, null, loadFromId);
			return folder;
		} catch (Exception e) {
			// Folder does not exist
		}
		return null;
	}
	
	private RPMObject load(ContractFolder folder, ContractScope scope,
			boolean loadFromId) throws Exception {
		if (loadFromId) {
			//load from an id
			RPMObject obj = new ContractFolder();
			obj.setID(folder.getID());
			return loadFromID(obj, scope);
		} else {
			//load from an xpath
			RPMObject[] objs = loadFromXpath("/CONTRACTFOLDER[id='"
					+ folder.getID() + "']", scope);
			if (objs == null || objs.length == 0) {
				return null;
			} else {
				return objs[0];
			}
		}
	}
	
}

