package com.ibm.rpm.client.junit.customfield.fixtures;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.CustomDatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author ikusakov
 * 
 */
public class CustomFieldAssignmentFixture extends RPMObjectFixture
{
    // =============
    // CONTEXT NAMES
    // =============

    public static final String ASSET_CONTEXT = "Asset";

    public static final String CLIENT_CONTEXT = "Client";

    public static final String DOCUMENT_CONTEXT = "Document";

    public static final String WBS_CONTEXT = "Wbs";

    public static final String SCOPE_CONTEXT = "Scope";

    public static final String RESOURCE_CONTEXT = "Resource";

    public static CustomFieldAssignment createSaveCustomFieldAssignment(
            String contextName, ClientTestController ctc) throws Exception
    {
        CustomFieldAssignment cfa = createCustomFieldAssignment(contextName,
            FieldInputType.Text, ctc);
        ctc.save(cfa, null);
        return cfa;
    }

    public static CustomFieldAssignment createSaveCustomFieldAssignment(
            String contextName, FieldInputType inputType,
            ClientTestController ctc) throws Exception
    {
        CustomFieldAssignment cfa = createCustomFieldAssignment(contextName,
            inputType, ctc);
        ctc.save(cfa, null);
        return cfa;
    }

    public static RPMObject createParent(String contextName,
            ClientTestController ctc) throws Exception
    {
        // Note a static fixture cannot be used as a parent
        RPMObject parent = null;
        if (CustomFieldAssignmentFixture.RESOURCE_CONTEXT.equals(contextName))
        {
            // parent = new Resource();
            // parent.setID(RESOURCE_ID);
            ResourceScope resourceScope = new ResourceScope();
            resourceScope.setCustomFieldAssignments(true);
            parent = ResourceFixture.createSaveFixture(resourceScope, ctc);
        }
        else if (CustomFieldAssignmentFixture.ASSET_CONTEXT.equals(contextName))
        {
            AssetScope as = new AssetScope();
            as.setCustomFieldAssignments(true);
            parent = AssetFixture.createSaveFixture(as, ctc);
        }
        else if (CustomFieldAssignmentFixture.WBS_CONTEXT.equals(contextName))
        {
            WorkElementScope scope = new WorkElementScope();
            scope.setCustomFieldAssignments(true);

            parent = WorkElementFixture.createSaveFixture(new Proposal(), null,
                ctc);
            parent = ctc.loadFromID(parent, scope);
        }
        else if (CustomFieldAssignmentFixture.CLIENT_CONTEXT.equals(contextName))
        {
            ClientCostCenterScope cccs = new ClientCostCenterScope();
            cccs.setCustomFieldAssignments(true);
            parent = ClientCostCenterFixture.createSaveFixture(cccs, ctc);
        }
        else if (CustomFieldAssignmentFixture.DOCUMENT_CONTEXT.equals(contextName))
        {
            WorkElement workElement = WorkElementFixture.retrieveStaticFixture(
                new Proposal(), ctc);

            WorkElementScope workElementScope = new WorkElementScope();
            workElementScope.setDocumentFolder(new DocumentScope());
            workElement = (WorkElement) ctc.loadFromID(workElement,
                workElementScope);

            DocumentFolder parentDocumentFolder = workElement.getDocumentFolder();

            DocumentElement documentElement = DocumentElementFixture.createSaveFixture(
                parentDocumentFolder, new Document(), 0, 0, ctc);

            DocumentScope documentScope = new DocumentScope();
            documentScope.setCustomFieldAssignments(true);

            DocumentElement loadedDocumentElement = (DocumentElement) ctc.loadFromID(
                documentElement, documentScope);

            parent = loadedDocumentElement;
        }
        else if (CustomFieldAssignmentFixture.SCOPE_CONTEXT.equals(contextName))
        {
            ScopeElementScope sec = new ScopeElementScope();
            sec.setCustomFieldAssignments(true);

            parent = ScopeElementFixture.createSaveScopeElement(ctc);
            parent = ctc.loadFromID(parent, sec);

        }
        else
        {
            throw new Exception("Unknown context: " + contextName);
        }

        return parent;
    }

    public static CustomField createCustomField(String contextName,
            FieldInputType inputType, ClientTestController ctc)
            throws Exception
    {
        CustomField cf = null;
        if (CustomFieldAssignmentFixture.RESOURCE_CONTEXT.equals(contextName))
        {
            cf = CustomFieldFixture.createSaveCustomField(
                CustomFieldFixture.CF_RESOURCE_CONTEXT, inputType, ctc);
        }
        else if (CustomFieldAssignmentFixture.ASSET_CONTEXT.equals(contextName))
        {
            cf = CustomFieldFixture.createSaveCustomField(
                CustomFieldFixture.CF_ASSET_CONTEXT, inputType, ctc);
        }
        else if (CustomFieldAssignmentFixture.WBS_CONTEXT.equals(contextName))
        {
            cf = CustomFieldFixture.createSaveCustomField(
                CustomFieldFixture.CF_WBS_SCOPE_CONTEXT, inputType, ctc);
        }
        else if (CustomFieldAssignmentFixture.CLIENT_CONTEXT.equals(contextName))
        {
            cf = CustomFieldFixture.createSaveCustomField(
                CustomFieldFixture.CF_CLIENT_CONTEXT, inputType, ctc);
        }
        else if (CustomFieldAssignmentFixture.DOCUMENT_CONTEXT.equals(contextName))
        {
            cf = CustomFieldFixture.createSaveCustomField(
                CustomFieldFixture.CF_DOCUMENT_CONTEXT, inputType, ctc);
        }
        else if (CustomFieldAssignmentFixture.SCOPE_CONTEXT.equals(contextName))
        {
            cf = CustomFieldFixture.createSaveCustomField(
                CustomFieldFixture.CF_WBS_SCOPE_CONTEXT, inputType, ctc);
        }
        else
        {
            throw new Exception("Unknown context: " + contextName);
        }

        return cf;
    }

    public static CustomFieldAssignment createCustomFieldAssignment(
            String contextName, FieldInputType inputType,
            ClientTestController ctc) throws Exception
    {
        CustomField cf = createCustomField(contextName, inputType, ctc);
        RPMObject parent = createParent(contextName, ctc);

        CustomFieldAssignment assignment = new CustomFieldAssignment();
        assignment.setCustomField(cf);

        // Set the datafieldCategory value
        DatafieldCategory dfCategory = cf.getDatafieldCategory();
        if (dfCategory != null && dfCategory instanceof CustomDatafieldCategory)
        {
            assignment.setValue(dfCategory.getChildren()[1]);
        }

        assignment.setParent(parent);

        return assignment;
    }

    public static void compare(RPMObject[] eArray1,
            RPMObject[] eArray2) throws AssertionFailedError,
            Exception
    {
        sortWithComparator(eArray1, RPMObjectIDComparator.INSTANCE);
        sortWithComparator(eArray2, RPMObjectIDComparator.INSTANCE);

        RPMObjectFixture.compare(eArray1, eArray2);
        if (eArray1 == null)
            return;

        for (int i = 0; i < eArray1.length; i++)
        {
            compare(eArray1[i], eArray2[i]);
        }
    }

    public static void compare(CustomFieldAssignment assignment1,
            CustomFieldAssignment assignment2) throws AssertionFailedError,
            Exception
    {
        RPMObjectFixture.compare(assignment1, assignment2);

        // The default value is expected if it was saved with a null value.
        if (assignment1.getValue() == null && assignment1.getCustomField() != null 
                && assignment1.getCustomField().getDefaultValue() != null)
        {
            compareValues(assignment1.getCustomField().getDefaultValue(), 
                assignment2.getValue());
        }
        else
        {
            compareValues(assignment1.getValue(), assignment2.getValue());
        }
        
        // for tests parent is optional - sometimes it is not loaded
        // because assignments do not load parents
        if (assignment1.getParent() != null && assignment2.getParent() != null)
        {
            Assert.assertEquals("Elements parent IDs are not the same",
                assignment1.getParent().getID(),
                assignment2.getParent().getID());
        }

        Assert.assertFalse(
            "First elements CustomField is null while second is not",
            assignment1.getCustomField() == null
                    && assignment2.getCustomField() != null);
        Assert.assertFalse(
            "Second elements CustomField is null while first is not",
            assignment2.getCustomField() == null
                    && assignment1.getCustomField() != null);
        Assert.assertEquals("Elements CustomField IDs are not the same",
            assignment1.getCustomField().getID(),
            assignment2.getCustomField().getID());
    }

    public static void compareValues(Object expected, Object actual)
    {
        if (expected instanceof Datafield)
        {
            assertEqualId("DefaultValue", (Datafield) expected,
                (Datafield) actual);
        }
        else
        {
            Assert.assertEquals(expected, actual);
        }
    }

    private static void assertEqualId(String message, RPMObject expected,
            RPMObject actual)
    {
        assertBothNullOrNot(message, expected, actual);
        if (expected != null)
        {
            Assert.assertEquals(message, expected.getID(), actual.getID());
        }
    }

    private static void assertBothNullOrNot(String message, Object expected,
            Object actual)
    {
        if (expected == null)
        {
            Assert.assertNull(message
                    + ": expected a null but actual is not null.", actual);
        }
        if (actual == null)
        {
            Assert.assertNull(message
                    + ": expected not null but actual is null.", expected);
        }
    }

    public static void saveLoadCompare(CustomFieldAssignment assignment,
            ClientTestController ctc) throws Exception
    {
        CustomFieldAssignment saved = (CustomFieldAssignment) ctc.save(
            assignment, null).getRpmObject();

        CustomFieldAssignmentFixture.compare(assignment, saved);

        CustomFieldAssignment loaded = (CustomFieldAssignment) ctc.loadFromID(
            assignment, null);

        CustomFieldAssignmentFixture.compare(assignment, loaded);
    }


}
