package com.ibm.rpm.client.junit.customfield.fixtures;

import java.text.ParseException;

import junit.framework.Assert;

import com.ibm.rpm.applicationadministration.containers.CustomDatafield;
import com.ibm.rpm.applicationadministration.containers.CustomDatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.containers.GenericCustomField;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.DateUtil;

public class CustomFieldFixture extends RPMObjectFixture
{
    // =============
    // CONTEXT NAMES
    // =============

    public static final String CF_ASSET_CONTEXT = "Asset";

    public static final String CF_CLIENT_CONTEXT = "Client";

    public static final String CF_DOCUMENT_CONTEXT = "Document";

    public static final String CF_WBS_SCOPE_CONTEXT = "WBS";

    public static final String CF_RESOURCE_CONTEXT = "Resource";

    // ===============
    // OBJECT BUILDERS
    // ===============

    public static final String NAME_PREFIX = HelperTestUtilities.getUnitTestPrefix() + HelperTestUtilities.getUserName();

    public static CustomField createSaveCustomField(String contextName, 
            FieldInputType inputType, ClientTestController ctc) throws Exception
    {
        return createSaveCustomField(contextName, inputType, new Integer(0), ctc);        
    }
    
    public static CustomField createSaveCustomField(String contextName,
            FieldInputType inputType, Integer secGroup, ClientTestController ctc)
            throws Exception
    {
        CustomField original = createCustomField(inputType, secGroup);
        
        //Ensure that customDatafieldCategory is created first.
        DatafieldCategory dfCategory = original.getDatafieldCategory();
        if( dfCategory != null && dfCategory instanceof CustomDatafieldCategory){
            dfCategory = (CustomDatafieldCategory)ctc.save(dfCategory, null).getRpmObject(); 
            original.setDatafieldCategory(dfCategory);
            original.setDefaultValue(dfCategory.getChildren()[0]);
        }
        
        //Ensure the parent is created first.
        CustomFieldCategory parent = (CustomFieldCategory)original.getParent();        
        parent.setContextName(contextName);
        original.setContextName(contextName);        
        parent = (CustomFieldCategory)ctc.save(original.getParent(), null).getRpmObject();
        //reassigned saved parent
        original.setParent(parent);
        
        CustomFieldScope cfScope = new CustomFieldScope();
        cfScope.setParent(new CustomFieldScope());
        original = (CustomField)ctc.save(original, cfScope).getRpmObject();
        return original;
    }

    public static CustomFieldCategory createCustomFieldCategory() throws Exception
    {
        return createCustomFieldCategory(0, FieldInputType.Text, new Integer(0));
    }

    public static CustomFieldCategory createCustomFieldCategory(Integer secGroup)
            throws Exception
    {
        return createCustomFieldCategory(0, FieldInputType.Text, secGroup);
    }

    public static CustomFieldCategory createCustomFieldCategory(int childrenCount)
        throws Exception
    {
        return createCustomFieldCategory(childrenCount, FieldInputType.Text, new Integer(0));
    }
    
    public static CustomFieldCategory createCustomFieldCategory(int childrenCount, 
            Integer secGroup) throws Exception
    {
        return createCustomFieldCategory(childrenCount, FieldInputType.Text, secGroup);
    }

    public static CustomFieldCategory createCustomFieldCategory(int childrenCount, 
            FieldInputType inputType) throws Exception
    {
        return createCustomFieldCategory(childrenCount, inputType, new Integer(0));
    }
    
    public static CustomFieldCategory createCustomFieldCategory(
            int childrenCount, FieldInputType inputType, Integer secGroup) throws Exception
    {
        CustomFieldCategory category = new CustomFieldCategory();

        category.setName(HelperTestUtilities.createName() + getSequenceNo());

        if (childrenCount > 0)
        {
            CustomField[] children = new CustomField[childrenCount];
            for (int i = 0; i < childrenCount; i++)
            {
                children[i] = createCustomField(category, inputType, secGroup);
            }
            category.setChildren(children);
        }
        return category;
    }

    public static CustomField createCustomField() throws Exception
    {
        return createCustomField(FieldInputType.Text, new Integer(0));
    }

    public static CustomField createCustomField(FieldInputType inputType)
    throws Exception
    {
        return createCustomField(inputType, new Integer(0));
    }
    
    public static CustomField createCustomField(Integer secGroup) throws Exception
    {
        return createCustomField(FieldInputType.Text, secGroup);
    }

    public static CustomField createCustomField(FieldInputType inputType,
            Integer secGroup) throws Exception
    {
        CustomFieldCategory category = createCustomFieldCategory(1, inputType, secGroup);
        CustomField customField = category.getChildren()[0];

        return customField;
    }
    
    public static CustomField createCustomField(CustomFieldCategory parent,
            FieldInputType inputType) throws Exception
    {
        return createCustomField(parent, inputType, new Integer(0));
    }
    
    public static CustomField createCustomField(CustomFieldCategory parent,
            FieldInputType inputType, Integer secGroup) throws Exception
    {
        CustomField customField = new CustomField();
        customField.setParent(parent);
        customField.setName(HelperTestUtilities.createName()+ getSequenceNo());
        customField.setSecurityGroup(secGroup);
        setInputType(customField, inputType);

        return customField;
    }

    public static void setInputType(CustomField customField,
            FieldInputType inputType) throws ParseException
    {
        DatafieldCategory category = null;
        Object defaultValue = null;
        if (FieldInputType.Checkbox.equals(inputType))
        {
            defaultValue = Boolean.TRUE;
        }
        else if (FieldInputType.Currency.equals(inputType))
        {
            defaultValue = new Double(987.65);
        }
        else if (FieldInputType.Date.equals(inputType))
        {
            defaultValue = DateUtil.convertToCalendarNoTime("2001-09-11");
        }
        else if (FieldInputType.Numeric.equals(inputType))
        {
            defaultValue = new Integer(987654321);
        }
        else if (FieldInputType.Text.equals(inputType))
        {
            defaultValue = "Hello World!";
        }
        else if (FieldInputType.Datafield.equals(inputType))
        {
            category = DatafieldFixture
                .createDatafieldCategory(DatafieldFixture.PROJECT_RTF_CATEGORY_ID);
            defaultValue = DatafieldFixture
                .createDatafieldRTF(DatafieldFixture.PROJECT_RTF_1);
        }
        else
        {
            // Custom datafield.
            inputType = FieldInputType.Datafield;
            category = DatafieldFixture.createCustomDatafieldCategory(2);
            defaultValue = category.getChildren()[0];
        }
        customField.setInputType(inputType);
        customField.setDatafieldCategory(category);
        customField.setDefaultValue(defaultValue);
    }

    // ==============
    // SCOPE BUILDERS
    // ==============

    public static CustomFieldScope createCustomFieldScope(int depth)
    {
        CustomFieldScope top = null;
        CustomFieldScope middle = null;
        CustomFieldScope bottom = null;
        if (depth > 0)
        {
            top = new CustomFieldScope();
            if (depth > 1)
            {
                middle = new CustomFieldScope();
                top.setChildren(middle);
                middle.setParent(top);
                if (depth > 2)
                {
                    bottom = new CustomFieldScope();
                    middle.setChildren(bottom);
                    bottom.setParent(middle);
                }
            }
        }
        return top;
    }

    // =============
    // MODIFICATIONS
    // =============

    public static void modify(GenericCustomField object, CustomFieldScope scope)
            throws Exception
    {
        if (object == null)
            return;
        object.setName(HelperTestUtilities.createName() + getSequenceNo());
        if (object instanceof CustomField)
            modifyCustomField((CustomField) object);
        else if (scope != null)
        {
            if (object instanceof CustomFieldCategory)
                modifyCustomFieldCategory((CustomFieldCategory) object, scope);
        }
    }

    public static CustomFieldCategory[] modifyCustomFieldCategories(
            CustomFieldCategory[] children, CustomFieldScope childrenScope)
            throws Exception
    {
        if (childrenScope == null)
            return children;
        int childrenCount = 0;
        if (children != null)
        {
            childrenCount = children.length;
            for (int i = 0; i < childrenCount; i++)
            {
                modify(children[i], childrenScope);
            }
        }
        return children;
    }

    private static void modifyCustomFieldCategory(CustomFieldCategory object,
            CustomFieldScope scope) throws Exception
    {
        CustomFieldScope childrenScope = scope.getChildren();
        if (childrenScope == null)
            return;
        CustomField[] children = object.getChildren();
        int childrenCount = 0;
        if (children != null)
        {
            childrenCount = children.length;
            for (int i = 0; i < childrenCount; i++)
            {
                switch (i % 3)
                {
                case 0: // Child modification.
                    modify(children[i], childrenScope);
                    break;
                case 1: // Child deletion.
                    children[i].setDeleted(Boolean.TRUE);
                    break;
                case 2: // Child ignored.
                    children[i] = null;
                    break;
                }
            }
        }
        // Add new children.
        CustomField[] newChildren = new CustomField[childrenCount
                + CREATION_COUNT];
        if (childrenCount > 0)
        {
            System.arraycopy(children, 0, newChildren, 0, childrenCount);
        }
        for (int i = 0; i < CREATION_COUNT; i++)
        {
            // Create new child.
            CustomField newChild = createCustomField(FieldInputType.Text);
            // Associate parent and child.
            newChildren[childrenCount + i] = newChild;
            newChild.setParent(object);
        }
        object.setChildren(newChildren);
    }

    private static void modifyCustomField(CustomField object)
    {
        object.setSecurityGroup(new Integer(43));
        object.setInputType(FieldInputType.Currency);
        object.setDatafieldCategory(null);
        object.setDefaultValue(new Double(9999.98));
    }

    private static final int CREATION_COUNT = 2;
    
    // ===========
    // SEQUENCE NO
    // ===========

    private static int getSequenceNo()
    {
        sequenceNo = (sequenceNo + 1) % 100;
        return sequenceNo;
    }

    private static int sequenceNo = (int) (System.currentTimeMillis() % 100L);

    // ===================
    // EQUALITY ASSERTIONS
    // ===================

    public static void assertIdEqual(GenericCustomField[] expected,
            GenericCustomField[] actual, CustomFieldScope childrenScope)
    {
        if (childrenScope == null)
        {
            Assert.assertNull("Actual should be null when scope is null.",
                actual);
        }
        else
        {
            assertBothNullOrNot("Children", expected, actual);
            if (expected != null)
            {
                Assert.assertTrue(
                    "Actual must have at least as many elements as expected.",
                    count(expected) <= count(actual));
                for (int i = 0; i < expected.length; i++)
                {
                    GenericCustomField expectedChild = expected[i];
                    GenericCustomField actualChild = findInArray(expectedChild,
                        actual);
                    Assert.assertNotNull(actualChild);
                }
            }
        }
    }

    public static void assertEquals(RPMObject expected, RPMObject actual,
            CustomFieldScope scope)
    {
        if (expected instanceof CustomFieldCategory)
            doAssertEquals((CustomFieldCategory) expected, actual, scope);
        else if (expected instanceof CustomField)
            doAssertEquals((CustomField) expected, actual);
    }

    private static void doAssertEquals(CustomField expected,
            RPMObject actualObject)
    {
        doAssertTypeIdNameEquals(expected, actualObject);
        if (expected != null)
        {
            CustomField actual = (CustomField) actualObject;
            //assertEqualId("DatafieldCategory",
            // expected.getDatafieldCategory(),
            // actual.getDatafieldCategory());
            doAssertDefaultEquals(expected.getDefaultValue(), actual
                .getDefaultValue());
            Assert.assertEquals("InputType", expected.getInputType(), actual
                .getInputType());
            Assert.assertEquals("SecurityGroup", expected.getSecurityGroup(),
                actual.getSecurityGroup());
        }
    }

    private static void assertEqualId(String message, RPMObject expected,
            RPMObject actual)
    {
        assertBothNullOrNot(message, expected, actual);
        if (expected != null)
        {
            Assert.assertEquals(message, expected.getID(), actual.getID());
        }
    }

    private static void assertBothNullOrNot(String message, Object expected,
            Object actual)
    {
        if (expected == null)
        {
            Assert.assertNull(message
                    + ": expected a null but actual is not null.", actual);
        }
        if (actual == null)
        {
            Assert.assertNull(message
                    + ": expected not null but actual is null.", expected);
        }
    }

    private static void doAssertDefaultEquals(Object expected, Object actual)
    {
        if (expected instanceof Datafield)
        {
            assertEqualId("DefaultValue", (Datafield) expected,
                (Datafield) actual);
        }
        else
        {
            Assert.assertEquals(expected, actual);
        }
    }

    private static void doAssertEquals(CustomFieldCategory expected,
            RPMObject actualObject, CustomFieldScope scope)
    {
        doAssertTypeIdNameEquals(expected, actualObject);
        if (expected != null && scope != null)
        {
            CustomFieldCategory actual = (CustomFieldCategory) actualObject;
            assertChildrenEqual(expected.getChildren(), actual.getChildren(),
                scope.getChildren());
        }
    }

    private static void assertChildrenEqual(GenericCustomField[] expected,
            GenericCustomField[] actual, CustomFieldScope childrenScope)
    {
        if (childrenScope == null)
        {
            Assert.assertNull("Actual should be null when scope is null.",
                actual);
        }
        else
        {
            assertBothNullOrNot("Children", expected, actual);
            if (expected != null)
            {
                Assert.assertTrue(
                    "Actual must have at least as many children as expected.",
                    count(expected) <= count(actual));
                for (int i = 0; i < expected.length; i++)
                {
                    GenericCustomField expectedChild = expected[i];
                    GenericCustomField actualChild = findInArray(expectedChild,
                        actual);
                    assertEquals(expectedChild, actualChild, childrenScope);
                }
            }
        }
    }

    private static int count(GenericCustomField[] children)
    {
        int count = 0;
        if (children != null)
        {
            for (int i = 0; i < children.length; i++)
            {
                GenericCustomField child = children[i];
                if (child != null
                        && (child.getDeleted() == null || !child.getDeleted()
                            .booleanValue()))
                    count++;
            }
        }
        return count;
    }

    public static GenericCustomField findInArray(GenericCustomField object,
            Object[] objects)
    {
        if (object != null)
        {
            String expectedID = object.getID();
            for (int i = 0; i < objects.length; i++)
            {
                GenericCustomField candidate = (GenericCustomField) objects[i];
                if (candidate != null && expectedID.equals(candidate.getID()))
                    return candidate;
            }
        }
        return null;
    }

    private static void doAssertTypeIdNameEquals(GenericCustomField expected,
            RPMObject actualObject)
    {
        if (expected == null)
        {
            Assert.assertNull("Expected a null but actual is not null.",
                actualObject);
            return;
        }
        Assert.assertTrue(actualObject instanceof GenericCustomField);
        GenericCustomField actual = (GenericCustomField) actualObject;
        // Assert.assertEquals("Type", expected.retrieveTypeID(),
        // actual.retrieveTypeID());
        Assert.assertEquals("ID", expected.getID(), actual.getID());
        Assert.assertEquals("Name", expected.getName(), actual.getName());
    }

    // ===================
    // DELETION ASSERTIONS
    // ===================

    /**
     * Asserts that all custom fields created for tests have been deleted from
     * the database.
     * 
     * @param controller
     */
    public static void assertAllDeleted(ClientTestController controller)
            throws Exception
    {
        RPMObject[] loaded = loadNonDeleted(controller);
        Assert.assertTrue("All custom fields should be deleted", loaded == null
                || loaded.length == 0);
    }

    public static void assertDeleted(ClientTestController controller,
            GenericCustomField original) throws Exception
    {
        doAssertDeleted(controller, original);
        if (original instanceof CustomFieldCategory)
            doAssertChildrenDeleted(controller,
                ((CustomFieldCategory) original).getChildren());
    }

    private static void doAssertChildrenDeleted(
            ClientTestController controller, GenericCustomField[] children)
            throws Exception
    {
        if (children != null)
        {
            for (int i = 0; i < children.length; i++)
            {
                assertDeleted(controller, children[i]);
            }
        }
    }

    private static void doAssertDeleted(ClientTestController controller,
            GenericCustomField original) throws Exception
    {
        try
        {
            controller.loadFromID(original, null);
            Assert.fail("Object '" + original.getID() + "'not deleted.");
        }
        catch (Exception x)
        {
            // Normal case: object should be deleted.
        }
    }

    // ======
    // DELETE
    // ======

    /**
     * Deletes all saved custom fields that have a name that starts with
     * <code>CustomFieldFixture.NAME_PREFIX</code>.
     * 
     * @param controller
     */
    public static void deleteAllCustomFields(ClientTestController controller)
            throws Exception
    {
        RPMObject[] chargeCodes = loadNonDeleted(controller);
        if (chargeCodes == null)
            return;
        // Delete all custom fields created for tests.
        for (int i = 0; i < chargeCodes.length; i++)
        {
            try
            {
                controller.delete(chargeCodes[i]);
            }
            catch (Exception x)
            {
                // Charge code might have been deleted when ancestor
                // was deleted.
            }
        }
    }

    private static RPMObject[] loadNonDeleted(ClientTestController controller)
            throws Exception
    {
        // Build query.
        final String query = "/GenericCustomField[starts-with(Name, '"
                + NAME_PREFIX + "')]";
        logger.debug("XPath query: " + query);

        // Load the set of objects that match the specified property.
        RPMObject[] loaded = controller.loadFromXpath(query, null);
        return loaded;
    }

    // ====
    // SAVE
    // ====

    public static GenericCustomField save(ClientTestController controller,
            GenericCustomField original) throws Exception
    {
        CustomFieldScope scope = createCustomFieldScope(2).getChildren();
        return save(controller, original, scope);
    }

    public static GenericCustomField save(ClientTestController controller,
            GenericCustomField original, CustomFieldScope scope)
            throws Exception
    {
        // Ensure the custom datafield is created first.
        if (original instanceof CustomField)
        {
            CustomField customField = (CustomField) original;
            Object defaultValue = customField.getDefaultValue();
            if (defaultValue instanceof CustomDatafield)
            {
                CustomDatafield datafield = (CustomDatafield) defaultValue;
                controller.save(datafield.getParent(), null, ReloadType.None);
            }
        }
        // Ensure the parent is created first.
        if (original.getParent() != null
                && original.getParent().getID() == null)
        {
            //contextName needs to be set to the parent
            original.getParent().setContextName(original.getContextName());
            controller.save(original.getParent(), null, ReloadType.None);
        }
        SingleResult result = controller.save(original, scope);
        GenericCustomField reloaded = (GenericCustomField) result
            .getRpmObject();
        CustomFieldFixture.assertEquals(original, reloaded, scope);
        return reloaded;
    }
}
