/*
 * Created on Sep 14, 2006
 */
package com.ibm.rpm.client.junit.customfield.fixtures;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.customfield.containers.CustomPortlet;
import com.ibm.rpm.customfield.containers.CustomPortletCategory;
import com.ibm.rpm.customfield.containers.GenericCustomPortlet;
import com.ibm.rpm.customfield.scope.CustomPortletScope;
import com.ibm.rpm.customfield.types.CustomPortletCategoryGroupType;
import com.ibm.rpm.framework.RPMObject;

public class CustomPortletFixture extends RPMObjectFixture
{
    public static final String CF_ASSET_CONTEXT = "Asset";

    public static final String CF_CLIENT_CONTEXT = "Client";

    public static final String CF_DOCUMENT_CONTEXT = "Document";

    public static final String CF_WBS_SCOPE_CONTEXT = "WBS";

    public static final String CF_RESOURCE_CONTEXT = "Resource";

    public static final String NAME_PREFIX = HelperTestUtilities
        .getUnitTestPrefix()
            + HelperTestUtilities.getUserName();
    
    private static int sequenceNo = (int) (System.currentTimeMillis() % 100L);
    private static int getSequenceNo()
    {
        sequenceNo = (sequenceNo + 1) % 100;
        return sequenceNo;
    }

    public static CustomPortletCategory createCustomPortletCategory(
            int childrenCount) throws Exception
    {
        CustomPortletCategory category = new CustomPortletCategory();
        category.setName(HelperTestUtilities.createName() + getSequenceNo());
        category.setGroupType(CustomPortletCategoryGroupType.DeliverableGroup);
        if (childrenCount > 0)
        {
            CustomPortlet[] children = new CustomPortlet[childrenCount];
            for (int i = 0; i < childrenCount; i++)
            {
                children[i] = createCustomPortlet(category);
            }
            category.setChildren(children);
        }
        return category;
    }

    public static CustomPortlet createCustomPortlet(CustomPortletCategory parent)
    {
        CustomPortlet customPortlet = new CustomPortlet();
        customPortlet.setParent(parent);
        customPortlet.setName(HelperTestUtilities.createName() + getSequenceNo());
        return customPortlet;
    }
    
    public static GenericCustomPortlet updateGenericCustomPortletName(
            GenericCustomPortlet customPortlet)
    {
        customPortlet.setName(HelperTestUtilities.createName() + getSequenceNo());   
        return customPortlet;
    }
    
    public static CustomPortletCategory addChildren(CustomPortletCategory category,
            int nb)
    {
        int nbChildren = 0;
        CustomPortlet[] portlets = category.getChildren();
        if(portlets != null)
        {
            nbChildren = portlets.length;
        }
        
        CustomPortlet[] newChildren = new CustomPortlet[nbChildren + nb];
        if (nbChildren > 0)
        {
            System.arraycopy(portlets, 0, newChildren, 0, nbChildren);
        }
        
        for (int i = 0; i < nb; i++)
        {
            // Create new child.
            CustomPortlet newChild = createCustomPortlet(category);
            // Associate parent and child.
            newChildren[nbChildren + i] = newChild;
        }
        category.setChildren(newChildren);
        return category;
    }

    public static void compare(CustomPortlet current, RPMObject expected)
    {
        Assert.assertEquals(current.getName(), ((CustomPortlet) expected)
            .getName());
    }
    
    public static void compare(CustomPortlet[] current, RPMObject[] expected)
    {
        if (expected != null)
        {
            Assert.assertTrue(
                "Current must have at least as many elements as expected.",
                count(expected) <= count(current));
            sortWithComparator(current, new RPMObjectIDComparator());
            sortWithComparator( expected, new RPMObjectIDComparator());
            for (int i = 0; i < current.length; i++)
            {
                compare(current[i], expected[i]);
            }
        }
    }
        
    private static int count(RPMObject[] children)
    {
        int count = 0;
        if (children != null)
        {
            for (int i = 0; i < children.length; i++)
            {
                GenericCustomPortlet child = (GenericCustomPortlet)children[i];
                if (child != null
                        && (child.getDeleted() == null || !child.getDeleted()
                            .booleanValue()))
                    count++;
            }
        }
        return count;
    }   
    
    public static void compare(CustomPortletCategory currentCategory,
            CustomPortletCategory expectedCategory, CustomPortletScope scope)
    {
        Assert.assertEquals(currentCategory.getName(), expectedCategory
            .getName());
        if (scope != null && scope.isChildren())
        {
            compare(currentCategory.getChildren(), expectedCategory
                .getChildren());
        }
    }
}
