package com.ibm.rpm.client.junit.customfield.testcases;

import com.ibm.rpm.applicationadministration.containers.DatafieldRTF;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldAssignmentFixture;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.DateUtil;

public class TestCustomFieldAssignmentTypes extends ClientTestController
{
    // ==============================
    // INITIALIZATION AND TERMINATION
    // ==============================

    protected void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        logger.info("tearDown");
        super.tearDown();
    }

    // ===============
    // SAVE/LOAD TESTS
    // ===============

    public void testSaveLoad_Text() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Text, this);
        acf.setValue("ABC");

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }

    public void testSaveLoad_CheckBoxTrue() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Checkbox, this);
        acf.setValue(Boolean.valueOf(true));

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }

    public void testSaveLoad_CheckBoxFalse() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Checkbox, this);
        acf.setValue(Boolean.valueOf(false));

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }

    public void testSaveLoad_Date() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Date, this);
        acf.setValue(DateUtil.convertToCalendarNoTime("2067-09-11"));

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }

    public void testSaveLoad_Numeric() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Numeric, this);
        acf.setValue(new Integer(100));

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }

    public void testSaveLoad_Currency() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Currency, this);
        acf.setValue(new Double(100.001D));

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }

    public void testSaveLoad_Datafield() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Datafield, this);

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }

    // ============
    // UPDATE TESTS
    // ============

    public void testUpdate_Text() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Text, this);
        acf.setValue("ABC");

        save(acf, null);
        acf.setValue("BCD");

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }

    public void testUpdate_Checkbox() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Checkbox, this);
        acf.setValue(Boolean.valueOf(false));

        save(acf, null);
        acf.setValue(Boolean.valueOf(true));

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }

    public void testUpdate_Date() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Date, this);
        acf.setValue(DateUtil.convertToCalendarNoTime("2067-09-11"));

        save(acf, null);
        acf.setValue(DateUtil.convertToCalendarNoTime("2057-08-10"));

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }

    public void testUpdate_Numeric() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Numeric, this);
        acf.setValue(new Integer(123));

        save(acf, null);
        acf.setValue(new Integer(456));

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }

    public void testUpdate_Currency() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Currency, this);
        acf.setValue(new Double(123.321));

        save(acf, null);
        acf.setValue(new Double(321.123));

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }
    
    public void testUpdate_Datafield() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Datafield, this);
        DatafieldRTF value = DatafieldFixture
            .createDatafieldRTF(DatafieldFixture.PROJECT_RTF_1);

        save(acf, null);

        value = DatafieldFixture
            .createDatafieldRTF(DatafieldFixture.PROJECT_RTF_2);

        acf.setValue(value);

        CustomFieldAssignmentFixture.saveLoadCompare(acf, this);
    }
    
    // ============
    // DELETE TESTS
    // ============

    public void testDelete() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Text, this);
        acf.setValue("ABC");

        CustomFieldAssignment saved = (CustomFieldAssignment) save(acf, null)
                .getRpmObject();
        // saving of an assignment cannot load parent itself
        saved.setParent(acf.getParent());

        delete(saved);

        try
        {
            assertNull("Deleted object should be null or throw an exception",
                    loadFromID(saved, null));
        }
        catch (RPMException rpmexception)
        {
        }
    }

    // ============
    // XPath tests
    // ============
    public void testXPathLoad() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.RESOURCE_CONTEXT,
                        FieldInputType.Text, this);
        acf.setValue("ABC");

        save(acf, null);

        RPMObject[] rpmObjects = loadFromXpath("/CustomFieldAssignment[Id='"
                + acf.getID() + "']", null);

        assertNotNull(rpmObjects);
        assertEquals(1, rpmObjects.length);
        assertTrue(rpmObjects[0] instanceof CustomFieldAssignment);
        CustomFieldAssignmentFixture.compare(acf,
                (CustomFieldAssignment) rpmObjects[0]);
    }
}
