package com.ibm.rpm.client.junit.customfield.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldAssignmentFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;

public class TestCustomFieldScopeIntegration extends ClientTestController
{
    // ==============================
    // INITIALIZATION AND TERMINATION
    // ==============================

    protected void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        logger.info("tearDown");
        super.tearDown();
    }

    // ============
    // XPath tests
    // ============
    public void testXPathLinkParent() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.SCOPE_CONTEXT,
                        FieldInputType.Text, this);
        acf.setValue("ABC");

        acf = (CustomFieldAssignment)save(acf, null).getRpmObject();

        RPMObject[] rpmObjects = loadFromXpath(
                "/ScopeElement/CustomFieldAssignment[Id='" + acf.getID() + "']",
                null);

        assertNotNull(rpmObjects);

        // for some unknown reason this XPath returns 2 same objects
        // this is either WBSManager or the framework problem 
        // this was here before: assertEquals(1, rpmObjects.length);
        for (int i = 0; i < rpmObjects.length; i++)
        {

            assertTrue(rpmObjects[i] instanceof CustomFieldAssignment);

            CustomFieldAssignmentFixture.compare(acf,
                    (CustomFieldAssignment) rpmObjects[i]);
        }
    }

    public void testXPathLinkSubtable() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.SCOPE_CONTEXT,
                        FieldInputType.Text, this);
        RPMObject parent = acf.getParent();
        acf.setValue("ABC");

        acf = (CustomFieldAssignment)save(acf, null).getRpmObject();

        RPMObject[] rpmObjects = loadFromXpath(
                "/ScopeElement[CustomFieldAssignment[Id='" + acf.getID() + "']]",
                null);

        assertNotNull(rpmObjects);
        assertEquals(1, rpmObjects.length);
        assertTrue(rpmObjects[0] instanceof ScopeElement);
        assertEquals(parent.getID(), rpmObjects[0].getID());
    }
    
    public void testXPathWithAssignedSpecificCustomFields() throws Exception
    {
               
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
            .createCustomFieldAssignment(CustomFieldAssignmentFixture.SCOPE_CONTEXT,
                FieldInputType.Text, this);
        acf.setValue("ABC");
        
        CustomField cf = acf.getCustomField();
        ScopeElement parent = (ScopeElement)acf.getParent();

        save(acf, null);

        // Check if the assigned customField is loaded
        RPMObject[] assigned = loadFromXpath(
                "/ScopeElement[id='" + parent.getID() + "']/CustomFieldAssignment[CustomField[id='" + cf.getID() + "']]",
                null);
        
        assertNotNull(assigned);
        assertEquals(1, assigned.length);
        assertTrue(assigned[0] instanceof CustomFieldAssignment);
        CustomFieldAssignmentFixture.compare(acf,
            (CustomFieldAssignment) assigned[0]);
    }

    public void testXPathWithAssignedSpecificCustomFieldCategory() throws Exception
    {
        CustomFieldAssignment [] asf = new CustomFieldAssignment[2];
        
        // CustomField default is a CustomDatafield
        asf[0] = CustomFieldAssignmentFixture
            .createCustomFieldAssignment(
                CustomFieldAssignmentFixture.SCOPE_CONTEXT, null,
                this);

        CustomField cf1 = asf[0].getCustomField();
        ScopeElement parent = (ScopeElement) asf[0].getParent();
        SaveResult result = save(asf[0], null);
        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        asf[0].setID(result.getRpmObject().getID());
        
        // Add a second cf in the category
        CustomField cf2 = CustomFieldFixture.createCustomField((CustomFieldCategory)cf1.getParent(), cf1.getInputType());
        result = save(cf2, null);
        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        cf2.setID(result.getRpmObject().getID());
        
        // Assign the second cf
        asf[1] = new CustomFieldAssignment();
        asf[1].setCustomField(cf2);
        asf[1].setParent(asf[0].getParent());
        result = save(asf[1], null);
        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        asf[1].setID(result.getRpmObject().getID());        
        
        // Check if the assigned customField is loaded
        RPMObject [] assigned = loadFromXpath(
            "/ScopeElement[id='" + parent.getID() + "']/CustomFieldAssignment[CustomFieldCategory[id='" + cf1.getParent().getID() + "']/CustomField]",
            null);

        assertNotNull(assigned);
        assertEquals(2, assigned.length);
        
        CustomFieldAssignmentFixture.compare(asf, assigned);     
    }
    
    public void testAssignAndUpdate() throws Exception
    {
        CustomField cf = CustomFieldFixture.createSaveCustomField(
                CustomFieldFixture.CF_WBS_SCOPE_CONTEXT, FieldInputType.Text,
                this);

        ScopeElementScope scope = new ScopeElementScope();
        scope.setCustomFieldAssignments(true);
        ScopeElement object = (ScopeElement) CustomFieldAssignmentFixture
                .createParent(CustomFieldAssignmentFixture.SCOPE_CONTEXT, this);

        //assigned custom field with the default value
        CustomFieldAssignment cfa = new CustomFieldAssignment();
        cfa.setCustomField(cf);
        cfa.setValue(cf.getDefaultValue());
        object.setCustomFieldAssignments(new CustomFieldAssignment[]{cfa});
        
        save(object, scope);

        ScopeElement loaded = (ScopeElement) loadFromID(object, scope);

        assertNotNull(loaded.getCustomFieldAssignments());
        assertEquals(1, loaded.getCustomFieldAssignments().length);
        assertEquals(loaded.getCustomFieldAssignments()[0].getValue(), cf.getDefaultValue());
       
        
        //update customfield with a null value
        object = loaded;
        object.getCustomFieldAssignments()[0].setValue(null);
 
        save(object, scope);

        loaded = (ScopeElement) loadFromID(object, scope);

        assertNotNull(loaded.getCustomFieldAssignments());
        assertEquals(1, loaded.getCustomFieldAssignments().length);
        assertEquals(loaded.getCustomFieldAssignments()[0].getValue(), null);
        
        //update customfield with a null value
        object = loaded;
        object.getCustomFieldAssignments()[0].setValue("ABC");
 
        save(object, scope);

        loaded = (ScopeElement) loadFromID(object, scope);

        assertNotNull(loaded.getCustomFieldAssignments());
        assertEquals(1, loaded.getCustomFieldAssignments().length);
        assertEquals(loaded.getCustomFieldAssignments()[0].getValue(), "ABC");

    }

}
