package com.ibm.rpm.client.junit.customfield.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldAssignmentFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author ikusakov
 */

public class TestCustomFieldWBSIntegration extends ClientTestController
{
    // ==============================
    // INITIALIZATION AND TERMINATION
    // ==============================

    protected void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        logger.info("tearDown");
        super.tearDown();
    }

    // ============
    // XPath tests
    // ============
    public void testXPathLinkParent() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.WBS_CONTEXT,
                        FieldInputType.Text, this);
        acf.setValue("ABC");

        acf = (CustomFieldAssignment)save(acf, null).getRpmObject();

        RPMObject[] rpmObjects = loadFromXpath(
                "/WorkElement/CustomFieldAssignment[Id='" + acf.getID() + "']",
                null);

        assertNotNull(rpmObjects);

        // for some unknown reason this XPath returns 2 same objects
        // this is either WBSManager or the framework problem
        // this was here before: assertEquals(1, rpmObjects.length);
        for (int i = 0; i < rpmObjects.length; i++)
        {

            assertTrue(rpmObjects[i] instanceof CustomFieldAssignment);

            CustomFieldAssignmentFixture.compare(acf,
                    (CustomFieldAssignment) rpmObjects[i]);
        }
    }

    public void testXPathLinkSubtable() throws Exception
    {
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
                .createCustomFieldAssignment(CustomFieldAssignmentFixture.WBS_CONTEXT,
                        FieldInputType.Text, this);
        RPMObject parent = acf.getParent();
        acf.setValue("ABC");

        acf = (CustomFieldAssignment)save(acf, null).getRpmObject();

        RPMObject[] rpmObjects = loadFromXpath(
                "/WorkElement[CustomFieldAssignment[Id='" + acf.getID() + "']]",
                null);

        assertNotNull(rpmObjects);
        assertEquals(1, rpmObjects.length);
        assertTrue(rpmObjects[0] instanceof WorkElement);
        assertEquals(parent.getID(), rpmObjects[0].getID());
    }
    
    public void testXPathWithAssignedSpecificCustomFields() throws Exception
    {
               
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
            .createCustomFieldAssignment(CustomFieldAssignmentFixture.WBS_CONTEXT,
                FieldInputType.Text, this);
        acf.setValue("ABC");
        
        CustomField cf = acf.getCustomField();
        WorkElement parent = (WorkElement)acf.getParent();

        save(acf, null);

        //check if the assigned customField is loaded
        RPMObject[] assigned = loadFromXpath(
                "/WorkElement[id='" + parent.getID() + "']/CustomFieldAssignment[CustomField[id='" + cf.getID() + "']]",
                null);
        
        assertNotNull(assigned);
        assertEquals(1, assigned.length);
        assertTrue(assigned[0] instanceof CustomFieldAssignment);
        CustomFieldAssignmentFixture.compare(acf,
            (CustomFieldAssignment) assigned[0]);
    }
    
    public void testXPathWithAssignedSpecificCustomFieldCategory() throws Exception
    {
               
        //CustomField default is a CustomDatafield
        CustomFieldAssignment acf = CustomFieldAssignmentFixture
            .createCustomFieldAssignment(
                CustomFieldAssignmentFixture.WBS_CONTEXT, null,
                this);

        CustomField cf = acf.getCustomField();
        WorkElement parent = (WorkElement) acf.getParent();

        //Add a second cf in the category
        CustomField cf2 = CustomFieldFixture.createCustomField((CustomFieldCategory)cf.getParent(), cf.getInputType());
        save(cf2, null);
        
        save(acf, null);        
        
        //check if the assigned customField is loaded
        RPMObject[] assigned = loadFromXpath(
            "/WorkElement[id='" + parent.getID() + "']/CustomFieldAssignment[CustomFieldCategory[id='" + cf.getParent().getID() + "']/CustomField]",
            null);
        

        assertNotNull(assigned);
        assertEquals(1, assigned.length);
        assertTrue(assigned[0] instanceof CustomFieldAssignment);
        CustomFieldAssignmentFixture.compareValues(acf.getValue(), ((CustomFieldAssignment)assigned[0]).getValue());
        WorkElementFixture.compare(parent,
            (WorkElement)((CustomFieldAssignment)assigned[0]).getParent(), null);        
        CustomFieldFixture.assertEquals(cf,
            ((CustomFieldAssignment)assigned[0]).getCustomField(), null);
    }

    public void testAssignAndUpdate() throws Exception
    {
        CustomField cf = CustomFieldFixture.createSaveCustomField(
                CustomFieldFixture.CF_WBS_SCOPE_CONTEXT, FieldInputType.Text,
                this);

        WorkElementScope scope = new WorkElementScope();
        scope.setCustomFieldAssignments(true);
        WorkElement object = (WorkElement) CustomFieldAssignmentFixture
                .createParent(CustomFieldAssignmentFixture.WBS_CONTEXT, this);

        //assigned custom field with the default value
        CustomFieldAssignment cfa = new CustomFieldAssignment();
        cfa.setCustomField(cf);
        cfa.setValue(cf.getDefaultValue());
        object.setCustomFieldAssignments(new CustomFieldAssignment[]{cfa});
        
        save(object, scope);

        WorkElement loaded = (WorkElement) loadFromID(object, scope);

        assertNotNull(loaded.getCustomFieldAssignments());
        assertEquals(1, loaded.getCustomFieldAssignments().length);
        assertEquals(loaded.getCustomFieldAssignments()[0].getValue(), cf.getDefaultValue());
       
        
        //update customfield with a null value
        object = loaded;
        object.getCustomFieldAssignments()[0].setValue(null);
 
        save(object, scope);

        loaded = (WorkElement) loadFromID(object, scope);

        assertNotNull(loaded.getCustomFieldAssignments());
        assertEquals(1, loaded.getCustomFieldAssignments().length);
        assertEquals(loaded.getCustomFieldAssignments()[0].getValue(), null);
        
        //update customfield with a null value
        object = loaded;
        object.getCustomFieldAssignments()[0].setValue("ABC");
 
        save(object, scope);

        loaded = (WorkElement) loadFromID(object, scope);

        assertNotNull(loaded.getCustomFieldAssignments());
        assertEquals(1, loaded.getCustomFieldAssignments().length);
        assertEquals(loaded.getCustomFieldAssignments()[0].getValue(), "ABC");
    }
}
