package com.ibm.rpm.client.junit.customfield.testcases.validation;

import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldAssignmentFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.customfield.constants.ValidationConstants;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.framework.constants.ErrorCodes;

public class TestCustomFieldAssignmentValidation extends GenericValidation
{
    private CustomFieldAssignment mainAssignment;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainAssignment = CustomFieldAssignmentFixture.createCustomFieldAssignment(
            CustomFieldAssignmentFixture.RESOURCE_CONTEXT, FieldInputType.Text,
            this);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithInvalidParent() throws Exception
    {
        DatafieldCategory wrongParent = new DatafieldCategory();
        wrongParent.setID("DUMMY_ID");
        mainAssignment.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE,
                mainAssignment.getClass().getName(), "parent",
                mainAssignment.getID())
        };

        verifySaveMustFail(mainAssignment, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws Exception
    {
        // Set field with maximum values.
        mainAssignment.setValue(getTestString(ValidationConstants.CUSTOMFIELDASSIGNMENT_VALUE_TEXT_MAX));

        doTestSuccessfulSave(mainAssignment);
    }

    public void testSuccessValidationNull() throws Exception
    {
        // Set field that accept null values.
        mainAssignment.setValue(null);

        doTestSuccessfulSave(mainAssignment);
    }

    public void testFailureValidation() throws Exception
    {
        // Set field with maximum values + 1.
        mainAssignment.setValue(getTestString(ValidationConstants.CUSTOMFIELDASSIGNMENT_VALUE_TEXT_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH,
                mainAssignment.getClass().getName(), "value",
                mainAssignment.getID()),
        };

        verifySaveMustFail(mainAssignment, null, expectedExceptions);
    }

    public void testFailureWithoutMandatoryFields() throws Exception
    {
        mainAssignment.setCustomField(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY,
                mainAssignment.getClass().getName(), "customField",
                mainAssignment.getID()),
        };

        verifySaveMustFail(mainAssignment, null, expectedExceptions);
    }

    public void testFailureWithoutParent() throws Exception
    {
        mainAssignment.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT,
                mainAssignment.getClass().getName(), "parent",
                mainAssignment.getID()),
        };

        verifySaveMustFail(mainAssignment, null, expectedExceptions);
    }

    public void testFailureWithoutParentID() throws Exception
    {
        mainAssignment.getParent().setID(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT_ID,
                mainAssignment.getClass().getName(), "parent",
                mainAssignment.getID()),
        };

        verifySaveMustFail(mainAssignment, null, expectedExceptions);
    }

    public void testFailureWithInvalidCustomField() throws Exception
    {

        CustomField cf = CustomFieldFixture.createSaveCustomField(
            CustomFieldFixture.CF_WBS_SCOPE_CONTEXT, FieldInputType.Text, this);

        mainAssignment.setCustomField(cf);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.customfield.constants.ErrorCodes.INVALID_CUSTOM_FIELD,
                mainAssignment.getClass().getName(), "customField",
                mainAssignment.getID()),
        };

        verifySaveMustFail(mainAssignment, null, expectedExceptions);
    }

    private void doTestSuccessfulSave(CustomFieldAssignment assignment)
            throws Exception
    {
        // Save.
        saveMustSucceed(assignment, CustomFieldAssignment.class, null);

        // Reload.
        CustomFieldAssignment reloadedAssignment = (CustomFieldAssignment) loadFromID(
            assignment, null);

        // Assert reloaded is equal to original.
        CustomFieldAssignmentFixture.compare(assignment, reloadedAssignment);
    }
}