/*
 * Created on Sep 22, 2005
 */
package com.ibm.rpm.client.junit.document.fixtures;

import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.DocumentState;
import com.ibm.rpm.applicationadministration.containers.NoteState;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RtfAssignmentFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceRoleAssignmentFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.document.types.ElementSecurityType;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class DocumentElementFixture extends RPMObjectFixture
{
    private static final int DEFAULT_NB_LEVELS = 1;

    private static final int DEFAULT_NB_CHILDREN_BY_LEVEL = 1;

    private static final int NB_ATTRIBUTE_ASSIGNMENTS = 2;

    private static RPMObject[] childrenGenericDocumentTypes = new RPMObject[]
    {
        //TODO activate all later
        //            new WPDoc(),
        //            new DelDoc(),
        new Document(),
    //            new Software(),
    //            new URL(),
    //            new NoteMinutes()
    };

	protected static ArrayList localSkipGetters = new ArrayList();
	protected static ArrayList localSkipSetters = new ArrayList();
	static
    {
    	//set skipGetters
	    localSkipGetters.addAll(skipGetters);
    	localSkipGetters.add("getMajorRevisionNumber");
    	localSkipGetters.add("getMinorRevisionNumber");
    	localSkipGetters.add("getRevisionNumber");
    	localSkipGetters.add("getElementNumber");//generated automatically
    	localSkipGetters.add("getLocked");  // TODO: see SP_LOCK_DOC Note in the DocumentsManager
    	localSkipGetters.add("getRUPLink");  // TODO: bug in client, see note in the DocumentsManager.FIELD_NAMES
    	localSkipGetters.add("getProcessLink");  // TODO: Same as RUPLink
        localSkipGetters.add("getLinkingStatus");
        localSkipGetters.add("getDocumentStatus");
        localSkipGetters.add("getElementSecurityType");
        localSkipGetters.add("getHostName");
        localSkipGetters.add("getCheckoutFolder");
        localSkipGetters.add("getHasAttachment");
        localSkipGetters.add("getDocumentSize");
        localSkipGetters.add("getDocumentCompressedSize");
        localSkipGetters.add("getLastUpdated");
        localSkipGetters.add("getCreationDate");
        localSkipGetters.add("getWorkflowCollaborationDate");
        localSkipGetters.add("getAssignedResources");
        localSkipGetters.add("getDocumentRank");
        localSkipGetters.add("getWbsSortingRank");
        localSkipGetters.add("getParentSortingRank");
        localSkipGetters.add("getTemplateSortingRank");
        

    	localSkipSetters.addAll(skipSetters);
    	localSkipSetters.add("setMajorRevisionNumber");//read only
    	localSkipSetters.add("setMinorRevisionNumber");//read only
    	localSkipSetters.add("setElementNumber");//read only
    	localSkipSetters.add("setElementSecurityType");//we set it ourselves not through HelperTestUtilities.setByIntrospection
    	localSkipSetters.add("setPublished");//we wet it ourselves not through HelperTestUtilities.setByIntrospection
        localSkipSetters.add("setLinkingStatus");
        localSkipSetters.add("setDocumentStatus");
        localSkipSetters.add("setElementSecurityType");
        localSkipSetters.add("setHostName");
        localSkipSetters.add("setCheckoutFolder");
        localSkipSetters.add("setHasAttachment");
        localSkipSetters.add("setDocumentSize");
        localSkipSetters.add("setDocumentCompressedSize");
        localSkipSetters.add("setLastUpdated");
        localSkipSetters.add("setLocked");//read only
        localSkipSetters.add("setCreationDate");
        localSkipSetters.add("getWorkflowCollaborationDate");
        localSkipSetters.add("setAssignedResources");
        localSkipSetters.add("setDocumentRank");
        localSkipGetters.add("setWbsSortingRank");
        localSkipGetters.add("setParentSortingRank");
        localSkipGetters.add("setTemplateSortingRank");
    }

    public static DocumentElement createSaveFixture(RPMObject parent, RPMObject rpmObject, ClientTestController ctc)
            throws Exception
    {
        return createSaveFixture(parent, rpmObject, DEFAULT_NB_LEVELS, DEFAULT_NB_CHILDREN_BY_LEVEL, ctc);
    }

    public static DocumentElement createSaveFixture(RPMObject parent, RPMObject rpmObject, int nbLevels, int nbChildrenByLevel, ClientTestController ctc)
    throws Exception
	{
		DocumentScope scope = createScope(rpmObject, 1, nbLevels, nbChildrenByLevel);
		DocumentElement docElement = createFixture(parent, rpmObject, nbChildrenByLevel, scope,
		        ctc);
		docElement = (DocumentElement) save(ctc, docElement, scope);
		return docElement;
	}
    
    public static DocumentElement createSaveFixture(RPMObject parent,
            RPMObject docObject, int nbChildren, DocumentScope scope,
            ClientTestController ctc) throws Exception
    {
        DocumentElement docElement = createFixture(parent, docObject,
                nbChildren, scope, ctc);
        docElement = (DocumentElement) save(ctc, docElement, scope);
        return docElement;
    }

    public static DocumentElement createFixture(RPMObject parent,
            RPMObject rpmObject, int nbChildrenByLevel, DocumentScope scope,
            ClientTestController ctc) throws Exception
    {
        return createDocumentElement(parent, rpmObject, random(),
                nbChildrenByLevel, scope, ctc);
    }

    private static DocumentElement instantiateDocumentElement(
            RPMObject rpmObject)
    {
        if (rpmObject instanceof DocumentFolder)
        {
            return new DocumentFolder();
        }
        else if (rpmObject instanceof WPDoc)
        {
            return new WPDoc();
        }
        else if (rpmObject instanceof DelDoc)
        {
            return new DelDoc();
        }
        else if (rpmObject instanceof Document)
        {
            return new Document();
        }
        else if (rpmObject instanceof Software)
        {
            return new Software();
        }
        else if (rpmObject instanceof UrlDocument)
        {
            return new UrlDocument();
        }
        else if (rpmObject instanceof NoteMinutes)
        {
            return new NoteMinutes();
        }
        else
        {
            Assert
                    .fail("DocumentElementFixture.instantiateDocumentElement : rpmObject "
                            + rpmObject + " not supported by this method.");
        }
        return null;
    }

    private static DocumentElement createDocumentElement(RPMObject parent,
            RPMObject rpmObject, int index, int nbChildrenByLevel,
            DocumentScope scope, ClientTestController ctc) throws Exception
    {
        DocumentElement docElement = instantiateDocumentElement(rpmObject);

        HelperTestUtilities.setByIntrospection(docElement, index,
                localSkipSetters);

        docElement.setParent(parent);

        if (isGenericDocument(docElement))
        {
            ((GenericDocument) docElement)
                    .setElementSecurityType(ElementSecurityType.ProjectLevel);

            if (isAbstractDocument(docElement))
            {
                ((AbstractDocument) docElement)
                        .setPublished(PublishedType.None);
            }
        }

        if (scope != null)
        {
            if (scope.getChildren() != null)
            {
                int numChildren = childrenGenericDocumentTypes.length;
                DocumentElement[] docElements = new DocumentElement[numChildren];

                int j = 0;
                while (j < numChildren)
                {
                    createChildren(parent, childrenGenericDocumentTypes[j],
                            nbChildrenByLevel, scope.getChildren(),
                            ctc, docElements, j++);
                }
                docElement.setChildren(docElements);
            }
        }
        //RTF - debug into how many elements it returns
        if (isNoteMinutes(docElement))
        { //only for NoteMinutes
            if (scope != null && scope.isRtfAssignments())
            {
                RtfAssignment[] rtfAssignments = RtfAssignmentFixture
                        .createAllAssignmentFixtures(docElement, ctc);
                ((NoteMinutes) docElement).setRtfAssignments(rtfAssignments);
            }
            
            //NoteMinutes State
            if(scope != null && scope.isState())
            {
            	NoteState noteState = (NoteState)DatafieldFixture.createState(docElement);
            	((NoteMinutes)docElement).setState(noteState);
            }
        }
        //attribute assignments
        if (isGenericDocument(docElement))
        {
            if (scope != null && scope.isAttributeAssignments())
            {
                AttributeAssignment[] attributeAssignments = new AttributeAssignment[NB_ATTRIBUTE_ASSIGNMENTS];
                for (int i = 0; i < NB_ATTRIBUTE_ASSIGNMENTS; i++)
                {
                    AttributeAssignment paa = AttributeAssignmentFixture
                            .createAssignmentFixture(docElement, ctc);
                    attributeAssignments[i] = paa;
                }
                ((GenericDocument) docElement)
                        .setAttributeAssignments(attributeAssignments);
            }
        }

        // Resource assignments
        if (scope != null && scope.isResourceAssignments()) {
        	ResourceRoleAssignment[] roleAssignments = new ResourceRoleAssignment[1];
        	roleAssignments[0] = ResourceRoleAssignmentFixture.createDocumentResourceRoleAssignment(docElement.getContextName(), docElement, ctc);
        	docElement.setResourceAssignments(roleAssignments);
        }
        
        // Document State
        if(isAbstractDocument(docElement)){
        	if(scope != null && scope.isState()) {
        		DocumentState documentState = (DocumentState)DatafieldFixture.createState(docElement);
        		((AbstractDocument)docElement).setState(documentState);
        	}
        }
        return docElement;
    }

    //update fields
    public static void updateFixtureFields(DocumentElement docElement,
            DocumentScope scope, ClientTestController ctc) throws Exception
    {
        updateFields(docElement, random(), scope, ctc);
    }

    //update structure
    public static void updateFixtureStructure(DocumentElement docElement,
            int deltaNbChildrenByLevel, DocumentScope scope,
            ClientTestController ctc) throws Exception
    {
        updateStructure(docElement, deltaNbChildrenByLevel, scope, ctc);
    }

    private static void updateStructure(DocumentElement docElement,
            int deltaNbChildrenByLevel, DocumentScope scope,
            ClientTestController ctc) throws Exception
    {
        if (scope != null)
        {
            //add new subfolders
            if (scope.getChildren() != null)
            {
                if (deltaNbChildrenByLevel > 0)
                {//add new elements
                    int numChildren = childrenGenericDocumentTypes.length;

                    DocumentElement[] docElements = new DocumentElement[numChildren];

                    int j = 0;
                    while (j < numChildren)
                    {
                        createChildren(docElement,
                                childrenGenericDocumentTypes[j],
                                deltaNbChildrenByLevel, scope
                                        .getChildren(), ctc,
                                docElements, j++);
                    }

                    ArrayList temp = mergeArrays(docElement
                            .getChildren(), docElements);
                    DocumentElement[] totalElements = new DocumentElement[temp
                            .size()];
                    temp.toArray(totalElements);
                    docElement.setChildren(totalElements);
                }
            }
        }
    }

    public static DocumentScope createScope(RPMObject rpmObject,
            int currentLevel, int maxLevels, int nbChildrenByLevel)
    {
        DocumentScope scope = new DocumentScope();
        if ((nbChildrenByLevel > 0 && currentLevel < maxLevels))
        {
            DocumentScope subScope = DocumentElementFixture.createScope(
                    rpmObject, currentLevel + 1, maxLevels, nbChildrenByLevel);
            subScope.setParent(scope);
            scope.setChildren(subScope);
        }
        //add attribute assignments to the scope
        scope.setAttributeAssignments(true);
        //add resource assignments to the scope
        scope.setResourceAssignments(true);
        //if noteMinutes, add rtf assignments
        if (isNoteMinutes(rpmObject))
        {
            scope.setRtfAssignments(true);
        }
        return scope;
    }

    private static void updateFields(DocumentElement docElement, int index,
            DocumentScope scope, ClientTestController ctc) throws Exception
    {
        HelperTestUtilities.setByIntrospection(docElement, index,
                localSkipSetters);
        if (scope != null)
        {
            if (scope.getChildren() != null)
            {
                if (docElement.getChildren() != null)
                {
                    for (int i = 0; i < docElement.getChildren().length; i++)
                    {
                        DocumentElementFixture.updateFixtureFields(docElement
                                .getChildren()[i], scope
                                .getChildren(), ctc);
                    }

                }
            }
        }
        // update rtf assignments
        if (isNoteMinutes(docElement))
        {
            if (scope != null && scope.isRtfAssignments())
            {
                RtfAssignment[] rtfAssignments = ((NoteMinutes) docElement)
                        .getRtfAssignments();
                if (rtfAssignments != null)
                {
                    for (int i = 0; i != rtfAssignments.length; ++i)
                    {
                        RtfAssignmentFixture.updateAssignmentFixture(rtfAssignments[i]);
                    }
                }
            }
        }
        //update attribute assignment - Generic Document
        if (isGenericDocument(docElement))
        {
            if (scope != null && scope.isAttributeAssignments())
            {
                AttributeAssignment[] attributeAssignments = ((GenericDocument) docElement)
                        .getAttributeAssignments();
                if (attributeAssignments != null)
                {
                    for (int i = 0; i < attributeAssignments.length; i++)
                    {
                        AttributeAssignmentFixture
                                .updateAssignmentFixture(attributeAssignments[i]);
                    }
                }
            }
        }

        // update resource assignments
        if (scope != null)
        {
            if (docElement.getResourceAssignments() != null)
            {
                ResourceRoleAssignment[] assignments = docElement
                        .getResourceAssignments();
                assignments = ResourceRoleAssignmentFixture
                        .removeAdminResourceAssignment(assignments);

                int children = 1;
                ResourceRoleAssignment[] rraArray = new ResourceRoleAssignment[children * 2];
                for (int i = 0; i < children; i++)
                {
                    //delete existing assignments
                    assignments[i].setDeleted(Boolean.TRUE);
                    rraArray[i] = assignments[i];

                    ResourceRoleAssignment rra = ResourceRoleAssignmentFixture
                            .createDocumentResourceRoleAssignment(docElement
                                    .getContextName(), docElement, ctc);
                    rraArray[children + i] = rra;
                }

                docElement.setResourceAssignments(rraArray);
            }
        }
    }

    private static void createChildren(RPMObject parent, RPMObject rpmObject,
            int nbChildrenByLevel, DocumentScope scope,
            ClientTestController ctc, DocumentElement[] docElements, int j)
            throws Exception
    {
        for (int i = 1; i <= nbChildrenByLevel; i++)
        {
            docElements[j] = DocumentElementFixture.createFixture(parent,
                    rpmObject, nbChildrenByLevel, scope, ctc);
            if (parent != null)
            {
                docElements[j].setParent(parent);
            }
        }
    }

    public static void compare(DocumentElement[] array1,
            DocumentElement[] array2) throws AssertionFailedError, Exception
    {
        compare(array1, array2, null);
    }

    public static void compare(DocumentElement[] array1,
            DocumentElement[] array2, DocumentScope scope)
            throws AssertionFailedError, Exception
    {

        sortWithComparator(array1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(array2, RPMObjectNameAndIDComparator.INSTANCE);

        RPMObjectFixture.compare(array1, array2);

        if (array1 != null && array2 != null)
        {
            for (int i = 0; i < array1.length; i++)
            {
                compare(array1[i], array2[i], scope);
            }
        }
    }

    public static void compare(DocumentElement obj1, DocumentElement obj2)
            throws AssertionFailedError, Exception
    {
        compare(obj1, obj2, null);
    }

    public static void compare(DocumentElement obj1, DocumentElement obj2,
            DocumentScope scope) throws AssertionFailedError, Exception
    {

        RPMObjectFixture.compareWithSkip(obj1, obj2, localSkipGetters);

        if (isDocumentElement(obj1))
        {
            DocumentElement element1 = obj1;
            DocumentElement element2 = obj2;

            if (scope != null && scope.getChildren() != null)
            {
                compare(element1.getChildren(), element2
                        .getChildren(), scope.getChildren());
            }
            if (scope != null && scope.isResourceAssignments())
            {
                ResourceRoleAssignmentFixture.compare(element1
                        .getResourceAssignments(), element2
                        .getResourceAssignments());
            }

            if (isGenericDocument(element1)) //attribute Assignment to be done
            // only for Generic Document
            {
                GenericDocument genericDocument1 = (GenericDocument) element1;
                GenericDocument genericDocument2 = (GenericDocument) element2;

                //compare element security type
                Assert.assertEquals(genericDocument1.getElementSecurityType(),
                        genericDocument2.getElementSecurityType());

                //compare the attribute assignments
                if (scope != null && scope.isAttributeAssignments())
                {
                    AttributeAssignment[] attributeAssignments1 = ((GenericDocument) element1)
                            .getAttributeAssignments();
                    AttributeAssignment[] attributeAssignments2 = ((GenericDocument) element2)
                            .getAttributeAssignments();
                    AttributeAssignmentFixture.compare(attributeAssignments1,
                            attributeAssignments2);
                }

                if (isNoteMinutes(genericDocument1))
                { //only for NoteMinutes
                    NoteMinutes noteMinutes1 = (NoteMinutes) genericDocument1;
                    NoteMinutes noteMinutes2 = (NoteMinutes) genericDocument2;

                    // compare the rtfAssignments
                    if (scope != null && scope.isRtfAssignments())
                    {
                        RtfAssignment[] rtfAssignments1 = noteMinutes1
                                .getRtfAssignments();
                        RtfAssignment[] rtfAssignments2 = noteMinutes2
                                .getRtfAssignments();
                        
                        Assert
                            .assertTrue((rtfAssignments1 == null && rtfAssignments2 == null)
                                    || (rtfAssignments1 != null
                                            && rtfAssignments2 != null && rtfAssignments1.length == rtfAssignments2.length));
                        
                        if (rtfAssignments1 != null
                                && rtfAssignments2 != null
                                && rtfAssignments1.length == rtfAssignments2.length)
                        {
                            RtfAssignment rtfAssignment1 = null;
                            RtfAssignment rtfAssignment2 = null;
                            
                            for (int i = 0; i != rtfAssignments1.length; ++i)
                            {
                                rtfAssignment1 = rtfAssignments1[i];
                                rtfAssignment2 = rtfAssignments2[i];
                                RtfAssignmentFixture.compare(rtfAssignment1,
                                    rtfAssignment2, false);
                            }
                        }
                    }
                    //compare NoteState
                    if (scope != null && scope.isState())
                    {
                        DatafieldFixture.compare(noteMinutes1.getState(),
                                noteMinutes2.getState());
                    }
                }
                else
                {//AbstractDocument
                    AbstractDocument abstractDocument1 = (AbstractDocument) genericDocument1;
                    AbstractDocument abstractDocument2 = (AbstractDocument) genericDocument2;

                    //compare NoteState
                    if (scope != null && scope.isState())
                    {
                        DatafieldFixture.compare(abstractDocument1.getState(),
                                abstractDocument2.getState());
                    }
                }
            }
        }
        else if (obj1 == null)
        {
            
        }
        else
        {
            Assert.fail("DocumentElementFixture.compare : class "
                    + obj1.getClass().getName() + " not supported.");
        }

    }

    private static boolean isDocumentElement(RPMObject rpmObject)
    {
        return rpmObject instanceof DocumentElement;
    }

    private static boolean isGenericDocument(RPMObject rpmObject)
    {
        return rpmObject instanceof GenericDocument;
    }

    private static boolean isAbstractDocument(RPMObject rpmObject)
    {
        return rpmObject instanceof AbstractDocument;
    }

    private static boolean isNoteMinutes(RPMObject rpmObject)
    {
        return rpmObject instanceof NoteMinutes;
    }

    private static ArrayList mergeArrays(Object[] array1, Object[] array2)
    {
        ArrayList temp = new ArrayList();
        addArray(temp, array1);
        addArray(temp, array2);
        return temp;
    }

    private static void addArray(ArrayList result, Object[] array)
    {
        if (array != null && array.length != 0)
        {
            for (int i = 0; i < array.length; i++)
            {
                result.add(array[i]);
            }
        }
    }

    public static DocumentFolder getParentDocumentFolder(RPMObject parent)
    {
        if (parent instanceof Asset)
        {
            return ((Asset) parent).getDocumentFolder();
        }
        else if (parent instanceof Resource)
        {
            return ((Resource) parent).getDocumentFolder();
        }
        else if (parent instanceof AbstractScope)
        {
            return ((AbstractScope) parent).getDocumentFolder();
        }
        else if (parent instanceof WorkElement)
        {
            return ((WorkElement) parent).getDocumentFolder();
        }
        else if (parent instanceof ClientCostCenter)
        {
            return ((ClientCostCenter) parent).getDocumentFolder();
        }
        else if (parent instanceof Pool)
        {
            return ((Pool) parent).getDocumentFolder();
        }
        else
        {
            Assert.assertNotNull(
                    "TestDocumentElement.getParentDocumentFolder is null",
                    parent);
            Assert
                    .fail("TestDocumentElement.getParentDocumentFolder : parent of type "
                            + parent.getClass().getName()
                            + " is not supported.");
        }
        return null;
    }

    public static RPMObject createSaveParent(RPMObject parent,
            ClientTestController ctc) throws Exception
    {
        RPMObject savedParent = null;

        DocumentScope documentScope = new DocumentScope();

        if (parent instanceof Asset)
        {
            AssetScope assetScope = new AssetScope();
            savedParent = AssetFixture.createSaveFixture(assetScope, ctc);
            assetScope.setDocumentFolder(documentScope);
            savedParent = ctc.loadFromID(savedParent, assetScope);
        }
        else if (parent instanceof Resource)
        {
            ResourceScope resourceScope = new ResourceScope();
            savedParent = ResourceFixture.createSaveFixture(resourceScope, ctc);
            resourceScope.setDocumentFolder(documentScope);
            savedParent = ctc.loadFromID(savedParent, resourceScope);
        }
        else if (parent instanceof WorkElement)
        {
            WorkElementScope scope = new WorkElementScope();
            scope.setDocumentFolder(documentScope);            
            WorkElement proposal = WorkElementFixture
            	.retrieveStaticFixture(new Proposal(), ctc);
            savedParent = ctc.loadFromID(proposal, scope);
        }
        else if (parent instanceof ClientCostCenter)
        {
            ClientCostCenterScope cccScope = new ClientCostCenterScope();
            savedParent = ClientCostCenterFixture.createSaveFixture(cccScope,
                    ctc);
            cccScope.setDocumentFolder(documentScope);
            savedParent = ctc.loadFromID(savedParent, cccScope);
        } else if(parent instanceof Pool){
            PoolScope poolScope = new PoolScope();
            Pool pool = PoolFixture.createFixture(false, 0, poolScope, ctc);
            savedParent = (Pool)save(ctc, pool, poolScope);
            poolScope.setDocumentFolder(documentScope);
            savedParent = ctc.loadFromID(savedParent, poolScope);
        }
        else if (parent instanceof Action)
        {
            ScopeFolder mainScopeFolder = ScopeElementFixture.createMainParent(ctc);
            savedParent = ScopeElementFixture.createActionFixture(null, 0, mainScopeFolder, ctc);
            savedParent = ctc.save(savedParent, null).getRpmObject();
            
            ScopeElementScope scope = new ScopeElementScope();   
            scope.setDocumentFolder(documentScope);
            savedParent = ctc.loadFromID(savedParent, scope);
        }
        else if (parent instanceof ChangeRequest)
        {
            ScopeFolder mainScopeFolder = ScopeElementFixture.createMainParent(ctc);
            savedParent = ScopeElementFixture.createChangeRequestFixture(null, 0, mainScopeFolder, ctc);
            savedParent = ctc.save(savedParent, null).getRpmObject();
            
            ScopeElementScope scope = new ScopeElementScope();
            scope.setDocumentFolder(documentScope);
            savedParent = ctc.loadFromID(savedParent, scope);
        }
        else if (parent instanceof Defect)
        {
            ScopeFolder mainScopeFolder = ScopeElementFixture.createMainParent(ctc);
            savedParent = ScopeElementFixture.createDefectFixture(null, 0, mainScopeFolder, ctc);
            savedParent = ctc.save(savedParent, null).getRpmObject();
            
            ScopeElementScope scope = new ScopeElementScope();
            scope.setDocumentFolder(documentScope);
            savedParent = ctc.loadFromID(savedParent, scope);
        }
        else if (parent instanceof Issue)
        {
            ScopeFolder mainScopeFolder = ScopeElementFixture.createMainParent(ctc);
            savedParent = ScopeElementFixture.createIssueFixture(null, 0, mainScopeFolder, ctc);
            savedParent = ctc.save(savedParent, null).getRpmObject();
            
            ScopeElementScope scope = new ScopeElementScope();
            scope.setDocumentFolder(documentScope);
            savedParent = ctc.loadFromID(savedParent, scope);
        }
        else if (parent instanceof Requirement)
        {
            ScopeFolder mainScopeFolder = ScopeElementFixture.createMainParent(ctc);
            savedParent = ScopeElementFixture.createRequirementFixture(null, 0, mainScopeFolder, ctc);
            savedParent = ctc.save(savedParent, null).getRpmObject();
            
            ScopeElementScope scope = new ScopeElementScope();
            scope.setDocumentFolder(documentScope);
            savedParent = ctc.loadFromID(savedParent, scope);
        }
        else if (parent instanceof Risk)
        {
            ScopeFolder mainScopeFolder = ScopeElementFixture.createMainParent(ctc);
            savedParent = ScopeElementFixture.createRiskFixture(null, 0, mainScopeFolder, ctc);
            savedParent = ctc.save(savedParent, null).getRpmObject();
            
            ScopeElementScope scope = new ScopeElementScope();
            scope.setDocumentFolder(documentScope);
            savedParent = ctc.loadFromID(savedParent, scope);
        }
        else if (parent instanceof ServiceRequest)
        {
            ScopeFolder mainScopeFolder = ScopeElementFixture.createMainParent(ctc);
            savedParent = ScopeElementFixture.createServiceRequestFixture(null, 0, mainScopeFolder, ctc);
            savedParent = ctc.save(savedParent, null).getRpmObject();
            
            ScopeElementScope scope = new ScopeElementScope();
            scope.setDocumentFolder(documentScope);
            savedParent = ctc.loadFromID(savedParent, scope);
        }
        else
        {
            logger
                    .debug("TestDocumentElement.createSaveParent : parent of type "
                            + parent.getClass().getName()
                            + " is not supported.");
        }
        return savedParent;
    }

}