/*
 * Created on Oct 26, 2005
 *
 */
package com.ibm.rpm.client.junit.document.fixtures;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.fixtures.AbstractModuleFixture;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.document.containers.DocumentModule;
import com.ibm.rpm.document.scope.DocumentModuleScope;

/**
 * @author karim
 *  
 */
public class DocumentModuleFixture extends AbstractModuleFixture
{
    public static DocumentModule createFixture(DocumentModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        return createModule(scope, ctc);
    }

    public static DocumentModule updateFixture(DocumentModule module,
            DocumentModuleScope scope, ClientTestController ctc) throws Exception
    {
        return updateModule(module, scope, ctc);
    }

    private static DocumentModule createModule(DocumentModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        DocumentModule module = new DocumentModule();

        if(scope!=null)
        {
            if(scope.getAttributeCategories()!=null)
            {
                //AttributeCategories
                AttributeCategory[] attributeCategories = (AttributeCategory[]) createElements(
                    scope.getAttributeCategories(), AttributeCategory.class, ctc);
                module.setAttributeCategories(attributeCategories);                
            }
            if(scope.getCustomFieldCategories()!=null)
            {
                CustomFieldCategory[] customFieldCategories = (CustomFieldCategory[]) createElements(
                        scope.getCustomFieldCategories(),
                        CustomFieldCategory.class, ctc);
                module.setCustomFieldCategories(customFieldCategories);
            }
        }

        return module;
    }

    private static DocumentModule updateModule(DocumentModule module,
            DocumentModuleScope scope, ClientTestController ctc) throws Exception
    {
        if (scope != null)
        {
            if(scope.getAttributeCategories()!=null)
            {
    		    //AttributeCategories
                AttributeCategory[] attributeCategories = (AttributeCategory[])updateElements(
                        module.getAttributeCategories(), scope
                                .getAttributeCategories(), ctc);
                module.setAttributeCategories(attributeCategories);
            }
            if(scope.getCustomFieldCategories()!=null)
            {
                //CustomFieldCategories
                module.setCustomFieldCategories(CustomFieldFixture
                    .modifyCustomFieldCategories(module.getCustomFieldCategories(),
                        scope.getCustomFieldCategories()));
            }

        }
        return module;
    }

    public static DocumentModuleScope createFullScope()
    {
        return createScope(true,true,true,true,true);
    }
    
    private static DocumentModuleScope createScope(
            boolean isAttributeCategories, boolean isDocumentStates,
            boolean isNoteStates, boolean isNoteMinutesRtf,
            boolean isCustomFields)
    {
        DocumentModuleScope scope = new DocumentModuleScope();

        if (isAttributeCategories)
        {
            scope.setAttributeCategories(new AttributeScope());
        }
        if (isDocumentStates)
        {
            scope.setDocumentStates(true);
        }
        if (isNoteStates)
        {
            scope.setNoteStates(true);
        }
        if (isNoteMinutesRtf)
        {
            scope.setNoteMinutesRtfs(true);
        }
        if (isCustomFields)
        {
            scope.setCustomFieldCategories(CustomFieldFixture
                .createCustomFieldScope(2));
        }

        return scope;
    }

    public static void compare(DocumentModule obj1, DocumentModule obj2,
            DocumentModuleScope scope) throws AssertionFailedError, Exception
    {
        if (scope != null)
        {
            if(scope.getAttributeCategories()!=null)
            {
                AttributeFixture.compare(obj1.getAttributeCategories(), obj2
                        .getAttributeCategories(), scope.getAttributeCategories());   
            }
            if(scope.isDocumentStates())
            {
                DatafieldFixture.compare(obj1.getDocumentStates(), obj2.getDocumentStates());
            }
            if(scope.isNoteStates())
            {
                DatafieldFixture.compare(obj1.getNoteStates(), obj2.getNoteStates());
            }
            if(scope.isNoteMinutesRtfs())
            {
                DatafieldFixture.compare(obj1.getNoteMinutesRtfs(), obj2.getNoteMinutesRtfs());
            }
            if(scope.getCustomFieldCategories()!=null)
            {
                CustomFieldFixture.assertIdEqual(obj1.getCustomFieldCategories(), obj2
                        .getCustomFieldCategories(), scope.getCustomFieldCategories());
            }
        }
    }
}
