package com.ibm.rpm.client.junit.document.testcases;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestDocumentElement extends ClientTestController
{
    private DocumentScope mainScope;

    private static final int NB_MAX_LEVELS = 2;

    private static final int NB_CHILDREN_BY_LEVEL = 1;

    private static final int DELTA_CHILDREN_BY_LEVEL = 1;

    // Global Variables
    RPMObject parent = null;

    DocumentElement documentElement = null;

    public TestDocumentElement()
    {
        // Default values
        parent = new Proposal();
        documentElement = new WPDoc();
    }

    public TestDocumentElement(String testName, RPMObject parent,
            DocumentElement documentElement)
    {
        setName(testName);
        this.parent = parent;
        this.documentElement = documentElement;
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        mainScope = DocumentElementFixture.createScope(documentElement, 1,
            NB_MAX_LEVELS, NB_CHILDREN_BY_LEVEL);

        // create/save a parent object (proposal, document, ...) depending on
        // the type of the parent sent to the constructor
        parent = DocumentElementFixture.createSaveParent(parent, this);

        // make sure the ID is null so that we create a new DocumentElement
        documentElement.setID(null);
    }

    public void testDocumentModifications() throws Exception
    {
        // write
        Document doc = new Document();
        doc = (Document) insertDocumentElement(doc, null, true);

        // read
        DocumentScope scope = new DocumentScope();
        scope.setAuthor(true);
        scope.setLastCheckedOutBy(true);
        scope.setContainingProject(new WorkElementScope());

        ArrayResult o = getApplicationInterface().loadFromXpath(sessionid,
            "/Document[@name='" + doc.getName() + "']", scope);
        Document result = (Document) o.getRpmObjectList()[0];

        assertEquals(result.getDocumentRank(), "N");
        assertNotNull(result.getLastCheckedOutBy());
        assertNotNull(result.getDocumentStatus());
        assertNotNull(result.getLinkingStatus());
        assertNotNull(result.getAssignedResources());
        assertNotNull(result.getAuthor());
        assertNotNull(result.getContainingProject());
        
        // since we create the document this should be always true.
        assertEquals(result.getAuthor(), result.getLastCheckedOutBy());

        if (result.getDocumentSize().intValue() < 0
                || result.getDocumentCompressedSize().intValue() < 0)
        {
            fail("Document size is negative");
        }
        
        if(result.getCreationDate().after(Calendar.getInstance()) || 
                result.getLastUpdated().after(Calendar.getInstance()))
        {
            fail("Document was created in the future.");
        }
    }  
    
    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        insert(documentElement, null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(documentElement, null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(documentElement, mainScope, true);

    }

    public void testInsertWithResourceAssignmentsLoadFromID()
            throws AssertionError, Exception
    {
        mainScope.setResourceAssignments(true);
        insert(documentElement, mainScope, true);

    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(documentElement, null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(documentElement, mainScope, true);
    }

    private void insert(DocumentElement documentElement, DocumentScope scope,
            boolean loadFromId) throws AssertionError, Exception
    {
        // insert a documentElement
        documentElement = insertDocumentElement(documentElement, scope,
            loadFromId);

        // delete the documentElement
        deleteDocumentElement(documentElement, loadFromId);
    }

    private DocumentElement insertDocumentElement(
            DocumentElement documentElement, DocumentScope scope,
            boolean loadFromId) throws Exception
    {
        RPMObject parentDocumentElement = DocumentElementFixture
            .getParentDocumentFolder(parent);
        documentElement = DocumentElementFixture.createFixture(
            parentDocumentElement, documentElement, NB_CHILDREN_BY_LEVEL,
            scope, this);

        // set visible to true
        if (documentElement instanceof GenericDocument)
        {
            ((GenericDocument) documentElement).setVisibleAtParent(new Boolean(
                true));
        }

        // Insert the DocumentElement into the DB
        SingleResult result = save(documentElement, scope);

        // compare the returned DocumentElement
        DocumentElement returnedDocumentElem = (DocumentElement) result
            .getRpmObject();
        DocumentElementFixture.compare(documentElement, returnedDocumentElem,
            scope);

        // reload the DocumentElement and compare it
        returnedDocumentElem = (DocumentElement) load(returnedDocumentElem,
            scope, loadFromId);
        DocumentElementFixture.compare(documentElement, returnedDocumentElem,
            scope);

        // return the DocumentElement
        return returnedDocumentElem;
    }

    private void update(DocumentElement documentElement, DocumentScope Scope,
            boolean loadFromId) throws AssertionError, Exception
    {
        // insert a DocumentElement
        documentElement = insertDocumentElement(documentElement, Scope,
            loadFromId);

        // update the DocumentElement
        documentElement = updateDocumentElementFields(documentElement, Scope,
            loadFromId);
        documentElement = updateDocumentElementStructure(documentElement,
            Scope, loadFromId);

        // delete the DocumentElement
        deleteDocumentElement(documentElement, loadFromId);
    }

    private DocumentElement updateDocumentElementFields(
            DocumentElement documentElement, DocumentScope scope,
            boolean loadFromId) throws Exception
    {
        return updateDocumentElement(documentElement, scope, loadFromId, false);
    }

    private DocumentElement updateDocumentElementStructure(
            DocumentElement documentElement, DocumentScope scope,
            boolean loadFromId) throws Exception
    {
        return updateDocumentElement(documentElement, scope, loadFromId, true);
    }

    private DocumentElement updateDocumentElement(
            DocumentElement documentElement, DocumentScope scope,
            boolean loadFromId, boolean updateStructure) throws Exception
    {
        if (updateStructure)
        {
            // updates the structure by adding elements
            DocumentElementFixture.updateFixtureStructure(documentElement,
                DELTA_CHILDREN_BY_LEVEL, scope, this);
        }
        else
        {
            // only updates the fields
            DocumentElementFixture.updateFixtureFields(documentElement, scope,
                this);
        }

        // Insert the updated DocumentElement into the DB
        SingleResult result = save(documentElement, scope);

        // compare the returned DocumentElement
        DocumentElement retDocumentElement = (DocumentElement) result
            .getRpmObject();
        DocumentElementFixture.compare(documentElement, retDocumentElement,
            scope);

        // reload the DocumentElement and compare it
        retDocumentElement = (DocumentElement) load(retDocumentElement, scope,
            loadFromId);
        DocumentElementFixture.compare(documentElement, retDocumentElement,
            scope);

        // return the DocumentElement
        return retDocumentElement;
    }

    private void deleteDocumentElement(DocumentElement documentElement,
            boolean loadFromId) throws Exception
    {

        // delete the DocumentElement
        delete(documentElement);

        // reload the deleted DocumentElement
        DocumentElement deletedDocumentElem = loadDeletedDocumentElem(
            documentElement, loadFromId);

        // Validate the deleted DocumentElement
        assertNull(deletedDocumentElem);
    }

    private DocumentElement loadDeletedDocumentElem(
            DocumentElement documentElement, boolean loadFromId)
            throws Exception
    {
        try
        {
            documentElement = (DocumentElement) load(documentElement, null,
                loadFromId);
            return documentElement;
        }
        catch (Exception e)
        {
            // DocumentElement does not exist
        }
        return null;
    }

    private RPMObject load(DocumentElement documentElement,
            DocumentScope scope, boolean loadFromId) throws Exception
    {
        RPMObject obj = null;

        if (loadFromId)
        {
            // load from an id
            if (documentElement instanceof WPDoc)
            {
                obj = new WPDoc();
            }
            else if (documentElement instanceof DelDoc)
            {
                obj = new DelDoc();
            }
            else if (documentElement instanceof Document)
            {
                obj = new Document();
            }
            else if (documentElement instanceof NoteMinutes)
            {
                obj = new NoteMinutes();
            }
            else if (documentElement instanceof Software)
            {
                obj = new Software();
            }
            else if (documentElement instanceof UrlDocument)
            {
                obj = new UrlDocument();
            }
            else if (documentElement instanceof DocumentFolder)
            {
                obj = new DocumentFolder();
            }
            else
            {
                logger.debug("error :Invalid Element for Document ");
            }

            obj.setID(documentElement.getID());
            obj.setContextName(documentElement.getContextName());

            return loadFromID(obj, scope);
        }
        else
        {
            // load from an xpath
            String containerName = StringUtil.getShortClassName(documentElement
                .getClass());
            RPMObject[] objs = loadFromXpath("/" + containerName + "[id='"
                    + documentElement.getID() + "']", scope);
            if (objs == null || objs.length == 0)
            {
                return null;
            }
            else
            {
                return objs[0];
            }
        }
    }

}