package com.ibm.rpm.client.junit.document.testcases;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.wbs.containers.WorkElement;

/**
 * @author dexmar00
 * @author ikusakov
 */
public class TestDocumentTransfer extends ClientTestController
{
    private static final int BUFFER_SIZE = 2048;

    private static final String CLIENT_CONTEXT = "Client";

    private static final String ASSET_CONTEXT = "Asset";

    private static final String SCOPE_CONTEXT = "Scope";

    private static final String WBS_CONTEXT = "WBS";

    private static final String RESOURCE_CONTEXT = "Resource";

    private static final String POOL_CONTEXT = "Pool";

    private static final int NB_CHILDREN_BY_LEVEL = 1;

    private static final String FILE_NAME = "Test.doc";

    // see setUp() method
    private static String SERVLET_URL = null;

    // some garbage content
    private static byte TEMP_FILE_CONTENT[] = new byte[0];
    
    private static byte TEMP_FILE_ZIPPED[] = new byte[0];
    
    private long timestamp = System.currentTimeMillis();
    
    static
    {
        // Creating a zipped version of temporary file
        try
        {
            InputStream systemResourceAsStream = ClassLoader
                .getSystemResourceAsStream(FILE_NAME);

            TEMP_FILE_CONTENT = new byte[systemResourceAsStream.available()];
            BufferedInputStream bis = new BufferedInputStream(
                systemResourceAsStream);

            bis.read(TEMP_FILE_CONTENT);
            
            systemResourceAsStream.close();
            bis.close();
            
            File f = File.createTempFile("rpmupload", "ziptmp.zip");
            FileOutputStream fos = new FileOutputStream(f);
            ZipOutputStream zipOutputStream = new ZipOutputStream(fos);
            ZipEntry entry = new ZipEntry(FILE_NAME);
            entry.setMethod(ZipEntry.DEFLATED);
            entry.setSize(TEMP_FILE_CONTENT.length);
            zipOutputStream.putNextEntry(entry);
            for (int i = 0; i < TEMP_FILE_CONTENT.length; i++)
            {
                zipOutputStream.write(TEMP_FILE_CONTENT[i]);
            }
            zipOutputStream.flush();
            entry.setSize(TEMP_FILE_CONTENT.length);
            zipOutputStream.closeEntry();
            zipOutputStream.finish();
            zipOutputStream.close();
            fos.close();
            TEMP_FILE_ZIPPED = new byte[(int) f.length()];
            java.io.FileInputStream fis = new java.io.FileInputStream(f);
            int available = fis.available();
            fis.read(TEMP_FILE_ZIPPED);
            fis.close();
            f.delete();
        }
        catch (Exception e)
        {
            logger.error("ERROR: cannot create temporary zip file: " + e);
        }
    }

    HttpURLConnection httpConnection = null;

    public void setUp() throws Exception
    {
        super.setUp();
        if (this.isRemote())
        {
            // converting: http://localhost:8080/rpm/services
            // to: http://localhost:8080/rpm/servlet/DocumentTransfer
            SERVLET_URL = this.getRemoteURL();
            if (SERVLET_URL != null)
            {
                SERVLET_URL = SERVLET_URL.replaceFirst("/services/?$",
                        "/servlet/DocumentTransfer");
            }
        assertTrue("Zipped array size should be > 0",
                TEMP_FILE_ZIPPED.length > 0);
    }
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
        // connection will be closed anyway
        if (httpConnection != null)
        {
            httpConnection.disconnect();
        }
    }

    public void testUploadDownloadWbs() throws Exception
    {
        caseGenericTest(WorkElement.class, WBS_CONTEXT);
    }

    public void testUploadDownloadScope() throws Exception
    {
        caseGenericTest(Action.class, SCOPE_CONTEXT);
    }

    public void testUploadDownloadPool() throws Exception
    {
        caseGenericTest(Pool.class, POOL_CONTEXT);
    }

    public void testUploadDownloadResource() throws Exception
    {
        caseGenericTest(Resource.class, RESOURCE_CONTEXT);
    }

    public void testUploadDownloadAsset() throws Exception
    {
        caseGenericTest(Asset.class, ASSET_CONTEXT);
    }

    public void testUploadDownloadClientCostCenter() throws Exception
    {
        caseGenericTest(ClientCostCenter.class, CLIENT_CONTEXT);
    }

    public void validateBadRequests(String contextName,
            DocumentElement document) throws Exception
    {
        if (!this.isRemote())
        {
            logger.info("Skipping local test");
            return;
        }

        validateBadActions(contextName, document);
        
        validateBadRequestOnDownloadAction(contextName, document, "DOWNLOADZIPPED");
        validateBadRequestOnDownloadAction(contextName, document, "DOWNLOADUNZIPPED");
        validateBadRequestOnUploadAction(contextName, document, "UPLOADZIPPED");
        validateBadRequestOnUploadAction(contextName, document, "UPLOADUNZIPPED");
    }
    
    private static class ParameterCheck
    {
        public String parameterName;
        public String validValue;
        public String invalidValue;
        public int errorOnInvalid = HttpURLConnection.HTTP_BAD_REQUEST;
        public String errorStringOnInvalid;
        public int errorOnMissing = HttpURLConnection.HTTP_BAD_REQUEST;
        public String errorStringOnMissing;
    }
    
    public void fillDownloadParameterList(List listToFill, String actionName, String contextName,
            DocumentElement document)
    {
        ParameterCheck addParam;
        
        addParam = new ParameterCheck();
        addParam.parameterName = "?ACTION";
        addParam.validValue = actionName;
        addParam.invalidValue = "JUNK";
        addParam.errorStringOnInvalid = "action";
        addParam.errorStringOnMissing = null;
        addParam.errorOnMissing=HttpURLConnection.HTTP_NOT_FOUND;
        listToFill.add(addParam);

        addParam = new ParameterCheck();
        addParam.parameterName = "&SESSION_ID";
        addParam.validValue = this.getSessionID();
        addParam.invalidValue = "82828282828";
        addParam.errorStringOnInvalid = "session";
        addParam.errorOnInvalid=HttpURLConnection.HTTP_UNAUTHORIZED;
        addParam.errorStringOnMissing = "session";
        listToFill.add(addParam);

        addParam = new ParameterCheck();
        addParam.parameterName = "&DOCUMENT_ID";
        addParam.validValue = document.getID();
        addParam.invalidValue = "23098723074";
        addParam.errorStringOnInvalid = "document";
        addParam.errorOnInvalid=HttpURLConnection.HTTP_NOT_FOUND;
        addParam.errorStringOnMissing = "document";
        listToFill.add(addParam);

        addParam = new ParameterCheck();
        addParam.parameterName = "&CONTEXT_NAME";
        addParam.validValue = contextName;
        addParam.invalidValue = "JUNK";
        addParam.errorStringOnInvalid = "context";
        addParam.errorStringOnMissing = "context";
        listToFill.add(addParam);
    }
    
    public void fillUploadParameterList(List listToFill, String actionName, String contextName,
            DocumentElement document)
    {
        ParameterCheck addParam;
        
        fillDownloadParameterList( listToFill, actionName, contextName, document);

        addParam = new ParameterCheck();
        addParam.parameterName = "&FILE";
        addParam.validValue = FILE_NAME;
        addParam.invalidValue = "JUNK**./.^??TEST";
        addParam.errorStringOnInvalid = "file";
        addParam.errorStringOnMissing = "file";
        listToFill.add(addParam);

    }
    
    private void validateBadRequestOnDownloadAction(String contextName,
            DocumentElement document, String actionToValidate) throws Exception
    {
        List parameterList = new ArrayList();
        fillDownloadParameterList(parameterList, actionToValidate, contextName,
            document);

        Iterator it = parameterList.iterator();
        while (it.hasNext())
        {
            ParameterCheck element = (ParameterCheck) it.next();
            validateParameter(element, parameterList);
        }
    }

    private void validateBadRequestOnUploadAction(String contextName,
            DocumentElement document, String actionToValidate) throws Exception
    {
        List parameterList = new ArrayList();
        fillUploadParameterList(parameterList, actionToValidate, contextName,
            document);

        Iterator it = parameterList.iterator();
        while (it.hasNext())
        {
            ParameterCheck element = (ParameterCheck) it.next();
            validateParameter(element, parameterList);
        }
    }
    
    private void validateParameter(ParameterCheck parameterToCheck,
            List parameterList) throws Exception
    {
        // test missing parameter
        StringBuffer request = new StringBuffer();
        request.append(SERVLET_URL);

        Iterator it = parameterList.iterator();
        while (it.hasNext())
        {
            ParameterCheck element = (ParameterCheck) it.next();
            if (element != parameterToCheck)
            {
                request.append(element.parameterName);
                request.append("=");
                request.append(element.validValue);
            }
        }

        validateBadRequest(request.toString(), parameterToCheck.errorOnMissing,
            parameterToCheck.errorStringOnMissing);

        // test invalid parameter
        request = new StringBuffer();
        request.append(SERVLET_URL);

        it = parameterList.iterator();
        while (it.hasNext())
        {
            ParameterCheck element = (ParameterCheck) it.next();
            if (element != parameterToCheck)
            {
                request.append(element.parameterName);
                request.append("=");
                request.append(element.validValue);
            }
            else
            {
                request.append(element.parameterName);
                request.append("=");
                request.append(element.invalidValue);
            }
        }

        validateBadRequest(request.toString(), parameterToCheck.errorOnInvalid,
            parameterToCheck.errorStringOnInvalid);
    }

    /**
     * @param contextName
     * @param document
     * @throws Exception
     */
    private void validateBadActions(String contextName, DocumentElement document) throws Exception
    {
        // bad request
        validateBadRequest(SERVLET_URL, HttpURLConnection.HTTP_BAD_REQUEST,  "action");
        
        // incorrect action
        validateBadRequest(SERVLET_URL + "?ACTION=CALAVERASYDIABLITOS"
                + "&SESSION_ID=" + this.getSessionID()
                + "&DOCUMENT_ID=" + document.getID()
                + "&CONTEXT_NAME=" + contextName,
                HttpURLConnection.HTTP_BAD_REQUEST, "action");
    }

    private void validateBadRequest(String getURL, int expectedHttpResponse, String subErrorString) throws Exception
    {
        logger.info("* Testing: bad url: " + getURL);
        URL url = new URL(getURL);
        HttpURLConnection httpConnection = (HttpURLConnection) url
                .openConnection();
        httpConnection.setRequestMethod("GET");
        httpConnection.connect();
        String responseMessage = httpConnection.getResponseMessage();
        int responseCode = httpConnection.getResponseCode();
        
        assertEquals("HTTP " + responseCode + " " + responseMessage,
                expectedHttpResponse, responseCode);
        
        if (subErrorString != null )
        {
            boolean stringDetected = responseMessage
                .toLowerCase().indexOf(subErrorString) != -1;
            
            assertTrue("Expected stubstring:" + subErrorString
                    + ", in error message:" + responseMessage, stringDetected);
        }
    }

    private void caseGenericTest(Class parentDocumentClass, String contextName)
            throws Exception
    {
        if (!this.isRemote())
        {
            logger.info("Skipping local test");
            return;
        }

        logger.info("=== Testing: "
                + StringUtil.getShortClassName(parentDocumentClass));

        DocumentElement zipDocument = createSaveDocument(parentDocumentClass);

        DocumentElement unzippedDocument = createSaveDocument(parentDocumentClass);

        logTimestamp("testSetup");
        validateEmptyDownload(contextName, zipDocument);
        logTimestamp("validateEmptyDownload");
        validateZipUpload(contextName, zipDocument);
        logTimestamp("validateZipUpload");
        validateUnzippedUpload(contextName, unzippedDocument);
        logTimestamp("validateUnzippedUpload");
        validateDownloadUsingGet(contextName, zipDocument);
        logTimestamp("validateDownloadUsingGet");
        validateBadRequests(contextName, zipDocument);
        logTimestamp("validateBadRequests");
        
    }

    private void validateDownloadUsingGet(String contextName,
            DocumentElement zipDocument) throws Exception
    {
        String getURL = SERVLET_URL + "?ACTION=DOWNLOADZIPPED"
                + "&DOCUMENT_ID=" + zipDocument.getID() + "&CONTEXT_NAME="
                + contextName + "&SESSION_ID=" + this.getSessionID();
        logger.info("* Testing: GET " + getURL);
        URL url = new URL(getURL);
        HttpURLConnection httpConnection = (HttpURLConnection) url
                .openConnection();
        // Enable the use of the http output stream to send the file
        httpConnection.setDoOutput(true);
        httpConnection.setDoInput(true);
        httpConnection.setUseCaches(false);
        httpConnection.setRequestMethod("GET");
        caseDownload(httpConnection, TEMP_FILE_CONTENT, zipDocument, true);
        httpConnection.disconnect();
        httpConnection = null;
    }

    /**
     * @param contextName
     * @param zipDocument
     * @throws Exception
     */
    private void validateEmptyDownload(String contextName,
            DocumentElement document) throws Exception
    {
        httpConnection = createConnection("DOWNLOADUNZIPPED", document.getID(),
            contextName);

        httpConnection.connect();
        String responseMessage = httpConnection.getResponseMessage();
        int responseCode = httpConnection.getResponseCode();

        assertEquals("HTTP " + responseCode + " " + responseMessage,
                HttpURLConnection.HTTP_NO_CONTENT, responseCode);

        httpConnection.disconnect();
    }

    /**
     * @param contextName
     * @param document
     * @throws Exception
     */
    private void validateUnzippedUpload(String contextName,
            DocumentElement document) throws Exception
    {
        httpConnection = createConnection("UPLOADUNZIPPED", document.getID(),
            contextName);
        caseUpload(httpConnection, TEMP_FILE_CONTENT);
        httpConnection.disconnect();
        logTimestamp("validateUnzippedUpload");
        validateDownloadZipAndUnzipped(contextName, document);
    }

    /**
     * @param contextName
     * @param document
     * @throws Exception
     */
    private void validateZipUpload(String contextName, DocumentElement document)
            throws Exception
    {
        httpConnection = createConnection("UPLOADZIPPED", document.getID(),
            contextName);
        caseUpload(httpConnection, TEMP_FILE_ZIPPED);
        httpConnection.disconnect();
        logTimestamp("validateZipUpload");
        validateDownloadZipAndUnzipped(contextName, document);
    }

    /**
     * @param contextName
     * @param document
     * @throws Exception
     */
    private void validateDownloadZipAndUnzipped(String contextName,
            DocumentElement document) throws Exception
    {
        httpConnection = createConnection("DOWNLOADZIPPED", document.getID(),
            contextName);
        caseDownload(httpConnection, TEMP_FILE_CONTENT, document, true);
        httpConnection.disconnect();
        logTimestamp("DOWNLOADZIPPED");

        
        httpConnection = createConnection("DOWNLOADUNZIPPED", document.getID(),
            contextName);
        caseDownload(httpConnection, TEMP_FILE_CONTENT, document, false);
        httpConnection.disconnect();
        logTimestamp("DOWNLOADUNZIPPED");
    }

    private void caseUpload(HttpURLConnection httpConnection, byte[] uploadArray)
            throws Exception
    {
        // we create and upload a temporary file
        File tempFile = File.createTempFile("rpmupload", "ziptmp.zip");
        FileOutputStream fos = new FileOutputStream(tempFile);
        fos.write(uploadArray);
        fos.close();

        RandomAccessFile f = new RandomAccessFile(tempFile, "r");
        if (f.length() > 2147483647)
        {
            // Real file size limit is dependant on the database setup.
            // Please refer to your database administrator.
            throw new Exception("Temporary file is too large.");
        }
        int contentLength = (int) f.length(); // Safe for file length <=
        // 2,147,483,647 bytes

        // Name of the file sent is required for upload of unzipped attachment.
        httpConnection.setRequestProperty("FILE", FILE_NAME);

        BufferedOutputStream bos = null;
        if (contentLength > 0)
        {
            httpConnection.setRequestProperty("CONTENT_LENGTH", Integer
                    .toString(contentLength));
            httpConnection.connect();
            bos = new BufferedOutputStream(httpConnection.getOutputStream());
            FileInputStream fis = new FileInputStream(f.getFD());
            int count;
            byte data[] = new byte[BUFFER_SIZE];
            int total = 0;
            while ((count = fis.read(data, 0, BUFFER_SIZE)) != -1)
            {
                bos.write(data, 0, count);
                total += count;
            }
            bos.flush();
            bos.close();
            fis.close();
        }

        // Get the result code
        String responseMessage = httpConnection.getResponseMessage();
        int responseCode = httpConnection.getResponseCode();

        tempFile.delete();

        assertEquals("HTTP " + responseCode + " " + responseMessage,
                HttpURLConnection.HTTP_OK, responseCode);
    }

    private void caseDownload(HttpURLConnection httpConnection,
            byte[] compareArray, DocumentElement document, boolean unzipFirst)
            throws Exception
    {
        // This is an example with the "get" method URL url = new
        //  URL("http://localhost:8080/rpm/servlet/DocumentTransfer?ACTION=DOWNLOADUNZIPPED&SESSION_ID=1128360214390&DOCUMENT_ID=41340DF17866436EA11451BF9A6E0D19&CONTEXT_NAME=WBS");
        // //
        httpConnection.connect();
        String responseMessage = httpConnection.getResponseMessage();
        int responseCode = httpConnection.getResponseCode();

        assertEquals("HTTP " + responseCode + " " + responseMessage,
                HttpURLConnection.HTTP_OK, responseCode);

        // result is OK(200), get the stream and write the file to some path
        // on thefile system
        String contentDisposition = httpConnection
                .getHeaderField("Content-Disposition");
        String contentType = httpConnection.getContentType();
        String contentEncoding = httpConnection.getContentEncoding();
        int contentLength = httpConnection.getContentLength();
        if (contentLength > 0)
        {
            String delimiter = "filename=\"";
            String filename = contentDisposition.substring(delimiter.length()
                    + contentDisposition.indexOf(delimiter), contentDisposition
                    .lastIndexOf('\"'));

            BufferedInputStream bis = new BufferedInputStream(httpConnection
                    .getInputStream());
            
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            
            boolean remaningRead = true;
            int readSize = 0;
            byte data[] = new byte[BUFFER_SIZE];
            byte destinationArray[] = null;
            
            if (unzipFirst)
            {
                ZipInputStream zis = new ZipInputStream(bis);
                
                ZipEntry entry = zis.getNextEntry();
                if (entry != null)
                {
                    while ((readSize = zis.read(data, 0, BUFFER_SIZE)) != -1)
                    {
                        baos.write(data, 0, readSize);
                    }
                }
            }
            else
            {
                while ((readSize = bis.read(data, 0, BUFFER_SIZE)) != -1)
                {
                    baos.write(data, 0, readSize);
                }
            }
            destinationArray = baos.toByteArray();
            
            assertEquals("Array size should match.",
                compareArray.length, destinationArray.length);

            for (int i = 0; i < compareArray.length; i++)
            {
                assertEquals("Reading file byte " + i, compareArray[i],
                    destinationArray[i]);
            }
        }
        else
        {
            fail("Received 0 content length");
        }
        
        Document loadedDocument = (Document) loadFromID(document, null);
        
        // TODO validate the document size is ok when the document structure
        // contains this information
    }

    private HttpURLConnection createConnection(String action, String docID,
            String contextName) throws Exception
    {
        logger.debug("* Testing: " + action);
        URL url = new URL(SERVLET_URL);
        HttpURLConnection httpConnection = (HttpURLConnection) url
                .openConnection();
        // Enable the use of the http output stream to send the file
        httpConnection.setDoOutput(true);
        httpConnection.setDoInput(true);
        httpConnection.setUseCaches(false);
        httpConnection.setRequestMethod("POST");
        // Action to do,
        //		UPLOADZIPPED=save a file that is already zipped by the client to the
        // blob column
        //		UPLOADUNZIPPED=the server will zip the file and save to the blob
        // column
        httpConnection.setRequestProperty("ACTION", action);

        // Session ID provided by the Authenticate Web Service
        httpConnection.setRequestProperty("SESSION_ID", this.getSessionID());

        // This DOCUMENT_ID must obtained by calling the DocumentManager Web
        // Service
        httpConnection.setRequestProperty("DOCUMENT_ID", docID);

        // This is the document context. The DocumentManager return this
        // information as part of Document RPMObject
        // Document Context Name
        httpConnection.setRequestProperty("CONTEXT_NAME", contextName);

        return httpConnection;
    }

    private DocumentElement createSaveDocument(Class documentParentClass)
            throws Exception
    {
        RPMObject parent = (RPMObject) documentParentClass.newInstance();
        parent = DocumentElementFixture.createSaveParent(parent, this);
        DocumentElement document = DocumentElementFixture.createFixture(
                DocumentElementFixture.getParentDocumentFolder(parent),
                new Document(), NB_CHILDREN_BY_LEVEL, null, this);
        return (DocumentElement) save(document, null).getRpmObject();
    }
    
    public void logTimestamp(String phase)
    {
        long newStamp = System.currentTimeMillis();
        long sec = (newStamp - timestamp)/1000;
        logger.info("exec time : " + sec + "s : " + phase);
        timestamp = newStamp;
    }
}