package com.ibm.rpm.client.junit.document.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test63305 extends ClientTestController {
	
    public void testCreateUrlWithRupLink() throws Exception
    {
        WorkElementScope workElementScope = createScope();
        Proposal proposal = createProposal(workElementScope);
        
        UrlDocument savedUrl = createUrlWithRupAndProcessLinks(proposal,workElementScope, true);
        
        assertNotNull(savedUrl);
        assertNotNull(savedUrl.getRUPLink());
        assertNotNull(savedUrl.getProcessLink());
        
        assertEquals(true, savedUrl.getRUPLink().booleanValue());
        assertEquals(false, savedUrl.getProcessLink().booleanValue());
    }

    
	public void testCreateUrlWithProcessLink() throws Exception
	{
		WorkElementScope workElementScope = createScope();
		Proposal proposal = createProposal(workElementScope);
		
		UrlDocument savedUrl = createUrlWithRupAndProcessLinks(proposal,workElementScope, false);
		
		assertNotNull(savedUrl);
		assertNotNull(savedUrl.getRUPLink());
		assertNotNull(savedUrl.getProcessLink());
		
		assertEquals(false, savedUrl.getRUPLink().booleanValue());
		assertEquals(true, savedUrl.getProcessLink().booleanValue());
	}
	
	
	public void testUpdateUrlRupAndProcessLinks() throws Exception
	{
		WorkElementScope workElementScope = createScope();
		Proposal proposal = createProposal(workElementScope);
		
		UrlDocument savedUrl = createUrlWithRupAndProcessLinks(proposal,workElementScope, true);
		
        savedUrl = updateAndValidateUrlWithRupAndProcessLinks(savedUrl, false);
        
        savedUrl = updateAndValidateUrlWithRupAndProcessLinks(savedUrl, true);
	}
	
	private UrlDocument createUrlWithRupAndProcessLinks(Proposal proposal, WorkElementScope workElementScope, boolean isSetRUPLink) throws Exception
	{
		UrlDocument url = new UrlDocument();
		url.setName("Proposal URL");

        if(isSetRUPLink)
            url.setRUPLink(Boolean.TRUE);
        else
            url.setProcessLink(Boolean.TRUE);

		proposal.getDocumentFolder().setChildren(new DocumentElement[] {url});
		
		SaveResult saveResult = save(proposal,workElementScope);
		proposal = (Proposal)saveResult.getRpmObject();
		
		validate(proposal);
		
		return (UrlDocument)proposal.getDocumentFolder().getChildren()[0];
	}
	
    private UrlDocument updateAndValidateUrlWithRupAndProcessLinks(UrlDocument urlDocument, boolean isSetRUPLink) throws Exception
    {
        urlDocument.setRUPLink(new Boolean(isSetRUPLink));
        urlDocument.setProcessLink(new Boolean(!isSetRUPLink));
        
//        assertNotNull(urlDocument.getParent());
        
        SaveResult saveResult = save(urlDocument,null);
        UrlDocument savedUrl = (UrlDocument)saveResult.getRpmObject();
        
        assertNotNull(savedUrl);
        assertNotNull(savedUrl.getRUPLink());
        assertNotNull(savedUrl.getProcessLink());
        
        assertEquals(isSetRUPLink, savedUrl.getRUPLink().booleanValue());
        assertEquals(!isSetRUPLink, savedUrl.getProcessLink().booleanValue());
        
        return savedUrl;
    }
    
	private void validate(Proposal proposal)
	{
		assertNotNull(proposal);
		assertNotNull(proposal.getDocumentFolder());
		assertNotNull(proposal.getDocumentFolder().getChildren());
		assertEquals(proposal.getDocumentFolder().getChildren().length,1);
		assertNotNull(proposal.getDocumentFolder().getChildren()[0]);
	}
	
	private WorkElementScope createScope()
	{
		WorkElementScope workElementScope = new WorkElementScope();
		DocumentScope documentScope = new DocumentScope();
		documentScope.setChildren(documentScope);
		workElementScope.setDocumentFolder(documentScope);
		return workElementScope;
	}
	
	private Proposal createProposal(WorkElementScope workElementScope)throws Exception
	{
		Proposal proposal = new Proposal();
		proposal.setName("WS_PROP_DEF_63305");
		
		SaveResult saveResult = save(proposal,workElementScope);
		proposal = (Proposal)saveResult.getRpmObject();
		
		assertNotNull(proposal);
		assertNotNull(proposal.getDocumentFolder());
		
		return proposal;
	}

}
