package com.ibm.rpm.client.junit.document.testcases.fixes;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.applicationadministration.types.SelectType;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.DocumentModule;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.document.scope.DocumentModuleScope;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestRATLC00307395 extends ClientTestController {
	
	private Attribute attribute  = null;
	
	protected void setUp() throws Exception
	{
		super.setUp();
		createSaveAttributes();
	}
	
	public void testDelDoc() throws Exception
	{
		runTestForDocumentElement(new DelDoc());
	}
	
	public void testDocument() throws Exception
	{
		runTestForDocumentElement(new Document());
	}
	
	public void testSoftware() throws Exception
	{
		runTestForDocumentElement(new Software());
	}
	
	public void testUrlDocument() throws Exception
	{
		runTestForDocumentElement(new UrlDocument());
	}
	
	public void testWPDoc() throws Exception
	{
		runTestForDocumentElement(new WPDoc());
	}
	
	public void testNoteMinutes() throws Exception
	{
		runTestForDocumentElement(new NoteMinutes());
	}
	
	private void runTestForDocumentElement(GenericDocument genericDocument) throws Exception
	{
		DocumentFolder rootFolderParent = createSaveRootDocumentFolder();
		
		genericDocument = (GenericDocument)DocumentElementFixture.createSaveFixture(rootFolderParent, genericDocument, 0, null, this);
		
		DocumentScope documentScope = new DocumentScope();
		documentScope.setAttributeAssignments(true);
		
		AttributeAssignment attributeAssignment = new AttributeAssignment();
		attributeAssignment.setAttribute(attribute);
		attributeAssignment.setValue("10");
		genericDocument.setAttributeAssignments(new AttributeAssignment[] {attributeAssignment});
		
		genericDocument = (GenericDocument)save(genericDocument,documentScope).getRpmObject();
		assertNotNull(genericDocument);
		assertNotNull(genericDocument.getAttributeAssignments());
		assertTrue(genericDocument.getAttributeAssignments().length == 1);
		assertNotNull(genericDocument.getAttributeAssignments()[0]);
		assertTrue(genericDocument.getAttributeAssignments()[0].getValue().equals("10"));
		
		genericDocument.getAttributeAssignments()[0].setValue("20");
//		 TODO: Uncomment the following line when running the test
//		genericDocument.getAttributeAssignments()[0].assignTypeID(null);
		genericDocument = (GenericDocument)save(genericDocument,documentScope).getRpmObject();
		
		assertNotNull(genericDocument);
		assertNotNull(genericDocument.getAttributeAssignments());
		assertTrue(genericDocument.getAttributeAssignments().length == 1);
		assertNotNull(genericDocument.getAttributeAssignments()[0]);
		assertTrue(genericDocument.getAttributeAssignments()[0].getValue().equals("20"));
	}
	
	private DocumentFolder createSaveRootDocumentFolder() throws Exception
	{
		WorkElementScope workElementScope = new WorkElementScope();
		workElementScope.setDocumentFolder(new DocumentScope());
		Proposal proposal = new Proposal();
		proposal.setName(HelperTestUtilities.createName());
		proposal = (Proposal)save(proposal, workElementScope).getRpmObject();
		assertNotNull(proposal);
		assertNotNull(proposal.getDocumentFolder());
		return (DocumentFolder)proposal.getDocumentFolder();
	}
	
	private void createSaveAttributes() throws Exception
	{
		DocumentModule module = new DocumentModule();

		AttributeCategory category = createAttributeHierarchy();
		
		module.setAttributeCategories(new AttributeCategory[]{category});
		AttributeScope attrScope = new AttributeScope();   	    	          
		attrScope.setCategories(attrScope);
		attrScope.setClassifications(attrScope);
		attrScope.setAttributes(attrScope);

		DocumentModuleScope moduleScope = new DocumentModuleScope();
		moduleScope.setAttributeCategories(attrScope);    	

		module = (DocumentModule)save(module, moduleScope).getRpmObject();
		
		assertNotNull(module);
		assertNotNull(module.getAttributeCategories());
		assertTrue(module.getAttributeCategories().length == 1);
		assertNotNull(module.getAttributeCategories()[0]);
		assertNotNull(module.getAttributeCategories()[0].getClassifications());
		assertTrue(module.getAttributeCategories()[0].getClassifications().length == 1);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0]);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0].getAttributes());
		assertTrue(module.getAttributeCategories()[0].getClassifications()[0].getAttributes().length == 1);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0].getAttributes()[0]);
		
		attribute = module.getAttributeCategories()[0].getClassifications()[0].getAttributes()[0];
	}
	
	private AttributeCategory createAttributeHierarchy() throws Exception
	{
		AttributeCategory category = new AttributeCategory();
		category.setName(HelperTestUtilities.createName());

		AttributeClassification  classification = new AttributeClassification();
		classification.setName(HelperTestUtilities.createName());
		classification.setSelectType(SelectType.Multi);

		Attribute attribute1 = new Attribute();
		attribute1.setName(HelperTestUtilities.createName());
		attribute1.setInputType(InputType.Numeric);

		classification.setAttributes(new Attribute[] {attribute1});

		category.setClassifications(new AttributeClassification[]{ classification });
		
		return category;
	}
}
