/*
 * Created on 9-May-2007
 *
 */
package com.ibm.rpm.client.junit.document.testcases.validation;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author karim
 *
 */
public class TestCompositeRelationshipValidation extends GenericValidation
{
    private RPMObject mainGrandParent;
    private DocumentFolder mainParent;
    private Document mainDocument;
    
    private RPMObjectScope mainGrandParentScope;
    private DocumentScope documentScope;
    
    protected void setUp() throws Exception
    {
        super.setUp();
        documentScope = new DocumentScope();
        documentScope.setChildren(documentScope);
    }
    
    protected void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testAssetFailureWithValidation() throws AssertionError, Exception
    {
        runSaveFailureWithValidation(new Asset());
    }
    
    public void testClientCostCenterFailureWithValidation() throws AssertionError, Exception
    {
        runSaveFailureWithValidation(new ClientCostCenter());
    }

    public void testPoolFailureWithValidation() throws AssertionError, Exception
    {
        runSaveFailureWithValidation(new Pool());
    }
    
    public void testResourceFailureWithValidation() throws AssertionError, Exception
    {
        runSaveFailureWithValidation(new Resource());
    }
    
    public void testWorkElementFailureWithValidation() throws AssertionError, Exception
    {
        runSaveFailureWithValidation(new Proposal());
    }
    
    public void testScopeElementFailureWithValidation() throws AssertionError, Exception
    {
        runSaveFailureWithValidation(new Action());
    }
    
    private void runSaveFailureWithValidation(RPMObject baseObject)throws Exception
    {
        createMainGrandParent(baseObject);
        createMainGrandParentScope(baseObject);

        loadMainGrandParent();
        loadMainParent();

        createDocument();
        saveAndValidate();
    }

    private void createMainGrandParent(RPMObject baseObject)throws Exception
    {
        mainGrandParent = DocumentElementFixture.createSaveParent(baseObject,this);
    }
    
    private void createMainGrandParentScope(RPMObject baseObject)throws Exception
    {
        if (baseObject instanceof Asset)
        {
            mainGrandParentScope = new AssetScope();
            ((AssetScope)mainGrandParentScope).setDocumentFolder(documentScope);
        }
        else if (baseObject instanceof ClientCostCenter)
        {
            mainGrandParentScope = new ClientCostCenterScope();
            ((ClientCostCenterScope)mainGrandParentScope).setDocumentFolder(documentScope);
        }
        else if(baseObject instanceof Pool)
        {
            mainGrandParentScope = new PoolScope();
            ((PoolScope)mainGrandParentScope).setDocumentFolder(documentScope);
        }
        else if (baseObject instanceof Resource)
        {
            mainGrandParentScope = new ResourceScope();
            ((ResourceScope)mainGrandParentScope).setDocumentFolder(documentScope);
        }
        else if (baseObject instanceof AbstractScope)
        {
            mainGrandParentScope = new ScopeElementScope();
            ((ScopeElementScope)mainGrandParentScope).setDocumentFolder(documentScope);
        }
        else if (baseObject instanceof WorkElement)
        {
            mainGrandParentScope = new WorkElementScope();
            ((WorkElementScope)mainGrandParentScope).setDocumentFolder(documentScope);
        }
        else
        {
            throw new Exception("createMainGrandParentScope : unsupported grand parent class "+baseObject.getClass());
        }
    }

    private void loadMainGrandParent() throws Exception
    {
        mainGrandParent = loadFromID(mainGrandParent, mainGrandParentScope);
    }
    
    private void loadMainParent() throws Exception
    {
        if (mainGrandParent instanceof Asset)
        {
            mainParent = ((Asset)mainGrandParent).getDocumentFolder();            
        }
        else if (mainGrandParent instanceof ClientCostCenter)
        {
            mainParent = ((ClientCostCenter)mainGrandParent).getDocumentFolder();
        }
        else if(mainGrandParent instanceof Pool)
        {
            mainParent = ((Pool)mainGrandParent).getDocumentFolder();
        }
        else if (mainGrandParent instanceof Resource)
        {
            mainParent = ((Resource)mainGrandParent).getDocumentFolder();
        }
        else if (mainGrandParent instanceof AbstractScope)
        {
            mainParent = ((AbstractScope)mainGrandParent).getDocumentFolder();
        }
        else if (mainGrandParent instanceof WorkElement)
        {
            mainParent = ((WorkElement)mainGrandParent).getDocumentFolder();
        }
        else
        {
            throw new Exception("loadMainParent : unsupported grand parent class "+mainGrandParent.getClass());
        }
    }
    
    private void createDocument()
    {
        mainDocument = new Document();
        mainDocument.setName(HelperTestUtilities.createName());
        mainParent.setChildren(new DocumentElement[]{mainDocument});        
    }

    private void saveAndValidate() throws AssertionError, Exception
    {
        mainDocument.setElementNumber(new Integer(1234));
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, mainDocument.getClass().getName(),
                "elementNumber", mainDocument.getID())
        };

        verifySaveMustFail(mainGrandParent, mainGrandParentScope, expectedExceptions);
    }
}
