/*
 * Created on 17-Nov-2005
 */
package com.ibm.rpm.client.junit.document.testcases.validation;

import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;

/**
 * @author dominiqu
 */
public class TestDelDocValidation extends GenericValidation
{
    private ClientCostCenter mainGrandParent;

    private DocumentFolder mainParent;

    private DelDoc mainDelDoc;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = ClientCostCenterFixture.createSaveFixture(null, this);
        
        // Load the ClientCostCenter's DocumentFolder
        ClientCostCenterScope scope = new ClientCostCenterScope();
        scope.setDocumentFolder(new DocumentScope());
        mainGrandParent = (ClientCostCenter) loadFromID(mainGrandParent, scope);
        mainParent = mainGrandParent.getDocumentFolder();

        mainDelDoc = new DelDoc();
        mainDelDoc.setParent(mainParent);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainDelDoc.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainDelDoc.getClass().getName(),
                "parent", mainDelDoc.getID())
        };

        verifySaveMustFail(mainDelDoc, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainDelDoc.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainDelDoc.getClass()
                .getName(), "parent", mainDelDoc.getID())
        };

        verifySaveMustFail(mainDelDoc, null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Create Document with the max values
        mainDelDoc.setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX));
        mainDelDoc
            .setKeywords(getTestString(ValidationConstants.ABSTRACTDOCUMENT_KEYWORDS_MAX));
        mainDelDoc
            .setReferenceNumber(getTestString(ValidationConstants.ABSTRACTDOCUMENT_REFERENCENUMBER_MAX));
        mainDelDoc
            .setSummary(getTestString(ValidationConstants.ABSTRACTDOCUMENT_SUMMARY_MAX));

        // Attempt to save the Document
        mainDelDoc = (DelDoc) saveMustSucceed(mainDelDoc, DelDoc.class, null);

        // Load the inserted Document from id
        RPMObject rpmObject = loadFromID(mainDelDoc, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof DelDoc);

        // Check that the loaded Document is identical to the one we
        // requested to save
        DocumentElementFixture.compare(mainDelDoc, ((DelDoc) rpmObject), null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Create Document with maximum+1 values
        mainDelDoc
            .setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX + 1));
        mainDelDoc
            .setKeywords(getTestString(ValidationConstants.ABSTRACTDOCUMENT_KEYWORDS_MAX + 1));
        mainDelDoc
            .setReferenceNumber(getTestString(ValidationConstants.ABSTRACTDOCUMENT_REFERENCENUMBER_MAX + 1));
        mainDelDoc
            .setSummary(getTestString(ValidationConstants.ABSTRACTDOCUMENT_SUMMARY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainDelDoc.getClass().getName(),
                "name", mainDelDoc.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainDelDoc.getClass().getName(),
                "keywords", mainDelDoc.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainDelDoc.getClass().getName(),
                "referenceNumber", mainDelDoc.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainDelDoc.getClass().getName(),
                "summary", mainDelDoc.getID())
        };

        verifySaveMustFail(mainDelDoc, null, expectedExceptions);
    }
}
