/*
 * Created on 17-Nov-2005
 */
package com.ibm.rpm.client.junit.document.testcases.validation;

import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author dominiqu
 */
public class TestDocumentFolderValidation extends GenericValidation
{
    private ClientCostCenter mainParent;

    private DocumentFolder mainDocumentFolder;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainParent = ClientCostCenterFixture.createSaveFixture(null, this);
        mainDocumentFolder = new DocumentFolder();
        mainParent.setDocumentFolder(mainDocumentFolder);
        mainDocumentFolder.setParent(mainParent);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainDocumentFolder.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainDocumentFolder.getClass()
                .getName(), "parent", mainDocumentFolder.getID())
        };

        verifySaveMustFail(mainDocumentFolder, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Document wrongParent = new Document();
        mainDocumentFolder.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainDocumentFolder
                .getClass().getName(), "parent", mainDocumentFolder.getID())
        };

        verifySaveMustFail(mainDocumentFolder, null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Create DocumentFolder with the max values
        mainDocumentFolder
            .setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX));

        // Attempt to save the DocumentFolder
        mainDocumentFolder = (DocumentFolder) saveMustSucceed(mainDocumentFolder,
            DocumentFolder.class, null);

        // Load the inserted DocumentFolder from id
        RPMObject rpmObject = loadFromID(mainDocumentFolder, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof DocumentFolder);

        // Check that the loaded DocumentFolder is identical to the one we
        // requested to save
        DocumentElementFixture.compare(mainDocumentFolder, ((DocumentFolder) rpmObject),
            null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Create DocumentFolder with maximum+1 values
        mainDocumentFolder
            .setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainDocumentFolder.getClass()
                .getName(), "name", mainDocumentFolder.getID())
        };

        verifySaveMustFail(mainDocumentFolder, null, expectedExceptions);
    }
}
