/*
 * Created on 17-Nov-2005
 */
package com.ibm.rpm.client.junit.document.testcases.validation;

import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;

/**
 * @author dominiqu
 */
public class TestDocumentValidation extends GenericValidation
{
    private ClientCostCenter mainGrandParent;
    
    private DocumentFolder mainParent;

    private Document mainDocument;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = ClientCostCenterFixture.createSaveFixture(null, this);    

        // Load the ClientCostCenter's DocumentFolder
        ClientCostCenterScope scope = new ClientCostCenterScope();
        scope.setDocumentFolder(new DocumentScope());
        mainGrandParent = (ClientCostCenter) loadFromID(mainGrandParent, scope);
        mainParent = mainGrandParent.getDocumentFolder();
        
        mainDocument = new Document();
        mainDocument.setParent(mainParent);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainDocument.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainDocument.getClass().getName(),
                "parent", mainDocument.getID())
        };

        verifySaveMustFail(mainDocument, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainDocument.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainDocument.getClass()
                .getName(), "parent", mainDocument.getID())
        };

        verifySaveMustFail(mainDocument, null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Create Document with the max values
        mainDocument.setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX));
        mainDocument
            .setKeywords(getTestString(ValidationConstants.ABSTRACTDOCUMENT_KEYWORDS_MAX));
        mainDocument
            .setReferenceNumber(getTestString(ValidationConstants.ABSTRACTDOCUMENT_REFERENCENUMBER_MAX));
        mainDocument
            .setSummary(getTestString(ValidationConstants.ABSTRACTDOCUMENT_SUMMARY_MAX));

        // Attempt to save the Document
        mainDocument = (Document) saveMustSucceed(mainDocument, Document.class, null);

        // Load the inserted Document from id
        RPMObject rpmObject = loadFromID(mainDocument, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Document);

        // Check that the loaded Document is identical to the one we
        // requested to save
        DocumentElementFixture.compare(mainDocument, ((Document) rpmObject), null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Create Document with maximum+1 values
        mainDocument
            .setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX + 1));
        mainDocument
            .setKeywords(getTestString(ValidationConstants.ABSTRACTDOCUMENT_KEYWORDS_MAX + 1));
        mainDocument
            .setReferenceNumber(getTestString(ValidationConstants.ABSTRACTDOCUMENT_REFERENCENUMBER_MAX + 1));
        mainDocument
            .setSummary(getTestString(ValidationConstants.ABSTRACTDOCUMENT_SUMMARY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainDocument.getClass().getName(),
                "name", mainDocument.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainDocument.getClass().getName(),
                "keywords", mainDocument.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainDocument.getClass().getName(),
                "referenceNumber", mainDocument.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainDocument.getClass().getName(),
                "summary", mainDocument.getID())
        };

        verifySaveMustFail(mainDocument, null, expectedExceptions);
    }
    
    public void testFailureLockFieldReadOnly() throws AssertionError, Exception
    {
    	mainDocument.setName(HelperTestUtilities.createName());
    	
    	// Attempt to save the Document
        mainDocument = (Document) saveMustSucceed(mainDocument, Document.class, null);
        
        mainDocument.setLocked(Boolean.TRUE);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, mainDocument.getClass()
                .getName(), "locked", mainDocument.getID())
        };
        
        verifySaveMustFail(mainDocument, null, expectedExceptions);
    }
    
    public void testFailureElementNumberFieldReadOnly() throws AssertionError, Exception
    {
    	mainDocument.setName(HelperTestUtilities.createName());
    	
    	// Attempt to save the Document
        mainDocument = (Document) saveMustSucceed(mainDocument, Document.class, null);
        
        mainDocument.setElementNumber(new Integer (mainDocument.getElementNumber().intValue() + 1));
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, mainDocument.getClass()
                .getName(), "elementNumber", mainDocument.getID())
        };
        
        verifySaveMustFail(mainDocument, null, expectedExceptions);
    }
    
    public void testMajorRevisionNumberFieldReadOnly() throws AssertionError, Exception
    {
    	mainDocument.setName(HelperTestUtilities.createName());
    	
    	// Attempt to save the Document
        mainDocument = (Document) saveMustSucceed(mainDocument, Document.class, null);
        
        mainDocument.setMajorRevisionNumber(new Integer(mainDocument.getMajorRevisionNumber().intValue() +  1));
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, mainDocument.getClass()
                .getName(), "majorRevisionNumber", mainDocument.getID())
        };
        
        verifySaveMustFail(mainDocument, null, expectedExceptions);
    }

    public void testMinorRevisionNumberFieldReadOnly() throws AssertionError, Exception
    {
    	mainDocument.setName(HelperTestUtilities.createName());
    	
    	// Attempt to save the Document
        mainDocument = (Document) saveMustSucceed(mainDocument, Document.class, null);
        
        mainDocument.setMinorRevisionNumber(new Integer(mainDocument.getMinorRevisionNumber().intValue() +  1));
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, mainDocument.getClass()
                .getName(), "minorRevisionNumber", mainDocument.getID())
        };
        
        verifySaveMustFail(mainDocument, null, expectedExceptions);
    }
}
