/*
 * Created on 17-Nov-2005
 */
package com.ibm.rpm.client.junit.document.testcases.validation;

import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;

/**
 * @author dominiqu
 */
public class TestNoteMinutesValidation extends GenericValidation
{
    private ChangeRequest mainGrandParent;

    private DocumentFolder mainParent;

    private NoteMinutes mainNoteMinutes;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = ScopeElementFixture.createSaveChangeRequest(this);

        // Load the ChangeRequest's DocumentFolder
        ScopeElementScope scope = new ScopeElementScope();
        scope.setDocumentFolder(new DocumentScope());
        mainGrandParent = (ChangeRequest) loadFromID(mainGrandParent, scope);
        mainParent = mainGrandParent.getDocumentFolder();

        mainNoteMinutes = new NoteMinutes();
        mainNoteMinutes.setParent(mainParent);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainNoteMinutes.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT,
                mainNoteMinutes.getClass().getName(), "parent", mainNoteMinutes.getID())
        };

        verifySaveMustFail(mainNoteMinutes, null, expectedExceptions);
    }

    public void testFailureWithInvalidDocumentParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainNoteMinutes.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainNoteMinutes.getClass()
                .getName(), "parent", mainNoteMinutes.getID())
        };

        verifySaveMustFail(mainNoteMinutes, null, expectedExceptions);
    }
    
    public void testFailureWithInvalidNotesMinutesParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        mainNoteMinutes.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainNoteMinutes.getClass()
                .getName(), "parent", mainNoteMinutes.getID())
        };

        verifySaveMustFail(mainNoteMinutes, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Create NoteMinutes with the max values
        mainNoteMinutes
            .setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX));
        mainNoteMinutes.setPriority(new Integer(
            ValidationConstants.NOTEMINUTES_PRIORITY_MAX));
        mainNoteMinutes.setRank(new Integer(ValidationConstants.NOTEMINUTES_RANK_MAX));

        // Attempt to save the NoteMinutes
        mainNoteMinutes = (NoteMinutes) saveMustSucceed(mainNoteMinutes,
            NoteMinutes.class, null);

        // Load the inserted NoteMinutes from id
        RPMObject rpmObject = loadFromID(mainNoteMinutes, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof NoteMinutes);

        // Check that the loaded NoteMinutes is identical to the one we
        // requested to save
        DocumentElementFixture.compare(mainNoteMinutes, ((NoteMinutes) rpmObject), null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Create NoteMinutes with the min values
        mainNoteMinutes.setPriority(new Integer(
            ValidationConstants.NOTEMINUTES_PRIORITY_MIN));
        mainNoteMinutes.setRank(new Integer(ValidationConstants.NOTEMINUTES_RANK_MIN));

        // Attempt to save the NoteMinutes
        mainNoteMinutes = (NoteMinutes) saveMustSucceed(mainNoteMinutes,
            NoteMinutes.class, null);

        // Load the inserted NoteMinutes from id
        RPMObject rpmObject = loadFromID(mainNoteMinutes, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof NoteMinutes);

        // Check that the loaded NoteMinutes is identical to the one we
        // requested to save
        DocumentElementFixture.compare(mainNoteMinutes, ((NoteMinutes) rpmObject), null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Create NoteMinutes with maximum+1 values
        mainNoteMinutes
            .setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX + 1));
        mainNoteMinutes.setPriority(new Integer(
            ValidationConstants.NOTEMINUTES_PRIORITY_MAX + 1));
        mainNoteMinutes
            .setRank(new Integer(ValidationConstants.NOTEMINUTES_RANK_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainNoteMinutes.getClass()
                .getName(), "name", mainNoteMinutes.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainNoteMinutes.getClass()
                .getName(), "priority", mainNoteMinutes.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainNoteMinutes.getClass()
                .getName(), "rank", mainNoteMinutes.getID()),
        };

        verifySaveMustFail(mainNoteMinutes, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Create NoteMinutes with minimum-1 values
        mainNoteMinutes.setPriority(new Integer(
            ValidationConstants.NOTEMINUTES_PRIORITY_MIN - 1));
        mainNoteMinutes
            .setRank(new Integer(ValidationConstants.NOTEMINUTES_RANK_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainNoteMinutes.getClass()
                .getName(), "priority", mainNoteMinutes.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainNoteMinutes.getClass()
                .getName(), "rank", mainNoteMinutes.getID()),
        };

        verifySaveMustFail(mainNoteMinutes, null, expectedExceptions);
    }
}
