/*
 * Created on 17-Nov-2005
 */
package com.ibm.rpm.client.junit.document.testcases.validation;

import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;

/**
 * @author dominiqu
 */
public class TestWPDocValidation extends GenericValidation
{
    private ClientCostCenter mainGrandParent;

    private DocumentFolder mainParent;

    private WPDoc mainWPDoc;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = ClientCostCenterFixture.createSaveFixture(null, this);

        // Load the ClientCostCenter's DocumentFolder
        ClientCostCenterScope scope = new ClientCostCenterScope();
        scope.setDocumentFolder(new DocumentScope());
        mainGrandParent = (ClientCostCenter) loadFromID(mainGrandParent, scope);
        mainParent = mainGrandParent.getDocumentFolder();

        mainWPDoc = new WPDoc();
        mainWPDoc.setParent(mainParent);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainWPDoc.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainWPDoc.getClass().getName(),
                "parent", mainWPDoc.getID())
        };

        verifySaveMustFail(mainWPDoc, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainWPDoc.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainWPDoc.getClass()
                .getName(), "parent", mainWPDoc.getID())
        };

        verifySaveMustFail(mainWPDoc, null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Create Document with the max values
        mainWPDoc.setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX));
        mainWPDoc
            .setKeywords(getTestString(ValidationConstants.ABSTRACTDOCUMENT_KEYWORDS_MAX));
        mainWPDoc
            .setReferenceNumber(getTestString(ValidationConstants.ABSTRACTDOCUMENT_REFERENCENUMBER_MAX));
        mainWPDoc
            .setSummary(getTestString(ValidationConstants.ABSTRACTDOCUMENT_SUMMARY_MAX));

        // Attempt to save the Document
        mainWPDoc = (WPDoc) saveMustSucceed(mainWPDoc, WPDoc.class, null);

        // Load the inserted Document from id
        RPMObject rpmObject = loadFromID(mainWPDoc, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof WPDoc);

        // Check that the loaded Document is identical to the one we
        // requested to save
        DocumentElementFixture.compare(mainWPDoc, ((WPDoc) rpmObject), null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Create Document with maximum+1 values
        mainWPDoc
            .setName(getTestString(ValidationConstants.DOCUMENTELEMENT_NAME_MAX + 1));
        mainWPDoc
            .setKeywords(getTestString(ValidationConstants.ABSTRACTDOCUMENT_KEYWORDS_MAX + 1));
        mainWPDoc
            .setReferenceNumber(getTestString(ValidationConstants.ABSTRACTDOCUMENT_REFERENCENUMBER_MAX + 1));
        mainWPDoc
            .setSummary(getTestString(ValidationConstants.ABSTRACTDOCUMENT_SUMMARY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainWPDoc.getClass().getName(),
                "name", mainWPDoc.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainWPDoc.getClass().getName(),
                "keywords", mainWPDoc.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainWPDoc.getClass().getName(),
                "referenceNumber", mainWPDoc.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainWPDoc.getClass().getName(),
                "summary", mainWPDoc.getID())
        };

        verifySaveMustFail(mainWPDoc, null, expectedExceptions);
    }
}
