/*
 * Created on Feb 21, 2006
 *
 */
package com.ibm.rpm.client.junit.financial.fixtures;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.AbstractModuleFixture;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.FinancialModule;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.scope.FinancialModuleScope;
import com.ibm.rpm.financial.scope.TimeCodeScope;


/**
 * @author karim
 *
 */
public class FinancialModuleFixture extends AbstractModuleFixture
{
    public static FinancialModule createFixture(
            FinancialModuleScope scope, ClientTestController ctc) throws Exception
    {
        return createModule( scope, ctc);
    }

    public static FinancialModule updateFixture(FinancialModule module,
            FinancialModuleScope scope, ClientTestController ctc) throws Exception
    {
        return updateModule(module, scope, ctc);
    }

    private static FinancialModule createModule(
            FinancialModuleScope scope, ClientTestController ctc) throws Exception
    {
        FinancialModule module = new FinancialModule();
        
        if(scope!=null){
            if (scope.getChargeCodeCategories() != null)
            {
                ChargeCodeCategory[] chargeCodeCategories = (ChargeCodeCategory[])createElements(
                        scope.getChargeCodeCategories(), ChargeCodeCategory.class, ctc);
                module.setChargeCodeCategories(chargeCodeCategories);
            }
        }
    	                                           	
        return module;
    }

    private static FinancialModule updateModule(FinancialModule module,
            FinancialModuleScope scope, ClientTestController ctc) throws Exception
    {
        if (scope != null)
        {
            if (scope.getChargeCodeCategories() != null)
            {
                ChargeCodeCategory[] chargeCodeCategories = (ChargeCodeCategory[]) updateElements(
                        module.getChargeCodeCategories(), scope
                                .getChargeCodeCategories(), ctc);
                module.setChargeCodeCategories(chargeCodeCategories);
            }
        }
        return module;
    }

    public static FinancialModuleScope createFullScope()
    {
        return createScope(true,true,true);
    }
    
    public static FinancialModuleScope createScope(boolean isChargeCodeCategories, boolean isChargeCodeStages,
    		boolean isTimeCodeGroups)
    {
        FinancialModuleScope scope = new FinancialModuleScope();

        if(isChargeCodeCategories){
            scope.setChargeCodeCategories(new ChargeCodeScope());
        }
        if(isChargeCodeStages){
            scope.setChargeCodeStates(isChargeCodeStages);
        }
        if(isTimeCodeGroups){
            scope.setTimeCodeGroups(new TimeCodeScope());
        }
        return scope;
    }

    public static void compare(FinancialModule obj1, FinancialModule obj2,
            FinancialModuleScope scope) throws AssertionFailedError, Exception
    {
        if (scope != null)
        {
            if (scope.getChargeCodeCategories() != null)
            {
                //TODO activate later
//                ChargeCodeFixture.compare(obj1.getChargeCodeCategories(), obj2
//                        .getChargeCodeCategories(), scope.getChargeCodeCategories());
            }
            if (scope.isChargeCodeStates())
            {
                DatafieldFixture.compare(obj1.getChargeCodeStates(), obj2
                        .getChargeCodeStates());
            }
            if (scope.getTimeCodeGroups() != null)
            {
                TimecodeFixture.compare(obj1.getTimeCodeGroups(), obj2
                        .getTimeCodeGroups(), scope
                        .getTimeCodeGroups());
            }
        }
    }
}
