package com.ibm.rpm.client.junit.financial.fixtures;

import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.scope.TimePhasedFinancialScope;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author ikusakov
 */
public class TimePhasedFinancialFixture extends RPMObjectFixture
{
    protected static ArrayList localSkipGetters = new ArrayList();

    static
    {
        //set skipGetters
        localSkipGetters.addAll(skipGetters);
        localSkipGetters.add("getTotalForYear"); //read-only, calculated by proc
    }
    
    public static TimePhasedFinancialScope createScope() throws Exception
    {
        TimePhasedFinancialScope tpfs = new TimePhasedFinancialScope();
        FinancialScope fs = FinancialFixture.createAllScope();

        tpfs.setParent(fs);

        return tpfs;
    }

    public static TimePhasedFinancial createWorksheetFixture(
            TimePhasedFinancialScope scope, ClientTestController ctc)
            throws Exception
    {
        return createFixture(WorksheetFinancial.class, scope, ctc);
    }

    public static TimePhasedFinancial createWbsFixture(
            TimePhasedFinancialScope scope, ClientTestController ctc)
            throws Exception
    {
        return createFixture(WbsFinancial.class, scope, ctc);
    }

    public static TimePhasedFinancial createFixture(Class parentContainer,
            TimePhasedFinancialScope scope, ClientTestController ctc)
            throws Exception
    {
        TimePhasedFinancial timephased = new TimePhasedFinancial();

        double estimate = 1001.0;
        if (scope != null && scope.getParent() != null)
        {
            FinancialScope parentScope = (FinancialScope)scope.getParent();
            parentScope.setParent(new WorkElementScope());
            parentScope.setChargeCode(true);
            AbstractFinancial parent = FinancialFixture.createFixture(true,
                    parentContainer, parentScope, ctc);
            //           FinancialScope financialScope = new FinancialScope();
            ctc.save(parent, parentScope);
            //            parent = (AbstractFinancial) ctc.loadFromID(parent,
            // financialScope);
            timephased.setParent(parent);
            // NO - tpfinancial.setID(parent.getParent().getID());
            estimate = parent.getEstimate().doubleValue();
        }

        // tpfinancial.setChargeCodeID(parent.getChargeCode().getID());
        timephased.setMonth01Value(new Double(1));
        timephased.setMonth02Value(new Double(2));
        timephased.setMonth03Value(new Double(3));
        timephased.setMonth04Value(new Double(4));
        timephased.setMonth05Value(new Double(5));
        timephased.setMonth06Value(new Double(6));
        timephased.setMonth07Value(new Double(7));
        timephased.setMonth08Value(new Double(8));
        timephased.setMonth09Value(new Double(9));
        timephased.setMonth10Value(new Double(10));
        timephased.setMonth11Value(new Double(11));
        timephased.setMonth12Value(new Double(estimate - 66));

        timephased.setBudgetType(BudgetType.Estimate);

        // TODO might want to take a year from Project
        timephased.setYear(new Integer(2010));

        return timephased;
    }

    public static void compare(TimePhasedFinancial f1, TimePhasedFinancial f2,
            TimePhasedFinancialScope scope) throws AssertionFailedError,
            Exception
    {
        RPMObjectFixture.compareWithSkip(f1, f2, localSkipGetters);

        if (scope == null)
            return;

        if (scope.getParent() != null)
        {
            if (f2.getParent() == null)
                Assert.fail("RPMObject in scope is null");

            //Be careful with the scope to avoid infinite loop
            FinancialScope parentScope = (FinancialScope)scope.getParent();
            if (parentScope != null)
            {
                parentScope.setParent(null);
            }

            FinancialFixture.compare((AbstractFinancial) f1.getParent(),
                    (AbstractFinancial) f2.getParent(), parentScope);
        }
    }
}