/*
 * Created on Aug 11, 2005
 *
 */
package com.ibm.rpm.client.junit.financial.fixtures;

import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.containers.AbstractTimeCode;
import com.ibm.rpm.financial.containers.GenericTimeCode;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeCategory;
import com.ibm.rpm.financial.containers.TimeCodeGroup;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;

/**
 * @author sailajap
 *  
 */
public class TimecodeFixture extends RPMObjectFixture
{
    protected static final int DEFAULT_NB_CATEGORIES = 2;

    protected static final int DEFAULT_NB_TIMECODES = 2;

    private static TimeCode STATIC_TIMECODE1_FIXTURE = null;

    private static TimeCode STATIC_TIMECODE2_FIXTURE = null;

    private static String XPATH_TIMECODE1_NAME = "STATIC_TIMECODE1_FIXTURE";

    private static String XPATH_TIMECODE2_NAME = "STATIC_TIMECODE2_FIXTURE";

    public static TimeCodeScope createScope(boolean includeParent,
            boolean includeChildren)
    {
        TimeCodeScope scope = new TimeCodeScope();

        if (includeParent)
            scope.setParent(scope);

        if (includeChildren)
            scope.setChildren(scope);

        return scope;
    }

    /**
     * Creates a one TimeCodeCategory and one TimeCode object underneath it and
     * returns it.
     * 
     * @param scope
     * @param isTimeCode1
     * @param ctc
     * @return
     * @throws Exception
     */
    public static TimeCode createSaveTimeCodeFixture(TimeCodeScope scope,
            boolean isTimeCode1, ClientTestController ctc) throws Exception
    {
        TimeCodeCategory category = createCategory(scope, isTimeCode1, false,
            ctc);
        category = (TimeCodeCategory) save(ctc, category, scope);

        return (TimeCode) category.getChildren()[0];
    }

    public static TimeCodeCategory createSaveFixture(TimeCodeScope scope,
            boolean isTimeCode1, ClientTestController ctc) throws Exception
    {
        TimeCodeCategory category = createCategory(scope, isTimeCode1, true,
            ctc);
        category = (TimeCodeCategory) save(ctc, category, scope);

        return category;
    }

    public static TimeCodeCategory createFixture(TimeCodeScope scope,
            boolean isTimeCode1, ClientTestController ctc) throws Exception
    {
        return createCategory(scope, isTimeCode1, true, ctc);
    }

    private static TimeCodeCategory createCategory(TimeCodeScope scope,
            boolean isTimeCode1, boolean isComplexStructure,
            ClientTestController ctc) throws Exception
    {
        TimeCodeGroup root = new TimeCodeGroup();
        if (isTimeCode1)
        {
            root = (TimeCodeGroup) ctc.loadFromXpath(
                "/TimeCodeGroup[id='TIMECODE_1']", null)[0];
        }
        else
        {
            root = (TimeCodeGroup) ctc.loadFromXpath(
                "/TimeCodeGroup[id='TIMECODE_2']", null)[0];
        }

        TimeCodeCategory category = new TimeCodeCategory();
        category.setParent(root);
        HelperTestUtilities.setByIntrospection(category, random(), skipSetters);

        if (scope != null)
        {
            if (scope.getChildren() != null)
            {
                if (isComplexStructure)
                {
                    TimeCodeCategory[] subCategories = new TimeCodeCategory[DEFAULT_NB_CATEGORIES];
                    for (int i = 0; i < DEFAULT_NB_CATEGORIES; i++)
                    {
                        TimeCodeCategory subCategory = new TimeCodeCategory();
                        subCategory.setParent(category);
                        HelperTestUtilities.setByIntrospection(subCategory,
                            random(), skipSetters);

                        // create TimeCodes underneath the subcategory
                        TimeCode[] timeCodes = new TimeCode[DEFAULT_NB_TIMECODES];
                        for (int j = 0; j < DEFAULT_NB_TIMECODES; j++)
                        {
                            TimeCode timeCode = TimecodeFixture
                                .createTimecode();
                            timeCode.setParent(subCategory);
                            timeCodes[j] = timeCode;
                        }
                        subCategory.setChildren(timeCodes);
                        subCategories[i] = subCategory;
                    }

                    // create TimeCodes underneath the category
                    TimeCode[] timeCodes = new TimeCode[DEFAULT_NB_TIMECODES];
                    for (int j = 0; j < DEFAULT_NB_TIMECODES; j++)
                    {
                        TimeCode timeCode = TimecodeFixture.createTimecode();
                        timeCode.setParent(category);
                        timeCodes[j] = timeCode;
                    }

                    ArrayList temp = mergeArrays(subCategories, timeCodes);
                    GenericTimeCode[] children = new GenericTimeCode[temp
                        .size()];
                    temp.toArray(children);
                    category.setChildren(children);
                }
                else
                {
                    TimeCode[] timeCodes = new TimeCode[1];
                    TimeCode timeCode = TimecodeFixture.createTimecode();
                    timeCode.setParent(category);
                    timeCodes[0] = timeCode;
                    category.setChildren(timeCodes);
                }
            }
        }
        return category;
    }

    private static TimeCode createTimecode() throws Exception
    {
        TimeCode timeCode = new TimeCode();
        HelperTestUtilities.setByIntrospection(timeCode, random(), skipSetters);
        return timeCode;
    }

    //updates the names
    public static void updateCategoryFixture(TimeCodeCategory category,
            TimeCodeScope scope, ClientTestController ctc) throws Exception
    {
        updateCategory(category, scope, random(), 0, 0, ctc);
    }

    public static TimeCode updateTimeCodeFixture(TimeCode timeCode)
            throws Exception
    {
        HelperTestUtilities.setByIntrospection(timeCode, random(), skipSetters);
        return timeCode;
    }

    public static void updateCategoryStructure(TimeCodeCategory category,
            int deltaNbCategories, int deltaNbTimecodes, TimeCodeScope scope,
            ClientTestController ctc) throws Exception
    {
        updateCategory(category, scope, random(), deltaNbCategories,
            deltaNbTimecodes, ctc);
    }

    private static void updateCategory(TimeCodeCategory category,
            TimeCodeScope scope, int index, int deltaNbCategories,
            int deltaNbTimecodes, ClientTestController ctc) throws Exception
    {
        HelperTestUtilities.setByIntrospection(category, index, skipSetters);
        if (scope != null && scope.getChildren() != null)
        {
            //Update timecode Names;
            GenericTimeCode[] children = category.getChildren();
            for (int i = 0; i < children.length; ++i)
            {
                HelperTestUtilities.setByIntrospection(children[i], index + i,
                    skipSetters);
            }
            if (deltaNbCategories > 0)
            {
                TimeCodeCategory newCateg = null;
                TimeCodeCategory[] newCategories = new TimeCodeCategory[deltaNbCategories];
                for (int i = 0; i < deltaNbCategories; ++i)
                {
                    newCateg = createFixture(scope.getChildren(), false, ctc);
                    newCateg.setParent(category);
                    newCategories[i] = newCateg;
                }
                ArrayList temp = mergeArrays(category.getChildren(),
                    newCategories);
                GenericTimeCode[] totalCategories = new GenericTimeCode[temp
                    .size()];
                temp.toArray(totalCategories);
                category.setChildren(totalCategories);
            }
            if (deltaNbTimecodes > 0)
            {
                TimeCode newTimeCode = null;
                TimeCode[] newTcs = new TimeCode[deltaNbTimecodes];
                for (int i = 0; i < deltaNbTimecodes; ++i)
                {
                    newTimeCode = createTimecode();
                    newTimeCode.setParent(category);
                    newTcs[i] = newTimeCode;
                }
                ArrayList temp = mergeArrays(category.getChildren(), newTcs);
                GenericTimeCode[] timeCodes = new GenericTimeCode[temp.size()];
                temp.toArray(timeCodes);
                category.setChildren(timeCodes);
            }
        }
    }

    public static void compare(AbstractTimeCode[] array1,
            AbstractTimeCode[] array2, TimeCodeScope scope)
            throws AssertionFailedError, Exception
    {
        sortWithComparator(array1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(array2, RPMObjectNameAndIDComparator.INSTANCE);

        RPMObjectFixture.compare(array1, array2);

        if (array1 != null && array2 != null)
        {
            for (int i = 0; i < array1.length; i++)
            {
                compare(array1[i], array2[i], scope);
            }
        }
    }

    public static void compare(AbstractTimeCode obj1, AbstractTimeCode obj2,
            TimeCodeScope scope) throws AssertionFailedError, Exception
    {
        if (obj1 == null && obj2 == null)
            return;

        if (obj1 instanceof TimeCodeGroup)
        {
            compareTimeCodeGroup((TimeCodeGroup) obj1, (TimeCodeGroup) obj2,
                scope);
        }
        else if (obj1 instanceof TimeCodeCategory)
        {
            compareTimeCodeCategory((TimeCodeCategory) obj1,
                (TimeCodeCategory) obj2, scope);
        }
        else if (obj1 instanceof TimeCode)
        {
            compareTimeCode((TimeCode) obj1, (TimeCode) obj2);
        }
        else
        {
            Assert.fail("TimeCodeFixture.compare : class "
                    + obj1.getClass().getName() + " not supported.");
        }
    }

    private static void compareTimeCodeGroup(TimeCodeGroup cat1,
            TimeCodeGroup cat2, TimeCodeScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(cat1, cat2);

        //compare children
        if (scope != null)
        {
            if (scope.getParent() != null)
            {
                compare(cat1.getParent(), cat2.getParent());
            }

            if (scope.getChildren() != null)
            {
                compare(cat1.getChildren(), cat2.getChildren(), scope
                    .getChildren());
            }
        }
    }

    private static void compareTimeCodeCategory(TimeCodeCategory cat1,
            TimeCodeCategory cat2, TimeCodeScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(cat1, cat2);

        //compare children
        if (scope != null)
        {
            if (scope.getParent() != null)
            {
                compare(cat1.getParent(), cat2.getParent());
            }

            if (scope.getChildren() != null)
            {
                compare(cat1.getChildren(), cat2.getChildren(), scope
                    .getChildren());
            }
        }
    }

    private static void compareTimeCode(TimeCode obj1, TimeCode obj2)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(obj1, obj2);
    }

    private static ArrayList mergeArrays(GenericTimeCode[] array1,
            GenericTimeCode[] array2)
    {
        ArrayList temp = new ArrayList();
        addArray(temp, array1);
        addArray(temp, array2);
        return temp;
    }

    private static void addArray(ArrayList result, GenericTimeCode[] array)
    {
        if (array != null && array.length != 0)
        {
            for (int i = 0; i < array.length; i++)
            {
                result.add(array[i]);
            }
        }
    }

    /**
     * Use this fixture if you want to test the update of the reference to a
     * calendar.
     * 
     * @param ctc
     * @return
     * @throws Exception
     */
    public static TimeCode retrieveStaticFixture(
        boolean isTimecode1, ClientTestController ctc) throws Exception
    {
        TimeCode timecode = null;
        String name = null;
        
        if (isTimecode1)
        {
            timecode = STATIC_TIMECODE1_FIXTURE;
            name = XPATH_TIMECODE1_NAME;
        }
        else
        {
            timecode = STATIC_TIMECODE2_FIXTURE;
            name = XPATH_TIMECODE2_NAME;
        }
        
        
        if (timecode == null)
        {            
            RPMObject[] rpmObjects = ctc.loadFromXpath("/Timecode[name = '"
                    + name + "']", null);

            if (rpmObjects != null && rpmObjects[0] != null)
            {
                timecode = (TimeCode) rpmObjects[0];
            }
        }

        if (timecode == null)
        {
            
            TimeCodeScope scope = TimecodeFixture.createScope(true, true);
            
            TimeCodeCategory cat = createCategory(scope, isTimecode1, false, ctc);            
            cat.setName(name);
            ((TimeCode)cat.getChildren()[0]).setName(name);
            
            cat = (TimeCodeCategory) saveWithoutDelete(ctc, cat, scope);
            timecode = ((TimeCode)cat.getChildren()[0]);
        }

        return timecode;

    }
}
