package com.ibm.rpm.client.junit.financial.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.financial.util.FinancialUtil;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.types.TimePhasedType;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * @author ikusakov
 */
public class TestFinancial extends ClientTestController
{

    FinancialScope allScope = null;

    FinancialScope minimalScope = null;

    protected void setUp() throws Exception
    {
        ClientTestController.logger.info("setUp");
        super.setUp();
        allScope = FinancialFixture.createAllScope();
        
        minimalScope = new FinancialScope();
        minimalScope.setParent(new WorkElementScope());
        minimalScope.setChargeCode(true);
    }

    public void tearDown() throws Exception
    {
        ClientTestController.logger.info("tearDown");
        super.tearDown();
    }

    public void testSimpleFromWbsPerspective() throws Exception
    {

        WorksheetFinancial financial = FinancialFixture.createWorksheetFixture(
                allScope, this);
        save(financial, allScope);

        String xpath = "/Proposal[ID = '" + financial.getParent().getID()
                + "']/WorksheetFinancials[ChargeCode[ID='"
                + financial.getChargeCode().getID() + "']]";
        RPMObject[] objects = loadFromXpath(xpath, null);
        assertNotNull(objects);
        assertTrue(objects.length > 0);

        WorksheetFinancial wsFin = (WorksheetFinancial) objects[0];
        // get the current estimate amount and add fundAmount to that and then
        // set the new estimate amount to WorksheetFinancial object
        wsFin.setEstimate(new Double(wsFin.getEstimate().doubleValue() + 1000));
        save(wsFin, null);

        WorksheetFinancial wsFinNew = (WorksheetFinancial) loadFromID(wsFin,
                allScope);
        // we cannot compare like this: FinancialFixture.compare(wsFin,
        // wsFinNew, allScope);
        // because calcualted fields in wsFin are not properly calculated

        assertEquals(wsFin.getEstimate(), wsFinNew.getEstimate());
    }

    public void testSimpleFromScopePerspective() throws Exception
    {
        ChangeRequest changeRequest = ScopeElementFixture
                .createSaveChangeRequest(this);

        // we cannot reuse allScope because parent class type defines the parent
        FinancialScope scopeFinancialScope = new FinancialScope();
        scopeFinancialScope.setChargeCode(true);
        scopeFinancialScope.setParent(new ScopeElementScope());
        scopeFinancialScope.setActualTimePhased(true);
        scopeFinancialScope.setEstimateTimePhased(true);

        WorksheetFinancial financial = FinancialFixture.createWorksheetFixture(
                scopeFinancialScope, changeRequest, this);
        save(financial, scopeFinancialScope);

        String xpath = "/ChangeRequest[ID = '" + financial.getParent().getID()
                + "']/WorksheetFinancial[ChargeCode[ID='"
                + financial.getChargeCode().getID() + "']]";
        RPMObject[] objects = loadFromXpath(xpath, null);
        assertNotNull(objects);
        assertTrue(objects.length > 0);

        WorksheetFinancial wsFin = (WorksheetFinancial) objects[0];
        // get the current estimate amount and add fundAmount to that and then
        // set the new estimate amount to WorksheetFinancial object
        wsFin.setEstimate(new Double(wsFin.getEstimate().doubleValue() + 1000));
        save(wsFin, null);

        WorksheetFinancial wsFinNew = (WorksheetFinancial) loadFromID(wsFin,
                scopeFinancialScope);
        // we cannot compare like this: FinancialFixture.compare(wsFin,
        // wsFinNew, allScope);
        // because calcualted fields in wsFin are not properly calculated

        assertEquals(wsFin.getEstimate(), wsFinNew.getEstimate());
    }

    public void testWorksheetInsertNoScopeLoadFromXPath()
            throws AssertionError, Exception
    {
        ClientTestController.logger
                .info("================= WorksheetFinancial");
        caseInsertNoScopeLoadFromXPath(FinancialFixture.createWorksheetFixture(
                allScope, this));
    }
    
    public void testWorksheetInsertTotalsNoScopeLoadFromXPath()
            throws AssertionError, Exception
    {
        ClientTestController.logger
            .info("================= WorksheetFinancial");

        // calling the fixture with minimal scope will create the totals
        // because timephased is not in scope (can't have both)
        caseInsertNoScopeLoadFromXPath(FinancialFixture.createWorksheetFixture(
            minimalScope, this));
    }
    
    public void testWbsInsertNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        ClientTestController.logger.info("================= WBSFinancial");
        caseInsertNoScopeLoadFromXPath(FinancialFixture.createWbsFixture(
                minimalScope, this));
    }
    
    public void testWbsInsertTotalsNoScopeLoadFromXPath()
            throws AssertionError, Exception
    {
        ClientTestController.logger.info("================= WBSFinancial");

        // calling the fixture with minimal scope will create the totals
        // because timephased is not in scope (can't have both)
        caseInsertNoScopeLoadFromXPath(FinancialFixture.createWbsFixture(
            minimalScope, this));
    }

    private void caseInsertNoScopeLoadFromXPath(AbstractFinancial financial)
            throws AssertionError, Exception
    {

        save(financial, allScope);
        AbstractFinancial saved = (AbstractFinancial) loadFromID(financial,
                minimalScope);
        FinancialFixture.compare(financial, saved, minimalScope);

        RPMObject[] rpmObjects = loadFromXpath(
                (financial instanceof WorksheetFinancial
                        ? "/WorksheetFinancial[Id='" : "/WBSFinancial[Id='")
                        + financial.getID() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects.length > 0);
        for (int i = 0; i < rpmObjects.length; i++)
        {
            assertTrue(rpmObjects[i] instanceof AbstractFinancial);
            if (financial.getID().equals(
                    ((AbstractFinancial) rpmObjects[i]).getID()))
            {
                FinancialFixture.compare(financial,
                        (AbstractFinancial) rpmObjects[i], null);
                break;
            }
        }
    }

    public void testWorksheetInsertNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        ClientTestController.logger
                .info("================= WorksheetFinancial");
        caseInsertNoScopeLoadFromID(FinancialFixture.createWorksheetFixture(
                allScope, this), allScope);
    }
    
    public void testWorksheetInsertTotalsNoScopeLoadFromID()
            throws AssertionError, Exception
    {
        ClientTestController.logger
            .info("================= WorksheetFinancial");

        // calling the fixture with minimal scope will create the totals
        // because timephased is not in scope (can't have both)
        caseInsertNoScopeLoadFromID(FinancialFixture.createWorksheetFixture(
            minimalScope, this), minimalScope);
    }

    public void testWbsInsertNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        ClientTestController.logger.info("================= WBSFinancial");
        caseInsertNoScopeLoadFromID(FinancialFixture.createWbsFixture(allScope,
                this), minimalScope);
    }
    
    public void testWbsInsertTotalsNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        ClientTestController.logger.info("================= WBSFinancial");
       
        // calling the fixture with minimal scope will create the totals
        // because timephased is not in scope (can't have both)
        caseInsertNoScopeLoadFromID(FinancialFixture.createWbsFixture(
            minimalScope, this), minimalScope);
    }

    private void caseInsertNoScopeLoadFromID(AbstractFinancial financial,
            FinancialScope scope) throws AssertionError, Exception
    {
        save(financial, scope);
        
        AbstractFinancial saved = (AbstractFinancial) loadFromID(financial,
                scope);
        
        FinancialUtil.nullToZero(financial);
        FinancialUtil.nullToZero(saved);
        
        FinancialFixture.compare(financial, saved, null);
        saved = (AbstractFinancial) loadFromID(saved, null);
        assertTrue("different object", financial != saved);
        assertNotNull(saved);
        assertNotNull(saved.getID());
        assertEquals(financial.getID(), saved.getID());
        FinancialFixture.compare(financial, saved, null);
    }

    public void testWorksheetDeleteNoScope() throws AssertionError, Exception
    {
        ClientTestController.logger
                .info("================= WorksheetFinancial");
        caseDeleteNoScope(FinancialFixture.createWorksheetFixture(allScope,
                this));
    }

    public void testWbsDeleteNoScope() throws AssertionError, Exception
    {
        ClientTestController.logger.info("================= WBSFinancial");
        caseDeleteNoScope(FinancialFixture.createWbsFixture(allScope, this));
    }

    private void caseDeleteNoScope(AbstractFinancial financial)
            throws AssertionError, Exception
    {
        save(financial, allScope);
        AbstractFinancial saved = (AbstractFinancial) loadFromID(financial,
                minimalScope);
        assertNotNull(saved);
        assertNotNull(saved.getID());
        assertNotNull(saved.getParent());
        assertNotNull(saved.getParent().getID());
        delete(saved);
        try
        {
            assertNull("Deleted object should be null or throw an exception",
                    loadFromID(saved, null));
        }
        catch (RPMException rpmexception)
        {
        }
    }

    public void testWorksheetUpdateNoScope() throws AssertionError, Exception
    {
        ClientTestController.logger
                .info("================= WorksheetFinancial");
        caseUpdateNoScope(FinancialFixture.createWorksheetFixture(allScope,
                this));
        // no update for WBSFinancial
    }
    
    public void testWorksheetTotalsUpdateNoScope() throws AssertionError, Exception
    {
        ClientTestController.logger
                .info("================= WorksheetFinancial");
        
        // calling the fixture with minimal scope will create the totals
        // because timephased is not in scope (can't have both)
        caseUpdateNoScope(FinancialFixture.createWorksheetFixture(minimalScope,
                this));
        // no update for WBSFinancial
    }

    private void caseUpdateNoScope(AbstractFinancial financial)
            throws AssertionError, Exception
    {
        save(financial, allScope);
        AbstractFinancial saved = (AbstractFinancial) loadFromID(financial,
                minimalScope);

        assertEquals(financial.getParent().getID(), saved.getParent().getID());
        saved.setActual(new Double(123));
        saved.setEstimate(new Double(234));
        saved.setETC(new Double(345));
//        saved.setProbabilityPercent(new Double(95));

        // generated

        // saved.getEstimate().doubleValue() * 0.95 = 222.3
        // we cannot use the formula directly, because it produces 222.2999999
        // and later gets in conflict with DB value
        saved.setProbabilityAmount(new Double(222.3));
        saved.setRemaining(new Double(saved.getEstimate().doubleValue()
                - saved.getActual().doubleValue()));
        saved.setEAC(new Double(saved.getActual().doubleValue()
                + saved.getETC().doubleValue()));
        saved.setEACVariance(new Double(saved.getETC().doubleValue()
                - saved.getRemaining().doubleValue()));
        saved.setBaselineVariance(new Double(saved.getEstimate().doubleValue()
                - saved.getBaseline().doubleValue()));
        AbstractFinancial returned = (AbstractFinancial) save(saved,
                minimalScope).getRpmObject();
        assertEquals(returned.getID(), saved.getID());
        FinancialFixture.compare(returned, saved, null);
        AbstractFinancial loaded = (AbstractFinancial) loadFromID(saved,
                minimalScope);

        assertNotNull(loaded);
        assertNotNull(loaded.getID());
        assertNotNull(loaded.getParent());
        assertNotNull(loaded.getParent().getID());
        assertEquals(loaded.getID(), saved.getID());
        FinancialFixture.compare(loaded, saved, null);
    }

    public void testWorksheetInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        ClientTestController.logger
                .info("================= WorksheetFinancial");
        caseInsertWithScopeLoadFromID(FinancialFixture.createWorksheetFixture(
                allScope, this), allScope);
    }
    
    public void testWorksheetInsertTotalsWithScopeLoadFromID()
            throws AssertionError, Exception
    {
        ClientTestController.logger
            .info("================= WorksheetFinancial");

        // calling the fixture with minimal scope will create the totals
        // because timephased is not in scope (can't have both)
        caseInsertWithScopeLoadFromID(FinancialFixture.createWorksheetFixture(
            minimalScope, this), minimalScope);
    }
    
    public void testWbsInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        ClientTestController.logger.info("================= WBSFinancial");
        caseInsertWithScopeLoadFromID(FinancialFixture.createWbsFixture(
                allScope, this), allScope);
    }
    
    public void testWbsInsertTotalsWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        ClientTestController.logger.info("================= WBSFinancial");
        
        // calling the fixture with minimal scope will create the totals
        // because timephased is not in scope (can't have both)
        caseInsertWithScopeLoadFromID(FinancialFixture.createWbsFixture(
                minimalScope, this), minimalScope);
    }
    
    private void caseInsertWithScopeLoadFromID(AbstractFinancial financial,
            FinancialScope scope) throws AssertionError, Exception
    {
        save(financial, scope);
        AbstractFinancial saved = (AbstractFinancial) loadFromID(financial,
                scope);

        if (scope != null && scope.isActualTimePhased())
        {
            assertNotNull(saved.getActualTimePhased());
            assertNotNull(saved.getActualTimePhased()[0]);
            assertNotNull(saved.getActualTimePhased()[0].getID());

            // timephased ID holds WorkElement ID
            assertEquals(financial.getParent().getID(), saved
                .getActualTimePhased()[0].getID());
            assertEquals(new Double(1.0D), saved.getActualTimePhased()[0]
                .getMonth01Value());
        }
        
        FinancialFixture.compare(financial, saved, scope);
        saved = (AbstractFinancial) loadFromID(saved, scope);

        assertNotNull(saved);
        assertNotNull(saved.getID());
        assertEquals(financial.getID(), saved.getID());

        FinancialFixture.compare(financial, saved, scope);
    }

    public void testWorksheetUpdateWithScope() throws AssertionError, Exception
    {
        ClientTestController.logger
                .info("================= WorksheetFinancial");
        caseUpdateWithScope(FinancialFixture.createWorksheetFixture(allScope,
                this), allScope);
        // no update for WbsFinancial
    }
    
    public void testWorksheetUpdateTotalsWithScope() throws AssertionError, Exception
    {
        ClientTestController.logger
                .info("================= WorksheetFinancial");
        
        // calling the fixture with minimal scope will create the totals
        // because timephased is not in scope (can't have both)
        caseUpdateWithScope(FinancialFixture.createWorksheetFixture(minimalScope,
                this), minimalScope);
        // no update for WbsFinancial
    }
    
    public void testWorksheetUpdateTotalsThroughParentWithScope() throws AssertionError, Exception
    {
        ClientTestController.logger
                .info("================= WorksheetFinancial");
        
        caseUpdateThroughParentWithScope(minimalScope);
        // no update for WbsFinancial
    }
    
    public void testWorksheetUpdateThroughParentWithScope() throws AssertionError, Exception
    {
        ClientTestController.logger
                .info("================= WorksheetFinancial");
        
        caseUpdateThroughParentWithScope(allScope);
        // no update for WbsFinancial
    }
       
    public void testWorksheetUpdateActual() throws AssertionError, Exception
    {
        FinancialScope fs = new FinancialScope();
        fs.setChargeCode(true);
        fs.setParent(new WorkElementScope());
        WorksheetFinancial wf = FinancialFixture.createWorksheetFixture(fs, this);
        wf = (WorksheetFinancial)save(wf, fs).getRpmObject();
        
        wf.setEstimate(new Double(20.55));
        wf.setActual(new Double(10.24));
        WorksheetFinancial financial = (WorksheetFinancial)save(wf, fs).getRpmObject();
        
        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(financial, fs);
        
        //changing actual will affect the calculated fields
        FinancialFixture.compare(reloaded, financial, fs);
        return;
    }

    private void caseUpdateWithScope(AbstractFinancial financial,
            FinancialScope scope)
            throws AssertionError, Exception
    {
        AbstractFinancial saved = (AbstractFinancial) save(financial, scope)
                .getRpmObject();
        saved = (AbstractFinancial) loadFromID(financial, scope);

        FinancialFixture.compare(financial, saved, scope);

        assertNotNull(saved);
        
        // have to update either the timePhased values or the totals
        if (scope != null
                && (scope.isActualTimePhased() || scope.isEstimateTimePhased()))
        {
            TimePhasedFinancial timephased = null;
            if (scope.isActualTimePhased())
            {
                assertNotNull(saved.getActualTimePhased());
                timephased = saved.getActualTimePhased()[0];
                timephased.setMonth09Value(new Double(timephased
                    .getMonth09Value().doubleValue() + 10D));
                timephased.setMonth10Value(new Double(timephased
                    .getMonth10Value().doubleValue() + 10D));
            }
            if (scope.isEstimateTimePhased())
            {

                timephased = saved.getEstimateTimePhased()[0];
                timephased.setMonth07Value(new Double(timephased
                    .getMonth07Value().doubleValue() + 100D));
                timephased.setMonth08Value(new Double(timephased
                    .getMonth08Value().doubleValue() + 100D));
            }
        }
        else
        {
            saved.setActual(new Double(saved.getActual().doubleValue() + 20D));
            saved.setEstimate(new Double(
                saved.getEstimate().doubleValue() + 200D));
            saved.setRemaining(new Double(saved.getEstimate().doubleValue()
                    - saved.getActual().doubleValue()));
            saved.setEAC(new Double(saved.getActual().doubleValue()
                    + saved.getETC().doubleValue()));
            saved.setEACVariance(new Double(saved.getETC().doubleValue()
                    - saved.getRemaining().doubleValue()));
            saved.setBaselineVariance(new Double(saved.getEstimate()
                .doubleValue()
                    - saved.getBaseline().doubleValue()));
        }

        saved = (AbstractFinancial) save(saved, scope).getRpmObject();

        AbstractFinancial updated = (AbstractFinancial) loadFromID(financial,
            scope);

        FinancialFixture.compare(saved, updated, scope);
    }
    
    private void caseUpdateThroughParentWithScope(FinancialScope scope)
            throws AssertionError, Exception
    {
        // create a WorksheetFinancial with no parent
        AbstractFinancial financial = (WorksheetFinancial)FinancialFixture.createFixture(false,
            WorksheetFinancial.class, scope, this);
        
        // set the parent 
        Project project = (Project) WorkElementFixture.createSaveFixture(new Project(),
            null, this);
        financial.setParent(project);

        AbstractFinancial saved = (AbstractFinancial) save(financial, scope)
            .getRpmObject();
        saved = (AbstractFinancial) loadFromID(financial, scope);

        FinancialFixture.compare(financial, saved, scope);

        assertNotNull(saved);

        if (scope != null
                && (scope.isActualTimePhased() || scope.isEstimateTimePhased()))
        {
            TimePhasedFinancial timephased = null;

            if (scope.isActualTimePhased())
            {
                assertNotNull(saved.getActualTimePhased());
                timephased = saved.getActualTimePhased()[0];
                timephased.setMonth09Value(new Double(timephased
                    .getMonth09Value().doubleValue() + 10D));
                timephased.setMonth10Value(new Double(timephased
                    .getMonth10Value().doubleValue() + 10D));
            }
            if (scope.isEstimateTimePhased())
            {
                timephased = saved.getEstimateTimePhased()[0];
                timephased.setMonth07Value(new Double(timephased
                    .getMonth07Value().doubleValue() + 100D));
                timephased.setMonth08Value(new Double(timephased
                    .getMonth08Value().doubleValue() + 100D));
            }
        }
        else
        {
            saved.setActual(new Double(saved.getActual().doubleValue() + 20D));
            saved.setEstimate(new Double(
                saved.getEstimate().doubleValue() + 200D));
            saved.setRemaining(new Double(saved.getEstimate().doubleValue()
                    - saved.getActual().doubleValue()));
            saved.setEAC(new Double(saved.getActual().doubleValue()
                    + saved.getETC().doubleValue()));
            saved.setEACVariance(new Double(saved.getETC().doubleValue()
                    - saved.getRemaining().doubleValue()));
            saved.setBaselineVariance(new Double(saved.getEstimate()
                .doubleValue()
                    - saved.getBaseline().doubleValue()));
        }

        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setWorksheetFinancials(scope);
        
        ((AggregateNode) project)
            .setWorksheetFinancials(new WorksheetFinancial[]
            {
                (WorksheetFinancial) saved
            });
        
        save(project, workElementScope);

        AbstractFinancial updatedFinancial = (AbstractFinancial) loadFromID(
            saved, scope);

        FinancialFixture.compare(saved, updatedFinancial, scope);
    }

//RicardoB    
// This test has been disabled because the logic it uses does not work anymore 
// We cannot save WorksheetFinancials seperately even if they have the same parent and
// expect the that FTE be calculated using the rule FTE(x) = Hours(x) / Minutes_Per_day
//
// The right way is to create a proposal and do proposal.setWorkSheetFinancials(WorkSheetFinancials[])
// after that you can modify each one and save them seperately .(AFAIK)    
    
    
//    public void testWorksheetFTECalculation() throws AssertionError, Exception
//    {
//        final int ACTUAL = 100;
//        final int ESTIMATED = 10;
//        final int ETC = 1;
//
//        WorksheetFinancial financialHours = FinancialFixture
//                .createWorksheetFixture(allScope, this);
//        ChargeCode chargeCodeHours = new ChargeCode();
//
//        chargeCodeHours.setID("BENEFIT_HOURS");
//        
//        financialHours.setChargeCode(chargeCodeHours);
//        financialHours.setActual(new Double(ACTUAL));
//        financialHours.setEstimate(new Double(ESTIMATED));
//        financialHours.setETC(new Double(ETC));
//        save(financialHours, null);
//
//        WorksheetFinancial financialFTE = FinancialFixture
//                .createWorksheetFixture(allScope, this);
//        ChargeCode chargeCodeFTE = new ChargeCode();
//
//        chargeCodeFTE.setID("BENEFIT_FULL_TIME_EQUIVALENT");
//
//        financialFTE.setChargeCode(chargeCodeFTE);
//
//        save(financialFTE, null);
//
//        financialFTE = (WorksheetFinancial) loadFromID(financialFTE, allScope);
//
//        assertEquals(new Double(12.5), financialFTE.getActual());
//        assertEquals(new Double(1.25), financialFTE.getEstimate());
//        assertEquals(new Double(0.125), financialFTE.getETC());
//    }

    public void testWbsFTECalculation() throws AssertionError, Exception
    {
        final int ACTUAL = 100;
        final int ESTIMATED = 10;
        final int ETC = 1;

        WbsFinancial financialHours = FinancialFixture.createWbsFixture(
                allScope, this);
        ChargeCode chargeCodeHours = ChargeCodeFixture.createSimpleChargeCode();
        save(chargeCodeHours.getParent(), null);
        save(chargeCodeHours, null);

        financialHours.setChargeCode(chargeCodeHours);
        financialHours.setActual(new Double(ACTUAL));
        financialHours.setEstimate(new Double(ESTIMATED));
        financialHours.setETC(new Double(ETC));
        save(financialHours, null);

        WbsFinancial financialFTE = FinancialFixture.createWbsFixture(allScope,
                this);
        
        ChargeCodeCategory category = new ChargeCodeCategory();
        category.setName(HelperTestUtilities.createName());
        save(category, null);
        
        ChargeCode chargeCodeFTE = ChargeCodeFixture.createSimpleChargeCode();
        save(chargeCodeFTE.getParent(), null);
        save(chargeCodeFTE, null);
        financialFTE.setChargeCode(chargeCodeFTE);

        save(financialFTE, null);

        financialFTE = (WbsFinancial) loadFromID(financialFTE, allScope);

        // properly calculated FTE values
        assertEquals(new Double(0), financialFTE.getActual());
        assertEquals(new Double(0), financialFTE.getEstimate());
        assertEquals(new Double(0), financialFTE.getETC());
    }

    public void testTimephasedFlagManual() throws Exception
    {
        // This will create financial with manual timephased values
        WorksheetFinancial financialManual = FinancialFixture
                .createWorksheetFixture(allScope, this);

        save(financialManual, allScope);
        AbstractFinancial saved = (AbstractFinancial) loadFromID(
                financialManual, allScope);

        assertEquals(TimePhasedType.ManuallyTimephased, saved.getTimePhased());
        FinancialFixture.compare(financialManual, saved, null);
    }

    public void testTimephasedFlagAuto() throws Exception
    {
        // This will create financial with auto timephased values
        WorksheetFinancial financialManual = FinancialFixture
                .createWorksheetFixture(minimalScope, this);

        save(financialManual, allScope); // allScope will make SP_S_PBB to be
        // called
        AbstractFinancial saved = (AbstractFinancial) loadFromID(
                financialManual, allScope);

        assertEquals(TimePhasedType.AutoTimephased, saved.getTimePhased());
        // TODO AutoTimphased seems not processed as expected
        // by the stored procedure. Cannot do much about it.

        //        assertNotNull(saved.getActualTimePhased());
        //        assertEquals(1, saved.getActualTimePhased().length);
        //        assertEquals(new Double(1),
        // saved.getActualTimePhased()[0].getMonth01Value());
    }
    
    public void testAbstractScopeWithScopeMilestoneParentIsNotReadonlyForCustomCC()
            throws Exception
    {
        FinancialScope mainFinancialScope = FinancialFixture
            .createAllScopeNoTimephased();

        ScopeElementScope mainElementScope = new ScopeElementScope();
        mainElementScope.setParent(new ScopeElementScope());
        mainElementScope.setChildren(mainElementScope);
        mainElementScope.setWbsFinancials(new FinancialScope());
        mainElementScope.getWbsFinancials().setChargeCode(true);

        mainElementScope.getChildren().setWbsFinancials(new FinancialScope());
        mainElementScope.getChildren().getWbsFinancials().setChargeCode(true);

        ChangeRequest scopeParent = ScopeElementFixture
            .createSaveChangeRequest(this);

        ScopeMilestone scopeMilestone = ScopeElementFixture
            .createScopeMilestoneFixture(mainElementScope, 1, scopeParent, this);

        WbsFinancial wbsFinancial = FinancialFixture.createWbsFixture(
            mainFinancialScope, this);

        scopeMilestone.setWbsFinancials(new WbsFinancial[]
        {
            wbsFinancial
        });

        SingleResult result = null;

        result = save(scopeMilestone, mainElementScope);

        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
    }
    
    public void testSimplenodeWithTaskParentIsNotReadonlyForCustomCC()
            throws Exception
    {
        FinancialScope mainFinancialScope = FinancialFixture
            .createAllScopeNoTimephased();

        WorkElementScope mainWEScope = new WorkElementScope();
        mainWEScope.setParent(new WorkElementScope());
        mainWEScope.setChildren(mainWEScope);
        mainWEScope.setWbsFinancials(new FinancialScope());
        mainWEScope.getWbsFinancials().setChargeCode(true);

        mainWEScope.getChildren().setWbsFinancials(new FinancialScope());
        mainWEScope.getChildren().getWbsFinancials().setChargeCode(true);

        Project parentProject = (Project) WorkElementFixture.createSaveFixture(
            new Project(), new WorkElementScope(), this);

        Task task = (Task) WorkElementFixture.createFixture(new Task(),
            new WorkElementScope(), this);

        WbsFinancial wbsFinancial = FinancialFixture.createWbsFixture(
            mainFinancialScope, this);

        task.setParent(parentProject);
        task.setWbsFinancials(new WbsFinancial[]
        {
            wbsFinancial
        });

        SingleResult result = null;

        result = save(task, mainWEScope);

        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());              
        
    }
    
    
    public void testSimplenodeIsNotReadonlyForCustomCCSavingUsingWF()
            throws Exception
    {
        //Phase 1: create Projet and save with wbs in scope
        FinancialScope mainFinancialScope = FinancialFixture
            .createAllScopeNoTimephased();

        WorkElementScope mainWEScope = new WorkElementScope();
        mainWEScope.setChildren(mainWEScope);
        mainWEScope.setWbsFinancials(new FinancialScope());
        mainWEScope.getWbsFinancials().setChargeCode(true);

        mainWEScope.getChildren().setWbsFinancials(new FinancialScope());
        mainWEScope.getChildren().getWbsFinancials().setChargeCode(true);

        Project parentProject = (Project) WorkElementFixture.createSaveFixture(
            new Project(), new WorkElementScope(), this);

        WbsFinancial wbsFinancial = FinancialFixture.createWbsFixture(
            mainFinancialScope, this);

        parentProject.setWbsFinancials(new WbsFinancial[]
        {
            wbsFinancial
        });

        SingleResult result = null;

        result = save(parentProject, mainWEScope);

        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(!result.isWarningFree());

        // Phase 2: add worksheet to proposal and save with worksheet
        //only in scope.

        assertTrue(result.getRpmObject() instanceof Project);

        WorkElementScope loadedMainWEScope = new WorkElementScope();
        loadedMainWEScope.setChildren(mainWEScope);
        loadedMainWEScope.setWorksheetFinancials(new FinancialScope());
        loadedMainWEScope.getWorksheetFinancials().setChargeCode(true);

        loadedMainWEScope.getChildren().setWorksheetFinancials(
            new FinancialScope());
        loadedMainWEScope.getChildren().getWorksheetFinancials().setChargeCode(
            true);

        Project loadedParent = (Project) result.getRpmObject();

        loadedParent.setWorksheetFinancials(FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(loadedParent,
                this));

        result = save(loadedParent, loadedMainWEScope);

        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
    }
    
    
    public void testWbsFinancialTaskIsNotReadonlyForCustomCC_59817()
            throws Exception
    {
        Proposal parent = null;

        // Proposal
        parent = new Proposal();
        parent.setName(HelperTestUtilities.createName());
        parent.setPublished(ProjectPublishedType.None);

        SaveResult sr = save(parent, new WorkElementScope());
        assertTrue(sr.getRpmObject() instanceof Proposal);
        parent = (Proposal) sr.getRpmObject();

        // create and set a charge code
        ChargeCode QAExpense = ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this);

        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setPublished(ProjectPublishedType.None);

        task.setParent(parent);
        parent.setChildren(new WorkElement[]
        {
            task
        });

        WbsFinancial wbsFin = new WbsFinancial();
        wbsFin.setChargeCode(QAExpense);
        wbsFin.setEstimate(new Double(20));

        task.setWbsFinancials(new WbsFinancial[]
        {
            wbsFin
        });

        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setParent(new WorkElementScope());

        WorkElementScope childScope = new WorkElementScope();
        childScope.setWbsFinancials(finScope);
        childScope.setParent(new WorkElementScope());

        WorkElementScope wes = new WorkElementScope();
        wes.setChildren(childScope);

        sr = save(task, childScope);
        assertTrue(sr.getRpmObject() instanceof Task);
        task = (Task) sr.getRpmObject();
        assertTrue(sr.isWarningFree());

        assertTrue(task.getWbsFinancials()[0].getEstimate().equals(
            new Double(20.0)));
        assertTrue(task.getWbsFinancials()[0].getActual().equals(
            new Double(0.0)));
        assertTrue(task.getWbsFinancials()[0].getRemaining().equals(
            new Double(20.0)));
        assertTrue(task.getWbsFinancials()[0].getETC().equals(new Double(20.0)));
        assertTrue(task.getWbsFinancials()[0].getEAC().equals(new Double(20.0)));

    }

    public void testWbsFinancialScopeTaskIsNotReadonlyForCustomCC_59817()
            throws Exception
    {
        ScopeElementScope mainElementScope = new ScopeElementScope();
        mainElementScope.setParent(new ScopeElementScope());
        mainElementScope.setWbsFinancials(new FinancialScope());
        mainElementScope.getWbsFinancials().setChargeCode(true);

        ChangeRequest scopeParent = ScopeElementFixture
            .createSaveChangeRequest(this);

        ScopeTask scopeTask = ScopeElementFixture.createScopeTaskFixture(
            mainElementScope, 1, scopeParent, this);

        // create and set a charge code
        ChargeCode QAExpense = ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this);

        WbsFinancial wbsFin = new WbsFinancial();
        wbsFin.setChargeCode(QAExpense);
        wbsFin.setEstimate(new Double(20));

        scopeTask.setWbsFinancials(new WbsFinancial[]
        {
            wbsFin
        });

        SaveResult sr = null;
        sr = save(scopeTask, mainElementScope);
        assertTrue(sr.getRpmObject() instanceof ScopeTask);
        scopeTask = (ScopeTask) sr.getRpmObject();
        assertTrue(sr.isWarningFree());

        assertTrue(scopeTask.getWbsFinancials()[0].getEstimate().equals(
            new Double(20.0)));
        assertTrue(scopeTask.getWbsFinancials()[0].getActual().equals(
            new Double(0.0)));
        assertTrue(scopeTask.getWbsFinancials()[0].getRemaining().equals(
            new Double(20.0)));
        assertTrue(scopeTask.getWbsFinancials()[0].getETC().equals(
            new Double(20.0)));
        assertTrue(scopeTask.getWbsFinancials()[0].getEAC().equals(
            new Double(20.0)));

    }    


}
