package com.ibm.rpm.client.junit.financial.testcases;

import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.financial.util.FinancialUtil;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.scope.TimePhasedFinancialScope;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.financial.types.TimePhasedType;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * @author rbadette
 * 
 * This class will test all financials basic business logic rules
 */
public class TestFinancialChargeCodeBusinessLogic extends ClientTestController
{
    private WorkElementScope wScope = null;

    private Proposal wMainProposal = null;

    private WorksheetFinancial[] wWSheetFinancial = null;

    private static final int HOUR = 0;

    private static final int AVG_COST = 1;

    private static final int EXP_LABOR = 2;


    public void setUp() throws Exception
    {
        super.setUp();
    }

    /**
     * @throws Exception
     *  
     */
    private void init() throws Exception
    {
        PortfolioScope wPFoScope = null;
        Portfolio wMainPortfolio = null;

        wPFoScope = new PortfolioScope();
        wScope = new WorkElementScope();

        //Setting Up Scope
        wScope.setWorksheetFinancials(new FinancialScope());
        wScope.getWorksheetFinancials().setChargeCode(true);
        wPFoScope.setProjects(wScope);

        //Creating PortFolio and Proposal
        SaveResult portfolioResult = null;

        wMainPortfolio = new Portfolio();
        wMainPortfolio.setName(HelperTestUtilities.createName());

        wMainProposal = new Proposal();
        wMainProposal.setName(HelperTestUtilities.createName());
        wMainProposal.setPortfolio(wMainPortfolio);

        wMainPortfolio.setProjects(new GenericProject[]
        {
            wMainProposal
        });

        //Saving PortFolio/Proposal according to scope
        portfolioResult = save(wMainPortfolio, wPFoScope);

        assertNotNull(portfolioResult);

        assertTrue(portfolioResult.getRpmObject() instanceof Portfolio);
        wMainPortfolio = (Portfolio) portfolioResult.getRpmObject();

        assertNotNull(wMainPortfolio.getProjects());
        assertTrue(wMainPortfolio.getProjects()[0] instanceof Proposal);
        wMainProposal = (Proposal) wMainPortfolio.getProjects()[0];

        wMainProposal.setAssignmentType(AssignmentType.Work);
        wMainProposal.setPublished(ProjectPublishedType.None);
        
        //Loading Financials System Charge Codes
        wWSheetFinancial = FinancialFixture
        .createEmptyWorksheetFinancialSystemChargeCodesArray(wMainProposal, this);
    }

    /**
     * This utility method will print the Financial matrix after each test so we
     * can compare the result and make changes accordingly.
     * 
     * @param result
     * @throws RPMException
     */
    private void printResult(SaveResult result) throws RPMException
    {
        AbstractFinancial[] financial = null;
        if (result.getRpmObject() instanceof Proposal)
        {
            Proposal parent = (Proposal) result.getRpmObject();
            if (parent.getWorksheetFinancials() != null
                    && parent.getWorksheetFinancials()[0] instanceof WorksheetFinancial)
            {
                financial = parent.getWorksheetFinancials();

            }
            else if (parent.getWbsFinancials() != null
                    && parent.getWbsFinancials()[0] instanceof WbsFinancial)
            {
                financial = parent.getWbsFinancials();
            }
        }
        else if (result.getRpmObject() instanceof WorkElement)
        {
            WorkElement parent = (WorkElement) result.getRpmObject();
            if (parent.getWbsFinancials() != null
                    && parent.getWbsFinancials()[0] instanceof WbsFinancial)
            {
                financial = parent.getWbsFinancials();
            }
        }
        else if (result.getRpmObject() instanceof WorksheetFinancial)
        {
            financial = new WorksheetFinancial[1];
            financial[0] = ((WorksheetFinancial) result.getRpmObject());
        }
        else if (result.getRpmObject() instanceof WbsFinancial)
        {
            financial = new WbsFinancial[1];
            financial[0] = ((WbsFinancial) result.getRpmObject());
        }
        else if (result.getRpmObject() instanceof AbstractAggregateScope)
        {
            AbstractAggregateScope parent = (AbstractAggregateScope) result
                .getRpmObject();
            if (parent.getWbsFinancials() != null
                    && parent.getWbsFinancials()[0] instanceof WbsFinancial)
            {
                financial = parent.getWbsFinancials();
            }
        }
        else if (result.getRpmObject() instanceof AbstractScopeTask)
        {
            AbstractScopeTask parent = (AbstractScopeTask) result
                .getRpmObject();
            if (parent.getWbsFinancials() != null
                    && parent.getWbsFinancials()[0] instanceof WbsFinancial)
            {
                financial = parent.getWbsFinancials();
            }
        }

        if (financial != null)
        {
            String title = null;
            logger.debug("	ESTIMATE	ACTUAL		REMAINING	ETC		EAC");

            for (int i = 0; i < financial.length; i++)
            {
                if (financial[i].getChargeCode() != null)
                {
                    Integer dbCode;

                    dbCode = FinancialUtil.getDBCodeForChargeCode(financial[i]
                        .getChargeCode().getFinancialCategory());

                    switch (dbCode.intValue())
                    {
                    case FinancialUtil.EXPENSE_HOUR:
                        title = "HOU:	";
                        break;
                    case FinancialUtil.EXPENSE_AVERAGE_COST:
                        title = "AVG:	";
                        break;
                    case FinancialUtil.EXPENSE_LABOR:
                        title = "ELB:	";
                        break;
                    case FinancialUtil.EXPENSE_FTE:
                        title = "FTE:	";
                        break;

                    default:
                        title = financial[i].getChargeCode().getName();

                    }

                    logger.debug(""
                            + title
                            + "  "
                            + FinancialUtil.nullToZero(
                                financial[i].getEstimate()).doubleValue()
                            + " 		"
                            + FinancialUtil
                                .nullToZero(financial[i].getActual())
                                .doubleValue()
                            + " 		"
                            + FinancialUtil.nullToZero(
                                financial[i].getRemaining()).doubleValue()
                            + " 		"
                            + FinancialUtil.nullToZero(financial[i].getETC())
                                .doubleValue()
                            + " 		"
                            + FinancialUtil.nullToZero(financial[i].getEAC())
                                .doubleValue() + "\n");
                }

            }
        }
        else
        {
            logger.debug("***ERROR PRINTING FINANCIAL INFORMATION***");
        }

    }

    /**
     * This utility method will validate the result from an operation with the
     * expected values of the Financial matrix.
     * 
     * @param rs
     * @param values
     * @throws RPMException
     */
    private void validateResult(SaveResult rs, double[] values)
            throws RPMException
    {
        AbstractFinancial[] financial = ((Proposal) rs.getRpmObject())
            .getWorksheetFinancials();

        AbstractFinancial hourExpense = FinancialUtil
            .getFinancialForChargeCode(FinancialUtil.EXPENSE_HOUR, financial);

        AbstractFinancial laborExpense = FinancialUtil
            .getFinancialForChargeCode(FinancialUtil.EXPENSE_LABOR, financial);

        AbstractFinancial avgExpense = FinancialUtil.getFinancialForChargeCode(
            FinancialUtil.EXPENSE_AVERAGE_COST, financial);

        AbstractFinancial fteExpense = FinancialUtil.getFinancialForChargeCode(
            FinancialUtil.EXPENSE_FTE, financial);

        // values array validation
        assertTrue(values.length == 20);

        // financials validation
        assertNotNull(hourExpense);
        assertNotNull(laborExpense);
        assertNotNull(avgExpense);
        assertNotNull(fteExpense);

        // HOUR
        assertTrue((hourExpense.getEstimate()).doubleValue() == (values[0])); // Hour
        // Estimate
        assertTrue((hourExpense.getActual()).doubleValue() == (values[1])); // Hour
        // Actual
        assertTrue((hourExpense.getRemaining()).doubleValue() == (values[2])); // Hour
        // Remaining
        assertTrue((hourExpense.getETC()).doubleValue() == (values[3])); // Hour
        // ETC
        assertTrue((hourExpense.getEAC()).doubleValue() == (values[4])); // Hour
        // EAC

        // AVERAGE_COST
        assertTrue((avgExpense.getEstimate()).doubleValue() == (values[5])); // AvgCost
        // Estimate
        assertTrue((avgExpense.getActual()).doubleValue() == (values[6])); // AvgCost
        // Actual
        assertTrue((avgExpense.getRemaining()).doubleValue() == (values[7])); // AvgCost
        // Remaining
        assertTrue((avgExpense.getETC()).doubleValue() == (values[8])); // AvgCost
        // ETC
        assertTrue((avgExpense.getEAC()).doubleValue() == (values[9])); // Avgcost
        // EAC

        // EXPENSE_LABOR
        assertTrue((laborExpense.getEstimate()).doubleValue() == (values[10])); // ExpLabor
        // Estimate
        assertTrue((laborExpense.getActual()).doubleValue() == (values[11])); // ExpLabor
        // Actual
        assertTrue((laborExpense.getRemaining()).doubleValue() == (values[12])); // ExpLabor
        // Remaining
        assertTrue((laborExpense.getETC()).doubleValue() == (values[13])); // ExpLabor
        // ETC
        assertTrue((laborExpense.getEAC()).doubleValue() == (values[14])); // ExpLabor
        // EAC

        // FTE
        assertTrue((fteExpense.getEstimate()).doubleValue() == (values[15])); // FTE
        // Estimate
        assertTrue((fteExpense.getActual()).doubleValue() == (values[16])); // FTE
        // Actual
        assertTrue((fteExpense.getRemaining()).doubleValue() == (values[17])); // FTE
        // Remaining
        assertTrue((fteExpense.getETC()).doubleValue() == (values[18])); // FTE
        // ETC
        assertTrue((fteExpense.getEAC()).doubleValue() == (values[19])); // FTE
        // EAC
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    // Single Cell Test-----------------|

    public void testSetHourEstimateChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[HOUR].setEstimate(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                20, 0, 20, 20, 20, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                2.50, 0, 2.50, 2.50, 2.50
        }; // FTE

        printResult(result);
        validateResult(result, values);

    }

    public void testSetExpenseLaborEstimateChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[EXP_LABOR].setEstimate(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 0, 0, 0, 0, //Hours
                0, 0, 0, 0, 0, //AverageCost
                20, 0, 20, 20, 20, //ExpenseLabor
                0, 0, 0, 0, 0
        }; //FTE

        validateResult(result, values);
        printResult(result);
    }

    public void testSetEmptyChargeCodes() throws Exception
    {
        init();
        

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 0, 0, 0, 0, //Hours
                0, 0, 0, 0, 0, //AverageCost
                0, 0, 0, 0, 0, //ExpenseLabor
                0, 0, 0, 0, 0
        }; //FTE

        validateResult(result, values);
        printResult(result);
    }

    public void testSetHourActualChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[HOUR].setActual(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 20, -20, 0, 20, //Hours
                0, 0, 0, 0, 0, //AverageCost
                0, 0, 0, 0, 0, //ExpenseLabor
                0, 2.5, -2.50, 0, 2.50
        }; //FTE

        validateResult(result, values);
        printResult(result);

    }

    public void testSetExpenseLaborActualChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[EXP_LABOR].setActual(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 0, 0, 0, 0, //Hours
                0, 0, 0, 0, 0, //AverageCost
                0, 20, -20, 0, 20, //ExpenseLabor
                0, 0, 0, 0, 0
        }; //FTE

        validateResult(result, values);
        printResult(result);

    }


    public void testSetHourETCChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[HOUR].setETC(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 0, 0, 20, 20, //Hours
                0, 0, 0, 0, 0, //AverageCost
                0, 0, 0, 0, 0, //ExpenseLabor
                0, 0, 0, 2.50, 2.50
        }; //FTE

        validateResult(result, values);
        printResult(result);
    }


    public void testSetExpenseLaborETCChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[EXP_LABOR].setETC(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 0, 0, 0, 0, //Hours
                0, 0, 0, 0, 0, //AverageCost
                0, 0, 0, 20, 20, //ExpenseLabor
                0, 0, 0, 0, 0
        }; //FTE

        validateResult(result, values);
        printResult(result);

    }


    // Dual Cell Test ------------------------------|

    //-----Estimate Couple

    public void testSetEstimateHourAverageChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[HOUR].setEstimate(new Double(20));
        wWSheetFinancial[AVG_COST].setEstimate(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                20, 0, 20, 20, 20, //Hours
                20, 0, 20, 20, 20, //AverageCost
                400, 0, 400, 400, 400, //ExpenseLabor
                2.50, 0, 2.50, 2.50, 2.50
        }; //FTE

        printResult(result);
        validateResult(result, values);


    }

    public void testSetEstimateHourExpLaborChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[HOUR].setEstimate(new Double(20));
        wWSheetFinancial[EXP_LABOR].setEstimate(new Double(400));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                20, 0, 20, 20, 20, //Hours
                20, 0, 20, 20, 20, //AverageCost
                400, 0, 400, 400, 400, //ExpenseLabor
                2.50, 0, 2.50, 2.50, 2.50
        }; //FTE

        validateResult(result, values);
        printResult(result);
    }

    public void testSetEstimateExpLaborAverageChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[EXP_LABOR].setEstimate(new Double(400));
        wWSheetFinancial[AVG_COST].setEstimate(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                20, 0, 20, 20, 20, //Hours
                20, 0, 20, 20, 20, //AverageCost
                400, 0, 400, 400, 400, //ExpenseLabor
                2.50, 0, 2.50, 2.50, 2.50
        }; //FTE

        validateResult(result, values);
        printResult(result);
    }

    //------Actual Couple

    public void testSetActualHourAverageChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[HOUR].setActual(new Double(20));
        wWSheetFinancial[AVG_COST].setActual(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 20, -20, 0, 20, //Hours
                0, 20, 20, 0, 20, //AverageCost
                0, 400, -400, 0, 400, //ExpenseLabor
                0, 2.50, -2.50, 0, 2.50
        }; //FTE

        printResult(result);
        validateResult(result, values);
    }

    public void testSetActualHourExpLaborChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[HOUR].setActual(new Double(20));
        wWSheetFinancial[EXP_LABOR].setActual(new Double(400));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 20, -20, 0, 20, //Hours
                0, 20, 20, 0, 20, //AverageCost
                0, 400, -400, 0, 400, //ExpenseLabor
                0, 2.50, -2.50, 0, 2.50
        }; //FTE

        validateResult(result, values);
        printResult(result);
    }

    public void testSetActualExpLaborAverageChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[EXP_LABOR].setActual(new Double(400));
        wWSheetFinancial[AVG_COST].setActual(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 20, -20, 0, 20, //Hours
                0, 20, 20, 0, 20, //AverageCost
                0, 400, -400, 0, 400, //ExpenseLabor
                0, 2.50, -2.50, 0, 2.50
        }; //FTE

        validateResult(result, values);
        printResult(result);
    }

    //------ETC Couple

    public void testSetETCHourAverageChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[HOUR].setETC(new Double(10));
        wWSheetFinancial[AVG_COST].setETC(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 0, 0, 10, 10, //Hours
                0, 0, 0, 20, 20, //AverageCost
                0, 0, 0, 200, 200, //ExpenseLabor
                0, 0, 0, 1.25, 1.25
        }; //FTE

        printResult(result);
        validateResult(result, values);
    }

    public void testSetETCHourExpLaborChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[HOUR].setETC(new Double(20));
        wWSheetFinancial[EXP_LABOR].setETC(new Double(400));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 0, 0, 20, 20, //Hours
                0, 0, 0, 20, 20, //AverageCost
                0, 0, 0, 400, 400, //ExpenseLabor
                0, 0, 0, 2.50, 2.50
        }; //FTE

        validateResult(result, values);
        printResult(result);
    }

    public void testSetETCExpLaborAverageChargeCodes() throws Exception
    {
        init();
        

        wWSheetFinancial[EXP_LABOR].setETC(new Double(400));
        wWSheetFinancial[AVG_COST].setETC(new Double(10));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 0, 0, 40, 40, //Hours
                0, 0, 0, 10, 10, //AverageCost
                0, 0, 0, 400, 400, //ExpenseLabor
                0, 0, 0, 5, 5
        }; //FTE

        validateResult(result, values);
        printResult(result);
    }

    //-----------------QA-TEST-SECTION---------------------//

    public void testDefect_45501() throws Exception
    {
        //when we are trying to set a value for the "System Charge Code:
        // Average Cost / Hr",
        //there is no error message and no exception launched. It just doesn't
        // appear.

        init();
        

        wWSheetFinancial[HOUR].setEstimate(new Double(20));
        wWSheetFinancial[HOUR].setActual(new Double(10));

        wWSheetFinancial[AVG_COST].setEstimate(new Double(20));
        wWSheetFinancial[AVG_COST].setActual(new Double(10));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                20, 10, 10, 10, 20, //Hours
                20, 10, 30, 30, 20, //AverageCost
                400, 100, 300, 300, 400, //ExpenseLabor
                2.50, 1.25, 1.25, 1.25, 2.50
        }; //FTE

        printResult(result);
        validateResult(result, values);
    }

    public void testDefect_46334() throws Exception
    {
        //testing that Expense (and Capital) Labor = Hours x Average Cost/Hr
        //
        //So if Hour=20 and Avg=20 ==> EL=400
        init();
        

        wWSheetFinancial[HOUR].setEstimate(new Double(20));
        wWSheetFinancial[AVG_COST].setEstimate(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                20, 0, 20, 20, 20, //Hours
                20, 0, 20, 20, 20, //AverageCost
                400, 0, 400, 400, 400, //ExpenseLabor
                2.50, 0, 2.50, 2.50, 2.50
        }; //FTE

        printResult(result);
        validateResult(result, values);

    }

    public void testDefect_46565() throws Exception
    {
        //testing that Expense (and Capital) Labor = Hours x Average Cost/Hr
        //So if Hour=20 and Avg=20 ==> EL=400
        init();
        

        //---------------PHASE 1
        wWSheetFinancial[HOUR].setEstimate(new Double(20));
        wWSheetFinancial[AVG_COST].setEstimate(new Double(20));

        wWSheetFinancial[HOUR].setActual(new Double(10));
        wWSheetFinancial[AVG_COST].setActual(new Double(10));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                20, 10, 10, 10, 20, //Hours
                20, 10, 30, 30, 20, //AverageCost
                400, 100, 300, 300, 400, //ExpenseLabor
                2.50, 1.25, 1.25, 1.25, 2.50
        }; //FTE

        validateResult(result, values);
        printResult(result);

        //---------------PHASE 2
        //Get the saved object and modify it and verify that the
        //changes has been done accordingly
        //Expense (and Capital) Labor = Hours x Average Cost/Hr

        Proposal pReloaded = (Proposal) result.getRpmObject();

        wWSheetFinancial[EXP_LABOR].setEstimate(new Double(600));

        pReloaded.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result2 = null;

        result2 = save(pReloaded, wScope);

        assertNotNull(result2);
        assertNull(result2.getErrors());
        assertNotNull(((Proposal) result2.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result2.isSuccessful());

        double[] values2 =
        {
                30, 10, 20, 20, 30, //Hours
                20, 10, 25, 25, 20, //AverageCost
                600, 100, 500, 500, 600, //ExpenseLabor
                3.75, 1.25, 2.50, 2.50, 3.75
        }; //FTE

        printResult(result2);
        validateResult(result2, values2);

    }

    public void testDefect_48115() throws Exception
    {

        //Setting Up-----------------------|

        //Proposal
        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);

        //Scope used to load Charge Codes from RPM
        ChargeCodeScope ccScope = new ChargeCodeScope();
        ccScope.setChildren(new ChargeCodeScope());
        ccScope.getChildren().setChildren(new ChargeCodeScope());

        RPMObject[] objs = null;
        objs = loadFromXpath("/ChargeCodeCategory[name='System Charge Codes']",
            ccScope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof ChargeCodeCategory);

        //Get All Charge Codes from the Charge Code Category
        ChargeCodeCategory ccCategory = (ChargeCodeCategory) objs[0];
        assertNotNull(ccCategory.getChildren());
        assertTrue(ccCategory.getChildren()[0] instanceof ChargeCodeCategory);

        assertEquals("Expense", ccCategory.getChildren()[0].getName());

        ccCategory = (ChargeCodeCategory) ccCategory.getChildren()[0];

        assertNotNull(ccCategory.getChildren());
        assertEquals("1. Hours", ccCategory.getChildren()[0].getName());
        assertEquals("2. Average Cost / Hr", ccCategory.getChildren()[1]
            .getName());
        assertEquals("3. Expense Labor", ccCategory.getChildren()[2].getName());
        assertEquals("4. FTE", ccCategory.getChildren()[3].getName());

        //Financial Work Sheets
        WorksheetFinancial[] wsFinancials = new WorksheetFinancial[4];

        wsFinancials[0] = new WorksheetFinancial();
        wsFinancials[0].setChargeCode((ChargeCode) ccCategory.getChildren()[0]);

        wsFinancials[1] = new WorksheetFinancial();
        wsFinancials[1].setChargeCode((ChargeCode) ccCategory.getChildren()[1]);

        wsFinancials[2] = new WorksheetFinancial();
        wsFinancials[2].setChargeCode((ChargeCode) ccCategory.getChildren()[2]);

        wsFinancials[3] = new WorksheetFinancial();
        wsFinancials[3].setChargeCode((ChargeCode) ccCategory.getChildren()[3]);

        WorkElementScope wes = new WorkElementScope();
        wes.setWorksheetFinancials(new FinancialScope());
        wes.getWorksheetFinancials().setChargeCode(true);

        //Linking objects together
        proposal.setWorksheetFinancials(wsFinancials);

        //Creating and Saving a Calendar with 6.5 hours per day
        RPMCalendar cal = new RPMCalendar();
        cal.setHoursPerDay(new Double(6.5));
        cal.setHoursPerWeek(new Double(6.5 * 5));
        cal.setName(HelperTestUtilities.createName());

        SaveResult saveResultCal = save(cal, null);
        assertNotNull(saveResultCal);

        //Getting the saved calendar and assigning it to the proposal
        cal = (RPMCalendar) saveResultCal.getRpmObject();
        proposal.setCalendar(cal);

        //Saving to RPM
        SaveResult saveResult = save(proposal, wes);
        assertNotNull(saveResult);
        assertNotNull(saveResult.getRpmObject());

        proposal = (Proposal) saveResult.getRpmObject();

        //Testing ----------------------|

        int hours = 45;

        //Getting HOURS on saved object and modifying it to 45 and saving it
        // again
        ChargeCode matchingCC = new ChargeCode();
        matchingCC.setName("1. Hours");

        WorksheetFinancial wsfHours = FinancialUtil
            .getWorksheetFinancialWithChargeCode(proposal, matchingCC);
        assertNotNull(wsfHours);

        wsfHours.setActual(new Double(hours));

        WorkElementScope wes2 = new WorkElementScope();
        wes2.setWorksheetFinancials(new FinancialScope());
        wes2.getWorksheetFinancials().setChargeCode(true);

        //Saving the modification HOURS = 45
        SaveResult saveResult2 = save(proposal, wes2);
        assertNotNull(saveResult2.getRpmObject());

        proposal = (Proposal) saveResult2.getRpmObject();

        //Reading HOURS back again
        matchingCC.setName("1. Hours");
        wsfHours = FinancialUtil.getWorksheetFinancialWithChargeCode(proposal,
            matchingCC);
        assertNotNull(wsfHours);

        //verifying that the value(45) we saved is still in there
        assertEquals(hours, wsfHours.getActual().intValue());

        //verifying FTE
        matchingCC.setName("4. FTE");
        WorksheetFinancial wsfFTE = FinancialUtil
            .getWorksheetFinancialWithChargeCode(proposal, matchingCC);
        assertNotNull(wsfFTE);
        //assertEquals( wsfHours.getActual() /
        // proposal.getCalendar().getHoursPerDay(),wsfFTE.getActual());

        assertEquals(FinancialUtil.roundDouble2Dec(new Double(wsfHours
            .getActual().doubleValue()
                / proposal.getCalendar().getHoursPerDay().doubleValue())),
            wsfFTE.getActual());
    }

    public void testDefect_56709() throws Exception
    {
        //Should not be able to load an object ID:PMO_PROJECT__TIMEPHASE_CALENDAR__
        //because it is an invalid ChargeCode but that exist in the DB for obscure purposes.
        //So we filtered it out in ChargeCodeManager.getFilter()
        RPMObject[] objs = null;

        objs = null;
        objs = loadFromXpath("/ChargeCode", null);

        assertNotNull(objs);

        for (int i = 0; i < objs.length; i++)
        {
            assertTrue(objs[i] instanceof ChargeCode);
            ChargeCode cc = (ChargeCode) objs[i];
            assertFalse(cc.getID().equals("PMO_PROJECT_TIMEPHASE_CALENDAR__"));
        }
    }

    //This test is working fine for the defect that was presented but while doing the
    //fix we encountered another defect that should be fixed. Two versions pf this test exist
    //
    // a) Saving the parent proposal with worksheetFinancials that contains Timephased infos
    // b) Saving the timephased infos directly individually
    //
    // This is version (a)
    //
    //After creating the worksheet and saving it, we created timephased infos and added to the worksheet 
    //and saved. By doing this, the total of the timephased expenseLabor will replace the worksheet value of
    //expenseLabor which is good. BUT we need to re-calculate Hour based on the values of EL and AVG for 
    //Estimate and Actual and re-calculate the whole matrix. This is a special case so we need to detect
    //when that happens and call a method to do the above.
    public void testDefect_53937_savingParent() throws Exception
    {
        //------------Part 1: Create a proposal with Worksheetfinancials
        // attached to it and save.

        init();
        

        wWSheetFinancial[AVG_COST].setEstimate(new Double(10));
        wWSheetFinancial[EXP_LABOR].setEstimate(new Double(100));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        double[] values =
        {
                10, 0, 10, 10, 10, //Hours
                10, 0, 10, 10, 10, //AverageCost
                100, 0, 100, 100, 100, //ExpenseLabor
                1.25, 0, 1.25, 1.25, 1.25
        //FTE
        };
        printResult(result);
        validateResult(result, values);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());

        //-----------Part 2: Get the proposal that was saved and attached some timephased to the EL worksheeFinancial

        Proposal reloadedProject = (Proposal) result.getRpmObject();
        WorksheetFinancial expenseWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_LABOR", reloadedProject.getWorksheetFinancials());

        //----------Part 3: create timephased informations and bind it to the
        // correct worksheetfinancial and save with
        //--------- a financial scope.

        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setEstimateTimePhased(true);
        finScope.setActualTimePhased(true);
        finScope.setParent(new WorkElementScope());

        Calendar cal = new GregorianCalendar();
        int currentYear = cal.get(Calendar.YEAR);

        // create estimates for 2006 yearly total is 24
        TimePhasedFinancial estimateTimePhased06 = new TimePhasedFinancial();
        estimateTimePhased06.setYear(new Integer(currentYear));
        estimateTimePhased06.setBudgetType(BudgetType.Estimate);
        estimateTimePhased06.setMonth01Value(new Double(2));
        estimateTimePhased06.setMonth02Value(new Double(2));
        estimateTimePhased06.setMonth03Value(new Double(2));
        estimateTimePhased06.setMonth04Value(new Double(2));
        estimateTimePhased06.setMonth05Value(new Double(2));
        estimateTimePhased06.setMonth06Value(new Double(2));
        estimateTimePhased06.setMonth07Value(new Double(2));
        estimateTimePhased06.setMonth08Value(new Double(2));
        estimateTimePhased06.setMonth09Value(new Double(2));
        estimateTimePhased06.setMonth10Value(new Double(2));
        estimateTimePhased06.setMonth11Value(new Double(2));
        estimateTimePhased06.setMonth12Value(new Double(2));

        // set monthly actual values for 2006; yearly total is 12
        TimePhasedFinancial actualTimePhased06 = new TimePhasedFinancial();
        actualTimePhased06.setYear(new Integer(currentYear));
        actualTimePhased06.setBudgetType(BudgetType.Actual);
        actualTimePhased06.setMonth01Value(new Double(1));
        actualTimePhased06.setMonth02Value(new Double(1));
        actualTimePhased06.setMonth03Value(new Double(1));
        actualTimePhased06.setMonth04Value(new Double(1));
        actualTimePhased06.setMonth05Value(new Double(1));
        actualTimePhased06.setMonth06Value(new Double(1));
        actualTimePhased06.setMonth07Value(new Double(1));
        actualTimePhased06.setMonth08Value(new Double(1));
        actualTimePhased06.setMonth09Value(new Double(1));
        actualTimePhased06.setMonth10Value(new Double(1));
        actualTimePhased06.setMonth11Value(new Double(1));
        actualTimePhased06.setMonth12Value(new Double(1));

        expenseWorksheet.setEstimateTimePhased(new TimePhasedFinancial[]
        {
            estimateTimePhased06
        });

        expenseWorksheet.setActualTimePhased(new TimePhasedFinancial[]
        {
            actualTimePhased06
        });

        //adjusting scope for timephase and saving
        wScope.setWorksheetFinancials(finScope);
        wScope.getWorksheetFinancials().setChargeCode(true);

        result = save(reloadedProject, wScope);

        double[] values2 =
        {
                2.4, 0, 2.4, 2.4, 2.4, //Hours
                10, 0, 5, 5, 10, //AverageCost
                24, 12, 12, 12, 24, //ExpenseLabor
                0.30, 0, 0.30, 0.30, 0.30
        //FTE
        };
        printResult(result);
        validateResult(result, values2);

        //----Loading the ExpenseLabor worksheet for validation

        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            expenseWorksheet, finScope);
        assertNotNull(reloaded);
        assertNotNull(reloaded.getEstimate());
        assertEquals(reloaded.getEstimate(), new Double(24.0)); // successful
        assertEquals(reloaded.getActual(), new Double(12.0)); // successful

        //--Re-Modifying ExpenseLabor which should modified the
        // timephase too

        Proposal reloadedProposal = (Proposal) loadFromID(wMainProposal, wScope);
        assertNotNull(reloadedProposal);
        assertNotNull(reloadedProposal.getWorksheetFinancials());

        WorksheetFinancial financialsReloaded[] = reloadedProposal
            .getWorksheetFinancials();

        WorksheetFinancial reloadWF_EL = getMatchingWorksheetinancial(
            "EXPENSE_LABOR", financialsReloaded);

        //setting new value for ExpenseLabor Actual
        reloadWF_EL.setActual(new Double(50));

        result = save(reloadedProposal, wScope);

        double[] values3 =
        {
                2.4, 0, 2.4, 2.4, 2.4, //Hours
                10, 0, -10.83, 0, 20.83, //AverageCost
                24, 50, -26, 0, 50, //ExpenseLabor
                0.30, 0, 0.30, 0.30, 0.30
        //FTE
        };
        printResult(result);
        validateResult(result, values3);

        //----Re-Loading again ExpenseLabor for validations

        reloaded = (WorksheetFinancial) loadFromID(expenseWorksheet, finScope);

        assertNotNull(reloaded);

        assertNotNull(reloaded.getEstimate());

        assertEquals(reloaded.getEstimate(), new Double(24.0)); // successful
        assertEquals(reloaded.getActual(), new Double(50.0)); // successful

        assertEquals(FinancialUtil.getCurrentMonthTimePhased(reloaded
            .getActualTimePhased()[0]), new Double(50));// successful
        assertEquals(FinancialUtil.getCurrentMonthTimePhased(reloaded
            .getEstimateTimePhased()[0]), new Double(2));// successful

        assertEquals(reloaded.getActualTimePhased()[0].getTotalForYear(),
            new Double(50));// successful
        assertEquals(reloaded.getEstimateTimePhased()[0].getTotalForYear(),
            new Double(24));// successful        

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());
    }

    //This test is working fine for the defect that was presented but while doing the
    //fix we encountered another defect that should be fixed. Two versions pf this test exist
    //
    // a) Saving the parent proposal with worksheetFinancials that contains Timephased infos
    // b) Saving the timephased infos directly individually
    //
    // This is version (b)
    //
    //After creating the worksheet and saving it, we created timephased infos and added to the worksheet 
    //and saved. By doing this, the total of the timephased expenseLabor will replace the worksheet value of
    //expenseLabor which is good. BUT we need to re-calculate Hour based on the values of EL and AVG for 
    //Estimate and Actual and re-calculate the whole matrix. This is a special case so we need to detect
    //when that happens and call a method to do the above.

    public void testDefect_53937_savingTimePhased() throws Exception
    {
        //------------Part 1: Create a proposal with Worksheetfinancials
        // attached to it and save.

        init();
        

        wWSheetFinancial[AVG_COST].setEstimate(new Double(10));
        wWSheetFinancial[EXP_LABOR].setEstimate(new Double(100));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        double[] values =
        {
                10, 0, 10, 10, 10, //Hours
                10, 0, 10, 10, 10, //AverageCost
                100, 0, 100, 100, 100, //ExpenseLabor
                1.25, 0, 1.25, 1.25, 1.25
        //FTE
        };
        printResult(result);
        validateResult(result, values);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());

        //-----------Part 2: Get the proposal that was saved and attached an estimate Timephased on it.

        Proposal reloadedProject = (Proposal) result.getRpmObject();
        WorksheetFinancial expenseWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_LABOR", reloadedProject.getWorksheetFinancials());

        //----------Part 3: create timephased informations and bind it to the
        // correct worksheetfinancial and save with
        //--------- a financial scope.  They are going to be saved one at a time.

        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setEstimateTimePhased(true);
        finScope.setActualTimePhased(true);
        finScope.setParent(new WorkElementScope());

        Calendar cal = new GregorianCalendar();
        int currentYear = cal.get(Calendar.YEAR);

        // create estimates for current Year
        TimePhasedFinancial estimateTimePhased = new TimePhasedFinancial();
        estimateTimePhased.setYear(new Integer(currentYear));
        estimateTimePhased.setBudgetType(BudgetType.Estimate);
        estimateTimePhased.setMonth01Value(new Double(2));
        estimateTimePhased.setMonth02Value(new Double(2));
        estimateTimePhased.setMonth03Value(new Double(2));
        estimateTimePhased.setMonth04Value(new Double(2));
        estimateTimePhased.setMonth05Value(new Double(2));
        estimateTimePhased.setMonth06Value(new Double(2));
        estimateTimePhased.setMonth07Value(new Double(2));
        estimateTimePhased.setMonth08Value(new Double(2));
        estimateTimePhased.setMonth09Value(new Double(2));
        estimateTimePhased.setMonth10Value(new Double(2));
        estimateTimePhased.setMonth11Value(new Double(2));
        estimateTimePhased.setMonth12Value(new Double(2));

        estimateTimePhased.setParent(expenseWorksheet);

        // set monthly actual values for current Year; yearly total is 12
        TimePhasedFinancial actualTimePhased = new TimePhasedFinancial();
        actualTimePhased.setYear(new Integer(currentYear));
        actualTimePhased.setBudgetType(BudgetType.Actual);
        actualTimePhased.setMonth01Value(new Double(1));
        actualTimePhased.setMonth02Value(new Double(1));
        actualTimePhased.setMonth03Value(new Double(1));
        actualTimePhased.setMonth04Value(new Double(1));
        actualTimePhased.setMonth05Value(new Double(1));
        actualTimePhased.setMonth06Value(new Double(1));
        actualTimePhased.setMonth07Value(new Double(1));
        actualTimePhased.setMonth08Value(new Double(1));
        actualTimePhased.setMonth09Value(new Double(1));
        actualTimePhased.setMonth10Value(new Double(1));
        actualTimePhased.setMonth11Value(new Double(1));
        actualTimePhased.setMonth12Value(new Double(1));

        actualTimePhased.setParent(expenseWorksheet);

        expenseWorksheet.setEstimateTimePhased(new TimePhasedFinancial[]
        {
            estimateTimePhased
        });

        expenseWorksheet.setActualTimePhased(new TimePhasedFinancial[]
        {
            actualTimePhased
        });

        TimePhasedFinancialScope tpFinScope = new TimePhasedFinancialScope();
        tpFinScope.setParent(finScope);

        //-----------------Validating saving for estimateTimePhased06 only

        SaveResult estimateTimePhasedResult = save(estimateTimePhased,
            tpFinScope);

        assertNotNull(estimateTimePhasedResult);
        assertNotNull(estimateTimePhasedResult.getRpmObject());
        assertTrue(estimateTimePhasedResult.getRpmObject() instanceof TimePhasedFinancial);
        assertTrue(estimateTimePhasedResult.isSuccessful());

        WorksheetFinancial reloadedEstimate = (WorksheetFinancial) loadFromID(
            expenseWorksheet, finScope);

        assertNotNull(reloadedEstimate);

        assertEquals(FinancialUtil.nullToZero(reloadedEstimate.getEstimate()),
            new Double(24.0)); // successful
        assertEquals(FinancialUtil.nullToZero(reloadedEstimate.getActual()),
            new Double(0.0)); // successful        
        assertEquals(FinancialUtil.nullToZero(reloadedEstimate.getETC()),
            new Double(24.0)); // successful

        assertNotNull(reloadedEstimate.getEstimateTimePhased());
        assertEquals(FinancialUtil.nullToZero(reloadedEstimate
            .getEstimateTimePhased()[0].getTotalForYear()), new Double(24));

        //---------------Validating saving for actualTimePhased06 only

        SaveResult actualTimePhasedResult = save(actualTimePhased, tpFinScope);

        assertNotNull(actualTimePhasedResult);
        assertNotNull(actualTimePhasedResult.getRpmObject());
        assertTrue(actualTimePhasedResult.getRpmObject() instanceof TimePhasedFinancial);
        assertTrue(actualTimePhasedResult.isSuccessful());

        WorksheetFinancial reloadedActual = (WorksheetFinancial) loadFromID(
            expenseWorksheet, finScope);

        assertNotNull(reloadedActual);

        assertEquals(FinancialUtil.nullToZero(reloadedActual.getEstimate()),
            new Double(24.0)); // successful
        assertEquals(FinancialUtil.nullToZero(reloadedActual.getActual()),
            new Double(12.0)); // successful        
        assertEquals(FinancialUtil.nullToZero(reloadedActual.getETC()),
            new Double(12.0)); // successful

        assertNotNull(reloadedActual.getEstimateTimePhased());
        assertNotNull(reloadedActual.getActualTimePhased());
        assertEquals(FinancialUtil.nullToZero(reloadedActual
            .getEstimateTimePhased()[0].getTotalForYear()), new Double(24));
        assertEquals(FinancialUtil.nullToZero(reloadedActual
            .getActualTimePhased()[0].getTotalForYear()), new Double(12));

        //-----------Loading Parent to verify the whole matrix

        wScope.setWorksheetFinancials(finScope);
        wScope.getWorksheetFinancials().setChargeCode(true);

        Proposal reloadedProposal = (Proposal) loadFromID(wMainProposal, wScope);
        assertNotNull(reloadedProposal);

        SaveResult mainResult = new SaveResult();
        mainResult.setRpmObject(reloadedProposal);

        double[] mainValues =
        {
                2.4, 0, 2.4, 2.4, 2.4, //Hours
                10, 0, 5, 5, 10, //AverageCost
                24, 12, 12, 12, 24, //ExpenseLabor
                0.30, 0, 0.30, 0.30, 0.30
        //FTE
        };
        printResult(mainResult);
        validateResult(mainResult, mainValues);

        //--Re-Loading the parent proposal and re-Modifying ExpenseLabor which should modified the
        // timephase too

        reloadedProposal = (Proposal) loadFromID(wMainProposal, wScope);
        assertNotNull(reloadedProposal);
        assertNotNull(reloadedProposal.getWorksheetFinancials());

        WorksheetFinancial financialsReloaded[] = reloadedProposal
            .getWorksheetFinancials();

        WorksheetFinancial reloadWF_EL = getMatchingWorksheetinancial(
            "EXPENSE_LABOR", financialsReloaded);

        reloadWF_EL.setActual(new Double(50));

        result = save(reloadedProposal, wScope);

        double[] values3 =
        {
                2.4, 0, 2.4, 2.4, 2.4, //Hours
                10, 0, -10.83, 0, 20.83, //AverageCost
                24, 50, -26, 0, 50, //ExpenseLabor
                0.30, 0, 0.30, 0.30, 0.30
        //FTE
        };
        printResult(result);
        validateResult(result, values3);

        //-----------Validating that worksheet and timephased infos are correct.

        WorksheetFinancial reloadedWS = (WorksheetFinancial) loadFromID(
            expenseWorksheet, finScope);
        assertNotNull(reloadedWS);

        assertNotNull(reloadedWS.getEstimate());

        assertEquals(reloadedWS.getEstimate(), new Double(24.0)); // successful
        assertEquals(reloadedWS.getActual(), new Double(50.0)); // successful

        assertEquals(FinancialUtil.getCurrentMonthTimePhased(reloadedWS
            .getActualTimePhased()[0]), new Double(50));// successful
        assertEquals(FinancialUtil.getCurrentMonthTimePhased(reloadedWS
            .getEstimateTimePhased()[0]), new Double(2));// successful

        assertEquals(reloadedWS.getActualTimePhased()[0].getTotalForYear(),
            new Double(50));// successful
        assertEquals(reloadedWS.getEstimateTimePhased()[0].getTotalForYear(),
            new Double(24));// successful        

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());
    }

    private WorksheetFinancial getMatchingWorksheetinancial(String chargecode,
            WorksheetFinancial[] financial)
    {
        int i = -1;
        int index = -1;

        WorksheetFinancial wrongWorksheet = new WorksheetFinancial();
        wrongWorksheet.setID("");

        if (chargecode == null || financial == null)
        {
            return wrongWorksheet;
        }
        else
        {
            for (i = 0; i < financial.length; i++)
            {
                if (financial[i].getChargeCode().getID().startsWith(chargecode))
                {
                    index = i;
                    break;
                }
            }
            return financial[index];
        }
    }

    public void testDefect_47934() throws Exception
    {
        //With RPM Client, adding Expense Charge Codes with values
        // automatically add the associated values in the Time-Phased tab.
        // When adding Expense Charge Codes with values via WS, Time-Phased
        // values are not added (e.g. myWorksheet.estimateTimePhased = null)
        // although myWorksheet.TimePhasedType is set to AutoTimephased.

        //------------Part 1: Create a proposal with Worksheetfinancials
        // attached to it and save.

        init();
        

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        printResult(result);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());

        //-----------Part 2: Get the proposal that was saved.

        Proposal reloadedProject = (Proposal) result.getRpmObject();

        //extracting expenseLabor from reloaded proposal
        WorksheetFinancial expenseLabor = getMatchingWorksheetinancial(
            "EXPENSE_LABOR", reloadedProject.getWorksheetFinancials());
        assertNotNull(expenseLabor);

        //extracting expenseHour from reloaded proposal
        WorksheetFinancial expenseHour = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", reloadedProject.getWorksheetFinancials());
        assertNotNull(expenseHour);

        //setting up new values for expenseHour.Estimate and
        // expenseLabour.Estimate
        int wsfHoursValue = 100;
        int wsfExpLabValue = 200;

        expenseHour.setEstimate(new Double(wsfHoursValue));
        expenseLabor.setEstimate(new Double(wsfExpLabValue));

        //activating auto timephased to follow defect description
        expenseHour.setTimePhased(TimePhasedType.AutoTimephased);
        expenseLabor.setTimePhased(TimePhasedType.AutoTimephased);

        //creating scope with timephased informations
        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setEstimateTimePhased(true);
        finScope.setActualTimePhased(true);
        finScope.setParent(new WorkElementScope());

        //adjusting scope for timephase and saving
        wScope.setWorksheetFinancials(finScope);
        wScope.getWorksheetFinancials().setChargeCode(true);

        result = save(reloadedProject, wScope);

        //---Validating that the timephased infos have been created for the worksheet

        assertNotNull(result);
        assertTrue(result.getRpmObject() instanceof Proposal);

        Proposal loadedProposal = (Proposal) result.getRpmObject();

        expenseLabor = getMatchingWorksheetinancial("EXPENSE_LABOR",
            loadedProposal.getWorksheetFinancials());

        assertNotNull(expenseLabor);

        expenseHour = getMatchingWorksheetinancial("EXPENSE_HOURS",
            loadedProposal.getWorksheetFinancials());

        assertNotNull(expenseHour);

        assertNotNull(expenseLabor.getEstimateTimePhased());
        assertNotNull(expenseHour.getEstimateTimePhased());

        assertEquals(expenseLabor.getEstimateTimePhased()[0].getTotalForYear(),
            new Double(200.0)); // successful
        assertEquals(expenseHour.getEstimateTimePhased()[0].getTotalForYear(),
            new Double(100.0)); // successful

        printResult(result);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());
    }

    //This test will create a project and save it, then create a non-system worksheetFinancial and attached it
    //to it and save again. A chargeCode will be created with some timephased infos for CURRENT Year and be bind to it, this time only
    //the timephased wiil be save. What we are looking for is to verify that the total of the timephased is going
    //to replace the value of the related worksheet even though the timephased are for last Year. Then we are going to
    //modify the worksheet value which should create new timephased infos for the current month of the current year
    // and reset the timephased for last Year.
    public void testCreateportfolio_Currentyear() throws Exception
    {
        //creating scope for timephased
        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setEstimateTimePhased(true);
        finScope.setParent(new WorkElementScope());

        // create a parent Project and WorksheetFinancial
        Project project = new Project();
        project.setName(HelperTestUtilities.createName());

        SaveResult projectResult = save(project, null);

        assertNotNull(projectResult);
        assertNotNull(projectResult.getRpmObject());
        assertNull(projectResult.getErrors());
        assertEquals(projectResult.isSuccessful(), true);

        //reloading saved project
        Project savedProject = (Project) projectResult.getRpmObject();

        //adding non-system worksheetFinancial
        WorksheetFinancial worksheetFinancial = new WorksheetFinancial();
        worksheetFinancial.setParent(savedProject);

        savedProject.setWorksheetFinancials(new WorksheetFinancial[]
        {
            worksheetFinancial
        });

        // create and set a charge code
        ChargeCode QAExpense = ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this);

        worksheetFinancial.setChargeCode(QAExpense);

        //testSaveTimephasedOnlyIgnoreParentModification
        //save the parent first
        SaveResult worksheetFinancialResult = save(worksheetFinancial, finScope);

        //validating that save is successfull
        assertNotNull(worksheetFinancialResult);
        assertNotNull(worksheetFinancialResult.getRpmObject());
        assertNull(worksheetFinancialResult.getErrors());
        assertEquals(worksheetFinancialResult.isSuccessful(), true);

        worksheetFinancial = (WorksheetFinancial) worksheetFinancialResult
            .getRpmObject();

        Calendar cal = new GregorianCalendar();
        int currentYear = cal.get(Calendar.YEAR);

        // create estimates timephased infos for last Year
        TimePhasedFinancial estimateTimePhased05 = new TimePhasedFinancial();
        estimateTimePhased05.setParent(worksheetFinancial);
        estimateTimePhased05.setYear(new Integer(currentYear));
        estimateTimePhased05.setBudgetType(BudgetType.Estimate);
        estimateTimePhased05.setMonth01Value(new Double(5.0));
        estimateTimePhased05.setMonth02Value(new Double(10.0));

        // binding timephased to worksheet
        worksheetFinancial.setEstimateTimePhased(new TimePhasedFinancial[]
        {
            estimateTimePhased05
        });

        // we're going to ignore this change because we're not saving the
        // parent
        worksheetFinancial.setEstimate(new Double(1.0));

        TimePhasedFinancialScope tpFinScope = new TimePhasedFinancialScope();
        tpFinScope.setParent(finScope);

        //saving Timephased ONLY so should ignore worksheet modification
        SaveResult estimateTimePhased05Result = save(estimateTimePhased05,
            tpFinScope);

        //validating that saving is fine and that the total timephased has replaced the worksheet value.
        assertNotNull(estimateTimePhased05Result);
        assertNotNull(estimateTimePhased05Result.getRpmObject());
        assertNull(estimateTimePhased05Result.getErrors());
        assertEquals(estimateTimePhased05Result.isSuccessful(), true);

        //Reloading Worksheet
        RPMObject result = loadFromID(worksheetFinancial, finScope);

        assertNotNull(result);
        WorksheetFinancial reloaded = (WorksheetFinancial) result;

        assertNotNull(reloaded);
        assertNotNull(reloaded.getEstimate());
        
        assertEquals(reloaded.getEstimate(), new Double(15.0));
        assertEquals(reloaded.getActual(), new Double(0.0));
        assertEquals(reloaded.getRemaining(), new Double(15.0));
        assertEquals(reloaded.getETC(), new Double(15.0));
        assertEquals(reloaded.getEAC(), new Double(15.0));
        
        //set a worksheet estimate then save the parent first
        reloaded.setEstimate(new Double(11.0));

        SaveResult worksheetFinancialSecondResult = save(reloaded, finScope);

        //Validating that Timpehased infos for last Year has been reset and that timephased infos
        //for current month of the current year has been created properly.
        assertNotNull(worksheetFinancialSecondResult);
        assertNotNull(worksheetFinancialSecondResult.getRpmObject());
        assertNull(worksheetFinancialSecondResult.getErrors());
        assertEquals(worksheetFinancialSecondResult.isSuccessful(), true);

        worksheetFinancial = (WorksheetFinancial) worksheetFinancialSecondResult
            .getRpmObject();

        printResult(worksheetFinancialSecondResult);

        assertEquals(worksheetFinancial.getEstimate(), new Double(11.0));
        assertEquals(worksheetFinancial.getActual(), new Double(0.0));
        assertEquals(worksheetFinancial.getRemaining(), new Double(11.0));
        assertEquals(worksheetFinancial.getETC(), new Double(11.0));
        assertEquals(worksheetFinancial.getEAC(), new Double(11.0));

        assertEquals(worksheetFinancial.getEstimateTimePhased()[0]
            .getMonth01Value(), new Double(0.0));//current Year
        assertEquals(worksheetFinancial.getEstimateTimePhased()[0]
            .getMonth02Value(), new Double(0.0));//current Year
        assertEquals(worksheetFinancial.getEstimateTimePhased()[0]
            .getTotalForYear(), new Double(11.0));//current Year
    }
    
    //This test will create a project and save it, then create a non-system worksheetFinancial and attached it
    //to it and save again. A chargeCode will be created with some timephased infos for LAST Year and be bind to it, this time only
    //the timephased wiil be save. What we are looking for is to verify that the total of the timephased is going
    //to replace the value of the related worksheet even though the timephased are for last Year. Then we are going to
    //modify the worksheet value which should create new timephased infos for the current month of the current year
    // and reset the timephased for last Year.
    public void testCreateportfolio_LastYear() throws Exception
    {
        //creating scope for timephased
        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setEstimateTimePhased(true);
        finScope.setParent(new WorkElementScope());

        // create a parent Project and WorksheetFinancial
        Project project = new Project();
        project.setName(HelperTestUtilities.createName());

        SaveResult projectResult = save(project, null);

        assertNotNull(projectResult);
        assertNotNull(projectResult.getRpmObject());
        assertNull(projectResult.getErrors());
        assertEquals(projectResult.isSuccessful(), true);

        //reloading saved project
        Project savedProject = (Project) projectResult.getRpmObject();

        //adding non-system worksheetFinancial
        WorksheetFinancial worksheetFinancial = new WorksheetFinancial();
        worksheetFinancial.setParent(savedProject);

        savedProject.setWorksheetFinancials(new WorksheetFinancial[]
        {
            worksheetFinancial
        });

        // create and set a charge code
        ChargeCode QAExpense = ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this);

        worksheetFinancial.setChargeCode(QAExpense);

        //testSaveTimephasedOnlyIgnoreParentModification
        //save the parent first
        SaveResult worksheetFinancialResult = save(worksheetFinancial, finScope);

        //validating that save is successfull
        assertNotNull(worksheetFinancialResult);
        assertNotNull(worksheetFinancialResult.getRpmObject());
        assertNull(worksheetFinancialResult.getErrors());
        assertEquals(worksheetFinancialResult.isSuccessful(), true);

        worksheetFinancial = (WorksheetFinancial) worksheetFinancialResult
            .getRpmObject();

        Calendar cal = new GregorianCalendar();
        int currentYear = cal.get(Calendar.YEAR);

        // create estimates timephased infos for last Year
        TimePhasedFinancial estimateTimePhased05 = new TimePhasedFinancial();
        estimateTimePhased05.setParent(worksheetFinancial);
        estimateTimePhased05.setYear(new Integer(currentYear-1));
        estimateTimePhased05.setBudgetType(BudgetType.Estimate);
        estimateTimePhased05.setMonth01Value(new Double(5.0));
        estimateTimePhased05.setMonth02Value(new Double(10.0));

        // binding timephased to worksheet
        worksheetFinancial.setEstimateTimePhased(new TimePhasedFinancial[]
        {
            estimateTimePhased05
        });

        // we're going to ignore this change because we're not saving the
        // parent
        worksheetFinancial.setEstimate(new Double(1.0));

        TimePhasedFinancialScope tpFinScope = new TimePhasedFinancialScope();
        tpFinScope.setParent(finScope);

        //saving Timephased ONLY so should ignore worksheet modification
        SaveResult estimateTimePhased05Result = save(estimateTimePhased05,
            tpFinScope);

        //validating that saving is fine and that the total timephased has replaced the worksheet value.
        assertNotNull(estimateTimePhased05Result);
        assertNotNull(estimateTimePhased05Result.getRpmObject());
        assertNull(estimateTimePhased05Result.getErrors());
        assertEquals(estimateTimePhased05Result.isSuccessful(), true);

        //Reloading Worksheet
        RPMObject result = loadFromID(worksheetFinancial, finScope);

        assertNotNull(result);
        WorksheetFinancial reloaded = (WorksheetFinancial) result;

        assertNotNull(reloaded);
        assertNotNull(reloaded.getEstimate());
        
        assertEquals(reloaded.getEstimate(), new Double(15.0));
        assertEquals(reloaded.getActual(), new Double(0.0));
        assertEquals(reloaded.getRemaining(), new Double(15.0));
        assertEquals(reloaded.getETC(), new Double(15.0));
        assertEquals(reloaded.getEAC(), new Double(15.0));


        //set a worksheet estimate then save the parent first
        reloaded.setEstimate(new Double(11.0));

        SaveResult worksheetFinancialSecondResult = save(reloaded, finScope);

        //Validating that Timpehased infos for last Year has been reset and that timephased infos
        //for current month of the current year has been created properly.
        assertNotNull(worksheetFinancialSecondResult);
        assertNotNull(worksheetFinancialSecondResult.getRpmObject());
        assertNull(worksheetFinancialSecondResult.getErrors());
        assertEquals(worksheetFinancialSecondResult.isSuccessful(), true);

        worksheetFinancial = (WorksheetFinancial) worksheetFinancialSecondResult
            .getRpmObject();

        printResult(worksheetFinancialSecondResult);

        assertEquals(worksheetFinancial.getEstimate(), new Double(11.0));
        assertEquals(worksheetFinancial.getActual(), new Double(0.0));
        assertEquals(worksheetFinancial.getRemaining(), new Double(11.0));
        assertEquals(worksheetFinancial.getETC(), new Double(11.0));
        assertEquals(worksheetFinancial.getEAC(), new Double(11.0));

        assertTrue(worksheetFinancial.getEstimateTimePhased().length==2);
        
        //Processing Last Year TimePhases
        if (worksheetFinancial.getEstimateTimePhased()[0].getYear().intValue() == (currentYear - 1))
        {
            assertEquals(worksheetFinancial.getEstimateTimePhased()[0]
                .getMonth01Value(), new Double(0.0));// last Year
            assertEquals(worksheetFinancial.getEstimateTimePhased()[0]
                .getMonth02Value(), new Double(0.0));// last Year
            assertEquals(worksheetFinancial.getEstimateTimePhased()[0]
                .getTotalForYear(), new Double(0.0));// last Year
            assertEquals(worksheetFinancial.getEstimateTimePhased()[1]
                .getTotalForYear(), new Double(11));// current Year

        }
        else //Inverted
        {
            assertEquals(worksheetFinancial.getEstimateTimePhased()[1]
                .getMonth01Value(), new Double(0.0));// last Year
            assertEquals(worksheetFinancial.getEstimateTimePhased()[1]
                .getMonth02Value(), new Double(0.0));// last Year
            assertEquals(worksheetFinancial.getEstimateTimePhased()[1]
                .getTotalForYear(), new Double(0.0));// last Year
            assertEquals(worksheetFinancial.getEstimateTimePhased()[0]
                .getTotalForYear(), new Double(11));// current Year

        }
        
    }


    // Remaining FTE = Remaining hours divided by Project Calendar daily hours
    // divided by project remaining duration days
    // if the current date is before the estimated finish date. Remaining
    // duration is considered one day if the current date is after the estimated
    // finish date
    public void testDefect60206_FTE() throws Exception
    {
        Calendar todayDate = FinancialUtil.getTodayDate();
        Calendar startDate = FinancialUtil.getTodayDate();
        startDate.add(Calendar.DATE, -7);
        Calendar finishDate = FinancialUtil.getTodayDate();
        finishDate.add(Calendar.DATE, 7);

        init();
        

        wMainProposal.setEstimatedStartDate(startDate);
        wMainProposal.setEstimatedFinishDate(finishDate);

        double hourEstimate = 20;
        double hourActual = 5;
        double hourETC = 15;

        wWSheetFinancial[HOUR].setEstimate(new Double(hourEstimate));
        wWSheetFinancial[HOUR].setActual(new Double(hourActual));
        wWSheetFinancial[HOUR].setETC(new Double(hourETC));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        printResult(result);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());

        Proposal reloadedProject = (Proposal) result.getRpmObject();

        RPMCalendar cal5Hours = reloadedProject.getCalendar();

        // Calculating Durations based on start/finish dates
        int actualDuration = 0;
        
        //If the current time is after the working hour for the day
        //according to the calendar we must calculate duration from start
        // to today otherwise we calculate from start to yesterday.
        if (FinancialUtil.isEndOfDayCompareToCalendar(cal5Hours, getDatabaseTimestamp(sessionid)))
        {
            actualDuration = FinancialUtil.getCalWorkingDaysDuration(
                cal5Hours, startDate, todayDate);
        }
        else
        {
            actualDuration = FinancialUtil.getCalWorkingDaysDuration(
                cal5Hours, startDate, FinancialUtil
                    .getYesterdayDate());
        }


        int remainingDuration = FinancialUtil.getCalWorkingDaysDuration(
            cal5Hours, todayDate, finishDate);

        // Reloading HOURS and FTE for validations
        WorksheetFinancial wsHoursFinancial = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", reloadedProject.getWorksheetFinancials());

        WorksheetFinancial wsFteFinancial = getMatchingWorksheetinancial(
            "EXPENSE_FULL_TIME_EQUIVALENT", reloadedProject
                .getWorksheetFinancials());

        // Calculating the expected values
        Double expectedFTEActual = new Double(hourActual
                / cal5Hours.getHoursPerDay().doubleValue() / actualDuration);

        Double expectedFTERemaining = new Double(wsHoursFinancial
            .getRemaining().doubleValue()
                / cal5Hours.getHoursPerDay().doubleValue() / remainingDuration);

        // Validating
        assertEquals(wsFteFinancial.getActual(), FinancialUtil
            .roundDouble2Dec(expectedFTEActual));
        assertEquals(wsFteFinancial.getRemaining(), FinancialUtil
            .roundDouble2Dec(expectedFTERemaining));

    }

    public void testDefectModifyingETCUpdateEstimateAndActual()
            throws Exception
    {
        //------------Part 1: Create a proposal with Worksheetfinancials
        // attached to it and save.

        init();
        

        wWSheetFinancial[AVG_COST].setEstimate(new Double(10));
        wWSheetFinancial[EXP_LABOR].setEstimate(new Double(100));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        double[] values =
        {
                10, 0, 10, 10, 10, //Hours
                10, 0, 10, 10, 10, //AverageCost
                100, 0, 100, 100, 100, //ExpenseLabor
                1.25, 0, 1.25, 1.25, 1.25
        //FTE
        };
        printResult(result);
        validateResult(result, values);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());

        // -----------Part 2: Get the proposal that was saved and attached some
        // timephased to the EL worksheeFinancial

        Proposal reloadedProject = (Proposal) result.getRpmObject();

        WorksheetFinancial expenseWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_LABOR", reloadedProject.getWorksheetFinancials());

        expenseWorksheet.setETC(new Double(50));

        result = save(reloadedProject, wScope);
        double[] values2 =
        {
                10, 0, 10, 5, 5, //Hours
                10, 0, 10, 10, 10, //AverageCost
                100, 0, 100, 50, 50, //ExpenseLabor
                1.25, 0, 1.25, 0.63, 0.63
        //FTE
        };
        printResult(result);
        validateResult(result, values2);

    }

    public void testSaveBlankAndUpdate() throws Exception
    {
        init();
        

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 0, 0, 0, 0, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                0, 0, 0, 0, 0
        }; // FTE

        printResult(result);
        validateResult(result, values);

        //phase 2 : update values
        Proposal reloadedProject = (Proposal) result.getRpmObject();

        WorksheetFinancial expenseWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_LABOR", reloadedProject.getWorksheetFinancials());

        expenseWorksheet.setEstimate(new Double(20));

        result = null;
        result = save(reloadedProject, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values2 =
        {
                0, 0, 0, 0, 0, // Hours
                0, 0, 0, 0, 0, // AverageCost
                20, 0, 20, 20, 20, // ExpenseLabor
                0, 0, 0, 0, 0
        }; // FTE

        printResult(result);
        validateResult(result, values2);

    }

    public void testSaveEstimateActualAndETCSimultaneouslyHourOnly()
            throws Exception
    {
        init();
        

        wWSheetFinancial[HOUR].setEstimate(new Double(20));
        wWSheetFinancial[HOUR].setActual(new Double(10));
        wWSheetFinancial[HOUR].setETC(new Double(5));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                20, 10, 10, 5, 15, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                2.50, 1.25, 1.25, 0.63, 1.88
        }; // FTE

        printResult(result);
        validateResult(result, values);

    }

    public void testSaveEstimateActualAndETCSimultaneouslyHourAndAverage()
            throws Exception
    {
        init();
        

        wWSheetFinancial[HOUR].setEstimate(new Double(20));
        wWSheetFinancial[HOUR].setActual(new Double(10));
        wWSheetFinancial[HOUR].setETC(new Double(5));

        wWSheetFinancial[AVG_COST].setEstimate(new Double(20));
        wWSheetFinancial[AVG_COST].setActual(new Double(10));
        wWSheetFinancial[AVG_COST].setETC(new Double(5));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                20, 10, 10, 5, 15, // Hours
                20, 10, 30, 5, 8.33, // AverageCost
                400, 100, 300, 25, 125, // ExpenseLabor
                2.50, 1.25, 1.25, 0.63, 1.88
        }; // FTE

        printResult(result);
        validateResult(result, values);

    }
    
    // Description
    // update Worksheet Expense Labour Total should update hours estimate /
    // Actual Values not Avrg Cost / Hours values
    public void testDefect_59610() throws Exception
    {
        // ------------Part 1: Create a proposal with Worksheetfinancials
        // attached to it and save.

        init();
        

        wWSheetFinancial[HOUR].setEstimate(new Double(10));
        wWSheetFinancial[AVG_COST].setEstimate(new Double(20));

        wMainProposal.setWorksheetFinancials(wWSheetFinancial);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        double[] values =
        {
                10, 0, 10, 10, 10, // Hours
                20, 0, 20, 20, 20, // AverageCost
                200, 0, 200, 200, 200, // ExpenseLabor
                1.25, 0, 1.25, 1.25, 1.25
        // FTE
        };
        printResult(result);
        validateResult(result, values);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());

        // -----------Part 2: Get the proposal that was saved and update
        //Expenselabor and verify that Avg is the same but Hours has been
        //calculated accorded to: Hour = EL / Avg
        Proposal reloadedProject = (Proposal) result.getRpmObject();

        WorksheetFinancial expenseWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_LABOR", reloadedProject.getWorksheetFinancials());

        expenseWorksheet.setEstimate(new Double(400));

        result = save(reloadedProject, wScope);
        double[] values2 =
        {
                20, 0, 20, 20, 20, //Hours
                20, 0, 20, 20, 20, //AverageCost
                400, 0, 400, 400, 400, //ExpenseLabor
                2.50, 0, 2.50, 2.50, 2.50
        //FTE
        };
        printResult(result);
        validateResult(result, values2);

    }
    
    
    
    public void testDefect_59831() throws Exception
    {
        init();
        
        
        wMainProposal.setWorksheetFinancials(wWSheetFinancial);
        
        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        double[] values =
        {
                0, 0, 0, 0, 0, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                0, 0, 0, 0, 0
        }; // FTE

        printResult(result);
        validateResult(result, values);
        
        Proposal reloadedProject = (Proposal) result.getRpmObject();

        WorksheetFinancial expenseHoursWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", reloadedProject.getWorksheetFinancials());
        
        WorksheetFinancial expenseAvgWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_AVERAGE_COST", reloadedProject.getWorksheetFinancials());


        expenseHoursWorksheet.setEstimate(new Double(10));
        expenseAvgWorksheet.setEstimate(new Double(20));
        
        result = save(reloadedProject, wScope);
        double[] values2 =
        {
                10, 0, 10, 10, 10, //Hours
                20, 0, 20, 20, 20, //AverageCost
                200, 0, 200, 200, 200, //ExpenseLabor
                1.25, 0, 1.25, 1.25, 1.25
        //FTE
        };
        printResult(result);
        validateResult(result, values2);
    }
    
    public void testDefect_61272_AVG_UPDATE() throws Exception
    {
        // --------------------INIT---------------//

        Proposal proposal = null;

        // Proposal
        proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);

        WorksheetFinancial[] wsFinancials = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

        WorkElementScope wes = new WorkElementScope();
        wes.setWorksheetFinancials(new FinancialScope());
        wes.getWorksheetFinancials().setChargeCode(true);

        // Linking objects together
        proposal.setWorksheetFinancials(wsFinancials);
        
        WorksheetFinancial expenseHoursWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", proposal.getWorksheetFinancials());
        
        WorksheetFinancial expenseAvgWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_AVERAGE_COST", proposal.getWorksheetFinancials());

        expenseAvgWorksheet.setEstimate(new Double(10));
        expenseHoursWorksheet.setEstimate(new Double(10));


        // creating proper scoping
        WorkElementScope wess = new WorkElementScope();
        wess.setWorksheetFinancials(new FinancialScope());
        wess.getWorksheetFinancials().setChargeCode(true);
        
        SaveResult result = null;
        result = save(proposal, wess);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());        
        
        // --------------------------Test---------------------------------|
        
        Proposal loaded = (Proposal) result.getRpmObject();
        WorksheetFinancial expenseAvgWorksheetReloaded = getMatchingWorksheetinancial(
            "EXPENSE_AVERAGE_COST", loaded.getWorksheetFinancials());

        expenseAvgWorksheetReloaded.setEstimate(new Double(20));
        
        result = null;
        result = save(loaded, wess);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

    }
    

    public void testDefect_61272() throws Exception
    {
        Proposal proposal = null;

        // Proposal
        proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);

        WorksheetFinancial[] wsFinancials = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

        WorkElementScope wes = new WorkElementScope();
        wes.setWorksheetFinancials(new FinancialScope());
        wes.getWorksheetFinancials().setChargeCode(true);

        // Linking objects together
        proposal.setWorksheetFinancials(wsFinancials);
        
        WorksheetFinancial expenseHoursWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", proposal.getWorksheetFinancials());
        
        WorksheetFinancial expenseAvgWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_AVERAGE_COST", proposal.getWorksheetFinancials());

        expenseAvgWorksheet.setEstimate(new Double(10));
        expenseHoursWorksheet.setEstimate(new Double(10));


        // creating proper scoping
        WorkElementScope wess = new WorkElementScope();
        wess.setWorksheetFinancials(new FinancialScope());
        wess.getWorksheetFinancials().setChargeCode(true);
        
        SaveResult result = null;
        result = save(proposal, wess);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());       
        

        double[] values =
        {
                10, 0, 10, 10, 10, //Hours
                10, 0, 10, 10, 10, //AverageCost
                100, 0, 100, 100, 100, //ExpenseLabor
                1.25, 0, 1.25, 1.25, 1.25
        //FTE
        };
        printResult(result);
        validateResult(result, values);

        
        // --------------------------Test---------------------------------|
        
        Proposal loaded = (Proposal) result.getRpmObject();
        WorksheetFinancial expenseAvgWorksheetReloaded = getMatchingWorksheetinancial(
            "EXPENSE_AVERAGE_COST", loaded.getWorksheetFinancials());

        expenseAvgWorksheetReloaded.setEstimate(new Double(20));
        
        result = null;
        result = save(loaded, wess);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());
        

        double[] values2 =
        {
                10, 0, 10, 10, 10, //Hours
                20, 0, 20, 20, 20, //AverageCost
                200, 0, 200, 200, 200, //ExpenseLabor
                1.25, 0, 1.25, 1.25, 1.25
        //FTE
        };
        printResult(result);
        validateResult(result, values2);
        
    }
    
    public void testDefect_59833() throws Exception
    {
        Proposal proposal = null;
        double expLabValue = 200.00;
        double avgCostValue = 50.00;

        // Proposal
        proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);

        WorksheetFinancial[] wsFinancials = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

        WorkElementScope wes = new WorkElementScope();
        wes.setWorksheetFinancials(new FinancialScope());
        wes.getWorksheetFinancials().setChargeCode(true);

        // Linking objects together
        proposal.setWorksheetFinancials(wsFinancials);

        WorksheetFinancial expenseLaborWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_LABOR", proposal.getWorksheetFinancials());

        WorksheetFinancial expenseAvgWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_AVERAGE_COST", proposal.getWorksheetFinancials());

        expenseAvgWorksheet.setEstimate(new Double(avgCostValue));
        expenseLaborWorksheet.setEstimate(new Double(expLabValue));

        // creating proper scoping
        WorkElementScope wess = new WorkElementScope();
        wess.setWorksheetFinancials(new FinancialScope());
        wess.getWorksheetFinancials().setChargeCode(true);

        SaveResult result = null;
        result = save(proposal, wess);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        WorksheetFinancial expenseHourWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", ((Proposal) result.getRpmObject())
                .getWorksheetFinancials());

        expenseLaborWorksheet = getMatchingWorksheetinancial("EXPENSE_LABOR",
            ((Proposal) result.getRpmObject()).getWorksheetFinancials());

        expenseAvgWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_AVERAGE_COST", ((Proposal) result.getRpmObject())
                .getWorksheetFinancials());

        assertTrue(expenseLaborWorksheet.getEstimate().doubleValue()
                / expenseAvgWorksheet.getEstimate().doubleValue() == expenseHourWorksheet
            .getEstimate().doubleValue());

        double[] values =
        {
                4, 0, 4, 4, 4, // Hours
                50, 0, 50, 50, 50, // AverageCost
                200, 0, 200, 200, 200, // ExpenseLabor
                0.50, 0, 0.50, 0.50, 0.50
        // FTE
        };
        printResult(result);
        validateResult(result, values);

    }
    
    public void testFinancialInputAsDouble() throws Exception
    {
        Proposal proposal = null;

        // Proposal
        proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPublished(ProjectPublishedType.None);

        WorksheetFinancial[] wsFinancials = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(proposal, this);

        WorkElementScope wes = new WorkElementScope();
        wes.setWorksheetFinancials(new FinancialScope());
        wes.getWorksheetFinancials().setChargeCode(true);

        // Linking objects together
        proposal.setWorksheetFinancials(wsFinancials);
        
        WorksheetFinancial expenseHoursWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_HOURS", proposal.getWorksheetFinancials());
        
        WorksheetFinancial expenseAvgWorksheet = getMatchingWorksheetinancial(
            "EXPENSE_AVERAGE_COST", proposal.getWorksheetFinancials());

        expenseAvgWorksheet.setEstimate(new Double(19));
        expenseAvgWorksheet.setActual(new Double(10));
        
        expenseHoursWorksheet.setEstimate(new Double(20));
        expenseHoursWorksheet.setActual(new Double(20));

        // creating proper scoping
        WorkElementScope wess = new WorkElementScope();
        wess.setWorksheetFinancials(new FinancialScope());
        wess.getWorksheetFinancials().setChargeCode(true);
        
        SaveResult result = null;
        result = save(proposal, wess);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());       
        

        double[] values =
        {
                20, 20, 0, 0, 20, //Hours
                19, 10, 0, 0, 19, //AverageCost
                380, 200, 180, 180, 380, //ExpenseLabor
                2.50, 2.50, 0, 0, 2.50
        //FTE
        };
        printResult(result);
        validateResult(result, values);

    }
    
    public void  testUpdateWorkElementWithWbsFinancial_Task() throws Exception
    {
        Proposal parent = null;

        // Proposal
        parent = new Proposal();
        parent.setName(HelperTestUtilities.createName());
        parent.setPublished(ProjectPublishedType.None);
        
        SaveResult sr = save(parent,new WorkElementScope());
        assertTrue(sr.getRpmObject() instanceof Proposal);
        parent = (Proposal)sr.getRpmObject();

        
        // create and set a charge code
        ChargeCode QAExpense = ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this);

        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setPublished(ProjectPublishedType.None);
        
        task.setParent(parent);
        parent.setChildren(new WorkElement[]{task});

        WbsFinancial wbsFin = new WbsFinancial();
        wbsFin.setChargeCode(QAExpense);
        wbsFin.setEstimate(new Double(20));
        
        task.setWbsFinancials(new WbsFinancial[]{wbsFin});
        
        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setParent(new WorkElementScope());
        
        WorkElementScope childScope = new WorkElementScope();
        childScope.setWbsFinancials(finScope);
        childScope.setParent(new WorkElementScope());
        
        WorkElementScope wes = new WorkElementScope();
        wes.setChildren(childScope);

        
        sr = save(task,childScope);
        assertTrue(sr.getRpmObject() instanceof Task);
        task = (Task)sr.getRpmObject();
        
        assertTrue(task.getWbsFinancials()[0].getEstimate().equals(new Double(20.0)));
        assertTrue(task.getWbsFinancials()[0].getActual().equals(new Double(0.0)));
        assertTrue(task.getWbsFinancials()[0].getRemaining().equals(new Double(20.0)));
        assertTrue(task.getWbsFinancials()[0].getETC().equals(new Double(20.0)));
        assertTrue(task.getWbsFinancials()[0].getEAC().equals(new Double(20.0)));
        
        printResult(sr);
        
    }
    
    public void  testUpdateWorkElementWithWbsFinancial_Milestone() throws Exception
    {
        Proposal parent = null;

        // Proposal
        parent = new Proposal();
        parent.setName(HelperTestUtilities.createName());
        parent.setPublished(ProjectPublishedType.None);
        
        SaveResult sr = save(parent,new WorkElementScope());
        assertTrue(sr.getRpmObject() instanceof Proposal);
        parent = (Proposal)sr.getRpmObject();

        
        // create and set a charge code
        ChargeCode QAExpense = ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this);

        Milestone milestone = new Milestone();
        milestone.setName(HelperTestUtilities.createName());
        milestone.setPublished(ProjectPublishedType.None);
        
        milestone.setParent(parent);
        parent.setChildren(new WorkElement[]{milestone});

        WbsFinancial wbsFin = new WbsFinancial();
        wbsFin.setChargeCode(QAExpense);
        wbsFin.setEstimate(new Double(20));
        
        milestone.setWbsFinancials(new WbsFinancial[]{wbsFin});
        
        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setParent(new WorkElementScope());
        
        WorkElementScope childScope = new WorkElementScope();
        childScope.setWbsFinancials(finScope);
        childScope.setParent(new WorkElementScope());
        
        WorkElementScope wes = new WorkElementScope();
        wes.setChildren(childScope);

        
        sr = save(milestone,childScope);
        assertTrue(sr.getRpmObject() instanceof Milestone);
        milestone = (Milestone)sr.getRpmObject();
        
        assertTrue(milestone.getWbsFinancials()[0].getEstimate().equals(new Double(20.0)));
        assertTrue(milestone.getWbsFinancials()[0].getActual().equals(new Double(0.0)));
        assertTrue(milestone.getWbsFinancials()[0].getRemaining().equals(new Double(20.0)));
        assertTrue(milestone.getWbsFinancials()[0].getETC().equals(new Double(20.0)));
        assertTrue(milestone.getWbsFinancials()[0].getEAC().equals(new Double(20.0)));
        
        printResult(sr);
        
    }

    

    

            
}
