/**
 * @author shahla
 */
package com.ibm.rpm.client.junit.financial.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Proposal;

public class TestTimeCodeAssignment extends ClientTestController 
{
    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(null, true);
    }
    
    public void testLoadParentFromXPath() throws AssertionError, Exception
    {
        // insert an element
        TimeCodeAssignment timeCodeAssignment = insertTimeCodeAssignment(null,
            true);

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/GenericProject[id = '"
                + timeCodeAssignment.getParent().getID()
                + "' and TimeCodeAssignment[id='" + timeCodeAssignment.getID()
                + "']]", null);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof GenericProject);
        assertEquals(timeCodeAssignment.getParent().getID(), ((GenericProject) objs[0])
            .getID());

    }
    
    public void testLoadFromXPathAsChild() throws AssertionError, Exception
    {
        // insert an element
        TimeCodeAssignment timeCodeAssignment = insertTimeCodeAssignment(null,
            true);

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/GenericProject[id='"
                + timeCodeAssignment.getParent().getID()
                + "']/TimeCodeAssignment[id='" + timeCodeAssignment.getID()
                + "']", null);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof TimeCodeAssignment);
        assertEquals(timeCodeAssignment.getID(), ((TimeCodeAssignment) objs[0])
            .getID());

    }

    private void insert(TimeCodeScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        // insert an element
        TimeCodeAssignment timeCodeAssignment = insertTimeCodeAssignment(scope,
            loadFromId);

        // delete the element
        deleteTimeCodeAssignment(timeCodeAssignment, loadFromId);
    }  
    
    private TimeCodeAssignment insertTimeCodeAssignment(TimeCodeScope scope,
            boolean loadFromID) throws AssertionError, Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);
        TimeCodeAssignment assignment = TimeCodeAssignmentFixture
            .createFixture(proposal, true, this);

        // Insert the element into the DB
        SingleResult result = save(assignment, scope);

        // compare the returned element
        TimeCodeAssignment returnedAssignment = (TimeCodeAssignment) result
            .getRpmObject();
        TimeCodeAssignmentFixture.compare(assignment, returnedAssignment);

        // reload the element and compare it
        returnedAssignment = (TimeCodeAssignment) load(returnedAssignment,
            scope, loadFromID);
        TimeCodeAssignmentFixture.compare(assignment, returnedAssignment);

        // return the element
        return returnedAssignment;
    }

    private void deleteTimeCodeAssignment(TimeCodeAssignment assignment,
            boolean loadFromId) throws Exception
    {
        // delete the element
        delete(assignment);

        // reload the deleted element
        TimeCodeAssignment deletedAssignment = loadDeletedTimeCodeAssignment(
            assignment, loadFromId);

        // validate deleted element
        assertNull(deletedAssignment);
    }

    private TimeCodeAssignment loadDeletedTimeCodeAssignment(
            TimeCodeAssignment assignment, boolean loadFromId) throws Exception
    {
        try
        {
            assignment = (TimeCodeAssignment) load(assignment, null, loadFromId);
            return assignment;
        }
        catch (Exception e)
        {
            // element does not exist
        }
        return null;
    }

    private RPMObject load(TimeCodeAssignment assignment, TimeCodeScope scope,
            boolean loadFromId) throws Exception
    {
        RPMObject object = null;

        if (loadFromId)
        {
            // load from an id
            TimeCodeAssignment obj = new TimeCodeAssignment();
            TimeCode timeCode = new TimeCode();
            timeCode.setID(assignment.getTimeCode().getID());
            obj.setID(assignment.getID());
            obj.setTimeCode(timeCode);
            object = loadFromID(obj, scope);
        }
        else
        {
            // load from an xpath
            RPMObject[] objs = loadFromXpath("/TimeCodeAssignment[id='"
                    + assignment.getID() + "']", scope);
            if (objs != null && objs.length > 0)

                object = objs[0];
        }

        return object;
    }
}