package com.ibm.rpm.client.junit.financial.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.TimePhasedFinancialFixture;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.scope.TimePhasedFinancialScope;

/**
 * @author ikusakov
 */
public class TestTimePhasedFinancial extends ClientTestController
{

    TimePhasedFinancial timephased;

    TimePhasedFinancialScope mainScope;

    public TestTimePhasedFinancial()
    {
        timephased = null;
        mainScope = null;
    }

    protected void setUp() throws Exception
    {
        ClientTestController.logger.info("setUp");
        super.setUp();
        mainScope = TimePhasedFinancialFixture.createScope();
        timephased = TimePhasedFinancialFixture.createWorksheetFixture(
                mainScope, this);
    }

    public void tearDown() throws Exception
    {
        ClientTestController.logger.info("tearDown");
        super.tearDown();
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        TimePhasedFinancial saved = (TimePhasedFinancial) save(timephased,
                mainScope).getRpmObject();
        assertNotNull(saved);
        TimePhasedFinancialFixture.compare(timephased, saved, null);
        saved.setParent(timephased.getParent());
        saved = (TimePhasedFinancial) loadFromID(saved, mainScope);
        assertTrue("different object", timephased != saved);
        assertNotNull(saved);
        assertNotNull(saved.getID());
        assertEquals(timephased.getParent().getParent().getID(), saved.getID());
        TimePhasedFinancialFixture.compare(timephased, saved, null);
    }

    public void testUpdateWithNoScope() throws AssertionError, Exception
    {
        TimePhasedFinancial saved = (TimePhasedFinancial) save(timephased,
                mainScope).getRpmObject();
        assertEquals(timephased.getParent().getID(), saved.getParent().getID());
        saved.setMonth01Value(new Double(
                saved.getMonth01Value().doubleValue() - 10D));
        saved.setMonth02Value(new Double(
                saved.getMonth02Value().doubleValue() + 10D));
        TimePhasedFinancial returned = (TimePhasedFinancial) save(saved,
                mainScope).getRpmObject();
        assertEquals(returned.getID(), saved.getID());
        TimePhasedFinancialFixture.compare(returned, saved, null);
        TimePhasedFinancial loaded = (TimePhasedFinancial) loadFromID(saved,
                mainScope);
        assertTrue("different object", loaded != saved);
        assertNotNull(loaded);
        assertNotNull(loaded.getID());
        assertEquals(loaded.getID(), saved.getID());
        TimePhasedFinancialFixture.compare(loaded, saved, null);
    }
}
