package com.ibm.rpm.client.junit.financial.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.financial.util.FinancialUtil;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

public class TestWorksheetSubElement extends ClientTestController
{
    private WorkElementScope wScope = null;

    private Proposal wMainProposal = null;

    private PortfolioScope wPFoScope = null;

    private Portfolio wMainPortfolio = null;

    private static final int HOUR = 0;

    private static final int AVG_COST = 1;

    public void setUp() throws Exception
    {
        super.setUp();

        init();
    }

    private void init() throws Exception
    {
        wPFoScope = new PortfolioScope();
        wScope = new WorkElementScope();

        // Setting Up Scope
        wScope.setWorksheetFinancials(new FinancialScope());
        wScope.getWorksheetFinancials().setChargeCode(true);
        wPFoScope.setProjects(wScope);

        // Creating PortFolio and Proposal
        SaveResult portfolioResult = null;

        wMainPortfolio = new Portfolio();
        wMainPortfolio.setName(HelperTestUtilities.createName());

        wMainProposal = new Proposal();
        wMainProposal.setName(HelperTestUtilities.createName());
        wMainProposal.setPortfolio(wMainPortfolio);

        wMainPortfolio.setProjects(new GenericProject[]
        {
            wMainProposal
        });

        // Saving PortFolio/Proposal according to scope
        portfolioResult = save(wMainPortfolio, wPFoScope);

        assertNotNull(portfolioResult);

        assertTrue(portfolioResult.getRpmObject() instanceof Portfolio);
        wMainPortfolio = (Portfolio) portfolioResult.getRpmObject();

        assertNotNull(wMainPortfolio.getProjects());
        assertTrue(wMainPortfolio.getProjects()[0] instanceof Proposal);
        wMainProposal = (Proposal) wMainPortfolio.getProjects()[0];

        wMainProposal.setAssignmentType(AssignmentType.Work);
        wMainProposal.setPublished(ProjectPublishedType.None);

    }

    /**
     * This utility method will print the Financial SubElement matrix after each test so we
     * can compare the result and make changes accordingly.
     * 
     * @param result
     * @throws RPMException
     */
    private void printResultForSubElement(WorksheetFinancial[] wf)
            throws RPMException
    {
        if (wf != null)
        {
            WorksheetFinancial[] financial = wf;

            String title = null;
            logger.debug("|------WORKSHEET SUB ELEMENT FINANCIAL MATRIX-----|");
            logger.debug("  ESTIMATE    ACTUAL      REMAINING   ETC     EAC");

            for (int i = 0; i < financial.length; i++)
            {
                if (financial[i].getChargeCode() != null)
                {
                    Integer dbCode;

                    dbCode = FinancialUtil.getDBCodeForChargeCode(financial[i]
                        .getChargeCode().getFinancialCategory());

                    switch (dbCode.intValue())
                    {
                    case FinancialUtil.EXPENSE_HOUR:
                        title = "HOU:   ";
                        break;
                    case FinancialUtil.EXPENSE_AVERAGE_COST:
                        title = "AVG:   ";
                        break;
                    case FinancialUtil.EXPENSE_LABOR:
                        title = "ELB:   ";
                        break;
                    case FinancialUtil.EXPENSE_FTE:
                        title = "FTE:   ";
                        break;

                    default:
                        title = financial[i].getChargeCode().getName();
                    }

                    logger.debug(""
                            + title
                            + "  "
                            + FinancialUtil.nullToZero(
                                financial[i].getSubElementEstimate()).doubleValue()
                            + "         "
                            + FinancialUtil
                                .nullToZero(financial[i].getSubElementActual())
                                .doubleValue()
                            + "         "
                            + FinancialUtil.nullToZero(
                                financial[i].getSubElementRemaining()).doubleValue()
                            + "         "
                            + FinancialUtil.nullToZero(financial[i].getSubElementETC())
                                .doubleValue()
                            + "         "
                            + FinancialUtil.nullToZero(financial[i].getSubElementEAC())
                                .doubleValue() + "\n");
                }
            }
        }
        else
        {
            logger.debug("|------WORKSHEET SUB ELEMENT FINANCIAL MATRIX-----|");
            logger.debug("              NO DATA AVAILABLE                    ");

        }

    }

    /**
     * This utility method will validate the result from an operation with the
     * expected expectedValues of the Financial SubElement matrix.
     * 
     * @param rs
     * @param expectedValues
     * @throws RPMException
     */
    private void validateResultForSubElement(WorksheetFinancial[] wf,
            double[] expectedValues,boolean noSubElementData) throws RPMException
    {
        assertNotNull(wf);
        assertNotNull(expectedValues);

        WorksheetFinancial[] financial = wf;

        if (financial != null)
        {

            WorksheetFinancial hourExpense = (WorksheetFinancial)FinancialUtil
                .getFinancialForChargeCode(FinancialUtil.EXPENSE_HOUR,
                    financial);

            WorksheetFinancial laborExpense = (WorksheetFinancial)FinancialUtil
                .getFinancialForChargeCode(FinancialUtil.EXPENSE_LABOR,
                    financial);

            WorksheetFinancial avgExpense = (WorksheetFinancial)FinancialUtil
                .getFinancialForChargeCode(FinancialUtil.EXPENSE_AVERAGE_COST,
                    financial);

            WorksheetFinancial fteExpense = (WorksheetFinancial)FinancialUtil
                .getFinancialForChargeCode(FinancialUtil.EXPENSE_FTE, financial);

            // expectedValues array validation
            assertTrue(expectedValues.length == 20);

            // financials validation            
            if (hourExpense == null && laborExpense == null
                    && avgExpense == null && fteExpense == null)
            {
                assertTrue(noSubElementData);
            }
            else
            {
                // HOUR

                assertTrue((hourExpense.getSubElementEstimate()).doubleValue() == (expectedValues[0])); // Estimate
                assertTrue((hourExpense.getSubElementActual()).doubleValue() == (expectedValues[1])); // Actual
                assertTrue((hourExpense.getSubElementRemaining()).doubleValue() == (expectedValues[2])); // Remaining
                assertTrue((hourExpense.getSubElementETC()).doubleValue() == (expectedValues[3]));    // ETC
                assertTrue((hourExpense.getSubElementEAC()).doubleValue() == (expectedValues[4]));// EAC

                // AVERAGE_COST
                assertTrue((avgExpense.getSubElementEstimate()).doubleValue() == (expectedValues[5])); // Estimate
                assertTrue((avgExpense.getSubElementActual()).doubleValue() == (expectedValues[6])); // Actual
                assertTrue((avgExpense.getSubElementRemaining()).doubleValue() == (expectedValues[7])); // Remaining
                assertTrue((avgExpense.getSubElementETC()).doubleValue() == (expectedValues[8]));    // ETC
                assertTrue((avgExpense.getSubElementEAC()).doubleValue() == (expectedValues[9])); // EAC

                // EXPENSE_LABOR
                assertTrue((laborExpense.getSubElementEstimate()).doubleValue() == (expectedValues[10])); //Estimate
                assertTrue((laborExpense.getSubElementActual()).doubleValue() == (expectedValues[11])); // Actual
                assertTrue((laborExpense.getSubElementRemaining()).doubleValue() == (expectedValues[12])); // Remaining
                assertTrue((laborExpense.getSubElementETC()).doubleValue() == (expectedValues[13])); // ETC
                assertTrue((laborExpense.getSubElementEAC()).doubleValue() == (expectedValues[14])); // EAC

            }
            
            
            // --It seems that FTE is not calculated for the SubElement
            // matrix
            // as seen in the client. We still calculate it anyway.
            // We will un-comments the lines below when the client does the
            // same.
            
            // FTE
            // assertTrue((fteExpense.getSubElementEstimate()).doubleValue() == (expectedValues[15]));// Estimate
            // assertTrue((fteExpense.getSubElementActual()).doubleValue() == (expectedValues[16])); // Actual
            // assertTrue((fteExpense.getSubElementRemaining()).doubleValue() == (expectedValues[17]));// Remaining
            // assertTrue((fteExpense.getSubElementETC()).doubleValue()== (expectedValues[18])); // ETC
            // assertTrue((fteExpense.getSubElementEAC()).doubleValue() == (expectedValues[19])); // EAC

        }
        else
        {
            logger.debug("Worksheet SubElement Data is empty...");
        }
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testCreateProposalWithNoChildren() throws Exception
    {
        //
        // |--Parent[Proposal]
        //

        //creating worksheet financial system charge codes array
        //add initial expectedValues for worksheet and bind them to the proposal.
        WorksheetFinancial[] wf = null;

        wf = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(wMainProposal,
                this);

        wf[HOUR].setEstimate(new Double(10));
        wf[AVG_COST].setEstimate(new Double(10));

        wf[HOUR].setActual(new Double(5));
        wf[AVG_COST].setActual(new Double(5));

        wMainProposal.setWorksheetFinancials(wf);

        //saving proposal with worksheet expectedValues
        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        Proposal parent = (Proposal) result.getRpmObject();

        //validating result for subElement
        double[] expectedValues =
        {
                0, 0, 0, 0, 0, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        validateResultForSubElement(parent.getWorksheetFinancials(),
            expectedValues,true);
        printResultForSubElement(parent.getWorksheetFinancials());
    }

    public void testCreateProposalWithDeliverableChildren() throws Exception
    {
        //
        // |--Parent[Proposal]
        //       | 
        //       |--Child[Deliverable]
        //

        // --- Phase 1: Create Parent and save.

        //creating worksheet financial system charge codes array
        //with initial expectedValues and bind them to parent.
        WorksheetFinancial[] wf = null;

        wf = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(wMainProposal,
                this);

        wf[HOUR].setEstimate(new Double(10));
        wf[AVG_COST].setEstimate(new Double(10));

        wf[HOUR].setActual(new Double(5));
        wf[AVG_COST].setActual(new Double(5));

        wMainProposal.setWorksheetFinancials(wf);

        //saving parent with worksheet informations
        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        Proposal parent = (Proposal) result.getRpmObject();

        //Validating financial subElement for parent
        double[] expectedValues =
        {
                0, 0, 0, 0, 0, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(parent.getWorksheetFinancials());
        validateResultForSubElement(parent.getWorksheetFinancials(),
            expectedValues,true);

        // --- Phase 2: Creating Child with own financial system charge codes
        //informations ,bind them to it and set as child of parent.

        // adding child in scope
        WorkElementScope childScope = new WorkElementScope();
        childScope.setWorksheetFinancials(new FinancialScope());
        childScope.getWorksheetFinancials().setChargeCode(true);
        wScope.setChildren(childScope);

        Deliverable child = new Deliverable();
        child.setName(HelperTestUtilities.createName());

        // Add child to parent proposal
        parent.setChildren(new WorkElement[]
        {
            child
        });

        // creating and binding worksheet Financial for child
        WorksheetFinancial[] childWF = null;
        childWF = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(child, this);

        childWF[HOUR].setEstimate(new Double(5));
        childWF[AVG_COST].setEstimate(new Double(5));

        childWF[HOUR].setActual(new Double(3));
        childWF[AVG_COST].setActual(new Double(3));

        child.setWorksheetFinancials(childWF);

        // saving parent with child
        result = null;
        result = save(parent, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Proposal);
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        parent = (Proposal) result.getRpmObject();
        assertNotNull(parent.getChildren());
        assertTrue(parent.getChildren()[0] instanceof Deliverable);
        Deliverable childDeliverable = (Deliverable) parent.getChildren()[0];

        // --- Phase 3: Validating financial SubElement for parent, child.

        // verifying parent sub element
        double[] expectedValuesParent =
        {
                5, 3, 2, 2, 5, // Hours
                5, 3, 8, 8, 5, // AverageCost
                25, 9, 16, 16, 25, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(parent.getWorksheetFinancials());
        validateResultForSubElement(parent.getWorksheetFinancials(),
            expectedValuesParent,false);

        // verifying child sub element
        double[] expectedValuesChild =
        {
                0, 0, 0, 0, 0, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(childDeliverable
            .getWorksheetFinancials());
        validateResultForSubElement(childDeliverable
            .getWorksheetFinancials(), expectedValuesChild,true);

    }

    public void testCreateProposalWithDeliverableChildrenAndWorkProductGrandChild()
            throws Exception
    {
        //
        // |--Parent[Proposal]
        //       | 
        //       |--Child[Deliverable]
        //            |
        //            |--GrandChild[WorkProduct]
        //

        // --- Phase 1: create parent with financials informations and save.

        //create financial system charge codes array  and bind them to parent
        WorksheetFinancial[] wf = null;

        wf = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(wMainProposal,
                this);

        wf[HOUR].setEstimate(new Double(10));
        wf[AVG_COST].setEstimate(new Double(10));

        wf[HOUR].setActual(new Double(5));
        wf[AVG_COST].setActual(new Double(5));

        wMainProposal.setWorksheetFinancials(wf);

        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        Proposal parent = (Proposal) result.getRpmObject();

        // --- Phase 2: create child with own financial informations

        // adding child in scope
        WorkElementScope childScope = new WorkElementScope();
        childScope.setWorksheetFinancials(new FinancialScope());
        childScope.getWorksheetFinancials().setChargeCode(true);
        wScope.setChildren(childScope);

        Deliverable child = new Deliverable();
        child.setName(HelperTestUtilities.createName());

        // Add child to parent proposal
        parent.setChildren(new WorkElement[]
        {
            child
        });

        // creating and binding worksheet Financial for child
        WorksheetFinancial[] childWF = null;
        childWF = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(child, this);

        childWF[HOUR].setEstimate(new Double(5));
        childWF[AVG_COST].setEstimate(new Double(5));

        childWF[HOUR].setActual(new Double(3));
        childWF[AVG_COST].setActual(new Double(3));

        child.setWorksheetFinancials(childWF);

        // saving parent with child
        result = null;
        result = save(parent, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Proposal);
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        parent = (Proposal) result.getRpmObject();
        assertNotNull(parent.getChildren());
        assertTrue(parent.getChildren()[0] instanceof Deliverable);
        Deliverable childDeliverable = (Deliverable) parent.getChildren()[0];

        // verifying parent sub element
        double[] expectedValuesParent0 =
        {
                5, 3, 2, 2, 5, // Hours
                5, 3, 8, 8, 5, // AverageCost
                25, 9, 16, 16, 25, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(parent.getWorksheetFinancials());
        validateResultForSubElement(parent.getWorksheetFinancials(),
            expectedValuesParent0,false);

        // verifying child sub element
        double[] expectedValuesChild0 =
        {
                0, 0, 0, 0, 0, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(childDeliverable
            .getWorksheetFinancials());
        validateResultForSubElement(childDeliverable
            .getWorksheetFinancials(), expectedValuesChild0,true);

        // --- Phase 3: create GrandChild with own financial informations

        // adding grandChild in scope
        WorkElementScope grandChildScope = new WorkElementScope();
        grandChildScope.setWorksheetFinancials(new FinancialScope());
        grandChildScope.getWorksheetFinancials().setChargeCode(true);
        wScope.getChildren().setChildren(grandChildScope);

        // creating grandChild workproduct

        WorkProduct grandChild = new WorkProduct();
        grandChild.setName(HelperTestUtilities.createName());

        // Add grandchild to parent Deliverable
        childDeliverable.setChildren(new WorkElement[]
        {
            grandChild
        });

        // creating and binding worksheet Financial for grandChild
        WorksheetFinancial[] grandChildWF = null;
        grandChildWF = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(grandChild,
                this);

        grandChildWF[HOUR].setEstimate(new Double(2));
        grandChildWF[AVG_COST].setEstimate(new Double(2));

        grandChildWF[HOUR].setActual(new Double(1));
        grandChildWF[AVG_COST].setActual(new Double(1));

        grandChild.setWorksheetFinancials(grandChildWF);

        // saving parent with child and grandChild
        result = null;
        result = save(parent, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Proposal);
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        parent = (Proposal) result.getRpmObject();
        assertNotNull(parent.getChildren());
        assertTrue(parent.getChildren()[0] instanceof Deliverable);
        childDeliverable = (Deliverable) parent.getChildren()[0];
        assertNotNull(childDeliverable.getChildren());
        assertTrue(childDeliverable.getChildren()[0] instanceof WorkProduct);
        WorkProduct grandChildWorkProduct = (WorkProduct) childDeliverable
            .getChildren()[0];

        // --- Phase 4: validating subElement for parent, child, grandchild

        // verifying parent sub element
        double[] expectedValuesParent =
        {
                5, 3, 2, 2, 5, // Hours
                5, 3, 8, 8, 5, // AverageCost
                25, 9, 16, 16, 25, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(parent.getWorksheetFinancials());
        validateResultForSubElement(parent.getWorksheetFinancials(),
            expectedValuesParent,false);

        // verifying child sub element
        double[] expectedValuesChild =
        {
                2, 1, 1, 1, 2, // Hours
                2, 1, 3, 3, 2, // AverageCost
                4, 1, 3, 3, 4, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(childDeliverable
            .getWorksheetFinancials());
        validateResultForSubElement(childDeliverable
            .getWorksheetFinancials(), expectedValuesChild,false);

        // verifying GrandChild sub element
        double[] expectedValuesGrandChild =
        {
                0, 0, 0, 0, 0, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(grandChildWorkProduct
            .getWorksheetFinancials());
        validateResultForSubElement(grandChildWorkProduct
            .getWorksheetFinancials(), expectedValuesGrandChild,true);

    }

    public void testCreateProposalWith2ChildrenAndWorkProductGrandChild()
            throws Exception
    {
        //
        // |--Parent[Proposal]
        //       | 
        //       |--Child1[Deliverable]
        //       |     |
        //       |     |--GrandChild[WorkProduct]
        //       |
        //       |--Child2[SummaryTask]
        //

        // --- Phase 1: create parent with financial informations and save.

        WorksheetFinancial[] wf = null;

        wf = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(wMainProposal,
                this);

        wf[HOUR].setEstimate(new Double(10));
        wf[AVG_COST].setEstimate(new Double(10));

        wf[HOUR].setActual(new Double(5));
        wf[AVG_COST].setActual(new Double(5));

        wMainProposal.setWorksheetFinancials(wf);

        // saving parent
        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        Proposal parent = (Proposal) result.getRpmObject();

        // --- Phase 2: creating child1 with own financial informations

        // adding child in scope
        WorkElementScope childScope = new WorkElementScope();
        childScope.setWorksheetFinancials(new FinancialScope());
        childScope.getWorksheetFinancials().setChargeCode(true);
        
        wScope.setChildren(childScope);

        Deliverable child = new Deliverable();
        child.setName(HelperTestUtilities.createName());

        // Add child to parent proposal
        parent.setChildren(new WorkElement[]
        {
            child
        });

        // creating and binding worksheet Financial for child
        WorksheetFinancial[] childWF = null;
        childWF = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(child, this);

        childWF[HOUR].setEstimate(new Double(5));
        childWF[AVG_COST].setEstimate(new Double(5));

        childWF[HOUR].setActual(new Double(3));
        childWF[AVG_COST].setActual(new Double(3));

        child.setWorksheetFinancials(childWF);

        // saving parent with child
        result = null;
        result = save(parent, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Proposal);
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        parent = (Proposal) result.getRpmObject();
        assertNotNull(parent.getChildren());
        assertTrue(parent.getChildren()[0] instanceof Deliverable);
        Deliverable childDeliverable = (Deliverable) parent.getChildren()[0];

        // verifying parent sub element
        double[] expectedValuesParent0 =
        {
                5, 3, 2, 2, 5, // Hours
                5, 3, 8, 8, 5, // AverageCost
                25, 9, 16, 16, 25, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(parent.getWorksheetFinancials());
        validateResultForSubElement(parent.getWorksheetFinancials(),
            expectedValuesParent0,false);

        // verifying child sub element
        double[] expectedValuesChild0 =
        {
                0, 0, 0, 0, 0, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(childDeliverable
            .getWorksheetFinancials());
        validateResultForSubElement(childDeliverable
            .getWorksheetFinancials(), expectedValuesChild0,true);

        // --- Phase 3: create grandChild for child1

        // adding grandChild in scope
        WorkElementScope grandChildScope = new WorkElementScope();
        grandChildScope.setWorksheetFinancials(new FinancialScope());
        grandChildScope.getWorksheetFinancials().setChargeCode(true);
        wScope.getChildren().setChildren(grandChildScope);

        // creating grandChild workproduct

        WorkProduct grandChild = new WorkProduct();
        grandChild.setName(HelperTestUtilities.createName());

        // Add child to parent Deliverable
        childDeliverable.setChildren(new WorkElement[]
        {
            grandChild
        });

        // creating and binding worksheet Financial for grandChild
        WorksheetFinancial[] grandChildWF = null;
        grandChildWF = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(grandChild,
                this);

        grandChildWF[HOUR].setEstimate(new Double(2));
        grandChildWF[AVG_COST].setEstimate(new Double(2));

        grandChildWF[HOUR].setActual(new Double(1));
        grandChildWF[AVG_COST].setActual(new Double(1));

        grandChild.setWorksheetFinancials(grandChildWF);

        // saving parent with child and grandChild
        result = null;
        result = save(parent, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Proposal);
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        parent = (Proposal) result.getRpmObject();
        assertNotNull(parent.getChildren());
        assertTrue(parent.getChildren()[0] instanceof Deliverable);
        childDeliverable = (Deliverable) parent.getChildren()[0];
        assertNotNull(childDeliverable.getChildren());
        assertTrue(childDeliverable.getChildren()[0] instanceof WorkProduct);
        WorkProduct grandChildWorkProduct = (WorkProduct) childDeliverable
            .getChildren()[0];

        // --- Phase 4: validating subelement infos for parent, child1, grandchild

        // verifying parent sub element
        double[] expectedValuesParent =
        {
                5, 3, 2, 2, 5, // Hours
                5, 3, 8, 8, 5, // AverageCost
                25, 9, 16, 16, 25, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(parent.getWorksheetFinancials());
        validateResultForSubElement(parent.getWorksheetFinancials(),
            expectedValuesParent,false);

        // verifying child sub element
        double[] expectedValuesChild =
        {
                2, 1, 1, 1, 2, // Hours
                2, 1, 3, 3, 2, // AverageCost
                4, 1, 3, 3, 4, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(childDeliverable
            .getWorksheetFinancials());
        validateResultForSubElement(childDeliverable
            .getWorksheetFinancials(), expectedValuesChild,false);

        // verifying GrandChild sub element
        double[] expectedValuesGrandChild =
        {
                0, 0, 0, 0, 0, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(grandChildWorkProduct
            .getWorksheetFinancials());
        validateResultForSubElement(grandChildWorkProduct
            .getWorksheetFinancials(), expectedValuesGrandChild,true);

        // --- Phase 5: create second child of parent

        SummaryTask childSummaryTask = new SummaryTask();
        childSummaryTask.setName(HelperTestUtilities.createName());

        // Add child2 and child to parent proposal
        parent.setChildren(new WorkElement[]
        {
                childDeliverable, childSummaryTask
        });

        // creating and binding worksheet Financial for child2
        WorksheetFinancial[] child2WF = null;
        child2WF = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(
                childSummaryTask, this);

        child2WF[HOUR].setEstimate(new Double(20));
        child2WF[AVG_COST].setEstimate(new Double(20));

        child2WF[HOUR].setActual(new Double(10));
        child2WF[AVG_COST].setActual(new Double(10));

        childSummaryTask.setWorksheetFinancials(child2WF);

        // saving parent with child1, child2, grandchild
        result = null;
        result = save(parent, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Proposal);
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        parent = (Proposal) result.getRpmObject();
        assertNotNull(parent.getChildren());
        assertTrue(parent.getChildren().length == 2);

        WorkElement[] children = parent.getChildren();

        // -- Phase 6 validating subElement infos for the whole family

        // verifying parent sub element
        double[] expectedValuesParent2 =
        {
                25, 13, 12, 12, 25, // Hours
                17, 8.38, 26.33, 26.33, 17, // AverageCost
                425, 109, 316, 316, 425, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        printResultForSubElement(parent.getWorksheetFinancials());
        validateResultForSubElement(parent.getWorksheetFinancials(),
            expectedValuesParent2,false);

        // verifying children
        for (int i = 0; i < children.length; i++)
        {
            //child 1
            if (children[i] instanceof Deliverable)
            {
                childDeliverable = (Deliverable) parent.getChildren()[i];

                // verifying child sub element
                double[] expectedValuesChild1 =
                {
                        2, 1, 1, 1, 2, // Hours
                        2, 1, 3, 3, 2, // AverageCost
                        4, 1, 3, 3, 4, // ExpenseLabor
                        0, 0, 0, 0, 0
                // fte
                };

                printResultForSubElement(childDeliverable
                    .getWorksheetFinancials());
                validateResultForSubElement(childDeliverable
                    .getWorksheetFinancials(), expectedValuesChild1,false);

                // verifying GrandChild sub element
                assertNotNull(childDeliverable.getChildren());
                assertTrue(childDeliverable.getChildren()[0] instanceof WorkProduct);
                grandChildWorkProduct = (WorkProduct) childDeliverable
                    .getChildren()[0];

                double[] expectedValuesGrandChild2 =
                {
                        0, 0, 0, 0, 0, // Hours
                        0, 0, 0, 0, 0, // AverageCost
                        0, 0, 0, 0, 0, // ExpenseLabor
                        0, 0, 0, 0, 0
                // fte
                };

                printResultForSubElement(grandChildWorkProduct
                    .getWorksheetFinancials());
                validateResultForSubElement(grandChildWorkProduct
                    .getWorksheetFinancials(),
                    expectedValuesGrandChild2,true);

            }
            else
            // child 2
            {
                childSummaryTask = (SummaryTask) parent.getChildren()[i];

                // verifying child sub element
                double[] expectedValuesChild2 =
                {
                        0, 0, 0, 0, 0, // Hours
                        0, 0, 0, 0, 0, // AverageCost
                        0, 0, 0, 0, 0, // ExpenseLabor
                        0, 0, 0, 0, 0
                // fte
                };

                printResultForSubElement(childSummaryTask
                    .getWorksheetFinancials());
                validateResultForSubElement(childSummaryTask
                    .getWorksheetFinancials(), expectedValuesChild2,true);

            }
        }

    }

    public void testCreateProposalWithDeliverableChildrenCustomChargeCode()
            throws Exception
    {
        //
        // |--Parent[Proposal]
        //       | 
        //       |--Child[Deliverable]
        //    

        // --- Phase 1

        WorksheetFinancial wf = null;

        //adding non-system worksheetFinancial
        wf = new WorksheetFinancial();
        wf.setParent(wMainProposal);

        // create and set a charge code for parent
        ChargeCode CustomCC = ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this);

        wf.setChargeCode(CustomCC);

        //creating financial infos for parent
        wf.setEstimate(new Double(20));
        wf.setActual(new Double(10));

        wMainProposal.setWorksheetFinancials(new WorksheetFinancial[]
        {
            wf
        });

        //saving parent with financial
        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        Proposal parent = (Proposal) result.getRpmObject();

        //validating parent subElement
        printResultForSubElement(parent.getWorksheetFinancials());

        WorksheetFinancial wfResult = parent.getWorksheetFinancials()[0];
        assertTrue(wfResult.getEstimate().equals(new Double(20)));
        assertTrue(wfResult.getActual().equals(new Double(10)));
        assertTrue(wfResult.getRemaining().equals(new Double(10)));
        assertTrue(wfResult.getETC().equals(new Double(10)));
        assertTrue(wfResult.getEAC().equals(new Double(20)));

        // --- Phase 2: creating child with own financial infos

        // adding child in scope
        WorkElementScope childScope = new WorkElementScope();
        childScope.setWorksheetFinancials(new FinancialScope());
        childScope.getWorksheetFinancials().setChargeCode(true);
        wScope.setChildren(childScope);

        Deliverable child = new Deliverable();
        child.setName(HelperTestUtilities.createName());

        // Add child to parent proposal
        parent.setChildren(new WorkElement[]
        {
            child
        });

        // creating and binding worksheet Financial for child
        WorksheetFinancial childWF = null;

        //adding non-system worksheetFinancial
        childWF = new WorksheetFinancial();
        childWF.setParent(child);

        // create and set a charge code for child
        ChargeCode CustomCCChild = ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this);

        childWF.setChargeCode(CustomCCChild);

        //creating financial infos for child
        childWF.setEstimate(new Double(10));
        childWF.setActual(new Double(5));

        child.setWorksheetFinancials(new WorksheetFinancial[]
        {
            childWF
        });

        // saving parent with child
        result = null;
        result = save(parent, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertTrue(result.getRpmObject() instanceof Proposal);
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        parent = (Proposal) result.getRpmObject();
        assertNotNull(parent.getChildren());
        assertTrue(parent.getChildren()[0] instanceof Deliverable);
        Deliverable childDeliverable = (Deliverable) parent.getChildren()[0];

        // --- Phase 3: Validating Subelement infos for parent,child

        //parent
        //since parent chargecode is not the same as the child
        //the parent will end-up with two chargeCodes infos
        //one that contains the worksheet and the other the subelement
        //        printResultForSubElement(parent.getWorksheetFinancialsSubElement());
        assertTrue(parent.getWorksheetFinancials().length == 2);

        logger.debug("|------PARENT WORKSHEET-----|");
        printResultForSubElement(parent.getWorksheetFinancials());

        WorksheetFinancial sbResult = null;

        for (int i = 0; i < 2; i++)
        {
            if (parent.getWorksheetFinancials()[i].getChargeCode().getName()
                .equals(CustomCC.getName()))
            {
                wfResult = parent.getWorksheetFinancials()[i];
                assertTrue(wfResult.getEstimate().equals(new Double(20)));
                assertTrue(wfResult.getActual().equals(new Double(10)));
                assertTrue(wfResult.getRemaining().equals(new Double(10)));
                assertTrue(wfResult.getETC().equals(new Double(10)));
                assertTrue(wfResult.getEAC().equals(new Double(20)));
            }
        }

        logger.debug("|------PARENT SUB_ELEMENT-----|");
        printResultForSubElement(parent.getWorksheetFinancials());

        for (int j = 0; j < 2; j++)
        {
            if (parent.getWorksheetFinancials()[j].getChargeCode()
                .getName().equals(CustomCCChild.getName()))
            {
                sbResult = parent.getWorksheetFinancials()[j];
                assertTrue(sbResult.getSubElementEstimate().equals(new Double(10)));
                assertTrue(sbResult.getSubElementActual().equals(new Double(5)));
                assertTrue(sbResult.getSubElementRemaining().equals(new Double(5)));
                assertTrue(sbResult.getSubElementETC().equals(new Double(5)));
                assertTrue(sbResult.getSubElementEAC().equals(new Double(10)));
            }
        }

        //child
        logger.debug("|------CHILD WORKSHEET-----|");
        printResultForSubElement(childDeliverable.getWorksheetFinancials());

        assertTrue(childDeliverable.getWorksheetFinancials().length == 1);

        wfResult = childDeliverable.getWorksheetFinancials()[0];
        assertTrue(wfResult.getEstimate().equals(new Double(10)));
        assertTrue(wfResult.getActual().equals(new Double(5)));
        assertTrue(wfResult.getRemaining().equals(new Double(5)));
        assertTrue(wfResult.getETC().equals(new Double(5)));
        assertTrue(wfResult.getEAC().equals(new Double(10)));

        logger.debug("|------CHILD SUB_ELEMENT-----|");
        printResultForSubElement(childDeliverable
            .getWorksheetFinancials());
        sbResult = childDeliverable.getWorksheetFinancials()[0];
        assertTrue(sbResult.getSubElementEstimate().equals(new Double(0)));
        assertTrue(sbResult.getSubElementActual().equals(new Double(0)));
        assertTrue(sbResult.getSubElementRemaining().equals(new Double(0)));
        assertTrue(sbResult.getSubElementETC().equals(new Double(0)));
        assertTrue(sbResult.getSubElementEAC().equals(new Double(0)));

    }

}
