/*
 * Created on 18-Aug-2006
 * @author mmatwiy
 * Test cases for defects 50337 and 52556.
 * 
 * 50337: Newly created or updated timephased financial estimates
 * and actuals are not being properly saved.
 * 
 * 52556: Worksheet totals not updated when timephased financial
 * estimates and actuals are saved.
 */
package com.ibm.rpm.client.junit.financial.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimePhasedFinancialFixture;
import com.ibm.rpm.client.junit.financial.util.FinancialUtil;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test50337And52556 extends ClientTestController
{
    private TimePhasedFinancial actualTimePhased05 = null;

    private TimePhasedFinancial actualTimePhased06 = null;

    private TimePhasedFinancial estimateTimePhased05 = null;

    private TimePhasedFinancial estimateTimePhased06 = null;

    private WorksheetFinancial worksheetFinancial = null;

    Project project = null;

    WorkElementScope workElementScope = null;

    public void setUp() throws Exception
    {
        super.setUp();

        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setChargeCode(true);
        finScope.setActualTimePhased(true);
        finScope.setEstimateTimePhased(true);

        workElementScope = new WorkElementScope();
        workElementScope.setWorksheetFinancials(finScope);

        // create a parent Project and WorksheetFinancial
        project = new Project();
        project.setName(HelperTestUtilities.createName());

        worksheetFinancial = new WorksheetFinancial();
        worksheetFinancial.setParent(project);
        save(project, workElementScope);

        // create actuals for 2005
        actualTimePhased05 = new TimePhasedFinancial();
        actualTimePhased05.setParent(worksheetFinancial);
        actualTimePhased05.setYear(new Integer(2005));
        actualTimePhased05.setBudgetType(BudgetType.Actual);

        // create actuals for 2006
        actualTimePhased06 = new TimePhasedFinancial();
        actualTimePhased06.setParent(worksheetFinancial);
        actualTimePhased06.setYear(new Integer(2006));
        actualTimePhased06.setBudgetType(BudgetType.Actual);

        // create estimates for 2005
        estimateTimePhased05 = new TimePhasedFinancial();
        estimateTimePhased05.setParent(worksheetFinancial);
        estimateTimePhased05.setYear(new Integer(2005));
        estimateTimePhased05.setBudgetType(BudgetType.Estimate);

        // create estimates for 2006
        estimateTimePhased06 = new TimePhasedFinancial();
        estimateTimePhased06.setParent(worksheetFinancial);
        estimateTimePhased06.setYear(new Integer(2006));
        estimateTimePhased06.setBudgetType(BudgetType.Estimate);

        worksheetFinancial.setActualTimePhased(new TimePhasedFinancial[]
        {
                actualTimePhased05, actualTimePhased06
        });

        worksheetFinancial.setEstimateTimePhased(new TimePhasedFinancial[]
        {
                estimateTimePhased05, estimateTimePhased06
        });

        project.setWorksheetFinancials(new WorksheetFinancial[]
        {
            worksheetFinancial
        });

        // create and set a charge code
        worksheetFinancial.setChargeCode(ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this));

    }

    public void testTimePhasedActualBiggerThanEstimate() throws Exception
    {
        // set monthly actual values for 2005; yearly total is 60
        actualTimePhased05.setMonth01Value(new Double(5));
        actualTimePhased05.setMonth02Value(new Double(5));
        actualTimePhased05.setMonth03Value(new Double(5));
        actualTimePhased05.setMonth04Value(new Double(5));
        actualTimePhased05.setMonth05Value(new Double(5));
        actualTimePhased05.setMonth06Value(new Double(5));
        actualTimePhased05.setMonth07Value(new Double(5));
        actualTimePhased05.setMonth08Value(new Double(5));
        actualTimePhased05.setMonth09Value(new Double(5));
        actualTimePhased05.setMonth10Value(new Double(5));
        actualTimePhased05.setMonth11Value(new Double(5));
        actualTimePhased05.setMonth12Value(new Double(5));

        // set monthly actual values for 2006; yearly total is 24       
        actualTimePhased06.setMonth01Value(new Double(3));
        actualTimePhased06.setMonth02Value(new Double(1));
        actualTimePhased06.setMonth03Value(new Double(3));
        actualTimePhased06.setMonth04Value(new Double(1));
        actualTimePhased06.setMonth05Value(new Double(3));
        actualTimePhased06.setMonth06Value(new Double(1));
        actualTimePhased06.setMonth07Value(new Double(3));
        actualTimePhased06.setMonth08Value(new Double(1));
        actualTimePhased06.setMonth09Value(new Double(3));
        actualTimePhased06.setMonth10Value(new Double(1));
        actualTimePhased06.setMonth11Value(new Double(3));
        actualTimePhased06.setMonth12Value(new Double(1));

        // set monthly estimate values for 2005; yearly total is 60
        estimateTimePhased05.setMonth01Value(new Double(4));
        estimateTimePhased05.setMonth02Value(new Double(4));
        estimateTimePhased05.setMonth03Value(new Double(4));
        estimateTimePhased05.setMonth04Value(new Double(4));
        estimateTimePhased05.setMonth05Value(new Double(5));
        estimateTimePhased05.setMonth06Value(new Double(5));
        estimateTimePhased05.setMonth07Value(new Double(5));
        estimateTimePhased05.setMonth08Value(new Double(5));
        estimateTimePhased05.setMonth09Value(new Double(6));
        estimateTimePhased05.setMonth10Value(new Double(6));
        estimateTimePhased05.setMonth11Value(new Double(6));
        estimateTimePhased05.setMonth12Value(new Double(6));

        // set monthly estimate values for 2006; yearly total is 18
        estimateTimePhased06.setMonth01Value(new Double(1));
        estimateTimePhased06.setMonth02Value(new Double(2));
        estimateTimePhased06.setMonth03Value(new Double(1));
        estimateTimePhased06.setMonth04Value(new Double(2));
        estimateTimePhased06.setMonth05Value(new Double(1));
        estimateTimePhased06.setMonth06Value(new Double(2));
        estimateTimePhased06.setMonth07Value(new Double(1));
        estimateTimePhased06.setMonth08Value(new Double(2));
        estimateTimePhased06.setMonth09Value(new Double(1));
        estimateTimePhased06.setMonth10Value(new Double(2));
        estimateTimePhased06.setMonth11Value(new Double(1));
        estimateTimePhased06.setMonth12Value(new Double(2));

        // save the project with the financials	
        save(project, workElementScope);

        // reload and check the values
        Project reloaded = (Project) loadFromID(project, workElementScope);
        assertNotNull(reloaded);

        WorksheetFinancial[] finsReloaded = reloaded.getWorksheetFinancials();
        assertNotNull(finsReloaded);

        for (int j = 0; j < finsReloaded.length; j++)
        {
            // get timephased financial actuals and compare them
            TimePhasedFinancial[] reloadedYearlyActuals = finsReloaded[j]
                .getActualTimePhased();
            assertNotNull(reloadedYearlyActuals);
            assertEquals(reloadedYearlyActuals.length, 2);

            for (int i = 0; i < reloadedYearlyActuals.length; i++)
            {
                if (reloadedYearlyActuals[i].getYear() != null)
                {
                    if (reloadedYearlyActuals[i].getYear().doubleValue() == 2005)
                    {
                        // compare the 2005 actuals
                        TimePhasedFinancialFixture.compare(
                            reloadedYearlyActuals[i], actualTimePhased05, null);

                        // calculated field
                        assertEquals(
                            reloadedYearlyActuals[i].getTotalForYear(),
                            FinancialUtil
                                .calculateTotalForYear(actualTimePhased05));
                    }
                    else
                    {
                        // compare the 2006 actuals
                        TimePhasedFinancialFixture.compare(
                            reloadedYearlyActuals[i], actualTimePhased06, null);

                        // calculated field
                        assertEquals(
                            reloadedYearlyActuals[i].getTotalForYear(),
                            FinancialUtil
                                .calculateTotalForYear(actualTimePhased06));
                    }
                }
            }

            // get timephased financial estimates and compare
            TimePhasedFinancial[] reloadedYearlyEstimates = finsReloaded[j]
                .getEstimateTimePhased();
            assertNotNull(reloadedYearlyEstimates);
            assertEquals(reloadedYearlyEstimates.length, 2);

            for (int i = 0; i < reloadedYearlyEstimates.length; i++)
            {
                if (reloadedYearlyEstimates[i].getYear() != null)
                {
                    // compare the 2005 estimates
                    if (reloadedYearlyEstimates[i].getYear().doubleValue() == 2005)
                    {
                        TimePhasedFinancialFixture.compare(
                            reloadedYearlyEstimates[i], estimateTimePhased05,
                            null);

                        // calculated field
                        assertEquals(reloadedYearlyEstimates[i]
                            .getTotalForYear(), FinancialUtil
                            .calculateTotalForYear(estimateTimePhased05));
                    }
                    else
                    {
                        // compare the 2006 estimates
                        TimePhasedFinancialFixture.compare(
                            reloadedYearlyEstimates[i], estimateTimePhased06,
                            null);

                        // calculated field
                        assertEquals(reloadedYearlyEstimates[i]
                            .getTotalForYear(), FinancialUtil
                            .calculateTotalForYear(estimateTimePhased06));
                    }
                }
            }

            // check the calculated worksheet totals
            assertEquals(finsReloaded[j].getActual(), new Double(84));
            assertEquals(finsReloaded[j].getEstimate(), new Double(78));
            assertEquals(finsReloaded[j].getETC(), new Double(0)); // estimate - actual
        }

    }

    public void testTimePhasedEstimateBiggerThanActual() throws Exception
    {
        // set monthly actual values for 2005; yearly total is 30
        actualTimePhased05.setMonth01Value(new Double(1));
        actualTimePhased05.setMonth02Value(new Double(2));
        actualTimePhased05.setMonth03Value(new Double(3));
        actualTimePhased05.setMonth04Value(new Double(4));
        actualTimePhased05.setMonth05Value(new Double(1));
        actualTimePhased05.setMonth06Value(new Double(2));
        actualTimePhased05.setMonth07Value(new Double(3));
        actualTimePhased05.setMonth08Value(new Double(4));
        actualTimePhased05.setMonth09Value(new Double(1));
        actualTimePhased05.setMonth10Value(new Double(2));
        actualTimePhased05.setMonth11Value(new Double(3));
        actualTimePhased05.setMonth12Value(new Double(4));

        // set monthly actual values for 2006; yearly total is 24
        actualTimePhased06.setMonth01Value(new Double(2));
        actualTimePhased06.setMonth02Value(new Double(2));
        actualTimePhased06.setMonth03Value(new Double(2));
        actualTimePhased06.setMonth04Value(new Double(2));
        actualTimePhased06.setMonth05Value(new Double(2));
        actualTimePhased06.setMonth06Value(new Double(2));
        actualTimePhased06.setMonth07Value(new Double(2));
        actualTimePhased06.setMonth08Value(new Double(2));
        actualTimePhased06.setMonth09Value(new Double(2));
        actualTimePhased06.setMonth10Value(new Double(2));
        actualTimePhased06.setMonth11Value(new Double(2));
        actualTimePhased06.setMonth12Value(new Double(2));

        // set monthly estimate values for 2005; yearly total is 24
        estimateTimePhased05.setMonth01Value(new Double(1));
        estimateTimePhased05.setMonth02Value(new Double(1));
        estimateTimePhased05.setMonth03Value(new Double(1));
        estimateTimePhased05.setMonth04Value(new Double(1));
        estimateTimePhased05.setMonth05Value(new Double(2));
        estimateTimePhased05.setMonth06Value(new Double(2));
        estimateTimePhased05.setMonth07Value(new Double(2));
        estimateTimePhased05.setMonth08Value(new Double(2));
        estimateTimePhased05.setMonth09Value(new Double(3));
        estimateTimePhased05.setMonth10Value(new Double(3));
        estimateTimePhased05.setMonth11Value(new Double(3));
        estimateTimePhased05.setMonth12Value(new Double(3));

        // set monthly estimate values for 2006; yearly total is 78
        estimateTimePhased06.setMonth01Value(new Double(5));
        estimateTimePhased06.setMonth02Value(new Double(5));
        estimateTimePhased06.setMonth03Value(new Double(5));
        estimateTimePhased06.setMonth04Value(new Double(6));
        estimateTimePhased06.setMonth05Value(new Double(6));
        estimateTimePhased06.setMonth06Value(new Double(6));
        estimateTimePhased06.setMonth07Value(new Double(7));
        estimateTimePhased06.setMonth08Value(new Double(7));
        estimateTimePhased06.setMonth09Value(new Double(7));
        estimateTimePhased06.setMonth10Value(new Double(8));
        estimateTimePhased06.setMonth11Value(new Double(8));
        estimateTimePhased06.setMonth12Value(new Double(8));

        // save the project with the financials	
        save(project, workElementScope);

        // reload and check the values
        Project reloaded = (Project) loadFromID(project, workElementScope);
        assertNotNull(reloaded);

        WorksheetFinancial[] finsReloaded = reloaded.getWorksheetFinancials();
        assertNotNull(finsReloaded);

        for (int j = 0; j < finsReloaded.length; j++)
        {
            // get timephased financial actuals
            TimePhasedFinancial[] reloadedYearlyActuals = finsReloaded[j]
                .getActualTimePhased();
            assertNotNull(reloadedYearlyActuals);
            assertEquals(reloadedYearlyActuals.length, 2);

            for (int i = 0; i < reloadedYearlyActuals.length; i++)
            {
                if (reloadedYearlyActuals[i].getYear() != null)
                {
                    if (reloadedYearlyActuals[i].getYear().doubleValue() == 2005)
                    {
                        // compare the 2005 actuals
                        TimePhasedFinancialFixture.compare(
                            reloadedYearlyActuals[i], actualTimePhased05, null);

                        // calculated field
                        assertEquals(
                            reloadedYearlyActuals[i].getTotalForYear(),
                            FinancialUtil
                                .calculateTotalForYear(actualTimePhased05));
                    }
                    else
                    {
                        // compare the 2006 actuals
                        TimePhasedFinancialFixture.compare(
                            reloadedYearlyActuals[i], actualTimePhased06, null);

                        // calculated field
                        assertEquals(
                            reloadedYearlyActuals[i].getTotalForYear(),
                            FinancialUtil
                                .calculateTotalForYear(actualTimePhased06));
                    }
                }
            }

            // get timephased financial estimates
            TimePhasedFinancial[] reloadedYearlyEstimates = finsReloaded[j]
                .getEstimateTimePhased();
            assertNotNull(reloadedYearlyEstimates);
            assertEquals(reloadedYearlyEstimates.length, 2);

            for (int i = 0; i < reloadedYearlyEstimates.length; i++)
            {
                if (reloadedYearlyEstimates[i].getYear() != null)
                {
                    // compare the 2005 estimates
                    if (reloadedYearlyEstimates[i].getYear().doubleValue() == 2005)
                    {
                        TimePhasedFinancialFixture.compare(
                            reloadedYearlyEstimates[i], estimateTimePhased05,
                            null);

                        // calculated field
                        assertEquals(reloadedYearlyEstimates[i]
                            .getTotalForYear(), FinancialUtil
                            .calculateTotalForYear(estimateTimePhased05));
                    }
                    else
                    {
                        // compare the 2006 estimates
                        TimePhasedFinancialFixture.compare(
                            reloadedYearlyEstimates[i], estimateTimePhased06,
                            null);

                        // calculated field
                        assertEquals(reloadedYearlyEstimates[i]
                            .getTotalForYear(), FinancialUtil
                            .calculateTotalForYear(estimateTimePhased06));
                    }
                }
            }

            // check the calculated worksheet totals
            assertEquals(finsReloaded[j].getActual(), new Double(54));
            assertEquals(finsReloaded[j].getEstimate(), new Double(102));
            assertEquals(finsReloaded[j].getETC(), new Double(48)); // estimate - actual
        }

    }
}