/*
 * Created on 15-Sep-2006
 * @author mmatwiy
 * 
 * Test cases to check the update of worksheet totals when
 * actuals or estimates are updated without the other in scope.
 */
package com.ibm.rpm.client.junit.financial.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test53981 extends ClientTestController
{
    private TimePhasedFinancial actualTimePhased05 = null;

    private TimePhasedFinancial actualTimePhased06 = null;

    private TimePhasedFinancial estimateTimePhased05 = null;

    private TimePhasedFinancial estimateTimePhased06 = null;

    private WorksheetFinancial worksheetFinancial = null;

    Project project = null;

    WorkElementScope workElementScope = null;

    public void setUp() throws Exception
    {
        super.setUp();

        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setActualTimePhased(true);
        finScope.setEstimateTimePhased(true);

        workElementScope = new WorkElementScope();
        workElementScope.setWorksheetFinancials(finScope);

        // create a parent Project and WorksheetFinancial
        project = new Project();
        project.setName(HelperTestUtilities.createName());

        worksheetFinancial = new WorksheetFinancial();
        worksheetFinancial.setParent(project);
        save(project, workElementScope);

        // create actuals for 2005
        actualTimePhased05 = new TimePhasedFinancial();
        actualTimePhased05.setParent(worksheetFinancial);
        actualTimePhased05.setYear(new Integer(2005));
        actualTimePhased05.setBudgetType(BudgetType.Actual);

        // create actuals for 2006
        actualTimePhased06 = new TimePhasedFinancial();
        actualTimePhased06.setParent(worksheetFinancial);
        actualTimePhased06.setYear(new Integer(2006));
        actualTimePhased06.setBudgetType(BudgetType.Actual);

        // create estimates for 2005
        estimateTimePhased05 = new TimePhasedFinancial();
        estimateTimePhased05.setParent(worksheetFinancial);
        estimateTimePhased05.setYear(new Integer(2005));
        estimateTimePhased05.setBudgetType(BudgetType.Estimate);

        // create estimates for 2006
        estimateTimePhased06 = new TimePhasedFinancial();
        estimateTimePhased06.setParent(worksheetFinancial);
        estimateTimePhased06.setYear(new Integer(2006));
        estimateTimePhased06.setBudgetType(BudgetType.Estimate);

        worksheetFinancial.setActualTimePhased(new TimePhasedFinancial[]
        {
                actualTimePhased05, actualTimePhased06
        });

        worksheetFinancial.setEstimateTimePhased(new TimePhasedFinancial[]
        {
                estimateTimePhased05, estimateTimePhased06
        });

        project.setWorksheetFinancials(new WorksheetFinancial[]
        {
            worksheetFinancial
        });

        // create and set a charge code
        worksheetFinancial.setChargeCode(ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this));

        // set monthly actual values for 2005; yearly total is 60
        actualTimePhased05.setMonth01Value(new Double(5));
        actualTimePhased05.setMonth02Value(new Double(5));
        actualTimePhased05.setMonth03Value(new Double(5));
        actualTimePhased05.setMonth04Value(new Double(5));
        actualTimePhased05.setMonth05Value(new Double(5));
        actualTimePhased05.setMonth06Value(new Double(5));
        actualTimePhased05.setMonth07Value(new Double(5));
        actualTimePhased05.setMonth08Value(new Double(5));
        actualTimePhased05.setMonth09Value(new Double(5));
        actualTimePhased05.setMonth10Value(new Double(5));
        actualTimePhased05.setMonth11Value(new Double(5));
        actualTimePhased05.setMonth12Value(new Double(5));

        // set monthly actual values for 2006; yearly total is 24       
        actualTimePhased06.setMonth01Value(new Double(3));
        actualTimePhased06.setMonth02Value(new Double(1));
        actualTimePhased06.setMonth03Value(new Double(3));
        actualTimePhased06.setMonth04Value(new Double(1));
        actualTimePhased06.setMonth05Value(new Double(3));
        actualTimePhased06.setMonth06Value(new Double(1));
        actualTimePhased06.setMonth07Value(new Double(3));
        actualTimePhased06.setMonth08Value(new Double(1));
        actualTimePhased06.setMonth09Value(new Double(3));
        actualTimePhased06.setMonth10Value(new Double(1));
        actualTimePhased06.setMonth11Value(new Double(3));
        actualTimePhased06.setMonth12Value(new Double(1));

        // set monthly estimate values for 2005; yearly total is 60
        estimateTimePhased05.setMonth01Value(new Double(4));
        estimateTimePhased05.setMonth02Value(new Double(4));
        estimateTimePhased05.setMonth03Value(new Double(4));
        estimateTimePhased05.setMonth04Value(new Double(4));
        estimateTimePhased05.setMonth05Value(new Double(5));
        estimateTimePhased05.setMonth06Value(new Double(5));
        estimateTimePhased05.setMonth07Value(new Double(5));
        estimateTimePhased05.setMonth08Value(new Double(5));
        estimateTimePhased05.setMonth09Value(new Double(6));
        estimateTimePhased05.setMonth10Value(new Double(6));
        estimateTimePhased05.setMonth11Value(new Double(6));
        estimateTimePhased05.setMonth12Value(new Double(6));

        // set monthly estimate values for 2006; yearly total is 18
        estimateTimePhased06.setMonth01Value(new Double(1));
        estimateTimePhased06.setMonth02Value(new Double(2));
        estimateTimePhased06.setMonth03Value(new Double(1));
        estimateTimePhased06.setMonth04Value(new Double(2));
        estimateTimePhased06.setMonth05Value(new Double(1));
        estimateTimePhased06.setMonth06Value(new Double(2));
        estimateTimePhased06.setMonth07Value(new Double(1));
        estimateTimePhased06.setMonth08Value(new Double(2));
        estimateTimePhased06.setMonth09Value(new Double(1));
        estimateTimePhased06.setMonth10Value(new Double(2));
        estimateTimePhased06.setMonth11Value(new Double(1));
        estimateTimePhased06.setMonth12Value(new Double(2));

        // save the project with the financials	
        save(project, workElementScope);
    }

    public void testUpdateTimePhasedActualsWithoutEstimates() throws Exception
    {
        // reload the project with its financials
        Project reloaded = (Project) loadFromID(project, workElementScope);
        assertNotNull(reloaded);

        WorksheetFinancial[] finsReloaded = reloaded.getWorksheetFinancials();
        assertNotNull(finsReloaded);

        for (int j = 0; j < finsReloaded.length; j++)
        {
            // get timephased financial actuals and update them
            TimePhasedFinancial[] reloadedYearlyActuals = finsReloaded[j]
                .getActualTimePhased();
            assertNotNull(reloadedYearlyActuals);
            assertEquals(reloadedYearlyActuals.length, 2);

            for (int i = 0; i < reloadedYearlyActuals.length; i++)
            {
                if (reloadedYearlyActuals[i].getYear() != null)
                {
                    // update the monthly values for 2005 and 2006
                    reloadedYearlyActuals[i].setMonth01Value(new Double(1.0));
                    reloadedYearlyActuals[i].setMonth02Value(new Double(2.0));
                    reloadedYearlyActuals[i].setMonth03Value(new Double(1.0));
                    reloadedYearlyActuals[i].setMonth04Value(new Double(2.0));
                    reloadedYearlyActuals[i].setMonth05Value(new Double(1.0));
                    reloadedYearlyActuals[i].setMonth06Value(new Double(2.0));
                    reloadedYearlyActuals[i].setMonth07Value(new Double(1.0));
                    reloadedYearlyActuals[i].setMonth08Value(new Double(2.0));
                    reloadedYearlyActuals[i].setMonth09Value(new Double(1.0));
                    reloadedYearlyActuals[i].setMonth10Value(new Double(2.0));
                    reloadedYearlyActuals[i].setMonth11Value(new Double(1.0));
                    reloadedYearlyActuals[i].setMonth12Value(new Double(2.0));
                }
            }
        }

        workElementScope.getWorksheetFinancials().setEstimateTimePhased(false);
        save(reloaded, workElementScope);

        // reload and check the values
        reloaded = (Project) loadFromID(reloaded, workElementScope);
        assertNotNull(reloaded);

        finsReloaded = reloaded.getWorksheetFinancials();
        assertNotNull(finsReloaded);

        for (int j = 0; j < finsReloaded.length; j++)
        {
            // get timephased financial actuals and compare them to the updated actuals
            TimePhasedFinancial[] reloadedYearlyActuals = finsReloaded[j]
                .getActualTimePhased();
            assertNotNull(reloadedYearlyActuals);
            assertEquals(reloadedYearlyActuals.length, 2);

            for (int i = 0; i < reloadedYearlyActuals.length; i++)
            {
                if (reloadedYearlyActuals[i].getYear() != null)
                {
                    for (int k = 0; k < reloadedYearlyActuals.length; k++)
                    {
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth01Value(),
                            new Double(1.0));
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth02Value(),
                            new Double(2.0));
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth03Value(),
                            new Double(1.0));
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth04Value(),
                            new Double(2.0));
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth05Value(),
                            new Double(1.0));
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth06Value(),
                            new Double(2.0));
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth07Value(),
                            new Double(1.0));
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth08Value(),
                            new Double(2.0));
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth09Value(),
                            new Double(1.0));
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth10Value(),
                            new Double(2.0));
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth11Value(),
                            new Double(1.0));
                        assertEquals(
                            reloadedYearlyActuals[k].getMonth12Value(),
                            new Double(2.0));
                    }
                }
            }

            // check the calculated worksheet totals
            assertEquals(finsReloaded[j].getActual(), new Double(36));
            assertEquals(finsReloaded[j].getEstimate(), new Double(78));
            assertEquals(finsReloaded[j].getETC(), new Double(42)); // estimate - actual
        }
    }

    public void testUpdateTimePhasedEstimatesWithoutActuals() throws Exception
    {
        // reload the project with its financials
        Project reloaded = (Project) loadFromID(project, workElementScope);
        assertNotNull(reloaded);

        WorksheetFinancial[] finsReloaded = reloaded.getWorksheetFinancials();
        assertNotNull(finsReloaded);

        for (int j = 0; j < finsReloaded.length; j++)
        {
            // get timephased financial estimates and update them
            TimePhasedFinancial[] reloadedYearlyEstimates = finsReloaded[j]
                .getEstimateTimePhased();
            assertNotNull(reloadedYearlyEstimates);
            assertEquals(reloadedYearlyEstimates.length, 2);

            for (int i = 0; i < reloadedYearlyEstimates.length; i++)
            {
                if (reloadedYearlyEstimates[i].getYear() != null)
                {
                    // update the monthly values for 2005 and 2006
                    reloadedYearlyEstimates[i].setMonth01Value(new Double(10));
                    reloadedYearlyEstimates[i].setMonth02Value(new Double(10));
                    reloadedYearlyEstimates[i].setMonth03Value(new Double(10));
                    reloadedYearlyEstimates[i].setMonth04Value(new Double(10));
                    reloadedYearlyEstimates[i].setMonth05Value(new Double(10));
                    reloadedYearlyEstimates[i].setMonth06Value(new Double(10));
                    reloadedYearlyEstimates[i].setMonth07Value(new Double(10));
                    reloadedYearlyEstimates[i].setMonth08Value(new Double(10));
                    reloadedYearlyEstimates[i].setMonth09Value(new Double(10));
                    reloadedYearlyEstimates[i].setMonth10Value(new Double(10));
                    reloadedYearlyEstimates[i].setMonth11Value(new Double(10));
                    reloadedYearlyEstimates[i].setMonth12Value(new Double(10));
                }
            }
        }

        workElementScope.getWorksheetFinancials().setActualTimePhased(false);
        save(reloaded, workElementScope);

        // reload and check the values
        reloaded = (Project) loadFromID(reloaded, workElementScope);
        assertNotNull(reloaded);

        finsReloaded = reloaded.getWorksheetFinancials();
        assertNotNull(finsReloaded);

        for (int j = 0; j < finsReloaded.length; j++)
        {
            // get timephased financial estimates and compare them to the updated estimates
            TimePhasedFinancial[] reloadedYearlyEstimates = finsReloaded[j]
                .getEstimateTimePhased();
            assertNotNull(reloadedYearlyEstimates);
            assertEquals(reloadedYearlyEstimates.length, 2);

            for (int i = 0; i < reloadedYearlyEstimates.length; i++)
            {
                if (reloadedYearlyEstimates[i].getYear() != null)
                {
                    for (int k = 0; k < reloadedYearlyEstimates.length; k++)
                    {
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth01Value(),
                            new Double(10));
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth02Value(),
                            new Double(10));
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth03Value(),
                            new Double(10));
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth04Value(),
                            new Double(10));
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth05Value(),
                            new Double(10));
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth06Value(),
                            new Double(10));
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth07Value(),
                            new Double(10));
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth08Value(),
                            new Double(10));
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth09Value(),
                            new Double(10));
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth10Value(),
                            new Double(10));
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth11Value(),
                            new Double(10));
                        assertEquals(
                            reloadedYearlyEstimates[k].getMonth12Value(),
                            new Double(10));
                    }
                }
            }

            // check the calculated worksheet totals
            assertEquals(finsReloaded[j].getActual(), new Double(84));
            assertEquals(finsReloaded[j].getEstimate(), new Double(240));
            assertEquals(finsReloaded[j].getETC(), new Double(156)); // estimate - actual
        }
    }
}