/*
 * Created on 27-Oct-06
 * @author mmatwiy
 * 
 * Test cases for defect 57002. 
 * If worksheet values already exist, adding timephased values wipes
 * out the previous worksheet values.
 */
package com.ibm.rpm.client.junit.financial.testcases.fixes;

import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test57002 extends ClientTestController
{
    private WorksheetFinancial worksheet = null;

    private FinancialScope finScope = null;

    private static final Double EXISTING_ESTIMATE = new Double(25.0);

    private static final Double EXISTING_ACTUAL = new Double(15.0);

    private static int currentYear = 0;

    private static int lastYear = 0;

    public void setUp() throws Exception
    {
        super.setUp();

        Calendar cal = new GregorianCalendar();
        currentYear = cal.get(Calendar.YEAR);
        lastYear = currentYear - 1;

        finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setParent(new WorkElementScope());

        // create a worksheet with existing actual and estimate values
        worksheet = new WorksheetFinancial();
        worksheet.setParent(WorkElementFixture.retrieveStaticFixture(
            new Project(), this));
        worksheet.setChargeCode(ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this));
        worksheet.setActual(EXISTING_ACTUAL);
        worksheet.setEstimate(EXISTING_ESTIMATE);

        worksheet = (WorksheetFinancial) save(worksheet, finScope)
            .getRpmObject();
    }

    public void testAddTimephasedEstimates() throws Exception
    {
        finScope.setEstimateTimePhased(true);

        // add timephased estimates to this worksheet
        // create estimates for last year
        TimePhasedFinancial estimateTimePhasedForLastYear = new TimePhasedFinancial();
        estimateTimePhasedForLastYear.setParent(worksheet);
        estimateTimePhasedForLastYear.setYear(new Integer(lastYear));
        estimateTimePhasedForLastYear.setBudgetType(BudgetType.Estimate);

        // set monthly estimate values for last year; yearly total is 60
        estimateTimePhasedForLastYear.setMonth01Value(new Double(4));
        estimateTimePhasedForLastYear.setMonth02Value(new Double(4));
        estimateTimePhasedForLastYear.setMonth03Value(new Double(4));
        estimateTimePhasedForLastYear.setMonth04Value(new Double(4));
        estimateTimePhasedForLastYear.setMonth05Value(new Double(5));
        estimateTimePhasedForLastYear.setMonth06Value(new Double(5));
        estimateTimePhasedForLastYear.setMonth07Value(new Double(5));
        estimateTimePhasedForLastYear.setMonth08Value(new Double(5));
        estimateTimePhasedForLastYear.setMonth09Value(new Double(6));
        estimateTimePhasedForLastYear.setMonth10Value(new Double(6));
        estimateTimePhasedForLastYear.setMonth11Value(new Double(6));
        estimateTimePhasedForLastYear.setMonth12Value(new Double(6));

        // create estimates for current year
        TimePhasedFinancial estimateTimePhasedForCurrentYear = new TimePhasedFinancial();
        estimateTimePhasedForCurrentYear.setParent(worksheet);
        estimateTimePhasedForCurrentYear.setYear(new Integer(currentYear));
        estimateTimePhasedForCurrentYear.setBudgetType(BudgetType.Estimate);

        // set monthly estimate values for current year; yearly total is 18
        estimateTimePhasedForCurrentYear.setMonth01Value(new Double(1));
        estimateTimePhasedForCurrentYear.setMonth02Value(new Double(2));
        estimateTimePhasedForCurrentYear.setMonth03Value(new Double(1));
        estimateTimePhasedForCurrentYear.setMonth04Value(new Double(2));
        estimateTimePhasedForCurrentYear.setMonth05Value(new Double(1));
        estimateTimePhasedForCurrentYear.setMonth06Value(new Double(2));
        estimateTimePhasedForCurrentYear.setMonth07Value(new Double(1));
        estimateTimePhasedForCurrentYear.setMonth08Value(new Double(2));
        estimateTimePhasedForCurrentYear.setMonth09Value(new Double(1));
        estimateTimePhasedForCurrentYear.setMonth10Value(new Double(2));
        estimateTimePhasedForCurrentYear.setMonth11Value(new Double(1));
        estimateTimePhasedForCurrentYear.setMonth12Value(new Double(2));

        worksheet.setEstimateTimePhased(new TimePhasedFinancial[]
        {
                estimateTimePhasedForLastYear, estimateTimePhasedForCurrentYear
        });

        // save the worksheet with 2 timephased values
        worksheet = (WorksheetFinancial) save(worksheet, finScope)
            .getRpmObject();

        // the worksheet estimate values should be the total timephased
        // monthly values for last and current year
        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheet, finScope);
        assertNotNull(reloaded);

        FinancialFixture.compare(worksheet, reloaded, finScope);
        assertEquals(reloaded.getEstimate(), new Double(60 + 18));
        assertEquals(reloaded.getActual(), EXISTING_ACTUAL);

    }

    public void testAddTimephasedActuals() throws Exception
    {
        finScope.setActualTimePhased(true);

        // add timephased actuals to this worksheet
        // create actuals for last year
        TimePhasedFinancial actualTimePhasedForLastYear = new TimePhasedFinancial();
        actualTimePhasedForLastYear.setParent(worksheet);
        actualTimePhasedForLastYear.setYear(new Integer(lastYear));
        actualTimePhasedForLastYear.setBudgetType(BudgetType.Actual);

        // set monthly actual values for last year; yearly total is 60
        setFixedTimephased(actualTimePhasedForLastYear, new Double(5));

        // create actuals for current year
        TimePhasedFinancial actualTimePhasedForCurrentYear = new TimePhasedFinancial();
        actualTimePhasedForCurrentYear.setParent(worksheet);
        actualTimePhasedForCurrentYear.setYear(new Integer(currentYear));
        actualTimePhasedForCurrentYear.setBudgetType(BudgetType.Actual);

        // set monthly actual values for current year; yearly total is 24
        actualTimePhasedForCurrentYear.setMonth01Value(new Double(3));
        actualTimePhasedForCurrentYear.setMonth02Value(new Double(1));
        actualTimePhasedForCurrentYear.setMonth03Value(new Double(3));
        actualTimePhasedForCurrentYear.setMonth04Value(new Double(1));
        actualTimePhasedForCurrentYear.setMonth05Value(new Double(3));
        actualTimePhasedForCurrentYear.setMonth06Value(new Double(1));
        actualTimePhasedForCurrentYear.setMonth07Value(new Double(3));
        actualTimePhasedForCurrentYear.setMonth08Value(new Double(1));
        actualTimePhasedForCurrentYear.setMonth09Value(new Double(3));
        actualTimePhasedForCurrentYear.setMonth10Value(new Double(1));
        actualTimePhasedForCurrentYear.setMonth11Value(new Double(3));
        actualTimePhasedForCurrentYear.setMonth12Value(new Double(1));

        worksheet.setActualTimePhased(new TimePhasedFinancial[]
        {
                actualTimePhasedForLastYear, actualTimePhasedForCurrentYear
        });

        // save the worksheet with 2 timephased values
        worksheet = (WorksheetFinancial) save(worksheet, finScope)
            .getRpmObject();

        // the worksheet actual values should be the total timephased
        // monthly values for last and current year
        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheet, finScope);
        assertNotNull(reloaded);

        FinancialFixture.compare(worksheet, reloaded, finScope);
        assertEquals(reloaded.getActual(), new Double(60 + 24));
        assertEquals(reloaded.getEstimate(), EXISTING_ESTIMATE);
    }

    /*
     * save some timephased values, check the worksheet value, then modify one
     * year and remove another and check the worksheet value again
     */
    public void testModifyTimephasedEstimateValues() throws Exception
    {
        finScope.setEstimateTimePhased(true);

        // add timephased estimates to this worksheet
        // create estimates for last year
        TimePhasedFinancial estimateTimePhasedForLastYear = new TimePhasedFinancial();
        estimateTimePhasedForLastYear.setParent(worksheet);
        estimateTimePhasedForLastYear.setYear(new Integer(lastYear));
        estimateTimePhasedForLastYear.setBudgetType(BudgetType.Estimate);

        // set monthly estimate values for last year; yearly total is 12
        setFixedTimephased(estimateTimePhasedForLastYear, new Double(1));

        // create estimates for current year
        TimePhasedFinancial estimateTimePhasedForCurrentYear = new TimePhasedFinancial();
        estimateTimePhasedForCurrentYear.setParent(worksheet);
        estimateTimePhasedForCurrentYear.setYear(new Integer(currentYear));
        estimateTimePhasedForCurrentYear.setBudgetType(BudgetType.Estimate);

        // set monthly estimate values for current year; yearly total is 48
        setFixedTimephased(estimateTimePhasedForCurrentYear, new Double(4));

        worksheet.setEstimateTimePhased(new TimePhasedFinancial[]
        {
                estimateTimePhasedForLastYear, estimateTimePhasedForCurrentYear
        });

        // save the worksheet with 2 timephased values
        worksheet = (WorksheetFinancial) save(worksheet, finScope)
            .getRpmObject();

        // the worksheet estimate values should be the total timephased
        // monthly values for last and current year
        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheet, finScope);
        assertNotNull(reloaded);

        FinancialFixture.compare(worksheet, reloaded, finScope);
        assertEquals(reloaded.getEstimate(), new Double(12 + 48));
        assertEquals(reloaded.getActual(), EXISTING_ACTUAL);

        // modify the last year values; new yearly value is 120
        setFixedTimephased(reloaded.getEstimateTimePhased()[0], new Double(10));

        // delete the current year values
        setFixedTimephased(reloaded.getEstimateTimePhased()[1], new Double(0));

        // save the worksheet with the 2 timephased values
        worksheet = (WorksheetFinancial) save(reloaded, finScope)
            .getRpmObject();

        // the worksheet estimate values should be the new total timephased
        // monthly values for last year + the existing estimate
        WorksheetFinancial reloaded2 = (WorksheetFinancial) loadFromID(
            worksheet, finScope);
        assertNotNull(reloaded2);

        FinancialFixture.compare(worksheet, reloaded2, finScope);
        assertEquals(reloaded2.getEstimate(), new Double(120));
        assertEquals(reloaded2.getActual(), EXISTING_ACTUAL);
    }

    /*
     * Save some timephased values, check the worksheet value, then delete the
     * timephased and check the worksheet value again
     */
    public void testClearTimephasedEstimateValues() throws Exception
    {
        finScope.setEstimateTimePhased(true);

        // add timephased estimates to this worksheet
        // create estimates for last year
        TimePhasedFinancial estimateTimePhasedForLastYear = new TimePhasedFinancial();
        estimateTimePhasedForLastYear.setParent(worksheet);
        estimateTimePhasedForLastYear.setYear(new Integer(lastYear));
        estimateTimePhasedForLastYear.setBudgetType(BudgetType.Estimate);

        // set monthly estimate values for last year; yearly total is 24
        setFixedTimephased(estimateTimePhasedForLastYear, new Double(2));

        // create estimates for current year
        TimePhasedFinancial estimateTimePhasedForCurrentYear = new TimePhasedFinancial();
        estimateTimePhasedForCurrentYear.setParent(worksheet);
        estimateTimePhasedForCurrentYear.setYear(new Integer(currentYear));
        estimateTimePhasedForCurrentYear.setBudgetType(BudgetType.Estimate);

        // set monthly estimate values for current year; yearly total is 12
        setFixedTimephased(estimateTimePhasedForCurrentYear, new Double(1));

        worksheet.setEstimateTimePhased(new TimePhasedFinancial[]
        {
                estimateTimePhasedForLastYear, estimateTimePhasedForCurrentYear
        });

        // save the worksheet with 2 timephased values
        worksheet = (WorksheetFinancial) save(worksheet, finScope)
            .getRpmObject();

        // the worksheet estimate values should be the total timephased
        // monthly values for last and current year
        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheet, finScope);
        assertNotNull(reloaded);

        FinancialFixture.compare(worksheet, reloaded, finScope);
        assertEquals(reloaded.getEstimate(), new Double(24 + 12));
        assertEquals(reloaded.getActual(), EXISTING_ACTUAL);

        // delete the last year values
        setFixedTimephased(reloaded.getEstimateTimePhased()[0], new Double(0));

        // delete the current year values
        setFixedTimephased(reloaded.getEstimateTimePhased()[1], new Double(0));

        // save the worksheet with the 2 timephased values
        worksheet = (WorksheetFinancial) save(reloaded, finScope)
            .getRpmObject();

        // the worksheet estimate value should be zero
        WorksheetFinancial reloaded2 = (WorksheetFinancial) loadFromID(
            worksheet, finScope);
        assertNotNull(reloaded);

        FinancialFixture.compare(worksheet, reloaded2, finScope);
        assertEquals(reloaded2.getEstimate(), new Double(0));
        assertEquals(reloaded2.getActual(), EXISTING_ACTUAL);
    }

    /*
     * Save some timephased values, check the worksheet value, then modify one
     * year and remove another and check the worksheet value again
     */
    public void testModifyTimephasedActualValues() throws Exception
    {
        finScope.setActualTimePhased(true);

        // add timephased actuals to this worksheet
        // create actuals for last year
        TimePhasedFinancial actualTimePhasedForLastYear = new TimePhasedFinancial();
        actualTimePhasedForLastYear.setParent(worksheet);
        actualTimePhasedForLastYear.setYear(new Integer(lastYear));
        actualTimePhasedForLastYear.setBudgetType(BudgetType.Actual);

        // set monthly actual values for last year; yearly total is 120
        setFixedTimephased(actualTimePhasedForLastYear, new Double(10));

        // create actuals for current year
        TimePhasedFinancial actualTimePhasedForCurrentYear = new TimePhasedFinancial();
        actualTimePhasedForCurrentYear.setParent(worksheet);
        actualTimePhasedForCurrentYear.setYear(new Integer(currentYear));
        actualTimePhasedForCurrentYear.setBudgetType(BudgetType.Actual);

        // set monthly actual values for current year; yearly total is 36
        setFixedTimephased(actualTimePhasedForCurrentYear, new Double(3));

        worksheet.setActualTimePhased(new TimePhasedFinancial[]
        {
                actualTimePhasedForLastYear, actualTimePhasedForCurrentYear
        });

        // save the worksheet with 2 timephased values
        worksheet = (WorksheetFinancial) save(worksheet, finScope)
            .getRpmObject();

        // the worksheet actual values should be the total timephased
        // monthly values for last and current year
        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheet, finScope);
        assertNotNull(reloaded);

        FinancialFixture.compare(worksheet, reloaded, finScope);
        assertEquals(reloaded.getActual(), new Double(120 + 36));
        assertEquals(reloaded.getEstimate(), EXISTING_ESTIMATE);

        // delete the last year values
        setFixedTimephased(worksheet.getActualTimePhased()[0], new Double(0));

        // modify the current year values; new yearly value is 24
        setFixedTimephased(worksheet.getActualTimePhased()[1], new Double(2));

        // save the worksheet with the 2 timephased values
        worksheet = (WorksheetFinancial) save(worksheet, finScope)
            .getRpmObject();

        // the worksheet actual values should be the new total timephased
        // monthly values for current year
        WorksheetFinancial reloaded2 = (WorksheetFinancial) loadFromID(
            worksheet, finScope);
        assertNotNull(reloaded);

        FinancialFixture.compare(worksheet, reloaded2, finScope);
        assertEquals(reloaded2.getActual(), new Double(24));
        assertEquals(reloaded2.getEstimate(), EXISTING_ESTIMATE);
    }

    /*
     * Save some timephased values, check the worksheet value, then delete the
     * timephased and check the worksheet value again
     */
    public void testClearTimephasedActualValues() throws Exception
    {
        finScope.setActualTimePhased(true);

        // add timephased actuals to this worksheet
        // create actuals for last year
        TimePhasedFinancial actualTimePhasedForLastYear = new TimePhasedFinancial();
        actualTimePhasedForLastYear.setParent(worksheet);
        actualTimePhasedForLastYear.setYear(new Integer(lastYear));
        actualTimePhasedForLastYear.setBudgetType(BudgetType.Actual);

        // set monthly actual values for last year; yearly total is 132
        setFixedTimephased(actualTimePhasedForLastYear, new Double(11));

        // create actuals for current year
        TimePhasedFinancial actualTimePhasedForCurrentYear = new TimePhasedFinancial();
        actualTimePhasedForCurrentYear.setParent(worksheet);
        actualTimePhasedForCurrentYear.setYear(new Integer(currentYear));
        actualTimePhasedForCurrentYear.setBudgetType(BudgetType.Actual);

        // set monthly actual values for current year; yearly total 156
        setFixedTimephased(actualTimePhasedForCurrentYear, new Double(13));

        worksheet.setActualTimePhased(new TimePhasedFinancial[]
        {
                actualTimePhasedForLastYear, actualTimePhasedForCurrentYear
        });

        // save the worksheet with 2 timephased values
        worksheet = (WorksheetFinancial) save(worksheet, finScope)
            .getRpmObject();

        // the worksheet actual values should be the total timephased
        // monthly values for last and current year
        WorksheetFinancial reloaded = (WorksheetFinancial) loadFromID(
            worksheet, finScope);
        assertNotNull(reloaded);

        FinancialFixture.compare(worksheet, reloaded, finScope);
        assertEquals(reloaded.getActual(), new Double(132 + 156));
        assertEquals(reloaded.getEstimate(), EXISTING_ESTIMATE);

        // delete the last year values
        setFixedTimephased(worksheet.getActualTimePhased()[0], new Double(0));

        // delete the current year values
        setFixedTimephased(worksheet.getActualTimePhased()[1], new Double(0));

        // save the worksheet with the 2 timephased values
        worksheet = (WorksheetFinancial) save(worksheet, finScope)
            .getRpmObject();

        // the worksheet actual value should be the zero
        WorksheetFinancial reloaded2 = (WorksheetFinancial) loadFromID(
            worksheet, finScope);
        assertNotNull(reloaded);

        FinancialFixture.compare(worksheet, reloaded2, finScope);
        assertEquals(reloaded2.getActual(), new Double(0));
        assertEquals(reloaded2.getEstimate(), EXISTING_ESTIMATE);
    }

    private void setFixedTimephased(TimePhasedFinancial timephased, Double value)
    {
        timephased.setMonth01Value(value);
        timephased.setMonth02Value(value);
        timephased.setMonth03Value(value);
        timephased.setMonth04Value(value);
        timephased.setMonth05Value(value);
        timephased.setMonth06Value(value);
        timephased.setMonth07Value(value);
        timephased.setMonth08Value(value);
        timephased.setMonth09Value(value);
        timephased.setMonth10Value(value);
        timephased.setMonth11Value(value);
        timephased.setMonth12Value(value);
    }

}
