package com.ibm.rpm.client.junit.financial.testcases.fixes;

import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.SecurityRoleFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.scope.SecurityGroupScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;
import com.ibm.rpm.wbs.types.ResourceType;

public class Test60207 extends ClientTestController
{    
    Proposal proposal;
    Project project;
    Deliverable deliverable;
    Task task;
    
    Calendar projectStartDate;
    Calendar projectFinishDate;
    
    Pool pool;
    Resource resource1;
    Resource resource2;
    Resource resource3;
    
    ResourceTaskAssignment rta1;
    ResourceTaskAssignment rta2;
    ResourceTaskAssignment rta3;
    
    WbsFinancial wbsHours;
    WbsFinancial wbsAvgCostHr;
    WbsFinancial wbsExpenseLabor;
    WbsFinancial wbsFTE;
    
    WorksheetFinancial wsHours;
    WorksheetFinancial wsAvgCostHr;
    WorksheetFinancial wsExpenseLabor;
    WorksheetFinancial wsFTE;

    
    Calendar resource1StartDate;
    Double resource1RegularCostPerHour = new Double(10.00);
    Double resource1SpecialCostPerHour = new Double(12.00);
    Double resource1SellingRatePerHour = new Double(20.00);
    Double resource1SpecialSellingCostPerHour = new Double(36.00);
    Double resource1EETC = new Double(2624.00);
    
    Calendar resource2StartDate;
    Double resource2RegularCostPerHour = new Double(20.00);
    Double resource2SpecialCostPerHour = new Double(26.00);
    Double resource2SellingRatePerHour = new Double(30.00);
    Double resource2SpecialSellingCostPerHour = new Double(32.00);
    Double resource2EETC = new Double(2424.00);
    
    Calendar resource3StartDate;
    Double resource3RegularCostPerHour = new Double(50.00);
    Double resource3SpecialCostPerHour = new Double(60.00);
    Double resource3SellingRatePerHour = new Double(30.00);
    Double resource3SpecialSellingCostPerHour = new Double(32.00);
    Double resource3EETC = new Double(840.00);
    
    RPMCalendar standardHoursRPMCalendar;
    
    String proposalName = "WS-FINANCIAL-SC6MP-PROPOSAL";
    String deliverableName = "WS-FINANCIAL-SC6MP-DELIV";
    String taskName = "WS-FINANCIAL-SC6MP-TASK";
    
    String poolName = "WS-FINANCIAL-SC6MP-POOL";
    String resource1Name = "WS-FINANCIAL-SC6MP-RES1";
    String resource2Name = "WS-FINANCIAL-SC6MP-RES2";
    String resource3Name = "WS-FINANCIAL-SC6MP-RES3";
    
    
    Double expectedExpenseHour = new Double(5888.00);
    Double expectedExpenseAvgCostHour = new Double(19.82);
    Double expectedExpenseLabor = new Double(116720.00);
    Double expectedExpenseFTE = new Double(736.00);
    Double expectedTotalLabor = new Double(116720.00);


    public void setUp() throws Exception
    {
        super.setUp();
       
        projectStartDate = new GregorianCalendar(2007,01,10);
        projectFinishDate = new GregorianCalendar(2008,06,11);
        
        proposal = createTopLevelProposal(this, proposalName, ProjectPublishedType.ToTeam,
                AssignmentType.Work, projectStartDate, projectFinishDate);
                
        deliverable = (Deliverable) WorkElementFixture.createSaveFixture(proposal, new Deliverable(), null, this);
        
        task = createTask(this, taskName, deliverable);
        
        standardHoursRPMCalendar= loadStandard8HoursRPMCalendar(this);
        
        pool = createPool(this, poolName);
        
        resource1 = createResource(this, pool, resource1Name, standardHoursRPMCalendar, new Boolean(true));
        resource2 = createResource(this, pool, resource2Name, standardHoursRPMCalendar, new Boolean(true));
        resource3 = createResource(this, pool, resource3Name, standardHoursRPMCalendar, new Boolean(true));        
    }

    
    public void tearDown() throws Exception
    {
        ClientTestController.logger.info("tearDown");
        
        if(proposal != null && proposal.getID() != null)
            this.delete(proposal);
        if(pool != null && pool.getID() != null)
            this.delete(pool);        

        super.tearDown();
    }
    
    public void testCreateResourcesRates() throws Exception
    {
        resource1 = createResourceRates(this, resource1, resource1RegularCostPerHour, resource1SpecialCostPerHour, resource1SellingRatePerHour, resource1SpecialSellingCostPerHour);
        assertEquals(resource1RegularCostPerHour, resource1.getRates().getRegularCostPerHour());
        assertEquals(resource1SpecialCostPerHour, resource1.getRates().getSpecialCostPerHour());
        assertEquals(resource1SellingRatePerHour, resource1.getRates().getSellingRatePerHour());
        assertEquals(resource1SpecialSellingCostPerHour, resource1.getRates().getSpecialSellingRatePerHour());
        
        resource2 = createResourceRates(this, resource2, resource2RegularCostPerHour, resource2SpecialCostPerHour, resource2SellingRatePerHour, resource2SpecialSellingCostPerHour);
        assertEquals(resource2RegularCostPerHour, resource2.getRates().getRegularCostPerHour());
        assertEquals(resource2SpecialCostPerHour, resource2.getRates().getSpecialCostPerHour());
        assertEquals(resource2SellingRatePerHour, resource2.getRates().getSellingRatePerHour());
        assertEquals(resource2SpecialSellingCostPerHour, resource2.getRates().getSpecialSellingRatePerHour());
        
        resource3 = createResourceRates(this, resource3, resource3RegularCostPerHour, resource3SpecialCostPerHour, resource3SellingRatePerHour, resource3SpecialSellingCostPerHour);
        assertEquals(resource3RegularCostPerHour, resource3.getRates().getRegularCostPerHour());
        assertEquals(resource3SpecialCostPerHour, resource3.getRates().getSpecialCostPerHour());
        assertEquals(resource3SellingRatePerHour, resource3.getRates().getSellingRatePerHour());
        assertEquals(resource3SpecialSellingCostPerHour, resource3.getRates().getSpecialSellingRatePerHour());
        
    }
    
    public void testAssignResourceToTask() throws Exception
    {
        testCreateResourcesRates();
        
        resource1StartDate = new GregorianCalendar(2007,0,10);
        resource2StartDate = new GregorianCalendar(2007,3,16);
        resource3StartDate = new GregorianCalendar(2007,7,10);
        
        SecurityRole securityRole = SecurityRoleFixture.createSecurityRole(task);
        
        rta1 = createResourceTaskAssignment(this, resource1, /*ctm.getSecurityRole("Task Responsible"),*/ resource1StartDate,securityRole, resource1EETC, ResourceType.Resource);
        rta2 = createResourceTaskAssignment(this, resource2, /*ctm.getSecurityRole("Task Team Member"),*/ resource2StartDate,securityRole, resource2EETC, ResourceType.Resource);
        rta3 = createResourceTaskAssignment(this, resource3, /*ctm.getSecurityRole("Task Team Member"),*/ resource3StartDate,securityRole, resource3EETC, ResourceType.Resource);
        
        task = updateAddResourceTaskAssignment(this, task, rta1);
        task = updateAddResourceTaskAssignment(this, task, rta2);
        task = updateAddResourceTaskAssignment(this, task, rta3);
    
    }
    
//    public void testValidationOnDeliverableWbsFinancialTimePhased() throws Exception
//    {       
//        testAssignResourceToTask();
//        
//        WorkElementScope scope = new WorkElementScope();
//        scope.setWorksheetFinancials(new FinancialScope());
//        scope.getWorksheetFinancials().setChargeCode(true);
//        scope.getWorksheetFinancials().setEstimateTimePhased(true);
//        //scope.setParent(new WorkElementScope());
//        
//        scope.setWbsFinancials(new FinancialScope());
//        scope.getWbsFinancials().setChargeCode(true);
//        scope.getWbsFinancials().setEstimateTimePhased(true);
//
//
//        
//        deliverable =  (Deliverable)loadFromID(deliverable, scope); 
//        
//        proposal =  (Proposal)loadFromID(proposal, scope); 
//        
//        
//        FinancialBusinessLogicUtil.printResult(proposal.getWorksheetFinancials(),"WORKSHEET");
//        FinancialBusinessLogicUtil.printResult(proposal.getWbsFinancials(),"WBS");
//
//        
//        
//        wsHours = (WorksheetFinancial)getMatchingWorksheetinancial("EXPENSE_HOURS", proposal.getWorksheetFinancials());
//        wsAvgCostHr = (WorksheetFinancial)getMatchingWorksheetinancial("EXPENSE_AVERAGE_COST",proposal.getWorksheetFinancials());
//        wsExpenseLabor = (WorksheetFinancial)getMatchingWorksheetinancial("EXPENSE_LABOR",proposal.getWorksheetFinancials());
//        wsFTE = (WorksheetFinancial)getMatchingWorksheetinancial("EXPENSE_FULL_TIME_EQUIVALENT",proposal.getWorksheetFinancials());
//        
//        wbsHours = (WbsFinancial)getMatchingWorksheetinancial("EXPENSE_HOURS", proposal.getWbsFinancials());
//        wbsAvgCostHr = (WbsFinancial)getMatchingWorksheetinancial("EXPENSE_AVERAGE_COST",proposal.getWbsFinancials());
//        wbsExpenseLabor = (WbsFinancial)getMatchingWorksheetinancial("EXPENSE_LABOR",proposal.getWbsFinancials());
//        wbsFTE = (WbsFinancial)getMatchingWorksheetinancial("EXPENSE_FULL_TIME_EQUIVALENT",proposal.getWbsFinancials());
//
//        
//        
//        //workSheet_Financial
//        assertNull(wsHours.getEstimateTimePhased()); 
//        assertNull(wsAvgCostHr.getEstimateTimePhased()); 
//        assertNull(wsExpenseLabor.getEstimateTimePhased());
//        assertNull(wsFTE.getEstimateTimePhased());
//        
//        
//        //wbs_Financial
//        assertNotNull(wbsHours.getEstimateTimePhased()); // Error!
//        assertNotNull(wbsAvgCostHr.getEstimateTimePhased()); // Error!
//        assertNotNull(wbsExpenseLabor.getEstimateTimePhased()); // Error!
//        assertNotNull(wbsFTE.getEstimateTimePhased()); // Error!
//
//        
//     }   
    
    private AbstractFinancial getMatchingWorksheetinancial(String chargecode,
            AbstractFinancial[] financial)
    {
        int i = -1;
        int index = -1;

        WorksheetFinancial wrongWorksheet = new WorksheetFinancial();
        wrongWorksheet.setID("");

        if (chargecode == null || financial == null)
        {
            return wrongWorksheet;
        }
        else
        {
            for (i = 0; i < financial.length; i++)
            {
                if (financial[i].getChargeCode().getID().startsWith(chargecode))
                {
                    index = i;
                    break;
                }
            }
            return financial[index];
        }
    }




    
    
    private  RPMCalendar loadStandard8HoursRPMCalendar(ClientTestController ctm)
            throws Exception

    {
        String calendarXpath = "/RPMCalendar[@name='Standard (Default)']";
        RPMObject[] objs = null;
        objs = ctm.loadFromXpath(calendarXpath, null);

        RPMCalendar cal = (RPMCalendar) objs[0];
        return cal;
    }
    
    
    private  Resource createResource(ClientTestController ctm, Pool parent,
    String name, RPMCalendar cal, Boolean active) throws Exception

    {
        Resource resource = new Resource();
        resource.setFullName(name);
        resource.setCalendar(cal);
        resource.setActive(active);
        resource.setParent(parent);

        ResourceScope resScope = new ResourceScope();
        resScope.setParent(new PoolScope());
        resScope.setCalendar(new CalendarScope());
        resScope.setSecurityGroup(new SecurityGroupScope());
//        resScope.setResourceSecurityRights(new Boolean(true));
        resScope.setDocumentFolder(new DocumentScope());

        SaveResult sr = ctm.save(resource, resScope);
        if (sr.getRpmObject() != null)
        {
            assertTrue(sr.getRpmObject() instanceof Resource);
            resource = (Resource) sr.getRpmObject();
            return resource;
        }
        else
        {
            return null;
        }
    } 

    
    private  ResourceTaskAssignment createResourceTaskAssignment(
            ClientTestController ctm, Resource resource,
            Calendar startDate, SecurityRole secRole,
            Double expectWorkDuration, ResourceType resType)
    {
        ResourceTaskAssignment rta = new ResourceTaskAssignment();
        rta.setResource(resource);
        rta.setResourceType(resType);
        rta.setSecurityRole(secRole);
        rta.setTaskAssignment(new TaskAssignment());
        rta.getTaskAssignment().setAssignmentStartDate(startDate);
        rta.getTaskAssignment().setEETCHours(expectWorkDuration);

        return rta;
    }

    
    private  Resource createResourceRates(ClientTestController ctm,
            Resource resource, Double regularCostPerHour,
            Double specialCostPerHour, Double sellingCostPerHour,
            Double specialSellingCostPerHour) throws Exception
    {
        ResourceRates rates = new ResourceRates();
        rates.setRegularCostPerHour(regularCostPerHour);
        rates.setSpecialCostPerHour(specialCostPerHour);
        rates.setSellingRatePerHour(sellingCostPerHour);
        rates.setSpecialSellingRatePerHour(specialSellingCostPerHour);

        resource.setRates(rates);
        ResourceScope resScope = new ResourceScope();
        resScope.setRates(true);

        SaveResult sr = ctm.save(resource, resScope);
        assertTrue(sr.getRpmObject() instanceof Resource);
        resource = (Resource) sr.getRpmObject();

        assertTrue(regularCostPerHour.doubleValue() == resource.getRates().getRegularCostPerHour().doubleValue());

        assertTrue(specialCostPerHour.doubleValue() == resource.getRates().getSpecialCostPerHour().doubleValue());

        assertTrue(sellingCostPerHour.doubleValue() == resource.getRates().getSellingRatePerHour().doubleValue());

        return resource;

    } 
    
    private  Task updateAddResourceTaskAssignment(ClientTestController ctm,
            Task task, ResourceTaskAssignment rta) throws Exception
    {
        task.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            rta
        });

        WorkElementScope wes = new WorkElementScope();
        wes.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        wes.getResourceTaskAssignments().setResource(new ResourceScope());
        wes.getResourceTaskAssignments().setTaskAssignment(new TaskAssignmentScope());
        wes.getResourceTaskAssignments().getResource().setCalendar(new CalendarScope());
        wes.setContainingProject(new WorkElementScope());

        SaveResult sr = ctm.save(task, wes);
        task = (Task) sr.getRpmObject();

        return task;
    } 
    
    
    private  Proposal createTopLevelProposal(ClientTestController ctm,
            String name, ProjectPublishedType publish,
            AssignmentType assignment, Calendar estimatedStartDate,
            Calendar estimatedFinishDate) throws Exception
    {
        Proposal prop = new Proposal();
        prop.setName(name);
        prop.setPublished(publish);
        prop.setAssignmentType(assignment);
        prop.setEstimatedFinishDate(estimatedFinishDate);
        prop.setEstimatedStartDate(estimatedStartDate);

        SaveResult sr = ctm.save(prop, null);
        assertTrue(sr.getRpmObject() instanceof Proposal);
        prop = (Proposal) sr.getRpmObject();

        return prop;
    } 

    private  Task createTask(ClientTestController ctm, String name,
            WorkElement parent) throws Exception

    {
        Task task = new Task();
        task.setName(name);
        task.setParent(parent);

        WorkElementScope wes = new WorkElementScope();
        wes.setParent(new WorkElementScope());
        wes.setContainingProject(new WorkElementScope());
        wes.setScheduleDates(true);
        wes.setDocumentFolder(new DocumentScope());

        SaveResult sr = ctm.save(task, wes);
        assertTrue(sr.getRpmObject() instanceof Task);
        task = (Task) sr.getRpmObject();

        return task;
    } 


    private  Pool createPool(ClientTestController ctm, String name)
            throws Exception
    {
        Pool pool = new Pool();
        pool.setName(name);

        SaveResult sr = ctm.save(pool, null);
        pool = (Pool) sr.getRpmObject();
        return pool;
    } 


}

