/**
 * Created on Feb 6, 2006
 * @author shahla
 */
package com.ibm.rpm.client.junit.financial.testcases.validation;

import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.constants.ValidationConstants;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.containers.TimeCodeGroup;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.containers.Proposal;


public class TestTimeCodeAssignmentValidation extends GenericValidation
{
    private TimeCodeAssignment timeCodeAssignment = null;

    public void setUp() throws Exception
    {
        super.setUp();

        timeCodeAssignment = new TimeCodeAssignment();
    }
    
    public void tearDown() throws Exception
    {
        super.tearDown();
    }
 
    public void testInsertAssignmentWithoutParent() throws AssertionError, Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, timeCodeAssignment
                .getClass().getName(), ValidationConstants.PARENT_FIELD,
                timeCodeAssignment.getID())
        };

        verifySaveMustFail(timeCodeAssignment, null, expectedExceptions);
    }
    
    public void testInsertAssignmentWithoutTimeCode() throws AssertionError, Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture
            .retrieveStaticFixture(new Proposal(), this);
        timeCodeAssignment.setParent(proposal);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, timeCodeAssignment
                .getClass().getName(), ValidationConstants.TIME_CODE_FIELD,
                timeCodeAssignment.getID())
        };

        verifySaveMustFail(timeCodeAssignment, null, expectedExceptions);
    }
    
    public void testFailureTimeCodeWithoutID() throws AssertionError, Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture
            .retrieveStaticFixture(new Proposal(), this);
        timeCodeAssignment.setParent(proposal);
        
        TimeCode timeCode = new TimeCode();
        timeCodeAssignment.setTimeCode(timeCode);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, timeCodeAssignment
                .getClass().getName(), ValidationConstants.TIME_CODE_FIELD,
                timeCodeAssignment.getID())
        };

        verifySaveMustFail(timeCodeAssignment, null, expectedExceptions);
    }
    
    public void testInsertAssignmentWithInvalidParent() throws AssertionError,
            Exception
    {
        
        RPMObject[] rpmObjects = loadFromXpath(
            "/TimeCodeGroup[id='TIMECODE_1']", null);
        TimeCodeGroup root = (TimeCodeGroup) rpmObjects[0];
        
        // set the parent
        timeCodeAssignment.setParent(root);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, timeCodeAssignment
                .getClass().getName(), ValidationConstants.PARENT_FIELD,
                timeCodeAssignment.getID())
        };

        verifySaveMustFail(timeCodeAssignment, null, expectedExceptions);
    }
    
    public void testInsertAssignmnetSuccessfully() throws AssertionError, Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture
            .retrieveStaticFixture(new Proposal(), this);
        
        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
        TimeCode timeCode = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, true, this);
        
        timeCodeAssignment.setParent(proposal);
        timeCodeAssignment.setTimeCode(timeCode);
        
        // Attempt to save the object
        timeCodeAssignment = (TimeCodeAssignment) saveMustSucceed(
            timeCodeAssignment, TimeCodeAssignment.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/TimeCodeAssignment[id='"
                + timeCodeAssignment.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof TimeCodeAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        TimeCodeAssignmentFixture.compare(timeCodeAssignment,
            (TimeCodeAssignment) rpmObjects[0]);
    }
}
