package com.ibm.rpm.client.junit.financial.testcases.validation;

import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.financial.util.FinancialUtil;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

public class TestWorksheetSubElementValidation extends GenericValidation
{
    
    private WorkElementScope wScope = null;

    private Proposal wMainProposal = null;

    private PortfolioScope wPFoScope = null;

    private Portfolio wMainPortfolio = null;


    public void setUp() throws Exception
    {
        super.setUp();
        
        init();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    private void init() throws Exception
    {
        wPFoScope = new PortfolioScope();
        wScope = new WorkElementScope();

        // Setting Up Scope
        wScope.setWorksheetFinancials(new FinancialScope());
        wScope.getWorksheetFinancials().setChargeCode(true);
        wPFoScope.setProjects(wScope);

        // Creating PortFolio and Proposal
        SaveResult portfolioResult = null;

        wMainPortfolio = new Portfolio();
        wMainPortfolio.setName(HelperTestUtilities.createName());

        wMainProposal = new Proposal();
        wMainProposal.setName(HelperTestUtilities.createName());
        wMainProposal.setPortfolio(wMainPortfolio);

        wMainPortfolio.setProjects(new GenericProject[]
        {
            wMainProposal
        });

        // Saving PortFolio/Proposal according to scope
        portfolioResult = save(wMainPortfolio, wPFoScope);

        assertNotNull(portfolioResult);

        assertTrue(portfolioResult.getRpmObject() instanceof Portfolio);
        wMainPortfolio = (Portfolio) portfolioResult.getRpmObject();

        assertNotNull(wMainPortfolio.getProjects());
        assertTrue(wMainPortfolio.getProjects()[0] instanceof Proposal);
        wMainProposal = (Proposal) wMainPortfolio.getProjects()[0];

        wMainProposal.setAssignmentType(AssignmentType.Work);
        wMainProposal.setPublished(ProjectPublishedType.None);

    }
    
    /**
     * This utility method will print the Financial SubElement matrix after each test so we
     * can compare the result and make changes accordingly.
     * 
     * @param result
     * @throws RPMException
     */
    private void printResultForSubElement(WorksheetFinancial[] wf)
            throws RPMException
    {
        if (wf != null)
        {
            WorksheetFinancial[] financial = wf;

            String title = null;
            logger.debug("|------WORKSHEET SUB ELEMENT FINANCIAL MATRIX-----|");
            logger.debug("  ESTIMATE    ACTUAL      REMAINING   ETC     EAC");

            for (int i = 0; i < financial.length; i++)
            {
                if (financial[i].getChargeCode() != null)
                {
                    Integer dbCode;

                    dbCode = FinancialUtil.getDBCodeForChargeCode(financial[i]
                        .getChargeCode().getFinancialCategory());

                    switch (dbCode.intValue())
                    {
                    case FinancialUtil.EXPENSE_HOUR:
                        title = "HOU:   ";
                        break;
                    case FinancialUtil.EXPENSE_AVERAGE_COST:
                        title = "AVG:   ";
                        break;
                    case FinancialUtil.EXPENSE_LABOR:
                        title = "ELB:   ";
                        break;
                    case FinancialUtil.EXPENSE_FTE:
                        title = "FTE:   ";
                        break;

                    default:
                        title = financial[i].getChargeCode().getName();
                    }

                    logger.debug(""
                            + title
                            + "  "
                            + FinancialUtil.nullToZero(
                                financial[i].getSubElementEstimate())
                                .doubleValue()
                            + "         "
                            + FinancialUtil.nullToZero(
                                financial[i].getSubElementActual())
                                .doubleValue()
                            + "         "
                            + FinancialUtil.nullToZero(
                                financial[i].getSubElementRemaining())
                                .doubleValue()
                            + "         "
                            + FinancialUtil.nullToZero(
                                financial[i].getSubElementETC()).doubleValue()
                            + "         "
                            + FinancialUtil.nullToZero(
                                financial[i].getSubElementEAC()).doubleValue()
                            + "\n");
                }
            }
        }
        else
        {
            logger.debug("|------WORKSHEET SUB ELEMENT FINANCIAL MATRIX-----|");
            logger.debug("              NO DATA AVAILABLE                    ");

        }

    }

    /**
     * This utility method will validate the result from an operation with the
     * expected expectedValues of the Financial SubElement matrix.
     * 
     * @param rs
     * @param expectedValues
     * @throws RPMException
     */
    private void validateResultForSubElement(WorksheetFinancial[] wf,
            double[] expectedValues, boolean noSubElementData)
            throws RPMException
    {
        assertNotNull(wf);
        assertNotNull(expectedValues);

        WorksheetFinancial[] financial = wf;

        if (financial != null)
        {

            WorksheetFinancial hourExpense = (WorksheetFinancial) FinancialUtil
                .getFinancialForChargeCode(FinancialUtil.EXPENSE_HOUR,
                    financial);

            WorksheetFinancial laborExpense = (WorksheetFinancial) FinancialUtil
                .getFinancialForChargeCode(FinancialUtil.EXPENSE_LABOR,
                    financial);

            WorksheetFinancial avgExpense = (WorksheetFinancial) FinancialUtil
                .getFinancialForChargeCode(FinancialUtil.EXPENSE_AVERAGE_COST,
                    financial);

            WorksheetFinancial fteExpense = (WorksheetFinancial) FinancialUtil
                .getFinancialForChargeCode(FinancialUtil.EXPENSE_FTE, financial);

            // expectedValues array validation
            assertTrue(expectedValues.length == 20);

            // financials validation
            if (hourExpense == null && laborExpense == null
                    && avgExpense == null && fteExpense == null)
            {
                assertTrue(noSubElementData);
            }
            else
            {
                // HOUR

                assertTrue((hourExpense.getSubElementEstimate()).doubleValue() == (expectedValues[0])); // Estimate
                assertTrue((hourExpense.getSubElementActual()).doubleValue() == (expectedValues[1])); // Actual
                assertTrue((hourExpense.getSubElementRemaining()).doubleValue() == (expectedValues[2])); // Remaining
                assertTrue((hourExpense.getSubElementETC()).doubleValue() == (expectedValues[3])); // ETC
                assertTrue((hourExpense.getSubElementEAC()).doubleValue() == (expectedValues[4]));// EAC

                // AVERAGE_COST
                assertTrue((avgExpense.getSubElementEstimate()).doubleValue() == (expectedValues[5])); // Estimate
                assertTrue((avgExpense.getSubElementActual()).doubleValue() == (expectedValues[6])); // Actual
                assertTrue((avgExpense.getSubElementRemaining()).doubleValue() == (expectedValues[7])); // Remaining
                assertTrue((avgExpense.getSubElementETC()).doubleValue() == (expectedValues[8])); // ETC
                assertTrue((avgExpense.getSubElementEAC()).doubleValue() == (expectedValues[9])); // EAC

                // EXPENSE_LABOR
                assertTrue((laborExpense.getSubElementEstimate()).doubleValue() == (expectedValues[10])); // Estimate
                assertTrue((laborExpense.getSubElementActual()).doubleValue() == (expectedValues[11])); // Actual
                assertTrue((laborExpense.getSubElementRemaining())
                    .doubleValue() == (expectedValues[12])); // Remaining
                assertTrue((laborExpense.getSubElementETC()).doubleValue() == (expectedValues[13])); // ETC
                assertTrue((laborExpense.getSubElementEAC()).doubleValue() == (expectedValues[14])); // EAC

            }

            // --It seems that FTE is not calculated for the SubElement
            // matrix
            // as seen in the client. We still calculate it anyway.
            // We will un-comments the lines below when the client does the
            // same.

            // FTE
            // assertTrue((fteExpense.getSubElementEstimate()).doubleValue() == (expectedValues[15]));// Estimate
            // assertTrue((fteExpense.getSubElementActual()).doubleValue() == (expectedValues[16])); // Actual
            // assertTrue((fteExpense.getSubElementRemaining()).doubleValue() == (expectedValues[17]));// Remaining
            // assertTrue((fteExpense.getSubElementETC()).doubleValue()== (expectedValues[18])); // ETC
            // assertTrue((fteExpense.getSubElementEAC()).doubleValue() == (expectedValues[19])); // EAC

        }
        else
        {
            logger.debug("Worksheet SubElement Data is empty...");
        }
    }

    
   
    public void testCreateProposalWithNoChildren_Update() throws Exception
    {
        //
        // |--Parent[Proposal]
        //

        //creating worksheet financial system charge codes array
        //add initial expectedValues for worksheet and bind them to the proposal.
        WorksheetFinancial[] wf = null;

        wf = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(wMainProposal,
                this);

        wMainProposal.setWorksheetFinancials(wf);

        //saving proposal with worksheet expectedValues
        SaveResult result = null;
        result = save(wMainProposal, wScope);

        assertNotNull(result);
        assertNull(result.getErrors());
        assertNotNull(((Proposal) result.getRpmObject())
            .getWorksheetFinancials());
        assertTrue(result.isSuccessful());

        Proposal parent = (Proposal) result.getRpmObject();

        //validating result for subElement
        double[] expectedValues =
        {
                0, 0, 0, 0, 0, // Hours
                0, 0, 0, 0, 0, // AverageCost
                0, 0, 0, 0, 0, // ExpenseLabor
                0, 0, 0, 0, 0
        // fte
        };

        validateResultForSubElement(parent.getWorksheetFinancials(),
            expectedValues,true);
        printResultForSubElement(parent.getWorksheetFinancials());
        
        parent.getWorksheetFinancials()[0].setSubElementEstimate(new Double(100));
        parent.getWorksheetFinancials()[0].setSubElementActual(new Double(100));
        parent.getWorksheetFinancials()[0].setSubElementRemaining(new Double(100));
        parent.getWorksheetFinancials()[0].setSubElementETC(new Double(100));
        parent.getWorksheetFinancials()[0].setSubElementEAC(new Double(100));      
        
        
        // List of expected Errors...
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, parent
                    .getWorksheetFinancials()[0].getClass().getName(),
                    "SubElement EAC", parent.getWorksheetFinancials()[0]
                        .getID()),

                createRPMException(ErrorCodes.READ_ONLY_FIELD, parent
                    .getWorksheetFinancials()[0].getClass().getName(),
                    "SubElement REMAINING", parent.getWorksheetFinancials()[0]
                        .getID()),

                createRPMException(ErrorCodes.READ_ONLY_FIELD, parent
                    .getWorksheetFinancials()[0].getClass().getName(),
                    "SubElement ETC", parent.getWorksheetFinancials()[0]
                        .getID()),

                createRPMException(ErrorCodes.READ_ONLY_FIELD, parent
                    .getWorksheetFinancials()[0].getClass().getName(),
                    "SubElement ESTIMATE", parent.getWorksheetFinancials()[0]
                        .getID()),

                createRPMException(ErrorCodes.READ_ONLY_FIELD, parent
                    .getWorksheetFinancials()[0].getClass().getName(),
                    "SubElement ACTUAL", parent.getWorksheetFinancials()[0]
                        .getID())
        };        

        // Testing that saving to RPM does not work and the required message is
        // thrown
        verifySaveMustFail(parent, wScope, null, expectedWarnings);

    }
    
    
    public void testCreateProposalWithNoChildren_Create() throws Exception
    {
        //
        // |--Parent[Proposal]
        //

        //creating worksheet financial system charge codes array
        //add initial expectedValues for worksheet and bind them to the proposal.
        WorksheetFinancial[] wf = null;

        wf = FinancialFixture
            .createEmptyWorksheetFinancialSystemChargeCodesArray(wMainProposal,
                this);

        wMainProposal.setWorksheetFinancials(wf);        
        
        wMainProposal.getWorksheetFinancials()[0].setSubElementEstimate(new Double(100));
        wMainProposal.getWorksheetFinancials()[0].setSubElementActual(new Double(100));
        wMainProposal.getWorksheetFinancials()[0].setSubElementRemaining(new Double(100));
        wMainProposal.getWorksheetFinancials()[0].setSubElementETC(new Double(100));
        wMainProposal.getWorksheetFinancials()[0].setSubElementEAC(new Double(100));      
        
        
        // List of expected Errors...
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, wMainProposal
                    .getWorksheetFinancials()[0].getClass().getName(),
                    "SubElement EAC", wMainProposal.getWorksheetFinancials()[0]
                        .getID()),

                createRPMException(ErrorCodes.READ_ONLY_FIELD, wMainProposal
                    .getWorksheetFinancials()[0].getClass().getName(),
                    "SubElement REMAINING", wMainProposal.getWorksheetFinancials()[0]
                        .getID()),

                createRPMException(ErrorCodes.READ_ONLY_FIELD, wMainProposal
                    .getWorksheetFinancials()[0].getClass().getName(),
                    "SubElement ETC", wMainProposal.getWorksheetFinancials()[0]
                        .getID()),

                createRPMException(ErrorCodes.READ_ONLY_FIELD, wMainProposal
                    .getWorksheetFinancials()[0].getClass().getName(),
                    "SubElement ESTIMATE", wMainProposal.getWorksheetFinancials()[0]
                        .getID()),

                createRPMException(ErrorCodes.READ_ONLY_FIELD, wMainProposal
                    .getWorksheetFinancials()[0].getClass().getName(),
                    "SubElement ACTUAL", wMainProposal.getWorksheetFinancials()[0]
                        .getID())
        };        

        // Testing that saving to RPM does not work and the required message is
        // thrown
        verifySaveMustFail(wMainProposal, wScope, null, expectedWarnings);

    }




    
    
    


}
