/*
 * Created on 8-Sep-2006
 * @author mmatwiy
 * Utility methods for container Financial.
 */
package com.ibm.rpm.client.junit.financial.util;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Comparator;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.CalendarWeekDay;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.containers.WorkInterval;
import com.ibm.rpm.applicationadministration.types.CalendarWeekDayType;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.wbs.containers.AggregateNode;

public class FinancialUtil
{
    public static final int EXPENSE_HOUR = 11;

    public static final int EXPENSE_AVERAGE_COST = 12;

    public static final int EXPENSE_LABOR = 14;

    public static final int EXPENSE_FTE = 13;
    
    public static final int INVALID_DB_CODE = -1;
    
    
    
    private static Map financialCategoryToDBMap;
    
    
    static {
		financialCategoryToDBMap = new HashMap();
		addToDBMap(FinancialCategory.Benefit, 2);
		addToDBMap(FinancialCategory.BenefitAverageCost, 32);
		addToDBMap(FinancialCategory.BenefitFTE, 33);
		addToDBMap(FinancialCategory.BenefitHour, 31);
		addToDBMap(FinancialCategory.BenefitLabor, 34);
		addToDBMap(FinancialCategory.Capital, 1);
		addToDBMap(FinancialCategory.CapitalAverageCost, 22);
		addToDBMap(FinancialCategory.CapitalFTE, 23);
		addToDBMap(FinancialCategory.CapitalHour, 21);
		addToDBMap(FinancialCategory.CapitalLabor, 24);
		addToDBMap(FinancialCategory.Contingency, 4);
		addToDBMap(FinancialCategory.Elimination, 3);
		addToDBMap(FinancialCategory.Expense, 0);
		addToDBMap(FinancialCategory.ExpenseAverageCost, 12);
		addToDBMap(FinancialCategory.ExpenseFTE, 13);
		addToDBMap(FinancialCategory.ExpenseHour, 11);
		addToDBMap(FinancialCategory.ExpenseLabor, 14);
		addToDBMap(FinancialCategory.TimePhaseCalendar, 10);

	}
    
    private static void addToDBMap(FinancialCategory fc, int dbCode)
    {
        Integer codeObject = new Integer(dbCode);
        financialCategoryToDBMap.put(fc, codeObject);
    }
    
    public static Integer toDBCode(FinancialCategory fc) throws RPMException
    {
        Integer value = new Integer(INVALID_DB_CODE);
        if (fc != null)
        {
            value = (Integer) financialCategoryToDBMap.get(fc);
        }
        return value;
    }


    public static Double calculateTotalForYear(TimePhasedFinancial timephased)
    {
        double total = nullToZero(timephased.getMonth01Value()).doubleValue()
                + nullToZero(timephased.getMonth02Value()).doubleValue()
                + nullToZero(timephased.getMonth03Value()).doubleValue()
                + nullToZero(timephased.getMonth04Value()).doubleValue()
                + nullToZero(timephased.getMonth05Value()).doubleValue()
                + nullToZero(timephased.getMonth06Value()).doubleValue()
                + nullToZero(timephased.getMonth07Value()).doubleValue()
                + nullToZero(timephased.getMonth08Value()).doubleValue()
                + nullToZero(timephased.getMonth09Value()).doubleValue()
                + nullToZero(timephased.getMonth10Value()).doubleValue()
                + nullToZero(timephased.getMonth11Value()).doubleValue()
                + nullToZero(timephased.getMonth12Value()).doubleValue();

        return new Double(total);
    }

    public static double calculateTimephasedEstimateTotal(
            AbstractFinancial financial)
    {
        double totalEstimate = 0.0;
        TimePhasedFinancial[] estimates = financial.getEstimateTimePhased();

        if (estimates != null)
        {
            for (int i = 0; i < estimates.length; i++)
            {
                totalEstimate += nullToZero(estimates[i].getMonth01Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth02Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth03Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth04Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth05Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth06Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth07Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth08Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth09Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth10Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth11Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth12Value())
                    .doubleValue();
            }
        }
        return totalEstimate;
    }

    public static double calculateTimephasedActualTotal(
            AbstractFinancial financial)
    {
        double totalActual = 0.0;
        TimePhasedFinancial[] actuals = financial.getActualTimePhased();

        if (actuals != null)
        {
            for (int i = 0; i < actuals.length; i++)
            {
                totalActual += nullToZero(actuals[i].getMonth01Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth02Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth03Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth04Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth05Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth06Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth07Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth08Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth09Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth10Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth11Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth12Value())
                    .doubleValue();
            }
        }
        return totalActual;
    }

    
    
    public static Double nullToZero(Double d)
    {
        return d == null ? new Double(0) : d;
    }
    
    public static void nullToZero( AbstractFinancial financial)
    {
        
        if (financial != null)
        {
            financial.setEstimate(roundDouble2Dec(financial
                .getEstimate()));

            financial.setActual(roundDouble2Dec(financial.getActual()));

            financial.setRemaining(roundDouble2Dec(financial
                .getRemaining()));

            financial.setETC(roundDouble2Dec(financial.getETC()));

            financial.setEAC(roundDouble2Dec(financial.getEAC()));

            financial.setEACVariance(roundDouble2Dec(financial
                .getEACVariance()));
        }
    }
 
    
    public static Double roundDouble2Dec(Double value)
    {
        if (value != null)
        {
            int decimalPlaces = 2;

            BigDecimal bd = new BigDecimal(value.doubleValue());

            bd = bd.setScale(decimalPlaces, BigDecimal.ROUND_HALF_UP);

            return new Double(bd.doubleValue());
        }
        else
        {
            return new Double(0);
        }
    }   
    
    
    
    public static AbstractFinancial getFinancialForChargeCode(int cCode,
            AbstractFinancial[] financials) throws RPMException
    {
        AbstractFinancial financial = null;

        if (financials != null)
        {
            for (int i = 0; i < financials.length; i++)
            {
                if (financials[i] != null
                        && financials[i].getChargeCode() != null
                        && financials[i].getChargeCode().getFinancialCategory() != null)
                {
                    Integer dbCode = toDBCode(financials[i].getChargeCode()
                        .getFinancialCategory());

                    if (dbCode.intValue() == cCode)
                    {
                        financial = financials[i];
                        break;
                    }
                }
            }
        }
        return financial;
    }
    
	
	public static Integer getDBCodeForChargeCode(FinancialCategory fc) throws RPMException
	{
		return toDBCode(fc);
	}
	
	
	public static WorksheetFinancial getWorksheetFinancialWithChargeCode(
            AggregateNode aggNode, ChargeCode cc)
    {
        if (aggNode != null && cc != null
                && aggNode.getWorksheetFinancials() != null
                && cc.getName() != null)
        {
            for (int i = 0; i < aggNode.getWorksheetFinancials().length; i++)
            {
                if (aggNode.getWorksheetFinancials()[i] != null
                        && aggNode.getWorksheetFinancials()[i].getChargeCode() != null
                        && cc.getName().equals(
                            aggNode.getWorksheetFinancials()[i].getChargeCode()
                                .getName()))
                {
                    return aggNode.getWorksheetFinancials()[i];
                }
            }
        }
        return null;
    }
    
       public static Double getCurrentMonthTimePhased(TimePhasedFinancial defaultTimePhased)
        {
            Calendar cal = new GregorianCalendar();

            int currentMonth = cal.get(Calendar.MONTH);
            
            Double currentMonthValue = null;

            if (defaultTimePhased != null)
            {
                switch (currentMonth)
                {
                case Calendar.JANUARY:
                    currentMonthValue = defaultTimePhased.getMonth01Value();
                    break;
                case Calendar.FEBRUARY:
                    currentMonthValue = defaultTimePhased.getMonth02Value();
                    break;
                case Calendar.MARCH:
                    currentMonthValue = defaultTimePhased.getMonth03Value();
                    break;
                case Calendar.APRIL:
                    currentMonthValue = defaultTimePhased.getMonth04Value();
                    break;
                case Calendar.MAY:
                    currentMonthValue = defaultTimePhased.getMonth05Value();
                    break;
                case Calendar.JUNE:
                    currentMonthValue = defaultTimePhased.getMonth06Value();
                    break;
                case Calendar.JULY:
                    currentMonthValue = defaultTimePhased.getMonth07Value();
                    break;
                case Calendar.AUGUST:
                    currentMonthValue = defaultTimePhased.getMonth08Value();
                    break;
                case Calendar.SEPTEMBER:
                    currentMonthValue = defaultTimePhased.getMonth09Value();
                    break;
                case Calendar.OCTOBER:
                    currentMonthValue = defaultTimePhased.getMonth10Value();
                    break;
                case Calendar.NOVEMBER:
                    currentMonthValue = defaultTimePhased.getMonth11Value();
                    break;
                case Calendar.DECEMBER:
                    currentMonthValue = defaultTimePhased.getMonth12Value();
                    break;
                }
            }
            
            return FinancialUtil.nullToZero(currentMonthValue);

        }
       

       

       public static boolean isWorkingDay(RPMCalendar cal, Calendar givenDay)
            throws Exception
    {
        if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY)
        {
            if (cal.getStandardSunday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY)
        {
            if (cal.getStandardMonday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.TUESDAY)
        {
            if (cal.getStandardTuesday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.WEDNESDAY)
        {
            if (cal.getStandardWednesday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.THURSDAY)
        {
            if (cal.getStandardThursday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.FRIDAY)
        {
            if (cal.getStandardFriday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY)
        {
            if (cal.getStandardSaturday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else
            return false;
    }
       
       public static boolean isEndOfDayCompareToCalendar(RPMCalendar refCalendar,
               Calendar today)
       {
           boolean isEndOfDay = false;

           if (refCalendar != null && today != null)
           {
               CalendarWeekDay weekDay = null;
               WorkInterval workInterval = null;
               WorkInterval winnerInterval = null;

               // find out which day is it...
               switch (today.get(Calendar.DAY_OF_WEEK))
               {
               case Calendar.SUNDAY:
                   weekDay = refCalendar.getStandardSunday();
                   break;
               case Calendar.MONDAY:
                   weekDay = refCalendar.getStandardMonday();
                   break;
               case Calendar.THURSDAY:
                   weekDay = refCalendar.getStandardThursday();
                   break;
               case Calendar.WEDNESDAY:
                   weekDay = refCalendar.getStandardWednesday();
                   break;
               case Calendar.TUESDAY:
                   weekDay = refCalendar.getStandardTuesday();
                   break;
               case Calendar.FRIDAY:
                   weekDay = refCalendar.getStandardFriday();
                   break;
               case Calendar.SATURDAY:
                   weekDay = refCalendar.getStandardSaturday();
                   break;

               }

               // Find highest time range in refCalendar
               if (weekDay != null && weekDay.getIntervals() != null)
               {
                   for (int i = 0; i < weekDay.getIntervals().length; i++)
                   {
                       workInterval = weekDay.getIntervals()[i];
                       winnerInterval = chooseHighestInterval(winnerInterval,
                           workInterval);
                   }

                   // verifying that we are at the end of workings hours for the day
                   String[] endtime = winnerInterval.getTo().split(":");

                   if (((today.get(Calendar.HOUR_OF_DAY) == Integer
                       .parseInt(endtime[0])) && (today.get(Calendar.MINUTE) > Integer
                       .parseInt(endtime[1])))
                           || (today.get(Calendar.HOUR_OF_DAY) > Integer
                               .parseInt(endtime[0])))
                   {
                       isEndOfDay = true;
                   }
               }

           }

           return isEndOfDay;
       }

       private static WorkInterval chooseHighestInterval(
               WorkInterval winnerInterval, WorkInterval workInterval)
       {
           WorkInterval winner = workInterval;
           String[] workTimeSlot = null;
           String[] winnerTimeSlot = null;

           if (winnerInterval != null)
           {
               workTimeSlot = workInterval.getTo().split(":");
               winnerTimeSlot = winnerInterval.getTo().split(":");

               if (workTimeSlot.length == 2 && winnerTimeSlot.length == 2)
               {
                   if (((Integer.parseInt(winnerTimeSlot[0]) == Integer
                       .parseInt(workTimeSlot[0])) && (Integer
                       .parseInt(winnerTimeSlot[1]) > Integer
                       .parseInt(workTimeSlot[1])))
                           || ((Integer.parseInt(winnerTimeSlot[0]) > Integer
                               .parseInt(workTimeSlot[0]))))
                   {
                       winner = winnerInterval;
                   }

               }
           }

           return winner;
       }
       

    public static int getCalWorkingDaysDuration(RPMCalendar cal,
            Calendar startDate, Calendar finishDate) throws Exception
    {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        Calendar givenDay = startDate;
        int duration = 1;

        if (cal != null && startDate != null && finishDate != null)
        {
            if ((formatter.format(startDate.getTime())).equals(formatter
                .format(finishDate.getTime())))
            {
                return duration;
            }

            else
            {
                while (!(formatter.format(givenDay.getTime())).equals(formatter
                    .format(finishDate.getTime())))

                {
                    givenDay = getNextDayDate(givenDay);
                    if (isWorkingDay(cal, givenDay))
                    {
                        duration++;
                    }
                }
            }
        }

        return duration;
    }

    public static Calendar getNextDayDate(Calendar aDate) throws Exception
    {
        Calendar result = new GregorianCalendar(aDate.get(Calendar.YEAR), aDate
            .get(Calendar.MONTH), aDate.get(Calendar.DATE));

        result.add(Calendar.DATE, 1);

        return result;

    }

    public static Calendar getTodayDate() throws Exception
    {
        Calendar today = Calendar.getInstance();
        return today;
    }

    public static Calendar getYesterdayDate() throws Exception
    {
        Calendar tomorrow = new GregorianCalendar();
        tomorrow.add(Calendar.DATE, -1);
        Calendar result = new GregorianCalendar(tomorrow.get(Calendar.YEAR),
            tomorrow.get(Calendar.MONTH), tomorrow.get(Calendar.DATE));
        return result;

    }
    
    public static void sortTimePhasedByYear(TimePhasedFinancial[] original)
    {
        if (original != null && original.length > 1)
        {
            Arrays.sort(original, new Comparator()
            {
                public int compare(Object o1, Object o2)
                {
                    TimePhasedFinancial s1 = (o1 == null) ? null
                            : (TimePhasedFinancial) o1;
                    TimePhasedFinancial s2 = (o2 == null) ? null
                            : (TimePhasedFinancial) o2;

                    if (o1 == null && o2 == null)
                    {
                        return 0;
                    }
                    else if (o1 == null || s1.getYear() == null)
                    {
                        return 1;
                    }
                    else if (o2 == null || s2.getYear() == null)
                    {
                        return 0;
                    }

                    return s1.getYear().compareTo(s2.getYear());
                }
            });

        }

    }


}