package com.ibm.rpm.client.junit.fixtures;

import java.lang.reflect.Array;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.applicationadministration.scope.GeographicalScope;
import com.ibm.rpm.applicationadministration.scope.OrganizationalScope;
import com.ibm.rpm.applicationadministration.scope.PoolDomainScope;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.ContactGroupFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.GeographicalFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.PoolDomainFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.FolderFixture;
import com.ibm.rpm.client.junit.contract.fixtures.ContractFolderFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.resource.fixtures.CompetencyFixture;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.SkillFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.contract.containers.GenericContract;
import com.ibm.rpm.contract.scope.ContractScope;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.scope.ScorecardFolderScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author karim
 *  
 */
public class AbstractModuleFixture extends RPMObjectFixture
{

    protected static RPMObject[] createElements(RPMObjectScope scope,
            Class baseClass, ClientTestController ctc) throws Exception
    {
        RPMObject[] objs = null;

        objs = (RPMObject[]) Array.newInstance(baseClass, 1);

        if (baseClass.equals(AttributeCategory.class))
        {
            objs[0] = AttributeFixture.createFixture(1, 1, 1,
                    (AttributeScope) scope);
        }
        else if (baseClass.equals(CustomFieldCategory.class))
        {
            objs[0] = CustomFieldFixture.createCustomFieldCategory(1);
        }
        else if (baseClass.equals(ClientFolder.class))
        {
            objs[0] = FolderFixture.createFixture(1, 0, 0,
                    (ClientCostCenterScope) scope, ctc);
        }
        else if (baseClass.equals(ResourceAttributeCompetencyCategory.class))
        {

            objs[0] = CompetencyFixture.createFixture(0, 0, 0, null, ctc);
        }
        else if (baseClass.equals(ResourceAttributeSkillCategory.class))
        {

            objs[0] = SkillFixture.createFixture(0, 0, 0, null, ctc);
        }
        else if (baseClass.equals(ContactGroup.class))
        {

            objs[0] = ContactGroupFixture.createFixture(0, null);
        }
        else if (baseClass.equals(Pool.class))
        {
            objs[0] = PoolFixture
                    .createFixture(true, 0, (PoolScope) scope, ctc);
        }
        else if (baseClass.equals(PoolDomain.class))
        {
            objs[0] = PoolDomainFixture.createFixture(0,
                    (PoolDomainScope) scope);
        }
        else if (baseClass.equals(RPMCalendar.class))
        {
            objs[0] = RPMCalendarFixture.createFixture((CalendarScope) scope,
                    0, 0);
        }
        else if (baseClass.equals(Organizational.class))
        {
            objs[0] = OrganizationalFixture.createFixture(0,
                    (OrganizationalScope) scope);
        }
        else if (baseClass.equals(Geographical.class))
        {
            objs[0] = GeographicalFixture.createFixture(0,
                    (GeographicalScope) scope);
        }
        else if (baseClass.equals(Portfolio.class))
        {
            objs[0] = PortfolioFixture.createFixture(0, 0,
                    (PortfolioScope) scope, ctc);
        }
        else if (baseClass.equals(GenericProject.class))
        {
            //create a workelement
            Proposal workelement = (Proposal) WorkElementFixture.createFixture(
                    new Proposal(), (WorkElementScope) scope, ctc);

            //set rootPortfolio as an attribute
            Portfolio rootTestPortfolio = PortfolioFixture
                    .loadOrCreateRootPortfolio(ctc);
            workelement.setPortfolio(rootTestPortfolio);

            objs[0] = WorkElementFixture.createFixture(workelement,
                    (WorkElementScope) scope, ctc);
        }
        else if (baseClass.equals(ScorecardFolder.class))
        {
            objs[0] = ScorecardFixture
                    .createFixture((ScorecardFolderScope) scope);
        }
        else if (baseClass.equals(ScopeFolder.class))
        {
            Proposal proposal = (Proposal) WorkElementFixture.createFixture(
                    new Proposal(), null, ctc);
            ctc.save(proposal, new WorkElementScope());

            RPMObject[] rpmObjects = ctc.loadFromXpath("/ScopeFolder[name='"
                    + proposal.getName() + "']", new ScopeElementScope());
            ((ScopeFolder) rpmObjects[0]).setParent(proposal);
            objs[0] = (ScopeFolder) rpmObjects[0];
        }
        else if (baseClass.equals(AssetFolder.class))
        {
            objs[0] = AssetFolderFixture.createFixture(false, 0, (AssetScope)scope, ctc);
        }
        else if (baseClass.equals(ChargeCodeCategory.class))
        {
            objs[0] = ChargeCodeFixture.createChargeCodeCategory(null, false);
        }
        else if (baseClass.equals(GenericContract.class))
        {
            objs[0] = ContractFolderFixture.createFixture(1,1,(ContractScope)scope, ctc);
        }

        return objs;

    }

    protected static RPMObject[] updateElements(RPMObject[] objs,
            RPMObjectScope scope, ClientTestController ctc) throws Exception
    {

        if (scope != null && objs != null)

        {
            RPMObject obj = null;
            for (int i = 0; i < objs.length; i++)
            {
                obj = objs[i];
                if (obj instanceof AttributeCategory)
                {
                    AttributeFixture.updateCategoryFixture(
                        (AttributeCategory) obj, (AttributeScope) scope);
                }
                else if (obj instanceof ClientFolder)
                {
                    FolderFixture.updateFixtureFields((ClientFolder) obj,
                        (ClientCostCenterScope) scope);
                }
                else if (obj instanceof Pool)
                {
                    PoolFixture.updateFixture((Pool) obj, ctc);
                }
                else if (obj instanceof PoolDomain)
                {
                    PoolDomainFixture.updateFixtureFields((PoolDomain) obj,
                        (PoolDomainScope) scope);
                }
                else if (obj instanceof RPMCalendar)
                {
                    RPMCalendarFixture.updateFixture((RPMCalendar) obj,
                        (CalendarScope) scope);
                }
                else if (obj instanceof Organizational)
                {
                    OrganizationalFixture.updateFixtureFields(
                        (Organizational) obj, (OrganizationalScope) scope);
                }
                else if (obj instanceof Geographical)
                {
                    GeographicalFixture.updateFixtureFields((Geographical) obj,
                        (GeographicalScope) scope);
                }
                else if (obj instanceof Portfolio)
                {
                    PortfolioFixture.updateFixtureFields((Portfolio) obj,
                        (PortfolioScope) scope, null);
                }
                else if (obj instanceof GenericProject)
                {
                    WorkElementFixture.updateFixtureFields(
                        (GenericProject) obj, (WorkElementScope) scope, ctc);
                }
                else if (obj instanceof ScorecardFolder)
                {
                    ScorecardFixture.updateFolder((ScorecardFolder) obj,
                        (ScorecardFolderScope) scope);
                }
                else if (obj instanceof ScopeFolder)
                {
                    ScopeFolder scopeFolder = (ScopeFolder)obj;
                    Boolean newPrivateFolderFlag = new Boolean(true);
                    if(scopeFolder.getPrivateFolder()!=null)
                    {
                        newPrivateFolderFlag = new Boolean(!scopeFolder.getPrivateFolder().booleanValue());
                    }
                    scopeFolder.setPrivateFolder(newPrivateFolderFlag);
                }
                else if (obj instanceof AssetFolder)
                {
                    //here the call to AssetFolderFixture.updateFixture 
                    //doesn't affect the current folder so we update manually
                    String oldName = ((AssetFolder)obj).getName();
                    ((AssetFolder)obj).setName(oldName+"_UPD");
                }
                else if (obj instanceof ChargeCodeCategory)
                {
                    String oldName = ((ChargeCodeCategory)obj).getName();
                    ((ChargeCodeCategory)obj).setName(oldName+"_UPD");
                }
                else if (obj instanceof GenericContract)
                {
                    String oldName = ((GenericContract)obj).getName();
                    ((GenericContract)obj).setName(oldName+"_UPD");
                }
            }
        }

        return objs;

    }
}
