/*
 * Created on Jun 28, 2005
 */
package com.ibm.rpm.client.junit.fixtures;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author gosselin
 */
public abstract class RPMObjectFixture
{

    public static final String ASSET_CONTEXT = "Asset";
    public static final String CLIENT_CONTEXT = "Client";
    public static final String DOCUMENT_CONTEXT = "Document";
    public static final String POOL_CONTEXT = "Pool";
    public static final String SCOPE_CONTEXT = "Scope";
    public static final String WBS_CONTEXT = "WBS";
    public static final String RESOURCE_CONTEXT = "Resource";

    public static String ASSET_DOCUMENT_CONTEXT = "Asset_Document";
    public static String CLIENT_DOCUMENT_CONTEXT = "Client_Document";
    public static String RESOURCE_DOCUMENT_CONTEXT = "Resource_Document";
    public static String SCOPE_DOCUMENT_CONTEXT = "Scope_Document";
    public static String WBS_DOCUMENT_CONTEXT = "Wbs_Document";

    protected static Log logger = LogFactory.getLog(RPMObjectFixture.class);

    /* List of methods that are not called through introspection for comparison */
    public static ArrayList skipSetters, skipGetters;

    static
    {
        //Setters
        skipSetters = new ArrayList();
        skipSetters.add("setID");
        skipSetters.add("setTypeID");
        skipSetters.add("setTypeIDModified");
        skipSetters.add("setRankID");
        skipSetters.add("setExternalID");
        skipSetters.add("setExternalIDModified");
        skipSetters.add("setDeleted");
        skipSetters.add("setDeletedModified");
        skipSetters.add("setModified");
        skipSetters.add("setNameModified");
        skipSetters.add("setContextName");
        //Getters
        skipGetters = new ArrayList();
        skipGetters.add("getID");
        skipGetters.add("getTypeID");
        skipGetters.add("getTypeIDModified");
        skipGetters.add("getRankID");
        skipGetters.add("getExternalID");
        skipGetters.add("getExternalIDModified");
        skipGetters.add("getDeleted");
        skipGetters.add("getDeletedModified");
        skipGetters.add("getModified");
        skipGetters.add("getNameModified");
        skipGetters.add("getContextName");
    }

    protected static void compare(Object[] pArray1, Object[] pArray2)
            throws AssertionFailedError, Exception
    {
        if(pArray1 != null && pArray1.length == 0)
            pArray1 = null;
        if(pArray2 != null && pArray2.length == 0)
            pArray2 = null;
        
        Assert.assertTrue((pArray1 == null && pArray2 == null)
                || (pArray1 != null && pArray2 != null));
        if (pArray1 == null)
            return;

        Assert.assertEquals(pArray1.length, pArray2.length);
    }

    protected static void compare(Object p1, Object p2)
    throws AssertionFailedError, Exception
    {
        compareWithSkip(p1, p2, skipGetters);
    }
    
    protected static void compareWithSkip(Object p1, Object p2, ArrayList localSkipGetters)
    throws AssertionFailedError, Exception{

        compareWithSkip(p1, p2, localSkipGetters, null);

    }

    protected static void compareWithSkip(Object p1, Object p2, 
            ArrayList localSkipGetters, String[] skipRegexPattern)
    throws AssertionFailedError, Exception
    {
        if (p2 != null) {
            Assert.assertFalse("object1 is null, while object2 is not null. Object: " + p2.getClass().getName(), p1 == null && p2 != null);
        }
        if (p1 != null) {
            Assert.assertFalse("object1 is not null, while object2 is null. Object: " + p1.getClass().getName(), p1 != null && p2 == null);                
        }
        if(p1 == null) return;

        Assert.assertEquals(p1.getClass(),p2.getClass());

        if((p1 instanceof RPMObject) && (((RPMObject) p1).getID() != null))
        {
            Assert.assertEquals(((RPMObject) p1).getID(), ((RPMObject) p2).getID());
        }
        HelperTestUtilities.assertEqualsByIntrospection(p1, p2, 
            localSkipGetters, skipRegexPattern);
    }

    protected static RPMObject save(ClientTestController ctc,
            RPMObject rpmObj, RPMObjectScope scope) throws AssertionError, Exception
    {
        SingleResult result;
        result = ctc.save(rpmObj, scope);

        Assert.assertNotNull(result);
        Assert.assertTrue(result.isSuccessful());
        Assert.assertNotNull(result.getRpmObject());        

        return result.getRpmObject();
    }
    
    protected static RPMObject saveWithoutDelete(ClientTestController ctc,
            RPMObject rpmObj, RPMObjectScope scope) throws AssertionError, Exception
    {
        SingleResult result;
        result = ctc.saveWithoutDelete(rpmObj, scope);

        Assert.assertNotNull(result);
        Assert.assertTrue(result.isSuccessful());
        Assert.assertNotNull(result.getRpmObject());        

        return result.getRpmObject();
    }

    protected static ArrayList removeNullAndDeletedRPMObjects(ArrayList pArrayList) {

        if(pArrayList == null){
            return null;
        }

        RPMObject object = null;
        for (int i = 0; i < pArrayList.size(); i++) {
            object = (RPMObject) pArrayList.get(i);
            if ((object == null)
                    || (object.getDeleted() != null && object.getDeleted()
                            .booleanValue())) {
                pArrayList.remove(i);
                i--;
            }
        }

        return pArrayList;

    }
    
    protected static RPMObject[] removeNullAndDeletedRPMObjects(RPMObject[] rpmArray)
    {
        if(rpmArray == null) return null;
        
        RPMObject[] newArray = null;
        ArrayList temp = new ArrayList(Arrays.asList(rpmArray));

        temp = removeNullAndDeletedRPMObjects(temp);

        if (temp != null && temp.size() > 0) {
            newArray = new RPMObject[temp.size()];
            temp.toArray(newArray);
        }

        return newArray;        
    }

    protected static void sortWithComparator(RPMObject[] array,
            Comparator comparator) {

        if (array != null) {
            Arrays.sort(array, comparator);
        }
    }
    
    private static int currentRandom = new Double(Math.random() * Integer.MAX_VALUE).intValue();
    
    public static int random()
    {
        return random(100);
    }
    
    public synchronized static int random(int max)
    {
        ++currentRandom;
        if( currentRandom == Integer.MAX_VALUE )
        {
            currentRandom = 0;
        }
        return currentRandom%max;
    }
    
    public static class RPMObjectNameAndIDComparator implements Comparator
    {
        public static final RPMObjectNameAndIDComparator INSTANCE = new RPMObjectNameAndIDComparator();
        
        public RPMObjectNameAndIDComparator()
        {
            super();
        }

        public int compare(Object o1, Object o2)
        {
            int retVal = compareNames(o1, o2);
            if( retVal == 0 )
            {
                retVal = compareIDs(o1, o2);
            }
            return retVal;
        }
    }
    
    public static class RPMObjectIDComparator implements Comparator
    {
        
        public static final RPMObjectIDComparator INSTANCE = new RPMObjectIDComparator();
        
        public RPMObjectIDComparator()
        {
            super();
        }

        public int compare(Object o1, Object o2)
        {
            return compareIDs(o1, o2);
        }
    }    
    
    public static int compareNames(Object o1, Object o2)
    {
        String key1 = null;
        String key2 = null;

        key1 = getObjectName(o1);

        key2 = getObjectName(o2);

        int retVal = compareStrings(key1, key2);
        return retVal;
    }

    public static String getObjectName(Object o1)
    {
        if( o1 == null )
        {
            return null;
        }
        
        String retVal = null;
        try
        {
            Method getterMethod = o1.getClass().getMethod("getName", null);
            Object objectRetVal = getterMethod.invoke(o1, null);
            if( objectRetVal != null )
                retVal = objectRetVal.toString();
        }
        catch (Exception e)
        {
            // ignore
        }
        return retVal;
    }
    
    public static int compareIDs(Object o1, Object o2)
    {
        String key1 = null;
        String key2 = null;

        if (o1 instanceof RPMObject)
        {
            RPMObject obj1 = (RPMObject) o1;
            key1 = obj1.getID();
        }

        if (o2  instanceof RPMObject)
        {
            RPMObject obj2 = (RPMObject) o2;
            key2 = obj2.getID();
        }

        int retVal = compareStrings(key1, key2);
        return retVal;
    }
    
    public static int compareStrings(String key1, String key2)
    {
        if (key1 == null && key2 == null)
        {
            return 0;
        }

        if (key1 == null && key2 != null)
        {
            return -1;
        }

        if (key1 != null && key2 == null)
        {
            return 1;
        }

        int retVal = key1.compareTo(key2);
        return retVal;
    }

}