package com.ibm.rpm.client.junit.framework.testcases;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.IdentityHashMap;

import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.metadata.model.MetadataUtility;
import com.ibm.rpm.metadata.model.PointerLink;

public class RpmTestFramework
{

    public static final RpmTestFramework INSTANCE = new RpmTestFramework();

    private IdentityHashMap fixtureMap = new IdentityHashMap();

    private IdentityHashMap scopeMap = new IdentityHashMap();

    private IdentityHashMap staticFixtureMap = new IdentityHashMap();

    private IdentityHashMap staticPointerMap = new IdentityHashMap();

    private IdentityHashMap enumMap = new IdentityHashMap();

    public PointerLink getStaticPointer(Class containerClass)
    {
        PointerLink retVal = (PointerLink) staticPointerMap.get(containerClass);
        if (retVal == null)
        {
            retVal = MetadataUtility.createPointer(containerClass);
            staticPointerMap.put(containerClass, retVal);
        }

        return retVal;
    }

    public RPMObject getStaticFixtureFromPointer(PointerLink pointer, int index)
            throws Exception
    {
        RPMObject retVal = null;

        HashMap pointerMap = (HashMap) staticFixtureMap.get(pointer);
        if (pointerMap == null)
        {
            pointerMap = new HashMap();
            staticFixtureMap.put(pointer, pointerMap);
        }

        Integer key = new Integer(index);

        retVal = (RPMObject) pointerMap.get(key);
        if (retVal == null)
        {
            PointerLink currentPointer = pointer;
            while (currentPointer != null)
            {
                retVal = findFixture(currentPointer.getPointerClass())
                    .loadStaticFixture(retVal, index);
                currentPointer = currentPointer.getChildLink();

            }
        }
        return retVal;
    }

    public RpmObjectAbstractTester findFixture(Class objectClass)
            throws Exception
    {
        RpmObjectAbstractTester rpmObjectAbstractTester = (RpmObjectAbstractTester) fixtureMap
            .get(objectClass);

        if (rpmObjectAbstractTester == null)
        {
            rpmObjectAbstractTester = createFixture(objectClass);

            registerFixture(rpmObjectAbstractTester);
        }

        return rpmObjectAbstractTester;
    }

    public static String extractFieldName(Method temp)
    {
        String retVal = temp.getName();

        if (retVal.startsWith("is"))
            retVal = retVal.substring(2).toLowerCase();
        else
            retVal = retVal.substring(3).toLowerCase();

        return retVal;
    }

    public static boolean isSetterMethod(Method temp)
    {
        Class[] parameterTypes = temp.getParameterTypes();
        return !Modifier.isStatic(temp.getModifiers())
                && temp.getName().startsWith("set")
                && parameterTypes.length == 1
                && !temp.getName().endsWith("Modified");
    }

    public static boolean isGetterMethod(Method temp)
    {
        Class[] parameterTypes = temp.getParameterTypes();
        return !Modifier.isStatic(temp.getModifiers())
                && (temp.getName().startsWith("get") || temp.getName()
                    .startsWith("is")) && parameterTypes.length == 0
                && !temp.getName().endsWith("Modified")
                && !temp.getName().equals("getClass");
    }

    public RpmObjectAbstractTester createFixture(Class objectClass)
            throws Exception
    {
        RpmObjectAbstractTester rpmObjectAbstractTester = null;

        try
        {
            String fixtureName = "com.ibm.rpm.client.junit"
                    + objectClass.getName().substring(11) + "Fixture";
            Class fixtureClass = Class.forName(fixtureName);
            rpmObjectAbstractTester = (RpmObjectAbstractTester) fixtureClass
                .getConstructor(null).newInstance(null);
        }
        catch (Exception e)
        {
            // Normal : we want to use the metadata
        }

        if (rpmObjectAbstractTester == null)
        {
            rpmObjectAbstractTester = new MetadataFixture(objectClass);
        }

        // rpmObjectAbstractTester.fillFieldList();

        return rpmObjectAbstractTester;
    }

    public void registerFixture(RpmObjectAbstractTester fixture)
    {
        fixtureMap.put(fixture.getObjectClass(), fixture);
    }
}