package com.ibm.rpm.client.junit.framework.testcases;

import java.util.Iterator;

import junit.framework.Assert;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.ibm.rpm.client.junit.ClientTestController;

/**
 * @author mcduffs
 */
public class SuiteGenerator
{

    public static TestSuite createAndPopulateSuite(Class classToTest)
    {
        TestSuite suite = new TestSuite();

        SuiteGenerator.populateSuite(suite, classToTest);

        return suite;
    }

    public static void populateSuite(TestSuite suite, Class classToTest)
    {
        ClientTestController.initTestController();
        RpmObjectAbstractTester findFixture;
        try
        {
            findFixture = RpmTestFramework.INSTANCE.findFixture(classToTest);
            findFixture.planTestList();
        }
        catch (Throwable e)
        {
            e.printStackTrace();
            Assert.fail(e.toString());
            return;
        }

        suite.addTest(SETUP);

        Iterator it = findFixture.getPendingTestList().iterator();
        while (it.hasNext())
        {
            PlannedTest element = (PlannedTest) it.next();
            suite.addTest(element);
        }

        suite.addTest(TEARDOWN);
    }

    public static final SetupTest SETUP = new SetupTest();

    public static final TeardownTest TEARDOWN = new TeardownTest();

    public static class SetupTest extends TestCase
    {

        public SetupTest()
        {
            super("runSetup");
        }

        public void runSetup() throws Exception
        {
            ClientTestController.INSTANCE.callSetup();
        }
    }

    public static class TeardownTest extends TestCase
    {

        public TeardownTest()
        {
            super("runTearDown");
        }

        public void runTearDown() throws Exception
        {
            ClientTestController.INSTANCE.callTearDown();
        }
    }

}
