package com.ibm.rpm.client.junit.framework.testcases;

import java.util.Calendar;
import java.util.TimeZone;

import junit.framework.TestCase;

import com.ibm.rpm.framework.util.CompareUtil;

public class TestCompareUtilsLocal extends TestCase
{
    public void testCalendarCompareSuccess()
    {
        long timeInMillis = Calendar.getInstance().getTimeInMillis();
       
        int nbTimezones = 12;
        Calendar[] calendars = createCalendars(nbTimezones, timeInMillis, true);
        compareCalendars(calendars, true); 
    }
    
    public void testCalendarCompareFailureOntimeInMillis()
    {
        long timeInMillis = Calendar.getInstance().getTimeInMillis();
       
        int nbTimezones = 12; 
        Calendar[] calendars = createCalendars(nbTimezones, timeInMillis, false);
        compareCalendars(calendars, false);
    }
    
    public void testCalendarCompareFailureOnHours()
    {
        long timeInMillis = Calendar.getInstance().getTimeInMillis();
       
        int nbTimezones = 12; 
        Calendar[] calendars = createCalendars(nbTimezones, timeInMillis, true);
        
        for(int i=0;i<calendars.length;i++)
        {
            calendars[i].set(Calendar.YEAR, 2037);
            calendars[i].set(Calendar.MONTH, 2);
            calendars[i].set(Calendar.DAY_OF_MONTH, 3);
            calendars[i].set(Calendar.HOUR_OF_DAY, 8);
        }
        
        compareCalendars(calendars, false);
    }
    
    private void compareCalendars(Calendar[] calendars, boolean expectedResult)
    {
        for(int i=0;i<calendars.length;i++)
        {
            for(int j=i+1;j<calendars.length;j++)
            {
                assertTrue(CompareUtil.rpmObjectsEquals(calendars[i], calendars[j])==expectedResult);
            }
        }
    }
    
    private Calendar[] createCalendars(int nbTimezones, long timeInMillis, boolean useSameTimeInMillis)
    {
        Calendar[] calendars = new Calendar[(nbTimezones*2)+1];
        String timezoneId = null;
        int delta = 0; 
        for(int i=-nbTimezones;i<=nbTimezones;i++)
        {
            timezoneId = constructTimezoneId(i);
            delta = (useSameTimeInMillis)?0:i;
            calendars[i+nbTimezones] = createCalendar(timezoneId, timeInMillis+delta);
        }
        return calendars;
    }
    
    private String constructTimezoneId(int offset)
    {
        String GMT = "GMT";
        if(offset>0)
            return GMT+"+"+offset;
        else if (offset<0)
            return GMT+offset;
        else
            return GMT;
    }
    
    private Calendar createCalendar(String timezoneId, long timeInMillis)
    {
        Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone(timezoneId));
        calendar.setTimeInMillis(timeInMillis);
        return calendar;
    }   
}
