package com.ibm.rpm.client.junit.framework.testcases;

import java.util.Calendar;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;

/**
 * Tests com.ibm.rpm.interfaces.impl.DotNetCalendarDeserializer from the
 * rpm-web-services project.
 */
public class TestDotNetNullDate extends ClientTestController
{
    public void testNullDate() throws Exception
    {
        // Creates, saves and reloads the parent pool.
        Pool parentPool = PoolFixture.createSaveFixture(this);

        // Creates an empty, .NET style date (0001-01-01T00:00:00).
        Calendar originalDate = Calendar.getInstance();
        originalDate.set(1, 0, 1, 0, 0, 0);
        logger.debug("\nOriginal date: " + originalDate.getTime() + "\n");

        // Creates, saves and reloads the resource.
        Resource resource = ResourceFixture.createFixture(parentPool, null,
                this);
        // Note: EffectiveStartDate is needed for business rule.
        resource.setEffectiveStartDate(originalDate);
        resource.setDiscontinueDate(originalDate);
        Resource autoReloadedRes = (Resource) save(resource, null)
                .getRpmObject();
        Assert.assertNotNull(autoReloadedRes.getID());

        // Validate the discontinue date.
        Calendar savedDate = autoReloadedRes.getDiscontinueDate();
        if (isRemote())
        {
            // When testing remotely.
            // Asserts that the .NET style, empty DateTime (0001-01-01T00:00:00)
            // is converted to a Java null Calendar.
            Assert.assertNull(savedDate);
        }
        else
        {
            // When testing locally.
            // Allows 1 second delta when comparing.
            Calendar thresholdDate = Calendar.getInstance();
            thresholdDate.set(1, 0, 1, 0, 0, 1);
            logger.debug("\nAuto-reloaded discontinue date: "
                    + savedDate.getTime() + " - Threshold date: "
                    + thresholdDate.getTime() + "\n");
            Assert.assertTrue(thresholdDate.after(savedDate));
        }
    }
}
