/*
 * Created on Sep 13, 2006
 */
package com.ibm.rpm.client.junit.framework.testcases;

import java.text.SimpleDateFormat;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;
import com.ibm.rpm.wbs.types.ResourceType;

public class TestManualCheckoutSummaryTimesheet extends ClientTestController
{
    public String projectName = "TCTIMESHEET_SAMPLE_PROJECT";

    public String taskName = "TCTIMESHEET_SAMPLE_TASK";

    public String resourceName = "TCTIMESHEET_SAMPLE_RES";

    public String poolName = "TCTIMESHEET_SAMPLE_POOL";

    public Project project = null;

    public Task task = null;

    public Resource resource = null;

    private TaskAssignment taskAssignment = null;

    private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

    public void setUp() throws Exception
    {
        super.setUp();
        getApplicationInterface().disableAutomaticCheckoutMode(getSessionID());
        createResource();
        createTask();
        assignResourceToTask();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testManualCheckoutAndTimesheet() throws Exception
    {
        String xpath = "Task[name = '" + taskName
                + "']/ResourceTaskAssignment[@resource[fullname = '"
                + resourceName + "']]/TaskAssignment";
        RPMObject[] rpmObjects = loadFromXpath(xpath, null);
        taskAssignment = (TaskAssignment) rpmObjects[0];

        createSummaryTimesheet();

        loadFromXPathAndUpdate();
    }

    private void loadFromXPathAndUpdate() throws Exception
    {
        SummaryTimesheet summaryTimesheet = null;
        String xpath = "/SummaryTimesheet[@resource[fullname = '"
                + resourceName
                + "'] and @workElement[name = '"
                + taskName + "']]";
        RPMObject[] rpmObjects = loadFromXpath(xpath, null);
        summaryTimesheet = (SummaryTimesheet) rpmObjects[0];
        summaryTimesheet.setID(null);
        summaryTimesheet.setSpecialThursday(new Double(5));
        checkOut(summaryTimesheet);
        SingleResult result2 = save(summaryTimesheet, null,
            ReloadType.ReloadResult);
        if (result2.isSuccessful())
        {
            summaryTimesheet = (SummaryTimesheet) result2.getRpmObject();
        }
        getApplicationInterface().checkIn(getSessionID(), summaryTimesheet);
    }

    private void createSummaryTimesheet() throws Exception
    {
        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();

        // Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(taskAssignment.getAssignmentStartDate());
        summaryTimesheet.setRegularMonday(new Double(8));

        // Add resource and task to the scope (needed to print these
        // information)
        SummaryTimesheetScope stScope = new SummaryTimesheetScope();
        stScope.setResource(true);
        stScope.setWorkElement(new WorkElementScope());

        SingleResult result1 = save(summaryTimesheet, stScope,
            ReloadType.ReloadResult);

        if (result1.isSuccessful())
        {
            summaryTimesheet = (SummaryTimesheet) result1.getRpmObject();
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        }
    }

    public void assignResourceToTask() throws Exception
    {

        System.out.println("Assign Resource to Task with a Security role");

        // Create resourceTaskAssignment
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();

        // Load the security role
        RPMObject[] rpmObjects = loadFromXpath(
            "/SecurityRole[name = 'Task Responsible']", null);

        SecurityRole securityRole = null;
        securityRole = (SecurityRole) rpmObjects[0];

        resourceTaskAssignment.setSecurityRole(securityRole);
        resourceTaskAssignment.setResource(resource);

        // Create TaskAssignment
        TaskAssignment taskAssignment = new TaskAssignment();
        resourceTaskAssignment.setTaskAssignment(taskAssignment);
        task.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            resourceTaskAssignment
        });

        // Create task scope
        WorkElementScope taskScope = new WorkElementScope();

        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setResource(new ResourceScope());
        TaskAssignmentScope taScope = new TaskAssignmentScope();
        rtaScope.setTaskAssignment(taScope);
        taskScope.setResourceTaskAssignments(rtaScope);

        // Save Task
        checkOut(task);
        SingleResult result2 = save(task, taskScope, ReloadType.None);

        // Copy Project proposed dates to plan dates (mandatory for timesheet
        // creation)

        SingleResult result3 = getApplicationInterface().copyProposedToPlan(
            getSessionID(), project, null);
        getApplicationInterface().checkIn(getSessionID(), task);
    }

    public void createTask() throws Exception
    {

        System.out.println("Create and save a Task under a Project");

        project = new Project();
        project.setName(projectName);

        // Publish project to team (mandatory for timesheet creation)
        project.setPublished(ProjectPublishedType.ToTeam);

        task = new Task();
        task.setName(taskName);

        project.setChildren(new Task[]
        {
            task
        });

        // Create projectScope
        WorkElementScope projectScope = new WorkElementScope();
        WorkElementScope taskScope = new WorkElementScope();

        // Contaning project is mandatory to create a ResourceTaskAssignment
        taskScope.setContainingProject(new WorkElementScope());
        projectScope.setChildren(taskScope);

        SingleResult result1 = save(project, projectScope,
            ReloadType.ReloadResult);

        project = (Project) result1.getRpmObject();
        task = (Task) project.getChildren()[0];
    }

    public void createResource() throws Exception
    {

        System.out.println("Create and save a Resource under a Pool");

        Pool pool = new Pool();
        pool.setName(poolName);

        resource = new Resource();
        resource.setFullName(resourceName);

        // Load the default calendar
        RPMObject[] rpmObjects = loadFromXpath(
            "/RPMCalendar[name = 'Standard (Default)']", null);
        RPMCalendar calendar = (RPMCalendar) rpmObjects[0];

        // Resource calendar is mandatory to assign a resource
        resource.setCalendar(calendar);

        pool.setResources(new Resource[]
        {
            resource
        });

        PoolScope poolScope = new PoolScope();
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        poolScope.setResources(resourceScope);

        SingleResult result2 = save(pool, poolScope, ReloadType.ReloadResult);

        pool = (Pool) result2.getRpmObject();
        resource = pool.getResources()[0];
    }
}
