/**
 * Created on Jan 19, 2006
 * @author shahla
 */
package com.ibm.rpm.client.junit.framework.testcases.util;

import com.ibm.rpm.financial.containers.AbstractTimeCode;
import com.ibm.rpm.financial.containers.GenericTimeCode;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeCategory;
import com.ibm.rpm.financial.containers.TimeCodeGroup;
import com.ibm.rpm.framework.util.StringUtil;


public class TimeCodeUtil
{
    public static boolean isChildOfTimeCode1(AbstractTimeCode timeCode)
            throws Exception
    {
        return isTimeCode1(getRootGroup(timeCode));
    }
    
    public static boolean isChildOfTimeCode2(AbstractTimeCode timeCode)
            throws Exception
    {
        return isTimeCode2(getRootGroup(timeCode));
    }
    
    public static boolean isTimeCode1(TimeCodeGroup timeCodeGroup)
    {
        if (timeCodeGroup == null)
            return false;
        
        return timeCodeGroup.getID().trim().equals("TIMECODE_1");
    }
    
    public static boolean isTimeCode2(TimeCodeGroup timeCodeGroup)
    {
        if (timeCodeGroup == null)
            return false;
        
        return timeCodeGroup.getID().trim().equals("TIMECODE_2");
    }
    
    public static TimeCode getFirstTimeCode(TimeCodeCategory category)
            throws Exception
    {
        GenericTimeCode[] children = category.getChildren();

        if (children == null || children.length == 0)
        {
            throw new Exception("category \"" + category.getName()
                    + "\" does not have children");
        }

        TimeCode timeCode = null;
        for (int i = 0; i < children.length && timeCode == null; i++)
        {
            if (children[i] instanceof TimeCode)
                timeCode = (TimeCode) children[i];
        }

        return timeCode;
    }
    
    public static TimeCodeGroup getRootGroup(AbstractTimeCode timeCode)
            throws Exception
    {
        TimeCodeCategory parent = null;
        TimeCodeGroup root = null;

        if (timeCode.getParent() instanceof TimeCodeCategory)
            parent = (TimeCodeCategory) timeCode.getParent();
        else if (timeCode.getParent() instanceof TimeCodeGroup)
            root = (TimeCodeGroup) timeCode.getParent();
        else
            throw new Exception("parent of "
                    + StringUtil.getShortClassName(timeCode.getClass()) + " \""
                    + timeCode.getName()
                    + "\" is neither TimeCodeCategory nor TimeCodeGroup!");

        if (parent != null)
        {
            while (root == null)
            {
                if (parent.getParent() instanceof TimeCodeCategory)
                    parent = (TimeCodeCategory) parent.getParent();
                else if (parent.getParent() instanceof TimeCodeGroup)
                    root = (TimeCodeGroup) parent.getParent();
                else
                    throw new Exception("parent of "
                        + StringUtil.getShortClassName(timeCode.getClass()) + " \""
                        + timeCode.getName()
                        + "\" is neither TimeCodeCategory nor TimeCodeGroup!");
            }
        }
        else
        {
            return null;
        }
        return root;
    }
}
