package com.ibm.rpm.client.junit.framework.testcases.validation;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;

/**
 * @author dominiqu
 */
public class TestLoginValidation extends GenericValidation
{
    protected void setUp() throws Exception
    {

        createInterfaceProvider();
        INTERFACE_PROVIDER.setUp();

        logger.info("=================");
        logger.info(this.getName());
        logger.info("=================");
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void performLoginMustFail(String username, String password,
            RPMExceptionMock expectedException) throws Exception
    {
        // Attempt to login
        try
        {
            sessionid = getAuthenticateInterface().login(username, password,
                DSN);
            // If we get this far, the login worked, and thus the validation
            // failed
            Assert.fail("Login validation failed: expected exceptions");
        }
        catch (RPMException e)
        {
            // Check that we received the right exception
            String errorMessage = verifyExceptions(new RPMExceptionMock[]
            {
                expectedException
            }, new RPMException[]
            {
                e
            });

            if (errorMessage.length() != 0)
                Assert.fail(errorMessage);
        }
    }

    public void testFailureUsernameValidationMax() throws Exception
    {
        String username = createFixedLengthString(ValidationConstants.RESOURCE_USERNAME_MAX + 1);
        String password = createFixedLengthString(ValidationConstants.RESOURCE_PASSWORD_MAX);

        RPMExceptionMock expectedException = createRPMException(
            ErrorCodes.MAX_LENGTH, null);

        performLoginMustFail(username, password, expectedException);
    }

    public void testFailurePasswordValidationMax() throws Exception
    {
        String username = createFixedLengthString(ValidationConstants.RESOURCE_USERNAME_MAX);
        String password = createFixedLengthString(ValidationConstants.RESOURCE_PASSWORD_MAX + 1);

        RPMExceptionMock expectedException = createRPMException(
            ErrorCodes.MAX_LENGTH, null);

        performLoginMustFail(username, password, expectedException);
    }

    public void testFailureValidationMin() throws Exception
    {
        String username = createFixedLengthString(ValidationConstants.RESOURCE_USERNAME_MIN - 1);
        String password = ""; // No minimum

        RPMExceptionMock expectedException = createRPMException(
            ErrorCodes.MIN_LENGTH, null);

        performLoginMustFail(username, password, expectedException);
    }
}
