/*
 * Created on 9-Nov-2005
 */
package com.ibm.rpm.client.junit.framework.testcases.xpath;

import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;

import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.metadata.model.MetadataStore;

/**
 * @author njarazi
 */
public class XpathTesterUtils
{
    //Constants for the XPath testing framework
    public static final String GREGORIAN_CALENDAR_STRING = "GregorianCalendar";
    public static final int NOT_FOUND = -1;
    public static final int COMPLEX_XPATH_QUERIES_NUMBER_TO_GENERATE = 2;
    public static final String GET_PREFIX = "get";
    public static final String GET_PARENT = "getParent";
    public static final String MODIFIED_TRAILER = "Modified";
    public static final String ID_MARKER = "ID";
    public static final String PERIOD = ".";
    public static final String SPACE = " ";
    public static final String EQUALS = "=";
    public static final String FORWARD_SLASH = "/";
    public static final String OPEN_BRACKET = "[";
    public static final String CLOSE_BRACKET = "]";
    public static final String SINGLE_QUOTE = "'";
    public static final String SEMI_COLON = ";";
    public static final String TRUE = "true";
    
    protected static ArrayList numericTypes = null;
    protected static ArrayList booleanFieldsWithoutQuotes = null;
    protected static ArrayList numericFieldsWithQuotes = null;
    
    static
    {
        numericTypes = new ArrayList();
        
        numericTypes.add(Integer.class);
        numericTypes.add(Integer[].class);
        numericTypes.add(Integer.TYPE);
        numericTypes.add(Byte.class);
        numericTypes.add(Byte[].class);
        numericTypes.add(Byte.TYPE);
        numericTypes.add(Short.class);
        numericTypes.add(Short[].class);
        numericTypes.add(Short.TYPE);
        numericTypes.add(Long.class);
        numericTypes.add(Long[].class);
        numericTypes.add(Long.TYPE);
        numericTypes.add(Float.class);
        numericTypes.add(Float[].class);
        numericTypes.add(Float.TYPE);
        numericTypes.add(Double.class);
        numericTypes.add(Double[].class);
        numericTypes.add(Double.TYPE);
        
        numericTypes.trimToSize();
    }
    
    static
    {
        booleanFieldsWithoutQuotes = new ArrayList();
        
        booleanFieldsWithoutQuotes.add("getMaskEmployeeInfo");
        
        booleanFieldsWithoutQuotes.trimToSize();
    }
    
    static
    {
        numericFieldsWithQuotes = new ArrayList();
        
        numericFieldsWithQuotes.add("getRank");
        
        numericFieldsWithQuotes.trimToSize();
    }
    
    /**
     * Convert a Calendar to a YYYY-MM-DD format.
     * @param calendar
     * @return
     */
    public static String convertToString(Calendar calendar){
        
        String date = null;
        
        if(calendar != null){
    		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
    		date = format.format(calendar.getTime());
        }
        
        return date;
        
    }
    
    public static boolean isStringAllDigits(String checkString)
    {
        for (int i = 0; i < checkString.length(); i++)
        {
            boolean check = Character.isDigit(checkString.charAt(i));

            if (!check)
            {
                return false;
            }
        }

        return true;
    }
    
    public static boolean isMethodReturnBoolean(Method method)
    {
        Class returnType = method.getReturnType();
        return isTypeBoolean(returnType);
    }
    
    public static boolean isTypeBoolean(Class returnType)
    {
        if (returnType == Boolean.class ||
                returnType == Boolean.TYPE ||
                returnType == Boolean[].class)
        {
            return true;
        }
       
        return false;
    }

    public static boolean isMethodReturnNumeric(Method method)
    {
        return isTypeNumeric(method.getReturnType());
    }
    
    public static boolean isTypeNumeric( Class typeClass )
    {
        return numericTypes.contains(typeClass);
    }
    
    public static boolean isEnumeratedType(Class typeClass)
    {
        return ( MetadataStore.getInstance().getEnumerationByJavaClass(typeClass) != null);
    }
    
    public static String getObjectName(RPMObject object)
    {
        String objectName = object.getClass().getName();
        objectName = objectName.substring(objectName
                .lastIndexOf(XpathTesterUtils.PERIOD) + 1, objectName.length());
        return objectName;
    }
}
