/*
 * Created on 13-Dec-2005
 */
package com.ibm.rpm.client.junit.framework.testcases.xpath.errors;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.framework.testcases.xpath.XpathTesterUtils;
import com.ibm.rpm.client.junit.framework.testcases.xpath.stats.StatsManager;

/**
 * @author njarazi
 */
public class ErrorHandler
{
    private boolean testFailed = false;
    
    public static final int MAX_ERROR_COUNT = 10;
    
    public ErrorHandler() { /* do nothing */ }
    
    public void handleError()
    {
        if (testFailed)
        {
	        //Exception thrown so that test fails...if not thrown, the test
	        //passes (in JUnit), although it has failed
	        String msg = StatsManager.getErrorLog() + StatsManager.getStatsLog();
	        StatsManager.clearStats();
	        StatsManager.clearErrorLog();
	        throw new AssertionFailedError(msg);
        }
    }
    
    public void logOtherError(String msg, String errorMsg)
    {
        testFailed = true;
        StatsManager.incrementErrorCount();
        
        if ( StatsManager.getErrorCount() > MAX_ERROR_COUNT )
        {
            // prevent stack overflow on unit test report 
            // generation by limiting the console output
            handleError();
            return;
        }

        StatsManager.appendFieldError(msg);
        if (errorMsg != null)
        {
            StatsManager.appendFieldErrorMessage(errorMsg);
        }
    }

    /**
     * Logs an error
     * 
     * @param xpath
     *            The xPath string used to query the DB
     * @param e
     *            The exception generated by the error
     */
    public void logXPathError(int queryNumber, String xpath, Exception e,
            boolean returnIsNull, boolean simpleType)
    {
        StringBuffer errorMsg = new StringBuffer();

        testFailed = true;
        
        StatsManager.incrementErrorCount();
        
        if ( StatsManager.getErrorCount() > MAX_ERROR_COUNT )
        {
            // prevent stack overflow on unit test report 
            // generation by limiting the console output
            handleError();
            return;
        }

        if (simpleType)
        {
            errorMsg.append("Simple #");
        }
        else
        {
            errorMsg.append("Complex #");
        }

        errorMsg.append(queryNumber).append(": ");

        errorMsg.append(StatsManager.NEW_LINE).append(StatsManager.FOUR_TABS)
                .append("Query: ").append(xpath).append(StatsManager.NEW_LINE);

        errorMsg.append(StatsManager.FOUR_TABS).append("Field: ");

        if (xpath.indexOf(XpathTesterUtils.EQUALS) != -1 && xpath.indexOf(XpathTesterUtils.OPEN_BRACKET) != -1)
        {
            errorMsg.append(xpath.substring(xpath.indexOf(XpathTesterUtils.OPEN_BRACKET) + 1,
                    xpath.indexOf(XpathTesterUtils.EQUALS)));
        }

        if (errorMsg.toString().endsWith("[ID"))
        {
            errorMsg.delete(errorMsg.length() - 3, errorMsg.length());
        }

        StatsManager.appendFieldError(errorMsg.toString());

        if (returnIsNull)
        {
            StatsManager
                    .appendFieldErrorMessage("Object returned by query is NULL");
        }
        else
        {
            if (e != null)
            {
                if (e instanceof ArrayIndexOutOfBoundsException)
                {
                    StatsManager
                            .appendFieldErrorMessage("Could not find the local object in the list of objects which were loaded from the database.");
                }
                else
                {
                    StatsManager.appendFieldErrorMessage("Error message: "
                            + e.getMessage());
                }
            }
        }

        StatsManager.appendNewLine();
    }
    
    /**
     * Method for printing stats and errors It should always be the last one of
     * the tests, so that all the stats and errors are logged and printed
     * properly
     */
    public static void printStatsAndErrorsInConsole()
    {
        StatsManager.printErrorLog();
        StatsManager.clearErrorLog();
        StatsManager.printStats();
        StatsManager.clearStats();
    }
    
    /**
     * @return Returns the testFailed.
     */
    public  boolean isTestFailed()
    {
        return testFailed;
    }
    /**
     * @param testFailed The testFailed to set.
     */
    public void setTestFailed(boolean testFailed)
    {
        this.testFailed = testFailed;
    }    
}
