/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.client.junit.framework.testcases.xpath.singletests;

import com.ibm.rpm.applicationadministration.scope.PoolDomainScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.framework.testcases.xpath.XpathTesterUtils;
import com.ibm.rpm.client.junit.framework.testcases.xpath.errors.ErrorHandler;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.scope.*;

/**
 * @author njarazi
 */
public class TestSupportedXPathFunctions extends ClientTestController
{
    private static final int VERIFICATION_LENGTH = 6;
    
    private ErrorHandler errorHandler = new ErrorHandler();
    
    private Pool myPool = null;

    private PoolScope mainScope = null;

    public void setUp() throws Exception
    {
        super.setUp();
        initWithFullScope();
        save(myPool, mainScope);
    }

    private void initWithFullScope()
    {
        try
        {
            mainScope = createScope();
            myPool = createObject();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    protected Pool createObject() throws Exception
    {
        return PoolFixture.createFixture(true, 1, mainScope, this);
    }

    protected PoolScope createScope() throws Exception
    {
        PoolScope parentScope = new PoolScope();
        PoolScope s = new PoolScope();

        s.setAttributeAssignments(true);
        s.setDomain(new PoolDomainScope());
        s.setParent(parentScope);
        s.setPools(new PoolScope());
        s.setPoolReferences(new PoolReferenceScope());
        s.setResources(new ResourceScope());

        return s;
    }

    public void testFunctionStartsWith()
    {
        StringBuffer xpath = new StringBuffer();

        xpath.append(XpathTesterUtils.FORWARD_SLASH)
        	.append("Pool")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("starts-with(Name, '")
            .append(myPool.getName().substring(0, VERIFICATION_LENGTH))
            .append("')")
            .append(XpathTesterUtils.CLOSE_BRACKET);

        try
        {
            RPMObject[] dbReturn = loadFromXpath(xpath.toString(), mainScope);

            for (int i = 0; i < dbReturn.length; i++)
            {
                if (!((Pool) dbReturn[i]).getName().startsWith(
                        myPool.getName().substring(0, VERIFICATION_LENGTH)))
                {
                    errorHandler.logOtherError("Got : " 
                            + ((Pool) dbReturn[i]).getName().substring(0, VERIFICATION_LENGTH) + " when expected "
                            + myPool.getName().substring(0, VERIFICATION_LENGTH),
                    	"The name of the loaded Pool does not start like expected");
                }
            }
        }
        catch (Exception e)
        {
            errorHandler.logOtherError("An error occured", e.getMessage());
        }
        
        errorHandler.handleError();
    }

    public void testFunctionContains()
    {
        StringBuffer xpath = new StringBuffer();

        xpath.append(XpathTesterUtils.FORWARD_SLASH)
        	.append("Pool")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("contains(Name, '")
            .append(myPool.getName().substring(2, VERIFICATION_LENGTH))
            .append("')")
            .append(XpathTesterUtils.CLOSE_BRACKET);

        try
        {
            RPMObject[] dbReturn = loadFromXpath(xpath.toString(), mainScope);

            for (int i = 0; i < dbReturn.length; i++)
            {
                if (!((Pool) dbReturn[i]).getName().startsWith(
                        myPool.getName().substring(0, VERIFICATION_LENGTH)))
                {
                    errorHandler.logOtherError("Got : " 
                            + ((Pool) dbReturn[i]).getName().substring(0, VERIFICATION_LENGTH) + " when expected "
                            + myPool.getName().substring(0, VERIFICATION_LENGTH),
                    	"The name of the loaded Pool does not start like expected");
                }
            }
        }
        catch (Exception e)
        {
            errorHandler.logOtherError("An error occured", e.getMessage());
        }
        
        errorHandler.handleError();
    }
}
