/*
 * Created on 8-Dec-2005
 */
package com.ibm.rpm.client.junit.framework.testcases.xpath.singletests;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.framework.testcases.xpath.XpathTesterUtils;
import com.ibm.rpm.client.junit.framework.testcases.xpath.errors.ErrorHandler;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author njarazi
 */
public class TestSupportedXPathOperators extends ClientTestController
{
    private ErrorHandler errorHandler = new ErrorHandler();
    
    private Resource myResource = null;

    private ResourceScope mainScope = null;

    private Pool parentPool;

    public void setUp() throws Exception
    {
        super.setUp();
        initWithFullScope();
        save(myResource, mainScope);
    }

    private void initWithFullScope()
    {
        try
        {
            mainScope = createScope();
            myResource = createObject();
        }
        catch (Exception e)
        {

            e.printStackTrace();
        }
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    private Resource createObject() throws Exception
    {
        //Parent pool is needed
        parentPool = PoolFixture.createSaveFixture(this);
        return ResourceFixture.createFixture(parentPool, mainScope, this);
    }

    private ResourceScope createScope() throws Exception
    {
        ResourceScope scope = new ResourceScope();
        scope.setRates(true);
        return scope;
    }

    public void testOperatrEquals()
    {
        StringBuffer xpath = new StringBuffer();

        xpath.append(XpathTesterUtils.FORWARD_SLASH)
        	.append("Resource")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("FamilyName = '")
        	.append(myResource.getFamilyName())
        	.append("'")
        	.append(XpathTesterUtils.CLOSE_BRACKET);

        try
        {
            RPMObject[] dbReturn = loadFromXpath(xpath.toString(), mainScope);

            if (dbReturn != null)
            {
	            for (int i = 0; i < dbReturn.length; i++)
	            {
	                if (!((Resource)dbReturn[i]).getFamilyName().equals(
	                        myResource.getFamilyName()))
	                {
	                    errorHandler.logOtherError("Got : "
	                                    + ((Resource) dbReturn[i]).getFamilyName()
	                                    + " when expected "
	                                    + myResource.getFamilyName(),
	                                    "The family name of the loaded Resource is not what was expected");
	                }
	            }
            }
        }
        catch (Exception e)
        {             
            errorHandler.logOtherError("An error occured", e
                    .getMessage());
        }
        
        errorHandler.handleError();
    }

    public void testOperatorNotEquals()
    {
        StringBuffer xpath = new StringBuffer();

        xpath.append(XpathTesterUtils.FORWARD_SLASH)
        	.append("Resource")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("FamilyName != 'Some family name'")
        	.append(XpathTesterUtils.CLOSE_BRACKET);

        try
        {
            RPMObject[] dbReturn = loadFromXpath(xpath.toString(), mainScope);

            if (dbReturn != null)
            {
	            for (int i = 0; i < dbReturn.length; i++)
	            {
	                if (((Resource)dbReturn[i]).getFamilyName().equals(
	                        "Some family name"))
	                {
	                    errorHandler
	                            .logOtherError(
	                                    "The family name of the loaded DB object was 'Some family name' but was expected to be different",
	                                    "The family name of the loaded Resource is not what was expected");
	                }
	            }
            }
        }
        catch (Exception e)
        {
            errorHandler.logOtherError("An error occured", e
                    .getMessage());
        }

    	errorHandler.handleError();
    }

    public void testOperatorLessThan()
    {
        final double rate = 100;
        StringBuffer xpath = new StringBuffer();

        xpath.append(XpathTesterUtils.FORWARD_SLASH)
        	.append("Resource")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("ResourceRates")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("RegularCostPerHour < ")
        	.append(rate)
        	.append(XpathTesterUtils.CLOSE_BRACKET)
        	.append(XpathTesterUtils.CLOSE_BRACKET);

        try
        {
            RPMObject[] dbReturn = loadFromXpath(xpath.toString(), mainScope);

            if (dbReturn != null)
            {
	            for (int i = 0; i < dbReturn.length; i++)
	            {
	                if (((Resource)dbReturn[i]).getRates() != null)
	                {
		                if (!(((Resource) dbReturn[i]).getRates()
		                        .getRegularCostPerHour().doubleValue() < rate))
		                {
		                    errorHandler
		                            .logOtherError(
		                                    "The regular cost per hour of the loaded DB object was "
		                                            + ((Resource) dbReturn[i])
		                                                    .getRates()
		                                                    .getRegularCostPerHour()
		                                                    .doubleValue()
		                                            + " and was >= than the expected rate of "
		                                            + rate,
		                                    "The regular cost per hour of the loaded Resource is not what was expected");
		                }
	                }
	            }
            }
        }
        catch (Exception e)
        {
            errorHandler.logOtherError("An error occured", e
                    .getMessage());
        }

        errorHandler.handleError();
    }

    public void testOperatorLessThanOrEqual()
    {
        final double rate = 100;
        StringBuffer xpath = new StringBuffer();

        xpath.append(XpathTesterUtils.FORWARD_SLASH)
        	.append("Resource")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("ResourceRates")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("RegularCostPerHour <= ")
        	.append(rate)
        	.append(XpathTesterUtils.CLOSE_BRACKET)
        	.append(XpathTesterUtils.CLOSE_BRACKET);

        try
        {
            RPMObject[] dbReturn = loadFromXpath(xpath.toString(), mainScope);
            if (dbReturn != null)
            {
	            for (int i = 0; i < dbReturn.length; i++)
	            {
	                if (((Resource)dbReturn[i]).getRates() != null)
	                {
		                if (!(((Resource) dbReturn[i]).getRates()
		                        .getRegularCostPerHour().doubleValue() <= rate))
		                {
		                    errorHandler
		                    		.logOtherError(
		                                    "The regular cost per hour of the loaded DB object was "
		                                            + ((Resource) dbReturn[i])
		                                                    .getRates()
		                                                    .getRegularCostPerHour()
		                                                    .doubleValue()
		                                            + " and was > than the expected rate of "
		                                            + rate,
		                                    "The regular cost per hour of the loaded Resource is not what was expected");
		                }
	                }
	            }
            }
        }
        catch (Exception e)
        {
            errorHandler.logOtherError("An error occured", e
                    .getMessage());
        }

        errorHandler.handleError();
    }

    public void testOperatorGreaterThan()
    {
        final double rate = 10;
        StringBuffer xpath = new StringBuffer();

        xpath.append(XpathTesterUtils.FORWARD_SLASH)
        	.append("Resource")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("ResourceRates")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("RegularCostPerHour > ")
        	.append(rate)
        	.append(XpathTesterUtils.CLOSE_BRACKET)
        	.append(XpathTesterUtils.CLOSE_BRACKET);

        try
        {
            RPMObject[] dbReturn = loadFromXpath(xpath.toString(), mainScope);

            if (dbReturn != null)
            {
	            for (int i = 0; i < dbReturn.length; i++)
	            {
	                if (((Resource)dbReturn[i]).getRates() != null)
	                {
		                if (!(((Resource) dbReturn[i]).getRates()
		                        .getRegularCostPerHour().doubleValue() > rate))
		                {
		                    errorHandler
		                            .logOtherError(
		                                    "The regular cost per hour of the loaded DB object was "
		                                            + ((Resource) dbReturn[i])
		                                                    .getRates()
		                                                    .getRegularCostPerHour()
		                                                    .doubleValue()
		                                            + " and was <= than the expected rate of "
		                                            + rate,
		                                    "The regular cost per hour of the loaded Resource is not what was expected");
		                }
	                }
	            }
            }
        }
        catch (Exception e)
        {
            errorHandler.logOtherError("An error occured", e
                    .getMessage());
        }

        errorHandler.handleError();
    }

    public void testOperatorGreaterThanOrEqual()
    {
        final double rate = 10;
        StringBuffer xpath = new StringBuffer();

        xpath.append(XpathTesterUtils.FORWARD_SLASH)
        	.append("Resource")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("ResourceRates")
        	.append(XpathTesterUtils.OPEN_BRACKET)
        	.append("RegularCostPerHour >= ")
        	.append(rate)
        	.append(XpathTesterUtils.CLOSE_BRACKET)
        	.append(XpathTesterUtils.CLOSE_BRACKET);

        try
        {
            RPMObject[] dbReturn = loadFromXpath(xpath.toString(), mainScope);

            if (dbReturn != null)
            {
	            for (int i = 0; i < dbReturn.length; i++)
	            {
	                if (((Resource)dbReturn[i]).getRates() != null)
	                {
		                if (!(((Resource) dbReturn[i]).getRates()
		                        .getRegularCostPerHour().doubleValue() >= rate))
		                {
		                    errorHandler
		                    		.logOtherError(
		                                    "The regular cost per hour of the loaded DB object was "
		                                            + ((Resource) dbReturn[i])
		                                                    .getRates()
		                                                    .getRegularCostPerHour()
		                                                    .doubleValue()
		                                            + " and was < than the expected rate of "
		                                            + rate,
		                                    "The regular cost per hour of the loaded Resource is not what was expected");
		                }
	                }
	            }
            }
        }
        catch (Exception e)
        {
            errorHandler.logOtherError("An error occured", e
                    .getMessage());
        }

        errorHandler.handleError();
    }
}