/*
 * Created on 15-Sep-2005
 */
package com.ibm.rpm.client.junit.framework.testcases.xpath.stats;

/**
 * @author njarazi
 */
public class StatsCounter
{
    /* SINGLETON CLASS */
    /** Instance of the singleton class */
    private static StatsCounter instance = new StatsCounter();
    
    /** Counter for errors */
    private int errorCount = 0;
    /** Counter for number of simple properties */
    private int simplePropertiesCount = 0;
    /** Counter for number of complex properties */
    private int complexPropertiesCount = 0;
    /** Counter for total number of properties */
    private int totalPropertiesCount = 0;
    
    private int depth = 1;

    private StatsCounter()
    { /* do nothing */
    }

    /**
     * Returns the instance of the counter
     * @return StatsCounter the singleton instance of the counter
     */
    public static StatsCounter getInstance()
    {
        return instance;
    }

    /*///////////////////////////////////////
     * Getters
     *///////////////////////////////////////
    /**
     * Returns the error count
     * @return int the number of errors
     */
    public int getErrorCount()
    {
        return errorCount;
    }
    
    /**
     * Returns the number of simple properties
     * @return int the number of simple properties
     */
    public int getSimplePropertiesCount()
    {
        return simplePropertiesCount;
    }
    
    /**
     * Returns the number of complex properties
     * @return int the number of complex properties
     */
    public int getComplexPropertiesCount()
    {
        return complexPropertiesCount;
    }
    
    /**
     * Returns the total number of properties
     * @return int the total number of properties
     */
    public int getTotalPropertiesCount()
    {
        return totalPropertiesCount;
    }
    
    /**
     * Returns the depth of the current comparison
     * @return int the depth of the current comparison
     */
    public int getCurrentDepth()
    {
        return depth;
    }
    
    /*///////////////////////////////////////
     * Incrementers
     *///////////////////////////////////////
    /**
     * Increments the number of errors by 1
     */
    public void incrementErrorCount()
    {
        errorCount++;
    }
    
    /**
     * Increments the number of simple properties by 1
     */
    public void incrementSimplePropertiesCount()
    {
        simplePropertiesCount++;
        incrementTotalPropertiesCount();
    }

    /**
     * Increments the number of complex properties by 1
     */
    public void incrementComplexPropertiesCount()
    {
        complexPropertiesCount++;
        incrementTotalPropertiesCount();
    }
    
    public void decrementComplexPropertiesCount()
    {
        complexPropertiesCount--;
        decrementTotalPropertiesCount();
    }
    
    /**
     * Increments the total number of properties by 1
     */
    private void incrementTotalPropertiesCount()
    {
        totalPropertiesCount++;
    }
    
    /**
     * Decrements the total number of properties by 1
     */
    private void decrementTotalPropertiesCount()
    {
        totalPropertiesCount--;
    }
    
    /**
     * Increments the search depth counter by one and returns the new value
     * @return the value of the depth counter
     */
    public int incrementDepth()
    {
        return depth++;
    }
    
    /**
     * Decrements the search depth counter by one and returns the new value
     * @return the value of the depth counter
     */
    public int decrementDepth()
    {
        return depth--;
    }

    /*///////////////////////////////////////
     * Resets
     *///////////////////////////////////////
    /**
     * Sets the number of simple properties counter to 0
     */
    public void resetSimplePropertiesCounter()
    {
        simplePropertiesCount = 0;
    }
    
    /**
     * Sets the number of complex properties counter to 0
     */
    public void resetComplexPropertiesCounter()
    {
        complexPropertiesCount = 0;
    }
    
    /**
     * Sets the total number of properties counter to 0
     */
    public void resetTotalPropertiesCounter()
    {
        totalPropertiesCount = 0;
    }
    
    /**
     * Sets the error counter to 0
     */
    public void resetErrorCounter()
    {
        errorCount = 0;
    }
    
    /**
     * Sets the depth counter to 0
     */
    public void resetDepth()
    {
        depth = 0;
    }
}
