/*
 * Created on Jul 13, 2005
 *
 */
package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;
import java.util.Comparator;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.containers.ResourceAttribute;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetency;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;

/**
 * @author karim
 */
public class CompetencyFixture extends RPMObjectFixture
{
	private static final int DEFAULT_NB_CLASSIFICATIONS = 1;
	private static final int DEFAULT_NB_COMPETENCIES = 1;
	private static final int DEFAULT_NB_PROFICIENCY_LEVELS = 3;
	
	protected static ProficiencyLevel[] availableProficiencyLevels = null;
	protected static ArrayList skipSettersForProficiencyLevel = new ArrayList();
	protected static ArrayList skipGettersForProficiencyLevel = new ArrayList();
    static
    {
    	//set skipSetters
    	skipSettersForProficiencyLevel.addAll(skipSetters);
    	skipSettersForProficiencyLevel.add("setName");

    	skipGettersForProficiencyLevel.addAll(skipGetters);
    	skipGettersForProficiencyLevel.add("getName");	
    }
	
    //creates a new structure
    public static ResourceAttributeCompetencyCategory createSaveFixture(ClientTestController ctc)throws Exception
	{
    	ResourceAttributeScope scope = createScope(DEFAULT_NB_CLASSIFICATIONS, DEFAULT_NB_COMPETENCIES, DEFAULT_NB_PROFICIENCY_LEVELS);
    	ResourceAttributeCompetencyCategory category = createFixture(DEFAULT_NB_CLASSIFICATIONS, DEFAULT_NB_COMPETENCIES, DEFAULT_NB_PROFICIENCY_LEVELS, scope, ctc);
       	category = (ResourceAttributeCompetencyCategory)save(ctc, category, scope);    		

    	return category;
    }
    
    public static ResourceAttributeCompetencyCategory createFixture(int nbClassifications, int nbCompetenciesByClass, int nbProficiencyLevelsByCompetency, ResourceAttributeScope scope, ClientTestController ctc)throws Exception
	{
    	if(availableProficiencyLevels==null){
    		initAvailableProficiencyLevels(ctc);
    	}
    	return createCategory(nbClassifications, nbCompetenciesByClass, nbProficiencyLevelsByCompetency, scope);
    }
    
    private static ResourceAttributeCompetencyCategory createCategory(int nbClassifications, int nbCompetenciesByClass, int nbProficiencyLevelsByCompetency, ResourceAttributeScope scope) throws Exception
    {
    	int index = random();
    	ResourceAttributeCompetencyCategory category = new ResourceAttributeCompetencyCategory();
    	HelperTestUtilities.setByIntrospection(category, index, skipSetters);
       
		if (scope != null && scope.getClassifications() != null) {
			if(nbClassifications>0){
		    	ResourceAttributeCompetencyClassification classification = null;
		    	ResourceAttributeCompetencyClassification[] classifications = new ResourceAttributeCompetencyClassification[nbClassifications];
		    	for(int i=0;i<nbClassifications;i++){
		    		classification = createClassification(nbCompetenciesByClass, nbProficiencyLevelsByCompetency, scope.getClassifications());
		    		classification.setParent(category);
		    		classifications[i] = classification;
		    	}
		    	category.setClassifications(classifications);	
			}
		}
		return category;
    }    
    
    private static ResourceAttributeCompetencyClassification createClassification(int nbCompetenciesByClass, int nbProficiencyLevelsByCompetency, ResourceAttributeScope scope) throws Exception
    {
    	int index = random();
    	ResourceAttributeCompetencyClassification classification = new ResourceAttributeCompetencyClassification();
    	HelperTestUtilities.setByIntrospection(classification, index, skipSetters);
       
		if (scope != null && scope.getAttributes() != null) {
			if(nbCompetenciesByClass>0){
		    	ResourceAttributeCompetency competency = null;
		    	ResourceAttributeCompetency[] competencies = new ResourceAttributeCompetency[nbCompetenciesByClass];
		    	for(int i=0;i<nbCompetenciesByClass;i++){
		    		competency = createCompetency(nbProficiencyLevelsByCompetency, scope.getAttributes());
		    		competency.setParent(classification);
		    		competencies[i] = competency;
		    	}
		    	classification.setAttributes(competencies);				
			}
		}
		return classification;
    }
    
    private static ResourceAttributeCompetency createCompetency(int nbProficiencyLevelsByCompetency, ResourceAttributeScope scope) throws Exception
    {
    	int index = random();
    	ResourceAttributeCompetency competency = new ResourceAttributeCompetency();
    	HelperTestUtilities.setByIntrospection(competency, index, skipSetters);
       
		if (scope != null && scope.isProficiencyLevels()) {
			if(nbProficiencyLevelsByCompetency>0){
				ResourceAttributeCompetencyProficiencyLevel competencyProficiencyLevel = null;
		    	ResourceAttributeCompetencyProficiencyLevel[] competencyProficiencyLevels = new ResourceAttributeCompetencyProficiencyLevel[nbProficiencyLevelsByCompetency];
		    	ProficiencyLevel proficiencyLevelDatafield;
		    	for(int i=0;i<nbProficiencyLevelsByCompetency;i++){
		    	    proficiencyLevelDatafield = availableProficiencyLevels[i%availableProficiencyLevels.length];
		    		competencyProficiencyLevel = createProficiencyLevel(proficiencyLevelDatafield);
		    		competencyProficiencyLevel.setParent(competency);
		    		competencyProficiencyLevels[i] = competencyProficiencyLevel;
		    	}
		    	competency.setProficiencyLevels(competencyProficiencyLevels);	
			}
		}
		return competency;
    }    
    
    private static ResourceAttributeCompetencyProficiencyLevel createProficiencyLevel(ProficiencyLevel proficiencyLevelDatafield) throws Exception
    {
    	int index = random();
    	//create rates
    	ResourceRates rates = new ResourceRates();
    	HelperTestUtilities.setByIntrospection(rates, index, skipSettersForProficiencyLevel);
    	
    	ResourceAttributeCompetencyProficiencyLevel competencyProficiencyLevel = new ResourceAttributeCompetencyProficiencyLevel();
    	HelperTestUtilities.setByIntrospection(competencyProficiencyLevel, index, skipSettersForProficiencyLevel);
    	
    	competencyProficiencyLevel.setProficiencyLevel(proficiencyLevelDatafield);
    	competencyProficiencyLevel.setRates(rates);
    	rates.setParent(competencyProficiencyLevel);
    	
		return competencyProficiencyLevel;
    }    
    
    
    
    
    //updates the names and rates
    public static void updateCategoryFixture(ResourceAttributeCompetencyCategory category, ResourceAttributeScope scope)throws Exception{
    	updateCategory(category, scope, random());
    }
    public static void updateClassificationFixture(ResourceAttributeCompetencyClassification classification, ResourceAttributeScope scope)throws Exception{
    	updateClassification(classification, scope, random());
    }
    public static void updateCompetencyFixture(ResourceAttributeCompetency competency, ResourceAttributeScope scope)throws Exception{
    	updateCompetency(competency, scope, random());
    }
    public static void updateProficiencyLevelFixture(ResourceAttributeCompetencyProficiencyLevel competencyProficiencyLevel)throws Exception{
    	updateProficiencyLevel(competencyProficiencyLevel, random());
    }
    
    private static void updateCategory(ResourceAttributeCompetencyCategory category, ResourceAttributeScope scope, int index) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(category, index, skipSetters);
		if (scope != null && scope.getClassifications() != null) {
			for(int i=0;i<category.getClassifications().length;i++){
				updateClassificationFixture(category.getClassifications()[i], scope.getClassifications());
			}
		}
    }    
    
    private static void updateClassification(ResourceAttributeCompetencyClassification classification, ResourceAttributeScope scope, int index) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(classification, index, skipSetters);
       
		if (scope != null && scope.getAttributes() != null) {
			for(int i=0;i<classification.getAttributes().length;i++){
				updateCompetencyFixture(classification.getAttributes()[i], scope.getAttributes());
			}
		}
    }
    
    private static void updateCompetency(ResourceAttributeCompetency competency, ResourceAttributeScope scope, int index) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(competency, index, skipSetters);
       
		if (scope != null && scope.isProficiencyLevels()) {
			for(int i=0;i<competency.getProficiencyLevels().length;i++){
				updateProficiencyLevelFixture(competency.getProficiencyLevels()[i]);
			}
		}
    }    
    
    private static void updateProficiencyLevel(ResourceAttributeCompetencyProficiencyLevel competencyProficiencyLevel, int index) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(competencyProficiencyLevel, index, skipSettersForProficiencyLevel);
    	HelperTestUtilities.setByIntrospection(competencyProficiencyLevel.getRates(), index, skipSettersForProficiencyLevel);
    }    
    
    
    
    
    //updates the structure add/remove/element
    public static void updateCategoryFixture(ResourceAttributeCompetencyCategory category, int deltaNbClassifications, int deltaNbCompetenciesByClass, int deltaNbProficiencyLevelsByCompetency, ResourceAttributeScope scope)throws Exception{
    	updateCategory(category, deltaNbClassifications, deltaNbCompetenciesByClass, deltaNbProficiencyLevelsByCompetency, scope);
    }
    public static void updateClassificationFixture(ResourceAttributeCompetencyClassification classification, int deltaNbCompetenciesByClass, int deltaNbProficiencyLevelsByCompetency, ResourceAttributeScope scope)throws Exception{
    	updateClassification(classification, deltaNbCompetenciesByClass, deltaNbProficiencyLevelsByCompetency, scope);
    }
    public static void updateCompetencyFixture(ResourceAttributeCompetency competency, int deltaNbProficiencyLevelsByCompetency, ResourceAttributeScope scope)throws Exception{
    	updateCompetency(competency, deltaNbProficiencyLevelsByCompetency, scope);
    }

    private static void updateCategory(ResourceAttributeCompetencyCategory category, int deltaNbClassifications, int deltaNbCompetenciesByClass, int deltaNbProficiencyLevelsByCompetency, ResourceAttributeScope scope)throws Exception{
		if (scope != null && scope.getClassifications() != null) {
			if(deltaNbClassifications>0){//add new elements
		    	ResourceAttributeCompetencyClassification newClassification = null;
		    	ResourceAttributeCompetencyClassification[] newClassifications = new ResourceAttributeCompetencyClassification[deltaNbClassifications];
		    	for(int i=0;i<deltaNbClassifications;i++){
		    		newClassification = createClassification(deltaNbCompetenciesByClass, deltaNbProficiencyLevelsByCompetency, scope.getClassifications());
		    		newClassification.setParent(category);
		    		newClassifications[i] = newClassification;
		    	}
		    	
		    	ArrayList temp = mergeArrays(category.getClassifications(), newClassifications);
		    	ResourceAttributeCompetencyClassification[] totalClassifications = new ResourceAttributeCompetencyClassification[temp.size()];
		    	temp.toArray(totalClassifications);
		    	
		    	category.setClassifications(totalClassifications);		    	
			}
			for(int i=0;i<category.getClassifications().length;i++){
				updateClassificationFixture(category.getClassifications()[i], deltaNbCompetenciesByClass, deltaNbProficiencyLevelsByCompetency, scope.getClassifications());
			}
		}
    }
    private static void updateClassification(ResourceAttributeCompetencyClassification classification, int deltaNbCompetenciesByClass, int deltaNbProficiencyLevelsByCompetency, ResourceAttributeScope scope)throws Exception{
		if (scope != null && scope.getAttributes() != null) {
			if(deltaNbCompetenciesByClass>0){//add new elements
		    	ResourceAttributeCompetency newCompetency = null;
		    	ResourceAttributeCompetency[] newCompetencies = new ResourceAttributeCompetency[deltaNbCompetenciesByClass];
		    	for(int i=0;i<deltaNbCompetenciesByClass;i++){
		    		newCompetency = createCompetency(deltaNbProficiencyLevelsByCompetency, scope.getAttributes());
		    		newCompetency.setParent(classification);
		    		newCompetencies[i] = newCompetency;
		    	}
		    	
		    	ArrayList temp = mergeArrays(classification.getAttributes(), newCompetencies);
		    	ResourceAttributeCompetency[] totalCompetencies = new ResourceAttributeCompetency[temp.size()];
		    	temp.toArray(totalCompetencies);
		    	
		    	classification.setAttributes(totalCompetencies);		    	
			}
			for(int i=0;i<classification.getAttributes().length;i++){
				updateCompetencyFixture(classification.getAttributes()[i], deltaNbProficiencyLevelsByCompetency, scope.getAttributes());
			}
		}
    }
    private static void updateCompetency(ResourceAttributeCompetency competency, int deltaNbProficiencyLevelsByCompetency, ResourceAttributeScope scope)throws Exception{
		if (scope != null && scope.isProficiencyLevels()) {
			if(deltaNbProficiencyLevelsByCompetency>0){//add new elements
			    int currentNbProficiencyLevels = (competency.getProficiencyLevels()!=null)?competency.getProficiencyLevels().length:0;
			    int nbPossibleAdditionalProficiencyLevels = getPossibleAdditionalProficiencyLevels(currentNbProficiencyLevels, deltaNbProficiencyLevelsByCompetency);
		    	ResourceAttributeCompetencyProficiencyLevel newProficiencyLevel = null;
		    	ResourceAttributeCompetencyProficiencyLevel[] newProficiencyLevels = new ResourceAttributeCompetencyProficiencyLevel[nbPossibleAdditionalProficiencyLevels];
		    	ProficiencyLevel proficiencyLevelDatafield;
		    	for(int i=0;i<nbPossibleAdditionalProficiencyLevels;i++){
		    	    proficiencyLevelDatafield = availableProficiencyLevels[(currentNbProficiencyLevels+i)%availableProficiencyLevels.length];
		    		newProficiencyLevel = createProficiencyLevel(proficiencyLevelDatafield);
		    		newProficiencyLevel.setParent(competency);
		    		newProficiencyLevels[i] = newProficiencyLevel;
		    	}
		    	
		    	ArrayList temp = mergeArrays(competency.getProficiencyLevels(), newProficiencyLevels);
		    	ResourceAttributeCompetencyProficiencyLevel[] totalProficiencyLevels = new ResourceAttributeCompetencyProficiencyLevel[temp.size()];
		    	temp.toArray(totalProficiencyLevels);
		    	
		    	competency.setProficiencyLevels(totalProficiencyLevels);		    	
			}
		}
    }
    
    private static int getPossibleAdditionalProficiencyLevels(int currentNbProficiencyLevels, int deltaNbProficiencyLevelsByCompetency){
        if(currentNbProficiencyLevels + deltaNbProficiencyLevelsByCompetency > DatafieldFixture.PROFICIENCYLEVELS.length){
            return DatafieldFixture.PROFICIENCYLEVELS.length - currentNbProficiencyLevels;
        } else {
            return deltaNbProficiencyLevelsByCompetency;
        }
    }

    //Compare section
	public static void compare(ResourceAttribute[] array1, ResourceAttribute[] array2,
			ResourceAttributeScope scope) throws AssertionFailedError, Exception{

    	sortWithComparator(array1, new RPMObjectComparator());
    	sortWithComparator(array2, new RPMObjectComparator()); 
		
		RPMObjectFixture.compare(array1, array2);

		for (int i = 0; i < array1.length; i++) {
			CompetencyFixture.compare(array1[i], array2[i], scope);
		}
	}
	
    public static void compare(ResourceAttribute obj1, ResourceAttribute obj2, ResourceAttributeScope scope)
	throws AssertionFailedError, Exception {
        
        if (obj1 instanceof ResourceAttributeCompetencyProficiencyLevel)
        {
            RPMObjectFixture.compareWithSkip(obj1, obj2, skipGettersForProficiencyLevel);
            compareCompetencyProficiencyLevel(obj1, obj2, scope);
        }
        else
        {
            RPMObjectFixture.compare(obj1, obj2);
            if(obj1 instanceof ResourceAttributeCompetencyCategory){
        		compareCompetencyCategory(obj1, obj2, scope);
        	} else if(obj1 instanceof ResourceAttributeCompetencyClassification){
        		compareCompetencyClassification(obj1, obj2, scope);
        	} else if(obj1 instanceof ResourceAttributeCompetency){
        		compareCompetency(obj1, obj2, scope);
        	} else if(obj1 instanceof ResourceAttributeCompetencyProficiencyLevel){
        		
        	} else {
        		Assert.fail("ResourceAttributeFixture.compare : class "+obj1.getClass().getName()+" not supported.");
        	}
        } 
    }
	
    public static void compareCompetencyCategory(ResourceAttribute obj1, ResourceAttribute obj2,
			ResourceAttributeScope scope) throws AssertionFailedError,
			Exception {

    	ResourceAttributeCompetencyCategory category1 = (ResourceAttributeCompetencyCategory)obj1;
    	ResourceAttributeCompetencyCategory category2 = (ResourceAttributeCompetencyCategory)obj2;

    	//compare children
		if (scope != null && scope.getClassifications() != null) {
			CompetencyFixture.compare(category1.getClassifications(), category2.getClassifications(), scope.getClassifications());
		}

	}
    
    public static void compareCompetencyClassification(ResourceAttribute obj1, ResourceAttribute obj2,
			ResourceAttributeScope scope) throws AssertionFailedError,
			Exception {

    	ResourceAttributeCompetencyClassification classification1 = (ResourceAttributeCompetencyClassification)obj1;
    	ResourceAttributeCompetencyClassification classification2 = (ResourceAttributeCompetencyClassification)obj2;

    	//compare children
		if (scope != null && scope.getAttributes() != null) {
			CompetencyFixture.compare(classification1.getAttributes(), classification2.getAttributes(), scope.getAttributes());
		}

	}

    public static void compareCompetency(ResourceAttribute obj1, ResourceAttribute obj2,
			ResourceAttributeScope scope) throws AssertionFailedError,
			Exception {

    	ResourceAttributeCompetency competency1 = (ResourceAttributeCompetency)obj1;
    	ResourceAttributeCompetency competency2 = (ResourceAttributeCompetency)obj2;

    	//compare children
		if (scope != null && scope.isProficiencyLevels()) {
			CompetencyFixture.compare(competency1.getProficiencyLevels(), competency2.getProficiencyLevels(), null);
		}

	}
    
    public static void compareCompetencyProficiencyLevel(ResourceAttribute obj1, ResourceAttribute obj2,
			ResourceAttributeScope scope) throws AssertionFailedError,
			Exception {

    	ResourceAttributeCompetencyProficiencyLevel proficiencyLevel1 = (ResourceAttributeCompetencyProficiencyLevel)obj1;
    	ResourceAttributeCompetencyProficiencyLevel proficiencyLevel2 = (ResourceAttributeCompetencyProficiencyLevel)obj2;

    	//create and use ResourceRatesFixture class
		CompetencyFixture.compareRates(proficiencyLevel1.getRates(), proficiencyLevel2.getRates(), null);
	}
    
    public static void compareRates(ResourceRates obj1, ResourceRates obj2,
			ResourceAttributeScope scope) throws AssertionFailedError,
			Exception {

    	//here we don't call the RPMObjectFixture.compare because it compares the IDs
    	//and in this case, the SP generates a new ID for the updated rates.
        Assert.assertNotNull(obj1);
        Assert.assertNotNull(obj2);
        HelperTestUtilities.assertEqualsByIntrospection(obj1, obj2, skipGetters);
	}
    
    
    private static void initAvailableProficiencyLevels(ClientTestController ctc)throws Exception{
		//set possible proficiency level name values
		int nbProficiencyLevels = DatafieldFixture.PROFICIENCYLEVELS.length;
		availableProficiencyLevels = new ProficiencyLevel[nbProficiencyLevels];
		for(int i=0;i<nbProficiencyLevels;i++){
		    availableProficiencyLevels[i] = createAvailableProficiencyLevel(DatafieldFixture.PROFICIENCYLEVELS[i],ctc);
		}
    }
    
    private static ProficiencyLevel createAvailableProficiencyLevel(String name, ClientTestController ctc)throws Exception{
    	ProficiencyLevel proficiencyLevel = DatafieldFixture.createProficiencyLevel(name);
    	proficiencyLevel = (ProficiencyLevel)ctc.loadFromID(proficiencyLevel, null);
    	return proficiencyLevel;
    }
    
    private static ArrayList mergeArrays(Object[] array1, Object[] array2){
    	ArrayList temp = new ArrayList();
    	addArray(temp, array1);
    	addArray(temp, array2);
    	return temp;
    }
    
   private static void addArray(ArrayList result, Object[] array){
    	if(array!=null && array.length!=0){
    		for(int i=0;i<array.length;i++){
    			result.add(array[i]);
    		}
    	}    	
    }
        
    public static ResourceAttributeScope createScope(int nbClassifications, int nbCompetenciesByClass, int nbProficiencyLevelsByCompetency){
    	ResourceAttributeScope scope = new ResourceAttributeScope();
    	if(nbClassifications>0){
    		ResourceAttributeScope classificationScope = new ResourceAttributeScope();
    		if(nbCompetenciesByClass>0){
        		ResourceAttributeScope competencyScope = new ResourceAttributeScope();
        		if(nbProficiencyLevelsByCompetency>0){
        			competencyScope.setProficiencyLevels(true);
        		}
        		competencyScope.setParent(classificationScope);
        		classificationScope.setAttributes(competencyScope);
    		}
    		classificationScope.setParent(scope);
    		scope.setClassifications(classificationScope);
    	}
    	return scope;
    }

    protected static class RPMObjectComparator implements Comparator {

        public RPMObjectComparator() {
            super();
        }
        
        public int compare(Object o1, Object o2){ 
            
            ResourceAttribute obj1 = (ResourceAttribute)o1;
            ResourceAttribute obj2 = (ResourceAttribute)o2;
            
            String key1 = null;
            String key2 = null;           
            
            if(obj1 != null && obj1.getID() != null && obj2 != null && obj2.getID() != null) {
                key1 = obj1.getID();
                key2 = obj2.getID();
            }            

            int compare = compareStrings(key1, key2);
            
            if(compare == 0){
                if(obj1 != null){
                    key1 = obj1.getName();
                }
                if(obj2 != null){
                    key2 = obj2.getName();
                }                
                
                compare = compareStrings(key1, key2);
            }
            
            if(compare == 0){
                if(obj1 != null){
                    key1 = obj1.getID();
                }
                if(obj2 != null){
                    key2 = obj2.getID();
                }                
                
                compare = compareStrings(key1, key2);
            }
            
            return compare;
        }
        
        private int compareStrings(String key1, String key2){
            if (key1 == null && key2 == null) {
                return 0;
            }
            if (key1 == null && key2 != null) {
                return -1;
            }
            if (key1 != null && key2 == null) {
                return 1;
            }
            return key1.compareTo(key2);
        }    
    }
}
