/*
 * Created on Sep 27, 2005
 */
package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.PoolDomainFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolReference;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.PoolReferenceScope;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author gosselin
 */
public class ComplexPoolFixture extends RPMObjectFixture
{
    protected static ArrayList poolSkipSetters, poolSkipGetters;

    static
    {
        //Setters
        poolSkipSetters = new ArrayList();
        poolSkipSetters.addAll(skipSetters);

        poolSkipGetters = new ArrayList();
        poolSkipGetters.addAll(skipGetters);
    }

    /**
     * @param createParent
     * @param nbChildren
     * @param nbLayers
     * @param scope
     * @throws Exception
     */
    public static Pool createFixture(boolean createParent, int nbPools, 
            int nbResources, ResourceScope resourceScope, int nbChildren,
            PoolScope scope, ClientTestController ctc) throws Exception
    {
        Pool mainPool = new Pool();
        HelperTestUtilities.setByIntrospection(mainPool, new Double(Math
                .random() * 100).intValue(), poolSkipSetters);

        mainPool.setAdministrativeTasks(null);
        mainPool.setClientCostCenter(null);
        mainPool.setSecurityGroup(null);
        //mainPool.setStatusUpdate(null);

        if (scope != null)
        {

            if (scope.getParent() != null && createParent)
            {
                Pool parentPool = PoolFixture.createFixture(false, 0, null, ctc);
                parentPool.setPools(new Pool[] { mainPool });
                mainPool.setParent(parentPool);
            }

            if (scope.getPools() != null)
            {

                Pool[] childrenPool = new Pool[nbPools];
                for (int i = 0; i < nbPools; i++)
                {
                    Pool p = PoolFixture.createFixture(false, nbChildren, scope
                            .getPools(), ctc);
                    
                    Resource[] rArray = new Resource[nbResources];
                    for (int j = 0; j < nbResources; j++)
                    {
                        Resource r = ResourceFixture.createFixture(p,
                                resourceScope, ctc);
                        rArray[j] = r;
                    }
                    p.setResources(rArray);
                    
                    p.setParent(mainPool);
                    childrenPool[i] = p;
                }
                mainPool.setPools(childrenPool);
            }

            if (scope.getResources() != null)
            {
                Resource[] rArray = new Resource[nbResources];
                
                ResourceScope rScope = resourceScope;
                rScope.setCalendar(null);
                RPMCalendar cal = new RPMCalendar();
                cal.setID("DEFAULT_CALENDAR");
                
                for (int i = 0; i < nbResources; i++)
                {
                    Resource r = ResourceFixture.createFixture(mainPool,
                            rScope, ctc);
                    r.setCalendar(cal);
                    rArray[i] = r;
                }
                mainPool.setResources(rArray);
            }

            if (scope.getDomain() != null)
            {
                mainPool.setDomain(PoolDomainFixture.createSaveFixture(
                        nbChildren, ctc));
            }

            if (scope.getPoolReferences() != null)
            {
                PoolReferenceScope rScope = new PoolReferenceScope();

                PoolReference[] rArray = new PoolReference[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    PoolReference r = PoolReferenceFixture.createFixture(mainPool,
                            rScope);
                    r.setParent(mainPool);
                    rArray[i] = r;
                }

                //Insert a null object
                rArray[nbChildren] = null;

                mainPool.setPoolReferences(rArray);
            }


        }
        return mainPool;
    }

    public static void compare(Pool p1, Pool p2, PoolScope scope)
            throws AssertionFailedError, Exception
    {
        throw new AssertionFailedError("no comparison implemented for ComplexPoolFixture");
    }

    public static void compare(Pool[] pArray1, Pool[] pArray2, PoolScope scope)
            throws AssertionFailedError, Exception
    {
        throw new AssertionFailedError("no comparison implemented for ComplexPoolFixture");
    }
}
