package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Comparator;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.containers.EmploymentHistory;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author lygagnon
 *  
 */
public class EmploymentHistoryFixture extends RPMObjectFixture 
{
    
    protected static ArrayList employmentHistorySkipSetters, employmentHistorySkipGetters;
    
    static
    {
        employmentHistorySkipSetters = new ArrayList();
        employmentHistorySkipSetters.addAll(skipSetters);
        employmentHistorySkipSetters.add("setName");
        
        employmentHistorySkipGetters = new ArrayList();
        employmentHistorySkipGetters.addAll(skipGetters);
        employmentHistorySkipGetters.add("getName");
    }    
    public static EmploymentHistory createFixture(Resource resource)
            throws Exception 
    {
        EmploymentHistory employment = new EmploymentHistory();
        HelperTestUtilities.setByIntrospection(employment, new Double(
                Math.random() * 100).intValue(), employmentHistorySkipSetters);
        Calendar calendar = Calendar.getInstance();
        calendar.set(2006, 2, 8);
        employment.setStartDate(calendar);
        calendar.set(2006, 2, 9);
        employment.setEndDate(calendar);
        employment.setParent(resource);
        return employment;
    }

    public static void compare(EmploymentHistory e1, EmploymentHistory e2)
            throws AssertionFailedError, Exception 
    {
        RPMObjectFixture.compareWithSkip(e1, e2, employmentHistorySkipGetters);
        ResourceFixture.compareWithoutCanProperties((Resource) e1.getParent(), (Resource) e2.getParent(), null);
    }

    public static void compare(EmploymentHistory[] eArray1,
            EmploymentHistory[] eArray2) throws AssertionFailedError, Exception 
    {
    	sortWithComparator(eArray1, new RPMObjectComparator());
    	sortWithComparator(eArray2, new RPMObjectComparator()); 
        
        RPMObjectFixture.compare(eArray1, eArray2);
        if (eArray1 == null)
            return;

        for (int i = 0; i < eArray1.length; i++) {
            compare(eArray1[i], eArray2[i]);
        }
    }
    
    protected static class RPMObjectComparator implements Comparator {
        /**
         * 
         */
        public RPMObjectComparator() {
            super();
        }
        
        public int compare(Object o1, Object o2){ 
            
            EmploymentHistory obj1 = (EmploymentHistory)o1;
            EmploymentHistory obj2 = (EmploymentHistory)o2;
            
            String key1 = null;
            String key2 = null;
            
            if(obj1 != null){
                key1 = obj1.getCompany();
            }
            if(obj2 != null){
                key2 = obj2.getCompany();
            }                
            
            int compare = compareStrings(key1, key2);
            
            if(compare == 0){
                if(obj1 != null){
                    key1 = obj1.getTitle();
                }
                if(obj2 != null){
                    key2 = obj2.getTitle();
                }                
                
                compare = compareStrings(key1, key2);
            }
            
            if(compare == 0){
                if(obj1 != null){
                    key1 = obj1.getID();
                }
                if(obj2 != null){
                    key2 = obj2.getID();
                }                
                
                compare = compareStrings(key1, key2);
            }
            
            return compare;
        }
        
        private int compareStrings(String key1, String key2){
            if (key1 == null && key2 == null) {
                return 0;
            }
            if (key1 == null && key2 != null) {
                return -1;
            }
            if (key1 != null && key2 == null) {
                return 1;
            }
            return key1.compareTo(key2);
        }
    }
}