package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;
import java.util.Comparator;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Language;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.LanguageProficiency;
import com.ibm.rpm.resource.containers.Resource;
/**
 * @author lygagnon
 *  
 */
public class LanguageProficiencyFixture extends RPMObjectFixture 
{

    protected static ArrayList languageProficiencySkipSetters, languageProficiencySkipGetters;
    
    static
    {
        languageProficiencySkipSetters = new ArrayList();
        languageProficiencySkipSetters.addAll(skipSetters);
        languageProficiencySkipSetters.add("setName");
        languageProficiencySkipSetters.add("setReadLevel");
        languageProficiencySkipSetters.add("setSpeakLevel");
        languageProficiencySkipSetters.add("setWriteLevel");
        
        languageProficiencySkipGetters = new ArrayList();
        languageProficiencySkipGetters.addAll(skipGetters);
        languageProficiencySkipGetters.add("getName");
    }      
    public static LanguageProficiency createFixture(Resource resource) throws Exception 
    {
        LanguageProficiency lp = new LanguageProficiency();
        HelperTestUtilities.setByIntrospection(lp, new Double(Math.random() * 100).intValue(),languageProficiencySkipSetters);
        
        //Add resource && resourceID
        lp.setParent(resource);
        
        //Creates languages
        Language lang = DatafieldFixture.createRandomLanguage();
        lang.setParent(lp);
        lp.setLanguage(lang);
        
        double id = Math.random() * 6;
        
        //Set a random valid read level
        id = ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN + (Math.random() * 
                (ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX - 
                ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN));
        lp.setReadLevel(new Integer(Math.round((float)id)));
        
        //Set a random valid speak level
        id = ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN + (Math.random() * 
                (ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX - 
                ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN));
        lp.setSpeakLevel(new Integer(Math.round((float)id)));
        
        //Set a random valid write level
        id = ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN + (Math.random() * 
                (ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MAX - 
                ValidationConstants.LANGUAGEPROFICIENCY_LEVEL_MIN));
        lp.setWriteLevel(new Integer(Math.round((float)id)));
        
        return lp;
    }

    public static void compare(LanguageProficiency l1, LanguageProficiency l2)
            throws AssertionFailedError, Exception {
        
        RPMObjectFixture.compareWithSkip(l1, l2, languageProficiencySkipGetters);

        DatafieldFixture.compare(l1.getLanguage(), l2.getLanguage());

        ResourceFixture.compareWithoutCanProperties((Resource) l1.getParent(), (Resource) l2
                .getParent(), null);

    }

    public static void compare(LanguageProficiency[] lArray1,
            LanguageProficiency[] lArray2) throws AssertionFailedError,
            Exception 
   {
    	sortWithComparator(lArray1, new RPMObjectComparator());
    	sortWithComparator(lArray2, new RPMObjectComparator()); 
        
        RPMObjectFixture.compare(lArray1, lArray2);
        if (lArray1 == null)
            return;

        for (int i = 0; i < lArray1.length; i++) {
            compare(lArray1[i], lArray2[i]);
        }
    }
    
    protected static class RPMObjectComparator implements Comparator {
        /**
         * 
         */
        public RPMObjectComparator() {
            super();
        }
        
        public int compare(Object o1, Object o2){ 
            
            LanguageProficiency obj1 = (LanguageProficiency)o1;
            LanguageProficiency obj2 = (LanguageProficiency)o2;
            
            String key1 = null;
            String key2 = null;
                   
            if(obj1 != null){
                if(obj1.getLanguage()!=null){
                    key1 = obj1.getLanguage().getID();                    
                }
            }
            if(obj2 != null){
                if(obj2.getLanguage()!=null){
                    key2 = obj2.getLanguage().getID();                    
                }
            }                
            
            int compare = compareStrings(key1, key2);
                      
            if(compare == 0){
                compare = obj1.getReadLevel().compareTo(obj2.getReadLevel());
            }
            
            if(compare == 0){
                compare = obj1.getSpeakLevel().compareTo(obj2.getSpeakLevel());
            }
            
            if(compare == 0){
                compare = obj1.getWriteLevel().compareTo(obj2.getWriteLevel());
            }
            
            return compare;
        }
        
        private int compareStrings(String key1, String key2){
            if (key1 == null && key2 == null) {
                return 0;
            }
            if (key1 == null && key2 != null) {
                return -1;
            }
            if (key1 != null && key2 == null) {
                return 1;
            }
            return key1.compareTo(key2);
        }
    }
}