package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.CitizenshipCountryCode;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.LocationPreferences;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author lygagnon
 */
public class LocationPreferencesFixture extends RPMObjectFixture

{
    protected static ArrayList assignmentSkipSetters, assignmentSkipGetters;
    
    static
    {
        assignmentSkipSetters = new ArrayList();
        assignmentSkipSetters.addAll(skipSetters);
        assignmentSkipSetters.add("setName");
        assignmentSkipSetters.add("setAcceptableTravelDuration");
        assignmentSkipSetters.add("setAcceptableRelocateDuration");
        
        assignmentSkipGetters = new ArrayList();
        assignmentSkipGetters.addAll(skipGetters);
        assignmentSkipGetters.add("getName");
        assignmentSkipGetters.add("getAcceptableTravelDuration");
        assignmentSkipGetters.add("getAcceptableRelocateDuration");
    } 

    public static LocationPreferences createFixture(Resource resource) throws Exception
    {
        int nbRelocateChildren = (new Double(Math.random() * 
                (ValidationConstants.LOCATIONPREFERENCES_RELOCATELOCATIONS_MAX+1))).intValue();
        int nbTravelChildren = (new Double(Math.random() * 
                (ValidationConstants.LOCATIONPREFERENCES_TRAVELLOCATIONS_MAX+1))).intValue();
        return createFixture(resource, nbRelocateChildren, nbTravelChildren);
    }
    
    public static LocationPreferences createFixture(Resource resource, int nbRelocateChildren, int nbTravelChildren) throws Exception
    {
        LocationPreferences locationPref = new LocationPreferences();

        CitizenshipCountryCode[] relocateArray = null;
        CitizenshipCountryCode[] travelArray = null;

        locationPref.setParent(resource);

        HelperTestUtilities.setByIntrospection(locationPref, new Double(
        Math.random() * 100).intValue(), assignmentSkipSetters);

        // Creates relocateLocations
        if (nbRelocateChildren > 0)
        {
            locationPref.setWillingtoRelocate(Boolean.TRUE);
            //TODO make it random later
            locationPref.setAcceptableRelocateDuration(new Integer(ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLERELOCATEDURATION_MIN));
            relocateArray = new CitizenshipCountryCode[nbRelocateChildren];
        } 
        else
        {
            locationPref.setWillingtoRelocate(Boolean.FALSE);
        }
        
        for (int i = 0; i < nbRelocateChildren; i++)
        {            
            CitizenshipCountryCode relocate = DatafieldFixture
                    .createRandomCitizenshipCountryCode();
            relocate.setParent(locationPref);
            relocateArray[i] = relocate;
        }
        
        locationPref.setRelocateLocations(relocateArray);
        
        // Creates travelLocations
        if (nbTravelChildren > 0)
        {
            locationPref.setWillingtoTravel(Boolean.TRUE);
            //TODO make it random later
            locationPref.setAcceptableTravelDuration(new Integer(ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLETRAVELDURATION_MIN));
            travelArray = new CitizenshipCountryCode[nbTravelChildren];
        } 
        else
        {
            locationPref.setWillingtoTravel(Boolean.FALSE);
        }

        for (int i = 0; i < nbTravelChildren; i++)
        {
            CitizenshipCountryCode travel = DatafieldFixture
                    .createRandomCitizenshipCountryCode();
            travel.setParent(locationPref);
            travelArray[i] = travel;
        }
        
        locationPref.setTravelLocations(travelArray);

        return locationPref;

    }

    public static void compare(LocationPreferences l1, LocationPreferences l2)

    throws AssertionFailedError, Exception

    {

        RPMObjectFixture.compareWithSkip(l1, l2, assignmentSkipGetters);

        // Compare Datafields relocateLocations

        DatafieldFixture.compare(l1.getRelocateLocations(), l2

        .getRelocateLocations());

        // Compare Datafields travelLocations

        DatafieldFixture.compare(l1.getTravelLocations(), l2

        .getTravelLocations());

    }

    public static void compare(LocationPreferences[] lArray1,

    LocationPreferences[] lArray2) throws AssertionFailedError,

    Exception

    {

        RPMObjectFixture.compare(lArray1, lArray2);

        if (lArray1 == null)

            return;

        for (int i = 0; i < lArray1.length; i++)

        {

            compare(lArray1[i], lArray2[i]);

        }

    }

}