/*
 * Created on Jun 28, 2005
 */
package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;
import java.util.Arrays;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.PoolDomainFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.timesheet.fixtures.AdministrativeTaskFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultAdministrativeTaskFixture;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolReference;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.resource.scope.PoolReferenceScope;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.containers.DefaultAdministrativeTask;


/**
 * @author gosselin
 */
public class PoolFixture extends RPMObjectFixture
{    
    protected static Pool STATIC_POOL_FIXTURE = null;

    protected static String XPATH_POOL_NAME = "STATIC_POOL_FIXTURE";
    
    protected static ArrayList poolSkipSetters, poolSkipGetters;
    
    static
    {
               
        //Setters
        poolSkipSetters = new ArrayList();
        poolSkipSetters.addAll(skipSetters);
        
        poolSkipGetters = new ArrayList();
        poolSkipGetters.addAll(skipGetters);


    }    
    
    public static Pool retrieveStaticFixture(ClientTestController ctc)
            throws Exception
    {
        if(STATIC_POOL_FIXTURE == null)
        {
            STATIC_POOL_FIXTURE = loadFixture(ctc, XPATH_POOL_NAME);
        }
        return STATIC_POOL_FIXTURE;
    }
    
    public static Pool loadFixture(ClientTestController ctc, String xpathName)
            throws Exception
    {
        RPMObject[] rpmObjects = null;
        Pool pool = null;
        String xpathQuery = "/Pool[name='" + xpathName + "']";
        rpmObjects = ctc.loadFromXpath(xpathQuery, new PoolScope());
        if(rpmObjects != null && rpmObjects.length > 0 && rpmObjects[0] != null)
        {
            pool = (Pool)rpmObjects[0];
        }
        else
        {
            pool = createFixture(false, 0, null, ctc);
            pool.setName(XPATH_POOL_NAME);
            pool = (Pool)saveWithoutDelete(ctc, pool, null);
        }
        return pool;
    }
    
    /**
     * @param createParent
     * @param nbChildren
     * @param nbLayers
     * @param scope
     * @throws Exception
     */
    public static Pool createFixture(boolean createParent, int nbChildren,
            PoolScope scope, ClientTestController ctc) throws Exception
    {
        Pool mainPool = new Pool();
        HelperTestUtilities.setByIntrospection(mainPool, new Double(Math.random() * 100)
                .intValue(), poolSkipSetters);

        
        mainPool.setClientCostCenter(null);
        mainPool.setSecurityGroup(null);
        //mainPool.setStatusUpdate(null);
      
        
        if(scope != null){
            
            if (scope.getParent() != null && createParent) {
                //Pool parentPool = createFixture(false, 0,null,ctc);
                Pool parentPool = createSaveFixture(ctc);
                parentPool.setPools(new Pool[] { mainPool });
                mainPool.setParent(parentPool);
            }
            
	        if (scope.getPools() != null)
	        {
	
	            Pool[] childrenPool = new Pool[nbChildren + 1];
	            for (int i = 0; i < nbChildren; i++)
	            {
	                Pool p = createFixture(false,nbChildren,scope.getPools(), ctc);
	                p.setParent(mainPool);
	                childrenPool[i] = p;
	            }
	            
	            //Insert a null object
	            childrenPool[nbChildren] = null;
	            
	            mainPool.setPools(childrenPool);
	        }
    
            if (scope.getResources() != null)
            {
                ResourceScope rScope = scope.getResources();
                rScope.setParent(null);

                Resource[] rArray = new Resource[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    Resource r = ResourceFixture.createFixture(mainPool, rScope, ctc);
                    rArray[i] = r;
                }
                
	            //Insert a null object
                rArray[nbChildren] = null;
                
                mainPool.setResources(rArray);
            }
            
            if(scope.getDomain() != null)
            {
                mainPool.setDomain(PoolDomainFixture.createSaveFixture(nbChildren, ctc));
            }
    
            if (scope.getPoolReferences() != null)
            {
                PoolReferenceScope rScope = new PoolReferenceScope();
    
                PoolReference[] rArray = new PoolReference[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    PoolReference r = PoolReferenceFixture.createFixture(mainPool, rScope);
                    r.setParent(mainPool);
                    rArray[i] = r;
                }
                
	            //Insert a null object
                rArray[nbChildren] = null;
                
                mainPool.setPoolReferences(rArray);
            }
            
            if (scope.isAttributeAssignments())
            {
                AttributeAssignment[] paaArray = new AttributeAssignment[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    AttributeAssignment paa = AttributeAssignmentFixture.createAssignmentFixture(mainPool, ctc);
                    paaArray[i] = paa;
                }
                
	            //Insert a null object
                paaArray[nbChildren] = null;
                
                mainPool.setAttributeAssignments(paaArray);
            }
    
            if (scope.isResourceAssignments())
            {
            	ResourceRoleAssignment[] roleAssignments = new ResourceRoleAssignment[1];
            	roleAssignments[0] = ResourceRoleAssignmentFixture.createPoolResourceRoleAssignment(mainPool.getContextName(), mainPool, ctc);
            	mainPool.setResourceAssignments(roleAssignments);
            }   
            
        	//Document Element
        	if(scope.getDocumentFolder()!=null){
        		DocumentFolder documentFolder = (DocumentFolder)DocumentElementFixture.createFixture(mainPool, new DocumentFolder(), 0, scope.getDocumentFolder(), ctc);
        		mainPool.setDocumentFolder(documentFolder);
        	}
        	
        	if (scope.isAdministrativeTasks())
            {

                DefaultAdministrativeTask[] rArray = new DefaultAdministrativeTask[nbChildren + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    DefaultAdministrativeTask r = DefaultAdministrativeTaskFixture
                        .createFixture(mainPool, ctc);
                    r.setParent(mainPool);
                    rArray[i] = r;
                }

                //Insert a null object
                rArray[nbChildren] = null;

                mainPool.setAdministrativeTasks(rArray);

            }
        }
        
        return mainPool;
    }

    public static Pool createSaveFixture(ClientTestController ctc) throws Exception {
        
        Pool pool = createFixture(false, 0, null, ctc);
        pool = (Pool) save(ctc, pool, null);
        
        return pool;

    }        //update fields    public static void updateFixtureFields(Pool pool, PoolScope scope) throws Exception    {    	updateFields(pool, random(), scope);    }    private static void updateFields(Pool pool, int index, PoolScope scope) throws Exception    {    	HelperTestUtilities.setByIntrospection(pool, index, skipSetters);		if (scope != null && scope.getPools() != null) {			for(int i=0;i<pool.getPools().length;i++){				updateFixtureFields(pool.getPools()[i], scope.getPools());			}		}    }    private static int random(){    	return new Double(Math.random() * 10000).intValue();    }
    /**
     * @param createParent
     * @param nbChildren
     * @param nbLayers
     * @param scope
     * @throws Exception
     */
        public static Pool updateFixture(Pool pool, ClientTestController ctc) throws Exception
        {
            
            String poolId = pool.getID();
            
            int nbChildren = 0;
            if(pool.getPools() != null){
                nbChildren = pool.getPools().length;
            }
            
            Pool  updatedPool = createFixture(false,0,null, ctc);
            updatedPool.setID(poolId);
            
            // Cannot be cleared
            updatedPool.setMaskEmployeeInfo(pool.getMaskEmployeeInfo());
            updatedPool.setPublish(pool.getPublish());
            
            if (pool.getParent() != null) {
                //Parent pool reference can't be changed
                updatedPool.setParent(pool.getParent());
            }
            
            if(pool.getResources() != null){
                    
                Resource[] resources = pool.getResources();
                Resource[] rArray = new Resource[nbChildren * 2];
                for (int i = 0; i < nbChildren; i++)
                {
                    //delete existing resource
                    resources[i].setDeleted(Boolean.TRUE);
                    rArray[i]= resources[i];
                    
                    //Add new resources
                    Resource r = ResourceFixture.createFixture(pool, null, ctc);
                    rArray[nbChildren + i] = r;
                }
                
                updatedPool.setResources(rArray);           
            }
    
            if (pool.getPools() != null)
            {
                Pool[] pools = pool.getPools();
                Pool[] childrenPool = new Pool[nbChildren * 2];
                for (int i = 0; i < nbChildren; i++)
                {
                    //Delete existing pool
                    pools[i].setDeleted(Boolean.TRUE);
                    childrenPool[i] = pools[i];
                    
                    //Update other pools and set a new ID in order to create
                    //new pools but keep the old structure to respect the scope
                    Pool p = updateFixture(pool.getPools()[i], ctc);
                    p.setID(null);
                    p.setParent(updatedPool);
                    childrenPool[nbChildren + i] = p;
                }
                
                updatedPool.setPools(childrenPool);
            }
    	        
            if(pool.getDomain() != null)
            {
                updatedPool.setDomain(PoolDomainFixture.createSaveFixture(nbChildren, ctc));
            }
            
            if (pool.getPoolReferences() != null)
            {
                
                PoolReferenceScope rScope = new PoolReferenceScope();
                
                PoolReference[] references = pool.getPoolReferences();
                PoolReference[] rArray = new PoolReference[nbChildren * 2 + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    //delete existing reference
                    references[i].setDeleted(Boolean.TRUE);
                    rArray[i] = references[i];
                    
                    //Add a new reference
                    PoolReference r = PoolReferenceFixture.createFixture(updatedPool, rScope);
                    r.setParent(updatedPool);
                    rArray[nbChildren + i] = r;
                }
                
                rArray[nbChildren * 2] = null;
                updatedPool.setPoolReferences(rArray);
            }
            
            if (pool.getAttributeAssignments() != null)
            {
                AttributeAssignment[] attributes = pool.getAttributeAssignments();
                AttributeAssignment[] paaArray = new AttributeAssignment[nbChildren * 2];
                for (int i = 0; i < nbChildren; i++)
                {
                    //delete existing atributes
                    attributes[i].setDeleted(Boolean.TRUE);
                    paaArray[i] = attributes[i];
                    
                    AttributeAssignment paa = AttributeAssignmentFixture
                    .createAssignmentFixture(updatedPool, ctc);
                    paaArray[nbChildren + i] = paa;
                }
                
                updatedPool.setAttributeAssignments(paaArray);
            }
            
            if (pool.getResourceAssignments() != null)
            {
            	ResourceRoleAssignment[] assignments = pool.getResourceAssignments();
            	assignments = ResourceRoleAssignmentFixture.removeAdminResourceAssignment(assignments);
            	
            	int children = 1;
            	ResourceRoleAssignment[] rraArray = new ResourceRoleAssignment[children * 2];
                for (int i = 0; i < children; i++)
                {
                    //delete existing assignments
                	assignments[i].setDeleted(Boolean.TRUE);
                	rraArray[i] = assignments[i];
                    
                	ResourceRoleAssignment rra = ResourceRoleAssignmentFixture
                    .createPoolResourceRoleAssignment(updatedPool.getContextName(), updatedPool, ctc);
                	rraArray[children + i] = rra;
                }
                
                updatedPool.setResourceAssignments(rraArray);
            }   
            
            if(pool.getAdministrativeTasks() != null){
                
                DefaultAdministrativeTask[] admintasks = pool.getAdministrativeTasks();
                DefaultAdministrativeTask[] rArray = new DefaultAdministrativeTask[nbChildren * 2 + 1];
                for (int i = 0; i < nbChildren; i++)
                {
                    //delete existing reference
                    admintasks[i].setDeleted(Boolean.TRUE);
                    rArray[i] = admintasks[i];
                    
                    //Add a new reference
                    DefaultAdministrativeTask r = DefaultAdministrativeTaskFixture.createFixture(updatedPool, ctc);
                    r.setParent(updatedPool);
                    rArray[nbChildren + i] = r;
                }
                
                rArray[nbChildren * 2] = null;
                updatedPool.setAdministrativeTasks(rArray);                
            }
            
            return updatedPool;
        }
    
    public static void compare(Pool p1, Pool p2, PoolScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(p1, p2, poolSkipGetters);
        
        if(scope == null) return;
        
        if (scope.isAttributeAssignments())
        {
            if (p2.getAttributeAssignments() == null)
                Assert.fail("RPMObject in scope is null");
            
            AttributeAssignmentFixture.compare(
                    p1.getAttributeAssignments(), p2.getAttributeAssignments());
        }
        
        if(scope.getDomain() != null)
        {
            if (p2.getDomain() == null)
                Assert.fail("RPMObject in scope is null");
            
            PoolDomainFixture.compare(p1.getDomain(), p2.getDomain(), null);
        }
        
        if(scope.getParent() != null)
        {
            if (p2.getParent() == null)
                Assert.fail("RPMObject in scope is null");
            
            //Be careful with the scope to avoid infinite loop
            PoolScope parentScope = (PoolScope)scope.getParent();
            if(parentScope != null)
            {
                parentScope.setPools(null);
            }
            
            PoolFixture.compare((Pool)p1.getParent(),(Pool) p2.getParent(),
                    parentScope);
        }
        
        if(scope.getPools() != null)
        {
            if (p2.getPools() == null)
                Assert.fail("RPMObject in scope is null");
            
            
            //Be careful with the scope to avoid infinite loop
            PoolScope childrenScope = scope.getPools();
            if(childrenScope != null)
            {
                childrenScope.setParent(null);
            }
            
            PoolFixture.compare(p1.getPools(), p2.getPools(), childrenScope);
        }
        
        if(scope.getPoolReferences() != null)
        {
            if (p2.getPoolReferences() == null)
                Assert.fail("RPMObject in scope is null");
            
            PoolReferenceScope rScope = scope.getPoolReferences();
            PoolReferenceFixture.compare(p1.getPoolReferences(), p2.getPoolReferences(), rScope);
        }
        
        if(scope.getResources() != null)
        {
            if (p2.getResources() == null)
                Assert.fail("RPMObject in scope is null");
            
            ResourceScope rScope = scope.getResources();
            rScope.setParent(null);
            ResourceFixture.compareWithoutCanProperties(p1.getResources(), p2.getResources(), rScope);
        }
        
        // DocumentFolder
        if(scope.getDocumentFolder()!=null){
    		DocumentFolder documentFolder1 =  p1.getDocumentFolder();
    		DocumentFolder documentFolder2 =  p2.getDocumentFolder();
    		DocumentElementFixture.compare(documentFolder1, documentFolder2, scope.getDocumentFolder());
        }
        
        // Administrative Task
        if (scope.isAdministrativeTasks())
        {
            if (p2.getAdministrativeTasks() == null)
                Assert.fail("RPMObject in scope is null");

            AdministrativeTaskFixture.compare(p1.getAdministrativeTasks(), p2
                .getAdministrativeTasks());
        }
    }
    
    
    public static void compare(Pool[] pArray1, Pool[] pArray2, PoolScope scope)
            throws AssertionFailedError, Exception
    {        
        pArray1 = removeDeletedPool(pArray1);
        
    	sortWithComparator(pArray1, RPMObjectNameAndIDComparator.INSTANCE);
    	sortWithComparator(pArray2, RPMObjectNameAndIDComparator.INSTANCE); 
        
        RPMObjectFixture.compare(pArray1, pArray2);
        if(pArray1 == null) return;
        
        for(int i = 0; i < pArray1.length; i++)
        {
            PoolFixture.compare(pArray1[i], pArray2[i], scope);
        }
    }
    
    public static Pool[] removeDeletedPool(
            Pool[] pArray) {

        Pool[] newArray = null;

        ArrayList temp = new ArrayList(Arrays.asList(pArray));

        temp = removeNullAndDeletedRPMObjects(temp);

        if (temp != null && temp.size() > 0) {
            newArray = new Pool[temp.size()];
            temp.toArray(newArray);
        }

        return newArray;

    }

}