/*
 * Created on Oct 26, 2005
 *
 */
package com.ibm.rpm.client.junit.resource.fixtures;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.applicationadministration.scope.PoolDomainScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.PoolDomainFixture;
import com.ibm.rpm.client.junit.fixtures.AbstractModuleFixture;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolModule;
import com.ibm.rpm.resource.scope.PoolModuleScope;
import com.ibm.rpm.resource.scope.PoolScope;

/**
 * @author karim
 *  
 */
public class PoolModuleFixture extends AbstractModuleFixture
{
    public static PoolModule createFixture(PoolModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        return createModule(scope, ctc);
    }

    public static PoolModule updateFixture(PoolModule module,
            PoolModuleScope scope, ClientTestController ctc) throws Exception
    {
        return updateModule(module, scope, ctc);
    }

    private static PoolModule createModule(PoolModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        PoolModule module = new PoolModule();

        if(scope!=null)
        {
            if(scope.getAttributeCategories()!=null)
            {
                AttributeCategory[] attributeCategories = (AttributeCategory[]) createElements(
                        scope.getAttributeCategories(), AttributeCategory.class, ctc);
                    module.setAttributeCategories(attributeCategories);                
            }
            if(scope.getPools()!=null)
            {
                Pool[] pools = (Pool[]) createElements(scope.getPools(), Pool.class,
                        ctc);
                    module.setPools(pools);                
            }
            if(scope.getPoolDomains()!=null)
            {
                PoolDomain[] poolDomains = (PoolDomain[]) createElements(scope
                        .getPoolDomains(), PoolDomain.class, ctc);
                    module.setPoolDomains(poolDomains);                
            }
        }

        return module;
    }

    private static PoolModule updateModule(PoolModule module,
            PoolModuleScope scope, ClientTestController ctc) throws Exception
    {
        if (scope != null)
        {
            if(scope.getAttributeCategories()!=null)
            {
                AttributeCategory[] attributeCategories = (AttributeCategory[]) updateElements(
                        module.getAttributeCategories(), scope.getAttributeCategories(), ctc);
                    module.setAttributeCategories(attributeCategories);
            }
            if(scope.getPools()!=null)
            {
                Pool[] pools = (Pool[]) updateElements(module.getPools(), scope
                        .getPools(), ctc);
                    module.setPools(pools);
            }
            if(scope.getPoolDomains()!=null)
            {
                PoolDomain[] poolDomains = (PoolDomain[]) updateElements(module
                        .getPoolDomains(), scope.getPoolDomains(), ctc);
                    module.setPoolDomains(poolDomains);
            }
        }
        return module;
    }

    public static PoolModuleScope createFullScope()
    {
        return createScope(true,true,true,true);
    }
    
    private static PoolModuleScope createScope(boolean isAttributeCategories,
            boolean isPools, boolean isPoolDomains, boolean isCompanySizes)
    {
        PoolModuleScope scope = new PoolModuleScope();

        if (isAttributeCategories)
        {
            scope.setAttributeCategories(new AttributeScope());
        }
        if (isPools)
        {
            scope.setPools(new PoolScope());
        }
        if (isPoolDomains)
        {
            scope.setPoolDomains(new PoolDomainScope());
        }
        if (isCompanySizes)
        {
            scope.setCompanySizes(true);
        }

        return scope;
    }

    public static void compare(PoolModule obj1, PoolModule obj2,
            PoolModuleScope scope) throws AssertionFailedError, Exception
    {
        if (scope != null)
        {
            if(scope.getAttributeCategories()!=null)
            {
                AttributeFixture.compare(obj1.getAttributeCategories(), obj2
                        .getAttributeCategories(), scope.getAttributeCategories());
            }
            if(scope.getPools()!=null)
            {
                PoolFixture.compare(obj1.getPools(), obj2.getPools(), scope.getPools());
            }
            if(scope.getPoolDomains()!=null)
            {
                PoolDomainFixture.compare(obj1.getPoolDomains(), obj2.getPoolDomains(),
                        scope.getPoolDomains());
            }
            if(scope.isCompanySizes())
            {
                DatafieldFixture.compare(obj1.getCompanySizes(), obj2.getCompanySizes());                
            }
        }
    }
}
