/*
 * Created on Jul 7, 2005
 */
package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.CompanySize;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolReference;
import com.ibm.rpm.resource.scope.PoolReferenceScope;

/**
 * @author gosselin
 */
public class PoolReferenceFixture extends RPMObjectFixture
{
    protected static class ContactNameComparator implements Comparator {
        /**
         * 
         */
        public ContactNameComparator() {
            super();
        }
        
        public int compare(Object o1, Object o2){ 
            
            PoolReference obj1 = (PoolReference)o1;
            PoolReference obj2 = (PoolReference)o2;
            
            String key1 = null;
            String key2 = null;
            
            if(obj1 != null){
                key1 = obj1.getContactName();
            }
            if(obj2 != null){
                key2 = obj2.getContactName();
            }                
            
            return compareStrings(key1, key2);
        }
    
    }    
    
    protected static ArrayList referenceSkipSetters, referenceSkipGetters;
    
    static
    {
        
        //Setters
        referenceSkipSetters = new ArrayList();
        referenceSkipSetters.addAll(skipSetters);
        referenceSkipSetters.add("setName");
        
        referenceSkipGetters = new ArrayList();
        referenceSkipGetters.addAll(skipGetters);
        referenceSkipGetters.add("getName");        

    }
    
    public static PoolReference createFixture(Pool pool, PoolReferenceScope scope) throws Exception
    {
        PoolReference ref = new PoolReference();
        HelperTestUtilities.setByIntrospection(ref, new Double(Math.random() * 100)
                .intValue(), referenceSkipSetters);
        
        if(scope.isCompanySize())
        {
            //CompanySize
            CompanySize cs = DatafieldFixture.createRandomCompanySize();
            ref.setCompanySize(cs);
        }
        
        ref.setParent(pool);

        
        return ref;
    }


    public static void compare(PoolReference r1, PoolReference r2, PoolReferenceScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(r1, r2, referenceSkipGetters);

        if (scope == null) return;
        
        if(scope.isCompanySize())
        {
            DatafieldFixture.compare(r1.getCompanySize(), r2.getCompanySize());
        }
        
        PoolFixture.compare((Pool)r1.getParent(), (Pool)r2.getParent(), null);
    }

    public static void compare(PoolReference[] pArray1, PoolReference[] pArray2, PoolReferenceScope scope)
            throws AssertionFailedError, Exception
    {
        pArray1 = removeDeletedReference(pArray1);
        
        sortWithComparator(pArray1, new ContactNameComparator());
        sortWithComparator(pArray2, new ContactNameComparator());
        
        RPMObjectFixture.compare(pArray1, pArray2);
        if(pArray1 == null) return;
        
        for(int i = 0; i < pArray1.length; i++)
        {
            compare(pArray1[i], pArray2[i], scope);
        }
    }
    
    public static PoolReference[] removeDeletedReference(
            PoolReference[] pArray) {

        PoolReference[] newArray = null;

        ArrayList temp = new ArrayList(Arrays.asList(pArray));

        temp = removeNullAndDeletedRPMObjects(temp);

        if (temp != null && temp.size() > 0) {
            newArray = new PoolReference[temp.size()];
            temp.toArray(newArray);
        }

        return newArray;

    }

}
