/*
 * Created on Oct 26, 2005
 *
 */
package com.ibm.rpm.client.junit.resource.fixtures;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.applicationadministration.scope.ContactGroupScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.ContactGroupFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.fixtures.AbstractModuleFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.containers.ResourceModule;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;
import com.ibm.rpm.resource.scope.ResourceModuleScope;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.scope.ScorecardFolderScope;

/**
 * @author karim
 *  
 */
public class ResourceModuleFixture extends AbstractModuleFixture
{
    public static ResourceModule createFixture(ResourceModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        return createModule(scope, ctc);
    }

    public static ResourceModule updateFixture(ResourceModule module,
            ResourceModuleScope scope, ClientTestController ctc) throws Exception
    {
        return updateModule(module, scope, ctc);
    }

    private static ResourceModule createModule(ResourceModuleScope scope,
            ClientTestController ctc) throws Exception
    {
        ResourceModule module = new ResourceModule();

        if (scope != null)
        {
            if(scope.getResourceAttributeCompetencyCategories()!=null)
            {
                ResourceAttributeCompetencyCategory[] racc = (ResourceAttributeCompetencyCategory[]) createElements(
                        scope.getResourceAttributeCompetencyCategories(),
                        ResourceAttributeCompetencyCategory.class, ctc);
                    module.setResourceAttributeCompetencyCategories(racc);                
            }
            if(scope.getResourceAttributeSkillCategories()!=null)
            {
                ResourceAttributeSkillCategory[] rasc = (ResourceAttributeSkillCategory[]) createElements(
                        scope.getResourceAttributeSkillCategories(),
                        ResourceAttributeSkillCategory.class, ctc);
                    module.setResourceAttributeSkillCategories(rasc);                
            }
            if(scope.getContactGroups()!=null)
            {
                ContactGroup[] contactgrp = (ContactGroup[]) createElements(scope
                        .getContactGroups(), ContactGroup.class, ctc);
                    module.setContactGroups(contactgrp);                
            }
            if(scope.getCustomFieldCategories()!=null)
            {
                module.setCustomFieldCategories(new CustomFieldCategory[]
                {
                    //creates a CustomFieldCategory with one child
                        CustomFieldFixture.createCustomFieldCategory(1)
                    });                
            }
            if (scope.getScorecardFolders() != null)
            {
                ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) createElements(
                    scope.getScorecardFolders(), ScorecardFolder.class, ctc);
                module.setScorecardFolders(scorecardFolders);
            }
        }

        return module;
    }

    private static ResourceModule updateModule(ResourceModule module,
            ResourceModuleScope scope, ClientTestController ctc) throws Exception
    {
        if (scope != null)
        {
            if(scope.getResourceAttributeCompetencyCategories()!=null)
            {
                ResourceAttributeCompetencyCategory[] racc = (ResourceAttributeCompetencyCategory[]) updateElements(
                        module.getResourceAttributeCompetencyCategories(), scope
                            .getResourceAttributeCompetencyCategories(), ctc);
                    module.setResourceAttributeCompetencyCategories(racc);
            }
            if(scope.getResourceAttributeSkillCategories()!=null)
            {
                ResourceAttributeSkillCategory[] rasc = (ResourceAttributeSkillCategory[]) updateElements(
                        module.getResourceAttributeSkillCategories(), scope
                            .getResourceAttributeSkillCategories(), ctc);
                    module.setResourceAttributeSkillCategories(rasc);
            }
            if(scope.getContactGroups()!=null)
            {
                ContactGroup[] cgroups = (ContactGroup[]) updateElements(module
                        .getContactGroups(), scope.getContactGroups(), ctc);
                    module.setContactGroups(cgroups);
            }
            if(scope.getCustomFieldCategories()!=null)
            {
                module.setCustomFieldCategories(CustomFieldFixture
                        .modifyCustomFieldCategories(module.getCustomFieldCategories(),
                            scope.getCustomFieldCategories()));
            }
        }
        return module;
    }

    public static ResourceModuleScope createFullScope()
    {
        return createScope(true, true, true, true,
                true, true, true, true, true, true, true, true, true, true, true);
    }
    
    private static ResourceModuleScope createScope(boolean isracc,
            boolean israsc, boolean iscgoups, boolean iscccode,
            boolean isempstatus, boolean isworkstatus, boolean isclassifns,
            boolean isindtype, boolean islangs, boolean isprolevels,
            boolean ispreranks, boolean isexp, boolean isworklocation,
            boolean isCustomFields, boolean getScorecardFolders)
    {
        ResourceModuleScope scope = new ResourceModuleScope();

        if (isracc)
        {
            scope
                .setResourceAttributeCompetencyCategories(new ResourceAttributeScope());
        }
        if (israsc)
        {
            scope
                .setResourceAttributeSkillCategories(new ResourceAttributeScope());
        }
        if (iscgoups)
        {
            scope.setContactGroups(new ContactGroupScope());
        }
        if (iscccode)
        {
            scope.setCitizenshipCountryCodes(iscccode);
        }
        if (isempstatus)
        {
            scope.setEmployeeStatus(isempstatus);
        }
        if (isworkstatus)
        {
            scope.setWorkingStatus(isworkstatus);
        }
        if (isclassifns)
        {
            scope.setClassifications(isclassifns);
        }
        if (isindtype)
        {
            scope.setIndustryTypes(isindtype);
        }
        if (islangs)
        {
            scope.setLanguages(islangs);
        }
        if (isprolevels)
        {
            scope.setProficiencyLevels(isprolevels);
        }
        if (ispreranks)
        {
            scope.setPreferenceRanks(ispreranks);
        }
        if (isexp)
        {
            scope.setExperiences(isexp);
        }
        if (isworklocation)
        {
            scope.setWorkLocations(isworklocation);
        }
        if (isCustomFields)
        {
            scope.setCustomFieldCategories(CustomFieldFixture
                .createCustomFieldScope(2));
        }
        if (getScorecardFolders)
        {
            ScorecardFolderScope sfScope = ScorecardFixture.createScope();          
            scope.setScorecardFolders(sfScope);
        }

        return scope;
    }

    public static void compare(ResourceModule obj1, ResourceModule obj2,
            ResourceModuleScope scope) throws AssertionFailedError, Exception
    {
        if(scope!=null)
        {
            if (scope.getResourceAttributeCompetencyCategories()!=null)
            {
                CompetencyFixture.compare(obj1
                        .getResourceAttributeCompetencyCategories(), obj2
                        .getResourceAttributeCompetencyCategories(), scope
                        .getResourceAttributeCompetencyCategories());
            }
            if (scope.getResourceAttributeSkillCategories()!=null)
            {
                SkillFixture.compare(obj1.getResourceAttributeSkillCategories(), obj2
                        .getResourceAttributeSkillCategories(), scope
                        .getResourceAttributeSkillCategories());
            }
            if (scope.getContactGroups()!=null)
            {
                ContactGroupFixture.compare(obj1.getContactGroups(), obj2
                        .getContactGroups(), scope.getContactGroups());
            }
            if (scope.isCitizenshipCountryCodes())
            {
                DatafieldFixture.compare(obj1.getCitizenshipCountryCodes(), obj2
                        .getCitizenshipCountryCodes());
            }
            if (scope.isEmployeeStatus())
            {
                DatafieldFixture.compare(obj1.getEmployeeStatus(), obj2
                        .getEmployeeStatus());
            }
            if (scope.isWorkingStatus())
            {
                DatafieldFixture.compare(obj1.getWorkingStatus(), obj2
                        .getWorkingStatus());
            }
            if (scope.isClassifications())
            {
                DatafieldFixture.compare(obj1.getClassifications(), obj2
                        .getClassifications());
            }
            if (scope.isIndustryTypes())
            {
                DatafieldFixture.compare(obj1.getIndustryTypes(), obj2
                        .getIndustryTypes());
            }
            if (scope.isLanguages())
            {
                DatafieldFixture.compare(obj1.getLanguages(), obj2.getLanguages());
            }
            if (scope.isProficiencyLevels())
            {
                DatafieldFixture.compare(obj1.getProficiencyLevels(), obj2
                        .getProficiencyLevels());
            }
            if (scope.isPreferenceRanks())
            {
                DatafieldFixture.compare(obj1.getPreferenceRanks(), obj2
                        .getPreferenceRanks());
            }
            if (scope.isExperiences())
            {
                DatafieldFixture.compare(obj1.getExperiences(), obj2.getExperiences());
            }
            if (scope.isWorkLocations())
            {
                DatafieldFixture.compare(obj1.getWorkLocations(), obj2
                        .getWorkLocations());
            }
            if (scope.getCustomFieldCategories()!=null)
            {
                CustomFieldFixture.assertIdEqual(obj1.getCustomFieldCategories(), obj2
                        .getCustomFieldCategories(), scope.getCustomFieldCategories());
            }
            if(scope.getScorecardFolders()!=null){
                ScorecardFixture.compare(obj1.getScorecardFolders(), obj2
                        .getScorecardFolders(), scope.getScorecardFolders());           
            }
        }
    }
}
