/*
 * Created on Dec 2, 2005
 *
 * 
 * Window - Preferences - Java - Code Style - Code Templates
 */
package com.ibm.rpm.client.junit.resource.fixtures;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.security.containers.SecurityRole;



/**
 * @author SandeepS
 *
 * 
 */
public class ResourceRoleAssignmentFixture extends RPMObjectFixture{
	protected static ArrayList localSkipGetters,localSkipSetters;

	public final static int ROLE_LEVEL_TYPE_ID = 186;
	
	static {
		localSkipSetters = new ArrayList();
		localSkipSetters.addAll(skipSetters);
		
		localSkipGetters = new ArrayList();
		localSkipGetters.addAll(skipGetters);
	}
	
	public static ResourceRoleAssignment createFixture(Resource resource, RPMObject parent,ClientTestController ctc)
	{
		SecurityRole securityRole = null; 
		//TODO: Create a SecurityRole using the SecurityRole Fixture
		return createFixture(resource,parent,securityRole);
	}

	/**
	 * @param resource
	 * @param parent
	 * @param securityRole
	 * @return
	 */
	public static ResourceRoleAssignment createFixture(Resource resource, RPMObject parent, SecurityRole securityRole) {
		if(resource!=null&&parent!=null)
		{
			return createResourceRoleAssignment(resource,parent,securityRole);
		}
		return null;
	}

	/**
	 * @param resource
	 * @param parent
	 * @param sra
	 * @return
	 */
	private static ResourceRoleAssignment createResourceRoleAssignment(Resource resource, RPMObject parent,SecurityRole securityRole) {
		ResourceRoleAssignment rra = new ResourceRoleAssignment();
		rra.setParent(parent);
		rra.setResource(resource);
		rra.setSecurityRole(securityRole);
		return rra;
	}
	
	public static ResourceRoleAssignment[] removeAdminResourceAssignment(
            ResourceRoleAssignment[] resArray)
    {
	    if (resArray == null)
            return resArray;
	    ArrayList temp = new ArrayList(Arrays.asList(resArray));
	    
        temp = RPMObjectFixture.removeNullAndDeletedRPMObjects(temp);
        
        for (int i = 0; i < temp.size(); i++)
        {
            ResourceRoleAssignment resRoleAss = (ResourceRoleAssignment) temp
                .get(i);
            if (resRoleAss.getResource() != null)
            {
                if (resRoleAss.getResource().getFullName().equals(
                    "ADMINISTRATOR"))
                {
                    temp.remove(resRoleAss);
                }
            }
        }
        
        temp = RPMObjectFixture.removeNullAndDeletedRPMObjects(temp);
        ResourceRoleAssignment[] rArray = new ResourceRoleAssignment[temp.size()];
	    return (ResourceRoleAssignment[])temp.toArray(rArray);
    }
	
	public static void compare(ResourceRoleAssignment[] array1, ResourceRoleAssignment[] array2) 
	throws AssertionFailedError, Exception {
	    array1 = removeAdminResourceAssignment(array1);
	    array2 = removeAdminResourceAssignment(array2);
	    
        sortWithComparator(array1, new RPMObjectResourceComparator());
        sortWithComparator(array2, new RPMObjectResourceComparator());

		if(array1 == null) return;

		RPMObjectFixture.compare(array1, array2);
		
		for (int i = 0; i < array1.length; i++) {
			compare(array1[i], array2[i]);
		}
	}
	
	public static void compare(ResourceRoleAssignment element1,ResourceRoleAssignment element2)
	throws AssertionFailedError, Exception {
		RPMObjectFixture.compareWithSkip(element1,element2,localSkipGetters);
		if(element1.getResource()!=null)
		{
			ResourceFixture.compareWithoutCanProperties(element1.getResource(),element2.getResource(),null);
		}
		
		if(element1.getParent() instanceof ScopeElement)
		{
			
		}
		
		//TODO: write the conditions for the parents of other type
	}
	
    public static ResourceRoleAssignment createDocumentResourceRoleAssignment(String contextName,
            RPMObject parent, ClientTestController ctc) throws Exception
    {
        ResourceRoleAssignment assignment = new ResourceRoleAssignment();

        HelperTestUtilities.setByIntrospection(assignment, random(), 
        		localSkipSetters);
        
        assignment.setParent(parent);
        
        Resource resource = ResourceFixture.createSaveFixture(ctc);
        assignment.setResource(resource);
        
        /*
        DocumentSecurityRole secRole = new DocumentSecurityRole();
        secRole.setName("TestDocumentSecurityRole");
        secRole.setSecurityRole(secRole);
        secRole.setType(DocumentSecurityRoleType.DocumentParticipant);
        secRole.assignTypeID(new Integer(ResourceRoleAssignmentFixture.ROLE_LEVEL_TYPE_ID));
        
        // TODO: loading appears to be an issue so ReloadType.None is passed for now
        SaveResult result = ctc.save(secRole, null, ReloadType.None);
        */


        SecurityRole docSecurityRole = new SecurityRole();
        docSecurityRole.setID("SCDOCUMENTRESPONSIBLE");

        assignment.setSecurityRole(docSecurityRole);  

        return assignment;

    }
	
    public static ResourceRoleAssignment createPoolResourceRoleAssignment(String contextName,
            RPMObject parent, ClientTestController ctc) throws Exception
    {
        ResourceRoleAssignment assignment = new ResourceRoleAssignment();

        HelperTestUtilities.setByIntrospection(assignment, random(), 
        		localSkipSetters);
        
        assignment.setParent(parent);
        
        Resource resource = ResourceFixture.createSaveFixture(ctc);
        assignment.setResource(resource);
        
        SecurityRole poolSecurityRole = new SecurityRole();
        poolSecurityRole.setID("SCPOOLPARTICIPANT");

        assignment.setSecurityRole(poolSecurityRole);  

        return assignment;

    }
	
	
    protected static class RPMObjectResourceComparator implements Comparator
    {
        /**
         *  
         */
        public RPMObjectResourceComparator()
        {
            super();
        }

        public int compare(Object o1, Object o2)
        {

            ResourceRoleAssignment rra1 = (ResourceRoleAssignment) o1;
            ResourceRoleAssignment rra2 = (ResourceRoleAssignment) o2;

            Resource obj1 = rra1.getResource();
            Resource obj2 = rra2.getResource();
            
            String key1 = null;
            String key2 = null;
            
            if(obj1 != null){
                key1 = obj1.getFullName();
            }
            if(obj2 != null){
                key2 = obj2.getFullName();
            }                
            
            if (key1 == null && key2 == null) {
                return 0;
            }
            if (key1 == null && key2 != null) {
                return -1;
            }
            if (key1 != null && key2 == null) {
                return 1;
            }
            return key1.compareTo(key2);
        }

    }

}
