/*
 * Created on Oct 26, 2005
 *
 */
package com.ibm.rpm.client.junit.resource.testcases;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.CompanySize;
import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.client.junit.framework.testcases.ModuleTestController;
import com.ibm.rpm.client.junit.resource.fixtures.PoolModuleFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolModule;
import com.ibm.rpm.resource.scope.PoolModuleScope;

/**
 * @author karim
 *  
 */
public class TestPoolModule extends ModuleTestController
{
    public TestPoolModule(String testName)
    {
        setName(testName);
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = PoolModuleFixture.createFullScope();
        moduleContaineName = "POOLMODULE";
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public RPMObject load(AbstractModule aModule, RPMObjectScope aScope)
            throws Exception
    {

        PoolModule module = (PoolModule) aModule;
        PoolModuleScope scope = (PoolModuleScope) aScope;

        PoolModule loadedModule = new PoolModule();

        if(scope!=null)
        {
            if(scope.getAttributeCategories()!=null)
            {
                AttributeCategory[] attributeCategories = (AttributeCategory[]) loadElements(
                        module.getAttributeCategories(), scope.getAttributeCategories(),AttributeCategory.class);
                    loadedModule.setAttributeCategories(attributeCategories);                
            }
            if(scope.getPools()!=null)
            {
                Pool[] pools = (Pool[])loadElements(module.getPools(), scope.getPools(),Pool.class);
                loadedModule.setPools(pools);                
            }
            if(scope.getPoolDomains()!=null)
            {
                PoolDomain[] poolDomains = (PoolDomain[]) loadElements(module.getPoolDomains(), scope.getPoolDomains(),PoolDomain.class);
                loadedModule.setPoolDomains(poolDomains);                
            }
            if(scope.isCompanySizes())
            {
                CompanySize[] companySizes = (CompanySize[]) loadDatafields(module.getCompanySizes(),true,CompanySize.class);
                loadedModule.setCompanySizes(companySizes);                
            }
        }
        return loadedModule;
    }


   
    protected void deleteModule(AbstractModule aModule, RPMObjectScope aScope)
            throws AssertionError, Exception
    {

        PoolModule module = (PoolModule) aModule;
        PoolModuleScope scope = (PoolModuleScope) aScope;
        
        //delete the element
        if (module != null && scope != null)
        {
            if (module.getAttributeCategories() != null && scope.getAttributeCategories()!=null)
            {
                delete(module.getAttributeCategories());
            }
            if (module.getPools() != null && scope.getPools()!=null)
            {
                delete(module.getPools());
            }
            if (module.getPoolDomains() != null && scope.getPoolDomains()!=null)
            {
                delete(module.getPoolDomains());
            }
            
            //reload the deleted element
            PoolModule deletedModule = (PoolModule) loadDeletedModule(module, scope);

            //Validate the deleted element
            if(scope.getAttributeCategories() !=null)
            {
                assertNull(deletedModule.getAttributeCategories());   
            }
            if(scope.getPools() !=null)
            {
                assertNull(deletedModule.getPools());                
            }
            if(scope.getPoolDomains() !=null)
            {
                assertNull(deletedModule.getPoolDomains());                
            }
        }
    }

    protected AbstractModule createFixture(RPMObjectScope scope)
            throws Exception
    {
        return PoolModuleFixture.createFixture((PoolModuleScope) scope, this);
    }

    protected AbstractModule updateFixture(AbstractModule module,
            RPMObjectScope scope) throws Exception
    {
        return PoolModuleFixture.updateFixture((PoolModule) module,
            (PoolModuleScope) scope, this);
    }

    protected void compare(AbstractModule module,
            AbstractModule returnedModule, RPMObjectScope scope)
            throws AssertionFailedError, Exception
    {
        PoolModuleFixture.compare((PoolModule) module,
            (PoolModule) returnedModule, (PoolModuleScope) scope);
    }
}
