package com.ibm.rpm.client.junit.resource.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.PoolReferenceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolReference;
import com.ibm.rpm.resource.scope.PoolReferenceScope;

/**
 * @author gosselin
 */
public class TestPoolReference extends ClientTestController
{
    private PoolReference mainRef;
    private PoolReferenceScope mainScope;

    public void setUp() throws Exception
    {
        super.setUp();
        mainScope = new PoolReferenceScope();
        mainScope.setCompanySize(true);
        
        Pool pool = PoolFixture.createFixture(false, 0, null, this);
        pool = (Pool)(save(pool,null)).getRpmObject();
        
        mainRef = PoolReferenceFixture.createFixture(pool, mainScope);
        
    }

    public void tearDown() throws Exception
    {

        super.tearDown();
    }


    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        mainRef = saveReference(mainRef, null);
        
        //load from an xpath
        RPMObject[] rpmObjects = loadFromXpath("/PoolReference[id='"
                + mainRef.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof PoolReference);
        
        logger.info("Scoping is null");
        logger.info("MainRef: " + mainRef);
        logger.info("LoadedObject: " + rpmObjects[0]);
        
        PoolReferenceFixture.compare(mainRef, (PoolReference) rpmObjects[0], null);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainRef = saveReference(mainRef, null);
        
        loadReferenceFromId(mainRef, null);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainRef = saveReference(mainRef, mainScope);
        
        loadReferenceFromId(mainRef,mainScope);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainRef = saveReference(mainRef, null);
        
        PoolReference updatedRef = new PoolReference();
        updatedRef.setID(mainRef.getID());
        HelperTestUtilities.setByIntrospection(updatedRef,00, RPMObjectFixture.skipSetters);
        updatedRef.setParent(mainRef.getParent());
        updatedRef.setCompanySize(mainRef.getCompanySize());

        //Save the modifications
        saveReference(updatedRef, null);
        
        loadReferenceFromId(updatedRef,null);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        //TODO        fail("not implemented");
    }

    private PoolReference saveReference(PoolReference p, PoolReferenceScope scope)
            throws AssertionError, Exception
    {
        SingleResult result;
        result = save(p, scope);

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof PoolReference);

        p = (PoolReference) result.getRpmObject();
        return p;
    }

    private void loadReferenceFromId(PoolReference objectToCompare, PoolReferenceScope scope) throws AssertionError,
    Exception {
        logger.info("loadPoolFromId");
        
        RPMObject obj = new PoolReference();
        obj.setID(mainRef.getID());
        RPMObject rpmObject = loadFromID(obj, scope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof PoolReference);
        
        if (scope != null) {
            logger.info("Scoping" + mainScope.toString());
        } else {
            logger.info("Scoping is null");
        }
        logger.info("MainRef: " + mainRef);
        logger.info("LoadedObject: " + rpmObject);
        
        PoolReferenceFixture.compare(objectToCompare, (PoolReference) rpmObject, null);
    }
}
