package com.ibm.rpm.client.junit.resource.testcases;

import java.util.Calendar;

import junit.framework.Assert;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.resource.fixtures.PassportFixture;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultAdministrativeTaskFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.PersonalTaskFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityGroup;
import com.ibm.rpm.security.scope.SecurityGroupScope;
import com.ibm.rpm.timesheet.containers.PersonalTask;
import com.ibm.rpm.timesheet.scope.PersonalProjectScope;

/**
 * @author Genevieve
 *  
 */
public class TestResource2 extends TestResource
{
    public void testInsertWithCompetenciesAssignment() throws AssertionError, Exception
    {
        mainScope.setCompetencyAttributeAssignments(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithContactGroup() throws AssertionError, Exception
    {
        mainScope.setContactGroupAssignments(true);
        createSaveAndReloadResource();
    }
    
    public void testInsertWithVerifiedBy() throws AssertionError, Exception
    {
        mainScope.setVerifiedBy(true);
        createSaveAndReloadResource();
    }
    
    public void testInsertWithReVerifiedBy() throws AssertionError, Exception
    {
        mainScope.setVerifiedBy(true);
        mainScope.setReVerifiedBy(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithCostCenter() throws AssertionError, Exception
    {
        mainScope.setResourceCostCenterAssignments(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithCurrency() throws AssertionError, Exception
    {
        mainScope.setCurrency(true);
        createSaveAndReloadResource(2);
    }

    public void testInsertWithEducation() throws AssertionError, Exception
    {
        mainScope.setEducations(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithEmployeeStatus() throws AssertionError, Exception
    {
        mainScope.setEmployeeStatus(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithEmploymentHistory() throws AssertionError, Exception
    {
        mainScope.setEmploymentHistories(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithExperience() throws AssertionError, Exception
    {
        mainScope.setExperience(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithGeographicalAssignment() throws AssertionError, Exception
    {
        mainScope.setGeographicalAssignment(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithIndustryExperience() throws AssertionError, Exception
    {
        mainScope.setIndustryExperiences(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithLanguageProficiency() throws AssertionError, Exception
    {
        mainScope.setLanguageProficiencies(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithOrganizationAssignment() throws AssertionError, Exception
    {
        mainScope.setOrganizationalAssignment(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithLocationPreference() throws AssertionError, Exception
    {
        mainScope.setLocationPreferences(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithProjectHistory() throws AssertionError, Exception
    {
        mainScope.setProjectHistories(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithCalendar() throws AssertionError, Exception
    {
        mainScope.setCalendar(new CalendarScope());
        createSaveAndReloadResource();
    }

    public void testInsertWithExecutiveAttribute() throws AssertionError, Exception
    {
        mainScope.setExecutiveAttributes(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithExecutiveAttributeClassification() throws AssertionError,
            Exception
    {
        mainScope.setExecutiveAttributeClassifications(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithExecutiveAttributeAndAttributeClassification()
            throws AssertionError, Exception
    {
        mainScope.setExecutiveAttributes(true);
        mainScope.setExecutiveAttributeClassifications(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithWorkingStatus() throws AssertionError, Exception
    {
        mainScope.setWorkingStatus(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithWorkingLocation() throws AssertionError, Exception
    {
        mainScope.setWorkLocation(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithSkillsAssignment() throws AssertionError, Exception
    {
        mainScope.setSkillAttributeAssignments(true);
        createSaveAndReloadResource();
    }

    public void testInsertWithResourceRates() throws AssertionError, Exception
    {
        mainScope.setRates(true);
        createSaveAndReloadResource(2);
    }
    
    public void testLoadWithPersonalProject() throws AssertionError, Exception
    {
        mainScope.setPersonalProject(new PersonalProjectScope());
        createSaveAndReloadResource();
    }
    
    public void testLoadWithPersonalTaskAssignment()
            throws AssertionError, Exception
    {
        mainScope.setCalendar(new CalendarScope());

        PersonalProjectScope pps = new PersonalProjectScope();
        pps.setPersonalTasks(true);
        mainScope.setPersonalProject(pps);

        mainResource = ResourceFixture.createFixture(parentPool, 1, mainScope,
            this);

        Resource saved = saveResource(mainResource, mainScope);

        mainScope.setPersonalTaskAssignments(true);

        PersonalTask personalTask = PersonalTaskFixture.createFixture(
            saved.getPersonalProject(), this);

        saved.getPersonalProject().setPersonalTasks(new PersonalTask[]
        {
            personalTask
        });

        saved = saveResource(saved, mainScope);
        assertNotNull(saved.getPersonalTaskAssignments());
    }
    
    public void testloadWithAdministrativeTaskAssignment() throws AssertionError, Exception
    {

        //add an default administrative task to parent pool
        DefaultAdministrativeTaskFixture.createSaveFixture(parentPool, this);
        
        mainScope.setAdministrativeTaskAssignments(true);
        mainScope.setCalendar(new CalendarScope());
        createSaveAndReloadResource();
    }


    public void testInsertWithPassport() throws AssertionError, Exception
    {
        mainScope.setPassport(true);
        createSaveAndReloadResource();
        mainResource.setPassport(PassportFixture.updateFixture(mainResource, 0));
        mainResource = saveResource(mainResource, mainScope);
        loadResourceFromId(mainResource, mainScope);
        mainResource.setPassport(PassportFixture.updateFixture(mainResource, 1));
        mainResource = saveResource(mainResource, mainScope);
        loadResourceFromId(mainResource, mainScope);
        mainResource.setPassport(PassportFixture.updateFixture(mainResource, 3));
        mainResource = saveResource(mainResource, mainScope);
        loadResourceFromId(mainResource, mainScope);
    }

    public void testInsertWithoutUsername() throws AssertionError, Exception
    {
        mainResource = ResourceFixture.createFixture(parentPool, 0, mainScope, this);
        mainResource.setUserName(null);
        mainResource = saveResource(mainResource, mainScope);

        // Verify that it generated a random username for us
        RPMObject obj = new Resource();
        obj.setID(mainResource.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Resource);
        assertNotNull(((Resource) rpmObject).getUserName());
    }

    public void testChangePasswordNewResource() throws AssertionError, Exception
    {
        mainScope.setCalendar(new CalendarScope());
        mainResource = ResourceFixture.createFixture(parentPool, mainScope, this);
        Calendar calendar = Calendar.getInstance();
        calendar.set(2005, 5, 1);
        mainResource.setEffectiveStartDate(calendar);
        mainResource.setActive(Boolean.TRUE);

        mainResource = saveResource(mainResource, mainScope);
        
        mainResource.getResourceSecurityRights().setCanUseWebServicesAPI(Boolean.TRUE);
        
        mainResource = (Resource)save(mainResource, mainScope).getRpmObject();

        changePassword(mainResource.getUserName(), "ibmrpm", "toto");
        performLogin(mainResource.getUserName(), "toto");
    }

    public void testMoveResource() throws AssertionError, Exception
    {
        createSaveAndReloadResource();

        PoolScope parentScope = new PoolScope();
        mainScope.setParent(parentScope);
        
        Pool newParentPool = PoolFixture.createSaveFixture(this);
        mainResource.setParent(newParentPool);
        
        Resource saved = saveResource(mainResource, mainScope);

        assertTrue(saved.getParent() != null);
        
        if (saved.getParent().getID().compareTo(newParentPool.getID()) != 0)
            Assert.fail("The resource was not moved to the new pool.");
        
        mainResource = (Resource) loadFromID(mainResource, mainScope);
        
        if (mainResource.getParent().getID().compareTo(newParentPool.getID()) != 0)
            Assert.fail("The resource was not moved to the new pool.");
    }
    
    public void testSecurityGroupLoad() throws Exception
    {
        Resource resource = ResourceFixture.createSaveFixture(this);

        SecurityGroup securityGroup = (SecurityGroup) loadFromXpath(
            "/SecurityGroup[name='Portfolio Manager']", null)[0];
        resource.setSecurityGroup(securityGroup);

        //Saving the proposal
        ResourceScope rscope = new ResourceScope();
        rscope.setSecurityGroup(new SecurityGroupScope());
        SingleResult sr2 = save(resource, rscope,
            ReloadType.ReloadResult);
        assertNotNull(sr2);
        assertTrue(sr2.isWarningFree());
        assertTrue(sr2.isSuccessful());
    }

    private void createSaveAndReloadResource() throws AssertionError, Exception
    {
        createSaveAndReloadResource(1);
    }

    private void createSaveAndReloadResource(int nbChildren) throws AssertionError,
            Exception
    {
        mainResource = ResourceFixture.createFixture(parentPool, nbChildren, mainScope,
            this);

        Resource saved = saveResource(mainResource, mainScope);
        mainResource.setID(saved.getID());

        loadResourceFromId(mainResource, mainScope);
    }
}
