package com.ibm.rpm.client.junit.resource.testcases;

import java.sql.Date;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.EducationFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

public class TestResourceEducation extends ClientTestController
{
    private Resource mainResource;
    private ResourceScope mainResourceScope;

    private Education mainEducation;

    public void setUp() throws Exception
    {
        super.setUp();
        
        mainResource = ResourceFixture.createSaveFixture(this);
        
        mainEducation = EducationFixture.createFixture(mainResource);
        mainResource.setEducations(new Education[] {mainEducation});
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainEducation = saveEducation(mainEducation);

        //load from an id
        RPMObject obj = new Education();
        obj.setID(mainEducation.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Education);
        EducationFixture.compare(mainEducation, (Education) rpmObject);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainEducation = saveEducation(mainEducation);
        //Create a new Education
        Education updatedEducation = EducationFixture.createFixture(mainResource);
        updatedEducation.setAppointedDate(HelperTestUtilities.convertToCalendar(Date
                .valueOf("2010-10-10")));
        updatedEducation.setCertificationGraduateDate(HelperTestUtilities.convertToCalendar(Date
                .valueOf("2022-22-22")));
        //By setting the ID to the mainEducation ID, the updatedEducation represent
        //the mainEducation with all its attribute updated.
        updatedEducation.setID(mainEducation.getID());
        
        //Save the modifications
        updatedEducation = saveEducation(updatedEducation);

        //load from an id
        RPMObject obj = new Education();
        obj.setID(mainEducation.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Education);
        EducationFixture.compare(updatedEducation, (Education) rpmObject);
    }

    private Education saveEducation(Education p) throws AssertionError, Exception
    {
        SingleResult result;
        result = save(p, null);

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof Education);

        p = (Education) result.getRpmObject();
        return p;
    }
}
