package com.ibm.rpm.client.junit.resource.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.EmploymentHistoryFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.EmploymentHistory;
import com.ibm.rpm.resource.containers.Resource;


public class TestResourceEmploymentHistory extends ClientTestController
{
    private Resource mainResource;

    private EmploymentHistory mainEmploymentHistory;

    public void setUp() throws Exception
    {
        super.setUp();
        
        mainResource = ResourceFixture.createSaveFixture(this);

        mainEmploymentHistory = EmploymentHistoryFixture.createFixture(mainResource);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
     	mainEmploymentHistory = saveEmploymentHistory(mainEmploymentHistory);

        //load from an xpath
        RPMObject[] rpmObjects = loadFromXpath("/EmploymentHistory[id='"
                + mainEmploymentHistory.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof EmploymentHistory);
        EmploymentHistoryFixture.compare(mainEmploymentHistory,
                (EmploymentHistory) rpmObjects[0]);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainEmploymentHistory = saveEmploymentHistory(mainEmploymentHistory);

        //load from an id
        RPMObject obj = new EmploymentHistory();
        obj.setID(mainEmploymentHistory.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof EmploymentHistory);
        EmploymentHistoryFixture.compare(mainEmploymentHistory, (EmploymentHistory) rpmObject);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainEmploymentHistory = saveEmploymentHistory(mainEmploymentHistory);
        //Create a new EmploymentHistory
        EmploymentHistory updatedEmploymentHistory = EmploymentHistoryFixture.createFixture(mainResource);
        //By setting the ID to the mainEmploymentHistory ID, the updatedEmploymentHistory represent
        //the mainEmploymentHistory with all its attribute updated.
        updatedEmploymentHistory.setID(mainEmploymentHistory.getID());

        //Save the modifications
        updatedEmploymentHistory = saveEmploymentHistory(updatedEmploymentHistory);

        //load from an id
        RPMObject obj = new EmploymentHistory();
        obj.setID(mainEmploymentHistory.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof EmploymentHistory);
        EmploymentHistoryFixture.compare(updatedEmploymentHistory, (EmploymentHistory) rpmObject);
    }

    private EmploymentHistory saveEmploymentHistory(EmploymentHistory p) throws AssertionError, Exception
    {
        SingleResult result = save(p, null);

        p = (EmploymentHistory) result.getRpmObject();
        return p;
    }
}
