package com.ibm.rpm.client.junit.resource.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.IndustryExperienceFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.IndustryExperience;
import com.ibm.rpm.resource.containers.Resource;


public class TestResourceIndustryExperience extends ClientTestController
{
    private Resource mainResource;

    private IndustryExperience mainIndustryExperience;
    public void setUp() throws Exception
    {
        super.setUp();
        
        mainResource = ResourceFixture.createSaveFixture(this);
        
        mainIndustryExperience = IndustryExperienceFixture.createFixture(mainResource);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        IndustryExperience ie = saveIndustryExperience(mainIndustryExperience);
        
        //load from an xpath
        RPMObject[] rpmObjects = loadFromXpath("/IndustryExperience[id='"
                + ie.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof IndustryExperience);
        
        IndustryExperienceFixture.compare(mainIndustryExperience, (IndustryExperience)rpmObjects[0]);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        IndustryExperience ie = saveIndustryExperience(mainIndustryExperience);

        //load from an id
        RPMObject obj = new IndustryExperience();
        obj.setID(ie.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof IndustryExperience);
        
        IndustryExperienceFixture.compare(mainIndustryExperience, (IndustryExperience) rpmObject);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainIndustryExperience = saveIndustryExperience(mainIndustryExperience);
        //Create a new IndustryExperience
        IndustryExperience updatedIndustryExperience = IndustryExperienceFixture.createFixture(mainResource);
        //By setting the ID to the mainIndustryExperience ID, the updatedIndustryExperience represent
        //the mainIndustryExperience with all its attribute updated.
        updatedIndustryExperience.setID(mainIndustryExperience.getID());

        //Save the modifications
        IndustryExperience ie = saveIndustryExperience(updatedIndustryExperience);

        //load from an id
        RPMObject obj = new IndustryExperience();
        obj.setID(ie.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof IndustryExperience);
        IndustryExperienceFixture.compare(updatedIndustryExperience, (IndustryExperience) rpmObject);
    }

    private IndustryExperience saveIndustryExperience(IndustryExperience p) throws AssertionError, Exception
    {
        SingleResult result;
        result = save(p, null);

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof IndustryExperience);

        p = (IndustryExperience) result.getRpmObject();
        return p;
    }
}
