package com.ibm.rpm.client.junit.resource.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.LanguageProficiencyFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.LanguageProficiency;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

public class TestResourceLanguageProficiency extends ClientTestController
{
    private Resource mainResource;
    private ResourceScope mainScope;

    public void setUp() throws Exception
    {
        super.setUp();
        
        mainResource = ResourceFixture.createSaveFixture(this);
        mainScope = new ResourceScope();
        mainScope.setLanguageProficiencies(true);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        insert(false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(true);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(true);
    }

	public void testUpdateDefaultFlagLoadFromID() throws AssertionError,
	    Exception
	{
	    updateDefaultFlag(true);
	}
	
	public void testInsertWithMultipleDefaults() throws AssertionError,
    	Exception
	{
	    LanguageProficiency [] lp = {
	            LanguageProficiencyFixture.createFixture(mainResource),
	            LanguageProficiencyFixture.createFixture(mainResource),
	            LanguageProficiencyFixture.createFixture(mainResource)
	    };
	    
	    lp[0].setDefaultLanguage(Boolean.TRUE);
	    lp[1].setDefaultLanguage(Boolean.TRUE);
	    lp[2].setDefaultLanguage(Boolean.TRUE);
	   	    
	    mainResource.setLanguageProficiencies(lp);
	    
	    SingleResult result = save(mainResource, mainScope);
	    mainResource = (Resource)result.getRpmObject();
	    
	    // Reload and ensure only one language is set as default
	    
	    mainResource = (Resource) loadFromID(mainResource, mainScope);
	    
	    assertNotNull(mainResource.getLanguageProficiencies());
	    
	    int numberOfDefaultLanguages = 0;
	    for (int i = mainResource.getLanguageProficiencies().length - 1; i >= 0; --i)
	    {
	        assertNotNull(mainResource.getLanguageProficiencies()[i].getDefaultLanguage());
	        if (mainResource.getLanguageProficiencies()[i].getDefaultLanguage().booleanValue() == true)
	            ++numberOfDefaultLanguages;
	    }
	    
	    assertTrue("The Resource's number of default languages should be exactly 1 but is " + numberOfDefaultLanguages, numberOfDefaultLanguages == 1);
	}
    
	private LanguageProficiency insert(boolean loadFromId)throws Exception
	{
	    return insertLanguageProficiency(loadFromId);
	}
	
	private LanguageProficiency update(boolean loadFromId)throws Exception
	{
	    LanguageProficiency lp = insertLanguageProficiency(loadFromId);
	    
	    lp = updateLanguageProficiency(lp, loadFromId);
	    
	    return lp;
	}
	
	private LanguageProficiency updateDefaultFlag(boolean loadFromId)throws Exception
	{
	    LanguageProficiency lp = insertLanguageProficiency(loadFromId);
	    
	    //set default to true
	    lp = updateDefaultFlag(lp, true, loadFromId);
	    
	    //set default to false
	    lp = updateDefaultFlag(lp, false, loadFromId);
	    
	    return lp;
	}
	
	private LanguageProficiency insertLanguageProficiency(boolean loadFromId)throws Exception
	{
		LanguageProficiency lp = LanguageProficiencyFixture.createFixture(mainResource);
		SingleResult result =  save(lp,null);
		
        //compare the returned lp group
		LanguageProficiency returnedLP = (LanguageProficiency)result.getRpmObject();
		LanguageProficiencyFixture.compare(lp, returnedLP);
		
		//reload the lp and compare it
		returnedLP = load(lp, loadFromId); 
		LanguageProficiencyFixture.compare(lp, returnedLP);
		
		//return the lp
		return returnedLP;
	}
	
	private LanguageProficiency updateLanguageProficiency(LanguageProficiency lp, boolean loadFromId)throws Exception
	{
	    //Create a new LanguageProficiency
	    LanguageProficiency updatedLanguageProficiency = LanguageProficiencyFixture.createFixture(mainResource);
	    //By setting the ID to the lp ID, the updatedLanguageProficiency represent
	    //the lp with all its attribute updated.
	    updatedLanguageProficiency.setID(lp.getID());
      
		SingleResult result =  save(updatedLanguageProficiency,null);
		
        //compare the returned lp group
		LanguageProficiency returnedLP = (LanguageProficiency)result.getRpmObject();
		LanguageProficiencyFixture.compare(updatedLanguageProficiency, returnedLP);
		
		//reload the lp and compare it
		returnedLP = load(lp, loadFromId); 
		LanguageProficiencyFixture.compare(updatedLanguageProficiency, returnedLP);
		
		//return the lp
		return returnedLP;
	}
	
	
	private LanguageProficiency updateDefaultFlag(LanguageProficiency lp, boolean defaultFlag, boolean loadFromId)throws Exception
	{
		lp.setDefaultLanguage(new Boolean(defaultFlag));
		SingleResult result =  save(lp,null);
		
        //compare the returned lp group
		LanguageProficiency returnedLP = (LanguageProficiency)result.getRpmObject();
		LanguageProficiencyFixture.compare(lp, returnedLP);
		
		//reload the lp and compare it
		returnedLP = load(lp, loadFromId); 
		LanguageProficiencyFixture.compare(lp, returnedLP);
		
		//return the lp
		return returnedLP;
	}
	
    private LanguageProficiency load(LanguageProficiency lp, boolean loadFromId) throws Exception
	{
    	if(loadFromId){
            //load from an id
            RPMObject obj = new LanguageProficiency();
            obj.setID(lp.getID());
            return (LanguageProficiency)loadFromID(obj, null);
    	}
    		//load from an xpath
        RPMObject[] objs = loadFromXpath("/LanguageProficiency[id='"
                + lp.getID() + "']", null);
        if (objs == null || objs.length == 0)
        {
            return null;
        }
        return (LanguageProficiency)objs[0];
    	
    }
}
