/*
 * Created on Oct 26, 2005
 *
 */
package com.ibm.rpm.client.junit.resource.testcases;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.CitizenshipCountryCode;
import com.ibm.rpm.applicationadministration.containers.Classification;
import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.applicationadministration.containers.EmployeeStatus;
import com.ibm.rpm.applicationadministration.containers.Experience;
import com.ibm.rpm.applicationadministration.containers.IndustryType;
import com.ibm.rpm.applicationadministration.containers.Language;
import com.ibm.rpm.applicationadministration.containers.PreferenceRank;
import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.applicationadministration.containers.WorkLocation;
import com.ibm.rpm.applicationadministration.containers.WorkingStatus;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.framework.testcases.ModuleTestController;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceModuleFixture;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.containers.ResourceModule;
import com.ibm.rpm.resource.scope.ResourceModuleScope;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;

/**
 * @author karim
 *  
 */
public class TestResourceModule extends ModuleTestController
{
    public TestResourceModule(String testName)
    {
        setName(testName);
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = ResourceModuleFixture.createFullScope();
        moduleContaineName = "RESOURCEMODULE";
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testXPathAllCustomFields() throws Exception
    {
        RPMObjectScope scope = CustomFieldFixture.createCustomFieldScope(1);
        RPMObject[] objs = loadFromXpath(
                "/ResourceModule/CustomFieldCategory/CustomField", scope);
        assertNotNull(objs);
    }

    public RPMObject load(AbstractModule aModule, RPMObjectScope aScope)
            throws Exception
    {
        ResourceModule module = (ResourceModule) aModule;
        ResourceModuleScope scope = (ResourceModuleScope) aScope;

        ResourceModule loadedModule = new ResourceModule();

        if(scope!=null)
        {
            if (scope.getResourceAttributeCompetencyCategories()!=null)
            {
                ResourceAttributeCompetencyCategory[] racc = (ResourceAttributeCompetencyCategory[]) loadElements(
                        module.getResourceAttributeCompetencyCategories(), scope
                                .getResourceAttributeCompetencyCategories(),
                        ResourceAttributeCompetencyCategory.class);
                loadedModule.setResourceAttributeCompetencyCategories(racc);
            }
            if (scope.getResourceAttributeSkillCategories()!=null)
            {
                ResourceAttributeSkillCategory[] rasc = (ResourceAttributeSkillCategory[]) loadElements(
                        module.getResourceAttributeSkillCategories(), scope
                                .getResourceAttributeSkillCategories(),
                        ResourceAttributeSkillCategory.class);
                loadedModule.setResourceAttributeSkillCategories(rasc);
            }
            if (scope.getContactGroups()!=null)
            {
                ContactGroup[] contactgroup = (ContactGroup[]) loadElements(module
                        .getContactGroups(), scope.getContactGroups(),
                        ContactGroup.class);
                loadedModule.setContactGroups(contactgroup);
            }
            if (scope.isCitizenshipCountryCodes())
            {
                CitizenshipCountryCode[] cccode = (CitizenshipCountryCode[]) loadDatafields(
                        module.getCitizenshipCountryCodes(), scope
                                .isCitizenshipCountryCodes(),
                        CitizenshipCountryCode.class);
                loadedModule.setCitizenshipCountryCodes(cccode);
            }
            if (scope.isEmployeeStatus())
            {
                EmployeeStatus[] estatus = (EmployeeStatus[]) loadDatafields(module
                        .getEmployeeStatus(), scope.isEmployeeStatus(),
                        EmployeeStatus.class);
                loadedModule.setEmployeeStatus(estatus);
            }
            if (scope.isWorkingStatus())
            {
                WorkingStatus[] wstatus = (WorkingStatus[]) loadDatafields(module
                        .getWorkingStatus(), scope.isWorkingStatus(),
                        WorkingStatus.class);
                loadedModule.setWorkingStatus(wstatus);
            }
            if (scope.isClassifications())
            {
                Classification[] classification = (Classification[]) loadDatafields(
                        module.getClassifications(), scope.isClassifications(),
                        Classification.class);
                loadedModule.setClassifications(classification);
            }
            if (scope.isIndustryTypes())
            {
                IndustryType[] indtype = (IndustryType[]) loadDatafields(module
                        .getIndustryTypes(), scope.isIndustryTypes(),
                        IndustryType.class);
                loadedModule.setIndustryTypes(indtype);
            }
            if (scope.isLanguages())
            {
                Language[] lang = (Language[]) loadDatafields(module.getLanguages(),
                        scope.isLanguages(), Language.class);
                loadedModule.setLanguages(lang);
            }
            if (scope.isProficiencyLevels())
            {
                ProficiencyLevel[] plevel = (ProficiencyLevel[]) loadDatafields(module
                        .getProficiencyLevels(), scope.isProficiencyLevels(),
                        ProficiencyLevel.class);
                loadedModule.setProficiencyLevels(plevel);
            }
            if (scope.isPreferenceRanks())
            {
                PreferenceRank[] prank = (PreferenceRank[]) loadDatafields(module
                        .getPreferenceRanks(), scope.isPreferenceRanks(),
                        PreferenceRank.class);
                loadedModule.setPreferenceRanks(prank);
            }
            if (scope.isExperiences())
            {
                Experience[] exp = (Experience[]) loadDatafields(module
                        .getExperiences(), scope.isExperiences(), Experience.class);
                loadedModule.setExperiences(exp);
            }
            if (scope.isWorkLocations())
            {
                WorkLocation[] wlocations = (WorkLocation[]) loadDatafields(module
                        .getWorkLocations(), scope.isWorkLocations(),
                        WorkLocation.class);
                loadedModule.setWorkLocations(wlocations);
            }
            if (scope.getCustomFieldCategories()!=null)
            {
                CustomFieldCategory[] customFieldCategories = (CustomFieldCategory[]) loadElements(
                        module.getCustomFieldCategories(), scope
                                .getCustomFieldCategories(), CustomFieldCategory.class);
                loadedModule.setCustomFieldCategories(customFieldCategories);
            }
            if(scope.getScorecardFolders()!=null){
                //ScorecardFolders
                ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) loadElements( 
                        module.getScorecardFolders(),
                        scope.getScorecardFolders(),
                        ScorecardFolder.class);
                loadedModule.setScorecardFolders(scorecardFolders);         
            }
        }

        return loadedModule;
    }

    protected void deleteModule(AbstractModule aModule, RPMObjectScope aScope)
            throws AssertionError, Exception
    {
        ResourceModule module = (ResourceModule) aModule;
        ResourceModuleScope scope = (ResourceModuleScope) aScope;
        
        //delete the element
        if (module != null && scope != null)
        {
            if (module.getResourceAttributeCompetencyCategories() != null && scope.getResourceAttributeCompetencyCategories()!=null)
            {
                delete(module.getResourceAttributeCompetencyCategories());                
            }
            if (module.getResourceAttributeSkillCategories() != null && scope.getResourceAttributeSkillCategories()!=null)
            {
                delete(module.getResourceAttributeSkillCategories());
            }
            if (module.getContactGroups() != null && scope.getContactGroups()!=null)
            {
                delete(module.getContactGroups());
            }
            if (module.getCustomFieldCategories() != null && scope.getCustomFieldCategories()!=null)
            {
                delete(module.getCustomFieldCategories());
            }
            if (module.getScorecardFolders() != null && scope.getScorecardFolders()!=null)
            {
                delete(module.getScorecardFolders());
            }
            
            //reload the deleted element
            ResourceModule deletedModule = (ResourceModule) loadDeletedModule(
                    module, scope);

            //Validate the deleted element
            if(scope.getResourceAttributeCompetencyCategories() !=null)
            {
                assertNull(deletedModule.getResourceAttributeCompetencyCategories());   
            }
            if(scope.getResourceAttributeSkillCategories() !=null)
            {
                assertNull(deletedModule.getResourceAttributeSkillCategories());                
            }
            if(scope.getContactGroups() !=null)
            {
                assertNull(deletedModule.getContactGroups());                
            }
            if(scope.getCustomFieldCategories() !=null)
            {
                assertNull(deletedModule.getCustomFieldCategories());                
            }
            if (module.getScorecardFolders() != null && scope.getScorecardFolders()!=null)
            {
                assertNull(deletedModule.getScorecardFolders());
            }
        }
    }

    protected AbstractModule createFixture(RPMObjectScope scope)
            throws Exception
    {
        return ResourceModuleFixture.createFixture((ResourceModuleScope) scope,
                this);
    }

    protected AbstractModule updateFixture(AbstractModule module,
            RPMObjectScope scope) throws Exception
    {
        return ResourceModuleFixture.updateFixture((ResourceModule) module,
                (ResourceModuleScope) scope, this);
    }

    protected void compare(AbstractModule module,
            AbstractModule returnedModule, RPMObjectScope scope)
            throws AssertionFailedError, Exception
    {
        ResourceModuleFixture.compare((ResourceModule) module,
                (ResourceModule) returnedModule, (ResourceModuleScope) scope);
    }
}
