package com.ibm.rpm.client.junit.resource.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.SkillFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;

/**
 * @author karim
 *
 */
public class TestSkill extends ClientTestController
{

    private ResourceAttributeScope mainScope;

    //Original structure
	private int nbClassifications = 1; 
	private int nbSkillsByClass = 2; 
	private int nbProficiencyLevelsBySkill = 2;
    
	//Additional elements in the structure
	private int deltaNbClassifications = 1; //must be >0
	private int deltaNbSkillsByClass = 1; //must be >0 
	private int deltaNbProficiencyLevelsBySkill = 1; //must be >0
	
    public TestSkill(String testName){
        setName(testName);
    }


    public void setUp() throws Exception
    {
        logger.info("setUp");
		super.setUp();
		mainScope = SkillFixture.createScope(nbClassifications, nbSkillsByClass, nbProficiencyLevelsBySkill);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError, Exception
	{
    	insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError, Exception
    {
    	insert(mainScope, true);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError, Exception
    {
    	update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError, Exception
    {
    	update(mainScope, true);
    }

    private void insert(ResourceAttributeScope scope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert an element
    	ResourceAttributeSkillCategory category = insertCategory(scope, loadFromId);
    	
    	//delete the element
    	deleteCategory(category, loadFromId);  
    }
    
    private void update(ResourceAttributeScope scope, boolean loadFromId) throws AssertionError, Exception
	{
        //insert an element
    	ResourceAttributeSkillCategory category = insertCategory(scope, loadFromId);
    	
    	category = updateNamesAndRates(category, scope, loadFromId);
    	
    	category = updateStructure(category, scope, loadFromId);
    	
    	//delete the element
    	deleteCategory(category, loadFromId);  
    }
    
    
    
    private ResourceAttributeSkillCategory insertCategory(ResourceAttributeScope scope, boolean loadFromId) throws Exception
    {
    	ResourceAttributeSkillCategory category = SkillFixture.createFixture(nbClassifications, nbSkillsByClass, nbProficiencyLevelsBySkill, scope, this);
    	       
        //Insert the element into the DB
        SingleResult result =  save(category,scope);

        //compare the returned element
        ResourceAttributeSkillCategory returnedCategory = (ResourceAttributeSkillCategory)result.getRpmObject();
        SkillFixture.compare(category, returnedCategory, scope);
        
        //reload the element and compare it
        returnedCategory = (ResourceAttributeSkillCategory)load(returnedCategory, scope, loadFromId);
        SkillFixture.compare(category, returnedCategory, scope);
        
        //return the assignment
        return returnedCategory;
    }
    
    private ResourceAttributeSkillCategory updateNamesAndRates(ResourceAttributeSkillCategory originalCategory, ResourceAttributeScope scope, boolean loadFromId) throws AssertionError, Exception
	{
    	return update(originalCategory, scope, loadFromId, false);
    }
    
    private ResourceAttributeSkillCategory updateStructure(ResourceAttributeSkillCategory originalCategory, ResourceAttributeScope scope, boolean loadFromId) throws AssertionError, Exception
	{
    	return update(originalCategory, scope, loadFromId, true);
    }
    
    private ResourceAttributeSkillCategory update(ResourceAttributeSkillCategory category, ResourceAttributeScope scope, boolean loadFromId, boolean updateStructure) throws AssertionError, Exception
	{
    	if(updateStructure){
            //updates the structure by adding elements 
        	SkillFixture.updateCategoryFixture(category, deltaNbClassifications, deltaNbSkillsByClass, deltaNbProficiencyLevelsBySkill, scope);    		
    	} else {
    		//only updates the names and the rates
        	SkillFixture.updateCategoryFixture(category, scope);
    	}

        //Insert the element into the DB
        SingleResult result =  save(category,scope);

        //compare the returned element
        ResourceAttributeSkillCategory returnedCategory = (ResourceAttributeSkillCategory)result.getRpmObject();
        SkillFixture.compare(category, returnedCategory, scope);
        
        //reload the element and compare it
        returnedCategory = (ResourceAttributeSkillCategory)load(returnedCategory, scope, loadFromId);
        SkillFixture.compare(category, returnedCategory, scope);
        
        //return the assignment
        return returnedCategory;
    }
    
    private void deleteCategory(ResourceAttributeSkillCategory category, boolean loadFromId)throws Exception
	{    	   	
        //delete the element
       	delete(category);
    	
        //reload the deleted element
       	ResourceAttributeSkillCategory deletedCategory = loadDeletedCategory(category, loadFromId);
        
        //validate deleted element
        assertNull(deletedCategory);
    }
    
    private ResourceAttributeSkillCategory loadDeletedCategory(ResourceAttributeSkillCategory category, boolean loadFromId)throws Exception{
        try{
        	category = (ResourceAttributeSkillCategory)load(category, null, loadFromId);
           	return category;
        }catch (Exception e){
            //element does not exist
        }
        return null;
    }
    
    private RPMObject load(ResourceAttributeSkillCategory category, ResourceAttributeScope scope, boolean loadFromId) throws Exception
	{
    	if(loadFromId){
            //load from an id
            RPMObject obj = new ResourceAttributeSkillCategory();
            obj.setID(category.getID());
            return loadFromID(obj, scope);
    	} else {
    		//load from an xpath
    		RPMObject[] objs = loadFromXpath("/RESOURCEATTRIBUTESKILLCATEGORY[id='"+ category.getID() + "']", scope);
    		if(objs==null || objs.length==0){
    			return null;
    		} else {
    			return objs[0];
    		}
    	}
    }
    
    private ResourceAttributeSkillCategory saveCategory(ResourceAttributeSkillCategory category, ResourceAttributeScope scope) throws AssertionError, Exception
    {
        SingleResult result = save(category, scope);

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof ResourceAttributeSkillCategory);

        ResourceAttributeSkillCategory returnedCategory =  (ResourceAttributeSkillCategory) result.getRpmObject();

        category.setID(returnedCategory.getID());
        
        return returnedCategory;
    }
}
