/*
 * Created on Nov 21, 2006
 * @author mmatwiy
 * Join conditions for loading rates from Resource, ResourceAttributeAssignment, and 
 * ResourceAttributeProficiencyLevel are missing.
 */
package com.ibm.rpm.client.junit.resource.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceAttributeAssignmentFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.scope.ResourceScope;

public class Test57111 extends ClientTestController
{
    public void testLoadRatesFromResource() throws Exception
    {
        // test the join condition
        ResourceScope scope = new ResourceScope();
        scope.setRates(true);
        Resource resource = ResourceFixture.createSaveFixture(scope, this);

        RPMObject[] objects = loadFromXpath("/Resource[username='"
                + resource.getUserName() + "']/@rates", null);

        assertNotNull(objects);
        assertNotNull(objects[0]);
        assertTrue(objects[0] instanceof ResourceRates);

        ResourceAttributeAssignmentFixture.compareRates(
            (ResourceRates) objects[0], resource.getRates());
    }

    public void testLoadResourceWithChildResourceRates() throws Exception
    {
        // test the sub join condition
        ResourceScope scope = new ResourceScope();
        scope.setRates(true);
        Resource resource = ResourceFixture.createSaveFixture(scope, this);

        RPMObject[] objects = loadFromXpath("/Resource[ResourceRates[id='"
                + resource.getRates().getID() + "']]", scope);

        assertNotNull(objects);
        assertNotNull(objects[0]);
        assertTrue(objects[0] instanceof Resource);

        ResourceFixture.compare((Resource) objects[0], resource, scope);
    }

    public void testLoadRatesFromResourceAttributeAssignment() throws Exception
    {
        // test the join condition
        ResourceScope scope = new ResourceScope();
        scope.setSkillAttributeAssignments(true);
        Resource resource = ResourceFixture.createSaveFixture(scope, this);

        RPMObject[] objects = loadFromXpath("/ResourceAttributeAssignment[id='"
                + resource.getSkillAttributeAssignments()[0].getID()
                + "']/@rates", null);

        assertNotNull(objects);
        assertNotNull(objects[0]);
        assertTrue(objects[0] instanceof ResourceRates);

        ResourceAttributeAssignmentFixture.compareRates(
            (ResourceRates) objects[0],
            resource.getSkillAttributeAssignments()[0].getRates());
    }

    public void testLoadResourceAttributeAssignmentWithChildResourceRates()
            throws Exception
    {
        // test the sub join condition
        ResourceScope scope = new ResourceScope();
        scope.setSkillAttributeAssignments(true);
        Resource resource = ResourceFixture.createSaveFixture(scope, this);

        RPMObject[] objects = loadFromXpath(
            "/Resource[@skillAttributeAssignments[ResourceRates[id='"
                    + resource.getSkillAttributeAssignments()[0].getRates()
                        .getID() + "']]]", scope);

        assertNotNull(objects);
        assertNotNull(objects[0]);
        assertTrue(objects[0] instanceof Resource);

        ResourceFixture.compare((Resource) objects[0], resource, scope);
    }

    public void testLoadRatesFromResourceAttributeProficiencyLevel()
            throws Exception
    {
        // test the join condition
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setSkillAttributeAssignments(true);
        Resource resource = ResourceFixture.createSaveFixture(resourceScope,
            this);

        RPMObject[] objects = loadFromXpath("/ResourceAttributeAssignment[id='"
                + resource.getSkillAttributeAssignments()[0].getID()
                + "']/ResourceAttributeSkillProficiencyLevel/@rates", null);

        assertNotNull(objects);
        assertNotNull(objects[0]);
        assertTrue(objects[0] instanceof ResourceRates);

        ResourceAttributeAssignmentFixture.compareRates(
            (ResourceRates) objects[0], resource.getSkillAttributeAssignments()[0].getRates());
    }

    public void testLoadResourceAttributeProficiencyLevelWithChildResourceRates()
            throws Exception
    {
        // test the sub-join condition
        ResourceScope scope = new ResourceScope();
        scope.setSkillAttributeAssignments(true);
        Resource resource = ResourceFixture.createSaveFixture(scope, this);

        RPMObject[] objects = loadFromXpath(
            "/Resource[@skillAttributeAssignments[ResourceAttributeSkillProficiencyLevel[ResourceRates[id='"
                    + resource.getSkillAttributeAssignments()[0]
                        .getProficiencyLevel().getRates().getID() + "']]]]",
            scope);

        assertNotNull(objects);
        assertNotNull(objects[0]);
        assertTrue(objects[0] instanceof Resource);

        ResourceFixture.compare((Resource) objects[0], resource, scope);
    }
}
