package com.ibm.rpm.client.junit.resource.testcases.validation;

import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.CompetencyFixture;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;

/**
 * @author dominiqu
 */
public class TestCompetencyValidation extends GenericValidation
{
    private ResourceAttributeScope mainScope;

    //Original structure
    private int nbClassifications = 1;

    private int nbCompetencysByClass = 2;

    private int nbProficiencyLevelsByCompetency = 2;

    ResourceAttributeCompetencyCategory mainCategory;

    public void setUp() throws Exception
    {
        super.setUp();
        mainScope = CompetencyFixture.createScope(nbClassifications,
            nbCompetencysByClass, nbProficiencyLevelsByCompetency);
        mainCategory = CompetencyFixture.createFixture(nbClassifications,
            nbCompetencysByClass, nbProficiencyLevelsByCompetency, mainScope,
            this);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testSuccessWithWarningForOverridenName() throws AssertionError,
            Exception
    {
        mainCategory.getClassifications()[0].getAttributes()[0].getProficiencyLevels()[0].setName(getTestString(ValidationConstants.RESOURCEATTRIBUTE_NAME_MAX));

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.resource.constants.ErrorCodes.WARNING_NAME_OVERRIDEN_BY_PROFICIENCY_LEVEL,
                mainCategory.getClassifications()[0].getAttributes()[0].getProficiencyLevels()[0].getClass().getName(),
                ValidationConstants.RESOURCEATTRIBUTE_NAME_FIELD, null)
        };

        verifySaveMustFail(mainCategory, mainScope, null, expectedWarnings);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        mainCategory.getClassifications()[0].getAttributes()[0].getProficiencyLevels()[0].setName(getTestString(ValidationConstants.RESOURCEATTRIBUTE_NAME_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                ErrorCodes.MAX_LENGTH,
                mainCategory.getClassifications()[0].getAttributes()[0].getProficiencyLevels()[0].getClass().getName(),
                ValidationConstants.RESOURCEATTRIBUTE_NAME_FIELD, null)
        };

        verifySaveMustFail(mainCategory, mainScope, expectedExceptions);
    }

    public void testFailureWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        mainCategory.getClassifications()[0].getAttributes()[0].getProficiencyLevels()[0].setName(null);
        mainCategory.getClassifications()[0].getAttributes()[0].getProficiencyLevels()[0].setProficiencyLevel(new ProficiencyLevel());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                ErrorCodes.NO_ID,
                mainCategory.getClassifications()[0].getAttributes()[0].getProficiencyLevels()[0].getClass().getName(),
                ValidationConstants.RESOURCEATTRIBUTECOMPETENCYPROFICIENCYLEVEL_PROFICIENCY_LEVEL_FIELD,
                null)
        };

        verifySaveMustFail(mainCategory, mainScope, expectedExceptions);
    }
}
