package com.ibm.rpm.client.junit.resource.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PassportFixture;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Passport;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author dominiqu
 */
public class TestPassportValidation extends GenericValidation
{
    private ResourceScope mainScope;

    private Pool parentPool;

    private Resource mainResource;

    private Passport mainPassport;

    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ResourceScope();
        mainScope.setPassport(true);
        parentPool = PoolFixture.createSaveFixture(this);
        mainResource = ResourceFixture.createFixture(parentPool, mainScope, this);
        mainResource.setParent(parentPool);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    // Note: testFailureWithoutParent and testFailureWithInvalidParent are not
    // significant
    // here, because it is impossible to save a Passport without saving a
    // Resource.
    // savePassportandLocationPreferences sets the parent to the correct
    // resource parent
    // before calling internalSave and performing validation.

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Create Passport with the maximum number of visas
        mainPassport = PassportFixture.createFixture(mainResource, new Integer(
            ValidationConstants.PASSPORT_VISAS_MAX));
        mainResource.setPassport(mainPassport);

        // Attempt to save the resource with Passport
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class, mainScope);

        // Load the inserted resource from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[id='" + mainResource.getID()
            + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);
        assertNotNull(((Resource) rpmObjects[0]).getPassport());

        // Check that the loaded passport is identical to the one we requested
        // to store
        PassportFixture.compare(mainPassport, ((Resource) rpmObjects[0]).getPassport());
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Create Passport with maximum+1 number of visas
        mainPassport = PassportFixture.createFixture(mainResource, new Integer(
            ValidationConstants.PASSPORT_VISAS_MAX + 1));
        mainResource.setPassport(mainPassport);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_SIZE, mainPassport.getClass().getName(),
                "visas", mainPassport.getID())
        };

        verifySaveMustFail(mainResource, mainScope, expectedExceptions);
    }

    public void testFailureValidationNull() throws AssertionError, Exception
    {
        // Create Passport with the maximum number of locations
        mainPassport = PassportFixture.createFixture(mainResource, new Integer(
            ValidationConstants.PASSPORT_VISAS_MAX + 1));

        // Set the Boolean value to false - no children - and verify that the
        // saving fails with children
        mainPassport.setHasValidPassport(Boolean.FALSE);
        mainResource.setPassport(mainPassport);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MUST_BE_NULL, mainPassport.getClass().getName(),
                "visas", mainPassport.getID())
        };

        verifySaveMustFail(mainResource, mainScope, expectedExceptions);
    }
}
