/*
 * Created on 6-Oct-2005
 */

package com.ibm.rpm.client.junit.resource.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.PoolReferenceFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolReference;
import com.ibm.rpm.resource.scope.PoolReferenceScope;

/**
 * @author dominiqu
 */
public class TestPoolReferenceValidation extends GenericValidation
{
    private PoolReference mainReference;

    private PoolReferenceScope mainScope;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainScope = new PoolReferenceScope();
        mainScope.setCompanySize(false);

        Pool pool = PoolFixture.createFixture(false, 0, null, this);
        pool = (Pool) (save(pool, null)).getRpmObject();

        mainReference = PoolReferenceFixture.createFixture(pool, mainScope);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainReference.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainReference.getClass().getName(),
                "parent", mainReference.getID())
        };

        verifySaveMustFail(mainReference, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        mainReference.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainReference.getClass()
                .getName(), "parent", mainReference.getID())
        };

        verifySaveMustFail(mainReference, null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        mainReference
            .setContactEmail(getTestString(ValidationConstants.REFERENCE_CONTACTEMAIL_MAX));
        mainReference
            .setContactFax(getTestString(ValidationConstants.REFERENCE_CONTACTFAX_MAX));
        mainReference
            .setContactName(getTestString(ValidationConstants.REFERENCE_CONTACTNAME_MAX));
        mainReference
            .setContactTelephone(getTestString(ValidationConstants.REFERENCE_CONTACTTELEPHONE_MAX));
        mainReference
            .setOrganizationAddress(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONADDRESS_MAX));
        mainReference
            .setOrganizationAddress(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONADDRESS_MAX));
        mainReference
            .setOrganizationFax(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONFAX_MAX));
        mainReference
            .setOrganizationName(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONNAME_MAX));
        mainReference
            .setOrganizationTelephone(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONTELEPHONE_MAX));
        mainReference
            .setOrganizationWebsite(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONWEBSITE_MAX));

        // Attempt to save the Reference object
        mainReference = (PoolReference) saveMustSucceed(mainReference,
            PoolReference.class, null);

        // Load the inserted reference from xpath
        // Load the inserted education from id
        RPMObject obj = new PoolReference();
        obj.setID(mainReference.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof PoolReference);

        // Check that the loaded reference is identical to the one we requested
        // to store
        PoolReferenceFixture.compare(mainReference, (PoolReference) rpmObject, null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        mainReference
            .setContactEmail(getTestString(ValidationConstants.REFERENCE_CONTACTEMAIL_MAX + 1));
        mainReference
            .setContactFax(getTestString(ValidationConstants.REFERENCE_CONTACTFAX_MAX + 1));
        mainReference
            .setContactName(getTestString(ValidationConstants.REFERENCE_CONTACTNAME_MAX + 1));
        mainReference
            .setContactTelephone(getTestString(ValidationConstants.REFERENCE_CONTACTTELEPHONE_MAX + 1));
        mainReference
            .setOrganizationAddress(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONADDRESS_MAX + 1));
        mainReference
            .setOrganizationAddress(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONADDRESS_MAX + 1));
        mainReference
            .setOrganizationFax(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONFAX_MAX + 1));
        mainReference
            .setOrganizationName(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONNAME_MAX + 1));
        mainReference
            .setOrganizationTelephone(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONTELEPHONE_MAX + 1));
        mainReference
            .setOrganizationWebsite(getTestString(ValidationConstants.REFERENCE_ORGANIZATIONWEBSITE_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainReference.getClass().getName(),
                "contactEmail", mainReference.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainReference.getClass().getName(),
                "contactFax", mainReference.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainReference.getClass().getName(),
                "contactName", mainReference.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainReference.getClass().getName(),
                "contactTelephone", mainReference.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainReference.getClass().getName(),
                "organizationAddress", mainReference.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainReference.getClass().getName(),
                "organizationFax", mainReference.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainReference.getClass().getName(),
                "organizationName", mainReference.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainReference.getClass().getName(),
                "organizationTelephone", mainReference.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainReference.getClass().getName(),
                "organizationWebsite", mainReference.getID())
        };

        verifySaveMustFail(mainReference, null, expectedExceptions);
    }
}
