/*
 * Created on 5-Oct-2005
 */
package com.ibm.rpm.client.junit.resource.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Pool;

/**
 * @author dominiqu
 */
public class TestPoolValidation extends GenericValidation
{
    private Pool mainPool;

    private Pool parent = null;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainPool = PoolFixture.createFixture(false, 0, null, this);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();

        mainPool.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainPool.getClass()
                .getName(), "parent", mainPool.getID())
        };

        verifySaveMustFail(mainPool, null, expectedExceptions);
    }

    public void testFailureInvalidChars() throws AssertionError, Exception
    {
        mainPool
            .setName(com.ibm.rpm.framework.constants.ValidationConstants.GENERIC_INVALIDCHARS);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_CHARS, mainPool.getClass().getName(),
                "name", mainPool.getID())
        };

        verifySaveMustFail(mainPool, null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        mainPool.setAddress(getTestString(ValidationConstants.POOL_ADDRESS_MAX));
        mainPool
            .setCompanyBackground(getTestString(ValidationConstants.POOL_COMPANYBACKGROUND_MAX));
        mainPool.setContactName(getTestString(ValidationConstants.POOL_CONTACTNAME_MAX));
        mainPool.setDescription(getTestString(ValidationConstants.POOL_DESCRIPTION_MAX));
        mainPool.setEmail(getTestString(ValidationConstants.POOL_EMAIL_MAX));
        mainPool.setFax(getTestString(ValidationConstants.POOL_FAX_MAX));
        mainPool
            .setIndustriesServed(getTestString(ValidationConstants.POOL_INDUSTRIESSERVED_MAX));
        mainPool
            .setLocationsServed(getTestString(ValidationConstants.POOL_LOCATIONSSERVED_MAX));
        mainPool
            .setOrganizationName(getTestString(ValidationConstants.POOL_ORGANIZATIONNAME_MAX));
        mainPool.setOwnership(getTestString(ValidationConstants.POOL_OWNERSHIP_MAX));
        mainPool
            .setProductsExpertiseUsed(getTestString(ValidationConstants.POOL_PRODUCTSEXPERTISEUSED_MAX));
        mainPool
            .setServicesOffered(getTestString(ValidationConstants.POOL_SERVICESOFFERED_MAX));
        mainPool
            .setServicesSpecialized(getTestString(ValidationConstants.POOL_SERVICESSPECIALIZED_MAX));
        mainPool.setTelephone(getTestString(ValidationConstants.POOL_TELEPHONE_MAX));
        createFixedLengthStringAlpha(ValidationConstants.RESOURCE_PASSWORD_MAX);

        // Set name with valid characters
        mainPool.setName(createFixedLengthStringAlpha(ValidationConstants.POOL_NAME_MAX));

        // Attempt to save the Pool object
        Pool saved = (Pool) saveMustSucceed(mainPool, Pool.class, null);
        mainPool.setID(saved.getID());

        // Load the inserted pool from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Pool[Id='" + mainPool.getID() + "']",
            null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Pool);

        // Check that the loaded pool is identical to the one we requested to
        // store
        PoolFixture.compare(mainPool, (Pool) rpmObjects[0], null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        mainPool.setAddress(getTestString(ValidationConstants.POOL_ADDRESS_MAX + 1));
        mainPool
            .setCompanyBackground(getTestString(ValidationConstants.POOL_COMPANYBACKGROUND_MAX + 1));
        mainPool
            .setContactName(getTestString(ValidationConstants.POOL_CONTACTNAME_MAX + 1));
        mainPool
            .setDescription(getTestString(ValidationConstants.POOL_DESCRIPTION_MAX + 1));
        mainPool.setEmail(getTestString(ValidationConstants.POOL_EMAIL_MAX + 1));
        mainPool.setFax(getTestString(ValidationConstants.POOL_FAX_MAX + 1));
        mainPool
            .setIndustriesServed(getTestString(ValidationConstants.POOL_INDUSTRIESSERVED_MAX + 1));
        mainPool
            .setLocationsServed(getTestString(ValidationConstants.POOL_LOCATIONSSERVED_MAX + 1));
        mainPool
            .setOrganizationName(getTestString(ValidationConstants.POOL_ORGANIZATIONNAME_MAX + 1));
        mainPool.setOwnership(getTestString(ValidationConstants.POOL_OWNERSHIP_MAX + 1));
        mainPool
            .setProductsExpertiseUsed(getTestString(ValidationConstants.POOL_PRODUCTSEXPERTISEUSED_MAX + 1));
        mainPool
            .setServicesOffered(getTestString(ValidationConstants.POOL_SERVICESOFFERED_MAX + 1));
        mainPool
            .setServicesSpecialized(getTestString(ValidationConstants.POOL_SERVICESSPECIALIZED_MAX + 1));
        mainPool.setTelephone(getTestString(ValidationConstants.POOL_TELEPHONE_MAX + 1));

        mainPool
            .setName(createFixedLengthStringAlpha(ValidationConstants.POOL_NAME_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "address", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "companyBackground", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "contactName", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "description", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "email", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "fax", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "industriesServed", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "locationsServed", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "name", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "organizationName", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "ownership", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "productsExpertiseUsed", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "servicesOffered", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "servicesSpecialized", mainPool.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainPool.getClass().getName(),
                "telephone", mainPool.getID())
        };

        verifySaveMustFail(mainPool, null, expectedExceptions);
    }
        
    public void testDefaultValuesInsert() throws AssertionError, Exception
    {
        Pool pool = new Pool();
        pool.setName("test pool");

        // Leave default values as null and save
        pool = (Pool) saveMustSucceed(pool, Pool.class, null);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Pool[id='" + pool.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Pool);

        // Check that the default values are now set
        Pool savedPool = (Pool) rpmObjects[0];
        assertNotNull(savedPool.getMaskEmployeeInfo());
        assertNotNull(savedPool.getPublish());

        // Save again to validate the default values
        pool = (Pool) saveMustSucceed(pool, Pool.class, null);

        // Load the inserted project from xpath
        rpmObjects = loadFromXpath("/Pool[id='" + pool.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Pool);

        // Check that the loaded Project is identical to the one we
        // requested to store
        PoolFixture.compare(pool, (Pool) rpmObjects[0], null);
    }

    public void testDefaultValuesUpdateToNull() throws AssertionError,
            Exception
    {
        Pool pool = new Pool();
        pool.setName("pool test name");

        // Leave default values as null and save
        pool = (Pool) saveMustSucceed(pool, Pool.class, null);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Pool[id='" + pool.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Pool);

        // Check that the default values are now set
        Pool savedPool = (Pool) rpmObjects[0];
        assertNotNull(savedPool.getMaskEmployeeInfo());
        assertNotNull(savedPool.getPublish());

        // Update the default values to null
        savedPool.setMaskEmployeeInfo(null);
        savedPool.setPublish(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedPool
                    .getClass().getName(),
                    ValidationConstants.POOL_MASK_EMPLOYEE_INFO_FIELD, savedPool
                        .getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedPool
                    .getClass().getName(), ValidationConstants.POOL_PUBLISH_FIELD,
                    savedPool.getID())
        };

        verifySaveMustFail(savedPool, null, expectedExceptions);

    }
}
