/*
 * Created on Oct 26, 2005
 *
 */
package com.ibm.rpm.client.junit.scope.testcases;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.ActionRTF;
import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestRTF;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestState;
import com.ibm.rpm.applicationadministration.containers.DefectRTF;
import com.ibm.rpm.applicationadministration.containers.DefectState;
import com.ibm.rpm.applicationadministration.containers.DeliverableRTF;
import com.ibm.rpm.applicationadministration.containers.IssueRTF;
import com.ibm.rpm.applicationadministration.containers.IssueState;
import com.ibm.rpm.applicationadministration.containers.MilestoneRTF;
import com.ibm.rpm.applicationadministration.containers.RequirementRTF;
import com.ibm.rpm.applicationadministration.containers.RequirementState;
import com.ibm.rpm.applicationadministration.containers.RiskRTF;
import com.ibm.rpm.applicationadministration.containers.RiskState;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestRTF;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestState;
import com.ibm.rpm.applicationadministration.containers.SummaryTaskRTF;
import com.ibm.rpm.applicationadministration.containers.TaskRTF;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.framework.testcases.ModuleTestController;
import com.ibm.rpm.client.junit.scope.fixtures.ScopeManagementModuleFixture;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.scope.ScopeManagementModuleScope;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author karim
 *  
 */
public class TestScopeManagementModule extends ModuleTestController
{
    private static final String SCOPE_CONTEXT = "Scope";
    
	public TestScopeManagementModule(String testName)
	{
	    setName(testName);
	}
	
	public void setUp() throws Exception
	{
	    logger.info("setUp");
	    super.setUp();
	
	    mainScope = ScopeManagementModuleFixture.createFullScope();
	    moduleContaineName = "SCOPEMANAGEMENTMODULE";
	}
	
	public void tearDown() throws Exception
	{
	    super.tearDown();
	}
	
public void testXPathAllCustomFields() throws Exception
    {
    	// Create a custom field so that the module has something to load.
    	CustomFieldFixture.createSaveCustomField(SCOPE_CONTEXT, FieldInputType.Date, this);
        RPMObjectScope scope = CustomFieldFixture.createCustomFieldScope(1);
        RPMObject[] objs = loadFromXpath(
                "/ScopeManagementModule/CustomFieldCategory/CustomField", scope);
        assertNotNull(objs);
    }


	public RPMObject load(AbstractModule aModule, RPMObjectScope aScope)
	        throws Exception
	{
	
	    ScopeManagementModule module = (ScopeManagementModule) aModule;
	    ScopeManagementModuleScope scope = (ScopeManagementModuleScope) aScope;
	
	    ScopeManagementModule loadedModule = new ScopeManagementModule();
	    
        if(scope!=null)
        {
            if(scope.getScopeFolders()!=null)
            {
        	    //loadScopeFolders
        	    ScopeFolder[] scopeFolders = (ScopeFolder[]) loadElements( 
        	    		module.getScopeFolders(),
        				scope.getScopeFolders(),
        				ScopeFolder.class);
        	    loadedModule.setScopeFolders(scopeFolders);
            }
            if(scope.getAttributeCategories()!=null)
            {
        	    //loadAttributeCategories
        	    AttributeCategory[] attributeCategories = loadAttributeCategories(
        	            module.getAttributeCategories(), scope.getAttributeCategories());
        	    loadedModule.setAttributeCategories(attributeCategories);
            }
            if(scope.getCustomFieldCategories()!=null)
            {
        	    //loadCustomFieldCategories
        	    CustomFieldCategory[] customFieldCategories = (CustomFieldCategory[]) loadElements(
                        module.getCustomFieldCategories(), scope
                                .getCustomFieldCategories(), CustomFieldCategory.class);
                loadedModule.setCustomFieldCategories(customFieldCategories);
            }
            if(scope.getScorecardFolders()!=null)
            {
        	    //loadScorecardFolders
        	    ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) loadElements( 
        	    		module.getScorecardFolders(),
        				scope.getScorecardFolders(),
        				ScorecardFolder.class);
        	    loadedModule.setScorecardFolders(scorecardFolders);
            }
            if(scope.isChangeRequestStates())
            {
        	    //loadChangeRequestStates
        	    ChangeRequestState[] changeRequestStates = (ChangeRequestState[]) loadDatafields(
        	    		module.getChangeRequestStates(), 
        				scope.isChangeRequestStates(),
        				ChangeRequestState.class);
        	    loadedModule.setChangeRequestStates(changeRequestStates);
            }
            if(scope.isIssueStates())
            {
        	    //loadIssueStates
        	    IssueState[] issueStates = (IssueState[]) loadDatafields(
        	    		module.getIssueStates(), 
        				scope.isIssueStates(),
        				IssueState.class);
        	    loadedModule.setIssueStates(issueStates);
            }
            if(scope.isRiskStates())
            {
        	    //loadRiskStates
        	    RiskState[] riskStates = (RiskState[]) loadDatafields(
        	    		module.getRiskStates(), 
        				scope.isRiskStates(),
        				RiskState.class);
        	    loadedModule.setRiskStates(riskStates);
            }
            if(scope.isRequirementStates())
            {
        	    //loadRequirementStates
        	    RequirementState[] requirementStates = (RequirementState[]) loadDatafields(
        	    		module.getRequirementStates(), 
        				scope.isRequirementStates(),
        				RequirementState.class);
        	    loadedModule.setRequirementStates(requirementStates);
            }
            if(scope.isDefectStates())
            {
        	    //loadDefectStates        
        	    DefectState[] defectStates = (DefectState[]) loadDatafields(
        	    		module.getDefectStates(), 
        				scope.isDefectStates(),
        				DefectState.class);
        	    loadedModule.setDefectStates(defectStates);
            }
            if(scope.isServiceRequestStates())
            {
        	    //loadServiceRequestStates        
        	    ServiceRequestState[] serviceRequestStates = (ServiceRequestState[]) loadDatafields(
        	    		module.getServiceRequestStates(), 
        				scope.isServiceRequestStates(),
        				ServiceRequestState.class);
        	    loadedModule.setServiceRequestStates(serviceRequestStates);
            }
            if(scope.isActionStates())
            {
        	    //loadActionStates        
        	    ActionState[] actionStates = (ActionState[]) loadDatafields(
        	    		module.getActionStates(), 
        				scope.isActionStates(),
        				ActionState.class);
        	    loadedModule.setActionStates(actionStates);
            }
            if(scope.isDeliverableRtfs())
            {
        	    //loadDeliverableRTF        
        	    DeliverableRTF[] deliverableRTFs = (DeliverableRTF[]) loadDatafields(
        	    		module.getDeliverableRtfs(), 
        				scope.isDeliverableRtfs(),
        				DeliverableRTF.class);
        	    loadedModule.setDeliverableRtfs(deliverableRTFs);
            }
            if(scope.isSummaryTaskRtfs())
            {
        	    //loadSummaryTaskRTF        
        	    SummaryTaskRTF[] summaryTaskRTFs = (SummaryTaskRTF[]) loadDatafields(
        	    		module.getSummaryTaskRtfs(), 
        				scope.isSummaryTaskRtfs(),
        				SummaryTaskRTF.class);
        	    loadedModule.setSummaryTaskRtfs(summaryTaskRTFs);
            }
            if(scope.isMilestoneRtfs())
            {
        	    //loadMilestoneRTF
        	    MilestoneRTF[] milestoneRTFs = (MilestoneRTF[]) loadDatafields(
        	    		module.getMilestoneRtfs(),
        				scope.isMilestoneRtfs(),
        				MilestoneRTF.class);
        	    loadedModule.setMilestoneRtfs(milestoneRTFs);
            }
            if(scope.isTaskRtfs())
            {
        	    //loadTaskRTF
        	    TaskRTF[] taskRTFs = (TaskRTF[]) loadDatafields(
        	    		module.getTaskRtfs(),
        				scope.isTaskRtfs(),
        				TaskRTF.class);
        	    loadedModule.setTaskRtfs(taskRTFs);
            }
            if(scope.isRequirementRtfs())
            {
        	    //loadRequirementRTF
        	    RequirementRTF[] requirementRTFs = (RequirementRTF[]) loadDatafields(
        	    		module.getRequirementRtfs(),
        				scope.isRequirementRtfs(),
        				RequirementRTF.class);
        	    loadedModule.setRequirementRtfs(requirementRTFs);
            }
            if(scope.isChangeRequestRtfs())
            {
        	    //loadChangeRequestRTF
        	    ChangeRequestRTF[] changeRequestRTFs = (ChangeRequestRTF[]) loadDatafields(
        	    		module.getChangeRequestRtfs(),
        				scope.isChangeRequestRtfs(),
        				ChangeRequestRTF.class);
        	    loadedModule.setChangeRequestRtfs(changeRequestRTFs);
            }
            if(scope.isServiceRequestRtfs())
            {
        	    //loadServiceRequestRTF
        	    ServiceRequestRTF[] serviceRequestRTFs = (ServiceRequestRTF[]) loadDatafields(
        	    		module.getServiceRequestRtfs(),
        				scope.isServiceRequestRtfs(),
        				ServiceRequestRTF.class);
        	    loadedModule.setServiceRequestRtfs(serviceRequestRTFs);
            }
            if(scope.isDefectRtfs())
            {
        	    //loadDefectRTF
        	    DefectRTF[] defectRTFs = (DefectRTF[]) loadDatafields(
        	    		module.getDefectRtfs(),
        				scope.isDefectRtfs(),
        				DefectRTF.class);
        	    loadedModule.setDefectRtfs(defectRTFs);
            }
            if(scope.isIssueRtfs())
            {
        	    //loadIssueRTF
        	    IssueRTF[] issueRTFs = (IssueRTF[]) loadDatafields(
        	    		module.getIssueRtfs(),
        				scope.isIssueRtfs(),
        				IssueRTF.class);
        	    loadedModule.setIssueRtfs(issueRTFs);
            }
            if(scope.isRiskRtfs())
            {
        	    //loadRiskRTF
        	    RiskRTF[] riskRTFs = (RiskRTF[]) loadDatafields(
        	    		module.getRiskRtfs(),
        				scope.isRiskRtfs(),
        				RiskRTF.class);
        	    loadedModule.setRiskRtfs(riskRTFs);
            }
            if(scope.isActionRtfs())
            {
        	    //loadActionRTF
        	    ActionRTF[] actionRTFs = (ActionRTF[]) loadDatafields(
        	    		module.getActionRtfs(),
        				scope.isActionRtfs(),
        				ActionRTF.class);
        	    loadedModule.setActionRtfs(actionRTFs);
            }
        }  
        
	    return loadedModule;
	}
	
	protected void deleteModule(AbstractModule aModule, RPMObjectScope aScope)
	        throws AssertionError, Exception
	{
	
	    ScopeManagementModule module = (ScopeManagementModule) aModule;
	    ScopeManagementModuleScope scope = (ScopeManagementModuleScope) aScope;
	    
	    //delete the element
	    if (module != null && scope != null)
	    {
	        if (module.getScopeFolders() != null && scope.getScopeFolders()!=null)
	        {
	            deleteScopeFolders(module.getScopeFolders());
	        }
	        if (module.getAttributeCategories() != null && scope.getAttributeCategories()!=null)
	        {
	            delete(module.getAttributeCategories());
	        }
	        if (module.getCustomFieldCategories() != null && scope.getCustomFieldCategories()!=null)
        	{
        	    delete(module.getCustomFieldCategories());
        	}
	        if (module.getScorecardFolders() != null && scope.getScorecardFolders()!=null)
	        {
	            delete(module.getScorecardFolders());
	        }
	        
		    //reload the deleted element
		    ScopeManagementModule deletedModule = (ScopeManagementModule) loadDeletedModule(module, scope);
		
		    //Validate the deleted element
		    if(scope.getScopeFolders()!=null)
		    {
			    assertNull(deletedModule.getScopeFolders());		        
		    }
		    if(scope.getAttributeCategories()!=null)
		    {
			    assertNull(deletedModule.getAttributeCategories());		        
		    }
		    if(scope.getCustomFieldCategories()!=null)
		    {
			    assertNull(deletedModule.getCustomFieldCategories());   
		    }
		    if(scope.getScorecardFolders()!=null)
		    {
			    assertNull(deletedModule.getScorecardFolders());		        
		    }
	    }
	}
	
	protected AbstractModule createFixture(RPMObjectScope scope)
	        throws Exception
	{
	    return ScopeManagementModuleFixture.createFixture((ScopeManagementModuleScope) scope, this);
	}
	
	protected AbstractModule updateFixture(AbstractModule module,
	        RPMObjectScope scope) throws Exception
	{
	    return ScopeManagementModuleFixture.updateFixture((ScopeManagementModule) module, (ScopeManagementModuleScope) scope, this);
	}
	
	protected void compare(AbstractModule module,
	        AbstractModule returnedModule, RPMObjectScope scope)
	        throws AssertionFailedError, Exception
	{
	    ScopeManagementModuleFixture.compare((ScopeManagementModule) module,
	        (ScopeManagementModule) returnedModule, (ScopeManagementModuleScope) scope);
	}   
	
	private void deleteScopeFolders(ScopeFolder[] scopeFolders)
	{
	    if(scopeFolders!=null)
	    {
	        for(int i=0;i<scopeFolders.length;i++)
	        {
	            try
	            {
	  	          RPMObject[] rpmObjects = loadFromXpath("/Proposal[name='"
	  	  	          + scopeFolders[i].getName() + "']", new WorkElementScope());
	  	          if(rpmObjects!=null && rpmObjects.length!=0)
	  	          {
	  	              //delete first row
	  	              delete(rpmObjects[0]);
	  	          }
	            }catch(Exception e)
	            {
	                //do nothing
	            }
	        }
	    }
	}
}