/*
 * Created on Sep 28, 2006
 *
 */
package com.ibm.rpm.client.junit.scopemanagement.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.scopemanagement.containers.ReqProImportStatus;
import com.ibm.rpm.scopemanagement.containers.ReqProRequirement;
import com.ibm.rpm.scopemanagement.containers.ReqProScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;

/**
 * @author karim
 *
 */
public class ReqProScopeFolderFixture extends RPMObjectFixture
{
	private static int DEFAULT_NB_REQPROSCOPEFOLDER_LEVELS = 1;
	private static int DEFAULT_NB_REQPROSCOPEFOLDERS_BY_LEVEL = 1;
	private static int DEFAULT_NB_REQPROREQUIREMENTS_BY_LEVEL = 1;
	
    protected static ArrayList localSkipGetters = new ArrayList();
    protected static ArrayList localSkipSetters = new ArrayList();
    static
    {
        // add additional methods to be skipped
        localSkipGetters.addAll(skipGetters);
        localSkipGetters.add("getWbsSortingRank");
        localSkipGetters.add("getParentSortingRank");
        localSkipGetters.add("getTemplateSortingRank");
    }
    
    public static ReqProScopeFolder createFixture(RPMObject parent,
    		ScopeElementScope scope) throws Exception
    {
        return createFixture(parent, 0, DEFAULT_NB_REQPROSCOPEFOLDER_LEVELS, DEFAULT_NB_REQPROSCOPEFOLDERS_BY_LEVEL, DEFAULT_NB_REQPROREQUIREMENTS_BY_LEVEL, scope);
    }
	
    public static ReqProScopeFolder createFixture(RPMObject parent,
			int currentLevel, int nbLevels, int nbReqProScopeFoldersByLevel,
			int nbReqProRequirementsByLevel, ScopeElementScope scope) throws Exception
    {
        return createReqProScopeFolder(parent, random(), currentLevel, nbLevels,
    			nbReqProScopeFoldersByLevel, nbReqProRequirementsByLevel, scope);
    }
    
    public static void updateFixture(ReqProScopeFolder reqProScopeFolder,
			ScopeElementScope scope) throws Exception
    {
        updateReqProScopeFolder(reqProScopeFolder, random(), scope);
    }
    
    private static ReqProScopeFolder createReqProScopeFolder(RPMObject parent,
			int index, int currentLevel, int nbLevels,
			int nbReqProScopeFoldersByLevel, int nbReqProRequirementsByLevel,
			ScopeElementScope scope) throws Exception
    {
    	ReqProScopeFolder reqProScopeFolder = new ReqProScopeFolder();

    	HelperTestUtilities.setByIntrospection(reqProScopeFolder, index, skipSetters);
    	
    	reqProScopeFolder.setParent(parent);

    	if(scope!=null)
    	{
    		if(scope.getReqProData()!=null)
    		{
    			ReqProImportStatus reqProImportStatus = ReqProImportStatusFixture.createFixture(reqProScopeFolder);
    			reqProScopeFolder.setReqProImportStatus(reqProImportStatus);
    		}
            if (scope.getChildren() != null)
            {
            	ArrayList children = new ArrayList();
            	//Add new level of ReqProScopeFolders
                if (currentLevel<nbLevels)
                {
                	for(int i=0;i<nbReqProScopeFoldersByLevel;i++)
                	{
                		children.add(createFixture(reqProScopeFolder, currentLevel+1, nbLevels, nbReqProScopeFoldersByLevel, nbReqProRequirementsByLevel, scope.getChildren()));
                	}
                }
        	
                //Add new ReqProRequirements
                for(int i=0;i<nbReqProRequirementsByLevel;i++)
                {
                	children.add(ReqProRequirementFixture.createFixture(reqProScopeFolder, scope.getChildren()));
                }
                
                if(children !=null && children.size()!=0)
                {
                	ScopeElement[] childrenArray = new ScopeElement[children.size()];
                	children.toArray(childrenArray);
                	reqProScopeFolder.setChildren(childrenArray);	
                }
            }    		
    	}
        return reqProScopeFolder;
    }
    
    private static void updateReqProScopeFolder(ReqProScopeFolder reqProScopeFolder,
			int index, ScopeElementScope scope) throws Exception
    {
    	HelperTestUtilities.setByIntrospection(reqProScopeFolder, index, skipSetters);
    	
    	if(scope!=null)
    	{
    		if(scope.getReqProData()!=null)
    		{
    			ReqProImportStatusFixture.updateFixture(reqProScopeFolder.getReqProImportStatus());
    		}
            if (scope.getChildren() != null)
            {
            	ScopeElement[] children = reqProScopeFolder.getChildren();
            	if(children!=null)
            	{
                    for(int i=0;i<children.length;i++)
                    {
                    	if(children[i] instanceof ReqProScopeFolder)
                    	{
                       	 	ReqProScopeFolderFixture.updateFixture((ReqProScopeFolder)children[i], scope.getChildren());                    		
                    	}
                    	else if (children[i] instanceof ReqProRequirement)
                    	{
                       	 	ReqProRequirementFixture.updateFixture((ReqProRequirement)children[i], scope.getChildren());
                    	}
                    }
            	}
            }            
    	}
    }
    
    public static void compare(ReqProScopeFolder[] pArray1,
    		ReqProScopeFolder[] pArray2, ScopeElementScope scope)
            throws AssertionFailedError, Exception
    {
    	
    	sortWithComparator(pArray1, RPMObjectIDComparator.INSTANCE);
    	sortWithComparator(pArray2, RPMObjectIDComparator.INSTANCE); 
    	
        RPMObjectFixture.compare(pArray1, pArray2);

        if (pArray1 == null)
            return;

        for (int i = 0; i < pArray1.length; i++)
        {
            compare(pArray1[i], pArray2[i], scope);
        }
    }
    
    
    public static void compare(ReqProScopeFolder f1, ReqProScopeFolder f2,
    		ScopeElementScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(f1, f2, localSkipGetters);
        if (scope == null)
            return;
        
        if(scope.getReqProData()!=null)
        {
            ReqProImportStatusFixture.compare(f1.getReqProImportStatus(), f2.getReqProImportStatus());
        }
        
        if(scope.getChildren()!=null)
        {
        	ReqProScopeFolder[] subFolders1 = getReqProScopeFolders(f1.getChildren());
        	ReqProScopeFolder[] subFolders2 = getReqProScopeFolders(f2.getChildren());
        	ReqProScopeFolderFixture.compare(subFolders1, subFolders2, scope.getChildren());
        	
        	ReqProRequirement[] requirements1 = getReqProRequirements(f1.getChildren());
        	ReqProRequirement[] requirements2 = getReqProRequirements(f2.getChildren());
        	ReqProRequirementFixture.compare(requirements1, requirements2, scope.getChildren());
        }
    }
    
    private static ReqProScopeFolder[] getReqProScopeFolders(ScopeElement[] scopeElements)
    {
    	ReqProScopeFolder[] folders = null;
    	if(scopeElements!=null || scopeElements.length!=0)
    	{
    		ArrayList foldersList = new ArrayList();
    		for(int i=0;i<scopeElements.length;i++)
    		{
    			if(scopeElements[i] instanceof ReqProScopeFolder)
    			{
    				foldersList.add(scopeElements[i]);
    			}
    		}
    		if(foldersList.size()!=0)
    		{
    			folders = new ReqProScopeFolder[foldersList.size()];
    			foldersList.toArray(folders);
    		}
    	}
    	return folders;
    }
    
    private static ReqProRequirement[] getReqProRequirements(ScopeElement[] scopeElements)
    {
    	ReqProRequirement[] requirements = null;
    	if(scopeElements!=null || scopeElements.length!=0)
    	{
    		ArrayList requirementsList = new ArrayList();
    		for(int i=0;i<scopeElements.length;i++)
    		{
    			if(scopeElements[i] instanceof ReqProRequirement)
    			{
    				requirementsList.add(scopeElements[i]);
    			}
    		}
    		if(requirementsList.size()!=0)
    		{
    			requirements = new ReqProRequirement[requirementsList.size()];
    			requirementsList.toArray(requirements);
    		}
    	}
    	return requirements;
    }
    
    public static ScopeElementScope getFullScope()
    {
        ScopeElementScope scope = new ScopeElementScope();
        scope.setReqProData(new RPMObjectScope());
        scope.setChildren(scope);
        return scope;
    }
    
}
