/*
 * Created on Sep 22, 2005
 */
package com.ibm.rpm.client.junit.scopemanagement.fixtures;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.AssetFinancialCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestState;
import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.applicationadministration.containers.RequirementState;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RtfAssignmentFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scope.fixtures.ScopeManagementModuleFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractRequest;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.AggregateScope;
import com.ibm.rpm.scopemanagement.containers.ApprovedCostBenefits;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.Impact;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.MitigationFactor;
import com.ibm.rpm.scopemanagement.containers.OrderOfMagnitudeCostBenefits;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.RiskMatrix;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeScheduleDate;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ScopeWorkProduct;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scopemanagement.scope.ScopeManagementModuleScope;
import com.ibm.rpm.scopemanagement.types.PrecisionType;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class ScopeElementFixture extends RPMObjectFixture
{
    protected static ArrayList localSkipGetters = new ArrayList();

    protected static ArrayList localSkipSetters = new ArrayList();

    protected static ArrayList riskMatrixSkipGetters = new ArrayList();

    private static Defect STATIC_DEFECT_FIXTURE = null;

    private static String XPATH_DEFECT_NAME = "STATIC_DEFECT_FIXTURE";

    private static ChangeRequest STATIC_CHANGE_REQUEST_FIXTURE = null;

    private static String XPATH_CHANGE_REQUEST_NAME = "STATIC_CHANGE_REQUEST_FIXTURE";

    private static ServiceRequest STATIC_SERVICE_REQUEST_FIXTURE = null;

    private static String XPATH_SERVICE_REQUEST_NAME = "STATIC_SERVICE_REQUEST_FIXTURE";

    private static Issue STATIC_ISSUE_FIXTURE = null;

    private static String XPATH_ISSUE_NAME = "STATIC_ISSUE_FIXTURE";

    private static Action STATIC_ACTION_FIXTURE = null;

    private static String XPATH_ACTION_NAME = "STATIC_ACTION_FIXTURE";

    private static Risk STATIC_RISK_FIXTURE = null;

    private static String XPATH_RISK_NAME = "STATIC_RISK_FIXTURE";

    private static Requirement STATIC_REQUIREMENT_FIXTURE = null;

    private static String XPATH_REQUIREMENT_NAME = "STATIC_REQUIREMENT_FIXTURE";

    static
    {
        localSkipGetters.addAll(skipGetters);
        localSkipGetters.add("getDuplicateNumber");
        localSkipGetters.add("getReferenceNumber");
        localSkipGetters.add("getTransferedUnderWbsRoot"); // until implemented
        
        localSkipGetters.add("getEstimatedFinishDate");
        localSkipGetters.add("getEstimatedDurationInMinutes");
               
        localSkipGetters.add("getTask"); // default is false, so comparison fails when inserting with null
        localSkipGetters.add("getWbsSortingRank");
        localSkipGetters.add("getParentSortingRank");
        localSkipGetters.add("getTemplateSortingRank");
        
        localSkipSetters.addAll(skipSetters);
        localSkipSetters.add("setApprovedByName");
        localSkipSetters.add("setClosedByName");
        localSkipSetters.add("setClosedByDate");
        localSkipSetters.add("setProposedByName");
        localSkipSetters.add("setWbsSortingRank");
        localSkipSetters.add("setParentSortingRank");
        localSkipSetters.add("setTemplateSortingRank");
//        localSkipSetters.add("setApprovedDate");
//        localSkipSetters.add("setProposedDate");
        localSkipSetters.add("setDuplicateNumber");
        localSkipSetters.add("setReferenceNumber");
        localSkipSetters.add("setTransferedUnderWbsRoot");
        localSkipSetters.add("setTask"); // task is dependent on defaultScope and vice versa
        localSkipSetters.add("setDefaultScope"); // Only one ScopeElement of a certain type per group can be set as default
        
        //estimatedFinishDate is a read-only field
        localSkipSetters.add("setEstimatedFinishDate");

        riskMatrixSkipGetters.addAll(skipGetters);
        
        riskMatrixSkipGetters.add("getExposurePercent"); // Because when you insert null, the default of 0 is returned and comparison fails  
        riskMatrixSkipGetters.add("getRiskMatrixColor"); // Because this is calculated automatically
        riskMatrixSkipGetters.add("getMitigationCost"); // Because when you insert null, the default of 0.0 is returned and comparison fails
        riskMatrixSkipGetters.add("getProbabilityCost"); // Because when you insert null, the default of 0.0 is returned and comparison fails
    }

    public static ScopeFolder createMainParent(ClientTestController ctc)
            throws Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createFixture(
            new Proposal(), null, ctc);
        proposal  = (Proposal) ctc.save(proposal, null).getRpmObject();

        RPMObject[] rpmObjects = ctc.loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", new ScopeElementScope());
        ScopeFolder mainFolder = (ScopeFolder) rpmObjects[0];

        return mainFolder;
    }

    /*
     * Use this method to create work elements for use as parents or as assigned
     * objects only. The work elements are reused.
     */
    public static ScopeElement retrieveStaticFixture(ScopeElement parent,
            ScopeElement baseScopeElement, ScopeElementScope scope,
            int nbChildren, ClientTestController ctc) throws Exception
    {
        RPMObject[] rpmObjects;
        ScopeElement scopeElement = null;

        if (baseScopeElement instanceof Requirement)
        {

            // Static fixture should always be reloaded from the DB
            rpmObjects = loadFixture(baseScopeElement, scope, ctc,
                XPATH_REQUIREMENT_NAME);

            if (rpmObjects != null && rpmObjects[0] != null)
            {
                scopeElement = (ScopeElement) rpmObjects[0];
            }
            else
            {
                // Create the ScopeElement using the fixture, set its name,
                // then save it
                scopeElement = createRequirementFixture(scope, nbChildren,
                    parent, ctc);
                scopeElement.setName(XPATH_REQUIREMENT_NAME);
                scopeElement = (ScopeElement) saveWithoutDelete(ctc,
                    scopeElement, scope);
                STATIC_REQUIREMENT_FIXTURE = (Requirement) scopeElement;
            }
        }
        else if (baseScopeElement instanceof Action)
        {
            // Static fixture should always be reloaded from the DB
            rpmObjects = loadFixture(baseScopeElement, scope, ctc,
                XPATH_ACTION_NAME);

            if (rpmObjects != null && rpmObjects[0] != null)
            {
                scopeElement = (ScopeElement) rpmObjects[0];
            }
            else
            {
                // Create the ScopeElement using the fixture, set its name,
                // then save it
                scopeElement = createActionFixture(scope, nbChildren, parent,
                    ctc);
                scopeElement.setName(XPATH_ACTION_NAME);
                scopeElement = (ScopeElement) saveWithoutDelete(ctc,
                    scopeElement, scope);
                STATIC_ACTION_FIXTURE = (Action) scopeElement;
            }
        }
        else if (baseScopeElement instanceof Issue)
        {
            // Static fixture should always be reloaded from the DB
            rpmObjects = loadFixture(baseScopeElement, scope, ctc,
                XPATH_ISSUE_NAME);

            if (rpmObjects != null && rpmObjects[0] != null)
            {
                scopeElement = (ScopeElement) rpmObjects[0];
            }
            else
            {
                // Create the ScopeElement using the fixture, set its name,
                // then save it
                scopeElement = createIssueFixture(scope, nbChildren, parent,
                    ctc);
                scopeElement.setName(XPATH_ISSUE_NAME);
                scopeElement = (ScopeElement) saveWithoutDelete(ctc,
                    scopeElement, scope);
                STATIC_ISSUE_FIXTURE = (Issue) scopeElement;
            }
        }
        else if (baseScopeElement instanceof ChangeRequest)
        {
            // Static fixture should always be reloaded from the DB
            rpmObjects = loadFixture(baseScopeElement, scope, ctc,
                XPATH_CHANGE_REQUEST_NAME);

            if (rpmObjects != null && rpmObjects[0] != null)
            {
                scopeElement = (ScopeElement) rpmObjects[0];
            }
            else
            {
                // Create the ScopeElement using the fixture, set its name,
                // then save it
                scopeElement = createChangeRequestFixture(scope, nbChildren,
                    parent, ctc);
                scopeElement.setName(XPATH_CHANGE_REQUEST_NAME);
                scopeElement = (ScopeElement) saveWithoutDelete(ctc,
                    scopeElement, scope);
                STATIC_CHANGE_REQUEST_FIXTURE = (ChangeRequest) scopeElement;
            }
        }
        else if (baseScopeElement instanceof ServiceRequest)
        {
            // Static fixture should always be reloaded from the DB
            rpmObjects = loadFixture(baseScopeElement, scope, ctc,
                XPATH_SERVICE_REQUEST_NAME);

            if (rpmObjects != null && rpmObjects[0] != null)
            {
                scopeElement = (ScopeElement) rpmObjects[0];
            }
            else
            {
                // Create the ScopeElement using the fixture, set its name,
                // then save it
                scopeElement = createServiceRequestFixture(scope, nbChildren,
                    parent, ctc);
                scopeElement.setName(XPATH_SERVICE_REQUEST_NAME);
                scopeElement = (ScopeElement) saveWithoutDelete(ctc,
                    scopeElement, scope);
                STATIC_SERVICE_REQUEST_FIXTURE = (ServiceRequest) scopeElement;
            }
        }
        else if (baseScopeElement instanceof Defect)
        {
            // Static fixture should always be reloaded from the DB
            rpmObjects = loadFixture(baseScopeElement, scope, ctc,
                XPATH_DEFECT_NAME);

            if (rpmObjects != null && rpmObjects[0] != null)
            {
                scopeElement = (ScopeElement) rpmObjects[0];
            }
            else
            {
                // Create the ScopeElement using the fixture, set its name,
                // then save it
                scopeElement = createDefectFixture(scope, nbChildren, parent,
                    ctc);
                scopeElement.setName(XPATH_DEFECT_NAME);
                scopeElement = (ScopeElement) saveWithoutDelete(ctc,
                    scopeElement, scope);
                STATIC_DEFECT_FIXTURE = (Defect) scopeElement;
            }
        }
        else if (baseScopeElement instanceof Risk)
        {
            // Static fixture should always be reloaded from the DB
            rpmObjects = loadFixture(baseScopeElement, scope, ctc,
                XPATH_RISK_NAME);

            if (rpmObjects != null && rpmObjects[0] != null)
            {
                scopeElement = (ScopeElement) rpmObjects[0];
            }
            else
            {
                // Create the ScopeElement using the fixture, set its name,
                // then save it
                scopeElement = createRiskFixture(scope, nbChildren, parent, ctc);
                scopeElement.setName(XPATH_RISK_NAME);
                scopeElement = (ScopeElement) saveWithoutDelete(ctc,
                    scopeElement, scope);
                STATIC_RISK_FIXTURE = (Risk) scopeElement;
            }
        }
        else
        {
            Assert
                .fail("ScopeElementFixture.retrieveStaticFixture : Invalid ScopeElement ");
        }

        return scopeElement;

    }

    private static RPMObject[] loadFixture(ScopeElement baseScopeElement,
            ScopeElementScope scope, ClientTestController ctc, String xpathName)
            throws Exception
    {
        // Try loading the ScopeElement from xpath
        RPMObject[] rpmObjects = null;

        rpmObjects = ctc.loadFromXpath("/"
                + StringUtil.getShortClassName(baseScopeElement.getClass())
                + "[name='" + xpathName + "']", scope);
        
        if (rpmObjects != null && rpmObjects[0] != null)
        {
            ScopeElement scopeElement = (ScopeElement) rpmObjects[0];
            if (ctc.isCheckedOut(scopeElement))
            {
                ctc.forceCheckinOnRPMObjectCheckOut(scopeElement);
            }
        }

        return rpmObjects;
    }

    public static ScopeElement createFixture(ScopeElementScope scope,
            ClientTestController ctc, int nbChildren) throws Exception
    {
        Proposal proposal = (Proposal)WorkElementFixture.retrieveStaticFixture(
                new Proposal(), ctc);

        proposal = (Proposal) ctc.loadFromID(proposal, new WorkElementScope());

        RPMObject[] rpmObjects = ctc.loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", new ScopeElementScope());
        ScopeFolder mainFolder = (ScopeFolder) rpmObjects[0];
        return createFolderFixture(scope, nbChildren, mainFolder, ctc);
    }

    public static ScopeFolder createFolderFixture(ScopeElementScope scope,
            int nbChildren, ScopeElement parent, ClientTestController ctc)
            throws Exception
    {
        ScopeFolder element = new ScopeFolder();

        HelperTestUtilities.setByIntrospection(element, random(), skipSetters);
        element.setParent(parent);
        createScopedContainers(element, scope, nbChildren, ctc);

        if (scope != null && scope.getChildren() != null)
        {
            ArrayList elementArray = new ArrayList();
            for (int i = 0; i < nbChildren; i++)
            {
                // elementArray.add(createFolderFixture(null, 0, element, ctc));
                elementArray.add(createRequirementFixture(scope, nbChildren,
                    element, ctc));
                //                elementArray.add(createChangeRequestFixture(null, 0, element,
                //                    ctc));
                //                elementArray.add(createServiceRequestFixture(null, 0,
                // element,
                //                    ctc));
                //                elementArray.add(createActionFixture(null, 0, element, ctc));
                //                elementArray.add(createDefectFixture(null, 0, element, ctc));
                //                elementArray.add(createIssueFixture(null, 0, element, ctc));
                //                elementArray.add(createRiskFixture(null, 0, element, ctc));
            }
            ScopeElement[] scopeElements = new ScopeElement[elementArray.size()];
            scopeElements = (ScopeElement[]) elementArray
                .toArray(scopeElements);
            element.setChildren(scopeElements);
        }
        return element;
    }
    
    public static ScopeFolder createParentSwitchFixture(
            ScopeElementScope scope, ScopeFolder root,
            ClientTestController ctc) throws Exception
    {
        ArrayList children = new ArrayList();
        //create a second folder
        ScopeFolder folder = createFolderFixture(scope, 0, root, ctc);
        children.add(folder);
        
        //create 2 Requirements
        Requirement req1 = createRequirementFixture(scope, 0, root, ctc);
        children.add(req1);        
        
        Requirement req2 = createRequirementFixture(scope, 0, root, ctc);
        children.add(req2);
        
        // create 2 ScopeDeliverables under a Requirement
        ScopeDeliverable deliv1 = createScopeDeliverableFixture(scope, 0,
            req1, ctc);        
        ScopeDeliverable deliv2 = createScopeDeliverableFixture(scope, 0,
            req1, ctc);        
        
        // create 2 Tasks
        ScopeTask task1 = createScopeTaskFixture(scope, 0, req1, ctc);
        ScopeTask task2 = createScopeTaskFixture(scope, 0, req1, ctc);

        req1.setChildren(new ScopeElement[]
        {
                deliv1, deliv2, task1, task2
        });
        ScopeElement[] scopeElements = new ScopeElement[children.size()];
        scopeElements = (ScopeElement[]) children
            .toArray(scopeElements);
        root.setChildren(scopeElements);
        return root;
    }

    public static Resource createResource()
    {
        Resource res = new Resource();
        res.setFullName("ADMINISTRATOR");
        
        return res;
    }
    
    public static Requirement createRequirementFixture(ScopeElementScope scope,
            int nbChildren, ScopeElement parent, ClientTestController ctc)
            throws Exception
    {
        Requirement element = new Requirement();
        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);

        RequirementState state = new RequirementState();
        state.setClosedState(Boolean.FALSE);
        element.setState(state);

        // Set values that were skipped
        element.setDefaultScope(Boolean.FALSE);
        element.setTask(Boolean.FALSE);
        //element.setEstimatedFinishDate(element.getEstimatedStartDate());
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.YEAR, (1+element.getEstimatedStartDate().get(Calendar.YEAR)));
        element.setEstimatedFinishDate(cal);
        
        element.setParent(parent);
        createScopedContainers(element, scope, nbChildren, ctc);

        element.setApprovedByName(createResource().getFullName());
        //element.setClosedByDate(null);
        //element.setClosedByName(null);
        element.setProposedByName(createResource().getFullName());
        element.setPublished(PublishedType.None);

        createScopedContainersForRequirement(element, scope, nbChildren, ctc);

        return element;
    }

    private static void createScopedContainersForRequirement(Requirement element, ScopeElementScope scope, int nbChildren, 
            ClientTestController ctc) throws Exception
    {
        if(scope != null) {
            if(scope.isScheduleDates()){
                
                //Proposed date is read-only
                
                // Approved Date
                ScopeScheduleDate approvedDate = ScopeScheduleDateFixture.createScopeScheduleDate(ctc);
                element.setApprovedDate(approvedDate);
                
            }
            
            if(scope.isWbsParent()) 
            {
                WorkElement itsParent = WorkElementFixture.createSaveFixture(new Task(), null, ctc);
                element.setWbsParent(itsParent);
            }
            
            if (scope.getChildren() != null)
            {
                ArrayList elementArray = new ArrayList();
                for (int i = 0; i < nbChildren; i++)
                {
                    //              scope.setScopeElements(null);
                    //              elementArray.add(createFolderFixture(scope, nbChildren,
                    //                      element, ctc));
                    //              elementArray.add(createRequirementFixture(scope, nbChildren,
                    //                      element, ctc));
                    elementArray.add(createChangeRequestFixture(scope, nbChildren,
                        element, ctc));
                    //              elementArray.add(createServiceRequestFixture(scope,
                    // nbChildren,
                    //                      element, ctc));
                    //              elementArray.add(createActionFixture(scope, nbChildren,
                    //                      element, ctc));
                    //              elementArray.add(createDefectFixture(scope, nbChildren,
                    //                      element, ctc));
                    //              elementArray.add(createIssueFixture(scope, nbChildren,
                    // element,
                    //                      ctc));
                    //              
                    //              scope.setScopeElements(scope);
                    elementArray.add(createRiskFixture(scope, nbChildren, element,
                        ctc));
                    //              
                    elementArray.add(createScopeMilestoneFixture(scope, nbChildren,
                        element, ctc));
                }
                ScopeElement[] scopeElements = new ScopeElement[elementArray.size()];
                scopeElements = (ScopeElement[]) elementArray
                    .toArray(scopeElements);
                element.setChildren(scopeElements);
                
                // task must be true when Requirement has children
                element.setTask(Boolean.FALSE);
            }
            
//            if(scope.getDocumentFolder() != null)
//            {
//                DocumentFolder itsfolder = (DocumentFolder) DocumentElementFixture.createFixture(element, new DocumentFolder(), 
//                    0, scope.getDocumentFolder(), ctc);
//                
//                element.setDocumentFolder(itsfolder);
//            }
        }
    }

    public static ChangeRequest createChangeRequestFixture(
            ScopeElementScope scope, int nbChildren, ScopeElement parent,
            ClientTestController ctc) throws Exception
    {
        ChangeRequest element = new ChangeRequest();

        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);
        ChangeRequestState state = new ChangeRequestState();
        state.setClosedState(Boolean.FALSE);
        element.setState(state);

        // Set values that were skipped
        element.setDefaultScope(Boolean.FALSE);
        element.setTask(Boolean.FALSE);
        //element.setEstimatedFinishDate(element.getEstimatedStartDate());
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.YEAR, (1+element.getEstimatedStartDate().get(Calendar.YEAR)));
        element.setEstimatedFinishDate(cal);
        
        element.setParent(parent);
        createScopedContainers(element, scope, nbChildren, ctc);

        // Other
        element.setApprovedByName(createResource().getFullName());
        //element.setClosedByName(null);
        element.setProposedByName(createResource().getFullName());
        element.setPublished(PublishedType.None);

        if(scope != null && scope.isScheduleDates()){
            
            //Proposed date is read-only
            
            // Approved Date
            ScopeScheduleDate approvedDate = ScopeScheduleDateFixture.createScopeScheduleDate(ctc);
            element.setApprovedDate(approvedDate);

        }
        
        if (scope != null && scope.getChildren() != null)
        {
            ArrayList elementArray = new ArrayList();
            for (int i = 0; i < nbChildren; i++)
            {
                //				elementArray.add(createFolderFixture(scope, nbChildren,
                //						element, ctc));
                //				elementArray.add(createRequirementFixture(scope, nbChildren,
                //						element, ctc));
                //				elementArray.add(createChangeRequestFixture(scope,
                // nbChildren,
                //						element, ctc));
                //				elementArray.add(createServiceRequestFixture(scope,
                // nbChildren,
                //						element, ctc));
                //				elementArray.add(createActionFixture(scope, nbChildren,
                //						element, ctc));
                //				elementArray.add(createDefectFixture(scope, nbChildren,
                //						element, ctc));
                //				elementArray.add(createIssueFixture(scope, nbChildren,
                // element,
                //						ctc));
                //				elementArray.add(createRiskFixture(scope, nbChildren,
                // element,
                //						ctc));
            }
            ScopeElement[] scopeElements = new ScopeElement[elementArray.size()];
            scopeElements = (ScopeElement[]) elementArray
                .toArray(scopeElements);
            element.setChildren(scopeElements);
        }
        return element;
    }

    public static ServiceRequest createServiceRequestFixture(
            ScopeElementScope scope, int nbChildren, ScopeElement parent,
            ClientTestController ctc) throws Exception
    {
        ServiceRequest element = new ServiceRequest();

        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);

        element.setDefaultScope(Boolean.FALSE);
        element.setTask(Boolean.FALSE);
       // element.setEstimatedFinishDate(element.getEstimatedStartDate());
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.YEAR, (1+element.getEstimatedStartDate().get(Calendar.YEAR)));
        element.setEstimatedFinishDate(cal);
        
        element.setParent(parent);
        createScopedContainers(element, scope, nbChildren, ctc);

        if(scope != null && scope.isScheduleDates()){
        
            //Proposed date is read-only
            
            // Approved Date
            ScopeScheduleDate approvedDate = ScopeScheduleDateFixture.createScopeScheduleDate(ctc);
            element.setApprovedDate(approvedDate);
        }

        // Other
        element.setApprovedByName(createResource().getFullName());
        //element.setClosedByName(null);
        element.setProposedByName(createResource().getFullName());
        element.setPublished(PublishedType.None);

        if (scope != null && scope.getChildren() != null)
        {
            ArrayList elementArray = new ArrayList();
            for (int i = 0; i < nbChildren; i++)
            {
                elementArray.add(createFolderFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createRequirementFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createChangeRequestFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createServiceRequestFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createActionFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createDefectFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createIssueFixture(scope, nbChildren, element,
                    ctc));
                elementArray.add(createRiskFixture(scope, nbChildren, element,
                    ctc));
            }
            ScopeElement[] scopeElements = new ScopeElement[elementArray.size()];
            scopeElements = (ScopeElement[]) elementArray
                .toArray(scopeElements);
            element.setChildren(scopeElements);
        }
        return element;
    }
    
    public static ScopeElement createScopeElementFixture(ScopeElementScope scope, int nbChildren, ScopeElement element,
    		ScopeElement parent, ClientTestController ctc) throws Exception
    {
    	ScopeElement returnElement = null;
    	if(element instanceof AbstractAggregateScope)
    	{
    		returnElement = createAbstractAggregateScopeFixture(scope, nbChildren, (AbstractAggregateScope)element, parent, ctc);
    	}
        else
        {
            throw new Exception("element of type "+element.getClass()+" not supported by this method.");
        }
    	return returnElement;
    }

    public static AbstractAggregateScope createAbstractAggregateScopeFixture(ScopeElementScope scope, int nbChildren,
    		AbstractAggregateScope element, ScopeElement parent, ClientTestController ctc) throws Exception
    {
    	AbstractAggregateScope returnElement = null;
    	if(element instanceof AbstractScope)
    	{
    		returnElement = createAbstractScopeFixture(scope, nbChildren, (AbstractScope)element, parent, ctc);
    	}
    	else if(element instanceof ScopeDeliverable)
    	{
    		returnElement = createScopeDeliverableFixture(scope, nbChildren, parent, ctc);
    	}
    	else if(element instanceof ScopeSummaryTask)
    	{
    		returnElement = createScopeSummaryTaskFixture(scope, nbChildren, parent, ctc);
    	}
    	else if(element instanceof ScopeWorkProduct)
    	{
    		// TODO: Create the Fixutre for this element
    		// returnElement = createScopeWorkProductFixture(scope, nbChildren, parent, ctc);
            throw new Exception("element of type "+element.getClass()+" not supported by this method.");
    	}
    	return returnElement;
    }
    
    public static AbstractScope createAbstractScopeFixture(ScopeElementScope scope, int nbChildren, AbstractScope element,
    	ScopeElement parent, ClientTestController ctc) throws Exception
    {
    	AbstractScope returnElement = null;
    	if(element instanceof Requirement)
    	{
    		returnElement = createRequirementFixture(scope, nbChildren, parent, ctc);
    	}
    	else if(element instanceof Action)
    	{
    		returnElement = createActionFixture(scope, nbChildren, parent, ctc);
    	}
    	else if(element instanceof Issue)
    	{
    		returnElement = createIssueFixture(scope, nbChildren, parent, ctc);
    	}
    	else if(element instanceof Risk)
    	{
    		returnElement = createRiskFixture(scope, nbChildren, parent, ctc);
    	}
    	else if(element instanceof ChangeRequest)
    	{
    		returnElement = createChangeRequestFixture(scope, nbChildren, parent, ctc);
    	}
    	else if(element instanceof Defect)
    	{
    		returnElement = createDefectFixture(scope, nbChildren, parent, ctc);
    	}
    	else if(element instanceof ServiceRequest)
    	{
    		returnElement = createServiceRequestFixture(scope, nbChildren, parent, ctc);
    	}
    	return returnElement;
    }
    
    public static Action createActionFixture(ScopeElementScope scope,
            int nbChildren, ScopeElement parent, ClientTestController ctc)
            throws Exception
    {
        Action element = new Action();
        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);
        element.setParent(parent);
        createScopedContainers(element, scope, nbChildren, ctc);

        // Set values that were skipped
        element.setDefaultScope(Boolean.FALSE);
        element.setTask(Boolean.FALSE);
       // element.setEstimatedFinishDate(element.getEstimatedStartDate());
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.YEAR, (1+element.getEstimatedStartDate().get(Calendar.YEAR)));
        element.setEstimatedFinishDate(cal);
        
        // Other
        element.setApprovedByName(createResource().getFullName());
        //element.setClosedByName(null);
        element.setProposedByName(createResource().getFullName());
        element.setPublished(PublishedType.None);
        
        if(scope != null && scope.isScheduleDates()){
            
            //Proposed date is read-only
            
            // Approved Date
            ScopeScheduleDate approvedDate = ScopeScheduleDateFixture.createScopeScheduleDate(ctc);
            element.setApprovedDate(approvedDate);
        }

        if (scope != null && scope.getChildren() != null)
        {
            ArrayList elementArray = new ArrayList();
            for (int i = 0; i < nbChildren; i++)
            {
                elementArray.add(createFolderFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createRequirementFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createChangeRequestFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createServiceRequestFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createActionFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createDefectFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createIssueFixture(scope, nbChildren, element,
                    ctc));
                elementArray.add(createRiskFixture(scope, nbChildren, element,
                    ctc));
            }
            ScopeElement[] scopeElements = new ScopeElement[elementArray.size()];
            scopeElements = (ScopeElement[]) elementArray
                .toArray(scopeElements);
            element.setChildren(scopeElements);
        }
        return element;
    }

    public static Defect createDefectFixture(ScopeElementScope scope,
            int nbChildren, ScopeElement parent, ClientTestController ctc)
            throws Exception
    {
        Defect element = new Defect();
        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);
        element.setParent(parent);
        createScopedContainers(element, scope, nbChildren, ctc);

        // Set values that were skipped
        element.setDefaultScope(Boolean.FALSE);
        element.setTask(Boolean.FALSE);
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.YEAR, (1+element.getEstimatedStartDate().get(Calendar.YEAR)));
        element.setEstimatedFinishDate(cal);
        
        // Other
        element.setApprovedByName(createResource().getFullName());
        //element.setClosedByName(null);
        element.setProposedByName(createResource().getFullName());
        element.setPublished(PublishedType.None);
        
        if(scope != null && scope.isScheduleDates()){
            
            //Proposed date is read-only
            
            // Approved Date
            ScopeScheduleDate approvedDate = ScopeScheduleDateFixture.createScopeScheduleDate(ctc);
            element.setApprovedDate(approvedDate);
        }

        if (scope != null && scope.getChildren() != null)
        {
            ArrayList elementArray = new ArrayList();
            for (int i = 0; i < nbChildren; i++)
            {
                elementArray.add(createFolderFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createRequirementFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createChangeRequestFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createServiceRequestFixture(scope, 0, element,
                    ctc));
                elementArray.add(createActionFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createDefectFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createIssueFixture(scope, nbChildren, element,
                    ctc));
                elementArray.add(createRiskFixture(scope, nbChildren, element,
                    ctc));
            }
            ScopeElement[] scopeElements = new ScopeElement[elementArray.size()];
            scopeElements = (ScopeElement[]) elementArray
                .toArray(scopeElements);
            element.setChildren(scopeElements);
        }
        return element;
    }

    public static Issue createIssueFixture(ScopeElementScope scope,
            int nbChildren, ScopeElement parent, ClientTestController ctc)
            throws Exception
    {
        Issue element = new Issue();
        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);
        
        element.setParent(parent);
        createScopedContainers(element, scope, nbChildren, ctc);

        // Set values that were skipped
        element.setDefaultScope(Boolean.FALSE);
        element.setTask(Boolean.FALSE);
        //element.setEstimatedFinishDate(element.getEstimatedStartDate());
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.YEAR, (1+element.getEstimatedStartDate().get(Calendar.YEAR)));
        element.setEstimatedFinishDate(cal);

        // Other
        element.setApprovedByName(createResource().getFullName());
        //element.setClosedByName(null);
        element.setProposedByName(createResource().getFullName());
        element.setPublished(PublishedType.None);
        
        if(scope != null && scope.isScheduleDates()){
            
            //Proposed date is read-only
            
            // Approved Date
            ScopeScheduleDate approvedDate = ScopeScheduleDateFixture.createScopeScheduleDate(ctc);
            element.setApprovedDate(approvedDate);
        }

        if (scope != null && scope.getChildren() != null)
        {
            ArrayList elementArray = new ArrayList();
            for (int i = 0; i < nbChildren; i++)
            {
                elementArray.add(createFolderFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createRequirementFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createChangeRequestFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createServiceRequestFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createActionFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createDefectFixture(scope, nbChildren,
                    element, ctc));
                elementArray.add(createIssueFixture(scope, nbChildren, element,
                    ctc));
                elementArray.add(createRiskFixture(scope, nbChildren, element,
                    ctc));
            }
            ScopeElement[] scopeElements = new ScopeElement[elementArray.size()];
            scopeElements = (ScopeElement[]) elementArray
                .toArray(scopeElements);
            element.setChildren(scopeElements);
        }
        return element;
    }

    public static Risk createRiskFixture(ScopeElementScope scope,
            int nbChildren, ScopeElement parent, ClientTestController ctc)
            throws Exception
    {
        Risk element = new Risk();
        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);
        element.setParent(parent);
        createScopedContainers(element, scope, nbChildren, ctc);

        // Set values that were skipped
        element.setDefaultScope(Boolean.FALSE);
        element.setTask(Boolean.FALSE);
        //element.setEstimatedFinishDate(element.getEstimatedStartDate());
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.YEAR, (1+element.getEstimatedStartDate().get(Calendar.YEAR)));
        element.setEstimatedFinishDate(cal);

        // Other
        element.setApprovedByName(createResource().getFullName());
        //element.setClosedByName(null);
        element.setProposedByName(createResource().getFullName());
        element.setPublished(PublishedType.None);
        
        if(scope != null && scope.isScheduleDates()){
            
            //Proposed date is read-only
            
            // Approved Date
            ScopeScheduleDate approvedDate = ScopeScheduleDateFixture.createScopeScheduleDate(ctc);
            element.setApprovedDate(approvedDate);
        }

        if (scope != null && scope.getChildren() != null)
        {
            ArrayList elementArray = new ArrayList();
            for (int i = 0; i < nbChildren; i++)
            {
                //                elementArray.add(createFolderFixture(null, 0, element, ctc));
                //                elementArray
                //                    .add(createRequirementFixture(null, 0, element, ctc));
                //                elementArray.add(createChangeRequestFixture(null, 0, element,
                //                    ctc));
                //                elementArray.add(createServiceRequestFixture(null, 0,
                // element,
                //                    ctc));
                //                elementArray.add(createActionFixture(null, 0, element, ctc));
                //                elementArray.add(createDefectFixture(null, 0, element, ctc));
                //                elementArray.add(createIssueFixture(null, 0, element, ctc));
                //                elementArray.add(createRiskFixture(null, 0, element, ctc));
                //                elementArray.add(createScopeDeliverableFixture(null, 0,
                // parent,
                //                    ctc));
                //                elementArray.add(createScopeSummaryTaskFixture(null, 0,
                // parent,
                //                    ctc));
                //                elementArray.add(createScopeMilestoneFixture(null, 0, parent,
                //                    ctc));
                //                elementArray.add(createScopeTaskFixture(null, 0, parent,
                // ctc));
            }
            ScopeElement[] scopeElements = new ScopeElement[elementArray.size()];
            scopeElements = (ScopeElement[]) elementArray
                .toArray(scopeElements);
            element.setChildren(scopeElements);
        }
        return element;
    }

    public static ScopeDeliverable createScopeDeliverableFixture(
            ScopeElementScope scope, int nbChildren, ScopeElement parent,
            ClientTestController ctc) throws Exception
    {
        ScopeDeliverable element = new ScopeDeliverable();
        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);
        createScopedContainers(element, scope, nbChildren, ctc);
        element.setParent(parent);

        // Set values that were skipped

        // Other
        element.setClosedByName(null);
        element.setProposedByName(createResource().getFullName());
        
        if(scope != null && scope.isScheduleDates()){
            
            //Nothing to set - Proposed date is read-only
            
        }

        if (scope != null && scope.getChildren() != null)
        {
            ArrayList elementArray = new ArrayList();
            for (int i = 0; i < nbChildren; i++)
            {
                elementArray.add(createScopeDeliverableFixture(null, 0, parent,
                    ctc));
                elementArray.add(createScopeSummaryTaskFixture(scope,
                    nbChildren, parent, ctc));
                elementArray.add(createScopeMilestoneFixture(scope, nbChildren,
                    parent, ctc));
                elementArray.add(createScopeTaskFixture(scope, nbChildren,
                    parent, ctc));
            }
            ScopeElement[] scopeElements = new ScopeElement[elementArray.size()];
            scopeElements = (ScopeElement[]) elementArray
                .toArray(scopeElements);
            element.setChildren(scopeElements);
        }
        return element;
    }

    public static ScopeSummaryTask createScopeSummaryTaskFixture(
            ScopeElementScope scope, int nbChildren, ScopeElement parent,
            ClientTestController ctc) throws Exception
    {
        ScopeSummaryTask element = new ScopeSummaryTask();
        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);
        createScopedContainers(element, scope, nbChildren, ctc);
        element.setParent(parent);

        // Set values that were skipped
        
        // Other
        element.setClosedByName(null);
        element.setProposedByName(createResource().getFullName());
        
        if(scope != null && scope.isScheduleDates()){
//          
            //Nothing to set - Proposed date is read-only
        }

        if (scope != null && scope.getChildren() != null)
        {
            ArrayList elementArray = new ArrayList();
            for (int i = 0; i < nbChildren; i++)
            {
//                elementArray.add(createScopeDeliverableFixture(null, 0, parent,
//                    ctc));
//                elementArray.add(createScopeSummaryTaskFixture(null, 0, parent,
//                    ctc));
//                elementArray.add(createScopeMilestoneFixture(scope, nbChildren,
//                    parent, ctc));
//                elementArray.add(createScopeTaskFixture(scope, nbChildren,
//                    parent, ctc));
            }
            ScopeElement[] scopeElements = new ScopeElement[elementArray.size()];
            scopeElements = (ScopeElement[]) elementArray
                .toArray(scopeElements);
            element.setChildren(scopeElements);
        }
        return element;
    }

    public static ScopeMilestone createScopeMilestoneFixture(
            ScopeElementScope scope, int nbChildren, ScopeElement parent,
            ClientTestController ctc) throws Exception
    {
        ScopeMilestone element = new ScopeMilestone();
        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);
        createScopedContainers(element, scope, nbChildren, ctc);
        element.setParent(parent);

        // Set values that were skipped
        element.setClosedByName(null);
        element.setProposedByName("ADMINISTRATOR");

        return element;
    }

    public static ScopeTask createScopeTaskFixture(ScopeElementScope scope,
            int nbChildren, ScopeElement parent, ClientTestController ctc)
            throws Exception
    {
        ScopeTask element = new ScopeTask();
        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);
        createScopedContainers(element, scope, nbChildren, ctc);

        // Set values that were skipped
        element.setClosedByName(null);
        element.setProposedByName("ADMINISTRATOR");

        element.setParent(parent);
        
        return element;
    }

    public static void createScopedContainers(ScopeElement element,
            ScopeElementScope scope, int nbChildren, ClientTestController ctc)
            throws Exception
    {
        if (scope == null || element == null)
            return;
        if(scope.isClosedByName())
        {
        	AbstractScope abstractScope = (AbstractScope)element;
        	abstractScope.setClosedByName(createResource().getFullName());
        }
        if (scope.getResourceTaskAssignments() != null)
        {
        	if(scope.getResourceTaskAssignments().getResource() != null)
        	{
        		if(element.getParent() instanceof ScopeElement)
        		{
        			ResourceTaskAssignment[] rtaArray = ((ScopeElement)element.getParent()).getResourceTaskAssignments();
        			if(rtaArray != null && rtaArray[0] != null)
        			{
        				Resource r = rtaArray[0].getResource();
        				if(r != null)
        				{
        					ResourceTaskAssignment rta = ResourceTaskAssignmentFixture.createFixture(element, r, null, scope.getResourceTaskAssignments(), false, ctc);
                			element.setResourceTaskAssignments(new ResourceTaskAssignment[]{rta});
        				}
        			}
        		}
        	}
            // Will be tested in ResourceTaskAssignment tests
//            ResourceRoleAssignment[] rraArray = new ResourceRoleAssignment[nbChildren + 1];
//            for (int i = 0; i < nbChildren; i++)
//            {
//                Resource r = ResourceFixture.createSaveFixture(null, ctc);
//                ResourceRoleAssignment rra = new ResourceRoleAssignment();
//                rra.setResource(r);
//                rra.setParent(element);
//                rraArray[i] = rra;
//            }
//            rraArray[nbChildren] = null;
//            element.setResourceAssignments(rraArray);
        }

        if (scope.getWorkElement() != null)
        {
            // TODO get a workElement - transfer the scope element under wbs
            // root
            // this might only be done in the update because scope element must
            // exist
            // before it can be transfered
        }

        if (scope.isApprovedCostBenefits()
                && ((element instanceof AbstractRequest)
                        || (element instanceof Requirement) || (element instanceof Action)))
        {
            ApprovedCostBenefits approvedCostBenefits = new ApprovedCostBenefits();
            HelperTestUtilities.setByIntrospection(approvedCostBenefits,
                random(), skipSetters);

            if (element instanceof AbstractRequest)
            {
                ((AbstractRequest) element)
                    .setApprovedCostBenefits(approvedCostBenefits);
            }
            else if (element instanceof Requirement)
            {
                ((Requirement) element)
                    .setApprovedCostBenefits(approvedCostBenefits);
            }
            else
            {
                ((Action) element)
                    .setApprovedCostBenefits(approvedCostBenefits);
            }
        }

        if (scope.isAsset() && element instanceof AbstractScope)
        {
            AbstractScope absScope = (AbstractScope) element;
            absScope.setAsset(AssetFixture.createSaveFixture(ctc));
        }

        if (scope.isAssetCode() && element instanceof AbstractScope)
        {
            AbstractScope absScope = (AbstractScope) element;
            absScope
                .setAssetCode(DatafieldFixture
                    .createAssetFinancialCategory(DatafieldFixture.ASSET_FINANCIAL_CATEGORY_PROD_SUPPORT));
        }

        if (scope.isAttributeAssignments()
                && (element instanceof AbstractAggregateScope || element instanceof AbstractScopeTask))
        {
            AttributeAssignment[] paaArray = new AttributeAssignment[nbChildren + 1];
            for (int i = 0; i < nbChildren; i++)
            {
                AttributeAssignment paa = AttributeAssignmentFixture
                    .createAssignmentFixture(element, ctc);
                paaArray[i] = paa;
            }

            //Insert a null object
            paaArray[nbChildren] = null;
            if (element instanceof AbstractAggregateScope)
            {
                AbstractAggregateScope absAgg = (AbstractAggregateScope) element;
                absAgg.setAttributeAssignments(paaArray);
            }

            if (element instanceof AbstractScopeTask)
            {
                AbstractScopeTask absScopeTask = (AbstractScopeTask) element;
                absScopeTask.setAttributeAssignments(paaArray);
            }
        }

        if (scope.isClientCostCenter() && element instanceof AbstractScope)
        {
            ClientCostCenter ccc = ClientCostCenterFixture
                .createSaveFixture(ctc);
            ((AbstractScope) element).setClientCostCenter(ccc);
        }

        if (scope.getDocumentFolder() != null
                && element instanceof AbstractScope)
        {
            // Can't create a DocumentFolder, must load the root after the scope element has been created
            if (element.getID() != null && ((AbstractScope)element).getDocumentFolder() != null) 
            {
	            DocumentFolder df = (DocumentFolder) DocumentElementFixture
	                    .createFixture(element, new DocumentFolder(), 0, scope.getDocumentFolder(), ctc);
	            df.setParent(((AbstractScope)element).getDocumentFolder());
	            ((AbstractScope)element).getDocumentFolder().setChildren(new DocumentElement[] { df });
            }
        }

        if (scope.isDuplicates() && element instanceof AbstractScope
                && element.getParent() != null)
        {
            AbstractScope abstractScope = (AbstractScope) element;

            // Decide which parent to use for the new ScopeElements - it must
            // have been previously saved
            AggregateScope parent = (AggregateScope) element.getParent();
            
            while (parent != null && parent.getID() == null)
            {
                parent = (AggregateScope) parent.getParent();
            }
            
            // Create duplicates only if me managed to get a valid parent with an id.
            if (parent != null)
            {
	            // Array for existing ScopeElements in the root ScopeFolder that has
	            // already been saved
	            // We will add the new ScopeElements to it
	            int numScopeElements = 0;
	            if (parent.getChildren() != null)
	            {
	                numScopeElements = parent.getChildren().length;
	            }
	            ScopeElement[] scopeElementsArray = new ScopeElement[numScopeElements
	                    + nbChildren];
	            for (int i = 0; i < numScopeElements; ++i)
	            {
	                scopeElementsArray[nbChildren + i] = parent.getChildren()[i];
	            }
	
	            // New duplicates
	            Duplicate[] duplicateArray = new Duplicate[nbChildren + 1];
	            for (int i = 0; i < nbChildren; ++i)
	            {
	                // Create new Duplicate with scopeElement of correct type
	                ScopeElement se = null;
	                if (abstractScope instanceof Action)
	                {
	                    se = ScopeElementFixture.createActionFixture(null, 0,
	                        parent, ctc);
	                }
	                else if (abstractScope instanceof ChangeRequest)
	                {
	                    se = ScopeElementFixture.createChangeRequestFixture(null,
	                        0, parent, ctc);
	                }
	                else if (abstractScope instanceof Defect)
	                {
	                    se = ScopeElementFixture.createDefectFixture(null, 0,
	                        parent, ctc);
	                }
	                else if (abstractScope instanceof Issue)
	                {
	                    se = ScopeElementFixture.createIssueFixture(null, 0,
	                        parent, ctc);
	                }
	                else if (abstractScope instanceof Requirement)
	                {
	                    se = ScopeElementFixture.createRequirementFixture(null, 0,
	                        parent, ctc);
	                }
	                else if (abstractScope instanceof Risk)
	                {
	                    se = ScopeElementFixture.createRiskFixture(null, 0, parent,
	                        ctc);
	                }
	                else if (abstractScope instanceof ServiceRequest)
	                {
	                    se = ScopeElementFixture.createServiceRequestFixture(null, 0, parent,
	                        ctc);
	                }
	                se = (AbstractScope)ctc.save(se, null).getRpmObject();
	
	                Duplicate pd = new Duplicate();
	                pd.setScopeElement(se);
	                pd.setParent(element);
	
	                scopeElementsArray[i] = se;
	                duplicateArray[i] = pd;
	            }
	            duplicateArray[nbChildren] = null;
	            abstractScope.setDuplicates(duplicateArray);
	            parent.setChildren(scopeElementsArray);
	        }
        }

        if (scope.isImpact() && element instanceof AbstractScope)
        {
            Impact impact = new Impact();
            HelperTestUtilities.setByIntrospection(impact, random(),
                skipSetters);
            ((AbstractScope) element).setImpact(impact);
        }

        if ((scope.getMitigationFactors() != null) && element instanceof Risk) {
			MitigationFactor[] mitigationFactorArray = new MitigationFactor[nbChildren + 1];
        	MitigationFactor mitigationFactor = null;
            for(int i=0; i <  nbChildren; i++) {
            	mitigationFactor= MitigationFactorFixture.createFixture( element, ctc); 
                mitigationFactorArray[i] = mitigationFactor;
            }
            mitigationFactorArray[nbChildren] = null;
        	((Risk)element).setMitigationFactors(mitigationFactorArray);
		}
		

        if (scope.isOrderOfMagnitudeCostBenefits()
                && element instanceof AbstractScope)
        {
            OrderOfMagnitudeCostBenefits orderOfMagnitudeCostBenefits = new OrderOfMagnitudeCostBenefits();
            HelperTestUtilities.setByIntrospection(
                orderOfMagnitudeCostBenefits, random(), skipSetters);
            ((AbstractScope) element)
                .setOrderOfMagnitudeCostBenefits(orderOfMagnitudeCostBenefits);
        }

        if (scope.isRiskMatrix() && element instanceof Risk)
        {
            RiskMatrix riskMatrix = new RiskMatrix();
            riskMatrix.setImpact(new Integer(40));
            riskMatrix.setProbability(new Integer(80));
            riskMatrix.setPrecision(PrecisionType.Medium);
            riskMatrix.setConsequenceCost(new Double(56.78));
            ((Risk) element).setRiskMatrix(riskMatrix);
        }

        if (scope.isRtfAssignments()
                && (element instanceof AbstractScopeTask || element instanceof AbstractAggregateScope))
        {
            RtfAssignment[] rtfAssignments = RtfAssignmentFixture
                .createAllAssignmentFixtures(element, ctc);

            if (element instanceof AbstractScopeTask)
            {
                ((AbstractScopeTask) element).setRtfAssignments(rtfAssignments);
            }

            if (element instanceof AbstractAggregateScope)
            {
                ((AbstractAggregateScope) element)
                    .setRtfAssignments(rtfAssignments);
            }
        }

        if (scope.getAssignedScorecard() != null
                && element instanceof AbstractScope)
        {
            if (element.getID() != null)
            {
                ScopeManagementModuleScope moduleScope = ScopeManagementModuleFixture
                .createFullScope();
                
                ScopeManagementModule module = ScopeManagementModuleFixture.createFixture(moduleScope, ctc);
                module = (ScopeManagementModule) ctc.save(module, moduleScope).getRpmObject();
               
                ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];
                
               element = (ScopeElement)PublishedScorecardFixture
                    .createAssignmentFixture(element, scope, ctc,
                        ReloadType.ReloadResult, scorecardFolder, 1);
            }
        }

        if (scope.isState() && element instanceof AbstractScope)
        {
            RPMObject[] result = ctc.loadFromXpath("/"
                    + StringUtil.getShortClassName(element.getClass())
                    + "State[value='Open']", null);
            ((AbstractScope) element).setState((DatafieldState) result[0]);
        }

        if (scope.getWorksheetFinancials() != null
                && element instanceof AbstractAggregateScope)
        {
            AbstractAggregateScope aaScope = (AbstractAggregateScope) element;

            WorksheetFinancial[] wfArray = new WorksheetFinancial[nbChildren];
            for (int i = 0; i < nbChildren; i++)
            {
                WorksheetFinancial wf = FinancialFixture
                    .createWorksheetFixture(scope.getWorksheetFinancials(),
                        aaScope, ctc);
                wfArray[i] = wf;
            }
            aaScope.setWorksheetFinancials(wfArray);
        }

        if (scope.getWbsFinancials() != null)
        {
            if (element instanceof AbstractAggregateScope)
            {
                AbstractAggregateScope aaScope = (AbstractAggregateScope) element;
                WbsFinancial[] wfArray = new WbsFinancial[nbChildren];
                for (int i = 0; i < nbChildren; i++)
                {
                    WbsFinancial wf = FinancialFixture.createWbsFixture(scope
                        .getWbsFinancials(), aaScope, ctc);
                    wfArray[i] = wf;
                }
                aaScope.setWbsFinancials(wfArray);
            }
            else if (element instanceof AbstractScope)
            {
                AbstractScope aaScope = (AbstractScope) element;
                WbsFinancial[] wfArray = new WbsFinancial[nbChildren];
                for (int i = 0; i < nbChildren; i++)
                {
                    WbsFinancial wf = FinancialFixture.createWbsFixture(scope
                        .getWbsFinancials(), aaScope, ctc);
                    wfArray[i] = wf;
                }
                aaScope.setWbsFinancials(wfArray);
            }
        }
    }

    public static ScopeElement updateFixture(ScopeElement element,
            ScopeElementScope scope, int nbChildren, ClientTestController ctc)
            throws Exception
    {
        if (scope == null || element == null)
            return null;

        HelperTestUtilities.setByIntrospection(element, random(),
            localSkipSetters);
       
        if (scope.getChildren() != null
                && element instanceof AggregateScope
                && ((AggregateScope) element).getChildren() != null)
        {
            ScopeElement[] children = ((AggregateScope) element).getChildren();

            for (int i = 0; i < children.length; ++i)
            {
                if (children[i] instanceof AggregateScope)
                {
                    children[i] = updateFixture(children[i], scope, nbChildren,
                        ctc);
                }
            }
            
            // task must be false when the object has children
            // Set values that were skipped
            if (element instanceof AbstractScope)
            {
                AbstractScope absScope = (AbstractScope) element;
                absScope.setTask(Boolean.FALSE);
                absScope.setEstimatedFinishDate(absScope.getEstimatedStartDate());
            }
        }
        else
        {
            // Set values that were skipped
            if (element instanceof AbstractScope)
            {
                AbstractScope absScope = (AbstractScope) element;
                absScope.setDefaultScope(Boolean.FALSE);
                absScope.setTask(Boolean.FALSE);
	            absScope.setEstimatedFinishDate(absScope.getEstimatedStartDate());
            }
        }

        return updateScopedContainer(element, scope, nbChildren, ctc);
    }

    public static ScopeElement updateScopedContainer(ScopeElement element,
            ScopeElementScope scope, int newSize, ClientTestController ctc)
            throws Exception
    {
        if (scope == null || element == null)
            return null;

        if (element.getID() == null)
            return element;
        
        if(element instanceof AbstractScope)
        {
            ((AbstractScope)element).setPublished(PublishedType.ToTemplate);
        }

        if (scope.getResourceTaskAssignments() != null)
        {
            // TODO make sure it is tested in ResourceTaskAssignment tests
//            ResourceRoleAssignment rra[] = element.getResourceTaskAssignments();
//
//            if (rra != null)
//            {
//
//                int nbChildren = rra.length;
//                ResourceRoleAssignment updatedRRA[] = new ResourceRoleAssignment[nbChildren * 2];
//                for (int i = 0; i < nbChildren; i++)
//                {
//                    //delete existing Resource Role Assignment
//                    rra[i].setDeleted(Boolean.TRUE);
//                    updatedRRA[i] = rra[i];
//
//                    //create new ResourceRole Assignments
//                    ResourceRoleAssignment resRolassgn = new ResourceRoleAssignment();
//                    resRolassgn.setParent(element);
//                    Resource resource = ResourceFixture.createSaveFixture(ctc);
//                    resRolassgn.setResource(resource);
//                    updatedRRA[nbChildren + i] = resRolassgn;
//                }
//                element.setResourceAssignments(updatedRRA);
//            }
        }

        if (scope.isApprovedCostBenefits()
                && ((element instanceof AbstractRequest)
                        || (element instanceof Requirement) || (element instanceof Action)))
        {
            if (element instanceof AbstractRequest)
            {
                if (((AbstractRequest) element).getApprovedCostBenefits() != null)
                {
                    HelperTestUtilities.setByIntrospection(
                        ((AbstractRequest) element).getApprovedCostBenefits(),
                        random(), skipSetters);
                }
            }
            else if (element instanceof Requirement)
            {
                if (((Requirement) element).getApprovedCostBenefits() != null)
                {
                    HelperTestUtilities.setByIntrospection(
                        ((Requirement) element).getApprovedCostBenefits(),
                        random(), skipSetters);
                }
            }
            else
            {
                if (((Action) element).getApprovedCostBenefits() != null)
                {
                    HelperTestUtilities.setByIntrospection(((Action) element)
                        .getApprovedCostBenefits(), random(), skipSetters);
                }
            }
        }

        if (scope.isAsset() && element instanceof AbstractScope)
        {
            if(element instanceof Requirement)
            {
                ((AbstractScope) element).setAsset(AssetFixture
                    .createSaveFixture(ctc));
            }
            else 
            {
                ((AbstractScope) element).setAsset(null);
            }
        }

        if (scope.isAssetCode() && element instanceof AbstractScope)
        {
            AbstractScope uScopeElement = (AbstractScope) element;
            AssetFinancialCategory a = uScopeElement.getAssetCode();

            if (a != null)
            {

                a.setID(DatafieldFixture.ASSET_FINANCIAL_CATEGORY_APP);
                a.setValue(DatafieldFixture.ASSET_FINANCIAL_CATEGORY_APP);
            }
        }

        if (scope.isAttributeAssignments()
                && (element instanceof AbstractAggregateScope || element instanceof AbstractScopeTask))
        {
            int nbChildren;
            AttributeAssignment updatedAssignment[];
            AttributeAssignment aAssignment[];
            if (element instanceof AbstractAggregateScope)
            {
                AbstractAggregateScope uScopeElement = (AbstractAggregateScope) element;
                aAssignment = uScopeElement.getAttributeAssignments();

                if (aAssignment != null)
                {

                    nbChildren = aAssignment.length;
                    updatedAssignment = new AttributeAssignment[2 * nbChildren];
                    for (int i = 0; i < nbChildren; i++)
                    {
                        //delete existing Assignment Attributes
                        aAssignment[i].setDeleted(Boolean.TRUE);
                        updatedAssignment[i] = aAssignment[i];

                        //Add new Assignment Attributes
                        AttributeAssignment paa = AttributeAssignmentFixture
                            .createAssignmentFixture(element, ctc);
                        updatedAssignment[nbChildren + i] = paa;
                    }
                    uScopeElement.setAttributeAssignments(updatedAssignment);
                }
            }
            else
            {
                AbstractScopeTask uScopeElement = (AbstractScopeTask) element;
                aAssignment = uScopeElement.getAttributeAssignments();

                if (aAssignment != null)
                {

                    nbChildren = aAssignment.length;
                    updatedAssignment = new AttributeAssignment[2 * nbChildren];
                    for (int i = 0; i < nbChildren; i++)
                    {
                        //delete existing Assignment Attributes
                        aAssignment[i].setDeleted(Boolean.TRUE);
                        updatedAssignment[i] = aAssignment[i];

                        //Add new Assignment Attributes
                        AttributeAssignment paa = AttributeAssignmentFixture
                            .createAssignmentFixture(element, ctc);
                        updatedAssignment[nbChildren + i] = paa;
                    }
                    uScopeElement.setAttributeAssignments(updatedAssignment);
                }
            }
        }

        if (scope.isClientCostCenter() && element instanceof AbstractScope)
        {
            ((AbstractScope) element)
                .setClientCostCenter(ClientCostCenterFixture
                    .createSaveFixture(ctc));
        }
        
        if (scope.getDocumentFolder() != null
                && element instanceof AbstractScope)
        {
            AbstractScope abstractScope = (AbstractScope) element;
            DocumentFolder rootDocumentFolder = abstractScope.getDocumentFolder();
            if(rootDocumentFolder!=null){
                DocumentFolder df = (DocumentFolder)DocumentElementFixture.createFixture(rootDocumentFolder, new DocumentFolder(), 0, scope.getDocumentFolder(), ctc);
                rootDocumentFolder.setChildren(new DocumentElement[]{df});
            }
        }        

        if (scope.isImpact() && element instanceof AbstractScope)
        {
            if (((AbstractScope) element).getImpact() != null)
            {
                HelperTestUtilities.setByIntrospection(
                    ((AbstractScope) element).getImpact(), random(),
                    skipSetters);
            }
        }

        if (scope.isOrderOfMagnitudeCostBenefits()
                && element instanceof AbstractScope)
        {
            if (((AbstractScope) element).getOrderOfMagnitudeCostBenefits() != null)
            {
                HelperTestUtilities
                    .setByIntrospection(((AbstractScope) element)
                        .getOrderOfMagnitudeCostBenefits(), random(),
                        skipSetters);
            }
        }

        if (scope.isRiskMatrix() && element instanceof Risk)
        {
            if (((Risk) element).getRiskMatrix() != null)
            {
                RiskMatrix riskMatrix = ((Risk) element).getRiskMatrix();
    			riskMatrix.setImpact(new Integer(20));
    			riskMatrix.setProbability(new Integer(60));
    			riskMatrix.setPrecision(PrecisionType.Low);
    			riskMatrix.setConsequenceCost(new Double(32.02));
//    			((Risk) element).setRiskMatrix(riskMatrix);
            }
            

        }

        if (scope.isRtfAssignments()
                && (element instanceof AbstractScopeTask || element instanceof AbstractAggregateScope))
        {
            RtfAssignment rtfAssignment[];
            if (element instanceof AbstractScopeTask)
            {
                rtfAssignment = ((AbstractScopeTask) element)
                    .getRtfAssignments();
            }
            else
            {
                rtfAssignment = ((AbstractAggregateScope) element)
                    .getRtfAssignments();
            }
            for (int i = 0; i < rtfAssignment.length; i++)
            {
                RtfAssignmentFixture.updateAssignmentFixture(rtfAssignment[i]);
            }
        }

        if (scope.getAssignedScorecard() != null
                && element instanceof AbstractScope)
        {
            ScopeManagementModuleScope moduleScope = ScopeManagementModuleFixture
            .createFullScope();
            
            ScopeManagementModule module = ScopeManagementModuleFixture.createFixture(moduleScope, ctc);
            module = (ScopeManagementModule) ctc.save(module, moduleScope).getRpmObject();
           
            ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];
            
            element = (ScopeElement) PublishedScorecardFixture
                .createAssignmentFixture(element, scope, ctc,
                    ReloadType.ReloadResult, scorecardFolder, 1);
        }

        if (scope.isDuplicates() && element instanceof AbstractScope
                && element.getParent() != null)
        {
            AbstractScope uScopeElement = (AbstractScope) element;

            // Decide which parent to use for the new ScopeElements - it must
            // have been previously saved
            AggregateScope parent;
            if (element.getParent().getID() != null)
            {
                parent = (AggregateScope) element.getParent();
            }
            else
            {
                parent = (AggregateScope) element.getParent().getParent();
            }

            int nbChildren = 0;
            Duplicate updatedDuplicate[];
            Duplicate aDuplicate[];
            aDuplicate = uScopeElement.getDuplicates();
            if (aDuplicate != null)
            {
                nbChildren = aDuplicate.length;
            }
            updatedDuplicate = new Duplicate[2 * nbChildren];

            if (nbChildren != 0)
            {
                // Array for existing ScopeElements in the root ScopeFolder that
                // has already been saved
                // We will add the new ScopeElements to it
                int numScopeElements = 0;
                if (parent.getChildren() != null)
                {
                    numScopeElements = parent.getChildren().length;
                }
                ScopeElement[] scopeElementsArray = new ScopeElement[numScopeElements
                        + nbChildren];
                for (int i = 0; i < numScopeElements; ++i)
                {
                    scopeElementsArray[nbChildren + i] = parent
                        .getChildren()[i];
                }

                for (int i = 0; i < nbChildren; ++i)
                {
                    // Delete existing Duplicate
                    if (aDuplicate[i] != null)
                    {
                        aDuplicate[i].setDeleted(Boolean.TRUE);
                        updatedDuplicate[i] = aDuplicate[i];

                        // Add new Duplicate with scopeElement of correct type
                        ScopeElement se = null;
                        if (uScopeElement instanceof Action)
                        {
                            se = ScopeElementFixture.createActionFixture(null,
                                0, parent, ctc);
                        }
                        else if (uScopeElement instanceof ChangeRequest)
                        {
                            se = ScopeElementFixture
                                .createChangeRequestFixture(null, 0, parent,
                                    ctc);
                        }
                        else if (uScopeElement instanceof Defect)
                        {
                            se = ScopeElementFixture.createDefectFixture(null,
                                0, parent, ctc);
                        }
                        else if (uScopeElement instanceof Issue)
                        {
                            se = ScopeElementFixture.createIssueFixture(null,
                                0, parent, ctc);
                        }
                        else if (uScopeElement instanceof Requirement)
                        {
                            se = ScopeElementFixture.createRequirementFixture(
                                null, 0, parent, ctc);
                        }
                        else if (uScopeElement instanceof Risk)
                        {
                            se = ScopeElementFixture.createRiskFixture(null, 0,
                                parent, ctc);
                        }
                        ctc.save(se, null);

                        Duplicate pd = new Duplicate();
                        pd.setScopeElement(se);
                        pd.setParent(element);

                        scopeElementsArray[i] = se;
                        updatedDuplicate[nbChildren + i] = pd;
                    }
                }
                uScopeElement.setDuplicates(updatedDuplicate);
                parent.setChildren(scopeElementsArray);
            }
        }

        if(scope.getMitigationFactors() != null && element instanceof Risk)
        {
            Risk risk = (Risk)element;
            MitigationFactor[] currentFactors = risk.getMitigationFactors();
            MitigationFactor[] mitigationFactorArray = new MitigationFactor[newSize];
            int nextIndex = 0;
            if(currentFactors != null)
            {
                nextIndex = currentFactors.length;
                for(int i=0; i < currentFactors.length; i++)
                {
                    mitigationFactorArray[i] = currentFactors[i];
                }
            }
            
            MitigationFactor mitigationFactor = null;
            for (int i = nextIndex; i < newSize; i++)
            {
                mitigationFactor = MitigationFactorFixture.createFixture(
                    element, ctc);
                mitigationFactorArray[i] = mitigationFactor;
            }
            risk.setMitigationFactors(mitigationFactorArray);
        }
        
        if (scope.getWorksheetFinancials() != null
                && element instanceof AbstractAggregateScope)
        {
            AbstractAggregateScope aaScope = (AbstractAggregateScope) element;

            WorksheetFinancial updatedWFArray[] = new WorksheetFinancial[newSize];
            for (int i = 0; i < newSize; i++)
            {
                //create new WorkSheet Financials
                WorksheetFinancial wf = FinancialFixture
                    .createWorksheetFixture(scope.getWorksheetFinancials(),
                        aaScope, ctc);
                wf.setEstimate(new Double(1000.0));
                wf.setProbabilityAmount(wf.getEstimate());
                wf.setActual(new Double(1003.0));
                wf.setRemaining(new Double(-3));
                wf.setETC(new Double(1004.0));
                wf.setEAC(new Double(2007.0));
                wf.setEACVariance(new Double(1007.0));
                wf.setBaselineVariance(new Double(1000.0));
                updatedWFArray[i] = wf;
            }
            aaScope.setWorksheetFinancials(updatedWFArray);
        }

        if (scope.getWbsFinancials() != null
                && (element instanceof AbstractAggregateScope || element instanceof AbstractScope))
        {
            int nbChildren;
            WbsFinancial updatedWbsFinanacial[];
            WbsFinancial wbsFinanacial[];
            if (element instanceof AbstractAggregateScope)
            {
                AbstractAggregateScope uScopeElement = (AbstractAggregateScope) element;
                wbsFinanacial = uScopeElement.getWbsFinancials();

                if (wbsFinanacial != null)
                {

                    nbChildren = wbsFinanacial.length;
                    updatedWbsFinanacial = new WbsFinancial[2 * nbChildren];
                    for (int i = 0; i < nbChildren; i++)
                    {
                        //delete existing WBSFinancials
                        wbsFinanacial[i].setDeleted(Boolean.TRUE);
                        updatedWbsFinanacial[i] = wbsFinanacial[i];

                        //Add new WBSFinancial
                        WbsFinancial wf = FinancialFixture.createWbsFixture(
                            scope.getWbsFinancials(), uScopeElement, ctc);
                        updatedWbsFinanacial[nbChildren + i] = wf;
                    }
                    uScopeElement.setWbsFinancials(updatedWbsFinanacial);
                }
                else
                {
                    if (element instanceof AbstractAggregateScope)
                    {
                        AbstractAggregateScope aaScope = (AbstractAggregateScope) element;
                        WbsFinancial[] wfArray = new WbsFinancial[newSize];
                        for (int i = 0; i < newSize; i++)
                        {
                            WbsFinancial wf = FinancialFixture
                                .createWbsFixture(scope.getWbsFinancials(),
                                    aaScope, ctc);
                            wfArray[i] = wf;
                        }
                        aaScope.setWbsFinancials(wfArray);
                    }
                    else if (element instanceof AbstractScope)
                    {
                        AbstractScope aaScope = (AbstractScope) element;
                        WbsFinancial[] wfArray = new WbsFinancial[newSize];
                        for (int i = 0; i < newSize; i++)
                        {
                            WbsFinancial wf = FinancialFixture
                                .createWbsFixture(scope.getWbsFinancials(),
                                    aaScope, ctc);
                            wfArray[i] = wf;
                        }
                        aaScope.setWbsFinancials(wfArray);
                    }
                }
            }
        }
        return element;
    }

    public static void compare(ScopeElement element1, ScopeElement element2,
            ScopeElementScope scope) throws AssertionFailedError, Exception
    {

        RPMObjectFixture.compareWithSkip(element1, element2, localSkipGetters);

        if (scope == null)
            return;

        //Handling Parents
        if (scope.getParent() != null)
        {
            ScopeElement parent1 = (ScopeElement) element1.getParent();
            ScopeElement parent2 = (ScopeElement) element2.getParent();
            if (parent1 != null && parent2 == null)
                Assert.fail("RPMObject in scope is null");

            ScopeElementScope parentScope = (ScopeElementScope)scope.getParent();
            parentScope.setChildren(null);

            ScopeElementFixture.compare(parent1, parent2, parentScope);
        }

        //Handling Children
        if (scope.getChildren() != null)
        {
            if (element1 instanceof AggregateScope)
            {
                ScopeElement scopeelements1[] = ((AggregateScope) element1)
                    .getChildren();
                ScopeElement scopeelements2[] = ((AggregateScope) element2)
                    .getChildren();

                ScopeElementScope childrenScope = scope.getChildren();
                childrenScope.setParent(null);

                ScopeElementFixture.compare(scopeelements1, scopeelements2,
                    childrenScope);
            }
        }

        //Handling Scope Elementes

        //ScheduleDates        
        if (scope.isScheduleDates())
        {

            if (element1 instanceof AbstractScope)
            {

                ScopeScheduleDate sd1 = ((AbstractScope) element1).getApprovedDate();
                ScopeScheduleDate sd2 = ((AbstractScope) element2).getApprovedDate();

                if (sd1 != null && sd2 == null)
                {
                    Assert.fail("RPMObject in scope is null");
                }
                
                ScopeScheduleDateFixture
                    .compare(sd1, sd2);
            }
        }   
        
        //ResourceRoleAssignment
        // Can't always compare... when you first insert, it will automatically assign the creator of the ScopeElement, even if you don't set a task assignment
        if (scope.getResourceTaskAssignments() != null)
        {
            ResourceTaskAssignment r1[] = element1.getResourceTaskAssignments();
            ResourceTaskAssignment r2[] = element2.getResourceTaskAssignments();

            ResourceTaskAssignmentFixture.compare(r1, r2);
        }

        //WorkElement
        if (scope.getWorkElement() != null)
        {
            WorkElement w1 = element1.getWorkElement();
            WorkElement w2 = element2.getWorkElement();

            if (w1 != null && w2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }

            WorkElementFixture.compare(w1, w2, scope.getWorkElement());
        }

        //ApprovedCostBenefits
        if (scope.isApprovedCostBenefits()
                && ((element1 instanceof AbstractRequest)
                        || (element1 instanceof Requirement) || (element1 instanceof Action)))
        {
            ApprovedCostBenefits a1 = null;
            ApprovedCostBenefits a2 = null;

            if (element1 instanceof AbstractRequest)
            {
                a1 = ((AbstractRequest) element1).getApprovedCostBenefits();
                a2 = ((AbstractRequest) element2).getApprovedCostBenefits();
            }
            else if (element1 instanceof Requirement)
            {
                a1 = ((Requirement) element1).getApprovedCostBenefits();
                a2 = ((Requirement) element2).getApprovedCostBenefits();
            }
            else
            {
                a1 = ((Action) element1).getApprovedCostBenefits();
                a2 = ((Action) element2).getApprovedCostBenefits();
            }

            if (a1 != null && a2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }

            RPMObjectFixture.compare(a1, a2);
        }

        //Asset
        //Child as the same asset as AbstractScope parent
        if (scope.isAsset() && element1 instanceof AbstractScope)
        {
            if (!(element1.getParent() instanceof AbstractScope))
            {
                Asset a1 = ((AbstractScope) element1).getAsset();
                Asset a2 = ((AbstractScope) element2).getAsset();

                if (a1 != null && a2 == null)
                {
                    Assert.fail("RPMObject in scope is null");
                }
                AssetFixture.compare(a1, a1, null);
            }
        }

        //AssetCode
        if (scope.isAssetCode() && element1 instanceof AbstractScope)
        {
            AssetFinancialCategory a1 = ((AbstractScope) element1)
                .getAssetCode();
            AssetFinancialCategory a2 = ((AbstractScope) element2)
                .getAssetCode();

            if (a1 != null && a2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }

            DatafieldFixture.compare(a1, a2);
        }

        //AssignedScorecard
        if (scope.getAssignedScorecard() != null
                && element1 instanceof AbstractScope)
        {
            AssignedScorecard a1 = ((AbstractScope) element1)
                .getAssignedScorecard();
            AssignedScorecard a2 = ((AbstractScope) element2)
                .getAssignedScorecard();

            if (a1 != null && a2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }

            PublishedScorecardFixture.compare(a1, a2, scope
                .getAssignedScorecard());
        }

        //AttributeAssignment
        if (scope.isAttributeAssignments()
                && (element1 instanceof AbstractAggregateScope || element1 instanceof AbstractScope))
        {
            AttributeAssignment a1[];
            AttributeAssignment a2[];

            if (element1 instanceof AbstractAggregateScope)
            {
                a1 = ((AbstractAggregateScope) element1)
                    .getAttributeAssignments();
                a2 = ((AbstractAggregateScope) element2)
                    .getAttributeAssignments();
            }
            else
            {
                a1 = ((AbstractScopeTask) element1).getAttributeAssignments();
                a2 = ((AbstractScopeTask) element2).getAttributeAssignments();
            }

            AttributeAssignmentFixture.compare(a1, a2);
        }

        //ClientCostCenter
        if (scope.isClientCostCenter() && element1 instanceof AbstractScope)
        {
            ClientCostCenter ccc1 = ((AbstractScope) element1)
                .getClientCostCenter();
            ClientCostCenter ccc2 = ((AbstractScope) element2)
                .getClientCostCenter();

            if (ccc1 != null && ccc2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }

            ClientCostCenterFixture.compare(ccc1, ccc2, null);
        }

        //DocumentFolder
        if (scope.getDocumentFolder() != null
                && element1 instanceof AbstractScope)
        {
            DocumentFolder d1 = ((AbstractScope) element1).getDocumentFolder();
            DocumentFolder d2 = ((AbstractScope) element2).getDocumentFolder();

            if (d1 != null && d2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }

            if (d1 != null) // If d1=null, then the DocumentFolder wasn't originally loaded (such as suring a create) and we cannot compare
            {
                DocumentElementFixture.compare(d1, d2, scope.getDocumentFolder());
            }
        }

        //Duplicates
        if (scope.isDuplicates() && element1 instanceof AbstractScope)
        {
            RPMObject[] objects1 = removeNullAndDeletedRPMObjects(((AbstractScope) element1)
                .getDuplicates());
            RPMObject[] objects2 = removeNullAndDeletedRPMObjects(((AbstractScope) element2)
                .getDuplicates());
            sortWithComparator(objects1, new RPMObjectIDComparator());
            sortWithComparator(objects2, new RPMObjectIDComparator());

            RPMObjectFixture.compare(objects1, objects2);

            if (objects1 != null)
            {
                for (int i = 0; i < objects1.length; i++)
                {
                    Duplicate d1 = (Duplicate) objects1[i];
                    Duplicate d2 = (Duplicate) objects2[i];
                    ScopeElementFixture.compare(d1.getScopeElement(), d2
                        .getScopeElement(), null);
                }
            }
        }

        //FinancialAssignments
        if (scope.getWbsFinancials() != null
                && (element1 instanceof AbstractScope || element2 instanceof AbstractScopeTask))
        {
//            WbsFinancial fa1[];
//            WbsFinancial fa2[];
//
//            if (element1 instanceof AbstractScope)
//            {
//                fa1 = ((AbstractScope) element1).getWbsFinancials();
//                fa2 = ((AbstractScope) element2).getWbsFinancials();
//            }
//            else
//            {
//                fa1 = ((AbstractScopeTask) element1).getWbsFinancials();
//                fa2 = ((AbstractScopeTask) element2).getWbsFinancials();
//            }

//            FinancialFixture.compare(fa1, fa2, scope.getWbsFinancial());
        }

        if (scope.getWorksheetFinancials() != null)
        {
            if (element1 instanceof AbstractAggregateScope)
            {
                //make sure it's not a Requirement
                if(!(element1 instanceof Requirement))
                {
//                    WorksheetFinancial[] wf1 = ((AbstractAggregateScope) element1)
//                        .getWorksheetFinancials();
//                    WorksheetFinancial[] wf2 = ((AbstractAggregateScope) element2)
//                        .getWorksheetFinancials();
//                    FinancialFixture.compare(wf1, wf2, scope
//                        .getWorksheetFinancial());
                }
            }
        }

        //Impact
        if (scope.isImpact() && element1 instanceof AbstractScope)
        {
            Impact i1 = ((AbstractScope) element1).getImpact();
            Impact i2 = ((AbstractScope) element2).getImpact();

            if (i1 != null && i2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }

            RPMObjectFixture.compare(i1, i2);
        }

        //  MitigationFactorAssignment
		if ((scope.getMitigationFactors() != null) && element1 instanceof Risk) {
			MitigationFactor m1[] = ((Risk) element1).getMitigationFactors();
			MitigationFactor m2[] = ((Risk) element2).getMitigationFactors();
			
			if (m1 != null && m2 == null) {
				Assert.fail("RPMObject in scope is null");
			}
			MitigationFactorFixture.compare(m1,m2);
		}

        if (scope.isOrderOfMagnitudeCostBenefits()
                && element1 instanceof AbstractScope)
        {
            OrderOfMagnitudeCostBenefits o1 = ((AbstractScope) element1)
                .getOrderOfMagnitudeCostBenefits();
            OrderOfMagnitudeCostBenefits o2 = ((AbstractScope) element2)
                .getOrderOfMagnitudeCostBenefits();

            if (o1 != null && o2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }

            RPMObjectFixture.compare(o1, o2);
        }

        //RiskMatrix
        if (scope.isRiskMatrix() && element1 instanceof Risk)
        {
            RiskMatrix r1 = ((Risk) element1).getRiskMatrix();
            RiskMatrix r2 = ((Risk) element2).getRiskMatrix();

            if (r1 != null && r2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }

            RPMObjectFixture.compareWithSkip(r1, r2, riskMatrixSkipGetters);

            if (r1 != null && r2 != null)
            {
                Assert.assertTrue(r1.getPrecision() == r2.getPrecision());
            }
        }

        //RTFAssignments
        if (scope.isRtfAssignments()
                && (element1 instanceof AbstractScopeTask || element2 instanceof AbstractAggregateScope))
        {
            RtfAssignment r1[];
            RtfAssignment r2[];

            if (element1 instanceof AbstractScopeTask)
            {
                r1 = ((AbstractScopeTask) element1).getRtfAssignments();
                r2 = ((AbstractScopeTask) element2).getRtfAssignments();
            }
            else
            {
                r1 = ((AbstractAggregateScope) element1).getRtfAssignments();
                r2 = ((AbstractAggregateScope) element2).getRtfAssignments();
            }

            RtfAssignmentFixture.compare(r1, r2);
        }

        //State
        if (scope.isState() && element1 instanceof AbstractScope)
        {
            DatafieldState state1 = ((AbstractScope) element1).getState();
            DatafieldState state2 = ((AbstractScope) element2).getState();

            if (state1 != null && state2 == null)
            {
                Assert.fail("RPMObject in scope is null");
            }

//            DatafieldFixture.compare(state1, state2);
        }

        //StatusUpdate
//        if (scope.isStatusUpdates()
//                && (element1 instanceof AbstractScopeTask || element1 instanceof AbstractAggregateScope))
//        {
//            StatusUpdate s1[];
//            StatusUpdate s2[];
//
//            if (element1 instanceof AbstractScopeTask)
//            {
//                s1 = ((AbstractScopeTask) element1).getStatusUpdates();
//                s2 = ((AbstractScopeTask) element2).getStatusUpdates();
//            }
//            else
//            {
//                s1 = ((AbstractAggregateScope) element1).getStatusUpdates();
//                s2 = ((AbstractAggregateScope) element2).getStatusUpdates();
//            }
//
//            if (s1 != null && s2 == null)
//            {
//                Assert.fail("RPMObject in scope is null");
//            }
//
//            //TODO Use the corresponding fixture to compare
//
//        }
    }

    public static void compare(ScopeElement[] sArray1, ScopeElement[] sArray2,
            ScopeElementScope scope) throws AssertionFailedError, Exception
    {

        if (sArray1 == null)
            return;

        sArray1 = removeDeletedScopeElements(sArray1);
        sortWithComparator(sArray1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(sArray2, RPMObjectNameAndIDComparator.INSTANCE);

        RPMObjectFixture.compare(sArray1, sArray2);

        if (sArray1 == null)
            return;

        for (int i = 0; i < sArray1.length; i++)
        {
            ScopeElementFixture.compare(sArray1[i], sArray2[i], scope);
        }
    }

    public static ScopeElement[] removeDeletedScopeElements(
            ScopeElement[] sArray)
    {

        ScopeElement[] newArray = null;
        ArrayList temp = new ArrayList(Arrays.asList(sArray));
        temp = removeNullAndDeletedRPMObjects(temp);

        if (temp != null && temp.size() > 0)
        {
            newArray = new ScopeElement[temp.size()];
            temp.toArray(newArray);
        }
        return newArray;
    }

    public static ArrayList mergeArrays(Object[] array1, Object[] array2)
    {
        ArrayList temp = new ArrayList();
        addArray(temp, array1);
        addArray(temp, array2);
        return temp;
    }

    private static void addArray(ArrayList result, Object[] array)
    {
        if (array != null && array.length != 0)
        {
            for (int i = 0; i < array.length; i++)
            {
                result.add(array[i]);
            }
        }
    }

    
    public static ScopeElement retrieveObject(ScopeElement element,
            ScopeElement objectType) throws Exception
    {
        if (Class.forName(objectType.getClass().getName()).isInstance(element))
            return element;
        if (element instanceof AggregateScope)
        {
            AggregateScope aggScope = (AggregateScope) element;
            if(aggScope.getChildren() == null) return null;
            ScopeElement[] scopeElements = aggScope.getChildren();
            for (int i = 0; i < scopeElements.length; i++)
            {
                ScopeElement rElt = retrieveObject(scopeElements[i], objectType);
                if(rElt != null) return rElt;
            }
        }
        return null;
    }
    
    /**
     * In case you need just any scope element for your tests, use this fixture,
     * it is currently creating/saving ChangeRequest with all the underlying structure
     * 
     * @param ctc
     * @return
     * @throws Exception
     */
    public static ScopeElement createSaveScopeElement(ClientTestController ctc)
    throws Exception {
       return createSaveChangeRequest(ctc); 
    }    
    
    public static ScopeElement createSaveScopeElement(ScopeElement rootParent,
			ScopeElement baseScopeElement, int nbChildren,
			ScopeElementScope scopeElementScope, ClientTestController ctc)
			throws Exception {
    		
    		if(baseScopeElement instanceof Requirement)
    		{
    			baseScopeElement = createRequirementFixture(scopeElementScope, nbChildren, rootParent, ctc);
    		}
    		else if(baseScopeElement instanceof ChangeRequest)
    		{
    			baseScopeElement = createChangeRequestFixture(scopeElementScope, nbChildren, rootParent, ctc);
    		}
    		else if(baseScopeElement instanceof ServiceRequest)
    		{
    			baseScopeElement = createServiceRequestFixture(scopeElementScope, nbChildren, rootParent, ctc);
    		}
    		else if(baseScopeElement instanceof Action)
    		{
    			baseScopeElement = createActionFixture(scopeElementScope, nbChildren, rootParent, ctc);
    		}
    		else if(baseScopeElement instanceof Defect)
    		{
    			baseScopeElement = createDefectFixture(scopeElementScope, nbChildren, rootParent, ctc);
    		}
    		else if(baseScopeElement instanceof Issue)
    		{
    			baseScopeElement = createIssueFixture(scopeElementScope, nbChildren, rootParent, ctc);
    		}
    		else if(baseScopeElement instanceof Risk)
    		{
    			baseScopeElement = createRiskFixture(scopeElementScope , nbChildren, rootParent, ctc);
    		}
    		else if(baseScopeElement instanceof ScopeMilestone)
    		{
    			ScopeElement parent = createSaveScopeElement(rootParent, new Requirement(), 1, null, ctc);
    			baseScopeElement = createScopeMilestoneFixture(scopeElementScope, nbChildren, parent, ctc);
    		}
    		else if(baseScopeElement instanceof ScopeTask)
    		{
    			ScopeElement parent = createSaveScopeElement(rootParent, new Requirement(), 1, null, ctc);
    			baseScopeElement = createScopeTaskFixture(scopeElementScope, nbChildren, parent, ctc);
    		}
    		else if(baseScopeElement instanceof ScopeDeliverable)
    		{
    			ScopeElement parent = createSaveScopeElement(rootParent, new Requirement(), 1, null, ctc);
    			baseScopeElement = createScopeDeliverableFixture(scopeElementScope, nbChildren, parent, ctc);
    		}
    		else if(baseScopeElement instanceof ScopeSummaryTask)
    		{
    			ScopeElement parent = createSaveScopeElement(rootParent, new Requirement(), 1, null, ctc);
    			baseScopeElement = createScopeSummaryTaskFixture(scopeElementScope, nbChildren, parent, ctc);
    		}
    		// TODO: support the other scope elements also
    		baseScopeElement = (ScopeElement)ctc.save(baseScopeElement,scopeElementScope).getRpmObject();
    		Assert.assertNotNull(baseScopeElement);
    		return baseScopeElement;
	}
    
    /**
     * Simple to use ChangeRequest fixture.
     * Creates/saves ChangeRequest with all the underlying structure.
     * 
     * If you need it loaded with a certain scope - just reload with scope 
     * after calling this method. Because it is too tricky to save the 
     * hierarchy with not null scope.
     * 
     * @param ctc
     * @return
     * @throws Exception
     */
    public static ChangeRequest createSaveChangeRequest(ClientTestController ctc)
            throws Exception
    {
        ScopeElementScope scopeScope = new ScopeElementScope();
        scopeScope.setChildren(scopeScope);
        ScopeFolder scopeFolder = (ScopeFolder) ScopeElementFixture
                .createFixture(scopeScope, ctc, 1);
        ChangeRequest changeRequest = (ChangeRequest) ScopeElementFixture
                .retrieveObject(scopeFolder, new ChangeRequest());
        recursiveScopeSave(scopeFolder, ctc);
        return changeRequest;
    }

    private static void recursiveScopeSave(AggregateScope scopeElement,
            ClientTestController ctc) throws Exception
    {
        SingleResult result = ctc.save(scopeElement, null);
        Assert.assertNull("ScopeElement save failed.", result.getErrors());
        scopeElement.setID(result.getRpmObject().getID());
        if (scopeElement.getChildren() != null
                && scopeElement.getChildren().length > 0)
        {
            for (int i = 0; i < scopeElement.getChildren().length; i++)
            {
                if (scopeElement.getChildren()[i] instanceof AggregateScope)
                {
                    recursiveScopeSave((AggregateScope) scopeElement
                            .getChildren()[i], ctc);
                }
                else
                {
                    ctc.save(scopeElement.getChildren()[i], null);
                }
            }
        }
    }    

    public static WorksheetFinancial[] createEmptyWorksheetFinancialSystemChargeCodesArray(
            AggregateScope parent, ClientTestController controller) throws Exception
    {
        WorksheetFinancial[] wWSheetFinancial = null;
        String SystemChargeCodeName = "System Charge Codes";
        RPMObject[] resultArray = null;
        ChargeCodeCategory wCCCategory = null;
        ChargeCodeCategory wExpenseCategory = null;
        
        ChargeCode wExpenseHour = null;
        ChargeCode wExpenseAvg = null;
        ChargeCode wExpenseLabor = null;
        ChargeCode wExpenseFTE = null;


        final int HOUR = 0;
        final int AVG_COST = 1;
        final int EXP_LABOR = 2;
        final int FTE = 3;

        // ChargeCodeScope needed to load the ChargeCodeCategory
        ChargeCodeScope wCCMainScope = new ChargeCodeScope();
        wCCMainScope.setChildren(new ChargeCodeScope());
        wCCMainScope.getChildren().setChildren(new ChargeCodeScope());

        resultArray = controller.loadFromXpath(
                "/ChargeCodeCategory[name='" + SystemChargeCodeName + "']",
                wCCMainScope);
        

        if (resultArray != null
                && resultArray[0] != null
                && (resultArray[0] instanceof ChargeCodeCategory)
                && parent != null)
        {
            wCCCategory = (ChargeCodeCategory) resultArray[0];
            wExpenseCategory = (ChargeCodeCategory) wCCCategory.getChildren()[0];
            
            //Creating Worksheet Financial
            wWSheetFinancial = new WorksheetFinancial[4];

            wExpenseHour = (ChargeCode) wExpenseCategory.getChildren()[0];
            wWSheetFinancial[HOUR] = new WorksheetFinancial();
            wWSheetFinancial[HOUR].setParent(parent);
            wWSheetFinancial[HOUR].setChargeCode(wExpenseHour);

            wExpenseAvg = (ChargeCode) wExpenseCategory.getChildren()[1];
            wWSheetFinancial[AVG_COST] = new WorksheetFinancial();
            wWSheetFinancial[AVG_COST].setParent(parent);
            wWSheetFinancial[AVG_COST].setChargeCode(wExpenseAvg);

            wExpenseLabor = (ChargeCode) wExpenseCategory.getChildren()[2];
            wWSheetFinancial[EXP_LABOR] = new WorksheetFinancial();
            wWSheetFinancial[EXP_LABOR].setParent(parent);
            wWSheetFinancial[EXP_LABOR].setChargeCode(wExpenseLabor);

            wExpenseFTE = (ChargeCode) wExpenseCategory.getChildren()[3];
            wWSheetFinancial[FTE] = new WorksheetFinancial();
            wWSheetFinancial[FTE].setParent(parent);
            wWSheetFinancial[FTE].setChargeCode(wExpenseFTE);

        }
        
        return wWSheetFinancial;        
    }

}
