/*
* Created on 12-Sep-2006
* @author rbadette
* Description
*/

package com.ibm.rpm.client.junit.scopemanagement.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;


public class TestCreateObjectFromTemplate extends ClientTestController
{
    protected ScopeElementScope mainScope;

    protected ScopeFolder mainFolder = null;

    protected WorkElementScope proposalScope;

    protected Proposal proposal = null;

    protected int nbChildren = 1;


    public void setUp() throws Exception
    {
        super.setUp();
        
        mainScope = new ScopeElementScope();
        mainScope.setParent(new ScopeElementScope());
        mainScope.setChildren(mainScope);

        proposalScope = new WorkElementScope();

        proposal = (Proposal) WorkElementFixture.createFixture(new Proposal(),proposalScope, this);
        proposal.setEstimatedFinishDate(proposal.getEstimatedStartDate());
        save(proposal, proposalScope);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", new ScopeElementScope());
        mainFolder = (ScopeFolder) rpmObjects[0];

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testActionWithTemplate() throws AssertionError, Exception {
		AbstractScope template = null;

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Action[published='"
				+ PublishedType._ToTemplate + "']", mainScope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Action);

		template = (Action) rpmObjects[0];

		Action newAction = new Action();
		newAction.setName("CAMERA_ACTION");
		newAction.setPublished(PublishedType.None);
		newAction.setParent(mainFolder);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newAction, template,
						mainScope);

		assertNotNull(result);

		assertTrue(result.getRpmObject() instanceof Action);
		assertEquals(result.getRpmObject().getID(), newAction.getID());
		assertTrue(result.isSuccessful());
	}

    public void testCreateActionWithTemplate() throws AssertionError,
			Exception {
		AbstractScope template = null;

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Action[published='"
				+ PublishedType._ToTemplate + "']", mainScope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Action);

		template = (Action) rpmObjects[0];

		Action newAction = new Action();
		newAction.setParent(mainFolder);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newAction, template,
						mainScope);

		assertNotNull(result);

		assertTrue(result.getRpmObject() instanceof Action);
		assertEquals(result.getRpmObject().getID(), newAction.getID());
		assertTrue(result.isSuccessful());
	}
    
    public void testCreateIssueWithTemplate() throws AssertionError, Exception {
		AbstractScope template = null;

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Issue[published='"
				+ PublishedType._ToTemplate + "']", mainScope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Issue);

		template = (Issue) rpmObjects[0];

		Issue newIssue = new Issue();
		newIssue.setParent(mainFolder);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newIssue, template,
						mainScope);

		assertNotNull(result);

		assertTrue(result.getRpmObject() instanceof Issue);
		assertEquals(result.getRpmObject().getID(), newIssue.getID());
		assertTrue(result.isSuccessful());
	}

    public void testCreateRequirementWithTemplate() throws AssertionError, Exception {
		AbstractScope template = null;

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Requirement[published='"
				+ PublishedType._ToTemplate + "']", mainScope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Requirement);

		template = (Requirement) rpmObjects[0];

		Requirement newRequirement = new Requirement();
		newRequirement.setParent(mainFolder);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newRequirement, template,
						mainScope);

		assertNotNull(result);

		assertTrue(result.getRpmObject() instanceof Requirement);
		assertEquals(result.getRpmObject().getID(), newRequirement.getID());
		assertTrue(result.isSuccessful());
	}


    public void testCreateRiskWithTemplate() throws AssertionError, Exception {
		AbstractScope template = null;

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Risk[published='"
				+ PublishedType._ToTemplate + "']", mainScope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Risk);

		template = (Risk) rpmObjects[0];

		Risk newRisk = new Risk();
		newRisk.setParent(mainFolder);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newRisk, template,
						mainScope);

		assertNotNull(result);

		assertTrue(result.getRpmObject() instanceof Risk);
		assertEquals(result.getRpmObject().getID(), newRisk.getID());
		assertTrue(result.isSuccessful());
	}

    public void testCreateChangeRequestWithTemplate() throws AssertionError, Exception {
		AbstractScope template = null;

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/ChangeRequest[published='"
				+ PublishedType._ToTemplate + "']", mainScope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof ChangeRequest);

		template = (ChangeRequest) rpmObjects[0];

		ChangeRequest newChangeRequest = new ChangeRequest();
		newChangeRequest.setParent(mainFolder);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newChangeRequest, template,
						mainScope);

		assertNotNull(result);

		assertTrue(result.getRpmObject() instanceof ChangeRequest);
		assertEquals(result.getRpmObject().getID(), newChangeRequest.getID());
		assertTrue(result.isSuccessful());
	}
    
    public void testCreateDefectWithTemplate() throws AssertionError, Exception {
		AbstractScope template = null;

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Defect[published='"
				+ PublishedType._ToTemplate + "']", mainScope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Defect);

		template = (Defect) rpmObjects[0];

		Defect newDefect = new Defect();
		newDefect.setParent(mainFolder);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newDefect, template,
						mainScope);

		assertNotNull(result);

		assertTrue(result.getRpmObject() instanceof Defect);
		assertEquals(result.getRpmObject().getID(), newDefect.getID());
		assertTrue(result.isSuccessful());
	}
    
    public void testCreateServiceRequestWithTemplate() throws AssertionError, Exception {
		AbstractScope template = null;

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/ServiceRequest[published='"
				+ PublishedType._ToTemplate + "']", mainScope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof ServiceRequest);

		template = (ServiceRequest) rpmObjects[0];

		ServiceRequest newServiceRequest = new ServiceRequest();
		newServiceRequest.setParent(mainFolder);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newServiceRequest, template,
						mainScope);

		assertNotNull(result);

		assertTrue(result.getRpmObject() instanceof ServiceRequest);
		assertEquals(result.getRpmObject().getID(), newServiceRequest.getID());
		assertTrue(result.isSuccessful());
	}

	public void testNullInputField() throws AssertionError, Exception {
		AbstractScope template = null;

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Action[published='"
				+ PublishedType._ToTemplate + "']", mainScope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Action);

		template = (Action) rpmObjects[0];

		Action newAction = null;

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newAction, template,
						mainScope);
		
		assertNotNull(result);
		assertFalse(result.isSuccessful());
	}


}
