package com.ibm.rpm.client.junit.scopemanagement.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ReqProScopeFolderFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.ReqProScopeFolder;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.MitigationFactorScope;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestScopeElement extends ClientTestController
{
    protected ScopeElementScope mainScope;

    protected ScopeFolder mainFolder = null;

    protected WorkElementScope proposalScope;

    protected Proposal proposal = null;

    protected int nbChildren = 1;

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = new ScopeElementScope();
        mainScope.setParent(new ScopeElementScope());
        mainScope.setChildren(mainScope);

        proposalScope = new WorkElementScope();

        proposal = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        save(proposal, new WorkElementScope());

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", new ScopeElementScope());
        mainFolder = (ScopeFolder) rpmObjects[0];
    }

    public void tearDown() throws Exception
    {
        delete(proposal);
        super.tearDown();
    }

    protected void update() throws AssertionError, Exception
    {
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        save(mainFolder, mainScope);
        mainFolder.setChildren(null);
        mainFolder = (ScopeFolder) ScopeElementFixture.updateFixture(
            mainFolder, mainScope, nbChildren, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    protected ScopeElement insertAndCompare(ScopeElement scopeElement,
            ScopeElementScope scope, boolean loadFromId) throws Exception
    {
        //Insert the ScopeElement into the DB
        SingleResult result = save(scopeElement, scope,  ReloadType.SavedResult);
        scopeElement.setID(result.getRpmObject().getID());

        //compare the returned ScopeElement
        ScopeElement returnedScopeElem = (ScopeElement) result.getRpmObject();
        ScopeElementFixture.compare(scopeElement, returnedScopeElem, scope);

        //reload the ScopeElement and compare it
        returnedScopeElem = (ScopeElement) load(returnedScopeElem, scope,
            loadFromId);
        ScopeElementFixture.compare(scopeElement, returnedScopeElem, scope);

        //return the ScopeElement
        return scopeElement;
    }

    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);

        SingleResult result = save(mainFolder, mainScope);
        ScopeElement returnedScopeElem = (ScopeElement) result.getRpmObject();
        ScopeElementFixture.compare(mainFolder, returnedScopeElem, mainScope);

        RPMObject[] objs = null;
        //      objs = loadFromXpath("/ScopeFolder[externalRef='" + mainFolder.get +
        // "']",
        //            mainScope);
        
        objs = loadFromXpath("/ScopeFolder[id='" + mainFolder.getID() + "']",
            mainScope);
        ScopeElementFixture.compare(mainFolder, (ScopeElement) objs[0],
            mainScope);
        
        objs = loadFromXpath("/ScopeFolder[name='" + mainFolder.getName()
                + "']", mainScope);
        ScopeElementFixture.compare(mainFolder, (ScopeElement) objs[0],
            mainScope);

        Requirement req = (Requirement) ScopeElementFixture.retrieveObject(
            mainFolder, new Requirement());
        objs = loadFromXpath("/Requirement[id='" + req.getID() + "']",
            mainScope);
        ScopeElementFixture.compare(req, (ScopeElement) objs[0],
            mainScope);
        
        objs = loadFromXpath("/Requirement[name='" + req.getName() + "']",
            mainScope);
        ScopeElementFixture.compare(req, (ScopeElement) objs[0],
            mainScope);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        mainScope = getFullScope();
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        update();
    }
    
    public void testReqProJoinCondition() throws AssertionError, Exception
    {
        // insert a requirement and make sure we can read it back
        ScopeElement parent = ScopeElementFixture.createMainParent(this);
        
        Requirement inserted = ScopeElementFixture.createRequirementFixture(null, 1, parent, this);
        
        inserted = (Requirement) save(inserted, null).getRpmObject();
        
        Requirement read = (Requirement)loadFromXpath("/Requirement[@id='" + inserted.getID() + "']", null)[0];
        
        assertEquals(inserted.getID(), read.getID());
        
        // create ReqproRequirements
        ScopeElementScope fixtureScope = new ScopeElementScope();
        fixtureScope.setChildren(new ScopeElementScope());
        ReqProScopeFolder scopeFolder = ReqProScopeFolderFixture.createFixture(null, fixtureScope);
        
    }

    public static ScopeElementScope getFullScope()
    {
        ScopeElementScope scope = new ScopeElementScope();
        scope.setApprovedCostBenefits(true);
        scope.setAsset(true);
        scope.setAssetCode(true);
        scope.setAttributeAssignments(true);
        scope.setClientCostCenter(true);
        scope.setDocumentFolder(new DocumentScope());
        scope.setDuplicates(true);
        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        scope.setWorksheetFinancials(finScope);
        scope.setWbsFinancials(finScope);
        scope.setImpact(true);

        //		mitigationFactor
        MitigationFactorScope mitScope = new MitigationFactorScope();
        scope.setMitigationFactors(mitScope);
        mitScope.setParent(scope);
        mitScope.setMitigationTriggers(true);

        scope.setOrderOfMagnitudeCostBenefits(true);
        scope.setParent(new ScopeElementScope());
        //scope.setResourceTaskAssignments(new ResourceTaskAssignmentScope()); // We don't want to test this in this class
        scope.setRiskMatrix(true);
        scope.setRtfAssignments(true);
        scope.setChildren(scope);
        scope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(scope));
//        scope.setState(true);
        //scope.setStatusUpdates(true);
        scope.setWorkElement(new WorkElementScope());
        return scope;
    }

    protected RPMObject load(ScopeElement scopeElement,
            ScopeElementScope scope, boolean loadFromId) throws Exception
    {
        RPMObject obj = null;

        if (loadFromId)
        {
            //load from an id
            if (scopeElement instanceof ScopeFolder)
            {
                obj = new ScopeFolder();
            }
            else if (scopeElement instanceof Requirement)
            {
                obj = new Requirement();
            }
            else if (scopeElement instanceof ChangeRequest)
            {
                obj = new ChangeRequest();
            }
            else if (scopeElement instanceof ServiceRequest)
            {
                obj = new ServiceRequest();
            }
            else if (scopeElement instanceof Action)
            {
                obj = new Action();
            }
            else if (scopeElement instanceof Defect)
            {
                obj = new Defect();
            }
            else if (scopeElement instanceof Issue)
            {
                obj = new Issue();
            }
            else if (scopeElement instanceof Risk)
            {
                obj = new Risk();
            }
            else if (scopeElement instanceof ScopeDeliverable)
            {
                obj = new ScopeDeliverable();
            }
            else if (scopeElement instanceof ScopeMilestone)
            {
                obj = new ScopeMilestone();
            }
            else if (scopeElement instanceof ScopeTask)
            {
                obj = new ScopeTask();
            }
            else if (scopeElement instanceof ScopeSummaryTask)
            {
                obj = new ScopeSummaryTask();
            }
            else
            {
                logger.debug("error :Invalid Element for Scope ");
            }
            obj.setID(scopeElement.getID());

            return loadFromID(obj, scope);
        }
        //load from an xpath
        RPMObject[] objs = loadFromXpath("/ScopeFolder[id='"
                + scopeElement.getID() + "']", scope);
        if (objs == null || objs.length == 0)
        {
            return null;
        }
        return objs[0];
    }
}