package com.ibm.rpm.client.junit.scopemanagement.testcases;

import com.ibm.rpm.applicationadministration.containers.RequirementState;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.MitigationFactorScope;

/**
 * @author gosselin
 */
public class TestScopeElement1 extends TestScopeElement
{
    public void testTransderUnderWBS() throws AssertionError, Exception
    {
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        save(mainFolder, mainScope);
        Requirement element = (Requirement) mainFolder.getChildren()[0];
        RequirementState state = new RequirementState();
        state.setClosedState(Boolean.FALSE);
        getApplicationInterface().transferUnderWBS(sessionid, null, element);
    }

    public void testInsertWithRiskMitigationFactor() throws AssertionError,
            Exception
    {
        MitigationFactorScope mitScope = new MitigationFactorScope();
        mitScope.setParent(mainScope);
        mitScope.setMitigationTriggers(true);
        mainScope.setMitigationFactors(mitScope);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope, 1,
            mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithApprovedCostBenefits() throws AssertionError,
            Exception
    {
        mainScope.setApprovedCostBenefits(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithAttributeAssignment() throws AssertionError,
            Exception
    {
        mainScope.setAttributeAssignments(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithAssetAndAssetCode() throws AssertionError,
            Exception
    {
        mainScope.setAsset(true);
        mainScope.setAssetCode(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithAssignedScorecard() throws AssertionError,
            Exception
    {
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        SingleResult result = save(mainFolder, mainScope);
        mainFolder.setID(result.getRpmObject().getID());
        mainScope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(mainScope));
        mainFolder = (ScopeFolder) ScopeElementFixture.updateFixture(
            mainFolder, mainScope, nbChildren, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithWorksheetFinancial() throws AssertionError,
            Exception
    {
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        SingleResult result = save(mainFolder, mainScope);
        mainFolder.setID(result.getRpmObject().getID());
        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        mainScope.setWorksheetFinancials(finScope);
        ScopeElementFixture.updateFixture(mainFolder, mainScope, nbChildren,
            this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithWbsFinancial() throws AssertionError, Exception
    {
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        SingleResult result = save(mainFolder, mainScope);
        mainFolder.setID(result.getRpmObject().getID());
        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        mainScope.setWbsFinancials(finScope);
        ScopeElementFixture.updateFixture(mainFolder, mainScope, nbChildren,
            this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithClientCostCenter() throws AssertionError,
            Exception
    {
        mainScope.setClientCostCenter(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithDocumentFolder() throws AssertionError, Exception
    {
        //TODO when this is fix, re-introduce the DocumentFolder in the scope
        // in the
        //getFullScope method.
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        //put DocumentFolder in the scope
        mainScope.setDocumentFolder(new DocumentScope());
        SingleResult result = save(mainFolder, mainScope);
        mainFolder = (ScopeFolder)result.getRpmObject();
        ScopeElementFixture.updateFixture(mainFolder, mainScope, nbChildren,
            this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithDuplicates() throws AssertionError, Exception
    {
        mainScope.setDuplicates(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithImpact() throws AssertionError, Exception
    {
        mainScope.setImpact(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithOrderOfMagnitudeCostBenefits()
            throws AssertionError, Exception
    {
        mainScope.setOrderOfMagnitudeCostBenefits(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithRtfAssignment() throws AssertionError, Exception
    {
        mainScope.setRtfAssignments(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

//  Will be tested in ResourceTaskAssignment tests
// On insert, the current resource is also assigned to the scope element, and we cannot perform a simple compare 
//    public void testInsertWithResourceAssignment() throws AssertionError,
//            Exception
//    {
//        mainScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
//        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
//            nbChildren, mainFolder, this);
//        insertAndCompare(mainFolder, mainScope, true);
//    }

    public void testInsertWithRiskMatrix() throws AssertionError, Exception
    {
        mainScope.setRiskMatrix(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testInsertWithState() throws AssertionError, Exception
    {
        mainScope.setState(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        insertAndCompare(mainFolder, mainScope, true);
    }
}
