package com.ibm.rpm.client.junit.scopemanagement.testcases;

import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.MitigationFactorScope;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;

/**
 * @author gosselin
 */
public class TestScopeElement3 extends TestScopeElement
{
    /**
     * Create a requirement with a valid wbsParent
     * @throws AssertionError
     * @throws Exception
     */
    public void testInsertWithWorkElementLink() throws AssertionError, Exception
    {   
        ScopeElementScope scopeScope = new ScopeElementScope();
        scopeScope.setWbsParent(true);
        ScopeFolder parentScopeFolder = ScopeElementFixture.createMainParent(this);
        ScopeElementFixture.createRequirementFixture(scopeScope, 5, parentScopeFolder, this);
    }
    
    public void testUpdateWithResourceAssignment() throws AssertionError,
            Exception
    {
        mainScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        save(mainFolder, mainScope);
        
        // Must load the saved ResourceTaskAssignments, because the 
        // create assigns the current resource to the ScopeElement
        mainFolder = (ScopeFolder) load(mainFolder, mainScope, true);

        mainFolder = (ScopeFolder) ScopeElementFixture.updateFixture(
            mainFolder, mainScope, nbChildren + 2, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testUpdateWithClientCostCenter() throws AssertionError,
            Exception
    {
        mainScope.setClientCostCenter(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        save(mainFolder, mainScope);

        mainFolder = (ScopeFolder) ScopeElementFixture.updateFixture(
            mainFolder, mainScope, nbChildren + 2, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    //    public void testUpdateWithDuplicates() throws AssertionError,
    //            Exception
    //    {
    //        mainScope.setDuplicates(true);
    //        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
    //            nbChildren, mainFolder, this);
    //        SingleResult result = save(mainFolder, mainScope);
    //        mainFolder.setID(result.getRpmObject().getID());
    //        
    //        mainFolder = (ScopeFolder) ScopeElementFixture.updateFixture(mainFolder,
    //            mainScope, nbChildren, this);
    //        insertAndCompare(mainFolder, mainScope, true);
    //    }

    public void testUpdateWithRtfAssignment() throws AssertionError, Exception
    {
        mainScope.setRtfAssignments(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        save(mainFolder, mainScope);

        mainFolder = (ScopeFolder) ScopeElementFixture.updateFixture(
            mainFolder, mainScope, nbChildren + 2, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testUpdateWithImpact() throws AssertionError, Exception
    {
        mainScope.setImpact(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        save(mainFolder, mainScope);

        mainFolder = (ScopeFolder) ScopeElementFixture.updateFixture(
            mainFolder, mainScope, nbChildren + 2, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testUpdateWithOrderOfMagnitudeCostBenefits()
            throws AssertionError, Exception
    {
        mainScope.setOrderOfMagnitudeCostBenefits(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        save(mainFolder, mainScope);

        mainFolder = (ScopeFolder) ScopeElementFixture.updateFixture(
            mainFolder, mainScope, nbChildren + 2, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testUpdateWithRiskWithMitigationFactor() throws AssertionError,
            Exception
    {
        MitigationFactorScope mitScope = new MitigationFactorScope();
        mainScope.setMitigationFactors(mitScope);
        mitScope.setParent(mainScope);
        mitScope.setMitigationTriggers(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope, 1,
            mainFolder, this);
        save(mainFolder, mainScope);

        mainFolder = (ScopeFolder) ScopeElementFixture.updateFixture(
            mainFolder, mainScope, nbChildren + 2, this);
        insertAndCompare(mainFolder, mainScope, true);
    }

    public void testUpdateWithRiskMatrix() throws AssertionError, Exception
    {
        mainScope.setRiskMatrix(true);
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);
        save(mainFolder, mainScope);

        mainFolder = (ScopeFolder) ScopeElementFixture.updateFixture(
            mainFolder, mainScope, nbChildren + 2, this);
        insertAndCompare(mainFolder, mainScope, true);
    }
    
    public void testInsertWithChargeCodeLoadFromXPath() throws AssertionError,
    Exception
    {
        
        mainFolder = ScopeElementFixture.createFolderFixture(mainScope,
            nbChildren, mainFolder, this);

        SingleResult result = save(mainFolder, mainScope);
        ScopeElement returnedScopeElem = (ScopeElement) result.getRpmObject();
        ScopeElementFixture.compare(mainFolder, returnedScopeElem, mainScope);

        RPMObject[] objs = null;


        Requirement req = (Requirement) ScopeElementFixture.retrieveObject(
            mainFolder, new Requirement());
        objs = loadFromXpath("/Requirement[id='" + req.getID() + "']",
            mainScope);
        
        FinancialScope fs = new FinancialScope();
        fs.setChargeCode(true);
        fs.setParent(mainScope);
        mainScope.setWorksheetFinancials(fs);
        
        WorksheetFinancial wf = FinancialFixture.createWorksheetFixture(fs,
            req, this);
        
        req.setWorksheetFinancials(new WorksheetFinancial[] {wf});
        result = save(mainFolder, mainScope, ReloadType.SavedResult);
        
        
        ScopeElementFixture.compare(req, (ScopeElement) objs[0], mainScope);
    }
}
