/*
 * Created on 29-May-2006
 * @author mmatwiy
 * Test the move and transfer of AbstractScope elements to WBS.
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestTransferUnderWBS extends ClientTestController
{
    ScopeFolder scopeFolder;

    ScopeElementScope scopeElementScope;

    WorkElementScope workElementScope;

    public void setUp() throws Exception
    {
        super.setUp();

        // create a Proposal and save it; this will create a ScopeFolder
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);
        scopeFolder = (ScopeFolder) rpmObjects[0];

        scopeElementScope = new ScopeElementScope();
        scopeElementScope.setProject(new WorkElementScope());
        scopeElementScope.setChildren(new ScopeElementScope());
        scopeElementScope.setParent(new ScopeElementScope());

        WorkElementScope childrenScope = new WorkElementScope();
        childrenScope.setScopeElement(new ScopeElementScope());

        workElementScope = new WorkElementScope();
        workElementScope.setChildren(childrenScope);
        workElementScope.setParent(new WorkElementScope());
        workElementScope.setScopeElement(new ScopeElementScope());

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testTransferUnderWBSRoot() throws Exception
    {
        Requirement requirement = new Requirement();
        requirement.setParent(scopeFolder);
        requirement.setName(HelperTestUtilities.createName());
        save(requirement, null);

        Result result = getApplicationInterface().transferUnderWBS(
            getSessionID(), null, requirement);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // retrieve the Proposal that contains the ScopeFolder
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[name='"
                + scopeFolder.getName() + "']", workElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        Proposal proposal = (Proposal) rpmObjects[0];

        // check that the requirement was transferred to the Proposal 
        // under WBS; it will be a SummaryTask that contains a Requirement
        // scope element
        assertNotNull(proposal.getChildren());
        assertTrue(proposal.getChildren()[0] instanceof SummaryTask);
        assertEquals(proposal.getChildren()[0].getID(), requirement.getID());
        assertTrue(proposal.getChildren()[0].getScopeElement() instanceof Requirement);

    }

    public void testTransferSubScopeFolderChildrenUnderWBSRoot()
            throws Exception
    {
        ScopeFolder subScopeFolder = new ScopeFolder();
        subScopeFolder.setName(HelperTestUtilities.createName());
        subScopeFolder.setParent(scopeFolder);

        Requirement requirement = new Requirement();
        requirement.setParent(subScopeFolder);
        requirement.setName(HelperTestUtilities.createName());

        Action action = new Action();
        action.setParent(subScopeFolder);
        action.setName(HelperTestUtilities.createName());

        ScopeElementScope subScopeElementScope = new ScopeElementScope();
        subScopeElementScope.setChildren(new ScopeElementScope());
        
        ScopeElementScope scopeElementScope = new ScopeElementScope();
        scopeElementScope.setChildren(subScopeElementScope);

        subScopeFolder.setChildren(new ScopeElement[]
        {
                requirement, action
        });
        save(subScopeFolder, scopeElementScope);

        // transfer the sub scope folder's first child (Requirement)
        Result result = getApplicationInterface().transferUnderWBS(
            getSessionID(), null,
            (AbstractScope) subScopeFolder.getChildren()[0]);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // retrieve the Proposal that contains the ScopeFolder
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[name='"
                + scopeFolder.getName() + "']", workElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        Proposal proposal = (Proposal) rpmObjects[0];

        // check that the requirement was transferred to the Proposal 
        // under WBS; it will be a SummaryTask that contains a Requirement
        // scope element
        assertNotNull(proposal.getChildren());
        assertTrue(proposal.getChildren()[0] instanceof SummaryTask);
        assertEquals(proposal.getChildren()[0].getID(), requirement.getID());
        assertTrue(proposal.getChildren()[0].getScopeElement() instanceof Requirement);

        // transfer the sub scope folder's second child (Action)
        result = getApplicationInterface().transferUnderWBS(getSessionID(),
            null, (AbstractScope) subScopeFolder.getChildren()[1]);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // retrieve the Proposal that contains the ScopeFolder
        rpmObjects = loadFromXpath("/Proposal[name='" + scopeFolder.getName()
                + "']", workElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        proposal = (Proposal) rpmObjects[0];

        // check that the action was transferred to the Proposal 
        // under WBS; it will be a SummaryTask that contains an Action
        // scope element
        assertNotNull(proposal.getChildren());
        assertTrue(proposal.getChildren()[1] instanceof SummaryTask);
        assertEquals(proposal.getChildren()[1].getID(), action.getID());
        assertTrue(proposal.getChildren()[1].getScopeElement() instanceof Action);
        
        // load the ScopeFolder to see that it still contains its children
        rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + proposal.getName() + "']", scopeElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);
        ScopeFolder reloadedScopeFolder = (ScopeFolder) rpmObjects[0];
        
        assertNotNull(reloadedScopeFolder.getChildren());
        assertTrue(reloadedScopeFolder.getChildren()[0] instanceof ScopeFolder);
        ScopeFolder reloadedSubScopeFolder = (ScopeFolder)reloadedScopeFolder.getChildren()[0];
        
        assertNotNull(reloadedSubScopeFolder.getChildren());
        assertTrue(reloadedSubScopeFolder.getChildren()[0] instanceof Requirement || 
            reloadedSubScopeFolder.getChildren()[0] instanceof Action);
        assertTrue(reloadedSubScopeFolder.getChildren()[1] instanceof Requirement || 
            reloadedSubScopeFolder.getChildren()[1] instanceof Action);
    }

    public void testDeletionOfTransferredScopeElement() throws Exception
    {
        Requirement requirement = new Requirement();
        requirement.setParent(scopeFolder);
        requirement.setName(HelperTestUtilities.createName());
        save(requirement, null);

        // retrieve the Proposal that contains the ScopeFolder
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[name='"
                + scopeFolder.getName() + "']", workElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        Proposal proposal = (Proposal) rpmObjects[0];

        Result result = getApplicationInterface().transferUnderWBS(
            getSessionID(), null, requirement);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // retrieve the Proposal that contains the ScopeFolder
        rpmObjects = loadFromXpath("/Proposal[name='" + scopeFolder.getName()
                + "']", workElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        proposal = (Proposal) rpmObjects[0];

        // check that the requirement was transferred to the Proposal 
        // under WBS
        assertNotNull(proposal.getChildren());
        assertEquals(proposal.getChildren()[0].getID(), requirement.getID());

        // must delete the transferred requirement prior to deleting the requirement
        delete(proposal.getChildren()[0]);

        rpmObjects = loadFromXpath("/SummaryTask[id='" + requirement.getID()
                + "']", null);

        assertNull(rpmObjects);

        // delete the original requirement
        delete(requirement);

        rpmObjects = loadFromXpath("/Requirement[id='" + requirement.getID()
                + "']", null);

        assertNull(rpmObjects);
    }

    public void testCrossMove() throws Exception
    {
        // create an AbstractScope and reload it to get a populated project field
        ServiceRequest serviceRequest = new ServiceRequest();
        serviceRequest.setParent(scopeFolder);
        serviceRequest.setName(HelperTestUtilities.createName());
        save(serviceRequest, scopeElementScope);

        serviceRequest = (ServiceRequest) loadFromID(serviceRequest,
            scopeElementScope);

        // create a WorkElement to move the AbstractScope to
        WorkOrganization workOrganization = new WorkOrganization();
        workOrganization.setName(HelperTestUtilities.createName());
        save(workOrganization, null);

        WorkProduct workProduct = new WorkProduct();
        workProduct.setName(HelperTestUtilities.createName());
        workProduct.setParent(workOrganization);
        save(workProduct, null);

        // cross move the AbstractScope to the WorkElement 
        Result result = getApplicationInterface().moveUnderWBS(getSessionID(),
            workProduct, serviceRequest);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // check that the project's ScopeFolder 
        // now contains the AbstractScope
        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + workOrganization.getName() + "']", scopeElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);

        // check that the serviceRequest was moved 
        ScopeFolder folder = (ScopeFolder) rpmObjects[0];
        assertNotNull(folder.getChildren());
        assertEquals(folder.getChildren()[0].getID(), serviceRequest.getID());
        assertTrue(folder.getChildren()[0] instanceof ServiceRequest);

    }

    public void testTransferWithin() throws Exception
    {
        // create an AbstractScope and reload it to get a populated project field
        Risk risk = new Risk();
        risk.setParent(scopeFolder);
        risk.setName(HelperTestUtilities.createName());
        save(risk, scopeElementScope);

        risk = (Risk) loadFromID(risk, scopeElementScope);

        // retrieve the Proposal that contains the ScopeFolder
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[name='"
                + scopeFolder.getName() + "']", workElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        Proposal proposal = (Proposal) rpmObjects[0];

        // transfer the AbstractScope to the Proposal
        Result result = getApplicationInterface().transferUnderWBS(
            getSessionID(), proposal, risk);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // check that the proposal now contains the AbstractScope
        proposal = (Proposal) loadFromID(proposal, workElementScope);

        // check that the risk was transferred to the Proposal 
        // under WBS; it will be a SummaryTask that contains a Risk
        // scope element
        assertNotNull(proposal);
        assertNotNull(proposal.getChildren());
        assertTrue(proposal.getChildren()[0] instanceof SummaryTask);
        assertEquals(proposal.getChildren()[0].getID(), risk.getID());
        assertTrue(proposal.getChildren()[0].getScopeElement() instanceof Risk);
    }

    public void testTransferWithin2() throws Exception
    {
        // create an AbstractScope and reload it to get a populated project field
        Risk risk = new Risk();
        risk.setParent(scopeFolder);
        risk.setName(HelperTestUtilities.createName());
        save(risk, scopeElementScope);

        risk = (Risk) loadFromID(risk, scopeElementScope);

        // retrieve the Proposal that contains the ScopeFolder
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[name='"
                + scopeFolder.getName() + "']", workElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        Proposal proposal = (Proposal) rpmObjects[0];

        WorkProduct workProduct = new WorkProduct();
        workProduct.setName(HelperTestUtilities.createName());
        workProduct.setParent(proposal);
        save(workProduct, workElementScope);

        // transfer the AbstractScope to the WorkProduct
        Result result = getApplicationInterface().transferUnderWBS(
            getSessionID(), workProduct, risk);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // check that the workProduct now contains the AbstractScope
        workProduct = (WorkProduct) loadFromID(workProduct, workElementScope);

        // check that the risk was transferred to the Proposal 
        // under WBS; it will be a SummaryTask that contains a Risk
        // scope element
        assertNotNull(workProduct);
        assertNotNull(workProduct.getChildren());
        assertTrue(workProduct.getChildren()[0] instanceof SummaryTask);
        assertEquals(workProduct.getChildren()[0].getID(), risk.getID());
        assertTrue(workProduct.getChildren()[0].getScopeElement() instanceof Risk);
    }

    public void testMoveWithin() throws Exception
    {
        // create an AbstractScope and reload it to get a populated project field
        Defect defect = new Defect();
        defect.setParent(scopeFolder);
        defect.setName(HelperTestUtilities.createName());
        save(defect, scopeElementScope);

        defect = (Defect) loadFromID(defect, scopeElementScope);

        // retrieve the Proposal that contains the ScopeFolder
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[name='"
                + scopeFolder.getName() + "']", workElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        Proposal proposal = (Proposal) rpmObjects[0];

        // transfer the AbstractScope to the Proposal
        Result result = getApplicationInterface().moveUnderWBS(getSessionID(),
            proposal, defect);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // check that the project's ScopeFolder 
        // now contains the AbstractScope
        rpmObjects = loadFromXpath("/ScopeFolder[name='" + proposal.getName()
                + "']", scopeElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);

        // check that the defect was moved 
        ScopeFolder folder = (ScopeFolder) rpmObjects[0];
        assertNotNull(folder.getChildren());
        assertEquals(folder.getChildren()[0].getID(), defect.getID());
        assertTrue(folder.getChildren()[0] instanceof Defect);
    }

    public void testMoveWithin2() throws Exception
    {
        // create an AbstractScope and reload it to get a populated project field
        Defect defect = new Defect();
        defect.setParent(scopeFolder);
        defect.setName(HelperTestUtilities.createName());
        save(defect, scopeElementScope);

        defect = (Defect) loadFromID(defect, scopeElementScope);

        // retrieve the Proposal that contains the ScopeFolder
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[name='"
                + scopeFolder.getName() + "']", workElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        Proposal proposal = (Proposal) rpmObjects[0];

        Deliverable deliverable = new Deliverable();
        deliverable.setName(HelperTestUtilities.createName());
        deliverable.setParent(proposal);
        save(deliverable, workElementScope);

        // transfer the AbstractScope to the Deliverable
        Result result = getApplicationInterface().moveUnderWBS(getSessionID(),
            deliverable, defect);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // check that the project's ScopeFolder 
        // now contains the AbstractScope
        rpmObjects = loadFromXpath("/ScopeFolder[name='" + proposal.getName()
                + "']", scopeElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);

        // check that the defect was moved
        ScopeFolder folder = (ScopeFolder) rpmObjects[0];
        assertNotNull(folder.getChildren());
        assertEquals(folder.getChildren()[0].getID(), defect.getID());
        assertTrue(folder.getChildren()[0] instanceof Defect);
    }

    public void testMoveAndTransferWithin() throws Exception
    {
        // create an AbstractScope and reload it to get a populated project field
        ChangeRequest changeRequest = new ChangeRequest();
        changeRequest.setParent(scopeFolder);
        changeRequest.setName(HelperTestUtilities.createName());
        save(changeRequest, scopeElementScope);

        changeRequest = (ChangeRequest) loadFromID(changeRequest,
            scopeElementScope);

        // retrieve the Proposal that contains the ScopeFolder
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[name='"
                + scopeFolder.getName() + "']", workElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        Proposal proposal = (Proposal) rpmObjects[0];

        // transfer the AbstractScope to the Proposal
        Result result = getApplicationInterface().moveAndTransferUnderWBS(
            getSessionID(), proposal, changeRequest);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // check that the proposal's ScopeFolder 
        // now contains the AbstractScope (move)
        rpmObjects = loadFromXpath("/ScopeFolder[name='" + proposal.getName()
                + "']", scopeElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);

        ScopeFolder folder = (ScopeFolder) rpmObjects[0];
        assertNotNull(folder.getChildren());
        assertEquals(folder.getChildren()[0].getID(), changeRequest.getID());
        assertTrue(folder.getChildren()[0] instanceof ChangeRequest);

        // check that the proposal now contains the AbstractScope (transfer)
        proposal = (Proposal) loadFromID(proposal, workElementScope);

        assertNotNull(proposal);
        assertNotNull(proposal.getChildren());
        assertEquals(proposal.getChildren()[0].getID(), changeRequest.getID());
        assertTrue(proposal.getChildren()[0].getScopeElement() instanceof ChangeRequest);
    }

    public void testMoveAndTransferWithin2() throws Exception
    {
        // create an AbstractScope and reload it to get a populated project field
        ChangeRequest changeRequest = new ChangeRequest();
        changeRequest.setParent(scopeFolder);
        changeRequest.setName(HelperTestUtilities.createName());
        save(changeRequest, scopeElementScope);

        changeRequest = (ChangeRequest) loadFromID(changeRequest,
            scopeElementScope);

        // retrieve the Proposal that contains the ScopeFolder
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[name='"
                + scopeFolder.getName() + "']", workElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        Proposal proposal = (Proposal) rpmObjects[0];

        SummaryTask summaryTask = new SummaryTask();
        summaryTask.setName(HelperTestUtilities.createName());
        summaryTask.setParent(proposal);
        save(summaryTask, workElementScope);

        // transfer the AbstractScope to the SummaryTask
        Result result = getApplicationInterface().moveAndTransferUnderWBS(
            getSessionID(), summaryTask, changeRequest);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // check that the proposal's ScopeFolder 
        // now contains the AbstractScope (move)
        rpmObjects = loadFromXpath("/ScopeFolder[name='" + proposal.getName()
                + "']", scopeElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);

        ScopeFolder folder = (ScopeFolder) rpmObjects[0];
        assertNotNull(folder.getChildren());
        assertEquals(folder.getChildren()[0].getID(), changeRequest.getID());

        // check that the SummaryTask now contains the AbstractScope (transfer)
        summaryTask = (SummaryTask) loadFromID(summaryTask, workElementScope);

        assertNotNull(summaryTask);
        assertNotNull(summaryTask.getChildren());
        assertEquals(summaryTask.getChildren()[0].getID(), changeRequest
            .getID());
        assertTrue(summaryTask.getChildren()[0] instanceof SummaryTask);
    }

    public void testCrossMoveAndTransfer() throws Exception
    {
        // create an AbstractScope and reload it to get a populated project field
        Issue issue = new Issue();
        issue.setParent(scopeFolder);
        issue.setName(HelperTestUtilities.createName());
        save(issue, scopeElementScope);

        issue = (Issue) loadFromID(issue, scopeElementScope);

        // create a WorkElement to move and transfer the AbstractScope to
        Project project = new Project();
        project.setName(HelperTestUtilities.createName());
        save(project, null);

        Deliverable deliverable = new Deliverable();
        deliverable.setName(HelperTestUtilities.createName());
        deliverable.setParent(project);
        save(deliverable, null);

        // cross move and transfer the AbstractScope to the WorkElement 
        Result result = getApplicationInterface().moveAndTransferUnderWBS(
            getSessionID(), deliverable, issue);

        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // check that the deliverable now contains the AbstractScope (transfer)
        deliverable = (Deliverable) loadFromID(deliverable, workElementScope);

        assertNotNull(deliverable);
        assertNotNull(deliverable.getChildren());
        assertEquals(deliverable.getChildren()[0].getID(), issue.getID());
        assertTrue(deliverable.getChildren()[0] instanceof SummaryTask);

        // the ScopeFolder associated with the project
        // should also contain the AbstractScope (move)
        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + project.getName() + "']", scopeElementScope);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);

        ScopeFolder folder = (ScopeFolder) rpmObjects[0];
        assertNotNull(folder.getChildren());
        assertEquals(folder.getChildren()[0].getID(), issue.getID());
        assertTrue(folder.getChildren()[0] instanceof Issue);

    }

}