package com.ibm.rpm.client.junit.scopemanagement.testcases.fixes;

import com.ibm.rpm.applicationadministration.containers.AssetFinancialCategory;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.clientcostcenters.fixtures.ClientCostCenterFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

public class Test64305 extends ClientTestController {
	
	private final int TYPE_CLIENT_COST_CENTER = 0;
	private final int TYPE_ASSET = 1;
	private final int TYPE_ASSET_CODE = 2;
	
	public void testSaveAbstractScopeElementWithCCC() throws Exception
	{
		saveAndValidateAbstractScopeElement(TYPE_CLIENT_COST_CENTER);
	}
	
	public void testUpdateCCCForAbstractScopeElement() throws Exception
	{
		updateAndValidateAbstractScopeElement(TYPE_CLIENT_COST_CENTER);
	}
	
	public void testSaveAbstractScopeElementWithAsset() throws Exception
	{
		saveAndValidateAbstractScopeElement(TYPE_ASSET);
	}
	
	public void testUpdateAssetForAbstractScopeElement() throws Exception
	{
		updateAndValidateAbstractScopeElement(TYPE_ASSET);
	}
	
	public void testSaveAbstractScopeElementWithAssetCode() throws Exception
	{
		saveAndValidateAbstractScopeElement(TYPE_ASSET_CODE);
	}
	
	public void testUpdateAssetCodeForAbstractScopeElement() throws Exception
	{
		updateAndValidateAbstractScopeElement(TYPE_ASSET_CODE);
	}
	
	private AbstractScope saveAndValidateAbstractScopeElement(int operationType) throws Exception
	{
		Proposal proposal = createProposal();
		
		ScopeFolder rootScopeFolder = loadRootScopeFolder(proposal);
		
		AbstractScope abstractScopeElement = createAbstractScopeElement(rootScopeFolder,operationType);
		
		assertNotNull(abstractScopeElement);
		
		switch (operationType) 
		{
			case TYPE_CLIENT_COST_CENTER:
				
				assertNotNull(abstractScopeElement.getClientCostCenter());
				assertNotNull(abstractScopeElement.getClientCostCenter().getID());
			
			break;
			
			case TYPE_ASSET:
				
				assertNotNull(abstractScopeElement.getAsset());
				assertNotNull(abstractScopeElement.getAsset().getID());
			
			break;
			
			case TYPE_ASSET_CODE:
				
				assertNotNull(abstractScopeElement.getAssetCode());
				assertNotNull(abstractScopeElement.getAssetCode().getID());	
			
			break;
			
			default:
			break;
		}
		
		return abstractScopeElement;
	}
	
	private void updateAndValidateAbstractScopeElement(int operationType) throws Exception
	{
		AbstractScope saved = saveAndValidateAbstractScopeElement(operationType);
		
		String previousCCCId = null;
		String previousAssetId = null;
		String previousAssetCodeId = null;
		
		switch (operationType) 
		{
			case TYPE_CLIENT_COST_CENTER:
				
				previousCCCId = saved.getClientCostCenter().getID();
			
			break;
			
			case TYPE_ASSET:
				
				previousAssetId = saved.getAsset().getID();
			
			break;
			
			case TYPE_ASSET_CODE:
				
				previousAssetCodeId = saved.getAssetCode().getID();	
			
			break;
			
			default:
			break;
		}
		
		saved = updateAbstractScopeElement(saved,operationType);
		
		assertNotNull(saved);
		
		switch (operationType) 
		{
			case TYPE_CLIENT_COST_CENTER:
				
				assertNotNull(saved.getClientCostCenter());
				assertNotNull(saved.getClientCostCenter().getID());
				assertTrue(!saved.getClientCostCenter().getID().equals(previousCCCId));
				
			break;
			
			case TYPE_ASSET:
				
				assertNotNull(saved.getAsset());
				assertNotNull(saved.getAsset().getID());
				assertTrue(!saved.getAsset().getID().equals(previousAssetId));
			
			break;
			
			case TYPE_ASSET_CODE:
				
				assertNotNull(saved.getAssetCode());
				assertNotNull(saved.getAssetCode().getID());
				assertFalse(saved.getAssetCode().getID().equals(previousAssetCodeId));	
			
			break;
			
			default:
			break;
		}
	}
	
	private Proposal createProposal() throws Exception
	{
		Proposal proposal = (Proposal)WorkElementFixture.createSaveFixture(new Proposal(), null, this);
		
		return proposal;
	}
	
	private ScopeFolder loadRootScopeFolder(Proposal proposal) throws Exception
	{
		String xPath = "/ScopeFolder[name = '" + proposal.getName() + "']";
		
		ScopeFolder rootScopeFolder = (ScopeFolder)loadOneFromXpath(xPath, null);
		
		assertNotNull(rootScopeFolder);
		
		return rootScopeFolder;
	}
	
	private AbstractScope createAbstractScopeElement(ScopeFolder parent,int operationType) throws Exception
	{
		Defect defect = new Defect();
		defect.setName(HelperTestUtilities.createName());
		defect.setParent(parent);
		ScopeElementScope scopeElementScope = new ScopeElementScope();
		
		switch (operationType) 
		{
			case TYPE_CLIENT_COST_CENTER:
				
				ClientCostCenter ccc = createAndSaveCCC();
				defect.setClientCostCenter(ccc);
				scopeElementScope.setClientCostCenter(true);
			
			break;
			
			case TYPE_ASSET:
				
				Asset asset = createAndSaveAsset();
				defect.setAsset(asset);
				scopeElementScope.setAsset(true);
			
			break;
			
			case TYPE_ASSET_CODE:
				
				AssetFinancialCategory assetFinancialCategoty = createAndSaveAssetFinancialCategory(DatafieldFixture.ASSET_FINANCIAL_CATEGORY_PROD_SUPPORT);
				defect.setAssetCode(assetFinancialCategoty);
				scopeElementScope.setAssetCode(true);
			
			break;
			
			default:
			break;
		}
		
		defect = (Defect)save(defect, scopeElementScope).getRpmObject();
		
		return defect;
	}
	
	private AbstractScope updateAbstractScopeElement(AbstractScope scopeElement,int operationType) throws Exception
	{
		ScopeElementScope scopeElementScope = new ScopeElementScope();
		
		switch (operationType) 
		{
			case TYPE_CLIENT_COST_CENTER:
				
				ClientCostCenter ccc = createAndSaveCCC();
				scopeElement.setClientCostCenter(ccc);
				scopeElementScope.setClientCostCenter(true);
			
			break;
			
			case TYPE_ASSET:
				
				Asset asset = createAndSaveAsset();
				scopeElement.setAsset(asset);
				scopeElementScope.setAsset(true);
			
			break;
			
			case TYPE_ASSET_CODE:
				
				AssetFinancialCategory assetFinancialCategoty = createAndSaveAssetFinancialCategory(DatafieldFixture.ASSET_FINANCIAL_CATEGORY_PREVENT_MAINTENANCE);
				scopeElement.setAssetCode(assetFinancialCategoty);
				scopeElementScope.setAssetCode(true);
			
			break;
			
			default:
			break;
		}
		
		return (AbstractScope)save(scopeElement,scopeElementScope).getRpmObject();
	}
	
	private ClientCostCenter createAndSaveCCC() throws Exception
	{
		ClientCostCenter ccc = ClientCostCenterFixture.createFixture(null, this);
		
		ccc = (ClientCostCenter)save(ccc, null).getRpmObject();
		
		assertNotNull(ccc);
		
		return ccc;
	}
	
	private Asset createAndSaveAsset() throws Exception
	{
    	Asset asset = AssetFixture.createFixture(null, this);
    	
    	SingleResult result = save(asset, null);

    	asset = (Asset) result.getRpmObject();
    	
    	assertNotNull(asset);
    	
    	return asset;
	}
	
	private AssetFinancialCategory createAndSaveAssetFinancialCategory(String category) throws Exception
	{
		AssetFinancialCategory assetFinancialCategory = DatafieldFixture.createAssetFinancialCategory(category);
		
		assetFinancialCategory = (AssetFinancialCategory)save(assetFinancialCategory, null).getRpmObject();
		
		assertNotNull(assetFinancialCategory);
		
		return assetFinancialCategory;
	}
}
