package com.ibm.rpm.client.junit.scopemanagement.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.RpmTestProperties;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.wbs.containers.Proposal;

public class Test65113 extends ClientTestController {
	
	private Proposal proposal = null;
	private ScopeElement parent = null;
	
	protected void setUp() throws Exception
    {
        super.setUp();
        proposal = createProposal();
        parent = createAndSaveScopeElement(proposal);
	}
	
	public void testSaveScopeMilestoneWithProposedByNameAsNull() throws Exception
	{
		Calendar calendar = Calendar.getInstance();
		ScopeMilestone scopeMilestone = saveScopeMilestoneWithCalendars(calendar,null);
		
		String currentUserFullName = getCurrentUser().getFullName();
		
		assertNotNull(scopeMilestone);
		assertEquals(currentUserFullName, scopeMilestone.getProposedByName());
		validateByDate(calendar,scopeMilestone.getProposedByDate());
	}
	
	public void testSaveScopeMilestoneWithClosedByNameAsNull() throws Exception
	{
		Calendar calendar = Calendar.getInstance();
		ScopeMilestone scopeMilestone = saveScopeMilestoneWithCalendars(null,calendar);
		
		String currentUserFullName = getCurrentUser().getFullName();
		
		assertNotNull(scopeMilestone);
		assertEquals(currentUserFullName, scopeMilestone.getClosedByName());
		validateByDate(calendar,scopeMilestone.getClosedByDate());
	}
	
	public void testSaveScopeMilestoneWithValidResourceNames() throws Exception
	{
		saveAndValidateScopeMilestoneWithValidResourceNames(true);
	}
	
	public void testSaveScopeMilestoneWithInvalidResourceNames() throws Exception
	{
		saveAndValidateScopeMilestoneWithValidResourceNames(false);
	}
	
	public void testUpdateScopeMilestoneWithNullResourceNames() throws Exception
	{
		ScopeMilestone scopeMilestone = saveAndValidateScopeMilestoneWithValidResourceNames(true);
		
		scopeMilestone.setProposedByName(null);
		scopeMilestone.setClosedByName(null);
			
		scopeMilestone = (ScopeMilestone)save(scopeMilestone,null).getRpmObject();
		
		assertNotNull(scopeMilestone);
		assertNull(scopeMilestone.getProposedByName());
		assertNull(scopeMilestone.getClosedByName());
		assertNotNull(scopeMilestone.getProposedByDate());
		assertNotNull(scopeMilestone.getClosedByDate());
	}
	
	public void testUpdateScopeMilestoneWithNullCalendars() throws Exception
	{
		ScopeMilestone scopeMilestone = saveAndValidateScopeMilestoneWithValidResourceNames(true);
		
		scopeMilestone.setProposedByDate(null);
		scopeMilestone.setClosedByDate(null);
			
		scopeMilestone = (ScopeMilestone)save(scopeMilestone,null).getRpmObject();
		
		assertNotNull(scopeMilestone);
		assertNotNull(scopeMilestone.getProposedByName());
		assertNotNull(scopeMilestone.getClosedByName());
		assertNull(scopeMilestone.getProposedByDate());
		assertNull(scopeMilestone.getClosedByDate());
	}
	
	private ScopeMilestone saveAndValidateScopeMilestoneWithValidResourceNames(boolean isValidResourceName) throws Exception
	{
		String proposedByResourceName;
		String closedByResourceName;
		
		if(isValidResourceName)
		{
			Resource proposedByResource = createAndSaveResource();
			Resource closedByResource = createAndSaveResource();
			
			proposedByResourceName = proposedByResource.getFullName();
			closedByResourceName = closedByResource.getFullName();
		}
		else
		{
			proposedByResourceName = HelperTestUtilities.createName();
			closedByResourceName = HelperTestUtilities.createName();
		}
		
		ScopeMilestone scopeMileStone = saveScopeMilestoneWithResourceNames(proposedByResourceName, closedByResourceName);
		
		assertNotNull(scopeMileStone);
		assertEquals(proposedByResourceName, scopeMileStone.getProposedByName());
		assertEquals(closedByResourceName, scopeMileStone.getClosedByName());
		assertNotNull(scopeMileStone.getProposedByDate());
		assertNotNull(scopeMileStone.getClosedByDate());
		
		return scopeMileStone;
	}
	
	private ScopeMilestone saveScopeMilestoneWithResourceNames(String proposedByResourceName, String closedByResourceName) throws Exception
	{
		ScopeMilestone scopeMileStone = new ScopeMilestone();
		scopeMileStone.setName(HelperTestUtilities.createName());
		scopeMileStone.setParent(parent);
		scopeMileStone.setProposedByName(proposedByResourceName);
		scopeMileStone.setClosedByName(closedByResourceName);
		
		scopeMileStone = (ScopeMilestone)save(scopeMileStone,null).getRpmObject();
		
		return scopeMileStone;
	}
	
	private ScopeMilestone saveScopeMilestoneWithCalendars(Calendar proposedByDate, Calendar closedByDate) throws Exception
	{
		ScopeMilestone scopeMileStone = new ScopeMilestone();
		scopeMileStone.setName(HelperTestUtilities.createName());
		scopeMileStone.setParent(parent);
		scopeMileStone.setProposedByDate(proposedByDate);
		scopeMileStone.setClosedByDate(closedByDate);
		
		scopeMileStone = (ScopeMilestone)save(scopeMileStone,null).getRpmObject();
		
		return scopeMileStone;
	}
	
	private Proposal createProposal() throws Exception
	{
		Proposal proposal = new Proposal();
		proposal.setName(HelperTestUtilities.createName());
		
		proposal = (Proposal)save(proposal, null).getRpmObject();
		
		assertNotNull(proposal);
		
		return proposal;
	}
	
	private ScopeElement createAndSaveScopeElement(Proposal proposal) throws Exception
	{
		String xPath = "/ScopeFolder [name = '" + proposal.getName() + "']";
		ScopeFolder rootScopeFolder = (ScopeFolder)loadOneFromXpath(xPath, null);
		assertNotNull(rootScopeFolder);
		
		Requirement requirement = new Requirement();
		requirement.setName(HelperTestUtilities.createName());
		requirement.setParent(rootScopeFolder);
		
		requirement = (Requirement)save(requirement, null).getRpmObject();
		assertNotNull(requirement);
		return requirement;
	}
	
	private Resource createAndSaveResource() throws Exception
	{
		Pool parentPool = PoolFixture.retrieveStaticFixture(this);
		Resource resource = ResourceFixture.createFixture(parentPool,null, this);

		resource = (Resource)save(resource, null ).getRpmObject();

		assertNotNull(resource);
		
		return resource;
	}
	
	private Resource getCurrentUser() throws Exception
	{
		String userName = RpmTestProperties.getProperty("rpm.test.user.name");
		Resource resource = (Resource)loadFromXpath("/Resource[username = '" + userName + "']", null)[0];
		assertNotNull(resource);
		return resource;
	}
	
	private void validateByDate(Calendar expectedByDate, Calendar byDate)
	{
		assertNotNull(byDate);
		assertEquals(expectedByDate.get(Calendar.YEAR),byDate.get(Calendar.YEAR));
		assertEquals(expectedByDate.get(Calendar.MONTH),byDate.get(Calendar.MONTH));
		assertEquals(expectedByDate.get(Calendar.DAY_OF_MONTH),byDate.get(Calendar.DAY_OF_MONTH));
	}
}
