/*
 * Created on 16-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.ApprovedCostBenefits;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestApprovedCostBenefitsValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private ScopeElementScope mainScope;

    private Proposal mainGrandParent;

    private ScopeFolder mainParent;

    private ChangeRequest mainChangeRequest;

    private ApprovedCostBenefits mainApprovedCostBenefits;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ScopeElementScope();
        mainScope.setApprovedCostBenefits(true);

        mainGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGrandParent = (Proposal) saveMustSucceed(mainGrandParent, Proposal.class,
            null);
        mainGrandParent = (Proposal) loadFromID(mainGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGrandParent.getName() + "']", new ScopeElementScope());

        mainParent = (ScopeFolder) rpmObjects[0];
        mainParent.setContextName(contextName);

        mainChangeRequest = new ChangeRequest();
        mainChangeRequest.setName("Name");
        mainChangeRequest.setParent(mainParent);

        mainApprovedCostBenefits = new ApprovedCostBenefits();
        mainChangeRequest.setApprovedCostBenefits(mainApprovedCostBenefits);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainApprovedCostBenefits.setBillableRevenue(new Double(
            ValidationConstants.ACB_BILLABLEREVENUE_MAX));
        mainApprovedCostBenefits.setCost(new Double(ValidationConstants.ACB_COST_MAX));
        mainApprovedCostBenefits.setNonBillableBenefits(new Double(
            ValidationConstants.ACB_NONBILLABLEBENEFITS_MAX));
        mainApprovedCostBenefits.setScheduleImpactInDays(new Integer(
            ValidationConstants.ACB_SCHEDULEIMPACTINDAYS_MAX));
        mainApprovedCostBenefits.setWorkInHours(new Integer(
            ValidationConstants.ACB_WORKINHOURS_MAX));

        // Attempt to save the ChangeRequest
        mainChangeRequest = (ChangeRequest) saveMustSucceed(mainChangeRequest,
            ChangeRequest.class, mainScope);

        // Load the inserted ChangeRequest from id
        RPMObject obj = new ChangeRequest();
        obj.setID(mainChangeRequest.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ChangeRequest);

        // Check that the loaded ChangeRequest is identical
        ScopeElementFixture.compare(mainChangeRequest, (ChangeRequest) rpmObject,
            mainScope);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainApprovedCostBenefits.setBillableRevenue(new Double(
            ValidationConstants.ACB_BILLABLEREVENUE_MIN));
        mainApprovedCostBenefits.setCost(new Double(ValidationConstants.ACB_COST_MIN));
        mainApprovedCostBenefits.setNonBillableBenefits(new Double(
            ValidationConstants.ACB_NONBILLABLEBENEFITS_MIN));
        mainApprovedCostBenefits.setScheduleImpactInDays(new Integer(
            ValidationConstants.ACB_SCHEDULEIMPACTINDAYS_MIN));
        mainApprovedCostBenefits.setWorkInHours(new Integer(
            ValidationConstants.ACB_WORKINHOURS_MIN));

        // Attempt to save the ChangeRequest
        mainChangeRequest = (ChangeRequest) saveMustSucceed(mainChangeRequest,
            ChangeRequest.class, mainScope);

        // Load the inserted ChangeRequest from id
        RPMObject obj = new ChangeRequest();
        obj.setID(mainChangeRequest.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ChangeRequest);

        // Check that the loaded ChangeRequest is identical
        ScopeElementFixture.compare(mainChangeRequest, (ChangeRequest) rpmObject,
            mainScope);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainApprovedCostBenefits.setBillableRevenue(new Double(
            ValidationConstants.ACB_BILLABLEREVENUE_MAX + 0.01));
        mainApprovedCostBenefits.setCost(new Double(
            ValidationConstants.ACB_COST_MAX + 0.01));
        mainApprovedCostBenefits.setNonBillableBenefits(new Double(
            ValidationConstants.ACB_NONBILLABLEBENEFITS_MAX + 0.01));
        mainApprovedCostBenefits.setScheduleImpactInDays(new Integer(
            ValidationConstants.ACB_SCHEDULEIMPACTINDAYS_MAX + 1));
        mainApprovedCostBenefits.setWorkInHours(new Integer(
            ValidationConstants.ACB_WORKINHOURS_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainApprovedCostBenefits
                .getClass().getName(), "billableRevenue", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainApprovedCostBenefits
                .getClass().getName(), "cost", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainApprovedCostBenefits
                .getClass().getName(), "nonBillableBenefits", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainApprovedCostBenefits
                .getClass().getName(), "scheduleImpactInDays", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainApprovedCostBenefits
                .getClass().getName(), "workInHours", null)
        };

        verifySaveMustFail(mainChangeRequest, mainScope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainApprovedCostBenefits.setBillableRevenue(new Double(
            ValidationConstants.ACB_BILLABLEREVENUE_MIN - 0.01));
        mainApprovedCostBenefits.setCost(new Double(
            ValidationConstants.ACB_COST_MIN - 0.01));
        mainApprovedCostBenefits.setNonBillableBenefits(new Double(
            ValidationConstants.ACB_NONBILLABLEBENEFITS_MIN - 0.01));
        mainApprovedCostBenefits.setScheduleImpactInDays(new Integer(
            ValidationConstants.ACB_SCHEDULEIMPACTINDAYS_MIN - 1));
        mainApprovedCostBenefits.setWorkInHours(new Integer(
            ValidationConstants.ACB_WORKINHOURS_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainApprovedCostBenefits
                .getClass().getName(), "billableRevenue", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainApprovedCostBenefits
                .getClass().getName(), "cost", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainApprovedCostBenefits
                .getClass().getName(), "nonBillableBenefits", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainApprovedCostBenefits
                .getClass().getName(), "scheduleImpactInDays", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainApprovedCostBenefits
                .getClass().getName(), "workInHours", null)
        };

        verifySaveMustFail(mainChangeRequest, mainScope, expectedExceptions);
    }
}
