/*
 * Created on 16-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestChangeRequestValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private Proposal mainGrandParent;

    private ScopeFolder mainParent;

    private ChangeRequest mainChangeRequest;

    private Calendar calendar1;

    private Calendar calendar2;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGrandParent = (Proposal) saveMustSucceed(mainGrandParent, Proposal.class,
            null);
        mainGrandParent = (Proposal) loadFromID(mainGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGrandParent.getName() + "']", new ScopeElementScope());

        mainParent = (ScopeFolder) rpmObjects[0];
        mainParent.setContextName(contextName);

        mainChangeRequest = new ChangeRequest();
        mainChangeRequest.setName("Name");
        mainChangeRequest.setParent(mainParent);

        calendar1 = Calendar.getInstance();
        calendar1.set(2009, 11, 31);
        calendar2 = Calendar.getInstance();
        calendar2.set(2009, 11, 30);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainChangeRequest.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainChangeRequest.getClass()
                .getName(), "parent", mainChangeRequest.getID())
        };

        verifySaveMustFail(mainChangeRequest, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainChangeRequest.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainChangeRequest.getClass()
                .getName(), "parent", mainChangeRequest.getID())
        };

        verifySaveMustFail(mainChangeRequest, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainChangeRequest.setName(getTestString(ValidationConstants.NAME_MAX));
        mainChangeRequest
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX));
        mainChangeRequest.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));
        mainChangeRequest.setRank(new Integer(ValidationConstants.RANK_MAX));
        mainChangeRequest.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MAX));

        // Dates
        mainChangeRequest.setEstimatedStartDate(calendar1);
        mainChangeRequest.setEstimatedFinishDate(calendar1);

        // Attempt to save the ChangeRequest
        mainChangeRequest = (ChangeRequest) saveMustSucceed(mainChangeRequest,
            ChangeRequest.class, null);

        // Load the inserted ChangeRequest from id
        RPMObject obj = new ChangeRequest();
        obj.setID(mainChangeRequest.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ChangeRequest);

        // Check that the loaded ChangeRequest is identical
        ScopeElementFixture.compare(mainChangeRequest, (ChangeRequest) rpmObject, null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainChangeRequest.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));
        mainChangeRequest.setRank(new Integer(ValidationConstants.RANK_MIN));
        mainChangeRequest.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MIN));

        // Attempt to save the ChangeRequest
        mainChangeRequest = (ChangeRequest) saveMustSucceed(mainChangeRequest,
            ChangeRequest.class, null);

        // Load the inserted ChangeRequest from id
        RPMObject obj = new ChangeRequest();
        obj.setID(mainChangeRequest.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ChangeRequest);

        // Check that the loaded ChangeRequest is identical
        ScopeElementFixture.compare(mainChangeRequest, (ChangeRequest) rpmObject, null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainChangeRequest.setName(getTestString(ValidationConstants.NAME_MAX + 1));
        mainChangeRequest.setProposedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_PROPOSED_BY_NAME_MAX + 1));
        mainChangeRequest.setApprovedByName(getTestString(ValidationConstants.APPROVED_BY_NAME_MAX + 1));
        mainChangeRequest.setClosedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_CLOSED_BY_NAME_MAX + 1));
        mainChangeRequest
            .setExternalReferenceNumber(getTestString(ValidationConstants.EXTERNALREFERENCENUMBER_MAX + 1));
        mainChangeRequest.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));
        mainChangeRequest.setRank(new Integer(ValidationConstants.RANK_MAX + 1));
        mainChangeRequest.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MAX + 1));

        // Dates
        mainChangeRequest.setEstimatedStartDate(calendar1);
        mainChangeRequest.setEstimatedFinishDate(calendar2);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainChangeRequest.getClass()
                .getName(), "name", mainChangeRequest.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainChangeRequest.getClass()
                .getName(), "proposedByName", mainChangeRequest.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainChangeRequest.getClass()
                .getName(), "approvedByName", mainChangeRequest.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainChangeRequest.getClass()
                .getName(), "closedByName", mainChangeRequest.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainChangeRequest.getClass()
                .getName(), "externalReferenceNumber", mainChangeRequest.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainChangeRequest.getClass()
                .getName(), "priority", mainChangeRequest.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainChangeRequest.getClass()
                .getName(), "rank", mainChangeRequest.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainChangeRequest.getClass()
                .getName(), "numberTimesReassigned", mainChangeRequest.getID()),
            createRPMException(ErrorCodes.INVALID_DATES, mainChangeRequest.getClass()
                .getName(), "estimatedStartDate", mainChangeRequest.getID())
        };

        verifySaveMustFail(mainChangeRequest, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainChangeRequest.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));
        mainChangeRequest.setRank(new Integer(ValidationConstants.RANK_MIN - 1));
        mainChangeRequest.setNumberTimesReassigned(new Integer(
            ValidationConstants.NUMBEROFTIMESREASSIGNED_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainChangeRequest.getClass()
                .getName(), "priority", mainChangeRequest.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainChangeRequest.getClass()
                .getName(), "rank", mainChangeRequest.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainChangeRequest.getClass()
                .getName(), "numberTimesReassigned", mainChangeRequest.getID())
        };

        verifySaveMustFail(mainChangeRequest, null, expectedExceptions);
    }

    public void testFailureInvalidType() throws AssertionError, Exception
    {
        ScopeElementScope scope = new ScopeElementScope();
        scope.setState(true);

        // ChangeRequest must have state of type ChangeRequestState
        mainChangeRequest.setState(new ActionState());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, mainChangeRequest.getClass()
                .getName(), "state", mainChangeRequest.getID())
        };

        verifySaveMustFail(mainChangeRequest, scope, expectedExceptions);
    }
}
