/*
* Created on 12-Sep-2006
* @author rbadette
* Description
*/

package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import java.util.GregorianCalendar;

import junit.framework.Assert;

import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.applicationadministration.containers.AssetFinancialCategory;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.scopemanagement.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ApprovedCostBenefits;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Impact;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.OrderOfMagnitudeCostBenefits;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.RiskMatrix;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeScheduleDate;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;


public class TestCreateObjectFromTemplateValidation extends GenericValidation
{
    protected ScopeElementScope mainScope;

    protected ScopeFolder mainFolder = null;

    protected WorkElementScope proposalScope;

    protected Proposal proposal = null;

    protected int nbChildren = 1;

    private GregorianCalendar calendar = null;

    private ScopeScheduleDate scopeDate = null;

    public void setUp() throws Exception
    {
        super.setUp();

        scopeDate = new ScopeScheduleDate();
        calendar = new GregorianCalendar();

        mainScope = new ScopeElementScope();
        mainScope.setParent(new ScopeElementScope());
        mainScope.setChildren(mainScope);

        proposalScope = new WorkElementScope();

        proposal = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            proposalScope, this);
        proposal.setEstimatedFinishDate(proposal.getEstimatedStartDate());
        save(proposal, proposalScope);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", new ScopeElementScope());
        mainFolder = (ScopeFolder) rpmObjects[0];
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    private void verifySaveMustFailForTemplate(RPMObject obj,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings, SingleResult saveResult)
            throws AssertionError, Exception
    {
        // Check that we received exceptions if we were expecting some
        // and that they are the correct ones
        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
            expectedWarnings, saveResult);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }

    public void testNullTemplateField() throws AssertionError, Exception
    {
        AbstractScope template = null;

        Action newAction = new Action();
        newAction.setParent(mainFolder);

        //List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
            createRPMException(
                com.ibm.rpm.framework.constants.ErrorCodes.CANNOT_CLEAR_FIELD,
                newAction.getClass().getName(),
                ValidationConstants.TEMPLATE_FIELD, newAction.getID())
        };

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, newAction, template, mainScope);

        verifySaveMustFailForTemplate(newAction, expectedErrors, null, result);
    }

    public void testInvalidTemplatePublishedTypeField() throws AssertionError,
            Exception
    {
        AbstractScope template = null;

        //Loading template from Xpath: published='ToTemplate'
        RPMObject[] rpmObjects = loadFromXpath("/Issue[published='"
                + ProjectPublishedType._ToTemplate + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Issue);

        template = (Issue) rpmObjects[0];
        template.setPublished(PublishedType.None);

        Issue newIssue = new Issue();
        newIssue.setParent(mainFolder);

        //List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
            createRPMException(ErrorCodes.INVALID_OBJECT_TEMPLATE_TYPE,
                newIssue.getClass().getName(),
                ValidationConstants.TEMPLATE_FIELD, newIssue.getID())
        };

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, newIssue, template, mainScope);

        verifySaveMustFailForTemplate(newIssue, expectedErrors, null, result);
    }

    public void testInvalidTemplateNullPublishedTypeField()
            throws AssertionError, Exception
    {
        AbstractScope template = null;

        //Loading template from Xpath: published='ToTemplate'
        RPMObject[] rpmObjects = loadFromXpath("/Risk[published='"
                + ProjectPublishedType._ToTemplate + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Risk);

        template = (Risk) rpmObjects[0];
        template.setPublished(null);

        Risk newRisk = new Risk();
        newRisk.setParent(mainFolder);

        //List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
            createRPMException(ErrorCodes.INVALID_OBJECT_TEMPLATE_TYPE, newRisk
                .getClass().getName(), ValidationConstants.TEMPLATE_FIELD,
                newRisk.getID())
        };

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, newRisk, template, mainScope);

        verifySaveMustFailForTemplate(newRisk, expectedErrors, null, result);
    }

    public void testInvalidInputID() throws AssertionError, Exception
    {
        AbstractScope template = null;

        //Loading template from Xpath: published='ToTemplate'
        RPMObject[] rpmObjects = loadFromXpath("/Requirement[published='"
                + ProjectPublishedType._ToTemplate + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Requirement);

        template = (Requirement) rpmObjects[0];

        Requirement existingRequirement = (Requirement) ScopeElementFixture
            .retrieveStaticFixture(mainFolder, new Requirement(),
                new ScopeElementScope(), 1, this);
        existingRequirement.setParent(mainFolder);

        //List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
            createRPMException(
                com.ibm.rpm.framework.constants.ErrorCodes.MUST_BE_NULL,
                existingRequirement.getClass().getName(),
                ValidationConstants.ID_FIELD, existingRequirement.getID())
        };

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, existingRequirement, template,
                mainScope);

        verifySaveMustFailForTemplate(existingRequirement, expectedErrors,
            null, result);
    }

    public void testInvalidTemplateAndInputID() throws AssertionError,
            Exception
    {
        AbstractScope newTemplate = new Action();

        Action existingAction = (Action) ScopeElementFixture
            .retrieveStaticFixture(mainFolder, new Action(),
                new ScopeElementScope(), 1, this);
        existingAction.setParent(mainFolder);

        //List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.MUST_BE_NULL,
                    existingAction.getClass().getName(),
                    ValidationConstants.ID_FIELD, existingAction.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.NO_ID,
                    existingAction.getClass().getName(),
                    ValidationConstants.TEMPLATE_FIELD, existingAction.getID()),
        };

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, existingAction, newTemplate,
                mainScope);

        verifySaveMustFailForTemplate(existingAction, expectedErrors, null,
            result);
    }

    public void testInputHasNoParent() throws AssertionError, Exception
    {
        AbstractScope template = null;

        //Loading template from Xpath: published='ToTemplate'
        RPMObject[] rpmObjects = loadFromXpath("/ChangeRequest[published='"
                + ProjectPublishedType._ToTemplate + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ChangeRequest);

        template = (ChangeRequest) rpmObjects[0];

        ChangeRequest newChgReq = new ChangeRequest();

        //List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
            createRPMException(
                com.ibm.rpm.framework.constants.ErrorCodes.MANDATORY, newChgReq
                    .getClass().getName(), ValidationConstants.PARENT_FIELD,
                newChgReq.getID())
        };

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, newChgReq, template, mainScope);

        verifySaveMustFailForTemplate(newChgReq, expectedErrors, null, result);
    }

    public void testInvalidInputObjectTypeAndTypeMismatch()
            throws AssertionError, Exception
    {
        AbstractScope template = null;

        //Loading template from Xpath: published='ToTemplate'
        RPMObject[] rpmObjects = loadFromXpath("/Defect[published='"
                + ProjectPublishedType._ToTemplate + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Defect);

        template = (Defect) rpmObjects[0];

        ScopeTask badInput = new ScopeTask();
        badInput.setParent(mainFolder);

        //List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.INVALID_TYPE,
                    badInput.getClass().getName(),
                    ValidationConstants.TEMPLATE_INPUT_FIELD, badInput.getID()),
                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.INVALID_TYPE,
                    badInput.getClass().getName(),
                    ValidationConstants.TEMPLATE_INPUT_FIELD, badInput.getID())
        };

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, badInput, template, mainScope);

        verifySaveMustFailForTemplate(badInput, expectedErrors, null, result);
    }

    public void testInvalidTemplateObjectTypeAndTypeMismatch()
            throws AssertionError, Exception
    {
        RPMObject template = null;

        //Loading template from Xpath: published='ToTemplate'
        RPMObject[] rpmObjects = loadFromXpath("/GenericProject[published='"
                + ProjectPublishedType._ToTemplate + "']",
            new WorkElementScope());
        assertNotNull(rpmObjects);

        template = rpmObjects[0];

        ServiceRequest newServReq = new ServiceRequest();
        newServReq.setParent(mainFolder);

        //List of expected Errors...
        RPMExceptionMock[] expectedErrors =
        {
                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.INVALID_TYPE,
                    template.getClass().getName(),
                    ValidationConstants.TEMPLATE_FIELD, template.getID()),
                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.INVALID_TYPE,
                    newServReq.getClass().getName(),
                    ValidationConstants.TEMPLATE_INPUT_FIELD, newServReq
                        .getID())
        };

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, newServReq, template,
                mainScope);

        verifySaveMustFailForTemplate(newServReq, expectedErrors, null, result);
    }

    public void testAbstractScopeReadOnlyFields() throws AssertionError,
            Exception
    {

        AbstractScope template = null;

        //Loading template from Xpath: published='ToTemplate'
        RPMObject[] rpmObjects = loadFromXpath("/Action[published='"
                + ProjectPublishedType._ToTemplate + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Action);

        template = (Action) rpmObjects[0];

        Action newAction = new Action();
        newAction.setParent(mainFolder);

        //List of expected Warnings...
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.APPROVED_BY_DATE_FIELD, newAction
                        .getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.APPROVED_BY_NAME_FIELD, newAction
                        .getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.APPROVED_DATE_FIELD, newAction.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.ASSET_CODE_FIELD, newAction.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.ASSET_FIELD, newAction.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.ASSIGNED_SCORECARD_FIELD, newAction
                        .getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.CLIENT_COST_CENTER_FIELD, newAction
                        .getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.DEFAULT_SCOPE_FIELD, newAction.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.DOCUMENT_FOLDER_FIELD, newAction
                        .getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.DUPLICATE_NUMBER_FIELD, newAction
                        .getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.EXTERNAL_REFERENCE_NUMBER_FIELD,
                    newAction.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.IMPACT_FIELD, newAction.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.ORDER_OF_MAGNITUDE_COSTBENEFITS_FIELD,
                    newAction.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.PUBLISHED_FIELD, newAction.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.RANK_FIELD, newAction.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.STATE_FIELD, newAction.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newAction.getClass().getName(),
                    ValidationConstants.TARGET_DATE_FIELD, newAction.getID()),

        };

        //Setting Values to fields
        newAction.setApprovedByDate(calendar);
        newAction.setApprovedByName(getValidResource().getFullName());
        newAction.setApprovedDate(scopeDate);
        newAction.setAssetCode(new AssetFinancialCategory());
        newAction.setAsset(new Asset());
        newAction.setAssignedScorecard(new AssignedScorecard());
        newAction.setClientCostCenter(new ClientCostCenter());
        newAction.setDefaultScope(new Boolean(false));
        newAction.setDocumentFolder(new DocumentFolder());
        newAction.setDuplicateNumber("");
        newAction.setExternalReferenceNumber("");
        newAction.setImpact(new Impact());
        newAction
            .setOrderOfMagnitudeCostBenefits(new OrderOfMagnitudeCostBenefits());
        newAction.setPublished(PublishedType.None);
        newAction.setRank(new Integer(0));
        newAction.setState(new ActionState());
        newAction.setTargetDate(calendar);

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, newAction, template, mainScope);

        verifySaveMustFailForTemplate(newAction, null, expectedWarnings, result);

    }

    //	   public void testAbstractRequestReadOnlyFields() throws AssertionError,
    //			Exception {
    //
    //		AbstractScope template = null;
    //
    //		//Loading template from Xpath: published='ToTemplate'
    //		RPMObject[] rpmObjects = loadFromXpath("/Defect[published='"
    //				+ PublishedType._ToTemplate + "']", mainScope);
    //		assertNotNull(rpmObjects);
    //		assertTrue(rpmObjects[0] instanceof Defect);
    //
    //		template = (Defect) rpmObjects[0];
    //
    //		Defect newDefect = new Defect();
    //		newDefect.setParent(mainFolder);
    //		
    //		
    //		//List of expected Warnings...
    //		RPMExceptionMock[] expectedWarnings = {
    //				createRPMException(
    //						com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
    //						newDefect.getClass().getName(),
    //						ValidationConstants.APPROVED_COST_BENEFITS_FIELD,
    //						newDefect.getID()),
    //
    //				createRPMException(
    //						com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
    //						newDefect.getClass().getName(),
    //						ValidationConstants.NUMBER_TIMES_REASSIGNED_FIELD,
    //						newDefect.getID()),
    //
    //				createRPMException(
    //						com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
    //						newDefect.getClass().getName(),
    //						ValidationConstants.OCCURED_DATE_TIME_FIELD, newDefect
    //								.getID()) };
    //		
    //		
    //		//Setting Values to fields
    //		newDefect.setApprovedCostBenefits(new ApprovedCostBenefits());
    //		newDefect.setNumberTimesReassigned(new Integer(1));
    //		newDefect.setOccuredDateTime(calendar);
    //
    //
    //		SingleResult result = getApplicationInterface()
    //				.createObjectFromTemplate(sessionid, newDefect, template,
    //						mainScope);
    //
    //		verifySaveMustFailForTemplate(newDefect, null, expectedWarnings, result);
    //
    //	}

    public void testActionReadOnlyFields() throws AssertionError, Exception
    {

        AbstractScope template = null;

        //Loading template from Xpath: published='ToTemplate'
        RPMObject[] rpmObjects = loadFromXpath("/Action[published='"
                + ProjectPublishedType._ToTemplate + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Action);

        template = (Action) rpmObjects[0];

        Action newAction = new Action();
        newAction.setParent(mainFolder);

        //List of expected Warnings...
        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                newAction.getClass().getName(),
                ValidationConstants.APPROVED_COST_BENEFITS_FIELD, newAction
                    .getID())
        };

        //Setting Values to fields
        newAction.setApprovedCostBenefits(new ApprovedCostBenefits());

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, newAction, template, mainScope);

        verifySaveMustFailForTemplate(newAction, null, expectedWarnings, result);

    }

    public void testIssueReadOnlyFields() throws AssertionError, Exception
    {

        AbstractScope template = null;

        //Loading template from Xpath: published='ToTemplate'
        RPMObject[] rpmObjects = loadFromXpath("/Issue[published='"
                + ProjectPublishedType._ToTemplate + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Issue);

        template = (Issue) rpmObjects[0];

        Issue newIssue = new Issue();
        newIssue.setParent(mainFolder);

        //List of expected Warnings...
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newIssue.getClass().getName(),
                    ValidationConstants.NUMBER_TIMES_REASSIGNED_FIELD, newIssue
                        .getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newIssue.getClass().getName(),
                    ValidationConstants.OCCURED_DATE_TIME_FIELD, newIssue
                        .getID())
        };

        //Setting Values to fields
        newIssue.setNumberTimesReassigned(new Integer(1));
        newIssue.setOccuredDateTime(calendar);

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, newIssue, template, mainScope);

        verifySaveMustFailForTemplate(newIssue, null, expectedWarnings, result);

    }

    public void testRequirementReadOnlyFields() throws AssertionError,
            Exception
    {

        AbstractScope template = null;

        //Loading template from Xpath: published='ToTemplate'
        RPMObject[] rpmObjects = loadFromXpath("/Requirement[published='"
                + ProjectPublishedType._ToTemplate + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Requirement);

        template = (Requirement) rpmObjects[0];

        Requirement newRequirement = new Requirement();
        newRequirement.setParent(mainFolder);

        //List of expected Warnings...
        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                newRequirement.getClass().getName(),
                ValidationConstants.APPROVED_COST_BENEFITS_FIELD,
                newRequirement.getID())
        };

        //Setting Values to fields
        newRequirement.setApprovedCostBenefits(new ApprovedCostBenefits());

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, newRequirement, template,
                mainScope);

        verifySaveMustFailForTemplate(newRequirement, null, expectedWarnings,
            result);

    }

    public void testRiskReadOnlyFields() throws AssertionError, Exception
    {

        AbstractScope template = null;

        //Loading template from Xpath: published='ToTemplate'
        RPMObject[] rpmObjects = loadFromXpath("/Risk[published='"
                + ProjectPublishedType._ToTemplate + "']", mainScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Risk);

        template = (Risk) rpmObjects[0];

        Risk newRisk = new Risk();
        newRisk.setParent(mainFolder);

        //List of expected Warnings...
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newRisk.getClass().getName(),
                    ValidationConstants.RISK_MATRIX_FIELD, newRisk.getID()),

                createRPMException(
                    com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD,
                    newRisk.getClass().getName(),
                    ValidationConstants.OCCURED_DATE_TIME_FIELD, newRisk
                        .getID())
        };

        //Setting Values to fields
        newRisk.setRiskMatrix(new RiskMatrix());
        newRisk.setOccuredDateTime(calendar);

        SingleResult result = getApplicationInterface()
            .createObjectFromTemplate(sessionid, newRisk, template, mainScope);

        verifySaveMustFailForTemplate(newRisk, null, expectedWarnings, result);

    }
}
