/*
 * Created on 16-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.OrderOfMagnitudeCostBenefits;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestOrderOfMagnitudeCostBenefitsValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private ScopeElementScope mainScope;

    private Proposal mainGrandParent;

    private ScopeFolder mainParent;

    private ChangeRequest mainChangeRequest;

    private OrderOfMagnitudeCostBenefits mainOrderOfMagnitudeCostBenefits;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ScopeElementScope();
        mainScope.setOrderOfMagnitudeCostBenefits(true);

        mainGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGrandParent = (Proposal) saveMustSucceed(mainGrandParent, Proposal.class,
            null);
        mainGrandParent = (Proposal) loadFromID(mainGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGrandParent.getName() + "']", new ScopeElementScope());

        mainParent = (ScopeFolder) rpmObjects[0];
        mainParent.setContextName(contextName);

        mainChangeRequest = new ChangeRequest();
        mainChangeRequest.setName("Name");
        mainChangeRequest.setParent(mainParent);

        mainOrderOfMagnitudeCostBenefits = new OrderOfMagnitudeCostBenefits();
        mainChangeRequest
            .setOrderOfMagnitudeCostBenefits(mainOrderOfMagnitudeCostBenefits);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeBillableRevenue(new Double(
            ValidationConstants.OOMCB_OOMBILLABLEREVENUE_MAX));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeCost(new Double(
            ValidationConstants.OOMCB_OOMCOST_MAX));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeHigh(new Double(
            ValidationConstants.OOMCB_OOMHIGH_MAX));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeLow(new Double(
            ValidationConstants.OOMCB_OOMLOW_MAX));
        mainOrderOfMagnitudeCostBenefits
            .setOrderOfMagnitudeNonBillableBenefits(new Double(
                ValidationConstants.OOMCB_OOMNONBILLABLEBENEFITS_MAX));

        // Attempt to save the ChangeRequest
        mainChangeRequest = (ChangeRequest) saveMustSucceed(mainChangeRequest,
            ChangeRequest.class, mainScope);

        // Load the inserted ChangeRequest from id
        RPMObject obj = new ChangeRequest();
        obj.setID(mainChangeRequest.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ChangeRequest);

        // Check that the loaded ChangeRequest is identical
        ScopeElementFixture.compare(mainChangeRequest, (ChangeRequest) rpmObject,
            mainScope);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeBillableRevenue(new Double(
            ValidationConstants.OOMCB_OOMBILLABLEREVENUE_MIN));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeCost(new Double(
            ValidationConstants.OOMCB_OOMCOST_MIN));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeHigh(new Double(
            ValidationConstants.OOMCB_OOMHIGH_MIN));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeLow(new Double(
            ValidationConstants.OOMCB_OOMLOW_MIN));
        mainOrderOfMagnitudeCostBenefits
            .setOrderOfMagnitudeNonBillableBenefits(new Double(
                ValidationConstants.OOMCB_OOMNONBILLABLEBENEFITS_MIN));

        // Attempt to save the ChangeRequest
        mainChangeRequest = (ChangeRequest) saveMustSucceed(mainChangeRequest,
            ChangeRequest.class, mainScope);

        // Load the inserted ChangeRequest from id
        RPMObject obj = new ChangeRequest();
        obj.setID(mainChangeRequest.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ChangeRequest);

        // Check that the loaded ChangeRequest is identical
        ScopeElementFixture.compare(mainChangeRequest, (ChangeRequest) rpmObject,
            mainScope);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeBillableRevenue(new Double(
            ValidationConstants.OOMCB_OOMBILLABLEREVENUE_MAX + 0.01));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeCost(new Double(
            ValidationConstants.OOMCB_OOMCOST_MAX + 0.01));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeHigh(new Double(
            ValidationConstants.OOMCB_OOMHIGH_MAX + 0.01));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeLow(new Double(
            ValidationConstants.OOMCB_OOMLOW_MAX + 0.01));
        mainOrderOfMagnitudeCostBenefits
            .setOrderOfMagnitudeNonBillableBenefits(new Double(
                ValidationConstants.OOMCB_OOMNONBILLABLEBENEFITS_MAX + 0.01));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainOrderOfMagnitudeCostBenefits
                .getClass().getName(), "orderOfMagnitudeBillableRevenue", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainOrderOfMagnitudeCostBenefits
                .getClass().getName(), "orderOfMagnitudeCost", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainOrderOfMagnitudeCostBenefits
                .getClass().getName(), "orderOfMagnitudeHigh", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainOrderOfMagnitudeCostBenefits
                .getClass().getName(), "orderOfMagnitudeLow", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainOrderOfMagnitudeCostBenefits
                .getClass().getName(), "orderOfMagnitudeNonBillableBenefits", null)
        };

        verifySaveMustFail(mainChangeRequest, mainScope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeBillableRevenue(new Double(
            ValidationConstants.OOMCB_OOMBILLABLEREVENUE_MIN - 0.01));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeCost(new Double(
            ValidationConstants.OOMCB_OOMCOST_MIN - 0.01));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeHigh(new Double(
            ValidationConstants.OOMCB_OOMHIGH_MIN - 0.01));
        mainOrderOfMagnitudeCostBenefits.setOrderOfMagnitudeLow(new Double(
            ValidationConstants.OOMCB_OOMLOW_MIN - 0.01));
        mainOrderOfMagnitudeCostBenefits
            .setOrderOfMagnitudeNonBillableBenefits(new Double(
                ValidationConstants.OOMCB_OOMNONBILLABLEBENEFITS_MIN - 0.01));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainOrderOfMagnitudeCostBenefits
                .getClass().getName(), "orderOfMagnitudeBillableRevenue", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainOrderOfMagnitudeCostBenefits
                .getClass().getName(), "orderOfMagnitudeCost", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainOrderOfMagnitudeCostBenefits
                .getClass().getName(), "orderOfMagnitudeHigh", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainOrderOfMagnitudeCostBenefits
                .getClass().getName(), "orderOfMagnitudeLow", null),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainOrderOfMagnitudeCostBenefits
                .getClass().getName(), "orderOfMagnitudeNonBillableBenefits", null)
        };

        verifySaveMustFail(mainChangeRequest, mainScope, expectedExceptions);
    }
}
