/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestScopeFolderValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private Proposal mainGrandParent;

    private ScopeFolder mainParent;

    private ScopeFolder mainScopeFolder;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGrandParent = (Proposal) WorkElementFixture.createFixture(
            new Proposal(), null, this);
        mainGrandParent = (Proposal) saveMustSucceed(mainGrandParent,
            Proposal.class, null);
        mainGrandParent = (Proposal) loadFromID(mainGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + mainGrandParent.getName() + "']", new ScopeElementScope());

        mainParent = (ScopeFolder) rpmObjects[0];
        mainParent.setContextName(contextName);

        mainScopeFolder = new ScopeFolder();
        mainScopeFolder.setParent(mainParent);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainScopeFolder.setName("TestScopeFolderValidation");
        mainScopeFolder.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainScopeFolder.getClass()
                .getName(), "parent", mainScopeFolder.getID())
        };

        verifySaveMustFail(mainScopeFolder, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        mainScopeFolder.setName("TestScopeFolderValidation");
        
        Education wrongParent = new Education();
        mainScopeFolder.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainScopeFolder
                .getClass().getName(), "parent", mainScopeFolder.getID())
        };

        verifySaveMustFail(mainScopeFolder, null, expectedExceptions);
    }

    public void testFailureWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, mainScopeFolder.getClass()
                .getName(), "name", mainScopeFolder.getID())
        };

        verifySaveMustFail(mainScopeFolder, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        mainScopeFolder.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MIN_LENGTH, mainScopeFolder
                .getClass().getName(), "name", mainScopeFolder.getID())
        };

        verifySaveMustFail(mainScopeFolder, null, expectedExceptions);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainScopeFolder.setName(getTestString(ValidationConstants.NAME_MAX));

        // Attempt to save the ScopeFolder
        mainScopeFolder = (ScopeFolder) saveMustSucceed(mainScopeFolder,
            ScopeFolder.class, null);

        // Load the inserted ScopeFolder from id
        RPMObject obj = new ScopeFolder();
        obj.setID(mainScopeFolder.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ScopeFolder);

        // Check that the loaded ScopeFolder is identical
        ScopeElementFixture.compare(mainScopeFolder, (ScopeFolder) rpmObject,
            null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainScopeFolder
            .setName(getTestString(ValidationConstants.NAME_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainScopeFolder
                .getClass().getName(), "name", mainScopeFolder.getID())
        };

        verifySaveMustFail(mainScopeFolder, null, expectedExceptions);
    }
}
