/*
 * Created on 16-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestScopeMilestoneValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private Proposal mainGreatGrandParent;

    private ScopeFolder mainGrandParent;

    private Action mainParent;
    
    private ScopeMilestone mainScopeMilestone;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGreatGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGreatGrandParent = (Proposal) saveMustSucceed(mainGreatGrandParent, Proposal.class,
            null);
        mainGreatGrandParent = (Proposal) loadFromID(mainGreatGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGreatGrandParent.getName() + "']", new ScopeElementScope());

        mainGrandParent = (ScopeFolder) rpmObjects[0];
        mainGrandParent.setContextName(contextName);

        mainParent = new Action();
        mainParent.setName("Name");
        mainParent.setParent(mainGrandParent);
        
        mainParent = (Action) saveMustSucceed(mainParent, Action.class, null);
        
        mainScopeMilestone = new ScopeMilestone();
        mainScopeMilestone.setName("Name");
        mainScopeMilestone.setParent(mainParent);
        mainParent.setChildren(new ScopeElement[] {mainScopeMilestone});
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainScopeMilestone.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainScopeMilestone.getClass()
                .getName(), "parent", mainScopeMilestone.getID())
        };

        verifySaveMustFail(mainScopeMilestone, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainScopeMilestone.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainScopeMilestone
                .getClass().getName(), "parent", mainScopeMilestone.getID())
        };

        verifySaveMustFail(mainScopeMilestone, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainScopeMilestone.setName(getTestString(ValidationConstants.NAME_MAX));
        mainScopeMilestone.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));

        // Attempt to save the ScopeMilestone
        mainScopeMilestone = (ScopeMilestone) saveMustSucceed(mainScopeMilestone,
            ScopeMilestone.class, null);

        // Load the inserted ScopeMilestone from id
        RPMObject obj = new ScopeMilestone();
        obj.setID(mainScopeMilestone.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ScopeMilestone);

        // Check that the loaded ScopeMilestone is identical
        ScopeElementFixture.compare(mainScopeMilestone, (ScopeMilestone) rpmObject, null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainScopeMilestone.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));

        // Attempt to save the ScopeMilestone
        mainScopeMilestone = (ScopeMilestone) saveMustSucceed(mainScopeMilestone,
            ScopeMilestone.class, null);

        // Load the inserted ScopeMilestone from id
        RPMObject obj = new ScopeMilestone();
        obj.setID(mainScopeMilestone.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ScopeMilestone);

        // Check that the loaded ScopeMilestone is identical
        ScopeElementFixture.compare(mainScopeMilestone, (ScopeMilestone) rpmObject, null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainScopeMilestone.setName(getTestString(ValidationConstants.NAME_MAX + 1));
        mainScopeMilestone.setProposedByName(getTestString(ValidationConstants.ABSTRACTSCOPETASK_PROPOSED_BY_NAME_MAX + 1));
        mainScopeMilestone.setClosedByName(getTestString(ValidationConstants.ABSTRACTSCOPETASK_CLOSED_BY_NAME_MAX + 1));
        mainScopeMilestone.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainScopeMilestone.getClass()
                .getName(), "name", mainScopeMilestone.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainScopeMilestone.getClass()
                        .getName(), ValidationConstants.ABSTRACTSCOPETASK_PROPOSED_BY_NAME_FIELD, mainScopeMilestone.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainScopeMilestone.getClass()
                        .getName(), ValidationConstants.ABSTRACTSCOPETASK_CLOSED_BY_NAME_FIELD, mainScopeMilestone.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainScopeMilestone.getClass()
                .getName(), "priority", mainScopeMilestone.getID())
        };

        verifySaveMustFail(mainScopeMilestone, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainScopeMilestone.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainScopeMilestone.getClass()
                .getName(), "priority", mainScopeMilestone.getID())
        };

        verifySaveMustFail(mainScopeMilestone, null, expectedExceptions);
    }
}
