/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestScopeSummaryTaskValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private Proposal mainGreatGrandParent;

    private ScopeFolder mainGrandParent;

    private Requirement mainParent;
    
    private ScopeSummaryTask mainScopeSummaryTask;
    
    private Calendar calendar1;

    private Calendar calendar2;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGreatGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGreatGrandParent = (Proposal) saveMustSucceed(mainGreatGrandParent, Proposal.class,
            null);
        mainGreatGrandParent = (Proposal) loadFromID(mainGreatGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGreatGrandParent.getName() + "']", new ScopeElementScope());

        mainGrandParent = (ScopeFolder) rpmObjects[0];
        mainGrandParent.setContextName(contextName);

        mainParent = new Requirement();
        mainParent.setName("Name");
        mainParent.setParent(mainGrandParent);
        
        mainParent = (Requirement) saveMustSucceed(mainParent, Requirement.class, null);
        
        mainScopeSummaryTask = new ScopeSummaryTask();
        mainScopeSummaryTask.setName("Name");
        mainScopeSummaryTask.setParent(mainParent);
        
        calendar1 = Calendar.getInstance();
        calendar1.set(2009, 11, 31);
        calendar2 = Calendar.getInstance();
        calendar2.set(2009, 11, 30);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainScopeSummaryTask.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainScopeSummaryTask.getClass().getName(),
                "parent", mainScopeSummaryTask.getID())
        };

        verifySaveMustFail(mainScopeSummaryTask, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainScopeSummaryTask.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainScopeSummaryTask.getClass()
                .getName(), "parent", mainScopeSummaryTask.getID())
        };

        verifySaveMustFail(mainScopeSummaryTask, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainScopeSummaryTask.setName(getTestString(ValidationConstants.NAME_MAX));
        mainScopeSummaryTask.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));

        // Dates
        mainScopeSummaryTask.setEstimatedStartDate(calendar1);
        mainScopeSummaryTask.setEstimatedFinishDate(calendar1);

        // Attempt to save the ScopeSummaryTask
        mainScopeSummaryTask = (ScopeSummaryTask) saveMustSucceed(mainScopeSummaryTask, ScopeSummaryTask.class, null);

        // Load the inserted ScopeSummaryTask from id
        RPMObject obj = new ScopeSummaryTask();
        obj.setID(mainScopeSummaryTask.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ScopeSummaryTask);

        // Check that the loaded ScopeSummaryTask is identical
        ScopeElementFixture.compare(mainScopeSummaryTask, (ScopeSummaryTask) rpmObject, null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainScopeSummaryTask.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));

        // Attempt to save the ScopeSummaryTask
        mainScopeSummaryTask = (ScopeSummaryTask) saveMustSucceed(mainScopeSummaryTask, ScopeSummaryTask.class, null);

        // Load the inserted ScopeSummaryTask from id
        RPMObject obj = new ScopeSummaryTask();
        obj.setID(mainScopeSummaryTask.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ScopeSummaryTask);

        // Check that the loaded ScopeSummaryTask is identical
        ScopeElementFixture.compare(mainScopeSummaryTask, (ScopeSummaryTask) rpmObject, null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainScopeSummaryTask.setName(getTestString(ValidationConstants.NAME_MAX + 1));
        mainScopeSummaryTask.setProposedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_PROPOSED_BY_NAME_MAX + 1));
        mainScopeSummaryTask.setClosedByName(getTestString(ValidationConstants.ABSTRACTAGGREGATESCOPE_CLOSED_BY_NAME_MAX + 1));
        mainScopeSummaryTask.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));

        // Dates
        mainScopeSummaryTask.setEstimatedStartDate(calendar1);
        mainScopeSummaryTask.setEstimatedFinishDate(calendar2);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainScopeSummaryTask.getClass().getName(),
                "name", mainScopeSummaryTask.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainScopeSummaryTask.getClass()
                .getName(), "proposedByName", mainScopeSummaryTask.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainScopeSummaryTask.getClass()
                .getName(), "closedByName", mainScopeSummaryTask.getID()),
                    
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainScopeSummaryTask.getClass().getName(),
                "priority", mainScopeSummaryTask.getID()),
            createRPMException(ErrorCodes.INVALID_DATES, mainScopeSummaryTask.getClass().getName(), 
                "estimatedStartDate", mainScopeSummaryTask.getID())
        };

        verifySaveMustFail(mainScopeSummaryTask, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainScopeSummaryTask.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainScopeSummaryTask.getClass().getName(),
                "priority", mainScopeSummaryTask.getID())
        };

        verifySaveMustFail(mainScopeSummaryTask, null, expectedExceptions);
    }

}
