/*
 * Created on 16-Dec-2005
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;

/**
 * @author dominiqu
 */
public class TestScopeTaskValidation extends GenericValidation
{
    private String contextName = AttributeFixture.SCOPE_CONTEXT;

    private Proposal mainGreatGrandParent;

    private ScopeFolder mainGrandParent;

    private Action mainParent;
    
    private ScopeTask mainScopeTask;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainGreatGrandParent = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            null, this);
        mainGreatGrandParent = (Proposal) saveMustSucceed(mainGreatGrandParent, Proposal.class,
            null);
        mainGreatGrandParent = (Proposal) loadFromID(mainGreatGrandParent, null);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + mainGreatGrandParent.getName() + "']", new ScopeElementScope());

        mainGrandParent = (ScopeFolder) rpmObjects[0];
        mainGrandParent.setContextName(contextName);

        mainParent = new Action();
        mainParent.setName("Name");
        mainParent.setParent(mainGrandParent);
        
        mainParent = (Action) saveMustSucceed(mainParent, Action.class, null);
        
        mainScopeTask = new ScopeTask();
        mainScopeTask.setName("Name");
        mainScopeTask.setParent(mainParent);
        mainParent.setChildren(new ScopeElement[] {mainScopeTask});
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainScopeTask.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainScopeTask.getClass()
                .getName(), "parent", mainScopeTask.getID())
        };

        verifySaveMustFail(mainScopeTask, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        Education wrongParent = new Education();
        mainScopeTask.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainScopeTask
                .getClass().getName(), "parent", mainScopeTask.getID())
        };

        verifySaveMustFail(mainScopeTask, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        mainScopeTask.setName(getTestString(ValidationConstants.NAME_MAX));
        mainScopeTask.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));

        // Attempt to save the ScopeTask
        mainScopeTask = (ScopeTask) saveMustSucceed(mainScopeTask,
            ScopeTask.class, null);

        // Load the inserted ScopeTask from id
        RPMObject obj = new ScopeTask();
        obj.setID(mainScopeTask.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ScopeTask);

        // Check that the loaded ScopeTask is identical
        ScopeElementFixture.compare(mainScopeTask, (ScopeTask) rpmObject, null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set field values with min and verify that they are accepted
        mainScopeTask.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));

        // Attempt to save the ScopeTask
        mainScopeTask = (ScopeTask) saveMustSucceed(mainScopeTask,
            ScopeTask.class, null);

        // Load the inserted ScopeTask from id
        RPMObject obj = new ScopeTask();
        obj.setID(mainScopeTask.getID());
        RPMObject rpmObject = loadFromID(obj, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof ScopeTask);

        // Check that the loaded ScopeTask is identical
        ScopeElementFixture.compare(mainScopeTask, (ScopeTask) rpmObject, null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set fields with max+1 and verify that they are not accepted
        mainScopeTask.setName(getTestString(ValidationConstants.NAME_MAX + 1));
        mainScopeTask.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainScopeTask.getClass()
                .getName(), "name", mainScopeTask.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainScopeTask.getClass()
                .getName(), "priority", mainScopeTask.getID())
        };

        verifySaveMustFail(mainScopeTask, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set fields with min-1 and verify that they are not accepted
        mainScopeTask.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainScopeTask.getClass()
                .getName(), "priority", mainScopeTask.getID())
        };

        verifySaveMustFail(mainScopeTask, null, expectedExceptions);
    }
}
