/*
 * Created on Aug 11, 2005
 *
 */
package com.ibm.rpm.client.junit.scorecard.fixtures;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.scope.AssetModuleScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.asset.fixtures.AssetModuleFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scope.fixtures.ScopeManagementModuleFixture;
import com.ibm.rpm.client.junit.scorecard.testcases.validation.ScorecardWBSValidationUtility;
import com.ibm.rpm.client.junit.wbs.fixtures.WbsModuleFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.scope.ScopeManagementModuleScope;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.containers.ScorecardQuestion;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.scorecard.scope.ScorecardCategoryScope;
import com.ibm.rpm.scorecard.scope.ScorecardFolderScope;
import com.ibm.rpm.scorecard.scope.ScorecardQuestionScope;
import com.ibm.rpm.scorecard.scope.ScorecardResponseScope;
import com.ibm.rpm.scorecard.scope.ScorecardScope;
import com.ibm.rpm.scorecard.types.ProbabilityType;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.scope.WbsModuleScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author ragoler
 *
 * A fixture to create scorecards and compare them
 */
public class ScorecardFixture extends RPMObjectFixture
{
    // initial size of the graph
    public static final int INITIAL_NB_SCORECARDS = 1;

    public static final int INITIAL_NB_CATEGORIES = 1;

    public static final int INITIAL_NB_QUESTIONS = 1;

    public static final int INITIAL_NB_RESPONSES = 2;

    // additional nodes to the graph
    public static final int NB_SCORECARD = 1;

    public static final int NB_CATEGORY = 1;

    public static final int NB_QUESTION = 1;

    public static final int NB_RESPONSE = 2;

    static
    {
        //Setters
        skipSetters.add("setProbabilityType");
        skipSetters.add("setDefault_");

        //Getters
        skipGetters.add("getProbabilityType");
    }

    public static ScorecardFolder createSaveFixture(AbstractModule module,
            ScorecardFolderScope scope, ClientTestController ctc, int[] size)
            throws Exception
    {
        ScorecardFolder folder = createFixture(scope, size);

        if (module instanceof WbsModule)
        {
            WbsModuleScope moduleScope = WbsModuleFixture.createScope(true, true,
                true, true, true, true, true, true, true, true, true, true, true,
                true, true, true, true);
            
            ((WbsModule)module).setScorecardFolders(new ScorecardFolder[]{folder});
            module = (WbsModule) ctc.save(module, moduleScope).getRpmObject();
        }
        else if (module instanceof ScopeManagementModule)
        {
            ScopeManagementModuleScope moduleScope = ScopeManagementModuleFixture
            .createFullScope();
            
            ((ScopeManagementModule)module).setScorecardFolders(new ScorecardFolder[]{folder});
            module = (ScopeManagementModule) ctc.save(module, moduleScope).getRpmObject();
        }
        else if (module instanceof AssetModule)
        {
            AssetModuleScope moduleScope = AssetModuleFixture.createFullScope();
            
            ((AssetModule)module).setScorecardFolders(new ScorecardFolder[]{folder});
            module = (AssetModule) ctc.save(module, moduleScope).getRpmObject();
        }

        return folder;
    }

    public static ScorecardFolder createSaveFixture(AbstractModule module,
            ScorecardFolderScope scope, ClientTestController ctc) 
            throws Exception
    {
        int[] size = new int[]
        {
                INITIAL_NB_SCORECARDS, INITIAL_NB_CATEGORIES,
                INITIAL_NB_QUESTIONS, INITIAL_NB_RESPONSES
        };

        return createSaveFixture(module, scope, ctc, size);

    }

    public static GenericProject assignScorecardToProject(String sessionID, ClientTestController ctc,
            GenericProject project) throws Exception
    {
        /* Create a proposal and scorecards */
        WorkElementScope scope = new WorkElementScope();
        ScorecardScope scorecardScope = PublishedScorecardFixture
            .createScorecardScope(scope);
        scope.setAssignedScorecards(scorecardScope);
    
        /* Assign the scorecard to the proposal */
        WbsModule module = (new ScorecardWBSValidationUtility()).createModule();
    
        project = (Proposal) ctc.getApplicationInterface()
            .assignScoreCard(sessionID, project, scope,
                module.getScorecardFolders()[0].getScorecards()[0],
                ReloadType.ReloadResult).getRpmObject();
        
        return project;
    }
    
    public static ScorecardFolder createSaveFixture(String contextName, ClientTestController ctc) throws Exception
    {
    	ScorecardFolder scorecardFolder = null;
    	
    	if(WBS_CONTEXT.equalsIgnoreCase(contextName))
    	{
    		WbsModule module = new WbsModule();
    		WbsModuleScope moduleScope = new WbsModuleScope();
    		moduleScope.setScorecardFolders(ScorecardFixture.createScope());
    		module = WbsModuleFixture.createFixture(moduleScope, ctc);
    		
    		module = (WbsModule)save(ctc, module, moduleScope);
    		
    		ClientTestController.assertNotNull(module.getScorecardFolders());
    		ClientTestController.assertTrue(module.getScorecardFolders().length > 0);
    		ClientTestController.assertNotNull(module.getScorecardFolders()[0]);
    		
    		scorecardFolder = module.getScorecardFolders()[0];
    	}
    	else if(SCOPE_CONTEXT.equalsIgnoreCase(contextName))
    	{
    		ScopeManagementModule module = new ScopeManagementModule();
    		ScopeManagementModuleScope moduleScope = new ScopeManagementModuleScope();
    		moduleScope.setScorecardFolders(ScorecardFixture.createScope());
    		module = ScopeManagementModuleFixture.createFixture(moduleScope, ctc);
    		
    		module = (ScopeManagementModule)save(ctc, module, moduleScope);
    		
    		ClientTestController.assertNotNull(module.getScorecardFolders());
    		ClientTestController.assertTrue(module.getScorecardFolders().length > 0);
    		ClientTestController.assertNotNull(module.getScorecardFolders()[0]);
    		
    		scorecardFolder = module.getScorecardFolders()[0];
    	}
    	else if(ASSET_CONTEXT.equalsIgnoreCase(contextName))
    	{
    		AssetModule module = new AssetModule();
    		AssetModuleScope moduleScope = new AssetModuleScope();
    		moduleScope.setScorecardFolders(ScorecardFixture.createScope());
    		module = AssetModuleFixture.createFixture(moduleScope, ctc);
    		
    		module = (AssetModule)save(ctc, module, moduleScope);
    		
    		ClientTestController.assertNotNull(module.getScorecardFolders());
    		ClientTestController.assertTrue(module.getScorecardFolders().length > 0);
    		ClientTestController.assertNotNull(module.getScorecardFolders()[0]);
    		
    		scorecardFolder = module.getScorecardFolders()[0];
    	}
    	return scorecardFolder;
    }
    
    public static ScorecardFolder saveFixture(ScorecardFolder folder,
            ScorecardFolderScope scope, ClientTestController ctc)
            throws Exception
    {
        folder = (ScorecardFolder) save(ctc, folder, scope);

        return folder;
    }

    /**
     * @param contextName
     * @param scope
     * @param size -
     *            an array of 4 dimensions for scorecard, category, question,
     *            response
     * @param ctc
     * @return
     * @throws Exception
     */
    public static ScorecardFolder createFixture(
            ScorecardFolderScope scope, int[] size) throws Exception
    {
        return createFolder(scope, size);
    }

    public static ScorecardFolder createFixture(
            ScorecardFolderScope scope) throws Exception
    {
        int[] size = new int[]
        {
                INITIAL_NB_SCORECARDS, INITIAL_NB_CATEGORIES,
                INITIAL_NB_QUESTIONS, INITIAL_NB_RESPONSES
        };

        return createFixture(scope, size);
    }
    
    public static ScorecardFolder createSimpleFixture()
    {
        ScorecardFolder scorecardFolder = new ScorecardFolder();
        scorecardFolder
            .setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - folder");

        Scorecard scorecard = new Scorecard();
        scorecard.setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - scorecard");
        scorecard.setActive(new Boolean(true));

        ScorecardCategory category = new ScorecardCategory();
        category.setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - category");

        ScorecardQuestion question = new ScorecardQuestion();
        question.setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - question");

        ScorecardResponse response1 = new ScorecardResponse();
        response1.setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - response 1");
        response1.setDefaultResponse(new Boolean(true));

        ScorecardResponse response2 = new ScorecardResponse();
        response2.setName(HelperTestUtilities.unitTestPrefix + "Test WBS scorecards - response 2");

        scorecardFolder.setScorecards(new Scorecard[]
        {
            scorecard
        });
        scorecard.setScorecardCategories(new ScorecardCategory[]
        {
            category
        });
        category.setScorecardQuestions(new ScorecardQuestion[]
        {
            question
        });
        question.setScorecardResponses(new ScorecardResponse[]
        {
                response1, response2
        });
        
        scorecard.setParent(scorecardFolder);

        return scorecardFolder;
    }

    /**
     * @param contextName
     * @param scope
     * @param size -
     *            an array of 4 dimensions for scorecard, category, question,
     *            response
     * @return
     * @throws Exception
     */
    private static ScorecardFolder createFolder(
            ScorecardFolderScope scope, int[] size) throws Exception
    {
        if (size.length != 4)
            throw new IllegalStateException(
                    "size must be of dimension 4 for scorecard, category, question, response");

        ScorecardFolder rpmObject = new ScorecardFolder();
        HelperTestUtilities.setByIntrospection(rpmObject, getNextInt(),
                skipSetters);

        if (scope != null && scope.getScorecards() != null)
        {
            Scorecard[] children = new Scorecard[size[0]];

            for (int i = 0; i < children.length; i++)
            {
                Scorecard child = createScorecard(scope
                        .getScorecards(), size);
                child.setParent(rpmObject);
                children[i] = child;
            }

            rpmObject.setScorecards(children);

        }
        return rpmObject;
    }

    private static Scorecard createScorecard(ScorecardScope scope, 
            int[] size) throws Exception
    {
        int index = getNextInt();
        Scorecard rpmObject = new Scorecard();
        HelperTestUtilities.setByIntrospection(rpmObject, index, skipSetters);
        rpmObject.setProbabilityType(ProbabilityType.CustomerSatisfaction);

        if (scope != null && scope.getScorecardCategories() != null)
        {
            ScorecardCategory[] children = new ScorecardCategory[size[1]];

            for (int i = 0; i < children.length; i++)
            {
                ScorecardCategory child = createCategory(scope
                        .getScorecardCategories(), size);
                child.setParent(rpmObject);
                children[i] = child;
            }

            rpmObject.setScorecardCategories(children);

        }
        return rpmObject;
    }

    private static ScorecardCategory createCategory(
            ScorecardCategoryScope scope, int[] size) throws Exception
    {
        int index = getNextInt();
        ScorecardCategory rpmObject = new ScorecardCategory();
        HelperTestUtilities.setByIntrospection(rpmObject, index, skipSetters);
        rpmObject.setProbabilityType(ProbabilityType.CustomerSatisfaction);

        if (scope != null && scope.getScorecardQuestions() != null)
        {
            ScorecardQuestion[] children = new ScorecardQuestion[size[2]];

            for (int i = 0; i < children.length; i++)
            {
                ScorecardQuestion child = createQuestion(scope
                        .getScorecardQuestions(), size);
                child.setParent(rpmObject);
                children[i] = child;
            }

            rpmObject.setScorecardQuestions(children);

        }
        return rpmObject;
    }

    private static ScorecardQuestion createQuestion(
            ScorecardQuestionScope scope, int[] size) throws Exception
    {
        int index = getNextInt();
        ScorecardQuestion rpmObject = new ScorecardQuestion();
        HelperTestUtilities.setByIntrospection(rpmObject, index, skipSetters);

        if (scope != null && scope.getScorecardResponses() != null)
        {
            ScorecardResponse[] children = new ScorecardResponse[size[3]];

            for (int i = 0; i < children.length; i++)
            {
                ScorecardResponse child = createResponse();
                child.setParent(rpmObject);
                children[i] = child;
            }

            setDefaultResponse(children);

            rpmObject.setScorecardResponses(children);

        }
        return rpmObject;
    }

    private static ScorecardResponse createResponse() throws Exception
    {
        int index = getNextInt();
        ScorecardResponse rpmObject = new ScorecardResponse();
        HelperTestUtilities.setByIntrospection(rpmObject, index, skipSetters);
        rpmObject.setDefaultResponse(new Boolean(false));

        return rpmObject;
    }

    /**
     * @param rpmObject
     * @param scope
     * @return
     * @throws Exception
     */
    public static ScorecardFolder updateFolder(ScorecardFolder rpmObject,
            ScorecardFolderScope scope) throws Exception
    {
        HelperTestUtilities.setByIntrospection(rpmObject, getNextInt(),
                skipSetters);

        if (scope != null && scope.getScorecards() != null)
        {
            Scorecard[] children = rpmObject.getScorecards();

            if (children != null)
            {

                for (int i = 0; i < children.length; i++)
                {
                    updateScorecard(children[i], scope
                            .getScorecards());
                }
            }

        }
        return rpmObject;
    }

    private static Scorecard updateScorecard(Scorecard rpmObject,
            ScorecardScope scope) throws Exception
    {
        int index = getNextInt();
        HelperTestUtilities.setByIntrospection(rpmObject, index, skipSetters);
        rpmObject.setProbabilityType(ProbabilityType.ProjectCosts);

        if (scope != null && scope.getScorecardCategories() != null)
        {
            ScorecardCategory[] children = rpmObject.getScorecardCategories();

            if (children != null)
            {
                for (int i = 0; i < children.length; i++)
                {
                    updateCategory(children[i], scope
                            .getScorecardCategories());
                }
            }
        }
        return rpmObject;
    }

    private static ScorecardCategory updateCategory(
            ScorecardCategory rpmObject, ScorecardCategoryScope scope)
            throws Exception
    {
        int index = getNextInt();
        HelperTestUtilities.setByIntrospection(rpmObject, index, skipSetters);
        rpmObject.setProbabilityType(ProbabilityType.ProjectBenefits);

        if (scope != null && scope.getScorecardQuestions() != null)
        {
            ScorecardQuestion[] children = rpmObject.getScorecardQuestions();

            if (children != null)
            {

                for (int i = 0; i < children.length; i++)
                {
                    updateQuestion(children[i], scope
                            .getScorecardQuestions());
                }
            }
        }
        return rpmObject;
    }

    private static ScorecardQuestion updateQuestion(
            ScorecardQuestion rpmObject, ScorecardQuestionScope scope)
            throws Exception
    {
        int index = getNextInt();
        HelperTestUtilities.setByIntrospection(rpmObject, index, skipSetters);

        if (scope != null && scope.getScorecardResponses() != null)
        {
            ScorecardResponse[] children = rpmObject.getScorecardResponses();

            if (children != null)
            {

                for (int i = 0; i < children.length; i++)
                {
                    updateResponse(children[i], scope
                            .getScorecardResponses());
                }
            }
            setDefaultResponse(children);

        }
        return rpmObject;
    }

    private static ScorecardResponse updateResponse(
            ScorecardResponse rpmObject, ScorecardResponseScope scope)
            throws Exception
    {
        int index = getNextInt();
        HelperTestUtilities.setByIntrospection(rpmObject, index, skipSetters);

        return rpmObject;
    }

    public static ScorecardFolder updateFolderStructure(
            ScorecardFolder rpmObject, ScorecardFolderScope scope)
            throws Exception
    {
        int[] size = new int[]
        {
                ScorecardFixture.NB_SCORECARD, ScorecardFixture.NB_CATEGORY,
                ScorecardFixture.NB_QUESTION, ScorecardFixture.NB_RESPONSE
        };

        return updateFolderStructure(rpmObject, scope, size);
    }

    /**
     * @param rpmObject
     * @param scope
     * @return
     * @throws Exception
     */
    public static ScorecardFolder updateFolderStructure(
            ScorecardFolder rpmObject, ScorecardFolderScope scope,
            int[] nbChildren) throws Exception
    {
        if (scope != null && scope.getScorecards() != null)
        {
            Scorecard[] children = rpmObject.getScorecards();

            if (nbChildren != null && nbChildren[0] > 0)
            {
                List list = new LinkedList();
                for (int i = 0; i < nbChildren[0]; i++)
                {

                    Scorecard temp = createScorecard(
                            scope.getScorecards(), nbChildren);
                    temp.setParent(rpmObject);
                    list.add(temp);
                }
                list.addAll(Arrays.asList(children));
                children = (Scorecard[]) list.toArray(new Scorecard[0]);
                rpmObject.setScorecards(children);
            }

            if (children != null)
            {

                for (int i = 0; i < children.length; i++)
                {
                    updateScorecardStructure(children[i],
                            scope.getScorecards(), nbChildren);
                }
            }

        }
        return rpmObject;
    }

    private static Scorecard updateScorecardStructure(Scorecard rpmObject,
            ScorecardScope scope, int[] nbChildren) throws Exception
    {
        if (scope != null && scope.getScorecardCategories() != null)
        {
            ScorecardCategory[] children = rpmObject.getScorecardCategories();

            if (nbChildren != null && nbChildren[1] > 0)
            {
                List list = new LinkedList();
                for (int i = 0; i < nbChildren[1]; i++)
                {
                    ScorecardCategory temp = createCategory(
                        scope.getScorecardCategories(), nbChildren);
                    temp.setParent(rpmObject);
                    list.add(temp);

                }
                list.addAll(Arrays.asList(children));
                children = (ScorecardCategory[]) list
                        .toArray(new ScorecardCategory[0]);
                rpmObject.setScorecardCategories(children);
            }

            if (children != null)
            {
                for (int i = 0; i < children.length; i++)
                {
                    updateCategoryStructure(
                            children[i], scope.getScorecardCategories(), nbChildren);
                }
            }
        }
        return rpmObject;
    }

    private static ScorecardCategory updateCategoryStructure(
            ScorecardCategory rpmObject, ScorecardCategoryScope scope,
            int[] nbChildren) throws Exception
    {
        if (scope != null && scope.getScorecardQuestions() != null)
        {
            ScorecardQuestion[] children = rpmObject.getScorecardQuestions();

            if (nbChildren != null && nbChildren[2] > 0)
            {
                List list = new LinkedList();
                for (int i = 0; i < nbChildren[2]; i++)
                {

                    ScorecardQuestion temp = createQuestion(
                        scope.getScorecardQuestions(), nbChildren);
                    temp.setParent(rpmObject);
                    list.add(temp);
                }
                list.addAll(Arrays.asList(children));
                children = (ScorecardQuestion[]) list
                        .toArray(new ScorecardQuestion[0]);
                rpmObject.setScorecardQuestions(children);
            }

            if (children != null)
            {

                for (int i = 0; i < children.length; i++)
                {
                    updateQuestionStructure(
                            children[i], scope.getScorecardQuestions(), nbChildren);
                }
            }
        }
        return rpmObject;
    }

    private static ScorecardQuestion updateQuestionStructure(
            ScorecardQuestion rpmObject, ScorecardQuestionScope scope,
            int[] nbChildren) throws Exception
    {
        if (scope != null && scope.getScorecardResponses() != null)
        {
            ScorecardResponse[] children = rpmObject.getScorecardResponses();

            if (nbChildren != null && nbChildren[3] > 0)
            {
                List list = new LinkedList();
                for (int i = 0; i < nbChildren[3]; i++)
                {
                    ScorecardResponse temp = createResponse();
                    temp.setParent(rpmObject);
                    list.add(temp);
                }
                list.addAll(Arrays.asList(children));
                children = (ScorecardResponse[]) list
                        .toArray(new ScorecardResponse[0]);
                rpmObject.setScorecardResponses(children);
            }

            if (children != null)
            {
                setDefaultResponse(children);
            }

        }
        return rpmObject;
    }

    private static void setDefaultResponse(ScorecardResponse[] children)
    {
        // set one child as default
        int defaultIndex;
        if (children.length > 1)
            defaultIndex = getNextInt() % children.length;
        else
            defaultIndex = 0;

        for (int i = 0; i < children.length; i++)
        {
            if (i == defaultIndex)
                children[i].setDefaultResponse(new Boolean(true));
            else
                children[i].setDefaultResponse(new Boolean(false));
        }

    }

    //Compare section
    public static void compare(RPMObject[] array1, RPMObject[] array2,
            AbstractScorecardScope scope) throws AssertionFailedError,
            Exception
    {

        if (array1 == null && array2 == null)
            return;

        if ((array1 == null && array2 != null)
                || (array1 != null && array2 == null))
            Assert.assertFalse("Both must be null or not null together", true);

        sortWithComparator(array1, RPMObjectIDComparator.INSTANCE);
        sortWithComparator(array2, RPMObjectIDComparator.INSTANCE);

        RPMObjectFixture.compare(array1, array2);

        for (int i = 0; i < array1.length; i++)
        {
            compare(array1[i], array2[i], scope);
        }
    }

    public static void compare(RPMObject obj1, RPMObject obj2,
            AbstractScorecardScope scope) throws AssertionFailedError,
            Exception
    {

        RPMObjectFixture.compare(obj1, obj2);

        if (scope != null && scope.getParent() != null)
        {
            if ((obj1.getParent() == null && obj2.getParent() != null)
                    || (obj1.getParent() != null && obj2.getParent() == null))
                Assert.assertTrue("one of the parents is null", false);

            Assert.assertEquals(obj1.getParent().getClass(), obj2.getParent()
                    .getClass());
            Assert.assertEquals(obj1.getParent().getID(), obj2.getParent()
                    .getID());
        }

        if (obj1 instanceof ScorecardFolder)
        {
            compareFolder((ScorecardFolder) obj1, (ScorecardFolder) obj2,
                    (ScorecardFolderScope) scope);
        }
        else if (obj1 instanceof Scorecard)
        {
            compareScorecard((Scorecard) obj1, (Scorecard) obj2,
                    (ScorecardScope) scope);
        }
        else if (obj1 instanceof ScorecardCategory)
        {
            compareCategory((ScorecardCategory) obj1, (ScorecardCategory) obj2,
                    (ScorecardCategoryScope) scope);
        }
        else if (obj1 instanceof ScorecardQuestion)
        {
            compareQuestion((ScorecardQuestion) obj1, (ScorecardQuestion) obj2,
                    (ScorecardQuestionScope) scope);
        }
        else if (obj1 instanceof ScorecardResponse)
        {
        }
        else
        {
            Assert.fail("AttributeFixture.compare : class "
                    + obj1.getClass().getName() + " not supported.");
        }

    }

    private static void compareFolder(ScorecardFolder obj1,
            ScorecardFolder obj2, ScorecardFolderScope scope)
            throws AssertionFailedError, Exception
    {

        //compare children
        if (scope != null && scope.getScorecards() != null)
        {
            compare(obj1.getScorecards(), obj2.getScorecards(), scope
                    .getScorecards());
        }

    }

    private static void compareScorecard(Scorecard obj1, Scorecard obj2,
            ScorecardScope scope) throws AssertionFailedError, Exception
    {

        //compare children
        if (scope != null && scope.getScorecardCategories() != null)
        {
            compare(obj1.getScorecardCategories(), obj2
                    .getScorecardCategories(), scope.getScorecardCategories());
        }

        Assert.assertEquals(obj1.getProbabilityType(), obj2
                .getProbabilityType());

    }

    private static void compareCategory(ScorecardCategory obj1,
            ScorecardCategory obj2, ScorecardCategoryScope scope)
            throws AssertionFailedError, Exception
    {

        //compare children
        if (scope != null && scope.getScorecardQuestions() != null)
        {
            compare(obj1.getScorecardQuestions(), obj2.getScorecardQuestions(),
                    scope.getScorecardQuestions());
        }

        Assert.assertEquals(obj1.getProbabilityType(), obj2
                .getProbabilityType());
    }

    private static void compareQuestion(ScorecardQuestion obj1,
            ScorecardQuestion obj2, ScorecardQuestionScope scope)
            throws AssertionFailedError, Exception
    {

        //compare children
        if (scope != null && scope.getScorecardResponses() != null)
        {
            compare(obj1.getScorecardResponses(), obj2.getScorecardResponses(),
                    scope.getScorecardResponses());
        }

    }

    public static ScorecardFolderScope createScope()
    {
        ScorecardResponseScope rScope = new ScorecardResponseScope();
        ScorecardQuestionScope qScope = new ScorecardQuestionScope();
        ScorecardCategoryScope cScope = new ScorecardCategoryScope();
        ScorecardScope sScope = new ScorecardScope();
        ScorecardFolderScope fScope = new ScorecardFolderScope();

        fScope.setScorecards(sScope);
        sScope.setScorecardCategories(cScope);
        cScope.setScorecardQuestions(qScope);
        qScope.setScorecardResponses(rScope);

        return fScope;
    }


    private static int INDEX = 1;

    private static int getNextInt()
    {
        return INDEX++;
    }

}