package com.ibm.rpm.client.junit.scorecard.testcases;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.scope.ScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestScorecardAssignments extends ClientTestController {
	
	ScorecardFolder scorecardFolder = null;
	ScorecardScope scoreCardScope = null;
	
	protected void setUp() throws Exception
	{
		super.setUp();
		scoreCardScope = PublishedScorecardFixture.createScorecardScope(null);
	}
	
	public void testLoadScorecardAssignmentWithParentForWbs() throws Exception
	{
		WorkElementScope workElementScope = new WorkElementScope();
		workElementScope.setAssignedScorecards(scoreCardScope);
		loadScorecardAssignmentWithParent(RPMObjectFixture.WBS_CONTEXT, new Proposal(), workElementScope);
	}
	
	public void testLoadScorecardAssignmentWithParentForScope() throws Exception
	{
		ScopeElementScope scopeElementScope = new ScopeElementScope();
		scopeElementScope.setAssignedScorecard(scoreCardScope);
		loadScorecardAssignmentWithParent(RPMObjectFixture.SCOPE_CONTEXT, new Requirement(), scopeElementScope);
	}
	
	public void testLoadScorecardAssignmentWithParentForAsset() throws Exception
	{
		AssetScope assetScope = new AssetScope();
		assetScope.setAssignedScorecard(scoreCardScope);
		loadScorecardAssignmentWithParent(RPMObjectFixture.ASSET_CONTEXT, new Asset(), assetScope);
	}
	
	private void loadScorecardAssignmentWithParent(String contextName, RPMObject parent, RPMObjectScope rpmObjectScope) throws Exception
	{
		scorecardFolder = ScorecardFixture.createSaveFixture(contextName, this);
		parent = PublishedScorecardFixture.createParent(parent, null, this);
		parent = save(parent,null).getRpmObject();
		AssignedScorecard scorecardAssignment = assignSaveScorecard(parent,rpmObjectScope);
		loadAndValidateScorecardAssignment(scorecardAssignment, parent);
	}
	
	private AssignedScorecard assignSaveScorecard(RPMObject parent,RPMObjectScope rpmObjectScope) throws Exception
	{
		parent = PublishedScorecardFixture.createAssignmentFixture(parent, rpmObjectScope, this,
				ReloadType.ReloadResult, scorecardFolder, 1);
		
		AssignedScorecard scorecardassignment = null;
		if(parent instanceof GenericProject)
		{
			scorecardassignment = ((GenericProject)parent).getAssignedScorecards()[0];
		}
        else if(parent instanceof AbstractScope)
		{
			scorecardassignment = ((AbstractScope)parent).getAssignedScorecard();
		}
        else if(parent instanceof Asset)
		{
			scorecardassignment = ((Asset)parent).getAssignedScorecard();
		}
        else
        {
            throw new Exception("Unsupported parent of type "+parent.getClass());
        }
		return scorecardassignment;
	}
	
	private void loadAndValidateScorecardAssignment(AssignedScorecard scorecardAssignment, RPMObject parent) throws Exception
	{
		ScorecardScope scorecardScope = new ScorecardScope();
		scorecardScope.setParent(new RPMObjectScope());
		scorecardAssignment = (AssignedScorecard)loadFromID(scorecardAssignment, scorecardScope);
		
		assertNotNull(scorecardAssignment);
		assertNotNull(scorecardAssignment.getParent());
		assertEquals(parent.getID(), scorecardAssignment.getParent().getID());
	}

}
