/*
 * Created on Jan 25, 2006
 * @author mmatwiy
 * Validation tests for container AssignedScorecardState for all contexts.
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AssignedScorecardState;


public abstract class AbstractTestAssignedScorecardStateValidation extends
        GenericValidation
{
    private AssignedScorecardState scorecardState;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void initializeValues(AssignedScorecardState state)
    {
        scorecardState = state;
    }

    protected void publishedNotSet() throws AssertionError, Exception
    {
        // Set pusblished field value and verify that it is not accepted
        scorecardState.setPublished(new Boolean(true));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(com.ibm.rpm.scorecard.constants.ErrorCodes.PUBLISHED_CAN_NOT_BE_SET,
                scorecardState.getClass().getName())
        };

        verifySaveMustFail(scorecardState, null, expectedExceptions);
    }

    protected void defaultValuesInsert() throws AssertionError, Exception
    {
        // Check that the default values are set
        assertNotNull(scorecardState.getPublished());
        assertNotNull(scorecardState.getActive());

        // Save again to validate the default values
        scorecardState = (AssignedScorecardState) saveMustSucceed(
            scorecardState, AssignedScorecardState.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/AssignedScorecardState[id='"
                + scorecardState.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof AssignedScorecardState);

        ScorecardAssignmentFixture.compare(scorecardState,
            (AssignedScorecardState) rpmObjects[0]);

    }

    protected void defaultValuesUpdateToNull() throws AssertionError, Exception
    {
        // Update the default values to null and verify validation errors
        scorecardState.setActive(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, scorecardState
                .getClass().getName(), ValidationConstants.ACTIVE_FIELD,
                scorecardState.getID())
        };

        verifySaveMustFail(scorecardState, null, expectedExceptions);

    }
}
