/*
 * Created on Dec 6, 2005 @author mmatwiy Validation tests for container
 * ScorecardCategory.
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;

public abstract class AbstractTestScorecardCategoryValidation extends
        GenericValidation
{
    private ScorecardCategory category;

    private AbstractModule module;

    private RPMObjectScope scope;

    public void setUp(AbstractModule abstractModule,
            RPMObjectScope moduleScope, ScorecardCategory scorecardCategory)
            throws Exception
    {
        super.setUp();

        module = abstractModule;
        scope = moduleScope;
        category = scorecardCategory;
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    protected void insertWithoutModule() throws AssertionError, Exception
    {
        // Set mandatory field
        category
            .setName(HelperTestUtilities.createName());

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MODULE_REQUIRED, category
                    .getClass().getName()),
                createRPMException(ErrorCodes.NO_PARENT, category.getClass()
                    .getName(), ValidationConstants.PARENT, category.getID())
        };

        verifySaveMustFail(category, null, expectedExceptions);
    }

    protected void insertWithoutMandatoryFields() throws AssertionError,
            Exception
    {

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, category.getClass()
                .getName(), ValidationConstants.NAME_FIELD, category.getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void insertSuccess() throws AssertionError, Exception
    {
        category
            .setName(HelperTestUtilities.createName());
        category
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardCategory[id='"
                + category.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardCategory);

        ScorecardFixture.compare(category, rpmObjects[0], null);

    }

    protected void successValidationMax() throws AssertionError, Exception
    {
        // Set field values with max values and verify that they are accepted
        category
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));
        category
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT));
        category.setWeight(new Integer(ValidationConstants.WEIGHT_MAX));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardCategory[id='"
                + category.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardCategory);

        ScorecardFixture.compare(category, rpmObjects[0], null);
    }

    protected void successValidationMin() throws AssertionError, Exception
    {
        // Set mandatory fields
        category
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        // Set field values with min values and verify that they are accepted
        category.setWeight(new Integer(ValidationConstants.WEIGHT_MIN));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardCategory[id='"
                + category.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardCategory);

        ScorecardFixture.compare(category, rpmObjects[0], null);
    }

    protected void failureValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength + 1 and verify that they are not
        // accepted
        category
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT + 1));
        category
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT + 1));
        category.setWeight(new Integer(ValidationConstants.WEIGHT_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, category.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, category
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, category.getClass()
                    .getName(), ValidationConstants.DESCRIPTION_FIELD, category
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, category.getClass()
                    .getName(), ValidationConstants.WEIGHT_FIELD, category
                    .getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void failureValidationMin() throws AssertionError, Exception
    {
        // Set field values with minimum value - 1 and verify that they are not
        // accepted
        category.setWeight(new Integer(ValidationConstants.WEIGHT_MIN - 1));
        category.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, category.getClass()
                    .getName(), ValidationConstants.WEIGHT_FIELD, category
                    .getID()),
                createRPMException(ErrorCodes.MIN_LENGTH, category.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, category
                    .getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void defaultValuesInsert() throws AssertionError, Exception
    {
        category
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardCategory[id='"
                + category.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardCategory);

        // Check that the default values are now set
        ScorecardCategory savedCategory = (ScorecardCategory) rpmObjects[0];
        assertNotNull(savedCategory.getProbabilityType());
        assertNotNull(savedCategory.getWeight());

        // Save again to validate the default values
        savedCategory = (ScorecardCategory) saveMustSucceed(savedCategory,
            ScorecardCategory.class, null);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/ScorecardCategory[id='" + category.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardCategory);

        ScorecardFixture.compare(category, rpmObjects[0], null);
    }

    protected void defaultValuesUpdateToNull() throws AssertionError, Exception
    {
        category
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardCategory[id='"
                + category.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardCategory);

        // Check that the default values are now set
        ScorecardCategory savedCategory = (ScorecardCategory) rpmObjects[0];
        assertNotNull(category.getWeight());
        assertNotNull(category.getProbabilityType());

        // Update the default values to null and verify validation errors
        savedCategory.setWeight(null);
        savedCategory.setProbabilityType(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedCategory
                    .getClass().getName(), ValidationConstants.WEIGHT_FIELD,
                    savedCategory.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedCategory
                    .getClass().getName(),
                    ValidationConstants.PROBABILITY_TYPE_FIELD, savedCategory
                        .getID())
        };

        verifySaveMustFail(savedCategory, null, expectedExceptions);

    }

}
