/*
 * Created on Dec 6, 2005 @author mmatwiy Validation tests for container
 * ScorecardFolder.
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;

public abstract class AbstractTestScorecardFolderValidation extends
        GenericValidation
{
    private AbstractModule module;

    private ScorecardFolder folder;

    private RPMObjectScope scope;

    public void setUp(AbstractModule abstractModule,
            RPMObjectScope moduleScope, ScorecardFolder scorecardFolder)
            throws Exception
    {
        super.setUp();

        module = abstractModule;
        scope = moduleScope;
        folder = scorecardFolder;

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    protected void insertWithoutModule() throws AssertionError, Exception
    {
        // Set mandatory field
        folder.setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MODULE_REQUIRED, folder
                    .getClass().getName()),
                createRPMException(ErrorCodes.NO_PARENT, folder.getClass()
                    .getName(), ValidationConstants.PARENT, folder.getID())

        };

        verifySaveMustFail(folder, null, expectedExceptions);
    }

    protected void insertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        folder.setName(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, folder.getClass()
                .getName(), ValidationConstants.NAME_FIELD, folder.getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void insertSuccess() throws AssertionError, Exception
    {
        folder.setName(HelperTestUtilities.createName());
        folder
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardFolder[id='"
                + folder.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardFolder);

        ScorecardFixture.compare(folder, rpmObjects[0], null);
    }

    protected void successValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        folder.setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));
        folder
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardFolder[id='"
                + folder.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardFolder);

        ScorecardFixture.compare(folder, rpmObjects[0], null);
    }

    protected void failureValidationMax() throws AssertionError, Exception
    {
        // Set field values with max value + 1 and verify that they are not
        // accepted
        folder
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT + 1));
        folder
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, folder.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, folder.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, folder.getClass()
                    .getName(), ValidationConstants.DESCRIPTION_FIELD, folder
                    .getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void failureValidationMin() throws AssertionError, Exception
    {
        // Set field values with min value - 1 and verify that they are not
        // accepted
        folder.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MIN_LENGTH, folder.getClass()
                .getName(), ValidationConstants.NAME_FIELD, folder.getID())

        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }
}