/*
 * Created on Dec 7, 2005 @author mmatwiy Validation tests for container
 * ScorecardResponse.
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;

public abstract class AbstractTestScorecardResponseValidation extends
        GenericValidation
{
    private ScorecardResponse response;

    private AbstractModule module;

    private RPMObjectScope scope;

    public void setUp(AbstractModule abstractModule,
            RPMObjectScope moduleScope, ScorecardResponse scorecardResponse)
            throws Exception
    {
        super.setUp();

        module = abstractModule;
        scope = moduleScope;
        response = scorecardResponse;
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    protected void insertWithoutModule() throws AssertionError, Exception
    {
        // Set mandatory field
        response
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NO_PARENT, response.getClass()
                    .getName(), ValidationConstants.PARENT, response.getID()),
                createRPMException(ErrorCodes.MODULE_REQUIRED, response
                    .getClass().getName())
        };

        verifySaveMustFail(response, null, expectedExceptions);
    }

    protected void insertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, response.getClass()
                .getName(), ValidationConstants.NAME_FIELD, response.getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void insertSuccess() throws AssertionError, Exception
    {
        response.setName(HelperTestUtilities.createName());
        response
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardResponse[id='"
                + response.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardResponse);

        // Check that the loaded object is identical to the one we
        // requested to store
        ScorecardFixture.compare(response, rpmObjects[0], null);
    }

    protected void successValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        response
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));
        response
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT));
        response.setWeight(new Integer(ValidationConstants.WEIGHT_MAX));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardResponse[id='"
                + response.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardResponse);

        // Check that the loaded object is identical to the one we
        // requested to store
        ScorecardFixture.compare(response, rpmObjects[0], null);
    }

    protected void failureValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength + 1 and verify that they are not
        // accepted
        response
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT + 1));
        response
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_CHARACTER_LIMIT + 1));
        response.setWeight(new Integer(ValidationConstants.WEIGHT_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, response.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, response
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, response.getClass()
                    .getName(), ValidationConstants.DESCRIPTION_FIELD, response
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, response.getClass()
                    .getName(), ValidationConstants.WEIGHT_FIELD, response
                    .getID())
        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void successValidationMin() throws AssertionError, Exception
    {
        // Set mandatory fields
        response
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        // Set field values with min value and verify that they are accepted
        response
            .setWeight(new Integer(ValidationConstants.WEIGHT_RESPONSE_MIN));

        // Attempt to save the object
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardResponse[id='"
                + response.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardResponse);

        // Check that the loaded object is identical to the one we
        // requested to store
        ScorecardFixture.compare(response, rpmObjects[0], null);
    }

    protected void failureValidationMin() throws AssertionError, Exception
    {
        // Set field values with min value - 1 and verify that they are not
        // accepted
        response.setName("");
        response.setWeight(new Integer(
            ValidationConstants.WEIGHT_RESPONSE_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, response.getClass()
                    .getName(), ValidationConstants.WEIGHT_FIELD, response
                    .getID()),
                createRPMException(ErrorCodes.MIN_LENGTH, response.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, response
                    .getID())

        };

        verifySaveMustFail(module, scope, expectedExceptions);
    }

    protected void defaultValuesInsert() throws AssertionError, Exception
    {
        response
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardResponse[id='"
                + response.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardResponse);

        // Check that the default values are now set
        ScorecardResponse savedResponse = (ScorecardResponse) rpmObjects[0];
        assertNotNull(savedResponse.getDefaultResponse());
        assertNotNull(savedResponse.getWeight());

        // Save again to validate the default values
        savedResponse = (ScorecardResponse) saveMustSucceed(savedResponse,
            ScorecardResponse.class, null);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/ScorecardResponse[id='" + response.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardResponse);

        ScorecardFixture.compare(response, rpmObjects[0], null);
    }

    protected void defaultValuesUpdateToNull() throws AssertionError, Exception
    {
        response
            .setName(getTestString(ValidationConstants.NAME_CHARACTER_LIMIT));

        // Leave default values as null and save
        module = (AbstractModule) saveMustSucceed(module, AbstractModule.class,
            scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ScorecardResponse[id='"
                + response.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScorecardResponse);

        // Check that the default values are now set
        ScorecardResponse savedResponse = (ScorecardResponse) rpmObjects[0];
        assertNotNull(response.getWeight());
        assertNotNull(response.getDefaultResponse());

        // Update the default values to null and verify validation errors
        savedResponse.setWeight(null);
        savedResponse.setDefaultResponse(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedResponse
                    .getClass().getName(), ValidationConstants.WEIGHT_FIELD,
                    savedResponse.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedResponse
                    .getClass().getName(),
                    ValidationConstants.DEFAULT_RESPONSE_FIELD, savedResponse
                        .getID())
        };

        verifySaveMustFail(savedResponse, null, expectedExceptions);

    }
}
