/*
 * Created on Jan 27, 2006
 * @author mmatwiy
 * Utility methods for use with ScopeManagement scorecard validation.
 */
package com.ibm.rpm.client.junit.scorecard.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scope.fixtures.ScopeManagementModuleFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scopemanagement.scope.ScopeManagementModuleScope;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.containers.ScorecardQuestion;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class ScorecardScopeValidationUtility extends GenericValidation
{
    public ScopeFolder createScopeFolder(ScopeElementScope scope)
            throws Exception
    {
        scope.setParent(new ScopeElementScope());

        Proposal proposal = (Proposal) WorkElementFixture.createFixture(
            new Proposal(), null, this);
        save(proposal, null);
        proposal = (Proposal) loadFromID(proposal, new WorkElementScope());

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", new ScopeElementScope());
        ScopeFolder scopeFolder = (ScopeFolder) rpmObjects[0];

        return scopeFolder;
    }

    public Action prepareActionAssignment(ScopeManagementModule module)
            throws Exception
    {
        ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];

        ScopeElementScope scope = new ScopeElementScope();
        ScopeFolder scopeFolder = createScopeFolder(scope);

        scope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(scope));

        Action action = ScopeElementFixture.createActionFixture(scope, 0,
            scopeFolder, this);
        save(action, scope);

        Action assignedAction = (Action) PublishedScorecardFixture
            .createAssignmentFixture(action, scope, this,
                ReloadType.ReloadResult, scorecardFolder, 1);

        return assignedAction;
    }

    public ChangeRequest prepareChangeRequestAssignment(
            ScopeManagementModule module) throws Exception
    {
        ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];

        ScopeElementScope scope = new ScopeElementScope();
        ScopeFolder scopeFolder = createScopeFolder(scope);

        scope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(scope));

        ChangeRequest changeRequest = ScopeElementFixture
            .createChangeRequestFixture(scope, 0, scopeFolder, this);
        save(changeRequest, scope);

        ChangeRequest assignedChangeRequest = (ChangeRequest) PublishedScorecardFixture
            .createAssignmentFixture(changeRequest, scope, this,
                ReloadType.ReloadResult, scorecardFolder, 1);

        return assignedChangeRequest;

    }

    public Defect prepareDefectAssignment(ScopeManagementModule module)
            throws Exception
    {
        ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];

        ScopeElementScope scope = new ScopeElementScope();
        ScopeFolder scopeFolder = createScopeFolder(scope);

        scope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(scope));

        Defect defect = ScopeElementFixture.createDefectFixture(scope, 0,
            scopeFolder, this);
        save(defect, scope);

        Defect assignedDefect = (Defect) PublishedScorecardFixture
            .createAssignmentFixture(defect, scope, this,
                ReloadType.ReloadResult, scorecardFolder, 1);

        return assignedDefect;

    }

    public Issue prepareIssueAssignment(ScopeManagementModule module)
            throws AssertionError, Exception
    {
        ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];

        ScopeElementScope scope = new ScopeElementScope();
        ScopeFolder scopeFolder = createScopeFolder(scope);

        scope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(scope));

        Issue issue = ScopeElementFixture.createIssueFixture(scope, 0,
            scopeFolder, this);
        save(issue, scope);

        Issue assignedIssue = (Issue) PublishedScorecardFixture
            .createAssignmentFixture(issue, scope, this,
                ReloadType.ReloadResult, scorecardFolder, 1);

        return assignedIssue;

    }

    public Requirement prepareRequirementAssignment(ScopeManagementModule module)
            throws AssertionError, Exception
    {
        ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];

        ScopeElementScope scope = new ScopeElementScope();
        ScopeFolder scopeFolder = createScopeFolder(scope);

        scope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(scope));

        Requirement requirement = ScopeElementFixture.createRequirementFixture(
            scope, 0, scopeFolder, this);
        save(requirement, scope);

        Requirement assignedRequirement = (Requirement) PublishedScorecardFixture
            .createAssignmentFixture(requirement, scope, this,
                ReloadType.ReloadResult, scorecardFolder, 1);

        return assignedRequirement;
    }

    public Risk prepareRiskAssignment(ScopeManagementModule module)
            throws AssertionError, Exception
    {
        ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];

        ScopeElementScope scope = new ScopeElementScope();
        ScopeFolder scopeFolder = createScopeFolder(scope);

        scope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(scope));

        Risk risk = ScopeElementFixture.createRiskFixture(scope, 0,
            scopeFolder, this);
        save(risk, scope);

        Risk assignedRisk = (Risk) PublishedScorecardFixture
            .createAssignmentFixture(risk, scope, this,
                ReloadType.ReloadResult, scorecardFolder, 1);

        return assignedRisk;

    }

    public ServiceRequest prepareServiceRequestAssignment(
            ScopeManagementModule module) throws AssertionError, Exception
    {
        ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];

        ScopeElementScope scope = new ScopeElementScope();
        ScopeFolder scopeFolder = createScopeFolder(scope);

        scope.setAssignedScorecard(PublishedScorecardFixture
            .createScorecardScope(scope));

        ServiceRequest serviceRequest = ScopeElementFixture
            .createServiceRequestFixture(scope, 0, scopeFolder, this);
        save(serviceRequest, scope);

        ServiceRequest assignedServiceRequest = (ServiceRequest) PublishedScorecardFixture
            .createAssignmentFixture(serviceRequest, scope, this,
                ReloadType.ReloadResult, scorecardFolder, 1);

        return assignedServiceRequest;

    }

    public ScopeManagementModule createModule()
            throws Exception
    {
        ScopeManagementModule module = new ScopeManagementModule();

        ScorecardFolder scorecardFolder = new ScorecardFolder();
        scorecardFolder
            .setName(HelperTestUtilities.unitTestPrefix + "Test scope management scorecards - folder");

        Scorecard scorecard = new Scorecard();
        scorecard
            .setName(HelperTestUtilities.unitTestPrefix + "Test scope management scorecards - scorecard");
        scorecard.setActive(new Boolean(true));

        ScorecardCategory category = new ScorecardCategory();
        category
            .setName(HelperTestUtilities.unitTestPrefix + "Test scope management scorecards  - category");

        ScorecardQuestion question = new ScorecardQuestion();
        question
            .setName(HelperTestUtilities.unitTestPrefix + "Test scope management scorecards  - question");

        ScorecardResponse response1 = new ScorecardResponse();
        response1
            .setName(HelperTestUtilities.unitTestPrefix + "Test scope management scorecards  - response 1");
        response1.setDefaultResponse(new Boolean(true));

        ScorecardResponse response2 = new ScorecardResponse();
        response2
            .setName(HelperTestUtilities.unitTestPrefix + "Test scope management scorecards - response 2");

        module.setScorecardFolders(new ScorecardFolder[]
        {
            scorecardFolder
        });
        scorecardFolder.setScorecards(new Scorecard[]
        {
            scorecard
        });
        scorecard.setScorecardCategories(new ScorecardCategory[]
        {
            category
        });
        category.setScorecardQuestions(new ScorecardQuestion[]
        {
            question
        });
        question.setScorecardResponses(new ScorecardResponse[]
        {
                response1, response2
        });

        ScopeManagementModuleScope moduleScope = ScopeManagementModuleFixture
            .createFullScope();

        module = (ScopeManagementModule) save(module, moduleScope)
            .getRpmObject();

        return module;
    }
}
