/*
 * Created on Nov 8, 2006
 *
 */
package com.ibm.rpm.client.junit.security.testcases;

import java.util.Iterator;
import java.util.Map;

import junit.framework.Assert;

import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.containers.SecurityRoleCategory;
import com.ibm.rpm.security.managers.SecurityUtil;
import com.ibm.rpm.security.managers.SecurityUtil.BaseFieldDescr;
import com.ibm.rpm.security.scope.SecurityRoleScope;

/**
 * @author Karim & Marc-Andr
 */
public class TestRoleSecurityFlagsLocal extends BaseRoleSecurityFlagsTestLocal
{

    public void testAllSecurityRoleFlags() throws Exception
    {
        // System.out.println("TEST - Starting testAllSecurityRoleFlags");
        boolean flagValue = true;
        SecurityRoleScope scope = new SecurityRoleScope();
        scope.setSecurityRoles(scope);
        RPMObject[] rpmObjects = loadFromXpath("/SecurityRoleCategory", scope);
        assertNotNull(rpmObjects);
        for (int i = 0; i < rpmObjects.length; i++)
        {
            SecurityRole[] parentSecurityRoles = ((SecurityRoleCategory) rpmObjects[i])
                .getSecurityRoles();
            Assert.assertNotNull(parentSecurityRoles);
            for (int j = 0; j < parentSecurityRoles.length; j++)
            {
                SecurityRole securityRole = prepareSecurityRole(
                    parentSecurityRoles[j], flagValue);
                Assert.assertNotNull(securityRole);
                verifyRoleFlags(securityRole, flagValue);
            }
        }
        // TODO repeat the exercise with flagValue = true
    }

    /**
     * Tests the database's security flags for singularity. Loads all the flag
     * and sets them to false. Then set one flags to true and check if the
     * others have been affected.
     * 
     * @author Marc-Andr
     */
//    public void testAllSecurityRoleFlagsSingularity() throws Exception
//    {
//        //        System.out.println("TEST - Starting testAllSecurityRoleFlagsSingularity");
//        SecurityRoleScope scope = new SecurityRoleScope();
//        scope.setSecurityRoles(scope);
//        RPMObject[] rpmObjects = loadFromXpath("/SecurityRoleCategory", scope);
//        assertNotNull(rpmObjects);
//
//        for (int i = 0; i < rpmObjects.length; i++)
//        {
//            SecurityRole[] parentSecurityRoles = ((SecurityRoleCategory) rpmObjects[i])
//                .getSecurityRoles();
//            // for (int j = 0; j < parentSecurityRoles.length; j++)
//            {
//                prepareSecurityRole(parentSecurityRoles[0], false);
//                verifyRoleFlagsSingularity(parentSecurityRoles[0]);
//
//            }
//        }
//    }

    protected void verifyRoleFlagsSingularity(SecurityRole parentSecurityRole)
            throws Exception
    {
        String name = getChildName(parentSecurityRole.getName());
        SecurityRole securityRole = loadSecurityRoleByName(name);
        Assert.assertNotNull(securityRole);
        //        System.out.println("TEST - Verifying "+name);
        Map mapMethods = SecurityUtil
            .getFieldDescrsMap(securityRole.getClass());
        Iterator it = mapMethods.entrySet().iterator();
        while (it.hasNext())
        {
            BaseFieldDescr desc = (BaseFieldDescr) ((Map.Entry) it.next())
                .getValue();
            setRoleFlag(securityRole, desc, true);
            saveWithoutDelete(securityRole, null);
            securityRole = loadSecurityRoleByName(name);

            verifyModifiedRoleFlags(securityRole, desc, false);
            setRoleFlags(securityRole, false);
            saveWithoutDelete(securityRole, null);

        }
    }

    protected void verifyModifiedRoleFlags(SecurityRole securityRole,
            BaseFieldDescr skipFlag, boolean flagValue) throws Exception
    {
        Map mapMethods = SecurityUtil
            .getFieldDescrsMap(securityRole.getClass());
        Iterator it = mapMethods.entrySet().iterator();
        while (it.hasNext())
        {
            BaseFieldDescr desc = (BaseFieldDescr) ((Map.Entry) it.next())
                .getValue();
            if (desc == skipFlag)
                verifyRoleFlag(securityRole, desc, !flagValue);
            else
                verifyRoleFlag(securityRole, desc, flagValue);
        }
    }

    public void testEmulatedInMemoryAllSecurityRoleFlags() throws Exception
    {
        System.out
            .println("TEST - Starting testEmulatedInMemoryAllSecurityRoleFlags");
        SecurityRoleScope scope = new SecurityRoleScope();
        scope.setSecurityRoles(scope);
        RPMObject[] rpmObjects = loadFromXpath("/SecurityRoleCategory", scope);
        assertNotNull(rpmObjects);

        for (int i = 0; i < rpmObjects.length; i++)
        {
            SecurityRole[] parentSecurityRoles = ((SecurityRoleCategory) rpmObjects[i])
                .getSecurityRoles();
            Assert.assertNotNull(parentSecurityRoles);
            for (int j = 0; j < parentSecurityRoles.length; j++)
            {
                SecurityRole securityRole = prepareInMemorySecurityRole(
                    parentSecurityRoles[j], true);
                Assert.assertNotNull(securityRole);
                verifyRoleFlags(securityRole, true);
            }
        }

        for (int i = 0; i < rpmObjects.length; i++)
        {
            SecurityRole[] parentSecurityRoles = ((SecurityRoleCategory) rpmObjects[i])
                .getSecurityRoles();
            Assert.assertNotNull(parentSecurityRoles);
            for (int j = 0; j < parentSecurityRoles.length; j++)
            {
                SecurityRole securityRole = prepareInMemorySecurityRole(
                    parentSecurityRoles[j], false);
                Assert.assertNotNull(securityRole);
                verifyRoleFlags(securityRole, false);
            }
        }
    }

    /**
     * Tests the database's security flags for singularity. Loads all the flag
     * and sets them to false. Then set one flags to true and check if the
     * others have been affected.
     * 
     * @author Marc-Andr
     */
    public void testEmulatedInMemoryAllSecurityRoleFlagsSingularity()
            throws Exception
    {
        System.out
            .println("TEST - Starting testEmulatedInMemoryAllSecurityRoleFlagsSingularity");
        SecurityRoleScope scope = new SecurityRoleScope();
        scope.setSecurityRoles(scope);
        RPMObject[] rpmObjects = loadFromXpath("/SecurityRoleCategory", scope);
        assertNotNull(rpmObjects);

        for (int i = 0; i < rpmObjects.length; i++)
        {
            SecurityRole[] parentSecurityRoles = ((SecurityRoleCategory) rpmObjects[i])
                .getSecurityRoles();
            SecurityRole securityRole = prepareInMemorySecurityRole(
                parentSecurityRoles[0], false);
            verifyEmulatedInMemoryRoleFlagsSingularity(securityRole);

        }
    }

    protected void verifyEmulatedInMemoryRoleFlagsSingularity(
            SecurityRole securityRole) throws Exception
    {
        Assert.assertNotNull(securityRole);
        Map mapMethods = SecurityUtil
            .getFieldDescrsMap(securityRole.getClass());
        Iterator it = mapMethods.entrySet().iterator();
        while (it.hasNext())
        {
            BaseFieldDescr desc = (BaseFieldDescr) ((Map.Entry) it.next())
                .getValue();
            setRoleFlag(securityRole, desc, true);
            SecurityUtil.setClearanceLevel(
                SecurityUtil.getClearanceLevels(securityRole,
                    new int[SecurityUtil.SECURITY_ROLE_LEVELS_COUNT]),
                securityRole);
            verifyModifiedRoleFlags(securityRole, desc, false);
            setRoleFlags(securityRole, false);
            SecurityUtil.setClearanceLevel(
                SecurityUtil.getClearanceLevels(securityRole,
                    new int[SecurityUtil.SECURITY_ROLE_LEVELS_COUNT]),
                securityRole);
        }
    }

}
