/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases.framework;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.ibm.rpm.client.junit.SpecificTestController;
import com.ibm.rpm.client.junit.UnitTestSessionUtility;
import com.ibm.rpm.framework.InternalSecuredArrayResult;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.security.ContainerSecurityDescriptor;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredFields;
import com.ibm.rpm.interfaces.impl.ServerFactory;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.Field;
import com.ibm.rpm.metadata.model.MetadataStore;
import com.ibm.rpm.scopemanagement.containers.ReqProConnection;
import com.ibm.rpm.scopemanagement.containers.ReqProIntegrationFolder;
import com.ibm.rpm.scopemanagement.containers.ReqProRequirement;
import com.ibm.rpm.scopemanagement.containers.ReqProScopeFolder;
import com.ibm.rpm.wbs.containers.Project;

/**
 * @author Yasser Lulu
 *
 */
public class TestMetadataRulesUtilityLocal extends
        BaseSecurityFrameworkTestLocal
{
    public void setUp() throws Exception
    {
        // switch to LDAP mode
        SpecificTestController.overrideOption("webServicesSecurityEnabled",
            Boolean.TRUE);
        SpecificTestController.overrideOption(
            RpmOptions.WEB_SERVICES_SECURITY_CONTROLLER_CLASS_NAME, RPM_SEC);
        super.setUp();
    }

    public void testReadOnlyFieldSecurityDescriptorHandler() throws Exception
    {

        MessageContext messageContext = new MessageContext(ServerFactory
            .getInstance());
        messageContext.setSession(UnitTestSessionUtility.INSTANCE, sessionid,
            "testing");
        Project project = new Project();
        project.setID("projectID");
        messageContext.addObject(project);

        ReqProRequirement reqProRequirement = new ReqProRequirement();
        reqProRequirement.setID("reqProRequirementID");
        messageContext.addObject(reqProRequirement);

        ReqProScopeFolder reqProScopeFolder = new ReqProScopeFolder();
        reqProScopeFolder.setID("reqProScopeFolder");
        messageContext.addObject(reqProScopeFolder);

        ReqProIntegrationFolder reqProIntegrationFolder = new ReqProIntegrationFolder();
        reqProIntegrationFolder.setID("reqProIntegrationFolderID");
        messageContext.addObject(reqProIntegrationFolder);

        ReqProConnection reqProConnection = new ReqProConnection();
        reqProConnection.setID("reqProConnectionID");
        reqProIntegrationFolder.setReqProConnection(reqProConnection);

        InternalSecuredArrayResult result = messageContext
            .toInternalSecuredArrayResult();
        Map map = result.getContainerSecurityDescriptorsMap();

        verify(map, reqProConnection);
        verify(map, reqProScopeFolder);
        verify(map, reqProIntegrationFolder);
        verify(map, reqProRequirement);
        verify(map, project);

    }

    private List getReadOnlyFields(Class rpmObjectClass)
    {
        List readOnlyFields = new ArrayList();
        Container container = MetadataStore.getInstance()
            .getContainerByJavaClass(rpmObjectClass);
        if (container != null)
        {
            Iterator it = container.getFields().iterator();
            while (it.hasNext())
            {
                Field field = (Field) it.next();
                if (field.isReadOnlyOnUpdate())
                {
                    readOnlyFields.add(field);
                }
            }
        }
        return readOnlyFields;
    }

    private ContainerSecurityDescriptor verifyFields(Map map, RPMObject rpmObj)
    {
        ContainerSecurityDescriptor descr = (ContainerSecurityDescriptor) map
            .get(rpmObj);
        assertNotNull(descr);

        List fields = getReadOnlyFields(rpmObj.getClass());
        Iterator it = fields.iterator();
        while (it.hasNext())
        {
            Field field = (Field) it.next();
            FieldSecurityDescriptor fieldDescr = (FieldSecurityDescriptor) descr
                .getFieldSecurityDescriptors().get(field.getName());
            assertNotNull(fieldDescr);
            assertFalse(fieldDescr.isCanEdit());
        }
        return descr;
    }

    /**
     * @param map
     * @param reqProScopeFolder
     */
    private void verify(Map map, ReqProScopeFolder reqProScopeFolder)
    {
        ContainerSecurityDescriptor descr = verifyFields(map, reqProScopeFolder);
        FieldSecurityDescriptor fieldDescr = (FieldSecurityDescriptor) descr
            .getFieldSecurityDescriptors().get("name");
        assertNotNull(fieldDescr);
        assertFalse(fieldDescr.isCanEdit());
    }

    /**
     * @param map
     * @param project
     */
    private void verify(Map map, Project project)
    {
        verifyFields(map, project);
    }

    /**
     * @param map
     * @param reqProRequirement
     */
    private void verify(Map map, ReqProRequirement reqProRequirement)
    {
        ContainerSecurityDescriptor descr = verifyFields(map, reqProRequirement);
        FieldSecurityDescriptor fieldDescr = (FieldSecurityDescriptor) descr
            .getFieldSecurityDescriptors().get("name");
        assertNotNull(fieldDescr);
        assertFalse(fieldDescr.isCanEdit());

        fieldDescr = (FieldSecurityDescriptor) descr
            .getRTFSecurityDescriptors().get(SecuredFields.INTEGERS[1]);
        assertNotNull(fieldDescr);
        assertFalse(fieldDescr.isCanEdit());
    }

    /**
     * @param map
     * @param reqProIntegrationFolder
     */
    private void verify(Map map, ReqProIntegrationFolder reqProIntegrationFolder)
    {
        ContainerSecurityDescriptor descr = verifyFields(map,
            reqProIntegrationFolder);
        FieldSecurityDescriptor fieldDescr = (FieldSecurityDescriptor) descr
            .getFieldSecurityDescriptors().get("name");
        assertNotNull(fieldDescr);
        assertFalse(fieldDescr.isCanEdit());
    }

    /**
     * @param map
     * @param reqProConnection
     */
    private void verify(Map map, ReqProConnection reqProConnection)
    {
        ContainerSecurityDescriptor descr = (ContainerSecurityDescriptor) map
            .get(reqProConnection);

        FieldSecurityDescriptor fieldDescr = (FieldSecurityDescriptor) descr
            .getFieldSecurityDescriptors().get("reqProID");
        assertNotNull(fieldDescr);
        assertFalse(fieldDescr.isCanEdit());
    }

}
