/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases.resource;

import com.ibm.rpm.client.junit.SpecificTestController;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.security.testcases.BaseSecurityTestLocal;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author Marc-Andre Lamothe
 *
 */
public class BaseResourceSecurityTestLocal extends BaseSecurityTestLocal
{

    protected void setUp() throws Exception
    {
        SpecificTestController.overrideOption(
            RpmOptions.WEB_SERVICES_SECURITY_CONTROLLER_CLASS_NAME, RES_SEC);
        super.setUp();
        assertTrue(getClass().getName().endsWith("Local"));
    }

    protected RPMObject publishToTemplate(RPMObject rpmObject, Resource resource)
            throws Exception
    {
        // Publish container to template (only works for WBS, Scope or Document elements)
        MessageContext messageContext = fabricateBasicMessageContext("PublishToTemplate");
        String name = "SP_U_PUBLISHED";
        Object args[] = new Object[4];
        args[0] = rpmObject.getID();
        args[1] = "2";
        args[2] = rpmObject.getParent().getID();
        args[3] = resource.getID();
        ManagerUtil.executeProcedure(this.getClass(), messageContext, name,
            args);
        return loadOneFromXpath("/"
                + StringUtil.getShortClassName(rpmObject.getClass()) + "[id='"
                + args[0] + "']", null);
    }

    protected Project loadOrCreateProjectTemplate(Resource resource)
            throws Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setParent(new WorkElementScope());
        // Try to load a template        
        RPMObject[] rpmObjects = loadFromXpath("/Project[published='"
                + PublishedType._ToTemplate + "']", scope);
        if (rpmObjects != null && rpmObjects[0] instanceof Project)
            return (Project) rpmObjects[0];
        // Create a template
        GenericProject parent = (GenericProject) WorkElementFixture
            .retrieveStaticFixture(new Project(), scope, this);
        Project project = (Project) WorkElementFixture.createFixture(parent,
            new Project(), scope, this);
        project = (Project) saveAndValidate(project, scope, true);
        project = (Project) publishToTemplate(project, resource);
        return project;
    }

    protected Requirement createScopeTemplate(Resource resource)
            throws Exception
    {
        // Prepare scope
        ScopeElementScope childScope = new ScopeElementScope();
        ScopeElementScope scope = new ScopeElementScope();
        childScope.setParent(scope);
        // Try to load a template        
        RPMObject[] rpmObjects = loadFromXpath("/Requirement[published='"
                + PublishedType._ToTemplate + "']", scope);
        if (rpmObjects != null && rpmObjects[0] instanceof Requirement)
            return (Requirement) rpmObjects[0];
        // Retreive project
        WorkElementScope projScope = new WorkElementScope();
        scope.setChildren(childScope);
        scope.setParent(projScope);
        projScope.setScopeElement(scope);
        GenericProject project = (GenericProject) WorkElementFixture
            .retrieveStaticFixture(new Project(), projScope, this);
        ScopeFolder folder = (ScopeFolder) project.getScopeElement();
        // Create scope element
        Requirement requirement = (Requirement) ScopeElementFixture
            .retrieveStaticFixture(folder, new Requirement(), childScope, 0,
                this);
        //requirement = (Requirement) saveAndValidate(requirement, childScope, true);
        requirement = (Requirement) publishToTemplate(requirement, resource);
        return requirement;
    }
}
