/*
 * Created on Feb 1, 2007
 *
 */
package com.ibm.rpm.client.junit.security.testcases.resource;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.scope.AssetScope;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFixture;
import com.ibm.rpm.client.junit.asset.fixtures.AssetFolderFixture;

/**
 * @author Karim, Marc-Andre Lamothe
 *
 */
public class TestAssetSecurityLocal extends BaseResourceSecurityTestLocal
{
    public void testCreateAsset() throws Exception
    {
        // Remove right to create 
        getCurrentUser().getResourceSecurityRights()
            .setCanCreateDeleteAssetsAndFolders(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Create an asset, should fail
        AssetFolder assetFolder = AssetFolderFixture.createFixture(false, 0,
            null, this);
        assetFolder = (AssetFolder) saveAndValidate(assetFolder, null, false);

        // Give right to create 
        getCurrentUser().getResourceSecurityRights()
            .setCanCreateDeleteAssetsAndFolders(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Create an asset, should succeed
        assetFolder = AssetFolderFixture.createFixture(false, 0, null, this);
        assetFolder = (AssetFolder) saveAndValidate(assetFolder, null, true);
    }

    public void testDeleteAsset() throws Exception
    {
        // Create an asset, should succeed
        AssetFolder assetFolder = AssetFolderFixture.createFixture(false, 0,
            null, this);
        assetFolder = (AssetFolder) saveAndValidate(assetFolder, null, true);

        // Remove the right to delete
        getCurrentUser().getResourceSecurityRights()
            .setCanCreateDeleteAssetsAndFolders(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Delete the asset, should fail
        deleteAndValidate(assetFolder, false);

        // Give the right to delete
        getCurrentUser().getResourceSecurityRights()
            .setCanCreateDeleteAssetsAndFolders(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Delete the asset, should succeed
        deleteAndValidate(assetFolder, true);
    }

    public void testDeleteAssetFromOthers() throws Exception
    {
        AssetScope scope = new AssetScope();
        // Creates an asset, should succeed
        Asset asset = AssetFixture.createFixture(scope, this);
        asset = (Asset) saveAndValidate(asset, scope, true);

        // Log out and log in as different resource
        if (!logoutAndLoginAs("SECURITY_TESTER", "rpm"))
            fail("Could not login under different user.");

        // Remove the right to delete
        getCurrentUser().getResourceSecurityRights()
            .setCanDeleteAssetsAndFoldersFromOthers(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Deletes the asset, should fail
        deleteAndValidate(asset, false);

        // Give the right to delete
        getCurrentUser().getResourceSecurityRights()
            .setCanDeleteAssetsAndFoldersFromOthers(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Deletes the asset, should succeed
        deleteAndValidate(asset, true);
    }

}