/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases.resource;

import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.security.testcases.BaseSecurityTestLocal;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author Yasser Lulu
 *
 */
public class TestConvertProjectToOrganizationLocal extends BaseSecurityTestLocal
{

    public void test()
    {
        // Added so the test will not fail because there is no test in file...
    }
    
    public void DISABLED_BECAUSE_YOU_CANNOT_SET_YOURSELF_BACK_AS_PROJECT_MANAGER_testConvertProjectToOrganization() throws Exception
    {
        ResourceScope resScope = new ResourceScope();
        resScope.setResourceSecurityRights(true);

        WorkElementScope scope = new WorkElementScope();
        Project inputProject = (Project) WorkElementFixture.createSaveFixture(
            null, new Project(), scope, this);
        scope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        inputProject = (Project) loadFromXpath("/Project[id='"
                + inputProject.getID() + "']", scope, null, null, null, null,
            null, null, null, null, null, false)[0];
        ResourceTaskAssignment originalAssignment = inputProject
            .getResourceTaskAssignments()[0];
        Resource res = originalAssignment.getResource();
        SecurityRole managerRole = originalAssignment.getSecurityRole();
        ResourceSecurityRights rights = res.getResourceSecurityRights();

        //create a different resource to make it a Manager
        final Pool pool = (Pool) loadOneFromXpath("/Pool[name='KARIM_POOL']",
            null);
        Resource resource = ResourceFixture.createFixture(pool, null, this);
        resource = (Resource) (save(resource, null).getRpmObject());

        //assign a new user as a project manager of the project
        ResourceTaskAssignment assignment = new ResourceTaskAssignment();
        assignment.setParent(inputProject);
        assignment.setResource(resource);
        assignment.setSecurityRole(managerRole);
        save(assignment, null);

        //make sure it that the flag is true
        Boolean original = rights.getCanConvertProjectToOrganization();
        if (original.booleanValue())
        {
            convertMustSucceed(inputProject);
            //now set it to false
            rights.setCanConvertProjectToOrganization(Boolean.FALSE);
            save(res, resScope);
            //verify it fails
            convertMustFail(inputProject);

            //now set resource back as Manager so that it must never fail
            save(originalAssignment, null);
            convertMustSucceed(inputProject);

            //set flag back to its original value
            rights.setCanConvertProjectToOrganization(Boolean.TRUE);
            save(res, resScope);
        }
        else
        {
            convertMustFail(inputProject);
            // now set it to true
            rights.setCanConvertProjectToOrganization(Boolean.TRUE);
            save(res, resScope);
            //verify it succeeds
            convertMustSucceed(inputProject);
            //now set it to false as it was 
            rights.setCanConvertProjectToOrganization(Boolean.FALSE);
            save(res, resScope);
            //now set resource back as Manager so that it must never fail
            save(originalAssignment, null);
            convertMustSucceed(inputProject);
        }
    }

    /**
     * @param inputProject
     */
    private void convertMustFail(Project inputProject)
    {
        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputProject, new WorkOrganization(),
            new WorkElementScope());
        assertFalse(result.isSuccessful());
        assertTrue(result.getErrors()[0].getErrorID() == com.ibm.rpm.framework.constants.ErrorCodes.INSUFFICIENT_SECURITY_RIGHTS);
    }

    /**
     * 
     */
    private void convertMustSucceed(Project inputProject)
    {
        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputProject, new WorkOrganization(),
            new WorkElementScope());
        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof WorkOrganization);
        assertEquals(result.getRpmObject().getID(), inputProject.getID());
        //revert it back to project
        result = getApplicationInterface().convertContainer(sessionid,
            result.getRpmObject(), new Project(), new WorkElementScope());
        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof Project);
        assertEquals(result.getRpmObject().getID(), inputProject.getID());
    }

}
