/*
 * Created on Jan 31, 2007
 *
 */
package com.ibm.rpm.client.junit.security.testcases.resource;

import java.util.Arrays;
import java.util.Calendar;

import junit.framework.Assert;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.ContactGroupAssignment;
import com.ibm.rpm.applicationadministration.containers.GeographicalAssignment;
import com.ibm.rpm.applicationadministration.containers.OrganizationalAssignment;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.customfield.fixtures.CustomFieldFixture;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.resource.fixtures.EmploymentHistoryFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceAttributeAssignmentFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceContactGroupAssignmentFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceGeographicalAssignmentFixture;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.EmploymentHistory;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author karim
 *
 */
public class TestPersonalSecurityLocal extends BaseResourceSecurityTestLocal
{

    protected void saveResourceAndValidateAndReload(ResourceScope scope,
            boolean canSave) throws Exception
    {
        //Save into the DB
        SingleResult result = getApplicationInterface().save(sessionid,
            getCurrentUser(), scope, ReloadType.ReloadResult);
        Assert.assertNotNull("Result is null", result);
        if (canSave)
        {
            actionMustSucceed(result, getCurrentUser().getClass());
        }
        else
        {
            actionMustFail(result);
        }
        loadResourceWithSecurity(scope);
    }

    public void testEditPersonalAddress() throws Exception
    {
        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditPersonalAddress(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        String address = getCurrentUser().getAddress1();
        getCurrentUser().setAddress1(address + "_UPDATED");
        saveResourceAndValidateAndReload(null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditPersonalAddress(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        getCurrentUser().setAddress1(address + "_UPDATED");
        saveResourceAndValidateAndReload(null, true);

        // Restore old value
        getCurrentUser().setAddress1(address);
        saveResourceAndValidateAndReload(null, true);
    }

    public void testEditPersonalBackground() throws Exception
    {
        // Set the scope
        ResourceScope scope = new ResourceScope();
        scope.setEmploymentHistories(true);
        loadResourceWithSecurity(scope);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalBackground(Boolean.FALSE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should fail
        EmploymentHistory[] eh = getCurrentUser().getEmploymentHistories();
        EmploymentHistory[] eh2 =
        {
            EmploymentHistoryFixture.createFixture(getCurrentUser())
        };
        getCurrentUser().setEmploymentHistories(eh2);
        saveResourceAndValidateAndReload(scope, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalBackground(Boolean.TRUE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should succeed
        eh2 = new EmploymentHistory[]
        {
            EmploymentHistoryFixture.createFixture(getCurrentUser())
        };
        getCurrentUser().setEmploymentHistories(eh2);
        saveResourceAndValidateAndReload(scope, true);

        // Restore old value
        getCurrentUser().setEmploymentHistories(eh);
        saveResourceAndValidateAndReload(scope, true);
    }

    public void testEditPersonalCalendar() throws Exception
    {
        // Set the scope
        ResourceScope scope = new ResourceScope();
        scope.setCalendar(new CalendarScope());
        loadResourceWithSecurity(scope);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalCalendar(Boolean.FALSE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should fail
        RPMCalendar cal = getCurrentUser().getCalendar();
        getCurrentUser().setCalendar(
            RPMCalendarFixture.createSaveFixture(null, this));
        saveResourceAndValidateAndReload(scope, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalCalendar(Boolean.TRUE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should succeed
        getCurrentUser().setCalendar(
            RPMCalendarFixture.createSaveFixture(null, this));
        saveResourceAndValidateAndReload(scope, true);

        // Restore old value
        getCurrentUser().setCalendar(cal);
        saveResourceAndValidateAndReload(scope, true);
    }

    public void testEditPersonalCompetencyCostsRates() throws Exception
    {
        // Set the scope
        ResourceScope scope = new ResourceScope();
        scope.setCompetencyAttributeAssignments(true);
        loadResourceWithSecurity(scope);

        // Make sure there is an assignment
        ResourceAttributeAssignment[] raa = getCurrentUser()
            .getCompetencyAttributeAssignments();
        if (raa == null || raa[0] == null)
        {
            ResourceAttributeAssignment a = ResourceAttributeAssignmentFixture
                .createFixture(ResourceAttributeAssignmentFixture.COMPETENCY,
                    getCurrentUser(), this);
            getCurrentUser().setCompetencyAttributeAssignments(
                new ResourceAttributeAssignment[]
                {
                    a
                });
            saveResourceAndValidateAndReload(scope, true);
        }
        raa = getCurrentUser().getCompetencyAttributeAssignments();

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalCompetencyCostsRates(Boolean.FALSE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should fail
        ResourceAttributeAssignment[] aa = getCurrentUser()
            .getCompetencyAttributeAssignments();
        ResourceAttributeAssignmentFixture.updateRates(aa[0]);
        saveResourceAndValidateAndReload(scope, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalCompetencyCostsRates(Boolean.TRUE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should succeed
        aa = getCurrentUser().getCompetencyAttributeAssignments();
        ResourceAttributeAssignmentFixture.updateRates(aa[0]);
        saveResourceAndValidateAndReload(scope, true);

        // Restore old value
        aa = getCurrentUser().getCompetencyAttributeAssignments();
        if (aa != null && aa.length != 0)
        {
            aa[0].setDeleted(new Boolean(true));
            saveResourceAndValidateAndReload(scope, true);
        }
    }

    public void testEditPersonalCompetencySkills() throws Exception
    {
        // Set the scope
        ResourceScope scope = new ResourceScope();
        scope.setCompetencyAttributeAssignments(true);
        loadResourceWithSecurity(scope);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalCompetencySkills(Boolean.FALSE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should fail
        ResourceAttributeAssignment raa = ResourceAttributeAssignmentFixture
            .createFixture(ResourceAttributeAssignmentFixture.COMPETENCY,
                getCurrentUser(), this);
        getCurrentUser().setCompetencyAttributeAssignments(
            new ResourceAttributeAssignment[]
            {
                raa
            });
        saveResourceAndValidateAndReload(scope, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalCompetencySkills(Boolean.TRUE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should succeed
        raa = ResourceAttributeAssignmentFixture.createFixture(
            ResourceAttributeAssignmentFixture.COMPETENCY, getCurrentUser(),
            this);
        getCurrentUser().setCompetencyAttributeAssignments(
            new ResourceAttributeAssignment[]
            {
                raa
            });
        saveResourceAndValidateAndReload(scope, true);

        // Restore old value
        ResourceAttributeAssignment[] aa = getCurrentUser()
            .getCompetencyAttributeAssignments();
        if (aa != null && aa.length != 0)
        {
            aa[0].setDeleted(new Boolean(true));
            saveResourceAndValidateAndReload(scope, true);
        }
    }

    public void testCreatePersonalDocuments() throws Exception
    {
        // Set the scope
        ResourceScope scope = new ResourceScope();
        DocumentScope docScope = DocumentElementFixture.createScope(null, 1, 2,
            1);
        scope.setDocumentFolder(docScope);
        loadResourceWithSecurity(scope);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalDocuments(Boolean.FALSE);
        saveCurrentUserSecurityRights(scope);

        // Add new document, should fail
        DocumentElement de = DocumentElementFixture.createFixture(
            getCurrentUser().getDocumentFolder(), new Document(), 0, docScope
                .getChildren(), this);
        saveAndValidate(de, docScope.getChildren(), false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalDocuments(Boolean.TRUE);
        saveCurrentUserSecurityRights(scope);

        // Add new document, should succeed
        de = DocumentElementFixture.createFixture(getCurrentUser()
            .getDocumentFolder(), new Document(), 0, docScope.getChildren(),
            this);
        de = (DocumentElement) saveAndValidate(de, docScope.getChildren(), true);

        // Restore old value
        deleteAndValidate(de, true);
    }

    public void testEditPersonalDocuments() throws Exception
    {
        // Set the scope
        DocumentScope docScope = DocumentElementFixture.createScope(null, 1, 2,
            1);
        ResourceScope scope = new ResourceScope();
        scope.setDocumentFolder(docScope);
        loadResourceWithSecurity(scope);

        // Add new document, should succeed
        DocumentElement de = DocumentElementFixture.createFixture(
            getCurrentUser().getDocumentFolder(), new Document(), 0, docScope
                .getChildren(), this);
        de = (DocumentElement) saveAndValidate(de, docScope.getChildren(), true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalDocuments(Boolean.FALSE);
        saveCurrentUserSecurityRights(scope);

        // Update the document, should fail
        de.setDocumentRank("test");
        saveAndValidate(de, null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalDocuments(Boolean.TRUE);
        saveCurrentUserSecurityRights(scope);

        // Update the document, should succeed
        de.setDocumentRank("test");
        saveAndValidate(de, null, true);

        // Restore old value
        deleteAndValidate(de, true);
    }

    public void testDeletePersonalDocuments() throws Exception
    {
        // Set the scope
        DocumentScope docScope = DocumentElementFixture.createScope(null, 1, 2,
            1);
        ResourceScope scope = new ResourceScope();
        scope.setDocumentFolder(docScope);
        loadResourceWithSecurity(scope);

        // Add new document, should succeed
        DocumentElement de = DocumentElementFixture.createFixture(
            getCurrentUser().getDocumentFolder(), new Document(), 0, docScope
                .getChildren(), this);
        de = (DocumentElement) saveAndValidate(de, docScope.getChildren(), true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalDocuments(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Delete the document, should fail
        deleteAndValidate(de, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalDocuments(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Delete the document, should succeed
        deleteAndValidate(de, true);
    }

    public void testEditPersonalEffectiveDate() throws Exception
    {

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalEffectiveDiscontinueActiveDates(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        Calendar cal = getCurrentUser().getEffectiveStartDate();
        Calendar cal2 = cal;
        cal2.setTimeInMillis(cal.getTimeInMillis() - 100);
        getCurrentUser().setEffectiveStartDate(cal2);
        saveResourceAndValidateAndReload(null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalEffectiveDiscontinueActiveDates(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        getCurrentUser().setEffectiveStartDate(cal2);
        saveResourceAndValidateAndReload(null, true);

        // Restore old value 
        getCurrentUser().setEffectiveStartDate(cal);
        saveResourceAndValidateAndReload(null, true);
    }

    public void testEditPersonalExecutiveAttributes() throws Exception
    {
        ResourceScope scope = new ResourceScope();
        scope.setExecutiveAttributes(true);
        loadResourceWithSecurity(scope);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalExecutiveAttributes(Boolean.FALSE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should fail
        AttributeAssignment[] aa =
        {
            AttributeAssignmentFixture.createAssignmentFixture(
                getCurrentUser(), this)
        };
        getCurrentUser().setExecutiveAttributes(aa);
        saveResourceAndValidateAndReload(scope, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalExecutiveAttributes(Boolean.TRUE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should succeed
        aa = new AttributeAssignment[]
        {
            AttributeAssignmentFixture.createAssignmentFixture(
                getCurrentUser(), this)
        };
        getCurrentUser().setExecutiveAttributes(aa);
        saveResourceAndValidateAndReload(scope, true);

        // Restore old value 
        delete(getCurrentUser().getExecutiveAttributes()[0]);
    }

    public void testEditPersonalDiscontinueDate() throws Exception
    {

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalEffectiveDiscontinueActiveDates(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        Calendar cal = getCurrentUser().getDiscontinueDate();
        Calendar cal2 = (cal == null ? Calendar.getInstance() : cal);
        cal2.setTimeInMillis(cal2.getTimeInMillis() + 100);
        getCurrentUser().setDiscontinueDate(cal2);
        saveResourceAndValidateAndReload(null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalEffectiveDiscontinueActiveDates(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        getCurrentUser().setDiscontinueDate(cal2);
        saveResourceAndValidateAndReload(null, true);

        // Restore old value 
        getCurrentUser().setDiscontinueDate(cal);
        saveResourceAndValidateAndReload(null, true);
    }

    public void testEditPersonalActive() throws Exception
    {
        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalEffectiveDiscontinueActiveDates(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        Boolean active = getCurrentUser().getActive();
        if (active == null)
        {
            getCurrentUser().setActive(Boolean.TRUE);
        }

        getCurrentUser().setActive(
            Boolean.valueOf(!getCurrentUser().getActive().booleanValue()));
        saveResourceAndValidateAndReload(null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalEffectiveDiscontinueActiveDates(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        getCurrentUser().setActive(
            Boolean.valueOf(!getCurrentUser().getActive().booleanValue()));
        saveResourceAndValidateAndReload(null, true);

        // Restore old value 
        getCurrentUser().setActive(active);
        saveResourceAndValidateAndReload(null, true);
    }

    public void testEditPersonalInformation() throws Exception
    {

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalInformation(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        String title = getCurrentUser().getTitle();
        getCurrentUser().setTitle(title + "_UPDATED");
        saveResourceAndValidateAndReload(null, false);

        // Give the right to edit
        getCurrentUser().setTitle(title);//restore
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalInformation(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        getCurrentUser().setTitle(title + "_UPDATED");
        saveResourceAndValidateAndReload(null, true);

        // Restore old value 
        getCurrentUser().setTitle(title);
        saveResourceAndValidateAndReload(null, true);
    }

    public void testEditPersonalName() throws Exception
    {

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditPersonalName(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        String name = getCurrentUser().getGivenName();
        getCurrentUser().setGivenName(name + "_UPDATED");
        saveResourceAndValidateAndReload(null, false);

        // Give the right to edit
        getCurrentUser().setGivenName(name);//restore
        getCurrentUser().getResourceSecurityRights().setCanEditPersonalName(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        getCurrentUser().setGivenName(name + "_UPDATED");
        saveResourceAndValidateAndReload(null, true);

        // Restore old value 
        getCurrentUser().setGivenName(name);
        saveResourceAndValidateAndReload(null, true);
    }

    public void testEditPersonalNickName() throws Exception
    {

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalNickName(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        String nickname = getCurrentUser().getNickName();
        getCurrentUser().setNickName(nickname + "_UPDATED");
        saveResourceAndValidateAndReload(null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalNickName(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        getCurrentUser().setNickName(nickname + "_UPDATED");
        saveResourceAndValidateAndReload(null, true);

        // Restore old value 
        getCurrentUser().setNickName(nickname);
        saveResourceAndValidateAndReload(null, true);
    }

    public void testEditPersonalOrganization() throws Exception
    {
        // Set up the Scope
        ResourceScope scope = new ResourceScope();
        scope.setOrganizationalAssignment(true);
        loadResourceWithSecurity(scope);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalOrganizationGroupGeographical(Boolean.FALSE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should fail
        OrganizationalAssignment assignment = getCurrentUser()
            .getOrganizationalAssignment();
        OrganizationalAssignment oa = new OrganizationalAssignment();
        oa.setOrganizational(OrganizationalFixture.createSaveFixture(0, this));
        getCurrentUser().setOrganizationalAssignment(oa);
        saveResourceAndValidateAndReload(scope, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalOrganizationGroupGeographical(Boolean.TRUE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should succeed
        oa = new OrganizationalAssignment();
        oa.setOrganizational(OrganizationalFixture.createSaveFixture(0, this));
        getCurrentUser().setOrganizationalAssignment(oa);
        saveResourceAndValidateAndReload(scope, true);

        // Restore old value 
        getCurrentUser().setOrganizationalAssignment(assignment);
        saveResourceAndValidateAndReload(scope, true);
    }

    public void testEditPersonalGroup() throws Exception
    {
        // Set up the Scope
        ResourceScope scope = new ResourceScope();
        scope.setContactGroupAssignments(true);
        loadResourceWithSecurity(scope);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalOrganizationGroupGeographical(Boolean.FALSE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should fail
        ContactGroupAssignment[] assignment = getCurrentUser()
            .getContactGroupAssignments();
        ContactGroupAssignment[] cga =
        {
            ResourceContactGroupAssignmentFixture.createAssignmentFixture(
                getCurrentUser(), this)
        };
        getCurrentUser().setContactGroupAssignments(cga);
        saveResourceAndValidateAndReload(scope, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalOrganizationGroupGeographical(Boolean.TRUE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should succeed
        cga = new ContactGroupAssignment[]
        {
            ResourceContactGroupAssignmentFixture.createAssignmentFixture(
                getCurrentUser(), this)
        };
        getCurrentUser().setContactGroupAssignments(cga);
        saveResourceAndValidateAndReload(scope, true);

        // Restore old value 
        getCurrentUser().setContactGroupAssignments(assignment);
        saveResourceAndValidateAndReload(scope, true);
    }

    public void testEditPersonalGeographical() throws Exception
    {
        // Set up the scope
        ResourceScope scope = new ResourceScope();
        scope.setGeographicalAssignment(true);
        loadResourceWithSecurity(scope);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalOrganizationGroupGeographical(Boolean.FALSE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should fail
        GeographicalAssignment assignment = getCurrentUser()
            .getGeographicalAssignment();
        GeographicalAssignment ga = ResourceGeographicalAssignmentFixture
            .createFixture(getCurrentUser(), this);
        getCurrentUser().setGeographicalAssignment(ga);
        saveResourceAndValidateAndReload(scope, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalOrganizationGroupGeographical(Boolean.TRUE);
        saveCurrentUserSecurityRights(scope);

        // Update the resource, should succeed
        ga = ResourceGeographicalAssignmentFixture.createFixture(
            getCurrentUser(), this);
        getCurrentUser().setGeographicalAssignment(ga);
        saveResourceAndValidateAndReload(scope, true);

        // Restore old value 
        getCurrentUser().setGeographicalAssignment(assignment);
        saveResourceAndValidateAndReload(scope, true);
    }

    //    Disabled because once a passsword is changed
    //    it cannot be changed back to it's original value.
    //    public void testEditPersonalPassword() throws Exception
    //    {
    //        Resource resource = loadResourceWithSecurity();
    //        
    //        // Remove the right to edit
    //        getCurrentUser().getResourceSecurityRights().setCanEditPersonalPassword(Boolean.FALSE);
    //        saveCurrentUserSecurityRights();
    //        resource = loadResourceWithSecurity();
    //        
    //        // Update the resource, should fail
    //        String password = resource.getPassword();
    //        resource.setPassword("test");
    //        saveAndValidate(resource, null, false);
    //        
    //        // Give the right to edit
    //        resource = loadResourceWithSecurity();
    //        getCurrentUser().getResourceSecurityRights().setCanEditPersonalPassword(Boolean.TRUE);
    //        saveCurrentUserSecurityRights();
    //        resource = loadResourceWithSecurity();
    //        
    //        // Update the resource, should succeed
    //        resource.setPassword("test");
    //        saveAndValidate(resource, null, true);
    //        
    //        // Restore old value 
    //        resource.setPassword(password);
    //        saveAndValidate(resource, null, true);
    //    }

    public void testEditPersonalRates() throws Exception
    {
        // Set the scope
        ResourceScope scope = new ResourceScope();
        scope.setRates(true);
        scope.setCurrency(true);
        loadResourceWithSecurity(scope);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditPersonalRates(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(scope);
        ResourceRates rates = getCurrentUser().getRates();

        // Update the rates, should fail
        ResourceRates rr = getCurrentUser().getRates();
        rr.setSellingRatePerHour(new Double(rr.getSellingRatePerHour()
            .intValue() + 10));
        saveAndValidate(rr, null, false);

        // Update the resource, should fail
        // Not valid because the Is Modified value is not set
        //XXX TODO revist the isModified()
        // resource.setRates(ResourceFixture.createRates());
        // saveAndValidate(resource, scope, false);

        // Give the right to edit
        loadResourceWithSecurity(scope);
        getCurrentUser().getResourceSecurityRights().setCanEditPersonalRates(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(scope);

        // Update the rates, should succeed
        // Not supported by the manager!!!
        // rr = resource.getRates();
        // rr.setSellingRatePerHour(new Double(rr.getSellingRatePerHour().intValue()+10));
        // saveAndValidate(rr, null, true);

        //Update the resource, should succeed
        getCurrentUser().setRates(ResourceFixture.createRates());
        saveResourceAndValidateAndReload(scope, true);

        // Restore old value 
        getCurrentUser().setRates(rates);
        saveResourceAndValidateAndReload(scope, true);
    }

    public void testEditPersonalUsername() throws Exception
    {
        String username = null;
        try
        {
            // Remove the right to edit
            getCurrentUser().getResourceSecurityRights()
                .setCanEditPersonalUserNameEmail(Boolean.FALSE);
            saveCurrentUserSecurityRights(null);

            // Update the resource, should fail
            username = getCurrentUser().getUserName();
            getCurrentUser().setUserName(username + "_UPDATED");
            saveResourceAndValidateAndReload(null, false);

            // Give the right to edit
            getCurrentUser().getResourceSecurityRights()
                .setCanEditPersonalUserNameEmail(Boolean.TRUE);
            saveCurrentUserSecurityRights(null);

            // Update the resource, should succeed
            getCurrentUser().setUserName(username + "_UPDATED");
            //            saveAndValidate(getCurrentUser(), null, true);
            saveResourceAndValidateAndReload(null, true);
        }
        finally
        {
            // Restore old value
            getCurrentUser().setUserName(username);
            saveResourceAndValidateAndReload(null, true);
        }
    }

    public void testEditPersonalEmail() throws Exception
    {

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalUserNameEmail(Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should fail
        String email = getCurrentUser().getEmailAddress();
        getCurrentUser().setEmailAddress(email + "_UPDATED");
        saveResourceAndValidateAndReload(null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalUserNameEmail(Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the resource, should succeed
        getCurrentUser().setEmailAddress(email + "_UPDATED");
        saveResourceAndValidateAndReload(null, true);

        // Restore old value
        getCurrentUser().setEmailAddress(email);
        saveResourceAndValidateAndReload(null, true);
    }

    public void testEditPersonalCustomFieldGroups() throws Exception
    {
        Boolean[] flagArray = new Boolean[10];
        Arrays.fill(flagArray, Boolean.TRUE);

        // Set up the scope
        ResourceScope scope = new ResourceScope();
        scope.setCustomFieldAssignments(true);
        loadResourceWithSecurity(scope);
        CustomFieldAssignment[] cfa = getCurrentUser()
            .getCustomFieldAssignments();

        // Create a custom field assignment, should succeed
        CustomFieldAssignment[] cfa2 =
        {
            new CustomFieldAssignment()
        };
        cfa2[0].setCustomField(CustomFieldFixture.createSaveCustomField(
            CustomFieldFixture.CF_RESOURCE_CONTEXT, FieldInputType.Text,
            new Integer(1), this));
        getCurrentUser().setCustomFieldAssignments(cfa2);
        saveResourceAndValidateAndReload(scope, true);

        // Remove the right to edit
        loadResourceWithSecurity(scope);
        flagArray[0] = Boolean.FALSE;
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalRecordCustomFieldsGroups(flagArray);
        saveCurrentUserSecurityRights(scope);

        // Update the custom field assignment, should fail
        cfa2 = getCurrentUser().getCustomFieldAssignments();
        cfa2[0].setValue(cfa2[0].getValue() + "_MOD");
        saveAndValidate(cfa2[0], null, false);

        // Give the rigth to edit
        loadResourceWithSecurity(scope);
        flagArray[0] = Boolean.TRUE;
        getCurrentUser().getResourceSecurityRights()
            .setCanEditPersonalRecordCustomFieldsGroups(flagArray);
        saveCurrentUserSecurityRights(scope);

        // Update the custom field assignment, should succeed
        cfa2 = getCurrentUser().getCustomFieldAssignments();
        cfa2[0].setValue("allo");
        saveAndValidate(cfa2[0], null, true);

        // Restore old value
        getCurrentUser().setCustomFieldAssignments(cfa);
        saveResourceAndValidateAndReload(null, true);
    }

}