/*
 * Created on Feb 1, 2007
 *
 */
package com.ibm.rpm.client.junit.security.testcases.resource;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.SecurityRoleFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.scope.SecurityRoleScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Task;

/**
 * @author karim
 *
 */
public class TestSecurityRoleSecurityLocal extends
        BaseResourceSecurityTestLocal
{
    private SecurityRole createFixture(RPMObject rpmObject) throws Exception
    {
    	SecurityRole parent = SecurityRoleFixture.createSecurityRole(rpmObject);
    	parent = (SecurityRole)loadFromID(parent, null);
        SecurityRole securityRole = new SecurityRole();
        securityRole.setName(HelperTestUtilities.createName());
        securityRole.setParent(parent);
        return securityRole;
    }

    public void testCreateProjectSecurityRole() throws Exception
    {
        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditWBSSecurity(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Create a security role, should fail
        SecurityRole securityRole = createFixture(new Project());
        securityRole = (SecurityRole) saveAndValidate(securityRole, null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditWBSSecurity(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Create a security role, should succeed
        securityRole = createFixture(new Project());
        securityRole = (SecurityRole) saveAndValidate(securityRole, null, true);
    }

    public void testCreateTaskSecurityRole() throws Exception
    {
    	getCurrentUser().getResourceSecurityRights().setCanEditWBSSecurity(
    			Boolean.FALSE);
    	saveCurrentUserSecurityRights(null);

    	// Create a security role, should fail
    	SecurityRole securityRole = createFixture(new Task());
    	securityRole = (SecurityRole) saveAndValidate(securityRole, null, false);

    	// Give the right to edit
    	getCurrentUser().getResourceSecurityRights().setCanEditWBSSecurity(
    			Boolean.TRUE);
    	saveCurrentUserSecurityRights(null);

    	// Create a security role, should succeed
    	securityRole = createFixture(new Task());
    	securityRole = (SecurityRole) saveAndValidate(securityRole, null, true);
    }
    
    public void testCreateDefectSecurityRole() throws Exception
    {
    	getCurrentUser().getResourceSecurityRights().setCanEditWBSSecurity(
    			Boolean.FALSE);
    	saveCurrentUserSecurityRights(null);

    	// Create a security role, should fail
    	SecurityRole securityRole = createFixture(new Defect());
    	securityRole = (SecurityRole) saveAndValidate(securityRole, null, false);

    	// Give the right to edit
    	getCurrentUser().getResourceSecurityRights().setCanEditWBSSecurity(
    			Boolean.TRUE);
    	saveCurrentUserSecurityRights(null);

    	// Create a security role, should succeed
    	securityRole = createFixture(new Defect());
    	securityRole = (SecurityRole) saveAndValidate(securityRole, null, true);
    }
    
    public void testDeleteSecurityRole() throws Exception
    {
        // Create a security role, should succeed
        SecurityRole securityRole = createFixture(new Project());
        securityRole = (SecurityRole) saveAndValidate(securityRole, null, true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditWBSSecurity(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Delete the container, should fail
        deleteAndValidate(securityRole, false);

        // Give the right to edit (for clean up)
        getCurrentUser().getResourceSecurityRights().setCanEditWBSSecurity(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Delete the container, should succeed
        deleteAndValidate(securityRole, true);
    }

    public void testUpdateSecurityRole() throws Exception
    {
        // Set up the scope
        SecurityRoleScope scope = new SecurityRoleScope();
        scope.setParent(new SecurityRoleScope());

        // Create a security role, should succeed
        SecurityRole securityRole = createFixture(new Project());
        securityRole = (SecurityRole) saveAndValidate(securityRole, scope, true);

        // Remove the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditWBSSecurity(
            Boolean.FALSE);
        saveCurrentUserSecurityRights(null);

        // Update the container, should fail
        securityRole.setModified();
        saveAndValidate(securityRole, null, false);

        // Give the right to edit
        getCurrentUser().getResourceSecurityRights().setCanEditWBSSecurity(
            Boolean.TRUE);
        saveCurrentUserSecurityRights(null);

        // Update the container, should succeed
        securityRole.setModified();
        saveAndValidate(securityRole, null, true);
    }

}
