package com.ibm.rpm.client.junit.security.testcases.role;

import java.util.ArrayList;

import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.document.types.ElementSecurityType;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public abstract class AbstractPlannedDocumentSecurityTestLocal extends
        AbstractPlannedSecurityTestLocal
{
    public AbstractPlannedDocumentSecurityTestLocal(Class securityRoleClass,
            RPMObject containerObject, RPMObject grandparentWorkElement,
            RPMObjectScope testObjectScope, String fieldToTest, String flagName)
    {
        super(securityRoleClass, containerObject, testObjectScope, fieldToTest,
            flagName);
        this.grandparentObject = grandparentWorkElement;
    }
    
    public AbstractPlannedDocumentSecurityTestLocal(Class securityRoleClass,
            RPMObject containerObject, RPMObject grandparentWorkElement,
            RPMObjectScope testObjectScope, ArrayList fieldsToTest, String flagName)
    {
        super(securityRoleClass, containerObject, testObjectScope, fieldsToTest,
            flagName);
        this.grandparentObject = grandparentWorkElement;
    }

    protected void prepareParentHierarchy() throws Exception
    {
        // create and save the grandparent of the document
        grandparentObjectSecurityRole = prepareSecurityRole(grandparentObject
            .getClass(), true);
        grandparentObject = (WorkElement) prepareContainer(grandparentObject,
            null, new WorkElementScope(), securityTestUser,
            grandparentObjectSecurityRole);

        // Set the grandparent scope used in the load operation
        WorkElementScope grandparentWorkElementScope = new WorkElementScope();
        DocumentScope documentFolderScope = new DocumentScope();
        documentFolderScope.setResourceAssignments(true);
        grandparentWorkElementScope.setDocumentFolder(documentFolderScope);
        grandparentWorkElementScope
            .setResourceTaskAssignments(new ResourceTaskAssignmentScope());

        // Load the grandparent with the parent DocumentFolder
        grandparentObject = (WorkElement) loadFromID(grandparentObject,
            grandparentWorkElementScope);
        assertNotNull(grandparentObject);

        // Get the parent DocumentFolder
        parentObject = DocumentElementFixture
            .getParentDocumentFolder(grandparentObject);
        assertNotNull(parentObject);
        parentObjectSecurityRole = prepareSecurityRole(parentObject.getClass(),
            true);
    }

    /**
     * Verify that test must fail for CanLock cases.
     * @throws Exception
     */
    protected void verifyMustFailScenarioForLock() throws Exception
    {
    	lockAndValidate((GenericDocument)testObject,false);
    	assertFalse(((GenericDocument)testObject).getLocked().booleanValue());
    }

    /**
     * Verify that test muss success for CanLock cases.
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForLock() throws Exception
    {
    	testObject = lockAndValidate((GenericDocument)testObject,true);
    	assertTrue(((GenericDocument)testObject).getLocked().booleanValue());
    }
    
    /**
     * Verify that test must fail for CanUnlock cases.
     * @throws Exception
     */
    protected void verifyMustFailScenarioForUnlock() throws Exception
    {
    	unLockAndValidate((GenericDocument)testObject,false);
    	assertTrue(((GenericDocument)testObject).getLocked().booleanValue());
    }

    /**
     * Verify that test muss success for CanUnlock cases.
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForUnlock() throws Exception
    {
    	testObject = unLockAndValidate((GenericDocument)testObject,true);
    	assertFalse(((GenericDocument)testObject).getLocked().booleanValue());
    }
    
    /**
     * Verify that test must fail for CanCheckOutOrCheckOutT cases.
     * @throws Exception
     */
    protected void verifyMustFailScenarioForCheckout() throws Exception
    {
    	checkOutAndValidate(testObject, false);
    }

    /**
     * Verify that test muss success for CanCheckOutOrCheckOutT cases.
     * @throws Exception
     */
    protected void verifyMustSuccessScenarioForCheckout() throws Exception
    {
    	checkOutAndValidate(testObject,true);
    	testObject = loadFromID(testObject, testObjectScope);
    	assertNotNull(testObject);
    	assertNotNull(((GenericDocument)testObject).getDocumentStatus());
    	assertEquals("CheckedOut", ((GenericDocument)testObject).getDocumentStatus().getValue());
    }
    
    /**
     * Change the ElementSecurityType value
     * 
     * @param value
     * @return
     * 			The Changed ElementSecurityType value
     * @throws Exception
     */
    private ElementSecurityType getChangedElementSecurityType(ElementSecurityType value) throws Exception
    {
    	if(ElementSecurityType.ByInvitation.equals(value))
    	{
    		value = ElementSecurityType.ProjectLevel;
    	}
    	else
    	{
    		value = ElementSecurityType.ByInvitation; 
    	}
    	
    	return value;
    }

    /**
     * Change a certain value.
     * 
     * @param object
     *            The object for which we want to change the value.
     * @return the new object which has been changed
     * @throws Exception
     */
    protected Object changeArgValue(Object object) throws Exception
    {
    	if(object instanceof ElementSecurityType)
    	{
    		object = getChangedElementSecurityType((ElementSecurityType)object);
    	}
    	else
    	{
    		object = super.changeArgValue(object);
    	}
    	return object;
    }
}
