package com.ibm.rpm.client.junit.security.testcases.role;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.AttributeAssignmentFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.client.junit.scorecard.testcases.validation.ScorecardScopeValidationUtility;
import com.ibm.rpm.client.junit.workflow.testcases.AbstractTestWorkflowProcess;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.scope.ScorecardScope;
import com.ibm.rpm.security.containers.DefectSecurityRole;
import com.ibm.rpm.security.containers.IssueSecurityRole;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.containers.TaskSecurityRole;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.AvailableWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class TestAbstractScopeSecurityRoleLocal extends
        BaseRoleSecurityTestLocal
{
    DefectSecurityRole defectSecurityRole;
    TaskSecurityRole taskSecurityRole;
    ScopeElementScope defectScope;
    WorkElementScope taskScope;
    Defect aDefect;
    Task aTask;
    Issue anIssue;
    RPMObject projectScopeFolder;
    ProjectSecurityRole enclosingProjectRole;
    Project parentProject;
    private IssueSecurityRole issueSecurityRole;
    private static final String PROCESS_NAME = "IssueProcess1";
    protected AbstractTestWorkflowProcess workflowAssigner = new AbstractTestWorkflowProcess();

    
    protected void setUp() throws Exception
    {
//        isEnableSecurity = false;
        super.setUp();
        
        /* Prepare global variables */
        defectScope = new ScopeElementScope();
        defectSecurityRole = (DefectSecurityRole) prepareSecurityRole(Defect.class, true);
        issueSecurityRole = (IssueSecurityRole) prepareSecurityRole(Issue.class, true);
        
        taskScope = new WorkElementScope();
        taskSecurityRole = (TaskSecurityRole) prepareSecurityRole(Task.class, true);
        
        /* Prepare Parent test environment */      
        enclosingProjectRole = (ProjectSecurityRole) prepareSecurityRole(Project.class, true);     
        GenericProject testProject = (GenericProject) prepareContainer(new Project(), null,           
            new WorkElementScope(), securityTestUser, enclosingProjectRole);
        parentProject = (Project) save(testProject, new WorkElementScope()).getRpmObject();
        
        /* init defectParent */
        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
            + testProject.getName() + "']", new ScopeElementScope());
        projectScopeFolder = rpmObjects[0];
        assertNotNull(projectScopeFolder);
        
        enclosingProjectRole.setAllDocumentPermissionsGranted(Boolean.FALSE);
        enclosingProjectRole.setAllScopeElementsPermissionsGranted(Boolean.FALSE);
        updateSecurityRole(enclosingProjectRole); 
    }
    
    
    /**
     * Three flags are interdependent to dissalow viewing : Attributes (view/edit/add+remove)
     * if only view is turned off, we can successfully read the information from the database
     *  
     * Allows to view the Attributes portlet in the Project, Deliverable, Task,  
     * Scope, Document and Notes/Minutes description view.
     * 
     * It protects attributeAssignments of AggregateNode, WorkElement
     * against illicit loading.
     *
     * @throws Exception
     */


    
    /**
     * TODO: June 20 : The API modifies the attribute assignments, even though the
     * values are read-only before the save for the SECURITY_TESTER user. After the save,
     * the new values appear successfully.
     * 
     * 
     * Allows to add or remove values in the Attributes portlet in the Project,
     * Deliverable, Task, Scope, Document and Notes/Minutes description view.
     * 
     */
    public void testCanAddRemoveAttributesForAggregateNode() throws Exception
    {
        /* Prepare test environment*/
        defectScope.setAttributeAssignments(true);
        aDefect = (Defect)prepareContainer(new Defect(), projectScopeFolder, defectScope, 
            securityTestUser, defectSecurityRole);
        
        /* Remove the right */
        flagOff(aDefect, "canAddRemoveAttributes");
        
        AttributeAssignment attributeassignment = AttributeAssignmentFixture.createAssignmentFixture(aDefect, this);
        Attribute mainAttribute = (Attribute) attributeassignment.getAttribute();
        mainAttribute.setInputType(InputType.Text);
        final String attributeValue = HelperTestUtilities.createName();
        attributeassignment.setValue(attributeValue);
        
        aDefect.setAttributeAssignments(new AttributeAssignment[] {
                attributeassignment
        });
        
        assertTrue(aDefect.getAttributeAssignments()[0] != null);
        saveAndValidate(aDefect, defectScope, false);
        
        flagOn(aDefect, "canAddRemoveAttributes");
        
        aDefect.setAttributeAssignments(new AttributeAssignment[] {
                attributeassignment
        });
        saveAndValidate(aDefect, defectScope, true);
        aDefect = (Defect) loadFromID(aDefect, defectScope);
        assertTrue(aDefect.getAttributeAssignments()[0] != null);
        assertTrue(aDefect.getAttributeAssignments()[0].getValue().equals(attributeValue));
    }
    
    /**
     * TODO: andreit Mat 30, 07 - the flag is not implemented at the moment.
     * Allows to view the "Client name" value in the Identification 
     * portlet of the Scope description view and in the Scope Management 
     * tree (all filters and all view). 
     */
    public void testCanViewClientName() throws Exception
    {
        /* Prepare test environment*/
        defectScope.setClientCostCenter(true);
        aDefect = (Defect)prepareContainer(new Defect(), projectScopeFolder, defectScope, 
            securityTestUser, defectSecurityRole);
        assertNotNull(aDefect);
        String beforeSwitch = aDefect.getClientCostCenter().getName();
        assertNotNull("Test case was not prepared properly", beforeSwitch);
        
        /* Remove the right to viewClientName */
        defectSecurityRole.setCanViewClientName(Boolean.FALSE);
        updateDefectSecurityRole();
        
        /* Try to read the client cost center */
        aDefect = (Defect) loadFromID(aDefect, defectScope);
        String afterSwitch = aDefect.getClientCostCenter().getName();
        
        assertNull(afterSwitch);
        assertFalse(beforeSwitch.equals(afterSwitch));
        
        
        /* Allow viewClientName */
        defectSecurityRole.setCanViewClientName(Boolean.TRUE);
        updateDefectSecurityRole();
        
        /* Try to read the client cost center */
        aDefect = (Defect) loadFromID(aDefect, defectScope);
        afterSwitch = aDefect.getClientCostCenter().getName();
        
        assertNotNull(afterSwitch);
        assertTrue(beforeSwitch.equals(afterSwitch));
    }
    
    /**
     * 
     * Allows to add or remove values in the Scorecard portlet in the Identification 
     * portlet of the Scope description view.
     * @throws Exception
     */
    public void testCanAddRemoveScorecards() throws Exception
    {     
        /* Prepare test environment*/
        ScorecardScope scorecardScope = PublishedScorecardFixture
            .createScorecardScope(defectScope);
        defectScope.setAssignedScorecard(scorecardScope);

        ScorecardScopeValidationUtility util = new ScorecardScopeValidationUtility();
        ScopeManagementModule module = util.createModule();
        Scorecard scorecardToAssign =  module.getScorecardFolders()[0].getScorecards()[0];
        
        aDefect =  (Defect)prepareContainer(new Defect(), projectScopeFolder, defectScope, 
            securityTestUser, defectSecurityRole);

        saveAndValidate(aDefect, defectScope, true);

        flagOff(aDefect, "canAddRemoveScorecards");
        
        SingleResult res = (getApplicationInterface()
            .assignScoreCard(getSessionID(), aDefect, defectScope,
                scorecardToAssign,
                ReloadType.ReloadResult));

        actionMustFail(res);
        
        /* Allow the operation */
        flagOn(aDefect, "canAddRemoveScorecards");
        
        aDefect = (Defect) (getApplicationInterface()
                .assignScoreCard(getSessionID(), aDefect, defectScope,
                    scorecardToAssign,
                    ReloadType.ReloadResult)).getRpmObject();
        
        assertNotNull(aDefect.getAssignedScorecard());
             
        /* Remove the right */
        flagOff(aDefect, "canAddRemoveScorecards");
 
        /* Try to delete */
        deleteMustFail(aDefect.getAssignedScorecard());
        
        /* Allow the operation */
        flagOn(aDefect, "canAddRemoveScorecards");
        
        deleteAndValidate(aDefect.getAssignedScorecard(), true);
    }
    
    protected void updateDefectSecurityRole() throws Exception
    {
        defectSecurityRole = (DefectSecurityRole) updateSecurityRole(defectSecurityRole);            
    }
    
    /**
     * TODO: andreit June 13 - startWorkflow fails with -433 (DB error). The 
     * test is not complete.
     * 
     * Allows to manage or modify a Workflow. This only applies to manual
     * Workflows.
     * 
     * You can manage the list of active workflow processes on any element. The
     * Manage Workflow dialog allows you to suspend, resume, or cancel a
     * workflow process. Note: you need to have a security right to be able to
     * manage workflow processes.
     * 
     * To manage the active workflow processes:
     * 
     * 1. Select any element with active workflow processes. 2. Right click and
     * select Communications from the menu and Manage Workflows from the second
     * menu.
     * 
     * 
     * 
     */
    /*
    DEBUG - {call SP_A_WORKFLOW(3,'5927E0EE4DC1494EA65BBD7E439DF8A6','61B5BE0E2AB04D9DBB62A20E9D11CC5D','D06047F5F3874A83BC7C779BF74D141B')} execution time : 10 (ms)
ERROR - 
*** begin dumpStoredProcedureStack ***
 *** ends dumpStoredProcedureStack *** 

ERROR - No error code value could be retrieved for the following key: -433
DEBUG - Rolling back the connection
ERROR - 
No error code value could be retrieved for the following key: -433 - Severity: Error - Error ID: -433
    at com.ibm.rpm.framework.ManagerUtil.executeProcedure(ManagerUtil.java:883)
    at com.ibm.rpm.framework.ManagerUtil.executeProcedure(ManagerUtil.java:827)
    at com.ibm.rpm.framework.AbstractRPMObjectManager.executeProcedure(AbstractRPMObjectManager.java:281)
    at com.ibm.rpm.workflow.managers.WorkflowManager.SP_A_WORKFLOW(WorkflowManager.java:1089)
    at com.ibm.rpm.workflow.managers.WorkflowManager.startProcess(WorkflowManager.java:964)
    at com.ibm.rpm.interfaces.impl.ManagerCaller.startWorkflow(ManagerCaller.java:870)
    at com.ibm.rpm.interfaces.Application.startWorkflow(Application.java:143)
     */
    public void testCanManageManualWorkflows() throws Exception
    {
        anIssue = (Issue)prepareContainer(new Issue(), projectScopeFolder, getWorkflowScope(), 
            securityTestUser, issueSecurityRole);
        
        associateWorkflowProcessToProject();
        
        anIssue = (Issue) loadFromXpath("/Issue[id='" + anIssue.getID() + "']", getWorkflowScope())[0];
        assertNotNull(anIssue.getWorkflowProcessesAvailable());
        assertNotNull(anIssue.getWorkflowProcessesAvailable()[0]);
        assertNotNull(anIssue.getWorkflowProcessesAvailable()[0].getWorkflowProcess());
        WorkflowProcess process = anIssue.getWorkflowProcessesAvailable()[0].getWorkflowProcess();
        
        flagOn(anIssue, "CanStartManualWorkflow");
        workflowAssigner.startProcess(process, anIssue, getWorkflowScope().getWorkflowProcessesRunning());
    }

    /**
     * TODO: andreit June 7 - report flag can be set programatically, even 
     * when the whole portlet is not editable in the delphi client.
     * Reproduced again on June 19
     * 
     * @throws Exception
     */
    public void testCanEditAssignedScorecard() throws Exception
    {
        /* Prepare test environment*/
        assignScorecardToDefect();
        
        /* Disallow the operation */
        flagOff(aDefect, "canEditAssignedScorecard");
        
        // the flags are related only in the delphi client.
        flagOff(aDefect, "canAddRemoveScorecards");

        /* Try to read the Assigned Scorecards */
        AssignedScorecard sc = 
            reloadDefectWithAssignedScorecards().getAssignedScorecard();

        sc.getAssignedScorecardCategories()[0].getAssignedScorecardQuestions()[0].setToReport(Boolean.TRUE);
        ScorecardScope scorecardScope = PublishedScorecardFixture
          .createScorecardScope(defectScope);
        saveAndValidate(sc, scorecardScope, false);
    }

    /**
     * Allows to view the Scorecard portlet in the Scope description view.
     */
    public void testCanViewAssignedScorecard() throws Exception
    {
        /* Prepare test environment*/
        assignScorecardToDefect();
        
        /* Disallow the operation */
        flagOff(aDefect, "canViewAssignedScorecard");
        
        /* Try to read the Assigned Scorecards */
        assertNull("Could read the scorecard!", 
            reloadDefectWithAssignedScorecards().getAssignedScorecard());
        
        /* Allow the operation */
        flagOn(aDefect, "canViewAssignedScorecard");
       
        assertNotNull("Could read the scorecard!", 
            reloadDefectWithAssignedScorecards().getAssignedScorecard());
    }
    
    /**
     * TODO: andreit June 13 - DB error 433 is thrown every time by an SP. Tried both with and without
     * security.
     * 
     * Allows to start a Workflow. Without this right the Workflow list in the
     * element's context menu will be empty even if some are assigned. Workflows
     * that are started using the API are considered manual-workflow (i.e., user
     * initiated as opposed to trigger initiated in which case it will be
     * considered scheduled-workflow
     * 
     */
    public void testCanStartManualWorkflow() throws Exception 
    {
        anIssue = (Issue)prepareContainer(new Issue(), projectScopeFolder, getWorkflowScope(), 
            securityTestUser, issueSecurityRole);
        
        associateWorkflowProcessToProject();
        
        anIssue = (Issue) loadFromXpath("/Issue[id='" + anIssue.getID() + "']", getWorkflowScope())[0];
        assertNotNull(anIssue.getWorkflowProcessesAvailable());
        assertNotNull(anIssue.getWorkflowProcessesAvailable()[0]);
        assertNotNull(anIssue.getWorkflowProcessesAvailable()[0].getWorkflowProcess());
        WorkflowProcess process = anIssue.getWorkflowProcessesAvailable()[0].getWorkflowProcess();
        
        issueSecurityRole.setCanStartManualWorkflow(Boolean.TRUE);
        enclosingProjectRole.setCanStartManualWorkflow(Boolean.TRUE);
        updateSecurityRole(issueSecurityRole);
        updateSecurityRole(enclosingProjectRole);
        
        workflowAssigner.startProcess(process, anIssue, getWorkflowScope().getWorkflowProcessesRunning());
    }
    
    private ScopeElementScope getWorkflowScope() throws Exception
    {
        WorkflowScope workflowScope = new WorkflowScope();
        workflowScope.setParent(new WorkflowScope());
        workflowScope.setChildren(new WorkflowScope());
        
        RunningWorkflowProcessScope runningScope = new RunningWorkflowProcessScope();
        runningScope.setChildren(workflowScope);
                
        AvailableWorkflowProcessScope availableScope = new AvailableWorkflowProcessScope();
        availableScope.setChildren(workflowScope);

        ScopeElementScope scopeElementScope = new ScopeElementScope();
        scopeElementScope.setWorkflowProcessesAvailable(availableScope);
        scopeElementScope.setWorkflowProcessesRunning(runningScope);
        
        return scopeElementScope;
    }
    
    private void associateWorkflowProcessToProject() throws Exception
    {
        WorkflowScope workflowScope = new WorkflowScope();
        workflowScope.setParent(new WorkflowScope());
        workflowScope.setChildren(new WorkflowScope());

        RPMObject[] objects = loadFromXpath("/WorkflowProcess[name='"
                + PROCESS_NAME + "']", workflowScope);
        assertNotNull(objects[0]);
        WorkflowProcess process = (WorkflowProcess) objects[0];

        workflowAssigner.associateProcess(process, parentProject, false);
    }

    private Defect reloadDefectWithAssignedScorecards() throws Exception
    {
        /* Try to read the Assigned Scorecards */
        ScorecardScope scorecardScope = PublishedScorecardFixture
        .createScorecardScope(defectScope);
        defectScope.setAssignedScorecard(scorecardScope);
        
        Defect myDefect = (Defect) loadFromID(aDefect, defectScope);
        
        return myDefect;
    }

    private AssignedScorecard assignScorecardToDefect() throws Exception
    {
        ScorecardScope scorecardScope = PublishedScorecardFixture
            .createScorecardScope(defectScope);
        defectScope.setAssignedScorecard(scorecardScope);
    
        ScorecardScopeValidationUtility util = new ScorecardScopeValidationUtility();
        ScopeManagementModule module = util.createModule();
        Scorecard scorecardToAssign = module.getScorecardFolders()[0].getScorecards()[0];
        
        aDefect =  (Defect)prepareContainer(new Defect(), projectScopeFolder, defectScope, 
            securityTestUser, defectSecurityRole);
    
        SaveResult result = saveWithoutDelete(aDefect, defectScope);
        assertNull(result.getErrors());
        assertTrue(result.isSuccessful());
    
        aDefect = (Defect) (getApplicationInterface()
                .assignScoreCard(getSessionID(), aDefect, defectScope,
                    scorecardToAssign,
                    ReloadType.ReloadResult)).getRpmObject();
        
        AssignedScorecard scorecard = aDefect.getAssignedScorecard();
        assertNotNull("Scorecard was not created succesfully", scorecard);
        
        return scorecard;
    }
    
    
    /**
     * Three flags are interdependent to dissalow viewing : Attributes (view/edit/add+remove)
     * if only view is turned off, we can successfully read the information from the database
     *  
     * Allows to view the Attributes portlet in the Project, Deliverable, Task,  
     * Scope, Document and Notes/Minutes description view.
     * 
     * It protects attributeAssignments of AggregateNode, WorkElement
     * against illicit loading.
     *
     * @throws Exception
     */
    public void testCanViewAttributesWithWorkElement() throws Exception
    {
        /* Prepare test environment*/
        taskScope.setAttributeAssignments(true);
        aTask = (Task) prepareContainer(new Task(), parentProject, taskScope, securityTestUser, taskSecurityRole);
        
        AttributeAssignment attributeassignment = AttributeAssignmentFixture.createAssignmentFixture(aTask, this);
        Attribute mainAttribute = (Attribute) attributeassignment.getAttribute();
        mainAttribute.setInputType(InputType.Text);
        final String attributeValue = HelperTestUtilities.createName();
        attributeassignment.setValue(attributeValue);
        
        aTask.setAttributeAssignments(new AttributeAssignment[] {
            attributeassignment
        });
        
        assertTrue(aTask.getAttributeAssignments()[0] != null);
        
        aTask = (Task) save(aTask, taskScope).getRpmObject();
        aTask = (Task) loadFromID(aTask, taskScope);
        
        assertTrue(aTask.getAttributeAssignments().length > 0);
        
        /* Remove the right to viewAttributes */
        flagOff(aTask, "canViewAttributes");
        flagOff(aTask, "canEditAttributes");
        flagOff(aTask, "canAddRemoveAttributes");

        /* Try to read the attribute assignments */
        Task reloadedTask = (Task)loadFromID(aTask, taskScope);
        assertTrue("Could read the attribute assignments whereas we shouldn't be able to", 
            reloadedTask.getAttributeAssignments() == null);
        reloadedTask = null;
        
        /* Allow viewClientName */
        flagOn(aTask, "canViewAttributes");
        
        /* Allow */
        reloadedTask = (Task)loadFromID(aTask, taskScope);
        assertTrue("Could read the attribute assignments whereas we shouldn't be able to", 
            reloadedTask.getAttributeAssignments()[0] != null);
    }

    /**
     * Three flags are interdependent to dissalow viewing : Attributes (view/edit/add+remove)
     * if only view is turned off, we can successfully read the information from the database
     *  
     * Allows to view the Attributes portlet in the Project, Deliverable, Task,  
     * Scope, Document and Notes/Minutes description view.
     * 
     * It protects attributeAssignments of AggregateNode, WorkElement
     * against illicit loading.
     *
     * @throws Exception
     */
    public void testCanViewAttributesWithAggregateNode() throws Exception
    {
        /* Prepare test environment*/
        defectScope.setAttributeAssignments(true);
        aDefect = (Defect)prepareContainer(new Defect(), projectScopeFolder, defectScope, 
            securityTestUser, defectSecurityRole);
        
        AttributeAssignment attributeassignment = AttributeAssignmentFixture.createAssignmentFixture(aDefect, this);
        Attribute mainAttribute = (Attribute) attributeassignment.getAttribute();
        mainAttribute.setInputType(InputType.Text);
        final String attributeValue = HelperTestUtilities.createName();
        attributeassignment.setValue(attributeValue);
        
        aDefect.setAttributeAssignments(new AttributeAssignment[] {
                attributeassignment
        });
        
        assertTrue(aDefect.getAttributeAssignments()[0] != null);
        aDefect = (Defect) save(aDefect, defectScope).getRpmObject();
        aDefect = (Defect) loadFromID(aDefect, defectScope);
        assertTrue(aDefect.getAttributeAssignments().length > 0);        
        
        /* Remove the right to viewAttributes */
        flagOff(aDefect, "canViewAttributes");
        
        /* Try to read the attribute assignments */
        Defect reloadedDefect = (Defect)loadFromID(aDefect, defectScope);
        assertTrue("Could read the attribute assignments whereas we shouldn't be able to", 
            reloadedDefect.getAttributeAssignments() == null);
        reloadedDefect = null;
        
        /* Allow viewClientName */
        flagOn(aDefect, "canViewAttributes");
        
        /* Try to read the attribute assignments */
        reloadedDefect = (Defect)loadFromID(aDefect, defectScope);
        assertTrue(reloadedDefect.getAttributeAssignments() != null &&
            reloadedDefect.getAttributeAssignments().length == 1 && 
            reloadedDefect.getAttributeAssignments()[0].getValue().equals(attributeValue));
    }
}
