package com.ibm.rpm.client.junit.security.testcases.role.fixes;

/**
 * @author Sachin Nayak
 */
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.RpmTestProperties;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.security.testcases.role.BaseRoleSecurityTestLocal;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.scope.PortfolioScope;

public class Test65063Local extends BaseRoleSecurityTestLocal {
	
	private Portfolio portfolios[];
	private Portfolio newPortfolios[];
	private Pool parentPool;
	private Resource resource;
	
	protected void setUp() throws Exception
	{
		super.setUp();
	}
	private Portfolio[] createPortfolios() throws Exception
	{
		Portfolio pfs[] = new Portfolio[2];
		PortfolioScope portfolioScope = new PortfolioScope();
		portfolioScope.setParent(new PortfolioScope());
		portfolioScope.setPortfolios(new PortfolioScope());
		for (int i=0; i < pfs.length ; i++)
		{
			pfs[i] = PortfolioFixture.createFixture(1, 0, portfolioScope, this);
			pfs[i] = (Portfolio)save(pfs[i], portfolioScope).getRpmObject();
			assertNotNull(pfs[i]);
			assertNotNull(pfs[i].getPortfolios());
			assertNotNull(pfs[i].getPortfolios()[0]);
		}
		return pfs;
	}
	private Resource createNewUser() throws Exception
	{		
		parentPool = PoolFixture.createSaveFixture(this);
		ResourceScope resourceScope = new ResourceScope();
		resourceScope.setCalendar(new CalendarScope());
		resourceScope.setParent(new PoolScope());
		Resource resource = ResourceFixture.createFixture(parentPool, 1, resourceScope,this);
		ResourceSecurityRights rsr = new ResourceSecurityRights();
		rsr.setCanUseWebServicesAPI(new Boolean(true));
		resource.setResourceSecurityRights(rsr);
		resourceScope.setResourceSecurityRights(true);
		resource.setPassword("rpm");
		SingleResult result = save(resource, resourceScope);
		resource = (Resource) result.getRpmObject();
		resource = (Resource)loadFromID(resource, resourceScope);
		changePassword(resource.getUserName(), "rpm", "ibm");

		return resource;
	}

	public void testLoadPortfolios() throws Exception
	{
		//Create Some Portfolios using the Test User Credentials
		portfolios = createPortfolios();
		
		//Create a new User, login and create portfolios with the new user credentials
		resource = createNewUser();
		logoutAndLoginAs(resource.getUserName(), "ibm");
		newPortfolios = createPortfolios();
		HashSet newPortfolioIDs = new HashSet();
		for (int i=0; i < newPortfolios.length ; i++)
		{
			newPortfolioIDs.add(newPortfolios[i].getID());
			newPortfolioIDs.add(newPortfolios[i].getPortfolios()[0].getID());
		}
		
		//Try to load the portfolios, it should load only current users portfolios
		String xPath = "/Portfolio";
		PortfolioScope portfolioScope = new PortfolioScope();
		portfolioScope.setParent(new PortfolioScope());
		portfolioScope.setPortfolios(new PortfolioScope());
		RPMObject rpmObjects[] = loadFromXpath(xPath, portfolioScope);
		assertNotNull(rpmObjects);
		assertEquals(rpmObjects.length, portfolios.length * 2);
		for(int i=0; i < rpmObjects.length ; i++)
		{
			assertNotNull(rpmObjects[i]);
			assertTrue(rpmObjects[i] instanceof Portfolio);
			assertTrue(newPortfolioIDs.contains(rpmObjects[i].getID()));
		}
		
		//Delete the portfolios created by the new user and change back to Test User
		for(int i=0; i < rpmObjects.length ; i++)
		{
			if(((Portfolio)rpmObjects[i]).getPortfolios() != null)
			{
				delete(rpmObjects[i]);
			}
		}
		logoutAndLoginAs(getSecurityTestUser().getUserName(),RpmTestProperties
	            .getProperty("rpm.test.security.user.password") );
	}
}
