/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases.role.fixes;

import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.security.testcases.role.BaseRoleSecurityTestLocal;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * @author Yasser Lulu
 *
 */
public class Test65218Local extends BaseRoleSecurityTestLocal
{

    public void testUpdateOnCreate() throws Exception
    {

        WorkElementScope projectScope = new WorkElementScope();
        ProjectSecurityRole projectSecurityRole = (ProjectSecurityRole) prepareSecurityRole(
            Project.class, true);
        projectSecurityRole.setAllDocumentPermissionsGranted(Boolean.FALSE);
        projectSecurityRole
            .setAllScopeElementsPermissionsGranted(Boolean.FALSE);
        projectSecurityRole = (ProjectSecurityRole) updateSecurityRole(projectSecurityRole);
        GenericProject testProject = (GenericProject) prepareContainer(
            new Project(), null, projectScope, securityTestUser,
            projectSecurityRole);

        ScopeElementScope scope = new ScopeElementScope();
        ScopeElementScope childrenScope = new ScopeElementScope();
        childrenScope.setRtfAssignments(true);
        scope.setChildren(childrenScope);
        scope.setProject(new WorkElementScope());

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + testProject.getName() + "']", new ScopeElementScope());
        ScopeFolder folder = ScopeElementFixture.createFolderFixture(scope, 0,
            (ScopeFolder) rpmObjects[0], this);

        ServiceRequest serviceRequest = ScopeElementFixture
            .createServiceRequestFixture(null, 0, folder, this);
        serviceRequest.setDefaultScope(null);

        folder.setChildren(new ScopeElement[]
        {
            serviceRequest
        });
        folder = (ScopeFolder) save(folder, scope)
            .getRpmObject();
        assertNotNull(folder);
        addObjectToDelete(folder);

        serviceRequest = (ServiceRequest) folder.getChildren()[0];

        // publish proposal to team
        Proposal proposal = (Proposal) folder.getProject();
        proposal.setPublished(ProjectPublishedType.ToTeam);
        proposal = (Proposal)save(proposal, null).getRpmObject();
        assertNotNull(proposal);

        // transfert unde wbs root
        Result result = transferUnderWBS((WorkElement) null, serviceRequest);
        assertTrue(result.isSuccessful());

        // get the associated task
        SummaryTask summaryTask = new SummaryTask();
        summaryTask.setID(serviceRequest.getID());
        WorkElementScope taskScope = new WorkElementScope();

        // Contaning project is mandatory to create a ResourceTaskAssignment
        taskScope.setContainingProject(new WorkElementScope());

        summaryTask = (SummaryTask) loadFromID(summaryTask, taskScope);
        assertNotNull(summaryTask);        
    }

}