package com.ibm.rpm.client.junit.security.testcases.role.scopemanagement;

import java.util.Calendar;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.Impact;
import com.ibm.rpm.scopemanagement.containers.OrderOfMagnitudeCostBenefits;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.security.containers.AbstractScopeSecurityRole;

public class AbstractTestScopeSecurityTestLocal extends
		AstractTestAggregateScopeSecurityTestLocal {

	protected void setUp() throws Exception
    {
        super.setUp();
    }

	protected void runEditState() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;

		/* While testing for state please add the createState method in the DataFieldFixture */
		 /* for the respective container that is being tested */
		DatafieldState closedState = DatafieldFixture.createClosedState(testScopeElement);
		abstractScopeElement.setState(closedState);

		runUpdatesFor(ABSTRACTSCOPE_STATE);

		closedState = ((AbstractScope)testScopeElement).getState();
		assertNotNull(closedState);
		assertTrue(closedState.getClosedState().booleanValue());
		assertEquals(DatafieldFixture.REQUIREMENT_STATE_CLOSED, closedState.getID());
	}

	protected void runViewExternalReference() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;
		String externalReference = "_MOD";
		if(abstractScopeElement.getExternalReferenceNumber() != null)
		{
			externalReference = abstractScopeElement.getExternalReferenceNumber() + externalReference;
		}
		abstractScopeElement.setExternalReferenceNumber(externalReference);

		updateSaveScopeElement(true);

		runLoadValueFor(ABSTRACTSCOPE_EXTERNAL_REFERENCE);

		assertEquals(externalReference, ((AbstractScope)testScopeElement).getExternalReferenceNumber().trim());
	}

	protected void runEditExternalReference() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;
		String externalReference = "_MOD";
		if(abstractScopeElement.getExternalReferenceNumber() != null)
		{
			externalReference = abstractScopeElement.getExternalReferenceNumber() + externalReference;
		}
		abstractScopeElement.setExternalReferenceNumber(externalReference);

		runUpdatesFor(ABSTRACTSCOPE_EXTERNAL_REFERENCE);

		assertEquals(externalReference, ((AbstractScope)testScopeElement).getExternalReferenceNumber().trim());
	}

	protected void runViewTargetDate() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;

		Calendar targetDate = modifyDate(abstractScopeElement.getTargetDate());
		abstractScopeElement.setTargetDate(targetDate);
		updateSaveScopeElement(true);

		runLoadValueFor(ABSTRACTSCOPE_TARGETDATE);
		assertNotNull(((AbstractScope)testScopeElement).getTargetDate());
		validateDate(targetDate, ((AbstractScope)testScopeElement).getTargetDate());
	}

	protected void runEditTargetDate() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;

		Calendar targetDate = modifyDate(abstractScopeElement.getTargetDate());
		abstractScopeElement.setTargetDate(targetDate);

		runUpdatesFor(ABSTRACTSCOPE_TARGETDATE);

		assertNotNull(((AbstractScope)testScopeElement).getTargetDate());
		validateDate(targetDate, ((AbstractScope)testScopeElement).getTargetDate());
	}

	protected void runViewApprovedBy() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;
		
		String approvedByName = modifyString(abstractScopeElement.getApprovedByName());
		Calendar approvedByDate = modifyDate(abstractScopeElement.getApprovedByDate());
		abstractScopeElement.setApprovedByDate(approvedByDate);
		abstractScopeElement.setApprovedByName(approvedByName);

		updateSaveScopeElement(true);

		runLoadValueFor(ABSTRACTSCOPE_APPROVEDBY);

		assertNotNull(((AbstractScope)testScopeElement).getApprovedByDate());
		validateDate(approvedByDate, ((AbstractScope)testScopeElement).getApprovedByDate());
		assertEquals(approvedByName, ((AbstractScope)testScopeElement).getApprovedByName());
	}

	protected void runEditApprovedBy() throws Exception
	{
		runEditApprovedByName();

		runEditApprovedByDate();
	}

	protected void runEditQuantificationImpact() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;

		Impact impact = new Impact();
        HelperTestUtilities.setByIntrospection(impact,
        		RPMObjectFixture.random(), RPMObjectFixture.skipSetters);
        abstractScopeElement.setImpact(impact);

        runUpdatesFor(ABSTRACTSCOPE_IMPACT);

        assertNotNull(((AbstractScope)testScopeElement).getImpact());
	}

	protected void runEditRank() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;

		int modifiedRank = 500;
		if(abstractScopeElement.getRank() != null)
		{
			modifiedRank = abstractScopeElement.getRank().intValue() + 1;
		}
		abstractScopeElement.setRank(new Integer(modifiedRank));

		runUpdatesFor(ABSTRACTSCOPE_RANK);

		assertEquals(new Integer(modifiedRank), ((AbstractScope)testScopeElement).getRank());
	}

	protected void runEditOrderOfMagnitudeCostBenefits() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;

		OrderOfMagnitudeCostBenefits orderOfMagnitudeCostBenefits = new OrderOfMagnitudeCostBenefits();
        HelperTestUtilities.setByIntrospection(
            orderOfMagnitudeCostBenefits, RPMObjectFixture.random(), RPMObjectFixture.skipSetters);
        abstractScopeElement.setOrderOfMagnitudeCostBenefits(orderOfMagnitudeCostBenefits);

        runUpdatesFor(ABSTRACTSCOPE_ORDEROF_MAGNITUDE_COSTBENEFITS);

        assertNotNull(((AbstractScope)testScopeElement).getOrderOfMagnitudeCostBenefits());
	}

	protected void runEditScorecardAssignments() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;

		PublishedScorecardFixture.updateAssignment(10, abstractScopeElement.getAssignedScorecard());

		runUpdatesFor(ABSTRACTSCOPE_SCORECARD_ASSIGNMENTS);
	}
	
	protected void runViewDuplicates(Duplicate duplicate) throws Exception
	{
		Duplicate duplicates[] = new Duplicate[] {duplicate}; 
		((AbstractScope)testScopeElement).setDuplicates(duplicates);
		updateSaveScopeElement(true);
		
		runLoadValueFor(ABSTRACTSCOPE_DUPLICATES);
		
		Duplicate savedDuplicates[] = ((AbstractScope)testScopeElement).getDuplicates(); 
		assertNotNull(savedDuplicates);
		assertTrue(savedDuplicates.length == 1);
		assertNotNull(savedDuplicates[0]);
		assertNotNull(((AbstractScope)testScopeElement).getDuplicateNumber());
	}
	
	protected void runEditDuplicates(Duplicate duplicate) throws Exception
	{
		runEditDuplicatesWithField(duplicate,ABSTRACTSCOPE_DUPLICATES);
	}
	
	protected void runEditDuplicateNumbers(Duplicate duplicate) throws Exception
	{
		runEditDuplicatesWithField(duplicate,ABSTRACTSCOPE_DUPLICATENUMBER);
	}
	
	protected void runEditDefaultScope() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;
		
		Boolean modifiedDefaultScope = modifyBooleanValue(abstractScopeElement.getDefaultScope());
		abstractScopeElement.setDefaultScope(modifiedDefaultScope);
		
		runUpdatesFor(ABSTRACTSCOPE_DEFAULTSCOPE);
		
		assertEquals(modifiedDefaultScope, ((AbstractScope)testScopeElement).getDefaultScope());
	}
	
	private void runEditApprovedByName() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;
		
		String approvedByName = modifyString(abstractScopeElement.getApprovedByName());
		abstractScopeElement.setApprovedByName(approvedByName);

		runUpdatesFor(ABSTRACTSCOPE_APPROVEDBY);

		assertEquals(approvedByName, ((AbstractScope)testScopeElement).getApprovedByName());
	}

	private void runEditApprovedByDate() throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;

		Calendar approvedByDate = modifyDate(abstractScopeElement.getApprovedByDate());
		abstractScopeElement.setApprovedByDate(approvedByDate);

		runUpdatesFor(ABSTRACTSCOPE_APPROVEDBY);

		assertNotNull(((AbstractScope)testScopeElement).getApprovedByDate());
		validateDate(approvedByDate, ((AbstractScope)testScopeElement).getApprovedByDate());
	}
	
	private void runEditDuplicatesWithField(Duplicate duplicate, int duplicateFieldType) throws Exception
	{
		Duplicate duplicates[] = new Duplicate[] {duplicate}; 
		((AbstractScope)testScopeElement).setDuplicates(duplicates);
		
		runUpdatesFor(duplicateFieldType);
		
		Duplicate savedDuplicates[] = ((AbstractScope)testScopeElement).getDuplicates(); 
		assertNotNull(savedDuplicates);
		assertTrue(savedDuplicates.length == 1);
		assertNotNull(savedDuplicates[0]);
		assertNotNull(((AbstractScope)testScopeElement).getDuplicateNumber());
	}
	
	protected void loadOrUpdateField(int fieldType, boolean isSecurityFlagOn, boolean isLoad, boolean found) throws Exception
	{
		switch(fieldType)
		{
			case ABSTRACTSCOPE_STATE:
				if(isLoad)
					throw new Exception("loadOrUpdateField: load Operation check is not enabled on the field: 'State'");
				else
					scopeElementSecurityRole.setCanEditState(new Boolean(isSecurityFlagOn));
				found = true;
				break;
			case ABSTRACTSCOPE_EXTERNAL_REFERENCE:
				if(isLoad)
				{
					scopeElementSecurityRole.setCanViewExternalReference(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						scopeElementSecurityRole.setCanEditExternalReference(new Boolean(isSecurityFlagOn));
				}
				else
				{
					scopeElementSecurityRole.setCanEditExternalReference(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						scopeElementSecurityRole.setCanViewExternalReference(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
			case ABSTRACTSCOPE_TARGETDATE:
				if(isLoad)
				{
					scopeElementSecurityRole.setCanViewTargetDate(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						scopeElementSecurityRole.setCanEditTargetDate(new Boolean(isSecurityFlagOn));
				}
				else
				{
					scopeElementSecurityRole.setCanEditTargetDate(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						scopeElementSecurityRole.setCanViewTargetDate(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
			case ABSTRACTSCOPE_APPROVEDBY:
				if(isLoad)
				{
					scopeElementSecurityRole.setCanViewApprovedBy(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						scopeElementSecurityRole.setCanEditApprovedBy(new Boolean(isSecurityFlagOn));
				}
				else
				{
					scopeElementSecurityRole.setCanEditApprovedBy(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						scopeElementSecurityRole.setCanViewApprovedBy(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
			case ABSTRACTSCOPE_IMPACT:
				if(isLoad)
				{
					scopeElementSecurityRole.setCanViewQuantificationImpact(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						scopeElementSecurityRole.setCanEditQuantificationImpact(new Boolean(isSecurityFlagOn));
				}
				else
				{
					scopeElementSecurityRole.setCanEditQuantificationImpact(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						scopeElementSecurityRole.setCanViewQuantificationImpact(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
			case ABSTRACTSCOPE_RANK:
				if(isLoad)
				{
					scopeElementSecurityRole.setCanViewRank(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						scopeElementSecurityRole.setCanEditRank(new Boolean(isSecurityFlagOn));
				}
				else
				{
					scopeElementSecurityRole.setCanEditRank(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						scopeElementSecurityRole.setCanViewRank(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
			case ABSTRACTSCOPE_ORDEROF_MAGNITUDE_COSTBENEFITS:
				if(isLoad)
				{
					scopeElementSecurityRole.setCanViewOrderOfMagnitudeCostBenefits(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						scopeElementSecurityRole.setCanEditOrderOfMagnitudeCostBenefits(new Boolean(isSecurityFlagOn));
				}
				else
				{
					scopeElementSecurityRole.setCanEditOrderOfMagnitudeCostBenefits(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						scopeElementSecurityRole.setCanViewOrderOfMagnitudeCostBenefits(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
			case ABSTRACTSCOPE_SCORECARD_ASSIGNMENTS:
				if(isLoad)
				{
					scopeElementSecurityRole.setCanViewAssignedScorecard(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						scopeElementSecurityRole.setCanEditAssignedScorecard(new Boolean(isSecurityFlagOn));
				}
				else
				{
					scopeElementSecurityRole.setCanEditAssignedScorecard(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						scopeElementSecurityRole.setCanViewAssignedScorecard(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
			case ABSTRACTSCOPE_DUPLICATENUMBER:
				if(isLoad)
				{
					scopeElementSecurityRole.setCanViewDuplicateNumber(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						scopeElementSecurityRole.setCanEditDuplicateNumbers(new Boolean(isSecurityFlagOn));
				}
				else
				{
					throw new Exception("loadOrUpdateField: Edit Operation check is invalid on the ReadOnly field: 'Duplicate Number'");
				}
				found = true;
				break;
			case ABSTRACTSCOPE_DUPLICATES:
				if(isLoad)
				{
					scopeElementSecurityRole.setCanViewDuplicates(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						scopeElementSecurityRole.setCanAddRemoveDuplicates(new Boolean(isSecurityFlagOn));
				}
				else
				{
					scopeElementSecurityRole.setCanAddRemoveDuplicates(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						scopeElementSecurityRole.setCanViewDuplicates(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
			case ABSTRACTSCOPE_DEFAULTSCOPE:
				if(isLoad)
					throw new Exception("loadOrUpdateField: load Operation check is not enabled on the field: 'Default Scope'");
				else
					scopeElementSecurityRole.setCanSetAsOrCancelDefault(new Boolean(isSecurityFlagOn));
				found = true;
				break;
		}
		super.loadOrUpdateField(fieldType,isSecurityFlagOn,isLoad,found);
	}

	protected void checkNullValueFor(int fieldType, boolean found, Object value[]) throws Exception
	{
		AbstractScope abstractScopeElement = (AbstractScope)testScopeElement;
		switch(fieldType)
		{
			case ABSTRACTSCOPE_EXTERNAL_REFERENCE:
				value = new Object[1];
				value[0] = abstractScopeElement.getExternalReferenceNumber();
				found = true;
				break;
			case ABSTRACTSCOPE_TARGETDATE:
				value = new Object[1];
				value[0] = abstractScopeElement.getTargetDate();
				found = true;
				break;
			case ABSTRACTSCOPE_APPROVEDBY:
				value = new Object[2];
				value[0] = abstractScopeElement.getApprovedByDate();
				value[1] = abstractScopeElement.getApprovedByName();
				found = true;
				break;
			case ABSTRACTSCOPE_IMPACT:
				value = new Object[1];
				value[0] = abstractScopeElement.getImpact();
				found = true;
				break;
			case ABSTRACTSCOPE_RANK:
				value = new Object[1];
				value[0] = abstractScopeElement.getRank();
				found = true;
				break;
			case ABSTRACTSCOPE_ORDEROF_MAGNITUDE_COSTBENEFITS:
				value = new Object[1];
				value[0] = abstractScopeElement.getOrderOfMagnitudeCostBenefits();
				found = true;
				break;
			case ABSTRACTSCOPE_SCORECARD_ASSIGNMENTS:
				value = new Object[1];
				value[0] = abstractScopeElement.getAssignedScorecard();
				found = true;
				break;
			case ABSTRACTSCOPE_DUPLICATENUMBER:
				value = new Object[1];
				value[0] = abstractScopeElement.getDuplicateNumber();
				found = true;
				break;
			case ABSTRACTSCOPE_DUPLICATES:
				value = abstractScopeElement.getDuplicates();
				found = true;
				break;
		}
		super.checkNullValueFor(fieldType, found, value);
	}

	/* OPERATION SECURITY FLAGS */
	protected void runCanTransferScopeElementsToWBSView() throws Exception
    {
		// Whether the access for breaking down WBS elements is required?
    	testScopeElement.setProject(parentProject);

		scopeElementSecurityRole.setCanTransferScopeElementsToWBSView(new Boolean(false));
    	scopeElementSecurityRole = (AbstractScopeSecurityRole)updateSecurityRole(scopeElementSecurityRole);

    	/* THIS OPERATION SHOULD FAIL */
    	transferUnderWBS((AbstractScope)testScopeElement, false);

    	scopeElementSecurityRole.setCanTransferScopeElementsToWBSView(new Boolean(true));
    	scopeElementSecurityRole = (AbstractScopeSecurityRole)updateSecurityRole(scopeElementSecurityRole);

    	/* THIS OPERATION SHOULD SUCCEED */
    	transferUnderWBS((AbstractScope)testScopeElement, true);
    }

	protected void createSaveScopeElementWithScorecardAssignments(AbstractScope abstractScopeElement) throws Exception
	{
		createSaveScopeElementWithSecurity(abstractScopeElement);

		scopeElementScope.setAssignedScorecard(PublishedScorecardFixture
	                .createScorecardScope(scopeElementScope));

		prepareScorecardAssignment(ReloadType.ReloadResult);

		assertNotNull(testScopeElement);
		assertNotNull(((AbstractScope)testScopeElement).getAssignedScorecard());
	}
	
	protected Duplicate createSaveDuplicate(AbstractScope abstractScope) throws Exception
	{
		abstractScope.setName(HelperTestUtilities.createName());
		abstractScope.setParent(parent);
		abstractScope = (AbstractScope)save(abstractScope, null).getRpmObject();
    	assertNotNull(abstractScope);
    	
    	Duplicate duplicate = new Duplicate();
    	duplicate.setScopeElement(abstractScope);
    	duplicate.setParent(testScopeElement);
    	return duplicate;
	}
	
	private void prepareScorecardAssignment(ReloadType reloadType) 
					throws Exception, AssertionError, RPMException, AssertionFailedError
	{
		ScorecardFolder scorecardFolder = ScorecardFixture.createSaveFixture(RPMObjectFixture.SCOPE_CONTEXT, this);

		testScopeElement = (AbstractScope) PublishedScorecardFixture
		.createAssignmentFixture(testScopeElement, scopeElementScope, this,
				reloadType, scorecardFolder, 1);
	}
}
