package com.ibm.rpm.client.junit.security.testcases.role.scopemanagement;

import com.ibm.rpm.client.junit.scopemanagement.fixtures.MitigationFactorFixture;
import com.ibm.rpm.scopemanagement.containers.MitigationFactor;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.RiskMatrix;
import com.ibm.rpm.scopemanagement.scope.MitigationFactorScope;
import com.ibm.rpm.scopemanagement.types.PrecisionType;
import com.ibm.rpm.security.containers.RiskSecurityRole;

/**
 *
 * @author SandeepS
 *
 */
public class TestRiskSecurityTestLocal extends AbstractTestScopeSecurityTestLocal {

	private final int RISK_MITIGATIONFACTORS = 0;
	private final int RISK_MATRIX = 1;

	protected void setUp() throws Exception
    {
		testClass = TEST_RISK;
        super.setUp();

        prepareSecurityRole(Risk.class);
    }

	public void testEditMitigationFactors() throws Exception
	{
		resetScopeElementScope();
		createSaveScopeElementWithSecurity(new Risk());
		scopeElementScope.setMitigationFactors(new MitigationFactorScope());

		MitigationFactor[] mitigationFactors = new MitigationFactor[1];
		mitigationFactors[0] = MitigationFactorFixture.createFixture((Risk)testScopeElement, this);
		((Risk)testScopeElement).setMitigationFactors(mitigationFactors);

		runUpdatesFor(RISK_MITIGATIONFACTORS);

		assertNotNull(testScopeElement);
		assertNotNull(((Risk)testScopeElement).getMitigationFactors());
		assertTrue(((Risk)testScopeElement).getMitigationFactors().length == 1);
		assertNotNull(((Risk)testScopeElement).getMitigationFactors()[0]);
	}

	public void testEditRiskMatrix() throws Exception
	{
		resetScopeElementScope();
		createSaveScopeElementWithSecurity(new Risk());
		scopeElementScope.setRiskMatrix(true);

		RiskMatrix riskMatrix = new RiskMatrix();
		riskMatrix.setImpact(new Integer(40));
		riskMatrix.setProbability(new Integer(80));
		riskMatrix.setPrecision(PrecisionType.Medium);
		riskMatrix.setConsequenceCost(new Double(56.78));
		((Risk)testScopeElement).setRiskMatrix(riskMatrix);

		runUpdatesFor(RISK_MATRIX);

		RiskMatrix actualRiskMatrix = ((Risk)testScopeElement).getRiskMatrix();
		assertNotNull(actualRiskMatrix);
		assertEquals(riskMatrix.getImpact(), actualRiskMatrix.getImpact());
        assertEquals(riskMatrix.getProbability(), actualRiskMatrix.getProbability());
        assertEquals(riskMatrix.getPrecision(), actualRiskMatrix.getPrecision());
        assertEquals(riskMatrix.getConsequenceCost(), actualRiskMatrix.getConsequenceCost());
	}

	public void testEditQuantificationImpact() throws Exception
    {
		resetScopeElementScope();
    	scopeElementScope.setImpact(true);

    	createSaveScopeElementWithSecurity(new Risk());

    	runEditQuantificationImpact();
    }

	protected void loadOrUpdateField(int fieldType, boolean isSecurityFlagOn, boolean isLoad, boolean found) throws Exception
	{
		RiskSecurityRole riskSecurityRole = (RiskSecurityRole)scopeElementSecurityRole;
		switch(fieldType)
		{
			case RISK_MITIGATIONFACTORS:
				if(isLoad)
				{
					riskSecurityRole.setCanViewMitigationFactors(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						riskSecurityRole.setCanEditMitigationFactors(new Boolean(isSecurityFlagOn));
				}
				else
				{
					riskSecurityRole.setCanEditMitigationFactors(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						riskSecurityRole.setCanViewMitigationFactors(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
			case RISK_MATRIX:
				if(isLoad)
				{
					riskSecurityRole.setCanViewRiskMatrix(new Boolean(isSecurityFlagOn));
					if(!isSecurityFlagOn)
						riskSecurityRole.setCanEditRiskMatrix(new Boolean(isSecurityFlagOn));
				}
				else
				{
					riskSecurityRole.setCanEditRiskMatrix(new Boolean(isSecurityFlagOn));
					if(isSecurityFlagOn)
						riskSecurityRole.setCanViewRiskMatrix(new Boolean(isSecurityFlagOn));
				}
				found = true;
				break;
		}
		super.loadOrUpdateField(fieldType, isSecurityFlagOn, isLoad, found);
	}

	protected void checkNullValueFor(int fieldType, boolean found, Object value[]) throws Exception
	{
		Risk risk= (Risk)testScopeElement;
		switch(fieldType)
		{
			case RISK_MITIGATIONFACTORS:
				value = new Object[1];
				value[0] = risk.getMitigationFactors();
				found = true;
				break;
			case RISK_MATRIX:
				value = new Object[1];
				value[0] = risk.getRiskMatrix();
				found = true;
				break;
		}
		super.checkNullValueFor(fieldType,found,value);
	}
}
