/**
 * @author gmeloche
 */
package com.ibm.rpm.client.junit.timesheet.fixtures;

import java.util.ArrayList;
import java.util.Calendar;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;

public class CreditFixture extends RPMObjectFixture

{
    protected static ArrayList localSkipSetters, localSkipGetters;

    static
    {
        // Setters
        localSkipSetters = new ArrayList();
        localSkipSetters.addAll(skipSetters);
        localSkipSetters.add("setAlwaysAvailable");

        // Getters
        localSkipGetters = new ArrayList();
        localSkipGetters.addAll(skipGetters);

    }

    public static Credit createSaveFixture(
            SummaryTimesheet creditWeekSummaryTimesheet, Calendar weekOf,
            CreditTimesheet timesheet, StepScope scope, ClientTestController ctc)
            throws Exception
    {
        Credit credit = createFixture(creditWeekSummaryTimesheet, weekOf,
            timesheet, scope, ctc);
        credit = (Credit) save(ctc, credit, scope);
        return credit;
    }

    /**
     * 
     * @param creditWeekSummaryTimesheet
     *            reference to a summaryTimesheet with approved time
     * @param scope
     *            the scope for the step
     * @param timesheets
     *            the timesheets for the step; it can be null
     * @param ctc
     *            the ClientTestController
     * @return
     * @throws Exception
     */
    public static Credit createFixture(
            SummaryTimesheet creditWeekSummaryTimesheet, Calendar weekOf,
            CreditTimesheet timesheet, StepScope scope, ClientTestController ctc)
            throws Exception
    {
        Calendar creditWeekOf = creditWeekSummaryTimesheet.getWeekOf();

        Credit credit = new Credit();

        credit.setTaskAssignment((TaskAssignment) creditWeekSummaryTimesheet
            .getTaskAssignment());
        HelperTestUtilities.setByIntrospection(credit, random(),
            localSkipSetters);

        // timesheet is a mandatory field
        if (timesheet == null)
        {
            timesheet = new CreditTimesheet();
            timesheet = CreditTimesheetFixture.createFixture(credit, weekOf,
                creditWeekOf, null, ctc);
        }
        credit.setCreditTimesheet(timesheet);

        if (scope != null)
        {
            if (scope.isTimeCodes())
            {

                GenericProject project = StepFixture.loadProject(credit, ctc);
                credit = (Credit) StepFixture.assignTimeCodes(credit, project,
                    ctc);
            }
        }

        return credit;
    }

    public static Credit updateFixture(Credit credit, StepScope scope,
            ClientTestController ctc) throws Exception
    {
        HelperTestUtilities.setByIntrospection(credit, random(),
            localSkipSetters);

        // update timesheets
        if (scope != null)
        {
            /*
             * if(scope.isStepTimesheets() && credit.getStepTimesheets() != null &&
             * credit.getStepTimesheets().length != 0) { StepTimesheet[]
             * timesheets = credit.getStepTimesheets(); for (int i = 0; i <
             * timesheets.length; i++) timesheets[i] =
             * StepTimesheetFixture.updateFixture(timesheets[i]);
             * 
             * credit.setStepTimesheets(timesheets); }
             */

            if (scope.isTimeCodes())
            {
                GenericProject project = StepFixture.loadProject(credit, ctc);
                credit = (Credit) StepFixture.assignTimeCodes(credit, project,
                    ctc);
            }

        }

        return credit;
    }

    public static void compare(Credit[] array1, Credit[] array2, StepScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(array1, array2);
        if (array1 == null)
            return;

        for (int i = 0; i < array1.length; i++)
            CreditFixture.compare(array1[i], array2[i], scope);
    }

    public static void compare(Credit obj1, Credit obj2, StepScope scope)
            throws AssertionFailedError, Exception
    {

        RPMObjectFixture.compareWithSkip(obj1, obj2, localSkipGetters);

        if (scope != null)
        {
            if (scope.isTaskAssignment())
            {
                TaskAssignmentFixture.compare(obj1.getTaskAssignment(), obj1
                    .getTaskAssignment(), null);
            }
            if (scope.isTimesheets())
            {
                CreditTimesheetFixture.compare(obj1.getCreditTimesheet(), obj2
                    .getCreditTimesheet(), null);
            }
            if (scope.isTimeCodes())
            {
                TimecodeFixture.compare(obj1.getTimeCode1(), obj2
                    .getTimeCode1(), null);
                TimecodeFixture.compare(obj1.getTimeCode2(), obj2
                    .getTimeCode2(), null);
            }

            //Project Resource and WorlElement are not compared because they
            // are read-only fields
        }
    }
}
