package com.ibm.rpm.client.junit.timesheet.fixtures;

import java.util.ArrayList;
import java.util.Arrays;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.containers.PersonalProject;
import com.ibm.rpm.timesheet.containers.PersonalTask;
import com.ibm.rpm.timesheet.scope.PersonalProjectScope;

/**
 * @author gmeloche
 *  
 */
public class PersonalTaskFixture extends RPMObjectFixture
{

    protected static ArrayList localSkipSetters, localSkipGetters;

    static
    {

        //Setters
        localSkipSetters = new ArrayList();
        localSkipSetters.addAll(skipSetters);
        localSkipSetters.add("setEstimatedFinishDate");
        localSkipSetters.add("setEstimatedStartDate");
        localSkipSetters.add("setExternalID");
        localSkipSetters.add("setRank");

        localSkipGetters = new ArrayList();
        localSkipGetters.addAll(skipGetters);
        localSkipGetters.add("getEstimatedFinishDate");
        localSkipGetters.add("getEstimatedStartDate");
        localSkipGetters.add("getExternalID");
        localSkipGetters.add("getRank");

    }

    public static PersonalTask createSaveFixture(PersonalProject parent, ClientTestController ctc)
            throws Exception
    {

        PersonalTask adminTask = createFixture(parent, ctc);
        adminTask = (PersonalTask) save(ctc, adminTask, null);

        return adminTask;

    }

    public static PersonalTask createFixture(PersonalProject parent,
            ClientTestController ctc) throws Exception
    {

        PersonalTask persoTask = new PersonalTask();

        if (parent == null)
        {
            ResourceScope scope = new ResourceScope();
            scope.setCalendar(new CalendarScope());
            scope.setPersonalProject(new PersonalProjectScope());
            
            parent = ResourceFixture.createSaveFixture(scope, ctc).getPersonalProject();

        }

        persoTask.setParent(parent);
        HelperTestUtilities.setByIntrospection(persoTask, new Double(Math
            .random() * 100).intValue(), localSkipSetters);

        return persoTask;

    }

    public static PersonalTask updateFixture(PersonalTask adminTask,
            ClientTestController ctc) throws Exception
    {

        HelperTestUtilities.setByIntrospection(adminTask, new Double(Math
            .random() * 100).intValue(), localSkipSetters);

        return adminTask;

    }

    public static void compare(PersonalTask t1, PersonalTask t2)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(t1, t2, localSkipGetters);
    }

    public static void compare(PersonalTask[] pArray1,
            PersonalTask[] pArray2)
            throws AssertionFailedError, Exception
    {
        pArray1 = removeDeletedObject(pArray1);

        sortWithComparator(pArray1, RPMObjectNameAndIDComparator.INSTANCE);
        sortWithComparator(pArray2, RPMObjectNameAndIDComparator.INSTANCE);

        RPMObjectFixture.compare(pArray1, pArray2);
        if (pArray1 == null)
            return;

        for (int i = 0; i < pArray1.length; i++)
        {
            PersonalTaskFixture.compare(pArray1[i], pArray2[i]);
        }
    }

    public static PersonalTask[] removeDeletedObject(
            PersonalTask[] pArray)
    {

        PersonalTask[] newArray = null;

        ArrayList temp = new ArrayList(Arrays.asList(pArray));

        temp = removeNullAndDeletedRPMObjects(temp);

        if (temp != null && temp.size() > 0)
        {
            newArray = new PersonalTask[temp.size()];
            temp.toArray(newArray);
        }

        return newArray;

    }

}
