/**
 * Created on Dec 2, 2005
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.fixtures;

import java.util.ArrayList;
import java.util.Calendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.framework.testcases.util.TimeCodeUtil;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.timesheet.containers.GenericStep;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class StepFixture extends RPMObjectFixture
{
    protected static ArrayList localSkipSetters, localSkipGetters;

    static {
        // Setters
        localSkipSetters = new ArrayList();
        localSkipSetters.addAll(skipSetters);
        // These values have to be within a certain range      
        localSkipSetters.add("setWeight");
        localSkipSetters.add("setAlwaysAvailable");
        localSkipSetters.add("setEETC");
        // read-only fields
        localSkipSetters.add("setBillable");
        localSkipSetters.add("setETCChanged");
        localSkipSetters.add("setPercentageComplete");

        
        // Getters
        localSkipGetters = new ArrayList();
        localSkipGetters.addAll(skipGetters);
        // set in the SP call
        localSkipGetters.add("getBillable");
        localSkipGetters.add("getETCChanged");
        localSkipGetters.add("getPercentageComplete");
        //localSkipGetters.add("getWeight");
        
        // calculated fields
        localSkipGetters.add("getEETC");
        
    }
    
    public static Step createSaveFixture(TaskAssignment taskAssignment,
            StepScope scope, StepTimesheet[] timesheets,
            ClientTestController ctc) throws Exception
    {
        Step step = createFixture(taskAssignment, scope, timesheets, ctc);
        step = (Step) save(ctc, step, scope);
        return step;
    }
    
    /**
     * 
     * @param taskAssignment			reference to the taskAssignment of the step
     * @param scope						the scope for the step
     * @param timesheets				the timesheets for the step; it 
     *                                  can be null
     * @param ctc						the ClientTestController
     * @return
     * @throws Exception
     */
    public static Step createFixture(TaskAssignment taskAssignment,
            StepScope scope, StepTimesheet[] timesheets,
            ClientTestController ctc) throws Exception
    {
        Step step = new Step();
        step.setTaskAssignment(taskAssignment);
        
        HelperTestUtilities.setByIntrospection(step, random(), localSkipSetters);
        step.setWeight(new Double(56.8));
        step.setAlwaysAvailable(Boolean.TRUE);
        
        // timesheets is a mandatory field
        if (timesheets == null)
        {
            
            Calendar assignmentStartDate = taskAssignment
                .getAssignmentStartDate();

            Calendar week1 = TimesheetTestUtil.getValidWeekOf(
                assignmentStartDate, TimesheetTestUtil.TIMESHEET1_WEEK);
            Calendar week2 = TimesheetTestUtil.getValidWeekOf(
                assignmentStartDate, TimesheetTestUtil.TIMESHEET2_WEEK);
            
            timesheets = new StepTimesheet[2];
            timesheets[0] = StepTimesheetFixture.createFixture(step, week1,
                null, ctc);
            timesheets[1] = StepTimesheetFixture.createFixture(step, week2,
                null, ctc);
        }
        step.setStepTimesheets(timesheets);
        

        
        if (scope != null)
        {           
            if (scope.isTimeCodes())
            {

                GenericProject project = loadProject(step, ctc);
                step = (Step)assignTimeCodes(step, project, ctc);
            }
        }
        
        return step;
    }
        
    public static Step updateFixture(Step step, StepScope scope,
            ClientTestController ctc) throws Exception 
    {
        int index = random();
        HelperTestUtilities.setByIntrospection(step, index, 
            localSkipSetters);
        step.setWeight(new Double(87.6));
        
        // update timesheets
        if (scope != null)
        {
            if(scope.isTimesheets() && 
                    step.getStepTimesheets() != null && 
                    step.getStepTimesheets().length != 0)
            {
                StepTimesheet[] timesheets = step.getStepTimesheets();
                for (int i = 0; i < timesheets.length; i++)
                    timesheets[i] = StepTimesheetFixture.updateFixture(timesheets[i]);

                step.setStepTimesheets(timesheets);
            }

            if (scope.isTimeCodes())
            {
                GenericProject project = loadProject(step, ctc);
                step = (Step)assignTimeCodes(step, project, ctc);
            }
            
        }
        
        return step;
    }
  
    public static StepScope createScope(boolean isStepTimesheets,
            boolean isTimecodes, boolean isProject, boolean isResource,
            boolean isWorkElement, boolean isTaskAssignment)
    {
        StepScope scope = new StepScope();

        if (isStepTimesheets)
            scope.setTimesheets(true);

        if (isTimecodes)
            scope.setTimeCodes(true);

        if (isProject)
            scope.setProject(true);
        
        if (isResource)
            scope.setResource(true);
        
        if (isWorkElement)
            scope.setWorkElement(new WorkElementScope());
        
        if(isTaskAssignment)
            scope.setTaskAssignment(true);

        return scope;
    }
    
    public static void compare(Step[] array1, 
            Step[] array2, StepScope scope)
    	throws AssertionFailedError, Exception
	{
        RPMObjectFixture.compare(array1, array2);
        if (array1 == null) return;
		
		for (int i = 0; i < array1.length; i++)
		    StepFixture.compare(array1[i], array2[i], scope);
	}
    
    public static void compare(Step obj1, Step obj2, 
            StepScope scope) throws AssertionFailedError, Exception
	{
//        logger.info("======================");
//        logger.info("ObjectToCompare: " + obj1);
//        logger.info("======================");
//        logger.info("LoadedObject: " + obj2);
        
		RPMObjectFixture.compareWithSkip(obj1, obj2, localSkipGetters);
		
        if (scope != null)
        {
            if (scope.isTaskAssignment())
            {
                TaskAssignmentFixture.compare(obj1.getTaskAssignment(), obj1
                    .getTaskAssignment(), null);
            }
            if (scope.isTimesheets())
            {
                StepTimesheetFixture.compare(obj1.getStepTimesheets(), 
                    obj2.getStepTimesheets(), null);
            }
            if (scope.isTimeCodes())
            {
                TimecodeFixture.compare(obj1.getTimeCode1(), obj2.getTimeCode1(), 
                    null);
                TimecodeFixture.compare(obj1.getTimeCode2(), obj2.getTimeCode2(), 
                    null);
            }
            
            //Project Resource and WorlElement are not compared because they
            // are read-only fields
        }
	}
    
    public static GenericStep assignTimeCodes(GenericStep step,
            GenericProject project, ClientTestController ctc) throws Exception
    {
        if (project != null)
        {
            TimeCodeAssignment[] timeCodeAssignments = project
                .getTimeCodeAssignments();

            if (timeCodeAssignments == null)
            {
                // create some TimeCodeAssignmnets
                TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
                    .createSaveFixture(project, true, ctc);
                TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
                    .createSaveFixture(project, false, ctc);
                timeCodeAssignments = new TimeCodeAssignment[2];
                timeCodeAssignments[0] = timecode1;
                timeCodeAssignments[1] = timecode2;
            }

            boolean timeCode1Assigned = false, timeCode2Assigned = false;
            for (int i = 0; i < timeCodeAssignments.length
                    && (!timeCode1Assigned || !timeCode2Assigned); i++)
            {
                TimeCode timeCode = timeCodeAssignments[i].getTimeCode();

                // re-load timecode with the parent in scope
                TimeCodeScope scope = new TimeCodeScope();
                scope.setParent(scope);
                timeCode = (TimeCode) ctc.loadFromID(timeCode, scope);

                if (!timeCode1Assigned
                        && TimeCodeUtil.isChildOfTimeCode1(timeCode))
                {
                    step.setTimeCode1(timeCode);
                    timeCode1Assigned = true;
                }
                else if (!timeCode2Assigned
                        && TimeCodeUtil.isChildOfTimeCode2(timeCode))
                {
                    step.setTimeCode2(timeCode);
                    timeCode2Assigned = true;
                }
            }
        }

        return step;
    }
    
    public static GenericProject loadProject(GenericStep step,
            ClientTestController ctc) throws AssertionError, Exception
    {

        GenericProject project = null;
        TaskAssignment taskAssignment = step.getTaskAssignment();

        if (taskAssignment == null)
        {
            Assert
                .fail("summaryTimesheet.taskAssignment cannot be null at this point.");
        }

        TaskAssignmentScope taskAssignmentScope = SummaryTimesheetFixture
            .addProjectAndResourceToTaskAssignmentScope(
                new TaskAssignmentScope(), false);

        taskAssignment = (TaskAssignment) ctc.loadFromID(taskAssignment,
            taskAssignmentScope);
        project = (GenericProject) ((WorkElement) taskAssignment.getParent()
            .getParent()).getContainingProject();

        return project;
    }
}
