/**
 * Created on Nov 17, 2005
 *
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.fixtures;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.framework.testcases.util.TimeCodeUtil;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.containers.AbstractTimesheet;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class SummaryTimesheetFixture extends RPMObjectFixture
{
    protected static ArrayList localSkipSetters, localSkipGetters;

    static
    {
        ////////////
        //Setters
        ////////////
        localSkipSetters = new ArrayList();
        localSkipSetters.addAll(skipSetters);

        // Read-only
        localSkipSetters.add("setWeekOf"); // Primary key
        localSkipSetters.add("setTotalIncurred");
        localSkipSetters.add("setTotalSpecial");
        localSkipSetters.add("setTotalRegular");
        localSkipSetters.add("setTotalWeek");
        localSkipSetters.add("setExpectedTotalWork");
        localSkipSetters.add("setExpectedWorkPerWeek");
        localSkipSetters.add("setScheduleWorkSchedule");
        localSkipSetters.add("setScheduleWorkBaseline");
        localSkipSetters.add("setComplete");
        localSkipSetters.add("setCostPerHour");
        localSkipSetters.add("setSellingRatePerHour");
        localSkipSetters.add("setSpecialCostPerHour");
        localSkipSetters.add("setSpecialSellingRatePerHour");

        // Skip these because they have to be in a certain range
        localSkipSetters.add("setRegularFriday");
        localSkipSetters.add("setRegularMonday");
        localSkipSetters.add("setRegularSaturday");
        localSkipSetters.add("setRegularSunday");
        localSkipSetters.add("setRegularThursday");
        localSkipSetters.add("setRegularTuesday");
        localSkipSetters.add("setRegularWednesday");
        localSkipSetters.add("setSpecialFriday");
        localSkipSetters.add("setSpecialMonday");
        localSkipSetters.add("setSpecialSaturday");
        localSkipSetters.add("setSpecialSunday");
        localSkipSetters.add("setSpecialThursday");
        localSkipSetters.add("setSpecialTuesday");
        localSkipSetters.add("setSpecialWednesday");
        localSkipSetters.add("setPercentageComplete");
        localSkipSetters.add("setForecastFinishDate");
        localSkipSetters.add("setForecastStartDate");
        ////////////
        //Getters
        ////////////
        localSkipGetters = new ArrayList();
        localSkipGetters.addAll(skipGetters);

        // Read-only fields
        localSkipGetters.add("getScheduleWorkBaseline");
        localSkipGetters.add("getExpectedTotalWork");
        localSkipGetters.add("getExpectedWorkPerWeek");
        localSkipGetters.add("getScheduleWorkSchedule");
        localSkipGetters.add("getTotalIncurred"); // calculated in SP
        localSkipGetters.add("getTotalSpecial"); // calculated in SP
        localSkipGetters.add("getTotalRegular"); // calculated in SP
        localSkipGetters.add("getTotalWeek"); // calculated in SP
        localSkipGetters.add("getCostPerHour");
        localSkipGetters.add("getSellingRatePerHour");
        localSkipGetters.add("getSpecialCostPerHour");
        localSkipGetters.add("getSpecialSellingRatePerHour");

        // calculated fields
        localSkipGetters.add("getEETCHours");
        localSkipGetters.add("getPercentageComplete");
        localSkipGetters.add("getComplete");
    }

    public static SummaryTimesheet createSaveFixture(GenericTaskAssignment taskAssignment,
            Calendar weekOf, SummaryTimesheetScope scope, boolean setHours, ClientTestController ctc)
            throws Exception
    {
        SummaryTimesheet summaryTimesheet = createFixture(taskAssignment, weekOf, scope, setHours, ctc);
        summaryTimesheet = (SummaryTimesheet) save(ctc, summaryTimesheet, scope);
        return summaryTimesheet;
    }

    public static SummaryTimesheet createFixture(GenericTaskAssignment taskAssignment,
            Calendar weekOf, SummaryTimesheetScope scope, boolean setHours, ClientTestController ctc)
            throws Exception
    {
        
        if(taskAssignment == null){
            taskAssignment = TaskAssignmentFixture.createSaveFixture(null, ctc);
        }
        
        if (weekOf == null)
        {
            if (taskAssignment instanceof TaskAssignment)
            {
                Calendar assignmentStartDate = ((TaskAssignment) taskAssignment)
                    .getAssignmentStartDate();

                weekOf = TimesheetTestUtil.getValidWeekOf(assignmentStartDate,
                    TimesheetTestUtil.CURRENT_WEEK);
            }
            else
            {
                weekOf = new GregorianCalendar();
            }
        }
        

        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);
        
        
        if (taskAssignment instanceof TaskAssignment)           
        {
            //set the forecastFinishDate
            summaryTimesheet.setForecastFinishDate(TimesheetTestUtil.getValidWeekOf(weekOf, 4));

            
            GenericProject project = loadProject(summaryTimesheet, ctc);

            // do both copy proposed to plan and publish project to the team
            TimesheetTestUtil.initProject(project, ctc);
            
            if (scope != null)
            {
                if (scope.isTimeCodes())
                    summaryTimesheet = assignTimeCodes(summaryTimesheet, project,
                        ctc);
            }
        }

        summaryTimesheet = updateFixture(summaryTimesheet, scope, setHours, ctc);

        return summaryTimesheet;
    }

    public static SummaryTimesheet updateFixture(
            SummaryTimesheet summaryTimesheet, SummaryTimesheetScope scope,
            boolean setHours, ClientTestController ctc) throws Exception
    {
        int index = random();
        HelperTestUtilities.setByIntrospection(summaryTimesheet, index,
            localSkipSetters);
        
        if (summaryTimesheet.getTaskAssignment() instanceof TaskAssignment)
        {
            //Update forecastFinishDate
            Calendar weekOf = summaryTimesheet.getWeekOf();
            summaryTimesheet.setForecastFinishDate(TimesheetTestUtil
                .getValidWeekOf(weekOf, 5));
        }
        
        if (setHours)
        {
            if (summaryTimesheet.getID() == null)
            {
                TimesheetTestUtil.setWorkedHours(summaryTimesheet, 7, 2);
            }
            else
            {
                TimesheetTestUtil.setWorkedHours(summaryTimesheet, 8, 3);
            }
        }

        if (scope != null)
        {
            
            if (summaryTimesheet.getTaskAssignment() instanceof TaskAssignment)
            {
                GenericProject project = loadProject(summaryTimesheet, ctc);

                if (scope.isTimeCodes())
                {
                    summaryTimesheet = assignTimeCodes(summaryTimesheet,
                        project, ctc);
                }

            }

            if (scope.getApprovalStatus() != null)
            {
                TimesheetApprovalStatus appStatus = summaryTimesheet
                    .getApprovalStatus();

                if (appStatus == null)
               {
                    appStatus = new TimesheetApprovalStatus();
                    
                    //resource, project and weekof are set in taken from the
                    //summaryTimesheet
                    
                    appStatus = TimesheetApprovalStatusFixture
                        .submitTimesheet(appStatus);
                }
                else
                {
                    appStatus = TimesheetApprovalStatusFixture
                        .rejectTimesheetAndSave(appStatus, scope
                            .getApprovalStatus(), ctc);
                }
                summaryTimesheet.setApprovalStatus(appStatus);
            }
        }

        //use a GenericTaskAssignment
        
        GenericTaskAssignment genericTaskAssignment = new GenericTaskAssignment();
        genericTaskAssignment.setID(summaryTimesheet.getTaskAssignment().getID());
        
        summaryTimesheet.setTaskAssignment(genericTaskAssignment);
        
        return summaryTimesheet;
    }

    public static SummaryTimesheetScope createScope(boolean isTimeCode,
            boolean isApprovalStatus, boolean isProject,
            boolean isResource, boolean isWorkElement)
    {
        SummaryTimesheetScope scope = new SummaryTimesheetScope();

        if (isTimeCode)
            scope.setTimeCodes(true);

        if (isApprovalStatus)
            scope.setApprovalStatus(TimesheetApprovalStatusFixture.createScope(
                true, true, false));

        if (isProject)
            scope.setProject(true);

        if (isResource)
            scope.setResource(true);
        
        if(isWorkElement)
            scope.setWorkElement(new WorkElementScope());

        return scope;
    }

    public static void compare(SummaryTimesheet[] array1,
            SummaryTimesheet[] array2, SummaryTimesheetScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compare(array1, array2);
        if (array1 == null)
            return;

        for (int i = 0; i < array1.length; i++)
            SummaryTimesheetFixture.compare(array1[i], array2[i], scope);
    }

    public static void compare(SummaryTimesheet obj1, SummaryTimesheet obj2,
            SummaryTimesheetScope scope) throws AssertionFailedError, Exception
    {
        //        logger.info("======================");
        //        logger.info("ObjectToCompare: " + obj1);
        //        logger.info("======================");
        //        logger.info("LoadedObject: " + obj2);

        // don't compare forecastFinishDate if the taskAssignment is completed
        if (obj1.getTaskAssignment() instanceof TaskAssignment
                && obj2.getTaskAssignment() instanceof TaskAssignment)
        {
            if ((((TaskAssignment) obj1.getTaskAssignment()).getComplete() != null && ((TaskAssignment) obj1
                .getTaskAssignment()).getComplete().booleanValue())
                    || ((TaskAssignment) obj2.getTaskAssignment())
                        .getComplete() != null
                    && ((TaskAssignment) obj2.getTaskAssignment())
                        .getComplete().booleanValue())
            {
                localSkipGetters.add("getForecastFinishDate");
            }
        }

        // don't compare forecastStartDate if there are actuals
        if (calculateTotalBillable(obj1) > 0
                || calculateTotalBillable(obj2) > 0)
        {
            localSkipGetters.add("getForecastStartDate");
        }
        
        RPMObjectFixture.compareWithSkip(obj1, obj2, localSkipGetters);

        Assert.assertEquals(obj1.getTaskAssignment().getID(), obj2
            .getTaskAssignment().getID());

        if (scope != null)
        {
            if (scope.isTimeCodes())
            {
                TimecodeFixture.compare(obj1.getTimeCode1(), obj2
                    .getTimeCode1(), null);
                TimecodeFixture.compare(obj1.getTimeCode2(), obj2
                    .getTimeCode2(), null);
            }
            if (scope.getApprovalStatus() != null)
            {
                TimesheetApprovalStatusFixture.compare(
                    obj1.getApprovalStatus(), obj2.getApprovalStatus(), scope
                        .getApprovalStatus());
            }
        }
    }

    public static TaskAssignmentScope addProjectAndResourceToTaskAssignmentScope(
            TaskAssignmentScope taskAssignmentScope,
            boolean isTimeCodesAssignmnet)
    {
        if (taskAssignmentScope == null)
            taskAssignmentScope = new TaskAssignmentScope();

        ResourceTaskAssignmentScope resourceTaskAssignmentScope = null;
        if (taskAssignmentScope.getParent() == null)
            resourceTaskAssignmentScope = new ResourceTaskAssignmentScope();
        else
            resourceTaskAssignmentScope = (ResourceTaskAssignmentScope) taskAssignmentScope
                .getParent();
        resourceTaskAssignmentScope.setResource(new ResourceScope());
        WorkElementScope workElementScope = null;
        if (resourceTaskAssignmentScope.getParent() == null)
            workElementScope = new WorkElementScope();
        else
            // TODO assuming only WorkElement scope
            workElementScope = (WorkElementScope) resourceTaskAssignmentScope
                .getParent();

        if (workElementScope.getContainingProject() == null)
            workElementScope.setContainingProject(new WorkElementScope());

        workElementScope.setScheduleDates(true);
        workElementScope.setTimeCodeAssignments(isTimeCodesAssignmnet);

        resourceTaskAssignmentScope.setParent(workElementScope);
        taskAssignmentScope.setParent(resourceTaskAssignmentScope);

        return taskAssignmentScope;
    }

    private static SummaryTimesheet assignTimeCodes(SummaryTimesheet timesheet,
            GenericProject project, ClientTestController ctc) throws Exception
    {
        if (project != null)
        {
            TimeCodeAssignment[] timeCodeAssignments = project
                .getTimeCodeAssignments();

            if (timeCodeAssignments == null)
            {
                // create some TimeCodeAssignmnets
                TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
                    .createSaveFixture(project, true, ctc);
                TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
                    .createSaveFixture(project, false, ctc);
                timeCodeAssignments = new TimeCodeAssignment[2];
                timeCodeAssignments[0] = timecode1;
                timeCodeAssignments[1] = timecode2;
            }

            boolean timeCode1Assigned = false, timeCode2Assigned = false;
            for (int i = 0; i < timeCodeAssignments.length
                    && (!timeCode1Assigned || !timeCode2Assigned); i++)
            {
                TimeCode timeCode = timeCodeAssignments[i].getTimeCode();

                // re-load timecode with the parent in scope
                TimeCodeScope scope = new TimeCodeScope();
                scope.setParent(scope);
                timeCode = (TimeCode) ctc.loadFromID(timeCode, scope);

                if (!timeCode1Assigned
                        && TimeCodeUtil.isChildOfTimeCode1(timeCode))
                {
                    timesheet.setTimeCode1(timeCode);
                    timeCode1Assigned = true;
                }
                else if (!timeCode2Assigned
                        && TimeCodeUtil.isChildOfTimeCode2(timeCode))
                {
                    timesheet.setTimeCode2(timeCode);
                    timeCode2Assigned = true;
                }
            }
        }

        return timesheet;
    }

    public static GenericProject loadProject(SummaryTimesheet summaryTimesheet,
            ClientTestController ctc) throws AssertionError, Exception
    {

        GenericProject project = null;
        TaskAssignment taskAssignment = (TaskAssignment)summaryTimesheet.getTaskAssignment();

        if (taskAssignment == null)
        {
            Assert.fail(
                "summaryTimesheet.taskAssignment cannot be null at this point.");
        }

        TaskAssignmentScope taskAssignmentScope = addProjectAndResourceToTaskAssignmentScope(
            new TaskAssignmentScope(), false);

        taskAssignment = (TaskAssignment) ctc.loadFromID(taskAssignment,
            taskAssignmentScope);
        project = (GenericProject) ((WorkElement) taskAssignment.getParent()
            .getParent()).getContainingProject();

        return project;
    }

    public static Resource loadResource(SummaryTimesheet summaryTimesheet,
            ClientTestController ctc) throws AssertionError, Exception
    {

        Resource resource = null;
        GenericTaskAssignment taskAssignment = summaryTimesheet.getTaskAssignment();

        if (taskAssignment == null)
        {
            Assert.fail(
                "summaryTimesheet.taskAssignment cannot be null at this point.");
        }

        TaskAssignmentScope taskAssignmentScope = addProjectAndResourceToTaskAssignmentScope(
            new TaskAssignmentScope(), false);

        taskAssignment = (TaskAssignment) ctc.loadFromID(taskAssignment,
            taskAssignmentScope);
        resource = ((ResourceTaskAssignment) taskAssignment.getParent())
            .getResource();

        return resource;
    }
    
    private static double calculateBillableRegular(AbstractTimesheet timesheet)
    {
        return timesheet.getRegularMonday().doubleValue()
                + timesheet.getRegularTuesday().doubleValue()
                + timesheet.getRegularWednesday().doubleValue()
                + timesheet.getRegularThursday().doubleValue()
                + timesheet.getRegularFriday().doubleValue()
                + timesheet.getRegularSaturday().doubleValue()
                + timesheet.getRegularSunday().doubleValue();
    }

    private static double calculateBillableSpecial(AbstractTimesheet timesheet)
    {
        return timesheet.getSpecialMonday().doubleValue()
                + timesheet.getSpecialTuesday().doubleValue()
                + timesheet.getSpecialWednesday().doubleValue()
                + timesheet.getSpecialThursday().doubleValue()
                + timesheet.getSpecialFriday().doubleValue()
                + timesheet.getSpecialSaturday().doubleValue()
                + timesheet.getSpecialSunday().doubleValue();
    }

    private static double calculateTotalBillable(AbstractTimesheet timesheet)
    {
        return calculateBillableRegular(timesheet)
                + calculateBillableSpecial(timesheet);
    }

}