/**
 * Created on Dec 12, 2005
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.scope.WorkElementScope;


public class TimesheetApprovalStatusFixture extends RPMObjectFixture
{
    private static String OLD_PASSWORD = "mytest";
    private static String NEW_PASSWORD = "mytest2";
    
    protected static ArrayList localSkipSetters, localSkipGetters;

    static {
        //Setters
        localSkipSetters = new ArrayList();
        localSkipSetters.addAll(skipSetters);
        localSkipSetters.add("setWeekOf");  // Primary Key
        localSkipSetters.add("setApprovalNotes"); // Read-only when status = submit
        
        //Getters
        localSkipGetters = new ArrayList();
        localSkipGetters.addAll(skipGetters);
        localSkipGetters.add("getApprovalDate"); // set by the SP call
    }
        
    public static TimesheetApprovalStatus updateFixture(
            TimesheetApprovalStatus appStatus) throws Exception 
    {
        HelperTestUtilities.setByIntrospection(appStatus, random(), 
            localSkipSetters);
        
        if(appStatus.getApprovalStatus() != TimesheetStatus.Submitted){
            appStatus.setApprovalNotes("Updated notes");
        }
        
        return appStatus;
    }
  
    public static TimesheetApprovalStatusScope createScope(boolean isProject, 
            boolean isResource, boolean isLastApprovedRejectedBy)
    {
        TimesheetApprovalStatusScope scope = new TimesheetApprovalStatusScope();

        if (isProject)
            scope.setProject(new WorkElementScope());
        
        if (isResource)
            scope.setResource(new ResourceScope());

        if (isLastApprovedRejectedBy)
            scope.setLastApprovedRejectedBy(new ResourceScope());
        
        return scope;
    }
    
    public static void compare(TimesheetApprovalStatus[] array1, 
            TimesheetApprovalStatus[] array2, TimesheetApprovalStatusScope scope)
    	throws AssertionFailedError, Exception
	{
        RPMObjectFixture.compare(array1, array2);
        if(array1 == null) return;
		
		for(int i = 0; i < array1.length; i++)
		    TimesheetApprovalStatusFixture.compare(array1[i], array2[i], scope);
	}
    
    public static void compare(TimesheetApprovalStatus obj1, 
            TimesheetApprovalStatus obj2, TimesheetApprovalStatusScope scope) 
    	throws AssertionFailedError, Exception
	{
		RPMObjectFixture.compareWithSkip(obj1, obj2, localSkipGetters);
		
        if (obj1 != null && obj2 != null && scope != null)
        {
            
            // Don't compare the project because it will be converted from 
            // proposal to project by the copyProposedToPlan() call. On addition,
            // some other fields (proposedStartDate, plannedStartDate, etc) will
            // be changed too.
        }
	}
    
    public static TimesheetApprovalStatus approveTimesheetAndSave(
            TimesheetApprovalStatus appStatus, TimesheetApprovalStatusScope scope,
            ClientTestController ctc) 
    	throws Exception 
    {
        appStatus.setApprovalStatus(TimesheetStatus.Approved);
        appStatus = (TimesheetApprovalStatus) save(ctc, appStatus, scope);
        
        return appStatus;
    }
    
    public static TimesheetApprovalStatus rejectTimesheetAndSave(
            TimesheetApprovalStatus appStatus, 
            TimesheetApprovalStatusScope scope, ClientTestController ctc) 
    	throws Exception 
    {
        
        appStatus.setApprovalStatus(TimesheetStatus.Rejected);
        appStatus = (TimesheetApprovalStatus) save(ctc, appStatus, scope);
        
        return appStatus;
    }
    
    public static TimesheetApprovalStatus submitTimesheet(
            TimesheetApprovalStatus appStatus) throws Exception 
    {
        appStatus.setApprovalStatus(TimesheetStatus.Submitted);
        return appStatus;
    }
    
    /**
     * 
     * @param appStatus
     * @param scope
     * @param changePassword	We should change the password the first time
     * 							the user logs in.
     * @param ctc
     * @return
     * @throws Exception
     */
    public static TimesheetApprovalStatus submitTimesheetAndSave(
            TimesheetApprovalStatus appStatus, TimesheetApprovalStatusScope scope,
            boolean changePassword, ClientTestController ctc) 
    	throws Exception 
    {
        // change the password the first time we log in
        Resource resource = appStatus.getResource();
        if (changePassword)
        {
            ResourceScope resourceScope = null;
            if (scope != null)
                resourceScope = scope.getResource();
            resource.setPassword(OLD_PASSWORD);
            resource = (Resource) save(ctc, resource, resourceScope);
            ctc.changePassword(resource.getUserName(), OLD_PASSWORD,
                NEW_PASSWORD);
        }
        
        // ensure that the resource is loaded
        if (resource.getUserName() == null)
            resource = (Resource)ctc.loadFromID(resource, null);
        
        // logout and re-login as the resource
        ctc.getAuthenticateInterface().logout(ClientTestController.sessionid);
        ctc.performLogin(resource.getUserName(), NEW_PASSWORD);
        
        appStatus = submitTimesheet(appStatus);
        
        // save
        appStatus = (TimesheetApprovalStatus) save(ctc, appStatus, scope);
        
        // logout and re-login as adminstrator
        ctc.getAuthenticateInterface().logout(ClientTestController.sessionid);
        ctc.performLogin();
        
        return appStatus;
    }
    
}
