package com.ibm.rpm.client.junit.timesheet.testcases;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultAdministrativeTaskFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.containers.AdministrativeTask;
import com.ibm.rpm.timesheet.containers.AdministrativeTaskAssignment;
import com.ibm.rpm.timesheet.containers.DefaultAdministrativeTask;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;

/**
 * @author gmeloche
 *  
 */
public class TestAdministrativeTaskAssignment extends ClientTestController
{
    private DefaultAdministrativeTask managerAdminTask = null;
    
    private Pool pool = null;
    
    private Resource user = null;

    protected void setUp() throws Exception
    {
        super.setUp();
        managerAdminTask = DefaultAdministrativeTaskFixture.createFixture(null, this);

        pool = (Pool)managerAdminTask.getParent();
        
        ResourceScope resScope = new ResourceScope();
        resScope.setCalendar(new CalendarScope());
             
        user = ResourceFixture.createFixture(pool, resScope, this);
        
        user = (Resource)save(user, resScope).getRpmObject();
    }

    public void testLoadAssignmentFromXpath() throws Exception
    {
        managerAdminTask = save(managerAdminTask);

        String xpath = "Resource[@id = '" + user.getID()
                + "']/administrativeTaskAssignment";
        
        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(new RPMObjectScope());
        
        RPMObject[] objs = loadFromXpath(xpath, scope);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof AdministrativeTaskAssignment);
        assertNotNull(objs[0].getParent());
        assertIdEquals(user, objs[0].getParent());
    }
    
    public void testLoadResourceFromXpath() throws Exception
    {
        managerAdminTask = save(managerAdminTask);

        // TODO SLegault: should use the user's admin task id, not the manager's.
        String xpath = "/Resource[administrativeTaskAssignment[administrativetask[@name='"
                + managerAdminTask.getName() + "']]]";

        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(new RPMObjectScope());

        RPMObject[] objs = loadFromXpath(xpath, scope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof Resource);
        assertIdEquals(user, objs[0]);
    }
    
    public void testLoadAdministrativeTaskFromXpath() throws Exception
    {
        managerAdminTask = save(managerAdminTask);
        
        // TODO SLegault: should use the user's admin task id, not the manager's.
        String xpath = "Resource[@id = '" + user.getID()
                + "']/administrativeTaskAssignment/administrativetask[@name='"
                + managerAdminTask.getName() + "']";

        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(new RPMObjectScope());
        
        RPMObject[] objs = loadFromXpath(xpath, scope);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof AdministrativeTask);
    }
    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public DefaultAdministrativeTask save(DefaultAdministrativeTask object) throws Exception
    {
        SingleResult result = save(managerAdminTask, null);
        return (DefaultAdministrativeTask) result.getRpmObject();
    }
    
    public void testShownInTimesheet() throws Exception
    {
        managerAdminTask = save(managerAdminTask);
        
        String xPath = "Resource[@id='" + user.getID()
                + "']/@administrativeTaskAssignments";
        GenericTaskAssignment taskAssignment = (GenericTaskAssignment) loadFromXpath(
                xPath, null)[0];
        doTestShownInTimesheet(taskAssignment);
    }

    private void doTestShownInTimesheet(GenericTaskAssignment taskAssignment)
            throws Exception
    {
        Boolean isShownInTimesheet = taskAssignment.getShownInTimesheet();
        assertNotNull(isShownInTimesheet);
        assertEquals(Boolean.FALSE, isShownInTimesheet);

        doTestUpdateShownInTimesheet(taskAssignment, Boolean.TRUE);
        doTestUpdateShownInTimesheet(taskAssignment, Boolean.FALSE);
    }

    private void doTestUpdateShownInTimesheet(
            GenericTaskAssignment originalTaskAssignment, Boolean value)
            throws Exception
    {
        Class originalClass = originalTaskAssignment.getClass();
        String typeName = StringUtil.getShortClassName(originalClass);
        originalTaskAssignment.setShownInTimesheet(value);
        save(originalTaskAssignment, null, ReloadType.None)
                .getRpmObject();

        GenericTaskAssignment reloadedTaskAssignment = (GenericTaskAssignment) originalClass
                .newInstance();
        reloadedTaskAssignment.setID(originalTaskAssignment.getID());
        reloadedTaskAssignment = (GenericTaskAssignment) loadFromID(
                reloadedTaskAssignment, null);
        Boolean isShownInTimesheet = reloadedTaskAssignment
                .getShownInTimesheet();
        assertNotNull(isShownInTimesheet);
        assertEquals(value, isShownInTimesheet);

        doTestLoadFromXPath(typeName, originalTaskAssignment.getID(), value);
    }

    private void doTestLoadFromXPath(String typeName, String assignmentId,
            Boolean value) throws Exception
    {
        RPMObject[] assignments = loadFromXpath("/" + typeName + "[@id='"
                + assignmentId + "' and @shownInTimesheet='" + value + "']",
                null);
        assertNotNull(assignments);
        assertEquals(1, assignments.length);

        String notValue = Boolean.toString(!value.booleanValue());
        assignments = loadFromXpath("/" + typeName + "[@id='" + assignmentId
                + "' and shownInTimesheet='" + notValue + "']", null);
        assertNull(assignments);
    }
}
