package com.ibm.rpm.client.junit.timesheet.testcases;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.timesheet.fixtures.CreditFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;

/**
 * @author gmeloche
 *  
 */
public class TestCredit extends ClientTestController
{
    
    private Calendar creditWeekOf = null;
    private Calendar weekOf = null;
    
    private SummaryTimesheet summaryTimesheet = null;

    private SummaryTimesheetScope summaryTimesheetScope = null;

    private StepScope creditScope = StepFixture.createScope(true, false, true, true,
        true, true);
    

    protected void setUp() throws Exception
    {
        //creates and submits a SummaryTimesheet
        super.setUp();

        TaskAssignment taskAssignment = TaskAssignmentFixture.createSaveFixture(null, this);
        
        creditWeekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment.getAssignmentStartDate(),
            TimesheetTestUtil.CREDIT_WEEK_OF);
        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment.getAssignmentStartDate(),
            TimesheetTestUtil.CURRENT_WEEK);
        
        summaryTimesheetScope = SummaryTimesheetFixture.createScope(false,
            true, false, false, false);
        summaryTimesheet = SummaryTimesheetFixture.createSaveFixture(taskAssignment,
           creditWeekOf, summaryTimesheetScope, true, this);

        TimesheetApprovalStatus approvalStatus = summaryTimesheet
            .getApprovalStatus();
        approvalStatus.setApprovalStatus(TimesheetStatus.Approved);
        
        SingleResult result = save(summaryTimesheet, summaryTimesheetScope);
        
        summaryTimesheet = (SummaryTimesheet)result.getRpmObject();

      
    }

    protected void tearDown() throws Exception
    {

        super.tearDown();
    }

    public void testInsertWithScope() throws Exception
    {
        
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            creditScope, this);
        
        initStep();
        
        SingleResult result = save(credit, creditScope);
        Credit savedCredit = (Credit)result.getRpmObject();
        
        CreditFixture.compare(credit, savedCredit, creditScope);
        
        TimesheetTestUtil.compareTotalsWithSummaryTimesheet(credit.getCreditTimesheet(), this);
    }
    
    public void testUpdateWithScope() throws Exception
    {
        
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            creditScope, this);
        
        initStep();
        
        SingleResult result = save(credit, creditScope);
        credit = (Credit)result.getRpmObject();
        
        credit.setName(HelperTestUtilities.createName());
        
        SingleResult result1 = save(credit, creditScope);
        Credit savedCredit = (Credit)result1.getRpmObject();
        
        CreditFixture.compare(credit, savedCredit, creditScope);
        
        TimesheetTestUtil.compareTotalsWithSummaryTimesheet(credit
            .getCreditTimesheet(), this);

    }

    public void testLoadFromXPathWithProject() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, false, true,
            false, false, false);
        
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit[" + " @project[id = '"
                + credit.getProject().getID() + "']]", scope);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof Credit);
        assertEquals(credit.getProject().getID(), ((Credit) objs[0])
            .getProject().getID());
    
    }
    public void testLoadChildFromXPath_Project() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, false, true,
            false, false, false);
        
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit/" + " @project[id = '"
                + credit.getProject().getID() + "']", null);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof GenericProject);
        assertEquals(credit.getProject().getID(), ((GenericProject) objs[0])
            .getID());
    
    }
    public void testLoadFromXPathWithResource() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, false, false,
            true, false, false);
        
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit[" + " @resource[id = '"
                + credit.getResource().getID() + "']]", scope);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof Credit);
        assertEquals(credit.getResource().getID(), ((Credit) objs[0])
            .getResource().getID());
    
    }

    public void testLoadChildFromXPath_Resource() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, false, false,
            true, false, false);
        
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit/" + " @resource[id = '"
                + credit.getResource().getID() + "']", null);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof Resource);
        assertEquals(credit.getResource().getID(), ((Resource) objs[0])
            .getID());
    
    }
    
    public void testLoadFromXPathWithTaskAssignment() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, false, false,
            false, false, true);
    
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit[" + " @taskAssignment[id = '"
                + credit.getTaskAssignment().getID() + "']]", scope);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof Credit);
        assertEquals(credit.getTaskAssignment().getID(),
            ((Credit) objs[0]).getTaskAssignment().getID());
    
    }
    
    public void testLoadChildFromXPath_TaskAssignment() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, false, false,
            false, false, true);
    
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit/" + " @taskAssignment[id = '"
                + credit.getTaskAssignment().getID() + "']", null);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof TaskAssignment);
        assertEquals(credit.getTaskAssignment().getID(),
            ((TaskAssignment) objs[0]).getID());
    
    }

    public void testLoadFromXPathWithWorkElement() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, false, false,
            false, true, false);
    
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit[" + " @workElement[id = '"
                + credit.getWorkElement().getID() + "']]", scope);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof Credit);
        assertEquals(credit.getWorkElement().getID(), ((Credit) objs[0])
            .getWorkElement().getID());
    
    }
    
    public void testLoadChildFromXPath_WorkElement() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, false, false,
            false, true, false);
    
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit/" + " @workElement[id = '"
                + credit.getWorkElement().getID() + "']", null);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof WorkElement);
        assertEquals(credit.getWorkElement().getID(), ((WorkElement) objs[0])
            .getID());
    
    }
    
    public void testLoadFromXPathWithTimecode1() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, true, false,
            false, false, false);
    
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit[" + " @timecode1[id = '"
                + credit.getTimeCode1().getID() + "']]", scope);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof Credit);
        assertEquals(credit.getTimeCode1().getID(), ((Credit) objs[0])
            .getTimeCode1().getID());
    
    }
    
    public void testLoadChildFromXPath_Timecode1() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, true, false,
            false, false, false);
    
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit/" + " @timecode1[id = '"
                + credit.getTimeCode1().getID() + "']", null);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof TimeCode);
        assertEquals(credit.getTimeCode1().getID(), ((TimeCode) objs[0])
            .getID());
    
    }
    
    public void testLoadChildFromXPathWithCreditTimesheet() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, true, false,
            false, false, false);
    
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit[" + " @creditTimesheet[id = '"
                + credit.getCreditTimesheet().getID() + "']]", scope);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof Credit);
        assertEquals(credit.getCreditTimesheet().getID(), ((Credit) objs[0])
            .getCreditTimesheet().getID());
    
    }
    
    public void testLoadChildFromXPath_CreditTimesheet() throws Exception
    {
    
        StepScope scope = StepFixture.createScope(true, true, false,
            false, false, false);
    
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf, null,
            scope, this);
        
        initStep();
        
        SingleResult result = save(credit, scope);
        credit = (Credit)result.getRpmObject();
    
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Credit/" + " @creditTimesheet[id = '"
                + credit.getCreditTimesheet().getID() + "']", null);
    
        assertNotNull(objs);
        assertTrue(objs[0] instanceof CreditTimesheet);
        assertEquals(credit.getCreditTimesheet().getID(), ((CreditTimesheet) objs[0])
            .getID());
    
    }
    
    public void testDeleteCredit() throws Exception
    {
        //note (gmeloche) this test fails because on a database defect
        
        Credit credit = CreditFixture.createFixture(summaryTimesheet, weekOf,
            null, creditScope, this);

        initStep();

        //remove actuals
        CreditTimesheet creditTimesheet = new CreditTimesheet();
        creditTimesheet.setWeekOf(credit.getCreditTimesheet().getWeekOf());
        creditTimesheet.setCreditWeekOf(credit.getCreditTimesheet()
            .getCreditWeekOf());

        credit.setCreditTimesheet(creditTimesheet);

        SingleResult result = save(credit, creditScope);
        Credit savedCredit = (Credit) result.getRpmObject();

        delete(savedCredit);

        boolean deleted = false;
        try
        {
            loadFromID(savedCredit, null);
        }
        catch (Exception e)
        {
            deleted = true;
        }
        assertTrue(deleted);

    }

    private void initStep() throws Exception
    {

        //Create a step with a StepTimesheet for the current week 

        StepScope stepScope = new StepScope();
        stepScope.setTimesheets(true);
        
        //create a summaryTimesheet for the weekOf
        SummaryTimesheetFixture.createSaveFixture(summaryTimesheet
            .getTaskAssignment(), weekOf, null, false, this);

        StepTimesheet stepTimesheet = new StepTimesheet();
        stepTimesheet.setWeekOf(weekOf);
        StepTimesheetFixture.updateFixture(stepTimesheet);

        StepFixture.createSaveFixture((TaskAssignment) summaryTimesheet
            .getTaskAssignment(), stepScope, new StepTimesheet[]
        {
            stepTimesheet
        }, this);

    }
}
