package com.ibm.rpm.client.junit.timesheet.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultAdministrativeTaskFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.timesheet.containers.DefaultAdministrativeTask;

/**
 * @author gmeloche
 *  
 */
public class TestDefaultAdministrativeTask extends ClientTestController
{

    DefaultAdministrativeTask adminTask = null;
    
    Pool pool = null;

    protected void setUp() throws Exception
    {
        super.setUp();
        adminTask = DefaultAdministrativeTaskFixture.createFixture(null, this);

        pool = (Pool)adminTask.getParent();
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertLoadFromXpath() throws Exception
    {

        adminTask = save(adminTask);

        DefaultAdministrativeTask reloaded = load(adminTask, true);

        DefaultAdministrativeTaskFixture.compare(adminTask, reloaded);
    }

    public void testInsertLoadFromId() throws Exception
    {

        adminTask = save(adminTask);

        DefaultAdministrativeTask reloaded = load(adminTask, false);

        DefaultAdministrativeTaskFixture.compare(adminTask, reloaded);
    }

    public void testUpdate() throws Exception
    {

        adminTask = save(adminTask);

        adminTask = DefaultAdministrativeTaskFixture.updateFixture(adminTask, this);
        
        adminTask = save(adminTask);

        DefaultAdministrativeTask reloaded = load(adminTask, true);

        DefaultAdministrativeTaskFixture.compare(adminTask, reloaded);

    }
    
    public void testDelete() throws Exception{
        
        adminTask = save(adminTask);
        
        delete(adminTask);
        
        boolean deleted = false;
        try
        {
            loadFromID(adminTask, null);
        }
        catch (Exception e)
        {
            deleted = true;
        }
        assertTrue(deleted);
        
    }
    
    public void testDeleteUsingFlag() throws Exception
    {
        PoolScope poolScope = new PoolScope();
        poolScope.setAdministrativeTasks(true);
        
        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(poolScope);
        
        SingleResult result = save(adminTask, scope);
        adminTask =  (DefaultAdministrativeTask) result.getRpmObject();      
        
        adminTask.setDeleted(Boolean.TRUE);
        
        SingleResult result2 = save(adminTask.getParent(), poolScope); 
        Pool parentPool =  (Pool) result2.getRpmObject(); 
        
        for (int i = 0; parentPool.getAdministrativeTasks() != null && i < parentPool.getAdministrativeTasks().length; ++i)
        {
            assertTrue("The administrative task was not deleted from the parent pool using deleted flag", adminTask.getID() != parentPool.getAdministrativeTasks()[i].getID());
        }
    }
    
    public void testLoadWithParent() throws Exception
    {
        adminTask = save(adminTask);

        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(new RPMObjectScope());

        DefaultAdministrativeTask reloaded = (DefaultAdministrativeTask) loadFromID(adminTask, scope);

        assertNotNull(reloaded.getParent());
        assertTrue(reloaded.getParent() instanceof Pool);

    }
    
    
    public void testLoadFromXpathFromPool() throws Exception
    {

        adminTask = save(adminTask);

        String xpath = "/Pool[id = '" + pool.getID() + "']/@administrativetasks";

        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(new RPMObjectScope());

        RPMObject[] objs = loadFromXpath(xpath, scope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof DefaultAdministrativeTask);
        assertNotNull(objs[0].getParent());
        assertIdEquals(pool, objs[0].getParent());
    }
    
    public void testLoadPoolFromXpath() throws Exception
    {

        adminTask = save(adminTask);

        String xpath = "/Pool[@administrativetasks[id = '" + adminTask.getID()
                + "']]";

        RPMObject[] objs = loadFromXpath(xpath, null);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof Pool);
        assertIdEquals(pool, objs[0]);
    }
    
    public DefaultAdministrativeTask load(DefaultAdministrativeTask adminTask,
            boolean useXPath) throws Exception
    {

        DefaultAdministrativeTask object = null;

        if (useXPath)
        {
            RPMObject[] objs = loadFromXpath("/DefaultAdministrativeTask[id='"
                    + adminTask.getID() + "']", null);
            if (objs != null && objs.length >= 1)
            {
                object = (DefaultAdministrativeTask) objs[0];
            }
        }
        else
        {
            DefaultAdministrativeTask temp = new DefaultAdministrativeTask();
            temp.setID(adminTask.getID());
            object = (DefaultAdministrativeTask) loadFromID(temp, null);
        }

        return object;
    }

    
    public DefaultAdministrativeTask save(DefaultAdministrativeTask object) throws Exception
    {
        SingleResult result = save(adminTask, null);
        return (DefaultAdministrativeTask) result.getRpmObject();
    }
}
