/**
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultStepFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.DefaultStep;
import com.ibm.rpm.timesheet.scope.DefaultStepScope;
import com.ibm.rpm.wbs.containers.Project;

public class TestDefaultStep extends ClientTestController 
{
    private DefaultStepScope mainScope = null;
    private Project parent = null;
    
    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        
        mainScope = DefaultStepFixture.createScope(true, true);
        parent = (Project) WorkElementFixture.retrieveStaticFixture(new Project(), 
            this);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        insert(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
    	Exception
    {
        insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(mainScope, true);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(mainScope, true);
    }
    
    private void insert(DefaultStepScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        //insert an element
        DefaultStep defaultStep = insertDefaultStep(scope, loadFromId);

        //delete the element
        deleteDefaultStep(defaultStep, loadFromId);
    }
    
    private void update(DefaultStepScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        //insert an element
        DefaultStep defaultStep = insertDefaultStep(scope, loadFromId);

        //update the element
        updateDefaultStep(defaultStep, scope, loadFromId);

        //delete the element
        deleteDefaultStep(defaultStep, loadFromId);
    }
    
    private DefaultStep insertDefaultStep(DefaultStepScope scope, 
            boolean loadFromId) throws AssertionError, Exception
    {
        DefaultStep defaultStep = DefaultStepFixture.createFixture(parent,
            scope, this);

        //Insert the element into the DB
        SingleResult result = save(defaultStep, scope);

        //compare the returned element
        DefaultStep returnedStep = (DefaultStep) result.getRpmObject();
        DefaultStepFixture.compare(defaultStep, returnedStep, scope);

        //reload the element and compare it
        returnedStep = (DefaultStep) load(returnedStep, scope, loadFromId);
        DefaultStepFixture.compare(defaultStep, returnedStep, scope);

        //return the element
        return returnedStep;
    }
    
    private void updateDefaultStep(DefaultStep defaultStep, 
            DefaultStepScope scope, boolean loadFromId) 
    	throws AssertionError, Exception
    {
        DefaultStepFixture.updateFixture(defaultStep, scope, this);

        //Insert the element into the DB
        SingleResult result = save(defaultStep, scope);

        //compare the returned element
        DefaultStep returnedStep = (DefaultStep) result.getRpmObject();
        DefaultStepFixture.compare(defaultStep, returnedStep, scope);

        //reload the element and compare it
        returnedStep = (DefaultStep) load(returnedStep, scope, loadFromId);
        DefaultStepFixture.compare(defaultStep, returnedStep, scope);
    }

    private void deleteDefaultStep(DefaultStep defaultStep, 
            boolean loadFromId) throws Exception
	{    	   	
        //delete the element
       	delete(defaultStep);
    	
        //reload the deleted element
       	DefaultStep deletedStep = loadDeletedStep(defaultStep, loadFromId);
        
        //validate deleted element
        assertNull(deletedStep);
    }
    
    private DefaultStep loadDeletedStep(DefaultStep defaultStep, 
            boolean loadFromId) throws Exception
    {
        try {
            defaultStep = (DefaultStep)load(defaultStep, null, loadFromId);
           	return defaultStep;
        } catch (Exception e) {
            //element does not exist
        }
        return null;
    }
    
    private RPMObject load(DefaultStep defaultStep, DefaultStepScope scope, 
            boolean loadFromId) throws Exception
	{
    	RPMObject object = null;
        
        if (loadFromId)
    	{
            //load from an id
            RPMObject obj = new DefaultStep();
            obj.setID(defaultStep.getID());
            object = loadFromID(obj, scope);
    	} else {
    		//load from an xpath
    		RPMObject[] objs = loadFromXpath("/DefaultStep[id='" + 
    		    defaultStep.getID()+ "']", scope);
    		
    		if (objs != null && objs.length > 0)
    			object =  objs[0];
    	}
        
        return object;
    }
}
