package com.ibm.rpm.client.junit.timesheet.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.PersonalTaskFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.PersonalProject;
import com.ibm.rpm.timesheet.containers.PersonalTask;

/**
 * @author gmeloche
 *  
 */
public class TestPersonalTask extends ClientTestController
{

    PersonalTask persoTask = null;

    protected void setUp() throws Exception
    {
        super.setUp();
        persoTask = PersonalTaskFixture.createFixture(null, this);

    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertLoadFromXpath() throws Exception
    {

        persoTask = save(persoTask);

        PersonalTask reloaded = load(persoTask, true);

        PersonalTaskFixture.compare(persoTask, reloaded);
    }

    public void testInsertLoadFromId() throws Exception
    {

        persoTask = save(persoTask);

        PersonalTask reloaded = load(persoTask, false);

        PersonalTaskFixture.compare(persoTask, reloaded);
    }

    public void testUpdate() throws Exception
    {

        persoTask = save(persoTask);

        persoTask = PersonalTaskFixture.updateFixture(persoTask, this);
        
        persoTask = save(persoTask);

        PersonalTask reloaded = load(persoTask, true);

        PersonalTaskFixture.compare(persoTask, reloaded);

    }
    
    public void testLoadWithParent() throws Exception
    {
        persoTask = save(persoTask);

        RPMObjectScope scope = new RPMObjectScope();
        scope.setParent(new RPMObjectScope());

        PersonalTask reloaded = (PersonalTask) loadFromID(persoTask, scope);

        assertNotNull(reloaded.getParent());
        assertTrue(reloaded.getParent() instanceof PersonalProject);

    }
    

    public PersonalTask load(PersonalTask persotask,
            boolean useXPath) throws Exception
    {

        PersonalTask object = null;

        if (useXPath)
        {
            RPMObject[] objs = loadFromXpath("/PersonalTask[id='"
                    + persotask.getID() + "']", null);
            if (objs != null && objs.length >= 1)
            {
                object = (PersonalTask) objs[0];
            }
        }
        else
        {
            PersonalTask temp = new PersonalTask();
            temp.setID(persotask.getID());
            object = (PersonalTask) loadFromID(temp, null);
        }

        return object;
    }
    
    public void testDelete() throws Exception{
        
        persoTask = save(persoTask);
        
        delete(persoTask);
        
        boolean deleted = false;
        try
        {
            loadFromID(persoTask, null);
        }
        catch (Exception e)
        {
            deleted = true;
        }
        assertTrue(deleted);
        
    }
    

    public PersonalTask save(PersonalTask object) throws Exception
    {

        SingleResult result = save(persoTask, null);
        return (PersonalTask) result.getRpmObject();
    }
}
