/**
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.testcases;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;

public class TestStep extends ClientTestController
{
    private StepScope mainScope = null;

    private TaskAssignment taskAssignment = null;

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        mainScope = StepFixture
            .createScope(true, true, false, true, false, true);
        taskAssignment = TaskAssignmentFixture.createSaveFixture(null, this);
        
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithScopeLoadFromxPath() throws AssertionError,
            Exception
    {
        insert(mainScope, false);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        insert(mainScope, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(mainScope, true);
    }

    public void testLoadFromXPathWithResource() throws Exception
    {

        StepScope scope = StepFixture.createScope(true, false, false,
            true, false, true);

        Step step = insertStep(scope, true);

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step[" + " @resource[id = '"
                + step.getResource().getID() + "']]", scope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof Step);
        assertEquals(((Step) objs[0]).getResource().getID(), step
            .getResource().getID());

    }
    
    public void testLoadChildFromXPath_Resource() throws Exception
    {

        StepScope scope = StepFixture.createScope(true, false, false,
            true, false, true);

        Step step = insertStep(scope, true);

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step/" + " @resource[id = '"
                + step.getResource().getID() + "']", null);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof Resource);
        assertEquals(((Resource) objs[0]).getID(), step
            .getResource().getID());

    }

    public void testLoadFromXPathWithProject() throws Exception
    {

        StepScope scope = StepFixture.createScope(true, false, true,
            false, false, true);

        Step step = insertStep(scope, true);

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step[" + " @project[id = '"
                + step.getProject().getID() + "']]", scope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof Step);
        assertEquals(step.getProject().getID(), ((Step) objs[0]).getProject()
            .getID());

    }

    public void testLoadChildFromXPath_Project() throws Exception
    {

        StepScope scope = StepFixture.createScope(true, false, true,
            false, false, true);

        Step step = insertStep(scope, true);

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step/" + " @project[id = '"
                + step.getProject().getID() + "']", null);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof GenericProject);
        assertEquals(step.getProject().getID(), ((GenericProject) objs[0]).getID());

    }
    
    public void testLoadFromXPathWithWorkElement() throws Exception
    {

        StepScope scope = StepFixture.createScope(true, false, false,
            false, true, true);

        Step step = insertStep(scope, true);

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step[" + " @workElement[id = '"
                + step.getWorkElement().getID() + "']]", scope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof Step);
        assertEquals(step.getWorkElement().getID(), ((Step) objs[0])
            .getWorkElement().getID());

    }
    
    public void testLoadChildFromXPath_WorkElement() throws Exception
    {

        StepScope scope = StepFixture.createScope(true, false, false,
            false, true, true);

        Step step = insertStep(scope, true);

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step/" + " @workElement[id = '"
                + step.getWorkElement().getID() + "']", null);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof WorkElement);
        assertEquals(step.getWorkElement().getID(), ((WorkElement) objs[0])
            .getID());

    }
    
    public void testLoadFromXPathWithTaskAssignment() throws Exception
    {

        StepScope scope = StepFixture.createScope(true, false, false,
            false, true, true);

        Step step = insertStep(scope, true);

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step[" + " @taskAssignment[id = '"
                + step.getTaskAssignment().getID() + "']]", scope);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof Step);
        assertEquals(step.getTaskAssignment().getID(), ((Step) objs[0])
            .getTaskAssignment().getID());

    }
    
    
    public void testLoadChildFromXPath_TaskAssignment() throws Exception
    {

        StepScope scope = StepFixture.createScope(true, false, false,
            false, true, true);

        Step step = insertStep(scope, true);

        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step/" + " @taskAssignment[id = '"
                + step.getTaskAssignment().getID() + "']", null);

        assertNotNull(objs);
        assertTrue(objs[0] instanceof TaskAssignment);
        assertEquals(step.getTaskAssignment().getID(), ((TaskAssignment) objs[0])
            .getID());

    }

    public void testLoadFromXPathWithTimeCode1() throws Exception{
        
        StepScope scope = StepFixture.createScope(true, true, false,
            false, false, false);

        Step step = insertStep(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step[@timecode1[id = '"
                + step.getTimeCode1().getID() + "']]",
                scope);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof Step);
        assertEquals(step.getTimeCode1().getID(),
            ((Step) objs[0]).getTimeCode1().getID());
        
    }
    
    public void testLoadChildFromXPath_TimeCode1() throws Exception{
        
        StepScope scope = StepFixture.createScope(true, true, false,
            false, false, false);

        Step step = insertStep(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step/@timecode1[id = '"
                + step.getTimeCode1().getID() + "']",
                null);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof TimeCode);
        assertEquals(step.getTimeCode1().getID(),
            ((TimeCode) objs[0]).getID());
        
    }
    

    public void testLoadFromXPathWithTimeCode2() throws Exception{
        
        StepScope scope = StepFixture.createScope(true, true, false,
            false, false, false);

        Step step = insertStep(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step[@timecode2[id = '"
                + step.getTimeCode2().getID() + "']]",
                scope);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof Step);
        assertEquals(step.getTimeCode2().getID(),
            ((Step) objs[0]).getTimeCode2().getID());
        
    }
    
    public void testLoadChildFromXPath_TimeCode2() throws Exception{
        
        StepScope scope = StepFixture.createScope(true, true, false,
            false, false, false);

        Step step = insertStep(scope, true);
        
        // load from an xpath
        RPMObject[] objs = loadFromXpath("/Step/@timecode2[id = '"
                + step.getTimeCode2().getID() + "']",
                null);
        
        assertNotNull(objs);
        assertTrue(objs[0] instanceof TimeCode);
        assertEquals(step.getTimeCode2().getID(),
            ((TimeCode) objs[0]).getID());
        
    }
    
    public void testDeleteStep() throws Exception
    {
        //note (gmeloche) this test fails because on a database defect
        
        Calendar week1 = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.TIMESHEET1_WEEK);
        Calendar week2 = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.TIMESHEET2_WEEK);

        //create summaryTimesheet for week 1 and submit it
        SummaryTimesheetScope summaryTimesheetScope = SummaryTimesheetFixture
            .createScope(false, true, false, false, false);
        SummaryTimesheet summaryTimesheet1 = SummaryTimesheetFixture
            .createSaveFixture(taskAssignment, week1, summaryTimesheetScope,
                true, this);

        TimesheetApprovalStatus approvalStatus = summaryTimesheet1
            .getApprovalStatus();
        approvalStatus.setApprovalStatus(TimesheetStatus.Approved);

        save(summaryTimesheet1, summaryTimesheetScope);
        //

        //create summaryTimesheet for week 2
        SummaryTimesheetFixture.createSaveFixture(taskAssignment, week2, null,
            true, this);
        //

        //create Step with stepTimesheet for week 2
        StepTimesheet timesheet = new StepTimesheet();
        timesheet.setWeekOf(week2);

        StepScope stepScope = StepFixture.createScope(true, false, false,
            false, false, false);
        Step step = StepFixture.createSaveFixture(taskAssignment, stepScope,
            new StepTimesheet[]
            {
                timesheet
            }, this);

        delete(step);

        boolean deleted = false;
        try
        {
            loadFromID(step, null);
        }
        catch (Exception e)
        {
            deleted = true;
        }
        assertTrue(deleted);
    }
    
    private void insert(StepScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {

        // insert an element
        insertStep(scope, loadFromId);

    }

    private void update(StepScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        // insert an element
        Step step = insertStep(scope, loadFromId);

        // update the element
        updateStep(step, scope, loadFromId);

    }

    private Step insertStep(StepScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {

        Step step = StepFixture.createFixture(taskAssignment,
            scope, null, this);

        // Insert the element into the DB
        SingleResult result = save(step, scope);      

        // compare the returned element
        Step returnedStep = (Step) result.getRpmObject();
        StepFixture.compare(step, returnedStep, scope);
        
        // reload the element and compare it
        returnedStep = (Step) load(returnedStep, scope, loadFromId);
        StepFixture.compare(step, returnedStep, scope);

        // return the element
        return returnedStep;
    }

    private void updateStep(Step step, StepScope scope,
            boolean loadFromId) throws AssertionError, Exception
    {
        step = StepFixture.updateFixture(step, scope, this);

        // Insert the element into the DB
        SingleResult result = save(step, scope);

        // compare the returned element
        Step returnedStep = (Step) result.getRpmObject();
        StepFixture.compare(step, returnedStep, scope);

        // reload the element and compare it
        returnedStep = (Step) load(returnedStep, scope, loadFromId);
        StepFixture.compare(step, returnedStep, scope);
    }

    private RPMObject load(Step step, StepScope scope,
            boolean loadFromId) throws Exception
    {

        RPMObject object = null;

        if (loadFromId)
        {
            // load from an id
            RPMObject obj = new Step();
            obj.setID(step.getID());
            object = loadFromID(obj, scope);
        }
        else
        {
            // load from an xpath
            RPMObject[] objs = loadFromXpath("Step[id='" + step.getID()
                    + "']", scope);

            if (objs != null || objs.length > 0)
                object = objs[0];
        }

        return object;
    }
}
