package com.ibm.rpm.client.junit.timesheet.testcases;

import java.util.Calendar;

import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestStepBusinessRules extends GenericValidation
{
    private TaskAssignment taskAssignment = null;
    
    private StepScope scope = null;

    private Double workHours = null;

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        scope = StepFixture.createScope(true, true, false, true, false, true);
        
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture.createScope(true, false, false, false);
        taskAssignment = TaskAssignmentFixture.createSaveFixture(taskAssignmentScope, this);
        workHours = new Double(2);
    }

    public void testDefaultValues() throws Exception
    {
        Step step = createStep();
        assertTrue(step.getAlwaysAvailable().booleanValue());
        Double zero = new Double(0);
        assertEquals(zero, step.getEETC());
        assertEquals(zero, step.getWeight());
        assertNotNull(step.getETCChanged());
        assertFalse(step.getETCChanged().booleanValue());
        assertNotNull(step.getAlwaysAvailable());
        assertTrue(step.getAlwaysAvailable().booleanValue());
        assertEquals(zero, step.getEETC());
        // TODO test billable flag
    }

    public void testOneStepWithEETCAndWeight() throws Exception
    {
        Double eetc = new Double(10);
        Double weight = new Double(50);
        Step step = createAndSetupStep(workHours, eetc, weight);

        SummaryTimesheet summaryTimesheet = loadSummaryTimesheet(step
            .getStepTimesheets()[0]);
        assertEquals(workHours, summaryTimesheet.getRegularMonday());
        // %C = stepWorkHours / (stepWorkHours + stepEETC) * stepWeight = 2 / 10
        // * 50% = 10%
        assertEquals(new Double(10), step.getTaskAssignment()
            .getPercentageComplete());
        TaskAssignment loadedTaskAssignment = loadTaskAssignment(step);
        // eetc = original eetc - workHours = 40 - 2 = 38
        assertEquals(new Double(38), loadedTaskAssignment.getEETCHours());
    }

    public void testOneStepWithoutEETCAndWeight() throws Exception
    {
        Step step = createAndSetupStep(workHours, null, null);

        SummaryTimesheet summaryTimesheet = loadSummaryTimesheet(step
            .getStepTimesheets()[0]);
        assertEquals(workHours, summaryTimesheet.getRegularMonday());
        // No eetc and weight, %C = workHours / (totalWorkHours + totalEETC) = 2
        // / 40 = 5%
        assertEquals(new Double(5), step.getTaskAssignment()
            .getPercentageComplete());
        TaskAssignment loadedTaskAssignment = loadTaskAssignment(step);
        // eetc = original eetc - workHours = 40 - 2 = 38
        assertEquals(new Double(38), loadedTaskAssignment.getEETCHours());
    }

    public void testOneStepWithEETC() throws Exception
    {
        Double eetc = new Double(20);
        Step step = createAndSetupStep(workHours, eetc, null);
        SummaryTimesheet summaryTimesheet = loadSummaryTimesheet(step
            .getStepTimesheets()[0]);
        assertEquals(workHours, summaryTimesheet.getRegularMonday());
        // No weight, %C = workHours / (totalWorkHours + totalEETC) = 2 / 40 =
        // 5%
        assertEquals(new Double(5), step.getTaskAssignment()
            .getPercentageComplete());
        TaskAssignment loadedTaskAssignment = loadTaskAssignment(step);
        // eetc = original eetc - workHours = 40 - 2 = 38
        assertEquals(new Double(38), loadedTaskAssignment.getEETCHours());
    }

    public void testOneStepWithWeight() throws Exception
    {
        Double weight = new Double(40);
        Step step = createAndSetupStep(workHours, null, weight);
        SummaryTimesheet summaryTimesheet = loadSummaryTimesheet(step
            .getStepTimesheets()[0]);
        assertEquals(workHours, summaryTimesheet.getRegularMonday());
        // No eetc, then %C = stepWeight
        assertEquals(weight, step.getTaskAssignment().getPercentageComplete());
        TaskAssignment loadedTaskAssignment = loadTaskAssignment(step);
        // eetc = original eetc - workHours = 40 - 2 = 38
        assertEquals(new Double(38), loadedTaskAssignment.getEETCHours());
    }

    public void testMoreStepsAllWithEETC() throws Exception
    {
        Step step1 = createAndSetupStep(workHours, new Double(10), null);
        Step step2 = createAndSetupStep(workHours, new Double(12), null);
        Step step3 = createAndSetupStep(workHours, new Double(18), null);
        SummaryTimesheet summaryTimesheet = loadSummaryTimesheet(step1
            .getStepTimesheets()[0]);
        assertEquals(new Double(6), summaryTimesheet.getRegularMonday());
        TaskAssignment loadedTaskAssignment = loadTaskAssignment(step1);
        // No weight, %C = totalWorkHours / (totalWorkHours + totalEETC) = 6 /
        // 40 = 15%
        assertEquals(new Double(15), loadedTaskAssignment
            .getPercentageComplete());
        // eetc = original eetc - workHours = 40 - 2 * 3 = 34
        assertEquals(new Double(34), loadedTaskAssignment.getEETCHours());
    }

    public void testMoreStepsAllwithWeight() throws Exception
    {
        Step step1 = createAndSetupStep(workHours, null, new Double(10));
        Step step2 = createAndSetupStep(workHours, null, new Double(12));
        Step step3 = createAndSetupStep(workHours, null, new Double(18));
        SummaryTimesheet summaryTimesheet = loadSummaryTimesheet(step1
            .getStepTimesheets()[0]);
        assertEquals(new Double(6), summaryTimesheet.getRegularMonday());
        TaskAssignment loadedTaskAssignment = loadTaskAssignment(step1);
        // No eetc, %C = totalWeight = 10 + 12 + 18 = 40%
        assertEquals(new Double(40), loadedTaskAssignment
            .getPercentageComplete());
        // eetc = original eetc - workHours = 40 - 2 * 3 = 34
        assertEquals(new Double(34), loadedTaskAssignment.getEETCHours());
    }

    public void testMoreStepsAllWithoutEETCAndWeight() throws Exception
    {
        Step step1 = createAndSetupStep(workHours, null, null);
        Step step2 = createAndSetupStep(workHours, null, null);
        Step step3 = createAndSetupStep(workHours, null, null);
        SummaryTimesheet summaryTimesheet = loadSummaryTimesheet(step1
            .getStepTimesheets()[0]);
        assertEquals(new Double(6), summaryTimesheet.getRegularMonday());
        TaskAssignment loadedTaskAssignment = loadTaskAssignment(step1);
        // No eetc and weight, %C = totalWorkHours / (totalWorkHours +
        // totalEETC) = 6 / 40 = 15%
        assertEquals(new Double(15), loadedTaskAssignment
            .getPercentageComplete());
        // eetc = original eetc - workHours = 40 - 2 * 3 = 34
        assertEquals(new Double(34), loadedTaskAssignment.getEETCHours());
    }

    public void testMoreStepsAllWithEETCAndWeight() throws Exception
    {
        Step step1 = createAndSetupStep(workHours, new Double(10), new Double(
            50));
        Step step2 = createAndSetupStep(workHours, new Double(20), new Double(
            30));
        Step step3 = createAndSetupStep(workHours, new Double(10), new Double(
            20));
        SummaryTimesheet summaryTimesheet = loadSummaryTimesheet(step1
            .getStepTimesheets()[0]);
        assertEquals(new Double(6), summaryTimesheet.getRegularMonday());
        TaskAssignment loadedTaskAssignment = loadTaskAssignment(step1);
        // %C = step1WorkHours / (step1WorkHours + step1EETC) * step1Weight +
        // step2WorkHours / (step2WorkHours + step2EETC) * step2Weight +
        // step3WorkHours / (step3WorkHours + step3EETC) * step3Weight = 2 / 10
        // * 50% + 2 / 20 * 30% + 2 / 10 * 20% = 17%
        assertEquals(new Double(17), loadedTaskAssignment
            .getPercentageComplete());
        // eetc = original eetc - workHours = 40 - 2 * 3 = 34
        assertEquals(new Double(34), loadedTaskAssignment.getEETCHours());
    }

    public void testWithMoreSteps() throws Exception
    {
        Step step1 = createAndSetupStep(workHours, null, new Double(50));
        Step step2 = createAndSetupStep(workHours, new Double(20), null);
        Step step3 = createAndSetupStep(workHours, new Double(10), new Double(
            20));
        Step step4 = createAndSetupStep(workHours, null, null);
        SummaryTimesheet summaryTimesheet = loadSummaryTimesheet(step1
            .getStepTimesheets()[0]);
        assertEquals(new Double(8), summaryTimesheet.getRegularMonday());
        TaskAssignment loadedTaskAssignment = loadTaskAssignment(step1);
        // step1%C = step1Weight = 50% since we don't have eetc
        // step2%C = step2WorkHours / (step2WorkHours + step2EETC) * step2Weight
        // = 2 / 20 * ((2 + 18) / (8 + 32 - 12) * (100 - 50 - 20)) = 58.28%
        assertEquals(new Double(58.28), loadedTaskAssignment
            .getPercentageComplete());
        // eetc = original eetc - workHours = 40 - 2 * 4 = 32
        assertEquals(new Double(32), loadedTaskAssignment.getEETCHours());
    }
    
    public void testFailureInsertWithTimecodeAssignedToSummaryTimesheet()
            throws Exception
    {
        // assign timecode to task
        assertNotNull(taskAssignment.getParent());
        assertNotNull(taskAssignment.getParent().getParent());
        assertTrue(taskAssignment.getParent().getParent() instanceof Task);
        Task task = (Task) taskAssignment.getParent().getParent();
        TimeCodeAssignment timeCodeAssignment = TimeCodeAssignmentFixture.createFixture(task, true, this);
        task.setTimeCodeAssignments(new TimeCodeAssignment[]
        {
                timeCodeAssignment
        });
        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setTimeCodeAssignments(true);
        workElementScope.setTimeCodes(true);
        SingleResult result = save(task, workElementScope);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof Task);
        task = (Task) result.getRpmObject();
        assertNotNull(task.getTimeCodeAssignments());
        
        Calendar assignmentStartDate = taskAssignment.getAssignmentStartDate();
        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(assignmentStartDate,
            TimesheetTestUtil.TIMESHEET1_WEEK);

        // create SummaryTimesheet
        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);
        summaryTimesheet.setTimeCode1(timeCodeAssignment.getTimeCode());

        // do both copy proposed to plan and publish project to the team
        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);
        TimesheetTestUtil.initProject(project, this);

        SummaryTimesheetScope summaryTimesheetScope = new SummaryTimesheetScope();
        summaryTimesheetScope.setTimeCodes(true);
        result = save(summaryTimesheet, summaryTimesheetScope);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet)result.getRpmObject();
        assertNotNull(summaryTimesheet.getTimeCode1());
        
        Step step = new Step();
        step.setTaskAssignment(taskAssignment);

        // timesheets is a mandatory field
        StepTimesheet stepTimesheet = new StepTimesheet();
        stepTimesheet.setParent(step);
        stepTimesheet.setWeekOf(weekOf);
        StepTimesheet[] stepTimesheets = new StepTimesheet[1];
        stepTimesheets[0] = stepTimesheet;
        step.setStepTimesheets(stepTimesheets);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.TIMECODE_ASSIGNED_TO_SUMMARY_TIMESHEET,
                StepTimesheet.class.getName())
        };
        verifySaveMustFail(step, scope, expectedExceptions);
    }

    private SummaryTimesheet loadSummaryTimesheet(StepTimesheet stepTimesheet)
            throws Exception
    {
        assertNotNull(stepTimesheet);
        assertNotNull(stepTimesheet.getParent());
        assertTrue(stepTimesheet.getParent() instanceof Step);
        TaskAssignment taskAssignment = ((Step) stepTimesheet.getParent())
            .getTaskAssignment();
        assertNotNull(taskAssignment);
        String taskAssignmentId = taskAssignment.getID();

        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setID(taskAssignmentId);
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(stepTimesheet.getWeekOf());

        // reload the summaryTimesheet
        summaryTimesheet = (SummaryTimesheet) loadFromID(summaryTimesheet, null);
        assertNotNull(summaryTimesheet);
        return summaryTimesheet;
    }

    private TaskAssignment loadTaskAssignment(Step step) throws Exception
    {
        assertNotNull(step);
        assertNotNull(step.getTaskAssignment());
        assertNotNull(step.getTaskAssignment().getID());
        TaskAssignment taskAssignment = (TaskAssignment) loadFromID(step
            .getTaskAssignment(), null);
        return taskAssignment;
    }

    private Step createAndSetupStep(Double workHours, Double eetc, Double weight)
            throws Exception
    {
        Step step = createStep();
        if (weight != null)
        {
            step.setWeight(weight);
        }
        if (eetc != null)
        {
            step.setEETC(eetc);
        }
        StepTimesheet stepTimesheet = step.getStepTimesheets()[0];
        stepTimesheet.setRegularMonday(workHours);

        step = saveStep(step, scope);
        assertNotNull(step.getStepTimesheets());
        assertEquals(1, step.getStepTimesheets().length);
        assertEquals(workHours, step.getStepTimesheets()[0].getRegularMonday());
        if (weight == null)
        {
            assertEquals(new Double(0), step.getWeight());
        }
        if (eetc != null)
        {
            assertTrue(step.getETCChanged().booleanValue());
        }

        return step;
    }

    private Step createStep() throws Exception
    {
        Calendar assignmentStartDate = taskAssignment.getAssignmentStartDate();
        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(assignmentStartDate,
            TimesheetTestUtil.TIMESHEET1_WEEK);

        // create SummaryTimesheet
        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        // do both copy proposed to plan and publish project to the team
        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);
        TimesheetTestUtil.initProject(project, this);

        SingleResult result = save(summaryTimesheet, null);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof SummaryTimesheet);

        Step step = new Step();
        step.setTaskAssignment(taskAssignment);

        // timesheets is a mandatory field
        StepTimesheet stepTimesheet = new StepTimesheet();
        stepTimesheet.setParent(step);
        stepTimesheet.setWeekOf(weekOf);
        StepTimesheet[] stepTimesheets = new StepTimesheet[1];
        stepTimesheets[0] = stepTimesheet;
        step.setStepTimesheets(stepTimesheets);

        step = saveStep(step, scope);

        assertNotNull(step.getStepTimesheets());
        assertEquals(1, step.getStepTimesheets().length);

        return step;
    }
    
    private Step saveStep(Step step, StepScope stepScope) throws Exception
    {
        SingleResult result = save(step, stepScope);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof Step);

        return (Step) result.getRpmObject();
    }
}
