/**
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.testcases;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetApprovalStatusFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class TestTimesheetApprovalStatus extends ClientTestController
{
    private TimesheetApprovalStatusScope mainScope = null;

    private Resource resource = null;
    private Calendar weekOf = null;
    private TaskAssignment taskAssignment = null;
    SummaryTimesheet summaryTimesheet = null;
    
    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        
        mainScope = TimesheetApprovalStatusFixture.createScope(true, true, true);
        
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture.
        	createScope(true, false, false, false);
        taskAssignment = TaskAssignmentFixture.createSaveFixture(
            taskAssignmentScope, this);
        
        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        SummaryTimesheetScope timesheetScope = SummaryTimesheetFixture.createScope(
            true, false, false, false, false);
        summaryTimesheet = SummaryTimesheetFixture.createSaveFixture(
            taskAssignment, weekOf, timesheetScope, true, this);
        
        resource = ((ResourceTaskAssignment) taskAssignment.getParent())
            .getResource();
        
        resource.getResourceSecurityRights().setCanUseWebServicesAPI(Boolean.TRUE);
        
        resource = (Resource)save(resource, null).getRpmObject();
        

    }
    
    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithNoScopeLoadFromXPath() 
    	throws AssertionError, Exception
    {
        insert(null, false);
    }
    
    public void testInsertWithNoScopeLoadFromID() 
    	throws AssertionError, Exception
    {
        insert(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError, Exception
    {
        insert(mainScope, true);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError, Exception
    {       
        update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError, Exception
    {
        update(mainScope, true);
    }
    
    public void testLoadFromXpathWithResource() throws Exception {

		TimesheetApprovalStatus status = insertTimesheetApprovalStatus(
				mainScope, true);

		String xpath = "/TimesheetApprovalStatus[" + " @resource[id = '"
				+ status.getResource().getID() + "']]";
		// load from an xpath
		loadFromXpathAndValidate(xpath);
	}

	public void testLoadFromXpathWithProject() throws Exception {

		TimesheetApprovalStatus status = insertTimesheetApprovalStatus(
				mainScope, true);

		String xpath = "/TimesheetApprovalStatus[" + " GenericProject[id = '"
				+ status.getProject().getID() + "']]";
		// load from an xpath
		loadFromXpathAndValidate(xpath);
	}

	public void testLoadFromXpathWithProjectAndResource() throws Exception {

		TimesheetApprovalStatus status = insertTimesheetApprovalStatus(
				mainScope, true);

		String xpath = "/TimesheetApprovalStatus[@resource[@id= '"
				+ status.getResource().getID()
				+ "'] and GenericProject[@id = '" + status.getProject().getID()
				+ "']]";
		// load from an xpath using IDs
		loadFromXpathAndValidate(xpath);

		xpath = "/TimesheetApprovalStatus[@resource[@username= '"
				+ status.getResource().getUserName()
				+ "'] and GenericProject[@name = '"
				+ ((GenericProject) status.getProject()).getName() + "']]";
		// load from an xpath using names
		loadFromXpathAndValidate(xpath);
	}

	public void testLoadFromXpathWithProjectAndResourceAndWeekOf()
			throws Exception {

		TimesheetApprovalStatus status = insertTimesheetApprovalStatus(
				mainScope, true);

		String weekOfValue = weekOf.get(Calendar.MONTH) + 1 + "/"
				+ status.getWeekOf().get(Calendar.DATE) + "/"
				+ weekOf.get(Calendar.YEAR);
		String xpath = "/TimesheetApprovalStatus[@resource[@id= '"
				+ status.getResource().getID()
				+ "'] and GenericProject[@id = '" + status.getProject().getID()
				+ "'] and @weekOf = '" + weekOfValue + "']";

		// load from an xpath
		loadFromXpathAndValidate(xpath);
	}

	private void loadFromXpathAndValidate(String xpath) throws AssertionError,
			Exception {
		RPMObject[] objs = loadFromXpath(xpath, mainScope);

		assertNotNull(objs);
		assertTrue(objs[0] instanceof TimesheetApprovalStatus);
	}

    private void insert(TimesheetApprovalStatusScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        // insert an element
        insertTimesheetApprovalStatus(scope, loadFromId);

        // Do not delete the element. It gets deleted when its summary timesheet
        // is deleted
    }

    private void update(TimesheetApprovalStatusScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        // insert an element
        TimesheetApprovalStatus appStatus = insertTimesheetApprovalStatus(scope, 
            loadFromId);

        // update the element
        updateTimesheetApprovalStatus(appStatus, scope, loadFromId);

        /// Do not delete the element. It gets deleted when its summary timesheet
        // is deleted
    }

    private TimesheetApprovalStatus insertTimesheetApprovalStatus(
            TimesheetApprovalStatusScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        TimesheetApprovalStatus appStatus = new TimesheetApprovalStatus();
        
        appStatus.setParent(summaryTimesheet);
        appStatus.setResource(resource);
        appStatus.setWeekOf(weekOf);
        
        // submit the timesheet for approval
        TimesheetApprovalStatus returnedAppStatus = TimesheetApprovalStatusFixture.
        	submitTimesheetAndSave(appStatus, scope, true, this);

        // compare the returned element
        TimesheetApprovalStatusFixture.compare(appStatus, returnedAppStatus, 
            scope);

        // reload the element and compare it
        returnedAppStatus = (TimesheetApprovalStatus) load(returnedAppStatus, 
            scope, loadFromId);
        TimesheetApprovalStatusFixture.compare(appStatus, returnedAppStatus, 
            scope);

        // return the element
        return returnedAppStatus;
    }

    private void updateTimesheetApprovalStatus(TimesheetApprovalStatus appStatus,
            TimesheetApprovalStatusScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        // Update the values
        TimesheetApprovalStatusFixture.updateFixture(appStatus);

        // Insert the element into the DB
        SingleResult result = save(appStatus, scope);

        // compare the returned element
        TimesheetApprovalStatus returnedAppStatus = 
            (TimesheetApprovalStatus) result.getRpmObject();
        TimesheetApprovalStatusFixture.compare(appStatus, returnedAppStatus,
            scope);

        // reload the element and compare it
        returnedAppStatus = (TimesheetApprovalStatus) load(returnedAppStatus,
            scope, loadFromId);
        TimesheetApprovalStatusFixture.compare(appStatus, returnedAppStatus,
            scope);
        
        // Approve the timesheet
        TimesheetApprovalStatus savedAppStatus = 
            TimesheetApprovalStatusFixture.approveTimesheetAndSave(
                returnedAppStatus, scope, this);

        // Re-load it and compare
        TimesheetApprovalStatus loadedAppStatus = 
            (TimesheetApprovalStatus) load(savedAppStatus, scope, loadFromId);
        TimesheetApprovalStatusFixture.compare(savedAppStatus, loadedAppStatus,
            scope);
        
        // Reject the timesheet
        savedAppStatus = TimesheetApprovalStatusFixture.rejectTimesheetAndSave(
            savedAppStatus, scope, this);
        
        // Re-load it and compare
        loadedAppStatus = (TimesheetApprovalStatus) load(savedAppStatus,
            scope, loadFromId);
        TimesheetApprovalStatusFixture.compare(savedAppStatus, loadedAppStatus,
            scope);
        
        // Re-submit the timesheet
        savedAppStatus = TimesheetApprovalStatusFixture.
        	submitTimesheetAndSave(savedAppStatus, scope, false, this);
        
        // Re-load it and compare
        loadedAppStatus = (TimesheetApprovalStatus) load(savedAppStatus,
            scope, loadFromId);
        TimesheetApprovalStatusFixture.compare(savedAppStatus, loadedAppStatus,
            scope);
    }

    private RPMObject load(TimesheetApprovalStatus appStatus, 
            TimesheetApprovalStatusScope scope, boolean loadFromId) 
    	throws Exception
    {
        RPMObject object = null; 
        
        if (loadFromId)
        {
            // load from an id
            TimesheetApprovalStatus obj = new TimesheetApprovalStatus();
            obj.setID(appStatus.getID());
            obj.setResource(appStatus.getResource());
            obj.setWeekOf(appStatus.getWeekOf());
            object =  loadFromID(obj, scope);
        }
        else
        {
            // load from an xpath
            RPMObject[] objs = loadFromXpath(
                "/TimesheetApprovalStatus[@resource[Id='" + 
                appStatus.getResource().getID() + "'] and GenericProject[Id='" + 
                appStatus.getID() + "'] and weekOf='" + 
                DateUtil.convertToString(appStatus.getWeekOf()) + "']", scope);

            if (objs != null && objs.length > 0){
                object =  objs[0];
            }
        }
        
        return object;
    }
}
