package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;
import com.ibm.rpm.wbs.types.ResourceType;

/**
 * @author gmeloche
 *
 */
public class Test50576 extends ClientTestController
{
    public static String projectName = null;

    public static String taskName = null;

    public static String resourceName = null;

    public static String poolName = null;
    
    public static Project project = null;

    public static Task task = null;

    public static Resource resource = null;
    
    private static TaskAssignment taskAssignment = null;

    
    public void testLoadSummaryTimesheetWithApprovalStatus() throws Exception{
        
        projectName = HelperTestUtilities.createName();
        taskName = HelperTestUtilities.createName();
        resourceName = HelperTestUtilities.createName();
        poolName = HelperTestUtilities.createName();
       
        createResource();
        createTask();
        assignResourceToTask();        
        
        //Load an existing TaskAssignment from xpath

        String xpath = "Task[id = '" + task.getID()
                + "']/ResourceTaskAssignment[@resource[id = '"
                + resource.getID() + "']]/TaskAssignment";

        RPMObject[] objects = loadFromXpath(xpath, null);
        assertNotNull(objects);
        
        taskAssignment = (TaskAssignment)objects[0];

        //create a SummaryTimesheet with an approvalStatus
        
        System.out.println("\nCreate a SummaryTimesheet");
        
        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();

        //Set the mandatory fields
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(taskAssignment.getAssignmentStartDate());
        summaryTimesheet.setRegularMonday(new Double(8));
        
        
        //Set approvalStatus
        TimesheetApprovalStatus approvalStatus = new TimesheetApprovalStatus();
        approvalStatus.setResource(resource);
        approvalStatus.setProject(project);
        approvalStatus.setApprovalStatus(TimesheetStatus.Submitted);
        summaryTimesheet.setApprovalStatus(approvalStatus);
        
        //Add approvalStatus to scope
        SummaryTimesheetScope stScope = new SummaryTimesheetScope();
        stScope.setApprovalStatus(new TimesheetApprovalStatusScope());

        SingleResult result1 = save(summaryTimesheet,stScope);
        
        assertNotNull(result1.getRpmObject());
        
        SummaryTimesheet loadedTimesheet = (SummaryTimesheet)result1.getRpmObject();
        assertNotNull(loadedTimesheet.getApprovalStatus());
    }
    
    
    
    
    private void createResource() throws Exception
    {
        Pool pool = new Pool();
        pool.setName(poolName);

        resource = new Resource();
        resource.setFullName(resourceName);

        //Load the default calendar
        RPMObject[] objects = loadFromXpath(
            "/RPMCalendar[name = 'Standard (Default)']", null);
        assertNotNull(objects);
        RPMCalendar calendar = (RPMCalendar)objects[0];

        //Resource calendar is mandatory to assign a resource
        resource.setCalendar(calendar);

        pool.setResources(new Resource[]
        {
            resource
        });

        PoolScope poolScope = new PoolScope();
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        poolScope.setResources(resourceScope);

        SingleResult result2 = save(pool, poolScope);
        assertNotNull(result2.getRpmObject());
        pool = (Pool) result2.getRpmObject();
        resource = pool.getResources()[0];

    }

    private void createTask() throws Exception
    {
        project = new Project();
        project.setName(projectName);

        //Publish project to team (mandatory for timesheet creation)
        project.setPublished(ProjectPublishedType.ToTeam);

        task = new Task();
        task.setName(taskName);

        project.setChildren(new Task[]
        {
            task
        });

        //Create projectScope
        WorkElementScope projectScope = new WorkElementScope();
        WorkElementScope taskScope = new WorkElementScope();

        //Contaning project is mandatory to create a ResourceTaskAssignment
        taskScope.setContainingProject(new WorkElementScope());
        projectScope.setChildren(taskScope);

        SingleResult result1 = save(project, projectScope);
        assertNotNull(result1.getRpmObject());
        project = (Project) result1.getRpmObject();
        task = (Task) project.getChildren()[0];
    }

    private void assignResourceToTask() throws Exception
    {
        //Create resourceTaskAssignment
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();

        //Load the security role
        RPMObject[] objects = loadFromXpath("/SecurityRole[name = 'Task Responsible']", null);
        assertNotNull(objects);
        SecurityRole securityRole = (SecurityRole) objects[0];

        resourceTaskAssignment.setSecurityRole(securityRole);
        resourceTaskAssignment.setResource(resource);

        //Create TaskAssignment
        TaskAssignment taskAssignment = new TaskAssignment();
        resourceTaskAssignment.setTaskAssignment(taskAssignment);
        task.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            resourceTaskAssignment
        });

        //Create task scope
        WorkElementScope taskScope = new WorkElementScope();

        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setResource(new ResourceScope());
        TaskAssignmentScope taScope = new TaskAssignmentScope();
        rtaScope.setTaskAssignment(taScope);
        taskScope.setResourceTaskAssignments(rtaScope);

        //Save Task
        SingleResult result2 = save(task, taskScope);
        assertNotNull(result2.getRpmObject());
        
        task = (Task)result2.getRpmObject();

        //Copy Project proposed dates to plan dates (mandatory for timesheet
        // creation)
        SingleResult result3 = copyProposedToPlan(project, null);
        assertNotNull(result3.getRpmObject());
        
        
    }
}
