package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.wbs.containers.TaskAssignment;

/**
 * @author gmeloche
 *  
 */
public class Test52296 extends ClientTestController
{

    TaskAssignment taskAssignment = null;

    Calendar weekOf1 = null;

    Calendar weekOf2 = null;

    protected void setUp() throws Exception
    {
        super.setUp();

        //Create a step with 2 summaryTimesheets and a StepTimesheet for the first week
        taskAssignment = TaskAssignmentFixture.createSaveFixture(null, this);

        weekOf1 = TimesheetTestUtil.getValidWeekOf(taskAssignment.getAssignmentStartDate(), 1);

        weekOf2 = TimesheetTestUtil.getValidWeekOf(weekOf1, 2);

        //summary timesheet week 1

        SummaryTimesheetFixture.createSaveFixture(taskAssignment, weekOf1,
            null, false, this);

        //summary timesheet week 2

        SummaryTimesheetFixture.createSaveFixture(taskAssignment, weekOf2,
            null, false, this);

        //step with step timesheet for week 1

        Step step = new Step();
        step.setTaskAssignment(taskAssignment);
        StepTimesheet timesheet1 = new StepTimesheet();
        timesheet1.setWeekOf(weekOf1);
        step.setStepTimesheets(new StepTimesheet[]
        {
            timesheet1
        });

        StepScope stepScope = new StepScope();
        stepScope.setTimesheets(true);

        save(step, stepScope);

    }

    public void testUpdateStepWithNewStepTimesheet() throws Exception
    {

        //load existing step from xpath without scope
        String xpath = "Step[taskassignment[id = '" + taskAssignment.getID()
                + "']]";

        RPMObject[] objs = loadFromXpath(xpath, null);
        assertNotNull(objs);
        assertTrue(objs[0] instanceof Step);
        Step step = (Step) objs[0];

        //Save existing Step with a new StepTimesheet
        
        StepScope scope = new StepScope();
        scope.setTimesheets(true);

        StepTimesheet timesheet = new StepTimesheet();
        timesheet.setWeekOf(weekOf2);
        step.setStepTimesheets(new StepTimesheet[]
        {
            timesheet
        });

        SingleResult result = save(step, scope);

        assertNotNull(result.getRpmObject());
        step = (Step) result.getRpmObject();
        assertNotNull(step.getStepTimesheets());
        
        //check that both StepTimesheets are loaded.
        assertEquals(2, step.getStepTimesheets().length);

    }
}
