package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;

public class Test65396 extends ClientTestController
{
    private TaskAssignment taskAssignment = null;

    private StepScope scope = null;
    
    private StepTimesheet stepTimesheet = null;

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        scope = StepFixture.createScope(true, true, false, true, false, true);
        taskAssignment = TaskAssignmentFixture.createSaveFixture(null, this);
    }

    public void test() throws Exception
    {
        Step step = createStep();
        stepTimesheet = step.getStepTimesheets()[0];
        
        // change hours
        setHoursAndValidate(new Double(2.0));
        
        // change hours
        setHoursAndValidate(new Double(4.0));
    }
    
    private void setHoursAndValidate(Double workHours) throws Exception
    {
        stepTimesheet.setRegularFriday(workHours);
        SingleResult result = this.save(stepTimesheet, null);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof StepTimesheet);
        stepTimesheet = (StepTimesheet)result.getRpmObject();
        assertEquals(workHours, stepTimesheet.getRegularFriday());
    }

    private Step createStep() throws Exception
    {
        Calendar assignmentStartDate = taskAssignment.getAssignmentStartDate();
        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(assignmentStartDate,
            TimesheetTestUtil.TIMESHEET1_WEEK);

        // create SummaryTimesheet
        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setTaskAssignment(taskAssignment);
        summaryTimesheet.setWeekOf(weekOf);

        // do both copy proposed to plan and publish project to the team
        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);
        TimesheetTestUtil.initProject(project, this);

        SingleResult result = save(summaryTimesheet, null);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        Step step = new Step();
        step.setTaskAssignment(taskAssignment);

        // timesheets is a mandatory field
        StepTimesheet stepTimesheet = new StepTimesheet();
        stepTimesheet.setParent(step);
        stepTimesheet.setWeekOf(weekOf);
        StepTimesheet[] stepTimesheets = new StepTimesheet[1];
        stepTimesheets[0] = stepTimesheet;
        step.setStepTimesheets(stepTimesheets);

        result = save(step, scope);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof Step);

        step = (Step)result.getRpmObject();
        assertNotNull(step.getStepTimesheets());
        assertEquals(1, step.getStepTimesheets().length);
        
        return (Step)result.getRpmObject();
    }
}
