package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultAdministrativeTaskFixture;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.AdministrativeProject;
import com.ibm.rpm.timesheet.scope.AdministrativeProjectScope;
import com.ibm.rpm.timesheet.scope.PersonalProjectScope;


public class TestAdministrativeProjectValidation extends GenericValidation
{
    AdministrativeProject adminProject = null;
    Resource resource = null;
    Pool pool = null;
       
    
    public void testFailureOnCreate() throws AssertionError, Exception
    {
        adminProject = new AdministrativeProject();
        adminProject.setID(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_CREATE,
                adminProject.getClass().getName()),
        };

        verifySaveMustFail(adminProject, null, expectedExceptions);
    }
    
    
    public void testFailureReadOnly() throws AssertionError, Exception
    {
        init();
        
        adminProject.setName("new name");

        RPMExceptionMock[] expectedWarning =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, adminProject.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, adminProject
                    .getID()),
        };

        verifySaveMustFail(adminProject, null, null, expectedWarning);
    }
    
    private void init()throws Exception{
        
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setCalendar(new CalendarScope());
        
        resource = ResourceFixture.createSaveFixture(scope, this);
        
        pool = (Pool)resource.getParent();
        
        //add a default administrative task to parent pool
        DefaultAdministrativeTaskFixture.createSaveFixture(pool, this);
        
        PersonalProjectScope pps = new PersonalProjectScope();
        pps.setAdministrativeProject(new AdministrativeProjectScope());
        
        
        //Add personalProject structure to resource scope
        scope.setPersonalProject(pps);
        
        resource = (Resource)loadFromID(resource, scope);
        
        adminProject = resource.getPersonalProject().getAdministrativeProject();
    }    

}
