package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultAdministrativeTaskFixture;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.AdministrativeTask;
import com.ibm.rpm.timesheet.scope.AdministrativeProjectScope;
import com.ibm.rpm.timesheet.scope.PersonalProjectScope;


public class TestAdministrativeTaskValidation extends GenericValidation
{
    AdministrativeTask adminTask = null;
    Resource resource = null;
    Pool pool = null;
       
    
    public void testFailureOnCreate() throws AssertionError, Exception
    {
        adminTask = new AdministrativeTask();
        adminTask.setID(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_CREATE,
                adminTask.getClass().getName()),
        };

        verifySaveMustFail(adminTask, null, expectedExceptions);
    }
    
    
    public void testFailureReadOnlyFields() throws Exception
    {
        init();
        
        adminTask.setName(HelperTestUtilities.createName());

        adminTask.setActual(new Double(5));
        adminTask.setEETC(new Double(3));
        adminTask.setFinishDate(new GregorianCalendar());
        adminTask.setPercentageCompleted(new Double(30));
        adminTask.setPercentageWork(new Double(25));
        adminTask.setStartDate(new GregorianCalendar());

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, adminTask
                    .getClass().getName(), ValidationConstants.NAME_FIELD,
                    adminTask.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, adminTask
                    .getClass().getName(), ValidationConstants.ACTUAL_FIELD,
                    adminTask.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, adminTask
                    .getClass().getName(), ValidationConstants.EETC_FIELD,
                    adminTask.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, adminTask
                    .getClass().getName(),
                    ValidationConstants.FINISH_DATE_FIELD, adminTask.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, adminTask
                    .getClass().getName(),
                    ValidationConstants.PERCENTAGE_COMPLETED_FIELD,
                    adminTask.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, adminTask
                    .getClass().getName(),
                    ValidationConstants.PERCENTAGE_WORK_FIELD, adminTask
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, adminTask
                    .getClass().getName(),
                    ValidationConstants.START_DATE_FIELD, adminTask.getID()),
        };

        verifySaveMustFail(adminTask, null, null, expectedWarnings);

    }
    
    private void init()throws Exception{
        
        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.setCalendar(new CalendarScope());
        
        resource = ResourceFixture.createSaveFixture(scope, this);
        
        pool = (Pool)resource.getParent();
        
        //add a default administrative task to parent pool
        DefaultAdministrativeTaskFixture.createSaveFixture(pool, this);
        
        AdministrativeProjectScope aps = new AdministrativeProjectScope();
        aps.setAdministrativeTasks(true);
        
        PersonalProjectScope pps = new PersonalProjectScope();
        pps.setAdministrativeProject(aps);
        
        
        //Add personalProject structure to resource scope
        scope.setPersonalProject(pps);
        
        resource = (Resource)loadFromID(resource, scope);
        
        adminTask = resource.getPersonalProject().getAdministrativeProject().getAdministrativeTasks()[0];
    }    

}
