package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.timesheet.fixtures.CreditFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.CreditTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class TestCreditValidation extends GenericValidation
{
    private SummaryTimesheet summaryTimesheet = null;

    private SummaryTimesheetScope summaryTimesheetScope = null;

    private StepScope creditScope = StepFixture.createScope(true, true, true, true,
        true, true);

    private Credit credit = new Credit();

    private TaskAssignment taskAssignment = null;
    
    private Calendar creditWeekOf = null;

    private Calendar weekOf = null;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);
        credit.setParent(portfolio);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MUST_ALWAYS_BE_NULL, credit
                .getClass().getName(), ValidationConstants.PARENT_FIELD, credit
                .getID())
        };

        verifySaveMustFail(credit, null, expectedExceptions);
    }

    public void testInsertWithoutTaskAssignment() throws AssertionError,
            Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, credit.getClass()
                .getName(), ValidationConstants.TASK_ASSIGNMENT_FIELD, credit
                .getID())
        };

        verifySaveMustFail(credit, null, expectedExceptions);
    }
    
    public void testInsertWithTaskAssignmentWithNullId() throws AssertionError,
            Exception
    {
        //set new TaskAssignment
        credit.setTaskAssignment(new TaskAssignment());
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, credit.getClass()
                .getName(), ValidationConstants.TASK_ASSIGNMENT_FIELD, credit
                .getID())
        };


        verifySaveMustFail(credit, null, expectedExceptions);
    }

    public void testInsertWithoutTimesheets() throws AssertionError, Exception
    {

        initSummaryTimesheetAndTaskAssignment(true);

        credit.setTaskAssignment(taskAssignment);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, credit.getClass()
                .getName(), ValidationConstants.CREDIT_TIMESHEET_FIELD, credit
                .getID())
        };

        verifySaveMustFail(credit, creditScope, expectedExceptions);
    }

    public void testInsertWithNoSummaryTimesheet() throws AssertionError, Exception
    {
        initSummaryTimesheetAndTaskAssignment(true);

        credit.setTaskAssignment(taskAssignment);

        // set the mandatory field
        CreditTimesheet timesheet = new CreditTimesheet();
        timesheet.setWeekOf(weekOf);
        timesheet.setCreditWeekOf(creditWeekOf);
        
        credit.setCreditTimesheet(timesheet);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.NO_SUMMARY_TIMESHEET_FOR_WEEKOF,
                timesheet.getClass().getName())
        };

        verifySaveMustFail(credit, creditScope, expectedExceptions);
        
    }
    
    public void testInsertWithoutCreditTimesheet() throws AssertionError, Exception
    {
        initSummaryTimesheetAndTaskAssignment(false);
        approvedSummaryTimesheet();

        credit.setTaskAssignment(taskAssignment);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, credit.getClass()
                .getName(), ValidationConstants.CREDIT_TIMESHEET_FIELD, credit.getID())
        };

        verifySaveMustFail(credit, creditScope, expectedExceptions);
    }
    
    public void testFailureCreditHoursExceedActualHours() throws AssertionError, Exception
    {
        initSummaryTimesheetAndTaskAssignment(false);
        approvedSummaryTimesheet();

        // Set the mandatory fields
        credit.setTaskAssignment(taskAssignment);
        credit.setCreditTimesheet(createTimesheet());
        
        initStep();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_ACTUAL_HOURS,
                CreditTimesheet.class.getName()),
        };

        verifySaveMustFail(credit, creditScope, expectedExceptions, null);

    }

    public void testFailureReadOnlyFields() throws AssertionError, Exception
    {
        initSummaryTimesheetAndTaskAssignment(true);
        approvedSummaryTimesheet();
    
        // Set the mandatory fields
        credit.setTaskAssignment(taskAssignment);
        credit.setCreditTimesheet(createTimesheet());
        
        initStep();
    
        credit.setResource(new Resource());
        credit.setProject(new Project());
        credit.setWorkElement(new Task());
    
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, credit
                    .getClass().getName(), ValidationConstants.RESOURCE_FIELD,
                    credit.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, credit
                    .getClass().getName(), ValidationConstants.PROJECT_FIELD,
                    credit.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, credit
                    .getClass().getName(),
                    ValidationConstants.WORK_ELEMENT_FIELD, credit.getID())
        };
    
        verifySaveMustFail(credit, creditScope, null, expectedWarnings);
    
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        initSummaryTimesheetAndTaskAssignment(true);
        approvedSummaryTimesheet();

        // Set the mandatory fields
        credit.setTaskAssignment(taskAssignment);
        credit.setCreditTimesheet(createTimesheet());
        
        initStep();

        credit
            .setName(getTestString(ValidationConstants.STEP_NAME_CHARACTER_LIMIT) + 1);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, credit.getClass()
                .getName(), ValidationConstants.NAME_FIELD, credit.getID()),
        };

        verifySaveMustFail(credit, creditScope, expectedExceptions);
    }

    public void testFailureTimeCodesNotAssigned() throws AssertionError,
            Exception
    {

        // TODO enable the test after implementing timeCode for credits
        fail("TimeCode is not implemented for Credit");
        
//        initSummaryTimesheetAndTaskAssignment(true);
//        approvedSummaryTimesheet();
//
//        // Set the mandatory fields
//        credit.setTaskAssignment(taskAssignment);
//        credit.setCreditTimesheet(createTimesheet());
//        
//        initStep();
//
//        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
//        TimeCode timecode1 = TimecodeFixture.createSaveTimeCodeFixture(
//            timeCodeScope, true, this);
//        TimeCode timecode2 = TimecodeFixture.createSaveTimeCodeFixture(
//            timeCodeScope, false, this);
//
//        credit.setTimeCode1(timecode1);
//        credit.setTimeCode2(timecode2);
//
//        RPMExceptionMock[] expectedExceptions =
//        {
//                createRPMException(
//                    com.ibm.rpm.timesheet.constants.ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
//                    credit.getClass().getName(),
//                    ValidationConstants.TIMECODE1_FIELD, credit.getID()),
//                createRPMException(
//                    com.ibm.rpm.timesheet.constants.ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
//                    credit.getClass().getName(),
//                    ValidationConstants.TIMECODE2_FIELD, credit.getID()),
//        };
//
//        verifySaveMustFail(credit, creditScope, expectedExceptions);
    }

    public void testFailureInvalidTimeCodes() throws AssertionError, Exception
    {
        // TODO enable the test after implementing timeCode for credits
        fail("TimeCode is not implemented for Credit");
        
//        initSummaryTimesheetAndTaskAssignment(true);
//        approvedSummaryTimesheet();
//        
//        //reload TaskAssignment with containing project in scope
//        
//        TaskAssignmentScope taScope = TaskAssignmentFixture.createScope(true, false, false, false);
//        taskAssignment = (TaskAssignment)loadFromID(taskAssignment, taScope);
//        
//        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
//            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
//        
//        // Set the mandatory fields
//        credit.setTaskAssignment(taskAssignment);
//        credit.setCreditTimesheet(createTimesheet());
//        
//        initStep();
//
//        ResourceTaskAssignment resourceTaskAssignment = (ResourceTaskAssignment) taskAssignment
//            .getParent();
//        WorkElement workElement = (WorkElement) resourceTaskAssignment
//            .getParent();
//        GenericProject project = (GenericProject) workElement
//            .getContainingProject();
//
//        // assign the timecodes to the project
//        TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
//            .createSaveFixture(project, true, this);
//        TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
//            .createSaveFixture(project, false, this);
//        project.setTimeCodeAssignments(new TimeCodeAssignment[]
//        {
//                timecode1, timecode2
//        });
//
//        credit.setTimeCode1(timecode2.getTimeCode());
//        credit.setTimeCode2(timecode1.getTimeCode());
//
//        RPMExceptionMock[] expectedExceptions =
//        {
//                createRPMException(
//                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, credit
//                        .getClass().getName(),
//                    ValidationConstants.TIMECODE1_FIELD, credit.getID()),
//                createRPMException(
//                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, credit
//                        .getClass().getName(),
//                    ValidationConstants.TIMECODE2_FIELD, credit.getID()),
//        };
//
//        verifySaveMustFail(credit, creditScope, expectedExceptions);
    }

    
    public void testUpdateWithNewCreditTimesheet() throws AssertionError,
            Exception
    {

        initSummaryTimesheetAndTaskAssignment(true);
        approvedSummaryTimesheet();

        // Set the mandatory fields
        credit.setTaskAssignment(taskAssignment);
        credit.setCreditTimesheet(createTimesheet());
        
        initStep();

        credit = (Credit) saveMustSucceed(credit, Credit.class, creditScope);

        CreditTimesheet timesheet = credit.getCreditTimesheet();

        CreditTimesheet newTimesheet = new CreditTimesheet();
        newTimesheet.setWeekOf(timesheet.getWeekOf());
        newTimesheet.setCreditWeekOf(timesheet.getCreditWeekOf());

        credit.setCreditTimesheet(newTimesheet);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_TIMESHEET_ALREADY_EXIST,
                credit.getClass().getName(),
                ValidationConstants.CREDIT_TIMESHEET_FIELD, credit.getID())
        };

        verifySaveMustFail(credit, creditScope, expectedExceptions);
    }
    
    public void testFailureCreditExceedsSummaryTimesheet() throws AssertionError, Exception
    {
        initSummaryTimesheetAndTaskAssignment(true);
        approvedSummaryTimesheet();

        // Set the mandatory fields
        credit.setTaskAssignment(taskAssignment);
        credit.setCreditTimesheet(createTimesheet());
        
        initStepWithoutHours();
        
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.REGULAR_MONDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.SPECIAL_MONDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.REGULAR_TUESDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.REGULAR_THURSDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.SPECIAL_THURSDAY_FIELD, null),
                    createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.REGULAR_FRIDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.SPECIAL_FRIDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.REGULAR_SATURDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.SPECIAL_SATURDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.REGULAR_SUNDAY_FIELD, null),
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_HOURS_CANNOT_EXCEED_SUMMARY_TIMESHEET_HOURS,
                    CreditTimesheet.class.getName(),
                    ValidationConstants.SPECIAL_SUNDAY_FIELD, null),
        };

        // Attempt to save the object
        verifySaveMustFail(credit, creditScope, expectedExceptions);
    }

    public void testInsertSuccessfully() throws AssertionError, Exception
    {
        initSummaryTimesheetAndTaskAssignment(true);
        approvedSummaryTimesheet();
    
        // Set the mandatory fields
        credit.setTaskAssignment(taskAssignment);
        credit.setCreditTimesheet(createTimesheet());
        
        initStep();
    
        // Attempt to save the object
        credit = (Credit) saveMustSucceed(credit, Credit.class, creditScope);
    
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Credit[id='" + credit.getID()
                + "']", creditScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Credit);
    
        // Check that the loaded object is identical to the one we
        // requested to store
        CreditFixture.compare(credit, (Credit) rpmObjects[0], creditScope);
    }
    
    public void testUpdateSuccessfullyWithoutTaskAssignment() throws AssertionError, Exception
    {
        initSummaryTimesheetAndTaskAssignment(true);
        approvedSummaryTimesheet();
    
        // Set the mandatory fields
        credit.setTaskAssignment(taskAssignment);
        credit.setCreditTimesheet(createTimesheet());
        
        initStep();
    
        // Attempt to save the object
        credit = (Credit) saveMustSucceed(credit, Credit.class, creditScope);
    
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Credit[id='" + credit.getID()
                + "']", creditScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Credit);
    
        Credit savedCredit = (Credit) rpmObjects[0];
        //remove the TaskAssignment
        savedCredit.setTaskAssignment(null);
        credit = (Credit) saveMustSucceed(savedCredit, Credit.class, creditScope);
    }

    private CreditTimesheet createTimesheet() throws Exception
    {
        RPMObjectScope stepScope = StepTimesheetFixture.createScope(true);
        CreditTimesheet creditTimesheet = new CreditTimesheet();
        creditTimesheet = CreditTimesheetFixture.createFixture(credit,
            weekOf, creditWeekOf,
            stepScope, this);
        return creditTimesheet;
    }

    private void initSummaryTimesheetAndTaskAssignment(boolean setHours) throws Exception
    {

        taskAssignment = TaskAssignmentFixture.createSaveFixture(null, this);
        
        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        creditWeekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CREDIT_WEEK_OF);
        
        summaryTimesheetScope = SummaryTimesheetFixture.createScope(false,
            true, false, false, false);
        summaryTimesheet = SummaryTimesheetFixture.createSaveFixture(taskAssignment,
            creditWeekOf, summaryTimesheetScope, setHours, this);

        

    }

    private void initStep() throws Exception
    {

        //Create a step with a StepTimesheet for the current week

        StepScope stepScope = new StepScope();
        stepScope.setTimesheets(true);

        StepTimesheet stepTimesheet = new StepTimesheet();
        stepTimesheet.setWeekOf(weekOf);
        
        StepTimesheetFixture.updateFixture(stepTimesheet);

        StepFixture.createSaveFixture((TaskAssignment) summaryTimesheet
            .getTaskAssignment(), stepScope, new StepTimesheet[]
        {
            stepTimesheet
        }, this);

    }

    private void initStepWithoutHours() throws Exception
    {
    
        //Create a step with a StepTimesheet for the current week
    
        StepScope stepScope = new StepScope();
        stepScope.setTimesheets(true);
    
        StepTimesheet stepTimesheet = new StepTimesheet();
        stepTimesheet.setWeekOf(weekOf);
    
        StepFixture.createSaveFixture((TaskAssignment) summaryTimesheet
            .getTaskAssignment(), stepScope, new StepTimesheet[]
        {
            stepTimesheet
        }, this);
    
    }

    private void approvedSummaryTimesheet() throws Exception
    {

        TimesheetApprovalStatus approvalStatus = summaryTimesheet
            .getApprovalStatus();
        approvalStatus.setApprovalStatus(TimesheetStatus.Approved);
       
        SingleResult result = save(summaryTimesheet, summaryTimesheetScope);
        summaryTimesheet = (SummaryTimesheet) result.getRpmObject();

    }



}
