/**
 * Created on Jan 4, 2006
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultStepFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.DefaultStep;
import com.ibm.rpm.timesheet.scope.DefaultStepScope;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;


public class TestDefaultStepValidation extends GenericValidation
{
    private DefaultStep defaultStep = null;
    private Project parentProject = null;
    private DefaultStepScope scope;

    public void setUp() throws Exception
    {
        super.setUp();
        
        scope = new DefaultStepScope();
        scope.setTimeCodes(true);

        defaultStep = new DefaultStep();
        parentProject = (Project) WorkElementFixture.retrieveStaticFixture(
            new Project(), this);
    }
    
    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);
        defaultStep.setParent(portfolio);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, 
                defaultStep.getClass().getName(), ValidationConstants.PARENT_FIELD, 
                defaultStep.getID())
        };

        verifySaveMustFail(defaultStep, null, expectedExceptions);
    }
    
    public void testInsertWithParentProposal() throws AssertionError, Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.retrieveStaticFixture(
            new Proposal(), this);

        // Set the mandatory fields
        defaultStep.setParent(proposal);

        // Attempt to save the object
        defaultStep = (DefaultStep) saveMustSucceed(defaultStep, 
            DefaultStep.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/DefaultStep[id='"
                + defaultStep.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof DefaultStep);

        // Check that the loaded object is identical to the one we
        // requested to store
        DefaultStepFixture.compare(defaultStep, 
            (DefaultStep) rpmObjects[0], null);
    }
    
    public void testInsertWithParentProject() throws AssertionError, Exception
    {
        Project project = (Project) WorkElementFixture.retrieveStaticFixture(
            new Project(), this);

        // Set the mandatory fields
        defaultStep.setParent(project);

        // Attempt to save the object
        defaultStep = (DefaultStep) saveMustSucceed(defaultStep, 
            DefaultStep.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/DefaultStep[id='"
                + defaultStep.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof DefaultStep);

        // Check that the loaded object is identical to the one we
        // requested to store
        DefaultStepFixture.compare(defaultStep, 
            (DefaultStep) rpmObjects[0], null);
    }
    
    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, 
                defaultStep.getClass().getName(), ValidationConstants.PARENT_FIELD, 
                defaultStep.getID())
        };

        verifySaveMustFail(defaultStep, null, expectedExceptions);
    }
    
    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set the mandatory fields
        defaultStep.setParent(parentProject);

        // Set string values with max length + 1 and verify that they are not
        // accepted
        defaultStep
            .setName(getTestString(ValidationConstants.STEP_NAME_CHARACTER_LIMIT + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, defaultStep
                .getClass().getName(), ValidationConstants.NAME_FIELD,
                defaultStep.getID()),
        };

        verifySaveMustFail(defaultStep, null, expectedExceptions);
    }
    
    public void testFailureTimeCodesNotAssigned() throws AssertionError, Exception
    {
        // TODO enable the test after implementing timeCode for DefaultStep
        fail("TimeCode is not implemented for DefaultStep");
        
//        // Set the mandatory fields
//        defaultStep.setParent(parentProject);
//        
//        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
//        TimeCode timecode1 = TimecodeFixture.createSaveTimeCodeFixture(
//            timeCodeScope, true, this);
//        TimeCode timecode2 = TimecodeFixture.createSaveTimeCodeFixture(
//            timeCodeScope, false, this); 
//        
//        defaultStep.setTimeCode1(timecode1);
//        defaultStep.setTimeCode2(timecode2);
//        
//        RPMExceptionMock[] expectedExceptions =
//        {
//                createRPMException(com.ibm.rpm.timesheet.constants.
//                    ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
//                    defaultStep.getClass().getName(),
//                    ValidationConstants.TIMECODE1_FIELD, defaultStep
//                        .getID()),
//                createRPMException(com.ibm.rpm.timesheet.constants.
//                    ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
//                    defaultStep.getClass().getName(),
//                    ValidationConstants.TIMECODE2_FIELD, defaultStep
//                        .getID()),
//        };
//
//        verifySaveMustFail(defaultStep, scope, expectedExceptions);
    }
    
    public void testFailureInvalidTimeCodes() throws AssertionError, Exception
    {
        // TODO enable the test after implementing timeCode for DefaultStep
        fail("TimeCode is not implemented for DefaultStep");
        
//        // Set the mandatory fields
//        defaultStep.setParent(parentProject);
//
//        // assign the timecodes to the project
//        TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
//            .createSaveFixture(parentProject, true, this);
//        TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
//            .createSaveFixture(parentProject, false, this);
//        parentProject.setTimeCodeAssignments(new TimeCodeAssignment[]
//        {
//                timecode1, timecode2
//        });
//        
//        defaultStep.setTimeCode1(timecode2.getTimeCode());
//        defaultStep.setTimeCode2(timecode1.getTimeCode());
//        
//        RPMExceptionMock[] expectedExceptions =
//        {
//                createRPMException(
//                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY,
//                    defaultStep.getClass().getName(),
//                    ValidationConstants.TIMECODE1_FIELD, defaultStep
//                        .getID()),
//                createRPMException(
//                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY,
//                    defaultStep.getClass().getName(),
//                    ValidationConstants.TIMECODE2_FIELD, defaultStep
//                        .getID()),
//        };
//
//        verifySaveMustFail(defaultStep, scope, expectedExceptions);
    }
    
    public void testInsertSuccess() throws AssertionError, Exception
    {
        defaultStep.setParent(parentProject);
        defaultStep
            .setName(getTestString(ValidationConstants.STEP_NAME_CHARACTER_LIMIT));

        // assign the timecodes to the project
        TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
            .createSaveFixture(parentProject, true, this);
        TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
            .createSaveFixture(parentProject, false, this);
        parentProject.setTimeCodeAssignments(new TimeCodeAssignment[]
        {
                timecode1, timecode2
        });

        defaultStep.setTimeCode1(timecode1.getTimeCode());
        defaultStep.setTimeCode2(timecode2.getTimeCode());

        // Attempt to save the object
        defaultStep = (DefaultStep) saveMustSucceed(defaultStep,
            DefaultStep.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/DefaultStep[id='"
                + defaultStep.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof DefaultStep);

        // Check that the loaded object is identical to the one we
        // requested to store
        DefaultStepFixture.compare(defaultStep,
            (DefaultStep) rpmObjects[0], scope);
    }
}
