/*
 * Created on Nov 30, 2005 
 * @author mmatwiy
 * Validation tests for container PersonalTask.
 */
package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.PersonalTaskFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.PersonalProject;
import com.ibm.rpm.timesheet.containers.PersonalTask;
import com.ibm.rpm.timesheet.scope.PersonalProjectScope;
import com.ibm.rpm.wbs.containers.Task;


public class TestPersonalTaskValidation extends GenericValidation
{
    private PersonalTask personalTask;
    
    private PersonalProject personalProject;

    public void setUp() throws Exception
    {
        super.setUp();

        personalTask = new PersonalTask();
        
        ResourceScope scope = new ResourceScope();
        scope.setCalendar(new CalendarScope());
        scope.setPersonalProject(new PersonalProjectScope());
        
        personalProject = ResourceFixture.createSaveFixture(scope, this).getPersonalProject();
     }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Task task = (Task) WorkElementFixture.retrieveStaticFixture(new Task(),
            this);

        personalTask
            .setName(getTestString(ValidationConstants.PERSONAL_TASK_NAME_CHARACTER_LIMIT));

        personalTask.setParent(task);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, personalTask
                .getClass().getName(), ValidationConstants.PARENT_FIELD, personalTask
                .getID())
        };

        verifySaveMustFail(personalTask, null, expectedExceptions);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        // Set the mandatory fields
        personalTask.setParent(personalProject);
        personalTask
            .setName(getTestString(ValidationConstants.PERSONAL_TASK_NAME_CHARACTER_LIMIT));
        //personalTask.setEstimatedStartDate(new GregorianCalendar(2005, 12, 11));
        //personalTask.setEstimatedFinishDate(new GregorianCalendar(2006, 12, 12));

        // Attempt to save the object
        personalTask = (PersonalTask) saveMustSucceed(personalTask,
            PersonalTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/PersonalTask[id='"
                + personalTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof PersonalTask);

        // Check that the loaded object is identical to the one we
        // requested to store
        PersonalTaskFixture.compare(personalTask, (PersonalTask) rpmObjects[0]);
    }

    public void testFailureReadOnlyFields() throws Exception
    {

        personalTask.setParent(personalProject);
        personalTask.setName(HelperTestUtilities.createName());

        personalTask.setActual(new Double(5));
        personalTask.setEETC(new Double(3));
        personalTask.setFinishDate(new GregorianCalendar());
        personalTask.setPercentageCompleted(new Double(30));
        personalTask.setPercentageWork(new Double(25));
        personalTask.setStartDate(new GregorianCalendar());

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, personalTask
                    .getClass().getName(), ValidationConstants.ACTUAL_FIELD,
                    personalTask.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, personalTask
                    .getClass().getName(), ValidationConstants.EETC_FIELD,
                    personalTask.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, personalTask
                    .getClass().getName(),
                    ValidationConstants.FINISH_DATE_FIELD, personalTask.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, personalTask
                    .getClass().getName(),
                    ValidationConstants.PERCENTAGE_COMPLETED_FIELD,
                    personalTask.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, personalTask
                    .getClass().getName(),
                    ValidationConstants.PERCENTAGE_WORK_FIELD, personalTask
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, personalTask
                    .getClass().getName(),
                    ValidationConstants.START_DATE_FIELD, personalTask.getID()),
        };

        verifySaveMustFail(personalTask, null, null, expectedWarnings);

    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set the mandatory fields
        personalTask.setParent(personalProject);

        // Set string values with max length + 1 and verify that they are not
        // accepted
        personalTask
            .setName(getTestString(ValidationConstants.PERSONAL_TASK_NAME_CHARACTER_LIMIT + 1));
 
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, personalTask.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, personalTask
                    .getID()),
        };

        verifySaveMustFail(personalTask, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set the mandatory fields
        personalTask.setParent(personalProject);
        personalTask
            .setName(getTestString(ValidationConstants.NAME_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MIN_LENGTH, personalTask.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, personalTask.getID())//,
        };

        verifySaveMustFail(personalTask, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
       
        personalTask.setParent(personalProject);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, personalTask.getClass()
                .getName(), ValidationConstants.NAME_FIELD, personalTask.getID())
        };

        verifySaveMustFail(personalTask, null, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        // Set the mandatory fields
        personalTask
            .setName(getTestString(ValidationConstants.PERSONAL_TASK_NAME_CHARACTER_LIMIT));

        personalTask.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, personalTask.getClass()
                .getName(), ValidationConstants.PARENT_FIELD, personalTask.getID())
        };

        verifySaveMustFail(personalTask, null, expectedExceptions);

    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set the mandatory fields
        personalTask.setParent(personalProject);

        // Set field values with maxLength and verify that they are accepted
        personalTask
            .setName(getTestString(ValidationConstants.PERSONAL_TASK_NAME_CHARACTER_LIMIT));

        // Attempt to save the PersonalTask object
        personalTask = (PersonalTask) saveMustSucceed(personalTask,
            PersonalTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/PersonalTask[id='"
                + personalTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof PersonalTask);

        // Check that the loaded PersonalTask is identical to the one we
        // requested to store
        PersonalTaskFixture.compare(personalTask, (PersonalTask) rpmObjects[0]);    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set the mandatory fields
        personalTask.setParent(personalProject);
        personalTask
            .setName(getTestString(ValidationConstants.NAME_MIN));

         // Attempt to save the PersonalTask object
        personalTask = (PersonalTask) saveMustSucceed(personalTask,
            PersonalTask.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/PersonalTask[id='"
                + personalTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof PersonalTask);

        // Check that the loaded PersonalTask is identical to the one we
        // requested to store
        PersonalTaskFixture.compare(personalTask, (PersonalTask) rpmObjects[0]);    }

    public void testInsertWithoutAnAssignedCalendar() throws AssertionError, Exception
    {
        // Set the mandatory fields
        personalTask
            .setName(getTestString(ValidationConstants.PERSONAL_TASK_NAME_CHARACTER_LIMIT));
    
        ResourceScope scope = new ResourceScope();
        scope.setPersonalProject(new PersonalProjectScope());
        
        Resource resource = ResourceFixture.createSaveFixture(scope, this);
        personalProject = resource.getPersonalProject();
        
        personalTask.setParent(personalProject);
    
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(com.ibm.rpm.timesheet.constants.ErrorCodes.MANDATORY_CALENDAR, personalTask.getClass()
                .getName(), ValidationConstants.PARENT_FIELD, null)
        };
    
        verifySaveMustFail(personalTask, null, expectedExceptions);
    
    }
}
