/**
 * Created on Jan 4, 2006
 * 
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class TestStepTimesheetValidation extends GenericValidation
{
    private StepTimesheet timesheet = null;

    private StepScope stepScope;
    
    private Calendar weekOf = null;
    
    private SummaryTimesheet summaryTimesheet = null;

    public void setUp() throws Exception
    {
        super.setUp();

        timesheet = new StepTimesheet();

        stepScope = new StepScope();
        stepScope.setTimesheets(true);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);
        timesheet.setParent(portfolio);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, timesheet
                .getClass().getName(), ValidationConstants.PARENT_FIELD,
                timesheet.getID())
        };

        verifySaveMustFail(timesheet, null, expectedExceptions);
    }

    public void testInsertSuccessfully() throws AssertionError, Exception
    {
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        
        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        createSummaryTimeSheet(taskAssignment, null);
        timesheet.setWeekOf(weekOf);
        Double hours = new Double(1);
        timesheet.setRegularMonday(hours);
        timesheet.setRegularTuesday(hours);

        Step step = StepFixture.createFixture(taskAssignment, null,
            new StepTimesheet[]
            {
                timesheet
            }, this);

        // Attempt to save the object
        step = (Step) saveMustSucceed(step, Step.class, stepScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("Step[id='"
                + timesheet.getParent().getID()
                + "']/StepTimesheet[weekof='"
                + DateUtil.convertToString(timesheet.getWeekOf()) + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof StepTimesheet);

        // Check that the loaded object is identical to the one we
        // requested to store
        StepTimesheetFixture.compare(timesheet, (StepTimesheet) rpmObjects[0],
            null);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, timesheet.getClass()
                .getName(), ValidationConstants.PARENT_FIELD, timesheet.getID())
        };

        verifySaveMustFail(timesheet, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        createSummaryTimeSheet(taskAssignment, null);
        Step step = StepFixture.createFixture(taskAssignment, null,
            new StepTimesheet[]
            {
                timesheet
            }, this);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, timesheet.getClass()
                .getName(), ValidationConstants.WEEK_OF_FIELD, timesheet
                .getID())
        };

        verifySaveMustFail(step, stepScope, expectedExceptions);
    }

    public void testFailureReadOnlyFields() throws AssertionError, Exception
    {
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        
        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        createSummaryTimeSheet(taskAssignment, null);

        // Set the mandatory fields
        timesheet.setWeekOf(weekOf);

        // set Read-only fields
        timesheet.setBillable(new Boolean(true));

        Step step = StepFixture.createFixture(taskAssignment, null,
            new StepTimesheet[]
            {
                timesheet
            }, this);

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, timesheet.getClass()
                .getName(), ValidationConstants.BILLABLE_FIELD, timesheet
                .getID())
        };

        verifySaveMustFail(step, stepScope, null, expectedWarnings);

        // Load the object from xpath
        RPMObject[] rpmObjects = loadFromXpath("Step[id='"
                + timesheet.getParent().getID()
                + "']/StepTimesheet[weekof='"
                + DateUtil.convertToString(timesheet.getWeekOf()) + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof StepTimesheet);

        StepTimesheet loadedTimesheet = (StepTimesheet) rpmObjects[0];

        // assert that the values we set did not get saved in the db
        assertNotSame(timesheet.getBillable(), loadedTimesheet.getBillable());
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        createSummaryTimeSheet(taskAssignment, null);
        
        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        // set the mandatory field
        timesheet.setWeekOf(weekOf);

        Double max_hours = new Double(
            ValidationConstants.MAX_WORKING_HOURS_PER_DAY + 1);
        timesheet
            .setSummaryNotes(getTestString(ValidationConstants.SUMMARY_NOTES_MAX) + 1);
        timesheet.setRegularMonday(max_hours);
        timesheet.setRegularTuesday(max_hours);
        timesheet.setRegularWednesday(max_hours);
        timesheet.setRegularThursday(max_hours);
        timesheet.setRegularFriday(max_hours);
        timesheet.setRegularSaturday(max_hours);
        timesheet.setRegularSunday(max_hours);
        timesheet.setSpecialMonday(max_hours);
        timesheet.setSpecialTuesday(max_hours);
        timesheet.setSpecialWednesday(max_hours);
        timesheet.setSpecialThursday(max_hours);
        timesheet.setSpecialFriday(max_hours);
        timesheet.setSpecialSaturday(max_hours);
        timesheet.setSpecialSunday(max_hours);

        Step step = StepFixture.createFixture(taskAssignment, null,
            new StepTimesheet[]
            {
                timesheet
            }, this);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, timesheet.getClass()
                    .getName(), ValidationConstants.SUMMARY_NOTES_FIELD,
                    timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_TUESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_THURSDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SATURDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SUNDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_THURSDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SATURDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SUNDAY_FIELD, timesheet.getID())
        };

        verifySaveMustFail(step, stepScope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        createSummaryTimeSheet(taskAssignment, null);

        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        // set the mandatory field
        timesheet.setWeekOf(weekOf);

        Double min_hours = new Double(
            ValidationConstants.MIN_WORKING_HOURS_PER_DAY - 1);
        timesheet.setRegularMonday(min_hours);
        timesheet.setRegularTuesday(min_hours);
        timesheet.setRegularWednesday(min_hours);
        timesheet.setRegularThursday(min_hours);
        timesheet.setRegularFriday(min_hours);
        timesheet.setRegularSaturday(min_hours);
        timesheet.setRegularSunday(min_hours);
        timesheet.setSpecialMonday(min_hours);
        timesheet.setSpecialTuesday(min_hours);
        timesheet.setSpecialWednesday(min_hours);
        timesheet.setSpecialThursday(min_hours);
        timesheet.setSpecialFriday(min_hours);
        timesheet.setSpecialSaturday(min_hours);
        timesheet.setSpecialSunday(min_hours);

        Step step = StepFixture.createFixture(taskAssignment, null,
            new StepTimesheet[]
            {
                timesheet
            }, this);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_TUESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_THURSDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SATURDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SUNDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_THURSDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SATURDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SUNDAY_FIELD, timesheet.getID())
        };

        verifySaveMustFail(step, stepScope, expectedExceptions);
    }

    public void testFailureInvalidValues() throws AssertionError, Exception
    {
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        createSummaryTimeSheet(taskAssignment, null);

        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        // Set the mandatory fields
        timesheet.setWeekOf(weekOf);

        Double hours = new Double(1.8);
        timesheet.setRegularMonday(hours);
        timesheet.setRegularTuesday(hours);
        timesheet.setRegularWednesday(hours);
        timesheet.setRegularThursday(hours);
        timesheet.setRegularFriday(hours);
        timesheet.setRegularSaturday(hours);
        timesheet.setRegularSunday(hours);
        timesheet.setSpecialMonday(hours);
        timesheet.setSpecialTuesday(hours);
        timesheet.setSpecialWednesday(hours);
        timesheet.setSpecialThursday(hours);
        timesheet.setSpecialFriday(hours);
        timesheet.setSpecialSaturday(hours);
        timesheet.setSpecialSunday(hours);

        Step step = StepFixture.createFixture(taskAssignment, null,
            new StepTimesheet[]
            {
                timesheet
            }, this);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_TUESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_THURSDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_WEDNESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SATURDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.REGULAR_SUNDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_MONDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_TUESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_THURSDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_WEDNESDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_FRIDAY_FIELD, timesheet.getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SATURDAY_FIELD, timesheet
                        .getID()),
                createRPMException(ErrorCodes.INVALID_HOUR, timesheet
                    .getClass().getName(),
                    ValidationConstants.SPECIAL_SUNDAY_FIELD, timesheet.getID())
        };

        verifySaveMustFail(step, stepScope, expectedExceptions);
    }

    private SummaryTimesheet createSummaryTimeSheet(TaskAssignment assignment,
            SummaryTimesheetScope scope) throws Exception
    {
        
        weekOf = TimesheetTestUtil.getValidWeekOf(assignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        summaryTimesheet = new SummaryTimesheet();
        summaryTimesheet.setTaskAssignment(assignment);
        summaryTimesheet.setWeekOf(weekOf);

        GenericProject project = SummaryTimesheetFixture.loadProject(
            summaryTimesheet, this);
        // do copy proposed to plan and publish to team
        TimesheetTestUtil.initProject(project, this);
        summaryTimesheet = (SummaryTimesheet) save(summaryTimesheet, scope)
            .getRpmObject();
        return summaryTimesheet;
    }

    public void testInsertWithNoSummaryTimesheet() throws AssertionError,
            Exception
    {
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment assignment = TaskAssignmentFixture.createSaveFixture(
            taskAssignmentScope, this);
        
        weekOf = TimesheetTestUtil.getValidWeekOf(assignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);

        Step step = StepFixture.createFixture(assignment, null, null, this);

        step = (Step) saveMustSucceed(step, Step.class, stepScope);

        // set the mandatory field
        timesheet = new StepTimesheet();
        timesheet.setWeekOf(weekOf);
        timesheet.setParent(step);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.NO_SUMMARY_TIMESHEET_FOR_WEEKOF,
                timesheet.getClass().getName())
        };

        verifySaveMustFail(timesheet, null, expectedExceptions);
    }

    public void testFailureModifyAfterSubmitForApproval()
            throws AssertionError, Exception
    {

        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        createSummaryTimeSheet(taskAssignment, null);
        
        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        
        // set the mandatory field
        timesheet.setWeekOf(weekOf);

        stepScope.setResource(true);
        stepScope.setProject(true);

        Step step = StepFixture.createSaveFixture(taskAssignment, stepScope,
            new StepTimesheet[]
            {
                timesheet
            }, this);

        // submit the timesheet for approval
        TimesheetApprovalStatus appStatus = new TimesheetApprovalStatus();
        appStatus.setApprovalStatus(TimesheetStatus.Submitted);
        appStatus.setParent(summaryTimesheet);
        appStatus.setResource(step.getResource());
        appStatus.setWeekOf(timesheet.getWeekOf());

        save(appStatus, null);

        // modify the timesheet and save
        timesheet.setRegularFriday(new Double(2));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.CANNOT_MODIFY_TIMESHEET,
                StringUtil.getShortClassName(appStatus.getClass()),
                ValidationConstants.APPROVAL_STATUS_FIELD, null)
        };

        verifySaveMustFail(timesheet, null, expectedExceptions);
    }
}
