/**
 * Created on Jan 4, 2006
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.StepTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class TestStepValidation extends GenericValidation
{
    private Step step = null;
    private TaskAssignment taskAssignment = null;
    private StepScope stepScope;

    public void setUp() throws Exception
    {
        super.setUp();

        TaskAssignmentScope scope = TaskAssignmentFixture.createScope(true,
            false, false, false);
        
        stepScope = new StepScope();
        stepScope.setTimeCodes(true);
        stepScope.setTimesheets(true);
        stepScope.setTaskAssignment(true);
        
        step = new Step();
        taskAssignment = TaskAssignmentFixture.createSaveFixture(scope,
            this);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);
        step.setParent(portfolio);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MUST_ALWAYS_BE_NULL, step
                .getClass().getName(), ValidationConstants.PARENT_FIELD, step
                .getID())
        };

        verifySaveMustFail(step, null, expectedExceptions);
    }

    public void testInsertWithoutTaskAssignment() throws AssertionError, Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, step
                .getClass().getName(),
                ValidationConstants.TASK_ASSIGNMENT_FIELD, step
                    .getID())
        };

        verifySaveMustFail(step, null, expectedExceptions);
    }
    
    public void testInsertWithoutTimesheets() throws AssertionError, Exception
    {
        step.setTaskAssignment(taskAssignment);
        
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, step.getClass()
                .getName(), ValidationConstants.STEP_TIMESHEETS_FIELD, step.getID())
        };

        verifySaveMustFail(step, stepScope, expectedExceptions);
    }

    public void testInsertWithNoSummaryTimesheet() throws AssertionError, Exception
    {
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment assignment = TaskAssignmentFixture.createSaveFixture(
            taskAssignmentScope, this);
        
        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.OTHER_WEEK);
        
        // set the mandatory field
        StepTimesheet timesheet = new StepTimesheet();
        timesheet.setWeekOf(weekOf);
        
        Step step = StepFixture.createFixture(assignment,
            null, new StepTimesheet[]
            {
                timesheet
            }, this);
        
       
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.NO_SUMMARY_TIMESHEET_FOR_WEEKOF,
                timesheet.getClass().getName())
        };
    
        verifySaveMustFail(step, stepScope, expectedExceptions);
    }

    public void testFailureReadOnlyFields() throws AssertionError, Exception
    {
        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(createTimesheets());

        step.setBillable(new Boolean(true));
        step.setETCChanged(new Boolean(true));
        
        step.setResource(new Resource());
        step.setProject(new Project());
        step.setWorkElement(new Task());

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, step
                    .getClass().getName(), ValidationConstants.BILLABLE_FIELD,
                    step.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, step
                    .getClass().getName(),
                    ValidationConstants.ETC_CHANGED_FIELD, step.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, step
                    .getClass().getName(), ValidationConstants.RESOURCE_FIELD,
                    step.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, step
                    .getClass().getName(), ValidationConstants.PROJECT_FIELD,
                    step.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, step
                    .getClass().getName(),
                    ValidationConstants.WORK_ELEMENT_FIELD, step.getID())
        };

        verifySaveMustFail(step, stepScope, null, expectedWarnings);
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Step[id='"
                + step.getID() + "']", stepScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Step);
        
        Step loadedStep = (Step)rpmObjects[0];
        
        // assert that the values we set did not get saved in the db
        assertNotSame(step.getBillable(), loadedStep.getBillable());
        assertNotSame(step.getETCChanged(), loadedStep.getETCChanged());
        
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(createTimesheets());
        
        step
            .setName(getTestString(ValidationConstants.STEP_NAME_CHARACTER_LIMIT) + 1);
        step.setWeight(new Double(ValidationConstants.PERCENTAGE_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, step.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, step
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, step.getClass()
                    .getName(), ValidationConstants.WEIGHT_FIELD, step
                    .getID())

        };

        verifySaveMustFail(step, stepScope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(createTimesheets());
        
        step.setEETC(new Double(ValidationConstants.EETC_MIN - 1));
        step.setWeight(new Double(ValidationConstants.PERCENTAGE_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.GREATER_THAN_OR_EQUAL_VALUE,
                    step.getClass().getName(),
                    ValidationConstants.EETC_FIELD, step.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, step.getClass()
                    .getName(), ValidationConstants.WEIGHT_FIELD, step
                    .getID())

        };

        verifySaveMustFail(step, stepScope, expectedExceptions);
    }
    
    public void testFailureWithInvalidValues() throws AssertionError, Exception
    {
        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(createTimesheets());
        
        step.setEETC(new Double(1.1));
        step.setWeight(new Double(90.2323232));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.INVALID_HOUR, step.getClass()
                    .getName(), ValidationConstants.EETC_FIELD, step
                    .getID()),
                createRPMException(ErrorCodes.MORE_THAN_TWO_DECIMAL_DIGITS,
                    step.getClass().getName(),
                    ValidationConstants.WEIGHT_FIELD, step.getID())

        };

        verifySaveMustFail(step, stepScope, expectedExceptions);
    }

    public void testFailureTimeCodesNotAssigned() throws AssertionError,
            Exception
    {
        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(createTimesheets());
        
        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
        TimeCode timecode1 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, true, this);
        TimeCode timecode2 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, false, this);

        step.setTimeCode1(timecode1);
        step.setTimeCode2(timecode2);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(com.ibm.rpm.timesheet.constants.
                    ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
                    step.getClass().getName(),
                    ValidationConstants.TIMECODE1_FIELD, step.getID()),
                createRPMException(com.ibm.rpm.timesheet.constants.
                    ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
                    step.getClass().getName(),
                    ValidationConstants.TIMECODE2_FIELD, step.getID()),
        };

        verifySaveMustFail(step, stepScope, expectedExceptions);
    }

    public void testFailureInvalidTimeCodes() throws AssertionError, Exception
    {
        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(createTimesheets());

        ResourceTaskAssignment resourceTaskAssignment = (ResourceTaskAssignment) taskAssignment
            .getParent();
        WorkElement workElement = (WorkElement) resourceTaskAssignment
            .getParent();
        GenericProject project = (GenericProject)workElement.getContainingProject();

        // assign the timecodes to the project
        TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
            .createSaveFixture(project, true, this);
        TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
            .createSaveFixture(project, false, this);
        project.setTimeCodeAssignments(new TimeCodeAssignment[]
        {
                timecode1, timecode2
        });

        step.setTimeCode1(timecode2.getTimeCode());
        step.setTimeCode2(timecode1.getTimeCode());

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(com.ibm.rpm.timesheet.constants.
                    ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
                    step.getClass().getName(),
                    ValidationConstants.TIMECODE1_FIELD, step.getID()),
                createRPMException(com.ibm.rpm.timesheet.constants.
                    ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT,
                    step.getClass().getName(),
                    ValidationConstants.TIMECODE2_FIELD, step.getID()),
        };

        verifySaveMustFail(step, stepScope, expectedExceptions);
    }
    
    public void testInsertSuccessfully() throws AssertionError, Exception
    {
        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(createTimesheets());
        
        // Attempt to save the object
        step = (Step) saveMustSucceed(step,
            Step.class, stepScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Step[id='"
                + step.getID() + "']", stepScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Step);

        // Check that the loaded object is identical to the one we
        // requested to store
        StepFixture.compare(step, (Step) rpmObjects[0], stepScope);
    }

    private StepTimesheet[] createTimesheets() throws Exception
    {
        return createTimesheets(null);
    }
    
    private StepTimesheet[] createTimesheets(SummaryTimesheetScope summaryTimesheetScope) throws Exception
    {
        RPMObjectScope timesheetScope = StepTimesheetFixture.createScope(true);
        StepTimesheet[] timesheets = new StepTimesheet[2];
        timesheets[0] = StepTimesheetFixture.createFixture(step,
            new GregorianCalendar(), timesheetScope, summaryTimesheetScope, this);
        timesheets[1] = StepTimesheetFixture.createFixture(step,
            null, timesheetScope, summaryTimesheetScope, this);
        return timesheets;
    }
    
    public void testFailureInvalidWeight() throws AssertionError, Exception
    {
        // Set the mandatory fields and set weight to 100%
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(createTimesheets());
        step.setWeight(new Double(100));

        step = (Step) saveMustSucceed(step,
            Step.class, stepScope);

        // create another step with a weight of 1%
        Step step2 = new Step();
        step2.setTaskAssignment(taskAssignment);
        step2.setStepTimesheets(createTimesheets());
        step2.setWeight(new Double(1));
        
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    com.ibm.rpm.timesheet.constants.ErrorCodes.
                    WEIGHT_GREATER_THAN_100, step
                        .getClass().getName(),
                    ValidationConstants.STEPS_FIELD, step2.getID())
        };

        verifySaveMustFail(step2, stepScope, expectedExceptions);
    }

    public void testFailureClearAlwaysAvailable() throws AssertionError, Exception
    {
        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(createTimesheets());
        
        // Attempt to save the object
        step = (Step) saveMustSucceed(step,
            Step.class, stepScope);
    
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Step[id='"
                + step.getID() + "']", stepScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Step);
    
        Step reloadedStep = (Step) rpmObjects[0];
        
        reloadedStep.setAlwaysAvailable(null);
        
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    ErrorCodes.CANNOT_CLEAR_FIELD, step
                        .getClass().getName(),
                    ValidationConstants.STEP_ALWAYS_AVAILABLE_FIELD, step.getID()),
        };

        verifySaveMustFail(reloadedStep, stepScope, expectedExceptions);
        
        
    }
    
    public void testFailureInsertWithTimecodeAssignedToSummaryTimesheet()
            throws Exception
    {
        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        SummaryTimesheetScope summaryTimesheetScope = SummaryTimesheetFixture
            .createScope(true, false, false, false, false);
        step.setStepTimesheets(createTimesheets(summaryTimesheetScope));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.TIMECODE_ASSIGNED_TO_SUMMARY_TIMESHEET,
                StepTimesheet.class.getName())
        };
        verifySaveMustFail(step, stepScope, expectedExceptions);
    }
    
    public void testSuccessValidTimeCodes() throws AssertionError, Exception
    {
        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(createTimesheets());

        ResourceTaskAssignment resourceTaskAssignment = (ResourceTaskAssignment) taskAssignment
            .getParent();
        WorkElement workElement = (WorkElement) resourceTaskAssignment
            .getParent();
        GenericProject project = (GenericProject)workElement.getContainingProject();

        // assign the timecodes to the project
        TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
            .createSaveFixture(project, true, this);
        TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
            .createSaveFixture(project, false, this);
        project.setTimeCodeAssignments(new TimeCodeAssignment[]
        {
                timecode1, timecode2
        });

        step.setTimeCode1(timecode1.getTimeCode());
        step.setTimeCode2(timecode2.getTimeCode());

        saveMustSucceed(step, Step.class, stepScope);
    }
    
    public void testSuccessValidTimeCodesAssignToTask() throws Exception
    {
        assertNotNull(taskAssignment.getParent());
        assertNotNull(taskAssignment.getParent().getParent());
        assertTrue(taskAssignment.getParent().getParent() instanceof WorkElement);
        WorkElement workElement = (WorkElement)taskAssignment.getParent().getParent();
        
        // Set the mandatory fields
        step.setTaskAssignment(taskAssignment);
        step.setStepTimesheets(createTimesheets());

        // assign the timecodes to the workElement not project
        TimeCodeAssignment timecode1 = TimeCodeAssignmentFixture
            .createSaveFixture(workElement, true, this);
        TimeCodeAssignment timecode2 = TimeCodeAssignmentFixture
            .createSaveFixture(workElement, false, this);
        workElement.setTimeCodeAssignments(new TimeCodeAssignment[]
        {
                timecode1, timecode2
        });

        step.setTimeCode1(timecode1.getTimeCode());
        step.setTimeCode2(timecode2.getTimeCode());

        GenericProject project = (GenericProject)workElement.getContainingProject();
        assertNull(project.getTimeCodeAssignments());
        saveMustSucceed(step, Step.class, stepScope);
    }
}
