/**
 * Created on Jan 4, 2006
 * @author shahla
 */
package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetApprovalStatusFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.TimesheetApprovalStatusScope;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class TestTimesheetApprovalStatusValidation extends GenericValidation
{
    private TimesheetApprovalStatus timesheetApprovalStatus = null;

    private GenericProject project = null;

    private Resource resource = null;
    
    private Calendar weekOf = null;
    
    private Calendar invalidWeekOf = null;
    
    private TimesheetApprovalStatusScope scope;
    
    private SummaryTimesheet summaryTimesheet = null;

    public void setUp() throws Exception
    {
        super.setUp();
        timesheetApprovalStatus = new TimesheetApprovalStatus();
    }
    
    private void init() throws Exception{
        
        scope = new TimesheetApprovalStatusScope();
        scope.setLastApprovedRejectedBy(new ResourceScope());

        project = (Proposal) WorkElementFixture.retrieveStaticFixture(
            new Proposal(), this);

        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        
        weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);

        invalidWeekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.OTHER_WEEK);
        
        ResourceTaskAssignment resourceTaskAssignment = (ResourceTaskAssignment) taskAssignment
            .getParent();
        summaryTimesheet = SummaryTimesheetFixture.createSaveFixture(
            taskAssignment, weekOf, null, true, this);

        resource = resourceTaskAssignment.getResource();
        project = (GenericProject) ((WorkElement) resourceTaskAssignment
            .getParent()).getContainingProject();

        
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithoutParent() throws AssertionError,
            Exception
    {
        
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NO_PARENT,
                    timesheetApprovalStatus.getClass().getName(),
                    ValidationConstants.PARENT_FIELD,
                    timesheetApprovalStatus.getID())
        };

        verifySaveMustFail(timesheetApprovalStatus, null, expectedExceptions);
    }
    
    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        init();
        
        timesheetApprovalStatus.setParent(summaryTimesheet);
        
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MANDATORY,
                    timesheetApprovalStatus.getClass().getName(),
                    ValidationConstants.APPROVAL_STATUS_FIELD, timesheetApprovalStatus
                        .getID()),
                createRPMException(ErrorCodes.MANDATORY,
                    timesheetApprovalStatus.getClass().getName(),
                    ValidationConstants.RESOURCE_FIELD, timesheetApprovalStatus
                        .getID()),
                createRPMException(ErrorCodes.MANDATORY,
                    timesheetApprovalStatus.getClass().getName(),
                    ValidationConstants.WEEK_OF_FIELD, timesheetApprovalStatus
                        .getID())
        };

        verifySaveMustFail(timesheetApprovalStatus, null, expectedExceptions);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        timesheetApprovalStatus.setParent(new Project());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, timesheetApprovalStatus
                .getClass().getName(), ValidationConstants.PARENT_FIELD,
                timesheetApprovalStatus.getID())
        };

        verifySaveMustFail(timesheetApprovalStatus, null, expectedExceptions);
    }

    public void testInsertSuccessful() throws AssertionError, Exception
    {
        init();
        
        // Set the parent
        timesheetApprovalStatus.setParent(summaryTimesheet);
        
        // Set the mandatory fields
        timesheetApprovalStatus.setResource(resource);
        timesheetApprovalStatus.setWeekOf(weekOf);
        timesheetApprovalStatus.setApprovalStatus(TimesheetStatus.Submitted);

        // Attempt to save the object
        timesheetApprovalStatus = (TimesheetApprovalStatus) saveMustSucceed(
            timesheetApprovalStatus, TimesheetApprovalStatus.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath(
            "/TimesheetApprovalStatus[@resource[Id='"
                    + timesheetApprovalStatus.getResource().getID()
                    + "'] and project[Id='"
                    + timesheetApprovalStatus.getID()
                    + "'] and weekOf='"
                    + DateUtil.convertToString(timesheetApprovalStatus
                        .getWeekOf()) + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof TimesheetApprovalStatus);

        // Check that the loaded object is identical to the one we
        // requested to store
        TimesheetApprovalStatusFixture.compare(timesheetApprovalStatus,
            (TimesheetApprovalStatus) rpmObjects[0], scope);
    }

    public void testFailureReadOnlyFields() throws AssertionError, Exception
    {
        init();
        
        // Set the parent
        timesheetApprovalStatus.setParent(summaryTimesheet);
        
        // Set the mandatory fields
        timesheetApprovalStatus.setResource(resource);       
        timesheetApprovalStatus.setWeekOf(weekOf);
        timesheetApprovalStatus.setApprovalStatus(TimesheetStatus.Submitted);

        // set the read-only fields
        timesheetApprovalStatus.setProject(project);
        timesheetApprovalStatus.setApprovalDate(weekOf);
        timesheetApprovalStatus.setLastApprovedRejectedBy(resource);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD,
                    timesheetApprovalStatus.getClass().getName(),
                    ValidationConstants.PROJECT_FIELD,
                    timesheetApprovalStatus.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD,
                    timesheetApprovalStatus.getClass().getName(),
                    ValidationConstants.APPROVAL_DATE_FIELD,
                    timesheetApprovalStatus.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD,
                    timesheetApprovalStatus.getClass().getName(),
                    ValidationConstants.LAST_APPROVED_BY_FIELD,
                    timesheetApprovalStatus.getID())
        };

        verifySaveMustFail(timesheetApprovalStatus, scope, null,
            expectedWarnings);

        // Load the object from xpath
        RPMObject[] rpmObjects = loadFromXpath(
            "/TimesheetApprovalStatus[@resource[Id='"
                    + timesheetApprovalStatus.getResource().getID()
                    + "'] and GenericProject[Id='"
                    + project.getID()
                    + "'] and weekOf='"
                    + DateUtil.convertToString(timesheetApprovalStatus
                        .getWeekOf()) + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof TimesheetApprovalStatus);

        TimesheetApprovalStatus loadedApprovalStatus = (TimesheetApprovalStatus) rpmObjects[0];

        // assert that the values we set did not get saved in the db
        assertNotSame(timesheetApprovalStatus.getApprovalDate(),
            loadedApprovalStatus.getApprovalDate());
        assertNotSame(resource.getID(), loadedApprovalStatus.getLastApprovedRejectedBy().getID());
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        init();
        
        // Set the parent
        timesheetApprovalStatus.setParent(summaryTimesheet);

        // Set the mandatory fields
        timesheetApprovalStatus.setResource(resource);
        timesheetApprovalStatus.setWeekOf(weekOf);
        timesheetApprovalStatus.setApprovalStatus(TimesheetStatus.ApprovedByProjectManager);


        
        timesheetApprovalStatus
            .setApprovalNotes(getTestString(ValidationConstants.APPROVAL_NOTES_MAX) + 1);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, timesheetApprovalStatus
                .getClass().getName(),
                ValidationConstants.APPROVAL_NOTES_FIELD,
                timesheetApprovalStatus.getID())
        };

        verifySaveMustFail(timesheetApprovalStatus, null, expectedExceptions);
    }

    public void testFailureSummaryTimesheetDoesntExist() throws AssertionError,
            Exception
    {
        init();
        
        SummaryTimesheet st = new SummaryTimesheet();
        st.setID("abc");
        st.setWeekOf(summaryTimesheet.getWeekOf());
        
        // Set the parent
        timesheetApprovalStatus.setParent(st);

        // Set the mandatory fields
        timesheetApprovalStatus.setResource(resource);
        timesheetApprovalStatus.setWeekOf(invalidWeekOf);
        timesheetApprovalStatus.setApprovalStatus(TimesheetStatus.Submitted);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.NO_SUMMARY_TIMESHEET_FOR_ID_AND_WEEKOF,
                summaryTimesheet.getClass().getName(),
                ValidationConstants.PARENT_FIELD, null)
        };

        verifySaveMustFail(timesheetApprovalStatus, null, expectedExceptions);
    }
}