package com.ibm.rpm.client.junit.wbs.fixtures;

/**
 * @author vnarayan
 *
 */


import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.types.TimeUnitType;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ElementDependencyScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.DependencyType;
import com.ibm.rpm.wbs.types.LeadLagType;




public class DependencyFixture extends RPMObjectFixture

{

	protected static ArrayList localSkipGetters = new ArrayList();
	protected static ArrayList localSkipSetters = new ArrayList();
	static
    {
    	//set skipGetters
    	localSkipGetters.addAll(skipGetters);
        
        // read-only; generated by stored proc
    	localSkipGetters.add("getCreatedBy");
        
        // amount needs to be rounded off prior to comparing
        // TODO: this should be removed when action 58864 is addressed - Myriam
        localSkipGetters.add("getAmount");

    	//set skipSetters
    	localSkipSetters.addAll(skipSetters);
    	localSkipSetters.add("setDeleted");
        localSkipSetters.add("setAmount");
        
        // read-only; generated by stored proc
        localSkipSetters.add("setCreatedBy");
    }



	public static ElementDependency createFixture(WorkElement parent,
            ElementDependencyScope scope, ClientTestController ctc)
            throws Exception
    {
        return createElementDependency(random(), parent, scope, ctc);
    } 
    
    public static void updateFixture(ElementDependency dependency) throws Exception
    {
        HelperTestUtilities.setByIntrospection(dependency, random(), skipSetters);
        dependency.setAmount(new Integer((int) (Math.random() * 1000 - 1)));
    }


	/**
	 * @param index
	 * @param isPredecessor - workelement will be set as the predecessor (parent) if set to true
	 * @param parent - if null, new proposal will be created and saved.
	 * @param ctc
	 * @return
	 * @throws Exception
	 */
	private static ElementDependency createElementDependency(int index,
            WorkElement parent, ElementDependencyScope scope,
            ClientTestController ctc) throws Exception
    {
        ElementDependency dependency = new ElementDependency();
        HelperTestUtilities.setByIntrospection(dependency, index,
            localSkipSetters);
        
        dependency.setAmount(new Integer((int) (Math.random() * 1000 - 1)));
        dependency.setDependencyType(DependencyType.FinishFinish);
        dependency.setLeadlagType(LeadLagType.Lag);
        dependency.setID(null);
        dependency.setUnitType(TimeUnitType.Days);

        if (parent == null)
        {
            parent = WorkElementFixture.retrieveStaticFixture(new Proposal(), ctc);
        }
        
        dependency.setParent(parent);
        
        // predecessor is mandatory
        dependency.setPredecessor(WorkElementFixture.createSaveFixture(
                new Proposal(), scope == null ? null : scope.getPredecessor(), ctc));
        
        // createdBy is read-only so we don't set it
        
        return dependency;
    }
	
	public static WorkElement setElementDependency(GenericProject parent,WorkElementScope scope ) 
	throws Exception {
			
		WorkElement wElement = parent;
		WorkElementScope temp = scope;
		if(temp != null) {
			if(wElement instanceof GenericProject) {
				WorkElement[] children = ((GenericProject)wElement).getChildren();
				Task task = null;//make task the elt to whom all other elts create dependencies 
				if(children != null) {
					for(int i = 0; i<children.length; i++) {
						if(children[i] instanceof Task) {
							task = (Task)children[i];
						}
					}
					if(task != null) {
						//create all dependencies and set task as the parent
						for(int i = 0; i < children.length ; i++) {
							if(!(children[i] instanceof Task)){
								ElementDependency dependency = new ElementDependency();
								HelperTestUtilities.setByIntrospection(dependency,random(),localSkipSetters);
								dependency.setDependencyType(DependencyType.FinishFinish);
								dependency.setLeadlagType(LeadLagType.Lag);
								dependency.setID(null);
								dependency.setUnitType(TimeUnitType.Days);
								dependency.setParent(children[i]);
								dependency.setPredecessor(task);
								children[i].setDependencies(new ElementDependency[]{dependency});
								if(children[i] instanceof Proposal){//we could make it generic project, but the test would be too heavy
									children[i] = setElementDependency((Proposal)children[i],scope.getContainingProject());
								}
							}
						}
					}
				}
			}
		}
		return parent;
	}

    public static void compare(ElementDependency d1, ElementDependency d2) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(d1, d2,localSkipGetters);
        
        // compare amount manually because it needs to be rounded off
        // TODO: this should be removed when action 58864 is addressed - Myriam
        int amount1 = d1.getAmount() == null ? 0 : d1.getAmount().intValue();
        int amount2 = d2.getAmount() == null ? 0 : d2.getAmount().intValue();
        Assert.assertEquals(amount1, amount2);
    }
    
	public static void compare(ElementDependency[] fArray1, ElementDependency[] fArray2) throws AssertionFailedError, Exception
    {
    	sortWithComparator(fArray1, RPMObjectIDComparator.INSTANCE);
    	sortWithComparator(fArray2, RPMObjectIDComparator.INSTANCE);
        RPMObjectFixture.compare(fArray1, fArray2);
        if(fArray1 == null) return;
        for (int i = 0; i < fArray1.length; i++)
        {
            DependencyFixture.compare(fArray1[i], fArray2[i]);
        }
    }

    public static ElementDependencyScope createScope()
    {
        ElementDependencyScope dependencyScope = new ElementDependencyScope();
        dependencyScope.setPredecessor(new WorkElementScope());
        dependencyScope.setCreatedBy(new ResourceScope());
        
        return dependencyScope;
    }

  

}
