/*
 * Created on Jan 25, 2006
 */
package com.ibm.rpm.client.junit.wbs.fixtures;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.GeographicalFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.wbs.containers.Profile;
import com.ibm.rpm.wbs.containers.ProfileCompetencyAssignment;
import com.ibm.rpm.wbs.containers.ProfileSkillAssignment;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.scope.ProfileScope;

public class ProfileFixture extends RPMObjectFixture
{
    protected static ArrayList localSkipSetters = new ArrayList();
    protected static ArrayList localSkipGetters = new ArrayList();
    static
    {
        localSkipSetters.addAll(skipSetters);
        localSkipSetters.add("setProfileName");
        localSkipGetters.addAll(skipGetters);
        localSkipGetters.add("getProfileName");
    }
    
    public static Profile createFixture(ProfileScope scope, int nbChildren,
            ResourceTaskAssignment parent, ClientTestController ctc)
            throws Exception
    {
        Profile profile = new Profile();
        HelperTestUtilities.setByIntrospection(profile, random(),
            localSkipSetters);

//        if (scope.getParent() != null)
//        {
//            if (parent == null)
//            {
//                parent = ResourceTaskAssignmentFixture
//                    .createSaveFixtureForProfile(
//                        new ResourceTaskAssignmentScope(), ctc);
//            }
//            profile.setParent(parent);
//        }
        
        if (scope.isProfileCompetencyAssignments())
        {
            ResourceAttributeCompetencyCategory type = 
                new ResourceAttributeCompetencyCategory();
            ProfileCompetencyAssignment[] assignments = 
                new ProfileCompetencyAssignment[nbChildren];

            for (int i = 0; i < nbChildren; i++)
            {
                assignments[i] = (ProfileCompetencyAssignment) ProfileResourceAttributeAssignmentFixture
                        .createFixture(type, profile, ctc);
            }
            profile.setProfileCompetencyAssignments(assignments);
        }

        if (scope.isProfileSkillAssignments())
        {
            ResourceAttributeSkillCategory type = new ResourceAttributeSkillCategory();
            ProfileSkillAssignment[] assignments = new ProfileSkillAssignment[nbChildren];

            for (int i = 0; i < nbChildren; i++)
            {
                assignments[i] = (ProfileSkillAssignment) ProfileResourceAttributeAssignmentFixture
                        .createFixture(type, profile, ctc);
            }
            profile.setProfileSkillAssignments(assignments);
        }

        if (scope.getGeographical() != null)
        {
            Geographical geo = GeographicalFixture.createSaveFixture(1, ctc);
            profile.setGeographical(geo);
        }

        if (scope.getOrganizational() != null)
        {
            Organizational org = OrganizationalFixture
                .createSaveFixture(1, ctc);
            profile.setOrganizational(org);
        }

        if (scope.getPool() != null)
        {
            Pool pool = PoolFixture.createSaveFixture(ctc);
            profile.setPool(pool);
        }

        return profile;
    }

    public static Profile createFixture(ResourceTaskAssignment parent,
            ProfileScope scope, ClientTestController ctc) throws Exception
    {
        scope.setParent(null);
        Profile profile = createFixture(scope, 0, null, ctc);
//        profile.setParent(parent);
        return profile;
    }


    public static void compare(Profile profile1, Profile profile2,
            ProfileScope scope) throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(profile1, profile2, localSkipGetters);
        
        if(scope != null)
        {
            if (scope.getGeographical() != null)
            {
                GeographicalFixture.compare(profile1.getGeographical(),
                    profile2.getGeographical());
            }
            
            if (scope.getOrganizational() != null)
            {
                OrganizationalFixture.compare(profile1.getOrganizational(),
                    profile2.getOrganizational());
            }
            
            if(scope.getParent() != null)
            {
                
            }
            
            if(scope.getPool() != null)
            {
                PoolFixture.compare(profile1.getPool(),
                    profile2.getPool());
            }
            
            if (scope.isProfileCompetencyAssignments())
            {
                ProfileResourceAttributeAssignmentFixture.compare(profile1
                    .getProfileCompetencyAssignments(), profile2
                    .getProfileCompetencyAssignments());
            }
            
            if (scope.isProfileSkillAssignments())
            {
                ProfileResourceAttributeAssignmentFixture.compare(profile1
                    .getProfileSkillAssignments(), profile2
                    .getProfileSkillAssignments());
            }
        }
    }
}
