/*
 * Created on Dec 12, 2005
 *
 */
package com.ibm.rpm.client.junit.wbs.fixtures;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.applicationadministration.scope.GeographicalScope;
import com.ibm.rpm.applicationadministration.scope.OrganizationalScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.SecurityRoleFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.containers.Profile;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ProfileScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author karim
 *  
 */
public class ResourceTaskAssignmentFixture extends RPMObjectFixture
{

    
    public static ResourceTaskAssignment createSaveResourceTaskAssignmentForTask(WorkElement task, ClientTestController ctc) throws Exception
    {
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();

        resourceTaskAssignment.setParent(task);
        Resource resource  = TaskAssignmentFixture.createProperResource(resourceTaskAssignment,
            ctc);
        resource = (Resource) save(ctc, resource, new ResourceScope());
        resourceTaskAssignment.setResource(resource);
        SecurityRole securityRole = SecurityRoleFixture.createSecurityRole(task);
        securityRole = (SecurityRole) ctc.loadFromID(securityRole, new RPMObjectScope());
        resourceTaskAssignment.setSecurityRole(securityRole);
        return resourceTaskAssignment;
    }
    
    public static ResourceTaskAssignment createFixture(
            ResourceTaskAssignmentScope scope, boolean isProjectAssignment,
            ClientTestController ctc) throws AssertionError, Exception
    {

        RPMObject parent = null;
        
        if (scope.getParent() instanceof ScopeElementScope)
        {
            parent = TaskAssignmentFixture.createProperScopeElement(ctc);
        }
        else
        {
            parent = TaskAssignmentFixture.createProperWorkElement(ctc);
        }

        ResourceTaskAssignment assignment = null;
        Resource resource = null;
        Profile profile = null;
        if (scope.getResource() != null)
        {
            resource = TaskAssignmentFixture.createProperResource(parent,
                    ctc);
        } else if (scope.getProfile() != null) {
            profile = ProfileFixture.createFixture(scope.getProfile(), 1,
                    null, ctc);
        }
        assignment = 
            parent instanceof WorkElement
            ? createFixture(parent,
                resource, profile, scope, isProjectAssignment, ctc)
            : createFixture(parent,
                    resource, profile, scope, isProjectAssignment, ctc);   

        return assignment;
    }

    /**
     * provide TaskAssignment in scope to get 'Affect to Project' otherwise
     * you'll get 'Assign as Participant'
     * 
     * Parent must not be null.
     * 
     * @param parentWorkElement
     * @param resource
     * @param scope
     * @param ctc
     * @return
     * @throws Exception
     */
    public static ResourceTaskAssignment createFixture(RPMObject parent,
            Resource resource, Profile profile,
            ResourceTaskAssignmentScope scope, boolean isProjectAssignment,
            ClientTestController ctc) throws Exception
    {
        ResourceTaskAssignment assignment = new ResourceTaskAssignment();
        
        if(parent == null)
        {
            throw new IllegalArgumentException("The parent parameter can not be null");
        }
        
        assignment.setParent(parent);
        assignment.setResource(resource);
        assignment.setProfile(profile);
        
        if (profile != null) {
//            profile.setParent(assignment);
        }
        
        // Mandatory field
        SecurityRole securityRole = SecurityRoleFixture.createSecurityRole(parent);
        assignment.setSecurityRole(securityRole);

        if (scope != null)
        {
            //Leave first assignment status as null
            assignment.setAssignmentStatus(null);

            if (scope.getTaskAssignment() != null)
            {
                TaskAssignment taskAssignment = TaskAssignmentFixture
                        .createTaskAssignmentUnsavableFixture(scope
                        .getTaskAssignment(), isProjectAssignment, ctc);
                taskAssignment.setParent(assignment);
                assignment.setTaskAssignment(taskAssignment);
            }
        }

        return assignment;
    }

    public static ResourceTaskAssignment createFixture(
            RPMObject parent, Profile profile,
            ClientTestController ctc) throws Exception
    {
        ResourceTaskAssignment assignment = new ResourceTaskAssignment();
        assignment.setSecurityRole(SecurityRoleFixture.createSecurityRole(parent));

        assignment.setParent(parent);
        assignment.setProfile(profile);
//        profile.setParent(assignment);
        assignment.setTaskAssignment(new TaskAssignment());

        return assignment;
    }

    public static ResourceTaskAssignment createSaveFixture(RPMObject parent,
            Resource resource, ResourceTaskAssignmentScope scope,
            boolean isProjectAssignment, ClientTestController ctc)
            throws AssertionError, Exception
    {
        ResourceTaskAssignment assignment = createFixture(
                parent, resource, null, scope, isProjectAssignment, ctc);
        assignment = (ResourceTaskAssignment) save(ctc, assignment,
                scope);
        return assignment;
    }

    public static ResourceTaskAssignment createSaveFixture(
            ResourceTaskAssignmentScope scope, boolean isProjectAssignment,
            ClientTestController ctc) throws AssertionError, Exception
    {
        ResourceTaskAssignment assignment = createFixture(
                scope, isProjectAssignment, ctc);
        assignment = (ResourceTaskAssignment) save(ctc, assignment,
                scope);
        return assignment;
    }

    public static ResourceTaskAssignment createSaveFixtureForProfile(
            ResourceTaskAssignmentScope scope, ClientTestController ctc)
            throws AssertionError, Exception
    {
        ResourceTaskAssignment assignment = createFixture(scope, false, ctc);

        assignment = (ResourceTaskAssignment) save(ctc, assignment,
                scope);

        return assignment;
    }

    public static void updateFixture(ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, ClientTestController ctc)
            throws Exception
    {
        /*
         * The security role should not be updated, unless the parent of the object changes.
         */
//        assignment.setSecurityRole(SecurityRoleFixture.createSecurityRole(assignment.getParent()));

        if (scope != null)
        {

            if (scope.isAssignmentStatus())
            {
                // TODO assignmentStatus
                // assignment.setAssignmentStatus(...);
            }
            if (scope.getTaskAssignment() != null
                    && assignment.getTaskAssignment() != null)
            {
                // the object itself always stays the same
                TaskAssignmentFixture.updateFixture(assignment
                        .getTaskAssignment(), scope.getTaskAssignment(), ctc);
            }
            if (scope.getProfile() != null && assignment.getProfile() != null)
            {
                assignment.setProfile(ProfileFixture.createFixture(scope
                    .getProfile(), 1, null, ctc));
            }
        }
    }

    public static ResourceTaskAssignmentScope createScope(boolean isParent,
            boolean isTaskAssignment, boolean isProfile, boolean isResource,
            boolean isAssignmentStatus)
    {
        ResourceTaskAssignmentScope scope = new ResourceTaskAssignmentScope();
        if (isParent)
        {
            WorkElementScope workElementScope = new WorkElementScope();
            workElementScope.setScheduleDates(true);
            workElementScope.setContainingProject(workElementScope);
            scope.setParent(workElementScope);
        }
        if (isTaskAssignment)
        {
            scope.setTaskAssignment(new TaskAssignmentScope());
        }
        if (isProfile)
        {
            ProfileScope pScope = new ProfileScope();
            pScope.setGeographical(new GeographicalScope());
            pScope.setOrganizational(new OrganizationalScope());
//            pScope.setParent(scope);
            pScope.setPool(new PoolScope());
            pScope.setProfileCompetencyAssignments(true);
            pScope.setProfileSkillAssignments(true);
            scope.setProfile(pScope);

        }
        if (isResource)
        {
            ResourceScope resourceScope = new ResourceScope();
            resourceScope.setCalendar(new CalendarScope());
            // ikusakov: if you want Pool parent here let's discuss this
            // otherwise don't put it - it breaks my tests (comparasion)
            resourceScope.setParent(null);
            scope.setResource(resourceScope);
        }
        scope.setAssignmentStatus(isAssignmentStatus);
        return scope;
    }

    public static void compare(ResourceTaskAssignment[] array1,
            ResourceTaskAssignment[] array2, ResourceTaskAssignmentScope scope)
            throws AssertionFailedError, Exception
    {
        sortWithComparator(array1, new RPMObjectIDComparator());
        sortWithComparator(array2, new RPMObjectIDComparator());

        RPMObjectFixture.compare(array1, array2);
        if (array1 == null)
            return;
        for (int i = 0; i < array1.length; i++)
        {
            ResourceTaskAssignmentFixture.compare(array1[i], array2[i], scope);
        }
    }

    public static void compare(ResourceTaskAssignment assignment1,
            ResourceTaskAssignment assignment2,
            ResourceTaskAssignmentScope scope) throws AssertionFailedError,
            Exception
    {
        Assert.assertEquals(assignment1.getResourceType().getValue(),
                assignment2.getResourceType().getValue());
        
//        Assert.assertEquals(assignment1.getSecurityRole().getID(),
//                assignment2.getSecurityRole().getID());
        if (scope != null)
        {
            if (scope.getParent() != null)
            {                
                if (assignment1.getParent() instanceof WorkElement)
                {
                    // we cannot just do compare on WorkElement, 
                    // because the type can change
                    Assert.assertTrue(assignment2.getParent() instanceof WorkElement);
                    Assert.assertEquals(assignment1.getParent().getID(), assignment2.getParent().getID());
                } 
                else if (assignment1.getParent() instanceof ScopeElement)
                {
                    // we cannot just do compare on WorkElement, 
                    // because the type can change (ChangeRequest into ScopeTask)                    
                    Assert.assertTrue(assignment2.getParent() instanceof ScopeElement);
                    Assert.assertEquals(assignment1.getParent().getID(), assignment2.getParent().getID());                    
                }
            }
            if (scope.getResource() != null)
            {
                ResourceFixture.compare(assignment1.getResource(), assignment2
                        .getResource(), scope.getResource());
            }
            if (scope.isAssignmentStatus()
                    && ((assignment1.getAssignmentStatus() != null) || (assignment2
                            .getAssignmentStatus() != null)))
            {
                DatafieldFixture.compare(assignment1.getAssignmentStatus(),
                        assignment2.getAssignmentStatus());
            }

            if (scope.getTaskAssignment() != null)
            {
                TaskAssignmentFixture.compare(assignment1.getTaskAssignment(),
                        assignment2.getTaskAssignment(), scope
                                .getTaskAssignment());
            }
            if (scope.getProfile() != null)
            {
                ProfileFixture.compare(assignment1.getProfile(), assignment2
                        .getProfile(), scope.getProfile());
            }
        }
    }
}
