/*
 * Created on Dec 17, 2005
 *
 */
package com.ibm.rpm.client.junit.wbs.fixtures;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.contract.fixtures.ContractFixture;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimeCodeAssignmentFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultStepFixture;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.timesheet.containers.DefaultStep;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ConstraintType;

/**
 * @author karim
 * 
 */
public class TaskAssignmentFixture extends RPMObjectFixture
{
    protected static ArrayList localSkipSetters, localSkipGetters;

    static
    {
        // Setters
        localSkipSetters = new ArrayList();
        localSkipSetters.addAll(skipSetters);
        // TODO figure out which fields to skip
        localSkipSetters.add("setExpectedFinishDate");
        localSkipSetters.add("setExpectedStartDate");
        localSkipSetters.add("setAssignmentFinishDate");
        localSkipSetters.add("setAssignmentStartDate");
        localSkipSetters.add("setBaselineFinishDate");
        localSkipSetters.add("setBaselineStartDate");
        localSkipSetters.add("setCalculatedLeveledFinishDate");
        localSkipSetters.add("setCalculatedLeveledStartDate");
        localSkipSetters.add("setForecastStartDate");
        localSkipSetters.add("setForecastFinishDate");
        localSkipSetters.add("setWorkSchedule");
        localSkipSetters.add("setBaselineWork");

        // Getters
        localSkipGetters = new ArrayList();
        localSkipGetters.addAll(skipGetters);
        // TODO figure out which fields to skip
        localSkipGetters.add("getAssignmentStartDate");

        // these values are set by the stored proc
        localSkipGetters.add("getActualWork");
        localSkipGetters.add("getWorkSchedule");
        localSkipGetters.add("getBaselineWork");
        localSkipGetters.add("getCostPerHour");
        localSkipGetters.add("getCalculatedLeveledWork");
        localSkipGetters.add("getDefaultLabor");
        localSkipGetters.add("getPercentageComplete");
        localSkipGetters.add("getPercentageWork");
        localSkipGetters.add("getSellingRatePerHour");
        localSkipGetters.add("getSpecialCostPerHour");
        localSkipGetters.add("getSpecialSellingRatePerHour");
        localSkipGetters.add("getProposedDurationHours");

        // All the *FinishDate fields are changed by the SP so they shouldn't be
        // compared
        localSkipGetters.add("getExpectedFinishDate");
        localSkipGetters.add("getAssignmentFinishDate");
        localSkipGetters.add("getBaselineFinishDate");
        localSkipGetters.add("getCalculatedLeveledFinishDate");
        localSkipGetters.add("getForecastFinishDate");
        localSkipGetters.add("getProposedFinishDate");

        localSkipGetters.add("getPercentageRateAdjustment");
        localSkipGetters.add("getRemainingHours");

        localSkipGetters.add("getUnitsPercentage");
        localSkipGetters.add("getLastScheduleWork");
        localSkipGetters.add("getDaysOfDelay");

        // When assignmentStartDate is set expectedStartDate is modified by
        // the SP call
        localSkipGetters.add("getExpectedStartDate");
        // When EETCHours is set expectedWork is modified by the SP call
        localSkipGetters.add("getExpectedWork");
        // When costLaborCode is set billable is modified by the SP call
        localSkipGetters.add("getBillable");
        // calculated field
        localSkipGetters.add("getEETCHours");
        localSkipGetters.add("getComplete");
        // When we do copyProposedToPlan these fields are modified by the SP
        // call
        localSkipGetters.add("getPlannedStartDate");
        localSkipGetters.add("getProposedStartDate");
        localSkipGetters.add("getPlannedFinishDate");
        localSkipGetters.add("getPlannedDurationHours");

        localSkipGetters.add("getShownInTimesheet");
    }

    /**
     * TaskAssignment is totally dependent on ResourceTaskAssignment, So use
     * other fixture methods, that does create it from ResourceTaskAssignment
     *  
     * @param scope
     * @param isProjectAssignment - used to set EETC on non-project assignments
     * @param ctc
     * @return
     * @throws Exception
     */
    public static TaskAssignment createTaskAssignmentUnsavableFixture(
            TaskAssignmentScope scope, boolean isProjectAssignment, ClientTestController ctc)
            throws Exception
    {
        TaskAssignment assignment = new TaskAssignment();

        // explicitly set fields
        GregorianCalendar calendar = new GregorianCalendar();
        // assignment.setExpectedStartDate(calendar); //-- read-only
        // assignment.setExpectedFinishDate(calendar); //-- read-only
        assignment.setAssignmentStartDate(calendar);
        // assignment.setAssignmentFinishDate(calendar); //-- read-only!!

        // assignment.setActualWork(new Double(0.0)); //-- read-only until
        // there's a timesheet entry
        // assignment.setDaysOfDelay(new Integer(0)); //-- read-only conditional
        // must be <= left for resource
        // assignment.setExpectedWork(new Double(8.0)); // --read-only !!
        // assignment.setPercentageComplete(new Double(0.0)); //-- read-only
        // when there's no
        // assignment.setPercentageWork(new Double(0.0)); //-- resource with an
        // actualStartDate
        // assignment.setLastScheduleWork(new Double(0.0)); // -- read-only
        assignment.setBillable(new Boolean(true));

        assignment.setCostPerHour(new Double(88.99));
        assignment.setDefaultLabor(new Boolean(false));
        assignment.setSellingRatePerHour(new Double(0.0));
        assignment.setSpecialCostPerHour(new Double(0.0));
        assignment.setSpecialSellingRatePerHour(new Double(0.0));
        // 0 .. 100
        assignment.setPercentageRateAdjustment(new Integer(50));
        assignment.setProposedDurationHours(new Double(8.0));
        
        // EETC must be null for project assignments
        if (!isProjectAssignment)
        {
            assignment.setEETCHours(new Double(40));
        }

        if (scope != null)
        {
            if (scope.getContract() != null)
            {
                Contract contract = ContractFixture.createSaveFixture(ctc);
                assignment.setContract(contract);
            }
            if (scope.getCostLaborCode() != null)
            {
                ChargeCode costLaborCode = ChargeCodeFixture
                    .createAndSaveSimpleChargeCode(ctc);
                assignment.setCostLaborCode(costLaborCode);
            }
            if (scope.getRevenueLaborCode() != null)
            {
                ChargeCode revenueLaborCode = ChargeCodeFixture
                    .createAndSaveSimpleChargeCode(ctc);
                assignment.setRevenueLaborCode(revenueLaborCode);
            }
        }
        return assignment;
    }

    /**
     * This method will creates a Proposal, a Task under that proposal, and then
     * will assign the resource to that task.
     * 
     * @param scope
     * @param ctc
     * @return
     * @throws Exception
     */
    public static TaskAssignment createSaveFixture(TaskAssignmentScope scope,
            ClientTestController ctc) throws Exception
    {
        TaskAssignment taskAssignment = createFixture(scope, ctc);
        ResourceTaskAssignment resourceTaskAssignment = (ResourceTaskAssignment) taskAssignment
            .getParent();

        if (scope == null)
            scope = new TaskAssignmentScope();
        if (scope.getParent() == null)
            scope.setParent(new ResourceTaskAssignmentScope());
        ((ResourceTaskAssignmentScope) scope.getParent())
            .setTaskAssignment(scope);

        resourceTaskAssignment = (ResourceTaskAssignment) ctc.save(
            resourceTaskAssignment, scope.getParent()).getRpmObject();

        return resourceTaskAssignment.getTaskAssignment();
    }

    public static TaskAssignment createSaveFixture(
            WorkElement parentWorkElement, Resource resource,
            TaskAssignmentScope scope, ClientTestController ctc)
            throws Exception
    {

        if (scope == null)
            scope = new TaskAssignmentScope();
        ResourceTaskAssignmentScope rtascope = (ResourceTaskAssignmentScope) scope
            .getParent();
        if (rtascope == null)
        {
            rtascope = new ResourceTaskAssignmentScope();
            scope.setParent(rtascope);
        }
        rtascope.setTaskAssignment(scope);

        WorkElementScope wescope = new WorkElementScope();
        wescope.setContainingProject(new WorkElementScope());
        rtascope.setParent(wescope);

        TaskAssignment taskAssignment = createFixture(parentWorkElement,
            resource, scope, ctc);
        ResourceTaskAssignment resourceTaskAssignment = (ResourceTaskAssignment) taskAssignment
            .getParent();

        resourceTaskAssignment.setParent(parentWorkElement);

        resourceTaskAssignment = (ResourceTaskAssignment) ctc.save(
            resourceTaskAssignment, scope.getParent()).getRpmObject();

        return resourceTaskAssignment.getTaskAssignment();
    }

    public static ScopeElement createProperScopeElement(ClientTestController ctc)
            throws Exception
    {
        ChangeRequest element = ScopeElementFixture
            .createSaveChangeRequest(ctc);
        element.setDefaultScope(Boolean.FALSE);
        element = (ChangeRequest) ctc.save(element, null).getRpmObject();
        ScopeElementScope scope = new ScopeElementScope();
        scope.setProject(new WorkElementScope());
        scope.setParent(new RPMObjectScope());
        element = (ChangeRequest) ctc.loadFromID(element, scope);
        return element;
    }

    /**
     * SP_SEARCH_2 is a very evil stored procedure that does a lot of
     * validation. You can not pass just any work element, it has to comply with
     * certain constrains which this method provides.
     * 
     * @param ctc
     * @return
     * @throws Exception
     */
    public static WorkElement createProperWorkElement(ClientTestController ctc)
            throws Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setContainingProject(scope);

        Proposal proposal = createProposal(scope, ctc);
        Task task = createTask(proposal, scope, ctc);

        return task;
    }

    /**
     * SP_SEARCH_2 is a very evil stored procedure that does a lot of
     * validation. You can not pass just any resource, it has to comply with
     * certain constrains which this method provides.
     * 
     * Required by SP_SEACRH_2: - has to be active - resource.setCalendar -
     * resource.setEffectiveStartDate
     * 
     * 
     * @param properWorkElement
     *            use createProperWorkElement() method for this
     * @param ctc
     * @return
     * @throws Exception
     */
    public static Resource createProperResource(
            RPMObject properWorkOrScopeElement, ClientTestController ctc)
            throws Exception
    {
        // Create the resource
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setParent(new PoolScope());
        resourceScope.setCalendar(new CalendarScope());

        Pool pool = PoolFixture.createSaveFixture(ctc);

        // the pool must be deleted last
        ctc.removeObjectToDelete(pool);
        ctc.addObjectToDelete(pool, true);

        Resource resource = ResourceFixture.createFixture(pool, resourceScope,
            ctc);

        return createProperResource(resource, properWorkOrScopeElement, ctc);
    }

    /**
     * Properly configures a resource to be used with a given workElement for
     * assigning resource to task
     * 
     * @param resource
     * @param properWorkOrScopeElement
     * @param ctc
     * @return
     * @throws Exception
     */
    public static Resource createProperResource(Resource resource,
            RPMObject properWorkOrScopeElement, ClientTestController ctc)
            throws Exception
    {
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());

        resource.setActive(new Boolean(true));
        resource.setCalendar(RPMCalendarFixture
            .retrieveStaticFixtureForInserts(ctc));

        Calendar startDate = null;
        if (properWorkOrScopeElement instanceof WorkElement)
        {
            startDate = ((WorkElement) properWorkOrScopeElement)
                .getContainingProject().getEstimatedStartDate();
        }
        else if (properWorkOrScopeElement instanceof ScopeElement
                && ((ScopeElement) properWorkOrScopeElement).getProject() != null)
        {
            startDate = ((ScopeElement) properWorkOrScopeElement).getProject()
                .getEstimatedStartDate();
        }

        resource.setEffectiveStartDate(startDate);
        resource = (Resource) save(ctc, resource, resourceScope);

        // resource will be deleted at the end with the pool
        ctc.removeObjectToDelete(resource);

        return resource;
    }

    /**
     * This method will creates a Proposal, a Task under that proposal, and then
     * will assign the resource to that task.
     * 
     * @param resource
     * @param scope
     * @param ctc
     * @return
     * @throws Exception
     */
    public static TaskAssignment createFixture(TaskAssignmentScope scope,
            ClientTestController ctc) throws Exception
    {
        WorkElement task = createProperWorkElement(ctc);
        Resource resource = createProperResource(task, ctc);

        if (scope == null)
            scope = new TaskAssignmentScope();

        return createFixture(task, resource, scope, ctc);
    }

    public static TaskAssignment createFixture(Resource resource,
            TaskAssignmentScope scope, ClientTestController ctc)
            throws Exception
    {
        WorkElement task = createProperWorkElement(ctc);
        return createFixture(task, resource, scope, ctc);
    }

    public static TaskAssignment createFixture(WorkElement parentWorkElement,
            Resource resource, TaskAssignmentScope scope,
            ClientTestController ctc) throws Exception
    {
        ResourceTaskAssignmentScope resourceTaskAssignmentScope = new ResourceTaskAssignmentScope();
        resourceTaskAssignmentScope.setTaskAssignment(scope);
        ResourceTaskAssignment resourceTaskAssignment = ResourceTaskAssignmentFixture
            .createFixture(parentWorkElement, resource, null,
                resourceTaskAssignmentScope, false, ctc);
        return resourceTaskAssignment.getTaskAssignment();
    }

    public static void updateFixture(TaskAssignment assignment,
            TaskAssignmentScope scope, ClientTestController ctc)
            throws Exception
    {
        Calendar startDate = new GregorianCalendar(2006, 1, 1);
        assignment.setAssignmentStartDate(startDate);
        // assignment.setEETCHours(new Double(16));
        assignment.setUnitsPercentage(new Double(120));
        assignment.setDaysOfDelay(new Integer(5));
        assignment.setDefaultLabor(new Boolean(true));
        assignment.setBillable(new Boolean(false));

        // 0 .. 100
        assignment.setPercentageRateAdjustment(new Integer(100));
        assignment.setCostPerHour(new Double(10));
        assignment.setSpecialCostPerHour(new Double(12));
        assignment.setSellingRatePerHour(new Double(11));
        assignment.setSpecialSellingRatePerHour(new Double(13));

        // TODO modify these fields
        // Security role
        // constrain

        // TODO complete later
        if (scope != null)
        {
            if (scope.getContract() != null)
            {
                Contract contract = ContractFixture.createSaveFixture(ctc);
                assignment.setContract(contract);
            }

            if (scope.getCostLaborCode() != null)
            {
                ChargeCode laborChargeCode = ChargeCodeFixture
                    .createAndSaveSimpleChargeCode(ctc);
                assignment.setCostLaborCode(laborChargeCode);
                assignment.setCostType(laborChargeCode.getFinancialCategory());
            }

            if (scope.getRevenueLaborCode() != null)
            {
                ChargeCode laborRevCode = ChargeCodeFixture
                    .createAndSaveSimpleChargeCode(ctc);
                assignment.setRevenueLaborCode(laborRevCode);
            }

            // TODO add other related objects here if applicable
        }
    }

    public static TaskAssignmentScope createScope(boolean isParent,
            boolean isContract, boolean isCostLaborCode,
            boolean isRevenueLaborCode)
    {
        TaskAssignmentScope scope = new TaskAssignmentScope();

        if (isParent)
        {
            scope.setParent(ResourceTaskAssignmentFixture.createScope(true,
                false, false, true, true));
        }

        if (isContract)
            scope.setContract(ContractFixture.createScope());

        if (isCostLaborCode)
            scope.setCostLaborCode(ChargeCodeFixture.createChargeCodeScope());

        if (isRevenueLaborCode)
            scope
                .setRevenueLaborCode(ChargeCodeFixture.createChargeCodeScope());

        // TODO implement later

        return scope;
    }

    public static void compare(TaskAssignment[] array1,
            TaskAssignment[] array2, TaskAssignmentScope scope)
            throws AssertionFailedError, Exception
    {
        // TODO implement comparator later
        // sortWithComparator(array1, new RPMObjectNameComparator());
        // sortWithComparator(array2, new RPMObjectNameComparator());
        RPMObjectFixture.compare(array1, array2);
        if (array1 == null)
            return;
        for (int i = 0; i < array1.length; i++)
        {
            TaskAssignmentFixture.compare(array1[i], array2[i], scope);
        }
    }

    public static void compare(TaskAssignment assignment1,
            TaskAssignment assignment2, TaskAssignmentScope scope)
            throws AssertionFailedError, Exception
    {
        RPMObjectFixture.compareWithSkip(assignment1, assignment2,
            localSkipGetters);

        // TODO complete later
        if (scope != null)
        {
            // if (scope.getSummaryTimesheets() != null)
            // {
            // SummaryTimesheetFixture.compare(assignment1
            // .getSummaryTimesheets(), assignment2
            // .getSummaryTimesheets());
            // }
            // if (scope.getActivities() != null)
            // {
            // //TODO use fixtures later
            // }
            if (scope.getContract() != null
                    && !(assignment1.getParent().getParent() instanceof ScopeElement))
            {
                ContractFixture.compare(assignment1.getContract(), assignment2
                    .getContract(), scope.getContract());
            }
            if (scope.getCostLaborCode() != null)
            {
                ChargeCodeFixture.compare(assignment1.getCostLaborCode(),
                    assignment2.getCostLaborCode());
            }
            if (scope.getRevenueLaborCode() != null)
            {
                ChargeCodeFixture.compare(assignment1.getRevenueLaborCode(),
                    assignment2.getRevenueLaborCode());
            }
            // TODO add other comparisons later
        }
    }
    
    public static void comparePercentageComplete(
            GenericTaskAssignment expected, GenericTaskAssignment actual)
    {
        Assert.assertNotNull(expected);
        Assert.assertNotNull(actual);
        Assert.assertTrue(expected instanceof TaskAssignment
                && actual instanceof TaskAssignment);
        Assert.assertEquals(
            ((TaskAssignment) expected).getPercentageComplete(),
            ((TaskAssignment) actual).getPercentageComplete());
    }
    
    public static void compareEETC(GenericTaskAssignment expected,
            GenericTaskAssignment actual)
    {
        Assert.assertNotNull(expected);
        Assert.assertNotNull(actual);
        Assert.assertTrue(expected instanceof TaskAssignment
                && actual instanceof TaskAssignment);
        Assert.assertEquals(((TaskAssignment) expected).getEETCHours(),
            ((TaskAssignment) actual).getEETCHours());
    }

    /**
     * Required by SP_SEARCH_2: - proposal.setEstimatedStartDate(new
     * GregorianCalendar()); (not null) - proposal.setCurrency(currency);
     * 
     * 
     * @param scope
     * @param ctc
     * @return
     * @throws Exception
     */
    private static Proposal createProposal(WorkElementScope scope,
            ClientTestController ctc) throws Exception
    {
        Proposal proposal = new Proposal();
        Portfolio parent = PortfolioFixture.loadOrCreateRootPortfolio(ctc);
        proposal.setPortfolio(parent);
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPriority(new Integer(500));
        proposal.setEstimatedStartDate(new GregorianCalendar());
        RPMCalendar calendar = RPMCalendarFixture
            .retrieveStaticFixtureForInserts(ctc);
        proposal.setCalendar(calendar);
        // hardcoded assignment type needed for Profile
        proposal.setAssignmentType(AssignmentType.Work);

        Currency currency = DatafieldFixture
            .createCurrency(DatafieldFixture.CURRENCY_USD);
        proposal.setCurrency(currency);

        if (scope != null && scope.getDefaultSteps() != null)
        {
            DefaultStep[] defaultSteps = new DefaultStep[2];
            defaultSteps[0] = DefaultStepFixture.createFixture(proposal, scope
                .getDefaultSteps(), ctc);
            defaultSteps[1] = DefaultStepFixture.createFixture(proposal, scope
                .getDefaultSteps(), ctc);
            proposal.setDefaultSteps(defaultSteps);
        }

        if (scope != null && scope.isTimeCodeAssignments())
        {
            TimeCodeAssignment timeCodeAssign1 = TimeCodeAssignmentFixture
                .createFixture(proposal, true, ctc);
            TimeCodeAssignment timeCodeAssign2 = TimeCodeAssignmentFixture
                .createFixture(proposal, false, ctc);
            proposal.setTimeCodeAssignments(new TimeCodeAssignment[]
            {
                    timeCodeAssign1, timeCodeAssign2
            });
            proposal.setTimeCode1(timeCodeAssign1.getTimeCode());
            proposal.setTimeCode2(timeCodeAssign2.getTimeCode());
        }

        proposal = (Proposal) save(ctc, proposal, scope);
        return proposal;
    }

    /**
     * - task.setConstraintDate(DateUtil.convertToCalendar("2050-12-31")); -
     * task.setConstraintType(ConstraintType.FinishNoEarlierThan);
     * 
     * @param parent
     * @param scope
     * @param ctc
     * @return
     * @throws Exception
     */
    private static Task createTask(WorkElement parent, WorkElementScope scope,
            ClientTestController ctc) throws Exception
    {
        Calendar today = new GregorianCalendar();
        Task task = new Task();

        task.setName(HelperTestUtilities.createName());
        task.setParent(parent);
        task.setPriority(new Integer(500));
        task.setEstimatedStartDate(today);

        task.setContainingProject(WorkElementFixture.retrieveStaticFixture(
            new Project(), ctc));

        task.setConstraintDate(DateUtil.convertToCalendarNoTime("2050-12-31"));
        task.setConstraintType(ConstraintType.FinishNoEarlierThan);

        task = (Task) save(ctc, task, scope);
        task.setParent(parent);
        return task;
    }
}
