package com.ibm.rpm.client.junit.wbs.testcases;

import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.RpmTestProperties;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * Base class for each type of ResourceTaskAssignment test
 * 
 * @author karim
 * @author ikusakov
 */
public abstract class AbstractTestResourceTaskAssignment extends ClientTestController
{
    protected ResourceTaskAssignmentScope mainScope = null;
    
    protected WorkElement parentWorkElement = null;
    
    protected boolean useProposalParent = false;

    protected Resource resource = null;
    
    private WorkElementScope workElementScope = null;
    
    public AbstractTestResourceTaskAssignment(String testName)
    {
        setName(testName);
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        // mainScope will be created in subclasses, this way:
        // mainScope = ResourceTaskAssignmentFixture.createScope(true, false, true,
        //  true, true, true, true);
        if (mainScope == null) {
            throw new Exception("Please define mainScope field in " +
                    "before calling this method.");
        }
        
        workElementScope = new WorkElementScope();
        workElementScope.setContainingProject(new WorkElementScope());
        workElementScope.setParent(new WorkElementScope());
        workElementScope
                .setResourceTaskAssignments(mainScope);
        
        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        save(proposal, workElementScope);
        
        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setContainingProject(WorkElementFixture
            .retrieveStaticFixture(new Project(), this));
        task.setParent(proposal);
        save(task, workElementScope);
        
        if (useProposalParent) {
            parentWorkElement = (Proposal) loadFromID(proposal, workElementScope);
        }
        else
        {
            parentWorkElement = task;
        }
        
        if (mainScope.getResource() != null)
        {
            ResourceScope resourceScope = new ResourceScope();
            resourceScope.setParent(new PoolScope());
            resourceScope.setCalendar(new CalendarScope());
            
            resource = new Resource();
            resource.setFullName("RTA Tester");
            resource.setParent(PoolFixture.createSaveFixture(this));
            resource.setCalendar(RPMCalendarFixture
                .retrieveStaticFixtureForInserts(this));
            save(resource, resourceScope);
            
            resource = (Resource) loadFromID(resource, resourceScope);
        }
    }

    public void tearDown() throws Exception
    {
        logger.info("tearDown");
        super.tearDown();
    }

    protected void loadFromWbsPerspective(boolean isProjectAssignment) throws Exception
    {
        ResourceTaskAssignment assignment = null;
            assignment = ResourceTaskAssignmentFixture
                .createFixture(parentWorkElement, resource, null, null, isProjectAssignment, this);
        
        ResourceTaskAssignment saved = (ResourceTaskAssignment) save(
                assignment, mainScope).getRpmObject();

        WorkElement parent = new WorkElement();
        parent.setID(parentWorkElement.getID());

        checkForProperAssignment(parent, saved, mainScope);
    }

    protected void insertUpdateFromWbsPerspective(boolean isProjectAssignment) throws Exception
    {
        // creating new assignment 
        ResourceTaskAssignment assignment = ResourceTaskAssignmentFixture
                .createFixture(parentWorkElement, resource, null, mainScope, isProjectAssignment, this);

        parentWorkElement.setResourceTaskAssignments(new ResourceTaskAssignment[] {assignment});

        // saving from WorkElement perspective
        parentWorkElement = (WorkElement) save(parentWorkElement, workElementScope).getRpmObject();
       
        checkForProperAssignment(parentWorkElement, assignment, mainScope);
        
        // now trying to update assignment and then again save it through
        // WorkElement
        ResourceTaskAssignmentFixture.updateFixture(assignment, mainScope, this);

        parentWorkElement = (WorkElement) save(parentWorkElement,
            workElementScope).getRpmObject();
        
        checkForProperAssignment(parentWorkElement, assignment, mainScope);
    }

    /**
     * Create a new ResourceTaskAssignment under a parent WOrkElement, then
     * attempt to delete it.
     * 
     * @param isProjectAssignment
     * @throws Exception
     */
    protected void deleteFromWbsPerspective(boolean isProjectAssignment) throws Exception
    {
        ResourceTaskAssignment assignment = ResourceTaskAssignmentFixture
                .createFixture(parentWorkElement, resource, null, mainScope, isProjectAssignment, this);

        save(assignment, mainScope).getRpmObject();

        WorkElement parent = new WorkElement();
        parent.setID(parentWorkElement.getID());
        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope
                .setResourceTaskAssignments(mainScope);
        parent = (WorkElement) loadFromID(parent, workElementScope);
        assertNotNull(parent.getResourceTaskAssignments());
        assertTrue("ResourceTaskAssignments array length", parent
                .getResourceTaskAssignments().length > 0);

        ResourceTaskAssignment newAssignment = null;
        for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
        {
            if (parent.getResourceTaskAssignments()[i].getID().equals(
                    assignment.getID()))
            {
                newAssignment = parent.getResourceTaskAssignments()[i];
                break;
            }
        }
        assertNotNull("Proper ResourceTaskAssignment was not loaded",
                newAssignment);
        newAssignment.setDeleted(Boolean.TRUE);

        parent = (WorkElement) save(parent, workElementScope).getRpmObject();
        parent = (WorkElement) loadFromID(parent, workElementScope);
        boolean found = false;
        if (parent.getResourceTaskAssignments() != null
                && parent.getResourceTaskAssignments().length > 0)
        {
            for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
            {
                if (parent.getResourceTaskAssignments()[i].getID().equals(
                        assignment.getID()))
                {
                    ResourceTaskAssignment curRta = parent.getResourceTaskAssignments()[i];
                    /*
                     * Due to SP changes to SP_D_TSKRES_ACT, when a RTA is deleted from RESOURCE_ASSIGNMENTS,
                     * RESOURCE_ID changes from the old value to the ID of the currently logged in user, and 
                     * is not deleted from the database (!)
                     */
                    assertNotNull(curRta.getResource());
                    if(curRta.getResource().getID() == resource.getID()) 
                    {
                        found = true;
                        break;
                    }
                }
            }
        }
        assertFalse("ResourceTaskAssignment should be deleted", found);
    }

    private void checkForProperAssignment(WorkElement parent,
            ResourceTaskAssignment assignmentToCompare, ResourceTaskAssignmentScope scope) throws Exception
    {
        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope
                .setResourceTaskAssignments(scope);
        parent = (WorkElement) loadFromID(parent, workElementScope);

        assertNotNull(parent.getResourceTaskAssignments());
        assertTrue("ResourceTaskAssignments array is empty", parent
                .getResourceTaskAssignments().length > 0);
        boolean found = false;
        for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
        {
            if (parent.getResourceTaskAssignments()[i].getID().equals(
                    assignmentToCompare.getID()))
            {
                ResourceTaskAssignmentFixture.compare(assignmentToCompare,
                        parent.getResourceTaskAssignments()[i], scope);
                found = true;
                break;
            }
        }

        assertTrue("Proper ResourceTaskAssignment was not loaded", found);
    }

    //testInsertWithNoScopeLoadFromXPath()
    //Not applicable because assignment cannot be loaded direclty from
    // XPath
    //they must be loaded through their parents

    protected void insert(ResourceTaskAssignmentScope scope,
            boolean isProjectAssignment, boolean loadFromId) throws Exception
    {
        //insert an element
        ResourceTaskAssignment assignment = insertAssignment(scope, isProjectAssignment, loadFromId);

        //delete the element
        /*
         * Will be deleted when the task itself is deleted. The code is not implemented in the stored procedure
         * to delete a resourcetaskassignment alone, at the moment. 
        *deleteAssignment(assignment, loadFromId);
        **/
    }

    protected void update(ResourceTaskAssignmentScope scope,
            boolean isProjectAssignment, boolean loadFromId) throws Exception
    {
        //insert an element
        ResourceTaskAssignment assignment = insertAssignment(scope, isProjectAssignment, loadFromId);

        //update the element
        assignment = updateAssignment(assignment, scope, loadFromId);

        //delete the element
        deleteAssignment(assignment, loadFromId);
    }

    protected ResourceTaskAssignment insertAssignment(
            ResourceTaskAssignmentScope scope, boolean isProjectAssignment,
            boolean loadFromId) throws Exception
    {
        //create and save a workelement
        ResourceTaskAssignment assignment = null;
        assignment = ResourceTaskAssignmentFixture
            .createFixture(parentWorkElement, resource, null, scope, isProjectAssignment, this);
        
        SingleResult result = save(assignment, scope);

        //compare the returned assignment
        ResourceTaskAssignment returnedAssignment = (ResourceTaskAssignment) result
                .getRpmObject();
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
                scope);

        //reload the assignment and compare it
        returnedAssignment = load(returnedAssignment, scope, loadFromId);
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
                scope);
        /*
         * Some unit tests call this method with a null scope, and
         * a resource will be assigned, but not returned properly.
         * 
         * Moreover, the scope is not implemented properly in the metadata,
         * and changing it would mean risking to break more unit tests.
         * 
         * Then, a null scope really won't load the resource associated, 
         * so the user obtains the behavior he expects. This hack is 
         * only for the unit tests related to RTA.
         */
//        returnedAssignment.setResource(assignment.getResource());
        return returnedAssignment;
    }

    protected ResourceTaskAssignment updateAssignment(
            ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, boolean loadFromId)
            throws Exception
    {
        ResourceTaskAssignmentFixture.updateFixture(assignment, scope, this);

        SingleResult result = save(assignment, scope);

        //compare the returned assignment
        ResourceTaskAssignment returnedAssignment = (ResourceTaskAssignment) result
                .getRpmObject();
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
                scope);

        //reload the assignment and compare it
        returnedAssignment = load(returnedAssignment, scope, loadFromId);
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
                scope);
        return returnedAssignment;
    }

    protected void deleteAssignment(ResourceTaskAssignment assignment,
            boolean loadFromId) throws Exception
    {
        /*
         * Attempt to delete the RTA only if the current user is not the administrator
         * since the SP throws an error message in that case, both in the client and through
         * the WS API.
         */
        if(assignment != null 
                && assignment.getID() != null 
                && (assignment.getResource() == null ||
                !RpmTestProperties.getProperty("rpm.test.user.name").equals(assignment.getResource().getUserName())))
        {
            delete(assignment);
            
            //reload the deleted element
            ResourceTaskAssignment deletedAssignment = loadDeletedAssignment(
                    assignment, loadFromId);
    
            /*
             * Due to SP changes to SP_D_TSKRES_ACT, when a RTA is deleted from RESOURCE_ASSIGNMENTS,
             * RESOURCE_ID changes from the old value to the ID of the currently logged in user, and 
             * is not deleted from the database (!)
             */
            assertNotNull(deletedAssignment);
            if(assignment.getResource() != null) 
            {
                assertNotNull(assignment.getResource());
                assertNotNull(deletedAssignment.getResource());
                assertTrue(!assignment.getResource().getID().equals(deletedAssignment.getResource().getID()));
            }
        }
    }

    protected ResourceTaskAssignment loadDeletedAssignment(
            ResourceTaskAssignment assignment, boolean loadFromId)
            throws Exception
    {
        try
        {
            ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
            rtaScope.setResource(new ResourceScope());
            assignment = load(assignment, rtaScope, loadFromId);
            return assignment;
        }
        catch (Throwable e)
        {
            //element does not exist
        }
        return null;
    }

    protected ResourceTaskAssignment load(ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, boolean loadFromId)
            throws Exception
    {
        if (loadFromId)
        {
            //load from an id
            RPMObject obj = new ResourceTaskAssignment();
            obj.setID(assignment.getID());
            return (ResourceTaskAssignment) loadFromID(obj, scope);
        }
        fail("Assignments cannot be loaded directly from XPath. They must be loaded via their parents.");
        return null;
    }
    
    protected void deleteAssignedWorkUpdatePercentDC(
            ResourceTaskAssignmentScope scope, boolean isProjectAssignment,
            boolean loadFromId) throws Exception
    {
        //insert an element
        ResourceTaskAssignment assignment = insertAssignment(scope,
            isProjectAssignment, loadFromId);

        Task task = (Task) assignment.getParent();

        //delete the element
        deleteAssignment(assignment, loadFromId);

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());

        task.setPlanDate(planDate);
        task = (Task) save(task, scope.getParent()).getRpmObject();
        task.setPercentageDurationCompleted(new Double(10));
        task = (Task) save(task, scope.getParent()).getRpmObject();

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Task[id='" + task.getID()
                + "']", scope.getParent());
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Task);

        // Check that the loaded object is identical to the one we requested 
        // to store and that the percentageDurationCompleted was properly saved
        Task reloadedTask = (Task) rpmObjects[0];
        assertEquals(reloadedTask.getPercentageDurationCompleted(), new Double(
            10));
        WorkElementFixture.compare(task, reloadedTask, (WorkElementScope) scope
            .getParent());       
        
    }
}