package com.ibm.rpm.client.junit.wbs.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;

/**
 * Base class for each type of ResourceTaskAssignment test
 * 
 * @author karim
 * @author ikusakov
 */
public abstract class AbstractTestScopeResourceTaskAssignment extends
        ClientTestController
{
    protected ResourceTaskAssignmentScope mainScope = null;

    protected ScopeElement parentScopeElement = null;

    protected Resource resource = null;

    public AbstractTestScopeResourceTaskAssignment(String testName)
    {
        setName(testName);
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        // mainScope will be created in subclasses, this way:
        // mainScope = ResourceTaskAssignmentFixture.createScope(true, false,
        // true,
        // 	true, true, true, true);
        if (mainScope == null)
        {
            throw new Exception("Please define mainScope field in "
                    + "before calling this method.");
        }
        parentScopeElement = TaskAssignmentFixture
                .createProperScopeElement(this);

        if (mainScope.getResource() != null)
        {
            resource = TaskAssignmentFixture.createProperResource(
                    parentScopeElement, this);
        }
    }

    public void tearDown() throws Exception
    {
        logger.info("tearDown");
        super.tearDown();
    }

    protected void loadFromCRIPerspective(boolean isProjectAssignment) throws Exception
    {
        ResourceTaskAssignment assignment = null;
        assignment = ResourceTaskAssignmentFixture.createFixture(
                parentScopeElement, resource, null, null, isProjectAssignment, this);

        ResourceTaskAssignment saved = (ResourceTaskAssignment) save(
                assignment, mainScope).getRpmObject();

        ChangeRequest parent = new ChangeRequest();
        parent.setID(parentScopeElement.getID());

        checkForProperAssignment(parent, saved, mainScope);
    }

    protected void insertUpdateFromCRIPerspective(boolean isProjectAssignment) throws Exception
    {
        ScopeElementScope scopeElementScope = new ScopeElementScope();
        scopeElementScope.setResourceTaskAssignments(mainScope);

        // creating new assignment 
        ResourceTaskAssignment assignment = ResourceTaskAssignmentFixture
            .createFixture(parentScopeElement, resource, null, mainScope,
                isProjectAssignment, this);

        parentScopeElement
            .setResourceTaskAssignments(new ResourceTaskAssignment[]
            {
                assignment
            });

        // saving from ScopeElement perspective
        parentScopeElement = (ScopeElement) save(parentScopeElement,
            scopeElementScope).getRpmObject();

        checkForProperAssignment(parentScopeElement, assignment, mainScope);

        // now trying to update assignment and then again save it through
        // the parent
        ResourceTaskAssignmentFixture
            .updateFixture(assignment, mainScope, this);

        parentScopeElement = (ScopeElement) save(parentScopeElement,
            scopeElementScope).getRpmObject();

        checkForProperAssignment(parentScopeElement, assignment, mainScope);
    }

    protected void deleteFromWbsPerspective(boolean isProjectAssignment) throws Exception
    {
        ResourceTaskAssignment assignment = null;
        assignment = ResourceTaskAssignmentFixture.createFixture(
            parentScopeElement, resource, null, mainScope, isProjectAssignment,
            this);

        save(assignment, mainScope).getRpmObject();

        ChangeRequest parent = new ChangeRequest();
        parent.setID(parentScopeElement.getID());
        ScopeElementScope ScopeElementScope = new ScopeElementScope();
        ScopeElementScope.setResourceTaskAssignments(mainScope);
        parent = (ChangeRequest) loadFromID(parent, ScopeElementScope);
        assertNotNull(parent.getResourceTaskAssignments());
        assertTrue("ResourceTaskAssignments array length", parent
                .getResourceTaskAssignments().length > 0);

        ResourceTaskAssignment newAssignment = null;
        for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
        {
            if (parent.getResourceTaskAssignments()[i].getID().equals(
                    assignment.getID()))
            {
                newAssignment = parent.getResourceTaskAssignments()[i];
                break;
            }
        }
        assertNotNull("Proper ResourceTaskAssignment was not loaded",
                newAssignment);
        newAssignment.setDeleted(Boolean.TRUE);

        parent = (ChangeRequest) save(parent, ScopeElementScope).getRpmObject();
        parent = (ChangeRequest) loadFromID(parent, ScopeElementScope);
        boolean found = false;
        if (parent.getResourceTaskAssignments() != null
                && parent.getResourceTaskAssignments().length > 0)
        {
            for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
            {
                if (parent.getResourceTaskAssignments()[i].getID().equals(
                        assignment.getID()))
                {
                    found = true;
                    break;
                }
            }
        }
        assertFalse("ResourceTaskAssignment should be deleted", found);
    }

    private void checkForProperAssignment(ScopeElement parent,
            ResourceTaskAssignment assignmentToCompare,
            ResourceTaskAssignmentScope scope) throws Exception
    {
        ScopeElementScope scopeElementScope = new ScopeElementScope();
        scopeElementScope.setResourceTaskAssignments(scope);
        parent = (ScopeElement) loadFromID(parent, scopeElementScope);

        assertNotNull(parent.getResourceTaskAssignments());
        assertTrue("ResourceTaskAssignments array is empty", parent
                .getResourceTaskAssignments().length > 0);
        boolean found = false;
        for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
        {
            if (parent.getResourceTaskAssignments()[i].getID().equals(
                    assignmentToCompare.getID()))
            {
                ResourceTaskAssignmentFixture.compare(assignmentToCompare,
                        parent.getResourceTaskAssignments()[i], scope);
                found = true;
                break;
            }
        }

        assertTrue("Proper ResourceTaskAssignment was not loaded", found);
    }

    protected void insertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        //Not applicable because assignment cannot be loaded direclty from
        // XPath
        //they must be loaded through their parents
        return;
    }

    protected void insert(ResourceTaskAssignmentScope scope,
            boolean isProjectAssignment, boolean loadFromId) throws Exception
    {
        //insert an element
        ResourceTaskAssignment assignment = insertAssignment(scope, isProjectAssignment, loadFromId);

        //delete the element
        deleteAssignment(assignment, loadFromId);
    }

    protected void update(ResourceTaskAssignmentScope scope,
            boolean isProjectAssignment, boolean loadFromId) throws Exception
    {
        //insert an element
        ResourceTaskAssignment assignment = insertAssignment(scope, isProjectAssignment, loadFromId);

        //update the element
        assignment = updateAssignment(assignment, scope, loadFromId);

        //delete the element
        deleteAssignment(assignment, loadFromId);
    }

    protected ResourceTaskAssignment insertAssignment(
            ResourceTaskAssignmentScope scope, boolean isProjectAssignment,
            boolean loadFromId) throws Exception
    {
        //create and save a ScopeElement
        ResourceTaskAssignment assignment = null;
        assignment = ResourceTaskAssignmentFixture.createFixture(
                parentScopeElement, resource, null, scope, isProjectAssignment, this);

        SingleResult result = save(assignment, scope);

        //compare the returned assignment
        ResourceTaskAssignment returnedAssignment = (ResourceTaskAssignment)loadFromID(result
                .getRpmObject(), scope);

        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
                scope);

        //reload the assignment and compare it
        returnedAssignment = load(returnedAssignment, scope, loadFromId);
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
                scope);
        return returnedAssignment;
    }

    protected ResourceTaskAssignment updateAssignment(
            ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, boolean loadFromId)
            throws Exception
    {
        ResourceTaskAssignmentFixture.updateFixture(assignment, scope, this);
        SecurityRole securityRole = new SecurityRole();
        securityRole.setID("SCCHANGEREQUESTRESPONSIBLE");
        assignment.setSecurityRole(securityRole);
        SingleResult result = save(assignment, scope);

        //compare the returned assignment
        ResourceTaskAssignment returnedAssignment = (ResourceTaskAssignment) result
                .getRpmObject();
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
                scope);

        //reload the assignment and compare it
        returnedAssignment = load(returnedAssignment, scope, loadFromId);
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
                scope);
        return returnedAssignment;
    }

    protected void deleteAssignment(ResourceTaskAssignment assignment,
            boolean loadFromId) throws Exception
    {
        //delete the element
        delete(assignment);

        //reload the deleted element
        ResourceTaskAssignment deletedAssignment = loadDeletedAssignment(
                assignment, loadFromId);

        //validate deleted element
        assertNull(deletedAssignment);
    }

    protected ResourceTaskAssignment loadDeletedAssignment(
            ResourceTaskAssignment assignment, boolean loadFromId)
            throws Exception
    {
        try
        {
            assignment = load(assignment, null, loadFromId);
            return assignment;
        }
        catch (Throwable e)
        {
            //element does not exist
        }
        return null;
    }

    protected ResourceTaskAssignment load(ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, boolean loadFromId)
            throws Exception
    {
        if (loadFromId)
        {
            //load from an id
            RPMObject obj = new ResourceTaskAssignment();
            obj.setID(assignment.getID());
            return (ResourceTaskAssignment) loadFromID(obj, scope);
        }
        fail("Assignments cannot be loaded directly from XPath. They must be loaded via their parents.");
        return null;
    }
}
