package com.ibm.rpm.client.junit.wbs.testcases;

/**
 * @author vnarayan
 */
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestDeliverable extends AbstractTestWorkElement 
{

	private WorkElementScope mainScope = null;
	private Portfolio rootTestPortfolio = null;
	
    private static final int DELTA_DELIVERABLES_BY_LEVEL = 1;
    private static final int DELTA_ELEMENTS_BY_LEVEL = 1;

	public TestDeliverable (String testName) {
		setName(testName);
	}

	public void setUp() throws Exception {
		logger.info("setUp");
		super.setUp();
		mainScope = WorkElementFixture.createScope(new Deliverable(), true,
            true, false, false, false, false, false, false, false, false,
            false, false, false, false,false);
		PortfolioScope rootPortfolioScope = new PortfolioScope();
		rootPortfolioScope.setProjects(mainScope);
		mainScope.setPortfolio(rootPortfolioScope);
		rootTestPortfolio = PortfolioFixture.loadOrCreateRootPortfolio(this);
	}

	public void tearDown() throws Exception {

		super.tearDown();
	}

	public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,Exception
	{
	    insertProposalAndDeliverable(null, false);
	}

	public void testInsertWithNoScopeLoadFromID() throws AssertionError,Exception
	{
	    insertProposalAndDeliverable(null, true);
	}

	public void testInsertWithScopeLoadFromID() throws AssertionError,Exception
	{
	    insertProposalAndDeliverable(mainScope, true);
	}

	public void testUpdateWithNoScopeLoadFromID() throws AssertionError,Exception
	{
		update(null, true);
	}

	public void testUpdateWithScopeLoadFromID() throws AssertionError,Exception
	{
		update(mainScope, true);
	}

	private Proposal insertProposalAndDeliverable(WorkElementScope scope,
			boolean loadFromId) throws Exception

	{
		//create a proposal
		Proposal proposal = (Proposal)WorkElementFixture.createFixture(new Proposal(),
		        null, this);
		//set rootPortfolio as parent
		proposal.setPortfolio(rootTestPortfolio);
		//create deliverable
		Deliverable deliverable = (Deliverable)WorkElementFixture.createFixture(
            new Deliverable(), scope, this);
		deliverable.setParent(proposal);
		//set deliverable as child of proposal and save.
		WorkElement[] workElements = new WorkElement[]{deliverable};
		proposal.setChildren(workElements);
		WorkElementScope proposalScope = new WorkElementScope();
		if(scope == null) scope = new WorkElementScope();//if no scope passed!
		proposalScope.setChildren(scope);
		SingleResult result = save(proposal, proposalScope);
		//compare the returned proposal
		Deliverable returnedDeliverable = (Deliverable)((Proposal) result.getRpmObject()).getChildren()[0];
		WorkElementFixture.compare(deliverable, returnedDeliverable, scope);
		//reload the Deliverable and compare it
		returnedDeliverable = (Deliverable) load(returnedDeliverable, scope, loadFromId);
		WorkElementFixture.compare(deliverable, returnedDeliverable, scope);
        assertNotNull(returnedDeliverable.getReferenceID()); // read-only - always populated by proc
		//return the Proposal
		return proposal;
	}

	private void update(WorkElementScope scope,
			boolean loadFromId) throws AssertionError, Exception
	{
		//insert a Proposal and Deliverable
		Proposal proposal = insertProposalAndDeliverable(scope, loadFromId);
		Deliverable deliverable = (Deliverable)proposal.getChildren()[0];
		//update
		deliverable = (Deliverable) updateWorkElementFields(deliverable, scope,
            loadFromId, DELTA_DELIVERABLES_BY_LEVEL, DELTA_ELEMENTS_BY_LEVEL);
        deliverable = (Deliverable) updateAggregateNodeStructure(deliverable,
            scope, loadFromId, DELTA_DELIVERABLES_BY_LEVEL,
            DELTA_ELEMENTS_BY_LEVEL);
	}
}


