/*
 * Created on 8-May-2006
 * 
 * Description: Test methods for creating generic projects with templates.
 */
package com.ibm.rpm.client.junit.wbs.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;


public class TestGenericProjectWithTemplate extends ClientTestController
{
    private WorkElementScope scope = null;

    public void setUp() throws Exception
    {
        super.setUp();
        scope = new WorkElementScope();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
 

    public void testCreateDeliverableWithTemplate() throws AssertionError,
			Exception {
    	//creating parent
		Proposal proposalParent = new Proposal();
		proposalParent.setName("WS-C#-ATC1-Proposal");

		//saving parent
		SingleResult saveResult = getApplicationInterface().save(sessionid,proposalParent, null,ReloadType.ReloadResult);
		proposalParent = (Proposal)saveResult.getRpmObject();

		Deliverable template = null;

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Deliverable[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Deliverable);
		template = (Deliverable) rpmObjects[0];
		
		//creating new deliverable
		Deliverable inputDev = new Deliverable();
		inputDev.setName("WS-C#-InputDeliverable");
		inputDev.setPublished(ProjectPublishedType.None);
		inputDev.setParent(proposalParent);

		//creating template from new deliverable
		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, inputDev, template,
						null);

		//verification
		assertNotNull(result);
		assertTrue(result.getRpmObject() instanceof Deliverable);
		assertEquals(result.getRpmObject().getID(), inputDev.getID());
		assertTrue(result.isSuccessful());
	}

    public void testCreateProjectTemplate() throws AssertionError,
			Exception {
		//creating parent
		Proposal proposalParent = new Proposal();
		proposalParent.setName("WS-TEST-Proposal");

		//saving parent
		SingleResult saveResult = getApplicationInterface().save(sessionid,
				proposalParent, null, ReloadType.ReloadResult);
		proposalParent = (Proposal) saveResult.getRpmObject();

		GenericProject template = null;

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Project[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Project);
		template = (Project) rpmObjects[0];

		//creating new deliverable
		Project inputDev = new Project();
		inputDev.setName("WS-Input-Project");
		inputDev.setPublished(ProjectPublishedType.None);
		inputDev.setParent(proposalParent);

		//creating template from new deliverable
		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, inputDev, template, null);

		//verification
		assertNotNull(result);
		assertTrue(result.getRpmObject() instanceof Project);
		assertEquals(result.getRpmObject().getID(), inputDev.getID());
		assertTrue(result.isSuccessful());
	}

    public void testCreateProjectWithTemplate() throws AssertionError,
			Exception {
		GenericProject template = null;

		GenericProject projectParent = (GenericProject) WorkElementFixture
				.createSaveFixture(null, new Project(), scope, this);

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Project[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Project);

		template = (Project) rpmObjects[0];

		Project newProject = new Project();
		newProject.setParent(projectParent);
		newProject.setPublished(ProjectPublishedType.None);


		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newProject, template,
						scope);

		assertNotNull(result);

		assertTrue(result.getRpmObject() instanceof Project);
		assertEquals(result.getRpmObject().getID(), newProject.getID());
		assertTrue(result.isSuccessful());
	}

	public void testCreateProposalWithTemplate() throws AssertionError,
			Exception {
		GenericProject template = null;

		GenericProject proposalParent = (GenericProject) WorkElementFixture
				.createSaveFixture(null, new Proposal(), scope, this);

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Proposal[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Proposal);

		template = (Proposal) rpmObjects[0];
		
		Proposal newProposal = new Proposal();
		newProposal.setParent(proposalParent);


		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newProposal, template,
						scope);

		assertNotNull(result);

		assertTrue(result.getRpmObject() instanceof Proposal);
		assertEquals(result.getRpmObject().getID(), newProposal.getID());
		assertTrue(result.isSuccessful());
	}

	public void testCreateWorkOrganizationWithTemplate() throws AssertionError,
			Exception {
		GenericProject template = null;

		GenericProject woParent = (GenericProject) WorkElementFixture
				.createSaveFixture(null, new WorkOrganization(), scope, this);

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof WorkOrganization);

		template = (WorkOrganization) rpmObjects[0];
		
		WorkOrganization newWO = new WorkOrganization();
		newWO.setParent(woParent);


		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newWO, template,
						scope);

		assertNotNull(result);

		assertTrue(result.getRpmObject() instanceof WorkOrganization);
		assertEquals(result.getRpmObject().getID(), newWO.getID());
		assertTrue(result.isSuccessful());
	}

	public void testCreateDeliverableProject() throws AssertionError, Exception {
		WorkElement template = null;

		WorkElement deliverableParent = WorkElementFixture.
				createSaveFixture(null, new Deliverable(), scope, this);

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Deliverable[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);

		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Deliverable);

		template = (Deliverable) rpmObjects[0];
		
		Deliverable newDeliverable = new Deliverable();
		newDeliverable.setParent(deliverableParent);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newDeliverable, template,
						scope);

		assertNotNull(result);
		assertTrue(result.isSuccessful());
	}

	public void testCreateSummaryTaskProject() throws AssertionError, Exception {
		WorkElement template = null;

		WorkElement summaryTaskParent = WorkElementFixture
				.createSaveFixture(null, new SummaryTask(), scope, this);

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof SummaryTask);

		template = (SummaryTask) rpmObjects[0];
		
		SummaryTask newSummaryTask = new SummaryTask();
		newSummaryTask.setParent(summaryTaskParent);


		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newSummaryTask, template,
						scope);

		assertNotNull(result);
		assertTrue(result.isSuccessful());
	}

	public void testCreateWorkProductProject() throws AssertionError, Exception {
		WorkElement template = null;

		WorkElement workProductParent = WorkElementFixture
				.createSaveFixture(null, new WorkProduct(), scope, this);

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/WorkProduct[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof WorkProduct);

		template = (WorkProduct) rpmObjects[0];
		
		WorkProduct newWorkProduct = new WorkProduct();
		newWorkProduct.setParent(workProductParent);


		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newWorkProduct, template,
						scope);

		assertNotNull(result);
		assertTrue(result.isSuccessful());
	}

	public void testCreateTaskProject() throws AssertionError, Exception {
		WorkElement template = null;

		WorkElement summaryTaskParent = WorkElementFixture
				.createSaveFixture(null, new SummaryTask(), scope, this);
		
		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Task[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Task);

		template = (Task) rpmObjects[0];
		
		Task newTask = new Task();
		newTask.setParent(summaryTaskParent);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newTask, template,
						scope);

		assertNotNull(result);

		assertTrue(result.isSuccessful());
	}

	public void testCreateMilestoneProject() throws AssertionError, Exception {
		WorkElement template = null;

		WorkElement deliverableParent = WorkElementFixture
				.createSaveFixture(null, new Deliverable(), scope, this);

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Milestone[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Milestone);

		template = (Milestone) rpmObjects[0];
		
		Milestone newMilestone = new Milestone();
		newMilestone.setParent(deliverableParent);


		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, newMilestone, template,
						scope);

		assertNotNull(result);

		assertTrue(result.isSuccessful());
	}	
}
