package com.ibm.rpm.client.junit.wbs.testcases;

/**
 * @author vnarayan
 */
import java.util.GregorianCalendar;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.applicationadministration.containers.OrganizationType;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.PublishedScorecardFixture;
import com.ibm.rpm.client.junit.scorecard.fixtures.ScorecardFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.DependencyFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WbsModuleFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.ProjectOrganizationalAssignment;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.PortfolioScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WbsModuleScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;

public class TestProposal extends ClientTestController
{
    private WorkElementScope mainScope = null;

    private Portfolio rootTestPortfolio = null;

    private static final int DELTA_PROPOSALS_BY_LEVEL = 1;

    private static final int DELTA_ELEMENTS_BY_LEVEL = 1;

    public TestProposal(String testName)
    {
        setName(testName);
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        mainScope = WorkElementFixture.createScope(new Proposal(), true, true,
            false, false, true, false, false, false, false, false, false,
            false, false, true, false);

        PortfolioScope rootPortfolioScope = new PortfolioScope();
        rootPortfolioScope.setProjects(mainScope);
        mainScope.setPortfolio(rootPortfolioScope);
        rootTestPortfolio = PortfolioFixture.loadOrCreateRootPortfolio(this);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testCreateOrganizationalAssignments() throws Exception
    {
        // TODO: Integrate this test in the normal test suite.
        // Good luck with the (static) fixture.

        // Create the proposal with one organizational assignment.
        Organizational organizational = OrganizationalFixture
            .createSaveFixture(0, this);
        OrganizationType organizationType = DatafieldFixture
            .createOrganizationType(DatafieldFixture.ORGANIZATION_TYPE_OWNING);
        ProjectOrganizationalAssignment organizationalAssignment = new ProjectOrganizationalAssignment();
        organizationalAssignment.setOrganizational(organizational);
        organizationalAssignment.setOrganizationType(organizationType);
        ProjectOrganizationalAssignment[] organizationalAssignments =
        {
            organizationalAssignment
        };
        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal.setPortfolio(rootTestPortfolio);
        proposal.setOrganizationalAssignments(organizationalAssignments);

        // Create the matching scope.
        WorkElementScope proposalScope = new WorkElementScope();
        proposalScope.setOrganizationalAssignments(true);

        // Save and reload the proposal.
        Proposal savedProposal = (Proposal) save(proposal, proposalScope)
            .getRpmObject();
        assertNotNull(savedProposal);
        ProjectOrganizationalAssignment[] savedOrganizationalAssignments = savedProposal
            .getOrganizationalAssignments();
        assertNotNull(savedOrganizationalAssignments);
        assertEquals(savedOrganizationalAssignments.length, 1);
        ProjectOrganizationalAssignment savedOrganizationalAssignment = savedOrganizationalAssignments[0];
        assertNotNull(savedOrganizationalAssignment);
        Organizational savedOrganizational = savedOrganizationalAssignment
            .getOrganizational();
        assertNotNull(savedOrganizational);
        OrganizationType savedOrganizationType = savedOrganizationalAssignment
            .getOrganizationType();
        assertNotNull(savedOrganizationType);

        // Assert organizational are equal.
        OrganizationalFixture
            .compare(organizational, savedOrganizational, null);
        DatafieldFixture.compare(organizationType, savedOrganizationType);
    }

    public void testInsertWithNoScopeLoadFromXPath() throws AssertionError,
            Exception
    {
        insertProposal(null, false);
    }

    public void testInsertWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        insertProposal(null, true);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        insertProposal(mainScope, true);
    }
    
    public void testInsertWithoutRtfAssignment() throws AssertionError, Exception
    {
        WorkElementScope proposalScope = new WorkElementScope();
        proposalScope.setRtfAssignments(true);
        
        insertProposal(proposalScope, true);
    }

    public void testUpdateWithNoScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(null, true);
    }

    public void testUpdateWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        update(mainScope, true);
    }

    public void testInsertWithScorecardAssignment() throws AssertionError,
            Exception
    {
        Proposal proposal = prepareScorecardAssignment();
        Proposal loaded = (Proposal) loadFromID(proposal, mainScope);
        WorkElementFixture.compare(proposal, loaded, mainScope);
        PublishedScorecardFixture.deleteScorecard(loaded, this);

    }

    public void testUpdateWithScorecardAssignment() throws AssertionError,
            Exception
    {
        Proposal proposal = prepareScorecardAssignment();
        for (int i = 0; i < proposal.getAssignedScorecards().length; i++)
            PublishedScorecardFixture.updateAssignment(1, proposal
                .getAssignedScorecards()[i]);
        save(proposal, mainScope, ReloadType.ReloadResult);
        Proposal loaded = (Proposal) loadFromID(proposal, mainScope);
        WorkElementFixture.compare(proposal, loaded, mainScope);
        PublishedScorecardFixture.deleteScorecard(loaded, this);

    }

    public void testInsertWithScorecardXpathCondition1() throws AssertionError,
            Exception
    {
        Proposal proposal = prepareScorecardAssignment();
        RPMObject[] rpmObjects = loadFromXpath(
            "/Proposal[/AssignedScorecard[Id='"
                    + proposal.getAssignedScorecards()[0].getID() + "']]",
            mainScope);
        Assert.assertEquals(rpmObjects.length, 1);

        WorkElementFixture.compare(proposal, (Proposal) rpmObjects[0],
            mainScope);

        PublishedScorecardFixture.deleteScorecard(proposal, this);

    }

    public void testInsertWithScorecardXpathCondition2() throws AssertionError,
            Exception
    {
        Proposal proposal = prepareScorecardAssignment();

        RPMObject[] rpmObjects = loadFromXpath(
            "/Proposal[/AssignedScorecard[/AssignedScorecardCategory[Id='"
                    + proposal.getAssignedScorecards()[0]
                        .getAssignedScorecardCategories()[0].getID() + "']]]",
            mainScope);
        Assert.assertEquals(rpmObjects.length, 1);

        WorkElementFixture.compare(proposal, (Proposal) rpmObjects[0],
            mainScope);

        PublishedScorecardFixture.deleteScorecard(proposal, this);

    }

    public void testPulishScorecard() throws AssertionError, Exception
    {
        Proposal assignedProposal = prepareScorecardAssignment();

        for (int i = 0; i < assignedProposal.getAssignedScorecards().length; i++)
        {
            AssignedScorecard publishedSC = publishScorecard(assignedProposal
                .getAssignedScorecards()[i], PublishedScorecardFixture
                .createScorecardScope(mainScope));
            PublishedScorecardFixture.comparePublishedScorecard(publishedSC);
        }

        Proposal loaded = (Proposal) loadFromID(assignedProposal, mainScope);

        for (int i = 0; i < assignedProposal.getAssignedScorecards().length; i++)
        {
            PublishedScorecardFixture.comparePublishedScorecard(loaded
                .getAssignedScorecards()[i]);
        }

    }

    public void testCommitAffectedResources() throws AssertionError, Exception
    {
        Proposal assignedProposal = prepareAssignedProposal();

        Proposal commitedProposal = (Proposal) getApplicationInterface()
            .commitAffectedResources(sessionid, assignedProposal, mainScope)
            .getRpmObject();

        // reload and compare
        Proposal loadedProposal = (Proposal) loadFromID(assignedProposal,
            mainScope);
        WorkElementFixture.compare(commitedProposal, loadedProposal, mainScope);
    }

    public void testCopyProposedToPlan() throws AssertionError, Exception
    {
        Proposal assignedProposal = prepareAssignedProposal();

        GenericProject plannedProject = (GenericProject) getApplicationInterface()
            .copyProposedToPlan(sessionid, assignedProposal, mainScope)
            .getRpmObject();

        // reload and compare
        GenericProject loadedProject = (GenericProject) loadFromID(
            assignedProposal, mainScope);
        WorkElementFixture.compare(plannedProject, loadedProject, mainScope);
    }

    public void testDependenciesWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        Proposal proposal = insertProposal(mainScope, true);
        updateDependencies(mainScope, proposal);
    }

    private Proposal prepareScorecardAssignment() throws Exception,
            AssertionError, RPMException, AssertionFailedError
    {
        mainScope.setAssignedScorecards(PublishedScorecardFixture
            .createScorecardScope(mainScope));

        Proposal returnedProposal = insertProposal(mainScope, true);

        WbsModuleScope moduleScope = new WbsModuleScope();
        moduleScope.setScorecardFolders(ScorecardFixture.createScope());

        WbsModule module = WbsModuleFixture.createFixture(moduleScope, this);
        module = (WbsModule) save(module, moduleScope).getRpmObject();

        ScorecardFolder scorecardFolder = module.getScorecardFolders()[0];

        Proposal assignedProposal = (Proposal) PublishedScorecardFixture
            .createAssignmentFixture(returnedProposal, mainScope, this,
                ReloadType.ReloadResult, scorecardFolder, 1);

        return assignedProposal;
    }

    private Proposal prepareAssignedProposal() throws Exception,
            AssertionError, RPMException, AssertionFailedError
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setPortfolio(new PortfolioScope());
        scope.setParent(new WorkElementScope());
        scope.setContainingProject(new WorkElementScope());
        Proposal proposal = insertProposal(scope, true);

        Resource resource = ResourceFixture.createSaveFixture(this);
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignmentFixture.createFixture(proposal, resource,
            taskAssignmentScope, this);

        return proposal;
    }

    private Proposal insertProposal(WorkElementScope scope, boolean loadFromId)
            throws Exception
    {
        //create and save a workelement
        Proposal workelement = (Proposal) WorkElementFixture.createFixture(
            new Proposal(), scope, this);

        //set attribute rootPortfolio 
        workelement.setPortfolio(rootTestPortfolio);

        SingleResult result = save(workelement, scope);

        //compare the returned workelement
        Proposal returnedProposal = (Proposal) result.getRpmObject();
        WorkElementFixture.compare(workelement, returnedProposal, scope);

        //reload the WorkElement and compare it
        returnedProposal = load(returnedProposal, scope, loadFromId);
        WorkElementFixture.compare(workelement, returnedProposal, scope);

        return returnedProposal;
    }

    private void update(WorkElementScope scope, boolean loadFromId)
            throws AssertionError, Exception
    {
        //insert
        Proposal proposal = insertProposal(scope, loadFromId);
        //update
        proposal = updateWorkElementFields(proposal, scope, loadFromId);

        proposal = updateWorkElementStructure(proposal, scope, loadFromId);
    }

    private Proposal updateWorkElementFields(Proposal proposal,
            WorkElementScope scope, boolean loadFromId) throws Exception
    {
        return updateWorkElement(proposal, scope, loadFromId, false);
    }

    private Proposal updateWorkElementStructure(Proposal proposal,
            WorkElementScope scope, boolean loadFromId) throws Exception
    {
        return updateWorkElement(proposal, scope, loadFromId, true);
    }

    private Proposal updateWorkElement(Proposal proposal,
            WorkElementScope scope, boolean loadFromId, boolean updateStructure)
            throws Exception
    {
        if (updateStructure)
        {//updates the structure by adding elements
            WorkElementFixture.updateFixtureStructure(proposal,
                DELTA_PROPOSALS_BY_LEVEL, DELTA_ELEMENTS_BY_LEVEL, scope, this);
        }
        else
        {//only updates the fields
            WorkElementFixture.updateFixtureFields(proposal, scope, this);
        }
        //Insert the updated workelement into the DB
        SingleResult result = save(proposal, scope);

        //compare the returned workelement
        Proposal returnedProposal = (Proposal) result.getRpmObject();
        WorkElementFixture.compare(proposal, returnedProposal, scope);

        //reload the workelement and compare it
        returnedProposal = load(returnedProposal, scope, loadFromId);
        WorkElementFixture.compare(proposal, returnedProposal, scope);

        //return the returnedProposal
        return returnedProposal;
    }

    public void testCodeSampleTransferUnderWBS() throws Exception
    {
        //Creating and saving a caldendar
        RPMCalendar calendar = new RPMCalendar();
        calendar.setName("Normal");
        calendar.setHoursPerDay(new Double(8));
        calendar.setHoursPerWeek(new Double(40));
        calendar = (RPMCalendar) save(calendar, null).getRpmObject();

        //Creating scope element
        WorkElementScope workElementScope = new WorkElementScope();
        PortfolioScope portfolioScope = new PortfolioScope();
        portfolioScope.setProjects(workElementScope);
        workElementScope.setPortfolio(portfolioScope);
        ScopeElementScope scopeElementScope = new ScopeElementScope();
        scopeElementScope.setChildren(scopeElementScope);
        workElementScope.setScopeElement(scopeElementScope);

        //Creating WBS objects and saving them
        Portfolio portfolio = new Portfolio();
        portfolio.setName("Portfolio Sample");

        Proposal proposal = new Proposal();
        proposal.setName("Proposal Sample");
        proposal.setPriority(new Integer(500));
        proposal.setEstimatedStartDate(new GregorianCalendar());
        proposal.setCalendar(calendar);
        proposal.setAssignmentType(AssignmentType.Work);
        proposal.setPortfolio(portfolio);

        portfolio.setProjects(new GenericProject[]
        {
            proposal
        });

        portfolio = (Portfolio) save(portfolio, portfolioScope).getRpmObject();

        proposal = (Proposal) portfolio.getProjects()[0];
        ScopeFolder folder = (ScopeFolder) proposal.getScopeElement();

        /////////////////////////////
        //This is a hack because part of workElement and scope are
        //still under development
        //        scopeElementScope.setWorkElement(workElementScope);
        //        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
        //                + proposal.getName() + "']", new ScopeElementScope());
        //        ScopeFolder folder = (ScopeFolder) rpmObjects[0];
        //        proposal.setScopeElement(folder);
        //        folder.setWorkElement(proposal);
        //////////////////////////////

        //Creating a defect under the Scope folder element for the proposal
        Defect defect = new Defect();
        defect.setName("Defect Sample");
        folder.setChildren(new ScopeElement[]
        {
            defect
        });

        folder = (ScopeFolder) save(folder, scopeElementScope).getRpmObject();
        defect = (Defect) folder.getChildren()[0];

        getApplicationInterface()
            .transferUnderWBS(getSessionID(), null, defect);

        return;
    }

    private void updateDependencies(WorkElementScope scope, Proposal proposal)
            throws Exception
    {
        while (scope != null)
        {
            scope.setDependencies(DependencyFixture.createScope());
            if (scope.getChildren() != null)
            {
                scope.getChildren().setDependencies(DependencyFixture.createScope());
            }
            scope = scope.getContainingProject();
        }
        
        proposal = (Proposal) WorkElementFixture.setWorkElementForDependencies(
            proposal, scope);
        SingleResult result = save(proposal, scope);
        WorkElement returnedProposal = (WorkElement) result.getRpmObject();
        //        WorkElementFixture.compare(proposal, returnedProposal, scope);
        returnedProposal = load((Proposal) returnedProposal, scope, true);
        WorkElementFixture.compare(proposal, returnedProposal, scope);

    }

    private Proposal load(Proposal Proposal, WorkElementScope scope,
            boolean loadFromId) throws Exception
    {
        if (loadFromId)
        {
            //load from an id
            RPMObject obj = new Proposal();
            obj.setID(Proposal.getID());
            return (Proposal) loadFromID(obj, scope);
        }
        //load from an xpath
        RPMObject[] objs = loadFromXpath("/PROPOSAL[id='" + Proposal.getID()
                + "']", scope);
        if (objs == null || objs.length == 0)
        {
            return null;
        }
        return (Proposal) objs[0];
    }
}
