/*
 * rbadette
 * Created on 27-Sep-2006
 *
 * This class test that an xpath query with the publishedType.ToPartialTemplate
 * does not yel an error.
 * 
 * ToPartialTemplate has been added as a backward compatibility flag.
 *
 */
package com.ibm.rpm.client.junit.wbs.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;


public class TestPublishedType extends ClientTestController 
{
    private WorkElementScope scope = null;

    public void setUp() throws Exception
    {
        super.setUp();
        scope = new WorkElementScope();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testLoadingProposalWithPartialPublished() throws AssertionError,
			Exception {

		//Loading template from Xpath: published='ToTemplate'
		ArrayResult result = getApplicationInterface().loadFromXpath(sessionid,"/Proposal[published='"
				+ ProjectPublishedType._ToPartialTemplate + "']",scope);
		//verification
		assertNotNull(result);
		if(result.getRpmObjectList()!= null && result.getRpmObjectList().length>0)
		{
			for(int i=0; i<result.getRpmObjectList().length; i++)
			{				
				assertTrue(result.getRpmObjectList()[i] instanceof Proposal);
				assertTrue((ProjectPublishedType.ToPartialTemplate.getValue()
						.equals(((Proposal) result.getRpmObjectList()[i])
								.getPublished().getValue())));
			}
		}
		assertTrue(result.isSuccessful());
	}
    
    public void testSavingProposalWithPartialPublished() throws AssertionError,
			Exception {
		//creating parent
		Proposal proposalParent = new Proposal();
		proposalParent.setName("WBS-PartialPublishedToTemplate");
		proposalParent.setPublished(ProjectPublishedType.ToPartialTemplate);

		//saving parent
		SingleResult saveResult = getApplicationInterface().save(sessionid,
				proposalParent, null, ReloadType.ReloadResult);

		//verification
		assertNotNull(saveResult);
		assertNotNull(saveResult.getRpmObject());
		assertTrue(saveResult.getRpmObject() instanceof WorkElement);
		WorkElementFixture.compare((WorkElement)proposalParent,(WorkElement)saveResult.getRpmObject(),null);
		assertTrue(saveResult.isSuccessful());
	}
    
    public void testLoadingDeliverableWithPartialPublished()
			throws AssertionError, Exception {

		//Loading template from Xpath: published='ToTemplate'
		ArrayResult result = getApplicationInterface().loadFromXpath(
				sessionid,
				"/Deliverable[published='" + ProjectPublishedType._ToPartialTemplate
						+ "']", scope);
		//verification
		assertNotNull(result);
		
		if(result.getRpmObjectList()!= null && result.getRpmObjectList().length>0)
		{
			for(int i=0; i<result.getRpmObjectList().length; i++)
			{				
				assertTrue(result.getRpmObjectList()[i] instanceof Deliverable);
				assertTrue((ProjectPublishedType.ToPartialTemplate.getValue()
						.equals(((Deliverable) result.getRpmObjectList()[i])
								.getPublished().getValue())));
			}
		}

		assertTrue(result.isSuccessful());
	}

	public void testSavingDeliverableWithPartialPublished() throws AssertionError,
			Exception {
		
		//creating parent
		Proposal proposalParent = new Proposal();
		proposalParent.setName("WS-TEST-Proposal");

		//saving parent
		SingleResult saveResult = getApplicationInterface().save(sessionid,
				proposalParent, null, ReloadType.ReloadResult);
		proposalParent = (Proposal) saveResult.getRpmObject();

		//creating Deliverable
		Deliverable deliverable = new Deliverable();
		deliverable.setName("WBS-PartialPublishedToTemplate");
		deliverable.setPublished(ProjectPublishedType.ToPartialTemplate);
		deliverable.setParent(proposalParent);

		//saving Deliverable
		SingleResult saveResultDeliverable = getApplicationInterface().save(sessionid,
				deliverable, null, ReloadType.ReloadResult);

		//verification
		assertNotNull(saveResultDeliverable);
		
		assertNotNull(saveResultDeliverable.getRpmObject());
		assertTrue(saveResultDeliverable.getRpmObject() instanceof Deliverable);
		WorkElementFixture.compare((WorkElement)deliverable,(WorkElement)saveResultDeliverable.getRpmObject(),null);

		assertTrue(saveResultDeliverable.isSuccessful());
	}


}
