package com.ibm.rpm.client.junit.wbs.testcases;

import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.SecurityRoleFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceAttributeAssignmentFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ProfileScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ResourceType;

/**
 * 
 * @author ikusakov
 */
public class TestResourceTaskAssignmentProfileAffect extends
        ClientTestController

{
    protected ResourceTaskAssignmentScope mainScope = null;

    protected RPMObject mainParent = null;

    public TestResourceTaskAssignmentProfileAffect(String testName)
    {
        setName(testName);
    }

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();
        mainScope = ResourceTaskAssignmentFixture.createScope(true, true, true,
            false, true);
    }

    public void tearDown() throws Exception
    {
        logger.info("tearDown");
        super.tearDown();
    }

    public void testLoadFromWbsPerspective() throws Exception
    {
        ResourceTaskAssignment assignment = null;
        assignment = ResourceTaskAssignmentFixture.createFixture(mainScope,
            false, this);

        WorkElement parent = (WorkElement) assignment.getParent();

        ResourceTaskAssignment saved = (ResourceTaskAssignment) save(
            assignment, mainScope).getRpmObject();

        checkForProperAssignment(parent, saved, mainScope);
    }

    public void testDeleteFromWbsPerspective() throws Exception
    {
        ResourceTaskAssignment assignment = null;
        assignment = ResourceTaskAssignmentFixture.createFixture(mainScope,
            false, this);

        WorkElement parent = (WorkElement) assignment.getParent();

        save(assignment, mainScope).getRpmObject();

        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setResourceTaskAssignments(mainScope);
        parent = (WorkElement) loadFromID(parent, workElementScope);
        assertNotNull(parent.getResourceTaskAssignments());
        assertTrue("ResourceTaskAssignments array length", parent
            .getResourceTaskAssignments().length > 0);

        ResourceTaskAssignment newAssignment = null;
        for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
        {
            if (parent.getResourceTaskAssignments()[i].getID().equals(
                assignment.getID()))
            {
                newAssignment = parent.getResourceTaskAssignments()[i];
                break;
            }
        }
        assertNotNull("Proper ResourceTaskAssignment was not loaded",
            newAssignment);
        newAssignment.setDeleted(Boolean.TRUE);

        parent = (WorkElement) save(parent, workElementScope).getRpmObject();
        parent = (WorkElement) loadFromID(parent, workElementScope);
        boolean found = false;
        if (parent.getResourceTaskAssignments() != null
                && parent.getResourceTaskAssignments().length > 0)
        {
            for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
            {
                if (parent.getResourceTaskAssignments()[i].getID().equals(
                    assignment.getID()))
                {
                    found = true;
                    break;
                }
            }
        }
        assertFalse("ResourceTaskAssignment should be deleted", found);
    }

    private void checkForProperAssignment(WorkElement parent,
            ResourceTaskAssignment assignmentToCompare,
            ResourceTaskAssignmentScope scope) throws Exception
    {
        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setResourceTaskAssignments(scope);
        parent = (WorkElement) loadFromID(parent, workElementScope);

        assertNotNull(parent.getResourceTaskAssignments());
        assertTrue("ResourceTaskAssignments array is empty", parent
            .getResourceTaskAssignments().length > 0);
        boolean found = false;
        for (int i = 0; i < parent.getResourceTaskAssignments().length; i++)
        {
            if (parent.getResourceTaskAssignments()[i].getID().equals(
                assignmentToCompare.getID()))
            {
                ResourceTaskAssignmentFixture.compare(assignmentToCompare,
                    parent.getResourceTaskAssignments()[i], scope);
                found = true;
                break;
            }
        }

        assertTrue("Proper ResourceTaskAssignment was not loaded", found);
    }

    public void testAssignProfileToDifferentTasks() throws AssertionError,
            Exception
    {
        ResourceTaskAssignment assignment = insertAssignment(mainScope, true);

        ResourceTaskAssignmentScope reScope = ResourceTaskAssignmentFixture
            .createScope(true, true, true, false, true);

        ResourceTaskAssignment reAss = ResourceTaskAssignmentFixture
            .createFixture(mainScope, false, this);

        reAss.setProfile(assignment.getProfile());

        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
        scope.setParent(scope);
        scope.setContainingProject(new WorkElementScope());
        Task t = new Task();
        t.setName("reAssignProfile");
        WbsScheduleDate scheduleDate = new WbsScheduleDate();
        Calendar calendar = Calendar.getInstance();
        //        calendar.set(2006, 3, 11);
        scheduleDate.setDurationInMinutes(new Integer(1440));
        scheduleDate.setStartDate(calendar);
        t.setContainingProject(WorkElementFixture.retrieveStaticFixture(
            new Project(), this));
        t.setProposedDate(scheduleDate);
        t.setParent(mainParent.getParent());
        t = (Task) save(t, scope).getRpmObject();

        reAss.setParent(t);
        reAss.getTaskAssignment().setBillable(Boolean.FALSE);
        save(reAss, reScope);

        deleteAssignment(assignment);
        deleteAssignment(reAss);
    }

    public void testInsertWithScopeLoadFromID() throws AssertionError,
            Exception
    {
        ResourceTaskAssignment assignment = insertAssignment(mainScope, true);
        deleteAssignment(assignment);
    }

    public void testReplaceProfileWithResource() throws AssertionError, Exception
    {
        ResourceTaskAssignment assignment = insertAssignment(mainScope, true);

        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        Resource resource = ResourceFixture.createSaveFixture(resourceScope,
            this);
        
        ResourceAttributeAssignment[] assignments = ResourceAttributeAssignmentFixture
            .createFixtureFromProfile(assignment.getProfile()
                .getProfileCompetencyAssignments());
        resource.setCompetencyAttributeAssignments(assignments);

        assignments = ResourceAttributeAssignmentFixture
        .createFixtureFromProfile(assignment.getProfile()
            .getProfileSkillAssignments());
        resource.setSkillAttributeAssignments(assignments);
        
        resourceScope.setCompetencyAttributeAssignments(true);
        resourceScope.setSkillAttributeAssignments(true);
        save(resource, resourceScope);
       
        ResourceTaskAssignment rtaAssignment = (ResourceTaskAssignment)getApplicationInterface()
            .replaceProfileWithResource(getSessionID(), assignment, mainScope, resource).getRpmObject();
        
        assertNotNull(rtaAssignment);
        assertNotNull(rtaAssignment.getResource());
        assertNull(rtaAssignment.getProfile());
    }
    
    public void testReplaceResourceWithProfile() throws AssertionError,
            Exception
    {
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        resourceScope.setCompetencyAttributeAssignments(true);
        resourceScope.setSkillAttributeAssignments(true);
 
        TaskAssignmentScope taScope = new TaskAssignmentScope();
        taScope.setParent(mainScope);
        
        mainScope.setResource(resourceScope);
        mainScope.setTaskAssignment(taScope);
        
        // Create the ResourceTaskAssignment's parent
        Task task = (Task) WorkElementFixture.retrieveStaticFixture(new Task(),
            this);
        
        Resource resource = ResourceFixture.createSaveFixture(resourceScope, this);
        
        // Create a ResourceTaskAssignment with a Resource and a TaskAssignment
        ResourceTaskAssignment assignment = new ResourceTaskAssignment();
        assignment.setParent(task);
        assignment.setSecurityRole(SecurityRoleFixture.createSecurityRole(assignment.getParent()));
        assignment.setResource(resource);
        
        TaskAssignment taskAssignment = new TaskAssignment();
        taskAssignment.setAssignmentStartDate(new GregorianCalendar());
        taskAssignment.setParent(assignment);
        
        assignment.setTaskAssignment(taskAssignment);
        
        // Save the ResourceTaskAssignment
        save(assignment, mainScope);
       
        // Set profile in the RTA scope so that the new profile gets reloaded
        mainScope.setProfile(new ProfileScope());
        ResourceTaskAssignment reloadedAssignment = (ResourceTaskAssignment) getApplicationInterface()
            .replaceResourceWithProfile(getSessionID(), assignment, mainScope)
            .getRpmObject();

        assertNotNull(reloadedAssignment);
        assertTrue(reloadedAssignment.getResourceType().equals(ResourceType.Resource) && reloadedAssignment.getResource().getID().equals("PMO_SUPERVISOR                  "));
    }
    
    public void testloadProfileInProjectFromXPath() throws Exception
    {
        ResourceTaskAssignment assignment = null;
        try
        {

            assignment = insertAssignment(mainScope, true);

            WorkElement task = (WorkElement) assignment.getParent();
            GenericProject project = (GenericProject) task
                .getContainingProject();

            RPMObject[] result = loadFromXpath("GenericProject[id = '"
                    + project.getID()
                    + "']/children/ResourceTaskAssignment/Profile", null);

            assertNotNull(result);
        }
        finally
        {
            deleteAssignment(assignment);
        }

    }

    public void testloadProfileInTaskFromXPath() throws Exception
    {
        ResourceTaskAssignment assignment = null;
        try
        {

            assignment = insertAssignment(mainScope, true);

            WorkElement task = (WorkElement) assignment.getParent();
            GenericProject project = (GenericProject) task
                .getContainingProject();

            RPMObject[] result = loadFromXpath("Task[containingProject[id = '"
                    + project.getID() + "']]/ResourceTaskAssignment/Profile",
                null);

            assertNotNull(result);
        }
        finally
        {
            deleteAssignment(assignment);
        }

    }

    private ResourceTaskAssignment insertAssignment(
            ResourceTaskAssignmentScope scope, boolean loadFromId)
            throws Exception
    {
        //create and save a workelement
        ResourceTaskAssignment assignment = null;
        assignment = ResourceTaskAssignmentFixture.createFixture(mainScope,
            false, this);

        mainParent = assignment.getParent();

        SingleResult result = save(assignment, scope);

        //compare the returned assignment
        ResourceTaskAssignment returnedAssignment = (ResourceTaskAssignment) result
            .getRpmObject();
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
            scope);

        //reload the assignment and compare it
        returnedAssignment = load(returnedAssignment, scope, loadFromId);
        ResourceTaskAssignmentFixture.compare(assignment, returnedAssignment,
            scope);
        return returnedAssignment;
    }

    private void deleteAssignment(ResourceTaskAssignment assignment) throws Exception
    {
        delete(assignment);
    }

    private ResourceTaskAssignment load(ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, boolean loadFromId)
            throws Exception
    {
        if (loadFromId)
        {
            //load from an id
            RPMObject obj = new ResourceTaskAssignment();
            obj.setID(assignment.getID());
            return (ResourceTaskAssignment) loadFromID(obj, scope);
        }
        fail("Assignments cannot be loaded directly from XPath. They must be loaded via their parents.");
        return null;
    }
}

