/*
 * Created on 15-May-2006 
 * @author mmatwiy 
 * TaskAssignment test methods.
 */
package com.ibm.rpm.client.junit.wbs.testcases;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestTaskAssignment extends ClientTestController
{
    ResourceTaskAssignment rta;

    ResourceTaskAssignmentScope rtaScope;

    public void setUp() throws Exception
    {
        super.setUp();

        SecurityRole securityRole = new SecurityRole();
        securityRole.setID("SCTASKRESPONSIBLE");

        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setContainingProject(new WorkElementScope());
        workElementScope.setParent(new WorkElementScope());

        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setParent(new PoolScope());
        resourceScope.setCalendar(new CalendarScope());

        Task parentTask = createTask(workElementScope);

        // create RTA with mandatory fields
        rta = new ResourceTaskAssignment();
        rta.setSecurityRole(securityRole);
        rta.setResource(createResource(resourceScope));

        rta.setParent(parentTask);

        rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setParent(workElementScope);
        rtaScope.setResource(resourceScope);
    }

    public void testModifyActualWork() throws Exception
    {
        ResourceTaskAssignment reloaded = copyProposedToPlan();
        reloaded.getTaskAssignment().setActualWork(new Double(32.0));

        SingleResult result = save(reloaded, rtaScope);

        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + reloaded.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        reloaded = (ResourceTaskAssignment) rpmObjects[0];

        logger.debug("actual work after reload: "
                + reloaded.getTaskAssignment().getActualWork());
        logger.debug("% work after reload: "
                + reloaded.getTaskAssignment().getPercentageWork());
        logger.debug("% complete after reload: "
                + reloaded.getTaskAssignment().getPercentageComplete());

        assertEquals(reloaded.getTaskAssignment().getActualWork(), new Double(
            32.0));
        assertEquals(reloaded.getTaskAssignment().getPercentageWork(),
            new Double(100.0));
        assertEquals(reloaded.getTaskAssignment().getPercentageComplete(),
            new Double(100.0));

    }
    
    public void testUpdateActualWorkAndEETC() throws Exception
    {
        ResourceTaskAssignment reloaded = copyProposedToPlan();
        reloaded.getTaskAssignment().setActualWork(new Double(4.0));
        reloaded.getTaskAssignment().setEETCHours(new Double(4.0));

        SingleResult result = save(reloaded, rtaScope);

        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + reloaded.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        reloaded = (ResourceTaskAssignment) rpmObjects[0];

        logger.debug("actual work after reload: "
                + reloaded.getTaskAssignment().getActualWork());
        logger.debug("% work after reload: "
                + reloaded.getTaskAssignment().getPercentageWork());
        logger.debug("% complete after reload: "
                + reloaded.getTaskAssignment().getPercentageComplete());

        assertEquals(reloaded.getTaskAssignment().getActualWork(), new Double(
            4.0));
        assertEquals(reloaded.getTaskAssignment().getEETCHours(), new Double(
            4.0));
    }


    public void testModifyPercentageComplete() throws Exception
    {
        ResourceTaskAssignment reloaded = copyProposedToPlan();

        reloaded.getTaskAssignment().setPercentageComplete(new Double(10.0));
        SingleResult result = save(reloaded, rtaScope);

        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + reloaded.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        reloaded = (ResourceTaskAssignment) rpmObjects[0];

        logger.debug("actual work after reload: "
                + reloaded.getTaskAssignment().getActualWork());
        logger.debug("%work after reload: "
                + reloaded.getTaskAssignment().getPercentageWork());
        logger.debug("%complete after reload: "
                + reloaded.getTaskAssignment().getPercentageComplete());

        assertEquals(reloaded.getTaskAssignment().getActualWork(),
            new Double(0));
        assertEquals(reloaded.getTaskAssignment().getPercentageWork(),
            new Double(0));
        assertEquals(reloaded.getTaskAssignment().getPercentageComplete(),
            new Double(10.0));

    }
    
    public void testModifyPercentageWork() throws Exception
    {
        // the percentageWork field is editable once proposed has been
        // copied to plan
        ResourceTaskAssignment reloaded = copyProposedToPlan();

        reloaded.getTaskAssignment().setPercentageWork(new Double(50.0));
        SingleResult result = save(reloaded, rtaScope);

        assertNotNull(result);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + reloaded.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        reloaded = (ResourceTaskAssignment) rpmObjects[0];

        logger.debug("actual work after reload: "
                + reloaded.getTaskAssignment().getActualWork());
        logger.debug("%work after reload: "
                + reloaded.getTaskAssignment().getPercentageWork());
        logger.debug("%complete after reload: "
                + reloaded.getTaskAssignment().getPercentageComplete());

        assertEquals(reloaded.getTaskAssignment().getActualWork(),
            new Double(4.0));
        assertEquals(reloaded.getTaskAssignment().getPercentageWork(),
            new Double(50.0));
        assertEquals(reloaded.getTaskAssignment().getPercentageComplete(),
            new Double(50.0));

    }

    private ResourceTaskAssignment copyProposedToPlan() throws Exception
    {
        TaskAssignmentScope taScope = new TaskAssignmentScope();
        rtaScope.setTaskAssignment(taScope);

        TaskAssignment taskAssignment = new TaskAssignment();
        rta.setTaskAssignment(taskAssignment);

        save(rta, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + rta.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        rta = (ResourceTaskAssignment) rpmObjects[0];
        assertNotNull(rta.getTaskAssignment().getID());

        ResourceTaskAssignment reloaded = (ResourceTaskAssignment) getApplicationInterface()
            .copyAssignmentProposedToPlan(getSessionID(), rta, rtaScope).getRpmObject();

        // now the plannedDurationHours should exist and
        // the actualWork, %work, and %complete fields should be editable
        assertNotNull(reloaded.getTaskAssignment());
        assertNotNull(reloaded.getTaskAssignment().getPlannedDurationHours());

        return reloaded;
    }

    private Resource createResource(ResourceScope resourceScope)
            throws Exception
    {
        Resource resource = new Resource();
        resource.setFullName("RTA Tester");
        resource.setParent(PoolFixture.createSaveFixture(this));
        resource.setCalendar(RPMCalendarFixture
            .retrieveStaticFixtureForInserts(this));
        save(resource, resourceScope);
        return resource;
    }

    private Task createTask(WorkElementScope workElementScope) throws Exception
    {
        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        proposal = (Proposal) save(proposal, workElementScope).getRpmObject();

        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setParent(proposal);

        task = (Task) save(task, workElementScope).getRpmObject();
        return task;
    }
    
    public void testShownInTimesheet() throws Exception
    {
        ResourceTaskAssignment reloaded = copyProposedToPlan();
        doTestShownInTimesheet(reloaded.getTaskAssignment());
    }

    private void doTestShownInTimesheet(GenericTaskAssignment taskAssignment)
            throws Exception
    {
        Boolean isShownInTimesheet = taskAssignment.getShownInTimesheet();
        assertNotNull(isShownInTimesheet);
        assertEquals(Boolean.FALSE, isShownInTimesheet);

        doTestUpdateShownInTimesheet(taskAssignment, Boolean.TRUE);
        doTestUpdateShownInTimesheet(taskAssignment, Boolean.FALSE);
    }

    private void doTestUpdateShownInTimesheet(
            GenericTaskAssignment originalTaskAssignment, Boolean value)
            throws Exception
    {
        Class originalClass = originalTaskAssignment.getClass();
        String typeName = StringUtil.getShortClassName(originalClass);
        originalTaskAssignment.setShownInTimesheet(value);
        save(originalTaskAssignment, null, ReloadType.None)
                .getRpmObject();

        GenericTaskAssignment reloadedTaskAssignment = (GenericTaskAssignment) originalClass
                .newInstance();
        reloadedTaskAssignment.setID(originalTaskAssignment.getID());
        reloadedTaskAssignment = (GenericTaskAssignment) loadFromID(
                reloadedTaskAssignment, null);
        Boolean isShownInTimesheet = reloadedTaskAssignment
                .getShownInTimesheet();
        assertNotNull(isShownInTimesheet);
        assertEquals(value, isShownInTimesheet);

        doTestLoadFromXPath(typeName, originalTaskAssignment.getID(), value);
    }

    private void doTestLoadFromXPath(String typeName, String assignmentId,
            Boolean value) throws Exception
    {
        RPMObject[] assignments = loadFromXpath("/" + typeName + "[@id='"
                + assignmentId + "' and @shownInTimesheet='" + value + "']",
                null);
        assertNotNull(assignments);
        assertEquals(1, assignments.length);

        String notValue = Boolean.toString(!value.booleanValue());
        assignments = loadFromXpath("/" + typeName + "[@id='" + assignmentId
                + "' and shownInTimesheet='" + notValue + "']", null);
        assertNull(assignments);
    }
}