package com.ibm.rpm.client.junit.wbs.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WbsScheduleDateFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;


/**
 * @author gmeloche
 *
 */
public class TestWbsScheduleDate extends ClientTestController
{

    private Task task = null;
   
    public TestWbsScheduleDate()
    {
        super();
    }


    protected void setUp() throws Exception
    {

        super.setUp();
        
        Project project = (Project)WorkElementFixture.createFixture(new Project(),null,this);
        project = (Project)save(project, null).getRpmObject();
        task = (Task)WorkElementFixture.createFixture(project, new Task(), null,this);
                        
    }
    
    
    /**
     *
     */

    protected void tearDown() throws Exception
    {

        super.tearDown();
    }
    
    
    public void testInsertTaskWithProposedScheduleDateWithNullStartDate()throws Exception{
        
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
       
        //Add proposedScheduleDate
        task = (Task)WbsScheduleDateFixture.addProposedScheduleDate(task, this);
        task.getProposedDate().setStartDate(null);
        
        insertAndCompare(task, scope);    
        
    } 
    public void testInsertTaskWithProposedScheduleDate()throws Exception{
        
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
       
        //Add proposedScheduleDate
        task = (Task)WbsScheduleDateFixture.addProposedScheduleDate(task, this);
       
        insertAndCompare(task, scope);    
        
    }  
    
    public void testInsertTaskWithPlanScheduleDate()throws Exception{
        
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
       
        //Add proposedScheduleDate
        task = (Task)WbsScheduleDateFixture.addProposedScheduleDate(task, this);
        task = (Task)save(task, scope).getRpmObject();
        
        //Add planScheduleDate
        task = (Task)WbsScheduleDateFixture.addPlanScheduleDate(task, this);
       
        insertAndCompare(task, scope);    
        
    } 
    
    public void testInsertTaskWithProposedAndPlanScheduleDate()throws Exception{
        
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
       
        //Add proposedScheduleDate
        task = (Task)WbsScheduleDateFixture.addProposedScheduleDate(task, this);
        task = (Task)save(task, scope).getRpmObject();
        
        //Add planScheduleDate
        task = (Task)WbsScheduleDateFixture.addPlanScheduleDate(task, this);
        task = (Task)save(task, scope).getRpmObject();
             
        insertAndCompare(task, scope);    
        
    } 
    
    public void testInsertTaskWithActualScheduleDateWithPartialDuration()throws Exception{
        
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
       
        //Add proposedScheduleDate
        task = (Task)WbsScheduleDateFixture.addProposedScheduleDate(task, this);
        task = (Task)save(task, scope).getRpmObject();
        
        //Add planScheduleDate        
        task = (Task)WbsScheduleDateFixture.addPlanScheduleDate(task, this);
        task = (Task)save(task, scope).getRpmObject();
        
        //Add actualScheduleDate
        task = (Task)WbsScheduleDateFixture.addActualScheduleDate(task, 75, this);
       
        insertAndCompare(task, scope);
        
    } 

    public void testInsertTaskWithActualScheduleDateWithNullDuration()throws Exception{
        
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
       
        //Add proposedScheduleDate
        task = (Task)WbsScheduleDateFixture.addProposedScheduleDate(task, this);
        task = (Task)save(task, scope).getRpmObject();
        
        //Add planScheduleDate        
        task = (Task)WbsScheduleDateFixture.addPlanScheduleDate(task, this);
        task = (Task)save(task, scope).getRpmObject();
        
        //Add actualScheduleDate
        task = (Task)WbsScheduleDateFixture.addActualScheduleDate(task, 0, this);
       
        insertAndCompare(task, scope);
        
    }
    
    public void testInsertTaskWithActualScheduleDateWithDurationCompleted()throws Exception{
        
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
       
        //Add proposedScheduleDate
        task = (Task)WbsScheduleDateFixture.addProposedScheduleDate(task, this);
        task = (Task)save(task, scope).getRpmObject();
        
        //Add planScheduleDate        
        task = (Task)WbsScheduleDateFixture.addPlanScheduleDate(task, this);
        task = (Task)save(task, scope).getRpmObject();
        
        //Add actualScheduleDate
        task = (Task)WbsScheduleDateFixture.addActualScheduleDate(task, 100, this);
      
        insertAndCompare(task, scope);
        
    }   
    
    public void testInsertTaskWithForecastDate()throws Exception{
        
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
       
        //Add proposedScheduleDate
        task = (Task)WbsScheduleDateFixture.addProposedScheduleDate(task, this);
        task = (Task)save(task, scope).getRpmObject();
        
        //Add planScheduleDate        
        task = (Task)WbsScheduleDateFixture.addPlanScheduleDate(task, this);
        task = (Task)save(task, scope).getRpmObject();
        
        //Add actualScheduleDate
        task = (Task)WbsScheduleDateFixture.addActualScheduleDate(task, 80, this);
        task = (Task)save(task, scope).getRpmObject();
        
        //Add forecastScheduleDate
        task = (Task)WbsScheduleDateFixture.addForecastScheduleDate(task,200, this);       
        task = (Task)save(task, scope).getRpmObject();
        
        insertAndCompare(task, scope);
        
    } 
    
    public void testLoadActualDateWithWorkHours()throws Exception{
        
        //Create a summaryTimesheet that will update the value of actualdate.workHours

        SummaryTimesheetScope sts = new SummaryTimesheetScope();
        sts.setWorkElement(new WorkElementScope());
        
        SummaryTimesheet summaryTimesheet = SummaryTimesheetFixture.createSaveFixture(null, null, sts, true, this);
        SingleResult result = save(summaryTimesheet, sts);
        summaryTimesheet = (SummaryTimesheet)result.getRpmObject();
        WorkElement we = summaryTimesheet.getWorkElement();
        
        WorkElementScope wes = new WorkElementScope();
        wes.setScheduleDates(true);
        
        RPMObject[] objs = loadFromXpath("Task[id = '"+ we.getID() +"']", wes);
        assertNotNull(objs);
        assertTrue(objs[0] instanceof WorkElement);
        WbsScheduleDate actualdate = ((WorkElement)objs[0]).getActualDate();
        assertNotNull(actualdate);
        assertNotNull(actualdate.getWorkHours());
        assertTrue(actualdate.getWorkHours().intValue() > 0);
        
    } 
    
    private SimpleNode load(SimpleNode object, WorkElementScope scope)
            throws Exception
    {
        object = (SimpleNode) loadFromID(object, scope);

        return object;

    }
    
    private SimpleNode insertAndCompare(SimpleNode workElement,
            WorkElementScope scope) throws Exception
    {
        //Insert the WorkElement into the DB
        SingleResult result = save(workElement, scope);

        SimpleNode returnedWorkElem = (SimpleNode) result.getRpmObject();

        //reload the WorkElement and compare it
        returnedWorkElem = load(returnedWorkElem, scope);
        WbsScheduleDateFixture.compareAllWbsScheduleDates(workElement, returnedWorkElem);

        //return the WorkElement
        return workElement;
    }

}
