/*
 * Created on Nov 29, 2006
 * @author mmatwiy
 *
 * Test method for defect 48202:
 * Update forecast schedule dates should update %DC, plan duration and plan finish date
 */
package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test48202 extends ClientTestController
{

    private static final Integer twoDaysFourHours = new Integer(1200);

    private static final Calendar today = Calendar.getInstance();

    public void test() throws Exception
    {
        // create a task with a parent proposal
        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        save(proposal, null);

        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setParent(proposal);
        save(task, null);

        // copy proposal to plan on the task
        copyProposedToPlan(task, null);

        WorkElementScope taskScope = new WorkElementScope();
        taskScope.setScheduleDates(true);

        // set task plan date to today and duration to 5 days
        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(today);
        planDate.setDurationInMinutes(new Integer(2400)); // 5 working days

        // set actual date to today and %DC to 50%
        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(today);

        task.setPlanDate(planDate);
        task.setActualDate(actualDate);
        task.setPercentageDurationCompleted(new Double(50));
        task = (Task) save(task, taskScope).getRpmObject();

        // set forecast schedule date duration to 2 days and 4 hours
        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(twoDaysFourHours);

        // save and reload
        task.setForecastDate(forecastDate);
        save(task, taskScope);

        Task reloaded = (Task) loadFromID(task, taskScope);

        // confirm that %DC is updated to 100%
        assertEquals(new Double(100), reloaded.getPercentageDurationCompleted());

        // confirm that plan and actual duration is 1200 minutes (2 days and 4
        // hours)
        assertEquals(twoDaysFourHours, reloaded.getPlanDate()
            .getDurationInMinutes());
        assertEquals(twoDaysFourHours, reloaded.getActualDate()
            .getDurationInMinutes());

        Calendar twoDaysLater = Calendar.getInstance();
        twoDaysLater.add(Calendar.DATE, 2);

        // confirm that plan and actual finish date are updated according to the
        // forecasted duration
        // we only compare the day of year, since this is the precision offered in the delphi client.
        assertEquals(twoDaysLater.get(Calendar.DAY_OF_YEAR), reloaded.getPlanDate().getFinishDate().get(Calendar.DAY_OF_YEAR));
        assertEquals(twoDaysLater.get(Calendar.DAY_OF_YEAR), reloaded.getActualDate().getFinishDate().get(Calendar.DAY_OF_YEAR));
    }
}