/*
 * Created on Nov 28, 2006
 * @author mmatwiy
 * 
 * Defect Description: When forecast duration in minutes is set without an 
 * actual start date (not legal), a read-only warning is returned but 
 * the forecast duration update still occurs.
 * 
 * Problems:
 *  - The checkpoint is supposed to reload the object in order to replace read-only
 * values. However, the cache was not being cleared prior to calling the checkpoints. 
 * - Because WbsScheduleDate containers use the same primary key as their
 * parent, a unique key does not get created when the containers are cached. This causes
 * any WbsScheduleDate container to be loaded from cache (ie. the expected date container
 * was being loaded in place of the forecast date container).
 * 
 * Resolutions:
 * - Clear the cache prior to calling the validation.
 * - Never cache WbsScheduleDate containers.
 */
package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test48204 extends ClientTestController
{
    public void test() throws Exception
    {
        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        save(proposal, null);
        
        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setParent(proposal);
        save(task, null);
                       
        copyProposedToPlan(task, null);
        
        WorkElementScope taskScope = new WorkElementScope();
        taskScope.setScheduleDates(true);
        
        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(2880));
        
        task.setForecastDate(forecastDate);
        SingleResult result = save(task, taskScope);
        assertTrue(result.isSuccessful());

        Task reloaded = (Task) loadFromID(task, taskScope);
        assertNull(reloaded.getForecastDate());       
    }
}
