/*
 * Created on 25-Oct-06
 * @author mmatwiy
 * 
 * Test case for defect 56934. Xpath comparisons on lastModifiedDate are done 
 * only on the Date part and not the time.
 * The field lastModifiedDate should enable the user to use hours-minutes-seconds 
 * in the xpath date comparison operations.
 * 
 * *** For an update on this defect, see Test56952.java.
 */
package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.util.GregorianCalendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.framework.testcases.xpath.XpathTesterUtils;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Task;

public class Test56934 extends ClientTestController
{
    public void test() throws Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // create a task and save it
        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setParent(parentProject);
        save(task, null);

        // load a task with the lastModifiedDate including the time
        RPMObject[] rpmObjects = loadFromXpath("/Task[lastModifiedDate>'"
                + XpathTesterUtils.convertToString(new GregorianCalendar())
                + " 00:00:01']", null);

        assertNotNull(rpmObjects);
        assertNotNull(rpmObjects[0]);

        Task reloaded = null;

        // find the task we just inserted
        for (int i = 0; i < rpmObjects.length; i++)
        {
            if (task.getID().equals(rpmObjects[i].getID()))
            {
                reloaded = (Task) rpmObjects[i];
                break;
            }
        }

        assertNotNull(reloaded);
        WorkElementFixture.compare(task, reloaded, null);
    }
}
