/*
 * create on 10-Jan-2007
 * 
 * Defect 59444: assigning work to a resource 
 * takes current timestamp into consideration.
 */
package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.text.SimpleDateFormat;
import java.util.Calendar;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author cobber
 */
public class Test59444 extends ClientTestController
{
    private Proposal proposal = null;

    private SummaryTask summaryTask1 = null;

    private SummaryTask summaryTask2 = null;

    private Task task = null;

    private WorkElementScope taskScope = null;

    private Resource resource = null;

    private TaskAssignment taskAssignment = null;

    private Calendar assignmentStartDate = null;

    public void setUp() throws Exception
    {
        super.setUp();

        // create a proposal
        proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);
        RPMObject[] rpmObjects = loadFromXpath("/Proposal[@name='"
                + proposal.getName() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Proposal);
        proposal = (Proposal) rpmObjects[0];

        // create level-one summary task
        summaryTask1 = (SummaryTask) WorkElementFixture.createSaveFixture(
            proposal, new SummaryTask(), null, this);
        rpmObjects = loadFromXpath("/SummaryTask[@name='"
                + summaryTask1.getName() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);
        summaryTask1 = (SummaryTask) rpmObjects[0];

        // create level-two summary task
        summaryTask2 = (SummaryTask) WorkElementFixture.createSaveFixture(
            summaryTask1, new SummaryTask(), null, this);
        rpmObjects = loadFromXpath("/SummaryTask[@name='"
                + summaryTask2.getName() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SummaryTask);
        summaryTask2 = (SummaryTask) rpmObjects[0];
    }

    private void init(int year, int month, int date, Double duration) throws Exception
    {
        // create a resource
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        resource = ResourceFixture.createSaveFixture(resourceScope, this);
        RPMObject[] rpmObjects = loadFromXpath("/Resource[@username='"
                + resource.getUserName() + "']", resourceScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);
        assertNotNull(((Resource) rpmObjects[0]).getCalendar());
        resource = (Resource) rpmObjects[0];

        // create a task
        task = (Task) WorkElementFixture.createSaveFixture(summaryTask2,
            new Task(), null, this);
        rpmObjects = loadFromXpath("/Task[@name='" + task.getName() + "']",
            null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Task);
        task = (Task) rpmObjects[0];

        // Load a SecurityRole
        rpmObjects = loadFromXpath("/SecurityRole[@name='Task Responsible']",
            null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof SecurityRole);
        SecurityRole securityRole = (SecurityRole) rpmObjects[0];

        // Create a ResourceTaskAssignment
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();
        resourceTaskAssignment.setParent(task);
        resourceTaskAssignment.setResource(resource);
        resourceTaskAssignment.setSecurityRole(securityRole);

        // create a TaskAssignment and set the AssignmentStartDate and Duration
        taskAssignment = new TaskAssignment();
        assignmentStartDate = Calendar.getInstance();
        assignmentStartDate.set(year, month, date);
        taskAssignment.setAssignmentStartDate(assignmentStartDate);

        // Assign a certain hours work duration
        taskAssignment.setProposedDurationHours(duration);
        taskAssignment.setParent(resourceTaskAssignment);
        
        resourceTaskAssignment.setTaskAssignment(taskAssignment);
        task.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            resourceTaskAssignment
        });
        task.setContainingProject(proposal);

        taskScope = new WorkElementScope();
        TaskAssignmentScope taskAssignmentScope = new TaskAssignmentScope();
        ResourceTaskAssignmentScope resourceTaskAssignmentScope = new ResourceTaskAssignmentScope();

        resourceTaskAssignmentScope.setResource(resourceScope);
        taskAssignmentScope.setParent(resourceTaskAssignmentScope);
        resourceTaskAssignmentScope.setTaskAssignment(taskAssignmentScope);
        taskScope.setResourceTaskAssignments(resourceTaskAssignmentScope);
        taskScope.setScheduleDates(true);
        resourceTaskAssignmentScope.setParent(taskScope);
    }

    public void testResourceTaskAssignmentAssignmentFinishDateWith8HoursDuration()
            throws Exception
    {
        init(2007, 0, 22, new Double(8.0));

        saveAndValidate();
    }

    public void testResourceTaskAssignmentAssignmentFinishDateWithLessThan8HoursDuration()
            throws Exception
    {
        init(2007, 0, 22, new Double(6.0));

        saveAndValidate();
    }

    public void testResourceTaskAssignmentAssignmentFinishDateWithMoreThan8HoursDurationStartOnNoneFriday()
            throws Exception
    {
        init(2007, 0, 22, new Double(20.0));

        saveAndValidate();
    }

    public void testResourceTaskAssignmentAssignmentFinishDateWithMoreThan8HoursDurationStartOnFriday()
            throws Exception
    {
        init(2007, 0, 26, new Double(20.0));

        saveAndValidate();
    }

    private void saveAndValidate() throws Exception
    {
        SaveResult saveResult = this.save(task, taskScope);
        assertTrue(saveResult.isSuccessful());
        assertNotNull(saveResult.getRpmObject());
        assertTrue(saveResult.getRpmObject() instanceof Task);

        // validate result
        Task savedTask = (Task) saveResult.getRpmObject();
        assertNotNull(savedTask.getResourceTaskAssignments());
        assertEquals(1, savedTask.getResourceTaskAssignments().length);
        ResourceTaskAssignment savedResourceTaskAssignment = savedTask
            .getResourceTaskAssignments()[0];
        assertNotNull(savedResourceTaskAssignment);
        TaskAssignment savedTaskAssignment = savedResourceTaskAssignment
            .getTaskAssignment();
        assertNotNull(savedTaskAssignment);
        Calendar assignmentFinishDate = savedTaskAssignment
            .getAssignmentFinishDate();
        assertNotNull(assignmentFinishDate);

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        Calendar expectedFinishDate = assignmentStartDate;
        
        // validate the finishDate
        assertNotNull(taskAssignment.getProposedDurationHours());
        for (int i = 0; i != ((int) Math.ceil((taskAssignment
            .getProposedDurationHours().doubleValue() / 8))) - 1; ++i)
        {
            expectedFinishDate.add(Calendar.DAY_OF_MONTH, 1);
            if (expectedFinishDate.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY)
            {
                expectedFinishDate.add(Calendar.DAY_OF_MONTH, 2);
            }
        }
        String finishDateExpected = format.format(expectedFinishDate.getTime());
        String finishDate = format.format(assignmentFinishDate.getTime());
        assertEquals(finishDateExpected, finishDate);
    }
}
