package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.document.fixtures.DocumentElementFixture;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test60001 extends ClientTestController {
	
	public void testLoadDocumentsForTransferredScopeElements() throws Exception
	{
		Proposal proposal = createSaveProposal();
		Requirement requirement = createSaveRequirement(proposal);
		Document document = createSaveDocument(requirement.getDocumentFolder());
		transferUnderWBS(proposal, requirement);
		proposal = loadProposalWithChildrenAndDocumentsReccursivelyAndValidate(proposal, new Document[] {document});
		
		WorkElement transferredElement = proposal.getChildren()[0];
		Document document1 = createSaveDocument(transferredElement.getDocumentFolder());
		proposal = loadProposalWithChildrenAndDocumentsReccursivelyAndValidate(proposal, new Document[] {document, document1});
	}
	
	public void testLoadDocumentsForWorkElement() throws Exception
	{
		Proposal proposal = createSaveProposal();
		Task task = createSaveTask(proposal);
		Document document = createSaveDocument(task.getDocumentFolder());
		proposal = loadProposalWithChildrenAndDocumentsReccursivelyAndValidate(proposal, new Document[] {document});
	}

	private Proposal createSaveProposal() throws Exception
	{
		WorkElementScope workElementScope = new WorkElementScope();
		workElementScope.setScopeElement(new ScopeElementScope());
		Proposal proposal = (Proposal)WorkElementFixture.createSaveFixture(new Proposal(), workElementScope, this);
		assertNotNull(proposal);
		assertNotNull(proposal.getScopeElement());
		assertTrue(proposal.getScopeElement() instanceof ScopeFolder);
		return proposal;
	}
	
	private Task createSaveTask(Proposal proposal) throws Exception
	{
		Task task = (Task)WorkElementFixture.createSaveFixture(proposal, new Task(), null, this);
		assertNotNull(task);
		
		WorkElementScope workElementScope = new WorkElementScope();
		workElementScope.setScopeElement(new ScopeElementScope());
		workElementScope.setDocumentFolder(new DocumentScope());
		task = (Task)loadFromID(task, workElementScope);
		assertNotNull(task);
		assertNotNull(task.getDocumentFolder());
		return task;
	}
	
	private Requirement createSaveRequirement(Proposal proposal) throws Exception
	{
		ScopeFolder parent = (ScopeFolder)proposal.getScopeElement();
		Requirement requirement = ScopeElementFixture.createRequirementFixture(null, 0, parent, this);
		ScopeElementScope scopeElementScope = new ScopeElementScope();
		scopeElementScope.setDocumentFolder(new DocumentScope());
		scopeElementScope.setProject(new WorkElementScope());
		requirement = (Requirement)save(requirement, scopeElementScope).getRpmObject();
		assertNotNull(requirement);
		assertNotNull(requirement.getDocumentFolder());
		assertNotNull(requirement.getProject());
		return requirement;
	}
	
	private Document createSaveDocument(DocumentElement parent) throws Exception
	{
		Document document = (Document)DocumentElementFixture.createSaveFixture(parent, new Document(), this);
		assertNotNull(document);
		return document;
	}
	
	private Proposal loadProposalWithChildrenAndDocumentsReccursively(Proposal proposal) throws Exception
	{
		WorkElementScope workElementScope = new WorkElementScope();
		DocumentScope documentScope = new DocumentScope();
		documentScope.setChildren(documentScope);
		workElementScope.setDocumentFolder(documentScope);
		workElementScope.setChildren(workElementScope);
		
		proposal = (Proposal)loadFromID(proposal, workElementScope);
		assertNotNull(proposal);
		return proposal;
	}
	
	private Proposal loadProposalWithChildrenAndDocumentsReccursivelyAndValidate(Proposal proposal, Document[] documents) throws Exception
	{
		proposal = loadProposalWithChildrenAndDocumentsReccursively(proposal);
		validate(proposal, documents);
		return proposal;
	}
	
	private void validate(Proposal proposal, Document[] documents)
	{
		assertNotNull(proposal);
		assertNotNull(proposal.getChildren());
		assertNotNull(proposal.getChildren()[0]);
		assertNotNull(proposal.getChildren()[0].getDocumentFolder());
		assertNotNull(proposal.getChildren()[0].getDocumentFolder().getChildren());
		assertTrue(proposal.getChildren()[0].getDocumentFolder().getChildren().length == documents.length);
		for(int i=0; i<documents.length; i++)
		{
			assertNotNull(proposal.getChildren()[0].getDocumentFolder().getChildren()[i]);
			assertTrue(proposal.getChildren()[0].getDocumentFolder().getChildren()[i] instanceof Document);
		}
		if(documents.length > 1)
		{
			assertFalse(proposal.getChildren()[0].getDocumentFolder().getChildren()[0].getID().equals(
					proposal.getChildren()[0].getDocumentFolder().getChildren()[1].getID()));
		}
	}
}
