/*
 * Created on 19-Jan-07
 * @author mmatwiy
 *
 * When I try to load a scope element that has been converted into task and transfered under wbs, 
 * i get the following error: SummaryTask: The scope is of type ScopeElementScope but must be 
 * WorkElementScope - Severity: Error - Class: SummaryTask - Error ID: 400025
 * 
 * The problem is that I do not know before hand if the object that i am about to load is a "regular" scope element or a one that has been transfered under wbs. That's why I'd have to create the scope as if it was a "regular" scope element. The API should be able to handle both cases under the hood.
 * 
 * Steps to reproduce:
 * 1- create a defect.
 * 2- assign the defect to a resource.
 * 3- transfere under wbs.
 * 4- Create a scope:
 * ScopeElementScope parent = new ScopeElementScope();
 * parent.setParent(parent);
 * ScopeElementScope scope = new ScopeElementScope();
 * scope.setParent(parent);
 * ResourceTaskAssignementScope rtaScope = new ResourceTaskAssignementScope();
 * rtaScope.setParent(parent);
 * scope.setResourceTaskAssignement(rtaScope);
 * 5- Load from xpath on "/Defect[@ID='3D59CECACC5F4D8593C52471B48776EF']" using the above scope.
 * Notice how you will get the following error message: 
 * SummaryTask: The scope is of type ScopeElementScope but must be WorkElementScope -
 * Severity: Error - Class: SummaryTask - Error ID: 400025
 */
package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class Test60433 extends ClientTestController
{
    public void test() throws Exception
    {
        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        RPMObject[] loadedObjects = loadFromXpath("/ScopeFolder[name='"
                + project.getName() + "']", null);
        assertNotNull(loadedObjects);
        assertNotNull(loadedObjects[0]);
        assertTrue(loadedObjects[0] instanceof ScopeFolder);
        ScopeFolder parent = (ScopeFolder) loadedObjects[0];

        Defect defect = new Defect();
        defect.setName(HelperTestUtilities.createName());
        defect.setParent(parent);
        save(defect, null);

        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());

        Resource resource = ResourceFixture.createSaveFixture(resourceScope,
            this);

        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setTaskAssignment(new TaskAssignmentScope());
        ResourceTaskAssignment resourceTaskAssignment = ResourceTaskAssignmentFixture
            .createFixture(defect, resource, null, rtaScope, false, this);

        save(resourceTaskAssignment, rtaScope);

        Result result = getApplicationInterface().transferUnderWBS(
            getSessionID(), null, defect);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        // defect's parent scope
        ScopeElementScope parentScope = new ScopeElementScope();
        // defect's granparent scope
        parentScope.setParent(new ScopeElementScope());

        ScopeElementScope defectScope = new ScopeElementScope();
        defectScope.setParent(parentScope);

        ResourceTaskAssignmentScope rtaLoadScope = new ResourceTaskAssignmentScope();
        defectScope.setResourceTaskAssignments(rtaLoadScope);

        RPMObject[] reloaded = loadFromXpath("/Defect[@ID='" + defect.getID()
                + "']", defectScope);
        assertNotNull(reloaded);
        assertEquals(1, reloaded.length);
        assertNotNull(reloaded[0]);
        assertTrue(reloaded[0] instanceof Defect);
        Defect reloadedDefect = (Defect) reloaded[0];

        assertNotNull(reloadedDefect.getResourceTaskAssignments());
        assertNotNull(reloadedDefect.getParent());
    }
}