package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * Description I assign a resource to task with the following expected start
 * date: 2006,11,25. However after the save operation the value of that field is
 * set to today's date. There's no mention in the documentation nor warning that
 * state that the field is read only.
 * 
 * @author andreit
 */
public class Test62932 extends ClientTestController
{
    private Calendar desiredForecastDate;

    public void setUp() throws Exception
    {
        super.setUp();
        desiredForecastDate = Calendar.getInstance();
        desiredForecastDate.set(Calendar.MONTH, Calendar.NOVEMBER);
        desiredForecastDate.set(Calendar.DAY_OF_MONTH, 25);
        desiredForecastDate.set(Calendar.YEAR, 2006);
    }

    /**
     * Try updating the expected start date. A warning should be thrown.
     * 
     * @throws Exception
     */
    public void testUpdateExpectedStartDate() throws Exception
    {
        TaskAssignment taskAssignment = TaskAssignmentFixture.createFixture(
            new TaskAssignmentScope(), this);

        ResourceTaskAssignment taskresourceTaskAssignment = (ResourceTaskAssignment) taskAssignment
            .getParent();

        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setTaskAssignment(new TaskAssignmentScope());

        taskresourceTaskAssignment.getTaskAssignment().setExpectedStartDate(
            desiredForecastDate);
        SaveResult result = save(taskresourceTaskAssignment, rtaScope);
        assertTrue(result.isSuccessful());
        assertFalse(result.isWarningFree());
        assertTrue(result.getWarnings().length == 1);
        assertTrue(result.getWarnings()[0].getErrorID() == ErrorCodes.READ_ONLY_FIELD);
        assertTrue(result.getWarnings()[0].getFieldName().equals(
            ValidationConstants.EXPECTED_START_DATE_FIELD));
    }

    /**
     * Create a TaskAssignment with an expected forecast date.
     * 
     * @throws Exception
     */
    public void testSetExpectedStartDate() throws Exception
    {
        // save a task
        WorkElementScope taskSaveScope = new WorkElementScope();
        taskSaveScope.setContainingProject(new WorkElementScope());
        
        Task parentTask = (Task) WorkElementFixture.createSaveFixture(
            new Task(), taskSaveScope, this);
        
        TaskAssignment ta = TaskAssignmentFixture.createFixture(parentTask,
            ResourceFixture.createSaveFixture(this), new TaskAssignmentScope(),
            this);
        ta.setExpectedStartDate(desiredForecastDate);

        // save a resource Task assignment
        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setTaskAssignment(new TaskAssignmentScope());

        ResourceScope resourceScope = new ResourceScope();  
        resourceScope.setCalendar(new CalendarScope());
        Resource resourceWork = ResourceFixture.createSaveFixture(resourceScope, this);
        ResourceTaskAssignment resourceTaskAssignment = ResourceTaskAssignmentFixture
            .createFixture(parentTask, resourceWork,
                null, rtaScope, false, this);
 
        
        WorkElementScope taskScope = new WorkElementScope();
        taskScope.setResourceTaskAssignments(rtaScope);
        resourceTaskAssignment.setTaskAssignment(ta);
        parentTask.setResourceTaskAssignments(new ResourceTaskAssignment[] { resourceTaskAssignment });
        
        SingleResult result = save(parentTask, taskScope);
        assertTrue(result.isSuccessful());
        assertFalse(result.isWarningFree());
        assertTrue(result.getWarnings().length == 1);
        assertTrue(result.getWarnings()[0].getErrorID() == ErrorCodes.READ_ONLY_FIELD);
        assertTrue(result.getWarnings()[0].getFieldName().equals(
            ValidationConstants.EXPECTED_START_DATE_FIELD));
    }
}