package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.wbs.constants.ErrorCodes;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test63291 extends ClientTestController
{

    /**
     * In the delphi client version 116, we can update start date, dur, durh and
     * finish date. We can also clear durd, durh and finish date.
     */
    public void testReadWriteProposedFieldsWhenNoWork() throws Exception
    {
        Task tsk = (Task) WorkElementFixture.createFixture(new Task(),
            new WorkElementScope(), this);
        tsk.setResourceTaskAssignments(null);
        tsk = (Task) save(tsk, new WorkElementScope()).getRpmObject();

        WorkElementScope taskScope = new WorkElementScope();
        taskScope.setScheduleDates(true);

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(Calendar.getInstance());
        proposedDate.setFinishDate(Calendar.getInstance());
        proposedDate.setHours(new Double(60));
        proposedDate.setDays(new Double(20));
        tsk.setProposedDate(proposedDate);

        SaveResult res = save(tsk, taskScope);
        assertNull(res.getWarnings());
    }

    /**
     * As soon as there is work assigned to the SimpleNode, we can't update :
     * (start/end) date, dur(h/d) - they are read-only in build 116 Note: At
     * least a resource has to have Work assigned (as opposed to being only a
     * participant)
     */
    public void testFailWriteProposedFieldsWhenWorkAssigned() throws Exception
    {
        /*
         * Associate a valid ResourceTaskAssignment to the task
         */
        WorkElementScope taskCreateScope = new WorkElementScope();
        taskCreateScope.setContainingProject(new WorkElementScope());
        Task task = (Task) WorkElementFixture.createFixture(new Task(),
            new WorkElementScope(), this);
        task = (Task) save(task, taskCreateScope).getRpmObject();

        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setTaskAssignment(new TaskAssignmentScope());
        ResourceTaskAssignment resourceAssignment = ResourceTaskAssignmentFixture
            .createFixture(rtaScope, false, this);
        Resource resource = ResourceFixture.createSaveFixture(this);
        RPMCalendar resourceCalendar = RPMCalendarFixture.createSaveFixture(
            new CalendarScope(), this);
        resource.setCalendar(resourceCalendar);
        resourceAssignment.setResource(resource);
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        resource = (Resource) save(resource, resourceScope).getRpmObject();

        task.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            resourceAssignment
        });
        WorkElementScope taskSaveScope = new WorkElementScope();
        taskSaveScope.setResourceTaskAssignments(rtaScope);
        task = (Task) save(task, taskSaveScope).getRpmObject();
        assertNotNull("Couldn't assign work to the resource for this task ",
            task.getResourceTaskAssignments()[0].getTaskAssignment());

        // Update the proposed date fields
        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(Calendar.getInstance());
        proposedDate.setFinishDate(Calendar.getInstance());
        proposedDate.setHours(new Double(50));
        proposedDate.setDays(new Double(20));
        task.setProposedDate(proposedDate);

        WorkElementScope taskUpdateScopeScope = new WorkElementScope();
        taskUpdateScopeScope.setScheduleDates(true);

        SaveResult updateResult = save(task, taskUpdateScopeScope);
        assertNotNull(updateResult.getWarnings());
        assertTrue(updateResult.getWarnings().length == 6);
        for (int i = 0; i < updateResult.getWarnings().length; i++)
        {
            int errorid = updateResult.getWarnings()[i].getErrorID();
            assertTrue("ErrorID is " + errorid, errorid == ErrorCodes.READ_ONLY_ONCE_WORK_ASSIGNED
                || errorid == com.ibm.rpm.framework.constants.ErrorCodes.READ_ONLY_FIELD);
        }
    }
}
