package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ConstraintType;

/**
 * WS - constrainDate should be read only when constrain type is
 * AsSoonAsPossible
 * 
 * 1- Create a Milesone. 2- set constraintType to AsSoonAsPossible. 3- set
 * constrainDate to some date. 4- save the milestone and notice how
 * constrainDate is silently ignored!
 * 
 * @author andreit
 */
public class Test63422 extends ClientTestController
{
    /**
     * both AsSoonAsPossible and constrainDate are set
     * 
     * @throws Exception
     */
    public void testReadOnlyConstrainDate() throws Exception
    {
        Project prj = (Project) WorkElementFixture.createFixture(new Project(),
            new WorkElementScope(), this);

        prj = (Project) save(prj, new WorkElementScope()).getRpmObject();

        Milestone milestone = new Milestone();
        milestone.setName(HelperTestUtilities.createName()
                + Milestone.class.getName());

        // start as soon as possible, but also set a date. should fail.
        milestone.setConstraintDate(DateUtil
            .convertToCalendarNoTime("2009-10-14"));
        milestone.setConstraintType(ConstraintType.AsSoonAsPossible);
        milestone.setParent(prj);

        Result res = saveMustFail(milestone, new WorkElementScope());
        assertTrue(!res.isSuccessful());
        assertTrue(res.getErrors().length == 2);
        assertEquals(res.getErrors()[0].getErrorID(), ErrorCodes.MUST_BE_NULL);
        assertEquals(res.getErrors()[1].getErrorID(), ErrorCodes.MUST_BE_NULL);
    }

    /**
     * First AsSoonAsPossible is set, then constraintdate.
     * 
     */
    public void testUpdateASAThenConstraintDate() throws Exception
    {
        Project prj = (Project) WorkElementFixture.createFixture(new Project(),
            new WorkElementScope(), this);

        prj = (Project) save(prj, new WorkElementScope()).getRpmObject();

        Milestone milestone = new Milestone();
        milestone.setName(HelperTestUtilities.createName()
                + Milestone.class.getName());
        milestone.setConstraintType(ConstraintType.AsSoonAsPossible);
        milestone.setParent(prj);

        Milestone reloadedMilestone = (Milestone) save(milestone,
            new WorkElementScope()).getRpmObject();

        reloadedMilestone.setConstraintDate(DateUtil
            .convertToCalendarNoTime("2009-10-14"));

        Result res = saveMustFail(reloadedMilestone, new WorkElementScope());
        assertTrue(!res.isSuccessful());
        assertTrue(res.getErrors().length == 1);
        assertEquals(res.getErrors()[0].getErrorID(), ErrorCodes.MUST_BE_NULL);
    }

    /**
     * First constraintDate, then AsSoonAsPossible
     */
    public void testUpdateConstraintDateThenASA() throws Exception
    {
        Project prj = (Project) WorkElementFixture.createFixture(new Project(),
            new WorkElementScope(), this);

        prj = (Project) save(prj, new WorkElementScope()).getRpmObject();

        Milestone milestone = new Milestone();
        milestone.setName(HelperTestUtilities.createName()
                + Milestone.class.getName());
        milestone.setConstraintType(ConstraintType.FinishNoEarlierThan);
        milestone.setConstraintDate(DateUtil
            .convertToCalendarNoTime("2009-10-14"));
        milestone.setParent(prj);

        Milestone reloadedMilestone = (Milestone) save(milestone,
            new WorkElementScope()).getRpmObject();
        reloadedMilestone.setConstraintType(ConstraintType.AsSoonAsPossible);

        Result res = saveMustFail(reloadedMilestone, new WorkElementScope());
        assertTrue(!res.isSuccessful());
        assertTrue(res.getErrors().length == 1);
        assertEquals(res.getErrors()[0].getErrorID(), ErrorCodes.MUST_BE_NULL);
    }

    public void testConstraintTypeNullUpdate() throws Exception
    {
        WorkElementScope wes = new WorkElementScope();
        Project prj = (Project) WorkElementFixture.createFixture(new Project(),
            wes, this);

        prj = (Project) save(prj, wes).getRpmObject();

        Milestone milestone = new Milestone();
        milestone.setName(HelperTestUtilities.createName()
                + Milestone.class.getName());
        milestone.setParent(prj);

        milestone = (Milestone) save(milestone, wes).getRpmObject();

        // test the default value to be As soon as possible
        assertEquals(ConstraintType.AsSoonAsPossible, milestone
            .getConstraintType());

        // reset to null, save must fail
        milestone.setConstraintType(null);

        Result res = saveMustFail(milestone, wes);
        assertTrue(!res.isSuccessful());
        assertTrue(res.getErrors().length == 1);
        assertEquals(res.getErrors()[0].getErrorID(),
            ErrorCodes.CANNOT_CLEAR_FIELD);
    }
}