package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.SecurityRoleFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;

/*
 * 5- Task task = new Task();
 task.setID("xxxxxxxxx");  // set it t the id of the task that you just created in delphi
 task.setPercentageDurationCompleted(new Double(10));
 save(task, null);
 6- notice how you get a warning "ask percentageDurationCompleted is a read-only field 
 when planDate.startDate does not have a value. It will be overwritten/ignored."
 */
public class Test63569 extends ClientTestController
{
    Task task;

    Project project;

    Resource assignedTo;

    public void setUp() throws Exception
    {
        super.setUp();

        /*
         * /* Create a duration-based portfolio and task
         */
        project = (Project) WorkElementFixture.createFixture(null,
            new Project(), new WorkElementScope(), this);
        project.setName(project.getName() + System.currentTimeMillis());
        project.setAssignmentType(AssignmentType.Duration);
        project = (Project) save(project, new WorkElementScope())
            .getRpmObject();

        assertNotNull(project);

        task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setParent(project);
        // task.setPriority(new Integer(500));

        // assign it to a resource
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();

        resourceTaskAssignment.setParent(task);
        Resource resource = TaskAssignmentFixture.createProperResource(
            resourceTaskAssignment, this);

        resource.setUserName(HelperTestUtilities.createName()
                + System.currentTimeMillis());
        resource = (Resource) save(resource, new ResourceScope())
            .getRpmObject();
        this.assignedTo = resource;
        resourceTaskAssignment.setResource(resource);
        SecurityRole securityRole = SecurityRoleFixture
            .createSecurityRole(task);
        securityRole = (SecurityRole) loadFromID(securityRole,
            new RPMObjectScope());
        resourceTaskAssignment.setSecurityRole(securityRole);

        task.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            resourceTaskAssignment
        });

        task = (Task) save(task, new WorkElementScope()).getRpmObject();
    }

    public void testUpdateDurationCompleted() throws Exception
    {     
        Task newTask = new Task();
        newTask.setID(task.getID());
        newTask = (Task) loadFromID(newTask, new WorkElementScope());
        newTask.setPercentageDurationCompleted(new Double(10));
        save(newTask, null).getRpmObject();
        
        Task savedTask =  (Task) loadFromID(newTask, new WorkElementScope());
        
        WorkElementFixture.compare(newTask, savedTask, new
             WorkElementScope());
    }
}
