package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Contour;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ContourType;

public class TestContour1 extends ClientTestController {
	
	private ResourceTaskAssignment assignment = null;
	private ResourceTaskAssignmentScope mainScope = null;
    private GenericProject project = null;
    Resource resource = null;
	
	public void setUp() throws Exception
	{
		super.setUp();
		project = createSaveProposal();
		mainScope = ResourceTaskAssignmentFixture.createScope(true, true,
				false, true, true);
		resource = createSaveResource();
	}
	
	public void testAffectScopeElementUsingContour() throws Exception
	{
		ScopeElement parent = createSaveRequirement(project.getScopeElement());
		assignment = ResourceTaskAssignmentFixture
						.createFixture(parent, resource, null, mainScope, false, this);
		assignment.setContour(createContour());
		assignment = (ResourceTaskAssignment)save(assignment, null).getRpmObject();
		assertNotNull(assignment);
	}
	
	public void testAffectProjectUsingContour() throws Exception
	{
		assignment = ResourceTaskAssignmentFixture
						.createFixture(project, resource, null, mainScope, false, this);
		assignment.setContour(createContour());
		assignment = (ResourceTaskAssignment)save(assignment, null).getRpmObject();
		assertNotNull(assignment);
	}
	
	private Proposal createSaveProposal() throws Exception
	{
		Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setScopeElement(new ScopeElementScope());
        workElementScope.setContainingProject(new WorkElementScope());
        proposal = (Proposal)save(proposal, workElementScope).getRpmObject();
        assertNotNull(proposal);
        assertNotNull(proposal.getScopeElement());
        assertTrue(ScopeFolder.class.isAssignableFrom(proposal.getScopeElement().getClass()));
        return proposal;
	}
	
	private Requirement createSaveRequirement(ScopeElement parent) throws Exception
	{
		Requirement requirement = new Requirement();
        requirement.setName(HelperTestUtilities.createName());
        requirement.setParent(parent);
        ScopeElementScope scopeElementScope = new ScopeElementScope();
        scopeElementScope.setProject(new WorkElementScope());
        requirement = (Requirement)save(requirement, scopeElementScope).getRpmObject();
        assertNotNull(requirement);
        assertNotNull(requirement.getProject());
        return requirement;
    }
	
	private Resource createSaveResource() throws Exception
	{
		ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        
        Resource resource = new Resource();
        resource.setFullName("RTA Tester");
        resource.setParent(PoolFixture.createSaveFixture(this));
        resource.setCalendar(RPMCalendarFixture
            .retrieveStaticFixtureForInserts(this));
        resource = (Resource)save(resource, resourceScope).getRpmObject();
        assertNotNull(resource);
        return resource;
	}
	
	private Contour createContour()
	{
		Contour contour = new Contour();
		contour.setContourType(ContourType.HoursPerDay);
		contour.setStartDate(Calendar.getInstance());
		contour.setWork(new Double(3));
		contour.setEffortInMinutes(new Integer(3 * 480));
		return contour;
	}
}
