package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.security.testcases.role.BaseRoleSecurityTestLocal;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.containers.TaskSecurityRole;
import com.ibm.rpm.wbs.containers.Contour;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ContourType;

public class TestContourLocal extends BaseRoleSecurityTestLocal {
	
	private ResourceTaskAssignment assignment = null;
	private ResourceTaskAssignmentScope mainScope = null;
	private ProjectSecurityRole grandParentSecurityRole = null;
	private TaskSecurityRole parentSecurityRole = null;
    
	public void setUp() throws Exception
	{
		super.setUp();
		Proposal proposal = createSaveProposal();
		WorkElement parentWorkElement = createSaveTask(proposal);
		mainScope = ResourceTaskAssignmentFixture.createScope(true, true,
				false, true, true);

		Resource resource = createSaveResource();
		assignment = ResourceTaskAssignmentFixture
				.createFixture(parentWorkElement, resource, null, mainScope, false, this);
		assignment.setContour(createContour());
	}
	
	public void testAssignResourceUsingContourWithBothFlagsOff() throws Exception
	{
		runTest(false, false, false);
	}
	
	public void testAssignResourceUsingContourWithGrandparentFlagOn() throws Exception
	{
		runTest(true, false, true);
	}
	
	public void testAssignResourceUsingContourWithParentFlagOn() throws Exception
	{
		runTest(false,true,true);
	}
	
	public void testAssignResourceUsingContourWithBothFlagsOn() throws Exception
	{
		runTest(true,true,true);
	}
	
	private void runTest(boolean grandParentFlagValue, boolean parentFlagValue, boolean canSave) throws Exception
	{
		if(grandParentFlagValue)
		{
			if(parentFlagValue)
			{
				turnFlagsOn(true,true);
			}
			else
			{
				turnFlagsOn(true, false);
				turnFlagsOff(false, true);
			}
		}
		else
		{
			if(parentFlagValue)
			{
				turnFlagsOff(true, false);
				turnFlagsOn(false, true);
			}
			else
			{
				turnFlagsOff(true, true);
			}
		}
		assignment = (ResourceTaskAssignment)saveAndValidate(assignment, null, canSave);
		if(canSave)
		{
			assertNotNull(assignment);
		}
	}
	
	private Contour createContour()
	{
		Contour contour = new Contour();
		contour.setContourType(ContourType.HoursPerDay);
		contour.setStartDate(Calendar.getInstance());
		contour.setWork(new Double(3));
		contour.setEffortInMinutes(new Integer(3 * 480));
		return contour;
	}
	
	private Proposal createSaveProposal() throws Exception
	{
		Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());
        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        proposal = (Proposal)save(proposal, workElementScope).getRpmObject();
        assertNotNull(proposal);
        assertNotNull(proposal.getResourceTaskAssignments());
        assertTrue(proposal.getResourceTaskAssignments().length == 1);
        assertNotNull(proposal.getResourceTaskAssignments()[0]);
        assertNotNull(proposal.getResourceTaskAssignments()[0].getSecurityRole());
        grandParentSecurityRole = (ProjectSecurityRole)proposal.getResourceTaskAssignments()[0].getSecurityRole();
        return proposal;
	}
	
	private Task createSaveTask(Proposal parent) throws Exception
	{
		Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setParent(parent);
        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        workElementScope.setContainingProject(new WorkElementScope());
        task = (Task)save(task, workElementScope).getRpmObject();
        assertNotNull(task);
        assertNotNull(task.getResourceTaskAssignments());
        assertTrue(task.getResourceTaskAssignments().length == 1);
        assertNotNull(task.getResourceTaskAssignments()[0]);
        assertNotNull(task.getResourceTaskAssignments()[0].getSecurityRole());
        parentSecurityRole = (TaskSecurityRole)task.getResourceTaskAssignments()[0].getSecurityRole();
        assertNotNull(task.getContainingProject());
        return task;
    }
	
	private Resource createSaveResource() throws Exception
	{
		ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        
        Resource resource = new Resource();
        resource.setFullName("RTA Tester");
        resource.setParent(PoolFixture.createSaveFixture(this));
        resource.setCalendar(RPMCalendarFixture
            .retrieveStaticFixtureForInserts(this));
        resource = (Resource)save(resource, resourceScope).getRpmObject();
        assertNotNull(resource);
        return resource;
	}
	
	private void turnFlagsOff(boolean forGrandParent, boolean forParent) throws Exception
	{
		if(forGrandParent)
		{
			grandParentSecurityRole.setCanAssignResource(Boolean.FALSE);
			grandParentSecurityRole = (ProjectSecurityRole)save(grandParentSecurityRole,null).getRpmObject();
			assertNotNull(grandParentSecurityRole);
			assertFalse(grandParentSecurityRole.getCanAssignResource().booleanValue());
		}
		if(forParent)
		{
			parentSecurityRole.setCanAssignResource(Boolean.FALSE);
			parentSecurityRole = (TaskSecurityRole)save(parentSecurityRole,null).getRpmObject();
			assertNotNull(parentSecurityRole);
			assertFalse(parentSecurityRole.getCanAssignResource().booleanValue());
		}
	}
	
	private void turnFlagsOn(boolean forGrandParent, boolean forParent) throws Exception
	{
		if(forGrandParent)
		{
			grandParentSecurityRole.setCanAssignResource(Boolean.TRUE);
			grandParentSecurityRole = (ProjectSecurityRole)save(grandParentSecurityRole,null).getRpmObject();
			assertNotNull(grandParentSecurityRole);
			assertTrue(grandParentSecurityRole.getCanAssignResource().booleanValue());
		}
		if(forParent)
		{
			parentSecurityRole.setCanAssignResource(Boolean.TRUE);
			parentSecurityRole = (TaskSecurityRole)save(parentSecurityRole,null).getRpmObject();
			assertNotNull(parentSecurityRole);
			assertTrue(parentSecurityRole.getCanAssignResource().booleanValue());
		}
	}
}
