package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.applicationadministration.types.SelectType;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WbsModuleScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestRATLC00307395 extends ClientTestController {

	static 	String 				projectName 	= "WBS_PROJECT";
	private Project 			project1		= null;
	private SummaryTask 		summaryTask		= null;
	private Task				task			= null;
	private WorkElementScope 	projectScope 	= null;
	private Attribute 			attribute 		= null;
	
	protected void setUp() throws Exception
	{
		super.setUp();
		createSaveAttributes();
	}
	
	public void testProject() throws Exception
	{
		runTestForWorkElement(new Project());
	}
	
	public void testProposal() throws Exception
	{
		runTestForWorkElement(new Proposal());
	}
	
	public void testWorkOrganization() throws Exception
	{
		runTestForWorkElement(new WorkOrganization());
	}
	
	public void testDeliverable() throws Exception
	{
		runTestForWorkElement(new Deliverable());
	}
	
	public void testWorkProduct() throws Exception
	{
		runTestForWorkElement(new WorkProduct());
	}
	
	public void testSummaryTask() throws Exception
	{
		runTestForWorkElement(new SummaryTask());
	}
	
	public void testTask() throws Exception
	{
		runTestForWorkElement(new Task());
	}
	
	public void testMilestone() throws Exception
	{
		runTestForWorkElement(new Milestone());
	}
	
	public void testTransferredRequirement() throws Exception
	{
		WorkElement transferredElement = createSaveTransferredElement(new Requirement());
		runTestForWorkElement(transferredElement);
	}
	
	public void testTransferredRequirementTask() throws Exception
	{
		WorkElement transferredElement = createSaveTransferredTask(new Requirement());
		runTestForWorkElement(transferredElement);
	}
	
	private void runTestForWorkElement(WorkElement workElement) throws Exception
	{
		if(workElement.getID() == null)
		{
			workElement = WorkElementFixture.createSaveFixture(workElement, null, this);
		}
		
		WorkElementScope workElementScope = new WorkElementScope();
		workElementScope.setAttributeAssignments(true);
		
		AttributeAssignment attributeAssignment = new AttributeAssignment();
		attributeAssignment.setAttribute(attribute);
		attributeAssignment.setValue("10");
		workElement.setAttributeAssignments(new AttributeAssignment[] {attributeAssignment});
		
		workElement = (WorkElement)save(workElement,workElementScope).getRpmObject();
		assertNotNull(workElement);
		assertNotNull(workElement.getAttributeAssignments());
		assertTrue(workElement.getAttributeAssignments().length == 1);
		assertNotNull(workElement.getAttributeAssignments()[0]);
		assertTrue(workElement.getAttributeAssignments()[0].getValue().equals("10"));
		
		workElement.getAttributeAssignments()[0].setValue("20");
		
		// TODO: Uncomment the following line when running the test
//		workElement.getAttributeAssignments()[0].assignTypeID(null);
		workElement = (WorkElement)save(workElement,workElementScope).getRpmObject();
		
		assertNotNull(workElement);
		assertNotNull(workElement.getAttributeAssignments());
		assertTrue(workElement.getAttributeAssignments().length == 1);
		assertNotNull(workElement.getAttributeAssignments()[0]);
		assertTrue(workElement.getAttributeAssignments()[0].getValue().equals("20"));
	}
	
	private WorkElement createSaveTransferredElement(AbstractScope scopeElement) throws Exception
	{
		WorkElement workElement = null;
		if(scopeElement.getID() == null)
		{
			WorkElementScope workElementScope = new WorkElementScope();
			workElementScope.setScopeElement(new ScopeElementScope());
			workElement = WorkElementFixture.createSaveFixture(new Proposal(), workElementScope, this);
			assertNotNull(workElement);
			assertNotNull(workElement.getScopeElement());

			ScopeElementScope scopeElementScope = new ScopeElementScope();
			scopeElementScope.setProject(new WorkElementScope());
			scopeElement = (AbstractScope)ScopeElementFixture.createSaveScopeElement(workElement.getScopeElement(), scopeElement, 0, scopeElementScope, this);
			assertNotNull(scopeElement);
			assertNotNull(scopeElement.getProject());
		}
		else
		{
			workElement = scopeElement.getProject();
		}
		transferUnderWBS(workElement, scopeElement);
		
		WorkElement transferredElement = new WorkElement();
		transferredElement.setID(scopeElement.getID());
		transferredElement = (WorkElement)loadFromID(transferredElement, null);
		return transferredElement;
	}
	
	private WorkElement createSaveTransferredTask(AbstractScope scopeElement) throws Exception
	{
		WorkElementScope workElementScope = new WorkElementScope();
		workElementScope.setScopeElement(new ScopeElementScope());
		WorkElement workElement = WorkElementFixture.createSaveFixture(new Proposal(), workElementScope, this);
		assertNotNull(workElement);
		assertNotNull(workElement.getScopeElement());

		scopeElement = (AbstractScope)ScopeElementFixture.createSaveScopeElement(workElement.getScopeElement(), scopeElement, 0, null, this);
		assertNotNull(scopeElement);
		
		ScopeElementScope scopeElementScope = new ScopeElementScope();
		scopeElementScope.setProject(new WorkElementScope());
		scopeElement.setTask(new Boolean(true));
		scopeElement = (AbstractScope)save(scopeElement,scopeElementScope).getRpmObject();
		assertNotNull(scopeElement);
		assertNotNull(scopeElement.getTask());
		assertTrue(scopeElement.getTask().booleanValue());
		assertNotNull(scopeElement.getProject());
		
		return createSaveTransferredElement(scopeElement);
	}
	
	public void test() throws Exception
	{
		WorkElementScope taskScope = new WorkElementScope();
		taskScope.setAttributeAssignments(true); // Save attribute assignment for Tasks
		
		taskScope.setChildren(taskScope);	

		projectScope = new WorkElementScope();
		projectScope.setAttributeAssignments(true); // Save attribute assignment for project 
		projectScope.setChildren(taskScope); 			

		// Create Project
		createProject();

		// We used the same project scope - workElementScope
		createSaveAttributesAssignmentsForProjectAndTask();

		// Retrieve all SummaryTasks and Tasks that have a attribute named "Production"  	
		String xPathStr = "Project[@attributeAssignments[/AttributeCategory[@name = 'My Category'] " +
		"/AttributeClassification/Attribute[name = 'Production']]]";

		RPMObject[] resultArray = loadFromXpath(xPathStr, projectScope);

		if (resultArray != null)
		{       	
			project1 = (Project)resultArray[0];
			AttributeAssignment[] attrAssign = project1.getAttributeAssignments();  
			if ( attrAssign != null)
			{
				for (int j = 0; j < attrAssign.length; j++)
				{
					attrAssign[j].setValue("Project attribute updated!!!!");
					// TODO: Uncomment the following line when running the test
//					attrAssign[j].assignTypeID(null);
					SaveResult saveRes = save(project1, projectScope);  
					project1 = (Project) saveRes.getRpmObject();
					assertNotNull(project1);
					assertNotNull(project1.getAttributeAssignments());
					assertTrue(project1.getAttributeAssignments().length==1);
					assertNotNull(project1.getAttributeAssignments()[0]);
					assertTrue(project1.getAttributeAssignments()[0].getValue().equals("Project attribute updated!!!!"));
				}
				if ( project1.getChildren() != null)
				{
					WorkElement[] elementList = project1.getChildren();
					for (int k = 0; k < elementList.length; k++)
					{
						if (elementList[k] instanceof SummaryTask )
						{
							summaryTask = (SummaryTask)elementList[k];
							AttributeAssignment[] list = summaryTask.getAttributeAssignments();  
							if ( list != null)
							{
								for (int l = 0; l < list.length; l++)
								{
									list[l].setValue("SummaryTask updated value!!!");
									// TODO: Uncomment the following line when running the test
//									list[l].assignTypeID(null);
									SaveResult saveRes = save(summaryTask, projectScope); 
									summaryTask = (SummaryTask) saveRes.getRpmObject();
									assertNotNull(summaryTask);
									assertNotNull(summaryTask.getAttributeAssignments());
									assertTrue(summaryTask.getAttributeAssignments().length==1);
									assertNotNull(summaryTask.getAttributeAssignments()[0]);
									assertTrue(summaryTask.getAttributeAssignments()[0].getValue().equals("SummaryTask updated value!!!"));
								}
							}	        					
							if (summaryTask.getChildren() != null)
							{  
								WorkElement[] taskList = summaryTask.getChildren();
								for (int m = 0; m < taskList.length; m++)	        					
								{ 
									task = (Task)taskList[m];
									AttributeAssignment[] attrList = task.getAttributeAssignments();
									if (attrList != null)
									{
										for (int n = 0; n < attrList.length; n++)
										{
											attrList[n].setValue("20");	 	
											// TODO: Uncomment the following line when running the test
//											attrList[n].assignTypeID(null);
											SaveResult saveRes = save(task, projectScope); 
											task = (Task) saveRes.getRpmObject();
											assertNotNull(task);
											assertNotNull(task.getAttributeAssignments());
											assertTrue(task.getAttributeAssignments().length==1);
											assertNotNull(task.getAttributeAssignments()[0]);
											assertTrue(task.getAttributeAssignments()[0].getValue().equals("20"));
										}
									}
								}			        				
							}
						}
					}
				}
			}
		}
	}

	// Create Project with Summary Task and Task
	private void createProject() throws Exception
	{		
		project1 = new Project();
		project1.setName(projectName);
		RPMCalendar projectCalendar = (RPMCalendar) loadFromXpath("/RPMCalendar[@name='Standard (Default)']", null)[0];
		assertNotNull(projectCalendar);
		
		project1.setCalendar(projectCalendar);

		// Create a summary task and task for project		
		SaveResult saveRes =  save(project1, projectScope);        
		project1 =  (Project)saveRes.getRpmObject();
		assertNotNull(project1);

		summaryTask = new SummaryTask();
		summaryTask.setName("Summary_Task_A");         
		summaryTask.setParent(project1);

		SaveResult saveResult = save(summaryTask, projectScope);
		summaryTask = (SummaryTask) saveResult.getRpmObject();
		assertNotNull(summaryTask);

		task = new Task();
		task.setName("Task_A1");    	
		task.setParent(summaryTask);
		task = (Task)save(task, projectScope).getRpmObject();
		assertNotNull(task);

		SaveResult saveRes2 =  save(project1, projectScope);        
		project1 =  (Project)saveRes2.getRpmObject();
		assertNotNull(project1);
	}

	private void createSaveAttributes() throws Exception
	{
		WbsModule module = new WbsModule();

		AttributeCategory category = new AttributeCategory();
		category.setName("My Category");

		AttributeClassification  classification = new AttributeClassification();
		classification.setName("My AssetUsage");
		classification.setSelectType(SelectType.Multi);

		Attribute attribute1 = new Attribute();
		attribute1.setName("Production");
		attribute1.setInputType(InputType.Text);

		Attribute attribute2 = new Attribute();
		attribute2.setName("Consumption");
		attribute2.setInputType(InputType.Numeric);

		classification.setAttributes(new Attribute[] {attribute1, attribute2});

		category.setClassifications(new AttributeClassification[]{ classification });
		module.setAttributeCategories(new AttributeCategory[]{category});
		AttributeScope attrScope = new AttributeScope();   	    	          
		attrScope.setCategories(attrScope);
		attrScope.setClassifications(attrScope);
		attrScope.setAttributes(attrScope);

		WbsModuleScope moduleScope = new WbsModuleScope();
		moduleScope.setAttributeCategories(attrScope);    	

		module = (WbsModule)save(module, moduleScope).getRpmObject();
		
		assertNotNull(module);
		assertNotNull(module.getAttributeCategories());
		assertTrue(module.getAttributeCategories().length == 1);
		assertNotNull(module.getAttributeCategories()[0]);
		assertNotNull(module.getAttributeCategories()[0].getClassifications());
		assertTrue(module.getAttributeCategories()[0].getClassifications().length == 1);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0]);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0].getAttributes());
		assertTrue(module.getAttributeCategories()[0].getClassifications()[0].getAttributes().length == 2);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0].getAttributes()[0]);
		assertNotNull(module.getAttributeCategories()[0].getClassifications()[0].getAttributes()[1]);
		
		attribute = module.getAttributeCategories()[0].getClassifications()[0].getAttributes()[1];
	}
	
	private void createSaveAttributesAssignmentsForProjectAndTask() throws Exception
	{
		String xpath1 = "/AttributeCategory[name ='My Category']/" +
		"AttributeClassification/Attribute[name = 'Production']";    	
		RPMObject[] attrList = loadFromXpath(xpath1, null); //No attribute scope
		assertNotNull(attrList);
		assertTrue(attrList.length > 0);
		if (attrList != null)
		{
			Attribute loadedAttribute = (Attribute)attrList[0];
			assertNotNull(loadedAttribute);
			AttributeAssignment assignment = new AttributeAssignment();
			assignment.setAttribute(loadedAttribute);
			assignment.setValue("This is a default text value");

			project1.setAttributeAssignments(new AttributeAssignment[] {assignment});

			SaveResult saveRes2 =  save(project1, projectScope);        
			project1 =  (Project)saveRes2.getRpmObject();
			assertNotNull(project1);
			assertNotNull(project1.getAttributeAssignments());
			assertTrue(project1.getAttributeAssignments().length == 1);
			assertNotNull(project1.getAttributeAssignments()[0]);
			assertTrue(project1.getAttributeAssignments()[0].getValue().equals("This is a default text value"));

			// Get the summary task
			WorkElement[] workElements = project1.getChildren();

			for (int j = 0; j < workElements.length; j++)
			{    			
				if (workElements[j] instanceof SummaryTask )
				{   				
					assignment = new AttributeAssignment();
					assignment.setAttribute(loadedAttribute);
					assignment.setValue("This is a default text value");
					
					summaryTask = (SummaryTask)workElements[j];
					summaryTask.setAttributeAssignments(new AttributeAssignment[] {assignment}); 
					SaveResult saveRes = save(summaryTask, projectScope); 
					summaryTask = (SummaryTask) saveRes.getRpmObject();
					
					assertNotNull(summaryTask);
					assertNotNull(summaryTask.getAttributeAssignments());
					assertTrue(summaryTask.getAttributeAssignments().length == 1);
					assertNotNull(summaryTask.getAttributeAssignments()[0]);
					assertTrue(summaryTask.getAttributeAssignments()[0].getValue().equals("This is a default text value"));
				}   			
			}
		}

		String xpath2 = "/AttributeCategory[name ='My Category']/AttributeClassification/Attribute[name = 'Consumption']";   	
		RPMObject[] attrList2 = loadFromXpath(xpath2, null); //No attribute scope
		assertNotNull(attrList2);
		assertTrue(attrList2.length > 0);
		if (attrList2 != null)
		{
			Attribute loadedAttribute2 = (Attribute)attrList2[0];
			assertNotNull(loadedAttribute2);
			AttributeAssignment assignment2 = new AttributeAssignment();
			assignment2.setAttribute(loadedAttribute2);
			assignment2.setValue("10");

			// Get the summary task
			WorkElement[] workElements = project1.getChildren();

			for (int j = 0; j < workElements.length; j++)
			{    			
				if (workElements[j] instanceof SummaryTask )
				{   				
					summaryTask = (SummaryTask)workElements[j];    	    		   	    		
					if ( summaryTask.getChildren() != null)
					{
						WorkElement[] elements = summaryTask.getChildren();
						for (int k = 0; k < elements.length; k++)
						{
							if (elements[k] instanceof Task )
							{   				
								task = (Task)elements[k];
								task.setAttributeAssignments(new AttributeAssignment[] {assignment2});
								SaveResult saveRes = save(task, projectScope);
								task = (Task) saveRes.getRpmObject();
								
								assertNotNull(task);
								assertNotNull(task.getAttributeAssignments());
								assertTrue(task.getAttributeAssignments().length == 1);
								assertNotNull(task.getAttributeAssignments()[0]);
								assertTrue(task.getAttributeAssignments()[0].getValue().equals("10"));
							}
						}
					}
				}   			
			}
		}
	}
}