/*
 * Created on Dec 2, 2005 
 * @author mmatwiy 
 * Common validation tests for containers Task and Milestone.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.State;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.TaskType;

public class AbstractTestSimpleNodeValidation extends AbstractTestWorkElementValidation
{
    private Project parentProject;

    WorkElementScope scope;

    public void setUp() throws Exception
    {
        super.setUp();

        parentProject = (Project) WorkElementFixture.retrieveStaticFixture(
            new Project(), this);

        scope = new WorkElementScope();
        scope.setScheduleDates(true);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testConditionalReadOnlyActualDurationInMinutes(
            WorkElement workElement) throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setDurationInMinutes(new Integer(390));

        // actualDate.durationInMinutes read-only if planDate.startDate is null
        // or if planDate.durationInMinutes = 0; neither needs to be set

        // actualDate.durationInMinutes read-only if actualDate.startDate is
        // null
        actualDate.setStartDate(null);

        workElement.setActualDate(actualDate);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_VALUE_MISSING,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                    ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_CONDITIONAL_VALUE_ZERO,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                    ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_MISSING,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                    workElement.getID())
        };

        verifySaveMustFail(workElement, scope, null, expectedWarnings);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);
        
        WorkElement loadedWorkElement = (WorkElement)rpmObjects[0];
        Integer loadedDurationInMinutes = loadedWorkElement.getActualDate() == null ? null :
            loadedWorkElement.getActualDate().getDurationInMinutes();      
        
        // assert that the value we set was not saved in the db
        assertNotSame(new Integer(400), loadedDurationInMinutes);
    }

    public void testConditionalReadOnlyActualStart(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar(2005, 12, 12));

        // actualDate.startDate read-only if planDate.startDate is null or
        // if planDate.durationInMinutes is 0; neither needs to be set

        workElement.setActualDate(actualDate);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_VALUE_MISSING,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                    ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_CONDITIONAL_VALUE_ZERO,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                    ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                    workElement.getID())
        };

        verifySaveMustFail(workElement, scope, null, expectedWarnings);
    }

    public void testConditionalReadOnlyAssignedWorkExists(
            WorkElement workElement) throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(90));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(180));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar());
        actualDate.setDurationInMinutes(new Integer(90));

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(390));

        workElement.setProposedDate(proposedDate);
        workElement.setPlanDate(planDate);
        workElement.setActualDate(actualDate);
        workElement.setForecastDate(forecastDate);

        // specified fields read-only if resourceTaskAssignments
        // exist & contain a taskAssignment
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();
        resourceTaskAssignment.setTaskAssignment(new TaskAssignment());
        ResourceTaskAssignment[] resourceTaskAssignments =
        {
            resourceTaskAssignment
        };
        workElement.setResourceTaskAssignments(resourceTaskAssignments);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.READ_ONLY_ONCE_WORK_ASSIGNED,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_DURATION_FIELD,
                    workElement.getID()),
                createRPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.READ_ONLY_ONCE_WORK_ASSIGNED,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                    workElement.getID()),
                createRPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.READ_ONLY_ONCE_WORK_ASSIGNED,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                    workElement.getID()),
                createRPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.READ_ONLY_ONCE_WORK_ASSIGNED,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_DURATION_FIELD,
                    workElement.getID()),
                createRPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.READ_ONLY_ONCE_WORK_ASSIGNED,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_START_FIELD,
                    workElement.getID()),
                createRPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.READ_ONLY_ONCE_WORK_ASSIGNED,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                    workElement.getID()),
                createRPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.READ_ONLY_ONCE_WORK_ASSIGNED,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                    workElement.getID())
        };

        verifySaveMustFail(workElement, scope, null, expectedWarnings);
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);
        
        WorkElement loadedWorkElement = (WorkElement)rpmObjects[0];
        Integer loadedActualDuration = loadedWorkElement.getActualDate() == null ? null :
            loadedWorkElement.getActualDate().getDurationInMinutes();      
        Integer loadedForecastDuration = loadedWorkElement.getForecastDate() == null ? null :
            loadedWorkElement.getForecastDate().getDurationInMinutes();      
        Integer loadedPlanDuration = loadedWorkElement.getPlanDate() == null ? null :
            loadedWorkElement.getPlanDate().getDurationInMinutes();      
        Integer loadedProposedDuration = loadedWorkElement.getProposedDate() == null ? null :
            loadedWorkElement.getProposedDate().getDurationInMinutes();      
        Calendar loadedActualStart = loadedWorkElement.getActualDate() == null ? null :
            loadedWorkElement.getActualDate().getStartDate();
        Calendar loadedPlanStart = loadedWorkElement.getPlanDate() == null ? null :
            loadedWorkElement.getPlanDate().getStartDate();
        Calendar loadedProposedStart = loadedWorkElement.getProposedDate() == null ? null :
            loadedWorkElement.getProposedDate().getStartDate();
        
        // assert that the value we set was not saved in the db
        assertNotSame(new Integer(100), loadedActualDuration);
        assertNotSame(new Integer(400), loadedForecastDuration);
        assertNotSame(new Integer(200), loadedPlanDuration);
        assertNotSame(new Integer(100), loadedProposedDuration);
        
        assertNotSame(new GregorianCalendar(), loadedActualStart);
        assertNotSame(new GregorianCalendar(), loadedPlanStart);
        assertNotSame(new GregorianCalendar(), loadedProposedStart);
    }

    public void testSuccessConditionalReadOnlyAssignedParticipantsExist(
            WorkElement workElement) throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(90));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(180));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar());
        actualDate.setDurationInMinutes(new Integer(90));

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(390));

        workElement.setProposedDate(proposedDate);
        workElement.setPlanDate(planDate);
        workElement.setActualDate(actualDate);
        workElement.setForecastDate(forecastDate);

        // create a resourceTaskAssignment with no taskAssignment attached
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();
        //TODO: resourceTaskAssignment.setSecurityRole(")
        ResourceTaskAssignment[] resourceTaskAssignments =
        {
            resourceTaskAssignment
        };
        workElement.setResourceTaskAssignments(resourceTaskAssignments);

        // Attempt to save the object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, (WorkElement) rpmObjects[0],
            scope);

    }

    public void testConditionalReadOnlyDurationsSame(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(600));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(495));

        // actualDate.startDate is read-only if
        // actualDate.durationInMinutes = planDate.durationInMinutes
        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar(2005, 12, 12));
        actualDate.setDurationInMinutes(new Integer(495));

        // forecastDate.durationInMinutes is read-only if
        // actualDate.durationInMinutes = planDate.durationInMinutes
        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(180));

        workElement.setPlanDate(planDate);
        workElement.setActualDate(actualDate);
        workElement.setProposedDate(proposedDate);
        workElement.setForecastDate(forecastDate);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    ErrorCodes.READ_ONLY_CONDITIONAL_VALUES_SAME, workElement
                        .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_DURATION_FIELD,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                    workElement.getID()),
                createRPMException(
                    ErrorCodes.READ_ONLY_CONDITIONAL_VALUES_SAME, workElement
                        .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                    workElement.getID())
        };

        verifySaveMustFail(workElement, scope, null, expectedWarnings);
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);
        
        WorkElement loadedWorkElement = (WorkElement)rpmObjects[0];
        Integer loadedForecastDuration = loadedWorkElement.getForecastDate() == null ? null :
            loadedWorkElement.getForecastDate().getDurationInMinutes();      
        Calendar loadedActualStart = loadedWorkElement.getActualDate() == null ? null :
            loadedWorkElement.getActualDate().getStartDate();
        
        // assert that the value we set was not saved in the db
        assertNotSame(new Integer(400), loadedForecastDuration);
        assertNotSame(new GregorianCalendar(2005, 12, 12), loadedActualStart);
    }

    public void testConditionalReadOnlyForecastDurationInMinutes(
            WorkElement workElement) throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(180));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(600));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(390));

        // forecastDate.durationInMinutes read-only if
        // actualDate.startDate is null

        workElement.setProposedDate(proposedDate);
        workElement.setPlanDate(planDate);
        workElement.setForecastDate(forecastDate);

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(ErrorCodes.READ_ONLY_VALUE_MISSING, workElement
                .getClass().getName(),
                ValidationConstants.SCHEDULE_DATE_FORECAST_DURATION_FIELD,
                ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                workElement.getID())
        };

        verifySaveMustFail(workElement, scope, null, expectedWarnings);
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);
        
        WorkElement loadedWorkElement = (WorkElement)rpmObjects[0];
        Integer loadedForecastDuration = loadedWorkElement.getForecastDate() == null ? null :
            loadedWorkElement.getForecastDate().getDurationInMinutes();      
        
        // assert that the value we set was not saved in the db
        assertNotSame(new Integer(200), loadedForecastDuration);
    }

    public void testConditionalReadOnlyPlanDurationInMinutes(
            WorkElement workElement) throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setDurationInMinutes(new Integer(4995));

        // planDate.durationInMinutes read-only when planDate.startDate
        // is null

        workElement.setPlanDate(planDate);

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(ErrorCodes.READ_ONLY_VALUE_MISSING, workElement
                .getClass().getName(),
                ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD, workElement
                    .getID())
        };

        verifySaveMustFail(workElement, scope, null, expectedWarnings);
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);
        
        WorkElement loadedWorkElement = (WorkElement)rpmObjects[0];
        Integer loadedPlanDuration = loadedWorkElement.getPlanDate() == null ? null :
            loadedWorkElement.getPlanDate().getDurationInMinutes();      
        
        // assert that the value we set was not saved in the db
        assertNotSame(new Integer(5000), loadedPlanDuration);
    }

    public void testConditionalReadOnlyPlanStartAndDuration(
            WorkElement workElement) throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar(2006, 3, 5));
        proposedDate.setDurationInMinutes(new Integer(390));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar(2006, 3, 6));
        planDate.setDurationInMinutes(new Integer(4995));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar(2006, 3, 7));
        actualDate.setDurationInMinutes(new Integer(3990));

        workElement.setProposedDate(proposedDate);
        workElement.setPlanDate(planDate);
        workElement.setActualDate(actualDate);

        // Attempt to save the object object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        WorkElement loadedWorkElement = (WorkElement) rpmObjects[0];

        // planDate.durationInMinutes and planDate.startDate read-only
        // when actualDate.startDate has a value
        loadedWorkElement.getPlanDate().setStartDate(new GregorianCalendar());
        loadedWorkElement.getPlanDate().setDurationInMinutes(new Integer(390));

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    loadedWorkElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                    loadedWorkElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    loadedWorkElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_START_FIELD,
                    loadedWorkElement.getID())
        };

        verifySaveMustFail(loadedWorkElement, scope, null, expectedWarnings);
        
        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        WorkElement reloadedWorkElement = (WorkElement)rpmObjects[0];

        Integer loadedPlanDuration = reloadedWorkElement.getPlanDate() == null ? null :
            reloadedWorkElement.getPlanDate().getDurationInMinutes();      
        Calendar loadedPlanStart = reloadedWorkElement.getPlanDate() == null ? null :
            reloadedWorkElement.getPlanDate().getStartDate();
        
        // assert that the value we set was not saved in the db
        assertNotSame(new Integer(5000), loadedPlanDuration);
        assertNotSame(new GregorianCalendar(2006, 3, 6), loadedPlanStart);

    }

    public void testConditionalReadOnlyProposedStartAndDuration(
            WorkElement workElement) throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(90));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar(2005, 12, 12));

        workElement.setProposedDate(proposedDate);
        workElement.setPlanDate(planDate);

        // save the workElement so that the above fields aren't set as modified
        workElement = (WorkElement) save(workElement, scope).getRpmObject();

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        WorkElement loadedWorkElement = (WorkElement) rpmObjects[0];

        WbsScheduleDate proposedDate2 = new WbsScheduleDate();
        proposedDate2.setStartDate(new GregorianCalendar());
        proposedDate2.setDurationInMinutes(new Integer(1995));

        loadedWorkElement.setProposedDate(proposedDate2);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    loadedWorkElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_DURATION_FIELD,
                    ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                    loadedWorkElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
                    loadedWorkElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_START_FIELD,
                    ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                    loadedWorkElement.getID())
        };

        verifySaveMustFail(loadedWorkElement, scope, null, expectedWarnings);
        
        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        WorkElement reloadedWorkElement = (WorkElement)rpmObjects[0];

        Integer loadedProposedDuration = reloadedWorkElement.getProposedDate() == null ? null :
            reloadedWorkElement.getProposedDate().getDurationInMinutes();      
        Calendar loadedProposedStart = reloadedWorkElement.getProposedDate() == null ? null :
            reloadedWorkElement.getProposedDate().getStartDate();
        
        // assert that the value we set was not saved in the db
        assertNotSame(new Integer(2000), loadedProposedDuration);
        assertNotSame(new GregorianCalendar(), loadedProposedStart);
    }

    public void testFailureClearPlanStart(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar(2006, 3, 6));
        planDate.setDurationInMinutes(new Integer(4995));

        workElement.setPlanDate(planDate);

        // on update, planDate.startDate cannot be cleared if it
        // previously contained a value

        // Attempt to save the object object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        WorkElement loadedWorkElement = (WorkElement) rpmObjects[0];

        loadedWorkElement.getPlanDate().setStartDate(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, loadedWorkElement
                .getClass().getName(),
                ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                loadedWorkElement.getID())
        };

        verifySaveMustFail(loadedWorkElement, scope, expectedExceptions);
    }

    public void testFailureClearProposedStart(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // proposedDate.startDate will be automatically set

        // Attempt to save the object object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        WorkElement loadedWorkElement = (WorkElement) rpmObjects[0];

        assertNotNull(loadedWorkElement.getProposedDate());
        loadedWorkElement.getProposedDate().setStartDate(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, loadedWorkElement
                .getClass().getName(),
                ValidationConstants.SCHEDULE_DATE_PROPOSED_START_FIELD,
                loadedWorkElement.getID())
        };

        verifySaveMustFail(loadedWorkElement, scope, expectedExceptions);
    }

    public void testFailureInvalidEstimatedFinishDate(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set the estimatedFinishDate to be prior to the estimatedStartDate
        workElement.setEstimatedStartDate(new GregorianCalendar(2005, 12, 12));
        workElement.setEstimatedFinishDate(new GregorianCalendar(2005, 12, 11));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_DATES, workElement.getClass()
                .getName(), ValidationConstants.ESTIMATED_START_DATE_FIELD,
                workElement.getID())
        };

        verifySaveMustFail(workElement, null, expectedExceptions);
    }

    public void testFailureInvalidState(WorkElement workElement)
            throws AssertionError, Exception
    {
        scope = new WorkElementScope();
        scope.setState(true);
        
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        workElement.setState(new State());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, workElement.getClass()
                .getName(), ValidationConstants.STATE_FIELD, workElement
                .getID())
        };

        verifySaveMustFail(workElement, scope, expectedExceptions);
    }

    public void testFailureInvalidTimeCodes(WorkElement workElement)
            throws AssertionError, Exception
    {
        scope = new WorkElementScope();
        scope.setTimeCodes(true);
        
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // assign the timecodes
        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
        TimeCode timecode1 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, true, this);
        TimeCode timecode2 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, false, this);

        workElement.setTimeCode1(timecode2);
        workElement.setTimeCode2(timecode1);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, workElement
                        .getClass().getName(),
                    ValidationConstants.TIME_CODE1_FIELD, workElement.getID()),
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, workElement
                        .getClass().getName(),
                    ValidationConstants.TIME_CODE2_FIELD, workElement.getID()),
        };

        verifySaveMustFail(workElement, scope, expectedExceptions);
    }

    public void testFailureValidationMax(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);

        // Set string values with max length + 1 and verify that they are not
        // accepted
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT + 1));
        workElement
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT + 1));
        workElement
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_ACTIVITY_REFERENCE_NUMBER_CHARACTER_LIMIT + 1));

        // Set integers to max value + 1 and verify that they are not accepted
        workElement.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, workElement
                    .getClass().getName(), ValidationConstants.NAME_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, workElement
                    .getClass().getName(), ValidationConstants.PRIORITY_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, workElement
                    .getClass().getName(),
                    ValidationConstants.REFERENCE_NUMBER_FIELD, workElement
                        .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, workElement
                    .getClass().getName(),
                    ValidationConstants.EXTERNAL_ID_FIELD, workElement.getID())

        };

        verifySaveMustFail(workElement, null, expectedExceptions);
    }

    public void testFailureValidationMin(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set integers to min value -1 and verify that they are not accepted
        workElement.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, workElement.getClass()
                .getName(), ValidationConstants.PRIORITY_FIELD, workElement.getID()),
        };

        verifySaveMustFail(workElement, null, expectedExceptions);
    }

    public void testFailureCompareDurations(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar(2006, 3, 5));
        proposedDate.setDurationInMinutes(new Integer(400));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar(2006, 3, 6));
        planDate.setDurationInMinutes(new Integer(5000));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar(2006, 3, 7));
        actualDate.setDurationInMinutes(new Integer(6000));

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(5000));

        workElement.setProposedDate(proposedDate);
        workElement.setPlanDate(planDate);
        workElement.setActualDate(actualDate);
        workElement.setForecastDate(forecastDate);

        // actualDate.durationInMinutes must be <= planDate.durationInMinutes
        // forecastDate.durationInMinutes must be >=
        // actualDate.durationInMinutes

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.LESS_THAN_OR_EQUAL, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                    ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.GREATER_THAN_OR_EQUAL_FIELD,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_DURATION_FIELD,
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                    workElement.getID()),

        };

        verifySaveMustFail(workElement, scope, expectedExceptions);
    }

    public void testInsertScheduleDatesSuccess(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        //        WbsScheduleDate proposedDate = new WbsScheduleDate();
        //        proposedDate.setStartDate(new GregorianCalendar(2006, 3, 5));
        //        proposedDate.setDurationInMinutes(new Integer(400));

        // proposed dates are automatically set

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar(2006, 3, 6));
        planDate.setDurationInMinutes(new Integer(4995));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar(2006, 3, 7));
        actualDate.setDurationInMinutes(new Integer(3990));

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(6990));

        workElement.setPlanDate(planDate);
        workElement.setActualDate(actualDate);
        workElement.setForecastDate(forecastDate);

        // Attempt to save the object object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        WorkElement loadedWorkElement = (WorkElement) rpmObjects[0];

        assertNotNull(loadedWorkElement.getPlanDate());
        assertNotNull(loadedWorkElement.getActualDate());
        // As milestones cannot have the forecast dates set since nobody works on them (cannot fill timesheet)
        // Also, we cannot touch the forecast dates from the WBS view of the client for either the Task ot Milestone
        if(!(workElement instanceof Milestone || workElement instanceof Task)){
        	assertNotNull(loadedWorkElement.getForecastDate());
        }
        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, loadedWorkElement, scope);

    }

    public void testInsertSuccess(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));
        workElement.setEstimatedStartDate(new GregorianCalendar(2005, 12, 11));
        workElement.setEstimatedFinishDate(new GregorianCalendar(2006, 12, 12));
        workElement
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_ACTIVITY_REFERENCE_NUMBER_CHARACTER_LIMIT));

        // Attempt to save the object object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, (WorkElement) rpmObjects[0],
            scope);
    }

    public void testInsertWithInvalidParent(WorkElement workElement)
            throws AssertionError, Exception
    {
        // create and set invalid parent
        Task parentTask = (Task) WorkElementFixture.retrieveStaticFixture(
            new Task(), this);

        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        workElement.setParent(parentTask);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, workElement
                .getClass().getName(), ValidationConstants.PARENT, workElement
                .getID())
        };

        verifySaveMustFail(workElement, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields(WorkElement workElement)
            throws AssertionError, Exception
    {
        workElement.setParent(parentProject);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, workElement.getClass()
                .getName(), ValidationConstants.NAME_FIELD, workElement.getID())
        };

        verifySaveMustFail(workElement, null, expectedExceptions);
    }

    public void testInsertWithoutParent(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        workElement.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, workElement.getClass()
                .getName(), ValidationConstants.PARENT, workElement.getID())
        };

        verifySaveMustFail(workElement, null, expectedExceptions);

    }

    public void testInsertWithParentDeliverable(WorkElement workElement)
            throws AssertionError, Exception
    {
        Deliverable parentDeliverable = (Deliverable) WorkElementFixture
            .retrieveStaticFixture(new Deliverable(), this);

        // Set the mandatory fields
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        workElement.setParent(parentDeliverable);

        // Attempt to save the WorkElement object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, null);

        // Load the inserted milestone from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, (WorkElement) rpmObjects[0],
            null);
    }

    public void testInsertWithParentProject(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        workElement.setParent(parentProject);

        // Attempt to save the WorkElement object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, null);

        // Load the inserted milestone from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, (WorkElement) rpmObjects[0],
            null);
    }

    public void testInsertWithParentProposal(WorkElement workElement)
            throws AssertionError, Exception
    {
        Proposal parentProposal = (Proposal) WorkElementFixture
            .retrieveStaticFixture(new Proposal(), this);

        // Set the mandatory fields
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        workElement.setParent(parentProposal);

        // Attempt to save the WorkElement object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, null);

        // Load the inserted milestone from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, (WorkElement) rpmObjects[0],
            null);
    }

    public void testInsertWithParentSummaryTask(WorkElement workElement)
            throws AssertionError, Exception
    {

        SummaryTask summaryTask = (SummaryTask) WorkElementFixture
            .retrieveStaticFixture(new SummaryTask(), this);

        // Set the mandatory fields
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        workElement.setParent(summaryTask);

        // Attempt to save the WorkElement object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, null);

        // Load the inserted milestone from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, (WorkElement) rpmObjects[0],
            null);
    }

    public void testInsertWithParentWorkOrganization(WorkElement workElement)
            throws AssertionError, Exception
    {

        WorkOrganization workOrganization = (WorkOrganization) WorkElementFixture
            .retrieveStaticFixture(new WorkOrganization(), this);

        // Set the mandatory fields
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        workElement.setParent(workOrganization);

        // Attempt to save the WorkElement object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, null);

        // Load the inserted milestone from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, (WorkElement) rpmObjects[0],
            null);
    }

    public void testInsertWithParentWorkProduct(WorkElement workElement)
            throws AssertionError, Exception
    {
        WorkProduct workProduct = (WorkProduct) WorkElementFixture
            .retrieveStaticFixture(new WorkProduct(), this);

        // Set the mandatory fields
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        workElement.setParent(workProduct);

        // Attempt to save the WorkElement object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, null);

        // Load the inserted milestone from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, (WorkElement) rpmObjects[0],
            null);
    }

    public void testReadOnlyActualDateFields(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setFinishDate(new GregorianCalendar());
        actualDate.setHours(new Double(3.3));
        actualDate.setDays(new Double(3));
        actualDate.setWorkHours(new Double(3.3));

        workElement.setActualDate(actualDate);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_FINISH_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_HOURS_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_DAYS_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_WORK_HOURS_FIELD,
                    workElement.getID())
        };

        verifySaveMustFail(workElement, scope, null, expectedWarnings);
    }

    public void testReadOnlyForecastDateFields(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setStartDate(new GregorianCalendar());
        forecastDate.setFinishDate(new GregorianCalendar());
        forecastDate.setHours(new Double(3.3));
        forecastDate.setDays(new Double(3));
        forecastDate.setWorkHours(new Double(3.3));

        workElement.setForecastDate(forecastDate);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_FINISH_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_START_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_HOURS_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_DAYS_FIELD,
                    workElement.getID()),
                createRPMException(
                    ErrorCodes.READ_ONLY_FIELD,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_WORK_HOURS_FIELD,
                    workElement.getID())
        };

        verifySaveMustFail(workElement, scope, null, expectedWarnings);
    }

    public void testReadOnlyPlanDateFields(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setFinishDate(new GregorianCalendar());
        planDate.setHours(new Double(3.3));
        planDate.setDays(new Double(3));
        planDate.setWorkHours(new Double(3.3));

        workElement.setPlanDate(planDate);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_FINISH_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_HOURS_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_DAYS_FIELD,
                    workElement.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_WORK_HOURS_FIELD,
                    workElement.getID())
        };

        verifySaveMustFail(workElement, scope, null, expectedWarnings);
    }

    public void testReadOnlyProposedDateFields(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setFinishDate(new GregorianCalendar());
        proposedDate.setHours(new Double(3.3));
        proposedDate.setDays(new Double(3));
        proposedDate.setWorkHours(new Double(3.3));

        workElement.setProposedDate(proposedDate);

        RPMExceptionMock[] expectedWarnings =
        {
//                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
//                    .getClass().getName(),
//                    ValidationConstants.SCHEDULE_DATE_PROPOSED_FINISH_FIELD,
//                    workElement.getID()),
//                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
//                    .getClass().getName(),
//                    ValidationConstants.SCHEDULE_DATE_PROPOSED_HOURS_FIELD,
//                    workElement.getID()),
//                createRPMException(ErrorCodes.READ_ONLY_FIELD, workElement
//                    .getClass().getName(),
//                    ValidationConstants.SCHEDULE_DATE_PROPOSED_DAYS_FIELD,
//                    workElement.getID()),
                createRPMException(
                    ErrorCodes.READ_ONLY_FIELD,
                    workElement.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_WORK_HOURS_FIELD,
                    workElement.getID())
        };

        verifySaveMustFail(workElement, scope, null, expectedWarnings);
    }

    public void testSuccessValidationMax(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);

        // Set field values with maxLength and verify that they are accepted
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));
        workElement
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT));
        workElement
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_ACTIVITY_REFERENCE_NUMBER_CHARACTER_LIMIT));

        // Set integers to max value and verify that they are accepted
        workElement.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));

        // Attempt to save the WorkElement object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, (WorkElement) rpmObjects[0],
            null);
    }

    public void testSuccessValidationMin(WorkElement workElement)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set integers to min value and verify that they are accepted
        workElement.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));

        // Attempt to save the WorkElement object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded WorkElement is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, (WorkElement) rpmObjects[0],
            null);
    }

    public void testSuccessValidState(WorkElement workElement)
            throws AssertionError, Exception
    {
        scope = new WorkElementScope();
        scope.setState(true);
        
        // Set the mandatory fields
        workElement.setParent(parentProject);
        workElement
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        workElement.setState(DatafieldFixture.createState(workElement));

        // Attempt to save the object
        workElement = (WorkElement) saveMustSucceed(workElement,
            WorkElement.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkElement[id='"
                + workElement.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkElement);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(workElement, (WorkElement) rpmObjects[0],
            scope);
    }
    
    public void testFailureEffortDrivenWithDefaultTask(SimpleNode simpleNode)
            throws AssertionError, Exception
    {
        // Set the mandatory fields
        simpleNode.setParent(parentProject);
        simpleNode
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // effortDriven cannot be true when taskType is DefaultTask
        simpleNode.setEffortDriven(Boolean.TRUE);
        simpleNode.setTaskType(TaskType.DefaultTask);

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.EFFORT_DRIVEN_NOT_VALID_WITH_DEFAULT_TASK,
                simpleNode.getClass().getName(),
                ValidationConstants.EFFORT_DRIVEN_FIELD, simpleNode.getID())
        };

        verifySaveMustFail(simpleNode, null, null, expectedWarnings);
    }
}