/**
 * 
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Contour;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.types.ContourType;

/**
 * @author abhay
 *
 */
public class TestContourValidation extends TestResourceTaskAssignmentValidation {

	public void setUp() throws Exception
	{
		super.setUp();
		rtaScope.setResource(resourceScope);
		rtaScope.setTaskAssignment(new TaskAssignmentScope());
		rtaScope.setParent(workElementScope);
	}

	public void testValidateContourASAPForWork() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.ASAP);
		contour.setStartDate(Calendar.getInstance());
		contour.setFinishDate(Calendar.getInstance());
		contour.setWork(new Double(4));
		resourceTask.setContour(contour);

		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.WORK_MUST_BE_NULL_WHEN_CONTOUR_TYPE_IS_ASAP,
						resourceTask.getClass().getName(),
						ValidationConstants.CONTOUR_FIELD, resourceTask
						.getID())
		                                                            });
	}

	public void testValidateContourASAPMissingStartDate() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.ASAP);
		contour.setFinishDate(Calendar.getInstance());
		resourceTask.setContour(contour);

		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.START_DATE_MISSING_FOR_CONTOUR,
						resourceTask.getClass().getName(),
						ValidationConstants.MANDATORy_FIELD, resourceTask
						.getID())
		                                                            });
	}

	//	Validating that Effort and FinishDate are mutually exclusive
	public void testValidateMutuallyExclusionForASAP() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.ASAP);
		contour.setStartDate(Calendar.getInstance());
		contour.setFinishDate(Calendar.getInstance());
		contour.setEffortInMinutes(new Integer(2));
		resourceTask.setContour(contour);

		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.MUTUALLY_EXCLUSIVE_EFFORT_AND_FINISH_DATE,
						resourceTask.getClass().getName(),
						ValidationConstants.CONTOUR_FIELD, resourceTask
						.getID())
		                                                            });
	}

	public void testValidateMutuallyExclusionForASAP2() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.ASAP);
		contour.setStartDate(Calendar.getInstance());
		resourceTask.setContour(contour);

		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.MUTUALLY_EXCLUSIVE_EFFORT_AND_FINISH_DATE,
						resourceTask.getClass().getName(),
						ValidationConstants.CONTOUR_FIELD, resourceTask
						.getID())
		                                                            });
	}
	
	public void testValidateContourHoursPerDayMissingStartDate() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.HoursPerDay);
		contour.setWork(new Double(4));
		contour.setFinishDate(Calendar.getInstance());
		resourceTask.setContour(contour);

		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.START_DATE_MISSING_FOR_CONTOUR,
						resourceTask.getClass().getName(),
						ValidationConstants.MANDATORy_FIELD, resourceTask
						.getID())
		                                                            });
	}

	public void testValidateContourHoursPerDayInvalidFinishDate() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.HoursPerDay);
		contour.setFinishDate(Calendar.getInstance());
		Calendar startDate = Calendar.getInstance();
		startDate.add(Calendar.DAY_OF_YEAR, 1); // so that finish date is before start date
		contour.setStartDate(startDate);
		contour.setEffortInMinutes(new Integer(0));
		contour.setWork(new Double(4));
		resourceTask.setContour(contour);

		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.FINISH_DATE_BEFORE_START_DATE,
						resourceTask.getClass().getName(),
						ValidationConstants.CONTOUR_FIELD, resourceTask
						.getID())
		                                                            });
	}

	public void testValidateInvalidFinishDateForASAP() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.ASAP);
		contour.setFinishDate(Calendar.getInstance());
		Calendar startDate = Calendar.getInstance();
		startDate.add(Calendar.DAY_OF_YEAR, 1); // so that finish date is before start date
		contour.setStartDate(startDate);
		resourceTask.setContour(contour);

		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.FINISH_DATE_BEFORE_START_DATE,
						resourceTask.getClass().getName(),
						ValidationConstants.CONTOUR_FIELD, resourceTask
						.getID())
		                                                            });
	}

	public void testValidateWithoutTaskAssignment() throws Exception
	{
		rtaScope.setTaskAssignment(null);		//task assignment should be null
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.HoursPerDay);
		Calendar startDate = Calendar.getInstance();
		contour.setStartDate(startDate);
		contour.setFinishDate(Calendar.getInstance());
		contour.setEffortInMinutes(new Integer(0));
		resourceTask.setContour(contour);
		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.CONTOUR_APPLICABLE_ONLY_FOR_EFFORT_ASSIGNMENT,
						resourceTask.getClass().getName(),
						ValidationConstants.CONTOUR_FIELD, resourceTask
						.getID())
		                                                            });
	}

	public void testValidateWithoutContourSet() throws Exception
	{
//		Let Contour be null	
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		resourceTask.getTaskAssignment().setProposedDurationHours(null);
		saveMustSucceed(resourceTask, resourceTask.getClass(), null);
	}

	public void testValidateContoursDuringUpdate() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		SingleResult sr = save(resourceTask, rtaScope);
		if(sr.getRpmObject()!=null)
		{
			resourceTask = (ResourceTaskAssignment)sr.getRpmObject();
		};
		Contour contour = new Contour();
		contour.setContourType(ContourType.ASAP);
		contour.setFinishDate(Calendar.getInstance());
		Calendar startDate = Calendar.getInstance();
		contour.setStartDate(startDate);
		resourceTask.setContour(contour);
		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.CANNOT_ASSIGN_CONTOUR_DURING_UPDATE_OF_TASK_ASSIGNMENTS,
						resourceTask.getClass().getName(),
						ValidationConstants.CONTOUR_FIELD, null)
		                                                            });
	}


	public void testValidateMutuallyExclusionWhenNotASAP() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.HoursPerDay);
		contour.setStartDate(Calendar.getInstance());
		contour.setFinishDate(Calendar.getInstance());
		contour.setEffortInMinutes(new Integer(2));
		resourceTask.setContour(contour);

		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.MUTUALLY_EXCLUSIVE_EFFORT_FINISH_DATE_AND_OCCURRENCES,
						resourceTask.getClass().getName(),
						ValidationConstants.CONTOUR_FIELD, resourceTask
						.getID())
		                                                            });
	}

	public void testValidateMutuallyExclusionWhenNotASAP2() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.HoursPerDay);
		contour.setStartDate(Calendar.getInstance());
		contour.setOccurrences(new Integer(2));
		contour.setEffortInMinutes(new Integer(2));
		resourceTask.setContour(contour);

		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.MUTUALLY_EXCLUSIVE_EFFORT_FINISH_DATE_AND_OCCURRENCES,
						resourceTask.getClass().getName(),
						ValidationConstants.CONTOUR_FIELD, resourceTask
						.getID())
		                                                            });
	}

	public void testValidateMutuallyExclusionWhenNotASAP3() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.HoursPerDay);
		contour.setStartDate(Calendar.getInstance());
		contour.setOccurrences(new Integer(2));
		contour.setFinishDate(Calendar.getInstance());
		resourceTask.setContour(contour);

		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.MUTUALLY_EXCLUSIVE_EFFORT_FINISH_DATE_AND_OCCURRENCES,
						resourceTask.getClass().getName(),
						ValidationConstants.CONTOUR_FIELD, resourceTask
						.getID())
		                                                            });
	}
	
	public void testInvalidContourType() throws Exception
	{
		resourceTask = ResourceTaskAssignmentFixture.createFixture(rtaScope, false, this);
		Contour contour = new Contour();
		contour.setContourType(ContourType.HoursPerMonth);
		resourceTask.setContour(contour);
		
		verifySaveMustFail(resourceTask, null, new RPMExceptionMock[]
		                                                            {
				createRPMException(ErrorCodes.UNSUPPORTED_CONTOUR,
						resourceTask.getClass().getName(),
						ValidationConstants.CONTOUR_FIELD, resourceTask
						.getID())
		                                                            });
	}
	
}
