package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.EarnedValues;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author rbadette
 */
public class TestEarnedValuesValidation extends GenericValidation
{
    private WorkElementScope mainScope = null;

    private Proposal proposal = null;

    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = new WorkElementScope();
        mainScope.setEarnedValues(true);

        proposal = (Proposal) WorkElementFixture.createFixture(new Proposal(),
            mainScope, this);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    public void testReadOnlyEVActivationState() throws AssertionError,
            Exception
    {
        // Basic project scoping
        mainScope.setParent(new WorkElementScope());
        mainScope.setContainingProject(new WorkElementScope());

        Calendar startDate = Calendar.getInstance();
        startDate.set(2007, 0, 1);

        Calendar finishDate = Calendar.getInstance();
        finishDate.set(2007, 0, 5);

        // Creating Project
        Project evProject = (Project) WorkElementFixture.createSaveFixture(
            new Project(), mainScope, this);

        evProject.setEstimatedStartDate(startDate);
        evProject.setEstimatedFinishDate(finishDate);
        evProject.setEarnedValuesCalculationState(new Boolean(true));

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, evProject.getClass()
                .getName(),
                ValidationConstants.EV_CALCULATION_ACTIVATION_STATE, evProject
                    .getID())
        };

        verifySaveMustFail(evProject, mainScope, null, expectedWarnings);
    }

    public void testLoadProposalWithEarnedValuesModifyAndSave()
            throws AssertionError, Exception
    {
        getApplicationInterface().save(sessionid, proposal, mainScope,
            ReloadType.ReloadResult);

        SingleResult testResult = getApplicationInterface().loadFromID(
            sessionid, proposal, mainScope);
        assertNotNull(testResult);
        assertTrue(testResult.isSuccessful());

        proposal = (Proposal) testResult.getRpmObject();
        assertNotNull(proposal);
        assertNotNull(proposal.getEarnedValues());

        proposal.getEarnedValues().setCriticalRatio_C(new Double(777));

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, proposal.getClass()
                .getName(), ValidationConstants.EARNED_VALUE_FIELD, proposal
                .getID())
        };

        verifySaveMustFail(proposal, mainScope, expectedError, null);
    }

    public void testReadOnlyWithNormalFieldEVModified() throws AssertionError,
            Exception
    {
        proposal.setEarnedValues(new EarnedValues());
        proposal.getEarnedValues().setActualCostWorkPerf(new Double(12));

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, proposal.getClass()
                .getName(), ValidationConstants.EARNED_VALUE_FIELD, proposal
                .getID())
        };
        
        verifySaveMustFail(proposal, mainScope, expectedError, null);
    }

    public void testReadOnlyWithCalculatedFieldEVModified()
            throws AssertionError, Exception
    {
        proposal.setEarnedValues(new EarnedValues());
        proposal.getEarnedValues().setCriticalRatio_C(new Double(21));

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, proposal.getClass()
                .getName(), ValidationConstants.EARNED_VALUE_FIELD, proposal
                .getID())
        };

        verifySaveMustFail(proposal, mainScope, expectedError, null);
    }

    public void testSaveProposalEarnedValuesWithEVObject()
            throws AssertionError, Exception
    {
        proposal.setEarnedValues(new EarnedValues());

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, proposal.getClass()
                .getName(), ValidationConstants.EARNED_VALUE_FIELD, proposal
                .getID())
        };

        verifySaveMustFail(proposal, mainScope, expectedError, null);
    }

    public void testInvalidTypeForProjectEVCalculation() throws AssertionError,
            Exception
    {
        Deliverable inputProject = (Deliverable) WorkElementFixture
            .createSaveFixture(new Deliverable(), new WorkElementScope(), this);

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, inputProject.getClass()
                .getName(), ValidationConstants.PROJECTS_FIELD, inputProject
                .getID())
        };

        SingleResult result = null;
        result = getApplicationInterface().enableProjectEVCalculation(
            sessionid, inputProject, null);
        verifyFailure(inputProject, expectedError, null, result);
    }

    public void testInvalidTypeForSavingProjectBaseline()
            throws AssertionError, Exception
    {
        ChangeRequest inputProject = ScopeElementFixture
            .createSaveChangeRequest(this);

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, inputProject.getClass()
                .getName(), ValidationConstants.PROJECTS_FIELD, inputProject
                .getID())
        };

        SingleResult result = null;
        result = getApplicationInterface().saveProjectBaseline(sessionid,
            inputProject, null);
        verifyFailure(inputProject, expectedError, null, result);
    }

    public void testInvalidTypeForRollupProjectEV() throws AssertionError,
            Exception
    {
        Deliverable inputProject = (Deliverable) WorkElementFixture
            .createSaveFixture(new Deliverable(), new WorkElementScope(), this);

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, inputProject.getClass()
                .getName(), ValidationConstants.PROJECTS_FIELD, inputProject
                .getID())
        };

        SingleResult result = null;
        result = getApplicationInterface().rollupProjectEV(sessionid,
            inputProject, null);
        verifyFailure(inputProject, expectedError, null, result);
    }

    public void testInvalidInputForProjectEVCalculation()
            throws AssertionError, Exception
    {
        Project inputProject = new Project();

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.NO_ID, inputProject.getClass()
                .getName(), ValidationConstants.PROJECTS_FIELD, inputProject
                .getID())
        };

        SingleResult result = null;
        result = getApplicationInterface().enableProjectEVCalculation(
            sessionid, inputProject, null);
        verifyFailure(inputProject, expectedError, null, result);
    }

    public void testInvalidInputForSavingProjectBaseline()
            throws AssertionError, Exception
    {
        Project inputProject = new Project();

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.NO_ID, inputProject.getClass()
                .getName(), ValidationConstants.PROJECTS_FIELD, inputProject
                .getID())
        };

        SingleResult result = null;
        result = getApplicationInterface().saveProjectBaseline(sessionid,
            inputProject, null);
        verifyFailure(inputProject, expectedError, null, result);
    }

    public void testInvalidInputForRollupProjectEV() throws AssertionError,
            Exception
    {
        Project inputProject = new Project();

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.NO_ID, inputProject.getClass()
                .getName(), ValidationConstants.PROJECTS_FIELD, inputProject
                .getID())
        };

        SingleResult result = null;
        result = getApplicationInterface().rollupProjectEV(sessionid,
            inputProject, null);
        verifyFailure(inputProject, expectedError, null, result);
    }

    public void testNullInputForRollupProjectEV() throws AssertionError,
            Exception
    {
        Project inputProject = null;

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.OBJECT_IS_NULL,
                ValidationConstants.PROJECTS_FIELD)
        };

        SingleResult result = null;
        result = getApplicationInterface().rollupProjectEV(sessionid,
            inputProject, null);

        verifyExceptions(expectedError, result.getErrors());
    }

    public void testNullInputSavingProjectBaseline() throws AssertionError,
            Exception
    {
        Project inputProject = null;

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.OBJECT_IS_NULL,
                ValidationConstants.PROJECTS_FIELD)
        };

        SingleResult result = null;
        result = getApplicationInterface().saveProjectBaseline(sessionid,
            inputProject, null);

        verifyExceptions(expectedError, result.getErrors());
    }

    public void testNullInputForEnableEVCalculation() throws AssertionError,
            Exception
    {
        Project inputProject = null;

        RPMExceptionMock[] expectedError =
        {
            createRPMException(ErrorCodes.OBJECT_IS_NULL,
                ValidationConstants.PROJECTS_FIELD)
        };

        SingleResult result = null;
        result = getApplicationInterface().enableProjectEVCalculation(
            sessionid, inputProject, null);

        verifyExceptions(expectedError, result.getErrors());
    }

    public void testactivateEVWeekDelaysWithInvalidWeek()
            throws AssertionError, Exception
    {
        String errorParams =

        "EV Delay (weeks)"
                + String.valueOf(ValidationConstants.EV_DELAY_DAYS_MIN)
                + String.valueOf(ValidationConstants.EV_DELAY_DAYS_MAX)
                + String.valueOf(9);

        RPMExceptionMock[] expectedWarning =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, errorParams)
        };

        Result result = null;
        result = getApplicationInterface().activateEVWeekDelays(sessionid,
            ValidationConstants.EV_DELAY_DAYS_MAX + 1);

        verifyExceptions(expectedWarning, result.getWarnings());
    }

}
