package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.GregorianCalendar;

import junit.framework.Assert;

import com.ibm.rpm.applicationadministration.containers.AssetFinancialCategory;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.CurrencyFixture;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ProjectCostCenter;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeCategory;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * @author rbadette
 * This class makes sure that all Validation errors and warnings are
 * generated in the right condition when creating a project with a template.
 */
public class TestGenericProjectWithTemplateValidation extends GenericValidation
{ 
    private WorkElementScope scope = null;

    private GregorianCalendar calendar = null;

    private WbsScheduleDate wbsDate = null;

    private TimeCode timeCode1 = null;

    private TimeCode timeCode2 = null;

    private RPMCalendar rpmCalendar = null;

    private Portfolio portFolio = null;

    public void setUp() throws Exception
    {
        super.setUp();

        scope = new WorkElementScope();
        calendar = new GregorianCalendar();

        //Time Code1 and Time Code2 
        //Since Timecode object cannot be deleted if we don't
        //want to fillUp the DB every time we are running tests
        //we will try to load them. (that is what is done in the fixture)
        TimeCodeScope timeCodeScope = new TimeCodeScope();
        timeCodeScope.setChildren(new TimeCodeScope());

        TimeCodeCategory cat1 = TimecodeFixture.createFixture(timeCodeScope,
            true, this);
        TimeCodeCategory cat2 = TimecodeFixture.createFixture(timeCodeScope,
            false, this);

        //Time Code 1
        for (int ii = 0; ii < cat1.getChildren().length; ii++)
        {
            if (cat1.getChildren()[ii] instanceof TimeCode)
            {
                timeCode1 = (TimeCode) cat1.getChildren()[ii];
                break;
            }
        }

        //Time Code 2
        for (int jj = 0; jj < cat2.getChildren().length; jj++)
        {
            if (cat2.getChildren()[jj] instanceof TimeCode)
            {
                timeCode2 = (TimeCode) cat2.getChildren()[jj];
                break;
            }
        }

        wbsDate = new WbsScheduleDate();
        rpmCalendar = RPMCalendarFixture.retrieveStaticFixtureForInserts(this);
        portFolio = PortfolioFixture.createSaveFixture(this);

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testReadOnlyFields() throws AssertionError, Exception {
		GenericProject template = null;

		GenericProject inputProject = (GenericProject) WorkElementFixture
				.createFixture(null, new Project(), scope, this);

		//List of expected Warnings...
		RPMExceptionMock[] expectedWarnings = {
				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.ASSET_CODE_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(), ValidationConstants.ASSET_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.ASSIGNMENT_TYPE_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.BASELINE_DATE_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.CALENDAR_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.CONTAINING_PROJECT_FIELD,
						inputProject.getID()),
				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.CURRENCY_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.DEFAULT_PROJECT_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.DOCUMENT_FOLDER_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.ESTIMATED_FINISH_DATE_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.ESTIMATED_START_DATE_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.EXPECTED_DATE_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.EXTERNAL_ID_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.INITIAL_BASELINE_DATE_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.LAST_MODIFIED_DATE_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.MINUTES_PER_DAY_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(), ValidationConstants.NAME_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.OPPORTUNITY_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.PORTFOLIOS_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.PREVIOUS_BASELINE_DATE_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.PROJECT_COSTCENTER_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.REFERENCE_ID_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.PUBLISHED_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(), ValidationConstants.PRIORITY_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.REFERENCE_NUMBER_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.SCOPE_ELEMENT_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.TIME_CODE1_FIELD, inputProject
								.getID()),

				createRPMException(ErrorCodes.READ_ONLY_FIELD, inputProject
						.getClass().getName(),
						ValidationConstants.TIME_CODE2_FIELD, inputProject
								.getID())

		};

		//Setting Values to fields
		inputProject.setAssetCode(new AssetFinancialCategory());
		inputProject.setAsset(new Asset());
		inputProject.setAssignmentType(AssignmentType
				.fromString(AssignmentType._Work));
		inputProject.setBaselineDate(wbsDate);
		inputProject.setCalendar(rpmCalendar);
		inputProject.setContainingProject(new WorkElement());
		inputProject.setContextName("");
		inputProject.setCurrency(CurrencyFixture.createFixture(true, 1,this));
		inputProject.setDefaultProject(new Boolean(true));
		inputProject.setDocumentFolder(new DocumentFolder());
		inputProject.setEstimatedFinishDate(calendar);
		inputProject.setEstimatedStartDate(calendar);
		inputProject.setExpectedDate(wbsDate);
		inputProject.setExternalID("");
		inputProject.setInitialBaselineDate(wbsDate);
		inputProject.setLastModifiedDate(calendar);
		inputProject.setMinutesPerDay(new Integer(8));
		inputProject.setName("");
		Opportunity opportunity = new Opportunity();
		opportunity.setParent(inputProject);
		inputProject.setOpportunity(opportunity);
		inputProject.setPortfolio(portFolio);
		inputProject.setPreviousBaselineDate(wbsDate);
		inputProject.setProjectCostCenter(new ProjectCostCenter());
		inputProject.setReferenceID(new Long(8));
		inputProject.setPublished(ProjectPublishedType.None);
		inputProject.setPriority(new Integer(8));
		inputProject.setReferenceNumber("");
		inputProject.setScopeElement(new ScopeElement());
		inputProject.setTimeCode1(timeCode1);
		inputProject.setTimeCode2(timeCode2);

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/GenericProject[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof GenericProject);

		template = (GenericProject) rpmObjects[0];

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, inputProject, template,
						scope);

		verifySaveMustFailForTemplate(inputProject, null, expectedWarnings,
				result);

	}

	private void verifySaveMustFailForTemplate(RPMObject obj,
			RPMExceptionMock[] expectedExceptions,
			RPMExceptionMock[] expectedWarnings, SingleResult saveResult)
			throws AssertionError, Exception {
		// Check that we received exceptions if we were expecting some
		// and that they are the correct ones
		String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
				expectedWarnings, saveResult);

		if (errorMessage.length() != 0)
			Assert.fail(errorMessage);
	}

	public void testNullTemplateField() throws AssertionError, Exception {
		GenericProject template = null;

		GenericProject inputProject = (GenericProject) WorkElementFixture
				.createFixture(null, new Project(), scope, this);

		//List of expected Errors...
		RPMExceptionMock[] expectedErrors = { createRPMException(
				ErrorCodes.CANNOT_CLEAR_FIELD, inputProject.getClass().getName(),
				ValidationConstants.TEMPLATE_FIELD, inputProject.getID()) };

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, inputProject, template,
						scope);

		verifySaveMustFailForTemplate(inputProject, expectedErrors, null,
				result);

	}

//	    public void testNullInputField() throws AssertionError, Exception {
//		GenericProject template = null;
//
//		GenericProject inputProject = (GenericProject) WorkElementFixture
//				.createFixture(null, new Project(), scope, this);
//
//		//Loading template from Xpath: published='ToTemplate'
//		RPMObject[] rpmObjects = loadFromXpath("/GenericProject[published='"
//				+ PublishedType._ToTemplate + "']", scope);
//		assertNotNull(rpmObjects);
//		assertTrue(rpmObjects[0] instanceof GenericProject);
//
//		template = (GenericProject) rpmObjects[0];
//
////		//List of expected Errors...
////		RPMExceptionMock[] expectedErrors = { createRPMException(
////				ErrorCodes.OBJECT_IS_NULL,
////				ValidationConstants.TEMPLATE_INPUT_FIELD, "Object",
////				inputProject.getID()) };
//		
//		//List of expected Errors...
//		RPMExceptionMock[] expectedErrors = { createRPMException(
//				ErrorCodes.OBJECT_IS_NULL,
//				inputProject.getClass().getName(), ValidationConstants.TEMPLATE_INPUT_FIELD,
//				inputProject.getID()) };
//
//
//		SingleResult result = getApplicationInterface()
//				.createObjectFromTemplate(sessionid, null, template, scope);
//
//		verifySaveMustFailForTemplate(inputProject, expectedErrors, null,
//				result);
//
//	}

	public void testInvalidTemplatePublishedTypeField() throws AssertionError,
			Exception {
		GenericProject template = null;

		GenericProject inputProject = (GenericProject) WorkElementFixture
				.createFixture(null, new Project(), scope, this);

		//List of expected Errors...
		RPMExceptionMock[] expectedErrors = { createRPMException(
				com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_PROJECT_TEMPLATE_TYPE,
				inputProject.getClass().getName(),
				ValidationConstants.TEMPLATE_FIELD, inputProject.getID()) };

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/GenericProject[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof GenericProject);

		template = (GenericProject) rpmObjects[0];

		template.setPublished(ProjectPublishedType.None);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, inputProject, template,
						scope);

		verifySaveMustFailForTemplate(inputProject, expectedErrors, null,
				result);

	}

	public void testInvalidTemplateNullPublishedTypeField()
			throws AssertionError, Exception {
		GenericProject template = null;

		GenericProject inputProject = (GenericProject) WorkElementFixture
				.createFixture(null, new Project(), scope, this);

		//List of expected Errors...
		RPMExceptionMock[] expectedErrors = { createRPMException(
				com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_PROJECT_TEMPLATE_TYPE,
				inputProject.getClass().getName(),
				ValidationConstants.TEMPLATE_FIELD, inputProject.getID()) };

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/GenericProject[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);
		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof GenericProject);

		template = (GenericProject) rpmObjects[0];

		template.setPublished(null);

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, inputProject, template,
						scope);

		verifySaveMustFailForTemplate(inputProject, expectedErrors, null,
				result);

	}

	public void testInvalidInputID() throws AssertionError, Exception {
		GenericProject template = null;
		GenericProject inputProject = null;

		inputProject = (Project) WorkElementFixture.retrieveStaticFixture(
				new Project(), this);

		//List of expected Errors...
		RPMExceptionMock[] expectedErrors = { createRPMException(
				ErrorCodes.MUST_BE_NULL, inputProject.getClass().getName(),
				ValidationConstants.ID_FIELD, inputProject.getID()) };

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmTemplateObjects = loadFromXpath(
				"/GenericProject[published='" + ProjectPublishedType._ToTemplate
						+ "']", scope);
		assertNotNull(rpmTemplateObjects);
		assertTrue(rpmTemplateObjects[0] instanceof GenericProject);

		template = (GenericProject) rpmTemplateObjects[0];

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, inputProject, template,
						scope);

		verifySaveMustFailForTemplate(inputProject, expectedErrors, null,
				result);

	}

	public void testInvalidTemplateAndInputID() throws AssertionError,
			Exception {
		GenericProject template = null;
		GenericProject inputProject = null;

		inputProject = (Project) WorkElementFixture.retrieveStaticFixture(
				new Project(), this);

		template = new Project();

		//List of expected Errors...
		RPMExceptionMock[] expectedErrors = {
				createRPMException(ErrorCodes.MUST_BE_NULL, inputProject
						.getClass().getName(), ValidationConstants.ID_FIELD,
						inputProject.getID()),

				createRPMException(ErrorCodes.NO_ID, inputProject.getClass()
						.getName(), ValidationConstants.TEMPLATE_FIELD,
						inputProject.getID()), };

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, inputProject, template,
						scope);

		verifySaveMustFailForTemplate(inputProject, expectedErrors, null,
				result);
	}

	public void testInputHasNoParent() throws AssertionError, Exception {
		WorkElement template = null;

		WorkElement inputProject = new Milestone();

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/Milestone[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);

		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof Milestone);

		template = (Milestone) rpmObjects[0];

		//List of expected Errors...
		RPMExceptionMock[] expectedErrors = { createRPMException(
				com.ibm.rpm.framework.constants.ErrorCodes.MANDATORY,
				inputProject.getClass().getName(), ValidationConstants.PARENT,
				inputProject.getID()) };

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, inputProject, template,
						scope);

		verifySaveMustFailForTemplate(inputProject, expectedErrors, null,
				result);

	}

	public void testInvalidInputObjectTypeAndTypeMismatch()
			throws AssertionError, Exception {
		WorkElement template = null;

		WorkElement inputProject = WorkElementFixture
				.createFixture(null, new Project(), scope, this);

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/SummaryTask[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);

		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof SummaryTask);
		template = (SummaryTask) rpmObjects[0];

		//List of expected Errors...
		RPMExceptionMock[] expectedErrors = {
				createRPMException(
						com.ibm.rpm.framework.constants.ErrorCodes.INVALID_TYPE,
						inputProject.getClass().getName(),
						ValidationConstants.TEMPLATE_INPUT_FIELD, inputProject
								.getID()),
				createRPMException(
						com.ibm.rpm.framework.constants.ErrorCodes.INVALID_TYPE,
						inputProject.getClass().getName(),
						ValidationConstants.TEMPLATE_INPUT_FIELD, inputProject
								.getID()) };

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, inputProject, template,
						scope);

		verifySaveMustFailForTemplate(inputProject, expectedErrors, null,
				result);

	}

	public void testInvalidTemplateObjectTypeAndTypeMismatch()
			throws AssertionError, Exception {
		WorkElement template = null;

		WorkElement inputProject = WorkElementFixture
				.createFixture(null, new SummaryTask(), scope, this);

		//Loading template from Xpath: published='ToTemplate'
		RPMObject[] rpmObjects = loadFromXpath("/GenericProject[published='"
				+ ProjectPublishedType._ToTemplate + "']", scope);

		assertNotNull(rpmObjects);
		assertTrue(rpmObjects[0] instanceof GenericProject);
		template = (GenericProject) rpmObjects[0];

		//List of expected Errors...
		RPMExceptionMock[] expectedErrors = {
				createRPMException(
						com.ibm.rpm.framework.constants.ErrorCodes.INVALID_TYPE,
						template.getClass().getName(),
						ValidationConstants.TEMPLATE_FIELD, template.getID()),
				createRPMException(
						com.ibm.rpm.framework.constants.ErrorCodes.INVALID_TYPE,
						inputProject.getClass().getName(),
						ValidationConstants.TEMPLATE_INPUT_FIELD, inputProject
								.getID()) };

		SingleResult result = getApplicationInterface()
				.createObjectFromTemplate(sessionid, inputProject, template,
						scope);

		verifySaveMustFailForTemplate(inputProject, expectedErrors, null,
				result);
	}
    

}
