/*
 * Created on Feb 20, 2006
 * @author mmatwiy 
 * Validation tests for container ProfileCompetencyAssignment
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.ResourceAttributeProficiencyLevel;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.ProfileCompetencyAssignment;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.scope.ProfileScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class TestProfileCompetencyAssignmentValidation extends
        GenericValidation
{
    private ProfileCompetencyAssignment assignment;

    public void setUp() throws Exception
    {
        super.setUp();

        assignment = new ProfileCompetencyAssignment();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        assignment.setParent(new Task());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, assignment
                .getClass().getName(), ValidationConstants.PARENT, assignment
                .getID())
        };

        verifySaveMustFail(assignment, null, expectedExceptions);
    }

    public void testInsertWithParentProfile() throws AssertionError, Exception
    {
        ProfileScope profileScope = new ProfileScope();
        profileScope.setProfileCompetencyAssignments(true);

        ResourceTaskAssignmentScope resourceTaskAssignmentScope = new ResourceTaskAssignmentScope();
        resourceTaskAssignmentScope.setProfile(profileScope);
        resourceTaskAssignmentScope.setTaskAssignment(new TaskAssignmentScope());

        ResourceTaskAssignment resourceTaskAssignment = ResourceTaskAssignmentFixture
            .createFixture(resourceTaskAssignmentScope, false, this);

        assignment.setParent(resourceTaskAssignment.getProfile());

        // Attempt to save the object
        resourceTaskAssignment = (ResourceTaskAssignment) saveMustSucceed(
            resourceTaskAssignment, ResourceTaskAssignment.class,
            resourceTaskAssignmentScope);

        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTaskAssignment.getID() + "']",
            resourceTaskAssignmentScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        assertNotNull(((ResourceTaskAssignment) rpmObjects[0]).getProfile());
        assertNotNull(((ResourceTaskAssignment) rpmObjects[0])
            .getProfile().getProfileCompetencyAssignments());
        assertNotNull(((ResourceTaskAssignment) rpmObjects[0])
            .getProfile().getProfileCompetencyAssignments()[0]);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTaskAssignment,
            (ResourceTaskAssignment)rpmObjects[0], resourceTaskAssignmentScope);

    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, assignment.getClass()
                .getName(), ValidationConstants.PARENT, assignment.getID())
        };

        verifySaveMustFail(assignment, null, expectedExceptions);
    }
    
    public void testInsertWithInvalidComptencies() throws AssertionError, Exception
    {
        ProfileScope profileScope = new ProfileScope();
        profileScope.setProfileSkillAssignments(true);

        ResourceTaskAssignmentScope resourceTaskAssignmentScope = new ResourceTaskAssignmentScope();
        resourceTaskAssignmentScope.setProfile(profileScope);
        resourceTaskAssignmentScope.setTaskAssignment(new TaskAssignmentScope());

        ResourceTaskAssignment resourceTaskAssignment = ResourceTaskAssignmentFixture
            .createFixture(resourceTaskAssignmentScope, false, this);

        assignment.setParent(resourceTaskAssignment.getProfile());

        ProfileCompetencyAssignment pca[] = new ProfileCompetencyAssignment[1];
		ResourceAttributeProficiencyLevel raProficiencyLevel = resourceTaskAssignment.getProfile().getProfileSkillAssignments()[0].getProficiencyLevel();
		pca[0] = new ProfileCompetencyAssignment();
		pca[0].setProficiencyLevel(raProficiencyLevel);
		resourceTaskAssignment.getProfile().setProfileCompetencyAssignments(pca);
		resourceTaskAssignment.getProfile().setProfileSkillAssignments(null);
        
        assignment.setParent(resourceTaskAssignment.getProfile());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(com.ibm.rpm.resource.constants.ErrorCodes.INVALID_COMPTENCY_PROFICIENCY_LEVEL, assignment.getClass()
                .getName(), ValidationConstants.PROFILE_PROFICIENCY_LEVEL_FIELD, assignment.getID())
        };        
        
        // Attempt to save the object
        verifySaveMustFail(resourceTaskAssignment, resourceTaskAssignmentScope, expectedExceptions);
    }

}
