/*
 * Created on Dec 2, 2005 
 * @author mmatwiy 
 * Validation tests for container Profile.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.ProfileResourceAttributeAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Profile;
import com.ibm.rpm.wbs.containers.ProfileCompetencyAssignment;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ProfileScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;

public class TestProfileValidation extends GenericValidation
{
    private ResourceTaskAssignment resourceTask;

    private SecurityRole securityRole;

    private ResourceTaskAssignmentScope rtaScope;

    private Profile profile;

    public void setUp() throws Exception
    {
        super.setUp();

        profile = new Profile();

        securityRole = new SecurityRole();
        securityRole.setID("SCTASKRESPONSIBLE");
        resourceTask = new ResourceTaskAssignment();
        resourceTask.setSecurityRole(securityRole);
        resourceTask.setProfile(profile);
        resourceTask.setTaskAssignment(new TaskAssignment());

        ProfileScope profileScope = new ProfileScope();
        profileScope.setProfileCompetencyAssignments(true);

        rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setProfile(profileScope);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        Milestone milestone = (Milestone) WorkElementFixture
            .retrieveStaticFixture(new Milestone(), this);
        resourceTask.setParent(milestone);
        
        // Set mandatory field profileCompetencyAssignments
        ProfileCompetencyAssignment[] assignments =
        {
            (ProfileCompetencyAssignment) ProfileResourceAttributeAssignmentFixture
                .createFixture(new ResourceAttributeCompetencyCategory(),
                    profile, this)
        };
        profile.setProfileCompetencyAssignments(assignments);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);

    }

    public void testInsertWithGrandparentTaskAndAssignmentTypeDuration()
            throws AssertionError, Exception
    {
        Task task = (Task) WorkElementFixture.retrieveStaticFixture(new Task(),
            this);
        resourceTask.setParent(task);

        Project project = new Project();
        project.setID("dummy id");
        project.setAssignmentType(AssignmentType.Duration);

        ((WorkElement) resourceTask.getParent()).setContainingProject(project);

        // Set mandatory field profileCompetencyAssignments
        ProfileCompetencyAssignment[] assignments =
        {
            (ProfileCompetencyAssignment) ProfileResourceAttributeAssignmentFixture
                .createFixture(new ResourceAttributeCompetencyCategory(),
                    profile, this)
        };
        profile.setProfileCompetencyAssignments(assignments);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.PROFILE_GRANDPARENT_ASSIGNMENT_TYPE_INVALID,
                profile.getClass().getName())
        };

        verifySaveMustFail(resourceTask, rtaScope, expectedExceptions);

    }

    public void testInsertWithGrandparentProjectAndAssignmentTypeDuration()
            throws AssertionError, Exception
    {
        Project project = new Project();
        project.setName(HelperTestUtilities.createName());
        project.setAssignmentType(AssignmentType.Duration);
        project.setContainingProject(WorkElementFixture.retrieveStaticFixture(
            new Project(), this));
        save(project, new WorkElementScope());

        resourceTask.setParent(project);
        
        SecurityRole role = new SecurityRole();
        role.setID("SCPROJECTMANAGER");
        resourceTask.setSecurityRole(role);
        
        // Set mandatory field profileCompetencyAssignments
        ProfileCompetencyAssignment[] assignments =
        {
            (ProfileCompetencyAssignment) ProfileResourceAttributeAssignmentFixture
                .createFixture(new ResourceAttributeCompetencyCategory(),
                    profile, this)
        };
        profile.setProfileCompetencyAssignments(assignments);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testInsertWithoutGrandparentProjectID() throws AssertionError,
            Exception
    {
        Proposal proposal = new Proposal();
        proposal.setAssignmentType(AssignmentType.Duration);
        proposal.setName("proposal name");
        
        save(proposal, null);

        resourceTask.setParent(proposal);

        SecurityRole role = new SecurityRole();
        role.setID("SCPROJECTMANAGER");
        resourceTask.setSecurityRole(role);
        
        // Set mandatory field profileCompetencyAssignments
        ProfileCompetencyAssignment[] assignments =
        {
            (ProfileCompetencyAssignment) ProfileResourceAttributeAssignmentFixture
                .createFixture(new ResourceAttributeCompetencyCategory(),
                    profile, this)
        };
        profile.setProfileCompetencyAssignments(assignments);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NO_ID, profile.getClass()
                    .getName(),
                    ValidationConstants.PROFILE_GRANDPARENT_PROJECT, profile
                        .getID())
        };

        verifySaveMustFail(resourceTask, rtaScope, expectedExceptions);

    }

    public void testFailureInsertWithoutMandatoryFields()
            throws AssertionError, Exception
    {
        Milestone milestone = (Milestone) WorkElementFixture
            .retrieveStaticFixture(new Milestone(), this);
        resourceTask.setParent(milestone);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, resourceTask.getProfile()
                .getClass().getName(),
                ValidationConstants.PROFILE_COMPETENCY_ASSIGNMENTS_FIELD,
                resourceTask.getProfile().getID())
        };

        verifySaveMustFail(resourceTask, rtaScope, expectedExceptions);
    }

    public void testFailureReadOnlyFields() throws AssertionError, Exception
    {
        Milestone milestone = (Milestone) WorkElementFixture
            .retrieveStaticFixture(new Milestone(), this);
        resourceTask.setParent(milestone);

        String name = "test read-only name";
        profile.setProfileName(name);

        // Set mandatory field profileCompetencyAssignments
        ProfileCompetencyAssignment[] assignments =
        {
            (ProfileCompetencyAssignment) ProfileResourceAttributeAssignmentFixture
                .createFixture(new ResourceAttributeCompetencyCategory(),
                    profile, this)
        };
        profile.setProfileCompetencyAssignments(assignments);

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(ErrorCodes.READ_ONLY_FIELD, resourceTask
                .getProfile().getClass().getName(),
                ValidationConstants.PROFILE_NAME_FIELD, resourceTask
                    .getProfile().getID())
        };

        verifySaveMustFail(resourceTask, rtaScope, null, expectedWarnings);
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);
        
        Profile loadedProfile = ((ResourceTaskAssignment)rpmObjects[0]).getProfile();
        
        // check that the value we set was not saved in the db
        assertNotSame(name, loadedProfile.getProfileName());
        
    }

    public void testFailureInsertParent() throws AssertionError, Exception
    {
        Milestone milestone = (Milestone) WorkElementFixture
            .retrieveStaticFixture(new Milestone(), this);
        resourceTask.setParent(milestone);

        ProfileCompetencyAssignment[] assignments =
        {
            (ProfileCompetencyAssignment) ProfileResourceAttributeAssignmentFixture
                .createFixture(new ResourceAttributeCompetencyCategory(),
                    profile, this)
        };
        profile.setProfileCompetencyAssignments(assignments);

        profile.setParent(resourceTask);

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, resourceTask
                .getProfile().getClass().getName(), ValidationConstants.PARENT,
                resourceTask.getProfile().getID())
        };

        verifySaveMustFail(resourceTask, rtaScope, null, expectedWarnings);
    }

}