/*
 * Created on Feb 20, 2006
 * @author mmatwiy 
 * Validation tests for container ProjectOrganizationalAssignment
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.OrganizationalFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ProjectOrganizationalAssignment;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestProjectOrganizationalAssignmentValidation extends
        GenericValidation
{
    private ProjectOrganizationalAssignment assignment;

    private WorkElementScope scope;

    public void setUp() throws Exception
    {
        super.setUp();

        assignment = new ProjectOrganizationalAssignment();

        scope = new WorkElementScope();
        scope.setOrganizationalAssignments(true);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        assignment.setParent(new Task());

        // Set mandatory fields
        assignment.setOrganizational(OrganizationalFixture.createSaveFixture(1,
            this));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, assignment
                .getClass().getName(), ValidationConstants.PARENT, assignment
                .getID())
        };

        verifySaveMustFail(assignment, null, expectedExceptions);
    }

    public void testInsertUpdateWithParentProject() throws AssertionError, Exception
    {
        Project project = (Project) WorkElementFixture.createFixture(
            new Project(), scope, this);
        assignment.setParent(project);

        // Set mandatory fields
        assignment.setOrganizational(OrganizationalFixture.createSaveFixture(1,
            this));

        ProjectOrganizationalAssignment[] assignments =
        {
            assignment
        };
        project.setOrganizationalAssignments(assignments);

        // Attempt to save the object
        project = (Project) saveMustSucceed(project, Project.class, scope);

        RPMObject[] rpmObjects = loadFromXpath(
            "/GenericProject[ProjectOrganizationalAssignment[id='"
                    + assignment.getID() + "']]", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof GenericProject);
        assertNotNull(((GenericProject) rpmObjects[0])
            .getOrganizationalAssignments());
        
        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(project, (Project) rpmObjects[0], scope);
        
        Project reloaded = (Project) rpmObjects[0];
        project = (Project) saveMustSucceed(reloaded, Project.class, scope);
        
        rpmObjects = loadFromXpath(
            "/GenericProject[ProjectOrganizationalAssignment[id='"
                    + assignment.getID() + "']]", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof GenericProject);
        assertNotNull(((GenericProject) rpmObjects[0])
            .getOrganizationalAssignments());
        
        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(reloaded, (Project) rpmObjects[0], scope);
    }

    public void testInsertWithParentProposal() throws AssertionError, Exception
    {
        Proposal proposal = (Proposal) WorkElementFixture.createFixture(
            new Proposal(), scope, this);
        assignment.setParent(proposal);

        // Set mandatory fields
        assignment.setOrganizational(OrganizationalFixture.createSaveFixture(1,
            this));

        ProjectOrganizationalAssignment[] assignments =
        {
            assignment
        };
        proposal.setOrganizationalAssignments(assignments);

        // Attempt to save the object
        proposal = (Proposal) saveMustSucceed(proposal, Proposal.class, scope);

        RPMObject[] rpmObjects = loadFromXpath(
            "/GenericProject[ProjectOrganizationalAssignment[id='"
                    + assignment.getID() + "']]", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof GenericProject);
        assertNotNull(((GenericProject) rpmObjects[0])
            .getOrganizationalAssignments());

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(proposal, (Proposal) rpmObjects[0], scope);
    }

    public void testInsertWithParentWorkOrganization() throws AssertionError,
            Exception
    {
        WorkOrganization workOrganization = (WorkOrganization) WorkElementFixture
            .createFixture(new WorkOrganization(), scope, this);
        assignment.setParent(workOrganization);

        // Set mandatory fields
        assignment.setOrganizational(OrganizationalFixture.createSaveFixture(1,
            this));

        ProjectOrganizationalAssignment[] assignments =
        {
            assignment
        };
        workOrganization.setOrganizationalAssignments(assignments);

        // Attempt to save the object
        workOrganization = (WorkOrganization) saveMustSucceed(workOrganization,
            WorkOrganization.class, scope);

        RPMObject[] rpmObjects = loadFromXpath(
            "/GenericProject[ProjectOrganizationalAssignment[id='"
                    + assignment.getID() + "']]", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof GenericProject);
        assertNotNull(((GenericProject) rpmObjects[0])
            .getOrganizationalAssignments());

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(workOrganization,
            (WorkOrganization) rpmObjects[0], scope);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        // Set mandatory fields
        assignment.setOrganizational(OrganizationalFixture.createSaveFixture(1,
            this));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, assignment.getClass()
                .getName(), ValidationConstants.PARENT, assignment.getID())
        };

        verifySaveMustFail(assignment, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        assignment.setParent(WorkElementFixture.retrieveStaticFixture(
            new Project(), this));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, assignment.getClass()
                .getName(), ValidationConstants.ORGANIZATIONAL_FIELD,
                assignment.getID())
        };

        verifySaveMustFail(assignment, null, expectedExceptions);

    }

    public void testFailureInsertDuplicateAssignment() throws AssertionError,
            Exception
    {
        assignment.setParent(WorkElementFixture.retrieveStaticFixture(
            new Project(), this));

        Project project = (Project) WorkElementFixture.createFixture(
            new Project(), scope, this);
        assignment.setParent(project);

        Organizational organizational = OrganizationalFixture.createSaveFixture(1, this);
        // Set mandatory fields
        assignment.setOrganizational(organizational);

        ProjectOrganizationalAssignment[] assignments = new ProjectOrganizationalAssignment[2];
        assignments[0] = assignment;
        project.setOrganizationalAssignments(assignments);

        // Attempt to save the object
        project = (Project) saveMustSucceed(project, Project.class, scope);

        RPMObject[] rpmObjects = loadFromXpath(
            "/GenericProject[ProjectOrganizationalAssignment[id='"
                    + assignment.getID() + "']]", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof GenericProject);
        Project reloadedProject = (Project) rpmObjects[0];
        assertNotNull(reloadedProject.getOrganizationalAssignments());

        ProjectOrganizationalAssignment assignment2 = new ProjectOrganizationalAssignment();
        assignment2.setParent(project);
        assignment2.setOrganizational(organizational);
        
        assignments[1] = assignment2;
        reloadedProject.setOrganizationalAssignments(assignments);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.DUPLICATE_ASSIGNMENT,
                assignment2.getClass().getName(),
                ValidationConstants.ORGANIZATIONAL_FIELD, assignment2.getID())
        };

        verifySaveMustFail(reloadedProject, scope, expectedExceptions);

    }
}