/*
 * Created on Dec 2, 2005 
 * @author mmatwiy 
 * Validation tests for container ResourceTaskAssignment.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import com.ibm.rpm.applicationadministration.containers.AssignmentStatus;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ProfileFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.ResourceTaskAssignmentFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.ProfileScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ResourceType;

public class TestResourceTaskAssignmentValidation extends GenericValidation
{
    protected ResourceTaskAssignment resourceTask;

    private SecurityRole securityRole;

    protected WorkElementScope workElementScope;

    protected ResourceTaskAssignmentScope rtaScope;

    protected ResourceScope resourceScope;

    private ProfileScope profileScope;

    private Proposal grandparentProposal;

    public void setUp() throws Exception
    {
        super.setUp();

        grandparentProposal = new Proposal();
        grandparentProposal.setName(HelperTestUtilities.createName());
        grandparentProposal = (Proposal) save(grandparentProposal,
            new WorkElementScope()).getRpmObject();

        resourceTask = new ResourceTaskAssignment();
        securityRole = new SecurityRole();
        securityRole.setID("SCPROJECTMANAGER");

        rtaScope = new ResourceTaskAssignmentScope();
        resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());

        profileScope = new ProfileScope();
        profileScope.setProfileCompetencyAssignments(true);

        workElementScope = new WorkElementScope();
        workElementScope.setParent(new WorkElementScope());
        workElementScope.setContainingProject(new WorkElementScope());
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);
        resourceTask.setParent(portfolio);

        // set mandatory fields
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        resourceTask.setSecurityRole(securityRole);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, resourceTask
                .getClass().getName(), ValidationConstants.PARENT, resourceTask
                .getID())
        };

        verifySaveMustFail(resourceTask, null, expectedExceptions);
    }

    public void testInsertWithProfileAndParentTask() throws AssertionError,
            Exception
    {
        rtaScope.setProfile(profileScope);

        //Task task = (Task)
        // TaskAssignmentFixture.createProperWorkElement(this);
        Task task = (Task) WorkElementFixture.retrieveStaticFixture(new Task(),
            this);

        // Set the mandatory fields
        resourceTask.setParent(task);
        resourceTask.setProfile(ProfileFixture.createFixture(profileScope, 1,
            resourceTask, this));
        
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCTASKRESPONSIBLE");
        
        resourceTask.setSecurityRole(localSecurityRole);
        resourceTask.setTaskAssignment(new TaskAssignment());

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testInsertWithParentMilestone() throws AssertionError,
            Exception
    {
        rtaScope.setResource(resourceScope);

        Milestone milestone = new Milestone();
        milestone.setName(HelperTestUtilities.createName());
        milestone.setParent(grandparentProposal);

        milestone = (Milestone) save(milestone, workElementScope)
            .getRpmObject();

        // Set the mandatory fields
        resourceTask.setParent(milestone);
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCTASKRESPONSIBLE");
        
        resourceTask.setSecurityRole(localSecurityRole);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testInsertWithParentProject() throws AssertionError, Exception
    {
        rtaScope.setResource(resourceScope);

        Project project = new Project();
        project.setName(HelperTestUtilities.createName());

        project = (Project) save(project, workElementScope).getRpmObject();

        // Set the mandatory fields
        resourceTask.setParent(project);
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        resourceTask.setSecurityRole(securityRole);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testInsertWithParentProposal() throws AssertionError, Exception
    {
        rtaScope.setResource(resourceScope);

        Proposal proposal = new Proposal();
        proposal.setName(HelperTestUtilities.createName());

        proposal = (Proposal) save(proposal, workElementScope).getRpmObject();

        // Set the mandatory fields
        resourceTask.setParent(proposal);
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        resourceTask.setSecurityRole(securityRole);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testInsertWithParentWorkOrganization() throws AssertionError,
            Exception
    {
        rtaScope.setResource(resourceScope);

        WorkOrganization workOrganization = new WorkOrganization();
        workOrganization.setName(HelperTestUtilities.createName());

        workOrganization = (WorkOrganization) save(workOrganization,
            workElementScope).getRpmObject();

        // Set the mandatory fields
        resourceTask.setParent(workOrganization);
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        resourceTask.setSecurityRole(securityRole);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testInsertWithParentDeliverable() throws AssertionError,
            Exception
    {
        Deliverable deliverable = new Deliverable();
        deliverable.setName(HelperTestUtilities.createName());
        deliverable.setParent(grandparentProposal);

        deliverable = (Deliverable) save(deliverable, workElementScope)
            .getRpmObject();

        // Set the mandatory fields
        resourceTask.setParent(deliverable);
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCDELIVERABLERESPONSIBLE");
        
        resourceTask.setSecurityRole(localSecurityRole);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testInsertWithParentWorkProduct() throws AssertionError,
            Exception
    {
        rtaScope.setResource(resourceScope);

        WorkProduct workProduct = new WorkProduct();
        workProduct.setName(HelperTestUtilities.createName());
        workProduct.setParent(grandparentProposal);

        workProduct = (WorkProduct) save(workProduct, workElementScope)
            .getRpmObject();

        // Set the mandatory fields
        resourceTask.setParent(workProduct);
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCDELIVERABLERESPONSIBLE");
        resourceTask.setSecurityRole(localSecurityRole);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testInsertWithParentSummaryTask() throws AssertionError,
            Exception
    {
        rtaScope.setResource(resourceScope);

        SummaryTask summaryTask = new SummaryTask();
        summaryTask.setName(HelperTestUtilities.createName());
        summaryTask.setParent(grandparentProposal);

        summaryTask = (SummaryTask) save(summaryTask, workElementScope)
            .getRpmObject();

        // Set the mandatory fields
        resourceTask.setParent(summaryTask);
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCDELIVERABLERESPONSIBLE");
        resourceTask.setSecurityRole(localSecurityRole);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testInsertWithProfileAndParentScopeElement()
            throws AssertionError, Exception
    {
        rtaScope.setProfile(profileScope);

        Proposal proposal = (Proposal) WorkElementFixture
            .retrieveStaticFixture(new Proposal(), this);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", new ScopeElementScope());
        ScopeFolder parent = (ScopeFolder) rpmObjects[0];

        ScopeElementScope changeRequestScope = new ScopeElementScope();
        changeRequestScope.setProject(new WorkElementScope());

        ChangeRequest changeRequest = new ChangeRequest();
        changeRequest.setName("changeRequest name");
        changeRequest.setParent(parent);

        changeRequest.setProject((Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this));
        changeRequest = (ChangeRequest) save(changeRequest, changeRequestScope)
            .getRpmObject();

        // Set the mandatory fields
        resourceTask.setParent(changeRequest);
        resourceTask.setProfile(ProfileFixture.createFixture(profileScope, 1,
            resourceTask, this));
        
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCCHANGEREQUESTRESPONSIBLE");
        
        resourceTask.setSecurityRole(localSecurityRole);
        resourceTask.setTaskAssignment(new TaskAssignment());

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testInsertWithParentScopeElement() throws AssertionError,
            Exception
    {
        rtaScope.setResource(resourceScope);

        Project project = (Project) WorkElementFixture.retrieveStaticFixture(
            new Project(), this);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + project.getName() + "']", new ScopeElementScope());
        ScopeFolder riskParent = (ScopeFolder) rpmObjects[0];

        Risk risk = new Risk();
        risk.setName("risk name");
        risk.setParent(riskParent);

        risk.setProject((Project) WorkElementFixture.retrieveStaticFixture(
            new Project(), this));
        save(risk, null);

        // Set the mandatory fields
        resourceTask.setParent(risk);
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCRISKRESPONSIBLE");
        
        resourceTask.setSecurityRole(localSecurityRole);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        resourceTask.setSecurityRole(securityRole);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, resourceTask.getClass()
                .getName(), ValidationConstants.PARENT, resourceTask.getID())
        };

        verifySaveMustFail(resourceTask, null, expectedExceptions);

    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        SummaryTask summaryTask = (SummaryTask) WorkElementFixture
            .retrieveStaticFixture(new SummaryTask(), this);

        resourceTask.setParent(summaryTask);

        // resource field or profile field is mandatory
        // securityRole field is mandatory
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MANDATORY, resourceTask
                    .getClass().getName(), ValidationConstants.RESOURCE_FIELD
                        + " or " + ValidationConstants.PROFILE_FIELD,
                    resourceTask.getID()),
                createRPMException(ErrorCodes.MANDATORY, resourceTask
                    .getClass().getName(),
                    ValidationConstants.SECURITY_ROLE_FIELD, resourceTask
                        .getID())
        };

        verifySaveMustFail(resourceTask, null, expectedExceptions);

    }

    public void testInsertReadOnlyFields() throws AssertionError, Exception
    {
        SummaryTask summaryTask = (SummaryTask) WorkElementFixture
            .retrieveStaticFixture(new SummaryTask(), this);

        resourceTask.setParent(summaryTask);

        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCDELIVERABLERESPONSIBLE");
        resourceTask.setSecurityRole(localSecurityRole);
        resourceTask.setResourceType(ResourceType.Resource);

        // assignmentStatus is a read-only field
        resourceTask.setAssignmentStatus(new AssignmentStatus());

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, resourceTask
                    .getClass().getName(),
                    ValidationConstants.ASSIGNMENT_STATUS_FIELD, resourceTask
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, resourceTask
                    .getClass().getName(),
                    ValidationConstants.RESOURCE_TYPE_FIELD, resourceTask
                        .getID())
        };

        ResourceTaskAssignmentScope scope = new ResourceTaskAssignmentScope();
        scope.setAssignmentStatus(true);
        
        verifySaveMustFail(resourceTask, scope, null, expectedWarnings);

    }

    public void testUpdateClearMandatoryFields() throws AssertionError,
            Exception
    {
        SummaryTask summaryTask = (SummaryTask) WorkElementFixture
            .retrieveStaticFixture(new SummaryTask(), this);

        // Set the mandatory fields
        resourceTask.setParent(summaryTask);
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCDELIVERABLERESPONSIBLE");
        resourceTask.setSecurityRole(localSecurityRole);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(resourceTask,
            ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        ResourceTaskAssignment loadedAssignment = (ResourceTaskAssignment) rpmObjects[0];

        // profile and resource can be set to null on update only
        // securityRole is always mandatory
        loadedAssignment.setProfile(null);
        loadedAssignment.setResource(null);

        // Attempt to save the object
        resourceTask = (ResourceTaskAssignment) saveMustSucceed(
            loadedAssignment, ResourceTaskAssignment.class, rtaScope);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/ResourceTaskAssignment[id='"
                + resourceTask.getID() + "']", rtaScope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ResourceTaskAssignment);

        // Check that the loaded object is identical to the one we
        // requested to store
        ResourceTaskAssignmentFixture.compare(resourceTask,
            (ResourceTaskAssignment) rpmObjects[0], rtaScope);
    }

    public void testFailureInsertProfileWithResource() throws AssertionError,
            Exception
    {
        Milestone milestone = (Milestone) WorkElementFixture
            .retrieveStaticFixture(new Milestone(), this);
        resourceTask.setParent(milestone);
        rtaScope.setProfile(profileScope);

        // set mandatory fields
        resourceTask.setProfile(ProfileFixture.createFixture(resourceTask,
            new ProfileScope(), this));
        
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCTASKRESPONSIBLE");
        
        resourceTask.setSecurityRole(localSecurityRole);
        resourceTask.setTaskAssignment(new TaskAssignment());

        resourceTask.setResource(new Resource());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                ErrorCodes.NULL_REQUIRED_WHEN_FIELD_VALUE_EXISTS, resourceTask
                    .getClass().getName(), ValidationConstants.PROFILE_FIELD,
                resourceTask.getID())
        };

        verifySaveMustFail(resourceTask, rtaScope, expectedExceptions);
    }

    public void testFailureInsertResourceWithoutMandatoryFields()
            throws AssertionError, Exception
    {
        Milestone milestone = new Milestone();
        milestone.setName("RTA parent milestone");
        milestone.setParent(WorkElementFixture.retrieveStaticFixture(
            new Project(), this));
        save(milestone, null);

        resourceTask.setParent(milestone);

        // set mandatory RTA fields
        
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCTASKRESPONSIBLE");
        
        resourceTask.setSecurityRole(localSecurityRole);
        resourceTask.setTaskAssignment(new TaskAssignment());

        // the RTA's parent's containing project and the calendar 
        // are mandatory when the resource field is populated
        Resource resource = new Resource();
        resource.setFullName("resource fullname");
        resource.setParent(PoolFixture.createSaveFixture(this));

        save(resource, resourceScope);

        resourceTask.setResource(resource);
        rtaScope.setResource(resourceScope);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CONDITIONAL_MANDATORY,
                    resourceTask.getClass().getName(),
                    ValidationConstants.RESOURCE_CALENDAR_FIELD, resourceTask
                        .getID()),
                createRPMException(ErrorCodes.CONDITIONAL_MANDATORY,
                    resourceTask.getClass().getName(),
                    ValidationConstants.PARENT_CONTAINING_PROJECT_FIELD,
                    resourceTask.getID())
        };

        verifySaveMustFail(resourceTask, rtaScope, expectedExceptions);
    }

    public void testFailureInsertProfileWithoutTaskAssignment()
            throws AssertionError, Exception
    {
        Milestone milestone = (Milestone) WorkElementFixture
            .retrieveStaticFixture(new Milestone(), this);
        resourceTask.setParent(milestone);
        rtaScope.setProfile(profileScope);

        // set mandatory fields
        resourceTask.setProfile(ProfileFixture.createFixture(resourceTask,
            new ProfileScope(), this));
        
        SecurityRole localSecurityRole = new SecurityRole();
        localSecurityRole.setID("SCTASKRESPONSIBLE");
        
        resourceTask.setSecurityRole(localSecurityRole);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CONDITIONAL_MANDATORY, resourceTask
                .getClass().getName(),
                ValidationConstants.TASK_ASSIGNMENT_FIELD, resourceTask.getID())
        };

        verifySaveMustFail(resourceTask, rtaScope, expectedExceptions);
    }

    public void testInsertWithoutParentID() throws AssertionError, Exception
    {
        // Set the mandatory fields
        resourceTask.setParent(new SummaryTask());
        resourceTask.setResource(ResourceFixture.createSaveFixture(
            resourceScope, this));
        resourceTask.setSecurityRole(securityRole);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT_ID, resourceTask.getClass()
                .getName(), ValidationConstants.PARENT, resourceTask.getID())
        };

        verifySaveMustFail(resourceTask, null, expectedExceptions);
    }
}