/*
 * Created on 31-May-2006
 * @author mmatwiy
 * Validation test cases for conversion of a WorkElement
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestWorkElementConversionValidation extends GenericValidation
{

    private WorkElementScope scope = null;

    public void setUp() throws Exception
    {
        super.setUp();
        scope = new WorkElementScope();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureConvertDeliverableToTaskWithChildren()
            throws AssertionError, Exception
    {
        scope.setChildren(new WorkElementScope());

        Deliverable inputDeliverable = (Deliverable) WorkElementFixture
            .createSaveFixture(null, new Deliverable(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputDeliverable, new Task(), scope);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID_WITH_CHILDREN,
                inputDeliverable.getClass().getName()),
        };

        verifyFailure(inputDeliverable, expectedExceptions, null, result);
    }

    public void testFailureConvertTaskToProject() throws AssertionError,
            Exception
    {
        Task inputTask = (Task) WorkElementFixture.createSaveFixture(null,
            new Task(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputTask, new Project(), scope);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
                inputTask.getClass().getName()),
        };

        verifyFailure(inputTask, expectedExceptions, null, result);
    }

    public void testFailureConvertProposalToTask() throws AssertionError,
            Exception
    {
        Proposal inputProposal = (Proposal) WorkElementFixture
            .createSaveFixture(null, new Proposal(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputProposal, new Task(), scope);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
                inputProposal.getClass().getName()),
        };

        verifyFailure(inputProposal, expectedExceptions, null, result);
    }

    public void testFailureConvertDeliverableToWorkOrganization()
            throws AssertionError, Exception
    {
        Deliverable inputDeliverable = (Deliverable) WorkElementFixture
            .createSaveFixture(null, new Deliverable(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputDeliverable, new WorkOrganization(), scope);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
                inputDeliverable.getClass().getName()),
        };

        verifyFailure(inputDeliverable, expectedExceptions, null, result);
    }

    public void testFailureConvertTaskNoID() throws AssertionError, Exception
    {
        Task inputTask = new Task();

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputTask, new Milestone(), scope);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.framework.constants.ErrorCodes.NO_ID, inputTask
                    .getClass().getName()),
        };

        verifyFailure(inputTask, expectedExceptions, null, result);
    }

    public void testFailureConvertInvalidInputType() throws AssertionError,
            Exception
    {
        ClientCostCenter clientCostCenter = new ClientCostCenter();

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, clientCostCenter, new Risk(), scope);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_RPMOBJECT_TYPE, null),
        };

        verifyFailure(clientCostCenter, expectedExceptions, null, result);
    }

    public void testFailureConvertInvalidOutputType() throws AssertionError,
            Exception
    {
        Deliverable inputDeliverable = (Deliverable) WorkElementFixture
            .createSaveFixture(null, new Deliverable(), scope, this);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputDeliverable, new Risk(), scope);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
                inputDeliverable.getClass().getName())
        };

        verifyFailure(inputDeliverable, expectedExceptions, null, result);
    }

    public void testFailureNullInputContainer() throws AssertionError,
            Exception
    {

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, null, new Project(), scope);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OBJECT_IS_NULL, null),
        };

        verifyFailure(null, expectedExceptions, null, result);
    }
}