/*
 * Created on Nov 16, 2005 
 * @author mmatwiy 
 * Validation of WBS container WorkOrganization.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.DeliverableState;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestWorkOrganizationValidation extends AbstractTestWorkElementValidation
{
    private WorkOrganization workOrganization;

    public void setUp() throws Exception
    {
        super.setUp();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        workOrganization = new WorkOrganization();

        // create and set invalid parent
        Task task = (Task) WorkElementFixture.retrieveStaticFixture(new Task(),
            this);

        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        workOrganization.setParent(task);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, workOrganization
                .getClass().getName(), ValidationConstants.PARENT,
                workOrganization.getID())
        };

        verifySaveMustFail(workOrganization, null, expectedExceptions);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        workOrganization = new WorkOrganization();

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        workOrganization
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_REFERENCE_NUMBER_CHARACTER_LIMIT));
        workOrganization.setEstimatedStartDate(new GregorianCalendar(2005, 12,
            11));
        workOrganization.setEstimatedFinishDate(new GregorianCalendar(2006, 12,
            12));
        workOrganization.setProbabilityPercent(new Integer((int) Math.random() * 100));

        Currency currency = DatafieldFixture
            .createCurrency(DatafieldFixture.CURRENCY_USD);
        workOrganization.setCurrency(currency);

        // Attempt to save the object
        workOrganization = (WorkOrganization) saveMustSucceed(workOrganization,
            WorkOrganization.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkOrganization);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(workOrganization,
            (WorkElement) rpmObjects[0], null);
    }

    public void testInsertSuccessModifiedEstimatedDates()
            throws AssertionError, Exception
    {
        workOrganization = new WorkOrganization();

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Set both estimated dates to today's date
        workOrganization.setEstimatedStartDate(Calendar.getInstance());
        workOrganization.setEstimatedFinishDate(Calendar.getInstance());

        // Attempt to save the object object
        workOrganization = (WorkOrganization) saveMustSucceed(workOrganization,
            WorkOrganization.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkOrganization);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(workOrganization,
            (WorkElement) rpmObjects[0], null);
    }

    public void testFailureInvalidEstimatedFinishDate() throws AssertionError,
            Exception
    {
        workOrganization = new WorkOrganization();

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Set the estimatedFinishDate to be prior to the estimatedStartDate
        workOrganization.setEstimatedStartDate(new GregorianCalendar(2005, 12,
            12));
        workOrganization.setEstimatedFinishDate(new GregorianCalendar(2005, 12,
            11));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_DATES, workOrganization
                .getClass().getName(),
                ValidationConstants.ESTIMATED_START_DATE_FIELD,
                workOrganization.getID())
        };

        verifySaveMustFail(workOrganization, null, expectedExceptions);
    }

    public void testFailureReadOnlyScheduleDates() throws AssertionError,
            Exception
    {
        workOrganization = new WorkOrganization();
        
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar());
        actualDate.setDurationInMinutes(new Integer(4000));

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(5000));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(3000));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(3000));

        workOrganization.setActualDate(actualDate);
        workOrganization.setForecastDate(forecastDate);
        workOrganization.setPlanDate(planDate);
        workOrganization.setProposedDate(proposedDate);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workOrganization
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_FIELD,
                    workOrganization.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workOrganization
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_FIELD,
                    workOrganization.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workOrganization
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_FIELD,
                    workOrganization.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, workOrganization
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_FIELD,
                    workOrganization.getID()),
        };

        verifySaveMustFail(workOrganization, scope, null, expectedWarnings);
        
        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkOrganization);
        
        WorkOrganization loadedOrganization = (WorkOrganization)rpmObjects[0];
        
        // assert that the values we set were not saved in the db
        assertEquals(null, loadedOrganization.getActualDate());
        assertEquals(null, loadedOrganization.getForecastDate());
        assertEquals(null, loadedOrganization.getPlanDate());
        assertNotSame(workOrganization.getProposedDate(), loadedOrganization.getProposedDate());
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        workOrganization = new WorkOrganization();

        // Set string values with max length + 1 and verify that they are not
        // accepted
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT + 1));
        workOrganization
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_REFERENCE_NUMBER_CHARACTER_LIMIT + 1));
        workOrganization
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT + 1));
        workOrganization.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MAX + 1));

        // Set integers to max value + 1 and verify that they are not accepted
        workOrganization.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, workOrganization
                    .getClass().getName(), ValidationConstants.NAME_FIELD,
                    workOrganization.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, workOrganization
                    .getClass().getName(),
                    ValidationConstants.REFERENCE_NUMBER_FIELD,
                    workOrganization.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, workOrganization
                    .getClass().getName(),
                    ValidationConstants.EXTERNAL_ID_FIELD, workOrganization
                        .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, workOrganization
                    .getClass().getName(), ValidationConstants.PRIORITY_FIELD,
                    workOrganization.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, workOrganization
                    .getClass().getName(),
                    ValidationConstants.PROBABILITY_PERCENT_FIELD,
                    workOrganization.getID()),
        };

        verifySaveMustFail(workOrganization, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        workOrganization = new WorkOrganization();

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Set integers to min value -1 and verify that they are not accepted
        workOrganization.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));
        workOrganization.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, workOrganization
                .getClass().getName(), ValidationConstants.PRIORITY_FIELD,
                workOrganization.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, workOrganization
                .getClass().getName(),
                ValidationConstants.PROBABILITY_PERCENT_FIELD,
                workOrganization.getID()),
        };

        verifySaveMustFail(workOrganization, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        workOrganization = new WorkOrganization();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, workOrganization
                .getClass().getName(), ValidationConstants.NAME_FIELD,
                workOrganization.getID())
        };

        verifySaveMustFail(workOrganization, null, expectedExceptions);
    }

    public void testInsertWithAttributePortfolio() throws AssertionError,
            Exception
    {
        workOrganization = new WorkOrganization();

        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);
        workOrganization.setPortfolio(portfolio);

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Attempt to save the WorkOrganization object
        workOrganization = (WorkOrganization) saveMustSucceed(workOrganization,
            WorkOrganization.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkOrganization);

        // Check that the loaded WorkOrganization is identical to the one we
        // requested to store
        WorkElementFixture.compare(workOrganization,
            (WorkOrganization) rpmObjects[0], null);
    }

    public void testInsertWithParentProject() throws AssertionError, Exception
    {
        workOrganization = new WorkOrganization();

        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        workOrganization.setParent(parentProject);

        // Attempt to save the WorkOrganization object
        workOrganization = (WorkOrganization) saveMustSucceed(workOrganization,
            WorkOrganization.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkOrganization);

        // Check that the loaded WorkOrganization is identical to the one we
        // requested to store
        WorkElementFixture.compare(workOrganization,
            (WorkOrganization) rpmObjects[0], null);
    }

    public void testInsertWithParentProposal() throws AssertionError, Exception
    {
        workOrganization = new WorkOrganization();

        Proposal parentProposal = (Proposal) WorkElementFixture
            .retrieveStaticFixture(new Proposal(), this);

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        workOrganization.setParent(parentProposal);

        // Attempt to save the WorkOrganization object
        workOrganization = (WorkOrganization) saveMustSucceed(workOrganization,
            WorkOrganization.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkOrganization);

        // Check that the loaded WorkOrganization is identical to the one we
        // requested to store
        WorkElementFixture.compare(workOrganization,
            (WorkOrganization) rpmObjects[0], null);
    }

    public void testInsertWithParentWorkOrganization() throws AssertionError,
            Exception
    {
        workOrganization = new WorkOrganization();

        WorkOrganization parentWorkOrganization = (WorkOrganization) WorkElementFixture
            .retrieveStaticFixture(new WorkOrganization(), this);

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        workOrganization.setParent(parentWorkOrganization);

        // Attempt to save the WorkOrganization object
        workOrganization = (WorkOrganization) saveMustSucceed(workOrganization,
            WorkOrganization.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkOrganization);

        // Check that the loaded WorkOrganization is identical to the one we
        // requested to store
        WorkElementFixture.compare(workOrganization,
            (WorkOrganization) rpmObjects[0], null);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        workOrganization = new WorkOrganization();

        // Set field values with maxLength and verify that they are accepted
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        workOrganization
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_REFERENCE_NUMBER_CHARACTER_LIMIT));
        workOrganization
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT));
        workOrganization.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MAX));

        // Set integers to max value and verify that they are accepted
        workOrganization.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));

        // Attempt to save the WorkOrganization object
        workOrganization = (WorkOrganization) saveMustSucceed(workOrganization,
            WorkOrganization.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkOrganization);

        // Check that the loaded WorkOrganization is identical to the one we
        // requested to store
        WorkElementFixture.compare(workOrganization,
            (WorkOrganization) rpmObjects[0], null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        workOrganization = new WorkOrganization();

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // Set integers to min value and verify that they are accepted
        workOrganization.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));
        workOrganization.setProbabilityPercent(new Integer(
            ValidationConstants.PROBABILITY_PERCENT_MIN));

        // Attempt to save the WorkOrganization object
        workOrganization = (WorkOrganization) saveMustSucceed(workOrganization,
            WorkOrganization.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkOrganization);

        // Check that the loaded WorkOrganization is identical to the one we
        // requested to store
        WorkElementFixture.compare(workOrganization,
            (WorkOrganization) rpmObjects[0], null);
    }

    public void testFailureInvalidState() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setState(true);

        workOrganization = new WorkOrganization();

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        workOrganization.setState(new DeliverableState());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, workOrganization
                .getClass().getName(), ValidationConstants.STATE_FIELD,
                workOrganization.getID())
        };

        verifySaveMustFail(workOrganization, scope, expectedExceptions);
    }

    public void testSuccessValidState() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setState(true);

        workOrganization = new WorkOrganization();

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        workOrganization.setState(DatafieldFixture
            .createState(workOrganization));

        // Attempt to save the WorkOrganization object
        workOrganization = (WorkOrganization) saveMustSucceed(workOrganization,
            WorkOrganization.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/WorkOrganization[id='"
                + workOrganization.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof WorkOrganization);

        // Check that the loaded WorkOrganization is identical to the one we
        // requested to store
        WorkElementFixture.compare(workOrganization,
            (WorkOrganization) rpmObjects[0], scope);
    }

    public void testFailureInvalidTimeCodes() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setTimeCodes(true);

        workOrganization = new WorkOrganization();

        // Set the mandatory fields
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));

        // assign the timecodes 
        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
        TimeCode timecode1 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, true, this);
        TimeCode timecode2 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, false, this);

        workOrganization.setTimeCode1(timecode2);
        workOrganization.setTimeCode2(timecode1);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY,
                    workOrganization.getClass().getName(),
                    ValidationConstants.TIME_CODE1_FIELD, workOrganization
                        .getID()),
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY,
                    workOrganization.getClass().getName(),
                    ValidationConstants.TIME_CODE2_FIELD, workOrganization
                        .getID()),
        };

        verifySaveMustFail(workOrganization, scope, expectedExceptions);
    }
    
    public void testReadOnlyFields() throws Exception
    {
        workOrganization = new WorkOrganization();
        workOrganization
            .setName(getTestString(ValidationConstants.PROJECT_NAME_CHARACTER_LIMIT));
        
        super.testReadOnlyFields(workOrganization);
    }
}
