/*
 * Created on 3-Oct-2006
 * @author mmatwiy
 * Comparison methods for Workflow containers.
 */
package com.ibm.rpm.client.junit.workflow.fixtures;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.applicationadministration.fixtures.ContactGroupFixture;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.security.fixtures.SecurityRoleFixture;
import com.ibm.rpm.workflow.containers.AvailableWorkflowProcess;
import com.ibm.rpm.workflow.containers.GenericWorkflow;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowCategory;
import com.ibm.rpm.workflow.containers.WorkflowGroup;
import com.ibm.rpm.workflow.containers.WorkflowRole;
import com.ibm.rpm.workflow.containers.WorkflowRoleMapping;
import com.ibm.rpm.workflow.scope.AvailableWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowRoleMappingScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class WorkflowFixture extends RPMObjectFixture
{

    public static void compare(GenericWorkflow w1, GenericWorkflow w2,
            WorkflowScope scope) throws AssertionFailedError, Exception
    {
        compare(w1, w2);

        if (scope != null && scope.getParent() != null)
        {
            if ((w1.getParent() == null && w2.getParent() != null)
                    || (w1.getParent() != null && w2.getParent() == null))

            {
                Assert
                    .fail("One of the GenericWorkflow's parents is null and the other is not.");
            }

            if (w1.getParent() != null && w2.getParent() != null)
            {
                Assert.assertEquals(w1.getParent().getClass(), w2.getParent()
                    .getClass());
                Assert.assertEquals(w1.getParent().getID(), w2.getParent()
                    .getID());
            }
        }

        if (scope != null && scope.getChildren() != null)
        {
            if (w1 instanceof WorkflowGroup)
            {
                WorkflowGroup group1 = (WorkflowGroup) w1;
                WorkflowGroup group2 = (WorkflowGroup) w2;

                WorkflowFixture.compare(group1, group2, scope.getChildren());
            }
            else if (w1 instanceof WorkflowCategory)
            {
                WorkflowCategory category1 = (WorkflowCategory) w1;
                WorkflowCategory category2 = (WorkflowCategory) w2;

                WorkflowFixture.compare(category1, category2, scope
                    .getChildren());
            }
        }
    }

    public static void compare(AvailableWorkflowProcess w1,
            AvailableWorkflowProcess w2, AvailableWorkflowProcessScope scope)
            throws AssertionFailedError, Exception
    {
        compare(w1, w2);

        if (scope != null && scope.getChildren() != null)
        {
            WorkflowFixture.compare(w1.getWorkflowProcess(), w2
                .getWorkflowProcess(), scope.getChildren());
        }
    }

    public static void compare(RunningWorkflowProcess w1,
            RunningWorkflowProcess w2, RunningWorkflowProcessScope scope)
            throws AssertionFailedError, Exception
    {
        compare(w1, w2);

        if (scope != null && scope.getChildren() != null)
        {
            WorkflowFixture.compare(w1.getWorkflowProcess(), w2
                .getWorkflowProcess(), scope.getChildren());
        }
    }

    public static void compare(WorkflowRole r1, WorkflowRole r2)
            throws AssertionFailedError, Exception
    {
        compare(r1, r2);
    }

    public static void compare(WorkflowRoleMapping w1, WorkflowRoleMapping w2,
            WorkflowRoleMappingScope scope) throws AssertionFailedError,
            Exception
    {
        compare(w1, w2);

        compare(w1.getWorkflowRole(), w2.getWorkflowRole());

        if (scope != null)
        {
            if (scope.getResource() != null)
            {
                ResourceFixture.compare(w1.getResource(), w2.getResource(),
                    scope.getResource());
            }
            if (scope.getContactGroup() != null)
            {
                ContactGroupFixture.compare(w1.getContactGroup(), w2
                    .getContactGroup(), scope.getContactGroup());
            }
            if (scope.getSecurityRole() != null)
            {
                SecurityRoleFixture.compare(w1.getSecurityRole(), w2
                    .getSecurityRole());
            }
        }
    }

}