/*
 * Created on 24-Oct-06
 * @author mmatwiy
 * Test workflow process control of a hanged process.
 */
package com.ibm.rpm.client.junit.workflow.testcases;

import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.constants.ErrorCodes;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class TestHangedWorkflowProcess extends AbstractTestWorkflowProcess
{
    private WorkflowScope workflowScope = null;

    private RunningWorkflowProcessScope runningScope = null;

    private WorkElementScope workElementScope = null;

    private WorkflowProcess process = null;

    private Proposal proposal = null;

    private static final String PROCESS_NAME = "HangedProcess";

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        workflowScope = new WorkflowScope();
        workflowScope.setParent(new WorkflowScope());
        workflowScope.setChildren(new WorkflowScope());

        runningScope = new RunningWorkflowProcessScope();
        runningScope.setChildren(workflowScope);

        workElementScope = new WorkElementScope();
        workElementScope.setWorkflowProcessesRunning(runningScope);

        //TODO: replace this load once the API is able to create a WorkflowProcess
        RPMObject[] objects = loadFromXpath("/WorkflowProcess[name='"
                + PROCESS_NAME + "']", workflowScope);
        assertNotNull(objects);
        process = (WorkflowProcess) objects[0];

        proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);
    }

    public void tearDown() throws Exception
    {
        // reload to get the running processes
        RPMObject[] objects = loadFromXpath("/Proposal[id='" + proposal.getID()
                + "']", workElementScope);
        assertNotNull(objects);
        proposal = (Proposal) objects[0];

        if (proposal.getWorkflowProcessesRunning() != null)
        {
            // cancel the running processes so that everything can be deleted
            for (int i = 0; i < proposal.getWorkflowProcessesRunning().length; i++)
            {
                cancelProcess(proposal.getWorkflowProcessesRunning()[i]);
            }
        }

        super.tearDown();
    }

    public void testWorkflowControl() throws Exception
    {
        // associate the process so that we can start it
        associateProcess(process, proposal, false);

        // start the process  
        SingleResult startResult = getApplicationInterface()
            .startWorkflow(getSessionID(), process, proposal, runningScope);
        
        assertTrue(ErrorCodes.HANGED_PROCESS == startResult.getErrors()[0].getErrorID());
        
        RunningWorkflowProcess runningProcess = (RunningWorkflowProcess) startResult.getRpmObject();

        assertNotNull(runningProcess);
        assertTrue(runningProcess.getState() == WorkflowState.Hanged);

        // suspend the process
        RunningWorkflowProcess suspendedProcess = suspendProcess(
            runningProcess, runningScope);
        assertTrue(suspendedProcess.getState() == WorkflowState.Suspended);

        // resume the process
        RunningWorkflowProcess resumedProcess = resumeProcess(suspendedProcess,
            runningScope);
        assertTrue(resumedProcess.getState() == WorkflowState.Hanged);

        // cancel the process
        cancelProcess(resumedProcess);

        // reload to check that the process has been cancelled
        RPMObject[] objects = loadFromXpath("/Proposal[id='" + proposal.getID()
                + "']", workElementScope);
        assertNotNull(objects);
        proposal = (Proposal) objects[0];

        assertNull(proposal.getWorkflowProcessesRunning());

    }

}
