/*
 * Created on 2-Oct-2006
 * @author mmatwiy
 * Validation test methods for associating a workflow to a project.
 */
package com.ibm.rpm.client.junit.workflow.testcases.validation;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.workflow.constants.ValidationConstants;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class TestAssociateWorkflowProcessValidation extends GenericValidation
{
    WorkflowScope scope = null;

    Risk risk = null;

    WorkOrganization workOrganization = null;

    public final static String RISK_PROCESS_NAME = "RiskProcess1";

    public void setUp() throws Exception
    {
        super.setUp();

        scope = new WorkflowScope();
    }

    public void testFailureAssociateNullProcessToNullGenericProject()
            throws Exception
    {
        // Associate a null WorkflowProcess object to a null GP
        Result result = getApplicationInterface().associateWorkflow(sessionid,
            null, null, true, true);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NO_ID, "WorkflowProcess"),
                createRPMException(ErrorCodes.NO_ID, "GenericProject")
        };

        verifyProcessFailure(null, expectedExceptions, null, result);
    }

    public void testFailureAssociateUnsavedProcessToUnsavedGenericProject()
            throws Exception
    {
        WorkflowProcess process = new WorkflowProcess();
        Project project = new Project();

        // Associate an unsaved WorkflowProcess object to an unsaved GP
        Result result = getApplicationInterface().associateWorkflow(sessionid,
            process, project, true, true);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NO_ID, process.getClass()
                    .getName()),
                createRPMException(ErrorCodes.NO_ID, project.getClass()
                    .getName())
        };

        verifyProcessFailure(process, expectedExceptions, null, result);
    }

    public void testFailureAssociateInactiveProcess() throws Exception
    {
        // create a target GenericProject
        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        // TODO: replace this with a fixture that creates a WorkflowProcess
        // once it's possible to create a WorkflowProcess using the API,
        // make sure it is not activated
        RPMObject[] objects = loadFromXpath(
            "/WorkflowProcess[name='RiskProcessInactive']", scope);
        assertNotNull(objects);
        WorkflowProcess process = (WorkflowProcess) objects[0];

        // Attempt to associate the inactive process
        Result result = getApplicationInterface().associateWorkflow(sessionid,
            process, project, true, true);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.workflow.constants.ErrorCodes.INACTIVE_WORKFLOW,
                process.getClass().getName(), ValidationConstants.ACTIVE_FIELD,
                process.getID())
        };

        verifyProcessFailure(process, expectedExceptions, null, result);
    }

    public void testFailureSetDefaultWithoutAssociate() throws Exception
    {
        // create a target GenericProject
        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        // TODO: replace this with a fixture that creates a WorkflowProcess
        // once it's possible to create a WorkflowProcess using the API,
        // make sure it is not activated
        RPMObject[] objects = loadFromXpath(
            "/WorkflowProcess[name='RiskProcess1']", scope);
        assertNotNull(objects);
        WorkflowProcess process = (WorkflowProcess) objects[0];

        // Attempt to set the default flag without the associate flag
        Result result = getApplicationInterface().associateWorkflow(sessionid,
            process, project, false, true);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.workflow.constants.ErrorCodes.WORKFLOW_DEFAULT_WITHOUT_ASSOCIATE,
                process.getClass().getName())
        };

        verifyProcessFailure(process, expectedExceptions, null, result);
    }

    private void verifyProcessFailure(RPMObject obj,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings, Result result)
            throws AssertionError, Exception
    {
        // Check that we received exceptions if we were expecting some
        // and that they are the correct ones
        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
            expectedWarnings, result);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }
}