package com.ibm.rpm.client.junit.xpath.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author Genevieve
 *  
 */
public class TestPoolResourceXPath extends ClientTestController
{

    Pool parentPool = null;

    Pool childPool = null;

    Resource resource = null;

    Pool loadedChildPool = null;

    Pool loadedParentPool = null;

    Resource loadedResource = null;

    public void setUp() throws Exception
    {
        super.setUp();
        PoolScope mainScope = new PoolScope();
        PoolScope scope = new PoolScope();
        mainScope.setPools(scope);
        scope.setPools(new PoolScope());
        scope.setResources(new ResourceScope());

        parentPool = PoolFixture.createFixture(false, 1, mainScope, this);
        SingleResult result = save(parentPool, mainScope);

        parentPool = (Pool) result.getRpmObject();
        childPool = parentPool.getPools()[0];
        resource = childPool.getResources()[0];
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testResourceInPoolXpath() throws Exception
    {

        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());

        RPMObject[] objects = loadFromXpath("/Pool[name = '"
                + childPool.getName() + "']/Resource[username='"
                + resource.getUserName() + "']", scope);

        assertNotNull(objects);
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof Resource);
            loadedResource = (Resource) objects[i];
            loadedChildPool = (Pool) loadedResource.getParent();
            assertTrue(loadedResource.getUserName().equalsIgnoreCase(
                    resource.getUserName()));
            assertTrue(loadedChildPool.getName().equalsIgnoreCase(
                    childPool.getName()));
        }
    }

    public void testResourceInPoolInPoolXpath() throws Exception
    {

        ResourceScope scope = new ResourceScope();
        scope.setParent(new PoolScope());
        scope.getParent().setParent(new PoolScope());

        RPMObject[] objects = loadFromXpath("/Pool[name = '"
                + parentPool.getName() + "']/Pool[name = '"
                + childPool.getName() + "']/Resource[username='"
                + resource.getUserName() + "']", scope);

        assertNotNull(objects);

        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof Resource);
            loadedResource = (Resource) objects[i];
            loadedChildPool = (Pool) loadedResource.getParent();
            loadedParentPool = (Pool) loadedChildPool.getParent();
            assertTrue(loadedResource.getUserName().equalsIgnoreCase(
                    resource.getUserName()));
            assertTrue(loadedChildPool.getName().equalsIgnoreCase(
                    childPool.getName()));
            assertTrue(loadedParentPool.getName().equalsIgnoreCase(
                    parentPool.getName()));
        }
    }

    public void testPoolInPoolXpath() throws Exception
    {

        PoolScope scope = new PoolScope();
        scope.setParent(new PoolScope());

        RPMObject[] objects = loadFromXpath("/Pool[name = '"
                + parentPool.getName() + "']/Pool[name = '"
                + childPool.getName() + "']", scope);

        assertNotNull(objects);
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof Pool);
            loadedChildPool = (Pool) objects[i];
            loadedParentPool = (Pool) loadedChildPool.getParent();
            assertTrue(loadedChildPool.getName().equalsIgnoreCase(
                    childPool.getName()));
            assertTrue(loadedParentPool.getName().equalsIgnoreCase(
                    parentPool.getName()));
        }
    }

    public void testPoolWithPoolXpath() throws Exception
    {

        PoolScope scope = new PoolScope();
        scope.setPools(new PoolScope());

        RPMObject[] objects = loadFromXpath("/Pool[name = '"
                + parentPool.getName() + "'][Pool[name = '"
                + childPool.getName() + "']]", scope);

        assertNotNull(objects);
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof Pool);
            loadedParentPool = (Pool) objects[i];
            loadedChildPool = loadedParentPool.getPools()[0];
            assertTrue(loadedChildPool.getName().equalsIgnoreCase(
                    childPool.getName()));
            assertTrue(loadedParentPool.getName().equalsIgnoreCase(
                    parentPool.getName()));
        }
    }

    public void testPoolWithResourceXpath() throws Exception
    {

        PoolScope scope = new PoolScope();
        scope.setResources(new ResourceScope());

        RPMObject[] objects = loadFromXpath("/Pool[name = '"
                + childPool.getName() + "'][Resource[username='"
                + resource.getUserName() + "']]", scope);

        assertNotNull(objects);
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof Pool);

            loadedChildPool = (Pool) objects[i];
            loadedResource = loadedChildPool.getResources()[0];
            assertTrue(loadedChildPool.getName().equalsIgnoreCase(
                    childPool.getName()));
            assertTrue(loadedResource.getUserName().equalsIgnoreCase(
                    resource.getUserName()));
        }
    }

    public void testPoolWithPoolWithResourceXpath() throws Exception
    {

        PoolScope scope = new PoolScope();
        scope.setPools(new PoolScope());
        scope.getPools().setResources(new ResourceScope());

        RPMObject[] objects = loadFromXpath("/Pool[name = '"
                + parentPool.getName() + "'][Pool[name = '"
                + childPool.getName() + "'][Resource[username='"
                + resource.getUserName() + "']]]", scope);

        assertNotNull(objects);
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof Pool);
            loadedParentPool = (Pool) objects[i];
            loadedChildPool = loadedParentPool.getPools()[0];
            loadedResource = loadedChildPool.getResources()[0];
            assertTrue(loadedParentPool.getName().equalsIgnoreCase(
                    parentPool.getName()));
            assertTrue(loadedChildPool.getName().equalsIgnoreCase(
                    childPool.getName()));
            assertTrue(loadedResource.getUserName().equalsIgnoreCase(
                    resource.getUserName()));
        }

    }
}
