package com.ibm.rpm.client.junit.xpath.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetency;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author Genevieve
 *  
 */
public class TestResourceAttributeAssignmentXPath extends ClientTestController
{

    private ResourceScope scope = null;

    private RPMObjectScope proficiencyScope = null;

    private Resource resource = null;

    private ResourceAttributeCompetency competency = null;

    private ResourceAttributeCompetencyProficiencyLevel proficiency = null;

    private Resource loadedResource = null;

    private ResourceAttributeCompetency loadedCompetency = null;

    private ResourceAttributeCompetencyProficiencyLevel loadedProficiency = null;

    public void setUp() throws Exception
    {
        super.setUp();

        scope = new ResourceScope();
        scope.setCompetencyAttributeAssignments(true);

        Pool pool = PoolFixture.createSaveFixture(this);
        resource = ResourceFixture.createFixture(pool, scope, this);
        
        SingleResult result = save(resource, scope);
        resource = (Resource) result.getRpmObject();
        
        proficiency = (ResourceAttributeCompetencyProficiencyLevel) resource
        .getCompetencyAttributeAssignments()[0]
        .getProficiencyLevel();
        
        //Reload proficiency with parent
        proficiencyScope = new RPMObjectScope();
        proficiencyScope.setParent(new ResourceAttributeScope());
        
        proficiency = (ResourceAttributeCompetencyProficiencyLevel) loadFromID(
                proficiency, proficiencyScope);
        competency = (ResourceAttributeCompetency) proficiency
                .getParent();


    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testResourceCompetencyAssignment() throws Exception
    {
       
        String xpath = "/Resource[userName='" + resource.getUserName() + "']"
            + "[@competencyattributeassignments[/ResourceAttributeCompetency[name='"
            + competency.getName()
            + "']/ResourceAttributeCompetencyProficiencyLevel[name='"
            + proficiency.getName() + "']]]";

        RPMObject[] objects = loadFromXpath(xpath, scope);
        
        assertNotNull(objects);
        assertEquals(objects.length, 1);
        assertTrue(objects[0] instanceof Resource);

        loadedResource = (Resource) objects[0];

        loadedProficiency = (ResourceAttributeCompetencyProficiencyLevel) loadedResource
            .getCompetencyAttributeAssignments()[0].getProficiencyLevel();

        loadedProficiency = (ResourceAttributeCompetencyProficiencyLevel) loadFromID(
            loadedProficiency, proficiencyScope);
        loadedCompetency = (ResourceAttributeCompetency) loadedProficiency
            .getParent();

        assertTrue(loadedResource.getUserName().equalsIgnoreCase(
            resource.getUserName()));
        assertTrue(loadedCompetency.getName().equalsIgnoreCase(
            competency.getName()));
        assertTrue(loadedProficiency.getName().equalsIgnoreCase(
            proficiency.getName()));

        

    }

}
