package com.ibm.rpm.client.junit.xpath.testcases;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.applicationadministration.types.SelectType;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultStepFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.timesheet.containers.DefaultStep;
import com.ibm.rpm.timesheet.scope.DefaultStepScope;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.scope.WbsModuleScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author gmeloche
 *  
 */
public class TestXPathUtil extends ClientTestController
{

    private Pool mainPool = null;

    private PoolScope mainScope = null;

    private Pool subPool = null;

    private Resource resource = null;

    private RPMCalendar calendar = null;

    private ChargeCodeCategory chargeCodeCategory = null;

    private ChargeCode chargeCode = null;

    private ChargeCodeScope chargeCodeScope = null;

    AttributeCategory rootcategory = null;

    AttributeClassification classification = null;

    Attribute attribute = null;

    public TestXPathUtil()
    {
        super();
    }

    protected void setUp() throws Exception
    {
        super.setUp();

    }

    private void createPoolObject() throws Exception
    {
        ResourceScope resScope = new ResourceScope();
        resScope.setCalendar(new CalendarScope());

        PoolScope subpoolScope = new PoolScope();
        subpoolScope.setResources(resScope);

        mainScope = new PoolScope();
        mainScope.setPools(subpoolScope);

        mainPool = PoolFixture.createFixture(false, 1, mainScope, this);

        mainPool = (Pool) save(mainPool, mainScope).getRpmObject();

        mainPool = (Pool) loadFromID(mainPool, mainScope);

        subPool = mainPool.getPools()[0];
        resource = subPool.getResources()[0];
        calendar = resource.getCalendar();

    }

    private void createChargeCodeObject() throws Exception
    {

        chargeCodeScope = ChargeCodeFixture.createChargeCodeTreeScope(2);
        chargeCodeCategory = (ChargeCodeCategory) ChargeCodeFixture
            .createGenericChargeCode(2, false);
        chargeCodeCategory = (ChargeCodeCategory) save(chargeCodeCategory,
            chargeCodeScope).getRpmObject();

        chargeCode = (ChargeCode) chargeCodeCategory.getChildren()[0];
    }

    private void createAttributeObject() throws Exception
    {
        int index = (int) (Math.random() * 100);

        WbsModule module = new WbsModule();

        rootcategory = new AttributeCategory();
        rootcategory.setName(HelperTestUtilities.createName() + index);
        rootcategory.setParent(module);

        classification = new AttributeClassification();
        classification.setName(HelperTestUtilities.createName() + index);
        classification.setSelectType(SelectType.Multi);
        rootcategory.setClassifications(new AttributeClassification[]
        {
            classification
        });

        attribute = new Attribute();
        attribute.setName(HelperTestUtilities.createName() + index);
        attribute.setInputType(InputType.Text);

        classification.setAttributes(new Attribute[]
        {
            attribute
        });

        module.setAttributeCategories(new AttributeCategory[]
        {
            rootcategory
        });

        AttributeScope attrscope = new AttributeScope();
        attrscope.setClassifications(attrscope);
        attrscope.setAttributes(attrscope);
        attrscope.setCategories(attrscope);

        WbsModuleScope modulescope = new WbsModuleScope();
        modulescope.setAttributeCategories(attrscope);

        save(module, modulescope).getRpmObject();

        RPMObject[] objects = loadFromXpath(
            "wbsmodule/attributecategory/attributeclassification/attribute[name = '"
                    + attribute.getName() + "']", null);
        attribute = (Attribute) objects[0];
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testLoadSimpleObject() throws Exception
    {

        //The manager of this object doesn't implements getFilter()
        DefaultStepScope scope = DefaultStepFixture.createScope(true,
            true);
        Project parent = (Project) WorkElementFixture.retrieveStaticFixture(
            new Project(), this);

        DefaultStep defaultStep = DefaultStepFixture.createFixture(
            parent, scope, this);

        defaultStep = (DefaultStep) save(defaultStep, scope).getRpmObject();

        String xpath = "DefaultStep[@id = '" + defaultStep.getID() + "']";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertTrue(objects.length == 1);

    }

    /*
     * /Pool[@id = '66D33ADC66C047DE83052A37D8F33A95']/pools[id =
     * '103D798B279B424B9ED5B69495D8CA5C']
     */
    public void testLoadWithParentAndField() throws Exception
    {

        createPoolObject();

        //@ before id shows that @ prefix has no impact when put in a
        // condition.

        String xpath = "/Pool[@id = '" + mainPool.getID() + "']/pools[id = '"
                + subPool.getID() + "']";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertTrue(objects.length == 1);
    }

    /*
     * /Pool[@id = '66D33ADC66C047DE83052A37D8F33A95']/pools[id =
     * '103D798B279B424B9ED5B69495D8CA5C']
     */
    public void testLoadWithParentAndForcedField() throws Exception
    {

        createPoolObject();

        String xpath = "/Pool[id = '" + mainPool.getID() + "']/@pools[id = '"
                + subPool.getID() + "']";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertTrue(objects.length == 1);
    }

    /*
     * /Pool[id = '66D33ADC66C047DE83052A37D8F33A95']/pools[id =
     * '103D798B279B424B9ED5B69495D8CA5C']/resources[id =
     * '32DCD9A3CBB7461BAA33D9C1BB9E06E0']
     */
    public void testLoadWithParentAndFieldAndField() throws Exception
    {
        createPoolObject();

        String xpath = "/Pool[id = '" + mainPool.getID() + "']/pools[id = '"
                + subPool.getID() + "']/resources[id = '" + resource.getID()
                + "']";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }

    /*
     * /Pool[id = '66D33ADC66C047DE83052A37D8F33A95'][pools[id =
     * '103D798B279B424B9ED5B69495D8CA5C']]
     */
    public void testLoadWithParentWithField_OldWay() throws Exception
    {
        createPoolObject();

        String xpath = "/Pool[id = '" + mainPool.getID() + "'][pools[id = '"
                + subPool.getID() + "']]";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }

    /*
     * /Pool[id = '66D33ADC66C047DE83052A37D8F33A95' and pools[id =
     * '103D798B279B424B9ED5B69495D8CA5C']]
     */
    public void testLoadWithParentWithField() throws Exception
    {
        createPoolObject();

        String xpath = "/Pool[id = '" + mainPool.getID() + "' and pools[id = '"
                + subPool.getID() + "']]";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }

    /*
     * /Pool[id = '12345' or pools[id = '103D798B279B424B9ED5B69495D8CA5C'] or
     * id = '123']
     */
    public void testLoadWithParentOrFieldOrField() throws Exception
    {
        createPoolObject();

        String xpath = "/Pool[id = '12345' or pools[id = '" + subPool.getID()
                + "'] or id = '123']";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }

    /*
     * /Pool[id = 'F7AE112930B54DEFAF2A119E44F2CAA4'][pools[id =
     * 'F36DAFE825F44EF1AFE7DD1F2DB23E5E'][resources[id =
     * '243D0C66F7E745F7B4B88E772A7FD640']]]
     */
    public void testLoadWithParentWithFieldWithField_OldWay() throws Exception
    {
        createPoolObject();

        String xpath = "/Pool[id = '" + mainPool.getID() + "'][pools[id = '"
                + subPool.getID() + "'][resources[id = '" + resource.getID()
                + "']]]";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }

    /*
     * /Pool[id = 'F7AE112930B54DEFAF2A119E44F2CAA4' and pools[id =
     * 'F36DAFE825F44EF1AFE7DD1F2DB23E5E' and resources[id =
     * '243D0C66F7E745F7B4B88E772A7FD640']]]
     */
    public void testLoadWithParentWithFieldWithField() throws Exception
    {
        createPoolObject();

        String xpath = "/Pool[id = '" + mainPool.getID() + "' and pools[id = '"
                + subPool.getID() + "' and resources[id = '" + resource.getID()
                + "']]]";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }

    /*
     * /Pool[id = 'F7AE112930B54DEFAF2A119E44F2CAA4'][pools[id =
     * 'F36DAFE825F44EF1AFE7DD1F2DB23E5E'][resources[id =
     * '243D0C66F7E745F7B4B88E772A7FD640'][calendar[id =
     * '2E102BE61F11468CA743C3BE483060A7']]]]
     */
    public void testLoadWithParentWithFieldinField_OldWay() throws Exception
    {

        createPoolObject();

        String xpath = "/Pool[id = '" + mainPool.getID() + "'][pools[id = '"
                + subPool.getID() + "'][resources[id = '" + resource.getID()
                + "'][calendar[id = '" + calendar.getID() + "']]]]";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }

    /*
     * /Pool[id = 'F7AE112930B54DEFAF2A119E44F2CAA4' and pools[id =
     * 'F36DAFE825F44EF1AFE7DD1F2DB23E5E' and resources[id =
     * '243D0C66F7E745F7B4B88E772A7FD640' and calendar[id =
     * '2E102BE61F11468CA743C3BE483060A7']]]]
     */
    public void testLoadWithParentWithFieldinField() throws Exception
    {

        createPoolObject();

        String xpath = "/Pool[id = '" + mainPool.getID() + "' and pools[id = '"
                + subPool.getID() + "' and resources[id = '" + resource.getID()
                + "' and calendar[id = '" + calendar.getID() + "']]]]";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }


    public void testLoadWithParentWithContainerWithContainerAndContainer()
            throws Exception
    {

        createPoolObject();

        String xpath = "/Pool[pools[@resources[fullname = '" + resource.getFullName()
                + "']] and @pools[name = '" + subPool.getName()
                + "']]";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }

    public void testLoadWithProjectWithAttributeAssignmentWithAttributeOrAttribute()
            throws Exception
    {

        createAttributeObject();
        Project project = new Project();
        project.setName(HelperTestUtilities.createName() + (int)(Math.random() * 100));
        AttributeAssignment assignment = new AttributeAssignment();
        assignment.setAttribute(attribute);
        project.setAttributeAssignments(new AttributeAssignment[]
        {
            assignment
        });

        WorkElementScope scope = new WorkElementScope();
        scope.setAttributeAssignments(true);
        project = (Project) save(project, scope).getRpmObject();

        String xpath = "project[attributeassignments[attribute[name = 'aaaa'] or attribute[name = '"
                + attribute.getName() + "']]]";

        RPMObject[] objects = loadFromXpath(xpath, scope);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }

    //"Project[attributeassignment[attribute[name = 'TextAttribute'] or
    // attribute[[name = 'TextAttribute']]]]"

    /*
     * /ChargeCodeCategory[id =
     * '8B7B4BFF0FFA495C988CA51BE196FCAA']/GenericChargeCode[id =
     * '331D351A62794F3DA0653BF224F14099']
     */
    public void testLoadChildrenWithGenericClassName() throws Exception
    {

        createChargeCodeObject();

        String xpath = "/ChargeCodeCategory[id = '"
                + chargeCodeCategory.getID() + "']/GenericChargeCode[id = '"
                + chargeCode.getID() + "']";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }

    /*
     * /ChargeCodeCategory[id =
     * '5F7AB72047004309BF5BB2832FBD913C']/ChargeCode[id =
     * '92C2899FF868483EAB1DEDC3509EC232']
     */
    public void testLoadChildrenWithSpecificClassName() throws Exception
    {

        createChargeCodeObject();

        String xpath = "/ChargeCodeCategory[id = '"
                + chargeCodeCategory.getID() + "']/ChargeCode[id = '"
                + chargeCode.getID() + "']";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertNotNull(objects[0]);

    }

    public void testLoadObjectWithNullField() throws Exception
    {
        PoolDomain domain = new PoolDomain();
        domain.setName(HelperTestUtilities.createName());

        domain = (PoolDomain) save(domain, null).getRpmObject();

        String xpath = "Pooldomain[@id = '" + domain.getID() + "' and isnull(@description) ]";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertTrue(objects.length == 1);

    }
    
    public void testLoadObjectWithNotNullField() throws Exception
    {
        PoolDomain domain = new PoolDomain();
        domain.setName(HelperTestUtilities.createName());
        domain.setDescription("My Domain");

        domain = (PoolDomain) save(domain, null).getRpmObject();

        String xpath = "Pooldomain[@id = '" + domain.getID() + "' and isnotnull(@description) ]";

        RPMObject[] objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertTrue(objects.length == 1);

    }
    

}
