package com.ibm.rpm.client.junit.xpath.testcases;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.applicationadministration.types.SelectType;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.communications.containers.GenericCommunication;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.communications.types.WorkflowResponseState;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.types.ElementSecurityType;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.financial.types.RevenueRecognitionMethodType;
import com.ibm.rpm.financial.types.TimePhasedType;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.types.ProbabilityType;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ConstraintType;
import com.ibm.rpm.wbs.types.DependencyType;
import com.ibm.rpm.wbs.types.LeadLagType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;
import com.ibm.rpm.wbs.types.ResourceType;
import com.ibm.rpm.wbs.types.ScopeElementSecurityType;

/**
 * @author gmeloche
 * 
 */
public class TestXPathWithEnumeratedTypes extends ClientTestController
{

    public TestXPathWithEnumeratedTypes()
    {
        super();
    }

    private void loadFromXpath(Class container, String fieldName,
            Class enumeratedTypeClass, Object enumeratedType) throws Exception
    {

        loadFromXpath(container, fieldName, enumeratedTypeClass,
            enumeratedType, "id = 'abc'");

    }

    private void loadFromXpath(Class container, String fieldName,
            Class enumeratedTypeClass, Object enumeratedType,
            String additionalCondition) throws Exception
    {

        logger.debug("Testing " + enumeratedTypeClass.getName());

        if (additionalCondition == null)
        {
            throw new Exception("Additionnal condition must be specified.");
        }

        StringBuffer sb = new StringBuffer();
        sb.append("/");
        sb.append(StringUtil.getShortClassName(container));
        sb.append("[");

        sb.append(fieldName);
        sb.append("=");
        sb.append("'");
        sb.append(enumeratedType);
        sb.append("'");

        // Additional condition to avoid loading objects.
        sb.append(" and ");
        sb.append(additionalCondition);

        sb.append("]");

        loadFromXpath(sb.toString(), null);

    }

    public void testInputType() throws Exception
    {
        loadFromXpath(Attribute.class, "inputType", InputType.class,
            InputType.Date);
    }

    public void testSelectType() throws Exception
    {
        loadFromXpath(AttributeClassification.class, "selectType",
            SelectType.class, SelectType.Multi);
    }

    public void testFieldInputType() throws Exception
    {
        loadFromXpath(CustomField.class, "inputType", FieldInputType.class,
            FieldInputType.Datafield);
    }

    public void testElementSecurityType() throws Exception
    {
        loadFromXpath(Document.class, "ElementSecurityType",
            ElementSecurityType.class, ElementSecurityType.ProjectLevel);
    }

    public void testScopeElementSecurityType() throws Exception
    {
        loadFromXpath(Requirement.class, "ElementSecurityType",
            ScopeElementSecurityType.class,
            ScopeElementSecurityType.ProjectLevel);
    }
    
    public void testPublishedType() throws Exception
    {
        loadFromXpath(Document.class, "published", PublishedType.class,
            PublishedType.ToTemplate);

        loadFromXpath(Issue.class, "published", PublishedType.class,
            PublishedType.ToTemplate);
    }

    // TODO uncomment when xpath is enabled on BudgetType
    // public void testBudgetType() throws Exception
    // {
    // loadFromXpath(TimePhasedFinancial.class, "budgetType", BudgetType.class,
    // BudgetType.Actual);
    // }

    public void testRevenueRecognitionMethodType() throws Exception
    {
        loadFromXpath(WbsFinancial.class, "revrectype",
            RevenueRecognitionMethodType.class,
            RevenueRecognitionMethodType.Ratio);
    }

    public void testTimePhasedType() throws Exception
    {
        loadFromXpath(WorksheetFinancial.class, "timePhased",
            TimePhasedType.class, TimePhasedType.ProjectAssignment);
    }

    public void testResourceType() throws Exception
    {
        loadFromXpath(ResourceTaskAssignment.class, "resourceType",
            ResourceType.class, ResourceType.Profile);
    }

    // TODO uncomment when xpath is enabled on RiskMatrix
    /*
     * public void testPrecisionType() throws Exception {
     * loadFromXpath(RiskMatrix.class, "precision", PrecisionType.class,
     * PrecisionType.High); }
     */

    public void testProbabilityType() throws Exception
    {
        loadFromXpath(Scorecard.class, "ProbabilityType",
            ProbabilityType.class, ProbabilityType.ProjectCosts);

        loadFromXpath(ScorecardCategory.class, "ProbabilityType",
            ProbabilityType.class, ProbabilityType.ProjectCosts);
    }

    public void testAssignmentType() throws Exception
    {
        loadFromXpath(Project.class, "AssignmentType", AssignmentType.class,
            AssignmentType.Work);
    }

    public void testProjectPublishedType() throws Exception
    {
        loadFromXpath(Project.class, "published", ProjectPublishedType.class,
            ProjectPublishedType.ToTemplate);
    }

    public void testConstraintType() throws Exception
    {
        loadFromXpath(Task.class, "constrainttype", ConstraintType.class,
            ConstraintType.AsLateAsPossible);
    }

    public void testDependencyType() throws Exception
    {
        loadFromXpath(ElementDependency.class, "DependencyType",
            DependencyType.class, DependencyType.StartStart);
    }

    public void testLeadLagType() throws Exception
    {
        loadFromXpath(ElementDependency.class, "LeadLagType",
            LeadLagType.class, LeadLagType.Lag);
    }

    // Uncomment when TaskAssignment.assignmentFinishDateType,
    // assignmentStartDateType are loaded by the manager
    /*
     * public void testDateType_FinishDate() throws Exception {
     * loadFromXpath(TaskAssignment.class, "assignmentFinishDateType",
     * DateType.class, DateType.Planned); }
     * 
     * public void testDateType_StartDate() throws Exception {
     * loadFromXpath(TaskAssignment.class, "assignmentStartDateType",
     * DateType.class, DateType.Forecasted); }
     */

    public void testTimesheetStatus() throws Exception
    {
        loadFromXpath(TimesheetApprovalStatus.class, "approvalStatus",
            TimesheetStatus.class, TimesheetStatus.Submitted,
            "approvalnotes = 'blabla'");
    }

    public void testFinancialCategory() throws Exception
    {
        loadFromXpath(ChargeCode.class, "financialcategory",
            FinancialCategory.class, FinancialCategory.ExpenseLabor);
    }

    public void testCommunicationsFolder() throws Exception
    {
        loadFromXpath(WorkflowResponse.class, "folder",
            CommunicationsFolder.class, CommunicationsFolder.Inbox);
    }

    public void testWorkflowState() throws Exception
    {
        loadFromXpath(GenericCommunication.class, "state", WorkflowState.class,
            WorkflowState.Open);
    }

    public void testWorkflowResponseState() throws Exception
    {
        loadFromXpath(WorkflowResponse.class, "state",
            WorkflowResponseState.class, WorkflowResponseState.NotComplete);
    }
}
