package com.ibm.rpm.client.junit.xpath.testcases;

import java.lang.reflect.Method;

import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.framework.RPMObject;

/**
 * @author gmeloche
 *
 */
public class TestXPathWithSpecialCharacter extends ClientTestController
{

    private PoolDomain object = new PoolDomain();
    private String xpath = null;
    private RPMObject[] objects = null;

    protected void setUp() throws Exception
    {
        super.setUp();
        
        PoolDomain baseObject = new PoolDomain();
        baseObject.setName("A");
        save(baseObject,null);
        
    
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }
    
    
    //' as &apos;
    public void testLoadWithApostrophe() throws Exception
    {
        String name = "'xpath's";
        object.setName(name);
        save(object, null);

        xpath = "/PoolDomain[name = '&apos;xpath&apos;s']";

        objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertTrue(objects.length == 1);
        
        xpath = "/PoolDomain[contains(name, '&apos;xpath')]";

        objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertTrue(objects.length == 1);
        
        xpath = "/PoolDomain[starts-with(name, '&apos;')]";

        objects = loadFromXpath(xpath, null);

        assertNotNull(objects);
        assertTrue(objects.length == 1);
    }
    
    //Test other special character that doesn't need to be replaced 
    
    public void testLoad1() throws Exception{
        testLoadWithSpecialCharacter("~");
    }
    
    public void testLoad2() throws Exception{
        testLoadWithSpecialCharacter("`");
    }
    
    public void testLoad3() throws Exception{
        testLoadWithSpecialCharacter("!");
    }
    
    public void testLoad4() throws Exception{
        testLoadWithSpecialCharacter("@");
    }
    
    public void testLoad5() throws Exception{
        testLoadWithSpecialCharacter("#");
    }
    
    public void testLoad6() throws Exception{
        testLoadWithSpecialCharacter("$");
    }
    
    public void testLoad7() throws Exception{
        testLoadWithSpecialCharacter("%");
    }
    
    public void testLoad8() throws Exception{
        testLoadWithSpecialCharacter("^");
    }
    
    public void testLoad9() throws Exception{
        testLoadWithSpecialCharacter("&");
    }
    
    public void testLoad10() throws Exception{
        testLoadWithSpecialCharacter("*");
    }
    
    public void testLoad11() throws Exception{
        testLoadWithSpecialCharacter("(");
    }
    
    public void testLoad12() throws Exception{
        testLoadWithSpecialCharacter(")");
    }
    
    public void testLoad13() throws Exception{
        testLoadWithSpecialCharacter("_");
    }
    
    public void testLoad14() throws Exception{
        testLoadWithSpecialCharacter("-");
    }
    
    public void testLoad15() throws Exception{
        testLoadWithSpecialCharacter("+");
    }
    
    public void testLoad16() throws Exception{
        testLoadWithSpecialCharacter("=");
    }
       
    public void testLoad17() throws Exception{
        testLoadWithSpecialCharacter("{");
    }
    
    public void testLoad18() throws Exception{
        testLoadWithSpecialCharacter("}");
    }
    
    public void testLoad19() throws Exception{
        testLoadWithSpecialCharacter("[");
    }
    
    public void testLoad20() throws Exception{
        testLoadWithSpecialCharacter("]");
    }
    
    public void testLoad21() throws Exception{
        testLoadWithSpecialCharacter("|");
    }
    
    public void testLoad22() throws Exception{
        testLoadWithSpecialCharacter("\\");
    }
    
    public void testLoad24() throws Exception{
        testLoadWithSpecialCharacter(":");
    }
    
    public void testLoad25() throws Exception{
        testLoadWithSpecialCharacter(";");
    }
    
    public void testLoad26() throws Exception{
        testLoadWithSpecialCharacter("\"");
    }
    
    public void testLoad27() throws Exception{
        testLoadWithSpecialCharacter("<");
    }
    public void testLoad28() throws Exception{
        testLoadWithSpecialCharacter(">");
    }
    public void testLoad29() throws Exception{
        testLoadWithSpecialCharacter(",");
    }
 
    public void testLoad30() throws Exception{
        testLoadWithSpecialCharacter(".");
    }
    
    public void testLoad31() throws Exception{
        testLoadWithSpecialCharacter("?");
    }
    
    public void testLoad32() throws Exception{
        testLoadWithSpecialCharacter("/");
    }
    
    public void testLoadMultipleEscapeCharacters() throws Exception
    {
        // test a string that contains multiple characters that need
        // to be escaped (% and _)
        testLoadWithSpecialCharacter("%hi?_the>/re%_.");
    }
    
    private void testLoadWithSpecialCharacter(String name)
            throws Exception
    {
        
        //Create another object with one character.
        
        //name = "xpath" + name + "s";
        object.setName(name);
        save(object, null);
        
        //Load with name equals special character
        xpath = "/PoolDomain[name = '" + name + "']";

        objects = loadFromXpath(xpath, null);

        assertNotNull("XPath: " + xpath + "fails.",objects);
        validateResults(name, false);
        
        //Load with name contains special character
        xpath = "/PoolDomain[contains(name, '" + name + "')]";

        objects = loadFromXpath(xpath, null);

        assertNotNull("XPath: " + xpath + "fails.",objects);
        validateResults(name, true);
        
    }
    
    /**
     * Validates that the name is equal or contains the expected caracter (% and _
     * need to be considered as normal caracters.)
     * TODO gmeloche fix xpath parser to escapes % and _ in the generated sql.
     * 
     * @param excepted
     * @param contains
     * @throws Exception
     */
    private void validateResults(String excepted, boolean contains)
            throws Exception
    {

        for (int i = 0; i < objects.length; i++)
        {
            RPMObject o = objects[i];
            Class clazz = o.getClass();

            Method method = clazz.getMethod("getName", null);
            String value = (String) method.invoke(o, null);
            
            assertTrue(value !=null);

            if (contains)
            {
                assertTrue(value.indexOf(excepted) >= 0);
            }
            else
            {
                assertEquals(excepted, value);
            }
        }
    }

}
