/*
 * Created on 14-Dec-06
 * @author mmatwiy
 * 
 * TimesheetApprovalStatus[@resource[@id='2D0C494A7EFB4B8A84601460BCA0D3B8']
 * and @project[@id='8AF8E9F3A9F54E2E9CD229C19B39AD27'] and @weekOf='2006-10-07']
 * 
 * The loadfromXpath call is returning the following error:
 * Cannot perform a loadFromXpath on <Resource> as a child container of 
 * <TimesheetApprovalStatus> without specifying the field name. 
 * Supported field names: [resource, lastapprovedrejectedby]. 
 * An object name starting with a @ is interpreted as a field. - Severity: Error - Error ID: 400305
 * 
 * But I did specify which field. I explicitly said '@resource'.
 * 
 * Resolution: There was a change made to XpathUtil in order to fix some metadata join problems.
 * I've removed that change and will re-investigate what needs to happen to fix the metadata joins.
 */
package com.ibm.rpm.client.junit.xpath.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetApprovalStatusFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class Test59289 extends ClientTestController
{
    public void test() throws Exception
    {
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);

        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);

        SummaryTimesheetScope timesheetScope = SummaryTimesheetFixture
            .createScope(true, false, false, false, false);
        SummaryTimesheet summaryTimesheet = SummaryTimesheetFixture
            .createSaveFixture(taskAssignment, weekOf, timesheetScope, true,
                this);

        Resource resource = ((ResourceTaskAssignment) taskAssignment
            .getParent()).getResource();

        resource.getResourceSecurityRights().setCanUseWebServicesAPI(
            Boolean.TRUE);

        resource = (Resource) save(resource, null).getRpmObject();

        TimesheetApprovalStatus appStatus = new TimesheetApprovalStatus();

        appStatus.setParent(summaryTimesheet);
        appStatus.setResource(resource);
        appStatus.setWeekOf(weekOf);

        // submit the timesheet for approval
        TimesheetApprovalStatusFixture.submitTimesheetAndSave(appStatus, null,
            true, this);

        loadFromXpath("/TimesheetApprovalStatus[@resource[@id='"
                + resource.getID() + "']]", null);
    }
}
