package com.ibm.rpm.client.junit.z;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.clientcostcenters.containers.GenericClient;
import com.ibm.rpm.contract.containers.GenericContract;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetency;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeSkill;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillClassification;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.WorkElement;

/**
 * Cleans up some key tables of the test database.
 * 
 * @author legaults
 */
public class TestZCleanup extends ClientTestController
{

    public void testDeleteTestPortfolios() throws Exception
    {
        deleteAllTestObjects(Portfolio.class);
    }

    public void testDeleteTestWorkElement() throws Exception
    {
        deleteAllTestObjects(WorkElement.class);
    }

    public void testDeleteTestScopes() throws Exception
    {
        deleteAllTestObjects(ScopeElement.class);
    }

    public void testDeleteTestAssets() throws Exception
    {
        deleteAllTestObjects(AssetFolder.class);
        deleteAllTestObjects(Asset.class);
    }

    public void testDeleteTestClientsAndCostCenters() throws Exception
    {
        // Note: most will not be deleted.
        deleteAllTestObjects(GenericClient.class);
    }

    public void testDeleteTestContracts() throws Exception
    {
        deleteAllTestObjects(GenericContract.class);
    }

    public void testDeleteTestDocuments() throws Exception
    {
        // Note: most will not be deleted.
        deleteAllTestObjects(GenericDocument.class);
        deleteAllTestObjects(DocumentFolder.class);
    }

    public void testDeleteTestCalendars() throws Exception
    {
        deleteAllTestObjects(RPMCalendar.class);
    }

    public void testDeleteTestOrganizationals() throws Exception
    {
        deleteAllTestObjects(Organizational.class);
    }

    public void testDeleteTestGeographicals() throws Exception
    {
        deleteAllTestObjects(Geographical.class);
    }

    public void testDeleteTestPoolDomains() throws Exception
    {
        deleteAllTestObjects(PoolDomain.class);
    }

    public void testDeleteTestContactGroups() throws Exception
    {
        deleteAllTestObjects(ContactGroup.class);
    }

    public void testDeleteTestAttributes() throws Exception
    {
        deleteAllTestObjects(Attribute.class);
        deleteAllTestObjects(AttributeCategory.class);
        deleteAllTestObjects(AttributeClassification.class);
    }

    public void testDeleteTestPoolsAndResources() throws Exception
    {
        //resources are deleted with the pools
        deleteAllTestObjects(Pool.class);

    }

    public void testDeleteTestResourceCompetencies() throws Exception
    {
        deleteAllTestObjects(ResourceAttributeCompetency.class);
        deleteAllTestObjects(ResourceAttributeCompetencyClassification.class);
        deleteAllTestObjects(ResourceAttributeCompetencyCategory.class);
    }

    public void testDeleteTestResourceSkills() throws Exception
    {
        deleteAllTestObjects(ResourceAttributeSkill.class);
        deleteAllTestObjects(ResourceAttributeSkillClassification.class);
        deleteAllTestObjects(ResourceAttributeSkillCategory.class);
    }

    public void testDeleteTestScorecards() throws Exception
    {
        deleteAllTestObjects(ScorecardFolder.class);
    }

    public void testDeleteTestChargeCodes() throws Exception
    {
        ChargeCodeFixture.deleteAllChargeCodes(this);
    }
}
