package com.ibm.rpm.clientcostcenters.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.CostCenterState;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.managers.AttributeAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.AttributeManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.applicationadministration.managers.RtfAssignmentManager;
import com.ibm.rpm.asset.managers.AssetCrossChargeManager;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.containers.Division;
import com.ibm.rpm.clientcostcenters.containers.GenericClient;
import com.ibm.rpm.clientcostcenters.containers.ProjectCostCenter;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentManager;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentUtil;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.framework.AbstractModuleManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.StatusUpdate;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.xpathparser.XPathContainer;

public class ClientManager extends AbstractRPMObjectManager {
	
	private static final HashMap FIELDPROPERTYMAP = new HashMap();

	private static final HashSet CONTAINERS = new HashSet();

	static {
	    CONTAINERS.add(GenericClient.class.getName());
		CONTAINERS.add(ClientFolder.class.getName());
		CONTAINERS.add(Division.class.getName());
		CONTAINERS.add(ClientCostCenter.class.getName());

		String[] objs = new String[CONTAINERS.size()];
		CONTAINERS.toArray(objs);
		for (int i = 0; i < objs.length; i++) {
			RPMManagerFactory.getInstance().addContainerToMap(objs[i],
					ClientManager.class);
		}
	}

	private static final String FLAG_YES = "Y";
	private static final String FLAG_NO = "N";
	
	/**
	 * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_LEVEL_ID = 1;

	public static final int TYPE_LEVEL_ID = Types.INTEGER;

	public static final String NAME_LEVEL_ID = "TMT_CLIENTS.LEVEL_ID";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

	/**
	 * Column TYPE_ID of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_TYPE_ID = 2;

	public static final int TYPE_TYPE_ID = Types.INTEGER;

	public static final String NAME_TYPE_ID = "TMT_CLIENTS.TYPE_ID";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

	/**
	 * Column MAX_RANK of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_MAX_RANK = 3;

	public static final int TYPE_MAX_RANK = Types.INTEGER;

	public static final String NAME_MAX_RANK = "TMT_CLIENTS.MAX_RANK";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

	/**
	 * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_CHILD_COUNT = 4;

	public static final int TYPE_CHILD_COUNT = Types.INTEGER;

	public static final String NAME_CHILD_COUNT = "TMT_CLIENTS.CHILD_COUNT";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

	/**
	 * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_DELETED_COUNT = 5;

	public static final int TYPE_DELETED_COUNT = Types.INTEGER;

	public static final String NAME_DELETED_COUNT = "TMT_CLIENTS.DELETED_COUNT";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

	/**
	 * Column ELEMENT_ID of type Types.CHAR mapped to String.
	 */
	public static final int ID_ELEMENT_ID = 6;

	public static final int TYPE_ELEMENT_ID = Types.CHAR;

	public static final String NAME_ELEMENT_ID = "TMT_CLIENTS.ELEMENT_ID";

	public static final String PROPERTY_ELEMENT_ID = "ID";
	static {
		FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
	}

	/**
	 * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_ELEMENT_NAME = 7;

	public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

	public static final String NAME_ELEMENT_NAME = "TMT_CLIENTS.ELEMENT_NAME";

	public static final String PROPERTY_ELEMENT_NAME = "NAME";
	static{
	    FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME,NAME_ELEMENT_NAME);
	}

	/**
	 * Column PARENT_ID of type Types.CHAR mapped to String.
	 */
	public static final int ID_PARENT_ID = 8;

	public static final int TYPE_PARENT_ID = Types.CHAR;

	public static final String NAME_PARENT_ID = "TMT_CLIENTS.PARENT_ID";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

	/**
	 * Column REC_USER of type Types.CHAR mapped to String.
	 */
	public static final int ID_REC_USER = 9;

	public static final int TYPE_REC_USER = Types.CHAR;

	public static final String NAME_REC_USER = "TMT_CLIENTS.REC_USER";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_REC_USER = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

	/**
	 * Column REC_STATUS of type Types.CHAR mapped to String.
	 */
	public static final int ID_REC_STATUS = 10;

	public static final int TYPE_REC_STATUS = Types.CHAR;

	public static final String NAME_REC_STATUS = "TMT_CLIENTS.REC_STATUS";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

	/**
	 * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
	 */
	public static final int ID_REC_DATETIME = 11;

	public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

	public static final String NAME_REC_DATETIME = "TMT_CLIENTS.REC_DATETIME";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

	/**
	 * Column LABEL of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_LABEL = 12;

	public static final int TYPE_LABEL = Types.VARCHAR;

	public static final String NAME_LABEL = "TMT_CLIENTS.LABEL";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_LABEL = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_LABEL,NAME_LABEL);}

	/**
	 * Column DESCRIPTION of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_DESCRIPTION = 13;

	public static final int TYPE_DESCRIPTION = Types.VARCHAR;

	public static final String NAME_DESCRIPTION = "TMT_CLIENTS.DESCRIPTION";

	public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
	
	static{ FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION,NAME_DESCRIPTION);}

	/**
	 * Column DEFAULT_FLAG of type Types.CHAR mapped to String.
	 */
	public static final int ID_DEFAULT_FLAG = 14;

	public static final int TYPE_DEFAULT_FLAG = Types.CHAR;

	public static final String NAME_DEFAULT_FLAG = "TMT_CLIENTS.DEFAULT_FLAG";
	
	public static final String PROPERTY_DEFAULT_FLAG = "DEFAULTFLAG";
	
	static{ FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_FLAG,SqlUtil.getBooleanValue(NAME_DEFAULT_FLAG, FLAG_YES));}

	/**
	 * Column RANK of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_RANK = 15;

	public static final int TYPE_RANK = Types.VARCHAR;

	public static final String NAME_RANK = "TMT_CLIENTS.RANK";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_RANK = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

	/**
	 * Column DOMAIN_ID of type Types.CHAR mapped to String.
	 */
	public static final int ID_DOMAIN_ID = 16;

	public static final int TYPE_DOMAIN_ID = Types.CHAR;

	public static final String NAME_DOMAIN_ID = "TMT_CLIENTS.DOMAIN_ID";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_DOMAIN_ID = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_DOMAIN_ID,NAME_DOMAIN_ID);}

	/**
	 * Column MANAGER_ID of type Types.CHAR mapped to String.
	 */
	public static final int ID_MANAGER_ID = 17;

	public static final int TYPE_MANAGER_ID = Types.CHAR;

	public static final String NAME_MANAGER_ID = "TMT_CLIENTS.MANAGER_ID";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_MANAGER_ID = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_MANAGER_ID,NAME_MANAGER_ID);}

	/**
	 * Column COMPANY_NAME of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_COMPANY_NAME = 18;

	public static final int TYPE_COMPANY_NAME = Types.VARCHAR;

	public static final String NAME_COMPANY_NAME = "TMT_CLIENTS.COMPANY_NAME";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_COMPANY_NAME = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_COMPANY_NAME,NAME_COMPANY_NAME);}

	/**
	 * Column ADDRESS of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_ADDRESS = 19;

	public static final int TYPE_ADDRESS = Types.VARCHAR;

	public static final String NAME_ADDRESS = "TMT_CLIENTS.ADDRESS";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_ADDRESS = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_ADDRESS,NAME_ADDRESS);}

	/**
	 * Column PHONE of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_PHONE = 20;

	public static final int TYPE_PHONE = Types.VARCHAR;

	public static final String NAME_PHONE = "TMT_CLIENTS.PHONE";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_PHONE = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_PHONE,NAME_PHONE);}

	/**
	 * Column FAX of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_FAX = 21;

	public static final int TYPE_FAX = Types.VARCHAR;

	public static final String NAME_FAX = "TMT_CLIENTS.FAX";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_FAX = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_FAX,NAME_FAX);}

	/**
	 * Column WEBSITE of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_WEBSITE = 22;

	public static final int TYPE_WEBSITE = Types.VARCHAR;

	public static final String NAME_WEBSITE = "TMT_CLIENTS.WEBSITE";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_WEBSITE = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_WEBSITE,NAME_WEBSITE);}

	/**
	 * Column CONTACT_NAME of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_CONTACT_NAME = 23;

	public static final int TYPE_CONTACT_NAME = Types.VARCHAR;

	public static final String NAME_CONTACT_NAME = "TMT_CLIENTS.CONTACT_NAME";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_CONTACT_NAME = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_CONTACT_NAME,NAME_CONTACT_NAME);}

	/**
	 * Column OTHER_LOCATION of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_OTHER_LOCATION = 24;

	public static final int TYPE_OTHER_LOCATION = Types.VARCHAR;

	public static final String NAME_OTHER_LOCATION = "TMT_CLIENTS.OTHER_LOCATION";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_OTHER_LOCATION = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_OTHER_LOCATION,NAME_OTHER_LOCATION);}

	/**
	 * Column YEAR_IN_BUSINESS of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_YEAR_IN_BUSINESS = 25;

	public static final int TYPE_YEAR_IN_BUSINESS = Types.INTEGER;

	public static final String NAME_YEAR_IN_BUSINESS = "TMT_CLIENTS.YEAR_IN_BUSINESS";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_YEAR_IN_BUSINESS = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_YEAR_IN_BUSINESS,NAME_YEAR_IN_BUSINESS);}

	/**
	 * Column CLIENT_MORE_100_EMPLOYEES of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_CLIENT_MORE_100_EMPLOYEES = 26;

	public static final int TYPE_CLIENT_MORE_100_EMPLOYEES = Types.INTEGER;

	public static final String NAME_CLIENT_MORE_100_EMPLOYEES = "TMT_CLIENTS.CLIENT_MORE_100_EMPLOYEES";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_CLIENT_MORE_100_EMPLOYEES =
	// "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_CLIENT_MORE_100_EMPLOYEES,NAME_CLIENT_MORE_100_EMPLOYEES);}

	/**
	 * Column CLIENT_MORE_1000_EMPLOYEES of type Types.INTEGER mapped to
	 * Integer.
	 */
	public static final int ID_CLIENT_MORE_1000_EMPLOYEES = 27;

	public static final int TYPE_CLIENT_MORE_1000_EMPLOYEES = Types.INTEGER;

	public static final String NAME_CLIENT_MORE_1000_EMPLOYEES = "TMT_CLIENTS.CLIENT_MORE_1000_EMPLOYEES";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_CLIENT_MORE_1000_EMPLOYEES =
	// "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_CLIENT_MORE_1000_EMPLOYEES,NAME_CLIENT_MORE_1000_EMPLOYEES);}

	/**
	 * Column CLIENT_MORE_10000_EMPLOYEES of type Types.INTEGER mapped to
	 * Integer.
	 */
	public static final int ID_CLIENT_MORE_10000_EMPLOYEES = 28;

	public static final int TYPE_CLIENT_MORE_10000_EMPLOYEES = Types.INTEGER;

	public static final String NAME_CLIENT_MORE_10000_EMPLOYEES = "TMT_CLIENTS.CLIENT_MORE_10000_EMPLOYEES";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_CLIENT_MORE_10000_EMPLOYEES =
	// "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_CLIENT_MORE_10000_EMPLOYEES,NAME_CLIENT_MORE_10000_EMPLOYEES);}

	/**
	 * Column NUMBER_EMPLOYEES of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_NUMBER_EMPLOYEES = 29;

	public static final int TYPE_NUMBER_EMPLOYEES = Types.INTEGER;

	public static final String NAME_NUMBER_EMPLOYEES = "TMT_CLIENTS.NUMBER_EMPLOYEES";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_NUMBER_EMPLOYEES = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_NUMBER_EMPLOYEES,NAME_NUMBER_EMPLOYEES);}

	/**
	 * Column ANNUAL_REVENUES of type Types.DECIMAL mapped to
	 * java.math.BigDecimal.
	 */
	public static final int ID_ANNUAL_REVENUES = 30;

	public static final int TYPE_ANNUAL_REVENUES = Types.DECIMAL;

	public static final String NAME_ANNUAL_REVENUES = "TMT_CLIENTS.ANNUAL_REVENUES";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_ANNUAL_REVENUES = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_ANNUAL_REVENUES,NAME_ANNUAL_REVENUES);}

	/**
	 * Column BILLING_ADDRESS of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_BILLING_ADDRESS = 31;

	public static final int TYPE_BILLING_ADDRESS = Types.VARCHAR;

	public static final String NAME_BILLING_ADDRESS = "TMT_CLIENTS.BILLING_ADDRESS";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_BILLING_ADDRESS = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_BILLING_ADDRESS,NAME_BILLING_ADDRESS);}

	/**
	 * Column BILLING_PHONE of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_BILLING_PHONE = 32;

	public static final int TYPE_BILLING_PHONE = Types.VARCHAR;

	public static final String NAME_BILLING_PHONE = "TMT_CLIENTS.BILLING_PHONE";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_BILLING_PHONE = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_BILLING_PHONE,NAME_BILLING_PHONE);}

	/**
	 * Column BILLING_FAX of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_BILLING_FAX = 33;

	public static final int TYPE_BILLING_FAX = Types.VARCHAR;

	public static final String NAME_BILLING_FAX = "TMT_CLIENTS.BILLING_FAX";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_BILLING_FAX = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_BILLING_FAX,NAME_BILLING_FAX);}

	/**
	 * Column BILLING_EMAIL of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_BILLING_EMAIL = 34;

	public static final int TYPE_BILLING_EMAIL = Types.VARCHAR;

	public static final String NAME_BILLING_EMAIL = "TMT_CLIENTS.BILLING_EMAIL";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_BILLING_EMAIL = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_BILLING_EMAIL,NAME_BILLING_EMAIL);}

	/**
	 * Column EQUAL_OPPORTUNITY of type Types.CHAR mapped to String.
	 */
	public static final int ID_EQUAL_OPPORTUNITY = 35;

	public static final int TYPE_EQUAL_OPPORTUNITY = Types.CHAR;

	public static final String NAME_EQUAL_OPPORTUNITY = "TMT_CLIENTS.EQUAL_OPPORTUNITY";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_EQUAL_OPPORTUNITY = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_EQUAL_OPPORTUNITY,NAME_EQUAL_OPPORTUNITY);}

	/**
	 * Column MAJOR_REVISION of type Types.SMALLINT mapped to Integer.
	 */
	public static final int ID_MAJOR_REVISION = 36;

	public static final int TYPE_MAJOR_REVISION = Types.SMALLINT;

	public static final String NAME_MAJOR_REVISION = "TMT_CLIENTS.MAJOR_REVISION";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_MAJOR_REVISION = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_MAJOR_REVISION,NAME_MAJOR_REVISION);}

	/**
	 * Column MINOR_REVISION of type Types.SMALLINT mapped to Integer.
	 */
	public static final int ID_MINOR_REVISION = 37;

	public static final int TYPE_MINOR_REVISION = Types.SMALLINT;

	public static final String NAME_MINOR_REVISION = "TMT_CLIENTS.MINOR_REVISION";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_MINOR_REVISION = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_MINOR_REVISION,NAME_MINOR_REVISION);}

	/**
	 * Column REVISION_IMPACT of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_REVISION_IMPACT = 38;

	public static final int TYPE_REVISION_IMPACT = Types.INTEGER;

	public static final String NAME_REVISION_IMPACT = "TMT_CLIENTS.REVISION_IMPACT";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_REVISION_IMPACT = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_REVISION_IMPACT,NAME_REVISION_IMPACT);}

	/**
	 * Column REVISION_TYPE of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_REVISION_TYPE = 39;

	public static final int TYPE_REVISION_TYPE = Types.INTEGER;

	public static final String NAME_REVISION_TYPE = "TMT_CLIENTS.REVISION_TYPE";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_REVISION_TYPE = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_REVISION_TYPE,NAME_REVISION_TYPE);}

	/**
	 * Column REVISION_HISTORY of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_REVISION_HISTORY = 40;

	public static final int TYPE_REVISION_HISTORY = Types.VARCHAR;

	public static final String NAME_REVISION_HISTORY = "TMT_CLIENTS.REVISION_HISTORY";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_REVISION_HISTORY = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_REVISION_HISTORY,NAME_REVISION_HISTORY);}

	/**
	 * Column PUBLIC_FLAG of type Types.CHAR mapped to String.
	 */
	public static final int ID_PUBLIC_FLAG = 41;

	public static final int TYPE_PUBLIC_FLAG = Types.CHAR;

	public static final String NAME_PUBLIC_FLAG = "TMT_CLIENTS.PUBLIC_FLAG";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_PUBLIC_FLAG = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_PUBLIC_FLAG,NAME_PUBLIC_FLAG);}

	/**
	 * Column COMPANY_BACKGROUND of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_COMPANY_BACKGROUND = 42;

	public static final int TYPE_COMPANY_BACKGROUND = Types.VARCHAR;

	public static final String NAME_COMPANY_BACKGROUND = "TMT_CLIENTS.COMPANY_BACKGROUND";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_COMPANY_BACKGROUND = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_COMPANY_BACKGROUND,NAME_COMPANY_BACKGROUND);}

	/**
	 * Column OWNERSHIP of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_OWNERSHIP = 43;

	public static final int TYPE_OWNERSHIP = Types.VARCHAR;

	public static final String NAME_OWNERSHIP = "TMT_CLIENTS.OWNERSHIP";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_OWNERSHIP = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_OWNERSHIP,NAME_OWNERSHIP);}

	/**
	 * Column SERVICES_OFFERED of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_SERVICES_OFFERED = 44;

	public static final int TYPE_SERVICES_OFFERED = Types.VARCHAR;

	public static final String NAME_SERVICES_OFFERED = "TMT_CLIENTS.SERVICES_OFFERED";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_SERVICES_OFFERED = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_SERVICES_OFFERED,NAME_SERVICES_OFFERED);}

	/**
	 * Column SERVICES_SPECIALIZED of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_SERVICES_SPECIALIZED = 45;

	public static final int TYPE_SERVICES_SPECIALIZED = Types.VARCHAR;

	public static final String NAME_SERVICES_SPECIALIZED = "TMT_CLIENTS.SERVICES_SPECIALIZED";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_SERVICES_SPECIALIZED =
	// "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_SERVICES_SPECIALIZED,NAME_SERVICES_SPECIALIZED);}

	/**
	 * Column INDUSTRIES_SERVED of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_INDUSTRIES_SERVED = 46;

	public static final int TYPE_INDUSTRIES_SERVED = Types.VARCHAR;

	public static final String NAME_INDUSTRIES_SERVED = "TMT_CLIENTS.INDUSTRIES_SERVED";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_INDUSTRIES_SERVED = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_INDUSTRIES_SERVED,NAME_INDUSTRIES_SERVED);}

	/**
	 * Column HIGHLIGHT of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_HIGHLIGHT = 47;

	public static final int TYPE_HIGHLIGHT = Types.VARCHAR;

	public static final String NAME_HIGHLIGHT = "TMT_CLIENTS.HIGHLIGHT";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_HIGHLIGHT = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_HIGHLIGHT,NAME_HIGHLIGHT);}

	/**
	 * Column LOCATION_SERVED of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_LOCATION_SERVED = 48;

	public static final int TYPE_LOCATION_SERVED = Types.VARCHAR;

	public static final String NAME_LOCATION_SERVED = "TMT_CLIENTS.LOCATION_SERVED";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_LOCATION_SERVED = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_LOCATION_SERVED,NAME_LOCATION_SERVED);}

	/**
	 * Column LOCAL of type Types.CHAR mapped to String.
	 */
	public static final int ID_LOCAL = 49;

	public static final int TYPE_LOCAL = Types.CHAR;

	public static final String NAME_LOCAL = "TMT_CLIENTS.LOCAL";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_LOCAL = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_LOCAL,NAME_LOCAL);}

	/**
	 * Column RTF_BITS of type Types.SMALLINT mapped to Integer.
	 */
	public static final int ID_RTF_BITS = 50;

	public static final int TYPE_RTF_BITS = Types.SMALLINT;

	public static final String NAME_RTF_BITS = "TMT_CLIENTS.RTF_BITS";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_RTF_BITS = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_RTF_BITS,NAME_RTF_BITS);}

	/**
	 * Column DOC_COUNT of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_DOC_COUNT = 51;

	public static final int TYPE_DOC_COUNT = Types.INTEGER;

	public static final String NAME_DOC_COUNT = "TMT_CLIENTS.DOC_COUNT";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_DOC_COUNT = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_DOC_COUNT,NAME_DOC_COUNT);}

	/**
	 * Column CURRENCY_ID of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_CURRENCY_ID = 52;

	public static final int TYPE_CURRENCY_ID = Types.INTEGER;

	public static final String NAME_CURRENCY_ID = "TMT_CLIENTS.CURRENCY_ID";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_CURRENCY_ID = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_CURRENCY_ID,NAME_CURRENCY_ID);}

	/**
	 * Column STAGE_ID of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_STAGE_ID = 53;

	public static final int TYPE_STAGE_ID = Types.INTEGER;

	public static final String NAME_STAGE_ID = "TMT_CLIENTS.STAGE_ID";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_STAGE_ID = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_STAGE_ID,NAME_STAGE_ID);}

	/**
	 * Column EXTERNAL_IDENTIFIER of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_EXTERNAL_IDENTIFIER = 54;

	public static final int TYPE_EXTERNAL_IDENTIFIER = Types.VARCHAR;

	public static final String NAME_EXTERNAL_IDENTIFIER = "TMT_CLIENTS.EXTERNAL_IDENTIFIER";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_EXTERNAL_IDENTIFIER = "PROPERTYNAME";
	//static{
	// FIELDPROPERTYMAP.put(PROPERTY_EXTERNAL_IDENTIFIER,NAME_EXTERNAL_IDENTIFIER);}

	private static final String TABLE_NAME = "TMT_CLIENTS";

	/**
	 * Create an array of type string containing all the fields of the
	 * TMT_CLIENTS table.
	 */
	private static final String[] FIELD_NAMES = { NAME_LEVEL_ID, NAME_TYPE_ID,
			NAME_MAX_RANK, NAME_CHILD_COUNT, NAME_DELETED_COUNT,
			NAME_ELEMENT_ID, NAME_ELEMENT_NAME, NAME_PARENT_ID, NAME_REC_USER,
			NAME_REC_STATUS, NAME_REC_DATETIME, NAME_LABEL, NAME_DESCRIPTION,
			NAME_DEFAULT_FLAG, NAME_RANK, NAME_DOMAIN_ID, NAME_MANAGER_ID,
			NAME_COMPANY_NAME, NAME_ADDRESS, NAME_PHONE, NAME_FAX,
			NAME_WEBSITE, NAME_CONTACT_NAME, NAME_OTHER_LOCATION,
			NAME_YEAR_IN_BUSINESS, NAME_CLIENT_MORE_100_EMPLOYEES,
			NAME_CLIENT_MORE_1000_EMPLOYEES, NAME_CLIENT_MORE_10000_EMPLOYEES,
			NAME_NUMBER_EMPLOYEES, NAME_ANNUAL_REVENUES, NAME_BILLING_ADDRESS,
			NAME_BILLING_PHONE, NAME_BILLING_FAX, NAME_BILLING_EMAIL,
			NAME_EQUAL_OPPORTUNITY, NAME_MAJOR_REVISION, NAME_MINOR_REVISION,
			NAME_REVISION_IMPACT, NAME_REVISION_TYPE, NAME_REVISION_HISTORY,
			NAME_PUBLIC_FLAG, NAME_COMPANY_BACKGROUND, NAME_OWNERSHIP,
			NAME_SERVICES_OFFERED, NAME_SERVICES_SPECIALIZED,
			NAME_INDUSTRIES_SERVED, NAME_HIGHLIGHT, NAME_LOCATION_SERVED,
			NAME_LOCAL, NAME_RTF_BITS, NAME_DOC_COUNT, NAME_CURRENCY_ID,
			NAME_STAGE_ID, NAME_EXTERNAL_IDENTIFIER };

	private static final String ALL_FIELDS;
	static {
		StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
		for (int i = 1; i < FIELD_NAMES.length; i++) {
			fieldcat.append(",");
			fieldcat.append(FIELD_NAMES[i]);
		}
		ALL_FIELDS = fieldcat.toString();
	};
	

	/**
	 * Creates a new ClientsManager instance.
	 * 
	 * @return ClientsManager
	 */
	public ClientManager() {
		super();
	}

	public String getTableName() {
		return TABLE_NAME;
	}

	protected String[] getFieldsNames() {
		return FIELD_NAMES;
	}

	protected String getAllFields() {
		return ALL_FIELDS;
	}

	public String getPrimaryKey() {
		return NAME_ELEMENT_ID;
	}
	
    public String getFilter()
    {
        SqlBuffer sql = new SqlBuffer();
        sql.append(NAME_REC_STATUS);
        sql.append(" != 'D'");
        sql.appendAnd();
        sql.append(NAME_ELEMENT_ID);
        sql.append(" IS NOT NULL ");
        
        return sql.toString();
    }
	protected Integer getContainerType(ResultSet resultSet)
			throws RPMException, SQLException {
		int typeId = resultSet.getInt(ID_TYPE_ID);
		return new Integer(typeId);
	}

	protected HashMap getPropertiesMap() {
		return FIELDPROPERTYMAP;
	}

	protected RPMObject createContainer(int typeId) throws RPMException {

		RPMObject object = null;

		switch (typeId) {

		case ClientFolder.TYPE_ID:
			object = new ClientFolder();
			break;
		case Division.TYPE_ID:
			object = new Division();
			break;
		case ClientCostCenter.TYPE_ID:
			object = new ClientCostCenter();
			break;
		case Proposal.TYPE_ID:
			object = new ProjectCostCenter();
			break;

		default:
		    ExceptionUtil.handleUnsupportedTypeId(this, typeId);
		}
		return object;
	}

	protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
			boolean isDelta) throws RPMException, SQLException, ParseException {
		
	    GenericClient temp = (GenericClient)rpmObject;
	    
		temp.setID(rs.getString(ID_ELEMENT_ID));

		String name = rs.getString(ID_ELEMENT_NAME);
		if (name != null) {
			name = name.trim();
		}

		if (isDelta) {
			temp.deltaName(name);
			temp.deltaDescription(rs.getString(ID_DESCRIPTION));
			temp.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
		} else {
			temp.setName(name);
			temp.setDescription(rs.getString(ID_DESCRIPTION));
			temp.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
		}
		
		if(isClientCostCenter(rpmObject)){
			ClientCostCenter clientCostCenter = (ClientCostCenter)rpmObject;
			
			String sDefaultFlag = rs.getString(ID_DEFAULT_FLAG);
			Boolean defaultFlag = (FLAG_YES.equals(sDefaultFlag))?new Boolean(true):new Boolean(false);
			
			String externalId = rs.getString(ID_EXTERNAL_IDENTIFIER);
			
			if(isDelta){
				clientCostCenter.deltaDefaultFlag(defaultFlag);
				clientCostCenter.deltaExternalID(externalId);
			} else {
				clientCostCenter.setDefaultFlag(defaultFlag);
				clientCostCenter.setExternalID(externalId);
			}
		}
		
		return rpmObject;
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
	 *      com.ibm.rpm.FieldValueMap, int)
	 */
	protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
			FieldValueMap relatedFields, int index) throws RPMException,
			SQLException {

		String parentId = resultSet.getString(ID_PARENT_ID);
		relatedFields.put(index, ID_PARENT_ID, parentId);

		String stageId = resultSet.getString(ID_STAGE_ID);
		relatedFields.put(index, ID_STAGE_ID, stageId);
		
		return relatedFields;
	}

	/**
	 *  
	 */

	protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
			throws RPMException, SQLException {

		if (rpmObject != null) {
			// rpmObject can be a ClientFolder, Division or Client/Cost Center
			SP_D_CLIENT(rpmObject, messageContext);
		}            	
	}

	/**
	 *  
	 */
	protected JoinCondition getJoinCondition(RPMObjectManager manager,
			String condition, int level, String containerName, String contextName) throws RPMException {
		JoinCondition joinCondition = null;

		int typeId = 0;
        if (containerName != null)
        {
            if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(GenericClient.class)))
            {
                typeId = 0;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(ClientFolder.class)))
            {
                typeId = ClientFolder.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(Division.class)))
            {
                typeId = Division.TYPE_ID;
            }
            else if (containerName.equalsIgnoreCase(StringUtil
                .getShortClassName(ClientCostCenter.class)))
            {
                typeId = ClientCostCenter.TYPE_ID;
            }
            else
            {
                ExceptionUtil.handleNoTypeId(this, containerName);
            }
            
        }

        SqlBuffer baseCondition = new SqlBuffer();
        baseCondition.appendIsNotNull(NAME_ELEMENT_ID);
        if (typeId != 0)
        {
            baseCondition.appendEqual(NAME_TYPE_ID, typeId);
        }
        else
        {
            //Assume that we are loading a ScopeElement
            int[] types = new int[]
            {
                    ClientFolder.TYPE_ID, Division.TYPE_ID,
                    ClientCostCenter.TYPE_ID
            };
            baseCondition.appendIn(NAME_TYPE_ID, types);
        }
        baseCondition.appendCondition(condition);
        baseCondition.appendCondition(getFilter());
		
		
		if (manager == null)
        {
            joinCondition = new JoinCondition(getTableName(), baseCondition);

        } else if (manager instanceof ClientManager) {
        	
			joinCondition = new JoinCondition();

			StringBuffer where = new StringBuffer();
			int rankLenght = 4 * level;

			String rank = TABLE_NAME + ".RANK";

			where.append("SUBSTR(");
			where.append(rank);
			where.append(",1,LENGTH(");
			where.append(rank);
			where.append(")- ");
			where.append(rankLenght);
			where.append(") IN (SELECT ");
			where.append(rank);
			where.append(" FROM ");
			where.append(TABLE_NAME);
			if (condition != null) {
				where.append(" WHERE ");
				where.append(condition);
			}

			joinCondition.setCondition(where.toString());
			
        } else if (manager instanceof AttributeAssignmentManager){
        	if(ContextUtil.CLIENT_CONTEXT.equalsIgnoreCase(contextName)){
                joinCondition = new JoinCondition();

                if(condition != null){
                	AttributeAssignmentManager attributeAssignmentManager = (AttributeAssignmentManager)manager;
	                StringBuffer where = new StringBuffer();
	                where.append(AttributeManager.NAME_ELEMENT_ID);
	                where.append(" IN (SELECT ");
	                where.append(AttributeAssignmentManager.NAME_PARENT_ID);
	                where.append(" FROM ");
	                where.append(attributeAssignmentManager.getTableName(contextName));
                    where.append(" WHERE ");
                    where.append(AttributeAssignmentManager.NAME_ELEMENT_ID);
	                where.append(" IN (SELECT ");
	                where.append(NAME_ELEMENT_ID);
	                where.append(" FROM ");
	                where.append(TABLE_NAME);
	                where.append(" WHERE ");
	                where.append(condition);

	                joinCondition.setCondition(where.toString());
                }
        	}
        } else if (manager instanceof AssetCrossChargeManager) {
                    // for XPath queries like
                    // /ClientCostCenter[<condition>]/CrossChargeFinancials
                    joinCondition = new JoinCondition();
                    StringBuffer where = new StringBuffer();
                    where.append(AssetCrossChargeManager.NAME_PROJECT_CLIENT_ID).append(" IN (");
                    where.append(" SELECT ").append(NAME_ELEMENT_ID);
                    where.append(" FROM ").append(getTableName());
                    if (condition != null)
                    {	
                    	where.append(" WHERE ");
                        where.append(condition);
                    }
                    joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof CustomFieldAssignmentManager)
        {
            if (ContextUtil.CLIENT_CONTEXT.equalsIgnoreCase(contextName))
            {
                // for XPath queries like
                // ClientCostCenter[<condition>]/AssignedCustomField
                joinCondition = new JoinCondition();
                StringBuffer where = new StringBuffer();
                where.append(CustomFieldAssignmentUtil
                        .getTableName(ContextUtil.CLIENT_CONTEXT));
                where.append(".");
                where.append(CustomFieldAssignmentManager.NAME_FOREIGN_ID);
                where.append(" IN (");
                where.append(" SELECT ").append(NAME_ELEMENT_ID);
                where.append(" FROM ").append(getTableName());
                if (condition != null)
                {
                    where.append(" WHERE ");
                    where.append(condition);
                }
                joinCondition.setCondition(where.toString());
            }
        }        
        else {//context not linked to this manager
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
		return joinCondition;
	}
    
	
	public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

    	if (manager instanceof AssetCrossChargeManager)
        {
            // for XPath queries like
            // /CrossChargeFinancials[ClientCostCenter[<condition>]}
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(AssetCrossChargeManager.NAME_PROJECT_CLIENT_ID).append(" IN (");
            where.append(" SELECT ").append(NAME_ELEMENT_ID);
            where.append(" FROM ").append(getTableName());
            joinCondition.setCondition(where.toString());
        }
    	else if (manager instanceof ClientManager)
    	{
    	    joinCondition = new JoinCondition();
    	    StringBuffer where = new StringBuffer();
    	    
    	    where.append(ClientManager.NAME_ELEMENT_ID);
    	    where.append(" IN ( ");
    	    where.append(" SELECT ");
    	    where.append(ClientManager.NAME_PARENT_ID);
    	    where.append(" FROM ");
    	    where.append(TABLE_NAME);
    	    joinCondition.setCondition(where.toString());
    	}
        else if (manager instanceof ScopeManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                ScopeManager.NAME_CLIENT_ID, NAME_ELEMENT_ID, TABLE_NAME,
                null);
        }
        else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
	    }

        return joinCondition;
    }
        
	protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
			String extraWhere, String contextName) throws RPMException {
		JoinCondition joinCondition = null;

		if (manager instanceof ClientManager) 
		{
		    joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID,
                TABLE_NAME, NAME_ELEMENT_ID + "=?)");
            
            // want the extraWhere outside of the subSelect
            joinCondition.appendCondition(extraWhere); 
        }
        else if (manager instanceof AbstractModuleManager)
        {            
            joinCondition = new JoinCondition();
            joinCondition.setCondition(extraWhere);            
            joinCondition.setTableName(getTableName()); 
		}
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}
		
		return joinCondition;
	}

	protected RPMObject loadRelatedObjects(RPMObject rpmObject,
			RPMObjectScope rpmScope, MessageContext messageContext,
			FieldValueMap relatedFields, int index, boolean isDelta)
			throws RPMException, SQLException, ParseException {

		if (rpmScope != null) {

			//Parent
			if (rpmScope.getParent() != null) {
				String oldParentId = (String) relatedFields.get(index,
						ID_PARENT_ID);
				RPMObject newParent = null;
				if (isDelta) {
					newParent = rpmObject.getParent();
					RPMObject oldParent = null;
					if (oldParentId != null) {
						//TODO create a class parent to AssetFolder,Division, CCC so
						// that it can be instantiated here
						//instead of always instantiating a folder
						oldParent = new ClientFolder();
						oldParent.setID(oldParentId);
						oldParent.setContextName(getContextName(oldParent));
						if (newParent != null && newParent.getID() != null
								&& newParent.getID().equals(oldParentId)) {
							oldParent = loadByPrimaryKey(newParent, rpmScope
									.getParent(), messageContext, true);
						} else {
							//In this case, old parent is loaded without the
							// scope to avoid problem with the cache
							oldParent = loadByPrimaryKey(oldParent, null,
									messageContext, false);
						}
					}
					rpmObject.deltaParent(oldParent);
				} else {
					if (oldParentId != null) {
						if (newParent == null) {//newParent not in cache
							//TODO create a class parent to AssetFolder,Division,
							// CCC so that it can be instantiated here
							//instead of always instantiating a folder
							newParent = new ClientFolder();
							newParent.setID(oldParentId);
							newParent.setContextName(getContextName(newParent));
							newParent = loadByPrimaryKey(newParent, rpmScope
									.getParent(), messageContext, false);
						}
						rpmObject.setParent(newParent);
					}
				}
			}

			
			if (rpmScope instanceof ClientCostCenterScope){
			    
				ClientCostCenterScope scope = (ClientCostCenterScope) rpmScope;
				//load children
				if (isFolder(rpmObject)) {
					ClientFolder folder = (ClientFolder) rpmObject;
					if (scope.getFolders() != null) {
						loadFolders(folder, scope.getFolders(),
								messageContext, isDelta);
					}
					if (scope.getDivisions() != null) {
						loadDivisions(folder, scope.getDivisions(),
								messageContext, isDelta);
					}
					if (scope.getClientCostCenters() != null) {
						loadClientCostCenters(folder, folder.getClientCostCenters(),
								scope.getClientCostCenters(), messageContext, isDelta);
					}
				} else if(isDivision(rpmObject)){
					Division division = (Division) rpmObject;
					if (scope.getClientCostCenters() != null) {
						loadClientCostCenters(division, division.getClientCostCenters(),
								scope.getClientCostCenters(), messageContext, isDelta);
					}
				} else if(isClientCostCenter(rpmObject)){
					ClientCostCenter clientCostCenter = (ClientCostCenter)rpmObject;
					
					if(scope.isState()){
					    String costCenterStageId = (String) relatedFields.get(index, ID_STAGE_ID);
					    loadState(clientCostCenter, costCenterStageId, messageContext, isDelta);
					}
					
					if(scope.isRtfAssignments()){
						loadClientRtfAssignments(clientCostCenter, null, messageContext, isDelta);									    
					}
									
		            //Attribute Assignments
		            if (scope.isAttributeAssignments()) {
		            	loadClientAttributeAssignments(clientCostCenter, null, messageContext, isDelta);
		            }
					
	                loadDocumentFolder(clientCostCenter, scope, messageContext, isDelta);
		            
		            if (scope.isCustomFieldAssignments())
		            {
                    clientCostCenter.setCustomFieldAssignments(CustomFieldAssignmentUtil
                        .loadAssignedCustomFields(this, clientCostCenter
                            .getCustomFieldAssignments(), clientCostCenter,
                            messageContext, isDelta));
	                propagateContextName(clientCostCenter, clientCostCenter
                            .getCustomFieldAssignments());
		            }
		            
		          	//TODO uncomment when StatusUpdates will be required
		            //status update should be loaded only if it is in the scope
					//loadClientCostCenterHistory(clientCostCenter, messageContext, isDelta);
				}
			}
		}
		return rpmObject;
	}

	private String getContextName(RPMObject rpmObject)throws RPMException{
		return ContextUtil.CLIENT_CONTEXT;
	}
	
    private void loadState(ClientCostCenter ccc, String stateId,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        CostCenterState costCenterStage = new CostCenterState();
        costCenterStage.setID(stateId);

        DatafieldManager manager = (DatafieldManager) getManagerInstance(costCenterStage
                .getClass().getName());
        costCenterStage = (CostCenterState) manager.loadByPrimaryKey(
                costCenterStage, null, messageContext, isDelta);
        if (isDelta)
        {
            ccc.deltaState(costCenterStage);
        }
        else
        {
            ccc.setState(costCenterStage);
        }
    }
    
    private void loadDocumentFolder(ClientCostCenter clientCostCenter,
            ClientCostCenterScope scope, MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        if(scope.getDocumentFolder()!=null){
            DocumentManager manager = (DocumentManager) getManagerInstance(DocumentFolder.class
                    .getName());

            RPMObject[] oldDocumentFolder = new RPMObject[]
            {
                    clientCostCenter.getDocumentFolder()
            };
            Object[] args = new Object[]
            {
                    clientCostCenter.getID()
            };
            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(clientCostCenter, oldDocumentFolder, scope.getDocumentFolder(),
                    messageContext, this, args, null, ContextUtil.CLIENT_DOCUMENT_CONTEXT, isDelta);

            if (temp != null && !temp.isEmpty())
            {
                DocumentFolder documentFolder = (DocumentFolder) temp.get(0);
                documentFolder.setContextName(ContextUtil.CLIENT_DOCUMENT_CONTEXT);
                clientCostCenter.setDocumentFolder(documentFolder);
            }   
        }
    }
    
    private ClientCostCenter saveDocumentFolder(ClientCostCenter clientCostCenter,
            ClientCostCenterScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (scope.getDocumentFolder()!=null && clientCostCenter.getDocumentFolder() != null)
        {
            DocumentManager manager = (DocumentManager) getManagerInstance(DocumentFolder.class
                    .getName());

            DocumentFolder documentFolder = clientCostCenter.getDocumentFolder();
            documentFolder.setParent(clientCostCenter);
            manager.internalSave(documentFolder, scope.getDocumentFolder(), messageContext);
            documentFolder.setContextName(ContextUtil.CLIENT_DOCUMENT_CONTEXT);
            clientCostCenter.setDocumentFolder(documentFolder);
        }
        return clientCostCenter;
    }
    
	private void loadClientAttributeAssignments(ClientCostCenter clientCostCenter, RPMObjectScope scope,
			MessageContext messageContext, boolean isDelta)
			throws RPMException, SQLException, ParseException {
    	AttributeAssignmentManager attributeAssignmentManager = (AttributeAssignmentManager) getManagerInstance(AttributeAssignment.class.getName());
    	
    	Object[] args = new Object[]{clientCostCenter.getID()};
        ArrayList temp = attributeAssignmentManager
                .loadByForeignKey(clientCostCenter.getAttributeAssignments(), null,
                        messageContext, this, args, null, getContextName(clientCostCenter), isDelta);
        if (temp != null && !temp.isEmpty()) {
            setParent(temp, clientCostCenter);
            AttributeAssignment[] aa = new AttributeAssignment[temp.size()];
            temp.toArray(aa);
            clientCostCenter.setAttributeAssignments(aa);
        }
	}
	
	private void loadClientRtfAssignments(ClientCostCenter clientCostCenter, RPMObjectScope scope,
			MessageContext messageContext, boolean isDelta)
			throws RPMException, SQLException, ParseException {
        
	    RtfAssignmentManager rtfAssignmentManager = (RtfAssignmentManager)getManagerInstance(RtfAssignment.class.getName());
		
		Object[] args = new Object[]{clientCostCenter.getID()};
	    ArrayList temp = rtfAssignmentManager.loadByForeignKeyWithSpecifiedParent(clientCostCenter, clientCostCenter.getRtfAssignments(), scope, messageContext, this, args, null, getContextName(clientCostCenter), isDelta);
		
	    RtfAssignment[] rtfAssignments = null;
		if(temp !=null && !temp.isEmpty()){
		    setParent(temp, clientCostCenter);
			rtfAssignments = new RtfAssignment[temp.size()];
			temp.toArray(rtfAssignments);			
			clientCostCenter.setRtfAssignments(rtfAssignments);
		}
	}
	
	private void loadClientCostCenterHistory(ClientCostCenter clientCostCenter, 
			MessageContext messageContext, boolean isDelta)
			throws RPMException, SQLException, ParseException{
//	  TODO load the revision from the document history manager
//		ArrayList statusUpdates = SP_S_CLIENT_REVISION(clientCostCenter, messageContext);
//		
//		StatusUpdate[] statusUpdatesArray = new StatusUpdate[statusUpdates.size()];
//		statusUpdates.toArray(statusUpdatesArray);
//		clientCostCenter.setStatusUpdates(statusUpdatesArray);
	}
	
	private void loadFolders(ClientFolder folder, ClientCostCenterScope scope,
			MessageContext messageContext, boolean isDelta)
			throws RPMException, SQLException, ParseException {
		ClientFolder[] oldChildren = folder.getFolders();
		ArrayList temp = loadByForeignKey(oldChildren, scope, messageContext,
				this, new Object[] { folder.getID() }, "TYPE_ID = "
						+ ClientFolder.TYPE_ID, getContextName(folder), isDelta);
		if (temp != null) {
			ClientFolder[] children = new ClientFolder[temp.size()];
			temp.toArray(children);
			//TODO check why no setParent is called
			folder.setFolders(children);
		}
	}

	private void loadDivisions(ClientFolder folder,
            ClientCostCenterScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        Division[] oldChildren = folder.getDivisions();
        ArrayList temp = loadByForeignKey(oldChildren, scope, messageContext,
            this, new Object[]
            {
                folder.getID()
            }, "TYPE_ID = " + Division.TYPE_ID, getContextName(folder),
            isDelta);
        if (temp != null)
        {
            Division[] children = new Division[temp.size()];
            temp.toArray(children);
            //TODO check why no setParent is called
            folder.setDivisions(children);
        }
    }

	private void loadClientCostCenters(RPMObject rpmObject,
            ClientCostCenter[] oldChildren, ClientCostCenterScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        ArrayList temp = loadByForeignKey(oldChildren, scope, messageContext,
            this, new Object[]
            {
                rpmObject.getID()
            }, "TYPE_ID = " + ClientCostCenter.TYPE_ID, getContextName(rpmObject), isDelta);
        
        if (temp != null)
        {
            ClientCostCenter[] children = new ClientCostCenter[temp.size()];
            temp.toArray(children);
            //TODO check why no setParent is called
            if (isFolder(rpmObject))
            {
                ((ClientFolder) rpmObject).setClientCostCenters(children);
            }
            else if (isDivision(rpmObject))
            {
                ((Division) rpmObject).setClientCostCenters(children);
            }
        }
    }

	/**
	 * @param pObject
	 * @throws SQLException
	 * @throws RPMException
	 */
	public void createObject(RPMObject rpmObject, MessageContext messageContext)
			throws SQLException, RPMException {
	
		// Set default values
		if (rpmObject instanceof ClientCostCenter) {
			if (((ClientCostCenter) rpmObject).getDefaultFlag() == null)
				((ClientCostCenter) rpmObject)
						.setDefaultFlag(new Boolean(false));
		}
		rpmObject.setID(SP_I_CLIENT((GenericClient) rpmObject, messageContext));            	
	}

	public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
			MessageContext messageContext) throws SQLException, RPMException,
			ParseException {
		
		ClientCostCenterScope scope = (ClientCostCenterScope) rpmScope;
		GenericClient temp = (GenericClient)rpmObject;

		if (temp.testNameModified()) {
			SP_U_CLIENT_NAME(temp, messageContext);
		}

		if (temp.testDescriptionModified()) {
			SP_U_CLIENT_DESC(temp, messageContext);
		}

		if (!isClientCostCenter(rpmObject)) {
			if (scope != null) {
				rpmObject = updateChildren(rpmObject, scope, messageContext);
			}
		} else {

			ClientCostCenter ccc = (ClientCostCenter) rpmObject;

			// Default flag if set to 'Y'
			if (ccc.testDefaultFlagModified()) {
				if (isDefaultClientCostCenter(ccc)) {
					SP_U_CLIENT_DEFAULT(ccc, messageContext);
				}
			}

			//CostCenterStage and externalId
			if ((scope != null && scope.isState() && ccc.testStateModified())
					|| ccc.testExternalIDModified()) {
				SP_U_CLIENT_STAGE_ID(ccc, messageContext);
			}

			//Rtf Values
			if (scope != null && scope.isRtfAssignments()) {
				saveRtfAssignments(ccc, rpmScope, messageContext);
			}

			// Client Attribute assignments
			if (scope != null && scope.isAttributeAssignments()) {
				saveAttributeAssignments(ccc, rpmScope, messageContext);
			}

			// Client Document Elements
			if (scope != null && scope.getDocumentFolder() != null) {
				saveDocumentFolder(ccc, scope, messageContext);
			}

			propagateContextName(ccc, ccc.getCustomFieldAssignments());
			// Custom Field assignments
			ccc
					.setCustomFieldAssignments((CustomFieldAssignment[]) updateChildren(
							ccc, ccc.getCustomFieldAssignments(), scope != null
									&& scope.isCustomFieldAssignments(),
							messageContext, null));

			//TODO uncomment when StatusUpdates will be required
			//SP_U_CLIENT_REVISION(rpmObject, messageContext);
		}

		return rpmObject;
	}

	private boolean isDefaultClientCostCenter(ClientCostCenter ccc){
		return ccc.getDefaultFlag()!=null && ccc.getDefaultFlag().booleanValue();
	}
	
	protected RPMObject updateChildren(RPMObject rpmObject,
			RPMObjectScope rpmScope, MessageContext messageContext)
			throws RPMException, SQLException,
			ParseException {

		ClientCostCenterScope scope = (ClientCostCenterScope) rpmScope;

		if (isFolder(rpmObject)) {

			ClientFolder folder = (ClientFolder) rpmObject;

			// save all children-Folders
			if(canSaveChildren(scope.getFolders(), folder.getFolders())){
				
				ClientFolder[] children = (ClientFolder[])saveChildren(folder, folder.getFolders(),
						scope.getFolders(), messageContext);
				
				folder.setFolders(children);
			}

			// save all children-Divisions
			if (canSaveChildren(scope.getDivisions(), folder.getDivisions())) {
				
				Division[] children = (Division[])saveChildren(folder, folder.getDivisions(),
						scope.getDivisions(), messageContext);
				
				folder.setDivisions(children);
			}

			// save all children-client/cost centers
			if (canSaveChildren(scope.getClientCostCenters(), folder.getClientCostCenters())) {
				
				ClientCostCenter[] children = (ClientCostCenter[])saveChildren(folder, folder.getClientCostCenters(),
						scope.getClientCostCenters(), messageContext);
				
				folder.setClientCostCenters(children);
			}

		} else if (isDivision(rpmObject)) {

			// save all children-client/cost centers
			Division division = (Division) rpmObject;
			if (canSaveChildren(scope.getClientCostCenters(), division.getClientCostCenters())) {
				
				ClientCostCenter[] children = (ClientCostCenter[])saveChildren(division, division.getClientCostCenters(),
						scope.getClientCostCenters(), messageContext);
				
				division.setClientCostCenters(children);
			}
		}
		return rpmObject;		
	}
	
	private RPMObject[] saveChildren(RPMObject parent, RPMObject[] children,
			ClientCostCenterScope childrenScope, MessageContext messageContext)
			throws RPMException, SQLException,
			ParseException
	{
	    children = updateChildren(parent, children, 
                childrenScope, messageContext, null);
		return children;
	}
	
	private boolean canSaveChildren(ClientCostCenterScope childrenScope, RPMObject[] children){
		return (childrenScope!=null && children!=null);
	}
	
	private boolean isFolder(RPMObject rpmObject) {
		return rpmObject.retrieveTypeID().intValue() == ClientFolder.TYPE_ID;
	}

	private boolean isDivision(RPMObject rpmObject) {
		return rpmObject.retrieveTypeID().intValue() == Division.TYPE_ID;
	}

	private boolean isClientCostCenter(RPMObject rpmObject) {
		return rpmObject.retrieveTypeID().intValue() == ClientCostCenter.TYPE_ID;
	}
	
    private ClientCostCenter saveRtfAssignments(ClientCostCenter cccObject,
			RPMObjectScope rpmScope, MessageContext messageContext)
			throws RPMException, SQLException,
			ParseException 
    {
		ClientCostCenterScope scope = (ClientCostCenterScope) rpmScope;
		cccObject.setRtfAssignments((RtfAssignment[]) 
                updateChildren(cccObject, 
                        cccObject.getRtfAssignments(), 
                        scope.isRtfAssignments(), 
                        messageContext, null));
		return cccObject;
	}
	
    private ClientCostCenter saveAttributeAssignments(ClientCostCenter cccObject,
			RPMObjectScope rpmScope, MessageContext messageContext)
			throws RPMException, SQLException,
			ParseException 
	{
		ClientCostCenterScope scope = (ClientCostCenterScope) rpmScope;
		cccObject.setAttributeAssignments((AttributeAssignment[]) 
                updateChildren(cccObject, 
                        cccObject.getAttributeAssignments(), 
                        scope.isAttributeAssignments(), 
                        messageContext, null));
		return cccObject;
	}

	private void SP_D_CLIENT(RPMObject rpmObject, MessageContext messageContext)
			throws RPMException, SQLException {
		String name = "SP_D_CLIENT";
		Object args[] = new Object[4];
		args[0] = rpmObject.getID();
        args[1] = ManagerUtil.getParentId( this, rpmObject, messageContext, getTableName());
		args[2] = "X";
		if (getUser(messageContext) != null) {
			args[3] = getUser(messageContext).getID();
		}
		//The needed parameters for the SP are:
		//ELEMENT_ID, PARENT_ID, REC_STATUS, REC_USER
		executeProcedure(messageContext, name, args);
	}

	private String SP_I_CLIENT(GenericClient rpmObject,
			MessageContext messageContext) throws SQLException, RPMException {
			    
	    String name = "SP_I_CLIENT";
		Object args[] = new Object[4];
		args[0] = rpmObject.getName();
		if (rpmObject.getParent() != null) {
			args[1] = rpmObject.getParent().getID();
		}
		args[2] = rpmObject.retrieveTypeID();
		args[3] = getUser(messageContext).getID();
		//The needed parameters for the SP are:
		//ELEMENT_NAME, PARENT_ID, TYPE_ID, REC_USER
		return executeProcedure(messageContext, name, args);
	}

	private String SP_U_CLIENT_NAME(GenericClient rpmObject,
			MessageContext messageContext) throws SQLException, RPMException {

		String elementId = null;

		String name = "SP_U_CLIENT_NAME";
		Object args[] = new Object[3];
		args[0] = rpmObject.getID();
		args[1] = rpmObject.getName();
		args[2] = getUser(messageContext).getID();
		elementId = executeProcedure(messageContext, name, args);

		return elementId;
	}

    private String SP_U_CLIENT_DESC(GenericClient rpmObject,
    		MessageContext messageContext) throws SQLException, RPMException {
    
    	String elementId = null;
    
    	String name = "SP_U_CLIENT_DESC";
    	Object args[] = new Object[4];
    	args[0] = rpmObject.getID();
    	args[1] = null; //not used
    	args[2] = rpmObject.getDescription();
    	args[3] = getUser(messageContext).getID();
    	elementId = executeProcedure(messageContext, name, args);
    
    	return elementId;
    }

	private String SP_U_CLIENT_DEFAULT(ClientCostCenter cccObject,
			MessageContext messageContext) throws SQLException, RPMException {

		String elementId = null;

		String name = "SP_U_CLIENT_DEFAULT";
		Object args[] = new Object[3];
		args[0] = cccObject.getID();
		args[1] = (cccObject.getDefaultFlag()!=null&&cccObject.getDefaultFlag().booleanValue())?FLAG_YES:FLAG_NO;
		args[2] = getUser(messageContext).getID();
		elementId = executeProcedure(messageContext, name, args);

		return elementId;
	}

	private void SP_U_CLIENT_STAGE_ID(ClientCostCenter ccc, MessageContext messageContext) throws SQLException, RPMException
	{
		final String procedureName = "SP_U_CLIENT_STAGE_ID";

		Object args[] = new Object[4];

		args[0] = ccc.getID();

		if(ccc.getState()!=null){
			args[1] = new Integer(ccc.getState().getID());		    
		}

		args[2] = ccc.getExternalID();

		args[3] = getUser(messageContext).getID();

		executeProcedure(messageContext, procedureName, args);
	}
	
	private String SP_U_CLIENT_REVISION(RPMObject rpmObject,
			MessageContext messageContext)
	throws SQLException, RPMException {
		
		ClientCostCenter ccc = (ClientCostCenter) rpmObject;
		
		String name = "SP_U_CLIENT_REVISION";
	
		Object args[] = new Object[6];
		args[0] = ccc.getID();
		args[1] = new Integer(0);
		args[2] = new Integer(137); //REVISION_IMPACT = NONE (Datafields with parentId = 131)
		args[3] = new Integer(920); //REVISION_TYPE = REVISION (Datafields with parentId = 423)
		
		StatusUpdate[] statusUpdates = ccc.getStatusUpdates();
		if(statusUpdates!=null && statusUpdates.length!=0){
			StatusUpdate statusUpdate = statusUpdates[statusUpdates.length - 1]; // getting the last one
			args[4] = statusUpdate.getChangeBrief();			
		}
		
		args[5] = getUser(messageContext).getID();		

		String elementId = executeProcedure(messageContext, name, args);

		return elementId;
	}
	
	private void SP_CKIN_CLIENT(RPMObject rpmObject, MessageContext messageContext)
	throws SQLException, RPMException {
		String name = "SP_CKIN_CLIENT";
		Object args[] = new Object[3];
		args[0] = rpmObject.getID();
		args[1] = new Integer(0);
		args[2] = getUser(messageContext).getID();
		executeProcedure(messageContext,name, args);
	}
	
	private void SP_CKOUT_CLIENT(RPMObject rpmObject, MessageContext messageContext)
	throws SQLException, RPMException {
		String name = "SP_CKOUT_CLIENT";
		Object args[] = new Object[3];
		args[0] = rpmObject.getID();
		args[1] = new Integer(0);
		args[2] = getUser(messageContext).getID();
		executeProcedure(messageContext,name, args);
	}
	
	private void SP_UCKOUT_CLIENT(RPMObject rpmObject, MessageContext messageContext)
	throws SQLException, RPMException {
		String name = "SP_UCKOUT_CLIENT";
		Object args[] = new Object[3];
		args[0] = rpmObject.getID();
		args[1] = new Integer(0);
		args[2] = getUser(messageContext).getID();
		executeProcedure(messageContext,name, args);
	}
	
	public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
	throws RPMException {
		try {
			SP_CKIN_CLIENT(rpmObject, messageContext);
		} catch (SQLException s) {
		    ExceptionUtil.handleException(this, s);
		}
		
	}
	
	/**
	 * 
	 */
	public void undoCheckoutObject(RPMObject rpmObject, MessageContext messageContext)
	throws RPMException {
		try {
			SP_UCKOUT_CLIENT(rpmObject, messageContext);
		} catch (SQLException s) {
		    ExceptionUtil.handleException(this, s);
		}
	}
	
	/**
	 * @param rpmObject
	 * 
	 */
	public void checkoutObject(RPMObject rpmObject, MessageContext messageContext)
	throws RPMException {
		try {
			SP_CKOUT_CLIENT(rpmObject, messageContext);
		} catch (SQLException s) {
		    ExceptionUtil.handleException(this, s);
		}
	}

    public boolean isCheckedOut(MessageContext context, RPMObject rpmObject)
            throws RPMException, SQLException
    {
        String status = ManagerUtil.retrieveRecStatus(context, rpmObject);

        if (status != null)
        {
            return status.equalsIgnoreCase("O");
        }
        return false;
    }
}