package com.ibm.rpm.clientcostcenters.managers;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.ClientRTF;
import com.ibm.rpm.applicationadministration.containers.CostCenterState;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.managers.AttributeManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.containers.ClientModule;
import com.ibm.rpm.clientcostcenters.containers.Division;
import com.ibm.rpm.clientcostcenters.containers.GenericClient;
import com.ibm.rpm.clientcostcenters.scope.ClientCostCenterScope;
import com.ibm.rpm.clientcostcenters.scope.ClientModuleScope;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.managers.CustomFieldManager;
import com.ibm.rpm.framework.AbstractModuleManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;

public class ClientModuleManager extends AbstractModuleManager
{
    private static Log logger = LogFactory.getLog(ClientModuleManager.class);

    protected Log getLogger()
    {
        return logger;
    }

    private static final HashSet CONTAINERS = new HashSet();
    static
    {
        CONTAINERS.add(ClientModule.class.getName());
        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    ClientModuleManager.class);
        }
    }

    {
        localContextName = ContextUtil.CLIENT_CONTEXT;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = new ClientModule();
        return object;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof AttributeManager)
        {
            if (localContextName.equalsIgnoreCase(contextName))
            {
                joinCondition = new JoinCondition();
            }
            else
            {//context not linked to this manager
                logger
                        .error("ClientModuleManager.getJoinCondition: contextName "
                                + contextName
                                + " is not linked with this manager.");
            }
        }
        else if (manager instanceof CustomFieldManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setCondition(CustomFieldManager
                    .createJoinCondition(localContextName));
        }
        else if (manager instanceof ClientManager)
        {
            joinCondition = new JoinCondition();
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    protected void loadChildren(AbstractModule abstractModule,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ClientModule module = (ClientModule) abstractModule;
        ClientModuleScope scope = (ClientModuleScope) rpmScope;
        
    	DatafieldManager datafieldManager = (DatafieldManager)getManagerInstance(DatafieldCategory.class);

        
        //Clients
        if (scope.getClients() != null)
        {
            GenericClient[] clients = loadClients(module, module.getClients(), scope
                    .getClients(), messageContext, isDelta);
            module.setClients(clients);
        }
        
        //AttributeCategories
        if (scope.getAttributeCategories() != null)
        {
            AttributeManager attributeManager = (AttributeManager) getManagerInstance(AttributeCategory.class);

            AttributeCategory[] attributeCategory = attributeManager
                .loadAttributeCategories(module, module.getAttributeCategories(), scope
                    .getAttributeCategories(), this, localContextName,
                    messageContext, isDelta);
            module.setAttributeCategories(attributeCategory);
        }
        
        //ClientRTFs
        if (scope.isClientRtfs())
        {
            ClientRTF[] clientRTFs = (ClientRTF[]) datafieldManager
                .loadDatafields(module, module.getClientRtfs(), ClientRTF.class,
                    ClientRTF.TYPE_ID, this, localContextName,
                    messageContext, isDelta);
            module.setClientRtfs(clientRTFs);
        }

        //CostCenterStages
        if (scope.isCostCenterStages())
        {
            CostCenterState[] costCenterStages = (CostCenterState[]) datafieldManager
                .loadDatafields(module, module.getCostCenterStages(),
                    CostCenterState.class, CostCenterState.TYPE_ID,
                    this, localContextName, messageContext, isDelta);
            module.setCostCenterStages(costCenterStages);
        }
        
        //CustomFields
        if (scope.getCustomFieldCategories() != null)
        {
            CustomFieldManager manager = (CustomFieldManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            CustomFieldCategory.class);
            CustomFieldCategory[] categories = manager
                    .loadCustomFieldCategories(localContextName,
                            scope.getCustomFieldCategories(), messageContext);
            module.setCustomFieldCategories(categories);
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        ClientModuleScope scope = (ClientModuleScope) rpmScope;
        ClientModule module = (ClientModule) rpmObject;
        module.setContextName(localContextName);
        
        //Clients
        if (scope.getClients() != null)
        {
            GenericClient[] clients = saveClients(module, module.getClients(),
                    scope.getClients(), messageContext);
            module.setClients(clients);
        }
        
        //AttributeCagories
        if (scope.getAttributeCategories() != null)
        {
            AttributeCategory[] attributeCategory = (AttributeCategory[]) saveElements(
                module, module.getAttributeCategories(), scope.getAttributeCategories(), messageContext,
                AttributeCategory.class);
            module.setAttributeCategories(attributeCategory);
        }
    
        //CustomFields
        if (scope.getCustomFieldCategories() != null)
        {
            propagateContextName(module, module.getCustomFieldCategories());
            module
                    .setCustomFieldCategories((CustomFieldCategory[]) updateChildren(
                            null, module.getCustomFieldCategories(),
                            scope.getCustomFieldCategories(), messageContext, null));
        }

        return module;
    }

    protected GenericClient[] loadClients(AbstractModule abstractModule, GenericClient[] clients,
            ClientCostCenterScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        String extraWhere = " LEVEL_ID = 0 AND (" + " TYPE_ID = "
                + ClientFolder.TYPE_ID + " OR " + " TYPE_ID = "
                + Division.TYPE_ID + " OR " + " TYPE_ID = "
                + ClientCostCenter.TYPE_ID + " )";
        ClientManager clientManager = (ClientManager) getManagerInstance(ClientCostCenter.class
                .getName());
        ArrayList temp = clientManager.loadByForeignKeyWithSpecifiedParent(abstractModule, clients, scope,
                messageContext, this, null, extraWhere, localContextName,
                isDelta);
        GenericClient[] loadedClients = null;
        if (temp != null)
        {
            loadedClients = new GenericClient[temp.size()];
            temp.toArray(loadedClients);
        }
        return loadedClients;
    }

    protected GenericClient[] saveClients(AbstractModule module,
            GenericClient[] clients, ClientCostCenterScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        propagateContextName(module, clients);
        GenericClient[] savedClients = (GenericClient[]) updateChildren(module,
                clients, scope, messageContext, null);
        return savedClients;
    }
}
