/*
 * Created on Sep 18, 2006
 */
package com.ibm.rpm.communications.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.communications.constants.ErrorCodes;
import com.ibm.rpm.communications.constants.ValidationConstants;
import com.ibm.rpm.communications.containers.ChecklistItem;
import com.ibm.rpm.communications.containers.GenericCommunication;
import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.VoteResponse;
import com.ibm.rpm.communications.containers.WorkflowAttachment;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.managers.CommunicationManager;
import com.ibm.rpm.communications.managers.WorkflowResponseManager;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.communications.types.WorkflowResponseState;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.EnumeratedTypeUtil;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.util.ResourceValidator;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;

/**
 * @author dominiqu
 */
public class CommunicationMethodCheckpoint extends AbstractCheckpoint
{
    private static final List GENERICCOMMUNICATION_PARENTS = new ArrayList();

    private static final Class NOTIFICATION_ATTACHMENTS[] =
    {
            Deliverable.class, Project.class, Proposal.class,
            SummaryTask.class, Task.class, WorkOrganization.class,
            WorkProduct.class, Action.class, ChangeRequest.class, Defect.class,
            Milestone.class, ScopeDeliverable.class, Issue.class,
            Requirement.class, Risk.class, ServiceRequest.class,
            DocumentFolder.class, DelDoc.class, Document.class,
            NoteMinutes.class, Software.class, UrlDocument.class, WPDoc.class
    };

    private static final Class STAFFINGREQUEST_ATTACHMENTS[] =
    {
            Project.class, Proposal.class
    };

    private static final List ALL_PARENTS = new ArrayList();

    static
    {
        ALL_PARENTS.add(Notification.class);
        ALL_PARENTS.add(StaffingRequest.class);
        ALL_PARENTS.add(WorkflowStepNotification.class);
    }

    private static final List WORKFLOWRESPONSE_SENDRESPONSE_PARENTS = new ArrayList();

    static
    {
        WORKFLOWRESPONSE_SENDRESPONSE_PARENTS.add(StaffingRequest.class);
        WORKFLOWRESPONSE_SENDRESPONSE_PARENTS.add(WorkflowStepNotification.class);
    }

    private static final List WORKFLOWRESPONSE_SAVERESPONSE_PARENTS = new ArrayList();

    static
    {
        WORKFLOWRESPONSE_SAVERESPONSE_PARENTS.add(WorkflowStepNotification.class);
    }

    protected static CommunicationMethodCheckpoint instance = new CommunicationMethodCheckpoint();

    public static CommunicationMethodCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSendGenericCommunication(RPMObject rpmObject,
            MessageContext msgCtx)
    {
        // Must delta parent because this method requires that
        // testParentModified be true
        rpmObject.deltaParent(new Notification());
        super.isValidParent(rpmObject, msgCtx, false,
            GENERICCOMMUNICATION_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject,
            GenericCommunication.class, msgCtx))
            return;

        // Initialize
        GenericCommunication mainGC = (GenericCommunication) rpmObject;

        // Validate mandatory fields on creation
        if (mainGC.getID() == null)
        {
            GenericValidator.validateMandatory(mainGC,
                ValidationConstants.GENERICCOMMUNICATION_ATTACHMENT_FIELD,
                mainGC.getWorkflowAttachment(), msgCtx);

            if (GenericValidator.validateMandatoryID(mainGC,
                ValidationConstants.GENERICCOMMUNICATION_AUTHOR_FIELD,
                mainGC.getAuthor(), msgCtx))

                if (GenericValidator.validateNotNull(mainGC,
                    ValidationConstants.GENERICCOMMUNICATION_REPLIES_FIELD,
                    mainGC.getResponses(), msgCtx))
                {
                    for (int i = 0; i < mainGC.getResponses().length; ++i)
                    {
                        if (GenericValidator.validateMandatoryID(
                            mainGC.getResponses()[i],
                            ValidationConstants.WORKFLOWRESPONSE_RESOURCE_FIELD,
                            mainGC.getResponses()[i].getResource(), msgCtx))
                        {
                            // Ensure that the resource has not already been
                            // specified in an another WorkflowReply
                            for (int j = 0; j < i; ++j)
                            {
                                if (mainGC.getResponses()[i].getResource().getID() != null
                                        && mainGC.getResponses()[i].getResource().getID() == mainGC.getResponses()[j].getResource().getID())
                                {
                                    String[] errorParams =
                                    {
                                            StringUtil.getShortClassName(mainGC.getClass()),
                                            mainGC.getResponses()[i].getResource().getID()
                                    };
                                    RPMException e = new RPMException(
                                        ErrorCodes.NO_DUPLICATE_RECIPIENTS,
                                        errorParams,
                                        mainGC.getClass().getName(),
                                        ValidationConstants.GENERICCOMMUNICATION_REPLIES_FIELD);
                                    addException(e, msgCtx);
                                }
                            }

                            // For StaffingRequest, the resource must be a
                            // Resource
                            // Manager
                            if (mainGC instanceof StaffingRequest)
                            {
                                ResourceValidator.validateResourceManager(
                                    mainGC.getResponses()[i],
                                    ValidationConstants.WORKFLOWRESPONSE_RESOURCE_FIELD,
                                    mainGC.getResponses()[i].getResource().getID(),
                                    "ID", msgCtx);
                            }
                            else
                            // For all others, the resource must exist
                            {
                                ResourceValidator.validateExistingResource(
                                    mainGC.getResponses()[i],
                                    ValidationConstants.WORKFLOWRESPONSE_RESOURCE_FIELD,
                                    mainGC.getResponses()[i].getResource().getID(),
                                    "ID", msgCtx);
                            }
                        }
                    }
                }

            // Field validation: subject
            GenericValidator.validateMaxLength(mainGC,
                ValidationConstants.GENERICCOMMUNICATION_SUBJECT_FIELD,
                mainGC.getSubject(),
                ValidationConstants.GENERICCOMMUNICATION_SUBJECT_MAX, msgCtx);

            // Field validation: comments
            GenericValidator.validateMaxLength(mainGC,
                ValidationConstants.GENERICCOMMUNICATION_COMMENTS_FIELD,
                mainGC.getComments(),
                ValidationConstants.GENERICCOMMUNICATION_COMMENTS_MAX, msgCtx);

            // Field validation: workflowAttachment
            validateAttachment(mainGC.getWorkflowAttachment(), msgCtx);
        }
    }

    public void validateAttachment(RPMObject rpmObject, MessageContext msgCtx)
    {
        // No need to validate the parent, as we set it internally
        // super.isValidParent(rpmObject, msgCtx, false,
        // WORKFLOWATTACHMENT_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject,
            WorkflowAttachment.class, msgCtx))
            return;

        // Initialize
        WorkflowAttachment mainWA = (WorkflowAttachment) rpmObject;

        // Validate mandatory fields on creation
        if (mainWA.getID() == null)
        {
            boolean valid = true;

            GenericValidator.validateMandatoryID(mainWA,
                ValidationConstants.WORKFLOWATTACHMENT_ATTACHMENT_FIELD,
                mainWA.getAttachment(), msgCtx);

            if (mainWA.getParent() instanceof StaffingRequest)
            {
                valid &= GenericValidator.validateType(mainWA,
                    STAFFINGREQUEST_ATTACHMENTS,
                    ValidationConstants.WORKFLOWATTACHMENT_ATTACHMENT_FIELD,
                    mainWA.getAttachment(), msgCtx);
            }
            else if (mainWA.getParent() instanceof Notification)
            {
                valid &= GenericValidator.validateType(mainWA,
                    NOTIFICATION_ATTACHMENTS,
                    ValidationConstants.WORKFLOWATTACHMENT_ATTACHMENT_FIELD,
                    mainWA.getAttachment(), msgCtx);
            }

            if (valid)
            {
                GenericValidator.validateExistingRPMObject(mainWA,
                    ValidationConstants.WORKFLOWATTACHMENT_ATTACHMENT_FIELD,
                    mainWA.getAttachment(), msgCtx);
            }
        }
    }

    public void validateSendWorkflowResponse(WorkflowResponse rpmObject,
            MessageContext msgCtx)
    {
        // Field validation: ID is mandatory
        if (!GenericValidator.validateMandatoryID("WorkflowResponse",
            rpmObject, msgCtx))
            return;
        
        // Field validation: parent
        if (!GenericValidator.validateMandatoryID(rpmObject,
            ValidationConstants.PARENT_FIELD, rpmObject.getParent(), msgCtx))
            return;

        if (!(((Class) WORKFLOWRESPONSE_SENDRESPONSE_PARENTS.get(0)).isInstance(rpmObject.getParent()) || ((Class) WORKFLOWRESPONSE_SENDRESPONSE_PARENTS.get(1)).isInstance(rpmObject.getParent())))
        {
            RPMException e = new RPMException(
                ErrorCodes.NO_SEND_RESPONSE,
                new String[]
                {
                    StringUtil.getShortClassName(rpmObject.getParent().getClass())
                }, rpmObject.getClass().getName(), null, rpmObject.getID());
            addException(e, msgCtx);
            return;
        }

        // Field validation: comments
        GenericValidator.validateMaxLength(rpmObject,
            ValidationConstants.WORKFLOWRESPONSE_COMMENTS_FIELD,
            rpmObject.getComments(),
            ValidationConstants.WORKFLOWRESPONSE_COMMENTS_MAX, msgCtx);

        if (rpmObject.getParent() instanceof StaffingRequest)
        {
            // Field validation: state is mandatory
            GenericValidator.validateMandatory(rpmObject,
                ValidationConstants.WORKFLOWRESPONSE_STATE_FIELD,
                rpmObject.getState(), msgCtx);

            // Field validation: state must be accepted or rejected
            ArrayList possibleValues = new ArrayList();
            possibleValues.add(WorkflowResponseState.Accepted);
            possibleValues.add(WorkflowResponseState.Rejected);

            GenericValidator.validatePossibleObjectValue(rpmObject,
                ValidationConstants.WORKFLOWRESPONSE_STATE_FIELD,
                rpmObject.getState(), possibleValues, msgCtx);

            // Field validation: effort is read-only
            if (rpmObject.getEffort() != null
                    && rpmObject.getEffort().intValue() != 0)
            {
                // Add warning informing that the value will not be updated
                RPMException e = new RPMException(
                    ErrorCodes.WARNING_EFFORT_IS_READ_ONLY);
                e.setSeverity(SeverityLevel.Warning);
                msgCtx.addExceptionNoThrow(e);
            }
        }
        else
        // parent is WorkflowStepNotification
        {
            if (!GenericValidator.validateMandatory(rpmObject,
                ValidationConstants.WORKFLOWRESPONSE_RESPONSE_FIELD,
                rpmObject.getResponse(), msgCtx))
                return;

            boolean voteReasonSelected = false;
            for (int i = 0; i < rpmObject.getResponse().length; ++i)
            {
                if (rpmObject.getResponse()[i] instanceof ChecklistItem)
                {
                    // Validate that all checklist items are complete
                    if (((ChecklistItem) rpmObject.getResponse()[i]).getComplete() != Boolean.TRUE)
                    {
                        RPMException e = new RPMException(
                            ErrorCodes.ALL_CHECKLIST_ITEMS_MUST_BE_COMPLETE,
                            rpmObject.getClass().getName(),
                            ValidationConstants.WORKFLOWRESPONSE_RESPONSE_FIELD,
                            rpmObject.getID());
                        addException(e, msgCtx);
                        break;
                    }
                }
                else
                {
                    // Validate that only one response reason is selected
                    for (int j = 0; j < ((VoteResponse) rpmObject.getResponse()[i]).getReasons().length; ++j)
                    {
                        if (voteReasonSelected == true
                                && ((VoteResponse) rpmObject.getResponse()[i]).getReasons()[j] != null
                                && ((VoteResponse) rpmObject.getResponse()[i]).getReasons()[j].getSelected() == Boolean.TRUE)
                        {
                            RPMException e = new RPMException(
                                ErrorCodes.ONLY_ONE_VOTE_RESPONSE_REASON_CAN_BE_SELECTED,
                                rpmObject.getClass().getName(),
                                ValidationConstants.WORKFLOWRESPONSE_RESPONSE_FIELD,
                                rpmObject.getID());
                            addException(e, msgCtx);
                            break;
                        }
                        else if (((VoteResponse) rpmObject.getResponse()[i]).getReasons()[j].getSelected() == Boolean.TRUE)
                        {
                            voteReasonSelected = true;
                        }
                    }
                }
            }
        }
    }

    public void validateSaveWorkflowResponse(WorkflowResponse rpmObject,
            MessageContext msgCtx)
    {
        // Field validation: ID is mandatory
        if (!GenericValidator.validateMandatoryID("WorkflowResponse",
            rpmObject, msgCtx))
            return;
        
        // Field validation: parent
        if (!(((Class) WORKFLOWRESPONSE_SAVERESPONSE_PARENTS.get(0)).isInstance(rpmObject.getParent())))
        {
            RPMException e = new RPMException(
                ErrorCodes.NO_SAVE_RESPONSE,
                new String[]
                {
                    StringUtil.getShortClassName(rpmObject.getParent().getClass())
                }, rpmObject.getClass().getName(), null, rpmObject.getID());
            addException(e, msgCtx);
            return;
        }

        // Field validation: comments
        GenericValidator.validateMaxLength(rpmObject,
            ValidationConstants.WORKFLOWRESPONSE_COMMENTS_FIELD,
            rpmObject.getComments(),
            ValidationConstants.WORKFLOWRESPONSE_COMMENTS_MAX, msgCtx);

        if (!GenericValidator.validateMandatory(rpmObject,
            ValidationConstants.WORKFLOWRESPONSE_RESPONSE_FIELD,
            rpmObject.getResponse(), msgCtx))
        {
            for (int i = 0; i < rpmObject.getResponse().length; ++i)
            {
                if (!(rpmObject.getResponse()[i] instanceof ChecklistItem))
                {
                    // Cannot save votes, only checklists
                    RPMException e = new RPMException(
                        ErrorCodes.NO_SAVE_RESPONSE,
                        new String[]
                        {
                            StringUtil.getShortClassName(rpmObject.getParent().getClass())
                                    + " with "
                                    + StringUtil.getShortClassName(rpmObject.getResponse()[0].getClass())
                        }, rpmObject.getClass().getName(),
                        ValidationConstants.WORKFLOWRESPONSE_RESPONSE_FIELD,
                        rpmObject.getID());
                    addException(e, msgCtx);
                    break;
                }
            }
        }

    }

    public boolean validateSaveEffort(WorkflowResponse rpmObject,
            MessageContext msgCtx) throws RPMException
    {
        boolean callStoredProcedure = false;

        // Load config flag, which is used to determine if we can update effort
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(WorkflowResponseManager.NAME_WKF_REPLY_ID);
        
        Object[] arguments = new Object[]
        {
            rpmObject.getID()
        };
        
        Integer configFlag = (Integer) ManagerUtil.selectColumnValue(
            Integer.class, rpmObject, WorkflowResponseManager.NAME_CONFIG_FLAG,
            WorkflowResponseManager.NAME_CONFIG_FLAG,
            WorkflowResponseManager.TABLE_NAME, condition, arguments, msgCtx);

        // The second bit of the config flag indicates if the
        // WorkflowResponse's resource is assigned to a task
        if ((configFlag != null) && ((configFlag.intValue() & 2) == 2))
        {
            callStoredProcedure = true;
        }
        else if (rpmObject.getEffort() != null
                && rpmObject.getEffort().intValue() != 0)
        {
            // Add warning informing that the value will not be updated
            RPMException e = new RPMException(
                ErrorCodes.WARNING_EFFORT_IS_READ_ONLY);
            e.setSeverity(SeverityLevel.Warning);
            msgCtx.addExceptionNoThrow(e);
        }

        return callStoredProcedure;
    }

    public void validateUpdateReadAndFolder(WorkflowResponse rpmObject,
            MessageContext msgCtx)
    {
        // Field validation: ID is mandatory
        if (!GenericValidator.validateMandatoryID(
            StringUtil.getShortClassName(rpmObject.getClass()), rpmObject,
            msgCtx))
            return;

        // Get the current folder
        CommunicationsFolder currentFolder = loadCommunicationsFolder(
            rpmObject, msgCtx);

        // Cannot undo delete
        if (currentFolder == CommunicationsFolder.NotVisible)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    rpmObject.getFolder().toString(),
                    "when the response has been permanently deleted and is no longer visible"
            };
            RPMException e = new RPMException(ErrorCodes.CANNOT_MOVE_TO_FOLDER,
                errorParams, rpmObject.getClass().getName(),
                ValidationConstants.WORKFLOWRESPONSE_FOLDER_FIELD,
                rpmObject.getID());
            addException(e, msgCtx);
        }
        // Cannot move to Inbox or Sent
        else if (rpmObject.getFolder() == CommunicationsFolder.Inbox
                || rpmObject.getFolder() == CommunicationsFolder.Sent)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    rpmObject.getFolder().toString(), ""
            };
            RPMException e = new RPMException(ErrorCodes.CANNOT_MOVE_TO_FOLDER,
                errorParams, rpmObject.getClass().getName(),
                ValidationConstants.WORKFLOWRESPONSE_FOLDER_FIELD,
                rpmObject.getID());
            addException(e, msgCtx);
        }
        // Cannot move to NotVisible
        else if (rpmObject.getFolder() == CommunicationsFolder.NotVisible)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    rpmObject.getFolder().toString(),
                    "because the Delete method must be used"
            };
            RPMException e = new RPMException(ErrorCodes.CANNOT_MOVE_TO_FOLDER,
                errorParams, rpmObject.getClass().getName(),
                ValidationConstants.WORKFLOWRESPONSE_FOLDER_FIELD,
                rpmObject.getID());
            addException(e, msgCtx);
        }
        // Cannot archive or delete WorkflowResponse with parent of type
        // WorkflowStepNotification if the Workflow is in Open state
        else if ((rpmObject.getFolder() == CommunicationsFolder.Archived || rpmObject.getFolder() == CommunicationsFolder.Deleted)
                && (rpmObject.getParent() instanceof WorkflowStepNotification && loadWorkflowState(
                    rpmObject.getParent(), msgCtx) == WorkflowState.Open))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    rpmObject.getFolder().toString(),
                    "when parent workflow is in Open state"
            };
            RPMException e = new RPMException(ErrorCodes.CANNOT_MOVE_TO_FOLDER,
                errorParams, rpmObject.getClass().getName(),
                ValidationConstants.WORKFLOWRESPONSE_FOLDER_FIELD,
                rpmObject.getID());
            addException(e, msgCtx);
        }
    }

    public void validateDelete(RPMObject rpmObject, MessageContext msgCtx)
    {
        // Field validation: ID is mandatory
        if (!GenericValidator.validateMandatoryID(
            StringUtil.getShortClassName(rpmObject.getClass()), rpmObject,
            msgCtx))
            return;

        // In the client, can only delete items that are in the deleted folder
        // This could be done here in the future should be decide it is
        // necessary

        // Cannot delete WorkflowResponse with parent of type
        // WorkflowStepNotification if the Workflow is in Open state
        if (rpmObject instanceof WorkflowStepNotification)
        {
            // Load the workflow's current state
            WorkflowState state = loadWorkflowState(rpmObject, msgCtx);

            if (state == WorkflowState.Open)
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(rpmObject.getClass()),
                        "when the workflow is in Open state"
                };
                RPMException e = new RPMException(
                    ErrorCodes.CANNOT_DELETE_COMMUNICATION, errorParams,
                    rpmObject.getClass().getName(), null, rpmObject.getID());
                addException(e, msgCtx);
            }
        }
    }

    private CommunicationsFolder loadCommunicationsFolder(RPMObject rpmObject,
            MessageContext msgCtx)
    {
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(WorkflowResponseManager.NAME_WKF_REPLY_ID);
        Integer currentFolderInt = null;
        try
        {
            Object[] arguments = new Object[]
            {
                rpmObject.getID()
            };
            
            currentFolderInt = (Integer) ManagerUtil.selectColumnValue(
                Integer.class, rpmObject,
                ValidationConstants.WORKFLOWRESPONSE_FOLDER_FIELD,
                WorkflowResponseManager.NAME_REPLY_STATUS + " + "
                        + WorkflowResponseManager.NAME_REPLY_TYPE,
                WorkflowResponseManager.TABLE_NAME, condition, arguments, msgCtx);
            if (currentFolderInt != null)
            {
                return (CommunicationsFolder) EnumeratedTypeUtil.getInstance().getType(
                    WorkflowResponse.class, CommunicationsFolder.class,
                    currentFolderInt);
            }
            return null;
        }
        catch (RPMException e)
        {
            addException(e, msgCtx);
            return null;
        }
    }

    private WorkflowState loadWorkflowState(RPMObject rpmObject,
            MessageContext msgCtx)
    {
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(CommunicationManager.NAME_ELEMENT_ID);

        try
        {
            Object[] arguments = new Object[]
            {
                rpmObject.getID()
            };
            
            String stateStr = (String) ManagerUtil.selectColumnValue(
                String.class, rpmObject,
                ValidationConstants.WORKFLOWRESPONSE_FOLDER_FIELD,
                CommunicationManager.NAME_END_CYCLE,
                CommunicationManager.TABLE_NAME, condition, arguments, msgCtx);
            if (stateStr != null)
            {
                return (WorkflowState) EnumeratedTypeUtil.getInstance().getType(
                    WorkflowStepNotification.class, WorkflowState.class,
                    Integer.valueOf(stateStr));
            }
            return null;
        }
        catch (RPMException e)
        {
            addException(e, msgCtx);
            return null;
        }
    }
}
