package com.ibm.rpm.communications.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.StringTokenizer;

import com.ibm.rpm.communications.containers.ChecklistItem;
import com.ibm.rpm.communications.containers.GenericResponseType;
import com.ibm.rpm.communications.containers.VoteResponse;
import com.ibm.rpm.communications.containers.VoteResponseReason;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.util.CommunicationUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;

public class WorkflowResponseChecklistManager extends AbstractRPMObjectManager
{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        WorkflowResponseChecklistManager.class);

    static
    {
        containerMap.add(GenericResponseType.class);
        containerMap.add(ChecklistItem.class);
        containerMap.add(VoteResponse.class);
        containerMap.add(VoteResponseReason.class);
    }

    /**
     * Column WKF_REPLY_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_WKF_REPLY_ID = 1;

    public static final int TYPE_WKF_REPLY_ID = Types.CHAR;

    public static final String NAME_WKF_REPLY_ID = "WKF_CHECKLIST.WKF_REPLY_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WKF_REPLY_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_WKF_REPLY_ID,NAME_WKF_REPLY_ID);}

    /**
     * Column CHECKLIST_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CHECKLIST_ID = 2;

    public static final int TYPE_CHECKLIST_ID = Types.CHAR;

    public static final String NAME_CHECKLIST_ID = "WKF_CHECKLIST.CHECKLIST_ID";

    public static final String PROPERTY_CHECKLIST_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CHECKLIST_ID, NAME_CHECKLIST_ID);
    }

    /**
     * Column CHECKLIST_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CHECKLIST_NAME = 3;

    public static final int TYPE_CHECKLIST_NAME = Types.VARCHAR;

    public static final String NAME_CHECKLIST_NAME = "WKF_CHECKLIST.CHECKLIST_NAME";

    public static final String PROPERTY_CHECKLIST_NAME_CHECKLIST_ITEM = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CHECKLIST_NAME_CHECKLIST_ITEM,
            NAME_CHECKLIST_NAME);
    }

    public static final String PROPERTY_CHECKLIST_NAME_VOTE_RESPONSE = "QUESTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CHECKLIST_NAME_VOTE_RESPONSE,
            NAME_CHECKLIST_NAME);
    }

    /**
     * Column WEIGHT_PERCENT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WEIGHT_PERCENT = 4;

    public static final int TYPE_WEIGHT_PERCENT = Types.INTEGER;

    public static final String NAME_WEIGHT_PERCENT = "WKF_CHECKLIST.WEIGHT_PERCENT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WEIGHT_PERCENT = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_WEIGHT_PERCENT,NAME_WEIGHT_PERCENT);}

    /**
     * Column RESPONSE_REASON of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RESPONSE_REASON = 5;

    public static final int TYPE_RESPONSE_REASON = Types.VARCHAR;

    public static final String NAME_RESPONSE_REASON = "WKF_CHECKLIST.RESPONSE_REASON";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RESPONSE_REASON = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_RESPONSE_REASON,NAME_RESPONSE_REASON);}

    /**
     * Column SELECTED_REASON of type Types.VARCHAR mapped to String.
     */
    public static final int ID_SELECTED_REASON = 6;

    public static final int TYPE_SELECTED_REASON = Types.VARCHAR;

    public static final String NAME_SELECTED_REASON = "WKF_CHECKLIST.SELECTED_REASON";

    public static final String PROPERTY_SELECTED_REASON = "COMPLETE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SELECTED_REASON, SqlUtil.getBooleanValue(
            NAME_SELECTED_REASON, "1"));
    }

    /**
     * Column CONFIG_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONFIG_FLAG = 7;

    public static final int TYPE_CONFIG_FLAG = Types.INTEGER;

    public static final String NAME_CONFIG_FLAG = "WKF_CHECKLIST.CONFIG_FLAG";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CONFIG_FLAG = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CONFIG_FLAG,NAME_CONFIG_FLAG);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 8;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "WKF_CHECKLIST.TYPE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 9;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "WKF_CHECKLIST.REC_DATETIME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    public static final String TABLE_NAME = "WKF_CHECKLIST";

    /**
     * Create an array of type string containing all the fields of the
     * WKF_CHECKLIST table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_WKF_REPLY_ID, NAME_CHECKLIST_ID, NAME_CHECKLIST_NAME,
            NAME_WEIGHT_PERCENT, NAME_RESPONSE_REASON, NAME_SELECTED_REASON,
            NAME_CONFIG_FLAG, NAME_TYPE_ID, NAME_REC_DATETIME
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new WorkflowResponseChecklistManager instance.
     * 
     * @return WorkflowResponseChecklistManager
     */
    public WorkflowResponseChecklistManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getFilter()
    {
        SqlBuffer filter = new SqlBuffer();

        filter.appendIsNotNull(NAME_WKF_REPLY_ID);
        filter.appendIsNotNull(NAME_CHECKLIST_ID);
        
        return filter.toString();
    }

    public String getPrimaryKey()
    {
        return null;
    }

    public String[] getPrimaryKeyArray(RPMObject rpmObject) throws RPMException
    {
        return new String[]
        {
                NAME_WKF_REPLY_ID, NAME_CHECKLIST_ID
        };
    }

    public void isValidPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        boolean valid = true;
        StringBuffer invalidPrimaryKeys = new StringBuffer();

        if (rpmObject instanceof GenericResponseType)
        {
            if (rpmObject.getID() == null)
            {
                invalidPrimaryKeys.append("ID");
                valid = false;
            }

            if (rpmObject.getParent().getID() == null)
            {
                if (invalidPrimaryKeys.length() != 0)
                    invalidPrimaryKeys.append(", ");
                invalidPrimaryKeys.append("Parent.ID");
                valid = false;
            }
        }

        if (!valid)
        {
            ExceptionUtil.handleMissingPrimaryKey(this, rpmObject,
                invalidPrimaryKeys.toString());
        }
    }

    protected RPMObject setPrimaryKeyFields(Object[] primaryKey,
            RPMObject rpmObject)
    {
        if (rpmObject instanceof GenericResponseType)
        {
            rpmObject.setID((String) primaryKey[1]);

            if (rpmObject.getParent() == null)
            {
                rpmObject.setParent(new WorkflowResponse());
            }
            rpmObject.getParent().setID((String) primaryKey[0]);
        }

        return rpmObject;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public ContainerMap getContainerMap()
    {
        return containerMap;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return containerMap.createContainer(typeId);
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        rpmObject.setID(rs.getString(ID_CHECKLIST_ID));

        if (rpmObject instanceof ChecklistItem)
        {
            ChecklistItem temp = (ChecklistItem) rpmObject;
            if (isDelta)
            {
                temp.deltaDescription(rs.getString(ID_CHECKLIST_NAME));
                temp.deltaComplete(decodeConfigFlag(rs.getInt(ID_CONFIG_FLAG)));
            }
            else
            {
                temp.setDescription(rs.getString(ID_CHECKLIST_NAME));
                temp.setComplete(decodeConfigFlag(rs.getInt(ID_CONFIG_FLAG)));
            }
        }
        else if (rpmObject instanceof VoteResponse)
        {
            VoteResponse temp = (VoteResponse) rpmObject;
            if (isDelta)
            {
                temp.deltaResponse(rs.getString(ID_CHECKLIST_NAME));
            }
            else
            {
                temp.setResponse(rs.getString(ID_CHECKLIST_NAME));
            }
            temp.setReasons(decodeReasons(rs.getString(ID_RESPONSE_REASON),
                rs.getString(ID_SELECTED_REASON), rs.getInt(ID_CONFIG_FLAG)));
        }

        return rpmObject;
    }

    private Boolean decodeConfigFlag(int completeInt)
    {
        return (completeInt == 1) ? Boolean.TRUE : Boolean.FALSE;
    }

    public static Integer encodeConfigFlag(Boolean complete)
    {
        return (complete == Boolean.TRUE) ? new Integer(1) : new Integer(0);
    }

    private VoteResponseReason[] decodeReasons(String reasons,
            String selectedReason, int configFlag)
    {
        if (reasons == null)
        {
            VoteResponseReason[] voteResponseReasons = new VoteResponseReason[1];
            voteResponseReasons[0] = new VoteResponseReason();
            voteResponseReasons[0].setReason("");
            if (decodeConfigFlag(configFlag) == Boolean.TRUE
                    && (selectedReason == null || selectedReason.compareTo("") == 0))
            {
                voteResponseReasons[0].setSelected(Boolean.TRUE);
            }

            return voteResponseReasons;
        }

        StringTokenizer reasonsST = new StringTokenizer(reasons, "\r\n");
        VoteResponseReason[] voteResponseReasons = new VoteResponseReason[reasonsST.countTokens() + 1];

        // A blank reason is possible
        voteResponseReasons[0] = new VoteResponseReason();
        voteResponseReasons[0].setReason("");
        if (decodeConfigFlag(configFlag) == Boolean.TRUE
                && (selectedReason == null || selectedReason.compareTo("") == 0))
        {
            voteResponseReasons[0].setSelected(Boolean.TRUE);
        }

        for (int i = 1; reasonsST.hasMoreTokens(); ++i)
        {
            String reason = reasonsST.nextToken();
            voteResponseReasons[i] = new VoteResponseReason();
            voteResponseReasons[i].setReason(reason);
            if (decodeConfigFlag(configFlag) == Boolean.TRUE && reason != null
                    && selectedReason != null
                    && reason.compareTo(selectedReason) == 0)
            {
                voteResponseReasons[i].setSelected(Boolean.TRUE);
            }
        }

        return voteResponseReasons;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_WKF_REPLY_ID);
        relatedFields.put(index, ID_WKF_REPLY_ID, parentId);

        return relatedFields;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WorkflowResponseManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_WKF_REPLY_ID,
                WorkflowResponseManager.NAME_WKF_REPLY_ID,
                WorkflowResponseManager.TABLE_NAME,
                WorkflowResponseManager.NAME_WKF_REPLY_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        loadParent(rpmObject, rpmScope, messageContext, relatedFields, index,
            isDelta);

        return rpmObject;
    }

    private void loadParent(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        if (rpmScope == null || rpmScope.getParent() == null)
            return;

        WorkflowResponseManager manager = (WorkflowResponseManager) RPMManagerFactory.getInstance().getRPMObjectManager(
            WorkflowResponse.class.getName());

        // Get the correct Parent ID
        String parentID = (String) relatedFields.get(index, ID_WKF_REPLY_ID);

        RPMObject parent;
        if (rpmObject.getParent() == null)
        {
            parent = new WorkflowResponse();
            parent.setID(parentID);
        }
        else
        {
            parent = rpmObject.getParent();
        }

        if (parent.getID() != null)
        {
            parent = manager.loadByPrimaryKey(parent, null, messageContext,
                isDelta);

            if (isDelta)
                rpmObject.deltaParent(parent);
            else
                rpmObject.setParent(parent);
        }
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoCreate(this, rpmObject);
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ExceptionUtil.handleNoUpdate(this, rpmObject);
        return rpmObject;
    }

    public void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

    // Call
    // SP_U_WKF_CHECKLIST('043615BEE35B4D68B177C46E16C5E9B5','C2AC807A897642D98434
    // 138091560C5F','reason 1 1',0,'PMO_SUPERVISOR')
    public void SP_U_WKF_CHECKLIST(ChecklistItem checklistItem,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_WKF_CHECKLIST";
        Object args[] = new Object[5];

        args[0] = checklistItem.getID();
        args[1] = checklistItem.getParent().getID();
        args[2] = null;
        args[3] = encodeConfigFlag(checklistItem.getComplete());
        args[4] = CommunicationUtil.getResourceId(
            (WorkflowResponse) checklistItem.getParent(), messageContext);

        executeProcedure(messageContext, procedureName, args);
    }

    public void SP_U_WKF_CHECKLIST(VoteResponse voteResponse,
            String selectedReason, MessageContext messageContext)
            throws SQLException, RPMException
    {
        final String procedureName = "SP_U_WKF_CHECKLIST";
        Object args[] = new Object[5];

        args[0] = voteResponse.getID();
        args[1] = voteResponse.getParent().getID();
        args[2] = selectedReason;
        args[3] = new Integer(0);
        args[4] = CommunicationUtil.getResourceId(
            (WorkflowResponse) voteResponse.getParent(), messageContext);

        executeProcedure(messageContext, procedureName, args);
    }
}
