/*
 * Created on Dec 13, 2006
 */
package com.ibm.rpm.communications.util;

import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.managers.WorkflowResponseManager;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.workflow.managers.WorkflowManager;

public class CommunicationUtil
{
    public static boolean isNotification(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == Notification.TYPE_ID;
    }

    public static boolean isWorkflowStepNotification(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == WorkflowStepNotification.TYPE_ID;
    }

    public static boolean isStaffingRequest(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == StaffingRequest.TYPE_ID;
    }

    public static String getResourceId(WorkflowResponse workflowResponse,
            MessageContext messageContext) throws RPMException
    {
        String id = null;
        if (workflowResponse.getResource() != null)
        {
            id = workflowResponse.getResource().getID();
        }
        if (id == null)
        {
            // Must load it
            SqlBuffer condition = new SqlBuffer();
            condition.appendEqualQuestionMark(WorkflowResponseManager.NAME_WKF_REPLY_ID);
            
            Object[] arguments = new Object[]
            {
                workflowResponse.getID()
            };
            
            ManagerUtil.selectColumnValue(String.class, workflowResponse,
                "resource", WorkflowResponseManager.NAME_RESOURCE_ID,
                WorkflowResponseManager.TABLE_NAME, condition, arguments, messageContext);
        }
        return id;
    }

    /**
     * All staffing requests and notifications have a hidden workflow parent,
     * that can have a different unique ID for each database. This function
     * selects the parent ID from the TMT_WORKFLOW_CYCLES table.
     * 
     * @param rpmObject
     * @param messageContext
     * @return
     * @throws RPMException
     */
    public static Object getRootParentID(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        SqlBuffer condition = new SqlBuffer();

        if (isNotification(rpmObject))
        {
            // Load the parent of all notifications
            condition.append(WorkflowManager.NAME_ELEMENT_NAME);
            condition.append("='Notification' AND ");
            condition.append(WorkflowManager.NAME_TYPE_ID);
            condition.append("=85 AND ");
            condition.append(WorkflowManager.NAME_PARENT_ID);
            condition.append("='NOTIFICATION'");

            return ManagerUtil.selectColumnValue(String.class, rpmObject, "ID",
                WorkflowManager.NAME_ELEMENT_ID, WorkflowManager.TABLE_NAME,
                condition, null, messageContext);
        }
        else if (isStaffingRequest(rpmObject))
        {
            // Load the parent of all staffing requests
            condition.append(WorkflowManager.NAME_ELEMENT_NAME);
            condition.append("='Staffing Request' AND ");
            condition.append(WorkflowManager.NAME_TYPE_ID);
            condition.append("=297 AND ");
            condition.append(WorkflowManager.NAME_PARENT_ID);
            condition.append("='STAFFINGREQUESTS'");

            return ManagerUtil.selectColumnValue(String.class, rpmObject, "ID",
                WorkflowManager.NAME_ELEMENT_ID, WorkflowManager.TABLE_NAME,
                condition, null, messageContext);
        }

        return null;
    }
}
