/*
 * Created on Sep 20, 2006
 */
package com.ibm.rpm.communications.util;

import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;

/**
 * @author dominiqu
 */
public class WorkflowAttachmentUtil
{
    private static final int SC_TABLE_WBS = 1; //TMT_WBS

    private static final int SC_TABLE_CRI = 3; //TMT_CRI

    private static final int SC_TABLE_DOC = 4; //TMT_DOCUMENTS

    private static final int SC_TABLE_RES = 7; //RESOURCES

    private static Map tableTypeByContext = new HashMap();

    private static Map tableTypeByTypeId = new HashMap();

    private static Map attachmentByTableType = new HashMap();

    private static Map attachmentByTypeId = new HashMap();

    static
    {

        // WBS
        tableTypeByContext.put(ContextUtil.WBS_CONTEXT, new Integer(
            SC_TABLE_WBS));
        attachmentByTableType.put(new Integer(SC_TABLE_WBS), WorkElement.class);

        // Scope
        tableTypeByContext.put(ContextUtil.SCOPE_CONTEXT, new Integer(
            SC_TABLE_CRI));
        attachmentByTableType.put(new Integer(SC_TABLE_CRI), ScopeElement.class);

        // Document
        tableTypeByContext.put(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT, new Integer(
            SC_TABLE_DOC));
        attachmentByTableType.put(new Integer(SC_TABLE_DOC),
            GenericDocument.class);

        // Resource
        tableTypeByContext.put(ContextUtil.RESOURCE_CONTEXT, new Integer(
            SC_TABLE_RES));
        attachmentByTableType.put(new Integer(SC_TABLE_RES), Resource.class);

        // All Supported Type IDs
        attachmentByTypeId.put(new Integer(Deliverable.TYPE_ID),
            Deliverable.class);
        attachmentByTypeId.put(new Integer(Project.TYPE_ID), Project.class);
        attachmentByTypeId.put(new Integer(Proposal.TYPE_ID), Proposal.class);
        attachmentByTypeId.put(new Integer(SummaryTask.TYPE_ID),
            SummaryTask.class);
        attachmentByTypeId.put(new Integer(WorkOrganization.TYPE_ID),
            WorkOrganization.class);
        attachmentByTypeId.put(new Integer(WorkProduct.TYPE_ID),
            WorkProduct.class);
        attachmentByTypeId.put(new Integer(Action.TYPE_ID), Action.class);
        attachmentByTypeId.put(new Integer(ChangeRequest.TYPE_ID),
            ChangeRequest.class);
        attachmentByTypeId.put(new Integer(Defect.TYPE_ID), Defect.class);
        attachmentByTypeId.put(new Integer(ScopeDeliverable.TYPE_ID),
            ScopeDeliverable.class);
        attachmentByTypeId.put(new Integer(Issue.TYPE_ID), Issue.class);
        attachmentByTypeId.put(new Integer(Requirement.TYPE_ID),
            Requirement.class);
        attachmentByTypeId.put(new Integer(Risk.TYPE_ID), Risk.class);
        attachmentByTypeId.put(new Integer(ServiceRequest.TYPE_ID),
            ServiceRequest.class);
        attachmentByTypeId.put(new Integer(ScopeSummaryTask.TYPE_ID),
            ScopeSummaryTask.class);
        attachmentByTypeId.put(new Integer(DocumentFolder.TYPE_ID),
            DocumentFolder.class);
        attachmentByTypeId.put(new Integer(DelDoc.TYPE_ID), DelDoc.class);
        attachmentByTypeId.put(new Integer(Document.TYPE_ID), Document.class);
        attachmentByTypeId.put(new Integer(NoteMinutes.TYPE_ID),
            NoteMinutes.class);
        attachmentByTypeId.put(new Integer(Software.TYPE_ID), Software.class);
        attachmentByTypeId.put(new Integer(UrlDocument.TYPE_ID),
            UrlDocument.class);
        attachmentByTypeId.put(new Integer(WPDoc.TYPE_ID), WPDoc.class);

        tableTypeByTypeId.put(Deliverable.class, new Integer(SC_TABLE_WBS));
        tableTypeByTypeId.put(Project.class, new Integer(SC_TABLE_WBS));
        tableTypeByTypeId.put(Proposal.class, new Integer(SC_TABLE_WBS));
        tableTypeByTypeId.put(SummaryTask.class, new Integer(SC_TABLE_WBS));
        tableTypeByTypeId.put(WorkOrganization.class, new Integer(SC_TABLE_WBS));
        tableTypeByTypeId.put(WorkProduct.class, new Integer(SC_TABLE_WBS));
        tableTypeByTypeId.put(WorkElement.class, new Integer(SC_TABLE_WBS));
        tableTypeByTypeId.put(Action.class, new Integer(SC_TABLE_CRI));
        tableTypeByTypeId.put(ChangeRequest.class, new Integer(SC_TABLE_CRI));
        tableTypeByTypeId.put(Defect.class, new Integer(SC_TABLE_CRI));
        tableTypeByTypeId.put(ScopeDeliverable.class, new Integer(SC_TABLE_CRI));
        tableTypeByTypeId.put(Issue.class, new Integer(SC_TABLE_CRI));
        tableTypeByTypeId.put(Requirement.class, new Integer(SC_TABLE_CRI));
        tableTypeByTypeId.put(Risk.class, new Integer(SC_TABLE_CRI));
        tableTypeByTypeId.put(ServiceRequest.class, new Integer(SC_TABLE_CRI));
        tableTypeByTypeId.put(ScopeSummaryTask.class, new Integer(SC_TABLE_CRI));
        tableTypeByTypeId.put(ScopeElement.class, new Integer(SC_TABLE_CRI));
        tableTypeByTypeId.put(DocumentFolder.class, new Integer(SC_TABLE_DOC));
        tableTypeByTypeId.put(DelDoc.class, new Integer(SC_TABLE_DOC));
        tableTypeByTypeId.put(Document.class, new Integer(SC_TABLE_DOC));
        tableTypeByTypeId.put(NoteMinutes.class, new Integer(SC_TABLE_DOC));
        tableTypeByTypeId.put(Software.class, new Integer(SC_TABLE_DOC));
        tableTypeByTypeId.put(UrlDocument.class, new Integer(SC_TABLE_DOC));
        tableTypeByTypeId.put(WPDoc.class, new Integer(SC_TABLE_DOC));
        tableTypeByTypeId.put(GenericDocument.class, new Integer(SC_TABLE_DOC));
        tableTypeByTypeId.put(Resource.class, new Integer(SC_TABLE_RES));
    }

    public static Integer getTableType(RPMObject object)
    {
        return (Integer) tableTypeByTypeId.get(object.getClass());
    }

    public static Integer getTableType(String contextName)
    {
        if (ContextUtil.getInstance().isDocumentContext(contextName))
        {
            contextName = ContextUtil.UMBRELLA_DOCUMENT_CONTEXT;
        }
        return (Integer) tableTypeByContext.get(contextName);
    }

    public static RPMObject createAttachmentInstanceFromTableType(
            Integer tableType, MessageContext messageContext)
    {
        Class attachmentClass = (Class) attachmentByTableType.get(tableType);
        RPMObject attachment = null;
        if (attachmentClass != null)
        {
            try
            {
                attachment = (RPMObject) attachmentClass.newInstance();
            }
            catch (InstantiationException e)
            {
                if (messageContext != null)
                {
                    messageContext.addExceptionNoThrow(e);
                }
            }
            catch (IllegalAccessException e)
            {
                if (messageContext != null)
                {
                    messageContext.addExceptionNoThrow(e);
                }
            }
        }
        return attachment;
    }

    public static RPMObject createAttachmentInstanceFromTypeId(Integer typeId, 
            MessageContext messageContext)
    {
        Class attachmentClass = (Class) attachmentByTypeId.get(typeId);
        RPMObject attachment = null;
        if (attachmentClass != null)
        {
            try
            {
                attachment = (RPMObject) attachmentClass.newInstance();
            }
            catch (InstantiationException e)
            {
                if (messageContext != null)
                {
                    messageContext.addExceptionNoThrow(e);
                }
            }
            catch (IllegalAccessException e)
            {
                if (messageContext != null)
                {
                    messageContext.addExceptionNoThrow(e);
                }
            }
        }
        return attachment;
    }
}