/*
 * Created on Dec 21, 2005
 * @author mmatwiy
 * Checkpoint validation of container ContractChargeSchedule.
 */
package com.ibm.rpm.contract.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.contract.constants.ValidationConstants;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.containers.ContractChargeSchedule;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

public class ContractChargeScheduleCheckpoint extends AbstractCheckpoint
{
    private static final List CHARGE_SCHEDULE_PARENT_LIST = new ArrayList();

    protected static ContractChargeScheduleCheckpoint instance = new ContractChargeScheduleCheckpoint();

    static
    {
        CHARGE_SCHEDULE_PARENT_LIST.add(Contract.class);
    }

    public static ContractChargeScheduleCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true,
            CHARGE_SCHEDULE_PARENT_LIST);

        ContractChargeSchedule chargeSchedule = (ContractChargeSchedule) rpmObject;

        // On update, the amount and schedule date fields cannot be cleared
        GenericValidator.validateDefaultValueUpdate(chargeSchedule,
            ValidationConstants.AMOUNT_FIELD, chargeSchedule.getAmount(),
            msgCtx);

        GenericValidator.validateDefaultValueUpdate(chargeSchedule,
            ValidationConstants.SCHEDULE_DATE_FIELD, chargeSchedule
                .getScheduleDate(), msgCtx);

        // The amount field must be in the correct range of values
        if (chargeSchedule.testAmountModified())
        {
            GenericValidator.validateRange(chargeSchedule,
                ValidationConstants.AMOUNT_FIELD, chargeSchedule.getAmount(),
                ValidationConstants.AMOUNT_MIN, ValidationConstants.AMOUNT_MAX,
                msgCtx);
        }
        
        // The date field must be in the correct range of values
        if (chargeSchedule.testScheduleDateModified())
        {
            GenericValidator.validateRange(chargeSchedule,
                ValidationConstants.SCHEDULE_DATE_FIELD, chargeSchedule.getScheduleDate(),
                ValidationConstants.SCHEDULE_DATE_MIN, ValidationConstants.SCHEDULE_DATE_MAX,
                msgCtx);
        }
    }

}
