/*
 * Created on Dec 21, 2005
 * @author mmatwiy
 * Checkpoint validation of container Contract.
 */
package com.ibm.rpm.contract.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.contract.constants.ValidationConstants;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.containers.ContractFolder;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

public class ContractCheckpoint extends GenericContractCheckpoint
{
    private static final List CONTRACT_PARENT_LIST = new ArrayList();

    protected static ContractCheckpoint instance = new ContractCheckpoint();

    static
    {
        CONTRACT_PARENT_LIST.add(ContractFolder.class);
    }

    public static ContractCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, msgCtx, false, CONTRACT_PARENT_LIST);

        Contract contract = (Contract) rpmObject;

        // On update, the contractType field cannot be cleared
        GenericValidator.validateDefaultValueUpdate(contract,
            ValidationConstants.CONTRACT_TYPE_FIELD,
            contract.getContractType(), msgCtx);

        if (contract.testDescriptionModified())
        {
            GenericValidator.validateMaxLength(contract,
                ValidationConstants.DESCRIPTION_FIELD, contract
                    .getDescription(), ValidationConstants.DESCRIPTION_MAX,
                msgCtx);
        }

        if (contract.testExternalReferenceModified())
        {
            GenericValidator.validateMaxLength(contract,
                ValidationConstants.EXTERNAL_REFERENCE_FIELD, contract
                    .getExternalReference(),
                ValidationConstants.EXTERNAL_REFERENCE_MAX, msgCtx);
        }

        if (contract.testMaximumAmountModified())
        {
            GenericValidator.validateRange(contract,
                ValidationConstants.MAXIMUM_AMOUNT_FIELD, contract
                    .getMaximumAmount(),
                ValidationConstants.MAXIMUM_AMOUNT_MIN,
                ValidationConstants.MAXIMUM_AMOUNT_MAX, msgCtx);
        }
        
        if(contract.testContractTypeModified())
        {
            if(contract.getContractType() != null && contract.getContractType().getID() == null)
            {
                GenericValidator.validateExistingRPMObject(contract, "contractType", 
                    contract.getContractType(), msgCtx);
            }
        }
    }
}
