package com.ibm.rpm.contract.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;

//TODO Copy next line in RPMInitializer.loadManagers()
//Class.forName(ContractBreakdownManager.class.getName());

public class ContractBreakdownManager extends AbstractRPMObjectManager{

    private static Log logger = LogFactory.getLog(ContractBreakdownManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static{
        //TODO Add the complete class name of all the containers that could be loaded by this managers.
        //CONTAINERS.add(Container.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ContractBreakdownManager.class);
        }
    }
    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "CONTRACT_BREAKDOWN.ELEMENT_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_ELEMENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}

    /**
     * Column CONTRACT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CONTRACT_ID = 2;
    public static final int TYPE_CONTRACT_ID = Types.CHAR;
    public static final String NAME_CONTRACT_ID = "CONTRACT_BREAKDOWN.CONTRACT_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CONTRACT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CONTRACT_ID,NAME_CONTRACT_ID);}

    /**
     * Column CHARGE_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_CHARGE_DATE = 3;
    public static final int TYPE_CHARGE_DATE = Types.DATE;
    public static final String NAME_CHARGE_DATE = "CONTRACT_BREAKDOWN.CHARGE_DATE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CHARGE_DATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHARGE_DATE,NAME_CHARGE_DATE);}

    /**
     * Column CHARGE_AMOUNT of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_CHARGE_AMOUNT = 4;
    public static final int TYPE_CHARGE_AMOUNT = Types.DECIMAL;
    public static final String NAME_CHARGE_AMOUNT = "CONTRACT_BREAKDOWN.CHARGE_AMOUNT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CHARGE_AMOUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHARGE_AMOUNT,NAME_CHARGE_AMOUNT);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 5;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "CONTRACT_BREAKDOWN.REC_USER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 6;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "CONTRACT_BREAKDOWN.REC_STATUS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 7;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "CONTRACT_BREAKDOWN.REC_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    private static final String TABLE_NAME = "CONTRACT_BREAKDOWN";

    /**
     * Create an array of type string containing all the fields of the CONTRACT_BREAKDOWN table.
     */
    private static final String[] FIELD_NAMES = 
    {
         NAME_ELEMENT_ID
        ,NAME_CONTRACT_ID
        ,NAME_CHARGE_DATE
        ,NAME_CHARGE_AMOUNT
        ,NAME_REC_USER
        ,NAME_REC_STATUS
        ,NAME_REC_DATETIME
    };

    private static final String ALL_FIELDS;
    static{
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    };

    /**
     * Creates a new ContractBreakdownManager instance.
     * @return ContractBreakdownManager
     */   
    public ContractBreakdownManager(){
        super();
    }

    public String getTableName() {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames(){
        return FIELD_NAMES;
    }

    protected String getAllFields() {
        return ALL_FIELDS;
    }  

    public String getPrimaryKey() {
        //TODO validate the primary key field name.
        //return NAME_ELEMENT_ID;
        return null;
    }  

    protected HashMap getPropertiesMap() {
        return FIELDPROPERTYMAP;
    }  

    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            if (condition != null)
            {
                where.append(condition);
            	 //uncomment if additionnal condition is needed
                //where.append(" AND ");
             }
             //TODO Add additionnal condition if needed
             //where.append(condition on status or type...);

             joinCondition.setCondition(where.toString());
    
        }

        //TODO Implement for managers that handles the children of the
        //     supported containers

        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
       }
    
        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement for managers that handles the parents of the
        //     supported containers
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);

        return joinCondition;
    }

    protected Integer getContainerType(ResultSet resultSet) throws RPMException, SQLException {

        //TODO validate the container type field name.
        //int typeId = resultSet.getInt(ID_TYPE_ID);
        //return new Integer(typeId);

    	  //If container type doesn't apply to this manager
        return new Integer(0);
    }  

    protected RPMObject createContainer(int typeId) throws RPMException {

         RPMObject object = null;

    switch (typeId) {

         //TODO create container of the requested type
         case 0:
         //case Container.TYPE_ID:
         //    object = new Container();
             break;

         default:
             ExceptionUtil.handleUnsupportedTypeId(this, typeId);
         }
         return object;
     }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        // TODO This methos must be implemented
        return rpmObject;

    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        // TODO Implement this method if you need values from the result set
        //      to load the related objects.
        return super.decodeRelatedFields(resultSet, relatedFields, index);

    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        // TODO Implement this method if you need to load children and related objects
        return super.loadRelatedObjects(rpmObject, rpmScope, messageContext,
        relatedFields, index, isDelta);
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        // TODO Implement for managers that can load the supported
        //      container as children
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // TODO Implement if the manager can create new objects. 

    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        // TODO Implement if the manager update objects 
        return rpmObject;

    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // TODO Implement if the manager can delete objects

    }
}
