/*
 * Created on Dec 12, 2005
 * 
 */
package com.ibm.rpm.customfield.managers;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.managers.AssetManager;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.managers.ClientManager;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.util.ArrayUtil;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsManager;

/**
 * Holds 2 methods that provide relations between AssignedCustomField and it's
 * parent, like Resource, WBS, etc.
 * 
 * The need for this helper class comes from the inability to precook objects at
 * AssignedCustomFieldManager itself. We need 2 kind of objects - ones that are
 * actually AssignedCustomFields, and procooked ones - those cooked from
 * CustomFields with default valued.
 * 
 * As soon as the user modifies defaultValue -
 * 
 * @author ikusakov
 */
public class CustomFieldAssignmentUtil
{
    private static final int SC_TABLE_WBS = 1; //TMT_WBS

    private static final int SC_TABLE_CRI = 3; //TMT_CRI

    private static final int SC_TABLE_DOC = 4; //TMT_DOCUMENTS

    private static final int SC_TABLE_RES = 7; //RESOURCES

    private static final int SC_TABLE_AST = 9; //TMT_ASSETS

    private static final int SC_TABLE_CLN = 11; //TMT_CLIENTS

    private static Map managerToContext = new HashMap();

    private static Map tableNameByContext = new HashMap();

    private static Map tableTypeByContext = new HashMap();
    
    private static Map parentTableTypeByContext = new HashMap();

    private static Map parentByContext = new HashMap();

    private static Map contextByParent = new HashMap();

    private static Map parentPKColumnByContext = new HashMap();

    static
    {

        // WBS
        tableNameByContext.put(ContextUtil.WBS_CONTEXT,
                "TMT_WBS_CRI_CUSTOM_FIELDS");
        tableTypeByContext.put(ContextUtil.WBS_CONTEXT, new Integer(
                SC_TABLE_WBS));
        parentTableTypeByContext.put(ContextUtil.WBS_CONTEXT, new Integer(
            SC_TABLE_WBS));
        parentByContext.put(ContextUtil.WBS_CONTEXT, WorkElement.class);
        contextByParent.put(WorkElement.class, ContextUtil.WBS_CONTEXT);
        parentPKColumnByContext.put(ContextUtil.WBS_CONTEXT,
                WbsManager.NAME_ELEMENT_ID); //  "TMT_WBS.ELEMENT_ID"
        managerToContext.put(WbsManager.class, ContextUtil.WBS_CONTEXT);

        // Scope
        tableNameByContext.put(ContextUtil.SCOPE_CONTEXT,
                "TMT_WBS_CRI_CUSTOM_FIELDS");
        // !!! Note, that we have to put SC_TABLE_WBS here instead of
        // SC_TABLE_CRI
        // because in TMT_CUSTOM_FIELD table it uses SC_TABLE_WBS for both cases
        tableTypeByContext.put(ContextUtil.SCOPE_CONTEXT, new Integer(
            SC_TABLE_WBS));
        parentTableTypeByContext.put(ContextUtil.SCOPE_CONTEXT, new Integer(
            SC_TABLE_CRI));
        parentByContext.put(ContextUtil.SCOPE_CONTEXT, ScopeElement.class);
        contextByParent.put(ScopeElement.class, ContextUtil.SCOPE_CONTEXT);
        parentPKColumnByContext.put(ContextUtil.SCOPE_CONTEXT,
                ScopeManager.NAME_ELEMENT_ID); // "TMT_CRI.ELEMENT_ID"
        managerToContext.put(ScopeManager.class, ContextUtil.SCOPE_CONTEXT);

        // Document
        tableNameByContext.put(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT,
                "TMT_DOC_CUSTOM_FIELDS");
        tableTypeByContext.put(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT, new Integer(
                SC_TABLE_DOC));
        parentTableTypeByContext.put(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT, new Integer(
            SC_TABLE_DOC));
        parentByContext
                .put(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT, GenericDocument.class);
        contextByParent
                .put(GenericDocument.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        parentPKColumnByContext.put(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT,
                DocumentManager.NAME_ELEMENT_ID); // "TMT_DOCUMENTS.ELEMENT_ID"
        managerToContext.put(DocumentManager.class,
                ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);

        // Resource
        tableNameByContext.put(ContextUtil.RESOURCE_CONTEXT,
                "TMT_RESOURCE_CUSTOM_FIELDS");
        tableTypeByContext.put(ContextUtil.RESOURCE_CONTEXT, new Integer(
                SC_TABLE_RES));
        parentTableTypeByContext.put(ContextUtil.RESOURCE_CONTEXT, new Integer(
            SC_TABLE_RES));
        parentByContext.put(ContextUtil.RESOURCE_CONTEXT, Resource.class);
        contextByParent.put(Resource.class, ContextUtil.RESOURCE_CONTEXT);
        parentPKColumnByContext.put(ContextUtil.RESOURCE_CONTEXT,
                ResourceManager.NAME_RESOURCE_ID); // "RESOURCES.RESOURCE_ID"
        managerToContext.put(ResourceManager.class,
                ContextUtil.RESOURCE_CONTEXT);

        // Asset
        tableNameByContext.put(ContextUtil.ASSET_CONTEXT,
                "TMT_ASSET_CUSTOM_FIELDS");
        tableTypeByContext.put(ContextUtil.ASSET_CONTEXT, new Integer(
                SC_TABLE_AST));
        parentTableTypeByContext.put(ContextUtil.ASSET_CONTEXT, new Integer(
            SC_TABLE_AST));
        parentByContext.put(ContextUtil.ASSET_CONTEXT, Asset.class);
        contextByParent.put(Asset.class, ContextUtil.ASSET_CONTEXT);
        parentPKColumnByContext.put(ContextUtil.ASSET_CONTEXT,
                AssetManager.NAME_ELEMENT_ID); // "TMT_ASSETS.ELEMENT_ID"
        managerToContext.put(AssetManager.class, ContextUtil.ASSET_CONTEXT);

        // Client
        tableNameByContext.put(ContextUtil.CLIENT_CONTEXT,
                "TMT_CLIENT_CUSTOM_FIELDS");
        tableTypeByContext.put(ContextUtil.CLIENT_CONTEXT, new Integer(
                SC_TABLE_CLN));
        parentTableTypeByContext.put(ContextUtil.CLIENT_CONTEXT, new Integer(
            SC_TABLE_CLN));
        parentByContext.put(ContextUtil.CLIENT_CONTEXT, ClientCostCenter.class);
        contextByParent.put(ClientCostCenter.class, ContextUtil.CLIENT_CONTEXT);
        parentPKColumnByContext.put(ContextUtil.CLIENT_CONTEXT,
                ClientManager.NAME_ELEMENT_ID); // "TMT_CLIENTS.ELEMENT_ID"
        managerToContext.put(ClientManager.class, ContextUtil.CLIENT_CONTEXT);
    }

    public static String getTableName(String contextName)
    {
        if (ContextUtil.getInstance().isDocumentContext(contextName))
        {
            contextName = ContextUtil.UMBRELLA_DOCUMENT_CONTEXT;
        }
        return (String) tableNameByContext.get(contextName);
    }

    public static Integer getTableType(String contextName)
    {
        if (ContextUtil.getInstance().isDocumentContext(contextName))
        {
            contextName = ContextUtil.UMBRELLA_DOCUMENT_CONTEXT;
        }
        return (Integer) tableTypeByContext.get(contextName);
    }
    
    public static Integer getParentTableType(String contextName)
    {
        if (ContextUtil.getInstance().isDocumentContext(contextName))
        {
            contextName = ContextUtil.UMBRELLA_DOCUMENT_CONTEXT;
        }
        return (Integer) parentTableTypeByContext.get(contextName);
    }

    public static String getParentPKColumn(String contextName)
    {
        if (ContextUtil.getInstance().isDocumentContext(contextName))
        {
            contextName = ContextUtil.UMBRELLA_DOCUMENT_CONTEXT;
        }
        return (String) parentPKColumnByContext.get(contextName);
    }

    public static String getContextByParent(RPMObject parent)
    {
        for (Iterator i = contextByParent.keySet().iterator(); i.hasNext();)
        {
            Class cls = (Class) i.next();
            if (cls.isInstance(parent))
            {
                return (String) contextByParent.get(cls);
            }
        }

        return null;
    }

    public static RPMObject createParentInstance(String contextName)
            throws RPMException
    {
        if (ContextUtil.getInstance().isDocumentContext(contextName))
        {
            contextName = ContextUtil.UMBRELLA_DOCUMENT_CONTEXT;
        }
        Class parentClass = (Class) parentByContext.get(contextName);
        RPMObject parent = null;
        if (parentClass != null)
        {
            try
            {
                parent = (RPMObject) parentClass.newInstance();
            }
            catch (InstantiationException e)
            {
                throw new RPMException(e);
            }
            catch (IllegalAccessException e)
            {
                throw new RPMException(e);
            }
        }
        return parent;
    }

    public static boolean isParentManager(RPMObjectManager manager)
    {
        return managerToContext.get(manager.getClass()) != null;
    }

    /**
     * Returns an array of CustomFieldAssignments.
     * 
     * @param manager
     *            ResourceManager, WBSManager, etc.
     * @param assignments
     *            actual assignments, needed for isDelta
     * @param parent
     *            Resource object, WorkElement object, etc.
     * @param messageContext
     * @param isDelta
     * @return
     * @throws RPMException
     * @throws ParseException
     * @throws SQLException
     */
    public static CustomFieldAssignment[] loadAssignedCustomFields(
            RPMObjectManager manager, CustomFieldAssignment[] assignments,
            RPMObject parent, MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        Integer tableCode = getTableType((String) managerToContext
            .get(manager.getClass()));

	    if (tableCode == null)
	    {
	        throw new RPMException(
	                "This manager has no association to a CustomField: "
	                        + manager.getClass().getName());
	    }
	    CustomFieldAssignmentManager rmanager = (CustomFieldAssignmentManager) RPMManagerFactory
	            .getInstance().getRPMObjectManager(
	                    CustomFieldAssignment.class.getName());

        ArrayList temp = rmanager.loadByForeignKeyWithSpecifiedParent(
            parent, assignments, null, messageContext, manager,
            new Object[]
            {
                    parent.getID()
            }, null, (String) managerToContext.get(manager.getClass()),
            isDelta);
        
        if(temp != null && !temp.isEmpty()){
            
            //CustomFieldAssignments with null customField are removed to avoid
            // problem when saving the parent
            for (Iterator iter = temp.iterator(); iter.hasNext();)
            {
                CustomFieldAssignment cfa = (CustomFieldAssignment) iter.next();
                if(cfa.getID() != null && cfa.getCustomField() == null){
                    iter.remove();
                }
                
            }
        }
        
        CustomFieldAssignment[] customFieldAssignments = (CustomFieldAssignment[])ArrayUtil.listToArray(CustomFieldAssignment.class, temp);

        return customFieldAssignments;
    }
}
