package com.ibm.rpm.customfield.managers;

import java.lang.reflect.Array;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.CustomDatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.containers.GenericCustomField;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.customfield.types.FieldInputTypeToDBMap;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlUtil;

/**
 * Manager for the TMT_CUSTOM_FIELDS table and related stored procedures.
 * 
 * <h3>Map Object Properties To TMT_CUSTOM_FIELDS Table Fields</h3>
 * <p>
 * <table border=1 cellspacing=0>
 * <tr>
 * <th>Object property</th>
 * <th>Table field</th>
 * <th>Values</th>
 * </tr>
 * <tr>
 * <td>RPMObject.TypeID</td>
 * <td>TYPE_ID</td>
 * <td>1,13,14,228,229:&nbsp;CustomFieldGroup; 64:&nbsp;CustomFieldCategory;
 * 342:&nbsp;CustomField</td>
 * </tr>
 * <tr>
 * <td>RPMObject.ID</td>
 * <td>ELEMENT_ID</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>RPMObject.Parent</td>
 * <td>PARENT_ID</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>GenericCustomField.Name</td>
 * <td>ELEMENT_NAME</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>CustomField.InputType</td>
 * <td>FIELD_TYPE_ID</td>
 * <td>0:&nbsp;Text(String); 1:&nbsp;Currency(Double); 2:&nbsp;Date(Calendar);
 * 3:&nbsp;Numeric(Integer); 4:&nbsp;Datafield(Datafield);
 * 5:&nbsp;Checkbox(Boolean)</td>
 * </tr>
 * <tr>
 * <td>CustomField.DatafieldCategory</td>
 * <td>CATEGORY_ID</td>
 * <td>Foreign key when CATEGORY_ID >= 10000: DATAFIELD.ELEMENT_ID. When
 * CATEGORY_ID < 10000: DATAFIELD.TYPE_ID=? AND DATAFIELD.LEVEL_ID=1.</td>
 * </tr>
 * <tr>
 * <td>CustomField.DefaultValue</td>
 * <td>DEFAULT_VALUE</td>
 * <td>Foreign key when Datafield: DATAFIELD.ELEMENT_ID=?.</td>
 * </tr>
 * <tr>
 * <td>CustomField.SecurityGroup</td>
 * <td>SECURITY_GROUP_ID</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>--</td>
 * <td>LEVEL_ID</td>
 * <td>0:&nbsp;CustomFieldGroup; 1:&nbsp;CustomFieldCategory;
 * 2:&nbsp;CustomField</td>
 * </tr>
 * <tr>
 * <td>--</td>
 * <td>TABLE_TYPE</td>
 * <td>1:&nbsp;WBS&nbsp;Scope; 4:&nbsp;Document; 7:&nbsp;Resource;
 * 9:&nbsp;Asset; 11:&nbsp;Client;</td>
 * </tr>
 * </table>
 * 
 * @author Simon Legault
 */
public class CustomFieldManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        CustomFieldManager.class);

    static
    {
        containerMap.add(GenericCustomField.class);
        containerMap.add(CustomFieldCategory.class);
        containerMap.add(CustomField.class);
    }

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 1;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_CUSTOM_FIELDS.RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 2;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_CUSTOM_FIELDS.PARENT_ID";

    public static final String PROPERTY_PARENT_ID = "PARENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID, NAME_PARENT_ID);
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 3;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_CUSTOM_FIELDS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 4;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_CUSTOM_FIELDS.TYPE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 5;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_CUSTOM_FIELDS.LEVEL_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 6;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_CUSTOM_FIELDS.MAX_RANK";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 7;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_CUSTOM_FIELDS.CHILD_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 8;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_CUSTOM_FIELDS.DELETED_COUNT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 9;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_CUSTOM_FIELDS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_CUSTOM_FIELDS.REC_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 11;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_CUSTOM_FIELDS.REC_USER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column TABLE_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TABLE_TYPE = 12;

    public static final int TYPE_TABLE_TYPE = Types.INTEGER;

    public static final String NAME_TABLE_TYPE = "TMT_CUSTOM_FIELDS.TABLE_TYPE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TABLE_TYPE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TABLE_TYPE,NAME_TABLE_TYPE);}

    /**
     * Column FIELD_TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FIELD_TYPE_ID = 13;

    public static final int TYPE_FIELD_TYPE_ID = Types.INTEGER;

    public static final String NAME_FIELD_TYPE_ID = "TMT_CUSTOM_FIELDS.FIELD_TYPE_ID";

    public static final String PROPERTY_FIELD_TYPE_ID = "INPUTTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FIELD_TYPE_ID, SqlUtil.castAsChar(
            NAME_FIELD_TYPE_ID, 1));
    }

    /**
     * Column CATEGORY_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CATEGORY_ID = 14;

    public static final int TYPE_CATEGORY_ID = Types.INTEGER;

    public static final String NAME_CATEGORY_ID = "TMT_CUSTOM_FIELDS.CATEGORY_ID";

    public static final String PROPERTY_CATEGORY_ID = "DATAFIELDCATEGORY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CATEGORY_ID, NAME_CATEGORY_ID);
    }

    /**
     * Column DEFAULT_VALUE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DEFAULT_VALUE = 15;

    public static final int TYPE_DEFAULT_VALUE = Types.VARCHAR;

    public static final String NAME_DEFAULT_VALUE = "TMT_CUSTOM_FIELDS.DEFAULT_VALUE";

    public static final String PROPERTY_DEFAULT_VALUE = "DEFAULTVALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_VALUE, NAME_DEFAULT_VALUE);
    }

    /**
     * Column SECURITY_GROUP_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SECURITY_GROUP_ID = 16;

    public static final int TYPE_SECURITY_GROUP_ID = Types.INTEGER;

    public static final String NAME_SECURITY_GROUP_ID = "TMT_CUSTOM_FIELDS.SECURITY_GROUP_ID";

    public static final String PROPERTY_SECURITY_GROUP_ID = "SECURITYGROUP";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SECURITY_GROUP_ID, NAME_SECURITY_GROUP_ID);
    }

    public static final String TABLE_NAME = "TMT_CUSTOM_FIELDS";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_CUSTOM_FIELDS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_RANK, NAME_PARENT_ID, NAME_ELEMENT_ID, NAME_TYPE_ID,
            NAME_LEVEL_ID, NAME_MAX_RANK, NAME_CHILD_COUNT, NAME_DELETED_COUNT,
            NAME_ELEMENT_NAME, NAME_REC_STATUS, NAME_REC_USER, NAME_TABLE_TYPE,
            NAME_FIELD_TYPE_ID, NAME_CATEGORY_ID, NAME_DEFAULT_VALUE,
            NAME_SECURITY_GROUP_ID
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new TmtCustomFieldsManager instance.
     * 
     * @return TmtCustomFieldsManager
     */
    public CustomFieldManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public ContainerMap getContainerMap()
    {
        return containerMap;
    }

    // ======
    // MODULE
    // ======

    public CustomFieldCategory[] loadCustomFieldCategories(String contextName,
            CustomFieldScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        String parentId = getGroupId(contextName);
        ArrayList categories = loadByWhere(
            null,
            null,
            scope,
            messageContext,
            NAME_TYPE_ID + "=64 AND " + NAME_PARENT_ID + "='" + parentId + '\'',
            null, TABLE_NAME, ALL_FIELDS, contextName, false);
        CustomFieldCategory[] categoryArray = null;

        if (categories != null)
        {
            categoryArray = new CustomFieldCategory[categories.size()];
            categories.toArray(categoryArray);
        }

        return categoryArray;
    }

    public static String createJoinCondition(String contextName)
            throws RPMException
    {
        // Returns table type condition, if required.
        String joinCondition = null;
        Integer tableType = getTableType(contextName);
        if (tableType != null)
        {
            joinCondition = NAME_TABLE_TYPE + '=' + tableType;
        }
        return joinCondition;
    }

    private String getGroupId(String contextName) throws RPMException
    {
        String groupId = (String) CONTEXT_NAME_TO_GROUP_ID_MAP.get(contextName);
        if (groupId == null)
        {
            String[] errorParams =
            {
                    CustomFieldManager.class.getName(), contextName
            };
            throw new RPMException(ErrorCodes.CONTEXT_INVALID, errorParams);
        }
        return groupId;
    }

    private static Map CONTEXT_NAME_TO_GROUP_ID_MAP;
    static
    {
        CONTEXT_NAME_TO_GROUP_ID_MAP = new HashMap();
        CONTEXT_NAME_TO_GROUP_ID_MAP.put(ContextUtil.ASSET_CONTEXT,
            "SC_ASSET_CUSTOM_FIELD_FOLDER____");
        CONTEXT_NAME_TO_GROUP_ID_MAP.put(ContextUtil.CLIENT_CONTEXT,
            "SC_CLIENT_CUSTOM_FIELD_FOLDER___");
        CONTEXT_NAME_TO_GROUP_ID_MAP.put(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT,
            "SC_DOCUMENT_CUSTOM_FIELD_FOLDER_");
        CONTEXT_NAME_TO_GROUP_ID_MAP.put(ContextUtil.RESOURCE_CONTEXT,
            "SC_RESOURCE_CUSTOM_FIELD_FOLDER_");
        CONTEXT_NAME_TO_GROUP_ID_MAP.put(ContextUtil.SCOPE_CONTEXT,
            "SC_WBS_CRI_CUSTOM_FIELD_FOLDER__");
        CONTEXT_NAME_TO_GROUP_ID_MAP.put(ContextUtil.WBS_CONTEXT,
            "SC_WBS_CRI_CUSTOM_FIELD_FOLDER__");
    }

    private static Integer getTableType(String contextName)
    {
        Integer tableType = (Integer) CONTEXT_NAME_TO_TABLE_TYPE_MAP.get(contextName);
        return tableType;
    }

    private static Map CONTEXT_NAME_TO_TABLE_TYPE_MAP;
    static
    {
        CONTEXT_NAME_TO_TABLE_TYPE_MAP = new HashMap();
        CONTEXT_NAME_TO_TABLE_TYPE_MAP.put(ContextUtil.ASSET_CONTEXT,
            new Integer(9));
        CONTEXT_NAME_TO_TABLE_TYPE_MAP.put(ContextUtil.CLIENT_CONTEXT,
            new Integer(11));
        CONTEXT_NAME_TO_TABLE_TYPE_MAP.put(ContextUtil.UMBRELLA_DOCUMENT_CONTEXT,
            new Integer(4));
        CONTEXT_NAME_TO_TABLE_TYPE_MAP.put(ContextUtil.RESOURCE_CONTEXT,
            new Integer(7));
        CONTEXT_NAME_TO_TABLE_TYPE_MAP.put(ContextUtil.SCOPE_CONTEXT,
            new Integer(1));
        CONTEXT_NAME_TO_TABLE_TYPE_MAP.put(ContextUtil.WBS_CONTEXT,
            new Integer(1));
    }

    // =================
    // CREATE AND UPDATE
    // =================

    /**
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject,
     *      boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        GenericCustomField genCustomField = (GenericCustomField) rpmObject;
        CustomFieldScope scope = (CustomFieldScope) rpmScope;
        String objectId = rpmObject.getID();

        // Update properties, if required.
        if (genCustomField.testNameModified())
        {
            SP_U_CUSTOM_FIELD_NAME(objectId, genCustomField.getName(),
                messageContext);
        }
        if (genCustomField instanceof CustomField)
        {
            CustomField customField = (CustomField) genCustomField;
            if (customField.testInputTypeModified())
            {
                Integer inputTypeCode = FieldInputTypeToDBMap.toDBCode(customField.getInputType());
                SP_U_CUSTOM_FIELD_TYPE(objectId, inputTypeCode, messageContext);
            }
            if (customField.testSecurityGroupModified())
            {
                SP_U_CUSTOM_FIELD_SECURITY(objectId,
                    customField.getSecurityGroup(), messageContext);
            }
            if ((FieldInputType.Datafield.equals(customField.getInputType())))
            {
                Integer categoryId = getCategoryId(customField, messageContext);
                if (customField.testDatafieldCategoryModified()
                        || (categoryId.intValue() != 0 && customField.getDatafieldCategory() == null))
                {
                    SP_U_CUSTOM_FIELD_CATEGORY(objectId, categoryId,
                        messageContext);
                }
            }
            if (customField.testDefaultValueModified())
            {
                String dbString = FieldInputTypeToDBMap.objectToDBString(
                    customField.getInputType(), customField.getDefaultValue());
                SP_U_CUSTOM_FIELD_DEFAULT(objectId, dbString, messageContext);
            }
            else if(customField.testInputTypeModified())
            {
            	// If the Input Type is modified and the default value is not modified,
            	// then the default value will be set to null to have the Input Type and 
            	// the default value to be in Sync.
            	SP_U_CUSTOM_FIELD_DEFAULT(customField.getID(), null, messageContext);
            }
        }

        // Update related objects.
        if (scope != null && genCustomField instanceof CustomFieldCategory)
        {
            CustomFieldCategory customFieldCategory = (CustomFieldCategory) genCustomField;
            CustomField[] children = customFieldCategory.getChildren();
            propagateContextName(customFieldCategory, children);
            customFieldCategory.setChildren((CustomField[]) updateChildren(
                customFieldCategory, children, scope.getChildren(),
                messageContext, null));
        }

        return genCustomField;
    }

    private Integer getCategoryId(CustomField customField,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        // RPM-Client WEIRD design:
        // When the datafield category is custom,
        // the foreign key is the datafield category's ID.
        // Otherwise, it is the datafield category's TYPE ID.
        Integer categoryId = null;
        if (FieldInputType.Datafield.equals(customField.getInputType()))
        {
            DatafieldCategory category = customField.getDatafieldCategory();
            if (category instanceof CustomDatafieldCategory)
            {
                // Use TMT_DATAFIELD.ELEMENT_ID as the foreign key.
                String categoryIdStr = category.getID();
                if (categoryIdStr != null)
                {
                    categoryId = new Integer(categoryIdStr);
                }
            }
            else
            {
                // Use TMT_DATAFIELD.TYPE_ID as the foreign key.
                // Try to get the category's type id.
                if (category != null)
                {
                    categoryId = category.retrieveTypeID();
                }
                // Try to get the default value's type id.
                if (categoryId == null)
                {
                    Datafield defaultValue = (Datafield) customField.getDefaultValue();
                    if (defaultValue != null)
                    {
                        categoryId = defaultValue.retrieveTypeID();
                    }
                }
                // Try to get the category's type id from the database.
                if (categoryId == null && category != null)
                {
                    RPMObjectManager manager = RPMManagerFactory.getInstance().getRPMObjectManager(
                        Datafield.class);
                    category = (DatafieldCategory) manager.loadByPrimaryKey(
                        category, null, messageContext, false);
                    customField.setDatafieldCategory(category);
                    categoryId = category.retrieveTypeID();
                }
            }
        }
        if (categoryId == null)
        {
            categoryId = new Integer(0);
        }
        return categoryId;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;

        switch (typeId)
        {
        case CustomField.TYPE_ID:
            object = new CustomField();
            break;
        case CustomFieldCategory.TYPE_ID:
            object = new CustomFieldCategory();
            break;
        default:
            RPMException e = new RPMException("TypeId " + typeId
                    + " is not supported");
            throw e.log();
        }
        return object;
    }

    /**
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#createObject(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.MessageContext)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        GenericCustomField genCustomField = (GenericCustomField) rpmObject;
        String parentId = getParentId(genCustomField);
        String id = SP_I_CUSTOM_FIELD(parentId, genCustomField.getName(),
            genCustomField.retrieveTypeID(), messageContext);
        rpmObject.setID(id);
    }

    private String getParentId(RPMObject rpmObject) throws RPMException
    {
        String parentId;
        if (rpmObject instanceof CustomFieldCategory)
        {
            parentId = getGroupId(rpmObject.getContextName());
        }
        else
        {
            parentId = rpmObject.getParent().getID();
        }
        return parentId;
    }

    // ====
    // LOAD
    // ====

    /**
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.framework.RPMObject, boolean)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        GenericCustomField genCustomField = (GenericCustomField) rpmObject;

        // Set common fields.
        if (isDelta)
        {
            genCustomField.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
            genCustomField.deltaName(rs.getString(ID_ELEMENT_NAME));
        }
        else
        {
            genCustomField.setID(rs.getString(ID_ELEMENT_ID));
            genCustomField.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
            genCustomField.setName(rs.getString(ID_ELEMENT_NAME));
        }

        // Set fields specific to custom fields.
        if (genCustomField instanceof CustomField)
        {
            CustomField customField = (CustomField) genCustomField;
            int inputTypeCode = rs.getInt(ID_FIELD_TYPE_ID);
            FieldInputType inputType = FieldInputTypeToDBMap.toFieldInputType(inputTypeCode);
            String defaultValueString = rs.getString(ID_DEFAULT_VALUE);

            Object defaultValue = FieldInputTypeToDBMap.dbStringToObject(
                customField, "defaultValue", inputType, defaultValueString,
                null);

            if (isDelta)
            {
                customField.deltaInputType(inputType);
                customField.deltaDefaultValue(defaultValue);
                customField.deltaSecurityGroup(new Integer(
                    rs.getInt(ID_SECURITY_GROUP_ID)));
            }
            else
            {
                customField.setInputType(inputType);
                customField.setDefaultValue(defaultValue);
                customField.setSecurityGroup(new Integer(
                    rs.getInt(ID_SECURITY_GROUP_ID)));
            }
        }
        return genCustomField;
    }

    /**
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        int datafieldCategoryIntId = resultSet.getInt(ID_CATEGORY_ID);
        String datafieldCategoryId = datafieldCategoryIntId == 0 ? null
                : String.valueOf(datafieldCategoryIntId);
        relatedFields.put(index, ID_CATEGORY_ID, datafieldCategoryId);

        int inputType = resultSet.getInt(ID_FIELD_TYPE_ID);
        if (inputType == 4)
        {
            String datafieldDefaultValueId = resultSet.getString(ID_DEFAULT_VALUE);
            relatedFields.put(index, ID_DEFAULT_VALUE, datafieldDefaultValueId);
        }

        return relatedFields;
    }

    /**
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#loadRelatedObjects(com.ibm.rpm.containers.RPMObject,
     *      com.ibm.rpm.scope.RPMObjectScope, com.ibm.rpm.MessageContext,
     *      boolean, com.ibm.rpm.FieldValueMap, int)
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope != null)
        {
            CustomFieldScope scope = (CustomFieldScope) rpmScope;

            // Load parent CustomFieldCategory, if required.
            RPMObject parent = null;
            CustomFieldScope parentScope = (CustomFieldScope) scope.getParent();
            if (parentScope != null && rpmObject instanceof CustomField)
            {
                parent = rpmObject.getParent();
                String parentID = (String) relatedFields.get(index,
                    ID_PARENT_ID);
                if (parentID != null)
                {
                    if (parent == null)
                        parent = new GenericCustomField();
                    parent.setID(parentID);
                    parent = loadByPrimaryKey(parent, parentScope,
                        messageContext, isDelta);
                }
            }
            rpmObject.setParent(parent);

            // Load children, if required.
            if (rpmObject instanceof CustomFieldCategory)
            {
                CustomFieldCategory current = (CustomFieldCategory) rpmObject;
                current.setChildren((CustomField[]) loadChildren(current,
                    current.getChildren(), scope.getChildren(),
                    CustomField.class, messageContext, isDelta));
            }
        }

        // Design note: Datafield category and default value are loaded even
        // with null scope.
        if (rpmObject instanceof CustomField)
        {
            CustomField customField = (CustomField) rpmObject;

            // Load datafield category, if required.
            String datafieldCategoryIdOrTypeId = (String) relatedFields.get(
                index, ID_CATEGORY_ID);
            if (datafieldCategoryIdOrTypeId != null)
            {
                DatafieldCategory datafieldCategory = customField.getDatafieldCategory();
                if (datafieldCategory == null)
                    datafieldCategory = new DatafieldCategory();
                setCategoryIdOrTypeId(datafieldCategory,
                    datafieldCategoryIdOrTypeId);
                RPMObjectManager manager = RPMManagerFactory.getInstance().getRPMObjectManager(
                    Datafield.class);
                datafieldCategory = (DatafieldCategory) manager.loadByPrimaryKey(
                    datafieldCategory, null, messageContext, isDelta);
                customField.setDatafieldCategory(datafieldCategory);

                // Load datafield default value, if required.
                String datafieldDefaultValueID = (String) relatedFields.get(
                    index, ID_DEFAULT_VALUE);
                if (datafieldDefaultValueID != null)
                {
                    Datafield datafieldDefaultValue = (Datafield) customField.getDefaultValue();
                    if (datafieldDefaultValue == null)
                        datafieldDefaultValue = new Datafield();
                    datafieldDefaultValue.setID(datafieldDefaultValueID);
                    datafieldDefaultValue = (Datafield) manager.loadByPrimaryKey(
                        datafieldDefaultValue, null, messageContext, isDelta);
                    customField.setDefaultValue(datafieldDefaultValue);
                }
            }
        }

        return rpmObject;
    }

    private void setCategoryIdOrTypeId(DatafieldCategory datafieldCategory,
            String datafieldCategoryId)
    {
        // RPM-Client WEIRD design:
        // When the datafield category value is a valid custom ID (>= 10000),
        // it is interpreted as the datafield category's ID.
        // Otherwise, it is interpreted as the datafield category's TYPE ID.
        // DatafieldManager is adjusted to allow this flexibility.
        Integer datafieldCategoryTypeId = Integer.valueOf(datafieldCategoryId);
        if (datafieldCategoryTypeId.intValue() < DatafieldManager.MIN_CUSTOM_ID)
        {
            datafieldCategory.assignTypeID(datafieldCategoryTypeId);
        }
        else
        {
            datafieldCategory.setID(datafieldCategoryId);
        }
    }

    private RPMObject[] loadChildren(RPMObject parent, RPMObject[] children,
            RPMObjectScope childrenScope, Class childrenClass,
            MessageContext messageContext, boolean isDelta)
            throws SQLException, RPMException, ParseException
    {
        if (childrenScope == null)
        {
            children = null;
        }
        else
        {
            // This parent ID is used as the foreign key for its
            // children.
            Object[] arguments =
            {
                parent.getID()
            };
            ArrayList childrenList = loadByForeignKey(children, childrenScope,
                messageContext, this, arguments, null, parent.getContextName(),
                isDelta);
            if (childrenList != null)
            {
                int childrenCount = childrenList.size();
                if (childrenCount > 0)
                {
                    children = (RPMObject[]) Array.newInstance(childrenClass,
                        childrenCount);
                    childrenList.toArray(children);
                    for (int i = 0; i < childrenCount; i++)
                    {
                        children[i].setParent(parent);
                    }
                }
            }
        }
        return children;
    }

    public String getForeignKey(RPMObject object)
    {
        if ((object instanceof CustomFieldCategory)
                || (object instanceof CustomField))
            return NAME_PARENT_ID;
        return null;
    }

    public String getFilter()
    {
        return NAME_ELEMENT_ID + " IS NOT NULL";
    }

    /**
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.framework.RPMObjectManager,
     *      java.lang.String, java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(
            RPMObjectManager callingManager, String extraWhere,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (callingManager instanceof CustomFieldManager)
        {
            // Design note: used when loading children from CustomFieldCategory.
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(NAME_PARENT_ID + "=?");
        }
        else if (CustomFieldAssignmentUtil.isParentManager(callingManager))
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();

            where.append(NAME_TYPE_ID);
            where.append(" = ");
            where.append(CustomField.TYPE_ID);
            where.append(" AND ");
            where.append(NAME_TABLE_TYPE);
            where.append(" = ? ");

            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }

            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, callingManager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }
    
    // ======
    // DELETE
    // ======

    /**
     * Deletes the specified <code>rpmObject</code> and all its descendants.
     * 
     * @see com.ibm.rpm.framework.RPMObjectManager#doDelete(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.MessageContext)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        String parentId = ManagerUtil.getParentId(this, rpmObject,
            messageContext, TABLE_NAME);
        SP_D_CUSTOM_FIELD(rpmObject.getID(), parentId, messageContext);
    }

    // ======================
    // STORED PROCEDURE CALLS
    // ======================

    /**
     * Creates a CustomField or a CustomFieldCategory. Initializes Name, Parent
     * and TypeID.
     * 
     * @return ID of the new chargecode.
     */
    private String SP_I_CUSTOM_FIELD(String parentId, String name,
            Integer typeId, MessageContext messageContext) throws SQLException,
            RPMException
    {
        Object args[] = new Object[4];
        args[0] = name;
        args[1] = parentId;
        args[2] = typeId;
        args[3] = getUser(messageContext).getID();

        String elementId = executeProcedure(messageContext,
            "SP_I_CUSTOM_FIELD", args);

        return elementId;
    }

    private void SP_U_CUSTOM_FIELD_NAME(String objectId, String name,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[3];
        args[0] = objectId;
        args[1] = name;
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, "SP_U_CUSTOM_FIELD_NAME", args);
    }

    private void SP_U_CUSTOM_FIELD_TYPE(String objectId, Integer inputTypeCode,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[3];
        args[0] = objectId;
        args[1] = inputTypeCode;
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, "SP_U_CUSTOM_FIELD_TYPE", args);
    }

    private void SP_U_CUSTOM_FIELD_SECURITY(String objectId,
            Integer securityGroup, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Object args[] = new Object[3];
        args[0] = objectId;
        args[1] = securityGroup;
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, "SP_U_CUSTOM_FIELD_SECURITY", args);
    }

    private void SP_U_CUSTOM_FIELD_CATEGORY(String objectId,
            Integer datafieldCategoryId, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Object args[] = new Object[3];
        args[0] = objectId;
        args[1] = datafieldCategoryId;
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, "SP_U_CUSTOM_FIELD_CATEGORY", args);
    }

    private void SP_U_CUSTOM_FIELD_DEFAULT(String objectId,
            String defaultValue, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Object args[] = new Object[3];
        args[0] = objectId;
        args[1] = defaultValue;
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, "SP_U_CUSTOM_FIELD_DEFAULT", args);
    }

    /**
     * Creates a CustomField or a CustomFieldCategory. Initialises Name, Parent
     * and TypeID.
     * 
     * @return ID of the new chargecode.
     */
    private void SP_D_CUSTOM_FIELD(String objectId, String parentId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[4];
        args[0] = objectId;
        args[1] = parentId;
        args[2] = "X";
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, "SP_D_CUSTOM_FIELD", args);
    }
}
