/*
 * Created on 15-Nov-2005
 */
package com.ibm.rpm.document.checkpoints;

import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

/**
 * @author dominiqu
 */
public class AbstractDocumentCheckpoint extends GenericDocumentCheckpoint
{
    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx);

        // Initialize
        AbstractDocument mainAbstractDocument = (AbstractDocument) rpmObject;

        // Field validation: keywords
        if (mainAbstractDocument.testKeywordsModified())
        {
            GenericValidator.validateMaxLength(mainAbstractDocument,
                "keywords", mainAbstractDocument.getKeywords(),
                ValidationConstants.ABSTRACTDOCUMENT_KEYWORDS_MAX, msgCtx);
        }

        // Field validation: referenceNumber
        if (mainAbstractDocument.testReferenceNumberModified())
        {
            GenericValidator.validateMaxLength(mainAbstractDocument,
                "referenceNumber", mainAbstractDocument.getReferenceNumber(),
                ValidationConstants.ABSTRACTDOCUMENT_REFERENCENUMBER_MAX, msgCtx);
        }

        // Field validation: summary
        if (mainAbstractDocument.testSummaryModified())
        {
            GenericValidator.validateMaxLength(mainAbstractDocument,
                "summary", mainAbstractDocument.getSummary(),
                ValidationConstants.ABSTRACTDOCUMENT_SUMMARY_MAX, msgCtx);
        }
        
        if(mainAbstractDocument.testMajorRevisionNumberModified())
        {
        	GenericValidator.validateReadOnlyError(mainAbstractDocument,
                    ValidationConstants.MAJOR_REVISION_NUMBER_FIELD, msgCtx);
        }
        
        if(mainAbstractDocument.testMinorRevisionNumberModified())
        {
        	GenericValidator.validateReadOnlyError(mainAbstractDocument,
                    ValidationConstants.MINOR_REVISION_NUMBER_FIELD, msgCtx);
        }
    }
    
}