/*
 * Created on 15-Nov-2005
 */
package com.ibm.rpm.document.checkpoints;

import java.util.Iterator;
import java.util.List;

import com.ibm.rpm.document.constants.ErrorCodes;
import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;

/**
 * @author dominiqu
 */
public class DocumentElementCheckpoint extends AbstractCheckpoint
{
    protected void validateSave(RPMObject rpmObject, MessageContext msgCtx, 
            boolean parentRequired, List validParentTypes)
    {
        super.isValidParent(rpmObject, msgCtx, parentRequired, validParentTypes);

        // Initialize
        DocumentElement mainDocumentElement = (DocumentElement) rpmObject;

        // Field validation: name
        if (mainDocumentElement.testNameModified())
        {
            GenericValidator.validateMaxLength(mainDocumentElement,
                "name", mainDocumentElement.getName(),
                ValidationConstants.DOCUMENTELEMENT_NAME_MAX, msgCtx);
        }
    }
    
    public static boolean validateDocumentElements(RPMObject rootParent, DocumentElement[] documentElements,
            List validDocumentTypes, MessageContext msgCtx)
    {
        if (!isValidDocumentElements(documentElements,
                validDocumentTypes))
        {
            String rootParentClassName = StringUtil.getShortClassName(rootParent.getClass());
            String[] errorParams =
            {
                    rootParentClassName, rootParentClassName,
                    getStringValidDocumentTypesList(validDocumentTypes)
            };
            RPMException e = new RPMException(
                    ErrorCodes.WRONG_DOCUMENT_ELEMENT_TYPE, errorParams);
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    private static boolean isValidDocumentElements(
            DocumentElement[] documentElements, List validDocumentTypes)
    {

        boolean isValid = true;

        if (documentElements == null || documentElements.length==0)
        {
            return true;
        }

        if (validDocumentTypes != null)
        {
            for(int i=0;i<documentElements.length && isValid;i++){
                isValid = isValid && isValidDocumentElement(documentElements[i],validDocumentTypes);
            }
        }

        return isValid;
    }

    private static boolean isValidDocumentElement(
            DocumentElement documentElement, List validDocumentTypes)
    {

        boolean isValid = false;

        if (documentElement == null)
        {
            return true;
        }

        if (validDocumentTypes != null)
        {

            Class documentType = null;
            Iterator iterator = validDocumentTypes.iterator();

            while (iterator.hasNext() && !isValid)
            {

                documentType = (Class) iterator.next();

                if (documentType.isInstance(documentElement))
                {
                    isValid = true;
                }

            }
        }

        DocumentElement[] subDocuments = documentElement.getChildren();
        if (isValid)
        {
            //validate children
            isValid = isValid && isValidDocumentElements(subDocuments, validDocumentTypes);
        }

        return isValid;
    }
    
    private static String getStringValidDocumentTypesList(
            List validDocumentTypes)
    {
        StringBuffer sb = new StringBuffer();
        if (validDocumentTypes != null)
        {

            Iterator iterator = validDocumentTypes.iterator();

            Class documentType = null;
            while (iterator.hasNext())
            {
                documentType = (Class) iterator.next();
                sb.append(StringUtil.getShortClassName(documentType));

                if (iterator.hasNext())
                {
                    sb.append(", ");
                }
            }
        }

        return sb.toString();
    }
    
    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        
    }
}
