/*
 * Created on 16-Nov-2005
 */
package com.ibm.rpm.document.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.document.constants.ErrorCodes;
import com.ibm.rpm.document.constants.ValidationConstants;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.document.types.DocumentStatusType;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.util.GenericValidator;

/**
 * @author dominiqu
 */
public class GenericDocumentCheckpoint extends DocumentElementCheckpoint
{
    private static final List GENERICDOCUMENT_PARENTS = new ArrayList();

    static
    {
        GENERICDOCUMENT_PARENTS.add(DocumentFolder.class);
        GENERICDOCUMENT_PARENTS.add(DelDoc.class);
        GENERICDOCUMENT_PARENTS.add(Document.class);
        GENERICDOCUMENT_PARENTS.add(NoteMinutes.class);
        GENERICDOCUMENT_PARENTS.add(Software.class);
        GENERICDOCUMENT_PARENTS.add(UrlDocument.class);
        GENERICDOCUMENT_PARENTS.add(WPDoc.class);
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, msgCtx, true, GENERICDOCUMENT_PARENTS);
     
        //ElementNumber value is ReadOnly
        if(((GenericDocument)rpmObject).testElementNumberModified())
        {
        	GenericValidator.validateReadOnlyError(rpmObject, ValidationConstants.ELEMENT_NUMBER_FIELD, msgCtx);
        }
        
        //lockedValue is ReadOnly
        if (((GenericDocument)rpmObject).testLockedModified())
        {
            GenericValidator.validateReadOnlyError(rpmObject,
                ValidationConstants.LOCKED_FIELD, msgCtx);
        }
    }
    
    public static void validateDocumentLockAndUnlock(GenericDocument document, boolean isLock, MessageContext msgCtx)
    {
    	 GenericValidator.validateMandatoryID("GenericDocument", document,
    			 msgCtx);
    	 
    	 if(isLock)
    	 {
    		 if(document.getLocked().booleanValue())
    		 {
    			 RPMException e = new RPMException(
    					 ErrorCodes.EXCEPTION_DOC_ALREADY_LOCKED, document
    					 .getClass().getName());
    			 addException(e, msgCtx);
    		 }
    		 else if(document.getDocumentStatus().equals(DocumentStatusType.CheckedOut))
    		 {
    			 RPMException e = new RPMException(
    					 ErrorCodes.EXCEPTION_LOCKING_CHECKEDOUT_DOC, document
    					 .getClass().getName());
    			 addException(e, msgCtx);
    		 }
    	 }
    	 else if(!document.getLocked().booleanValue())
    	 {
    		 RPMException e = new RPMException(
    				 ErrorCodes.EXCEPTION_DOC_ALREADY_UNLOCKED, document
    				 .getClass().getName());
    		 addException(e, msgCtx);
    	 }
    }
    
    public static boolean canLockUnlock(GenericDocument document,
			boolean isLock, MessageContext msgCtx) {
		boolean canLockUnlock = true;

		if (isLock) {
			try {
				canLockUnlock = (ISecurityController.INSTANCE.canLockDocument(
						msgCtx, document).type != SecurityValidationResult.FALSE);
			} catch (RPMException e) {
				addException(SecurityValidationResult.makeException(e), msgCtx);
				canLockUnlock = false;
			}
		} else {
			try {
				canLockUnlock = (ISecurityController.INSTANCE
						.canUnlockDocument(msgCtx, document).type != SecurityValidationResult.FALSE);
			} catch (RPMException e) {
				addException(SecurityValidationResult.makeException(e), msgCtx);
				canLockUnlock = false;
			}
		}
		return canLockUnlock;
	}
}
