/*
 * Created on Oct 25, 2005
 *
 */
package com.ibm.rpm.document.managers;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.DocumentState;
import com.ibm.rpm.applicationadministration.containers.NoteMinutesRTF;
import com.ibm.rpm.applicationadministration.containers.NoteState;
import com.ibm.rpm.applicationadministration.managers.AttributeManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.managers.CustomFieldManager;
import com.ibm.rpm.document.containers.DocumentModule;
import com.ibm.rpm.document.scope.DocumentModuleScope;
import com.ibm.rpm.framework.AbstractModuleManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;

/**
 * @author karim
 *  
 */
public class DocumentModuleManager extends AbstractModuleManager
{
    private static Log logger = LogFactory.getLog(DocumentModuleManager.class);

    protected Log getLogger()
    {
        return logger;
    }

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(DocumentModule.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    DocumentModuleManager.class);
        }
    }

    {
        localContextName = ContextUtil.UMBRELLA_DOCUMENT_CONTEXT;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = new DocumentModule();
        return object;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof AttributeManager)
        {
            if (localContextName.equalsIgnoreCase(contextName))
            {
                joinCondition = new JoinCondition();
            }
            else
            {//context not linked to this manager
                logger
                        .error("DocumentModuleManager.getJoinCondition: contextName "
                                + contextName
                                + " is not linked with this manager.");
            }
        }
        else if (manager instanceof CustomFieldManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setCondition(CustomFieldManager
                    .createJoinCondition(localContextName));
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected void loadChildren(AbstractModule abstractModule,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        DocumentModule module = (DocumentModule) abstractModule;
        DocumentModuleScope scope = (DocumentModuleScope) rpmScope;
        
    	DatafieldManager datafieldManager = (DatafieldManager)getManagerInstance(DatafieldCategory.class);


        //Attributes
        if (scope.getAttributeCategories() != null)
        {
            AttributeManager attributeManager = (AttributeManager) getManagerInstance(AttributeCategory.class);

            AttributeCategory[] attributeCategory = attributeManager
                .loadAttributeCategories(abstractModule, module.getAttributeCategories(), scope
                    .getAttributeCategories(), this, localContextName,
                    messageContext, isDelta);
            module.setAttributeCategories(attributeCategory);
        }

        //DocumentStates
        if(scope.isDocumentStates())
        {        	
        	DocumentState[] documentStates = (DocumentState[]) datafieldManager
                .loadDatafields(module, module.getDocumentStates(), DocumentState.class,
                    DocumentState.TYPE_ID, this, localContextName,
                    messageContext, isDelta);
            module.setDocumentStates(documentStates);
        }

        //NoteStates
        if(scope.isNoteStates())
        {        	
        	NoteState[] noteStates = (NoteState[]) datafieldManager
                .loadDatafields(module, module.getNoteStates(), NoteState.class,
                    NoteState.TYPE_ID, this, localContextName,
                    messageContext, isDelta);
            module.setNoteStates(noteStates);
        }

        //NoteMinutesRTFs
        if(scope.isNoteMinutesRtfs())
        {        	
        	NoteMinutesRTF[] noteMinutesRTFs = (NoteMinutesRTF[]) datafieldManager
                .loadDatafields(module, module.getNoteMinutesRtfs(),
                    NoteMinutesRTF.class, NoteMinutesRTF.TYPE_ID,
                    this, localContextName, messageContext, isDelta);
            module.setNoteMinutesRtfs(noteMinutesRTFs);
        }
        
        //CustomFields
        if (scope.getCustomFieldCategories() != null)
        {
            CustomFieldManager manager = (CustomFieldManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            CustomFieldCategory.class);
            CustomFieldCategory[] categories = manager
                    .loadCustomFieldCategories(localContextName,
                            scope.getCustomFieldCategories(), messageContext);
            module.setCustomFieldCategories(categories);
        }

    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        DocumentModuleScope scope = (DocumentModuleScope) rpmScope;
        DocumentModule module = (DocumentModule) rpmObject;

        if(scope!=null){
            //AttributeCategories
            if (scope.getAttributeCategories() != null)
            {
                AttributeCategory[] attributeCategory = (AttributeCategory[]) saveElements(
                    module, module.getAttributeCategories(), scope.getAttributeCategories(), messageContext,
                    AttributeCategory.class);
                module.setAttributeCategories(attributeCategory);
            }

            //CustomFields
            if (scope.getCustomFieldCategories() != null)
            {
                propagateContextName(module, module.getCustomFieldCategories());
                module
                        .setCustomFieldCategories((CustomFieldCategory[]) updateChildren(
                                null, module.getCustomFieldCategories(),
                                scope.getCustomFieldCategories(), messageContext, null));
            }
        }

        return module;
    }
}
