package com.ibm.rpm.document.transfer;

import java.net.HttpURLConnection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.interfaces.impl.ServerFactory;

public class DocumentTransferServlet extends AbstractDocTransferServlet
{
    protected static Log logger = LogFactory
        .getLog(DocumentTransferServlet.class);

    public DocumentTransferServlet()
    {
        this(ServerFactory.getInstance());
    }

    public DocumentTransferServlet(ServerFactory setfactory)
    {
        super(setfactory);
    }


    protected void internalExecuteRequest(HttpServletRequest request,
            HttpServletResponse response) throws Exception
    {

        // ---- getting parameters
        String action = null;
        String contextName = null;
        String docID = null;
        String sessionID = null;
        String filename = null;

        if (request.getMethod().equalsIgnoreCase("GET"))
        {
            // The get method have the variables in the URL of the http
            // request.
            action = request.getParameter(DocumentTransfer.ACTION);
            if (action != null)
            {
                action = action.trim();
            }
            contextName = request
                .getParameter(DocumentTransfer.HEADER_DOCUMENT_CONTEXT_NAME);
            docID = request.getParameter(DocumentTransfer.HEADER_DOCUMENT_ID);
            sessionID = request
                .getParameter(DocumentTransfer.HEADER_SESSION_ID);
            filename = request.getParameter(DocumentTransfer.HEADER_FILE);
        }
        else
        // POST
        {
            // The put and post method have the variables in the header
            // section of the request.
            action = request.getHeader(DocumentTransfer.ACTION);
            if (action != null)
            {
                action = action.trim();
            }
            contextName = request
                .getHeader(DocumentTransfer.HEADER_DOCUMENT_CONTEXT_NAME);
            docID = request.getHeader(DocumentTransfer.HEADER_DOCUMENT_ID);
            sessionID = request.getHeader(DocumentTransfer.HEADER_SESSION_ID);
            filename = request.getHeader(DocumentTransfer.HEADER_FILE);
        }
        
        // ============================== DEFECT 68517 ======================================
        // If the filename contains the absolute path truncate it to just the name of the file
        if(filename != null && filename.length() > 0)
        {
        	int index = -1;
        	index = filename.lastIndexOf('\\');
        	if(index == -1)
        	{
        		index = filename.lastIndexOf('/');
        	}
        	if(index != -1)
        	{
        		filename = filename.substring(index+1);
        	}
        }
        //=====================================================================================
        
        logger.info("Incomming document transfer request. Action = " + action
                + ", context name = " + contextName + ", session ID = "
                + sessionID + ", file name = " + filename + ", document ID = "
                + docID + ", request method = " + request.getMethod());

        // ------ checking for valid parameters
        if (action == null || sessionID == null || docID == null
                || contextName == null)
        {
            sendResponseError(
                response,
                HttpURLConnection.HTTP_BAD_REQUEST,
                "Request must contain DOCUMENT_ID, CONTEXT_NAME, ACTION, SESSION_ID and CONTEXT_NAME parameters");

            return;
        }

        if ((sessionID == null)
                || (!DocumentTransfer.isSessionIdValid(factory
                    .getManagerCaller(), sessionID)))
        {
            sendResponseError(response, HttpURLConnection.HTTP_UNAUTHORIZED,
                "Invalid Session ID : " + sessionID);
            return;
        }

        boolean found = false;
        for (int i = 0; (!found && i<VALID_CONTEXTS.length); i++)
        {
            if (VALID_CONTEXTS[i].equalsIgnoreCase(contextName))
            {
                found = true;
            }
        }

        if (!found)
        {
            sendResponseError(response, HttpURLConnection.HTTP_BAD_REQUEST,
                "Unknown context name : " + contextName);
            return;
        }

        if (DocumentTransfer.ACTION_DOWNLOADZIP.equalsIgnoreCase(action))
        {
            download(request, response, DocumentTransfer.ACTION_DOWNLOADZIP,
                contextName, docID, sessionID, true);
            return;
        }
        else if (DocumentTransfer.ACTION_DOWNLOADUNZIP.equalsIgnoreCase(action))
        {
            download(request, response, DocumentTransfer.ACTION_DOWNLOADUNZIP,
                contextName, docID, sessionID, false);
            return;
        }
        else if (DocumentTransfer.ACTION_UPLOADZIP.equalsIgnoreCase(action))
        {
            if (!validateFileName(response, filename))
            {
                return;
            }

            upload(request, response, request.getInputStream(),
                DocumentTransfer.ACTION_UPLOADZIP, contextName, docID,
                sessionID, filename);

            return;
        }
        else if (DocumentTransfer.ACTION_UPLOADUNZIP.equalsIgnoreCase(action))
        {
            if (!validateFileName(response, filename))
            {
                return;
            }

            upload(request, response, request.getInputStream(),
                DocumentTransfer.ACTION_UPLOADUNZIP, contextName, docID,
                sessionID, filename);

            return;
        }
        else
        {
            sendResponseError(response, HttpURLConnection.HTTP_BAD_REQUEST,
                "Unrecognized action: " + action);
            return;
        }

    }
}