/*
 * Created on Dec 20, 2006
 * @author Christian Gosselin
 */
package com.ibm.rpm.document.transfer;

import java.io.InputStream;
import java.net.HttpURLConnection;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.interfaces.impl.ServerFactory;

public class MultipartDocumentTransferServlet extends AbstractDocTransferServlet
{
    protected static Log logger = LogFactory
        .getLog(MultipartDocumentTransferServlet.class);
    
    public MultipartDocumentTransferServlet()
    {
        super();
    }

    public MultipartDocumentTransferServlet(ServerFactory setfactory)
    {
        super(setfactory);
    }

    protected void internalExecuteRequest(HttpServletRequest request,
            HttpServletResponse response) throws Exception
    {

        // ---- getting parameters
        String action = null;
        String contextName = null;
        String docID = null;
        String sessionID = null;
        String filename = null;
        List items = null;
        if (request.getMethod().equalsIgnoreCase("GET"))
        {
            // The get method have the variables in the URL of the http
            // request.
            action = request.getParameter(DocumentTransfer.ACTION);
            if (action != null)
            {
                action = action.trim();
            }
            contextName = request
                .getParameter(DocumentTransfer.HEADER_DOCUMENT_CONTEXT_NAME);
            docID = request.getParameter(DocumentTransfer.HEADER_DOCUMENT_ID);
            sessionID = request
                .getParameter(DocumentTransfer.HEADER_SESSION_ID);
            filename = request.getParameter(DocumentTransfer.HEADER_FILE);
        }
        else
        {
            // Post
            FileItemFactory factory = new DiskFileItemFactory();
            ServletFileUpload upload = new ServletFileUpload(factory);
            items = upload.parseRequest(request);
            Iterator iter = items.iterator();
            logger.debug("Incomming document transfer request ");
            while (iter.hasNext())
            {
                FileItem fi = (FileItem) iter.next();
                logger.debug("Name: " + fi.getName() + " fieldName "
                        + fi.getFieldName() + " isForm: " + fi.isFormField()
                        + " Content type: " + fi.getContentType() + " String: "
                        + fi.getString());
                if (fi.getFieldName().equals(DocumentTransfer.ACTION))
                {
                    action = fi.getString();
                    if (action != null)
                    {
                        action = action.trim();
                    }
                }
                else if (fi.getFieldName().equals(
                    DocumentTransfer.HEADER_DOCUMENT_CONTEXT_NAME))
                {
                    contextName = fi.getString();
                }
                else if (fi.getFieldName().equals(
                    DocumentTransfer.HEADER_DOCUMENT_ID))
                {
                    docID = fi.getString();
                }
                else if (fi.getFieldName().equals(
                    DocumentTransfer.HEADER_SESSION_ID))
                {
                    sessionID = fi.getString();
                }
                else if (fi.getFieldName().equals(DocumentTransfer.HEADER_FILE))
                {
                    filename = fi.getName();
                }
            }
        }
        
        //============================== DEFECT 68517 ======================================
        // If the filename contains the absolute path truncate it to just the name of the file
        if(filename != null && filename.length() > 0)
        {
        	int index = -1;
        	index = filename.lastIndexOf('\\');
        	if(index == -1)
        	{
        		index = filename.lastIndexOf('/');
        	}
        	if(index != -1)
        	{
        		filename = filename.substring(index+1);
        	}
        }
        //=====================================================================================

        logger.info("Incomming document transfer request. Action = " + action
                + ", context name = " + contextName + ", session ID = "
                + sessionID + ", file name = " + filename + ", document ID = "
                + docID + ", request method = " + request.getMethod());

        // ------ checking for valid parameters
        if (action == null || sessionID == null || docID == null
                || contextName == null)
        {
            sendResponseError(
                response,
                HttpURLConnection.HTTP_BAD_REQUEST,
                "Request must contain DOCUMENT_ID, CONTEXT_NAME, ACTION, SESSION_ID and CONTEXT_NAME parameters");

            return;
        }

        if ((sessionID == null)
                || (!DocumentTransfer.isSessionIdValid(factory
                    .getManagerCaller(), sessionID)))
        {
            sendResponseError(response, HttpURLConnection.HTTP_UNAUTHORIZED,
                "Invalid Session ID : " + sessionID);
            return;
        }

        boolean found = false;
        for (int i = 0; i < VALID_CONTEXTS.length; i++)
        {
            if (VALID_CONTEXTS[i].equalsIgnoreCase(contextName))
            {
                found = true;
            }
        }

        if (!found)
        {
            sendResponseError(response, HttpURLConnection.HTTP_BAD_REQUEST,
                "Unknown context name : " + contextName);
            return;
        }

        if (DocumentTransfer.ACTION_DOWNLOADZIP.equalsIgnoreCase(action))
        {
            download(request, response, DocumentTransfer.ACTION_DOWNLOADZIP,
                contextName, docID, sessionID, true);
            return;
        }
        else if (DocumentTransfer.ACTION_DOWNLOADUNZIP.equalsIgnoreCase(action))
        {
            download(request, response, DocumentTransfer.ACTION_DOWNLOADUNZIP,
                contextName, docID, sessionID, false);
            return;
        }
        else if (DocumentTransfer.ACTION_UPLOADZIP.equalsIgnoreCase(action))
        {
            if (!validateFileName(response, filename))
            {
                return;
            }

            InputStream is = getRequestFileInputStream(items);
            if (is != null)
            {
                upload(request, response, is,
                    DocumentTransfer.ACTION_UPLOADZIP, contextName, docID,
                    sessionID, filename);
            }
            else
            {
                sendResponseError(response,
                    HttpURLConnection.HTTP_INTERNAL_ERROR,
                    "No document attached");
            }
            return;
        }
        else if (DocumentTransfer.ACTION_UPLOADUNZIP.equalsIgnoreCase(action))
        {
            if (!validateFileName(response, filename))
            {
                return;
            }

            InputStream is = getRequestFileInputStream(items);
            if (is != null)
            {
                upload(request, response, is,
                    DocumentTransfer.ACTION_UPLOADUNZIP, contextName, docID,
                    sessionID, filename);
            }
            else
            {
                sendResponseError(response,
                    HttpURLConnection.HTTP_INTERNAL_ERROR,
                    "No document attached");
            }
            return;
        }
        else
        {
            sendResponseError(response, HttpURLConnection.HTTP_BAD_REQUEST,
                "Unrecognized action: " + action);
            return;
        }

    }

    /**
     * This method returns an InputStream on the first file found in the HTTP
     * request
     * 
     * @param request
     * @return
     * @throws Exception
     */
    protected InputStream getRequestFileInputStream(List items)
            throws Exception
    {
        Iterator iter = items.iterator();
        while (iter.hasNext())
        {
            FileItem fi = (FileItem) iter.next();
            if (!fi.isFormField())
            {
                InputStream is = fi.getInputStream();
                return is;
            }
        }
        return null;
    }
}
