/**
 * Created on Feb 3, 2006
 * @author shahla
 */
package com.ibm.rpm.financial.checkpoints;

import com.ibm.rpm.financial.constants.ValidationConstants;
import com.ibm.rpm.financial.containers.AbstractTimeCode;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

public abstract class AbstractTimeCodeCheckpoint extends AbstractCheckpoint
{
    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        if (rpmObject instanceof AbstractTimeCode)
        {
            // Validate nesting level - not yet tested thus commented
            //GenericValidator.validateNestingLevel(rpmObject, "TMT_TIMECODES",
            //    ValidationConstants.TIME_CODE_NESTINGLEVEL_MAX, msgCtx);
            
            // validate the common fields
            AbstractTimeCode timeCode = (AbstractTimeCode) rpmObject;
            validateName(timeCode, msgCtx);
            validateDescription(timeCode, msgCtx);
        }
    }
    
    private void validateName(AbstractTimeCode timeCode, MessageContext msgCtx)
    {
        boolean valid = GenericValidator.validateMandatory(timeCode,
            ValidationConstants.NAME_FIELD, timeCode.getName(), msgCtx);
        if (valid && timeCode.testNameModified())
        {
            GenericValidator.validateMaxLength(timeCode,
                ValidationConstants.NAME_FIELD, timeCode.getName(),
                ValidationConstants.TIME_CODE_NAME_MAX, msgCtx);
            GenericValidator.validateMinLength(timeCode,
                ValidationConstants.NAME_FIELD, timeCode.getName(),
                ValidationConstants.TIME_CODE_NAME_MIN, msgCtx);
        }
    }
    
    private void validateDescription(AbstractTimeCode timeCode,
            MessageContext msgCtx)
    {
        if (timeCode.testDescriptionModified())
        {
            GenericValidator.validateMaxLength(timeCode,
                ValidationConstants.DESCRIPTION_FIELD, timeCode
                    .getDescription(),
                ValidationConstants.TIME_CODE_DESCRIPTION_MAX, msgCtx);
        }
    }
}
