package com.ibm.rpm.financial.managers;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.ChargeCodeState;
import com.ibm.rpm.financial.checkpoints.GenericChargeCodeCheckpoint;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.GenericChargeCode;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.financial.util.FinancialCategoryUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.wbs.managers.TaskAssignmentManager;
import com.ibm.rpm.xpathparser.XPathContainer;

/**
 * Manager for the TMT_CHARGECODE table and related stored procedures.
 * 
 * <h3>Map ChargeCode Properties To TMT_CHARGECODE Table</h3>
 * <p>
 * <table border=1 cellspacing=0>
 * <tr>
 * <th>Object property</th>
 * <th>Table field</th>
 * <th>Values</th>
 * </tr>
 * <tr>
 * <td>ID</td>
 * <td>ELEMENT_ID</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>TypeID</td>
 * <td>TYPE_ID</td>
 * <td>8</td>
 * </tr>
 * <tr>
 * <td>Name</td>
 * <td>ELEMENT_NAME</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>Description</td>
 * <td>DESCRIPTION</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>ExternalID</td>
 * <td>EXTERNAL_IDENTIFIER</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>Parent</td>
 * <td>PARENT_ID</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>AssociatedChargeCode</td>
 * <td>ASSOCIATE_CHARGECODE_ID</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>FinancialCategory</td>
 * <td>CAPITAL_EXPENSE</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>State</td>
 * <td>STAGE_ID</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>Billable</td>
 * <td>BILLABLE</td>
 * <td>&nbsp;</td>
 * </tr>
 * </table>
 * <p>
 * <h3>Map ChargeCodeCategory Properties To TMT_CHARGECODE Table</h3>
 * <p>
 * <table border=1 cellspacing=0>
 * <tr>
 * <th>Object property</th>
 * <th>Table field</th>
 * <th>Values</th>
 * </tr>
 * <tr>
 * <td>ID</td>
 * <td>ELEMENT_ID</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>TypeID</td>
 * <td>TYPE_ID</td>
 * <td>64</td>
 * </tr>
 * <tr>
 * <td>Name</td>
 * <td>ELEMENT_NAME</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>Description</td>
 * <td>DESCRIPTION</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>ExternalID</td>
 * <td>EXTERNAL_IDENTIFIER</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>Parent</td>
 * <td>PARENT_ID</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>Children</td>
 * <td>PARENT_ID</td>
 * <td>Reverse relation</td>
 * </tr>
 * </table>
 * 
 * @author Simon Legault
 */
public class ChargeCodeManager extends AbstractRPMObjectManager
{
    private static Log logger = LogFactory.getLog(ChargeCodeManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        ChargeCodeManager.class);
    
    static
    {
        containerMap.add(GenericChargeCode.class);
        containerMap.add(ChargeCode.class);
        containerMap.add(ChargeCodeCategory.class);
    }

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. public static
     * final String PROPERTY_LEVEL_ID = "PROPERTYNAME"; static{
     * FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}
     */
    public static final int ID_LEVEL_ID = 1;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_CHARGECODES.LEVEL_ID";

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_CHARGECODES.TYPE_ID";

    public static final String PROPERTY_TYPE_ID = "TYPE_ID";

    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID, NAME_TYPE_ID);
    }

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case. public static final
     * String PROPERTY_MAX_RANK = "PROPERTYNAME"; static{
     * FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}
     */
    public static final int ID_MAX_RANK = 3;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_CHARGECODES.MAX_RANK";

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. public static
     * final String PROPERTY_CHILD_COUNT = "PROPERTYNAME"; static{
     * FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}
     */
    public static final int ID_CHILD_COUNT = 4;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_CHARGECODES.CHILD_COUNT";

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. public static
     * final String PROPERTY_DELETED_COUNT = "PROPERTYNAME"; static{
     * FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}
     */
    public static final int ID_DELETED_COUNT = 5;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_CHARGECODES.DELETED_COUNT";

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_CHARGECODES.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;

    public static final int TYPE_ELEMENT_NAME = Types.CHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_CHARGECODES.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_CHARGECODES.PARENT_ID";

    public static final String PROPERTY_PARENT_ID = "PARENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID, NAME_PARENT_ID);
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. public static
     * final String PROPERTY_REC_USER = "PROPERTYNAME"; static{
     * FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}
     */
    public static final int ID_REC_USER = 9;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_CHARGECODES.REC_USER";

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. public static
     * final String PROPERTY_REC_STATUS = "PROPERTYNAME"; static{
     * FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_CHARGECODES.REC_STATUS";

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. public static
     * final String PROPERTY_REC_DATETIME = "PROPERTYNAME"; static{
     * FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}
     */
    public static final int ID_REC_DATETIME = 11;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "TMT_CHARGECODES.REC_DATETIME";

    /**
     * Column BILLABLE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BILLABLE = 12;

    public static final int TYPE_BILLABLE = Types.SMALLINT;

    public static final String NAME_BILLABLE = "TMT_CHARGECODES.BILLABLE";

    public static final String PROPERTY_BILLABLE = "BILLABLE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BILLABLE, SqlUtil.getBooleanValue(NAME_BILLABLE, 1));
    }

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. public static
     * final String PROPERTY_RANK = "PROPERTYNAME"; static{
     * FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}
     */
    public static final int ID_RANK = 13;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_CHARGECODES.RANK";

    /**
     * Column CAPITAL_EXPENSE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_CAPITAL_EXPENSE = 14;

    public static final int TYPE_CAPITAL_EXPENSE = Types.SMALLINT;

    public static final String NAME_CAPITAL_EXPENSE = "TMT_CHARGECODES.CAPITAL_EXPENSE";

    public static final String PROPERTY_CAPITAL_EXPENSE = "FINANCIALCATEGORY";

    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CAPITAL_EXPENSE, SqlUtil.castAsChar(NAME_CAPITAL_EXPENSE, 2));
    }

    /**
     * Column ASSOCIATE_CHARGECODE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ASSOCIATE_CHARGECODE_ID = 15;

    public static final int TYPE_ASSOCIATE_CHARGECODE_ID = Types.CHAR;

    public static final String NAME_ASSOCIATE_CHARGECODE_ID = "TMT_CHARGECODES.ASSOCIATE_CHARGECODE_ID";

    public static final String PROPERTY_ASSOCIATE_CHARGECODE_ID = "ASSOCIATEDCHARGECODE";

    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ASSOCIATE_CHARGECODE_ID,
                NAME_ASSOCIATE_CHARGECODE_ID);
    }

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 16;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "TMT_CHARGECODES.DESCRIPTION";

    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";

    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column CAPITAL_EXPENSE_ID of type Types.CHAR mapped to String.
     * 
     * REPLACE PROPERTYNAME with container property in upper case. public static
     * final String PROPERTY_CAPITAL_EXPENSE_ID = "PROPERTYNAME"; static{
     * FIELDPROPERTYMAP.put(PROPERTY_CAPITAL_EXPENSE_ID,NAME_CAPITAL_EXPENSE_ID);}
     */
    public static final int ID_CAPITAL_EXPENSE_ID = 17;

    public static final int TYPE_CAPITAL_EXPENSE_ID = Types.CHAR;

    public static final String NAME_CAPITAL_EXPENSE_ID = "TMT_CHARGECODES.CAPITAL_EXPENSE_ID";

    /**
     * Column STAGE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_STAGE_ID = 18;

    public static final int TYPE_STAGE_ID = Types.INTEGER;

    public static final String NAME_STAGE_ID = "TMT_CHARGECODES.STAGE_ID";

    public static final String PROPERTY_STAGE_ID = "STATE";

    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_STAGE_ID, NAME_STAGE_ID);
    }

    /**
     * Column EXTERNAL_IDENTIFIER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_EXTERNAL_IDENTIFIER = 19;

    public static final int TYPE_EXTERNAL_IDENTIFIER = Types.VARCHAR;

    public static final String NAME_EXTERNAL_IDENTIFIER = "TMT_CHARGECODES.EXTERNAL_IDENTIFIER";

    public static final String PROPERTY_EXTERNAL_IDENTIFIER = "EXTERNALID";

    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EXTERNAL_IDENTIFIER,
                NAME_EXTERNAL_IDENTIFIER);
    }

    private static final String[] FIELD_NAMES =
    {
            NAME_LEVEL_ID, NAME_TYPE_ID, NAME_MAX_RANK, NAME_CHILD_COUNT,
            NAME_DELETED_COUNT, NAME_ELEMENT_ID, NAME_ELEMENT_NAME,
            NAME_PARENT_ID, NAME_REC_USER, NAME_REC_STATUS, NAME_REC_DATETIME,
            NAME_BILLABLE, NAME_RANK, NAME_CAPITAL_EXPENSE,
            NAME_ASSOCIATE_CHARGECODE_ID, NAME_DESCRIPTION,
            NAME_CAPITAL_EXPENSE_ID, NAME_STAGE_ID, NAME_EXTERNAL_IDENTIFIER
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    private static final String TABLE_NAME = "TMT_CHARGECODES";

    /**
     */
    public ChargeCodeManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    public String getAllFields()
    {
        return ALL_FIELDS;
    }

    /**
     * @return
     */
    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws SQLException
    {

        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    public String getForeignKey(RPMObject object)
    {
        if (object instanceof GenericChargeCode)
            return NAME_PARENT_ID;
        return null;
    }

    public String getFilter()
    {
        StringBuffer filter = new StringBuffer();

        filter.append(NAME_ELEMENT_ID);
        filter.append(" IS NOT NULL AND ");
        filter.append(NAME_REC_STATUS);
        filter.append("!='D' AND ");
        filter.append(NAME_ELEMENT_ID);
        filter.append(" !='PMO_PROJECT_TIMEPHASE_CALENDAR__'");        

        return filter.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getFieldName(java.lang.String)
     */
    public String getFieldName(String property)
    {
        return (String) FIELDPROPERTYMAP.get(property);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String, int)
     */
    protected JoinCondition getJoinCondition(RPMObjectManager otherManager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // Create the basic filtering condition.
        String baseCondition = createBaseCondition(containerName, condition);

        JoinCondition joinCondition = new JoinCondition();
        if (otherManager == null)
        {
            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(baseCondition.toString());
        }
        else if (otherManager instanceof ChargeCodeManager)
        {
            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_ELEMENT_ID);
            where.append(" FROM ");
            where.append(TABLE_NAME);
            where.append(" WHERE ");
            where.append(baseCondition);
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, otherManager,
                    ExceptionUtil.JOIN);
        }

        return joinCondition;
    }
    
    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = new JoinCondition();

        if (manager instanceof ChargeCodeManager)
        {
            // For XPath queries like:
            // /ChargeCodeCategory[ChargeCode[<condition>]]
            // /ChargeCodeCategory[ChargeCodeCategory[<condition>]]
            // /ChargeCodeCategory[GenericChargeCode[<condition>]]
            joinCondition = JoinCondition.createSubSelect(NAME_ELEMENT_ID,
                NAME_PARENT_ID, TABLE_NAME, getFinalCondition(container));
        }
        else if (manager instanceof FinancialManager)
        {
            // For XPath queries like:
            // /WorksheetFinancial[ChargeCode[<condition>]]
            // /WBSFinancial[ChargeCode[<condition>]]
            // /AbstractFinancial[ChargeCode[<condition>]]
            joinCondition = JoinCondition.createSubSelect(
                FinancialManager.NAME_CHARGECODE_ID, NAME_ELEMENT_ID,
                TABLE_NAME, getFinalCondition(container));

        }
        else if (manager instanceof TaskAssignmentManager)
        {
            // For XPath queries like:
            // /TaskAssignment[ChargeCode[<condition>]]
            if ("costLaborCode".equals(container.getPreviousFieldName()))
            {
                joinCondition = JoinCondition.createSubSelect(
                    TaskAssignmentManager.NAME_LABOR_COST_CC_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
            }
            else
            {
                joinCondition = JoinCondition.createSubSelect(
                    TaskAssignmentManager.NAME_LABOR_REVENUE_CC_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
            }
        }
        // TODO Add ExpenseItem when its manager exists.
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        }
        
        return joinCondition;
    }

    private String createBaseCondition(String containerName,
            String otherCondition) throws RPMException
    {
        // Add the basic filtering condition.
        StringBuffer baseCondition = new StringBuffer();
        baseCondition.append(getFilter());

        // Add the type ID condition, if required.
        int typeId = getTypeId(containerName);
        if (typeId != 0)
        {
            baseCondition.append(" AND ");
            baseCondition.append(NAME_TYPE_ID);
            baseCondition.append('=');
            baseCondition.append(typeId);
        }

        // Add the other condition, if required.
        if (otherCondition != null)
        {
            baseCondition.append(" AND ");
            baseCondition.append(otherCondition);
        }

        return baseCondition.toString();
    }
    
    private String getFinalCondition(XPathContainer container)
    {
        // Add the basic filtering condition.
        SqlBuffer finalCondition = new SqlBuffer(getFilter());

        // Add the type ID condition, if required.
        Collection typeIds = containerMap.getTypeIds(container.getType());
        finalCondition.appendIn(NAME_TYPE_ID, typeIds);

        return finalCondition.toString();
    }

    private int getTypeId(String containerName) throws RPMException
    {
        // Get the type ID for the container.
        int typeId;
        if (StringUtil.getShortClassName(ChargeCode.class).equalsIgnoreCase(
                containerName))
        {
            typeId = ChargeCode.TYPE_ID;
        }
        else if (StringUtil.getShortClassName(ChargeCodeCategory.class)
                .equalsIgnoreCase(containerName))
        {
            typeId = ChargeCodeCategory.TYPE_ID;
        }
        else if (StringUtil.getShortClassName(GenericChargeCode.class)
                .equalsIgnoreCase(containerName))
        {
            typeId = 0;
        }
        else
        {
            throw new RPMException("Unsupported container " + containerName);
        }

        return typeId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected java.util.HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#loadRelatedObjects(com.ibm.rpm.containers.RPMObject,
     *      com.ibm.rpm.scope.RPMObjectScope, com.ibm.rpm.MessageContext,
     *      boolean, com.ibm.rpm.FieldValueMap, int)
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope != null)
        {
            // Load parent ChargeCodeCategory, if required.
            RPMObjectScope parentScope = rpmScope.getParent();
            if (parentScope != null && !isDelta)
            {
                RPMObject parent = null;
                String parentID = (String) relatedFields.get(index,
                        ID_PARENT_ID);
                if (parentID != null)
                {
                    parent = new ChargeCodeCategory();
                    parent.setID(parentID);
                    parent = loadByPrimaryKey(parent, parentScope,
                            messageContext, isDelta);
                }
                rpmObject.setParent(parent);
            }
            
            if(rpmScope instanceof ChargeCodeScope)
            {           
	            ChargeCodeScope scope = (ChargeCodeScope) rpmScope;
	
	            if (rpmObject instanceof ChargeCode)
	            {
		                ChargeCode chargeCode = (ChargeCode) rpmObject;
		
		                // Load associated ChargeCode, if required.
		                ChargeCodeScope associatedScope = scope
		                        .getAssociatedChargeCode();
		                ChargeCode associatedChargeCode = null;
		                if (associatedScope != null)
		                {
		                    String associatedID = (String) relatedFields.get(index,
		                            ID_ASSOCIATE_CHARGECODE_ID);
		                    associatedChargeCode = new ChargeCode();
		                    associatedChargeCode.setID(associatedID);
		                    if (isDelta)
		                    {
		                        chargeCode
		                                .deltaAssociatedChargeCode(associatedChargeCode);
		                    }
		                    else
		                    {
		                        if (associatedID == null)
		                        {
		                            associatedChargeCode = null;
		                        }
		                        else
		                        {
		                            associatedChargeCode = (ChargeCode) loadByPrimaryKey(
		                                    associatedChargeCode, associatedScope,
		                                    messageContext, isDelta);
		                            if (associatedChargeCode != null)
		                                associatedChargeCode
		                                        .setAssociatedChargeCode(chargeCode);
		                        }
		                        chargeCode
		                                .setAssociatedChargeCode(associatedChargeCode);
		                    }
		                }
		
		                // Load state, if required.
		                if (scope.isState())
		                {
		                    String stateID = (String) relatedFields.get(index,
		                            ID_STAGE_ID);
		                    ChargeCodeState state = new ChargeCodeState();
		                    state.setID(stateID);
		                    if (isDelta)
		                    {
		                        chargeCode.deltaState(state);
		                    }
		                    else
		                    {
		                        if (stateID == null)
		                        {
		                            state = null;
		                        }
		                        else
		                        {
		                            RPMObjectManager manager = RPMManagerFactory
		                                    .getInstance().getRPMObjectManager(
		                                            ChargeCodeState.class);
		                            state = (ChargeCodeState) manager.loadByPrimaryKey(
		                                    state, null, messageContext, isDelta);
		                        }
		                        chargeCode.setState(state);
		                    }
		                }
		            }
		
		            // Load children GenericChargeCode, if required.
		            if (rpmObject instanceof ChargeCodeCategory)
		            {
		                ChargeCodeCategory category = (ChargeCodeCategory) rpmObject;
		                ChargeCodeScope childrenScope = scope.getChildren();
		                if (childrenScope == null)
		                {
		                    category.setChildren(null);
		                }
		                else
		                {
		                    // This parent ID is used as the foreign key for its
		                    // children.
		                    Object[] arguments =
		                    {
		                        category.getID()
		                    };
		                    ArrayList childrenList = loadByForeignKey(category
		                            .getChildren(), childrenScope, messageContext,
		                            this, arguments, null, category.getContextName(),
		                            isDelta);
		                    GenericChargeCode[] children = null;
		                    if (childrenList != null)
		                    {
		                        int childrenCount = childrenList.size();
		                        if (childrenCount > 0)
		                        {
		                            children = new GenericChargeCode[childrenCount];
		                            childrenList.toArray(children);
		                        }
		                    }
		                    category.setChildren(children);
		                }
		            }
            }
        }
        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.containers.RPMObject, boolean)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject pObject,
            boolean isDelta) throws RPMException, SQLException
    {

        GenericChargeCode genChargeCode = (GenericChargeCode) pObject;

        String name = rs.getString(ID_ELEMENT_NAME);
        if(name!=null)
        {
            name = name.trim();
        }
        
        // Set common fields.
        if (isDelta)
        {
            genChargeCode.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
            genChargeCode.deltaName(name);
            genChargeCode.deltaDescription(rs.getString(ID_DESCRIPTION));
            genChargeCode.deltaExternalID(rs.getString(ID_EXTERNAL_IDENTIFIER));
        }
        else
        {
            genChargeCode.setID(rs.getString(ID_ELEMENT_ID));
            genChargeCode.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
            genChargeCode.setName(name);
            genChargeCode.setDescription(rs.getString(ID_DESCRIPTION));
            genChargeCode.setExternalID(rs.getString(ID_EXTERNAL_IDENTIFIER));
        }

        // Set fields specific to charge codes.
        if (genChargeCode instanceof ChargeCode)
        {
            ChargeCode chargeCode = (ChargeCode) genChargeCode;
            int financialCategoryCode = rs.getInt(ID_CAPITAL_EXPENSE);
            FinancialCategory financialCategory = FinancialCategoryUtil
                    .toFinancialCategory(financialCategoryCode);
            if (isDelta)
            {
                chargeCode
                        .deltaBillable(new Boolean(rs.getBoolean(ID_BILLABLE)));
                chargeCode.deltaFinancialCategory(financialCategory);
            }
            else
            {
                chargeCode.setBillable(new Boolean(rs.getBoolean(ID_BILLABLE)));
                chargeCode.setFinancialCategory(financialCategory);
            }
        }

        return pObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        String associatedId = resultSet.getString(ID_ASSOCIATE_CHARGECODE_ID);
        relatedFields.put(index, ID_ASSOCIATE_CHARGECODE_ID, associatedId);

        int intStateId = resultSet.getInt(ID_STAGE_ID);
        String stageId = intStateId == 0 ? null : String.valueOf(intStateId);
        relatedFields.put(index, ID_STAGE_ID, stageId);

        return relatedFields;
    }

    /**
     * Logically deletes the specified <code>genChargeCode</code>. Important:
     * fails if the <code>genChargeCode</code> or any of its descendants has a
     * non-null <code>ExternalID</code>.
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        //Modifications on system charge code are not allowed.
        if (GenericChargeCodeCheckpoint
            .isChargeCodePartOfDefaultChargeCodeList(rpmObject.getID()))
            return;
        String parentId = ManagerUtil.getParentId(this, rpmObject, messageContext, TABLE_NAME);
        SP_D_CHARGECODE(rpmObject.getID(), parentId, messageContext);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(
            RPMObjectManager callingManager, String extraWhere,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        // TODO Implement getForeignKeyCondition for
        // ExpenseItem, TaskAssignment, etc.
        if (callingManager instanceof ChargeCodeManager)
        {
            // Design note: used when loading ChargeCodeCategory's children.
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            where.append(NAME_PARENT_ID);
            where.append("=?");

            joinCondition.setCondition(where.toString());
        }
        else if (callingManager == null || callingManager instanceof FinancialModuleManager)
        {            
            joinCondition = new JoinCondition();
            joinCondition.setCondition(extraWhere);            
            joinCondition.setTableName(getTableName()); 
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, callingManager,
                    ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;

        switch (typeId)
        {
        case ChargeCode.TYPE_ID:
            object = new ChargeCode();
            break;
        case ChargeCodeCategory.TYPE_ID:
            object = new ChargeCodeCategory();
            break;
        default:
            RPMException e = new RPMException("TypeId " + typeId
                    + " is not supported");
            throw e.log();
        }
        return object;
    }

    /**
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject,
     *      boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        //Modifications on system charge code are not allowed.
        if (GenericChargeCodeCheckpoint
            .isChargeCodePartOfDefaultChargeCodeList(rpmObject.getID()))
            return rpmObject;
        GenericChargeCode genChargeCode = (GenericChargeCode) rpmObject;
        ChargeCodeScope scope = (ChargeCodeScope) rpmScope;

        // Update properties, if required.
        conditionalSP_U_CHARGECODE(genChargeCode, scope, messageContext);
        conditionalSP_U_CC_STAGE_ID(genChargeCode, scope, messageContext);

        // Update related objects.
        if (scope != null)
        {
            if (genChargeCode instanceof ChargeCode)
            {
                ChargeCode chargeCode = (ChargeCode) genChargeCode;
                updateAssociatedChargeCode(chargeCode, scope, messageContext);
            }
            else if (genChargeCode instanceof ChargeCodeCategory)
            {
                updateChildren((ChargeCodeCategory) genChargeCode, scope,
                        messageContext);
            }
        }

        return genChargeCode;
    }

    private void conditionalSP_U_CHARGECODE(GenericChargeCode genChargeCode,
            ChargeCodeScope scope, MessageContext messageContext)
            throws SQLException, RPMException
    {
        boolean willCall = genChargeCode.testDescriptionModified()
                || genChargeCode.testNameModified();

        Integer financialCategoryCode = new Integer(0);
        Boolean billable = Boolean.FALSE;

        if (genChargeCode instanceof ChargeCode)
        {
            ChargeCode chargeCode = (ChargeCode) genChargeCode;
            willCall = willCall || chargeCode.testBillableModified()
                    || chargeCode.testFinancialCategoryModified();
            if (willCall)
            {
                financialCategoryCode = FinancialCategoryUtil
                        .toDBCode(chargeCode.getFinancialCategory());
                Boolean bill = chargeCode.getBillable();
                if (bill != null)
                    billable = bill;
            }
        }

        if (willCall)
        {
            String iD = genChargeCode.getID();
            String name = genChargeCode.getName();
            String description = genChargeCode.getDescription();
            SP_U_CHARGECODE(iD, name, description, financialCategoryCode,
                    billable, messageContext);
        }
    }

    private void conditionalSP_U_CC_STAGE_ID(GenericChargeCode genChargeCode,
            ChargeCodeScope scope, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Integer stateID = null;
        boolean willCall = genChargeCode.testExternalIDModified();
        if (genChargeCode instanceof ChargeCode)
        {
            if (scope != null && scope.isState())
            {
                ChargeCode chargeCode = (ChargeCode) genChargeCode;
                ChargeCodeState state = chargeCode.getState();
                if (state != null)
                {
                    String stateIDstr = state.getID();
                    if (stateIDstr != null)
                        stateID = Integer.valueOf(stateIDstr);
                }
                willCall = willCall || (chargeCode.testStateModified());
            }
        }

        if (willCall)
        {
            String thisID = genChargeCode.getID();
            if (stateID == null)
                stateID = new Integer(6801);
            String externalID = genChargeCode.getExternalID();
            SP_U_CC_STAGE_ID(thisID, stateID, externalID, messageContext);
        }
    }

    private void updateAssociatedChargeCode(ChargeCode chargeCode,
            ChargeCodeScope scope, MessageContext messageContext)
            throws RPMException, SQLException
    {
        // Update association (not the associated object), if required.
        ChargeCode associatedChargeCode = chargeCode.getAssociatedChargeCode();
        if (scope.getAssociatedChargeCode() != null
                && chargeCode.testAssociatedChargeCodeModified())
        {
            String associatedID = null;
            if (associatedChargeCode != null)
            {
                associatedID = associatedChargeCode.getID();
                // Ensure the association is bidirectionnal.
                associatedChargeCode.setAssociatedChargeCode(chargeCode);
                // Skip the call if the association is already made.
                // Note: This patch is required because calling SP_U_ASS_CC
                // twice for the same association fails AND corrupts the 
                // association in the database.
                if (associatedID != null)
                {
	                String dbAssociatedId = getAssociatedId(chargeCode.getID(),
	                        messageContext);
	                if (associatedID.equals(dbAssociatedId))
	                {
	                    return;
	                }
                }
            }
            SP_U_ASS_CC(chargeCode.getID(), associatedID, messageContext);
        }
    }

    private static String getAssociatedId(String id, MessageContext msgCtx)
            throws SQLException
    {
        Connection connection = null;
        ResultSet resultSet = null;
        String associatedId = null;

        String statementText = "SELECT " + NAME_ASSOCIATE_CHARGECODE_ID
                + " FROM " + TABLE_NAME + " WHERE " + NAME_ELEMENT_ID + "='"
                + id + '\'';
        logger.debug(statementText);
        try
        {
            connection = Manager.getConnection(msgCtx);
            statementText = Manager.makeStatementUncommitedRead(statementText,
                    connection);
            PreparedStatement statement = connection.prepareStatement(
                    statementText, ResultSet.TYPE_SCROLL_INSENSITIVE,
                    ResultSet.CONCUR_READ_ONLY);
            
            resultSet = ManagerUtil.executeSqlQuery(msgCtx, statement,
                statementText, null);
            resultSet.next();
            associatedId = resultSet.getString(1);
        }
        finally
        {
            Manager.close(msgCtx, resultSet);
            Manager.releaseConnection(msgCtx, connection);
        }

        return associatedId;
    }

    private void updateChildren(ChargeCodeCategory category,
            ChargeCodeScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        GenericChargeCode[] children = category.getChildren();
        ChargeCodeScope childrenScope = scope.getChildren();
        children = (GenericChargeCode[]) updateChildren(category, children,
                childrenScope, messageContext, GenericChargeCode.class);
        category.setChildren(children);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        rpmObject.setID(SP_I_CHARGECODE((GenericChargeCode) rpmObject,
                messageContext));
    }

    /**
     * Creates a charge code. Initialises Name, Parent and TypeID.
     * 
     * @return ID of the new chargecode.
     */
    private String SP_I_CHARGECODE(GenericChargeCode object,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[4];
        args[0] = object.getName();

        if (object.getParent() != null)
            args[1] = object.getParent().getID();

        args[2] = object.retrieveTypeID();
        args[3] = getUser(messageContext).getID();

        String name = "SP_I_CHARGECODE";
        String elementId = executeProcedure(messageContext, name, args);

        return elementId;
    }

    /**
     * Updates Name, Description, FinancialCategory, and Billable.
     */
    private void SP_U_CHARGECODE(String iD, String name, String description,
            Integer financialCategoryCode, Boolean billable,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[6];
        args[0] = iD;
        args[1] = name;
        args[2] = description;
        args[3] = financialCategoryCode;
        args[4] = billable;
        args[5] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_U_CHARGECODE", args);
    }

    /**
     * Updates ExternalID (null is allowed) and State (null is allowed).
     */
    private void SP_U_CC_STAGE_ID(String thisID, Integer stateID,
            String externalID, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Object args[] = new Object[4];
        args[0] = thisID;
        args[1] = stateID;
        args[2] = externalID;
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_U_CC_STAGE_ID", args);
    }

    /**
     * Updates the charge code association.
     */
    private void SP_U_ASS_CC(String thisID, String associatedID,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[3];
        args[0] = thisID;
        args[1] = associatedID;
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_U_ASS_CC", args);
    }

    /**
     * Deletes the specified charge code and all its descendants. All of these
     * records must have a NULL EXTERNAL_IDENTIFIER.
     */
    private void SP_D_CHARGECODE(String id, String parentId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[4];
        // Initialize object identifier.
        args[0] = id;
        // Initialize parent identifier.
        args[1] = parentId;
        // Initialize deletion flag: "D" for logical or "X" for physical.
        args[2] = "X";
        // Initialize user identifier.
        args[3] = getUser(messageContext).getID();
        // Calls the procedure.
        executeProcedure(messageContext, "SP_D_CHARGECODE", args);
    }
}
