/*
 * Created on Feb 21, 2006
 *
 */
package com.ibm.rpm.financial.managers;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.ChargeCodeState;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.FinancialModule;
import com.ibm.rpm.financial.containers.TimeCodeGroup;
import com.ibm.rpm.financial.scope.FinancialModuleScope;
import com.ibm.rpm.framework.AbstractModuleManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;

/**
 * @author karim
 *
 */
public class FinancialModuleManager extends AbstractModuleManager
{
    private static Log logger = LogFactory.getLog(FinancialModuleManager.class);

    protected Log getLogger()
    {
        return logger;
    }

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(FinancialModule.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    FinancialModuleManager.class);
        }
    }

    {
        localContextName = ContextUtil.UMBRELLA_FINANCIAL_CONTEXT;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = new FinancialModule();
        return object;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof ChargeCodeManager)
        {
                joinCondition = new JoinCondition();
        }
        else if (manager instanceof TimecodeManager)
        {
                joinCondition = new JoinCondition();
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected void loadChildren(AbstractModule abstractModule,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        FinancialModule module = (FinancialModule) abstractModule;
        FinancialModuleScope scope = (FinancialModuleScope) rpmScope;
        
    	DatafieldManager datafieldManager = (DatafieldManager)getManagerInstance(DatafieldCategory.class);

        if(scope!=null)
        {
            if(scope.getChargeCodeCategories()!=null)
            {
                String extraWhere = " LEVEL_ID = 0 ";
            	ChargeCodeCategory[] chargeCodeCategories = (ChargeCodeCategory[]) loadElements(module, module.getChargeCodeCategories(),
            			scope.getChargeCodeCategories(),ChargeCodeCategory.class, 
    					null, extraWhere, messageContext, isDelta );        
                module.setChargeCodeCategories(chargeCodeCategories);
            }
            if(scope.isChargeCodeStates())
            {
            	ChargeCodeState[] chargeCodeStates = (ChargeCodeState[]) datafieldManager
                    .loadDatafields(module, module.getChargeCodeStates(),
                        ChargeCodeState.class, ChargeCodeState.TYPE_ID,
                        this, localContextName, messageContext, isDelta);
                module.setChargeCodeStates(chargeCodeStates);
            }
            if(scope.getTimeCodeGroups()!=null)
            {
                String extraWhere = " LEVEL_ID = 0 ";
            	TimeCodeGroup[] timeCodeGroups = (TimeCodeGroup[]) loadElements(module, module.getTimeCodeGroups(),
            			scope.getTimeCodeGroups(),TimeCodeGroup.class, 
    					null, extraWhere, messageContext, isDelta );        
                module.setTimeCodeGroups(timeCodeGroups);
            }
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        FinancialModuleScope scope = (FinancialModuleScope) rpmScope;
        FinancialModule module = (FinancialModule) rpmObject;
        module.setContextName(localContextName);
        
        if(scope!=null)
        {
            if(scope.getChargeCodeCategories()!=null)
            {
            	ChargeCodeCategory[] chargeCodeCategories = (ChargeCodeCategory[]) saveElements(
                        module, module.getChargeCodeCategories(), scope
                                .getChargeCodeCategories(), messageContext,
                        ChargeCodeCategory.class);
                module.setChargeCodeCategories(chargeCodeCategories);
            }
        }
        return module;
    }
}
