package com.ibm.rpm.financial.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;

import com.ibm.rpm.applicationadministration.containers.ReconciliationReason;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.financial.util.ProjectReconciliationStateUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.managers.WbsManager;

/**
 * Manager for the PROJECT_RECONCILIATION table and related stored procedures.
 * 
 * <h3>State transition matrix</h3>
 * <p>
 * <table border=1 cellspacing=0>
 * <tr>
 * <th>From\To</th>
 * <th>0-None</th>
 * <th>1-Submitted</th>
 * <th>2-Synchonized</th>
 * <th>3-SynchonizedAndSubmitted</th>
 * <th>4-Accepted</th>
 * <th>5-Rejected</th>
 * </tr>
 * <tr>
 * <th>0-None</th>
 * <td>&nbsp;</td>
 * <td>Submit</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <th>1-Submitted</th>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>Synchronize</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <th>2-Synchonized</th>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>Submit</td>
 * <td>Accept</td>
 * <td>Reject</td>
 * </tr>
 * <tr>
 * <th>3-SynchonizedAndSubmitted</th>
 * <td>&nbsp;</td>
 * <td>Accept <b>or </b> Reject</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <th>4-Accepted</th>
 * <td>&nbsp;</td>
 * <td>Submit</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <th>5-Rejected</th>
 * <td>&nbsp;</td>
 * <td>Submit</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * </table>
 * 
 * <h3>How to trigger a transitions with the API</h3>
 * <p>
 * <table border=1 cellspacing=0>
 * <tr>
 * <th>Transition</th>
 * <th>Trigger</th>
 * </tr>
 * <tr>
 * <td>Submit</td>
 * <td>ProjectReconciliation.state=Synchronized <b>or </b>
 * ProjectReconciliation.state=SynchronizedAndSubmitted</td>
 * </tr>
 * <tr>
 * <td>Synchronize</td>
 * <td>
 * GenericProject.projectReconciliationStatus.projectSynchronized=Boolean.TRUE
 * </td>
 * </tr>
 * <tr>
 * <td>Accept</td>
 * <td>ProjectReconciliation.state=Accepted</td>
 * </tr>
 * <tr>
 * <td>Reject</td>
 * <td>ProjectReconciliation.state=Rejected</td>
 * </tr>
 * </table>
 * 
 * <h3>Transitions implementation</h3>
 * <p>
 * Note that SP_U_RECONCILIATION_STATE's parameter (1, 4 or 5) represents a
 * transition code, not the target state.
 * </p>
 * <table border=1 cellspacing=0>
 * <tr>
 * <th>Transition</th>
 * <th>Implementation</th>
 * </tr>
 * <tr>
 * <td>Submit</td>
 * <td>ProjectReconciliationManager.SP_U_PRJ_SUBMITTED_RECON(1);
 * ...SP_U_RECONCILIATION_STATE(1)</td>
 * </tr>
 * <tr>
 * <td>Synchronize</td>
 * <td>ProjectReconciliationStatusManager.SP_SYNCH_PRJ_RECON</td>
 * </tr>
 * <tr>
 * <td>Accept</td>
 * <td>ProjectReconciliationManager.SP_U_RECONCILIATION_STATE(4)</td>
 * </tr>
 * <tr>
 * <td>Reject</td>
 * <td>ProjectReconciliationManager.SP_U_RECONCILIATION_STATE(5)</td>
 * </tr>
 * </table>
 */
public class ProjectReconciliationManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    protected static ContainerMap containerMap = new ContainerMap(
            ProjectReconciliationManager.class);
    static
    {
        //TODO
        //containerMap.add(ProjectReconciliation.class);

    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "PROJECT_RECONCILIATION.ELEMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ELEMENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 2;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "PROJECT_RECONCILIATION.PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 3;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "PROJECT_RECONCILIATION.RESOURCE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RESOURCE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID,NAME_RESOURCE_ID);}

    /**
     * Column FOREIGN_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_FOREIGN_ID = 4;

    public static final int TYPE_FOREIGN_ID = Types.CHAR;

    public static final String NAME_FOREIGN_ID = "PROJECT_RECONCILIATION.FOREIGN_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FOREIGN_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FOREIGN_ID,NAME_FOREIGN_ID);}

    /**
     * Column FOREIGN_TABLE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_FOREIGN_TABLE = 5;

    public static final int TYPE_FOREIGN_TABLE = Types.SMALLINT;

    public static final String NAME_FOREIGN_TABLE = "PROJECT_RECONCILIATION.FOREIGN_TABLE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FOREIGN_TABLE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FOREIGN_TABLE,NAME_FOREIGN_TABLE);}

    /**
     * Column FOREIGN_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_FOREIGN_DATE = 6;

    public static final int TYPE_FOREIGN_DATE = Types.DATE;

    public static final String NAME_FOREIGN_DATE = "PROJECT_RECONCILIATION.FOREIGN_DATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FOREIGN_DATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FOREIGN_DATE,NAME_FOREIGN_DATE);}

    /**
     * Column TYPE_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_TYPE_ID = 7;

    public static final int TYPE_TYPE_ID = Types.SMALLINT;

    public static final String NAME_TYPE_ID = "PROJECT_RECONCILIATION.TYPE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column COST_CENTER_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_COST_CENTER_ID = 8;

    public static final int TYPE_COST_CENTER_ID = Types.CHAR;

    public static final String NAME_COST_CENTER_ID = "PROJECT_RECONCILIATION.COST_CENTER_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_COST_CENTER_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_COST_CENTER_ID,NAME_COST_CENTER_ID);}

    /**
     * Column DEFAULT_LABOR_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_DEFAULT_LABOR_FLAG = 9;

    public static final int TYPE_DEFAULT_LABOR_FLAG = Types.SMALLINT;

    public static final String NAME_DEFAULT_LABOR_FLAG = "PROJECT_RECONCILIATION.DEFAULT_LABOR_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DEFAULT_LABOR_FLAG = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_LABOR_FLAG,NAME_DEFAULT_LABOR_FLAG);}

    /**
     * Column REGULAR_TOTAL_QUANTITY of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_REGULAR_TOTAL_QUANTITY = 10;

    public static final int TYPE_REGULAR_TOTAL_QUANTITY = Types.DECIMAL;

    public static final String NAME_REGULAR_TOTAL_QUANTITY = "PROJECT_RECONCILIATION.REGULAR_TOTAL_QUANTITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REGULAR_TOTAL_QUANTITY =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REGULAR_TOTAL_QUANTITY,NAME_REGULAR_TOTAL_QUANTITY);}

    /**
     * Column REGULAR_DEFAULT_QUANTITY of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_REGULAR_DEFAULT_QUANTITY = 11;

    public static final int TYPE_REGULAR_DEFAULT_QUANTITY = Types.DECIMAL;

    public static final String NAME_REGULAR_DEFAULT_QUANTITY = "PROJECT_RECONCILIATION.REGULAR_DEFAULT_QUANTITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REGULAR_DEFAULT_QUANTITY =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REGULAR_DEFAULT_QUANTITY,NAME_REGULAR_DEFAULT_QUANTITY);}

    /**
     * Column REGULAR_QUANTITY of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_REGULAR_QUANTITY = 12;

    public static final int TYPE_REGULAR_QUANTITY = Types.DECIMAL;

    public static final String NAME_REGULAR_QUANTITY = "PROJECT_RECONCILIATION.REGULAR_QUANTITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REGULAR_QUANTITY = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_REGULAR_QUANTITY,NAME_REGULAR_QUANTITY);}

    /**
     * Column REGULAR_RATE of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_REGULAR_RATE = 13;

    public static final int TYPE_REGULAR_RATE = Types.DECIMAL;

    public static final String NAME_REGULAR_RATE = "PROJECT_RECONCILIATION.REGULAR_RATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REGULAR_RATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REGULAR_RATE,NAME_REGULAR_RATE);}

    /**
     * Column SPECIAL_TOTAL_QUANTITY of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_SPECIAL_TOTAL_QUANTITY = 14;

    public static final int TYPE_SPECIAL_TOTAL_QUANTITY = Types.DECIMAL;

    public static final String NAME_SPECIAL_TOTAL_QUANTITY = "PROJECT_RECONCILIATION.SPECIAL_TOTAL_QUANTITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SPECIAL_TOTAL_QUANTITY =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SPECIAL_TOTAL_QUANTITY,NAME_SPECIAL_TOTAL_QUANTITY);}

    /**
     * Column SPECIAL_DEFAULT_QUANTITY of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_SPECIAL_DEFAULT_QUANTITY = 15;

    public static final int TYPE_SPECIAL_DEFAULT_QUANTITY = Types.DECIMAL;

    public static final String NAME_SPECIAL_DEFAULT_QUANTITY = "PROJECT_RECONCILIATION.SPECIAL_DEFAULT_QUANTITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SPECIAL_DEFAULT_QUANTITY =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SPECIAL_DEFAULT_QUANTITY,NAME_SPECIAL_DEFAULT_QUANTITY);}

    /**
     * Column SPECIAL_QUANTITY of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_SPECIAL_QUANTITY = 16;

    public static final int TYPE_SPECIAL_QUANTITY = Types.DECIMAL;

    public static final String NAME_SPECIAL_QUANTITY = "PROJECT_RECONCILIATION.SPECIAL_QUANTITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SPECIAL_QUANTITY = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_SPECIAL_QUANTITY,NAME_SPECIAL_QUANTITY);}

    /**
     * Column SPECIAL_RATE of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_SPECIAL_RATE = 17;

    public static final int TYPE_SPECIAL_RATE = Types.DECIMAL;

    public static final String NAME_SPECIAL_RATE = "PROJECT_RECONCILIATION.SPECIAL_RATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SPECIAL_RATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SPECIAL_RATE,NAME_SPECIAL_RATE);}

    /**
     * Column TAX1 of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_TAX1 = 18;

    public static final int TYPE_TAX1 = Types.DECIMAL;

    public static final String NAME_TAX1 = "PROJECT_RECONCILIATION.TAX1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TAX1 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TAX1,NAME_TAX1);}

    /**
     * Column TAX2 of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_TAX2 = 19;

    public static final int TYPE_TAX2 = Types.DECIMAL;

    public static final String NAME_TAX2 = "PROJECT_RECONCILIATION.TAX2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TAX2 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TAX2,NAME_TAX2);}

    /**
     * Column ADJ_REGULAR_QUANTITY of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ADJ_REGULAR_QUANTITY = 20;

    public static final int TYPE_ADJ_REGULAR_QUANTITY = Types.DECIMAL;

    public static final String NAME_ADJ_REGULAR_QUANTITY = "PROJECT_RECONCILIATION.ADJ_REGULAR_QUANTITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ADJ_REGULAR_QUANTITY =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ADJ_REGULAR_QUANTITY,NAME_ADJ_REGULAR_QUANTITY);}

    /**
     * Column ADJ_REGULAR_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ADJ_REGULAR_RATE = 21;

    public static final int TYPE_ADJ_REGULAR_RATE = Types.DECIMAL;

    public static final String NAME_ADJ_REGULAR_RATE = "PROJECT_RECONCILIATION.ADJ_REGULAR_RATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ADJ_REGULAR_RATE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ADJ_REGULAR_RATE,NAME_ADJ_REGULAR_RATE);}

    /**
     * Column ADJ_SPECIAL_QUANTITY of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ADJ_SPECIAL_QUANTITY = 22;

    public static final int TYPE_ADJ_SPECIAL_QUANTITY = Types.DECIMAL;

    public static final String NAME_ADJ_SPECIAL_QUANTITY = "PROJECT_RECONCILIATION.ADJ_SPECIAL_QUANTITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ADJ_SPECIAL_QUANTITY =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ADJ_SPECIAL_QUANTITY,NAME_ADJ_SPECIAL_QUANTITY);}

    /**
     * Column ADJ_SPECIAL_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ADJ_SPECIAL_RATE = 23;

    public static final int TYPE_ADJ_SPECIAL_RATE = Types.DECIMAL;

    public static final String NAME_ADJ_SPECIAL_RATE = "PROJECT_RECONCILIATION.ADJ_SPECIAL_RATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ADJ_SPECIAL_RATE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_ADJ_SPECIAL_RATE,NAME_ADJ_SPECIAL_RATE);}

    /**
     * Column ADJ_TAX1 of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_ADJ_TAX1 = 24;

    public static final int TYPE_ADJ_TAX1 = Types.DECIMAL;

    public static final String NAME_ADJ_TAX1 = "PROJECT_RECONCILIATION.ADJ_TAX1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ADJ_TAX1 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ADJ_TAX1,NAME_ADJ_TAX1);}

    /**
     * Column ADJ_TAX2 of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_ADJ_TAX2 = 25;

    public static final int TYPE_ADJ_TAX2 = Types.DECIMAL;

    public static final String NAME_ADJ_TAX2 = "PROJECT_RECONCILIATION.ADJ_TAX2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ADJ_TAX2 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ADJ_TAX2,NAME_ADJ_TAX2);}

    /**
     * Column LEDGER_REGULAR_QUANTITY of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_LEDGER_REGULAR_QUANTITY = 26;

    public static final int TYPE_LEDGER_REGULAR_QUANTITY = Types.DECIMAL;

    public static final String NAME_LEDGER_REGULAR_QUANTITY = "PROJECT_RECONCILIATION.LEDGER_REGULAR_QUANTITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEDGER_REGULAR_QUANTITY =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LEDGER_REGULAR_QUANTITY,NAME_LEDGER_REGULAR_QUANTITY);}

    /**
     * Column LEDGER_REGULAR_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_LEDGER_REGULAR_RATE = 27;

    public static final int TYPE_LEDGER_REGULAR_RATE = Types.DECIMAL;

    public static final String NAME_LEDGER_REGULAR_RATE = "PROJECT_RECONCILIATION.LEDGER_REGULAR_RATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEDGER_REGULAR_RATE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LEDGER_REGULAR_RATE,NAME_LEDGER_REGULAR_RATE);}

    /**
     * Column LEDGER_SPECIAL_QUANTITY of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_LEDGER_SPECIAL_QUANTITY = 28;

    public static final int TYPE_LEDGER_SPECIAL_QUANTITY = Types.DECIMAL;

    public static final String NAME_LEDGER_SPECIAL_QUANTITY = "PROJECT_RECONCILIATION.LEDGER_SPECIAL_QUANTITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEDGER_SPECIAL_QUANTITY =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LEDGER_SPECIAL_QUANTITY,NAME_LEDGER_SPECIAL_QUANTITY);}

    /**
     * Column LEDGER_SPECIAL_RATE of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_LEDGER_SPECIAL_RATE = 29;

    public static final int TYPE_LEDGER_SPECIAL_RATE = Types.DECIMAL;

    public static final String NAME_LEDGER_SPECIAL_RATE = "PROJECT_RECONCILIATION.LEDGER_SPECIAL_RATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEDGER_SPECIAL_RATE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LEDGER_SPECIAL_RATE,NAME_LEDGER_SPECIAL_RATE);}

    /**
     * Column LEDGER_TAX1 of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_LEDGER_TAX1 = 30;

    public static final int TYPE_LEDGER_TAX1 = Types.DECIMAL;

    public static final String NAME_LEDGER_TAX1 = "PROJECT_RECONCILIATION.LEDGER_TAX1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEDGER_TAX1 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEDGER_TAX1,NAME_LEDGER_TAX1);}

    /**
     * Column LEDGER_TAX2 of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_LEDGER_TAX2 = 31;

    public static final int TYPE_LEDGER_TAX2 = Types.DECIMAL;

    public static final String NAME_LEDGER_TAX2 = "PROJECT_RECONCILIATION.LEDGER_TAX2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEDGER_TAX2 = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEDGER_TAX2,NAME_LEDGER_TAX2);}

    /**
     * Column CHARGE_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_CHARGE_DATE = 32;

    public static final int TYPE_CHARGE_DATE = Types.DATE;

    public static final String NAME_CHARGE_DATE = "PROJECT_RECONCILIATION.CHARGE_DATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHARGE_DATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHARGE_DATE,NAME_CHARGE_DATE);}

    /**
     * Column RECONCILIATION_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_RECONCILIATION_DATE = 33;

    public static final int TYPE_RECONCILIATION_DATE = Types.DATE;

    public static final String NAME_RECONCILIATION_DATE = "PROJECT_RECONCILIATION.RECONCILIATION_DATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RECONCILIATION_DATE = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_RECONCILIATION_DATE,NAME_RECONCILIATION_DATE);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 34;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "PROJECT_RECONCILIATION.DESCRIPTION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DESCRIPTION = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION,NAME_DESCRIPTION);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 35;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "PROJECT_RECONCILIATION.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 36;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "PROJECT_RECONCILIATION.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 37;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "PROJECT_RECONCILIATION.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column WEEK_OF of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_WEEK_OF = 38;

    public static final int TYPE_WEEK_OF = Types.DATE;

    public static final String NAME_WEEK_OF = "PROJECT_RECONCILIATION.WEEK_OF";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WEEK_OF = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WEEK_OF,NAME_WEEK_OF);}

    /**
     * Column CONTRACT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CONTRACT_ID = 39;

    public static final int TYPE_CONTRACT_ID = Types.CHAR;

    public static final String NAME_CONTRACT_ID = "PROJECT_RECONCILIATION.CONTRACT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONTRACT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CONTRACT_ID,NAME_CONTRACT_ID);}

    /**
     * Column REASON_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REASON_ID = 40;

    public static final int TYPE_REASON_ID = Types.INTEGER;

    public static final String NAME_REASON_ID = "PROJECT_RECONCILIATION.REASON_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REASON_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REASON_ID,NAME_REASON_ID);}

    /**
     * Column STATE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_STATE = 41;

    public static final int TYPE_STATE = Types.SMALLINT;

    public static final String NAME_STATE = "PROJECT_RECONCILIATION.STATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_STATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_STATE,NAME_STATE);}

    /**
     * Column INVOICE_NUMBER of type Types.VARCHAR mapped to String.
     */
    public static final int ID_INVOICE_NUMBER = 42;

    public static final int TYPE_INVOICE_NUMBER = Types.VARCHAR;

    public static final String NAME_INVOICE_NUMBER = "PROJECT_RECONCILIATION.INVOICE_NUMBER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_INVOICE_NUMBER = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_INVOICE_NUMBER,NAME_INVOICE_NUMBER);}

    public static final String TABLE_NAME = "PROJECT_RECONCILIATION";

    /**
     * Create an array of type string containing all the fields of the
     * PROJECT_RECONCILIATION table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_PROJECT_ID, NAME_RESOURCE_ID,
            NAME_FOREIGN_ID, NAME_FOREIGN_TABLE, NAME_FOREIGN_DATE,
            NAME_TYPE_ID, NAME_COST_CENTER_ID, NAME_DEFAULT_LABOR_FLAG,
            NAME_REGULAR_TOTAL_QUANTITY, NAME_REGULAR_DEFAULT_QUANTITY,
            NAME_REGULAR_QUANTITY, NAME_REGULAR_RATE,
            NAME_SPECIAL_TOTAL_QUANTITY, NAME_SPECIAL_DEFAULT_QUANTITY,
            NAME_SPECIAL_QUANTITY, NAME_SPECIAL_RATE, NAME_TAX1, NAME_TAX2,
            NAME_ADJ_REGULAR_QUANTITY, NAME_ADJ_REGULAR_RATE,
            NAME_ADJ_SPECIAL_QUANTITY, NAME_ADJ_SPECIAL_RATE, NAME_ADJ_TAX1,
            NAME_ADJ_TAX2, NAME_LEDGER_REGULAR_QUANTITY,
            NAME_LEDGER_REGULAR_RATE, NAME_LEDGER_SPECIAL_QUANTITY,
            NAME_LEDGER_SPECIAL_RATE, NAME_LEDGER_TAX1, NAME_LEDGER_TAX2,
            NAME_CHARGE_DATE, NAME_RECONCILIATION_DATE, NAME_DESCRIPTION,
            NAME_REC_USER, NAME_REC_STATUS, NAME_REC_DATETIME, NAME_WEEK_OF,
            NAME_CONTRACT_ID, NAME_REASON_ID, NAME_STATE, NAME_INVOICE_NUMBER
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        //TODO remove comments to implement ProjectReconciliation
        
        //RPMObject object = new ProjectReconciliation();
        //return object;
        
        return null;
    }

    /**
     * Creates a new ProjectReconciliationManager instance.
     * 
     * @return ProjectReconciliationManager
     */
    public ProjectReconciliationManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        Integer type = getContainerType(resultSet);
        return createContainer(type.intValue());
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        //TODO remove comments to implement ProjectReconciliation
/*        ProjectReconciliation reconciliation = (ProjectReconciliation) rpmObject;
        reconciliation.setID(rs.getString(ID_ELEMENT_ID));

        Integer stateCode = new Integer(rs.getInt(ID_STATE));
        ReconciliationState state = ProjectReconciliationStateUtil
                .getReconciliationState(stateCode);

        if (isDelta)
        {
            reconciliation.deltaState(state);
        }
        else
        {
            reconciliation.setState(state);

            // read-only fields
            reconciliation.setInvoiceNumber(rs.getString(ID_INVOICE_NUMBER));
            reconciliation.setLastEditDate(Manager.getCalendar(rs,
                    ID_REC_DATETIME));
        }
*/
        return rpmObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        relatedFields.put(index, ID_RESOURCE_ID, resultSet
                .getString(ID_RESOURCE_ID));
        relatedFields.put(index, ID_PROJECT_ID, resultSet
                .getString(ID_PROJECT_ID));
        relatedFields.put(index, ID_WEEK_OF, resultSet.getString(ID_WEEK_OF));
        relatedFields.put(index, ID_REC_USER, resultSet.getString(ID_REC_USER));
        relatedFields.put(index, ID_REASON_ID, resultSet
                .getString(ID_REASON_ID));

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        //TODO remove comments to implement ProjectReconciliation
/*        ProjectReconciliation reconciliation = (ProjectReconciliation) rpmObject;

        loadDoneBy(reconciliation, messageContext, relatedFields, index,
                isDelta);
        loadReconciliationReason(reconciliation, messageContext, relatedFields,
                index, isDelta);
        loadParent(reconciliation, messageContext, relatedFields, index,
                isDelta);
        // must load after parent
        loadTimesheetApprovalStatus(reconciliation, messageContext,
                relatedFields, index, isDelta);*/

        return rpmObject;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WbsManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_PROJECT_ID,
                    WbsManager.NAME_PROJECT_ID, WbsManager.TABLE_NAME,
                    WbsManager.NAME_PROJECT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // Object creation not valid; load ProjectReconciliation records
        // through GenericProject container
        //TODO remove comments to implement ProjectReconciliation
/*        ExceptionUtil.handleUnsupportedAPICall(this, StringUtil
                .getShortClassName(ProjectReconciliation.class));*/
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
         //TODO remove comments to implement ProjectReconciliation
/*        ProjectReconciliation reconciliation = (ProjectReconciliation) rpmObject;

        if (reconciliation.testStateModified())
        {
            // Updates the submit state of the project.
            ReconciliationState newState = reconciliation.getState();
            if (newState == ReconciliationState.Synchronized
                    || newState == ReconciliationState.SynchronizedAndSubmitted)
            {
                String projectId = ManagerUtil.getParentId(this,
                        reconciliation, messageContext, TABLE_NAME, null,
                        NAME_PROJECT_ID);
                SP_U_PRJ_SUBMITTED_RECON(projectId, messageContext);
            }
            // Triggers a reconciliation state transition.
            SP_U_RECONCILIATION_STATE(reconciliation, messageContext);
        }

        return reconciliation;
        
        */
        
        return null;
    }

//  TODO remove comments to implement ProjectReconciliation
/*    private void loadDoneBy(ProjectReconciliation reconciliation,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        Resource doneBy = new Resource();

        doneBy.setID((String) relatedFields.get(index, ID_REC_USER));

        RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(Resource.class);

        doneBy = (Resource) manager.loadByPrimaryKey(doneBy, null,
                messageContext, isDelta);

        if (isDelta)
        {
            reconciliation.deltaDoneBy(doneBy);
        }
        else
        {
            reconciliation.setDoneBy(doneBy);
        }
    }

    private void loadTimesheetApprovalStatus(
            ProjectReconciliation reconciliation,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        TimesheetApprovalStatus status = new TimesheetApprovalStatus();

        Resource resource = new Resource();
        resource.setID((String) relatedFields.get(index, ID_RESOURCE_ID));
        status.setResource(resource);

        Project project = new Project();
        project.setID((String) relatedFields.get(index, ID_PROJECT_ID));
        status.setProject(project);

        String weekOf = (String) relatedFields.get(index, ID_WEEK_OF);

        if (weekOf != null)
        {
            status.setWeekOf(DateUtil.convertToCalendar(weekOf));
        }

        RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(TimesheetApprovalStatus.class);

        try
        {
            status = (TimesheetApprovalStatus) manager.loadByPrimaryKey(status,
                    null, messageContext, isDelta);
        }
        catch (RPMException e)
        {
            // TODO: replace this try catch with a check on whether or not a
            // timesheet approval status exists for this reconciliation
            // record (ie. do a select)
            status = null;
        }

        if (isDelta)
        {
            reconciliation.deltaTimesheetApprovalStatus(status);
        }
        else
        {
            reconciliation.setTimesheetApprovalStatus(status);
        }
    }

    private void loadReconciliationReason(ProjectReconciliation reconciliation,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        ReconciliationReason reason = new ReconciliationReason();
        reason.setID((String) relatedFields.get(index, ID_REASON_ID));

        if (!reason.getID().equals("0"))
        {
            DatafieldManager manager = (DatafieldManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                            ReconciliationReason.class.getName());

            reason = (ReconciliationReason) manager.loadByPrimaryKey(reason,
                    null, messageContext, isDelta);

            if (isDelta)
            {
                reconciliation.deltaReconciliationReason(reason);
            }
            else
            {
                reconciliation.setReconciliationReason(reason);
            }
        }
    }

    private void loadParent(ProjectReconciliation reconciliation,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        GenericProject parent = new GenericProject();

        parent.setID((String) relatedFields.get(index, ID_PROJECT_ID));

        RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(GenericProject.class);

        parent = (GenericProject) manager.loadByPrimaryKey(parent, null,
                messageContext, isDelta);

        if (isDelta)
        {
            reconciliation.deltaParent(parent);
        }
        else
        {
            reconciliation.setParent(parent);
        }
    }

    private void SP_U_PRJ_SUBMITTED_RECON(String projectId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[3];
        args[0] = projectId;
        args[1] = new Integer(1); // Project has submitted reconciliation
        // record(s).
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, "SP_U_PRJ_SUBMITTED_RECON", args);
    }

    private void SP_U_RECONCILIATION_STATE(
            ProjectReconciliation reconciliation, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Integer transitionCommand = ProjectReconciliationStateUtil
                .getTransitionCommand(reconciliation.getState());

        Object args[] = new Object[3];
        args[0] = reconciliation.getID();
        args[1] = transitionCommand;
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, "SP_U_RECONCILIATION_STATE", args);
    }

*/
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // TODO Implement delete?
    }
    
    
}
