package com.ibm.rpm.financial.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.financial.constants.ValidationConstants;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.scope.TimePhasedFinancialScope;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.financial.util.BudgetTypeToDBMap;
import com.ibm.rpm.financial.util.FinancialBusinessLogicUtil;
import com.ibm.rpm.financial.util.FinancialCategoryUtil;
import com.ibm.rpm.financial.util.FinancialUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.framework.util.ValidateSerializer;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.xpathparser.XPathContainer;

/**
 * Timephased breakdown for Financials Corresponds to: PROJECT_BUDGET_BREAKDOWN
 * Primary key: - ELEMENT_ID (WorkElement or ScopeElement ID,
 * timePhasedFinancial.getParent().getID()) used as TimePhasedFinancial ID -
 * CHARGECODE_ID (timePhasedFinancial.getParent().getChargeCode().getID() -
 * BUDGET_TYPE - BUDGET_YEAR
 * 
 * @author ikusakov
 */

public class TimePhasedFinancialManager extends AbstractRPMObjectManager
{

    private static Log logger = LogFactory
            .getLog(TimePhasedFinancialManager.class);

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(TimePhasedFinancial.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    TimePhasedFinancialManager.class);
        }
    }

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 1;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "PROJECT_BUDGET_BREAKDOWN.PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 2;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "PROJECT_BUDGET_BREAKDOWN.ELEMENT_ID";

    // HACK: TimePhasedFinancial ID is actually a WorkElement ID
    // why? because the framework requires ID and TimePhasedFinancial does not
    // have any
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column CHARGECODE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CHARGECODE_ID = 3;

    public static final int TYPE_CHARGECODE_ID = Types.CHAR;

    public static final String NAME_CHARGECODE_ID = "PROJECT_BUDGET_BREAKDOWN.CHARGECODE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHARGECODE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHARGECODE_ID,NAME_CHARGECODE_ID);}

    /**
     * Column BUDGET_TYPE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BUDGET_TYPE = 4;

    public static final int TYPE_BUDGET_TYPE = Types.SMALLINT;

    public static final String NAME_BUDGET_TYPE = "PROJECT_BUDGET_BREAKDOWN.BUDGET_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_BUDGET_TYPE = "BUDGETTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BUDGET_TYPE, SqlUtil.castAsChar(NAME_BUDGET_TYPE, 2));
    }

    /**
     * Column BUDGET_YEAR of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BUDGET_YEAR = 5;

    public static final int TYPE_BUDGET_YEAR = Types.SMALLINT;

    public static final String NAME_BUDGET_YEAR = "PROJECT_BUDGET_BREAKDOWN.BUDGET_YEAR";

    public static final String PROPERTY_BUDGET_YEAR = "YEAR";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BUDGET_YEAR, NAME_BUDGET_YEAR);
    }

    /**
     * Column MONTH_01_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_01_AMOUNT = 6;

    public static final int TYPE_MONTH_01_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_01_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_01_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_01_AMOUNT = "MONTH01VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_01_AMOUNT, NAME_MONTH_01_AMOUNT);
    }

    /**
     * Column MONTH_02_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_02_AMOUNT = 7;

    public static final int TYPE_MONTH_02_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_02_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_02_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_02_AMOUNT = "MONTH02VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_02_AMOUNT, NAME_MONTH_02_AMOUNT);
    }

    /**
     * Column MONTH_03_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_03_AMOUNT = 8;

    public static final int TYPE_MONTH_03_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_03_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_03_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_03_AMOUNT = "MONTH03VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_03_AMOUNT, NAME_MONTH_03_AMOUNT);
    }

    /**
     * Column MONTH_04_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_04_AMOUNT = 9;

    public static final int TYPE_MONTH_04_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_04_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_04_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_04_AMOUNT = "MONTH04VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_04_AMOUNT, NAME_MONTH_04_AMOUNT);
    }

    /**
     * Column MONTH_05_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_05_AMOUNT = 10;

    public static final int TYPE_MONTH_05_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_05_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_05_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_05_AMOUNT = "MONTH05VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_05_AMOUNT, NAME_MONTH_05_AMOUNT);
    }

    /**
     * Column MONTH_06_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_06_AMOUNT = 11;

    public static final int TYPE_MONTH_06_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_06_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_06_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_06_AMOUNT = "MONTH06VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_06_AMOUNT, NAME_MONTH_06_AMOUNT);
    }

    /**
     * Column MONTH_07_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_07_AMOUNT = 12;

    public static final int TYPE_MONTH_07_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_07_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_07_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_07_AMOUNT = "MONTH07VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_07_AMOUNT, NAME_MONTH_07_AMOUNT);
    }

    /**
     * Column MONTH_08_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_08_AMOUNT = 13;

    public static final int TYPE_MONTH_08_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_08_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_08_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_08_AMOUNT = "MONTH08VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_08_AMOUNT, NAME_MONTH_08_AMOUNT);
    }

    /**
     * Column MONTH_09_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_09_AMOUNT = 14;

    public static final int TYPE_MONTH_09_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_09_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_09_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_09_AMOUNT = "MONTH09VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_09_AMOUNT, NAME_MONTH_09_AMOUNT);
    }

    /**
     * Column MONTH_10_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_10_AMOUNT = 15;

    public static final int TYPE_MONTH_10_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_10_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_10_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_10_AMOUNT = "MONTH10VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_10_AMOUNT, NAME_MONTH_10_AMOUNT);
    }

    /**
     * Column MONTH_11_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_11_AMOUNT = 16;

    public static final int TYPE_MONTH_11_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_11_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_11_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_11_AMOUNT = "MONTH11VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_11_AMOUNT, NAME_MONTH_11_AMOUNT);
    }

    /**
     * Column MONTH_12_AMOUNT of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_MONTH_12_AMOUNT = 17;

    public static final int TYPE_MONTH_12_AMOUNT = Types.DECIMAL;

    public static final String NAME_MONTH_12_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.MONTH_12_AMOUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_MONTH_12_AMOUNT = "MONTH12VALUE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONTH_12_AMOUNT, NAME_MONTH_12_AMOUNT);
    }

    /**
     * Column TOTAL_AMOUNT of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_TOTAL_AMOUNT = 18;

    public static final int TYPE_TOTAL_AMOUNT = Types.DECIMAL;

    public static final String NAME_TOTAL_AMOUNT = "PROJECT_BUDGET_BREAKDOWN.TOTAL_AMOUNT";

    public static final String PROPERTY_TOTAL_AMOUNT = "TOTALFORYEAR";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TOTAL_AMOUNT, NAME_TOTAL_AMOUNT);
    }

    /**
     * Column PROCESS_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_PROCESS_FLAG = 19;

    public static final int TYPE_PROCESS_FLAG = Types.CHAR;

    public static final String NAME_PROCESS_FLAG = "PROJECT_BUDGET_BREAKDOWN.PROCESS_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROCESS_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROCESS_FLAG,NAME_PROCESS_FLAG);}

    public static final String TABLE_NAME = "PROJECT_BUDGET_BREAKDOWN";

    /**
     * Create an array of type string containing all the fields of the
     * PROJECT_BUDGET_BREAKDOWN table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_PROJECT_ID, NAME_ELEMENT_ID, NAME_CHARGECODE_ID,
            NAME_BUDGET_TYPE, NAME_BUDGET_YEAR, NAME_MONTH_01_AMOUNT,
            NAME_MONTH_02_AMOUNT, NAME_MONTH_03_AMOUNT, NAME_MONTH_04_AMOUNT,
            NAME_MONTH_05_AMOUNT, NAME_MONTH_06_AMOUNT, NAME_MONTH_07_AMOUNT,
            NAME_MONTH_08_AMOUNT, NAME_MONTH_09_AMOUNT, NAME_MONTH_10_AMOUNT,
            NAME_MONTH_11_AMOUNT, NAME_MONTH_12_AMOUNT, NAME_TOTAL_AMOUNT,
            NAME_PROCESS_FLAG
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new TimePhasedFinancialManager instance.
     * 
     * @return TimePhasedFinancialManager
     */
    public TimePhasedFinancialManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    // override
    public String[] getPrimaryKeyArray(RPMObject rpmObject) throws RPMException
    {

        // we ignore PROJECT_ID, because ELEMENT_ID is system-unique
        return new String[]
        {
                NAME_ELEMENT_ID, NAME_CHARGECODE_ID, NAME_BUDGET_TYPE,
                NAME_BUDGET_YEAR
        };
    }

    /**
     * override
     */
    public Object[] getPrimaryKeyValues(RPMObject rpmObject)
    {
        TimePhasedFinancial timephased = (TimePhasedFinancial) rpmObject;
        
        // It's possible that the chargeCodeId is null because it's not set
        // in setPrimaryKeyFields
        String chargeCodeID = null;
        
        if (rpmObject.getParent() != null
                && ((AbstractFinancial) rpmObject.getParent()).getChargeCode() != null
                && ((AbstractFinancial) rpmObject.getParent()).getChargeCode()
                        .getID() != null)
        {
            chargeCodeID = ((AbstractFinancial) rpmObject.getParent())
                    .getChargeCode().getID();
        }

        // budgetType mandatory; verified in isValidPrimaryKey
        Integer budgetType = BudgetTypeToDBMap.getIntegerBudgetType(timephased
            .getParent() instanceof WorksheetFinancial, timephased
            .getBudgetType());
        
        return new Object[]
        {
                rpmObject.getID(), // this must be null for new objects
                chargeCodeID, budgetType, timephased.getYear()
        };
    }
    
    public void isValidPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        TimePhasedFinancial timephased = (TimePhasedFinancial) rpmObject;
        StringBuffer invalidPrimaryKeys = new StringBuffer();

        if (timephased.getParent() == null
                || ((AbstractFinancial) timephased.getParent()).getChargeCode() == null
                || ((AbstractFinancial) timephased.getParent()).getChargeCode()
                    .getID() == null)
        {
            invalidPrimaryKeys.append(ValidationConstants.PARENT_CHARGE_CODE_FIELD
                    + " ID, ");
        }

        if (timephased.getBudgetType() == null)
        {
            invalidPrimaryKeys.append(ValidationConstants.BUDGET_TYPE_FIELD);
        }

        if (invalidPrimaryKeys.length() > 0)
        {
            if (messageContext.isRootObject(rpmObject))
            {
                ExceptionUtil.handleMissingPrimaryKey(this, rpmObject,
                    invalidPrimaryKeys.toString());
            }
            else
            {
                String args[] =
                {
                        "Invalid primary key, missing fields: " + invalidPrimaryKeys.toString()
                };
                throw new RPMException(ErrorCodes.INTERNAL_ERROR, args);
            }
        }
    }

    /**
     * @param primaryKey
     * @param rpmObject
     *            sets the fields that are part of the primaryKey Override if
     *            your primary contains more than one field
     */
    protected RPMObject setPrimaryKeyFields(Object[] primaryKey,
            RPMObject rpmObject)
    {
        ((TimePhasedFinancial) rpmObject).setID((String) primaryKey[0]);

        // Ignoring ChargeCodeID in primaryKey[1]

        ((TimePhasedFinancial) rpmObject).setBudgetType(BudgetTypeToDBMap
                .getBudgetTypeFromInt(Integer.valueOf((String) primaryKey[2])));

        ((TimePhasedFinancial) rpmObject).setYear(Integer
                .valueOf((String) primaryKey[3]));

        return rpmObject;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        // TimePhasedFinancial has no children
        if (manager == null)
        {
            // XPath queries like:
            // /WorksheetFinancial/TimePhasedFinancial

            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof FinancialManager)
        {

            // XPath queries like:
            // /WorksheetFinancial[TimePhasedFinancial[Total=1000]]

            joinCondition = new JoinCondition();

            // adding PROJECT_BUDGET_BREAKDOWN table to the condition
            joinCondition.setTableName(getTableName());
            StringBuffer where = new StringBuffer();
            where.append(FinancialManager.NAME_ELEMENT_ID);
            where.append(" = ");
            where.append(NAME_ELEMENT_ID);
            where.append(" AND ");
            where.append(FinancialManager.NAME_CHARGECODE_ID);
            where.append(" = ");
            where.append(NAME_CHARGECODE_ID);
            where.append(" AND ");
            where.append(TimePhasedFinancialManager.NAME_BUDGET_TYPE);
            where.append(" = ");
            where.append(BudgetTypeToDBMap.getIntegerBudgetTypeFromFieldname(
                container.getType(), container.getPreviousFieldName()));

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        return new Integer(0);
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;

        switch (typeId)
        {
        case 0:
            object = new TimePhasedFinancial();
            break;
        default:
            RPMException e = new RPMException("TypeId " + typeId
                    + " is not supported");
            logger.error(e);
            throw e;
        }
        return object;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        TimePhasedFinancial tpf = (TimePhasedFinancial) rpmObject;

        if (isDelta)
        {
            tpf.deltaMonth01Value(new Double(rs.getDouble(ID_MONTH_01_AMOUNT)));
            tpf.deltaMonth02Value(new Double(rs.getDouble(ID_MONTH_02_AMOUNT)));
            tpf.deltaMonth03Value(new Double(rs.getDouble(ID_MONTH_03_AMOUNT)));
            tpf.deltaMonth04Value(new Double(rs.getDouble(ID_MONTH_04_AMOUNT)));
            tpf.deltaMonth05Value(new Double(rs.getDouble(ID_MONTH_05_AMOUNT)));
            tpf.deltaMonth06Value(new Double(rs.getDouble(ID_MONTH_06_AMOUNT)));
            tpf.deltaMonth07Value(new Double(rs.getDouble(ID_MONTH_07_AMOUNT)));
            tpf.deltaMonth08Value(new Double(rs.getDouble(ID_MONTH_08_AMOUNT)));
            tpf.deltaMonth09Value(new Double(rs.getDouble(ID_MONTH_09_AMOUNT)));
            tpf.deltaMonth10Value(new Double(rs.getDouble(ID_MONTH_10_AMOUNT)));
            tpf.deltaMonth11Value(new Double(rs.getDouble(ID_MONTH_11_AMOUNT)));
            tpf.deltaMonth12Value(new Double(rs.getDouble(ID_MONTH_12_AMOUNT)));
        }
        else
        {
            tpf.setMonth01Value(new Double(rs.getDouble(ID_MONTH_01_AMOUNT)));
            tpf.setMonth02Value(new Double(rs.getDouble(ID_MONTH_02_AMOUNT)));
            tpf.setMonth03Value(new Double(rs.getDouble(ID_MONTH_03_AMOUNT)));
            tpf.setMonth04Value(new Double(rs.getDouble(ID_MONTH_04_AMOUNT)));
            tpf.setMonth05Value(new Double(rs.getDouble(ID_MONTH_05_AMOUNT)));
            tpf.setMonth06Value(new Double(rs.getDouble(ID_MONTH_06_AMOUNT)));
            tpf.setMonth07Value(new Double(rs.getDouble(ID_MONTH_07_AMOUNT)));
            tpf.setMonth08Value(new Double(rs.getDouble(ID_MONTH_08_AMOUNT)));
            tpf.setMonth09Value(new Double(rs.getDouble(ID_MONTH_09_AMOUNT)));
            tpf.setMonth10Value(new Double(rs.getDouble(ID_MONTH_10_AMOUNT)));
            tpf.setMonth11Value(new Double(rs.getDouble(ID_MONTH_11_AMOUNT)));
            tpf.setMonth12Value(new Double(rs.getDouble(ID_MONTH_12_AMOUNT)));

            // read-only

            // HACK: TimePhasedFinancial ID is actually a WorkElement ID
            // why? because the framework requires ID and TimePhasedFinancial
            // does not have any
            tpf.setID(rs.getString(ID_ELEMENT_ID));

            tpf
                    .setBudgetType(BudgetTypeToDBMap
                            .getBudgetTypeFromInt(new Integer(rs
                                    .getInt(ID_BUDGET_TYPE))));
            tpf.setYear(new Integer(rs.getInt(ID_BUDGET_YEAR)));
            tpf.setTotalForYear(new Double(rs.getDouble(ID_TOTAL_AMOUNT)));
        }

        return tpf;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        relatedFields = super.decodeRelatedFields(resultSet, relatedFields,
                index);

        // we need it later to identify parent type: WorksheetFinancial or
        // WBSFinancial
        relatedFields.put(index, ID_BUDGET_TYPE, new Integer(resultSet
                .getInt(ID_BUDGET_TYPE)));

        relatedFields.put(index, ID_CHARGECODE_ID, resultSet
                .getString(ID_CHARGECODE_ID));

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        // Ignoring isDelta, because parent cannot be modified
        if (rpmScope != null && !isDelta)
        {
            if (rpmScope.getParent() != null
                    && rpmObject.getParent() == null)
            {
                // determine parent type by BUDGET_TYPE
                boolean isWorksheet = BudgetTypeToDBMap
                        .isWorksheetBudgetType((Integer) relatedFields.get(
                                index, ID_BUDGET_TYPE));

                FinancialManager financialManager = (FinancialManager) RPMManagerFactory
                        .getInstance().getRPMObjectManager(
                                isWorksheet ? WorksheetFinancial.class
                                        .getName() : WbsFinancial.class
                                        .getName());

                Object args[] = new Object[2];
                args[0] = rpmObject.getID();
                args[1] = relatedFields.get(index, ID_CHARGECODE_ID);

                ArrayList results = financialManager.loadByForeignKey(null,
                    rpmScope.getParent(), messageContext, this, args, null,
                    null, isDelta);

                if (results == null || results.size() != 1)
                {
                    ExceptionUtil.handleException(this, new RPMException(
                            "1 financial should be found for element: "
                                    + rpmObject.getID()
                                    + " and ChargeCode "
                                    + relatedFields
                                            .get(index, ID_CHARGECODE_ID)));
                }

                rpmObject.setParent((AbstractFinancial) results.get(0));
            }
        }

        return rpmObject;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof FinancialManager)
        {
            joinCondition = new JoinCondition();

            /*
             * loading from PROJECT_BUDGET_BREAKDOWN by:
             * 
             * ELEMENT_ID, CHARGECODE_ID, BUDGET_TYPE
             * 
             * it means all BUDGET_YEAR will be fetched
             */
            StringBuffer where = new StringBuffer();

            where.append(NAME_ELEMENT_ID);
            where.append(" = ? AND ");
            where.append(NAME_CHARGECODE_ID);
            where.append(" = ? AND ");
            where.append(NAME_BUDGET_TYPE);
            where.append(" = ?");

            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());
            }

            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // no SP call - Update will be called next by the framework and it will
        // take care of insert

        rpmObject.setModified();
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        RPMObjectManager manager = null;
        AbstractFinancial parent = null;

        TimePhasedFinancial timePhasedFinancial = (TimePhasedFinancial) rpmObject;
        
        
        if (timePhasedFinancial.testMonth01ValueModified()
                || timePhasedFinancial.testMonth02ValueModified()
                || timePhasedFinancial.testMonth03ValueModified()
                || timePhasedFinancial.testMonth04ValueModified()
                || timePhasedFinancial.testMonth05ValueModified()
                || timePhasedFinancial.testMonth06ValueModified()
                || timePhasedFinancial.testMonth07ValueModified()
                || timePhasedFinancial.testMonth08ValueModified()
                || timePhasedFinancial.testMonth09ValueModified()
                || timePhasedFinancial.testMonth10ValueModified()
                || timePhasedFinancial.testMonth11ValueModified()
                || timePhasedFinancial.testMonth12ValueModified())
        {          
            // if the timephased container is being saved directly (not through parent)
            // and if the parent is of type WorksheetFinancial, then we need to update
            // the worksheet totals based on the timephased values
            ValidateSerializer vs = messageContext.getObjectsToValidate();
            if (vs == null
                    || !vs.before(timePhasedFinancial.getParent(),
                        timePhasedFinancial))
            {
                if (timePhasedFinancial.getParent() instanceof WorksheetFinancial ||
                        timePhasedFinancial.getParent() instanceof WbsFinancial)
                {
                    TimePhasedFinancial reloaded = null;

                    if (timePhasedFinancial.getID() != null)
                    {
                        // if the object exists, load the existing container
                        reloaded = (TimePhasedFinancial) load(
                            timePhasedFinancial, rpmScope, messageContext);
                    }

                    // determine the difference in monthly amounts,
                    // then deduct or add the amount to the worksheet total
                    double difference = FinancialUtil.calculateTotalDifference(
                        timePhasedFinancial, reloaded);

                    manager = getManagerInstance(timePhasedFinancial
                        .getParent().getClass().getName());                    

                    // we want to reload the parent because any changes made to
                    // the parent
                    // when the timephasedFinancial container is saved directly
                    // should not be saved
                    FinancialScope fnScope = new FinancialScope();
                    fnScope.setChargeCode(true);
                    fnScope.setActualTimePhased(true);
                    fnScope.setEstimateTimePhased(true);
                    //fnScope.setParent(new WorkElementScope());
                    
                    //We should be carefull since the parent might be a ScopeElement or a workElement
                    if (timePhasedFinancial.getParent() != null
                            && timePhasedFinancial.getParent().getParent() != null)
                    {
                        if (timePhasedFinancial.getParent().getParent() instanceof ScopeElement)
                        {
                            fnScope.setParent(new ScopeElementScope());
                        }
                        else if (timePhasedFinancial.getParent().getParent() instanceof WorkElement)
                        {
                            fnScope.setParent(new WorkElementScope());
                        }
                    }
                    
                    parent = (AbstractFinancial) manager.load(
                        timePhasedFinancial.getParent(), fnScope,
                        messageContext);
                    
                    //Since the Timephased isaved directly, we must re-calculate the
                    // Worksheet matrix since
                    //as mentionned above the the timephased values are going to roll-back
                    // to the
                    //worksheet
                    
                    //Case: we are saving a timephase directly
                    if ((rpmScope != null)
                            && (rpmScope instanceof TimePhasedFinancialScope)
                            && (rpmScope.getParent() != null)
                            && (rpmScope.getParent() instanceof FinancialScope))
                    {
                        updateWorksheetFinancialsMatrix(rpmScope,
                            messageContext, parent, timePhasedFinancial,
                            manager, difference);
                    }
                    else //Case: when we are saving a parent that has finanancials and timpehased
                    {
                        updateWorsheetValues(messageContext,
                            timePhasedFinancial, manager, parent,
                            difference,true);
                    }
                }
            }

            
            // update the timephasedFinancial (must occur after above reload)
            timePhasedFinancial.setID(SP_U_PBB(timePhasedFinancial, messageContext));
        }
        
        return timePhasedFinancial;
    }
    

    //If we only have timephase for the current year, we need to replace the
    //worksheet with that value. Otherwise we add it the current total of
    //worksheet.
    //We need to remember that there are two ways for that code to get called:
    //1- we are saving a proposal with worksheet financials
    //2- we are saving a worksheet financial
    //3- we are saving a timephased
    //
    // 1&2 will call FinancialManager.UpdateObject()
    // 3 will call TimePhasedManager.UpdateObject()
    private void updateWorsheetValues(MessageContext messageContext,
            TimePhasedFinancial timePhasedFinancial,
            RPMObjectManager manager,
            AbstractFinancial parent, double difference, boolean savingWorksheet)
            throws SQLException, RPMException, ParseException
    {
        boolean isCurrentYear = (FinancialUtil.getCurrentYear() == FinancialUtil
            .nullToZero(timePhasedFinancial.getYear()).intValue());

        if (timePhasedFinancial.getBudgetType() == BudgetType.Actual)
        {
            //case: we have only timephase for current year only
            if (parent.getActualTimePhased() != null
                    && parent.getActualTimePhased().length <= 1
                    && isCurrentYear)

            {
                parent.setActual(new Double(difference));
            }
            //case: we have timephase for more years than the current one
            else if (FinancialUtil
                .calculateTimephasedActualTotal(timePhasedFinancial) > 0
                    || !savingWorksheet)
            {
                parent.setActual(new Double(FinancialUtil.nullToZero(
                    parent.getActual()).doubleValue()
                        + difference));
            }
        }
        else if (timePhasedFinancial.getBudgetType() == BudgetType.Estimate)
        {
            //case: we have only timephase for current year only
            if (parent.getEstimateTimePhased() != null
                    && parent.getEstimateTimePhased().length <= 1
                    && isCurrentYear)

            {
                parent.setEstimate(new Double(difference));
            }
            //case: we have timephase for more years than the current one
            else if (FinancialUtil
                .calculateTimephasedEstimateTotal(timePhasedFinancial) > 0
                    || !savingWorksheet)
            {
                parent.setEstimate(new Double(FinancialUtil.nullToZero(
                    parent.getEstimate()).doubleValue()
                        + difference));
            }
        }

        //Update the rest of the line
        double estimateMinusActual = FinancialUtil.nullToZero(
            parent.getEstimate()).doubleValue()
                - FinancialUtil.nullToZero(parent.getActual()).doubleValue();

        parent.setRemaining(new Double(estimateMinusActual));

        parent.setETC(estimateMinusActual > 0 ? new Double(estimateMinusActual)
                : new Double(0));

        parent.setEAC(new Double(FinancialUtil.nullToZero(parent.getETC())
            .doubleValue()
                + FinancialUtil.nullToZero(parent.getActual()).doubleValue()));
        

        // we are not saving FTE anymore
        if (!FinancialUtil.isFTEFinancial(parent))
        {
            if(timePhasedFinancial.getParent() instanceof WorksheetFinancial)
            {
                ((WorksheetFinancialManager)manager).SP_U_BUDGET(parent, messageContext);
            }
            else if(timePhasedFinancial.getParent() instanceof WbsFinancial)
            {
                ((WbsFinancialManager)manager).SP_U_BUDGET(parent, messageContext);
            }
        }
    }
    
    // This Method will update the whole worksheet financial matrix if a valid
    // system
    // chargeCode is found otherwise it will only update the worksheet financial
    // at hand.
    private void updateWorksheetFinancialsMatrix(RPMObjectScope rpmScope,
            MessageContext messageContext, AbstractFinancial parent,
            TimePhasedFinancial timePhasedFinancial,
            RPMObjectManager finMan, double difference) throws RPMException,
            ParseException, SQLException
    {
        WorkElement workElementGrandParent;
        AggregateNode grandparent;

            FinancialScope parentScope = (FinancialScope) rpmScope.getParent();
            boolean isProperlyTimephased = (parentScope.isActualTimePhased() || parentScope
                .isEstimateTimePhased());

            if (isProperlyTimephased && parent != null
                    && (parent instanceof WorksheetFinancial || parent instanceof WbsFinancial))
            {
                //we need to load the worksheet financial parent (grandParent of the timephased)
                //so we can get all 4 worksheets required to do the calculations.
                FinancialScope financialScope = new FinancialScope();
                financialScope.setParent(new RPMObjectScope());

                FinancialScope childScope = new FinancialScope();
                childScope.setChargeCode(true);

                WorkElementScope wScope = new WorkElementScope();
                wScope.setWorksheetFinancials(childScope);
                wScope.setWbsFinancials(childScope);

                WbsManager manager = (WbsManager) getManagerInstance(WorkElement.class
                    .getName());
                

                workElementGrandParent = (WorkElement) manager.load(
                    timePhasedFinancial.getParent().getParent(), wScope,
                    messageContext);

                grandparent = (AggregateNode) workElementGrandParent;

                //Validating that we have indeed systemchagecodes
                if (grandparent != null
                        && (grandparent.getWorksheetFinancials() != null||grandparent.getWbsFinancials()!=null)
                        && (FinancialCategoryUtil
                            .isValidSystemExpenseChargeCodeCategory(grandparent
                                .getWorksheetFinancials()) ||
                                FinancialCategoryUtil.isValidSystemExpenseChargeCodeCategory(grandparent.getWbsFinancials())))
                {
                    //for some odd reasons when we are saving a timephased directly
                    //that has a valid system chargecode we should not add the total of the 
                    //timephased to the worksheetValues but replace them.(UI style)
                    if (timePhasedFinancial.getBudgetType() == BudgetType.Actual)
                    {
                        parent.setActual(new Double(difference));
                    }
                    else if (timePhasedFinancial.getBudgetType() == BudgetType.Estimate)
                    {
                        parent.setEstimate(new Double(difference));
                    }                    
                    
                    double estimateMinusActual = FinancialUtil.nullToZero(
                        parent.getEstimate()).doubleValue()
                            - FinancialUtil.nullToZero(parent.getActual())
                                .doubleValue();
  
                    parent.setETC(estimateMinusActual > 0 ? new Double(
                        estimateMinusActual) : new Double(0));

                    
                    //we need to get the changes that was made to the current worksheet
                    //and replace the DB version with it and do the matrix calculations.
                    //If we don't do that the SPs are going to calculate Estimate and Actuals
                    //with the wrong values so we must do it before saving.
                    
                    AbstractFinancial[] financials = null;
                    if(timePhasedFinancial.getParent() instanceof WorksheetFinancial)
                    {
                         financials = grandparent.getWorksheetFinancials();                         

                    }
                    else if(timePhasedFinancial.getParent() instanceof WbsFinancial)
                    {
                        financials = grandparent.getWbsFinancials();
                    }

                    AbstractFinancial reloadWF_EL = FinancialUtil
                    .getMatchingWorksheetFinancial(parent.getChargeCode()
                        .getID(), financials);

                    
                    //updating bd version of current worksheet with new timephased values
                    if (reloadWF_EL != null)
                    {
                        reloadWF_EL.setActual(parent.getActual());
                        reloadWF_EL.setEstimate(parent.getEstimate());
                        reloadWF_EL.setRemaining(parent.getRemaining());
                        reloadWF_EL.setETC(parent.getETC());
                        reloadWF_EL.setEAC(parent.getEAC());
                        reloadWF_EL.setEACVariance(parent.getEACVariance());
                    }

                    FinancialBusinessLogicUtil.saveUpdateTimephasedToBD(financials,
                        messageContext, getManagerInstance(financials[0].getClass()
                            .getName()));

                }
                else //saving a timephased that is not for a systemChargecode so we do same basic
                    //calculations.
                {
                    updateWorsheetValues(messageContext,
                        timePhasedFinancial, finMan, parent,
                        difference,false);
                }
            }
            
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // No deletion SP, rows are deleted with the project itself
    }

    private Double null2zero(Double d)
    {
        return d == null ? new Double(0) : d;
    }

    public String SP_U_PBB(TimePhasedFinancial timePhasedFinancial,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_PBB";

        Object args[] = new Object[17];

        // project_id (it is actually an element_id)
        String elementID = timePhasedFinancial.getID();
        if (elementID == null)
        {
            // either the timephased id or the parent's parent id are mandatory (see checkpoint)
            elementID = timePhasedFinancial.getParent().getParent().getID();
        }

        args[0] = elementID;

        // budget_year
        args[1] = timePhasedFinancial.getYear() == null ? new Integer(0)
                : timePhasedFinancial.getYear();

        // the parent's charge code id is mandatory (see checkpoint)
        args[2] = ((AbstractFinancial) timePhasedFinancial.getParent())
            .getChargeCode().getID();

        // budget_type
        args[3] = BudgetTypeToDBMap.getIntegerBudgetType(timePhasedFinancial
                .getParent() instanceof WorksheetFinancial, timePhasedFinancial
                .getBudgetType());

        // month_01_amount
        args[4] = null2zero(timePhasedFinancial.getMonth01Value());

        // month_02_amount
        args[5] = null2zero(timePhasedFinancial.getMonth02Value());

        // month_03_amount
        args[6] = null2zero(timePhasedFinancial.getMonth03Value());

        // month_04_amount
        args[7] = null2zero(timePhasedFinancial.getMonth04Value());

        // month_05_amount
        args[8] = null2zero(timePhasedFinancial.getMonth05Value());

        // month_06_amount
        args[9] = null2zero(timePhasedFinancial.getMonth06Value());

        // month_07_amount
        args[10] = null2zero(timePhasedFinancial.getMonth07Value());

        // month_08_amount
        args[11] = null2zero(timePhasedFinancial.getMonth08Value());

        // month_09_amount
        args[12] = null2zero(timePhasedFinancial.getMonth09Value());

        // month_10_amount
        args[13] = null2zero(timePhasedFinancial.getMonth10Value());

        // month_11_amount
        args[14] = null2zero(timePhasedFinancial.getMonth11Value());

        // month_12_amount
        args[15] = null2zero(timePhasedFinancial.getMonth12Value());

        args[16] = getUser(messageContext).getID();

        return executeProcedure(messageContext, procedureName, args);
    }
}
