package com.ibm.rpm.financial.managers;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.financial.containers.AbstractTimeCode;
import com.ibm.rpm.financial.containers.GenericTimeCode;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeCategory;
import com.ibm.rpm.financial.containers.TimeCodeGroup;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ArrayUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.timesheet.managers.StepManager;
import com.ibm.rpm.timesheet.managers.SummaryTimesheetManager;
import com.ibm.rpm.xpathparser.XPathContainer;

public class TimecodeManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(TimeCode.class.getName());
        CONTAINERS.add(TimeCodeCategory.class.getName());
        CONTAINERS.add(TimeCodeGroup.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                TimecodeManager.class);
        }
    }
    
    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;
    public static final int TYPE_LEVEL_ID = Types.INTEGER;
    public static final String NAME_LEVEL_ID = "TMT_TIMECODES.LEVEL_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;
    public static final int TYPE_TYPE_ID = Types.INTEGER;
    public static final String NAME_TYPE_ID = "TMT_TIMECODES.TYPE_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;
    public static final int TYPE_MAX_RANK = Types.INTEGER;
    public static final String NAME_MAX_RANK = "TMT_TIMECODES.MAX_RANK";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;
    public static final int TYPE_CHILD_COUNT = Types.INTEGER;
    public static final String NAME_CHILD_COUNT = "TMT_TIMECODES.CHILD_COUNT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;
    public static final int TYPE_DELETED_COUNT = Types.INTEGER;
    public static final String NAME_DELETED_COUNT = "TMT_TIMECODES.DELETED_COUNT";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "TMT_TIMECODES.ELEMENT_ID";
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;
    public static final int TYPE_ELEMENT_NAME = Types.CHAR;
    public static final String NAME_ELEMENT_NAME = "TMT_TIMECODES.ELEMENT_NAME";
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;
    public static final int TYPE_PARENT_ID = Types.CHAR;
    public static final String NAME_PARENT_ID = "TMT_TIMECODES.PARENT_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 9;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "TMT_TIMECODES.REC_USER";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 10;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "TMT_TIMECODES.REC_STATUS";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    public static final String PROPERTY_REC_STATUS = "REC_STATUS";
    static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 11;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "TMT_TIMECODES.REC_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column DEFAULT_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_DEFAULT_FLAG = 12;
    public static final int TYPE_DEFAULT_FLAG = Types.CHAR;
    public static final String NAME_DEFAULT_FLAG = "TMT_TIMECODES.DEFAULT_FLAG";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_DEFAULT_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_FLAG,NAME_DEFAULT_FLAG);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 13;
    public static final int TYPE_RANK = Types.VARCHAR;
    public static final String NAME_RANK = "TMT_TIMECODES.RANK";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column FS_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FS_CODE = 14;
    public static final int TYPE_FS_CODE = Types.VARCHAR;
    public static final String NAME_FS_CODE = "TMT_TIMECODES.FS_CODE";
    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_FS_CODE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_FS_CODE,NAME_FS_CODE);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 15;
    public static final int TYPE_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_DESCRIPTION = "TMT_TIMECODES.DESCRIPTION";
    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    public static final String TABLE_NAME = "TMT_TIMECODES";

    /**
     * Create an array of type string containing all the fields of the TMT_TIMECODES table.
     */
    private static final String[] FIELD_NAMES = 
    {
         NAME_LEVEL_ID
        ,NAME_TYPE_ID
        ,NAME_MAX_RANK
        ,NAME_CHILD_COUNT
        ,NAME_DELETED_COUNT
        ,NAME_ELEMENT_ID
        ,NAME_ELEMENT_NAME
        ,NAME_PARENT_ID
        ,NAME_REC_USER
        ,NAME_REC_STATUS
        ,NAME_REC_DATETIME
        ,NAME_DEFAULT_FLAG
        ,NAME_RANK
        ,NAME_FS_CODE
        ,NAME_DESCRIPTION
    };

    private static final String ALL_FIELDS;
    static{
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    };

    /**
     * Creates a new TimecodeManager instance.
     * @return TimecodeManager
     */   
    public TimecodeManager()
    {
        super();
    }

    public String getTableName() 
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields() 
    {
        return ALL_FIELDS;
    }  

    public String getPrimaryKey() 
    {  
        return NAME_ELEMENT_ID;
    }  

    protected HashMap getPropertiesMap() 
    {
        return FIELDPROPERTYMAP;
    }  
    
    public String getFilter()
    {
        SqlBuffer sb = new SqlBuffer();
        sb.append(NAME_REC_STATUS + " != 'D'");
        sb.appendAnd();
        sb.append(NAME_ELEMENT_NAME + " IS NOT NULL");
        
        return sb.toString();
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition,
            int level, String containerName, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        
        int typeId = 0;        
        if(containerName.equalsIgnoreCase(StringUtil.getShortClassName(TimeCodeCategory.class)))
        {
            typeId = TimeCodeCategory.TYPE_ID;
        }
        else if(containerName.equalsIgnoreCase(StringUtil.getShortClassName(TimeCodeGroup.class)))
        {
            typeId = TimeCodeGroup.TYPE_ID;
        }
        else if(containerName.equalsIgnoreCase(StringUtil.getShortClassName(TimeCode.class)))
        {
            typeId = TimeCode.TYPE_ID;
        }
        
        if (manager == null)
        {
            joinCondition = new JoinCondition(getTableName());
            if (typeId != 0)
            {
                joinCondition.appendEqual(NAME_TYPE_ID, typeId);
            }
            joinCondition.appendCondition(getFilter());
            joinCondition.appendCondition(condition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    
    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        
        if (manager instanceof SummaryTimesheetManager)
        {
            if ("timecode1".equalsIgnoreCase(container.getPreviousFieldName()))
            {
                joinCondition = JoinCondition.createSubSelect(
                    SummaryTimesheetManager.NAME_TIMECODE_1_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, getFilter());
            }
            else
            {
                joinCondition = JoinCondition.createSubSelect(
                    SummaryTimesheetManager.NAME_TIMECODE_2_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, getFilter());
            }
        }
        else if (manager instanceof StepManager)
        {
            if ("timecode1".equalsIgnoreCase(container.getPreviousFieldName()))
            {
                joinCondition = JoinCondition.createSubSelect(
                    StepManager.NAME_TIMECODE_1_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, getFilter());
            }
            else
            {
                joinCondition = JoinCondition.createSubSelect(
                    StepManager.NAME_TIMECODE_2_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, getFilter());
            }
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return new Integer(typeId);
    }  

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = null;

        switch (typeId)
        {
        	case TimeCode.TYPE_ID:
        	    object = new TimeCode();
            	break;
        	case TimeCodeCategory.TYPE_ID:
        	    object = new TimeCodeCategory();
            	break;
        	case TimeCodeGroup.TYPE_ID:
        	    object = new TimeCodeGroup();
            	break;
        	default:
        	    ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }
        return object;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        AbstractTimeCode timeCode = (AbstractTimeCode) rpmObject;
        
        boolean deleted = false;
        
        
        if (rs.getString(ID_REC_STATUS).equals("D"))
            deleted = true;
        
        String name = rs.getString(ID_ELEMENT_NAME);
        if (name != null)
            name = name.trim();

        if (isDelta)
        {
            timeCode.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
            timeCode.deltaDescription(rs.getString(ID_DESCRIPTION));
            timeCode.deltaName(name);
        }
        else
        {
            timeCode.setID(rs.getString(ID_ELEMENT_ID));
            timeCode.setDeleted(new Boolean(deleted));
            timeCode.setDescription(rs.getString(ID_DESCRIPTION));
            timeCode.setName(name);
        }
        
        return rpmObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
    	 String parentId = resultSet.getString(ID_PARENT_ID);
         relatedFields.put(index, ID_PARENT_ID, parentId);
         
         return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
    	if (rpmScope != null)
        {
            TimeCodeScope scope = (TimeCodeScope) rpmScope;

            if (scope.getParent() != null
                    && !(rpmObject instanceof TimeCodeGroup))
            {
                loadParent(rpmObject, scope, messageContext,
                    relatedFields, index, isDelta);
            }

            if (scope.getChildren() != null && !(rpmObject instanceof TimeCode))
            {
                GenericTimeCode[] categories = null;
                if (rpmObject instanceof TimeCodeCategory)
                {
                    categories = ((TimeCodeCategory) rpmObject)
                        .getChildren();
                }
                else
                {
                    categories = ((TimeCodeGroup) rpmObject).getChildren();
                }
                
                loadChildren(rpmObject, categories, scope.getChildren(), 
                    messageContext, isDelta);
            }
        }
		return rpmObject;
    }
    
    private void loadParent(RPMObject rpmObject,
            TimeCodeScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String oldParentId = (String) relatedFields.get(index, ID_PARENT_ID);
        RPMObject newParent = null;
        if (isDelta)
        {
            newParent = rpmObject.getParent();
            RPMObject oldParent = null;
            if (oldParentId != null)
            {
                oldParent = new TimeCodeCategory();
                oldParent.setID(oldParentId);
                if (newParent != null && newParent.getID() != null
                        && newParent.getID().equals(oldParentId))
                {
                    oldParent = loadByPrimaryKey(newParent, scope.getParent(),
                        messageContext, true);
                }
                else
                {
                    //In this case, old parent is loaded without the
                    // scope to avoid problem with the cache
                    oldParent = loadByPrimaryKey(oldParent, null,
                        messageContext, false);
                }
            }
            rpmObject.deltaParent(oldParent);
        }
        else
        {
            if (oldParentId != null)
            {
                if (newParent == null)
                {   //newParent not in cache						
                    newParent = new TimeCodeCategory();
                    newParent.setID(oldParentId);
                    newParent = loadByPrimaryKey(newParent, scope.getParent(),
                        messageContext, false);
                }
                rpmObject.setParent(newParent);
            }
        }
    }
    
    private void loadChildren(RPMObject parent, GenericTimeCode[] oldChildren,
            TimeCodeScope childrenScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ArrayList childrenList = null;
        if (childrenScope != null)
        {
            Object[] arguments =
            {
                parent.getID()
            };
            childrenList = loadByForeignKey(oldChildren, childrenScope,
                    messageContext, this, arguments, null, null, isDelta);
        }
        if (parent instanceof TimeCodeCategory)
        {
            GenericTimeCode[] children = (GenericTimeCode[]) ArrayUtil
                    .listToArray(GenericTimeCode.class, childrenList);
            ((TimeCodeCategory) parent).setChildren(children);
        }
        else
        {
            TimeCodeCategory[] children = (TimeCodeCategory[]) ArrayUtil
                    .listToArray(TimeCodeCategory.class, childrenList);
            ((TimeCodeGroup) parent).setChildren(children);
        }
    }

	protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof TimecodeManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            SqlBuffer where = new SqlBuffer();
            where.append(NAME_PARENT_ID);
            where.append(" =?");
            int[] types = new int[]
            {
                    TimeCodeCategory.TYPE_ID, TimeCodeGroup.TYPE_ID,
                    TimeCode.TYPE_ID
            };
            where.appendIn(NAME_TYPE_ID, types);
            
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof TimeCodeAssignmentManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            where.append(NAME_ELEMENT_ID);
            where.append(" =?");
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof FinancialModuleManager)
        {            
            joinCondition = new JoinCondition();
            joinCondition.setCondition(extraWhere);            
            joinCondition.setTableName(getTableName()); 
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (rpmObject instanceof TimeCodeGroup)
            ExceptionUtil.handleNoCreate(this, rpmObject);
    	rpmObject.setID(SP_I_TIMECODE((AbstractTimeCode)rpmObject, messageContext));
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (rpmObject instanceof TimeCodeGroup)
        {
            ExceptionUtil.handleNoUpdate(this, rpmObject);
        }
        
        AbstractTimeCode timeCode = (AbstractTimeCode) rpmObject;

        if (timeCode.testNameModified() || timeCode.testDescriptionModified()
                || timeCode.testParentModified())
        {
            SP_U_TIMECODE(timeCode, messageContext);
        }
        
        if (rpmScope != null)
        {
            if (timeCode instanceof TimeCodeCategory)
            {
                saveChildren((TimeCodeCategory) timeCode,
                    (TimeCodeScope) rpmScope, messageContext);
            }
        }
        return rpmObject;
    }

    private void saveChildren(TimeCodeCategory category, TimeCodeScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        if (category.getChildren() != null & scope.getChildren() != null)
        {
            GenericTimeCode[] children = (GenericTimeCode[]) updateChildren(
                category, category.getChildren(), scope.getChildren(),
                messageContext, GenericTimeCode.class);
            category.setChildren(children);
        }
    }
    
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (rpmObject instanceof TimeCodeGroup)
            ExceptionUtil.handleNoDelete(this, rpmObject);
        SP_D_TIMECODE((AbstractTimeCode) rpmObject, messageContext);
    }
    
    public ArrayList searchTimeCodes(String workElementID,
            String scopeElementID, boolean isTimeCode1,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ArrayList array = new ArrayList();
        if (workElementID != null)
        {
            array.addAll(SP_S_XXX_A_TC("SP_S_WBS_A_TC", workElementID,
                isTimeCode1, messageContext));
        }

        if (scopeElementID != null)
        {
            array.addAll(SP_S_XXX_A_TC("SP_S_CRI_A_TC", scopeElementID,
                isTimeCode1, messageContext));
        }

        return array;
    }

    public ArrayList SP_S_XXX_A_TC(String spName, String rpmObjectID,
            boolean isTimeCode1, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        Object args[] = new Object[3];
        args[0] = rpmObjectID;
        args[1] = isTimeCode1 ? new Integer(4) : new Integer(3);
        args[2] = getUser(messageContext).getID().trim();
        
        Connection c = null; 
        ResultSet rs = null;
        ArrayList array = new ArrayList();
        try
        {
            c = Manager.getConnection(messageContext);
            rs = Manager.executeStoredProcedure(c, spName, args, true,
                messageContext);
            Manager.checkForErrorsAndThrowMessage(rs, false);
            
            while (rs.next())
            {
                TimeCode timeCode = new TimeCode();
                timeCode.setID(rs.getString(5));
                timeCode.setDeleted(Boolean.FALSE);
                timeCode.setDescription(rs.getString(10));
                timeCode.setName(rs.getString(6));
                array.add(timeCode);
            }
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return array;
    }


    private String SP_I_TIMECODE(AbstractTimeCode timeCode,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_I_TIMECODE";
        Object args[] = new Object[4];

        args[0] = timeCode.getName();  // name
        args[1] = timeCode.getParent().getID();  // parent_id
        args[2] = timeCode.retrieveTypeID();   // type_id
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, procedureName, args);
    }
    
    private String SP_U_TIMECODE(AbstractTimeCode timeCode,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_TIMECODE";
        Object args[] = new Object[4];

        args[0] = timeCode.getID();  // id
        args[1] = timeCode.getName();  // name
        args[2] = timeCode.getDescription();  // description
        args[3] = getUser(messageContext).getID();  // rec_user

        return executeProcedure(messageContext, name, args);
    }
    
    private void SP_D_TIMECODE(AbstractTimeCode timeCode,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_D_TIMECODE";
        Object args[] = new Object[4];

        args[0] = timeCode.getID(); // id
        args[1] = ManagerUtil.getParentId(this, timeCode, messageContext,
            TABLE_NAME); //parent_id
        args[2] = "D"; // del_flag (TimeCode deletion MUST be soft-delete)
        args[3] = getUser(messageContext).getID(); // rec_user

        executeDeleteProcedure(timeCode, messageContext, name, args);
    }
}

