package com.ibm.rpm.financial.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashSet;
import java.util.Map;

import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.types.TimePhasedType;
import com.ibm.rpm.financial.util.FinancialBusinessLogicUtil;
import com.ibm.rpm.financial.util.FinancialUtil;
import com.ibm.rpm.financial.util.RevenueRecognitionToDBMap;
import com.ibm.rpm.financial.util.TimePhasedTypeToDBMap;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.wbs.containers.WorkElement;

/**
 * The following table defines container fields, mapping to the database, and
 * GUI business-logic. <br>
 * <table border=1>
 * <tr>
 * <td><b>Property </b>
 * </p>
 * <p>
 * <b>(Class member) </b></td>
 * <td colspan=2><b>Origin </b>
 * </p>
 * <p>
 * <b>(RPM DB Field or computation) </b></td>
 * <td><b>RPM Client View </b>
 * </p>
 * <p>
 * <b>(Portlet-Band-Column) </b></td>
 * <td><b>Readonly </b></td>
 * <td><b>&nbsp; </b></td>
 * </tr>
 * <tr>
 * <td colspan=2>ID</td>
 * <td>BUDGETS.BUDGET_ID</td>
 * <td>Work structure pane</td>
 * <td>yes</td>
 * <td><b>&nbsp; </b></td>
 * </tr>
 * <tr>
 * <td colspan=2>Deleted</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>yes</td>
 * <td><b>&nbsp; </b></td>
 * </tr>
 * <tr>
 * <td colspan=2>TypeID</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>yes</td>
 * <td><b>&nbsp; </b></td>
 * </tr>
 * <tr>
 * <td colspan=2>ContextName</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * <td>yes</td>
 * <td><b>&nbsp; </b></td>
 * </tr>
 * <tr>
 * <td colspan=2>Parent</td>
 * <td>WorkElement that matches the BUDGETS.ELEMENT_ID</td>
 * <td>&nbsp;</td>
 * <td>yes</td>
 * <td><b>&nbsp; </b></td>
 * </tr>
 * <tr>
 * <td>Estimate</td>
 * <td colspan=2>BUDGETS.BUDGET</td>
 * <td>Financial-WBS-Estimate</td>
 * <td>&nbsp;</td>
 * <td><b>Rule: </b>Estimate &gt;= Actual.
 * </p>
 * <p>
 * <b>If not: </b> Estimate := Actual.</td>
 * </tr>
 * <tr>
 * <td>Actual</td>
 * <td colspan=2>BUDGETS.ACTUAL</td>
 * <td>Financial-WBS-Actual</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>Remaining</td>
 * <td colspan=2>Estimate - Actual</td>
 * <td>Financial-WBS-Remaining</td>
 * <td>yes</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>ETC</td>
 * <td colspan=2>BUDGETS.ETC</td>
 * <td>Financial-WBS-ETC</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>EAC</td>
 * <td colspan=2>Actual + ETC</td>
 * <td>Financial-WBS-EAC</td>
 * <td>yes</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>EACVariance</td>
 * <td colspan=2>ETC - Remaining</td>
 * <td>Financial-WBS-EACVariance</td>
 * <td>yes</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>Baseline</td>
 * <td colspan=2>BUDGETS.BUDGET_BASELINE1</td>
 * <td>Financial-WBS-Baseline</td>
 * <td>yes</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>BaselineInitial</td>
 * <td colspan=2>BUDGETS.BUDGET_BASELINE3</td>
 * <td>Financial-WBS-Baseline (Initial)</td>
 * <td>yes</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>BaselinePrevious</td>
 * <td colspan=2>BUDGETS.BUDGET_BASELINE2</td>
 * <td>Financial-WBS-Baseline (Previous)</td>
 * <td>yes</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>BaselineVariance</td>
 * <td colspan=2>Estimate - Baseline</td>
 * <td>Financial-WBS-Baseline Variance</td>
 * <td>yes</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>Billed</td>
 * <td colspan=2>BUDGETS.CHARGED</td>
 * <td>Financial-WBS-Billed</td>
 * <td>yes</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>GL</td>
 * <td colspan=2>BUDGETS.LEDGER</td>
 * <td>Financial-WBS-GL</td>
 * <td>yes</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>estimateTimePhased</td>
 * <td colspan=2>&lt;aggregation&gt;</td>
 * <td>Financial-Time-Phased-Cell in Source column</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>actualTimePhased</td>
 * <td colspan=2>&lt;aggregation&gt;</td>
 * <td>Financial-Time-Phased-Cell in Source column</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>BaselineTimePhased</td>
 * <td colspan=2>&lt;aggregation&gt;</td>
 * <td>Financial-Time-Phased-Cell in Source column</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>BaselinePreviousTimePhased</td>
 * <td colspan=2>&lt;aggregation&gt;</td>
 * <td>Financial-Time-Phased-Cell in Source column</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>BaselineInitialTimePhased</td>
 * <td colspan=2>&lt;aggregation&gt;</td>
 * <td>Financial-Time-Phased-Cell in Source column</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>GLTimePhased</td>
 * <td colspan=2>&lt;aggregation&gt;</td>
 * <td>Financial-Time-Phased-Cell in Source column</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>BilledTimePhased</td>
 * <td colspan=2>&lt;aggregation&gt;</td>
 * <td>Financial-Time-Phased-Cell in Source column</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>ChargeCode</td>
 * <td colspan=2>&lt;aggregation&gt;</td>
 * <td>Financial-Name-Charge Codes</td>
 * <td>yes</td>
 * <td><b>&nbsp; </b></td>
 * </tr>
 * <tr>
 * <td>Type</td>
 * <td colspan=2>&lt;aggregation&gt;</td>
 * <td>Financial-WBS-Rec/Rev</td>
 * <td>yes</td>
 * <td><b>&nbsp; </b></td>
 * </tr>
 * <tr height=0>
 * <td></td>
 * <td></td>
 * <td></td>
 * <td></td>
 * <td></td>
 * <td></td>
 * </tr>
 * </table>
 * 
 * @author ikusakov
 *  
 */
public class WbsFinancialManager extends FinancialManager
{

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(WbsFinancial.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    WbsFinancialManager.class);
        }
    }

    public static final String PROPERTY_BUDGET_BASELINE1 = "BASELINE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BUDGET_BASELINE1, NAME_BUDGET_BASELINE1);
    }

    public static final String PROPERTY_BUDGET_BASELINE2 = "BASELINEINITIAL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BUDGET_BASELINE2, NAME_BUDGET_BASELINE2);
    }

    public static final String PROPERTY_BUDGET_BASELINE3 = "BASELINEPREVIOUS";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BUDGET_BASELINE3, NAME_BUDGET_BASELINE3);
    }

    public static final String PROPERTY_BUDGET = "ESTIMATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BUDGET, NAME_BUDGET);
    }

    public static final String PROPERTY_ACTUAL = "ACTUAL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ACTUAL, NAME_ACTUAL);
    }

    public static final String PROPERTY_ETC = "ETC";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ETC, NAME_ETC);
    }

    public static final String PROPERTY_PROB_AMOUNT = "PROBABILITYAMOUNT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROB_AMOUNT, NAME_PROB_AMOUNT);
    }

    public static final String PROPERTY_ACTUAL_TIMEPHASED_FLAG = "TIMEPHASED";
    static
    {
          FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_TIMEPHASED_FLAG, SqlUtil
              .castAsChar(NAME_ACTUAL_TIMEPHASED_FLAG, 1));
    }

    // Handles WBSFinancial
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        WbsFinancial financial = (WbsFinancial) rpmObject;

        financial.setID(rs.getString(ID_BUDGET_ID));

        if (isDelta)
        {
            financial.deltaActual(new Double(rs.getDouble(ID_ACTUAL)));
            financial.deltaEstimate(new Double(rs.getDouble(ID_BUDGET)));
            financial.deltaETC(new Double(rs.getDouble(ID_ETC)));
            
            financial.deltaProbabilityPercent(new Double(rs
                    .getInt(ID_PROB_PERCENT)
                    / PROB_PERCENT_DIV));
        }
        else
        {
            financial.setActual(new Double(rs.getDouble(ID_ACTUAL)));
            financial.setEstimate(new Double(rs.getDouble(ID_BUDGET)));
            financial.setETC(new Double(rs.getDouble(ID_ETC)));
            
            financial.setProbabilityPercent(new Double(rs
                    .getInt(ID_PROB_PERCENT)
                    / PROB_PERCENT_DIV));

            // read-only fields
            financial
                    .setBaseline(new Double(rs.getDouble(ID_BUDGET_BASELINE1)));
            financial.setBaselineInitial(new Double(rs
                    .getDouble(ID_BUDGET_BASELINE3)));
            financial.setBaselinePrevious(new Double(rs
                    .getDouble(ID_BUDGET_BASELINE2)));
            financial.setBilled(new Double(rs.getDouble(ID_CHARGED)));
            financial.setGL(new Double(rs.getDouble(ID_LEDGER)));
            financial.setProbabilityAmount(new Double(rs
                    .getDouble(ID_PROB_AMOUNT)));
            financial.setTimePhased(TimePhasedTypeToDBMap.toType(rs
                    .getInt(ID_ACTUAL_TIMEPHASED_FLAG)));

            /*
             * TODO: implement N/A business rule for RevRecType if
             * isSystemChargeCode(Id) then begin AText := Translate('N/A'); end
             * else // if treeType = 2 {Benefit} then begin if
             * Anode.Values[colHiddenAssociatedID.index] = '' then AText :=
             * Translate('N/A'); end;
             */
            financial.setRevRecType(RevenueRecognitionToDBMap
                    .getRevRecTypeFromInteger(new Integer(rs
                            .getInt(ID_AUTO_FLAG))));

        }

        return financial;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {

        RPMObject object = null;

        switch (typeId)
        {
        case 0:
            object = new WbsFinancial();
            break;

        default:
            RPMException e = new RPMException("TypeId " + typeId
                    + " is not supported");
            logger.error(e);
            throw e;
        }
        return object;
    }

    /**
     * FTE (Full Time Equivalent) line is treated exclusively in RPM Client UI
     * The values from the DB are never displayed, but calculated values appear,
     * based on TMT_WBS table, SP_S_FTE() results and Financial object for Hours
     * 
     * This method is called from loadRelatedObjects() method, because it has
     * messageContext and relatedFields
     * 
     * Business rules are located in AssignBudgetViewFrame.pas see methods
     * LoadDataEx() and updateFTE()
     * 
     * @param rpmObject
     *            FTE financial object
     * @param messageContext
     * @param relatedFields
     * @param index
     * @return modified FTE object
     */
    protected RPMObject calculateFTE(boolean isWorkElementParent,
            RPMObject rpmObject, MessageContext messageContext,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException, ParseException
    {
        AbstractFinancial financial = (AbstractFinancial) rpmObject;

        String workElementID = (String) relatedFields.get(index, ID_ELEMENT_ID);
        String chargeCodeID = (String) relatedFields.get(index,
                ID_CHARGECODE_ID);

        // 2) we need some data from TMT_WBS and TMT_CRI which is currently not
        // in the
        // WorkElement container
        Map wbsData = loadWbsDataForFTE(workElementID, messageContext);

        if (wbsData != null)
        {
            // 1) we need financial for Hours
            FinancialManager manager = (FinancialManager) getManagerInstance(WbsFinancial.class
                    .getName());

            WbsFinancial hoursFinancial = (WbsFinancial) loadHoursFinancial(
                    manager, workElementID, chargeCodeID, messageContext);

	        // 3) we need data from SP_S_FTE
	        Map spData = SP_S_FTE(isWorkElementParent, workElementID,
	                messageContext);
	
	        // Finally, we can calculate
	        FinancialBusinessLogicUtil.calculateFTEForWbsFinancial(
	                (WbsFinancial) financial, hoursFinancial, wbsData,spData,messageContext);
        }

        return financial;
    }

    public void SP_U_BUDGET(AbstractFinancial financial,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        final String procedureName = "SP_U_BUDGET";

        // To avoid deletion of the data in the same table row, we need to load
        // the opposit object and use it's values
        Object args[] = new Object[20];
        // Set the table type: 1 for a WorkElement and 3 for a ScopeElement.
        if (ContextUtil.SCOPE_CONTEXT.equals(financial.getContextName())
                || (financial.getParent() != null && financial.getParent() instanceof ScopeElement))
        {
            args[0] = new Long(UPDATE_SCOPE_ELEMENT);
        }
        else if (ContextUtil.WBS_CONTEXT.equals(financial.getContextName())
                || (financial.getParent() != null && financial.getParent() instanceof WorkElement))
        {
            args[0] = new Long(UPDATE_WORK_ELEMENT);
        }
        else
        {
            ExceptionUtil.handleUnsupportedContext(this, financial,
                "SP_U_BUDGET");
        }
        args[1] = financial.getID();
        // An integer flag denoting revenue recognition method of a budget.
        // 0 for Manual Method, 1 for Ratio and 2 for Flow-Thru.
        args[2] = RevenueRecognitionToDBMap.getIntegerRevRecType(financial
            .getRevRecType());
        if (args[2] == null)
        {
            args[2] = new Integer(0); // default value
        }

        // update of timephased_flag
        Long timephased_flag = TimePhasedTypeToDBMap
            .toDBCode(/*opposite*/financial.getTimePhased());
        if (timephased_flag == null)
        {
            timephased_flag = TimePhasedTypeToDBMap
                .toDBCode(TimePhasedType.ManuallyTimephased);
        }
        args[3] = timephased_flag;

        // update of actual_timephased_flag
        Long actual_timephased_flag = TimePhasedTypeToDBMap.toDBCode(financial
            .getTimePhased());
        if (actual_timephased_flag == null)
        {
            actual_timephased_flag = TimePhasedTypeToDBMap
                .toDBCode(TimePhasedType.ManuallyTimephased);
        }
        args[4] = actual_timephased_flag;

        // The probability percent of Benefits budget.
        // prob_percent
        int probPercent = financial.getProbabilityPercent() == null ? 10000
                : (int) Math.round(financial.getProbabilityPercent()
                    .doubleValue()
                        * PROB_PERCENT_DIV);
        args[5] = new Long(probPercent);
        // The probability amount from Benefits budget passed in exponentational
        // form.
        // prob_amount
        args[6] = new Double(FinancialUtil.nullToZero(financial.getEstimate())
            .doubleValue()
                * probPercent / PROB_PERCENT_DIV / 100);
        // The worksheet probability amount from benefits passed in
        // exponentational form.
        // estimated_prob_amount
        args[7] = new Double(FinancialUtil.nullToZero(financial.getEstimate())
            .doubleValue()
                * probPercent / PROB_PERCENT_DIV / 100);
        // The worksheet estimated budget amount passed in exponentational form.
        // estimated_budget
        args[8] = FinancialUtil.nullToZero(financial.getEstimate());
        // The worksheet actual amount passed in exponentational form.
        // estimated_actual
        args[9] = FinancialUtil.nullToZero(financial.getActual());
        // The worksheet ETC amount passed in exponentational form.
        // estimated_etc
        args[10] = FinancialUtil.nullToZero(/*opposite*/financial.getETC());
        // The worksheet paid to date amount passed in exponenetational form.
        // estimated_paid_todate
        args[11] = new Double(0);
        // NOT USED
        // contingency_budget
        args[12] = new Double(0);
        // NOT USED
        // contingency_etc
        args[13] = new Double(0);
        // The WBS estimated budget amount passed in exponentational form.
        // budget
        args[14] = FinancialUtil.nullToZero(financial.getEstimate());
        // The WBS actual amount passed in exponentational form.
        // actual
        args[15] = FinancialUtil.nullToZero(financial.getActual());
        // The WBS ETC amount passed in exponentational form.
        // etc
        args[16] = FinancialUtil.nullToZero(financial.getETC());
        // The WBS paid to date amount passed in exponenetational form.
        // paid_todate
        args[17] = new Double(0);
        // Tolerance flag.
        // tolerance
        args[18] = new Long(0);
        // The id of a user who updates the budget information of wbs/cri
        // elements.
        args[19] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }
}
