package com.ibm.rpm.financial.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.types.BudgetType;

/**
 * Contains BudgetType <--> PROJECT_BUDGET_BREAKDOWN.BUDGET_TYPE conversion code
 * 
 * @author ikusakov
 */
public class BudgetTypeToDBMap
{
    private static final HashMap BUDGETTYPEMAP_WORKSHEET = new HashMap();

    private static final int BUDGETTYPES_ARRAY_SIZE = 30;

    private static final ArrayList BUDGETTYPEARRAY_WORKSHEET = new ArrayList(
            BUDGETTYPES_ARRAY_SIZE);

    static
    {
        BUDGETTYPEMAP_WORKSHEET.put(BudgetType.Actual, new Integer(4));
        BUDGETTYPEMAP_WORKSHEET.put(BudgetType.Estimate, new Integer(3));
        BUDGETTYPEMAP_WORKSHEET.put(BudgetType.Baseline, new Integer(5));
        BUDGETTYPEMAP_WORKSHEET.put(BudgetType.BaselineInitial, new Integer(7));
        BUDGETTYPEMAP_WORKSHEET
                .put(BudgetType.BaselinePrevious, new Integer(6));
        BUDGETTYPEMAP_WORKSHEET.put(BudgetType.Billed, new Integer(14));
        BUDGETTYPEMAP_WORKSHEET.put(BudgetType.GL, new Integer(16));

        //      ensuring elements are there
        for (int i = 0; i < BUDGETTYPES_ARRAY_SIZE; i++)
        {
            BUDGETTYPEARRAY_WORKSHEET.add(null);
        }

        for (Iterator i = BUDGETTYPEMAP_WORKSHEET.keySet().iterator(); i
                .hasNext();)
        {
            BudgetType bt = (BudgetType) i.next();
            BUDGETTYPEARRAY_WORKSHEET.set(((Integer) BUDGETTYPEMAP_WORKSHEET
                    .get(bt)).intValue(), bt);
        }
    }

    private static final HashMap BUDGETTYPEMAP_WBS = new HashMap();

    private static final ArrayList BUDGETTYPEARRAY_WBS = new ArrayList(
            BUDGETTYPES_ARRAY_SIZE);

    static
    {
        BUDGETTYPEMAP_WBS.put(BudgetType.Actual, new Integer(1));
        BUDGETTYPEMAP_WBS.put(BudgetType.Estimate, new Integer(2));
        BUDGETTYPEMAP_WBS.put(BudgetType.Baseline, new Integer(8));
        BUDGETTYPEMAP_WBS.put(BudgetType.BaselineInitial, new Integer(10));
        BUDGETTYPEMAP_WBS.put(BudgetType.BaselinePrevious, new Integer(9));
        BUDGETTYPEMAP_WBS.put(BudgetType.Billed, new Integer(13));
        BUDGETTYPEMAP_WBS.put(BudgetType.GL, new Integer(15));

        // ensuring elements are there
        for (int i = 0; i < BUDGETTYPES_ARRAY_SIZE; i++)
        {
            BUDGETTYPEARRAY_WBS.add(null);
        }

        for (Iterator i = BUDGETTYPEMAP_WBS.keySet().iterator(); i.hasNext();)
        {
            BudgetType bt = (BudgetType) i.next();
            BUDGETTYPEARRAY_WBS.set(((Integer) BUDGETTYPEMAP_WBS.get(bt))
                    .intValue(), bt);
        }
    }

    /**
     * Converts BudgetType object into it's Integer representation in the
     * database
     * 
     * This method is used internally, and from WorksheetFinancial manager to
     * prepare arguments for loadByForeignKey
     * 
     * @param isWorksheet
     *            true if the parent is WorksheetFinancial, false - WBSFinancial
     * @param type
     *            Budget Type to convert
     * @return Integer value of the budget type or null if not found
     */
    public static Integer getIntegerBudgetType(boolean isWorksheet,
            BudgetType type)
    {
        Integer budgetType = null;
        if (type != null && type.equals(BudgetType.Calendar))
        {
            return new Integer(0);
        }
        else if (isWorksheet)
        {
            budgetType = (Integer) BUDGETTYPEMAP_WORKSHEET.get(type);
        }
        else
        {
            budgetType = (Integer) BUDGETTYPEMAP_WBS.get(type);
        }

        return budgetType;
    }

    /**
     * Converts DB integer value from PROJECT_BUDGET_BREAKDOWN.BUDGET_TYPE column
     * into BudgetType object
     * 
     * @param value
     * @return BudgetType or null if no such type
     */
    public static BudgetType getBudgetTypeFromInt(Integer value)
    {
        BudgetType budgetType = null;

        if (value != null)
        {
            int index = value.intValue();

            if (index == 0)
            {
                return BudgetType.Calendar;
            }
            else
            {
                budgetType = (BudgetType) BUDGETTYPEARRAY_WORKSHEET.get(index);
                if (budgetType == null)
                {
                    budgetType = (BudgetType) BUDGETTYPEARRAY_WBS.get(index);
                }
            }
        }

        return budgetType;
    }
    
    /**
     * Checks if given integer value from PROJECT_BUDGET_BREAKDOWN.BUDGET_TYPE
     * corresponds to WorksheetFinancial 
     * 
     * @param value
     * @return
     */
    public static boolean isWorksheetBudgetType(Integer value) {
        
        if (value == null || value.intValue() >= BUDGETTYPES_ARRAY_SIZE) return false;
        
        return BUDGETTYPEARRAY_WORKSHEET.get(value.intValue()) != null;
    }
    
    /**
     * Checks if given integer value from PROJECT_BUDGET_BREAKDOWN.BUDGET_TYPE
     * corresponds to WBSFinancial 
     *      
     * @param value
     * @return
     */
    public static boolean isWBSBudgetType(Integer value) {
        
        if (value == null || value.intValue() >= BUDGETTYPES_ARRAY_SIZE) return false;
        
        return BUDGETTYPEARRAY_WBS.get(value.intValue()) != null;
    }
    
    public static Integer getIntegerBudgetTypeFromFieldname(Class financial, String fieldname){
        
        boolean isWorksheet = (financial.getClass() == WorksheetFinancial.class);

        BudgetType budgetType = null;
        Integer integerBudgetType = null;

        if (fieldname != null)
        {
            if (fieldname.equalsIgnoreCase("actualTimePhased"))
            {
                budgetType = BudgetType.Actual;
            }
            else if (fieldname.equalsIgnoreCase("baselineTimePhased"))
            {
                budgetType = BudgetType.Baseline;
            }
            else if (fieldname.equalsIgnoreCase("baselineInitialTimePhased"))
            {
                budgetType = BudgetType.BaselineInitial;
            }
            else if (fieldname.equalsIgnoreCase("baselinePreviousTimePhased"))
            {
                budgetType = BudgetType.BaselinePrevious;
            }
            else if (fieldname.equalsIgnoreCase("billedTimePhased"))
            {
                budgetType = BudgetType.Billed;
            }
            else if (fieldname.equalsIgnoreCase("estimateTimePhased"))
            {
                budgetType = BudgetType.Estimate;
            }
            else if (fieldname.equalsIgnoreCase("GLTimePhased"))
            {
                budgetType = BudgetType.GL;
            }

            integerBudgetType = BudgetTypeToDBMap.getIntegerBudgetType(
                isWorksheet, budgetType);
        }
        return integerBudgetType;
    }
}
