/*
 * Created on 8-Sep-2006
 * @author mmatwiy
 * Utility methods for container Financial.
 */
package com.ibm.rpm.financial.util;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import com.ibm.rpm.applicationadministration.containers.CalendarWeekDay;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.containers.WorkInterval;
import com.ibm.rpm.applicationadministration.types.CalendarWeekDayType;
import com.ibm.rpm.financial.constants.ValidationConstants;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.managers.ChargeCodeManager;
import com.ibm.rpm.financial.managers.FinancialManager;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.EarnedValues;

public class FinancialUtil
{
    public final static ArrayList DEFAULT_CHARGE_CODE_ID = new ArrayList();
    
    static
    {
        DEFAULT_CHARGE_CODE_ID.add("SYSTEM_CHARGE_CODES_FOLDER");
        DEFAULT_CHARGE_CODE_ID.add("SYSTEM_EXPENSE_FOLDER");
        DEFAULT_CHARGE_CODE_ID.add("EXPENSE_HOURS");
        DEFAULT_CHARGE_CODE_ID.add("EXPENSE_AVERAGE_COST");
        DEFAULT_CHARGE_CODE_ID.add("EXPENSE_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("EXPENSE_FULL_TIME_EQUIVALENT");
        DEFAULT_CHARGE_CODE_ID.add("EXPENSE_CONTRACTOR_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("SYSTEM_CAPITAL_FOLDER");
        DEFAULT_CHARGE_CODE_ID.add("CAPITAL_HOURS");
        DEFAULT_CHARGE_CODE_ID.add("CAPITAL_AVERAGE_COST");
        DEFAULT_CHARGE_CODE_ID.add("CAPITAL_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("CAPITAL_FULL_TIME_EQUIVALENT");
        DEFAULT_CHARGE_CODE_ID.add("CAPITAL_CONTRACTOR_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("SYSTEM_BENEFIT_FOLDER");
        DEFAULT_CHARGE_CODE_ID.add("BENEFIT_HOURS");
        DEFAULT_CHARGE_CODE_ID.add("BENEFIT_AVERAGE_RATE");
        DEFAULT_CHARGE_CODE_ID.add("BILLABLE_BENEFIT_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("BENEFIT_FULL_TIME_EQUIVALENT");
        DEFAULT_CHARGE_CODE_ID.add("BENEFIT_CONTRACTOR_LABOR");
        DEFAULT_CHARGE_CODE_ID.add("PMO_PROJECT_TIMEPHASE_CALENDAR__");
        DEFAULT_CHARGE_CODE_ID.add("CUSTOM_CHARGE_CODES_FOLDER______");
    }

    public static final int COL_ESTIMATE = 0;
    public static final int COL_ACTUAL = 1;
    public static final int COL_REMAINING = 2;
    public static final int COL_ETC = 3;
    public static final int COL_EAC = 4;
    
    public static final int EXPENSE_HOUR = 11;

    public static final int EXPENSE_AVERAGE_COST = 12;

    public static final int EXPENSE_LABOR = 14;

    public static final int EXPENSE_FTE = 13;
    
    private static final int HOUR_INDEX = 0;
    private static final int AVG_COST_INDEX = 1;
    private static final int EXP_LABOR_INDEX = 2;
    private static final int FTE_INDEX = 3;


    public static boolean isTimePhasedActualAllYearsModified(
            AbstractFinancial financial)
    {
        boolean timephasedModified = false;

        // checking if timephased actuals have been modified
        if (financial.getActualTimePhased() != null
                && financial.getActualTimePhased().length > 0)
        {
            for (int i = 0; i < financial.getActualTimePhased().length; i++)
            {
                timephasedModified = isTimePhasedModified(financial
                    .getActualTimePhased()[i]);

                if (timephasedModified)
                {
                    break;
                }
            }
        }
        return timephasedModified;
    }

    public static boolean isTimePhasedEstimateAllYearsModified(
            AbstractFinancial financial)
    {
        boolean timephasedModified = false;

        // checking if timephased estimates have been modified
        if (!timephasedModified && financial.getEstimateTimePhased() != null
                && financial.getEstimateTimePhased().length > 0)
        {
            for (int i = 0; i < financial.getEstimateTimePhased().length; i++)
            {
                timephasedModified = isTimePhasedModified(financial
                    .getEstimateTimePhased()[i]);

                if (timephasedModified)
                {
                    break;
                }
            }
        }
        return timephasedModified;
    }

    public static boolean isTimePhasedModified(TimePhasedFinancial timephased)
    {
        boolean modified = false;

        if (timephased.testMonth01ValueModified()
                && timephased.getMonth01Value() != null
                || timephased.testMonth02ValueModified()
                && timephased.getMonth02Value() != null
                || timephased.testMonth03ValueModified()
                && timephased.getMonth03Value() != null
                || timephased.testMonth04ValueModified()
                && timephased.getMonth04Value() != null
                || timephased.testMonth05ValueModified()
                && timephased.getMonth05Value() != null
                || timephased.testMonth06ValueModified()
                && timephased.getMonth06Value() != null
                || timephased.testMonth07ValueModified()
                && timephased.getMonth07Value() != null
                || timephased.testMonth08ValueModified()
                && timephased.getMonth08Value() != null
                || timephased.testMonth09ValueModified()
                && timephased.getMonth09Value() != null
                || timephased.testMonth10ValueModified()
                && timephased.getMonth10Value() != null
                || timephased.testMonth11ValueModified()
                && timephased.getMonth11Value() != null
                || timephased.testMonth12ValueModified()
                && timephased.getMonth12Value() != null
                || timephased.testYearModified()
                && timephased.getYear() != null) 
        {
            modified = true;
        }

        return modified;
    }
    
    public static boolean isEarnedValuesModified(EarnedValues ev)
    {
        boolean modified = false;
        if (ev != null)
        {
            if (ev.testBudgetAtCompletionModified()
                    && ev.getBudgetAtCompletion() != null
                    || ev.testSchedAtCompletionModified()
                    && ev.getSchedAtCompletion() != null
                    || ev.testBudgetedCostWorkSchedModified()
                    && ev.getBudgetedCostWorkSched() != null
                    || ev.testBudgetedCostWorkPerfModified()
                    && ev.getBudgetedCostWorkPerf() != null
                    || ev.testActualCostWorkPerfModified()
                    && ev.getActualCostWorkPerf() != null
                    || ev.testEstimateToCompleteModified()
                    && ev.getEstimateToComplete() != null
                    || ev.testEstimateAtCompleteModified()
                    && ev.getEstimateAtComplete() != null
                    || ev.testEstimatedCompletionDateModified()
                    && ev.getEstimatedCompletionDate() != null
                    || ev.testBudgetedCostWorkSchedForecastModified()
                    && ev.getBudgetedCostWorkSchedForecast() != null
                    || ev.testBudgetedCostWorkPerfForecastModified()
                    && ev.getBudgetedCostWorkPerfForecast() != null
                    || ev.testActualCostWorkPerfModified()
                    && ev.getActualCostWorkPerf() != null
                    || ev.testEstimateToCompleteForecastModified()
                    && ev.getEstimateToCompleteForecast() != null
                    || ev.testEstimateAtCompleteForecastModified()
                    && ev.getEstimateAtCompleteForecast() != null
                    || ev.testScheduleVariance_CModified()
                    && ev.getScheduleVariance_C() != null
                    || ev.testCostVariance_CModified()
                    && ev.getCostVariance_C() != null
                    || ev.testTimeVarianceModified()
                    && ev.getTimeVariance() != null
                    || ev.testPercentComplete_CModified()
                    && ev.getPercentComplete_C() != null
                    || ev.testSchedPerformIndex_CModified()
                    && ev.getSchedPerformIndex_C() != null
                    || ev.testCostPerformIndex_CModified()
                    && ev.getCostPerformIndex_C() != null
                    || ev.testCriticalRatio_CModified()
                    && ev.getCriticalRatio_C() != null
                    || ev.testToCompletePerformIndex_CModified()
                    && ev.getToCompletePerformIndex_C() != null
                    || ev.testVarianceAtCompletion_CModified()
                    && ev.getVarianceAtCompletion_C() != null
                    || ev.testSchedPerforIndexAtComp_CModified()
                    && ev.getSchedPerforIndexAtComp_C() != null
                    || ev.testCriticalRatioAtComp_CModified()
                    && ev.getCriticalRatioAtComp_C() != null
                    || ev.testTimeEstimateAtComp_CModified()
                    && ev.getTimeEstimateAtComp_C() != null
                    || ev.testTimeVarianceAtComp_CModified()
                    && ev.getTimeVarianceAtComp_C() != null)

            {
                modified = true;
            }
        }

        return modified;
    }
    

    public static double calculateTimephasedEstimateTotal(
            AbstractFinancial financial)
    {
        double totalEstimate = 0.0;
        TimePhasedFinancial[] estimates = financial.getEstimateTimePhased();

        if (estimates != null)
        {
            for (int i = 0; i < estimates.length; i++)
            {
                totalEstimate += nullToZero(estimates[i].getMonth01Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth02Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth03Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth04Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth05Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth06Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth07Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth08Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth09Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth10Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth11Value())
                    .doubleValue();
                totalEstimate += nullToZero(estimates[i].getMonth12Value())
                    .doubleValue();
            }
        }
        return totalEstimate;
    }

    public static double calculateTimephasedActualTotal(
            AbstractFinancial financial)
    {
        double totalActual = 0.0;
        TimePhasedFinancial[] actuals = financial.getActualTimePhased();

        if (actuals != null)
        {
            for (int i = 0; i < actuals.length; i++)
            {
                totalActual += nullToZero(actuals[i].getMonth01Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth02Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth03Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth04Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth05Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth06Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth07Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth08Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth09Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth10Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth11Value())
                    .doubleValue();
                totalActual += nullToZero(actuals[i].getMonth12Value())
                    .doubleValue();
            }
        }
        return totalActual;
    }
    
    
    
    public static double calculateTimephasedEstimateTotal(
            TimePhasedFinancial financial)
    {
        double totalEstimate = 0.0;
        TimePhasedFinancial estimates = financial;

        if (estimates != null)
        {
            totalEstimate += nullToZero(estimates.getMonth01Value())
                .doubleValue();
            totalEstimate += nullToZero(estimates.getMonth02Value())
                .doubleValue();
            totalEstimate += nullToZero(estimates.getMonth03Value())
                .doubleValue();
            totalEstimate += nullToZero(estimates.getMonth04Value())
                .doubleValue();
            totalEstimate += nullToZero(estimates.getMonth05Value())
                .doubleValue();
            totalEstimate += nullToZero(estimates.getMonth06Value())
                .doubleValue();
            totalEstimate += nullToZero(estimates.getMonth07Value())
                .doubleValue();
            totalEstimate += nullToZero(estimates.getMonth08Value())
                .doubleValue();
            totalEstimate += nullToZero(estimates.getMonth09Value())
                .doubleValue();
            totalEstimate += nullToZero(estimates.getMonth10Value())
                .doubleValue();
            totalEstimate += nullToZero(estimates.getMonth11Value())
                .doubleValue();
            totalEstimate += nullToZero(estimates.getMonth12Value())
                .doubleValue();

        }
        return totalEstimate;
    }

    public static double calculateTimephasedActualTotal(
            TimePhasedFinancial financial)
    {
        double totalActual = 0.0;
        TimePhasedFinancial actuals = financial;

        if (actuals != null)
        {
            totalActual += nullToZero(actuals.getMonth01Value()).doubleValue();
            totalActual += nullToZero(actuals.getMonth02Value()).doubleValue();
            totalActual += nullToZero(actuals.getMonth03Value()).doubleValue();
            totalActual += nullToZero(actuals.getMonth04Value()).doubleValue();
            totalActual += nullToZero(actuals.getMonth05Value()).doubleValue();
            totalActual += nullToZero(actuals.getMonth06Value()).doubleValue();
            totalActual += nullToZero(actuals.getMonth07Value()).doubleValue();
            totalActual += nullToZero(actuals.getMonth08Value()).doubleValue();
            totalActual += nullToZero(actuals.getMonth09Value()).doubleValue();
            totalActual += nullToZero(actuals.getMonth10Value()).doubleValue();
            totalActual += nullToZero(actuals.getMonth11Value()).doubleValue();
            totalActual += nullToZero(actuals.getMonth12Value()).doubleValue();

        }
        return totalActual;
    }


    public static double loadFieldFromBudgets(AbstractFinancial financial,
            String fieldName, MessageContext messageContext)
            throws RPMException
    {
        // SELECT fieldName FROM BUDGETS where BUDGET_ID = ?
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(FinancialManager.NAME_BUDGET_ID);
        
        Object[] arguments = new Object[]
        {
            financial.getID()
        };
        
        Double doubleResult = (Double) ManagerUtil.selectColumnValue(
            Double.class, financial, fieldName, fieldName,
            FinancialManager.TABLE_NAME, condition, arguments, messageContext);

        return (doubleResult == null ? 0 : doubleResult.doubleValue());
    }
    
    

    public static Double nullToZero(Double d)
    {
        d = fixNegativeZero(d);
        return d == null ? new Double(0) : d;
    }
 
    public static Integer nullToZero(Integer d)
    {
        d = fixNegativeZero(d);
        return d == null ? new Integer(0) : d;
    }    
    
    protected static double negative2Zero(double d)
    {
        d = fixNegativeZero(d);
        return (d < 0) ? 0.0: d;
    }
    
    
    public static Double fixNegativeZero(Double val)
    {
        if (val != null && val.doubleValue() == -0.0)
        {
            val = new Double(0.0);
        }

        return val;
    }
    
    
    public static double fixNegativeZero(double val)
    {
        if (val == -0.0)
        {
            val = 0.0;
        }

        return val;
    }

    public static Integer fixNegativeZero(Integer val)
    {
        if (val != null && val.intValue() == -0)
        {
            val = new Integer(0);
        }

        return val;
    }    

    
    public static Double roundDouble2Dec(Double value)
    {
        if (value != null)
        {
            int decimalPlaces = 2;

            BigDecimal bd = new BigDecimal(value.doubleValue());

            bd = bd.setScale(decimalPlaces, BigDecimal.ROUND_HALF_UP);

            return new Double(bd.doubleValue());
        }
        else
        {
            return new Double(0);
        }
    }

    

    

    
    public static AbstractFinancial getFinancialForChargeCode(int cCode,
            AbstractFinancial[] financials) throws RPMException
    {
        AbstractFinancial financial = null;

        if (financials != null)
        {
            for (int i = 0; i < financials.length; i++)
            {
                if (financials[i] != null
                        && financials[i].getChargeCode() != null
                        && financials[i].getChargeCode().getFinancialCategory() != null)
                {
                    Integer dbCode = FinancialCategoryUtil
                        .toDBCode(financials[i].getChargeCode()
                            .getFinancialCategory());

                    if (dbCode.intValue() == cCode)
                    {
                        financial = financials[i];
                        break;
                    }
                }
            }
        }
        return financial;
    }
    
    static public AbstractFinancial getMatchingWorksheetFinancial(
            String chargecode, AbstractFinancial[] financial)
    {
        int i = -1;
        int index = -1;

        AbstractFinancial result = null;

        if (chargecode == null || financial == null)
        {
            return result;
        }
        else
        {
            for (i = 0; i < financial.length; i++)
            {
                if (financial[i] != null
                        && financial[i].getChargeCode() != null
                        && financial[i].getChargeCode().getID() != null
                        && financial[i].getChargeCode().getID().startsWith(
                            chargecode))
                {
                    index = i;
                    result = financial[index];
                    break;
                }
            }
            return result;
        }
    }

    
    
    public static WorksheetFinancial getWorksheetFinancialWithChargeCode(
            AbstractAggregateScope aaScope, ChargeCode cc)
    {
        if (aaScope != null && cc != null
                && aaScope.getWorksheetFinancials() != null
                && cc.getName() != null)
        {
            for (int i = 0; i < aaScope.getWorksheetFinancials().length; i++)
            {
                if (aaScope.getWorksheetFinancials()[i] != null
                        && aaScope.getWorksheetFinancials()[i].getChargeCode() != null
                        && cc.getName().equals(
                            aaScope.getWorksheetFinancials()[i].getChargeCode()
                                .getName()))
                {
                    return aaScope.getWorksheetFinancials()[i];
                }
            }
        }
        return null;
    }
    
	public static WorksheetFinancial getWorksheetFinancialWithChargeCode(
            AggregateNode aggNode, ChargeCode cc)
    {
        if (aggNode != null && cc != null
                && aggNode.getWorksheetFinancials() != null
                && cc.getName() != null)
        {
            for (int i = 0; i < aggNode.getWorksheetFinancials().length; i++)
            {
                if (aggNode.getWorksheetFinancials()[i] != null
                        && aggNode.getWorksheetFinancials()[i].getChargeCode() != null
                        && cc.getName().equals(
                            aggNode.getWorksheetFinancials()[i].getChargeCode()
                                .getName()))
                {
                    return aggNode.getWorksheetFinancials()[i];
                }
            }
        }
        return null;
    }
	
	  /*
     * Calculates the total difference of all the monthly values for two
     * TimephasedFinancial containers. The difference is positing if t1 has a
     * larger total and negative is t1 has a smaller total.
     */
    public static double calculateTotalDifference(TimePhasedFinancial t1,
            TimePhasedFinancial t2)
    {
        double difference = 0.0;

        if (t1 != null)
        {
            if (t2 != null)
            {
                difference = (nullToZero(t1.getMonth01Value()).doubleValue() - nullToZero(
                    t2.getMonth01Value()).doubleValue())
                        + (nullToZero(t1.getMonth02Value()).doubleValue() - nullToZero(
                            t2.getMonth02Value()).doubleValue())
                        + (nullToZero(t1.getMonth03Value()).doubleValue() - nullToZero(
                            t2.getMonth03Value()).doubleValue())
                        + (nullToZero(t1.getMonth04Value()).doubleValue() - nullToZero(
                            t2.getMonth04Value()).doubleValue())
                        + (nullToZero(t1.getMonth05Value()).doubleValue() - nullToZero(
                            t2.getMonth05Value()).doubleValue())
                        + (nullToZero(t1.getMonth06Value()).doubleValue() - nullToZero(
                            t2.getMonth06Value()).doubleValue())
                        + (nullToZero(t1.getMonth07Value()).doubleValue() - nullToZero(
                            t2.getMonth07Value()).doubleValue())
                        + (nullToZero(t1.getMonth08Value()).doubleValue() - nullToZero(
                            t2.getMonth08Value()).doubleValue())
                        + (nullToZero(t1.getMonth09Value()).doubleValue() - nullToZero(
                            t2.getMonth09Value()).doubleValue())
                        + (nullToZero(t1.getMonth10Value()).doubleValue() - nullToZero(
                            t2.getMonth10Value()).doubleValue())
                        + (nullToZero(t1.getMonth11Value()).doubleValue() - nullToZero(
                            t2.getMonth11Value()).doubleValue())
                        + (nullToZero(t1.getMonth12Value()).doubleValue() - nullToZero(
                            t2.getMonth12Value()).doubleValue());
            }
            else
            {
                // if t2 is null, add up the t1 monthly values
                difference = nullToZero(t1.getMonth01Value()).doubleValue()
                        + nullToZero(t1.getMonth02Value()).doubleValue()
                        + nullToZero(t1.getMonth03Value()).doubleValue()
                        + nullToZero(t1.getMonth04Value()).doubleValue()
                        + nullToZero(t1.getMonth05Value()).doubleValue()
                        + nullToZero(t1.getMonth06Value()).doubleValue()
                        + nullToZero(t1.getMonth07Value()).doubleValue()
                        + nullToZero(t1.getMonth08Value()).doubleValue()
                        + nullToZero(t1.getMonth09Value()).doubleValue()
                        + nullToZero(t1.getMonth10Value()).doubleValue()
                        + nullToZero(t1.getMonth11Value()).doubleValue()
                        + nullToZero(t1.getMonth12Value()).doubleValue();
            }
        }

        return difference;
    }

    public static AbstractFinancial loadExistingWorksheetValues(
            AbstractFinancial financial, FinancialManager manager,
            MessageContext messageContext) throws ParseException, SQLException,
            RPMException
    {
        FinancialScope scope = new FinancialScope();
        scope.setActualTimePhased(true);
        scope.setEstimateTimePhased(true);

        AbstractFinancial reloaded = (AbstractFinancial) manager.load(
            financial, scope, messageContext);

        return reloaded;
    }
    
    public static WbsFinancial loadExistingWbsValues(
            WbsFinancial financial, FinancialManager manager,
            MessageContext messageContext) throws ParseException, SQLException,
            RPMException
    {
        FinancialScope scope = new FinancialScope();
        scope.setActualTimePhased(true);
        scope.setEstimateTimePhased(true);

        WbsFinancial reloaded = (WbsFinancial) manager.load(
            financial, scope, messageContext);

        return reloaded;
    }
    
    public static WorksheetFinancial loadExistingWorksheetValues(
            WorksheetFinancial financial, FinancialManager manager,
            MessageContext messageContext) throws ParseException, SQLException,
            RPMException
    {
        FinancialScope scope = new FinancialScope();
        scope.setActualTimePhased(true);
        scope.setEstimateTimePhased(true);

        WorksheetFinancial reloaded = (WorksheetFinancial) manager.load(
            financial, scope, messageContext);

        return reloaded;
    }


    public static WorksheetFinancial[] createEmptyWorksheetFinancialSystemChargeCodesArray(
            String sessionID, AggregateNode parent,
            AbstractRPMObjectManager manager, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        WorksheetFinancial[] wWSheetFinancial = null;
        String SystemChargeCodeName = "System Charge Codes";
        ArrayList resultArray = null;
        ChargeCodeCategory wCCCategory = null;
        ChargeCodeCategory wExpenseCategory = null;

        ChargeCode wExpenseHour = null;
        ChargeCode wExpenseAvg = null;
        ChargeCode wExpenseLabor = null;
        ChargeCode wExpenseFTE = null;

        // ChargeCodeScope needed to load the ChargeCodeCategory
        ChargeCodeScope wCCMainScope = new ChargeCodeScope();
        wCCMainScope.setChildren(new ChargeCodeScope());
        wCCMainScope.getChildren().setChildren(new ChargeCodeScope());

        ChargeCodeManager chargeCodeManager = (ChargeCodeManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(ChargeCode.class.getName());

        SqlBuffer extraWhere = new SqlBuffer();
        extraWhere.appendEqual(ChargeCodeManager.NAME_ELEMENT_NAME,
            SystemChargeCodeName);
        extraWhere.appendEqual(ChargeCodeManager.NAME_TYPE_ID,
            ChargeCodeCategory.TYPE_ID);

        resultArray = chargeCodeManager.loadByForeignKey(null, wCCMainScope,
            messageContext, null, null, extraWhere.toString(), null, false);

        if (resultArray != null && resultArray.get(0) != null
                && (resultArray.get(0) instanceof ChargeCodeCategory)
                && parent != null)
        {
            wCCCategory = (ChargeCodeCategory) resultArray.get(0);
            wExpenseCategory = (ChargeCodeCategory) wCCCategory.getChildren()[0];

            //Creating Worksheet Financial
            wWSheetFinancial = new WorksheetFinancial[4];

            wExpenseHour = (ChargeCode) wExpenseCategory.getChildren()[0];
            wWSheetFinancial[HOUR_INDEX] = new WorksheetFinancial();
            wWSheetFinancial[HOUR_INDEX].setParent(parent);
            wWSheetFinancial[HOUR_INDEX].setChargeCode(wExpenseHour);

            wExpenseAvg = (ChargeCode) wExpenseCategory.getChildren()[1];
            wWSheetFinancial[AVG_COST_INDEX] = new WorksheetFinancial();
            wWSheetFinancial[AVG_COST_INDEX].setParent(parent);
            wWSheetFinancial[AVG_COST_INDEX].setChargeCode(wExpenseAvg);

            wExpenseLabor = (ChargeCode) wExpenseCategory.getChildren()[2];
            wWSheetFinancial[EXP_LABOR_INDEX] = new WorksheetFinancial();
            wWSheetFinancial[EXP_LABOR_INDEX].setParent(parent);
            wWSheetFinancial[EXP_LABOR_INDEX].setChargeCode(wExpenseLabor);

            wExpenseFTE = (ChargeCode) wExpenseCategory.getChildren()[3];
            wWSheetFinancial[FTE_INDEX] = new WorksheetFinancial();
            wWSheetFinancial[FTE_INDEX].setParent(parent);
            wWSheetFinancial[FTE_INDEX].setChargeCode(wExpenseFTE);

        }

        return wWSheetFinancial;
    }
    
    public static boolean isChargeCodeAssigned(ChargeCode node,
            MessageContext messageContext)
    {
        boolean state = false;

        Class[] returnTypes =
        {
            String.class
        };

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(FinancialManager.NAME_ELEMENT_ID);
        stmt.appendFrom(FinancialManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqual(FinancialManager.NAME_CHARGECODE_ID, node.getID());

        List result = null;
        try
        {
            result = ManagerUtil.select(returnTypes, node,
                (node.getName() == null) ? ValidationConstants.CHARGE_CODE_FIELD : node.getName(), stmt,
                null, messageContext);
        }
        catch (RPMException e)
        {
            state = false;
        }

        // result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            state = true;
        }

        return state;
    }

    
    
    public static boolean isWorkingDay(RPMCalendar cal, Calendar givenDay)
            throws Exception
    {
        if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY)
        {
            if (cal.getStandardSunday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY)
        {
            if (cal.getStandardMonday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.TUESDAY)
        {
            if (cal.getStandardTuesday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.WEDNESDAY)
        {
            if (cal.getStandardWednesday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.THURSDAY)
        {
            if (cal.getStandardThursday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.FRIDAY)
        {
            if (cal.getStandardFriday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else if (givenDay.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY)
        {
            if (cal.getStandardSaturday().getType() == CalendarWeekDayType.NonWorking)
                return false;
            else
                return true;
        }
        else
            return false;
    }

    public static int getCalWorkingDaysDuration(RPMCalendar cal,
            Calendar startDate, Calendar finishDate) throws Exception
    {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        Calendar givenDay = startDate;
        int duration = 1;

        if (cal != null && startDate != null && finishDate != null)
        {
            if ((formatter.format(startDate.getTime())).equals(formatter
                .format(finishDate.getTime())))
            {
                return duration;
            }

            else
            {
                while (!(formatter.format(givenDay.getTime())).equals(formatter
                    .format(finishDate.getTime())))

                {
                    givenDay = getNextDayDate(givenDay);
                    if (isWorkingDay(cal, givenDay))
                    {
                        duration++;
                    }
                }
            }
        }

        return duration;
    }

    public static Calendar getNextDayDate(Calendar aDate) throws Exception
    {
        Calendar result = DateUtil.getCalendarInstance();
        result.set(Calendar.YEAR, aDate.get(Calendar.YEAR));
        result.set(Calendar.MONTH, aDate.get(Calendar.MONTH));
        result.set(Calendar.DATE, aDate.get(Calendar.DATE));
        
        result.add(Calendar.DATE, 1);

        return result;

    }

    public static Calendar getTodayDate() throws Exception
    {
        Calendar today = DateUtil.getCalendarInstance();
        return today;
    }

    public static Calendar getYesterdayDate() throws Exception
    {
        Calendar yesterday = DateUtil.getCalendarInstance();
        yesterday.add(Calendar.DATE, -1);

        Calendar result = DateUtil.getCalendarInstance();
        result.set(Calendar.YEAR, yesterday.get(Calendar.YEAR));
        result.set(Calendar.MONTH, yesterday.get(Calendar.MONTH));
        result.set(Calendar.DATE, yesterday.get(Calendar.DATE));
        
        return result;

    }
    
    
    public static boolean isFTEFinancial(AbstractFinancial financial) throws RPMException
    {
        boolean state = false;
        
        if(financial!=null)
        {
            if (financial.getChargeCode() != null
                    && financial.getChargeCode().getFinancialCategory() != null)
            {
                Integer dbCode = FinancialCategoryUtil.toDBCode(financial
                    .getChargeCode().getFinancialCategory());

                if (dbCode.intValue() == FinancialUtil.EXPENSE_FTE)
                {
                    state=true;
                }
            }
        }
        
        return state;
    }
    
    public static boolean isEndOfDayCompareToCalendar(RPMCalendar refCalendar,
            Calendar today)
    {
        boolean isEndOfDay = false;

        if (refCalendar != null && today != null)
        {
            CalendarWeekDay weekDay = null;
            WorkInterval workInterval = null;
            WorkInterval winnerInterval = null;

            // find out which day is it...
            switch (today.get(Calendar.DAY_OF_WEEK))
            {
            case Calendar.SUNDAY:
                weekDay = refCalendar.getStandardSunday();
                break;
            case Calendar.MONDAY:
                weekDay = refCalendar.getStandardMonday();
                break;
            case Calendar.THURSDAY:
                weekDay = refCalendar.getStandardThursday();
                break;
            case Calendar.WEDNESDAY:
                weekDay = refCalendar.getStandardWednesday();
                break;
            case Calendar.TUESDAY:
                weekDay = refCalendar.getStandardTuesday();
                break;
            case Calendar.FRIDAY:
                weekDay = refCalendar.getStandardFriday();
                break;
            case Calendar.SATURDAY:
                weekDay = refCalendar.getStandardSaturday();
                break;

            }

            // Find highest time range in refCalendar
            if (weekDay != null && weekDay.getIntervals() != null)
            {
                for (int i = 0; i < weekDay.getIntervals().length; i++)
                {
                    workInterval = weekDay.getIntervals()[i];
                    winnerInterval = chooseHighestInterval(winnerInterval,
                        workInterval);
                }

                // verifying that we are at the end of workings hours for the day
                String[] endtime = winnerInterval.getTo().split(":");

                if (((today.get(Calendar.HOUR_OF_DAY) == Integer
                    .parseInt(endtime[0])) && (today.get(Calendar.MINUTE) > Integer
                    .parseInt(endtime[1])))
                        || (today.get(Calendar.HOUR_OF_DAY) > Integer
                            .parseInt(endtime[0])))
                {
                    isEndOfDay = true;
                }
            }

        }

        return isEndOfDay;
    }

    private static WorkInterval chooseHighestInterval(
            WorkInterval winnerInterval, WorkInterval workInterval)
    {
        WorkInterval winner = workInterval;
        String[] workTimeSlot = null;
        String[] winnerTimeSlot = null;

        if (winnerInterval != null)
        {
            workTimeSlot = workInterval.getTo().split(":");
            winnerTimeSlot = winnerInterval.getTo().split(":");

            if (workTimeSlot.length == 2 && winnerTimeSlot.length == 2)
            {
                if (((Integer.parseInt(winnerTimeSlot[0]) == Integer
                    .parseInt(workTimeSlot[0])) && (Integer
                    .parseInt(winnerTimeSlot[1]) > Integer
                    .parseInt(workTimeSlot[1])))
                        || ((Integer.parseInt(winnerTimeSlot[0]) > Integer
                            .parseInt(workTimeSlot[0]))))
                {
                    winner = winnerInterval;
                }

            }
        }

        return winner;
    }
    
    public static boolean isFinancialLineModified(AbstractFinancial financial)
    {
        boolean state = false;
        
        if (financial != null
                && (financial.testEstimateModified()
                        || financial.testActualModified()
                        || financial.testRemainingModified()
                        || financial.testETCModified()
                        || financial.testEACModified() || financial
                    .testEACVarianceModified()))
        {
            state = true;
        }
        
        return state;
    }
    
    public static boolean isFinancialLineEmpty(AbstractFinancial financial)
    {
        boolean state = false;
        
        if (financial != null
                && ((financial.getEstimate()==null || financial.getEstimate().doubleValue()==0)
                        && (financial.getActual()==null || financial.getActual().doubleValue()==0)
                        && (financial.getRemaining()==null || financial.getRemaining().doubleValue()==0)
                        && (financial.getETC()==null || financial.getETC().doubleValue()==0)
                        && (financial.getEAC()==null || financial.getEAC().doubleValue()==0)))
        {
            state = true;
        }
        
        return state;
    }
    
    public static boolean isFinancialColumnModified(int col,
            AbstractFinancial hour, AbstractFinancial avg, AbstractFinancial el)
    {
        boolean state = false;

        if (hour != null && avg != null && el != null)
        {
            switch (col)
            {
            case COL_ESTIMATE:
                if (hour.testEstimateModified() || avg.testEstimateModified()
                        || el.testEstimateModified())
                {
                    state = true;
                }
                break;
            case COL_ACTUAL:
                if (hour.testActualModified() || avg.testActualModified()
                        || el.testActualModified())
                {
                    state = true;
                }
                break;
            case COL_REMAINING:
                if (hour.testRemainingModified() || avg.testRemainingModified()
                        || el.testRemainingModified())
                {
                    state = true;
                }
                break;
            case COL_ETC:
                if (hour.testETCModified() || avg.testETCModified()
                        || el.testETCModified())
                {
                    state = true;
                }
                break;
            case COL_EAC:
                if (hour.testEACModified() || avg.testEACModified()
                        || el.testEACModified())
                {
                    state = true;
                }
                break;
            }
        }

        return state;
    }

    public static int getCurrentYear()
    {
        Calendar cal = DateUtil.getCalendarInstance();
        return cal.get(Calendar.YEAR);
    }

       
}