package com.ibm.rpm.framework;

import java.lang.reflect.Array;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Stack;

import org.apache.commons.logging.Log;

import com.ibm.rpm.framework.checkpoints.ScopeCheckpoint;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.stats.StatsUtil;
import com.ibm.rpm.xpathparser.Table;

/**
 * 
 * @author karim
 *  
 */
public abstract class AbstractModuleManager extends AbstractRPMObjectManager
{
    protected String localContextName;

    protected abstract Log getLogger();

    protected abstract void loadChildren(AbstractModule module,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException;

    public String getTableName()
    {
        return null;
    }

    protected String getAllFields()
    {
        return null;
    }

    protected String[] getFieldsNames()
    {
        return null;
    }

    protected HashMap getPropertiesMap()
    {
        return null;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        ExceptionUtil.handleMethodException(this,
            ErrorCodes.UNSUPPORTED_METHOD, ExceptionUtil.GET_CONTAINER_TYPE);
        
        return null;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleMethodException(this,
            ErrorCodes.UNSUPPORTED_METHOD, ExceptionUtil.DECODE_ROW);
        
        return null;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        ExceptionUtil.handleMethodException(this,
            ErrorCodes.UNSUPPORTED_METHOD, ExceptionUtil.FOREIGN_KEY);
        
        return null;
    }

    public final ArrayList load(Stack tableStack, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {

        if (messageContext == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.AM_INVALID_MC);
        }
        
        ScopeCheckpoint.getInstance().isValid(
            ((Table) tableStack.peek()).getObjectName(), rpmScope);
        
        ArrayList containers = null;
        
        if (!tableStack.isEmpty())
        {
            Table table = (Table) tableStack.peek();
            
            if (table.getConditions() != null)
            {
                String errorParams[] =
                {
                        table.getObjectName()
                };
                ExceptionUtil.handleRPMException(this,
                    ErrorCodes.XPATH_UNSUPPORTED_CONDITION, errorParams);
            }
            
            containers = new ArrayList();

            AbstractModule abstractModule = (AbstractModule) loadModule(
                rpmScope, messageContext);
            if (abstractModule != null)
            {
                containers.add(abstractModule);
            }

            if (containers.isEmpty())
            {
                containers = null;
            }
        }
        return containers;
    }

    public RPMObject load(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ScopeCheckpoint.getInstance().isValid(rpmObject, rpmScope);
        messageContext.getCache().resetLoadedContainers();
        if (messageContext == null)
        {
            ExceptionUtil.handleRPMException(this, ErrorCodes.AM_INVALID_MC);
        }
        return loadModule(rpmScope, messageContext);
    }

    private RPMObject loadModule(RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        AbstractModule abstractModule = (AbstractModule) createContainer(0);
        abstractModule.setContextName(localContextName);
        if (rpmScope != null)
        {
            loadChildren(abstractModule, rpmScope, messageContext, false);
        }
        return abstractModule;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        //Do nothing
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

    //Save section
    
    protected RPMObject[] saveElements(AbstractModule module, RPMObject[] objs,
            RPMObjectScope scope, MessageContext messageContext,
            Class childrenBaseClass) throws RPMException, SQLException,
            ParseException
    {
        module.setContextName(localContextName);
        propagateContextName(module, objs);
        RPMObject[] savedObjs = updateChildren(module, objs, scope,
            messageContext, childrenBaseClass);
        return savedObjs;
    } 

    protected RPMObject[] saveElements(AbstractModule module, RPMObject[] objs,
            boolean scope, MessageContext messageContext,
            Class childrenBaseClass) throws RPMException, SQLException,
            ParseException
    {
        module.setContextName(localContextName);
        propagateContextName(module, objs);
        RPMObject[] savedObjs = updateChildren(module, objs, scope,
            messageContext, childrenBaseClass);
        return savedObjs;
    } 
   
    protected RPMObject[] loadElements(AbstractModule abstractModule, RPMObject[] oldObjs,
            RPMObjectScope scope, Class baseClass, Object[] args,
            String extraWhere, MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        AbstractRPMObjectManager manager = (AbstractRPMObjectManager)getManagerInstance(baseClass.getName());
        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(abstractModule, oldObjs, scope,
            messageContext, this, args, extraWhere, localContextName, isDelta);
        RPMObject[] objs = null;
        if (temp != null)
        {
            objs = (RPMObject[]) Array.newInstance(baseClass, temp.size());
            temp.toArray(objs);
        }
        return objs;
    }
    
    // Same code as AbstractRPMObjectManager.doUpdateChildren
    // but reloads the child before the call to internalSave so that 
    // the child's modified flags get set.
    // 
    // This is a special case because modules don't have IDs even once 
    // they're saved. Can't call save directly because internalValidateSave 
    // will get called repeatedly.
    protected RPMObject[] doUpdateChildren(RPMObject parent,
            RPMObject[] children, RPMObjectScope childrenScope,
            MessageContext messageContext, Class childrenBaseClass)
            throws RPMException, SQLException, ParseException
    {
        // DESIGN NOTE: children manager is selected from the type of the
        // first non-null child.
        if (children != null)
        {
            // Update all children.
            RPMManagerFactory managerFactory = RPMManagerFactory.getInstance();
            RPMObjectManager childrenManager = null;
            ArrayList updatedChildrenList = new ArrayList();
            for (int i = 0; i < children.length; i++)
            {
                RPMObject originalChild = children[i];
                if (originalChild != null)
                {
                    // Select the right manager for children, if required.
                    if (childrenManager == null)
                    {
                        childrenManager = managerFactory
                            .getRPMObjectManager(originalChild);
                    }
                    originalChild.setParent(parent);
                    
                    // reload to set modified flags
                    if (originalChild.getID() != null)
                    {
                        //stats
                        long preloadStartTime = System.currentTimeMillis();
                        
                        messageContext.getCache().resetLoadedContainers();
                        originalChild = childrenManager.loadByPrimaryKey(
                            originalChild, childrenScope, messageContext, true);
                        
                        long preloadEndTime = System.currentTimeMillis();
                        StatsUtil.setPreloadDuration(messageContext, preloadEndTime-preloadStartTime);
                    }
                    
                    // save the child using the normal call
                    RPMObject updatedChild = childrenManager.internalSave(
                            originalChild, childrenScope, messageContext);
                    if (updatedChild != null)
                    {
                        updatedChildrenList.add(updatedChild);
                    }
                }
            }

            // Convert the children list to a children array.
            int updatedChildrenCount = updatedChildrenList.size();
            if (updatedChildrenCount > 0)
            {
                // Get the children base class from the first child, if
                // required.
                if (childrenBaseClass == null)
                {
                    childrenBaseClass = updatedChildrenList.get(0).getClass();
                }
                children = (RPMObject[]) Array.newInstance(childrenBaseClass,
                    updatedChildrenCount);
                updatedChildrenList.toArray(children);
            }
        }
        return children;
    }
}
