/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework;

import java.sql.ResultSet;

/**
 * @author Yasser Lulu
 *
 */
public final class RestoreRelevantFilteredFieldsParameter
{

    private AbstractRPMObjectManager manager;

    private RPMObject rpmObject;

    private RPMObject otherRpmObject;

    private Object[] primaryKey;

    private ResultSet resultSet;

    private MessageContext messageContext;

    private String contextName;

    private RPMObjectScope rpmScope;

    private boolean executedGetOtherRpmObject;

    public RestoreRelevantFilteredFieldsParameter(
            AbstractRPMObjectManager manager, RPMObject rpmObject,
            Object[] primaryKey, ResultSet resultSet,
            MessageContext messageContext, String contextName,
            RPMObjectScope rpmScope)
    {
        this.manager = manager;
        this.rpmObject = rpmObject;
        this.primaryKey = primaryKey;
        this.resultSet = resultSet;
        this.messageContext = messageContext;
        this.contextName = contextName;
        this.rpmScope = rpmScope;
    }

    public MessageContext getMessageContext()
    {
        return messageContext;
    }

    public RPMObject getOtherRpmObject()
    {
        if (!executedGetOtherRpmObject)
        {
            executedGetOtherRpmObject = true;

            try
            {
                otherRpmObject = manager.createContainer(primaryKey, resultSet,
                    messageContext, contextName, rpmScope, false);//don't add to cache
                otherRpmObject = manager.decodeRow(resultSet, otherRpmObject,
                    false);
            }
            catch (Exception ex)
            {
                RpmLog.getLog().warn(null, ex);
            }
        }

        return otherRpmObject;
    }

    public RPMObject getRpmObject()
    {
        return rpmObject;
    }

    public RPMObjectScope getRpmScope()
    {
        return rpmScope;
    }

}
