package com.ibm.rpm.framework;

import java.util.HashMap;
import java.util.Map;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NoInitialContextException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Those options come directly from the JNDI configuration.
 * 
 * Here is the process to add a new option:
 * 
 * 1 - In the web.xml in rpm-web-services, add the new variable definition. 2 -
 * in the ibm-web-bnd.xmi in rpm-web-services, add the new variable mapping. 3 -
 * in RpmOptions.java from rpm-data, add a constant for the variable name. 4 -
 * in RpmOptions.java from rpm-data, add a constant for the default variable
 * value. 5 - in RpmOptions.java from rpm-data, add a getter method to fetch the
 * variable. 6 - in RpmOptions.java from rpm-data, update
 * overrideAllOptoinsUsingDefaultValues() and logInitialOptions().
 */
public class RpmOptions
{
    public static final Integer DEFAULT_WEB_SERVICES_SESSION_TIMEOUT_DELAY_IN_SECONDS = new Integer(
        3600);

    public static final Boolean DEFAULT_WEB_SERVICES_CHECK_API_SECURITY_FLAG = Boolean.TRUE;

    public static final Boolean DEFAULT_WEB_SERVICES_ENABLED = Boolean.TRUE;

    public static final Boolean DEFAULT_WEB_SERVICES_SECURITY_ENABLED = Boolean.TRUE;

    public static final Boolean DEFAULT_USE_LDAP_OPTION = Boolean.FALSE;

    public static final String DEFAULT_LDAP_CONFIGURATION = "";

    public static final Boolean DEFAULT_WEB_SERVICES_SESSION_TIMEOUT_ENABLED = Boolean.TRUE;

    public static final String DEFAULT_WEB_SERVICES_SECURITY_CONTROLLER_CLASS_NAME = "";//for none
    //TODO eventaully, the default controller class name should be the fully qualified name of RPMSecurityControllerImpl class
    //public static final String DEFAULT_WEB_SERVICES_SECURITY_CONTROLLER_CLASS_NAME = "com.ibm.rpm.framework.security.controller.impl.rpm.RPMSecurityControllerImpl";//for RPM security controller

    // TODO re-enable this by default in 7.1.1.1
    public static final Boolean DEFAULT_WEB_SERVICES_SINGLE_CONNECTION_PER_CONTEXT = Boolean.FALSE;
    
    // NOTE: ALL THOSE STRINGS MUST SYNCHRONIZED WITH THE WEB.XML FILE
    // NOTE: ALL THOSE STRINGS MUST SYNCHRONIZED WITH THE WEB.XML FILE
    // NOTE: ALL THOSE STRINGS MUST SYNCHRONIZED WITH THE WEB.XML FILE
    public static final String WEB_SERVICES_ENABLED = "webServicesEnabled";

    public static final String WEB_SERVICES_CHECK_API_SECURITY_FLAG = "webServicesUseApiSecurityFlagOnLogin";

    public static final String WEB_SERVICES_SESSION_TIMEOUT_ENABLED = "webServicesUseSessionTimeout";

    public static final String WEB_SERVICES_SESSION_TIMEOUT_DELAY_IN_SECONDS = "webServicesSessionTimeoutDelayInSeconds";

    public static final String WEB_SERVICES_SECURITY_ENABLED = "webServicesSecurityEnabled";

    public static final String USE_LDAP_OPTION = "useLdapAuthentication";

    public static final String LDAP_CONFIGURATION = "ldapConfiguration";

    public static final String WEB_SERVICES_SECURITY_CONTROLLER_CLASS_NAME = "webServicesSecurityControllerClassName";

    public static final String WEB_SERVICES_SINGLE_CONNECTION_PER_CONTEXT = "webServicesSingleConnectionPerContext";
    
    // NOTE: ALL THOSE STRINGS MUST SYNCHRONIZED WITH THE WEB.XML FILE
    // NOTE: ALL THOSE STRINGS MUST SYNCHRONIZED WITH THE WEB.XML FILE
    // NOTE: ALL THOSE STRINGS MUST SYNCHRONIZED WITH THE WEB.XML FILE

    private Map overrideOptions = new HashMap();

    private boolean bypassJndi = false;

    public void overrideAllOptoinsUsingDefaultValues()
    {
        overrideOption(WEB_SERVICES_ENABLED, DEFAULT_WEB_SERVICES_ENABLED);
        overrideOption(WEB_SERVICES_CHECK_API_SECURITY_FLAG,
            DEFAULT_WEB_SERVICES_CHECK_API_SECURITY_FLAG);
        overrideOption(WEB_SERVICES_SESSION_TIMEOUT_ENABLED,
            DEFAULT_WEB_SERVICES_SESSION_TIMEOUT_ENABLED);
        overrideOption(WEB_SERVICES_SESSION_TIMEOUT_DELAY_IN_SECONDS,
            DEFAULT_WEB_SERVICES_SESSION_TIMEOUT_DELAY_IN_SECONDS);
        overrideOption(WEB_SERVICES_SECURITY_ENABLED,
            DEFAULT_WEB_SERVICES_SECURITY_ENABLED);
        overrideOption(USE_LDAP_OPTION, DEFAULT_USE_LDAP_OPTION);
        overrideOption(LDAP_CONFIGURATION, DEFAULT_LDAP_CONFIGURATION);
        overrideOption(WEB_SERVICES_SINGLE_CONNECTION_PER_CONTEXT,
            DEFAULT_WEB_SERVICES_SINGLE_CONNECTION_PER_CONTEXT);
    }

    public boolean isSingleConnectionPerContext()
    {
        // TODO re-enable this by default in 7.1.1.1
        return false;
        // return getBoolean(WEB_SERVICES_SINGLE_CONNECTION_PER_CONTEXT,
        //    DEFAULT_WEB_SERVICES_SINGLE_CONNECTION_PER_CONTEXT).booleanValue();
    }
    
    public boolean isLdapUsed()
    {
        return getBoolean(USE_LDAP_OPTION, DEFAULT_USE_LDAP_OPTION)
            .booleanValue();
    }

    public String getLdapConfiguration()
    {
        return getString(LDAP_CONFIGURATION, DEFAULT_LDAP_CONFIGURATION);
    }

    public boolean isWebServicesSecurityEnabled()
    {
        return getBoolean(WEB_SERVICES_SECURITY_ENABLED,
            DEFAULT_WEB_SERVICES_SECURITY_ENABLED).booleanValue();
    }

    public boolean isSecurityFlagUsed()
    {
        return getBoolean(WEB_SERVICES_CHECK_API_SECURITY_FLAG,
            DEFAULT_WEB_SERVICES_CHECK_API_SECURITY_FLAG).booleanValue();
    }

    public boolean isWebServicesEnabled()
    {
        return getBoolean(WEB_SERVICES_ENABLED, DEFAULT_WEB_SERVICES_ENABLED)
            .booleanValue();
    }

    public int getSessionTimeoutInSeconds()
    {
        int retVal = getInteger(WEB_SERVICES_SESSION_TIMEOUT_DELAY_IN_SECONDS,
            DEFAULT_WEB_SERVICES_SESSION_TIMEOUT_DELAY_IN_SECONDS).intValue();
        if (retVal < 1)
        {
            logBadOption(WEB_SERVICES_SESSION_TIMEOUT_DELAY_IN_SECONDS,
                "Invalid number. Expected a positive integer bigger than 0.",
                Integer.toString(retVal));
            retVal = DEFAULT_WEB_SERVICES_SESSION_TIMEOUT_DELAY_IN_SECONDS
                .intValue();
        }
        return retVal;
    }

    public boolean isSessionTimeoutUsed()
    {
        return getBoolean(WEB_SERVICES_SESSION_TIMEOUT_ENABLED,
            DEFAULT_WEB_SERVICES_SESSION_TIMEOUT_ENABLED).booleanValue();
    }

    public void logInitialOptions()
    {
        if (isLdapUsed())
        {
            logger.info("Authentication method: Using LDAP Authentication");
        }
        else
        {
            logger.info("Authentication method: Using RPM Authentication");
        }

        logger.info("Web Services Enabled: " + isWebServicesEnabled());
        logger.info("Web Services Application Security Enabled: "
                + isWebServicesSecurityEnabled());
        logger.info("Web Services Security Flag used on Login: "
                + isSecurityFlagUsed());
        logger.info("Web Services Session Timeout Enabled: "
                + isSessionTimeoutUsed());
        logger.info("Web Services Session Timeout in Seconds: "
                + getSessionTimeoutInSeconds());
        logger.info("LDAP Configuration: " + getLdapConfiguration());
    }

    private Map badOptionMap = new HashMap();

    /**
     * Log a bad option. This method is synchronized to make sure the
     * badOptionMap is kept consistent.
     * 
     * This method will make sure that a bad parameter is logged only once
     * instead of being logged everytime it's accessed.
     * 
     * @param optionName
     *            the option name
     * @param reason
     *            the reason why the parameter is bad
     * @param badValue
     *            the bad value in string form
     */
    public synchronized void logBadOption(String optionName, String reason,
            String badValue)
    {
        String currentBadValue = (String) badOptionMap.get(optionName);

        // only log bad options once since they might be fetched very often
        if (currentBadValue != null && badValue != null
                && !badValue.equals(currentBadValue))
        {
            logger.error("Invalid option specified : " + optionName
                    + ". Reason: " + reason + ". Current Invalid Value: "
                    + badValue);

            badOptionMap.put(optionName, badValue);
        }
    }

    protected static Log logger = LogFactory.getLog(RpmOptions.class);

    private static RpmOptions INSTANCE = new RpmOptions();

    public static RpmOptions getDefaultOptions()
    {
        return INSTANCE;
    }

    /**
     * Returns the value of the option identified by <code>propertyName</code>.
     * Returns null if the option is not found.
     * 
     * @param propertyName
     *            Name of the option to be returned.
     * @return Value of the option, as a String. Nullable.
     */
    public String getString(String propertyName)
    {
        return getString(propertyName, null);
    }

    /**
     * Returns the value of the option identified by <code>propertyName</code>.
     * If the value is null, then the <code>defaultValue</code> will be
     * returned.
     * 
     * @param propertyName
     *            Name of the option to be returned.
     * @param defaultValue
     *            Default value for this option.
     * @return Value of the option, as a String. Nullable.
     */
    public String getString(String propertyName, String defaultValue)
    {
        Object objectValue = getObjectFromJndi(propertyName, defaultValue);
        String value = defaultValue;
        if (objectValue != null)
        {
            value = objectValue.toString();
        }
        return value;
    }

    /**
     * Returns the value of the option identified by <code>propertyName</code>.
     * Returns null if the option is not found.
     * 
     * @param propertyName
     *            Name of the option to be returned.
     * @return Value of the option, as an Integer. Nullable.
     */
    public Integer getInteger(String propertyName)
    {
        return getInteger(propertyName, null);
    }

    /**
     * Returns the value of the option identified by <code>propertyName</code>.
     * If the value is null, then the <code>defaultValue</code> will be
     * returned.
     * 
     * @param propertyName
     *            Name of the option to be returned.
     * @param defaultValue
     *            Default value for this option.
     * @return Value of the option, as an Integer. Nullable.
     */
    public Integer getInteger(String propertyName, Integer defaultValue)
    {
        Integer value = defaultValue;
        try
        {
            Object val = getObjectFromJndi(propertyName, defaultValue);
            if (val instanceof Integer)
            {
                value = (Integer) val;
            }
            else if (val instanceof Number)
            {
                Number numeric = (Number) val;
                value = new Integer(numeric.intValue());
            }
            else if (val instanceof String)
            {
                value = new Integer((String) val);
            }
            else
            {
                logBadOption(
                    propertyName,
                    "Invalid object type generated. Expected java.lang.Integer.",
                    value.getClass().toString());
            }
        }
        catch (Exception e)
        {
            logBadOption(propertyName, "Invalid number defined: " + e + ".",
                value.toString());
        }

        return value;
    }

    /**
     * Returns the value of the option identified by <code>propertyName</code>.
     * Returns null if the option is not found.
     * 
     * @param propertyName
     *            Name of the option to be returned.
     * @return Value of the option, as an Integer. Nullable.
     */
    public Boolean getBoolean(String propertyName)
    {
        return getBoolean(propertyName, null);
    }

    /**
     * Returns the value of the option identified by <code>propertyName</code>.
     * If the value is null, then the <code>defaultValue</code> will be
     * returned.
     * 
     * @param propertyName
     *            Name of the option to be returned.
     * @param defaultValue
     *            Default value for this option.
     * @return Value of the option, as an Boolean. Nullable.
     */
    public Boolean getBoolean(String propertyName, Boolean defaultValue)
    {
        Boolean value = defaultValue;

        Object val = getObjectFromJndi(propertyName, defaultValue);
        if (val != null)
        {
            if (val instanceof Boolean)
            {
                value = (Boolean) val;
            }
            else if (val instanceof String)
            {
                String toString = (String) val;
                if (toString.equalsIgnoreCase(Boolean.FALSE.toString()))
                {
                    value = Boolean.FALSE;
                }
                else if (toString.equalsIgnoreCase(Boolean.TRUE.toString()))
                {
                    value = Boolean.TRUE;
                }
                else
                {
                    logBadOption(
                        propertyName,
                        "Invalid boolean value genearted. Expected TRUE or FALSE.",
                        value.toString());
                }
            }
            else
            {
                logBadOption(
                    propertyName,
                    "Invalid object type generated. Expected java.lang.Boolean.",
                    value.getClass().toString());
            }
        }
        return value;
    }

    private Object getObjectFromJndi(String parameterName, Object defaultValue)
    {

        // first we check if there is an override for this option
        Object retVal = overrideOptions.get(parameterName);

        if (retVal == null && !bypassJndi)
        {
            // we have no override, fetch the real value from the JNDI
            try
            {
                Context ctx = new InitialContext();
                retVal = ctx.lookup("java:comp/env/" + parameterName);
            }
            catch (NoInitialContextException ex)
            {
                // running in standalone mode,
                logger.debug("JNDI not initialized. "
                        + "Bypassing all future references to the JNDI. "
                        + ex.getMessage());

                bypassJndi = true;
            }
            catch (Exception ex)
            {
                logger.debug("Failed to read JNDI parameter " + parameterName
                        + " because of error : " + ex.getMessage()
                        + ". Overriding with default : " + defaultValue, ex);

                overrideOption(parameterName, defaultValue);
            }
        }
        return retVal;
    }

    public void overrideOption(String optionName, Object value)
    {
        overrideOptions.put(optionName, value);
    }

    public void clearOptionOverride(String optionName)
    {
        overrideOptions.remove(optionName);
    }

    public void clearAllOptionOverride()
    {
        overrideOptions.clear();
    }

    public String getWebServiceSecurityControllerClassName()
    {
        return getString(WEB_SERVICES_SECURITY_CONTROLLER_CLASS_NAME,
            DEFAULT_WEB_SERVICES_SECURITY_CONTROLLER_CLASS_NAME);
    }
}
