/*
 * Created on Nov 27, 2006
 */
package com.ibm.rpm.framework.checkpoints;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SessionCheckoutsDAO;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.resource.containers.Resource;

public class ForceCheckinCheckpoint
{
    public static boolean isForceCheckinValidForUser(String operation,
            Resource user, MessageContext messageContext) throws RPMException
    {
        RPMObject currentUser = messageContext.getUser();
        if (currentUser != null && currentUser.getID() != null
                && currentUser.getID().equals(user.getID()))
        {
            return true;
        }
        
        String[] errorParams = new String[]
        {
                operation, "user"
        };
        RPMException e = new RPMException(ErrorCodes.CANNOT_FORCE_CHECKIN,
            errorParams);
        e.setSeverity(SeverityLevel.Error);
        messageContext.addExceptionNoThrow(e);
        return false;
    }

    public static boolean isForceCheckinValidForSession(String operation,
            String targetSessionID, MessageContext messageContext)
            throws RPMException
    {
        Resource owner = (Resource) messageContext.getUser();
        SessionCheckoutsDAO dao = messageContext.getFactory()
            .getSessionCheckoutDAO();
        
        if (dao.isSessionOwner(targetSessionID, owner, messageContext))
        {
            return true;
        }
        
        String[] errorParams = new String[]
        {
                operation, "session"
        };
        RPMException e = new RPMException(ErrorCodes.CANNOT_FORCE_CHECKIN,
            errorParams);
        e.setSeverity(SeverityLevel.Error);
        messageContext.addExceptionNoThrow(e);
        return false;
    }

    public static boolean isForceCheckinValidForRPMObject(String operation,
            String rpmObjectCKID, MessageContext messageContext)
            throws RPMException
    {
        Resource owner = (Resource) messageContext.getUser();
        SessionCheckoutsDAO dao = messageContext.getFactory()
            .getSessionCheckoutDAO();
        Resource rpmUserCheckout = messageContext.getCheckOutRecUser();
        
        if (dao.isCheckoutOwner(rpmObjectCKID, owner, messageContext)
                && rpmUserCheckout != null && rpmUserCheckout.getID().equals(
                    owner.getID()))
        {
            return true;
        }

        String[] errorParams = new String[]
        {
                operation, "user"
        };
        RPMException e = new RPMException(ErrorCodes.CANNOT_FORCE_CHECKIN,
            errorParams);
        e.setSeverity(SeverityLevel.Error);
        messageContext.addExceptionNoThrow(e);
        return false;
    }
}
