/*
 * Created on 21-Nov-2005
 */
package com.ibm.rpm.framework.checkpoints;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.RPMObjectScopeUtil;
import com.ibm.rpm.framework.util.StringUtil;

/**
 * @author dominiqu
 */
public class ScopeCheckpoint
{
    protected static ScopeCheckpoint instance = new ScopeCheckpoint();

    public static ScopeCheckpoint getInstance()
    {
        return instance;
    }

    /**
     * @param rpmObject
     * @param rpmScope
     * @throws RPMException
     *             With ErrorCode INVALID_SCOPE if rpmScope is not valid for
     *             rpmObject.
     */
    public void isValid(RPMObject rpmObject, RPMObjectScope rpmScope) throws RPMException
    {
        isValid(rpmObject.getClass().getName(), rpmScope);
    }

    /**
     * @param rpmObjectName
     * @param rpmScope
     * @throws RPMException
     *             With ErrorCode INVALID_SCOPE if rpmScope is not valid for
     *             rpmObjectName.
     */
    public void isValid(String rpmObjectName, RPMObjectScope rpmScope)
            throws RPMException
    {
        if (rpmScope == null)
            return;

        if(rpmScope.getClass() != RPMObjectScope.class){
	        Class correctScopeType = RPMObjectScopeUtil.getInstance().getScope(
	            StringUtil.getShortClassName(rpmObjectName));
	        if (correctScopeType != rpmScope.getClass())
	        {
	            String[] errorParams =
	            {
	                StringUtil.getShortClassName(rpmObjectName),
	                StringUtil.getShortClassName(rpmScope.getClass()),
	                StringUtil.getShortClassName(correctScopeType)
	            };
	            throw new RPMException(ErrorCodes.INVALID_SCOPE, errorParams, rpmObjectName);
	        }
        }
    }

    /**
     * Verified if rpmScope is valid for rpmObjectName. If it isn't, adds an
     * RPMException with ErrorCode INVALID_SCOPE to messageContext.
     * 
     * @param rpmObject
     * @param rpmScope
     * @param messageContext
     * @return true if rpmScope is valid for rpmObjectName, false otherwise.
     */
    public boolean isValid(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext)
    {
        if (rpmScope == null)
            return true;

        if (rpmScope.getClass() != RPMObjectScope.class)
        {
            Class correctScopeType = RPMObjectScopeUtil.getInstance().getScope(
                rpmObject.getClass());
            if (correctScopeType != rpmScope.getClass())
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(rpmObject.getClass()),
                        StringUtil.getShortClassName(rpmScope.getClass()),
                        StringUtil.getShortClassName(correctScopeType)
                };
                RPMException e = new RPMException(ErrorCodes.INVALID_SCOPE,
                    errorParams, rpmObject.getClass().getName());
                GenericValidator.addException(e, messageContext);
                return false;
            }
        }
        return true;
    }
}
