/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.framework.RPMException;

/**
 * @author Yasser Lulu & Marc-Andr Lamothe
 * 
 */
public class ContainerSecurityDescriptor extends AbstractSecurityDescriptor
{
    public static final ContainerSecurityDescriptor EMPTY_CONTAINER_DESCR = new ContainerSecurityDescriptor(
        "");

    private String rpmObjectId;

    private boolean canDelete;

    private Map fieldSecurityDescriptors = Collections.EMPTY_MAP;

    private Map operationSecurityDescriptors = Collections.EMPTY_MAP;

    private Map creationOperationSecurityDescriptors = Collections.EMPTY_MAP;

    private Map rtfSecurityDescriptors = Collections.EMPTY_MAP;

    private Map customFieldSecurityDescriptors = Collections.EMPTY_MAP;

    public ContainerSecurityDescriptor(String rpmObjectId)
    {
        this.rpmObjectId = rpmObjectId;
    }

    public final String getRpmObjectId()
    {
        return rpmObjectId;
    }

    /**
     * @return the canDelete
     */
    public boolean isCanDelete()
    {
        return canDelete;
    }

    /**
     * @param canDelete the canDelete to set
     */
    public void setCanDelete(boolean canDelete)
    {
        this.canDelete = canDelete;
    }

    /**
     * @return the fieldSecurityDescriptors
     */
    public Map getFieldSecurityDescriptors()
    {
        return fieldSecurityDescriptors;
    }

    /**
     * @param fieldSecurityDescriptors the fieldSecurityDescriptors to set
     */
    public void setFieldSecurityDescriptors(Map fieldSecurityDescriptors)
    {
        this.fieldSecurityDescriptors = fieldSecurityDescriptors;
    }

    /**
     * @return the operationSecurityDescriptors
     */
    public Map getOperationSecurityDescriptors()
    {
        return operationSecurityDescriptors;
    }

    /**
     * @param operationSecurityDescriptors the operationSecurityDescriptors to set
     */
    public void setOperationSecurityDescriptors(Map operationSecurityDescriptors)
    {
        this.operationSecurityDescriptors = operationSecurityDescriptors;
    }

    /**
     * @return the creationOperationSecurityDescriptors
     */
    public Map getCreationOperationSecurityDescriptors()
    {
        return creationOperationSecurityDescriptors;
    }

    /**
     * @param creationOperationSecurityDescriptors the creationOperationSecurityDescriptors to set
     */
    public void setCreationOperationSecurityDescriptors(
            Map creationOperationSecurityDescriptors)
    {
        this.creationOperationSecurityDescriptors = creationOperationSecurityDescriptors;
    }

    /**
     * @return the rtfAssignmentSecurityDescriptors
     */
    public Map getRTFSecurityDescriptors()
    {
        return rtfSecurityDescriptors;
    }

    /**
     * @param rtfAssignmentSecurityDescriptors the rtfAssignmentSecurityDescriptors to set
     */
    public void setRTFSecurityDescriptors(
            Map rtfSecurityDescriptors)
    {
        this.rtfSecurityDescriptors = rtfSecurityDescriptors;
    }

    /**
     * @return the customFieldAssignmentSecurityDescriptors
     */
    public Map getCustomFieldSecurityDescriptors()
    {
        return customFieldSecurityDescriptors;
    }

    /**
     * @param customFieldAssignmentSecurityDescriptors the customFieldAssignmentSecurityDescriptors to set
     */
    public void setCustomFieldSecurityDescriptors(
            Map customFieldSecurityDescriptors)
    {
        this.customFieldSecurityDescriptors = customFieldSecurityDescriptors;
    }  
   

    //Combine boolean values of both descriptors using an OR logical connective. Creates a new 
    //ContainerSecurityDescriptor instance with the combined results and returns it 
    AbstractSecurityDescriptor combine(AbstractSecurityDescriptor other,
            boolean usingOr) throws RPMException
    {
        if (!(other instanceof ContainerSecurityDescriptor))
        {
            throw new RPMException(
                "Cannot combine an object that is not a ContainerSecurityDescriptors");
        }

        ContainerSecurityDescriptor otherContainerDescr = (ContainerSecurityDescriptor) other;
       

        if (otherContainerDescr instanceof NullContainerSecurityDescriptor)
        {
            return this;//should not combine with NullContainerSecurityDescriptor
        }

        ContainerSecurityDescriptor result = new ContainerSecurityDescriptor(
            getRpmObjectId());
        result.setFieldSecurityDescriptors(new HashMap());
        result.setOperationSecurityDescriptors(new HashMap());
        result.setCreationOperationSecurityDescriptors(new HashMap());
        result.setRTFSecurityDescriptors(new HashMap());
        result.setCustomFieldSecurityDescriptors(new HashMap());

        if (usingOr)
        {
            result.setCanDelete(isCanDelete()
                    || otherContainerDescr.isCanDelete());
        }
        else
        {
            result.setCanDelete(isCanDelete()
                    && otherContainerDescr.isCanDelete());
        }
        SecurityDescriptorUtil.combineSecurityDescriptorsMaps(result
            .getFieldSecurityDescriptors(), getFieldSecurityDescriptors(),
            otherContainerDescr.getFieldSecurityDescriptors(), usingOr);

        SecurityDescriptorUtil.combineSecurityDescriptorsMaps(result
            .getOperationSecurityDescriptors(),
            getOperationSecurityDescriptors(), otherContainerDescr
                .getOperationSecurityDescriptors(), usingOr);

        SecurityDescriptorUtil.combineSecurityDescriptorsMaps(result
            .getCreationOperationSecurityDescriptors(),
            getCreationOperationSecurityDescriptors(), otherContainerDescr
                .getCreationOperationSecurityDescriptors(), usingOr);

        SecurityDescriptorUtil.combineSecurityDescriptorsMaps(result
            .getRTFSecurityDescriptors(), getRTFSecurityDescriptors(),
            otherContainerDescr.getRTFSecurityDescriptors(), usingOr);

        SecurityDescriptorUtil.combineSecurityDescriptorsMaps(result
            .getCustomFieldSecurityDescriptors(),
            getCustomFieldSecurityDescriptors(), otherContainerDescr
                .getCustomFieldSecurityDescriptors(), usingOr);

        return result;
    }

}
