/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security;

import com.ibm.rpm.framework.RPMException;

/**
 * @author Yasser Lulu
 * 
 */
public class FieldSecurityDescriptor extends BaseSecurityDescriptor
{
    private boolean canView;

    private boolean canEdit;

    /**
     * @return the canEdit
     */
    public boolean isCanEdit()
    {
        return canEdit;
    }

    /**
     * @param canEdit the canEdit to set
     */
    public void setCanEdit(boolean canEdit)
    {
        this.canEdit = canEdit;
    }

    /**
     * @return the canView
     */
    public boolean isCanView()
    {
        return canView;
    }

    /**
     * @param canView the canView to set
     */
    public void setCanView(boolean canView)
    {
        this.canView = canView;
    }

    /**
     

     /**
     * @author Marc-Andr Lamothe
     * @return this == param
     */
    public boolean equals(Object obj)
    {
        return obj instanceof FieldSecurityDescriptor && super.equals(obj)
                && isCanView() == ((FieldSecurityDescriptor) obj).isCanView()
                && isCanEdit() == ((FieldSecurityDescriptor) obj).isCanEdit();
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.security.AbstractSecurityDescriptor#combine(com.ibm.rpm.framework.security.AbstractSecurityDescriptor)
     */
    AbstractSecurityDescriptor combine(AbstractSecurityDescriptor other,
            boolean usingOr) throws RPMException
    {
        if (!(other instanceof FieldSecurityDescriptor))
        {
            throw new RPMException(
                "Cannot combine an object that is not a FieldSecurityDescriptor");
        }
        FieldSecurityDescriptor otherDescr = (FieldSecurityDescriptor) other;
        if (!getName().equals(otherDescr.getName()))
        {
            throw new RPMException(
                "Cannot combine BaseConatinerFieldSecurityDescriptor of different field");
        }
        FieldSecurityDescriptor result = new FieldSecurityDescriptor();
        result.setName(getName());
        if (usingOr)
        {
            result.setCanEdit(isCanEdit() || otherDescr.isCanEdit());
            result.setCanView(isCanView() || otherDescr.isCanView());
        }
        else
        {
            result.setCanEdit(isCanEdit() && otherDescr.isCanEdit());
            result.setCanView(isCanView() && otherDescr.isCanView());
        }
        return result;
    }

}
