/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security;

import com.ibm.rpm.framework.RPMException;

/**
 * @author Yasser Lulu 
 *
 * This class to be used for RPMObjects that have no security enforced since it defaults
 * basically to true -as if there is no security at all. the reason we added this class 
 * is that some clients (the web-ui) expect a ContainerSecurityDescriptor to be associated with
 * every rpm-object returned or else they will disable everything on this rpm-object, 
 * so, if an rpm-object has no security at all, we will associate this NullContainerSecurityDescriptor
 * so that the client will not disable it 
 */
public final class NullContainerSecurityDescriptor extends
        ContainerSecurityDescriptor
{

    public NullContainerSecurityDescriptor(String rpmObjectId)
    {
        super(rpmObjectId);
        super.setCanDelete(true);
    }

    AbstractSecurityDescriptor combine(AbstractSecurityDescriptor other,
            boolean usingOr) throws RPMException
    {
        return other;
    }

    public boolean equals(Object obj)
    {
        if (obj instanceof NullContainerSecurityDescriptor)
        {
            NullContainerSecurityDescriptor other = (NullContainerSecurityDescriptor) obj;
            return getRpmObjectId().equals(other.getRpmObjectId());
        }
        return false;
    }
}
