/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.ibm.rpm.framework.RPMException;

/**
 * @author Yasser Lulu
 *
 */
public final class SecurityDescriptorUtil
{

    private SecurityDescriptorUtil()
    {
        //
    }

    //we could have used a functor here, however a simple boolean flag will do
    public static final void combineSecurityDescriptorsMaps(
            Map combinedSecurityDescriptorsMap, Map descrsMap_1,
            Map descrsMap_2, boolean usingOr) throws RPMException
    {
        if (descrsMap_1.isEmpty() && descrsMap_2.isEmpty())
        {
            return;
        }
        Set keys = new HashSet(descrsMap_1.keySet());
        keys.addAll(descrsMap_2.keySet());
        Iterator it = keys.iterator();
        while (it.hasNext())
        {
            Object key = it.next();
            AbstractSecurityDescriptor descr_1 = (AbstractSecurityDescriptor) descrsMap_1
                .get(key);
            AbstractSecurityDescriptor descr_2 = (AbstractSecurityDescriptor) descrsMap_2
                .get(key);
            if (descr_1 != null)
            {
                if (descr_2 != null)
                {
                    combinedSecurityDescriptorsMap.put(key, descr_1.combine(descr_2, usingOr));
                }
                else
                {
                    combinedSecurityDescriptorsMap.put(key, descr_1);
                }
            }
            else
            {
                //since we got here, then descr_1 is null which means this can't be null
                combinedSecurityDescriptorsMap.put(key, descr_2);
            }
        }
    }

}
