/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller;

import java.util.ArrayList;
import java.util.Map;

import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.RestoreRelevantFilteredFieldsParameter;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;

/**
 * @author Yasser Lulu
 * 
 */
public class BaseSecurityController implements ISecurityController
{

    public void fillContainerSecurityDescriptors(
            Map containerSecurityDescriptorsMap, MessageContext messageContext)
            throws RPMException
    {
        //do nothing since no RPMObject is secured, therefore Descrs aren't needed, even empty ones
    }

    public SecurityValidationResult applyFieldFiltering(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            String fieldName, MessageContext messageContext)
            throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            String fieldName, MessageContext messageContext)
            throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canView(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult restoreRelevantFilteredFields(
            RestoreRelevantFilteredFieldsParameter param) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canAcceptTaskAssignmentChanges(
            MessageContext messageContext, TaskAssignment taskAssignment,
            TaskAssignmentScope scope, boolean acceptEETC,
            boolean acceptForecastDates) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canAssignScoreCard(
            MessageContext messageContext, RPMObject objectToAssign,
            RPMObjectScope objectToAssignScope, Scorecard scoreCardToAssign,
            ReloadType reloadType) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canAssociateWorkflow(
            MessageContext messageContext, WorkflowProcess process,
            GenericProject genericProject, boolean associate, boolean isDefault)
            throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canCancelWorkflow(
            MessageContext messageContext, RunningWorkflowProcess runningProcess)
            throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canCheckOut(MessageContext messageContext,
            RPMObject rpmObject, boolean isTimeless) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canConvertContainer(
            MessageContext messageContext, RPMObject inputContainer,
            RPMObject outputContainer, RPMObjectScope rpmScope)
            throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canCopyAssignmentProposedToPlan(
            MessageContext messageContext,
            ResourceTaskAssignment resourceTaskAssignment,
            ResourceTaskAssignmentScope scope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canCopyProposedToPlan(
            MessageContext messageContext, WorkElement workElement,
            WorkElementScope workElementScope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canCreateObjectFromTemplate(
            MessageContext messageContext, RPMObject inputObject,
            RPMObject template, RPMObjectScope scope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canCreate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canDelete(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canMoveAndTransferUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canMoveUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canPublishScorecard(
            MessageContext messageContext, AssignedScorecard assignedScorecard,
            AbstractScorecardScope scorecardScope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canRejectTaskAssignmentChanges(
            MessageContext messageContext, TaskAssignment taskAssignment,
            TaskAssignmentScope scope, boolean rejectEETC,
            boolean rejectForecastDates) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canReplaceProfileWithResource(
            MessageContext messageContext, ResourceTaskAssignment rta,
            ResourceTaskAssignmentScope rtaScope, Resource resource)
            throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canReplaceResourceWithProfile(
            MessageContext messageContext, ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canResumeWorkflow(
            MessageContext messageContext,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canRollupProjectEV(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope scope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canUpdate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canSaveProjectBaseline(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope scope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canStartWorkflow(
            MessageContext messageContext, WorkflowProcess process,
            RPMObject object, RunningWorkflowProcessScope scope)
            throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canSuspendWorkflow(
            MessageContext messageContext,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canTransferUnderScope(
            MessageContext messageContext, AbstractAggregateScope scopeElement,
            WorkElement workElement) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canTransferUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
    
    public SecurityValidationResult canLockDocument(
			MessageContext messageContext, GenericDocument documentToLock)
			throws RPMException 
	{
		return SecurityValidationResult.UNDEFINED_RESULT;
	}

    public SecurityValidationResult canUnlockDocument(
			MessageContext messageContext, GenericDocument documentToUnlock)
			throws RPMException 
	{
		return SecurityValidationResult.UNDEFINED_RESULT;
	}
    
    public ArrayList applyFiltering(MessageContext messageContext,
            ArrayList loadedObjects) throws RPMException
    {
        return loadedObjects;
    }

    public SecurityValidationResult canUpdateOnCreate(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope rpmScope) throws RPMException
    {
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
}