/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author Yasser Lulu
 *
 */
public final class SecurityValidationResult
{
    public static final class ValueType
    {
        public final int ordinal;

        public final String name;

        private ValueType(int ordinal, String name)
        {
            this.ordinal = ordinal;
            this.name = name;
        }
    }

    public static final class RPMSecurityException extends RPMException
    {
        private static final String[] GENERAL_FAILURE_MESSAGE_PARAM = new String[]
        {
            "be granted. An exception occured while processing security validation"
        };

        private RPMSecurityException(String[] errorParams)
        {
            super(ErrorCodes.INSUFFICIENT_SECURITY_RIGHTS, errorParams);
        }

        public RPMSecurityException(Exception cause)
        {
            super(ErrorCodes.INSUFFICIENT_SECURITY_RIGHTS,
                GENERAL_FAILURE_MESSAGE_PARAM, cause);
        }
    }

    public static final ValueType UNDEFINED = new ValueType(0, "UNDEFINED");

    public static final ValueType TRUE = new ValueType(1, "TRUE");

    public static final ValueType FALSE = new ValueType(2, "FALSE");

    public static final SecurityValidationResult TRUE_RESULT = new SecurityValidationResult(
        null, TRUE);

    public static final SecurityValidationResult UNDEFINED_RESULT = new SecurityValidationResult(
        null, UNDEFINED);

    public final ValueType type;

    public final RPMSecurityException securityException;

    private SecurityValidationResult(RPMSecurityException securityException,
            ValueType type)
    {
        this.securityException = securityException;
        this.type = type;
    }

    public static SecurityValidationResult makeFalseResult(String operationName)
    {
        return new SecurityValidationResult(new RPMSecurityException(
            new String[]
            {
                operationName
            }), FALSE);
    }
    
    public static RPMException makeException(Exception cause)
    {
        return new RPMSecurityException(cause);
    }
}
