package com.ibm.rpm.framework.security.controller.impl;

import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.document.containers.DocumentBlobDetails;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.framework.LoadParams;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.RpmLog;
import com.ibm.rpm.framework.RPMObjectCache.RemovedContainerFromCache;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.security.SecurityFlags.VIRTUAL;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult.ValueType;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.framework.util.ContainerFieldInfoMap;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.EnumeratedTypeUtil;
import com.ibm.rpm.framework.util.FieldInfo;
import com.ibm.rpm.framework.util.RPMObjectScopeUtil;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Passport;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.MitigationFactor;
import com.ibm.rpm.scopemanagement.containers.ReqProConnection;
import com.ibm.rpm.scopemanagement.containers.ReqProImportStatus;
import com.ibm.rpm.scopemanagement.containers.RiskMatrix;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.security.managers.SecurityMask;
import com.ibm.rpm.timesheet.containers.AbstractTimesheet;
import com.ibm.rpm.timesheet.containers.DefaultStep;
import com.ibm.rpm.timesheet.containers.GenericStep;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.ProjectOrganizationalAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowRoleMapping;
import com.ibm.rpm.workflow.util.WorkflowUtil;
import com.ibm.rpm.xpathparser.Table;
import com.ibm.rpm.xpathparser.XPathToStack;
import com.ibm.rpm.xpathparser.XPathUtil;

public class SecurityControllerUtil
{

    private static final Object NULL_CACHED_VALUE = new Object();

    public static class RPMObjectKey
    {
        public final RPMObject rpmObject;

        private final String id;

        public RPMObjectKey(RPMObject rpmobject)
        {
            this.rpmObject = rpmobject;
            //rpmObject id at time of this RPMObjectKey instantiation, could be null if
            //the rpmObject is being created, however, if the very same rpmObject was used after
            //creation -(i.e., an id got allocated and setID was called on the very same rpmObject)
            //the equals/hashCode functions will not confuse them to be equal keys. they will be 
            //different -as is intended.
            id = rpmObject.getID();
        }

        public int hashCode()
        {
            if (id == null)
            {
                return rpmObject.hashCode();
            }
            return id.hashCode();
        }

        public boolean equals(Object obj)
        {
            if (obj instanceof RPMObjectKey)
            {
                RPMObject other = ((RPMObjectKey) obj).rpmObject;
                if (rpmObject.getClass().equals(other.getClass()))
                {
                    String otherID = other.getID();                    
                    return (id == null) ? otherID == null : id.equals(otherID);
                }
            }
            return false;
        }
    }

    public static final class ReferencedObjectInfo extends RPMObjectKey
    {
        public final RPMObject referencingRpmObject;

        public final FieldInfo fieldInfo;

        public boolean canViewObject;

        public ReferencedObjectInfo(RPMObject referencingRpmObject,
                RPMObject rpmobject, FieldInfo fieldInfo)
        {
            super(rpmobject);
            this.referencingRpmObject = referencingRpmObject;
            this.fieldInfo = fieldInfo;
        }

        // topLevelObject means an object that was part of the object list
        // of the MessageContext (i.,e., in the messageContext.getObjects()
        // list)
        // these elements are the direct result of a query, so we don't have
        // the field that holds them nor the referencingRpmObject, which means
        // it will be null
        public boolean isTopLevelObject()
        {
            return (fieldInfo == null);
        }
    }

    //XXX if we don't need the LoadParams anymore, so maybe we can get rid of this object altegether?
    public static class LoadingResult
    {
        public final RPMObject loadedRPMObject;

        public final LoadParams loadParams;

        LoadingResult(RPMObject loadedRPMObject, LoadParams loadParams)
        {
            this.loadedRPMObject = loadedRPMObject;
            this.loadParams = loadParams;
        }
    }

    private static abstract class RPMObjectLoader
    {
        protected final RPMObjectScope rpmScope;

        protected final MessageContext messageContext;

        private final boolean forceBuildSecurity;

        protected RPMObjectLoader(RPMObjectScope rpmScope,
                MessageContext messageContext, boolean forceBuildSecurity)
        {
            this.messageContext = messageContext;
            this.rpmScope = rpmScope;
            this.forceBuildSecurity = forceBuildSecurity;
        }

        final LoadingResult load()
        {
            final LoadParams params = messageContext.getLoadParams();
            boolean originalForceBuildSecurity = false;
            try
            {
                if (params == null)
                {
                    LoadParams temParams = new LoadParams();
                    temParams.setInternalSecurityCallCounter(1);// clearly this
                    // is the first
                    // time
                    temParams.setForceBuildSecurity(forceBuildSecurity);
                    messageContext.setLoadParams(temParams);
                }
                else
                {
                    params.setInternalSecurityCallCounter(params
                        .getInternalSecurityCallCounter() + 1);// increment
                    if (forceBuildSecurity)
                    {
                        originalForceBuildSecurity = params
                            .isForceBuildSecurity();
                        params.setForceBuildSecurity(true);
                    }
                }
                return new LoadingResult(doLoad(), messageContext
                    .getLoadParams());
            }
            catch (Exception ex)
            {
                RpmLog.getLog().warn(
                    "problem loading RPMObject with RPMObjectLoader ", ex);
                return new LoadingResult(null, messageContext.getLoadParams());
            }
            finally
            {
                if (params == null)
                {
                    messageContext.setLoadParams(null);
                }
                else
                {
                    params.setInternalSecurityCallCounter(params
                        .getInternalSecurityCallCounter() - 1);// decrement
                    if (forceBuildSecurity)
                    {
                        params
                            .setForceBuildSecurity(originalForceBuildSecurity);
                    }
                }
            }
        }

        protected abstract RPMObject doLoad() throws Exception;
    }

    private static WbsManager wbsManager;

    private static WbsManager getWbsManager() throws RPMException
    {
        if (wbsManager == null)
        {
            wbsManager = (WbsManager) RPMManagerFactory.getInstance()
                .getRPMObjectManager(WorkElement.class);
        }
        return wbsManager;

    }

    private static class RPMObjectLoaderByPrimaryKey extends RPMObjectLoader
    {
        private final RPMObject rpmObject;

        public RPMObjectLoaderByPrimaryKey(RPMObjectScope rpmScope,
                MessageContext messageContext, RPMObject rpmObject,
                boolean forceBuildSecurity)
        {
            super(rpmScope, messageContext, forceBuildSecurity);
            this.rpmObject = rpmObject;
        }

        protected RPMObject doLoad() throws Exception
        {
            RemovedContainerFromCache removedContainerFromCache = null;
            try
            {
                removedContainerFromCache = messageContext.getCache()
                    .removeContainerFromCache(rpmObject);
                RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(rpmObject.getClass());
                if (manager != null)
                {
                    return manager.loadByPrimaryKey(rpmObject, rpmScope,
                        messageContext, false);
                }
            }
            finally
            {
                if (removedContainerFromCache != null)
                {
                    messageContext.getCache().restoreContainerToCache(
                        removedContainerFromCache);
                }
            }
            return null;
        }
    }

    private static class RPMObjectLoaderByXPath extends RPMObjectLoader
    {
        private final String xpath;

        public RPMObjectLoaderByXPath(RPMObjectScope rpmScope,
                MessageContext messageContext, String xpath)
        {
            super(rpmScope, messageContext, false);
            this.xpath = xpath;
        }

        protected RPMObject doLoad() throws Exception
        {
            Stack stack = XPathToStack.parse(xpath);
            stack = XPathUtil.getinstance().validateAndPrepareTableStack(stack,
                stack.size() - 1);
            Table peek = (Table) stack.peek();
            RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(peek.getObjectName());
            List list = manager.load(stack, rpmScope, messageContext);
            if (list != null && list.size() > 0)
            {
                return (RPMObject) list.get(0);
            }
            return null;
        }
    }

    private static final boolean RECURSE = true;

    private static final boolean CONTAINMENT = true;

    public static Set getDirectlyReferencedObjectsInfo(
            RPMObject referencingRpmObject, Set objects) throws RPMException
    {
        getObjectsInfo(referencingRpmObject, objects, !RECURSE, !CONTAINMENT);
        return objects;
    }

    public static Set getDirectChildrenObjectsInfo(RPMObject parentRpmObject,
            Set objects) throws RPMException
    {
        getObjectsInfo(parentRpmObject, objects, !RECURSE, CONTAINMENT);
        return objects;
    }

    public static Set getAllChildrenObjectsInfo(RPMObject parentRpmObject,
            Set objects) throws RPMException
    {
        getObjectsInfo(parentRpmObject, objects, RECURSE, CONTAINMENT);
        return objects;
    }

    public static Set getAllObjectsInfo(ReferencedObjectInfo refInfo,
            Set objects) throws RPMException
    {
        objects.add(refInfo);
        getAllObjectsInfoRecursively(SecurityControllerUtil
            .getDirectChildrenObjectsInfo(refInfo.rpmObject, new HashSet()),
            objects);
        getAllObjectsInfoRecursively(
            SecurityControllerUtil.getDirectlyReferencedObjectsInfo(
                refInfo.rpmObject, new HashSet()), objects);
        return objects;
    }

    public static Set getFilteringResultFlatRPMObjectInfos(
            MessageContext messageContext,
            ISecurityController securityController)
    {
        Set visitedRPMObjectInfos = null;
        Map genericCacheMap = messageContext.getRPMSecurityCache()
            .getGenericCacheMap();
        Map map = (Map) genericCacheMap.get(SecurityControllerUtil.class);
        if (map != null)
        {
            visitedRPMObjectInfos = (Set) map.get(securityController);
        }

        if (visitedRPMObjectInfos != null)
        {
            //remove non-visible entries that we visited
            Iterator it = visitedRPMObjectInfos.iterator();
            while (it.hasNext())
            {
                if (!((ReferencedObjectInfo) it.next()).canViewObject)
                {
                    it.remove();
                }
            }
        }

        return visitedRPMObjectInfos;
    }

    public static Set getFlatRPMObjectInfos(MessageContext messageContext)
            throws RPMException
    {
        Set flatRPMObjectInfos = new HashSet();
        Iterator it = messageContext.getObjects().iterator();
        while (it.hasNext())
        {
            RPMObject rpmObject = (RPMObject) it.next();
            ReferencedObjectInfo refInfo = new ReferencedObjectInfo(null,
                rpmObject, null);
            if (!flatRPMObjectInfos.contains(refInfo))
            {
                SecurityControllerUtil.getAllObjectsInfo(refInfo,
                    flatRPMObjectInfos);
            }
        }
        return flatRPMObjectInfos;
    }

    private static void getAllObjectsInfoRecursively(Set rpmObjectInfos,
            Set objects) throws RPMException
    {
        Iterator it = rpmObjectInfos.iterator();
        while (it.hasNext())
        {
            ReferencedObjectInfo refInfo = (ReferencedObjectInfo) it.next();
            if (objects.add(refInfo))
            {
                getAllObjectsInfoRecursively(SecurityControllerUtil
                    .getDirectChildrenObjectsInfo(refInfo.rpmObject,
                        new HashSet()), objects);
                getAllObjectsInfoRecursively(SecurityControllerUtil
                    .getDirectlyReferencedObjectsInfo(refInfo.rpmObject,
                        new HashSet()), objects);
            }
        }
    }

    private static void getObjectsInfo(RPMObject referencingRpmObject,
            Set objects, boolean recurse, boolean containment)
            throws RPMException
    {
        try
        {
            FieldInfo[] fieldInfos = ContainerFieldInfoMap.getInstance()
                .getFieldInfos(referencingRpmObject.getClass());
            if (fieldInfos != null)
            {
                for (int i = 0; i < fieldInfos.length; i++)
                {
                    FieldInfo fieldInfo = fieldInfos[i];
                    if (fieldInfo.isComposite() == containment)
                    {
                        Method fieldGetter = fieldInfo.getFieldGetter();
                        if (fieldGetter != null)
                        {
                            Object fieldValue = fieldGetter.invoke(
                                referencingRpmObject, null);
                            if (fieldValue != null)
                            {
                                if (fieldValue.getClass().isArray())
                                {
                                    int length = Array.getLength(fieldValue);
                                    for (int j = 0; j < length; j++)
                                    {
                                        RPMObject obj = (RPMObject) Array.get(
                                            fieldValue, j);
                                        if (obj != null)
                                        {
                                            if (objects
                                                .add(new ReferencedObjectInfo(
                                                    referencingRpmObject, obj,
                                                    fieldInfo))
                                                    && recurse)
                                            {
                                                getObjectsInfo(obj, objects,
                                                    recurse, containment);
                                            }
                                        }
                                    }
                                }
                                else
                                {
                                    RPMObject obj = (RPMObject) fieldValue;
                                    if (objects.add(new ReferencedObjectInfo(
                                        referencingRpmObject, obj, fieldInfo))
                                            && recurse)
                                    {
                                        getObjectsInfo(obj, objects, recurse,
                                            containment);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        catch (Exception e)
        {
            throw new RPMException(e).log();
        }
    }

    public static void replaceFieldMappingEntryFlags(
            Integer[] canReadSecurityFlags, Integer[] canWriteSecurityFlags,
            Map mappingEntryMap)
    {
        Iterator it = mappingEntryMap.values().iterator();
        while (it.hasNext())
        {
            FieldMappingEntry entry = (FieldMappingEntry) it.next();
            entry.canReadSecurityFlags = canReadSecurityFlags;
            entry.canWriteSecurityFlags = canWriteSecurityFlags;
        }
    }

    /**
     * Returns a new scope according to the type of rpmObject.
     */
    public static final RPMObjectScope getRPMObjectScope(RPMObject rpmObject,
            RPMObjectScope parentScope)
    {
        RPMObjectScope scope = null;
        Class scopeClass = RPMObjectScopeUtil.getInstance().getScope(
            rpmObject.getClass());

        if (scopeClass == null)
        {
            scope = new RPMObjectScope();
        }
        else
        {
            try
            {
                scope = (RPMObjectScope) scopeClass.newInstance();

            }
            catch (Exception ex)
            {
                RpmLog.getLog().warn("problem creating RPMObjectScope", ex);
            }
        }

        if ((scope != null) && (parentScope != null))
        {
            scope.setParent(parentScope);
        }
        return scope;
    }

    /**
     * Loads the rpmObject by it's ID from the database.
     */
    public static final RPMObject loadObject(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope parentScope)
    {
        if (rpmObject.getID() != null)
        {
            RPMObjectScope scope = getRPMObjectScope(rpmObject, parentScope);
            if (scope != null)
            {
                try
                {
                    return loadByPrimaryKey(rpmObject, scope, messageContext,
                        false).loadedRPMObject;
                }
                catch (Exception ex)
                {
                    RpmLog.getLog().warn(
                        "problem loading RPMObject with ID "
                                + rpmObject.getID(), ex);
                }
            }
        }
        return null;
    }

    // XXX TODO this code should be part of the manager, we copied it here
    // because we need
    // to increment the InternalSecurityCallCounter
    public static LoadingResult loadByXpath(MessageContext messageContext,
            String xpath, RPMObjectScope scope)
    {
        return new RPMObjectLoaderByXPath(scope, messageContext, xpath).load();
    }

    //TO be called only by trusted code (security code, AND only when creating 
    //an object.
    public static void cacheParentOfCreatedRPMObject(
            MessageContext messageContext, RPMObject rpmObject, RPMObject parent)
    {
        Map cacheMap = messageContext.getRPMSecurityCache()
            .getRPMObjectCacheMap(rpmObject);
        cacheMap.put("loadParent", parent);
    }

    public static final RPMObject loadParent(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope parentScope)
    {
        /*
         * ResourceSecurityRights is not a container in the Database, it is
         * impossible to reload it, we must trust the value that is given to us...
         */
        if (rpmObject.getID() == null
                || rpmObject instanceof ResourceSecurityRights
                || rpmObject instanceof Passport)
        {
            return rpmObject.getParent();
        }

        try
        {
            //Check the cache
            Map cacheMap = messageContext.getRPMSecurityCache()
                .getRPMObjectCacheMap(rpmObject);
            Object parentObject = cacheMap.get("loadParent");
            if (parentObject != null)
            {
                return (parentObject == NULL_CACHED_VALUE) ? null
                        : (RPMObject) parentObject;
            }

            //try MessageContext cache
            RPMObject rpmObjectInCache = messageContext.getCache()
                .getContainerInCache(rpmObject);
            if (rpmObjectInCache != null)
            {
                RPMObject parent = rpmObjectInCache.getParent();
                if (parent != null)
                {
                    cacheMap.put("loadParent", parent);
                    return parent;
                }
            }

            RPMObject loadedObject = loadObject(messageContext, rpmObject,
                parentScope);
            if (loadedObject != null)
            {
                if (loadedObject.getParent() != null)
                {
                    cacheMap.put("loadParent", loadedObject.getParent());
                    return loadedObject.getParent();
                }
            }

            cacheMap.put("loadParent", NULL_CACHED_VALUE);
            return null;
        }
        catch (Exception ex)
        {
            RpmLog.getLog().warn(
                "problem loading parent of RPMObject with ID "
                        + rpmObject.getID(), ex);
        }
        return null;
    }

    /**
     * Loads the parent of an rpmObject.
     */
    public static final RPMObject loadParent(MessageContext messageContext,
            RPMObject rpmObject)
    {
        RPMObjectScope parentScope = null;
        //We're doing this because the FinancialManager will throw an exception 
        //if parent's scope is not one of those two???
        if (rpmObject instanceof AbstractFinancial)
        {
            if (ContextUtil.SCOPE_CONTEXT.equals(rpmObject.getContextName())
                    || (rpmObject.getParent() != null && rpmObject.getParent() instanceof ScopeElement))
            {
                parentScope = new ScopeElementScope();
            }
            else if (ContextUtil.WBS_CONTEXT.equals(rpmObject.getContextName())
                    || (rpmObject.getParent() != null && rpmObject.getParent() instanceof WorkElement))
            {
                parentScope = new WorkElementScope();
            }
        }
        if (rpmObject instanceof TaskAssignment)
        {
            parentScope = new ResourceTaskAssignmentScope();
        }
        if (parentScope == null)
        {
            parentScope = new RPMObjectScope();
        }
        return loadParent(messageContext, rpmObject, parentScope);
    }

    public static final String loadResourceIdWithDocument(
            MessageContext messageContext, DocumentElement rpmObject)
    {
        if (rpmObject.getID() == null)
        {
            RPMObject parent = rpmObject.getParent();
            if (parent == null)
            {
                return null;
            }
            if (parent instanceof Resource)
            {
                return parent.getID();
            }
            if (parent instanceof DocumentElement
                    && !(parent.equals(rpmObject)))
            {
                return loadResourceIdWithDocument(messageContext,
                    (DocumentElement) parent);
            }
            return null;
        }
        else
        {
            //          Check the cache
            Map cacheMap = messageContext.getRPMSecurityCache()
                .getRPMObjectCacheMap(rpmObject);
            Object idObject = cacheMap.get("loadResourceIdWithDocument");
            if (idObject != null)
            {
                return (idObject == NULL_CACHED_VALUE) ? null
                        : (String) idObject;
            }

            SqlBuffer buffer = new SqlBuffer();
            buffer.appendEqualQuestionMark("TMT_RESOURCE_DOCUMENTS.ELEMENT_ID");

            Object[] arguments = new Object[]
            {
                rpmObject.getID()
            };

            try
            {
                String id = (String) ManagerUtil
                    .selectColumnValue(String.class, rpmObject, null,
                        "TMT_RESOURCE_DOCUMENTS.PROJECT_ID",
                        "TMT_RESOURCE_DOCUMENTS", buffer, arguments,
                        messageContext);
                if (id != null)
                {
                    cacheMap.put("loadResourceIdWithDocument", id);
                }
                else
                {
                    cacheMap.put("loadResourceIdWithDocument",
                        NULL_CACHED_VALUE);
                }
                return id;
            }
            catch (RPMException e)
            {
                RpmLog.getLog().warn(null, e);
            }
            return null;
        }
    }

    //    /**
    //     * Loads the owner of an rpmObject.
    //     */
    //    public static final String loadResourceIdWithCalendar(
    //            MessageContext messageContext, RPMCalendar rpmObject)
    //    {
    //        if (rpmObject.getID() == null)
    //        {
    //            return (rpmObject.getParent() instanceof Resource) ? rpmObject
    //                .getParent().getID() : null;
    //        }
    //        else
    //        {
    //            //Check the cache
    //            Map cacheMap = messageContext.getRPMSecurityCache()
    //                .getRPMObjectCacheMap(rpmObject);
    //            Object idObject = cacheMap.get("loadResourceIdWithCalendar");
    //            if (idObject != null)
    //            {
    //                return (idObject == NULL_CACHED_VALUE) ? null
    //                        : (String) idObject;
    //            }
    //
    //            SqlBuffer buffer = new SqlBuffer();
    //            // Value to check in the WHERE clause 
    //            buffer.appendEqual("RESOURCES.CALENDAR_ID", rpmObject.getID());
    //            try
    //            {
    //                // Value to select and return
    //                String id = (String) ManagerUtil.selectColumnValue(
    //                    String.class, rpmObject, null, "RESOURCES.RESOURCE_ID",
    //                    "RESOURCES", buffer, messageContext);
    //                if (id != null)
    //                {
    //                    cacheMap.put("loadResourceIdWithCalendar", id);
    //                }
    //                else
    //                {
    //                    cacheMap.put("loadResourceIdWithCalendar",
    //                        NULL_CACHED_VALUE);
    //                }
    //                return id;
    //            }
    //            catch (RPMException e)
    //            {
    //                RpmLog.getLog().warn(null, e);
    //            }
    //            return null;
    //        }
    //    }

    /**
     * Loads the owner of an rpmObject.
     */
    public static final String loadResourceIdWithRates(
            MessageContext messageContext, ResourceRates rpmObject)
    {
        if (rpmObject.getID() == null)
        {
            if (rpmObject.getParent() instanceof Resource)
            {
                return rpmObject.getParent().getID();
            }
            else if (rpmObject.getParent() != null)
            {
                if (rpmObject.getParent().getParent() instanceof Resource)
                {
                    return rpmObject.getParent().getParent().getID();
                }
                else
                {
                    return null;//(Resource) loadParent(rpmObject.getParent());
                }
            }
        }
        else
        {
            //Check the cache
            Map cacheMap = messageContext.getRPMSecurityCache()
                .getRPMObjectCacheMap(rpmObject);
            Object idObject = cacheMap.get("loadResourceIdWithRates");
            if (idObject != null)
            {
                return (idObject == NULL_CACHED_VALUE) ? null
                        : (String) idObject;
            }

            SqlBuffer buffer = new SqlBuffer();
            // Value to check in the WHERE clause 
            buffer.appendEqualQuestionMark("RESOURCES.RATE_ID");

            Object[] arguments = new Object[]
            {
                rpmObject.getID()
            };

            try
            {
                // Value to select and return
                String id = (String) ManagerUtil.selectColumnValue(
                    String.class, rpmObject, null, "RESOURCES.RESOURCE_ID",
                    "RESOURCES", buffer, arguments, messageContext);
                if (id != null)
                {
                    cacheMap.put("loadResourceIdWithRates", id);
                }
                else
                {
                    cacheMap.put("loadResourceIdWithRates", NULL_CACHED_VALUE);
                }
                return id;
            }
            catch (RPMException e)
            {
                RpmLog.getLog().warn(null, e);
            }
        }
        return null;
    }

    //    /**
    //     * Loads the owner of an rpmObject.
    //     */
    //    public static final boolean isCalendarForWorkElement(
    //            MessageContext messageContext, RPMCalendar rpmObject)
    //    {
    //        if (rpmObject.getID() == null)
    //        {
    //            return (rpmObject.getParent() instanceof WorkElement);
    //        }
    //        else if (rpmObject.getID().startsWith("DEFAULT_CALENDAR"))
    //        {
    //            return false;
    //        }
    //        else
    //        {
    //            //Check the cache
    //            Map cacheMap = messageContext.getRPMSecurityCache()
    //                .getRPMObjectCacheMap(rpmObject);
    //            Boolean value = (Boolean) cacheMap.get("isCalendarForWorkElement");
    //            if (value != null)
    //            {
    //                return value.booleanValue();
    //            }
    //
    //            SqlBuffer buffer = new SqlBuffer();
    //            // Value to check in the WHERE clause 
    //            buffer.appendEqual("TMT_WBS.CALENDAR_ID", rpmObject.getID());
    //            try
    //            {
    //                // Value to select and return
    //                String id = (String) ManagerUtil.selectColumnValue(
    //                    String.class, rpmObject, null, "TMT_WBS.ELEMENT_ID",
    //                    "TMT_WBS", buffer, messageContext);
    //                if (id != null)
    //                {
    //                    cacheMap.put("isCalendarForWorkElement", Boolean.TRUE);
    //                    return true;
    //                }
    //                else
    //                {
    //                    cacheMap.put("isCalendarForWorkElement", Boolean.FALSE);
    //                    return false;
    //                }
    //            }
    //            catch (RPMException e)
    //            {
    //                RpmLog.getLog().warn(null, e);
    //            }
    //            return false;
    //        }
    //    }

    /**
     * Loads the owner of an rpmObject.
     */
    public static final String loadResourceIdWithResourceAttributeAssignmentWithRates(
            MessageContext messageContext, ResourceRates rpmObject)
    {

        if (rpmObject.getID() == null)
        {
            if (rpmObject.getParent() instanceof ResourceAttributeAssignment)
            {
                if (rpmObject.getParent().getParent() instanceof Resource)
                {
                    return rpmObject.getParent().getParent().getID();
                }
            }
        }
        else
        {
            //Check the cache
            Map cacheMap = messageContext.getRPMSecurityCache()
                .getRPMObjectCacheMap(rpmObject);
            Object idObject = cacheMap
                .get("loadResourceIdWithResourceAttributeAssignmentWithRates");
            if (idObject != null)
            {
                return (idObject == NULL_CACHED_VALUE) ? null
                        : (String) idObject;
            }

            SqlBuffer buffer = new SqlBuffer();
            // Values to in the WHERE clause 
            buffer.appendEqualQuestionMark("TMT_COMPETENCIES.RATE_ID");

            Object[] arguments = new Object[]
            {
                rpmObject.getID()
            };

            try
            {
                // Value to select
                String id = (String) ManagerUtil.selectColumnValue(
                    String.class, rpmObject, null,
                    "TMT_COMPETENCIES.RESOURCE_ID", "TMT_COMPETENCIES", buffer,
                    arguments, messageContext);
                if (id != null)
                {
                    cacheMap
                        .put(
                            "loadResourceIdWithResourceAttributeAssignmentWithRates",
                            id);
                }
                else
                {
                    cacheMap
                        .put(
                            "loadResourceIdWithResourceAttributeAssignmentWithRates",
                            NULL_CACHED_VALUE);
                }
                return id;
            }
            catch (RPMException e)
            {
                RpmLog.getLog().warn(null, e);
            }
        }
        return null;
    }

    public static PublishedType loadAbstractDocumentPublishedType(
            AbstractDocument rpmObject, MessageContext messageContext)
    {
        if (rpmObject.getID() == null)
        {
            return rpmObject.getPublished();
        }

        //Check the cache
        Map cacheMap = messageContext.getRPMSecurityCache()
            .getRPMObjectCacheMap(rpmObject);
        Object typeObject = cacheMap.get("loadAbstractDocumentPublishedType");
        if (typeObject != null)
        {
            return (typeObject == NULL_CACHED_VALUE) ? null
                    : (PublishedType) typeObject;
        }

        try
        {
            AbstractDocument rpmObjectInCache = (AbstractDocument) messageContext
                .getCache().getContainerInCache(rpmObject);
            if (rpmObjectInCache != null)
            {
                if (rpmObjectInCache.getPublished() != null)
                {
                    cacheMap.put("loadAbstractDocumentPublishedType",
                        rpmObjectInCache.getPublished());
                    return rpmObjectInCache.getPublished();
                }
            }
        }
        catch (Exception ex)
        {
            //ignore
        }

        AbstractDocument loadedObject = (AbstractDocument) loadObject(
            messageContext, rpmObject, null);
        if (loadedObject != null)
        {
            if (loadedObject.getPublished() != null)
            {
                cacheMap.put("loadAbstractDocumentPublishedType", loadedObject
                    .getPublished());
                return loadedObject.getPublished();
            }
        }
        return null;
    }

    public static ProjectPublishedType loadWorkElementPublishedType(
            WorkElement rpmObject, MessageContext messageContext)
    {
        if (rpmObject.getID() == null)
        {
            return rpmObject.getPublished();
        }

        //Check the cache
        Map cacheMap = messageContext.getRPMSecurityCache()
            .getRPMObjectCacheMap(rpmObject);
        Object typeObject = cacheMap.get("loadWorkElementPublishedType");
        if (typeObject != null)
        {
            return (typeObject == NULL_CACHED_VALUE) ? null
                    : (ProjectPublishedType) typeObject;
        }

        try
        {
            WorkElement rpmObjectInCache = (WorkElement) messageContext
                .getCache().getContainerInCache(rpmObject);
            if (rpmObjectInCache != null)
            {
                if (rpmObjectInCache.getPublished() != null)
                {
                    cacheMap.put("loadWorkElementPublishedType",
                        rpmObjectInCache.getPublished());
                    return rpmObjectInCache.getPublished();
                }
            }
        }
        catch (Exception ex)
        {
            //ignore
        }

        SqlBuffer buffer = new SqlBuffer();
        // Values to in the WHERE clause 
        buffer.appendEqualQuestionMark("TMT_WBS.ELEMENT_ID");

        Object[] arguments = new Object[]
        {
            rpmObject.getID()
        };

        try
        {
            ProjectPublishedType type = null;
            // Value to select
            String typeStr = (String) ManagerUtil.selectColumnValue(
                String.class, rpmObject, null, "TMT_WBS.PUBLISHED", "TMT_WBS",
                buffer, arguments, messageContext);

            if (typeStr != null)
            {
                type = getWbsManager().decodeProjectPublishedType(
                    Integer.valueOf(typeStr).intValue());
            }
            if (type != null)
            {
                cacheMap.put("loadWorkElementPublishedType", type);
            }
            else
            {
                cacheMap.put("loadWorkElementPublishedType", NULL_CACHED_VALUE);
            }
            return type;
        }
        catch (RPMException e)
        {
            RpmLog.getLog().warn(null, e);
        }

        return null;
    }

    public static PublishedType loadAbstractScopePublishedType(
            AbstractScope rpmObject, MessageContext messageContext)
    {
        if (rpmObject.getID() == null)
        {
            return rpmObject.getPublished();
        }

        //Check the cache
        Map cacheMap = messageContext.getRPMSecurityCache()
            .getRPMObjectCacheMap(rpmObject);
        Object typeObject = cacheMap.get("loadAbstractScopePublishedType");
        if (typeObject != null)
        {
            return (typeObject == NULL_CACHED_VALUE) ? null
                    : (PublishedType) typeObject;
        }

        try
        {
            AbstractScope rpmObjectInCache = (AbstractScope) messageContext
                .getCache().getContainerInCache(rpmObject);
            if (rpmObjectInCache != null)
            {
                if (rpmObjectInCache.getPublished() != null)
                {
                    cacheMap.put("loadAbstractScopePublishedType",
                        rpmObjectInCache.getPublished());
                    return rpmObjectInCache.getPublished();
                }
            }
        }
        catch (Exception ex)
        {
            //ignore
        }

        SqlBuffer buffer = new SqlBuffer();
        // Values to in the WHERE clause 
        buffer.appendEqualQuestionMark("TMT_CRI.ELEMENT_ID");

        Object[] arguments = new Object[]
        {
            rpmObject.getID()
        };

        try
        {
            PublishedType type = null;
            // Value to select
            String typeString = (String) ManagerUtil.selectColumnValue(
                String.class, rpmObject, null, "TMT_CRI.TEMPLATED", "TMT_CRI",
                buffer, arguments, messageContext);

            if (typeString != null)
            {
                type = (PublishedType) EnumeratedTypeUtil.getInstance()
                    .getType(ScopeElement.class, PublishedType.class,
                        typeString);
            }
            if (type != null)
            {
                cacheMap.put("loadAbstractScopePublishedType", type);
            }
            else
            {
                cacheMap.put("loadAbstractScopePublishedType",
                    NULL_CACHED_VALUE);
            }
            return type;
        }
        catch (RPMException e)
        {
            RpmLog.getLog().warn(null, e);
        }

        return null;
    }

    public static boolean isInternalSecurityCall(MessageContext messageContext)
    {
        if (messageContext.getLoadParams() != null
                && messageContext.getLoadParams()
                    .getInternalSecurityCallCounter() > 0)
        {
            return true;
        }
        return false;
    }

    /**
     * Returns true if the security flag's bit is 1, else returns false.
     */
    public static boolean calculateFlagValue(Integer securityFlag,
            CombinedSecurityFlags combinedSecurityFlags)
    {
        int[] securityFlags = combinedSecurityFlags.getCombinedFlags();
        int mask = SecurityMask.MASK_ARRAY[securityFlag.intValue() % 32];
        int level = securityFlag.intValue() / 32;
        return ((securityFlags[level] & mask) == mask);
    }

    /**
     * Returns false if all the securityFlags are false, if at least one is true
     * then returns true.
     */
    public static SecurityValidationResult calculateFlagResultUsingOr(
            Integer[] securityFlags,
            CombinedSecurityFlags combinedSecurityFlags, String message,
            Class container)
    {
        if (securityFlags == null || securityFlags.length == 0)
        {
            return SecurityValidationResult.UNDEFINED_RESULT;
        }

        for (int i = 0; i < securityFlags.length; i++)
        {
            if (calculateFlagValue(securityFlags[i], combinedSecurityFlags))
            {
                return SecurityValidationResult.TRUE_RESULT;
            }
        }
        return SecurityValidationResult.makeFalseResult(MessageFormat.format(
            message, new Object[]
            {
                StringUtil.getShortClassName(container)
            }));
    }

    /**
     * Returns false if any of the securityFlags is false, else return true since all are
     */
    public static SecurityValidationResult calculateFlagResultUsingAnd(
            Integer[] securityFlags,
            CombinedSecurityFlags combinedSecurityFlags, String message,
            Class container)
    {
        if (securityFlags == null || securityFlags.length == 0)
        {
            return SecurityValidationResult.UNDEFINED_RESULT;
        }

        for (int i = 0; i < securityFlags.length; i++)
        {
            if (!calculateFlagValue(securityFlags[i], combinedSecurityFlags))
            {
                return SecurityValidationResult.makeFalseResult(MessageFormat
                    .format(message, new Object[]
                    {
                        StringUtil.getShortClassName(container)
                    }));
            }
        }
        return SecurityValidationResult.TRUE_RESULT;
    }

    public static SecurityValidationResult calculateVirtualFlagResult(
            Integer securityFlag, CombinedSecurityFlags combinedSecurityFlags,
            String message, Class container)
    {
        if (securityFlag == VIRTUAL.TRUE)
        {
            return SecurityValidationResult.TRUE_RESULT;
        }
        if (securityFlag == VIRTUAL.FALSE)
        {
            return SecurityValidationResult.makeFalseResult(MessageFormat
                .format(message, new Object[]
                {
                    StringUtil.getShortClassName(container)
                }));
        }
        return SecurityControllerUtil.calculateFlagResult(securityFlag,
            combinedSecurityFlags, message, container);
    }

    /**
     * Returns false if all the securityFlags are false, if at least one is true
     * then returns true.
     * @param container TODO
     */
    public static SecurityValidationResult calculateFlagResult(
            Integer securityFlag, CombinedSecurityFlags combinedSecurityFlags,
            String message, Class container)
    {
        if (securityFlag == null)
        {
            return SecurityValidationResult.UNDEFINED_RESULT;
        }
        if (calculateFlagValue(securityFlag, combinedSecurityFlags))
        {
            return SecurityValidationResult.TRUE_RESULT;
        }
        else
        {
            return SecurityValidationResult.makeFalseResult(MessageFormat
                .format(message, new Object[]
                {
                    StringUtil.getShortClassName(container)
                }));
        }
    }

    // #########################################################

    private static Set getSecurityRolesFromGenericStep(
            MessageContext messageContext, GenericStep genericStep)
            throws RPMException
    {
        if (genericStep.getID() == null)
        {
            return getSecurityRoles(messageContext, genericStep
                .getTaskAssignment());
        }

        try
        {
            GenericStep rpmObjectInCache = (GenericStep) messageContext
                .getCache().getContainerInCache(genericStep);
            if (rpmObjectInCache != null)
            {
                if (rpmObjectInCache.getTaskAssignment() != null)
                {
                    return getSecurityRoles(messageContext, rpmObjectInCache
                        .getTaskAssignment());
                }
            }
        }
        catch (Exception ex)
        {
            //ignore
        }

        // ignore the project value passed and load the project a fresh
        // to be sure that no one tried to pass in a different project
        StepScope scope = new StepScope();
        scope.setTaskAssignment(true);
        GenericStep loadedStep = (GenericStep) loadByPrimaryKey(genericStep,
            scope, messageContext, false).loadedRPMObject;
        if (loadedStep == null || loadedStep.getTaskAssignment() == null)
        {
            return Collections.EMPTY_SET;
        }
        return getSecurityRoles(messageContext, loadedStep.getTaskAssignment());
    }

    private static Set getSecurityRolesFromParent(
            MessageContext messageContext, RPMObject rpmObject)
            throws RPMException
    {
        if (rpmObject.getID() == null)
        {
            return getSecurityRoles(messageContext, rpmObject.getParent());
        }

        RPMObject parent = loadParent(messageContext, rpmObject);
        if (parent == null)
        {
            return Collections.EMPTY_SET;
        }

        return getSecurityRoles(messageContext, parent);
    }

    /**
     * @param messageContext
     * @param assignment
     * @return
     */
    private static Set getSecurityRolesFromProjectOrganizationalAssignment(
            MessageContext messageContext,
            ProjectOrganizationalAssignment rpmObject) throws RPMException
    {
        //Check the cache
        Map cacheMap = messageContext.getRPMSecurityCache()
            .getRPMObjectCacheMap(rpmObject);
        String projectIdStr = null;
        Object projectIdObject = cacheMap
            .get("getSecurityRolesFromProjectOrganizationalAssignment");
        if (projectIdObject == null)
        {
            SqlBuffer buffer = new SqlBuffer();
            // Value to check in the WHERE clause 
            buffer.appendEqualQuestionMark("PRJ_ASSIGNED_ORG.ELEMENT_ID");

            Object[] arguments = new Object[]
            {
                rpmObject.getID()
            };

            try
            {
                // Value to select and return
                projectIdStr = (String) ManagerUtil.selectColumnValue(
                    String.class, null, null, "PRJ_ASSIGNED_ORG.PROJECT_ID",
                    "PRJ_ASSIGNED_ORG", buffer, arguments, messageContext);
                if (projectIdStr != null)
                {
                    cacheMap.put(
                        "getSecurityRolesFromProjectOrganizationalAssignment",
                        projectIdStr);
                }
                else
                {
                    cacheMap.put(
                        "getSecurityRolesFromProjectOrganizationalAssignment",
                        NULL_CACHED_VALUE);
                }
            }
            catch (RPMException e)
            {
                RpmLog.getLog().warn(null, e);
            }
        }
        else if (projectIdObject != NULL_CACHED_VALUE)
        {
            projectIdStr = (String) projectIdObject;
        }

        if (projectIdStr != null)
        {
            GenericProject proj = new GenericProject();
            proj.setID((String) projectIdObject);
            return getSecurityRoles(messageContext, proj);
        }

        return Collections.EMPTY_SET;
    }

    /**
     * @param messageContext
     * @param summaryTimesheet
     * @param rpmObjectScope
     * @param object
     * @return
     */
    private static Set getSecurityRolesFromSummaryTimesheet(
            MessageContext messageContext, SummaryTimesheet summaryTimesheet)
            throws RPMException
    {

        if (summaryTimesheet.getID() == null)
        {
            // we'll ignore Personal/AdministrativeTaskAssignment
            if (summaryTimesheet.getTaskAssignment() instanceof TaskAssignment)
            {
                return getSecurityRoles(messageContext, summaryTimesheet
                    .getTaskAssignment());
            }
            return null;
        }

        try
        {
            SummaryTimesheet rpmObjectInCache = (SummaryTimesheet) messageContext
                .getCache().getContainerInCache(summaryTimesheet);
            if (rpmObjectInCache != null)
            {
                if (rpmObjectInCache.getTaskAssignment() != null)
                {
                    if (rpmObjectInCache.getTaskAssignment() instanceof TaskAssignment)
                    {
                        return getSecurityRoles(messageContext,
                            rpmObjectInCache.getTaskAssignment());
                    }
                    return null;
                }
            }
        }
        catch (Exception ex)
        {
            //ignore
        }

        // ignore the project value passed and load the project a fresh
        // to be sure that no one tried to pass in a different taskAssignment
        SummaryTimesheetScope scope = new SummaryTimesheetScope();
        SummaryTimesheet loadedSummaryTimesheet = (SummaryTimesheet) loadByPrimaryKey(
            summaryTimesheet, scope, messageContext, false).loadedRPMObject;
        if (loadedSummaryTimesheet == null
                || (loadedSummaryTimesheet.getTaskAssignment() == null))
        {
            return Collections.EMPTY_SET;
        }

        if (loadedSummaryTimesheet.getTaskAssignment() instanceof TaskAssignment)
        {
            return getSecurityRoles(messageContext, loadedSummaryTimesheet
                .getTaskAssignment());
        }
        return null;
    }

    public static Set getSecurityRoles(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException
    {
        if (rpmObject == null)
        {
            return Collections.EMPTY_SET;
        }
        else if (rpmObject.getID() != null)
        {
            //Check the cache
            Map cacheMap = messageContext.getRPMSecurityCache()
                .getRPMObjectCacheMap(rpmObject);
            Object rolesObject = cacheMap.get("getSecurityRoles");
            if (rolesObject != null)
            {
                return (rolesObject == NULL_CACHED_VALUE) ? null
                        : (Set) rolesObject;
            }
        }

        if ((rpmObject instanceof WorkElement)
                || (rpmObject instanceof ScopeElement)
                || (rpmObject instanceof DocumentElement))
        {
            //don't cache WorkElement, ScopeElement, DocumentElement roles
            if (rpmObject.getID() == null)
            {
                if (rpmObject.getParent() == null
                        || (rpmObject == rpmObject.getParent()))//Proposals are set by WS code to be parent's of themselves, so we should avoid getting into infinte loop here)
                {
                    return null;//;Collections.EMPTY_SET;// Project creation is
                    // special case, there is a resource flag for it
                }
                return getSecurityRoles(messageContext, rpmObject.getParent());
            }
            Set roles = messageContext.getRPMSecurityCache()
                .getLoadedSecurityRolesSet(rpmObject);
            if (roles != null)
            {
                return roles;
            }

            //if this is a resource document, then no point ion reloading to get security
            //roles because it is not supported by the SP DB
            if (rpmObject instanceof DocumentElement
                    && rpmObject.getContextName() != null)
            {
                if (!(ContextUtil.getInstance().isScopeContext(
                    rpmObject.getContextName()) || ContextUtil.getInstance()
                    .isWbsContext(rpmObject.getContextName())))
                {
                    return null;
                }
            }

            LoadingResult result = loadByPrimaryKey(rpmObject, null,
                messageContext, true);

            if ((result.loadedRPMObject == null)
                    || !messageContext.isSuccessful())
            {
                //If we failed to load the element, then prevent loading it again by adding
                // roles == EMPTY_SET. We mihgt have failed becuase the element is not viewable
                //and got filtered by the SP, or, because the element is logically/deleted
                //In any case, it means we have avoid loading it again by security-util simply
                //by setting roles empty_roles in the cache so that on next load, we'll find the
                //cached set and return it to the caller
                messageContext.getRPMSecurityCache().associateWithEmptyRoleSet(
                    rpmObject);
                return Collections.EMPTY_SET;
            }
            //now it must have been added to the cache
            return messageContext.getRPMSecurityCache()
                .getLoadedSecurityRolesSet(rpmObject);
        }
        else if (rpmObject instanceof GenericStep)
        {
            Set roles = getSecurityRolesFromGenericStep(messageContext,
                (GenericStep) rpmObject);
            messageContext.getRPMSecurityCache()
                .getRPMObjectCacheMap(rpmObject).put("getSecurityRoles",
                    (roles == null) ? NULL_CACHED_VALUE : roles);
            return roles;
        }
        else if (rpmObject instanceof SummaryTimesheet)
        {
            Set roles = getSecurityRolesFromSummaryTimesheet(messageContext,
                (SummaryTimesheet) rpmObject);
            messageContext.getRPMSecurityCache()
                .getRPMObjectCacheMap(rpmObject).put("getSecurityRoles",
                    (roles == null) ? NULL_CACHED_VALUE : roles);
            return roles;
        }
        else if (rpmObject instanceof ProjectOrganizationalAssignment)
        {
            // The manager for this ProjectOrganizationalAssignment does not
            // support loading the parent
            // which is the Project, so we have to do it manually in this
            // convoluted way here
            Set roles = getSecurityRolesFromProjectOrganizationalAssignment(
                messageContext, (ProjectOrganizationalAssignment) rpmObject);
            messageContext.getRPMSecurityCache()
                .getRPMObjectCacheMap(rpmObject).put("getSecurityRoles",
                    (roles == null) ? NULL_CACHED_VALUE : roles);
            return roles;
        }
        else if (rpmObject instanceof Opportunity
                || rpmObject instanceof RiskMatrix
                || rpmObject instanceof MitigationFactor
                || rpmObject instanceof ReqProConnection
                || rpmObject instanceof ReqProImportStatus
                || rpmObject instanceof DefaultStep
                || rpmObject instanceof AbstractFinancial
                || rpmObject instanceof TimeCodeAssignment
                || rpmObject instanceof DocumentBlobDetails
                || rpmObject instanceof WorkflowRoleMapping
                || rpmObject instanceof ResourceRoleAssignment
                || rpmObject instanceof ElementDependency
                || rpmObject instanceof Duplicate
                || rpmObject instanceof AssignedScorecard
                || rpmObject instanceof CustomFieldAssignment
                || rpmObject instanceof AttributeAssignment
                || rpmObject instanceof RtfAssignment
                || rpmObject instanceof TaskAssignment
                || rpmObject instanceof AbstractTimesheet)
        {
            Set roles = getSecurityRolesFromParent(messageContext, rpmObject);
            messageContext.getRPMSecurityCache()
                .getRPMObjectCacheMap(rpmObject).put("getSecurityRoles",
                    (roles == null) ? NULL_CACHED_VALUE : roles);
            return roles;
        }

        // all other rpmobjects as not related to projects
        return null;
    }

    public static LoadingResult loadByPrimaryKey(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean forceBuildSecurity)
    {
        return new RPMObjectLoaderByPrimaryKey(rpmScope, messageContext,
            rpmObject, forceBuildSecurity).load();
    }

    public static boolean isResourceAssignedToScopeElement(
            ScopeElement rpmObject, String resourceId,
            MessageContext messageContext)
    {
        //select * from RESOURCE_ASSIGNMENTS where ELEMENT_ID = '004C4E8390F7436B9222D1F7458E21E9' and RESOURCE_ID = '4BCFC2BA69DC491E8E16C525DEE0C8CF'

        //Check the cache
        Map cacheMap = messageContext.getRPMSecurityCache()
            .getRPMObjectCacheMap(rpmObject);
        Boolean value = (Boolean) cacheMap
            .get("isResourceAssignedToScopeElement");
        if (value != null)
        {
            return value.booleanValue();
        }

        SqlBuffer buffer = new SqlBuffer();
        // Value to check in the WHERE clause 
        buffer.appendEqualQuestionMark("RESOURCE_ASSIGNMENTS.ELEMENT_ID");
        buffer.appendEqualQuestionMark("RESOURCE_ASSIGNMENTS.RESOURCE_ID");

        Object[] arguments = new Object[]
        {
                rpmObject.getID(), resourceId
        };

        try
        {
            // Value to select and return
            String id = (String) ManagerUtil.selectColumnValue(String.class,
                null, null, "RESOURCE_ASSIGNMENTS.RES_ASSIGNMENT_ID",
                "RESOURCE_ASSIGNMENTS", buffer, arguments, messageContext);
            if (id != null)
            {
                cacheMap.put("isResourceAssignedToScopeElement", Boolean.TRUE);
                return true;
            }
            else
            {
                cacheMap.put("isResourceAssignedToScopeElement", Boolean.FALSE);
                return false;
            }
        }
        catch (RPMException e)
        {
            RpmLog.getLog().warn(null, e);
        }

        return false;
    }

    public static final Object getValueFromMap(Class rpmObjectClassKey, Map map)
    {
        Object value = map.get(rpmObjectClassKey);
        while (value == null)
        {
            rpmObjectClassKey = rpmObjectClassKey.getSuperclass();
            if (rpmObjectClassKey == null)
            {
                return null;
            }
            value = map.get(rpmObjectClassKey);
        }
        return value;
    }

    // Logic: if either result is TRUE then the compounded result is true
    //else if any is FALSE then the compounded result is false
    //else the compounded result is true -since all are UNDEFINED in this case-.
    public static boolean or(ValueType first, ValueType second)
    {
        return (first == SecurityValidationResult.TRUE
                || second == SecurityValidationResult.TRUE || (first == SecurityValidationResult.UNDEFINED && second == SecurityValidationResult.UNDEFINED));
    }

    //Logic: if either result is FALSE then the compounded result is false
    //else it is true since it is a mix between TRUE/UNDEFINED .
    public static boolean and(ValueType first, ValueType second)
    {
        return !(first == SecurityValidationResult.FALSE || second == SecurityValidationResult.FALSE);
    }

    //if an exception occurs we'll return UNDEFINED
    public static ValueType isRunningWorkflowProcessManual(
            RunningWorkflowProcess runningWorkflowProcess,
            MessageContext messageContext)
    {
        //Check the cache
        Map cacheMap = messageContext.getRPMSecurityCache()
            .getRPMObjectCacheMap(runningWorkflowProcess);
        ValueType value = (ValueType) cacheMap
            .get("isRunningWorkflowProcessManual");
        if (value != null)
        {
            return value;
        }

        try
        {
            if (WorkflowUtil.isRunningWorkflowProcessManual(
                runningWorkflowProcess, messageContext))
            {
                cacheMap.put("isRunningWorkflowProcessManual",
                    SecurityValidationResult.TRUE);
                return SecurityValidationResult.TRUE;
            }
            else
            {
                cacheMap.put("isRunningWorkflowProcessManual",
                    SecurityValidationResult.FALSE);
                return SecurityValidationResult.FALSE;
            }
        }
        catch (Exception ex)
        {
            RpmLog.getLog().warn(
                "problem getting if RunningWorkflowProcess is Manual from DB",
                ex);
            cacheMap.put("isRunningWorkflowProcessManual",
                SecurityValidationResult.UNDEFINED);
            return SecurityValidationResult.UNDEFINED;
        }
    }

    public static RPMException extractSecurityException(
            MessageContext messageContext)
    {
        if (!messageContext.getErrors().isEmpty())
        {
            Iterator it = messageContext.getErrors().iterator();
            while (it.hasNext())
            {
                Object ex = it.next();
                if (ex instanceof RPMException)
                {
                    RPMException rpmEx = (RPMException) ex;
                    if (rpmEx.getErrorID() == ErrorCodes.INSUFFICIENT_SECURITY_RIGHTS)
                    {
                        return rpmEx;
                    }
                }
            }
        }
        return null;
    }

    public static int[] makeArray(int size, int value)
    {
        int[] a = new int[size];
        Arrays.fill(a, value);
        return a;
    }

    public static ArrayList applyFiltering(MessageContext messageContext,
            ArrayList loadedObjects, ISecurityController securityController)
            throws RPMException
    {
        //This is a result of internal call, then don't filter it
        if ((loadedObjects == null) || (loadedObjects.isEmpty())
                || isInternalSecurityCall(messageContext))
        {
            return null;
        }

        ArrayList filteredRPMObjects = new ArrayList();
        Set visitedRPMObjectInfos = new HashSet();
        Iterator it = loadedObjects.iterator();
        while (it.hasNext())
        {
            RPMObject rpmObject = (RPMObject) it.next();
            ReferencedObjectInfo refInfo = new ReferencedObjectInfo(null,
                rpmObject, null);
            visitedRPMObjectInfos.add(refInfo);
            if (securityController.applyFieldFiltering(rpmObject,
                messageContext).type != SecurityValidationResult.FALSE)
            {
                refInfo.canViewObject = true;//update can view since by default it is false
                filteredRPMObjects.add(rpmObject);
                filterRelatedObjectInfoSet(messageContext,
                    getDirectChildrenObjectsInfo(rpmObject, new HashSet()),
                    visitedRPMObjectInfos, securityController);
                filterRelatedObjectInfoSet(messageContext,
                    getDirectlyReferencedObjectsInfo(rpmObject, new HashSet()),
                    visitedRPMObjectInfos, securityController);
            }
        }

        Map genericCacheMap = messageContext.getRPMSecurityCache()
            .getGenericCacheMap();
        Map map = (Map) genericCacheMap.get(SecurityControllerUtil.class);
        if (map == null)
        {
            map = new HashMap();
            genericCacheMap.put(SecurityControllerUtil.class, map);
        }
        map.put(securityController, visitedRPMObjectInfos);
        return filteredRPMObjects;
    }

    private static void filterRelatedObjectInfoSet(
            MessageContext messageContext, Set rpmObjectInfos,
            Set visitedRPMObjectInfos, ISecurityController securityController)
            throws RPMException
    {
        Iterator it = rpmObjectInfos.iterator();
        while (it.hasNext())
        {
            ReferencedObjectInfo refInfo = (ReferencedObjectInfo) it.next();
            if (!visitedRPMObjectInfos.contains(refInfo))
            {
                visitedRPMObjectInfos.add(refInfo);
                if ((securityController.applyFieldFiltering(refInfo.rpmObject,
                    messageContext).type != SecurityValidationResult.FALSE))
                {
                    refInfo.canViewObject = true;//update can view since by default it is false
                    filterRelatedObjectInfoSet(messageContext,
                        getDirectChildrenObjectsInfo(refInfo.rpmObject,
                            new HashSet()), visitedRPMObjectInfos,
                        securityController);
                    filterRelatedObjectInfoSet(messageContext,
                        getDirectlyReferencedObjectsInfo(refInfo.rpmObject,
                            new HashSet()), visitedRPMObjectInfos,
                        securityController);
                }
            }
        }
    }
}
