/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.resource;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.security.containers.AbstractSecurityGroup;
import com.ibm.rpm.security.managers.SecurityMask;
import com.ibm.rpm.security.managers.SecurityUtil;

/**
 * @author Yasser Lulu
 *
 */
class CombinedResourceSecurityFlags extends CombinedSecurityFlags
{

    public static final CombinedResourceSecurityFlags ALL_TRUE = new CombinedResourceSecurityFlags(
        SecurityControllerUtil.makeArray(
            SecurityUtil.SECURITY_GROUP_LEVELS_COUNT, -1));

    public static final CombinedResourceSecurityFlags ALL_FALSE = new CombinedResourceSecurityFlags(
        new int[SecurityUtil.SECURITY_GROUP_LEVELS_COUNT]);

    private CombinedResourceSecurityFlags(int[] combinedFlags)
    {
        super(combinedFlags);
    }
    
    private static final int MASK = SecurityMask.MASK_ARRAY[RESOURCE.CanViewPersonalSecurity.intValue() % 32];
    private static final int LEVEL = RESOURCE.CanViewPersonalSecurity.intValue() / 32; 

    public CombinedResourceSecurityFlags(AbstractSecurityGroup securityGroup) throws RPMException
    {
        super(SecurityUtil.getClearanceLevels(securityGroup,
            new int[SecurityUtil.SECURITY_GROUP_LEVELS_COUNT]));
        
        //Based on WebUi request, we will always make the value of RESOURCE.CanViewPersonalSecurity
        //flag true (effectively disabling it)
        getCombinedFlags()[LEVEL] |= MASK;        
    }
    
   
    public CombinedSecurityFlags getALL_FALSE()
    {
        return ALL_FALSE;
    }
    
    
    public CombinedSecurityFlags getALL_TRUE()
    {
        return ALL_TRUE;
    }

}
