package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;

class ResourceBasedAbstractScopeSecurityController extends
        ResourceBasedAbstractAggregateScopeSecurityController
{

    public Class getContainerClass()
    {
        return AbstractScope.class;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("task", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assignedScorecard",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assetCode", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowProcessesAvailable",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("impact", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("duplicates", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("orderOfMagnitudeCostBenefits",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("documentFolder", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("approvedByName", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("externalReferenceNumber",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedAbstractScopeMappingEntry("published",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            RESOURCE.CanDeleteTemplates,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("duplicateNumber",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("state", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("approvedByDate", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowProcessesRunning",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("clientCostCenter",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("rank", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("targetDate", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("approvedDate", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defaultScope", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("asset", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void generateOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateOperationMappingEntries(mappingEntryMap);
        OperationMappingEntry operationInfo = null;

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.PublishLocalWBSTemplates, getContainerClass());
        operationInfo.securityFlag = RESOURCE.CanPublishLocalWBSTemplates;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.PublishToMethodWBSTemplates, getContainerClass());
        operationInfo.securityFlag = RESOURCE.CanPublishToMethodWBSTemplates;
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }
    
    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {

        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                AbstractScope abstractScope = (AbstractScope) rpmObject;
                PublishedType type = SecurityControllerUtil
                    .loadAbstractScopePublishedType(abstractScope,
                        messageContext);
                if (type == PublishedType.ToTemplate)
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        RESOURCE.CanDeleteTemplates, securityFlags,
                        getErrorMessage(), rpmObject.getClass());
                }
                return SecurityValidationResult.TRUE_RESULT;
            }
        };
        return deletionEntry;
    }

}
