package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.GenericDatafield;
import com.ibm.rpm.applicationadministration.containers.Geographical;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.applicationadministration.containers.PoolDomain;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.asset.containers.GenericAsset;
import com.ibm.rpm.clientcostcenters.containers.GenericClient;
import com.ibm.rpm.contract.containers.GenericContract;
import com.ibm.rpm.customfield.containers.GenericCustomField;
import com.ibm.rpm.customfield.containers.GenericCustomPortlet;
import com.ibm.rpm.financial.containers.GenericChargeCode;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.interfaces.Application;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.wbs.containers.GenericProject;

class ResourceBasedApplicationSecurityController extends
        ResourceBasedContainerSecurityController
{

    public Class getContainerClass()
    {
        return Application.class;
    }

    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateContactGroup, getContainerClass(),
            ContactGroup.class);
        creationInfo.securityFlag = RESOURCE.CanEditLocationGroups;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateGenericDataField, getContainerClass(),
            GenericDatafield.class);
        creationInfo.securityFlag = RESOURCE.CanEditDatafieldAdmin;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateCurrency, getContainerClass(),
            Currency.class);
        creationInfo.securityFlag = RESOURCE.CanEditCurrencyExchangeRates;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateGenericClient, getContainerClass(),
            GenericClient.class);
        creationInfo.securityFlag = RESOURCE.CanCreateClientCostCenterDivisionFolders;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateGenericContract, getContainerClass(),
            GenericContract.class);
        creationInfo.securityFlag = RESOURCE.CanCreateContractsAndFolders;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateGenericCustomField, getContainerClass(),
            GenericCustomField.class);
        creationInfo.securityFlag = RESOURCE.CanEditCustomFieldsAdmin;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
        
        //--------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateGenericCustomPortlet, getContainerClass(),
            GenericCustomPortlet.class);
        creationInfo.securityFlag = RESOURCE.CanEditCustomFieldsAdmin;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateGenericProject, getContainerClass(),
            GenericProject.class);
        creationInfo.securityFlag = RESOURCE.CanCreateEmptyProjects;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateGeographical, getContainerClass(),
            Geographical.class);
        creationInfo.securityFlag = RESOURCE.CanEditLocationGeographies;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateOrganizational, getContainerClass(),
            Organizational.class);
        creationInfo.securityFlag = RESOURCE.CanEditLocationOrganizations;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreatePoolDomain, getContainerClass(),
            PoolDomain.class);
        creationInfo.securityFlag = RESOURCE.CanEditLocationPoolDomains;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreatePool, getContainerClass(), Pool.class);
        creationInfo.securityFlag = RESOURCE.CanCreateResourcePools;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateRPMCalendar, getContainerClass(),
            RPMCalendar.class);
        creationInfo.securityFlag = RESOURCE.CanEditCalendarAdmin;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
        
        // --------------------------------------------------------
        creationInfo = new ResourceBasedCreateGenericAssetMappingEntry(
            SecuredOperations.CreateGenericAsset, getContainerClass(),
            GenericAsset.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateGenericChargeCode, getContainerClass(),
            GenericChargeCode.class);
        creationInfo.securityFlag = RESOURCE.CanEditAccountCodes;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
        
        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateGenericAttribute, getContainerClass(),
            AttributeCategory.class);
        creationInfo.securityFlag = RESOURCE.CanEditAttributeAdmin;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

}
