package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.OperationSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.resource.containers.Resource;

class ResourceBasedCreateAttributeAssignmentMappingEntry extends
        CreationOperationMappingEntry
{

    public ResourceBasedCreateAttributeAssignmentMappingEntry(
            String operationName, Class container, Class childrenClass)
    {
        super(operationName, container, childrenClass);
    }

    public SecurityValidationResult canPerform(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        RPMObject parent = rpmObject.getParent();
        if (parent.getID() != null
                && messageContext.getUser().getID().equals(parent.getID()))
        {
            return SecurityControllerUtil.calculateFlagResult(
                RESOURCE.CanEditPersonalExecutiveAttributes, securityFlags,
                getErrorMessage(), rpmObject.getClass());
        }
        else
        {
            return SecurityControllerUtil.calculateFlagResult(
                RESOURCE.CanEditResourceAttributeAdmin, securityFlags,
                getErrorMessage(), rpmObject.getClass());
        }
    }

    public void buildCreationOperationSecurityDescriptors(
            Map creationOperationSecurityDescrs,
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        OperationSecurityDescriptor operationSecurityDescr = null;

        if (objectInfo.rpmObject instanceof Resource
                && messageContext.getUser().getID().equals(objectInfo.rpmObject.getID()))
        {
            operationSecurityDescr = new OperationSecurityDescriptor();
            operationSecurityDescr
                .setName(SecuredOperations.CreateAttributeAssignment);
            operationSecurityDescr.setCanPerform(SecurityControllerUtil
                .calculateFlagValue(RESOURCE.CanEditPersonalExecutiveAttributes,
                    combinedSecurityFlags));
            creationOperationSecurityDescrs.put(operationSecurityDescr.getName(),
                operationSecurityDescr);
        }
        else
        {
            operationSecurityDescr = new OperationSecurityDescriptor();
            operationSecurityDescr
                .setName(SecuredOperations.CreateAttributeAssignment);
            operationSecurityDescr.setCanPerform(SecurityControllerUtil
                .calculateFlagValue(RESOURCE.CanEditResourceAttributeAdmin,
                    combinedSecurityFlags));
            creationOperationSecurityDescrs.put(operationSecurityDescr.getName(),
                operationSecurityDescr);
        }
    }
}
