package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.OperationSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.resource.containers.Resource;

class ResourceBasedCreateDocumentElementMappingEntry extends
        CreationOperationMappingEntry
{

    public ResourceBasedCreateDocumentElementMappingEntry(String operationName,
            Class container, Class childrenClass)
    {
        super(operationName, container, childrenClass);
    }

    public SecurityValidationResult canPerform(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        String resourceID = SecurityControllerUtil.loadResourceIdWithDocument(
            messageContext, (DocumentElement) rpmObject);
        if (resourceID != null)
        {
            if (messageContext.getUser().getID().equals(resourceID))
            {
                return SecurityControllerUtil.calculateFlagResult(
                    RESOURCE.CanEditPersonalDocuments, securityFlags,
                    getErrorMessage(), rpmObject.getClass());
            }
            else
            {
                return SecurityControllerUtil.calculateFlagResult(
                    RESOURCE.CanEditResourceDocuments, securityFlags,
                    getErrorMessage(), rpmObject.getClass());
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public void buildCreationOperationSecurityDescriptors(
            Map creationOperationSecurityDescrs,
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        OperationSecurityDescriptor operationSecurityDescr = null;

        if (objectInfo.rpmObject instanceof Resource
                && messageContext.getUser().getID().equals(
                    objectInfo.rpmObject.getID()))
        {
            operationSecurityDescr = new OperationSecurityDescriptor();
            operationSecurityDescr.setName(SecuredOperations.CreateDocuments);
            operationSecurityDescr.setCanPerform(SecurityControllerUtil
                .calculateFlagValue(RESOURCE.CanEditPersonalDocuments,
                    combinedSecurityFlags));
            creationOperationSecurityDescrs.put(operationSecurityDescr
                .getName(), operationSecurityDescr);
        }
        else
        {
            operationSecurityDescr = new OperationSecurityDescriptor();
            operationSecurityDescr.setName(SecuredOperations.CreateDocuments);
            operationSecurityDescr.setCanPerform(SecurityControllerUtil
                .calculateFlagValue(RESOURCE.CanEditResourceDocuments,
                    combinedSecurityFlags));
            creationOperationSecurityDescrs.put(operationSecurityDescr
                .getName(), operationSecurityDescr);
        }
    }
}
