/**
 * 
 */
package com.ibm.rpm.framework.security.controller.impl.resource;

import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.resource.containers.Resource;

class ResourceBasedCustomFieldAssignmentMappingEntry extends
        FieldMappingEntry
{
    public ResourceBasedCustomFieldAssignmentMappingEntry(String fieldName,
            Class container)
    {
        super(fieldName, container);
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (canReadSecurityFlags != null && canReadSecurityFlags.length >= 20) // XXX
        {
            CustomFieldAssignment customFieldAssignment = (CustomFieldAssignment) rpmObject;

            RPMObject parent = SecurityControllerUtil.loadParent(
                messageContext, rpmObject);
            if (parent instanceof Resource)
            {
                if (customFieldAssignment.getCustomField() == null)
                    return SecurityValidationResult.UNDEFINED_RESULT;
                if (customFieldAssignment.getCustomField().getSecurityGroup() == null)
                    return SecurityValidationResult.UNDEFINED_RESULT;
                if (customFieldAssignment.getCustomField().getSecurityGroup()
                    .intValue() > 0
                        && customFieldAssignment.getCustomField()
                            .getSecurityGroup().intValue() <= 10)
                {
                    int group = customFieldAssignment.getCustomField()
                        .getSecurityGroup().intValue();
                    if (parent.getID() != null
                        && messageContext.getUser().getID().equals(parent.getID()))
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            canReadSecurityFlags[group - 1], securityFlags,
                            getViewErrorMessage(), rpmObject.getClass());
                    }
                    else
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            canReadSecurityFlags[10 + group - 1],
                            securityFlags, getViewErrorMessage(), rpmObject.getClass());
                    }
                }
            }
            return SecurityValidationResult.TRUE_RESULT;
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (canWriteSecurityFlags != null && canWriteSecurityFlags.length >= 20) // XXX
        {
            CustomFieldAssignment customFieldAssignment = (CustomFieldAssignment) rpmObject;
            RPMObject parent = SecurityControllerUtil.loadParent(
                messageContext, rpmObject);
            if (parent instanceof Resource)
            {
                if (customFieldAssignment.getCustomField() == null)
                    return SecurityValidationResult.UNDEFINED_RESULT;
                if (customFieldAssignment.getCustomField().getSecurityGroup() == null)
                    return SecurityValidationResult.UNDEFINED_RESULT;
                if (customFieldAssignment.getCustomField().getSecurityGroup()
                    .intValue() > 0
                        && customFieldAssignment.getCustomField()
                            .getSecurityGroup().intValue() <= 10)
                {
                    int group = customFieldAssignment.getCustomField()
                        .getSecurityGroup().intValue();
                    if (parent.getID() != null
                        && messageContext.getUser().getID().equals(parent.getID()))
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            canWriteSecurityFlags[group - 1], securityFlags,
                            getEditErrorMessage(), rpmObject.getClass());
                    }
                    else
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            canWriteSecurityFlags[10 + group - 1],
                            securityFlags, getEditErrorMessage(), rpmObject.getClass());
                    }
                }
            }
            return SecurityValidationResult.TRUE_RESULT;
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
}