/**
 * 
 */
package com.ibm.rpm.framework.security.controller.impl.resource;

import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;

class ResourceBasedDocumentElementMappingEntry extends FieldMappingEntry
{
    public ResourceBasedDocumentElementMappingEntry(String fieldName,
            Class container)
    {
        super(fieldName, container);
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        String resourceID = SecurityControllerUtil.loadResourceIdWithDocument(
            messageContext, (DocumentElement) rpmObject);
        if (resourceID != null)
        {
            if (messageContext.getUser().getID().equals(resourceID))
            {
                return SecurityControllerUtil.calculateFlagResult(
                    RESOURCE.CanViewPersonalDocuments, securityFlags,
                    getViewErrorMessage(), rpmObject.getClass());
            }
            else
            {
                return SecurityControllerUtil.calculateFlagResult(
                    RESOURCE.CanViewResourceDocuments, securityFlags,
                    getViewErrorMessage(), rpmObject.getClass());
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        String resourceID = SecurityControllerUtil.loadResourceIdWithDocument(
            messageContext, (DocumentElement) rpmObject);
        if (resourceID != null)
        {
            if (messageContext.getUser().getID().equals(resourceID))
            {
                return SecurityControllerUtil.calculateFlagResult(
                    RESOURCE.CanEditPersonalDocuments, securityFlags,
                    getEditErrorMessage(), rpmObject.getClass());
            }
            else
            {
                return SecurityControllerUtil.calculateFlagResult(
                    RESOURCE.CanEditResourceDocuments, securityFlags,
                    getEditErrorMessage(), rpmObject.getClass());
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
}