package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.ViewOperationMappingEntry;
import com.ibm.rpm.resource.containers.EmploymentHistory;
import com.ibm.rpm.resource.containers.Resource;

class ResourceBasedEmploymentHistorySecurityController extends
        ResourceBasedContainerSecurityController
{

    public Class getContainerClass()
    {
        return EmploymentHistory.class;
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                RPMObject parent = SecurityControllerUtil.loadParent(
                    messageContext, rpmObject);
                if (parent instanceof Resource)
                {
                    if (parent.getID().equals(messageContext.getUser().getID()))
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanEditPersonalBackground, securityFlags,
                            getErrorMessage(), rpmObject.getClass());
                    }
                    else
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanEditResourceBackground, securityFlags,
                            getErrorMessage(), rpmObject.getClass());
                    }
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        ResourceBasedEmploymentHistoryMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEmploymentHistoryMappingEntry("title",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEmploymentHistoryMappingEntry("startDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEmploymentHistoryMappingEntry("company",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEmploymentHistoryMappingEntry("endDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEmploymentHistoryMappingEntry(
            "responsibilities", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEmploymentHistoryMappingEntry("salary",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEmploymentHistoryMappingEntry(
            "description", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedEmploymentHistoryMappingEntry("location",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected ViewOperationMappingEntry generateViewOperationMappingEntry()
    {
        ViewOperationMappingEntry viewEntry = new ViewOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags, MessageContext messageContext)
            {
                RPMObject parent = rpmObject.getParent();
                if (parent instanceof Resource)
                {
                    if (parent.getID() != null
                        && messageContext.getUser().getID().equals(parent.getID()))
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanViewPersonalBackground,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                    else
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanViewResourceBackground,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        return viewEntry;
    }

}
