package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

class ResourceBasedGenericProjectSecurityController extends
        ResourceBasedAggregateNodeSecurityController
{

    public Class getContainerClass()
    {
        return GenericProject.class;
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                WorkElement workElement = (WorkElement) rpmObject;
                ProjectPublishedType type = SecurityControllerUtil
                    .loadWorkElementPublishedType(workElement, messageContext);
                if (type == ProjectPublishedType.ToTemplate)
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        RESOURCE.CanDeleteTemplates, securityFlags,
                        getErrorMessage(), rpmObject.getClass());
                }
                return SecurityControllerUtil.calculateFlagResult(
                    RESOURCE.CanDeleteProjects, securityFlags,
                    getErrorMessage(), rpmObject.getClass());
            }
        };
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defectScorecard",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defectAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("calendar", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("riskAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("requirementAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowRoleMappings",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("portfolio", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assignedScorecards",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("deliverableAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("actionScorecard",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("documentAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("riskScorecard", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("earnedValuesCalculationState",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("probabilityPercent",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defaultSteps", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("issueAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("changeRequestScorecard",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("currency", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("issueScorecard", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("taskAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("requirementScorecard",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("serviceRequestScorecard",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("projectCostCenter",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defaultProject", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("actionAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("changeRequestAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("serviceRequestAttributeAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("opportunity", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("organizationalAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assignmentType", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void generateOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateOperationMappingEntries(mappingEntryMap);
        OperationMappingEntry operationInfo = null;

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.UpdateScorecardAdmin, getContainerClass());
        operationInfo.securityFlag = RESOURCE.CanUpdateScorecardAdmin;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.ViewArchivedProjects, getContainerClass());
        operationInfo.securityFlag = RESOURCE.CanViewArchivedProject;
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }

}
