package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.ViewOperationMappingEntry;
import com.ibm.rpm.resource.containers.ProjectHistory;
import com.ibm.rpm.resource.containers.Resource;

class ResourceBasedProjectHistorySecurityController extends
        ResourceBasedContainerSecurityController
{

    public Class getContainerClass()
    {
        return ProjectHistory.class;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedProjectHistoryMappingEntry("role",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedProjectHistoryMappingEntry("endDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedProjectHistoryMappingEntry("timeSpent",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedProjectHistoryMappingEntry(
            "projectLength", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedProjectHistoryMappingEntry(
            "classification", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedProjectHistoryMappingEntry("industry",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedProjectHistoryMappingEntry("description",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedProjectHistoryMappingEntry("client",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedProjectHistoryMappingEntry(
            "skillsResponsibilities", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedProjectHistoryMappingEntry("projectName",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedProjectHistoryMappingEntry("startDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected ViewOperationMappingEntry generateViewOperationMappingEntry()
    {
        ViewOperationMappingEntry viewEntry = new ViewOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags, MessageContext messageContext)
            {
                RPMObject parent = rpmObject.getParent();
                if (parent instanceof Resource)
                {
                    if (parent.getID() != null
                        && messageContext.getUser().getID().equals(parent.getID()))
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanViewPersonalBackground,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                    else
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            RESOURCE.CanViewResourceBackground,
                            securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        return viewEntry;
    }

}
