/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Collection;
import java.util.Iterator;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.managers.SecurityUtil;

/**
 * @author Yasser Lulu
 *
 */
class CombinedSecurityRoleFlags extends CombinedSecurityFlags
{

    public static final CombinedSecurityRoleFlags ALL_TRUE = new CombinedSecurityRoleFlags(
        SecurityControllerUtil.makeArray(
            SecurityUtil.SECURITY_ROLE_LEVELS_COUNT, -1));

    public static final CombinedSecurityRoleFlags ALL_FALSE = new CombinedSecurityRoleFlags(
        new int[SecurityUtil.SECURITY_ROLE_LEVELS_COUNT]);
    

    private CombinedSecurityRoleFlags(int[] combinedFlags)
    {
        super(combinedFlags);
    }

    public CombinedSecurityRoleFlags(Collection securityRoles) throws RPMException
    {
        super(extractSecurityFlags(securityRoles));
    }

    private static int[] extractSecurityFlags(Collection securityRoles) throws RPMException
    {
        int[] combinedFlags = new int[SecurityUtil.SECURITY_ROLE_LEVELS_COUNT];
        Iterator it = securityRoles.iterator();
        while (it.hasNext())
        {
            int[] flags = SecurityUtil.getClearanceLevels((SecurityRole) it
                .next(), new int[SecurityUtil.SECURITY_ROLE_LEVELS_COUNT]);
            for (int i = 0; i < flags.length; ++i)
            {
                combinedFlags[i] |= flags[i];//all flags have distinct bits
            }
        }
        return combinedFlags;
    }
    
    public CombinedSecurityFlags getALL_FALSE()
    {
        return ALL_FALSE;
    }
    
    
    public CombinedSecurityFlags getALL_TRUE()
    {
        return ALL_TRUE;
    }

}
