package com.ibm.rpm.framework.security.controller.impl.role;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;

class FieldMappingEntryWithSuperFlag extends FieldMappingEntry
{

    final Integer superFlag;

    final FieldMappingEntry entry;

    FieldMappingEntryWithSuperFlag(Integer superFlag,
            FieldMappingEntry entry)
    {
        super(entry.name, entry.container);
        this.superFlag = superFlag;
        this.entry = entry;
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        SecurityValidationResult result = SecurityControllerUtil
            .calculateFlagResult(superFlag, securityFlags,
                getViewErrorMessage(), rpmObject.getClass());
        if (result.type == SecurityValidationResult.TRUE)
        {
            return result; // super flag is true, then allow it
        }
        return entry.canRead(rpmObject, securityFlags, messageContext);
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        SecurityValidationResult result = SecurityControllerUtil
            .calculateFlagResult(superFlag, securityFlags,
                getEditErrorMessage(), rpmObject.getClass());
        if (result.type == SecurityValidationResult.TRUE)
        {
            return result; // super flag is true, then allow it
        }
        return entry.canWrite(rpmObject, securityFlags, messageContext);
    }

}
