package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.impl.ControllerSecuredOperations;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;

class RoleBasedAbstractAggregateScopeSecurityController extends
        RoleBasedAggregateScopeSecurityController
{
    public Class getContainerClass()
    {
        return AbstractAggregateScope.class;
    }

    protected void doGenerateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;
        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateAttributeAssignment,
            getContainerClass(), AttributeAssignment.class);
        creationInfo.securityFlag = ROLE.CanAddRemoveAttributes;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        //  --------------------------------------------------------
        creationInfo = new RoleBasedCreateScopeElementTaskAssignmentMappingEntry(
            ControllerSecuredOperations.CreateResourceTaskAssignment,
            getContainerClass(), ResourceTaskAssignment.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected void doGenerateFieldMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("closedByDate",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewClosedBy,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditClosedBy,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("attributeAssignments",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewAttributes,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveAttributes,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("estimatedDurationInMinutes",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("proposedByName",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewProposedBy,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditProposedBy,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("closedByName",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewClosedBy,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditClosedBy,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("proposedByDate",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewProposedBy,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditProposedBy,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("wbsFinancials",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("estimatedFinishDate",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewESTStartAndESTFinish,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditESTStartAndESTFinish,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("estimatedStartDate",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewESTStartAndESTFinish,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditESTStartAndESTFinish,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("statusUpdates",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("rtfAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("worksheetFinancials",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("transferedUnderWbsRoot",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("proposedDate",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDatesBaseline,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditDatesBaseline,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("referenceNumber",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewReferenceNumber,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("priority",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewPriority,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditPriority,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

}
