/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author Yasser Lulu
 *
 */
class RoleBasedAbstractFinancialForTimePhasedFieldsMappingEntry extends
        FieldMappingEntry
{

    public RoleBasedAbstractFinancialForTimePhasedFieldsMappingEntry(
            String fieldName, Class container)
    {
        super(fieldName, container);
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry#canRead(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags, com.ibm.rpm.framework.MessageContext)
     */
    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (canReadSecurityFlags != null)
        {
            RPMObject parent = SecurityControllerUtil.loadParent(
                messageContext, rpmObject, new WorkElementScope());
            if (parent instanceof GenericProject)
            {
                return SecurityControllerUtil.calculateFlagResultUsingOr(
                    canReadSecurityFlags, securityFlags, getViewErrorMessage(),
                    rpmObject.getClass());
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry#canWrite(com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags, com.ibm.rpm.framework.MessageContext)
     */
    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (canWriteSecurityFlags != null)
        {
            RPMObject parent = SecurityControllerUtil.loadParent(
                messageContext, rpmObject, new WorkElementScope());
            if (parent instanceof GenericProject)
            {
                return SecurityControllerUtil.calculateFlagResultUsingOr(
                    canWriteSecurityFlags, securityFlags, getViewErrorMessage(),
                    rpmObject.getClass());
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

}
