/**
 * 
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;

class RoleBasedAbstractFinancialMappingEntry extends FieldMappingEntry
{
    private final Map canReadSecurityFlagsMap;

    private final Map canWriteSecurityFlagsMap;

    public RoleBasedAbstractFinancialMappingEntry(String fieldName,
            Class container, Map canReadSecurityFlagsMap,
            Map canWriteSecurityFlagsMap)
    {
        super(fieldName, container);
        this.canReadSecurityFlagsMap = canReadSecurityFlagsMap;
        this.canWriteSecurityFlagsMap = canWriteSecurityFlagsMap;
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        ChargeCode chargeCode = ((AbstractFinancial) rpmObject).getChargeCode();
        if (chargeCode != null)
        {
            FinancialCategory category = chargeCode.getFinancialCategory();
            if (category != null)
            {
                Integer[] readFlags = (Integer[]) canReadSecurityFlagsMap
                    .get(category.getValue());
                if (readFlags != null)
                {
                    return SecurityControllerUtil.calculateFlagResultUsingOr(
                        readFlags, securityFlags, getViewErrorMessage(),
                        rpmObject.getClass());
                }
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        ChargeCode chargeCode = ((AbstractFinancial) rpmObject).getChargeCode();
        if (chargeCode != null)
        {
            FinancialCategory category = chargeCode.getFinancialCategory();
            if (category != null)
            {
                Integer[] readFlags = (Integer[]) canWriteSecurityFlagsMap
                    .get(category.getValue());
                if (readFlags != null)
                {
                    return SecurityControllerUtil.calculateFlagResultUsingOr(
                        readFlags, securityFlags, getEditErrorMessage(),
                        rpmObject.getClass());
                }
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
}