package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.ViewOperationMappingEntry;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.wbs.containers.GenericProject;

class RoleBasedAssignedScorecardSecurityController extends
        RoleBasedAbstractScorecardSecurityController
{

    public Class getContainerClass()
    {
        return AssignedScorecard.class;
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                RPMObject parent = SecurityControllerUtil.loadParent(
                    messageContext, rpmObject);
                if (parent instanceof AbstractScope)
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanAddRemoveScorecards, securityFlags,
                        getErrorMessage(), rpmObject.getClass());
                }
                else if (parent instanceof GenericProject)
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanAddRemoveProjectScorecard, securityFlags,
                        getErrorMessage(), rpmObject.getClass());
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);

        Integer[] canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewAssignedScorecard,//It is the same as SecurityFlags.CanViewProjectScorecard
        };
        Integer[] canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditAssignedScorecard,//It is the same as SecurityFlags.CanEditProjectScorecard
        };

        SecurityControllerUtil.replaceFieldMappingEntryFlags(
            canReadSecurityFlags, canWriteSecurityFlags, mappingEntryMap);

        FieldMappingEntry fieldInfo = null;
        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("score", getContainerClass());
        fieldInfo.canReadSecurityFlags = canReadSecurityFlags;
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assignedScorecardCategories",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = canReadSecurityFlags;
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("scorecardState", getContainerClass());
        fieldInfo.canReadSecurityFlags = canReadSecurityFlags;
        fieldInfo.canWriteSecurityFlags = canWriteSecurityFlags;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }
    
    protected void generateOperationMappingEntries(Map mappingEntryMap)
    {
        OperationMappingEntry operationInfo = null;

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.PublishScoreCard, getContainerClass());
        operationInfo.securityFlag = ROLE.CanEditAssignedScorecard;
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }

    protected ViewOperationMappingEntry generateViewOperationMappingEntry()
    {
        ViewOperationMappingEntry viewEntry = new ViewOperationMappingEntry(
            getContainerClass());
        viewEntry.securityFlag = ROLE.CanViewAssignedScorecard;
        return viewEntry;
    }

}
