/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;

/**
 * @author Yasser Lulu
 *
 */
class RoleBasedChargeCodeMappingEntry extends FieldMappingEntry
{
    private final Map canReadSecurityFlagsMap;

    private final Map canWriteSecurityFlagsMap;

    public RoleBasedChargeCodeMappingEntry(String fieldName, Class container,
            Map canReadSecurityFlagsMap, Map canWriteSecurityFlagsMap)
    {
        super(fieldName, container);
        this.canReadSecurityFlagsMap = canReadSecurityFlagsMap;
        this.canWriteSecurityFlagsMap = canWriteSecurityFlagsMap;
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        FinancialCategory category = ((ChargeCode) rpmObject).getFinancialCategory();
        if (category != null)
        {
            Integer[] readFlags = (Integer[]) canReadSecurityFlagsMap
                .get(category.getValue());
            if (readFlags != null)
            {
                return SecurityControllerUtil.calculateFlagResultUsingOr(
                    readFlags, securityFlags, getViewErrorMessage(), rpmObject
                        .getClass());
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        FinancialCategory category = ((ChargeCode) rpmObject).getFinancialCategory();
        if (category != null)
        {
            Integer[] readFlags = (Integer[]) canWriteSecurityFlagsMap
                .get(category.getValue());
            if (readFlags != null)
            {
                return SecurityControllerUtil.calculateFlagResultUsingOr(
                    readFlags, securityFlags, getEditErrorMessage(), rpmObject
                        .getClass());
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

}
