package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;

class RoleBasedChargeCodeSecurityController extends
        RoleBasedGenericChargeCodeSecurityController
{

    public Class getContainerClass()
    {
        return ChargeCode.class;
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                SecurityValidationResult result = SecurityValidationResult.UNDEFINED_RESULT;// = super.canDelete(rpmObject,
                //    securityFlags, messageContext);
                //if (result.type == SecurityValidationResult.FALSE)
                //{
                //    return result;
                //}
                FinancialCategory category = ((ChargeCode) rpmObject).getFinancialCategory();
                if (category != null)
                {
                    if (category.getValue().equals(
                        FinancialCategory._Benefit))
                    {
                        return SecurityControllerUtil
                            .calculateFlagResult(
                                ROLE.CanAddRemoveBudgetCostRevenueFinancialsBenefits,
                                securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                    else if (category.getValue().equals(
                        FinancialCategory._Expense)
                            || category.getValue().equals(
                                FinancialCategory._Capital))
                    {
                        return SecurityControllerUtil
                            .calculateFlagResult(
                                ROLE.CanAddRemoveBudgetCostRevenueFinancialsExpenseCapital,
                                securityFlags, getErrorMessage(), rpmObject.getClass());
                    }
                }
                return result;
            }
        };
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);

        // --------------------------------------------------------
        // Override all parent's entries by adding these flags        

        Map canReadSecurityFlagsMap = new HashMap();
        Map canWriteSecurityFlagsMap = new HashMap();

        //Benefits
        Integer[] flags = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsBenefits,
        };
        canReadSecurityFlagsMap.put(FinancialCategory._Benefit, flags);
        flags = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsBenefits,
        };
        canWriteSecurityFlagsMap.put(FinancialCategory._Benefit, flags);

        //Expense & Capital
        flags = new Integer[]
        {
            ROLE.CanViewBudgetCostRevenueFinancialsExpenseCapital,
        };
        canReadSecurityFlagsMap.put(FinancialCategory._Expense, flags);
        canReadSecurityFlagsMap.put(FinancialCategory._Capital, flags);
        flags = new Integer[]
        {
            ROLE.CanEditBudgetCostRevenueFinancialsExpenseCapital,
        };
        canWriteSecurityFlagsMap.put(FinancialCategory._Expense, flags);
        canWriteSecurityFlagsMap.put(FinancialCategory._Capital, flags);

        Iterator it = mappingEntryMap.entrySet().iterator();
        while (it.hasNext())
        {
            Entry entry = (Entry) it.next();
            entry.setValue(new RoleBasedChargeCodeMappingEntry((String) entry
                .getKey(), getContainerClass(), canReadSecurityFlagsMap,
                canWriteSecurityFlagsMap));
        }

        RoleBasedChargeCodeMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new RoleBasedChargeCodeMappingEntry("associatedChargeCode",
            ChargeCode.class, canReadSecurityFlagsMap, canWriteSecurityFlagsMap);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedChargeCodeMappingEntry("financialCategory",
            ChargeCode.class, canReadSecurityFlagsMap, canWriteSecurityFlagsMap);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedChargeCodeMappingEntry("state",
            ChargeCode.class, canReadSecurityFlagsMap, canWriteSecurityFlagsMap);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedChargeCodeMappingEntry("billable",
            ChargeCode.class, canReadSecurityFlagsMap, canWriteSecurityFlagsMap);
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

}
