package com.ibm.rpm.framework.security.controller.impl.role;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.OperationSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;

class RoleBasedCreateScopeElementChildMappingEntry extends
        CreationOperationMappingEntry
{
    private Map scopeElementCreationflags;

    public RoleBasedCreateScopeElementChildMappingEntry(String operationName,
            Class container, Class childrenClass)
    {
        super(operationName, container, childrenClass);
    }

    private Map getCreationFlagsMap()
    {
        if (scopeElementCreationflags == null)
        {
            scopeElementCreationflags = new HashMap();
            scopeElementCreationflags.put(ChangeRequest.class,
                ROLE.CanCreateChangeRequests);
            scopeElementCreationflags.put(Action.class, ROLE.CanCreateActions);
            scopeElementCreationflags.put(Defect.class, ROLE.CanCreateDefect);
            scopeElementCreationflags.put(Duplicate.class,
                ROLE.CanAddRemoveDuplicates);
            scopeElementCreationflags.put(Issue.class, ROLE.CanCreateIssues);
            scopeElementCreationflags.put(Requirement.class,
                ROLE.CanCreateRequirements);
            scopeElementCreationflags.put(Risk.class, ROLE.CanCreateRisks);
            scopeElementCreationflags.put(ServiceRequest.class,
                ROLE.CanCreateServiceRequests);
            scopeElementCreationflags.put(ScopeFolder.class,
                ROLE.CanCreateScopeElementsFolders);
        }
        return scopeElementCreationflags;
    }

    public SecurityValidationResult canPerform(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (SecurityControllerUtil.calculateFlagValue(
            ROLE.CanBreakdownScopeElements, securityFlags))
        {
            Integer flag = (Integer) getCreationFlagsMap().get(
                rpmObject.getClass());
            if (flag != null)
            {
                return SecurityControllerUtil.calculateFlagResult(flag,
                    securityFlags, getErrorMessage(), rpmObject.getClass());
            }
            return SecurityValidationResult.UNDEFINED_RESULT;
        }
        return SecurityValidationResult.makeFalseResult(MessageFormat.format(
            getErrorMessage(), new Object[]
            {
                StringUtil.getShortClassName(rpmObject.getClass())
            }));
    }

    public void buildCreationOperationSecurityDescriptors(
            Map creationOperationSecurityDescrs,
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {

        boolean canBreakDown = SecurityControllerUtil.calculateFlagValue(
            ROLE.CanBreakdownScopeElements, combinedSecurityFlags);
        // ------------------------------------------
        OperationSecurityDescriptor operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr.setName(SecuredOperations.CreateChangeRequests);
        operationSecurityDescr.setCanPerform(canBreakDown
                && SecurityControllerUtil.calculateFlagValue(
                    ROLE.CanCreateChangeRequests, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr.getName(),
            operationSecurityDescr);
        // --------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr.setName(SecuredOperations.CreateActions);
        operationSecurityDescr.setCanPerform(canBreakDown
                && SecurityControllerUtil.calculateFlagValue(
                    ROLE.CanCreateActions, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr.getName(),
            operationSecurityDescr);
        // --------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr.setName(SecuredOperations.CreateDefects);
        operationSecurityDescr.setCanPerform(canBreakDown
                && SecurityControllerUtil.calculateFlagValue(
                    ROLE.CanCreateDefect, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr.getName(),
            operationSecurityDescr);
        // --------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr.setName(SecuredOperations.CreateDuplicates);
        operationSecurityDescr.setCanPerform(canBreakDown
                && SecurityControllerUtil.calculateFlagValue(
                    ROLE.CanAddRemoveDuplicates, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr.getName(),
            operationSecurityDescr);
        // --------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr.setName(SecuredOperations.CreateIssues);
        operationSecurityDescr.setCanPerform(canBreakDown
                && SecurityControllerUtil.calculateFlagValue(
                    ROLE.CanCreateIssues, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr.getName(),
            operationSecurityDescr);
        // --------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr.setName(SecuredOperations.CreateRequirements);
        operationSecurityDescr.setCanPerform(canBreakDown
                && SecurityControllerUtil.calculateFlagValue(
                    ROLE.CanCreateRequirements, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr.getName(),
            operationSecurityDescr);
        // --------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr.setName(SecuredOperations.CreateRisks);
        operationSecurityDescr.setCanPerform(canBreakDown
                && SecurityControllerUtil.calculateFlagValue(
                    ROLE.CanCreateRisks, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr.getName(),
            operationSecurityDescr);
        // --------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr.setName(SecuredOperations.CreateServiceRequests);
        operationSecurityDescr.setCanPerform(canBreakDown
                && SecurityControllerUtil.calculateFlagValue(
                    ROLE.CanCreateServiceRequests, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr.getName(),
            operationSecurityDescr);
        // --------------------------------------------------
        operationSecurityDescr = new OperationSecurityDescriptor();
        operationSecurityDescr.setName(SecuredOperations.CreateScopeFolders);
        operationSecurityDescr.setCanPerform(canBreakDown
                && SecurityControllerUtil.calculateFlagValue(
                    ROLE.CanCreateScopeElementsFolders, combinedSecurityFlags));
        creationOperationSecurityDescrs.put(operationSecurityDescr.getName(),
            operationSecurityDescr);
    }
}
