/**
 * 
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult.ValueType;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.resource.containers.Resource;

class RoleBasedCustomFieldAssignmentMappingEntry extends FieldMappingEntry
{
    public RoleBasedCustomFieldAssignmentMappingEntry(String fieldName,
            Class container)
    {
        super(fieldName, container);
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (canReadSecurityFlags != null)
        {
            CustomFieldAssignment customFieldAssignment = (CustomFieldAssignment) rpmObject;
            RPMObject parent = SecurityControllerUtil.loadParent(
                messageContext, rpmObject);
            if (!(parent instanceof Resource))
            {
                if (customFieldAssignment.getCustomField() == null)
                    return SecurityValidationResult.UNDEFINED_RESULT; // TODO Reload the assignment with the custom field
                if (customFieldAssignment.getCustomField().getSecurityGroup() == null)
                    return SecurityValidationResult.UNDEFINED_RESULT;// TODO Reload the customfield with the security group
                if (customFieldAssignment.getCustomField().getSecurityGroup()
                    .intValue() > 0
                        && customFieldAssignment.getCustomField()
                            .getSecurityGroup().intValue() <= 10)
                {
                    int group = customFieldAssignment.getCustomField()
                        .getSecurityGroup().intValue();
                    return SecurityControllerUtil.calculateFlagResult(
                        canReadSecurityFlags[group - 1], securityFlags,
                        getViewErrorMessage(), rpmObject.getClass());
                }
            }
            return SecurityValidationResult.TRUE_RESULT;
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (canWriteSecurityFlags != null)
        {
            CustomFieldAssignment customFieldAssignment = (CustomFieldAssignment) rpmObject;
            
            RPMObject parent = SecurityControllerUtil.loadParent(
                messageContext, rpmObject);
            if (!(parent instanceof Resource))
            {
                if (customFieldAssignment.getCustomField() == null)
                    return SecurityValidationResult.UNDEFINED_RESULT;// TODO Reload the assignment with the custom field
                if (customFieldAssignment.getCustomField().getSecurityGroup() == null)
                    return SecurityValidationResult.UNDEFINED_RESULT;// TODO Reload the customfield with the security group
                if (customFieldAssignment.getCustomField().getSecurityGroup()
                    .intValue() > 0
                        && customFieldAssignment.getCustomField()
                            .getSecurityGroup().intValue() <= 10)
                {
                    int group = customFieldAssignment.getCustomField()
                        .getSecurityGroup().intValue();
                    return SecurityControllerUtil.calculateFlagResult(
                        canWriteSecurityFlags[group - 1], securityFlags,
                        getEditErrorMessage(), rpmObject.getClass());
                }
            }
            return SecurityValidationResult.TRUE_RESULT;
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
    
    /* 
     * If the value of this customfield is not a primitive type then
     * we must verify the is_Modified value inside the value's container 
     */
    public ValueType isModified(RPMObject rpmObject)
    {
        CustomFieldAssignment customFieldAssignment = (CustomFieldAssignment) rpmObject;
        Object value = customFieldAssignment.getValue();
        if (value instanceof Datafield)
        {
            return RoleBasedSecurityControllerImpl
                .getRoleBasedContainerSecurityController(value.getClass())
                .isModified((RPMObject) value);
        }
        return super.isModified(rpmObject);
    }
}