package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;

class RoleBasedDocumentFolderSecurityController extends
        RoleBasedDocumentElementSecurityController
{

    public Class getContainerClass()
    {
        return DocumentFolder.class;
    }

    protected void doGenerateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;
        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateDocumentsFolders, getContainerClass(),
            DocumentFolder.class);
        creationInfo.securityFlag = ROLE.CanCreateDocumentsFolders;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateDocuments, getContainerClass(),
            AbstractDocument.class);
        creationInfo.securityFlag = ROLE.CanCreateDocuments;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateNoteMinutes, getContainerClass(),
            NoteMinutes.class);
        creationInfo.securityFlag = ROLE.CanCreateNoteMinutes;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected DeletionOperationMappingEntry doGenerateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                if (rpmObject.getDeleted().booleanValue())
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanPermanentlyDeleteFolder, securityFlags,
                         getErrorMessage(), rpmObject.getClass());
                }
                else
                {
                    return ((FieldMappingEntry) getFieldMappingEntries().get(
                        "deleted")).canWrite(rpmObject, securityFlags,
                        messageContext);
                }
            }
        };
        return deletionEntry;
    }

    protected void doGenerateFieldMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        // Override parent's entry
        fieldInfo = new FieldMappingEntry("name", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanRenameFolders,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedDocumentElementDeletedMappingEntry("deleted",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanDeleteFolders, ROLE.CanDeletePublicDocuments,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("privateFolder", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanPublishRenderPrivateFolder,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }
  
    // Adds the CreationOperation descriptors of the scope's DocumentFolder 
    // to the scope's CreationOperation descriptors because the WebUI does not
    // receive the scope's DocumentFolder when it is empty but still requires security
    protected Map buildCreationOperationSecurityDescriptors(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        return super.buildCreationOperationSecurityDescriptors(objectInfo,
            combinedSecurityFlags, messageContext);
    }
    
}
