package com.ibm.rpm.framework.security.controller.impl.role;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.document.containers.AbstractDocument;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredFields;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.framework.util.StringUtil;

class RoleBasedGenericDocumentSecurityController extends
        RoleBasedDocumentElementSecurityController
{

    public Class getContainerClass()
    {
        return GenericDocument.class;
    }

    protected void doGenerateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;
        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateAttributeAssignment,
            getContainerClass(), AttributeAssignment.class);
        creationInfo.securityFlag = ROLE.CanAddRemoveAttributes;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new RoleBasedCreateDocumentsMappingEntry(
            SecuredOperations.CreateDocuments, getContainerClass(),
            AbstractDocument.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new RoleBasedCreateDocumentFolderMappingEntry(
            SecuredOperations.CreateDocumentsFolders, getContainerClass(),
            DocumentFolder.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new RoleBasedCreateNoteMinutesMappingEntry(
            SecuredOperations.CreateNoteMinutes, getContainerClass(),
            NoteMinutes.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected DeletionOperationMappingEntry doGenerateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                if (rpmObject.getDeleted().booleanValue())
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanPermanentlyDeleteDocuments, securityFlags,
                        "Permanently Delete DocumentElement", rpmObject.getClass());
                }
                else
                {
                    return ((FieldMappingEntry) getFieldMappingEntries().get(
                        "deleted")).canWrite(rpmObject, securityFlags,
                        messageContext);
                }
            }
        };
        return deletionEntry;
    }

    protected void doGenerateFieldMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("children", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanBreakdown,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("hasAttachment", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
        
        // --------------------------------------------------------
        fieldInfo = new RoleBasedDocumentElementDeletedMappingEntry("deleted",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanDelete, ROLE.CanDeletePublicDocuments,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowProcessesRunning",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedDocumentPropertiesMappingEntry(
            "elementSecurityType", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewElementSecurity,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditElementSecurity,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("parentSortingRank",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("lastUpdated", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("hostName", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("documentStatus", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("documentSize", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("templateSortingRank",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("wbsSortingRank", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("creationDate", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("documentCompressedSize",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("author", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowCollaborationDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowProcessesAvailable",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("checkoutFolder", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("customFieldAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("lastCheckedOutBy",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedDocumentPropertiesMappingEntry(
            "statusUpdates", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedDocumentPropertiesMappingEntry(
            "visibleAtParent", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewVisibleAtParent,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditVisibleAtParent,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedDocumentElementLockedMappingEntry("locked",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanLockUnlock, ROLE.CanUnlockAnyDocuments,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedDocumentPropertiesMappingEntry(
            "attributeAssignments", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewAttributes,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveAttributes,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("documentBlobDetails",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void doGenerateOperationMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateOperationMappingEntries(mappingEntryMap);
        OperationMappingEntry operationInfo = null;

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.ViewDocumentProperties, getContainerClass());
        operationInfo.securityFlag = ROLE.CanViewDocumentProperties;
        mappingEntryMap.put(operationInfo.name, operationInfo);
        
        //----------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.LockDocument, getContainerClass());
        operationInfo.securityFlag = ROLE.CanLockUnlock;
        mappingEntryMap.put(operationInfo.name, operationInfo);
        
        //----------------------------------------------------------
        operationInfo = new OperationMappingEntry(
			SecuredOperations.UnlockDocument, getContainerClass())
        {
        	public SecurityValidationResult canPerform(RPMObject rpmObject,
        			CombinedSecurityFlags securityFlags,
        			MessageContext messageContext) {
        		if (SecurityControllerUtil.calculateFlagValue(securityFlag,
        				securityFlags)
        				&& SecurityControllerUtil.calculateFlagValue(
        						ROLE.CanUnlockAny, securityFlags)) {
        			return SecurityValidationResult.TRUE_RESULT;
        		}
        		return SecurityValidationResult.makeFalseResult(MessageFormat
        				.format(getErrorMessage(), new Object[] { StringUtil
        					.getShortClassName(rpmObject.getClass()) }));
        	}
        };
		operationInfo.securityFlag = ROLE.CanLockUnlock;
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }

    protected Map buildCustomFieldSecurityDescriptors(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        Map customFieldSecurityDescrs = new HashMap();
        FieldSecurityDescriptor customFieldSecurityDescr = null;

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup1);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups1,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups1,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[1],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup2);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups2,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups2,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[2],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup3);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups3,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups3,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[3],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup4);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups4,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups4,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[4],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup5);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups5,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups5,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[5],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup6);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups6,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups6,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[6],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup7);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups7,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups7,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[7],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup8);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups8,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups8,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[8],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup9);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups9,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups9,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[9],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup10);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups10,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups10,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[10],
            customFieldSecurityDescr);

        return customFieldSecurityDescrs;
    }

}
