package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredFields;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.scopemanagement.containers.Issue;

class RoleBasedIssueSecurityController extends
        RoleBasedAbstractScopeSecurityController
{

    public Class getContainerClass()
    {
        return Issue.class;
    }

    protected void doGenerateFieldMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("numberTimesReassigned", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewTimesReassigned,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditTimesReassigned,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("occuredDateTime", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewOccurredDateTime,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditOccurredDateTime,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected Map buildRTFSecurityDescriptors(ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        Map rtfSecurityDescrs = new HashMap();
        FieldSecurityDescriptor rtfSecurityDescr = null;

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF1);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewIssueRTFs1, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditIssueRTFs1, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[1], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF2);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewIssueRTFs2, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditIssueRTFs2, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[2], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF3);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewIssueRTFs3, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditIssueRTFs3, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[3], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF4);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewIssueRTFs4, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditIssueRTFs4, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[4], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF5);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewIssueRTFs5, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditIssueRTFs5, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[5], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF6);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewIssueRTFs6, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditIssueRTFs6, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[6], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF7);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewIssueRTFs7, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditIssueRTFs7, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[7], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF8);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewIssueRTFs8, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditIssueRTFs8, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[8], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF9);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewIssueRTFs9, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditIssueRTFs9, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[9], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF10);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewIssueRTFs10, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditIssueRTFs10, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[10], rtfSecurityDescr);

        return rtfSecurityDescrs;
    }

}
