package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredFields;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.impl.CheckOutOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;

class RoleBasedNoteMinutesSecurityController extends
        RoleBasedGenericDocumentSecurityController
{

    public Class getContainerClass()
    {
        return NoteMinutes.class;
    }

    protected void doGenerateFieldMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        // Overwrite the inherited field because it is different security
        fieldInfo = new FieldMappingEntry("visibleAtParent",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewNoteMinutes,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditNoteMinutes,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("state", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("rank", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("rtfAssignments", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDescription,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditDescription,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        //--------------------------------------------------------
        fieldInfo = new FieldMappingEntry("state", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditState,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
        
        //--------------------------------------------------------
        fieldInfo = new FieldMappingEntry("rank", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewRank,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditRank,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
        
        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("priority", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewPriority,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditPriority,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        /*
         * Removing the CanViewDocumentProperties security from the map because
         * it does not apply to notes/minutes.
         */
        Iterator it = mappingEntryMap.entrySet().iterator();
        while (it.hasNext())
        {
            Entry mapEntry = (Entry) it.next();
            FieldMappingEntry fieldEntry = (FieldMappingEntry) mapEntry
                .getValue();
            if (fieldEntry instanceof RoleBasedDocumentPropertiesMappingEntry)
            {
                fieldInfo = new FieldMappingEntry(fieldEntry.name,
                    fieldEntry.container);
                fieldInfo.canReadSecurityFlags = fieldEntry.canReadSecurityFlags;
                fieldInfo.canWriteSecurityFlags = fieldEntry.canWriteSecurityFlags;
                mapEntry.setValue(fieldInfo);
            }
        }
    }

    protected void doGenerateOperationMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateOperationMappingEntries(mappingEntryMap);

        // --------------------------------------------------------
        // Removing thess operation because it is not applicable to notes/minutes.
        mappingEntryMap.remove(SecuredOperations.ViewDocumentProperties);
        mappingEntryMap.remove(SecuredOperations.CheckOutOrCheckOutTo);
    }

    //XXX Workaround for WebUI: Although the rtfAssignment in NoteMinutes is not a typical rtfAssignment
    //we are building a FieldSecurityDescriptor for it as a workaround for WebUI
    protected Map buildRTFSecurityDescriptors(ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        Map rtfSecurityDescrs = new HashMap();
        FieldSecurityDescriptor rtfSecurityDescr = null;

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF1);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewDescription, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditDescription, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[1], rtfSecurityDescr);

        return rtfSecurityDescrs;
    }
    
    protected CheckOutOperationMappingEntry doGenerateCheckOutOperationMappingEntry()
    {
        //The ROLE.CanCheckOutOrCheckOutTo is not applicable to NoteMinurtes, so, we'll
        //return our operation that has no flag at all -meaning it will return UNDEFINED
        CheckOutOperationMappingEntry checkOutEntry = new CheckOutOperationMappingEntry(
            getContainerClass());
        return checkOutEntry;
    }

}
