/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;

/**
 * @author Yasser Lulu
 * 
 */
class RoleBasedResourceTaskAssignmentMappingEntry extends FieldMappingEntry
{
    public RoleBasedResourceTaskAssignmentMappingEntry(String fieldName,
            Class container)
    {
        super(fieldName, container);
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        // TODO Validate if should return true of false?
        return SecurityValidationResult.TRUE_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (canWriteSecurityFlags != null)
        {
            RPMObject parent = SecurityControllerUtil.loadParent(
                messageContext, rpmObject);
            if (parent instanceof GenericProject)
            {
                return SecurityControllerUtil.calculateFlagResult(
                    canWriteSecurityFlags[0], securityFlags,
                    getEditErrorMessage(), rpmObject.getClass());
            }
            else
            {
                ResourceTaskAssignment rta = (ResourceTaskAssignment) rpmObject;
                if (rta.getTaskAssignment() != null)
                {
                    if (rta.getProfile() != null)
                    {
                        return SecurityControllerUtil.calculateFlagResult(
                            canWriteSecurityFlags[3], securityFlags,
                            getEditErrorMessage(), rpmObject.getClass());
                    }
                    else
                    {
                        SecurityControllerUtil.calculateFlagResult(
                            canWriteSecurityFlags[1], securityFlags,
                            getEditErrorMessage(), rpmObject.getClass());
                    }
                }
                else
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        canWriteSecurityFlags[2], securityFlags,
                        getEditErrorMessage(), rpmObject.getClass());
                }
            }

        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
}
