package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;

class RoleBasedResourceTaskAssignmentSecurityController extends
        RoleBasedContainerSecurityController
{

    public Class getContainerClass()
    {
        return ResourceTaskAssignment.class;
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                ResourceTaskAssignment assignment = (ResourceTaskAssignment) rpmObject;
                if (assignment.getResource() != null)
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanRemoveResourceOrParticipant, securityFlags,
                        getErrorMessage(), rpmObject.getClass());
                }
                return SecurityControllerUtil.calculateFlagResult(
                    ROLE.CanRemoveProfile, securityFlags,
                    getErrorMessage(), rpmObject.getClass());
            }
        };
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("securityRole", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedResourceTaskAssignmentMappingEntry("resource",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanAssignOrAffectResourceToProjectWithEffort,
                ROLE.CanAssignResource, ROLE.CanAssignParticipant,
                ROLE.CanReplaceProfile,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("taskAssignment", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assignmentStatus",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("resourceType", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedProfileTaskAssignmentMappingEntry("profile",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanAssignOrAffectProfileToProject, ROLE.CanAssignProfile,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void generateOperationMappingEntries(Map mappingEntryMap)
    {
        OperationMappingEntry operationInfo = null;

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.CopyProposedToPlan, getContainerClass());
        operationInfo.securityFlag = ROLE.CanCopyProposedToPlan;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        //--------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.ReplaceProfileWithResource, getContainerClass())
        {

            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                ResourceTaskAssignment rta = (ResourceTaskAssignment) rpmObject;
                boolean assignResource = true;
                if (rta.getParent() instanceof GenericProject)
                {
                    assignResource = SecurityControllerUtil.calculateFlagValue(
                        ROLE.CanRemoveProfile, securityFlags)
                            && SecurityControllerUtil
                                .calculateFlagValue(
                                    ROLE.CanAssignOrAffectResourceToProjectWithEffort,
                                    securityFlags);
                }
                if (assignResource)
                {
                    return super.canPerform(rpmObject, securityFlags,
                        messageContext);
                }
                return SecurityValidationResult.makeFalseResult(name);
            }
        };
        operationInfo.securityFlag = ROLE.CanReplaceProfile;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        //--------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.ReplaceResourceWithProfile, getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                if (SecurityControllerUtil.calculateFlagValue(
                    ROLE.CanRemoveResourceOrParticipant, securityFlags))
                {
                    ResourceTaskAssignment rta = (ResourceTaskAssignment) rpmObject;
                    boolean assignProfile = true;
                    if (rta.getParent() instanceof GenericProject)
                    {
                        assignProfile = SecurityControllerUtil
                            .calculateFlagValue(
                                ROLE.CanAssignOrAffectProfileToProject,
                                securityFlags);
                    }
                    if (assignProfile)
                    {
                        return super.canPerform(rpmObject, securityFlags,
                            messageContext);
                    }
                }
                return SecurityValidationResult.makeFalseResult(name);
            }
        };
        operationInfo.securityFlag = ROLE.CanAssignProfile;
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }

}
