/**
 * 
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;

class RoleBasedRtfAssignmentMappingEntry extends FieldMappingEntry
{
    public Map canReadSecurityFlagsMap = new HashMap();

    public Map canWriteSecurityFlagsMap = new HashMap();

    public RoleBasedRtfAssignmentMappingEntry(String fieldName, Class container)
    {
        super(fieldName, container);
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        RPMObject parent = SecurityControllerUtil.loadParent(
            messageContext, rpmObject);
        if (parent != null)
        {
            Integer[] readFlags = (Integer[]) SecurityControllerUtil
                .getValueFromMap(parent.getClass(), canReadSecurityFlagsMap);
            if (readFlags != null)
            {
                if (parent instanceof NoteMinutes)
                {
                    return SecurityControllerUtil.calculateFlagResultUsingOr(
                        readFlags, securityFlags, getViewErrorMessage(),
                        rpmObject.getClass());
                }
                else
                {
                    RtfAssignment assignment = (RtfAssignment) rpmObject;
                    if (assignment.getRtf() == null)
                        return SecurityValidationResult.UNDEFINED_RESULT;// TODO Reload the assignment with the rtf
                    if (assignment.getRtf().getName() == null)
                        return SecurityValidationResult.UNDEFINED_RESULT;// TODO Reload the rtf with the name
                    int rtf = new Integer(assignment.getRtf().getName())
                        .intValue()-1;
                    return SecurityControllerUtil.calculateFlagResult(
                        readFlags[rtf], securityFlags, getViewErrorMessage(),
                        rpmObject.getClass());
                }
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        RPMObject parent = SecurityControllerUtil.loadParent(
            messageContext, rpmObject);
        if (parent != null)
        {
            Integer[] writeFlags = (Integer[]) SecurityControllerUtil
                .getValueFromMap(parent.getClass(), canWriteSecurityFlagsMap);
            if (writeFlags != null)
            {
                if (parent instanceof NoteMinutes)
                {
                    return SecurityControllerUtil.calculateFlagResultUsingOr(
                        writeFlags, securityFlags, getEditErrorMessage(),
                        rpmObject.getClass());
                }
                else
                {
                    RtfAssignment assignment = (RtfAssignment) rpmObject;
                    if (assignment.getRtf() == null)
                        return SecurityValidationResult.UNDEFINED_RESULT;// TODO Reload the assignment with the rtf
                    if (assignment.getRtf().getName() == null)
                        return SecurityValidationResult.UNDEFINED_RESULT;// TODO Reload the rtf with the name
                    int rtf = new Integer(assignment.getRtf().getName())
                        .intValue()-1;
                    return SecurityControllerUtil.calculateFlagResult(
                        writeFlags[rtf], securityFlags, getEditErrorMessage(),
                        rpmObject.getClass());
                }
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
}