/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import java.text.MessageFormat;
import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.util.StringUtil;

/**
 * @author Yasser Lulu
 *
 */
class RoleBasedScheduleDateMappingEntry extends FieldMappingEntry
{
    private final Map canReadSecurityFlagsMap;

    private final Map canWriteSecurityFlagsMap;

    public RoleBasedScheduleDateMappingEntry(String fieldName, Class container,
            Map canReadSecurityFlagsMap, Map canWriteSecurityFlagsMap)
    {
        super(fieldName, container);
        this.canReadSecurityFlagsMap = canReadSecurityFlagsMap;
        this.canWriteSecurityFlagsMap = canWriteSecurityFlagsMap;
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        Integer[] readFlags = (Integer[]) canReadSecurityFlagsMap.get(rpmObject
            .getContextName());
        if (readFlags != null)
        {
            return SecurityControllerUtil.calculateFlagResultUsingOr(readFlags,
                securityFlags, getViewErrorMessage(), rpmObject.getClass());
        }
        //if it is not in the map, we'll return false to be safe
        return SecurityValidationResult.makeFalseResult(MessageFormat.format(
            getViewErrorMessage(), new Object[]
            {
                StringUtil.getShortClassName(rpmObject.getClass())
            }));
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        Integer[] readFlags = (Integer[]) canWriteSecurityFlagsMap
            .get(rpmObject.getContextName());
        if (readFlags != null)
        {
            return SecurityControllerUtil.calculateFlagResultUsingOr(readFlags,
                securityFlags, getEditErrorMessage(), rpmObject.getClass());
        }
        //if it is not in the map, we'll return false to be safe
        return SecurityValidationResult.makeFalseResult(MessageFormat.format(
            getEditErrorMessage(), new Object[]
            {
                StringUtil.getShortClassName(rpmObject.getClass())
            }));
    }

}
