package com.ibm.rpm.framework.security.controller.impl.role;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredFields;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;

class RoleBasedScopeElementSecurityController extends
        RoleBasedContainerSecurityControllerWithSuperFlag
{

    public Class getContainerClass()
    {
        return ScopeElement.class;
    }

    protected final Integer getSuperFlag()
    {
        return ROLE.AllScopeElementsPermissionsGranted;
    }

    protected DeletionOperationMappingEntry doGenerateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {

                if (SecurityControllerUtil
                    .isResourceAssignedToScopeElement((ScopeElement)rpmObject, messageContext
                        .getUser().getID(), messageContext))
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanDeleteScopeElements, securityFlags,
                        getErrorMessage(), rpmObject.getClass());

                }
                return SecurityControllerUtil.calculateFlagResult(
                    ROLE.CanDeletePublicScopeElements, securityFlags,
                    getErrorMessage(), rpmObject.getClass());
            }
        };
        return deletionEntry;
    }

    protected void doGenerateFieldMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateFieldMappingEntries(mappingEntryMap);

        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("project", getContainerClass());
        fieldInfo.canReadSecurityFlags = null;
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanTransferOrMoveScopeElementsAcrossProjects,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("templateSortingRank",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workElement", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("wbsParent", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("name", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanRenameScopeElements,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("elementSecurityType",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewElementSecurity,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditElementSecurity,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("wbsSortingRank", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("parentSortingRank",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("customFieldAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("resourceTaskAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void doGenerateOperationMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateOperationMappingEntries(mappingEntryMap);

        OperationMappingEntry operationInfo = null;

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.TransferOrMoveScopeElementsAcrossProjects,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanTransferOrMoveScopeElementsAcrossProjects;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.TransferScopeElementsToWBSView,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanTransferScopeElementsToWBSView;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.BreakdownScopeElements, getContainerClass());
        operationInfo.securityFlag = ROLE.CanBreakdownScopeElements;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.CloseElement, getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {

                if (SecurityControllerUtil.calculateFlagValue(securityFlag,
                    securityFlags)
                        && SecurityControllerUtil.calculateFlagValue(
                            ROLE.CanEditState, securityFlags))
                {
                    return SecurityValidationResult.TRUE_RESULT;
                }
                return SecurityValidationResult.makeFalseResult(MessageFormat
                    .format(getErrorMessage(), new Object[]
                    {
                        StringUtil.getShortClassName(rpmObject.getClass())
                    }));
            }
        };
        operationInfo.securityFlag = ROLE.CanCloseElement;
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }

    protected final Map buildCustomFieldSecurityDescriptors(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        Map customFieldSecurityDescrs = new HashMap();
        FieldSecurityDescriptor customFieldSecurityDescr = null;

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup1);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups1,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups1,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[1],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup2);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups2,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups2,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[2],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup3);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups3,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups3,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[3],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup4);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups4,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups4,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[4],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup5);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups5,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups5,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[5],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup6);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups6,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups6,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[6],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup7);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups7,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups7,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[7],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup8);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups8,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups8,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[8],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup9);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups9,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups9,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[9],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup10);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups10,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups10,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[10],
            customFieldSecurityDescr);

        return customFieldSecurityDescrs;
    }

}
