package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.ControllerSecuredOperations;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;

class RoleBasedScopeFolderSecurityController extends
        RoleBasedAggregateScopeSecurityController
{

    public Class getContainerClass()
    {
        return ScopeFolder.class;
    }

    protected void doGenerateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;
        
        // --------------------------------------------------------
        creationInfo = new RoleBasedCreateScopeElementChildMappingEntry(
            ControllerSecuredOperations.CreateScopeElement,
            getContainerClass(), ScopeElement.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected DeletionOperationMappingEntry doGenerateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                if (rpmObject.getDeleted().booleanValue())
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanPermanentlyDeleteFolder, securityFlags,
                        getErrorMessage(), rpmObject.getClass());
                }
                else
                {
                    return ((FieldMappingEntry) getFieldMappingEntries().get(
                        "deleted")).canWrite(rpmObject, securityFlags,
                        messageContext);
                }
            }
        };
        return deletionEntry;
    }

    protected void doGenerateFieldMappingEntries(Map mappingEntryMap)
    {
        super.doGenerateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        // Override parent's entry
        fieldInfo = new FieldMappingEntry("name", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanRenameFolders,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedScopeElementDeletedMappingEntry("deleted",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanDeleteFolders, ROLE.CanDeletePublicScopeElements,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("privateFolder", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanPublishRenderPrivateFolder,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }
    
    // expose the CreationOperation descriptors of the ScopeFolder 
    // to the wbs CreationOperation descriptors because the WebUI needs it for performance 
    // enhancement reason
    protected Map buildCreationOperationSecurityDescriptors(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        return super.buildCreationOperationSecurityDescriptors(objectInfo,
            combinedSecurityFlags, messageContext);
    }

}
