/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredFields;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;

/**
 * @author Yasser Lulu
 *
 */
public class RoleBasedScopeMilestoneSecurityController extends
        RoleBasedAbstractScopeTaskSecurityController
{

    public Class getContainerClass()
    {
        return ScopeMilestone.class;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.security.controller.impl.ContainerSecurityController#buildRTFSecurityDescriptors(com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo, com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags, com.ibm.rpm.framework.MessageContext)
     */
    protected Map buildRTFSecurityDescriptors(ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {

        Map rtfSecurityDescrs = new HashMap();
        FieldSecurityDescriptor rtfSecurityDescr = null;

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF1);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewMilestoneRTFs1, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditMilestoneRTFs1, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[1], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF2);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewMilestoneRTFs2, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditMilestoneRTFs2, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[2], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF3);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewMilestoneRTFs3, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditMilestoneRTFs3, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[3], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF4);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewMilestoneRTFs4, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditMilestoneRTFs4, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[4], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF5);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewMilestoneRTFs5, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditMilestoneRTFs5, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[5], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF6);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewMilestoneRTFs6, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditMilestoneRTFs6, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[6], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF7);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewMilestoneRTFs7, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditMilestoneRTFs7, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[7], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF8);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewMilestoneRTFs8, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditMilestoneRTFs8, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[8], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF9);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewMilestoneRTFs9, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditMilestoneRTFs9, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[9], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF10);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewMilestoneRTFs10, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditMilestoneRTFs10, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[10], rtfSecurityDescr);

        return rtfSecurityDescrs;

    }

}
