package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.ViewOperationMappingEntry;
import com.ibm.rpm.scopemanagement.containers.ScopeScheduleDate;

class RoleBasedScopeScheduleDateSecurityController extends
        RoleBasedScheduleDateSecurityController
{

    public Class getContainerClass()
    {
        return ScopeScheduleDate.class;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);

        // --------------------------------------------------------
        // Override all parent's entries by adding these flags        
        // RULES: EVERY TYPE MUST HAVE NON_NULL flags ENTRY IN THE MAP -even if it has no flags

        Map canReadSecurityFlagsMap = new HashMap();
        Map canWriteSecurityFlagsMap = new HashMap();

        // ScopeScheduleDate.APPROVED
        Integer[] flags = new Integer[]
        {
            ROLE.CanViewDatesApproved,
        };
        canReadSecurityFlagsMap.put(ScopeScheduleDate.APPROVED, flags);
        flags = new Integer[]
        {
            ROLE.CanEditDatesApproved,
        };
        canWriteSecurityFlagsMap.put(ScopeScheduleDate.APPROVED, flags);

        // ScopeScheduleDate.PROPOSED
        flags = null;
        canReadSecurityFlagsMap.put(ScopeScheduleDate.PROPOSED, flags);
        flags = null;
        canWriteSecurityFlagsMap.put(ScopeScheduleDate.PROPOSED, flags);

        Iterator it = mappingEntryMap.entrySet().iterator();
        while (it.hasNext())
        {
            Entry entry = (Entry) it.next();
            entry.setValue(new RoleBasedScheduleDateMappingEntry((String) entry
                .getKey(), getContainerClass(), canReadSecurityFlagsMap,
                canWriteSecurityFlagsMap));
        }
    }

    protected ViewOperationMappingEntry generateViewOperationMappingEntry()
    {
        ViewOperationMappingEntry viewEntry = new ViewOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags, MessageContext messageContext)
            {
                if (rpmObject.getContextName() == ScopeScheduleDate.APPROVED)
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanViewDatesApproved, securityFlags,
                        getErrorMessage(), rpmObject.getClass());
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        return viewEntry;
    }

}
