/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.ContainerSecurityController;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerImpl;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.ViewOperationMappingEntry;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.timesheet.containers.AbstractTimesheet;
import com.ibm.rpm.timesheet.containers.GenericStep;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;

/**
 * @author Yasser Lulu
 * 
 */
public class RoleBasedSecurityControllerImpl extends SecurityControllerImpl
{
    static final Map ROLE_SECURITY_CONTROLLERS = new HashMap();

    static
    {
        addContainerController(
            new RoleBasedAbstractAggregateScopeSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAbstractDocumentSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAbstractFinancialSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAbstractRequestSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedAbstractScopeSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAbstractScopeTaskSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAbstractScorecardCategorySecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAbstractScorecardElementSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAbstractScorecardEntitySecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAbstractScorecardQuestionSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAbstractScorecardResponseSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAbstractScorecardSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedActionSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedAggregateNodeSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedAggregateScopeSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedApprovedCostBenefitsSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedAssetSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAssignedScorecardCategorySecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAssignedScorecardQuestionSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAssignedScorecardResponseSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAssignedScorecardSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAssignedScorecardStateSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAttributeAssignmentSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedAvailableWorkflowProcessSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        //        addContainerController(
        //            new RoleBasedCalendarExceptionSecurityController(),
        //            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedChangeRequestSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedChargeCodeCategorySecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedChargeCodeSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedChargeCodeStateSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedClientCostCenterSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedCurrencySecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedCustomFieldAssignmentSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedDatafieldCategorySecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedDatafieldRTFSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedDatafieldStateSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedDefectSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedDeliverableSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedDocumentElementSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedDocumentFolderSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedDuplicateSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedElementDependencySecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedGenericChargeCodeSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedGenericClientSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedGenericDatafieldSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedGenericDocumentSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedGenericProjectSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedGenericTaskAssignmentSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedGenericWorkflowSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedImpactSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedIssueSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedMilestoneSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedMitigationFactorSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedMitigationTriggerSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedNoteMinutesSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedOpportunitySecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedOrderOfMagnitudeCostBenefitsSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedProjectOrganizationalAssignmentSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedReqProIntegrationFolderSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedReqProRequirementSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedReqProScopeFolderSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedRequirementSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedResourceTaskAssignmentSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedRiskMatrixSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedRiskSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        //        addContainerController(new RoleBasedRPMCalendarSecurityController(),
        //            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedRtfAssignmentSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedScheduleDateSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedScopeElementSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedScopeFolderSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedScopeScheduleDateSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedScorecardCategorySecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedScorecardFolderSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedScorecardQuestionSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedScorecardResponseSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedScorecardSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedServiceRequestSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedSimpleNodeSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedStateSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedSummaryTaskSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedTaskAssignmentSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedTaskSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedTimeCodeAssignmentSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedTimePhasedFinancialSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedUrlDocumentSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedValueDatafieldSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedWbsScheduleDateSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedWorkElementSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedWorkflowProcessSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedWorkflowRoleMappingSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(new RoleBasedWorkProductSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
        addContainerController(
            new RoleBasedRunningWorkflowProcessSecurityController(),
            ROLE_SECURITY_CONTROLLERS);

        addContainerController(new RoleBasedScopeTaskSecurityController(),
            ROLE_SECURITY_CONTROLLERS);

        addContainerController(new RoleBasedScopeMilestoneSecurityController(),
            ROLE_SECURITY_CONTROLLERS);
    }

    public static final ContainerSecurityController getRoleBasedContainerSecurityController(
            Class rpmObjectClass)
    {
        return (ContainerSecurityController) SecurityControllerUtil
            .getValueFromMap(rpmObjectClass, ROLE_SECURITY_CONTROLLERS);
    }

    private Map specialDeletionOperations = new HashMap();

    private Map specialViewOperations = new HashMap();

    private ViewOperationMappingEntry getSpecialViewOperationMappingEntry(
            RPMObject rpmObject)
    {
        ViewOperationMappingEntry operation = (ViewOperationMappingEntry) specialViewOperations
            .get(rpmObject.getClass());
        if (operation == null)
        {
            operation = new ViewOperationMappingEntry(rpmObject.getClass());
            specialViewOperations.put(rpmObject.getClass(), operation);
        }
        return operation;
    }

    private DeletionOperationMappingEntry getSpecialDeletionOperationMappingEntry(
            RPMObject rpmObject)
    {
        DeletionOperationMappingEntry operation = (DeletionOperationMappingEntry) specialDeletionOperations
            .get(rpmObject.getClass());
        if (operation == null)
        {
            operation = new DeletionOperationMappingEntry(rpmObject.getClass());
            specialDeletionOperations.put(rpmObject.getClass(), operation);
        }
        return operation;
    }

    protected CombinedSecurityFlags getCombinedSecurityFlags(
            RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        CombinedSecurityRoleFlags flags = null;
        Map cacheMap = null;//Check and updating cache is only done if the object id != null
        //Check the Cache
        if (rpmObject.getID() != null)
        {
            cacheMap = messageContext.getRPMSecurityCache()
                .getRPMObjectCacheMap(rpmObject);
            flags = (CombinedSecurityRoleFlags) cacheMap
                .get("CombinedSecurityRoleFlags");
            if (flags != null)
            {
                return flags;
            }
        }

        Set securityRoles = SecurityControllerUtil.getSecurityRoles(
            messageContext, rpmObject);
        if (securityRoles != null && !securityRoles.isEmpty())
        {
            flags = new CombinedSecurityRoleFlags(securityRoles);
        }
        else
        {
            //null security-roles-set means the object is NOT subject to security, therefore, allow everything
            if (securityRoles == null)
            {
                flags = CombinedSecurityRoleFlags.ALL_TRUE;
            }
            else
            {
                //Empty security-roles-set, it means an object is subject to security but we couldn't
                //get the roles (most probably the parent WBS/Doc/Scope element is not visible to us 
                //and when we tried to load it the secured SP DB return null) therefore, disable everything
                flags = CombinedSecurityRoleFlags.ALL_FALSE;
            }
        }

        //cache only if id != null, otherwise, it is a create, where the object
        //uses its parent roles for now, but then it might have its own roles applied
        if (rpmObject.getID() != null)
        {
            cacheMap.put("CombinedSecurityRoleFlags", flags);
        }
        return flags;
    }

    protected ContainerSecurityController getContainerSecurityController(
            Class rpmObjectClass)
    {
        return getRoleBasedContainerSecurityController(rpmObjectClass);
    }

    public SecurityValidationResult canAssociateWorkflow(
            MessageContext messageContext, WorkflowProcess process,
            GenericProject genericProject, boolean associate, boolean isDefault)
            throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(process
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                genericProject, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(process,
                    SecuredOperations.EditWorkflowElementsAssociations,
                    securityFlags,//we'll use the project's flags here
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canCreate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException
    {
        if (rpmObject.getParent() != null)
        {
            SecurityControllerUtil.cacheParentOfCreatedRPMObject(
                messageContext, rpmObject, rpmObject.getParent());

            ContainerSecurityController controller = getContainerSecurityController(rpmObject
                .getParent().getClass());
            if (controller != null)
            {
                CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                    rpmObject.getParent(), messageContext);
                if (securityFlags != null)
                {
                    return controller.canCreateChild(rpmObject, securityFlags,
                        messageContext);
                }
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canTransferUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(abstractScope
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                abstractScope, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(abstractScope,
                    SecuredOperations.TransferScopeElementsToWBSView,
                    securityFlags, messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
    
    public SecurityValidationResult canLockDocument(
			MessageContext messageContext, GenericDocument documentToLock)
			throws RPMException 
	{
		ContainerSecurityController controller = getContainerSecurityController(documentToLock
				.getClass());
		if (controller != null) {
			CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
					documentToLock, messageContext);
			if (securityFlags != null) {
				return controller.canPerform(documentToLock,
						SecuredOperations.LockDocument, securityFlags,
						messageContext);
			}
		}
		return SecurityValidationResult.UNDEFINED_RESULT;
	}

	public SecurityValidationResult canUnlockDocument(
			MessageContext messageContext, GenericDocument documentToUnlock)
			throws RPMException 
	{
		ContainerSecurityController controller = getContainerSecurityController(documentToUnlock
				.getClass());
		if (controller != null) {
			CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
					documentToUnlock, messageContext);
			if (securityFlags != null) {
				return controller.canPerform(documentToUnlock,
						SecuredOperations.UnlockDocument, securityFlags,
						messageContext);
			}
		}
		return SecurityValidationResult.UNDEFINED_RESULT;
	}

    public SecurityValidationResult canMoveAndTransferUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(abstractScope
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                abstractScope, messageContext);
            if (securityFlags != null)
            {
                return controller
                    .canPerform(
                        abstractScope,
                        SecuredOperations.TransferOrMoveScopeElementsAcrossProjects,
                        securityFlags, messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canMoveUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(abstractScope
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                abstractScope, messageContext);
            if (securityFlags != null)
            {
                return controller
                    .canPerform(
                        abstractScope,
                        SecuredOperations.TransferOrMoveScopeElementsAcrossProjects,
                        securityFlags, messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canTransferUnderScope(
            MessageContext messageContext, AbstractAggregateScope scopeElement,
            WorkElement workElement) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(workElement
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                workElement, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(workElement,
                    SecuredOperations.TransferWBSItemsToScopeManagementView,
                    securityFlags, messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canAcceptTaskAssignmentChanges(
            MessageContext messageContext, TaskAssignment taskAssignment,
            TaskAssignmentScope scope, boolean acceptEETC,
            boolean acceptForecastDates) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(taskAssignment
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                taskAssignment, messageContext);
            if (securityFlags != null)
            {
                SecurityValidationResult result = controller.canPerform(
                    taskAssignment, SecuredOperations.ViewScheduleInformation,
                    securityFlags, messageContext);
                if (result.type == SecurityValidationResult.FALSE)
                {
                    return result;
                }
                if (acceptForecastDates)
                {
                    return controller
                        .canPerform(
                            taskAssignment,
                            SecuredOperations.AcceptRejectVarianceAcceptRejectForecastDates,
                            securityFlags, messageContext);
                }
            }
        }

        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canRejectTaskAssignmentChanges(
            MessageContext messageContext, TaskAssignment taskAssignment,
            TaskAssignmentScope scope, boolean rejectEETC,
            boolean rejectForecastDates) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(taskAssignment
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                taskAssignment, messageContext);
            if (securityFlags != null)
            {
                SecurityValidationResult result = controller.canPerform(
                    taskAssignment, SecuredOperations.ViewScheduleInformation,
                    securityFlags, messageContext);
                if (result.type == SecurityValidationResult.FALSE)
                {
                    return result;
                }

                if (rejectForecastDates)
                {
                    return controller
                        .canPerform(
                            taskAssignment,
                            SecuredOperations.AcceptRejectVarianceAcceptRejectForecastDates,
                            securityFlags, messageContext);
                }
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canConvertContainer(
            MessageContext messageContext, RPMObject inputContainer,
            RPMObject outputContainer, RPMObjectScope rpmScope)
            throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(inputContainer
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                inputContainer, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(inputContainer,
                    SecuredOperations.ChangeElementType, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canCopyProposedToPlan(
            MessageContext messageContext, WorkElement workElement,
            WorkElementScope workElementScope) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(workElement
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                workElement, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(workElement,
                    SecuredOperations.CopyProposedToPlan, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canSaveProjectBaseline(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope scope) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rpmObject
            .getClass());
        if (controller != null)
        {
            if (rpmObject instanceof GenericProject)
            {
                CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                    rpmObject, messageContext);
                if (securityFlags != null)
                {
                    SecurityValidationResult result = null;
                    // Validates if can overwrite the previous baseline by saving a
                    // third time.
                    if (((WorkElement) rpmObject).getPreviousBaselineDate() != null)
                    {
                        result = controller.canPerform(rpmObject,
                            SecuredOperations.SaveBaselineUnlimited,
                            securityFlags, messageContext);
                        if (result.type == SecurityValidationResult.FALSE)
                        {
                            return result;
                        }
                    }
                    // TODO Need a way to dissociate saving one or many part of the
                    // baseline
                    // For now the API only supports saving 3 at once so all 3 flags
                    // must be
                    // true.
                    result = controller.canPerform(rpmObject,
                        SecuredOperations.SaveBaselineCrossCharges,
                        securityFlags, messageContext);
                    if (result.type == SecurityValidationResult.FALSE)
                    {
                        return result;
                    }

                    result = controller
                        .canPerform(
                            rpmObject,
                            SecuredOperations.SaveBaselineWBSDatesWBSFinancialsAndAssignments,
                            securityFlags, messageContext);
                    if (result.type == SecurityValidationResult.FALSE)
                    {
                        return result;
                    }

                    return controller
                        .canPerform(
                            rpmObject,
                            SecuredOperations.SaveBaselineEstDatesWBSFinancialsAndAffectedAssignments,
                            securityFlags, messageContext);
                }
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canRollupProjectEV(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope scope) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rpmObject
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                rpmObject, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(rpmObject,
                    SecuredOperations.RollupEV, securityFlags, messageContext);
            }
        }
        return SecurityValidationResult.TRUE_RESULT;
    }

    public SecurityValidationResult canAssignScoreCard(
            MessageContext messageContext, RPMObject objectToAssign,
            RPMObjectScope objectToAssignScope, Scorecard scoreCardToAssign,
            ReloadType reloadType) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(objectToAssign
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                objectToAssign, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(objectToAssign,
                    SecuredOperations.AssignScoreCard, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canCopyAssignmentProposedToPlan(
            MessageContext messageContext,
            ResourceTaskAssignment resourceTaskAssignment,
            ResourceTaskAssignmentScope scope) throws RPMException
    {
        //The same as CopyProposedToPlan
        ContainerSecurityController controller = getContainerSecurityController(resourceTaskAssignment
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                resourceTaskAssignment, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(resourceTaskAssignment,
                    SecuredOperations.CopyProposedToPlan, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canPublishScorecard(
            MessageContext messageContext, AssignedScorecard assignedScorecard,
            AbstractScorecardScope scorecardScope) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(assignedScorecard
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                assignedScorecard, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(assignedScorecard,
                    SecuredOperations.PublishScoreCard, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canReplaceProfileWithResource(
            MessageContext messageContext, ResourceTaskAssignment rta,
            ResourceTaskAssignmentScope rtaScope, Resource resource)
            throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rta
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(rta,
                messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(rta,
                    SecuredOperations.ReplaceProfileWithResource,
                    securityFlags, messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canReplaceResourceWithProfile(
            MessageContext messageContext, ResourceTaskAssignment rta,
            ResourceTaskAssignmentScope rtaScope) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rta
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(rta,
                messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(rta,
                    SecuredOperations.ReplaceResourceWithProfile,
                    securityFlags, messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canStartWorkflow(
            MessageContext messageContext, WorkflowProcess process,
            RPMObject object, RunningWorkflowProcessScope scope)
            throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(process
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                process, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(process,
                    SecuredOperations.StartWorkflow, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canSuspendWorkflow(
            MessageContext messageContext,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(runningProcess
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                runningProcess, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(runningProcess,
                    SecuredOperations.ManageWorkflow, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canCancelWorkflow(
            MessageContext messageContext, RunningWorkflowProcess runningProcess)
            throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(runningProcess
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                runningProcess, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(runningProcess,
                    SecuredOperations.ManageWorkflow, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canResumeWorkflow(
            MessageContext messageContext,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(runningProcess
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                runningProcess, messageContext);
            if (securityFlags != null)
            {
                return controller.canPerform(runningProcess,
                    SecuredOperations.ManageWorkflow, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    //Override parent to handle special cases
    public SecurityValidationResult canView(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        //handle special objects
        if (rpmObject instanceof AbstractTimesheet
                || rpmObject instanceof GenericStep)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                rpmObject, messageContext);
            if (securityFlags != null)
            {
                if (securityFlags.isALL_FALSE())
                {
                    return SecurityValidationResult.makeFalseResult("view "
                            + StringUtil
                                .getShortClassName(rpmObject.getClass()));
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        }
        return super.canView(rpmObject, messageContext);
    }

    //Override parent to handle special cases
    public SecurityValidationResult canDelete(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException
    {
        //handle special objects
        if (rpmObject instanceof AbstractTimesheet
                || rpmObject instanceof GenericStep)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                rpmObject, messageContext);
            if (securityFlags != null)
            {
                if (securityFlags.isALL_FALSE())
                {
                    return SecurityValidationResult.makeFalseResult("delete "
                            + StringUtil
                                .getShortClassName(rpmObject.getClass()));
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        }
        return super.canDelete(messageContext, rpmObject);
    }
}
