/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.role;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.wbs.containers.TaskAssignment;

/**
 * @author Yasser Lulu
 * 
 */
class RoleBasedTaskAssignmentMappingEntry extends FieldMappingEntry
{
    public RoleBasedTaskAssignmentMappingEntry(String fieldName, Class container)
    {
        super(fieldName, container);
    }

    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        return SecurityValidationResult.TRUE_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (canWriteSecurityFlags != null)
        {
            if (((TaskAssignment)rpmObject).getWorkSchedule() == null)
            { // Task assignment without effort
                return SecurityControllerUtil.calculateFlagResult(
                    canWriteSecurityFlags[0], securityFlags,
                    getEditErrorMessage(), rpmObject.getClass());
            }
            return SecurityControllerUtil.calculateFlagResult(
                canWriteSecurityFlags[1], securityFlags, getEditErrorMessage(),
                rpmObject.getClass());
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }
}
