package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.wbs.containers.TaskAssignment;

class RoleBasedTaskAssignmentSecurityController extends
        RoleBasedGenericTaskAssignmentSecurityController
{

    public Class getContainerClass()
    {
        return TaskAssignment.class;
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                Double actualWork = ((TaskAssignment) rpmObject)
                    .getActualWork();
                if ((actualWork != null) && (actualWork.doubleValue() > 0.0))
                {
                    return SecurityControllerUtil.calculateFlagResult(
                        ROLE.CanDeleteWBSItemsWithActualAssignments,
                        securityFlags, "delete {0} with actual assignments", 
                        rpmObject.getClass());
                }
                return SecurityValidationResult.UNDEFINED_RESULT;
            }
        };
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workSchedule", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assignmentStartDateType",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedTaskAssignmentMappingEntry("daysOfDelay",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanModifyAssignment, ROLE.CanModifyEffortDurationTask,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("baselineWork", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewScheduleInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("plannedDurationHours",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewScheduleInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("specialCostPerHour",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewResourceProfileCost,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditResourceProfileCost,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedTaskAssignmentMappingEntry(
            "assignmentFinishDate", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanModifyAssignment, ROLE.CanModifyEffortDurationTask,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("expectedStartDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("calculatedLeveledWork",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("billable", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("costType", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("expectedWork", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewScheduleInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("calculatedLeveledStartDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("costPerHour", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewResourceProfileCost,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditResourceProfileCost,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("forecastFinishDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("calculatedLeveledFinishDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedTaskAssignmentMappingEntry("percentageWork",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanModifyAssignment, ROLE.CanModifyEffortDurationTask,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("forecastStartDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("baselineStartDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("percentageDurationComplete",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("actualFinishDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("proposedDurationHours",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assignmentFinishDateType",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedTaskAssignmentMappingEntry(
            "percentageComplete", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanModifyAssignment, ROLE.CanModifyEffortDurationTask,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("defaultLabor", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("costLaborCode", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("sellingRatePerHour",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewResourceProfileRates,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditResourceProfileRates,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedTaskAssignmentMappingEntry("EETCHours",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanModifyAssignment, ROLE.CanModifyEffortDurationTask,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("actualWork", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewScheduleInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanModifyAssignment, ROLE.CanModifyEffortDurationTask,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("revenueLaborCode",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("contract", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("specialSellingRatePerHour",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewResourceProfileRates,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditResourceProfileRates,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("expectedFinishDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedTaskAssignmentMappingEntry("unitsPercentage",
            getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanModifyAssignment, ROLE.CanModifyEffortDurationTask,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("percentageRateAdjustment",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("lastScheduleWork",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("complete", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("remainingHours", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("baselineFinishDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new RoleBasedTaskAssignmentMappingEntry(
            "assignmentStartDate", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                ROLE.CanModifyAssignment, ROLE.CanModifyEffortDurationTask,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("actualStartDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void generateOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateOperationMappingEntries(mappingEntryMap);
        OperationMappingEntry operationInfo = null;

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.AcceptRejectVarianceAcceptRejectForecastDates,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanAcceptRejectVarianceAcceptRejectForecastDates;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.ViewScheduleInformation, getContainerClass());
        operationInfo.securityFlag = ROLE.CanViewScheduleInformation;
        mappingEntryMap.put(operationInfo.name, operationInfo);
    }

}
