package com.ibm.rpm.framework.security.controller.impl.role;

import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredFields;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.wbs.containers.Task;

class RoleBasedTaskSecurityController extends
        RoleBasedSimpleNodeSecurityController
{

    public Class getContainerClass()
    {
        return Task.class;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        super.generateFieldMappingEntries(mappingEntryMap);
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("percentageDurationCompleted",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDurationComplete,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditDurationComplete,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected Map buildRTFSecurityDescriptors(ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        Map rtfSecurityDescrs = new HashMap();
        FieldSecurityDescriptor rtfSecurityDescr = null;

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF1);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewTaskRTFs1, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditTaskRTFs1, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[1], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF2);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewTaskRTFs2, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditTaskRTFs2, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[2], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF3);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewTaskRTFs3, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditTaskRTFs3, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[3], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF4);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewTaskRTFs4, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditTaskRTFs4, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[4], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF5);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewTaskRTFs5, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditTaskRTFs5, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[5], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF6);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewTaskRTFs6, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditTaskRTFs6, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[6], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF7);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewTaskRTFs7, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditTaskRTFs7, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[7], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF8);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewTaskRTFs8, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditTaskRTFs8, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[8], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF9);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewTaskRTFs9, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditTaskRTFs9, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[9], rtfSecurityDescr);

        // --------------------------------------------------------
        rtfSecurityDescr = new FieldSecurityDescriptor();
        rtfSecurityDescr.setName(SecuredFields.RTF10);
        rtfSecurityDescr.setCanView(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanViewTaskRTFs10, combinedSecurityFlags));
        rtfSecurityDescr.setCanEdit(SecurityControllerUtil.calculateFlagValue(
            ROLE.CanEditTaskRTFs10, combinedSecurityFlags));
        rtfSecurityDescrs.put(SecuredFields.INTEGERS[10], rtfSecurityDescr);

        return rtfSecurityDescrs;
    }

}
