package com.ibm.rpm.framework.security.controller.impl.role;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredFields;
import com.ibm.rpm.framework.security.SecuredOperations;
import com.ibm.rpm.framework.security.SecurityFlags.ROLE;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.ControllerSecuredOperations;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.OperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;

class RoleBasedWorkElementSecurityController extends
        RoleBasedContainerSecurityController
{

    public Class getContainerClass()
    {
        return WorkElement.class;
    }

    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;
        // --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateAttributeAssignment,
            getContainerClass(), AttributeAssignment.class);
        creationInfo.securityFlag = ROLE.CanAddRemoveAttributes;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        //      --------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateElementDependency, getContainerClass(),
            ElementDependency.class);
        creationInfo.securityFlag = ROLE.CanAddRemoveDependency;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        //      --------------------------------------------------------
        creationInfo = new RoleBasedCreateWorkElementResourceTaskAssignmentMappingEntry(
            ControllerSecuredOperations.CreateResourceTaskAssignment,
            getContainerClass(), ResourceTaskAssignment.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
        
        //     ------------------------------------------------------------
        creationInfo = new CreationOperationMappingEntry(
            SecuredOperations.CreateTimeCodeAssignment, getContainerClass(),
            TimeCodeAssignment.class);
        creationInfo.securityFlag = ROLE.CanAddRemoveTimeCodes;
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    

    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass());
        deletionEntry.securityFlag = ROLE.CanDeleteWBSItems;
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowProcessesAvailable",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewWorkflowElementAssociations,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditWorkflowElementsAssociations,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("lastModifiedDate",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("dependencies", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveDependency,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("state", getContainerClass())
        {
            public SecurityValidationResult canRead(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                return SecurityValidationResult.TRUE_RESULT;
            }

            public SecurityValidationResult canWrite(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                SecurityValidationResult result = SecurityControllerUtil
                    .calculateFlagResult(ROLE.CanEditState, securityFlags,
                        getEditErrorMessage(), rpmObject.getClass());
                if (result.type != SecurityValidationResult.FALSE)
                {
                	WorkElement workElement = (WorkElement)rpmObject;
                	if (workElement.testStateModified())
                	{
                        DatafieldState state = workElement.getState();
                        if (state != null)
                        {
                            Boolean b = state.getClosedState();
                            if (b != null && b.booleanValue())

                                result = SecurityControllerUtil
                                    .calculateFlagResult(ROLE.CanCloseElement,
                                        securityFlags, getEditErrorMessage(),
                                        rpmObject.getClass());
                        }
                	}
                }
                return result;
            }
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("timeCodeAssignments",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewTimeCodes,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveTimeCodes,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("closedByDate", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("actualDate", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewScheduleInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditDatesActual,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("planDate", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewScheduleInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditDatesPlan,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("estimatedStartDate",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewESTStartAndESTFinish,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditESTStartAndESTFinish,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("documentFolder", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("initialBaselineDate",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewScheduleInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("previousBaselineDate",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewScheduleInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("estimatedFinishDate",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewESTStartAndESTFinish,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditESTStartAndESTFinish,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("referenceID", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("priority", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewWBSPriority,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditWBSPriority,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("earnedValues", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("workflowProcessesRunning",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("scopeElement", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("resourceTaskAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("assetCode", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("statusUpdates", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("closedByName", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("referenceNumber",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewReferenceNumber,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditReferenceNumber,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("minutesPerDay", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("forecastDate", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewScheduleInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("timeCode2", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDefaultTimeCodes,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditDefaultTimeCodes,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("name", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanRenameWBSItems,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("baselineDate", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDatesBaseline,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("timeCode1", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewDefaultTimeCodes,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanEditDefaultTimeCodes,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("externalID", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("wbsFinancials", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("asset", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("wbsSortingRank", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("rtfAssignments", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("published", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("expectedDate", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewScheduleInformation,
        };
        fieldInfo.canWriteSecurityFlags = null;
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("templateSortingRank",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("customFieldAssignments",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("containingProject",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("attributeAssignments",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
            ROLE.CanViewAttributes,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            ROLE.CanAddRemoveAttributes,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected void generateOperationMappingEntries(Map mappingEntryMap)
    {
        OperationMappingEntry operationInfo = null;

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.CopyProposedToPlan, getContainerClass());
        operationInfo.securityFlag = ROLE.CanCopyProposedToPlan;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.ChangeElementType, getContainerClass());
        operationInfo.securityFlag = ROLE.CanChangeWBSItemTypes;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        //--------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.TransferWBSItemsToScopeManagementView,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanTransferWBSItemsToScopeManagementView;
        mappingEntryMap.put(operationInfo.name, operationInfo);

        //      --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.EditWorkflowElementsAssociations,
            getContainerClass());
        operationInfo.securityFlag = ROLE.CanEditWorkflowElementsAssociations;
        mappingEntryMap.put(operationInfo.name, operationInfo);
        
        // --------------------------------------------------------
        operationInfo = new OperationMappingEntry(
            SecuredOperations.CloseElement, getContainerClass())
        {
            public SecurityValidationResult canPerform(RPMObject rpmObject,
                    CombinedSecurityFlags securityFlags,
                    MessageContext messageContext)
            {
                if (SecurityControllerUtil.calculateFlagValue(securityFlag,
                    securityFlags)
                        && SecurityControllerUtil.calculateFlagValue(
                            ROLE.CanEditState, securityFlags))
                {
                    return SecurityValidationResult.TRUE_RESULT;
                }
                return SecurityValidationResult.makeFalseResult(MessageFormat
                    .format(getErrorMessage(), new Object[]
                    {
                        StringUtil.getShortClassName(rpmObject.getClass())
                    }));
            }
        };
        operationInfo.securityFlag = ROLE.CanCloseElement;
        mappingEntryMap.put(operationInfo.name, operationInfo);
}

    protected Map buildCustomFieldSecurityDescriptors(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        Map customFieldSecurityDescrs = new HashMap();
        FieldSecurityDescriptor customFieldSecurityDescr = null;

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup1);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups1,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups1,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[1],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup2);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups2,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups2,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[2],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup3);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups3,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups3,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[3],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup4);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups4,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups4,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[4],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup5);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups5,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups5,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[5],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup6);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups6,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups6,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[6],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup7);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups7,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups7,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[7],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup8);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups8,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups8,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[8],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup9);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups9,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups9,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[9],
            customFieldSecurityDescr);

        // --------------------------------------------------------
        customFieldSecurityDescr = new FieldSecurityDescriptor();
        customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup10);
        customFieldSecurityDescr.setCanView(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanViewCustomFieldsGroups10,
                combinedSecurityFlags));
        customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
            .calculateFlagValue(ROLE.CanEditCustomFieldsGroups10,
                combinedSecurityFlags));
        customFieldSecurityDescrs.put(SecuredFields.INTEGERS[10],
            customFieldSecurityDescr);

        return customFieldSecurityDescrs;
    }

    // Adds the CreationOperation descriptors of the project's DocumentFolder 
    // to the project's CreationOperation descriptors because the WebUI does not
    // receive the project's DocumentFolder when it is empty but still requires security 
    protected Map buildCreationOperationSecurityDescriptors(ReferencedObjectInfo objectInfo, CombinedSecurityFlags combinedSecurityFlags, MessageContext messageContext)
    {
        Map map = super.buildCreationOperationSecurityDescriptors(objectInfo,
            combinedSecurityFlags, messageContext);
        RoleBasedDocumentFolderSecurityController docFoldercontroller = 
            (RoleBasedDocumentFolderSecurityController) RoleBasedSecurityControllerImpl
                .getRoleBasedContainerSecurityController(DocumentFolder.class);
        if (docFoldercontroller != null)
        {
            map.putAll(docFoldercontroller.buildCreationOperationSecurityDescriptors(
                objectInfo, combinedSecurityFlags, messageContext));
        }
        
        
        RoleBasedScopeFolderSecurityController scopeFoldercontroller = 
            (RoleBasedScopeFolderSecurityController) RoleBasedSecurityControllerImpl
                .getRoleBasedContainerSecurityController(ScopeFolder.class);
        if (scopeFoldercontroller != null)
        {
            map.putAll(scopeFoldercontroller.buildCreationOperationSecurityDescriptors(
                objectInfo, combinedSecurityFlags, messageContext));
        }
        return map;
    }
    
}
