package com.ibm.rpm.framework.util;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.RestoreRelevantFilteredFieldsParameter;

public class CheckpointCaller
{
    public static boolean isValidSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException
    {
        AbstractCheckpoint checkpoint = 
            ContainerToCheckpointMap.getCheckpoint(rpmObject);
        boolean isValid = true;
        if (checkpoint != null)
        {
            isValid = checkpoint.isValidSave(rpmObject, rpmScope, messageContext);
        }
        return isValid;
    }
    
    public static boolean canSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext)
    {
        return AbstractCheckpoint.canSave(rpmObject, rpmScope, messageContext);
    }    
    
    public static boolean canUpdateOnCreate(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext)           
    {
        return AbstractCheckpoint.canUpdateOnCreate(rpmObject, rpmScope, messageContext);
    }
    
    
    public static boolean restoreRelevantFilteredFields(RestoreRelevantFilteredFieldsParameter param)
    {
        return AbstractCheckpoint.restoreRelevantFilteredFields(param);
    }

    // TODO: Call from AbstractRPMObjectManager, adjust signature, use in specific checkpoints.
    public static boolean isValidXPath(Class containerClass, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException
    {
        AbstractCheckpoint checkpoint = 
            ContainerToCheckpointMap.getCheckpoint(containerClass);
        boolean isValid = true;
        if (checkpoint != null)
        {
            isValid = checkpoint.isValidXPath(containerClass, rpmScope, messageContext);
        }
        return isValid;
    }

    // TODO: Call from AbstractRPMObjectManager, adjust signature, use in specific checkpoints.
    public static boolean isValidLoad(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException
    {
        AbstractCheckpoint checkpoint = 
            ContainerToCheckpointMap.getCheckpoint(rpmObject);
        boolean isValid = true;
        if (checkpoint != null)
        {
            isValid = checkpoint.isValidLoad(rpmObject, rpmScope, messageContext);
        }
        return isValid;
    }    

    public static boolean isValidDelete(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        AbstractCheckpoint checkpoint = 
            ContainerToCheckpointMap.getCheckpoint(rpmObject);
        boolean isValid = true;
        if (checkpoint != null)
        {
            isValid = checkpoint.isValidDelete(rpmObject, messageContext);
        }
        return isValid;
    }
    
    public static boolean canDelete(RPMObject rpmObject,
            MessageContext messageContext)
    {
        return AbstractCheckpoint.canDelete(rpmObject, messageContext);
    }

}
