package com.ibm.rpm.framework.util;

import java.util.HashMap;

import com.ibm.rpm.framework.RPMException;

/**
 * This class is used to match containers to their full name.
 *
 * @author gmeloche
 */
public class ContainerClassMap
{

    private HashMap containerClassMap = new HashMap();

    private static ContainerClassMap instance = new ContainerClassMap();

    public static ContainerClassMap getInstance()
    {
        return instance;
    }

    /**
     * Adds the container and matching class to the map if the container is
     * not already in the map.
     * 
     * @param containerClass
     *            The container name to add to the map.
     */
    public void addContainerClass(Class containerClass)
    {
        if (containerClass != null)
        {
            String shortName = StringUtil.getShortClassName(containerClass.getName())
                .toUpperCase();

            if (containerClassMap.get(shortName) == null)
            {
                containerClassMap.put(shortName, containerClass);
            }
        }
    }

    /**
     * Returns the matching class for a container. Returns null if the container
     * is not in the mapped.
     *
     * @param containerShortName
     *            The container for which to retrieve the class.
     */
    public Class getContainerClass(String containerShortName) throws RPMException
    {
        Class containerClass = null;

        containerClass = (Class) containerClassMap.get(containerShortName.toUpperCase());

        return containerClass;
    }
}
