package com.ibm.rpm.framework.util;

import java.lang.reflect.Method;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * @author gmeloche
 *  
 */
public class ContainerFieldInfoMap
{
    private static ContainerFieldInfoMap instance = new ContainerFieldInfoMap();

    private Map containerFieldmap = new Hashtable();

    private ContainerFieldInfoMap()
    {
        super();
    }

    public static ContainerFieldInfoMap getInstance()
    {
        return instance;
    }

    /**
     * Map the field name from the metadata to a FieldInfo then map the
     * container's classname from the metadata to the previous map.
     * 
     * @param container
     * @param fieldName
     * @param fieldGetter
     * @param scopeGetter
     * @param isComposite
     * @param isAlwaysInScope
     */
    public void addFieldInfo(Class container, String fieldName,
            Method fieldGetter, Method scopeGetter, boolean isComposite,
            boolean isAlwaysInScope)
    {

        Map fields = (Hashtable) containerFieldmap.get(container);

        if (fields == null)
        {
            fields = new Hashtable();
            containerFieldmap.put(container, fields);
        }

        //TODO validate field getter is not null
        
        FieldInfo fieldInfo = new FieldInfo(fieldName);
        fieldInfo.setFieldGetter(fieldGetter);
        fieldInfo.setScopeGetter(scopeGetter);
        fieldInfo.setComposite(isComposite);
        fieldInfo.setAlwaysInScope(isAlwaysInScope);

        fields.put(fieldName.toUpperCase(), fieldInfo);

    }

    public FieldInfo[] getFieldInfos(Class container)
    {

        FieldInfo[] fieldInfos = null;

        Map fields = (Hashtable) containerFieldmap.get(container);

        if (fields != null)
        {
            fieldInfos = new FieldInfo[fields.size()];
            fields.values().toArray(fieldInfos);
        }

        return fieldInfos;
    }
    
    public Map getFieldMap(Class container)
    {
        return (Hashtable) containerFieldmap.get(container);
    }
    
    public int getNumberOfFieldsMapped()
    {
        int count = 0;
        Set keys = containerFieldmap.keySet();

        for (Iterator iter = keys.iterator(); iter.hasNext();)
        {
            Class element = (Class) iter.next();
            FieldInfo[] infos = getFieldInfos(element);
            if (infos != null)
            {
                count += infos.length;
            }

        }
        return count;
    }

}
