package com.ibm.rpm.framework.util;

import java.util.ArrayList;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;

/**
 * @author gmeloche
 *  
 */
public class ContextUtil
{
	private static Log logger = LogFactory.getLog(ContextUtil.class);
  
    public final static String NONE = "NotApplicable";
    public final static String CLIENT_CONTEXT = "Client";
    public final static String ASSET_CONTEXT = "Asset";
    public final static String POOL_CONTEXT = "Pool";
    public final static String SCOPE_CONTEXT = "Scope";
    public final static String WBS_CONTEXT = "WBS";
    public final static String RESOURCE_CONTEXT = "Resource";
    public final static String PERSONAL_CONTEXT = "Personal";
    public final static String ADMINISTRATIVE_CONTEXT = "Administrative";
    
    //Document contexts
    public final static String ASSET_DOCUMENT_CONTEXT = "Asset_Document";
    public final static String CLIENT_DOCUMENT_CONTEXT = "Client_Document";
    public final static String RESOURCE_DOCUMENT_CONTEXT = "Resource_Document";
    public final static String SCOPE_DOCUMENT_CONTEXT = "Scope_Document";
    public final static String WBS_DOCUMENT_CONTEXT = "Wbs_Document";
    public final static String POOL_DOCUMENT_CONTEXT = "Pool_Document";
    
    //Multiple context (internal use only)
    public final static String UMBRELLA_ATTRIBUTE_CONTEXT = "Attribute";
    public final static String UMBRELLA_RTF_CONTEXT = "Rtf";
    public final static String UMBRELLA_DOCUMENT_CONTEXT = "Document";
    public final static String UMBRELLA_SCORECARD_CONTEXT = "Scorecard";
    public final static String UMBRELLA_CUSTOM_FIELD_CONTEXT = "CustomField";
    public final static String CUSTOM_PORTLET_CONTEXT = "CustomPortlet";
    public final static String UMBRELLA_FINANCIAL_CONTEXT = "Financial";
    public final static String UMBRELLA_RESOURCE_TASK_ASSIGNMENT_CONTEXT = "ResourceTaskAssignment";
    public final static String UMBRElLA_GENERIC_TASK_ASSIGNMENT_CONTEXT = "GenericTaskAssignment";
    
    private final HashMap CONTAINERCONTEXTMAP = new HashMap();

    private static ContextUtil singleton = new ContextUtil();

    /**
     *  
     */
    private ContextUtil()
    {
        super();        
    }
    
    synchronized public static ContextUtil getInstance()
    {
        
        
        return singleton;
    } 
    
    
    public void addContainerToMap(String containerName, String contextName)
    {

        if (containerName != null)
        {
            String actualContextName = (String) CONTAINERCONTEXTMAP
                    .get(containerName.toUpperCase());
            if (actualContextName != null)
            {
                if (!actualContextName.equalsIgnoreCase(contextName))
                {
                    logger.error("The container name " + containerName
                            + " is already mapped to " + actualContextName + " context ");
                }
            }
            else
            {
                if (contextName != null)
                {
                    CONTAINERCONTEXTMAP.put(containerName.toUpperCase(),
                            contextName);

                    //Add container without the package name
                    CONTAINERCONTEXTMAP.put(StringUtil.getShortClassName(
                            containerName).toUpperCase(), contextName);
                }
            }
        }
    }
    
    public String getContextName(RPMObject rpmObject)throws RPMException{
    	return getContextName(rpmObject.getClass().getName());
    }
    
    public String getContextName(String containerName)throws RPMException{
        
        String contextName = null;
        
        if (containerName != null)
        {
            contextName = (String) CONTAINERCONTEXTMAP.get(containerName.toUpperCase());
            if (contextName == null)
            {                
                RPMException e = new RPMException("The container name "
                        + containerName + " is not mapped to a context.");
                throw e.log();
            }
        }
        return contextName;
    }
               
    public String[] getContextNames(String objectName) throws RPMException{
        
        String[] contextNames;
        String contextName = ContextUtil.getInstance().getContextName(
                objectName);

        if (contextName.equalsIgnoreCase(UMBRELLA_ATTRIBUTE_CONTEXT))
        {
            contextNames = new String[]
            {
                    CLIENT_CONTEXT, ASSET_CONTEXT, POOL_CONTEXT,
                    WBS_CONTEXT, UMBRELLA_DOCUMENT_CONTEXT, SCOPE_CONTEXT,
                    RESOURCE_CONTEXT
            };
        }
        else if (contextName.equalsIgnoreCase(UMBRELLA_RTF_CONTEXT))
        {
            contextNames = new String[]
            {
                    ASSET_CONTEXT, CLIENT_CONTEXT, UMBRELLA_DOCUMENT_CONTEXT, 
                    SCOPE_CONTEXT, WBS_CONTEXT
            };
        }
        else if (contextName.equalsIgnoreCase(UMBRELLA_DOCUMENT_CONTEXT))
        {
            contextNames = new String[]
            {
                    //Put only WBS_CONTEXT to load Wbs and Scope documents else
                    //the documents will be loaded twice.
                    //The Wbs and Scope documents are saved in the same table.                    
                    ASSET_DOCUMENT_CONTEXT, CLIENT_DOCUMENT_CONTEXT,
                    WBS_DOCUMENT_CONTEXT, RESOURCE_DOCUMENT_CONTEXT,
                    POOL_DOCUMENT_CONTEXT
            };
        }
        else if (contextName.equalsIgnoreCase(UMBRELLA_SCORECARD_CONTEXT))
        {
            contextNames = new String[]
            {
                    ASSET_CONTEXT, SCOPE_CONTEXT, WBS_CONTEXT, RESOURCE_CONTEXT
            };
        }
        else if (contextName.equalsIgnoreCase(UMBRELLA_CUSTOM_FIELD_CONTEXT))
        {
            contextNames = new String[]
            {
                    WBS_CONTEXT, SCOPE_CONTEXT, UMBRELLA_DOCUMENT_CONTEXT, 
                    ASSET_CONTEXT, RESOURCE_CONTEXT, CLIENT_CONTEXT
            };
        }
        else if (contextName.equalsIgnoreCase(UMBRELLA_FINANCIAL_CONTEXT))
        {
            contextNames = new String[]
            {
                    WBS_CONTEXT, SCOPE_CONTEXT
            };
        }
        else if (contextName.equalsIgnoreCase(UMBRELLA_RESOURCE_TASK_ASSIGNMENT_CONTEXT))
        {
            contextNames = new String[]
            {
                    WBS_CONTEXT, SCOPE_CONTEXT
            };
        }  
        else if (contextName.equalsIgnoreCase(UMBRElLA_GENERIC_TASK_ASSIGNMENT_CONTEXT))
        {
            contextNames = new String[]
            {
                    WBS_CONTEXT, PERSONAL_CONTEXT, ADMINISTRATIVE_CONTEXT
            };
        }
        else
        {
            contextNames = new String[]
            {
                contextName
            };
        }
        
        return contextNames;
    }
    
    public boolean isDocumentContext(String contextName){
        boolean isDocumentContext = contextName != null &&
        (contextName.equalsIgnoreCase(ASSET_DOCUMENT_CONTEXT)
                || contextName.equalsIgnoreCase(CLIENT_DOCUMENT_CONTEXT)
                || contextName.equalsIgnoreCase(RESOURCE_DOCUMENT_CONTEXT)
                || contextName.equalsIgnoreCase(SCOPE_DOCUMENT_CONTEXT)
                || contextName.equalsIgnoreCase(WBS_DOCUMENT_CONTEXT)
                || contextName.equalsIgnoreCase(POOL_DOCUMENT_CONTEXT));
        return isDocumentContext;
    }
    
    public boolean isScopeContext(String contextName){
        boolean isScopeContext = contextName != null &&
        (contextName.equalsIgnoreCase(SCOPE_CONTEXT)
                || contextName.equalsIgnoreCase(SCOPE_DOCUMENT_CONTEXT));
        return isScopeContext;
    }

    public boolean isWbsContext(String contextName){
        boolean isScopeContext = contextName != null &&
        (contextName.equalsIgnoreCase(WBS_CONTEXT)
                || contextName.equalsIgnoreCase(WBS_DOCUMENT_CONTEXT));
        return isScopeContext;
    }
    
    public boolean isResourceContext(String contextName){
        boolean isResourceContext = contextName != null &&
        (contextName.equalsIgnoreCase(RESOURCE_CONTEXT)
                || contextName.equalsIgnoreCase(RESOURCE_DOCUMENT_CONTEXT));
        return isResourceContext;
    }
    
    public boolean isAssetContext(String contextName) {
    	boolean isAssetContext = contextName != null &&
        (contextName.equalsIgnoreCase(ASSET_CONTEXT)
                || contextName.equalsIgnoreCase(ASSET_DOCUMENT_CONTEXT));
        return isAssetContext;
    }

    /*
     * 60302
     * 
     * if exactly one is an umbrella context, and the other a subcontext, reduce
     * calls to include only that common context.
     */
    public static String[] determineValidContexts(final String[] contextNames,
            final String[] parentContextNames)
    {
        String[] ret = contextNames;

        if (contextNames == null || contextNames[0] == ContextUtil.NONE
                || parentContextNames[0] == ContextUtil.NONE)
        {
            ret = contextNames;
        }
        else if (isParentContext(contextNames, parentContextNames)
                || isParentContext(parentContextNames, contextNames))
        {
            ArrayList queryTheseContextsOnly = new ArrayList();
            for (int i = 0; i < contextNames.length; i++)
            {
                for (int pi = 0; pi < parentContextNames.length; pi++)
                {
                    if (contextNames[i].equals(parentContextNames[pi]))
                    {
                        queryTheseContextsOnly.add(contextNames[i]);
                        break;
                    }
                }

                if(queryTheseContextsOnly.size() == 0) // because of Test49573
                {
                    ret = contextNames;
                } else {
                    ret = (String[]) queryTheseContextsOnly.toArray(new String[]{});
                }
            }
        }

        return ret;
    }
    
        
    private static boolean isParentContext(String[] contextNames,
            String[] parentContextNames)
    {
        boolean related = false;

        if (contextNames.length > 1 && parentContextNames.length == 1 || 
                parentContextNames.length > 1 && contextNames.length == 1 )
        {
            related = true; 
        }

        return related;
    }
}