/*
 * Created on 24-May-2006
 * @author mmatwiy
 * Utility class used to retrieve datafield values from xpath.
 */
package com.ibm.rpm.framework.util;

import java.util.List;
import java.util.Stack;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.xpathparser.Table;
import com.ibm.rpm.xpathparser.XPathToStack;

public class DatafieldUtil
{
    // gets the datafield from the DB with a specified name and type
    public static List getDatafieldValue(String type, String name,
            RPMObjectScope scope, MessageContext messageContext)
            throws RPMException
    {
        List result = null;

        try
        {
            String xpath = "/" + type + "[name='" + name + "']";            
            Stack stack = XPathToStack.parse(xpath);            
            Table peek = (Table) stack.peek();
            RPMObjectManager objectManager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(peek.getObjectName());
            result = objectManager.load(stack, scope, messageContext);
        }
        catch (Exception e)
        {
            ExceptionUtil.handleException(type, e);
        }

        return result;
    }

    public static String getDefaultCurrencyID(MessageContext messageContext)
            throws RPMException
    {
        String currencyID = null;

        // SELECT TMT_DATAFIELDS.ELEMENT_ID FROM TMT_DATAFIELDS 
        // WHERE TMT_DATAFIELDS.LEVEL_ID=? AND TMT_DATAFIELDS.TYPE_ID=? 
        // AND TMT_DATAFIELDS.DESCRIPTION IS NOT NULL 
        // ORDER BY TMT_DATAFIELDS.RANK WITH UR
        
        DatafieldManager datafieldManager = new DatafieldManager();
        
        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(DatafieldManager.NAME_ELEMENT_ID);
        stmt.appendFrom(datafieldManager.getTableName());
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(DatafieldManager.NAME_LEVEL_ID);
        stmt.appendEqualQuestionMark(DatafieldManager.NAME_TYPE_ID);
        stmt.appendIsNotNull(DatafieldManager.NAME_DESCRIPTION);
        stmt.appendOrderBy(DatafieldManager.NAME_RANK);

        List result = null;
        try
        {
            result = ManagerUtil.select(null, null, null, stmt, new Object[]
            {
                    new Integer(2), new Integer(Currency.TYPE_ID)
            }, messageContext);
        }
        catch (RPMException e)
        {
            ExceptionUtil.handleException(DatafieldUtil.class, e);
            result = null;
        }

        if (result != null && result.get(0) != null
                && ((List) result.get(0)).get(0) != null)
        {
            Number number = (((Number) ((List) result.get(0)).get(0)));
            if(number != null){
                currencyID = number.toString();
            }
        }
        
        return currencyID;
    }

}