package com.ibm.rpm.framework.util;

import java.util.ArrayList;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;

/**
 * @author gmeloche
 *  
 */
public class EnumeratedTypeUtil
{
    private static Log logger = LogFactory.getLog(EnumeratedTypeUtil.class);

    private HashMap CONTAINERMAP = new HashMap();
    
    private static String ENUMERATEDTYPE_TO_VALUE = "ENUMERATEDTYPE_TO_VALUE";
    
    private static String ENUMERATEDTYPE_TO_VALUES = "ENUMERATEDTYPE_TO_VALUES";
    
    private static String VALUE_TO_ENUMERATEDTYPE = "VALUE_TO_ENUMERATEDTYPE";

    private static EnumeratedTypeUtil singleton = new EnumeratedTypeUtil();

    /**
     *  
     */
    private EnumeratedTypeUtil()
    {
        super();
    }

    synchronized public static EnumeratedTypeUtil getInstance()
    {

        return singleton;
    }

    /**
     * 
     * @param enumeratedTypeClass
     *            Enumerated type class
     * @param type
     *            Specific type of an enumerated type
     * @param value
     *            Database value equivalent to this type
     * @param isCreateValue
     *            true if this value is used when the object is inserted in the
     *            database
     *            Only one value per specific type can have isCreateValue set to true, 
     *            otherwise this function will return an error
     * @param specificContainer
     *            null if the values are the same for all containers
     */
    public void addEnumeratedType(Class enumeratedTypeClass, Object type, Object value,
            boolean isCreateValue, Class specificContainer)
    {

        if (enumeratedTypeClass == null)
        {
            return;
        }
        
        if(type != null && value == null){
            return;
        }
        
        HashMap enumeratedTypeToValue = null;
        HashMap enumeratedTypeToValues = null;
        HashMap valueToEnumeratedType = null;
        
        //If no specific containers are specified, it will be mapped to RPMObject
        if(specificContainer == null){
            specificContainer = RPMObject.class;
        }
        
        HashMap containerMap = (HashMap)CONTAINERMAP.get(specificContainer);
        if(containerMap == null){
            containerMap = new HashMap();
            enumeratedTypeToValue = new HashMap();
            enumeratedTypeToValues = new HashMap();
            valueToEnumeratedType = new HashMap();
            containerMap.put(ENUMERATEDTYPE_TO_VALUE, enumeratedTypeToValue);
            containerMap.put(ENUMERATEDTYPE_TO_VALUES, enumeratedTypeToValues);
            containerMap.put(VALUE_TO_ENUMERATEDTYPE, valueToEnumeratedType);
            CONTAINERMAP.put(specificContainer, containerMap);
        }else{
            enumeratedTypeToValue = (HashMap)containerMap.get(ENUMERATEDTYPE_TO_VALUE);
            enumeratedTypeToValues = (HashMap)containerMap.get(ENUMERATEDTYPE_TO_VALUES);
            valueToEnumeratedType = (HashMap)containerMap.get(VALUE_TO_ENUMERATEDTYPE);
        }
 
        //ENUMERATEDTYPE_TO_VALUE
        // Map type as an uppercase string to an object
        // This map will contain only the create value to be used for insert statements
        if (isCreateValue)
        {
            HashMap typeMap = (HashMap) enumeratedTypeToValue
                .get(enumeratedTypeClass);
            if (typeMap == null)
            {
                typeMap = new HashMap();
                enumeratedTypeToValue.put(enumeratedTypeClass, typeMap);
            }

            //note: A null type indicates  that the enumerated type is not supported
            if (type != null)
            {

                Object o = typeMap.get(type.toString().toUpperCase());
                if (o != null)
                {
                    if (o != value)
                    {
                        logger.error(StringUtil
                            .getShortClassName(enumeratedTypeClass)
                                + "."
                                + type.toString()
                                + " is already mapped to value: " + o + ".");
                    }
                }
                else
                {
                    typeMap.put(type.toString().toUpperCase(), value);
                }
            }
        }

        //ENUMERATEDTYPE_TO_VALUES
        // This map contains all possible values, for loading
        HashMap typeMapMultipleValues = (HashMap) enumeratedTypeToValues
            .get(enumeratedTypeClass);
        if (typeMapMultipleValues == null)
        {
            typeMapMultipleValues = new HashMap();
            enumeratedTypeToValues.put(enumeratedTypeClass, typeMapMultipleValues);
        }
    
        //note: A null type indicates  that the enumerated type is not supported
        if (type != null)
        {
            Object o = typeMapMultipleValues.get(type.toString().toUpperCase());
            if (o == null)
            {
                o = new ArrayList();
            }
            ((ArrayList) o).add(value);
            typeMapMultipleValues.put(type.toString().toUpperCase(), o);
        }
    
        //VALUE_TO_ENUMERATEDTYPE
        //More than one type can be mapped to the same value
        HashMap valueMap = (HashMap) valueToEnumeratedType
            .get(enumeratedTypeClass);
        if (valueMap == null)
        {
            valueMap = new HashMap();
            valueToEnumeratedType.put(enumeratedTypeClass, valueMap);
        }
        valueMap.put(value, type);
    }

    public Object getValue(Class specificContainer, Class enumeratedTypeClass, String type)
    {

        Object value = null;
        HashMap enumeratedTypeToValue = null;
        
        HashMap containerMap = getContainerMap(specificContainer);
       
        enumeratedTypeToValue = (HashMap)containerMap.get(ENUMERATEDTYPE_TO_VALUE);
        
        if (enumeratedTypeToValue != null)
        {

            HashMap typeMap = (HashMap) enumeratedTypeToValue
                .get(enumeratedTypeClass);

            if (typeMap != null)
            {
                value = typeMap.get(type.toUpperCase());
                if (value == null)
                {
                    logger.error(StringUtil
                        .getShortClassName(enumeratedTypeClass)
                            + "."
                            + type.toString()
                            + " is not mapped to database values.");
                }
            }
            else
            {
                logger.error(StringUtil.getShortClassName(enumeratedTypeClass)
                        + " is not mapped to database values.");
            }
        }

        return value;
    }
    
    public ArrayList getValues(Class specificContainer, Class enumeratedTypeClass, String type)
    {

        Object values = null;
        HashMap enumeratedTypeToValues = null;
        
        HashMap containerMap = getContainerMap(specificContainer);
       
        enumeratedTypeToValues = (HashMap)containerMap.get(ENUMERATEDTYPE_TO_VALUES);
        
        if (enumeratedTypeToValues != null)
        {

            HashMap typeMapMultipleValues = (HashMap) enumeratedTypeToValues
                .get(enumeratedTypeClass);

            if (typeMapMultipleValues != null)
            {
                values = typeMapMultipleValues.get(type.toUpperCase());
                if (values == null)
                {
                    logger.error(StringUtil
                        .getShortClassName(enumeratedTypeClass)
                            + "."
                            + type.toString()
                            + " is not mapped to database values.");
                }
            }
            else
            {
                logger.error(StringUtil.getShortClassName(enumeratedTypeClass)
                        + " is not mapped to database values.");
            }
        }

        return (ArrayList) values;
    }
    
    public boolean isEnumeratedType(Class specificContainer, Class enumeratedTypeClass)
    {
        if(enumeratedTypeClass == null){
            return false;
        }
        
        HashMap enumeratedTypeToValue = null;
        
        HashMap containerMap = getContainerMap(specificContainer);
            
        enumeratedTypeToValue = (HashMap)containerMap.get(ENUMERATEDTYPE_TO_VALUE);
        
        return (enumeratedTypeToValue.get(enumeratedTypeClass) != null);
    }


    public Object getType(Class specificContainer, Class enumeratedTypeClass, Object value)
            throws RPMException
    {
        
        Object type = null;
        HashMap valueToEnumeratedType = null;
        
        HashMap containerMap = getContainerMap(specificContainer);
        
        valueToEnumeratedType = (HashMap)containerMap.get(VALUE_TO_ENUMERATEDTYPE);
        
        
        if (valueToEnumeratedType != null)
        {

            HashMap valueMap = (HashMap) valueToEnumeratedType
                .get(enumeratedTypeClass);

            if (valueMap != null)
            {
                type = valueMap.get(value);
                if (type == null)
                {
                    ExceptionUtil.handleInvalidEnumeratedType(
                        enumeratedTypeClass, value);
                }
            }
            else
            {
                logger.error(StringUtil.getShortClassName(enumeratedTypeClass)
                        + " is not mapped to database values.");
            }
        }

        return type;

    }
    
    private HashMap getContainerMap(Class specificContainer){
        
        HashMap containerMap = (HashMap)CONTAINERMAP.get(specificContainer);
        if(containerMap == null){
            containerMap = (HashMap)CONTAINERMAP.get(RPMObject.class);            
        }
        
        return containerMap;
        
    }

}
