/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.ibm.rpm.framework.security.ContainerSecurityDescriptor;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.Field;
import com.ibm.rpm.metadata.model.MetadataStore;

/**
 * @author Yasser Lulu
 *
 */

public class FieldSecurityDescriptorHandler
{
    protected final Class rpmObjectClass;

    protected List readOnlyFields;

    public FieldSecurityDescriptorHandler(Class rpmObjectClass)
    {
        this.rpmObjectClass = rpmObjectClass;
    }

    protected List getReadOnlyFields()
    {
        if (readOnlyFields == null)
        {
            readOnlyFields = new ArrayList();
            Container container = MetadataStore.getInstance()
                .getContainerByJavaClass(rpmObjectClass);
            if (container != null)
            {
                Iterator it = container.getFields().iterator();
                while (it.hasNext())
                {
                    Field field = (Field) it.next();
                    if (field.isReadOnlyOnUpdate())
                    {
                        readOnlyFields.add(field);
                    }
                }
            }
        }
        return readOnlyFields;
    }

    protected Map getFieldSecurityDescriptors(
            ContainerSecurityDescriptor containerDescr)
    {
        Map map = containerDescr.getFieldSecurityDescriptors();
        if (map == null || map.isEmpty())
        {
            map = new HashMap();
            containerDescr.setFieldSecurityDescriptors(map);
        }
        return map;
    }

    protected void setReadOnly(Map map, String fieldName)
    {
        FieldSecurityDescriptor descr = (FieldSecurityDescriptor) map
            .get(fieldName);
        if (descr == null)
        {
            descr = new FieldSecurityDescriptor();
            descr.setName(fieldName);
            descr.setCanView(true);
            map.put(descr.getName(), descr);
        }
        descr.setCanEdit(false);
    }

    protected void applyReadOnlyRestrictions(ContainerSecurityDescriptor containerDescr)
    {
        List fields = getReadOnlyFields();
        if (!fields.isEmpty())
        {
            Map map = getFieldSecurityDescriptors(containerDescr);
            Iterator it = fields.iterator();
            while (it.hasNext())
            {
                Field field = (Field) it.next();
                setReadOnly(map, field.getName());
            }
        }
    }
}