/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.util;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rpm.framework.security.ContainerSecurityDescriptor;
import com.ibm.rpm.scopemanagement.util.ReqProConnectionFieldSecurityDescriptorHandler;
import com.ibm.rpm.scopemanagement.util.ReqProIntegrationFolderFieldSecurityDescriptorHandler;
import com.ibm.rpm.scopemanagement.util.ReqProRequirementFieldSecurityDescriptorHandler;
import com.ibm.rpm.scopemanagement.util.ReqProScopeFolderFieldSecurityDescriptorHandler;

/**
 * @author Yasser Lulu
 *
 */
public class MetadataRulesUtility
{
    private static Map HANDLERS_MAP = new HashMap();

    static
    {
        //Add special handlers, the rest can use the generic one
        FieldSecurityDescriptorHandler handler = new ReqProRequirementFieldSecurityDescriptorHandler();
        HANDLERS_MAP.put(handler.rpmObjectClass, handler);

        handler = new ReqProIntegrationFolderFieldSecurityDescriptorHandler();
        HANDLERS_MAP.put(handler.rpmObjectClass, handler);

        handler = new ReqProConnectionFieldSecurityDescriptorHandler();
        HANDLERS_MAP.put(handler.rpmObjectClass, handler);
        
        handler = new ReqProScopeFolderFieldSecurityDescriptorHandler();
        HANDLERS_MAP.put(handler.rpmObjectClass, handler);
    }

    private MetadataRulesUtility()
    {
        //
    }

    private static FieldSecurityDescriptorHandler getFieldSecurityDescriptorHandler(
            Class rpmObjectClass)
    {
        FieldSecurityDescriptorHandler handler = (FieldSecurityDescriptorHandler) HANDLERS_MAP
            .get(rpmObjectClass);
        if (handler == null)
        {
            handler = new FieldSecurityDescriptorHandler(rpmObjectClass);
            HANDLERS_MAP.put(handler.rpmObjectClass, handler);
        }
        return handler;
    }

    public static void applyReadOnlyRestrictions(
            Map containerSecurityDescriptorsMap)
    {
        Iterator it = containerSecurityDescriptorsMap.entrySet().iterator();
        while (it.hasNext())
        {
            Map.Entry entry = (Map.Entry) it.next();
            FieldSecurityDescriptorHandler handler = getFieldSecurityDescriptorHandler(entry
                .getKey().getClass());
            handler
                .applyReadOnlyRestrictions((ContainerSecurityDescriptor) entry
                    .getValue());
        }
    }
}
