/*
 * Created on 18-Nov-2005
 */
package com.ibm.rpm.framework.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.MetadataStore;
import com.ibm.rpm.metadata.model.Scope;

/**
 * This class is used to match containers to their corresponding, and valid,
 * scopes. Only one scope can be specified per container.
 *
 * @author dominiqu
 */
public class RPMObjectScopeUtil
{
    private static Log logger = LogFactory.getLog(RPMObjectScopeUtil.class);

    private static RPMObjectScopeUtil instance = new RPMObjectScopeUtil();

    public static RPMObjectScopeUtil getInstance()
    {
        return instance;
    }

    /**
     * Returns the matching scope for a container. Returns null if the container
     * is not in the map.
     *
     * @param container
     *            The container for which to retrieve the valid scope.
     */
    public Class getScope(Class container)
    {
        Class scope = null;

        MetadataStore store = MetadataStore.getInstance();
        Container containerForScope = store.getContainerByJavaClass(container);
        if (containerForScope == null )
        {
            logger.error("Failed to read metadata for container: " + container.getName());
        }
        else
        {
            Scope scopeMetadata = containerForScope.getScopeMetadata();
            if ( scopeMetadata != null )
            {
                scope = scopeMetadata.getJavaClass();
            }
        }
        return scope;
    }

    /**
     * Returns the matching scope for the short name of a container. Returns null
     * if the container is not in the map.
     *
     * @param rpmObjectName
     *            The container for which to retrieve the valid scope without the package name.
     */
    public Class getScope(String rpmObjectName)
    {
        Class scope = null;

        MetadataStore store = MetadataStore.getInstance();
        Container containerForScope = store.getContainerByName(rpmObjectName);
        if (containerForScope == null )
        {
            logger.error("Failed to read metadata for container: " + rpmObjectName);
        }
        else
        {
            Scope scopeMetadata = containerForScope.getScopeMetadata();
            if ( scopeMetadata != null )
            {
                scope = scopeMetadata.getJavaClass();
            }
        }

        return scope;
    }
}
