package com.ibm.rpm.framework.util;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.Property;
import com.ibm.rpm.framework.RPMException;

public class SystemInfo
{
    /**
     * Returns system information as a list of properties (key/value pairs).
     * 
     * @return The list of Property.
     */
    public static ArrayList getSystemInfo(MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // Gets fixed system information.
        ArrayList systemInfo = AboutProperties.getInstance().getAllProperties(
                messageContext);
        // Gets dynamic system information.
        String dbVersion = getDatabaseVersion(messageContext);
        Property dbVersionProperty = new Property(
                "com.ibm.rpm.ws.database.version", dbVersion);
        systemInfo.add(dbVersionProperty);

        return systemInfo;
    }

    public static String getDatabaseVersion(MessageContext context)
            throws RPMException, SQLException, ParseException
    {
        String version = null;
        Connection c = null;
        ResultSet rs = null;
        try
        {
            c = Manager.getConnection(context);
            rs = Manager.executeStoredProcedure(c, "SP_GET_VERSION",
                    new Object[]
                    {}, true, context);

            rs.next();
            version = rs.getString(2);
        }
        finally
        {
            Manager.close(context, rs);
            Manager.commitAndCloseConnection(context, c);
        }
        return version;
    }
}
