/*
 * Created on Apr 29, 2005
 *
 */
package com.ibm.rpm.framework.util;

import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author dexmar00
 *
 * This class is used to implement a Thread Safe, reflection based toString() method.
 */
public class ToStringUtil {
	
	public static Log logger = LogFactory.getLog(ToStringUtil.class);
	
	public static class ThreadLocalTabs extends ThreadLocal {
		public static final String INDENT = "  ";

		protected Object initialValue() {
			return "";
		}

		public void incIndent() {
			super.set(((String) super.get()) + INDENT);
		}

		public void decIndent() {
			String currentTabs = (String) super.get();
			if (currentTabs.length() > INDENT.length()) {
				super.set(currentTabs.substring(INDENT.length()));
			}
		}

		public String getIndent() {
			return (String) super.get();
		}
	}

	public static class ThreadLocalObjectsPrintedPreviously extends
			ThreadLocal {
	    protected Object initialValue() {
			return new HashMap();
		}

		public boolean contains(Object o) {
			return ((HashMap) super.get()).containsKey(o);
		}

		public void add(Object o) {
			((HashMap) super.get()).put(o, o);
		}

		public void remove(Object o) {
			((HashMap) super.get()).remove(o);
		}
	}

	public static ThreadLocalTabs tabs = new ThreadLocalTabs();

	public static ThreadLocalObjectsPrintedPreviously objectsPrintedPreviously = new ThreadLocalObjectsPrintedPreviously();

	public static String arrayToString(Object o) {
		StringBuffer sb = new StringBuffer();
		int arrayLength = Array.getLength(o);
		sb.append("{");tabs.incIndent();
		for(int i=0;i<arrayLength;i++) {
			Object element = Array.get(o,i);
			if ( element != null && element.getClass().isArray() ) {
				tabs.incIndent();
				sb.append("\n"+tabs.getIndent()+"{"+arrayToString(element)+"\n");
				tabs.decIndent();
			}
			else {
				if (element instanceof String)
					sb.append("'"+element+"'");
				else
					sb.append(""+element);
			}
			if (i<(arrayLength-1)) sb.append(",");
		}
		tabs.decIndent(); sb.append("}");
		return sb.toString();
	}
	public static String getTabbedString(String o) {
		return tabs.getIndent()+o;
	}
	public static String getText(Object o) {
		if (o==null) return "";
		tabs.incIndent();
		StringBuffer sb = new StringBuffer();
		if (objectsPrintedPreviously.contains(o)) {
			sb.append(o.getClass().getName()+" "+"(Object Ref:"+o.hashCode()+" ... already printed...)");
		}
		else {
			sb.append("\n"+tabs.getIndent()+o.getClass().getName()+" "+"(Object Ref:"+o.hashCode()+")");
			tabs.incIndent();
			objectsPrintedPreviously.add(o);
			Method[] method = o.getClass().getMethods();
			if (method!=null){
				for(int i=0; i<method.length;i++) {
					if ((method[i].getName().startsWith("get")||method[i].getName().startsWith("is")) && !method[i].getName().startsWith("getClass"))
					{
						String attributeName;
						if(method[i].getName().startsWith("is"))
						{
						    attributeName = method[i].getName().substring(2);
						}
						else
						{
						    attributeName = method[i].getName().substring(3);
						}
						
						if ((attributeName!=null) && (attributeName.length()>0)) 
						{
							attributeName=(""+attributeName.charAt(0)).toLowerCase()+attributeName.substring(1);
							try {
								Object ob = method[i].invoke(o, null);
								if ( ob != null && ob.getClass().isArray() ) {
									sb.append("\n"+tabs.getIndent()+attributeName+"["+Array.getLength(ob)+"]="+arrayToString(ob));
								}
								else {
									sb.append("\n"+tabs.getIndent()+attributeName+"="+ob+"");
								}
							} catch (IllegalArgumentException e) {
								sb.append("IllegalArgumentException found:"+e.getMessage());
								logger.error(e);
							} catch (IllegalAccessException e) {
								sb.append("IllegalAccessException found:"+e.getMessage());
								logger.error(e);
							} catch (InvocationTargetException e) {
								sb.append("InvocationTargetException found:"+e.getMessage());
								logger.error(e);
							}	
						}	
					}
				}
			}
			sb.append("\n");
			objectsPrintedPreviously.remove(o);
			tabs.decIndent();
		}
		tabs.decIndent();
		return sb.toString();
	}
}
