/*
 * Created on Mar 7, 2006
 *
 */
package com.ibm.rpm.framework.util.stats;

import java.util.TreeSet;

import org.apache.commons.logging.Log;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.StringUtil;

/**
 * @author karim
 *
 */
public class StatsUtil
{
    public static final String SAVE_OPERATION = "SAVE";
    public static final String DELETE_OPERATION = "DELETE";
    public static final String LOADFROMXPATH_OPERATION = "LOAD_FROM_XPATH";
    public static final String LOADFROMID_OPERATION = "LOAD_FROM_ID";
    public static final String LOADFROMARRAY_OPERATION = "LOAD_ARRAY_FROM_ID";
    public static final String WORKFLOW_START_OPERATION = "START_WORKFLOW";
    public static final String WORKFLOW_SUSPEND_OPERATION = "SUSPEND_WORKFLOW"; 
    public static final String WORKFLOW_RESUME_OPERATION = "RESUME_WORKFLOW"; 
    public static final String WORKFLOW_CANCEL_OPERATION = "CANCEL_WORKFLOW"; 
    public static final String WORKFLOW_ASSOCIATE_OPERATION = "ASSOCIATE_WORKFLOW"; 
    public static final String CHECKOUT_OPERATION = "CHECKOUT";    
    public static final String FORCE_CHECKIN_OPERATION = "FORCE_CHECKIN";    
    public static final String CHECKIN_OPERATION = "CHECKIN";    
    
    private static TreeSet SUPPORTED_OPERATION_TYPES = new TreeSet();
    static{
        SUPPORTED_OPERATION_TYPES.add(SAVE_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(DELETE_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(LOADFROMXPATH_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(LOADFROMID_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(LOADFROMARRAY_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(WORKFLOW_START_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(WORKFLOW_SUSPEND_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(WORKFLOW_RESUME_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(WORKFLOW_CANCEL_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(WORKFLOW_ASSOCIATE_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(CHECKOUT_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(FORCE_CHECKIN_OPERATION);
        SUPPORTED_OPERATION_TYPES.add(CHECKIN_OPERATION);
    }
    
    public static long initOperation(MessageContext messageContext, String name, String description)
    {
        //create only supported operations
        if(SUPPORTED_OPERATION_TYPES.contains(name))
        {
            Operation operation = new Operation();
            operation.setName(name);
            operation.setDescription(description);
            messageContext.addStatsOperation(operation);
        }
        return System.currentTimeMillis();
    }
   
    public static long initOperation(MessageContext messageContext, String name, RPMObject rpmObject)
    {
        //create only supported operations
        if(SUPPORTED_OPERATION_TYPES.contains(name))
        {
            Operation operation = new Operation();
            operation.setName(name);
            if(rpmObject!=null)
            {
                String containerName = StringUtil.getShortClassName(rpmObject.getClass());
                String id = (rpmObject.getID()==null)?"":"[ID='"+rpmObject.getID()+"']";
                operation.setDescription(containerName+id);   
            }
            messageContext.addStatsOperation(operation);
        }
        return System.currentTimeMillis();
    }
    
    public static void endOperation(MessageContext messageContext, 
            long operationStartTime, Log logger)
    {
        long operationEndTime = System.currentTimeMillis();
        setTotalDuration(messageContext, operationEndTime-operationStartTime);
        displayOperation(messageContext, logger);
    }
    
    public static void updateSPDuration(MessageContext messageContext, long duration)
    {
        Operation operation = messageContext.getStatsOperation();
        if(operation!=null)
        {
            operation.addSPDuration(duration);   
        }
    }
    
    public static void setSelectDuration(MessageContext messageContext, long duration)
    {
        Operation operation = messageContext.getStatsOperation();
        if(operation!=null)
        {
            operation.setSelectDuration(duration);   
        }
    }
    
    public static void setPreloadDuration(MessageContext messageContext, long duration)
    {
        Operation operation = messageContext.getStatsOperation();
        if(operation!=null)
        {
            operation.setPreloadDuration(duration);   
        }
    }
    
    public static void setReloadDuration(MessageContext messageContext, long duration)
    {
        Operation operation = messageContext.getStatsOperation();
        if(operation!=null)
        {
            operation.setReloadDuration(duration);   
        }
    }
    
    public static void setTotalDuration(MessageContext messageContext, long duration)
    {
        Operation operation = messageContext.getStatsOperation();
        if(operation!=null)
        {
            operation.setTotalDuration(duration);   
        }
    }
    
    public static boolean isRegisterSelectDuration(MessageContext messageContext)
    {
        Operation operation = messageContext.getStatsOperation();
        String registerSelectDuration = messageContext.getSelectDurationFlag();
        if(operation!=null && registerSelectDuration==null)
        {
            messageContext.addSelectDurationFlag();
            return true;
        }
        return false;
    }

    
    public static void updateCreateConnectionDuration(MessageContext messageContext, long duration)
    {
        Operation operation = messageContext.getStatsOperation();
        if(operation!=null)
        {
            operation.addCreateConnectionDuration(duration);   
        }
    }
    public static void updateReleaseConnectionDuration(MessageContext messageContext, long duration)
    {
        Operation operation = messageContext.getStatsOperation();
        if(operation!=null)
        {
            operation.addReleaseConnectionDuration(duration);   
        }
    }
    
    /**
     * @param messageContext
     * @param logger
     */
    public static void displayOperation(MessageContext messageContext, Log logger)
    {
        Operation operation = messageContext.getStatsOperation(); 
        if(operation!=null)
        {
            String description = (operation.getDescription()==null)?"":" : "+operation.getDescription();
            
            StringBuffer log = new StringBuffer("\n -------------------------------------------------------------------------------------\n");
            
            log.append("Operation "+operation.getName() + description + "\n");

            log.append("	CREATE CONNECTION Execution time = "+operation.getCreateConnectionDuration()+" (ms)\n");
            
            if(isSaveOperation(operation))
            {
                log.append("	PRELOAD Execution time = "+operation.getPreloadDuration()+" (ms)\n");
                log.append("	SP Execution time = "+operation.getSpDuration()+" (ms)\n");
                log.append("	RELOAD Execution time = "+operation.getReloadDuration()+" (ms)\n");                
                log.append("	PRELOAD Object Count = "+operation.getDeltaLoadObjectCount()+"\n");                
            }
            else if (isLoadOperation(operation))
            {
                log.append("	SELECT Execution time = "+operation.getSelectDuration()+" (ms)\n");
                log.append("\tCache Hits = "+messageContext.getCache().getCacheHitCount()+"\n");
                log.append("\tCache Miss = "+messageContext.getCache().getCacheMissCount()+"\n");
            }
            else if (isDeleteOperation(operation))
            {
                log.append("	SP Execution time = "+operation.getSpDuration()+" (ms)\n");
            }
            else if (isWorkFlowOperation(operation))
            {
                log.append("	SP Execution time = "+operation.getSpDuration()+" (ms)\n");
            }

            log.append("	LOAD Object Count = "+operation.getLoadedObjectCount()+"\n");                
            log.append("	Database Query Count = "+operation.getSelectStatementCount()+"\n");                
            log.append("	Stored Procedure Count = "+operation.getStoredProcedureCount()+"\n");                

            log.append("	Total Execution time = "+operation.getTotalDuration()+" (ms)\n");
            log.append(" -------------------------------------------------------------------------------------\n");        
            logger.debug(log);        
        }
    }
    
    private static boolean isSaveOperation(Operation operation)
    {
        return isSameOperation(operation, SAVE_OPERATION);
    }
    
    private static boolean isLoadOperation(Operation operation)
    {
        return isSameOperation(operation, LOADFROMXPATH_OPERATION);
    }
    
    private static boolean isDeleteOperation(Operation operation)
    {
        return isSameOperation(operation, DELETE_OPERATION);
    }
    
    private static boolean isWorkFlowOperation(Operation operation)
    {
        return isSameOperation(operation, WORKFLOW_START_OPERATION)
                || isSameOperation(operation, WORKFLOW_SUSPEND_OPERATION)
                || isSameOperation(operation, WORKFLOW_RESUME_OPERATION)
                || isSameOperation(operation, WORKFLOW_CANCEL_OPERATION)
                || isSameOperation(operation, WORKFLOW_ASSOCIATE_OPERATION);
    }
    
    public static boolean isCheckoutOperation(Operation operation)
    {
        return isSameOperation(operation, CHECKOUT_OPERATION);
    }
    
    public static boolean isForceCheckinOperation(Operation operation)
    {
        return isSameOperation(operation, FORCE_CHECKIN_OPERATION);
    }
    
    public static boolean isCheckinOperation(Operation operation)
    {
        return isSameOperation(operation, CHECKIN_OPERATION);
    }
    
    private static boolean isSameOperation(Operation operation, String name)
    {
        if(operation!=null)
        {
            return name.equals(operation.getName());
        }
        return false; 
    }
}
