package com.ibm.rpm.init;

import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.ContainerFieldInfoMap;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.Field;
import com.ibm.rpm.metadata.model.MetadataStore;

/**
 * This is a replacement class for PredefinedFieldsBizMetaInfo which was being
 * updated manually. To switch between implementations, update
 * RPMInitializer.init()
 * 
 * @author andreit
 * 
 */
public class ContainerChecker
{
    /**
     * This shared object is used by internal saves, implicitly.
     */
    private static ContainerFieldInfoMap map = ContainerFieldInfoMap
        .getInstance();

    private static MetadataStore store = MetadataStore.getInstance();

    private static Log logger = LogFactory.getLog(ContainerChecker.class);

    /**
     * Load the required information from the new metadata API and publish it by
     * respecting the old interface.
     * 
     */
    public static void init()
    {
        List containerList = store.getContainers();
        Iterator containerIterator = containerList.iterator();

        int containerCounter = 0;
        Container container = null;
        while (containerIterator.hasNext())
        {
            container = (Container) containerIterator.next();
            addContainer(container);
            containerCounter++;
        }

        logger.debug("Number of containers loaded from metadata "
                + containerCounter);
        logger.debug("--- Finished loading the metadata ---");
    }

    /**
     * Add each container, one by one
     * 
     * @param container
     */
    private static void addContainer(Container container)
    {

        List containerFields = container.getFields();
        Iterator containerFieldsIterator = containerFields.iterator();
        Method scopeGetterMethod = null;

        while (containerFieldsIterator.hasNext())
        {
            Field field = (Field) containerFieldsIterator.next();

            if (field.getName().equalsIgnoreCase("parent")
                    || !RPMObject.class.isAssignableFrom(field
                        .getJavaTypeNoArray()))
            {
                continue;
            }
            Field fieldScope = field.getScopeField();
            if (fieldScope != null)
            {
                scopeGetterMethod = fieldScope.getGetterMethod();
            }
            else
            {

                scopeGetterMethod = null;
            }

            boolean createAlwaysInScope = false;
            if(scopeGetterMethod == null)
            {
                createAlwaysInScope = true;
            }
            
            map.addFieldInfo(container.getJavaClass(), field.getName(), field
                .getGetterMethod(), scopeGetterMethod, field.getComposite() 
                .booleanValue(), createAlwaysInScope);
        }
    }

    public static void main(String[] args)
    {
        ContainerChecker.init();
    }
}
