package com.ibm.rpm.init;

import com.ibm.rpm.applicationadministration.containers.CalendarWeekDay;
import com.ibm.rpm.applicationadministration.types.CalendarExceptionType;
import com.ibm.rpm.applicationadministration.types.CalendarWeekDayType;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.applicationadministration.types.SelectType;
import com.ibm.rpm.communications.types.CommunicationsFolder;
import com.ibm.rpm.communications.types.WorkflowResponseState;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.customfield.types.CustomPortletCategoryGroupType;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.document.types.DocumentLinkingStatus;
import com.ibm.rpm.document.types.ElementSecurityType;
import com.ibm.rpm.financial.types.BudgetType;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.financial.types.RevenueRecognitionMethodType;
import com.ibm.rpm.financial.types.TimePhasedType;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.types.DateType;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.framework.util.EnumeratedTypeUtil;
import com.ibm.rpm.scopemanagement.types.PrecisionType;
import com.ibm.rpm.scorecard.types.ProbabilityType;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ConstraintType;
import com.ibm.rpm.wbs.types.DependencyType;
import com.ibm.rpm.wbs.types.LeadLagType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;
import com.ibm.rpm.wbs.types.ResourceType;
import com.ibm.rpm.wbs.types.ScopeElementSecurityType;

/**
 * @author gmeloche
 *
 */
public class EnumeratedTypeInitializer
{
    public static void init(){
        
        //ApplicationAdministration
        
        //CalendarWeekDayType not supported because it can take more than one value in the database
        addNotSupportedEnumeratedType(CalendarWeekDayType.class);
        
/*        addEnumeratedType(CalendarWeekDayType.class, CalendarWeekDayType.NonDefaultWorking, new Integer(6), true, null);
        addEnumeratedType(CalendarWeekDayType.class, CalendarWeekDayType.NonDefaultWorking, new Integer(4), false, null);        
        addEnumeratedType(CalendarWeekDayType.class, CalendarWeekDayType.NonWorking, new Integer(5), true, null);
        addEnumeratedType(CalendarWeekDayType.class, CalendarWeekDayType.NonWorking, new Integer(0), false, null);
        addEnumeratedType(CalendarWeekDayType.class, CalendarWeekDayType.NonWorking, new Integer(3), false, null);        
        addEnumeratedType(CalendarWeekDayType.class, CalendarWeekDayType.UseDefault, new Integer(1), true, null);
*/       
        addEnumeratedType(CalendarExceptionType.class, CalendarExceptionType.NonWorking, new Integer(3), true, null);
        addEnumeratedType(CalendarExceptionType.class, CalendarExceptionType.NonDefaultWorking, new Integer(4), true, null);        
        
        addEnumeratedType(InputType.class, InputType.None, new Integer(0), true, null);
        addEnumeratedType(InputType.class, InputType.Text, new Integer(1), true, null);
        addEnumeratedType(InputType.class, InputType.Date, new Integer(2), true, null);
        addEnumeratedType(InputType.class, InputType.Numeric, new Integer(3), true, null);
        addEnumeratedType(InputType.class, InputType.RiskTolerance, new Integer(4), true, null);
        addEnumeratedType(InputType.class, InputType.RiskAppetite, new Integer(5), true, null);
        
        addEnumeratedType(SelectType.class, SelectType.Multi, new Integer(0), true, null);
        addEnumeratedType(SelectType.class, SelectType.Single, new Integer(1), true, null);
                
        //CustomField
        addEnumeratedType(FieldInputType.class, FieldInputType.Text, new Integer(0), true, null);
        addEnumeratedType(FieldInputType.class, FieldInputType.Currency, new Integer(1), true, null);
        addEnumeratedType(FieldInputType.class, FieldInputType.Date, new Integer(2), true, null);
        addEnumeratedType(FieldInputType.class, FieldInputType.Numeric, new Integer(3), true, null);
        addEnumeratedType(FieldInputType.class, FieldInputType.Datafield, new Integer(4), true, null);
        addEnumeratedType(FieldInputType.class, FieldInputType.Checkbox, new Integer(5), true, null);
        
        //CustomPortlet
        addEnumeratedType(CustomPortletCategoryGroupType.class, CustomPortletCategoryGroupType.AssetGroup, new Integer(9), true, null);
        addEnumeratedType(CustomPortletCategoryGroupType.class, CustomPortletCategoryGroupType.ClientGroup, new Integer(11), true, null);
        addEnumeratedType(CustomPortletCategoryGroupType.class, CustomPortletCategoryGroupType.DeliverableGroup, new Integer(2), true, null);
        addEnumeratedType(CustomPortletCategoryGroupType.class, CustomPortletCategoryGroupType.DocumentGroup, new Integer(14), true, null);
        addEnumeratedType(CustomPortletCategoryGroupType.class, CustomPortletCategoryGroupType.ProjectGroup, new Integer(1), true, null);
        addEnumeratedType(CustomPortletCategoryGroupType.class, CustomPortletCategoryGroupType.ResourceGroup, new Integer(13), true, null);
        addEnumeratedType(CustomPortletCategoryGroupType.class, CustomPortletCategoryGroupType.ScopeGroup, new Integer(4), true, null);
        addEnumeratedType(CustomPortletCategoryGroupType.class, CustomPortletCategoryGroupType.TaskGroup, new Integer(3), true, null);
     
        //Document
        addEnumeratedType(ElementSecurityType.class, ElementSecurityType.ByInvitation, "Y", true, null);
        addEnumeratedType(ElementSecurityType.class, ElementSecurityType.ProjectLevel, "N", true, null);
        addEnumeratedType(ElementSecurityType.class, ElementSecurityType.Unrestricted, "U", true, null);
        
        addEnumeratedType(PublishedType.class, PublishedType.None, "N", true, null);
        addEnumeratedType(PublishedType.class, PublishedType.ToTemplate, "Y", true, null);
        
        addEnumeratedType(DocumentLinkingStatus.class, DocumentLinkingStatus.IsLink, "Y", true, null);
        addEnumeratedType(DocumentLinkingStatus.class, DocumentLinkingStatus.IsNotReferenced, "N", true, null);
        addEnumeratedType(DocumentLinkingStatus.class, DocumentLinkingStatus.IsReferenced, "P", true, null);
        
        //TODO Expense
        //mapEnumeratedType(InvoiceStatus.class, InvoiceStatus.Committed, ..., true, null);
        //mapEnumeratedType(InvoiceStatus.class, InvoiceStatus.Incurred, ..., true, null);
        
        //Financial
        
        //BudgetType not supported because it can take more than one value in the database depending in the parent class
        addNotSupportedEnumeratedType(BudgetType.class);
        
 /*       addEnumeratedType(BudgetType.class, BudgetType.Actual, new Integer(1), true, WbsFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.Estimate, new Integer(2), true, WbsFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.Baseline, new Integer(8), true, WbsFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.BaselineInitial, new Integer(10), true, WbsFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.BaselinePrevious, new Integer(9), true, WbsFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.Billed, new Integer(13), true, WbsFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.GL, new Integer(15), true, WbsFinancial.class);

        addEnumeratedType(BudgetType.class, BudgetType.Actual, new Integer(4), true, WorksheetFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.Estimate, new Integer(3), true, WorksheetFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.Baseline, new Integer(5), true, WorksheetFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.BaselineInitial, new Integer(7), true, WorksheetFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.BaselinePrevious, new Integer(6), true, WorksheetFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.Billed, new Integer(14), true, WorksheetFinancial.class);
        addEnumeratedType(BudgetType.class, BudgetType.GL, new Integer(16), true, WorksheetFinancial.class);      
*/        
        addEnumeratedType(FinancialCategory.class, FinancialCategory.Benefit, new Integer(2), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.BenefitAverageCost, new Integer(32), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.BenefitFTE, new Integer(33), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.BenefitHour, new Integer(31), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.BenefitLabor, new Integer(34), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.Capital, new Integer(1), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.CapitalAverageCost, new Integer(22), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.CapitalFTE, new Integer(23), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.CapitalHour, new Integer(21), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.CapitalLabor, new Integer(24), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.Contingency, new Integer(4), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.Elimination, new Integer(3), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.Expense, new Integer(0), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.ExpenseAverageCost, new Integer(12), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.ExpenseFTE, new Integer(13), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.ExpenseHour, new Integer(11), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.ExpenseLabor, new Integer(14), true, null);
        addEnumeratedType(FinancialCategory.class, FinancialCategory.TimePhaseCalendar, new Integer(10), true, null);

        //TODO validate RevenueRecognitionMethodType.NA = 0
        //addEnumeratedType(RevenueRecognitionMethodType.class, RevenueRecognitionMethodType.NA, new Integer(0), true, null);
        addEnumeratedType(RevenueRecognitionMethodType.class, RevenueRecognitionMethodType.Manual, new Integer(0), true, null);
        addEnumeratedType(RevenueRecognitionMethodType.class, RevenueRecognitionMethodType.Ratio, new Integer(1), true, null);
        addEnumeratedType(RevenueRecognitionMethodType.class, RevenueRecognitionMethodType.FlowThru, new Integer(2), true, null);
    
        addEnumeratedType(TimePhasedType.class, TimePhasedType.NotTimephased, new Integer(0), true, null);
        addEnumeratedType(TimePhasedType.class, TimePhasedType.ProjectAssignment, new Integer(1), true, null);
        addEnumeratedType(TimePhasedType.class, TimePhasedType.AutoTimephased, new Integer(2), true, null);
        addEnumeratedType(TimePhasedType.class, TimePhasedType.ManuallyTimephased, new Integer(3), true, null);
        
        //framework
        
        //TODO map DateType (used in TaskAssignment)
        addEnumeratedType(DateType.class, DateType.Actual, null, true, null);
        addEnumeratedType(DateType.class, DateType.Forecasted, null, true, null);
        addEnumeratedType(DateType.class, DateType.Planned, null, true, null);
        addEnumeratedType(DateType.class, DateType.Proposed, null, true, null);

        //scopemanagement
        addEnumeratedType(PrecisionType.class, PrecisionType.Low, new Integer(0), true, null);
        addEnumeratedType(PrecisionType.class, PrecisionType.Medium, new Integer(1), true, null);
        addEnumeratedType(PrecisionType.class, PrecisionType.High, new Integer(2), true, null);
        
        addEnumeratedType(ScopeElementSecurityType.class, ScopeElementSecurityType.ByInvitation, "Y", true, null);
        addEnumeratedType(ScopeElementSecurityType.class, ScopeElementSecurityType.ProjectLevel, "N", true, null);        
        
        //TODO scorecard
        addEnumeratedType(ProbabilityType.class, ProbabilityType.None, new Integer(0), true, null);
        addEnumeratedType(ProbabilityType.class, ProbabilityType.ProjectBenefits, new Integer(1), true, null);
        addEnumeratedType(ProbabilityType.class, ProbabilityType.ProjectCosts, new Integer(2), true, null);
        addEnumeratedType(ProbabilityType.class, ProbabilityType.CustomerSatisfaction, new Integer(3), true, null);
        addEnumeratedType(ProbabilityType.class, ProbabilityType.ProjectSchedule, new Integer(4), true, null);
        addEnumeratedType(ProbabilityType.class, ProbabilityType.ProjectRisk, new Integer(5), true, null);

        //TODO security
        
        // wbs
        addEnumeratedType(AssignmentType.class, AssignmentType.Duration, new Integer(1), true, null);
        addEnumeratedType(AssignmentType.class, AssignmentType.Work, new Integer(0), true, null);

        addEnumeratedType(ConstraintType.class, ConstraintType.AsSoonAsPossible, new Integer(0), true, null);
        addEnumeratedType(ConstraintType.class, ConstraintType.AsLateAsPossible, new Integer(1), true, null);
        addEnumeratedType(ConstraintType.class, ConstraintType.FinishNoEarlierThan, new Integer(2), true, null);
        addEnumeratedType(ConstraintType.class, ConstraintType.FinishNoLaterThan, new Integer(3), true, null);
        addEnumeratedType(ConstraintType.class, ConstraintType.MustFinishOn, new Integer(4), true, null);
        addEnumeratedType(ConstraintType.class, ConstraintType.MustStartOn, new Integer(5), true, null);
        addEnumeratedType(ConstraintType.class, ConstraintType.StartNoEarlierThan, new Integer(6), true, null);
        addEnumeratedType(ConstraintType.class, ConstraintType.StartNoLaterThan, new Integer(7), true, null);

        addEnumeratedType(DependencyType.class, DependencyType.FinishStart, new Integer(0), true, null);
        addEnumeratedType(DependencyType.class, DependencyType.StartStart, new Integer(1), true, null);
        addEnumeratedType(DependencyType.class, DependencyType.FinishFinish, new Integer(2), true, null);
        addEnumeratedType(DependencyType.class, DependencyType.StartFinish, new Integer(3), true, null);
        
        addEnumeratedType(LeadLagType.class, LeadLagType.Lead, new Integer(1), true, null);
        addEnumeratedType(LeadLagType.class, LeadLagType.Lag, new Integer(0), true, null);

        addEnumeratedType(ProjectPublishedType.class, ProjectPublishedType.None, new Integer(0), true, null);
        addEnumeratedType(ProjectPublishedType.class, ProjectPublishedType.ToTeam, new Integer(1), true, null);
        addEnumeratedType(ProjectPublishedType.class, ProjectPublishedType.ToTemplate, new Integer(2), true, null);
        addEnumeratedType(ProjectPublishedType.class, ProjectPublishedType.ToPartialTemplate, new Integer(3), true, null);
        
 
        addEnumeratedType(ResourceType.class, ResourceType.Resource, new Integer(0), true, null);
        addEnumeratedType(ResourceType.class, ResourceType.Profile, new Integer(1), true, null);

        addEnumeratedType(TimesheetStatus.class, TimesheetStatus.Submitted, new Integer(12), true, null);
        addEnumeratedType(TimesheetStatus.class, TimesheetStatus.Approved, new Integer(1), true, null);
        addEnumeratedType(TimesheetStatus.class, TimesheetStatus.Rejected, new Integer(2), true, null);
        addEnumeratedType(TimesheetStatus.class, TimesheetStatus.ApprovedByProjectManager, new Integer(5), true, null);
        addEnumeratedType(TimesheetStatus.class, TimesheetStatus.ApprovedByResourceManager, new Integer(9), true, null);
        addEnumeratedType(TimesheetStatus.class, TimesheetStatus.RejectedByProjectManager, new Integer(14), true, null);
        addEnumeratedType(TimesheetStatus.class, TimesheetStatus.RejectedByResourceManager, new Integer(18), true, null);
        
        // Communications
        
        addEnumeratedType(CommunicationsFolder.class, CommunicationsFolder.Inbox, new Integer(31), true, null);
        addEnumeratedType(CommunicationsFolder.class, CommunicationsFolder.Sent, new Integer(0), true, null);
        addEnumeratedType(CommunicationsFolder.class, CommunicationsFolder.Archived, new Integer(1), true, null);
        addEnumeratedType(CommunicationsFolder.class, CommunicationsFolder.Archived, new Integer(32), false, null);
        addEnumeratedType(CommunicationsFolder.class, CommunicationsFolder.Deleted, new Integer(2), true, null);
        addEnumeratedType(CommunicationsFolder.class, CommunicationsFolder.Deleted, new Integer(33), false, null);
        addEnumeratedType(CommunicationsFolder.class, CommunicationsFolder.NotVisible, new Integer(3), true, null);
        addEnumeratedType(CommunicationsFolder.class, CommunicationsFolder.NotVisible, new Integer(34), false, null);
        
        addEnumeratedType(WorkflowState.class, WorkflowState.Initial, null, true, null);
        addEnumeratedType(WorkflowState.class, WorkflowState.Open, new Integer(0), true, null);
        addEnumeratedType(WorkflowState.class, WorkflowState.Closed, new Integer(1), true, null);
        addEnumeratedType(WorkflowState.class, WorkflowState.Suspended, new Integer(2), true, null);
        addEnumeratedType(WorkflowState.class, WorkflowState.Hanged, new Integer(3), true, null);
        addEnumeratedType(WorkflowState.class, WorkflowState.Canceled, new Integer(7), true, null);
        
        addEnumeratedType(WorkflowResponseState.class, WorkflowResponseState.NotComplete, new Integer(0), true, null);
        addEnumeratedType(WorkflowResponseState.class, WorkflowResponseState.Accepted, new Integer(1), true, null);
        addEnumeratedType(WorkflowResponseState.class, WorkflowResponseState.Rejected, new Integer(2), true, null);
    }
    
    /**
     * @param enumeratedTypeClass (e.g. TimesheetStatus.class)
     * @param type ( e.g. TimesheetStatus.Submitted)
     * @param value
     *            Database Value that matches the type.
     * @param isCreateValue
     *            true if this value is used on creation. false if the value can
     *            be found in the database but cannot be set by the api. (NOTE:
     *            These additionnal values are not support in xpath queries.)
     * @param specificContainer
     *            specify only if the type can be mapped to different database values
     *            depending on the owner of the EnumeratedType.
     */
    private static void addEnumeratedType(Class enumeratedTypeClass, Object type, Object value, boolean isCreateValue, Class specificContainer){
        
        EnumeratedTypeUtil.getInstance().addEnumeratedType(enumeratedTypeClass, type, value, isCreateValue, specificContainer);
    }
    
    /**
     * If an EnumeratedType class is mapped as "Not Supported", an exception
     * will be returned if a user tries to do an xpath on a field of this type.
     */
    private static void addNotSupportedEnumeratedType(Class enumeratedTypeClass)
    {

        EnumeratedTypeUtil.getInstance().addEnumeratedType(enumeratedTypeClass,
            null, null, true, null);
    }
    
    //Method used for internal testing
    public static void main(String[] args) throws RPMException
    {

        init();
        
        Integer value = (Integer)EnumeratedTypeUtil.getInstance().getValue(CalendarWeekDay.class, CalendarWeekDayType.class, CalendarWeekDayType.NonWorking.toString());
        System.out.println(value);
        
        CalendarWeekDayType type = (CalendarWeekDayType)EnumeratedTypeUtil.getInstance().getType(CalendarWeekDay.class, CalendarWeekDayType.class, new Integer(3));
        System.out.println(type);
    
        //Check exceptions
        EnumeratedTypeUtil.getInstance().getValue(CalendarWeekDay.class, InputType.class, InputType.Date.toString());
        EnumeratedTypeUtil.getInstance().getType(CalendarWeekDay.class, InputType.class, new Integer(4));
        EnumeratedTypeUtil.getInstance().getType(CalendarWeekDay.class, CalendarWeekDayType.class,
            new Integer(4));

    }
}
