package com.ibm.rpm.init;

import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;
import java.util.TimeZone;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xalan.Version;

import com.ibm.rpm.applicationadministration.containers.*;
import com.ibm.rpm.applicationadministration.managers.AdministrationModuleManager;
import com.ibm.rpm.applicationadministration.managers.AttributeAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.AttributeManager;
import com.ibm.rpm.applicationadministration.managers.CalendarExceptionManager;
import com.ibm.rpm.applicationadministration.managers.ContactGroupAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.ContactGroupsManager;
import com.ibm.rpm.applicationadministration.managers.CurrencyExchangeManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.applicationadministration.managers.GeographicalAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.GeographicalManager;
import com.ibm.rpm.applicationadministration.managers.OrganizationalAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.OrganizationalManager;
import com.ibm.rpm.applicationadministration.managers.PoolDomainsManager;
import com.ibm.rpm.applicationadministration.managers.RPMCalendarManager;
import com.ibm.rpm.applicationadministration.managers.RtfAssignmentManager;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.AssetFolder;
import com.ibm.rpm.asset.containers.AssetModule;
import com.ibm.rpm.asset.containers.AssetOrganizationalAssignment;
import com.ibm.rpm.asset.containers.AssetResourceAssignment;
import com.ibm.rpm.asset.containers.CrossChargeFinancials;
import com.ibm.rpm.asset.containers.DepreciationSchedule;
import com.ibm.rpm.asset.containers.GenericAsset;
import com.ibm.rpm.asset.containers.MaintenanceEstimates;
import com.ibm.rpm.asset.managers.AssetCrossChargeManager;
import com.ibm.rpm.asset.managers.AssetDepreciationManager;
import com.ibm.rpm.asset.managers.AssetMaintenanceManager;
import com.ibm.rpm.asset.managers.AssetManager;
import com.ibm.rpm.asset.managers.AssetModuleManager;
import com.ibm.rpm.asset.managers.AssetOrganizationsManager;
import com.ibm.rpm.asset.managers.AssetResourcesManager;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenterAssignment;
import com.ibm.rpm.clientcostcenters.containers.ClientFolder;
import com.ibm.rpm.clientcostcenters.containers.ClientModule;
import com.ibm.rpm.clientcostcenters.containers.Division;
import com.ibm.rpm.clientcostcenters.containers.GenericClient;
import com.ibm.rpm.clientcostcenters.managers.ClientManager;
import com.ibm.rpm.clientcostcenters.managers.ClientModuleManager;
import com.ibm.rpm.communications.containers.ChecklistItem;
import com.ibm.rpm.communications.containers.GenericCommunication;
import com.ibm.rpm.communications.containers.GenericResponseType;
import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.VoteResponse;
import com.ibm.rpm.communications.containers.WorkflowAttachment;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.containers.WorkflowStepNotification;
import com.ibm.rpm.communications.managers.CommunicationManager;
import com.ibm.rpm.communications.managers.WorkflowAttachmentManager;
import com.ibm.rpm.communications.managers.WorkflowResponseChecklistManager;
import com.ibm.rpm.communications.managers.WorkflowResponseManager;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.containers.ContractChargeSchedule;
import com.ibm.rpm.contract.containers.ContractFolder;
import com.ibm.rpm.contract.containers.ContractModule;
import com.ibm.rpm.contract.containers.GenericContract;
import com.ibm.rpm.contract.managers.ContractChargeScheduleManager;
import com.ibm.rpm.contract.managers.ContractManager;
import com.ibm.rpm.contract.managers.ContractModuleManager;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.containers.CustomPortlet;
import com.ibm.rpm.customfield.containers.CustomPortletCategory;
import com.ibm.rpm.customfield.containers.GenericCustomField;
import com.ibm.rpm.customfield.containers.GenericCustomPortlet;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentManager;
import com.ibm.rpm.customfield.managers.CustomFieldManager;
import com.ibm.rpm.customfield.managers.CustomPortletManager;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentBlobDetails;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.DocumentModule;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.containers.NoteMinutes;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.document.managers.DocumentBlobDetailManager;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.document.managers.DocumentModuleManager;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.FinancialModule;
import com.ibm.rpm.financial.containers.GenericChargeCode;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.containers.TimeCodeCategory;
import com.ibm.rpm.financial.containers.TimeCodeGroup;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.managers.ChargeCodeManager;
import com.ibm.rpm.financial.managers.FinancialModuleManager;
import com.ibm.rpm.financial.managers.ProjectReconciliationManager;
import com.ibm.rpm.financial.managers.ProjectReconciliationStatusManager;
import com.ibm.rpm.financial.managers.TimeCodeAssignmentManager;
import com.ibm.rpm.financial.managers.TimePhasedFinancialManager;
import com.ibm.rpm.financial.managers.TimecodeManager;
import com.ibm.rpm.financial.managers.WbsFinancialManager;
import com.ibm.rpm.financial.managers.WorksheetFinancialManager;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.containers.StatusUpdate;
import com.ibm.rpm.framework.util.ContainerToCheckpointMap;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.MetadataStore;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.resource.containers.EmploymentHistory;
import com.ibm.rpm.resource.containers.IndustryExperience;
import com.ibm.rpm.resource.containers.LanguageProficiency;
import com.ibm.rpm.resource.containers.LocationPreferences;
import com.ibm.rpm.resource.containers.Passport;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolModule;
import com.ibm.rpm.resource.containers.PoolReference;
import com.ibm.rpm.resource.containers.ProjectHistory;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttribute;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetency;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceAttributeProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceAttributeSkill;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceCostCenterAssignment;
import com.ibm.rpm.resource.containers.ResourceModule;
import com.ibm.rpm.resource.containers.ResourceRates;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.resource.managers.PoolManager;
import com.ibm.rpm.resource.managers.PoolModuleManager;
import com.ibm.rpm.resource.managers.PoolReferenceManager;
import com.ibm.rpm.resource.managers.ResourceAttributeAssignmentManager;
import com.ibm.rpm.resource.managers.ResourceAttributeManager;
import com.ibm.rpm.resource.managers.ResourceCostCenterManager;
import com.ibm.rpm.resource.managers.ResourceEducationManager;
import com.ibm.rpm.resource.managers.ResourceEmploymentHistoryManager;
import com.ibm.rpm.resource.managers.ResourceIndustryExperienceManager;
import com.ibm.rpm.resource.managers.ResourceLanguageProficiencyManager;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.resource.managers.ResourceModuleManager;
import com.ibm.rpm.resource.managers.ResourceProjectHistoryManager;
import com.ibm.rpm.resource.managers.ResourceRatesManager;
import com.ibm.rpm.resource.managers.ResourceRoleAssignmentManager;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.AggregateScope;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.MitigationFactor;
import com.ibm.rpm.scopemanagement.containers.MitigationTrigger;
import com.ibm.rpm.scopemanagement.containers.ReqProConnection;
import com.ibm.rpm.scopemanagement.containers.ReqProImportStatus;
import com.ibm.rpm.scopemanagement.containers.ReqProIntegrationFolder;
import com.ibm.rpm.scopemanagement.containers.ReqProRequirement;
import com.ibm.rpm.scopemanagement.containers.ReqProScopeFolder;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.RiskMatrix;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeScheduleDate;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ScopeWorkProduct;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.managers.MitigationFactorManager;
import com.ibm.rpm.scopemanagement.managers.ReqProManager;
import com.ibm.rpm.scopemanagement.managers.ScopeManagementModuleManager;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scopemanagement.managers.ScopeScheduleDateManager;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.AssignedScorecardCategory;
import com.ibm.rpm.scorecard.containers.AssignedScorecardQuestion;
import com.ibm.rpm.scorecard.containers.AssignedScorecardResponse;
import com.ibm.rpm.scorecard.containers.AssignedScorecardState;
import com.ibm.rpm.scorecard.containers.ResourceAssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.containers.ScorecardQuestion;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;
import com.ibm.rpm.scorecard.managers.PublishedScorecardManager;
import com.ibm.rpm.scorecard.managers.ScorecardAssignmentManager;
import com.ibm.rpm.scorecard.managers.ScorecardManager;
import com.ibm.rpm.security.containers.ActionItemSecurityRole;
import com.ibm.rpm.security.containers.ChangeRequestSecurityRole;
import com.ibm.rpm.security.containers.DefectSecurityRole;
import com.ibm.rpm.security.containers.DeliverableSecurityRole;
import com.ibm.rpm.security.containers.DocumentSecurityRole;
import com.ibm.rpm.security.containers.FolderSecurityRole;
import com.ibm.rpm.security.containers.GenericSecurityRole;
import com.ibm.rpm.security.containers.IssueSecurityRole;
import com.ibm.rpm.security.containers.NotesMinutesSecurityRole;
import com.ibm.rpm.security.containers.PoolSecurityRole;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.containers.RequirementSecurityRole;
import com.ibm.rpm.security.containers.RiskSecurityRole;
import com.ibm.rpm.security.containers.SecurityGroup;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.containers.SecurityRoleCategory;
import com.ibm.rpm.security.containers.ServiceRequestSecurityRole;
import com.ibm.rpm.security.containers.TaskSecurityRole;
import com.ibm.rpm.security.containers.TemplateFolderSecurityRole;
import com.ibm.rpm.security.managers.SecurityGroupManager;
import com.ibm.rpm.security.managers.SecurityRoleManager;
import com.ibm.rpm.timesheet.containers.AdministrativeProject;
import com.ibm.rpm.timesheet.containers.AdministrativeTask;
import com.ibm.rpm.timesheet.containers.AdministrativeTaskAssignment;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.DefaultAdministrativeTask;
import com.ibm.rpm.timesheet.containers.DefaultStep;
import com.ibm.rpm.timesheet.containers.GenericStep;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.timesheet.containers.GenericTimesheetElement;
import com.ibm.rpm.timesheet.containers.PersonalProject;
import com.ibm.rpm.timesheet.containers.PersonalTask;
import com.ibm.rpm.timesheet.containers.PersonalTaskAssignment;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.managers.AdministrativeTaskAssignmentManager;
import com.ibm.rpm.timesheet.managers.DefaultAdministrativeTaskManager;
import com.ibm.rpm.timesheet.managers.DefaultStepManager;
import com.ibm.rpm.timesheet.managers.GenericTaskAssignmentManager;
import com.ibm.rpm.timesheet.managers.PersonalTaskAssignmentManager;
import com.ibm.rpm.timesheet.managers.StepManager;
import com.ibm.rpm.timesheet.managers.StepTimesheetManager;
import com.ibm.rpm.timesheet.managers.SummaryTimesheetManager;
import com.ibm.rpm.timesheet.managers.TimesheetApprovalStatusManager;
import com.ibm.rpm.timesheet.managers.TimesheetTaskManager;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Profile;
import com.ibm.rpm.wbs.containers.ProfileCompetencyAssignment;
import com.ibm.rpm.wbs.containers.ProfileSkillAssignment;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ProjectOrganizationalAssignment;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.managers.DependenciesManager;
import com.ibm.rpm.wbs.managers.OpportunityManager;
import com.ibm.rpm.wbs.managers.PortfoliosManager;
import com.ibm.rpm.wbs.managers.ProfileCompetencyAssignmentManager;
import com.ibm.rpm.wbs.managers.ProfileManager;
import com.ibm.rpm.wbs.managers.ProfileSkillAssignmentManager;
import com.ibm.rpm.wbs.managers.ProjectOrganizationalAssignmentManager;
import com.ibm.rpm.wbs.managers.ResourceTaskAssignmentManager;
import com.ibm.rpm.wbs.managers.TaskAssignmentManager;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.managers.WbsModuleManager;
import com.ibm.rpm.wbs.managers.WbsScheduleDateManager;
import com.ibm.rpm.workflow.containers.AvailableWorkflowProcess;
import com.ibm.rpm.workflow.containers.ExternalAction;
import com.ibm.rpm.workflow.containers.ExternalActionCategory;
import com.ibm.rpm.workflow.containers.GenericWorkflow;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowCategory;
import com.ibm.rpm.workflow.containers.WorkflowGroup;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowRole;
import com.ibm.rpm.workflow.containers.WorkflowRoleMapping;
import com.ibm.rpm.workflow.managers.AvailableWorkflowProcessManager;
import com.ibm.rpm.workflow.managers.ExternalActionManager;
import com.ibm.rpm.workflow.managers.RunningWorkflowProcessManager;
import com.ibm.rpm.workflow.managers.WorkflowManager;
import com.ibm.rpm.workflow.managers.WorkflowRoleManager;
import com.ibm.rpm.workflow.managers.WorkflowRoleMappingManager;

/**
 * @author gmeloche
 */
public abstract class RPMInitializer
{
    private static Log logger = LogFactory.getLog(RPMInitializer.class);

    private static boolean isInitialized = false;

    public static void init()
    {
        if (isInitialized)
        {
            logger.debug("RPMInitializer.init() was called twice.");
        }
        else
        {
            isInitialized = true;

            printRPMVersion();
            printXalanVersion();

            initContainersBizMetaInfo();
            logger.debug("RPMInitializer.initContainersBizMetaInfo() done");
            EnumeratedTypeInitializer.init();
            logger.debug("EnumeratedTypeInitializer.init() done");
            ContainerChecker.init();
            //This call is left there to validate the mapping on start up
            FieldsBizMetaInfoInitializer.init();
            logger.debug("FieldsBizMetaInfoInitializer.init() done");
            //
            
            initCheckpointMap();
            logger.debug("initCheckpointMap() done");         
            
            loadManagers();
            logger.debug("RPMInitializer.loadManagers() done");
        }
    }

    /**
     * Initializes BUSINESS meta information about a container class. For now,
     * business meta-information includes scope, validation checkpoint and
     * context name. Note that implementation information is initialized in each
     * manager.
     */
    private static void initContainersBizMetaInfo()
    {
        // TODO: Include scope for each container.

        //Application administration
        initMetaInfo(AdministrationModule.class,
            ContextUtil.NONE);

        //Attributes
        initMetaInfo(GenericAttribute.class, ContextUtil.UMBRELLA_ATTRIBUTE_CONTEXT);
        initMetaInfo(AttributeCategory.class, ContextUtil.UMBRELLA_ATTRIBUTE_CONTEXT);
        initMetaInfo(AttributeClassification.class, ContextUtil.UMBRELLA_ATTRIBUTE_CONTEXT);
        initMetaInfo(Attribute.class, ContextUtil.UMBRELLA_ATTRIBUTE_CONTEXT);
        initMetaInfo(AttributeAssignment.class, ContextUtil.UMBRELLA_ATTRIBUTE_CONTEXT);

        //Rtfs
        initMetaInfo(DatafieldRTF.class, ContextUtil.NONE);
        initMetaInfo(ActionRTF.class, ContextUtil.NONE);
        initMetaInfo(AssetRTF.class, ContextUtil.NONE);
        initMetaInfo(ChangeRequestRTF.class, ContextUtil.NONE);
        initMetaInfo(ClientRTF.class, ContextUtil.NONE);
        initMetaInfo(CommonScopeRTF.class, ContextUtil.NONE);
        initMetaInfo(CommonWbsRTF.class, ContextUtil.NONE);
        initMetaInfo(DefectRTF.class, ContextUtil.NONE);
        initMetaInfo(DeliverableRTF.class, ContextUtil.NONE);
        initMetaInfo(IssueRTF.class, ContextUtil.NONE);
        initMetaInfo(MilestoneRTF.class, ContextUtil.NONE);
        initMetaInfo(NoteMinutesRTF.class, ContextUtil.NONE);
        initMetaInfo(ProjectRTF.class, ContextUtil.NONE);
        initMetaInfo(RequirementRTF.class, ContextUtil.NONE);
        initMetaInfo(RiskRTF.class, ContextUtil.NONE);
        initMetaInfo(ServiceRequestRTF.class, ContextUtil.NONE);
        initMetaInfo(SummaryTaskRTF.class, ContextUtil.NONE);
        initMetaInfo(TaskRTF.class, ContextUtil.NONE);
        initMetaInfo(WorkProductRTF.class, ContextUtil.NONE);

        initMetaInfo(RtfAssignment.class, ContextUtil.UMBRELLA_RTF_CONTEXT);

        //States
        initMetaInfo(ActionState.class, ContextUtil.NONE);
        initMetaInfo(AssetState.class, ContextUtil.NONE);
        initMetaInfo(ChangeRequestState.class, ContextUtil.NONE);
        initMetaInfo(ChargeCodeState.class, ContextUtil.NONE);
        initMetaInfo(CostCenterState.class, ContextUtil.NONE);
        initMetaInfo(DefectState.class, ContextUtil.NONE);
        initMetaInfo(DeliverableState.class, ContextUtil.NONE);
        initMetaInfo(DocumentState.class, ContextUtil.NONE);
        initMetaInfo(IssueState.class, ContextUtil.NONE);
        initMetaInfo(NoteState.class, ContextUtil.NONE);
        initMetaInfo(RequirementState.class, ContextUtil.NONE);
        initMetaInfo(RiskState.class, ContextUtil.NONE);
        initMetaInfo(ScorecardState.class, ContextUtil.NONE);
        initMetaInfo(ServiceRequestState.class, ContextUtil.NONE);
        initMetaInfo(State.class, ContextUtil.NONE);
        initMetaInfo(TaskState.class, ContextUtil.NONE);

        //Datafield (context not used)
        initMetaInfo(GenericDatafield.class, ContextUtil.NONE);
        initMetaInfo(DatafieldCategory.class, ContextUtil.NONE);
        initMetaInfo(CustomDatafieldCategory.class, ContextUtil.NONE);
        initMetaInfo(Datafield.class, ContextUtil.NONE);
        initMetaInfo(ValueDatafield.class, ContextUtil.NONE);
        initMetaInfo(CustomDatafield.class, ContextUtil.NONE);
        initMetaInfo(Currency.class, ContextUtil.NONE);

        initMetaInfo(AssetFinancialCategory.class, ContextUtil.NONE);
        initMetaInfo(AssetCategory.class, ContextUtil.NONE);
        initMetaInfo(AssetRevisionImpact.class, ContextUtil.NONE);
        initMetaInfo(AssetRevisionType.class, ContextUtil.NONE);
        initMetaInfo(AssignmentStatus.class, ContextUtil.NONE);
        initMetaInfo(BenefitDatafield1.class, ContextUtil.NONE);
        initMetaInfo(BenefitDatafield2.class, ContextUtil.NONE);
        initMetaInfo(BenefitDatafieldLabelText.class, ContextUtil.NONE);
        initMetaInfo(BenefitLabelText.class, ContextUtil.NONE);
        initMetaInfo(CapitalDatafield1.class, ContextUtil.NONE);
        initMetaInfo(CapitalDatafield2.class, ContextUtil.NONE);
        initMetaInfo(CapitalDatafieldLabelText.class, ContextUtil.NONE);
        initMetaInfo(CapitalLabelText.class, ContextUtil.NONE);
        initMetaInfo(ChargeType.class, ContextUtil.NONE);
        initMetaInfo(CitizenshipCountryCode.class, ContextUtil.NONE);
        initMetaInfo(Classification.class, ContextUtil.NONE);
        initMetaInfo(CompanyNameCode.class, ContextUtil.NONE);
        initMetaInfo(CompanySize.class, ContextUtil.NONE);
        initMetaInfo(ContractType.class, ContextUtil.NONE);
        initMetaInfo(DocumentRevisionType.class, ContextUtil.NONE);
        initMetaInfo(EmployeeStatus.class, ContextUtil.NONE);
        initMetaInfo(ExpenseCapitalCommonBenefitDatafield1.class, ContextUtil.NONE);
        initMetaInfo(ExpenseCapitalCommonBenefitDatafield2.class, ContextUtil.NONE);
        initMetaInfo(ExpenseCapitalCommonDatafieldLabelText.class, ContextUtil.NONE);
        initMetaInfo(ExpenseDatafield1.class, ContextUtil.NONE);
        initMetaInfo(ExpenseDatafield2.class, ContextUtil.NONE);
        initMetaInfo(ExpenseDatafieldLabelText.class, ContextUtil.NONE);
        initMetaInfo(ExpenseLabelText.class, ContextUtil.NONE);
        initMetaInfo(Experience.class, ContextUtil.NONE);
        initMetaInfo(IndustryType.class, ContextUtil.NONE);
        initMetaInfo(Language.class, ContextUtil.NONE);
        initMetaInfo(OrganizationType.class, ContextUtil.NONE);
        initMetaInfo(PaymentType.class, ContextUtil.NONE);
        initMetaInfo(PreferenceRank.class, ContextUtil.NONE);
        initMetaInfo(ProficiencyLevel.class, ContextUtil.NONE);
        initMetaInfo(ReconciliationReason.class, ContextUtil.NONE);
        initMetaInfo(RequirementRevisionType.class, ContextUtil.NONE);
        initMetaInfo(ResourceCustomFields.class, ContextUtil.NONE);
        initMetaInfo(ResourceRevisionType.class, ContextUtil.NONE);
        initMetaInfo(RevisionImpact.class, ContextUtil.NONE);
        initMetaInfo(ScheduleImpact.class, ContextUtil.NONE);
        initMetaInfo(ScopeElementsRevisionType.class, ContextUtil.NONE);
        initMetaInfo(ScorecardRevisionType.class, ContextUtil.NONE);
        initMetaInfo(WbsRevisionType.class, ContextUtil.NONE);
        initMetaInfo(WorkingStatus.class, ContextUtil.NONE);
        initMetaInfo(WorkLocation.class, ContextUtil.NONE);

        //Scorecards
        initMetaInfo(ResourceAssignedScorecard.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(AssignedScorecard.class, ContextUtil.UMBRELLA_SCORECARD_CONTEXT);
        initMetaInfo(AssignedScorecardState.class, ContextUtil.UMBRELLA_SCORECARD_CONTEXT);
        initMetaInfo(AssignedScorecardCategory.class,
            ContextUtil.UMBRELLA_SCORECARD_CONTEXT);
        initMetaInfo(AssignedScorecardQuestion.class,
            ContextUtil.UMBRELLA_SCORECARD_CONTEXT);
        initMetaInfo(AssignedScorecardResponse.class,
            ContextUtil.UMBRELLA_SCORECARD_CONTEXT);
        initMetaInfo(ScorecardFolder.class, ContextUtil.UMBRELLA_SCORECARD_CONTEXT);
        initMetaInfo(Scorecard.class, ContextUtil.UMBRELLA_SCORECARD_CONTEXT);
        initMetaInfo(ScorecardCategory.class, ContextUtil.UMBRELLA_SCORECARD_CONTEXT);
        initMetaInfo(ScorecardQuestion.class, ContextUtil.UMBRELLA_SCORECARD_CONTEXT);
        initMetaInfo(ScorecardResponse.class, ContextUtil.UMBRELLA_SCORECARD_CONTEXT);

        //Geographical
        initMetaInfo(Geographical.class, ContextUtil.NONE);
        initMetaInfo(Region.class, ContextUtil.NONE);
        initMetaInfo(Country.class, ContextUtil.NONE);
        initMetaInfo(CountryState.class, ContextUtil.NONE);
        initMetaInfo(City.class, ContextUtil.NONE);
        initMetaInfo(Location.class, ContextUtil.NONE);
        initMetaInfo(GeographicalAssignment.class, ContextUtil.NONE);

        //Organizational
        initMetaInfo(Organizational.class, ContextUtil.NONE);
        initMetaInfo(Organization.class, ContextUtil.NONE);
        initMetaInfo(OrganizationalAssignment.class, ContextUtil.NONE);
        initMetaInfo(Department.class, ContextUtil.NONE);
        initMetaInfo(Group.class, ContextUtil.NONE);
        initMetaInfo(Office.class, ContextUtil.NONE);

        //Other
        initMetaInfo(ArchivedProject.class, ContextUtil.NONE);
        initMetaInfo(RPMCalendar.class, ContextUtil.NONE);
        initMetaInfo(CalendarWeekDay.class, ContextUtil.NONE);
        initMetaInfo(CalendarException.class, ContextUtil.NONE);
        initMetaInfo(CurrencyExchangeRange.class, ContextUtil.NONE);
        initMetaInfo(PoolDomain.class, ContextUtil.NONE);
        initMetaInfo(ContactGroup.class, ContextUtil.NONE);
        initMetaInfo(ContactGroupAssignment.class, ContextUtil.NONE);
        initMetaInfo(StatusUpdate.class, ContextUtil.NONE);

        // Asset
        initMetaInfo(GenericAsset.class, ContextUtil.ASSET_CONTEXT);
        initMetaInfo(Asset.class, ContextUtil.ASSET_CONTEXT);
        initMetaInfo(AssetFolder.class, ContextUtil.ASSET_CONTEXT);
        initMetaInfo(AssetResourceAssignment.class, ContextUtil.ASSET_CONTEXT);
        initMetaInfo(CrossChargeFinancials.class, ContextUtil.ASSET_CONTEXT);
        initMetaInfo(DepreciationSchedule.class, ContextUtil.ASSET_CONTEXT);
        initMetaInfo(MaintenanceEstimates.class, ContextUtil.ASSET_CONTEXT);
        initMetaInfo(AssetOrganizationalAssignment.class, ContextUtil.ASSET_CONTEXT);

        //Client
        initMetaInfo(ClientCostCenter.class, ContextUtil.CLIENT_CONTEXT);
        initMetaInfo(ClientCostCenterAssignment.class, ContextUtil.CLIENT_CONTEXT);
        initMetaInfo(ClientFolder.class, ContextUtil.CLIENT_CONTEXT);
        initMetaInfo(Division.class, ContextUtil.CLIENT_CONTEXT);
        initMetaInfo(GenericClient.class, ContextUtil.CLIENT_CONTEXT);

        // Contract
        initMetaInfo(GenericContract.class, ContextUtil.NONE);
        initMetaInfo(Contract.class, ContextUtil.NONE);
        initMetaInfo(ContractFolder.class, ContextUtil.NONE);
        initMetaInfo(ContractChargeSchedule.class, ContextUtil.NONE);

        // Document
        initMetaInfo(DelDoc.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        initMetaInfo(Document.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        initMetaInfo(DocumentElement.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        initMetaInfo(DocumentFolder.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        initMetaInfo(GenericDocument.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        initMetaInfo(NoteMinutes.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        initMetaInfo(Software.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        initMetaInfo(UrlDocument.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        initMetaInfo(WPDoc.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        initMetaInfo(DocumentBlobDetails.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);

        //Resource
        initMetaInfo(Pool.class, ContextUtil.POOL_CONTEXT);
        initMetaInfo(Education.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(EmploymentHistory.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(IndustryExperience.class, ContextUtil.NONE);
        initMetaInfo(LanguageProficiency.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(LocationPreferences.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(Passport.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ProjectHistory.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(PoolReference.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(Resource.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceRates.class, ContextUtil.RESOURCE_CONTEXT);

        initMetaInfo(ResourceAttribute.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceAttributeProficiencyLevel.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceAttributeAssignment.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceAttributeCompetencyCategory.class,
            ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceAttributeCompetencyClassification.class,
            ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceAttributeCompetency.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceAttributeCompetencyProficiencyLevel.class,
            ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceAttributeSkillCategory.class,
            ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceAttributeSkillClassification.class,
            ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceAttributeSkill.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceAttributeSkillProficiencyLevel.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ResourceCostCenterAssignment.class, ContextUtil.RESOURCE_CONTEXT);

        initMetaInfo(ResourceRoleAssignment.class, ContextUtil.NONE);

        //Modules
        initMetaInfo(AssetModule.class, ContextUtil.ASSET_CONTEXT);
        initMetaInfo(ClientModule.class, ContextUtil.CLIENT_CONTEXT);
        initMetaInfo(DocumentModule.class, ContextUtil.UMBRELLA_DOCUMENT_CONTEXT);
        initMetaInfo(PoolModule.class, ContextUtil.POOL_CONTEXT);
        initMetaInfo(ResourceModule.class, ContextUtil.RESOURCE_CONTEXT);
        initMetaInfo(ScopeManagementModule.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(WbsModule.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(ContractModule.class, ContextUtil.NONE);
        initMetaInfo(FinancialModule.class, ContextUtil.UMBRELLA_FINANCIAL_CONTEXT);

        // Scope management
        // TODO: Validate context name.
        initMetaInfo(RiskMatrix.class, ContextUtil.NONE);
        initMetaInfo(MitigationFactor.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(MitigationTrigger.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(ScopeScheduleDate.class, ContextUtil.NONE);
        initMetaInfo(Duplicate.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(ScopeElement.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(ScopeFolder.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(Requirement.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(ChangeRequest.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(ServiceRequest.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(Action.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(Defect.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(Issue.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(Risk.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(ScopeDeliverable.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(ScopeSummaryTask.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(ScopeMilestone.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(ScopeTask.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(AggregateScope.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(ScopeWorkProduct.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(AbstractScope.class, ContextUtil.SCOPE_CONTEXT);

        // Security groups
        initMetaInfo(SecurityGroup.class, ContextUtil.NONE);

        // Security roles
        initMetaInfo(GenericSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(SecurityRoleCategory.class, ContextUtil.NONE);
        initMetaInfo(SecurityRole.class, ContextUtil.NONE);
        initMetaInfo(ProjectSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(DeliverableSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(TaskSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(ChangeRequestSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(IssueSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(RiskSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(ServiceRequestSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(RequirementSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(DefectSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(ActionItemSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(DocumentSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(NotesMinutesSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(PoolSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(FolderSecurityRole.class, ContextUtil.NONE);
        initMetaInfo(TemplateFolderSecurityRole.class, ContextUtil.NONE);

        // WBS
        initMetaInfo(SimpleNode.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(GenericProject.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(Portfolio.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(Opportunity.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(ElementDependency.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(ResourceTaskAssignment.class, ContextUtil.UMBRELLA_RESOURCE_TASK_ASSIGNMENT_CONTEXT);
        initMetaInfo(WbsScheduleDate.class, ContextUtil.NONE);
        initMetaInfo(TaskAssignment.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(WorkElement.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(Deliverable.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(SummaryTask.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(WorkProduct.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(AggregateNode.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(Proposal.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(Project.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(WorkOrganization.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(SimpleNode.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(Task.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(Milestone.class, ContextUtil.WBS_CONTEXT);

        //Profile
        initMetaInfo(Profile.class, ContextUtil.UMBRELLA_RESOURCE_TASK_ASSIGNMENT_CONTEXT);
        initMetaInfo(ProjectOrganizationalAssignment.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(ProfileCompetencyAssignment.class, ContextUtil.WBS_CONTEXT);
        initMetaInfo(ProfileSkillAssignment.class, ContextUtil.WBS_CONTEXT);

        // Financial.
        initMetaInfo(AbstractFinancial.class, ContextUtil.NONE);
        initMetaInfo(ChargeCodeCategory.class, ContextUtil.NONE);
        initMetaInfo(ChargeCode.class, ContextUtil.NONE);
        initMetaInfo(GenericChargeCode.class, ContextUtil.NONE);
        initMetaInfo(WorksheetFinancial.class, ContextUtil.UMBRELLA_FINANCIAL_CONTEXT);
        initMetaInfo(WbsFinancial.class, ContextUtil.UMBRELLA_FINANCIAL_CONTEXT);
        initMetaInfo(TimePhasedFinancial.class, ContextUtil.UMBRELLA_FINANCIAL_CONTEXT);
        initMetaInfo(TimeCode.class, ContextUtil.NONE);
        initMetaInfo(TimeCodeCategory.class, ContextUtil.NONE);
        initMetaInfo(TimeCodeGroup.class, ContextUtil.NONE);
        initMetaInfo(TimeCodeAssignment.class, ContextUtil.NONE);

        //TODO uncomment if needed
        //initMetaInfo(ProjectReconciliation.class, null,
        //    ProjectReconciliationCheckpoint.class, ContextUtil.NONE);
        //initMetaInfo(ProjectReconciliationStatus.class, null, null,
        //    ContextUtil.NONE);

        // Custom fields
        initMetaInfo(CustomFieldAssignment.class, ContextUtil.UMBRELLA_CUSTOM_FIELD_CONTEXT);
        initMetaInfo(GenericCustomField.class, ContextUtil.NONE);
        initMetaInfo(CustomFieldCategory.class, ContextUtil.NONE);
        initMetaInfo(CustomField.class, ContextUtil.NONE);
        initMetaInfo(CustomPortlet.class, ContextUtil.NONE);
        initMetaInfo(CustomPortletCategory.class, ContextUtil.NONE);
        initMetaInfo(GenericCustomPortlet.class, ContextUtil.NONE);

        // Timesheet
        initMetaInfo(Step.class, ContextUtil.NONE);
        initMetaInfo(GenericStep.class, ContextUtil.NONE);
        initMetaInfo(DefaultStep.class, ContextUtil.NONE);

        initMetaInfo(SummaryTimesheet.class, ContextUtil.NONE);
        initMetaInfo(StepTimesheet.class, ContextUtil.NONE);
        initMetaInfo(TimesheetApprovalStatus.class, ContextUtil.NONE);
        initMetaInfo(Credit.class, ContextUtil.NONE);
        initMetaInfo(CreditTimesheet.class, ContextUtil.NONE);

        initMetaInfo(GenericTimesheetElement.class, ContextUtil.NONE);

        initMetaInfo(PersonalTask.class, ContextUtil.NONE);
        initMetaInfo(PersonalProject.class, ContextUtil.NONE);
        initMetaInfo(AdministrativeTask.class, ContextUtil.NONE);
        initMetaInfo(AdministrativeProject.class, ContextUtil.NONE);
        initMetaInfo(DefaultAdministrativeTask.class, ContextUtil.NONE);
        initMetaInfo(PersonalTaskAssignment.class, ContextUtil.NONE);
        initMetaInfo(AdministrativeTaskAssignment.class, ContextUtil.NONE);
        initMetaInfo(GenericTaskAssignment.class, ContextUtil.NONE);

        // Communications
        initMetaInfo(GenericCommunication.class, ContextUtil.NONE);
        initMetaInfo(Notification.class, ContextUtil.NONE);
        initMetaInfo(WorkflowStepNotification.class, ContextUtil.NONE);
        initMetaInfo(StaffingRequest.class, ContextUtil.NONE);
        initMetaInfo(WorkflowAttachment.class, ContextUtil.NONE);
        initMetaInfo(WorkflowResponse.class, ContextUtil.NONE);
        initMetaInfo(GenericResponseType.class, ContextUtil.NONE);
        initMetaInfo(ChecklistItem.class, ContextUtil.NONE);
        initMetaInfo(VoteResponse.class, ContextUtil.NONE);

        // Workflow
        initMetaInfo(GenericWorkflow.class, ContextUtil.NONE);
        initMetaInfo(WorkflowProcess.class, ContextUtil.NONE);
        initMetaInfo(WorkflowCategory.class, ContextUtil.NONE);
        initMetaInfo(WorkflowGroup.class, ContextUtil.NONE);
        initMetaInfo(RunningWorkflowProcess.class,
            ContextUtil.NONE);
        initMetaInfo(AvailableWorkflowProcess.class,
            ContextUtil.NONE);
        initMetaInfo(WorkflowRoleMapping.class, ContextUtil.NONE);
        initMetaInfo(ExternalAction.class, ContextUtil.NONE);
        initMetaInfo(ExternalActionCategory.class, ContextUtil.NONE);
        initMetaInfo(WorkflowRole.class, ContextUtil.NONE);

        //ReqPro
        initMetaInfo(ReqProIntegrationFolder.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(ReqProScopeFolder.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(ReqProRequirement.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(ReqProConnection.class, ContextUtil.SCOPE_CONTEXT);
        initMetaInfo(ReqProImportStatus.class, ContextUtil.SCOPE_CONTEXT);
    }

    /**
     * Initializes the business meta-information for the given
     * <code>containerClass</code>.
     * @param containerClass
     *            Class of the container.
     * @param contextName
     *            Name of the default context for the given container class.
     */
    private static void initMetaInfo(Class containerClass, String contextName)
    {
        // TODO: Use metadata to load the contextNames.

        if (containerClass == null)
        {
            logger.error("initMetaInfo: containerClass must never be null.");
        }
        if (!RPMObject.class.isAssignableFrom(containerClass))
        {
            logger.error(containerClass.getName()
                    + " must be an RPMObject descendant.");
        }
        ContextUtil.getInstance().addContainerToMap(containerClass.getName(),
            contextName);
    }

    public static void main(String[] args) throws RPMException
    {
        logger.info("Starts");
        init();
        logger.info("Ends");
    }

    private static void loadManagers()
    {
        try
        {
            //ApplicationAdministration
            Class.forName(RPMCalendarManager.class.getName());
            Class.forName(CalendarExceptionManager.class.getName());
            Class.forName(ContactGroupsManager.class.getName());
            Class.forName(CurrencyExchangeManager.class.getName());
            Class.forName(DatafieldManager.class.getName());
            Class.forName(GeographicalManager.class.getName());
            Class.forName(OrganizationalManager.class.getName());
            Class.forName(RtfAssignmentManager.class.getName());
            Class.forName(AdministrationModuleManager.class.getName());
    
            //Resource
            //Class.forName(PoolDocumentManager.class.getName());
            Class.forName(PoolDomainsManager.class.getName());
            Class.forName(PoolManager.class.getName());
            Class.forName(PoolReferenceManager.class.getName());
            Class.forName(ResourceRoleAssignmentManager.class.getName());
            Class.forName(ResourceCostCenterManager.class.getName());
            Class.forName(ResourceAttributeAssignmentManager.class.getName());
            Class.forName(ResourceAttributeManager.class.getName());
            Class.forName(ContactGroupAssignmentManager.class.getName());
            Class.forName(ResourceEducationManager.class.getName());
            Class.forName(ResourceEmploymentHistoryManager.class.getName());
            Class.forName(GeographicalAssignmentManager.class.getName());
            Class.forName(ResourceIndustryExperienceManager.class.getName());
            Class.forName(ResourceLanguageProficiencyManager.class.getName());
            Class.forName(ResourceManager.class.getName());
            Class.forName(OrganizationalAssignmentManager.class.getName());
            Class.forName(ResourceProjectHistoryManager.class.getName());
            Class.forName(ResourceRatesManager.class.getName());
    
            //Security
            Class.forName(SecurityGroupManager.class.getName());
            Class.forName(SecurityRoleManager.class.getName());
    
            //Attribute managers
            Class.forName(AttributeManager.class.getName());
            Class.forName(AttributeAssignmentManager.class.getName());
    
            //ClientCostCenter
            Class.forName(ClientManager.class.getName());
            //contract
            Class.forName(ContractManager.class.getName());
            Class.forName(ContractChargeScheduleManager.class.getName());
    
            //Documents
            Class.forName(DocumentManager.class.getName());
            Class.forName(DocumentBlobDetailManager.class.getName());
    
            //Scope Management
            Class.forName(ScopeManager.class.getName());
            Class.forName(MitigationFactorManager.class.getName());
            Class.forName(ScopeScheduleDateManager.class.getName());
    
            //Assets
            //          Class.forName(AssetFolderManager.class.getName());
            Class.forName(AssetManager.class.getName());
            Class.forName(AssetDepreciationManager.class.getName());
            Class.forName(AssetMaintenanceManager.class.getName());
            Class.forName(AssetOrganizationsManager.class.getName());
            Class.forName(AssetResourcesManager.class.getName());
            Class.forName(AssetCrossChargeManager.class.getName());
    
            //scorecard
            Class.forName(ScorecardManager.class.getName());
            Class.forName(PublishedScorecardManager.class.getName());
            Class.forName(ScorecardAssignmentManager.class.getName());
    
            //Modules
            Class.forName(AssetModuleManager.class.getName());
            Class.forName(ClientModuleManager.class.getName());
            Class.forName(ContractModuleManager.class.getName());
            Class.forName(DocumentModuleManager.class.getName());
            Class.forName(FinancialModuleManager.class.getName());
            Class.forName(PoolModuleManager.class.getName());
            Class.forName(ResourceModuleManager.class.getName());
            Class.forName(ScopeManagementModuleManager.class.getName());
            Class.forName(WbsModuleManager.class.getName());
    
            //Work Management
            Class.forName(PortfoliosManager.class.getName());
            Class.forName(WbsManager.class.getName());
            Class.forName(ResourceTaskAssignmentManager.class.getName());
            Class.forName(TaskAssignmentManager.class.getName());
            Class.forName(ProfileSkillAssignmentManager.class.getName());
            Class.forName(ProfileCompetencyAssignmentManager.class.getName());
            Class.forName(ProfileManager.class.getName());
            Class.forName(OpportunityManager.class.getName());
            Class.forName(DependenciesManager.class.getName());
            Class.forName(WbsScheduleDateManager.class.getName());
            Class.forName(ProjectOrganizationalAssignmentManager.class.getName());
    
            //Profile
            Class.forName(ProfileManager.class.getName());
            Class.forName(ProfileCompetencyAssignmentManager.class.getName());
            Class.forName(ProfileSkillAssignmentManager.class.getName());
    
            // Financial.
            Class.forName(ChargeCodeManager.class.getName());
            Class.forName(WorksheetFinancialManager.class.getName());
            Class.forName(WbsFinancialManager.class.getName());
            Class.forName(TimePhasedFinancialManager.class.getName());
            Class.forName(TimecodeManager.class.getName());
            Class.forName(TimeCodeAssignmentManager.class.getName());
            Class.forName(ProjectReconciliationManager.class.getName());
            Class.forName(ProjectReconciliationStatusManager.class.getName());
    
            // Custom Fields.
            Class.forName(CustomFieldManager.class.getName());
            Class.forName(CustomFieldAssignmentManager.class.getName());
            Class.forName(CustomPortletManager.class.getName());
    
            // Timesheet
            Class.forName(SummaryTimesheetManager.class.getName());
            Class.forName(StepManager.class.getName());
            Class.forName(StepTimesheetManager.class.getName());
            Class.forName(TimesheetApprovalStatusManager.class.getName());
            Class.forName(DefaultStepManager.class.getName());
            Class.forName(TimesheetTaskManager.class.getName());
            Class.forName(AdministrativeTaskAssignmentManager.class.getName());
            Class.forName(PersonalTaskAssignmentManager.class.getName());
            Class.forName(GenericTaskAssignmentManager.class.getName());
            Class.forName(DefaultAdministrativeTaskManager.class.getName());
            
            // Communications
            Class.forName(CommunicationManager.class.getName());
            Class.forName(WorkflowResponseManager.class.getName());
            Class.forName(WorkflowAttachmentManager.class.getName());
            Class.forName(WorkflowResponseChecklistManager.class.getName());
    
            // Workflow
            Class.forName(WorkflowManager.class.getName());
            Class.forName(RunningWorkflowProcessManager.class.getName());
            Class.forName(AvailableWorkflowProcessManager.class.getName());
            Class.forName(WorkflowRoleMappingManager.class.getName());
            Class.forName(WorkflowRoleManager.class.getName());
            Class.forName(ExternalActionManager.class.getName());
    
            //ReqPro
            Class.forName(ReqProManager.class.getName());
        }
        catch (ClassNotFoundException e)
        {
            logger.error(e);
        }
    }

    private static void printRPMVersion()
    {

        String version = "<not available>";
        String build = "<not available>";

        try
        {
            ResourceBundle props = ResourceBundle.getBundle("build-info");

            version = props.getString("version.name");
            if (props.getString("label") != null)
            {
                build = props.getString("label");
                build = StringUtil.substringAfterOccurenceOf(
                    props.getString("label"), ".");
            }
        }
        catch (Exception e)
        {
            //build-info is not available
        }

        logger.info("Starting RPM Web Services version " + version + " build "
                + build);

    }

    private static void printXalanVersion()
    {

        int major = Version.getMajorVersionNum();
        int release = Version.getReleaseVersionNum();
        int maintenance = Version.getMaintenanceVersionNum();

        logger.info("Loaded Xalan version: " + major + "." + release + "."
                + maintenance);

        if (major != 2 || !(release == 6 || release == 7))
        {
            logger.warn("Xalan version: "
                    + major
                    + "."
                    + release
                    + ".x"
                    + " may cause unexpected errors when using"
                    + " the loadFromXpath method. Supported version are 2.6.x and 2.7.x.\n");
        }
    }
    
    private static void initCheckpointMap()
    {

        List containers = MetadataStore.getInstance().getContainers();

        for (Iterator iter = containers.iterator(); iter.hasNext();)
        {
            Container container = (Container) iter.next();
            Class containerClass = container.getJavaClass();

            if (!container.isAbstractClass())
            {
                String checkpoint = container.getCheckpoint();
                Class checkpointClass = null;
                if (checkpoint != null)
                {
                    try
                    {
                        checkpointClass = Class.forName(checkpoint);
                    }
                    catch (Exception e)
                    {
                        logger
                            .error("Failed to match checkpoint metadata. Child Class:"
                                    + containerClass
                                    + ", Checkpoint Class: "
                                    + checkpoint);
                    }
                }
                ContainerToCheckpointMap.add(containerClass, checkpointClass);
            }
        }
    }
}
