package com.ibm.rpm.interfaces;

import java.util.Calendar;

import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.scope.WorkflowResponseScope;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.AlreadyCheckedOutResult;
import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.PropertyArray;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectArray;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.interfaces.impl.ManagerCaller;
import com.ibm.rpm.interfaces.impl.ServerFactory;
import com.ibm.rpm.interfaces.wrapper.ResourceArray;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;

public class Application
{
    private ServerFactory factory;
    
    public Application()
    {
        this(ServerFactory.getInstance());
    }
    
    private ManagerCaller getManagerCaller()
    {
        return factory.getManagerCaller();
    }
    
    public Application(ServerFactory factoryToUse)
    {
        factory = factoryToUse;
    }

    public Result checkOut(String sessionID, RPMObject rpmObject,
            boolean isTimeless)
    {
        return getManagerCaller().checkOut(sessionID, rpmObject, isTimeless)
            .toResult();
    }

    public Result checkIn(String sessionID, RPMObject rpmObject)
    {
        return getManagerCaller().checkIn(sessionID, rpmObject).toResult();
    }

    public Calendar  getDatabaseTimestamp(String sessionID) throws RPMException
    {
        return getManagerCaller().getDatabaseTimestamp(sessionID);
    }
    
    public AlreadyCheckedOutResult isCheckedOut(String sessionID,
            RPMObject rpmObject)
    {
        return getManagerCaller().isCheckedOut(sessionID, rpmObject);
    }

    public Result forceCheckinOnSessionsCheckOut(String sessionID,
            String targetSessionID)
    {
        return getManagerCaller().forceCheckinOnSessionsCheckOut(sessionID,
            targetSessionID).toResult();
    }

    public Result forceCheckinOnUsersCheckOut(String sessionID, Resource user)
    {
        return getManagerCaller().forceCheckinOnUsersCheckOut(sessionID, user)
            .toResult();
    }

    public Result forceCheckinOnRPMObjectCheckOut(String sessionID,
            RPMObject rpmObject)
    {
        return getManagerCaller().forceCheckinOnRPMObjectCheckOut(sessionID,
            rpmObject).toResult();
    }

    public SingleResult save(String sessionID, RPMObject rpmObject,
            RPMObjectScope rpmObjectScope, ReloadType reloadType)
    {
        return getManagerCaller().save(sessionID, rpmObject, rpmObjectScope,
            reloadType).toSingleResult();
    }

    public Result delete(String sessionID, RPMObject rpmObject)
    {
        return getManagerCaller().delete(sessionID, rpmObject).toResult();
    }

    public ArrayResult loadFromXpath(String sessionID, String xpath,
            RPMObjectScope scope)
    {
        return getManagerCaller().load(sessionID, xpath, scope).toArrayResult();
    }

    public SingleResult loadFromID(String sessionID, RPMObject rpmObject,
            RPMObjectScope scope)
    {
        return getManagerCaller().load(sessionID, rpmObject, scope).toSingleResult();
    }

    public ArrayResult loadArrayFromID(String sessionID,
            RPMObjectArray rpmObjectArray, RPMObjectScope scope)
    {
        return getManagerCaller().load(sessionID, rpmObjectArray.getRPMObjectList(),
            scope).toArrayResult();
    }

    public PropertyArray getSystemInfo(String sessionID) throws RPMException
    {
        return getManagerCaller().getSystemInfo(sessionID);
    }

    public SingleResult startWorkflow(String sessionID,
            WorkflowProcess process, RPMObject object,
            RunningWorkflowProcessScope scope)
    {
        return getManagerCaller().startWorkflow(sessionID, process, object, scope)
            .toSingleResult();
    }

    public Result associateWorkflow(String sessionID, WorkflowProcess process,
            GenericProject genericProject, boolean associate, boolean isDefault)
    {
        return getManagerCaller().associateWorkflow(sessionID, process,
            genericProject, associate, isDefault).toResult();
    }

    public SingleResult suspendWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope)
    {
        return getManagerCaller().suspendWorkflow(sessionID, runningProcess, scope)
            .toSingleResult();
    }

    public SingleResult resumeWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope)
    {
        return getManagerCaller().resumeWorkflow(sessionID, runningProcess, scope)
            .toSingleResult();
    }

    public Result cancelWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess)
    {
        return getManagerCaller().cancelWorkflow(sessionID, runningProcess)
            .toResult();
    }
    
    private Resource[] getResourceArrayFromWrapper( ResourceArray x)
    {
        return x==null?null:x.getResourceArray();
    }

    public SingleResult sendNotification(String sessionID,
            GenericCommunicationScope scope, RPMObject attachment,
            ResourceArray recipients, String subject, String comments)
    {
        return getManagerCaller().sendNotification(sessionID, scope, attachment,
            getResourceArrayFromWrapper(recipients), subject, comments).toSingleResult();
    }

    public SingleResult sendStaffingRequest(String sessionID,
            GenericCommunicationScope scope, RPMObject attachment,
            ResourceArray recipients, String subject, String comments,
            Calendar dueDate)
    {
        return getManagerCaller().sendStaffingRequest(sessionID, scope, attachment,
            getResourceArrayFromWrapper(recipients), subject, comments, dueDate).toSingleResult();
    }

    public SingleResult saveWorkflowResponse(String sessionID,
            WorkflowResponse response, WorkflowResponseScope scope)
    {
        return getManagerCaller().saveWorkflowResponse(sessionID, response, scope)
            .toSingleResult();
    }
    
    public SingleResult sendWorkflowResponse(String sessionID,
            WorkflowResponse response, WorkflowResponseScope scope)
    {
        return getManagerCaller().sendWorkflowResponse(sessionID, response, scope)
            .toSingleResult();
    }

    public SingleResult updateReadAndFolder(String sessionID,
            WorkflowResponse response, WorkflowResponseScope scope)
    {
        return getManagerCaller().updateReadAndFolder(sessionID, response, scope)
            .toSingleResult();
    }
    
    /**
     * The assignScorecard method will assign a Scorecard object to an
     * RPMObject. There are three types of RPMObjects that are amenable for
     * scorecard assignment: 1) Asset; 2) GenericProject and its subobjects; 3)
     * AbstractScope and its subobjects. For more information see those objects
     * documentation. The assumption for the assignment is that the Scorecard is
     * already been created in the design phase and that the RPMObject is
     * already been saved. After the assignment, any change in the Scorecard
     * will not affect the AssignedScorecard.
     * 
     * @param sessionID -
     *            The session identifier obtained during the login call. If this
     *            parameter isnt valid, the method will fail with a RPM
     *            Exception
     * @param objectToAssign -
     *            the object that you want to assign the scorecard.
     * @param scoreCardToAssign -
     *            the scorecard to assign
     * @return - the object that you wanted to assign with the scorecard
     *         assigned to it
     */
    public SingleResult assignScoreCard(String sessionID,
            RPMObject objectToAssign, RPMObjectScope objectToAssignScope,
            Scorecard scoreCardToAssign, ReloadType reloadType)
    {
        return getManagerCaller().assignScoreCard(sessionID, objectToAssign,
            objectToAssignScope, scoreCardToAssign, reloadType)
            .toSingleResult();
    }
    
    /**
     * The assignScorecard method will assign a Scorecard object to a Resource.
     * Note that a Resource can have at most one scorecard per Project or Proposal.
     * In order for a scorecard to be assigned to her, the Resource also needs to 
     * have work assigned for that project.
     * 
     * @param sessionID -
     *            The session identifier obtained during the login call. If this
     *            parameter isnt valid, the method will fail with a RPM
     *            Exception
     * @param project
     *            Either an instance of Project, Proposal or WorkOrganization     
     * @param objectToAssign -
     *            the object that you want to assign the scorecard.
     * @param scoreCardToAssign -
     *            the scorecard to assign
     * @return - the object that you wanted to assign with the scorecard
     *         assigned to it
     */
    public SingleResult assignScoreCardToResource(String sessionID, GenericProject project,
            Resource objectToAssign, ResourceScope objectToAssignScope,
            Scorecard scoreCardToAssign, ReloadType reloadType)
    {
        return getManagerCaller().assignScoreCardToResource(sessionID, project, objectToAssign,
            objectToAssignScope, scoreCardToAssign, reloadType)
            .toSingleResult();
    }

    /**
     * The publishScorecard publishes a an assignedScorecard. This has two
     * effects: 1) the assignedScorecard is available for portfolio analysis; 2)
     * the assignedScorecard values are been versioned and a user can return to
     * history assignedScorecard values and compare between versions. A user can
     * publish an assignedScorecard as much times as desired, each such publish
     * marks new version. Once an assignedScorecard is been published, it will
     * continue to be as such. A user can deactivate the assignedScorecard using
     * the AssignedScorecardState.
     * 
     * 
     * @param sessionID -
     *            The session identifier obtained during the login call. If this
     *            parameter isnt valid, the method will fail with a RPM
     *            Exception
     * @param assignedScorecard -
     *            The assignedScorecard to publish.
     * @param scorecardScope -
     *            This scope defines the assignedScorecard elements that will be
     *            loaded and returned. A null will load only the
     *            assignedScorecard.
     * @return AssignedScorecard object. This object is the assignedScorecard
     *         after the publish was done
     */
    public SingleResult publishScorecard(String sessionID,
            AssignedScorecard assignedScorecard,
            AbstractScorecardScope scorecardScope)
    {
        return getManagerCaller().publishScorecard(sessionID, assignedScorecard,
            scorecardScope).toSingleResult();
    }

    public SingleResult acceptTaskAssignmentChanges(String sessionID,
            TaskAssignment taskAssignment, TaskAssignmentScope scope,
            boolean acceptEETC, boolean acceptForecastDates)
    {
        return getManagerCaller().acceptRejectTaskAssignmentChanges(sessionID,
            taskAssignment, scope, true, acceptEETC, acceptForecastDates)
            .toSingleResult();
    }

    public SingleResult rejectTaskAssignmentChanges(String sessionID,
            TaskAssignment taskAssignment, TaskAssignmentScope scope,
            boolean rejectEETC, boolean rejectForecastDates)
    {
        return getManagerCaller().acceptRejectTaskAssignmentChanges(sessionID,
            taskAssignment, scope, false, rejectEETC, rejectForecastDates)
            .toSingleResult();
    }

    public SingleResult copyProposedToPlan(String sessionID,
            WorkElement workElement, WorkElementScope workElementScope)
    {
        return getManagerCaller().copyProposedToPlan(sessionID, workElement,
            workElementScope).toSingleResult();
    }

    public SingleResult copyAssignmentProposedToPlan(String sessionID,
            ResourceTaskAssignment resourceTaskAssignment,
            ResourceTaskAssignmentScope scope)
    {
        return getManagerCaller().copyAssignmentProposedToPlan(sessionID,
            resourceTaskAssignment, scope).toSingleResult();
    }

    public SingleResult commitAffectedResources(String sessionID,
            GenericProject genericProject, WorkElementScope workElementScope)
    {
        return getManagerCaller().commitAffectedResources(sessionID, genericProject,
            workElementScope).toSingleResult();
    }

    public SingleResult replaceProfileWithResource(String sessionID,
            ResourceTaskAssignment rta, ResourceTaskAssignmentScope rtaScope,
            Resource resource)
    {
        return getManagerCaller().replaceProfileWithResource(sessionID, rta,
            rtaScope, resource).toSingleResult();
    }

    public SingleResult replaceResourceWithProfile(String sessionID,
            ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope)
    {
        return getManagerCaller().replaceResourceWithProfile(sessionID, assignment,
            rtaScope).toSingleResult();
    }

    public Result transferUnderScope(String sessionID,
            AbstractAggregateScope scopeElement, WorkElement workElement)
    {
        return getManagerCaller().transferUnderScope(sessionID, scopeElement,
            workElement).toResult();
    }

    public Result transferUnderWBS(String sessionID, WorkElement workElement,
            AbstractScope abstractScope)
    {
        return getManagerCaller().transferUnderWBS(sessionID, workElement,
            abstractScope).toResult();
    }

    public Result moveUnderWBS(String sessionID, WorkElement workElement,
            AbstractScope abstractScope)
    {
        return getManagerCaller().
            moveUnderWBS(sessionID, workElement, abstractScope).toResult();
    }

    public Result moveAndTransferUnderWBS(String sessionID,
            WorkElement workElement, AbstractScope abstractScope)
    {
        return getManagerCaller().moveAndTransferUnderWBS(sessionID, workElement,
            abstractScope).toResult();
    }

    public SingleResult convertContainer(String sessionID,
            RPMObject inputContainer, RPMObject outputContainer,
            RPMObjectScope rpmScope)
    {
        return getManagerCaller().convertContainer(sessionID, inputContainer,
            outputContainer, rpmScope).toSingleResult();
    }

    public SingleResult saveProjectBaseline(String sessionID,
            RPMObject projectObject, RPMObjectScope scope)
    {
        return getManagerCaller().saveProjectBaseline(sessionID, projectObject,
            scope).toSingleResult();
    }

    public SingleResult rollupProjectEV(String sessionID,
            RPMObject projectObject, RPMObjectScope scope)
    {
        return getManagerCaller().rollupProjectEV(sessionID, projectObject,
            scope).toSingleResult();
    }
    
    public SingleResult enableProjectEVCalculation(String sessionID,
            RPMObject projectObject, RPMObjectScope scope)
    {
        return getManagerCaller().enableProjectEVCalculation(sessionID, projectObject,
            scope).toSingleResult();
    }
    
    public Result activateEVWeekDelays(String sessionID, int evDelays)
    {
        return getManagerCaller().activateEVWeekDelays(sessionID, evDelays)
            .toResult();
    }


    public SingleResult createObjectFromTemplate(String sessionID,
            RPMObject inputObject, RPMObject template, RPMObjectScope scope)
    {
        return getManagerCaller().createObjectFromTemplate(sessionID, inputObject,
            template, scope).toSingleResult();
    }

    

    public Result disableAutomaticCheckoutMode(String sessionID)
    {
        return getManagerCaller().setAutomaticCheckout(sessionID, false).toResult();
    }

    public Result enableAutomaticCheckoutMode(String sessionID)
    {
        return getManagerCaller().setAutomaticCheckout(sessionID, true).toResult();
    }

    public Result moveBeforeSibling(String sessionID, ScopeElement objectToMove,
            ScopeElement nextSibling)
    {
        return getManagerCaller().moveBeforeSibling(sessionID, objectToMove,
            nextSibling).toResult();
    }
    
    public SingleResult lockDocument(String sessionid, GenericDocument documentToLock)
    {
    	return getManagerCaller().lockDocument(sessionid, documentToLock).toSingleResult();
    }
    
    public SingleResult unLockDocument(String sessionid, GenericDocument documentToUnlock)
    {
    	return getManagerCaller().unLockDocument(sessionid, documentToUnlock).toSingleResult();
    }
}