/*
 * Created on May 31, 2006
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package com.ibm.rpm.interfaces;

import java.lang.reflect.Array;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author eturcot
 * 
 * Replace cyclic references with a clone whose properties are causing the
 * cycles are replaced with null.
 */
public class CyclicReferenceRemover
{
    private static Log log = LogFactory.getLog(CyclicReferenceRemover.class);

    private static Class rpmOjectClass;

    private static Class rpmOjectScopeClass;

    static
    {
        try
        {
            rpmOjectClass = Class.forName("com.ibm.rpm.framework.RPMObject");
            rpmOjectScopeClass = Class
                .forName("com.ibm.rpm.framework.RPMObjectScope");
        }
        catch (ClassNotFoundException e)
        {
            log.fatal("Can not initialize CyclicReferenceRemover", e);
        }
    }

    public static void remove(Object obj)
    {
        try
        {
            if (obj != null)
            {
                Map stack = new IdentityHashMap();
                Map clones = new IdentityHashMap();
                replaceCycles(obj, stack, clones);
            }
        }
        catch (Exception e)
        {
            log.debug(e);
        }
    }

    public static void remove(Object[] objs)
    {
        if (objs != null)
        {
            for (int i = 0; i < objs.length; i++)
            {
                try
                {

                    Object obj = objs[i];
                    if (obj != null)
                    {
                        Map stack = new IdentityHashMap();
                        Map clones = new IdentityHashMap();
                        replaceCycles(obj, stack, clones);
                    }
                }
                catch (Exception e)
                {
                    log.debug(e);
                }
            }
        }
    }

    private static void replaceCycles(Object obj, Map stack, Map clones)
            throws Exception
    {
        stack.put(obj, obj);
        Map properties = PropertyUtils.describe(obj);
        for (Iterator it = properties.keySet().iterator(); it.hasNext();)
        {
            String key = (String) it.next();
            Object value = properties.get(key);
            if (value != null)
            {
                if (stack.containsValue(value))
                {
                    Object clone = cloneObjectWithoutReferences(value, stack,
                        clones);
                    PropertyUtils.setSimpleProperty(obj, key, clone);
                }
                else if (rpmOjectClass.isAssignableFrom(value.getClass())
                        || rpmOjectScopeClass
                            .isAssignableFrom(value.getClass()))
                {
                    replaceCycles(value, stack, clones);
                }
                else if (value.getClass().isArray())
                {
                    inspectArray(value, stack, clones);
                }
            }
        }
        stack.remove(obj);
    }

    private static void inspectArray(Object array, Map stack, Map clones)
            throws Exception
    {
        int length = Array.getLength(array);
        for (int i = 0; i < length; i++)
        {
            Object obj = Array.get(array, i);
            if (rpmOjectClass.isAssignableFrom(obj.getClass())
                    || rpmOjectScopeClass.isAssignableFrom(obj.getClass()))
            {
                replaceCycles(obj, stack, clones);
            }
        }
    }

    private static Object cloneObjectWithoutReferences(Object obj, Map stack,
            Map clones) throws Exception
    {
        Object clone = clones.get(obj);
        if (clone == null)
        {
            clone = BeanUtils.cloneBean(obj);
            clones.put(obj, clone);
        }
        Map properties = PropertyUtils.describe(obj);
        for (Iterator it = properties.keySet().iterator(); it.hasNext();)
        {
            String key = (String) it.next();
            Object value = properties.get(key);
            if (value != null)
            {
                if (stack.containsValue(value))
                {
                    PropertyUtils.setSimpleProperty(clone, key, null);
                }
                else if (value.getClass().isArray()
                        && removeArray(value, stack))
                {
                    PropertyUtils.setSimpleProperty(clone, key, null);
                }
            }
        }

        return clone;
    }

    private static boolean removeArray(Object array, Map stack)
            throws Exception
    {
        int length = Array.getLength(array);
        for (int i = 0; i < length; i++)
        {
            Object value = Array.get(array, i);
            if (value != null && stack.containsValue(value))
            {
                return true;
            }
        }

        return false;
    }

}
