package com.ibm.rpm.interfaces;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;

/**
 * @author eturcot
 */
public class TestCyclicReferenceRemover extends TestCase {
	
	private static Log log = LogFactory.getLog(TestCyclicReferenceRemover.class);
	
    public static void main(String[] args) {
        junit.awtui.TestRunner.run(TestCyclicReferenceRemover.class);
    }

    public TestCyclicReferenceRemover(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }
    
    public void testSingleResult() throws Exception {
    	log.debug("testSingleResult() starting...");
    	
    	// Mock objects
    	Project project = new Project();
    	Task task1 = new Task();
    	Task task2 = new Task();
    	task1.setContainingProject(project);
    	
    	SummaryTask summary1 = new SummaryTask();
    	SummaryTask summary2 = new SummaryTask();
    	SummaryTask summary3 = new SummaryTask();
    	SummaryTask summary4 = new SummaryTask();
    	summary1.setChildren(new WorkElement[]{summary2, summary3});
    	summary2.setChildren(new WorkElement[]{summary3});
    	summary3.setChildren(new WorkElement[]{summary4});
    	summary4.setParent(summary2);
    	summary1.setID("11");
    	summary2.setID("22");
    	summary3.setID("33");
    	summary3.setParent(project);
    	Asset asset = new Asset();
    	summary3.setAsset(asset);
    	summary4.setID("44");
    	
    	WorkElement[] tasks = {task1, task2, summary1, summary2, summary4};  
    	
    	project.setChildren(tasks);
    	project.setDefaultProject(Boolean.TRUE);
    	project.setExternalID("69");
    	project.setPriority(new Integer(5));
    	
    	Currency currency = new Currency();
    	currency.setName("Canada");
    	currency.setParent(project);
    	project.setCurrency(currency);    	
    	
    	SingleResult result = new SingleResult();
    	result.setRpmObject(project);
    	
    	CyclicReferenceRemover.remove(project);
    	
    	Project clonedProjectTask = (Project)project.getChildren()[0].getContainingProject();
    	Assert.assertNull("clonedProjectTask : children", clonedProjectTask.getChildren());
    	Assert.assertNull("clonedProjectTask : currency", clonedProjectTask.getCurrency());
    	
    	Project clonedProjectCurrency = (Project)project.getCurrency().getParent();
    	Assert.assertNull("clonedProjectCurrency : currency", clonedProjectCurrency.getCurrency());
    	Assert.assertNull("clonedProjectCurrency : children", clonedProjectCurrency.getChildren());
    	
    	SummaryTask clonedSummaryTask = (SummaryTask)project.getChildren()[4].getParent();
    	Assert.assertNull("clonedSummaryTask : children", clonedSummaryTask.getChildren());
    	
    	SummaryTask summaryTask = (SummaryTask)project.getChildren()[2];
    	Assert.assertEquals("summary1", summaryTask.getID(), "11");
    	summaryTask = (SummaryTask)summaryTask.getChildren()[0];
    	Assert.assertEquals("summary2", summaryTask.getID(), "22");
    	Assert.assertNotNull("summary2 : children", summaryTask.getChildren());
    	summaryTask = (SummaryTask)summaryTask.getChildren()[0];
    	Assert.assertEquals("summary3", summaryTask.getID(), "33");
    	Assert.assertNotSame("summary3.parent == project", summaryTask.getParent(), project);
    	Assert.assertSame("summary3.asset == asset", summaryTask.getAsset(), asset);
    	
    	log.debug("testSingleResult() finished.");
    }

}
