package com.ibm.rpm.interfaces.impl;

import java.util.Calendar;

import javax.xml.namespace.QName;

import org.apache.axis.Constants;
import org.apache.axis.encoding.ser.CalendarDeserializer;

/**
 * Intercepts deserialization to convert "Microsoft .NET 1.1" empty DateTime
 * (0001-01-01T00:00:00.0000000) to Java null Calendar.
 */
public class DotNetCalendarDeserializer extends CalendarDeserializer
{
    // In Axis 1.3.0: QName("http://www.w3.org/2001/XMLSchema", "dateTime").
    private static final QName CALENDAR_QNAME = Constants.XSD_DATETIME;

    // Example with eastern standard timezone: 0001-01-01T00:00:00.0000000-05:00
    private static final String DOT_NET_NULL_DATE_PREFIX = "0001-01-01T";

    public DotNetCalendarDeserializer()
    {
        super(Calendar.class, DotNetCalendarDeserializer.CALENDAR_QNAME);
    }

    public Object makeValue(String source)
    {
        // Design note: converts DOT_NET_NULL_DATE_PREFIX date to a null date.
        Calendar calendar;
        if (source != null && source.startsWith(DOT_NET_NULL_DATE_PREFIX))
        {
            calendar = null;
        }
        else
        {
            calendar = (Calendar) super.makeValue(source);
        }
        return calendar;
    }
}
