package com.ibm.rpm.interfaces.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.authentication.LdapAuthenticationController;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * Implements the authentication using both an LDAP server (queried using JNDI)
 * and RPM's database.
 * <p>
 * Testing: for instance, use a registered user in IBM's bluepages. You will use
 * the usual Intranet username (ex.: legaults@ca.ibm.com) and password. Ensure
 * you create a Resource in RPM's database with the same Username. Ensure this
 * resource is Active, has a Calendar assigned and has an Effective Start Date
 * set to today or in the past. Then check all System Security Rights for this
 * resource.
 * 
 * 
 * @author legaults, mcduffs
 */
public class LdapAuthentication extends DatabaseAuthentication
{
    private static Log logger = LogFactory.getLog(LdapAuthentication.class);

    private LdapAuthenticationController jndiController;

    public LdapAuthentication(ServerFactory factory)
    {
        super(factory);
        jndiController = new LdapAuthenticationController(factory.getCurrentOptions());
    }

    public String login(String username, String password, String dsn)
            throws RPMException
    {
        try
        {
            jndiController.authenticate(username, password);
        }
        catch (Throwable x)
        {
            throw ManagerCaller.handleException(x);
        }
        return login("SP_LOGON_SECURITY", username, null, dsn);
    }

    public void changePassword(String username, String oldPw, String newPw,
            String dsn) throws RPMException
    { 
        // Authenticate.changePassword operation is not supported with LDAP authentication.
        RPMException ex = new RPMException(ErrorCodes.CHANGER_PASSWORD_NOT_SUPPORTED_ON_EXTERNAL_AUTHENTICATION);
        logger.error(ex.getMessage(), ex);
        throw ex;
    }
    public static final String LDAP_NAME = "LdapOptions";
    

}
