/*
 * Created on 6-Oct-2005
 */
package com.ibm.rpm.resource.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Education;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author dominiqu
 */
public class EducationCheckpoint extends AbstractCheckpoint
{
    private static final Class EDUCATION_PARENTS = Resource.class;

    protected static EducationCheckpoint instance = new EducationCheckpoint();

    public static EducationCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, EDUCATION_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, Education.class,
            msgCtx))
            return;

        // Initialize
        Education mainEducation = (Education) rpmObject;

        // Field validation: organization
        if (mainEducation.testOrganizationModified())
        {
            GenericValidator.validateMaxLength(mainEducation, "organization",
                mainEducation.getOrganization(),
                ValidationConstants.EDUCATION_ORGANIZATION_MAX, msgCtx);
        }

        // Field validation: degreeProgram
        if (mainEducation.testDegreeProgramModified())
        {
            GenericValidator.validateMaxLength(mainEducation, "degreeProgram",
                mainEducation.getDegreeProgram(),
                ValidationConstants.EDUCATION_DEGREEPROGRAM_MAX, msgCtx);
        }

        // Field validation: specialization
        if (mainEducation.testSpecializationModified())
        {
            GenericValidator.validateMaxLength(mainEducation, "specialization",
                mainEducation.getSpecialization(),
                ValidationConstants.EDUCATION_SPECIALIZATION_MAX, msgCtx);
        }

        // Field validation: honoursAwards
        if (mainEducation.testHonoursAwardsModified())
        {
            GenericValidator.validateMaxLength(mainEducation, "honoursAwards",
                mainEducation.getHonoursAwards(),
                ValidationConstants.EDUCATION_HONOURSAWARDS_MAX, msgCtx);
        }
        if (mainEducation.testAppointedDateModified())
        {
            GenericValidator.validateRange(mainEducation,
                ValidationConstants.EDUCATION_APPOINTED_DATE_FIELD,
                mainEducation.getAppointedDate(),
                ValidationConstants.DATE_FIELD_MIN,
                ValidationConstants.DATE_FIELD_MAX, msgCtx);
        }
        if (mainEducation.testCertificationGraduateDateModified())
        {
            GenericValidator.validateRange(mainEducation,
                ValidationConstants.EDUCATION_CERTIFICATION_GRADUATE_DATE_FIELD,
                mainEducation.getCertificationGraduateDate(),
                ValidationConstants.DATE_FIELD_MIN,
                ValidationConstants.DATE_FIELD_MAX, msgCtx);
        }
    }
}