/*
 * Created on 7-Oct-2005
 */
package com.ibm.rpm.resource.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.EmploymentHistory;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author dominiqu
 */
public class EmploymentHistoryCheckpoint extends AbstractCheckpoint
{
    private static final Class EMPLOYMENTHISTORY_PARENTS = Resource.class;

    protected static EmploymentHistoryCheckpoint instance = new EmploymentHistoryCheckpoint();

    public static EmploymentHistoryCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, EMPLOYMENTHISTORY_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject,
            EmploymentHistory.class, msgCtx))
            return;

        // Initialize
        EmploymentHistory mainEmploymentHistory = (EmploymentHistory) rpmObject;
        GenericValidator.validateMandatory(mainEmploymentHistory, "company",
            mainEmploymentHistory.getCompany(), msgCtx);

        // Field validation: company
        if (mainEmploymentHistory.testCompanyModified())
        {
            GenericValidator.validateMaxLength(mainEmploymentHistory,
                "company", mainEmploymentHistory.getCompany(),
                ValidationConstants.EMPLOYMENTHISTORY_COMPANY_MAX, msgCtx);
            GenericValidator.validateMinLength(mainEmploymentHistory,
                "company", mainEmploymentHistory.getCompany(),
                ValidationConstants.EMPLOYMENTHISTORY_COMPANY_MIN, msgCtx);
        }

        // Field validation: title
        if (mainEmploymentHistory.testTitleModified())
        {
            GenericValidator.validateMaxLength(mainEmploymentHistory, "title",
                mainEmploymentHistory.getTitle(),
                ValidationConstants.EMPLOYMENTHISTORY_TITLE_MAX, msgCtx);
        }

        // Field validation: description
        if (mainEmploymentHistory.testDescriptionModified())
        {
            GenericValidator.validateMaxLength(mainEmploymentHistory,
                "description", mainEmploymentHistory.getDescription(),
                ValidationConstants.EMPLOYMENTHISTORY_DESCRIPTION_MAX, msgCtx);
        }

        // Field validation: location
        if (mainEmploymentHistory.testLocationModified())
        {
            GenericValidator.validateMaxLength(mainEmploymentHistory,
                "location", mainEmploymentHistory.getLocation(),
                ValidationConstants.EMPLOYMENTHISTORY_LOCATION_MAX, msgCtx);
        }

        // Field validation: responsibilities
        if (mainEmploymentHistory.testResponsibilitiesModified())
        {
            GenericValidator.validateMaxLength(mainEmploymentHistory,
                "responsibilities",
                mainEmploymentHistory.getResponsibilities(),
                ValidationConstants.EMPLOYMENTHISTORY_RESPONSIBILITIES_MAX,
                msgCtx);
        }

        if (mainEmploymentHistory.testStartDateModified())
        {
            GenericValidator.validateRange(mainEmploymentHistory,
                ValidationConstants.EMPLOYMENTHISTORY_START_DATE_FIELD,
                mainEmploymentHistory.getStartDate(),
                ValidationConstants.DATE_FIELD_MIN,
                ValidationConstants.DATE_FIELD_MAX, msgCtx);
        }
        if (mainEmploymentHistory.testEndDateModified())
        {
            GenericValidator.validateRange(mainEmploymentHistory,
                ValidationConstants.EMPLOYMENTHISTORY_END_DATE_FIELD,
                mainEmploymentHistory.getEndDate(),
                ValidationConstants.DATE_FIELD_MIN,
                ValidationConstants.DATE_FIELD_MAX, msgCtx);
        }
        // Field validation: startDate and endDate
        if (mainEmploymentHistory.testStartDateModified()
                || mainEmploymentHistory.testEndDateModified())
        {
            if (mainEmploymentHistory.getStartDate() != null)
            {
                GenericValidator.validateDates(mainEmploymentHistory,
                    "startDate", mainEmploymentHistory.getStartDate(),
                    "endDate", mainEmploymentHistory.getEndDate(), msgCtx);
            }
        }

        // Field validation: salary
        if (mainEmploymentHistory.testSalaryModified())
        {
            GenericValidator.validateRange(mainEmploymentHistory, "salary",
                mainEmploymentHistory.getSalary(),
                ValidationConstants.EMPLOYMENTHISTORY_SALARY_MIN,
                ValidationConstants.EMPLOYMENTHISTORY_SALARY_MAX, msgCtx);
        }
    }
    
}