/*
 * Created on 7-Oct-2005
 */
package com.ibm.rpm.resource.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.ProjectHistory;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author dominiqu
 */
public class ProjectHistoryCheckpoint extends AbstractCheckpoint
{
    private static final Class PROJECTHISTORY_PARENTS = Resource.class;

    protected static ProjectHistoryCheckpoint instance = new ProjectHistoryCheckpoint();

    public static ProjectHistoryCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, PROJECTHISTORY_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, ProjectHistory.class,
            msgCtx))
            return;

        // Initialize
        ProjectHistory mainProjectHistory = (ProjectHistory) rpmObject;

        // Field validation: client
        if (mainProjectHistory.testClientModified())
        {
            GenericValidator.validateMaxLength(mainProjectHistory,
                "client", mainProjectHistory.getClient(),
                ValidationConstants.PROJECTHISTORY_CLIENT_MAX, msgCtx);
        }

        // Field validation: description
        if (mainProjectHistory.testDescriptionModified())
        {
            GenericValidator.validateMaxLength(mainProjectHistory,
                "description", mainProjectHistory.getDescription(),
                ValidationConstants.PROJECTHISTORY_DESCRIPTION_MAX, msgCtx);
        }

        // Field validation: projectLength
        if (mainProjectHistory.testProjectLengthModified())
        {
            GenericValidator.validateMaxLength(mainProjectHistory,
                "projectLength", mainProjectHistory.getProjectLength(),
                ValidationConstants.PROJECTHISTORY_PROJECTLENGTH_MAX, msgCtx);
        }

        // Field validation: timeSpent
        if (mainProjectHistory.testTimeSpentModified())
        {
            GenericValidator.validateMaxLength(mainProjectHistory,
                "timeSpent", mainProjectHistory.getTimeSpent(),
                ValidationConstants.PROJECTHISTORY_TIMESPENT_MAX, msgCtx);
        }

        // Field validation: role
        if (mainProjectHistory.testRoleModified())
        {
            GenericValidator.validateMaxLength(mainProjectHistory,
                "role", mainProjectHistory.getRole(),
                ValidationConstants.PROJECTHISTORY_ROLE_MAX, msgCtx);
        }

        // Field validation: skillsResponsibilities
        if (mainProjectHistory.testSkillsResponsibilitiesModified())
        {
            GenericValidator.validateMaxLength(mainProjectHistory,
                "skillsResponsibilities", mainProjectHistory
                    .getSkillsResponsibilities(),
                ValidationConstants.PROJECTHISTORY_SKILLSRESPONSIBILITIES_MAX,
                msgCtx);
        }

        // Field validation: projectName
        if (mainProjectHistory.testProjectNameModified())
        {
            GenericValidator.validateMaxLength(mainProjectHistory,
                "projectName", mainProjectHistory.getProjectName(),
                ValidationConstants.PROJECTHISTORY_PROJECTNAME_MAX, msgCtx);
        }

        // Range validation: startDate
        if (mainProjectHistory.testStartDateModified())
        {
            GenericValidator.validateRange(mainProjectHistory,
                ValidationConstants.PROJECTHISTORY_START_DATE_FIELD,
                mainProjectHistory.getStartDate(),
                ValidationConstants.DATE_FIELD_MIN,
                ValidationConstants.DATE_FIELD_MAX, msgCtx);
        }
        
        // Range validation: endDate
        if (mainProjectHistory.testEndDateModified())
        {
            GenericValidator.validateRange(mainProjectHistory,
                ValidationConstants.PROJECTHISTORY_END_DATE_FIELD,
                mainProjectHistory.getEndDate(),
                ValidationConstants.DATE_FIELD_MIN,
                ValidationConstants.DATE_FIELD_MAX, msgCtx);
        }
        
        // Field validation: startDate and endDate
        if (mainProjectHistory.getStartDate() != null
                && mainProjectHistory.getEndDate() != null)
        {
            if (mainProjectHistory.testStartDateModified()
                    || mainProjectHistory.testEndDateModified())
            {
                GenericValidator.validateDates(mainProjectHistory,
                    "startDate", mainProjectHistory.getStartDate(), "endDate",
                    mainProjectHistory.getEndDate(), msgCtx);
            }
        }
    }
}