/*
 * Created on Sep 16, 2005
 */
package com.ibm.rpm.resource.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;

/**
 * @author gosselin
 */
public class ResourceAttributeAssignmentCheckpoint extends AbstractCheckpoint
{
    private static final Class RESOURCEATTRIBUTEASSIGNMENT_PARENTS = Resource.class;

    protected static ResourceAttributeAssignmentCheckpoint instance = new ResourceAttributeAssignmentCheckpoint();

    public static ResourceAttributeAssignmentCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true,
            RESOURCEATTRIBUTEASSIGNMENT_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject,
            ResourceAttributeAssignment.class, msgCtx))
            return;

        // Initialize
        ResourceAttributeAssignment mainRAA = (ResourceAttributeAssignment) rpmObject;

        // Field validation: proficiencyLevel
        GenericValidator.validateMandatoryID(mainRAA, "proficiencyLevel", mainRAA
            .getProficiencyLevel(), msgCtx);

        // Field validation: experience
        // Must not be null, and have ID on ResourceAttributeAssignment update
        if (mainRAA.getID() != null)
        {
            GenericValidator.validateMandatoryID(mainRAA, "experience", mainRAA
                .getExperience(), msgCtx);
        }

        // Field validation: preferenceRank
        // Must not be null, and have ID on ResourceAttributeAssignment update
        if (mainRAA.getID() != null)
        {
            GenericValidator.validateMandatoryID(mainRAA, "preferenceRank", mainRAA
                .getPreferenceRank(), msgCtx);
        }

        // Note: Resource setModified does not set the following fields as
        // modified so we cannot check if they have been modified before
        // validating

        // Field validation: description
        GenericValidator.validateMaxLength(mainRAA, "description",
            mainRAA.getDescription(),
            ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX, msgCtx);

        // Field validation: lastUsedLocation
        GenericValidator.validateMaxLength(mainRAA, "lastUsedLocation",
            mainRAA.getLastUsedLocation(),
            ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX, msgCtx);

        // Field validation: mandatoryDetails
        GenericValidator.validateMaxLength(mainRAA, "mandatoryDetails",
            mainRAA.getMandatoryDetails(),
            ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX, msgCtx);

        // Field validation: rates
        if (mainRAA.getRates() != null)
        {
            isValidParent(mainRAA.getRates(),
                msgCtx, true, ResourceAttributeAssignment.class);
        }

        // Field validation: defaultFlag
        GenericValidator.validateRange(mainRAA, "defaultFlag", mainRAA
            .getDefaultFlag(), 0, 1, msgCtx);

        // Field validation: strategicFlag
        GenericValidator.validateRange(mainRAA, "strategicFlag", mainRAA
            .getStrategicFlag(), 0, 1, msgCtx);
    }
    
}
