/*
 * Created on Sep 16, 2005
 */
package com.ibm.rpm.resource.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.ResourceAttribute;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetency;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceAttributeSkill;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillClassification;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceModule;

public class ResourceAttributeCheckpoint extends AbstractCheckpoint
{
    private static final List RESOURCEATTRIBUTECOMPETENCYCATEGORY_PARENTS = new ArrayList();

    private static final Class RESOURCEATTRIBUTECOMPETENCYCLASSIFICATION_PARENTS = ResourceAttributeCompetencyCategory.class;

    private static final Class RESOURCEATTRIBUTECOMPETENCY_PARENTS = ResourceAttributeCompetencyClassification.class;

    private static final Class RESOURCEATTRIBUTECOMPETENCYPROFICIENCYLEVEL_PARENTS = ResourceAttributeCompetency.class;

    private static final List RESOURCEATTRIBUTESKILLCATEGORY_PARENTS = new ArrayList();

    private static final Class RESOURCEATTRIBUTESKILLCLASSIFICATION_PARENTS = ResourceAttributeSkillCategory.class;

    private static final Class RESOURCEATTRIBUTESKILL_PARENTS = ResourceAttributeSkillClassification.class;

    private static final Class RESOURCEATTRIBUTESKILLPROFICIENCYLEVEL_PARENTS = ResourceAttributeSkill.class;

    static
    {
        RESOURCEATTRIBUTECOMPETENCYCATEGORY_PARENTS.add(ResourceModule.class);
        RESOURCEATTRIBUTECOMPETENCYCATEGORY_PARENTS
            .add(ResourceAttributeCompetencyCategory.class);

        RESOURCEATTRIBUTESKILLCATEGORY_PARENTS.add(ResourceModule.class);
        RESOURCEATTRIBUTESKILLCATEGORY_PARENTS
            .add(ResourceAttributeSkillCategory.class);
    }

    protected static ResourceAttributeCheckpoint instance = new ResourceAttributeCheckpoint();

    public static ResourceAttributeCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        if (!GenericValidator.validateInstance(rpmObject,
            ResourceAttribute.class, msgCtx))
            return;

        // Initialize
        ResourceAttribute mainResAttr = (ResourceAttribute) rpmObject;

        // Parent validation
        if (mainResAttr instanceof ResourceAttributeCompetencyCategory)
        {
            super.isValidParent(rpmObject, msgCtx, false,
                RESOURCEATTRIBUTECOMPETENCYCATEGORY_PARENTS);
        }
        else if (mainResAttr instanceof ResourceAttributeCompetencyClassification)
        {
            super.isValidParent(rpmObject, msgCtx, true,
                RESOURCEATTRIBUTECOMPETENCYCLASSIFICATION_PARENTS);
        }
        else if (mainResAttr instanceof ResourceAttributeCompetency)
        {
            super.isValidParent(rpmObject, msgCtx, true,
                RESOURCEATTRIBUTECOMPETENCY_PARENTS);
        }
        else if (mainResAttr instanceof ResourceAttributeCompetencyProficiencyLevel)
        {
            super.isValidParent(rpmObject, msgCtx, true,
                RESOURCEATTRIBUTECOMPETENCYPROFICIENCYLEVEL_PARENTS);
            
            // Validation for mandatory fields: one of name or proficiency level (with ID) must be specified
            if (mainResAttr.getID() == null && mainResAttr.getName() == null)
            {
                GenericValidator.validateMandatoryID(mainResAttr, 
                    ValidationConstants.RESOURCEATTRIBUTECOMPETENCYPROFICIENCYLEVEL_PROFICIENCY_LEVEL_FIELD,
                    ((ResourceAttributeCompetencyProficiencyLevel) mainResAttr).getProficiencyLevel(), msgCtx);
            }
        }
        else if (mainResAttr instanceof ResourceAttributeSkillCategory)
        {
            super.isValidParent(rpmObject, msgCtx, false,
                RESOURCEATTRIBUTESKILLCATEGORY_PARENTS);
        }
        else if (mainResAttr instanceof ResourceAttributeSkillClassification)
        {
            super.isValidParent(rpmObject, msgCtx, true,
                RESOURCEATTRIBUTESKILLCLASSIFICATION_PARENTS);
        }
        else if (mainResAttr instanceof ResourceAttributeSkill)
        {
            super.isValidParent(rpmObject, msgCtx, true,
                RESOURCEATTRIBUTESKILL_PARENTS);
        }
        else if (mainResAttr instanceof ResourceAttributeSkillProficiencyLevel)
        {
            super.isValidParent(rpmObject, msgCtx, true,
                RESOURCEATTRIBUTESKILLPROFICIENCYLEVEL_PARENTS);
            
            // Validation for mandatory fields on creation: one of name or proficiency level (with ID) must be specified
            if (mainResAttr.getID() == null && mainResAttr.getName() == null)
            {
                GenericValidator.validateMandatoryID(mainResAttr, 
                    ValidationConstants.RESOURCEATTRIBUTESKILLPROFICIENCYLEVEL_PROFICIENCY_LEVEL_FIELD,
                    ((ResourceAttributeSkillProficiencyLevel) mainResAttr).getProficiencyLevel(), msgCtx);
            }
        }

        // Field validation: name
        if (mainResAttr.testNameModified())
        {
            GenericValidator.validateMaxLength(mainResAttr, "name", mainResAttr
                .getName(), ValidationConstants.RESOURCEATTRIBUTE_NAME_MAX,
                msgCtx);
        }
    }

}
