/*
 * Created on Sep 15, 2005
 */
package com.ibm.rpm.resource.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.document.checkpoints.DocumentElementCheckpoint;
import com.ibm.rpm.document.containers.DelDoc;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.containers.Software;
import com.ibm.rpm.document.containers.UrlDocument;
import com.ibm.rpm.document.containers.WPDoc;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.resource.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.LocationPreferences;
import com.ibm.rpm.resource.containers.Passport;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.Visa;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.resource.util.ResourceValidator;

public class ResourceCheckpoint extends AbstractCheckpoint
{
    private static final Class RESOURCE_PARENTS = Pool.class;

    private static final List RESOURCE_CHILDREN_DOCUMENT_LIST = new ArrayList();

    protected static ResourceCheckpoint instance = new ResourceCheckpoint();

    static
    {
        RESOURCE_CHILDREN_DOCUMENT_LIST.add(DocumentFolder.class);
        RESOURCE_CHILDREN_DOCUMENT_LIST.add(DelDoc.class);
        RESOURCE_CHILDREN_DOCUMENT_LIST.add(Document.class);
        RESOURCE_CHILDREN_DOCUMENT_LIST.add(Software.class);
        RESOURCE_CHILDREN_DOCUMENT_LIST.add(UrlDocument.class);
        RESOURCE_CHILDREN_DOCUMENT_LIST.add(WPDoc.class);
    }

    public static ResourceCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, RESOURCE_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject, Resource.class,
            msgCtx))
            return;

        // Initialize
        Resource mainResource = (Resource) rpmObject;
        ResourceScope scope = (ResourceScope) rpmScope;

        // Validate nesting level - not yet tested thus commented
        //GenericValidator.validateNestingLevel(mainResource, "TMT_POOLS",
        //    ValidationConstants.RESOURCEMANAGEMENT_NESTINGLEVEL_MAX, msgCtx);
        
        // Field validation: userName
        // userName is only mandatory if the resource already exists
        if (mainResource.getID() != null)
        {
            GenericValidator.validateMandatory(mainResource, "userName",
                mainResource.getUserName(), msgCtx);
        }

        if (mainResource.testUserNameModified())
        {
            GenericValidator.validateMinLength(mainResource, "userName",
                mainResource.getUserName(),
                ValidationConstants.RESOURCE_USERNAME_MIN, msgCtx);

            GenericValidator.validateMaxLength(mainResource, "userName",
                mainResource.getUserName(),
                ValidationConstants.RESOURCE_USERNAME_MAX, msgCtx);
        }

        GenericValidator
            .validateCharacters(
                mainResource,
                "userName",
                mainResource.getUserName(),
                com.ibm.rpm.framework.constants.ValidationConstants.GENERIC_INVALIDCHARS,
                msgCtx);
        GenericValidator.validateLeadingSpace(mainResource, "userName",
            mainResource.getUserName(), msgCtx);
        GenericValidator.validateUnique(mainResource, "userName", mainResource
            .getUserName(), "RESOURCES", "LOGON_NAME", "RESOURCE_ID",
            mainResource.getID(), msgCtx);

        // Field validation: address1
        if (mainResource.testAddress1Modified())
        {
            GenericValidator.validateMaxLength(mainResource, "address1",
                mainResource.getAddress1(),
                ValidationConstants.RESOURCE_ADDRESS1_MAX, msgCtx);
        }

        // Field validation: address2
        if (mainResource.testAddress2Modified())
        {
            GenericValidator.validateMaxLength(mainResource, "address2",
                mainResource.getAddress2(),
                ValidationConstants.RESOURCE_ADDRESS2_MAX, msgCtx);
        }

        // Field validation: alternativePhone
        if (mainResource.testAlternativePhoneModified())
        {
            GenericValidator.validateMaxLength(mainResource,
                "alternativePhone", mainResource.getAlternativePhone(),
                ValidationConstants.RESOURCE_ALTERNATIVEPHONE_MAX, msgCtx);
        }

        // Field validation: companyCode
        if (mainResource.testCompanyCodeModified())
        {
            GenericValidator.validateMaxLength(mainResource, "companyCode",
                mainResource.getCompanyCode(),
                ValidationConstants.RESOURCE_COMPANYCODE_MAX, msgCtx);
        }

        // Field validation: countryCode
        if (mainResource.testCountryCodeModified())
        {
            GenericValidator.validateMaxLength(mainResource, "countryCode",
                mainResource.getCountryCode(),
                ValidationConstants.RESOURCE_COUNTRYCODE_MAX, msgCtx);
        }

        // Field validation: custom1
        if (mainResource.testCustom1Modified())
        {
            GenericValidator.validateMaxLength(mainResource, "custom1",
                mainResource.getCustom1(),
                ValidationConstants.RESOURCE_CUSTOM_MAX, msgCtx);
        }

        // Field validation: custom2
        if (mainResource.testCustom2Modified())
        {
            GenericValidator.validateMaxLength(mainResource, "custom2",
                mainResource.getCustom2(),
                ValidationConstants.RESOURCE_CUSTOM_MAX, msgCtx);
        }

        // Field validation: custom3
        if (mainResource.testCustom3Modified())
        {
            GenericValidator.validateMaxLength(mainResource, "custom3",
                mainResource.getCustom3(),
                ValidationConstants.RESOURCE_CUSTOM_MAX, msgCtx);
        }

        // Field validation: custom4
        if (mainResource.testCustom4Modified())
        {
            GenericValidator.validateMaxLength(mainResource, "custom4",
                mainResource.getCustom4(),
                ValidationConstants.RESOURCE_CUSTOM_MAX, msgCtx);
        }

        // Field validation: custom5
        if (mainResource.testCustom5Modified())
        {
            GenericValidator.validateMaxLength(mainResource, "custom5",
                mainResource.getCustom5(),
                ValidationConstants.RESOURCE_CUSTOM_MAX, msgCtx);
        }

        // TODO RESOURCE_DASHBOARD_MAX if not readonly

        // Field validation: departmentCode
        if (mainResource.testDepartmentCodeModified())
        {
            GenericValidator.validateMaxLength(mainResource, "departmentCode",
                mainResource.getDepartmentCode(),
                ValidationConstants.RESOURCE_DEPARTMENTCODE_MAX, msgCtx);
        }

        // Field validation: divisionCode
        if (mainResource.testDivisionCodeModified())
        {
            GenericValidator.validateMaxLength(mainResource, "divisionCode",
                mainResource.getDivisionCode(),
                ValidationConstants.RESOURCE_DIVISIONCODE_MAX, msgCtx);
        }

        // Field validation: emailAddress
        if (mainResource.testEmailAddressModified())
        {
            GenericValidator.validateMaxLength(mainResource, "emailAddress",
                mainResource.getEmailAddress(),
                ValidationConstants.RESOURCE_EMAILADDRESS_MAX, msgCtx);
        }

        // Field validation: employeeCode
        if (mainResource.testEmployeeCodeModified())
        {
            GenericValidator.validateMaxLength(mainResource, "employeeCode",
                mainResource.getEmployeeCode(),
                ValidationConstants.RESOURCE_EMPLOYEECODE_MAX, msgCtx);
        }

        // Field validation: externalID
        if (mainResource.testExternalIDModified())
        {
            GenericValidator.validateMaxLength(mainResource, "externalID",
                mainResource.getExternalID(),
                ValidationConstants.RESOURCE_EXTERNALID_MAX, msgCtx);
        }

        // Field validation: familyName
        if (mainResource.testFamilyNameModified())
        {
            GenericValidator.validateMaxLength(mainResource, "familyName",
                mainResource.getFamilyName(),
                ValidationConstants.RESOURCE_FAMILYNAME_MAX, msgCtx);
        }

        // Field validation: fax
        if (mainResource.testFaxModified())
        {
            GenericValidator.validateMaxLength(mainResource, "fax",
                mainResource.getFax(), ValidationConstants.RESOURCE_FAX_MAX,
                msgCtx);
        }

        // Field validation: fullName
        GenericValidator.validateMandatory(mainResource, "fullName",
            mainResource.getFullName(), msgCtx);

        if (mainResource.testFullNameModified())
        {
            GenericValidator.validateMinLength(mainResource, "fullName",
                mainResource.getFullName(),
                ValidationConstants.RESOURCE_FULLNAME_MIN, msgCtx);

            GenericValidator.validateMaxLength(mainResource, "fullName",
                mainResource.getFullName(),
                ValidationConstants.RESOURCE_FULLNAME_MAX, msgCtx);
        }

        GenericValidator
            .validateCharacters(
                mainResource,
                "fullName",
                mainResource.getFullName(),
                com.ibm.rpm.framework.constants.ValidationConstants.GENERIC_INVALIDCHARS,
                msgCtx);

        // Field validation: givenName
        if (mainResource.testGivenNameModified())
        {
            GenericValidator.validateMaxLength(mainResource, "givenName",
                mainResource.getGivenName(),
                ValidationConstants.RESOURCE_GIVENNAME_MAX, msgCtx);
        }

        // Field validation: homePhone
        if (mainResource.testHomePhoneModified())
        {
            GenericValidator.validateMaxLength(mainResource, "homePhone",
                mainResource.getHomePhone(),
                ValidationConstants.RESOURCE_HOMEPHONE_MAX, msgCtx);
        }

        // Field validation: interviewExperience
        if (mainResource.testInterviewExperienceModified())
        {
            GenericValidator.validateMaxLength(mainResource,
                "interviewExperience", mainResource.getInterviewExperience(),
                ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX, msgCtx);
        }

        // TODO RESOURCE_LOGONREMINDER_MAX if not readonly

        // Field validation: middleName
        if (mainResource.testMiddleNameModified())
        {
            GenericValidator.validateMaxLength(mainResource, "middleName",
                mainResource.getMiddleName(),
                ValidationConstants.RESOURCE_MIDDLENAME_MAX, msgCtx);
        }

        // Field validation: mobile
        if (mainResource.testMobileModified())
        {
            GenericValidator.validateMaxLength(mainResource, "mobile",
                mainResource.getMobile(),
                ValidationConstants.RESOURCE_MOBILE_MAX, msgCtx);
        }

        // Field validation: myAchievements
        if (mainResource.testMyAchievementsModified())
        {
            GenericValidator.validateMaxLength(mainResource, "myAchievements",
                mainResource.getMyAchievements(),
                ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX, msgCtx);
        }

        // Field validation: myPersonalPassions
        if (mainResource.testMyPersonalPassionsModified())
        {
            GenericValidator.validateMaxLength(mainResource,
                "myPersonalPassions", mainResource.getMyPersonalPassions(),
                ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX, msgCtx);
        }

        // Field validation: myPortalURL1
        GenericValidator.validateMaxLength(mainResource,
            ValidationConstants.RESOURCE_MY_PORTAL_URL1_FIELD, mainResource
                .getMyPortalURL1(),
            ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX, msgCtx);
        
        // Field validation: myPortalURL2
        GenericValidator.validateMaxLength(mainResource,
            ValidationConstants.RESOURCE_MY_PORTAL_URL2_FIELD, mainResource
                .getMyPortalURL2(),
            ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX, msgCtx);
        
        // Field validation: myPortalURL3
        GenericValidator.validateMaxLength(mainResource,
            ValidationConstants.RESOURCE_MY_PORTAL_URL3_FIELD, mainResource
                .getMyPortalURL3(),
            ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX, msgCtx);
        
        // Field validation: myPortalURL4
        GenericValidator.validateMaxLength(mainResource,
            ValidationConstants.RESOURCE_MY_PORTAL_URL4_FIELD, mainResource
                .getMyPortalURL4(),
            ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX, msgCtx);
        
        // Field validation: myPortalURL5
        GenericValidator.validateMaxLength(mainResource,
            ValidationConstants.RESOURCE_MY_PORTAL_URL5_FIELD, mainResource
                .getMyPortalURL5(),
            ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX, msgCtx);

        // Field validation: mySummaryOfSkills
        if (mainResource.testMySummaryOfSkillsModified())
        {
            GenericValidator.validateMaxLength(mainResource,
                "mySummaryOfSkills", mainResource.getMySummaryOfSkills(),
                ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX, msgCtx);
        }

        // Field validation: myWebsite
        if (mainResource.testMyWebsiteModified())
        {
            GenericValidator.validateMaxLength(mainResource, "myWebsite",
                mainResource.getMyWebsite(),
                ValidationConstants.RESOURCE_MYWEBSITE_MAX, msgCtx);
        }

        // Field validation: nickName
        if (mainResource.testNickNameModified())
        {
            GenericValidator.validateMaxLength(mainResource, "nickName",
                mainResource.getNickName(),
                ValidationConstants.RESOURCE_NICKNAME_MAX, msgCtx);

            GenericValidator
                .validateCharacters(
                    mainResource,
                    "nickName",
                    mainResource.getNickName(),
                    com.ibm.rpm.framework.constants.ValidationConstants.GENERIC_INVALIDCHARS,
                    msgCtx);
        }

        // Field validation: pager
        if (mainResource.testPagerModified())
        {
            GenericValidator.validateMaxLength(mainResource, "pager",
                mainResource.getPager(),
                ValidationConstants.RESOURCE_PAGER_MAX, msgCtx);
        }

        // Field validation: password
        if (mainResource.testPasswordModified())
        {
            GenericValidator.validateMaxLength(mainResource, "password",
                mainResource.getPassword(),
                ValidationConstants.RESOURCE_PASSWORD_MAX, msgCtx);
        }

        if (mainResource.testPasswordModified()
                || mainResource.testFullNameModified())
        {
            GenericValidator.validateFieldsNotTheSame(mainResource, "fullName",
                mainResource.getFullName(), "password", mainResource
                    .getPassword(), msgCtx);
        }

        // Field validation: title
        if (mainResource.testTitleModified())
        {
            GenericValidator.validateMaxLength(mainResource, "title",
                mainResource.getTitle(),
                ValidationConstants.RESOURCE_TITLE_MAX, msgCtx);
        }

        // Field validation: weeklyMaxHours
        if (mainResource.testWeeklyMaxHoursModified())
        {
            GenericValidator.validateRange(mainResource, "weeklyMaxHours",
                mainResource.getWeeklyMaxHours(),
                ValidationConstants.RESOURCE_WEEKLYMAXHOURS_MIN,
                ValidationConstants.RESOURCE_WEEKLYMAXHOURS_MAX, msgCtx);
        }

        // Field validation: weeklyMinHours
        if (mainResource.testWeeklyMinHoursModified())
        {
            GenericValidator.validateRange(mainResource, "weeklyMinHours",
                mainResource.getWeeklyMaxHours(),
                ValidationConstants.RESOURCE_WEEKLYMINHOURS_MIN,
                ValidationConstants.RESOURCE_WEEKLYMINHOURS_MAX, msgCtx);
        }

        // Field validation: whatIDoReallyWell
        if (mainResource.testWhatIDoReallyWellModified())
        {
            GenericValidator.validateMaxLength(mainResource,
                "whatIDoReallyWell", mainResource.getWhatIDoReallyWell(),
                ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX, msgCtx);
        }

        // Field validation: workPhone
        if (mainResource.testWorkPhoneModified())
        {
            GenericValidator.validateMaxLength(mainResource, "workPhone",
                mainResource.getWorkPhone(),
                ValidationConstants.RESOURCE_WORKPHONE_MAX, msgCtx);
        }

        // Field validation: zipPostalCode
        if (mainResource.testZipPostalCodeModified())
        {
            GenericValidator.validateMaxLength(mainResource, "zipPostalCode",
                mainResource.getZipPostalCode(),
                ValidationConstants.RESOURCE_ZIPPOSTALCODE_MAX, msgCtx);
        }

        // Field validation: effectiveStartDate and discontinueDate
        if (mainResource.testEffectiveStartDateModified()
                || mainResource.testDiscontinueDateModified())
        {
            GenericValidator.validateDates(mainResource, "effectiveStartDate",
                mainResource.getEffectiveStartDate(), "discontinueDate",
                mainResource.getDiscontinueDate(), msgCtx);
        }

        // reVerified and verified Validation
        if (mainResource.getVerifiedBy() == null
                && mainResource.getReVerifiedBy() != null)
        {
            RPMException e = new RPMException(
                ErrorCodes.REVERIFIEDBY_SPECIFIED_WITHOUT_A_VERIFIEDBY_RESOURCE,
                mainResource.getClass().getName());
            addException(e, msgCtx);
        }

        // verifiedBy Validation
        if (mainResource.testVerifiedByModified())
        {
            if (mainResource.getVerifiedBy() != null)
            {
                // Verify that verifiedBy is a valid resource id
                ResourceValidator.validateExistingResource(mainResource,
                    "verifiedBy", mainResource.getVerifiedBy().getID(), "ID",
                    msgCtx);
            }
        }

        // reVerifiedBy Validation
        if (mainResource.testReVerifiedByModified())
        {
            if (mainResource.getReVerifiedBy() != null)
            {
                // Verify that reVerifiedBy is a valid resource id
                ResourceValidator.validateExistingResource(mainResource,
                    "reVerifiedBy", mainResource.getReVerifiedBy().getID(),
                    "ID", msgCtx);
            }
        }

        if (scope != null)
        {            
            // Note: Resource setModified does not set the inner fields of children
            // classes as
            //       modified so we cannot check if they have been modified before
            // validating

            // Location Preferences Validation
            if (scope.isLocationPreferences()
                    && mainResource.testLocationPreferencesModified())
            {
                LocationPreferences mainLP = mainResource
                    .getLocationPreferences();

                super.isValidParent(mainLP, msgCtx, false, false,
                    Resource.class);

                // Relocation
                if (GenericValidator.getBoolean(mainLP.getWillingtoRelocate()))
                {
                    // Field validation: relocateLocations
                    if (mainLP.getRelocateLocations() != null)
                    {
                        GenericValidator
                            .validateMaxSize(
                                mainLP,
                                "relocateLocations",
                                mainLP.getRelocateLocations().length,
                                ValidationConstants.LOCATIONPREFERENCES_RELOCATELOCATIONS_MAX,
                                msgCtx);
                    }

                    // Field validation: acceptableRelocateDuration
                    GenericValidator
                        .validateRange(
                            mainLP,
                            "acceptableRelocateDuration",
                            mainLP.getAcceptableRelocateDuration(),
                            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLERELOCATEDURATION_MIN,
                            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLERELOCATEDURATION_MAX,
                            msgCtx);
                }
                else
                {
                    // Field validation: relocateLocations
                    GenericValidator.validateMustBeNull(mainLP,
                        "relocateLocations", mainLP.getRelocateLocations(),
                        "willingToRelocate is false", msgCtx);

                    // Field validation: acceptableRelocateDuration
                    GenericValidator.validateMustBeNull(mainLP,
                        "acceptableRelocateDuration", mainLP
                            .getAcceptableRelocateDuration(),
                        "willingToRelocate is false", msgCtx);
                }

                // Travel
                if (GenericValidator.getBoolean(mainLP.getWillingtoTravel()))
                {
                    // Field validation: travelLocations
                    if (mainLP.getTravelLocations() != null)
                    {
                        GenericValidator
                            .validateMaxSize(
                                mainLP,
                                "travelLocations",
                                mainLP.getTravelLocations().length,
                                ValidationConstants.LOCATIONPREFERENCES_TRAVELLOCATIONS_MAX,
                                msgCtx);
                    }

                    // Field validation: acceptableTravelDuration
                    GenericValidator
                        .validateRange(
                            mainLP,
                            "acceptableTravelDuration",
                            mainLP.getAcceptableTravelDuration(),
                            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLETRAVELDURATION_MIN,
                            ValidationConstants.LOCATIONPREFERENCES_ACCEPTABLETRAVELDURATION_MAX,
                            msgCtx);
                }
                else
                {
                    // Field validation: travelLocations
                    GenericValidator.validateMustBeNull(mainLP,
                        "travelLocations", mainLP.getTravelLocations(),
                        "willingToTravel is false", msgCtx);

                    // Field validation: acceptableTravelDuration
                    GenericValidator.validateMustBeNull(mainLP,
                        "acceptableTravelDuration", mainLP
                            .getAcceptableTravelDuration(),
                        "willingToTravel is false", msgCtx);
                }
            }

            // Passport & Visa Validation
            if (scope.isPassport() && mainResource.testPassportModified())
            {
                Passport mainPassport = mainResource.getPassport();

                super.isValidParent(mainPassport, msgCtx, false, false,
                    Resource.class);

                if (GenericValidator.getBoolean(mainPassport
                    .getHasValidPassport()))
                {
                    // Field validation: visas
                    if (mainPassport.getVisas() != null)
                    {
                        if (GenericValidator.validateMaxSize(mainPassport,
                            "visas", mainPassport.getVisas().length,
                            ValidationConstants.PASSPORT_VISAS_MAX, msgCtx))
                        {
                            // Verify that the Visa objects are valid
                            Visa[] visas = mainPassport.getVisas();
                            for (int i = 0; i < visas.length; ++i)
                                super.isValidParent(visas[i], msgCtx, false,
                                    false, Passport.class);
                        }
                    }
                }
                else
                {
                    // Field validation: visas
                    GenericValidator.validateMustBeNull(mainPassport, "visas",
                        mainPassport.getVisas(), "hasValidPassport is false",
                        msgCtx);
                }
            }
            
            if (scope.getDocumentFolder() != null)
            {
                // DocumentElements validation
                DocumentElementCheckpoint.validateDocumentElements(
                    mainResource, new DocumentElement[]
                    {
                        mainResource.getDocumentFolder()
                    }, RESOURCE_CHILDREN_DOCUMENT_LIST, msgCtx);
            }
            
            if (scope.isCurrency())
            {                
                GenericValidator.validateDefaultValueUpdate(mainResource,
                    ValidationConstants.RESOURCE_CURRENCY_FIELD, mainResource
                        .getCurrency(), msgCtx);
            }
            
            if (scope.isExperience())
            {
                GenericValidator.validateDefaultValueUpdate(mainResource,
                    ValidationConstants.RESOURCE_EXPERIENCE_FIELD, mainResource
                        .getExperience(), msgCtx);
            }
            
            if (scope.isWorkLocation())
            {
                GenericValidator.validateDefaultValueUpdate(mainResource,
                    ValidationConstants.RESOURCE_WORK_LOCATION_FIELD, mainResource
                        .getWorkLocation(), msgCtx);
            }
            if (scope.getCalendar() != null)
            {
                if (mainResource.getCalendar() != null)
                {
                    GenericValidator.validateMandatory(mainResource,
                        ValidationConstants.RESOURCE_CALENDAR_FIELD + ".ID",
                        mainResource.getCalendar().getID(), msgCtx);
                }
            }
            if (scope.getSecurityGroup() != null)
            {
                if (mainResource.getSecurityGroup() != null)
                {
                    GenericValidator.validateMandatory(mainResource,
                        ValidationConstants.RESOURCE_SECURITY_GROUP_FIELD + ".ID",
                        mainResource.getSecurityGroup().getID(), msgCtx);
                }
            }
            
        }

        // on update, the fields with default values cannot be cleared
        GenericValidator.validateDefaultValueUpdate(mainResource,
            ValidationConstants.RESOURCE_ACTIVE_FIELD, mainResource
                .getActive(), msgCtx);
        
        GenericValidator.validateDefaultValueUpdate(mainResource,
            ValidationConstants.RESOURCE_EFFECTIVE_START_DATE_FIELD, mainResource
                .getEffectiveStartDate(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(mainResource,
            ValidationConstants.RESOURCE_OVERRIDE_ORIGINAL_DOCUMENT_ON_CHECKOUT_FIELD, mainResource
                .getOverrideOriginalDocumentOnCheckout(), msgCtx);
        
        GenericValidator.validateDefaultValueUpdate(mainResource,
            ValidationConstants.RESOURCE_PASSWORD_FIELD, mainResource
                .getPassword(), msgCtx);
        
        GenericValidator.validateDefaultValueUpdate(mainResource,
            ValidationConstants.RESOURCE_WEEKLY_MAX_HOURS_FIELD, mainResource
                .getWeeklyMaxHours(), msgCtx);

    }
    
}