package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.Experience;
import com.ibm.rpm.applicationadministration.containers.PreferenceRank;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceAttributeProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceRates;


public class ResourceAttributeAssignmentManager extends AbstractRPMObjectManager{

	private static Log logger = LogFactory.getLog(ResourceAttributeAssignmentManager.class);
	
    private static final HashMap FIELDPROPERTYMAP = new HashMap();
    private static final HashSet CONTAINERS = new HashSet();

    static{
        CONTAINERS.add(ResourceAttributeAssignment.class.getName());
        
        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ResourceAttributeAssignmentManager.class);
        }
    }
    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 1;
    public static final int TYPE_LEVEL_ID = Types.INTEGER;
    public static final String NAME_LEVEL_ID = "TMT_COMPETENCIES.LEVEL_ID";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_LEVEL_ID = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 2;
    public static final int TYPE_TYPE_ID = Types.INTEGER;
    public static final String NAME_TYPE_ID = "TMT_COMPETENCIES.TYPE_ID";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_TYPE_ID = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 3;
    public static final int TYPE_MAX_RANK = Types.INTEGER;
    public static final String NAME_MAX_RANK = "TMT_COMPETENCIES.MAX_RANK";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_MAX_RANK = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 4;
    public static final int TYPE_CHILD_COUNT = Types.INTEGER;
    public static final String NAME_CHILD_COUNT = "TMT_COMPETENCIES.CHILD_COUNT";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_CHILD_COUNT = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 5;
    public static final int TYPE_DELETED_COUNT = Types.INTEGER;
    public static final String NAME_DELETED_COUNT = "TMT_COMPETENCIES.DELETED_COUNT";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_DELETED_COUNT = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 6;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "TMT_COMPETENCIES.ELEMENT_ID";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 7;
    public static final int TYPE_ELEMENT_NAME = Types.CHAR;
    public static final String NAME_ELEMENT_NAME = "TMT_COMPETENCIES.ELEMENT_NAME";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME,NAME_ELEMENT_NAME);}

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 8;
    public static final int TYPE_PARENT_ID = Types.CHAR;
    public static final String NAME_PARENT_ID = "TMT_COMPETENCIES.PARENT_ID";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_PARENT_ID = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 9;
    public static final int TYPE_RESOURCE_ID = Types.CHAR;
    public static final String NAME_RESOURCE_ID = "TMT_COMPETENCIES.RESOURCE_ID";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_RESOURCE_ID = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID,NAME_RESOURCE_ID);}

    /**
     * Column PROFICIENCY_LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROFICIENCY_LEVEL_ID = 10;
    public static final int TYPE_PROFICIENCY_LEVEL_ID = Types.INTEGER;
    public static final String NAME_PROFICIENCY_LEVEL_ID = "TMT_COMPETENCIES.PROFICIENCY_LEVEL_ID";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_PROFICIENCY_LEVEL_ID = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROFICIENCY_LEVEL_ID,NAME_PROFICIENCY_LEVEL_ID);}

    /**
     * Column EXPERIENCE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPERIENCE_ID = 11;
    public static final int TYPE_EXPERIENCE_ID = Types.INTEGER;
    public static final String NAME_EXPERIENCE_ID = "TMT_COMPETENCIES.EXPERIENCE_ID";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_EXPERIENCE_ID = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_EXPERIENCE_ID,NAME_EXPERIENCE_ID);}

    /**
     * Column PREFERENCE_LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PREFERENCE_LEVEL_ID = 12;
    public static final int TYPE_PREFERENCE_LEVEL_ID = Types.INTEGER;
    public static final String NAME_PREFERENCE_LEVEL_ID = "TMT_COMPETENCIES.PREFERENCE_LEVEL_ID";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_PREFERENCE_LEVEL_ID = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PREFERENCE_LEVEL_ID,NAME_PREFERENCE_LEVEL_ID);}

    /**
     * Column RATE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RATE_ID = 13;
    public static final int TYPE_RATE_ID = Types.CHAR;
    public static final String NAME_RATE_ID = "TMT_COMPETENCIES.RATE_ID";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_RATE_ID = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RATE_ID,NAME_RATE_ID);}

    /**
     * Column LAST_USED_DATETIME of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_LAST_USED_DATETIME = 14;
    public static final int TYPE_LAST_USED_DATETIME = Types.DATE;
    public static final String NAME_LAST_USED_DATETIME = "TMT_COMPETENCIES.LAST_USED_DATETIME";
    public static final String PROPERTY_LAST_USED_DATETIME = "LASTUSEDDATETIME";
    static{ FIELDPROPERTYMAP.put(PROPERTY_LAST_USED_DATETIME,NAME_LAST_USED_DATETIME);}

    /**
     * Column CREATED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_CREATED_BY = 15;
    public static final int TYPE_CREATED_BY = Types.CHAR;
    public static final String NAME_CREATED_BY = "TMT_COMPETENCIES.CREATED_BY";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_CREATED_BY = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CREATED_BY,NAME_CREATED_BY);}

    /**
     * Column CREATED_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CREATED_DATETIME = 16;
    public static final int TYPE_CREATED_DATETIME = Types.TIMESTAMP;
    public static final String NAME_CREATED_DATETIME = "TMT_COMPETENCIES.CREATED_DATETIME";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_CREATED_DATETIME = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CREATED_DATETIME,NAME_CREATED_DATETIME);}

    /**
     * Column DEFAULT_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_DEFAULT_FLAG = 17;
    public static final int TYPE_DEFAULT_FLAG = Types.CHAR;
    public static final String NAME_DEFAULT_FLAG = "TMT_COMPETENCIES.DEFAULT_FLAG";
    public static final String PROPERTY_DEFAULT_FLAG = "DEFAULTFLAG";
    static{ FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_FLAG, SqlUtil.getBooleanValue(NAME_DEFAULT_FLAG, "1"));}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 18;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "TMT_COMPETENCIES.REC_USER";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_REC_USER = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 19;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "TMT_COMPETENCIES.REC_STATUS";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_REC_STATUS = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 20;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "TMT_COMPETENCIES.REC_DATETIME";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_REC_DATETIME = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 21;
    public static final int TYPE_RANK = Types.VARCHAR;
    public static final String NAME_RANK = "TMT_COMPETENCIES.RANK";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    //public static final String PROPERTY_RANK = "propertyname";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RANK,NAME_RANK);}

    /**
     * Column LAST_USED_LOCATION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LAST_USED_LOCATION = 22;
    public static final int TYPE_LAST_USED_LOCATION = Types.VARCHAR;
    public static final String NAME_LAST_USED_LOCATION = "TMT_COMPETENCIES.LAST_USED_LOCATION";
    public static final String PROPERTY_LAST_USED_LOCATION = "LASTUSEDLOCATION";
    static{ FIELDPROPERTYMAP.put(PROPERTY_LAST_USED_LOCATION,NAME_LAST_USED_LOCATION);}

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 23;
    public static final int TYPE_DESCRIPTION = Types.VARCHAR;
    public static final String NAME_DESCRIPTION = "TMT_COMPETENCIES.DESCRIPTION";
    //uncomment and replace propertyname with the container property loaded from this column if it applies.
    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static{ FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION,NAME_DESCRIPTION);}

    /**
     * Column STRATEGIC_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_STRATEGIC_FLAG = 24;
    public static final int TYPE_STRATEGIC_FLAG = Types.CHAR;
    public static final String NAME_STRATEGIC_FLAG = "TMT_COMPETENCIES.STRATEGIC_FLAG";
    public static final String PROPERTY_STRATEGIC_FLAG = "STRATEGICFLAG";
    static{ FIELDPROPERTYMAP.put(PROPERTY_STRATEGIC_FLAG, SqlUtil.getBooleanValue(NAME_STRATEGIC_FLAG, "1"));}

    /**
     * Column TARGET_ATTAINMENT_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_TARGET_ATTAINMENT_DATE = 25;
    public static final int TYPE_TARGET_ATTAINMENT_DATE = Types.DATE;
    public static final String NAME_TARGET_ATTAINMENT_DATE = "TMT_COMPETENCIES.TARGET_ATTAINMENT_DATE";
    public static final String PROPERTY_TARGET_ATTAINMENT_DATE = "TARGETATTAINMENTDATE";
    static{ FIELDPROPERTYMAP.put(PROPERTY_TARGET_ATTAINMENT_DATE,NAME_TARGET_ATTAINMENT_DATE);}

    /**
     * Column ACQUIRED_ON of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_ACQUIRED_ON = 26;
    public static final int TYPE_ACQUIRED_ON = Types.DATE;
    public static final String NAME_ACQUIRED_ON = "TMT_COMPETENCIES.ACQUIRED_ON";
    public static final String PROPERTY_ACQUIRED_ON = "ACQUIREDON";
    static{ FIELDPROPERTYMAP.put(PROPERTY_ACQUIRED_ON,NAME_ACQUIRED_ON);}

    /**
     * Column MANDATORY of type Types.CHAR mapped to String.
     */
    public static final int ID_MANDATORY = 27;
    public static final int TYPE_MANDATORY = Types.CHAR;
    public static final String NAME_MANDATORY = "TMT_COMPETENCIES.MANDATORY";
    public static final String PROPERTY_MANDATORY = "MANDATORY";
    static{ FIELDPROPERTYMAP.put(PROPERTY_MANDATORY, SqlUtil.getBooleanValue(NAME_MANDATORY, "1"));}

    /**
     * Column MANDATORY_DETAILS of type Types.VARCHAR mapped to String.
     */
    public static final int ID_MANDATORY_DETAILS = 28;
    public static final int TYPE_MANDATORY_DETAILS = Types.VARCHAR;
    public static final String NAME_MANDATORY_DETAILS = "TMT_COMPETENCIES.MANDATORY_DETAILS";
    public static final String PROPERTY_MANDATORY_DETAILS = "MANDATORYDETAILS";
    static{ FIELDPROPERTYMAP.put(PROPERTY_MANDATORY_DETAILS,NAME_MANDATORY_DETAILS);}


    private static final String TABLE_NAME = "TMT_COMPETENCIES";
    
    /**
     * Create an array of type string containing all the fields of the TMT_COMPETENCIES table.
     */
    private static final String[] FIELD_NAMES =
    {
         NAME_LEVEL_ID
        ,NAME_TYPE_ID
        ,NAME_MAX_RANK
        ,NAME_CHILD_COUNT
        ,NAME_DELETED_COUNT
        ,NAME_ELEMENT_ID
        ,NAME_ELEMENT_NAME
        ,NAME_PARENT_ID
        ,NAME_RESOURCE_ID
        ,NAME_PROFICIENCY_LEVEL_ID
        ,NAME_EXPERIENCE_ID
        ,NAME_PREFERENCE_LEVEL_ID
        ,NAME_RATE_ID
        ,NAME_LAST_USED_DATETIME
        ,NAME_CREATED_BY
        ,NAME_CREATED_DATETIME
        ,NAME_DEFAULT_FLAG
        ,NAME_REC_USER
        ,NAME_REC_STATUS
        ,NAME_REC_DATETIME
        ,NAME_RANK
        ,NAME_LAST_USED_LOCATION
        ,NAME_DESCRIPTION
        ,NAME_STRATEGIC_FLAG
        ,NAME_TARGET_ATTAINMENT_DATE
        ,NAME_ACQUIRED_ON
        ,NAME_MANDATORY
        ,NAME_MANDATORY_DETAILS

    };

    private static final String ALL_FIELDS;
    static{
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for(int i = 1 ; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS= fieldcat.toString();
    };


    public ResourceAttributeAssignmentManager(){
        super();
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName() {
        return TABLE_NAME;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames(){
        return FIELD_NAMES;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
     protected String getAllFields() {
        return ALL_FIELDS;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
     public String getPrimaryKey() {
        return NAME_ELEMENT_ID;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws RPMException,
            SQLException {
        
        int typeId = resultSet.getInt(ID_TYPE_ID);        
        return new Integer(typeId);
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
     protected HashMap getPropertiesMap() {
        return FIELDPROPERTYMAP;
    }

     protected RPMObject createContainer(int typeId) throws RPMException {

        RPMObject object = null;

        switch (typeId) {
        	case ResourceAttributeAssignment.TYPE_ID : 
        		object = new ResourceAttributeAssignment();
        		break;
	        default:
	            ExceptionUtil.handleUnsupportedTypeId(this, typeId);
        }

        return object;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject pObject, boolean isDelta)
            throws RPMException, SQLException, ParseException {
     
    	ResourceAttributeAssignment assignment = (ResourceAttributeAssignment)pObject;
    	assignment.setID(rs.getString(ID_ELEMENT_ID));
    	
    	if(assignment.getRates()!=null && assignment.getRates().getID()==null){
    		assignment.getRates().setID(rs.getString(ID_RATE_ID));
    	}
    	
        String name = rs.getString(ID_ELEMENT_NAME);
        if(name!=null){
        	name = name.trim();
        }
    	
    	if(isDelta){
	    	assignment.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
	    	//extra columns
	    	assignment.deltaLastUsedDateTime(DateUtil.convertToCalendar(rs.getDate(ID_LAST_USED_DATETIME)));
	    	assignment.deltaDefaultFlag(new Integer(rs.getInt(ID_DEFAULT_FLAG)));
	    	assignment.deltaLastUsedLocation(rs.getString(ID_LAST_USED_LOCATION));
	    	assignment.deltaDescription(rs.getString(ID_DESCRIPTION));
	    	assignment.deltaStrategicFlag(rs.getString(ID_STRATEGIC_FLAG));
	    	assignment.deltaTargetAttainmentDate(DateUtil.convertToCalendar(rs.getDate(ID_TARGET_ATTAINMENT_DATE)));
	    	assignment.deltaAcquiredOn(DateUtil.convertToCalendar(rs.getDate(ID_ACQUIRED_ON)));
	    	assignment.deltaMandatory(rs.getString(ID_MANDATORY));
	    	assignment.deltaMandatoryDetails(rs.getString(ID_MANDATORY_DETAILS));

		} else {
	    	assignment.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
	    	//extra columns
	    	assignment.setLastUsedDateTime(DateUtil.convertToCalendar(rs.getDate(ID_LAST_USED_DATETIME)));
	    	assignment.setDefaultFlag(new Integer(rs.getInt(ID_DEFAULT_FLAG)));
	    	assignment.setLastUsedLocation(rs.getString(ID_LAST_USED_LOCATION));
	    	assignment.setDescription(rs.getString(ID_DESCRIPTION));
	    	assignment.setStrategicFlag(rs.getString(ID_STRATEGIC_FLAG));
	    	assignment.setTargetAttainmentDate(DateUtil.convertToCalendar(rs.getDate(ID_TARGET_ATTAINMENT_DATE)));
	    	assignment.setAcquiredOn(DateUtil.convertToCalendar(rs.getDate(ID_ACQUIRED_ON)));
	    	assignment.setMandatory(rs.getString(ID_MANDATORY));
	    	assignment.setMandatoryDetails(rs.getString(ID_MANDATORY_DETAILS));
			
		}

        return pObject;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet, com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException, SQLException{
    
        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        String resourceId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, resourceId);

        String experienceId = resultSet.getString(ID_EXPERIENCE_ID);
        relatedFields.put(index, ID_EXPERIENCE_ID, experienceId);

        String prefLevelId = resultSet.getString(ID_PREFERENCE_LEVEL_ID);
        relatedFields.put(index, ID_PREFERENCE_LEVEL_ID, prefLevelId);
        
        String rateId = resultSet.getString(ID_RATE_ID);
        relatedFields.put(index, ID_RATE_ID, rateId );    
        
        return relatedFields;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext) throws RPMException, SQLException {
        if(rpmObject != null){
        	ResourceAttributeAssignment assignment = (ResourceAttributeAssignment)rpmObject;        	
        	//validate the appropriate fields before calling the SPs
        	validateBeforeDelete(assignment);
        	//call the delete SP
        	SP_UA_RES_COMPET(assignment, messageContext);
        }
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager, String condition, int level, String containerName, String contextName) throws RPMException
	{
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            ExceptionUtil.handleNoParentManager(this, containerName);
        }
        else if (manager instanceof ResourceAttributeManager)
        {
            joinCondition = new JoinCondition();

            SqlBuffer where = new SqlBuffer();
            where.append("(" + NAME_LEVEL_ID + " >= ");
            where.append(level);
            where.append(")");
            where.appendSubSelect(ResourceAttributeManager.NAME_ELEMENT_ID,
                NAME_PARENT_ID, TABLE_NAME, condition);

            joinCondition.setCondition(where.toString());
        }
        else if(manager instanceof ResourceManager)
        {    
            joinCondition = new JoinCondition();
            
            StringBuffer where = new StringBuffer();
            where.append(NAME_TYPE_ID);
    	    where.append(" = ");
    	    where.append(" 13 ");
    	    if(condition != null){
    	        where.append(" AND ");
    	        where.append(condition);
    	    }
    	    
    	    joinCondition.setCondition(where.toString());            
        }
        else if (manager instanceof DatafieldManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(DatafieldManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            where.append(NAME_PREFERENCE_LEVEL_ID);
            where.append(" FROM ");
            where.append(getTableName());
            
            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ResourceRatesManager)
        {
            joinCondition = new JoinCondition();

            joinCondition.setTableName(manager.getTableName());
            
            StringBuffer where = new StringBuffer();
            where.append(manager.getPrimaryKey());
            where.append(" IN (SELECT ");
            where.append(NAME_RATE_ID);
            where.append(" FROM ");
            where.append(TABLE_NAME);
            
            if (condition != null)
            {
                where.append(" WHERE ");
                where.append(condition);
            }
            
            joinCondition.setCondition(where.toString());
        }
        else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        return joinCondition;
    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(ResourceManager.NAME_RESOURCE_ID);
            where.append(" IN (SELECT ");
            where.append(ResourceAttributeAssignmentManager.NAME_RESOURCE_ID);
            where.append(" FROM ");
            where.append(getTableName());
            //condition on ResourceAttributeAssignment type
            where.append(" WHERE ");
            where.append(NAME_TYPE_ID);
            where.append(" = ");
            where.append(ResourceAttributeAssignment.TYPE_ID);            
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else
        {

            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);

        }

        return joinCondition;
    }
        
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException {
        
    	JoinCondition joinCondition = null;
        
        if(manager instanceof ResourceManager){
    	    
            //where clause of the join condition
    	    StringBuffer condition = new StringBuffer();
    	    condition.append(NAME_RESOURCE_ID);
    	    condition.append(" = ?");
                        
            joinCondition = getJoinCondition(manager,condition.toString(),1, null, null);
            if(extraWhere != null && extraWhere.length() > 0){
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(NAME_PARENT_ID);
                sb.append(" IN (SELECT ");
                sb.append(NAME_ELEMENT_ID);
                sb.append(" FROM ");
                sb.append(TABLE_NAME);
                sb.append(" WHERE ");
                sb.append(extraWhere);
                
                joinCondition.setCondition(sb.toString());
            }
            joinCondition.setTableName(getTableName());
        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}            
            return joinCondition;
    }    

    protected RPMObject loadRelatedObjects(RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext, FieldValueMap relatedFields, int index, boolean isDelta) throws RPMException, SQLException, ParseException {
             
       //Everything is loaded without scope

    //Parent proficiency level
        String oldParentId = (String)relatedFields.get(index,ID_PARENT_ID);
        ResourceAttributeProficiencyLevel newParent = null;
        ResourceAttributeManager resourceAttributeManager = (ResourceAttributeManager)RPMManagerFactory.getInstance().getRPMObjectManager(ResourceAttributeCompetencyProficiencyLevel.class.getName());                    					                	
        if(isDelta){
            newParent = ((ResourceAttributeAssignment)rpmObject).getProficiencyLevel();
            ResourceAttributeProficiencyLevel oldParent = null;
            if(oldParentId!=null){
                oldParent = new ResourceAttributeProficiencyLevel();
                oldParent.setID(oldParentId);
                if(newParent!=null && newParent.getID()!=null && newParent.getID().equals(oldParentId)){
                    oldParent = (ResourceAttributeProficiencyLevel)resourceAttributeManager.loadByPrimaryKey(newParent, null,messageContext, true);
                } else {
                    //In this case, old parent is loaded without the scope to avoid problem with the cache
                    oldParent = (ResourceAttributeProficiencyLevel)resourceAttributeManager.loadByPrimaryKey(oldParent, null,messageContext, false);
                }
            }
            deltaParent(rpmObject,oldParent);
        } else {
            if(oldParentId != null){
                if(newParent==null){//parent not in cache
                    newParent = new ResourceAttributeProficiencyLevel();
                    newParent.setID(oldParentId);
                    newParent = (ResourceAttributeProficiencyLevel)resourceAttributeManager.loadByPrimaryKey(newParent, null,messageContext, false);	                				
                }
                ((ResourceAttributeAssignment)rpmObject).setProficiencyLevel(newParent);
            }
        }

        //Related resource
        String resourceId = (String)relatedFields.get(index,ID_RESOURCE_ID);
        if (resourceId != null) {
            ResourceManager resourceManager = (ResourceManager)RPMManagerFactory.getInstance().getRPMObjectManager(Resource.class.getName());
            Resource resource = new Resource();
            resource.setID(resourceId);
            resource = (Resource)resourceManager.loadByPrimaryKey(resource, null, messageContext, isDelta);
            if(isDelta){
                ((ResourceAttributeAssignment)rpmObject).deltaParent(resource);
            } else {
                ((ResourceAttributeAssignment)rpmObject).setParent(resource);	
            }
        } else {
            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.RESOURCE_ID);
        }
            
            //experience
        String experienceId = (String)relatedFields.get(index,ID_EXPERIENCE_ID);
        if (experienceId != null) {
            DatafieldManager datafieldManager = (DatafieldManager)RPMManagerFactory.getInstance().getRPMObjectManager(Experience.class.getName());	                        
            Experience experience = new Experience();
            experience.setID(experienceId);
            experience = (Experience)datafieldManager.loadByPrimaryKey(experience, null, messageContext, isDelta);
            if(isDelta){
                ((ResourceAttributeAssignment)rpmObject).deltaExperience(experience);	
            } else {
                ((ResourceAttributeAssignment)rpmObject).setExperience(experience);
            }
        } else {
            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.EXPERIENCE_ID);
        }
            
        //preference rank
        String preferenceRankId = (String)relatedFields.get(index,ID_PREFERENCE_LEVEL_ID);
        if (preferenceRankId != null) {
            DatafieldManager datafieldManager = (DatafieldManager)RPMManagerFactory.getInstance().getRPMObjectManager(PreferenceRank.class.getName());	                        
            PreferenceRank preferenceRank = new PreferenceRank();
            preferenceRank.setID(preferenceRankId);
            preferenceRank = (PreferenceRank)datafieldManager.loadByPrimaryKey(preferenceRank, null, messageContext, isDelta);
            if(isDelta){
                ((ResourceAttributeAssignment)rpmObject).deltaPreferenceRank(preferenceRank);	
            } else {
                ((ResourceAttributeAssignment)rpmObject).setPreferenceRank(preferenceRank);
            }
        } else {
            ExceptionUtil.handleNoId(this, rpmObject, ExceptionUtil.PREFERENCE_RANK_ID);
        }
        
        //rates
        loadRates((ResourceAttributeAssignment)rpmObject, messageContext, relatedFields, index, isDelta);
        
        return rpmObject;
    }

    private void loadRates(ResourceAttributeAssignment assignment, 
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException {

    	ResourceRates rates = null;
        if (assignment.getRates() != null)
        {
            rates = assignment.getRates();
        }
        else
        {
            rates = new ResourceRates();
            rates.setID((String) relatedFields.get(index, ID_RATE_ID));
        }

        if (rates.getID() != null)
        {
            ResourceRatesManager manager = (ResourceRatesManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ResourceRates.class.getName());

            rates = (ResourceRates) manager.loadByPrimaryKey(rates, null,
                messageContext, isDelta);

            if (isDelta)
            {
                assignment.deltaRates(rates);
            }
            else
            {
                assignment.setRates(rates);
            }
        }
    }
    
    private void deltaParent(RPMObject rpmChild, RPMObject rpmParent) throws RPMException {
		ResourceAttributeProficiencyLevel parent = (ResourceAttributeProficiencyLevel)rpmParent;
		ResourceAttributeAssignment child = (ResourceAttributeAssignment)rpmChild;
		            
		child.deltaProficiencyLevel(parent);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext) throws SQLException,
            RPMException, ParseException
    {
    	ResourceAttributeAssignment assignment = (ResourceAttributeAssignment) rpmObject;
        
    	//TODO investigate how the name is set

    	assignment.setID(SP_A_RES_COMPET(assignment, messageContext));
    	
    	if(assignment.getDefaultFlag()!= null)
    	{
    	    SP_U_COMPET_DF(assignment, messageContext);
    	}
    	
    	SP_U_COMPET(assignment, null, messageContext);
    	
		if(isRatesNotNull(assignment.getRates())){
			SP_U_COMPET_RATE(assignment, messageContext);
    	}
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject,
     *      boolean)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
    	ResourceAttributeAssignment assignment = (ResourceAttributeAssignment) rpmObject;
    	
        if (rpmObject != null){
        	//TODO investigate how the name is set
        	
        	//rates update
        	if(assignment.testRatesModified() || isRatesModified(assignment.getRates())){
        		SP_U_COMPET_RATE(assignment, messageContext);
        	}
        	//strategic flag update
        	if(assignment.testStrategicFlagModified()){
        		SP_U_COMPET_STRATEGIC(assignment, messageContext);
        	}
        	//rest of attributes update
        	if(isRestOfAttributesModified(assignment)){
        		SP_U_COMPET(assignment, rpmScope, messageContext);
        	}
        }

        return assignment;
    }
      
    private boolean isRatesNotNull(ResourceRates rates){
		return	(rates!=null)&&
				((rates.getRegularCostPerHour()!=null && rates.getRegularCostPerHour().doubleValue()!=0.0)||
				 (rates.getSellingRatePerHour()!=null && rates.getSellingRatePerHour().doubleValue()!=0.0)||
				 (rates.getSpecialCostPerHour()!=null && rates.getSpecialCostPerHour().doubleValue()!=0.0)||
				 (rates.getSpecialSellingRatePerHour()!=null && rates.getSpecialSellingRatePerHour().doubleValue()!=0.0));    	
    }
    
    private boolean isRatesModified(ResourceRates rates){
    	return 	(rates!=null)&&
				((rates.testRegularCostPerHourModified())||
				 (rates.testSellingRatePerHourModified())||
				 (rates.testSpecialCostPerHourModified())||
				 (rates.testSpecialSellingRatePerHourModified()));
    }
    
    private boolean isRestOfAttributesModified(ResourceAttributeAssignment assignment){
    	//TODO investigate if default flag, name should be included here
    	return 	(assignment.testAcquiredOnModified())||
				(assignment.testDescriptionModified())||
				(assignment.testExperienceModified())||
				(assignment.testLastUsedDateTimeModified())||
				(assignment.testLastUsedLocationModified())||
				(assignment.testMandatoryDetailsModified())||
				(assignment.testMandatoryModified())||
				(assignment.testPreferenceRankModified())||
				(assignment.testTargetAttainmentDateModified());
    }

    private void validateBeforeDelete(ResourceAttributeAssignment assignment)throws RPMException{
    	if(assignment.getParent()==null || assignment.getParent().getID()==null){
			RPMException e = new RPMException("ResourceAttributeAssignmentMgr DELETE: assignment has no resource associated with it.");
			//log the error
			logger.error(e);
			throw e;
    	}
    }
    
    private String SP_A_RES_COMPET(ResourceAttributeAssignment assignment, MessageContext messageContext) throws SQLException, RPMException
    {
       	String sp_name = "SP_A_RES_COMPET";

       	Object args[] = new Object[3];
    	args[0] = assignment.getParent().getID();
     	args[1] = assignment.getProficiencyLevel().getID();
    	args[2] = getUser(messageContext).getID();

    	String elementId = executeProcedure(messageContext, sp_name, args);
    	return elementId;
    }
    
    private void SP_U_COMPET_DF(ResourceAttributeAssignment assignment, MessageContext messageContext) throws SQLException, RPMException
    {
    	String sp_name = "SP_U_COMPET_DF";

		Object args[] = new Object[4];
		args[0] = assignment.getID();
		args[1] = assignment.getProficiencyLevel().getID();
		args[2] = assignment.getDefaultFlag();
		args[3] = getUser(messageContext).getID();

		executeProcedure(messageContext, sp_name, args);
    }
    
    private void SP_U_COMPET(ResourceAttributeAssignment assignment, RPMObjectScope rpmScope, MessageContext messageContext) throws SQLException, RPMException
    {
    	String sp_name = "SP_U_COMPET";

		Object args[] = new Object[13];
		args[0] = assignment.getID();
		args[1] = assignment.getProficiencyLevel().getID();
		//args[2] = assignment.getName();
		args[3] = assignment.getDescription();
		args[4] = getExperience(assignment, rpmScope, messageContext);
		args[5] = getPreferenceRank(assignment, rpmScope, messageContext);
		args[6] = convertCalendar(assignment.getLastUsedDateTime());
		args[7] = assignment.getLastUsedLocation();
		args[8] = convertCalendar(assignment.getTargetAttainmentDate());
		args[9] = convertCalendar(assignment.getAcquiredOn());
		args[10] = assignment.getMandatory();
		args[11] = assignment.getMandatoryDetails();
		args[12] = getUser(messageContext).getID();

		executeProcedure(messageContext, sp_name, args);
    }

    private void SP_U_COMPET_RATE(ResourceAttributeAssignment assignment, MessageContext messageContext)throws RPMException, SQLException
    {
       	String sp_name = "SP_U_COMPET_RATE";

       	Object args[] = new Object[7];
    	args[0] = assignment.getID();
     	args[1] = assignment.getProficiencyLevel().getID();
    	args[2] = (assignment.getRates()!=null && assignment.getRates().getRegularCostPerHour()!=null)?assignment.getRates().getRegularCostPerHour():new Double(0);
    	args[3] = (assignment.getRates()!=null && assignment.getRates().getSellingRatePerHour()!=null)?assignment.getRates().getSellingRatePerHour():new Double(0);
    	args[4] = (assignment.getRates()!=null && assignment.getRates().getSpecialCostPerHour()!=null)?assignment.getRates().getSpecialCostPerHour():new Double(0);
    	args[5] = (assignment.getRates()!=null && assignment.getRates().getSpecialSellingRatePerHour()!=null)?assignment.getRates().getSpecialSellingRatePerHour():new Double(0);
    	args[6] = getUser(messageContext).getID();

    	executeProcedure(messageContext, sp_name, args);
    }
    
    private void SP_U_COMPET_STRATEGIC(ResourceAttributeAssignment assignment, MessageContext messageContext) throws SQLException, RPMException
    {
    	String sp_name = "SP_U_COMPET_STRATEGIC";

		Object args[] = new Object[3];
		args[0] = assignment.getID();
		args[1] = assignment.getStrategicFlag();
		args[2] = getUser(messageContext).getID();

		executeProcedure(messageContext, sp_name, args);
    }
    
    private void SP_UA_RES_COMPET(ResourceAttributeAssignment assignment, MessageContext messageContext)throws RPMException, SQLException
    {
       	String sp_name = "SP_UA_RES_COMPET";

       	Object args[] = new Object[3];
    	args[0] = assignment.getParent().getID();
     	args[1] = assignment.getID();
    	args[2] = getUser(messageContext).getID();

    	executeProcedure(messageContext, sp_name, args);
    }
    
    private Integer getExperience(ResourceAttributeAssignment assignment, RPMObjectScope rpmScope, MessageContext messageContext) throws RPMException
    {
    	
        Experience experience = assignment.getExperience();
    	
    	if (experience == null){
    	    try{
		        //TODO see how we can find the default Experience datafield
    	        experience = new Experience();
    	        experience.setID("2771");
                DatafieldManager manager = (DatafieldManager)RPMManagerFactory.getInstance().getRPMObjectManager(Experience.class.getName());                
		        experience = (Experience)manager.loadByPrimaryKey(experience,null,messageContext, false);
		        assignment.setExperience(experience);
    	        
    	    }catch(Exception s){
    			RPMException e = new RPMException(s);
    			//log the error
    			logger.error(e);
    			throw e;
    	    }
    	}
    	    
    	return new Integer(experience.getID());        

    }
    
    private Integer getPreferenceRank(ResourceAttributeAssignment assignment, RPMObjectScope rpmScope, MessageContext messageContext) throws RPMException
    {
    	PreferenceRank preferenceRank = assignment.getPreferenceRank();  	
    	
    	if (preferenceRank == null){
    	    try{
		        //TODO see how we can find the default PreferenceRank datafield
    	        preferenceRank = new PreferenceRank();
    	        preferenceRank.setID("2759");
                DatafieldManager manager = (DatafieldManager)RPMManagerFactory.getInstance().getRPMObjectManager(PreferenceRank.class.getName());                
		        preferenceRank = (PreferenceRank)manager.loadByPrimaryKey(preferenceRank, null, messageContext, false);
		        assignment.setPreferenceRank(preferenceRank);
    	        
    	    }catch(Exception s){
    			RPMException e = new RPMException(s);
    			//log the error
    			logger.error(e);
    			throw e;
    	    }
    	}
    	    
    	return new Integer(preferenceRank.getID());

    }
    
    private String convertCalendar(Calendar calendar)throws RPMException{
        if (calendar != null) {
            return calendar.get(Calendar.YEAR) + "-"
                    + calendar.get(Calendar.MONTH) + "-"
                    + calendar.get(Calendar.DAY_OF_MONTH);
        }
        return null;
    }

}
