package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.Classification;
import com.ibm.rpm.applicationadministration.containers.IndustryType;
import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.resource.containers.IndustryExperience;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.xpathparser.XPathContainer;

public final class ResourceIndustryExperienceManager extends
        AbstractRPMObjectManager
{
    private static final HashSet CONTAINERS = new HashSet();

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    static
    {
        CONTAINERS.add(IndustryExperience.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ResourceIndustryExperienceManager.class);
        }
    }

    /**
     * Column EXPERIENCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_EXPERIENCE_ID = 1;

    public static final int TYPE_EXPERIENCE_ID = Types.CHAR;

    public static final String NAME_EXPERIENCE_ID = "RESOURCE_INDUSTRY_EXPERIENCE.EXPERIENCE_ID";

    public static final String PROPERTY_EXPERIENCE_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EXPERIENCE_ID, NAME_EXPERIENCE_ID);
    }

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 2;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "RESOURCE_INDUSTRY_EXPERIENCE.RESOURCE_ID";

    public static final String PROPERTY_RESOURCE_ID = "RESOURCEID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID, NAME_RESOURCE_ID);
    }

    /**
     * Column CLASSIFICATION of type Types.CHAR mapped to String.
     */
    public static final int ID_CLASSIFICATION = 3;

    public static final int TYPE_CLASSIFICATION = Types.CHAR;

    public static final String NAME_CLASSIFICATION = "RESOURCE_INDUSTRY_EXPERIENCE.CLASSIFICATION";

    public static final String PROPERTY_CLASSIFICATION = "CLASSIFICATION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CLASSIFICATION, NAME_CLASSIFICATION);
    }

    /**
     * Column INDUSTRY_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_INDUSTRY_ID = 4;

    public static final int TYPE_INDUSTRY_ID = Types.INTEGER;

    public static final String NAME_INDUSTRY_ID = "RESOURCE_INDUSTRY_EXPERIENCE.INDUSTRY_ID";

    public static final String PROPERTY_INDUSTRY_ID = "INDUSTRY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_INDUSTRY_ID, NAME_INDUSTRY_ID);
    }

    /**
     * Column PROF_LEVEL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROF_LEVEL = 5;

    public static final int TYPE_PROF_LEVEL = Types.INTEGER;

    public static final String NAME_PROF_LEVEL = "RESOURCE_INDUSTRY_EXPERIENCE.PROF_LEVEL";

    public static final String PROPERTY_PROF_LEVEL = "PROFICIENCY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROF_LEVEL, NAME_PROF_LEVEL);
    }

    public static final String TABLE_NAME = "RESOURCE_INDUSTRY_EXPERIENCE";

    /**
     * Create an array of type string containing all the fields of the
     * RESOURCE_INDUSTRY_EXPERIENCE table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_EXPERIENCE_ID, NAME_RESOURCE_ID, NAME_CLASSIFICATION,
            NAME_INDUSTRY_ID, NAME_PROF_LEVEL
    };

    /**
     * Field that contains the comma separated fields of the
     * RESOURCE_INDUSTRY_EXPERIENCE table.
     */
    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    public ResourceIndustryExperienceManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return NAME_EXPERIENCE_ID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        return new Integer(0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getForeignKeyCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String)
     */
    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager instanceof ResourceManager)
        {
            
            joinCondition = new JoinCondition(getTableName(), getFilter());

            StringBuffer where = new StringBuffer();
            where.append(NAME_RESOURCE_ID);
            where.append(" =?");
            if (extraWhere != null && extraWhere.length() > 0)
            {
                where.append(" AND ");
                where.append(extraWhere);
            }
            
            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof ResourceIndustryExperienceManager)
        {
            
            joinCondition = new JoinCondition(getTableName(), getFilter());

            StringBuffer where = new StringBuffer();
            where.append(NAME_EXPERIENCE_ID);
            where.append(" =?");
            if (extraWhere != null && extraWhere.length() > 0)
            {
                where.append(" AND ");
                where.append(extraWhere);
            }
            
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.managers.AbstractRPMObjectManager,
     *      java.lang.String, int)
     */
    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

        }
        else if (manager instanceof ResourceManager)
        {

            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();

            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof ResourceIndustryExperienceManager)
        {

            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();

            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof DatafieldManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            where.append(DatafieldManager.NAME_ELEMENT_ID);
            where.append(" IN (SELECT ");
            if (container.getFieldName().equalsIgnoreCase("classification"))
            {
                where.append(" CAST( ");
            }
            where.append(FIELDPROPERTYMAP.get(container.getFieldName()
                .toUpperCase()));
            if (container.getFieldName().equalsIgnoreCase("classification"))
            {
                where.append(" AS INTEGER) ");
            }
            where.append(" FROM ");
            where.append(getTableName());

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.SUB_TABLE_JOIN);

        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId)
    {
        RPMObject object = null;

        object = new IndustryExperience();
        return object;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#loadFromResultSet(RPMObject[],
     *      java.sql.ResultSet, com.ibm.rpm.scope.RPMObjectScope, boolean)
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        IndustryExperience object = (IndustryExperience) rpmObject;

        //Everything is loaded without scope

        //Related resource is always loaded without scope
        String resourceId = (String) relatedFields.get(index, ID_RESOURCE_ID);
        if (resourceId != null)
        {
            ResourceManager resourceManager = (ResourceManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(Resource.class.getName());
            Resource resource = new Resource();
            resource.setID(resourceId);
            resource = (Resource) resourceManager.loadByPrimaryKey(resource,
                null, messageContext, isDelta);
            if (isDelta)
            {
                object.deltaParent(resource);
            }
            else
            {
                object.setParent(resource);
            }
        }
        else
        {
            ExceptionUtil
                .handleNoId(this, rpmObject, ExceptionUtil.RESOURCE_ID);
        }

        DatafieldManager dtManager = (DatafieldManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(Classification.class.getName());

        //Classification
        String classificationId = (String) relatedFields.get(index,
            ID_CLASSIFICATION);
        
        if( classificationId != null )
        {
            classificationId = classificationId.trim();
	        Classification classification = new Classification();
	        classification.setID(classificationId);
	
	        classification = (Classification) dtManager.loadByPrimaryKey(
	            classification, null, messageContext, isDelta);
	        if (isDelta)
	        {
	            object.deltaClassification(classification);
	        }
	        else
	        {
	            object.setClassification(classification);
	        }
        } 

        //IndustryType
        String industryTypeId = (String) relatedFields.get(index,
            ID_INDUSTRY_ID);
        if(industryTypeId != null)
        {
	        IndustryType industry = new IndustryType();
	        industry.setID(industryTypeId);
	
	        industry = (IndustryType) dtManager.loadByPrimaryKey(industry, null,
	            messageContext, isDelta);
	        if (isDelta)
	        {
	            object.deltaIndustry(industry);
	        }
	        else
	        {
	            object.setIndustry(industry);
	        }
        }

        //Proficiency Level
        String proficiencyLevelId = (String) relatedFields.get(index,
            ID_PROF_LEVEL);
        if(proficiencyLevelId != null)
        {
	        ProficiencyLevel proficiency = new ProficiencyLevel();
	        proficiency.setID(proficiencyLevelId);
	
	        proficiency = (ProficiencyLevel) dtManager.loadByPrimaryKey(
	            proficiency, null, messageContext, isDelta);
	        if (isDelta)
	        {
	            object.deltaProficiency(proficiency);
	        }
	        else
	        {
	            object.setProficiency(proficiency);
	        }
        }

        return rpmObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.containers.RPMObject)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws SQLException
    {

        RPMObject pObject = rpmObject;

        pObject.setID(rs.getString(ID_EXPERIENCE_ID));

        return pObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String parentId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, parentId);

        String classificationId = resultSet.getString(ID_CLASSIFICATION);
        relatedFields.put(index, ID_CLASSIFICATION, classificationId);

        String industryTypeId = resultSet.getString(ID_INDUSTRY_ID);
        relatedFields.put(index, ID_INDUSTRY_ID, industryTypeId);

        String proficiencyLevelId = resultSet.getString(ID_PROF_LEVEL);
        relatedFields.put(index, ID_PROF_LEVEL, proficiencyLevelId);

        return relatedFields;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createObject(com.ibm.rpm.containers.RPMObject)
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        IndustryExperience pObject = (IndustryExperience) rpmObject;
        pObject.setID(SP_I_BGINDUSTRYEXP(pObject, messageContext));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#updateObject(com.ibm.rpm.containers.RPMObject)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        IndustryExperience pObject = (IndustryExperience) rpmObject;
        if (pObject.testClassificationModified()
                || pObject.testIndustryModified()
                || pObject.testProficiencyModified())
        {
            SP_U_BGINDUSTRYEXP(pObject, messageContext);
        }

        return pObject;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {

        if (rpmObject != null)
        {
            SP_D_BGINDUSTRYEXP((IndustryExperience) rpmObject, messageContext);
        }
    }

    private String SP_I_BGINDUSTRYEXP(IndustryExperience industryexperience,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_I_BGINDUSTRYEXP";
        Object args[] = new Object[4];
        args[0] = null;
        if (industryexperience.getParent() != null)
        {
            args[1] = industryexperience.getParent().getID();
        }
        args[2] = new Integer(98);
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }

    private void SP_U_BGINDUSTRYEXP(IndustryExperience industryexperience,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_BGINDUSTRYEXP";
        Object args[] = new Object[5];
        args[0] = industryexperience.getID();
        if (industryexperience.getClassification() != null)
        {
            args[1] = industryexperience.getClassification().getID();
        }

        if (industryexperience.getIndustry() != null)
        {
            args[2] = new Integer(industryexperience.getIndustry().getID());
        }

        if (industryexperience.getProficiency() != null)
        {
            args[3] = new Integer(industryexperience.getProficiency().getID());
        }

        args[4] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_D_BGINDUSTRYEXP(IndustryExperience industryexperience,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_BGINDUSTRYEXP";
        Object args[] = new Object[4];
        args[0] = industryexperience.getID();
        args[1] = ManagerUtil.getParentId(this, industryexperience,
            messageContext, TABLE_NAME, NAME_EXPERIENCE_ID, NAME_RESOURCE_ID);
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        
        executeProcedure(messageContext, name, args);
    }
}
