package com.ibm.rpm.resource.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;


import com.ibm.rpm.applicationadministration.containers.Language;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.resource.containers.LanguageProficiency;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author Simon Liu
 */
public final class ResourceLanguageProficiencyManager extends
        AbstractRPMObjectManager
{


    private static final HashSet CONTAINERS = new HashSet();

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    static
    {
        CONTAINERS.add(LanguageProficiency.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ResourceLanguageProficiencyManager.class);
        }
    }

    /**
     * Column READINESS_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_READINESS_ID = 1;

    public static final int TYPE_READINESS_ID = Types.CHAR;

    public static final String NAME_READINESS_ID = "RESOURCE_READINESS.READINESS_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_READINESS_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_READINESS_ID, NAME_READINESS_ID);
    }

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 2;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "RESOURCE_READINESS.RESOURCE_ID";

    public static final String PROPERTY_RESOURCE_ID = "RESOURCEID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID, NAME_RESOURCE_ID);
    }

    /**
     * Column LANGUAGE_ID of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_LANGUAGE_ID = 3;

    public static final int TYPE_LANGUAGE_ID = Types.INTEGER;

    public static final String NAME_LANGUAGE_ID = "RESOURCE_READINESS.LANGUAGE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LANGUAGE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LANGUAGE_ID,NAME_LANGUAGE_ID);}

    /**
     * Column READ_LEVEL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_READ_LEVEL = 4;

    public static final int TYPE_READ_LEVEL = Types.INTEGER;

    public static final String NAME_READ_LEVEL = "RESOURCE_READINESS.READ_LEVEL";

    public static final String PROPERTY_READ_LEVEL = "READLEVEL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_READ_LEVEL, NAME_READ_LEVEL);
    }

    /**
     * Column SPEAK_LEVEL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SPEAK_LEVEL = 5;

    public static final int TYPE_SPEAK_LEVEL = Types.INTEGER;

    public static final String NAME_SPEAK_LEVEL = "RESOURCE_READINESS.SPEAK_LEVEL";

    public static final String PROPERTY_SPEAK_LEVEL = "SPEAKLEVEL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SPEAK_LEVEL, NAME_SPEAK_LEVEL);
    }

    /**
     * Column WRITE_LEVEL of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_WRITE_LEVEL = 6;

    public static final int TYPE_WRITE_LEVEL = Types.INTEGER;

    public static final String NAME_WRITE_LEVEL = "RESOURCE_READINESS.WRITE_LEVEL";

    public static final String PROPERTY_WRITE_LEVEL = "WRITELEVEL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WRITE_LEVEL, NAME_WRITE_LEVEL);
    }

    /**
     * Column DEFAULT_LANGUAGE of type Types.CHAR mapped to String.
     */
    public static final int ID_DEFAULT_LANGUAGE = 7;

    public static final int TYPE_DEFAULT_LANGUAGE = Types.CHAR;

    public static final String NAME_DEFAULT_LANGUAGE = "RESOURCE_READINESS.DEFAULT_LANGUAGE";

    public static final String PROPERTY_DEFAULT_LANGUAGE = "DEFAULTLANGUAGE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DEFAULT_LANGUAGE, SqlUtil
            .getBooleanValue(NAME_DEFAULT_LANGUAGE, "Y"));
    }

    private static final String TABLE_NAME = "RESOURCE_READINESS";

    /**
     * Create an array of type string containing all the fields of the
     * RESOURCE_READINESS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_READINESS_ID, NAME_RESOURCE_ID, NAME_LANGUAGE_ID,
            NAME_READ_LEVEL, NAME_SPEAK_LEVEL, NAME_WRITE_LEVEL,
            NAME_DEFAULT_LANGUAGE
    };

    /**
     * Field that contains the comma separated fields of the RESOURCE_READINESS
     * table.
     */
    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    public ResourceLanguageProficiencyManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_READINESS_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        return new Integer(0);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager instanceof ResourceManager)
        {

            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(" RESOURCE_ID ");
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof ResourceLanguageProficiencyManager)
        {

            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(" READINESS_ID ");
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, null);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            if (condition != null)
            {
                joinCondition.setCondition(condition);
            }

        }
        else if (manager instanceof ResourceManager
                || manager instanceof ResourceLanguageProficiencyManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            if (condition != null)
            {
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else if (manager instanceof DatafieldManager)
        {
            joinCondition = new JoinCondition();

            joinCondition
                .setTableName(ResourceLanguageProficiencyManager.TABLE_NAME);

            StringBuffer where = new StringBuffer();
            where.append(DatafieldManager.NAME_ELEMENT_ID);
            where.append(" = ");
            where.append(ResourceLanguageProficiencyManager.NAME_LANGUAGE_ID);
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement method getSubTableJoinCondition if needed
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.SUB_TABLE_JOIN);

        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#createContainer(int)
     */
    protected RPMObject createContainer(int typeId)
    {
        RPMObject object = null;

        object = new LanguageProficiency();
        return object;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        LanguageProficiency object = (LanguageProficiency) rpmObject;

        //Related resource is always loaded without scope
        String resourceId = (String) relatedFields.get(index, ID_RESOURCE_ID);
        if (resourceId != null)
        {
            ResourceManager resourceManager = (ResourceManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(Resource.class.getName());
            Resource resource = new Resource();
            resource.setID(resourceId);
            resource = (Resource) resourceManager.loadByPrimaryKey(resource,
                null, messageContext, isDelta);
            if (isDelta)
            {
                object.deltaParent(resource);
            }
            else
            {
                object.setParent(resource);
            }
        }
        else
        {
            ExceptionUtil
                .handleNoId(this, rpmObject, ExceptionUtil.RESOURCE_ID);
        }

        DatafieldManager dtManager = (DatafieldManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(Language.class.getName());

        //Language proficiency
        String languageProficiencyId = (String) relatedFields.get(index,
            ID_LANGUAGE_ID);

        Language language;
        if (languageProficiencyId == null)
        {
            language = null;
        }
        else
        {
            language = new Language();
	        language.setID(languageProficiencyId);
	        language = (Language) dtManager.loadByPrimaryKey(language, null,
	            messageContext, isDelta);
        }
        if (isDelta)
        {
            object.deltaLanguage(language);
        }
        else
        {
            object.setLanguage(language);
        }

        return rpmObject;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws SQLException
    {

        RPMObject pObject = rpmObject;
        Language lang = new Language();
        lang.setID(rs.getString(ID_READINESS_ID));
        boolean def = (rs.getString(ID_DEFAULT_LANGUAGE).equals("Y"));

        pObject.setID(rs.getString(ID_READINESS_ID));
        if (isDelta)
        {
            ((LanguageProficiency) pObject).deltaReadLevel(new Integer(rs
                .getInt(ID_READ_LEVEL)));
            ((LanguageProficiency) pObject).deltaSpeakLevel(new Integer(rs
                .getInt(ID_SPEAK_LEVEL)));
            ((LanguageProficiency) pObject).deltaWriteLevel(new Integer(rs
                .getInt(ID_WRITE_LEVEL)));
            ((LanguageProficiency) pObject).deltaDefaultLanguage(new Boolean(
                def));
        }
        else
        {
            ((LanguageProficiency) pObject).setReadLevel(new Integer(rs
                .getInt(ID_READ_LEVEL)));
            ((LanguageProficiency) pObject).setSpeakLevel(new Integer(rs
                .getInt(ID_SPEAK_LEVEL)));
            ((LanguageProficiency) pObject).setWriteLevel(new Integer(rs
                .getInt(ID_WRITE_LEVEL)));
            ((LanguageProficiency) pObject)
                .setDefaultLanguage(new Boolean(def));

        }

        return pObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String parentId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, parentId);

        String languageProficiencyId = resultSet.getString(ID_LANGUAGE_ID);
        relatedFields.put(index, ID_LANGUAGE_ID, languageProficiencyId);

        return relatedFields;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        LanguageProficiency pObject = (LanguageProficiency) rpmObject;
        pObject.setID(SP_I_RESREADINESS(pObject, messageContext));
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        LanguageProficiency pObject = (LanguageProficiency) rpmObject;
        if (pObject.testLanguageModified() || pObject.testReadLevelModified()
                || pObject.testSpeakLevelModified()
                || pObject.testWriteLevelModified()
                || pObject.testDefaultLanguageModified())
        {
            if (pObject.testDefaultLanguageModified() && pObject.getDefaultLanguage() != null 
                    && pObject.getDefaultLanguage().booleanValue() == true)
            {
                // If this language is being set to default, then unset previous default language
                ArrayList defaultLanguagesForResource = loadByWhere(null, null, null, messageContext,
                    NAME_DEFAULT_LANGUAGE + "='Y' AND " + NAME_READINESS_ID + "!='" + pObject.getID()
                    + "' AND " + NAME_RESOURCE_ID + " IN (SELECT " + NAME_RESOURCE_ID 
                    + " FROM " + TABLE_NAME + " WHERE " + NAME_READINESS_ID + "='" + pObject.getID()
                    + "\')", null, TABLE_NAME, ALL_FIELDS, null,
                    false);
                
                if (defaultLanguagesForResource != null)
                {
	                for (Iterator i = defaultLanguagesForResource.iterator(); i.hasNext();) 
	                {
	                    LanguageProficiency lp = (LanguageProficiency) i.next();
	                    lp.setDefaultLanguage(Boolean.FALSE);
	                    SP_U_RESREADINESS(lp, messageContext);
	                }
                }
            }
            
            SP_U_RESREADINESS(pObject, messageContext);
        }

        return pObject;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {

        if (rpmObject != null)
        {
            SP_D_RESREADINESS((LanguageProficiency) rpmObject, messageContext);
        }
    }

    private String SP_I_RESREADINESS(LanguageProficiency languageproficiency,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_I_RESREADINESS";
        Object args[] = new Object[4];
        args[0] = null;
        if (languageproficiency.getParent() != null)
        {
            args[1] = languageproficiency.getParent().getID();
        }
        args[2] = new Integer(104);
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }

    private void SP_U_RESREADINESS(LanguageProficiency languageproficiency,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_RESREADINESS";
        Object args[] = new Object[7];
        args[0] = languageproficiency.getID();
        if (languageproficiency.getLanguage() != null)
            args[1] = languageproficiency.getLanguage().getID();
        else
            args[1] = null;
        if (languageproficiency.getReadLevel() != null)
        {
            args[2] = new Integer(languageproficiency.getReadLevel().intValue());
        }
        if (languageproficiency.getSpeakLevel() != null)
        {
            args[3] = new Integer(languageproficiency.getSpeakLevel()
                .intValue());
        }
        if (languageproficiency.getWriteLevel() != null)
        {
            args[4] = new Integer(languageproficiency.getWriteLevel()
                .intValue());
        }
        if (languageproficiency.getDefaultLanguage() != null
                && languageproficiency.getDefaultLanguage().booleanValue())
            args[5] = "Y";
        else
            args[5] = "N";
        args[6] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_D_RESREADINESS(LanguageProficiency languageproficiency,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_RESREADINESS";
        Object args[] = new Object[4];
        args[0] = languageproficiency.getID();
        args[1] = ManagerUtil.getParentId(this, languageproficiency,
            messageContext, TABLE_NAME, "READINESS_ID", "RESOURCE_ID");
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        
        executeProcedure(messageContext, name, args);
    }
}
