/*
 * Created on Oct 25, 2005
 *
 */
package com.ibm.rpm.resource.managers;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.CitizenshipCountryCode;
import com.ibm.rpm.applicationadministration.containers.Classification;
import com.ibm.rpm.applicationadministration.containers.ContactGroup;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.EmployeeStatus;
import com.ibm.rpm.applicationadministration.containers.Experience;
import com.ibm.rpm.applicationadministration.containers.IndustryType;
import com.ibm.rpm.applicationadministration.containers.Language;
import com.ibm.rpm.applicationadministration.containers.PreferenceRank;
import com.ibm.rpm.applicationadministration.containers.ProficiencyLevel;
import com.ibm.rpm.applicationadministration.containers.WorkLocation;
import com.ibm.rpm.applicationadministration.containers.WorkingStatus;
import com.ibm.rpm.applicationadministration.managers.AttributeManager;
import com.ibm.rpm.applicationadministration.managers.ContactGroupsManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.managers.CustomFieldManager;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.framework.AbstractModuleManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.resource.containers.ResourceAttribute;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyCategory;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillCategory;
import com.ibm.rpm.resource.containers.ResourceModule;
import com.ibm.rpm.resource.scope.ResourceAttributeScope;
import com.ibm.rpm.resource.scope.ResourceModuleScope;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.managers.ScorecardManager;

/**
 * @author karim
 *  
 */
public class ResourceModuleManager extends AbstractModuleManager
{
    private static Log logger = LogFactory.getLog(ResourceModuleManager.class);

    protected Log getLogger()
    {
        return logger;
    }

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(ResourceModule.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ResourceModuleManager.class);
        }
    }

    {
        localContextName = ContextUtil.RESOURCE_CONTEXT;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = new ResourceModule();
        return object;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof AttributeManager)
        {
            if (localContextName.equalsIgnoreCase(contextName))
            {
                joinCondition = new JoinCondition();
            }
            else
            {//context not linked to this manager
                logger
                    .error("ResourceModuleManager.getJoinCondition: contextName "
                            + contextName + " is not linked with this manager.");
            }
        }
        else if (manager instanceof ResourceAttributeManager)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof ContactGroupsManager)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof CustomFieldManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setCondition(CustomFieldManager
                .createJoinCondition(localContextName));
        }
        else if (manager instanceof ScorecardManager)
        {
            /*
             * For XPaths like "/RESOURCEMODULE/ScorecardFolder[ID='__']"
             */     
            joinCondition = new JoinCondition();
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    private ResourceAttribute[] loadResourceAttributes(AbstractModule abstractModule, 
            ResourceAttribute[] oldObjs, ResourceAttributeScope scope,
            int typeId, Class baseClass, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        String extraWhere = " TYPE_ID = " + typeId;
        return (ResourceAttribute[]) loadElements(abstractModule, oldObjs, scope, baseClass,
            null, extraWhere, messageContext, isDelta);
    }

    private ResourceAttribute[] saveResourceAttributes(AbstractModule module,
            ResourceAttribute[] resourceAttributes,
            ResourceAttributeScope scope, MessageContext messageContext,
            Class childrenBaseClass) throws RPMException, SQLException,
            ParseException
    {
        return (ResourceAttribute[]) saveElements(module, resourceAttributes,
            scope, messageContext, childrenBaseClass);
    }

    //verify consistency
    protected void loadChildren(AbstractModule abstractModule,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ResourceModule resourceModule = (ResourceModule) abstractModule;
        ResourceModuleScope scope = (ResourceModuleScope) rpmScope;

        DatafieldManager datafieldManager = (DatafieldManager) getManagerInstance(DatafieldCategory.class);

        /*
         * Load ResourceAttributeCompetencyCategory[]
         */

        if (scope.getResourceAttributeCompetencyCategories() != null)
        {
            ResourceAttributeCompetencyCategory[] competencyCategories = (ResourceAttributeCompetencyCategory[]) loadResourceAttributes(
                    resourceModule, resourceModule
                            .getResourceAttributeCompetencyCategories(), scope
                            .getResourceAttributeCompetencyCategories(),
                    ResourceAttributeCompetencyCategory.TYPE_ID,
                    ResourceAttributeCompetencyCategory.class, messageContext,
                    isDelta);
            resourceModule
                    .setResourceAttributeCompetencyCategories(competencyCategories);
        }

        /*
         * Load ResourceAttributeSkillCategory[]
         */

        if (scope.getResourceAttributeSkillCategories() != null)
        {
            ResourceAttributeSkillCategory[] skillCategories = (ResourceAttributeSkillCategory[]) loadResourceAttributes(
                    resourceModule,
                resourceModule.getResourceAttributeSkillCategories(), scope
                    .getResourceAttributeSkillCategories(),
                ResourceAttributeSkillCategory.TYPE_ID,
                ResourceAttributeSkillCategory.class, messageContext, isDelta);

            resourceModule.setResourceAttributeSkillCategories(skillCategories);
        }

        if (scope.getContactGroups() != null)
        {
            String extraWhere = " TYPE_ID = " + ContactGroup.TYPE_ID;
            ContactGroup[] contactgroup = (ContactGroup[]) loadElements(
                    resourceModule,
                resourceModule.getContactGroups(), scope.getContactGroups(),
                ContactGroup.class, null, extraWhere, messageContext, isDelta);
            resourceModule.setContactGroups(contactgroup);
        }

        if (scope.isCitizenshipCountryCodes())
        {

            CitizenshipCountryCode[] cccode = (CitizenshipCountryCode[]) datafieldManager
                .loadDatafields(resourceModule, resourceModule.getCitizenshipCountryCodes(),
                    CitizenshipCountryCode.class,
                    CitizenshipCountryCode.TYPE_ID, this,
                    localContextName, messageContext, isDelta);

            resourceModule.setCitizenshipCountryCodes(cccode);
        }
        if (scope.isEmployeeStatus())
        {
            EmployeeStatus[] estatus = (EmployeeStatus[]) datafieldManager
                .loadDatafields(resourceModule, resourceModule.getEmployeeStatus(),
                    EmployeeStatus.class, EmployeeStatus.TYPE_ID,
                    this, localContextName, messageContext, isDelta);

            resourceModule.setEmployeeStatus(estatus);
        }
        if (scope.isWorkingStatus())
        {
            WorkingStatus[] wstatus = (WorkingStatus[]) datafieldManager
                .loadDatafields(resourceModule, resourceModule.getWorkingStatus(),
                    WorkingStatus.class, WorkingStatus.TYPE_ID,
                    this, localContextName, messageContext, isDelta);

            resourceModule.setWorkingStatus(wstatus);
        }
        if (scope.isClassifications())
        {
            Classification[] classification = (Classification[]) datafieldManager
                .loadDatafields(resourceModule, resourceModule.getClassifications(),
                    Classification.class, Classification.TYPE_ID,
                    this, localContextName, messageContext, isDelta);

            resourceModule.setClassifications(classification);
        }
        if (scope.isIndustryTypes())
        {
            IndustryType[] industrytype = (IndustryType[]) datafieldManager
                .loadDatafields(resourceModule, resourceModule.getIndustryTypes(),
                    IndustryType.class, IndustryType.TYPE_ID, this,
                    localContextName, messageContext, isDelta);
            resourceModule.setIndustryTypes(industrytype);

        }
        if (scope.isLanguages())
        {
            Language[] language = (Language[]) datafieldManager.loadDatafields(
                    resourceModule, 
                resourceModule.getLanguages(), Language.class,
                Language.TYPE_ID, this, localContextName, messageContext, isDelta);
            resourceModule.setLanguages(language);

        }
        if (scope.isProficiencyLevels())
        {
            ProficiencyLevel[] prolevel = (ProficiencyLevel[]) datafieldManager
                .loadDatafields(resourceModule, resourceModule.getProficiencyLevels(),
                    ProficiencyLevel.class, ProficiencyLevel.TYPE_ID,
                    this, localContextName, messageContext, isDelta);
            resourceModule.setProficiencyLevels(prolevel);

        }
        if (scope.isPreferenceRanks())
        {
            PreferenceRank[] prerank = (PreferenceRank[]) datafieldManager
                .loadDatafields(resourceModule, resourceModule.getPreferenceRanks(),
                    PreferenceRank.class, PreferenceRank.TYPE_ID,
                    this, localContextName, messageContext, isDelta);

            resourceModule.setPreferenceRanks(prerank);

        }
        if (scope.isExperiences())
        {
            Experience[] experience = (Experience[]) datafieldManager
                .loadDatafields(resourceModule, resourceModule.getExperiences(),
                    Experience.class, Experience.TYPE_ID, this,
                    localContextName, messageContext, isDelta);

            resourceModule.setExperiences(experience);

        }
        if (scope.isWorkLocations())
        {
            WorkLocation[] wlocation = (WorkLocation[]) datafieldManager
                .loadDatafields(resourceModule, resourceModule.getWorkLocations(),
                    WorkLocation.class, WorkLocation.TYPE_ID, this,
                    localContextName, messageContext, isDelta);

            resourceModule.setWorkLocations(wlocation);

        }
        if (scope.getScorecardFolders() != null)
        {
            String extraWhere = " TYPE_ID = "+ ScorecardFolder.TYPE_ID;
            ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) loadElements(
                resourceModule, resourceModule.getScorecardFolders(), scope
                            .getScorecardFolders(), ScorecardFolder.class,
                    null, extraWhere, messageContext, isDelta);        
            resourceModule.setScorecardFolders(scorecardFolders);
        }

        CustomFieldScope customFieldScope = scope.getCustomFieldCategories();
        if (customFieldScope != null)
        {
            CustomFieldManager manager = (CustomFieldManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(CustomFieldCategory.class);
            CustomFieldCategory[] categories = manager
                .loadCustomFieldCategories(localContextName, customFieldScope,
                    messageContext);
            resourceModule.setCustomFieldCategories(categories);
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        ResourceModuleScope scope = (ResourceModuleScope) rpmScope;
        ResourceModule resourceModule = (ResourceModule) rpmObject;

        resourceModule.setContextName(localContextName);

        // Save ResourceAttributesCompetencyCategory[]
        if (scope.getResourceAttributeCompetencyCategories() != null)
        {
            ResourceAttributeCompetencyCategory[] racc = (ResourceAttributeCompetencyCategory[]) saveResourceAttributes(
                resourceModule, resourceModule
                    .getResourceAttributeCompetencyCategories(), scope
                    .getResourceAttributeCompetencyCategories(),
                messageContext, ResourceAttributeCompetencyCategory.class);

            resourceModule.setResourceAttributeCompetencyCategories(racc);
        }

        // save ResourceAttributesSkillCategory[]
        if (scope.getResourceAttributeSkillCategories() != null)
        {
            ResourceAttributeSkillCategory[] rasc = (ResourceAttributeSkillCategory[]) saveResourceAttributes(
                resourceModule, resourceModule
                    .getResourceAttributeSkillCategories(), scope
                    .getResourceAttributeSkillCategories(), messageContext,
                ResourceAttributeSkillCategory.class);

            resourceModule.setResourceAttributeSkillCategories(rasc);
        }
        // save ContactGroups
        if (scope.getContactGroups() != null)
        {

            ContactGroup[] contactgrp = (ContactGroup[]) saveElements(
                resourceModule, resourceModule.getContactGroups(), scope
                    .getContactGroups(), messageContext, ContactGroup.class);
            resourceModule.setContactGroups(contactgrp);
        }
        CustomFieldScope customFieldScope = scope.getCustomFieldCategories();
        if (customFieldScope != null)
        {
            propagateContextName(resourceModule, resourceModule
                .getCustomFieldCategories());
            resourceModule
                .setCustomFieldCategories((CustomFieldCategory[]) updateChildren(
                    null, resourceModule.getCustomFieldCategories(),
                    customFieldScope, messageContext, null));
        }
        //ScorecardFolders
        if(scope.getScorecardFolders() != null)
        {
            ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) saveElements(
                resourceModule, resourceModule.getScorecardFolders(),
                    scope.getScorecardFolders(), messageContext,
                    ScorecardFolder.class );
            resourceModule.setScorecardFolders(scorecardFolders);
        }       

        return resourceModule;
    }
}
