/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import java.util.List;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;

/**
 * @author dominiqu
 */
public abstract class AbstractAggregateScopeCheckpoint extends
        ScopeElementCheckpoint
{
    protected void validateSave(RPMObject rpmObject, MessageContext msgCtx,
            boolean parentRequired, List validParentTypes)
    {
        super.validateSave(rpmObject, msgCtx, parentRequired, validParentTypes);

        // Initialize
        AbstractAggregateScope mainAbstractAggregateScope = (AbstractAggregateScope) rpmObject;

        // Field validation: closedByName
        if (mainAbstractAggregateScope.testClosedByNameModified())
        {
        	if(mainAbstractAggregateScope.getClosedByName() != null)
        	{
        	 GenericValidator.validateMaxLength(mainAbstractAggregateScope,
                     ValidationConstants.ABSTRACTAGGREGATESCOPE_CLOSED_BY_NAME_FIELD, mainAbstractAggregateScope
                         .getClosedByName(), ValidationConstants.ABSTRACTAGGREGATESCOPE_CLOSED_BY_NAME_MAX, msgCtx);
        	}
        }

        // Field validation: estimatedStartDate and estimatedFinishDate
        if (mainAbstractAggregateScope.testEstimatedStartDateModified()
                || mainAbstractAggregateScope.testEstimatedFinishDateModified())
        {
            GenericValidator
                .validateDates(
                    mainAbstractAggregateScope,
                    ValidationConstants.ABSTRACTAGGREGATESCOPE_ESTIMATED_START_DATE_FIELD,
                    mainAbstractAggregateScope.getEstimatedStartDate(),
                    ValidationConstants.ABSTRACTAGGREGATESCOPE_ESTIMATED_FINISH_DATE_FIELD,
                    mainAbstractAggregateScope.getEstimatedFinishDate(), msgCtx);
        }

        // Field validation: priority
        if (mainAbstractAggregateScope.testPriorityModified())
        {
            GenericValidator.validateRange(mainAbstractAggregateScope,
                ValidationConstants.ABSTRACTAGGREGATESCOPE_PRIORITY_FIELD,
                mainAbstractAggregateScope.getPriority(),
                ValidationConstants.PRIORITY_MIN,
                ValidationConstants.PRIORITY_MAX, msgCtx);
        }

        // Field validation: proposedByName
        if (mainAbstractAggregateScope.testProposedByNameModified())
        {
        	if(mainAbstractAggregateScope.getProposedByName() != null)
        	{
        	 GenericValidator.validateMaxLength(mainAbstractAggregateScope,
                     ValidationConstants.ABSTRACTAGGREGATESCOPE_PROPOSED_BY_NAME_FIELD, mainAbstractAggregateScope
                         .getProposedByName(), ValidationConstants.ABSTRACTAGGREGATESCOPE_PROPOSED_BY_NAME_MAX, msgCtx);
        	}
        }
    }
}