/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.types.PublishedType;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.AbstractRequest;
import com.ibm.rpm.scopemanagement.containers.ApprovedCostBenefits;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;

/**
 * @author dominiqu
 */
public abstract class AbstractRequestCheckpoint extends AbstractScopeCheckpoint
{
    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx);

        // Initialize
        AbstractRequest mainAbstractRequest = (AbstractRequest) rpmObject;
        ScopeElementScope scope = (ScopeElementScope) rpmScope;

        // Container validation: approvedCostBenefits
        if (scope != null && scope.isApprovedCostBenefits())
        {
            ApprovedCostBenefits approvedCostBenefits = mainAbstractRequest.getApprovedCostBenefits();
            if( approvedCostBenefits == null )
            {
                if ( rpmObject.getID() != null )
                {
                    // field can't be reset to null after object creation
                    GenericValidator.validateNotNull(rpmObject, "approvedCostBenefits", approvedCostBenefits, msgCtx);
                }
            }
            else
            {
                ApprovedCostBenefitsCheckpoint.getInstance().validateSave(
                    approvedCostBenefits, msgCtx);
            }
        }
        
        // Field validation: numberTimesReassigned
        if (mainAbstractRequest.testNumberTimesReassignedModified())
        {
            GenericValidator
                .validateRange(
                    mainAbstractRequest,
                    ValidationConstants.ABSTRACTREQUEST_NUMBER_TIMES_REASSIGNED_FIELD,
                    mainAbstractRequest.getNumberTimesReassigned(),
                    ValidationConstants.NUMBEROFTIMESREASSIGNED_MIN,
                    ValidationConstants.NUMBEROFTIMESREASSIGNED_MAX, msgCtx);
        }
    }
    
    public void validateAbstractRequestCreation(AbstractRequest inputObject,
            MessageContext msgCtx)
    {    	
	        //validate template is in fact a template
	        if (((AbstractRequest) msgCtx.getTemplate()).getPublished() == null
				|| !(PublishedType.ToTemplate.getValue()
						.equals(((AbstractRequest) msgCtx.getTemplate())
								.getPublished().getValue()))) {
			String[] errorParams = { StringUtil.getShortClassName(inputObject
					.getClass())
					+ " " + ValidationConstants.TEMPLATE_FIELD

			};
			//invalid project template
			RPMException e = new RPMException(
					ErrorCodes.INVALID_OBJECT_TEMPLATE_TYPE,
					errorParams, inputObject.getClass().getName(),
					ValidationConstants.TEMPLATE_FIELD, inputObject.getID());
			addException(e, msgCtx);
		} 
	        else {
	        	validateAbstractRequestReadOnlyFields(inputObject, msgCtx);

		}
    }
    
    /**
	 * @param inputObject
	 * @param msgCtx
	 */
	protected void validateAbstractRequestReadOnlyFields(AbstractRequest inputObject, MessageContext msgCtx) {

		super.validateAbstractScopeReadOnlyFields(inputObject, msgCtx);
		
		//-----Common AbstractRequest fiels validation
		
		//Field validation: Cost Benefits
		if (inputObject.testApprovedCostBenefitsModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.APPROVED_COST_BENEFITS_FIELD, msgCtx);
		}
		
		//Field validation: Number Times Reassigned
		if (inputObject.testNumberTimesReassignedModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.NUMBER_TIMES_REASSIGNED_FIELD, msgCtx);
		}
		
		
		//Field validation: Occured Date Time
		if (inputObject.testOccuredDateTimeModified())
		{
		    GenericValidator.validateReadOnly(inputObject,
		        ValidationConstants.OCCURED_DATE_TIME_FIELD, msgCtx);
		}
		
		
	}

}