/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMContainer;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.ApprovedCostBenefits;

/**
 * @author dominiqu
 */
public class ApprovedCostBenefitsCheckpoint
{
    protected static ApprovedCostBenefitsCheckpoint instance = new ApprovedCostBenefitsCheckpoint();

    public static ApprovedCostBenefitsCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMContainer rpmContainer, MessageContext msgCtx)
    {
        // Initialize
        ApprovedCostBenefits mainACB = (ApprovedCostBenefits) rpmContainer;

        // Note: setModified does not set the inner fields of children classes
        // as modified so we cannot check if they have been modified before
        // validating

        // Field validation: billableRevenue
        GenericValidator.validateRange(mainACB,
            ValidationConstants.APPROVEDCOSTBENEFITS_BILLABLE_REVENUE_FIELD,
            mainACB.getBillableRevenue(),
            ValidationConstants.ACB_BILLABLEREVENUE_MIN,
            ValidationConstants.ACB_BILLABLEREVENUE_MAX, msgCtx);

        // Field validation: cost
        GenericValidator.validateRange(mainACB,
            ValidationConstants.APPROVEDCOSTBENEFITS_COST_FIELD, mainACB
                .getCost(), ValidationConstants.ACB_COST_MIN,
            ValidationConstants.ACB_COST_MAX, msgCtx);

        // Field validation: nonBillableBenefits
        GenericValidator
            .validateRange(
                mainACB,
                ValidationConstants.APPROVEDCOSTBENEFITS_NON_BILLABLE_BENEFITS_FIELD,
                mainACB.getNonBillableBenefits(),
                ValidationConstants.ACB_NONBILLABLEBENEFITS_MIN,
                ValidationConstants.ACB_NONBILLABLEBENEFITS_MAX, msgCtx);

        // Field validation: scheduleImpactInDays
        GenericValidator
            .validateRange(
                mainACB,
                ValidationConstants.APPROVEDCOSTBENEFITS_SCHEDULE_IMPACT_IN_DAYS_FIELD,
                mainACB.getScheduleImpactInDays(),
                ValidationConstants.ACB_SCHEDULEIMPACTINDAYS_MIN,
                ValidationConstants.ACB_SCHEDULEIMPACTINDAYS_MAX, msgCtx);

        // Field validation: workInHours
        GenericValidator.validateRange(mainACB,
            ValidationConstants.APPROVEDCOSTBENEFITS_WORK_IN_HOURS_FIELD,
            mainACB.getWorkInHours(), ValidationConstants.ACB_WORKINHOURS_MIN,
            ValidationConstants.ACB_WORKINHOURS_MAX, msgCtx);
    }
}