/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMContainer;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Impact;

/**
 * @author dominiqu
 */
public class ImpactCheckpoint extends AbstractCheckpoint
{
    protected static ImpactCheckpoint instance = new ImpactCheckpoint();

    public static ImpactCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMContainer rpmContainer, MessageContext msgCtx)
    {
        // Initialize
        Impact mainImpact = (Impact) rpmContainer;

        // Field validation: orderOfMagnitudeHighWorkInHours
        if (mainImpact.testOrderOfMagnitudeHighWorkInHoursModified())
        {
            GenericValidator
                .validateRange(
                    mainImpact,
                    ValidationConstants.IMPACT_ORDER_OF_MAGNITUDE_HIGH_WORK_IN_HOURS_FIELD,
                    mainImpact.getOrderOfMagnitudeHighWorkInHours(),
                    ValidationConstants.IMPACT_OOMHIGHWORKINHOURS_MIN,
                    ValidationConstants.IMPACT_OOMHIGHWORKINHOURS_MAX, msgCtx);
        }

        // Field validation: orderOfMagnitudeLowWorkInHours
        if (mainImpact.testOrderOfMagnitudeLowWorkInHoursModified())
        {
            GenericValidator
                .validateRange(
                    mainImpact,
                    ValidationConstants.IMPACT_ORDER_OF_MAGNITUDE_LOW_WORK_IN_HOURS_FIELD,
                    mainImpact.getOrderOfMagnitudeLowWorkInHours(),
                    ValidationConstants.IMPACT_OOMLOWWORKINHOURS_MIN,
                    ValidationConstants.IMPACT_OOMLOWWORKINHOURS_MAX, msgCtx);
        }

        // Field validation: orderOfMagnitudeWorkInHours
        if (mainImpact.testOrderOfMagnitudeWorkInHoursModified())
        {
            GenericValidator
                .validateRange(
                    mainImpact,
                    ValidationConstants.IMPACT_ORDER_OF_MAGNITUDE_WORK_IN_HOURS_FIELD,
                    mainImpact.getOrderOfMagnitudeWorkInHours(),
                    ValidationConstants.IMPACT_OOMWORKINHOURS_MIN,
                    ValidationConstants.IMPACT_OOMWORKINHOURS_MAX, msgCtx);
        }

        // Field validation: scheduledImpactInDays
        if (mainImpact.testScheduledImpactInDaysModified())
        {
            GenericValidator.validateRange(mainImpact,
                ValidationConstants.IMPACT_SCHEDULED_IMPACT_IN_DAYS_FIELD,
                mainImpact.getScheduledImpactInDays(),
                ValidationConstants.IMPACT_SCHEDULEIMPACTINDAYS_MIN,
                ValidationConstants.IMPACT_SCHEDULEIMPACTINDAYS_MAX, msgCtx);
        }
    }
}