/*
 * Created on 7-Dec-2005
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.MitigationFactor;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.scope.MitigationFactorScope;

/**
 * @author dominiqu
 */
public class MitigationFactorCheckpoint extends AbstractCheckpoint
{
    private static final Class MITIGATIONFACTOR_PARENTS = Risk.class;

    protected static MitigationFactorCheckpoint instance = new MitigationFactorCheckpoint();

    public static MitigationFactorCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, false,
            MITIGATIONFACTOR_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject,
            MitigationFactor.class, msgCtx))
            return;

        // Initialize
        MitigationFactor mainMitigationFactor = (MitigationFactor) rpmObject;
        MitigationFactorScope scope = (MitigationFactorScope) rpmScope;

        // Field validation: description
        if (mainMitigationFactor.testDescriptionModified())
        {
            GenericValidator.validateMaxLength(mainMitigationFactor,
                ValidationConstants.DESCRIPTION_FIELD, mainMitigationFactor
                    .getDescription(),
                ValidationConstants.MITIGATIONFACTOR_DESCRIPTION_MAX, msgCtx);
        }

        // Field validation: mitigationCost
        if (mainMitigationFactor.testMitigationCostModified())
        {
            GenericValidator
                .validateRange(mainMitigationFactor,
                    ValidationConstants.MITIGATIONFACTOR_MITIGATION_COST_FIELD,
                    mainMitigationFactor.getMitigationCost(),
                ValidationConstants.MITIGATIONFACTOR_MITIGATIONCOST_MIN,
                    ValidationConstants.MITIGATIONFACTOR_MITIGATIONCOST_MAX,
                    msgCtx);
        }

        // Field validation: priority
        if (mainMitigationFactor.testPriorityModified())
        {
            GenericValidator.validateRange(mainMitigationFactor,
                ValidationConstants.MITIGATIONFACTOR_PRIORITY_FIELD,
                mainMitigationFactor.getPriority(),
                ValidationConstants.MITIGATIONFACTOR_PRIORITY_MIN,
                ValidationConstants.MITIGATIONFACTOR_PRIORITY_MAX, msgCtx);
        }

        // Field validation: mitigationTriggers
        if (scope != null && scope.isMitigationTriggers()
                && mainMitigationFactor.getMitigationTriggers() != null)
        {
            MitigationTriggerCheckpoint mtc = MitigationTriggerCheckpoint
                .getInstance();
            for (int i = 0; i < mainMitigationFactor.getMitigationTriggers().length; ++i)
            {
                if (mainMitigationFactor.getMitigationTriggers()[i] != null)
                {
                    mtc.validateSave(mainMitigationFactor
                        .getMitigationTriggers()[i], rpmScope, msgCtx);
                }
            }
        }
    }
}
