/*
 * Created on Aug 18, 2006
 *
 */
package com.ibm.rpm.scopemanagement.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.ReqProImportStatus;
import com.ibm.rpm.scopemanagement.containers.ReqProRequirement;
import com.ibm.rpm.scopemanagement.containers.ReqProScopeFolder;

/**
 * @author karim
 *
 */
public class ReqProImportStatusCheckpoint extends AbstractCheckpoint{

	//TODO add read only constraint on ReqProID
    private static final List REQPRO_IMPORTSTATUS_PARENTS = new ArrayList();

    static
    {
    	REQPRO_IMPORTSTATUS_PARENTS.add(ReqProScopeFolder.class);
    	REQPRO_IMPORTSTATUS_PARENTS.add(ReqProRequirement.class);
    }
	
    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        // Initialize
        ReqProImportStatus reqProImportStatus = (ReqProImportStatus) rpmObject;
    	
        boolean isNew = reqProImportStatus.getID()==null;
        
        if(isNew)
        {
            super.isValidParent(rpmObject, msgCtx, true, REQPRO_IMPORTSTATUS_PARENTS);        	
        }
        else
        {
        	//parent is always required in the create mode
            super.isValidParent(rpmObject, msgCtx, false, REQPRO_IMPORTSTATUS_PARENTS);
        }

        // Field validation: ReqProID
        if(isNew || reqProImportStatus.testReqProIDModified())
        {
        	if(!isNew)
        	{
        		GenericValidator.validateReadOnly(reqProImportStatus,
                        ValidationConstants.REQPRO_ID_FIELD,
                        msgCtx);
        	}
        	GenericValidator.validateMandatory(reqProImportStatus,
					ValidationConstants.REQPRO_ID_FIELD, reqProImportStatus
							.getReqProID(), msgCtx);
        	GenericValidator.validateMinLength(reqProImportStatus,
                    ValidationConstants.REQPRO_ID_FIELD,
                    reqProImportStatus.getReqProID(), ValidationConstants.REQPRO_ID_MIN,
                    msgCtx);
        	GenericValidator.validateMaxLength(reqProImportStatus,
                    ValidationConstants.REQPRO_ID_FIELD,
                    reqProImportStatus.getReqProID(), ValidationConstants.REQPRO_ID_MAX,
                    msgCtx);
        }
        // Field validation: ReqProLastImportedDateTime
        if(isNew || reqProImportStatus.testReqProLastImportedDateTimeModified())
        {
        	GenericValidator.validateMandatory(reqProImportStatus,
					ValidationConstants.REQPRO_LASTIMPORTEDDATETIME_FIELD,
					reqProImportStatus.getReqProLastImportedDateTime(), msgCtx);
        }
    }
}

