/*
 * Created on Aug 30, 2006
 *
 */
package com.ibm.rpm.scopemanagement.managers;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Collection;
import java.util.HashMap;
import java.util.TreeSet;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.scopemanagement.containers.ReqProConnection;
import com.ibm.rpm.scopemanagement.containers.ReqProImportStatus;
import com.ibm.rpm.scopemanagement.containers.ReqProIntegrationFolder;
import com.ibm.rpm.scopemanagement.containers.ReqProRequirement;
import com.ibm.rpm.scopemanagement.containers.ReqProScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.util.ReqProUtil;
import com.ibm.rpm.xpathparser.XPathContainer;

/**
 * @author karim
 *
 */
public class ReqProManager extends AbstractRPMObjectManager {

    // =============
    // CONTAINER MAP
    // =============

    private static ContainerMap containerMap = new ContainerMap(
    		ReqProManager.class);
    static
    {
        containerMap.add(ReqProConnection.class);
        containerMap.add(ReqProImportStatus.class);
    }
	
    
    // =========
    // FIELD MAP
    // =========

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    /**
     * Column CRI_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CRI_ID = 1;

    public static final int TYPE_CRI_ID = Types.CHAR;

    public static final String NAME_CRI_ID = "REQPRO_CRI.CRI_ID";
    public static final String PROPERTY_CRI_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CRI_ID, NAME_CRI_ID);
    }
    
    /**
     * Column REQPRO_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_REQPRO_ID = 2;

    public static final int TYPE_REQPRO_ID = Types.CHAR;

    public static final String NAME_REQPRO_ID = "REQPRO_CRI.REQPRO_ID";
    
    public static final String PROPERTY_REQPRO_ID = "REQPROID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REQPRO_ID, NAME_REQPRO_ID);
    }
    
    /**
     * Column REQPRO_SERVERURL of type Types.VARCHAR mapped to String.
     */
    public static final int ID_REQPRO_SERVERURL = 3;

    public static final int TYPE_REQPRO_SERVERURL = Types.VARCHAR;

    public static final String NAME_REQPRO_SERVERURL = "REQPRO_CRI.REQPRO_SERVERURL";
    
    
    /**
     * Column REQPRO_PROJECTPATH of type Types.VARCHAR mapped to String.
     */
    public static final int ID_REQPRO_PROJECTPATH = 4;

    public static final int TYPE_REQPRO_PROJECTPATH = Types.VARCHAR;

    public static final String NAME_REQPRO_PROJECTPATH = "REQPRO_CRI.REQPRO_PROJECTPATH";
    
    
    /**
     * Column LOGON of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LOGON = 5;

    public static final int TYPE_LOGON = Types.VARCHAR;

    public static final String NAME_LOGON = "REQPRO_CRI.LOGON";
    
    
    /**
     * Column PASSWORD of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PASSWORD = 6;

    public static final int TYPE_PASSWORD = Types.VARCHAR;

    public static final String NAME_PASSWORD = "REQPRO_CRI.PASSWORD";
    
    
    /**
     * Column REQPRO_LASTIMPORTEDDATETIME of type Types.TIMESTAMP mapped to String.
     */
    public static final int ID_REQPRO_LASTIMPORTEDDATETIME = 7;

    public static final int TYPE_REQPRO_LASTIMPORTEDDATETIME = Types.TIMESTAMP;

    public static final String NAME_REQPRO_LASTIMPORTEDDATETIME = "REQPRO_CRI.REQPRO_LASTIMPORTEDDATETIME";
    

    /**
     * Column SUB_TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SUB_TYPE_ID = 8;

    public static final int TYPE_SUB_TYPE_ID = Types.INTEGER;

    public static final String NAME_SUB_TYPE_ID = "REQPRO_CRI.SUB_TYPE_ID";
    
    /**
     * Column CRI_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_REQPRO_PARENT_ID = 9;

    public static final int TYPE_REQPRO_PARENT_ID = Types.CHAR;

    public static final String NAME_REQPRO_PARENT_ID = "REQPRO_CRI.REQPRO_PARENT_ID";
    
    public static final String PROPERTY_REQPRO_PARENT_ID = "REQPROPARENTID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REQPRO_PARENT_ID, NAME_REQPRO_PARENT_ID);
    }
    
    
    /**
     * Column CRI_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_REQPRO_VERSION_NUMBER = 10;

    public static final int TYPE_REQPRO_VERSION_NUMBER = Types.VARCHAR;

    public static final String NAME_REQPRO_VERSION_NUMBER = "REQPRO_CRI.REQPRO_VERSION_NUMBER";
    
    /**
     * Column CRI_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CREATE_ON_UPDATE = 11;

    public static final int TYPE_CREATE_ON_UPDATE = Types.CHAR;

    public static final String NAME_CREATE_ON_UPDATE = "REQPRO_CRI.CREATE_ON_UPDATE";
    
    /**
     * Column CRI_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_DELETE_ON_REMOVED = 12;

    public static final int TYPE_DELETE_ON_REMOVED = Types.CHAR;

    public static final String NAME_DELETE_ON_REMOVED = "REQPRO_CRI.DELETE_ON_REMOVED";
    
    
    public static final String TABLE_NAME = "REQPRO_CRI";
    
    /**
     * Create an array of type string containing all the fields of the REQPRO_CRI
     * table.
     */
    private static final String[] FIELD_NAMES =
    {
    		NAME_CRI_ID, NAME_REQPRO_ID, NAME_REQPRO_SERVERURL, NAME_REQPRO_PROJECTPATH,
    		NAME_LOGON, NAME_PASSWORD, NAME_REQPRO_LASTIMPORTEDDATETIME, 
			NAME_SUB_TYPE_ID, NAME_REQPRO_PARENT_ID, NAME_REQPRO_VERSION_NUMBER,
            NAME_CREATE_ON_UPDATE, NAME_DELETE_ON_REMOVED
    };
    
    private static final String ALL_FIELDS;

    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);

        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }

        ALL_FIELDS = fieldcat.toString();
    }
    
    /**
     * Creates a new ReqProManager instance.
     *
     * @return WBSManager
     */
    public ReqProManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_CRI_ID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet) throws SQLException
    {
        int typeId = 0;
        return new Integer(typeId);
    }
    
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        SqlBuffer baseCondition = new SqlBuffer();
        
        if(container.getType().equals(ReqProConnection.class))
        {
        	baseCondition.appendEqual(NAME_SUB_TYPE_ID, new Integer(ReqProIntegrationFolder.SUB_TYPE_ID));
        }
        else
        {
        	Collection subTypeIds = new TreeSet();
        	subTypeIds.add(new Integer(ReqProScopeFolder.SUB_TYPE_ID));
        	subTypeIds.add(new Integer(ReqProRequirement.SUB_TYPE_ID));
        	baseCondition.appendIn(NAME_SUB_TYPE_ID, subTypeIds);
        }
        
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition(TABLE_NAME, baseCondition.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }
    
    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager, XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition join = null;
        
        if(manager instanceof ScopeManager)
        {
            join = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append("ELEMENT_ID IN (");
            where.append(" SELECT ").append(NAME_CRI_ID);
            where.append(" FROM ").append(getTableName());
            
            join.setCondition(where.toString());
        }    
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }
        return join;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        int subTypeID = resultSet.getInt(ID_SUB_TYPE_ID);  

        if(subTypeID!=0)
        {
        	Integer iSubTypeID = new Integer(subTypeID);
            //Here if the subTypeID != 0, we use it to 
            //create the ReqPro containers instead of the typeId
        	if(ReqProUtil.isReqProIntegrationFolder(iSubTypeID))
        	{
        		return new ReqProConnection();
        	}
        	else if(ReqProUtil.isReqProScopeFolder(iSubTypeID) || ReqProUtil.isReqProRequirement(iSubTypeID))
        	{
        		return new ReqProImportStatus();
        	}
        }
		ExceptionUtil.handleUnsupportedSubTypeId(this, subTypeID);
       	return null;
    }

    public RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws SQLException
    {
    	rpmObject.setID(rs.getString(ID_CRI_ID));
    	
    	String reqProID = rs.getString(ID_REQPRO_ID);
    	if(reqProID!=null)
    	{
    		reqProID = reqProID.trim();
    	}
    	
    	if(rpmObject instanceof ReqProConnection)
    	{
    		ReqProConnection reqProConnection = (ReqProConnection)rpmObject;
    		
    		if(isDelta)
    		{
        		reqProConnection.deltaReqProID(reqProID);
    			reqProConnection.deltaReqProLogon(rs.getString(ID_LOGON));
    			reqProConnection.deltaReqProPassword(rs.getString(ID_PASSWORD));
    			reqProConnection.deltaReqProProjectPath(rs.getString(ID_REQPRO_PROJECTPATH));
    			reqProConnection.deltaReqProServerURL(rs.getString(ID_REQPRO_SERVERURL));
                reqProConnection.deltaCreateNewWhenUpdated(decodeCreateNewWhenUpdated(rs));
                reqProConnection.deltaDeleteOnServerRemoval(decodeDeleteOnServerRemoval(rs));
    		} 
    		else
    		{
        		reqProConnection.setReqProID(reqProID);
    			reqProConnection.setReqProLogon(rs.getString(ID_LOGON));
    			reqProConnection.setReqProPassword(rs.getString(ID_PASSWORD));
    			reqProConnection.setReqProProjectPath(rs.getString(ID_REQPRO_PROJECTPATH));
    			reqProConnection.setReqProServerURL(rs.getString(ID_REQPRO_SERVERURL));    
                reqProConnection.setCreateNewWhenUpdated(decodeCreateNewWhenUpdated(rs));
                reqProConnection.setDeleteOnServerRemoval(decodeDeleteOnServerRemoval(rs));
    		}
    	}
    	else
        // ReqProImportStatus
        {
            ReqProImportStatus reqProImportStatus = (ReqProImportStatus) rpmObject;
            if (isDelta)
            {
                reqProImportStatus.deltaReqProID(reqProID);
                reqProImportStatus.deltaReqProLastImportedDateTime(Manager
                    .getTimestamp(rs, ID_REQPRO_LASTIMPORTEDDATETIME));
                reqProImportStatus.deltaReqProParentID(rs
                    .getString(ID_REQPRO_PARENT_ID));
                reqProImportStatus.deltaReqProVersion(rs
                    .getString(ID_REQPRO_VERSION_NUMBER));
            }
            else
            {
                reqProImportStatus.setReqProID(reqProID);
                reqProImportStatus.setReqProLastImportedDateTime(Manager
                    .getTimestamp(rs, ID_REQPRO_LASTIMPORTEDDATETIME));
                reqProImportStatus.setReqProParentID(rs
                    .getString(ID_REQPRO_PARENT_ID));
                reqProImportStatus.setReqProVersion(rs
                    .getString(ID_REQPRO_VERSION_NUMBER));
            }
        }

        return rpmObject;
    }

    private Boolean decodeDeleteOnServerRemoval(ResultSet rs) throws SQLException
    {
        Boolean retVal = Boolean.FALSE;
        String dbVal = rs.getString(ID_DELETE_ON_REMOVED);
        
        if ("Y".equalsIgnoreCase(dbVal))
        {
            retVal = Boolean.TRUE;
        }
        
        return retVal;
    }

    private Boolean decodeCreateNewWhenUpdated(ResultSet rs) throws SQLException
    {
        Boolean retVal = Boolean.FALSE;
        String dbVal = rs.getString(ID_CREATE_ON_UPDATE);
        
        if ("Y".equalsIgnoreCase(dbVal))
        {
            retVal = Boolean.TRUE;
        }
        
        return retVal;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException, SQLException
    {
        String parentId = resultSet.getString(ID_CRI_ID);
        relatedFields.put(index, ID_CRI_ID, parentId);
        
        return relatedFields;
    }
    
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
    	if(rpmObject instanceof ReqProConnection)
    	{
    		ReqProConnection reqProConnection = (ReqProConnection)rpmObject;
        	//load decrypted password using the SP
    		String password = decodePassword(reqProConnection, messageContext);
    		if(isDelta)
    		{
    			reqProConnection.deltaReqProPassword(password);
    		}
    		else
    		{
    			reqProConnection.setReqProPassword(password);
    		}
    	}

    	if(rpmScope!=null && rpmScope.getParent()!=null)
    	{
            ScopeManager parentManager = (ScopeManager) getManagerInstance(ScopeElement.class
                    .getName());
        	
            String oldParentId = (String) relatedFields.get(index, ID_CRI_ID);
    		RPMObject newParent = null;
    		if (isDelta) {
    			newParent = rpmObject.getParent();
    			RPMObject oldParent = null;
    			if (oldParentId != null) {
    				oldParent = new ScopeElement();
    				oldParent.setID(oldParentId);
    				if (newParent != null && newParent.getID() != null
    						&& newParent.getID().equals(oldParentId)) {
    					oldParent = parentManager.loadByPrimaryKey(newParent, rpmScope.getParent(),
    							messageContext, true);
    				} else {
    					//In this case, old parent is loaded without the
    					// scope to avoid problem with the cache
    					oldParent = parentManager.loadByPrimaryKey(oldParent, rpmScope.getParent(),
    							messageContext, false);
    				}
    			}
    			rpmObject.deltaParent(oldParent);
    		} else {
    			if (oldParentId != null) {
    				if (newParent == null) {//newParent not in cache
    					newParent = new ScopeElement();
    					newParent.setID(oldParentId);
    					newParent = parentManager.loadByPrimaryKey(newParent, rpmScope
    							.getParent(), messageContext, false);
    				}
    				rpmObject.setParent(newParent);
    			}
    		}    		
    	}
		return rpmObject;
    }
    
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {	
    	if(rpmObject instanceof ReqProConnection)
    	{
            rpmObject.setID(SP_I_REQPRO_CRI_INTEGRATION((ReqProConnection)rpmObject, messageContext));    		
    	}
    	else if (rpmObject instanceof ReqProImportStatus)
    	{
            rpmObject.setID(SP_I_REQPRO_CRI_ELEMENT((ReqProImportStatus)rpmObject, messageContext));
    	}
    }
    
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
    	if(rpmObject instanceof ReqProConnection)
    	{
    		updateReqProConnection((ReqProConnection)rpmObject, messageContext);   		
    	}
    	else if (rpmObject instanceof ReqProImportStatus)
    	{
    		updateReqProImportStatus((ReqProImportStatus)rpmObject, messageContext);
    	}
        return rpmObject;
    }
    
    private void updateReqProConnection(ReqProConnection reqProConnection,
            MessageContext messageContext) throws SQLException, RPMException
    {
    	if (reqProConnection.testReqProServerURLModified()
				|| reqProConnection.testReqProProjectPathModified()
				|| reqProConnection.testReqProLogonModified()
				|| reqProConnection.testReqProPasswordModified())
    	{
    		SP_U_CRI_REQPRO_INTEGRATION(reqProConnection, messageContext);
    	}
    }
    
    private void updateReqProImportStatus(ReqProImportStatus reqProImportStatus,
            MessageContext messageContext) throws SQLException, RPMException
    {
    	if (reqProImportStatus.testReqProLastImportedDateTimeModified())
    	{
    		SP_U_CRI_REQPRO_ELEMENT(reqProImportStatus, messageContext);
    	}
    }
    
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
			throws RPMException, SQLException 
	{
        ExceptionUtil.handleNoDelete(this, rpmObject);
	}
    
    private String SP_I_REQPRO_CRI_INTEGRATION(
			ReqProConnection reqProConnection, MessageContext messageContext)
			throws SQLException, RPMException {
		String name = "SP_I_REQPRO_CRI_INTEGRATION";
		Object args[] = new Object[9];
		args[0] = reqProConnection.getParent().getID();
		args[1] = reqProConnection.getReqProID();
		args[2] = ReqProUtil.getSubTypeIDFromParent(reqProConnection);
		args[3] = reqProConnection.getReqProServerURL();
		args[4] = reqProConnection.getReqProProjectPath();
		args[5] = reqProConnection.getReqProLogon();
		args[6] = reqProConnection.getReqProPassword();
		args[7] = null;
		args[8] = getUser(messageContext).getID();
		return executeProcedure(messageContext, name, args);
	}
    
    private String SP_I_REQPRO_CRI_ELEMENT(ReqProImportStatus reqProImportStatus,
            MessageContext messageContext) throws SQLException, RPMException
    {
		String name = "SP_I_REQPRO_CRI_ELEMENT";
		Object args[] = new Object[7];
		args[0] = reqProImportStatus.getParent().getID();
		args[1] = reqProImportStatus.getReqProID();
        args[2] = reqProImportStatus.getReqProParentID(); 
		args[3] = ReqProUtil.getSubTypeIDFromParent(reqProImportStatus);
		args[4] = null;
        args[5] = reqProImportStatus.getReqProVersion();
		args[6] = getUser(messageContext).getID();
		return executeProcedure(messageContext, name, args);
    }
    
    
    
    private void SP_U_CRI_REQPRO_INTEGRATION(ReqProConnection reqProConnection,
            MessageContext messageContext) throws SQLException, RPMException
    {
		String name = "SP_U_REQPRO_CRI_INTEGRATION";
		Object args[] = new Object[6];
		args[0] = reqProConnection.getID();
		args[1] = reqProConnection.getReqProServerURL();
		args[2] = reqProConnection.getReqProProjectPath();
		args[3] = reqProConnection.getReqProLogon();
		args[4] = reqProConnection.getReqProPassword();
		args[5] = getUser(messageContext).getID();
		executeProcedure(messageContext, name, args);
    }
    
    private void SP_U_CRI_REQPRO_ELEMENT(ReqProImportStatus reqProImportStatus,
            MessageContext messageContext) throws SQLException, RPMException
    {
		String name = "SP_U_REQPRO_CRI_ELEMENT";
		Object args[] = new Object[4];
		args[0] = reqProImportStatus.getID();
		args[1] = DateUtil.convertToStringWithTime(reqProImportStatus.getReqProLastImportedDateTime()); 
        args[2] = reqProImportStatus.getReqProVersion();
		args[3] = getUser(messageContext).getID();
		executeProcedure(messageContext, name, args);
    }
    
    private String decodePassword(ReqProConnection reqProConnection,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_REQPRO_SELECT";
        String value = null;
        Object args[] = new Object[1];

        args[0] = reqProConnection.getID();
        
        Connection c = null;
        ResultSet rs = null;
        try
        {
            c = Manager.getConnection(messageContext);
            rs = Manager.executeStoredProcedure(c, name, args, true,
                messageContext);
            if (rs.next())
            {
                value = rs.getString(7);
            }
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return value;
    }
}
