/*
 * Created on Oct 25, 2005
 *
 */
package com.ibm.rpm.scopemanagement.managers;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.ActionRTF;
import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestRTF;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestState;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.DefectRTF;
import com.ibm.rpm.applicationadministration.containers.DefectState;
import com.ibm.rpm.applicationadministration.containers.DeliverableRTF;
import com.ibm.rpm.applicationadministration.containers.IssueRTF;
import com.ibm.rpm.applicationadministration.containers.IssueState;
import com.ibm.rpm.applicationadministration.containers.MilestoneRTF;
import com.ibm.rpm.applicationadministration.containers.RequirementRTF;
import com.ibm.rpm.applicationadministration.containers.RequirementState;
import com.ibm.rpm.applicationadministration.containers.RiskRTF;
import com.ibm.rpm.applicationadministration.containers.RiskState;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestRTF;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestState;
import com.ibm.rpm.applicationadministration.containers.SummaryTaskRTF;
import com.ibm.rpm.applicationadministration.containers.TaskRTF;
import com.ibm.rpm.applicationadministration.managers.AttributeManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.managers.CustomFieldManager;
import com.ibm.rpm.customfield.scope.CustomFieldScope;
import com.ibm.rpm.framework.AbstractModuleManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeManagementModule;
import com.ibm.rpm.scopemanagement.scope.ScopeManagementModuleScope;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.managers.ScorecardManager;

/**
 * @author karim
 *  
 */
public class ScopeManagementModuleManager extends AbstractModuleManager
{
    private static Log logger = LogFactory
        .getLog(ScopeManagementModuleManager.class);

    protected Log getLogger()
    {
        return logger;
    }

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(ScopeManagementModule.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ScopeManagementModuleManager.class);
        }
    }

    {
        localContextName = ContextUtil.SCOPE_CONTEXT;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = new ScopeManagementModule();
        return object;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof AttributeManager)
        {
            if (localContextName.equalsIgnoreCase(contextName))
            {
                joinCondition = new JoinCondition();
            }
            else
            {//context not linked to this manager
                logger
                    .error("ScopeManagementModuleManager.getJoinCondition: contextName "
                            + contextName + " is not linked with this manager.");
            }
        }
        else if (manager instanceof CustomFieldManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setCondition(CustomFieldManager
                .createJoinCondition(localContextName));
        }
        else if (manager instanceof ScorecardManager)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof ScopeManager)
        {
            joinCondition = new JoinCondition();
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected void loadChildren(AbstractModule abstractModule,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        ScopeManagementModule module = (ScopeManagementModule) abstractModule;
        ScopeManagementModuleScope scope = (ScopeManagementModuleScope) rpmScope;

        DatafieldManager datafieldManager = (DatafieldManager) getManagerInstance(DatafieldCategory.class);

        if (scope != null)
        {
            //loadScopeFolders
            if (scope.getScopeFolders() != null)
            {
                String extraWhere = " TYPE_ID = " + ScopeFolder.TYPE_ID;
                ScopeFolder[] scopeFolders = (ScopeFolder[]) loadElements(
                        module, module.getScopeFolders(), scope
                                .getScopeFolders(), ScopeFolder.class, null,
                        extraWhere, messageContext, isDelta);
                module.setScopeFolders(scopeFolders);
            }

            //loadAttributes
            if (scope.getAttributeCategories() != null)
            {
                AttributeManager attributeManager = (AttributeManager) getManagerInstance(AttributeCategory.class);

                AttributeCategory[] attributeCategory = attributeManager
                    .loadAttributeCategories(module, module.getAttributeCategories(), scope
                        .getAttributeCategories(), this, localContextName,
                        messageContext, isDelta);
                module.setAttributeCategories(attributeCategory);
            }

            CustomFieldScope customFieldScope = scope
                .getCustomFieldCategories();
            if (customFieldScope != null)
            {
                CustomFieldManager manager = (CustomFieldManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(
                        CustomFieldCategory.class);
                CustomFieldCategory[] categories = manager
                    .loadCustomFieldCategories(localContextName,
                        customFieldScope, messageContext);
                module.setCustomFieldCategories(categories);
            }

            if (scope.isChangeRequestStates())
            {
                ChangeRequestState[] changeRequestStates = (ChangeRequestState[]) datafieldManager
                    .loadDatafields(module, module.getChangeRequestStates(),
                        ChangeRequestState.class, ChangeRequestState.TYPE_ID,
                        this, localContextName, messageContext, isDelta);
                module.setChangeRequestStates(changeRequestStates);
            }
            if (scope.isIssueStates())
            {
                IssueState[] issueStates = (IssueState[]) datafieldManager
                    .loadDatafields(module, module.getIssueStates(), IssueState.class,
                        IssueState.TYPE_ID, this, localContextName,
                        messageContext, isDelta);
                module.setIssueStates(issueStates);
            }
            if (scope.isRiskStates())
            {
                RiskState[] riskStates = (RiskState[]) datafieldManager
                    .loadDatafields(module, module.getRiskStates(), RiskState.class,
                        RiskState.TYPE_ID, this, localContextName,
                        messageContext, isDelta);
                module.setRiskStates(riskStates);
            }
            if (scope.isRequirementStates())
            {
                RequirementState[] requirementStates = (RequirementState[]) datafieldManager
                    .loadDatafields(module, module.getRequirementStates(),
                        RequirementState.class, RequirementState.TYPE_ID,
                        this, localContextName, messageContext, isDelta);
                module.setRequirementStates(requirementStates);
            }
            if (scope.isDefectStates())
            {
                DefectState[] defectStates = (DefectState[]) datafieldManager
                    .loadDatafields(module, module.getDefectStates(), DefectState.class,
                        DefectState.TYPE_ID, this, localContextName,
                        messageContext, isDelta);
                module.setDefectStates(defectStates);
            }
            if (scope.isServiceRequestStates())
            {
                ServiceRequestState[] serviceRequestStates = (ServiceRequestState[]) datafieldManager
                    .loadDatafields(module, module.getServiceRequestStates(),
                        ServiceRequestState.class, ServiceRequestState.TYPE_ID,
                        this, localContextName, messageContext, isDelta);
                module.setServiceRequestStates(serviceRequestStates);
            }
            if (scope.isActionStates())
            {
                ActionState[] actionStates = (ActionState[]) datafieldManager
                    .loadDatafields(module, module.getActionStates(), ActionState.class,
                        ActionState.TYPE_ID, this, localContextName,
                        messageContext, isDelta);
                module.setActionStates(actionStates);
            }
            if (scope.isDeliverableRtfs())
            {
                DeliverableRTF[] deliverableRTFs = (DeliverableRTF[]) datafieldManager
                    .loadDatafields(module, module.getDeliverableRtfs(),
                        DeliverableRTF.class, DeliverableRTF.TYPE_ID,
                        this, localContextName, messageContext, isDelta);
                module.setDeliverableRtfs(deliverableRTFs);
            }

            if (scope.isSummaryTaskRtfs())
            {
                SummaryTaskRTF[] summaryTaskRTFs = (SummaryTaskRTF[]) datafieldManager
                    .loadDatafields(module, module.getSummaryTaskRtfs(),
                        SummaryTaskRTF.class, SummaryTaskRTF.TYPE_ID,
                        this, localContextName, messageContext, isDelta);
                module.setSummaryTaskRtfs(summaryTaskRTFs);
            }
            if (scope.isMilestoneRtfs())
            {
                MilestoneRTF[] milestoneRTFs = (MilestoneRTF[]) datafieldManager
                    .loadDatafields(module, module.getMilestoneRtfs(),
                        MilestoneRTF.class, MilestoneRTF.TYPE_ID,
                        this, localContextName, messageContext, isDelta);
                module.setMilestoneRtfs(milestoneRTFs);
            }
            if (scope.isTaskRtfs())
            {
                TaskRTF[] taskRTFs = (TaskRTF[]) datafieldManager
                    .loadDatafields(module, module.getTaskRtfs(), TaskRTF.class,
                        TaskRTF.TYPE_ID, this, localContextName,
                        messageContext, isDelta);
                module.setTaskRtfs(taskRTFs);
            }
            if (scope.isRequirementRtfs())
            {
                RequirementRTF[] requirementRTFs = (RequirementRTF[]) datafieldManager
                    .loadDatafields(module, module.getRequirementRtfs(),
                        RequirementRTF.class, RequirementRTF.TYPE_ID,
                        this, localContextName, messageContext, isDelta);
                module.setRequirementRtfs(requirementRTFs);
            }

            if (scope.isChangeRequestRtfs())
            {
                ChangeRequestRTF[] changeRequestRTFs = (ChangeRequestRTF[]) datafieldManager
                    .loadDatafields(module, module.getChangeRequestRtfs(),
                        ChangeRequestRTF.class, ChangeRequestRTF.TYPE_ID,
                        this, localContextName, messageContext, isDelta);
                module.setChangeRequestRtfs(changeRequestRTFs);
            }
            if (scope.isServiceRequestRtfs())
            {
                ServiceRequestRTF[] serviceRequestRTFs = (ServiceRequestRTF[]) datafieldManager
                    .loadDatafields(module, module.getServiceRequestRtfs(),
                        ServiceRequestRTF.class, ServiceRequestRTF.TYPE_ID,
                        this, localContextName, messageContext, isDelta);
                module.setServiceRequestRtfs(serviceRequestRTFs);

            }
            if (scope.isDefectRtfs())
            {
                DefectRTF[] defectRTFs = (DefectRTF[]) datafieldManager
                    .loadDatafields(module, module.getDefectRtfs(), DefectRTF.class,
                        DefectRTF.TYPE_ID, this, localContextName,
                        messageContext, isDelta);
                module.setDefectRtfs(defectRTFs);
            }
            if (scope.isIssueRtfs())
            {
                IssueRTF[] issueRTFs = (IssueRTF[]) datafieldManager
                    .loadDatafields(module, module.getIssueRtfs(), IssueRTF.class,
                        IssueRTF.TYPE_ID, this, localContextName,
                        messageContext, isDelta);
                module.setIssueRtfs(issueRTFs);
            }

            if (scope.isRiskRtfs())
            {
                RiskRTF[] riskRTFs = (RiskRTF[]) datafieldManager
                    .loadDatafields(module, module.getRiskRtfs(), RiskRTF.class,
                        RiskRTF.TYPE_ID, this, localContextName,
                        messageContext, isDelta);
                module.setRiskRtfs(riskRTFs);
            }
            if (scope.isActionRtfs())
            {
                ActionRTF[] actionRTFs = (ActionRTF[]) datafieldManager
                    .loadDatafields(module, module.getActionRtfs(), ActionRTF.class,
                        ActionRTF.TYPE_ID, this, localContextName,
                        messageContext, isDelta);
                module.setActionRtfs(actionRTFs);
            }

            //loadScorecardFolder
            if (scope.getScorecardFolders() != null)
            {
                String extraWhere = " TYPE_ID = " + ScorecardFolder.TYPE_ID;
                ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) loadElements(
                        module, module.getScorecardFolders(), scope.getScorecardFolders(),
                    ScorecardFolder.class, null, extraWhere, messageContext,
                    isDelta);
                module.setScorecardFolders(scorecardFolders);
            }
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        ScopeManagementModuleScope scope = (ScopeManagementModuleScope) rpmScope;
        ScopeManagementModule module = (ScopeManagementModule) rpmObject;

        if (scope != null)
        {
            if (scope.getScopeFolders() != null)
            {
                ScopeFolder[] scopeFolders = (ScopeFolder[]) saveElements(
                    module, module.getScopeFolders(), scope.getScopeFolders(),
                    messageContext, ScopeFolder.class);
                module.setScopeFolders(scopeFolders);

            }

            if (scope.getAttributeCategories() != null)
            {
                AttributeCategory[] attributeCategory = (AttributeCategory[]) saveElements(
                    module, module.getAttributeCategories(), scope.getAttributeCategories(), messageContext,
                    AttributeCategory.class);
                module.setAttributeCategories(attributeCategory);
            }

            if (scope.getCustomFieldCategories() != null)
            {
                propagateContextName(module, module.getCustomFieldCategories());
                module
                    .setCustomFieldCategories((CustomFieldCategory[]) updateChildren(
                        null, module.getCustomFieldCategories(), scope
                            .getCustomFieldCategories(), messageContext, null));
            }

            if (scope.getScorecardFolders() != null)
            {
                ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) saveElements(
                    module, module.getScorecardFolders(), scope
                        .getScorecardFolders(), messageContext,
                    ScorecardFolder.class);
                module.setScorecardFolders(scorecardFolders);

            }
        }

        return module;
    }
}
