/*
 * Created on 2-Jun-2006 @author mmatwiy Utility methods for the ScopeManagement
 * section
 */
package com.ibm.rpm.scopemanagement.util;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.List;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsUtil;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class ScopeManagementUtil
{

    public static boolean isTransferredScopeElement(ScopeElement scopeElement,
            MessageContext messageContext) throws RPMException
    {
        String wbsId = null;
        boolean isTransferred = false;

        if (scopeElement != null && scopeElement.getID() != null)
        {
            // SELECT TMT_CRI.WBS_ID FROM TMT_CRI
            // WHERE TMT_CRI.ELEMENT_ID=?
            ScopeManager scopeManager = new ScopeManager();

            SqlBuffer stmt = new SqlBuffer();
            stmt.appendSelect();
            stmt.append(ScopeManager.NAME_WBS_ID);
            stmt.appendFrom(scopeManager.getTableName());
            stmt.appendWhere();
            stmt.appendEqualQuestionMark(ScopeManager.NAME_ELEMENT_ID);

            List result = null;
            try
            {
                result = ManagerUtil.select(new Class[]
                {
                    String.class
                }, null, null, stmt, new Object[]
                {
                    scopeElement.getID()
                }, messageContext);
            }
            catch (RPMException e)
            {
                ExceptionUtil.handleException(WbsUtil.class, e);
                result = null;
            }

            if (result != null && !result.isEmpty() && result.get(0) != null
                    && ((List) result.get(0)).get(0) != null)
            {

                wbsId = (((String) ((List) result.get(0)).get(0)).toString());
            }

            if (wbsId != null && wbsId.equals(scopeElement.getID()))
            {
                isTransferred = true;
            }
        }
        return isTransferred;
    }

    public static String getScopeElementProjectId(ScopeElement scopeElement,
            MessageContext messageContext) throws RPMException
    {
        String projectId = null;

        if (scopeElement != null && scopeElement.getID() != null)
        {
            // SELECT TMT_CRI.PROJECT_ID FROM TMT_CRI
            // WHERE TMT_CRI.ELEMENT_ID=?
            ScopeManager scopeManager = new ScopeManager();

            SqlBuffer stmt = new SqlBuffer();
            stmt.appendSelect();
            stmt.append(ScopeManager.NAME_PROJECT_ID);
            stmt.appendFrom(scopeManager.getTableName());
            stmt.appendWhere();
            stmt.appendEqualQuestionMark(ScopeManager.NAME_ELEMENT_ID);

            List result = null;
            try
            {
                result = ManagerUtil.select(new Class[]
                {
                    String.class
                }, null, null, stmt, new Object[]
                {
                    scopeElement.getID()
                }, messageContext);
            }
            catch (RPMException e)
            {
                ExceptionUtil.handleException(WbsUtil.class, e);
                result = null;
            }

            if (result != null && !result.isEmpty() && result.get(0) != null
                    && ((List) result.get(0)).get(0) != null)
            {

                projectId = (((String) ((List) result.get(0)).get(0)).toString());
            }

        }
        return projectId;
    }

    public static boolean scopeElementExists(String elementId,
            MessageContext messageContext) throws RPMException
    {
        boolean exists = false;

        ScopeManager scopeManager = new ScopeManager();

        // SELECT TMT_CRI.ELEMENT_ID FROM TMT_CRI
        // WHERE TMT_CRI.ELEMENT_ID = ?
        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(ScopeManager.NAME_ELEMENT_ID);
        stmt.appendFrom(scopeManager.getTableName());
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(ScopeManager.NAME_ELEMENT_ID);

        List result = null;
        try
        {
            result = ManagerUtil.select(null, null, null, stmt, new Object[]
            {
                elementId
            }, messageContext);
        }
        catch (RPMException e)
        {
            ExceptionUtil.handleException(WbsUtil.class, e);
            result = null;
        }

        if (result != null && !result.isEmpty() && result.get(0) != null
                && ((List) result.get(0)).get(0) != null)
        {
            exists = true;
        }
        return exists;
    }

    public static Integer getAbstractScopeTaskTypeID(AbstractScope abstractScope)
    {
        // return the equivalent task object ID
        // these values come from pmo_cnst.h
        Integer taskTypeId = null;

        if (abstractScope instanceof ChangeRequest)
        {
            taskTypeId = new Integer(ScopeManager.TYPE_ID_CHANGE_REQUEST_TASK);
        }
        else if (abstractScope instanceof Risk)
        {
            taskTypeId = new Integer(ScopeManager.TYPE_ID_RISK_TASK);
        }
        else if (abstractScope instanceof Issue)
        {
            taskTypeId = new Integer(ScopeManager.TYPE_ID_ISSUE_TASK);
        }
        else if (abstractScope instanceof Defect)
        {
            taskTypeId = new Integer(ScopeManager.TYPE_ID_DEFECT_TASK);
        }
        else if (abstractScope instanceof ServiceRequest)
        {
            taskTypeId = new Integer(ScopeManager.TYPE_ID_SERVICE_REQUEST_TASK);
        }
        else if (abstractScope instanceof Requirement)
        {
            taskTypeId = new Integer(ScopeManager.TYPE_ID_REQUIREMENT_TASK);
        }
        else if (abstractScope instanceof Action)
        {
            taskTypeId = new Integer(ScopeManager.TYPE_ID_ACTION_TASK);
        }

        return taskTypeId;
    }

    public static Integer getAbstractScopeTypeIDFromTask(
            AbstractScope abstractScope)
    {
        // return the equivalent object ID
        Integer typeId = null;

        if (abstractScope instanceof ChangeRequest)
        {
            typeId = new Integer(ChangeRequest.TYPE_ID);
        }
        else if (abstractScope instanceof Risk)
        {
            typeId = new Integer(Risk.TYPE_ID);
        }
        else if (abstractScope instanceof Issue)
        {
            typeId = new Integer(Issue.TYPE_ID);
        }
        else if (abstractScope instanceof Defect)
        {
            typeId = new Integer(Defect.TYPE_ID);
        }
        else if (abstractScope instanceof ServiceRequest)
        {
            typeId = new Integer(ServiceRequest.TYPE_ID);
        }
        else if (abstractScope instanceof Requirement)
        {
            typeId = new Integer(Requirement.TYPE_ID);
        }
        else if (abstractScope instanceof Action)
        {
            typeId = new Integer(Action.TYPE_ID);
        }

        return typeId;
    }

    public static boolean isAbstractScopeTask(AbstractScope abstractScope,
            MessageContext messageContext) throws RPMException
    {
        Integer typeId = null;
        boolean isTask = false;

        if (abstractScope != null && abstractScope.getID() != null)
        {
            // SELECT TMT_CRI.TYPE_ID FROM TMT_CRI
            // WHERE TMT_CRI.ELEMENT_ID=?
            ScopeManager scopeManager = new ScopeManager();

            SqlBuffer stmt = new SqlBuffer();
            stmt.appendSelect();
            stmt.append(ScopeManager.NAME_TYPE_ID);
            stmt.appendFrom(scopeManager.getTableName());
            stmt.appendWhere();
            stmt.appendEqualQuestionMark(ScopeManager.NAME_ELEMENT_ID);

            List result = null;
            try
            {
                result = ManagerUtil.select(new Class[]
                {
                    Integer.class
                }, null, null, stmt, new Object[]
                {
                    abstractScope.getID()
                }, messageContext);
            }
            catch (RPMException e)
            {
                ExceptionUtil.handleException(WbsUtil.class, e);
                result = null;
            }

            if (result != null && !result.isEmpty() && result.get(0) != null
                    && ((List) result.get(0)).get(0) != null)
            {

                typeId = (((Integer) ((List) result.get(0)).get(0)));

                if (ScopeManager.abstractScopeTaskIds.contains(typeId))
                {
                    isTask = true;
                }
            }
        }
        return isTask;
    }

    public static RPMObject loadScopeElement(RPMObject parent,
            MessageContext messageContext) throws RPMException, ParseException,
            SQLException
    {
        if (parent instanceof WorkElement
                && ((WorkElement) parent).getScopeElement() == null
                && isPotentialTransferedScopeElement(parent))
        {
            try
            {
	            // Retrieve corresponding ScopeElement, in case the parent was
	            // transferred under WBS
	            ScopeElement scopeElement = new ScopeElement();
	            scopeElement.setID(parent.getID());
                
                if (scopeElement.getID() != null)
                {
                    ScopeManager manager = (ScopeManager) RPMManagerFactory
                        .getInstance().getRPMObjectManager(
                            ScopeElement.class.getName());

                    ScopeElement tmp = (ScopeElement) manager.loadByPrimaryKey(
                        scopeElement, null, messageContext, false);

                    if (tmp != null)
                    {
                        ((WorkElement) parent).setScopeElement(tmp);
                    }
                }
            }
            catch(RPMException e)
            {
                if (e.getErrorID() != ErrorCodes.AM_INVALID_RPMOBJECTID)
                {
                    throw e;
                }
            }
        }
        return parent;
    }
    
    public static GenericProject loadScopeProject(ScopeElement scopeElement,
            WorkElementScope workElementScope, MessageContext msgCtx)
    {
        GenericProject loadedGenericProject = null;
        
        try
        {
            GenericProject genericProject = new GenericProject();
            genericProject.setID(getScopeElementProjectId(
                scopeElement, msgCtx));

            if (genericProject.getID() != null)
            {
                RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(GenericProject.class);
                loadedGenericProject = (GenericProject) manager.loadByPrimaryKey(
                    genericProject, workElementScope, msgCtx, false);
            }
        }
        catch (Exception e)
        {
            RPMException rpmException = new RPMException(e);
            rpmException.assignContainer(scopeElement);
            msgCtx.addExceptionNoThrow(rpmException);
        }
        
        return loadedGenericProject;
    }
    
    private static boolean isPotentialTransferedScopeElement(RPMObject rpmObject)
    {
        if(rpmObject == null)
            return false;
        
        if(rpmObject.retrieveTypeID()!=null)
        {
            if(rpmObject instanceof SummaryTask && rpmObject.retrieveTypeID().intValue()!=SummaryTask.TYPE_ID)
            {
                return true;
            }
            else if (rpmObject instanceof Task && rpmObject.retrieveTypeID().intValue()!=Task.TYPE_ID)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        //here we don't have the type_id, so we don't take any chances
        return true;
    }
}