/*
 * Created on Dec 4, 2005
 * @author mmatwiy
 * 
 * Checkpoint validation of container AbstractScorecardElement. This class
 * is not called directly; it is extended by other container checkpoints
 * (AssignedScorecard classes only).
 */

package com.ibm.rpm.scorecard.checkpoints;

import java.util.List;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AbstractScorecardElement;
import com.ibm.rpm.scorecard.managers.PublishedScorecardManager;

public abstract class AbstractAssignedScorecardElementCheckpoint extends
        AbstractAssignedScorecardEntityCheckpoint
{

    protected void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx, boolean parentRequired, List validParentTypes)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx, parentRequired, validParentTypes);

        // Initialize
        AbstractScorecardElement scorecardElement = (AbstractScorecardElement) rpmObject;

        // Weight is read-only for an assigned scorecard element
        if (scorecardElement.testWeightModified())
        {
            GenericValidator.validateReadOnly(scorecardElement,
                ValidationConstants.WEIGHT_FIELD, msgCtx);

            //reset the field to its original value
            Integer weight = loadScorecardElementWeight(scorecardElement, msgCtx);
            scorecardElement.setWeight(weight);
        }
    }

    private Integer loadScorecardElementWeight(
            AbstractScorecardElement scorecardElement, MessageContext msgCtx)
    {
        SqlBuffer sql = new SqlBuffer();
        sql.appendEqualQuestionMark(PublishedScorecardManager.NAME_ELEMENT_ID);
        sql.appendNotEqual(PublishedScorecardManager.NAME_REC_STATUS,"D");
        
        Object[] arguments = new Object[]
        {
            scorecardElement.getID()
        };
        
        Integer weight = null;
        try
        {
            weight = (Integer) ManagerUtil.selectColumnValue(Integer.class,
                null, null, PublishedScorecardManager.NAME_WEIGHT,
                PublishedScorecardManager.TABLE_NAME, sql, arguments, msgCtx);
        }
        catch (RPMException e1)
        {
            e1.printStackTrace();
        }
        return weight;
    }
}