/*
 * Created on Dec 4, 2005
 * @author mmatwiy
 * 
 * Checkpoint validation of container AbstractScorecardEntity. This class
 * is not called directly; it is extended by other container checkpoints
 * (assigned scorecard elements only).
 */
package com.ibm.rpm.scorecard.checkpoints;

import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AbstractScorecardEntity;
import com.ibm.rpm.scorecard.managers.PublishedScorecardManager;

public abstract class AbstractAssignedScorecardEntityCheckpoint extends
        AbstractCheckpoint
{

    protected void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx, boolean parentRequired, List validParentTypes)
    {

        super
            .isValidParent(rpmObject, msgCtx, parentRequired, validParentTypes);

        // Initialize
        AbstractScorecardEntity scorecardEntity = (AbstractScorecardEntity) rpmObject;

        // Name is read-only for an assigned scorecard element
        if (scorecardEntity.testNameModified())
        {
            GenericValidator.validateReadOnly(scorecardEntity,
                ValidationConstants.NAME_FIELD, msgCtx);

            //reset the field to its original value
            String name = loadScorecardElementName(scorecardEntity, msgCtx);
            scorecardEntity.setName(name);
        }
    }

    private String loadScorecardElementName(
            AbstractScorecardEntity scorecardEntity, MessageContext msgCtx)
    {
        SqlBuffer sql = new SqlBuffer();
        sql.appendEqualQuestionMark(PublishedScorecardManager.NAME_ELEMENT_ID);
        sql.appendNotEqual(PublishedScorecardManager.NAME_REC_STATUS,"D");
        
        Object[] arguments = new Object[]
        {
            scorecardEntity.getID()
        };
        
        String name = null;
        try
        {
            name = (String) ManagerUtil.selectColumnValue(String.class, null,
                null, PublishedScorecardManager.NAME_ELEMENT_NAME,
                PublishedScorecardManager.TABLE_NAME, sql, arguments, msgCtx);
        }
        catch (RPMException e)
        {
            RPMException rpmException = new RPMException(e);
            addException(rpmException, msgCtx);
        }
        return name;
    }
}