/*
 * Created on Dec 4, 2005
 * @author mmatwiy
 * Checkpoint validation of container AbstractScorecardEntity. This class
 * is not called directly; it is extended by other container checkpoints.
 */
package com.ibm.rpm.scorecard.checkpoints;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.framework.util.ValidateSerializer;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AbstractScorecardEntity;
import com.ibm.rpm.scorecard.containers.Scorecard;

public abstract class AbstractScorecardEntityCheckpoint extends
        AbstractCheckpoint
{
    protected void validateSave(RPMObject rpmObject, MessageContext msgCtx,
            boolean parentRequired, List validParentTypes)
    {
        super
            .isValidParent(rpmObject, msgCtx, parentRequired, validParentTypes);

        // Initialize
        AbstractScorecardEntity scorecardEntity = (AbstractScorecardEntity) rpmObject;

        if (scorecardEntity.getID() == null)
        {
            // all scorecard containers must be initially saved through a module 
            ValidateSerializer vs = msgCtx.getObjectsToValidate();
            ArrayList objects = vs.getArrayList();
            Iterator iterator = objects.iterator();

            boolean moduleFound = false;
            while (iterator.hasNext())
            {
                RPMObject next = (RPMObject) iterator.next();
                if (next instanceof AbstractModule)
                {
                    moduleFound = true;
                }
                else if (next instanceof AbstractScorecardEntity
                        && !moduleFound)
                {
                    // the module object must occur first in the list; this shows
                    // that the AbstractScorecardEntity was saved using a module
                    String errorParams[] =
                    {
                        StringUtil
                            .getShortClassName(scorecardEntity.getClass())
                    };

                    RPMException e = new RPMException(
                        ErrorCodes.MODULE_REQUIRED, errorParams,
                        scorecardEntity.getClass().getName());
                    addException(e, msgCtx);
                }
            }
        }

        GenericValidator.validateMandatory(scorecardEntity,
            ValidationConstants.NAME_FIELD, scorecardEntity.getName(), msgCtx);

        // Field validation of name
        if (scorecardEntity.testNameModified())
        {
            GenericValidator.validateMinLength(scorecardEntity,
                ValidationConstants.NAME_FIELD, scorecardEntity.getName(),
                ValidationConstants.NAME_MIN, msgCtx);

            int maxLength = ValidationConstants.NAME_CHARACTER_LIMIT;
            if (scorecardEntity instanceof Scorecard)
            {
                maxLength = ValidationConstants.SCORECARD_NAME_CHARACTER_LIMIT;
            }
            
            GenericValidator.validateMaxLength(scorecardEntity,
                ValidationConstants.NAME_FIELD, scorecardEntity.getName(),
                maxLength, msgCtx);
        }

        // Field validation of description
        if (scorecardEntity.testDescriptionModified())
        {
            GenericValidator.validateMaxLength(scorecardEntity,
                ValidationConstants.DESCRIPTION_FIELD, scorecardEntity
                    .getDescription(),
                ValidationConstants.DESCRIPTION_CHARACTER_LIMIT, msgCtx);
        }
    }
}
