/*
 * Created on Dec 5, 2005
 * @author mmatwiy
 * Checkpoint validation of container AssignedScorecard.
 */
package com.ibm.rpm.scorecard.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AbstractScorecardElement;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.managers.PublishedScorecardManager;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;

public class AssignedScorecardCheckpoint extends AbstractCheckpoint
{
    protected static AssignedScorecardCheckpoint instance = new AssignedScorecardCheckpoint();

    private static final List PARENT_LIST = new ArrayList();

    static
    {
        PARENT_LIST.add(Asset.class);
        PARENT_LIST.add(GenericProject.class);
        PARENT_LIST.add(AbstractScope.class);
    }

    public static AssignedScorecardCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, PARENT_LIST);

        AssignedScorecard scorecard = (AssignedScorecard) rpmObject;

        GenericValidator.validateMandatory(scorecard,
            ValidationConstants.NAME_FIELD, scorecard.getName(), msgCtx);

        // On update, the score and probabilityType fields cannot be cleared
        GenericValidator.validateDefaultValueUpdate(scorecard,
            ValidationConstants.SCORE_FIELD, scorecard.getScore(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(scorecard,
            ValidationConstants.PROBABILITY_TYPE_FIELD, scorecard
                .getProbabilityType(), msgCtx);

        // Field validation of name
        if (scorecard.testNameModified())
        {
            GenericValidator.validateMinLength(scorecard,
                ValidationConstants.NAME_FIELD, scorecard.getName(),
                ValidationConstants.NAME_MIN, msgCtx);

            GenericValidator.validateMaxLength(scorecard,
                ValidationConstants.NAME_FIELD, scorecard.getName(),
                ValidationConstants.SCORECARD_NAME_CHARACTER_LIMIT, msgCtx);
        }

        // Weight is read-only for an assigned scorecard element
        if (scorecard.testWeightModified())
        {
            GenericValidator.validateReadOnly(scorecard,
                ValidationConstants.WEIGHT_FIELD, msgCtx);

            //reset the read-only field to its original value
            Integer weight = loadScorecardElementWeight(scorecard, msgCtx);
            scorecard.setWeight(weight);
            
//            // reset the read-only field to its previous value
//            AssignedScorecard loadedScorecard = loadScorecard(scorecard,
//                rpmScope, msgCtx);
//
//            if (loadedScorecard != null)
//            {
//                scorecard.setWeight(loadedScorecard.getWeight());
//            }
//            else
//            {
//                scorecard.setWeight(null);
//            }
        }
    }

    //TODO merge with the method of AbstractAssignedScorecardElementCheckpoint
    private Integer loadScorecardElementWeight(
            AbstractScorecardElement scorecardElement, MessageContext msgCtx)
    {
        SqlBuffer sql = new SqlBuffer();
        sql.appendEqualQuestionMark(PublishedScorecardManager.NAME_ELEMENT_ID);
        sql.appendNotEqual(PublishedScorecardManager.NAME_REC_STATUS,"D");
        
        Object[] arguments = new Object[]
        {
            scorecardElement.getID()
        };
        
        Integer weight = null;
        try
        {
            weight = (Integer) ManagerUtil.selectColumnValue(Integer.class,
                null, null, PublishedScorecardManager.NAME_WEIGHT,
                PublishedScorecardManager.TABLE_NAME, sql, arguments, msgCtx);
        }
        catch (RPMException e)
        {
            RPMException rpmException = new RPMException(e);
            addException(rpmException, msgCtx);
        }
        return weight;
    }
    
    public final boolean canPublishScorecard(
            AssignedScorecard assignedScorecard,
            AbstractScorecardScope scorecardScope, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canPublishScorecard(
                messageContext, assignedScorecard, scorecardScope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }
}