/*
 * Created on Dec 5, 2005
 * @author mmatwiy
 * Checkpoint validation of container AssignedScorecardQuestion.
 */
package com.ibm.rpm.scorecard.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scorecard.constants.ErrorCodes;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AssignedScorecardCategory;
import com.ibm.rpm.scorecard.containers.AssignedScorecardQuestion;
import com.ibm.rpm.scorecard.containers.AssignedScorecardResponse;
import com.ibm.rpm.scorecard.scope.ScorecardQuestionScope;

public class AssignedScorecardQuestionCheckpoint extends
        AbstractAssignedScorecardElementCheckpoint
{
    protected static AssignedScorecardQuestionCheckpoint instance = new AssignedScorecardQuestionCheckpoint();

    private static final List PARENT_LIST = new ArrayList();

    static
    {
        PARENT_LIST.add(AssignedScorecardCategory.class);
    }

    public static AssignedScorecardQuestionCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx, true, PARENT_LIST);

        AssignedScorecardQuestion question = (AssignedScorecardQuestion) rpmObject;
        ScorecardQuestionScope scope = (ScorecardQuestionScope) rpmScope;

        // On update, the score, responseDate, and toReport
        // fields cannot be cleared
        GenericValidator.validateDefaultValueUpdate(question,
            ValidationConstants.SCORE_FIELD, question.getScore(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(question,
            ValidationConstants.TOREPORT_FIELD, question.getToReport(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(question,
            ValidationConstants.RESPONSE_DATE_FIELD,
            question.getResponseDate(), msgCtx);
        

        if (scope != null && scope.getScorecardResponses() != null)
        {
            AssignedScorecardResponse[] responses = question
                .getAssignedScorecardResponses();

            // Check that there is 1 response selected for this question
            int selectedCount = 0;
            if (responses != null)
            {
                for (int i = 0; i < responses.length; i++)
                {
                    if (responses[i].getSelectedAnswer().booleanValue())
                    {
                        selectedCount++;
                    }
                }
            }

            if (selectedCount != 1)
            {
                String[] errorParams =
                {
                    StringUtil.getShortClassName(rpmObject.getClass())
                };

                RPMException e = new RPMException(
                    ErrorCodes.WRONG_NUMBER_OF_RESPONSES_FOR_QUESTION,
                    errorParams, rpmObject.getClass().getName());
                addException(e, msgCtx);
            }
        }
    }
}