/*
 * Created on Dec 5, 2005
 * @author mmatwiy
 * Checkpoint validation of container AssignedScorecardResponse.
 */
package com.ibm.rpm.scorecard.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AssignedScorecardQuestion;
import com.ibm.rpm.scorecard.containers.AssignedScorecardResponse;

public class AssignedScorecardResponseCheckpoint extends
        AbstractAssignedScorecardElementCheckpoint
{
    protected static AssignedScorecardResponseCheckpoint instance = new AssignedScorecardResponseCheckpoint();

    private static final List PARENT_LIST = new ArrayList();

    static
    {
        PARENT_LIST.add(AssignedScorecardQuestion.class);
    }

    public static AssignedScorecardResponseCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx, true, PARENT_LIST);

        AssignedScorecardResponse response = (AssignedScorecardResponse) rpmObject;

        // On update, the score and selectedAnswer fields cannot be cleared
        GenericValidator.validateDefaultValueUpdate(response,
            ValidationConstants.SCORE_FIELD, response.getScore(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(response,
            ValidationConstants.SELECTED_ANSWER_FIELD, response.getSelectedAnswer(), msgCtx);

        // validate length of comments
        if (response.testCommentsModified())
        {
            GenericValidator.validateMaxLength(response,
                ValidationConstants.COMMENTS_FIELD, response.getComments(),
                ValidationConstants.COMMENTS_CHARACTER_LIMIT, msgCtx);
        }
    }
}
