/*
 * Created on Dec 5, 2005
 * @author mmatwiy
 * Checkpoint validation of container AssignedScorecardState.
 */
package com.ibm.rpm.scorecard.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scorecard.constants.ErrorCodes;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.AssignedScorecardState;

public class AssignedScorecardStateCheckpoint extends AbstractCheckpoint
{
    protected static AssignedScorecardStateCheckpoint instance = new AssignedScorecardStateCheckpoint();

    public static AssignedScorecardStateCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        AssignedScorecardState state = (AssignedScorecardState) rpmObject;
        
        // On update, the active field cannot be cleared
        GenericValidator.validateDefaultValueUpdate(state,
            ValidationConstants.ACTIVE_FIELD, state.getActive(), msgCtx);

        
        if (state.testPublishedModified() && state.getPublished() != null)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(AssignedScorecardState.class)
            };
            
            RPMException e = new RPMException(ErrorCodes.PUBLISHED_CAN_NOT_BE_SET,
                errorParams, rpmObject.getClass().getName());
            addException(e, msgCtx);
        }
    }
}
