/*
 * Created on Dec 4, 2005
 * @author mmatwiy
 * Checkpoint validation of container ScorecardQuestion.
 */
package com.ibm.rpm.scorecard.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scorecard.constants.ValidationConstants;
import com.ibm.rpm.scorecard.containers.ScorecardCategory;
import com.ibm.rpm.scorecard.containers.ScorecardQuestion;
import com.ibm.rpm.scorecard.containers.ScorecardResponse;
import com.ibm.rpm.scorecard.scope.ScorecardQuestionScope;

public class ScorecardQuestionCheckpoint extends
        AbstractScorecardElementCheckpoint
{
    private static final List SCORECARDQUESTION_PARENT_LIST = new ArrayList();

    protected static ScorecardQuestionCheckpoint instance = new ScorecardQuestionCheckpoint();

    static
    {
        SCORECARDQUESTION_PARENT_LIST.add(ScorecardCategory.class);
    }

    public static ScorecardQuestionCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, msgCtx, true,
            SCORECARDQUESTION_PARENT_LIST);

        ScorecardQuestion question = (ScorecardQuestion) rpmObject;
        ScorecardQuestionScope scope = (ScorecardQuestionScope) rpmScope;

        // On update, the toReport field cannot be cleared
        GenericValidator.validateDefaultValueUpdate(question,
            ValidationConstants.TOREPORT_FIELD, question.getToReport(), msgCtx);

        // Field validation of weight
        if (question.testWeightModified())
        {
            GenericValidator.validateRange(question,
                ValidationConstants.WEIGHT_FIELD, question.getWeight(),
                ValidationConstants.WEIGHT_MIN, ValidationConstants.WEIGHT_MAX,
                msgCtx);
        }
        
        ScorecardResponse[] responses = question.getScorecardResponses();

        if (responses != null && scope != null && scope.getScorecardResponses() != null)
        {
            // Check that there is exactly one default response per question
            int defaultCount = 0;
            for (int i = 0; i < responses.length; i++)
            {
                if (responses[i] != null && responses[i].getDefaultResponse() != null
                        && responses[i].getDefaultResponse().booleanValue())
                {
                    defaultCount++;
                }
            }

            if (defaultCount != 1)
            {
                GenericValidator.validateDefaultCount(question,
                    ValidationConstants.RESPONSES_FIELD, msgCtx);
            }
        }
    }

}
