/*
 * Created on Mar 23, 2006
 *
 */
package com.ibm.rpm.security.checkpoints;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.util.AttributeValidator;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.Field;
import com.ibm.rpm.metadata.model.MetadataStore;
import com.ibm.rpm.security.containers.SecurityGroup;

/**
 * @author karim
 *
 */
public class SecurityGroupCheckpoint extends AbstractCheckpoint
{


    private static final List PARENTS = new ArrayList();

    static
    {
        PARENTS.add(SecurityGroup.class);
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {

        if (!GenericValidator.validateInstance(rpmObject, SecurityGroup.class,
            msgCtx))
            return;

        if (!super.isValidParent(rpmObject, msgCtx, true, PARENTS))
            return;

        // Initialize
        SecurityGroup mainSecurityGroup = (SecurityGroup) rpmObject;

        if (!AttributeValidator
            .validateMandatory(mainSecurityGroup,
                ValidationConstants.NAME_FIELD, mainSecurityGroup.getName(),
                msgCtx))
            return;
        
        if (mainSecurityGroup.retrieveTypeID().intValue() != SecurityGroup.TYPE_ID)
        {
            // TODO add exception

            RPMException e = new RPMException(
                "Expected typeID SecurityGroup.TYPE_ID (307) But it was: "
                        + mainSecurityGroup.retrieveTypeID().intValue());
            addException(e, msgCtx);
            return;
        }

        if (rpmObject.getID() != null)
        {
            // Field validation: name
            if (mainSecurityGroup.testNameModified())
            {
                if (!AttributeValidator.validateMinLength(mainSecurityGroup,
                    ValidationConstants.NAME_FIELD,
                    mainSecurityGroup.getName(),
                    ValidationConstants.SECURITY_GROUP_NAME_MIN, msgCtx))
                    return;

                if (!AttributeValidator.validateMaxLength(mainSecurityGroup,
                    ValidationConstants.NAME_FIELD,
                    mainSecurityGroup.getName(),
                    ValidationConstants.SECURITY_GROUP_NAME_MAX, msgCtx))
                    return;

            }

            
            Container container = MetadataStore.getInstance()
                .getContainerByJavaClass(rpmObject.getClass());

            checkNullUpdate(rpmObject, container, msgCtx);

        }
        else
        {
            if (AttributeValidator.validateMinLength(mainSecurityGroup,
                ValidationConstants.NAME_FIELD, mainSecurityGroup.getName(),
                ValidationConstants.SECURITY_GROUP_NAME_MIN, msgCtx))
            {
                AttributeValidator.validateMaxLength(mainSecurityGroup,
                    ValidationConstants.NAME_FIELD,
                    mainSecurityGroup.getName(),
                    ValidationConstants.SECURITY_GROUP_NAME_MAX, msgCtx);
            }

        }
    }
    
    public boolean checkNullUpdate(RPMObject objectToFill,
            Container classInformation, MessageContext msgCtx)
    {
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();

            if (!fieldInfo.canBeNullValue())
            {
                // get the current value
                Method methodToInvoke = fieldInfo.getGetterMethod();
                Object currValue = null;
                try
                {
                    currValue = methodToInvoke.invoke(objectToFill,
                        new Object[0]);
                }
                catch (Exception e)
                {
                    RPMException ex = new RPMException("Failed to get field: "
                            + fieldInfo.getName() + " value: ", e);
                    addException(ex, msgCtx);
                    return false;

                }

                if (currValue == null)
                {
                    String[] errorParams =
                    {
                        StringUtil.getShortClassName(objectToFill.getClass())
                                + " " + fieldInfo.getName()
                    };
                    RPMException e = new RPMException(
                        ErrorCodes.CANNOT_CLEAR_FIELD, errorParams,
                        objectToFill.getClass().getName(), fieldInfo.getName(),
                        objectToFill.getID());
                    addException(e, msgCtx);
                    return false;
                }
            }
        }

        return true;
    }

}
