/*
 * Created on Mar 23, 2006
 *
 */
package com.ibm.rpm.security.checkpoints;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.util.AttributeValidator;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.Field;
import com.ibm.rpm.metadata.model.MetadataStore;
import com.ibm.rpm.security.containers.ActionItemSecurityRole;
import com.ibm.rpm.security.containers.ChangeRequestSecurityRole;
import com.ibm.rpm.security.containers.DefectSecurityRole;
import com.ibm.rpm.security.containers.DeliverableSecurityRole;
import com.ibm.rpm.security.containers.DocumentSecurityRole;
import com.ibm.rpm.security.containers.FolderSecurityRole;
import com.ibm.rpm.security.containers.GenericSecurityRole;
import com.ibm.rpm.security.containers.IssueSecurityRole;
import com.ibm.rpm.security.containers.NotesMinutesSecurityRole;
import com.ibm.rpm.security.containers.PoolSecurityRole;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.containers.RequirementSecurityRole;
import com.ibm.rpm.security.containers.RiskSecurityRole;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.containers.SecurityRoleCategory;
import com.ibm.rpm.security.containers.ServiceRequestSecurityRole;
import com.ibm.rpm.security.containers.TaskSecurityRole;
import com.ibm.rpm.security.containers.TemplateFolderSecurityRole;
import com.ibm.rpm.security.managers.SecurityRoleManager;

/**
 * @author karim
 *
 */
public class SecurityRoleCheckpoint extends AbstractCheckpoint
{
    private static final List PARENTS = new ArrayList();

    static
    {
        PARENTS.add(SecurityRoleCategory.class);
        PARENTS.add(ProjectSecurityRole.class);
        PARENTS.add(DeliverableSecurityRole.class);
        PARENTS.add(TaskSecurityRole.class);
        PARENTS.add(ChangeRequestSecurityRole.class);
        PARENTS.add(IssueSecurityRole.class);
        PARENTS.add(RiskSecurityRole.class);
        PARENTS.add(ServiceRequestSecurityRole.class);
        PARENTS.add(RequirementSecurityRole.class);
        PARENTS.add(DefectSecurityRole.class);
        PARENTS.add(ActionItemSecurityRole.class);
        PARENTS.add(DocumentSecurityRole.class);
        PARENTS.add(NotesMinutesSecurityRole.class);
        PARENTS.add(PoolSecurityRole.class);
        PARENTS.add(FolderSecurityRole.class);
        PARENTS.add(TemplateFolderSecurityRole.class);
    }

    protected static SecurityRoleCheckpoint instance = new SecurityRoleCheckpoint();

    public static SecurityRoleCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {

        if (!GenericValidator.validateInstance(rpmObject, SecurityRole.class,
            msgCtx))
            return;

        if (!super.isValidParent(rpmObject, msgCtx, true, PARENTS))
            return;

        // Initialize
        GenericSecurityRole mainGenericSecurityRole = (GenericSecurityRole) rpmObject;

        if (!AttributeValidator.validateMandatory(mainGenericSecurityRole,
            ValidationConstants.NAME_FIELD, mainGenericSecurityRole.getName(),
            msgCtx))
            return;
        if (rpmObject.getID() == null)
        {
        	validateCreate(mainGenericSecurityRole, rpmScope, msgCtx);
        }
        else
        {
            validateUpdate(mainGenericSecurityRole, rpmScope, msgCtx);
        }
    }

    private void validateCreate(GenericSecurityRole mainGenericSecurityRole, RPMObjectScope rpmScope, MessageContext msgCtx)
    {
        if (validateSecurityRoleParentForCreation(mainGenericSecurityRole, rpmScope,
            msgCtx))
        {
            if (mainGenericSecurityRole.getName() == null)
            {
                RPMException e = new RPMException("Missing name");
                addException(e, msgCtx);
            }
            else if (AttributeValidator.validateMinLength(
                mainGenericSecurityRole, ValidationConstants.NAME_FIELD,
                mainGenericSecurityRole.getName(),
                ValidationConstants.SECURITY_ROLE_NAME_MIN, msgCtx))
            {
                AttributeValidator.validateMaxLength(mainGenericSecurityRole,
                    ValidationConstants.NAME_FIELD, mainGenericSecurityRole
                        .getName(), ValidationConstants.SECURITY_ROLE_NAME_MAX,
                    msgCtx);
            }
        }
    }
    
    private boolean validateSecurityRoleParentForCreation(GenericSecurityRole mainGenericSecurityRole, RPMObjectScope rpmScope, MessageContext msgCtx)
    {
    	boolean retVal = true;
    	
        super.isValidParent(mainGenericSecurityRole, msgCtx, true, true,
            PARENTS);

        Integer typeId = null;
        try
        {
            SqlBuffer sql = new SqlBuffer();
            //here the parent cannot be null as it will be caught earlier in the validation process
            sql.appendEqualQuestionMark(SecurityRoleManager.NAME_ELEMENT_ID);
            sql.appendAnd();
            sql.append(new SecurityRoleManager().getFilter());
            
            Object[] arguments = new Object[]
            {
                mainGenericSecurityRole.getParent().getID()
            };
            
            typeId = (Integer) ManagerUtil.selectColumnValue(Integer.class,
                null, null, SecurityRoleManager.NAME_TYPE_ID,
                SecurityRoleManager.TABLE_NAME, sql, arguments, msgCtx);
            
            if(typeId == null)
            {
                retVal = false;
                RPMException ex = new RPMException("Failed to load the SecurityRole Parent of the object: "
                        + mainGenericSecurityRole.getName());
                addException(ex, msgCtx);
            }
            else if(typeId.intValue() != SecurityRole.TYPE_ID)
            {
                retVal = false;
                RPMException e = new RPMException(
                        "Security roles can only be created under parent security roles of level 1.");
                addException(e, msgCtx);
            }
        }
        catch (RPMException e1)
        {
            e1.printStackTrace();
        }
        
    	return retVal;
    }
    
    private void validateUpdate(GenericSecurityRole mainGenericSecurityRole, RPMObjectScope rpmScope, MessageContext msgCtx)
    {
        // Field validation: name
        if (mainGenericSecurityRole.testNameModified())
        {
            if (!AttributeValidator.validateMinLength(
                mainGenericSecurityRole, ValidationConstants.NAME_FIELD,
                mainGenericSecurityRole.getName(),
                ValidationConstants.SECURITY_ROLE_NAME_MIN, msgCtx))
                return;

            if (!AttributeValidator.validateMaxLength(
                mainGenericSecurityRole, ValidationConstants.NAME_FIELD,
                mainGenericSecurityRole.getName(),
                ValidationConstants.SECURITY_ROLE_NAME_MAX, msgCtx))
                return;

        }

        Container container = MetadataStore.getInstance()
            .getContainerByJavaClass(mainGenericSecurityRole.getClass());
        checkNullUpdateExcludeParentField(mainGenericSecurityRole, container, msgCtx);
    }
    
    public boolean checkNullUpdateExcludeParentField(RPMObject objectToFill,
            Container classInformation, MessageContext msgCtx)
    {
        Iterator it = classInformation.getFields().iterator();
        while (it.hasNext())
        {
            Field fieldInfo = (Field) it.next();

            if (!fieldInfo.getName().equals("parent") && !fieldInfo.canBeNullValue())
            {
                // get the current value
                Method methodToInvoke = fieldInfo.getGetterMethod();
                Object currValue = null;
                try
                {
                    currValue = methodToInvoke.invoke(objectToFill,
                        new Object[0]);
                }
                catch (Exception e)
                {
                    RPMException ex = new RPMException("Failed to get field: "
                            + fieldInfo.getName() + " value: ", e);
                    addException(ex, msgCtx);
                    return false;
                }

                if (currValue == null)
                {
                    String[] errorParams =
                    {
                        StringUtil.getShortClassName(objectToFill.getClass())
                                + " " + fieldInfo.getName()
                    };
                    RPMException e = new RPMException(
                        ErrorCodes.CANNOT_CLEAR_FIELD, errorParams,
                        objectToFill.getClass().getName(), fieldInfo.getName(),
                        objectToFill.getID());
                    addException(e, msgCtx);
                    return false;
                }
            }
        }

        return true;
    }
}
