package com.ibm.rpm.timesheet.checkpoints;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.timesheet.constants.ErrorCodes;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.AbstractTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.managers.TimesheetApprovalStatusManager;
import com.ibm.rpm.timesheet.types.TimesheetStatus;
import com.ibm.rpm.timesheet.util.TimesheetManagerUtil;
import com.ibm.rpm.timesheet.util.TimesheetStatusUtil;
import com.ibm.rpm.wbs.managers.TaskAssignmentManager;

public abstract class AbstractTimesheetCheckpoint extends AbstractCheckpoint
{
    public void validateSave(RPMObject rpmObject, MessageContext msgCtx)
    {

        AbstractTimesheet timesheet = (AbstractTimesheet) rpmObject;
        
        // Field validation: regularMonday
        if (timesheet.testRegularMondayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.REGULAR_MONDAY_FIELD, timesheet
                    .getRegularMonday(), msgCtx);
        }

        // Field validation: regularTuesday
        if (timesheet.testRegularTuesdayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.REGULAR_TUESDAY_FIELD, timesheet
                    .getRegularTuesday(), msgCtx);
        }

        // Field validation: regularWednesday
        if (timesheet.testRegularWednesdayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.REGULAR_WEDNESDAY_FIELD, timesheet
                    .getRegularWednesday(), msgCtx);
        }

        // Field validation: regularThursday
        if (timesheet.testRegularThursdayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.REGULAR_THURSDAY_FIELD, timesheet
                    .getRegularThursday(), msgCtx);
        }

        // Field validation: regularFriday
        if (timesheet.testRegularFridayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.REGULAR_FRIDAY_FIELD, timesheet
                    .getRegularFriday(), msgCtx);
        }

        // Field validation: regularSaturday
        if (timesheet.testRegularSaturdayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.REGULAR_SATURDAY_FIELD, timesheet
                    .getRegularSaturday(), msgCtx);
        }

        // Field validation: regularSunday
        if (timesheet.testRegularSundayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.REGULAR_SUNDAY_FIELD, timesheet
                    .getRegularSunday(), msgCtx);
        }

        // Field validation: specialMonday
        if (timesheet.testSpecialMondayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.SPECIAL_MONDAY_FIELD, timesheet
                    .getSpecialMonday(), msgCtx);
        }

        // Field validation: specialTuesday
        if (timesheet.testSpecialTuesdayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.SPECIAL_TUESDAY_FIELD, timesheet
                    .getSpecialTuesday(), msgCtx);
        }

        // Field validation: specialWednesday
        if (timesheet.testSpecialWednesdayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.SPECIAL_WEDNESDAY_FIELD, timesheet
                    .getSpecialWednesday(), msgCtx);
        }

        // Field validation: specialThursday
        if (timesheet.testSpecialThursdayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.SPECIAL_THURSDAY_FIELD, timesheet
                    .getSpecialThursday(), msgCtx);
        }

        // Field validation: specialFriday
        if (timesheet.testSpecialFridayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.SPECIAL_FRIDAY_FIELD, timesheet
                    .getSpecialFriday(), msgCtx);
        }

        // Field validation: specialSaturday
        if (timesheet.testSpecialSaturdayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.SPECIAL_SATURDAY_FIELD, timesheet
                    .getSpecialSaturday(), msgCtx);
        }

        // Field validation: specialSunday
        if (timesheet.testSpecialSundayModified())
        {
            validateWorkingHours(timesheet,
                ValidationConstants.SPECIAL_SUNDAY_FIELD, timesheet
                    .getSpecialSunday(), msgCtx);
        }

        validSummaryNotes(timesheet, msgCtx);
    }
    
    private boolean validSummaryNotes(AbstractTimesheet timesheet,
            MessageContext msgCtx)
    {
        if (timesheet.testSummaryNotesModified())
        {
            return GenericValidator.validateMaxLength(timesheet,
                ValidationConstants.SUMMARY_NOTES_FIELD, timesheet
                    .getSummaryNotes(), ValidationConstants.SUMMARY_NOTES_MAX,
                msgCtx);
        }
        return true;
    }
    
    protected boolean validateWorkingHours(AbstractTimesheet timesheet,
            String fieldName, Double fieldValue, MessageContext msgCtx)
    {
        boolean valid = true;

        valid &= GenericValidator.validateRange(timesheet, fieldName,
            fieldValue, ValidationConstants.MIN_WORKING_HOURS_PER_DAY,
            ValidationConstants.MAX_WORKING_HOURS_PER_DAY, msgCtx);

        valid &= GenericValidator.validateWorkingHour(timesheet, fieldName,
            fieldValue, msgCtx);

        return valid;
    }
    
    protected boolean validApprovalStatus(SummaryTimesheet summaryTimesheet, 
            MessageContext msgCtx) 
    {
        boolean valid = true;
        TimesheetApprovalStatus appStatus = null;
        
        try
        {
            appStatus = TimesheetManagerUtil.loadTimesheetApprovalStatus(summaryTimesheet, msgCtx);
        }
        catch (RPMException e1)
        {
            addException(e1, msgCtx);
            valid = false;
        }
        
        if (appStatus != null && valid)
        {
            // if the the timesheet has been really submitted for
            // approval, the approvalDate could not be null
            if (appStatus.getApprovalDate() != null)
            {
                TimesheetStatus status = appStatus.getApprovalStatus();

                if (TimesheetStatus.Submitted.equals(status)
                        || TimesheetStatus.Approved.equals(status)
                        || TimesheetStatus.ApprovedByProjectManager
                            .equals(status)
                        || TimesheetStatus.ApprovedByResourceManager
                            .equals(status))
                {
                    RPMException e = new RPMException(
                        ErrorCodes.CANNOT_MODIFY_TIMESHEET, StringUtil
                            .getShortClassName(appStatus.getClass()),
                        ValidationConstants.APPROVAL_STATUS_FIELD, appStatus
                            .getID());
                    addException(e, msgCtx);
                    valid = false;
                }
            }
        }
        
        return valid;
    }

    protected boolean validWeekOf(AbstractTimesheet timesheet,
            MessageContext msgCtx)
    {
        return GenericValidator.validateMandatory(timesheet,
            ValidationConstants.WEEK_OF_FIELD, timesheet.getWeekOf(),
            msgCtx);
    }
}
