package com.ibm.rpm.timesheet.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.AdministrativeProject;
import com.ibm.rpm.timesheet.containers.PersonalProject;


public class AdministrativeProjectCheckpoint extends TimesheetElementCheckpoint
{
    private static final List ADMINISTRATIVE_PROJECT_PARENT_LIST = new ArrayList();

    protected static AdministrativeProjectCheckpoint instance = new AdministrativeProjectCheckpoint();

    static
    {
        ADMINISTRATIVE_PROJECT_PARENT_LIST.add(PersonalProject.class);
    }

    public static AdministrativeProjectCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        AdministrativeProject administrativeProject = (AdministrativeProject) rpmObject;

        super.isValidParent(rpmObject, msgCtx, false,
            ADMINISTRATIVE_PROJECT_PARENT_LIST);

        if (!msgCtx.isSuccessful())
        {
            return;
        }

        //cannot create
        GenericValidator.validateCannotCreate(administrativeProject,
            msgCtx);

        if (administrativeProject.testNameModified())
        {
            GenericValidator.validateReadOnly(administrativeProject,
                ValidationConstants.NAME_FIELD, msgCtx);
        }

    }

}
