package com.ibm.rpm.timesheet.checkpoints;

import java.util.List;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.managers.StepTimesheetManager;
import com.ibm.rpm.timesheet.scope.StepScope;

public class CreditCheckpoint extends AbstractStepCheckpoint
{
    protected static CreditCheckpoint instance = new CreditCheckpoint();

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {

        if (!GenericValidator.validateInstance(rpmObject, Credit.class, msgCtx))
            return;

        super.validateSave(rpmObject, rpmScope, msgCtx);

        if (!msgCtx.isSuccessful())
        {
            return;
        }

        Credit credit = (Credit) rpmObject;
        StepScope scope = (StepScope) rpmScope;
        
        validateName(credit, msgCtx);

        validateCreditTimesheet(credit, scope, msgCtx);

    }

    private void validateCreditTimesheet(Credit credit, StepScope scope,
            MessageContext msgCtx)
    {
        boolean valid = false;

        if (scope != null && scope.isTimesheets())
        {
            if (credit.getCreditTimesheet() != null)
            {
                valid = true;
            }

        }
        else
        {
            if (credit.getID() == null)
            {
                valid = false;
            }
            else
            {
                // Check in the database that REALLY there
                // is no timesheet for the credit
                SqlBuffer stmt = new SqlBuffer();
                stmt.appendSelect(StepTimesheetManager.NAME_ACTIVITY_ID);
                stmt.appendFrom(StepTimesheetManager.TABLE_NAME);
                stmt.appendWhere();
                stmt
                    .appendEqualQuestionMark(StepTimesheetManager.NAME_ACTIVITY_ID);

                List result = null;
                try
                {
                    result = ManagerUtil.select(null,
                        credit, ValidationConstants.STEP_TIMESHEETS_FIELD,
                        stmt, new Object[]
                        {
                            credit.getID()
                        }, msgCtx);
                }
                catch (RPMException e)
                {
                    addException(e, msgCtx);
                    valid = false;
                }

                valid = result != null && result.size() > 0;
            }
        }

        if (!valid)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(credit.getClass()) + " "
                        + ValidationConstants.CREDIT_TIMESHEET_FIELD
            };
            RPMException e = new RPMException(ErrorCodes.MANDATORY,
                errorParams, credit.getClass().getName(),
                ValidationConstants.CREDIT_TIMESHEET_FIELD, credit.getID());
            addException(e, msgCtx);
        }
        
        
        //validate that the user is not adding a new timesheet
        if (scope != null && scope.isTimesheets())
        {
            if (credit.getCreditTimesheet() != null)
            {
                CreditTimesheet timesheet = credit.getCreditTimesheet();
                if (timesheet != null && timesheet.getID() == null
                        && credit.getID() != null)
                {

                    valid = false;

                    String[] errorParams =
                    {
                            credit.getID()
                    };
                    RPMException e = new RPMException(
                        com.ibm.rpm.timesheet.constants.ErrorCodes.CREDIT_TIMESHEET_ALREADY_EXIST,
                        errorParams, credit.getClass().getName(),
                        ValidationConstants.CREDIT_TIMESHEET_FIELD, credit
                            .getID());
                    addException(e, msgCtx);
                }
            }

        }

    }

}