/*
 * Created on Jan 16, 2006
 * @author mmatwiy
 * Validation checkpoint for Timesheet container TimesheetTask.
 */
package com.ibm.rpm.timesheet.checkpoints;

import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.AbstractTimesheetElement;
import com.ibm.rpm.timesheet.containers.GenericTimesheetElement;

public class TimesheetElementCheckpoint extends AbstractCheckpoint
{
    protected void validateSave(RPMObject rpmObject, MessageContext msgCtx, 
            boolean parentRequired, List validParentTypes)
    {
        super
            .isValidParent(rpmObject, msgCtx, parentRequired, validParentTypes);

        AbstractTimesheetElement timesheetTask = (AbstractTimesheetElement) rpmObject;
        
        // Check that the name exists
        GenericValidator
            .validateMandatory(timesheetTask,
                ValidationConstants.NAME_FIELD, timesheetTask.getName(),
                msgCtx);
        
        if (timesheetTask.testNameModified())
        {
            // Check that the name is at least 1 character long
            GenericValidator
                .validateMinLength(timesheetTask,
                    ValidationConstants.NAME_FIELD, timesheetTask.getName(),
                    ValidationConstants.NAME_MIN, msgCtx);
        }
    }
    
    protected void validateReadOnly(GenericTimesheetElement timesheetElement,
            MessageContext messageContext)
    {

        if (timesheetElement.testActualModified())
        {
            GenericValidator.validateReadOnly(timesheetElement,
                ValidationConstants.ACTUAL_FIELD, messageContext);
        }

        if (timesheetElement.testEETCModified())
        {
            GenericValidator.validateReadOnly(timesheetElement,
                ValidationConstants.EETC_FIELD, messageContext);
        }

        if (timesheetElement.testFinishDateModified())
        {
            GenericValidator.validateReadOnly(timesheetElement,
                ValidationConstants.FINISH_DATE_FIELD, messageContext);
        }

        if (timesheetElement.testPercentageCompletedModified())
        {
            GenericValidator.validateReadOnly(timesheetElement,
                ValidationConstants.PERCENTAGE_COMPLETED_FIELD, messageContext);
        }

        if (timesheetElement.testPercentageWorkModified())
        {
            GenericValidator.validateReadOnly(timesheetElement,
                ValidationConstants.PERCENTAGE_WORK_FIELD, messageContext);
        }

        if (timesheetElement.testStartDateModified())
        {
            GenericValidator.validateReadOnly(timesheetElement,
                ValidationConstants.START_DATE_FIELD, messageContext);
        }
    }
}
