package com.ibm.rpm.timesheet.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.managers.PoolManager;
import com.ibm.rpm.timesheet.containers.AbstractTimesheetElement;
import com.ibm.rpm.timesheet.containers.DefaultAdministrativeTask;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.managers.WbsManager;



public class DefaultAdministrativeTaskManager extends AbstractRPMObjectManager {


	private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        DefaultAdministrativeTaskManager.class);
    static
    {
        containerMap.add(DefaultAdministrativeTask.class);
    }

	/**
	 * Column PROJECT_ID of type Types.CHAR mapped to String.
	 */
	public static final int ID_PROJECT_ID = 1;

	public static final int TYPE_PROJECT_ID = Types.CHAR;

	public static final String NAME_PROJECT_ID = "TMT_WBS.PROJECT_ID";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

	/**
	 * Column PARENT_ID of type Types.CHAR mapped to String.
	 */
	public static final int ID_PARENT_ID = 2;

	public static final int TYPE_PARENT_ID = Types.CHAR;

	public static final String NAME_PARENT_ID = "TMT_WBS.PARENT_ID";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

	/**
	 * Column ELEMENT_ID of type Types.CHAR mapped to String.
	 */
	public static final int ID_ELEMENT_ID = 3;

	public static final int TYPE_ELEMENT_ID = Types.CHAR;

	public static final String NAME_ELEMENT_ID = "TMT_WBS.ELEMENT_ID";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static { FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID); }

	/**
	 * Column TYPE_ID of type Types.INTEGER mapped to Integer.
	 */
	public static final int ID_TYPE_ID = 4;

	public static final int TYPE_TYPE_ID = Types.INTEGER;

	public static final String NAME_TYPE_ID = "TMT_WBS.TYPE_ID";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

	/**
	 * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
	 */
	public static final int ID_ELEMENT_NAME = 5;

	public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

	public static final String NAME_ELEMENT_NAME = "TMT_WBS.ELEMENT_NAME";

	public static final String PROPERTY_ELEMENT_NAME = "NAME";
	static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME,NAME_ELEMENT_NAME);}

	/**
	 * Column CREATED_BY of type Types.CHAR mapped to String.
	 */
	public static final int ID_CREATED_BY = 6;

	public static final int TYPE_CREATED_BY = Types.CHAR;

	public static final String NAME_CREATED_BY = "TMT_WBS.CREATED_BY";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_CREATED_BY = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_CREATED_BY,NAME_CREATED_BY);}
	
	/**
	 * Column REC_STATUS of type Types.CHAR mapped to String.
	 */
	public static final int ID_REC_STATUS = 7;

	public static final int TYPE_REC_STATUS = Types.CHAR;

	public static final String NAME_REC_STATUS = "TMT_WBS.REC_STATUS";

	//uncomment and replace PROPERTYNAME with the container property loaded
	// from this column in upper case.
	//public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
	//static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

	public static final String NAME_POOL_ID = "TMT_WBS.POOL_ID";
	
	public static final String TABLE_NAME = "TMT_WBS";

	/**
	 * Create an array of type string containing all the fields of the TMT_WBS
	 * table.
	 */
	private static final String[] FIELD_NAMES =
    {
            NAME_PROJECT_ID, NAME_PARENT_ID, NAME_ELEMENT_ID, NAME_TYPE_ID,
            NAME_ELEMENT_NAME, NAME_CREATED_BY, 
			NAME_REC_STATUS
    };

	private static final String ALL_FIELDS;
	static {
		StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
		for (int i = 1; i < FIELD_NAMES.length; i++) {
			fieldcat.append(",");
			fieldcat.append(FIELD_NAMES[i]);
		}
		ALL_FIELDS = fieldcat.toString();
	};


	/**
	 * 
	 */
	public DefaultAdministrativeTaskManager() {
		super();
	}
	
	public String getTableName() {
		return TABLE_NAME;
	}

	protected String[] getFieldsNames() {
		return FIELD_NAMES;
	}

	protected String getAllFields() {
		return ALL_FIELDS;
	}

	public String getPrimaryKey() {
		return NAME_ELEMENT_ID;
	}

	protected HashMap getPropertiesMap() {
		return FIELDPROPERTYMAP;
	}
	
    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        return new DefaultAdministrativeTask();
    }

	public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        DefaultAdministrativeTask admintask = (DefaultAdministrativeTask) rpmObject;
        rpmObject.setID(SP_I_ADM_TASK(admintask, messageContext));	    
    }

	protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
			boolean isDelta) throws RPMException, SQLException, ParseException {
        
	    AbstractTimesheetElement temp = (AbstractTimesheetElement) rpmObject;
        temp.setID(rs.getString(ID_ELEMENT_ID));
        String name = rs.getString(ID_ELEMENT_NAME);	
        if (name != null) {
            name = name.trim();
        }
        if (isDelta) {
            temp.deltaName(name);
        } else {
            temp.setName(name);
        }
        return temp;        
	}

	protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
	    SP_D_ADM_TASK((DefaultAdministrativeTask) rpmObject, messageContext);
    }

	
    public String getFilter()
    {
        SqlBuffer sql = new SqlBuffer();
        sql.appendIsNotNull(NAME_ELEMENT_ID);
        sql.appendAnd();
        sql.append(NAME_REC_STATUS);
        sql.append(" != 'D' ");
        
        return sql.toString();
        
    }
	protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        //DefaultAdministrativeTask
        if (manager instanceof PoolManager)
        {
            joinCondition = new JoinCondition(getTableName(), getFilter());
            joinCondition.appendEqual(NAME_TYPE_ID, DefaultAdministrativeTask.TYPE_ID);
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID,
                TABLE_NAME, WbsManager.NAME_POOL_ID + "= ? ");
            joinCondition.appendEqual(NAME_TYPE_ID, SummaryTask.TYPE_ID);
        }
         else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

	protected JoinCondition getJoinCondition(RPMObjectManager manager,
			String condition, int level, String containerName,
			String contextName) throws RPMException {
		
	    JoinCondition joinCondition = null;
        if (manager == null){
	        joinCondition = new JoinCondition(getTableName(), getFilter());
	        joinCondition.appendEqual(NAME_TYPE_ID, DefaultAdministrativeTask.TYPE_ID);
	        
		}
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

		return joinCondition;
	}

	protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
			String condition, int level, String containerName,
			String contextName) throws RPMException {

	    JoinCondition joinCondition = null;
	    
	    if (manager instanceof PoolManager)
        {
            SqlBuffer where = new SqlBuffer();
            where.appendSubSelect(
                DefaultAdministrativeTaskManager.NAME_ELEMENT_ID, NAME_PARENT_ID,
                TABLE_NAME, getFilter());

            joinCondition = JoinCondition.createSubSelect(
                PoolManager.NAME_ELEMENT_ID,
                NAME_POOL_ID,
                DefaultAdministrativeTaskManager.TABLE_NAME, where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

	protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        if (rpmScope != null && rpmScope.getParent() != null)
        {
            String parentId = retrieveAdministrativeTaskPoolId(
                (DefaultAdministrativeTask) rpmObject, messageContext);

            Pool pool = null;
            if (parentId != null)
            {
                pool = new Pool();
                pool.setID(parentId);

                RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(Pool.class);
                pool = (Pool) manager.loadByPrimaryKey(pool, rpmScope
                    .getParent(), messageContext, false);
            }

            rpmObject.setParent(pool);

        }

        return rpmObject;
    }
	
	protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
			FieldValueMap relatedFields, int index) throws RPMException,
			SQLException {
        String projectId = resultSet.getString(ID_PROJECT_ID);
        relatedFields.put(index, ID_PROJECT_ID, projectId);
        return relatedFields;
	}

	public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        AbstractTimesheetElement timesheetTask = (AbstractTimesheetElement) rpmObject;

        SP_U_ADM_TASK_NAME((DefaultAdministrativeTask) timesheetTask,
            messageContext);

        return rpmObject;
    }
	
	private String SP_I_ADM_TASK(DefaultAdministrativeTask adminTask,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_I_ADM_TASK";
        Object args[] = new Object[4];

        String parentId = retrieveAdministrativeTaskParent(adminTask,
            messageContext);

        args[0] = adminTask.getName(); // name
        args[1] = parentId; // parent_id
        args[2] = adminTask.retrieveTypeID(); // type_id
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, procedureName, args);
    }
	
	private String SP_U_ADM_TASK_NAME(DefaultAdministrativeTask administrativeTask,
			MessageContext messageContext) throws SQLException, RPMException 
    {
		final String procedureName = "SP_U_ADM_TASK_NAME";
		Object args[] = new Object[3];
		
		args[0] = administrativeTask.getID();  // id
		args[1] = administrativeTask.getName(); // name
		args[2] = getUser(messageContext).getID();
		
		return executeProcedure(messageContext, procedureName, args);
	}

	private String SP_D_ADM_TASK(DefaultAdministrativeTask adminTask,
			MessageContext messageContext) throws SQLException, RPMException 
    {
		final String procedureName = "SP_D_ADM_TASK";
		
		//note: we cannot use parent.getId to delete the object, we need the
        // actual parentid from the database
		adminTask.setParent(null);
		
		Object args[] = new Object[4];
	
		args[0] = adminTask.getID();  // id
		args[1] = ManagerUtil.getParentId(this, adminTask, messageContext, TABLE_NAME);// parent_id
		args[2] = "X";
		args[3] = getUser(messageContext).getID();
		
		return executeProcedure(messageContext, procedureName, args);
	}
	
	private String retrieveAdministrativeTaskParent(
            DefaultAdministrativeTask administrativeTask, MessageContext messageContext)
            throws RPMException
    {

        String parentId = null;

        Pool pool = (Pool) administrativeTask.getParent();

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(WbsManager.NAME_ELEMENT_ID);
        stmt.appendFrom(WbsManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqual(NAME_TYPE_ID, SummaryTask.TYPE_ID);
        stmt.appendEqualQuestionMark(WbsManager.NAME_POOL_ID);

        List result = ManagerUtil.select(new Class[]
        {
            String.class
        }, null, null, stmt, new Object[]
        {
            pool.getID()
        }, messageContext);

        //result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            ArrayList columns = (ArrayList) result.get(0);
            parentId = (String) columns.get(0);
        }

        return parentId;

    }
    
    private static String retrieveAdministrativeTaskPoolId(
            DefaultAdministrativeTask administrativeTask, MessageContext messageContext)
            throws RPMException
    {

        String value = null;

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(NAME_POOL_ID);
        stmt.appendFrom(TABLE_NAME);
        stmt.appendWhere();
        stmt.appendSubSelect(NAME_ELEMENT_ID, NAME_PARENT_ID, TABLE_NAME, NAME_ELEMENT_ID + " = ?");

        List result = ManagerUtil.select(null, null, null, stmt, new Object[]
        {
            administrativeTask.getID()
        }, messageContext);

        //result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            ArrayList columns = (ArrayList) result.get(0);
            value = (String) columns.get(0);
        }

        return value;
    }
}
