package com.ibm.rpm.timesheet.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;

import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.managers.TimecodeManager;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.timesheet.containers.DefaultStep;
import com.ibm.rpm.timesheet.scope.DefaultStepScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class DefaultStepManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        DefaultStepManager.class);

    static
    {
        containerMap.add(DefaultStep.class);
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;
    public static final int TYPE_ELEMENT_ID = Types.CHAR;
    public static final String NAME_ELEMENT_ID = "FAVORITE_ACTIVITIES.ELEMENT_ID";
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 2;
    public static final int TYPE_ELEMENT_NAME = Types.CHAR;
    public static final String NAME_ELEMENT_NAME = "FAVORITE_ACTIVITIES.ELEMENT_NAME";
    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column BILLABLE of type Types.CHAR mapped to String.
     */
    public static final int ID_BILLABLE = 3;
    public static final int TYPE_BILLABLE = Types.CHAR;
    public static final String NAME_BILLABLE = "FAVORITE_ACTIVITIES.BILLABLE";
    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BILLABLE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_BILLABLE,NAME_BILLABLE);}

    /**
     * Column TIMECODE_1_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_TIMECODE_1_ID = 4;
    public static final int TYPE_TIMECODE_1_ID = Types.CHAR;
    public static final String NAME_TIMECODE_1_ID = "FAVORITE_ACTIVITIES.TIMECODE_1_ID";
    public static final String PROPERTY_TIMECODE_1_ID = "TIMECODE1ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_1_ID, NAME_TIMECODE_1_ID);
    }

    /**
     * Column TIMECODE_2_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_TIMECODE_2_ID = 5;
    public static final int TYPE_TIMECODE_2_ID = Types.CHAR;
    public static final String NAME_TIMECODE_2_ID = "FAVORITE_ACTIVITIES.TIMECODE_2_ID";
    public static final String PROPERTY_TIMECODE_2_ID = "TIMECODE2ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_2_ID, NAME_TIMECODE_2_ID);
    }

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 6;
    public static final int TYPE_PROJECT_ID = Types.CHAR;
    public static final String NAME_PROJECT_ID = "FAVORITE_ACTIVITIES.PROJECT_ID";
    public static final String PROPERTY_PROJECT_ID = "PROJECTID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID, NAME_PROJECT_ID);
    }

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 7;
    public static final int TYPE_RESOURCE_ID = Types.CHAR;
    public static final String NAME_RESOURCE_ID = "FAVORITE_ACTIVITIES.RESOURCE_ID";
    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RESOURCE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID,NAME_RESOURCE_ID);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 8;
    public static final int TYPE_REC_USER = Types.CHAR;
    public static final String NAME_REC_USER = "FAVORITE_ACTIVITIES.REC_USER";
    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 9;
    public static final int TYPE_REC_STATUS = Types.CHAR;
    public static final String NAME_REC_STATUS = "FAVORITE_ACTIVITIES.REC_STATUS";
    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 10;
    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;
    public static final String NAME_REC_DATETIME = "FAVORITE_ACTIVITIES.REC_DATETIME";
    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    public static final String TABLE_NAME = "FAVORITE_ACTIVITIES";

    /**
     * Create an array of type string containing all the fields of the
     * FAVORITE_ACTIVITIES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_ELEMENT_NAME, NAME_BILLABLE,
            NAME_TIMECODE_1_ID, NAME_TIMECODE_2_ID, NAME_PROJECT_ID,
            NAME_RESOURCE_ID, NAME_REC_USER, NAME_REC_STATUS,
            NAME_REC_DATETIME
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new FavoriteActivityManager instance.
     * 
     * @return FavoriteActivityManager
     */
    public DefaultStepManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public ContainerMap getContainerMap()
    {
        return containerMap;
    }
    
    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        return new Integer(0);
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        return new DefaultStep();
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        DefaultStep step = (DefaultStep) rpmObject;

        if (isDelta)
            step.deltaName(rs.getString(ID_ELEMENT_NAME));
        else
            step.setName(rs.getString(ID_ELEMENT_NAME));

        return rpmObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String projectId = resultSet.getString(ID_PROJECT_ID);
        relatedFields.put(index, ID_PROJECT_ID, projectId);

        String timeCode1Id = resultSet.getString(ID_TIMECODE_1_ID);
        relatedFields.put(index, ID_TIMECODE_1_ID, timeCode1Id);

        String timeCode2Id = resultSet.getString(ID_TIMECODE_2_ID);
        relatedFields.put(index, ID_TIMECODE_2_ID, timeCode2Id);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        DefaultStep step = (DefaultStep)rpmObject;

        
        if (rpmScope != null) 
        {
            if (rpmScope.getParent() != null)
            {
                loadParent(step, rpmScope, messageContext, relatedFields, 
                    index, isDelta);
            }

            if(rpmScope instanceof DefaultStepScope){
            
                DefaultStepScope scope = (DefaultStepScope)rpmScope;
                
                if (scope.isTimeCodes())
                {
                    loadTimeCodes(step, scope, messageContext,
                        relatedFields, index, isDelta);
                }
            }
        }
        
        return rpmObject;
    }

    private void loadParent(DefaultStep step, 
            RPMObjectScope rpmscope, MessageContext messageContext, 
            FieldValueMap relatedFields, int index, boolean isDelta) 
    	throws RPMException, SQLException, ParseException
    {
        if (rpmscope.getParent() == null)
            return;
        
        String oldParentId = (String) relatedFields.get(index, ID_PROJECT_ID);
        RPMObject newParent = null;

        WbsManager manager = (WbsManager) getManagerInstance(Project.class.
            getName());

        if (isDelta)
        {
            newParent = step.getParent();
            RPMObject oldParent = null;
            if (oldParentId != null)
            {
                oldParent = new GenericProject();
                oldParent.setID(oldParentId);
                if (newParent != null && newParent.getID() != null
                        && newParent.getID().equals(oldParentId))
                {
                    oldParent = manager.loadByPrimaryKey(newParent, 
                        rpmscope.getParent(), messageContext, true);
                }
                else
                {
                    //In this case, old parent is loaded without the
                    // scope to avoid problem with the cache
                    oldParent = manager.loadByPrimaryKey(oldParent, 
                        rpmscope.getParent(), messageContext, false);
                }
            }
            step.deltaParent(oldParent);
        }
        else
        {
            if (oldParentId != null)
            {
                if (newParent == null)
                {//newParent not in cache
                    newParent = new GenericProject();
                    newParent.setID(oldParentId);
                    newParent = manager.loadByPrimaryKey(newParent, 
                        rpmscope.getParent(), messageContext, false);
                }
                step.setParent(newParent);
            }
        }
    }

    private void loadTimeCodes(DefaultStep step, 
            DefaultStepScope scope, MessageContext messageContext, 
            FieldValueMap relatedFields, int index, boolean isDelta) 
    	throws RPMException, SQLException, ParseException
    {
        if (!scope.isTimeCodes())
            return;
        
        String timeCode1Id = (String) relatedFields.get(index, ID_TIMECODE_1_ID);
        TimeCode oldTimeCode1 = step.getTimeCode1();
        TimeCode loadedTimeCode1 = loadTimeCode(null,
            messageContext, timeCode1Id, oldTimeCode1, isDelta);

        String timeCode2Id = (String) relatedFields.get(index, ID_TIMECODE_2_ID);
        TimeCode oldTimeCode2 = step.getTimeCode2();
        TimeCode loadedTimeCode2 = loadTimeCode(null,
            messageContext, timeCode2Id, oldTimeCode2, isDelta);

        if (isDelta)
        {
            step.deltaTimeCode1(loadedTimeCode1);
            step.deltaTimeCode2(loadedTimeCode2);
        }
        else
        {
            step.setTimeCode1(loadedTimeCode1);
            step.setTimeCode2(loadedTimeCode2);
        }            
    }
    
    private TimeCode loadTimeCode(TimeCodeScope timeCodeScope, 
            MessageContext messageContext, String oldTimeCodeID, 
            TimeCode previousTimeCode, boolean isDelta)
    	throws RPMException, SQLException, ParseException
    {
        TimecodeManager manager = (TimecodeManager) getManagerInstance(
            TimeCode.class.getName());

        TimeCode newTimeCode = null;
        if (isDelta)
        {
            newTimeCode = previousTimeCode;
            TimeCode oldTimeCode = null;
            if (oldTimeCodeID != null)
            {
                oldTimeCode = new TimeCode();
                oldTimeCode.setID(oldTimeCodeID);
                if (newTimeCode != null && newTimeCode.getID() != null
                        && newTimeCode.getID().equals(oldTimeCodeID))
                {
                    oldTimeCode = (TimeCode)manager.loadByPrimaryKey(newTimeCode, 
                        timeCodeScope, messageContext, true);
                }
                else
                {
                    //In this case, old timecode is loaded without the
                    // scope to avoid problem with the cache
                    oldTimeCode = (TimeCode)manager.loadByPrimaryKey(oldTimeCode, 
                        timeCodeScope, messageContext, false);
                }
            }
            return oldTimeCode;
        }
        else if (oldTimeCodeID != null)
        {
            if (newTimeCode == null)
            { 
                // newTimecode not in cache
                newTimeCode = new TimeCode();
                newTimeCode.setID(oldTimeCodeID);
                newTimeCode = (TimeCode) manager.loadByPrimaryKey(newTimeCode,
                    timeCodeScope, messageContext, false);
            }
            return newTimeCode;
        }
        return null;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if(manager instanceof WbsManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(DefaultStepManager.NAME_PROJECT_ID);
            where.append(" = ?");           
          
            if (extraWhere != null && extraWhere.length() > 0)
            {
                where.append(joinCondition.getCondition());
                where.append(" AND ");
                where.append(extraWhere);              
            }
            
            joinCondition.setCondition(where.toString());  
            
            joinCondition.setTableName(getTableName());
        }
		else
		{
	        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
	                ExceptionUtil.FOREIGN_KEY);
		}        
        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        DefaultStep step = (DefaultStep) rpmObject;
        step.setID(SP_I_FAV_ACT(step, messageContext));
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        DefaultStep step = (DefaultStep)rpmObject;
        DefaultStepScope scope = (DefaultStepScope)rpmScope;

        if (step.testNameModified()
                || step.testTimeCode1Modified()
                || step.testTimeCode2Modified())
        {
            if (scope != null && 
               (scope.getParent() == null || !scope.isTimeCodes()))
            {
                //TODO investigate why a reload with full scope is needed
                reloadFavoriteActivityWithFullScope(step, messageContext);
            } else if (scope == null) {
                // load the parent
                setParent(step, messageContext);
            }
            
            SP_U_FAV_ACT(step, messageContext);
        }

        return rpmObject;
    }
    
    private DefaultStep reloadFavoriteActivityWithFullScope(
            DefaultStep step, MessageContext messageContext) 
    	throws RPMException, ParseException, SQLException
    {
        DefaultStepScope fullScope = new DefaultStepScope();
        fullScope.setParent(new WorkElementScope());
        fullScope.setTimeCodes(true);
        
        return (DefaultStep)loadByPrimaryKey(step, fullScope, 
            messageContext, false);
    }
    
    private void setParent(DefaultStep step, MessageContext messageContext)
            throws RPMException, ParseException, SQLException
    {
        if (step.getParent() != null)
            return;

        //reload the object with its parent in the scope, to be able to
        // retrieve the parent ID
        RPMObject current = new DefaultStep();
        current.setID(step.getID());
        DefaultStepScope scope = new DefaultStepScope();
        scope.setParent(new WorkElementScope());
        loadByPrimaryKey(current, scope, messageContext, false);
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (rpmObject != null)
            SP_D_FAV_ACT((DefaultStep)rpmObject, messageContext);
    }

    private String SP_I_FAV_ACT(DefaultStep step, 
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_I_FAV_ACT";
        Object args[] = new Object[4];
        args[0] = step.getName(); // name
        args[1] = step.getParent().getID(); // parent id
        args[2] = new Integer(0); // type_id
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, procedureName, args);
    }

    private void SP_D_FAV_ACT(DefaultStep step, 
            MessageContext messageContext) throws SQLException, RPMException
    {
        String parentID = (step.getParent() != null) ?
                step.getParent().getID() : null;
        
        final String procedureName = "SP_D_FAV_ACT";
        Object args[] = new Object[4];
        args[0] = step.getID(); // id
        args[1] = parentID; // parent_id
        args[2] = new String("X"); // del_flag
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_U_FAV_ACT(DefaultStep step, 
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_FAV_ACT";
        Object args[] = new Object[5];
        args[0] = step.getID(); // id
        args[1] = step.getName(); // name
        args[2] = (step.getTimeCode1() == null) ? null : step
            .getTimeCode1().getID(); // timecode_1_id
        args[3] = (step.getTimeCode2() == null) ? null : step
            .getTimeCode2().getID(); // timecode_2_id
        args[4] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }
}
