package com.ibm.rpm.timesheet.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashSet;

import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.timesheet.containers.PersonalTask;
import com.ibm.rpm.timesheet.containers.PersonalTaskAssignment;
import com.ibm.rpm.wbs.managers.WbsManager;

/**
 * @author gmeloche
 *
 */
public class PersonalTaskAssignmentManager extends AbstractTimesheetTaskAssignmentManager
{
    private static final HashSet CONTAINERS = new HashSet();

    static{
        CONTAINERS.add(PersonalTaskAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
            CONTAINERS.toArray(objs);
            for (int i = 0; i < objs.length; i++) {
                RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                PersonalTaskAssignmentManager.class);
        }
    }
    
    public String getFilter(){
        
        
        StringBuffer whereWbs = new StringBuffer();
        whereWbs.append(WbsManager.NAME_ELEMENT_ID);
        whereWbs.append(" IS NOT NULL ");
        whereWbs.append(" AND ");
        whereWbs.append(WbsManager.NAME_REC_STATUS);
        whereWbs.append(" != 'D'");
        
        SqlBuffer sql = new SqlBuffer();
        sql.appendSubSelect(NAME_TASK_ID, WbsManager.NAME_ELEMENT_ID, WbsManager.TABLE_NAME, whereWbs.toString());
        sql.appendEqual(WbsManager.NAME_TYPE_ID,PersonalTask.TYPE_ID);
        sql.appendCloseParenthesis();
        sql.appendSubSelect(NAME_RESOURCE_ID, ResourceManager.NAME_RESOURCE_ID, ResourceManager.TABLE_NAME, new ResourceManager().getFilter());
        sql.appendCloseParenthesis();
        
        return sql.toString();
    }
    
    
    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        return new PersonalTaskAssignment();
    }
	protected RPMObject loadRelatedObjects(RPMObject rpmObject,
			RPMObjectScope rpmScope, MessageContext messageContext,
			FieldValueMap relatedFields, int index, boolean isDelta)
			throws RPMException, SQLException, ParseException {
		
		PersonalTaskAssignment assignment = (PersonalTaskAssignment)rpmObject;
		
		//load the personal tasks
		TimesheetTaskManager mgr = (TimesheetTaskManager) getManagerInstance(PersonalTask.class
            .getName());
        PersonalTask task = new PersonalTask();
        task.setID((String) relatedFields.get(index, ID_TASK_ID));
        task = (PersonalTask) mgr.loadByPrimaryKey(task, null, messageContext,
            isDelta);
        if (isDelta)
        {
            assignment.deltaPersonalTask(task);
        }
        else
        {
            assignment.setPersonalTask(task);
        }
        
        //load the parent
        loadParent(assignment, relatedFields, index, isDelta, messageContext);
  
        return assignment;
	}
	
	protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition(TABLE_NAME, getFilter());
            if (condition != null)
            {
                joinCondition.setCondition(condition);
            }
        }
        else if (manager instanceof TimesheetTaskManager)
        {

            joinCondition = JoinCondition.createSubSelect(
                TimesheetTaskManager.NAME_ELEMENT_ID, NAME_TASK_ID, TABLE_NAME,
                getFilter());

        }
        else
        {

            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }
}
