package com.ibm.rpm.timesheet.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.managers.TimecodeManager;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.ArrayUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.RPMObjectUtil;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.timesheet.containers.AbstractTimesheet;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.GenericStep;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.util.TimesheetManagerUtil;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.TaskAssignmentManager;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.xpathparser.XPathContainer;

public class StepManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    // =============
    // CONTAINER MAP
    // =============

    private static ContainerMap containerMap = new ContainerMap(
        StepManager.class);
    static
    {
        containerMap.add(GenericStep.class);
        containerMap.add(Step.class);
        containerMap.add(Credit.class);
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "ACTIVITIES.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ELEMENT_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 2;

    public static final int TYPE_ELEMENT_NAME = Types.CHAR;

    public static final String NAME_ELEMENT_NAME = "ACTIVITIES.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column TIMECODE_1_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_TIMECODE_1_ID = 3;

    public static final int TYPE_TIMECODE_1_ID = Types.CHAR;

    public static final String NAME_TIMECODE_1_ID = "ACTIVITIES.TIMECODE_1_ID";

    public static final String PROPERTY_TIMECODE_1_ID = "TIMECODE1ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_1_ID, NAME_TIMECODE_1_ID);
    }

    /**
     * Column TIMECODE_1_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_TIMECODE_1_NAME = 4;

    public static final int TYPE_TIMECODE_1_NAME = Types.CHAR;

    public static final String NAME_TIMECODE_1_NAME = "ACTIVITIES.TIMECODE_1_NAME";

    public static final String PROPERTY_TIMECODE_1_NAME = "TIMECODE1NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_1_NAME, NAME_TIMECODE_1_NAME);
    }

    /**
     * Column TIMECODE_2_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_TIMECODE_2_ID = 5;

    public static final int TYPE_TIMECODE_2_ID = Types.CHAR;

    public static final String NAME_TIMECODE_2_ID = "ACTIVITIES.TIMECODE_2_ID";

    public static final String PROPERTY_TIMECODE_2_ID = "TIMECODE2ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_2_ID, NAME_TIMECODE_2_ID);
    }

    /**
     * Column TIMECODE_2_NAME of type Types.CHAR mapped to String.
     */
    public static final int ID_TIMECODE_2_NAME = 6;

    public static final int TYPE_TIMECODE_2_NAME = Types.CHAR;

    public static final String NAME_TIMECODE_2_NAME = "ACTIVITIES.TIMECODE_2_NAME";

    public static final String PROPERTY_TIMECODE_2_NAME = "TIMECODE2NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_2_NAME, NAME_TIMECODE_2_NAME);
    }

    /**
     * Column TSK_ASSIGNMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_TSK_ASSIGNMENT_ID = 7;

    public static final int TYPE_TSK_ASSIGNMENT_ID = Types.CHAR;

    public static final String NAME_TSK_ASSIGNMENT_ID = "ACTIVITIES.TSK_ASSIGNMENT_ID";

    public static final String PROPERTY_TSK_ASSIGNMENT_ID = "TASKASSIGNMENTID";
    static
    {
        FIELDPROPERTYMAP
            .put(PROPERTY_TSK_ASSIGNMENT_ID, NAME_TSK_ASSIGNMENT_ID);
    }

    /**
     * Column BILLABLE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BILLABLE = 8;

    public static final int TYPE_BILLABLE = Types.SMALLINT;

    public static final String NAME_BILLABLE = "ACTIVITIES.BILLABLE";

    public static final String PROPERTY_BILLABLE = "BILLABLE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BILLABLE, NAME_BILLABLE);
    }

    /**
     * Column EFFORT_ASSIGNMENT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_EFFORT_ASSIGNMENT = 9;

    public static final int TYPE_EFFORT_ASSIGNMENT = Types.BIGINT;

    public static final String NAME_EFFORT_ASSIGNMENT = "ACTIVITIES.EFFORT_ASSIGNMENT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EFFORT_ASSIGNMENT = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_EFFORT_ASSIGNMENT,NAME_EFFORT_ASSIGNMENT);}

    /**
     * Column WORK_TODATE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_WORK_TODATE = 10;

    public static final int TYPE_WORK_TODATE = Types.BIGINT;

    public static final String NAME_WORK_TODATE = "ACTIVITIES.WORK_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WORK_TODATE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WORK_TODATE,NAME_WORK_TODATE);}

    /**
     * Column ETC of type Types.BIGINT mapped to Long.
     */
    public static final int ID_ETC = 11;

    public static final int TYPE_ETC = Types.BIGINT;

    public static final String NAME_ETC = "ACTIVITIES.ETC";

    public static final String PROPERTY_ETC = "EETC";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ETC, NAME_ETC);
    }

    /**
     * Column MILESTONE_PERCENT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MILESTONE_PERCENT = 12;

    public static final int TYPE_MILESTONE_PERCENT = Types.SMALLINT;

    public static final String NAME_MILESTONE_PERCENT = "ACTIVITIES.MILESTONE_PERCENT";

    public static final String PROPERTY_MILESTONE_PERCENT = "PERCENTAGECOMPLETE";
    static
    {
        FIELDPROPERTYMAP
            .put(PROPERTY_MILESTONE_PERCENT, NAME_MILESTONE_PERCENT);
    }

    /**
     * Column EDPERCENT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_EDPERCENT = 13;

    public static final int TYPE_EDPERCENT = Types.SMALLINT;

    public static final String NAME_EDPERCENT = "ACTIVITIES.EDPERCENT";

    public static final String PROPERTY_EDPERCENT = "WEIGHT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EDPERCENT, NAME_EDPERCENT);
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 14;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "ACTIVITIES.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 15;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "ACTIVITIES.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 16;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "ACTIVITIES.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column ETC_MODIFIED of type Types.CHAR mapped to String.
     */
    public static final int ID_ETC_MODIFIED = 17;

    public static final int TYPE_ETC_MODIFIED = Types.CHAR;

    public static final String NAME_ETC_MODIFIED = "ACTIVITIES.ETC_MODIFIED";

    public static final String PROPERTY_ETC_MODIFIED = "ETCCHANGED";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ETC_MODIFIED, NAME_ETC_MODIFIED);
    }

    /**
     * Column TYPE_ID of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_TYPE_ID = 18;

    public static final int TYPE_TYPE_ID = Types.SMALLINT;

    public static final String NAME_TYPE_ID = "ACTIVITIES.TYPE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column SHOW_STEP of type Types.CHAR mapped to String.
     */
    public static final int ID_SHOW_STEP = 19;

    public static final int TYPE_SHOW_STEP = Types.CHAR;

    public static final String NAME_SHOW_STEP = "ACTIVITIES.SHOW_STEP";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SHOW_STEP = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_SHOW_STEP,NAME_SHOW_STEP);}

    public static final String TABLE_NAME = "ACTIVITIES";

    /**
     * Create an array of type string containing all the fields of the
     * ACTIVITIES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_ELEMENT_NAME, NAME_TIMECODE_1_ID,
            NAME_TIMECODE_1_NAME, NAME_TIMECODE_2_ID, NAME_TIMECODE_2_NAME,
            NAME_TSK_ASSIGNMENT_ID, NAME_BILLABLE, NAME_EFFORT_ASSIGNMENT,
            NAME_WORK_TODATE, NAME_ETC, NAME_MILESTONE_PERCENT, NAME_EDPERCENT,
            NAME_REC_USER, NAME_REC_STATUS, NAME_REC_DATETIME,
            NAME_ETC_MODIFIED, NAME_TYPE_ID, NAME_SHOW_STEP
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new ActivityManager instance.
     */
    public StepManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    private String getFinalCondition(XPathContainer container)
            throws RPMException
    {
        // Add the basic filtering condition.
        SqlBuffer finalCondition = new SqlBuffer(getFilter());

        // Add the type ID condition, if required.
        Collection typeIds = containerMap.getTypeIds(container.getType());
        finalCondition.appendIn(NAME_TYPE_ID, typeIds);

        return finalCondition.toString();
    }

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager == null)
        {
            // example: /Step
            joinCondition = new JoinCondition(getTableName(),
                getFinalCondition(container));
        }
        else if (manager instanceof StepTimesheetManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                StepTimesheetManager.NAME_ACTIVITY_ID, NAME_ELEMENT_ID,
                getTableName(), getFinalCondition(container));

        }
        else if (manager instanceof TaskAssignmentManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
                NAME_TSK_ASSIGNMENT_ID, TABLE_NAME,
                getFinalCondition(container));

        }
        else if (manager instanceof ResourceManager)
        {
            //Step/@resource
            SqlBuffer subCondition = new SqlBuffer();
            subCondition.appendSubSelect(
                TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
                NAME_TSK_ASSIGNMENT_ID, TABLE_NAME,
                getFinalCondition(container));

            joinCondition = JoinCondition.createSubSelect(
                ResourceManager.NAME_RESOURCE_ID,
                TaskAssignmentManager.NAME_RESOURCE_ID,
                TaskAssignmentManager.TABLE_NAME, subCondition.toString());

        }
        else if (manager instanceof WbsManager)
        {
            //Step/@project
            if ("project".equalsIgnoreCase(container.getFieldName()))
            {
                SqlBuffer subCondition = new SqlBuffer();
                subCondition.appendSubSelect(
                    TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
                    NAME_TSK_ASSIGNMENT_ID, TABLE_NAME,
                    getFinalCondition(container));

                joinCondition = JoinCondition.createSubSelect(
                    WbsManager.NAME_ELEMENT_ID,
                    TaskAssignmentManager.NAME_PROJECT_ID,
                    TaskAssignmentManager.TABLE_NAME, subCondition.toString());
            }
            else
            {
                SqlBuffer subCondition = new SqlBuffer();
                subCondition.appendSubSelect(
                    TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
                    NAME_TSK_ASSIGNMENT_ID, TABLE_NAME,
                    getFinalCondition(container));

                joinCondition = JoinCondition.createSubSelect(
                    WbsManager.NAME_ELEMENT_ID,
                    TaskAssignmentManager.NAME_TASK_ID,
                    TaskAssignmentManager.TABLE_NAME, subCondition.toString());
            }

        }        
        else if (manager instanceof TimecodeManager)
        {

            joinCondition = new JoinCondition();
            if ("timecode1".equalsIgnoreCase(container.getFieldName()))
            {
                // example: /Step/@timecode1
                joinCondition.appendSubSelect(TimecodeManager.NAME_ELEMENT_ID,
                    NAME_TIMECODE_1_ID, TABLE_NAME, getFilter());
            }
            else
            {
                // example: /Step/@timecode2
                joinCondition.appendSubSelect(TimecodeManager.NAME_ELEMENT_ID,
                    NAME_TIMECODE_2_ID, TABLE_NAME, getFilter());
            }
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.SUB_TABLE_JOIN);

        return null;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return containerMap.createContainer(typeId);
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        GenericStep object = (GenericStep) rpmObject;

        if (isDelta)
        {
            object.deltaName(rs.getString(ID_ELEMENT_NAME));
        }
        else
        {
            object.setName(rs.getString(ID_ELEMENT_NAME));
        }

        if (object instanceof Step)
        {

            Step step = (Step) object;

            // get billable value
            Boolean billable = decodeBillable(rs.getShort(ID_BILLABLE));
            //get the ETC_Modified field
            Boolean etcChanged = decodeETCChanged(rs.getString(ID_ETC_MODIFIED));

            // get the show_step (always available) field
            boolean alwaysAvailable = false;
            String alwaysAvailable_value = rs.getString(ID_SHOW_STEP);
            if (alwaysAvailable_value.equals("Y"))
                alwaysAvailable = true;
            else if (alwaysAvailable_value.equals("N"))
                alwaysAvailable = false;
            else
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(getClass().getName()),
                        "of field alwaysAvailable ", "true or false"
                };

                RPMException e = new RPMException(ErrorCodes.INVALID_VALUE,
                    errorParams, getClass().getName(), NAME_SHOW_STEP);
                e.setSeverity(SeverityLevel.SevereError);
                throw e;
            }

            // double percentCompleted = (double) rs.getInt(ID_MILESTONE_PERCENT) / 100;
            Double weight = decodeWeight(rs.getInt(ID_EDPERCENT));

            if (isDelta)
            {
                step.deltaBillable(billable);
                step.deltaEETC(DateUtil.convertMinutesToHours(rs
                    .getLong(ID_ETC)));
                step.deltaWeight(weight);
                step.deltaETCChanged(etcChanged);
                step.deltaAlwaysAvailable(new Boolean(alwaysAvailable));

            }
            else
            {
                step.setBillable(billable);
                step
                    .setEETC(DateUtil.convertMinutesToHours(rs.getLong(ID_ETC)));
                step.setWeight(weight);
                step.setETCChanged(etcChanged);
                step.setAlwaysAvailable(new Boolean(alwaysAvailable));

            }
        }

        return rpmObject;
    }

    public Boolean decodeBillable(short billable_value) throws RPMException
    {
        // get billable value
        boolean billable = false;

        if (billable_value == 0)
            billable = false;
        else if (billable_value == 1)
            billable = true;
        else
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(getClass().getName()),
                    "of field billable ", "true or false"
            };
            RPMException e = new RPMException(ErrorCodes.INVALID_VALUE,
                errorParams, getClass().getName(), NAME_BILLABLE);
            e.setSeverity(SeverityLevel.SevereError);
            throw e;
        }
        return new Boolean(billable);
    }
    
    public Boolean decodeETCChanged(String etc_modified_value) throws RPMException
    {
        boolean etc_modified = false;
        if (etc_modified_value.equals("Y"))
            etc_modified = true;
        else if (etc_modified_value.equals("N"))
            etc_modified = false;
        else
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(getClass().getName()),
                    "of field etcChanged ", "true or false"
            };

            RPMException e = new RPMException(ErrorCodes.INVALID_VALUE,
                errorParams, getClass().getName(), NAME_ETC_MODIFIED);
            e.setSeverity(SeverityLevel.SevereError);
            throw e;
        }
        return new Boolean(etc_modified);
    }
    
    public Double decodeWeight(int weight) throws RPMException
    {
        return new Double((double) weight / 100);
    }
    
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String timeCode1Id = resultSet.getString(ID_TIMECODE_1_ID);
        relatedFields.put(index, ID_TIMECODE_1_ID, timeCode1Id);

        String timeCode2Id = resultSet.getString(ID_TIMECODE_2_ID);
        relatedFields.put(index, ID_TIMECODE_2_ID, timeCode2Id);

        String parentId = resultSet.getString(ID_TSK_ASSIGNMENT_ID);
        relatedFields.put(index, ID_TSK_ASSIGNMENT_ID, parentId);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        GenericStep step = (GenericStep) rpmObject;

        if (rpmScope != null && rpmScope instanceof StepScope)
        {
            StepScope scope = (StepScope) rpmScope;

            if (scope.isTaskAssignment())
            {
                loadTaskAssignment(step, scope, messageContext, relatedFields,
                    index, isDelta);
            }

            if (scope.isTimesheets())
            {
                if (step instanceof Step)
                {
                    loadStepTimesheets((Step) step, scope, messageContext,
                        isDelta);
                }
                else if (step instanceof Credit)
                {
                    loadCreditTimesheet((Credit) step, scope, messageContext,
                        isDelta);
                }
            }

            if (scope.isTimeCodes())
            {
                loadTimeCodes(step, scope, messageContext, relatedFields,
                    index, isDelta);
            }

            if (scope.isProject())
            {
                loadProject(step, messageContext, relatedFields, index, isDelta);
            }

            if (scope.isResource())
            {
                loadResource(step, messageContext, relatedFields, index,
                    isDelta);
            }

            if (scope.getWorkElement() != null)
            {
                loadWorkElement(step, scope, messageContext, relatedFields,
                    index, isDelta);
            }
        }

        return rpmObject;
    }

    private void loadTaskAssignment(GenericStep step, RPMObjectScope rpmscope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        TaskAssignment taskAssignment = null;

        String taskAssignmentId = (String) relatedFields.get(index,
            ID_TSK_ASSIGNMENT_ID);

        if (taskAssignmentId != null)
        {
            taskAssignment = new TaskAssignment();
            taskAssignment.setID(taskAssignmentId);

            RPMObjectManager manager = getManagerInstance(TaskAssignment.class);

            taskAssignment = (TaskAssignment) manager.loadByPrimaryKey(
                taskAssignment, null, messageContext, isDelta);
        }

        if (isDelta)
        {
            step.deltaTaskAssignment(taskAssignment);
        }
        else
        {
            step.setTaskAssignment(taskAssignment);
        }

    }

    private void loadStepTimesheets(Step step, StepScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        StepTimesheetManager manager = (StepTimesheetManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(StepTimesheet.class.getName());

        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(step, step
            .getStepTimesheets(), null, messageContext, this, new Object[]
        {
            step.getID()
        }, null, null, isDelta);

        step.setStepTimesheets((StepTimesheet[]) ArrayUtil.listToArray(
            StepTimesheet.class, temp));

    }

    private void loadCreditTimesheet(Credit credit, StepScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        StepTimesheetManager manager = (StepTimesheetManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(StepTimesheet.class.getName());

        CreditTimesheet[] timesheets = null;
        if (credit.getCreditTimesheet() != null)
        {
            timesheets = new CreditTimesheet[]
            {
                credit.getCreditTimesheet()
            };
        }

        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(credit,
            timesheets, null, messageContext, this, new Object[]
            {
                credit.getID()
            }, null, null, isDelta);

        if (temp != null && !temp.isEmpty())
        {
            //Credit can only have one creditTimesheet
            credit.setCreditTimesheet((CreditTimesheet) temp.get(0));
        }

    }

    private void loadWorkElement(GenericStep step, StepScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {

        WorkElement workElement = step.getWorkElement();

        String taskAssignmentId = (String) relatedFields.get(index,
            ID_TSK_ASSIGNMENT_ID);

        String workElementId = TimesheetManagerUtil
            .retrieveTaskAssignment_TaskId(taskAssignmentId, messageContext);

        if (workElement == null)
        {
            workElement = new WorkElement();
            workElement.setID(workElementId);
        }

        RPMObjectManager manager = getManagerInstance(workElement.getClass());

        workElement = (WorkElement) manager.loadByPrimaryKey(workElement, scope
            .getWorkElement(), messageContext, isDelta);

        if (isDelta)
        {
            step.deltaWorkElement(workElement);
        }
        else
        {
            step.setWorkElement(workElement);
        }

    }

    private void loadResource(GenericStep step, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        Resource resource = null;

        String taskAssignmentId = (String) relatedFields.get(index,
            ID_TSK_ASSIGNMENT_ID);

        String resourceId = TimesheetManagerUtil
            .retrieveTaskAssignment_ResourceId(taskAssignmentId, messageContext);

        if (resourceId != null)
        {
            resource = new Resource();
            resource.setID(resourceId);

            RPMObjectManager manager = getManagerInstance(Resource.class);

            resource = (Resource) manager.loadByPrimaryKey(resource, null,
                messageContext, isDelta);
        }

        if (isDelta)
        {
            step.deltaResource(resource);
        }
        else
        {
            step.setResource(resource);
        }

    }

    private void loadProject(GenericStep step, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {

        GenericProject project = null;

        String taskAssignmentId = (String) relatedFields.get(index,
            ID_TSK_ASSIGNMENT_ID);

        String projectId = TimesheetManagerUtil
            .retrieveTaskAssignment_ProjectId(taskAssignmentId, messageContext);

        if (projectId != null)
        {
            project = new GenericProject();
            project.setID(projectId);

            RPMObjectManager manager = getManagerInstance(GenericProject.class);

            project = (GenericProject) manager.loadByPrimaryKey(project, null,
                messageContext, isDelta);
        }

        if (isDelta)
        {
            step.deltaProject(project);
        }
        else
        {
            step.setProject(project);
        }

    }

    private TimeCode loadTimeCode(TimeCodeScope timeCodeScope,
            MessageContext messageContext, String oldTimeCodeID,
            TimeCode previousTimeCode, boolean isDelta) throws RPMException,
            SQLException, ParseException
    {
        TimecodeManager manager = (TimecodeManager) getManagerInstance(TimeCode.class
            .getName());

        TimeCode newTimeCode = null;
        if (isDelta)
        {
            newTimeCode = previousTimeCode;
            TimeCode oldTimeCode = null;
            if (oldTimeCodeID != null)
            {
                oldTimeCode = new TimeCode();
                oldTimeCode.setID(oldTimeCodeID);
                if (newTimeCode != null && newTimeCode.getID() != null
                        && newTimeCode.getID().equals(oldTimeCodeID))
                {
                    oldTimeCode = (TimeCode) manager.loadByPrimaryKey(
                        newTimeCode, timeCodeScope, messageContext, true);
                }
                else
                {
                    //In this case, old timecode is loaded without the
                    // scope to avoid problem with the cache
                    oldTimeCode = (TimeCode) manager.loadByPrimaryKey(
                        oldTimeCode, timeCodeScope, messageContext, false);
                }
            }
            return oldTimeCode;
        }

        if (oldTimeCodeID != null)
        {
            if (newTimeCode == null)
            { //newTimecode not in cache
                newTimeCode = new TimeCode();
                newTimeCode.setID(oldTimeCodeID);
                newTimeCode = (TimeCode) manager.loadByPrimaryKey(newTimeCode,
                    timeCodeScope, messageContext, false);
            }
            return newTimeCode;
        }

        return null;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition condition = null;

        //This is needed by the summaryTimesheetManager even though there is
        //no relation between these objects in the containers.
        if (manager instanceof SummaryTimesheetManager)
        {
            condition = new JoinCondition(TABLE_NAME, extraWhere);
            condition
                .appendEqualQuestionMark(StepManager.NAME_TSK_ASSIGNMENT_ID);
        }
        else
        {

            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return condition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        GenericStep object = (GenericStep) rpmObject;
        object.setID(SP_I_ACT(object, messageContext));

        //set de default values

        if (object instanceof Step)
        {

            Step step = (Step) object;

            if (step.getAlwaysAvailable() == null)
            {
                step.setAlwaysAvailable(Boolean.TRUE);
            }
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        GenericStep object = (GenericStep) rpmObject;
        StepScope scope = (StepScope) rpmScope;

        boolean modified = object.testNameModified()
                || object.testTimeCode1Modified()
                || object.testTimeCode2Modified();

        if (object instanceof Step)
        {
            Step step = (Step) rpmObject;

            saveStepTimesheets(step, scope, messageContext);
            modified = modified || step.testEETCModified()
                    || step.testWeightModified()
                    || step.testAlwaysAvailableModified();
        }
        else if (object instanceof Credit)
        {
            saveCreditTimesheet((Credit) object, scope, messageContext);
        }
        
        if (modified)
        {
            SP_U_ACT_NAME(object, scope, messageContext);
        }
        
        // load the TaskAssignment and recalculate %C
        GenericTaskAssignment genericTaskAssignment = TimesheetManagerUtil
            .reloadAbstractTaskAssignment(object.getTaskAssignment(),
                messageContext);
        if (genericTaskAssignment instanceof TaskAssignment)
        {
            TaskAssignment taskAssignment = (TaskAssignment) genericTaskAssignment;
            TaskAssignmentManager manager = (TaskAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(taskAssignment);
            double percentageComplete = TimesheetManagerUtil
                .calculatePercentageCompleteWithSteps(object, messageContext);
            RPMObjectUtil.deltaAndSet(taskAssignment, ValidationConstants.PERCENTAGE_COMPLETE_FIELD,
                new Double(percentageComplete));
            manager.internalSave(taskAssignment, null, messageContext);
        }

        return rpmObject;
    }

    private void saveStepTimesheets(Step step, StepScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {

        step.setStepTimesheets((StepTimesheet[]) updateChildren(step, step
            .getStepTimesheets(), scope.isTimesheets(), messageContext,
            StepTimesheet.class));
    }

    private void saveCreditTimesheet(Credit credit, StepScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {

        credit.setCreditTimesheet((CreditTimesheet) updateChild(credit, credit
            .getCreditTimesheet(), scope.isTimesheets(), messageContext));
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        GenericStep genericStep = (GenericStep) rpmObject;

        if (rpmObject != null)
            SP_D_ACT(genericStep, messageContext);
    }

    protected boolean isCheckingOutOnCreate()
    {
        // Required to disable checking out/in on create because checkin
        // has a side effect on all other objects of that timesheet for
        // that user.
        return false;
    }
    
    public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        String taskAssignmentId = TimesheetManagerUtil
            .retrieveStep_TaskAssignmentId(rpmObject.getID(), messageContext);

        TimesheetManagerUtil.checkinObject(this, taskAssignmentId,
            messageContext);
    }

    public void undoCheckOut(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        String taskAssignmentId = TimesheetManagerUtil
        .retrieveStep_TaskAssignmentId(rpmObject.getID(), messageContext);
        
        TimesheetManagerUtil.undoCheckOut(this, taskAssignmentId,
            messageContext);
    }

    public void checkoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        String taskAssignmentId = TimesheetManagerUtil
            .retrieveStep_TaskAssignmentId(rpmObject.getID(), messageContext);
        
        TimesheetManagerUtil.checkoutObject(this, taskAssignmentId,
            messageContext);
    }

    public boolean isCheckedOut(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException, SQLException
    {
        String taskAssignmentId = TimesheetManagerUtil
            .retrieveStep_TaskAssignmentId(rpmObject.getID(), messageContext);

        TaskAssignment taskAssignment = new TaskAssignment();
        taskAssignment.setID(taskAssignmentId);

        return TimesheetManagerUtil
            .isCheckedOut(messageContext, taskAssignment);
    }
    
    public String getCheckInCheckOutPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        String taskAssignmentId = TimesheetManagerUtil
            .retrieveStep_TaskAssignmentId(rpmObject.getID(), messageContext);

        return TimesheetManagerUtil.getCheckInCheckOutPrimaryKey(
            taskAssignmentId, messageContext).toString();
    }
    
    private String SP_I_ACT(GenericStep object, MessageContext messageContext)
            throws SQLException, RPMException
    {
        int typeId = 0;
        if (object instanceof Step)
        {
            typeId = Step.TYPE_ID;
        }
        else if (object instanceof Credit)
        {
            typeId = Credit.TYPE_ID;
        }

        String name = "SP_I_ACT";
        Object args[] = new Object[4];
        args[0] = object.getName(); //p_name
        args[1] = object.getTaskAssignment().getID(); //p_tsk_assignment_id
        args[2] = new Integer(typeId); //p_type_id
        args[3] = getUser(messageContext).getID(); //p_rec_user

        return executeProcedure(messageContext, name, args);
    }

    private String SP_D_ACT(GenericStep step, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String weekOf = null;

        if (step instanceof Step)
        {
            StepTimesheet[] stepTimesheets = ((Step) step).getStepTimesheets();
            if (stepTimesheets != null && stepTimesheets[0] != null)
            {
                weekOf = DateUtil
                    .convertToString(stepTimesheets[0].getWeekOf());
            }
        }
        else if (step instanceof Credit)
        {
            CreditTimesheet creditTimesheet = ((Credit) step)
                .getCreditTimesheet();
            if (creditTimesheet != null)
            {
                weekOf = DateUtil.convertToString(creditTimesheet.getWeekOf());
            }
        }

        String name = "SP_D_ACT";
        Object args[] = new Object[4];
        args[0] = step.getID(); //p_id
        args[1] = weekOf; //p_week_of
        args[2] = "X"; //p_del_flag
        args[3] = getUser(messageContext).getID(); //p_rec_user

        return executeProcedure(messageContext, name, args);
    }

    private String SP_U_ACT_NAME(GenericStep object, StepScope scope,
            MessageContext messageContext) throws SQLException, RPMException
    {

        //Timecodes
        String timecode1Id = null;
        String timecode2Id = null;
        TimeCode timecode1 = object.getTimeCode1();
        if (timecode1 != null)
        {
            timecode1Id = timecode1.getID();
        }
        TimeCode timecode2 = object.getTimeCode2();
        if (timecode2 != null)
        {
            timecode2Id = timecode2.getID();
        }

        if (scope == null || !scope.isTimeCodes())
        {
            timecode1Id = TimesheetManagerUtil.retrieveStep_Timecode1Id(object
                .getID(), messageContext);
            timecode2Id = TimesheetManagerUtil.retrieveStep_Timecode2Id(object
                .getID(), messageContext);
        }

        //always reload the timecode names
        String timecode1name = null;
        if (timecode1Id != null)
        {
            timecode1name = TimesheetManagerUtil.retrieveTimecode_Name(
                timecode1Id, messageContext);
        }
        String timecode2name = null;
        if (timecode2Id != null)
        {
            timecode2name = TimesheetManagerUtil.retrieveTimecode_Name(
                timecode2Id, messageContext);
        }

        //Task assignment data
        String taskAssignmentId = null;
        TaskAssignment taskAssignment = object.getTaskAssignment();
        if (taskAssignment != null)
        {
            taskAssignmentId = taskAssignment.getID();
        }
        else
        {
            taskAssignmentId = TimesheetManagerUtil
                .retrieveStep_TaskAssignmentId(object.getID(), messageContext);
            taskAssignment = new TaskAssignment();
            taskAssignment.setID(taskAssignmentId);
        }
        
        //always reload taskAssignment to get the latest values
        taskAssignment = (TaskAssignment) TimesheetManagerUtil
            .reloadAbstractTaskAssignment(taskAssignment, messageContext);
        object.setTaskAssignment(taskAssignment);
               
        AbstractTimesheet[] timesheets = null;
        if (object instanceof Step)
        {
            timesheets = ((Step) object).getStepTimesheets();
        }
        else if (object instanceof Credit)
        {
            CreditTimesheet timesheet = ((Credit) object).getCreditTimesheet();
            if (timesheet != null)
            {
                timesheets = new AbstractTimesheet[]
                {
                    timesheet
                };
            }
        }
        initTimesheetsWorkingHours(timesheets);

        double totalHoursWorked = calculateHoursWorked(timesheets);
        double expectedWork = 0.0;
        double eetc = 0.0;
        double percentageCompleted = 0.0;
        double weight = 0.0;
        String etc_modified = "Y";
        String alwaysAvailable = null;

        if (object instanceof Step)
        {
            Step step = (Step) object;
            
            if (taskAssignment.getExpectedWork() != null)
            {
                expectedWork = taskAssignment.getExpectedWork().doubleValue();
            }
            
			//double workTodate = 0;
			//if (taskAssignment.getActualWork() != null)
			//{
			//    workTodate = taskAssignment.getActualWork().doubleValue();
			//}

            if (step.getEETC() != null)
            {
                eetc = step.getEETC().doubleValue();
            }
            
            if (totalHoursWorked + eetc != 0.0)
            {
                percentageCompleted = totalHoursWorked * 100.0 / (totalHoursWorked + eetc);
            }

            if (step.getWeight() != null)
            {
                weight = step.getWeight().doubleValue();
            }

            if ((step.getEETC() == null || step.getEETC().doubleValue() == 0.0)
                    && step.getETCChanged() != null
                    && !step.getETCChanged().booleanValue())
                etc_modified = "N";

            alwaysAvailable = "N";
            if (step.getAlwaysAvailable() != null
                    && step.getAlwaysAvailable().booleanValue())
                alwaysAvailable = "Y";
        }

        String name = "SP_U_ACT_NAME";
        Object args[] = new Object[14];
        args[0] = object.getID(); //p_id
        args[1] = taskAssignmentId; //p_tsk_assignment_id
        args[2] = object.getName(); //p_name
        args[3] = timecode1Id; //p_timecode_1_id
        args[4] = timecode1name; //p_timecode_1_name
        args[5] = timecode2Id; //p_timecode_2_id
        args[6] = timecode2name; //p_timecode_2_name
        args[7] = DateUtil.convertHoursToMinutes(new Double(expectedWork)); //p_sched_work
        args[8] = DateUtil.convertHoursToMinutes(new Double(eetc)); //p_etc
        args[9] = TimesheetManagerUtil.toRoundedPerTenThousand(percentageCompleted); //p_milestone_percent
        args[10] = new Integer((int) ((weight * 100.0) + 0.5)); //p_edpercent
        args[11] = etc_modified; //p_etc_modified
        args[12] = alwaysAvailable; //p_show_step
        args[13] = getUser(messageContext).getID(); //p_rec_user

        return executeProcedure(messageContext, name, args);
    }

    private double calculateHoursWorked(AbstractTimesheet[] timesheets)
    {
        double totalHours = 0;

        if (timesheets == null)
            return 0;

        for (int i = 0; i < timesheets.length; i++)
        {
            totalHours += TimesheetManagerUtil
                .calculateTotalBillable(timesheets[i]);
        }

        return totalHours;
    }

    private void initTimesheetsWorkingHours(AbstractTimesheet[] timesheets)
    {
        if (timesheets == null)
            return;

        for (int i = 0; i < timesheets.length; i++)
        {
            timesheets[i] = TimesheetManagerUtil
                .initWorkingHours(timesheets[i]);
        }
    }

    private void loadTimeCodes(GenericStep step, StepScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        if (!scope.isTimeCodes())
            return;

        String timeCode1Id = (String) relatedFields
            .get(index, ID_TIMECODE_1_ID);
        TimeCode oldTimeCode1 = step.getTimeCode1();
        TimeCode loadedTimeCode1 = loadTimeCode(null, messageContext,
            timeCode1Id, oldTimeCode1, isDelta);

        String timeCode2Id = (String) relatedFields
            .get(index, ID_TIMECODE_2_ID);
        TimeCode oldTimeCode2 = step.getTimeCode2();
        TimeCode loadedTimeCode2 = loadTimeCode(null, messageContext,
            timeCode2Id, oldTimeCode2, isDelta);

        if (isDelta)
        {
            step.deltaTimeCode1(loadedTimeCode1);
            step.deltaTimeCode2(loadedTimeCode2);
        }
        else
        {
            step.setTimeCode1(loadedTimeCode1);
            step.setTimeCode2(loadedTimeCode2);
        }
    }
}
