package com.ibm.rpm.timesheet.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.RPMObjectUtil;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.AbstractTimesheet;
import com.ibm.rpm.timesheet.containers.Credit;
import com.ibm.rpm.timesheet.containers.CreditTimesheet;
import com.ibm.rpm.timesheet.containers.GenericStep;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.util.TimesheetManagerUtil;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.xpathparser.XPathContainer;

public class StepTimesheetManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(StepTimesheet.class.getName());
        CONTAINERS.add(CreditTimesheet.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                StepTimesheetManager.class);
        }
    }

    /**
     * Column ACTIVITY_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ACTIVITY_ID = 1;

    public static final int TYPE_ACTIVITY_ID = Types.CHAR;

    public static final String NAME_ACTIVITY_ID = "DETAIL_TIMESHEETS.ACTIVITY_ID";

    public static final String PROPERTY_ACTIVITY_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ACTIVITY_ID, NAME_ACTIVITY_ID);
    }

    /**
     * Column WEEK_OF of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_WEEK_OF = 2;

    public static final int TYPE_WEEK_OF = Types.DATE;

    public static final String NAME_WEEK_OF = "DETAIL_TIMESHEETS.WEEK_OF";

    public static final String PROPERTY_WEEK_OF = "WEEKOF";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEEK_OF, NAME_WEEK_OF);
    }

    /**
     * Column MONDAY_REGULAR of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MONDAY_REGULAR = 3;

    public static final int TYPE_MONDAY_REGULAR = Types.SMALLINT;

    public static final String NAME_MONDAY_REGULAR = "DETAIL_TIMESHEETS.MONDAY_REGULAR";

    public static final String PROPERTY_MONDAY_REGULAR = "REGULARMONDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONDAY_REGULAR, NAME_MONDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column MONDAY_SPECIAL of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MONDAY_SPECIAL = 4;

    public static final int TYPE_MONDAY_SPECIAL = Types.SMALLINT;

    public static final String NAME_MONDAY_SPECIAL = "DETAIL_TIMESHEETS.MONDAY_SPECIAL";

    public static final String PROPERTY_MONDAY_SPECIAL = "SPECIALMONDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MONDAY_SPECIAL, NAME_MONDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column TUESDAY_REGULAR of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_TUESDAY_REGULAR = 5;

    public static final int TYPE_TUESDAY_REGULAR = Types.SMALLINT;

    public static final String NAME_TUESDAY_REGULAR = "DETAIL_TIMESHEETS.TUESDAY_REGULAR";

    public static final String PROPERTY_TUESDAY_REGULAR = "REGULARTUESDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TUESDAY_REGULAR, NAME_TUESDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column TUESDAY_SPECIAL of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_TUESDAY_SPECIAL = 6;

    public static final int TYPE_TUESDAY_SPECIAL = Types.SMALLINT;

    public static final String NAME_TUESDAY_SPECIAL = "DETAIL_TIMESHEETS.TUESDAY_SPECIAL";

    public static final String PROPERTY_TUESDAY_SPECIAL = "SPECIALTUESDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_TUESDAY_SPECIAL, NAME_TUESDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column WEDNESDAY_REGULAR of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_WEDNESDAY_REGULAR = 7;

    public static final int TYPE_WEDNESDAY_REGULAR = Types.SMALLINT;

    public static final String NAME_WEDNESDAY_REGULAR = "DETAIL_TIMESHEETS.WEDNESDAY_REGULAR";

    public static final String PROPERTY_WEDNESDAY_REGULAR = "REGULARWEDNESDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEDNESDAY_REGULAR, NAME_WEDNESDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column WEDNESDAY_SPECIAL of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_WEDNESDAY_SPECIAL = 8;

    public static final int TYPE_WEDNESDAY_SPECIAL = Types.SMALLINT;

    public static final String NAME_WEDNESDAY_SPECIAL = "DETAIL_TIMESHEETS.WEDNESDAY_SPECIAL";

    public static final String PROPERTY_WEDNESDAY_SPECIAL = "SPECIALWEDNESDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_WEDNESDAY_SPECIAL, NAME_WEDNESDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column THURSDAY_REGULAR of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_THURSDAY_REGULAR = 9;

    public static final int TYPE_THURSDAY_REGULAR = Types.SMALLINT;

    public static final String NAME_THURSDAY_REGULAR = "DETAIL_TIMESHEETS.THURSDAY_REGULAR";

    public static final String PROPERTY_THURSDAY_REGULAR = "REGULARTHURSDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_THURSDAY_REGULAR, NAME_THURSDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column THURSDAY_SPECIAL of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_THURSDAY_SPECIAL = 10;

    public static final int TYPE_THURSDAY_SPECIAL = Types.SMALLINT;

    public static final String NAME_THURSDAY_SPECIAL = "DETAIL_TIMESHEETS.THURSDAY_SPECIAL";

    public static final String PROPERTY_THURSDAY_SPECIAL = "SPECIALTHURSDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_THURSDAY_SPECIAL, NAME_THURSDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column FRIDAY_REGULAR of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_FRIDAY_REGULAR = 11;

    public static final int TYPE_FRIDAY_REGULAR = Types.SMALLINT;

    public static final String NAME_FRIDAY_REGULAR = "DETAIL_TIMESHEETS.FRIDAY_REGULAR";

    public static final String PROPERTY_FRIDAY_REGULAR = "REGULARFRIDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FRIDAY_REGULAR, NAME_FRIDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column FRIDAY_SPECIAL of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_FRIDAY_SPECIAL = 12;

    public static final int TYPE_FRIDAY_SPECIAL = Types.SMALLINT;

    public static final String NAME_FRIDAY_SPECIAL = "DETAIL_TIMESHEETS.FRIDAY_SPECIAL";

    public static final String PROPERTY_FRIDAY_SPECIAL = "SPECIALFRIDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FRIDAY_SPECIAL, NAME_FRIDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column SATURDAY_REGULAR of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SATURDAY_REGULAR = 13;

    public static final int TYPE_SATURDAY_REGULAR = Types.SMALLINT;

    public static final String NAME_SATURDAY_REGULAR = "DETAIL_TIMESHEETS.SATURDAY_REGULAR";

    public static final String PROPERTY_SATURDAY_REGULAR = "REGULARSATURDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SATURDAY_REGULAR, NAME_SATURDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column SATURDAY_SPECIAL of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SATURDAY_SPECIAL = 14;

    public static final int TYPE_SATURDAY_SPECIAL = Types.SMALLINT;

    public static final String NAME_SATURDAY_SPECIAL = "DETAIL_TIMESHEETS.SATURDAY_SPECIAL";

    public static final String PROPERTY_SATURDAY_SPECIAL = "SPECIALSATURDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SATURDAY_SPECIAL, NAME_SATURDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column SUNDAY_REGULAR of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SUNDAY_REGULAR = 15;

    public static final int TYPE_SUNDAY_REGULAR = Types.SMALLINT;

    public static final String NAME_SUNDAY_REGULAR = "DETAIL_TIMESHEETS.SUNDAY_REGULAR";

    public static final String PROPERTY_SUNDAY_REGULAR = "REGULARSUNDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SUNDAY_REGULAR, NAME_SUNDAY_REGULAR
                + " /60.0 ");
    }

    /**
     * Column SUNDAY_SPECIAL of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SUNDAY_SPECIAL = 16;

    public static final int TYPE_SUNDAY_SPECIAL = Types.SMALLINT;

    public static final String NAME_SUNDAY_SPECIAL = "DETAIL_TIMESHEETS.SUNDAY_SPECIAL";

    public static final String PROPERTY_SUNDAY_SPECIAL = "SPECIALSUNDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_SUNDAY_SPECIAL, NAME_SUNDAY_SPECIAL
                + " /60.0 ");
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 17;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "DETAIL_TIMESHEETS.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 18;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "DETAIL_TIMESHEETS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 19;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "DETAIL_TIMESHEETS.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column NOTES of type Types.VARCHAR mapped to String.
     */
    public static final int ID_NOTES = 20;

    public static final int TYPE_NOTES = Types.VARCHAR;

    public static final String NAME_NOTES = "DETAIL_TIMESHEETS.NOTES";

    public static final String PROPERTY_NOTES = "SUMMARYNOTES";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_NOTES, NAME_NOTES);
    }

    /**
     * Column PDA_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_PDA_FLAG = 21;

    public static final int TYPE_PDA_FLAG = Types.CHAR;

    public static final String NAME_PDA_FLAG = "DETAIL_TIMESHEETS.PDA_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PDA_FLAG = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PDA_FLAG,NAME_PDA_FLAG);}

    /**
     * Column BILLABLE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BILLABLE = 22;

    public static final int TYPE_BILLABLE = Types.SMALLINT;

    public static final String NAME_BILLABLE = "DETAIL_TIMESHEETS.BILLABLE";

    public static final String PROPERTY_BILLABLE = "BILLABLE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_BILLABLE, SqlUtil.getBooleanValue(
            NAME_BILLABLE, 1));
    }

    /**
     * Column CREDIT_WEEK_OF of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_CREDIT_WEEK_OF = 23;

    public static final int TYPE_CREDIT_WEEK_OF = Types.DATE;

    public static final String NAME_CREDIT_WEEK_OF = "DETAIL_TIMESHEETS.CREDIT_WEEK_OF";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREDIT_WEEK_OF = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_CREDIT_WEEK_OF,NAME_CREDIT_WEEK_OF);}

    public static final String TABLE_NAME = "DETAIL_TIMESHEETS";

    /**
     * Create an array of type string containing all the fields of the
     * DETAIL_TIMESHEETS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ACTIVITY_ID, NAME_WEEK_OF, NAME_MONDAY_REGULAR,
            NAME_MONDAY_SPECIAL, NAME_TUESDAY_REGULAR, NAME_TUESDAY_SPECIAL,
            NAME_WEDNESDAY_REGULAR, NAME_WEDNESDAY_SPECIAL,
            NAME_THURSDAY_REGULAR, NAME_THURSDAY_SPECIAL, NAME_FRIDAY_REGULAR,
            NAME_FRIDAY_SPECIAL, NAME_SATURDAY_REGULAR, NAME_SATURDAY_SPECIAL,
            NAME_SUNDAY_REGULAR, NAME_SUNDAY_SPECIAL, NAME_REC_USER,
            NAME_REC_STATUS, NAME_REC_DATETIME, NAME_NOTES, NAME_PDA_FLAG,
            NAME_BILLABLE, NAME_CREDIT_WEEK_OF
    };

    private static final String ALL_FIELDS;


    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new TimesheetManager instance.
     */
    public StepTimesheetManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return null;
    }

    public String[] getPrimaryKeyArray(RPMObject rpmObject)
    {
        return new String[]
        {
                NAME_ACTIVITY_ID, NAME_WEEK_OF
        };
    }

    public Object[] getPrimaryKeyValues(RPMObject rpmObject)
    {
        AbstractTimesheet timesheet = (AbstractTimesheet) rpmObject;
        String stepId = null;
        RPMObject parent = timesheet.getParent();
        if (parent != null && parent instanceof Step
                || parent instanceof Credit)
        {
            stepId = parent.getID();
        }
        String weekOf = DateUtil.convertToString(timesheet.getWeekOf());

        return new Object[]
        {
                stepId, weekOf
        };
    }

    protected RPMObject setPrimaryKeyFields(Object[] primaryKey,
            RPMObject rpmObject) throws RPMException
    {
        AbstractTimesheet timesheet = (AbstractTimesheet) rpmObject;
        // Set the timesheet's ID to the step's ID
        timesheet.setID((String) primaryKey[0]);

        GenericStep genericStep = null;
        if (timesheet instanceof StepTimesheet)
        {
            genericStep = new Step();
        }
        else if (timesheet instanceof CreditTimesheet)
        {
            genericStep = new Credit();
        }     
        
        genericStep.setID((String) primaryKey[0]);
        timesheet.setParent(genericStep);

        // Set week-of
        Calendar weekOf = null;
        try
        {
            weekOf = DateUtil
                .convertToCalendarNoTime((String) primaryKey[1]);
        }
        catch (ParseException e)
        {
            ExceptionUtil.handleException(Manager.class, e);
        }
        timesheet.setWeekOf(weekOf);

        return timesheet;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public String getFilter()
    {
        return NAME_REC_STATUS + " != 'D' ";
    }
    
    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
        }

        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    


    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager instanceof StepManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.appendSubSelect(StepManager.NAME_ELEMENT_ID,
                NAME_ACTIVITY_ID, TABLE_NAME, getFilter());
        }
        else
        {

            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        // Container type doesn't apply to this manager
        return new Integer(0);
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {

        RPMObject object = null;

        String stepId = resultSet.getString(ID_ACTIVITY_ID);
        int parentTypeid = TimesheetManagerUtil.retrieveStep_TypeId(stepId,
            messageContext);

        switch (parentTypeid)
        {
        case Step.TYPE_ID:
            object = new StepTimesheet();
            break;
        case Credit.TYPE_ID:
            object = new CreditTimesheet();
            break;
        default:
            ExceptionUtil.handleUnsupportedTypeId(this, parentTypeid);
        }

        return object;

    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        AbstractTimesheet timesheet = (AbstractTimesheet) rpmObject;

        //weekOf is part of the primaryKey
        timesheet.setWeekOf(Manager.getCalendar(rs, ID_WEEK_OF));

        // get billable value
        Boolean billable = decodeBillable(rs.getShort(ID_BILLABLE));
        
        if (isDelta)
        {
            timesheet.deltaRegularFriday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_FRIDAY_REGULAR)));
            timesheet.deltaRegularMonday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_MONDAY_REGULAR)));
            timesheet.deltaRegularSaturday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_SATURDAY_REGULAR)));
            timesheet.deltaRegularSunday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_SUNDAY_REGULAR)));
            timesheet.deltaRegularThursday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_THURSDAY_REGULAR)));
            timesheet.deltaRegularTuesday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_TUESDAY_REGULAR)));
            timesheet.deltaRegularWednesday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_WEDNESDAY_REGULAR)));
            timesheet.deltaSpecialFriday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_FRIDAY_SPECIAL)));
            timesheet.deltaSpecialMonday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_MONDAY_SPECIAL)));
            timesheet.deltaSpecialSaturday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_SATURDAY_SPECIAL)));
            timesheet.deltaSpecialSunday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_SUNDAY_SPECIAL)));
            timesheet.deltaSpecialThursday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_THURSDAY_SPECIAL)));
            timesheet.deltaSpecialTuesday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_TUESDAY_SPECIAL)));
            timesheet.deltaSpecialWednesday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_WEDNESDAY_SPECIAL)));
            timesheet.deltaSummaryNotes(rs.getString(ID_NOTES));
        }
        else
        {
            timesheet.setRegularFriday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_FRIDAY_REGULAR)));
            timesheet.setRegularMonday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_MONDAY_REGULAR)));
            timesheet.setRegularSaturday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_SATURDAY_REGULAR)));
            timesheet.setRegularSunday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_SUNDAY_REGULAR)));
            timesheet.setRegularThursday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_THURSDAY_REGULAR)));
            timesheet.setRegularTuesday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_TUESDAY_REGULAR)));
            timesheet.setRegularWednesday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_WEDNESDAY_REGULAR)));
            timesheet.setSpecialFriday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_FRIDAY_SPECIAL)));
            timesheet.setSpecialMonday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_MONDAY_SPECIAL)));
            timesheet.setSpecialSaturday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_SATURDAY_SPECIAL)));
            timesheet.setSpecialSunday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_SUNDAY_SPECIAL)));
            timesheet.setSpecialThursday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_THURSDAY_SPECIAL)));
            timesheet.setSpecialTuesday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_TUESDAY_SPECIAL)));
            timesheet.setSpecialWednesday(DateUtil.convertMinutesToHours(rs
                .getInt(ID_WEDNESDAY_SPECIAL)));
            timesheet.setSummaryNotes(rs.getString(ID_NOTES));
        }
        
        if (timesheet instanceof StepTimesheet)
        {

            StepTimesheet st = (StepTimesheet) timesheet;

            if (isDelta)
            {
                st.deltaBillable(billable);
            }
            else
            {
                st.setBillable(billable);
            }
        }
        
        if (timesheet instanceof CreditTimesheet)
        {

            CreditTimesheet ct = (CreditTimesheet) timesheet;

            if (isDelta)
            {
                ct
                    .deltaCreditWeekOf(Manager.getCalendar(rs,
                        ID_CREDIT_WEEK_OF));
            }
            else
            {
                ct.setCreditWeekOf(Manager.getCalendar(rs, ID_CREDIT_WEEK_OF));
            }

        }

        return rpmObject;
    }

    public Boolean decodeBillable(short billable_value) throws RPMException
    {
        // get billable value
        boolean billable = false;

        if (billable_value == 0)
            billable = false;
        else if (billable_value == 1)
            billable = true;
        else
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(getClass().getName()),
                    "of field billable ", "true or false"
            };
            RPMException e = new RPMException(ErrorCodes.INVALID_VALUE,
                errorParams, getClass().getName(), NAME_BILLABLE);
            e.setSeverity(SeverityLevel.SevereError);
            throw e;
        }
        return new Boolean(billable);
    }
    
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String stepId = resultSet.getString(ID_ACTIVITY_ID);
        relatedFields.put(index, ID_ACTIVITY_ID, stepId);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        // Always load the parent, event if it's not in the scope
        loadParent((AbstractTimesheet) rpmObject, rpmScope, messageContext,
            relatedFields, index, isDelta);

        return rpmObject;
    }

    private void loadParent(AbstractTimesheet timesheet, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {

        RPMObjectScope parentScope = null;
        if (rpmScope != null)
        {
            parentScope = rpmScope.getParent();
        }

        GenericStep parent = (GenericStep) timesheet.getParent();

        String stepId = (String) relatedFields.get(index, ID_ACTIVITY_ID);

        if (parent == null)
        {

            parent = new GenericStep();
            parent.setID(stepId);
        }

        StepManager stepManager = (StepManager) getManagerInstance(Step.class
            .getName());

        parent = (GenericStep) stepManager.loadByPrimaryKey(parent,
            parentScope, messageContext, false);

        //note: we are not using the delta method because step
        //cannot be changed (it is part of the
        //primaryKey).

        timesheet.setParent(parent);

    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof StepManager)
        {
            joinCondition = new JoinCondition();
            StringBuffer where = new StringBuffer();
            where.append(StepTimesheetManager.NAME_ACTIVITY_ID);
            where.append(" = ?");

            if (extraWhere != null && extraWhere.length() > 0)
            {
                where.append(joinCondition.getCondition());
                where.append(" AND ");
                where.append(extraWhere);
            }

            joinCondition.setCondition(where.toString());

            joinCondition.setTableName(getTableName());
        } 
        //This is needed by the summaryTimesheetManager even though there is
        //no relation between these objects in the containers.
        else if (manager instanceof SummaryTimesheetManager)
        {

            SqlBuffer where = new SqlBuffer();
            where.appendEqualQuestionMark(StepManager.NAME_TSK_ASSIGNMENT_ID);

            joinCondition = new JoinCondition(TABLE_NAME, extraWhere);
            joinCondition.appendSubSelect(NAME_ACTIVITY_ID,
                StepManager.NAME_ELEMENT_ID, StepManager.TABLE_NAME, where
                    .toString());
            joinCondition.appendEqualQuestionMark(NAME_WEEK_OF);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        AbstractTimesheet timesheet = (AbstractTimesheet) rpmObject;

        // Retrieve the start of the week
        TimesheetManagerUtil.setValidWeekOf(timesheet, messageContext);

        if (timesheet instanceof CreditTimesheet)
        {
            Calendar creditWeekOf = TimesheetManagerUtil.SP_S_WEEKOF(((CreditTimesheet) timesheet)
                .getCreditWeekOf(), messageContext);
            ((CreditTimesheet) timesheet).setCreditWeekOf(creditWeekOf);
        }
        
        timesheet = TimesheetManagerUtil.initWorkingHours(timesheet);
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        AbstractTimesheet timesheet = (AbstractTimesheet) rpmObject;
        boolean workingHoursChanged = TimesheetManagerUtil
            .hasNewActualReported(timesheet);

        if (workingHoursChanged || timesheet.testWeekOfModified()
                || timesheet.testSummaryNotesModified())
        {
            timesheet.setID(SP_U_TS_DETAIL(timesheet, messageContext));
          
            if (workingHoursChanged)
            {
                timesheet = TimesheetManagerUtil.initWorkingHours(timesheet);

                // update the corresponding SummaryTimesheet
                SummaryTimesheetManager summaryTimesheetManager = (SummaryTimesheetManager) getManagerInstance(SummaryTimesheet.class);

                summaryTimesheetManager.updateSummaryTimesheet(timesheet,
                    messageContext);
                
                // update Step EETC
                if (timesheet.getParent() instanceof Step)
                {
                    Step step = (Step) timesheet.getParent();
                    if (step.getEETC() != null && step.getEETC().doubleValue() != 0)
                    {
                        double eetc = step.getEETC().doubleValue()
                                - TimesheetManagerUtil
                                    .calculateTotalBillable(timesheet);
                        if (eetc < 0)
                        {
                            eetc = 0;
                        }
                        RPMObjectUtil.deltaAndSet(step, ValidationConstants.EETC_FIELD,
                            new Double(eetc));
                    }
                }
            }
        }

        return timesheet;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

    private String SP_U_TS_DETAIL(AbstractTimesheet timesheet,
            MessageContext messageContext) throws SQLException, RPMException
    {
        GenericStep step = (GenericStep) timesheet.getParent();
        String taskAssignmentId = null;
        TaskAssignment taskAssignment = step.getTaskAssignment();
        if (taskAssignment != null)
        {
            taskAssignmentId = taskAssignment.getID();
        }
        else
        {
            taskAssignmentId = TimesheetManagerUtil
                .retrieveStep_TaskAssignmentId(step.getID(), messageContext);
        }
        
        int billable = TimesheetManagerUtil.retrieveTaskAssignment_Billable(
            taskAssignmentId, messageContext);
              
        String creditWeekOf = null;        
        if (timesheet instanceof CreditTimesheet)
        {
            creditWeekOf = DateUtil
                .convertToString(((CreditTimesheet) timesheet)
                    .getCreditWeekOf());
        }

        String name = "SP_U_TS_DETAIL";
        Object args[] = new Object[22];
        args[0] = step.getID(); //p_id
        args[1] = taskAssignmentId; //p_tsk_assignment_id
        args[2] = new Integer(billable); //p_billable
        args[3] = DateUtil.convertToString(timesheet.getWeekOf()); //p_week_of
        args[4] = DateUtil.convertHoursToMinutes(timesheet.getRegularMonday()); //p_monday_regular
        args[5] = DateUtil.convertHoursToMinutes(timesheet.getSpecialMonday()); //p_monday_special
        args[6] = DateUtil.convertHoursToMinutes(timesheet.getRegularTuesday()); //p_tuesday_regular
        args[7] = DateUtil.convertHoursToMinutes(timesheet.getSpecialTuesday()); //p_tuesday_special
        args[8] = DateUtil.convertHoursToMinutes(timesheet
            .getRegularWednesday()); //p_wednesday_regular
        args[9] = DateUtil.convertHoursToMinutes(timesheet
            .getSpecialWednesday()); //p_wednesday_special
        args[10] = DateUtil.convertHoursToMinutes(timesheet
            .getRegularThursday()); //p_thursday_regular
        args[11] = DateUtil.convertHoursToMinutes(timesheet
            .getSpecialThursday()); //p_thursday_special
        args[12] = DateUtil.convertHoursToMinutes(timesheet.getRegularFriday()); //p_friday_regular
        args[13] = DateUtil.convertHoursToMinutes(timesheet.getSpecialFriday()); //p_friday_special
        args[14] = DateUtil.convertHoursToMinutes(timesheet
            .getRegularSaturday()); //p_saturday_regular
        args[15] = DateUtil.convertHoursToMinutes(timesheet
            .getSpecialSaturday()); //p_saturday_special
        args[16] = DateUtil.convertHoursToMinutes(timesheet.getRegularSunday()); //p_sunday_regular
        args[17] = DateUtil.convertHoursToMinutes(timesheet.getSpecialSunday()); //p_sunday_special
        args[18] = null; //p_pda_flag
        args[19] = timesheet.getSummaryNotes(); //p_notes
        args[20] = creditWeekOf; //p_credit_week_of
        args[21] = getUser(messageContext).getID(); //p_rec_user

        return executeProcedure(messageContext, name, args);
    }
}
