package com.ibm.rpm.timesheet.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ArrayUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.timesheet.containers.AbstractTimesheetElement;
import com.ibm.rpm.timesheet.containers.AdministrativeProject;
import com.ibm.rpm.timesheet.containers.AdministrativeTask;
import com.ibm.rpm.timesheet.containers.GenericTimesheetElement;
import com.ibm.rpm.timesheet.containers.PersonalProject;
import com.ibm.rpm.timesheet.containers.PersonalTask;
import com.ibm.rpm.timesheet.containers.PersonalTaskAssignment;
import com.ibm.rpm.timesheet.scope.AdministrativeProjectScope;
import com.ibm.rpm.timesheet.scope.PersonalProjectScope;
import com.ibm.rpm.timesheet.util.TimesheetManagerUtil;

public class TimesheetTaskManager extends AbstractRPMObjectManager
{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        TimesheetTaskManager.class);
    static
    {
        containerMap.add(PersonalTask.class);
        containerMap.add(PersonalProject.class);
        containerMap.add(AdministrativeTask.class);
        containerMap.add(AdministrativeProject.class);
        containerMap.add(GenericTimesheetElement.class);
    }

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 1;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "TMT_WBS.PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 2;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_WBS.PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 3;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_WBS.ELEMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 4;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_WBS.TYPE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 5;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_WBS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column CREATED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_CREATED_BY = 6;

    public static final int TYPE_CREATED_BY = Types.CHAR;

    public static final String NAME_CREATED_BY = "TMT_WBS.CREATED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREATED_BY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CREATED_BY,NAME_CREATED_BY);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 7;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_WBS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column WORK_TODATE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_WORK_TODATE = 8;

    public static final int TYPE_WORK_TODATE = Types.BIGINT;

    public static final String NAME_WORK_TODATE = "TMT_WBS.WORK_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WORK_TODATE = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_WORK_TODATE,NAME_WORK_TODATE); }

    /**
     * Column ETC of type Types.BIGINT mapped to Long.
     */
    public static final int ID_ETC = 9;

    public static final int TYPE_ETC = Types.BIGINT;

    public static final String NAME_ETC = "TMT_WBS.ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ETC = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ETC,NAME_ETC); }

    /**
     * Column FINISH_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FINISH_DT = 10;

    public static final int TYPE_FINISH_DT = Types.TIMESTAMP;

    public static final String NAME_FINISH_DT = "TMT_WBS.FINISH_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FINISH_DT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_FINISH_DT,NAME_FINISH_DT); }

    /**
     * Column WORK_PERCENT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_WORK_PERCENT = 11;

    public static final int TYPE_WORK_PERCENT = Types.SMALLINT;

    public static final String NAME_WORK_PERCENT = "TMT_WBS.WORK_PERCENT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WORK_PERCENT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_WORK_PERCENT,NAME_WORK_PERCENT); }

    /**
     * Column START_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_START_DT = 12;

    public static final int TYPE_START_DT = Types.TIMESTAMP;

    public static final String NAME_START_DT = "TMT_WBS.START_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_START_DT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_START_DT,NAME_START_DT); }

    public static final String TABLE_NAME = "TMT_WBS";

    /**
     * Create an array of type string containing all the fields of the TMT_WBS
     * table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_PROJECT_ID, NAME_PARENT_ID, NAME_ELEMENT_ID, NAME_TYPE_ID,
            NAME_ELEMENT_NAME, NAME_CREATED_BY, NAME_REC_STATUS,
            NAME_WORK_TODATE, NAME_ETC, NAME_FINISH_DT, NAME_WORK_PERCENT,
            NAME_START_DT
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * 
     */
    public TimesheetTaskManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return containerMap.createContainer(typeId);
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {

        //Only PersonalTasks can be created
        if (rpmObject instanceof PersonalTask)
        {
            rpmObject.setID(SP_I_SUMMARY_TS((PersonalTask) rpmObject,
                messageContext));
        }

    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        GenericTimesheetElement temp = (GenericTimesheetElement) rpmObject;
        temp.setID(rs.getString(ID_ELEMENT_ID));
        String name = rs.getString(ID_ELEMENT_NAME);

        double actual = rs.getDouble(ID_WORK_TODATE);
        double EETC = rs.getDouble(ID_ETC) / 100.0;

        int perc = rs.getInt(ID_WORK_PERCENT);
        double percentageCompleted = perc / 100.0;
        double percentageWork = actual / (EETC + actual);

        Calendar finishDate = Manager.getCalendar(rs, ID_FINISH_DT);
        Calendar startDate = Manager.getCalendar(rs, ID_START_DT);

        if (name != null)
        {
            name = name.trim();
        }
        if (isDelta)
        {
            temp.deltaName(name);
            temp.deltaActual(new Double(actual));
            temp.deltaEETC(new Double(EETC));
            temp.deltaFinishDate(finishDate);
            temp.deltaPercentageCompleted(new Double(percentageCompleted));
            temp.deltaPercentageWork(new Double(percentageWork));
            temp.deltaStartDate(startDate);

        }
        else
        {
            temp.setName(name);
            temp.setActual(new Double(actual));
            temp.setEETC(new Double(EETC));
            temp.setFinishDate(finishDate);
            temp.setPercentageCompleted(new Double(percentageCompleted));
            temp.setPercentageWork(new Double(percentageWork));
            temp.setStartDate(startDate);

        }
        return temp;

    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        //Only PersonalTasks can be deleted
        if (rpmObject instanceof PersonalTask)
        {
            SP_D_SUMMARY_TS((PersonalTask) rpmObject, messageContext);
        }

    }

    public String getFilter()
    {
        SqlBuffer sql = new SqlBuffer();
        sql.appendIsNotNull(NAME_ELEMENT_ID);
        sql.appendCondition(NAME_REC_STATUS + "!='D'");
        return sql.toString();
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        //PersonalProject
        if (manager instanceof ResourceManager)
        {
            joinCondition = new JoinCondition(getTableName());
            joinCondition.appendEqual(NAME_TYPE_ID, PersonalProject.TYPE_ID);
            joinCondition.appendEqualQuestionMark(NAME_CREATED_BY);
        }
        else if (manager instanceof TimesheetTaskManager)
        {
            joinCondition = new JoinCondition(getTableName(), getFilter());
            joinCondition.appendEqualQuestionMark(NAME_TYPE_ID);
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID,
                TABLE_NAME, getFilter());
            joinCondition.appendEqualQuestionMark(NAME_ELEMENT_ID);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#getJoinCondition(com.ibm.rpm.framework.RPMObjectManager,
     *      java.lang.String, int, java.lang.String, java.lang.String)
     */
    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());
            int typeId = getTypeId(containerName);
            if (typeId != -1)
            {
                StringBuffer where = new StringBuffer();
                where.append(NAME_TYPE_ID);
                where.append(" = ");
                where.append(typeId);
                if (condition != null)
                {
                    where.append(" AND ");
                    where.append(condition);
                }
                where.append(" AND ");
                where.append(getFilter());
                joinCondition.setCondition(where.toString());
            }
            else
            {
                if (condition != null)
                {
                    joinCondition.setCondition(condition);
                }
            }
        }
        else if (manager instanceof AbstractTimesheetTaskAssignmentManager)
        {           
            joinCondition = new JoinCondition();
            joinCondition.setCondition("");//added to avoid appending it to a null later
            StringBuffer where = new StringBuffer();
            where.append(getPrimaryKey());
            where.append(" IN (");
            where.append("SELECT ");
            where.append(AbstractTimesheetTaskAssignmentManager.NAME_TASK_ID);
            where.append(" FROM ");
            where.append(manager.getTableName());
            where.append(" WHERE ");
            where.append(manager.getPrimaryKey());
            where.append(" = ?)");
            joinCondition.setCondition(joinCondition.getCondition()
                    + where.toString());
        }

        return joinCondition;
    }

    /**
     * @param containerName
     * @return
     */
    private int getTypeId(String containerName)
    {
        if (PersonalProject.class.getName().endsWith(containerName))
        {
            return PersonalProject.TYPE_ID;    
        }
        else if (PersonalTask.class.getName().endsWith(containerName))
        {
            return PersonalTask.TYPE_ID;
        }
        else if (AdministrativeTask.class.getName().endsWith(containerName))
        {
            return AdministrativeTask.TYPE_ID;
        }
        else if (AdministrativeProject.class.getName().endsWith(containerName))
        {
            return AdministrativeProject.TYPE_ID;
        }
        return -1;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joincondition = null;

        if (manager instanceof PersonalTaskAssignmentManager)
        {
            joincondition = JoinCondition.createSubSelect(
                PersonalTaskAssignmentManager.NAME_TASK_ID, NAME_ELEMENT_ID,
                TABLE_NAME, getFilter());
        }
        else if (manager instanceof AdministrativeTaskAssignmentManager)
        {
            joincondition = JoinCondition.createSubSelect(
                    AdministrativeTaskAssignmentManager.NAME_TASK_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, getFilter());
            joincondition.appendEqualFields(NAME_CREATED_BY,
                    AdministrativeTaskAssignmentManager.NAME_RESOURCE_ID);
        }
        else if (manager instanceof TimesheetApprovalStatusManager)
        {
            joincondition = JoinCondition.createSubSelect(
                TimesheetApprovalStatusManager.NAME_ELEMENT_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFilter());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joincondition;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        //parent of a PersonalProject is a Resource while for all other type it
        // is a GenericTimesheetElement

        if (rpmScope != null && rpmScope.getParent() != null)
        {
            String parentId = null;
            if (rpmObject instanceof PersonalProject)
            {
                parentId = (String) relatedFields.get(index, ID_CREATED_BY);

                Resource resource = null;
                if (parentId != null)
                {
                    resource = new Resource();
                    resource.setID(parentId);

                    RPMObjectManager manager = RPMManagerFactory.getInstance()
                        .getRPMObjectManager(Resource.class);
                    resource = (Resource) manager.loadByPrimaryKey(resource,
                        rpmScope.getParent(), messageContext, false);
                }

                rpmObject.setParent(resource);
            }
            else
            {
                parentId = (String) relatedFields.get(index, ID_PARENT_ID);

                GenericTimesheetElement parent = null;
                if (parentId != null)
                {
                    parent = new GenericTimesheetElement();
                    parent.setID(parentId);

                    parent = (GenericTimesheetElement) loadByPrimaryKey(parent,
                        rpmScope.getParent(), messageContext, false);
                }

                rpmObject.setParent(parent);
            }
        }

        if (rpmScope instanceof PersonalProjectScope)
        {

            PersonalProjectScope scope = (PersonalProjectScope) rpmScope;
            PersonalProject object = (PersonalProject) rpmObject;

            loadPersonalTasks(object, scope, messageContext, isDelta);
            loadAdministrativeProject(object, scope, messageContext, isDelta);

        }
        else if (rpmScope instanceof AdministrativeProjectScope)
        {

            AdministrativeProjectScope scope = (AdministrativeProjectScope) rpmScope;
            AdministrativeProject object = (AdministrativeProject) rpmObject;

            loadAdministrativeTasks(object, scope, messageContext, isDelta);

        }

        return rpmObject;
    }

    private void loadPersonalTasks(PersonalProject object,
            PersonalProjectScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        if (scope.isPersonalTasks())
        {
            ArrayList temp = loadByForeignKey(object.getPersonalTasks(), null,
                messageContext, this, new Object[]
                {
                        new Integer(PersonalTask.TYPE_ID), object.getID()
                }, null, null, isDelta);

            object.setPersonalTasks((PersonalTask[]) ArrayUtil.listToArray(
                PersonalTask.class, temp));
        }
    }

    private void loadAdministrativeProject(PersonalProject object,
            PersonalProjectScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        if (scope.getAdministrativeProject() != null)
        {

            ArrayList temp = loadByForeignKey(new RPMObject[]
            {
                object.getAdministrativeProject()
            }, scope.getAdministrativeProject(), messageContext, this,
                new Object[]
                {
                        new Integer(AdministrativeProject.TYPE_ID),
                        object.getID()
                }, null, null, isDelta);

            if (temp != null && !temp.isEmpty())
            {
                object.setAdministrativeProject((AdministrativeProject) temp
                    .get(0));
            }
        }
    }

    private void loadAdministrativeTasks(AdministrativeProject object,
            AdministrativeProjectScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        if (scope.isAdministrativeTasks())
        {
            ArrayList temp = loadByForeignKey(object.getAdministrativeTasks(),
                null, messageContext, this, new Object[]
                {
                        new Integer(AdministrativeTask.TYPE_ID), object.getID()
                }, null, null, isDelta);

            object.setAdministrativeTasks((AdministrativeTask[]) ArrayUtil
                .listToArray(AdministrativeTask.class, temp));
        }
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {

        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        String resourceId = resultSet.getString(ID_CREATED_BY);
        relatedFields.put(index, ID_CREATED_BY, resourceId);

        return relatedFields;
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        AbstractTimesheetElement timesheetTask = (AbstractTimesheetElement) rpmObject;

        if (timesheetTask.testNameModified())
        {
            if (timesheetTask instanceof PersonalTask)
            {
                SP_U_WBS_NAME((PersonalTask) timesheetTask, messageContext);
            }
        }

        //TODO update other fields of personalTask and administrative task

        if (rpmScope instanceof PersonalProjectScope)
        {
            PersonalProjectScope scope = (PersonalProjectScope) rpmScope;
            PersonalProject object = (PersonalProject) rpmObject;

            savePersonalTasks(object, scope, messageContext);
        }
        else if (rpmScope instanceof AdministrativeProjectScope)
        {
            AdministrativeProjectScope scope = (AdministrativeProjectScope) rpmScope;
            AdministrativeProject object = (AdministrativeProject) rpmObject;

            saveAdministrativeTasks(object, scope, messageContext);
        }

        return rpmObject;
    }

    private PersonalProject savePersonalTasks(PersonalProject object,
            PersonalProjectScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

        PersonalTask[] children = object.getPersonalTasks();
        children = (PersonalTask[]) updateChildren(object, children, scope
            .isPersonalTasks(), messageContext, null);
        object.setPersonalTasks(children);
        return object;

    }

    private AdministrativeProject saveAdministrativeTasks(
            AdministrativeProject object, AdministrativeProjectScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {

        AdministrativeTask[] children = object.getAdministrativeTasks();
        children = (AdministrativeTask[]) updateChildren(object, children,
            scope.isAdministrativeTasks(), messageContext, null);
        object.setAdministrativeTasks(children);
        return object;

    }

    private String SP_I_SUMMARY_TS(PersonalTask personalTask,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_I_SUMMARY_TS";
        Object args[] = new Object[5];

        String resourceId = retrievePersonalTaskResource(personalTask,
            messageContext);

        args[0] = personalTask.getName(); // name
        args[1] = personalTask.getParent().getID(); // parent_id
        args[2] = personalTask.retrieveTypeID(); // type_id
        args[3] = DateUtil.convertToString(DateUtil.getCalendarInstance());
        //note: the last parameter must be the id of the resource that will owned the personal task
        args[4] = resourceId;

        //note: The id returned by the sp is the taskAssignmentId
        String assignmentId = executeProcedure(messageContext, procedureName,
            args);

        String personalTaskId = TimesheetManagerUtil
            .retrieveTaskAssignment_TaskId(assignmentId, messageContext);

        return personalTaskId;
    }

    private String SP_U_WBS_NAME(PersonalTask personalTask,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_WBS_NAME";
        Object args[] = new Object[3];

        args[0] = personalTask.getID(); // id
        args[1] = personalTask.getName(); // name
        args[2] = getUser(messageContext).getID();

        return executeProcedure(messageContext, procedureName, args);
    }

    private void SP_D_SUMMARY_TS(PersonalTask personalTask,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_D_SUMMARY_TS";
        Object args[] = new Object[4];

        args[0] = retrievePersonalTaskAssignmentId(personalTask, messageContext); //the taskassignment id related to the personal task
        args[1] = ManagerUtil.getParentId(this, personalTask, messageContext,
            TABLE_NAME);// parent_id: parent of personalTask in tmt_wbs
        args[2] = "X"; // del_flag
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    private String retrievePersonalTaskResource(PersonalTask personalTask,
            MessageContext messageContext) throws RPMException
    {

        String resourceId = null;

        PersonalProject personalProject = (PersonalProject) personalTask
            .getParent();

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(TimesheetTaskManager.NAME_CREATED_BY);
        stmt.appendFrom(TimesheetTaskManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(TimesheetTaskManager.NAME_ELEMENT_ID);

        List result = ManagerUtil.select(new Class[]
        {
            String.class
        }, null, null, stmt, new Object[]
        {
            personalProject.getID()
        }, messageContext);

        //result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            ArrayList columns = (ArrayList) result.get(0);
            resourceId = (String) columns.get(0);
        }

        return resourceId;

    }

    private static String retrievePersonalTaskAssignmentId(
            PersonalTask personalTask, MessageContext messageContext)
            throws RPMException
    {

        String value = null;

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(PersonalTaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID);
        stmt.appendFrom(PersonalTaskAssignmentManager.TABLE_NAME);
        stmt.appendWhere();
        stmt
            .appendEqualQuestionMark(PersonalTaskAssignmentManager.NAME_TASK_ID);

        List result = ManagerUtil.select(null, null, null, stmt, new Object[]
        {
            personalTask.getID()
        }, messageContext);

        //result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            ArrayList columns = (ArrayList) result.get(0);
            value = (String) columns.get(0);
        }

        return value;
    }

    protected boolean isCheckingOutOnCreate()
    {
        // Required to disable checking out/in on create because checkin
        // has a side effect on all other objects of that timesheet for
        // that user.
        return false;
    }
    
    public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        if (rpmObject instanceof PersonalTask)
        {
            String taskAssignmentId = retrievePersonalTaskAssignmentId(
                (PersonalTask) rpmObject, messageContext);

            TimesheetManagerUtil.checkinObject(this, taskAssignmentId,
                messageContext);
        }
    }

    public void undoCheckoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        if (rpmObject instanceof PersonalTask)
        {
            String taskAssignmentId = retrievePersonalTaskAssignmentId(
                (PersonalTask) rpmObject, messageContext);

            TimesheetManagerUtil.undoCheckOut(this, taskAssignmentId,
                messageContext);
        }
    }

    public void checkoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        if (rpmObject instanceof PersonalTask)
        {
            String taskAssignmentId = retrievePersonalTaskAssignmentId(
                (PersonalTask) rpmObject, messageContext);

            TimesheetManagerUtil.checkoutObject(this, taskAssignmentId,
                messageContext);
        }

    }

    public boolean isCheckedOut(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException, SQLException
    {

        if (rpmObject instanceof PersonalTask)
        {
            String taskAssignmentId = retrievePersonalTaskAssignmentId(
                (PersonalTask) rpmObject, messageContext);

            PersonalTaskAssignment taskAssignment = new PersonalTaskAssignment();
            taskAssignment.setID(taskAssignmentId);

            return TimesheetManagerUtil.isCheckedOut(messageContext,
                taskAssignment);
        }

        return false;
    }

    public String getCheckInCheckOutPrimaryKey(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        if (rpmObject instanceof PersonalTask)
        {
            String taskAssignmentId = retrievePersonalTaskAssignmentId(
                (PersonalTask) rpmObject, messageContext);

            return TimesheetManagerUtil.getCheckInCheckOutPrimaryKey(
                taskAssignmentId, messageContext).toString();
        }
        
        return null;
    }
}
