/*
 * Created on Nov 17, 2005 
 * @author mmatwiy 
 * Checkpoint validation for WBS Deliverable container.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.applicationadministration.containers.DeliverableState;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class DeliverableCheckpoint extends AbstractProjectActivityCheckpoint
{
    private static final List DELIVERABLE_PARENT_LIST = new ArrayList();

    protected static DeliverableCheckpoint instance = new DeliverableCheckpoint();

    static
    {
        DELIVERABLE_PARENT_LIST.add(Proposal.class);
        DELIVERABLE_PARENT_LIST.add(Project.class);
        DELIVERABLE_PARENT_LIST.add(WorkOrganization.class);
        DELIVERABLE_PARENT_LIST.add(Deliverable.class);
        DELIVERABLE_PARENT_LIST.add(WorkProduct.class);
        DELIVERABLE_PARENT_LIST.add(SummaryTask.class);
    }

    public static DeliverableCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx, true,
            DELIVERABLE_PARENT_LIST);

        Deliverable deliverable = (Deliverable) rpmObject;
        WorkElementScope scope = (WorkElementScope) rpmScope;

        if (scope != null && scope.isState() && deliverable.testStateModified())
        {
            // deliverable.state must be of type DeliverableState
            GenericValidator
                .validateType(rpmObject, DeliverableState.class,
                    ValidationConstants.STATE_FIELD, deliverable.getState(),
                    msgCtx);
        }
    }

}