/*
 * Created on Sep 19, 2005
 */
package com.ibm.rpm.wbs.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.wbs.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ProjectOrganizationalAssignment;
import com.ibm.rpm.wbs.managers.ProjectOrganizationalAssignmentManager;

/**
 * @author gosselin
 */
public class ProjectOrganizationalAssignmentCheckpoint extends
        AbstractCheckpoint
{
    private static final Class ORGANIZATIONALASSIGNMENT_PARENTS = GenericProject.class;

    protected static ProjectOrganizationalAssignmentCheckpoint instance = new ProjectOrganizationalAssignmentCheckpoint();

    public static ProjectOrganizationalAssignmentCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true,
            ORGANIZATIONALASSIGNMENT_PARENTS);

        if (!GenericValidator.validateInstance(rpmObject,
            ProjectOrganizationalAssignment.class, msgCtx))
            return;

        // Initialize
        ProjectOrganizationalAssignment mainOA = (ProjectOrganizationalAssignment) rpmObject;

        // Field validation: organizational
        GenericValidator.validateMandatoryID(mainOA,
            ValidationConstants.ORGANIZATIONAL_FIELD, mainOA
                .getOrganizational(), msgCtx);

        if (msgCtx.isSuccessful() && mainOA.testOrganizationalModified())
        {
            // Check if assignment already exists; only check this if 
            // the parent is valid and the ID exists.
            validateAlreadyExistsInDB(mainOA, msgCtx);
        }

    }


    private void validateAlreadyExistsInDB(
            ProjectOrganizationalAssignment mainOA, MessageContext msgCtx)
    {
        if (mainOA.getParent().getID() != null
                && mainOA.getOrganizational().getID() != null)
        {
            try
            {
                SqlBuffer stmt = new SqlBuffer();
                stmt.appendSelect();
                stmt
                    .append(ProjectOrganizationalAssignmentManager.NAME_ELEMENT_ID);
                stmt
                    .appendFrom(ProjectOrganizationalAssignmentManager.TABLE_NAME);
                stmt.appendWhere();
                stmt
                    .appendEqualQuestionMark(ProjectOrganizationalAssignmentManager.NAME_PARENT_ID);
                stmt
                    .appendEqualQuestionMark(ProjectOrganizationalAssignmentManager.NAME_ORG_ID);

                List result = ManagerUtil.select(null, null, null, stmt, new Object[]
                {
                        mainOA.getParent().getID(),
                        mainOA.getOrganizational().getID()

                }, msgCtx);

                Object value = null;

                //result should not be null or empty at this point
                if (result != null && result.size() > 0)
                {
                    ArrayList columns = (ArrayList) result.get(0);
                    value = columns.get(0);
                }

                if (value != null)
                {
                    String[] errorParams =
                    {
                            StringUtil.getShortClassName(mainOA
                                .getOrganizational().getClass()),
                            mainOA.getOrganizational().getID(),
                            StringUtil.getShortClassName(mainOA.getParent()
                                .getClass()), 
                            mainOA.getParent().getID()
                    };
                    RPMException e = new RPMException(
                        ErrorCodes.DUPLICATE_ASSIGNMENT, errorParams, mainOA
                            .getClass().getName(),
                        ValidationConstants.ORGANIZATIONAL_FIELD, mainOA
                            .getID());
                    addException(e, msgCtx);
                }
            }
            catch (RPMException e)
            {
                addException(e, msgCtx);
            }
        }

    }
}