/*
 * Created on Nov 17, 2005 
 * @author mmatwiy 
 * Checkpoint validation for WBS SummaryTask container.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.applicationadministration.containers.ActionState;
import com.ibm.rpm.applicationadministration.containers.ChangeRequestState;
import com.ibm.rpm.applicationadministration.containers.DefectState;
import com.ibm.rpm.applicationadministration.containers.DeliverableState;
import com.ibm.rpm.applicationadministration.containers.IssueState;
import com.ibm.rpm.applicationadministration.containers.RequirementState;
import com.ibm.rpm.applicationadministration.containers.RiskState;
import com.ibm.rpm.applicationadministration.containers.ServiceRequestState;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.util.ScopeManagementUtil;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class SummaryTaskCheckpoint extends AbstractProjectActivityCheckpoint
{
    private static final List SUMMARYTASK_PARENT_LIST = new ArrayList();

    protected static SummaryTaskCheckpoint instance = new SummaryTaskCheckpoint();

    static
    {
        SUMMARYTASK_PARENT_LIST.add(Proposal.class);
        SUMMARYTASK_PARENT_LIST.add(Project.class);
        SUMMARYTASK_PARENT_LIST.add(WorkOrganization.class);
        SUMMARYTASK_PARENT_LIST.add(Deliverable.class);
        SUMMARYTASK_PARENT_LIST.add(WorkProduct.class);
        SUMMARYTASK_PARENT_LIST.add(SummaryTask.class);
    }

    public static SummaryTaskCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx, true,
            SUMMARYTASK_PARENT_LIST);

        SummaryTask summaryTask = (SummaryTask) rpmObject;
        WorkElementScope scope = (WorkElementScope) rpmScope;

        if (scope != null && scope.isState() && summaryTask.testStateModified())
        {
            ScopeElement correspondingScopeElement = null;
            try
            {
                correspondingScopeElement = ((WorkElement) ScopeManagementUtil.loadScopeElement(
                    rpmObject, msgCtx)).getScopeElement();
            }
            catch (Exception e)
            {
                msgCtx.addExceptionNoThrow(e);
            }

            if (correspondingScopeElement instanceof Action)
            {
                GenericValidator.validateType(rpmObject, ActionState.class,
                    ValidationConstants.STATE_FIELD, summaryTask.getState(),
                    msgCtx);
            }
            else if (correspondingScopeElement instanceof ChangeRequest)
            {
                GenericValidator.validateType(rpmObject,
                    ChangeRequestState.class, ValidationConstants.STATE_FIELD,
                    summaryTask.getState(), msgCtx);
            }
            else if (correspondingScopeElement instanceof Defect)
            {
                GenericValidator.validateType(rpmObject, DefectState.class,
                    ValidationConstants.STATE_FIELD, summaryTask.getState(),
                    msgCtx);
            }
            else if (correspondingScopeElement instanceof Issue)
            {
                GenericValidator.validateType(rpmObject, IssueState.class,
                    ValidationConstants.STATE_FIELD, summaryTask.getState(),
                    msgCtx);
            }
            else if (correspondingScopeElement instanceof Requirement)
            {
                GenericValidator.validateType(rpmObject,
                    RequirementState.class, ValidationConstants.STATE_FIELD,
                    summaryTask.getState(), msgCtx);
            }
            else if (correspondingScopeElement instanceof Risk)
            {
                GenericValidator.validateType(rpmObject, RiskState.class,
                    ValidationConstants.STATE_FIELD, summaryTask.getState(),
                    msgCtx);
            }
            else if (correspondingScopeElement instanceof ServiceRequest)
            {
                GenericValidator.validateType(rpmObject,
                    ServiceRequestState.class, ValidationConstants.STATE_FIELD,
                    summaryTask.getState(), msgCtx);
            }
            else
            // If we get here, (correspondingScopeElement == null)
            // summaryTask.state must be of type DeliverableState
            // when the SummaryTask is not a transferred scope element
            {
                GenericValidator.validateType(rpmObject,
                    DeliverableState.class, ValidationConstants.STATE_FIELD,
                    summaryTask.getState(), msgCtx);
            }
        }
    }
}
