/*
 * Created on Nov 17, 2005 @author mmatwiy Checkpoint validation of WBS
 * container Task.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.managers.WbsManager;

public class TaskCheckpoint extends SimpleNodeCheckpoint
{
    private static final List TASK_PARENT_LIST = new ArrayList();

    protected static TaskCheckpoint instance = new TaskCheckpoint();

    static
    {
        TASK_PARENT_LIST.add(Proposal.class);
        TASK_PARENT_LIST.add(Project.class);
        TASK_PARENT_LIST.add(WorkOrganization.class);
        TASK_PARENT_LIST.add(Deliverable.class);
        TASK_PARENT_LIST.add(WorkProduct.class);
        TASK_PARENT_LIST.add(SummaryTask.class);
    }

    public static TaskCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.validateSave(rpmObject, rpmScope, msgCtx, true, TASK_PARENT_LIST);

        Task task = (Task) rpmObject;

        int actualDurationInMinutes = (task.getActualDate() == null ? 0 : (task
            .getActualDate().getDurationInMinutes() == null ? 0 : task
            .getActualDate().getDurationInMinutes().intValue()));

        int planDurationInMinutes = (task.getPlanDate() == null ? 0 : (task
            .getPlanDate().getDurationInMinutes() == null ? 0 : task
            .getPlanDate().getDurationInMinutes().intValue()));

        Calendar planStartDate = (task.getPlanDate() == null ? null : task
            .getPlanDate().getStartDate());

        if (task.testPercentageDurationCompletedModified())
        {
            // Read-only if planDate.startDate does not have a value
            boolean percentDurationCompletedValid = GenericValidator
                .validateReadOnlyConditionMissing(task,
                    ValidationConstants.PERCENT_DURATION_COMPLETED_FIELD,
                    ValidationConstants.SCHEDULE_DATE_PLAN_START_FIELD,
                    planStartDate, msgCtx);

            // Read-only if work has been assigned
            percentDurationCompletedValid &= validateReadOnlyWorkAssigned(task,
                ValidationConstants.PERCENT_DURATION_COMPLETED_FIELD, msgCtx);

            // Read-only if actualDate.durationInMinutes ==
            // planDate.durationInMinutes
            if (planDurationInMinutes != 0)
            {
                percentDurationCompletedValid &= GenericValidator
                    .validateReadOnlyConditionalFieldsSame(
                        task,
                        ValidationConstants.PERCENT_DURATION_COMPLETED_FIELD,
                        ValidationConstants.SCHEDULE_DATE_ACTUAL_DURATION_FIELD,
                        ValidationConstants.SCHEDULE_DATE_PLAN_DURATION_FIELD,
                        actualDurationInMinutes, planDurationInMinutes, msgCtx);
            }

            if (!percentDurationCompletedValid)
            {
              // reset the field to its original value
                Double originalPercentageDurationCompleted = loadTaskPercentageDurationCompleted(task, msgCtx); 
                task.setPercentageDurationCompleted(originalPercentageDurationCompleted);
            }

            // Check that the percentDurationCompleted is in the correct range
            // of values
            GenericValidator.validateRange(task,
                ValidationConstants.PERCENT_DURATION_COMPLETED_FIELD, task
                    .getPercentageDurationCompleted(),
                ValidationConstants.PERCENT_DURATION_COMPLETED_MIN,
                ValidationConstants.PERCENT_DURATION_COMPLETED_MAX, msgCtx);
        }
    }

    private Double loadTaskPercentageDurationCompleted(Task task, MessageContext msgCtx)
    {
        Double percentageDurationCompleted = null;

        if (task.getID() != null)
        {
            WbsManager wbsManager = new WbsManager();
            
            SqlBuffer sql = new SqlBuffer();
            sql.appendEqualQuestionMark(WbsManager.NAME_ELEMENT_ID);
            sql.appendAnd();
            sql.append(wbsManager.getFilter());
            
            Object[] arguments = new Object[]
            {
                task.getID()
            };
            
            try
            {
                percentageDurationCompleted = (Double) ManagerUtil
                    .selectColumnValue(Double.class, null, null,
                        WbsManager.NAME_MILESTONE_PERCENT,
                        WbsManager.TABLE_NAME, sql, arguments, msgCtx);
                //special logic in the manager
                percentageDurationCompleted = wbsManager.decodePercentageDurationCompleted(percentageDurationCompleted);
            }
            catch (Exception e)
            {
                RPMException rpmException = new RPMException(e);
                addException(rpmException, msgCtx);
            }
        }

        return percentageDurationCompleted;
    }
}