package com.ibm.rpm.wbs.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.managers.ResourceAttributeManager;
import com.ibm.rpm.wbs.containers.Profile;
import com.ibm.rpm.wbs.containers.ProfileCompetencyAssignment;

public class ProfileCompetencyAssignmentManager extends
        AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(ProfileCompetencyAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ProfileCompetencyAssignmentManager.class);
        }
    }

    /**
     * Column RES_ASSIGNMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RES_ASSIGNMENT_ID = 1;

    public static final int TYPE_RES_ASSIGNMENT_ID = Types.CHAR;

    public static final String NAME_RES_ASSIGNMENT_ID = "RESOURCE_ASSIGN_ROLES.RES_ASSIGNMENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RES_ASSIGNMENT_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_RES_ASSIGNMENT_ID,NAME_RES_ASSIGNMENT_ID);}

    /**
     * Column ROLE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ROLE_ID = 2;

    public static final int TYPE_ROLE_ID = Types.CHAR;

    public static final String NAME_ROLE_ID = "RESOURCE_ASSIGN_ROLES.ROLE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_ROLE_ID = "ROLEID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ROLE_ID, NAME_ROLE_ID);
    }

    /**
     * Column PROFICIENCY_LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROFICIENCY_LEVEL_ID = 3;

    public static final int TYPE_PROFICIENCY_LEVEL_ID = Types.INTEGER;

    public static final String NAME_PROFICIENCY_LEVEL_ID = "RESOURCE_ASSIGN_ROLES.PROFICIENCY_LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROFICIENCY_LEVEL_ID =
    // "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PROFICIENCY_LEVEL_ID,NAME_PROFICIENCY_LEVEL_ID);}

    /**
     * Column PREFERENCE_LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PREFERENCE_LEVEL_ID = 4;

    public static final int TYPE_PREFERENCE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_PREFERENCE_LEVEL_ID = "RESOURCE_ASSIGN_ROLES.PREFERENCE_LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PREFERENCE_LEVEL_ID = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_PREFERENCE_LEVEL_ID,NAME_PREFERENCE_LEVEL_ID);}

    /**
     * Column EXPERIENCE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_EXPERIENCE_ID = 5;

    public static final int TYPE_EXPERIENCE_ID = Types.INTEGER;

    public static final String NAME_EXPERIENCE_ID = "RESOURCE_ASSIGN_ROLES.EXPERIENCE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EXPERIENCE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_EXPERIENCE_ID,NAME_EXPERIENCE_ID);}

    /**
     * Column LAST_USED_DATETIME of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_LAST_USED_DATETIME = 6;

    public static final int TYPE_LAST_USED_DATETIME = Types.DATE;

    public static final String NAME_LAST_USED_DATETIME = "RESOURCE_ASSIGN_ROLES.LAST_USED_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_USED_DATETIME = "PROPERTYNAME";
    //static{
    // FIELDPROPERTYMAP.put(PROPERTY_LAST_USED_DATETIME,NAME_LAST_USED_DATETIME);}

    /**
     * Column PRIMARY_ROLE of type Types.CHAR mapped to String.
     */
    public static final int ID_PRIMARY_ROLE = 7;

    public static final int TYPE_PRIMARY_ROLE = Types.CHAR;

    public static final String NAME_PRIMARY_ROLE = "RESOURCE_ASSIGN_ROLES.PRIMARY_ROLE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PRIMARY_ROLE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PRIMARY_ROLE,NAME_PRIMARY_ROLE);}

    private static final String TABLE_NAME = "RESOURCE_ASSIGN_ROLES";

    /**
     * Create an array of type string containing all the fields of the
     * RESOURCE_ASSIGN_ROLES table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_RES_ASSIGNMENT_ID, NAME_ROLE_ID, NAME_PROFICIENCY_LEVEL_ID,
            NAME_PREFERENCE_LEVEL_ID, NAME_EXPERIENCE_ID,
            NAME_LAST_USED_DATETIME, NAME_PRIMARY_ROLE
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new ProfileCompetencyAssignmentManager instance.
     * 
     * @return ProfileCompetencyAssignmentManager
     */
    public ProfileCompetencyAssignmentManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_RES_ASSIGNMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {
        return new Integer(0);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        return new ProfileCompetencyAssignment();
    }

    /**
     *  
     */

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        //object creation done in ProfileManager 
    }

    /**
     *  
     */

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        rpmObject.setID(rs.getString(ID_RES_ASSIGNMENT_ID));
        return rpmObject;
    }
    
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_RES_ASSIGNMENT_ID);
        relatedFields.put(index, ID_RES_ASSIGNMENT_ID, parentId);
        
        String roleId = resultSet.getString(ID_ROLE_ID);
        relatedFields.put(index, ID_ROLE_ID, roleId);
        return relatedFields;
    }
    
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        ProfileCompetencyAssignment assignment = 
            (ProfileCompetencyAssignment)rpmObject;
        loadProfiencyLevel(assignment, messageContext, relatedFields, index, isDelta);
        loadProfile(assignment, messageContext, relatedFields, index, isDelta);
        return assignment;
    }
    
    private void loadProfiencyLevel(ProfileCompetencyAssignment assignment,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        String proLevelID = (String) relatedFields.get(index, ID_ROLE_ID);

        if (proLevelID != null)
        {
            ResourceAttributeManager manager = (ResourceAttributeManager) RPMManagerFactory
                .getInstance()
                .getRPMObjectManager(
                    ResourceAttributeCompetencyProficiencyLevel.class.getName());
            
            ResourceAttributeCompetencyProficiencyLevel proLevel = 
                new ResourceAttributeCompetencyProficiencyLevel();
            proLevel.setID(proLevelID);
            proLevel = 
                (ResourceAttributeCompetencyProficiencyLevel)manager.
                loadByPrimaryKey(proLevel, null, messageContext, isDelta);
            if(isDelta)
            {
                assignment.deltaProficiencyLevel(proLevel);
            }
            else
            {
                assignment.setProficiencyLevel(proLevel);
            }
        }
    }
    
    private void loadProfile(ProfileCompetencyAssignment assignment,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        if(assignment.getParent() != null) return;
        
        String profileID = (String) relatedFields.get(index,
            ID_RES_ASSIGNMENT_ID);
        
        if (profileID != null)
        {
            ProfileManager manager = (ProfileManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(Profile.class.getName());
            Profile profile = new Profile();
            profile.setID(profileID);
            profile = (Profile) manager.loadByPrimaryKey(profile,
                null, messageContext, isDelta);
            if (isDelta)
            {
                assignment.deltaParent(profile);
            }
            else
            {
                assignment.setParent(profile);
            }
        }
        else
        {
            ExceptionUtil
                .handleNoId(this, assignment, ExceptionUtil.PROFILE_ID);
        }
    }

    /**
     *  
     */

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // TODO Auto-generated method stub

    }

    /**
     *  
     */

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ProfileManager)
        {

            //where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(" RES_ASSIGNMENT_ID ");
            condition.append(" = ?");

            joinCondition = getJoinCondition(manager, condition.toString(), 1,
                null, contextName);
            if (extraWhere != null && extraWhere.length() > 0)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(joinCondition.getCondition());
                sb.append(" AND ");
                sb.append(extraWhere);
                joinCondition.setCondition(sb.toString());

            }
            joinCondition.setTableName(getTableName(contextName));

        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    /**
     *  
     */

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            if (condition != null)
            {
                where.append(condition);
                //uncomment if additionnal condition is needed
                //where.append(" AND ");
            }
            joinCondition.setCondition(where.toString());

        }
        else if(manager instanceof ProfileManager)
        {
            joinCondition = new JoinCondition();

            StringBuffer where = new StringBuffer();
            if (condition != null)
            {
                where.append(condition);
            }
            joinCondition.setCondition(where.toString());
        }

        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    /**
     *  
     */

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        // TODO Implement for managers that handles the parents of the supported
        // containers
        JoinCondition joinCondition = null;

        ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
            ExceptionUtil.SUB_TABLE_JOIN);

        return joinCondition;
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        // TODO Auto-generated method stub
        return null;
    }
}
