package com.ibm.rpm.wbs.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.applicationadministration.containers.Organization;
import com.ibm.rpm.applicationadministration.containers.OrganizationType;
import com.ibm.rpm.applicationadministration.containers.Organizational;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.wbs.containers.ProjectOrganizationalAssignment;

/**
 * Manager for the PRJ_ASSIGNED_ORG table and related stored procedures.
 * 
 * <h3>Map Object Properties To PRJ_ASSIGNED_ORG Table Fields</h3>
 * <p>
 * <table border=1 cellspacing=0>
 * <tr>
 * <th>Object property</th>
 * <th>Table field</th>
 * <th>Comment</th>
 * </tr>
 * <tr>
 * <td>RPMObject.TypeID</td>
 * <td>--</td>
 * <td>Always 0</td>
 * </tr>
 * <tr>
 * <td>RPMObject.ID</td>
 * <td>ELEMENT_ID</td>
 * <td>Primary key</td>
 * </tr>
 * <tr>
 * <td>RPMObject.Parent</td>
 * <td>PROJECT_ID</td>
 * <td>Foreign key: TMT_WBS.ELEMENT_ID</td>
 * </tr>
 * <tr>
 * <td>ProjectOrganizationalAssignment.Organizational</td>
 * <td>ORG_ID</td>
 * <td>Foreign key: TMT_ORGANIZATIONS. ELEMENT_ID</td>
 * </tr>
 * <tr>
 * <td>ProjectOrganizationalAssignment.OrganizationType</td>
 * <td>DATAFIELD</td>
 * <td>Foreign key: TMT_DATAFIELDS.ELEMENT_ID</td>
 * </tr>
 * </table>
 * 
 * @author Simon Legault
 */
public class ProjectOrganizationalAssignmentManager extends
        AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(ProjectOrganizationalAssignment.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    ProjectOrganizationalAssignmentManager.class);
        }
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "PRJ_ASSIGNED_ORG.ELEMENT_ID";
    
    public static final String NAME_PARENT_ID = "PRJ_ASSIGNED_ORG.PROJECT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 2;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "PRJ_ASSIGNED_ORG.PROJECT_ID";

    public static final String PROPERTY_PROJECT_ID = "PARENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID, NAME_PROJECT_ID);
    }

    /**
     * Column ORG_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ORG_ID = 3;

    public static final int TYPE_ORG_ID = Types.CHAR;

    public static final String NAME_ORG_ID = "PRJ_ASSIGNED_ORG.ORG_ID";

    public static final String PROPERTY_ORG_ID = "ORGANIZATIONAL";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ORG_ID, NAME_ORG_ID);
    }

    /**
     * Column DATAFIELD of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DATAFIELD = 4;

    public static final int TYPE_DATAFIELD = Types.INTEGER;

    public static final String NAME_DATAFIELD = "PRJ_ASSIGNED_ORG.DATAFIELD";

    public static final String PROPERTY_DATAFIELD = "ORGANIZATIONTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DATAFIELD, NAME_DATAFIELD);
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 5;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "PRJ_ASSIGNED_ORG.REC_USER";

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 6;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "PRJ_ASSIGNED_ORG.REC_DATETIME";

    public static final String TABLE_NAME = "PRJ_ASSIGNED_ORG";

    /**
     * Create an array of type string containing all the fields of the
     * PRJ_ASSIGNED_ORG table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_PROJECT_ID, NAME_ORG_ID, NAME_DATAFIELD,
            NAME_REC_USER, NAME_REC_DATETIME
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new PrjAssignedOrgManager instance.
     * 
     * @return PrjAssignedOrgManager
     */
    public ProjectOrganizationalAssignmentManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String otherCondition, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WbsManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME, otherCondition);
            joinCondition.appendCondition("PROJECT_ID=?");
        }
        // Usefull?
		//else if (manager instanceof OrganizationalManager)
		//{
		//    joinCondition = new JoinCondition(otherCondition, TABLE_NAME);
		//    joinCondition.appendCondition("ORG_ID=?");
		//}
        // Usefull?
		//else if (manager instanceof ProjectOrganizationalAssignmentManager)
		//{
		//    joinCondition = new JoinCondition(otherCondition, TABLE_NAME);
		//    joinCondition.appendCondition("ELEMENT_ID=?");
		//}
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);
        }
        return joinCondition;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager == null)
        {
            ExceptionUtil.handleNoParentManager(this, containerName);
        }
        // Usefull?
        //else if (manager instanceof WbsManager)
        //{
        //    joinCondition = new JoinCondition(condition);
        //}
        // Usefull?
        //else if (manager instanceof ProjectOrganizationalAssignmentManager)
        //{
        //    joinCondition = new JoinCondition(condition);
        //}
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        
        // For XPath like: /Project[ProjectOrganizationalAssignment[<condition>]]
        joinCondition = JoinCondition.createSubSelect(WbsManager.NAME_ELEMENT_ID,
            NAME_PROJECT_ID, TABLE_NAME, condition);
        
        return joinCondition;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        return new ProjectOrganizationalAssignment();
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        rpmObject.setID(rs.getString(ID_ELEMENT_ID));
        return rpmObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        // Design note: Always loaded from the parent.
        // => No need to load the parent (project) here.
        relatedFields.put(index, ID_ORG_ID, resultSet.getString(ID_ORG_ID));
        relatedFields.put(index, ID_DATAFIELD, ManagerUtil
                .intIdToStringId(resultSet.getInt(ID_DATAFIELD)));
        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        // Design note: Always loaded from the parent.
        // => No need to load the parent (project) here.
        // => Scope has been tested by parent's manager (WbsManager).
        ProjectOrganizationalAssignment object = (ProjectOrganizationalAssignment) rpmObject;

        // Load organization.
        String organizationalId = (String) relatedFields.get(index, ID_ORG_ID);
        if ((organizationalId != null))
        {
            Organizational organizational = new Organizational();
            organizational.setID(organizationalId);
            if (isDelta)
            {
                object.deltaOrganizational(organizational);
            }
            else
            {
                RPMObjectManager manager = RPMManagerFactory.getInstance()
                        .getRPMObjectManager(Organization.class);
                organizational = (Organizational) manager.loadByPrimaryKey(
                        organizational, null, messageContext, isDelta);
                object.setOrganizational(organizational);
            }
        }

        // Load organizationType.
        String organizationTypeId = (String) relatedFields.get(index,
                ID_DATAFIELD);
        if ((organizationTypeId != null))
        {
            OrganizationType organizationType = new OrganizationType();
            organizationType.setID(organizationTypeId);
            if (isDelta)
            {
                object.deltaOrganizationType(organizationType);
            }
            else
            {
                RPMObjectManager manager = RPMManagerFactory.getInstance()
                        .getRPMObjectManager(OrganizationType.class);
                organizationType = (OrganizationType) manager.loadByPrimaryKey(
                        organizationType, null, messageContext, isDelta);
                object.setOrganizationType(organizationType);
            }
        }

        return rpmObject;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ProjectOrganizationalAssignment assignment = (ProjectOrganizationalAssignment) rpmObject;
        String assignmentId = SP_A_PRJ_ASSIGNED_ORG(assignment.getParent()
                .getID(), assignment.getOrganizational().getID(),
                messageContext);
        assignment.setID(assignmentId);
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        // Design note: Update and delete NEVER propagate through an
        // association and here, organizational and organisationalType are
        // associations. Only the association itself (not the associated object)
        // is updated.
        ProjectOrganizationalAssignment assignment = (ProjectOrganizationalAssignment) rpmObject;
        Integer organizationTypeId = ManagerUtil.getIntId(assignment
                .getOrganizationType());
        SP_U_PRJ_ASSIGNED_ORG(assignment.getID(), organizationTypeId,
                messageContext);
        return rpmObject;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        // Design note: Update and delete NEVER propagate through an
        // association and here, organizational and organisationalType are
        // associations.
        ProjectOrganizationalAssignment assignment = (ProjectOrganizationalAssignment) rpmObject;
        SP_UA_PRJ_ASSIGNED_ORG(assignment.getParent().getID(), assignment.getID(), messageContext);
    }

    private String SP_A_PRJ_ASSIGNED_ORG(String projectId,
            String organizationId, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Object args[] = new Object[3];
        args[0] = projectId;
        args[1] = organizationId;
        args[2] = getUser(messageContext).getID();
        return executeProcedure(messageContext, "SP_A_PRJ_ASSIGNED_ORG", args);
    }

    private void SP_U_PRJ_ASSIGNED_ORG(String id, Integer organizationTypeId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[4];
        args[0] = id;
        args[1] = organizationTypeId;
        args[2] = new Integer(0); // Not even used by the stored proc!
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_U_PRJ_ASSIGNED_ORG", args);
    }
    
    private String SP_UA_PRJ_ASSIGNED_ORG(String projectId,
            String assignmentId, MessageContext messageContext)
            throws SQLException, RPMException
    {
        Object args[] = new Object[3];
        args[0] = projectId;
        args[1] = assignmentId;
        args[2] = getUser(messageContext).getID();
        return executeProcedure(messageContext, "SP_UA_PRJ_ASSIGNED_ORG", args);
    }
}
