/*
 * Created on Oct 18, 2005
 */
package com.ibm.rpm.wbs.managers;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import com.ibm.rpm.applicationadministration.containers.AssignmentStatus;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.CheckpointCaller;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.resource.checkpoints.ResourceCheckpoint;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.managers.SecurityRoleManager;
import com.ibm.rpm.wbs.checkpoints.ResourceTaskAssignmentCheckpoint;
import com.ibm.rpm.wbs.checkpoints.WbsMethodCheckpoint;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Profile;
import com.ibm.rpm.wbs.containers.ProfileCompetencyAssignment;
import com.ibm.rpm.wbs.containers.ProfileSkillAssignment;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ProfileScope;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ResourceType;

public class ResourceTaskAssignmentManager extends AbstractRPMObjectManager
{
    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    private static final String RESOURCE_TYPE_RESOURCE = "0";

    private static final String RESOURCE_TYPE_PROFILE = "1";

    private static final int DEFAULT_WORK_LOCATION = 265;

    static
    {
        CONTAINERS.add(ResourceTaskAssignment.class.getName());
        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                ResourceTaskAssignmentManager.class);
        }
    }

    /**
     * Column RES_ASSIGNMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RES_ASSIGNMENT_ID = 1;

    public static final int TYPE_RES_ASSIGNMENT_ID = Types.CHAR;

    public static final String NAME_RES_ASSIGNMENT_ID = "RESOURCE_ASSIGNMENTS.RES_ASSIGNMENT_ID";

    public static final String PROPERTY_RES_ASSIGNMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP
            .put(PROPERTY_RES_ASSIGNMENT_ID, NAME_RES_ASSIGNMENT_ID);
    }

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_PROJECT_ID = 2;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "RESOURCE_ASSIGNMENTS.PROJECT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_ELEMENT_ID = 3;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "RESOURCE_ASSIGNMENTS.ELEMENT_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ELEMENT_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID,NAME_ELEMENT_ID);}

    /**
     * Column ELEMENT_TYPE of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_ELEMENT_TYPE = 4;

    public static final int TYPE_ELEMENT_TYPE = Types.INTEGER;

    public static final String NAME_ELEMENT_TYPE = "RESOURCE_ASSIGNMENTS.ELEMENT_TYPE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ELEMENT_TYPE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_TYPE,NAME_ELEMENT_TYPE);}

    /**
     * Column PROFILE_ID of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_PROFILE_ID = 5;

    public static final int TYPE_PROFILE_ID = Types.CHAR;

    public static final String NAME_PROFILE_ID = "RESOURCE_ASSIGNMENTS.PROFILE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PROFILE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PROFILE_ID,NAME_PROFILE_ID);}

    /**
     * Column RESOURCE_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_RESOURCE_ID = 6;

    public static final int TYPE_RESOURCE_ID = Types.CHAR;

    public static final String NAME_RESOURCE_ID = "RESOURCE_ASSIGNMENTS.RESOURCE_ID";

    public static final String PROPERTY_RESOURCE_ID = "RESOURCEID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_ID, NAME_RESOURCE_ID);
    }

    /**
     * Column RESOURCE_TYPE of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_RESOURCE_TYPE = 7;

    public static final int TYPE_RESOURCE_TYPE = Types.CHAR;

    public static final String NAME_RESOURCE_TYPE = "RESOURCE_ASSIGNMENTS.RESOURCE_TYPE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    public static final String PROPERTY_RESOURCE_TYPE = "RESOURCETYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RESOURCE_TYPE, NAME_RESOURCE_TYPE);
    }

    /**
     * Column WORK_LOCATION of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_WORK_LOCATION = 8;

    public static final int TYPE_WORK_LOCATION = Types.INTEGER;

    public static final String NAME_WORK_LOCATION = "RESOURCE_ASSIGNMENTS.WORK_LOCATION";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_WORK_LOCATION = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_WORK_LOCATION,NAME_WORK_LOCATION);}

    /**
     * Column PARTICIPANT of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_PARTICIPANT = 9;

    public static final int TYPE_PARTICIPANT = Types.INTEGER;

    public static final String NAME_PARTICIPANT = "RESOURCE_ASSIGNMENTS.PARTICIPANT";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_PARTICIPANT = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_PARTICIPANT,NAME_PARTICIPANT);}

    /**
     * Column SECURITY_ID of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_SECURITY_ID = 10;

    public static final int TYPE_SECURITY_ID = Types.CHAR;

    public static final String NAME_SECURITY_ID = "RESOURCE_ASSIGNMENTS.SECURITY_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_SECURITY_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_SECURITY_ID,NAME_SECURITY_ID);}

    /**
     * Column ASSIGN_STATUS of type Types.INTEGER mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_ASSIGN_STATUS = 11;

    public static final int TYPE_ASSIGN_STATUS = Types.INTEGER;

    public static final String NAME_ASSIGN_STATUS = "RESOURCE_ASSIGNMENTS.ASSIGN_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_ASSIGN_STATUS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_ASSIGN_STATUS,NAME_ASSIGN_STATUS);}

    /**
     * Column TIMECODE_ID of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_TIMECODE_ID = 12;

    public static final int TYPE_TIMECODE_ID = Types.CHAR;

    public static final String NAME_TIMECODE_ID = "RESOURCE_ASSIGNMENTS.TIMECODE_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TIMECODE_ID = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_ID,NAME_TIMECODE_ID);}

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_STATUS = 13;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "RESOURCE_ASSIGNMENTS.REC_STATUS";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String. REPLACE PROPERTYNAME
     * with container property in upper case.
     */
    public static final int ID_REC_USER = 14;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "RESOURCE_ASSIGNMENTS.REC_USER";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     * REPLACE PROPERTYNAME with container property in upper case.
     */
    public static final int ID_REC_DATETIME = 15;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "RESOURCE_ASSIGNMENTS.REC_DATETIME";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column TABLE_TYPE of type Types.SMALLINT mapped to Integer. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_TABLE_TYPE = 16;

    public static final int TYPE_TABLE_TYPE = Types.SMALLINT;

    public static final String NAME_TABLE_TYPE = "RESOURCE_ASSIGNMENTS.TABLE_TYPE";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_TABLE_TYPE = "PROPERTYNAME";
    // static{ FIELDPROPERTYMAP.put(PROPERTY_TABLE_TYPE,NAME_TABLE_TYPE);}

    /**
     * Column CRI_SECURITY_ID of type Types.VARCHAR mapped to String. REPLACE
     * PROPERTYNAME with container property in upper case.
     */
    public static final int ID_CRI_SECURITY_ID = 17;

    public static final int TYPE_CRI_SECURITY_ID = Types.VARCHAR;

    public static final String NAME_CRI_SECURITY_ID = "RESOURCE_ASSIGNMENTS.CRI_SECURITY_ID";

    // uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    // public static final String PROPERTY_CRI_SECURITY_ID = "PROPERTYNAME";
    // static{
    // FIELDPROPERTYMAP.put(PROPERTY_CRI_SECURITY_ID,NAME_CRI_SECURITY_ID);}

    public static final String TABLE_NAME = "RESOURCE_ASSIGNMENTS";

    /**
     * Create an array of type string containing all the fields of the
     * RESOURCE_ASSIGNMENTS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_RES_ASSIGNMENT_ID, NAME_PROJECT_ID, NAME_ELEMENT_ID,
            NAME_ELEMENT_TYPE, NAME_PROFILE_ID, NAME_RESOURCE_ID,
            NAME_RESOURCE_TYPE, NAME_WORK_LOCATION, NAME_PARTICIPANT,
            NAME_SECURITY_ID, NAME_ASSIGN_STATUS, NAME_TIMECODE_ID,
            NAME_REC_STATUS, NAME_REC_USER, NAME_REC_DATETIME, NAME_TABLE_TYPE,
            NAME_CRI_SECURITY_ID
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    public ResourceTaskAssignmentManager()
    {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getTableName()
     */
    public String getTableName()
    {
        return TABLE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.containers.AbstractRPMObjectManager#getFieldsNames()
     */
    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getAllFields()
     */
    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPrimaryKey()
     */
    public String getPrimaryKey()
    {
        return NAME_RES_ASSIGNMENT_ID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getContainerType(java.sql.ResultSet)
     */
    protected Integer getContainerType(ResultSet resultSet)
            throws RPMException, SQLException
    {

        return new Integer(0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getPropertiesMap()
     */
    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getValidContainerNames()
     */
    protected HashSet getValidContainerNames()
    {
        return CONTAINERS;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager == null)
        {
            // example: /Proposal[ID='123']/ResourceTaskAssignment
            joinCondition = new JoinCondition();
            joinCondition.setTableName(getTableName());

            StringBuffer where = new StringBuffer();
            where.append(getFilter());
            if (condition != null)
            {
                where.append(" AND ");
                where.append(condition);
            }

            joinCondition.setCondition(where.toString());

        }
        else if (manager instanceof ProfileManager)
        {
            // example: /Proposal/ResourceTaskAssignment/Profile
            // NOTE: NAME_RESOURCE_ID is used instead of NAME_PROFILE_ID
            // because they are very evil in DB team...
            joinCondition = JoinCondition.createSubSelect(
                ProfileManager.NAME_PROFILE_ID, NAME_RESOURCE_ID, TABLE_NAME,
                condition);
        }
        else if (manager instanceof TaskAssignmentManager)
        {
            // example: /Proposal/ResourceTaskAssignment/TaskAssignment
            joinCondition = JoinCondition.createSubSelect(
                TaskAssignmentManager.NAME_RES_ASSIGNMENT_ID,
                NAME_RES_ASSIGNMENT_ID, TABLE_NAME, condition);
        }
        else if (manager instanceof ResourceManager)
        {
            // example: /Proposal/ResourceTaskAssignment/Resource
            joinCondition = JoinCondition.createSubSelect(
                ResourceManager.NAME_RESOURCE_ID, NAME_RESOURCE_ID, TABLE_NAME,
                condition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }
        return joinCondition;
    }

    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof WbsManager)
        {
            // example: /Proposal[ResourceTaskAssignment[ID='123']]
            joinCondition = JoinCondition.createSubSelect(
                WbsManager.NAME_ELEMENT_ID, NAME_ELEMENT_ID, TABLE_NAME,
                condition);
        }
        else if (manager instanceof ScopeManager)
        {
            // example: /Defect[ResourceTaskAssignment[ID='123']]
            joinCondition = JoinCondition.createSubSelect(
                ScopeManager.NAME_ELEMENT_ID, NAME_ELEMENT_ID, TABLE_NAME,
                condition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof ResourceManager)
        {
            // where clause of the join condition
            StringBuffer condition = new StringBuffer();
            condition.append(ResourceManager.NAME_RESOURCE_ID);
            condition.append(" = ?");

            joinCondition = new JoinCondition();
            if (extraWhere != null && extraWhere.length() > 0)
            {
                condition.append(" AND ");
                condition.append(extraWhere);
            }
            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(condition.toString());
        }
        else if (manager instanceof ProfileManager)
        {
            StringBuffer condition = new StringBuffer();
            condition.append(NAME_RESOURCE_ID);
            condition.append(" = ?");
            joinCondition = new JoinCondition();
            if (extraWhere != null && extraWhere.length() > 0)
            {
                condition.append(joinCondition.getCondition());
                condition.append(extraWhere);
            }
            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(condition.toString());
        }
        else if (manager instanceof ScopeManager)
        {
            StringBuffer condition = new StringBuffer();
            condition.append(NAME_ELEMENT_ID);
            condition.append(" = ?");

            joinCondition = new JoinCondition();
            if (extraWhere != null && extraWhere.length() > 0)
            {
                condition.append(" AND ");
                condition.append(extraWhere);
            }
            joinCondition.setCondition(condition.toString());
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof WbsManager)
        {
            StringBuffer condition = new StringBuffer();
            condition.append(NAME_ELEMENT_ID);
            condition.append(" = ?");

            joinCondition = new JoinCondition();
            if (extraWhere != null && extraWhere.length() > 0)
            {
                condition.append(" AND ");
                condition.append(extraWhere);
            }
            joinCondition.setCondition(condition.toString());
            joinCondition.setTableName(getTableName());
        }
        else if (manager instanceof ResourceTaskAssignmentManager)
        {
            // called on create, when needs to know ID after SP_SEARCH_2
            StringBuffer condition = new StringBuffer();

            condition.append(NAME_ELEMENT_ID);
            condition.append(" = ? AND ");
            condition.append(NAME_RESOURCE_ID);
            condition.append(" = ?");

            joinCondition = new JoinCondition();
            if (extraWhere != null && extraWhere.length() > 0)
            {
                condition.append(" AND ");
                condition.append(extraWhere);
            }
            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(condition.toString());
        }
        else if (manager instanceof TaskAssignmentManager)
        {
            StringBuffer subSelectCondition = new StringBuffer();
            subSelectCondition
                .append(TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID);
            subSelectCondition.append(" = ?");

            SqlBuffer condition = new SqlBuffer();
            condition
                .appendSubSelect(NAME_RES_ASSIGNMENT_ID,
                    TaskAssignmentManager.NAME_RES_ASSIGNMENT_ID,
                    TaskAssignmentManager.TABLE_NAME, subSelectCondition
                        .toString());

            joinCondition = new JoinCondition();
            if (extraWhere != null && extraWhere.length() > 0)
            {
                condition.append(" AND ");
                condition.append(extraWhere);
            }
            joinCondition.setTableName(getTableName());
            joinCondition.setCondition(condition.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    protected RPMObject createContainer(int typeId)
    {
        return new ResourceTaskAssignment();
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        ResourceTaskAssignment assignment = (ResourceTaskAssignment) rpmObject;

        // Mandatory
        loadSecurityRole(assignment, messageContext, relatedFields, index,
            isDelta);

        if (rpmScope != null)
        {
            // TODO: Always need to load Parent,
            // so probably we should throw if scope.getParent() == null
            // because scope is used to identify parents type (WorkElement or
            // ScopeElement)
            if (rpmScope.getParent() != null)
            {
                loadParent(assignment, rpmScope, messageContext, relatedFields,
                    index, isDelta);
            }

            if (rpmScope instanceof ResourceTaskAssignmentScope)
            {
                ResourceTaskAssignmentScope scope = (ResourceTaskAssignmentScope) rpmScope;

                if (ResourceType.Profile.equals(assignment.getResourceType()))
                {
                    loadProfile(assignment, scope, messageContext,
                        relatedFields, index, isDelta);
                }
                else
                {
                    loadResource(assignment, scope, messageContext,
                        relatedFields, index, isDelta);
                }

                loadAssignmentStatus(assignment, scope, messageContext,
                    relatedFields, index, isDelta);
                loadTaskAssignment(assignment, scope, messageContext, isDelta);

            }
        }

        return rpmObject;
    }

    private void loadParent(ResourceTaskAssignment assignment,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String parentID = (String) relatedFields.get(index, ID_ELEMENT_ID);
        int parentTypeID = ((Integer) relatedFields.get(index, ID_ELEMENT_TYPE))
            .intValue();
        RPMObject parentContainer = WbsUtil.getContainerByTypeID(parentTypeID);

        if (parentID != null)
        {
            RPMObject parent = null;

            if (parentContainer instanceof ScopeElement)
            {
                ScopeManager manager = (ScopeManager) getManagerInstance(ScopeElement.class
                    .getName());
                parentContainer.setID(parentID);

                RPMObjectScope parentScope = (rpmScope != null) ? rpmScope
                    .getParent() : null;
                parent = manager.loadByPrimaryKey(parentContainer, parentScope,
                    messageContext, isDelta);
            }
            else
            {

                WbsManager manager = (WbsManager) getManagerInstance(WorkElement.class
                    .getName());
                if (parentContainer == null)
                {
                    parentContainer = new WorkElement();
                }
                parentContainer.setID(parentID);

                RPMObjectScope parentScope = (rpmScope != null) ? rpmScope
                    .getParent() : null;
                parent = manager.loadByPrimaryKey(parentContainer, parentScope,
                    messageContext, isDelta);
            }
            if (isDelta)
            {
                assignment.deltaParent(parent);
            }
            else
            {
                assignment.setParent(parent);
            }
        }
        else
        {
            ExceptionUtil
                .handleNoId(this, assignment, ExceptionUtil.ELEMENT_ID);
        }
    }

    private void loadTaskAssignment(ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (scope.getTaskAssignment() != null)
        {
            TaskAssignmentManager manager = (TaskAssignmentManager) getManagerInstance(TaskAssignment.class
                .getName());
            Object[] args = new Object[]
            {
                assignment.getID()
            };
            ArrayList temp = manager.loadByForeignKey(new RPMObject[]
            {
                assignment.getTaskAssignment()
            }, scope.getTaskAssignment(), messageContext, this, args, null,
                null, isDelta);

            // TODO Validate that a ResourceTaskAssignmet can have only one
            // TaskAssignment
            TaskAssignment taskAssignment = null;
            if (temp != null)
            {
                taskAssignment = (TaskAssignment) temp.get(0);
                taskAssignment.setParent(assignment);
                assignment.setTaskAssignment(taskAssignment);
            }
        }
    }

    private void loadProfile(ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (scope.getProfile() != null)
        {
            String profileId = (String) relatedFields
                .get(index, ID_RESOURCE_ID);
            if (profileId != null)
            {
                Profile profile = new Profile();
                profile.setID(profileId);
                ProfileManager manager = (ProfileManager) getManagerInstance(Profile.class
                    .getName());
                profile = (Profile) manager.loadByPrimaryKey(profile, scope
                    .getProfile(), messageContext, isDelta);
                if (isDelta)
                {
                    assignment.deltaProfile(profile);
                }
                else
                {
                    assignment.setProfile(profile);
                }
            }
        }
    }

    private void loadResource(ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        String resourceId = (String) relatedFields.get(index, ID_RESOURCE_ID);
        if (resourceId != null)
        {
            ResourceManager resourceManager = (ResourceManager) getManagerInstance(Resource.class
                .getName());
            Resource resource = new Resource();
            resource.setID(resourceId);
            RPMObjectScope resourceScope = (scope != null) ? scope
                .getResource() : null;
            resource = (Resource) resourceManager.loadByPrimaryKey(resource,    
                resourceScope, messageContext, isDelta);
            if (isDelta)
            {
                assignment.deltaResource(resource);
            }
            else
            {
                assignment.setResource(resource);
            }
        }
        else
        {
            ExceptionUtil.handleNoId(this, assignment,
                ExceptionUtil.RESOURCE_ID);
        }
    }

    private void loadAssignmentStatus(ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (scope.isAssignmentStatus())
        {
            String assignmentStatusId = (String) relatedFields.get(index,
                ID_ASSIGN_STATUS);
            if (assignmentStatusId != null && !assignmentStatusId.equals("0"))
            {
                AssignmentStatus assignmentStatus = new AssignmentStatus();
                assignmentStatus.setID(assignmentStatusId);
                DatafieldManager manager = (DatafieldManager) getManagerInstance(AssignmentStatus.class
                    .getName());
                assignmentStatus = (AssignmentStatus) manager.loadByPrimaryKey(
                    assignmentStatus, null, messageContext, isDelta);
                if (isDelta)
                {
                    assignment.deltaAssignmentStatus(assignmentStatus);
                }
                else
                {
                    assignment.setAssignmentStatus(assignmentStatus);
                }
            }
        }
    }

    private void loadSecurityRole(ResourceTaskAssignment assignment,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        String securityId = (String) relatedFields.get(index, ID_SECURITY_ID);
        if (securityId != null)
        {
            SecurityRole securityRole = new SecurityRole();
            securityRole.setID(securityId.trim());
            if (isDelta)
            {
                assignment.deltaSecurityRole(securityRole);
            }
            else
            {
                SecurityRoleManager manager = (SecurityRoleManager) getManagerInstance(SecurityRole.class);
                securityRole = (SecurityRole) manager.loadByPrimaryKey(
                    securityRole, null, messageContext, isDelta);
                assignment.setSecurityRole(securityRole);
            }
        }
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws SQLException
    {
        ResourceTaskAssignment pObject = (ResourceTaskAssignment) rpmObject;
        pObject.setID(rs.getString(ID_RES_ASSIGNMENT_ID));

        ResourceType resourceType = decodeResourceType(rs
            .getString(ID_RESOURCE_TYPE));

        if (isDelta)
        {
            pObject.deltaResourceType(resourceType);
        }
        else
        {
            pObject.setResourceType(resourceType);
        }

        return pObject;
    }

    private ResourceType decodeResourceType(String resourceType)
    {
        // TODO add exception later
        if (resourceType != null)
        {
            if (RESOURCE_TYPE_RESOURCE.equals(resourceType))
            {
                return ResourceType.Resource;
            }
            else if (RESOURCE_TYPE_PROFILE.equals(resourceType))
            {
                return ResourceType.Profile;
            }
        }
        return null;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String assStatusId = resultSet.getString(ID_ASSIGN_STATUS);
        relatedFields.put(index, ID_ASSIGN_STATUS, assStatusId);

        String parentId = resultSet.getString(ID_ELEMENT_ID);
        relatedFields.put(index, ID_ELEMENT_ID, parentId);

        Integer parentType = new Integer(resultSet.getInt(ID_ELEMENT_TYPE));
        relatedFields.put(index, ID_ELEMENT_TYPE, parentType);

        String profileId = resultSet.getString(ID_PROFILE_ID);
        relatedFields.put(index, ID_PROFILE_ID, profileId);

        String resId = resultSet.getString(ID_RESOURCE_ID);
        relatedFields.put(index, ID_RESOURCE_ID, resId);

        String workLocationId = resultSet.getString(ID_WORK_LOCATION);
        relatedFields.put(index, ID_WORK_LOCATION, workLocationId);

        String securityId = resultSet.getString(ID_SECURITY_ID);
        relatedFields.put(index, ID_SECURITY_ID, securityId);

        return relatedFields;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        ResourceTaskAssignment assignment = (ResourceTaskAssignment) rpmObject;

        // There are three cases where one would need to create a
        // ResourceTaskAssignment
        // 1- Profile affected to a task
        // 2- Resource affected to a task
        // 3- Resource assigned to a task

        // NOTE:
        // Profile kind of assignment is done in updateObject,
        // because a scope is needed

        if (assignment.getResource() != null)
        {

            assignment.setResourceType(ResourceType.Resource);

            // simple hack to distinguish Resource Affect vs Resource Assign
            if (assignment.getTaskAssignment() != null)
            {
                TaskAssignment taskAssignment = assignment.getTaskAssignment();
                // making sure the parent field is there
                // it is important for creation process (see create in
                // TaskAssigmentManager)
                taskAssignment.setParent(assignment);
                // Affect to Project

                // this call will create both ResourceTaskAssignment and
                // TaskAssignment
                // we retrieve ResourceTaskAssignment.ID now, but we do not
                // touch
                // TaskAssignment until its createObject method, because if we
                // set it's ID now,
                // then when isValid will be called on TaskAssignment it will be
                // treated
                // as already existing object (ID != null) which is not what we
                // want

                SP_SEARCH_2(assignment, true, messageContext);
                getIDAfterSpSearch2(assignment, messageContext);

                // when work is assigned to ScopeElements, the ScopeElement is
                // converted
                // to its Task equivalent: eg. an Action becomes an ActionTask
                if (assignment.getParent() instanceof AbstractScope)
                {
                    ((AbstractScope) assignment.getParent())
                        .setTask(new Boolean(true));
                }
            }
            else
            {
                // Assign as Patricipant
                if (assignment.getParent() instanceof WorkElement)
                {
                    assignment.setID(SP_A_TSKRES(assignment, messageContext));
                }
                else if (assignment.getParent() instanceof ScopeElement)
                {
                    assignment.setID(SP_A_CRIRES(assignment, messageContext));
                }
            }
        }
        else if (assignment.getProfile() != null)
        {
            assignment.setResourceType(ResourceType.Profile);
        }
    }

    private void getIDAfterSpSearch2(
            ResourceTaskAssignment resourceTaskAssignment,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        // Assuming:
        // assignment != null
        // assignment.getParent() != null
        // assignment.getResource() != null or assignment.getProfile() != null

        // first, getting resourceTaskAssignment.ID
        String parentID = resourceTaskAssignment.getParent().getID();
        String resourceID = resourceTaskAssignment.getResource() != null
                ? resourceTaskAssignment.getResource().getID()
                : resourceTaskAssignment.getProfile().getID();
        ArrayList reloadedObjects = loadByForeignKeyWithSpecifiedParent(
            resourceTaskAssignment.getParent(), null, null, messageContext,
            this, new Object[]
            {
                    parentID, resourceID
            }, null, null, false);

        if (reloadedObjects == null)
        {
            throw new RPMException(
                "ResourceTaskAssignmentManager.createObject : ResourceTaskAssignment object "
                        + "was not created by the stored procedure.");
        }
        else if (reloadedObjects.size() == 1)
        {
            ResourceTaskAssignment object = (ResourceTaskAssignment) reloadedObjects
                .get(0);
            resourceTaskAssignment.setID(object.getID());
        }
        else
        {
            throw new RPMException(
                "ResourceTaskAssignmentManager.createObject : One ResourceTaskAssignment "
                        + "object expected, instead got "
                        + reloadedObjects.size());
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        ResourceTaskAssignment assignment = (ResourceTaskAssignment) rpmObject;

        // Note, that in case of Profile this method CREATES
        // ResourceTaskAssignment

        if (rpmScope != null)
        {
            ResourceTaskAssignmentScope scope = (ResourceTaskAssignmentScope) rpmScope;

            // Profile has to come before taskAssignment!!!
            if (scope.getProfile() != null
                    && ResourceType.Profile
                        .equals(assignment.getResourceType()))
            {
                ProfileManager manager = (ProfileManager) getManagerInstance(Profile.class
                    .getName());
                Profile profileWithID = null;
                if (assignment.getProfile() != null
                        && assignment.getProfile().getID() == null)
                {
                    Profile profile = assignment.getProfile();
                    profile.setID(SP_I_PROFILE(assignment, messageContext));

                    // set the IDs back in the assignments (roles and skills)
                    if (profile.getProfileCompetencyAssignments() != null)
                    {
                        ProfileCompetencyAssignment[] assignments = profile
                            .getProfileCompetencyAssignments();

                        if (assignments != null && assignments.length > 0)
                        {
                            ProfileCompetencyAssignment pca = null;
                            for (int i = 0; i < assignments.length; i++)
                            {
                                pca = assignments[i];
                                pca.setID(profile.getID());
                            }
                        }
                    }

                    if (profile.getProfileSkillAssignments() != null)
                    {
                        ProfileSkillAssignment[] assignments = profile
                            .getProfileSkillAssignments();
                        if (assignments != null && assignments.length > 0)
                        {
                            ProfileSkillAssignment psa = null;
                            for (int i = 0; i < assignments.length; i++)
                            {
                                psa = assignments[i];
                                psa.setID(profile.getID());
                            }
                        }
                    }
                    SP_SEARCH_2(assignment, true, messageContext);
                }
                else if (assignment.getProfile() != null)
                {
                    assignment.getProfile().setParent(assignment);
                    profileWithID = (Profile) manager.loadByPrimaryKey(
                        assignment.getProfile(), scope.getProfile(),
                        messageContext, false);
                    assignment.setProfile(profileWithID);
                    SP_SEARCH_1(assignment, messageContext);
                }
                // set assigment ID
                getIDAfterSpSearch2(assignment, messageContext);
            }

            if (scope.getTaskAssignment() != null
                    && assignment.getTaskAssignment() != null)
            {
                TaskAssignmentManager manager = (TaskAssignmentManager) getManagerInstance(TaskAssignment.class
                    .getName());
                // ensure the correct parent, otherwise the save will fail
                assignment.getTaskAssignment().setParent(assignment);

                manager.internalSave(assignment.getTaskAssignment(), scope
                    .getTaskAssignment(), messageContext);
            }
        }

        // we need to put this normal update after scope update
        // because scope update actually creates the object in case of Profile
        if (assignment.testSecurityRoleModified() && assignment.getID() != null)
        {
            SP_U_PARTICIPANT(assignment, messageContext);
        }

        return assignment;
    }

    public String[] executeSearch(ResourceTaskAssignment assignment, MessageContext messageContext) throws RPMException, SQLException
    {
    	return SP_SEARCH_2(assignment, false, messageContext);
    }
    
    /* 
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#delete(com.ibm.rpm.containers.RPMObject)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {

        if (rpmObject != null)
        {
            ResourceTaskAssignment resourceAssignment = (ResourceTaskAssignment) rpmObject;

            // we need all this data to determine proper deletion
            ResourceTaskAssignmentScope scope = new ResourceTaskAssignmentScope();
            scope.setTaskAssignment(new TaskAssignmentScope());
            scope.setProfile(new ProfileScope());
            scope.setResource(new ResourceScope());

            if (resourceAssignment.getParent() != null)
            {
                if (resourceAssignment.getParent() instanceof WorkElement)
                {
                    scope.setParent(new WorkElementScope());
                }
                else
                {
                    scope.setParent(new ScopeElementScope());
                }
            }

            /*
             * Even if we have Resource in scope, the rta is in cache, so it 
             * won't be reloaded. The problem is that it has not loaded
             * its resource, in certain code paths, so we remove it from the 
             * cache first, then reload the object with the real data.
             */
            messageContext.getCache().removeContainerFromCache(resourceAssignment);
            resourceAssignment = (ResourceTaskAssignment) loadByPrimaryKey(
                resourceAssignment, scope, messageContext, false);

            // if it was already deleted - just return
            if (resourceAssignment == null)
                return;

            if (ResourceType.Resource.equals(resourceAssignment
                .getResourceType()))
            {
                if (resourceAssignment.getTaskAssignment() != null)
                {
                    // delete the taskAssignment and rta
                    SP_D_TSKRES_ACT(resourceAssignment.getTaskAssignment(),
                        messageContext);

                    /*
                     * The SP_D_TSKRES_ACT SP has changed -- entries are not erased from the DB anymore
                     * by calling it, so the following would always return true.
                     */
//                    if (assignmentExistsInDB(resourceAssignment, messageContext))
//                    {
//                        SP_D_TSKRES_ACT(resourceAssignment, messageContext);
//                    }
                }
                else
                {
                    // delete the resourceTaskAssignment
                    SP_D_TSKRES_ACT(resourceAssignment, messageContext);
                }

            }
            else if (ResourceType.Profile.equals(resourceAssignment
                .getResourceType()))
            {

                // Profile assignment
                SP_D_TSKRES_ACT(resourceAssignment, messageContext);

            }
            else
            {

                ExceptionUtil.handleException(
                    ResourceTaskAssignmentManager.class, new RPMException(
                        "Unsupported ResourceType: "
                                + resourceAssignment.getResourceType()));
            }
        }
    }

    public ResourceTaskAssignment replaceResourceWithProfile(
            ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceTaskAssignment result = null;
        
        if (CheckpointCaller.isValidDelete(assignment, messageContext)
                && CheckpointCaller.canDelete(assignment, messageContext)
                && ResourceTaskAssignmentCheckpoint.getInstance()
                    .canReplaceResourceWithProfile(assignment, rtaScope,
                        messageContext))
        {
            SP_D_TSKRES_ACT_RES2PROF(assignment, messageContext);

            messageContext.getCache().removeContainerFromCache(assignment);

            result = (ResourceTaskAssignment) loadByPrimaryKey(
                assignment, rtaScope, messageContext, false);
        }
        
        return result;
    }

    public ResourceTaskAssignment replaceProfileWithResource(
            ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope, Resource resource,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        CheckpointCaller.isValidSave(assignment.getProfile(), null,
            messageContext);
        if (messageContext.isSuccessful()
                && CheckpointCaller.canSave(assignment.getProfile(), null,
                    messageContext))
        {
            ResourceCheckpoint.getInstance().isValidSave(resource, null,
                messageContext);
            if (messageContext.isSuccessful()
                    && ResourceCheckpoint.getInstance().canSave(resource, null,
                        messageContext)
                    && ResourceTaskAssignmentCheckpoint.getInstance()
                        .canReplaceProfileWithResource(assignment, rtaScope,
                            resource, messageContext))
            {
                SP_A_RESTOPROF(assignment, resource, messageContext);

                ResourceTaskAssignment resourceTaskAssignment = (ResourceTaskAssignment) loadByPrimaryKey(
                    assignment, rtaScope, messageContext, false);
                return resourceTaskAssignment;
            }
        }

        return null;
    }

    public ResourceTaskAssignment copyAssignmentProposedToPlan(
            ResourceTaskAssignment resourceTaskAssignment,
            ResourceTaskAssignmentScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ResourceTaskAssignment reloadedElement = null;

        WbsMethodCheckpoint.getInstance().validateCopyAssignmentProposedToPlan(
            resourceTaskAssignment, messageContext);

        if (messageContext.isSuccessful()
                && WbsMethodCheckpoint.getInstance()
                    .canCopyAssignmentProposedToPlan(resourceTaskAssignment,
                        scope, messageContext))
        {
            SP_COMMIT_ASSIGN(resourceTaskAssignment, messageContext);
            reloadedElement = (ResourceTaskAssignment) load(
                resourceTaskAssignment, scope, messageContext);
        }

        return reloadedElement;
    }

    public String getFilter()
    {
        // filter out the records that are associated with deleted resources and
        // profiles

        // (
        // in (select element_id from tmt_wbs where rec_status != 'D')
        // OR in (select element_id from tmt_wbs where rec_status != 'D')
        // )
        // and (resource_id in (select resource_id from resources where
        // rec_status != 'D')
        // or resource_id in (select profile_id from PROFILES where rec_status
        // != 'D'))

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendOpenParenthesis();
        stmt.appendSubSelect(NAME_ELEMENT_ID, WbsManager.NAME_ELEMENT_ID,
            WbsManager.TABLE_NAME, WbsManager.NAME_REC_STATUS + " != 'D'");
        stmt.appendCloseParenthesis();

        stmt.appendOrSubSelect(NAME_ELEMENT_ID, ScopeManager.NAME_ELEMENT_ID,
            ScopeManager.TABLE_NAME, ScopeManager.NAME_REC_STATUS + " != 'D'");
        stmt.appendCloseParenthesis();
        stmt.appendCloseParenthesis();
        stmt.appendAnd();
        stmt.appendOpenParenthesis();
        stmt.appendSubSelect(NAME_RESOURCE_ID,
            ResourceManager.NAME_RESOURCE_ID, ResourceManager.TABLE_NAME,
            ResourceManager.NAME_REC_STATUS + " != 'D'");
        stmt.appendCloseParenthesis();
        stmt.appendOrSubSelect(NAME_RESOURCE_ID,
            ProfileManager.NAME_PROFILE_ID, ProfileManager.TABLE_NAME,
            ProfileManager.NAME_REC_STATUS + " != 'D'");
        stmt.appendCloseParenthesis();
        stmt.appendCloseParenthesis();

        return stmt.toString();
    }
    
   private String SP_A_TSKRES(ResourceTaskAssignment assignment,
            MessageContext msgContext) throws SQLException, RPMException
    {
        // assert assignment.getParent() instanceof ScopeElement

        String name = "SP_A_TSKRES";
        Object args[] = new Object[3];

        RPMObject parentWorkElement = assignment.getParent();
        RPMObject resource = assignment.getResource();

        args[0] = parentWorkElement.getID();
        args[1] = resource.getID();
        args[2] = getUser(msgContext).getID();

        return executeProcedure(msgContext, name, args);
    }

    private String SP_A_CRIRES(ResourceTaskAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        // assert assignment.getResource() != null
        // assert assignment.getParent() != null
        // assert assignment.getParent() instanceof ScopeElement

        final String procedureName = "SP_A_CRIRES";

        Object args[] = new Object[3];

        // ScopeElement id : SC_WCHAR
        args[0] = assignment.getParent().getID();

        // res_id : SC_WCHAR
        args[1] = assignment.getResource().getID();

        args[2] = getUser(messageContext).getID();

        String objectId = executeProcedure(messageContext, procedureName, args);

        return objectId;
    }

    private void SP_U_PARTICIPANT(ResourceTaskAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        /**
         * SP_U_PARTICIPANT assign_id='2492B3F20EEA4505B868C36B105EE517' //
         * WorkElement ID ??? security_id=null planned_start='2006-02-01'
         * planned_finish='2006-02-09 17:00:00.0' delay=0 units=0 work_percent=0
         * actual=0 res_id='PMO_SUPERVISOR'
         */
        String name = "SP_U_PARTICIPANT";
        Object args[] = new Object[9];
        args[0] = assignment.getID();

        if (assignment.getSecurityRole() != null)
            args[1] = assignment.getSecurityRole().getID();

        args[2] = null;
        args[3] = null;
        args[4] = new Integer(0);
        args[5] = new Integer(0);
        args[6] = new Integer(0);
        args[7] = new Integer(0);
        args[8] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_D_TSKRES_ACT(ResourceTaskAssignment assignment,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_TSKRES_ACT";
        Object args[] = new Object[5];

        args[0] = assignment.getID();

        // TODO confirm the value to send
        // 3 options were found
        // 0 remove Remaining Affected work
        // 2 Replace Resource With Profile
        // 3 Remove Remaining Assigned And Affected Work
        args[1] = new Integer(3);

        if (assignment.getResource() != null)
        {
            args[2] = assignment.getResource().getID();
        }
        else if (assignment.getProfile() != null)
        {
            args[2] = assignment.getProfile().getID();
        }
        else
        {
            args[2] = null;
        }

        args[3] = getUser(messageContext).getID();

        // TODO confirm the value to send
        // 1 check for actual
        // 0 delete assignment
        args[4] = new Integer(1);

        executeProcedure(messageContext, name, args);
    }

    private void SP_D_TSKRES_ACT(TaskAssignment assignment,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_TSKRES_ACT";
        Object args[] = new Object[5];

        ResourceTaskAssignment resourceTaskAssignment = (ResourceTaskAssignment) assignment
            .getParent();
        RPMObject resource = resourceTaskAssignment.getResource();

        args[0] = assignment.getID();

        // TODO confirm the value to send
        // 3 options were found
        // 0 remove Remaining Affected work
        // 2 Replace Resource With Profile
        // 3 Remove Remaining Assigned And Affected Work

        // if the RTA was created from a GenericProject, use bitflag 3
        if (resourceTaskAssignment.getParent() instanceof GenericProject)
        {
            args[1] = new Integer(3);
        }
        else
        {
            args[1] = new Integer(0);
        }

        args[2] = resource.getID();
        args[3] = getUser(messageContext).getID();

        // TODO confirm the value to send
        // 2 options were found
        // 1 check for actual
        // 0 delete assignment
        args[4] = new Integer(1);

        executeProcedure(messageContext, name, args);
    }

    /**
     * Delete the resource task assignment and replace it with a generic profile.
     */
    private void SP_D_TSKRES_ACT_RES2PROF(ResourceTaskAssignment assignment,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_TSKRES_ACT";
        Object args[] = new Object[5];

        if (assignment.getTaskAssignment() != null)
        {
            args[0] = assignment.getTaskAssignment().getID();
        }

        args[1] = new Integer(1); // means replace with profile

        if (assignment.getResource() != null)
        {
            args[2] = assignment.getResource().getID();
        }
        // else if (assignment.getProfile() != null)
        // {
        // args[2] = assignment.getProfile().getID();
        // }
        // else
        // {
        // args[2] = null;
        // }

        args[3] = getUser(messageContext).getID();

        // 1 check for actual
        // 0 delete assignment
        args[4] = new Integer(1);

        executeProcedure(messageContext, name, args);
    }

    // TODO: check when this SP has to be called and call it
    private void SP_D_PRJRES(ResourceTaskAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_D_PRJRES";

        Object args[] = new Object[7];

        // SP_D_PRJRES id='142A9DE43D4B46FE88272049CA02F7A2' bitflag=112
        // res_id='91665B7650D24C41B7BD6FAF04000BAF' plan_id=null sched_id=null
        // participant_id=null rec_user='PMO_SUPERVISOR'

        // id : SC_WCHAR (project_id )
        args[0] = assignment.getParent().getID();

        // bitflag : SC_LONG_PARAM (delete or replace option )
        // TODO resolve magic number 112
        // There are several delete options, see the SP documentation (re-docs)
        args[1] = new Long(112);

        // res_id : SC_WCHAR (deleted resource_id )
        args[2] = assignment.getResource().getID();

        // plan_id : SC_WCHAR (resource_id used to replace planned assignments
        // (null for profile) )
        args[3] = null;

        // sched_id : SC_WCHAR (resource_id used to replace sched assignments
        // (null for profile) )
        args[4] = null;

        // participant_id : SC_WCHAR (resource_id used to replace without effort
        // assignments )
        args[5] = null;

        args[6] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    private String SP_I_PROFILE(ResourceTaskAssignment parent,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Profile profile = parent.getProfile();
        final String procedureName = "SP_I_PROFILE";
        Object args[] = new Object[7];

        RPMObject grandparent = null;
        if (parent.getParent() instanceof WorkElement)
        {
            grandparent = parent.getParent();
        }
        else
        {
            grandparent = parent.getParent();
        }

        if (grandparent != null)
        {
            if (grandparent instanceof WorkElement
                    && ((WorkElement) grandparent).getContainingProject() != null)
            {
                args[0] = ((WorkElement) grandparent).getContainingProject()
                    .getID();
            }
            else if (grandparent instanceof ScopeElement
                    && ((ScopeElement) grandparent).getProject() != null)
            {
                args[0] = ((ScopeElement) grandparent).getProject().getID();
            }
        }

        args[1] = ProfileManager.retrieveCompetenciesID(profile);
        args[2] = ProfileManager.retrieveSkillsID(profile);

        if (profile.getOrganizational() != null)
        {
            args[3] = profile.getOrganizational().getID();
        }
        else
        {
            args[3] = null;
        }

        if (profile.getGeographical() != null)
        {
            args[4] = profile.getGeographical().getID();
        }
        else
        {
            args[4] = null;
        }

        if (profile.getPool() != null)
        {
            args[5] = profile.getPool().getID();
        }
        else
        {
            args[5] = null;
        }

        args[6] = getUser(messageContext).getID();
        return executeProcedure(messageContext, procedureName, args);
    }
    
    private String[] SP_SEARCH_2(ResourceTaskAssignment assignment,  boolean isAssign, MessageContext messageContext) throws SQLException, RPMException
    {
    	String returnIDs[] = null;
    	Object args[] = new Object[27];
    	Object[] contourValues;
    	Integer plannedSchedule = null;
    	String fName = null;
    	String competencies = null;
        String skills = null;
    	String ID = null;
        String searchOrAssignID = null;
        Long parentType = new Long(0);
        String name = "SP_SEARCH_2";
    	
        if (ResourceType.Profile.equals(assignment.getResourceType()))
        {
        	if(isAssign)
        	{
        		ID = assignment.getProfile().getID();
        	}
            competencies = ProfileManager.retrieveCompetenciesID(assignment
                .getProfile());
            skills = ProfileManager.retrieveSkillsID(assignment.getProfile());
            plannedSchedule = new Integer(12);
        }
        else
        {
        	if(isAssign)
        	{
        		ID = assignment.getResource().getID();
        	}
        	else
        	{
        		fName = assignment.getResource().getFullName();
        	}
            plannedSchedule = new Integer(16);
        }
    	
        if(isAssign)
    	{
        	searchOrAssignID = assignment.getParent().getID();
        	parentType = new Long(WbsUtil.getSpSearch2Type(assignment.getParent()));
    	}
    	else
    	{
    		if(assignment.getParent() instanceof WorkElement)
    		{
    			searchOrAssignID = ((WorkElement)assignment.getParent()).getContainingProject().getID();
    		}
    		else
    		{
    			searchOrAssignID = ((ScopeElement)assignment.getParent()).getProject().getID();
    		}
    	}

        args[0] = searchOrAssignID;
        contourValues = WbsUtil.getContourValues(assignment);
        args[1] = contourValues[0] == null ? null : DateUtil
        		.convertToStringWithTimeZero((Calendar)contourValues[0]); // StartDate
        args[2] = contourValues[1] == null ? null : DateUtil
        		.convertToStringWithTimeZero((Calendar)contourValues[1]); // FinishDate 
        args[3] = contourValues[2]; //Effort 
        args[4] = contourValues[3]; //Work
        args[5] = contourValues[4]; //BitFlag
        args[6] = contourValues[5]; //Unit of Effort
        args[7] = contourValues[6]; //Unit of Work
        args[8] = contourValues[7]; //Contour type
        args[9] = contourValues[8]; // Occurrences
        args[10] = plannedSchedule; // Considered planned schedule
        args[11] = fName; // FName
        args[12] = null; // Cost
        args[13] = null; // OrgId
        args[14] = null; // Geo_id
        args[15] = ID; // Pool_id
        // note that the RPM client always sends the default WorkLocation
        args[16] = new Integer(DEFAULT_WORK_LOCATION);
        args[17] = competencies; // roles
        args[18] = skills; // skill
        args[19] = null; // shw_role
        args[20] = null; // shw_skill
        // TODO resolve magic number 50
        args[21] = new Integer(50); // max_rows
        args[22] = new Integer(0); // update
        args[23] = new Integer(0); // work_type
        args[24] = parentType; // type
        args[25] = null; // last_id
        args[26] = getUser(messageContext).getID();

        // SP_SEARCH_2 returns different results all the time, so we cannot use
        // the
        // framework method executeProcedure(messageContext, procedureName,
        // args);
        Connection c = null;
        ResultSet rs = null;
        try
        {
            c = Manager.getConnection(messageContext);
            rs = Manager.executeStoredProcedure(c, name, args, true,
                messageContext);

            // primitive SP_SEARCH_2 error retrieving
            if (rs.getMetaData().getColumnCount() > 0)
            {
                if (rs.getMetaData().getColumnCount() == 4)
                {
                    rs.next();
                    String sqlError = rs.getString(4);
                    if (sqlError != null && sqlError.startsWith("SQL"))
                    {
                        ExceptionUtil.handleException(this, new RPMException(
                            "SP_SEARCH_2: " + sqlError));
                    }
                }
                if ("ERROR".equals(rs.getMetaData().getColumnName(1)))
                {
                    rs.next();
                    ExceptionUtil.handleRPMException(this, rs.getInt(1));
                }
            }
            
            // Retrieveing rows for the search call
            if(!isAssign)
            {
            	ArrayList list = new ArrayList();
            	while(rs.next())
            	{
            		list.add(rs.getString(1));
            	}
            	if(!list.isEmpty())
            	{
            		returnIDs = new String[list.size()];
            		returnIDs = (String[])list.toArray(returnIDs);
            	}
            }
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        
    	return returnIDs;
    }

    private void SP_SEARCH_1(ResourceTaskAssignment assignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_SEARCH_1";

        RPMObject parent = assignment.getParent();

        Calendar startDate = null;
        if (assignment.getTaskAssignment() != null
                && assignment.getTaskAssignment().getAssignmentStartDate() != null)
        {
            startDate = assignment.getTaskAssignment().getAssignmentStartDate();
        }
        else if (parent instanceof WorkElement)
        {
            startDate = ((WorkElement) parent).getEstimatedStartDate();
        }
        else if (parent instanceof ScopeElement
                && ((ScopeElement) parent).getProject() != null)
        {
            startDate = ((ScopeElement) parent).getProject()
                .getEstimatedStartDate();
        }

        Integer effort = null;
        if (assignment.getTaskAssignment() != null
                && assignment.getTaskAssignment().getProposedDurationHours() != null
                && assignment.getTaskAssignment().getProposedDurationHours()
                    .doubleValue() > 0.0)
        {
            effort = DateUtil.convertHoursToMinutes(assignment
                .getTaskAssignment().getProposedDurationHours()); // Effort
        }
        else
        {
            effort = new Integer(480); // Default value like in the UI
        }

        String profResID = null;
        Integer plannedSchedule = null;
        // this padding is added at the end of the ID 0 for Resource, 1 for
        // Profile
        int padding = 0;
        if (assignment.getResourceType().equals(ResourceType.Profile))
        {
            padding = 1;
            profResID = assignment.getProfile().getID() + padding;
            plannedSchedule = new Integer(12);
        }
        else
        {
            padding = 0;
            profResID = assignment.getResource().getID() + padding;
            plannedSchedule = new Integer(16);
        }

        Object args[] = new Object[15];

        args[0] = parent.getID();
        args[1] = DateUtil.convertToStringWithTime(startDate);
        args[2] = null;
        args[3] = effort;

        args[4] = new Long(0); // work
        args[5] = new Long(0); // bit flag
        args[6] = new Long(0); // eunit
        args[7] = new Long(0); // unit
        args[8] = new Long(0); // contour
        args[9] = new Long(0); // occurence

        args[10] = plannedSchedule;
        args[11] = new Long(0); // workType

        args[12] = new Long(WbsUtil.getTypeIDByContainer(parent)); // type
        args[13] = profResID;
        args[14] = getUser(messageContext).getID();

        Connection c = null;
        ResultSet rs = null;
        try
        {
            c = Manager.getConnection(messageContext);
            rs = Manager.executeStoredProcedure(c, procedureName, args, true,
                messageContext);
            // rs.next();

            // primitive SP_SEARCH_2 error retrieving
            if (rs.getMetaData().getColumnCount() > 0)
            {
                if (rs.getMetaData().getColumnCount() == 4)
                {
                    String sqlError = rs.getString(4);
                    if (sqlError != null && sqlError.startsWith("SQL"))
                    {
                        ExceptionUtil.handleException(this, new RPMException(
                            "SP_SEARCH_1: " + sqlError));
                    }
                }
                if ("ERROR".equals(rs.getMetaData().getColumnName(1)))
                {
                    ExceptionUtil.handleRPMException(this, rs.getInt(1));
                }
            }
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
    }

    private void SP_A_RESTOPROF(ResourceTaskAssignment assignment,
            Resource resource, MessageContext messageContext)
            throws SQLException, RPMException
    {
        final String procedureName = "SP_A_RESTOPROF";
        Object args[] = new Object[15];

        RPMObject parent = assignment.getParent();

        args[0] = parent.getID();

        args[1] = assignment.getProfile().getID();
        args[2] = resource.getID();

        Calendar startDate = null;
        if (assignment.getTaskAssignment() != null
                && assignment.getTaskAssignment().getAssignmentStartDate() != null)
        {
            startDate = assignment.getTaskAssignment().getAssignmentStartDate();
        }
        else if (parent instanceof WorkElement)
        {
            startDate = ((WorkElement) parent).getEstimatedStartDate();
        }
        else if (parent instanceof ScopeElement
                && ((ScopeElement) parent).getProject() != null)
        {
            startDate = ((ScopeElement) parent).getProject()
                .getEstimatedStartDate();
        }

        args[3] = DateUtil.convertToStringWithTime(startDate); // StartDate
        args[4] = null;
        if (assignment.getTaskAssignment() != null
                && assignment.getTaskAssignment().getProposedDurationHours() != null
                && assignment.getTaskAssignment().getProposedDurationHours()
                    .doubleValue() > 0.0)
        {
            args[5] = DateUtil.convertHoursToMinutes(assignment
                .getTaskAssignment().getProposedDurationHours()); // Effort
        }
        else
        {
            args[5] = new Integer(480); // Default value like in the UI
        }
        args[6] = new Long(0);
        args[7] = new Long(0);
        args[8] = new Long(0);
        args[9] = new Long(0);
        args[10] = new Long(0);
        args[11] = new Long(0);
        args[12] = new Long(16);
        args[13] = new Long(1);
        args[14] = getUser(messageContext).getID();
        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_COMMIT_ASSIGN(
            ResourceTaskAssignment resourceTaskAssignment,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_COMMIT_ASSIGN";
        Object args[] = new Object[2];

        args[0] = resourceTaskAssignment.getTaskAssignment().getID(); // id
        args[1] = new Long(2); // bitflag

        executeProcedure(messageContext, procedureName, args);
    }

    private boolean assignmentExistsInDB(ResourceTaskAssignment assignment,
            MessageContext msgCtx) throws RPMException
    {
        boolean exists = false;

        if (assignment != null && assignment.getID() != null)
        {
            SqlBuffer stmt = new SqlBuffer();
            stmt.appendSelect();
            stmt.append(NAME_RES_ASSIGNMENT_ID);
            stmt.appendFrom(TABLE_NAME);
            stmt.appendWhere();
            stmt.appendEqualQuestionMark(NAME_RES_ASSIGNMENT_ID);

            List result = ManagerUtil.select(null, null, null, stmt,
                new Object[]
                {
                    assignment.getID(),

                }, msgCtx);

            Object value = null;

            // result should not be null or empty at this point
            if (result != null && result.size() > 0)
            {
                ArrayList columns = (ArrayList) result.get(0);
                value = columns.get(0);
            }

            if (value != null)
            {
                exists = true;
            }
        }

        return exists;
    }

}