/*
 * Created on Oct 25, 2005
 *
 */
package com.ibm.rpm.wbs.managers;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.applicationadministration.containers.AssetFinancialCategory;
import com.ibm.rpm.applicationadministration.containers.AssignmentStatus;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.DeliverableRTF;
import com.ibm.rpm.applicationadministration.containers.DeliverableState;
import com.ibm.rpm.applicationadministration.containers.MilestoneRTF;
import com.ibm.rpm.applicationadministration.containers.ProjectRTF;
import com.ibm.rpm.applicationadministration.containers.State;
import com.ibm.rpm.applicationadministration.containers.SummaryTaskRTF;
import com.ibm.rpm.applicationadministration.containers.TaskRTF;
import com.ibm.rpm.applicationadministration.containers.TaskState;
import com.ibm.rpm.applicationadministration.containers.WorkLocation;
import com.ibm.rpm.applicationadministration.containers.WorkProductRTF;
import com.ibm.rpm.applicationadministration.managers.AttributeManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.customfield.containers.CustomFieldCategory;
import com.ibm.rpm.customfield.managers.CustomFieldManager;
import com.ibm.rpm.framework.AbstractModuleManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.scorecard.containers.ScorecardFolder;
import com.ibm.rpm.scorecard.managers.ScorecardManager;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.WbsModule;
import com.ibm.rpm.wbs.scope.WbsModuleScope;

/**
 * @author karim
 *  
 */
public class WbsModuleManager extends AbstractModuleManager
{
    private static Log logger = LogFactory.getLog(WbsModuleManager.class);

    protected Log getLogger()
    {
        return logger;
    }

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        CONTAINERS.add(WbsModule.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                WbsModuleManager.class);
        }
    }

    {
        localContextName = ContextUtil.WBS_CONTEXT;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {
        RPMObject object = new WbsModule();
        return object;
    }

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition();
        }
        else if (manager instanceof AttributeManager)
        {
            if (localContextName.equalsIgnoreCase(contextName))
            {
                joinCondition = new JoinCondition();
            }
            else
            {//context not linked to this manager
                logger.error("WbsModuleManager.getJoinCondition: contextName "
                        + contextName + " is not linked with this manager.");
            }
        }
        else if (manager instanceof PortfoliosManager)
        {
                joinCondition = new JoinCondition();
        }
        else if (manager instanceof WbsManager)
        {
                joinCondition = new JoinCondition();
        }
        else if (manager instanceof DatafieldManager)
        {
                joinCondition = new JoinCondition();
        }
        else if (manager instanceof ScorecardManager)
        {
                joinCondition = new JoinCondition();
        }
        
        else if (manager instanceof CustomFieldManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.setCondition(CustomFieldManager
                    .createJoinCondition(localContextName));
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    protected void loadChildren(AbstractModule abstractModule,
            RPMObjectScope rpmScope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        WbsModule module = (WbsModule) abstractModule;
        WbsModuleScope scope = (WbsModuleScope) rpmScope;
        
    	DatafieldManager datafieldManager = (DatafieldManager)getManagerInstance(DatafieldCategory.class);

        if(scope!=null)
        {
            //loadPortfolios
            if (scope.getPortfolios() != null)
            {
    	        Portfolio[] portfolios = (Portfolio[]) loadElements(module, module.getPortfolios(),
            			scope.getPortfolios(),Portfolio.class, 
    					null, null, messageContext, isDelta );        
    	        module.setPortfolios(portfolios);
            }            
            //loadGenericProjects
            if (scope.getGenericProjects() != null)
            {
            	GenericProject[] genericProjects = (GenericProject[]) loadElements(module, module.getGenericProjects(),
            			scope.getGenericProjects(),GenericProject.class, 
    					null, null, messageContext, isDelta );        
                module.setGenericProjects(genericProjects);
            }
            //loadAttributes
            if (scope.getAttributeCategories() != null)
            {
                AttributeManager attributeManager = (AttributeManager) getManagerInstance(AttributeCategory.class);

                AttributeCategory[] attributeCategory = attributeManager
                    .loadAttributeCategories(abstractModule, module.getAttributeCategories(), scope
                        .getAttributeCategories(), this, localContextName,
                        messageContext, isDelta);
                module.setAttributeCategories(attributeCategory);
            }
            
            if(scope.isAssetFinancialCategories())
            {        	
            	AssetFinancialCategory[] assetFinancialCategories = (AssetFinancialCategory[]) datafieldManager.loadDatafields (
            	        module, module.getAssetFinancialCategories(),
            			AssetFinancialCategory.class,AssetFinancialCategory.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setAssetFinancialCategories(assetFinancialCategories);
            }
            if(scope.isAssignmentStatus())
            {        	
            	AssignmentStatus[] assignmentStatus = (AssignmentStatus[]) datafieldManager.loadDatafields ( 
            	        module, module.getAssignmentStatus(),
            			AssignmentStatus.class,AssignmentStatus.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setAssignmentStatus(assignmentStatus);
            }
            if(scope.isWorkLocations())
            {        	
            	WorkLocation[] workLocations = (WorkLocation[]) datafieldManager.loadDatafields ( 
            	        module, module.getWorkLocations(),
            			WorkLocation.class,WorkLocation.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setWorkLocations(workLocations);
            }
            if(scope.isStates())
            {        	
            	State[] states = (State[]) datafieldManager.loadDatafields ( 
            	        module, module.getStates(),
            			State.class,State.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setStates(states);
            }
            if(scope.isDeliverableStates())
            {        	
            	DeliverableState[] deliverableStates = (DeliverableState[]) datafieldManager.loadDatafields ( 
            	        module, module.getDeliverableStates(),
            			DeliverableState.class,DeliverableState.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setDeliverableStates(deliverableStates);
            }
            if(scope.isTaskStates())
            {        	
            	TaskState[] taskStates = (TaskState[]) datafieldManager.loadDatafields ( 
            	        module, module.getTaskStates(),
            			TaskState.class,TaskState.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setTaskStates(taskStates);
            }
            if(scope.isProjectRtfs())
            {        	
            	ProjectRTF[] projectRTFs = (ProjectRTF[]) datafieldManager.loadDatafields ( 
            	        module, module.getProjectRtfs(),
            			ProjectRTF.class,ProjectRTF.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setProjectRtfs(projectRTFs);
            }
            if(scope.isDeliverableRtfs())
            {        	
            	DeliverableRTF[] deliverableRTFs = (DeliverableRTF[]) datafieldManager.loadDatafields ( 
            	        module, module.getDeliverableRtfs(),
            			DeliverableRTF.class,DeliverableRTF.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setDeliverableRtfs(deliverableRTFs);
            }
            if(scope.isWorkProductRtfs())
            {        	
            	WorkProductRTF[] workProductRTFs = (WorkProductRTF[]) datafieldManager.loadDatafields ( 
            	        module, module.getWorkProductRtfs(),
            			WorkProductRTF.class,WorkProductRTF.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setWorkProductRtfs(workProductRTFs);
            }
            if(scope.isSummaryTaskRtfs())
            {        	
            	SummaryTaskRTF[] summaryTaskRTFs = (SummaryTaskRTF[]) datafieldManager.loadDatafields ( 
            	        module, module.getSummaryTaskRtfs(),
            			SummaryTaskRTF.class,SummaryTaskRTF.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setSummaryTaskRtfs(summaryTaskRTFs);
            }
            if(scope.isMilestoneRtfs())
            {        	
            	MilestoneRTF[] milestoneRTFs = (MilestoneRTF[]) datafieldManager.loadDatafields ( 
            	        module, module.getMilestoneRtfs(),
            			MilestoneRTF.class,MilestoneRTF.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setMilestoneRtfs(milestoneRTFs);
            }
            if(scope.isTaskRtfs())
            {        	
            	TaskRTF[] taskRTFs = (TaskRTF[]) datafieldManager.loadDatafields ( 
            	        module, module.getTaskRtfs(),
            			TaskRTF.class,TaskRTF.TYPE_ID,
    					this, localContextName,messageContext, isDelta);   	        	
            	module.setTaskRtfs(taskRTFs);
            }
            //loadScorecardFolder
            if (scope.getScorecardFolders() != null)
            {
            	String extraWhere = " TYPE_ID = "+ ScorecardFolder.TYPE_ID;
    	        ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) loadElements(
    	                module, module.getScorecardFolders(),
            			scope.getScorecardFolders(),ScorecardFolder.class, 
    					null, extraWhere, messageContext,
                    isDelta );        
                module.setScorecardFolders(scorecardFolders);
            }
            if (scope.getCustomFieldCategories() != null)
            {
                CustomFieldManager manager = (CustomFieldManager) RPMManagerFactory
                        .getInstance().getRPMObjectManager(
                                CustomFieldCategory.class);
                CustomFieldCategory[] categories = manager
                        .loadCustomFieldCategories(localContextName,
                                scope.getCustomFieldCategories(), messageContext);
                module.setCustomFieldCategories(categories);
            }
        }
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {

        WbsModuleScope scope = (WbsModuleScope) rpmScope;
        WbsModule module = (WbsModule) rpmObject;
        module.setContextName(localContextName);
        
        if(scope!=null)
        {
            if(scope.getPortfolios() != null)
            {
            	Portfolio[] portfolios = (Portfolio[]) saveElements(
            			module, module.getPortfolios(),
    					scope.getPortfolios(), messageContext,
    					Portfolio.class );
            	module.setPortfolios(portfolios);

            }
            
            if(scope.getGenericProjects() != null)
            {
            	GenericProject[] genericProjects = (GenericProject[]) saveElements(
            			module, module.getGenericProjects(),
    					scope.getGenericProjects(), messageContext,
    					GenericProject.class );
            	module.setGenericProjects(genericProjects);

            }

            if (scope.getAttributeCategories() != null)
            {
                AttributeCategory[] attributeCategory = (AttributeCategory[]) saveElements(
                    module, module.getAttributeCategories(), scope.getAttributeCategories(), messageContext,
                    AttributeCategory.class);
                module.setAttributeCategories(attributeCategory);
            }

            if(scope.getScorecardFolders() != null)
            {
            	ScorecardFolder[] scorecardFolders = (ScorecardFolder[]) saveElements(
            			module, module.getScorecardFolders(),
    					scope.getScorecardFolders(), messageContext,
    					ScorecardFolder.class );
            	module.setScorecardFolders(scorecardFolders);

            }

            if (scope.getCustomFieldCategories() != null)
            {
                CustomFieldCategory[] customFieldCategories = (CustomFieldCategory[])saveElements(
            			module, module.getCustomFieldCategories(),
    					scope.getCustomFieldCategories(), messageContext,
    					CustomFieldCategory.class );
            	module.setCustomFieldCategories(customFieldCategories);
            }
        }

        return module;
    }
}
