/*
 * Created on Feb 7, 2006
 */
package com.ibm.rpm.wbs.managers;

import java.util.Calendar;
import java.util.List;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ScopeWorkProduct;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Contour;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.EarnedValues;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.types.ContourType;

/**
 * Resusable utility code for all Wbs-related managers 
 * 
 * @author ikusakov
 */
public class WbsUtil
{
    public static final String EXPENSE_HOUR_PREFIX = "EXPENSE_HOUR";

    public static final String EXPENSE_LABOR_PREFIX = "EXPENSE_LABOR";

    public static final String PMO_PROJECT_TIMEPHASE_CALENDAR = "PMO_PROJECT_TIMEPHASE_CALENDAR__";

    public static final String EXPENSE_FULL_TIME_EQUIVALENT = "EXPENSE_FULL_TIME_EQUIVALENT";

    public static final String EXPENSE_AVERAGE_COST = "EXPENSE_AVERAGE_COST";

    public static final String BUDGETS_CHARGECODE_ID = "BUDGETS.CHARGECODE_ID";

    public static final int SEC_PER_MIN = 60;

    public static final int DEFAULT_HOUR_PER_DAY = 8;

    public static final int SC_BIT_ROLLUP = 1073741824; //  0x40000000
    
    public static final int SC_PROJECT  = 1;
    public static final int SC_PROPOSAL = 154;
    //     WBS deliverable types
    public static final int SC_DELIVERABLE = 2;
    public static final int SC_WBS_CHANGE_REQUEST = 36;
    public static final int SC_WBS_RISK = 37;
    public static final int SC_WBS_ISSUE = 39;
    public static final int SC_WORK_PRODUCT = 58;
    public static final int SC_SUMMARY_TASK = 60;
    public static final int SC_WBS_REQUIREMENT = 225;
    public static final int SC_WBS_DEFECT = 230;
    public static final int SC_WBS_SERVICE_REQUEST = 231;
    public static final int SC_WBS_ACTION_ITEM = 259;  
    public static final int SC_TASK = 3;
    public static final int SC_MILESTONE = 59;
    
    public static final int SC_CRI_CHANGE_REQUEST = 4;
    public static final int SC_CRI_RISK = 5;    
    public static final int SC_CRI_ISSUE = 6;
    public static final int SC_CRI_DELIVERABLE = 35;    
    public static final int SC_CRI_REQUIREMENT = 213;
    public static final int SC_CRI_WORK_PRODUCT = 215;  
    public static final int SC_CRI_DEFECT = 226;    
    public static final int SC_CRI_SERVICE_REQUEST = 227;   
    public static final int SC_CRI_SUMMARY_TASK = 244;  
    public static final int SC_CRI_ACTION_ITEM = 257;    
    public static final int SC_CRI_TASK = 38;

    public static final int SC_CRI_CHANGE_REQUEST_TASK = 65;    
    public static final int SC_CRI_RISK_TASK = 66;  
    public static final int SC_CRI_ISSUE_TASK = 67; 
    public static final int SC_CRI_DEFECT_TASK = 233;   
    public static final int SC_CRI_SERVICE_REQUEST_TASK = 234;  
    public static final int SC_CRI_REQUIREMENT_TASK = 235;  
    public static final int SC_CRI_MILESTONE = 247; 
    public static final int SC_CRI_ACTION_ITEM_TASK = 258;
    
    //Earned Values Columns Constant
    
    public static final String TNG_HOME_PATH = "TNG_HOME_PATH";

    public static final String RUP_HOME_PATH = "RUP_HOME_PATH";

    public static final String DLV_BUDGET_SETTING_BITS = "DLV_BUDGET_SETTING_BITS";

    public static final String COPY_CRI_ADOPT_PARTICIPANTS = "COPY_CRI_ADOPT_PARTICIPANTS";

    public static final String COPY_CRI_CONSIDER_ASSIGNMENTS = "COPY_CRI_CONSIDER_ASSIGNMENTS";

    public static final String ASSIGNMENT_BILLABLE_FLAG = "ASSIGNMENT_BILLABLE_FLAG";

    public static final String ROLLUP_DELIVERABLE_ESTIMATED = "ROLLUP_DELIVERABLE_ESTIMATED";

    public static final String ROLLUP_PROJECT_ESTIMATED = "ROLLUP_PROJECT_ESTIMATED";

    public static final String BUDGET_SETTING_BITS = "BUDGET_SETTING_BITS";
    
    public static final String NAME_PROJECT_ID = "PROJECT_DETAILS.PROJECT_ID";
    
    public static final String TABLE_NAME = "PROJECT_DETAILS";
    
    public final static String EV_DELAY_DAYS_STR = "EV_DELAY_DAYS";
    
    public static final String ENABLE_EV_CALCULATION = "ENABLE_EV_CALCULATION";
    
    public final static String EARNED_VALUES = "EarnedValues";
    
    public final static String WORKSHEET_FINANCIAL_SUB_ELEMENT = "WorkSheetFinancialSubElement";
    
    // Contour Constants
    private static final Integer BIT_FLAG_NON_CONTOURED = new Integer(0);
    private static final Integer BIT_FLAG_CONTOURED = new Integer(1);
    
    private static final Integer CONTOUR_VALUE_ASAP = new Integer(0);
    private static final Integer CONTOUR_VALUE_HOURS_PER_DAY = new Integer(4096);
    private static final Integer CONTOUR_VALUE_PERCENTAGE_DAY = new Integer(4096);
    
    private static final Integer HOURSPERDAY_WORK_UNIT = new Integer(0);
    private static final Integer PERCENTAGE_WORK_UNIT = new Integer(100);
    
    private static final Integer DEFAULT_VALUE = new Integer(0);
    // TODO: Declare the rest of the types also
    
    // This constant should be used only for the assignments without contours
    // And also when effort is not specified.
    private static final Double DEFAULT_TOTAL_EFFORT = new Double(8.0);
    
    /**
     * SP_SEARCH_2, which inserts objects, does it when 
     * the parameter 25 (type) is not zero.
     * This parameter holds the assignment parent type (Project, Task, etc.)
     * as some special constants. The following code will produce 
     * this constant based on a class of the rpmObject you provide
     * 
     * @param rpmObject Project, Proposal, Task, etc. related to assignment
     * @return
     */
    public static int getSpSearch2Type(RPMObject rpmObject)
    {
        int p = SC_BIT_ROLLUP;
        return getTypeIDByContainer(rpmObject) + p;
    }
    
    public static int getTypeIDByContainer(RPMObject rpmObject)
    {
        int ret = 0;
        if(rpmObject instanceof WorkElement)
        {
            if(rpmObject instanceof Project)
            {
                ret =  SC_PROJECT; 
            }
            else if(rpmObject instanceof Proposal)
            {
                ret =   SC_PROPOSAL;
            }
            else if(rpmObject instanceof SummaryTask)
            {
                ret =   SC_SUMMARY_TASK;
            }
            else if(rpmObject instanceof WorkProduct)
            {
                ret =   SC_WORK_PRODUCT;
            }
            else if(rpmObject instanceof Milestone)
            {
                ret =   SC_MILESTONE;
            }
            else if(rpmObject instanceof Task)
            {
                ret =   SC_TASK;
            }
            else if(rpmObject instanceof Deliverable)
            {
                ret =   SC_DELIVERABLE;
            }
        }
        else if (rpmObject instanceof ScopeElement)
        {    
            if(rpmObject instanceof ChangeRequest)
            {
                ret =   SC_CRI_CHANGE_REQUEST;
            }           
            else if(rpmObject instanceof ServiceRequest)
            {
                ret =   SC_CRI_SERVICE_REQUEST;
            }
            else if(rpmObject instanceof Action)
            {
                ret =   SC_CRI_ACTION_ITEM;
            }                       
            else if(rpmObject instanceof Defect)
            {
                ret =   SC_CRI_DEFECT;
            }           
            else if(rpmObject instanceof Requirement)
            {
                ret =   SC_CRI_REQUIREMENT;
            }
            else if(rpmObject instanceof Risk)
            {
                ret =   SC_CRI_RISK;
            }                       
            else if(rpmObject instanceof Issue)
            {
                ret =   SC_CRI_ISSUE;
            }
            else if(rpmObject instanceof ScopeTask)
            {
                ret = SC_CRI_TASK;
            }
            else if(rpmObject instanceof ScopeDeliverable)
            {
                ret = SC_CRI_DELIVERABLE;
            }
            else if(rpmObject instanceof ScopeWorkProduct)
            {
                ret = SC_CRI_WORK_PRODUCT;
            }
            else if(rpmObject instanceof ScopeSummaryTask)
            {
                ret = SC_CRI_SUMMARY_TASK;
            }
        }
        return ret;
    }
    
    public static RPMObject getContainerByTypeID(int typeID) 
    {
        switch(typeID)
        {
            // WorkElements
            case Project.TYPE_ID: return new Project();
            case Proposal.TYPE_ID: return new Proposal();
            case Task.TYPE_ID: return new Task();
            case 236: return new Task();
            case 237: return new Task();
            case 238: return new Task();
            case 239: return new Task();
            case 240: return new Task();
            case 241: return new Task();
            case 260: return new Task();
            case SummaryTask.TYPE_ID: return new SummaryTask();
            case 36: return new SummaryTask();
            case 37: return new SummaryTask();
            case 39: return new SummaryTask();
            case 225: return new SummaryTask();
            case 230: return new SummaryTask();
            case 231: return new SummaryTask();
            case 259: return new SummaryTask();
            case Deliverable.TYPE_ID: return new Deliverable();
            case WorkProduct.TYPE_ID: return new WorkProduct();
            case Milestone.TYPE_ID: return new Milestone();
            // ScopeElements
            case Action.TYPE_ID: return new Action();
            case SC_CRI_ACTION_ITEM_TASK: return new Action();
            case ChangeRequest.TYPE_ID: return new ChangeRequest();
            case SC_CRI_CHANGE_REQUEST_TASK: return new ChangeRequest();
            case ServiceRequest.TYPE_ID: return new ServiceRequest();
            case SC_CRI_SERVICE_REQUEST_TASK: return new ServiceRequest();
            case Requirement.TYPE_ID: return new Requirement();
            case SC_CRI_REQUIREMENT_TASK: return new Requirement();
            case Defect.TYPE_ID: return new Defect();
            case SC_CRI_DEFECT_TASK: return new Defect();
            case Issue.TYPE_ID: return new Issue();
            case SC_CRI_ISSUE_TASK: return new Issue();
            case Risk.TYPE_ID: return new Risk();
            case SC_CRI_RISK_TASK: return new Risk();
        }
        
        return null;        
    }

    public static String getWorkElementProjectID(WorkElement workElement, 
            MessageContext messageContext) throws RPMException
    {
        String projectId = null;
        
        if (workElement != null && workElement.getID() != null)
        {
            // SELECT TMT_WBS.PROJECT_ID FROM TMT_WBS
            // WHERE TMT_WBS.ELEMENT_ID=?             
            WbsManager wbsManager = new WbsManager();
            
            SqlBuffer stmt = new SqlBuffer();
            stmt.appendSelect();
            stmt.append(WbsManager.NAME_PROJECT_ID);
            stmt.appendFrom(wbsManager.getTableName());
            stmt.appendWhere();
            stmt.appendEqualQuestionMark(WbsManager.NAME_ELEMENT_ID);

            List result = null;
            try
            {
                result = ManagerUtil.select(new Class[]
                {
                    String.class
                }, null, null, stmt, new Object[]
                {
                    workElement.getID()
                }, messageContext);
            }
            catch (RPMException e)
            {
                ExceptionUtil.handleException(WbsUtil.class, e);
                result = null;
            }

            if (result != null && result.get(0) != null
                    && ((List) result.get(0)).get(0) != null)
            {

                projectId = (((String) ((List) result.get(0)).get(0)).toString());
            }
            
            return projectId;
        }
        
        return projectId;
    }
    
       
    public static void updateClientEVWithDBValues(
            EarnedValues clientEarnedValues, EarnedValues dbValues,
            WorkElement workElement)
    {
        int hourPerDay = getValidHourPerDay(workElement);

        clientEarnedValues.setBudgetAtCompletion(fixNegativeZero(dbValues
            .getBudgetAtCompletion()));

        // SAC = Baseline3_duration /hourPerDay/60 in the client
        // so we need to get the hourPerDay from the Parent Project Calendar
        // otherwise we set it to the default 8h/day
        clientEarnedValues.setSchedAtCompletion(new Double((dbValues
            .getSchedAtCompletion().intValue() / hourPerDay)
                / SEC_PER_MIN));

        clientEarnedValues.setBudgetedCostWorkSched(fixNegativeZero(dbValues
            .getBudgetedCostWorkSched()));
        clientEarnedValues.setBudgetedCostWorkPerf(fixNegativeZero(dbValues
            .getBudgetedCostWorkPerf()));
        clientEarnedValues.setActualCostWorkPerf(fixNegativeZero(dbValues
            .getActualCostWorkPerf()));

        // TV (Time Variance) = Round(TV/100)
        clientEarnedValues.setTimeVariance(new Integer(new Double(Math
            .rint(dbValues.getTimeVariance().doubleValue() / 100)).intValue()));

        clientEarnedValues.setEstimateToComplete(fixNegativeZero(dbValues
            .getEstimateToComplete()));
        clientEarnedValues.setEstimateAtComplete(fixNegativeZero(dbValues
            .getEstimateAtComplete()));
        clientEarnedValues.setEstimatedCompletionDate(dbValues
            .getEstimatedCompletionDate());
        clientEarnedValues
            .setBudgetedCostWorkSchedForecast(fixNegativeZero(dbValues
                .getBudgetedCostWorkSchedForecast()));
        clientEarnedValues
            .setBudgetedCostWorkPerfForecast(fixNegativeZero(dbValues
                .getBudgetedCostWorkPerfForecast()));
        clientEarnedValues
            .setActualCostWorkPerfForecast(fixNegativeZero(dbValues
                .getActualCostWorkPerfForecast()));
        clientEarnedValues
            .setEstimateToCompleteForecast(fixNegativeZero(dbValues
                .getEstimateToCompleteForecast()));
        clientEarnedValues
            .setEstimateAtCompleteForecast(fixNegativeZero(dbValues
                .getEstimateAtCompleteForecast()));

        // --calculated Earned Values

        clientEarnedValues.setScheduleVariance_C(fixNegativeZero(dbValues
            .getScheduleVariance_C()));
        clientEarnedValues.setCostVariance_C(fixNegativeZero(dbValues
            .getCostVariance_C()));
        clientEarnedValues.setPercentComplete_C(fixNegativeZero(dbValues
            .getPercentComplete_C()));
        clientEarnedValues.setSchedPerformIndex_C(fixNegativeZero(dbValues
            .getSchedPerformIndex_C()));
        clientEarnedValues.setCostPerformIndex_C(fixNegativeZero(dbValues
            .getCostPerformIndex_C()));
        clientEarnedValues.setCriticalRatio_C(fixNegativeZero(dbValues
            .getCriticalRatio_C()));
        clientEarnedValues.setToCompletePerformIndex_C(fixNegativeZero(dbValues
            .getToCompletePerformIndex_C()));
        clientEarnedValues.setVarianceAtCompletion_C(fixNegativeZero(dbValues
            .getVarianceAtCompletion_C()));
        clientEarnedValues.setSchedPerforIndexAtComp_C(fixNegativeZero(dbValues
            .getSchedPerforIndexAtComp_C()));

        clientEarnedValues.setCriticalRatioAtComp_C(new Double(
            fixNegativeZero(Math.round(dbValues.getCriticalRatioAtComp_C()
                .doubleValue()))));

        // TEAC = TEAC /hourPerDay/60 in the client
        // so we need to get the hourPerDay from the Parent Project Calendar
        // otherwise we set it to the default 8h/day
        clientEarnedValues
            .setTimeEstimateAtComp_C(new Double(fixNegativeZero((dbValues
                .getTimeEstimateAtComp_C().doubleValue() / hourPerDay)
                    / SEC_PER_MIN)));

        // TVAC = TVAC /hourPerDay/60 in the client
        // so we need to get the hourPerDay from the Parent Project Calendar
        // otherwise we set it to the default 8h/day
        clientEarnedValues
            .setTimeVarianceAtComp_C(new Double(fixNegativeZero((dbValues
                .getTimeVarianceAtComp_C().doubleValue() / hourPerDay)
                    / SEC_PER_MIN)));

    }

    public static Double fixNegativeZero(Double val)
    {
        if (val != null && val.doubleValue() == -0.0)
        {
            val = new Double(0.0);
        }

        return val;
    }

    public static double fixNegativeZero(double val)
    {
        if (val == -0.0)
        {
            val = 0.0;
        }

        return val;
    }

    private static int getValidHourPerDay(WorkElement workElement)
    {
        WorkElement we = workElement;
        int hourPerDay = DEFAULT_HOUR_PER_DAY;

        while ((we.getParent() != null)
                && !(we.getParent() instanceof Portfolio))
        {
            we = (WorkElement) we.getParent();
        }

        if (we instanceof GenericProject)
        {
            GenericProject gp = (GenericProject) we;

            if (gp.getCalendar() != null
                    && gp.getCalendar().getHoursPerDay() != null)
            {
                hourPerDay = gp.getCalendar().getHoursPerDay().intValue();
            }
        }
        return hourPerDay;
    }

    public static EarnedValues processEarnedValues(FieldValueMap relatedFields,
            int index) throws RPMException
    {
        // we are creating a EarnedValue object that has all normal fields and
        // calculated ones

        EarnedValues tmpEarnedValues = new EarnedValues();

        // step 1: get the new DB values

        tmpEarnedValues.setBudgetAtCompletion((Double) (relatedFields.get(
            index, WbsManager.ID_BAC)));
        tmpEarnedValues.setSchedAtCompletion(new Double( ((Integer)relatedFields.get(
            index, WbsManager.ID_BASELINE3_DURATION)).doubleValue()));
        tmpEarnedValues.setBudgetedCostWorkSched((Double) (relatedFields.get(
            index, WbsManager.ID_BCWS)));
        tmpEarnedValues.setBudgetedCostWorkPerf((Double) (relatedFields.get(
            index, WbsManager.ID_BCWP)));
        tmpEarnedValues.setActualCostWorkPerf((Double) (relatedFields.get(
            index, WbsManager.ID_ACWP)));

        tmpEarnedValues.setTimeVariance((Integer) (relatedFields.get(index,
            WbsManager.ID_TIME_VARIANCE)));

        tmpEarnedValues.setEstimateToComplete((Double) (relatedFields.get(
            index, WbsManager.ID_EV_ETC)));
        tmpEarnedValues.setEstimateAtComplete((Double) (relatedFields.get(
            index, WbsManager.ID_EAC)));
        tmpEarnedValues.setEstimatedCompletionDate((Calendar) relatedFields
            .get(index, WbsManager.ID_BASELINE3_FINISH));
        tmpEarnedValues
            .setBudgetedCostWorkSchedForecast((Double) (relatedFields.get(
                index, WbsManager.ID_BCWS_FORECAST)));
        tmpEarnedValues.setBudgetedCostWorkPerfForecast((Double) (relatedFields
            .get(index, WbsManager.ID_BCWP_FORECAST)));
        tmpEarnedValues.setActualCostWorkPerfForecast((Double) (relatedFields
            .get(index, WbsManager.ID_ACWP_FORECAST)));
        tmpEarnedValues.setEstimateToCompleteForecast((Double) (relatedFields
            .get(index, WbsManager.ID_EV_ETC_FORECAST)));
        tmpEarnedValues.setEstimateAtCompleteForecast((Double) (relatedFields
            .get(index, WbsManager.ID_EAC_FORECAST)));

        // step 2: process calculated fields

        // Schedule Variance: (BCWP - BCWS (EV - PV))
        double sv = tmpEarnedValues.getBudgetedCostWorkPerf().doubleValue()
                - tmpEarnedValues.getBudgetedCostWorkSched().doubleValue();
        tmpEarnedValues.setScheduleVariance_C(new Double(sv));

        // Cost Variance: (BCWP - ACWP (EV - AC))
        double cv = tmpEarnedValues.getBudgetedCostWorkPerf().doubleValue()
                - tmpEarnedValues.getActualCostWorkPerf().doubleValue();
        tmpEarnedValues.setCostVariance_C(new Double(cv));

        // Percent Complete: ((BCWP / BAC) * 100 ((EV / BAC) * 100))
        double pc = 0;
        if (tmpEarnedValues.getBudgetAtCompletion().doubleValue() != 0)
        {
            pc = (tmpEarnedValues.getBudgetedCostWorkPerf().doubleValue() / tmpEarnedValues
                .getBudgetAtCompletion().doubleValue()) * 100;
        }
        tmpEarnedValues.setPercentComplete_C(new Double(pc));

        // Schedule Performance index: (BCWP / BCWS (EV / PV))
        double spi = 0;
        if (tmpEarnedValues.getBudgetedCostWorkSched().doubleValue() != 0)
        {
            spi = tmpEarnedValues.getBudgetedCostWorkPerf().doubleValue()
                    / tmpEarnedValues.getBudgetedCostWorkSched().doubleValue();

        }
        tmpEarnedValues.setSchedPerformIndex_C(new Double(spi));

        // cost performace index: (BCWP / ACWP (EV / AC))
        double cpi = 0;
        if (tmpEarnedValues.getActualCostWorkPerf().doubleValue() != 0)
        {
            cpi = tmpEarnedValues.getBudgetedCostWorkPerf().doubleValue()
                    / tmpEarnedValues.getActualCostWorkPerf().doubleValue();
        }
        tmpEarnedValues.setCostPerformIndex_C(new Double(cpi));

        // Critical Ratio: (SPI * CPI)
        double criticalRatio = tmpEarnedValues.getSchedPerformIndex_C()
            .doubleValue()
                * tmpEarnedValues.getCostPerformIndex_C().doubleValue();
        tmpEarnedValues.setCriticalRatio_C(new Double(criticalRatio));

        // To complete performance index: ((BAC - BCWP) / (BAC - ACWP) ((BAC-EV)
        // / (BAC-AC)))
        double tcpi = 0;
        if ((tmpEarnedValues.getBudgetAtCompletion().doubleValue() - tmpEarnedValues
            .getActualCostWorkPerf().doubleValue()) != 0)
        {
            tcpi = (tmpEarnedValues.getBudgetAtCompletion().doubleValue() - tmpEarnedValues
                .getBudgetedCostWorkPerf().doubleValue())
                    / (tmpEarnedValues.getBudgetAtCompletion().doubleValue() - tmpEarnedValues
                        .getActualCostWorkPerf().doubleValue());
        }
        tmpEarnedValues.setToCompletePerformIndex_C(new Double(tcpi));

        // Variance at completion: (BAC - EAC)
        double vac = tmpEarnedValues.getBudgetAtCompletion().doubleValue()
                - tmpEarnedValues.getEstimateAtComplete().doubleValue();
        tmpEarnedValues.setVarianceAtCompletion_C(new Double(vac));

        // Schedule performance index at completion: (BAC * (SPI - 1))
        double svac_spi = 0;
        svac_spi = tmpEarnedValues.getBudgetAtCompletion().doubleValue()
                * (tmpEarnedValues.getSchedPerformIndex_C().doubleValue() - 1);
        tmpEarnedValues.setSchedPerforIndexAtComp_C(new Double(svac_spi));

        // Critial ratio at completion: (BAC * (CR - 1))
        double svac_cr = 0;
        svac_cr = tmpEarnedValues.getBudgetAtCompletion().doubleValue()
                * (tmpEarnedValues.getCriticalRatio_C().doubleValue() - 1);
        tmpEarnedValues.setCriticalRatioAtComp_C(new Double(svac_cr));

        // Time estimate at completion: (SAC / SPI)
        double teac = 0;
        if (tmpEarnedValues.getSchedPerformIndex_C().doubleValue() != 0)
        {
            teac = tmpEarnedValues.getSchedAtCompletion().doubleValue()
                    / tmpEarnedValues.getSchedPerformIndex_C().doubleValue();
        }
        tmpEarnedValues.setTimeEstimateAtComp_C(new Double(teac));

        // Time Variance at completion: (SAC - TEAC)
        double tvac = tmpEarnedValues.getSchedAtCompletion().doubleValue()
                - tmpEarnedValues.getTimeEstimateAtComp_C().doubleValue();
        tmpEarnedValues.setTimeVarianceAtComp_C(new Double(tvac));

        return tmpEarnedValues;

    }

    // checks if work has been assigned; does not include assigned participants
    public static boolean isWorkAssigned(WorkElement workElement)
    {
        boolean workAssigned = false;
        ResourceTaskAssignment[] assignments = workElement
            .getResourceTaskAssignments();

        if (assignments != null)
        {
            for (int i = 0; i < assignments.length; i++)
            {
                if (assignments[i].getTaskAssignment() != null)
                {
                    workAssigned = true;
                    break;
                }
            }

        }
        return workAssigned;
    }

    public static RPMObject loadResourceTaskAssignmentParentType(
            MessageContext messageContext, ResourceTaskAssignment assignment)
    {
        RPMObject parentContainer = null; 
        Integer parentTypeId = null;

        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(
            ResourceTaskAssignmentManager.NAME_RES_ASSIGNMENT_ID);

        Object[] arguments = new Object[]
        {
            assignment.getID()
        };
        
        try
        {
            parentTypeId = (Integer) ManagerUtil.selectColumnValue(
                Integer.class, null, null,
                ResourceTaskAssignmentManager.NAME_ELEMENT_TYPE,
                ResourceTaskAssignmentManager.TABLE_NAME, condition,
                arguments, messageContext);
        }
        catch (RPMException e)
        {
            messageContext.addExceptionNoThrow(e);
        }

        if (parentTypeId != null)
        {
            parentContainer = WbsUtil.getContainerByTypeID(parentTypeId
                .intValue());
        }

        return parentContainer;
    }
    
   
    public static boolean getEVCalculationActivationState(
            GenericProject project, MessageContext messageContext)
            throws RPMException
    {
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(NAME_PROJECT_ID);

        Object[] arguments = new Object[]
        {
            project.getID()
        };
        
        Long evCalculationState = ((Long) ManagerUtil.selectColumnValue(
            Long.class, project,
            ValidationConstants.EV_CALCULATION_ACTIVATION_STATE,
            ENABLE_EV_CALCULATION, TABLE_NAME, condition, arguments,
            messageContext));

        if (evCalculationState == null)
        {
            evCalculationState = new Long(0);
        }

        return (evCalculationState.intValue() == 1) ? true : false;
    }
    
    
    public static List loadProjectDetailsEV_Infos(
            MessageContext messageContext, WorkElement workelement)
            throws RPMException
    {
        Class[] returnTypes =
        {
                Long.class, Long.class, Long.class, Long.class, String.class,
                String.class, Long.class, String.class, String.class
        };

        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(BUDGET_SETTING_BITS);
        stmt.appendComma();
        stmt.append(ROLLUP_PROJECT_ESTIMATED);
        stmt.appendComma();
        stmt.append(ROLLUP_DELIVERABLE_ESTIMATED);
        stmt.appendComma();
        stmt.append(ASSIGNMENT_BILLABLE_FLAG);
        stmt.appendComma();
        stmt.append(COPY_CRI_CONSIDER_ASSIGNMENTS);
        stmt.appendComma();
        stmt.append(COPY_CRI_ADOPT_PARTICIPANTS);
        stmt.appendComma();
        stmt.append(DLV_BUDGET_SETTING_BITS);
        stmt.appendComma();
        stmt.append(RUP_HOME_PATH);
        stmt.appendComma();
        stmt.append(TNG_HOME_PATH);
        stmt.appendFrom(TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqual(NAME_PROJECT_ID, workelement.getID());

        List result = ManagerUtil.select(returnTypes, workelement,
            EARNED_VALUES, stmt, null, messageContext);

        return result;
    }

    public static boolean isTaskAssignmentInPlannedStage(
            GenericTaskAssignment taskAssignment, MessageContext messageContext)
    {
        if (taskAssignment == null || taskAssignment.getID() == null)
        {
            return false;
        }
        
        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect("count(*)");
        stmt.appendFrom(TaskAssignmentManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendIsNull(TaskAssignmentManager.NAME_PLANNED_START);
        stmt.appendIsNull(TaskAssignmentManager.NAME_PLANNED_FINISH);
        stmt.appendIsNotNull(TaskAssignmentManager.NAME_SCHED_START);
        stmt.appendIsNotNull(TaskAssignmentManager.NAME_SCHED_FINISH);
        stmt.appendEqual(TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
            taskAssignment.getID());

        List result = null;
        try
        {
            result = ManagerUtil.select(new Class[]
            {
                Integer.class
            }, null, null, stmt, null, messageContext);
        }
        catch (RPMException e)
        {
            messageContext.addExceptionNoThrow(e);
        }

        if (result == null || result.get(0) == null
                || ((List) result.get(0)).get(0) == null
                || ((Integer) ((List) result.get(0)).get(0)).intValue() == 0)
        {
            return false;
        }

        return true;
    }
    
    // returnValues[0] = Start Date
    // returnValues[1] = Finish Date
    // returnValues[2] = Effort
    // returnvalues[3] = Work
    // returnValues[4] = Bit flag
    // returnValues[5] = Unit of Effort
    // returnValues[6] = Unit of Work
    // returnValues[7] = Contour Value
    // returnValues[8] = Occurrences
    public static Object[] getContourValues(ResourceTaskAssignment assignment)
    {
    	Contour contour = assignment.getContour();
    	Object returnValues[] = new Object[9];
    	
    	if(contour == null)
    	{
    		returnValues[0] = getStartDateFromAssignment(assignment);
    		returnValues[1] = null; //finishDate
    		returnValues[2] = DateUtil.convertHoursToMinutes(DEFAULT_TOTAL_EFFORT);
    		if (assignment.getTaskAssignment() != null
	    			&& assignment.getTaskAssignment().getProposedDurationHours() != null
	    			&& assignment.getTaskAssignment().getProposedDurationHours()
	    			.doubleValue() > 0.0)
	    	{
	    		returnValues[2] = DateUtil.convertHoursToMinutes(assignment
	    				.getTaskAssignment().getProposedDurationHours()); // Effort
	    	}
    		returnValues[3] = new Integer(480); // Work
    		for(int i=4;i<returnValues.length; i++)
        	{
        		returnValues[i] = DEFAULT_VALUE;
        	}
    	}
    	else
    	{
    		ContourType contourType = contour.getContourType();

    		// Contour is of type ASAP
    		if(contourType.equals(ContourType.ASAP))
    		{
    			returnValues[0] = contour.getStartDate(); //StartDate
    			if(contour.getEffortInMinutes() == null || contour.getEffortInMinutes().intValue() <= 0) // Finish Date
        		{
    				returnValues[1] = contour.getFinishDate();
    				returnValues[2] = DEFAULT_VALUE;
        		}
        		else // Effort
        		{
        			returnValues[2] = contour.getEffortInMinutes();
        		}
    			returnValues[3] = new Integer(0); //Work
    			returnValues[4] = BIT_FLAG_NON_CONTOURED; 
    			returnValues[5] = DEFAULT_VALUE; // Unit of Effort
    			returnValues[6] = DEFAULT_VALUE; // Unit of Work
    			returnValues[7] = CONTOUR_VALUE_ASAP;
    			returnValues[8] = DEFAULT_VALUE; // Occurrences
    		}
    		
    		// Contour is if type HoursPerDay
    		else if(contourType.equals(ContourType.HoursPerDay))
    		{
    			boolean isEffortAssigned = false;
    			returnValues[0] = contour.getStartDate(); //StartDate
    			if(contour.getEffortInMinutes() == null || contour.getEffortInMinutes().intValue() <= 0) // Finish Date
        		{
    				if(contour.getFinishDate() != null)
    				{
    					returnValues[1] = contour.getFinishDate();
    					isEffortAssigned = true;
    				}
    				returnValues[2] = DEFAULT_VALUE;
    			}
    			else // Effort
        		{
        			returnValues[2] = contour.getEffortInMinutes();
        			isEffortAssigned = true;
        		}
        		
    			if(contour.getWork() != null && contour.getWork().doubleValue() > 0)
        		{
        			returnValues[3] = DateUtil.convertHoursToMinutes(contour.getWork()); 
        		}
    			else
    			{
    				returnValues[3] = new Integer(480);
    			}
    			returnValues[4] = BIT_FLAG_CONTOURED;
    			returnValues[5] = DEFAULT_VALUE; // Unit of Effort
    			returnValues[6] = HOURSPERDAY_WORK_UNIT; // Unit of Work
    			returnValues[7] = CONTOUR_VALUE_HOURS_PER_DAY;
    			if(!isEffortAssigned)
    			{
    				returnValues[8] = contour.getOccurrences(); // Occurrences
    			}
    			else
    			{
    				returnValues[8] = DEFAULT_VALUE;
    			}
    		}
    		
    		// Contour is of type %Day
    		else if(contourType.equals(ContourType.PercentilePerDay))
    		{
    			boolean isEffortAssigned = false;
    			returnValues[0] = contour.getStartDate(); //StartDate
    			if(contour.getEffortInMinutes() == null || contour.getEffortInMinutes().intValue() <= 0) // Finish Date
        		{
    				if(contour.getFinishDate() != null)
    				{
    					returnValues[1] = contour.getFinishDate();
    					isEffortAssigned = true;
    				}
    				returnValues[2] = DEFAULT_VALUE;
    			}
    			else // Effort
        		{
        			returnValues[2] = contour.getEffortInMinutes();
        			isEffortAssigned = true;
        		}
    			if(contour.getWork() != null && contour.getWork().doubleValue() > 0)
    			{
    				double work = contour.getWork().doubleValue();
    				double percentage = 100.0;
    				double percentWork = work * percentage;
    				Double percentileWork = new Double(percentWork);
    				returnValues[3] = new Integer(percentileWork.intValue());
    			}
    			else
    			{
    				returnValues[3] = new Integer(100);
    			}
    			returnValues[4] = BIT_FLAG_CONTOURED;
    			returnValues[5] = DEFAULT_VALUE; // Unit of Effort
    			returnValues[6] = PERCENTAGE_WORK_UNIT;
    			returnValues[7] = CONTOUR_VALUE_PERCENTAGE_DAY;
    			if(!isEffortAssigned)
    			{
    				returnValues[8] = contour.getOccurrences(); // Occurrences
    			}
    			else
    			{
    				returnValues[8] = DEFAULT_VALUE;
    			}
    		}

    		// TODO: Implement for other Contour types

    	}
    	return returnValues;
    }
    
    private static Calendar getStartDateFromAssignment(ResourceTaskAssignment assignment)
    {
    	Calendar startDate = null;
    	RPMObject parent = assignment.getParent();
    	if (assignment.getTaskAssignment() != null
                && assignment.getTaskAssignment().getAssignmentStartDate() != null)
        {
			startDate = assignment.getTaskAssignment().getAssignmentStartDate();
        }
        else if (parent instanceof WorkElement)
        {
        	startDate = ((WorkElement) parent).getEstimatedStartDate();
        }
        else if (parent instanceof ScopeElement
                && ((ScopeElement) parent).getProject() != null)
        {
        	startDate = ((ScopeElement) parent).getProject()
                .getEstimatedStartDate();
        }
    	return startDate;
    }
}
