/*
 * Created on 04-Dec-2006
 * @author vikasbs
 */
package com.ibm.rpm.workflow.checkpoints;

import java.util.ArrayList;
import java.util.List;


import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.workflow.constants.ValidationConstants;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.workflow.containers.ExternalAction;
import com.ibm.rpm.workflow.containers.ExternalActionCategory;

public class ExternalActionCheckpoint extends AbstractCheckpoint
{
    private static final List EXTERNAL_ACTION_PARENT_LIST = new ArrayList();

    static
    {
        EXTERNAL_ACTION_PARENT_LIST.add(ExternalActionCategory.class);
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        // validate the instance and the parent
        if (!GenericValidator.validateInstance(rpmObject, ExternalAction.class,
                msgCtx)
                || !isValidParent(rpmObject, msgCtx, false,
                        EXTERNAL_ACTION_PARENT_LIST))
        {
            return;
        }
        validateName((ExternalAction)rpmObject, msgCtx);
        validatePath((ExternalAction)rpmObject, msgCtx);
    }

    private void validateName(ExternalAction externalAction,
            MessageContext msgCtx)
    {
        boolean valid = GenericValidator.validateMandatory(externalAction,
                ValidationConstants.NAME_FIELD, externalAction.getName(),
                msgCtx);
        if (valid && externalAction.testNameModified())
        {
            GenericValidator.validateMaxLength(externalAction,
                    ValidationConstants.NAME_FIELD, externalAction.getName(),
                    ValidationConstants.EXTERNAL_ACTION_NAME_MAX, msgCtx);
            GenericValidator.validateMinLength(externalAction,
                    ValidationConstants.NAME_FIELD, externalAction.getName(),
                    ValidationConstants.EXTERNAL_ACTION_NAME_MIN, msgCtx);
        }
    }
    
    private void validatePath(ExternalAction externalAction,
            MessageContext msgCtx)
    {
        boolean valid = GenericValidator.validateMandatory(externalAction,
                ValidationConstants.PATH_FIELD, externalAction.getName(),
                msgCtx);
        if (valid && externalAction.testNameModified())
        {
            GenericValidator.validateMaxLength(externalAction,
                    ValidationConstants.PATH_FIELD, externalAction.getPath(),
                    ValidationConstants.EXTERNAL_ACTION_PATH_MAX, msgCtx);
            GenericValidator.validateMinLength(externalAction,
                    ValidationConstants.PATH_FIELD, externalAction.getPath(),
                    ValidationConstants.EXTERNAL_ACTION_PATH_MIN, msgCtx);
        }
    }
}
