package com.ibm.rpm.workflow.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;

import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.workflow.containers.AvailableWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.AvailableWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class AvailableWorkflowProcessManager extends AbstractRPMObjectManager
{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static ContainerMap containerMap = new ContainerMap(
        AvailableWorkflowProcessManager.class);

    static
    {
        containerMap.add(AvailableWorkflowProcess.class);
    }

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 1;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "PROJECT_WKF_PROCESS.PROJECT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded from this column in upper case.
    //public static final String PROPERTY_PROJECT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID,NAME_PROJECT_ID);}

    /**
     * Column PROCESS_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROCESS_ID = 2;

    public static final int TYPE_PROCESS_ID = Types.CHAR;

    public static final String NAME_PROCESS_ID = "PROJECT_WKF_PROCESS.PROCESS_ID";

    public static final String PROPERTY_PROCESS_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROCESS_ID, NAME_PROCESS_ID);
    }

    /**
     * Column CONFIG_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONFIG_FLAG = 3;

    public static final int TYPE_CONFIG_FLAG = Types.INTEGER;

    public static final String NAME_CONFIG_FLAG = "PROJECT_WKF_PROCESS.CONFIG_FLAG";

    public static final String PROPERTY_CONFIG_FLAG = "DEFAULTFLAG";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONFIG_FLAG, SqlUtil.getBooleanValue(
            NAME_CONFIG_FLAG, 3));
    }

    public static final String TABLE_NAME = "PROJECT_WKF_PROCESS";

    /**
     * Create an array of type string containing all the fields of the PROJECT_WKF_PROCESS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_PROJECT_ID, NAME_PROCESS_ID, NAME_CONFIG_FLAG
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new ProjectWkfProcessManager instance.
     * @return ProjectWkfProcessManager
     */
    public AvailableWorkflowProcessManager()
    {
        super();
    }
    
    public ContainerMap getContainerMap()
    {
        return containerMap;
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_PROCESS_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        RPMObject container = null;
        try
        {
            container = (RPMObject) AvailableWorkflowProcess.class.newInstance();
        }
        catch (Exception x)
        {
            ExceptionUtil.handleException(this, x);
        }

        return container;
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        AvailableWorkflowProcess availableProcess = (AvailableWorkflowProcess) rpmObject;

        // all fields are read-only
        availableProcess.setID(rs.getString(ID_PROCESS_ID));
        availableProcess.setDefaultFlag(decodeDefaultFlag(rs
            .getInt(ID_CONFIG_FLAG)));

        return availableProcess;
    }

    private Boolean decodeDefaultFlag(int configFlag)
    {
        Boolean defaultFlag = null;

        if (configFlag == 3)
        {
            defaultFlag = new Boolean(true);
        }
        else
        {
            defaultFlag = new Boolean(false);
        }

        return defaultFlag;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String workflowProcessId = resultSet.getString(ID_PROCESS_ID);
        relatedFields.put(index, ID_PROCESS_ID, workflowProcessId);

        return relatedFields;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        AvailableWorkflowProcessScope scope = (AvailableWorkflowProcessScope) rpmScope;

        if (scope != null && scope.getChildren() != null)
        {
            loadWorkflowProcess((AvailableWorkflowProcess) rpmObject, scope
                .getChildren(), messageContext, isDelta);
        }

        return rpmObject;
    }

    private void loadWorkflowProcess(AvailableWorkflowProcess availableProcess,
            WorkflowScope scope, MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        WorkflowManager manager = (WorkflowManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(WorkflowProcess.class.getName());

        RPMObject[] oldWorkflowProcess = new RPMObject[]
        {
            availableProcess.getWorkflowProcess()
        };

        Object[] args = new Object[]
        {
            availableProcess.getID()
        };

        ArrayList temp = manager.loadByForeignKey(oldWorkflowProcess, scope,
            messageContext, this, args, null, null, isDelta);

        if (temp != null && !temp.isEmpty())
        {
            WorkflowProcess workflowProcess = (WorkflowProcess) temp.get(0);
            availableProcess.setWorkflowProcess(workflowProcess);
        }

    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WbsManager || manager instanceof ScopeManager
                || manager instanceof DocumentManager)
        {
            // for loading WorkElement.workflowProcessesAvailable,
            // AbstractScope.workflowProcessesAvailable, and 
            // GenericDocument.workflowProcessesAvailable
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_PROJECT_ID,
                WbsManager.NAME_ELEMENT_ID, WbsManager.TABLE_NAME,
                WbsManager.NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoCreate(this, rpmObject);
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ExceptionUtil.handleNoUpdate(this, rpmObject);
        return rpmObject;
    }

    public void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ExceptionUtil.handleNoDelete(this, rpmObject);
    }

}