package com.ibm.rpm.workflow.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;

import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ArrayUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.workflow.containers.ExternalAction;
import com.ibm.rpm.workflow.containers.ExternalActionCategory;
import com.ibm.rpm.workflow.scope.ExternalActionScope;
import com.ibm.rpm.xpathparser.XPathContainer;

public class ExternalActionManager extends AbstractRPMObjectManager
{

    private static ContainerMap containerMap = new ContainerMap(
            ExternalActionManager.class);
    static
    {
        containerMap.add(ExternalAction.class);
        containerMap.add(ExternalActionCategory.class);
    }

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 1;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_EXTERNAL_ACTIONS.RANK";

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 2;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_EXTERNAL_ACTIONS.PARENT_ID";

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 3;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_EXTERNAL_ACTIONS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 4;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_EXTERNAL_ACTIONS.TYPE_ID";

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 5;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_EXTERNAL_ACTIONS.LEVEL_ID";

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 6;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_EXTERNAL_ACTIONS.MAX_RANK";

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 7;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_EXTERNAL_ACTIONS.CHILD_COUNT";

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 8;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_EXTERNAL_ACTIONS.DELETED_COUNT";

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 9;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_EXTERNAL_ACTIONS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 10;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_EXTERNAL_ACTIONS.REC_STATUS";

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 11;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_EXTERNAL_ACTIONS.REC_USER";

    /**
     * Column ACTION_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ACTION_TYPE = 12;

    public static final int TYPE_ACTION_TYPE = Types.INTEGER;

    public static final String NAME_ACTION_TYPE = "TMT_EXTERNAL_ACTIONS.ACTION_TYPE";

    /**
     * Column ELEMENT_TYPES of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ELEMENT_TYPES = 13;

    public static final int TYPE_ELEMENT_TYPES = Types.INTEGER;

    public static final String NAME_ELEMENT_TYPES = "TMT_EXTERNAL_ACTIONS.ELEMENT_TYPES";

    /**
     * Column PATH of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PATH = 14;

    public static final int TYPE_PATH = Types.VARCHAR;

    public static final String NAME_PATH = "TMT_EXTERNAL_ACTIONS.PATH";

    public static final String PROPERTY_PATH = "PATH";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PATH, NAME_PATH);
    }

    /**
     * Column CONFIG_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_CONFIG_FLAG = 15;

    public static final int TYPE_CONFIG_FLAG = Types.SMALLINT;

    public static final String NAME_CONFIG_FLAG = "TMT_EXTERNAL_ACTIONS.CONFIG_FLAG";

    public static final String PROPERTY_CONFIG_FLAG = "BLOCKING";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONFIG_FLAG, NAME_CONFIG_FLAG);
    }

    private static final String TABLE_NAME = "TMT_EXTERNAL_ACTIONS";

    /**
     * Create an array of type string containing all the fields of the
     * TMT_EXTERNAL_ACTIONS table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_RANK, NAME_PARENT_ID, NAME_ELEMENT_ID, NAME_TYPE_ID,
            NAME_LEVEL_ID, NAME_MAX_RANK, NAME_CHILD_COUNT, NAME_DELETED_COUNT,
            NAME_ELEMENT_NAME, NAME_REC_STATUS, NAME_REC_USER,
            NAME_ACTION_TYPE, NAME_ELEMENT_TYPES, NAME_PATH, NAME_CONFIG_FLAG
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new ExternalActionManager instance.
     * 
     * @return ExternalActionManager
     */
    public ExternalActionManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return containerMap.createContainer(typeId);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        String name = rpmObject instanceof ExternalAction
                ? ((ExternalAction) rpmObject).getName()
                : ((ExternalActionCategory) rpmObject).getName();
        Integer typeId = rpmObject.retrieveTypeID();
        RPMObject parent = rpmObject.getParent();
        String parentId = null;
        if (parent != null)
        {
            parentId = parent.getID();
        }
        String id = SP_I_EXTERNAL_ACTIONS(parentId, name, typeId,
                messageContext);

        rpmObject.setID(id);
    }

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        if (rpmObject instanceof ExternalAction)
        {
            ExternalAction extAct = (ExternalAction) rpmObject;

            String name = rs.getString(ID_ELEMENT_NAME);
            if (name != null)
            {
                name = name.trim();
            }
            String path = rs.getString(ID_PATH);
            Boolean blocking = new Boolean(rs.getBoolean(ID_CONFIG_FLAG));
            if (isDelta)
            {
                extAct.deltaName(name);
                extAct.deltaPath(path);
                extAct.deltaBlocking(blocking);
            }
            else
            {
                extAct.setName(name);
                extAct.setPath(path);
                extAct.setBlocking(blocking);
            }
        }
        else
        {
            ExternalActionCategory extAct = (ExternalActionCategory) rpmObject;
            String name = rs.getString(ID_ELEMENT_NAME);
            if (name != null)
            {
                name = name.trim();
            }
            if (isDelta)
            {
                extAct.deltaName(name);
            }
            else
            {
                extAct.setName(name);
            }
        }
        return rpmObject;
    }

    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        String parentId = ManagerUtil.getParentId(this, rpmObject,
                messageContext, TABLE_NAME);
        SP_D_EXTERNAL_ACTIONS(rpmObject.getID(), parentId, messageContext);
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope != null)
        {
            // Load parent ExternalActionCategory, if required.
            RPMObjectScope parentScope = rpmScope.getParent();
            if (parentScope != null && !isDelta)
            {
                RPMObject parent = null;
                String parentID = (String) relatedFields.get(index,
                        ID_PARENT_ID);
                if (parentID != null)
                {
                    parent = new ExternalActionCategory();
                    parent.setID(parentID);
                    parent = loadByPrimaryKey(parent, parentScope,
                            messageContext, isDelta);
                }
                rpmObject.setParent(parent);
            }

            if (rpmScope instanceof ExternalActionScope)
            {
                ExternalActionScope scope = (ExternalActionScope) rpmScope;

                // Load children, if required.
                if (rpmObject instanceof ExternalActionCategory)
                {
                    ExternalActionCategory category = (ExternalActionCategory) rpmObject;
                    boolean childrenScope = scope.isChildren();
                    if (!childrenScope)
                    {
                        category.setChildren(null);
                    }
                    else
                    {
                        // This parent ID is used as the foreign key for its
                        // children.
                        Object[] arguments =
                        {
                            category.getID()
                        };
                        ArrayList childrenList = loadByForeignKey(category
                                .getChildren(), null, messageContext, this,
                                arguments, null, category.getContextName(),
                                isDelta);

                        ExternalAction[] children = (ExternalAction[]) ArrayUtil
                                .listToArray(ExternalAction.class, childrenList);
                        category.setChildren(children);
                    }
                }
            }
        }
        return rpmObject;
    }

    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        relatedFields.put(index, ID_PARENT_ID, resultSet
                .getString(ID_PARENT_ID));
        return relatedFields;
    }

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        if (rpmObject instanceof ExternalAction)
        {
            ExternalAction extAct = (ExternalAction) rpmObject;
            if (extAct.testNameModified() || extAct.testPathModified())
            {
                SP_U_EXTERNAL_ACTIONS(extAct.getID(), extAct.getName(), extAct
                        .getPath(), messageContext);
            }
            if (extAct.testBlockingModified())
            {
                SP_U_EA_STATE(extAct.getID(), extAct.getBlocking(),
                        messageContext);
            }
        }
        else
        {
            ExternalActionCategory extActCat = (ExternalActionCategory) rpmObject;
            if (extActCat.testNameModified())
            {
                SP_U_EXTERNAL_ACTIONS(extActCat.getID(), extActCat.getName(),
                        null, messageContext);
            }
            ExternalActionScope scope = (ExternalActionScope) rpmScope;
            if (scope != null)
            {
                updateChildren(extActCat, extActCat.getChildren(), scope
                        .isChildren(), messageContext, null);
            }
        }
        return rpmObject;
    }

    public JoinCondition getJoinCondition(RPMObjectManager callingManager,
            XPathContainer container, String contextName) throws RPMException
    {
        // Create the basic filtering condition.
        String finalCondition = getFinalCondition(container);

        JoinCondition joinCondition = new JoinCondition();
        if (callingManager == null)
        {
            joinCondition.setTableName(TABLE_NAME);
            joinCondition.appendCondition(finalCondition);
        }
        else if (callingManager instanceof ExternalActionManager)
        {
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID,
                    TABLE_NAME, finalCondition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, callingManager,
                    ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    private String getFinalCondition(XPathContainer container)
            throws RPMException
    {
        // Add the container filter, if required.
        SqlBuffer finalCondition = getContainerFilter(container.getType());

        return finalCondition.toString();
    }

    /**
     * Get the right filter for the container.
     */
    private SqlBuffer getContainerFilter(Class containerClass)
            throws RPMException
    {
        SqlBuffer filter = new SqlBuffer(getFilter());

        Integer typeId = containerMap.getTypeId(containerClass);
        if (typeId != null)
        {
            filter.appendEqual(NAME_TYPE_ID, typeId);
        }
        else
        {
            throw new RPMException("Unsupported container " + containerClass);
        }
        return filter;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        String finalCondition = getFinalCondition(container);

        if (manager instanceof ExternalActionManager)
        {
            // For XPath queries like:
            // /ExternalActionCategory[ExternalAction[<condition>]]
            joinCondition = JoinCondition.createSubSelect(NAME_ELEMENT_ID,
                    NAME_PARENT_ID, TABLE_NAME, finalCondition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    protected JoinCondition getForeignKeyCondition(
            RPMObjectManager callingManager, String extraCondition,
            String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (callingManager instanceof ExternalActionManager)
        {
            // Note: used when loading children from ExternalActioCategory.
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendEqualQuestionMark(NAME_PARENT_ID);
            joinCondition.appendCondition(extraCondition);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, callingManager,
                    ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    private String SP_I_EXTERNAL_ACTIONS(String parentId, String name,
            Integer typeId, MessageContext messageContext) throws SQLException,
            RPMException
    {
        Object args[] = new Object[4];
        args[0] = name;
        args[1] = parentId;
        args[2] = typeId;
        args[3] = getUser(messageContext).getID();
        return executeProcedure(messageContext, "SP_I_EXTERNAL_ACTIONS", args);
    }

    private void SP_D_EXTERNAL_ACTIONS(String id, String parentId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[4];
        args[0] = id;
        args[1] = parentId;
        args[2] = "X";
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_D_EXTERNAL_ACTIONS", args);
    }

    private void SP_U_EXTERNAL_ACTIONS(String id, String name, String path,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[6];
        args[0] = id;
        args[1] = name;
        args[2] = new Integer(0);
        args[3] = new Integer(0);
        args[4] = path;
        args[5] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_U_EXTERNAL_ACTIONS", args);
    }

    private void SP_U_EA_STATE(String id, Boolean blocking,
            MessageContext messageContext) throws SQLException, RPMException
    {
        Object args[] = new Object[3];
        args[0] = id;
        args[1] = blocking;
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, "SP_U_EA_STATE", args);
    }
}
