/*
 * Created on 18-Sep-2006
 * @author mmatwiy
 * Utility methods for the workflow section.
 */
package com.ibm.rpm.workflow.util;

import java.util.ArrayList;
import java.util.Iterator;

import com.ibm.rpm.communications.managers.CommunicationManager;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.document.managers.DocumentUtil;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.EnumeratedTypeUtil;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.workflow.constants.ValidationConstants;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowRole;
import com.ibm.rpm.workflow.containers.WorkflowRoleMapping;
import com.ibm.rpm.workflow.managers.AvailableWorkflowProcessManager;
import com.ibm.rpm.workflow.managers.RunningWorkflowProcessManager;
import com.ibm.rpm.workflow.managers.WorkflowManager;

public class WorkflowUtil
{
    public static String getProcessName(WorkflowProcess process,
            MessageContext messageContext) throws RPMException
    {
        // SELECT element_name FROM tmt_workflow_cycles
        // where element_id = ?        
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(WbsManager.NAME_ELEMENT_ID);
        
        Object[] arguments = new Object[]
        {
            process.getID()
        };
        
        String name = (String) ManagerUtil.selectColumnValue(Integer.class,
            process, ValidationConstants.NAME_FIELD,
            WbsManager.NAME_ELEMENT_NAME, WorkflowManager.TABLE_NAME,
            condition, arguments, messageContext);

        return name != null ? name.trim() : name;
    }

    public static boolean isProcessActive(WorkflowProcess process,
            MessageContext messageContext) throws RPMException
    {
        // select config_flag from TMT_WORKFLOW_CYCLES where
        // element_id = ?
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(WbsManager.NAME_ELEMENT_ID);
        
        Object[] arguments = new Object[]
        {
            process.getID()
        };
        
        Integer value = (Integer) ManagerUtil.selectColumnValue(Integer.class,
            process, ValidationConstants.ACTIVE_FIELD,
            WbsManager.NAME_CONFIG_FLAG, WorkflowManager.TABLE_NAME, condition,
            arguments, messageContext);

        boolean active = (value.intValue() == 3);

        return active;
    }

    public static WorkflowState loadWorkflowProcessState(
            RunningWorkflowProcess runningProcess, MessageContext messageContext)
            throws RPMException
    {
        // SELECT end_cycle FROM tmt_workflows WHERE element_id = ?
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(CommunicationManager.NAME_ELEMENT_ID);

        Object[] arguments = new Object[]
        {
            runningProcess.getID()
        };
        
        String index = (String) ManagerUtil.selectColumnValue(String.class,
            runningProcess, ValidationConstants.STATE_FIELD,
            CommunicationManager.NAME_END_CYCLE,
            CommunicationManager.TABLE_NAME, condition, arguments,
            messageContext);

        WorkflowState state = (WorkflowState) EnumeratedTypeUtil.getInstance()
            .getType(null, WorkflowState.class, new Integer(index));

        return state;
    }

    public static String loadWorkflowRank(WorkflowProcess process,
            MessageContext messageContext) throws RPMException
    {
        // SELECT rank FROM tmt_workflow_cycles where element_id = ?
        String rank = null;

        if (process != null)
        {
            SqlBuffer condition = new SqlBuffer();
            condition.appendEqualQuestionMark(WorkflowManager.NAME_ELEMENT_ID);
            
            Object[] arguments = new Object[]
            {
                process.getID()
            };
            
            rank = (String) ManagerUtil.selectColumnValue(String.class,
                process, null, WorkflowManager.NAME_RANK,
                WorkflowManager.TABLE_NAME, condition, arguments,
                messageContext);
        }
        return rank;
    }

    public static String loadAssociatedProjectID(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        String associatedProjectID = null;

        if (rpmObject instanceof GenericProject)
        {
            associatedProjectID = rpmObject.getID();
        }
        else if (rpmObject instanceof WorkElement)
        {
            if (rpmObject.getParent() != null
                    && rpmObject.getParent() instanceof GenericProject)
            {
                associatedProjectID = rpmObject.getParent().getID();
            }

            if (associatedProjectID == null)
            {
                SqlBuffer condition = new SqlBuffer();
                condition.appendEqualQuestionMark(WbsManager.NAME_ELEMENT_ID);
                
                Object[] arguments = new Object[]
                {
                    rpmObject.getID()
                };
                
                associatedProjectID = (String) ManagerUtil
                    .selectColumnValue(String.class, rpmObject, null,
                        WbsManager.NAME_PROJECT_ID, WbsManager.TABLE_NAME,
                        condition, arguments, messageContext);
            }
        }
        else if (rpmObject instanceof AbstractScope)
        {
            SqlBuffer condition = new SqlBuffer();
            condition.appendEqualQuestionMark(ScopeManager.NAME_ELEMENT_ID);
            
            Object[] arguments = new Object[]
            {
                rpmObject.getID()
            };
            
            associatedProjectID = (String) ManagerUtil.selectColumnValue(
                String.class, rpmObject, null, ScopeManager.NAME_PROJECT_ID,
                ScopeManager.TABLE_NAME, condition, arguments, messageContext);
        }
        else if (rpmObject instanceof GenericDocument)
        {
            SqlBuffer condition = new SqlBuffer();
            condition.appendEqualQuestionMark(DocumentManager.NAME_ELEMENT_ID);
            
            Object[] arguments = new Object[]
            {
                rpmObject.getID()
            };
            
            associatedProjectID = (String) ManagerUtil.selectColumnValue(
                String.class, rpmObject, null, DocumentManager.NAME_PROJECT_ID,
                DocumentUtil.getInstance().getTableName(
                    ContextUtil.UMBRELLA_DOCUMENT_CONTEXT), condition,
                arguments, messageContext);
        }

        return associatedProjectID;
    }

    public static boolean isWorkflowAssociated(WorkflowProcess process,
            String projectId, MessageContext messageContext)
            throws RPMException
    {
        // SELECT config_flag from PROJECT_WKF_PROCESS 
        // WHERE PROJECT_ID = ? AND PROCESS_ID = ?
        boolean associated = false;

        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(AvailableWorkflowProcessManager.NAME_PROCESS_ID);
        condition.appendEqualQuestionMark(AvailableWorkflowProcessManager.NAME_PROJECT_ID);
        
        Object[] arguments = new Object[]
        {
                process.getID(), projectId
        };
        
        Integer configFlag = (Integer) ManagerUtil.selectColumnValue(
            Integer.class, process, null,
            AvailableWorkflowProcessManager.NAME_CONFIG_FLAG,
            AvailableWorkflowProcessManager.TABLE_NAME, condition, arguments,
            messageContext);

        if (configFlag != null
                && (configFlag.intValue() == 1 || configFlag.intValue() == 3))
        {
            associated = true;
        }

        return associated;
    }

    public static String loadWorkflowProcessRankThroughParent(
            RPMObject rpmObject, Integer groupTypeId,
            MessageContext messageContext) throws RPMException
    {
        // load the rank by the parent type ID
        String rank = null;

        if (groupTypeId != null)
        {
            // SELECT RANK FROM TMT_WORKFLOW_CYCLES
            // WHERE PARENT_ID IS NULL AND TYPE_ID = ?
            SqlBuffer condition = new SqlBuffer();
            condition.appendIsNull(WorkflowManager.NAME_PARENT_ID);
            condition.appendEqualQuestionMark(WorkflowManager.NAME_TYPE_ID);
            
            Object[] arguments = new Object[]
            {
                groupTypeId
            };

            rank = (String) ManagerUtil.selectColumnValue(String.class,
                rpmObject, null, WorkflowManager.NAME_RANK,
                WorkflowManager.TABLE_NAME, condition, arguments,
                messageContext);
        }

        return rank;
    }

    public static WorkflowRoleMapping[] mergeLoadedWorkflowMappings(
            ArrayList listIDs, ArrayList listNames, ArrayList mappings,
            GenericProject genericProject)
    {
        ArrayList returnMappings = new ArrayList();

        // add the list of loaded mappings to the return list
        if (mappings != null)
        {
            returnMappings.addAll(mappings);
        }

        if (listIDs != null && listNames != null)
        {
            Iterator ids = listIDs.iterator();
            Iterator names = listNames.iterator();

            // iterate through the IDs returned by the stored proc and check if
            // they were already added to the return list; if not, add them
            while (ids.hasNext())
            {
                String roleId = (String) ids.next();
                String roleName = (String) names.next();
                boolean found = false;

                Iterator iterator = returnMappings.iterator();
                while (iterator.hasNext())
                {
                    WorkflowRoleMapping mapping = (WorkflowRoleMapping) iterator
                        .next();

                    if (mapping.getWorkflowRole() != null
                            && mapping.getWorkflowRole() != null
                            && mapping.getWorkflowRole().getID().equals(roleId))
                    {
                        found = true;
                        break;
                    }
                }

                if (!found)
                {
                    // not already loaded; add the data returned by the proc 
                    // to the return list
                    WorkflowRole role = new WorkflowRole();
                    role.setName(roleName);
                    role.setID(roleId);

                    WorkflowRoleMapping newMapping = new WorkflowRoleMapping();
                    newMapping.setParent(genericProject);
                    newMapping.setWorkflowRole(role);

                    returnMappings.add(newMapping);
                }
            }
        }

        return (WorkflowRoleMapping[]) returnMappings
            .toArray(new WorkflowRoleMapping[0]);
    }

    /*
     * Finds and returns the RunningWorkflowProcess that contains a WorkflowProcess
     * with the given WorkflowProcess ID.
     */
    public static RunningWorkflowProcess findRunningProcessWithId(
            RunningWorkflowProcess[] runningProcesses, String processId)
    {
        RunningWorkflowProcess runningProcess = null;

        if (runningProcesses != null)
        {
            for (int i = 0; i < runningProcesses.length; i++)
            {
                if (runningProcesses[i].getWorkflowProcess() != null
                        && processId.equals(runningProcesses[i]
                            .getWorkflowProcess().getID()))
                {
                    runningProcess = runningProcesses[i];
                    break;
                }
            }
        }
        return runningProcess;
    }

    public static Integer loadGroupIdFromRunningProcessId(
            RunningWorkflowProcess runningProcess, MessageContext messageContext)
            throws RPMException
    {
        // SELECT wkf_group_id FROM tmt_workflows where element_id = ?
        Integer groupId = null;

        if (runningProcess != null && runningProcess.getID() != null)
        {
            SqlBuffer condition = new SqlBuffer();
            condition.appendEqualQuestionMark(
                RunningWorkflowProcessManager.NAME_ELEMENT_ID);
            
            Object[] arguments = new Object[]
            {
                runningProcess.getID()
            };
            
            groupId = (Integer) ManagerUtil.selectColumnValue(String.class,
                runningProcess, null,
                RunningWorkflowProcessManager.NAME_WKF_GROUP_ID,
                RunningWorkflowProcessManager.TABLE_NAME, condition, arguments,
                messageContext);
        }
        return groupId;
    }    
    
    public static boolean isRunningWorkflowProcessManual(
            RunningWorkflowProcess runningWorkflowProcess,
            MessageContext messageContext) throws RPMException
    {
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(RunningWorkflowProcessManager.NAME_ELEMENT_ID);
        
        Object[] arguments = new Object[]
        {
            runningWorkflowProcess.getID()
        };
        
        Integer status = (Integer) ManagerUtil.selectColumnValue(Integer.class,
            runningWorkflowProcess, null,
            RunningWorkflowProcessManager.NAME_WKF_STATUS,
            RunningWorkflowProcessManager.TABLE_NAME, condition, arguments,
            messageContext);

        if (status == null)
        {
            throw new RPMException(
                "Problem getting RunningWorkflowProcess status from DB");
        }

        return ((status.intValue() & 2) == 2);
    }
    
}