package com.ibm.rpm.xpathparser;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;

/**
 * @author gmeloche
 */
public class ContainerFieldMap
{
    private static ContainerFieldMap instance = null;

    private HashMap containerMap = new HashMap();

    private static ArrayList excludeList = new ArrayList();

    static
    {
        excludeList.add("getClass");
        excludeList.add("getDeleted");
        excludeList.add("getParent");
        excludeList.add("getContextName");
    }

    private ContainerFieldMap()
    {
        super();
    }

    public static ContainerFieldMap getInstance()
    {

        if (instance == null)
        {
            instance = new ContainerFieldMap();
        }

        return instance;
    }

    public synchronized HashMap getFieldMap(Class containerType)
            throws RPMException
    {

        HashMap fields = (HashMap) containerMap.get(containerType);

        if (fields == null)
        {
            fields = createFieldMap(containerType);
            containerMap.put(containerType, fields);
        }

        return fields;
    }

    private HashMap createFieldMap(Class containerType) throws RPMException
    {

        HashMap fields = new HashMap();

        Method[] methods = containerType.getMethods();

        for (int i = 0; i < methods.length; i++)
        {
            Method method = methods[i];
            String methodName = method.getName();
            if ((methodName.startsWith("get") || methodName.startsWith("is"))
                    && !excludeList.contains(methodName))
            {
                Class fieldType = method.getReturnType();

                if (fieldType.isArray())
                {
                    fieldType = fieldType.getComponentType();
                }

                //Only RPMObject fields are added for now
                //if (RPMObject.class.isAssignableFrom(fieldType))
                //{

                    String fieldname = null;
                    if (methodName.startsWith("get"))
                    {
                        fieldname = method.getName().substring(3).toLowerCase();
                    }
                    else
                    {
                        fieldname = method.getName().substring(2).toLowerCase();
                    }

                    fields.put(fieldname, fieldType);
                //}
            }
        }

        return fields;
    }

    /**
     * 
     * @param containerType
     * @param fieldName
     * @return
     * @throws RPMException
     */
    public boolean fieldExistInContainer(Class containerType, String fieldName)
            throws RPMException
    {
        boolean exist = false;

        HashMap fields = getFieldMap(containerType);

        exist = fields.containsKey(fieldName.toUpperCase());

        return exist;
    }

    /**
     * 
     * @param containerType
     * @param fieldName
     * @return
     * @throws RPMException
     */
    public Class getFieldType(Class containerType, String fieldName)
            throws RPMException
    {
        Class type = null;

        HashMap fields = getFieldMap(containerType);

        if (fields != null)
        {
            type = (Class) fields.get(fieldName.toLowerCase());
        }
        return type;
    }

}
