package com.ibm.rpm.xpathparser;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Stack;

import com.ibm.rpm.framework.util.StringUtil;

public class Table implements Cloneable
{
    public Table(String objectName, Table parent)
    {
        this.objectName = objectName;
        this.parent = parent;
    }

    private String objectName = null;

    private Table parent = null;

    private Predicate predicate = null;

    private ArrayList conditions = null;

    private String fieldName = null;

    private Class objectType = null;

    public String getObjectName()
    {
        return objectName;
    }

    public Table getParent()
    {
        return parent;
    }

    public String toString()
    {
        StringBuffer buffer = new StringBuffer("\n***Object:" + objectName + "***");
        buffer.append("\nType:");
        if (objectType != null)
            buffer.append(objectType.getName());
        buffer.append("\nField:");
        if (fieldName != null)
            buffer.append(fieldName);
        buffer.append("\nParent:");
        if (parent != null)
            buffer.append(parent.getObjectName());
        buffer.append("\nPredicates:");
        if (predicate != null)
        {
            if (conditions == null)
            {
                buffer.append(predicate.toString());
            }
            else
            {
                buffer.append("see Conditions.");
            }
        }
        buffer.append("\nConditions:");
        if (conditions != null)
            buffer.append(conditions.toString());
        buffer.append("\n");
        return buffer.toString();
    }

    /**
     * @param currentPredicate
     */
    public void setPredicate(Predicate currentPredicate)
    {
        this.predicate = currentPredicate;
    }

    public Predicate getPredicate()
    {
        return this.predicate;
    }

    public Object clone()
    {

        Table t = null;

        try
        {
            t = (Table) super.clone();
            if(predicate != null){
                predicate = (Predicate) predicate.clone();
            }
            if(conditions != null){
                ArrayList newConditions = new ArrayList();
                for (Iterator iter = conditions.iterator(); iter.hasNext();)
                {
                    Condition condition = (Condition) iter.next();
                    Condition newCondition = (Condition)condition.clone();
                    newConditions.add(newCondition);                   
                }
                conditions = newConditions;
            }
            
        }
        catch (CloneNotSupportedException e)
        {
            //Do nothing
        }

        return t;
    }

    public static Stack cloneTableStack(Stack initialStack)
    {

        Stack newStack = new Stack();

        if (!initialStack.isEmpty())
        {

            Iterator iterator = initialStack.iterator();

            while (iterator.hasNext())
            {
                Table initial = (Table) iterator.next();
                Table t = (Table) initial.clone();
                newStack.insertElementAt(t, newStack.size());

            }
        }

        return newStack;
    }

    /**
     * @return Returns the fieldName.
     */
    public String getFieldName()
    {
        return fieldName;
    }

    /**
     * @param fieldName
     *            The fieldName to set.
     */
    public void setFieldName(String fieldName)
    {
        this.fieldName = fieldName;
    }

    /**
     * @return Returns the objectType.
     */
    public Class getObjectType()
    {
        return objectType;
    }

    /**
     * @param objectType
     *            The objectType to set.
     */
    public void setObjectType(Class objectType)
    {
        this.objectType = objectType;
        this.objectName = StringUtil.getShortClassName(objectType);
    }

    /**
     * @return Returns the conditions.
     */
    public ArrayList getConditions()
    {
        return conditions;
    }

    /**
     * @param conditions
     *            The conditions to set.
     */
    public void setConditions(ArrayList conditions)
    {
        this.conditions = conditions;
    }
}