package com.ibm.rpm.xpathparser.steps;

import org.apache.xml.utils.ObjectVector;
import org.apache.xpath.XPathException;
import org.apache.xpath.compiler.OpMapVector;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.xpathparser.FieldCondition;
import com.ibm.rpm.xpathparser.Predicate;
import com.ibm.rpm.xpathparser.ReflectOpCodes;
import com.ibm.rpm.xpathparser.Table;

public abstract class Comparator extends XPathStep
{

    private FieldCondition condition = null;

    /**
     * @param op_xpath
     * @param opMap
     * @param tokenQueue
     * @param parent
     */
    public Comparator(int op_xpath, OpMapVector opMap, ObjectVector tokenQueue,
            XPathStep parent)
    {
        super(op_xpath, opMap, tokenQueue, parent);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.xpathparser.steps.XPathStep#notifyPostBuild(com.ibm.rpm.xpathparser.steps.XPathStep)
     */
    protected void notifyPreBuild(XPathStep predicate) throws XPathException,
            RPMException
    {
        XPathStep parent = getParent();
        int operation = parent.getOperation();
        Predicate pred = null;

        condition = new FieldCondition(getOperation());

        if (operation == ReflectOpCodes.OP_PREDICATE)
        {
            Table tparent = ((From) parent.getParent()).getTable();
            pred = tparent.getPredicate();
            if (pred == null)
            {

                pred = new Predicate(Predicate.CONDITION);
                tparent.setPredicate(pred);
            }
            else
            {
                /*
                 * Note: at this point the predicate should always be null but
                 * this is needed to support old syntax (/parent[id
                 * ='0'][child])
                 */
                pred.setType(ReflectOpCodes.OP_AND);
            }
        }
        else if (operation == ReflectOpCodes.OP_AND
                || operation == ReflectOpCodes.OP_OR
                || operation == ReflectOpCodes.OP_FUNCTION)
        {
            pred = findPredicate(parent);

        }
        else
        {
            String[] params = new String[]
            {
                "Operation " + operation + " not supported"
            };
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, params);
        }
        

        pred.add(condition);

    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.xpathparser.steps.XPathStep#notifyPreBuild(com.ibm.rpm.xpathparser.steps.XPathStep)
     */
    protected void notifyPostBuild(XPathStep predicate)
    {

    }

    public int build(int position) throws XPathException, RPMException
    {
        setLength(opMap.elementAt(++position) - 2);
        notifyPreBuild(this);
        position = super.build(++position);
        notifyPostBuild(this);
        return position;
    }

    /**
     * @return Returns the condition.
     */
    public FieldCondition getCondition()
    {
        return condition;
    }

    /**
     * @param condition
     *            The condition to set.
     */
    protected void setCondition(FieldCondition condition)
    {
        this.condition = condition;
    }
}
