package com.ibm.rpm.xpathparser.steps;

import org.apache.xml.utils.ObjectVector;
import org.apache.xpath.XPathException;
import org.apache.xpath.compiler.OpMapVector;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.xpathparser.Predicate;
import com.ibm.rpm.xpathparser.ReflectOpCodes;
import com.ibm.rpm.xpathparser.Table;

/**
 * @author kuschel
 * 
 * [OP_GROUP] [length] {expression}
 * 
 * returns: XNodeSet XNumber XString XBoolean XRTree XObject
 * @xsl.usage advanced
 */
public class Group extends XPathStep
{
    private Predicate predicate = null;

    /**
     * @param op_xpath
     * @param opMap
     * @param tokenQueue
     * @param step
     */
    protected Group(OpMapVector opMap, ObjectVector tokenQueue, XPathStep step)
    {
        this(ReflectOpCodes.OP_GROUP, opMap, tokenQueue, step);
        // TODO Auto-generated constructor stub
    }

    /**
     * @param op_and
     * @param opMap
     * @param tokenQueue
     * @param step
     */
    protected Group(int op_and, OpMapVector opMap, ObjectVector tokenQueue,
            XPathStep step)
    {
        super(op_and, opMap, tokenQueue, step);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.xpathparser.steps.XPathStep#notifyPostBuild(com.ibm.rpm.xpathparser.steps.Predicate)
     */
    protected void notifyPreBuild(XPathStep predicateStep)
            throws XPathException, RPMException
    {
        XPathStep parent = getParent();        
        int operationCode = parent.getOperation();
        Predicate parentPredicate = null;
        boolean isGroup = false;
        
        if (operationCode == ReflectOpCodes.OP_FUNCTION)
        {
            parent = parent.getParent();
            operationCode = parent.getOperation();
        }
        
        if (operationCode == ReflectOpCodes.OP_AND
                || operationCode == ReflectOpCodes.OP_OR)
        {
            isGroup = true;
            parentPredicate = ((Group)parent).getPredicate();
            parent = findGroupParent(parent);
            operationCode = parent.getOperation();
        }
        
        if (operationCode == ReflectOpCodes.OP_PREDICATE)
        {
            Table ptable = ((From) parent.getParent()).getTable();
            this.predicate = new Predicate(getOperation());
            if(isGroup){
                if(parentPredicate == null){
                    String[] params = new String[]
                    {
                        predicate + " should be defined: " + ptable
                    };
                    throw new RPMException(ErrorCodes.INTERNAL_ERROR, params);
                    
                }
                parentPredicate.add(this.predicate);
            }else{
                ptable.setPredicate(this.predicate);
            }
        }
        else if (operationCode == ReflectOpCodes.OP_XPATH)
        {
            throw new RPMException(ErrorCodes.XPATH_INVALID_GROUP);
        }
        else
        {
            String[] params = new String[]
            {
                "Operation " + operationCode + " not supported"
            };
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, params);
        }
    }

    /**
     * @return
     */
    public Predicate getPredicate()
    {
        return predicate;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.rpm.xpathparser.steps.XPathStep#notifyPreBuild(com.ibm.rpm.xpathparser.steps.Predicate)
     */
    protected void notifyPostBuild(XPathStep predicateStep)
    {

    }

    public int build(int position) throws XPathException, RPMException
    {
        setLength(opMap.elementAt(++position) - 2);
        notifyPreBuild(this);
        position = super.build(++position);
        notifyPostBuild(this);
        return position;
    }

    /**
     * @param predicate
     *            The predicate to set.
     */
    protected void setPredicate(Predicate predicate)
    {
        this.predicate = predicate;
    }
}
