package com.ibm.rpm.xpathparser.steps;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Stack;

import org.apache.xml.utils.ObjectVector;
import org.apache.xpath.XPathException;
import org.apache.xpath.compiler.OpMapVector;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.xpathparser.steps.XPathPredicate;
import com.ibm.rpm.xpathparser.Predicate;
import com.ibm.rpm.xpathparser.ReflectOpCodes;
import com.ibm.rpm.xpathparser.Table;

public abstract class XPathStep
{
    protected OpMapVector opMap = null;

    private int operation = 0;

    protected ObjectVector tokenQueue = null;

    private XPathStep parent = null;

    /**
     * @param op_xpath
     * @param tokenQueue
     * @param parent
     * @param opMap2
     */
    protected XPathStep(int op_xpath, OpMapVector opMap,
            ObjectVector tokenQueue, XPathStep parent)
    {
        this.parent = parent;
        this.opMap = opMap;
        this.operation = op_xpath;
        this.opMap = opMap;
        this.tokenQueue = tokenQueue;
    }

    private int length = 0;

    private ArrayList children = new ArrayList();

    public void setLength(int length)
    {
        this.length = length;
    }

    public int getLength()
    {
        return length;
    }

    public XPathStep[] getChildren()
    {
        return (XPathStep[]) children.toArray(new XPathStep[0]);
    }

    Table mytable = null;

    Stack lstack = null;

    public int build(int position) throws XPathException, RPMException
    {
        int opcode = 0;
        boolean breakit = false;
        int rlength = getLength() + position;
        XPathStep unit = null;
        while ((opcode = opMap.elementAt(position)) != 0)
        {
            if (position >= rlength)
                break;

            if (opcode == ReflectOpCodes.OP_PREDICATE)
            {
                unit = new XPathPredicate(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_OR)
            {
                unit = new Or(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_AND)
            {
                unit = new And(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_NOTEQUALS)
            {
                unit = new NotEquals(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_EQUALS)
            {
                unit = new Equals(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_LITERAL)
            {
                unit = new Literal(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_NUMBERLIT)
            {
                unit = new NumberLiteral(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_GROUP)
            {
                unit = new Group(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_LOCATIONPATH)
            {
                unit = new LocationPath(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.NODENAME)
            {
                unit = new Name(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.NODETYPE_ROOT)
            {
                unit = new Root(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.FROM_ROOT)
            {
                unit = new FromRoot(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.FROM_CHILDREN)
            {
                unit = new FromChildren(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.FROM_DESCENDANTS_OR_SELF)
            {
                unit = new FromDescendantsOrSelf(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_LT)
            {
                unit = new LessThen(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_LTE)
            {
                unit = new LessThenEquals(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_GT)
            {
                unit = new GreaterThen(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_GTE)
            {
                unit = new GreaterThenEquals(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_FUNCTION)
            {
                unit = new Function(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.OP_ARGUMENT)
            {
                unit = new Argument(opMap, tokenQueue, this);
            }
            else if (opcode == ReflectOpCodes.FROM_ATTRIBUTES)
            {
                unit = new FromAttribute(opMap, tokenQueue, this);
            }
            else if (opcode == 0)
            {
                breakit = true;
            }
            /**
             * [ENDOP] Some operators may like to have a terminator.
             * 
             * @xsl.usage advanced
             */
            else if (opcode == ReflectOpCodes.ENDOP)
            {
                position++;
                continue;
            }

            /**
             * [ELEMWILDCARD] Means ELEMWILDCARD ("*"), used instead of string
             * index in some places.
             * 
             * @xsl.usage advanced else if( opcode == ReflectOpCodes.ELEMWILDCARD :
             *            list.add(new XPathStep(this.parser, "Element
             *            Wildcards", opcode, 0, null, ++this.parser.start)); }
             */

            /**
             * [OP_LTE] (less-than-or-equals) [length] {number expression}
             * {number expression}
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */

            /*
             * else if( opcode == ReflectOpCodes.OP_LTE : list .add(new
             * XPathStep(this.parser, "Operation Less than equals", opcode,
             * this.parser.omap.elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */
            /**
             * [OP_LT] (less-than) [length] {number expression} {number
             * expression}
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_LT : list.add(new
             * XPathStep(this.parser, "Operation Less than", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */
            /**
             * [OP_GTE] (greater-than-or-equals) [length] {number expression}
             * {number expression}
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_GTE : list .add(new
             * XPathStep(this.parser, "Operation Greater than equals", opcode,
             * this.parser.omap.elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */

            /**
             * [OP_GT] (greater-than) [length] {number expression} {number
             * expression}
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_GT : list.add(new
             * XPathStep(this.parser, "Operation Greater than", opcode,
             * this.parser.omap.elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */

            /**
             * [OP_PLUS] [length] {number expression} {number expression}
             * 
             * returns : XNumber
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_PLUS : list.add(new
             * XPathStep(this.parser, "Operation Plus", opcode, this.parser.omap
             * .elementAt(++this.parser.start) - 2, null, ++this.parser.start)); }
             */

            /**
             * [OP_MINUS] [length] {number expression} {number expression}
             * 
             * returns : XNumber
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_MINUS : list.add(new
             * XPathStep(this.parser, "Operation Minus", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */

            /**
             * [OP_MULT] [length] {number expression} {number expression}
             * 
             * returns : XNumber
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_MULT : list.add(new
             * XPathStep(this.parser, "Operation Multiply", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */

            /**
             * [OP_DIV] [length] {number expression} {number expression}
             * 
             * returns : XNumber
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_DIV : list.add(new
             * XPathStep(this.parser, "Operation Divide", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */
            /**
             * [OP_MOD] [length] {number expression} {number expression}
             * 
             * returns : XNumber
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_MOD : list.add(new
             * XPathStep(this.parser, "Operation Modulus", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */
            /**
             * [OP_QUO] [length] {number expression} {number expression}
             * 
             * returns : XNumber
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_QUO : list.add(new
             * XPathStep(this.parser, "Operation Quotient", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */

            /**
             * [OP_NEG] [length] {number expression}
             * 
             * returns : XNumber
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_NEG : list.add(new
             * XPathStep(this.parser, "Operation Negative", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */
            /**
             * [OP_STRING] (cast operation) [length] {expression}
             * 
             * returns : XString
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_STRING : list.add(new
             * XPathStep(this.parser, "Operation String", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */
            /**
             * [OP_BOOL] (cast operation) [length] {expression}
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_BOOL : list.add(new
             * XPathStep(this.parser, "Operation Boolean", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */

            /**
             * [OP_NUMBER] (cast operation) [length] {expression}
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_NUMBER : list.add(new
             * XPathStep(this.parser, "Operation Number", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */

            /**
             * [OP_UNION] [length] {PathExpr}+
             * 
             * returns : XNodeSet
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_UNION : list.add(new
             * XPathStep(this.parser, "Operation Union", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */

            /**
             * [OP_VARIABLE] [4] [index to namespace token, or EMPTY] [index to
             * function name token]
             * 
             * returns : XString
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_VARIABLE : list.add(new
             * XPathStep(this.parser, "Operation Variable", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 4, (String)
             * this.parser.tqueue
             * .elementAt(this.parser.omap.elementAt(this.parser.start += 2)),
             * ++this.parser.start)); }
             */
            /**
             *  
             */

            /**
             * [OP_EXTFUNCTION] (Extension function.) [length] [index to
             * namespace token] [index to function name token] {OP_ARGUMENT}
             * 
             * returns : XNodeSet XNumber XString XBoolean XRTree XObject
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_EXTFUNCTION : list.add(new
             * XPathStep(this.parser, "Operation Extension Function", opcode,
             * this.parser.omap.elementAt(++this.parser.start) - 4, (String)
             * this.parser.tqueue.elementAt(this.parser.omap
             * .elementAt(this.parser.start += 2)), ++this.parser.start)); }
             */
            /**
             * [OP_FUNCTION] [length] [FUNC_name] {OP_ARGUMENT} [ENDOP]
             * 
             * returns : XNodeSet XNumber XString XBoolean XRTree XObject
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_FUNCTION : list.add(new
             * XPathStep(this.parser, "Operation Function", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 3,
             * String.valueOf(this.parser.omap .elementAt(++this.parser.start)),
             * ++this.parser.start)); }
             */

            /**
             * [OP_ARGUMENT] (Function argument.) [length] {expression}
             * 
             * returns : XNodeSet XNumber XString XBoolean XRTree XObject
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_ARGUMENT : list.add(new
             * XPathStep(this.parser, "Operation Argument", opcode,
             * this.parser.omap .elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */
            /**
             * [OP_NUMBERLIT] (Number literal.) [3] [index to token]
             * 
             * returns : XString
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_NUMBERLIT : list.add(new
             * XPathStep(this.parser, "Operation Number Literab", opcode,
             * this.parser.omap.elementAt(++this.parser.start) - 3, (String)
             * this.parser.tqueue
             * .elementAt(this.parser.omap.elementAt(++this.parser.start)),
             * ++this.parser.start)); }
             */
            // else if( opcode == ReflectOpCodes.LOCATIONPATHEX_MASK = 0x0000FFFF;
            // else if( opcode == ReflectOpCodes.LOCATIONPATHEX_ISSIMPLE = 0x00010000;
            // else if( opcode == ReflectOpCodes.OP_LOCATIONPATH_EX = (28 |
            // 0x00010000);
            /**
             * [OP_MATCHPATTERN] [length] {PathExpr}+
             * 
             * returns : XNodeSet
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_MATCHPATTERN : list.add(new
             * XPathStep(this.parser, "Operation Match Pattern", opcode,
             * this.parser.omap.elementAt(++this.parser.start) - 2, null,
             * ++this.parser.start)); }
             */
            /**
             * [OP_LOCATIONPATHPATTERN] [length] {FROM_stepType} |
             * {function}{predicate} [ENDOP] returns : XNodeSet
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.OP_LOCATIONPATHPATTERN : list
             * .add(new XPathStep( this.parser, "Operation Location Path
             * Pattern", opcode, this.parser.omap.elementAt(++this.parser.start) -
             * 2, null, ++this.parser.start)); }
             */
            /**
             * [NODETYPE_COMMENT] No size or arguments. Note : must not
             * overlap function OP number!
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.NODETYPE_COMMENT : list.add(new
             * XPathStep(this.parser, "Node Type Comment", opcode, 0, null,
             * ++this.parser.start)); }
             */
            /**
             * [NODETYPE_TEXT] No size or arguments. Note : must not overlap
             * function OP number!
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.NODETYPE_TEXT : list.add(new
             * XPathStep(this.parser, "Node Type Text", opcode, 0, null,
             * ++this.parser.start)); }
             */
            /**
             * [NODETYPE_PI] [index to token] Note : must not overlap function
             * OP number!
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.NODETYPE_PI : list.add(new
             * XPathStep(this.parser, "Node Type PI", opcode, 0, String
             * .valueOf(this.parser.omap.elementAt(++this.parser.start)),
             * ++this.parser.start)); }
             */

            /**
             * [NODETYPE_NODE] No size or arguments. Note : must not overlap
             * function OP number!
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.NODETYPE_NODE : list.add(new
             * XPathStep(this.parser, "Node Type Node", opcode, 0, null,
             * ++this.parser.start)); }
             */

            /**
             * [NODENAME] [index to ns token or EMPTY] [index to name token]
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */

            /**
             * [NODETYPE_ROOT] No size or arguments.
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */

            /**
             * [NODETYPE_ANY] No size or arguments.
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.NODETYPE_ANYELEMENT : list.add(new
             * XPathStep(this.parser, "Node Type Any Element", opcode, 0, null,
             * ++this.parser.start)); }
             */

            /**
             * [NODETYPE_ANY] No size or arguments.
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */
            /*
             * else if( opcode == ReflectOpCodes.NODETYPE_FUNCTEST : list.add(new
             * XPathStep(this.parser, "Node Type Function Test", opcode, 0,
             * null, ++this.parser.start)); }
             */
            /**
             * [FROM_stepType] [length, including predicates] [length of just
             * the step, without the predicates] {node test} {predicates}?
             * 
             * returns : XBoolean
             * 
             * @xsl.usage advanced
             */

            else
            {
                String name = "operation code = " + opcode;
                String[] params = new String[]{
                       name
                };
                throw new RPMException(ErrorCodes.XPATH_UNSUPPORTED_OPERATION_CODE, params);
            }
            if (breakit)
                break;
            children.add(unit);
            position = unit.build(position);
        }
        return position;
    }

    /**
     * @param predicate
     * @throws XPathException
     */
    protected abstract void notifyPostBuild(XPathStep predicate)
            throws XPathException, RPMException;

    /**
     * @param predicate
     * @throws XPathException
     */
    protected abstract void notifyPreBuild(XPathStep predicate)
            throws XPathException, RPMException;

    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        buffer.append("Name(" + String.valueOf(operation) + "):"
                + getClass().getName());
        buffer.append(" Length: " + String.valueOf(length));
        buffer.append(System.getProperty("line.separator"));
        if (!children.isEmpty())
        {
            buffer.append("[" + System.getProperty("line.separator"));
            Iterator iter = children.iterator();
            while (iter.hasNext())
            {
                buffer.append("\t" + (((XPathStep) iter.next()).toString()));
            }
            buffer.append("]" + System.getProperty("line.separator"));
        }
        return buffer.toString();
    }

    /**
     * @return Returns the parent.
     */
    public XPathStep getParent()
    {
        return parent;
    }

    /**
     * @param parent
     *            The parent to set.
     */
    protected void setParent(XPathStep parent)
    {
        this.parent = parent;
    }

    /**
     * @return Returns the operation.
     */
    public int getOperation()
    {
        return operation;
    }

    /**
     * @param operation
     *            The operation to set.
     */
    protected void setOperation(int operation)
    {
        this.operation = operation;
    }
    
    protected XPathStep findGroupParent(XPathStep step){
        
        XPathStep parent = step;
        int operationCode = parent.getOperation();
        
        if (operationCode == ReflectOpCodes.OP_AND
                || operationCode == ReflectOpCodes.OP_OR
                || operationCode == ReflectOpCodes.OP_FUNCTION)
        {
            parent = findGroupParent(parent.getParent());
        }
        
        return parent;
    }
    
    protected Predicate findPredicate(XPathStep group){
        
        Predicate pred = null;
        
        if(group != null){
            
            if(!(group instanceof Group)){
                pred = findPredicate(group.getParent());
            }else{
                pred = ((Group)group).getPredicate();
            }
        }
        return pred;
    }
}