package com.ibm.rpm.xpathparser.test;

import java.util.ArrayList;
import java.util.Stack;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.xpathparser.ContainerCondition;
import com.ibm.rpm.xpathparser.FieldCondition;
import com.ibm.rpm.xpathparser.Predicate;
import com.ibm.rpm.xpathparser.ReflectOpCodes;
import com.ibm.rpm.xpathparser.Table;
import com.ibm.rpm.xpathparser.XPathToStack;

/**
 * @author gmeloche
 *  
 */
public class TestXPathSyntax extends TestCase
{
    String xpath;

    //Opeators
    private String NONE = (new Predicate(Predicate.CONDITION)).getStringType();
    private String AND = (new Predicate(ReflectOpCodes.OP_AND)).getStringType();
    private String OR = (new Predicate(ReflectOpCodes.OP_OR)).getStringType();
    
    //Operations
    private int EQUALS = ReflectOpCodes.OP_EQUALS;
    private int ISNULL = XPathToStack.FUNC_ISNULL;
    private int ISNOTNULL = XPathToStack.FUNC_ISNOTNULL;

    private static Log logger = LogFactory.getLog(TestXPathSyntax.class);

    private Stack parseXPath() throws Exception
    {
        logger.info(xpath);

        Stack stack = XPathToStack.parse(xpath);

        assertNotNull(stack);

        logger.info(stack.toString());

        return stack;
    }

    private void compareTable(Table table, String objectName,
            int numberOfCondition)
    {
        assertNotNull(table);
        assertEquals(objectName, table.getObjectName());
        ArrayList conditions = table.getConditions();
        if (numberOfCondition == 0)
        {
            assertNull(conditions);
        }
        else
        {
            assertNotNull(conditions);
            assertTrue(conditions.size() == numberOfCondition);
        }
    }

    private void compareFieldCondition(Object condition, String operator,
            String property, int operation, Object value)
    {

        assertTrue(condition instanceof FieldCondition);
        FieldCondition fCond = (FieldCondition) condition;

        assertEquals(operator, fCond.getOperator());
        assertEquals(property, fCond.getProperty());
        assertEquals(operation, fCond.getOperation());
        assertEquals(value, fCond.getValue());
    }

    private void compareContainerCondition(Object condition,
            String operator, String objectName, int numberOfCondition)
    {
        assertTrue(condition instanceof ContainerCondition);
        ContainerCondition cCond = (ContainerCondition) condition;

        assertEquals(operator, cCond.getOperator());
        Stack subTables = cCond.getSubTables();
        Table subtable = (Table) subTables.peek();
        compareTable(subtable, objectName, numberOfCondition);

    }
    
    private void compareFieldConditionInContainerCondition(Object containerCondition, int conditionIndex, String fieldOperator,
            String fieldProperty, int fieldOperation,
            Object fieldValue, String parentName)
    {
        
        assertTrue(containerCondition instanceof ContainerCondition);
        ContainerCondition cCond = (ContainerCondition) containerCondition;

        Stack subTables = cCond.getSubTables();
        Table subtable = (Table) subTables.peek();
        assertEquals(parentName, subtable.getParent().getObjectName());
      
        ArrayList conditions = subtable.getConditions();
        assertTrue(conditions.size() >= conditionIndex + 1);
        
        Object fieldCondition = conditions.get(conditionIndex);

        compareFieldCondition(fieldCondition, fieldOperator, 
            fieldProperty, fieldOperation, fieldValue);

    }

    private Object getContainerConditionInContainerCondition(Object containerCondition, int conditionIndex)
    {
        
        assertTrue(containerCondition instanceof ContainerCondition);
        ContainerCondition cCond = (ContainerCondition) containerCondition;

        Stack subTables = cCond.getSubTables();
        Table subtable = (Table) subTables.peek();
      
        ArrayList conditions = subtable.getConditions();
        assertTrue(conditions.size() >= conditionIndex + 1);
        
        Object subContainerCondition = conditions.get(conditionIndex);
        assertNotNull(subContainerCondition);

        return subContainerCondition;

    }

    public void test1Object() throws Exception
    {

        xpath = "/Object";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Object", 0);
    }

    public void test1ObjectWithCondition() throws Exception
    {

        xpath = "/Object[name = 'objectName']";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Object", 1);
        ArrayList conditions = table1.getConditions();
        assertTrue(conditions.get(0) instanceof FieldCondition);
        compareFieldCondition(conditions.get(0), NONE, "name", EQUALS,
            "'objectName'");
    }

    public void test1ObjectWith1Child() throws Exception
    {

        xpath = "Parent[child[name = 'objectName']]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 1);
        
        ArrayList conditions = table1.getConditions();
        compareContainerCondition(conditions.get(0), NONE, "child", 1);
        compareFieldConditionInContainerCondition(conditions.get(0), 0, NONE, "name", EQUALS, "'objectName'", "Parent");

    }
       
    
    public void test1ObjectWith2Children() throws Exception
    {

        xpath = "Parent[child1[name = 'objectName'] and child2[name = 'object2Name']]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 2);
        
        ArrayList conditions = table1.getConditions();
        compareContainerCondition(conditions.get(0), NONE, "child1", 1);
        compareFieldConditionInContainerCondition(conditions.get(0), 0, NONE, "name", EQUALS, "'objectName'", "Parent");
        compareContainerCondition(conditions.get(1), AND, "child2", 1);
        compareFieldConditionInContainerCondition(conditions.get(1), 0, NONE, "name", EQUALS, "'object2Name'", "Parent");

    }
    
    public void test1ObjectWithFieldAndContainer_OldWay() throws Exception
    {

        xpath = "/Parent[id = 123][child[name = 'objectName']]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 2);
        
        ArrayList conditions = table1.getConditions();
        compareFieldCondition(conditions.get(0), NONE, "id", EQUALS,
        "123");
        compareContainerCondition(conditions.get(1), AND, "child", 1);
        compareFieldConditionInContainerCondition(conditions.get(1), 0, NONE, "name", EQUALS, "'objectName'", "Parent");

    }

    public void test1ObjectWithFieldAndContainer() throws Exception
    {

        xpath = "/Parent[id = 123 and child[name = 'objectName']]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 2);
        
        ArrayList conditions = table1.getConditions();
        compareFieldCondition(conditions.get(0), NONE, "id", EQUALS,
        "123");
        compareContainerCondition(conditions.get(1), AND, "child", 1);
        compareFieldConditionInContainerCondition(conditions.get(1), 0, NONE, "name", EQUALS, "'objectName'", "Parent");
    }

    public void test1ObjectWithContainerAndContainer() throws Exception
    {
    
        xpath = "/Parent[child1[name = 'objectName'] and child2[value = 'objectValue']]";
    
        Stack stack = parseXPath();
    
        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 2);
        
        ArrayList conditions = table1.getConditions();
        compareContainerCondition(conditions.get(0), null, "child1", 1);
        compareFieldConditionInContainerCondition(conditions.get(0), 0, NONE, "name", EQUALS, "'objectName'", "Parent");

        compareContainerCondition(conditions.get(1), AND, "child2", 1);
        compareFieldConditionInContainerCondition(conditions.get(1), 0, NONE, "value", EQUALS, "'objectValue'", "Parent");
    }
    
    public void test1ObjectWithContainerAndField_OldWay() throws Exception
    {

        xpath = "/Parent[child[name = 'objectName']][id = 123]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 2);
        
        ArrayList conditions = table1.getConditions();
        compareContainerCondition(conditions.get(0), NONE, "child", 1);
        compareFieldConditionInContainerCondition(conditions.get(0), 0, NONE, "name", EQUALS, "'objectName'", "Parent");
        
        compareFieldCondition(conditions.get(1), AND, "id", EQUALS,
        "123");


    }

    public void test1ObjectWithContainerAndField() throws Exception
    {

        xpath = "/Parent[child[name = 'objectName'] and id = 123]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 2);
        
        ArrayList conditions = table1.getConditions();
        compareContainerCondition(conditions.get(0), NONE, "child", 1);
        compareFieldConditionInContainerCondition(conditions.get(0), 0, NONE, "name", EQUALS, "'objectName'", "Parent");
        
        compareFieldCondition(conditions.get(1), AND, "id", EQUALS,
        "123");


    }

    public void test1ObjectWithFieldORContainer() throws Exception
    {

        xpath = "/Parent[id = 123 or child[name = 'objectName']]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 2);
        
        ArrayList conditions = table1.getConditions();
        compareFieldCondition(conditions.get(0), NONE, "id", EQUALS,
        "123");
        compareContainerCondition(conditions.get(1), OR, "child", 1);
        compareFieldConditionInContainerCondition(conditions.get(1), 0, NONE, "name", EQUALS, "'objectName'", "Parent");
    }

    public void test1ObjectWith4Conditions() throws Exception
    {

        xpath = "/Parent[field1 = '123' or child1[name = 'objectName'] and field2 = '123' or child2[name = 'objectName']]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 4);
        
        ArrayList conditions = table1.getConditions();
        compareFieldCondition(conditions.get(0), NONE, "field1", EQUALS,
        "'123'");
        
        compareContainerCondition(conditions.get(1), OR, "child1", 1);
        compareFieldConditionInContainerCondition(conditions.get(1), 0, NONE, "name", EQUALS, "'objectName'", "Parent");
        
        compareFieldCondition(conditions.get(2), AND, "field2", EQUALS,
        "'123'");
        
        compareContainerCondition(conditions.get(3), OR, "child2", 1);
        compareFieldConditionInContainerCondition(conditions.get(3), 0, NONE, "name", EQUALS, "'objectName'", "Parent");
    }

    public void test2Objects() throws Exception
    {

        xpath = "Parent/child[name = 'objectName']";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "child", 1);
        
        ArrayList conditions = table1.getConditions();
        compareFieldCondition(conditions.get(0),NONE, "name", EQUALS, "'objectName'" );
        
        stack.pop();
        Table table2 = (Table) stack.peek();
        compareTable(table2, "Parent", 0);

    }
    
    public void test1ObjectWithContainerConditionwith2Objects() throws Exception
    {

        xpath = "Parent[parent2/child[name = 'objectName']]";
        //xpath = "Parent[parent2/child]";
        //xpath = "Parent[parent2[child[name = 'objectName']]]";        
        //xpath = "Parent[parent2[child]]";
        
        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 1);
        
        ArrayList conditions = table1.getConditions();

        compareContainerCondition(conditions.get(0), NONE, "child", 1);        
        compareFieldConditionInContainerCondition(conditions.get(0), 0, NONE, "name", EQUALS, "'objectName'", "parent2");

        //next table
        Stack subStack = ((ContainerCondition)conditions.get(0)).getSubTables();
        subStack.pop();
        Table table2 = (Table)subStack.peek();
        compareTable(table2,"parent2", 0);      

    }
    
    public void test2ObjectsWith4And2Conditions() throws Exception
    {

        xpath = "/Parent1[field1 = 123 or child1[name = 'objectName'] or field2 = 123 or child2[name = 'objectName']]/Parent2[child3[name = 'object3Name'] or field1 = '456']";

        Stack stack = parseXPath();

        //Table 1
        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent2", 2);
        
        ArrayList conditions1 = table1.getConditions();
        
        compareContainerCondition(conditions1.get(0), NONE, "child3", 1);
        compareFieldConditionInContainerCondition(conditions1.get(0), 0, NONE, "name", EQUALS, "'object3Name'", "Parent2");
        
        compareFieldCondition(conditions1.get(1), OR, "field1", EQUALS,
        "'456'");
        
        //Table 2
        stack.pop();
        Table table2 = (Table) stack.peek();
        compareTable(table2, "Parent1", 4);
        
        ArrayList conditions2 = table2.getConditions();
        compareFieldCondition(conditions2.get(0), NONE, "field1", EQUALS,
        "123");
        
        compareContainerCondition(conditions2.get(1), OR, "child1", 1);
        compareFieldConditionInContainerCondition(conditions2.get(1), 0, NONE, "name", EQUALS, "'objectName'", "Parent1");
        
        compareFieldCondition(conditions2.get(2), OR, "field2", EQUALS,
        "123");
        
        compareContainerCondition(conditions2.get(3), OR, "child2", 1);
        compareFieldConditionInContainerCondition(conditions2.get(3), 0, NONE, "name", EQUALS, "'objectName'", "Parent1");

    }
    
    public void test1ObjectWithChildwith2SubConditions() throws Exception
    {

        xpath = "Parent[child[name = 'objectName' or id = 123]]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 1);
        
        ArrayList conditions = table1.getConditions();
        compareContainerCondition(conditions.get(0), NONE, "child", 2);
        compareFieldConditionInContainerCondition(conditions.get(0), 0, NONE, "name", EQUALS, "'objectName'", "Parent");
        compareFieldConditionInContainerCondition(conditions.get(0), 1, OR, "id", EQUALS, "123", "Parent");

    }

    public void test1ObjectWithFieldAnd3LevelSubTable() throws Exception
    {

        xpath = "/Parent[id = 123 and child1[name = 'objectName' and child2[name = 'child2Name' or child3[id = 456]]]]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Parent", 2);
        
        ArrayList conditions = table1.getConditions();
        compareFieldCondition(conditions.get(0), NONE, "id", EQUALS,
        "123");
        compareContainerCondition(conditions.get(1), AND, "child1", 2);
        compareFieldConditionInContainerCondition(conditions.get(1), 0, NONE, "name", EQUALS, "'objectName'", "Parent");
        
        Object subContainerCondition1 = getContainerConditionInContainerCondition(conditions.get(1), 1);
        compareContainerCondition(subContainerCondition1, AND, "child2" , 2);
        compareFieldConditionInContainerCondition(subContainerCondition1, 0, NONE, "name", EQUALS, "'child2Name'", "child1");
        
        Object subContainerCondition2 = getContainerConditionInContainerCondition(subContainerCondition1, 1);
        compareContainerCondition(subContainerCondition2, OR, "child3" , 1);
        compareFieldConditionInContainerCondition(subContainerCondition2, 0, NONE, "id", EQUALS, "456", "child2");
        
    }
    
    public void testIsNull() throws Exception
    {
        xpath = "/Object[isnull(name)]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Object", 1);
        ArrayList conditions = table1.getConditions();
        assertTrue(conditions.get(0) instanceof FieldCondition);
        compareFieldCondition(conditions.get(0), NONE, "name", ISNULL,
            "NULL");
    }
    
    public void testIsNotNull() throws Exception
    {
        xpath = "/Object[isnotnull(name)]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Object", 1);
        ArrayList conditions = table1.getConditions();
        assertTrue(conditions.get(0) instanceof FieldCondition);
        compareFieldCondition(conditions.get(0), NONE, "name", ISNOTNULL,
            "NULL");
    }
    
    public void testIsNullOrField() throws Exception
    {
        xpath = "/Object[isnull(name) or name = 'a']";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Object", 2);
        ArrayList conditions = table1.getConditions();
        assertTrue(conditions.get(0) instanceof FieldCondition);
        compareFieldCondition(conditions.get(0), NONE, "name", ISNULL,
            "NULL");
        assertTrue(conditions.get(1) instanceof FieldCondition);
        compareFieldCondition(conditions.get(1), OR, "name", EQUALS,
            "'a'");
        
    }
    
    public void testFieldOrIsNullOrField() throws Exception
    {
        xpath = "/Object[name = 'b' or isnull(name) or name = 'a']";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Object", 3);
        ArrayList conditions = table1.getConditions();
        assertTrue(conditions.get(0) instanceof FieldCondition);
        compareFieldCondition(conditions.get(0), NONE, "name", EQUALS, "'b'");
        assertTrue(conditions.get(1) instanceof FieldCondition);
        compareFieldCondition(conditions.get(1), OR, "name", ISNULL, "NULL");
        assertTrue(conditions.get(2) instanceof FieldCondition);
        compareFieldCondition(conditions.get(2), OR, "name", EQUALS, "'a'");
    }
    
    public void testObjectWithChildWithNullFieldAndContainerCondition() throws Exception{
    
    	xpath = "/Object[child1[isnull(field1) and child2[name = 'a']]]";

        Stack stack = parseXPath();

        Table table1 = (Table) stack.peek();
        compareTable(table1, "Object", 1);
        ArrayList conditions = table1.getConditions();
        assertTrue(conditions.get(0) instanceof ContainerCondition);
        compareContainerCondition(conditions.get(0), NONE, "child1" , 2);
        

        compareFieldConditionInContainerCondition(conditions.get(0), 0, NONE, "field1", ISNULL, "NULL", "Object");
        
        Object subCondition = getContainerConditionInContainerCondition(conditions.get(0),1);
        compareContainerCondition(subCondition , AND, "child2", 1);
        compareFieldConditionInContainerCondition(subCondition, 0, NONE, "name", EQUALS, "'a'", "child1");

    }
    
    public void testObjectWithChildWithContainerConditionAndNullField() throws Exception{
        
        	xpath = "/Object[child1[child2[name = 'a']] and isnull(field1)]";

            Stack stack = parseXPath();

            Table table1 = (Table) stack.peek();
            compareTable(table1, "Object", 1);
            ArrayList conditions = table1.getConditions();
            assertTrue(conditions.get(0) instanceof ContainerCondition);
            compareContainerCondition(conditions.get(0), NONE, "child1" , 2);
            
            Object subCondition = getContainerConditionInContainerCondition(conditions.get(0),0);
            compareContainerCondition(subCondition , AND, "child2", 1);
            compareFieldConditionInContainerCondition(subCondition, 0, NONE, "name", EQUALS, "'a'", "child1");
            
            compareFieldConditionInContainerCondition(conditions.get(0), 1, NONE, "field1", ISNULL, "NULL", "Object");
            


        }
}
