package com.ibm.rpm.client.junit.financial.testcases.xpath;

import java.util.ArrayList;

import junit.framework.AssertionFailedError;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.framework.testcases.xpath.ObjectXPathTester;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;

/**
 * @author ikusakov
 */
public class TestWorksheetFinancialObjectsXPath extends ObjectXPathTester
{
    public void setUp() throws Exception
    {
        super.setUp();
        ClientTestController.logger.info("setUp");
    }

    public void tearDown() throws Exception
    {
        ClientTestController.logger.info("tearDown");
        super.tearDown();
    }

    // testing some queries manually, until xpath testing framework is complete
    public void testRelations() throws Exception
    {
        RPMObject[] rpmObjects = loadFromXpath("/WorksheetFinancial[Id='"
                + myObject.getID() + "']/baselineTimephased", null);

        assertNotNull(rpmObjects);
        assertEquals(2, rpmObjects.length);
        assertTrue(rpmObjects[0] instanceof TimePhasedFinancial);
        assertTrue(rpmObjects[1] instanceof TimePhasedFinancial);

        rpmObjects = loadFromXpath(
                "/WorksheetFinancial[baselineTimephased[Id='"
                        + myObject.getParent().getID() + "']]", null);

        assertNotNull(rpmObjects);
        assertTrue(2 <= rpmObjects.length);
        assertTrue(rpmObjects[0] instanceof WorksheetFinancial);
        assertTrue(rpmObjects[1] instanceof WorksheetFinancial);

        // TODO SLegault: should match by id.
        // objectCompare(myObject, rpmObjects[1], null);

        rpmObjects = loadFromXpath(
                "/WorksheetFinancial[ChargeCode[Id='EXPENSE_HOURS']]", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects.length > 0);
        assertTrue(rpmObjects[0] instanceof WorksheetFinancial);

        rpmObjects = loadFromXpath("/AggregateNode[Id='"
                + myObject.getParent().getID() + "']/worksheetFinancials", null);

        assertNotNull(rpmObjects);
        assertTrue(1 <= rpmObjects.length);
        assertTrue(rpmObjects[0] instanceof WorksheetFinancial);
        
        // TODO SLegault: should match by id.
        // objectCompare(myObject, rpmObjects[rpmObjects.length - 1], null);
    }

    protected RPMObject createObject() throws Exception
    {
        return FinancialFixture.createWorksheetFixture(
                (FinancialScope) mainScope, this);
    }

    protected RPMObjectScope createScope() throws Exception
    {
        return FinancialFixture.createAllScope();
    }

    /**
     * @param actual
     * @throws AssertionFailedError
     * @throws Exception
     */
    protected void objectCompare(RPMObject expected, RPMObject actual,
            RPMObjectScope scope) throws AssertionFailedError, Exception
    {
        FinancialFixture.compare((AbstractFinancial) expected,
                (AbstractFinancial) actual, (FinancialScope) scope);
    }

    protected void internalLoadRPMObjectFromXPathAndCompare(
            RPMObject localObject, String xPath, RPMObjectScope scope,
            String idOfObjectToFind, int queryNumber)
    {
        doObjectComparison(localObject, loadRPMObjectFromXPath(localObject
                .getClass(), xPath, scope, idOfObjectToFind, queryNumber));
    }
    
    protected ArrayList generateLocalSkipList()
    {
        ArrayList l = new ArrayList();

        l.add("getRemaining");
        l.add("getEAC");
        l.add("getEACVariance");
        l.add("getETC");
        l.add("getActual");
        l.add("getBaselineVariance");
        l.add("getEstimate");
        l.add("getProbabilityAmount");
        l.add("getRevRecType");
        l.add("getTimePhased");        
        
        l.trimToSize();
        return l;
    }
}
