package com.ibm.rpm.wbs.managers;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import com.ibm.rpm.applicationadministration.containers.AssetFinancialCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.Currency;
import com.ibm.rpm.applicationadministration.containers.DatafieldState;
import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.applicationadministration.containers.State;
import com.ibm.rpm.applicationadministration.managers.AttributeAssignmentManager;
import com.ibm.rpm.applicationadministration.managers.AttributeManager;
import com.ibm.rpm.applicationadministration.managers.DatafieldManager;
import com.ibm.rpm.applicationadministration.managers.RPMCalendarManager;
import com.ibm.rpm.applicationadministration.managers.RtfAssignmentManager;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.applicationadministration.util.AttributeUtil;
import com.ibm.rpm.asset.managers.AssetCrossChargeManager;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentManager;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentUtil;
import com.ibm.rpm.document.containers.Document;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.financial.checkpoints.FinancialCheckpoint;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.managers.FinancialManager;
import com.ibm.rpm.financial.managers.ProjectReconciliationStatusManager;
import com.ibm.rpm.financial.managers.TimeCodeAssignmentManager;
import com.ibm.rpm.financial.managers.TimecodeManager;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.financial.util.FinancialBusinessLogicUtil;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.ContainerMap;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.LoadParams;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.ContainerToCheckpointMap;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.ScheduleDateUtil;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.SqlUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scopemanagement.util.ScopeManagementUtil;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.managers.PublishedScorecardManager;
import com.ibm.rpm.scorecard.managers.ScorecardManager;
import com.ibm.rpm.timesheet.containers.DefaultStep;
import com.ibm.rpm.timesheet.managers.DefaultStepManager;
import com.ibm.rpm.timesheet.managers.StepManager;
import com.ibm.rpm.timesheet.managers.SummaryTimesheetManager;
import com.ibm.rpm.timesheet.managers.TimesheetApprovalStatusManager;
import com.ibm.rpm.timesheet.scope.DefaultStepScope;
import com.ibm.rpm.wbs.checkpoints.GenericProjectCheckpoint;
import com.ibm.rpm.wbs.checkpoints.WbsMethodCheckpoint;
import com.ibm.rpm.wbs.checkpoints.WorkElementCheckpoint;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.EarnedValues;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ProjectOrganizationalAssignment;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.ElementDependencyScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ConstraintType;
import com.ibm.rpm.wbs.types.ProjectPublishedType;
import com.ibm.rpm.wbs.types.TaskType;
import com.ibm.rpm.workflow.containers.AvailableWorkflowProcess;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowRoleMapping;
import com.ibm.rpm.workflow.managers.AvailableWorkflowProcessManager;
import com.ibm.rpm.workflow.managers.RunningWorkflowProcessManager;
import com.ibm.rpm.workflow.managers.WorkflowManager;
import com.ibm.rpm.workflow.managers.WorkflowRoleMappingManager;
import com.ibm.rpm.workflow.scope.AvailableWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowRoleMappingScope;
import com.ibm.rpm.workflow.util.WorkflowUtil;
import com.ibm.rpm.xpathparser.XPathContainer;


/**
 * Manager for the TMT_WBS table and related stored procedures.
 *
 * <h3>WBS Elements Comming From Scope Management</h3>
 * <p>
 * Note that before conversion, Scope elements behave like summary tasks in WBS.
 * After conversion, they behave like tasks.
 * <p>
 * <table border=1 cellspacing=0>
 * <tr>
 * <th>Scope Element</th>
 * <th>TMT_CRI.TYPE_ID</th>
 * <th>TMT_WBS.TYPE_ID</th>
 * <th>Task Element</th>
 * <th>TMT_CRI.TYPE_ID</th>
 * <th>TMT_WBS.TYPE_ID</th>
 * </tr>
 * <tr>
 * <td>Requirement</td>
 * <td>213</td>
 * <td>225</td>
 * <td>Requirement Task</td>
 * <td>235</td>
 * <td>241</td>
 * </tr>
 * <tr>
 * <td>Change Request</td>
 * <td>4</td>
 * <td>36</td>
 * <td>Change Request Task</td>
 * <td>65</td>
 * <td>236</td>
 * </tr>
 * <tr>
 * <td>Service Request</td>
 * <td>227</td>
 * <td>231</td>
 * <td>Service Request Task</td>
 * <td>234</td>
 * <td>240</td>
 * </tr>
 * <tr>
 * <td>Action</td>
 * <td>257</td>
 * <td>259</td>
 * <td>Action Task</td>
 * <td>258</td>
 * <td>260</td>
 * </tr>
 * <tr>
 * <td>Defect</td>
 * <td>226</td>
 * <td>230</td>
 * <td>Defect Task</td>
 * <td>233</td>
 * <td>239</td>
 * </tr>
 * <tr>
 * <td>Issue</td>
 * <td>6</td>
 * <td>39</td>
 * <td>Issue Task</td>
 * <td>67</td>
 * <td>238</td>
 * </tr>
 * <tr>
 * <td>Risk</td>
 * <td>5</td>
 * <td>37</td>
 * <td>Risk Task</td>
 * <td>66</td>
 * <td>237</td>
 * </tr>
 */
public class WbsManager extends AbstractRPMObjectManager
{
    //private static Log logger = LogFactory.getLog(WbsManager.class);

    private static final String DEFAULT_ASSET_CODE = "6501";

    private static final String DEFAULT_CALENDAR_ID = "DEFAULT_CALENDAR";

    // =============
    // CONTAINER MAP
    // =============

    private static ContainerMap containerMap = new ContainerMap(
        WbsManager.class);
    static
    {
        containerMap.add(WorkElement.class);
        containerMap.add(Deliverable.class);
        containerMap.add(SummaryTask.class);
        containerMap.add(WorkProduct.class);
        containerMap.add(AggregateNode.class);
        containerMap.add(GenericProject.class);
        containerMap.add(Proposal.class);
        containerMap.add(Project.class);
        containerMap.add(WorkOrganization.class);
        containerMap.add(SimpleNode.class);
        containerMap.add(Task.class);
        containerMap.add(Milestone.class);
        

        // Extra type ids mapped to SummaryTask and Task.
        containerMap.addExtraTypeIds(SummaryTask.class, new int[]
        {
                36, 37, 39, 225, 230, 231, 259
        });
        containerMap.addExtraTypeIds(Task.class, new int[]
        {
                236, 237, 238, 239, 240, 241, 260
        });
    }
    
    public ContainerMap getContainerMap()
    {
        return containerMap;
    }

    // =========
    // FIELD MAP
    // =========

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    /**
     * Column RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANK = 1;

    public static final int TYPE_RANK = Types.VARCHAR;

    public static final String NAME_RANK = "TMT_WBS.RANK";

    public static final String PROPERTY_RANK = "WBSSORTINGRANK";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_RANK, NAME_RANK);
    }

    /**
     * Column WBS_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_WBS_CODE = 2;

    public static final int TYPE_WBS_CODE = Types.VARCHAR;

    public static final String NAME_WBS_CODE = "TMT_WBS.WBS_CODE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WBS_CODE = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_WBS_CODE,NAME_WBS_CODE);}

    /**
     * Column PROJECT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PROJECT_ID = 3;

    public static final int TYPE_PROJECT_ID = Types.CHAR;

    public static final String NAME_PROJECT_ID = "TMT_WBS.PROJECT_ID";

    public static final String PROPERTY_PROJECT_ID = "PROJECTID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROJECT_ID, NAME_PROJECT_ID);
    }

    /**
     * Column PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_PARENT_ID = 4;

    public static final int TYPE_PARENT_ID = Types.CHAR;

    public static final String NAME_PARENT_ID = "TMT_WBS.PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PARENT_ID,NAME_PARENT_ID);}

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 5;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "TMT_WBS.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column TYPE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TYPE_ID = 6;

    public static final int TYPE_TYPE_ID = Types.INTEGER;

    public static final String NAME_TYPE_ID = "TMT_WBS.TYPE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TYPE_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_TYPE_ID,NAME_TYPE_ID);}

    /**
     * Column LEVEL_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_ID = 7;

    public static final int TYPE_LEVEL_ID = Types.INTEGER;

    public static final String NAME_LEVEL_ID = "TMT_WBS.LEVEL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_LEVEL_ID,NAME_LEVEL_ID);}

    /**
     * Column MAX_RANK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MAX_RANK = 8;

    public static final int TYPE_MAX_RANK = Types.INTEGER;

    public static final String NAME_MAX_RANK = "TMT_WBS.MAX_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAX_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_MAX_RANK,NAME_MAX_RANK);}

    /**
     * Column CHILD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CHILD_COUNT = 9;

    public static final int TYPE_CHILD_COUNT = Types.INTEGER;

    public static final String NAME_CHILD_COUNT = "TMT_WBS.CHILD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHILD_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CHILD_COUNT,NAME_CHILD_COUNT);}

    /**
     * Column DELETED_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DELETED_COUNT = 10;

    public static final int TYPE_DELETED_COUNT = Types.INTEGER;

    public static final String NAME_DELETED_COUNT = "TMT_WBS.DELETED_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DELETED_COUNT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_DELETED_COUNT,NAME_DELETED_COUNT);}

    /**
     * Column ELEMENT_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ELEMENT_NAME = 11;

    public static final int TYPE_ELEMENT_NAME = Types.VARCHAR;

    public static final String NAME_ELEMENT_NAME = "TMT_WBS.ELEMENT_NAME";

    public static final String PROPERTY_ELEMENT_NAME = "NAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_NAME, NAME_ELEMENT_NAME);
    }

    /**
     * Column REF_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_REF_CODE = 12;

    public static final int TYPE_REF_CODE = Types.VARCHAR;

    public static final String NAME_REF_CODE = "TMT_WBS.REF_CODE";

    public static final String PROPERTY_REF_CODE = "REFERENCENUMBER";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REF_CODE, NAME_REF_CODE);
    }

    /**
     * Column PRIORITY of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_PRIORITY = 13;

    public static final int TYPE_PRIORITY = Types.SMALLINT;

    public static final String NAME_PRIORITY = "TMT_WBS.PRIORITY";

    public static final String PROPERTY_PRIORITY = "PRIORITY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PRIORITY, NAME_PRIORITY);
    }

    /**
     * Column PUBLISHED of type Types.CHAR mapped to String.
     */
    public static final int ID_PUBLISHED = 14;

    public static final int TYPE_PUBLISHED = Types.CHAR;

    public static final String NAME_PUBLISHED = "TMT_WBS.PUBLISHED";

    public static final String PROPERTY_PUBLISHED = "PUBLISHED";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PUBLISHED, NAME_PUBLISHED);
    }

    /**
     * Column PUBLISHED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_PUBLISHED_BY = 15;

    public static final int TYPE_PUBLISHED_BY = Types.CHAR;

    public static final String NAME_PUBLISHED_BY = "TMT_WBS.PUBLISHED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PUBLISHED_BY = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_PUBLISHED_BY,NAME_PUBLISHED_BY);}

    /**
     * Column T_PARENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_T_PARENT_ID = 16;

    public static final int TYPE_T_PARENT_ID = Types.CHAR;

    public static final String NAME_T_PARENT_ID = "TMT_WBS.T_PARENT_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_T_PARENT_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_T_PARENT_ID,NAME_T_PARENT_ID);}

    /**
     * Column T_RANK of type Types.VARCHAR mapped to String.
     */
    public static final int ID_T_RANK = 17;

    public static final int TYPE_T_RANK = Types.VARCHAR;

    public static final String NAME_T_RANK = "TMT_WBS.T_RANK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_T_RANK = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_T_RANK,NAME_T_RANK);}

    /**
     * Column CONSTRAINT_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONSTRAINT_TYPE = 18;

    public static final int TYPE_CONSTRAINT_TYPE = Types.INTEGER;

    public static final String NAME_CONSTRAINT_TYPE = "TMT_WBS.CONSTRAINT_TYPE";

    public static final String PROPERTY_CONSTRAINT_TYPE = "CONSTRAINTTYPE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONSTRAINT_TYPE, SqlUtil.castAsChar(
            NAME_CONSTRAINT_TYPE, 1));
    }

    /**
     * Column CONSTRAINT_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CONSTRAINT_DT = 19;

    public static final int TYPE_CONSTRAINT_DT = Types.TIMESTAMP;

    // This custom DB function returns the date portion of a timestamp field
    // It was written for use in both DB2 and Oracle
    public static final String NAME_CONSTRAINT_DT = "TMT_WBS.CONSTRAINT_DT";

    public static final String PROPERTY_CONSTRAINT_DT = "CONSTRAINTDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONSTRAINT_DT, SqlUtil.convertTimestampToDate(NAME_CONSTRAINT_DT));
    }

    /**
     * Column CURRENCY_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CURRENCY_ID = 20;

    public static final int TYPE_CURRENCY_ID = Types.INTEGER;

    public static final String NAME_CURRENCY_ID = "TMT_WBS.CURRENCY_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CURRENCY_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CURRENCY_ID,NAME_CURRENCY_ID);}

    /**
     * Column CALENDAR_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_CALENDAR_ID = 21;

    public static final int TYPE_CALENDAR_ID = Types.CHAR;

    public static final String NAME_CALENDAR_ID = "TMT_WBS.CALENDAR_ID";

    //public static final String PROPERTY_CALENDAR_ID = "CALENDAR";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_CALENDAR_ID,NAME_CALENDAR_ID);}

    /**
     * Column MINUTE_PER_DAY of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MINUTE_PER_DAY = 22;

    public static final int TYPE_MINUTE_PER_DAY = Types.SMALLINT;

    public static final String NAME_MINUTE_PER_DAY = "TMT_WBS.MINUTE_PER_DAY";

    public static final String PROPERTY_MINUTE_PER_DAY = "MINUTESPERDAY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MINUTE_PER_DAY, NAME_MINUTE_PER_DAY);
    }

    /**
     * Column MINUTE_PER_WEEK of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MINUTE_PER_WEEK = 23;

    public static final int TYPE_MINUTE_PER_WEEK = Types.SMALLINT;

    public static final String NAME_MINUTE_PER_WEEK = "TMT_WBS.MINUTE_PER_WEEK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MINUTE_PER_WEEK = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_MINUTE_PER_WEEK,NAME_MINUTE_PER_WEEK); }

    /**
     * Column METHOD of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_METHOD = 24;

    public static final int TYPE_METHOD = Types.INTEGER;

    public static final String NAME_METHOD = "TMT_WBS.METHOD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_METHOD = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_METHOD,NAME_METHOD);}

    /**
     * Column BUFFER_PERCENT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BUFFER_PERCENT = 25;

    public static final int TYPE_BUFFER_PERCENT = Types.SMALLINT;

    public static final String NAME_BUFFER_PERCENT = "TMT_WBS.BUFFER_PERCENT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BUFFER_PERCENT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BUFFER_PERCENT,NAME_BUFFER_PERCENT); }

    /**
     * Column START_FINISH_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_START_FINISH_FLAG = 26;

    public static final int TYPE_START_FINISH_FLAG = Types.CHAR;

    public static final String NAME_START_FINISH_FLAG = "TMT_WBS.START_FINISH_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_START_FINISH_FLAG = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_START_FINISH_FLAG,NAME_START_FINISH_FLAG);
    // }

    /**
     * Column START_FINISH_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_START_FINISH_DATE = 27;

    public static final int TYPE_START_FINISH_DATE = Types.DATE;

    public static final String NAME_START_FINISH_DATE = "TMT_WBS.START_FINISH_DATE";

    public static final String PROPERTY_START_FINISH_DATE = "ESTIMATEDSTARTDATE";
    static
    {
        FIELDPROPERTYMAP
            .put(PROPERTY_START_FINISH_DATE, NAME_START_FINISH_DATE);
    }

    /**
     * Column SD of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_SD = 28;

    public static final int TYPE_SD = Types.SMALLINT;

    public static final String NAME_SD = "TMT_WBS.SD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SD = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_SD,NAME_SD); }

    /**
     * Column FD of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_FD = 29;

    public static final int TYPE_FD = Types.SMALLINT;

    public static final String NAME_FD = "TMT_WBS.FD";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FD = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_FD,NAME_FD); }

    /**
     * Column START_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_START_DT = 30;

    public static final int TYPE_START_DT = Types.TIMESTAMP;

    public static final String NAME_START_DT = "TMT_WBS.START_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_START_DT = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_START_DT,NAME_START_DT); }

    /**
     * Column FINISH_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FINISH_DT = 31;

    public static final int TYPE_FINISH_DT = Types.TIMESTAMP;

    public static final String NAME_FINISH_DT = "TMT_WBS.FINISH_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FINISH_DT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_FINISH_DT,NAME_FINISH_DT); }

    /**
     * Column BASELINE1_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_START = 32;

    public static final int TYPE_BASELINE1_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_START = "TMT_WBS.BASELINE1_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_START = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_START,NAME_BASELINE1_START); }

    /**
     * Column BASELINE1_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_FINISH = 33;

    public static final int TYPE_BASELINE1_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_FINISH = "TMT_WBS.BASELINE1_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_FINISH = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_FINISH,NAME_BASELINE1_FINISH); }

    /**
     * Column BASELINE2_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_START = 34;

    public static final int TYPE_BASELINE2_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_START = "TMT_WBS.BASELINE2_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_START = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_START,NAME_BASELINE2_START); }

    /**
     * Column BASELINE2_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_FINISH = 35;

    public static final int TYPE_BASELINE2_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_FINISH = "TMT_WBS.BASELINE2_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_FINISH = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_FINISH,NAME_BASELINE2_FINISH); }

    /**
     * Column BASELINE3_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE3_START = 36;

    public static final int TYPE_BASELINE3_START = Types.TIMESTAMP;

    public static final String NAME_BASELINE3_START = "TMT_WBS.BASELINE3_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_START = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_START,NAME_BASELINE3_START); }

    /**
     * Column BASELINE3_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE3_FINISH = 37;

    public static final int TYPE_BASELINE3_FINISH = Types.TIMESTAMP;

    public static final String NAME_BASELINE3_FINISH = "TMT_WBS.BASELINE3_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_FINISH = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_FINISH,NAME_BASELINE3_FINISH); }

    /**
     * Column PLANNED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PLANNED_START = 38;

    public static final int TYPE_PLANNED_START = Types.TIMESTAMP;

    public static final String NAME_PLANNED_START = "TMT_WBS.PLANNED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_START = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_PLANNED_START,NAME_PLANNED_START);
    // }

    /**
     * Column PLANNED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PLANNED_FINISH = 39;

    public static final int TYPE_PLANNED_FINISH = Types.TIMESTAMP;

    public static final String NAME_PLANNED_FINISH = "TMT_WBS.PLANNED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_FINISH = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_FINISH,NAME_PLANNED_FINISH); }

    /**
     * Column PROPOSED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PROPOSED_START = 40;

    public static final int TYPE_PROPOSED_START = Types.TIMESTAMP;

    public static final String NAME_PROPOSED_START = "TMT_WBS.PROPOSED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_START = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_START,NAME_PROPOSED_START); }

    /**
     * Column PROPOSED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_PROPOSED_FINISH = 41;

    public static final int TYPE_PROPOSED_FINISH = Types.TIMESTAMP;

    public static final String NAME_PROPOSED_FINISH = "TMT_WBS.PROPOSED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_FINISH = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_FINISH,NAME_PROPOSED_FINISH); }

    /**
     * Column SCHED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SCHED_START = 42;

    public static final int TYPE_SCHED_START = Types.TIMESTAMP;

    public static final String NAME_SCHED_START = "TMT_WBS.SCHED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_START = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_SCHED_START,NAME_SCHED_START); }

    /**
     * Column SCHED_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_SCHED_FINISH = 43;

    public static final int TYPE_SCHED_FINISH = Types.TIMESTAMP;

    public static final String NAME_SCHED_FINISH = "TMT_WBS.SCHED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_FINISH = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_SCHED_FINISH,NAME_SCHED_FINISH); }

    /**
     * Column FORECAST_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FORECAST_START = 44;

    public static final int TYPE_FORECAST_START = Types.TIMESTAMP;

    public static final String NAME_FORECAST_START = "TMT_WBS.FORECAST_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_START = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_START,NAME_FORECAST_START); }

    /**
     * Column FORECAST_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_FORECAST_FINISH = 45;

    public static final int TYPE_FORECAST_FINISH = Types.TIMESTAMP;

    public static final String NAME_FORECAST_FINISH = "TMT_WBS.FORECAST_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_FINISH = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_FINISH,NAME_FORECAST_FINISH); }

    /**
     * Column ACTUAL_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ACTUAL_START = 46;

    public static final int TYPE_ACTUAL_START = Types.TIMESTAMP;

    public static final String NAME_ACTUAL_START = "TMT_WBS.ACTUAL_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_START = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_START,NAME_ACTUAL_START); }

    /**
     * Column ACTUAL_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_ACTUAL_FINISH = 47;

    public static final int TYPE_ACTUAL_FINISH = Types.TIMESTAMP;

    public static final String NAME_ACTUAL_FINISH = "TMT_WBS.ACTUAL_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_FINISH = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_FINISH,NAME_ACTUAL_FINISH);
    // }

    /**
     * Column UNRESOLVED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_UNRESOLVED_DURATION = 48;

    public static final int TYPE_UNRESOLVED_DURATION = Types.INTEGER;

    public static final String NAME_UNRESOLVED_DURATION = "TMT_WBS.UNRESOLVED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_UNRESOLVED_DURATION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_UNRESOLVED_DURATION,NAME_UNRESOLVED_DURATION);
    // }

    /**
     * Column DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DURATION = 49;

    public static final int TYPE_DURATION = Types.INTEGER;

    public static final String NAME_DURATION = "TMT_WBS.DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DURATION = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_DURATION,NAME_DURATION); }

    /**
     * Column PLANNED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PLANNED_DURATION = 50;

    public static final int TYPE_PLANNED_DURATION = Types.INTEGER;

    public static final String NAME_PLANNED_DURATION = "TMT_WBS.PLANNED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_DURATION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_DURATION,NAME_PLANNED_DURATION); }

    /**
     * Column PROPOSED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROPOSED_DURATION = 51;

    public static final int TYPE_PROPOSED_DURATION = Types.INTEGER;

    public static final String NAME_PROPOSED_DURATION = "TMT_WBS.PROPOSED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_DURATION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_DURATION,NAME_PROPOSED_DURATION);
    // }

    /**
     * Column SCHED_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCHED_DURATION = 52;

    public static final int TYPE_SCHED_DURATION = Types.INTEGER;

    public static final String NAME_SCHED_DURATION = "TMT_WBS.SCHED_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_DURATION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_SCHED_DURATION,NAME_SCHED_DURATION); }

    /**
     * Column FORECAST_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FORECAST_DURATION = 53;

    public static final int TYPE_FORECAST_DURATION = Types.INTEGER;

    public static final String NAME_FORECAST_DURATION = "TMT_WBS.FORECAST_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_DURATION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_DURATION,NAME_FORECAST_DURATION);
    // }

    /**
     * Column ACTUAL_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ACTUAL_DURATION = 54;

    public static final int TYPE_ACTUAL_DURATION = Types.INTEGER;

    public static final String NAME_ACTUAL_DURATION = "TMT_WBS.ACTUAL_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_DURATION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_DURATION,NAME_ACTUAL_DURATION); }

    /**
     * Column BASELINE1_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE1_DURATION = 55;

    public static final int TYPE_BASELINE1_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE1_DURATION = "TMT_WBS.BASELINE1_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_DURATION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_DURATION,NAME_BASELINE1_DURATION);
    // }

    /**
     * Column BASELINE2_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE2_DURATION = 56;

    public static final int TYPE_BASELINE2_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE2_DURATION = "TMT_WBS.BASELINE2_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_DURATION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_DURATION,NAME_BASELINE2_DURATION);
    // }

    /**
     * Column BASELINE3_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE3_DURATION = 57;

    public static final int TYPE_BASELINE3_DURATION = Types.INTEGER;

    public static final String NAME_BASELINE3_DURATION = "TMT_WBS.BASELINE3_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_DURATION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_DURATION,NAME_BASELINE3_DURATION);
    // }

    /**
     * Column VARIANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_VARIANCE = 58;

    public static final int TYPE_VARIANCE = Types.INTEGER;

    public static final String NAME_VARIANCE = "TMT_WBS.VARIANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_VARIANCE = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_VARIANCE,NAME_VARIANCE); }

    /**
     * Column FORECAST_VARIANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FORECAST_VARIANCE = 59;

    public static final int TYPE_FORECAST_VARIANCE = Types.INTEGER;

    public static final String NAME_FORECAST_VARIANCE = "TMT_WBS.FORECAST_VARIANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_VARIANCE = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_VARIANCE,NAME_FORECAST_VARIANCE);
    // }

    /**
     * Column ACTUAL_VARIANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ACTUAL_VARIANCE = 60;

    public static final int TYPE_ACTUAL_VARIANCE = Types.INTEGER;

    public static final String NAME_ACTUAL_VARIANCE = "TMT_WBS.ACTUAL_VARIANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_VARIANCE = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_VARIANCE,NAME_ACTUAL_VARIANCE); }

    /**
     * Column BASELINE1_VARIANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE1_VARIANCE = 61;

    public static final int TYPE_BASELINE1_VARIANCE = Types.INTEGER;

    public static final String NAME_BASELINE1_VARIANCE = "TMT_WBS.BASELINE1_VARIANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_VARIANCE = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_VARIANCE,NAME_BASELINE1_VARIANCE);
    // }

    /**
     * Column PLANNED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_PLANNED_EFFORT = 62;

    public static final int TYPE_PLANNED_EFFORT = Types.BIGINT;

    public static final String NAME_PLANNED_EFFORT = "TMT_WBS.PLANNED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_EFFORT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_EFFORT,NAME_PLANNED_EFFORT); }

    /**
     * Column PROPOSED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_PROPOSED_EFFORT = 63;

    public static final int TYPE_PROPOSED_EFFORT = Types.BIGINT;

    public static final String NAME_PROPOSED_EFFORT = "TMT_WBS.PROPOSED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_EFFORT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_EFFORT,NAME_PROPOSED_EFFORT); }

    /**
     * Column SCHED_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_SCHED_EFFORT = 64;

    public static final int TYPE_SCHED_EFFORT = Types.BIGINT;

    public static final String NAME_SCHED_EFFORT = "TMT_WBS.SCHED_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCHED_EFFORT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_SCHED_EFFORT,NAME_SCHED_EFFORT); }

    /**
     * Column FORECAST_EFFORT of type Types.BIGINT mapped to Long
     */
    public static final int ID_FORECAST_EFFORT = 65;

    public static final int TYPE_FORECAST_EFFORT = Types.BIGINT;

    public static final String NAME_FORECAST_EFFORT = "TMT_WBS.FORECAST_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_EFFORT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_EFFORT,NAME_FORECAST_EFFORT); }

    /**
     * Column ACTUAL_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_ACTUAL_EFFORT = 66;

    public static final int TYPE_ACTUAL_EFFORT = Types.BIGINT;

    public static final String NAME_ACTUAL_EFFORT = "TMT_WBS.ACTUAL_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_EFFORT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_EFFORT,NAME_ACTUAL_EFFORT);
    // }

    /**
     * Column BASELINE1_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BASELINE1_EFFORT = 67;

    public static final int TYPE_BASELINE1_EFFORT = Types.BIGINT;

    public static final String NAME_BASELINE1_EFFORT = "TMT_WBS.BASELINE1_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_EFFORT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_EFFORT,NAME_BASELINE1_EFFORT); }

    /**
     * Column BASELINE2_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BASELINE2_EFFORT = 68;

    public static final int TYPE_BASELINE2_EFFORT = Types.BIGINT;

    public static final String NAME_BASELINE2_EFFORT = "TMT_WBS.BASELINE2_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_EFFORT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_EFFORT,NAME_BASELINE2_EFFORT); }

    /**
     * Column BASELINE3_EFFORT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BASELINE3_EFFORT = 69;

    public static final int TYPE_BASELINE3_EFFORT = Types.BIGINT;

    public static final String NAME_BASELINE3_EFFORT = "TMT_WBS.BASELINE3_EFFORT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_EFFORT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_EFFORT,NAME_BASELINE3_EFFORT); }

    /**
     * Column WORK_TODATE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_WORK_TODATE = 70;

    public static final int TYPE_WORK_TODATE = Types.BIGINT;

    public static final String NAME_WORK_TODATE = "TMT_WBS.WORK_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WORK_TODATE = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_WORK_TODATE,NAME_WORK_TODATE); }

    /**
     * Column ETC of type Types.BIGINT mapped to Long.
     */
    public static final int ID_ETC = 71;

    public static final int TYPE_ETC = Types.BIGINT;

    public static final String NAME_ETC = "TMT_WBS.ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ETC = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ETC,NAME_ETC); }

    /**
     * Column CREATED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_CREATED_BY = 72;

    public static final int TYPE_CREATED_BY = Types.CHAR;

    public static final String NAME_CREATED_BY = "TMT_WBS.CREATED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREATED_BY = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_CREATED_BY,NAME_CREATED_BY); }

    /**
     * Column CREATED_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CREATED_DT = 73;

    public static final int TYPE_CREATED_DT = Types.TIMESTAMP;

    public static final String NAME_CREATED_DT = "TMT_WBS.CREATED_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CREATED_DT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_CREATED_DT,NAME_CREATED_DT); }

    /**
     * Column PROBABILITY of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_PROBABILITY = 74;

    public static final int TYPE_PROBABILITY = Types.INTEGER;

    public static final String NAME_PROBABILITY = "TMT_WBS.PROBABILITY";

    public static final String PROPERTY_PROBABILITY = "PROBABILITYPERCENT";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PROBABILITY, NAME_PROBABILITY);
    }

    /**
     * Column STAGE_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_STAGE_ID = 75;

    public static final int TYPE_STAGE_ID = Types.INTEGER;

    public static final String NAME_STAGE_ID = "TMT_WBS.STAGE_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_STAGE_ID = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_STAGE_ID,NAME_STAGE_ID); }

    /**
     * Column MILESTONE_PERCENT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MILESTONE_PERCENT = 76;

    public static final int TYPE_MILESTONE_PERCENT = Types.SMALLINT;

    public static final String NAME_MILESTONE_PERCENT = "TMT_WBS.MILESTONE_PERCENT";

    public static final String PROPERTY_MILESTONE_PERCENT = "PERCENTAGEDURATIONCOMPLETED";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MILESTONE_PERCENT, NAME_MILESTONE_PERCENT
                + " / 100");
    }

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 77;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "TMT_WBS.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER); }

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 78;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "TMT_WBS.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS); }

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 79;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    // We need to be able to query the timestamp portion of this field
    // so we can't use function timestamp_to_date - the timestamp must be
    // included in all xpath queries for this field
    public static final String NAME_REC_DATETIME = "TMT_WBS.REC_DATETIME";

    public static final String PROPERTY_REC_DATETIME = "LASTMODIFIEDDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME, NAME_REC_DATETIME);
    }

    /**
     * Column LAST_WF_TYPE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LAST_WF_TYPE = 80;

    public static final int TYPE_LAST_WF_TYPE = Types.VARCHAR;

    public static final String NAME_LAST_WF_TYPE = "TMT_WBS.LAST_WF_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_WF_TYPE = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_LAST_WF_TYPE,NAME_LAST_WF_TYPE); }

    /**
     * Column LAST_WF_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_LAST_WF_BY = 81;

    public static final int TYPE_LAST_WF_BY = Types.CHAR;

    public static final String NAME_LAST_WF_BY = "TMT_WBS.LAST_WF_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_WF_BY = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_LAST_WF_BY,NAME_LAST_WF_BY); }

    /**
     * Column LAST_WF_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LAST_WF_DT = 82;

    public static final int TYPE_LAST_WF_DT = Types.TIMESTAMP;

    public static final String NAME_LAST_WF_DT = "TMT_WBS.LAST_WF_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_WF_DT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_LAST_WF_DT,NAME_LAST_WF_DT); }

    /**
     * Column MAJOR_REVISION of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MAJOR_REVISION = 83;

    public static final int TYPE_MAJOR_REVISION = Types.SMALLINT;

    public static final String NAME_MAJOR_REVISION = "TMT_WBS.MAJOR_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MAJOR_REVISION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_MAJOR_REVISION,NAME_MAJOR_REVISION); }

    /**
     * Column MINOR_REVISION of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_MINOR_REVISION = 84;

    public static final int TYPE_MINOR_REVISION = Types.SMALLINT;

    public static final String NAME_MINOR_REVISION = "TMT_WBS.MINOR_REVISION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MINOR_REVISION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_MINOR_REVISION,NAME_MINOR_REVISION); }

    /**
     * Column REVISION_IMPACT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REVISION_IMPACT = 85;

    public static final int TYPE_REVISION_IMPACT = Types.INTEGER;

    public static final String NAME_REVISION_IMPACT = "TMT_WBS.REVISION_IMPACT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_IMPACT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_IMPACT,NAME_REVISION_IMPACT); }

    /**
     * Column REVISION_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REVISION_TYPE = 86;

    public static final int TYPE_REVISION_TYPE = Types.INTEGER;

    public static final String NAME_REVISION_TYPE = "TMT_WBS.REVISION_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_TYPE = "PROPERTYNAME"
    //static { FIELDPROPERTYMAP.put(PROPERTY_REVISION_TYPE,NAME_REVISION_TYPE);
    // }

    /**
     * Column REVISION_HISTORY of type Types.VARCHAR mapped to String.
     */
    public static final int ID_REVISION_HISTORY = 87;

    public static final int TYPE_REVISION_HISTORY = Types.VARCHAR;

    public static final String NAME_REVISION_HISTORY = "TMT_WBS.REVISION_HISTORY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REVISION_HISTORY = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_REVISION_HISTORY,NAME_REVISION_HISTORY); }

    /**
     * Column CLOSED_BY of type Types.CHAR mapped to String.
     */
    public static final int ID_CLOSED_BY = 88;

    public static final int TYPE_CLOSED_BY = Types.CHAR;

    public static final String NAME_CLOSED_BY = "TMT_WBS.CLOSED_BY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CLOSED_BY = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_CLOSED_BY,NAME_CLOSED_BY); }}

    /**
     * Column TOOLTIP_DESCR of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TOOLTIP_DESCR = 89;

    public static final int TYPE_TOOLTIP_DESCR = Types.VARCHAR;

    public static final String NAME_TOOLTIP_DESCR = "TMT_WBS.TOOLTIP_DESCR";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TOOLTIP_DESCR = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_TOOLTIP_DESCR,NAME_TOOLTIP_DESCR);
    // }

    /**
     * Column TOTAL_SLACK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TOTAL_SLACK = 90;

    public static final int TYPE_TOTAL_SLACK = Types.INTEGER;

    public static final String NAME_TOTAL_SLACK = "TMT_WBS.TOTAL_SLACK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TOTAL_SLACK = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_TOTAL_SLACK,NAME_TOTAL_SLACK); }

    /**
     * Column FREE_SLACK of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FREE_SLACK = 91;

    public static final int TYPE_FREE_SLACK = Types.INTEGER;

    public static final String NAME_FREE_SLACK = "TMT_WBS.FREE_SLACK";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FREE_SLACK = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_FREE_SLACK,NAME_FREE_SLACK); }

    /**
     * Column CRITICAL of type Types.CHAR mapped to String.
     */
    public static final int ID_CRITICAL = 92;

    public static final int TYPE_CRITICAL = Types.CHAR;

    public static final String NAME_CRITICAL = "TMT_WBS.CRITICAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CRITICAL = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_CRITICAL,NAME_CRITICAL); }

    /**
     * Column ESTIMATION_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATION_TOTAL = 93;

    public static final int TYPE_ESTIMATION_TOTAL = Types.DECIMAL;

    public static final String NAME_ESTIMATION_TOTAL = "TMT_WBS.ESTIMATION_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATION_TOTAL = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATION_TOTAL,NAME_ESTIMATION_TOTAL); }

    /**
     * Column ESTIMATION_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATION_BUDGET = 94;

    public static final int TYPE_ESTIMATION_BUDGET = Types.DECIMAL;

    public static final String NAME_ESTIMATION_BUDGET = "TMT_WBS.ESTIMATION_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATION_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATION_BUDGET,NAME_ESTIMATION_BUDGET);
    // }

    /**
     * Column CURRENT_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_CURRENT_BUDGET = 95;

    public static final int TYPE_CURRENT_BUDGET = Types.DECIMAL;

    public static final String NAME_CURRENT_BUDGET = "TMT_WBS.CURRENT_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CURRENT_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_CURRENT_BUDGET,NAME_CURRENT_BUDGET); }

    /**
     * Column FORECAST_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_FORECAST_BUDGET = 96;

    public static final int TYPE_FORECAST_BUDGET = Types.DECIMAL;

    public static final String NAME_FORECAST_BUDGET = "TMT_WBS.FORECAST_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FORECAST_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_FORECAST_BUDGET,NAME_FORECAST_BUDGET); }

    /**
     * Column ACTUAL_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ACTUAL_BUDGET = 97;

    public static final int TYPE_ACTUAL_BUDGET = Types.DECIMAL;

    public static final String NAME_ACTUAL_BUDGET = "TMT_WBS.ACTUAL_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACTUAL_BUDGET = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ACTUAL_BUDGET,NAME_ACTUAL_BUDGET);
    // }

    /**
     * Column BASELINE1_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE1_BUDGET = 98;

    public static final int TYPE_BASELINE1_BUDGET = Types.DECIMAL;

    public static final String NAME_BASELINE1_BUDGET = "TMT_WBS.BASELINE1_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_BUDGET,NAME_BASELINE1_BUDGET); }

    /**
     * Column BASELINE2_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE2_BUDGET = 99;

    public static final int TYPE_BASELINE2_BUDGET = Types.DECIMAL;

    public static final String NAME_BASELINE2_BUDGET = "TMT_WBS.BASELINE2_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_BUDGET,NAME_BASELINE2_BUDGET); }

    /**
     * Column BASELINE3_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BASELINE3_BUDGET = 100;

    public static final int TYPE_BASELINE3_BUDGET = Types.DECIMAL;

    public static final String NAME_BASELINE3_BUDGET = "TMT_WBS.BASELINE3_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_BUDGET,NAME_BASELINE3_BUDGET); }

    /**
     * Column R_ESTIMATION_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATION_TOTAL = 101;

    public static final int TYPE_R_ESTIMATION_TOTAL = Types.DECIMAL;

    public static final String NAME_R_ESTIMATION_TOTAL = "TMT_WBS.R_ESTIMATION_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATION_TOTAL = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATION_TOTAL,NAME_R_ESTIMATION_TOTAL);
    // }

    /**
     * Column R_ESTIMATION_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATION_BUDGET = 102;

    public static final int TYPE_R_ESTIMATION_BUDGET = Types.DECIMAL;

    public static final String NAME_R_ESTIMATION_BUDGET = "TMT_WBS.R_ESTIMATION_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATION_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATION_BUDGET,NAME_R_ESTIMATION_BUDGET);
    // }

    /**
     * Column R_CURRENT_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_CURRENT_BUDGET = 103;

    public static final int TYPE_R_CURRENT_BUDGET = Types.DECIMAL;

    public static final String NAME_R_CURRENT_BUDGET = "TMT_WBS.R_CURRENT_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_CURRENT_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_CURRENT_BUDGET,NAME_R_CURRENT_BUDGET); }

    /**
     * Column R_FORECAST_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_FORECAST_BUDGET = 104;

    public static final int TYPE_R_FORECAST_BUDGET = Types.DECIMAL;

    public static final String NAME_R_FORECAST_BUDGET = "TMT_WBS.R_FORECAST_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_FORECAST_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_FORECAST_BUDGET,NAME_R_FORECAST_BUDGET);
    // }

    /**
     * Column R_ACTUAL_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ACTUAL_BUDGET = 105;

    public static final int TYPE_R_ACTUAL_BUDGET = Types.DECIMAL;

    public static final String NAME_R_ACTUAL_BUDGET = "TMT_WBS.R_ACTUAL_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ACTUAL_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ACTUAL_BUDGET,NAME_R_ACTUAL_BUDGET); }

    /**
     * Column R_BASELINE1_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_BASELINE1_BUDGET = 106;

    public static final int TYPE_R_BASELINE1_BUDGET = Types.DECIMAL;

    public static final String NAME_R_BASELINE1_BUDGET = "TMT_WBS.R_BASELINE1_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_BASELINE1_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_BASELINE1_BUDGET,NAME_R_BASELINE1_BUDGET);
    // }

    /**
     * Column R_BASELINE2_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_BASELINE2_BUDGET = 107;

    public static final int TYPE_R_BASELINE2_BUDGET = Types.DECIMAL;

    public static final String NAME_R_BASELINE2_BUDGET = "TMT_WBS.R_BASELINE2_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_BASELINE2_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_BASELINE2_BUDGET,NAME_R_BASELINE2_BUDGET);
    // }

    /**
     * Column R_BASELINE3_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_BASELINE3_BUDGET = 108;

    public static final int TYPE_R_BASELINE3_BUDGET = Types.DECIMAL;

    public static final String NAME_R_BASELINE3_BUDGET = "TMT_WBS.R_BASELINE3_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_BASELINE3_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_BASELINE3_BUDGET,NAME_R_BASELINE3_BUDGET);
    // }

    /**
     * Column S_ESTIMATION_TOTAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATION_TOTAL = 109;

    public static final int TYPE_S_ESTIMATION_TOTAL = Types.DECIMAL;

    public static final String NAME_S_ESTIMATION_TOTAL = "TMT_WBS.S_ESTIMATION_TOTAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATION_TOTAL = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATION_TOTAL,NAME_S_ESTIMATION_TOTAL);
    // }

    /**
     * Column S_ESTIMATION_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATION_BUDGET = 110;

    public static final int TYPE_S_ESTIMATION_BUDGET = Types.DECIMAL;

    public static final String NAME_S_ESTIMATION_BUDGET = "TMT_WBS.S_ESTIMATION_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATION_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATION_BUDGET,NAME_S_ESTIMATION_BUDGET);
    // }

    /**
     * Column S_CURRENT_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_CURRENT_BUDGET = 111;

    public static final int TYPE_S_CURRENT_BUDGET = Types.DECIMAL;

    public static final String NAME_S_CURRENT_BUDGET = "TMT_WBS.S_CURRENT_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_CURRENT_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_CURRENT_BUDGET,NAME_S_CURRENT_BUDGET); }

    /**
     * Column S_FORECAST_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_FORECAST_BUDGET = 112;

    public static final int TYPE_S_FORECAST_BUDGET = Types.DECIMAL;

    public static final String NAME_S_FORECAST_BUDGET = "TMT_WBS.S_FORECAST_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_FORECAST_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_FORECAST_BUDGET,NAME_S_FORECAST_BUDGET);
    // }

    /**
     * Column S_ACTUAL_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ACTUAL_BUDGET = 113;

    public static final int TYPE_S_ACTUAL_BUDGET = Types.DECIMAL;

    public static final String NAME_S_ACTUAL_BUDGET = "TMT_WBS.S_ACTUAL_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ACTUAL_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_ACTUAL_BUDGET,NAME_S_ACTUAL_BUDGET); }

    /**
     * Column S_BASELINE1_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_BASELINE1_BUDGET = 114;

    public static final int TYPE_S_BASELINE1_BUDGET = Types.DECIMAL;

    public static final String NAME_S_BASELINE1_BUDGET = "TMT_WBS.S_BASELINE1_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_BASELINE1_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_BASELINE1_BUDGET,NAME_S_BASELINE1_BUDGET);
    // }

    /**
     * Column S_BASELINE2_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_BASELINE2_BUDGET = 115;

    public static final int TYPE_S_BASELINE2_BUDGET = Types.DECIMAL;

    public static final String NAME_S_BASELINE2_BUDGET = "TMT_WBS.S_BASELINE2_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_BASELINE2_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_BASELINE2_BUDGET,NAME_S_BASELINE2_BUDGET);
    // }

    /**
     * Column S_BASELINE3_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_BASELINE3_BUDGET = 116;

    public static final int TYPE_S_BASELINE3_BUDGET = Types.DECIMAL;

    public static final String NAME_S_BASELINE3_BUDGET = "TMT_WBS.S_BASELINE3_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_BASELINE3_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_BASELINE3_BUDGET,NAME_S_BASELINE3_BUDGET);
    // }

    /**
     * Column DEPENDENCIES of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DEPENDENCIES = 117;

    public static final int TYPE_DEPENDENCIES = Types.VARCHAR;

    public static final String NAME_DEPENDENCIES = "TMT_WBS.DEPENDENCIES";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DEPENDENCIES = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_DEPENDENCIES,NAME_DEPENDENCIES); }

    /**
     * Column ROLLUP_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_ROLLUP_FLAG = 118;

    public static final int TYPE_ROLLUP_FLAG = Types.CHAR;

    public static final String NAME_ROLLUP_FLAG = "TMT_WBS.ROLLUP_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ROLLUP_FLAG = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ROLLUP_FLAG,NAME_ROLLUP_FLAG); }

    /**
     * Column EV_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_EV_FLAG = 119;

    public static final int TYPE_EV_FLAG = Types.CHAR;

    public static final String NAME_EV_FLAG = "TMT_WBS.EV_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EV_FLAG = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_EV_FLAG,NAME_EV_FLAG); }

    /**
     * Column CRI_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CRI_COUNT = 120;

    public static final int TYPE_CRI_COUNT = Types.INTEGER;

    public static final String NAME_CRI_COUNT = "TMT_WBS.CRI_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CRI_COUNT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_CRI_COUNT,NAME_CRI_COUNT); }

    /**
     * Column DOC_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_DOC_COUNT = 121;

    public static final int TYPE_DOC_COUNT = Types.INTEGER;

    public static final String NAME_DOC_COUNT = "TMT_WBS.DOC_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DOC_COUNT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_DOC_COUNT,NAME_DOC_COUNT); }

    /**
     * Column MSP_ID of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_MSP_ID = 122;

    public static final int TYPE_MSP_ID = Types.INTEGER;

    public static final String NAME_MSP_ID = "TMT_WBS.MSP_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MSP_ID = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_MSP_ID,NAME_MSP_ID); }

    /**
     * Column BASELINE_WORK_TODATE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_BASELINE_WORK_TODATE = 123;

    public static final int TYPE_BASELINE_WORK_TODATE = Types.BIGINT;

    public static final String NAME_BASELINE_WORK_TODATE = "TMT_WBS.BASELINE_WORK_TODATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE_WORK_TODATE =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE_WORK_TODATE,NAME_BASELINE_WORK_TODATE);
    // }

    /**
     * Column ASSIGN_COUNT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_ASSIGN_COUNT = 124;

    public static final int TYPE_ASSIGN_COUNT = Types.SMALLINT;

    public static final String NAME_ASSIGN_COUNT = "TMT_WBS.ASSIGN_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSIGN_COUNT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ASSIGN_COUNT,NAME_ASSIGN_COUNT); }

    /**
     * Column ASSIGN_NAMES of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ASSIGN_NAMES = 125;

    public static final int TYPE_ASSIGN_NAMES = Types.VARCHAR;

    public static final String NAME_ASSIGN_NAMES = "TMT_WBS.ASSIGN_NAMES";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSIGN_NAMES = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ASSIGN_NAMES,NAME_ASSIGN_NAMES); }

    /**
     * Column RTF_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_RTF_BITS = 126;

    public static final int TYPE_RTF_BITS = Types.INTEGER;

    public static final String NAME_RTF_BITS = "TMT_WBS.RTF_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RTF_BITS = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_RTF_BITS,NAME_RTF_BITS); }

    /**
     * Column LAST_CHECKIN of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LAST_CHECKIN = 127;

    public static final int TYPE_LAST_CHECKIN = Types.TIMESTAMP;

    public static final String NAME_LAST_CHECKIN = "TMT_WBS.LAST_CHECKIN";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LAST_CHECKIN = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_LAST_CHECKIN,NAME_LAST_CHECKIN); }

    /**
     * Column BAC of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_BAC = 128;

    public static final int TYPE_BAC = Types.DECIMAL;

    public static final String NAME_BAC = "TMT_WBS.BAC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BAC = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_BAC,NAME_BAC); }

    /**
     * Column EAC of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_EAC = 129;

    public static final int TYPE_EAC = Types.DECIMAL;

    public static final String NAME_EAC = "TMT_WBS.EAC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EAC = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_EAC,NAME_EAC); }

    /**
     * Column BCWS of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_BCWS = 130;

    public static final int TYPE_BCWS = Types.DECIMAL;

    public static final String NAME_BCWS = "TMT_WBS.BCWS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BCWS = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_BCWS,NAME_BCWS); }

    /**
     * Column BCWP of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_BCWP = 131;

    public static final int TYPE_BCWP = Types.DECIMAL;

    public static final String NAME_BCWP = "TMT_WBS.BCWP";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BCWP = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_BCWP,NAME_BCWP); }

    /**
     * Column ACWP of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_ACWP = 132;

    public static final int TYPE_ACWP = Types.DECIMAL;

    public static final String NAME_ACWP = "TMT_WBS.ACWP";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACWP = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ACWP,NAME_ACWP); }

    /**
     * Column TRANSFER_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_TRANSFER_FLAG = 133;

    public static final int TYPE_TRANSFER_FLAG = Types.CHAR;

    public static final String NAME_TRANSFER_FLAG = "TMT_WBS.TRANSFER_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TRANSFER_FLAG = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_TRANSFER_FLAG,NAME_TRANSFER_FLAG);
    // }

    /**
     * Column EFFORT_DRIVEN of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_EFFORT_DRIVEN = 134;

    public static final int TYPE_EFFORT_DRIVEN = Types.SMALLINT;

    public static final String NAME_EFFORT_DRIVEN = "TMT_WBS.EFFORT_DRIVEN";

    static
    {
        FIELDPROPERTYMAP.put("EFFORTDRIVEN", SqlUtil.getBooleanValue(
            NAME_EFFORT_DRIVEN, 1));
        FIELDPROPERTYMAP.put("ASSIGNMENTTYPE", SqlUtil.castAsChar(NAME_EFFORT_DRIVEN, 1));
    }

    /**
     * Column BUBBLE_COLOR of type Types.SMALLINT mapped to Integer.
     * REMOVED COLUMN, KEPT HERE TO KEEP THE ID COUNT ACCURATE
     */
    // public static final int ID_BUBBLE_COLOR = 135;
    // public static final int TYPE_BUBBLE_COLOR = Types.SMALLINT; 
    // public static final String NAME_BUBBLE_COLOR = "TMT_WBS.BUBBLE_COLOR";
    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BUBBLE_COLOR = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_BUBBLE_COLOR,NAME_BUBBLE_COLOR); }

    /**
     * Column BUBBLE_PUBLISHED_DT of type Types.TIMESTAMP mapped to
     * java.util.Date.
     * REMOVED COLUMN, KEPT HERE TO KEEP THE ID COUNT ACCURATE
     */
    // public static final int ID_BUBBLE_PUBLISHED_DT = 136;
    // public static final int TYPE_BUBBLE_PUBLISHED_DT = Types.TIMESTAMP;
    // public static final String NAME_BUBBLE_PUBLISHED_DT = "TMT_WBS.BUBBLE_PUBLISHED_DT";
    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BUBBLE_PUBLISHED_DT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BUBBLE_PUBLISHED_DT,NAME_BUBBLE_PUBLISHED_DT);
    // }

    /**
     * Column A_CRI_COUNT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_A_CRI_COUNT = 137;

    public static final int TYPE_A_CRI_COUNT = Types.SMALLINT;

    public static final String NAME_A_CRI_COUNT = "TMT_WBS.A_CRI_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_A_CRI_COUNT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_A_CRI_COUNT,NAME_A_CRI_COUNT); }

    /**
     * Column TASK_TYPE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_TASK_TYPE = 138;

    public static final int TYPE_TASK_TYPE = Types.SMALLINT;

    public static final String NAME_TASK_TYPE = "TMT_WBS.TASK_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TASK_TYPE = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_TASK_TYPE,NAME_TASK_TYPE); }

    /**
     * Column FS_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FS_CODE = 139;

    public static final int TYPE_FS_CODE = Types.VARCHAR;

    public static final String NAME_FS_CODE = "TMT_WBS.FS_CODE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FS_CODE = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_FS_CODE,NAME_FS_CODE); }

    /**
     * Column FS_STATE of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_FS_STATE = 140;

    public static final int TYPE_FS_STATE = Types.SMALLINT;

    public static final String NAME_FS_STATE = "TMT_WBS.FS_STATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FS_STATE = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_FS_STATE,NAME_FS_STATE); }

    /**
     * Column FS_LABOR_CODE of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FS_LABOR_CODE = 141;

    public static final int TYPE_FS_LABOR_CODE = Types.VARCHAR;

    public static final String NAME_FS_LABOR_CODE = "TMT_WBS.FS_LABOR_CODE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_FS_LABOR_CODE = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_FS_LABOR_CODE,NAME_FS_LABOR_CODE);
    // }

    /**
     * Column EV_ETC of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_EV_ETC = 142;

    public static final int TYPE_EV_ETC = Types.DECIMAL;

    public static final String NAME_EV_ETC = "TMT_WBS.EV_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EV_ETC = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_EV_ETC,NAME_EV_ETC); }

    /**
     * Column REC_FLAGS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_REC_FLAGS = 143;

    public static final int TYPE_REC_FLAGS = Types.INTEGER;

    public static final String NAME_REC_FLAGS = "TMT_WBS.REC_FLAGS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_FLAGS = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_REC_FLAGS,NAME_REC_FLAGS); }

    /**
     * Column ROLLUP_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ROLLUP_BITS = 144;

    public static final int TYPE_ROLLUP_BITS = Types.INTEGER;

    public static final String NAME_ROLLUP_BITS = "TMT_WBS.ROLLUP_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ROLLUP_BITS = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ROLLUP_BITS,NAME_ROLLUP_BITS); }

    /**
     * Column UNRESOLVED_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_UNRESOLVED_START = 145;

    public static final int TYPE_UNRESOLVED_START = Types.TIMESTAMP;

    public static final String NAME_UNRESOLVED_START = "TMT_WBS.UNRESOLVED_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_UNRESOLVED_START = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_UNRESOLVED_START,NAME_UNRESOLVED_START); }

    /**
     * Column UNRESOLVED_FINISH of type Types.TIMESTAMP mapped to
     * java.util.Date.
     */
    public static final int ID_UNRESOLVED_FINISH = 146;

    public static final int TYPE_UNRESOLVED_FINISH = Types.TIMESTAMP;

    public static final String NAME_UNRESOLVED_FINISH = "TMT_WBS.UNRESOLVED_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_UNRESOLVED_FINISH = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_UNRESOLVED_FINISH,NAME_UNRESOLVED_FINISH);
    // }

    /**
     * Column PLANNED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PLANNED_BUDGET = 147;

    public static final int TYPE_PLANNED_BUDGET = Types.DECIMAL;

    public static final String NAME_PLANNED_BUDGET = "TMT_WBS.PLANNED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PLANNED_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_PLANNED_BUDGET,NAME_PLANNED_BUDGET); }

    /**
     * Column R_PLANNED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_PLANNED_BUDGET = 148;

    public static final int TYPE_R_PLANNED_BUDGET = Types.DECIMAL;

    public static final String NAME_R_PLANNED_BUDGET = "TMT_WBS.R_PLANNED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_PLANNED_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_PLANNED_BUDGET,NAME_R_PLANNED_BUDGET); }

    /**
     * Column S_PLANNED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_PLANNED_BUDGET = 149;

    public static final int TYPE_S_PLANNED_BUDGET = Types.DECIMAL;

    public static final String NAME_S_PLANNED_BUDGET = "TMT_WBS.S_PLANNED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_PLANNED_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_PLANNED_BUDGET,NAME_S_PLANNED_BUDGET); }

    /**
     * Column PROPOSED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_PROPOSED_BUDGET = 150;

    public static final int TYPE_PROPOSED_BUDGET = Types.DECIMAL;

    public static final String NAME_PROPOSED_BUDGET = "TMT_WBS.PROPOSED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PROPOSED_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_PROPOSED_BUDGET,NAME_PROPOSED_BUDGET); }

    /**
     * Column R_PROPOSED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_PROPOSED_BUDGET = 151;

    public static final int TYPE_R_PROPOSED_BUDGET = Types.DECIMAL;

    public static final String NAME_R_PROPOSED_BUDGET = "TMT_WBS.R_PROPOSED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_PROPOSED_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_PROPOSED_BUDGET,NAME_R_PROPOSED_BUDGET);
    // }

    /**
     * Column S_PROPOSED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_PROPOSED_BUDGET = 152;

    public static final int TYPE_S_PROPOSED_BUDGET = Types.DECIMAL;

    public static final String NAME_S_PROPOSED_BUDGET = "TMT_WBS.S_PROPOSED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_PROPOSED_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_PROPOSED_BUDGET,NAME_S_PROPOSED_BUDGET);
    // }

    /**
     * Column EARLY_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_EARLY_START = 153;

    public static final int TYPE_EARLY_START = Types.TIMESTAMP;

    public static final String NAME_EARLY_START = "TMT_WBS.EARLY_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EARLY_START = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_EARLY_START,NAME_EARLY_START); }

    /**
     * Column LATE_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LATE_START = 154;

    public static final int TYPE_LATE_START = Types.TIMESTAMP;

    public static final String NAME_LATE_START = "TMT_WBS.LATE_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LATE_START = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_LATE_START,NAME_LATE_START); }

    /**
     * Column EARLY_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_EARLY_FINISH = 155;

    public static final int TYPE_EARLY_FINISH = Types.TIMESTAMP;

    public static final String NAME_EARLY_FINISH = "TMT_WBS.EARLY_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EARLY_FINISH = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_EARLY_FINISH,NAME_EARLY_FINISH); }

    /**
     * Column LATE_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LATE_FINISH = 156;

    public static final int TYPE_LATE_FINISH = Types.TIMESTAMP;

    public static final String NAME_LATE_FINISH = "TMT_WBS.LATE_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LATE_FINISH = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_LATE_FINISH,NAME_LATE_FINISH); }

    /**
     * Column LEVEL_INDEX of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_INDEX = 157;

    public static final int TYPE_LEVEL_INDEX = Types.INTEGER;

    public static final String NAME_LEVEL_INDEX = "TMT_WBS.LEVEL_INDEX";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_INDEX = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_LEVEL_INDEX,NAME_LEVEL_INDEX); }

    /**
     * Column CALC_PRIORITY of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_CALC_PRIORITY = 158;

    public static final int TYPE_CALC_PRIORITY = Types.SMALLINT;

    public static final String NAME_CALC_PRIORITY = "TMT_WBS.CALC_PRIORITY";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CALC_PRIORITY = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_CALC_PRIORITY,NAME_CALC_PRIORITY);
    // }

    /**
     * Column LEVEL_CONFLICT of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LEVEL_CONFLICT = 159;

    public static final int TYPE_LEVEL_CONFLICT = Types.VARCHAR;

    public static final String NAME_LEVEL_CONFLICT = "TMT_WBS.LEVEL_CONFLICT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_CONFLICT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_LEVEL_CONFLICT,NAME_LEVEL_CONFLICT); }

    /**
     * Column LEVEL_START of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LEVEL_START = 160;

    public static final int TYPE_LEVEL_START = Types.TIMESTAMP;

    public static final String NAME_LEVEL_START = "TMT_WBS.LEVEL_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_START = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_LEVEL_START,NAME_LEVEL_START); }

    /**
     * Column LEVEL_FINISH of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_LEVEL_FINISH = 161;

    public static final int TYPE_LEVEL_FINISH = Types.TIMESTAMP;

    public static final String NAME_LEVEL_FINISH = "TMT_WBS.LEVEL_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_FINISH = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_LEVEL_FINISH,NAME_LEVEL_FINISH); }

    /**
     * Column LEVEL_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_DURATION = 162;

    public static final int TYPE_LEVEL_DURATION = Types.INTEGER;

    public static final String NAME_LEVEL_DURATION = "TMT_WBS.LEVEL_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_DURATION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_LEVEL_DURATION,NAME_LEVEL_DURATION); }

    /**
     * Column SCORECARD_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SCORECARD_COUNT = 163;

    public static final int TYPE_SCORECARD_COUNT = Types.INTEGER;

    public static final String NAME_SCORECARD_COUNT = "TMT_WBS.SCORECARD_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SCORECARD_COUNT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_SCORECARD_COUNT,NAME_SCORECARD_COUNT); }

    /**
     * Column FINISH_START_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_FINISH_START_DATE = 164;

    public static final int TYPE_FINISH_START_DATE = Types.DATE;

    public static final String NAME_FINISH_START_DATE = "TMT_WBS.FINISH_START_DATE";

    public static final String PROPERTY_FINISH_START_DATE = "ESTIMATEDFINISHDATE";
    static
    {
        FIELDPROPERTYMAP
            .put(PROPERTY_FINISH_START_DATE, NAME_FINISH_START_DATE);
    }

    /**
     * Column TOTAL_DURATION of type Types.BIGINT mapped to Long.
     */
    public static final int ID_TOTAL_DURATION = 165;

    public static final int TYPE_TOTAL_DURATION = Types.BIGINT;

    public static final String NAME_TOTAL_DURATION = "TMT_WBS.TOTAL_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TOTAL_DURATION = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_TOTAL_DURATION,NAME_TOTAL_DURATION); }

    /**
     * Column PREVIOUS_WKF_STEP of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PREVIOUS_WKF_STEP = 166;

    public static final int TYPE_PREVIOUS_WKF_STEP = Types.VARCHAR;

    public static final String NAME_PREVIOUS_WKF_STEP = "TMT_WBS.PREVIOUS_WKF_STEP";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_PREVIOUS_WKF_STEP = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_PREVIOUS_WKF_STEP,NAME_PREVIOUS_WKF_STEP);
    // }

    /**
     * Column DEPEND_FLAG of type Types.CHAR mapped to String.
     */
    public static final int ID_DEPEND_FLAG = 167;

    public static final int TYPE_DEPEND_FLAG = Types.CHAR;

    public static final String NAME_DEPEND_FLAG = "TMT_WBS.DEPEND_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_DEPEND_FLAG = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_DEPEND_FLAG,NAME_DEPEND_FLAG); }

    /**
     * Column START_FINISH_DURATION of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_START_FINISH_DURATION = 168;

    public static final int TYPE_START_FINISH_DURATION = Types.INTEGER;

    public static final String NAME_START_FINISH_DURATION = "TMT_WBS.START_FINISH_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_START_FINISH_DURATION =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_START_FINISH_DURATION,NAME_START_FINISH_DURATION);
    // }

    /**
     * Column CONVERTED_SCORE of type Types.BIGINT mapped to Long.
     */
    public static final int ID_CONVERTED_SCORE = 169;

    public static final int TYPE_CONVERTED_SCORE = Types.BIGINT;

    public static final String NAME_CONVERTED_SCORE = "TMT_WBS.CONVERTED_SCORE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONVERTED_SCORE = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_SCORE,NAME_CONVERTED_SCORE); }

    /**
     * Column CONVERTED_WEIGHT of type Types.BIGINT mapped to Long.
     */
    public static final int ID_CONVERTED_WEIGHT = 170;

    public static final int TYPE_CONVERTED_WEIGHT = Types.BIGINT;

    public static final String NAME_CONVERTED_WEIGHT = "TMT_WBS.CONVERTED_WEIGHT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONVERTED_WEIGHT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_CONVERTED_WEIGHT,NAME_CONVERTED_WEIGHT); }

    /**
     * Column TIMEPHASED_BITS of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TIMEPHASED_BITS = 171;

    public static final int TYPE_TIMEPHASED_BITS = Types.INTEGER;

    public static final String NAME_TIMEPHASED_BITS = "TMT_WBS.TIMEPHASED_BITS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TIMEPHASED_BITS = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_TIMEPHASED_BITS,NAME_TIMEPHASED_BITS); }

    /**
     * Column ESTIMATED_BASELINE1_START of type Types.DATE mapped to
     * java.util.Date.
     */
    public static final int ID_ESTIMATED_BASELINE1_START = 172;

    public static final int TYPE_ESTIMATED_BASELINE1_START = Types.DATE;

    public static final String NAME_ESTIMATED_BASELINE1_START = "TMT_WBS.ESTIMATED_BASELINE1_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE1_START =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE1_START,NAME_ESTIMATED_BASELINE1_START);
    // }

    /**
     * Column ESTIMATED_BASELINE1_FINISH of type Types.DATE mapped to
     * java.util.Date.
     */
    public static final int ID_ESTIMATED_BASELINE1_FINISH = 173;

    public static final int TYPE_ESTIMATED_BASELINE1_FINISH = Types.DATE;

    public static final String NAME_ESTIMATED_BASELINE1_FINISH = "TMT_WBS.ESTIMATED_BASELINE1_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE1_FINISH =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE1_FINISH,NAME_ESTIMATED_BASELINE1_FINISH);
    // }

    /**
     * Column ESTIMATED_BASELINE2_START of type Types.DATE mapped to
     * java.util.Date.
     */
    public static final int ID_ESTIMATED_BASELINE2_START = 174;

    public static final int TYPE_ESTIMATED_BASELINE2_START = Types.DATE;

    public static final String NAME_ESTIMATED_BASELINE2_START = "TMT_WBS.ESTIMATED_BASELINE2_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE2_START =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE2_START,NAME_ESTIMATED_BASELINE2_START);
    // }

    /**
     * Column ESTIMATED_BASELINE2_FINISH of type Types.DATE mapped to
     * java.util.Date.
     */
    public static final int ID_ESTIMATED_BASELINE2_FINISH = 175;

    public static final int TYPE_ESTIMATED_BASELINE2_FINISH = Types.DATE;

    public static final String NAME_ESTIMATED_BASELINE2_FINISH = "TMT_WBS.ESTIMATED_BASELINE2_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE2_FINISH =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE2_FINISH,NAME_ESTIMATED_BASELINE2_FINISH);
    // }

    /**
     * Column ESTIMATED_BASELINE3_START of type Types.DATE mapped to
     * java.util.Date.
     */
    public static final int ID_ESTIMATED_BASELINE3_START = 176;

    public static final int TYPE_ESTIMATED_BASELINE3_START = Types.DATE;

    public static final String NAME_ESTIMATED_BASELINE3_START = "TMT_WBS.ESTIMATED_BASELINE3_START";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE3_START =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE3_START,NAME_ESTIMATED_BASELINE3_START);
    // }

    /**
     * Column ESTIMATED_BASELINE3_FINISH of type Types.DATE mapped to
     * java.util.Date.
     */
    public static final int ID_ESTIMATED_BASELINE3_FINISH = 177;

    public static final int TYPE_ESTIMATED_BASELINE3_FINISH = Types.DATE;

    public static final String NAME_ESTIMATED_BASELINE3_FINISH = "TMT_WBS.ESTIMATED_BASELINE3_FINISH";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE3_FINISH =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE3_FINISH,NAME_ESTIMATED_BASELINE3_FINISH);
    // }

    /**
     * Column ESTIMATED_BASELINE1_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_BASELINE1_BUDGET = 178;

    public static final int TYPE_ESTIMATED_BASELINE1_BUDGET = Types.DECIMAL;

    public static final String NAME_ESTIMATED_BASELINE1_BUDGET = "TMT_WBS.ESTIMATED_BASELINE1_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE1_BUDGET =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE1_BUDGET,NAME_ESTIMATED_BASELINE1_BUDGET);
    // }

    /**
     * Column ESTIMATED_BASELINE2_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_BASELINE2_BUDGET = 179;

    public static final int TYPE_ESTIMATED_BASELINE2_BUDGET = Types.DECIMAL;

    public static final String NAME_ESTIMATED_BASELINE2_BUDGET = "TMT_WBS.ESTIMATED_BASELINE2_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE2_BUDGET =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE2_BUDGET,NAME_ESTIMATED_BASELINE2_BUDGET);
    // }

    /**
     * Column ESTIMATED_BASELINE3_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_BASELINE3_BUDGET = 180;

    public static final int TYPE_ESTIMATED_BASELINE3_BUDGET = Types.DECIMAL;

    public static final String NAME_ESTIMATED_BASELINE3_BUDGET = "TMT_WBS.ESTIMATED_BASELINE3_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE3_BUDGET =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE3_BUDGET,NAME_ESTIMATED_BASELINE3_BUDGET);
    // }

    /**
     * Column R_ESTIMATED_BASELINE1_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_BASELINE1_BUDGET = 181;

    public static final int TYPE_R_ESTIMATED_BASELINE1_BUDGET = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_BASELINE1_BUDGET = "TMT_WBS.R_ESTIMATED_BASELINE1_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_BASELINE1_BUDGET =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_BASELINE1_BUDGET,NAME_R_ESTIMATED_BASELINE1_BUDGET);
    // }

    /**
     * Column R_ESTIMATED_BASELINE2_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_BASELINE2_BUDGET = 182;

    public static final int TYPE_R_ESTIMATED_BASELINE2_BUDGET = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_BASELINE2_BUDGET = "TMT_WBS.R_ESTIMATED_BASELINE2_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_BASELINE2_BUDGET =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_BASELINE2_BUDGET,NAME_R_ESTIMATED_BASELINE2_BUDGET);
    // }

    /**
     * Column R_ESTIMATED_BASELINE3_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_BASELINE3_BUDGET = 183;

    public static final int TYPE_R_ESTIMATED_BASELINE3_BUDGET = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_BASELINE3_BUDGET = "TMT_WBS.R_ESTIMATED_BASELINE3_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_BASELINE3_BUDGET =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_BASELINE3_BUDGET,NAME_R_ESTIMATED_BASELINE3_BUDGET);
    // }

    /**
     * Column S_ESTIMATED_BASELINE1_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATED_BASELINE1_BUDGET = 184;

    public static final int TYPE_S_ESTIMATED_BASELINE1_BUDGET = Types.DECIMAL;

    public static final String NAME_S_ESTIMATED_BASELINE1_BUDGET = "TMT_WBS.S_ESTIMATED_BASELINE1_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATED_BASELINE1_BUDGET =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATED_BASELINE1_BUDGET,NAME_S_ESTIMATED_BASELINE1_BUDGET);
    // }

    /**
     * Column S_ESTIMATED_BASELINE2_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATED_BASELINE2_BUDGET = 185;

    public static final int TYPE_S_ESTIMATED_BASELINE2_BUDGET = Types.DECIMAL;

    public static final String NAME_S_ESTIMATED_BASELINE2_BUDGET = "TMT_WBS.S_ESTIMATED_BASELINE2_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATED_BASELINE2_BUDGET =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATED_BASELINE2_BUDGET,NAME_S_ESTIMATED_BASELINE2_BUDGET);
    // }

    /**
     * Column S_ESTIMATED_BASELINE3_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATED_BASELINE3_BUDGET = 186;

    public static final int TYPE_S_ESTIMATED_BASELINE3_BUDGET = Types.DECIMAL;

    public static final String NAME_S_ESTIMATED_BASELINE3_BUDGET = "TMT_WBS.S_ESTIMATED_BASELINE3_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATED_BASELINE3_BUDGET =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATED_BASELINE3_BUDGET,NAME_S_ESTIMATED_BASELINE3_BUDGET);
    // }

    /**
     * Column ESTIMATED_ACTUAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_ACTUAL = 187;

    public static final int TYPE_ESTIMATED_ACTUAL = Types.DECIMAL;

    public static final String NAME_ESTIMATED_ACTUAL = "TMT_WBS.ESTIMATED_ACTUAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_ACTUAL = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_ACTUAL,NAME_ESTIMATED_ACTUAL); }

    /**
     * Column ESTIMATED_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_ETC = 188;

    public static final int TYPE_ESTIMATED_ETC = Types.DECIMAL;

    public static final String NAME_ESTIMATED_ETC = "TMT_WBS.ESTIMATED_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_ETC = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_ETC,NAME_ESTIMATED_ETC);
    // }

    /**
     * Column R_ESTIMATED_ACTUAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_ACTUAL = 189;

    public static final int TYPE_R_ESTIMATED_ACTUAL = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_ACTUAL = "TMT_WBS.R_ESTIMATED_ACTUAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_ACTUAL = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_ACTUAL,NAME_R_ESTIMATED_ACTUAL);
    // }

    /**
     * Column R_ESTIMATED_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_ETC = 190;

    public static final int TYPE_R_ESTIMATED_ETC = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_ETC = "TMT_WBS.R_ESTIMATED_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_ETC = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_ETC,NAME_R_ESTIMATED_ETC); }

    /**
     * Column S_ESTIMATED_ACTUAL of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATED_ACTUAL = 191;

    public static final int TYPE_S_ESTIMATED_ACTUAL = Types.DECIMAL;

    public static final String NAME_S_ESTIMATED_ACTUAL = "TMT_WBS.S_ESTIMATED_ACTUAL";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATED_ACTUAL = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATED_ACTUAL,NAME_S_ESTIMATED_ACTUAL);
    // }

    /**
     * Column S_ESTIMATED_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATED_ETC = 192;

    public static final int TYPE_S_ESTIMATED_ETC = Types.DECIMAL;

    public static final String NAME_S_ESTIMATED_ETC = "TMT_WBS.S_ESTIMATED_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATED_ETC = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATED_ETC,NAME_S_ESTIMATED_ETC); }

    /**
     * Column ESTIMATED_HOURS of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_HOURS = 193;

    public static final int TYPE_ESTIMATED_HOURS = Types.DECIMAL;

    public static final String NAME_ESTIMATED_HOURS = "TMT_WBS.ESTIMATED_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_HOURS = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_HOURS,NAME_ESTIMATED_HOURS); }

    /**
     * Column ESTIMATED_ACTUAL_HOURS of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_ACTUAL_HOURS = 194;

    public static final int TYPE_ESTIMATED_ACTUAL_HOURS = Types.DECIMAL;

    public static final String NAME_ESTIMATED_ACTUAL_HOURS = "TMT_WBS.ESTIMATED_ACTUAL_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_ACTUAL_HOURS =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_ACTUAL_HOURS,NAME_ESTIMATED_ACTUAL_HOURS);
    // }

    /**
     * Column ESTIMATED_HOURS_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_HOURS_ETC = 195;

    public static final int TYPE_ESTIMATED_HOURS_ETC = Types.DECIMAL;

    public static final String NAME_ESTIMATED_HOURS_ETC = "TMT_WBS.ESTIMATED_HOURS_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_HOURS_ETC = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_HOURS_ETC,NAME_ESTIMATED_HOURS_ETC);
    // }

    /**
     * Column R_ESTIMATED_HOURS of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_HOURS = 196;

    public static final int TYPE_R_ESTIMATED_HOURS = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_HOURS = "TMT_WBS.R_ESTIMATED_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_HOURS = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_HOURS,NAME_R_ESTIMATED_HOURS);
    // }

    /**
     * Column R_ESTIMATED_ACTUAL_HOURS of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_ACTUAL_HOURS = 197;

    public static final int TYPE_R_ESTIMATED_ACTUAL_HOURS = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_ACTUAL_HOURS = "TMT_WBS.R_ESTIMATED_ACTUAL_HOURS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_ACTUAL_HOURS =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_ACTUAL_HOURS,NAME_R_ESTIMATED_ACTUAL_HOURS);
    // }

    /**
     * Column R_ESTIMATED_HOURS_ETC of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_HOURS_ETC = 198;

    public static final int TYPE_R_ESTIMATED_HOURS_ETC = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_HOURS_ETC = "TMT_WBS.R_ESTIMATED_HOURS_ETC";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_HOURS_ETC =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_HOURS_ETC,NAME_R_ESTIMATED_HOURS_ETC);
    // }

    /**
     * Column ESTIMATED_HOURS_BASELINE1 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_HOURS_BASELINE1 = 199;

    public static final int TYPE_ESTIMATED_HOURS_BASELINE1 = Types.DECIMAL;

    public static final String NAME_ESTIMATED_HOURS_BASELINE1 = "TMT_WBS.ESTIMATED_HOURS_BASELINE1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_HOURS_BASELINE1 =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_HOURS_BASELINE1,NAME_ESTIMATED_HOURS_BASELINE1);
    // }

    /**
     * Column ESTIMATED_HOURS_BASELINE2 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_HOURS_BASELINE2 = 200;

    public static final int TYPE_ESTIMATED_HOURS_BASELINE2 = Types.DECIMAL;

    public static final String NAME_ESTIMATED_HOURS_BASELINE2 = "TMT_WBS.ESTIMATED_HOURS_BASELINE2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_HOURS_BASELINE2 =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_HOURS_BASELINE2,NAME_ESTIMATED_HOURS_BASELINE2);
    // }

    /**
     * Column ESTIMATED_HOURS_BASELINE3 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_HOURS_BASELINE3 = 201;

    public static final int TYPE_ESTIMATED_HOURS_BASELINE3 = Types.DECIMAL;

    public static final String NAME_ESTIMATED_HOURS_BASELINE3 = "TMT_WBS.ESTIMATED_HOURS_BASELINE3";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_HOURS_BASELINE3 =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_HOURS_BASELINE3,NAME_ESTIMATED_HOURS_BASELINE3);
    // }

    /**
     * Column R_ESTIMATED_HOURS_BASELINE1 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_HOURS_BASELINE1 = 202;

    public static final int TYPE_R_ESTIMATED_HOURS_BASELINE1 = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_HOURS_BASELINE1 = "TMT_WBS.R_ESTIMATED_HOURS_BASELINE1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_HOURS_BASELINE1 =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_HOURS_BASELINE1,NAME_R_ESTIMATED_HOURS_BASELINE1);
    // }

    /**
     * Column R_ESTIMATED_HOURS_BASELINE2 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_HOURS_BASELINE2 = 203;

    public static final int TYPE_R_ESTIMATED_HOURS_BASELINE2 = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_HOURS_BASELINE2 = "TMT_WBS.R_ESTIMATED_HOURS_BASELINE2";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_HOURS_BASELINE2 = //
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_HOURS_BASELINE2,NAME_R_ESTIMATED_HOURS_BASELINE2);
    // }

    /**
     * Column R_ESTIMATED_HOURS_BASELINE3 of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_HOURS_BASELINE3 = 204;

    public static final int TYPE_R_ESTIMATED_HOURS_BASELINE3 = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_HOURS_BASELINE3 = "TMT_WBS.R_ESTIMATED_HOURS_BASELINE3";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_HOURS_BASELINE3 =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_HOURS_BASELINE3,NAME_R_ESTIMATED_HOURS_BASELINE3);
    // }

    /**
     * Column BASELINE1_EST_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_EST_DT = 205;

    public static final int TYPE_BASELINE1_EST_DT = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_EST_DT = "TMT_WBS.BASELINE1_EST_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_EST_DT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_EST_DT,NAME_BASELINE1_EST_DT); }

    /**
     * Column BASELINE2_EST_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_EST_DT = 206;

    public static final int TYPE_BASELINE2_EST_DT = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_EST_DT = "TMT_WBS.BASELINE2_EST_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_EST_DT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_EST_DT,NAME_BASELINE2_EST_DT); }

    /**
     * Column BASELINE3_EST_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE3_EST_DT = 207;

    public static final int TYPE_BASELINE3_EST_DT = Types.TIMESTAMP;

    public static final String NAME_BASELINE3_EST_DT = "TMT_WBS.BASELINE3_EST_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_EST_DT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_EST_DT,NAME_BASELINE3_EST_DT); }

    /**
     * Column BASELINE1_SCH_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE1_SCH_DT = 208;

    public static final int TYPE_BASELINE1_SCH_DT = Types.TIMESTAMP;

    public static final String NAME_BASELINE1_SCH_DT = "TMT_WBS.BASELINE1_SCH_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE1_SCH_DT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE1_SCH_DT,NAME_BASELINE1_SCH_DT); }

    /**
     * Column BASELINE2_SCH_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE2_SCH_DT = 209;

    public static final int TYPE_BASELINE2_SCH_DT = Types.TIMESTAMP;

    public static final String NAME_BASELINE2_SCH_DT = "TMT_WBS.BASELINE2_SCH_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE2_SCH_DT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE2_SCH_DT,NAME_BASELINE2_SCH_DT); }

    /**
     * Column BASELINE3_SCH_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_BASELINE3_SCH_DT = 210;

    public static final int TYPE_BASELINE3_SCH_DT = Types.TIMESTAMP;

    public static final String NAME_BASELINE3_SCH_DT = "TMT_WBS.BASELINE3_SCH_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_SCH_DT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_SCH_DT,NAME_BASELINE3_SCH_DT); }

    /**
     * Column CUSTOM_INT_1 of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CUSTOM_INT_1 = 211;

    public static final int TYPE_CUSTOM_INT_1 = Types.INTEGER;

    public static final String NAME_CUSTOM_INT_1 = "TMT_WBS.CUSTOM_INT_1";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CUSTOM_INT_1 = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_CUSTOM_INT_1,NAME_CUSTOM_INT_1); }

    /**
     * Column SUBPROJECT_COUNT of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_SUBPROJECT_COUNT = 212;

    public static final int TYPE_SUBPROJECT_COUNT = Types.INTEGER;

    public static final String NAME_SUBPROJECT_COUNT = "TMT_WBS.SUBPROJECT_COUNT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_SUBPROJECT_COUNT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_SUBPROJECT_COUNT,NAME_SUBPROJECT_COUNT); }

    /**
     * Column ESTIMATED_BASELINE1_DURATION of type Types.INTEGER mapped to
     * Integer.
     */
    public static final int ID_ESTIMATED_BASELINE1_DURATION = 213;

    public static final int TYPE_ESTIMATED_BASELINE1_DURATION = Types.INTEGER;

    public static final String NAME_ESTIMATED_BASELINE1_DURATION = "TMT_WBS.ESTIMATED_BASELINE1_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE1_DURATION =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE1_DURATION,NAME_ESTIMATED_BASELINE1_DURATION);
    // }

    /**
     * Column ESTIMATED_BASELINE2_DURATION of type Types.INTEGER mapped to
     * Integer.
     */
    public static final int ID_ESTIMATED_BASELINE2_DURATION = 214;

    public static final int TYPE_ESTIMATED_BASELINE2_DURATION = Types.INTEGER;

    public static final String NAME_ESTIMATED_BASELINE2_DURATION = "TMT_WBS.ESTIMATED_BASELINE2_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE2_DURATION =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE2_DURATION,NAME_ESTIMATED_BASELINE2_DURATION);
    // }

    /**
     * Column ESTIMATED_BASELINE3_DURATION of type Types.INTEGER mapped to
     * Integer.
     */
    public static final int ID_ESTIMATED_BASELINE3_DURATION = 215;

    public static final int TYPE_ESTIMATED_BASELINE3_DURATION = Types.INTEGER;

    public static final String NAME_ESTIMATED_BASELINE3_DURATION = "TMT_WBS.ESTIMATED_BASELINE3_DURATION";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_BASELINE3_DURATION =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_BASELINE3_DURATION,NAME_ESTIMATED_BASELINE3_DURATION);
    // }

    /**
     * Column TIMECODE_1_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TIMECODE_1_ID = 216;

    public static final int TYPE_TIMECODE_1_ID = Types.VARCHAR;

    public static final String NAME_TIMECODE_1_ID = "TMT_WBS.TIMECODE_1_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TIMECODE_1_ID = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_1_ID,NAME_TIMECODE_1_ID);
    // }

    /**
     * Column TIMECODE_2_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TIMECODE_2_ID = 217;

    public static final int TYPE_TIMECODE_2_ID = Types.VARCHAR;

    public static final String NAME_TIMECODE_2_ID = "TMT_WBS.TIMECODE_2_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TIMECODE_2_ID = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_TIMECODE_2_ID,NAME_TIMECODE_2_ID);
    // }

    /**
     * Column RISK_EXPOSURE of type Types.INTEGER mapped to Integer
     */
    public static final int ID_RISK_EXPOSURE = 218;

    public static final int TYPE_RISK_EXPOSURE = Types.INTEGER;

    public static final String NAME_RISK_EXPOSURE = "TMT_WBS.RISK_EXPOSURE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_RISK_EXPOSURE = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_RISK_EXPOSURE,NAME_RISK_EXPOSURE);
    // }

    /**
     * Column CONSEQUENSE_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_CONSEQUENSE_COST = 219;

    public static final int TYPE_CONSEQUENSE_COST = Types.DECIMAL;

    public static final String NAME_CONSEQUENSE_COST = "TMT_WBS.CONSEQUENSE_COST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CONSEQUENSE_COST = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_CONSEQUENSE_COST,NAME_CONSEQUENSE_COST); }

    /**
     * Column EAC_FORECAST of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_EAC_FORECAST = 220;

    public static final int TYPE_EAC_FORECAST = Types.DECIMAL;

    public static final String NAME_EAC_FORECAST = "TMT_WBS.EAC_FORECAST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EAC_FORECAST = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_EAC_FORECAST,NAME_EAC_FORECAST); }

    /**
     * Column EV_ETC_FORECAST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_EV_ETC_FORECAST = 221;

    public static final int TYPE_EV_ETC_FORECAST = Types.DECIMAL;

    public static final String NAME_EV_ETC_FORECAST = "TMT_WBS.EV_ETC_FORECAST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_EV_ETC_FORECAST = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_EV_ETC_FORECAST,NAME_EV_ETC_FORECAST); }

    /**
     * Column LEVEL_STATE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_STATE = 222;

    public static final int TYPE_LEVEL_STATE = Types.INTEGER;

    public static final String NAME_LEVEL_STATE = "TMT_WBS.LEVEL_STATE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_STATE = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_LEVEL_STATE,NAME_LEVEL_STATE); }

    /**
     * Column LEVEL_CONSTRAINT_TYPE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_LEVEL_CONSTRAINT_TYPE = 223;

    public static final int TYPE_LEVEL_CONSTRAINT_TYPE = Types.INTEGER;

    public static final String NAME_LEVEL_CONSTRAINT_TYPE = "TMT_WBS.LEVEL_CONSTRAINT_TYPE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_CONSTRAINT_TYPE =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_LEVEL_CONSTRAINT_TYPE,NAME_LEVEL_CONSTRAINT_TYPE);
    // }

    /**
     * Column LEVEL_CONSTRAINT_DT of type Types.VARCHAR mapped to String.
     */
    public static final int ID_LEVEL_CONSTRAINT_DT = 224;

    public static final int TYPE_LEVEL_CONSTRAINT_DT = Types.VARCHAR;

    public static final String NAME_LEVEL_CONSTRAINT_DT = "TMT_WBS.LEVEL_CONSTRAINT_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEVEL_CONSTRAINT_DT = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_LEVEL_CONSTRAINT_DT,NAME_LEVEL_CONSTRAINT_DT);
    // }

    /**
     * Column WORK_PERCENT of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_WORK_PERCENT = 225;

    public static final int TYPE_WORK_PERCENT = Types.SMALLINT;

    public static final String NAME_WORK_PERCENT = "TMT_WBS.WORK_PERCENT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_WORK_PERCENT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_WORK_PERCENT,NAME_WORK_PERCENT); }

    /**
     * Column MP_STATUS_DT of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_MP_STATUS_DT = 226;

    public static final int TYPE_MP_STATUS_DT = Types.TIMESTAMP;

    public static final String NAME_MP_STATUS_DT = "TMT_WBS.MP_STATUS_DT";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_MP_STATUS_DT = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_MP_STATUS_DT,NAME_MP_STATUS_DT); }

    /**
     * Column REFERENCE_NUMBER of type Types.BIGINT mapped to Long.
     */
    public static final int ID_REFERENCE_NUMBER = 227;

    public static final int TYPE_REFERENCE_NUMBER = Types.BIGINT;

    public static final String NAME_REFERENCE_NUMBER = "TMT_WBS.REFERENCE_NUMBER";

    public static final String PROPERTY_REFERENCE_NUMBER = "REFERENCEID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_REFERENCE_NUMBER, NAME_REFERENCE_NUMBER);
    }

    /**
     * Column ASSET_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ASSET_ID = 228;

    public static final int TYPE_ASSET_ID = Types.CHAR;

    public static final String NAME_ASSET_ID = "TMT_WBS.ASSET_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSET_ID = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ASSET_ID,NAME_ASSET_ID); }

    /**
     * Column ASSET_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ASSET_NAME = 229;

    public static final int TYPE_ASSET_NAME = Types.VARCHAR;

    public static final String NAME_ASSET_NAME = "TMT_WBS.ASSET_NAME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSET_NAME = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ASSET_NAME,NAME_ASSET_NAME); }

    /**
     * Column FINANCIAL_CATEGORY of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_FINANCIAL_CATEGORY = 230;

    public static final int TYPE_FINANCIAL_CATEGORY = Types.INTEGER;

    public static final String NAME_FINANCIAL_CATEGORY = "TMT_WBS.FINANCIAL_CATEGORY";

    public static final String PROPERTY_FINANCIAL_CATEGORY = "ASSETCODE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_FINANCIAL_CATEGORY,
            NAME_FINANCIAL_CATEGORY);
    }

    /**
     * Column POOL_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_POOL_ID = 231;

    public static final int TYPE_POOL_ID = Types.CHAR;

    public static final String NAME_POOL_ID = "TMT_WBS.POOL_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_POOL_ID = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_POOL_ID,NAME_POOL_ID); }

    /**
     * Column ASSET_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_ASSET_FLAG = 232;

    public static final int TYPE_ASSET_FLAG = Types.INTEGER;

    public static final String NAME_ASSET_FLAG = "TMT_WBS.ASSET_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSET_FLAG = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ASSET_FLAG,NAME_ASSET_FLAG); }

    /**
     * Column BASELINE3_START_VARIANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE3_START_VARIANCE = 233;

    public static final int TYPE_BASELINE3_START_VARIANCE = Types.INTEGER;

    public static final String NAME_BASELINE3_START_VARIANCE = "TMT_WBS.BASELINE3_START_VARIANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_START_VARIANCE =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_START_VARIANCE,NAME_BASELINE3_START_VARIANCE);
    // }

    /**
     * Column BASELINE3_FINISH_VARIANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_BASELINE3_FINISH_VARIANCE = 234;

    public static final int TYPE_BASELINE3_FINISH_VARIANCE = Types.INTEGER;

    public static final String NAME_BASELINE3_FINISH_VARIANCE = "TMT_WBS.BASELINE3_FINISH_VARIANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE3_FINISH_VARIANCE =
    // "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_BASELINE3_FINISH_VARIANCE,NAME_BASELINE3_FINISH_VARIANCE);
    // }

    /**
     * Column ELEMENT_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_ELEMENT_FLAG = 235;

    public static final int TYPE_ELEMENT_FLAG = Types.SMALLINT;

    public static final String NAME_ELEMENT_FLAG = "TMT_WBS.ELEMENT_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ELEMENT_FLAG = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_FLAG,NAME_ELEMENT_FLAG); }

    /**
     * Column BASELINE_FLAG of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_BASELINE_FLAG = 236;

    public static final int TYPE_BASELINE_FLAG = Types.SMALLINT;

    public static final String NAME_BASELINE_FLAG = "TMT_WBS.BASELINE_FLAG";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BASELINE_FLAG = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_BASELINE_FLAG,NAME_BASELINE_FLAG);
    // }

    /**
     * Column CHARGED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_CHARGED_BUDGET = 237;

    public static final int TYPE_CHARGED_BUDGET = Types.DECIMAL;

    public static final String NAME_CHARGED_BUDGET = "TMT_WBS.CHARGED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_CHARGED_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_CHARGED_BUDGET,NAME_CHARGED_BUDGET); }

    /**
     * Column R_CHARGED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_CHARGED_BUDGET = 238;

    public static final int TYPE_R_CHARGED_BUDGET = Types.DECIMAL;

    public static final String NAME_R_CHARGED_BUDGET = "TMT_WBS.R_CHARGED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_CHARGED_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_CHARGED_BUDGET,NAME_R_CHARGED_BUDGET); }

    /**
     * Column S_CHARGED_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_CHARGED_BUDGET = 239;

    public static final int TYPE_S_CHARGED_BUDGET = Types.DECIMAL;

    public static final String NAME_S_CHARGED_BUDGET = "TMT_WBS.S_CHARGED_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_CHARGED_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_CHARGED_BUDGET,NAME_S_CHARGED_BUDGET); }

    /**
     * Column LEDGER_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_LEDGER_BUDGET = 240;

    public static final int TYPE_LEDGER_BUDGET = Types.DECIMAL;

    public static final String NAME_LEDGER_BUDGET = "TMT_WBS.LEDGER_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_LEDGER_BUDGET = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_LEDGER_BUDGET,NAME_LEDGER_BUDGET);
    // }

    /**
     * Column R_LEDGER_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_LEDGER_BUDGET = 241;

    public static final int TYPE_R_LEDGER_BUDGET = Types.DECIMAL;

    public static final String NAME_R_LEDGER_BUDGET = "TMT_WBS.R_LEDGER_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_LEDGER_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_LEDGER_BUDGET,NAME_R_LEDGER_BUDGET); }

    /**
     * Column S_LEDGER_BUDGET of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_LEDGER_BUDGET = 242;

    public static final int TYPE_S_LEDGER_BUDGET = Types.DECIMAL;

    public static final String NAME_S_LEDGER_BUDGET = "TMT_WBS.S_LEDGER_BUDGET";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_LEDGER_BUDGET = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_LEDGER_BUDGET,NAME_S_LEDGER_BUDGET); }

    /**
     * Column ESTIMATED_CHARGED of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_CHARGED = 243;

    public static final int TYPE_ESTIMATED_CHARGED = Types.DECIMAL;

    public static final String NAME_ESTIMATED_CHARGED = "TMT_WBS.ESTIMATED_CHARGED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_CHARGED = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_CHARGED,NAME_ESTIMATED_CHARGED);
    // }

    /**
     * Column R_ESTIMATED_CHARGED of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_CHARGED = 244;

    public static final int TYPE_R_ESTIMATED_CHARGED = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_CHARGED = "TMT_WBS.R_ESTIMATED_CHARGED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_CHARGED = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_CHARGED,NAME_R_ESTIMATED_CHARGED);
    // }

    /**
     * Column S_ESTIMATED_CHARGED of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATED_CHARGED = 245;

    public static final int TYPE_S_ESTIMATED_CHARGED = Types.DECIMAL;

    public static final String NAME_S_ESTIMATED_CHARGED = "TMT_WBS.S_ESTIMATED_CHARGED";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATED_CHARGED = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATED_CHARGED,NAME_S_ESTIMATED_CHARGED);
    // }

    /**
     * Column ESTIMATED_LEDGER of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_LEDGER = 246;

    public static final int TYPE_ESTIMATED_LEDGER = Types.DECIMAL;

    public static final String NAME_ESTIMATED_LEDGER = "TMT_WBS.ESTIMATED_LEDGER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ESTIMATED_LEDGER = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_LEDGER,NAME_ESTIMATED_LEDGER); }

    /**
     * Column R_ESTIMATED_LEDGER of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_R_ESTIMATED_LEDGER = 247;

    public static final int TYPE_R_ESTIMATED_LEDGER = Types.DECIMAL;

    public static final String NAME_R_ESTIMATED_LEDGER = "TMT_WBS.R_ESTIMATED_LEDGER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_R_ESTIMATED_LEDGER = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_R_ESTIMATED_LEDGER,NAME_R_ESTIMATED_LEDGER);
    // }

    /**
     * Column S_ESTIMATED_LEDGER of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_S_ESTIMATED_LEDGER = 248;

    public static final int TYPE_S_ESTIMATED_LEDGER = Types.DECIMAL;

    public static final String NAME_S_ESTIMATED_LEDGER = "TMT_WBS.S_ESTIMATED_LEDGER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_S_ESTIMATED_LEDGER = "PROPERTYNAME";
    //static {
    // FIELDPROPERTYMAP.put(PROPERTY_S_ESTIMATED_LEDGER,NAME_S_ESTIMATED_LEDGER);
    // }

    /**
     * Column BCWS_FORECAST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BCWS_FORECAST = 249;

    public static final int TYPE_BCWS_FORECAST = Types.DECIMAL;

    public static final String NAME_BCWS_FORECAST = "TMT_WBS.BCWS_FORECAST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BCWS_FORECAST = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_BCWS_FORECAST,NAME_BCWS_FORECAST);
    // }

    /**
     * Column BCWP_FORECAST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_BCWP_FORECAST = 250;

    public static final int TYPE_BCWP_FORECAST = Types.DECIMAL;

    public static final String NAME_BCWP_FORECAST = "TMT_WBS.BCWP_FORECAST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_BCWP_FORECAST = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_BCWP_FORECAST,NAME_BCWP_FORECAST);
    // }

    /**
     * Column ACWP_FORECAST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ACWP_FORECAST = 251;

    public static final int TYPE_ACWP_FORECAST = Types.DECIMAL;

    public static final String NAME_ACWP_FORECAST = "TMT_WBS.ACWP_FORECAST";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ACWP_FORECAST = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_ACWP_FORECAST,NAME_ACWP_FORECAST);
    // }

    /**
     * Column TIME_VARIANCE of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_TIME_VARIANCE = 252;

    public static final int TYPE_TIME_VARIANCE = Types.INTEGER;

    public static final String NAME_TIME_VARIANCE = "TMT_WBS.TIME_VARIANCE";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_TIME_VARIANCE = "PROPERTYNAME";
    //static { FIELDPROPERTYMAP.put(PROPERTY_TIME_VARIANCE,NAME_TIME_VARIANCE);
    // }

    /**
     * Column EXTERNAL_ID of type Types.VARCHAR mapped to String.
     */
    public static final int ID_EXTERNAL_ID = 253;

    public static final int TYPE_EXTERNAL_ID = Types.VARCHAR;

    public static final String NAME_EXTERNAL_ID = "TMT_WBS.EXTERNAL_ID";

    public static final String PROPERTY_EXTERNAL_ID = "EXTERNALID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_EXTERNAL_ID, NAME_EXTERNAL_ID);
    }

    /**
     * Column CONFIG_FLAG of type Types.INTEGER mapped to Integer.
     */
    public static final int ID_CONFIG_FLAG = 254;

    public static final int TYPE_CONFIG_FLAG = Types.INTEGER;

    public static final String NAME_CONFIG_FLAG = "TMT_WBS.CONFIG_FLAG";

    public static final String PROPERTY_CONFIG_FLAG = "MANDATORY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CONFIG_FLAG, SqlUtil.getBooleanValue(
            NAME_CONFIG_FLAG, 1));
    }

    /**
     * Column CLOSED_BY_NAME of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CLOSED_BY_NAME = 255;

    public static final int TYPE_CLOSED_BY_NAME = Types.VARCHAR;

    public static final String NAME_CLOSED_BY_NAME = "TMT_WBS.CLOSED_BY_NAME";

    public static final String PROPERTY_CLOSED_BY_NAME = "CLOSEDBYNAME";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CLOSED_BY_NAME, NAME_CLOSED_BY_NAME);
    }

    /**
     * Column CLOSED_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_CLOSED_DATETIME = 256;

    public static final int TYPE_CLOSED_DATETIME = Types.TIMESTAMP;

    public static final String NAME_CLOSED_DATETIME = "TMT_WBS.CLOSED_DATETIME";

    public static final String PROPERTY_CLOSED_DATETIME = "CLOSEDBYDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_CLOSED_DATETIME, SqlUtil
            .convertTimestampToDate(NAME_CLOSED_DATETIME));
    }

    public static final String TABLE_NAME = "TMT_WBS";

    /**
     * Create an array of type string containing all the fields of the TMT_WBS
     * table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_RANK, NAME_WBS_CODE, NAME_PROJECT_ID, NAME_PARENT_ID,
            NAME_ELEMENT_ID, NAME_TYPE_ID, NAME_LEVEL_ID, NAME_MAX_RANK,
            NAME_CHILD_COUNT, NAME_DELETED_COUNT, NAME_ELEMENT_NAME,
            NAME_REF_CODE, NAME_PRIORITY, NAME_PUBLISHED, NAME_PUBLISHED_BY,
            NAME_T_PARENT_ID, NAME_T_RANK, NAME_CONSTRAINT_TYPE,
            NAME_CONSTRAINT_DT, NAME_CURRENCY_ID, NAME_CALENDAR_ID,
            NAME_MINUTE_PER_DAY, NAME_MINUTE_PER_WEEK, NAME_METHOD,
            NAME_BUFFER_PERCENT, NAME_START_FINISH_FLAG,
            NAME_START_FINISH_DATE, NAME_SD, NAME_FD, NAME_START_DT,
            NAME_FINISH_DT, NAME_BASELINE1_START, NAME_BASELINE1_FINISH,
            NAME_BASELINE2_START, NAME_BASELINE2_FINISH, NAME_BASELINE3_START,
            NAME_BASELINE3_FINISH, NAME_PLANNED_START, NAME_PLANNED_FINISH,
            NAME_PROPOSED_START, NAME_PROPOSED_FINISH, NAME_SCHED_START,
            NAME_SCHED_FINISH, NAME_FORECAST_START, NAME_FORECAST_FINISH,
            NAME_ACTUAL_START, NAME_ACTUAL_FINISH, NAME_UNRESOLVED_DURATION,
            NAME_DURATION, NAME_PLANNED_DURATION, NAME_PROPOSED_DURATION,
            NAME_SCHED_DURATION, NAME_FORECAST_DURATION, NAME_ACTUAL_DURATION,
            NAME_BASELINE1_DURATION, NAME_BASELINE2_DURATION,
            NAME_BASELINE3_DURATION, NAME_VARIANCE, NAME_FORECAST_VARIANCE,
            NAME_ACTUAL_VARIANCE, NAME_BASELINE1_VARIANCE, NAME_PLANNED_EFFORT,
            NAME_PROPOSED_EFFORT, NAME_SCHED_EFFORT, NAME_FORECAST_EFFORT,
            NAME_ACTUAL_EFFORT, NAME_BASELINE1_EFFORT, NAME_BASELINE2_EFFORT,
            NAME_BASELINE3_EFFORT, NAME_WORK_TODATE, NAME_ETC, NAME_CREATED_BY,
            NAME_CREATED_DT, NAME_PROBABILITY, NAME_STAGE_ID,
            NAME_MILESTONE_PERCENT, NAME_REC_USER, NAME_REC_STATUS,
            NAME_REC_DATETIME, NAME_LAST_WF_TYPE, NAME_LAST_WF_BY,
            NAME_LAST_WF_DT, NAME_MAJOR_REVISION, NAME_MINOR_REVISION,
            NAME_REVISION_IMPACT, NAME_REVISION_TYPE, NAME_REVISION_HISTORY,
            NAME_CLOSED_BY, NAME_TOOLTIP_DESCR, NAME_TOTAL_SLACK,
            NAME_FREE_SLACK, NAME_CRITICAL, NAME_ESTIMATION_TOTAL,
            NAME_ESTIMATION_BUDGET, NAME_CURRENT_BUDGET, NAME_FORECAST_BUDGET,
            NAME_ACTUAL_BUDGET, NAME_BASELINE1_BUDGET, NAME_BASELINE2_BUDGET,
            NAME_BASELINE3_BUDGET, NAME_R_ESTIMATION_TOTAL,
            NAME_R_ESTIMATION_BUDGET, NAME_R_CURRENT_BUDGET,
            NAME_R_FORECAST_BUDGET, NAME_R_ACTUAL_BUDGET,
            NAME_R_BASELINE1_BUDGET, NAME_R_BASELINE2_BUDGET,
            NAME_R_BASELINE3_BUDGET, NAME_S_ESTIMATION_TOTAL,
            NAME_S_ESTIMATION_BUDGET, NAME_S_CURRENT_BUDGET,
            NAME_S_FORECAST_BUDGET, NAME_S_ACTUAL_BUDGET,
            NAME_S_BASELINE1_BUDGET, NAME_S_BASELINE2_BUDGET,
            NAME_S_BASELINE3_BUDGET, NAME_DEPENDENCIES, NAME_ROLLUP_FLAG,
            NAME_EV_FLAG, NAME_CRI_COUNT, NAME_DOC_COUNT, NAME_MSP_ID,
            NAME_BASELINE_WORK_TODATE, NAME_ASSIGN_COUNT, NAME_ASSIGN_NAMES,
            NAME_RTF_BITS, NAME_LAST_CHECKIN, NAME_BAC, NAME_EAC, NAME_BCWS,
            NAME_BCWP, NAME_ACWP, NAME_TRANSFER_FLAG, NAME_EFFORT_DRIVEN,
            NAME_EFFORT_DRIVEN, NAME_EFFORT_DRIVEN, NAME_A_CRI_COUNT,
            NAME_TASK_TYPE, NAME_FS_CODE, NAME_FS_STATE, NAME_FS_LABOR_CODE,
            NAME_EV_ETC, NAME_REC_FLAGS, NAME_ROLLUP_BITS,
            NAME_UNRESOLVED_START, NAME_UNRESOLVED_FINISH, NAME_PLANNED_BUDGET,
            NAME_R_PLANNED_BUDGET, NAME_S_PLANNED_BUDGET, NAME_PROPOSED_BUDGET,
            NAME_R_PROPOSED_BUDGET, NAME_S_PROPOSED_BUDGET, NAME_EARLY_START,
            NAME_LATE_START, NAME_EARLY_FINISH, NAME_LATE_FINISH,
            NAME_LEVEL_INDEX, NAME_CALC_PRIORITY, NAME_LEVEL_CONFLICT,
            NAME_LEVEL_START, NAME_LEVEL_FINISH, NAME_LEVEL_DURATION,
            NAME_SCORECARD_COUNT, NAME_FINISH_START_DATE, NAME_TOTAL_DURATION,
            NAME_PREVIOUS_WKF_STEP, NAME_DEPEND_FLAG,
            NAME_START_FINISH_DURATION, NAME_CONVERTED_SCORE,
            NAME_CONVERTED_WEIGHT, NAME_TIMEPHASED_BITS,
            NAME_ESTIMATED_BASELINE1_START, NAME_ESTIMATED_BASELINE1_FINISH,
            NAME_ESTIMATED_BASELINE2_START, NAME_ESTIMATED_BASELINE2_FINISH,
            NAME_ESTIMATED_BASELINE3_START, NAME_ESTIMATED_BASELINE3_FINISH,
            NAME_ESTIMATED_BASELINE1_BUDGET, NAME_ESTIMATED_BASELINE2_BUDGET,
            NAME_ESTIMATED_BASELINE3_BUDGET, NAME_R_ESTIMATED_BASELINE1_BUDGET,
            NAME_R_ESTIMATED_BASELINE2_BUDGET,
            NAME_R_ESTIMATED_BASELINE3_BUDGET,
            NAME_S_ESTIMATED_BASELINE1_BUDGET,
            NAME_S_ESTIMATED_BASELINE2_BUDGET,
            NAME_S_ESTIMATED_BASELINE3_BUDGET, NAME_ESTIMATED_ACTUAL,
            NAME_ESTIMATED_ETC, NAME_R_ESTIMATED_ACTUAL, NAME_R_ESTIMATED_ETC,
            NAME_S_ESTIMATED_ACTUAL, NAME_S_ESTIMATED_ETC,
            NAME_ESTIMATED_HOURS, NAME_ESTIMATED_ACTUAL_HOURS,
            NAME_ESTIMATED_HOURS_ETC, NAME_R_ESTIMATED_HOURS,
            NAME_R_ESTIMATED_ACTUAL_HOURS, NAME_R_ESTIMATED_HOURS_ETC,
            NAME_ESTIMATED_HOURS_BASELINE1, NAME_ESTIMATED_HOURS_BASELINE2,
            NAME_ESTIMATED_HOURS_BASELINE3, NAME_R_ESTIMATED_HOURS_BASELINE1,
            NAME_R_ESTIMATED_HOURS_BASELINE2, NAME_R_ESTIMATED_HOURS_BASELINE3,
            NAME_BASELINE1_EST_DT, NAME_BASELINE2_EST_DT,
            NAME_BASELINE3_EST_DT, NAME_BASELINE1_SCH_DT,
            NAME_BASELINE2_SCH_DT, NAME_BASELINE3_SCH_DT, NAME_CUSTOM_INT_1,
            NAME_SUBPROJECT_COUNT, NAME_ESTIMATED_BASELINE1_DURATION,
            NAME_ESTIMATED_BASELINE2_DURATION,
            NAME_ESTIMATED_BASELINE3_DURATION, NAME_TIMECODE_1_ID,
            NAME_TIMECODE_2_ID, NAME_RISK_EXPOSURE, NAME_CONSEQUENSE_COST,
            NAME_EAC_FORECAST, NAME_EV_ETC_FORECAST, NAME_LEVEL_STATE,
            NAME_LEVEL_CONSTRAINT_TYPE, NAME_LEVEL_CONSTRAINT_DT,
            NAME_WORK_PERCENT, NAME_MP_STATUS_DT, NAME_REFERENCE_NUMBER,
            NAME_ASSET_ID, NAME_ASSET_NAME, NAME_FINANCIAL_CATEGORY,
            NAME_POOL_ID, NAME_ASSET_FLAG, NAME_BASELINE3_START_VARIANCE,
            NAME_BASELINE3_FINISH_VARIANCE, NAME_ELEMENT_FLAG,
            NAME_BASELINE_FLAG, NAME_CHARGED_BUDGET, NAME_R_CHARGED_BUDGET,
            NAME_S_CHARGED_BUDGET, NAME_LEDGER_BUDGET, NAME_R_LEDGER_BUDGET,
            NAME_S_LEDGER_BUDGET, NAME_ESTIMATED_CHARGED,
            NAME_R_ESTIMATED_CHARGED, NAME_S_ESTIMATED_CHARGED,
            NAME_ESTIMATED_LEDGER, NAME_R_ESTIMATED_LEDGER,
            NAME_S_ESTIMATED_LEDGER, NAME_BCWS_FORECAST, NAME_BCWP_FORECAST,
            NAME_ACWP_FORECAST, NAME_TIME_VARIANCE, NAME_EXTERNAL_ID,
            NAME_CONFIG_FLAG, NAME_CLOSED_BY_NAME, NAME_CLOSED_DATETIME
    };

    private static final String ALL_FIELDS;

    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);

        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }

        ALL_FIELDS = fieldcat.toString();
    }

    /**
     * Creates a new WBSManager instance.
     *
     * @return WBSManager
     */
    public WbsManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(ResultSet resultSet,
            MessageContext messageContext) throws RPMException, SQLException
    {
        int typeId = resultSet.getInt(ID_TYPE_ID);
        return containerMap.createContainer(typeId);
    }

    private boolean isGenericProject(RPMObject rpmObject)
    {
        return rpmObject instanceof GenericProject;
    }

    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {
        if (rpmObject == null)
        {
            return;
        }

        // Set default field values for fields common to all WBS containers
        if (((WorkElement) rpmObject).getAssetCode() == null)
        {
            AssetFinancialCategory category = new AssetFinancialCategory();
            category.setID(DEFAULT_ASSET_CODE);
            category.setValue(DEFAULT_ASSET_CODE);

            ((WorkElement) rpmObject).setAssetCode(category);
        }
        if (((WorkElement) rpmObject).getEstimatedStartDate() == null)
        {
            ((WorkElement) rpmObject)
                .setEstimatedStartDate(DateUtil.getCalendarInstance());
        }
        if (((WorkElement) rpmObject).getPriority() == null)
        {
            ((WorkElement) rpmObject).setPriority(new Integer(500));
        }

        // Set default values specific to a Task
        if (rpmObject instanceof Task)
        {
            Task task = (Task) rpmObject;
            
            if (task.getPercentageDurationCompleted() == null)
            {
                task.setPercentageDurationCompleted(new Double(
                    0.00));
            }
        }

        // Set default values specific to GenericProjects
        if (isGenericProject(rpmObject))
        {
            // Set default field values
            if (((GenericProject) rpmObject).getAssignmentType() == null)
            {
                ((GenericProject) rpmObject)
                    .setAssignmentType(AssignmentType.Work);
            }
            if (((GenericProject) rpmObject).getCalendar() == null)
            {
                RPMCalendar calendar = new RPMCalendar();
                calendar.setID(DEFAULT_CALENDAR_ID);
                ((GenericProject) rpmObject).setCalendar(calendar);
            }
            if (((GenericProject) rpmObject).getDefaultProject() == null)
            {
                ((GenericProject) rpmObject).setDefaultProject(new Boolean(
                    false));
            }
            if (((GenericProject) rpmObject).getProbabilityPercent() == null)
            {
                ((GenericProject) rpmObject).setProbabilityPercent(new Integer(
                    100));
            }
        }
        
      if (((WorkElement) rpmObject).getPublished() == null)
      {
          ((WorkElement) rpmObject)
              .setPublished(ProjectPublishedType.None);
      }

        if (rpmObject instanceof SimpleNode)
        {
            SimpleNode node = (SimpleNode) rpmObject;
            
            if (node.getConstraintType() == null)
            {
                node.setConstraintType(ConstraintType.AsSoonAsPossible);
            }
            if (node.getTaskType() == null)
            {
                node.setTaskType(TaskType.DefaultTask);
            }
            if (node.getEffortDriven() == null)
            {
                node.setEffortDriven(new Boolean(false));
            }
            if (node.getMandatory() == null)
            {
                node.setMandatory(new Boolean(false));
            }
        }

        rpmObject.setID(SP_I_WBS(rpmObject, messageContext));

        // the call to update estimated dates must occur here in order for
        // default values to be set; applies to all WorkElement containers
        SP_U_WBS_START_FINISH_DATES(rpmObject, messageContext);

        if (isGenericProject(rpmObject))
        {
            SP_U_WBS_REQUEST(rpmObject, messageContext);
            SP_U_DESCRIPTION(rpmObject, messageContext);
            SP_U_WBS_METHOD(rpmObject, messageContext);
            // needed only if created under portfolio
            if (((GenericProject) rpmObject).getPortfolio() != null
                    || (rpmObject.getParent() != null && rpmObject.getParent() instanceof Portfolio))
            {
                SP_M_WBS(rpmObject, messageContext);
            }

            //reloading the currency, since this object is mandatory and is not reloaded
            //in case the object was created with a ReloadType.SavedResult
            Currency currency = loadCurrency(rpmObject, messageContext, false);
            ((GenericProject) rpmObject).setCurrency(currency);

        }
    }

    /**
     * @param rpmObject
     * @return true if the param obj is of type Portfolio.
     */
    private boolean isPortfolio(RPMObject rpmObject)
    {
        return rpmObject.retrieveTypeID().intValue() == Portfolio.TYPE_ID;
    }

    public ProjectPublishedType decodeProjectPublishedType(int type)
            throws RPMException
    {
        switch (type)
        {
        case 0:
            return ProjectPublishedType.None;
        case 1:
            return ProjectPublishedType.ToTeam;
        case 2:
            return ProjectPublishedType.ToTemplate;
        case 3:
            return ProjectPublishedType.ToPartialTemplate;

        default:
            String[] errorParams =
            {
                    StringUtil.getShortClassName(getClass().getName()),
                    "of field published ",
                    ProjectPublishedType._None + " or "
                            + ProjectPublishedType._ToTeam + " or"
                            + ProjectPublishedType._ToTemplate + " or "
                            + ProjectPublishedType._ToPartialTemplate
            };
            throw new RPMException(ErrorCodes.INVALID_VALUE, errorParams,
                getClass().getName(), NAME_PUBLISHED);
        }
    }

    private String encodePublishedType(ProjectPublishedType publishedType)
            throws RPMException
    {
        if (ProjectPublishedType.None.equals(publishedType))
        {
            return "0";
        }
        else if (ProjectPublishedType.ToTeam.equals(publishedType))
        {
            return "1";
        }
        else if (ProjectPublishedType.ToTemplate.equals(publishedType))
        {
            return "2";
        }
        else if (ProjectPublishedType.ToPartialTemplate.equals(publishedType))
        {
            return "3";
        }

        else
        {
            String[] errorParams =
            {
                     StringUtil.getShortClassName(getClass().getName()),
                    "of field published ",
                    ProjectPublishedType._None + " or "
                            + ProjectPublishedType._ToTeam + " or"
                            + ProjectPublishedType._ToTemplate + " or "
                            + ProjectPublishedType._ToPartialTemplate
            };
            throw new RPMException(ErrorCodes.INVALID_VALUE, errorParams,
                getClass().getName(), NAME_PUBLISHED);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#decodeRow(java.sql.ResultSet,
     *      com.ibm.rpm.framework.RPMObject, boolean)
     */
    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        WorkElement workElement = (WorkElement) rpmObject;
        workElement.setID(rs.getString(ID_ELEMENT_ID));
        String name = rs.getString(ID_ELEMENT_NAME);
        if (name != null)
        {
            name = name.trim();
        }

        int mins = rs.getInt(ID_MINUTE_PER_DAY);

        if (isDelta)
        {
            workElement.deltaName(name);
            workElement.deltaMinutesPerDay(new Integer(mins));
            workElement.deltaTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
            workElement.deltaEstimatedStartDate(Manager.getCalendar(rs,
                ID_START_FINISH_DATE));
            workElement.deltaEstimatedFinishDate(Manager.getCalendar(rs,
                ID_FINISH_START_DATE));
            workElement
                .deltaLastModifiedDate(Manager.getTimestamp(rs, ID_REC_DATETIME));
            workElement.deltaReferenceNumber(rs.getString(ID_REF_CODE));
            workElement.deltaWbsSortingRank(rs.getString(ID_RANK));
            workElement.deltaTemplateSortingRank(rs.getString(ID_T_RANK));
            workElement.deltaPriority(new Integer(rs.getInt(ID_PRIORITY)));
            workElement.deltaReferenceID(new Long(rs.getLong(ID_REFERENCE_NUMBER)));
            workElement.deltaClosedByDate(Manager.getCalendar(rs, ID_CLOSED_DATETIME));
            workElement.deltaClosedByName(rs.getString(ID_CLOSED_BY_NAME));
            
            if (workElement instanceof GenericProject)
            {
                GenericProject genericProject = (GenericProject) workElement;

                AssignmentType assignmentType = decodeAssignmentType(rs
                    .getInt(ID_EFFORT_DRIVEN));
                genericProject.deltaAssignmentType(assignmentType);

                genericProject.deltaProbabilityPercent(new Integer(rs
                    .getInt(ID_PROBABILITY)));
            }
            
            ProjectPublishedType pType = decodeProjectPublishedType(rs
                .getInt(ID_PUBLISHED));
            workElement.deltaPublished(pType);
            

            workElement.deltaExternalID(rs.getString(ID_EXTERNAL_ID));
            if (workElement instanceof SimpleNode)
            {
                SimpleNode node = (SimpleNode) workElement;
                node.deltaConstraintType(getConstraintType(rs
                    .getInt(ID_CONSTRAINT_TYPE)));
                node.deltaConstraintDate(Manager.getTimestamp(rs,
                    ID_CONSTRAINT_DT));
                node.deltaEffortDriven(decodeEffortDriven(rs.getInt(ID_TASK_TYPE)));
                node.deltaTaskType(decodeTaskType(rs.getInt(ID_TASK_TYPE)));
                node.deltaMandatory(Manager.getBoolean(rs, ID_CONFIG_FLAG));
            }
            if (workElement instanceof Task)
            {
                Task task = (Task) workElement;
                Double percentageDurationCompleted = new Double(rs.getDouble(ID_MILESTONE_PERCENT));
                task.deltaPercentageDurationCompleted(decodePercentageDurationCompleted(percentageDurationCompleted));
            }

        }
        else
        {
            workElement.setName(name);
            workElement.setMinutesPerDay(new Integer(mins));
            workElement.assignTypeID(new Integer(rs.getInt(ID_TYPE_ID)));
            workElement.setEstimatedStartDate(Manager.getCalendar(rs,
                ID_START_FINISH_DATE));
            workElement.setEstimatedFinishDate(Manager.getCalendar(rs,
                ID_FINISH_START_DATE));
            workElement.setLastModifiedDate(Manager.getTimestamp(rs, ID_REC_DATETIME));
            workElement.setReferenceNumber(rs.getString(ID_REF_CODE));
            workElement.setWbsSortingRank(rs.getString(ID_RANK));
            workElement.setTemplateSortingRank(rs.getString(ID_T_RANK));
            workElement.setPriority(new Integer(rs.getInt(ID_PRIORITY)));
            workElement.setReferenceID(new Long(rs.getLong(ID_REFERENCE_NUMBER)));
            workElement.setClosedByDate(Manager.getCalendar(rs, ID_CLOSED_DATETIME));
            workElement.setClosedByName(rs.getString(ID_CLOSED_BY_NAME));            
            
            if (workElement instanceof GenericProject)
            {
                GenericProject genericProject = (GenericProject) workElement;

                AssignmentType assignmentType = decodeAssignmentType(rs
                    .getInt(ID_EFFORT_DRIVEN));
                genericProject.setAssignmentType(assignmentType);

                genericProject.setProbabilityPercent(new Integer(rs
                    .getInt(ID_PROBABILITY)));
            }
            
            ProjectPublishedType pType = decodeProjectPublishedType(rs
                .getInt(ID_PUBLISHED));
            workElement.setPublished(pType);
            
            workElement.setExternalID(rs.getString(ID_EXTERNAL_ID));

            if (workElement instanceof SimpleNode)
            {
                SimpleNode node = (SimpleNode) workElement;
                
                node.setConstraintType(getConstraintType(rs
                    .getInt(ID_CONSTRAINT_TYPE)));
                node.setConstraintDate(Manager
                    .getTimestamp(rs, ID_CONSTRAINT_DT));
                
                node.setTaskType(decodeTaskType(rs.getInt(ID_TASK_TYPE)));
                node.setEffortDriven(decodeEffortDriven(rs.getInt(ID_TASK_TYPE)));
                node.setMandatory(Manager.getBoolean(rs, ID_CONFIG_FLAG));
            }
            if (workElement instanceof Task)
            {
                Task task = (Task) workElement;
                Double percentageDurationCompleted = new Double(rs.getDouble(ID_MILESTONE_PERCENT));
                task.setPercentageDurationCompleted(decodePercentageDurationCompleted(percentageDurationCompleted));
            }

        }

        //TODO include other attr here
        return rpmObject;
    }
    
    public Double decodePercentageDurationCompleted(Double percentageDurationCompleted) throws SQLException
    {
        if (percentageDurationCompleted != null && percentageDurationCompleted.doubleValue()!= 0.0)
        {
            // have to divide %DC by 100 for proper display
            percentageDurationCompleted = new Double(percentageDurationCompleted
                .doubleValue() / 100);
        }
        return percentageDurationCompleted;
    }
    
    private TaskType decodeTaskType(int type)
    {
        // can't use the EnumeratedTypeInitializer for TaskType b/c there are 
        // 2 values (1 and 11) that map to FixedDuration depending on the value of EFFORT_DRIVEN
        return (type == 10 ? TaskType.DefaultTask : TaskType.FixedDuration);
    }
    
    private Boolean decodeEffortDriven(int type)
    {
        // if the task type is 1 or 10, effort driven is false
        // if the task type is 11, effort driven is true
        // the EFFORT_DRIVEN database column is not used
        return (type == 11 ? Boolean.TRUE : Boolean.FALSE);
    }
    
    private Boolean decodeDefaultProjectFlag(GenericProject genericProject,
            MessageContext messageContext) throws RPMException, SQLException
    {
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark("element_id");

        Object[] arguments = new Object[]
        {
            "SC_DEFAULT_PROJECT_IN_SYSTEM___ "
        };
        
        String defaultGenericProjectId = (String) ManagerUtil
            .selectColumnValue(String.class, null, null, "element_name",
                "TMT_SETTINGS", condition, arguments, messageContext);

        if (defaultGenericProjectId != null
                && defaultGenericProjectId.trim()
                    .equals(genericProject.getID()))
        {
            return new Boolean(true);
        }

        return new Boolean(false);
    }

    private AssignmentType decodeAssignmentType(int value)
    {
        return (value == 1) ? AssignmentType.Duration : AssignmentType.Work;
    }

    private Integer encodeAssignmentType(AssignmentType assignmentType)
    {
        if (assignmentType != null
                && assignmentType.equals(AssignmentType.Duration))
        {
            return new Integer(1);
        }
        return new Integer(0);
    }

    private ConstraintType getConstraintType(int type)
    {
        switch (type)
        {
        case 0:
            return ConstraintType.AsSoonAsPossible;
        case 1:
            return ConstraintType.AsLateAsPossible;
        case 2:
            return ConstraintType.FinishNoEarlierThan;
        case 3:
            return ConstraintType.FinishNoLaterThan;
        case 4:
            return ConstraintType.MustFinishOn;
        case 5:
            return ConstraintType.MustStartOn;
        case 6:
            return ConstraintType.StartNoEarlierThan;
        case 7:
            return ConstraintType.StartNoLaterThan;
        default:
            return ConstraintType.AsSoonAsPossible;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.framework.RPMObjectManager#delete(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.MessageContext)
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (rpmObject != null)
        {
            SP_D_WBS(rpmObject, messageContext);
        }
    }

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager instanceof WbsManager)
        {
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_PARENT_ID, NAME_ELEMENT_ID,
                TABLE_NAME, NAME_ELEMENT_ID + "=?");
            joinCondition.appendCondition(extraWhere);
        }
        else if (manager instanceof PortfoliosManager)
        {
            // For loading: Portfolio.projects
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendSubSelect(NAME_ELEMENT_ID,
                PortfoliosManager.NAME_ELEMENT_ID,
                PortfoliosManager.TABLE_NAME, PortfoliosManager.NAME_PARENT_ID
                        + "=?");
            joinCondition.appendCondition(extraWhere); // Unused.
        }
        else if (manager instanceof WbsModuleManager)
        {
            // For loading: WbsModule.genericProjects
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendEqual(NAME_LEVEL_ID, 0);
            joinCondition.appendIn(NAME_TYPE_ID, containerMap
                .getTypeIds(GenericProject.class));
            joinCondition.appendCondition(extraWhere);
        }
        else if (manager instanceof DocumentManager)
        {
            // For loading: Document.externalParent
            joinCondition = new JoinCondition(TABLE_NAME);
            joinCondition.appendCondition(extraWhere);
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.FOREIGN_KEY);
        }

        return joinCondition;
    }

    public JoinCondition getJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;

        if (manager == null)
        {
            joinCondition = new JoinCondition(TABLE_NAME,
                getFinalCondition(container));
        }
        else if (manager instanceof WbsManager)
        {
            // For XPath like: /Project[<condition>]/workElements
            joinCondition = JoinCondition.createSubSelect(NAME_PARENT_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
        }
        else if (manager instanceof PortfoliosManager)
        {
            // For XPath like: /GenericProject[<condition>]/portfolio
            // Equivalent to (same code!): /Portfolio[projects[<condition>]]
            joinCondition = JoinCondition.createSubSelect(
                PortfoliosManager.NAME_ELEMENT_ID,
                PortfoliosManager.NAME_PARENT_ID, PortfoliosManager.TABLE_NAME,
                null);
            joinCondition.appendWhere();
            joinCondition.appendSubSelect(PortfoliosManager.NAME_ELEMENT_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
        }
        else if (manager instanceof AttributeAssignmentManager)
        {
            if (ContextUtil.WBS_CONTEXT.equalsIgnoreCase(contextName))
            {
                joinCondition = JoinCondition.createSubSelect(
                    AttributeManager.NAME_ELEMENT_ID,
                    AttributeAssignmentManager.NAME_PARENT_ID,
                    AttributeAssignmentManager
                        .getTableNameFromContext(contextName), null);
                joinCondition.appendWhere();
                joinCondition.appendSubSelect(
                    AttributeAssignmentManager.NAME_ELEMENT_ID,
                    NAME_ELEMENT_ID, getTableName(contextName),
                    getFinalCondition(container));
            }
        }
        else if (manager instanceof FinancialManager)
        {
            if (ContextUtil.WBS_CONTEXT.equalsIgnoreCase(contextName))
            {
                joinCondition = JoinCondition.createSubSelect(
                    FinancialManager.NAME_ELEMENT_ID, NAME_ELEMENT_ID,
                    TABLE_NAME, getFinalCondition(container));
            }
        }
        else if (manager instanceof AssetCrossChargeManager)
        {
            // For XPath like: /Proposal[<condition>]/CrossChargeFinancials
            joinCondition = JoinCondition.createSubSelect(
                AssetCrossChargeManager.NAME_PROJECT_CLIENT_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
        }
        else if (manager instanceof CustomFieldAssignmentManager)
        {
            if (ContextUtil.WBS_CONTEXT.equalsIgnoreCase(contextName))
            {
                // For XPath like: WorkElement[<condition>]/AssignedCustomField
                String foreignFieldName = CustomFieldAssignmentUtil
                    .getTableName(ContextUtil.WBS_CONTEXT)
                        + '.' + CustomFieldAssignmentManager.NAME_FOREIGN_ID;
                joinCondition = JoinCondition.createSubSelect(foreignFieldName,
                    NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
            }
        }
        else if (manager instanceof DatafieldManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                DatafieldManager.NAME_ELEMENT_ID, NAME_CURRENCY_ID, TABLE_NAME,
                getFinalCondition(container));
        }
        else if (manager instanceof RPMCalendarManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                RPMCalendarManager.NAME_ELEMENT_ID, NAME_CALENDAR_ID,
                TABLE_NAME, getFinalCondition(container));
        }
        else if (manager instanceof DatafieldManager)
        {
            String columnName;
            if (container.getShortTypeName().equalsIgnoreCase(
                StringUtil.getShortClassName(DatafieldState.class.getName())))
                columnName = NAME_STAGE_ID;
            else
                // assetCode
                columnName = NAME_FINANCIAL_CATEGORY;
            joinCondition = JoinCondition.createSubSelect(
                DatafieldManager.NAME_ELEMENT_ID, columnName, TABLE_NAME,
                getFinalCondition(container));
        }
        else if (manager instanceof ResourceTaskAssignmentManager)
        {
            if (ContextUtil.WBS_CONTEXT.equalsIgnoreCase(contextName)
                    || ContextUtil.RESOURCE_CONTEXT
                        .equalsIgnoreCase(contextName))
            {
                joinCondition = JoinCondition.createSubSelect(
                    ResourceTaskAssignmentManager.NAME_ELEMENT_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
            }
        }
        else if (manager instanceof ProjectReconciliationStatusManager)
        {
            joinCondition = JoinCondition.createSubSelect(
                ProjectReconciliationStatusManager.NAME_PROJECT_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
        }
        else if (manager instanceof TimeCodeAssignmentManager){
            
            joinCondition = JoinCondition.createSubSelect(
                TimeCodeAssignmentManager.NAME_ELEMENT_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
        }
        else if (manager instanceof DocumentManager){
            if (ContextUtil.getInstance().isWbsContext(contextName))
            {
            	joinCondition = new JoinCondition();
            	joinCondition.setCondition(DocumentManager.NAME_LEVEL_ID
                        + " = 2 AND "
                        + JoinCondition.createSubSelect(
                            DocumentManager.NAME_PROJECT_ID, NAME_PROJECT_ID,
                            TABLE_NAME, getFinalCondition(container))
                            .toString());
            }
        }        
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.JOIN);
        }

        return joinCondition;
    }

    public JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            XPathContainer container, String contextName) throws RPMException
    {
        JoinCondition joinCondition = null;
        if (manager instanceof WbsManager)
        {
            // For XPath like: /Proposal[Task[<condition>]]
            if ("children".equals(container.getPreviousFieldName()))
            {
                joinCondition = JoinCondition.createSubSelect(NAME_ELEMENT_ID,
                    NAME_PARENT_ID, TABLE_NAME, getFinalCondition(container));
            }
            //containingProject
            else
            {
                SqlBuffer finalCondition = new SqlBuffer(getFilter());
                //can only map to a GenericProject
                Collection typeIds = containerMap
                    .getTypeIds(GenericProject.class);
                finalCondition.appendIn(NAME_TYPE_ID, typeIds);

                joinCondition = JoinCondition.createSubSelect(NAME_PROJECT_ID,
                    NAME_ELEMENT_ID, TABLE_NAME, finalCondition.toString());
            }
        }
        else if (manager instanceof AssetCrossChargeManager)
        {
            // For XPath like: /CrossChargeFinancials[<condition>]/Proposal
            joinCondition = JoinCondition.createSubSelect(
                AssetCrossChargeManager.NAME_PROJECT_CLIENT_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
        }
        else if (manager instanceof PortfoliosManager)
        {
            // For XPath like: /Portfolio[projects[<condition>]]
            // Equivalent to (same code!):
            // /GenericProject[<condition>]/portfolio
            joinCondition = JoinCondition.createSubSelect(
                PortfoliosManager.NAME_ELEMENT_ID,
                PortfoliosManager.NAME_PARENT_ID, PortfoliosManager.TABLE_NAME,
                null);
            joinCondition.appendWhere();
            joinCondition.appendSubSelect(PortfoliosManager.NAME_ELEMENT_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFinalCondition(container));
        }
        else if (manager instanceof SummaryTimesheetManager)
        {
            if ("project".equals(container.getPreviousFieldName()))
            {

                joinCondition = new JoinCondition();
                joinCondition.appendSubSelect(
                    SummaryTimesheetManager.NAME_PROJECT_ID, NAME_ELEMENT_ID,
                    TABLE_NAME, getFilter());
                Collection typeIds = containerMap
                    .getTypeIds(GenericProject.class);
                joinCondition.appendIn(NAME_TYPE_ID, typeIds);
            }
            else
            {
                SqlBuffer subCondition = new SqlBuffer();
                subCondition.appendSubSelect(
                    TaskAssignmentManager.NAME_TASK_ID, NAME_ELEMENT_ID,
                    TABLE_NAME, getFilter());

                joinCondition = JoinCondition.createSubSelect(
                    SummaryTimesheetManager.NAME_TSK_ASSIGNMENT_ID,
                    TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
                    TaskAssignmentManager.TABLE_NAME, subCondition.toString());
            }

        }
        else if (manager instanceof StepManager)
        {
            if ("project".equals(container.getPreviousFieldName()))
            {

                SqlBuffer subCondition = new SqlBuffer();
                subCondition.appendSubSelect(
                    TaskAssignmentManager.NAME_PROJECT_ID, NAME_ELEMENT_ID,
                    TABLE_NAME, getFilter());

                joinCondition = JoinCondition.createSubSelect(
                    StepManager.NAME_TSK_ASSIGNMENT_ID,
                    TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
                    TaskAssignmentManager.TABLE_NAME, subCondition.toString());
            }
            else
            {
                SqlBuffer subCondition = new SqlBuffer();
                subCondition.appendSubSelect(
                    TaskAssignmentManager.NAME_TASK_ID, NAME_ELEMENT_ID,
                    TABLE_NAME, getFilter());

                joinCondition = JoinCondition.createSubSelect(
                    StepManager.NAME_TSK_ASSIGNMENT_ID,
                    TaskAssignmentManager.NAME_TSK_ASSIGNMENT_ID,
                    TaskAssignmentManager.TABLE_NAME, subCondition.toString());
            }
        }
        else if (manager instanceof TimesheetApprovalStatusManager)
        {
            joinCondition = new JoinCondition();
            joinCondition.appendSubSelect(
                TimesheetApprovalStatusManager.NAME_ELEMENT_ID,
                NAME_ELEMENT_ID, TABLE_NAME, getFilter());

        }
        else if (manager instanceof PublishedScorecardManager)
        {
            // For XPath like: /ResourceAssignedScorecard/Project
            joinCondition = JoinCondition.createSubSelect(
                PublishedScorecardManager.NAME_PROJECT_ID,
                NAME_PROJECT_ID, TABLE_NAME, getFinalCondition(container));
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                ExceptionUtil.SUB_TABLE_JOIN);
        }

        return joinCondition;
    }

    public String getFilter()
    {
        StringBuffer filter = new StringBuffer();

        filter.append(NAME_ELEMENT_ID);
        filter.append(" IS NOT NULL ");

        filter.append(" AND ");

        filter.append(NAME_REC_STATUS);
        filter.append(" != 'D'");

        // make sure the container being loaded is of a type_id compatible 
        // with WBS... some other containers are stored in TMT_WBS
        Collection typeIds = containerMap.getTypeIds(WorkElement.class);
        SqlBuffer buffer = new SqlBuffer();
        buffer.appendIn(NAME_TYPE_ID, typeIds);

        filter.append(" AND ");
        filter.append(buffer.toString());

        return filter.toString();
    }

    private String getFinalCondition(XPathContainer container)
            throws RPMException
    {
        // Add the basic filtering condition.
        SqlBuffer finalCondition = new SqlBuffer(getFilter());

        // Add the type ID condition, if required.
        Collection typeIds = containerMap.getTypeIds(container.getType());
        finalCondition.appendIn(NAME_TYPE_ID, typeIds);

        return finalCondition.toString();
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.framework.AbstractRPMObjectManager#updateObject(com.ibm.rpm.framework.RPMObject,
     *      com.ibm.rpm.framework.RPMObjectScope,
     *      com.ibm.rpm.framework.MessageContext)
     */
    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        WorkElementScope scope = (WorkElementScope) rpmScope;
        WorkElement workelement = (WorkElement) rpmObject;

        boolean bGenericProject = isGenericProject(workelement);
        boolean bDatesModified = workelement.testEstimatedStartDateModified()
                || workelement.testEstimatedFinishDateModified();

        if (bDatesModified)
        {
            SP_U_WBS_START_FINISH_DATES(workelement, messageContext);
        }

        boolean bDesc = workelement.testExternalIDModified()
                || workelement.testPriorityModified()
                || workelement.testReferenceNumberModified();

        bDesc = bDesc
                || (bGenericProject && scope != null && scope.isOpportunity() && ((GenericProject) workelement)
                    .testProbabilityPercentModified());

        if (bDesc)
        {
            SP_U_DESCRIPTION(workelement, messageContext);
        }

        boolean bMethodModified = bGenericProject
                && ((GenericProject) workelement).testAssignmentTypeModified();

        if (bMethodModified)
        {
            SP_U_WBS_METHOD(workelement, messageContext);
        }

        if (workelement.testNameModified())
        {
            SP_U_WBS_NAME(workelement, messageContext);
        }

        if(workelement.testClosedByNameModified() || workelement.testClosedByDateModified())
        {
        	SP_U_CLOSE_WBS_BY_NAME(workelement, messageContext);
        }
        
        if (bGenericProject)
        {
            GenericProject genericProject = (GenericProject) workelement;
            if (genericProject.testPublishedModified())
                SP_U_PUBLISHED(genericProject, messageContext);
        }

        if (rpmObject instanceof SimpleNode)
        {
            SimpleNode node = (SimpleNode) rpmObject;
            
            if (node.testConstraintDateModified()
                    || node.testConstraintTypeModified()
                    || node.testTaskTypeModified()
                    || node.testEffortDrivenModified())
            {
                SP_U_CONSTRAINT(node, messageContext);
            }
            
            if (node.testMandatoryModified())
            {
                SP_U_ELEMENT_CONFIG(node, messageContext);
            }
        }

        //default project
        if (bGenericProject
                && ((GenericProject) rpmObject).testDefaultProjectModified()
                && ((GenericProject) rpmObject).getDefaultProject() != null
                && ((GenericProject) rpmObject).getDefaultProject()
                    .booleanValue())
        {
            SP_U_DEFAULT_PROJECT((GenericProject) rpmObject, messageContext);
        }
        if (scope != null)
        {
            if ((scope.isAsset() && workelement.testAssetModified())
                    || (scope.isAssetCode() && workelement
                        .testAssetCodeModified()))
            {
                SP_U_WBS_ASSET(workelement, messageContext);
            }

            if (scope.isState() && workelement.testStateModified())
            {
                SP_U_STAGE_ID(rpmObject, messageContext);
            }

            // ResourceTaskAssignments has to be updated before ScheduleDates
            if (scope.getResourceTaskAssignments() != null)
            {
                updateChildren(rpmObject, ((WorkElement) rpmObject)
                    .getResourceTaskAssignments(), scope
                    .getResourceTaskAssignments(), messageContext, null);
            }

            //Schedule Dates.
            if (scope.isScheduleDates())
            {
                if (workelement instanceof Task
                        || workelement instanceof Milestone)
                {
                    saveScheduleDates(workelement, scope, messageContext);
                }
            }

            //Rtf Values
            if (scope.isRtfAssignments())
            {
                saveRtfAssignments(workelement, rpmScope, messageContext);
            }

            //Attribute Assignments
            if (scope.isAttributeAssignments())
            {
                saveAttributeAssignments(workelement, messageContext);
            }

            if (scope.isRelatedAttributeAssignments() && bGenericProject)
            {
                saveChildrenAttributeAssignments((GenericProject) rpmObject,
                    messageContext);
            }

            //Element Dependencies
            if (scope.getDependencies() != null)
            {
                saveDependencies(workelement, scope.getDependencies(),
                    messageContext);
            }

            // Save Favorite Activities
            if (scope.getDefaultSteps() != null)
            {
                saveFavoriteActivities(workelement, messageContext);
            }

            if (workelement instanceof AggregateNode)
            {
                AggregateNode node = (AggregateNode) workelement;
                if (scope.getChildren() != null)
                {
                    node.setChildren((WorkElement[]) updateChildren(
                        workelement, node.getChildren(), scope.getChildren(),
                        messageContext, WorkElement.class));
                }

                if (scope.getWorksheetFinancials() != null)
                {
                    updateFinancials(((AggregateNode) workelement)
                        .getWorksheetFinancials(), WorksheetFinancial.class,
                        scope.getWorksheetFinancials(), messageContext);
                }
            }

            if (bGenericProject)
            {
                GenericProject genProject = (GenericProject) workelement;

                if (scope.isOpportunity()
                        && genProject.testOpportunityModified())
                {
                    OpportunityManager projMgr = (OpportunityManager) getManagerInstance(Opportunity.class
                        .getName());

                    projMgr.internalSave(((GenericProject) rpmObject)
                        .getOpportunity(), null, messageContext);
                }

                genProject
                    .setOrganizationalAssignments((ProjectOrganizationalAssignment[]) updateChildren(
                        genProject, genProject.getOrganizationalAssignments(),
                        scope.isOrganizationalAssignments(), messageContext,
                        null));

                //TODO remove comments to implement ProjectReconciliation
/*                ProjectReconciliationStatus state = genProject
                    .getProjectReconciliationStatus();
                if (state != null)
                {
                    state.setID(genProject.getID());
                    genProject
                        .setProjectReconciliationStatus((ProjectReconciliationStatus) updateChild(
                            genProject, genProject
                                .getProjectReconciliationStatus(), scope
                                .isProjectReconciliationStatus(),
                            messageContext));
                }

                genProject
                    .setProjectReconciliations((ProjectReconciliation[]) updateChildren(
                        genProject, genProject.getProjectReconciliations(),
                        scope.isProjectReconciliations(), messageContext, null));*/
            }

            if (scope.getWbsFinancials() != null)
            {
                updateFinancials(workelement.getWbsFinancials(),
                    WbsFinancial.class, scope.getWbsFinancials(),
                    messageContext);
            }

            // Document folders
            if (scope.getDocumentFolder() != null)
            {
                saveDocumentFolder(workelement, scope, messageContext);
            }

            //Custom fields
            propagateContextName(workelement, workelement
                .getCustomFieldAssignments());
            workelement
                .setCustomFieldAssignments((CustomFieldAssignment[]) updateChildren(
                    workelement, workelement.getCustomFieldAssignments(), scope
                        .isCustomFieldAssignments(), messageContext, null));

            if (scope.isTimeCodeAssignments())
            {
                saveTimeCodeAssignments(workelement, messageContext);
            }

            if (scope.getAssignedScorecards() != null)
            {
                saveAssignedScorecards((GenericProject) rpmObject, scope,
                    messageContext);
            }

            if (scope.getScopeScorecards() != null)
            {
                saveScopeScorecards((GenericProject) rpmObject, scope,
                    messageContext);
            }

            if (scope.isTimeCodes())
            {
                if (workelement.testTimeCode1Modified()
                        || workelement.testTimeCode2Modified())
                {
                    SP_U_WBS_TIMECODES(rpmObject, messageContext);
                }
            }
        }

        // update %dc after schedule dates
        if (rpmObject instanceof Task)
        {
            if (((Task) rpmObject).testPercentageDurationCompletedModified())
            {
                SP_U_ACTUAL((Task) rpmObject, messageContext);
            }
        }

        return rpmObject;
    }

    private WorkElement saveTimeCodeAssignments(WorkElement workElement,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        workElement
            .setTimeCodeAssignments((TimeCodeAssignment[]) updateChildren(
                workElement, workElement.getTimeCodeAssignments(), true,
                messageContext, TimeCodeAssignment.class));

        return workElement;
    }

    private void saveScheduleDates(WorkElement object, WorkElementScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {

        RPMObjectManager scheduleDateManager = getManagerInstance(WbsScheduleDate.class
            .getName());

        //note: expectedDate, baselineDate, initialBaselineDate and
        //previousBaselineDates are read-only

        //DO NOT CHANGE THE ORDER IN WHICH THE SCHEDULE DATES ARE SAVED

        //proposedDate
        WbsScheduleDate proposedDate = object.getProposedDate();
        if (proposedDate != null)
        {
        	//this default is set db on create
            if (proposedDate.getStartDate() == null)
            {
                proposedDate.setStartDate(object.getEstimatedStartDate());
            }
            proposedDate.setContextName(WbsScheduleDate.PROPOSED);
            proposedDate.setParent(object);
            proposedDate = (WbsScheduleDate) scheduleDateManager.internalSave(proposedDate,
                null, messageContext);
        }
        //plannedDate
        WbsScheduleDate planDate = object.getPlanDate();
        if (planDate != null)
        {
            planDate.setContextName(WbsScheduleDate.PLAN);
            planDate.setParent(object);
            planDate = (WbsScheduleDate) scheduleDateManager.internalSave(planDate, null,
                messageContext);
        }

        //actualDate
        WbsScheduleDate actualDate = object.getActualDate();
        if (actualDate != null)
        {
            actualDate.setContextName(WbsScheduleDate.ACTUAL);
            actualDate.setParent(object);
            actualDate = (WbsScheduleDate) scheduleDateManager.internalSave(actualDate,
                null, messageContext);
        }

        //forecastDate
        WbsScheduleDate forecastdate = object.getForecastDate();
        if (forecastdate != null)
        {
            forecastdate.setContextName(WbsScheduleDate.FORECAST);
            forecastdate.setParent(object);
            forecastdate = (WbsScheduleDate) scheduleDateManager.internalSave(forecastdate,
                null, messageContext);
        }
    }

    private void updateFinancials(AbstractFinancial[] financials,
            Class container, FinancialScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (financials == null || financials.length == 0)
            return;

        // validating that Average is not set alone...
        // NB: Since the WorkelementCheckpoint is called before the Financial
        // checkPoint we cannot do this validation in the FinancialCheckPoint
        // because the decode related fields hasn't being call yet.
        FinancialCheckpoint checkpoint = (FinancialCheckpoint) ContainerToCheckpointMap
            .getCheckpoint(financials[0]);

        if (checkpoint!=null && !checkpoint.validateFinancials(messageContext, financials))
        {
            FinancialBusinessLogicUtil.printResult(financials,"BEFORE: processFinancialMatrixLines");
            FinancialBusinessLogicUtil.processFinancialMatrixLines(financials);

            FinancialBusinessLogicUtil.printResult(financials,"BEFORE:processFinancialMatrixColumns");
            FinancialBusinessLogicUtil
                .processFinancialMatrixColumns(financials);
            FinancialBusinessLogicUtil.printResult(financials,"AFTER:processFinancialMatrixColumns");

            FinancialManager manager = (FinancialManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(container.getName());

            for (int i = 0; i < financials.length; i++)
            {
                manager.internalSave(financials[i], scope, messageContext);
                FinancialBusinessLogicUtil.printResult(financials,"AFTER: manager.internalSave(financials[i]");

            }

            FinancialBusinessLogicUtil.printResult(financials,"AFTER ALL  manager.internalSave(financials[]");
            
            // If we had timephased we must re-calculate the worksheet matrix to
            // take into account that
            // the total timephased are going to roll back to the related
            // worksheet financials charge code

            if (scope != null
                    && (scope.isActualTimePhased() || scope
                        .isEstimateTimePhased()))
            {
                FinancialBusinessLogicUtil.saveUpdateTimephasedToBD(financials,
                    messageContext, getManagerInstance(financials[0].getClass()
                        .getName()));
                FinancialBusinessLogicUtil.printResult(financials,"AFTER:wbsManager.saveUpdateTimephasedToBD");
            }

        }

    }
    
 
    private void saveAssignedScorecards(GenericProject project,
            WorkElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        //NOTE: Only already assigned scorecards can be saved. A Scorecard can
        // only be assigned by using the Application.assignScorecard method of
        // the API.

        AssignedScorecard[] scoreCards = project.getAssignedScorecards();
        if (scoreCards != null)
        {
            for (int i = 0; i < scoreCards.length; ++i)
                scoreCards[i] = (AssignedScorecard) updateChild(project,
                    scoreCards[i], scope.getAssignedScorecards(),
                    messageContext);
            project.setAssignedScorecards(scoreCards);
        }
    }

    private void saveScopeScorecards(GenericProject project,
            WorkElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ScorecardManager manager = (ScorecardManager) getManagerInstance(Scorecard.class
            .getName());

        //Action scorecard
        if (project.getActionScorecard() != null)
        {
            manager.internalSave(project.getActionScorecard(), scope
                .getScopeScorecards(), messageContext);
        }

        //Defect scorecard
        if (project.getDefectScorecard() != null)
        {
            manager.internalSave(project.getDefectScorecard(), scope
                .getScopeScorecards(), messageContext);
        }

        //Issue scorecard
        if (project.getIssueScorecard() != null)
        {
            manager.internalSave(project.getIssueScorecard(), scope
                .getScopeScorecards(), messageContext);
        }

        //ServiceRequest scorecard
        if (project.getServiceRequestScorecard() != null)
        {
            manager.internalSave(project.getServiceRequestScorecard(), scope
                .getScopeScorecards(), messageContext);
        }

        //Risk scorecard
        if (project.getRiskScorecard() != null)
        {
            manager.internalSave(project.getRiskScorecard(), scope
                .getScopeScorecards(), messageContext);
        }

        //Requiremen scorecard
        if (project.getRequirementScorecard() != null)
        {
            manager.internalSave(project.getRequirementScorecard(), scope
                .getScopeScorecards(), messageContext);
        }

        //ChangeRequest scorecard
        if (project.getChangeRequestScorecard() != null)
        {
            manager.internalSave(project.getChangeRequestScorecard(), scope
                .getScopeScorecards(), messageContext);
        }

    }

    private WorkElement saveDependencies(WorkElement workelement,
            ElementDependencyScope scope, MessageContext messageContext)
            throws RPMException, ParseException, SQLException
    {
        ArrayList temp;

        if (workelement.getDependencies() != null)
        {
            temp = new ArrayList();

            ElementDependency[] children = workelement.getDependencies();

            DependenciesManager dependencyManager = (DependenciesManager) getManagerInstance(ElementDependency.class
                .getName());

            for (int i = 0; i < children.length; i++)
            {
                if (children[i] != null)
                {
                    dependencyManager.internalSave(children[i], scope,
                        messageContext);
                    temp.add(children[i]);
                }
            }

            if (temp.size() > 0)
            {
                children = new ElementDependency[temp.size()];
                temp.toArray(children);
            }

            workelement.setDependencies(children);
        }

        return workelement;
    }

    private void saveFavoriteActivities(WorkElement workElement,
            MessageContext messageContext) throws RPMException, ParseException,
            SQLException
    {
        if (!(workElement instanceof GenericProject))
            return;

        GenericProject project = (GenericProject) workElement;
        DefaultStep[] favoriteActivities = project.getDefaultSteps();
        ArrayList temp;

        if (favoriteActivities != null)
        {
            temp = new ArrayList();

            DefaultStepManager manager = (DefaultStepManager) getManagerInstance(DefaultStep.class
                .getName());

            for (int i = 0; i < favoriteActivities.length; i++)
            {
                if (favoriteActivities[i] != null)
                {
                    manager.internalSave(favoriteActivities[i], null,
                        messageContext);
                    temp.add(favoriteActivities[i]);
                }
            }

            if (temp.size() > 0)
            {
                favoriteActivities = new DefaultStep[temp.size()];
                temp.toArray(favoriteActivities);
            }

            project.setDefaultSteps(favoriteActivities);
        }
    }

    private void saveChildrenAttributeAssignments(
            GenericProject genericProject, MessageContext messageContext)
            throws RPMException, ParseException, SQLException
    {
        AttributeAssignmentManager attributeAssignmentManager = (AttributeAssignmentManager) getManagerInstance(AttributeAssignment.class
            .getName());

        //SCOPE related assignments
        AttributeAssignment[] actionAttributeAssignments = saveChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getActionAttributeAssignments(), new Action(), messageContext);

        genericProject
            .setActionAttributeAssignments(actionAttributeAssignments);

        AttributeAssignment[] changeRequestAttributeAssignments = saveChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getChangeRequestAttributeAssignments(), new ChangeRequest(),
            messageContext);

        genericProject
            .setChangeRequestAttributeAssignments(changeRequestAttributeAssignments);

        AttributeAssignment[] defectAttributeAssignments = saveChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getDefectAttributeAssignments(), new Defect(), messageContext);

        genericProject
            .setDefectAttributeAssignments(defectAttributeAssignments);

        AttributeAssignment[] issueAttributeAssignments = saveChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getIssueAttributeAssignments(), new Issue(), messageContext);

        genericProject.setIssueAttributeAssignments(issueAttributeAssignments);

        AttributeAssignment[] requirementAttributeAssignments = saveChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getRequirementAttributeAssignments(), new Requirement(),
            messageContext);

        genericProject
            .setRequirementAttributeAssignments(requirementAttributeAssignments);

        AttributeAssignment[] riskAttributeAssignments = saveChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getRiskAttributeAssignments(), new Risk(), messageContext);

        genericProject.setRiskAttributeAssignments(riskAttributeAssignments);

        AttributeAssignment[] serviceRequestAttributeAssignments = saveChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getServiceRequestAttributeAssignments(), new ServiceRequest(),
            messageContext);

        genericProject
            .setServiceRequestAttributeAssignments(serviceRequestAttributeAssignments);

        //WBS related assignments
        AttributeAssignment[] deliverableAttributeAssignments = saveChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getDeliverableAttributeAssignments(), new Deliverable(),
            messageContext);

        genericProject
            .setDeliverableAttributeAssignments(deliverableAttributeAssignments);

        AttributeAssignment[] taskAttributeAssignments = saveChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getTaskAttributeAssignments(), new Task(), messageContext);

        genericProject.setTaskAttributeAssignments(taskAttributeAssignments);

        //Document related assignments
        AttributeAssignment[] documentAttributeAssignments = saveChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getDocumentAttributeAssignments(), new Document(),
            messageContext);

        genericProject
            .setDocumentAttributeAssignments(documentAttributeAssignments);
    }

    private AttributeAssignment[] saveChildrenAttributeAssignments(
            AttributeAssignmentManager attributeAssignmentManager,
            GenericProject genericProject, AttributeAssignment[] children,
            RPMObject assignmentParent, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        assignmentParent.setID(genericProject.getID());

        ArrayList temp = new ArrayList();

        if (children != null)
        {
            for (int i = 0; i < children.length; i++)
            {
                if (children[i] != null)
                {
                    children[i].setParent(assignmentParent);

                    //Assign the parent type id to the children before
                    // reloading
                    //The typeId is mandatory when reloading the assignment
                    // when isDelta = true;
                    if (children[i].retrieveTypeID() == null)
                    {
                        children[i].assignTypeID(assignmentParent
                            .retrieveTypeID());
                    }

                    attributeAssignmentManager.internalSave(children[i], null,
                        messageContext);

                    //(re)setting the children parent
                    children[i].setParent(genericProject);

                    temp.add(children[i]);
                }
            }
        }

        AttributeAssignment[] newChildren = null;

        if (temp.size() > 0)
        {
            newChildren = new AttributeAssignment[temp.size()];
            temp.toArray(newChildren);
        }

        return newChildren;
    }

    private void saveAttributeAssignments(WorkElement workElement,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        ArrayList temp;

        if (workElement.getAttributeAssignments() != null)
        {
            temp = new ArrayList();

            AttributeAssignment[] children = workElement
                .getAttributeAssignments();

            AttributeAssignmentManager attributeAssignmentManager = (AttributeAssignmentManager) getManagerInstance(

            AttributeAssignment.class.getName());

            for (int i = 0; i < children.length; i++)
            {
                if (children[i] != null)
                {
                    children[i].setParent(workElement);

                    attributeAssignmentManager.internalSave(children[i], null,
                        messageContext);

                    temp.add(children[i]);
                }
            }

            AttributeAssignment[] newChildren = null;

            if (temp.size() > 0)
            {
                newChildren = new AttributeAssignment[temp.size()];
                temp.toArray(newChildren);
            }

            workElement.setAttributeAssignments(newChildren);
        }
    }

    private WorkElement saveRtfAssignments(WorkElement workelement,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        ArrayList savedChildrenList;

        WorkElementScope scope = (WorkElementScope) rpmScope;

        if (scope.isRtfAssignments() && workelement.getRtfAssignments() != null)
        {
            savedChildrenList = new ArrayList();

            RtfAssignment[] children = workelement.getRtfAssignments();

            RtfAssignmentManager rtfAssignmentManager = (RtfAssignmentManager) getManagerInstance(RtfAssignment.class
                .getName());

            RtfAssignment assignment = null;
            for (int i = 0; i < children.length; i++)
            {
                if (children[i] != null)
                {
                    assignment = children[i];
                    
                    assignment.setParent(workelement);
                    assignment = (RtfAssignment) rtfAssignmentManager.internalSave(assignment, null,
                        messageContext);
                    savedChildrenList.add(assignment);
                }
            }

            if (savedChildrenList.size() > 0)
            {
                children = (RtfAssignment[]) savedChildrenList.toArray(children);
            }

            workelement.setRtfAssignments(children);
        }

        return workelement;
    }

    //    private WorkElement saveScoreCardAssignments(GenericProject project,
    //          RPMObjectScope rpmScope, MessageContext messageContext)
    //          throws RPMException, SQLException,
    //          ParseException {
    //
    //      ArrayList temp;
    //      WorkElementScope scope = (WorkElementScope) rpmScope;
    //
    //      if (scope.isRtfAssignments()
    //              && project.getRtfAssignments() != null) {
    //
    //          temp = new ArrayList();
    //          ScorecardAssignment[] children = project.getScorecardAssignments();
    //
    //          ScorecardAssignmentManager scm =
    // (ScorecardAssignmentManager)getManagerInstance(ScorecardAssignment.class.getName());
    //
    //          for (int i = 0; i < children.length; i++) {
    //              if (children[i] != null) {
    //                  Boolean deleted = children[i].getDeleted();
    //                  if (deleted != null && deleted.booleanValue()) {
    //                      scm.delete(children[i],
    //                              messageContext);
    //                  } else {
    //                      children[i].setParent(project);
    //                      scm.internalSave(
    //                              children[i], null, messageContext);
    //                      temp.add(children[i]);
    //                  }
    //              }
    //          }
    //
    //          if (temp.size() > 0) {
    //              children = new ScorecardAssignment[temp.size()];
    //              temp.toArray(children);
    //          }
    //
    //          project.setScorecardAssignments(children);
    //      }
    //
    //      return project;
    //  }
    //TODO remove later
    //    private void saveChildren(WorkElement parent, WorkElementScope scope,
    //            WorkElement children[], MessageContext messageContext)
    //            throws RPMException, SQLException, ParseException
    //    {
    //        if (scope == null || children == null)
    //            return;
    //
    //        ArrayList temp = new ArrayList();
    //
    //        for (int i = 0; i < children.length; i++)
    //        {
    //            if (children[i] != null)
    //            {
    //                if (children[i].getDeleted().booleanValue())
    //                {
    //                    delete(children[i], messageContext);
    //                }
    //                else
    //                {
    //                    children[i].setParent(parent);
    //
    //                    internalSave(children[i], scope.getWorkElements(), messageContext);
    //
    //                    temp.add(children[i]);
    //                }
    //            }
    //        }
    //
    //        if (temp.size() > 0)
    //        {
    //            children = new WorkElement[temp.size()];
    //            temp.toArray(children);
    //        }
    //
    //        ((AggregateNode) parent).setWorkElements((WorkElement[]) children);
    //    }

    public void checkinObject(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException
    {
        try
        {
            SP_CHECK_IN(rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }

    public void undoCheckoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        try
        {
            SP_UNDO_CHECKOUT(rpmObject, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }

    public void checkoutObject(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        try
        {
            String workflowId = messageContext.getWorkflowId();
            if (workflowId == null)
                SP_CHECK_OUT(rpmObject, messageContext);
            else
                SP_CHECK_OUT_WKF(rpmObject, workflowId, messageContext);
        }
        catch (SQLException s)
        {
            ExceptionUtil.handleException(this, s);
        }
    }

    public boolean isCheckedOut(MessageContext context, RPMObject rpmObject)
            throws RPMException, SQLException
    {
        String status = ManagerUtil.retrieveRecStatus(context, rpmObject);

        if (status != null)
        {
            return status.equalsIgnoreCase("O");
        }
        return false;
    }

    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        WorkElement workElement = (WorkElement) rpmObject;
        
        if (rpmObject instanceof GenericProject)
        {
            //load defaultProjectFlag from tmt_settings table
            loadDefaultProjectFlag((GenericProject) rpmObject, messageContext,
                isDelta);
            
            // Load earnedValues calculation activation state
            boolean evState = WbsUtil.getEVCalculationActivationState(
                ((GenericProject) rpmObject), messageContext);

            if (isDelta)
            {
                ((GenericProject) rpmObject)
                    .deltaEarnedValuesCalculationState(new Boolean(evState));
            }
            else
            {
                ((GenericProject) rpmObject)
                    .setEarnedValuesCalculationState(new Boolean(evState));
            }
        }
        

        if (rpmScope != null)
        {
            // load parent
            if (rpmScope.getParent() != null)
            {
                loadParent(rpmObject, rpmScope, messageContext, relatedFields,
                    index, isDelta);
            }
            
            if(rpmScope instanceof WorkElementScope){
                
                WorkElementScope scope = (WorkElementScope) rpmScope;
           

                //Loading EarnValues if in scope...
                if(scope.isEarnedValues())
                {
                    loadEarnedValues(workElement,scope,messageContext,relatedFields,index,isDelta);
                }
                
                if (scope.isAssetCode())
                {
                    loadAssetCode(workElement, messageContext, relatedFields,
                        index, isDelta);
                }
    
                // load WbsScheduleDates
                if (scope.isScheduleDates())
                {
                    loadScheduleDates(workElement, scope, messageContext, isDelta);
                }
    
                // load project
                if (scope.getContainingProject() != null)
                {
                    loadProject(workElement, scope.getContainingProject(),
                        messageContext, relatedFields, index, isDelta);
                    // load the project's default steps
                    if (scope.getDefaultSteps() != null
                            && workElement.getContainingProject() != null)
                    {
                        loadFavoriteActivities((GenericProject) workElement
                            .getContainingProject(), scope.getDefaultSteps(),
                            messageContext, isDelta);
                    }
                }
    
                //load assignments
                if (scope.isRtfAssignments())
                {
                    loadWBSRtfAssignments((WorkElement) rpmObject, null,
                        messageContext, isDelta);
                }
    
                if (scope.isAttributeAssignments())
                {
                    loadAttributeAssignments(workElement, messageContext, isDelta);
                }
    
                if (scope.isRelatedAttributeAssignments()
                        && rpmObject instanceof GenericProject)
                {
                    loadChildrenAttributeAssignments((GenericProject) workElement,
                        messageContext, isDelta);
                }
    
                // load Time Code assignments
                if (scope.isTimeCodeAssignments())
                {
                    loadTimeCodeAssignments(workElement, messageContext, isDelta);
                }
    
                if (scope.isTimeCodes())
                {
                    loadTimeCodes(workElement, messageContext, relatedFields,
                        index, isDelta);
                }
    
                //load State
                loadState(rpmObject, scope, messageContext, relatedFields, index,
                    isDelta);
    
                //load dependencies
                if (scope.getDependencies() != null)
                {
                    loadDependencies((WorkElement) rpmObject, scope
                        .getDependencies(), messageContext, isDelta);
                }
                
                if (scope.getWorkflowProcessesAvailable() != null)
                {
                    loadAvailableWorkflowProcesses(workElement, scope
                        .getWorkflowProcessesAvailable(), messageContext, isDelta);
                }
                
                if (scope.getWorkflowProcessesRunning() != null)
                {
                    loadRunningWorkflowProcesses(workElement, scope
                        .getWorkflowProcessesRunning(), messageContext, isDelta);
                }
    
                //load children
                if (rpmObject instanceof AggregateNode)
                {
                    AggregateNode parent = (AggregateNode) rpmObject;
                    WorkElementScope childrenScope = scope.getChildren();
                    parent.setChildren(loadWorkElements(parent, parent
                        .getChildren(), childrenScope, messageContext, isDelta));  
                    
                }
    
                if (rpmObject instanceof GenericProject)
                {
                    //load project reconciliation
                    //TODO uncomment to implement ProjectReconciliation
    /*                if (scope.isProjectReconciliations())
                    {
                        loadProjectReconciliations((GenericProject) rpmObject,
                            messageContext, isDelta);
                    }*/
                    
    
                    //load opportunity
                    if (scope.isOpportunity())
                    {
                        OpportunityManager projMgr = (OpportunityManager) getManagerInstance(Opportunity.class
                            .getName());
                        Opportunity opportunity = new Opportunity();
                        opportunity.setID(rpmObject.getID());
                        opportunity = (Opportunity) projMgr.loadByPrimaryKey(
                            opportunity, null, messageContext, isDelta);
    
                        if (isDelta)
                        {
                            ((GenericProject) rpmObject)
                                .deltaOpportunity(opportunity);
                        }
                        else
                        {
                            ((GenericProject) rpmObject)
                                .setOpportunity(opportunity);
                        }
                    }
    
                    //Load the published scorecards
                    if (scope.getAssignedScorecards() != null)
                    {
                        loadPublishedScorecards((GenericProject) workElement,
                            scope, messageContext, isDelta);
                    }
                    if (scope.getScopeScorecards() != null)
                    {
                        loadScopeScorecards((GenericProject) workElement, scope,
                            messageContext, isDelta);
                    }
    
                    if (scope.isOrganizationalAssignments())
                    {
                        loadOrganizationalAssignments((GenericProject) workElement,
                            messageContext, isDelta);
                    }
    
                    //loadProjectSynchronizationState
                    //TODO uncomment to implement ProjectReconciliation
    /*                if (scope.isProjectReconciliationStatus())
                    {
                        loadProjectReconciliationStatus(
                            (GenericProject) workElement, messageContext, isDelta);
    
                    }*/
                    
                    if (scope.getWorkflowRoleMappings() != null)
                    {
                        loadWorkflowRoleMappings((GenericProject) workElement,
                            scope.getWorkflowRoleMappings(), messageContext, isDelta);
                    }
                    
                    if (scope.getPortfolio() != null)
                    {

                        loadPortfolio((GenericProject) workElement, scope, messageContext,
                            relatedFields, index, isDelta);
                    }
                        
                }
    
                if (scope.getDocumentFolder() != null)
                {
                    loadDocumentFolder(workElement, scope, messageContext, isDelta);
                }
    
                //TODO This part was added for a demo an may not be fully
                // functionnal
    
                //load ResourceTaskAssignments
                loadResourceTaskAssignments((WorkElement) rpmObject, scope,
                    messageContext, isDelta);
    
                if (scope.isCustomFieldAssignments())
                {
                    workElement.setCustomFieldAssignments(CustomFieldAssignmentUtil
                        .loadAssignedCustomFields(this, workElement
                            .getCustomFieldAssignments(), workElement,
                            messageContext, isDelta));
                    propagateContextName(workElement, workElement
                        .getCustomFieldAssignments());
                }
            
            }
        }

        if (rpmObject instanceof GenericProject)
        {
            //load calendar
            loadCalendar(rpmObject, messageContext, relatedFields, index,
                isDelta);

            //load currency
            loadCurrency(rpmObject, messageContext, relatedFields, index,
                isDelta);
        }
        
        if (rpmScope != null && rpmScope instanceof WorkElementScope)
        {
            WorkElementScope scope = (WorkElementScope) rpmScope;

            loadFinancials(workElement, scope, messageContext, isDelta);
            loadScopeElement(workElement, scope, messageContext, isDelta);
        }

        loadAssetCode(workElement, messageContext, relatedFields, index,
            isDelta);

        return rpmObject;
    }

    
    private void loadEarnedValues(WorkElement workElement,
            WorkElementScope scope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException
    {
        // user earnedValues
        EarnedValues userEarnedValues = workElement.getEarnedValues();

        // Calulated EarnedValues (as seen in the client)
        EarnedValues clientEarnedValues = new EarnedValues();

        // db earnedValues
        EarnedValues dbValues = WbsUtil.processEarnedValues(relatedFields,
            index);

        // If userEarnedValues is null we will create one so that a load will
        // fetch the data form the DB.
        if (userEarnedValues == null)
        {
            userEarnedValues = new EarnedValues();
            workElement.setEarnedValues(userEarnedValues);
        }

        // updating client EV with database EV values
        WbsUtil.updateClientEVWithDBValues(clientEarnedValues, dbValues,
            workElement);

        if (isDelta)
        {
            userEarnedValues.deltaBudgetAtCompletion(clientEarnedValues
                .getBudgetAtCompletion());
            userEarnedValues.deltaSchedAtCompletion(clientEarnedValues
                .getSchedAtCompletion());
            userEarnedValues.deltaBudgetedCostWorkSched(clientEarnedValues
                .getBudgetedCostWorkSched());
            userEarnedValues.deltaBudgetedCostWorkPerf(clientEarnedValues
                .getBudgetedCostWorkPerf());
            userEarnedValues.deltaActualCostWorkPerf(clientEarnedValues
                .getActualCostWorkPerf());
            userEarnedValues.deltaTimeVariance(clientEarnedValues
                .getTimeVariance());
            userEarnedValues.deltaEstimateToComplete(clientEarnedValues
                .getEstimateToComplete());
            userEarnedValues.deltaEstimateAtComplete(clientEarnedValues
                .getEstimateAtComplete());
            userEarnedValues.deltaEstimatedCompletionDate(clientEarnedValues
                .getEstimatedCompletionDate());
            userEarnedValues
                .deltaBudgetedCostWorkSchedForecast(clientEarnedValues
                    .getBudgetedCostWorkSchedForecast());
            userEarnedValues
                .deltaBudgetedCostWorkPerfForecast(clientEarnedValues
                    .getBudgetedCostWorkPerfForecast());
            userEarnedValues.deltaActualCostWorkPerfForecast(clientEarnedValues
                .getActualCostWorkPerfForecast());
            userEarnedValues.deltaEstimateToCompleteForecast(clientEarnedValues
                .getEstimateToCompleteForecast());
            userEarnedValues.deltaEstimateAtCompleteForecast(clientEarnedValues
                .getEstimateAtCompleteForecast());

            // calculated fields
            userEarnedValues.deltaScheduleVariance_C(clientEarnedValues
                .getScheduleVariance_C());
            userEarnedValues.deltaCostVariance_C(clientEarnedValues
                .getCostVariance_C());
            userEarnedValues.deltaPercentComplete_C(clientEarnedValues
                .getPercentComplete_C());
            userEarnedValues.deltaSchedPerformIndex_C(clientEarnedValues
                .getSchedPerformIndex_C());
            userEarnedValues.deltaCostPerformIndex_C(clientEarnedValues
                .getCostPerformIndex_C());
            userEarnedValues.deltaCriticalRatio_C(clientEarnedValues
                .getCriticalRatio_C());
            userEarnedValues.deltaToCompletePerformIndex_C(clientEarnedValues
                .getToCompletePerformIndex_C());
            userEarnedValues.deltaVarianceAtCompletion_C(clientEarnedValues
                .getVarianceAtCompletion_C());
            userEarnedValues.deltaSchedPerforIndexAtComp_C(clientEarnedValues
                .getSchedPerforIndexAtComp_C());
            userEarnedValues.deltaCriticalRatioAtComp_C(clientEarnedValues
                .getCriticalRatioAtComp_C());
            userEarnedValues.deltaTimeEstimateAtComp_C(clientEarnedValues
                .getTimeEstimateAtComp_C());
            userEarnedValues.deltaTimeVarianceAtComp_C(clientEarnedValues
                .getTimeVarianceAtComp_C());

        }
        else
        {
            workElement.setEarnedValues(clientEarnedValues);
        }
    }


     private void loadAssetCode(WorkElement parent,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        String assetCodeId = (String) relatedFields.get(index,
            ID_FINANCIAL_CATEGORY);
        AssetFinancialCategory assetCode = null;
        if (assetCodeId != null && !isDelta)
        {
            assetCode = new AssetFinancialCategory();
            assetCode.setID(assetCodeId);
            RPMObjectManager manager = getManagerInstance(AssetFinancialCategory.class
                .getName());
            assetCode = (AssetFinancialCategory) manager.loadByPrimaryKey(
                assetCode, null, messageContext, false);
        }
        if (isDelta)
        {
            parent.deltaAssetCode(assetCode);
        }
        else
        {
            parent.setAssetCode(assetCode);
        }
    }

    private void loadOrganizationalAssignments(GenericProject genProject,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        RPMObjectManager manager = RPMManagerFactory.getInstance()
            .getRPMObjectManager(ProjectOrganizationalAssignment.class);

        Object[] arguments =
        {
            genProject.getID()
        };
        ArrayList childrenList = manager.loadByForeignKey(genProject
            .getOrganizationalAssignments(), null, messageContext, this,
            arguments, null, genProject.getContextName(), isDelta);
        ProjectOrganizationalAssignment[] children = null;
        if (childrenList != null)
        {
            int childrenCount = childrenList.size();
            if (childrenCount > 0)
            {
                children = new ProjectOrganizationalAssignment[childrenCount];
                childrenList.toArray(children);
            }
        }
        genProject.setOrganizationalAssignments(children);
    }

    private void loadParent(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {

        String oldParentId = (String) relatedFields.get(index, ID_PARENT_ID);
        RPMObject newParent = null;
        if (isDelta)
        {
            newParent = rpmObject.getParent();
            RPMObject oldParent = null;
            if (oldParentId != null)
            {
                oldParent = new WorkElement();
                oldParent.setID(oldParentId);
                if (newParent != null && newParent.getID() != null
                        && newParent.getID().equals(oldParentId))
                {
                    oldParent = loadByPrimaryKey(newParent, rpmScope
                        .getParent(), messageContext, true);
                }
                else
                {
                    //In this case, old parent is loaded without the
                    // scope to avoid problem with the cache
                    oldParent = loadByPrimaryKey(oldParent, null,
                        messageContext, false);
                }
            }
            rpmObject.deltaParent(oldParent);
        }
        else
        {
            if (oldParentId != null)
            {
                if (newParent == null)
                {//newParent not in cache
                    newParent = new WorkElement();
                    newParent.setID(oldParentId);
                    newParent = loadByPrimaryKey(newParent, rpmScope
                        .getParent(), messageContext, false);
                }
                rpmObject.setParent(newParent);
            }
        }
    }

    private void loadPortfolio(GenericProject object,
            WorkElementScope rpmscope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        RPMObjectManager manager = RPMManagerFactory.getInstance()
            .getRPMObjectManager(Portfolio.class);

        ArrayList temp = manager.loadByForeignKey(new RPMObject[]
        {
            object.getPortfolio()
        }, null, messageContext, this, new Object[]
        {
            object.getID()
        }, null, null, isDelta);

        Portfolio portfolio = null;
        if (temp != null)
        {
            Portfolio[] objs = new Portfolio[temp.size()];
            temp.toArray(objs);
            portfolio = objs[0];
        }
        if (isDelta)
        {
            object.deltaPortfolio(portfolio);
        }
        else
        {
            object.setPortfolio(portfolio);
        }

    }
    
    
    private void loadScheduleDates(WorkElement oWorkElement, WorkElementScope scope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (!scope.isScheduleDates())
        {
            return;
        }

        //Note: If all the ScheduleDate fields are null,
        //the object should not be set in the ScopeElement

        //Cannot use the isDelta method on ScheduleDate objects

        RPMObjectManager oScheduleDateManager = getManagerInstance(WbsScheduleDate.class
            .getName());

        //expectedDate
        WbsScheduleDate expectedDate = oWorkElement.getExpectedDate();
        if (expectedDate == null)
        {
            expectedDate = new WbsScheduleDate();
        }
        expectedDate.setID(oWorkElement.getID());
        expectedDate.setContextName(WbsScheduleDate.EXPECTED);
        expectedDate = (WbsScheduleDate) oScheduleDateManager.loadByPrimaryKey(
            expectedDate, null, messageContext, isDelta);

        if (ScheduleDateUtil.isEmpty(expectedDate))
        {
            expectedDate = null;
        }

        oWorkElement.setExpectedDate(expectedDate);

        //proposedDate
        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setID(oWorkElement.getID());
        proposedDate.setContextName(WbsScheduleDate.PROPOSED);
        proposedDate = (WbsScheduleDate) oScheduleDateManager.loadByPrimaryKey(
            proposedDate, null, messageContext, false);

        if (ScheduleDateUtil.isEmpty(proposedDate))
        {
            proposedDate = null;
        }

        // Proposed Dates are editable only for the class SimpleNode ( Task and Milestone). 
        // For the rest they are read only. Hence setting delta flag is done only for 
        // the sub classes of SimpleNode.
        if(isDelta && SimpleNode.class.isAssignableFrom(oWorkElement.getClass()))
        {
        	if(oWorkElement.getProposedDate() != null)
        	{
        		oWorkElement.getProposedDate().setContextName(WbsScheduleDate.PROPOSED);
        	}
        	oWorkElement.deltaProposedDate(proposedDate);
        }
        else
        {
        	oWorkElement.setProposedDate(proposedDate);
        }
        if(oWorkElement.getProposedDate() != null)
        {
        	oWorkElement.getProposedDate().setID(oWorkElement.getID());
        }
        
        //planDate
        WbsScheduleDate planDate = oWorkElement.getPlanDate();
        if (planDate == null)
        {
            planDate = new WbsScheduleDate();
        }
        planDate.setID(oWorkElement.getID());
        planDate.setContextName(WbsScheduleDate.PLAN);
        planDate = (WbsScheduleDate) oScheduleDateManager.loadByPrimaryKey(planDate,
            null, messageContext, isDelta);

        if (ScheduleDateUtil.isEmpty(planDate))
        {
            planDate = null;
        }

        oWorkElement.setPlanDate(planDate);

        //baselineDate
        WbsScheduleDate baselineDate = oWorkElement.getBaselineDate();
        if (baselineDate == null)
        {
            baselineDate = new WbsScheduleDate();
        }
        baselineDate.setID(oWorkElement.getID());
        baselineDate.setContextName(WbsScheduleDate.BASELINE);
        baselineDate = (WbsScheduleDate) oScheduleDateManager.loadByPrimaryKey(
            baselineDate, null, messageContext, isDelta);

        if (ScheduleDateUtil.isEmpty(baselineDate))
        {
            baselineDate = null;
        }

        oWorkElement.setBaselineDate(baselineDate);

        //initialBaselineDate
        WbsScheduleDate initialBaselineDate = oWorkElement
            .getInitialBaselineDate();
        if (initialBaselineDate == null)
        {
            initialBaselineDate = new WbsScheduleDate();
        }
        initialBaselineDate.setID(oWorkElement.getID());
        initialBaselineDate.setContextName(WbsScheduleDate.INITIALBASELINE);
        initialBaselineDate = (WbsScheduleDate) oScheduleDateManager.loadByPrimaryKey(
            initialBaselineDate, null, messageContext, isDelta);

        if (ScheduleDateUtil.isEmpty(initialBaselineDate))
        {
            initialBaselineDate = null;
        }

        oWorkElement.setInitialBaselineDate(initialBaselineDate);

        //previousBaselineDate
        WbsScheduleDate previousBaselineDate = oWorkElement
            .getPreviousBaselineDate();
        if (previousBaselineDate == null)
        {
            previousBaselineDate = new WbsScheduleDate();
        }
        previousBaselineDate.setID(oWorkElement.getID());
        previousBaselineDate
            .setContextName(WbsScheduleDate.PREVIOUSBASELINE);
        previousBaselineDate = (WbsScheduleDate) oScheduleDateManager.loadByPrimaryKey(
            previousBaselineDate, null, messageContext, isDelta);

        if (ScheduleDateUtil.isEmpty(previousBaselineDate))
        {
            previousBaselineDate = null;
        }

        oWorkElement.setPreviousBaselineDate(previousBaselineDate);

        //actualDate
        WbsScheduleDate actualDate = oWorkElement.getActualDate();
        if (actualDate == null)
        {
            actualDate = new WbsScheduleDate();
        }
        actualDate.setID(oWorkElement.getID());
        actualDate.setContextName(WbsScheduleDate.ACTUAL);
        actualDate = (WbsScheduleDate) oScheduleDateManager.loadByPrimaryKey(actualDate,
            null, messageContext, isDelta);

        if (ScheduleDateUtil.isEmpty(actualDate))
        {
            actualDate = null;
        }

        oWorkElement.setActualDate(actualDate);

        //forecastDate
        WbsScheduleDate forecastDate = oWorkElement.getForecastDate();
        if (forecastDate == null)
        {
            forecastDate = new WbsScheduleDate();
        }
        forecastDate.setID(oWorkElement.getID());
        forecastDate.setContextName(WbsScheduleDate.FORECAST);
        forecastDate = (WbsScheduleDate) oScheduleDateManager.loadByPrimaryKey(
            forecastDate, null, messageContext, isDelta);

        if (ScheduleDateUtil.isEmpty(forecastDate))
        {
            forecastDate = null;
        }

        oWorkElement.setForecastDate(forecastDate);
    }

    private void loadDefaultProjectFlag(GenericProject genericProject,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException
    {
        Boolean defaultProjectFlag = decodeDefaultProjectFlag(genericProject,
            messageContext);
        if (isDelta)
        {
            genericProject.deltaDefaultProject(defaultProjectFlag);
        }
        else
        {
            genericProject.setDefaultProject(defaultProjectFlag);
        }
    }

    private void loadTimeCodeAssignments(WorkElement workElement,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        TimeCodeAssignmentManager manager = (TimeCodeAssignmentManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(
                TimeCodeAssignment.class.getName());

        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
            workElement, workElement.getTimeCodeAssignments(), null,
            messageContext, this, new Object[]
            {
                workElement.getID()
            }, null, null, isDelta);

        if (temp != null && temp.size() > 0)
        {
            TimeCodeAssignment[] tcAssigns = new TimeCodeAssignment[temp.size()];
            temp.toArray(tcAssigns);
            workElement.setTimeCodeAssignments(tcAssigns);
        }
    }

    private void loadTimeCodes(WorkElement workElement,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        String timeCode1Id = (String) relatedFields
            .get(index, ID_TIMECODE_1_ID);
        TimeCode oldTimeCode1 = workElement.getTimeCode1();
        TimeCode loadedTimeCode1 = loadTimeCode(oldTimeCode1, null,
            messageContext, timeCode1Id, isDelta);

        String timeCode2Id = (String) relatedFields
            .get(index, ID_TIMECODE_2_ID);
        TimeCode oldTimeCode2 = workElement.getTimeCode2();
        TimeCode loadedTimeCode2 = loadTimeCode(oldTimeCode2, null,
            messageContext, timeCode2Id, isDelta);

        if (isDelta)
        {
            workElement.deltaTimeCode1(loadedTimeCode1);
            workElement.deltaTimeCode2(loadedTimeCode2);
        }
        else
        {
            workElement.setTimeCode1(loadedTimeCode1);
            workElement.setTimeCode2(loadedTimeCode2);
        }
    }

    private TimeCode loadTimeCode(TimeCode previousTimeCode,
            TimeCodeScope timeCodeScope, MessageContext messageContext,
            String oldTimeCodeID, boolean isDelta) throws RPMException,
            SQLException, ParseException
    {
        TimecodeManager manager = (TimecodeManager) getManagerInstance(TimeCode.class
            .getName());

        TimeCode newTimeCode = null;
        if (isDelta)
        {
            newTimeCode = previousTimeCode;
            TimeCode oldTimeCode = null;
            if (oldTimeCodeID != null)
            {
                oldTimeCode = new TimeCode();
                oldTimeCode.setID(oldTimeCodeID);
                if (newTimeCode != null && newTimeCode.getID() != null
                        && newTimeCode.getID().equals(oldTimeCodeID))
                {
                    oldTimeCode = (TimeCode) manager.loadByPrimaryKey(
                        newTimeCode, timeCodeScope, messageContext, true);
                }
                else
                {
                    //In this case, old timecode is loaded without the
                    // scope to avoid problem with the cache
                    oldTimeCode = (TimeCode) manager.loadByPrimaryKey(
                        oldTimeCode, timeCodeScope, messageContext, false);
                }
            }
            return oldTimeCode;
        }
        else

        if (oldTimeCodeID != null)
        {
            newTimeCode = new TimeCode();
            newTimeCode.setID(oldTimeCodeID);
            newTimeCode = (TimeCode) manager.loadByPrimaryKey(newTimeCode,
                timeCodeScope, messageContext, false);

            return newTimeCode;
        }

        return null;
    }

    private void loadPublishedScorecards(GenericProject project,
            WorkElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        if (scope != null && scope.getAssignedScorecards() != null)
        {

            PublishedScorecardManager manager = (PublishedScorecardManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    AssignedScorecard.class.getName());

            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                project, project.getAssignedScorecards(), scope
                    .getAssignedScorecards(), messageContext, this,
                new Object[]
                {
                    project.getID()
                }, null, project.getContextName(), isDelta);

            AssignedScorecard[] scoreCards = null;
            if (temp != null && !temp.isEmpty())
            {
                scoreCards = new AssignedScorecard[temp.size()];
                temp.toArray(scoreCards);
                project.setAssignedScorecards(scoreCards);
            }
        }//if
    }

    // TODO: DELETE this method if it is not used.
    //private void loadTimeCodes(WorkElement workElement,
    //        MessageContext messageContext, FieldValueMap relatedFields,
    //        int index, boolean isDelta)
    //  throws RPMException, SQLException, ParseException
    //{
    //    String timeCode1Id = (String) relatedFields.get(index, ID_TIMECODE_1_ID);
    //    TimeCode oldTimeCode1 = workElement.getTimeCode1();
    //    TimeCode loadedTimeCode1 = loadTimeCode(null,
    //        messageContext, timeCode1Id, oldTimeCode1, isDelta);
    //
    //    String timeCode2Id = (String) relatedFields.get(index, ID_TIMECODE_2_ID);
    //    TimeCode oldTimeCode2 = workElement.getTimeCode2();
    //    TimeCode loadedTimeCode2 = loadTimeCode(null,
    //        messageContext, timeCode2Id, oldTimeCode2, isDelta);
    //
    //    if (isDelta)
    //    {
    //        workElement.deltaTimeCode1(loadedTimeCode1);
    //        workElement.deltaTimeCode2(loadedTimeCode2);
    //    }
    //    else
    //    {
    //        workElement.setTimeCode1(loadedTimeCode1);
    //        workElement.setTimeCode2(loadedTimeCode2);
    //    }
    //}

    // TODO: DELETE this method if it is not used.
    //private TimeCode loadTimeCode(TimeCodeScope timeCodeScope,
    //        MessageContext messageContext, String oldTimeCodeID,
    //        TimeCode previousTimeCode, boolean isDelta)
    //        throws RPMException, SQLException, ParseException
    //{
    //    TimecodeManager manager = (TimecodeManager) getManagerInstance(
    //        TimeCode.class.getName());
    //
    //    TimeCode newTimeCode = null;
    //    if (isDelta)
    //    {
    //        newTimeCode = previousTimeCode;
    //        TimeCode oldTimeCode = null;
    //        if (oldTimeCodeID != null)
    //        {
    //            oldTimeCode = new TimeCode();
    //            oldTimeCode.setID(oldTimeCodeID);
    //            if (newTimeCode != null && newTimeCode.getID() != null
    //                    && newTimeCode.getID().equals(oldTimeCodeID))
    //            {
    //                oldTimeCode = (TimeCode)manager.loadByPrimaryKey(newTimeCode,
    //                    timeCodeScope, messageContext, true);
    //            }
    //            else
    //            {
    //                //In this case, old timecode is loaded without the
    //                // scope to avoid problem with the cache
    //                oldTimeCode = (TimeCode)manager.loadByPrimaryKey(oldTimeCode,
    //                    timeCodeScope, messageContext, false);
    //            }
    //        }
    //        return oldTimeCode;
    //    }
    //    else
    //    {
    //        if (oldTimeCodeID != null)
    //        {
    //            newTimeCode = (TimeCode)getContainerInCache(oldTimeCodeID,
    //                messageContext);
    //            if (newTimeCode == null)
    //            { //newTimecode not in cache
    //                newTimeCode = new TimeCode();
    //                newTimeCode.setID(oldTimeCodeID);
    //                newTimeCode = (TimeCode)manager.loadByPrimaryKey(newTimeCode,
    //                    timeCodeScope, messageContext, false);
    //            }
    //            return newTimeCode;
    //        }
    //    }
    //    return null;
    //}

    private void loadScopeScorecards(GenericProject project,
            WorkElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        //TODO load AssignedScorecards with the PublishedScorecardManager

        //            ScorecardAssignmentManager manager = (ScorecardAssignmentManager)
        // RPMManagerFactory
        //            .getInstance().getRPMObjectManager(
        //                    ScoreCardAssignment.class.getName());
        //
        //            //Action Score Card Assignments
        //            String extraWhere = "TYPE_ID = " + Action.TYPE_ID;
        //            ArrayList temp = manager.loadByForeignKey(
        //                  project.getActionScoreCardAssignments(),
        //                  scope.getAssignedScorecards(), messageContext, this,
        //                  new Object[]
        //                             {
        //                          project.getID()
        //                  }, extraWhere, project.getContextName(), isDelta);
        //            ScoreCardAssignment[] scAssigns = null;
        //            if(temp != null && !temp.isEmpty()){
        //              scAssigns = new ScoreCardAssignment[temp.size()];
        //              temp.toArray(scAssigns);
        //              for(int i=0; i<scAssigns.length;++i){
        //                  scAssigns[i].setParent(project);
        //                  scAssigns[i].setContextName(ContextUtil.SCOPE_CONTEXT);
        //              }
        //              project.setActionScoreCardAssignments(scAssigns);
        //            }
        //            //Defect Score Card Assignments
        //            extraWhere = "TYPE_ID = " + Defect.TYPE_ID;
        //            temp = manager.loadByForeignKey(
        //                  project.getDefectScoreCardAssignments(),
        //                  scope.getAssignedScorecards(), messageContext, this,
        //                  new Object[]
        //                             {
        //                          project.getID()
        //                  }, extraWhere, project.getContextName(), isDelta);
        //
        //            if(temp != null && !temp.isEmpty()){
        //              scAssigns = new ScoreCardAssignment[temp.size()];
        //              temp.toArray(scAssigns);
        //              for(int i=0; i<scAssigns.length;++i){
        //                  scAssigns[i].setParent(project);
        //                  scAssigns[i].setContextName(ContextUtil.SCOPE_CONTEXT);
        //              }
        //              project.setDefectScoreCardAssignments(scAssigns);
        //            }
        //            //Issue Score Card Assignments
        //            extraWhere = "TYPE_ID = " + Issue.TYPE_ID;
        //            temp = manager.loadByForeignKey(
        //                  project.getIssueScoreCardAssignments(),
        //                  scope.getAssignedScorecards(), messageContext, this,
        //                  new Object[]
        //                             {
        //                          project.getID()
        //                  }, extraWhere, project.getContextName(), isDelta);
        //
        //            if(temp != null && !temp.isEmpty()){
        //              scAssigns = new ScoreCardAssignment[temp.size()];
        //              temp.toArray(scAssigns);
        //              for(int i=0; i<scAssigns.length;++i){
        //                  scAssigns[i].setParent(project);
        //                  scAssigns[i].setContextName(ContextUtil.SCOPE_CONTEXT);
        //              }
        //              project.setIssueScoreCardAssignments(scAssigns);
        //            }
        //            //Change Request Score Card Assignments
        //            extraWhere = "TYPE_ID = " + ChangeRequest.TYPE_ID;
        //            temp = manager.loadByForeignKey(
        //                  project.getChangeRequestScoreCardAssignments(),
        //                  scope.getAssignedScorecards(), messageContext, this,
        //                  new Object[]
        //                             {
        //                          project.getID()
        //                  }, extraWhere, project.getContextName(), isDelta);
        //
        //            if(temp != null && !temp.isEmpty()){
        //              scAssigns = new ScoreCardAssignment[temp.size()];
        //              temp.toArray(scAssigns);
        //              for(int i=0; i<scAssigns.length;++i){
        //                  scAssigns[i].setParent(project);
        //
        //              }
        //              project.setChangeRequestScoreCardAssignments(scAssigns);
        //            }
        //            //Risk Score Card Assignments
        //            extraWhere = "TYPE_ID = " + Risk.TYPE_ID;
        //            temp = manager.loadByForeignKey(
        //                  project.getRiskScoreCardAssignments(),
        //                  scope.getAssignedScorecards(), messageContext, this,
        //                  new Object[]
        //                             {
        //                          project.getID()
        //                  }, extraWhere, project.getContextName(), isDelta);
        //
        //            if(temp != null && !temp.isEmpty()){
        //              scAssigns = new ScoreCardAssignment[temp.size()];
        //              temp.toArray(scAssigns);
        //              for(int i=0; i<scAssigns.length;++i)
        //                  scAssigns[i].setParent(project);
        //              project.setRiskScoreCardAssignments(scAssigns);
        //            }
        //            //Requirement Score Card Assignments
        //            extraWhere = "TYPE_ID = " + Requirement.TYPE_ID;
        //            temp = manager.loadByForeignKey(
        //                  project.getRequirementScoreCardAssignments(),
        //                  scope.getAssignedScorecards(), messageContext, this,
        //                  new Object[]
        //                             {
        //                          project.getID()
        //                  }, extraWhere, project.getContextName(), isDelta);
        //            scAssigns = null;
        //            if(temp != null && !temp.isEmpty()){
        //              scAssigns = new ScoreCardAssignment[temp.size()];
        //              temp.toArray(scAssigns);
        //              for(int i=0; i<scAssigns.length;++i)
        //                  scAssigns[i].setParent(project);
        //              project.setRequirementScoreCardAssignments(scAssigns);
        //            }
        //            //Service Requirement Score Card Assignments
        //            extraWhere = "TYPE_ID = " + ServiceRequest.TYPE_ID;
        //            temp = manager.loadByForeignKey(
        //                  project.getActionScoreCardAssignments(),
        //                  scope.getAssignedScorecards(), messageContext, this,
        //                  new Object[]
        //                             {
        //                          project.getID()
        //                  }, extraWhere, project.getContextName(), isDelta);
        //
        //            if(temp != null && !temp.isEmpty()){
        //              scAssigns = new ScoreCardAssignment[temp.size()];
        //              temp.toArray(scAssigns);
        //              for(int i=0; i<scAssigns.length;++i)
        //                  scAssigns[i].setParent(project);
        //              project.setServiceRequestScoreCardAssignments(scAssigns);
        //            }

        //TODO: load other scorecards too.
    }

    private void loadProject(WorkElement workElement, WorkElementScope scope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {
        String oldProjectId = (String) relatedFields.get(index, ID_PROJECT_ID);
        GenericProject newProject = null;

        if (isDelta)
        {
            newProject = (GenericProject) workElement.getContainingProject();
            GenericProject oldProject = null;
            if (oldProjectId != null)
            {
                oldProject = new GenericProject();
                oldProject.setID(oldProjectId);
                if (newProject != null && newProject.getID() != null
                        && newProject.getID().equals(oldProjectId))
                {
                    oldProject = (GenericProject) loadByPrimaryKey(newProject,
                        scope, messageContext, true);
                }
                else
                {
                    //In this case, old parent is loaded without the
                    // scope to avoid problem with the cache
                    oldProject = (GenericProject) loadByPrimaryKey(oldProject,
                        scope, messageContext, false);
                }
            }
            workElement.deltaContainingProject(oldProject);
        }
        else
        {
            if (oldProjectId != null)
            {
                newProject = new Project();
                newProject.setID(oldProjectId);
                newProject = (GenericProject) loadByPrimaryKey(newProject,
                    scope, messageContext, false);

                workElement.setContainingProject(newProject);
            }
        }
    }

    private void loadScopeElement(WorkElement workElement,
            WorkElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        if (scope != null && scope.getScopeElement() != null)
        {
            ScopeManager manager = (ScopeManager) RPMManagerFactory
                .getInstance()
                .getRPMObjectManager(ScopeElement.class.getName());

            if (workElement instanceof GenericProject)
            {
                ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                    null, new RPMObject[]
                    {
                        workElement.getScopeElement()
                    }, scope.getScopeElement(), messageContext, this,
                    new Object[]
                    {
                        workElement.getID()
                    }, null, null, isDelta);

                if (temp != null && temp.size() > 0)
                {
                    workElement.setScopeElement((ScopeElement) temp.get(0));
                }
            }
            else
            {
                // if a scope element with the same primary key exists, load it
                if (ScopeManagementUtil.scopeElementExists(workElement.getID(),
                    messageContext))
                {
                    ScopeElement scopeElement = (ScopeElement) manager
                        .loadByPrimaryKey(workElement, scope.getScopeElement(),
                            messageContext, false);

                    workElement.setScopeElement(scopeElement);
                }
            }
        }
    }

    private void loadFinancials(WorkElement workElement,
            WorkElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        if (scope == null)
            return;

        if (scope.getWbsFinancials() != null)
        {
            FinancialManager manager = (FinancialManager) RPMManagerFactory
                .getInstance()
                .getRPMObjectManager(WbsFinancial.class.getName());

            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                workElement, workElement.getWbsFinancials(), scope
                    .getWbsFinancials(), messageContext, this, new Object[]
                {
                    workElement.getID()
                }, null, null, isDelta);

            if (temp != null && temp.size() > 0)
            {
                WbsFinancial[] financials = new WbsFinancial[temp.size()];
                temp.toArray(financials);

                if (!isDelta)
                {
                    FinancialBusinessLogicUtil.printResult(financials,"BEFORE:decodeBusinessRulesForWFinancialsOnLoading:WBS");
                    FinancialBusinessLogicUtil
                        .decodeBusinessRulesForWFinancialsOnLoading(financials,
                            messageContext);
                }

                workElement.setWbsFinancials(financials);
            }
            propagateContextName(workElement, workElement.getWbsFinancials());
        }

        if ((workElement instanceof AggregateNode)
                && (scope.getWorksheetFinancials() != null))
        {
            FinancialManager manager = (FinancialManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    WorksheetFinancial.class.getName());

            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                workElement, ((AggregateNode) workElement)
                    .getWorksheetFinancials(), scope.getWorksheetFinancials(),
                messageContext, this, new Object[]
                {
                    workElement.getID()
                }, null, null, isDelta);

            if (temp != null && temp.size() > 0)
            {
                WorksheetFinancial[] financials = new WorksheetFinancial[temp
                    .size()];

                temp.toArray(financials);

                if (!isDelta)
                {
                    FinancialBusinessLogicUtil.printResult(financials,"BEFORE:decodeBusinessRulesForWFinancialsOnLoading");
                    FinancialBusinessLogicUtil
                        .decodeBusinessRulesForWFinancialsOnLoading(financials,
                            messageContext);
                    FinancialBusinessLogicUtil.processSubElementCalculation(financials, messageContext);
                }

                ((AggregateNode) workElement)
                    .setWorksheetFinancials(financials);
            }
            propagateContextName(workElement, ((AggregateNode) workElement)
                .getWorksheetFinancials());
        }        

    }
    
    private WorkElement saveDocumentFolder(WorkElement workElement,
            WorkElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        if (scope.getDocumentFolder() != null
                && workElement.getDocumentFolder() != null)
        {
            DocumentManager manager = (DocumentManager) getManagerInstance(DocumentFolder.class
                .getName());

            DocumentFolder documentFolder = workElement.getDocumentFolder();
            documentFolder.setParent(workElement);
            manager.internalSave(documentFolder, scope.getDocumentFolder(),
                messageContext);
            documentFolder.setContextName(ContextUtil.WBS_DOCUMENT_CONTEXT);
            workElement.setDocumentFolder(documentFolder);
        }
        return workElement;
    }

    private void loadDocumentFolder(WorkElement workElement,
            WorkElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        if (scope.getDocumentFolder() != null)
        {
            DocumentManager manager = (DocumentManager) getManagerInstance(DocumentFolder.class
                .getName());

            RPMObject[] oldDocumentFolder = new RPMObject[]
            {
                workElement.getDocumentFolder()
            };

            Object[] args = new Object[]
            {
                workElement.getID()

            };

            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                workElement, oldDocumentFolder, scope.getDocumentFolder(),
                messageContext, this, args, null,
                ContextUtil.WBS_DOCUMENT_CONTEXT, isDelta);
            if (temp != null && !temp.isEmpty())
            {
                DocumentFolder documentFolder = (DocumentFolder) temp.get(0);
                documentFolder.setContextName(ContextUtil.WBS_DOCUMENT_CONTEXT);
                workElement.setDocumentFolder(documentFolder);
                
                //remove documentFolder from messageContext so that other workElements
                //in the same GenericProject will not point to the same object
                messageContext.getCache().removeContainerFromCache(documentFolder);
            }
        }

    }

    private void loadAttributeAssignments(WorkElement workElement,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        AttributeAssignmentManager attributeAssignmentManager = (AttributeAssignmentManager) getManagerInstance(AttributeAssignment.class
            .getName());

        String contextName = ContextUtil.getInstance().getContextName(
            workElement.getClass().getName());

        Integer typeId = null;

        String extraWhere = null;

        if (workElement instanceof GenericProject)
        {
            typeId = AttributeUtil.getInstance().getTableType(contextName);
            extraWhere = "TYPE_ID = " + typeId;
        }

        ArrayList temp = attributeAssignmentManager
            .loadByForeignKeyWithSpecifiedParent(workElement, workElement
                .getAttributeAssignments(), null, messageContext, this,
                new Object[]
                {
                    workElement.getID()
                }, extraWhere, contextName, isDelta);

        AttributeAssignment[] attributeAssignments = null;

        if (temp != null)
        {
            attributeAssignments = new AttributeAssignment[temp.size()];
            temp.toArray(attributeAssignments);
        }

        workElement.setAttributeAssignments(attributeAssignments);
    }

    private void loadChildrenAttributeAssignments(
            GenericProject genericProject, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {

        AttributeAssignmentManager attributeAssignmentManager = (AttributeAssignmentManager) getManagerInstance(AttributeAssignment.class
            .getName());

        //SCOPE related assignments
        AttributeAssignment[] actionAttributeAssignments = loadChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getActionAttributeAssignments(), new Action(), messageContext,
            isDelta);

        genericProject
            .setActionAttributeAssignments(actionAttributeAssignments);

        AttributeAssignment[] changeRequestAttributeAssignments = loadChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getChangeRequestAttributeAssignments(), new ChangeRequest(),
            messageContext, isDelta);

        genericProject
            .setChangeRequestAttributeAssignments(changeRequestAttributeAssignments);

        AttributeAssignment[] defectAttributeAssignments = loadChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getDefectAttributeAssignments(), new Defect(), messageContext,
            isDelta);

        genericProject
            .setDefectAttributeAssignments(defectAttributeAssignments);

        AttributeAssignment[] issueAttributeAssignments = loadChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getIssueAttributeAssignments(), new Issue(), messageContext,
            isDelta);

        genericProject.setIssueAttributeAssignments(issueAttributeAssignments);

        AttributeAssignment[] requirementAttributeAssignments = loadChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getRequirementAttributeAssignments(), new Requirement(),
            messageContext, isDelta);

        genericProject
            .setRequirementAttributeAssignments(requirementAttributeAssignments);

        AttributeAssignment[] riskAttributeAssignments = loadChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getRiskAttributeAssignments(), new Risk(), messageContext,
            isDelta);

        genericProject.setRiskAttributeAssignments(riskAttributeAssignments);

        AttributeAssignment[] serviceRequestAttributeAssignments = loadChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getServiceRequestAttributeAssignments(), new ServiceRequest(),
            messageContext, isDelta);

        genericProject
            .setServiceRequestAttributeAssignments(serviceRequestAttributeAssignments);

        //WBS related assignments
        AttributeAssignment[] deliverableAttributeAssignments = loadChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getDeliverableAttributeAssignments(), new Deliverable(),
            messageContext, isDelta);

        genericProject
            .setDeliverableAttributeAssignments(deliverableAttributeAssignments);

        AttributeAssignment[] taskAttributeAssignments = loadChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getTaskAttributeAssignments(), new Task(), messageContext,
            isDelta);

        genericProject.setTaskAttributeAssignments(taskAttributeAssignments);

        //Document related assignments
        AttributeAssignment[] documentAttributeAssignments = loadChildrenAttributeAssignments(
            attributeAssignmentManager, genericProject, genericProject
                .getDocumentAttributeAssignments(), new Document(),
            messageContext, isDelta);

        genericProject
            .setDocumentAttributeAssignments(documentAttributeAssignments);
    }

    private AttributeAssignment[] loadChildrenAttributeAssignments(
            AttributeAssignmentManager attributeAssignmentManager,
            GenericProject genericProject,
            AttributeAssignment[] oldAssignments, RPMObject assignmentParent,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        String contextName = ContextUtil.getInstance().getContextName(
            assignmentParent.getClass().getName());

        assignmentParent.setID(genericProject.getID());

        String extraWhere = "TYPE_ID = " + assignmentParent.retrieveTypeID();

        //Assign the parent type id to the children before reloading
        //The typeId is mandatory when reloading the assignment when isDelta =
        // true;
        if (isDelta && oldAssignments != null)
        {
            for (int i = 0; i < oldAssignments.length; i++)
            {
                oldAssignments[i].assignTypeID(assignmentParent
                    .retrieveTypeID());
            }
        }

        ArrayList temp = attributeAssignmentManager
            .loadByForeignKeyWithSpecifiedParent(assignmentParent,
                oldAssignments, null, messageContext, this, new Object[]
                {
                    genericProject.getID()
                }, extraWhere, contextName, isDelta);

        AttributeAssignment[] attributeAssignments = null;

        if (temp != null)
        {
            attributeAssignments = new AttributeAssignment[temp.size()];
            temp.toArray(attributeAssignments);

            for (int i = 0; i < attributeAssignments.length; i++)
            {
                attributeAssignments[i].setParent(genericProject);
            }
        }
        return attributeAssignments;
    }

    private void loadCalendar(RPMObject rpmObject,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {

        GenericProject project = (GenericProject) rpmObject;
        RPMObjectManager calMgr = getManagerInstance(RPMCalendar.class
            .getName());

        ArrayList temp = calMgr.loadByForeignKey(new RPMObject[]
        {
            project.getCalendar()
        }, new CalendarScope(), messageContext, this, new Object[]
        {
            project.getID()
        }, null, null, isDelta);

        RPMCalendar calendar = null;
        if (temp != null)
        {
            RPMCalendar[] calendars = new RPMCalendar[temp.size()];
            temp.toArray(calendars);
            calendar = calendars[0];
        }

        if (isDelta)
        {
            project.deltaCalendar(calendar);
        }
        else
        {
            project.setCalendar(calendar);
        }
    }

    private void loadState(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, SQLException,
            ParseException
    {
        WorkElement workelement = (WorkElement) rpmObject;
        WorkElementScope scope = (WorkElementScope) rpmScope;

        if (scope != null && scope.isState())
        {
            DatafieldState state = new DatafieldState();

            state.setID((String) relatedFields.get(index, ID_STAGE_ID));

            DatafieldManager manager = (DatafieldManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(State.class.getName());

            state = (DatafieldState) manager.loadByPrimaryKey(state, null,
                messageContext, isDelta);

            if (isDelta)
            {
                workelement.deltaState(state);
            }
            else
            {
                workelement.setState(state);
            }
        }
    }

    private Currency loadCurrency(RPMObject rpmObject,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, ParseException, SQLException
    {
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(getPrimaryKey());
        
        Object[] arguments = new Object[]
        {
            rpmObject.getID()
        };
        
        Integer currencyID = (Integer) ManagerUtil.selectColumnValue(
            Integer.class, rpmObject, "currency", NAME_CURRENCY_ID, TABLE_NAME,
            condition, arguments, messageContext);

        RPMObjectManager manager = RPMManagerFactory.getInstance()
            .getRPMObjectManager(Currency.class.getName());

        Currency currency = new Currency();
        currency.setID(currencyID.toString());
        currency = (Currency) manager.loadByPrimaryKey(currency, null,
            messageContext, isDelta);

        return currency;
    }

    private void loadCurrency(RPMObject rpmObject,
            MessageContext messageContext, FieldValueMap relatedFields,
            int index, boolean isDelta) throws RPMException, ParseException,
            SQLException
    {
        Currency currency = new Currency();

        currency.setID((String) relatedFields.get(index, ID_CURRENCY_ID));

        if (!currency.getID().equals("0"))
        {
            RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(Currency.class);

            currency = (Currency) manager.loadByPrimaryKey(currency, null,
                messageContext, isDelta);

            if (isDelta)
            {
                ((GenericProject) rpmObject).deltaCurrency(currency);
            }
            else
            {
                ((GenericProject) rpmObject).setCurrency(currency);
            }
        }
    }

    private void loadFavoriteActivities(GenericProject project,
            DefaultStepScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        DefaultStepManager manager = (DefaultStepManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(DefaultStep.class.getName());

        ArrayList temp = manager.loadByForeignKey(project.getDefaultSteps(),
            scope, messageContext, this, new Object[]
            {
                project.getID()
            }, null, null, isDelta);

        DefaultStep[] defaultSteps = new DefaultStep[0];
        if (temp != null)
        {
            setDefaultStepsParent(temp, project);
            defaultSteps = new DefaultStep[temp.size()];
            temp.toArray(defaultSteps);
            project.setDefaultSteps(defaultSteps);
        }
    }

    private void setDefaultStepsParent(ArrayList defaultSteps,
            GenericProject parent)
    {
        if (defaultSteps != null && !defaultSteps.isEmpty())
        {
            for (int i = 0; i < defaultSteps.size(); i++)
                ((RPMObject) defaultSteps.get(i)).setParent(parent);
        }
    }

    private WorkElement[] loadWorkElements(RPMObject parent,
            WorkElement[] workelements, WorkElementScope childrenScope,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        WorkElement[] newChildren = null;
        if (childrenScope != null)
        {
            ArrayList temp = loadByForeignKey(workelements, childrenScope,
                messageContext, this, new Object[]
                {
                    parent.getID()
                }, null, null, isDelta);

            if (temp != null)
            {
                newChildren = new WorkElement[temp.size()];
                temp.toArray(newChildren);
            }
        }
        return newChildren;
    }

    private void loadResourceTaskAssignments(WorkElement workElement,
            WorkElementScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, ParseException, SQLException
    {
        if (scope != null && scope.getResourceTaskAssignments() != null)
        {
            ResourceTaskAssignmentManager manager = (ResourceTaskAssignmentManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(
                    ResourceTaskAssignment.class.getName());

            String extraWhere = ResourceTaskAssignmentManager.NAME_ELEMENT_TYPE
                    + " = " + workElement.retrieveTypeID();

            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                workElement, workElement.getResourceTaskAssignments(), scope
                    .getResourceTaskAssignments(), messageContext, this,
                new Object[]
                {
                    workElement.getID()
                }, extraWhere, null, isDelta);

            if (temp != null)
            {
                ResourceTaskAssignment[] taskAss = new ResourceTaskAssignment[temp
                    .size()];
                temp.toArray(taskAss);
                workElement.setResourceTaskAssignments(taskAss);
                propagateContextName(workElement, workElement
                    .getResourceTaskAssignments());
            }
        }
    }

    private void loadDependencies(WorkElement workelement,
            ElementDependencyScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        DependenciesManager dependencyManager = (DependenciesManager) getManagerInstance(ElementDependency.class
            .getName());
        Object[] args = new Object[]
        {
            workelement.getID()
        };
        ArrayList temp = dependencyManager.loadByForeignKeyWithSpecifiedParent(
            workelement, workelement.getDependencies(), scope, messageContext,
            this, args, null, getContextName(workelement), isDelta);
        ElementDependency[] dependencies = null;
        if (temp != null && !temp.isEmpty())
        {
            dependencies = new ElementDependency[temp.size()];
            temp.toArray(dependencies);
        }
        workelement.setDependencies(dependencies);
    }
    
    private void loadRunningWorkflowProcesses(WorkElement workElement,
            RunningWorkflowProcessScope scope, MessageContext messageContext,
            boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        RunningWorkflowProcessManager manager = (RunningWorkflowProcessManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(RunningWorkflowProcess.class.getName());

        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
            workElement, workElement.getWorkflowProcessesRunning(), scope,
            messageContext, this,
            new Object[]
        {
            workElement.getID()
        }, null, null, isDelta);

        RunningWorkflowProcess[] workflowProcesses = new RunningWorkflowProcess[0];
        if (temp != null)
        {
            workflowProcesses = new RunningWorkflowProcess[temp.size()];
            temp.toArray(workflowProcesses);
            workElement.setWorkflowProcessesRunning(workflowProcesses);
        }
    }
    
    private void loadAvailableWorkflowProcesses(WorkElement workElement,
            AvailableWorkflowProcessScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        AvailableWorkflowProcessManager manager = (AvailableWorkflowProcessManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(
                AvailableWorkflowProcess.class.getName());

        Integer groupTypeId = SP_S_WKF_PROCESS_TYPE(workElement
            .retrieveTypeID(), messageContext);

        String rank = WorkflowUtil.loadWorkflowProcessRankThroughParent(
            workElement, groupTypeId, messageContext);
        
        if (rank != null) // rank should never be null
        {
            SqlBuffer extraWhere = new SqlBuffer();
            extraWhere.appendSubSelect(
                AvailableWorkflowProcessManager.NAME_PROCESS_ID,
                WorkflowManager.NAME_ELEMENT_ID, WorkflowManager.TABLE_NAME,
                WorkflowManager.NAME_RANK + " like '" + rank + "%')");

            ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
                workElement, workElement.getWorkflowProcessesAvailable(),
                scope, messageContext, this, new Object[]
                {
                    workElement.getID()
                }, extraWhere.toString(), null, isDelta);

            AvailableWorkflowProcess[] workflowProcesses = new AvailableWorkflowProcess[0];
            if (temp != null)
            {
                workflowProcesses = new AvailableWorkflowProcess[temp.size()];
                temp.toArray(workflowProcesses);
                workElement.setWorkflowProcessesAvailable(workflowProcesses);
            }
        }
    }
    
    private void loadWorkflowRoleMappings(GenericProject genericProject,
            WorkflowRoleMappingScope scope, MessageContext messageContext,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {
        WorkflowRoleMappingManager manager = (WorkflowRoleMappingManager) RPMManagerFactory
            .getInstance().getRPMObjectManager(
                WorkflowRoleMapping.class.getName());

        ArrayList temp = manager.loadByForeignKeyWithSpecifiedParent(
            genericProject, genericProject.getWorkflowRoleMappings(), scope,
            messageContext, this, new Object[]
            {
                genericProject.getID()
            }, null, null, isDelta);

       
        // Add the roles that have no explicit mappings.
        // The PROJECT_WKF_ROLES table (WorkflowRoleMappingManager) does not include
        // a role if its resource, contract group, and security role IDs are all null.
        // They must all appear in the list of loading mappings.
        genericProject.setWorkflowRoleMappings(SP_S_PRJ_WKF_ROLE(
            genericProject, temp, messageContext));
 
    }
    
    //TODO remove comments to implement ProjectReconciliation
/*    private void loadProjectReconciliations(GenericProject genProject,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        RPMObjectManager manager = RPMManagerFactory.getInstance()
            .getRPMObjectManager(ProjectReconciliation.class);

        Object[] arguments =
        {
            genProject.getID()
        };

        ArrayList childrenList = manager.loadByForeignKey(genProject
            .getProjectReconciliations(), null, messageContext, this,
            arguments, null, genProject.getContextName(), isDelta);
        ProjectReconciliation[] children = null;

        if (childrenList != null)
        {
            int childrenCount = childrenList.size();
            if (childrenCount > 0)
            {
                children = new ProjectReconciliation[childrenCount];
                childrenList.toArray(children);
            }
        }
        genProject.setProjectReconciliations(children);
    }*/

    //TODO remove comments to implement ProjectReconciliation
/*    private void loadProjectReconciliationStatus(GenericProject parent,
            MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {

        ProjectReconciliationStatus state = parent
            .getProjectReconciliationStatus();
        if (state == null)
        {
            state = new ProjectReconciliationStatus();
        }
        state.setID(parent.getID());

        RPMObjectManager manager = getManagerInstance(ProjectReconciliationStatus.class
            .getName());
        state = (ProjectReconciliationStatus) manager.loadByPrimaryKey(state,
            null, messageContext, isDelta);

        //note: cannot use isdelta because the id doesn't changes.
        parent.setProjectReconciliationStatus(state);

    }*/

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.rpm.managers.AbstractRPMObjectManager#getRelatedFields(java.sql.ResultSet,
     *      com.ibm.rpm.FieldValueMap, int)
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_PARENT_ID);
        relatedFields.put(index, ID_PARENT_ID, parentId);

        String currencyId = resultSet.getString(ID_CURRENCY_ID);
        relatedFields.put(index, ID_CURRENCY_ID, currencyId);

        String calendarId = resultSet.getString(ID_CALENDAR_ID);
        relatedFields.put(index, ID_CALENDAR_ID, calendarId);

        String stageId = resultSet.getString(ID_STAGE_ID);
        relatedFields.put(index, ID_STAGE_ID, stageId);

        String projectId = resultSet.getString(ID_PROJECT_ID);
        relatedFields.put(index, ID_PROJECT_ID, projectId);

        String timeCode1Id = resultSet.getString(ID_TIMECODE_1_ID);
        relatedFields.put(index, ID_TIMECODE_1_ID, timeCode1Id);

        String timeCode2Id = resultSet.getString(ID_TIMECODE_2_ID);
        relatedFields.put(index, ID_TIMECODE_2_ID, timeCode2Id);

        String assetCodeId = ManagerUtil.intIdToStringId(resultSet
            .getInt(ID_FINANCIAL_CATEGORY));
        relatedFields.put(index, ID_FINANCIAL_CATEGORY, assetCodeId);
        
        //earned values
        
        Double budgetAtCompletion = new Double(resultSet.getDouble(ID_BAC));
        relatedFields.put(index, ID_BAC, budgetAtCompletion); 
        
        Integer schedAtCompletion = new Integer(resultSet
            .getInt(ID_BASELINE3_DURATION));
        relatedFields.put(index, ID_BASELINE3_DURATION, schedAtCompletion);

        Double budgetedWorkSched = new Double(resultSet.getDouble(ID_BCWS));
        relatedFields.put(index, ID_BCWS, budgetedWorkSched);

        Double budgetedWorkPerform = new Double(resultSet.getDouble(ID_BCWP));
        relatedFields.put(index, ID_BCWP, budgetedWorkPerform);

        Double actualWorkPerform = new Double(resultSet.getDouble(ID_ACWP));
        relatedFields.put(index, ID_ACWP, actualWorkPerform);

        Integer timeVariance = new Integer(resultSet
            .getInt(ID_TIME_VARIANCE));
        relatedFields.put(index, ID_TIME_VARIANCE, timeVariance);

        Double estimateToComplete = new Double(resultSet.getDouble(ID_EV_ETC));
        relatedFields.put(index, ID_EV_ETC, estimateToComplete);

        Double estimateAtComplete = new Double(resultSet.getDouble(ID_EAC));
        relatedFields.put(index, ID_EAC, estimateAtComplete);

        Calendar estimatedCompletionDate = Manager.getCalendar(resultSet,
            ID_BASELINE3_FINISH);
        relatedFields.put(index, ID_BASELINE3_FINISH, estimatedCompletionDate);

        Double budgetedWorkSchedForecast = new Double(resultSet
            .getDouble(ID_BCWS_FORECAST));
        relatedFields.put(index, ID_BCWS_FORECAST, budgetedWorkSchedForecast);

        Double budgetedWorkPerformForecast = new Double(resultSet
            .getDouble(ID_BCWP_FORECAST));
        relatedFields.put(index, ID_BCWP_FORECAST, budgetedWorkPerformForecast);

        Double actualWorkPerformForecast = new Double(resultSet
            .getDouble(ID_ACWP_FORECAST));
        relatedFields.put(index, ID_ACWP_FORECAST, actualWorkPerformForecast);

        Double estimateToCompleteForecast = new Double(resultSet
            .getDouble(ID_EV_ETC_FORECAST));
        relatedFields
            .put(index, ID_EV_ETC_FORECAST, estimateToCompleteForecast);

        Double estimateAtCompleteForecast = new Double(resultSet
            .getDouble(ID_EAC_FORECAST));
        relatedFields.put(index, ID_EAC_FORECAST, estimateAtCompleteForecast);

        return relatedFields;
    }

    private void loadWBSRtfAssignments(WorkElement workelement,
            RPMObjectScope scope, MessageContext messageContext, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        RtfAssignmentManager rtfAssignmentManager = (RtfAssignmentManager) getManagerInstance(RtfAssignment.class
            .getName());

        Object[] args = new Object[]
        {
            workelement.getID()
        };

        String contextName = getContextName(workelement);
        
        if (workelement instanceof SummaryTask || workelement instanceof Task)
        {
            if (ScopeManagementUtil.scopeElementExists(workelement.getID(), messageContext))
            {
                // workelement is a tranferred ScopeElement and 
                // we must load the rtfAssignments from the Scope context
                contextName = ContextUtil.SCOPE_CONTEXT;
            }
        }
        
        ArrayList loadedRtfAssignments = rtfAssignmentManager
            .loadByForeignKeyWithSpecifiedParent(workelement, workelement
                .getRtfAssignments(), scope, messageContext, this, args, null,
                contextName, isDelta);

        RtfAssignment[] rtfAssignments = null;

        if (loadedRtfAssignments != null && !loadedRtfAssignments.isEmpty())
        {
            rtfAssignments = new RtfAssignment[loadedRtfAssignments.size()];
            loadedRtfAssignments.toArray(rtfAssignments);
        }

        workelement.setRtfAssignments(rtfAssignments);
    }

    public WorkElement copyProposedToPlan(WorkElement workElement,
            WorkElementScope scope, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
        WorkElement reloadedElement = null;

        WbsMethodCheckpoint.getInstance().validateCopyProposedToPlan(
            workElement, messageContext);

        if (messageContext.isSuccessful()
                && WbsMethodCheckpoint.getInstance().canCopyProposedToPlan(
                    workElement, scope, messageContext))
        {
            SP_COMMIT(workElement, messageContext);

            // Reload graph (default behavior) if the save was successful
            if (workElement != null && getPrimaryKeyString(workElement) != null)
            {
                ReloadType reloadType = messageContext.getReloadType();
                if (reloadType == null || reloadType == ReloadType.ReloadResult)
                {
                    reloadedElement = (WorkElement) load(workElement, scope,
                        messageContext);
                }
            }

            messageContext.getCache().resetLoadedContainers();
        }

        return reloadedElement;
    }

    public GenericProject commitAffectedResources(
            GenericProject genericProject, WorkElementScope scope,
            MessageContext messageContext) throws RPMException, SQLException,
            ParseException
    {
        GenericProject reloadedProject = null;
        SP_COMMIT_ASSIGN(genericProject, messageContext);
        // Reload graph (default behavior) if the save was successful
        if (genericProject != null
                && getPrimaryKeyString(genericProject) != null)
        {
            ReloadType reloadType = messageContext.getReloadType();
            if (reloadType == null || reloadType == ReloadType.ReloadResult)
            {
                reloadedProject = (GenericProject) load(genericProject, scope,
                    messageContext);
            }
        }

        messageContext.getCache().resetLoadedContainers();

        return reloadedProject;
    }

    public WorkElement convertWorkElement(WorkElement inputWorkElement,
            RPMObject outputWorkElement, RPMObjectScope rpmScope,
            MessageContext msgCtx) throws RPMException, SQLException,
            ParseException
    {
        // the inputWorkElement must be previously saved 
        // and the outputWorkElement must be of the correct type
        WbsMethodCheckpoint.getInstance().validateConvertWorkElement(
            inputWorkElement, outputWorkElement, rpmScope, msgCtx);

        WorkElement outputElement = null;

        if (msgCtx.isSuccessful())
        {
            if (WbsMethodCheckpoint.getInstance().canConvertWorkElement(
                inputWorkElement, outputWorkElement, rpmScope, msgCtx))
            {
                SP_U_WBS_TYPE(inputWorkElement,
                    (WorkElement) outputWorkElement, msgCtx);
                
                //To be safe, remove inputWorkElement from messageContext cache -if it was there-
                //since now it has been converted which means the instance that 
                //-might- exist in the cache is not valid anymore
                msgCtx.getCache().removeContainerFromCache(inputWorkElement);

                outputElement = (WorkElement) loadByPrimaryKey(
                    inputWorkElement, rpmScope, msgCtx, false);
            }
        }

        return outputElement;
    }

    public RPMObject createObjectFromTemplate(RPMObject inputObject,
            RPMObjectScope scope, MessageContext msgCtx) throws RPMException,
            SQLException, ParseException
    {
        RPMObject outputProject = null;

        //InputObject and Template cannot be null
        if (WbsMethodCheckpoint.getInstance().isTemplateAndInputAvailable(
            inputObject, msgCtx))
        {
            //Validating input and template
            WbsMethodCheckpoint.getInstance().validateTemplateAndInput(
                inputObject, msgCtx);

            //only if no errors are found...and input and template objects are valid
            if (msgCtx.isSuccessful())
            {              
                if (WbsMethodCheckpoint.getInstance().canCreateFromTemplate(
                    inputObject, msgCtx.getTemplate(), scope, msgCtx))
                {
                    //---WorkElement Case
                    if (!(inputObject instanceof GenericProject 
                            && msgCtx.getTemplate() != null && msgCtx.getTemplate() instanceof GenericProject))
                    {
                        WorkElement inputWEObject = (WorkElement) inputObject;
                        inputWEObject.setModified();
                        WorkElementCheckpoint checkpoint = (WorkElementCheckpoint) ContainerToCheckpointMap
                            .getCheckpoint(inputObject);
                        checkpoint.validateWorkElementTemplateCreation(
                            inputWEObject, msgCtx);
    
                        if (msgCtx.isSuccessful())
                        {
                            inputWEObject.setID(SP_C_WBS(inputWEObject, msgCtx));
    
                            outputProject = load(inputWEObject, scope, msgCtx);
                        }
    
                    }
                    else //---GenericProject Case
                    {
                        GenericProject inputGPObject = (GenericProject) inputObject;
                        inputGPObject.setModified();
                        GenericProjectCheckpoint checkpoint = (GenericProjectCheckpoint) ContainerToCheckpointMap
                            .getCheckpoint(inputObject);
                        checkpoint.validateProjectTemplateCreation(inputGPObject,
                            msgCtx);
                        
                        if (msgCtx.isSuccessful())
                        {
                            createObject(inputGPObject, msgCtx);
    
                            SP_C_WBS(inputGPObject, msgCtx);
    
                            outputProject = load(inputGPObject, scope, msgCtx);
                        }
                    }  
                }
            }
        }

        return outputProject;
    }
    
    public RPMObject saveProjectBaseline(RPMObject workElementObject,
            RPMObjectScope scope, MessageContext msgCtx) throws RPMException,
            SQLException, ParseException
    {
        RPMObject outputProject = null;

        // project cannot be null
        if (WbsMethodCheckpoint.getInstance().isProjectAvailable(
            workElementObject, msgCtx))
        {
            // Validating project
            WbsMethodCheckpoint.getInstance().validateProjectInput(
                workElementObject,new Class[]{WorkElement.class}, msgCtx);

            // only if no errors are found...and project object is
            // valid
            if (msgCtx.isSuccessful()
                    && WbsMethodCheckpoint.getInstance()
                        .canSaveProjectBaseline(workElementObject, scope,
                            msgCtx))
            {
                workElementObject
                    .setID(SP_COMMIT(workElementObject, msgCtx, 23));
                workElementObject
                    .setID(SP_COMMIT(workElementObject, msgCtx, 24));
                outputProject = load(workElementObject, scope, msgCtx);
            }
        }

        return outputProject;
    }
    
    public RPMObject rollupProjectEV(RPMObject workElementObject,
            RPMObjectScope scope, MessageContext msgCtx) throws RPMException,
            SQLException, ParseException
    {
        RPMObject outputProject = null;

        // project cannot be null
        if (WbsMethodCheckpoint.getInstance().isProjectAvailable(
            workElementObject, msgCtx))
        {
            // Validating project
            WbsMethodCheckpoint.getInstance().validateProjectInput(
                workElementObject,new Class[]{GenericProject.class}, msgCtx);

            // only if no errors are found...and project object is
            // valid
            if (msgCtx.isSuccessful()
                    && WbsMethodCheckpoint.getInstance().canRollupProjectEV(
                        workElementObject, scope, msgCtx))
            {
                workElementObject
                    .setID(SP_ROLLUP_EV(workElementObject, msgCtx));
                outputProject = load(workElementObject, scope, msgCtx);
            }
        }

        return outputProject;
    }
    
    public RPMObject enableProjectEVCalculation(RPMObject workElementObject,
            RPMObjectScope scope, MessageContext msgCtx) throws RPMException,
            SQLException, ParseException
    {
        RPMObject outputProject = null;

        // project cannot be null
        if (WbsMethodCheckpoint.getInstance().isProjectAvailable(
            workElementObject, msgCtx))
        {
            // Validating project
            WbsMethodCheckpoint.getInstance().validateProjectInput(
                workElementObject,new Class[]{GenericProject.class}, msgCtx);

            // only if no errors are found...and project object is
            // valid
            if (msgCtx.isSuccessful())
            {             
                workElementObject
                    .setID(SP_U_PROJ_BITS(workElementObject, msgCtx));
                outputProject = load(workElementObject, scope, msgCtx);
            }
        }

        return outputProject;
    }
    
    public MessageContext activateEVWeekDelays(MessageContext msgCtx,
            int evDelays) throws RPMException, SQLException, ParseException
    {
        // Validating week delay range
        WbsMethodCheckpoint.getInstance()
            .validateEVWeekDelays(msgCtx, evDelays);

        // only if no errors are found...
        if (msgCtx.isSuccessful() && !msgCtx.hasWarnings())
        {
            SP_U_SETTING(msgCtx, evDelays);
        }

        return msgCtx;
    }

 
    public void transferUnderScope(AbstractAggregateScope scopeElement,
            WorkElement workElement, MessageContext messageContext)
            throws RPMException, ParseException, SQLException
    {
        if (scopeElement != null
                && scopeElement.getID() != null
                && (scopeElement.getProject() == null || scopeElement
                    .getProject().getID() == null))
        {
            // reload the scopeElement to get a populated project field
            ScopeManager manager = (ScopeManager) getManagerInstance(ScopeElement.class
                .getName());

            ScopeElementScope scopeElementScope = new ScopeElementScope();
            scopeElementScope.setProject(new WorkElementScope());

            ScopeElement reloadedScopeElement = (AbstractAggregateScope) manager
                .load(scopeElement, scopeElementScope, messageContext);

            scopeElement.setProject(reloadedScopeElement.getProject());
        }

        WbsMethodCheckpoint.getInstance().validateTransferUnderScope(
            scopeElement, workElement, messageContext);

        if (messageContext.isSuccessful()
                && WbsMethodCheckpoint.getInstance().canTransferUnderScope(
                    scopeElement, workElement, messageContext))
        {
            SP_A_WBS_TO_CRI(workElement, scopeElement, messageContext);
        }
    }

    private String getContextName(RPMObject rpmObject) throws RPMException
    {
        return ContextUtil.getInstance().getContextName(rpmObject);
    }

    private String SP_U_WBS_NAME(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_WBS_NAME";
        Object args[] = new Object[3];

        args[0] = rpmObject.getID();
        args[1] = ((WorkElement) rpmObject).getName();
        args[2] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }

    //This is being called when identification portlet is modified.
    private void SP_U_PUBLISHED(GenericProject genericProject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_PUBLISHED";
        Object args[] = new Object[4];

        args[0] = genericProject.getID();
        args[1] = encodePublishedType(genericProject.getPublished());
        args[2] = (genericProject.getParent() != null) ? genericProject
            .getParent().getID() : null;
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    // Unused.
    //private void SP_U_WBS_OPPORTUN(RPMObject rpmObject,
    //        MessageContext messageContext) throws SQLException, RPMException
    //{
    //    String name = "SP_U_WBS_OPPORTUN";
    //
    //    GenericProject project = (GenericProject) rpmObject;
    //
    //    Object args[] = new Object[15];
    //
    //    args[0] = rpmObject.getID();
    //    args[1] = project.getOpportunity().getExternalClient();
    //    args[2] = project.getOpportunity().getOrderOfMagnitudeCost();
    //    args[3] = project.getOpportunity().getOrderOfMagnitudeRevenueBenefits();
    //    args[4] = project.getOpportunity().getBenefitBeginsMonths();
    //    args[5] = project.getOpportunity().getBenefitDurationMonths();
    //    args[6] = DateUtil.convertToString(project.getOpportunity()
    //            .getPursuitStart());
    //    args[7] = DateUtil.convertToString(project.getOpportunity()
    //            .getPursuitFinish());
    //    args[8] = DateUtil.convertToString(project.getOpportunity()
    //            .getForecastClosed());
    //    args[9] = project.getOpportunity().getOpportunityTrackingNumber();
    //    args[10] = project.getOpportunity().getOpportunityOwner();
    //    args[11] = project.getOpportunity().getContactType();
    //    args[12] = project.getOpportunity().getContactNumber();
    //    args[13] = project.getOpportunity().getCustomerNumber();
    //    args[14] = getUser(messageContext).getID();
    //
    //    executeProcedure(messageContext, name, args);
    //}

    //SP_U_CLOSE_WBS_BY_NAME('53E117ECCF9B4C41B256184531999D7A',null,null,'5EFCC9DF8DEE4C078AB4A92FE4794B6E')
    private void SP_U_CLOSE_WBS_BY_NAME(WorkElement workElement, MessageContext messageContext) throws SQLException, RPMException
    {
    	String name = "SP_U_CLOSE_WBS_BY_NAME";

    	Object args[] = new Object[4];

        args[0] = workElement.getID();
        args[1] = workElement.getClosedByName();
        args[2] = DateUtil.convertToStringWithTime(workElement.getClosedByDate());
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);

    }
    
    private void SP_U_STAGE_ID(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_STAGE_ID";

        WorkElement workElement = (WorkElement) rpmObject;

        Object args[] = new Object[4];

        args[0] = workElement.getID();
        args[1] = new Integer(1);//TMT_WBS table
        args[2] = new Integer(Integer.parseInt(workElement.getState().getID()));//TODO
        // chk.
        // SC_LONG_PARAM
        // *p_stage_id,
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_CHECK_IN(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_CHECK_IN";

        Object args[] = new Object[3];

        args[0] = rpmObject.getID();
        args[1] = new Integer(0);//TODO bit flag.. not being used ?
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_CHECK_OUT_WKF(RPMObject rpmObject, String workflowID,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_CHECK_OUT_WKF";
        Object args[] = new Object[4];
        args[0] = rpmObject.getID();
        args[1] = workflowID;
        args[2] = new Integer(1); // table type for work element.
        args[3] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    private void SP_CHECK_OUT(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {

        String name = "SP_CHECK_OUT";

        Object args[] = new Object[3];

        args[0] = rpmObject.getID();
        args[1] = new Integer(0);//bit flag.. not being used ?
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private String SP_UNDO_CHECKOUT(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_UNDO_CHECKOUT";

        Object args[] = new Object[3];

        args[0] = rpmObject.getID();
        args[1] = new Integer(0);//TODO Chk the bit flag.
        args[2] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }

    private String SP_I_WBS(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_I_WBS";

        WorkElement workElement = (WorkElement) rpmObject;

        Object args[] = new Object[4];

        args[0] = workElement.getName();

        if (workElement.getParent() != null
                && !(workElement.getParent() instanceof AbstractModule)
                && !isPortfolio(workElement.getParent()))
        {
            args[1] = workElement.getParent().getID();
        }

        args[2] = workElement.retrieveTypeID();
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }

    private String SP_C_WBS(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_C_WBS";

        Object args[] = new Object[4];

        WorkElement workElement = (WorkElement) rpmObject;

        args[1] = messageContext.getTemplate().getID(); //source: template

        if (isGenericProject(workElement) //case: GP
                && isGenericProject(messageContext.getTemplate()))
        {
            args[0] = workElement.getID(); //target: project
            args[2] = new Long(2); //bitFlag: 2 for ProjectTemplate
        }
        else
        //case: WorkElement
        {
            args[0] = workElement.getParent().getID(); //target: project
            args[2] = new Long(0); //bitFlag: 0 for AggregateNode/SimpleNode template
        }

        args[3] = getUser(messageContext).getID(); //user: 'PMO_SUPERVISOR'

        return executeProcedure(messageContext, name, args);
    }
    
    private String SP_COMMIT(RPMObject rpmObject,
            MessageContext messageContext, long bitflag) throws SQLException,
            RPMException
    {
        String name = "SP_COMMIT";

        Object args[] = new Object[2];

        WorkElement workElement = (WorkElement) rpmObject;

        args[0] = workElement.getID();

        args[1] = new Long(bitflag);

        return executeProcedure(messageContext, name, args);
    }

    private String SP_ROLLUP_EV(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_ROLLUP_EV";

        Object args[] = new Object[2];

        WorkElement workElement = (WorkElement) rpmObject;

        args[0] = workElement.getID();

        args[1] = new Long(2);

        return executeProcedure(messageContext, name, args);
    }
    
    
    private String SP_U_SETTING(MessageContext messageContext, int evDelay) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_SETTING";

        Object args[] = new Object[3];
        
        args[0] = new String(WbsUtil.EV_DELAY_DAYS_STR);

        args[1] = new Long(evDelay);

        args[2] = getUser(messageContext).getID();

        return executeProcedure(messageContext, procedureName, args);
    }

    

    private String SP_U_PROJ_BITS(RPMObject object,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_PROJ_BITS";    
        
        Object args[] = new Object[20];

        WorkElement workelement = (WorkElement) object;

        List result = null;
        result = WbsUtil
            .loadProjectDetailsEV_Infos(messageContext, workelement);

        ArrayList columns = null;

        // result should not be null or empty at this point
        if (result != null && result.size() > 0 && result.get(0) != null)
        {
            columns = (ArrayList) result.get(0);

            // SC_WCHAR --> TMT_WBS.ELEMENT_ID or TMT_WBS.PROJECT_ID
            args[0] = workelement.getID();

            // timephase_flag : SC_WCHAR (No associated column)
            args[1] = new String();

            // budget_setting_bits : SC_LONG_PARAM
            // PROJECT_DETAILS.BUDGET_SETTING_BITS
            args[2] = (Long) columns.get(0);

            // enable_ev_calculation : SC_LONG_PARAM
            // PROJECT_DETAILS.ENABLE_EV_CALCULATION
            args[3] = new Long(1);

            // chargeback_formula : SC_LONG_PARAM (Not Used)
            args[4] = new Long(0);

            // RollupProjectEstimated : SC_LONG_PARAM
            // PROJECT_DETAILS.ROLLUP_PROJECT_ESTIMATED
            args[5] = (Long) columns.get(1);

            // RollupDeliverableEstimated : SC_LONG_PARAM
            // PROJECT_DETAILS.ROLLUP_DELIVERABLE_ESTIMATED
            args[6] = (Long) columns.get(2);

            // notify_tm_rejected : SC_WCHAR
            args[7] = new String();

            // notify_tm_accepted : SC_WCHAR
            args[8] = new String();

            // notify_pm_submitted : SC_WCHAR
            args[9] = new String();

            // notify_rm_submitted : SC_WCHAR
            args[10] = new String();

            // notify_rm_accepted : SC_WCHAR
            args[11] = new String();

            // assignment_billable_flag : SC_LONG_PARAM
            // PROJECT_DETAILS.ASSIGNMENT_BILLABLE_FLAG
            args[12] = (Long) columns.get(3);

            // def_cri_consider_assign : SC_WCHAR
            // PROJECT_DETAILS.COPY_CRI_CONSIDER_ASSIGNMENTS
            args[13] = (String) columns.get(4);

            // def_cri_adopt_participants : SC_WCHAR
            // PROJECT_DETAILS.COPY_CRI_ADOPT_PARTICIPANTS
            args[14] = (String) columns.get(5);

            // dlv_budget_setting_bits : SC_LONG_PARAM
            // PROJECT_DETAILS.DLV_BUDGET_SETTING_BITS
            args[15] = (Long) columns.get(6);

            // attachment_view_bits : SC_LONG_PARAM (this field is used for
            // setting
            // the default view of document, )
            args[16] = new Long(0);

            // rup_home_path : SC_WCHAR --> PROJECT_DETAILS.RUP_HOME_PATH
            args[17] = (String) columns.get(7);

            // tng_home_path : SC_WCHAR --> PROJECT_DETAILS.TNG_HOME_PATH
            args[18] = (String) columns.get(8);

            args[19] = getUser(messageContext).getID();
        }
        
        return executeProcedure(messageContext, procedureName, args);
    }

    private void SP_D_WBS(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException
    {
        String name = "SP_D_WBS";

        Object args[] = new Object[4];

        args[0] = rpmObject.getID();

        args[1] = ManagerUtil.getParentId(this, rpmObject, messageContext,
            getTableName());

        args[2] = "Q";//TODO chk when it has to be X

        if (getUser(messageContext) != null)
        {
            args[3] = getUser(messageContext).getID();
        }

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_WBS_START_FINISH_DATES(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_WBS_START_FINISH_DATES";

        WorkElement workElement = (WorkElement) rpmObject;

        Object args[] = new Object[4];

        args[0] = workElement.getID();
        args[1] = DateUtil.convertToString(workElement.getEstimatedStartDate());
        args[2] = DateUtil
            .convertToString(workElement.getEstimatedFinishDate());
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_WBS_REQUEST(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_WBS_REQUEST";

        GenericProject genericProject = (GenericProject) rpmObject;

        Object args[] = new Object[11];

        args[0] = genericProject.getID();
        args[1] = "0";//TODO when should the start_finish flag be 1??
        args[2] = new Integer(0);//p_buffer_percent,
        args[3] = new Integer(0);//p_reporting_freq,
        args[4] = DateUtil.convertToString(genericProject
            .getEstimatedStartDate());
        args[5] = DateUtil.convertToString(genericProject
            .getEstimatedFinishDate());
        args[6] = new Integer(480);//p_minute_per_day
        if (genericProject.getCalendar() != null)
            args[7] = genericProject.getCalendar().getID();//p_calendar_id
        else
            //Default calendar parameter
            args[7] = "DEFAULT_CALENDAR";
        if (genericProject.getCurrency() != null)
            args[8] = new Integer(genericProject.getCurrency().getID());//p_currency_id
        else
            args[8] = new Integer(219);
        args[9] = "Test";//TODO Need to find out *p_request_descr.
        args[10] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_DESCRIPTION(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_DESCRIPTION";

        WorkElement workElement = (WorkElement) rpmObject;

        Object args[] = new Object[8];

        args[0] = workElement.getID();
        args[1] = workElement.getReferenceNumber();
        args[2] = workElement.getExternalID();
        args[3] = (workElement.getPriority() != null) ? workElement.getPriority()
                : new Integer(500);//500 priority default value

        if (workElement instanceof GenericProject)
        {
            args[4] = ((GenericProject) workElement).getProbabilityPercent();
        }
        else
        {
            args[4] = new Integer(100);
        }

        args[5] = workElement.retrieveTypeID();
        args[6] = new Integer(0);
        args[7] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_WBS_METHOD(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {

        String name = "SP_U_WBS_METHOD";

        Object args[] = new Object[4];

        GenericProject genericProject = (GenericProject) rpmObject;

        args[0] = genericProject.getID();
        args[1] = new Integer(0);//p_method, TODO find how to retrieve this
        // value
        args[2] = encodeAssignmentType(genericProject.getAssignmentType());
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_M_WBS(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException
    {
        //SP_M_WBS('224950B6B4C64B37BE242FE668E60A34','92924AA6BD554C30AC69DE4DD48F981D',6,'PMO_SUPERVISOR')

        String name = "SP_M_WBS";

        Object args[] = new Object[4];

        GenericProject genericProject = (GenericProject) rpmObject;

        if (genericProject.getPortfolio() != null)
        {
            args[0] = genericProject.getPortfolio().getID();
        }
        else
        {
            args[0] = genericProject.getParent().getID();
        }

        args[1] = genericProject.getID();
        /*
         *
         * p_bitflag: the first bit used to indicate moving operation (moving as
         * child (0) or sibling move (1))
         *
         * second bit used to indicate which object will be moved (wbs (0) or
         * PORTFOLIO (1))
         *
         * third bit used to indicate varify external dependencies.
         *
         */
        args[2] = new Long(6);//TODO know when to set the bitflag
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_U_CONSTRAINT(SimpleNode node, MessageContext messageContext)
            throws SQLException, RPMException
    {
        // 0 = Default Task - this is the default task type
        Integer taskType = new Integer(0);
        
        if (node.getTaskType() == TaskType.FixedDuration)
        {
            if (node.getEffortDriven() != null
                    && node.getEffortDriven().booleanValue())
            {
                taskType = new Integer(2);
            }
            else
            {
                taskType = new Integer(3);
            }
        }
        // else taskType is Default Task, as initialized
        
        Object args[] = new Object[5];
        args[0] = node.getID();
        args[1] = DateUtil.convertToString(node.getConstraintDate());
        args[2] = getConstraintTypeValue(node.getConstraintType());
        args[3] = taskType; 
        args[4] = getUser(messageContext).getID();

        executeProcedure(messageContext, "SP_U_CONSTRAINT", args);
    }

    private static Integer getConstraintTypeValue(ConstraintType constraintType)
    {

        if (ConstraintType.AsSoonAsPossible.equals(constraintType))
        {
            return new Integer(0);
        }
        else if (ConstraintType.AsLateAsPossible.equals(constraintType))
        {
            return new Integer(1);
        }
        else if (ConstraintType.FinishNoEarlierThan.equals(constraintType))
        {
            return new Integer(2);
        }
        else if (ConstraintType.FinishNoLaterThan.equals(constraintType))
        {
            return new Integer(3);
        }
        else if (ConstraintType.MustFinishOn.equals(constraintType))
        {
            return new Integer(4);
        }
        else if (ConstraintType.MustStartOn.equals(constraintType))
        {
            return new Integer(5);
        }
        else if (ConstraintType.StartNoEarlierThan.equals(constraintType))
        {
            return new Integer(6);
        }
        else if (ConstraintType.StartNoEarlierThan.equals(constraintType))
        {
            return new Integer(7);
        }
        else
        {
            return new Integer(0);
        }
    }

    private void SP_U_WBS_TIMECODES(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_U_WBS_TIMECODES";
        WorkElement workElement = (WorkElement) rpmObject;

        Object args[] = new Object[4];

        args[0] = workElement.getID();
        if (workElement.getTimeCode1() != null)
            args[1] = workElement.getTimeCode1().getID();

        if (workElement.getTimeCode2() != null)
            args[2] = workElement.getTimeCode2().getID();

        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    //  Done by Iftach Ragoler 25/12/05
    private String SP_U_WBS_ASSET(RPMObject rpmObject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_WBS_ASSET";
        WorkElement workElement = (WorkElement) rpmObject;
        Object args[] = new Object[5];
        args[0] = rpmObject.getID();
        args[1] = ManagerUtil.getId(workElement.getAsset());
        args[2] = workElement.getAsset() == null ? null : workElement
            .getAsset().getName();
        args[3] = ManagerUtil.getIntId(workElement.getAssetCode());
        args[4] = getUser(messageContext).getID();
        return executeProcedure(messageContext, name, args);
    }

    private void SP_U_DEFAULT_PROJECT(GenericProject genericProject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_DEFAULT_PROJECT";
        Object args[] = new Object[3];
        args[0] = genericProject.getID();
        args[1] = new Integer(1);
        args[2] = getUser(messageContext).getID();
        executeProcedure(messageContext, name, args);
    }

    //private void SP_U_MLST_PERCENT(Task task,
    //        MessageContext messageContext) throws SQLException, RPMException
    //{
    //    String name = "SP_U_MLST_PERCENT";
    //    Object args[] = new Object[3];
    //    args[0] = task.getID();
    //    // must multiply by 100 for use in proc
    //    args[1] = new Double(task.getPercentageDurationCompleted().doubleValue() * 100);
    //    args[2] = getUser(messageContext).getID();
    //    executeProcedure(messageContext, name, args);
    //}

    private void SP_COMMIT(WorkElement workElement,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_COMMIT";
        Object args[] = new Object[2];

        args[0] = workElement.getID(); // id
        args[1] = new Long(2); // bitflag

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_COMMIT_ASSIGN(GenericProject genericProject,
            MessageContext messageContext) throws SQLException, RPMException
    {
        final String procedureName = "SP_COMMIT_ASSIGN";
        Object args[] = new Object[2];

        args[0] = genericProject.getID(); // id
        args[1] = new Long(2); // bitflag

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_U_WBS_TYPE(WorkElement inputWorkElement,
            WorkElement outputWorkElement, MessageContext messageContext)
            throws SQLException, RPMException
    {
        final String procedureName = "SP_U_WBS_TYPE";
        Object args[] = new Object[3];

        args[0] = inputWorkElement.getID(); // id
        args[1] = outputWorkElement.retrieveTypeID(); // bitflag
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, procedureName, args);
    }

    private void SP_U_ACTUAL(Task task, MessageContext messageContext)
            throws SQLException, RPMException
    {
        // used to update the percentageDurationComplete for a Task
        // also updates the actualDate.durationInMinutes
        String name = "SP_U_ACTUAL";

        Object args[] = new Object[7];

        args[0] = task.getID(); //bit-flag
        args[1] = new Integer(0);
        args[2] = null; // startDate not applicable for update %DC
        args[3] = null; // finishDate is read-only

        if (task.getPercentageDurationCompleted() != null)
        {
            args[4] = new Double(task.getPercentageDurationCompleted()
                .doubleValue() * 100);
        }

        args[5] = new Integer(1); // p_close_task_right unused parameter
        args[6] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    private void SP_A_WBS_TO_CRI(WorkElement workElement,
            ScopeElement scopeElement, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_A_WBS_TO_CRI";
        Object args[] = new Object[3];

        if (workElement != null)
        {
            args[0] = workElement.getID();
        }
        if (scopeElement != null)
        {
            args[1] = scopeElement.getID();
        }
        args[2] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }
    
    private Integer SP_S_WKF_PROCESS_TYPE(Integer typeId,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_S_WKF_PROCESS_TYPE";
        Integer value = null;
        Object args[] = new Object[2];

        args[0] = typeId;
        args[1] = getUser(messageContext).getID();
        
        // we cannot call framework method executeProcedure directly 
        // because SP_S_* procedures do not return an error code
        Connection c = null;
        ResultSet rs = null;
        try
        {
            c = Manager.getConnection(messageContext);
            rs = Manager.executeStoredProcedure(c, name, args, true,
                messageContext);
            if (rs.next())
            {
                value = new Integer(rs.getInt(1));
            }
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }
        return value;
    }
    
    /*
     * This method loads a list of workflow role IDs and a list of role names,
     * then removes the ones that already exist in the mappings list provided.
     */
    public WorkflowRoleMapping[] SP_S_PRJ_WKF_ROLE(GenericProject genericProject,
            ArrayList mappings, MessageContext messageContext)
            throws SQLException, RPMException
    {
        String name = "SP_S_PRJ_WKF_ROLE";
        Object args[] = new Object[4];
        ArrayList listIDs = new ArrayList();
        ArrayList listNames = new ArrayList();

        args[0] = genericProject.getID();
        args[1] = null;
        args[2] = new Integer(0);
        args[3] = getUser(messageContext).getID();

        // we cannot call framework method executeProcedure directly 
        // because SP_S_* procedures do not return an error code
        Connection c = null;
        ResultSet rs = null;
        try
        {
            c = Manager.getConnection(messageContext);
            rs = Manager.executeStoredProcedure(c, name, args, true,
                messageContext);

            while (rs.next())
            {
                listIDs.add(rs.getObject(1));
                listNames.add(rs.getObject(4));
            }
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }

        // return a list of mappings with no duplicates
        return WorkflowUtil.mergeLoadedWorkflowMappings(listIDs, listNames,
            mappings, genericProject);
    }
    
    protected void decodeSecurityRoleIds(MessageContext msgContext,
            ResultSet rs, String contextName) throws RPMException, SQLException
    {
        String objectId = rs.getString(ID_ELEMENT_ID);
        String securityRoleIds = (String) rs.getObject("SECURITY_ID");
        ManagerUtil.setSecurityRoleIds(msgContext, WorkElement.class, objectId,
            securityRoleIds);
    }
    
    protected Object loadWithSecurity(MessageContext messageContext, Connection c,
            String selectStmtStr, Object[] argValues, String contextName)
            throws SQLException, RPMException
    {
        String name = "SP_S_WBS_PAGES";

        Object args[] = new Object[15];

        String[] output = ManagerUtil.processSelectStatementBeforeSecurityCall(selectStmtStr, argValues);        
        String columnList = output[0];
        String whereClause = output[2];
        
        //Get loadParams from the messageContext
        LoadParams loadParams = messageContext.getLoadParams();
        Integer startIndex = (loadParams!=null && !loadParams.isAlreadySet()&&loadParams.getStartIndex()!=null)?loadParams.getStartIndex():new Integer(1);
        // endIndex Default '0' = load all
        Integer endIndex = (loadParams!=null && !loadParams.isAlreadySet()&&loadParams.getEndIndex()!=null)?loadParams.getEndIndex():new Integer(0);
        String orderBy = (loadParams!=null && !loadParams.isAlreadySet())?loadParams.getOrderBy():null;
        // buildSecurity Default '0' = don't build security columns
        //for internal security mechanism, the loadParams.isForceBuildSecurity is used
//        Integer buildSecurity = (loadParams!=null && ((!loadParams.isAlreadySet() && loadParams.isBuildSecurity())||(loadParams.isForceBuildSecurity())))?new Integer(1):new Integer(0);
        //To improve performance, we will always build the security
        
        // idsType Default '1' = All Project (values are mutually exclusive even if separate arguments exist for projectIDs/portfolioIDs and orgFilterIds)
        //      1=return all projects that are visible by resource 
        //      2=ids is a list of project IDs 
        //      3=ids is a list of portfolio IDs 
        //      4=ids is a organization filter id
        Integer idsType =  (loadParams!=null && !loadParams.isAlreadySet() && loadParams.getIdsType()!=null)?loadParams.getIdsType():new Integer(1);
        String idsList =  (loadParams!=null && !loadParams.isAlreadySet())?loadParams.getIdsList():null;
        String attributesIdsList =  (loadParams!=null && !loadParams.isAlreadySet())?loadParams.getAttributesIdsList():null;
        // viewType Default to All (0:all items, 1: project types, 2:deliverable types, 3:all task types, 4:my assignments)
        Integer viewType =  (loadParams!=null && !loadParams.isAlreadySet()&&loadParams.getViewType()!=null)?loadParams.getViewType():new Integer(0); 
        // inclusions Default to All (0=Exclude Closed and Completed;32=Include Closed;64=Include Completed;96=Include Closed and Completed)        
        Integer inclusions = (loadParams!=null && !loadParams.isAlreadySet()&&loadParams.getInclusions()!=null)?loadParams.getInclusions():new Integer(96); 

        whereClause = (loadParams!=null && !loadParams.isAlreadySet() && loadParams.getExtraWhere()!=null)?(whereClause+" "+loadParams.getExtraWhere()+" "):whereClause;
        
        boolean isProjectIdListUseful = (idsType.intValue()==2)||(idsType.intValue()==3);
        
        if (!isProjectIdListUseful)
        {
            idsList = null;
        }
        
        if (idsList == null || idsList.length() == 0)
        {
            String tableName = getTableName();
            ManagerUtil.IDStorage storage = ManagerUtil.getRestrictedProjectIdList(messageContext,
                name, whereClause, idsType, idsList, tableName);
            idsList = storage.idsList;
            idsType = storage.idsType;
        }
        
        args[0] = idsType; // p_bitflag
        args[1] = idsList; // p_ids
        args[2] = (idsType.intValue()==4) ? idsList:null; // p_org_filter_id
        args[3] = getUser(messageContext).getID(); // p_resource_id
        args[4] = viewType;//p_view_type
        args[5] = new Integer(0);//p_stage_id  0 : all stages(filtering will be in the where clause)
        args[6] = new Integer(0);//p_type_id   0 : all types (filtering will be in the where clause)
        args[7] = attributesIdsList; //filter_string
        args[8] = new Integer(1);//Always build_security (see comment above)
        args[9] = inclusions; // p_active_record
        args[10] = whereClause; // p_where_clause
        args[11] = orderBy;//p_order_by
        args[12] = columnList;
        args[13] = startIndex;//p_start_from
        args[14] = endIndex;//p_end_to
        
        return Manager.executeStoredProcedure(c, name, args, spParamsTypes, true, messageContext, true, useDBStatementsForLOBS);
    }

    private static final int[] spParamsTypes = new int[]{
        java.sql.Types.INTEGER,java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.INTEGER,
        java.sql.Types.INTEGER,java.sql.Types.INTEGER,java.sql.Types.CHAR,java.sql.Types.INTEGER,java.sql.Types.INTEGER,
        java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.CHAR,java.sql.Types.INTEGER,java.sql.Types.INTEGER
    }; 

    
    private void SP_U_ELEMENT_CONFIG(SimpleNode node,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_ELEMENT_CONFIG";
        Object args[] = new Object[4];
        
        Integer configFlag = node.getMandatory() != null
                && node.getMandatory().booleanValue() ? new Integer(1)
                : new Integer(0);
        
        args[0] = node.getID();
        args[1] = new Integer(1);
        args[2] = configFlag;
        args[3] = getUser(messageContext).getID();
        
        executeProcedure(messageContext, name, args);
    }    
    
    
 }